from collections.abc import Mapping
from typing import List
from boto3.resources.collection import ResourceCollection
from botocore.waiter import Waiter
from botocore.paginate import Paginator
from botocore.client import BaseClient


class ACM(BaseClient):
    """<fullname>AWS Certificate Manager</fullname> Welcome to the AWS Certificate Manager (ACM) API documentation. You can use ACM to manage SSL/TLS certificates for your AWS-based websites and applications. For general information about using ACM, see the <a href="https://docs.aws.amazon.com/acm/latest/userguide/"> <i>AWS Certificate Manager User Guide</i> </a>."""

    class CertificateValidatedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_certificate` for valid parameters"""
            pass


    class ListCertificatesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_certificates` for valid parameters"""
            pass


    class AddTagsToCertificateRequest(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class DeleteCertificateRequest(Mapping):
        pass
    
    class DescribeCertificateRequest(Mapping):
        pass
    
    class DescribeCertificateResponse(Mapping):
        pass
    
    class ExportCertificateRequest(Mapping):
        pass
    
    class ExportCertificateResponse(Mapping):
        pass
    
    class PassphraseBlob(object):
        pass
    
    class GetCertificateRequest(Mapping):
        pass
    
    class GetCertificateResponse(Mapping):
        pass
    
    class ImportCertificateRequest(Mapping):
        pass
    
    class ImportCertificateResponse(Mapping):
        pass
    
    class CertificateBodyBlob(object):
        pass
    
    class PrivateKeyBlob(object):
        pass
    
    class CertificateChainBlob(object):
        pass
    
    class ListCertificatesRequest(Mapping):
        pass
    
    class ListCertificatesResponse(Mapping):
        pass
    
    class CertificateStatus(object):
        pass
    
    class Filters(Mapping):
        pass
    
    class ListTagsForCertificateRequest(Mapping):
        pass
    
    class ListTagsForCertificateResponse(Mapping):
        pass
    
    class RemoveTagsFromCertificateRequest(Mapping):
        pass
    
    class RenewCertificateRequest(Mapping):
        pass
    
    class RequestCertificateRequest(Mapping):
        pass
    
    class RequestCertificateResponse(Mapping):
        pass
    
    class DomainNameString(object):
        pass
    
    class DomainValidationOption(Mapping):
        pass
    
    class CertificateOptions(Mapping):
        pass
    
    class ResendValidationEmailRequest(Mapping):
        pass
    
    class UpdateCertificateOptionsRequest(Mapping):
        pass
    
    def add_tags_to_certificate(self, Tags: List[Tag], CertificateArn: str):
        """Adds one or more tags to an ACM certificate. Tags are labels that you can use to identify and organize your AWS resources. Each tag consists of a <code>key</code> and an optional <code>value</code>. You specify the certificate on input by its Amazon Resource Name (ARN). You specify the tag by using a key-value pair.  You can apply a tag to just one certificate if you want to identify a specific characteristic of that certificate, or you can apply the same tag to multiple certificates if you want to filter for a common relationship among those certificates. Similarly, you can apply the same tag to multiple resources if you want to specify a relationship among those resources. For example, you can add the same tag to an ACM certificate and an Elastic Load Balancing load balancer to indicate that they are both used by the same website. For more information, see <a href="https://docs.aws.amazon.com/acm/latest/userguide/tags.html">Tagging ACM certificates</a>.  To remove one or more tags, use the <a>RemoveTagsFromCertificate</a> action. To view all of the tags that have been applied to the certificate, use the <a>ListTagsForCertificate</a> action. 

        :param self:
        :param Tags:  The key-value pair that defines the tag. The tag value is optional.<br/>CertificateArn:  String that contains the ARN of the ACM certificate to which the tag is to be applied. This must be of the form:  <code>arn:aws:acm:region:123456789012:certificate/12345678-1234-1234-1234-123456789012</code>  For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>. <br/>
        :return:         """
        pass

    def delete_certificate(self, CertificateArn: str):
        """Deletes a certificate and its associated private key. If this action succeeds, the certificate no longer appears in the list that can be displayed by calling the <a>ListCertificates</a> action or be retrieved by calling the <a>GetCertificate</a> action. The certificate will not be available for use by AWS services integrated with ACM.  <note> You cannot delete an ACM certificate that is being used by another AWS service. To delete a certificate that is in use, the certificate association must first be removed. </note>

        :param self:
        :param CertificateArn:  String that contains the ARN of the ACM certificate to be deleted. This must be of the form:  <code>arn:aws:acm:region:123456789012:certificate/12345678-1234-1234-1234-123456789012</code>  For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>
        :return:         """
        pass

    def describe_certificate(self, CertificateArn: str) -> DescribeCertificateResponse:
        """Returns detailed metadata about the specified ACM certificate.

        :param self:
        :param CertificateArn:  The Amazon Resource Name (ARN) of the ACM certificate. The ARN must have the following form:  <code>arn:aws:acm:region:123456789012:certificate/12345678-1234-1234-1234-123456789012</code>  For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>
        :return: Certificate:  Metadata about an ACM certificate.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> Value that specifies whether to add the certificate to a transparency log. Certificate transparency makes it possible to detect SSL certificates that have been mistakenly or maliciously issued. A browser might respond to certificate that has not been logged by showing an error message. The logs are cryptographically secure. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>: Value that specifies whether to add the certificate to a transparency log. Certificate transparency makes it possible to detect SSL certificates that have been mistakenly or maliciously issued. A browser might respond to certificate that has not been logged by showing an error message. The logs are cryptographically secure. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RenewalEligibility</i> Specifies whether the certificate is eligible for renewal. At this time, only exported private certificates can be renewed with the <a>RenewCertificate</a> command.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateAuthorityArn</i> The Amazon Resource Name (ARN) of the ACM PCA private certificate authority (CA) that issued the certificate. This has the following format:   <code>arn:aws:acm-pca:region:account:certificate-authority/12345678-1234-1234-1234-123456789012</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExtendedKeyUsages</i> Contains a list of Extended Key Usage X.509 v3 extension objects. Each object specifies a purpose for which the certificate public key can be used and consists of a name and an object identifier (OID). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyUsages</i> A list of Key Usage X.509 v3 extension objects. Each object is a string value that identifies the purpose of the public key contained in the certificate. Possible extension values include DIGITAL_SIGNATURE, KEY_ENCHIPHERMENT, NON_REPUDIATION, and more.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RenewalSummary</i> Contains information about the status of ACM's <a href="https://docs.aws.amazon.com/acm/latest/userguide/acm-renewal.html">managed renewal</a> for the certificate. This field exists only when the certificate type is <code>AMAZON_ISSUED</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RenewalSummary</b>: Contains information about the status of ACM's <a href="https://docs.aws.amazon.com/acm/latest/userguide/acm-renewal.html">managed renewal</a> for the certificate. This field exists only when the certificate type is <code>AMAZON_ISSUED</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The source of the certificate. For certificates provided by ACM, this value is <code>AMAZON_ISSUED</code>. For certificates that you imported with <a>ImportCertificate</a>, this value is <code>IMPORTED</code>. ACM does not provide <a href="https://docs.aws.amazon.com/acm/latest/userguide/acm-renewal.html">managed renewal</a> for imported certificates. For more information about the differences between certificates that you import and those that ACM provides, see <a href="https://docs.aws.amazon.com/acm/latest/userguide/import-certificate.html">Importing Certificates</a> in the <i>AWS Certificate Manager User Guide</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureReason</i> The reason the certificate request failed. This value exists only when the certificate status is <code>FAILED</code>. For more information, see <a href="https://docs.aws.amazon.com/acm/latest/userguide/troubleshooting.html#troubleshooting-failed">Certificate Request Failed</a> in the <i>AWS Certificate Manager User Guide</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InUseBy</i> A list of ARNs for the AWS resources that are using the certificate. A certificate can be used by multiple AWS resources. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SignatureAlgorithm</i> The algorithm that was used to sign the certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyAlgorithm</i> The algorithm that was used to generate the public-private key pair.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotAfter</i> The time after which the certificate is not valid.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotBefore</i> The time before which the certificate is not valid.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RevocationReason</i> The reason the certificate was revoked. This value exists only when the certificate status is <code>REVOKED</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RevokedAt</i> The time at which the certificate was revoked. This value exists only when the certificate status is <code>REVOKED</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImportedAt</i> The date and time at which the certificate was imported. This value exists only when the certificate type is <code>IMPORTED</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IssuedAt</i> The time at which the certificate was issued. This value exists only when the certificate type is <code>AMAZON_ISSUED</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i> The time at which the certificate was requested. This value exists only when the certificate type is <code>AMAZON_ISSUED</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Issuer</i> The name of the certificate authority that issued and signed the certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subject</i> The name of the entity that is associated with the public key contained in the certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Serial</i> The serial number of the certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainValidationOptions</i> Contains information about the initial validation of each domain name that occurs as a result of the <a>RequestCertificate</a> request. This field exists only when the certificate type is <code>AMAZON_ISSUED</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubjectAlternativeNames</i> One or more domain names (subject alternative names) included in the certificate. This list contains the domain names that are bound to the public key that is contained in the certificate. The subject alternative names include the canonical domain name (CN) of the certificate and additional domain names that can be used to connect to the website. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainName</i> The fully qualified domain name for the certificate, such as www.example.com or example.com.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateArn</i> The Amazon Resource Name (ARN) of the certificate. For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the <i>AWS General Reference</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdatedAt</i> The time at which the renewal summary was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RenewalStatusReason</i> The reason that a renewal request was unsuccessful.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainValidationOptions</i> Contains information about the validation of each domain name in the certificate, as it pertains to ACM's <a href="https://docs.aws.amazon.com/acm/latest/userguide/acm-renewal.html">managed renewal</a>. This is different from the initial validation that occurs as a result of the <a>RequestCertificate</a> request. This field exists only when the certificate type is <code>AMAZON_ISSUED</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RenewalStatus</i> The status of ACM's <a href="https://docs.aws.amazon.com/acm/latest/userguide/acm-renewal.html">managed renewal</a> of the certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateTransparencyLoggingPreference</i> You can opt out of certificate transparency logging by specifying the <code>DISABLED</code> option. Opt in by specifying <code>ENABLED</code>. <br/>        """
        pass

    def export_certificate(self, Passphrase: PassphraseBlob, CertificateArn: str) -> ExportCertificateResponse:
        """Exports a private certificate issued by a private certificate authority (CA) for use anywhere. You can export the certificate, the certificate chain, and the encrypted private key associated with the public key embedded in the certificate. You must store the private key securely. The private key is a 2048 bit RSA key. You must provide a passphrase for the private key when exporting it. You can use the following OpenSSL command to decrypt it later. Provide the passphrase when prompted.   <code>openssl rsa -in encrypted_key.pem -out decrypted_key.pem</code> 

        :param self:
        :param Passphrase:  Passphrase to associate with the encrypted exported private key. If you want to later decrypt the private key, you must have the passphrase. You can use the following OpenSSL command to decrypt a private key:   <code>openssl rsa -in encrypted_key.pem -out decrypted_key.pem</code> <br/>CertificateArn:  An Amazon Resource Name (ARN) of the issued certificate. This must be of the form:  <code>arn:aws:acm:region:account:certificate/12345678-1234-1234-1234-123456789012</code> <br/>
        :return: PrivateKey:  The encrypted private key associated with the public key in the certificate. The key is output in PKCS #8 format and is base64 PEM-encoded. <br/>CertificateChain:  The base64 PEM-encoded certificate chain. This does not include the certificate that you are exporting.<br/>Certificate:  The base64 PEM-encoded certificate.<br/>        """
        pass

    def get_certificate(self, CertificateArn: str) -> GetCertificateResponse:
        """Retrieves a certificate specified by an ARN and its certificate chain . The chain is an ordered list of certificates that contains the end entity certificate, intermediate certificates of subordinate CAs, and the root certificate in that order. The certificate and certificate chain are base64 encoded. If you want to decode the certificate to see the individual fields, you can use OpenSSL.

        :param self:
        :param CertificateArn:  String that contains a certificate ARN in the following format:  <code>arn:aws:acm:region:123456789012:certificate/12345678-1234-1234-1234-123456789012</code>  For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>
        :return: CertificateChain:  The certificate chain that contains the root certificate issued by the certificate authority (CA).<br/>Certificate:  String that contains the ACM certificate represented by the ARN specified at input.<br/>        """
        pass

    def import_certificate(self, PrivateKey: PrivateKeyBlob, Certificate: CertificateBodyBlob, CertificateArn: str=None, CertificateChain: CertificateChainBlob=None) -> ImportCertificateResponse:
        """Imports a certificate into AWS Certificate Manager (ACM) to use with services that are integrated with ACM. Note that <a href="https://docs.aws.amazon.com/acm/latest/userguide/acm-services.html">integrated services</a> allow only certificate types and keys they support to be associated with their resources. Further, their support differs depending on whether the certificate is imported into IAM or into ACM. For more information, see the documentation for each service. For more information about importing certificates into ACM, see <a href="https://docs.aws.amazon.com/acm/latest/userguide/import-certificate.html">Importing Certificates</a> in the <i>AWS Certificate Manager User Guide</i>.  <note> ACM does not provide <a href="https://docs.aws.amazon.com/acm/latest/userguide/acm-renewal.html">managed renewal</a> for certificates that you import. </note> Note the following guidelines when importing third party certificates: <ul> <li> You must enter the private key that matches the certificate you are importing. </li> <li> The private key must be unencrypted. You cannot import a private key that is protected by a password or a passphrase. </li> <li> If the certificate you are importing is not self-signed, you must enter its certificate chain. </li> <li> If a certificate chain is included, the issuer must be the subject of one of the certificates in the chain. </li> <li> The certificate, private key, and certificate chain must be PEM-encoded. </li> <li> The current time must be between the <code>Not Before</code> and <code>Not After</code> certificate fields. </li> <li> The <code>Issuer</code> field must not be empty. </li> <li> The OCSP authority URL, if present, must not exceed 1000 characters. </li> <li> To import a new certificate, omit the <code>CertificateArn</code> argument. Include this argument only when you want to replace a previously imported certificate. </li> <li> When you import a certificate by using the CLI, you must specify the certificate, the certificate chain, and the private key by their file names preceded by <code>file://</code>. For example, you can specify a certificate saved in the <code>C:\temp</code> folder as <code>file://C:\temp\certificate_to_import.pem</code>. If you are making an HTTP or HTTPS Query request, include these arguments as BLOBs.  </li> <li> When you import a certificate by using an SDK, you must specify the certificate, the certificate chain, and the private key files in the manner required by the programming language you're using.  </li> </ul> This operation returns the <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Name (ARN)</a> of the imported certificate.

        :param self:
        :param CertificateChain:  The PEM encoded certificate chain.<br/>PrivateKey:  The private key that matches the public key in the certificate.<br/>Certificate:  The certificate to import.<br/>CertificateArn:  The <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Name (ARN)</a> of an imported certificate to replace. To import a new certificate, omit this field. <br/>
        :return: CertificateArn:  The <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Name (ARN)</a> of the imported certificate.<br/>        """
        pass

    def list_certificates(self, CertificateStatuses: List[CertificateStatus]=None, Includes: Filters=None, NextToken: str=None, MaxItems: int=None) -> ListCertificatesResponse:
        """Retrieves a list of certificate ARNs and domain names. You can request that only certificates that match a specific status be listed. You can also filter by specific attributes of the certificate. 

        :param self:
        :param MaxItems:  Use this parameter when paginating results to specify the maximum number of items to return in the response. If additional items exist beyond the number you specify, the <code>NextToken</code> element is sent in the response. Use this <code>NextToken</code> value in a subsequent request to retrieve additional items.<br/>NextToken:  Use this parameter only when paginating results and only in a subsequent request after you receive a response with truncated results. Set it to the value of <code>NextToken</code> from the response you just received.<br/>Includes:  Filter the certificate list. For more information, see the <a>Filters</a> structure.:<br/>CertificateStatuses:  Filter the certificate list by status value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>keyTypes</i> Specify one or more algorithms that can be used to generate key pairs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>keyUsage</i> Specify one or more <a>KeyUsage</a> extension values.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>extendedKeyUsage</i> Specify one or more <a>ExtendedKeyUsage</a> extension values.<br/>
        :return: CertificateSummaryList:  A list of ACM certificates.<br/>NextToken:  When the list is truncated, this value is present and contains the value to use for the <code>NextToken</code> parameter in a subsequent pagination request.<br/>        """
        pass

    def list_tags_for_certificate(self, CertificateArn: str) -> ListTagsForCertificateResponse:
        """Lists the tags that have been applied to the ACM certificate. Use the certificate's Amazon Resource Name (ARN) to specify the certificate. To add a tag to an ACM certificate, use the <a>AddTagsToCertificate</a> action. To delete a tag, use the <a>RemoveTagsFromCertificate</a> action. 

        :param self:
        :param CertificateArn:  String that contains the ARN of the ACM certificate for which you want to list the tags. This must have the following form:  <code>arn:aws:acm:region:123456789012:certificate/12345678-1234-1234-1234-123456789012</code>  For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>. <br/>
        :return: Tags:  The key-value pairs that define the applied tags.<br/>        """
        pass

    def remove_tags_from_certificate(self, Tags: List[Tag], CertificateArn: str):
        """Remove one or more tags from an ACM certificate. A tag consists of a key-value pair. If you do not specify the value portion of the tag when calling this function, the tag will be removed regardless of value. If you specify a value, the tag is removed only if it is associated with the specified value.  To add tags to a certificate, use the <a>AddTagsToCertificate</a> action. To view all of the tags that have been applied to a specific ACM certificate, use the <a>ListTagsForCertificate</a> action. 

        :param self:
        :param Tags:  The key-value pair that defines the tag to remove.<br/>CertificateArn:  String that contains the ARN of the ACM Certificate with one or more tags that you want to remove. This must be of the form:  <code>arn:aws:acm:region:123456789012:certificate/12345678-1234-1234-1234-123456789012</code>  For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>. <br/>
        :return:         """
        pass

    def renew_certificate(self, CertificateArn: str):
        """Renews an eligable ACM certificate. At this time, only exported private certificates can be renewed with this operation. In order to renew your ACM PCA certificates with ACM, you must first <a href="https://docs.aws.amazon.com/acm-pca/latest/userguide/PcaPermissions.html">grant the ACM service principal permission to do so</a>. For more information, see <a href="https://docs.aws.amazon.com/acm/latest/userguide/manual-renewal.html">Testing Managed Renewal</a> in the ACM User Guide.

        :param self:
        :param CertificateArn:  String that contains the ARN of the ACM certificate to be renewed. This must be of the form:  <code>arn:aws:acm:region:123456789012:certificate/12345678-1234-1234-1234-123456789012</code>  For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>
        :return:         """
        pass

    def request_certificate(self, DomainName: str, ValidationMethod: str=None, SubjectAlternativeNames: List[DomainNameString]=None, IdempotencyToken: str=None, DomainValidationOptions: List[DomainValidationOption]=None, Options: CertificateOptions=None, CertificateAuthorityArn: str=None) -> RequestCertificateResponse:
        """Requests an ACM certificate for use with other AWS services. To request an ACM certificate, you must specify a fully qualified domain name (FQDN) in the <code>DomainName</code> parameter. You can also specify additional FQDNs in the <code>SubjectAlternativeNames</code> parameter.  If you are requesting a private certificate, domain validation is not required. If you are requesting a public certificate, each domain name that you specify must be validated to verify that you own or control the domain. You can use <a href="https://docs.aws.amazon.com/acm/latest/userguide/gs-acm-validate-dns.html">DNS validation</a> or <a href="https://docs.aws.amazon.com/acm/latest/userguide/gs-acm-validate-email.html">email validation</a>. We recommend that you use DNS validation. ACM issues public certificates after receiving approval from the domain owner. 

        :param self:
        :param CertificateAuthorityArn:  The Amazon Resource Name (ARN) of the private certificate authority (CA) that will be used to issue the certificate. If you do not provide an ARN and you are trying to request a private certificate, ACM will attempt to issue a public certificate. For more information about private CAs, see the <a href="https://docs.aws.amazon.com/acm-pca/latest/userguide/PcaWelcome.html">AWS Certificate Manager Private Certificate Authority (PCA)</a> user guide. The ARN must have the following form:   <code>arn:aws:acm-pca:region:account:certificate-authority/12345678-1234-1234-1234-123456789012</code> <br/>Options:  Currently, you can use this parameter to specify whether to add the certificate to a certificate transparency log. Certificate transparency makes it possible to detect SSL/TLS certificates that have been mistakenly or maliciously issued. Certificates that have not been logged typically produce an error message in a browser. For more information, see <a href="https://docs.aws.amazon.com/acm/latest/userguide/acm-bestpractices.html#best-practices-transparency">Opting Out of Certificate Transparency Logging</a>.:<br/>DomainValidationOptions:  The domain name that you want ACM to use to send you emails so that you can validate domain ownership.<br/>IdempotencyToken:  Customer chosen string that can be used to distinguish between calls to <code>RequestCertificate</code>. Idempotency tokens time out after one hour. Therefore, if you call <code>RequestCertificate</code> multiple times with the same idempotency token within one hour, ACM recognizes that you are requesting only one certificate and will issue only one. If you change the idempotency token for each call, ACM recognizes that you are requesting multiple certificates.<br/>SubjectAlternativeNames:  Additional FQDNs to be included in the Subject Alternative Name extension of the ACM certificate. For example, add the name www.example.net to a certificate for which the <code>DomainName</code> field is www.example.com if users can reach your site by using either name. The maximum number of domain names that you can add to an ACM certificate is 100. However, the initial limit is 10 domain names. If you need more than 10 names, you must request a limit increase. For more information, see <a href="https://docs.aws.amazon.com/acm/latest/userguide/acm-limits.html">Limits</a>.  The maximum length of a SAN DNS name is 253 octets. The name is made up of multiple labels separated by periods. No label can be longer than 63 octets. Consider the following examples:  <ul> <li>  <code>(63 octets).(63 octets).(63 octets).(61 octets)</code> is legal because the total length is 253 octets (63+1+63+1+63+1+61) and no label exceeds 63 octets. </li> <li>  <code>(64 octets).(63 octets).(63 octets).(61 octets)</code> is not legal because the total length exceeds 253 octets (64+1+63+1+63+1+61) and the first label exceeds 63 octets. </li> <li>  <code>(63 octets).(63 octets).(63 octets).(62 octets)</code> is not legal because the total length of the DNS name (63+1+63+1+63+1+62) exceeds 253 octets. </li> </ul><br/>ValidationMethod:  The method you want to use if you are requesting a public certificate to validate that you own or control domain. You can <a href="https://docs.aws.amazon.com/acm/latest/userguide/gs-acm-validate-dns.html">validate with DNS</a> or <a href="https://docs.aws.amazon.com/acm/latest/userguide/gs-acm-validate-email.html">validate with email</a>. We recommend that you use DNS validation. <br/>DomainName:   Fully qualified domain name (FQDN), such as www.example.com, that you want to secure with an ACM certificate. Use an asterisk (*) to create a wildcard certificate that protects several sites in the same domain. For example, *.example.com protects www.example.com, site.example.com, and images.example.com.   The first domain name you enter cannot exceed 63 octets, including periods. Each subsequent Subject Alternative Name (SAN), however, can be up to 253 octets in length. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateTransparencyLoggingPreference</i> You can opt out of certificate transparency logging by specifying the <code>DISABLED</code> option. Opt in by specifying <code>ENABLED</code>. <br/>
        :return: CertificateArn:  String that contains the ARN of the issued certificate. This must be of the form:  <code>arn:aws:acm:us-east-1:123456789012:certificate/12345678-1234-1234-1234-123456789012</code> <br/>        """
        pass

    def resend_validation_email(self, ValidationDomain: str, Domain: str, CertificateArn: str):
        """Resends the email that requests domain ownership validation. The domain owner or an authorized representative must approve the ACM certificate before it can be issued. The certificate can be approved by clicking a link in the mail to navigate to the Amazon certificate approval website and then clicking <b>I Approve</b>. However, the validation email can be blocked by spam filters. Therefore, if you do not receive the original mail, you can request that the mail be resent within 72 hours of requesting the ACM certificate. If more than 72 hours have elapsed since your original request or since your last attempt to resend validation mail, you must request a new certificate. For more information about setting up your contact email addresses, see <a href="https://docs.aws.amazon.com/acm/latest/userguide/setup-email.html">Configure Email for your Domain</a>. 

        :param self:
        :param ValidationDomain:  The base validation domain that will act as the suffix of the email addresses that are used to send the emails. This must be the same as the <code>Domain</code> value or a superdomain of the <code>Domain</code> value. For example, if you requested a certificate for <code>site.subdomain.example.com</code> and specify a <b>ValidationDomain</b> of <code>subdomain.example.com</code>, ACM sends email to the domain registrant, technical contact, and administrative contact in WHOIS and the following five addresses: <ul> <li> admin@subdomain.example.com </li> <li> administrator@subdomain.example.com </li> <li> hostmaster@subdomain.example.com </li> <li> postmaster@subdomain.example.com </li> <li> webmaster@subdomain.example.com </li> </ul><br/>Domain:  The fully qualified domain name (FQDN) of the certificate that needs to be validated.<br/>CertificateArn:  String that contains the ARN of the requested certificate. The certificate ARN is generated and returned by the <a>RequestCertificate</a> action as soon as the request is made. By default, using this parameter causes email to be sent to all top-level domains you specified in the certificate request. The ARN must be of the form:   <code>arn:aws:acm:us-east-1:123456789012:certificate/12345678-1234-1234-1234-123456789012</code> <br/>
        :return:         """
        pass

    def update_certificate_options(self, Options: CertificateOptions, CertificateArn: str):
        """Updates a certificate. Currently, you can use this function to specify whether to opt in to or out of recording your certificate in a certificate transparency log. For more information, see <a href="https://docs.aws.amazon.com/acm/latest/userguide/acm-bestpractices.html#best-practices-transparency"> Opting Out of Certificate Transparency Logging</a>. 

        :param self:
        :param Options:  Use to update the options for your certificate. Currently, you can specify whether to add your certificate to a transparency log. Certificate transparency makes it possible to detect SSL/TLS certificates that have been mistakenly or maliciously issued. Certificates that have not been logged typically produce an error message in a browser. :<br/>CertificateArn:  ARN of the requested certificate to update. This must be of the form:  <code>arn:aws:acm:us-east-1:<i>account</i>:certificate/<i>12345678-1234-1234-1234-123456789012</i> </code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateTransparencyLoggingPreference</i> You can opt out of certificate transparency logging by specifying the <code>DISABLED</code> option. Opt in by specifying <code>ENABLED</code>. <br/>
        :return:         """
        pass



class ACMPCA(BaseClient):
    """This is the <i>ACM Private CA API Reference</i>. It provides descriptions, syntax, and usage examples for each of the actions and data types involved in creating and managing private certificate authorities (CA) for your organization. The documentation for each action shows the Query API request parameters and the XML response. Alternatively, you can use one of the AWS SDKs to access an API that's tailored to the programming language or platform that you're using. For more information, see <a href="https://aws.amazon.com/tools/#SDKs">AWS SDKs</a>. <note> Each ACM Private CA API action has a throttling limit which determines the number of times the action can be called per second. For more information, see <a href="https://docs.aws.amazon.com/acm-pca/latest/userguide/PcaLimits.html#PcaLimits-api">API Rate Limits in ACM Private CA</a> in the ACM Private CA user guide. </note>"""

    class AuditReportCreatedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_certificate_authority_audit_report` for valid parameters"""
            pass
    class CertificateAuthorityCSRCreatedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `get_certificate_authority_csr` for valid parameters"""
            pass
    class CertificateIssuedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `get_certificate` for valid parameters"""
            pass


    class ListCertificateAuthoritiesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_certificate_authorities` for valid parameters"""
            pass
    class ListTagsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_tags` for valid parameters"""
            pass
    class ListPermissionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_permissions` for valid parameters"""
            pass


    class CreateCertificateAuthorityRequest(Mapping):
        pass
    
    class CreateCertificateAuthorityResponse(Mapping):
        pass
    
    class CertificateAuthorityConfiguration(Mapping):
        pass
    
    class RevocationConfiguration(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreateCertificateAuthorityAuditReportRequest(Mapping):
        pass
    
    class CreateCertificateAuthorityAuditReportResponse(Mapping):
        pass
    
    class CreatePermissionRequest(Mapping):
        pass
    
    class ActionType(object):
        pass
    
    class DeleteCertificateAuthorityRequest(Mapping):
        pass
    
    class DeletePermissionRequest(Mapping):
        pass
    
    class DescribeCertificateAuthorityRequest(Mapping):
        pass
    
    class DescribeCertificateAuthorityResponse(Mapping):
        pass
    
    class DescribeCertificateAuthorityAuditReportRequest(Mapping):
        pass
    
    class DescribeCertificateAuthorityAuditReportResponse(Mapping):
        pass
    
    class GetCertificateRequest(Mapping):
        pass
    
    class GetCertificateResponse(Mapping):
        pass
    
    class GetCertificateAuthorityCertificateRequest(Mapping):
        pass
    
    class GetCertificateAuthorityCertificateResponse(Mapping):
        pass
    
    class GetCertificateAuthorityCsrRequest(Mapping):
        pass
    
    class GetCertificateAuthorityCsrResponse(Mapping):
        pass
    
    class ImportCertificateAuthorityCertificateRequest(Mapping):
        pass
    
    class CertificateBodyBlob(object):
        pass
    
    class CertificateChainBlob(object):
        pass
    
    class IssueCertificateRequest(Mapping):
        pass
    
    class IssueCertificateResponse(Mapping):
        pass
    
    class CsrBlob(object):
        pass
    
    class Validity(Mapping):
        pass
    
    class ListCertificateAuthoritiesRequest(Mapping):
        pass
    
    class ListCertificateAuthoritiesResponse(Mapping):
        pass
    
    class ListPermissionsRequest(Mapping):
        pass
    
    class ListPermissionsResponse(Mapping):
        pass
    
    class ListTagsRequest(Mapping):
        pass
    
    class ListTagsResponse(Mapping):
        pass
    
    class RestoreCertificateAuthorityRequest(Mapping):
        pass
    
    class RevokeCertificateRequest(Mapping):
        pass
    
    class TagCertificateAuthorityRequest(Mapping):
        pass
    
    class UntagCertificateAuthorityRequest(Mapping):
        pass
    
    class UpdateCertificateAuthorityRequest(Mapping):
        pass
    
    def create_certificate_authority(self, CertificateAuthorityType: str, CertificateAuthorityConfiguration: CertificateAuthorityConfiguration, RevocationConfiguration: RevocationConfiguration=None, IdempotencyToken: str=None, Tags: List[Tag]=None) -> CreateCertificateAuthorityResponse:
        """Creates a root or subordinate private certificate authority (CA). You must specify the CA configuration, the certificate revocation list (CRL) configuration, the CA type, and an optional idempotency token to avoid accidental creation of multiple CAs. The CA configuration specifies the name of the algorithm and key size to be used to create the CA private key, the type of signing algorithm that the CA uses, and X.500 subject information. The CRL configuration specifies the CRL expiration period in days (the validity period of the CRL), the Amazon S3 bucket that will contain the CRL, and a CNAME alias for the S3 bucket that is included in certificates issued by the CA. If successful, this action returns the Amazon Resource Name (ARN) of the CA.

        :param self:
        :param Tags:  Key-value pairs that will be attached to the new private CA. You can associate up to 50 tags with a private CA. For information using tags with  IAM to manage permissions, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_iam-tags.html">Controlling Access Using IAM Tags</a>.<br/>IdempotencyToken:  Alphanumeric string that can be used to distinguish between calls to <b>CreateCertificateAuthority</b>. Idempotency tokens time out after five minutes. Therefore, if you call <b>CreateCertificateAuthority</b> multiple times with the same idempotency token within a five minute period, ACM Private CA recognizes that you are requesting only one certificate. As a result, ACM Private CA issues only one. If you change the idempotency token for each call, however, ACM Private CA recognizes that you are requesting multiple certificates.<br/>CertificateAuthorityType:  The type of the certificate authority.<br/>RevocationConfiguration:  Contains a Boolean value that you can use to enable a certification revocation list (CRL) for the CA, the name of the S3 bucket to which ACM Private CA will write the CRL, and an optional CNAME alias that you can use to hide the name of your bucket in the <b>CRL Distribution Points</b> extension of your CA certificate. For more information, see the <a>CrlConfiguration</a> structure. :<br/>CertificateAuthorityConfiguration:  Name and bit size of the private key algorithm, the name of the signing algorithm, and X.500 certificate subject information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subject</i> Structure that contains X.500 distinguished name information for your private CA.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Subject</b>: Structure that contains X.500 distinguished name information for your private CA.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SigningAlgorithm</i> Name of the algorithm your private CA uses to sign certificate requests.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyAlgorithm</i> Type of the public key algorithm and size, in bits, of the key pair that your CA creates when it issues a certificate. When you create a subordinate CA, you must use a key algorithm supported by the parent CA.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>GenerationQualifier</i> Typically a qualifier appended to the name of an individual. Examples include Jr. for junior, Sr. for senior, and III for third.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Pseudonym</i> Typically a shortened version of a longer <b>GivenName</b>. For example, Jonathan is often shortened to John. Elizabeth is often shortened to Beth, Liz, or Eliza.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Initials</i> Concatenation that typically contains the first letter of the <b>GivenName</b>, the first letter of the middle name if one exists, and the first letter of the <b>SurName</b>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>GivenName</i> First name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Surname</i> Family name. In the US and the UK, for example, the surname of an individual is ordered last. In Asian cultures the surname is typically ordered first.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Title</i> A title such as Mr. or Ms., which is pre-pended to the name to refer formally to the certificate subject.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Locality</i> The locality (such as a city or town) in which the certificate subject is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SerialNumber</i> The certificate serial number.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CommonName</i> Fully qualified domain name (FQDN) associated with the certificate subject.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> State in which the subject of the certificate is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DistinguishedNameQualifier</i> Disambiguating information for the certificate subject.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationalUnit</i> A subdivision or unit of the organization (such as sales or finance) with which the certificate subject is affiliated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Organization</i> Legal name of the organization with which the certificate subject is affiliated. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Country</i> Two-digit code that specifies the country in which the certificate subject located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CrlConfiguration</i> Configuration of the certificate revocation list (CRL), if any, maintained by your private CA.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CrlConfiguration</b>: Configuration of the certificate revocation list (CRL), if any, maintained by your private CA.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3BucketName</i> Name of the S3 bucket that contains the CRL. If you do not provide a value for the <b>CustomCname</b> argument, the name of your S3 bucket is placed into the <b>CRL Distribution Points</b> extension of the issued certificate. You can change the name of your bucket by calling the <a>UpdateCertificateAuthority</a> action. You must specify a bucket policy that allows ACM Private CA to write the CRL to your bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomCname</i> Name inserted into the certificate <b>CRL Distribution Points</b> extension that enables the use of an alias for the CRL distribution point. Use this value if you don't want the name of your S3 bucket to be public.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpirationInDays</i> Number of days until a certificate expires.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Boolean value that specifies whether certificate revocation lists (CRLs) are enabled. You can use this value to enable certificate revocation for a new CA when you call the <a>CreateCertificateAuthority</a> action or for an existing CA when you call the <a>UpdateCertificateAuthority</a> action. <br/>
        :return: CertificateAuthorityArn:  If successful, the Amazon Resource Name (ARN) of the certificate authority (CA). This is of the form:   <code>arn:aws:acm-pca:<i>region</i>:<i>account</i>:certificate-authority/<i>12345678-1234-1234-1234-123456789012</i> </code>. <br/>        """
        pass

    def create_certificate_authority_audit_report(self, AuditReportResponseFormat: str, S3BucketName: str, CertificateAuthorityArn: str) -> CreateCertificateAuthorityAuditReportResponse:
        """Creates an audit report that lists every time that your CA private key is used. The report is saved in the Amazon S3 bucket that you specify on input. The <a>IssueCertificate</a> and <a>RevokeCertificate</a> actions use the private key.

        :param self:
        :param AuditReportResponseFormat:  The format in which to create the report. This can be either <b>JSON</b> or <b>CSV</b>.<br/>S3BucketName:  The name of the S3 bucket that will contain the audit report.<br/>CertificateAuthorityArn:  The Amazon Resource Name (ARN) of the CA to be audited. This is of the form:  <code>arn:aws:acm-pca:<i>region</i>:<i>account</i>:certificate-authority/<i>12345678-1234-1234-1234-123456789012</i> </code>.<br/>
        :return: S3Key:  The <b>key</b> that uniquely identifies the report file in your S3 bucket.<br/>AuditReportId:  An alphanumeric string that contains a report identifier.<br/>        """
        pass

    def create_permission(self, Actions: List[ActionType], Principal: str, CertificateAuthorityArn: str, SourceAccount: str=None):
        """Assigns permissions from a private CA to a designated AWS service. Services are specified by their service principals and can be given permission to create and retrieve certificates on a private CA. Services can also be given permission to list the active permissions that the private CA has granted. For ACM to automatically renew your private CA's certificates, you must assign all possible permissions from the CA to the ACM service principal. At this time, you can only assign permissions to ACM (<code>acm.amazonaws.com</code>). Permissions can be revoked with the <a>DeletePermission</a> action and listed with the <a>ListPermissions</a> action.

        :param self:
        :param Actions:  The actions that the specified AWS service principal can use. These include <code>IssueCertificate</code>, <code>GetCertificate</code>, and <code>ListPermissions</code>.<br/>SourceAccount:  The ID of the calling account.<br/>Principal:  The AWS service or identity that receives the permission. At this time, the only valid principal is <code>acm.amazonaws.com</code>.<br/>CertificateAuthorityArn:  The Amazon Resource Name (ARN) of the CA that grants the permissions. You can find the ARN by calling the <a>ListCertificateAuthorities</a> action. This must have the following form:   <code>arn:aws:acm-pca:<i>region</i>:<i>account</i>:certificate-authority/<i>12345678-1234-1234-1234-123456789012</i> </code>. <br/>
        :return:         """
        pass

    def delete_certificate_authority(self, CertificateAuthorityArn: str, PermanentDeletionTimeInDays: int=None):
        """Deletes a private certificate authority (CA). You must provide the Amazon Resource Name (ARN) of the private CA that you want to delete. You can find the ARN by calling the <a>ListCertificateAuthorities</a> action.  <note> Deleting a CA will invalidate other CAs and certificates below it in your CA hierarchy. </note> Before you can delete a CA that you have created and activated, you must disable it. To do this, call the <a>UpdateCertificateAuthority</a> action and set the <b>CertificateAuthorityStatus</b> parameter to <code>DISABLED</code>.  Additionally, you can delete a CA if you are waiting for it to be created (that is, the status of the CA is <code>CREATING</code>). You can also delete it if the CA has been created but you haven't yet imported the signed certificate into ACM Private CA (that is, the status of the CA is <code>PENDING_CERTIFICATE</code>).  When you successfully call <a>DeleteCertificateAuthority</a>, the CA's status changes to <code>DELETED</code>. However, the CA won't be permanently deleted until the restoration period has passed. By default, if you do not set the <code>PermanentDeletionTimeInDays</code> parameter, the CA remains restorable for 30 days. You can set the parameter from 7 to 30 days. The <a>DescribeCertificateAuthority</a> action returns the time remaining in the restoration window of a private CA in the <code>DELETED</code> state. To restore an eligible CA, call the <a>RestoreCertificateAuthority</a> action.

        :param self:
        :param PermanentDeletionTimeInDays:  The number of days to make a CA restorable after it has been deleted. This can be anywhere from 7 to 30 days, with 30 being the default.<br/>CertificateAuthorityArn:  The Amazon Resource Name (ARN) that was returned when you called <a>CreateCertificateAuthority</a>. This must have the following form:   <code>arn:aws:acm-pca:<i>region</i>:<i>account</i>:certificate-authority/<i>12345678-1234-1234-1234-123456789012</i> </code>. <br/>
        :return:         """
        pass

    def delete_permission(self, Principal: str, CertificateAuthorityArn: str, SourceAccount: str=None):
        """Revokes permissions that a private CA assigned to a designated AWS service. Permissions can be created with the <a>CreatePermission</a> action and listed with the <a>ListPermissions</a> action. 

        :param self:
        :param SourceAccount:  The AWS account that calls this action.<br/>Principal:  The AWS service or identity that will have its CA permissions revoked. At this time, the only valid service principal is <code>acm.amazonaws.com</code> <br/>CertificateAuthorityArn:  The Amazon Resource Number (ARN) of the private CA that issued the permissions. You can find the CA's ARN by calling the <a>ListCertificateAuthorities</a> action. This must have the following form:   <code>arn:aws:acm-pca:<i>region</i>:<i>account</i>:certificate-authority/<i>12345678-1234-1234-1234-123456789012</i> </code>. <br/>
        :return:         """
        pass

    def describe_certificate_authority(self, CertificateAuthorityArn: str) -> DescribeCertificateAuthorityResponse:
        """Lists information about your private certificate authority (CA). You specify the private CA on input by its ARN (Amazon Resource Name). The output contains the status of your CA. This can be any of the following:  <ul> <li>  <code>CREATING</code> - ACM Private CA is creating your private certificate authority. </li> <li>  <code>PENDING_CERTIFICATE</code> - The certificate is pending. You must use your ACM Private CA-hosted or on-premises root or subordinate CA to sign your private CA CSR and then import it into PCA.  </li> <li>  <code>ACTIVE</code> - Your private CA is active. </li> <li>  <code>DISABLED</code> - Your private CA has been disabled. </li> <li>  <code>EXPIRED</code> - Your private CA certificate has expired. </li> <li>  <code>FAILED</code> - Your private CA has failed. Your CA can fail because of problems such a network outage or backend AWS failure or other errors. A failed CA can never return to the pending state. You must create a new CA.  </li> <li>  <code>DELETED</code> - Your private CA is within the restoration period, after which it is permanently deleted. The length of time remaining in the CA's restoration period is also included in this action's output. </li> </ul>

        :param self:
        :param CertificateAuthorityArn:  The Amazon Resource Name (ARN) that was returned when you called <a>CreateCertificateAuthority</a>. This must be of the form:   <code>arn:aws:acm-pca:<i>region</i>:<i>account</i>:certificate-authority/<i>12345678-1234-1234-1234-123456789012</i> </code>. <br/>
        :return: CertificateAuthority:  A <a>CertificateAuthority</a> structure that contains information about your private CA.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestorableUntil</i> The period during which a deleted CA can be restored. For more information, see the <code>PermanentDeletionTimeInDays</code> parameter of the <a>DeleteCertificateAuthorityRequest</a> action. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RevocationConfiguration</i> Information about the certificate revocation list (CRL) created and maintained by your private CA. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RevocationConfiguration</b>: Information about the certificate revocation list (CRL) created and maintained by your private CA. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateAuthorityConfiguration</i> Your private CA configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CertificateAuthorityConfiguration</b>: Your private CA configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureReason</i> Reason the request to create your private CA failed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotAfter</i> Date and time after which your private CA certificate is not valid.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotBefore</i> Date and time before which your private CA certificate is not valid.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Status of your private CA.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Serial</i> Serial number of your private CA.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> Type of your private CA.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastStateChangeAt</i> Date and time at which your private CA was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i> Date and time at which your private CA was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> Amazon Resource Name (ARN) for your private certificate authority (CA). The format is <code> <i>12345678-1234-1234-1234-123456789012</i> </code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SigningAlgorithm</i> Name of the algorithm your private CA uses to sign certificate requests.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyAlgorithm</i> Type of the public key algorithm and size, in bits, of the key pair that your CA creates when it issues a certificate. When you create a subordinate CA, you must use a key algorithm supported by the parent CA.<br/>Structure that contains X.500 distinguished name information for your private CA.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Subject</b>: Structure that contains X.500 distinguished name information for your private CA.Configuration of the certificate revocation list (CRL), if any, maintained by your private CA.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CrlConfiguration</b>: Configuration of the certificate revocation list (CRL), if any, maintained by your private CA.        """
        pass

    def describe_certificate_authority_audit_report(self, AuditReportId: str, CertificateAuthorityArn: str) -> DescribeCertificateAuthorityAuditReportResponse:
        """Lists information about a specific audit report created by calling the <a>CreateCertificateAuthorityAuditReport</a> action. Audit information is created every time the certificate authority (CA) private key is used. The private key is used when you call the <a>IssueCertificate</a> action or the <a>RevokeCertificate</a> action. 

        :param self:
        :param AuditReportId:  The report ID returned by calling the <a>CreateCertificateAuthorityAuditReport</a> action.<br/>CertificateAuthorityArn:  The Amazon Resource Name (ARN) of the private CA. This must be of the form:  <code>arn:aws:acm-pca:<i>region</i>:<i>account</i>:certificate-authority/<i>12345678-1234-1234-1234-123456789012</i> </code>. <br/>
        :return: CreatedAt:  The date and time at which the report was created.<br/>S3Key:  S3 <b>key</b> that uniquely identifies the report file in your S3 bucket.<br/>S3BucketName:  Name of the S3 bucket that contains the report.<br/>AuditReportStatus:  Specifies whether report creation is in progress, has succeeded, or has failed.<br/>        """
        pass

    def get_certificate(self, CertificateArn: str, CertificateAuthorityArn: str) -> GetCertificateResponse:
        """Retrieves a certificate from your private CA. The ARN of the certificate is returned when you call the <a>IssueCertificate</a> action. You must specify both the ARN of your private CA and the ARN of the issued certificate when calling the <b>GetCertificate</b> action. You can retrieve the certificate if it is in the <b>ISSUED</b> state. You can call the <a>CreateCertificateAuthorityAuditReport</a> action to create a report that contains information about all of the certificates issued and revoked by your private CA. 

        :param self:
        :param CertificateArn:  The ARN of the issued certificate. The ARN contains the certificate serial number and must be in the following form:   <code>arn:aws:acm-pca:<i>region</i>:<i>account</i>:certificate-authority/<i>12345678-1234-1234-1234-123456789012</i>/certificate/<i>286535153982981100925020015808220737245</i> </code> <br/>CertificateAuthorityArn:  The Amazon Resource Name (ARN) that was returned when you called <a>CreateCertificateAuthority</a>. This must be of the form:   <code>arn:aws:acm-pca:<i>region</i>:<i>account</i>:certificate-authority/<i>12345678-1234-1234-1234-123456789012</i> </code>. <br/>
        :return: CertificateChain:  The base64 PEM-encoded certificate chain that chains up to the on-premises root CA certificate that you used to sign your private CA certificate. <br/>Certificate:  The base64 PEM-encoded certificate specified by the <code>CertificateArn</code> parameter.<br/>        """
        pass

    def get_certificate_authority_certificate(self, CertificateAuthorityArn: str) -> GetCertificateAuthorityCertificateResponse:
        """Retrieves the certificate and certificate chain for your private certificate authority (CA). Both the certificate and the chain are base64 PEM-encoded. The chain does not include the CA certificate. Each certificate in the chain signs the one before it. 

        :param self:
        :param CertificateAuthorityArn:  The Amazon Resource Name (ARN) of your private CA. This is of the form:  <code>arn:aws:acm-pca:<i>region</i>:<i>account</i>:certificate-authority/<i>12345678-1234-1234-1234-123456789012</i> </code>. <br/>
        :return: CertificateChain:  Base64-encoded certificate chain that includes any intermediate certificates and chains up to root on-premises certificate that you used to sign your private CA certificate. The chain does not include your private CA certificate. If this is a root CA, the value will be null.<br/>Certificate:  Base64-encoded certificate authority (CA) certificate.<br/>        """
        pass

    def get_certificate_authority_csr(self, CertificateAuthorityArn: str) -> GetCertificateAuthorityCsrResponse:
        """Retrieves the certificate signing request (CSR) for your private certificate authority (CA). The CSR is created when you call the <a>CreateCertificateAuthority</a> action. Sign the CSR with your ACM Private CA-hosted or on-premises root or subordinate CA. Then import the signed certificate back into ACM Private CA by calling the <a>ImportCertificateAuthorityCertificate</a> action. The CSR is returned as a base64 PEM-encoded string. 

        :param self:
        :param CertificateAuthorityArn:  The Amazon Resource Name (ARN) that was returned when you called the <a>CreateCertificateAuthority</a> action. This must be of the form:   <code>arn:aws:acm-pca:<i>region</i>:<i>account</i>:certificate-authority/<i>12345678-1234-1234-1234-123456789012</i> </code> <br/>
        :return: Csr:  The base64 PEM-encoded certificate signing request (CSR) for your private CA certificate.<br/>        """
        pass

    def import_certificate_authority_certificate(self, Certificate: CertificateBodyBlob, CertificateAuthorityArn: str, CertificateChain: CertificateChainBlob=None):
        """Imports a signed private CA certificate into ACM Private CA. This action is used when you are using a chain of trust whose root is located outside ACM Private CA. Before you can call this action, the following preparations must in place: <ol> <li> In ACM Private CA, call the <a>CreateCertificateAuthority</a> action to create the private CA that that you plan to back with the imported certificate. </li> <li> Call the <a>GetCertificateAuthorityCsr</a> action to generate a certificate signing request (CSR). </li> <li> Sign the CSR using a root or intermediate CA hosted either by an on-premises PKI hierarchy or a commercial CA.. </li> <li> Create a certificate chain and copy the signed certificate and the certificate chain to your working directory. </li> </ol> The following requirements apply when you import a CA certificate. <ul> <li> You cannot import a non-self-signed certificate for use as a root CA. </li> <li> You cannot import a self-signed certificate for use as a subordinate CA. </li> <li> Your certificate chain must not include the private CA certificate that you are importing. </li> <li> Your ACM Private CA-hosted or on-premises CA certificate must be the last certificate in your chain. The subordinate certificate, if any, that your root CA signed must be next to last. The subordinate certificate signed by the preceding subordinate CA must come next, and so on until your chain is built.  </li> <li> The chain must be PEM-encoded. </li> </ul>

        :param self:
        :param CertificateChain:  A PEM-encoded file that contains all of your certificates, other than the certificate you're importing, chaining up to your root CA. Your ACM Private CA-hosted or on-premises root certificate is the last in the chain, and each certificate in the chain signs the one preceding.  This parameter must be supplied when you import a subordinate CA. When you import a root CA, there is no chain.<br/>Certificate:  The PEM-encoded certificate for a private CA. This may be a self-signed certificate in the case of a root CA, or it may be signed by another CA that you control.<br/>CertificateAuthorityArn:  The Amazon Resource Name (ARN) that was returned when you called <a>CreateCertificateAuthority</a>. This must be of the form:   <code>arn:aws:acm-pca:<i>region</i>:<i>account</i>:certificate-authority/<i>12345678-1234-1234-1234-123456789012</i> </code> <br/>
        :return:         """
        pass

    def issue_certificate(self, Validity: Validity, SigningAlgorithm: str, Csr: CsrBlob, CertificateAuthorityArn: str, TemplateArn: str=None, IdempotencyToken: str=None) -> IssueCertificateResponse:
        """Uses your private certificate authority (CA) to issue a client certificate. This action returns the Amazon Resource Name (ARN) of the certificate. You can retrieve the certificate by calling the <a>GetCertificate</a> action and specifying the ARN.  <note> You cannot use the ACM <b>ListCertificateAuthorities</b> action to retrieve the ARNs of the certificates that you issue by using ACM Private CA. </note>

        :param self:
        :param IdempotencyToken:  Custom string that can be used to distinguish between calls to the <b>IssueCertificate</b> action. Idempotency tokens time out after one hour. Therefore, if you call <b>IssueCertificate</b> multiple times with the same idempotency token within 5 minutes, ACM Private CA recognizes that you are requesting only one certificate and will issue only one. If you change the idempotency token for each call, PCA recognizes that you are requesting multiple certificates.<br/>Validity:  The type of the validity period.:<br/>TemplateArn:  Specifies a custom configuration template to use when issuing a certificate. If this parameter is not provided, ACM Private CA defaults to the <code>EndEntityCertificate/V1</code> template. The following service-owned <code>TemplateArn</code> values are supported by ACM Private CA:  <ul> <li> arn:aws:acm-pca:::template/EndEntityCertificate/V1 </li> <li> arn:aws:acm-pca:::template/SubordinateCACertificate_PathLen0/V1 </li> <li> arn:aws:acm-pca:::template/SubordinateCACertificate_PathLen1/V1 </li> <li> arn:aws:acm-pca:::template/SubordinateCACertificate_PathLen2/V1 </li> <li> arn:aws:acm-pca:::template/SubordinateCACertificate_PathLen3/V1 </li> <li> arn:aws:acm-pca:::template/RootCACertificate/V1 </li> </ul> For more information, see <a href="https://docs.aws.amazon.com/acm-pca/latest/userguide/UsingTemplates.html">Using Templates</a>.<br/>SigningAlgorithm:  The name of the algorithm that will be used to sign the certificate to be issued.<br/>Csr:  The certificate signing request (CSR) for the certificate you want to issue. You can use the following OpenSSL command to create the CSR and a 2048 bit RSA private key.   <code>openssl req -new -newkey rsa:2048 -days 365 -keyout private/test_cert_priv_key.pem -out csr/test_cert_.csr</code>  If you have a configuration file, you can use the following OpenSSL command. The <code>usr_cert</code> block in the configuration file contains your X509 version 3 extensions.   <code>openssl req -new -config openssl_rsa.cnf -extensions usr_cert -newkey rsa:2048 -days -365 -keyout private/test_cert_priv_key.pem -out csr/test_cert_.csr</code> <br/>CertificateAuthorityArn:  The Amazon Resource Name (ARN) that was returned when you called <a>CreateCertificateAuthority</a>. This must be of the form:  <code>arn:aws:acm-pca:<i>region</i>:<i>account</i>:certificate-authority/<i>12345678-1234-1234-1234-123456789012</i> </code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> Specifies whether the <code>Value</code> parameter represents days, months, or years.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> Time period.<br/>
        :return: CertificateArn:  The Amazon Resource Name (ARN) of the issued certificate and the certificate serial number. This is of the form:  <code>arn:aws:acm-pca:<i>region</i>:<i>account</i>:certificate-authority/<i>12345678-1234-1234-1234-123456789012</i>/certificate/<i>286535153982981100925020015808220737245</i> </code> <br/>        """
        pass

    def list_certificate_authorities(self, NextToken: str=None, MaxResults: int=None) -> ListCertificateAuthoritiesResponse:
        """Lists the private certificate authorities that you created by using the <a>CreateCertificateAuthority</a> action.

        :param self:
        :param MaxResults:  Use this parameter when paginating results to specify the maximum number of items to return in the response on each page. If additional items exist beyond the number you specify, the <code>NextToken</code> element is sent in the response. Use this <code>NextToken</code> value in a subsequent request to retrieve additional items.<br/>NextToken:  Use this parameter when paginating results in a subsequent request after you receive a response with truncated results. Set it to the value of the <code>NextToken</code> parameter from the response you just received.<br/>
        :return: NextToken:  When the list is truncated, this value is present and should be used for the <code>NextToken</code> parameter in a subsequent pagination request.<br/>CertificateAuthorities:  Summary information about each certificate authority you have created.<br/>        """
        pass

    def list_permissions(self, CertificateAuthorityArn: str, NextToken: str=None, MaxResults: int=None) -> ListPermissionsResponse:
        """Lists all the permissions, if any, that have been assigned by a private CA. Permissions can be granted with the <a>CreatePermission</a> action and revoked with the <a>DeletePermission</a> action.

        :param self:
        :param MaxResults:  When paginating results, use this parameter to specify the maximum number of items to return in the response. If additional items exist beyond the number you specify, the <b>NextToken</b> element is sent in the response. Use this <b>NextToken</b> value in a subsequent request to retrieve additional items.<br/>NextToken:  When paginating results, use this parameter in a subsequent request after you receive a response with truncated results. Set it to the value of <b>NextToken</b> from the response you just received.<br/>CertificateAuthorityArn:  The Amazon Resource Number (ARN) of the private CA to inspect. You can find the ARN by calling the <a>ListCertificateAuthorities</a> action. This must be of the form: <code>arn:aws:acm-pca:region:account:certificate-authority/12345678-1234-1234-1234-123456789012</code> You can get a private CA's ARN by running the <a>ListCertificateAuthorities</a> action.<br/>
        :return: NextToken:  When the list is truncated, this value is present and should be used for the <b>NextToken</b> parameter in a subsequent pagination request. <br/>Permissions:  Summary information about each permission assigned by the specified private CA, including the action enabled, the policy provided, and the time of creation.<br/>        """
        pass

    def list_tags(self, CertificateAuthorityArn: str, NextToken: str=None, MaxResults: int=None) -> ListTagsResponse:
        """Lists the tags, if any, that are associated with your private CA. Tags are labels that you can use to identify and organize your CAs. Each tag consists of a key and an optional value. Call the <a>TagCertificateAuthority</a> action to add one or more tags to your CA. Call the <a>UntagCertificateAuthority</a> action to remove tags. 

        :param self:
        :param MaxResults:  Use this parameter when paginating results to specify the maximum number of items to return in the response. If additional items exist beyond the number you specify, the <b>NextToken</b> element is sent in the response. Use this <b>NextToken</b> value in a subsequent request to retrieve additional items.<br/>NextToken:  Use this parameter when paginating results in a subsequent request after you receive a response with truncated results. Set it to the value of <b>NextToken</b> from the response you just received.<br/>CertificateAuthorityArn:  The Amazon Resource Name (ARN) that was returned when you called the <a>CreateCertificateAuthority</a> action. This must be of the form:   <code>arn:aws:acm-pca:<i>region</i>:<i>account</i>:certificate-authority/<i>12345678-1234-1234-1234-123456789012</i> </code> <br/>
        :return: NextToken:  When the list is truncated, this value is present and should be used for the <b>NextToken</b> parameter in a subsequent pagination request. <br/>Tags:  The tags associated with your private CA.<br/>        """
        pass

    def restore_certificate_authority(self, CertificateAuthorityArn: str):
        """Restores a certificate authority (CA) that is in the <code>DELETED</code> state. You can restore a CA during the period that you defined in the <b>PermanentDeletionTimeInDays</b> parameter of the <a>DeleteCertificateAuthority</a> action. Currently, you can specify 7 to 30 days. If you did not specify a <b>PermanentDeletionTimeInDays</b> value, by default you can restore the CA at any time in a 30 day period. You can check the time remaining in the restoration period of a private CA in the <code>DELETED</code> state by calling the <a>DescribeCertificateAuthority</a> or <a>ListCertificateAuthorities</a> actions. The status of a restored CA is set to its pre-deletion status when the <b>RestoreCertificateAuthority</b> action returns. To change its status to <code>ACTIVE</code>, call the <a>UpdateCertificateAuthority</a> action. If the private CA was in the <code>PENDING_CERTIFICATE</code> state at deletion, you must use the <a>ImportCertificateAuthorityCertificate</a> action to import a certificate authority into the private CA before it can be activated. You cannot restore a CA after the restoration period has ended.

        :param self:
        :param CertificateAuthorityArn:  The Amazon Resource Name (ARN) that was returned when you called the <a>CreateCertificateAuthority</a> action. This must be of the form:   <code>arn:aws:acm-pca:<i>region</i>:<i>account</i>:certificate-authority/<i>12345678-1234-1234-1234-123456789012</i> </code> <br/>
        :return:         """
        pass

    def revoke_certificate(self, RevocationReason: str, CertificateSerial: str, CertificateAuthorityArn: str):
        """Revokes a certificate that was issued inside ACM Private CA. If you enable a certificate revocation list (CRL) when you create or update your private CA, information about the revoked certificates will be included in the CRL. ACM Private CA writes the CRL to an S3 bucket that you specify. For more information about revocation, see the <a>CrlConfiguration</a> structure. ACM Private CA also writes revocation information to the audit report. For more information, see <a>CreateCertificateAuthorityAuditReport</a>.  <note> You cannot revoke a root CA self-signed certificate. </note>

        :param self:
        :param RevocationReason:  Specifies why you revoked the certificate.<br/>CertificateSerial:  Serial number of the certificate to be revoked. This must be in hexadecimal format. You can retrieve the serial number by calling <a>GetCertificate</a> with the Amazon Resource Name (ARN) of the certificate you want and the ARN of your private CA. The <b>GetCertificate</b> action retrieves the certificate in the PEM format. You can use the following OpenSSL command to list the certificate in text format and copy the hexadecimal serial number.   <code>openssl x509 -in <i>file_path</i> -text -noout</code>  You can also copy the serial number from the console or use the <a href="https://docs.aws.amazon.com/acm/latest/APIReference/API_DescribeCertificate.html">DescribeCertificate</a> action in the <i>AWS Certificate Manager API Reference</i>. <br/>CertificateAuthorityArn:  Amazon Resource Name (ARN) of the private CA that issued the certificate to be revoked. This must be of the form:  <code>arn:aws:acm-pca:<i>region</i>:<i>account</i>:certificate-authority/<i>12345678-1234-1234-1234-123456789012</i> </code> <br/>
        :return:         """
        pass

    def tag_certificate_authority(self, Tags: List[Tag], CertificateAuthorityArn: str):
        """Adds one or more tags to your private CA. Tags are labels that you can use to identify and organize your AWS resources. Each tag consists of a key and an optional value. You specify the private CA on input by its Amazon Resource Name (ARN). You specify the tag by using a key-value pair. You can apply a tag to just one private CA if you want to identify a specific characteristic of that CA, or you can apply the same tag to multiple private CAs if you want to filter for a common relationship among those CAs. To remove one or more tags, use the <a>UntagCertificateAuthority</a> action. Call the <a>ListTags</a> action to see what tags are associated with your CA. 

        :param self:
        :param Tags:  List of tags to be associated with the CA.<br/>CertificateAuthorityArn:  The Amazon Resource Name (ARN) that was returned when you called <a>CreateCertificateAuthority</a>. This must be of the form:   <code>arn:aws:acm-pca:<i>region</i>:<i>account</i>:certificate-authority/<i>12345678-1234-1234-1234-123456789012</i> </code> <br/>
        :return:         """
        pass

    def untag_certificate_authority(self, Tags: List[Tag], CertificateAuthorityArn: str):
        """Remove one or more tags from your private CA. A tag consists of a key-value pair. If you do not specify the value portion of the tag when calling this action, the tag will be removed regardless of value. If you specify a value, the tag is removed only if it is associated with the specified value. To add tags to a private CA, use the <a>TagCertificateAuthority</a>. Call the <a>ListTags</a> action to see what tags are associated with your CA. 

        :param self:
        :param Tags:  List of tags to be removed from the CA.<br/>CertificateAuthorityArn:  The Amazon Resource Name (ARN) that was returned when you called <a>CreateCertificateAuthority</a>. This must be of the form:   <code>arn:aws:acm-pca:<i>region</i>:<i>account</i>:certificate-authority/<i>12345678-1234-1234-1234-123456789012</i> </code> <br/>
        :return:         """
        pass

    def update_certificate_authority(self, CertificateAuthorityArn: str, RevocationConfiguration: RevocationConfiguration=None, Status: str=None):
        """Updates the status or configuration of a private certificate authority (CA). Your private CA must be in the <code>ACTIVE</code> or <code>DISABLED</code> state before you can update it. You can disable a private CA that is in the <code>ACTIVE</code> state or make a CA that is in the <code>DISABLED</code> state active again.

        :param self:
        :param Status:  Status of your private CA.<br/>RevocationConfiguration:  Revocation information for your private CA.:<br/>CertificateAuthorityArn:  Amazon Resource Name (ARN) of the private CA that issued the certificate to be revoked. This must be of the form:  <code>arn:aws:acm-pca:<i>region</i>:<i>account</i>:certificate-authority/<i>12345678-1234-1234-1234-123456789012</i> </code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CrlConfiguration</i> Configuration of the certificate revocation list (CRL), if any, maintained by your private CA.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CrlConfiguration</b>: Configuration of the certificate revocation list (CRL), if any, maintained by your private CA.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3BucketName</i> Name of the S3 bucket that contains the CRL. If you do not provide a value for the <b>CustomCname</b> argument, the name of your S3 bucket is placed into the <b>CRL Distribution Points</b> extension of the issued certificate. You can change the name of your bucket by calling the <a>UpdateCertificateAuthority</a> action. You must specify a bucket policy that allows ACM Private CA to write the CRL to your bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomCname</i> Name inserted into the certificate <b>CRL Distribution Points</b> extension that enables the use of an alias for the CRL distribution point. Use this value if you don't want the name of your S3 bucket to be public.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpirationInDays</i> Number of days until a certificate expires.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Boolean value that specifies whether certificate revocation lists (CRLs) are enabled. You can use this value to enable certificate revocation for a new CA when you call the <a>CreateCertificateAuthority</a> action or for an existing CA when you call the <a>UpdateCertificateAuthority</a> action. <br/>
        :return:         """
        pass



class AlexaForBusiness(BaseClient):
    """Alexa for Business helps you use Alexa in your organization. Alexa for Business provides you with the tools to manage Alexa devices, enroll your users, and assign skills, at scale. You can build your own context-aware voice skills using the Alexa Skills Kit and the Alexa for Business API operations. You can also make these available as private skills for your organization. Alexa for Business makes it efficient to voice-enable your products and services, thus providing context-aware voice experiences for your customers. Device makers building with the Alexa Voice Service (AVS) can create fully integrated solutions, register their products with Alexa for Business, and manage them as shared devices in their organization. """


    class ListSkillsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_skills` for valid parameters"""
            pass
    class SearchUsersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `search_users` for valid parameters"""
            pass
    class ListTagsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_tags` for valid parameters"""
            pass
    class SearchProfilesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `search_profiles` for valid parameters"""
            pass
    class SearchSkillGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `search_skill_groups` for valid parameters"""
            pass
    class SearchDevicesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `search_devices` for valid parameters"""
            pass
    class SearchRoomsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `search_rooms` for valid parameters"""
            pass
    class ListBusinessReportSchedulesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_business_report_schedules` for valid parameters"""
            pass
    class ListConferenceProvidersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_conference_providers` for valid parameters"""
            pass
    class ListDeviceEventsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_device_events` for valid parameters"""
            pass
    class ListSkillsStoreCategoriesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_skills_store_categories` for valid parameters"""
            pass
    class ListSkillsStoreSkillsByCategoryPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_skills_store_skills_by_category` for valid parameters"""
            pass
    class ListSmartHomeAppliancesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_smart_home_appliances` for valid parameters"""
            pass


    class ApproveSkillRequest(Mapping):
        pass
    
    class ApproveSkillResponse(Mapping):
        pass
    
    class AssociateContactWithAddressBookRequest(Mapping):
        pass
    
    class AssociateContactWithAddressBookResponse(Mapping):
        pass
    
    class AssociateDeviceWithNetworkProfileRequest(Mapping):
        pass
    
    class AssociateDeviceWithNetworkProfileResponse(Mapping):
        pass
    
    class AssociateDeviceWithRoomRequest(Mapping):
        pass
    
    class AssociateDeviceWithRoomResponse(Mapping):
        pass
    
    class AssociateSkillGroupWithRoomRequest(Mapping):
        pass
    
    class AssociateSkillGroupWithRoomResponse(Mapping):
        pass
    
    class AssociateSkillWithSkillGroupRequest(Mapping):
        pass
    
    class AssociateSkillWithSkillGroupResponse(Mapping):
        pass
    
    class AssociateSkillWithUsersRequest(Mapping):
        pass
    
    class AssociateSkillWithUsersResponse(Mapping):
        pass
    
    class CreateAddressBookRequest(Mapping):
        pass
    
    class CreateAddressBookResponse(Mapping):
        pass
    
    class CreateBusinessReportScheduleRequest(Mapping):
        pass
    
    class CreateBusinessReportScheduleResponse(Mapping):
        pass
    
    class BusinessReportContentRange(Mapping):
        pass
    
    class BusinessReportRecurrence(Mapping):
        pass
    
    class CreateConferenceProviderRequest(Mapping):
        pass
    
    class CreateConferenceProviderResponse(Mapping):
        pass
    
    class IPDialIn(Mapping):
        pass
    
    class PSTNDialIn(Mapping):
        pass
    
    class MeetingSetting(Mapping):
        pass
    
    class CreateContactRequest(Mapping):
        pass
    
    class CreateContactResponse(Mapping):
        pass
    
    class PhoneNumber(Mapping):
        pass
    
    class SipAddress(Mapping):
        pass
    
    class CreateGatewayGroupRequest(Mapping):
        pass
    
    class CreateGatewayGroupResponse(Mapping):
        pass
    
    class CreateNetworkProfileRequest(Mapping):
        pass
    
    class CreateNetworkProfileResponse(Mapping):
        pass
    
    class TrustAnchor(object):
        pass
    
    class CreateProfileRequest(Mapping):
        pass
    
    class CreateProfileResponse(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class CreateRoomRequest(Mapping):
        pass
    
    class CreateRoomResponse(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreateSkillGroupRequest(Mapping):
        pass
    
    class CreateSkillGroupResponse(Mapping):
        pass
    
    class CreateUserRequest(Mapping):
        pass
    
    class CreateUserResponse(Mapping):
        pass
    
    class DeleteAddressBookRequest(Mapping):
        pass
    
    class DeleteAddressBookResponse(Mapping):
        pass
    
    class DeleteBusinessReportScheduleRequest(Mapping):
        pass
    
    class DeleteBusinessReportScheduleResponse(Mapping):
        pass
    
    class DeleteConferenceProviderRequest(Mapping):
        pass
    
    class DeleteConferenceProviderResponse(Mapping):
        pass
    
    class DeleteContactRequest(Mapping):
        pass
    
    class DeleteContactResponse(Mapping):
        pass
    
    class DeleteDeviceRequest(Mapping):
        pass
    
    class DeleteDeviceResponse(Mapping):
        pass
    
    class DeleteDeviceUsageDataRequest(Mapping):
        pass
    
    class DeleteDeviceUsageDataResponse(Mapping):
        pass
    
    class DeleteGatewayGroupRequest(Mapping):
        pass
    
    class DeleteGatewayGroupResponse(Mapping):
        pass
    
    class DeleteNetworkProfileRequest(Mapping):
        pass
    
    class DeleteNetworkProfileResponse(Mapping):
        pass
    
    class DeleteProfileRequest(Mapping):
        pass
    
    class DeleteProfileResponse(Mapping):
        pass
    
    class DeleteRoomRequest(Mapping):
        pass
    
    class DeleteRoomResponse(Mapping):
        pass
    
    class DeleteRoomSkillParameterRequest(Mapping):
        pass
    
    class DeleteRoomSkillParameterResponse(Mapping):
        pass
    
    class DeleteSkillAuthorizationRequest(Mapping):
        pass
    
    class DeleteSkillAuthorizationResponse(Mapping):
        pass
    
    class DeleteSkillGroupRequest(Mapping):
        pass
    
    class DeleteSkillGroupResponse(Mapping):
        pass
    
    class DeleteUserRequest(Mapping):
        pass
    
    class DeleteUserResponse(Mapping):
        pass
    
    class DisassociateContactFromAddressBookRequest(Mapping):
        pass
    
    class DisassociateContactFromAddressBookResponse(Mapping):
        pass
    
    class DisassociateDeviceFromRoomRequest(Mapping):
        pass
    
    class DisassociateDeviceFromRoomResponse(Mapping):
        pass
    
    class DisassociateSkillFromSkillGroupRequest(Mapping):
        pass
    
    class DisassociateSkillFromSkillGroupResponse(Mapping):
        pass
    
    class DisassociateSkillFromUsersRequest(Mapping):
        pass
    
    class DisassociateSkillFromUsersResponse(Mapping):
        pass
    
    class DisassociateSkillGroupFromRoomRequest(Mapping):
        pass
    
    class DisassociateSkillGroupFromRoomResponse(Mapping):
        pass
    
    class ForgetSmartHomeAppliancesRequest(Mapping):
        pass
    
    class ForgetSmartHomeAppliancesResponse(Mapping):
        pass
    
    class GetAddressBookRequest(Mapping):
        pass
    
    class GetAddressBookResponse(Mapping):
        pass
    
    class GetConferencePreferenceRequest(Mapping):
        pass
    
    class GetConferencePreferenceResponse(Mapping):
        pass
    
    class GetConferenceProviderRequest(Mapping):
        pass
    
    class GetConferenceProviderResponse(Mapping):
        pass
    
    class GetContactRequest(Mapping):
        pass
    
    class GetContactResponse(Mapping):
        pass
    
    class GetDeviceRequest(Mapping):
        pass
    
    class GetDeviceResponse(Mapping):
        pass
    
    class GetGatewayRequest(Mapping):
        pass
    
    class GetGatewayResponse(Mapping):
        pass
    
    class GetGatewayGroupRequest(Mapping):
        pass
    
    class GetGatewayGroupResponse(Mapping):
        pass
    
    class GetInvitationConfigurationRequest(Mapping):
        pass
    
    class GetInvitationConfigurationResponse(Mapping):
        pass
    
    class GetNetworkProfileRequest(Mapping):
        pass
    
    class GetNetworkProfileResponse(Mapping):
        pass
    
    class GetProfileRequest(Mapping):
        pass
    
    class GetProfileResponse(Mapping):
        pass
    
    class GetRoomRequest(Mapping):
        pass
    
    class GetRoomResponse(Mapping):
        pass
    
    class GetRoomSkillParameterRequest(Mapping):
        pass
    
    class GetRoomSkillParameterResponse(Mapping):
        pass
    
    class GetSkillGroupRequest(Mapping):
        pass
    
    class GetSkillGroupResponse(Mapping):
        pass
    
    class ListBusinessReportSchedulesRequest(Mapping):
        pass
    
    class ListBusinessReportSchedulesResponse(Mapping):
        pass
    
    class ListConferenceProvidersRequest(Mapping):
        pass
    
    class ListConferenceProvidersResponse(Mapping):
        pass
    
    class ListDeviceEventsRequest(Mapping):
        pass
    
    class ListDeviceEventsResponse(Mapping):
        pass
    
    class ListGatewayGroupsRequest(Mapping):
        pass
    
    class ListGatewayGroupsResponse(Mapping):
        pass
    
    class ListGatewaysRequest(Mapping):
        pass
    
    class ListGatewaysResponse(Mapping):
        pass
    
    class ListSkillsRequest(Mapping):
        pass
    
    class ListSkillsResponse(Mapping):
        pass
    
    class ListSkillsStoreCategoriesRequest(Mapping):
        pass
    
    class ListSkillsStoreCategoriesResponse(Mapping):
        pass
    
    class ListSkillsStoreSkillsByCategoryRequest(Mapping):
        pass
    
    class ListSkillsStoreSkillsByCategoryResponse(Mapping):
        pass
    
    class CategoryId(object):
        pass
    
    class ListSmartHomeAppliancesRequest(Mapping):
        pass
    
    class ListSmartHomeAppliancesResponse(Mapping):
        pass
    
    class ListTagsRequest(Mapping):
        pass
    
    class ListTagsResponse(Mapping):
        pass
    
    class PutConferencePreferenceRequest(Mapping):
        pass
    
    class PutConferencePreferenceResponse(Mapping):
        pass
    
    class ConferencePreference(Mapping):
        pass
    
    class PutInvitationConfigurationRequest(Mapping):
        pass
    
    class PutInvitationConfigurationResponse(Mapping):
        pass
    
    class SkillId(object):
        pass
    
    class PutRoomSkillParameterRequest(Mapping):
        pass
    
    class PutRoomSkillParameterResponse(Mapping):
        pass
    
    class RoomSkillParameter(Mapping):
        pass
    
    class PutSkillAuthorizationRequest(Mapping):
        pass
    
    class PutSkillAuthorizationResponse(Mapping):
        pass
    
    class AuthorizationResult(object):
        pass
    
    class RegisterAVSDeviceRequest(Mapping):
        pass
    
    class RegisterAVSDeviceResponse(Mapping):
        pass
    
    class RejectSkillRequest(Mapping):
        pass
    
    class RejectSkillResponse(Mapping):
        pass
    
    class ResolveRoomRequest(Mapping):
        pass
    
    class ResolveRoomResponse(Mapping):
        pass
    
    class RevokeInvitationRequest(Mapping):
        pass
    
    class RevokeInvitationResponse(Mapping):
        pass
    
    class SearchAddressBooksRequest(Mapping):
        pass
    
    class SearchAddressBooksResponse(Mapping):
        pass
    
    class Filter(Mapping):
        pass
    
    class Sort(Mapping):
        pass
    
    class SearchContactsRequest(Mapping):
        pass
    
    class SearchContactsResponse(Mapping):
        pass
    
    class SearchDevicesRequest(Mapping):
        pass
    
    class SearchDevicesResponse(Mapping):
        pass
    
    class SearchNetworkProfilesRequest(Mapping):
        pass
    
    class SearchNetworkProfilesResponse(Mapping):
        pass
    
    class SearchProfilesRequest(Mapping):
        pass
    
    class SearchProfilesResponse(Mapping):
        pass
    
    class SearchRoomsRequest(Mapping):
        pass
    
    class SearchRoomsResponse(Mapping):
        pass
    
    class SearchSkillGroupsRequest(Mapping):
        pass
    
    class SearchSkillGroupsResponse(Mapping):
        pass
    
    class SearchUsersRequest(Mapping):
        pass
    
    class SearchUsersResponse(Mapping):
        pass
    
    class SendAnnouncementRequest(Mapping):
        pass
    
    class SendAnnouncementResponse(Mapping):
        pass
    
    class Content(Mapping):
        pass
    
    class SendInvitationRequest(Mapping):
        pass
    
    class SendInvitationResponse(Mapping):
        pass
    
    class StartDeviceSyncRequest(Mapping):
        pass
    
    class StartDeviceSyncResponse(Mapping):
        pass
    
    class Feature(object):
        pass
    
    class StartSmartHomeApplianceDiscoveryRequest(Mapping):
        pass
    
    class StartSmartHomeApplianceDiscoveryResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateAddressBookRequest(Mapping):
        pass
    
    class UpdateAddressBookResponse(Mapping):
        pass
    
    class UpdateBusinessReportScheduleRequest(Mapping):
        pass
    
    class UpdateBusinessReportScheduleResponse(Mapping):
        pass
    
    class UpdateConferenceProviderRequest(Mapping):
        pass
    
    class UpdateConferenceProviderResponse(Mapping):
        pass
    
    class UpdateContactRequest(Mapping):
        pass
    
    class UpdateContactResponse(Mapping):
        pass
    
    class UpdateDeviceRequest(Mapping):
        pass
    
    class UpdateDeviceResponse(Mapping):
        pass
    
    class UpdateGatewayRequest(Mapping):
        pass
    
    class UpdateGatewayResponse(Mapping):
        pass
    
    class UpdateGatewayGroupRequest(Mapping):
        pass
    
    class UpdateGatewayGroupResponse(Mapping):
        pass
    
    class UpdateNetworkProfileRequest(Mapping):
        pass
    
    class UpdateNetworkProfileResponse(Mapping):
        pass
    
    class UpdateProfileRequest(Mapping):
        pass
    
    class UpdateProfileResponse(Mapping):
        pass
    
    class UpdateRoomRequest(Mapping):
        pass
    
    class UpdateRoomResponse(Mapping):
        pass
    
    class UpdateSkillGroupRequest(Mapping):
        pass
    
    class UpdateSkillGroupResponse(Mapping):
        pass
    
    def approve_skill(self, SkillId: str) -> ApproveSkillResponse:
        """Associates a skill with the organization under the customer's AWS account. If a skill is private, the user implicitly accepts access to this skill during enablement.

        :param self:
        :param SkillId:  The unique identifier of the skill.<br/>
        :return:         """
        pass

    def associate_contact_with_address_book(self, AddressBookArn: str, ContactArn: str) -> AssociateContactWithAddressBookResponse:
        """Associates a contact with a given address book.

        :param self:
        :param AddressBookArn:  The ARN of the address book with which to associate the contact.<br/>ContactArn:  The ARN of the contact to associate with an address book.<br/>
        :return:         """
        pass

    def associate_device_with_network_profile(self, NetworkProfileArn: str, DeviceArn: str) -> AssociateDeviceWithNetworkProfileResponse:
        """Associates a device with the specified network profile.

        :param self:
        :param NetworkProfileArn:  The ARN of the network profile to associate with a device.<br/>DeviceArn:  The device ARN.<br/>
        :return:         """
        pass

    def associate_device_with_room(self, DeviceArn: str=None, RoomArn: str=None) -> AssociateDeviceWithRoomResponse:
        """Associates a device with a given room. This applies all the settings from the room profile to the device, and all the skills in any skill groups added to that room. This operation requires the device to be online, or else a manual sync is required. 

        :param self:
        :param RoomArn:  The ARN of the room with which to associate the device. Required.<br/>DeviceArn:  The ARN of the device to associate to a room. Required.<br/>
        :return:         """
        pass

    def associate_skill_group_with_room(self, SkillGroupArn: str=None, RoomArn: str=None) -> AssociateSkillGroupWithRoomResponse:
        """Associates a skill group with a given room. This enables all skills in the associated skill group on all devices in the room.

        :param self:
        :param RoomArn:  The ARN of the room with which to associate the skill group. Required.<br/>SkillGroupArn:  The ARN of the skill group to associate with a room. Required.<br/>
        :return:         """
        pass

    def associate_skill_with_skill_group(self, SkillId: str, SkillGroupArn: str=None) -> AssociateSkillWithSkillGroupResponse:
        """Associates a skill with a skill group.

        :param self:
        :param SkillId:  The unique identifier of the skill.<br/>SkillGroupArn:  The ARN of the skill group to associate the skill to. Required.<br/>
        :return:         """
        pass

    def associate_skill_with_users(self, SkillId: str) -> AssociateSkillWithUsersResponse:
        """Makes a private skill available for enrolled users to enable on their devices.

        :param self:
        :param SkillId:  The private skill ID you want to make available to enrolled users.<br/>
        :return:         """
        pass

    def create_address_book(self, Name: str, Description: str=None, ClientRequestToken: str=None) -> CreateAddressBookResponse:
        """Creates an address book with the specified details.

        :param self:
        :param ClientRequestToken:  A unique, user-specified identifier for the request that ensures idempotency.<br/>Description:  The description of the address book.<br/>Name:  The name of the address book.<br/>
        :return: AddressBookArn:  The ARN of the newly created address book.<br/>        """
        pass

    def create_business_report_schedule(self, ContentRange: BusinessReportContentRange, Format: str, ScheduleName: str=None, S3BucketName: str=None, S3KeyPrefix: str=None, Recurrence: BusinessReportRecurrence=None, ClientRequestToken: str=None) -> CreateBusinessReportScheduleResponse:
        """Creates a recurring schedule for usage reports to deliver to the specified S3 location with a specified daily or weekly interval.

        :param self:
        :param ClientRequestToken:  The client request token.<br/>Recurrence:  The recurrence of the reports. If this isn't specified, the report will only be delivered one time when the API is called. :<br/>ContentRange:  The content range of the reports.:<br/>Format:  The format of the generated report (individual CSV files or zipped files of individual files).<br/>S3KeyPrefix:  The S3 key where the report is delivered.<br/>S3BucketName:  The S3 bucket name of the output reports. If this isn't specified, the report can be retrieved from a download link by calling ListBusinessReportSchedule. <br/>ScheduleName:  The name identifier of the schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Interval</i> The interval of the content range.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartDate</i> The start date.<br/>
        :return: ScheduleArn:  The ARN of the business report schedule.<br/>        """
        pass

    def create_conference_provider(self, MeetingSetting: MeetingSetting, ConferenceProviderType: str, ConferenceProviderName: str, IPDialIn: IPDialIn=None, PSTNDialIn: PSTNDialIn=None, ClientRequestToken: str=None) -> CreateConferenceProviderResponse:
        """Adds a new conference provider under the user's AWS account.

        :param self:
        :param ClientRequestToken:  The request token of the client.<br/>MeetingSetting:  The meeting settings for the conference provider.:<br/>PSTNDialIn:  The information for PSTN conferencing.:<br/>IPDialIn:  The IP endpoint and protocol for calling.:<br/>ConferenceProviderType:  Represents a type within a list of predefined types.<br/>ConferenceProviderName:  The name of the conference provider.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CommsProtocol</i> The protocol, including SIP, SIPS, and H323.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> The IP address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OneClickPinDelay</i> The delay duration before Alexa enters the conference pin with dual-tone multi-frequency (DTMF). Each number on the dial pad corresponds to a DTMF tone, which is how we send data over the telephone network.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OneClickIdDelay</i> The delay duration before Alexa enters the conference ID with dual-tone multi-frequency (DTMF). Each number on the dial pad corresponds to a DTMF tone, which is how we send data over the telephone network.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneNumber</i> The phone number to call to join the conference.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CountryCode</i> The zip code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequirePin</i> The values that indicate whether the pin is always required.<br/>
        :return: ConferenceProviderArn:  The ARN of the newly-created conference provider.<br/>        """
        pass

    def create_contact(self, FirstName: str, DisplayName: str=None, LastName: str=None, PhoneNumber: str=None, PhoneNumbers: List[PhoneNumber]=None, SipAddresses: List[SipAddress]=None, ClientRequestToken: str=None) -> CreateContactResponse:
        """Creates a contact with the specified details.

        :param self:
        :param ClientRequestToken:  A unique, user-specified identifier for this request that ensures idempotency.<br/>SipAddresses:  The list of SIP addresses for the contact.<br/>PhoneNumbers:  The list of phone numbers for the contact.<br/>PhoneNumber:  The phone number of the contact in E.164 format. The phone number type defaults to WORK. You can specify PhoneNumber or PhoneNumbers. We recommend that you use PhoneNumbers, which lets you specify the phone number type and multiple numbers.<br/>LastName:  The last name of the contact that is used to call the contact on the device.<br/>FirstName:  The first name of the contact that is used to call the contact on the device.<br/>DisplayName:  The name of the contact to display on the console.<br/>
        :return: ContactArn:  The ARN of the newly created address book.<br/>        """
        pass

    def create_gateway_group(self, ClientRequestToken: str, Name: str, Description: str=None) -> CreateGatewayGroupResponse:
        """Creates a gateway group with the specified details.

        :param self:
        :param ClientRequestToken:   A unique, user-specified identifier for the request that ensures idempotency.<br/>Description:  The description of the gateway group.<br/>Name:  The name of the gateway group.<br/>
        :return: GatewayGroupArn:  The ARN of the created gateway group.<br/>        """
        pass

    def create_network_profile(self, ClientRequestToken: str, SecurityType: str, Ssid: str, NetworkProfileName: str, Description: str=None, EapMethod: str=None, CurrentPassword: str=None, NextPassword: str=None, CertificateAuthorityArn: str=None, TrustAnchors: List[TrustAnchor]=None) -> CreateNetworkProfileResponse:
        """Creates a network profile with the specified details.

        :param self:
        :param ClientRequestToken:  A unique, user-specified identifier for the request that ensures idempotency.<br/>TrustAnchors:  The root certificates of your authentication server that is installed on your devices and used to trust your authentication server during EAP negotiation. <br/>CertificateAuthorityArn:  The ARN of the Private Certificate Authority (PCA) created in AWS Certificate Manager (ACM). This is used to issue certificates to the devices. <br/>NextPassword:  The next, or subsequent, password of the Wi-Fi network. This password is asynchronously transmitted to the device and is used when the password of the network changes to NextPassword. <br/>CurrentPassword:  The current password of the Wi-Fi network.<br/>EapMethod:  The authentication standard that is used in the EAP framework. Currently, EAP_TLS is supported.<br/>SecurityType:  The security type of the Wi-Fi network. This can be WPA2_ENTERPRISE, WPA2_PSK, WPA_PSK, WEP, or OPEN.<br/>Ssid:  The SSID of the Wi-Fi network.<br/>Description:  Detailed information about a device's network profile.<br/>NetworkProfileName:  The name of the network profile associated with a device.<br/>
        :return: NetworkProfileArn:  The ARN of the network profile associated with a device.<br/>        """
        pass

    def create_profile(self, WakeWord: str, TemperatureUnit: str, DistanceUnit: str, Address: str, Timezone: str, ProfileName: str, ClientRequestToken: str=None, SetupModeDisabled: Boolean=None, MaxVolumeLimit: int=None, PSTNEnabled: Boolean=None) -> CreateProfileResponse:
        """Creates a new room profile with the specified details.

        :param self:
        :param PSTNEnabled:  Whether PSTN calling is enabled.<br/>MaxVolumeLimit:  The maximum volume limit for a room profile.<br/>SetupModeDisabled:  Whether room profile setup is enabled.<br/>ClientRequestToken:  The user-specified token that is used during the creation of a profile.<br/>WakeWord:  A wake word for Alexa, Echo, Amazon, or a computer.<br/>TemperatureUnit:  The temperature unit to be used by devices in the profile.<br/>DistanceUnit:  The distance unit to be used by devices in the profile.<br/>Address:  The valid address for the room.<br/>Timezone:  The time zone used by a room profile.<br/>ProfileName:  The name of a room profile.<br/>
        :return: ProfileArn:  The ARN of the newly created room profile in the response.<br/>        """
        pass

    def create_room(self, RoomName: str, Description: str=None, ProfileArn: str=None, ProviderCalendarId: str=None, ClientRequestToken: str=None, Tags: List[Tag]=None) -> CreateRoomResponse:
        """Creates a room with the specified details.

        :param self:
        :param Tags:  The tags for the room.<br/>ClientRequestToken:  A unique, user-specified identifier for this request that ensures idempotency. <br/>ProviderCalendarId:  The calendar ARN for the room.<br/>ProfileArn:  The profile ARN for the room.<br/>Description:  The description for the room.<br/>RoomName:  The name for the room.<br/>
        :return: RoomArn:  The ARN of the newly created room in the response.<br/>        """
        pass

    def create_skill_group(self, SkillGroupName: str, Description: str=None, ClientRequestToken: str=None) -> CreateSkillGroupResponse:
        """Creates a skill group with a specified name and description.

        :param self:
        :param ClientRequestToken:  A unique, user-specified identifier for this request that ensures idempotency. <br/>Description:  The description for the skill group.<br/>SkillGroupName:  The name for the skill group.<br/>
        :return: SkillGroupArn:  The ARN of the newly created skill group in the response.<br/>        """
        pass

    def create_user(self, UserId: str, FirstName: str=None, LastName: str=None, Email: str=None, ClientRequestToken: str=None, Tags: List[Tag]=None) -> CreateUserResponse:
        """Creates a user.

        :param self:
        :param Tags:  The tags for the user.<br/>ClientRequestToken:  A unique, user-specified identifier for this request that ensures idempotency. <br/>Email:  The email address for the user.<br/>LastName:  The last name for the user.<br/>FirstName:  The first name for the user.<br/>UserId:  The ARN for the user.<br/>
        :return: UserArn:  The ARN of the newly created user in the response.<br/>        """
        pass

    def delete_address_book(self, AddressBookArn: str) -> DeleteAddressBookResponse:
        """Deletes an address book by the address book ARN.

        :param self:
        :param AddressBookArn:  The ARN of the address book to delete.<br/>
        :return:         """
        pass

    def delete_business_report_schedule(self, ScheduleArn: str) -> DeleteBusinessReportScheduleResponse:
        """Deletes the recurring report delivery schedule with the specified schedule ARN.

        :param self:
        :param ScheduleArn:  The ARN of the business report schedule.<br/>
        :return:         """
        pass

    def delete_conference_provider(self, ConferenceProviderArn: str) -> DeleteConferenceProviderResponse:
        """Deletes a conference provider.

        :param self:
        :param ConferenceProviderArn:  The ARN of the conference provider.<br/>
        :return:         """
        pass

    def delete_contact(self, ContactArn: str) -> DeleteContactResponse:
        """Deletes a contact by the contact ARN.

        :param self:
        :param ContactArn:  The ARN of the contact to delete.<br/>
        :return:         """
        pass

    def delete_device(self, DeviceArn: str) -> DeleteDeviceResponse:
        """Removes a device from Alexa For Business.

        :param self:
        :param DeviceArn:  The ARN of the device for which to request details.<br/>
        :return:         """
        pass

    def delete_device_usage_data(self, DeviceUsageType: str, DeviceArn: str) -> DeleteDeviceUsageDataResponse:
        """When this action is called for a specified shared device, it allows authorized users to delete the device's entire previous history of voice input data and associated response data. This action can be called once every 24 hours for a specific shared device. When this action is called for a specified shared device, it allows authorized users to delete the device's entire previous history of voice input data. This action can be called once every 24 hours for a specific shared device. 

        :param self:
        :param DeviceUsageType:  The type of usage data to delete.<br/>DeviceArn:  The ARN of the device.<br/>
        :return:         """
        pass

    def delete_gateway_group(self, GatewayGroupArn: str) -> DeleteGatewayGroupResponse:
        """Deletes a gateway group.

        :param self:
        :param GatewayGroupArn:  The ARN of the gateway group to delete.<br/>
        :return:         """
        pass

    def delete_network_profile(self, NetworkProfileArn: str) -> DeleteNetworkProfileResponse:
        """Deletes a network profile by the network profile ARN.

        :param self:
        :param NetworkProfileArn:  The ARN of the network profile associated with a device.<br/>
        :return:         """
        pass

    def delete_profile(self, ProfileArn: str=None) -> DeleteProfileResponse:
        """Deletes a room profile by the profile ARN.

        :param self:
        :param ProfileArn:  The ARN of the room profile to delete. Required.<br/>
        :return:         """
        pass

    def delete_room(self, RoomArn: str=None) -> DeleteRoomResponse:
        """Deletes a room by the room ARN.

        :param self:
        :param RoomArn:  The ARN of the room to delete. Required.<br/>
        :return:         """
        pass

    def delete_room_skill_parameter(self, ParameterKey: str, SkillId: str, RoomArn: str=None) -> DeleteRoomSkillParameterResponse:
        """Deletes room skill parameter details by room, skill, and parameter key ID.

        :param self:
        :param ParameterKey:  The room skill parameter key for which to remove details.<br/>SkillId:  The ID of the skill from which to remove the room skill parameter details.<br/>RoomArn:  The ARN of the room from which to remove the room skill parameter details.<br/>
        :return:         """
        pass

    def delete_skill_authorization(self, SkillId: str, RoomArn: str=None) -> DeleteSkillAuthorizationResponse:
        """Unlinks a third-party account from a skill.

        :param self:
        :param RoomArn:  The room that the skill is authorized for.<br/>SkillId:  The unique identifier of a skill.<br/>
        :return:         """
        pass

    def delete_skill_group(self, SkillGroupArn: str=None) -> DeleteSkillGroupResponse:
        """Deletes a skill group by skill group ARN.

        :param self:
        :param SkillGroupArn:  The ARN of the skill group to delete. Required.<br/>
        :return:         """
        pass

    def delete_user(self, EnrollmentId: str, UserArn: str=None) -> DeleteUserResponse:
        """Deletes a specified user by user ARN and enrollment ARN.

        :param self:
        :param EnrollmentId:  The ARN of the user's enrollment in the organization. Required.<br/>UserArn:  The ARN of the user to delete in the organization. Required.<br/>
        :return:         """
        pass

    def disassociate_contact_from_address_book(self, AddressBookArn: str, ContactArn: str) -> DisassociateContactFromAddressBookResponse:
        """Disassociates a contact from a given address book.

        :param self:
        :param AddressBookArn:  The ARN of the address from which to disassociate the contact.<br/>ContactArn:  The ARN of the contact to disassociate from an address book.<br/>
        :return:         """
        pass

    def disassociate_device_from_room(self, DeviceArn: str=None) -> DisassociateDeviceFromRoomResponse:
        """Disassociates a device from its current room. The device continues to be connected to the Wi-Fi network and is still registered to the account. The device settings and skills are removed from the room.

        :param self:
        :param DeviceArn:  The ARN of the device to disassociate from a room. Required.<br/>
        :return:         """
        pass

    def disassociate_skill_from_skill_group(self, SkillId: str, SkillGroupArn: str=None) -> DisassociateSkillFromSkillGroupResponse:
        """Disassociates a skill from a skill group.

        :param self:
        :param SkillId:  The ARN of a skill group to associate to a skill.<br/>SkillGroupArn:  The unique identifier of a skill. Required.<br/>
        :return:         """
        pass

    def disassociate_skill_from_users(self, SkillId: str) -> DisassociateSkillFromUsersResponse:
        """Makes a private skill unavailable for enrolled users and prevents them from enabling it on their devices.

        :param self:
        :param SkillId:   The private skill ID you want to make unavailable for enrolled users.<br/>
        :return:         """
        pass

    def disassociate_skill_group_from_room(self, SkillGroupArn: str=None, RoomArn: str=None) -> DisassociateSkillGroupFromRoomResponse:
        """Disassociates a skill group from a specified room. This disables all skills in the skill group on all devices in the room.

        :param self:
        :param RoomArn:  The ARN of the room from which the skill group is to be disassociated. Required.<br/>SkillGroupArn:  The ARN of the skill group to disassociate from a room. Required.<br/>
        :return:         """
        pass

    def forget_smart_home_appliances(self, RoomArn: str) -> ForgetSmartHomeAppliancesResponse:
        """Forgets smart home appliances associated to a room.

        :param self:
        :param RoomArn:  The room that the appliances are associated with.<br/>
        :return:         """
        pass

    def get_address_book(self, AddressBookArn: str) -> GetAddressBookResponse:
        """Gets address the book details by the address book ARN.

        :param self:
        :param AddressBookArn:  The ARN of the address book for which to request details.<br/>
        :return: AddressBook:  The details of the requested address book.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the address book.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the address book.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressBookArn</i> The ARN of the address book.<br/>        """
        pass

    def get_conference_preference(self) -> GetConferencePreferenceResponse:
        """Retrieves the existing conference preferences.

        :param self:
        :param 
        :return: Preference:  The conference preference.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultConferenceProviderArn</i> The ARN of the default conference provider.<br/>        """
        pass

    def get_conference_provider(self, ConferenceProviderArn: str) -> GetConferenceProviderResponse:
        """Gets details about a specific conference provider.

        :param self:
        :param ConferenceProviderArn:  The ARN of the newly created conference provider.<br/>
        :return: ConferenceProvider:  The conference provider.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MeetingSetting</i> The meeting settings for the conference provider.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>MeetingSetting</b>: The meeting settings for the conference provider.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PSTNDialIn</i> The information for PSTN conferencing.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PSTNDialIn</b>: The information for PSTN conferencing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IPDialIn</i> The IP endpoint and protocol for calling.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>IPDialIn</b>: The IP endpoint and protocol for calling.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of conference providers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the conference provider.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN of the newly created conference provider.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CommsProtocol</i> The protocol, including SIP, SIPS, and H323.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> The IP address.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OneClickPinDelay</i> The delay duration before Alexa enters the conference pin with dual-tone multi-frequency (DTMF). Each number on the dial pad corresponds to a DTMF tone, which is how we send data over the telephone network.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OneClickIdDelay</i> The delay duration before Alexa enters the conference ID with dual-tone multi-frequency (DTMF). Each number on the dial pad corresponds to a DTMF tone, which is how we send data over the telephone network.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneNumber</i> The phone number to call to join the conference.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CountryCode</i> The zip code.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RequirePin</i> The values that indicate whether the pin is always required.<br/>        """
        pass

    def get_contact(self, ContactArn: str) -> GetContactResponse:
        """Gets the contact details by the contact ARN.

        :param self:
        :param ContactArn:  The ARN of the contact for which to request details.<br/>
        :return: Contact:  The details of the requested contact.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SipAddresses</i> The list of SIP addresses for the contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneNumbers</i> The list of phone numbers for the contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneNumber</i> The phone number of the contact. The phone number type defaults to WORK. You can either specify PhoneNumber or PhoneNumbers. We recommend that you use PhoneNumbers, which lets you specify the phone number type and multiple numbers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastName</i> The last name of the contact, used to call the contact on the device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FirstName</i> The first name of the contact, used to call the contact on the device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayName</i> The name of the contact to display on the console.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContactArn</i> The ARN of the contact.<br/>        """
        pass

    def get_device(self, DeviceArn: str=None) -> GetDeviceResponse:
        """Gets the details of a device by device ARN.

        :param self:
        :param DeviceArn:  The ARN of the device for which to request details. Required.<br/>
        :return: Device:  The details of the device requested. Required.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkProfileInfo</i> Detailed information about a device's network profile.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NetworkProfileInfo</b>: Detailed information about a device's network profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceStatusInfo</i> Detailed information about a device's status.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DeviceStatusInfo</b>: Detailed information about a device's status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceStatus</i> The status of a device. If the status is not READY, check the DeviceStatusInfo value for details.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoomArn</i> The room ARN of a device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MacAddress</i> The MAC address of a device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SoftwareVersion</i> The software version of a device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceName</i> The name of a device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceType</i> The type of a device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceSerialNumber</i> The serial number of a device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceArn</i> The ARN of a device.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectionStatus</i> The latest available information about the connection status of a device. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceStatusDetails</i> One or more device status detail descriptions.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateExpirationTime</i> The time (in epoch) when the certificate expires.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateArn</i> The ARN of the certificate associated with a device.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkProfileArn</i> The ARN of the network profile associated with a device.<br/>        """
        pass

    def get_gateway(self, GatewayArn: str) -> GetGatewayResponse:
        """Retrieves the details of a gateway.

        :param self:
        :param GatewayArn:  The ARN of the gateway to get.<br/>
        :return: Gateway:  The details of the gateway.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SoftwareVersion</i> The software version of the gateway. The gateway automatically updates its software version during normal operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GatewayGroupArn</i> The ARN of the gateway group that the gateway is associated to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN of the gateway.<br/>        """
        pass

    def get_gateway_group(self, GatewayGroupArn: str) -> GetGatewayGroupResponse:
        """Retrieves the details of a gateway group.

        :param self:
        :param GatewayGroupArn:  The ARN of the gateway group to get.<br/>
        :return: GatewayGroup:  The details of the gateway group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the gateway group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the gateway group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN of the gateway group.<br/>        """
        pass

    def get_invitation_configuration(self) -> GetInvitationConfigurationResponse:
        """Retrieves the configured values for the user enrollment invitation email template.

        :param self:
        :param 
        :return: PrivateSkillIds:  The list of private skill IDs that you want to recommend to the user to enable in the invitation.<br/>ContactEmail:  The email ID of the organization or individual contact that the enrolled user can use. <br/>OrganizationName:  The name of the organization sending the enrollment invite to a user.<br/>        """
        pass

    def get_network_profile(self, NetworkProfileArn: str) -> GetNetworkProfileResponse:
        """Gets the network profile details by the network profile ARN.

        :param self:
        :param NetworkProfileArn:  The ARN of the network profile associated with a device.<br/>
        :return: NetworkProfile:  The network profile associated with a device.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrustAnchors</i> The root certificates of your authentication server, which is installed on your devices and used to trust your authentication server during EAP negotiation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateAuthorityArn</i> The ARN of the Private Certificate Authority (PCA) created in AWS Certificate Manager (ACM). This is used to issue certificates to the devices. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NextPassword</i> The next, or subsequent, password of the Wi-Fi network. This password is asynchronously transmitted to the device and is used when the password of the network changes to NextPassword. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentPassword</i> The current password of the Wi-Fi network.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EapMethod</i> The authentication standard that is used in the EAP framework. Currently, EAP_TLS is supported. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityType</i> The security type of the Wi-Fi network. This can be WPA2_ENTERPRISE, WPA2_PSK, WPA_PSK, WEP, or OPEN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Ssid</i> The SSID of the Wi-Fi network.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> Detailed information about a device's network profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkProfileName</i> The name of the network profile associated with a device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkProfileArn</i> The ARN of the network profile associated with a device.<br/>        """
        pass

    def get_profile(self, ProfileArn: str=None) -> GetProfileResponse:
        """Gets the details of a room profile by profile ARN.

        :param self:
        :param ProfileArn:  The ARN of the room profile for which to request details. Required.<br/>
        :return: Profile:  The details of the room profile requested. Required.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressBookArn</i> The ARN of the address book.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PSTNEnabled</i> The PSTN setting of a room profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxVolumeLimit</i> The max volume limit of a room profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SetupModeDisabled</i> The setup mode of a room profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WakeWord</i> The wake word of a room profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TemperatureUnit</i> The temperature unit of a room profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DistanceUnit</i> The distance unit of a room profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timezone</i> The time zone of a room profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The address of a room profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsDefault</i> Retrieves if the profile is default or not.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProfileName</i> The name of a room profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProfileArn</i> The ARN of a room profile.<br/>        """
        pass

    def get_room(self, RoomArn: str=None) -> GetRoomResponse:
        """Gets room details by room ARN.

        :param self:
        :param RoomArn:  The ARN of the room for which to request details. Required.<br/>
        :return: Room:  The details of the room requested.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProfileArn</i> The profile ARN of a room.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProviderCalendarId</i> The provider calendar ARN of a room.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of a room.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoomName</i> The name of a room.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoomArn</i> The ARN of a room.<br/>        """
        pass

    def get_room_skill_parameter(self, ParameterKey: str, SkillId: str, RoomArn: str=None) -> GetRoomSkillParameterResponse:
        """Gets room skill parameter details by room, skill, and parameter key ARN.

        :param self:
        :param ParameterKey:  The room skill parameter key for which to get details. Required.<br/>SkillId:  The ARN of the skill from which to get the room skill parameter details. Required.<br/>RoomArn:  The ARN of the room from which to get the room skill parameter details. <br/>
        :return: RoomSkillParameter:  The details of the room skill parameter requested. Required.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterValue</i> The parameter value of a room skill parameter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterKey</i> The parameter key of a room skill parameter. ParameterKey is an enumerated type that only takes “DEFAULT” or “SCOPE” as valid values.<br/>        """
        pass

    def get_skill_group(self, SkillGroupArn: str=None) -> GetSkillGroupResponse:
        """Gets skill group details by skill group ARN.

        :param self:
        :param SkillGroupArn:  The ARN of the skill group for which to get details. Required.<br/>
        :return: SkillGroup:  The details of the skill group requested. Required.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of a skill group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SkillGroupName</i> The name of a skill group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SkillGroupArn</i> The ARN of a skill group.<br/>        """
        pass

    def list_business_report_schedules(self, NextToken: str=None, MaxResults: int=None) -> ListBusinessReportSchedulesResponse:
        """Lists the details of the schedules that a user configured.

        :param self:
        :param MaxResults:  The maximum number of schedules listed in the call.<br/>NextToken:  The token used to list the remaining schedules from the previous API call.<br/>
        :return: NextToken:  The token used to list the remaining schedules from the previous API call.<br/>BusinessReportSchedules:  The schedule of the reports.<br/>        """
        pass

    def list_conference_providers(self, NextToken: str=None, MaxResults: int=None) -> ListConferenceProvidersResponse:
        """Lists conference providers under a specific AWS account.

        :param self:
        :param MaxResults:  The maximum number of conference providers to be returned, per paginated calls.<br/>NextToken:  The tokens used for pagination.<br/>
        :return: NextToken:  The tokens used for pagination.<br/>ConferenceProviders:  The conference providers.<br/>        """
        pass

    def list_device_events(self, DeviceArn: str, EventType: str=None, NextToken: str=None, MaxResults: int=None) -> ListDeviceEventsResponse:
        """Lists the device event history, including device connection status, for up to 30 days.

        :param self:
        :param MaxResults:  The maximum number of results to include in the response. The default value is 50. If more results exist than the specified MaxResults value, a token is included in the response so that the remaining results can be retrieved. <br/>NextToken:  An optional token returned from a prior request. Use this token for pagination of results from this action. If this parameter is specified, the response only includes results beyond the token, up to the value specified by MaxResults. When the end of results is reached, the response has a value of null.<br/>EventType:  The event type to filter device events. If EventType isn't specified, this returns a list of all device events in reverse chronological order. If EventType is specified, this returns a list of device events for that EventType in reverse chronological order. <br/>DeviceArn:  The ARN of a device.<br/>
        :return: NextToken:  The token returned to indicate that there is more data available.<br/>DeviceEvents:  The device events requested for the device ARN.<br/>        """
        pass

    def list_gateway_groups(self, NextToken: str=None, MaxResults: int=None) -> ListGatewayGroupsResponse:
        """Retrieves a list of gateway group summaries. Use GetGatewayGroup to retrieve details of a specific gateway group.

        :param self:
        :param MaxResults:  The maximum number of gateway group summaries to return. The default is 50.<br/>NextToken:  The token used to paginate though multiple pages of gateway group summaries.<br/>
        :return: NextToken:  The token used to paginate though multiple pages of gateway group summaries.<br/>GatewayGroups:  The gateway groups in the list.<br/>        """
        pass

    def list_gateways(self, GatewayGroupArn: str=None, NextToken: str=None, MaxResults: int=None) -> ListGatewaysResponse:
        """Retrieves a list of gateway summaries. Use GetGateway to retrieve details of a specific gateway. An optional gateway group ARN can be provided to only retrieve gateway summaries of gateways that are associated with that gateway group ARN.

        :param self:
        :param MaxResults:  The maximum number of gateway summaries to return. The default is 50.<br/>NextToken:  The token used to paginate though multiple pages of gateway summaries.<br/>GatewayGroupArn:  The gateway group ARN for which to list gateways.<br/>
        :return: NextToken:  The token used to paginate though multiple pages of gateway summaries.<br/>Gateways:  The gateways in the list.<br/>        """
        pass

    def list_skills(self, SkillGroupArn: str=None, EnablementType: str=None, SkillType: str=None, NextToken: str=None, MaxResults: int=None) -> ListSkillsResponse:
        """Lists all enabled skills in a specific skill group.

        :param self:
        :param MaxResults:  The maximum number of results to include in the response. If more results exist than the specified <code>MaxResults</code> value, a token is included in the response so that the remaining results can be retrieved. Required.<br/>NextToken:  An optional token returned from a prior request. Use this token for pagination of results from this action. If this parameter is specified, the response includes only results beyond the token, up to the value specified by <code>MaxResults</code>. Required.<br/>SkillType:  Whether the skill is publicly available or is a private skill.<br/>EnablementType:  Whether the skill is enabled under the user's account, or if it requires linking to be used.<br/>SkillGroupArn:  The ARN of the skill group for which to list enabled skills. Required.<br/>
        :return: NextToken:  The token returned to indicate that there is more data available.<br/>SkillSummaries:  The list of enabled skills requested. Required.<br/>        """
        pass

    def list_skills_store_categories(self, NextToken: str=None, MaxResults: int=None) -> ListSkillsStoreCategoriesResponse:
        """Lists all categories in the Alexa skill store.

        :param self:
        :param MaxResults:  The maximum number of categories returned, per paginated calls.<br/>NextToken:  The tokens used for pagination.<br/>
        :return: NextToken:  The tokens used for pagination.<br/>CategoryList:  The list of categories.<br/>        """
        pass

    def list_skills_store_skills_by_category(self, CategoryId: CategoryId, NextToken: str=None, MaxResults: int=None) -> ListSkillsStoreSkillsByCategoryResponse:
        """Lists all skills in the Alexa skill store by category.

        :param self:
        :param MaxResults:  The maximum number of skills returned per paginated calls.<br/>NextToken:  The tokens used for pagination.<br/>CategoryId:  The category ID for which the skills are being retrieved from the skill store.<br/>
        :return: NextToken:  The tokens used for pagination.<br/>SkillsStoreSkills:  The skill store skills.<br/>        """
        pass

    def list_smart_home_appliances(self, RoomArn: str, MaxResults: int=None, NextToken: str=None) -> ListSmartHomeAppliancesResponse:
        """Lists all of the smart home appliances associated with a room.

        :param self:
        :param NextToken:  The tokens used for pagination.<br/>MaxResults:  The maximum number of appliances to be returned, per paginated calls.<br/>RoomArn:  The room that the appliances are associated with.<br/>
        :return: NextToken:  The tokens used for pagination.<br/>SmartHomeAppliances:  The smart home appliances.<br/>        """
        pass

    def list_tags(self, Arn: str, NextToken: str=None, MaxResults: int=None) -> ListTagsResponse:
        """Lists all tags for the specified resource.

        :param self:
        :param MaxResults:  The maximum number of results to include in the response. If more results exist than the specified <code>MaxResults</code> value, a token is included in the response so that the remaining results can be retrieved.<br/>NextToken:  An optional token returned from a prior request. Use this token for pagination of results from this action. If this parameter is specified, the response includes only results beyond the token, up to the value specified by <code>MaxResults</code>. <br/>Arn:  The ARN of the specified resource for which to list tags.<br/>
        :return: NextToken:  The token returned to indicate that there is more data available.<br/>Tags:  The tags requested for the specified resource.<br/>        """
        pass

    def put_conference_preference(self, ConferencePreference: ConferencePreference) -> PutConferencePreferenceResponse:
        """Sets the conference preferences on a specific conference provider at the account level.

        :param self:
        :param ConferencePreference:  The conference preference of a specific conference provider.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultConferenceProviderArn</i> The ARN of the default conference provider.<br/>
        :return:         """
        pass

    def put_invitation_configuration(self, OrganizationName: str, ContactEmail: str=None, PrivateSkillIds: List[SkillId]=None) -> PutInvitationConfigurationResponse:
        """Configures the email template for the user enrollment invitation with the specified attributes.

        :param self:
        :param PrivateSkillIds:  The list of private skill IDs that you want to recommend to the user to enable in the invitation.<br/>ContactEmail:  The email ID of the organization or individual contact that the enrolled user can use. <br/>OrganizationName:  The name of the organization sending the enrollment invite to a user.<br/>
        :return:         """
        pass

    def put_room_skill_parameter(self, RoomSkillParameter: RoomSkillParameter, SkillId: str, RoomArn: str=None) -> PutRoomSkillParameterResponse:
        """Updates room skill parameter details by room, skill, and parameter key ID. Not all skills have a room skill parameter.

        :param self:
        :param RoomSkillParameter:  The updated room skill parameter. Required.:<br/>SkillId:  The ARN of the skill associated with the room skill parameter. Required.<br/>RoomArn:  The ARN of the room associated with the room skill parameter. Required.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterValue</i> The parameter value of a room skill parameter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterKey</i> The parameter key of a room skill parameter. ParameterKey is an enumerated type that only takes “DEFAULT” or “SCOPE” as valid values.<br/>
        :return:         """
        pass

    def put_skill_authorization(self, SkillId: str, AuthorizationResult: AuthorizationResult, RoomArn: str=None) -> PutSkillAuthorizationResponse:
        """Links a user's account to a third-party skill provider. If this API operation is called by an assumed IAM role, the skill being linked must be a private skill. Also, the skill must be owned by the AWS account that assumed the IAM role.

        :param self:
        :param RoomArn:  The room that the skill is authorized for.<br/>SkillId:  The unique identifier of a skill.<br/>AuthorizationResult:  The authorization result specific to OAUTH code grant output. "Code” must be populated in the AuthorizationResult map to establish the authorization.<br/>
        :return:         """
        pass

    def register_avs_device(self, AmazonId: str, DeviceSerialNumber: str, ProductId: str, UserCode: str, ClientId: str) -> RegisterAVSDeviceResponse:
        """Registers an Alexa-enabled device built by an Original Equipment Manufacturer (OEM) using Alexa Voice Service (AVS).

        :param self:
        :param AmazonId:  The device type ID for your AVS device generated by Amazon when the OEM creates a new product on Amazon's Developer Console.<br/>DeviceSerialNumber:  The key generated by the OEM that uniquely identifies a specified instance of your AVS device.<br/>ProductId:  The product ID used to identify your AVS device during authorization.<br/>UserCode:  The code that is obtained after your AVS device has made a POST request to LWA as a part of the Device Authorization Request component of the OAuth code-based linking specification.<br/>ClientId:  The client ID of the OEM used for code-based linking authorization on an AVS device.<br/>
        :return: DeviceArn:  The ARN of the device.<br/>        """
        pass

    def reject_skill(self, SkillId: str) -> RejectSkillResponse:
        """Disassociates a skill from the organization under a user's AWS account. If the skill is a private skill, it moves to an AcceptStatus of PENDING. Any private or public skill that is rejected can be added later by calling the ApproveSkill API. 

        :param self:
        :param SkillId:  The unique identifier of the skill.<br/>
        :return:         """
        pass

    def resolve_room(self, SkillId: str, UserId: str) -> ResolveRoomResponse:
        """Determines the details for the room from which a skill request was invoked. This operation is used by skill developers.

        :param self:
        :param SkillId:  The ARN of the skill that was requested. Required.<br/>UserId:  The ARN of the user. Required.<br/>
        :return: RoomSkillParameters:  Response to get the room profile request. Required.<br/>RoomName:  The name of the room from which the skill request was invoked.<br/>RoomArn:  The ARN of the room from which the skill request was invoked.<br/>        """
        pass

    def revoke_invitation(self, UserArn: str=None, EnrollmentId: str=None) -> RevokeInvitationResponse:
        """Revokes an invitation and invalidates the enrollment URL.

        :param self:
        :param EnrollmentId:  The ARN of the enrollment invitation to revoke. Required.<br/>UserArn:  The ARN of the user for whom to revoke an enrollment invitation. Required.<br/>
        :return:         """
        pass

    def search_address_books(self, Filters: List[Filter]=None, SortCriteria: List[Sort]=None, NextToken: str=None, MaxResults: int=None) -> SearchAddressBooksResponse:
        """Searches address books and lists the ones that meet a set of filter and sort criteria.

        :param self:
        :param MaxResults:  The maximum number of results to include in the response. If more results exist than the specified MaxResults value, a token is included in the response so that the remaining results can be retrieved.<br/>NextToken:  An optional token returned from a prior request. Use this token for pagination of results from this action. If this parameter is specified, the response only includes results beyond the token, up to the value specified by MaxResults.<br/>SortCriteria:  The sort order to use in listing the specified set of address books. The supported sort key is AddressBookName.<br/>Filters:  The filters to use to list a specified set of address books. The supported filter key is AddressBookName.<br/>
        :return: TotalCount:  The total number of address books returned.<br/>NextToken:  The token returned to indicate that there is more data available.<br/>AddressBooks:  The address books that meet the specified set of filter criteria, in sort order.<br/>        """
        pass

    def search_contacts(self, Filters: List[Filter]=None, SortCriteria: List[Sort]=None, NextToken: str=None, MaxResults: int=None) -> SearchContactsResponse:
        """Searches contacts and lists the ones that meet a set of filter and sort criteria.

        :param self:
        :param MaxResults:  The maximum number of results to include in the response. If more results exist than the specified MaxResults value, a token is included in the response so that the remaining results can be retrieved.<br/>NextToken:  An optional token returned from a prior request. Use this token for pagination of results from this action. If this parameter is specified, the response only includes results beyond the token, up to the value specified by MaxResults.<br/>SortCriteria:  The sort order to use in listing the specified set of contacts. The supported sort keys are DisplayName, FirstName, and LastName.<br/>Filters:  The filters to use to list a specified set of address books. The supported filter keys are DisplayName, FirstName, LastName, and AddressBookArns.<br/>
        :return: TotalCount:  The total number of contacts returned.<br/>NextToken:  The token returned to indicate that there is more data available.<br/>Contacts:  The contacts that meet the specified set of filter criteria, in sort order.<br/>        """
        pass

    def search_devices(self, NextToken: str=None, MaxResults: int=None, Filters: List[Filter]=None, SortCriteria: List[Sort]=None) -> SearchDevicesResponse:
        """Searches devices and lists the ones that meet a set of filter criteria.

        :param self:
        :param SortCriteria:  The sort order to use in listing the specified set of devices. Supported sort keys are DeviceName, DeviceStatus, RoomName, DeviceType, DeviceSerialNumber, ConnectionStatus, NetworkProfileName, NetworkProfileArn, Feature, and FailureCode.<br/>Filters:  The filters to use to list a specified set of devices. Supported filter keys are DeviceName, DeviceStatus, DeviceStatusDetailCode, RoomName, DeviceType, DeviceSerialNumber, UnassociatedOnly, ConnectionStatus (ONLINE and OFFLINE), NetworkProfileName, NetworkProfileArn, Feature, and FailureCode.<br/>MaxResults:  The maximum number of results to include in the response. If more results exist than the specified <code>MaxResults</code> value, a token is included in the response so that the remaining results can be retrieved.<br/>NextToken:  An optional token returned from a prior request. Use this token for pagination of results from this action. If this parameter is specified, the response includes only results beyond the token, up to the value specified by <code>MaxResults</code>.<br/>
        :return: TotalCount:  The total number of devices returned.<br/>NextToken:  The token returned to indicate that there is more data available.<br/>Devices:  The devices that meet the specified set of filter criteria, in sort order.<br/>        """
        pass

    def search_network_profiles(self, NextToken: str=None, MaxResults: int=None, Filters: List[Filter]=None, SortCriteria: List[Sort]=None) -> SearchNetworkProfilesResponse:
        """Searches network profiles and lists the ones that meet a set of filter and sort criteria.

        :param self:
        :param SortCriteria:  The sort order to use to list the specified set of network profiles. Valid sort criteria includes NetworkProfileName, Ssid, and SecurityType.<br/>Filters:  The filters to use to list a specified set of network profiles. Valid filters are NetworkProfileName, Ssid, and SecurityType.<br/>MaxResults:  The maximum number of results to include in the response. If more results exist than the specified MaxResults value, a token is included in the response so that the remaining results can be retrieved. <br/>NextToken:  An optional token returned from a prior request. Use this token for pagination of results from this action. If this parameter is specified, the response includes only results beyond the token, up to the value specified by MaxResults. <br/>
        :return: TotalCount:  The total number of network profiles returned.<br/>NextToken:  An optional token returned from a prior request. Use this token for pagination of results from this action. If this parameter is specified, the response includes only results beyond the token, up to the value specified by MaxResults.<br/>NetworkProfiles:  The network profiles that meet the specified set of filter criteria, in sort order. It is a list of NetworkProfileData objects. <br/>        """
        pass

    def search_profiles(self, NextToken: str=None, MaxResults: int=None, Filters: List[Filter]=None, SortCriteria: List[Sort]=None) -> SearchProfilesResponse:
        """Searches room profiles and lists the ones that meet a set of filter criteria.

        :param self:
        :param SortCriteria:  The sort order to use in listing the specified set of room profiles. Supported sort keys are ProfileName and Address.<br/>Filters:  The filters to use to list a specified set of room profiles. Supported filter keys are ProfileName and Address. Required. <br/>MaxResults:  The maximum number of results to include in the response. If more results exist than the specified <code>MaxResults</code> value, a token is included in the response so that the remaining results can be retrieved.<br/>NextToken:  An optional token returned from a prior request. Use this token for pagination of results from this action. If this parameter is specified, the response includes only results beyond the token, up to the value specified by <code>MaxResults</code>.<br/>
        :return: TotalCount:  The total number of room profiles returned.<br/>NextToken:  The token returned to indicate that there is more data available.<br/>Profiles:  The profiles that meet the specified set of filter criteria, in sort order.<br/>        """
        pass

    def search_rooms(self, NextToken: str=None, MaxResults: int=None, Filters: List[Filter]=None, SortCriteria: List[Sort]=None) -> SearchRoomsResponse:
        """Searches rooms and lists the ones that meet a set of filter and sort criteria.

        :param self:
        :param SortCriteria:  The sort order to use in listing the specified set of rooms. The supported sort keys are RoomName and ProfileName.<br/>Filters:  The filters to use to list a specified set of rooms. The supported filter keys are RoomName and ProfileName.<br/>MaxResults:  The maximum number of results to include in the response. If more results exist than the specified <code>MaxResults</code> value, a token is included in the response so that the remaining results can be retrieved. <br/>NextToken:  An optional token returned from a prior request. Use this token for pagination of results from this action. If this parameter is specified, the response includes only results beyond the token, up to the value specified by <code>MaxResults</code>.<br/>
        :return: TotalCount:  The total number of rooms returned.<br/>NextToken:  The token returned to indicate that there is more data available.<br/>Rooms:  The rooms that meet the specified set of filter criteria, in sort order.<br/>        """
        pass

    def search_skill_groups(self, NextToken: str=None, MaxResults: int=None, Filters: List[Filter]=None, SortCriteria: List[Sort]=None) -> SearchSkillGroupsResponse:
        """Searches skill groups and lists the ones that meet a set of filter and sort criteria.

        :param self:
        :param SortCriteria:  The sort order to use in listing the specified set of skill groups. The supported sort key is SkillGroupName. <br/>Filters:  The filters to use to list a specified set of skill groups. The supported filter key is SkillGroupName. <br/>MaxResults:  The maximum number of results to include in the response. If more results exist than the specified <code>MaxResults</code> value, a token is included in the response so that the remaining results can be retrieved. <br/>NextToken:  An optional token returned from a prior request. Use this token for pagination of results from this action. If this parameter is specified, the response includes only results beyond the token, up to the value specified by <code>MaxResults</code>. Required.<br/>
        :return: TotalCount:  The total number of skill groups returned.<br/>NextToken:  The token returned to indicate that there is more data available.<br/>SkillGroups:  The skill groups that meet the filter criteria, in sort order.<br/>        """
        pass

    def search_users(self, NextToken: str=None, MaxResults: int=None, Filters: List[Filter]=None, SortCriteria: List[Sort]=None) -> SearchUsersResponse:
        """Searches users and lists the ones that meet a set of filter and sort criteria.

        :param self:
        :param SortCriteria:  The sort order to use in listing the filtered set of users. Required. Supported sort keys are UserId, FirstName, LastName, Email, and EnrollmentStatus.<br/>Filters:  The filters to use for listing a specific set of users. Required. Supported filter keys are UserId, FirstName, LastName, Email, and EnrollmentStatus.<br/>MaxResults:  The maximum number of results to include in the response. If more results exist than the specified <code>MaxResults</code> value, a token is included in the response so that the remaining results can be retrieved. Required.<br/>NextToken:  An optional token returned from a prior request. Use this token for pagination of results from this action. If this parameter is specified, the response includes only results beyond the token, up to the value specified by <code>MaxResults</code>. Required.<br/>
        :return: TotalCount:  The total number of users returned.<br/>NextToken:  The token returned to indicate that there is more data available.<br/>Users:  The users that meet the specified set of filter criteria, in sort order.<br/>        """
        pass

    def send_announcement(self, ClientRequestToken: str, Content: Content, RoomFilters: List[Filter], TimeToLiveInSeconds: int=None) -> SendAnnouncementResponse:
        """Triggers an asynchronous flow to send text, SSML, or audio announcements to rooms that are identified by a search or filter. 

        :param self:
        :param ClientRequestToken:  The unique, user-specified identifier for the request that ensures idempotency.<br/>TimeToLiveInSeconds:  The time to live for an announcement. Default is 300. If delivery doesn't occur within this time, the announcement is not delivered.<br/>Content:  The announcement content. This can contain only one of the three possible announcement types (text, SSML or audio).:<br/>RoomFilters:  The filters to use to send an announcement to a specified list of rooms. The supported filter keys are RoomName, ProfileName, RoomArn, and ProfileArn. To send to all rooms, specify an empty RoomFilters list.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AudioList</i> The list of audio messages.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SsmlList</i> The list of SSML messages.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TextList</i> The list of text messages.<br/>
        :return: AnnouncementArn:  The identifier of the announcement.<br/>        """
        pass

    def send_invitation(self, UserArn: str=None) -> SendInvitationResponse:
        """Sends an enrollment invitation email with a URL to a user. The URL is valid for 30 days or until you call this operation again, whichever comes first. 

        :param self:
        :param UserArn:  The ARN of the user to whom to send an invitation. Required.<br/>
        :return:         """
        pass

    def start_device_sync(self, Features: List[Feature], RoomArn: str=None, DeviceArn: str=None) -> StartDeviceSyncResponse:
        """Resets a device and its account to the known default settings. This clears all information and settings set by previous users in the following ways: <ul> <li> Bluetooth - This unpairs all bluetooth devices paired with your echo device. </li> <li> Volume - This resets the echo device's volume to the default value. </li> <li> Notifications - This clears all notifications from your echo device. </li> <li> Lists - This clears all to-do items from your echo device. </li> <li> Settings - This internally syncs the room's profile (if the device is assigned to a room), contacts, address books, delegation access for account linking, and communications (if enabled on the room profile). </li> </ul>

        :param self:
        :param Features:  Request structure to start the device sync. Required.<br/>DeviceArn:  The ARN of the device to sync. Required.<br/>RoomArn:  The ARN of the room with which the device to sync is associated. Required.<br/>
        :return:         """
        pass

    def start_smart_home_appliance_discovery(self, RoomArn: str) -> StartSmartHomeApplianceDiscoveryResponse:
        """Initiates the discovery of any smart home appliances associated with the room.

        :param self:
        :param RoomArn:  The room where smart home appliance discovery was initiated.<br/>
        :return:         """
        pass

    def tag_resource(self, Tags: List[Tag], Arn: str) -> TagResourceResponse:
        """Adds metadata tags to a specified resource.

        :param self:
        :param Tags:  The tags to be added to the specified resource. Do not provide system tags. Required. <br/>Arn:  The ARN of the resource to which to add metadata tags. Required. <br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[TagKey], Arn: str) -> UntagResourceResponse:
        """Removes metadata tags from a specified resource.

        :param self:
        :param TagKeys:  The tags to be removed from the specified resource. Do not provide system tags. Required. <br/>Arn:  The ARN of the resource from which to remove metadata tags. Required. <br/>
        :return:         """
        pass

    def update_address_book(self, AddressBookArn: str, Name: str=None, Description: str=None) -> UpdateAddressBookResponse:
        """Updates address book details by the address book ARN.

        :param self:
        :param Description:  The updated description of the room.<br/>Name:  The updated name of the room.<br/>AddressBookArn:  The ARN of the room to update.<br/>
        :return:         """
        pass

    def update_business_report_schedule(self, ScheduleArn: str, S3BucketName: str=None, S3KeyPrefix: str=None, Format: str=None, ScheduleName: str=None, Recurrence: BusinessReportRecurrence=None) -> UpdateBusinessReportScheduleResponse:
        """Updates the configuration of the report delivery schedule with the specified schedule ARN.

        :param self:
        :param Recurrence:  The recurrence of the reports.:<br/>ScheduleName:  The name identifier of the schedule.<br/>Format:  The format of the generated report (individual CSV files or zipped files of individual files).<br/>S3KeyPrefix:  The S3 key where the report is delivered.<br/>S3BucketName:  The S3 location of the output reports.<br/>ScheduleArn:  The ARN of the business report schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartDate</i> The start date.<br/>
        :return:         """
        pass

    def update_conference_provider(self, MeetingSetting: MeetingSetting, ConferenceProviderType: str, ConferenceProviderArn: str, IPDialIn: IPDialIn=None, PSTNDialIn: PSTNDialIn=None) -> UpdateConferenceProviderResponse:
        """Updates an existing conference provider's settings.

        :param self:
        :param MeetingSetting:  The meeting settings for the conference provider.:<br/>PSTNDialIn:  The information for PSTN conferencing.:<br/>IPDialIn:  The IP endpoint and protocol for calling.:<br/>ConferenceProviderType:  The type of the conference provider.<br/>ConferenceProviderArn:  The ARN of the conference provider.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CommsProtocol</i> The protocol, including SIP, SIPS, and H323.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> The IP address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OneClickPinDelay</i> The delay duration before Alexa enters the conference pin with dual-tone multi-frequency (DTMF). Each number on the dial pad corresponds to a DTMF tone, which is how we send data over the telephone network.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OneClickIdDelay</i> The delay duration before Alexa enters the conference ID with dual-tone multi-frequency (DTMF). Each number on the dial pad corresponds to a DTMF tone, which is how we send data over the telephone network.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneNumber</i> The phone number to call to join the conference.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CountryCode</i> The zip code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequirePin</i> The values that indicate whether the pin is always required.<br/>
        :return:         """
        pass

    def update_contact(self, ContactArn: str, DisplayName: str=None, FirstName: str=None, LastName: str=None, PhoneNumber: str=None, PhoneNumbers: List[PhoneNumber]=None, SipAddresses: List[SipAddress]=None) -> UpdateContactResponse:
        """Updates the contact details by the contact ARN.

        :param self:
        :param SipAddresses:  The list of SIP addresses for the contact.<br/>PhoneNumbers:  The list of phone numbers for the contact.<br/>PhoneNumber:  The updated phone number of the contact. The phone number type defaults to WORK. You can either specify PhoneNumber or PhoneNumbers. We recommend that you use PhoneNumbers, which lets you specify the phone number type and multiple numbers.<br/>LastName:  The updated last name of the contact.<br/>FirstName:  The updated first name of the contact.<br/>DisplayName:  The updated display name of the contact.<br/>ContactArn:  The ARN of the contact to update.<br/>
        :return:         """
        pass

    def update_device(self, DeviceArn: str=None, DeviceName: str=None) -> UpdateDeviceResponse:
        """Updates the device name by device ARN.

        :param self:
        :param DeviceName:  The updated device name. Required.<br/>DeviceArn:  The ARN of the device to update. Required.<br/>
        :return:         """
        pass

    def update_gateway(self, GatewayArn: str, Name: str=None, Description: str=None, SoftwareVersion: str=None) -> UpdateGatewayResponse:
        """Updates the details of a gateway. If any optional field is not provided, the existing corresponding value is left unmodified.

        :param self:
        :param SoftwareVersion:  The updated software version of the gateway. The gateway automatically updates its software version during normal operation.<br/>Description:  The updated description of the gateway.<br/>Name:  The updated name of the gateway.<br/>GatewayArn:  The ARN of the gateway to update.<br/>
        :return:         """
        pass

    def update_gateway_group(self, GatewayGroupArn: str, Name: str=None, Description: str=None) -> UpdateGatewayGroupResponse:
        """Updates the details of a gateway group. If any optional field is not provided, the existing corresponding value is left unmodified.

        :param self:
        :param Description:  The updated description of the gateway group.<br/>Name:  The updated name of the gateway group.<br/>GatewayGroupArn:  The ARN of the gateway group to update.<br/>
        :return:         """
        pass

    def update_network_profile(self, NetworkProfileArn: str, NetworkProfileName: str=None, Description: str=None, CurrentPassword: str=None, NextPassword: str=None, CertificateAuthorityArn: str=None, TrustAnchors: List[TrustAnchor]=None) -> UpdateNetworkProfileResponse:
        """Updates a network profile by the network profile ARN.

        :param self:
        :param TrustAnchors:  The root certificate(s) of your authentication server that will be installed on your devices and used to trust your authentication server during EAP negotiation. <br/>CertificateAuthorityArn:  The ARN of the Private Certificate Authority (PCA) created in AWS Certificate Manager (ACM). This is used to issue certificates to the devices. <br/>NextPassword:  The next, or subsequent, password of the Wi-Fi network. This password is asynchronously transmitted to the device and is used when the password of the network changes to NextPassword. <br/>CurrentPassword:  The current password of the Wi-Fi network.<br/>Description:  Detailed information about a device's network profile.<br/>NetworkProfileName:  The name of the network profile associated with a device.<br/>NetworkProfileArn:  The ARN of the network profile associated with a device.<br/>
        :return:         """
        pass

    def update_profile(self, ProfileArn: str=None, ProfileName: str=None, IsDefault: Boolean=None, Timezone: str=None, Address: str=None, DistanceUnit: str=None, TemperatureUnit: str=None, WakeWord: str=None, SetupModeDisabled: Boolean=None, MaxVolumeLimit: int=None, PSTNEnabled: Boolean=None) -> UpdateProfileResponse:
        """Updates an existing room profile by room profile ARN.

        :param self:
        :param PSTNEnabled:  Whether the PSTN setting of the room profile is enabled.<br/>MaxVolumeLimit:  The updated maximum volume limit for the room profile.<br/>SetupModeDisabled:  Whether the setup mode of the profile is enabled.<br/>WakeWord:  The updated wake word for the room profile.<br/>TemperatureUnit:  The updated temperature unit for the room profile.<br/>DistanceUnit:  The updated distance unit for the room profile.<br/>Address:  The updated address for the room profile.<br/>Timezone:  The updated timezone for the room profile.<br/>IsDefault:  Sets the profile as default if selected. If this is missing, no update is done to the default status.<br/>ProfileName:  The updated name for the room profile.<br/>ProfileArn:  The ARN of the room profile to update. Required.<br/>
        :return:         """
        pass

    def update_room(self, RoomArn: str=None, RoomName: str=None, Description: str=None, ProviderCalendarId: str=None, ProfileArn: str=None) -> UpdateRoomResponse:
        """Updates room details by room ARN.

        :param self:
        :param ProfileArn:  The updated profile ARN for the room.<br/>ProviderCalendarId:  The updated provider calendar ARN for the room.<br/>Description:  The updated description for the room.<br/>RoomName:  The updated name for the room.<br/>RoomArn:  The ARN of the room to update. <br/>
        :return:         """
        pass

    def update_skill_group(self, SkillGroupArn: str=None, SkillGroupName: str=None, Description: str=None) -> UpdateSkillGroupResponse:
        """Updates skill group details by skill group ARN.

        :param self:
        :param Description:  The updated description for the skill group.<br/>SkillGroupName:  The updated name for the skill group.<br/>SkillGroupArn:  The ARN of the skill group to update. <br/>
        :return:         """
        pass



class Amplify(BaseClient):
    """ Amplify is a fully managed continuous deployment and hosting service for modern web apps. """


    class ListAppsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_apps` for valid parameters"""
            pass
    class ListBranchesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_branches` for valid parameters"""
            pass
    class ListDomainAssociationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_domain_associations` for valid parameters"""
            pass
    class ListJobsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_jobs` for valid parameters"""
            pass


    class CreateAppRequest(Mapping):
        pass
    
    class CreateAppResult(Mapping):
        pass
    
    class EnvironmentVariables(object):
        pass
    
    class EnableBranchAutoBuild(object):
        pass
    
    class EnableBasicAuth(object):
        pass
    
    class CustomRule(Mapping):
        pass
    
    class TagMap(object):
        pass
    
    class EnableAutoBranchCreation(object):
        pass
    
    class AutoBranchCreationPattern(object):
        pass
    
    class AutoBranchCreationConfig(Mapping):
        pass
    
    class CreateBranchRequest(Mapping):
        pass
    
    class CreateBranchResult(Mapping):
        pass
    
    class EnableNotification(object):
        pass
    
    class EnableAutoBuild(object):
        pass
    
    class CreateDeploymentRequest(Mapping):
        pass
    
    class CreateDeploymentResult(Mapping):
        pass
    
    class FileMap(object):
        pass
    
    class CreateDomainAssociationRequest(Mapping):
        pass
    
    class CreateDomainAssociationResult(Mapping):
        pass
    
    class EnableAutoSubDomain(object):
        pass
    
    class SubDomainSetting(Mapping):
        pass
    
    class CreateWebhookRequest(Mapping):
        pass
    
    class CreateWebhookResult(Mapping):
        pass
    
    class DeleteAppRequest(Mapping):
        pass
    
    class DeleteAppResult(Mapping):
        pass
    
    class DeleteBranchRequest(Mapping):
        pass
    
    class DeleteBranchResult(Mapping):
        pass
    
    class DeleteDomainAssociationRequest(Mapping):
        pass
    
    class DeleteDomainAssociationResult(Mapping):
        pass
    
    class DeleteJobRequest(Mapping):
        pass
    
    class DeleteJobResult(Mapping):
        pass
    
    class DeleteWebhookRequest(Mapping):
        pass
    
    class DeleteWebhookResult(Mapping):
        pass
    
    class GetAppRequest(Mapping):
        pass
    
    class GetAppResult(Mapping):
        pass
    
    class GetBranchRequest(Mapping):
        pass
    
    class GetBranchResult(Mapping):
        pass
    
    class GetDomainAssociationRequest(Mapping):
        pass
    
    class GetDomainAssociationResult(Mapping):
        pass
    
    class GetJobRequest(Mapping):
        pass
    
    class GetJobResult(Mapping):
        pass
    
    class GetWebhookRequest(Mapping):
        pass
    
    class GetWebhookResult(Mapping):
        pass
    
    class ListAppsRequest(Mapping):
        pass
    
    class ListAppsResult(Mapping):
        pass
    
    class ListBranchesRequest(Mapping):
        pass
    
    class ListBranchesResult(Mapping):
        pass
    
    class ListDomainAssociationsRequest(Mapping):
        pass
    
    class ListDomainAssociationsResult(Mapping):
        pass
    
    class ListJobsRequest(Mapping):
        pass
    
    class ListJobsResult(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class ListWebhooksRequest(Mapping):
        pass
    
    class ListWebhooksResult(Mapping):
        pass
    
    class StartDeploymentRequest(Mapping):
        pass
    
    class StartDeploymentResult(Mapping):
        pass
    
    class StartJobRequest(Mapping):
        pass
    
    class StartJobResult(Mapping):
        pass
    
    class CommitTime(object):
        pass
    
    class StopJobRequest(Mapping):
        pass
    
    class StopJobResult(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateAppRequest(Mapping):
        pass
    
    class UpdateAppResult(Mapping):
        pass
    
    class UpdateBranchRequest(Mapping):
        pass
    
    class UpdateBranchResult(Mapping):
        pass
    
    class UpdateDomainAssociationRequest(Mapping):
        pass
    
    class UpdateDomainAssociationResult(Mapping):
        pass
    
    class UpdateWebhookRequest(Mapping):
        pass
    
    class UpdateWebhookResult(Mapping):
        pass
    
    def create_app(self, name: str, description: str=None, repository: str=None, platform: str=None, iamServiceRoleArn: str=None, oauthToken: str=None, accessToken: str=None, environmentVariables: EnvironmentVariables=None, enableBranchAutoBuild: EnableBranchAutoBuild=None, enableBasicAuth: EnableBasicAuth=None, basicAuthCredentials: str=None, customRules: List[CustomRule]=None, tags: TagMap=None, buildSpec: str=None, enableAutoBranchCreation: EnableAutoBranchCreation=None, autoBranchCreationPatterns: List[AutoBranchCreationPattern]=None, autoBranchCreationConfig: AutoBranchCreationConfig=None) -> CreateAppResult:
        """ Creates a new Amplify App. 

        :param self:
        :param autoBranchCreationConfig:   Automated branch creation config for the Amplify App. :<br/>autoBranchCreationPatterns:   Automated branch creation glob patterns for the Amplify App. <br/>enableAutoBranchCreation:   Enables automated branch creation for the Amplify App. <br/>buildSpec:   BuildSpec for an Amplify App <br/>tags:   Tag for an Amplify App <br/>customRules:   Custom rewrite / redirect rules for an Amplify App. <br/>basicAuthCredentials:   Credentials for Basic Authorization for an Amplify App. <br/>enableBasicAuth:   Enable Basic Authorization for an Amplify App, this will apply to all branches part of this App. <br/>enableBranchAutoBuild:   Enable the auto building of branches for an Amplify App. <br/>environmentVariables:   Environment variables map for an Amplify App. <br/>accessToken:   Personal Access token for 3rd party source control system for an Amplify App, used to create webhook and read-only deploy key. Token is not stored. <br/>oauthToken:   OAuth token for 3rd party source control system for an Amplify App, used to create webhook and read-only deploy key. OAuth token is not stored. <br/>iamServiceRoleArn:   AWS IAM service role for an Amplify App <br/>platform:   Platform / framework for an Amplify App <br/>repository:   Repository for an Amplify App <br/>description:   Description for an Amplify App <br/>name:   Name for the Amplify App <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>buildSpec</i>  BuildSpec for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableBasicAuth</i>  Enables Basic Auth for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>basicAuthCredentials</i>  Basic Authorization credentials for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>environmentVariables</i>  Environment Variables for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableAutoBuild</i>  Enables auto building for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>framework</i>  Framework for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stage</i>  Stage for the auto created branch. <br/>
        :return: app:   Amplify App represents different branches of a repository for building, deploying, and hosting. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>autoBranchCreationConfig</i>  Automated branch creation config for the Amplify App. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>autoBranchCreationConfig</b>:  Automated branch creation config for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>autoBranchCreationPatterns</i>  Automated branch creation glob patterns for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableAutoBranchCreation</i>  Enables automated branch creation for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>buildSpec</i>  BuildSpec content for Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>productionBranch</i>  Structure with Production Branch information. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>productionBranch</b>:  Structure with Production Branch information. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customRules</i>  Custom redirect / rewrite rules for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>basicAuthCredentials</i>  Basic Authorization credentials for branches for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableBasicAuth</i>  Enables Basic Authorization for branches for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableBranchAutoBuild</i>  Enables auto-building of branches for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultDomain</i>  Default domain for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>environmentVariables</i>  Environment Variables for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>iamServiceRoleArn</i>  IAM service role ARN for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>updateTime</i>  Update date / time for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createTime</i>  Create date / time for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>platform</i>  Platform for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>repository</i>  Repository for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i>  Description for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i>  Tag for Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i>  Name for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>appArn</i>  ARN for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>appId</i>  Unique Id for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>branchName</i>  Branch Name for Production Branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>thumbnailUrl</i>  Thumbnail URL for Production Branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i>  Status of Production Branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastDeployTime</i>  Last Deploy Time of Production Branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>buildSpec</i>  BuildSpec for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>enableBasicAuth</i>  Enables Basic Auth for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>basicAuthCredentials</i>  Basic Authorization credentials for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>environmentVariables</i>  Environment Variables for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>enableAutoBuild</i>  Enables auto building for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>framework</i>  Framework for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>stage</i>  Stage for the auto created branch. <br/>        """
        pass

    def create_branch(self, branchName: str, appId: str, description: str=None, stage: str=None, framework: str=None, enableNotification: EnableNotification=None, enableAutoBuild: EnableAutoBuild=None, environmentVariables: EnvironmentVariables=None, basicAuthCredentials: str=None, enableBasicAuth: EnableBasicAuth=None, tags: TagMap=None, buildSpec: str=None, ttl: str=None, displayName: str=None) -> CreateBranchResult:
        """ Creates a new Branch for an Amplify App. 

        :param self:
        :param displayName:   Display name for a branch, will use as the default domain prefix. <br/>ttl:   The content TTL for the website in seconds. <br/>buildSpec:   BuildSpec for the branch. <br/>tags:   Tag for the branch. <br/>enableBasicAuth:   Enables Basic Auth for the branch. <br/>basicAuthCredentials:   Basic Authorization credentials for the branch. <br/>environmentVariables:   Environment Variables for the branch. <br/>enableAutoBuild:   Enables auto building for the branch. <br/>enableNotification:   Enables notifications for the branch. <br/>framework:   Framework for the branch. <br/>stage:   Stage for the branch. <br/>description:   Description for the branch. <br/>branchName:   Name for the branch. <br/>appId:   Unique Id for an Amplify App. <br/>
        :return: branch:   Branch structure for an Amplify App. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>associatedResources</i>  List of custom resources that are linked to this branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ttl</i>  The content TTL for the website in seconds. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>buildSpec</i>  BuildSpec content for branch for Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>basicAuthCredentials</i>  Basic Authorization credentials for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>thumbnailUrl</i>  Thumbnail URL for the branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableBasicAuth</i>  Enables Basic Authorization for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>totalNumberOfJobs</i>  Total number of Jobs part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>activeJobId</i>  Id of the active job for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>framework</i>  Framework for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customDomains</i>  Custom domains for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableAutoBuild</i>  Enables auto-building on push for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>environmentVariables</i>  Environment Variables specific to a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>updateTime</i>  Last updated date and time for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createTime</i>  Creation date and time for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableNotification</i>  Enables notifications for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>displayName</i>  Display name for a branch, will use as the default domain prefix. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stage</i>  Stage for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i>  Tag for branch for Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i>  Description for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>branchName</i>  Name for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>branchArn</i>  ARN for a branch, part of an Amplify App. <br/>        """
        pass

    def create_deployment(self, branchName: str, appId: str, fileMap: FileMap=None) -> CreateDeploymentResult:
        """ Create a deployment for manual deploy apps. (Apps are not connected to repository) 

        :param self:
        :param fileMap:   Optional file map that contains file name as the key and file content md5 hash as the value. If this argument is provided, the service will generate different upload url per file. Otherwise, the service will only generate a single upload url for the zipped files. <br/>branchName:   Name for the branch, for the Job. <br/>appId:   Unique Id for an Amplify App. <br/>
        :return: zipUploadUrl:   When the fileMap argument is NOT provided. This zipUploadUrl will be returned. <br/>fileUploadUrls:   When the fileMap argument is provided in the request, the fileUploadUrls will contain a map of file names to upload url. <br/>jobId:   The jobId for this deployment, will supply to start deployment api. <br/>        """
        pass

    def create_domain_association(self, subDomainSettings: List[SubDomainSetting], domainName: str, appId: str, enableAutoSubDomain: EnableAutoSubDomain=None) -> CreateDomainAssociationResult:
        """ Create a new DomainAssociation on an App 

        :param self:
        :param subDomainSettings:   Setting structure for the Subdomain. <br/>enableAutoSubDomain:   Enables automated creation of Subdomains for branches. <br/>domainName:   Domain name for the Domain Association. <br/>appId:   Unique Id for an Amplify App. <br/>
        :return: domainAssociation:   Domain Association structure. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>subDomains</i>  Subdomains for the Domain Association. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>certificateVerificationDNSRecord</i>  DNS Record for certificate verification. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusReason</i>  Reason for the current status of the Domain Association. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>domainStatus</i>  Status fo the Domain Association. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableAutoSubDomain</i>  Enables automated creation of Subdomains for branches. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>domainName</i>  Name of the domain. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>domainAssociationArn</i>  ARN for the Domain Association. <br/>        """
        pass

    def create_webhook(self, branchName: str, appId: str, description: str=None) -> CreateWebhookResult:
        """ Create a new webhook on an App. 

        :param self:
        :param description:   Description for a webhook. <br/>branchName:   Name for a branch, part of an Amplify App. <br/>appId:   Unique Id for an Amplify App. <br/>
        :return: webhook:   Webhook structure. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>updateTime</i>  Update date / time for a webhook. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createTime</i>  Create date / time for a webhook. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i>  Description for a webhook. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>branchName</i>  Name for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>webhookUrl</i>  Url of the webhook. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>webhookId</i>  Id of the webhook. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>webhookArn</i>  ARN for the webhook. <br/>        """
        pass

    def delete_app(self, appId: str) -> DeleteAppResult:
        """ Delete an existing Amplify App by appId. 

        :param self:
        :param appId:   Unique Id for an Amplify App. <br/>
        :return: app:   Amplify App represents different branches of a repository for building, deploying, and hosting. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>autoBranchCreationConfig</i>  Automated branch creation config for the Amplify App. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>autoBranchCreationConfig</b>:  Automated branch creation config for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>autoBranchCreationPatterns</i>  Automated branch creation glob patterns for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableAutoBranchCreation</i>  Enables automated branch creation for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>buildSpec</i>  BuildSpec content for Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>productionBranch</i>  Structure with Production Branch information. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>productionBranch</b>:  Structure with Production Branch information. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customRules</i>  Custom redirect / rewrite rules for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>basicAuthCredentials</i>  Basic Authorization credentials for branches for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableBasicAuth</i>  Enables Basic Authorization for branches for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableBranchAutoBuild</i>  Enables auto-building of branches for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultDomain</i>  Default domain for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>environmentVariables</i>  Environment Variables for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>iamServiceRoleArn</i>  IAM service role ARN for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>updateTime</i>  Update date / time for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createTime</i>  Create date / time for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>platform</i>  Platform for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>repository</i>  Repository for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i>  Description for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i>  Tag for Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i>  Name for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>appArn</i>  ARN for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>appId</i>  Unique Id for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>branchName</i>  Branch Name for Production Branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>thumbnailUrl</i>  Thumbnail URL for Production Branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i>  Status of Production Branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastDeployTime</i>  Last Deploy Time of Production Branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>buildSpec</i>  BuildSpec for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>enableBasicAuth</i>  Enables Basic Auth for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>basicAuthCredentials</i>  Basic Authorization credentials for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>environmentVariables</i>  Environment Variables for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>enableAutoBuild</i>  Enables auto building for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>framework</i>  Framework for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>stage</i>  Stage for the auto created branch. <br/>        """
        pass

    def delete_branch(self, branchName: str, appId: str) -> DeleteBranchResult:
        """ Deletes a branch for an Amplify App. 

        :param self:
        :param branchName:   Name for the branch. <br/>appId:   Unique Id for an Amplify App. <br/>
        :return: branch:   Branch structure for an Amplify App. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>associatedResources</i>  List of custom resources that are linked to this branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ttl</i>  The content TTL for the website in seconds. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>buildSpec</i>  BuildSpec content for branch for Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>basicAuthCredentials</i>  Basic Authorization credentials for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>thumbnailUrl</i>  Thumbnail URL for the branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableBasicAuth</i>  Enables Basic Authorization for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>totalNumberOfJobs</i>  Total number of Jobs part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>activeJobId</i>  Id of the active job for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>framework</i>  Framework for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customDomains</i>  Custom domains for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableAutoBuild</i>  Enables auto-building on push for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>environmentVariables</i>  Environment Variables specific to a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>updateTime</i>  Last updated date and time for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createTime</i>  Creation date and time for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableNotification</i>  Enables notifications for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>displayName</i>  Display name for a branch, will use as the default domain prefix. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stage</i>  Stage for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i>  Tag for branch for Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i>  Description for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>branchName</i>  Name for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>branchArn</i>  ARN for a branch, part of an Amplify App. <br/>        """
        pass

    def delete_domain_association(self, domainName: str, appId: str) -> DeleteDomainAssociationResult:
        """ Deletes a DomainAssociation. 

        :param self:
        :param domainName:   Name of the domain. <br/>appId:   Unique Id for an Amplify App. <br/>
        :return: domainAssociation:   Structure for Domain Association, which associates a custom domain with an Amplify App. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>subDomains</i>  Subdomains for the Domain Association. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>certificateVerificationDNSRecord</i>  DNS Record for certificate verification. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusReason</i>  Reason for the current status of the Domain Association. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>domainStatus</i>  Status fo the Domain Association. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableAutoSubDomain</i>  Enables automated creation of Subdomains for branches. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>domainName</i>  Name of the domain. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>domainAssociationArn</i>  ARN for the Domain Association. <br/>        """
        pass

    def delete_job(self, jobId: str, branchName: str, appId: str) -> DeleteJobResult:
        """ Delete a job, for an Amplify branch, part of Amplify App. 

        :param self:
        :param jobId:   Unique Id for the Job. <br/>branchName:   Name for the branch, for the Job. <br/>appId:   Unique Id for an Amplify App. <br/>
        :return: jobSummary:   Structure for the summary of a Job. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jobType</i>  Type for the Job. \n "RELEASE": Manually released from source by using StartJob API. "RETRY": Manually retried by using StartJob API. "WEB_HOOK": Automatically triggered by WebHooks. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>endTime</i>  End date / time for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i>  Status for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>startTime</i>  Start date / time for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>commitTime</i>  Commit date / time for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>commitMessage</i>  Commit message from 3rd party repository provider for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>commitId</i>  Commit Id from 3rd party repository provider for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jobId</i>  Unique Id for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jobArn</i>  Arn for the Job. <br/>        """
        pass

    def delete_webhook(self, webhookId: str) -> DeleteWebhookResult:
        """ Deletes a webhook. 

        :param self:
        :param webhookId:   Unique Id for a webhook. <br/>
        :return: webhook:   Webhook structure. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>updateTime</i>  Update date / time for a webhook. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createTime</i>  Create date / time for a webhook. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i>  Description for a webhook. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>branchName</i>  Name for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>webhookUrl</i>  Url of the webhook. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>webhookId</i>  Id of the webhook. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>webhookArn</i>  ARN for the webhook. <br/>        """
        pass

    def get_app(self, appId: str) -> GetAppResult:
        """ Retrieves an existing Amplify App by appId. 

        :param self:
        :param appId:   Unique Id for an Amplify App. <br/>
        :return: app:   Amplify App represents different branches of a repository for building, deploying, and hosting. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>autoBranchCreationConfig</i>  Automated branch creation config for the Amplify App. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>autoBranchCreationConfig</b>:  Automated branch creation config for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>autoBranchCreationPatterns</i>  Automated branch creation glob patterns for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableAutoBranchCreation</i>  Enables automated branch creation for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>buildSpec</i>  BuildSpec content for Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>productionBranch</i>  Structure with Production Branch information. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>productionBranch</b>:  Structure with Production Branch information. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customRules</i>  Custom redirect / rewrite rules for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>basicAuthCredentials</i>  Basic Authorization credentials for branches for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableBasicAuth</i>  Enables Basic Authorization for branches for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableBranchAutoBuild</i>  Enables auto-building of branches for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultDomain</i>  Default domain for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>environmentVariables</i>  Environment Variables for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>iamServiceRoleArn</i>  IAM service role ARN for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>updateTime</i>  Update date / time for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createTime</i>  Create date / time for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>platform</i>  Platform for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>repository</i>  Repository for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i>  Description for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i>  Tag for Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i>  Name for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>appArn</i>  ARN for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>appId</i>  Unique Id for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>branchName</i>  Branch Name for Production Branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>thumbnailUrl</i>  Thumbnail URL for Production Branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i>  Status of Production Branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastDeployTime</i>  Last Deploy Time of Production Branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>buildSpec</i>  BuildSpec for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>enableBasicAuth</i>  Enables Basic Auth for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>basicAuthCredentials</i>  Basic Authorization credentials for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>environmentVariables</i>  Environment Variables for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>enableAutoBuild</i>  Enables auto building for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>framework</i>  Framework for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>stage</i>  Stage for the auto created branch. <br/>        """
        pass

    def get_branch(self, branchName: str, appId: str) -> GetBranchResult:
        """ Retrieves a branch for an Amplify App. 

        :param self:
        :param branchName:   Name for the branch. <br/>appId:   Unique Id for an Amplify App. <br/>
        :return: branch:   Branch for an Amplify App, which maps to a 3rd party repository branch. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>associatedResources</i>  List of custom resources that are linked to this branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ttl</i>  The content TTL for the website in seconds. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>buildSpec</i>  BuildSpec content for branch for Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>basicAuthCredentials</i>  Basic Authorization credentials for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>thumbnailUrl</i>  Thumbnail URL for the branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableBasicAuth</i>  Enables Basic Authorization for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>totalNumberOfJobs</i>  Total number of Jobs part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>activeJobId</i>  Id of the active job for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>framework</i>  Framework for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customDomains</i>  Custom domains for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableAutoBuild</i>  Enables auto-building on push for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>environmentVariables</i>  Environment Variables specific to a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>updateTime</i>  Last updated date and time for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createTime</i>  Creation date and time for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableNotification</i>  Enables notifications for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>displayName</i>  Display name for a branch, will use as the default domain prefix. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stage</i>  Stage for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i>  Tag for branch for Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i>  Description for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>branchName</i>  Name for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>branchArn</i>  ARN for a branch, part of an Amplify App. <br/>        """
        pass

    def get_domain_association(self, domainName: str, appId: str) -> GetDomainAssociationResult:
        """ Retrieves domain info that corresponds to an appId and domainName. 

        :param self:
        :param domainName:   Name of the domain. <br/>appId:   Unique Id for an Amplify App. <br/>
        :return: domainAssociation:   Domain Association structure. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>subDomains</i>  Subdomains for the Domain Association. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>certificateVerificationDNSRecord</i>  DNS Record for certificate verification. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusReason</i>  Reason for the current status of the Domain Association. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>domainStatus</i>  Status fo the Domain Association. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableAutoSubDomain</i>  Enables automated creation of Subdomains for branches. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>domainName</i>  Name of the domain. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>domainAssociationArn</i>  ARN for the Domain Association. <br/>        """
        pass

    def get_job(self, jobId: str, branchName: str, appId: str) -> GetJobResult:
        """ Get a job for a branch, part of an Amplify App. 

        :param self:
        :param jobId:   Unique Id for the Job. <br/>branchName:   Name for the branch, for the Job. <br/>appId:   Unique Id for an Amplify App. <br/>
        :return: job:   Structure for an execution job for an Amplify App. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>steps</i>  Execution steps for an execution job, for an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>summary</i>  Summary for an execution job for an Amplify App. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>summary</b>:  Summary for an execution job for an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>jobType</i>  Type for the Job. \n "RELEASE": Manually released from source by using StartJob API. "RETRY": Manually retried by using StartJob API. "WEB_HOOK": Automatically triggered by WebHooks. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>endTime</i>  End date / time for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i>  Status for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>startTime</i>  Start date / time for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>commitTime</i>  Commit date / time for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>commitMessage</i>  Commit message from 3rd party repository provider for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>commitId</i>  Commit Id from 3rd party repository provider for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>jobId</i>  Unique Id for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>jobArn</i>  Arn for the Job. <br/>        """
        pass

    def get_webhook(self, webhookId: str) -> GetWebhookResult:
        """ Retrieves webhook info that corresponds to a webhookId. 

        :param self:
        :param webhookId:   Unique Id for a webhook. <br/>
        :return: webhook:   Webhook structure. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>updateTime</i>  Update date / time for a webhook. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createTime</i>  Create date / time for a webhook. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i>  Description for a webhook. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>branchName</i>  Name for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>webhookUrl</i>  Url of the webhook. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>webhookId</i>  Id of the webhook. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>webhookArn</i>  ARN for the webhook. <br/>        """
        pass

    def list_apps(self, nextToken: str=None, maxResults: int=None) -> ListAppsResult:
        """ Lists existing Amplify Apps. 

        :param self:
        :param maxResults:   Maximum number of records to list in a single response. <br/>nextToken:   Pagination token. If non-null pagination token is returned in a result, then pass its value in another request to fetch more entries. <br/>
        :return: nextToken:   Pagination token. Set to null to start listing Apps from start. If non-null pagination token is returned in a result, then pass its value in here to list more projects. <br/>apps:   List of Amplify Apps. <br/>        """
        pass

    def list_branches(self, appId: str, nextToken: str=None, maxResults: int=None) -> ListBranchesResult:
        """ Lists branches for an Amplify App. 

        :param self:
        :param maxResults:   Maximum number of records to list in a single response. <br/>nextToken:   Pagination token. Set to null to start listing branches from start. If a non-null pagination token is returned in a result, then pass its value in here to list more branches. <br/>appId:   Unique Id for an Amplify App. <br/>
        :return: nextToken:   Pagination token. If non-null pagination token is returned in a result, then pass its value in another request to fetch more entries. <br/>branches:   List of branches for an Amplify App. <br/>        """
        pass

    def list_domain_associations(self, appId: str, nextToken: str=None, maxResults: int=None) -> ListDomainAssociationsResult:
        """ List domains with an app 

        :param self:
        :param maxResults:   Maximum number of records to list in a single response. <br/>nextToken:   Pagination token. Set to null to start listing Apps from start. If non-null pagination token is returned in a result, then pass its value in here to list more projects. <br/>appId:   Unique Id for an Amplify App. <br/>
        :return: nextToken:   Pagination token. If non-null pagination token is returned in a result, then pass its value in another request to fetch more entries. <br/>domainAssociations:   List of Domain Associations. <br/>        """
        pass

    def list_jobs(self, branchName: str, appId: str, nextToken: str=None, maxResults: int=None) -> ListJobsResult:
        """ List Jobs for a branch, part of an Amplify App. 

        :param self:
        :param maxResults:   Maximum number of records to list in a single response. <br/>nextToken:   Pagination token. Set to null to start listing steps from start. If a non-null pagination token is returned in a result, then pass its value in here to list more steps. <br/>branchName:   Name for a branch. <br/>appId:   Unique Id for an Amplify App. <br/>
        :return: nextToken:   Pagination token. If non-null pagination token is returned in a result, then pass its value in another request to fetch more entries. <br/>jobSummaries:   Result structure for list job result request. <br/>        """
        pass

    def list_tags_for_resource(self, resourceArn: str) -> ListTagsForResourceResponse:
        """ List tags for resource. 

        :param self:
        :param resourceArn:   Resource arn used to list tags. <br/>
        :return: tags:   Tags result for response. <br/>        """
        pass

    def list_webhooks(self, appId: str, nextToken: str=None, maxResults: int=None) -> ListWebhooksResult:
        """ List webhooks with an app. 

        :param self:
        :param maxResults:   Maximum number of records to list in a single response. <br/>nextToken:   Pagination token. Set to null to start listing webhooks from start. If non-null pagination token is returned in a result, then pass its value in here to list more webhooks. <br/>appId:   Unique Id for an Amplify App. <br/>
        :return: nextToken:   Pagination token. If non-null pagination token is returned in a result, then pass its value in another request to fetch more entries. <br/>webhooks:   List of webhooks. <br/>        """
        pass

    def start_deployment(self, branchName: str, appId: str, jobId: str=None, sourceUrl: str=None) -> StartDeploymentResult:
        """ Start a deployment for manual deploy apps. (Apps are not connected to repository) 

        :param self:
        :param sourceUrl:   The sourceUrl for this deployment, used when calling start deployment without create deployment. SourceUrl can be any HTTP GET url that is public accessible and downloads a single zip. <br/>jobId:   The job id for this deployment, generated by create deployment request. <br/>branchName:   Name for the branch, for the Job. <br/>appId:   Unique Id for an Amplify App. <br/>
        :return: jobSummary:   Summary for the Job. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jobType</i>  Type for the Job. \n "RELEASE": Manually released from source by using StartJob API. "RETRY": Manually retried by using StartJob API. "WEB_HOOK": Automatically triggered by WebHooks. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>endTime</i>  End date / time for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i>  Status for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>startTime</i>  Start date / time for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>commitTime</i>  Commit date / time for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>commitMessage</i>  Commit message from 3rd party repository provider for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>commitId</i>  Commit Id from 3rd party repository provider for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jobId</i>  Unique Id for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jobArn</i>  Arn for the Job. <br/>        """
        pass

    def start_job(self, jobType: str, branchName: str, appId: str, jobId: str=None, jobReason: str=None, commitId: str=None, commitMessage: str=None, commitTime: CommitTime=None) -> StartJobResult:
        """ Starts a new job for a branch, part of an Amplify App. 

        :param self:
        :param commitTime:   Commit date / time for the Job. <br/>commitMessage:   Commit message from 3rd party repository provider for the Job. <br/>commitId:   Commit Id from 3rd party repository provider for the Job. <br/>jobReason:   Descriptive reason for starting this job. <br/>jobType:   Type for the Job. Available JobTypes are: \n "RELEASE": Start a new job with the latest change from the specified branch. Only available for apps that have connected to a repository. "RETRY": Retry an existing job. JobId is required for this type of job. <br/>jobId:   Unique Id for an existing job. Required for "RETRY" JobType. <br/>branchName:   Name for the branch, for the Job. <br/>appId:   Unique Id for an Amplify App. <br/>
        :return: jobSummary:   Summary for the Job. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jobType</i>  Type for the Job. \n "RELEASE": Manually released from source by using StartJob API. "RETRY": Manually retried by using StartJob API. "WEB_HOOK": Automatically triggered by WebHooks. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>endTime</i>  End date / time for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i>  Status for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>startTime</i>  Start date / time for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>commitTime</i>  Commit date / time for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>commitMessage</i>  Commit message from 3rd party repository provider for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>commitId</i>  Commit Id from 3rd party repository provider for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jobId</i>  Unique Id for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jobArn</i>  Arn for the Job. <br/>        """
        pass

    def stop_job(self, jobId: str, branchName: str, appId: str) -> StopJobResult:
        """ Stop a job that is in progress, for an Amplify branch, part of Amplify App. 

        :param self:
        :param jobId:   Unique Id for the Job. <br/>branchName:   Name for the branch, for the Job. <br/>appId:   Unique Id for an Amplify App. <br/>
        :return: jobSummary:   Summary for the Job. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jobType</i>  Type for the Job. \n "RELEASE": Manually released from source by using StartJob API. "RETRY": Manually retried by using StartJob API. "WEB_HOOK": Automatically triggered by WebHooks. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>endTime</i>  End date / time for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i>  Status for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>startTime</i>  Start date / time for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>commitTime</i>  Commit date / time for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>commitMessage</i>  Commit message from 3rd party repository provider for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>commitId</i>  Commit Id from 3rd party repository provider for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jobId</i>  Unique Id for the Job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jobArn</i>  Arn for the Job. <br/>        """
        pass

    def tag_resource(self, tags: TagMap, resourceArn: str) -> TagResourceResponse:
        """ Tag resource with tag key and value. 

        :param self:
        :param tags:   Tags used to tag resource. <br/>resourceArn:   Resource arn used to tag resource. <br/>
        :return:         """
        pass

    def untag_resource(self, tagKeys: List[TagKey], resourceArn: str) -> UntagResourceResponse:
        """ Untag resource with resourceArn. 

        :param self:
        :param tagKeys:   Tag keys used to untag resource. <br/>resourceArn:   Resource arn used to untag resource. <br/>
        :return:         """
        pass

    def update_app(self, appId: str, name: str=None, description: str=None, platform: str=None, iamServiceRoleArn: str=None, environmentVariables: EnvironmentVariables=None, enableBranchAutoBuild: EnableAutoBuild=None, enableBasicAuth: EnableBasicAuth=None, basicAuthCredentials: str=None, customRules: List[CustomRule]=None, buildSpec: str=None, enableAutoBranchCreation: EnableAutoBranchCreation=None, autoBranchCreationPatterns: List[AutoBranchCreationPattern]=None, autoBranchCreationConfig: AutoBranchCreationConfig=None) -> UpdateAppResult:
        """ Updates an existing Amplify App. 

        :param self:
        :param autoBranchCreationConfig:   Automated branch creation config for the Amplify App. :<br/>autoBranchCreationPatterns:   Automated branch creation glob patterns for the Amplify App. <br/>enableAutoBranchCreation:   Enables automated branch creation for the Amplify App. <br/>buildSpec:   BuildSpec for an Amplify App. <br/>customRules:   Custom redirect / rewrite rules for an Amplify App. <br/>basicAuthCredentials:   Basic Authorization credentials for an Amplify App. <br/>enableBasicAuth:   Enables Basic Authorization for an Amplify App. <br/>enableBranchAutoBuild:   Enables branch auto-building for an Amplify App. <br/>environmentVariables:   Environment Variables for an Amplify App. <br/>iamServiceRoleArn:   IAM service role for an Amplify App. <br/>platform:   Platform for an Amplify App. <br/>description:   Description for an Amplify App. <br/>name:   Name for an Amplify App. <br/>appId:   Unique Id for an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>buildSpec</i>  BuildSpec for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableBasicAuth</i>  Enables Basic Auth for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>basicAuthCredentials</i>  Basic Authorization credentials for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>environmentVariables</i>  Environment Variables for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableAutoBuild</i>  Enables auto building for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>framework</i>  Framework for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stage</i>  Stage for the auto created branch. <br/>
        :return: app:   App structure for the updated App. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>autoBranchCreationConfig</i>  Automated branch creation config for the Amplify App. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>autoBranchCreationConfig</b>:  Automated branch creation config for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>autoBranchCreationPatterns</i>  Automated branch creation glob patterns for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableAutoBranchCreation</i>  Enables automated branch creation for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>buildSpec</i>  BuildSpec content for Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>productionBranch</i>  Structure with Production Branch information. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>productionBranch</b>:  Structure with Production Branch information. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customRules</i>  Custom redirect / rewrite rules for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>basicAuthCredentials</i>  Basic Authorization credentials for branches for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableBasicAuth</i>  Enables Basic Authorization for branches for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableBranchAutoBuild</i>  Enables auto-building of branches for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultDomain</i>  Default domain for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>environmentVariables</i>  Environment Variables for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>iamServiceRoleArn</i>  IAM service role ARN for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>updateTime</i>  Update date / time for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createTime</i>  Create date / time for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>platform</i>  Platform for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>repository</i>  Repository for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i>  Description for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i>  Tag for Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i>  Name for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>appArn</i>  ARN for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>appId</i>  Unique Id for the Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>branchName</i>  Branch Name for Production Branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>thumbnailUrl</i>  Thumbnail URL for Production Branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i>  Status of Production Branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastDeployTime</i>  Last Deploy Time of Production Branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>buildSpec</i>  BuildSpec for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>enableBasicAuth</i>  Enables Basic Auth for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>basicAuthCredentials</i>  Basic Authorization credentials for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>environmentVariables</i>  Environment Variables for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>enableAutoBuild</i>  Enables auto building for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>framework</i>  Framework for the auto created branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>stage</i>  Stage for the auto created branch. <br/>        """
        pass

    def update_branch(self, branchName: str, appId: str, description: str=None, framework: str=None, stage: str=None, enableNotification: EnableNotification=None, enableAutoBuild: EnableAutoBuild=None, environmentVariables: EnvironmentVariables=None, basicAuthCredentials: str=None, enableBasicAuth: EnableBasicAuth=None, buildSpec: str=None, ttl: str=None, displayName: str=None) -> UpdateBranchResult:
        """ Updates a branch for an Amplify App. 

        :param self:
        :param displayName:   Display name for a branch, will use as the default domain prefix. <br/>ttl:   The content TTL for the website in seconds. <br/>buildSpec:   BuildSpec for the branch. <br/>enableBasicAuth:   Enables Basic Auth for the branch. <br/>basicAuthCredentials:   Basic Authorization credentials for the branch. <br/>environmentVariables:   Environment Variables for the branch. <br/>enableAutoBuild:   Enables auto building for the branch. <br/>enableNotification:   Enables notifications for the branch. <br/>stage:   Stage for the branch. <br/>framework:   Framework for the branch. <br/>description:   Description for the branch. <br/>branchName:   Name for the branch. <br/>appId:   Unique Id for an Amplify App. <br/>
        :return: branch:   Branch structure for an Amplify App. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>associatedResources</i>  List of custom resources that are linked to this branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ttl</i>  The content TTL for the website in seconds. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>buildSpec</i>  BuildSpec content for branch for Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>basicAuthCredentials</i>  Basic Authorization credentials for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>thumbnailUrl</i>  Thumbnail URL for the branch. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableBasicAuth</i>  Enables Basic Authorization for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>totalNumberOfJobs</i>  Total number of Jobs part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>activeJobId</i>  Id of the active job for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>framework</i>  Framework for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customDomains</i>  Custom domains for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableAutoBuild</i>  Enables auto-building on push for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>environmentVariables</i>  Environment Variables specific to a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>updateTime</i>  Last updated date and time for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createTime</i>  Creation date and time for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableNotification</i>  Enables notifications for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>displayName</i>  Display name for a branch, will use as the default domain prefix. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stage</i>  Stage for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i>  Tag for branch for Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i>  Description for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>branchName</i>  Name for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>branchArn</i>  ARN for a branch, part of an Amplify App. <br/>        """
        pass

    def update_domain_association(self, subDomainSettings: List[SubDomainSetting], domainName: str, appId: str, enableAutoSubDomain: EnableAutoSubDomain=None) -> UpdateDomainAssociationResult:
        """ Create a new DomainAssociation on an App 

        :param self:
        :param subDomainSettings:   Setting structure for the Subdomain. <br/>enableAutoSubDomain:   Enables automated creation of Subdomains for branches. <br/>domainName:   Name of the domain. <br/>appId:   Unique Id for an Amplify App. <br/>
        :return: domainAssociation:   Domain Association structure. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>subDomains</i>  Subdomains for the Domain Association. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>certificateVerificationDNSRecord</i>  DNS Record for certificate verification. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusReason</i>  Reason for the current status of the Domain Association. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>domainStatus</i>  Status fo the Domain Association. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableAutoSubDomain</i>  Enables automated creation of Subdomains for branches. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>domainName</i>  Name of the domain. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>domainAssociationArn</i>  ARN for the Domain Association. <br/>        """
        pass

    def update_webhook(self, webhookId: str, branchName: str=None, description: str=None) -> UpdateWebhookResult:
        """ Update a webhook. 

        :param self:
        :param description:   Description for a webhook. <br/>branchName:   Name for a branch, part of an Amplify App. <br/>webhookId:   Unique Id for a webhook. <br/>
        :return: webhook:   Webhook structure. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>updateTime</i>  Update date / time for a webhook. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createTime</i>  Create date / time for a webhook. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i>  Description for a webhook. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>branchName</i>  Name for a branch, part of an Amplify App. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>webhookUrl</i>  Url of the webhook. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>webhookId</i>  Id of the webhook. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>webhookArn</i>  ARN for the webhook. <br/>        """
        pass



class APIGateway(BaseClient):
    """<fullname>Amazon API Gateway</fullname> Amazon API Gateway helps developers deliver robust, secure, and scalable mobile and web application back ends. API Gateway allows developers to securely connect mobile and web applications to APIs that run on AWS Lambda, Amazon EC2, or other publicly addressable web services that are hosted outside of AWS."""


    class GetApiKeysPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_api_keys` for valid parameters"""
            pass
    class GetBasePathMappingsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_base_path_mappings` for valid parameters"""
            pass
    class GetClientCertificatesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_client_certificates` for valid parameters"""
            pass
    class GetDeploymentsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_deployments` for valid parameters"""
            pass
    class GetDomainNamesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_domain_names` for valid parameters"""
            pass
    class GetModelsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_models` for valid parameters"""
            pass
    class GetResourcesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_resources` for valid parameters"""
            pass
    class GetRestApisPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_rest_apis` for valid parameters"""
            pass
    class GetUsagePaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_usage` for valid parameters"""
            pass
    class GetUsagePlansPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_usage_plans` for valid parameters"""
            pass
    class GetUsagePlanKeysPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_usage_plan_keys` for valid parameters"""
            pass
    class GetVpcLinksPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_vpc_links` for valid parameters"""
            pass
    class GetAuthorizersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_authorizers` for valid parameters"""
            pass
    class GetDocumentationPartsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_documentation_parts` for valid parameters"""
            pass
    class GetDocumentationVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_documentation_versions` for valid parameters"""
            pass
    class GetGatewayResponsesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_gateway_responses` for valid parameters"""
            pass
    class GetRequestValidatorsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_request_validators` for valid parameters"""
            pass
    class GetSdkTypesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_sdk_types` for valid parameters"""
            pass


    class CreateApiKeyRequest(Mapping):
        pass
    
    class ApiKey(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class StageKey(Mapping):
        pass
    
    class MapOfStringToString(object):
        pass
    
    class CreateAuthorizerRequest(Mapping):
        pass
    
    class Authorizer(Mapping):
        pass
    
    class ProviderARN(object):
        pass
    
    class CreateBasePathMappingRequest(Mapping):
        pass
    
    class BasePathMapping(Mapping):
        pass
    
    class CreateDeploymentRequest(Mapping):
        pass
    
    class Deployment(Mapping):
        pass
    
    class NullableBoolean(object):
        pass
    
    class DeploymentCanarySettings(Mapping):
        pass
    
    class CreateDocumentationPartRequest(Mapping):
        pass
    
    class DocumentationPart(Mapping):
        pass
    
    class DocumentationPartLocation(Mapping):
        pass
    
    class CreateDocumentationVersionRequest(Mapping):
        pass
    
    class DocumentationVersion(Mapping):
        pass
    
    class CreateDomainNameRequest(Mapping):
        pass
    
    class DomainName(Mapping):
        pass
    
    class EndpointConfiguration(Mapping):
        pass
    
    class CreateModelRequest(Mapping):
        pass
    
    class Model(Mapping):
        pass
    
    class CreateRequestValidatorRequest(Mapping):
        pass
    
    class RequestValidator(Mapping):
        pass
    
    class CreateResourceRequest(Mapping):
        pass
    
    class Resource(Mapping):
        pass
    
    class CreateRestApiRequest(Mapping):
        pass
    
    class RestApi(Mapping):
        pass
    
    class String(object):
        pass
    
    class CreateStageRequest(Mapping):
        pass
    
    class Stage(Mapping):
        pass
    
    class CanarySettings(Mapping):
        pass
    
    class CreateUsagePlanRequest(Mapping):
        pass
    
    class UsagePlan(Mapping):
        pass
    
    class ApiStage(Mapping):
        pass
    
    class ThrottleSettings(Mapping):
        pass
    
    class QuotaSettings(Mapping):
        pass
    
    class CreateUsagePlanKeyRequest(Mapping):
        pass
    
    class UsagePlanKey(Mapping):
        pass
    
    class CreateVpcLinkRequest(Mapping):
        pass
    
    class VpcLink(Mapping):
        pass
    
    class DeleteApiKeyRequest(Mapping):
        pass
    
    class DeleteAuthorizerRequest(Mapping):
        pass
    
    class DeleteBasePathMappingRequest(Mapping):
        pass
    
    class DeleteClientCertificateRequest(Mapping):
        pass
    
    class DeleteDeploymentRequest(Mapping):
        pass
    
    class DeleteDocumentationPartRequest(Mapping):
        pass
    
    class DeleteDocumentationVersionRequest(Mapping):
        pass
    
    class DeleteDomainNameRequest(Mapping):
        pass
    
    class DeleteGatewayResponseRequest(Mapping):
        pass
    
    class DeleteIntegrationRequest(Mapping):
        pass
    
    class DeleteIntegrationResponseRequest(Mapping):
        pass
    
    class DeleteMethodRequest(Mapping):
        pass
    
    class DeleteMethodResponseRequest(Mapping):
        pass
    
    class DeleteModelRequest(Mapping):
        pass
    
    class DeleteRequestValidatorRequest(Mapping):
        pass
    
    class DeleteResourceRequest(Mapping):
        pass
    
    class DeleteRestApiRequest(Mapping):
        pass
    
    class DeleteStageRequest(Mapping):
        pass
    
    class DeleteUsagePlanRequest(Mapping):
        pass
    
    class DeleteUsagePlanKeyRequest(Mapping):
        pass
    
    class DeleteVpcLinkRequest(Mapping):
        pass
    
    class FlushStageAuthorizersCacheRequest(Mapping):
        pass
    
    class FlushStageCacheRequest(Mapping):
        pass
    
    class GenerateClientCertificateRequest(Mapping):
        pass
    
    class ClientCertificate(Mapping):
        pass
    
    class GetAccountRequest(Mapping):
        pass
    
    class Account(Mapping):
        pass
    
    class GetApiKeyRequest(Mapping):
        pass
    
    class GetApiKeysRequest(Mapping):
        pass
    
    class ApiKeys(Mapping):
        pass
    
    class GetAuthorizerRequest(Mapping):
        pass
    
    class GetAuthorizersRequest(Mapping):
        pass
    
    class Authorizers(Mapping):
        pass
    
    class GetBasePathMappingRequest(Mapping):
        pass
    
    class GetBasePathMappingsRequest(Mapping):
        pass
    
    class BasePathMappings(Mapping):
        pass
    
    class GetClientCertificateRequest(Mapping):
        pass
    
    class GetClientCertificatesRequest(Mapping):
        pass
    
    class ClientCertificates(Mapping):
        pass
    
    class GetDeploymentRequest(Mapping):
        pass
    
    class GetDeploymentsRequest(Mapping):
        pass
    
    class Deployments(Mapping):
        pass
    
    class GetDocumentationPartRequest(Mapping):
        pass
    
    class GetDocumentationPartsRequest(Mapping):
        pass
    
    class DocumentationParts(Mapping):
        pass
    
    class GetDocumentationVersionRequest(Mapping):
        pass
    
    class GetDocumentationVersionsRequest(Mapping):
        pass
    
    class DocumentationVersions(Mapping):
        pass
    
    class GetDomainNameRequest(Mapping):
        pass
    
    class GetDomainNamesRequest(Mapping):
        pass
    
    class DomainNames(Mapping):
        pass
    
    class GetExportRequest(Mapping):
        pass
    
    class ExportResponse(Mapping):
        pass
    
    class GetGatewayResponseRequest(Mapping):
        pass
    
    class GatewayResponse(Mapping):
        pass
    
    class GetGatewayResponsesRequest(Mapping):
        pass
    
    class GatewayResponses(Mapping):
        pass
    
    class GetIntegrationRequest(Mapping):
        pass
    
    class Integration(Mapping):
        pass
    
    class GetIntegrationResponseRequest(Mapping):
        pass
    
    class IntegrationResponse(Mapping):
        pass
    
    class GetMethodRequest(Mapping):
        pass
    
    class Method(Mapping):
        pass
    
    class GetMethodResponseRequest(Mapping):
        pass
    
    class MethodResponse(Mapping):
        pass
    
    class GetModelRequest(Mapping):
        pass
    
    class GetModelTemplateRequest(Mapping):
        pass
    
    class Template(Mapping):
        pass
    
    class GetModelsRequest(Mapping):
        pass
    
    class Models(Mapping):
        pass
    
    class GetRequestValidatorRequest(Mapping):
        pass
    
    class GetRequestValidatorsRequest(Mapping):
        pass
    
    class RequestValidators(Mapping):
        pass
    
    class GetResourceRequest(Mapping):
        pass
    
    class GetResourcesRequest(Mapping):
        pass
    
    class Resources(Mapping):
        pass
    
    class GetRestApiRequest(Mapping):
        pass
    
    class GetRestApisRequest(Mapping):
        pass
    
    class RestApis(Mapping):
        pass
    
    class GetSdkRequest(Mapping):
        pass
    
    class SdkResponse(Mapping):
        pass
    
    class GetSdkTypeRequest(Mapping):
        pass
    
    class SdkType(Mapping):
        pass
    
    class GetSdkTypesRequest(Mapping):
        pass
    
    class SdkTypes(Mapping):
        pass
    
    class GetStageRequest(Mapping):
        pass
    
    class GetStagesRequest(Mapping):
        pass
    
    class Stages(Mapping):
        pass
    
    class GetTagsRequest(Mapping):
        pass
    
    class Tags(Mapping):
        pass
    
    class GetUsageRequest(Mapping):
        pass
    
    class Usage(Mapping):
        pass
    
    class GetUsagePlanRequest(Mapping):
        pass
    
    class GetUsagePlanKeyRequest(Mapping):
        pass
    
    class GetUsagePlanKeysRequest(Mapping):
        pass
    
    class UsagePlanKeys(Mapping):
        pass
    
    class GetUsagePlansRequest(Mapping):
        pass
    
    class UsagePlans(Mapping):
        pass
    
    class GetVpcLinkRequest(Mapping):
        pass
    
    class GetVpcLinksRequest(Mapping):
        pass
    
    class VpcLinks(Mapping):
        pass
    
    class ImportApiKeysRequest(Mapping):
        pass
    
    class ApiKeyIds(Mapping):
        pass
    
    class Blob(object):
        pass
    
    class ImportDocumentationPartsRequest(Mapping):
        pass
    
    class DocumentationPartIds(Mapping):
        pass
    
    class ImportRestApiRequest(Mapping):
        pass
    
    class PutGatewayResponseRequest(Mapping):
        pass
    
    class PutIntegrationRequest(Mapping):
        pass
    
    class PutIntegrationResponseRequest(Mapping):
        pass
    
    class PutMethodRequest(Mapping):
        pass
    
    class MapOfStringToBoolean(object):
        pass
    
    class PutMethodResponseRequest(Mapping):
        pass
    
    class PutRestApiRequest(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TestInvokeAuthorizerRequest(Mapping):
        pass
    
    class TestInvokeAuthorizerResponse(Mapping):
        pass
    
    class MapOfStringToList(object):
        pass
    
    class TestInvokeMethodRequest(Mapping):
        pass
    
    class TestInvokeMethodResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UpdateAccountRequest(Mapping):
        pass
    
    class PatchOperation(Mapping):
        pass
    
    class UpdateApiKeyRequest(Mapping):
        pass
    
    class UpdateAuthorizerRequest(Mapping):
        pass
    
    class UpdateBasePathMappingRequest(Mapping):
        pass
    
    class UpdateClientCertificateRequest(Mapping):
        pass
    
    class UpdateDeploymentRequest(Mapping):
        pass
    
    class UpdateDocumentationPartRequest(Mapping):
        pass
    
    class UpdateDocumentationVersionRequest(Mapping):
        pass
    
    class UpdateDomainNameRequest(Mapping):
        pass
    
    class UpdateGatewayResponseRequest(Mapping):
        pass
    
    class UpdateIntegrationRequest(Mapping):
        pass
    
    class UpdateIntegrationResponseRequest(Mapping):
        pass
    
    class UpdateMethodRequest(Mapping):
        pass
    
    class UpdateMethodResponseRequest(Mapping):
        pass
    
    class UpdateModelRequest(Mapping):
        pass
    
    class UpdateRequestValidatorRequest(Mapping):
        pass
    
    class UpdateResourceRequest(Mapping):
        pass
    
    class UpdateRestApiRequest(Mapping):
        pass
    
    class UpdateStageRequest(Mapping):
        pass
    
    class UpdateUsageRequest(Mapping):
        pass
    
    class UpdateUsagePlanRequest(Mapping):
        pass
    
    class UpdateVpcLinkRequest(Mapping):
        pass
    
    def create_api_key(self, name: str=None, description: str=None, enabled: Boolean=None, generateDistinctId: Boolean=None, value: str=None, stageKeys: List[StageKey]=None, customerId: str=None, tags: MapOfStringToString=None) -> ApiKey:
        """Create an <a>ApiKey</a> resource.  <div class="seeAlso"><a href="https://docs.aws.amazon.com/cli/latest/reference/apigateway/create-api-key.html">AWS CLI</a></div>

        :param self:
        :param tags:  The key-value map of strings. The valid character set is [a-zA-Z+-=._:/]. The tag key can be up to 128 characters and must not start with <code>aws:</code>. The tag value can be up to 256 characters.<br/>customerId:  An AWS Marketplace customer identifier , when integrating with the AWS SaaS Marketplace.<br/>stageKeys:  DEPRECATED FOR USAGE PLANS - Specifies stages associated with the API key.<br/>value:  Specifies a value of the API key.<br/>generateDistinctId:  Specifies whether (<code>true</code>) or not (<code>false</code>) the key identifier is distinct from the created API key value.<br/>enabled:  Specifies whether the <a>ApiKey</a> can be used by callers.<br/>description:  The description of the <a>ApiKey</a>.<br/>name:  The name of the <a>ApiKey</a>.<br/>
        :return: tags:  The collection of tags. Each tag element is associated with a given resource.<br/>stageKeys:  A list of <a>Stage</a> resources that are associated with the <a>ApiKey</a> resource.<br/>lastUpdatedDate:  The timestamp when the API Key was last updated.<br/>createdDate:  The timestamp when the API Key was created.<br/>enabled:  Specifies whether the API Key can be used by callers.<br/>description:  The description of the API Key.<br/>customerId:  An AWS Marketplace customer identifier , when integrating with the AWS SaaS Marketplace.<br/>name:  The name of the API Key.<br/>value:  The value of the API Key.<br/>id:  The identifier of the API Key.<br/>        """
        pass

    def create_authorizer(self, type: str, name: str, restApiId: str, providerARNs: List[ProviderARN]=None, authType: str=None, authorizerUri: str=None, authorizerCredentials: str=None, identitySource: str=None, identityValidationExpression: str=None, authorizerResultTtlInSeconds: int=None) -> Authorizer:
        """Adds a new <a>Authorizer</a> resource to an existing <a>RestApi</a> resource. <div class="seeAlso"><a href="https://docs.aws.amazon.com/cli/latest/reference/apigateway/create-authorizer.html">AWS CLI</a></div>

        :param self:
        :param authorizerResultTtlInSeconds:  The TTL in seconds of cached authorizer results. If it equals 0, authorization caching is disabled. If it is greater than 0, API Gateway will cache authorizer responses. If this field is not set, the default value is 300. The maximum value is 3600, or 1 hour.<br/>identityValidationExpression:  A validation expression for the incoming identity token. For <code>TOKEN</code> authorizers, this value is a regular expression. For <code>COGNITO_USER_POOLS</code> authorizers, API Gateway will match the <code>aud</code> field of the incoming token from the client against the specified regular expression. It will invoke the authorizer's Lambda function when there is a match. Otherwise, it will return a 401 Unauthorized response without calling the Lambda function. The validation expression does not apply to the <code>REQUEST</code> authorizer.<br/>identitySource:  The identity source for which authorization is requested. <ul><li>For a <code>TOKEN</code> or <code>COGNITO_USER_POOLS</code> authorizer, this is required and specifies the request header mapping expression for the custom header holding the authorization token submitted by the client. For example, if the token header name is <code>Auth</code>, the header mapping expression is <code>method.request.header.Auth</code>.</li><li>For the <code>REQUEST</code> authorizer, this is required when authorization caching is enabled. The value is a comma-separated string of one or more mapping expressions of the specified request parameters. For example, if an <code>Auth</code> header, a <code>Name</code> query string parameter are defined as identity sources, this value is <code>method.request.header.Auth, method.request.querystring.Name</code>. These parameters will be used to derive the authorization caching key and to perform runtime validation of the <code>REQUEST</code> authorizer by verifying all of the identity-related request parameters are present, not null and non-empty. Only when this is true does the authorizer invoke the authorizer Lambda function, otherwise, it returns a 401 Unauthorized response without calling the Lambda function. The valid value is a string of comma-separated mapping expressions of the specified request parameters. When the authorization caching is not enabled, this property is optional.</li></ul><br/>authorizerCredentials:  Specifies the required credentials as an IAM role for API Gateway to invoke the authorizer. To specify an IAM role for API Gateway to assume, use the role's Amazon Resource Name (ARN). To use resource-based permissions on the Lambda function, specify null.<br/>authorizerUri:  Specifies the authorizer's Uniform Resource Identifier (URI). For <code>TOKEN</code> or <code>REQUEST</code> authorizers, this must be a well-formed Lambda function URI, for example, <code>arn:aws:apigateway:us-west-2:lambda:path/2015-03-31/functions/arn:aws:lambda:us-west-2:{account_id}:function:{lambda_function_name}/invocations</code>. In general, the URI has this form <code>arn:aws:apigateway:{region}:lambda:path/{service_api}</code>, where <code>{region}</code> is the same as the region hosting the Lambda function, <code>path</code> indicates that the remaining substring in the URI should be treated as the path to the resource, including the initial <code>/</code>. For Lambda functions, this is usually of the form <code>/2015-03-31/functions/[FunctionARN]/invocations</code>.<br/>authType:  Optional customer-defined field, used in OpenAPI imports and exports without functional impact.<br/>providerARNs:  A list of the Amazon Cognito user pool ARNs for the <code>COGNITO_USER_POOLS</code> authorizer. Each element is of this format: <code>arn:aws:cognito-idp:{region}:{account_id}:userpool/{user_pool_id}</code>. For a <code>TOKEN</code> or <code>REQUEST</code> authorizer, this is not defined. <br/>type:  [Required] The authorizer type. Valid values are <code>TOKEN</code> for a Lambda function using a single authorization token submitted in a custom header, <code>REQUEST</code> for a Lambda function using incoming request parameters, and <code>COGNITO_USER_POOLS</code> for using an Amazon Cognito user pool.<br/>name:  [Required] The name of the authorizer.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: authorizerResultTtlInSeconds:  The TTL in seconds of cached authorizer results. If it equals 0, authorization caching is disabled. If it is greater than 0, API Gateway will cache authorizer responses. If this field is not set, the default value is 300. The maximum value is 3600, or 1 hour.<br/>identityValidationExpression:  A validation expression for the incoming identity token. For <code>TOKEN</code> authorizers, this value is a regular expression. For <code>COGNITO_USER_POOLS</code> authorizers, API Gateway will match the <code>aud</code> field of the incoming token from the client against the specified regular expression. It will invoke the authorizer's Lambda function when there is a match. Otherwise, it will return a 401 Unauthorized response without calling the Lambda function. The validation expression does not apply to the <code>REQUEST</code> authorizer.<br/>identitySource:  The identity source for which authorization is requested. <ul><li>For a <code>TOKEN</code> or <code>COGNITO_USER_POOLS</code> authorizer, this is required and specifies the request header mapping expression for the custom header holding the authorization token submitted by the client. For example, if the token header name is <code>Auth</code>, the header mapping expression is <code>method.request.header.Auth</code>.</li><li>For the <code>REQUEST</code> authorizer, this is required when authorization caching is enabled. The value is a comma-separated string of one or more mapping expressions of the specified request parameters. For example, if an <code>Auth</code> header, a <code>Name</code> query string parameter are defined as identity sources, this value is <code>method.request.header.Auth, method.request.querystring.Name</code>. These parameters will be used to derive the authorization caching key and to perform runtime validation of the <code>REQUEST</code> authorizer by verifying all of the identity-related request parameters are present, not null and non-empty. Only when this is true does the authorizer invoke the authorizer Lambda function, otherwise, it returns a 401 Unauthorized response without calling the Lambda function. The valid value is a string of comma-separated mapping expressions of the specified request parameters. When the authorization caching is not enabled, this property is optional.</li></ul><br/>authorizerCredentials:  Specifies the required credentials as an IAM role for API Gateway to invoke the authorizer. To specify an IAM role for API Gateway to assume, use the role's Amazon Resource Name (ARN). To use resource-based permissions on the Lambda function, specify null.<br/>authorizerUri:  Specifies the authorizer's Uniform Resource Identifier (URI). For <code>TOKEN</code> or <code>REQUEST</code> authorizers, this must be a well-formed Lambda function URI, for example, <code>arn:aws:apigateway:us-west-2:lambda:path/2015-03-31/functions/arn:aws:lambda:us-west-2:{account_id}:function:{lambda_function_name}/invocations</code>. In general, the URI has this form <code>arn:aws:apigateway:{region}:lambda:path/{service_api}</code>, where <code>{region}</code> is the same as the region hosting the Lambda function, <code>path</code> indicates that the remaining substring in the URI should be treated as the path to the resource, including the initial <code>/</code>. For Lambda functions, this is usually of the form <code>/2015-03-31/functions/[FunctionARN]/invocations</code>.<br/>authType:  Optional customer-defined field, used in OpenAPI imports and exports without functional impact.<br/>providerARNs:  A list of the Amazon Cognito user pool ARNs for the <code>COGNITO_USER_POOLS</code> authorizer. Each element is of this format: <code>arn:aws:cognito-idp:{region}:{account_id}:userpool/{user_pool_id}</code>. For a <code>TOKEN</code> or <code>REQUEST</code> authorizer, this is not defined. <br/>type:  The authorizer type. Valid values are <code>TOKEN</code> for a Lambda function using a single authorization token submitted in a custom header, <code>REQUEST</code> for a Lambda function using incoming request parameters, and <code>COGNITO_USER_POOLS</code> for using an Amazon Cognito user pool.<br/>name:  [Required] The name of the authorizer.<br/>id:  The identifier for the authorizer resource.<br/>        """
        pass

    def create_base_path_mapping(self, restApiId: str, domainName: str, basePath: str=None, stage: str=None) -> BasePathMapping:
        """Creates a new <a>BasePathMapping</a> resource.

        :param self:
        :param stage:  The name of the API's stage that you want to use for this mapping. Specify '(none)' if you do not want callers to explicitly specify the stage name after any base path name.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>basePath:  The base path name that callers of the API must provide as part of the URL after the domain name. This value must be unique for all of the mappings across a single API. Specify '(none)' if you do not want callers to specify a base path name after the domain name.<br/>domainName:  [Required] The domain name of the <a>BasePathMapping</a> resource to create.<br/>
        :return: stage:  The name of the associated stage.<br/>restApiId:  The string identifier of the associated <a>RestApi</a>.<br/>basePath:  The base path name that callers of the API must provide as part of the URL after the domain name.<br/>        """
        pass

    def create_deployment(self, restApiId: str, stageName: str=None, stageDescription: str=None, description: str=None, cacheClusterEnabled: NullableBoolean=None, cacheClusterSize: str=None, variables: MapOfStringToString=None, canarySettings: DeploymentCanarySettings=None, tracingEnabled: NullableBoolean=None) -> Deployment:
        """Creates a <a>Deployment</a> resource, which makes a specified <a>RestApi</a> callable over the internet.

        :param self:
        :param tracingEnabled:  Specifies whether active tracing with X-ray is enabled for the <a>Stage</a>.<br/>canarySettings:  The input configuration for the canary deployment when the deployment is a canary release deployment. :<br/>variables:  A map that defines the stage variables for the <a>Stage</a> resource that is associated with the new deployment. Variable names can have alphanumeric and underscore characters, and the values must match <code>[A-Za-z0-9-._~:/?#&amp;=,]+</code>.<br/>cacheClusterSize:  Specifies the cache cluster size for the <a>Stage</a> resource specified in the input, if a cache cluster is enabled.<br/>cacheClusterEnabled:  Enables a cache cluster for the <a>Stage</a> resource specified in the input.<br/>description:  The description for the <a>Deployment</a> resource to create.<br/>stageDescription:  The description of the <a>Stage</a> resource for the <a>Deployment</a> resource to create.<br/>stageName:  The name of the <a>Stage</a> resource for the <a>Deployment</a> resource to create.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>useStageCache</i> A Boolean flag to indicate whether the canary release deployment uses the stage cache or not.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stageVariableOverrides</i> A stage variable overrides used for the canary release deployment. They can override existing stage variables or add new stage variables for the canary release deployment. These stage variables are represented as a string-to-string map between stage variable names and their values.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>percentTraffic</i> The percentage (0.0-100.0) of traffic routed to the canary deployment.<br/>
        :return: apiSummary:  A summary of the <a>RestApi</a> at the date and time that the deployment resource was created.<br/>createdDate:  The date and time that the deployment resource was created.<br/>description:  The description for the deployment resource.<br/>id:  The identifier for the deployment resource.<br/>        """
        pass

    def create_documentation_part(self, properties: str, location: DocumentationPartLocation, restApiId: str) -> DocumentationPart:
        """

        :param self:
        :param properties:  [Required] The new documentation content map of the targeted API entity. Enclosed key-value pairs are API-specific, but only OpenAPI-compliant key-value pairs can be exported and, hence, published.<br/>location:  [Required] The location of the targeted API entity of the to-be-created documentation part.:<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the targeted API entity. It is a valid and required field for the API entity types of <code>AUTHORIZER</code>, <code>MODEL</code>, <code>PATH_PARAMETER</code>, <code>QUERY_PARAMETER</code>, <code>REQUEST_HEADER</code>, <code>REQUEST_BODY</code> and <code>RESPONSE_HEADER</code>. It is an invalid field for any other entity type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusCode</i> The HTTP status code of a response. It is a valid field for the API entity types of <code>RESPONSE</code>, <code>RESPONSE_HEADER</code>, and <code>RESPONSE_BODY</code>. The default value is <code>*</code> for any status code. When an applicable child entity inherits the content of an entity of the same type with more general specifications of the other <code>location</code> attributes, the child entity's <code>statusCode</code> attribute must match that of the parent entity exactly.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>method</i> The HTTP verb of a method. It is a valid field for the API entity types of <code>METHOD</code>, <code>PATH_PARAMETER</code>, <code>QUERY_PARAMETER</code>, <code>REQUEST_HEADER</code>, <code>REQUEST_BODY</code>, <code>RESPONSE</code>, <code>RESPONSE_HEADER</code>, and <code>RESPONSE_BODY</code>. The default value is <code>*</code> for any method. When an applicable child entity inherits the content of an entity of the same type with more general specifications of the other <code>location</code> attributes, the child entity's <code>method</code> attribute must match that of the parent entity exactly.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>path</i> The URL path of the target. It is a valid field for the API entity types of <code>RESOURCE</code>, <code>METHOD</code>, <code>PATH_PARAMETER</code>, <code>QUERY_PARAMETER</code>, <code>REQUEST_HEADER</code>, <code>REQUEST_BODY</code>, <code>RESPONSE</code>, <code>RESPONSE_HEADER</code>, and <code>RESPONSE_BODY</code>. The default value is <code>/</code> for the root resource. When an applicable child entity inherits the content of another entity of the same type with more general specifications of the other <code>location</code> attributes, the child entity's <code>path</code> attribute must match that of the parent entity as a prefix.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> [Required] The type of API entity to which the documentation content applies. Valid values are <code>API</code>, <code>AUTHORIZER</code>, <code>MODEL</code>, <code>RESOURCE</code>, <code>METHOD</code>, <code>PATH_PARAMETER</code>, <code>QUERY_PARAMETER</code>, <code>REQUEST_HEADER</code>, <code>REQUEST_BODY</code>, <code>RESPONSE</code>, <code>RESPONSE_HEADER</code>, and <code>RESPONSE_BODY</code>. Content inheritance does not apply to any entity of the <code>API</code>, <code>AUTHORIZER</code>, <code>METHOD</code>, <code>MODEL</code>, <code>REQUEST_BODY</code>, or <code>RESOURCE</code> type.<br/>
        :return: properties:  A content map of API-specific key-value pairs describing the targeted API entity. The map must be encoded as a JSON string, e.g., <code>"{ \"description\": \"The API does ...\" }"</code>. Only OpenAPI-compliant documentation-related fields from the <literal>properties</literal> map are exported and, hence, published as part of the API entity definitions, while the original documentation parts are exported in a OpenAPI extension of <code>x-amazon-apigateway-documentation</code>.<br/>location:  The location of the API entity to which the documentation applies. Valid fields depend on the targeted API entity type. All the valid location fields are not required. If not explicitly specified, a valid location field is treated as a wildcard and associated documentation content may be inherited by matching entities, unless overridden.:<br/>id:  The <a>DocumentationPart</a> identifier, generated by API Gateway when the <code>DocumentationPart</code> is created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the targeted API entity. It is a valid and required field for the API entity types of <code>AUTHORIZER</code>, <code>MODEL</code>, <code>PATH_PARAMETER</code>, <code>QUERY_PARAMETER</code>, <code>REQUEST_HEADER</code>, <code>REQUEST_BODY</code> and <code>RESPONSE_HEADER</code>. It is an invalid field for any other entity type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusCode</i> The HTTP status code of a response. It is a valid field for the API entity types of <code>RESPONSE</code>, <code>RESPONSE_HEADER</code>, and <code>RESPONSE_BODY</code>. The default value is <code>*</code> for any status code. When an applicable child entity inherits the content of an entity of the same type with more general specifications of the other <code>location</code> attributes, the child entity's <code>statusCode</code> attribute must match that of the parent entity exactly.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>method</i> The HTTP verb of a method. It is a valid field for the API entity types of <code>METHOD</code>, <code>PATH_PARAMETER</code>, <code>QUERY_PARAMETER</code>, <code>REQUEST_HEADER</code>, <code>REQUEST_BODY</code>, <code>RESPONSE</code>, <code>RESPONSE_HEADER</code>, and <code>RESPONSE_BODY</code>. The default value is <code>*</code> for any method. When an applicable child entity inherits the content of an entity of the same type with more general specifications of the other <code>location</code> attributes, the child entity's <code>method</code> attribute must match that of the parent entity exactly.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>path</i> The URL path of the target. It is a valid field for the API entity types of <code>RESOURCE</code>, <code>METHOD</code>, <code>PATH_PARAMETER</code>, <code>QUERY_PARAMETER</code>, <code>REQUEST_HEADER</code>, <code>REQUEST_BODY</code>, <code>RESPONSE</code>, <code>RESPONSE_HEADER</code>, and <code>RESPONSE_BODY</code>. The default value is <code>/</code> for the root resource. When an applicable child entity inherits the content of another entity of the same type with more general specifications of the other <code>location</code> attributes, the child entity's <code>path</code> attribute must match that of the parent entity as a prefix.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> [Required] The type of API entity to which the documentation content applies. Valid values are <code>API</code>, <code>AUTHORIZER</code>, <code>MODEL</code>, <code>RESOURCE</code>, <code>METHOD</code>, <code>PATH_PARAMETER</code>, <code>QUERY_PARAMETER</code>, <code>REQUEST_HEADER</code>, <code>REQUEST_BODY</code>, <code>RESPONSE</code>, <code>RESPONSE_HEADER</code>, and <code>RESPONSE_BODY</code>. Content inheritance does not apply to any entity of the <code>API</code>, <code>AUTHORIZER</code>, <code>METHOD</code>, <code>MODEL</code>, <code>REQUEST_BODY</code>, or <code>RESOURCE</code> type.<br/>        """
        pass

    def create_documentation_version(self, documentationVersion: str, restApiId: str, stageName: str=None, description: str=None) -> DocumentationVersion:
        """

        :param self:
        :param description:  A description about the new documentation snapshot.<br/>stageName:  The stage name to be associated with the new documentation snapshot.<br/>documentationVersion:  [Required] The version identifier of the new snapshot.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: description:  The description of the API documentation snapshot.<br/>createdDate:  The date when the API documentation snapshot is created.<br/>version:  The version identifier of the API documentation snapshot.<br/>        """
        pass

    def create_domain_name(self, domainName: str, certificateName: str=None, certificateBody: str=None, certificatePrivateKey: str=None, certificateChain: str=None, certificateArn: str=None, regionalCertificateName: str=None, regionalCertificateArn: str=None, endpointConfiguration: EndpointConfiguration=None, tags: MapOfStringToString=None, securityPolicy: str=None) -> DomainName:
        """Creates a new domain name.

        :param self:
        :param securityPolicy:  The Transport Layer Security (TLS) version + cipher suite for this <a>DomainName</a>. The valid values are <code>TLS_1_0</code> and <code>TLS_1_2</code>.<br/>tags:  The key-value map of strings. The valid character set is [a-zA-Z+-=._:/]. The tag key can be up to 128 characters and must not start with <code>aws:</code>. The tag value can be up to 256 characters.<br/>endpointConfiguration:  The endpoint configuration of this <a>DomainName</a> showing the endpoint types of the domain name. :<br/>regionalCertificateArn:  The reference to an AWS-managed certificate that will be used by regional endpoint for this domain name. AWS Certificate Manager is the only supported source.<br/>regionalCertificateName:  The user-friendly name of the certificate that will be used by regional endpoint for this domain name.<br/>certificateArn:  The reference to an AWS-managed certificate that will be used by edge-optimized endpoint for this domain name. AWS Certificate Manager is the only supported source.<br/>certificateChain:  [Deprecated] The intermediate certificates and optionally the root certificate, one after the other without any blank lines, used by an edge-optimized endpoint for this domain name. If you include the root certificate, your certificate chain must start with intermediate certificates and end with the root certificate. Use the intermediate certificates that were provided by your certificate authority. Do not include any intermediaries that are not in the chain of trust path.<br/>certificatePrivateKey:  [Deprecated] Your edge-optimized endpoint's domain name certificate's private key.<br/>certificateBody:  [Deprecated] The body of the server certificate that will be used by edge-optimized endpoint for this domain name provided by your certificate authority.<br/>certificateName:  The user-friendly name of the certificate that will be used by edge-optimized endpoint for this domain name.<br/>domainName:  [Required] The name of the <a>DomainName</a> resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>types</i> A list of endpoint types of an API (<a>RestApi</a>) or its custom domain name (<a>DomainName</a>). For an edge-optimized API and its custom domain name, the endpoint type is <code>"EDGE"</code>. For a regional API and its custom domain name, the endpoint type is <code>REGIONAL</code>. For a private API, the endpoint type is <code>PRIVATE</code>.<br/>
        :return: tags:  The collection of tags. Each tag element is associated with a given resource.<br/>securityPolicy:  The Transport Layer Security (TLS) version + cipher suite for this <a>DomainName</a>. The valid values are <code>TLS_1_0</code> and <code>TLS_1_2</code>.<br/>domainNameStatusMessage:  An optional text message containing detailed information about status of the <a>DomainName</a> migration.<br/>domainNameStatus:  The status of the <a>DomainName</a> migration. The valid values are <code>AVAILABLE</code> and <code>UPDATING</code>. If the status is <code>UPDATING</code>, the domain cannot be modified further until the existing operation is complete. If it is <code>AVAILABLE</code>, the domain can be updated.<br/>endpointConfiguration:  The endpoint configuration of this <a>DomainName</a> showing the endpoint types of the domain name. :<br/>distributionHostedZoneId:  The region-agnostic Amazon Route 53 Hosted Zone ID of the edge-optimized endpoint. The valid value is <code>Z2FDTNDATAQYW2</code> for all the regions. For more information, see <a href="https://docs.aws.amazon.com/apigateway/latest/developerguide/apigateway-regional-api-custom-domain-create.html">Set up a Regional Custom Domain Name</a> and <a href="https://docs.aws.amazon.com/general/latest/gr/rande.html#apigateway_region">AWS Regions and Endpoints for API Gateway</a>. <br/>distributionDomainName:  The domain name of the Amazon CloudFront distribution associated with this custom domain name for an edge-optimized endpoint. You set up this association when adding a DNS record pointing the custom domain name to this distribution name. For more information about CloudFront distributions, see the <a href="https://aws.amazon.com/documentation/cloudfront/" target="_blank">Amazon CloudFront documentation</a>.<br/>regionalCertificateArn:  The reference to an AWS-managed certificate that will be used for validating the regional domain name. AWS Certificate Manager is the only supported source.<br/>regionalCertificateName:  The name of the certificate that will be used for validating the regional domain name.<br/>regionalHostedZoneId:  The region-specific Amazon Route 53 Hosted Zone ID of the regional endpoint. For more information, see <a href="https://docs.aws.amazon.com/apigateway/latest/developerguide/apigateway-regional-api-custom-domain-create.html">Set up a Regional Custom Domain Name</a> and <a href="https://docs.aws.amazon.com/general/latest/gr/rande.html#apigateway_region">AWS Regions and Endpoints for API Gateway</a>. <br/>regionalDomainName:  The domain name associated with the regional endpoint for this custom domain name. You set up this association by adding a DNS record that points the custom domain name to this regional domain name. The regional domain name is returned by API Gateway when you create a regional endpoint.<br/>certificateUploadDate:  The timestamp when the certificate that was used by edge-optimized endpoint for this domain name was uploaded.<br/>certificateArn:  The reference to an AWS-managed certificate that will be used by edge-optimized endpoint for this domain name. AWS Certificate Manager is the only supported source.<br/>certificateName:  The name of the certificate that will be used by edge-optimized endpoint for this domain name.<br/>domainName:  The custom domain name as an API host name, for example, <code>my-api.example.com</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>types</i> A list of endpoint types of an API (<a>RestApi</a>) or its custom domain name (<a>DomainName</a>). For an edge-optimized API and its custom domain name, the endpoint type is <code>"EDGE"</code>. For a regional API and its custom domain name, the endpoint type is <code>REGIONAL</code>. For a private API, the endpoint type is <code>PRIVATE</code>.<br/>        """
        pass

    def create_model(self, contentType: str, name: str, restApiId: str, description: str=None, schema: str=None) -> Model:
        """Adds a new <a>Model</a> resource to an existing <a>RestApi</a> resource.

        :param self:
        :param contentType:  [Required] The content-type for the model.<br/>schema:  The schema for the model. For <code>application/json</code> models, this should be <a href="https://tools.ietf.org/html/draft-zyp-json-schema-04" target="_blank">JSON schema draft 4</a> model.<br/>description:  The description of the model.<br/>name:  [Required] The name of the model. Must be alphanumeric.<br/>restApiId:  [Required] The <a>RestApi</a> identifier under which the <a>Model</a> will be created.<br/>
        :return: contentType:  The content-type for the model.<br/>schema:  The schema for the model. For <code>application/json</code> models, this should be <a href="https://tools.ietf.org/html/draft-zyp-json-schema-04" target="_blank">JSON schema draft 4</a> model. Do not include "\*/" characters in the description of any properties because such "\*/" characters may be interpreted as the closing marker for comments in some languages, such as Java or JavaScript, causing the installation of your API's SDK generated by API Gateway to fail.<br/>description:  The description of the model.<br/>name:  The name of the model. Must be an alphanumeric string.<br/>id:  The identifier for the model resource.<br/>        """
        pass

    def create_request_validator(self, restApiId: str, name: str=None, validateRequestBody: Boolean=None, validateRequestParameters: Boolean=None) -> RequestValidator:
        """Creates a <a>ReqeustValidator</a> of a given <a>RestApi</a>.

        :param self:
        :param validateRequestParameters:  A Boolean flag to indicate whether to validate request parameters, <code>true</code>, or not <code>false</code>.<br/>validateRequestBody:  A Boolean flag to indicate whether to validate request body according to the configured model schema for the method (<code>true</code>) or not (<code>false</code>).<br/>name:  The name of the to-be-created <a>RequestValidator</a>.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: validateRequestParameters:  A Boolean flag to indicate whether to validate request parameters (<code>true</code>) or not (<code>false</code>).<br/>validateRequestBody:  A Boolean flag to indicate whether to validate a request body according to the configured <a>Model</a> schema.<br/>name:  The name of this <a>RequestValidator</a><br/>id:  The identifier of this <a>RequestValidator</a>.<br/>        """
        pass

    def create_resource(self, pathPart: str, parentId: str, restApiId: str) -> Resource:
        """Creates a <a>Resource</a> resource.

        :param self:
        :param pathPart:  The last path segment for this resource.<br/>parentId:  [Required] The parent resource's identifier.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: resourceMethods:  Gets an API resource's method of a given HTTP verb. <div class="remarks"> The resource methods are a map of methods indexed by methods' HTTP verbs enabled on the resource. This method map is included in the <code>200 OK</code> response of the <code>GET /restapis/{restapi_id}/resources/{resource_id}</code> or <code>GET /restapis/{restapi_id}/resources/{resource_id}?embed=methods</code> request. <h4>Example: Get the GET method of an API resource</h4> <h5>Request</h5> <pre><code>GET /restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET HTTP/1.1 Content-Type: application/json Host: apigateway.us-east-1.amazonaws.com X-Amz-Date: 20170223T031827Z Authorization: AWS4-HMAC-SHA256 Credential={access_key_ID}/20170223/us-east-1/apigateway/aws4_request, SignedHeaders=content-type;host;x-amz-date, Signature={sig4_hash}</code></pre> <h5>Response</h5> <pre><code>{ "_links": { "curies": [ { "href": "https://docs.aws.amazon.com/apigateway/latest/developerguide/restapi-integration-{rel}.html", "name": "integration", "templated": true }, { "href": "https://docs.aws.amazon.com/apigateway/latest/developerguide/restapi-integration-response-{rel}.html", "name": "integrationresponse", "templated": true }, { "href": "https://docs.aws.amazon.com/apigateway/latest/developerguide/restapi-method-{rel}.html", "name": "method", "templated": true }, { "href": "https://docs.aws.amazon.com/apigateway/latest/developerguide/restapi-method-response-{rel}.html", "name": "methodresponse", "templated": true } ], "self": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET", "name": "GET", "title": "GET" }, "integration:put": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration" }, "method:delete": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET" }, "method:integration": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration" }, "method:responses": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/responses/200", "name": "200", "title": "200" }, "method:update": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET" }, "methodresponse:put": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/responses/{status_code}", "templated": true } }, "apiKeyRequired": false, "authorizationType": "NONE", "httpMethod": "GET", "_embedded": { "method:integration": { "_links": { "self": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration" }, "integration:delete": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration" }, "integration:responses": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200", "name": "200", "title": "200" }, "integration:update": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration" }, "integrationresponse:put": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/{status_code}", "templated": true } }, "cacheKeyParameters": [], "cacheNamespace": "3kzxbg5sa2", "credentials": "arn:aws:iam::123456789012:role/apigAwsProxyRole", "httpMethod": "POST", "passthroughBehavior": "WHEN_NO_MATCH", "requestParameters": { "integration.request.header.Content-Type": "'application/x-amz-json-1.1'" }, "requestTemplates": { "application/json": "{\n}" }, "type": "AWS", "uri": "arn:aws:apigateway:us-east-1:kinesis:action/ListStreams", "_embedded": { "integration:responses": { "_links": { "self": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200", "name": "200", "title": "200" }, "integrationresponse:delete": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200" }, "integrationresponse:update": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200" } }, "responseParameters": { "method.response.header.Content-Type": "'application/xml'" }, "responseTemplates": { "application/json": "$util.urlDecode(\"%3CkinesisStreams%3E#foreach($stream in $input.path('$.StreamNames'))%3Cstream%3E%3Cname%3E$stream%3C/name%3E%3C/stream%3E#end%3C/kinesisStreams%3E\")\n" }, "statusCode": "200" } } }, "method:responses": { "_links": { "self": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/responses/200", "name": "200", "title": "200" }, "methodresponse:delete": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/responses/200" }, "methodresponse:update": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/responses/200" } }, "responseModels": { "application/json": "Empty" }, "responseParameters": { "method.response.header.Content-Type": false }, "statusCode": "200" } } }</code></pre> If the <code>OPTIONS</code> is enabled on the resource, you can follow the example here to get that method. Just replace the <code>GET</code> of the last path segment in the request URL with <code>OPTIONS</code>. </div> <div class="seeAlso"> </div><br/>path:  The full path for this resource.<br/>pathPart:  The last path segment for this resource.<br/>parentId:  The parent resource's identifier.<br/>id:  The resource's identifier.<br/>        """
        pass

    def create_rest_api(self, name: str, description: str=None, version: str=None, cloneFrom: str=None, binaryMediaTypes: List[String]=None, minimumCompressionSize: int=None, apiKeySource: str=None, endpointConfiguration: EndpointConfiguration=None, policy: str=None, tags: MapOfStringToString=None) -> RestApi:
        """Creates a new <a>RestApi</a> resource.

        :param self:
        :param tags:  The key-value map of strings. The valid character set is [a-zA-Z+-=._:/]. The tag key can be up to 128 characters and must not start with <code>aws:</code>. The tag value can be up to 256 characters.<br/>policy:  A stringified JSON policy document that applies to this RestApi regardless of the caller and <a>Method</a> configuration.<br/>endpointConfiguration:  The endpoint configuration of this <a>RestApi</a> showing the endpoint types of the API. :<br/>apiKeySource:  The source of the API key for metering requests according to a usage plan. Valid values are: <ul><li><code>HEADER</code> to read the API key from the <code>X-API-Key</code> header of a request. </li><li><code>AUTHORIZER</code> to read the API key from the <code>UsageIdentifierKey</code> from a custom authorizer.</li></ul> <br/>minimumCompressionSize:  A nullable integer that is used to enable compression (with non-negative between 0 and 10485760 (10M) bytes, inclusive) or disable compression (with a null value) on an API. When compression is enabled, compression or decompression is not applied on the payload if the payload size is smaller than this value. Setting it to zero allows compression for any payload size.<br/>binaryMediaTypes:  The list of binary media types supported by the <a>RestApi</a>. By default, the <a>RestApi</a> supports only UTF-8-encoded text payloads.<br/>cloneFrom:  The ID of the <a>RestApi</a> that you want to clone from.<br/>version:  A version identifier for the API.<br/>description:  The description of the <a>RestApi</a>.<br/>name:  [Required] The name of the <a>RestApi</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>types</i> A list of endpoint types of an API (<a>RestApi</a>) or its custom domain name (<a>DomainName</a>). For an edge-optimized API and its custom domain name, the endpoint type is <code>"EDGE"</code>. For a regional API and its custom domain name, the endpoint type is <code>REGIONAL</code>. For a private API, the endpoint type is <code>PRIVATE</code>.<br/>
        :return: tags:  The collection of tags. Each tag element is associated with a given resource.<br/>policy:  A stringified JSON policy document that applies to this RestApi regardless of the caller and <a>Method</a> configuration.<br/>endpointConfiguration:  The endpoint configuration of this <a>RestApi</a> showing the endpoint types of the API. :<br/>apiKeySource:  The source of the API key for metering requests according to a usage plan. Valid values are: <ul><li><code>HEADER</code> to read the API key from the <code>X-API-Key</code> header of a request. </li><li><code>AUTHORIZER</code> to read the API key from the <code>UsageIdentifierKey</code> from a custom authorizer.</li></ul> <br/>minimumCompressionSize:  A nullable integer that is used to enable compression (with non-negative between 0 and 10485760 (10M) bytes, inclusive) or disable compression (with a null value) on an API. When compression is enabled, compression or decompression is not applied on the payload if the payload size is smaller than this value. Setting it to zero allows compression for any payload size.<br/>binaryMediaTypes:  The list of binary media types supported by the <a>RestApi</a>. By default, the <a>RestApi</a> supports only UTF-8-encoded text payloads.<br/>warnings:  The warning messages reported when <code>failonwarnings</code> is turned on during API import.<br/>version:  A version identifier for the API.<br/>createdDate:  The timestamp when the API was created.<br/>description:  The API's description.<br/>name:  The API's name.<br/>id:  The API's identifier. This identifier is unique across all of your APIs in API Gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>types</i> A list of endpoint types of an API (<a>RestApi</a>) or its custom domain name (<a>DomainName</a>). For an edge-optimized API and its custom domain name, the endpoint type is <code>"EDGE"</code>. For a regional API and its custom domain name, the endpoint type is <code>REGIONAL</code>. For a private API, the endpoint type is <code>PRIVATE</code>.<br/>        """
        pass

    def create_stage(self, deploymentId: str, stageName: str, restApiId: str, description: str=None, cacheClusterEnabled: Boolean=None, cacheClusterSize: str=None, variables: MapOfStringToString=None, documentationVersion: str=None, canarySettings: CanarySettings=None, tracingEnabled: Boolean=None, tags: MapOfStringToString=None) -> Stage:
        """Creates a new <a>Stage</a> resource that references a pre-existing <a>Deployment</a> for the API. 

        :param self:
        :param tags:  The key-value map of strings. The valid character set is [a-zA-Z+-=._:/]. The tag key can be up to 128 characters and must not start with <code>aws:</code>. The tag value can be up to 256 characters.<br/>tracingEnabled:  Specifies whether active tracing with X-ray is enabled for the <a>Stage</a>.<br/>canarySettings:  The canary deployment settings of this stage.:<br/>documentationVersion:  The version of the associated API documentation.<br/>variables:  A map that defines the stage variables for the new <a>Stage</a> resource. Variable names can have alphanumeric and underscore characters, and the values must match <code>[A-Za-z0-9-._~:/?#&amp;=,]+</code>.<br/>cacheClusterSize:  The stage's cache cluster size.<br/>cacheClusterEnabled:  Whether cache clustering is enabled for the stage.<br/>description:  The description of the <a>Stage</a> resource.<br/>deploymentId:  [Required] The identifier of the <a>Deployment</a> resource for the <a>Stage</a> resource.<br/>stageName:  [Required] The name for the <a>Stage</a> resource. Stage names can only contain alphanumeric characters, hyphens, and underscores. Maximum length is 128 characters.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>useStageCache</i> A Boolean flag to indicate whether the canary deployment uses the stage cache or not.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stageVariableOverrides</i> Stage variables overridden for a canary release deployment, including new stage variables introduced in the canary. These stage variables are represented as a string-to-string map between stage variable names and their values.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentId</i> The ID of the canary deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>percentTraffic</i> The percent (0-100) of traffic diverted to a canary deployment.<br/>
        :return: lastUpdatedDate:  The timestamp when the stage last updated.<br/>createdDate:  The timestamp when the stage was created.<br/>tags:  The collection of tags. Each tag element is associated with a given resource.<br/>webAclArn:  The ARN of the WebAcl associated with the <a>Stage</a>.<br/>tracingEnabled:  Specifies whether active tracing with X-ray is enabled for the <a>Stage</a>.<br/>canarySettings:  Settings for the canary deployment in this stage.:<br/>accessLogSettings:  Settings for logging access in this stage.:<br/>documentationVersion:  The version of the associated API documentation.<br/>variables:  A map that defines the stage variables for a <a>Stage</a> resource. Variable names can have alphanumeric and underscore characters, and the values must match <code>[A-Za-z0-9-._~:/?#&amp;=,]+</code>.<br/>methodSettings:  A map that defines the method settings for a <a>Stage</a> resource. Keys (designated as <code>/{method_setting_key</code> below) are method paths defined as <code>{resource_path}/{http_method}</code> for an individual method override, or <code>/\*/\*</code> for overriding all methods in the stage. <br/>cacheClusterStatus:  The status of the cache cluster for the stage, if enabled.<br/>cacheClusterSize:  The size of the cache cluster for the stage, if enabled.<br/>cacheClusterEnabled:  Specifies whether a cache cluster is enabled for the stage.<br/>description:  The stage's description.<br/>stageName:  The name of the stage is the first path segment in the Uniform Resource Identifier (URI) of a call to API Gateway. Stage names can only contain alphanumeric characters, hyphens, and underscores. Maximum length is 128 characters.<br/>clientCertificateId:  The identifier of a client certificate for an API stage.<br/>deploymentId:  The identifier of the <a>Deployment</a> that the stage points to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>destinationArn</i> The ARN of the CloudWatch Logs log group to receive access logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>format</i> A single line format of the access logs of data, as specified by selected <a href="https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-mapping-template-reference.html#context-variable-reference">$context variables</a>. The format must include at least <code>$context.requestId</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>useStageCache</i> A Boolean flag to indicate whether the canary deployment uses the stage cache or not.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stageVariableOverrides</i> Stage variables overridden for a canary release deployment, including new stage variables introduced in the canary. These stage variables are represented as a string-to-string map between stage variable names and their values.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentId</i> The ID of the canary deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>percentTraffic</i> The percent (0-100) of traffic diverted to a canary deployment.<br/>        """
        pass

    def create_usage_plan(self, name: str, description: str=None, apiStages: List[ApiStage]=None, throttle: ThrottleSettings=None, quota: QuotaSettings=None, tags: MapOfStringToString=None) -> UsagePlan:
        """Creates a usage plan with the throttle and quota limits, as well as the associated API stages, specified in the payload. 

        :param self:
        :param tags:  The key-value map of strings. The valid character set is [a-zA-Z+-=._:/]. The tag key can be up to 128 characters and must not start with <code>aws:</code>. The tag value can be up to 256 characters.<br/>quota:  The quota of the usage plan.:<br/>throttle:  The throttling limits of the usage plan.:<br/>apiStages:  The associated API stages of the usage plan.<br/>description:  The description of the usage plan.<br/>name:  [Required] The name of the usage plan.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>rateLimit</i> The API request steady-state rate limit.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>burstLimit</i> The API request burst limit, the maximum rate limit over a time ranging from one to a few seconds, depending upon whether the underlying token bucket is at its full capacity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>period</i> The time period in which the limit applies. Valid values are "DAY", "WEEK" or "MONTH".<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>offset</i> The number of requests subtracted from the given limit in the initial time period.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>limit</i> The maximum number of requests that can be made in a given time period.<br/>
        :return: tags:  The collection of tags. Each tag element is associated with a given resource.<br/>productCode:  The AWS Markeplace product identifier to associate with the usage plan as a SaaS product on AWS Marketplace.<br/>quota:  The maximum number of permitted requests per a given unit time interval.:<br/>throttle:  The request throttle limits of a usage plan.:<br/>apiStages:  The associated API stages of a usage plan.<br/>description:  The description of a usage plan.<br/>name:  The name of a usage plan.<br/>id:  The identifier of a <a>UsagePlan</a> resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>rateLimit</i> The API request steady-state rate limit.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>burstLimit</i> The API request burst limit, the maximum rate limit over a time ranging from one to a few seconds, depending upon whether the underlying token bucket is at its full capacity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>period</i> The time period in which the limit applies. Valid values are "DAY", "WEEK" or "MONTH".<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>offset</i> The number of requests subtracted from the given limit in the initial time period.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>limit</i> The maximum number of requests that can be made in a given time period.<br/>        """
        pass

    def create_usage_plan_key(self, keyType: str, keyId: str, usagePlanId: str) -> UsagePlanKey:
        """Creates a usage plan key for adding an existing API key to a usage plan.

        :param self:
        :param keyType:  [Required] The type of a <a>UsagePlanKey</a> resource for a plan customer.<br/>keyId:  [Required] The identifier of a <a>UsagePlanKey</a> resource for a plan customer.<br/>usagePlanId:  [Required] The Id of the <a>UsagePlan</a> resource representing the usage plan containing the to-be-created <a>UsagePlanKey</a> resource representing a plan customer.<br/>
        :return: name:  The name of a usage plan key.<br/>value:  The value of a usage plan key.<br/>type:  The type of a usage plan key. Currently, the valid key type is <code>API_KEY</code>.<br/>id:  The Id of a usage plan key.<br/>        """
        pass

    def create_vpc_link(self, targetArns: List[String], name: str, description: str=None, tags: MapOfStringToString=None) -> VpcLink:
        """Creates a VPC link, under the caller's account in a selected region, in an asynchronous operation that typically takes 2-4 minutes to complete and become operational. The caller must have permissions to create and update VPC Endpoint services.

        :param self:
        :param tags:  The key-value map of strings. The valid character set is [a-zA-Z+-=._:/]. The tag key can be up to 128 characters and must not start with <code>aws:</code>. The tag value can be up to 256 characters.<br/>targetArns:  [Required] The ARNs of network load balancers of the VPC targeted by the VPC link. The network load balancers must be owned by the same AWS account of the API owner.<br/>description:  The description of the VPC link.<br/>name:  [Required] The name used to label and identify the VPC link.<br/>
        :return: tags:  The collection of tags. Each tag element is associated with a given resource.<br/>statusMessage:  A description about the VPC link status.<br/>status:  The status of the VPC link. The valid values are <code>AVAILABLE</code>, <code>PENDING</code>, <code>DELETING</code>, or <code>FAILED</code>. Deploying an API will wait if the status is <code>PENDING</code> and will fail if the status is <code>DELETING</code>. <br/>targetArns:  The ARNs of network load balancers of the VPC targeted by the VPC link. The network load balancers must be owned by the same AWS account of the API owner.<br/>description:  The description of the VPC link.<br/>name:  The name used to label and identify the VPC link.<br/>id:  The identifier of the <a>VpcLink</a>. It is used in an <a>Integration</a> to reference this <a>VpcLink</a>.<br/>        """
        pass

    def delete_api_key(self, apiKey: str):
        """Deletes the <a>ApiKey</a> resource.

        :param self:
        :param apiKey:  [Required] The identifier of the <a>ApiKey</a> resource to be deleted.<br/>
        :return:         """
        pass

    def delete_authorizer(self, authorizerId: str, restApiId: str):
        """Deletes an existing <a>Authorizer</a> resource. <div class="seeAlso"><a href="https://docs.aws.amazon.com/cli/latest/reference/apigateway/delete-authorizer.html">AWS CLI</a></div>

        :param self:
        :param authorizerId:  [Required] The identifier of the <a>Authorizer</a> resource.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return:         """
        pass

    def delete_base_path_mapping(self, basePath: str, domainName: str):
        """Deletes the <a>BasePathMapping</a> resource.

        :param self:
        :param basePath:  [Required] The base path name of the <a>BasePathMapping</a> resource to delete. To specify an empty base path, set this parameter to <code>'(none)'</code>.<br/>domainName:  [Required] The domain name of the <a>BasePathMapping</a> resource to delete.<br/>
        :return:         """
        pass

    def delete_client_certificate(self, clientCertificateId: str):
        """Deletes the <a>ClientCertificate</a> resource.

        :param self:
        :param clientCertificateId:  [Required] The identifier of the <a>ClientCertificate</a> resource to be deleted.<br/>
        :return:         """
        pass

    def delete_deployment(self, deploymentId: str, restApiId: str):
        """Deletes a <a>Deployment</a> resource. Deleting a deployment will only succeed if there are no <a>Stage</a> resources associated with it.

        :param self:
        :param deploymentId:  [Required] The identifier of the <a>Deployment</a> resource to delete.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return:         """
        pass

    def delete_documentation_part(self, documentationPartId: str, restApiId: str):
        """

        :param self:
        :param documentationPartId:  [Required] The identifier of the to-be-deleted documentation part.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return:         """
        pass

    def delete_documentation_version(self, documentationVersion: str, restApiId: str):
        """

        :param self:
        :param documentationVersion:  [Required] The version identifier of a to-be-deleted documentation snapshot.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return:         """
        pass

    def delete_domain_name(self, domainName: str):
        """Deletes the <a>DomainName</a> resource.

        :param self:
        :param domainName:  [Required] The name of the <a>DomainName</a> resource to be deleted.<br/>
        :return:         """
        pass

    def delete_gateway_response(self, responseType: str, restApiId: str):
        """Clears any customization of a <a>GatewayResponse</a> of a specified response type on the given <a>RestApi</a> and resets it with the default settings.

        :param self:
        :param responseType:  [Required] The response type of the associated <a>GatewayResponse</a>. Valid values are <ul><li>ACCESS_DENIED</li><li>API_CONFIGURATION_ERROR</li><li>AUTHORIZER_FAILURE</li><li> AUTHORIZER_CONFIGURATION_ERROR</li><li>BAD_REQUEST_PARAMETERS</li><li>BAD_REQUEST_BODY</li><li>DEFAULT_4XX</li><li>DEFAULT_5XX</li><li>EXPIRED_TOKEN</li><li>INVALID_SIGNATURE</li><li>INTEGRATION_FAILURE</li><li>INTEGRATION_TIMEOUT</li><li>INVALID_API_KEY</li><li>MISSING_AUTHENTICATION_TOKEN</li><li> QUOTA_EXCEEDED</li><li>REQUEST_TOO_LARGE</li><li>RESOURCE_NOT_FOUND</li><li>THROTTLED</li><li>UNAUTHORIZED</li><li>UNSUPPORTED_MEDIA_TYPE</li></ul> <br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return:         """
        pass

    def delete_integration(self, httpMethod: str, resourceId: str, restApiId: str):
        """Represents a delete integration.

        :param self:
        :param httpMethod:  [Required] Specifies a delete integration request's HTTP method.<br/>resourceId:  [Required] Specifies a delete integration request's resource identifier.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return:         """
        pass

    def delete_integration_response(self, statusCode: str, httpMethod: str, resourceId: str, restApiId: str):
        """Represents a delete integration response.

        :param self:
        :param statusCode:  [Required] Specifies a delete integration response request's status code.<br/>httpMethod:  [Required] Specifies a delete integration response request's HTTP method.<br/>resourceId:  [Required] Specifies a delete integration response request's resource identifier.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return:         """
        pass

    def delete_method(self, httpMethod: str, resourceId: str, restApiId: str):
        """Deletes an existing <a>Method</a> resource.

        :param self:
        :param httpMethod:  [Required] The HTTP verb of the <a>Method</a> resource.<br/>resourceId:  [Required] The <a>Resource</a> identifier for the <a>Method</a> resource.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return:         """
        pass

    def delete_method_response(self, statusCode: str, httpMethod: str, resourceId: str, restApiId: str):
        """Deletes an existing <a>MethodResponse</a> resource.

        :param self:
        :param statusCode:  [Required] The status code identifier for the <a>MethodResponse</a> resource.<br/>httpMethod:  [Required] The HTTP verb of the <a>Method</a> resource.<br/>resourceId:  [Required] The <a>Resource</a> identifier for the <a>MethodResponse</a> resource.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return:         """
        pass

    def delete_model(self, modelName: str, restApiId: str):
        """Deletes a model.

        :param self:
        :param modelName:  [Required] The name of the model to delete.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return:         """
        pass

    def delete_request_validator(self, requestValidatorId: str, restApiId: str):
        """Deletes a <a>RequestValidator</a> of a given <a>RestApi</a>.

        :param self:
        :param requestValidatorId:  [Required] The identifier of the <a>RequestValidator</a> to be deleted.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return:         """
        pass

    def delete_resource(self, resourceId: str, restApiId: str):
        """Deletes a <a>Resource</a> resource.

        :param self:
        :param resourceId:  [Required] The identifier of the <a>Resource</a> resource.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return:         """
        pass

    def delete_rest_api(self, restApiId: str):
        """Deletes the specified API.

        :param self:
        :param restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return:         """
        pass

    def delete_stage(self, stageName: str, restApiId: str):
        """Deletes a <a>Stage</a> resource.

        :param self:
        :param stageName:  [Required] The name of the <a>Stage</a> resource to delete.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return:         """
        pass

    def delete_usage_plan(self, usagePlanId: str):
        """Deletes a usage plan of a given plan Id.

        :param self:
        :param usagePlanId:  [Required] The Id of the to-be-deleted usage plan.<br/>
        :return:         """
        pass

    def delete_usage_plan_key(self, keyId: str, usagePlanId: str):
        """Deletes a usage plan key and remove the underlying API key from the associated usage plan.

        :param self:
        :param keyId:  [Required] The Id of the <a>UsagePlanKey</a> resource to be deleted.<br/>usagePlanId:  [Required] The Id of the <a>UsagePlan</a> resource representing the usage plan containing the to-be-deleted <a>UsagePlanKey</a> resource representing a plan customer.<br/>
        :return:         """
        pass

    def delete_vpc_link(self, vpcLinkId: str):
        """Deletes an existing <a>VpcLink</a> of a specified identifier.

        :param self:
        :param vpcLinkId:  [Required] The identifier of the <a>VpcLink</a>. It is used in an <a>Integration</a> to reference this <a>VpcLink</a>.<br/>
        :return:         """
        pass

    def flush_stage_authorizers_cache(self, stageName: str, restApiId: str):
        """Flushes all authorizer cache entries on a stage.

        :param self:
        :param stageName:  The name of the stage to flush.<br/>restApiId:  The string identifier of the associated <a>RestApi</a>.<br/>
        :return:         """
        pass

    def flush_stage_cache(self, stageName: str, restApiId: str):
        """Flushes a stage's cache.

        :param self:
        :param stageName:  [Required] The name of the stage to flush its cache.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return:         """
        pass

    def generate_client_certificate(self, description: str=None, tags: MapOfStringToString=None) -> ClientCertificate:
        """Generates a <a>ClientCertificate</a> resource.

        :param self:
        :param tags:  The key-value map of strings. The valid character set is [a-zA-Z+-=._:/]. The tag key can be up to 128 characters and must not start with <code>aws:</code>. The tag value can be up to 256 characters.<br/>description:  The description of the <a>ClientCertificate</a>.<br/>
        :return: tags:  The collection of tags. Each tag element is associated with a given resource.<br/>expirationDate:  The timestamp when the client certificate will expire.<br/>createdDate:  The timestamp when the client certificate was created.<br/>pemEncodedCertificate:  The PEM-encoded public key of the client certificate, which can be used to configure certificate authentication in the integration endpoint .<br/>description:  The description of the client certificate.<br/>clientCertificateId:  The identifier of the client certificate.<br/>        """
        pass

    def get_account(self) -> Account:
        """Gets information about the current <a>Account</a> resource.

        :param self:
        :param 
        :return: apiKeyVersion:  The version of the API keys used for the account.<br/>features:  A list of features supported for the account. When usage plans are enabled, the features list will include an entry of <code>"UsagePlans"</code>.<br/>throttleSettings:  Specifies the API request limits configured for the current <a>Account</a>.:<br/>cloudwatchRoleArn:  The ARN of an Amazon CloudWatch role for the current <a>Account</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>rateLimit</i> The API request steady-state rate limit.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>burstLimit</i> The API request burst limit, the maximum rate limit over a time ranging from one to a few seconds, depending upon whether the underlying token bucket is at its full capacity.<br/>        """
        pass

    def get_api_key(self, apiKey: str, includeValue: NullableBoolean=None) -> ApiKey:
        """Gets information about the current <a>ApiKey</a> resource.

        :param self:
        :param includeValue:  A boolean flag to specify whether (<code>true</code>) or not (<code>false</code>) the result contains the key value.<br/>apiKey:  [Required] The identifier of the <a>ApiKey</a> resource.<br/>
        :return: tags:  The collection of tags. Each tag element is associated with a given resource.<br/>stageKeys:  A list of <a>Stage</a> resources that are associated with the <a>ApiKey</a> resource.<br/>lastUpdatedDate:  The timestamp when the API Key was last updated.<br/>createdDate:  The timestamp when the API Key was created.<br/>enabled:  Specifies whether the API Key can be used by callers.<br/>description:  The description of the API Key.<br/>customerId:  An AWS Marketplace customer identifier , when integrating with the AWS SaaS Marketplace.<br/>name:  The name of the API Key.<br/>value:  The value of the API Key.<br/>id:  The identifier of the API Key.<br/>        """
        pass

    def get_api_keys(self, position: str=None, limit: int=None, nameQuery: str=None, customerId: str=None, includeValues: NullableBoolean=None) -> ApiKeys:
        """Gets information about the current <a>ApiKeys</a> resource.

        :param self:
        :param includeValues:  A boolean flag to specify whether (<code>true</code>) or not (<code>false</code>) the result contains key values.<br/>customerId:  The identifier of a customer in AWS Marketplace or an external system, such as a developer portal.<br/>nameQuery:  The name of queried API keys.<br/>limit:  The maximum number of returned results per page. The default value is 25 and the maximum value is 500.<br/>position:  The current pagination position in the paged result set.<br/>
        :return: items:  The current page of elements from this collection.<br/>position:  <br/>warnings:  A list of warning messages logged during the import of API keys when the <code>failOnWarnings</code> option is set to true.<br/>        """
        pass

    def get_authorizer(self, authorizerId: str, restApiId: str) -> Authorizer:
        """Describe an existing <a>Authorizer</a> resource. <div class="seeAlso"><a href="https://docs.aws.amazon.com/cli/latest/reference/apigateway/get-authorizer.html">AWS CLI</a></div>

        :param self:
        :param authorizerId:  [Required] The identifier of the <a>Authorizer</a> resource.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: authorizerResultTtlInSeconds:  The TTL in seconds of cached authorizer results. If it equals 0, authorization caching is disabled. If it is greater than 0, API Gateway will cache authorizer responses. If this field is not set, the default value is 300. The maximum value is 3600, or 1 hour.<br/>identityValidationExpression:  A validation expression for the incoming identity token. For <code>TOKEN</code> authorizers, this value is a regular expression. For <code>COGNITO_USER_POOLS</code> authorizers, API Gateway will match the <code>aud</code> field of the incoming token from the client against the specified regular expression. It will invoke the authorizer's Lambda function when there is a match. Otherwise, it will return a 401 Unauthorized response without calling the Lambda function. The validation expression does not apply to the <code>REQUEST</code> authorizer.<br/>identitySource:  The identity source for which authorization is requested. <ul><li>For a <code>TOKEN</code> or <code>COGNITO_USER_POOLS</code> authorizer, this is required and specifies the request header mapping expression for the custom header holding the authorization token submitted by the client. For example, if the token header name is <code>Auth</code>, the header mapping expression is <code>method.request.header.Auth</code>.</li><li>For the <code>REQUEST</code> authorizer, this is required when authorization caching is enabled. The value is a comma-separated string of one or more mapping expressions of the specified request parameters. For example, if an <code>Auth</code> header, a <code>Name</code> query string parameter are defined as identity sources, this value is <code>method.request.header.Auth, method.request.querystring.Name</code>. These parameters will be used to derive the authorization caching key and to perform runtime validation of the <code>REQUEST</code> authorizer by verifying all of the identity-related request parameters are present, not null and non-empty. Only when this is true does the authorizer invoke the authorizer Lambda function, otherwise, it returns a 401 Unauthorized response without calling the Lambda function. The valid value is a string of comma-separated mapping expressions of the specified request parameters. When the authorization caching is not enabled, this property is optional.</li></ul><br/>authorizerCredentials:  Specifies the required credentials as an IAM role for API Gateway to invoke the authorizer. To specify an IAM role for API Gateway to assume, use the role's Amazon Resource Name (ARN). To use resource-based permissions on the Lambda function, specify null.<br/>authorizerUri:  Specifies the authorizer's Uniform Resource Identifier (URI). For <code>TOKEN</code> or <code>REQUEST</code> authorizers, this must be a well-formed Lambda function URI, for example, <code>arn:aws:apigateway:us-west-2:lambda:path/2015-03-31/functions/arn:aws:lambda:us-west-2:{account_id}:function:{lambda_function_name}/invocations</code>. In general, the URI has this form <code>arn:aws:apigateway:{region}:lambda:path/{service_api}</code>, where <code>{region}</code> is the same as the region hosting the Lambda function, <code>path</code> indicates that the remaining substring in the URI should be treated as the path to the resource, including the initial <code>/</code>. For Lambda functions, this is usually of the form <code>/2015-03-31/functions/[FunctionARN]/invocations</code>.<br/>authType:  Optional customer-defined field, used in OpenAPI imports and exports without functional impact.<br/>providerARNs:  A list of the Amazon Cognito user pool ARNs for the <code>COGNITO_USER_POOLS</code> authorizer. Each element is of this format: <code>arn:aws:cognito-idp:{region}:{account_id}:userpool/{user_pool_id}</code>. For a <code>TOKEN</code> or <code>REQUEST</code> authorizer, this is not defined. <br/>type:  The authorizer type. Valid values are <code>TOKEN</code> for a Lambda function using a single authorization token submitted in a custom header, <code>REQUEST</code> for a Lambda function using incoming request parameters, and <code>COGNITO_USER_POOLS</code> for using an Amazon Cognito user pool.<br/>name:  [Required] The name of the authorizer.<br/>id:  The identifier for the authorizer resource.<br/>        """
        pass

    def get_authorizers(self, restApiId: str, position: str=None, limit: int=None) -> Authorizers:
        """Describe an existing <a>Authorizers</a> resource. <div class="seeAlso"><a href="https://docs.aws.amazon.com/cli/latest/reference/apigateway/get-authorizers.html">AWS CLI</a></div>

        :param self:
        :param limit:  The maximum number of returned results per page. The default value is 25 and the maximum value is 500.<br/>position:  The current pagination position in the paged result set.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: items:  The current page of elements from this collection.<br/>position:  <br/>        """
        pass

    def get_base_path_mapping(self, basePath: str, domainName: str) -> BasePathMapping:
        """Describe a <a>BasePathMapping</a> resource.

        :param self:
        :param basePath:  [Required] The base path name that callers of the API must provide as part of the URL after the domain name. This value must be unique for all of the mappings across a single API. Specify '(none)' if you do not want callers to specify any base path name after the domain name.<br/>domainName:  [Required] The domain name of the <a>BasePathMapping</a> resource to be described.<br/>
        :return: stage:  The name of the associated stage.<br/>restApiId:  The string identifier of the associated <a>RestApi</a>.<br/>basePath:  The base path name that callers of the API must provide as part of the URL after the domain name.<br/>        """
        pass

    def get_base_path_mappings(self, domainName: str, position: str=None, limit: int=None) -> BasePathMappings:
        """Represents a collection of <a>BasePathMapping</a> resources.

        :param self:
        :param limit:  The maximum number of returned results per page. The default value is 25 and the maximum value is 500.<br/>position:  The current pagination position in the paged result set.<br/>domainName:  [Required] The domain name of a <a>BasePathMapping</a> resource.<br/>
        :return: items:  The current page of elements from this collection.<br/>position:  <br/>        """
        pass

    def get_client_certificate(self, clientCertificateId: str) -> ClientCertificate:
        """Gets information about the current <a>ClientCertificate</a> resource.

        :param self:
        :param clientCertificateId:  [Required] The identifier of the <a>ClientCertificate</a> resource to be described.<br/>
        :return: tags:  The collection of tags. Each tag element is associated with a given resource.<br/>expirationDate:  The timestamp when the client certificate will expire.<br/>createdDate:  The timestamp when the client certificate was created.<br/>pemEncodedCertificate:  The PEM-encoded public key of the client certificate, which can be used to configure certificate authentication in the integration endpoint .<br/>description:  The description of the client certificate.<br/>clientCertificateId:  The identifier of the client certificate.<br/>        """
        pass

    def get_client_certificates(self, position: str=None, limit: int=None) -> ClientCertificates:
        """Gets a collection of <a>ClientCertificate</a> resources.

        :param self:
        :param limit:  The maximum number of returned results per page. The default value is 25 and the maximum value is 500.<br/>position:  The current pagination position in the paged result set.<br/>
        :return: items:  The current page of elements from this collection.<br/>position:  <br/>        """
        pass

    def get_deployment(self, deploymentId: str, restApiId: str, embed: List[String]=None) -> Deployment:
        """Gets information about a <a>Deployment</a> resource.

        :param self:
        :param embed:  A query parameter to retrieve the specified embedded resources of the returned <a>Deployment</a> resource in the response. In a REST API call, this <code>embed</code> parameter value is a list of comma-separated strings, as in <code>GET /restapis/{restapi_id}/deployments/{deployment_id}?embed=var1,var2</code>. The SDK and other platform-dependent libraries might use a different format for the list. Currently, this request supports only retrieval of the embedded API summary this way. Hence, the parameter value must be a single-valued list containing only the <code>"apisummary"</code> string. For example, <code>GET /restapis/{restapi_id}/deployments/{deployment_id}?embed=apisummary</code>.<br/>deploymentId:  [Required] The identifier of the <a>Deployment</a> resource to get information about.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: apiSummary:  A summary of the <a>RestApi</a> at the date and time that the deployment resource was created.<br/>createdDate:  The date and time that the deployment resource was created.<br/>description:  The description for the deployment resource.<br/>id:  The identifier for the deployment resource.<br/>        """
        pass

    def get_deployments(self, restApiId: str, position: str=None, limit: int=None) -> Deployments:
        """Gets information about a <a>Deployments</a> collection.

        :param self:
        :param limit:  The maximum number of returned results per page. The default value is 25 and the maximum value is 500.<br/>position:  The current pagination position in the paged result set.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: items:  The current page of elements from this collection.<br/>position:  <br/>        """
        pass

    def get_documentation_part(self, documentationPartId: str, restApiId: str) -> DocumentationPart:
        """

        :param self:
        :param documentationPartId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: properties:  A content map of API-specific key-value pairs describing the targeted API entity. The map must be encoded as a JSON string, e.g., <code>"{ \"description\": \"The API does ...\" }"</code>. Only OpenAPI-compliant documentation-related fields from the <literal>properties</literal> map are exported and, hence, published as part of the API entity definitions, while the original documentation parts are exported in a OpenAPI extension of <code>x-amazon-apigateway-documentation</code>.<br/>location:  The location of the API entity to which the documentation applies. Valid fields depend on the targeted API entity type. All the valid location fields are not required. If not explicitly specified, a valid location field is treated as a wildcard and associated documentation content may be inherited by matching entities, unless overridden.:<br/>id:  The <a>DocumentationPart</a> identifier, generated by API Gateway when the <code>DocumentationPart</code> is created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the targeted API entity. It is a valid and required field for the API entity types of <code>AUTHORIZER</code>, <code>MODEL</code>, <code>PATH_PARAMETER</code>, <code>QUERY_PARAMETER</code>, <code>REQUEST_HEADER</code>, <code>REQUEST_BODY</code> and <code>RESPONSE_HEADER</code>. It is an invalid field for any other entity type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusCode</i> The HTTP status code of a response. It is a valid field for the API entity types of <code>RESPONSE</code>, <code>RESPONSE_HEADER</code>, and <code>RESPONSE_BODY</code>. The default value is <code>*</code> for any status code. When an applicable child entity inherits the content of an entity of the same type with more general specifications of the other <code>location</code> attributes, the child entity's <code>statusCode</code> attribute must match that of the parent entity exactly.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>method</i> The HTTP verb of a method. It is a valid field for the API entity types of <code>METHOD</code>, <code>PATH_PARAMETER</code>, <code>QUERY_PARAMETER</code>, <code>REQUEST_HEADER</code>, <code>REQUEST_BODY</code>, <code>RESPONSE</code>, <code>RESPONSE_HEADER</code>, and <code>RESPONSE_BODY</code>. The default value is <code>*</code> for any method. When an applicable child entity inherits the content of an entity of the same type with more general specifications of the other <code>location</code> attributes, the child entity's <code>method</code> attribute must match that of the parent entity exactly.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>path</i> The URL path of the target. It is a valid field for the API entity types of <code>RESOURCE</code>, <code>METHOD</code>, <code>PATH_PARAMETER</code>, <code>QUERY_PARAMETER</code>, <code>REQUEST_HEADER</code>, <code>REQUEST_BODY</code>, <code>RESPONSE</code>, <code>RESPONSE_HEADER</code>, and <code>RESPONSE_BODY</code>. The default value is <code>/</code> for the root resource. When an applicable child entity inherits the content of another entity of the same type with more general specifications of the other <code>location</code> attributes, the child entity's <code>path</code> attribute must match that of the parent entity as a prefix.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> [Required] The type of API entity to which the documentation content applies. Valid values are <code>API</code>, <code>AUTHORIZER</code>, <code>MODEL</code>, <code>RESOURCE</code>, <code>METHOD</code>, <code>PATH_PARAMETER</code>, <code>QUERY_PARAMETER</code>, <code>REQUEST_HEADER</code>, <code>REQUEST_BODY</code>, <code>RESPONSE</code>, <code>RESPONSE_HEADER</code>, and <code>RESPONSE_BODY</code>. Content inheritance does not apply to any entity of the <code>API</code>, <code>AUTHORIZER</code>, <code>METHOD</code>, <code>MODEL</code>, <code>REQUEST_BODY</code>, or <code>RESOURCE</code> type.<br/>        """
        pass

    def get_documentation_parts(self, restApiId: str, type: str=None, nameQuery: str=None, path: str=None, position: str=None, limit: int=None, locationStatus: str=None) -> DocumentationParts:
        """

        :param self:
        :param locationStatus:  The status of the API documentation parts to retrieve. Valid values are <code>DOCUMENTED</code> for retrieving <a>DocumentationPart</a> resources with content and <code>UNDOCUMENTED</code> for <a>DocumentationPart</a> resources without content.<br/>limit:  The maximum number of returned results per page. The default value is 25 and the maximum value is 500.<br/>position:  The current pagination position in the paged result set.<br/>path:  The path of API entities of the to-be-retrieved documentation parts.<br/>nameQuery:  The name of API entities of the to-be-retrieved documentation parts.<br/>type:  The type of API entities of the to-be-retrieved documentation parts. <br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: items:  The current page of elements from this collection.<br/>position:  <br/>        """
        pass

    def get_documentation_version(self, documentationVersion: str, restApiId: str) -> DocumentationVersion:
        """

        :param self:
        :param documentationVersion:  [Required] The version identifier of the to-be-retrieved documentation snapshot.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: description:  The description of the API documentation snapshot.<br/>createdDate:  The date when the API documentation snapshot is created.<br/>version:  The version identifier of the API documentation snapshot.<br/>        """
        pass

    def get_documentation_versions(self, restApiId: str, position: str=None, limit: int=None) -> DocumentationVersions:
        """

        :param self:
        :param limit:  The maximum number of returned results per page. The default value is 25 and the maximum value is 500.<br/>position:  The current pagination position in the paged result set.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: items:  The current page of elements from this collection.<br/>position:  <br/>        """
        pass

    def get_domain_name(self, domainName: str) -> DomainName:
        """Represents a domain name that is contained in a simpler, more intuitive URL that can be called.

        :param self:
        :param domainName:  [Required] The name of the <a>DomainName</a> resource.<br/>
        :return: tags:  The collection of tags. Each tag element is associated with a given resource.<br/>securityPolicy:  The Transport Layer Security (TLS) version + cipher suite for this <a>DomainName</a>. The valid values are <code>TLS_1_0</code> and <code>TLS_1_2</code>.<br/>domainNameStatusMessage:  An optional text message containing detailed information about status of the <a>DomainName</a> migration.<br/>domainNameStatus:  The status of the <a>DomainName</a> migration. The valid values are <code>AVAILABLE</code> and <code>UPDATING</code>. If the status is <code>UPDATING</code>, the domain cannot be modified further until the existing operation is complete. If it is <code>AVAILABLE</code>, the domain can be updated.<br/>endpointConfiguration:  The endpoint configuration of this <a>DomainName</a> showing the endpoint types of the domain name. :<br/>distributionHostedZoneId:  The region-agnostic Amazon Route 53 Hosted Zone ID of the edge-optimized endpoint. The valid value is <code>Z2FDTNDATAQYW2</code> for all the regions. For more information, see <a href="https://docs.aws.amazon.com/apigateway/latest/developerguide/apigateway-regional-api-custom-domain-create.html">Set up a Regional Custom Domain Name</a> and <a href="https://docs.aws.amazon.com/general/latest/gr/rande.html#apigateway_region">AWS Regions and Endpoints for API Gateway</a>. <br/>distributionDomainName:  The domain name of the Amazon CloudFront distribution associated with this custom domain name for an edge-optimized endpoint. You set up this association when adding a DNS record pointing the custom domain name to this distribution name. For more information about CloudFront distributions, see the <a href="https://aws.amazon.com/documentation/cloudfront/" target="_blank">Amazon CloudFront documentation</a>.<br/>regionalCertificateArn:  The reference to an AWS-managed certificate that will be used for validating the regional domain name. AWS Certificate Manager is the only supported source.<br/>regionalCertificateName:  The name of the certificate that will be used for validating the regional domain name.<br/>regionalHostedZoneId:  The region-specific Amazon Route 53 Hosted Zone ID of the regional endpoint. For more information, see <a href="https://docs.aws.amazon.com/apigateway/latest/developerguide/apigateway-regional-api-custom-domain-create.html">Set up a Regional Custom Domain Name</a> and <a href="https://docs.aws.amazon.com/general/latest/gr/rande.html#apigateway_region">AWS Regions and Endpoints for API Gateway</a>. <br/>regionalDomainName:  The domain name associated with the regional endpoint for this custom domain name. You set up this association by adding a DNS record that points the custom domain name to this regional domain name. The regional domain name is returned by API Gateway when you create a regional endpoint.<br/>certificateUploadDate:  The timestamp when the certificate that was used by edge-optimized endpoint for this domain name was uploaded.<br/>certificateArn:  The reference to an AWS-managed certificate that will be used by edge-optimized endpoint for this domain name. AWS Certificate Manager is the only supported source.<br/>certificateName:  The name of the certificate that will be used by edge-optimized endpoint for this domain name.<br/>domainName:  The custom domain name as an API host name, for example, <code>my-api.example.com</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>types</i> A list of endpoint types of an API (<a>RestApi</a>) or its custom domain name (<a>DomainName</a>). For an edge-optimized API and its custom domain name, the endpoint type is <code>"EDGE"</code>. For a regional API and its custom domain name, the endpoint type is <code>REGIONAL</code>. For a private API, the endpoint type is <code>PRIVATE</code>.<br/>        """
        pass

    def get_domain_names(self, position: str=None, limit: int=None) -> DomainNames:
        """Represents a collection of <a>DomainName</a> resources.

        :param self:
        :param limit:  The maximum number of returned results per page. The default value is 25 and the maximum value is 500.<br/>position:  The current pagination position in the paged result set.<br/>
        :return: items:  The current page of elements from this collection.<br/>position:  <br/>        """
        pass

    def get_export(self, exportType: str, stageName: str, restApiId: str, parameters: MapOfStringToString=None, accepts: str=None) -> ExportResponse:
        """Exports a deployed version of a <a>RestApi</a> in a specified format.

        :param self:
        :param accepts:  The content-type of the export, for example <code>application/json</code>. Currently <code>application/json</code> and <code>application/yaml</code> are supported for <code>exportType</code> of<code>oas30</code> and <code>swagger</code>. This should be specified in the <code>Accept</code> header for direct API requests.<br/>parameters:  A key-value map of query string parameters that specify properties of the export, depending on the requested <code>exportType</code>. For <code>exportType</code> <code>oas30</code> and <code>swagger</code>, any combination of the following parameters are supported: <code>extensions='integrations'</code> or <code>extensions='apigateway'</code> will export the API with x-amazon-apigateway-integration extensions. <code>extensions='authorizers'</code> will export the API with x-amazon-apigateway-authorizer extensions. <code>postman</code> will export the API with Postman extensions, allowing for import to the Postman tool<br/>exportType:  [Required] The type of export. Acceptable values are 'oas30' for OpenAPI 3.0.x and 'swagger' for Swagger/OpenAPI 2.0.<br/>stageName:  [Required] The name of the <a>Stage</a> that will be exported.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: body:  The binary blob response to <a>GetExport</a>, which contains the export.<br/>contentDisposition:  The content-disposition header value in the HTTP response.<br/>contentType:  The content-type header value in the HTTP response. This will correspond to a valid 'accept' type in the request.<br/>        """
        pass

    def get_gateway_response(self, responseType: str, restApiId: str) -> GatewayResponse:
        """Gets a <a>GatewayResponse</a> of a specified response type on the given <a>RestApi</a>.

        :param self:
        :param responseType:  [Required] The response type of the associated <a>GatewayResponse</a>. Valid values are <ul><li>ACCESS_DENIED</li><li>API_CONFIGURATION_ERROR</li><li>AUTHORIZER_FAILURE</li><li> AUTHORIZER_CONFIGURATION_ERROR</li><li>BAD_REQUEST_PARAMETERS</li><li>BAD_REQUEST_BODY</li><li>DEFAULT_4XX</li><li>DEFAULT_5XX</li><li>EXPIRED_TOKEN</li><li>INVALID_SIGNATURE</li><li>INTEGRATION_FAILURE</li><li>INTEGRATION_TIMEOUT</li><li>INVALID_API_KEY</li><li>MISSING_AUTHENTICATION_TOKEN</li><li> QUOTA_EXCEEDED</li><li>REQUEST_TOO_LARGE</li><li>RESOURCE_NOT_FOUND</li><li>THROTTLED</li><li>UNAUTHORIZED</li><li>UNSUPPORTED_MEDIA_TYPE</li></ul> <br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: defaultResponse:  A Boolean flag to indicate whether this <a>GatewayResponse</a> is the default gateway response (<code>true</code>) or not (<code>false</code>). A default gateway response is one generated by API Gateway without any customization by an API developer. <br/>responseTemplates:  Response templates of the <a>GatewayResponse</a> as a string-to-string map of key-value pairs.<br/>responseParameters:  Response parameters (paths, query strings and headers) of the <a>GatewayResponse</a> as a string-to-string map of key-value pairs.<br/>statusCode:  The HTTP status code for this <a>GatewayResponse</a>.<br/>responseType:  The response type of the associated <a>GatewayResponse</a>. Valid values are <ul><li>ACCESS_DENIED</li><li>API_CONFIGURATION_ERROR</li><li>AUTHORIZER_FAILURE</li><li> AUTHORIZER_CONFIGURATION_ERROR</li><li>BAD_REQUEST_PARAMETERS</li><li>BAD_REQUEST_BODY</li><li>DEFAULT_4XX</li><li>DEFAULT_5XX</li><li>EXPIRED_TOKEN</li><li>INVALID_SIGNATURE</li><li>INTEGRATION_FAILURE</li><li>INTEGRATION_TIMEOUT</li><li>INVALID_API_KEY</li><li>MISSING_AUTHENTICATION_TOKEN</li><li> QUOTA_EXCEEDED</li><li>REQUEST_TOO_LARGE</li><li>RESOURCE_NOT_FOUND</li><li>THROTTLED</li><li>UNAUTHORIZED</li><li>UNSUPPORTED_MEDIA_TYPE</li></ul> <br/>        """
        pass

    def get_gateway_responses(self, restApiId: str, position: str=None, limit: int=None) -> GatewayResponses:
        """Gets the <a>GatewayResponses</a> collection on the given <a>RestApi</a>. If an API developer has not added any definitions for gateway responses, the result will be the API Gateway-generated default <a>GatewayResponses</a> collection for the supported response types.

        :param self:
        :param limit:  The maximum number of returned results per page. The default value is 25 and the maximum value is 500. The <a>GatewayResponses</a> collection does not support pagination and the limit does not apply here.<br/>position:  The current pagination position in the paged result set. The <a>GatewayResponse</a> collection does not support pagination and the position does not apply here.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: items:  Returns the entire collection, because of no pagination support.<br/>position:  <br/>        """
        pass

    def get_integration(self, httpMethod: str, resourceId: str, restApiId: str) -> Integration:
        """Get the integration settings.

        :param self:
        :param httpMethod:  [Required] Specifies a get integration request's HTTP method.<br/>resourceId:  [Required] Specifies a get integration request's resource identifier<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: integrationResponses:  Specifies the integration's responses. <div class="remarks"> <p/> <h4>Example: Get integration responses of a method</h4> <h5>Request</h5> <p/> <pre><code>GET /restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200 HTTP/1.1 Content-Type: application/json Host: apigateway.us-east-1.amazonaws.com X-Amz-Date: 20160607T191449Z Authorization: AWS4-HMAC-SHA256 Credential={access_key_ID}/20160607/us-east-1/apigateway/aws4_request, SignedHeaders=content-type;host;x-amz-date, Signature={sig4_hash} </code></pre> <h5>Response</h5> The successful response returns <code>200 OK</code> status and a payload as follows: <pre><code>{ "_links": { "curies": { "href": "https://docs.aws.amazon.com/apigateway/latest/developerguide/restapi-integration-response-{rel}.html", "name": "integrationresponse", "templated": true }, "self": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200", "title": "200" }, "integrationresponse:delete": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200" }, "integrationresponse:update": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200" } }, "responseParameters": { "method.response.header.Content-Type": "'application/xml'" }, "responseTemplates": { "application/json": "$util.urlDecode(\"%3CkinesisStreams%3E#foreach($stream in $input.path('$.StreamNames'))%3Cstream%3E%3Cname%3E$stream%3C/name%3E%3C/stream%3E#end%3C/kinesisStreams%3E\")\n" }, "statusCode": "200" }</code></pre> <p/> </div> <div class="seeAlso"> <a href="https://docs.aws.amazon.com/apigateway/latest/developerguide/how-to-create-api.html">Creating an API</a> </div><br/>cacheKeyParameters:  A list of request parameters whose values API Gateway caches. To be valid values for <code>cacheKeyParameters</code>, these parameters must also be specified for <a>Method</a> <code>requestParameters</code>.<br/>cacheNamespace:  An API-specific tag group of related cached parameters. To be valid values for <code>cacheKeyParameters</code>, these parameters must also be specified for <a>Method</a> <code>requestParameters</code>.<br/>timeoutInMillis:  Custom timeout between 50 and 29,000 milliseconds. The default value is 29,000 milliseconds or 29 seconds.<br/>contentHandling:  Specifies how to handle request payload content type conversions. Supported values are <code>CONVERT_TO_BINARY</code> and <code>CONVERT_TO_TEXT</code>, with the following behaviors: <ul> <li><code>CONVERT_TO_BINARY</code>: Converts a request payload from a Base64-encoded string to the corresponding binary blob.</li> <li><code>CONVERT_TO_TEXT</code>: Converts a request payload from a binary blob to a Base64-encoded string.</li> </ul> If this property is not defined, the request payload will be passed through from the method request to integration request without modification, provided that the <code>passthroughBehavior</code> is configured to support payload pass-through.<br/>passthroughBehavior:  <div>  Specifies how the method request body of an unmapped content type will be passed through the integration request to the back end without transformation. A content type is unmapped if no mapping template is defined in the integration or the content type does not match any of the mapped content types, as specified in <code>requestTemplates</code>. The valid value is one of the following:  <ul> <li> <code>WHEN_NO_MATCH</code>: passes the method request body through the integration request to the back end without transformation when the method request content type does not match any content type associated with the mapping templates defined in the integration request. </li> <li> <code>WHEN_NO_TEMPLATES</code>: passes the method request body through the integration request to the back end without transformation when no mapping template is defined in the integration request. If a template is defined when this option is selected, the method request of an unmapped content-type will be rejected with an HTTP <code>415 Unsupported Media Type</code> response. </li> <li> <code>NEVER</code>: rejects the method request with an HTTP <code>415 Unsupported Media Type</code> response when either the method request content type does not match any content type associated with the mapping templates defined in the integration request or no mapping template is defined in the integration request. </li> </ul> </div><br/>requestTemplates:  Represents a map of Velocity templates that are applied on the request payload based on the value of the Content-Type header sent by the client. The content type value is the key in this map, and the template (as a String) is the value.<br/>requestParameters:  A key-value map specifying request parameters that are passed from the method request to the back end. The key is an integration request parameter name and the associated value is a method request parameter value or static value that must be enclosed within single quotes and pre-encoded as required by the back end. The method request parameter value must match the pattern of <code>method.request.{location}.{name}</code>, where <code>location</code> is <code>querystring</code>, <code>path</code>, or <code>header</code> and <code>name</code> must be a valid and unique method request parameter name.<br/>credentials:  Specifies the credentials required for the integration, if any. For AWS integrations, three options are available. To specify an IAM Role for API Gateway to assume, use the role's Amazon Resource Name (ARN). To require that the caller's identity be passed through from the request, specify the string <code>arn:aws:iam::\*:user/\*</code>. To use resource-based permissions on supported AWS services, specify null.<br/>connectionId:  The (<a href="https://docs.aws.amazon.com/apigateway/api-reference/resource/vpc-link/#id"><code>id</code></a>) of the <a>VpcLink</a> used for the integration when <code>connectionType=VPC_LINK</code> and undefined, otherwise.<br/>connectionType:  The type of the network connection to the integration endpoint. The valid value is <code>INTERNET</code> for connections through the public routable internet or <code>VPC_LINK</code> for private connections between API Gateway and a network load balancer in a VPC. The default value is <code>INTERNET</code>.<br/>uri:  Specifies Uniform Resource Identifier (URI) of the integration endpoint. <ul> <li> For <code>HTTP</code> or <code>HTTP_PROXY</code> integrations, the URI must be a fully formed, encoded HTTP(S) URL according to the <a target="_blank" href="https://en.wikipedia.org/wiki/Uniform_Resource_Identifier">RFC-3986 specification</a>, for either standard integration, where <code>connectionType</code> is not <code>VPC_LINK</code>, or private integration, where <code>connectionType</code> is <code>VPC_LINK</code>. For a private HTTP integration, the URI is not used for routing.  </li> <li> For <code>AWS</code> or <code>AWS_PROXY</code> integrations, the URI is of the form <code>arn:aws:apigateway:{region}:{subdomain.service|service}:path|action/{service_api}</code>. Here, <code>{Region}</code> is the API Gateway region (e.g., <code>us-east-1</code>); <code>{service}</code> is the name of the integrated AWS service (e.g., <code>s3</code>); and <code>{subdomain}</code> is a designated subdomain supported by certain AWS service for fast host-name lookup. <code>action</code> can be used for an AWS service action-based API, using an <code>Action={name}&amp;{p1}={v1}&amp;p2={v2}...</code> query string. The ensuing <code>{service_api}</code> refers to a supported action <code>{name}</code> plus any required input parameters. Alternatively, <code>path</code> can be used for an AWS service path-based API. The ensuing <code>service_api</code> refers to the path to an AWS service resource, including the region of the integrated AWS service, if applicable. For example, for integration with the S3 API of <code><a href="https://docs.aws.amazon.com/AmazonS3/latest/API/RESTObjectGET.html">GetObject</a></code>, the <code>uri</code> can be either <code>arn:aws:apigateway:us-west-2:s3:action/GetObject&amp;Bucket={bucket}&amp;Key={key}</code> or <code>arn:aws:apigateway:us-west-2:s3:path/{bucket}/{key}</code> </li></ul><br/>httpMethod:  Specifies the integration's HTTP method type.<br/>type:  Specifies an API method integration type. The valid value is one of the following: <ul> <li><code>AWS</code>: for integrating the API method request with an AWS service action, including the Lambda function-invoking action. With the Lambda function-invoking action, this is referred to as the Lambda custom integration. With any other AWS service action, this is known as AWS integration.</li> <li><code>AWS_PROXY</code>: for integrating the API method request with the Lambda function-invoking action with the client request passed through as-is. This integration is also referred to as the Lambda proxy integration.</li> <li><code>HTTP</code>: for integrating the API method request with an HTTP endpoint, including a private HTTP endpoint within a VPC. This integration is also referred to as the HTTP custom integration.</li> <li><code>HTTP_PROXY</code>: for integrating the API method request with an HTTP endpoint, including a private HTTP endpoint within a VPC, with the client request passed through as-is. This is also referred to as the HTTP proxy integration.</li> <li><code>MOCK</code>: for integrating the API method request with API Gateway as a "loop-back" endpoint without invoking any backend.</li> </ul> For the HTTP and HTTP proxy integrations, each integration can specify a protocol (<code>http/https</code>), port and path. Standard 80 and 443 ports are supported as well as custom ports above 1024. An HTTP or HTTP proxy integration with a <code>connectionType</code> of <code>VPC_LINK</code> is referred to as a private integration and uses a <a>VpcLink</a> to connect API Gateway to a network load balancer of a VPC.<br/>        """
        pass

    def get_integration_response(self, statusCode: str, httpMethod: str, resourceId: str, restApiId: str) -> IntegrationResponse:
        """Represents a get integration response.

        :param self:
        :param statusCode:  [Required] Specifies a get integration response request's status code.<br/>httpMethod:  [Required] Specifies a get integration response request's HTTP method.<br/>resourceId:  [Required] Specifies a get integration response request's resource identifier.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: contentHandling:  Specifies how to handle response payload content type conversions. Supported values are <code>CONVERT_TO_BINARY</code> and <code>CONVERT_TO_TEXT</code>, with the following behaviors: <ul> <li><code>CONVERT_TO_BINARY</code>: Converts a response payload from a Base64-encoded string to the corresponding binary blob.</li> <li><code>CONVERT_TO_TEXT</code>: Converts a response payload from a binary blob to a Base64-encoded string.</li> </ul> If this property is not defined, the response payload will be passed through from the integration response to the method response without modification.<br/>responseTemplates:  Specifies the templates used to transform the integration response body. Response templates are represented as a key/value map, with a content-type as the key and a template as the value.<br/>responseParameters:  A key-value map specifying response parameters that are passed to the method response from the back end. The key is a method response header parameter name and the mapped value is an integration response header value, a static value enclosed within a pair of single quotes, or a JSON expression from the integration response body. The mapping key must match the pattern of <code>method.response.header.{name}</code>, where <code>name</code> is a valid and unique header name. The mapped non-static value must match the pattern of <code>integration.response.header.{name}</code> or <code>integration.response.body.{JSON-expression}</code>, where <code>name</code> is a valid and unique response header name and <code>JSON-expression</code> is a valid JSON expression without the <code>$</code> prefix.<br/>selectionPattern:  Specifies the regular expression (regex) pattern used to choose an integration response based on the response from the back end. For example, if the success response returns nothing and the error response returns some string, you could use the <code>.+</code> regex to match error response. However, make sure that the error response does not contain any newline (<code>\n</code>) character in such cases. If the back end is an AWS Lambda function, the AWS Lambda function error header is matched. For all other HTTP and AWS back ends, the HTTP status code is matched.<br/>statusCode:  Specifies the status code that is used to map the integration response to an existing <a>MethodResponse</a>.<br/>        """
        pass

    def get_method(self, httpMethod: str, resourceId: str, restApiId: str) -> Method:
        """Describe an existing <a>Method</a> resource.

        :param self:
        :param httpMethod:  [Required] Specifies the method request's HTTP method type.<br/>resourceId:  [Required] The <a>Resource</a> identifier for the <a>Method</a> resource.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: authorizationScopes:  A list of authorization scopes configured on the method. The scopes are used with a <code>COGNITO_USER_POOLS</code> authorizer to authorize the method invocation. The authorization works by matching the method scopes against the scopes parsed from the access token in the incoming request. The method invocation is authorized if any method scopes matches a claimed scope in the access token. Otherwise, the invocation is not authorized. When the method scope is configured, the client must provide an access token instead of an identity token for authorization purposes.<br/>methodIntegration:  Gets the method's integration responsible for passing the client-submitted request to the back end and performing necessary transformations to make the request compliant with the back end. <div class="remarks"> <p/> <h4>Example: </h4> <h5>Request</h5> <p/> <pre><code>GET /restapis/uojnr9hd57/resources/0cjtch/methods/GET/integration HTTP/1.1 Content-Type: application/json Host: apigateway.us-east-1.amazonaws.com Content-Length: 117 X-Amz-Date: 20160613T213210Z Authorization: AWS4-HMAC-SHA256 Credential={access_key_ID}/20160613/us-east-1/apigateway/aws4_request, SignedHeaders=content-type;host;x-amz-date, Signature={sig4_hash}</code></pre> <h5>Response</h5> The successful response returns a <code>200 OK</code> status code and a payload similar to the following: <pre><code>{ "_links": { "curies": [ { "href": "https://docs.aws.amazon.com/apigateway/latest/developerguide/restapi-integration-{rel}.html", "name": "integration", "templated": true }, { "href": "https://docs.aws.amazon.com/apigateway/latest/developerguide/restapi-integration-response-{rel}.html", "name": "integrationresponse", "templated": true } ], "self": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/integration" }, "integration:delete": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/integration" }, "integration:responses": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/integration/responses/200", "name": "200", "title": "200" }, "integration:update": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/integration" }, "integrationresponse:put": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/integration/responses/{status_code}", "templated": true } }, "cacheKeyParameters": [], "cacheNamespace": "0cjtch", "credentials": "arn:aws:iam::123456789012:role/apigAwsProxyRole", "httpMethod": "POST", "passthroughBehavior": "WHEN_NO_MATCH", "requestTemplates": { "application/json": "{\n \"a\": \"$input.params('operand1')\",\n \"b\": \"$input.params('operand2')\", \n \"op\": \"$input.params('operator')\" \n}" }, "type": "AWS", "uri": "arn:aws:apigateway:us-west-2:lambda:path//2015-03-31/functions/arn:aws:lambda:us-west-2:123456789012:function:Calc/invocations", "_embedded": { "integration:responses": { "_links": { "self": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/integration/responses/200", "name": "200", "title": "200" }, "integrationresponse:delete": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/integration/responses/200" }, "integrationresponse:update": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/integration/responses/200" } }, "responseParameters": { "method.response.header.operator": "integration.response.body.op", "method.response.header.operand_2": "integration.response.body.b", "method.response.header.operand_1": "integration.response.body.a" }, "responseTemplates": { "application/json": "#set($res = $input.path('$'))\n{\n \"result\": \"$res.a, $res.b, $res.op => $res.c\",\n \"a\" : \"$res.a\",\n \"b\" : \"$res.b\",\n \"op\" : \"$res.op\",\n \"c\" : \"$res.c\"\n}" }, "selectionPattern": "", "statusCode": "200" } } }</code></pre> <p/> </div> <div class="seeAlso"> <a href="https://docs.aws.amazon.com/cli/latest/reference/apigateway/get-integration.html">AWS CLI</a> </div>:<br/>methodResponses:  Gets a method response associated with a given HTTP status code.  <div class="remarks"> The collection of method responses are encapsulated in a key-value map, where the key is a response's HTTP status code and the value is a <a>MethodResponse</a> resource that specifies the response returned to the caller from the back end through the integration response. <h4>Example: Get a 200 OK response of a GET method</h4> <h5>Request</h5> <p/> <pre><code>GET /restapis/uojnr9hd57/resources/0cjtch/methods/GET/responses/200 HTTP/1.1 Content-Type: application/json Host: apigateway.us-east-1.amazonaws.com Content-Length: 117 X-Amz-Date: 20160613T215008Z Authorization: AWS4-HMAC-SHA256 Credential={access_key_ID}/20160613/us-east-1/apigateway/aws4_request, SignedHeaders=content-type;host;x-amz-date, Signature={sig4_hash}</code></pre> <h5>Response</h5> The successful response returns a <code>200 OK</code> status code and a payload similar to the following: <pre><code>{ "_links": { "curies": { "href": "https://docs.aws.amazon.com/apigateway/latest/developerguide/restapi-method-response-{rel}.html", "name": "methodresponse", "templated": true }, "self": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/responses/200", "title": "200" }, "methodresponse:delete": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/responses/200" }, "methodresponse:update": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/responses/200" } }, "responseModels": { "application/json": "Empty" }, "responseParameters": { "method.response.header.operator": false, "method.response.header.operand_2": false, "method.response.header.operand_1": false }, "statusCode": "200" }</code></pre> <p/> </div> <div class="seeAlso"> <a href="https://docs.aws.amazon.com/cli/latest/reference/apigateway/get-method-response.html">AWS CLI</a> </div><br/>requestModels:  A key-value map specifying data schemas, represented by <a>Model</a> resources, (as the mapped value) of the request payloads of given content types (as the mapping key).<br/>requestParameters:  A key-value map defining required or optional method request parameters that can be accepted by API Gateway. A key is a method request parameter name matching the pattern of <code>method.request.{location}.{name}</code>, where <code>location</code> is <code>querystring</code>, <code>path</code>, or <code>header</code> and <code>name</code> is a valid and unique parameter name. The value associated with the key is a Boolean flag indicating whether the parameter is required (<code>true</code>) or optional (<code>false</code>). The method request parameter names defined here are available in <a>Integration</a> to be mapped to integration request parameters or templates.<br/>operationName:  A human-friendly operation identifier for the method. For example, you can assign the <code>operationName</code> of <code>ListPets</code> for the <code>GET /pets</code> method in the <code>PetStore</code> example.<br/>requestValidatorId:  The identifier of a <a>RequestValidator</a> for request validation.<br/>apiKeyRequired:  A boolean flag specifying whether a valid <a>ApiKey</a> is required to invoke this method.<br/>authorizerId:  The identifier of an <a>Authorizer</a> to use on this method. The <code>authorizationType</code> must be <code>CUSTOM</code>.<br/>authorizationType:  The method's authorization type. Valid values are <code>NONE</code> for open access, <code>AWS_IAM</code> for using AWS IAM permissions, <code>CUSTOM</code> for using a custom authorizer, or <code>COGNITO_USER_POOLS</code> for using a Cognito user pool.<br/>httpMethod:  The method's HTTP verb.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>integrationResponses</i> Specifies the integration's responses. <div class="remarks"> <p/> <h4>Example: Get integration responses of a method</h4> <h5>Request</h5> <p/> <pre><code>GET /restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200 HTTP/1.1 Content-Type: application/json Host: apigateway.us-east-1.amazonaws.com X-Amz-Date: 20160607T191449Z Authorization: AWS4-HMAC-SHA256 Credential={access_key_ID}/20160607/us-east-1/apigateway/aws4_request, SignedHeaders=content-type;host;x-amz-date, Signature={sig4_hash} </code></pre> <h5>Response</h5> The successful response returns <code>200 OK</code> status and a payload as follows: <pre><code>{ "_links": { "curies": { "href": "https://docs.aws.amazon.com/apigateway/latest/developerguide/restapi-integration-response-{rel}.html", "name": "integrationresponse", "templated": true }, "self": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200", "title": "200" }, "integrationresponse:delete": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200" }, "integrationresponse:update": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200" } }, "responseParameters": { "method.response.header.Content-Type": "'application/xml'" }, "responseTemplates": { "application/json": "$util.urlDecode(\"%3CkinesisStreams%3E#foreach($stream in $input.path('$.StreamNames'))%3Cstream%3E%3Cname%3E$stream%3C/name%3E%3C/stream%3E#end%3C/kinesisStreams%3E\")\n" }, "statusCode": "200" }</code></pre> <p/> </div> <div class="seeAlso"> <a href="https://docs.aws.amazon.com/apigateway/latest/developerguide/how-to-create-api.html">Creating an API</a> </div><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cacheKeyParameters</i> A list of request parameters whose values API Gateway caches. To be valid values for <code>cacheKeyParameters</code>, these parameters must also be specified for <a>Method</a> <code>requestParameters</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cacheNamespace</i> An API-specific tag group of related cached parameters. To be valid values for <code>cacheKeyParameters</code>, these parameters must also be specified for <a>Method</a> <code>requestParameters</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>timeoutInMillis</i> Custom timeout between 50 and 29,000 milliseconds. The default value is 29,000 milliseconds or 29 seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>contentHandling</i> Specifies how to handle request payload content type conversions. Supported values are <code>CONVERT_TO_BINARY</code> and <code>CONVERT_TO_TEXT</code>, with the following behaviors: <ul> <li><code>CONVERT_TO_BINARY</code>: Converts a request payload from a Base64-encoded string to the corresponding binary blob.</li> <li><code>CONVERT_TO_TEXT</code>: Converts a request payload from a binary blob to a Base64-encoded string.</li> </ul> If this property is not defined, the request payload will be passed through from the method request to integration request without modification, provided that the <code>passthroughBehavior</code> is configured to support payload pass-through.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>passthroughBehavior</i> <div>  Specifies how the method request body of an unmapped content type will be passed through the integration request to the back end without transformation. A content type is unmapped if no mapping template is defined in the integration or the content type does not match any of the mapped content types, as specified in <code>requestTemplates</code>. The valid value is one of the following:  <ul> <li> <code>WHEN_NO_MATCH</code>: passes the method request body through the integration request to the back end without transformation when the method request content type does not match any content type associated with the mapping templates defined in the integration request. </li> <li> <code>WHEN_NO_TEMPLATES</code>: passes the method request body through the integration request to the back end without transformation when no mapping template is defined in the integration request. If a template is defined when this option is selected, the method request of an unmapped content-type will be rejected with an HTTP <code>415 Unsupported Media Type</code> response. </li> <li> <code>NEVER</code>: rejects the method request with an HTTP <code>415 Unsupported Media Type</code> response when either the method request content type does not match any content type associated with the mapping templates defined in the integration request or no mapping template is defined in the integration request. </li> </ul> </div><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>requestTemplates</i> Represents a map of Velocity templates that are applied on the request payload based on the value of the Content-Type header sent by the client. The content type value is the key in this map, and the template (as a String) is the value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>requestParameters</i> A key-value map specifying request parameters that are passed from the method request to the back end. The key is an integration request parameter name and the associated value is a method request parameter value or static value that must be enclosed within single quotes and pre-encoded as required by the back end. The method request parameter value must match the pattern of <code>method.request.{location}.{name}</code>, where <code>location</code> is <code>querystring</code>, <code>path</code>, or <code>header</code> and <code>name</code> must be a valid and unique method request parameter name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>credentials</i> Specifies the credentials required for the integration, if any. For AWS integrations, three options are available. To specify an IAM Role for API Gateway to assume, use the role's Amazon Resource Name (ARN). To require that the caller's identity be passed through from the request, specify the string <code>arn:aws:iam::\*:user/\*</code>. To use resource-based permissions on supported AWS services, specify null.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>connectionId</i> The (<a href="https://docs.aws.amazon.com/apigateway/api-reference/resource/vpc-link/#id"><code>id</code></a>) of the <a>VpcLink</a> used for the integration when <code>connectionType=VPC_LINK</code> and undefined, otherwise.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>connectionType</i> The type of the network connection to the integration endpoint. The valid value is <code>INTERNET</code> for connections through the public routable internet or <code>VPC_LINK</code> for private connections between API Gateway and a network load balancer in a VPC. The default value is <code>INTERNET</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>uri</i> Specifies Uniform Resource Identifier (URI) of the integration endpoint. <ul> <li> For <code>HTTP</code> or <code>HTTP_PROXY</code> integrations, the URI must be a fully formed, encoded HTTP(S) URL according to the <a target="_blank" href="https://en.wikipedia.org/wiki/Uniform_Resource_Identifier">RFC-3986 specification</a>, for either standard integration, where <code>connectionType</code> is not <code>VPC_LINK</code>, or private integration, where <code>connectionType</code> is <code>VPC_LINK</code>. For a private HTTP integration, the URI is not used for routing.  </li> <li> For <code>AWS</code> or <code>AWS_PROXY</code> integrations, the URI is of the form <code>arn:aws:apigateway:{region}:{subdomain.service|service}:path|action/{service_api}</code>. Here, <code>{Region}</code> is the API Gateway region (e.g., <code>us-east-1</code>); <code>{service}</code> is the name of the integrated AWS service (e.g., <code>s3</code>); and <code>{subdomain}</code> is a designated subdomain supported by certain AWS service for fast host-name lookup. <code>action</code> can be used for an AWS service action-based API, using an <code>Action={name}&amp;{p1}={v1}&amp;p2={v2}...</code> query string. The ensuing <code>{service_api}</code> refers to a supported action <code>{name}</code> plus any required input parameters. Alternatively, <code>path</code> can be used for an AWS service path-based API. The ensuing <code>service_api</code> refers to the path to an AWS service resource, including the region of the integrated AWS service, if applicable. For example, for integration with the S3 API of <code><a href="https://docs.aws.amazon.com/AmazonS3/latest/API/RESTObjectGET.html">GetObject</a></code>, the <code>uri</code> can be either <code>arn:aws:apigateway:us-west-2:s3:action/GetObject&amp;Bucket={bucket}&amp;Key={key}</code> or <code>arn:aws:apigateway:us-west-2:s3:path/{bucket}/{key}</code> </li></ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>httpMethod</i> Specifies the integration's HTTP method type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> Specifies an API method integration type. The valid value is one of the following: <ul> <li><code>AWS</code>: for integrating the API method request with an AWS service action, including the Lambda function-invoking action. With the Lambda function-invoking action, this is referred to as the Lambda custom integration. With any other AWS service action, this is known as AWS integration.</li> <li><code>AWS_PROXY</code>: for integrating the API method request with the Lambda function-invoking action with the client request passed through as-is. This integration is also referred to as the Lambda proxy integration.</li> <li><code>HTTP</code>: for integrating the API method request with an HTTP endpoint, including a private HTTP endpoint within a VPC. This integration is also referred to as the HTTP custom integration.</li> <li><code>HTTP_PROXY</code>: for integrating the API method request with an HTTP endpoint, including a private HTTP endpoint within a VPC, with the client request passed through as-is. This is also referred to as the HTTP proxy integration.</li> <li><code>MOCK</code>: for integrating the API method request with API Gateway as a "loop-back" endpoint without invoking any backend.</li> </ul> For the HTTP and HTTP proxy integrations, each integration can specify a protocol (<code>http/https</code>), port and path. Standard 80 and 443 ports are supported as well as custom ports above 1024. An HTTP or HTTP proxy integration with a <code>connectionType</code> of <code>VPC_LINK</code> is referred to as a private integration and uses a <a>VpcLink</a> to connect API Gateway to a network load balancer of a VPC.<br/>        """
        pass

    def get_method_response(self, statusCode: str, httpMethod: str, resourceId: str, restApiId: str) -> MethodResponse:
        """Describes a <a>MethodResponse</a> resource.

        :param self:
        :param statusCode:  [Required] The status code for the <a>MethodResponse</a> resource.<br/>httpMethod:  [Required] The HTTP verb of the <a>Method</a> resource.<br/>resourceId:  [Required] The <a>Resource</a> identifier for the <a>MethodResponse</a> resource.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: responseModels:  Specifies the <a>Model</a> resources used for the response's content-type. Response models are represented as a key/value map, with a content-type as the key and a <a>Model</a> name as the value.<br/>responseParameters:  A key-value map specifying required or optional response parameters that API Gateway can send back to the caller. A key defines a method response header and the value specifies whether the associated method response header is required or not. The expression of the key must match the pattern <code>method.response.header.{name}</code>, where <code>name</code> is a valid and unique header name. API Gateway passes certain integration response data to the method response headers specified here according to the mapping you prescribe in the API's <a>IntegrationResponse</a>. The integration response data that can be mapped include an integration response header expressed in <code>integration.response.header.{name}</code>, a static value enclosed within a pair of single quotes (e.g., <code>'application/json'</code>), or a JSON expression from the back-end response payload in the form of <code>integration.response.body.{JSON-expression}</code>, where <code>JSON-expression</code> is a valid JSON expression without the <code>$</code> prefix.)<br/>statusCode:  The method response's status code.<br/>        """
        pass

    def get_model(self, modelName: str, restApiId: str, flatten: Boolean=None) -> Model:
        """Describes an existing model defined for a <a>RestApi</a> resource.

        :param self:
        :param flatten:  A query parameter of a Boolean value to resolve (<code>true</code>) all external model references and returns a flattened model schema or not (<code>false</code>) The default is <code>false</code>.<br/>modelName:  [Required] The name of the model as an identifier.<br/>restApiId:  [Required] The <a>RestApi</a> identifier under which the <a>Model</a> exists.<br/>
        :return: contentType:  The content-type for the model.<br/>schema:  The schema for the model. For <code>application/json</code> models, this should be <a href="https://tools.ietf.org/html/draft-zyp-json-schema-04" target="_blank">JSON schema draft 4</a> model. Do not include "\*/" characters in the description of any properties because such "\*/" characters may be interpreted as the closing marker for comments in some languages, such as Java or JavaScript, causing the installation of your API's SDK generated by API Gateway to fail.<br/>description:  The description of the model.<br/>name:  The name of the model. Must be an alphanumeric string.<br/>id:  The identifier for the model resource.<br/>        """
        pass

    def get_model_template(self, modelName: str, restApiId: str) -> Template:
        """Generates a sample mapping template that can be used to transform a payload into the structure of a model.

        :param self:
        :param modelName:  [Required] The name of the model for which to generate a template.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: value:  The Apache <a href="https://velocity.apache.org/engine/devel/vtl-reference-guide.html" target="_blank">Velocity Template Language (VTL)</a> template content used for the template resource.<br/>        """
        pass

    def get_models(self, restApiId: str, position: str=None, limit: int=None) -> Models:
        """Describes existing <a>Models</a> defined for a <a>RestApi</a> resource.

        :param self:
        :param limit:  The maximum number of returned results per page. The default value is 25 and the maximum value is 500.<br/>position:  The current pagination position in the paged result set.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: items:  The current page of elements from this collection.<br/>position:  <br/>        """
        pass

    def get_request_validator(self, requestValidatorId: str, restApiId: str) -> RequestValidator:
        """Gets a <a>RequestValidator</a> of a given <a>RestApi</a>.

        :param self:
        :param requestValidatorId:  [Required] The identifier of the <a>RequestValidator</a> to be retrieved.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: validateRequestParameters:  A Boolean flag to indicate whether to validate request parameters (<code>true</code>) or not (<code>false</code>).<br/>validateRequestBody:  A Boolean flag to indicate whether to validate a request body according to the configured <a>Model</a> schema.<br/>name:  The name of this <a>RequestValidator</a><br/>id:  The identifier of this <a>RequestValidator</a>.<br/>        """
        pass

    def get_request_validators(self, restApiId: str, position: str=None, limit: int=None) -> RequestValidators:
        """Gets the <a>RequestValidators</a> collection of a given <a>RestApi</a>.

        :param self:
        :param limit:  The maximum number of returned results per page. The default value is 25 and the maximum value is 500.<br/>position:  The current pagination position in the paged result set.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: items:  The current page of elements from this collection.<br/>position:  <br/>        """
        pass

    def get_resource(self, resourceId: str, restApiId: str, embed: List[String]=None) -> Resource:
        """Lists information about a resource.

        :param self:
        :param embed:  A query parameter to retrieve the specified resources embedded in the returned <a>Resource</a> representation in the response. This <code>embed</code> parameter value is a list of comma-separated strings. Currently, the request supports only retrieval of the embedded <a>Method</a> resources this way. The query parameter value must be a single-valued list and contain the <code>"methods"</code> string. For example, <code>GET /restapis/{restapi_id}/resources/{resource_id}?embed=methods</code>.<br/>resourceId:  [Required] The identifier for the <a>Resource</a> resource.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: resourceMethods:  Gets an API resource's method of a given HTTP verb. <div class="remarks"> The resource methods are a map of methods indexed by methods' HTTP verbs enabled on the resource. This method map is included in the <code>200 OK</code> response of the <code>GET /restapis/{restapi_id}/resources/{resource_id}</code> or <code>GET /restapis/{restapi_id}/resources/{resource_id}?embed=methods</code> request. <h4>Example: Get the GET method of an API resource</h4> <h5>Request</h5> <pre><code>GET /restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET HTTP/1.1 Content-Type: application/json Host: apigateway.us-east-1.amazonaws.com X-Amz-Date: 20170223T031827Z Authorization: AWS4-HMAC-SHA256 Credential={access_key_ID}/20170223/us-east-1/apigateway/aws4_request, SignedHeaders=content-type;host;x-amz-date, Signature={sig4_hash}</code></pre> <h5>Response</h5> <pre><code>{ "_links": { "curies": [ { "href": "https://docs.aws.amazon.com/apigateway/latest/developerguide/restapi-integration-{rel}.html", "name": "integration", "templated": true }, { "href": "https://docs.aws.amazon.com/apigateway/latest/developerguide/restapi-integration-response-{rel}.html", "name": "integrationresponse", "templated": true }, { "href": "https://docs.aws.amazon.com/apigateway/latest/developerguide/restapi-method-{rel}.html", "name": "method", "templated": true }, { "href": "https://docs.aws.amazon.com/apigateway/latest/developerguide/restapi-method-response-{rel}.html", "name": "methodresponse", "templated": true } ], "self": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET", "name": "GET", "title": "GET" }, "integration:put": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration" }, "method:delete": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET" }, "method:integration": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration" }, "method:responses": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/responses/200", "name": "200", "title": "200" }, "method:update": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET" }, "methodresponse:put": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/responses/{status_code}", "templated": true } }, "apiKeyRequired": false, "authorizationType": "NONE", "httpMethod": "GET", "_embedded": { "method:integration": { "_links": { "self": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration" }, "integration:delete": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration" }, "integration:responses": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200", "name": "200", "title": "200" }, "integration:update": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration" }, "integrationresponse:put": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/{status_code}", "templated": true } }, "cacheKeyParameters": [], "cacheNamespace": "3kzxbg5sa2", "credentials": "arn:aws:iam::123456789012:role/apigAwsProxyRole", "httpMethod": "POST", "passthroughBehavior": "WHEN_NO_MATCH", "requestParameters": { "integration.request.header.Content-Type": "'application/x-amz-json-1.1'" }, "requestTemplates": { "application/json": "{\n}" }, "type": "AWS", "uri": "arn:aws:apigateway:us-east-1:kinesis:action/ListStreams", "_embedded": { "integration:responses": { "_links": { "self": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200", "name": "200", "title": "200" }, "integrationresponse:delete": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200" }, "integrationresponse:update": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200" } }, "responseParameters": { "method.response.header.Content-Type": "'application/xml'" }, "responseTemplates": { "application/json": "$util.urlDecode(\"%3CkinesisStreams%3E#foreach($stream in $input.path('$.StreamNames'))%3Cstream%3E%3Cname%3E$stream%3C/name%3E%3C/stream%3E#end%3C/kinesisStreams%3E\")\n" }, "statusCode": "200" } } }, "method:responses": { "_links": { "self": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/responses/200", "name": "200", "title": "200" }, "methodresponse:delete": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/responses/200" }, "methodresponse:update": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/responses/200" } }, "responseModels": { "application/json": "Empty" }, "responseParameters": { "method.response.header.Content-Type": false }, "statusCode": "200" } } }</code></pre> If the <code>OPTIONS</code> is enabled on the resource, you can follow the example here to get that method. Just replace the <code>GET</code> of the last path segment in the request URL with <code>OPTIONS</code>. </div> <div class="seeAlso"> </div><br/>path:  The full path for this resource.<br/>pathPart:  The last path segment for this resource.<br/>parentId:  The parent resource's identifier.<br/>id:  The resource's identifier.<br/>        """
        pass

    def get_resources(self, restApiId: str, position: str=None, limit: int=None, embed: List[String]=None) -> Resources:
        """Lists information about a collection of <a>Resource</a> resources.

        :param self:
        :param embed:  A query parameter used to retrieve the specified resources embedded in the returned <a>Resources</a> resource in the response. This <code>embed</code> parameter value is a list of comma-separated strings. Currently, the request supports only retrieval of the embedded <a>Method</a> resources this way. The query parameter value must be a single-valued list and contain the <code>"methods"</code> string. For example, <code>GET /restapis/{restapi_id}/resources?embed=methods</code>.<br/>limit:  The maximum number of returned results per page. The default value is 25 and the maximum value is 500.<br/>position:  The current pagination position in the paged result set.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: items:  The current page of elements from this collection.<br/>position:  <br/>        """
        pass

    def get_rest_api(self, restApiId: str) -> RestApi:
        """Lists the <a>RestApi</a> resource in the collection.

        :param self:
        :param restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: tags:  The collection of tags. Each tag element is associated with a given resource.<br/>policy:  A stringified JSON policy document that applies to this RestApi regardless of the caller and <a>Method</a> configuration.<br/>endpointConfiguration:  The endpoint configuration of this <a>RestApi</a> showing the endpoint types of the API. :<br/>apiKeySource:  The source of the API key for metering requests according to a usage plan. Valid values are: <ul><li><code>HEADER</code> to read the API key from the <code>X-API-Key</code> header of a request. </li><li><code>AUTHORIZER</code> to read the API key from the <code>UsageIdentifierKey</code> from a custom authorizer.</li></ul> <br/>minimumCompressionSize:  A nullable integer that is used to enable compression (with non-negative between 0 and 10485760 (10M) bytes, inclusive) or disable compression (with a null value) on an API. When compression is enabled, compression or decompression is not applied on the payload if the payload size is smaller than this value. Setting it to zero allows compression for any payload size.<br/>binaryMediaTypes:  The list of binary media types supported by the <a>RestApi</a>. By default, the <a>RestApi</a> supports only UTF-8-encoded text payloads.<br/>warnings:  The warning messages reported when <code>failonwarnings</code> is turned on during API import.<br/>version:  A version identifier for the API.<br/>createdDate:  The timestamp when the API was created.<br/>description:  The API's description.<br/>name:  The API's name.<br/>id:  The API's identifier. This identifier is unique across all of your APIs in API Gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>types</i> A list of endpoint types of an API (<a>RestApi</a>) or its custom domain name (<a>DomainName</a>). For an edge-optimized API and its custom domain name, the endpoint type is <code>"EDGE"</code>. For a regional API and its custom domain name, the endpoint type is <code>REGIONAL</code>. For a private API, the endpoint type is <code>PRIVATE</code>.<br/>        """
        pass

    def get_rest_apis(self, position: str=None, limit: int=None) -> RestApis:
        """Lists the <a>RestApis</a> resources for your collection.

        :param self:
        :param limit:  The maximum number of returned results per page. The default value is 25 and the maximum value is 500.<br/>position:  The current pagination position in the paged result set.<br/>
        :return: items:  The current page of elements from this collection.<br/>position:  <br/>        """
        pass

    def get_sdk(self, sdkType: str, stageName: str, restApiId: str, parameters: MapOfStringToString=None) -> SdkResponse:
        """Generates a client SDK for a <a>RestApi</a> and <a>Stage</a>.

        :param self:
        :param parameters:  A string-to-string key-value map of query parameters <code>sdkType</code>-dependent properties of the SDK. For <code>sdkType</code> of <code>objectivec</code> or <code>swift</code>, a parameter named <code>classPrefix</code> is required. For <code>sdkType</code> of <code>android</code>, parameters named <code>groupId</code>, <code>artifactId</code>, <code>artifactVersion</code>, and <code>invokerPackage</code> are required. For <code>sdkType</code> of <code>java</code>, parameters named <code>serviceName</code> and <code>javaPackageName</code> are required. <br/>sdkType:  [Required] The language for the generated SDK. Currently <code>java</code>, <code>javascript</code>, <code>android</code>, <code>objectivec</code> (for iOS), <code>swift</code> (for iOS), and <code>ruby</code> are supported.<br/>stageName:  [Required] The name of the <a>Stage</a> that the SDK will use.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: body:  The binary blob response to <a>GetSdk</a>, which contains the generated SDK.<br/>contentDisposition:  The content-disposition header value in the HTTP response.<br/>contentType:  The content-type header value in the HTTP response.<br/>        """
        pass

    def get_sdk_type(self, id: str) -> SdkType:
        """

        :param self:
        :param id:  [Required] The identifier of the queried <a>SdkType</a> instance.<br/>
        :return: configurationProperties:  A list of configuration properties of an <a>SdkType</a>.<br/>description:  The description of an <a>SdkType</a>.<br/>friendlyName:  The user-friendly name of an <a>SdkType</a> instance.<br/>id:  The identifier of an <a>SdkType</a> instance.<br/>        """
        pass

    def get_sdk_types(self, position: str=None, limit: int=None) -> SdkTypes:
        """

        :param self:
        :param limit:  The maximum number of returned results per page. The default value is 25 and the maximum value is 500.<br/>position:  The current pagination position in the paged result set.<br/>
        :return: items:  The current page of elements from this collection.<br/>position:  <br/>        """
        pass

    def get_stage(self, stageName: str, restApiId: str) -> Stage:
        """Gets information about a <a>Stage</a> resource.

        :param self:
        :param stageName:  [Required] The name of the <a>Stage</a> resource to get information about.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: lastUpdatedDate:  The timestamp when the stage last updated.<br/>createdDate:  The timestamp when the stage was created.<br/>tags:  The collection of tags. Each tag element is associated with a given resource.<br/>webAclArn:  The ARN of the WebAcl associated with the <a>Stage</a>.<br/>tracingEnabled:  Specifies whether active tracing with X-ray is enabled for the <a>Stage</a>.<br/>canarySettings:  Settings for the canary deployment in this stage.:<br/>accessLogSettings:  Settings for logging access in this stage.:<br/>documentationVersion:  The version of the associated API documentation.<br/>variables:  A map that defines the stage variables for a <a>Stage</a> resource. Variable names can have alphanumeric and underscore characters, and the values must match <code>[A-Za-z0-9-._~:/?#&amp;=,]+</code>.<br/>methodSettings:  A map that defines the method settings for a <a>Stage</a> resource. Keys (designated as <code>/{method_setting_key</code> below) are method paths defined as <code>{resource_path}/{http_method}</code> for an individual method override, or <code>/\*/\*</code> for overriding all methods in the stage. <br/>cacheClusterStatus:  The status of the cache cluster for the stage, if enabled.<br/>cacheClusterSize:  The size of the cache cluster for the stage, if enabled.<br/>cacheClusterEnabled:  Specifies whether a cache cluster is enabled for the stage.<br/>description:  The stage's description.<br/>stageName:  The name of the stage is the first path segment in the Uniform Resource Identifier (URI) of a call to API Gateway. Stage names can only contain alphanumeric characters, hyphens, and underscores. Maximum length is 128 characters.<br/>clientCertificateId:  The identifier of a client certificate for an API stage.<br/>deploymentId:  The identifier of the <a>Deployment</a> that the stage points to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>destinationArn</i> The ARN of the CloudWatch Logs log group to receive access logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>format</i> A single line format of the access logs of data, as specified by selected <a href="https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-mapping-template-reference.html#context-variable-reference">$context variables</a>. The format must include at least <code>$context.requestId</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>useStageCache</i> A Boolean flag to indicate whether the canary deployment uses the stage cache or not.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stageVariableOverrides</i> Stage variables overridden for a canary release deployment, including new stage variables introduced in the canary. These stage variables are represented as a string-to-string map between stage variable names and their values.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentId</i> The ID of the canary deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>percentTraffic</i> The percent (0-100) of traffic diverted to a canary deployment.<br/>        """
        pass

    def get_stages(self, restApiId: str, deploymentId: str=None) -> Stages:
        """Gets information about one or more <a>Stage</a> resources.

        :param self:
        :param deploymentId:  The stages' deployment identifiers.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: item:  The current page of elements from this collection.<br/>        """
        pass

    def get_tags(self, resourceArn: str, position: str=None, limit: int=None) -> Tags:
        """Gets the <a>Tags</a> collection for a given resource.

        :param self:
        :param limit:  (Not currently supported) The maximum number of returned results per page. The default value is 25 and the maximum value is 500.<br/>position:  (Not currently supported) The current pagination position in the paged result set.<br/>resourceArn:  [Required] The ARN of a resource that can be tagged. The resource ARN must be URL-encoded.<br/>
        :return: tags:  The collection of tags. Each tag element is associated with a given resource.<br/>        """
        pass

    def get_usage(self, endDate: str, startDate: str, usagePlanId: str, keyId: str=None, position: str=None, limit: int=None) -> Usage:
        """Gets the usage data of a usage plan in a specified time interval.

        :param self:
        :param limit:  The maximum number of returned results per page. The default value is 25 and the maximum value is 500.<br/>position:  The current pagination position in the paged result set.<br/>endDate:  [Required] The ending date (e.g., 2016-12-31) of the usage data.<br/>startDate:  [Required] The starting date (e.g., 2016-01-01) of the usage data.<br/>keyId:  The Id of the API key associated with the resultant usage data.<br/>usagePlanId:  [Required] The Id of the usage plan associated with the usage data.<br/>
        :return: items:  The usage data, as daily logs of used and remaining quotas, over the specified time interval indexed over the API keys in a usage plan. For example, <code>{..., "values" : { "{api_key}" : [ [0, 100], [10, 90], [100, 10]]}</code>, where <code>{api_key}</code> stands for an API key value and the daily log entry is of the format <code>[used quota, remaining quota]</code>.<br/>position:  <br/>endDate:  The ending date of the usage data.<br/>startDate:  The starting date of the usage data.<br/>usagePlanId:  The plan Id associated with this usage data.<br/>        """
        pass

    def get_usage_plan(self, usagePlanId: str) -> UsagePlan:
        """Gets a usage plan of a given plan identifier.

        :param self:
        :param usagePlanId:  [Required] The identifier of the <a>UsagePlan</a> resource to be retrieved.<br/>
        :return: tags:  The collection of tags. Each tag element is associated with a given resource.<br/>productCode:  The AWS Markeplace product identifier to associate with the usage plan as a SaaS product on AWS Marketplace.<br/>quota:  The maximum number of permitted requests per a given unit time interval.:<br/>throttle:  The request throttle limits of a usage plan.:<br/>apiStages:  The associated API stages of a usage plan.<br/>description:  The description of a usage plan.<br/>name:  The name of a usage plan.<br/>id:  The identifier of a <a>UsagePlan</a> resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>rateLimit</i> The API request steady-state rate limit.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>burstLimit</i> The API request burst limit, the maximum rate limit over a time ranging from one to a few seconds, depending upon whether the underlying token bucket is at its full capacity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>period</i> The time period in which the limit applies. Valid values are "DAY", "WEEK" or "MONTH".<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>offset</i> The number of requests subtracted from the given limit in the initial time period.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>limit</i> The maximum number of requests that can be made in a given time period.<br/>        """
        pass

    def get_usage_plan_key(self, keyId: str, usagePlanId: str) -> UsagePlanKey:
        """Gets a usage plan key of a given key identifier.

        :param self:
        :param keyId:  [Required] The key Id of the to-be-retrieved <a>UsagePlanKey</a> resource representing a plan customer.<br/>usagePlanId:  [Required] The Id of the <a>UsagePlan</a> resource representing the usage plan containing the to-be-retrieved <a>UsagePlanKey</a> resource representing a plan customer.<br/>
        :return: name:  The name of a usage plan key.<br/>value:  The value of a usage plan key.<br/>type:  The type of a usage plan key. Currently, the valid key type is <code>API_KEY</code>.<br/>id:  The Id of a usage plan key.<br/>        """
        pass

    def get_usage_plan_keys(self, usagePlanId: str, position: str=None, limit: int=None, nameQuery: str=None) -> UsagePlanKeys:
        """Gets all the usage plan keys representing the API keys added to a specified usage plan.

        :param self:
        :param nameQuery:  A query parameter specifying the name of the to-be-returned usage plan keys.<br/>limit:  The maximum number of returned results per page. The default value is 25 and the maximum value is 500.<br/>position:  The current pagination position in the paged result set.<br/>usagePlanId:  [Required] The Id of the <a>UsagePlan</a> resource representing the usage plan containing the to-be-retrieved <a>UsagePlanKey</a> resource representing a plan customer.<br/>
        :return: items:  The current page of elements from this collection.<br/>position:  <br/>        """
        pass

    def get_usage_plans(self, position: str=None, keyId: str=None, limit: int=None) -> UsagePlans:
        """Gets all the usage plans of the caller's account.

        :param self:
        :param limit:  The maximum number of returned results per page. The default value is 25 and the maximum value is 500.<br/>keyId:  The identifier of the API key associated with the usage plans.<br/>position:  The current pagination position in the paged result set.<br/>
        :return: items:  The current page of elements from this collection.<br/>position:  <br/>        """
        pass

    def get_vpc_link(self, vpcLinkId: str) -> VpcLink:
        """Gets a specified VPC link under the caller's account in a region.

        :param self:
        :param vpcLinkId:  [Required] The identifier of the <a>VpcLink</a>. It is used in an <a>Integration</a> to reference this <a>VpcLink</a>.<br/>
        :return: tags:  The collection of tags. Each tag element is associated with a given resource.<br/>statusMessage:  A description about the VPC link status.<br/>status:  The status of the VPC link. The valid values are <code>AVAILABLE</code>, <code>PENDING</code>, <code>DELETING</code>, or <code>FAILED</code>. Deploying an API will wait if the status is <code>PENDING</code> and will fail if the status is <code>DELETING</code>. <br/>targetArns:  The ARNs of network load balancers of the VPC targeted by the VPC link. The network load balancers must be owned by the same AWS account of the API owner.<br/>description:  The description of the VPC link.<br/>name:  The name used to label and identify the VPC link.<br/>id:  The identifier of the <a>VpcLink</a>. It is used in an <a>Integration</a> to reference this <a>VpcLink</a>.<br/>        """
        pass

    def get_vpc_links(self, position: str=None, limit: int=None) -> VpcLinks:
        """Gets the <a>VpcLinks</a> collection under the caller's account in a selected region.

        :param self:
        :param limit:  The maximum number of returned results per page. The default value is 25 and the maximum value is 500.<br/>position:  The current pagination position in the paged result set.<br/>
        :return: items:  The current page of elements from this collection.<br/>position:  <br/>        """
        pass

    def import_api_keys(self, format: str, body: Blob, failOnWarnings: Boolean=None) -> ApiKeyIds:
        """Import API keys from an external source, such as a CSV-formatted file.

        :param self:
        :param failOnWarnings:  A query parameter to indicate whether to rollback <a>ApiKey</a> importation (<code>true</code>) or not (<code>false</code>) when error is encountered.<br/>format:  A query parameter to specify the input format to imported API keys. Currently, only the <code>csv</code> format is supported.<br/>body:  The payload of the POST request to import API keys. For the payload format, see <a href="https://docs.aws.amazon.com/apigateway/latest/developerguide/api-key-file-format.html">API Key File Format</a>.<br/>
        :return: warnings:  A list of warning messages.<br/>ids:  A list of all the <a>ApiKey</a> identifiers.<br/>        """
        pass

    def import_documentation_parts(self, body: Blob, restApiId: str, mode: str=None, failOnWarnings: Boolean=None) -> DocumentationPartIds:
        """

        :param self:
        :param body:  [Required] Raw byte array representing the to-be-imported documentation parts. To import from an OpenAPI file, this is a JSON object.<br/>failOnWarnings:  A query parameter to specify whether to rollback the documentation importation (<code>true</code>) or not (<code>false</code>) when a warning is encountered. The default value is <code>false</code>.<br/>mode:  A query parameter to indicate whether to overwrite (<code>OVERWRITE</code>) any existing <a>DocumentationParts</a> definition or to merge (<code>MERGE</code>) the new definition into the existing one. The default value is <code>MERGE</code>.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: warnings:  A list of warning messages reported during import of documentation parts.<br/>ids:  A list of the returned documentation part identifiers.<br/>        """
        pass

    def import_rest_api(self, body: Blob, failOnWarnings: Boolean=None, parameters: MapOfStringToString=None) -> RestApi:
        """A feature of the API Gateway control service for creating a new API from an external API definition file.

        :param self:
        :param body:  [Required] The POST request body containing external API definitions. Currently, only OpenAPI definition JSON/YAML files are supported. The maximum size of the API definition file is 2MB.<br/>parameters:  A key-value map of context-specific query string parameters specifying the behavior of different API importing operations. The following shows operation-specific parameters and their supported values.  To exclude <a>DocumentationParts</a> from the import, set <code>parameters</code> as <code>ignore=documentation</code>.  To configure the endpoint type, set <code>parameters</code> as <code>endpointConfigurationTypes=EDGE</code>, <code>endpointConfigurationTypes=REGIONAL</code>, or <code>endpointConfigurationTypes=PRIVATE</code>. The default endpoint type is <code>EDGE</code>.  To handle imported <code>basepath</code>, set <code>parameters</code> as <code>basepath=ignore</code>, <code>basepath=prepend</code> or <code>basepath=split</code>. For example, the AWS CLI command to exclude documentation from the imported API is: <pre><code>aws apigateway import-rest-api --parameters ignore=documentation --body 'file:///path/to/imported-api-body.json'</code></pre> The AWS CLI command to set the regional endpoint on the imported API is: <pre><code>aws apigateway import-rest-api --parameters endpointConfigurationTypes=REGIONAL --body 'file:///path/to/imported-api-body.json'</code></pre><br/>failOnWarnings:  A query parameter to indicate whether to rollback the API creation (<code>true</code>) or not (<code>false</code>) when a warning is encountered. The default value is <code>false</code>.<br/>
        :return: tags:  The collection of tags. Each tag element is associated with a given resource.<br/>policy:  A stringified JSON policy document that applies to this RestApi regardless of the caller and <a>Method</a> configuration.<br/>endpointConfiguration:  The endpoint configuration of this <a>RestApi</a> showing the endpoint types of the API. :<br/>apiKeySource:  The source of the API key for metering requests according to a usage plan. Valid values are: <ul><li><code>HEADER</code> to read the API key from the <code>X-API-Key</code> header of a request. </li><li><code>AUTHORIZER</code> to read the API key from the <code>UsageIdentifierKey</code> from a custom authorizer.</li></ul> <br/>minimumCompressionSize:  A nullable integer that is used to enable compression (with non-negative between 0 and 10485760 (10M) bytes, inclusive) or disable compression (with a null value) on an API. When compression is enabled, compression or decompression is not applied on the payload if the payload size is smaller than this value. Setting it to zero allows compression for any payload size.<br/>binaryMediaTypes:  The list of binary media types supported by the <a>RestApi</a>. By default, the <a>RestApi</a> supports only UTF-8-encoded text payloads.<br/>warnings:  The warning messages reported when <code>failonwarnings</code> is turned on during API import.<br/>version:  A version identifier for the API.<br/>createdDate:  The timestamp when the API was created.<br/>description:  The API's description.<br/>name:  The API's name.<br/>id:  The API's identifier. This identifier is unique across all of your APIs in API Gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>types</i> A list of endpoint types of an API (<a>RestApi</a>) or its custom domain name (<a>DomainName</a>). For an edge-optimized API and its custom domain name, the endpoint type is <code>"EDGE"</code>. For a regional API and its custom domain name, the endpoint type is <code>REGIONAL</code>. For a private API, the endpoint type is <code>PRIVATE</code>.<br/>        """
        pass

    def put_gateway_response(self, responseType: str, restApiId: str, statusCode: str=None, responseParameters: MapOfStringToString=None, responseTemplates: MapOfStringToString=None) -> GatewayResponse:
        """Creates a customization of a <a>GatewayResponse</a> of a specified response type and status code on the given <a>RestApi</a>.

        :param self:
        :param responseTemplates:  Response templates of the <a>GatewayResponse</a> as a string-to-string map of key-value pairs.<br/>responseParameters:  Response parameters (paths, query strings and headers) of the <a>GatewayResponse</a> as a string-to-string map of key-value pairs.<br/>statusCode:  The HTTP status code of the <a>GatewayResponse</a>.<br/>responseType:  [Required] The response type of the associated <a>GatewayResponse</a>. Valid values are <ul><li>ACCESS_DENIED</li><li>API_CONFIGURATION_ERROR</li><li>AUTHORIZER_FAILURE</li><li> AUTHORIZER_CONFIGURATION_ERROR</li><li>BAD_REQUEST_PARAMETERS</li><li>BAD_REQUEST_BODY</li><li>DEFAULT_4XX</li><li>DEFAULT_5XX</li><li>EXPIRED_TOKEN</li><li>INVALID_SIGNATURE</li><li>INTEGRATION_FAILURE</li><li>INTEGRATION_TIMEOUT</li><li>INVALID_API_KEY</li><li>MISSING_AUTHENTICATION_TOKEN</li><li> QUOTA_EXCEEDED</li><li>REQUEST_TOO_LARGE</li><li>RESOURCE_NOT_FOUND</li><li>THROTTLED</li><li>UNAUTHORIZED</li><li>UNSUPPORTED_MEDIA_TYPE</li></ul> <br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: defaultResponse:  A Boolean flag to indicate whether this <a>GatewayResponse</a> is the default gateway response (<code>true</code>) or not (<code>false</code>). A default gateway response is one generated by API Gateway without any customization by an API developer. <br/>responseTemplates:  Response templates of the <a>GatewayResponse</a> as a string-to-string map of key-value pairs.<br/>responseParameters:  Response parameters (paths, query strings and headers) of the <a>GatewayResponse</a> as a string-to-string map of key-value pairs.<br/>statusCode:  The HTTP status code for this <a>GatewayResponse</a>.<br/>responseType:  The response type of the associated <a>GatewayResponse</a>. Valid values are <ul><li>ACCESS_DENIED</li><li>API_CONFIGURATION_ERROR</li><li>AUTHORIZER_FAILURE</li><li> AUTHORIZER_CONFIGURATION_ERROR</li><li>BAD_REQUEST_PARAMETERS</li><li>BAD_REQUEST_BODY</li><li>DEFAULT_4XX</li><li>DEFAULT_5XX</li><li>EXPIRED_TOKEN</li><li>INVALID_SIGNATURE</li><li>INTEGRATION_FAILURE</li><li>INTEGRATION_TIMEOUT</li><li>INVALID_API_KEY</li><li>MISSING_AUTHENTICATION_TOKEN</li><li> QUOTA_EXCEEDED</li><li>REQUEST_TOO_LARGE</li><li>RESOURCE_NOT_FOUND</li><li>THROTTLED</li><li>UNAUTHORIZED</li><li>UNSUPPORTED_MEDIA_TYPE</li></ul> <br/>        """
        pass

    def put_integration(self, type: str, httpMethod: str, resourceId: str, restApiId: str, integrationHttpMethod: str=None, uri: str=None, connectionType: str=None, connectionId: str=None, credentials: str=None, requestParameters: MapOfStringToString=None, requestTemplates: MapOfStringToString=None, passthroughBehavior: str=None, cacheNamespace: str=None, cacheKeyParameters: List[String]=None, contentHandling: str=None, timeoutInMillis: int=None) -> Integration:
        """Sets up a method's integration.

        :param self:
        :param timeoutInMillis:  Custom timeout between 50 and 29,000 milliseconds. The default value is 29,000 milliseconds or 29 seconds.<br/>contentHandling:  Specifies how to handle request payload content type conversions. Supported values are <code>CONVERT_TO_BINARY</code> and <code>CONVERT_TO_TEXT</code>, with the following behaviors: <ul> <li><code>CONVERT_TO_BINARY</code>: Converts a request payload from a Base64-encoded string to the corresponding binary blob.</li> <li><code>CONVERT_TO_TEXT</code>: Converts a request payload from a binary blob to a Base64-encoded string.</li> </ul> If this property is not defined, the request payload will be passed through from the method request to integration request without modification, provided that the <code>passthroughBehavior</code> is configured to support payload pass-through.<br/>cacheKeyParameters:  An API-specific tag group of related cached parameters.<br/>cacheNamespace:  A list of request parameters whose values are to be cached.<br/>passthroughBehavior:  Specifies the pass-through behavior for incoming requests based on the Content-Type header in the request, and the available mapping templates specified as the <code>requestTemplates</code> property on the Integration resource. There are three valid values: <code>WHEN_NO_MATCH</code>, <code>WHEN_NO_TEMPLATES</code>, and <code>NEVER</code>.  <ul> <li><code>WHEN_NO_MATCH</code> passes the request body for unmapped content types through to the integration back end without transformation.</li> <li><code>NEVER</code> rejects unmapped content types with an HTTP 415 'Unsupported Media Type' response.</li> <li><code>WHEN_NO_TEMPLATES</code> allows pass-through when the integration has NO content types mapped to templates. However if there is at least one content type defined, unmapped content types will be rejected with the same 415 response.</li> </ul><br/>requestTemplates:  Represents a map of Velocity templates that are applied on the request payload based on the value of the Content-Type header sent by the client. The content type value is the key in this map, and the template (as a String) is the value.<br/>requestParameters:  A key-value map specifying request parameters that are passed from the method request to the back end. The key is an integration request parameter name and the associated value is a method request parameter value or static value that must be enclosed within single quotes and pre-encoded as required by the back end. The method request parameter value must match the pattern of <code>method.request.{location}.{name}</code>, where <code>location</code> is <code>querystring</code>, <code>path</code>, or <code>header</code> and <code>name</code> must be a valid and unique method request parameter name.<br/>credentials:  Specifies whether credentials are required for a put integration.<br/>connectionId:  The (<a href="https://docs.aws.amazon.com/apigateway/api-reference/resource/vpc-link/#id"><code>id</code></a>) of the <a>VpcLink</a> used for the integration when <code>connectionType=VPC_LINK</code> and undefined, otherwise.<br/>connectionType:  The type of the network connection to the integration endpoint. The valid value is <code>INTERNET</code> for connections through the public routable internet or <code>VPC_LINK</code> for private connections between API Gateway and a network load balancer in a VPC. The default value is <code>INTERNET</code>.<br/>uri:  Specifies Uniform Resource Identifier (URI) of the integration endpoint. <ul> <li> For <code>HTTP</code> or <code>HTTP_PROXY</code> integrations, the URI must be a fully formed, encoded HTTP(S) URL according to the <a target="_blank" href="https://en.wikipedia.org/wiki/Uniform_Resource_Identifier">RFC-3986 specification</a>, for either standard integration, where <code>connectionType</code> is not <code>VPC_LINK</code>, or private integration, where <code>connectionType</code> is <code>VPC_LINK</code>. For a private HTTP integration, the URI is not used for routing.  </li> <li> For <code>AWS</code> or <code>AWS_PROXY</code> integrations, the URI is of the form <code>arn:aws:apigateway:{region}:{subdomain.service|service}:path|action/{service_api}</code>. Here, <code>{Region}</code> is the API Gateway region (e.g., <code>us-east-1</code>); <code>{service}</code> is the name of the integrated AWS service (e.g., <code>s3</code>); and <code>{subdomain}</code> is a designated subdomain supported by certain AWS service for fast host-name lookup. <code>action</code> can be used for an AWS service action-based API, using an <code>Action={name}&amp;{p1}={v1}&amp;p2={v2}...</code> query string. The ensuing <code>{service_api}</code> refers to a supported action <code>{name}</code> plus any required input parameters. Alternatively, <code>path</code> can be used for an AWS service path-based API. The ensuing <code>service_api</code> refers to the path to an AWS service resource, including the region of the integrated AWS service, if applicable. For example, for integration with the S3 API of <code><a href="https://docs.aws.amazon.com/AmazonS3/latest/API/RESTObjectGET.html">GetObject</a></code>, the <code>uri</code> can be either <code>arn:aws:apigateway:us-west-2:s3:action/GetObject&amp;Bucket={bucket}&amp;Key={key}</code> or <code>arn:aws:apigateway:us-west-2:s3:path/{bucket}/{key}</code> </li></ul><br/>integrationHttpMethod:  Specifies a put integration HTTP method. When the integration type is HTTP or AWS, this field is required.<br/>type:  [Required] Specifies a put integration input's type.<br/>httpMethod:  [Required] Specifies a put integration request's HTTP method.<br/>resourceId:  [Required] Specifies a put integration request's resource ID.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: integrationResponses:  Specifies the integration's responses. <div class="remarks"> <p/> <h4>Example: Get integration responses of a method</h4> <h5>Request</h5> <p/> <pre><code>GET /restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200 HTTP/1.1 Content-Type: application/json Host: apigateway.us-east-1.amazonaws.com X-Amz-Date: 20160607T191449Z Authorization: AWS4-HMAC-SHA256 Credential={access_key_ID}/20160607/us-east-1/apigateway/aws4_request, SignedHeaders=content-type;host;x-amz-date, Signature={sig4_hash} </code></pre> <h5>Response</h5> The successful response returns <code>200 OK</code> status and a payload as follows: <pre><code>{ "_links": { "curies": { "href": "https://docs.aws.amazon.com/apigateway/latest/developerguide/restapi-integration-response-{rel}.html", "name": "integrationresponse", "templated": true }, "self": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200", "title": "200" }, "integrationresponse:delete": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200" }, "integrationresponse:update": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200" } }, "responseParameters": { "method.response.header.Content-Type": "'application/xml'" }, "responseTemplates": { "application/json": "$util.urlDecode(\"%3CkinesisStreams%3E#foreach($stream in $input.path('$.StreamNames'))%3Cstream%3E%3Cname%3E$stream%3C/name%3E%3C/stream%3E#end%3C/kinesisStreams%3E\")\n" }, "statusCode": "200" }</code></pre> <p/> </div> <div class="seeAlso"> <a href="https://docs.aws.amazon.com/apigateway/latest/developerguide/how-to-create-api.html">Creating an API</a> </div><br/>cacheKeyParameters:  A list of request parameters whose values API Gateway caches. To be valid values for <code>cacheKeyParameters</code>, these parameters must also be specified for <a>Method</a> <code>requestParameters</code>.<br/>cacheNamespace:  An API-specific tag group of related cached parameters. To be valid values for <code>cacheKeyParameters</code>, these parameters must also be specified for <a>Method</a> <code>requestParameters</code>.<br/>timeoutInMillis:  Custom timeout between 50 and 29,000 milliseconds. The default value is 29,000 milliseconds or 29 seconds.<br/>contentHandling:  Specifies how to handle request payload content type conversions. Supported values are <code>CONVERT_TO_BINARY</code> and <code>CONVERT_TO_TEXT</code>, with the following behaviors: <ul> <li><code>CONVERT_TO_BINARY</code>: Converts a request payload from a Base64-encoded string to the corresponding binary blob.</li> <li><code>CONVERT_TO_TEXT</code>: Converts a request payload from a binary blob to a Base64-encoded string.</li> </ul> If this property is not defined, the request payload will be passed through from the method request to integration request without modification, provided that the <code>passthroughBehavior</code> is configured to support payload pass-through.<br/>passthroughBehavior:  <div>  Specifies how the method request body of an unmapped content type will be passed through the integration request to the back end without transformation. A content type is unmapped if no mapping template is defined in the integration or the content type does not match any of the mapped content types, as specified in <code>requestTemplates</code>. The valid value is one of the following:  <ul> <li> <code>WHEN_NO_MATCH</code>: passes the method request body through the integration request to the back end without transformation when the method request content type does not match any content type associated with the mapping templates defined in the integration request. </li> <li> <code>WHEN_NO_TEMPLATES</code>: passes the method request body through the integration request to the back end without transformation when no mapping template is defined in the integration request. If a template is defined when this option is selected, the method request of an unmapped content-type will be rejected with an HTTP <code>415 Unsupported Media Type</code> response. </li> <li> <code>NEVER</code>: rejects the method request with an HTTP <code>415 Unsupported Media Type</code> response when either the method request content type does not match any content type associated with the mapping templates defined in the integration request or no mapping template is defined in the integration request. </li> </ul> </div><br/>requestTemplates:  Represents a map of Velocity templates that are applied on the request payload based on the value of the Content-Type header sent by the client. The content type value is the key in this map, and the template (as a String) is the value.<br/>requestParameters:  A key-value map specifying request parameters that are passed from the method request to the back end. The key is an integration request parameter name and the associated value is a method request parameter value or static value that must be enclosed within single quotes and pre-encoded as required by the back end. The method request parameter value must match the pattern of <code>method.request.{location}.{name}</code>, where <code>location</code> is <code>querystring</code>, <code>path</code>, or <code>header</code> and <code>name</code> must be a valid and unique method request parameter name.<br/>credentials:  Specifies the credentials required for the integration, if any. For AWS integrations, three options are available. To specify an IAM Role for API Gateway to assume, use the role's Amazon Resource Name (ARN). To require that the caller's identity be passed through from the request, specify the string <code>arn:aws:iam::\*:user/\*</code>. To use resource-based permissions on supported AWS services, specify null.<br/>connectionId:  The (<a href="https://docs.aws.amazon.com/apigateway/api-reference/resource/vpc-link/#id"><code>id</code></a>) of the <a>VpcLink</a> used for the integration when <code>connectionType=VPC_LINK</code> and undefined, otherwise.<br/>connectionType:  The type of the network connection to the integration endpoint. The valid value is <code>INTERNET</code> for connections through the public routable internet or <code>VPC_LINK</code> for private connections between API Gateway and a network load balancer in a VPC. The default value is <code>INTERNET</code>.<br/>uri:  Specifies Uniform Resource Identifier (URI) of the integration endpoint. <ul> <li> For <code>HTTP</code> or <code>HTTP_PROXY</code> integrations, the URI must be a fully formed, encoded HTTP(S) URL according to the <a target="_blank" href="https://en.wikipedia.org/wiki/Uniform_Resource_Identifier">RFC-3986 specification</a>, for either standard integration, where <code>connectionType</code> is not <code>VPC_LINK</code>, or private integration, where <code>connectionType</code> is <code>VPC_LINK</code>. For a private HTTP integration, the URI is not used for routing.  </li> <li> For <code>AWS</code> or <code>AWS_PROXY</code> integrations, the URI is of the form <code>arn:aws:apigateway:{region}:{subdomain.service|service}:path|action/{service_api}</code>. Here, <code>{Region}</code> is the API Gateway region (e.g., <code>us-east-1</code>); <code>{service}</code> is the name of the integrated AWS service (e.g., <code>s3</code>); and <code>{subdomain}</code> is a designated subdomain supported by certain AWS service for fast host-name lookup. <code>action</code> can be used for an AWS service action-based API, using an <code>Action={name}&amp;{p1}={v1}&amp;p2={v2}...</code> query string. The ensuing <code>{service_api}</code> refers to a supported action <code>{name}</code> plus any required input parameters. Alternatively, <code>path</code> can be used for an AWS service path-based API. The ensuing <code>service_api</code> refers to the path to an AWS service resource, including the region of the integrated AWS service, if applicable. For example, for integration with the S3 API of <code><a href="https://docs.aws.amazon.com/AmazonS3/latest/API/RESTObjectGET.html">GetObject</a></code>, the <code>uri</code> can be either <code>arn:aws:apigateway:us-west-2:s3:action/GetObject&amp;Bucket={bucket}&amp;Key={key}</code> or <code>arn:aws:apigateway:us-west-2:s3:path/{bucket}/{key}</code> </li></ul><br/>httpMethod:  Specifies the integration's HTTP method type.<br/>type:  Specifies an API method integration type. The valid value is one of the following: <ul> <li><code>AWS</code>: for integrating the API method request with an AWS service action, including the Lambda function-invoking action. With the Lambda function-invoking action, this is referred to as the Lambda custom integration. With any other AWS service action, this is known as AWS integration.</li> <li><code>AWS_PROXY</code>: for integrating the API method request with the Lambda function-invoking action with the client request passed through as-is. This integration is also referred to as the Lambda proxy integration.</li> <li><code>HTTP</code>: for integrating the API method request with an HTTP endpoint, including a private HTTP endpoint within a VPC. This integration is also referred to as the HTTP custom integration.</li> <li><code>HTTP_PROXY</code>: for integrating the API method request with an HTTP endpoint, including a private HTTP endpoint within a VPC, with the client request passed through as-is. This is also referred to as the HTTP proxy integration.</li> <li><code>MOCK</code>: for integrating the API method request with API Gateway as a "loop-back" endpoint without invoking any backend.</li> </ul> For the HTTP and HTTP proxy integrations, each integration can specify a protocol (<code>http/https</code>), port and path. Standard 80 and 443 ports are supported as well as custom ports above 1024. An HTTP or HTTP proxy integration with a <code>connectionType</code> of <code>VPC_LINK</code> is referred to as a private integration and uses a <a>VpcLink</a> to connect API Gateway to a network load balancer of a VPC.<br/>        """
        pass

    def put_integration_response(self, statusCode: str, httpMethod: str, resourceId: str, restApiId: str, selectionPattern: str=None, responseParameters: MapOfStringToString=None, responseTemplates: MapOfStringToString=None, contentHandling: str=None) -> IntegrationResponse:
        """Represents a put integration.

        :param self:
        :param contentHandling:  Specifies how to handle response payload content type conversions. Supported values are <code>CONVERT_TO_BINARY</code> and <code>CONVERT_TO_TEXT</code>, with the following behaviors: <ul> <li><code>CONVERT_TO_BINARY</code>: Converts a response payload from a Base64-encoded string to the corresponding binary blob.</li> <li><code>CONVERT_TO_TEXT</code>: Converts a response payload from a binary blob to a Base64-encoded string.</li> </ul> If this property is not defined, the response payload will be passed through from the integration response to the method response without modification.<br/>responseTemplates:  Specifies a put integration response's templates.<br/>responseParameters:  A key-value map specifying response parameters that are passed to the method response from the back end. The key is a method response header parameter name and the mapped value is an integration response header value, a static value enclosed within a pair of single quotes, or a JSON expression from the integration response body. The mapping key must match the pattern of <code>method.response.header.{name}</code>, where <code>name</code> is a valid and unique header name. The mapped non-static value must match the pattern of <code>integration.response.header.{name}</code> or <code>integration.response.body.{JSON-expression}</code>, where <code>name</code> must be a valid and unique response header name and <code>JSON-expression</code> a valid JSON expression without the <code>$</code> prefix.<br/>selectionPattern:  Specifies the selection pattern of a put integration response.<br/>statusCode:  [Required] Specifies the status code that is used to map the integration response to an existing <a>MethodResponse</a>.<br/>httpMethod:  [Required] Specifies a put integration response request's HTTP method.<br/>resourceId:  [Required] Specifies a put integration response request's resource identifier.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: contentHandling:  Specifies how to handle response payload content type conversions. Supported values are <code>CONVERT_TO_BINARY</code> and <code>CONVERT_TO_TEXT</code>, with the following behaviors: <ul> <li><code>CONVERT_TO_BINARY</code>: Converts a response payload from a Base64-encoded string to the corresponding binary blob.</li> <li><code>CONVERT_TO_TEXT</code>: Converts a response payload from a binary blob to a Base64-encoded string.</li> </ul> If this property is not defined, the response payload will be passed through from the integration response to the method response without modification.<br/>responseTemplates:  Specifies the templates used to transform the integration response body. Response templates are represented as a key/value map, with a content-type as the key and a template as the value.<br/>responseParameters:  A key-value map specifying response parameters that are passed to the method response from the back end. The key is a method response header parameter name and the mapped value is an integration response header value, a static value enclosed within a pair of single quotes, or a JSON expression from the integration response body. The mapping key must match the pattern of <code>method.response.header.{name}</code>, where <code>name</code> is a valid and unique header name. The mapped non-static value must match the pattern of <code>integration.response.header.{name}</code> or <code>integration.response.body.{JSON-expression}</code>, where <code>name</code> is a valid and unique response header name and <code>JSON-expression</code> is a valid JSON expression without the <code>$</code> prefix.<br/>selectionPattern:  Specifies the regular expression (regex) pattern used to choose an integration response based on the response from the back end. For example, if the success response returns nothing and the error response returns some string, you could use the <code>.+</code> regex to match error response. However, make sure that the error response does not contain any newline (<code>\n</code>) character in such cases. If the back end is an AWS Lambda function, the AWS Lambda function error header is matched. For all other HTTP and AWS back ends, the HTTP status code is matched.<br/>statusCode:  Specifies the status code that is used to map the integration response to an existing <a>MethodResponse</a>.<br/>        """
        pass

    def put_method(self, authorizationType: str, httpMethod: str, resourceId: str, restApiId: str, authorizerId: str=None, apiKeyRequired: Boolean=None, operationName: str=None, requestParameters: MapOfStringToBoolean=None, requestModels: MapOfStringToString=None, requestValidatorId: str=None, authorizationScopes: List[String]=None) -> Method:
        """Add a method to an existing <a>Resource</a> resource.

        :param self:
        :param authorizationScopes:  A list of authorization scopes configured on the method. The scopes are used with a <code>COGNITO_USER_POOLS</code> authorizer to authorize the method invocation. The authorization works by matching the method scopes against the scopes parsed from the access token in the incoming request. The method invocation is authorized if any method scopes matches a claimed scope in the access token. Otherwise, the invocation is not authorized. When the method scope is configured, the client must provide an access token instead of an identity token for authorization purposes.<br/>requestValidatorId:  The identifier of a <a>RequestValidator</a> for validating the method request.<br/>requestModels:  Specifies the <a>Model</a> resources used for the request's content type. Request models are represented as a key/value map, with a content type as the key and a <a>Model</a> name as the value.<br/>requestParameters:  A key-value map defining required or optional method request parameters that can be accepted by API Gateway. A key defines a method request parameter name matching the pattern of <code>method.request.{location}.{name}</code>, where <code>location</code> is <code>querystring</code>, <code>path</code>, or <code>header</code> and <code>name</code> is a valid and unique parameter name. The value associated with the key is a Boolean flag indicating whether the parameter is required (<code>true</code>) or optional (<code>false</code>). The method request parameter names defined here are available in <a>Integration</a> to be mapped to integration request parameters or body-mapping templates.<br/>operationName:  A human-friendly operation identifier for the method. For example, you can assign the <code>operationName</code> of <code>ListPets</code> for the <code>GET /pets</code> method in the <code>PetStore</code> example.<br/>apiKeyRequired:  Specifies whether the method required a valid <a>ApiKey</a>.<br/>authorizerId:  Specifies the identifier of an <a>Authorizer</a> to use on this Method, if the type is CUSTOM or COGNITO_USER_POOLS. The authorizer identifier is generated by API Gateway when you created the authorizer.<br/>authorizationType:  [Required] The method's authorization type. Valid values are <code>NONE</code> for open access, <code>AWS_IAM</code> for using AWS IAM permissions, <code>CUSTOM</code> for using a custom authorizer, or <code>COGNITO_USER_POOLS</code> for using a Cognito user pool.<br/>httpMethod:  [Required] Specifies the method request's HTTP method type.<br/>resourceId:  [Required] The <a>Resource</a> identifier for the new <a>Method</a> resource.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: authorizationScopes:  A list of authorization scopes configured on the method. The scopes are used with a <code>COGNITO_USER_POOLS</code> authorizer to authorize the method invocation. The authorization works by matching the method scopes against the scopes parsed from the access token in the incoming request. The method invocation is authorized if any method scopes matches a claimed scope in the access token. Otherwise, the invocation is not authorized. When the method scope is configured, the client must provide an access token instead of an identity token for authorization purposes.<br/>methodIntegration:  Gets the method's integration responsible for passing the client-submitted request to the back end and performing necessary transformations to make the request compliant with the back end. <div class="remarks"> <p/> <h4>Example: </h4> <h5>Request</h5> <p/> <pre><code>GET /restapis/uojnr9hd57/resources/0cjtch/methods/GET/integration HTTP/1.1 Content-Type: application/json Host: apigateway.us-east-1.amazonaws.com Content-Length: 117 X-Amz-Date: 20160613T213210Z Authorization: AWS4-HMAC-SHA256 Credential={access_key_ID}/20160613/us-east-1/apigateway/aws4_request, SignedHeaders=content-type;host;x-amz-date, Signature={sig4_hash}</code></pre> <h5>Response</h5> The successful response returns a <code>200 OK</code> status code and a payload similar to the following: <pre><code>{ "_links": { "curies": [ { "href": "https://docs.aws.amazon.com/apigateway/latest/developerguide/restapi-integration-{rel}.html", "name": "integration", "templated": true }, { "href": "https://docs.aws.amazon.com/apigateway/latest/developerguide/restapi-integration-response-{rel}.html", "name": "integrationresponse", "templated": true } ], "self": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/integration" }, "integration:delete": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/integration" }, "integration:responses": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/integration/responses/200", "name": "200", "title": "200" }, "integration:update": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/integration" }, "integrationresponse:put": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/integration/responses/{status_code}", "templated": true } }, "cacheKeyParameters": [], "cacheNamespace": "0cjtch", "credentials": "arn:aws:iam::123456789012:role/apigAwsProxyRole", "httpMethod": "POST", "passthroughBehavior": "WHEN_NO_MATCH", "requestTemplates": { "application/json": "{\n \"a\": \"$input.params('operand1')\",\n \"b\": \"$input.params('operand2')\", \n \"op\": \"$input.params('operator')\" \n}" }, "type": "AWS", "uri": "arn:aws:apigateway:us-west-2:lambda:path//2015-03-31/functions/arn:aws:lambda:us-west-2:123456789012:function:Calc/invocations", "_embedded": { "integration:responses": { "_links": { "self": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/integration/responses/200", "name": "200", "title": "200" }, "integrationresponse:delete": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/integration/responses/200" }, "integrationresponse:update": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/integration/responses/200" } }, "responseParameters": { "method.response.header.operator": "integration.response.body.op", "method.response.header.operand_2": "integration.response.body.b", "method.response.header.operand_1": "integration.response.body.a" }, "responseTemplates": { "application/json": "#set($res = $input.path('$'))\n{\n \"result\": \"$res.a, $res.b, $res.op => $res.c\",\n \"a\" : \"$res.a\",\n \"b\" : \"$res.b\",\n \"op\" : \"$res.op\",\n \"c\" : \"$res.c\"\n}" }, "selectionPattern": "", "statusCode": "200" } } }</code></pre> <p/> </div> <div class="seeAlso"> <a href="https://docs.aws.amazon.com/cli/latest/reference/apigateway/get-integration.html">AWS CLI</a> </div>:<br/>methodResponses:  Gets a method response associated with a given HTTP status code.  <div class="remarks"> The collection of method responses are encapsulated in a key-value map, where the key is a response's HTTP status code and the value is a <a>MethodResponse</a> resource that specifies the response returned to the caller from the back end through the integration response. <h4>Example: Get a 200 OK response of a GET method</h4> <h5>Request</h5> <p/> <pre><code>GET /restapis/uojnr9hd57/resources/0cjtch/methods/GET/responses/200 HTTP/1.1 Content-Type: application/json Host: apigateway.us-east-1.amazonaws.com Content-Length: 117 X-Amz-Date: 20160613T215008Z Authorization: AWS4-HMAC-SHA256 Credential={access_key_ID}/20160613/us-east-1/apigateway/aws4_request, SignedHeaders=content-type;host;x-amz-date, Signature={sig4_hash}</code></pre> <h5>Response</h5> The successful response returns a <code>200 OK</code> status code and a payload similar to the following: <pre><code>{ "_links": { "curies": { "href": "https://docs.aws.amazon.com/apigateway/latest/developerguide/restapi-method-response-{rel}.html", "name": "methodresponse", "templated": true }, "self": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/responses/200", "title": "200" }, "methodresponse:delete": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/responses/200" }, "methodresponse:update": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/responses/200" } }, "responseModels": { "application/json": "Empty" }, "responseParameters": { "method.response.header.operator": false, "method.response.header.operand_2": false, "method.response.header.operand_1": false }, "statusCode": "200" }</code></pre> <p/> </div> <div class="seeAlso"> <a href="https://docs.aws.amazon.com/cli/latest/reference/apigateway/get-method-response.html">AWS CLI</a> </div><br/>requestModels:  A key-value map specifying data schemas, represented by <a>Model</a> resources, (as the mapped value) of the request payloads of given content types (as the mapping key).<br/>requestParameters:  A key-value map defining required or optional method request parameters that can be accepted by API Gateway. A key is a method request parameter name matching the pattern of <code>method.request.{location}.{name}</code>, where <code>location</code> is <code>querystring</code>, <code>path</code>, or <code>header</code> and <code>name</code> is a valid and unique parameter name. The value associated with the key is a Boolean flag indicating whether the parameter is required (<code>true</code>) or optional (<code>false</code>). The method request parameter names defined here are available in <a>Integration</a> to be mapped to integration request parameters or templates.<br/>operationName:  A human-friendly operation identifier for the method. For example, you can assign the <code>operationName</code> of <code>ListPets</code> for the <code>GET /pets</code> method in the <code>PetStore</code> example.<br/>requestValidatorId:  The identifier of a <a>RequestValidator</a> for request validation.<br/>apiKeyRequired:  A boolean flag specifying whether a valid <a>ApiKey</a> is required to invoke this method.<br/>authorizerId:  The identifier of an <a>Authorizer</a> to use on this method. The <code>authorizationType</code> must be <code>CUSTOM</code>.<br/>authorizationType:  The method's authorization type. Valid values are <code>NONE</code> for open access, <code>AWS_IAM</code> for using AWS IAM permissions, <code>CUSTOM</code> for using a custom authorizer, or <code>COGNITO_USER_POOLS</code> for using a Cognito user pool.<br/>httpMethod:  The method's HTTP verb.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>integrationResponses</i> Specifies the integration's responses. <div class="remarks"> <p/> <h4>Example: Get integration responses of a method</h4> <h5>Request</h5> <p/> <pre><code>GET /restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200 HTTP/1.1 Content-Type: application/json Host: apigateway.us-east-1.amazonaws.com X-Amz-Date: 20160607T191449Z Authorization: AWS4-HMAC-SHA256 Credential={access_key_ID}/20160607/us-east-1/apigateway/aws4_request, SignedHeaders=content-type;host;x-amz-date, Signature={sig4_hash} </code></pre> <h5>Response</h5> The successful response returns <code>200 OK</code> status and a payload as follows: <pre><code>{ "_links": { "curies": { "href": "https://docs.aws.amazon.com/apigateway/latest/developerguide/restapi-integration-response-{rel}.html", "name": "integrationresponse", "templated": true }, "self": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200", "title": "200" }, "integrationresponse:delete": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200" }, "integrationresponse:update": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200" } }, "responseParameters": { "method.response.header.Content-Type": "'application/xml'" }, "responseTemplates": { "application/json": "$util.urlDecode(\"%3CkinesisStreams%3E#foreach($stream in $input.path('$.StreamNames'))%3Cstream%3E%3Cname%3E$stream%3C/name%3E%3C/stream%3E#end%3C/kinesisStreams%3E\")\n" }, "statusCode": "200" }</code></pre> <p/> </div> <div class="seeAlso"> <a href="https://docs.aws.amazon.com/apigateway/latest/developerguide/how-to-create-api.html">Creating an API</a> </div><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cacheKeyParameters</i> A list of request parameters whose values API Gateway caches. To be valid values for <code>cacheKeyParameters</code>, these parameters must also be specified for <a>Method</a> <code>requestParameters</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cacheNamespace</i> An API-specific tag group of related cached parameters. To be valid values for <code>cacheKeyParameters</code>, these parameters must also be specified for <a>Method</a> <code>requestParameters</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>timeoutInMillis</i> Custom timeout between 50 and 29,000 milliseconds. The default value is 29,000 milliseconds or 29 seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>contentHandling</i> Specifies how to handle request payload content type conversions. Supported values are <code>CONVERT_TO_BINARY</code> and <code>CONVERT_TO_TEXT</code>, with the following behaviors: <ul> <li><code>CONVERT_TO_BINARY</code>: Converts a request payload from a Base64-encoded string to the corresponding binary blob.</li> <li><code>CONVERT_TO_TEXT</code>: Converts a request payload from a binary blob to a Base64-encoded string.</li> </ul> If this property is not defined, the request payload will be passed through from the method request to integration request without modification, provided that the <code>passthroughBehavior</code> is configured to support payload pass-through.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>passthroughBehavior</i> <div>  Specifies how the method request body of an unmapped content type will be passed through the integration request to the back end without transformation. A content type is unmapped if no mapping template is defined in the integration or the content type does not match any of the mapped content types, as specified in <code>requestTemplates</code>. The valid value is one of the following:  <ul> <li> <code>WHEN_NO_MATCH</code>: passes the method request body through the integration request to the back end without transformation when the method request content type does not match any content type associated with the mapping templates defined in the integration request. </li> <li> <code>WHEN_NO_TEMPLATES</code>: passes the method request body through the integration request to the back end without transformation when no mapping template is defined in the integration request. If a template is defined when this option is selected, the method request of an unmapped content-type will be rejected with an HTTP <code>415 Unsupported Media Type</code> response. </li> <li> <code>NEVER</code>: rejects the method request with an HTTP <code>415 Unsupported Media Type</code> response when either the method request content type does not match any content type associated with the mapping templates defined in the integration request or no mapping template is defined in the integration request. </li> </ul> </div><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>requestTemplates</i> Represents a map of Velocity templates that are applied on the request payload based on the value of the Content-Type header sent by the client. The content type value is the key in this map, and the template (as a String) is the value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>requestParameters</i> A key-value map specifying request parameters that are passed from the method request to the back end. The key is an integration request parameter name and the associated value is a method request parameter value or static value that must be enclosed within single quotes and pre-encoded as required by the back end. The method request parameter value must match the pattern of <code>method.request.{location}.{name}</code>, where <code>location</code> is <code>querystring</code>, <code>path</code>, or <code>header</code> and <code>name</code> must be a valid and unique method request parameter name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>credentials</i> Specifies the credentials required for the integration, if any. For AWS integrations, three options are available. To specify an IAM Role for API Gateway to assume, use the role's Amazon Resource Name (ARN). To require that the caller's identity be passed through from the request, specify the string <code>arn:aws:iam::\*:user/\*</code>. To use resource-based permissions on supported AWS services, specify null.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>connectionId</i> The (<a href="https://docs.aws.amazon.com/apigateway/api-reference/resource/vpc-link/#id"><code>id</code></a>) of the <a>VpcLink</a> used for the integration when <code>connectionType=VPC_LINK</code> and undefined, otherwise.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>connectionType</i> The type of the network connection to the integration endpoint. The valid value is <code>INTERNET</code> for connections through the public routable internet or <code>VPC_LINK</code> for private connections between API Gateway and a network load balancer in a VPC. The default value is <code>INTERNET</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>uri</i> Specifies Uniform Resource Identifier (URI) of the integration endpoint. <ul> <li> For <code>HTTP</code> or <code>HTTP_PROXY</code> integrations, the URI must be a fully formed, encoded HTTP(S) URL according to the <a target="_blank" href="https://en.wikipedia.org/wiki/Uniform_Resource_Identifier">RFC-3986 specification</a>, for either standard integration, where <code>connectionType</code> is not <code>VPC_LINK</code>, or private integration, where <code>connectionType</code> is <code>VPC_LINK</code>. For a private HTTP integration, the URI is not used for routing.  </li> <li> For <code>AWS</code> or <code>AWS_PROXY</code> integrations, the URI is of the form <code>arn:aws:apigateway:{region}:{subdomain.service|service}:path|action/{service_api}</code>. Here, <code>{Region}</code> is the API Gateway region (e.g., <code>us-east-1</code>); <code>{service}</code> is the name of the integrated AWS service (e.g., <code>s3</code>); and <code>{subdomain}</code> is a designated subdomain supported by certain AWS service for fast host-name lookup. <code>action</code> can be used for an AWS service action-based API, using an <code>Action={name}&amp;{p1}={v1}&amp;p2={v2}...</code> query string. The ensuing <code>{service_api}</code> refers to a supported action <code>{name}</code> plus any required input parameters. Alternatively, <code>path</code> can be used for an AWS service path-based API. The ensuing <code>service_api</code> refers to the path to an AWS service resource, including the region of the integrated AWS service, if applicable. For example, for integration with the S3 API of <code><a href="https://docs.aws.amazon.com/AmazonS3/latest/API/RESTObjectGET.html">GetObject</a></code>, the <code>uri</code> can be either <code>arn:aws:apigateway:us-west-2:s3:action/GetObject&amp;Bucket={bucket}&amp;Key={key}</code> or <code>arn:aws:apigateway:us-west-2:s3:path/{bucket}/{key}</code> </li></ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>httpMethod</i> Specifies the integration's HTTP method type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> Specifies an API method integration type. The valid value is one of the following: <ul> <li><code>AWS</code>: for integrating the API method request with an AWS service action, including the Lambda function-invoking action. With the Lambda function-invoking action, this is referred to as the Lambda custom integration. With any other AWS service action, this is known as AWS integration.</li> <li><code>AWS_PROXY</code>: for integrating the API method request with the Lambda function-invoking action with the client request passed through as-is. This integration is also referred to as the Lambda proxy integration.</li> <li><code>HTTP</code>: for integrating the API method request with an HTTP endpoint, including a private HTTP endpoint within a VPC. This integration is also referred to as the HTTP custom integration.</li> <li><code>HTTP_PROXY</code>: for integrating the API method request with an HTTP endpoint, including a private HTTP endpoint within a VPC, with the client request passed through as-is. This is also referred to as the HTTP proxy integration.</li> <li><code>MOCK</code>: for integrating the API method request with API Gateway as a "loop-back" endpoint without invoking any backend.</li> </ul> For the HTTP and HTTP proxy integrations, each integration can specify a protocol (<code>http/https</code>), port and path. Standard 80 and 443 ports are supported as well as custom ports above 1024. An HTTP or HTTP proxy integration with a <code>connectionType</code> of <code>VPC_LINK</code> is referred to as a private integration and uses a <a>VpcLink</a> to connect API Gateway to a network load balancer of a VPC.<br/>        """
        pass

    def put_method_response(self, statusCode: str, httpMethod: str, resourceId: str, restApiId: str, responseParameters: MapOfStringToBoolean=None, responseModels: MapOfStringToString=None) -> MethodResponse:
        """Adds a <a>MethodResponse</a> to an existing <a>Method</a> resource.

        :param self:
        :param responseModels:  Specifies the <a>Model</a> resources used for the response's content type. Response models are represented as a key/value map, with a content type as the key and a <a>Model</a> name as the value.<br/>responseParameters:  A key-value map specifying required or optional response parameters that API Gateway can send back to the caller. A key defines a method response header name and the associated value is a Boolean flag indicating whether the method response parameter is required or not. The method response header names must match the pattern of <code>method.response.header.{name}</code>, where <code>name</code> is a valid and unique header name. The response parameter names defined here are available in the integration response to be mapped from an integration response header expressed in <code>integration.response.header.{name}</code>, a static value enclosed within a pair of single quotes (e.g., <code>'application/json'</code>), or a JSON expression from the back-end response payload in the form of <code>integration.response.body.{JSON-expression}</code>, where <code>JSON-expression</code> is a valid JSON expression without the <code>$</code> prefix.)<br/>statusCode:  [Required] The method response's status code.<br/>httpMethod:  [Required] The HTTP verb of the <a>Method</a> resource.<br/>resourceId:  [Required] The <a>Resource</a> identifier for the <a>Method</a> resource.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: responseModels:  Specifies the <a>Model</a> resources used for the response's content-type. Response models are represented as a key/value map, with a content-type as the key and a <a>Model</a> name as the value.<br/>responseParameters:  A key-value map specifying required or optional response parameters that API Gateway can send back to the caller. A key defines a method response header and the value specifies whether the associated method response header is required or not. The expression of the key must match the pattern <code>method.response.header.{name}</code>, where <code>name</code> is a valid and unique header name. API Gateway passes certain integration response data to the method response headers specified here according to the mapping you prescribe in the API's <a>IntegrationResponse</a>. The integration response data that can be mapped include an integration response header expressed in <code>integration.response.header.{name}</code>, a static value enclosed within a pair of single quotes (e.g., <code>'application/json'</code>), or a JSON expression from the back-end response payload in the form of <code>integration.response.body.{JSON-expression}</code>, where <code>JSON-expression</code> is a valid JSON expression without the <code>$</code> prefix.)<br/>statusCode:  The method response's status code.<br/>        """
        pass

    def put_rest_api(self, body: Blob, restApiId: str, mode: str=None, failOnWarnings: Boolean=None, parameters: MapOfStringToString=None) -> RestApi:
        """A feature of the API Gateway control service for updating an existing API with an input of external API definitions. The update can take the form of merging the supplied definition into the existing API or overwriting the existing API.

        :param self:
        :param body:  [Required] The PUT request body containing external API definitions. Currently, only OpenAPI definition JSON/YAML files are supported. The maximum size of the API definition file is 2MB.<br/>parameters:  Custom header parameters as part of the request. For example, to exclude <a>DocumentationParts</a> from an imported API, set <code>ignore=documentation</code> as a <code>parameters</code> value, as in the AWS CLI command of <code>aws apigateway import-rest-api --parameters ignore=documentation --body 'file:///path/to/imported-api-body.json'</code>.<br/>failOnWarnings:  A query parameter to indicate whether to rollback the API update (<code>true</code>) or not (<code>false</code>) when a warning is encountered. The default value is <code>false</code>.<br/>mode:  The <code>mode</code> query parameter to specify the update mode. Valid values are "merge" and "overwrite". By default, the update mode is "merge".<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: tags:  The collection of tags. Each tag element is associated with a given resource.<br/>policy:  A stringified JSON policy document that applies to this RestApi regardless of the caller and <a>Method</a> configuration.<br/>endpointConfiguration:  The endpoint configuration of this <a>RestApi</a> showing the endpoint types of the API. :<br/>apiKeySource:  The source of the API key for metering requests according to a usage plan. Valid values are: <ul><li><code>HEADER</code> to read the API key from the <code>X-API-Key</code> header of a request. </li><li><code>AUTHORIZER</code> to read the API key from the <code>UsageIdentifierKey</code> from a custom authorizer.</li></ul> <br/>minimumCompressionSize:  A nullable integer that is used to enable compression (with non-negative between 0 and 10485760 (10M) bytes, inclusive) or disable compression (with a null value) on an API. When compression is enabled, compression or decompression is not applied on the payload if the payload size is smaller than this value. Setting it to zero allows compression for any payload size.<br/>binaryMediaTypes:  The list of binary media types supported by the <a>RestApi</a>. By default, the <a>RestApi</a> supports only UTF-8-encoded text payloads.<br/>warnings:  The warning messages reported when <code>failonwarnings</code> is turned on during API import.<br/>version:  A version identifier for the API.<br/>createdDate:  The timestamp when the API was created.<br/>description:  The API's description.<br/>name:  The API's name.<br/>id:  The API's identifier. This identifier is unique across all of your APIs in API Gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>types</i> A list of endpoint types of an API (<a>RestApi</a>) or its custom domain name (<a>DomainName</a>). For an edge-optimized API and its custom domain name, the endpoint type is <code>"EDGE"</code>. For a regional API and its custom domain name, the endpoint type is <code>REGIONAL</code>. For a private API, the endpoint type is <code>PRIVATE</code>.<br/>        """
        pass

    def tag_resource(self, tags: MapOfStringToString, resourceArn: str):
        """Adds or updates a tag on a given resource.

        :param self:
        :param tags:  [Required] The key-value map of strings. The valid character set is [a-zA-Z+-=._:/]. The tag key can be up to 128 characters and must not start with <code>aws:</code>. The tag value can be up to 256 characters.<br/>resourceArn:  [Required] The ARN of a resource that can be tagged. The resource ARN must be URL-encoded.<br/>
        :return:         """
        pass

    def test_invoke_authorizer(self, authorizerId: str, restApiId: str, headers: MapOfStringToString=None, multiValueHeaders: MapOfStringToList=None, pathWithQueryString: str=None, body: str=None, stageVariables: MapOfStringToString=None, additionalContext: MapOfStringToString=None) -> TestInvokeAuthorizerResponse:
        """Simulate the execution of an <a>Authorizer</a> in your <a>RestApi</a> with headers, parameters, and an incoming request body. <div class="seeAlso"> <a href="https://docs.aws.amazon.com/apigateway/latest/developerguide/apigateway-use-lambda-authorizer.html">Use Lambda Function as Authorizer</a> <a href="https://docs.aws.amazon.com/apigateway/latest/developerguide/apigateway-integrate-with-cognito.html">Use Cognito User Pool as Authorizer</a> </div>

        :param self:
        :param additionalContext:  [Optional] A key-value map of additional context variables.<br/>stageVariables:  A key-value map of stage variables to simulate an invocation on a deployed <a>Stage</a>.<br/>body:  [Optional] The simulated request body of an incoming invocation request.<br/>pathWithQueryString:  [Optional] The URI path, including query string, of the simulated invocation request. Use this to specify path parameters and query string parameters.<br/>multiValueHeaders:  [Optional] The headers as a map from string to list of values to simulate an incoming invocation request. This is where the incoming authorization token, or identity source, may be specified.<br/>headers:  [Required] A key-value map of headers to simulate an incoming invocation request. This is where the incoming authorization token, or identity source, should be specified.<br/>authorizerId:  [Required] Specifies a test invoke authorizer request's <a>Authorizer</a> ID.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: claims:  The <a href="https://openid.net/specs/openid-connect-core-1_0.html#StandardClaims">open identity claims</a>, with any supported custom attributes, returned from the Cognito Your User Pool configured for the API.<br/>authorization:  <br/>policy:  The JSON policy document returned by the <a>Authorizer</a><br/>principalId:  The principal identity returned by the <a>Authorizer</a><br/>latency:  The execution latency of the test authorizer request.<br/>log:  The API Gateway execution log for the test authorizer request.<br/>clientStatus:  The HTTP status code that the client would have received. Value is 0 if the authorizer succeeded.<br/>        """
        pass

    def test_invoke_method(self, httpMethod: str, resourceId: str, restApiId: str, pathWithQueryString: str=None, body: str=None, headers: MapOfStringToString=None, multiValueHeaders: MapOfStringToList=None, clientCertificateId: str=None, stageVariables: MapOfStringToString=None) -> TestInvokeMethodResponse:
        """Simulate the execution of a <a>Method</a> in your <a>RestApi</a> with headers, parameters, and an incoming request body.

        :param self:
        :param stageVariables:  A key-value map of stage variables to simulate an invocation on a deployed <a>Stage</a>.<br/>clientCertificateId:  A <a>ClientCertificate</a> identifier to use in the test invocation. API Gateway will use the certificate when making the HTTPS request to the defined back-end endpoint.<br/>multiValueHeaders:  The headers as a map from string to list of values to simulate an incoming invocation request.<br/>headers:  A key-value map of headers to simulate an incoming invocation request.<br/>body:  The simulated request body of an incoming invocation request.<br/>pathWithQueryString:  The URI path, including query string, of the simulated invocation request. Use this to specify path parameters and query string parameters.<br/>httpMethod:  [Required] Specifies a test invoke method request's HTTP method.<br/>resourceId:  [Required] Specifies a test invoke method request's resource ID.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: latency:  The execution latency of the test invoke request.<br/>log:  The API Gateway execution log for the test invoke request.<br/>multiValueHeaders:  The headers of the HTTP response as a map from string to list of values.<br/>headers:  The headers of the HTTP response.<br/>body:  The body of the HTTP response.<br/>status:  The HTTP status code.<br/>        """
        pass

    def untag_resource(self, tagKeys: List[String], resourceArn: str):
        """Removes a tag from a given resource.

        :param self:
        :param tagKeys:  [Required] The Tag keys to delete.<br/>resourceArn:  [Required] The ARN of a resource that can be tagged. The resource ARN must be URL-encoded.<br/>
        :return:         """
        pass

    def update_account(self, patchOperations: List[PatchOperation]=None) -> Account:
        """Changes information about the current <a>Account</a> resource.

        :param self:
        :param patchOperations:  A list of update operations to be applied to the specified resource and in the order specified in this list.<br/>
        :return: apiKeyVersion:  The version of the API keys used for the account.<br/>features:  A list of features supported for the account. When usage plans are enabled, the features list will include an entry of <code>"UsagePlans"</code>.<br/>throttleSettings:  Specifies the API request limits configured for the current <a>Account</a>.:<br/>cloudwatchRoleArn:  The ARN of an Amazon CloudWatch role for the current <a>Account</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>rateLimit</i> The API request steady-state rate limit.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>burstLimit</i> The API request burst limit, the maximum rate limit over a time ranging from one to a few seconds, depending upon whether the underlying token bucket is at its full capacity.<br/>        """
        pass

    def update_api_key(self, apiKey: str, patchOperations: List[PatchOperation]=None) -> ApiKey:
        """Changes information about an <a>ApiKey</a> resource.

        :param self:
        :param patchOperations:  A list of update operations to be applied to the specified resource and in the order specified in this list.<br/>apiKey:  [Required] The identifier of the <a>ApiKey</a> resource to be updated.<br/>
        :return: tags:  The collection of tags. Each tag element is associated with a given resource.<br/>stageKeys:  A list of <a>Stage</a> resources that are associated with the <a>ApiKey</a> resource.<br/>lastUpdatedDate:  The timestamp when the API Key was last updated.<br/>createdDate:  The timestamp when the API Key was created.<br/>enabled:  Specifies whether the API Key can be used by callers.<br/>description:  The description of the API Key.<br/>customerId:  An AWS Marketplace customer identifier , when integrating with the AWS SaaS Marketplace.<br/>name:  The name of the API Key.<br/>value:  The value of the API Key.<br/>id:  The identifier of the API Key.<br/>        """
        pass

    def update_authorizer(self, authorizerId: str, restApiId: str, patchOperations: List[PatchOperation]=None) -> Authorizer:
        """Updates an existing <a>Authorizer</a> resource. <div class="seeAlso"><a href="https://docs.aws.amazon.com/cli/latest/reference/apigateway/update-authorizer.html">AWS CLI</a></div>

        :param self:
        :param patchOperations:  A list of update operations to be applied to the specified resource and in the order specified in this list.<br/>authorizerId:  [Required] The identifier of the <a>Authorizer</a> resource.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: authorizerResultTtlInSeconds:  The TTL in seconds of cached authorizer results. If it equals 0, authorization caching is disabled. If it is greater than 0, API Gateway will cache authorizer responses. If this field is not set, the default value is 300. The maximum value is 3600, or 1 hour.<br/>identityValidationExpression:  A validation expression for the incoming identity token. For <code>TOKEN</code> authorizers, this value is a regular expression. For <code>COGNITO_USER_POOLS</code> authorizers, API Gateway will match the <code>aud</code> field of the incoming token from the client against the specified regular expression. It will invoke the authorizer's Lambda function when there is a match. Otherwise, it will return a 401 Unauthorized response without calling the Lambda function. The validation expression does not apply to the <code>REQUEST</code> authorizer.<br/>identitySource:  The identity source for which authorization is requested. <ul><li>For a <code>TOKEN</code> or <code>COGNITO_USER_POOLS</code> authorizer, this is required and specifies the request header mapping expression for the custom header holding the authorization token submitted by the client. For example, if the token header name is <code>Auth</code>, the header mapping expression is <code>method.request.header.Auth</code>.</li><li>For the <code>REQUEST</code> authorizer, this is required when authorization caching is enabled. The value is a comma-separated string of one or more mapping expressions of the specified request parameters. For example, if an <code>Auth</code> header, a <code>Name</code> query string parameter are defined as identity sources, this value is <code>method.request.header.Auth, method.request.querystring.Name</code>. These parameters will be used to derive the authorization caching key and to perform runtime validation of the <code>REQUEST</code> authorizer by verifying all of the identity-related request parameters are present, not null and non-empty. Only when this is true does the authorizer invoke the authorizer Lambda function, otherwise, it returns a 401 Unauthorized response without calling the Lambda function. The valid value is a string of comma-separated mapping expressions of the specified request parameters. When the authorization caching is not enabled, this property is optional.</li></ul><br/>authorizerCredentials:  Specifies the required credentials as an IAM role for API Gateway to invoke the authorizer. To specify an IAM role for API Gateway to assume, use the role's Amazon Resource Name (ARN). To use resource-based permissions on the Lambda function, specify null.<br/>authorizerUri:  Specifies the authorizer's Uniform Resource Identifier (URI). For <code>TOKEN</code> or <code>REQUEST</code> authorizers, this must be a well-formed Lambda function URI, for example, <code>arn:aws:apigateway:us-west-2:lambda:path/2015-03-31/functions/arn:aws:lambda:us-west-2:{account_id}:function:{lambda_function_name}/invocations</code>. In general, the URI has this form <code>arn:aws:apigateway:{region}:lambda:path/{service_api}</code>, where <code>{region}</code> is the same as the region hosting the Lambda function, <code>path</code> indicates that the remaining substring in the URI should be treated as the path to the resource, including the initial <code>/</code>. For Lambda functions, this is usually of the form <code>/2015-03-31/functions/[FunctionARN]/invocations</code>.<br/>authType:  Optional customer-defined field, used in OpenAPI imports and exports without functional impact.<br/>providerARNs:  A list of the Amazon Cognito user pool ARNs for the <code>COGNITO_USER_POOLS</code> authorizer. Each element is of this format: <code>arn:aws:cognito-idp:{region}:{account_id}:userpool/{user_pool_id}</code>. For a <code>TOKEN</code> or <code>REQUEST</code> authorizer, this is not defined. <br/>type:  The authorizer type. Valid values are <code>TOKEN</code> for a Lambda function using a single authorization token submitted in a custom header, <code>REQUEST</code> for a Lambda function using incoming request parameters, and <code>COGNITO_USER_POOLS</code> for using an Amazon Cognito user pool.<br/>name:  [Required] The name of the authorizer.<br/>id:  The identifier for the authorizer resource.<br/>        """
        pass

    def update_base_path_mapping(self, basePath: str, domainName: str, patchOperations: List[PatchOperation]=None) -> BasePathMapping:
        """Changes information about the <a>BasePathMapping</a> resource.

        :param self:
        :param patchOperations:  A list of update operations to be applied to the specified resource and in the order specified in this list.<br/>basePath:  [Required] The base path of the <a>BasePathMapping</a> resource to change. To specify an empty base path, set this parameter to <code>'(none)'</code>.<br/>domainName:  [Required] The domain name of the <a>BasePathMapping</a> resource to change.<br/>
        :return: stage:  The name of the associated stage.<br/>restApiId:  The string identifier of the associated <a>RestApi</a>.<br/>basePath:  The base path name that callers of the API must provide as part of the URL after the domain name.<br/>        """
        pass

    def update_client_certificate(self, clientCertificateId: str, patchOperations: List[PatchOperation]=None) -> ClientCertificate:
        """Changes information about an <a>ClientCertificate</a> resource.

        :param self:
        :param patchOperations:  A list of update operations to be applied to the specified resource and in the order specified in this list.<br/>clientCertificateId:  [Required] The identifier of the <a>ClientCertificate</a> resource to be updated.<br/>
        :return: tags:  The collection of tags. Each tag element is associated with a given resource.<br/>expirationDate:  The timestamp when the client certificate will expire.<br/>createdDate:  The timestamp when the client certificate was created.<br/>pemEncodedCertificate:  The PEM-encoded public key of the client certificate, which can be used to configure certificate authentication in the integration endpoint .<br/>description:  The description of the client certificate.<br/>clientCertificateId:  The identifier of the client certificate.<br/>        """
        pass

    def update_deployment(self, deploymentId: str, restApiId: str, patchOperations: List[PatchOperation]=None) -> Deployment:
        """Changes information about a <a>Deployment</a> resource.

        :param self:
        :param patchOperations:  A list of update operations to be applied to the specified resource and in the order specified in this list.<br/>deploymentId:  The replacement identifier for the <a>Deployment</a> resource to change information about.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: apiSummary:  A summary of the <a>RestApi</a> at the date and time that the deployment resource was created.<br/>createdDate:  The date and time that the deployment resource was created.<br/>description:  The description for the deployment resource.<br/>id:  The identifier for the deployment resource.<br/>        """
        pass

    def update_documentation_part(self, documentationPartId: str, restApiId: str, patchOperations: List[PatchOperation]=None) -> DocumentationPart:
        """

        :param self:
        :param patchOperations:  A list of update operations to be applied to the specified resource and in the order specified in this list.<br/>documentationPartId:  [Required] The identifier of the to-be-updated documentation part.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: properties:  A content map of API-specific key-value pairs describing the targeted API entity. The map must be encoded as a JSON string, e.g., <code>"{ \"description\": \"The API does ...\" }"</code>. Only OpenAPI-compliant documentation-related fields from the <literal>properties</literal> map are exported and, hence, published as part of the API entity definitions, while the original documentation parts are exported in a OpenAPI extension of <code>x-amazon-apigateway-documentation</code>.<br/>location:  The location of the API entity to which the documentation applies. Valid fields depend on the targeted API entity type. All the valid location fields are not required. If not explicitly specified, a valid location field is treated as a wildcard and associated documentation content may be inherited by matching entities, unless overridden.:<br/>id:  The <a>DocumentationPart</a> identifier, generated by API Gateway when the <code>DocumentationPart</code> is created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the targeted API entity. It is a valid and required field for the API entity types of <code>AUTHORIZER</code>, <code>MODEL</code>, <code>PATH_PARAMETER</code>, <code>QUERY_PARAMETER</code>, <code>REQUEST_HEADER</code>, <code>REQUEST_BODY</code> and <code>RESPONSE_HEADER</code>. It is an invalid field for any other entity type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusCode</i> The HTTP status code of a response. It is a valid field for the API entity types of <code>RESPONSE</code>, <code>RESPONSE_HEADER</code>, and <code>RESPONSE_BODY</code>. The default value is <code>*</code> for any status code. When an applicable child entity inherits the content of an entity of the same type with more general specifications of the other <code>location</code> attributes, the child entity's <code>statusCode</code> attribute must match that of the parent entity exactly.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>method</i> The HTTP verb of a method. It is a valid field for the API entity types of <code>METHOD</code>, <code>PATH_PARAMETER</code>, <code>QUERY_PARAMETER</code>, <code>REQUEST_HEADER</code>, <code>REQUEST_BODY</code>, <code>RESPONSE</code>, <code>RESPONSE_HEADER</code>, and <code>RESPONSE_BODY</code>. The default value is <code>*</code> for any method. When an applicable child entity inherits the content of an entity of the same type with more general specifications of the other <code>location</code> attributes, the child entity's <code>method</code> attribute must match that of the parent entity exactly.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>path</i> The URL path of the target. It is a valid field for the API entity types of <code>RESOURCE</code>, <code>METHOD</code>, <code>PATH_PARAMETER</code>, <code>QUERY_PARAMETER</code>, <code>REQUEST_HEADER</code>, <code>REQUEST_BODY</code>, <code>RESPONSE</code>, <code>RESPONSE_HEADER</code>, and <code>RESPONSE_BODY</code>. The default value is <code>/</code> for the root resource. When an applicable child entity inherits the content of another entity of the same type with more general specifications of the other <code>location</code> attributes, the child entity's <code>path</code> attribute must match that of the parent entity as a prefix.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> [Required] The type of API entity to which the documentation content applies. Valid values are <code>API</code>, <code>AUTHORIZER</code>, <code>MODEL</code>, <code>RESOURCE</code>, <code>METHOD</code>, <code>PATH_PARAMETER</code>, <code>QUERY_PARAMETER</code>, <code>REQUEST_HEADER</code>, <code>REQUEST_BODY</code>, <code>RESPONSE</code>, <code>RESPONSE_HEADER</code>, and <code>RESPONSE_BODY</code>. Content inheritance does not apply to any entity of the <code>API</code>, <code>AUTHORIZER</code>, <code>METHOD</code>, <code>MODEL</code>, <code>REQUEST_BODY</code>, or <code>RESOURCE</code> type.<br/>        """
        pass

    def update_documentation_version(self, documentationVersion: str, restApiId: str, patchOperations: List[PatchOperation]=None) -> DocumentationVersion:
        """

        :param self:
        :param patchOperations:  A list of update operations to be applied to the specified resource and in the order specified in this list.<br/>documentationVersion:  [Required] The version identifier of the to-be-updated documentation version.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>..<br/>
        :return: description:  The description of the API documentation snapshot.<br/>createdDate:  The date when the API documentation snapshot is created.<br/>version:  The version identifier of the API documentation snapshot.<br/>        """
        pass

    def update_domain_name(self, domainName: str, patchOperations: List[PatchOperation]=None) -> DomainName:
        """Changes information about the <a>DomainName</a> resource.

        :param self:
        :param patchOperations:  A list of update operations to be applied to the specified resource and in the order specified in this list.<br/>domainName:  [Required] The name of the <a>DomainName</a> resource to be changed.<br/>
        :return: tags:  The collection of tags. Each tag element is associated with a given resource.<br/>securityPolicy:  The Transport Layer Security (TLS) version + cipher suite for this <a>DomainName</a>. The valid values are <code>TLS_1_0</code> and <code>TLS_1_2</code>.<br/>domainNameStatusMessage:  An optional text message containing detailed information about status of the <a>DomainName</a> migration.<br/>domainNameStatus:  The status of the <a>DomainName</a> migration. The valid values are <code>AVAILABLE</code> and <code>UPDATING</code>. If the status is <code>UPDATING</code>, the domain cannot be modified further until the existing operation is complete. If it is <code>AVAILABLE</code>, the domain can be updated.<br/>endpointConfiguration:  The endpoint configuration of this <a>DomainName</a> showing the endpoint types of the domain name. :<br/>distributionHostedZoneId:  The region-agnostic Amazon Route 53 Hosted Zone ID of the edge-optimized endpoint. The valid value is <code>Z2FDTNDATAQYW2</code> for all the regions. For more information, see <a href="https://docs.aws.amazon.com/apigateway/latest/developerguide/apigateway-regional-api-custom-domain-create.html">Set up a Regional Custom Domain Name</a> and <a href="https://docs.aws.amazon.com/general/latest/gr/rande.html#apigateway_region">AWS Regions and Endpoints for API Gateway</a>. <br/>distributionDomainName:  The domain name of the Amazon CloudFront distribution associated with this custom domain name for an edge-optimized endpoint. You set up this association when adding a DNS record pointing the custom domain name to this distribution name. For more information about CloudFront distributions, see the <a href="https://aws.amazon.com/documentation/cloudfront/" target="_blank">Amazon CloudFront documentation</a>.<br/>regionalCertificateArn:  The reference to an AWS-managed certificate that will be used for validating the regional domain name. AWS Certificate Manager is the only supported source.<br/>regionalCertificateName:  The name of the certificate that will be used for validating the regional domain name.<br/>regionalHostedZoneId:  The region-specific Amazon Route 53 Hosted Zone ID of the regional endpoint. For more information, see <a href="https://docs.aws.amazon.com/apigateway/latest/developerguide/apigateway-regional-api-custom-domain-create.html">Set up a Regional Custom Domain Name</a> and <a href="https://docs.aws.amazon.com/general/latest/gr/rande.html#apigateway_region">AWS Regions and Endpoints for API Gateway</a>. <br/>regionalDomainName:  The domain name associated with the regional endpoint for this custom domain name. You set up this association by adding a DNS record that points the custom domain name to this regional domain name. The regional domain name is returned by API Gateway when you create a regional endpoint.<br/>certificateUploadDate:  The timestamp when the certificate that was used by edge-optimized endpoint for this domain name was uploaded.<br/>certificateArn:  The reference to an AWS-managed certificate that will be used by edge-optimized endpoint for this domain name. AWS Certificate Manager is the only supported source.<br/>certificateName:  The name of the certificate that will be used by edge-optimized endpoint for this domain name.<br/>domainName:  The custom domain name as an API host name, for example, <code>my-api.example.com</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>types</i> A list of endpoint types of an API (<a>RestApi</a>) or its custom domain name (<a>DomainName</a>). For an edge-optimized API and its custom domain name, the endpoint type is <code>"EDGE"</code>. For a regional API and its custom domain name, the endpoint type is <code>REGIONAL</code>. For a private API, the endpoint type is <code>PRIVATE</code>.<br/>        """
        pass

    def update_gateway_response(self, responseType: str, restApiId: str, patchOperations: List[PatchOperation]=None) -> GatewayResponse:
        """Updates a <a>GatewayResponse</a> of a specified response type on the given <a>RestApi</a>.

        :param self:
        :param patchOperations:  A list of update operations to be applied to the specified resource and in the order specified in this list.<br/>responseType:  [Required] The response type of the associated <a>GatewayResponse</a>. Valid values are <ul><li>ACCESS_DENIED</li><li>API_CONFIGURATION_ERROR</li><li>AUTHORIZER_FAILURE</li><li> AUTHORIZER_CONFIGURATION_ERROR</li><li>BAD_REQUEST_PARAMETERS</li><li>BAD_REQUEST_BODY</li><li>DEFAULT_4XX</li><li>DEFAULT_5XX</li><li>EXPIRED_TOKEN</li><li>INVALID_SIGNATURE</li><li>INTEGRATION_FAILURE</li><li>INTEGRATION_TIMEOUT</li><li>INVALID_API_KEY</li><li>MISSING_AUTHENTICATION_TOKEN</li><li> QUOTA_EXCEEDED</li><li>REQUEST_TOO_LARGE</li><li>RESOURCE_NOT_FOUND</li><li>THROTTLED</li><li>UNAUTHORIZED</li><li>UNSUPPORTED_MEDIA_TYPE</li></ul> <br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: defaultResponse:  A Boolean flag to indicate whether this <a>GatewayResponse</a> is the default gateway response (<code>true</code>) or not (<code>false</code>). A default gateway response is one generated by API Gateway without any customization by an API developer. <br/>responseTemplates:  Response templates of the <a>GatewayResponse</a> as a string-to-string map of key-value pairs.<br/>responseParameters:  Response parameters (paths, query strings and headers) of the <a>GatewayResponse</a> as a string-to-string map of key-value pairs.<br/>statusCode:  The HTTP status code for this <a>GatewayResponse</a>.<br/>responseType:  The response type of the associated <a>GatewayResponse</a>. Valid values are <ul><li>ACCESS_DENIED</li><li>API_CONFIGURATION_ERROR</li><li>AUTHORIZER_FAILURE</li><li> AUTHORIZER_CONFIGURATION_ERROR</li><li>BAD_REQUEST_PARAMETERS</li><li>BAD_REQUEST_BODY</li><li>DEFAULT_4XX</li><li>DEFAULT_5XX</li><li>EXPIRED_TOKEN</li><li>INVALID_SIGNATURE</li><li>INTEGRATION_FAILURE</li><li>INTEGRATION_TIMEOUT</li><li>INVALID_API_KEY</li><li>MISSING_AUTHENTICATION_TOKEN</li><li> QUOTA_EXCEEDED</li><li>REQUEST_TOO_LARGE</li><li>RESOURCE_NOT_FOUND</li><li>THROTTLED</li><li>UNAUTHORIZED</li><li>UNSUPPORTED_MEDIA_TYPE</li></ul> <br/>        """
        pass

    def update_integration(self, httpMethod: str, resourceId: str, restApiId: str, patchOperations: List[PatchOperation]=None) -> Integration:
        """Represents an update integration.

        :param self:
        :param patchOperations:  A list of update operations to be applied to the specified resource and in the order specified in this list.<br/>httpMethod:  [Required] Represents an update integration request's HTTP method.<br/>resourceId:  [Required] Represents an update integration request's resource identifier.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: integrationResponses:  Specifies the integration's responses. <div class="remarks"> <p/> <h4>Example: Get integration responses of a method</h4> <h5>Request</h5> <p/> <pre><code>GET /restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200 HTTP/1.1 Content-Type: application/json Host: apigateway.us-east-1.amazonaws.com X-Amz-Date: 20160607T191449Z Authorization: AWS4-HMAC-SHA256 Credential={access_key_ID}/20160607/us-east-1/apigateway/aws4_request, SignedHeaders=content-type;host;x-amz-date, Signature={sig4_hash} </code></pre> <h5>Response</h5> The successful response returns <code>200 OK</code> status and a payload as follows: <pre><code>{ "_links": { "curies": { "href": "https://docs.aws.amazon.com/apigateway/latest/developerguide/restapi-integration-response-{rel}.html", "name": "integrationresponse", "templated": true }, "self": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200", "title": "200" }, "integrationresponse:delete": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200" }, "integrationresponse:update": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200" } }, "responseParameters": { "method.response.header.Content-Type": "'application/xml'" }, "responseTemplates": { "application/json": "$util.urlDecode(\"%3CkinesisStreams%3E#foreach($stream in $input.path('$.StreamNames'))%3Cstream%3E%3Cname%3E$stream%3C/name%3E%3C/stream%3E#end%3C/kinesisStreams%3E\")\n" }, "statusCode": "200" }</code></pre> <p/> </div> <div class="seeAlso"> <a href="https://docs.aws.amazon.com/apigateway/latest/developerguide/how-to-create-api.html">Creating an API</a> </div><br/>cacheKeyParameters:  A list of request parameters whose values API Gateway caches. To be valid values for <code>cacheKeyParameters</code>, these parameters must also be specified for <a>Method</a> <code>requestParameters</code>.<br/>cacheNamespace:  An API-specific tag group of related cached parameters. To be valid values for <code>cacheKeyParameters</code>, these parameters must also be specified for <a>Method</a> <code>requestParameters</code>.<br/>timeoutInMillis:  Custom timeout between 50 and 29,000 milliseconds. The default value is 29,000 milliseconds or 29 seconds.<br/>contentHandling:  Specifies how to handle request payload content type conversions. Supported values are <code>CONVERT_TO_BINARY</code> and <code>CONVERT_TO_TEXT</code>, with the following behaviors: <ul> <li><code>CONVERT_TO_BINARY</code>: Converts a request payload from a Base64-encoded string to the corresponding binary blob.</li> <li><code>CONVERT_TO_TEXT</code>: Converts a request payload from a binary blob to a Base64-encoded string.</li> </ul> If this property is not defined, the request payload will be passed through from the method request to integration request without modification, provided that the <code>passthroughBehavior</code> is configured to support payload pass-through.<br/>passthroughBehavior:  <div>  Specifies how the method request body of an unmapped content type will be passed through the integration request to the back end without transformation. A content type is unmapped if no mapping template is defined in the integration or the content type does not match any of the mapped content types, as specified in <code>requestTemplates</code>. The valid value is one of the following:  <ul> <li> <code>WHEN_NO_MATCH</code>: passes the method request body through the integration request to the back end without transformation when the method request content type does not match any content type associated with the mapping templates defined in the integration request. </li> <li> <code>WHEN_NO_TEMPLATES</code>: passes the method request body through the integration request to the back end without transformation when no mapping template is defined in the integration request. If a template is defined when this option is selected, the method request of an unmapped content-type will be rejected with an HTTP <code>415 Unsupported Media Type</code> response. </li> <li> <code>NEVER</code>: rejects the method request with an HTTP <code>415 Unsupported Media Type</code> response when either the method request content type does not match any content type associated with the mapping templates defined in the integration request or no mapping template is defined in the integration request. </li> </ul> </div><br/>requestTemplates:  Represents a map of Velocity templates that are applied on the request payload based on the value of the Content-Type header sent by the client. The content type value is the key in this map, and the template (as a String) is the value.<br/>requestParameters:  A key-value map specifying request parameters that are passed from the method request to the back end. The key is an integration request parameter name and the associated value is a method request parameter value or static value that must be enclosed within single quotes and pre-encoded as required by the back end. The method request parameter value must match the pattern of <code>method.request.{location}.{name}</code>, where <code>location</code> is <code>querystring</code>, <code>path</code>, or <code>header</code> and <code>name</code> must be a valid and unique method request parameter name.<br/>credentials:  Specifies the credentials required for the integration, if any. For AWS integrations, three options are available. To specify an IAM Role for API Gateway to assume, use the role's Amazon Resource Name (ARN). To require that the caller's identity be passed through from the request, specify the string <code>arn:aws:iam::\*:user/\*</code>. To use resource-based permissions on supported AWS services, specify null.<br/>connectionId:  The (<a href="https://docs.aws.amazon.com/apigateway/api-reference/resource/vpc-link/#id"><code>id</code></a>) of the <a>VpcLink</a> used for the integration when <code>connectionType=VPC_LINK</code> and undefined, otherwise.<br/>connectionType:  The type of the network connection to the integration endpoint. The valid value is <code>INTERNET</code> for connections through the public routable internet or <code>VPC_LINK</code> for private connections between API Gateway and a network load balancer in a VPC. The default value is <code>INTERNET</code>.<br/>uri:  Specifies Uniform Resource Identifier (URI) of the integration endpoint. <ul> <li> For <code>HTTP</code> or <code>HTTP_PROXY</code> integrations, the URI must be a fully formed, encoded HTTP(S) URL according to the <a target="_blank" href="https://en.wikipedia.org/wiki/Uniform_Resource_Identifier">RFC-3986 specification</a>, for either standard integration, where <code>connectionType</code> is not <code>VPC_LINK</code>, or private integration, where <code>connectionType</code> is <code>VPC_LINK</code>. For a private HTTP integration, the URI is not used for routing.  </li> <li> For <code>AWS</code> or <code>AWS_PROXY</code> integrations, the URI is of the form <code>arn:aws:apigateway:{region}:{subdomain.service|service}:path|action/{service_api}</code>. Here, <code>{Region}</code> is the API Gateway region (e.g., <code>us-east-1</code>); <code>{service}</code> is the name of the integrated AWS service (e.g., <code>s3</code>); and <code>{subdomain}</code> is a designated subdomain supported by certain AWS service for fast host-name lookup. <code>action</code> can be used for an AWS service action-based API, using an <code>Action={name}&amp;{p1}={v1}&amp;p2={v2}...</code> query string. The ensuing <code>{service_api}</code> refers to a supported action <code>{name}</code> plus any required input parameters. Alternatively, <code>path</code> can be used for an AWS service path-based API. The ensuing <code>service_api</code> refers to the path to an AWS service resource, including the region of the integrated AWS service, if applicable. For example, for integration with the S3 API of <code><a href="https://docs.aws.amazon.com/AmazonS3/latest/API/RESTObjectGET.html">GetObject</a></code>, the <code>uri</code> can be either <code>arn:aws:apigateway:us-west-2:s3:action/GetObject&amp;Bucket={bucket}&amp;Key={key}</code> or <code>arn:aws:apigateway:us-west-2:s3:path/{bucket}/{key}</code> </li></ul><br/>httpMethod:  Specifies the integration's HTTP method type.<br/>type:  Specifies an API method integration type. The valid value is one of the following: <ul> <li><code>AWS</code>: for integrating the API method request with an AWS service action, including the Lambda function-invoking action. With the Lambda function-invoking action, this is referred to as the Lambda custom integration. With any other AWS service action, this is known as AWS integration.</li> <li><code>AWS_PROXY</code>: for integrating the API method request with the Lambda function-invoking action with the client request passed through as-is. This integration is also referred to as the Lambda proxy integration.</li> <li><code>HTTP</code>: for integrating the API method request with an HTTP endpoint, including a private HTTP endpoint within a VPC. This integration is also referred to as the HTTP custom integration.</li> <li><code>HTTP_PROXY</code>: for integrating the API method request with an HTTP endpoint, including a private HTTP endpoint within a VPC, with the client request passed through as-is. This is also referred to as the HTTP proxy integration.</li> <li><code>MOCK</code>: for integrating the API method request with API Gateway as a "loop-back" endpoint without invoking any backend.</li> </ul> For the HTTP and HTTP proxy integrations, each integration can specify a protocol (<code>http/https</code>), port and path. Standard 80 and 443 ports are supported as well as custom ports above 1024. An HTTP or HTTP proxy integration with a <code>connectionType</code> of <code>VPC_LINK</code> is referred to as a private integration and uses a <a>VpcLink</a> to connect API Gateway to a network load balancer of a VPC.<br/>        """
        pass

    def update_integration_response(self, statusCode: str, httpMethod: str, resourceId: str, restApiId: str, patchOperations: List[PatchOperation]=None) -> IntegrationResponse:
        """Represents an update integration response.

        :param self:
        :param patchOperations:  A list of update operations to be applied to the specified resource and in the order specified in this list.<br/>statusCode:  [Required] Specifies an update integration response request's status code.<br/>httpMethod:  [Required] Specifies an update integration response request's HTTP method.<br/>resourceId:  [Required] Specifies an update integration response request's resource identifier.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: contentHandling:  Specifies how to handle response payload content type conversions. Supported values are <code>CONVERT_TO_BINARY</code> and <code>CONVERT_TO_TEXT</code>, with the following behaviors: <ul> <li><code>CONVERT_TO_BINARY</code>: Converts a response payload from a Base64-encoded string to the corresponding binary blob.</li> <li><code>CONVERT_TO_TEXT</code>: Converts a response payload from a binary blob to a Base64-encoded string.</li> </ul> If this property is not defined, the response payload will be passed through from the integration response to the method response without modification.<br/>responseTemplates:  Specifies the templates used to transform the integration response body. Response templates are represented as a key/value map, with a content-type as the key and a template as the value.<br/>responseParameters:  A key-value map specifying response parameters that are passed to the method response from the back end. The key is a method response header parameter name and the mapped value is an integration response header value, a static value enclosed within a pair of single quotes, or a JSON expression from the integration response body. The mapping key must match the pattern of <code>method.response.header.{name}</code>, where <code>name</code> is a valid and unique header name. The mapped non-static value must match the pattern of <code>integration.response.header.{name}</code> or <code>integration.response.body.{JSON-expression}</code>, where <code>name</code> is a valid and unique response header name and <code>JSON-expression</code> is a valid JSON expression without the <code>$</code> prefix.<br/>selectionPattern:  Specifies the regular expression (regex) pattern used to choose an integration response based on the response from the back end. For example, if the success response returns nothing and the error response returns some string, you could use the <code>.+</code> regex to match error response. However, make sure that the error response does not contain any newline (<code>\n</code>) character in such cases. If the back end is an AWS Lambda function, the AWS Lambda function error header is matched. For all other HTTP and AWS back ends, the HTTP status code is matched.<br/>statusCode:  Specifies the status code that is used to map the integration response to an existing <a>MethodResponse</a>.<br/>        """
        pass

    def update_method(self, httpMethod: str, resourceId: str, restApiId: str, patchOperations: List[PatchOperation]=None) -> Method:
        """Updates an existing <a>Method</a> resource.

        :param self:
        :param patchOperations:  A list of update operations to be applied to the specified resource and in the order specified in this list.<br/>httpMethod:  [Required] The HTTP verb of the <a>Method</a> resource.<br/>resourceId:  [Required] The <a>Resource</a> identifier for the <a>Method</a> resource.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: authorizationScopes:  A list of authorization scopes configured on the method. The scopes are used with a <code>COGNITO_USER_POOLS</code> authorizer to authorize the method invocation. The authorization works by matching the method scopes against the scopes parsed from the access token in the incoming request. The method invocation is authorized if any method scopes matches a claimed scope in the access token. Otherwise, the invocation is not authorized. When the method scope is configured, the client must provide an access token instead of an identity token for authorization purposes.<br/>methodIntegration:  Gets the method's integration responsible for passing the client-submitted request to the back end and performing necessary transformations to make the request compliant with the back end. <div class="remarks"> <p/> <h4>Example: </h4> <h5>Request</h5> <p/> <pre><code>GET /restapis/uojnr9hd57/resources/0cjtch/methods/GET/integration HTTP/1.1 Content-Type: application/json Host: apigateway.us-east-1.amazonaws.com Content-Length: 117 X-Amz-Date: 20160613T213210Z Authorization: AWS4-HMAC-SHA256 Credential={access_key_ID}/20160613/us-east-1/apigateway/aws4_request, SignedHeaders=content-type;host;x-amz-date, Signature={sig4_hash}</code></pre> <h5>Response</h5> The successful response returns a <code>200 OK</code> status code and a payload similar to the following: <pre><code>{ "_links": { "curies": [ { "href": "https://docs.aws.amazon.com/apigateway/latest/developerguide/restapi-integration-{rel}.html", "name": "integration", "templated": true }, { "href": "https://docs.aws.amazon.com/apigateway/latest/developerguide/restapi-integration-response-{rel}.html", "name": "integrationresponse", "templated": true } ], "self": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/integration" }, "integration:delete": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/integration" }, "integration:responses": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/integration/responses/200", "name": "200", "title": "200" }, "integration:update": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/integration" }, "integrationresponse:put": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/integration/responses/{status_code}", "templated": true } }, "cacheKeyParameters": [], "cacheNamespace": "0cjtch", "credentials": "arn:aws:iam::123456789012:role/apigAwsProxyRole", "httpMethod": "POST", "passthroughBehavior": "WHEN_NO_MATCH", "requestTemplates": { "application/json": "{\n \"a\": \"$input.params('operand1')\",\n \"b\": \"$input.params('operand2')\", \n \"op\": \"$input.params('operator')\" \n}" }, "type": "AWS", "uri": "arn:aws:apigateway:us-west-2:lambda:path//2015-03-31/functions/arn:aws:lambda:us-west-2:123456789012:function:Calc/invocations", "_embedded": { "integration:responses": { "_links": { "self": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/integration/responses/200", "name": "200", "title": "200" }, "integrationresponse:delete": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/integration/responses/200" }, "integrationresponse:update": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/integration/responses/200" } }, "responseParameters": { "method.response.header.operator": "integration.response.body.op", "method.response.header.operand_2": "integration.response.body.b", "method.response.header.operand_1": "integration.response.body.a" }, "responseTemplates": { "application/json": "#set($res = $input.path('$'))\n{\n \"result\": \"$res.a, $res.b, $res.op => $res.c\",\n \"a\" : \"$res.a\",\n \"b\" : \"$res.b\",\n \"op\" : \"$res.op\",\n \"c\" : \"$res.c\"\n}" }, "selectionPattern": "", "statusCode": "200" } } }</code></pre> <p/> </div> <div class="seeAlso"> <a href="https://docs.aws.amazon.com/cli/latest/reference/apigateway/get-integration.html">AWS CLI</a> </div>:<br/>methodResponses:  Gets a method response associated with a given HTTP status code.  <div class="remarks"> The collection of method responses are encapsulated in a key-value map, where the key is a response's HTTP status code and the value is a <a>MethodResponse</a> resource that specifies the response returned to the caller from the back end through the integration response. <h4>Example: Get a 200 OK response of a GET method</h4> <h5>Request</h5> <p/> <pre><code>GET /restapis/uojnr9hd57/resources/0cjtch/methods/GET/responses/200 HTTP/1.1 Content-Type: application/json Host: apigateway.us-east-1.amazonaws.com Content-Length: 117 X-Amz-Date: 20160613T215008Z Authorization: AWS4-HMAC-SHA256 Credential={access_key_ID}/20160613/us-east-1/apigateway/aws4_request, SignedHeaders=content-type;host;x-amz-date, Signature={sig4_hash}</code></pre> <h5>Response</h5> The successful response returns a <code>200 OK</code> status code and a payload similar to the following: <pre><code>{ "_links": { "curies": { "href": "https://docs.aws.amazon.com/apigateway/latest/developerguide/restapi-method-response-{rel}.html", "name": "methodresponse", "templated": true }, "self": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/responses/200", "title": "200" }, "methodresponse:delete": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/responses/200" }, "methodresponse:update": { "href": "/restapis/uojnr9hd57/resources/0cjtch/methods/GET/responses/200" } }, "responseModels": { "application/json": "Empty" }, "responseParameters": { "method.response.header.operator": false, "method.response.header.operand_2": false, "method.response.header.operand_1": false }, "statusCode": "200" }</code></pre> <p/> </div> <div class="seeAlso"> <a href="https://docs.aws.amazon.com/cli/latest/reference/apigateway/get-method-response.html">AWS CLI</a> </div><br/>requestModels:  A key-value map specifying data schemas, represented by <a>Model</a> resources, (as the mapped value) of the request payloads of given content types (as the mapping key).<br/>requestParameters:  A key-value map defining required or optional method request parameters that can be accepted by API Gateway. A key is a method request parameter name matching the pattern of <code>method.request.{location}.{name}</code>, where <code>location</code> is <code>querystring</code>, <code>path</code>, or <code>header</code> and <code>name</code> is a valid and unique parameter name. The value associated with the key is a Boolean flag indicating whether the parameter is required (<code>true</code>) or optional (<code>false</code>). The method request parameter names defined here are available in <a>Integration</a> to be mapped to integration request parameters or templates.<br/>operationName:  A human-friendly operation identifier for the method. For example, you can assign the <code>operationName</code> of <code>ListPets</code> for the <code>GET /pets</code> method in the <code>PetStore</code> example.<br/>requestValidatorId:  The identifier of a <a>RequestValidator</a> for request validation.<br/>apiKeyRequired:  A boolean flag specifying whether a valid <a>ApiKey</a> is required to invoke this method.<br/>authorizerId:  The identifier of an <a>Authorizer</a> to use on this method. The <code>authorizationType</code> must be <code>CUSTOM</code>.<br/>authorizationType:  The method's authorization type. Valid values are <code>NONE</code> for open access, <code>AWS_IAM</code> for using AWS IAM permissions, <code>CUSTOM</code> for using a custom authorizer, or <code>COGNITO_USER_POOLS</code> for using a Cognito user pool.<br/>httpMethod:  The method's HTTP verb.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>integrationResponses</i> Specifies the integration's responses. <div class="remarks"> <p/> <h4>Example: Get integration responses of a method</h4> <h5>Request</h5> <p/> <pre><code>GET /restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200 HTTP/1.1 Content-Type: application/json Host: apigateway.us-east-1.amazonaws.com X-Amz-Date: 20160607T191449Z Authorization: AWS4-HMAC-SHA256 Credential={access_key_ID}/20160607/us-east-1/apigateway/aws4_request, SignedHeaders=content-type;host;x-amz-date, Signature={sig4_hash} </code></pre> <h5>Response</h5> The successful response returns <code>200 OK</code> status and a payload as follows: <pre><code>{ "_links": { "curies": { "href": "https://docs.aws.amazon.com/apigateway/latest/developerguide/restapi-integration-response-{rel}.html", "name": "integrationresponse", "templated": true }, "self": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200", "title": "200" }, "integrationresponse:delete": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200" }, "integrationresponse:update": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200" } }, "responseParameters": { "method.response.header.Content-Type": "'application/xml'" }, "responseTemplates": { "application/json": "$util.urlDecode(\"%3CkinesisStreams%3E#foreach($stream in $input.path('$.StreamNames'))%3Cstream%3E%3Cname%3E$stream%3C/name%3E%3C/stream%3E#end%3C/kinesisStreams%3E\")\n" }, "statusCode": "200" }</code></pre> <p/> </div> <div class="seeAlso"> <a href="https://docs.aws.amazon.com/apigateway/latest/developerguide/how-to-create-api.html">Creating an API</a> </div><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cacheKeyParameters</i> A list of request parameters whose values API Gateway caches. To be valid values for <code>cacheKeyParameters</code>, these parameters must also be specified for <a>Method</a> <code>requestParameters</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cacheNamespace</i> An API-specific tag group of related cached parameters. To be valid values for <code>cacheKeyParameters</code>, these parameters must also be specified for <a>Method</a> <code>requestParameters</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>timeoutInMillis</i> Custom timeout between 50 and 29,000 milliseconds. The default value is 29,000 milliseconds or 29 seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>contentHandling</i> Specifies how to handle request payload content type conversions. Supported values are <code>CONVERT_TO_BINARY</code> and <code>CONVERT_TO_TEXT</code>, with the following behaviors: <ul> <li><code>CONVERT_TO_BINARY</code>: Converts a request payload from a Base64-encoded string to the corresponding binary blob.</li> <li><code>CONVERT_TO_TEXT</code>: Converts a request payload from a binary blob to a Base64-encoded string.</li> </ul> If this property is not defined, the request payload will be passed through from the method request to integration request without modification, provided that the <code>passthroughBehavior</code> is configured to support payload pass-through.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>passthroughBehavior</i> <div>  Specifies how the method request body of an unmapped content type will be passed through the integration request to the back end without transformation. A content type is unmapped if no mapping template is defined in the integration or the content type does not match any of the mapped content types, as specified in <code>requestTemplates</code>. The valid value is one of the following:  <ul> <li> <code>WHEN_NO_MATCH</code>: passes the method request body through the integration request to the back end without transformation when the method request content type does not match any content type associated with the mapping templates defined in the integration request. </li> <li> <code>WHEN_NO_TEMPLATES</code>: passes the method request body through the integration request to the back end without transformation when no mapping template is defined in the integration request. If a template is defined when this option is selected, the method request of an unmapped content-type will be rejected with an HTTP <code>415 Unsupported Media Type</code> response. </li> <li> <code>NEVER</code>: rejects the method request with an HTTP <code>415 Unsupported Media Type</code> response when either the method request content type does not match any content type associated with the mapping templates defined in the integration request or no mapping template is defined in the integration request. </li> </ul> </div><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>requestTemplates</i> Represents a map of Velocity templates that are applied on the request payload based on the value of the Content-Type header sent by the client. The content type value is the key in this map, and the template (as a String) is the value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>requestParameters</i> A key-value map specifying request parameters that are passed from the method request to the back end. The key is an integration request parameter name and the associated value is a method request parameter value or static value that must be enclosed within single quotes and pre-encoded as required by the back end. The method request parameter value must match the pattern of <code>method.request.{location}.{name}</code>, where <code>location</code> is <code>querystring</code>, <code>path</code>, or <code>header</code> and <code>name</code> must be a valid and unique method request parameter name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>credentials</i> Specifies the credentials required for the integration, if any. For AWS integrations, three options are available. To specify an IAM Role for API Gateway to assume, use the role's Amazon Resource Name (ARN). To require that the caller's identity be passed through from the request, specify the string <code>arn:aws:iam::\*:user/\*</code>. To use resource-based permissions on supported AWS services, specify null.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>connectionId</i> The (<a href="https://docs.aws.amazon.com/apigateway/api-reference/resource/vpc-link/#id"><code>id</code></a>) of the <a>VpcLink</a> used for the integration when <code>connectionType=VPC_LINK</code> and undefined, otherwise.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>connectionType</i> The type of the network connection to the integration endpoint. The valid value is <code>INTERNET</code> for connections through the public routable internet or <code>VPC_LINK</code> for private connections between API Gateway and a network load balancer in a VPC. The default value is <code>INTERNET</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>uri</i> Specifies Uniform Resource Identifier (URI) of the integration endpoint. <ul> <li> For <code>HTTP</code> or <code>HTTP_PROXY</code> integrations, the URI must be a fully formed, encoded HTTP(S) URL according to the <a target="_blank" href="https://en.wikipedia.org/wiki/Uniform_Resource_Identifier">RFC-3986 specification</a>, for either standard integration, where <code>connectionType</code> is not <code>VPC_LINK</code>, or private integration, where <code>connectionType</code> is <code>VPC_LINK</code>. For a private HTTP integration, the URI is not used for routing.  </li> <li> For <code>AWS</code> or <code>AWS_PROXY</code> integrations, the URI is of the form <code>arn:aws:apigateway:{region}:{subdomain.service|service}:path|action/{service_api}</code>. Here, <code>{Region}</code> is the API Gateway region (e.g., <code>us-east-1</code>); <code>{service}</code> is the name of the integrated AWS service (e.g., <code>s3</code>); and <code>{subdomain}</code> is a designated subdomain supported by certain AWS service for fast host-name lookup. <code>action</code> can be used for an AWS service action-based API, using an <code>Action={name}&amp;{p1}={v1}&amp;p2={v2}...</code> query string. The ensuing <code>{service_api}</code> refers to a supported action <code>{name}</code> plus any required input parameters. Alternatively, <code>path</code> can be used for an AWS service path-based API. The ensuing <code>service_api</code> refers to the path to an AWS service resource, including the region of the integrated AWS service, if applicable. For example, for integration with the S3 API of <code><a href="https://docs.aws.amazon.com/AmazonS3/latest/API/RESTObjectGET.html">GetObject</a></code>, the <code>uri</code> can be either <code>arn:aws:apigateway:us-west-2:s3:action/GetObject&amp;Bucket={bucket}&amp;Key={key}</code> or <code>arn:aws:apigateway:us-west-2:s3:path/{bucket}/{key}</code> </li></ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>httpMethod</i> Specifies the integration's HTTP method type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> Specifies an API method integration type. The valid value is one of the following: <ul> <li><code>AWS</code>: for integrating the API method request with an AWS service action, including the Lambda function-invoking action. With the Lambda function-invoking action, this is referred to as the Lambda custom integration. With any other AWS service action, this is known as AWS integration.</li> <li><code>AWS_PROXY</code>: for integrating the API method request with the Lambda function-invoking action with the client request passed through as-is. This integration is also referred to as the Lambda proxy integration.</li> <li><code>HTTP</code>: for integrating the API method request with an HTTP endpoint, including a private HTTP endpoint within a VPC. This integration is also referred to as the HTTP custom integration.</li> <li><code>HTTP_PROXY</code>: for integrating the API method request with an HTTP endpoint, including a private HTTP endpoint within a VPC, with the client request passed through as-is. This is also referred to as the HTTP proxy integration.</li> <li><code>MOCK</code>: for integrating the API method request with API Gateway as a "loop-back" endpoint without invoking any backend.</li> </ul> For the HTTP and HTTP proxy integrations, each integration can specify a protocol (<code>http/https</code>), port and path. Standard 80 and 443 ports are supported as well as custom ports above 1024. An HTTP or HTTP proxy integration with a <code>connectionType</code> of <code>VPC_LINK</code> is referred to as a private integration and uses a <a>VpcLink</a> to connect API Gateway to a network load balancer of a VPC.<br/>        """
        pass

    def update_method_response(self, statusCode: str, httpMethod: str, resourceId: str, restApiId: str, patchOperations: List[PatchOperation]=None) -> MethodResponse:
        """Updates an existing <a>MethodResponse</a> resource.

        :param self:
        :param patchOperations:  A list of update operations to be applied to the specified resource and in the order specified in this list.<br/>statusCode:  [Required] The status code for the <a>MethodResponse</a> resource.<br/>httpMethod:  [Required] The HTTP verb of the <a>Method</a> resource.<br/>resourceId:  [Required] The <a>Resource</a> identifier for the <a>MethodResponse</a> resource.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: responseModels:  Specifies the <a>Model</a> resources used for the response's content-type. Response models are represented as a key/value map, with a content-type as the key and a <a>Model</a> name as the value.<br/>responseParameters:  A key-value map specifying required or optional response parameters that API Gateway can send back to the caller. A key defines a method response header and the value specifies whether the associated method response header is required or not. The expression of the key must match the pattern <code>method.response.header.{name}</code>, where <code>name</code> is a valid and unique header name. API Gateway passes certain integration response data to the method response headers specified here according to the mapping you prescribe in the API's <a>IntegrationResponse</a>. The integration response data that can be mapped include an integration response header expressed in <code>integration.response.header.{name}</code>, a static value enclosed within a pair of single quotes (e.g., <code>'application/json'</code>), or a JSON expression from the back-end response payload in the form of <code>integration.response.body.{JSON-expression}</code>, where <code>JSON-expression</code> is a valid JSON expression without the <code>$</code> prefix.)<br/>statusCode:  The method response's status code.<br/>        """
        pass

    def update_model(self, modelName: str, restApiId: str, patchOperations: List[PatchOperation]=None) -> Model:
        """Changes information about a model.

        :param self:
        :param patchOperations:  A list of update operations to be applied to the specified resource and in the order specified in this list.<br/>modelName:  [Required] The name of the model to update.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: contentType:  The content-type for the model.<br/>schema:  The schema for the model. For <code>application/json</code> models, this should be <a href="https://tools.ietf.org/html/draft-zyp-json-schema-04" target="_blank">JSON schema draft 4</a> model. Do not include "\*/" characters in the description of any properties because such "\*/" characters may be interpreted as the closing marker for comments in some languages, such as Java or JavaScript, causing the installation of your API's SDK generated by API Gateway to fail.<br/>description:  The description of the model.<br/>name:  The name of the model. Must be an alphanumeric string.<br/>id:  The identifier for the model resource.<br/>        """
        pass

    def update_request_validator(self, requestValidatorId: str, restApiId: str, patchOperations: List[PatchOperation]=None) -> RequestValidator:
        """Updates a <a>RequestValidator</a> of a given <a>RestApi</a>.

        :param self:
        :param patchOperations:  A list of update operations to be applied to the specified resource and in the order specified in this list.<br/>requestValidatorId:  [Required] The identifier of <a>RequestValidator</a> to be updated.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: validateRequestParameters:  A Boolean flag to indicate whether to validate request parameters (<code>true</code>) or not (<code>false</code>).<br/>validateRequestBody:  A Boolean flag to indicate whether to validate a request body according to the configured <a>Model</a> schema.<br/>name:  The name of this <a>RequestValidator</a><br/>id:  The identifier of this <a>RequestValidator</a>.<br/>        """
        pass

    def update_resource(self, resourceId: str, restApiId: str, patchOperations: List[PatchOperation]=None) -> Resource:
        """Changes information about a <a>Resource</a> resource.

        :param self:
        :param patchOperations:  A list of update operations to be applied to the specified resource and in the order specified in this list.<br/>resourceId:  [Required] The identifier of the <a>Resource</a> resource.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: resourceMethods:  Gets an API resource's method of a given HTTP verb. <div class="remarks"> The resource methods are a map of methods indexed by methods' HTTP verbs enabled on the resource. This method map is included in the <code>200 OK</code> response of the <code>GET /restapis/{restapi_id}/resources/{resource_id}</code> or <code>GET /restapis/{restapi_id}/resources/{resource_id}?embed=methods</code> request. <h4>Example: Get the GET method of an API resource</h4> <h5>Request</h5> <pre><code>GET /restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET HTTP/1.1 Content-Type: application/json Host: apigateway.us-east-1.amazonaws.com X-Amz-Date: 20170223T031827Z Authorization: AWS4-HMAC-SHA256 Credential={access_key_ID}/20170223/us-east-1/apigateway/aws4_request, SignedHeaders=content-type;host;x-amz-date, Signature={sig4_hash}</code></pre> <h5>Response</h5> <pre><code>{ "_links": { "curies": [ { "href": "https://docs.aws.amazon.com/apigateway/latest/developerguide/restapi-integration-{rel}.html", "name": "integration", "templated": true }, { "href": "https://docs.aws.amazon.com/apigateway/latest/developerguide/restapi-integration-response-{rel}.html", "name": "integrationresponse", "templated": true }, { "href": "https://docs.aws.amazon.com/apigateway/latest/developerguide/restapi-method-{rel}.html", "name": "method", "templated": true }, { "href": "https://docs.aws.amazon.com/apigateway/latest/developerguide/restapi-method-response-{rel}.html", "name": "methodresponse", "templated": true } ], "self": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET", "name": "GET", "title": "GET" }, "integration:put": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration" }, "method:delete": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET" }, "method:integration": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration" }, "method:responses": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/responses/200", "name": "200", "title": "200" }, "method:update": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET" }, "methodresponse:put": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/responses/{status_code}", "templated": true } }, "apiKeyRequired": false, "authorizationType": "NONE", "httpMethod": "GET", "_embedded": { "method:integration": { "_links": { "self": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration" }, "integration:delete": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration" }, "integration:responses": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200", "name": "200", "title": "200" }, "integration:update": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration" }, "integrationresponse:put": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/{status_code}", "templated": true } }, "cacheKeyParameters": [], "cacheNamespace": "3kzxbg5sa2", "credentials": "arn:aws:iam::123456789012:role/apigAwsProxyRole", "httpMethod": "POST", "passthroughBehavior": "WHEN_NO_MATCH", "requestParameters": { "integration.request.header.Content-Type": "'application/x-amz-json-1.1'" }, "requestTemplates": { "application/json": "{\n}" }, "type": "AWS", "uri": "arn:aws:apigateway:us-east-1:kinesis:action/ListStreams", "_embedded": { "integration:responses": { "_links": { "self": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200", "name": "200", "title": "200" }, "integrationresponse:delete": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200" }, "integrationresponse:update": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/integration/responses/200" } }, "responseParameters": { "method.response.header.Content-Type": "'application/xml'" }, "responseTemplates": { "application/json": "$util.urlDecode(\"%3CkinesisStreams%3E#foreach($stream in $input.path('$.StreamNames'))%3Cstream%3E%3Cname%3E$stream%3C/name%3E%3C/stream%3E#end%3C/kinesisStreams%3E\")\n" }, "statusCode": "200" } } }, "method:responses": { "_links": { "self": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/responses/200", "name": "200", "title": "200" }, "methodresponse:delete": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/responses/200" }, "methodresponse:update": { "href": "/restapis/fugvjdxtri/resources/3kzxbg5sa2/methods/GET/responses/200" } }, "responseModels": { "application/json": "Empty" }, "responseParameters": { "method.response.header.Content-Type": false }, "statusCode": "200" } } }</code></pre> If the <code>OPTIONS</code> is enabled on the resource, you can follow the example here to get that method. Just replace the <code>GET</code> of the last path segment in the request URL with <code>OPTIONS</code>. </div> <div class="seeAlso"> </div><br/>path:  The full path for this resource.<br/>pathPart:  The last path segment for this resource.<br/>parentId:  The parent resource's identifier.<br/>id:  The resource's identifier.<br/>        """
        pass

    def update_rest_api(self, restApiId: str, patchOperations: List[PatchOperation]=None) -> RestApi:
        """Changes information about the specified API.

        :param self:
        :param patchOperations:  A list of update operations to be applied to the specified resource and in the order specified in this list.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: tags:  The collection of tags. Each tag element is associated with a given resource.<br/>policy:  A stringified JSON policy document that applies to this RestApi regardless of the caller and <a>Method</a> configuration.<br/>endpointConfiguration:  The endpoint configuration of this <a>RestApi</a> showing the endpoint types of the API. :<br/>apiKeySource:  The source of the API key for metering requests according to a usage plan. Valid values are: <ul><li><code>HEADER</code> to read the API key from the <code>X-API-Key</code> header of a request. </li><li><code>AUTHORIZER</code> to read the API key from the <code>UsageIdentifierKey</code> from a custom authorizer.</li></ul> <br/>minimumCompressionSize:  A nullable integer that is used to enable compression (with non-negative between 0 and 10485760 (10M) bytes, inclusive) or disable compression (with a null value) on an API. When compression is enabled, compression or decompression is not applied on the payload if the payload size is smaller than this value. Setting it to zero allows compression for any payload size.<br/>binaryMediaTypes:  The list of binary media types supported by the <a>RestApi</a>. By default, the <a>RestApi</a> supports only UTF-8-encoded text payloads.<br/>warnings:  The warning messages reported when <code>failonwarnings</code> is turned on during API import.<br/>version:  A version identifier for the API.<br/>createdDate:  The timestamp when the API was created.<br/>description:  The API's description.<br/>name:  The API's name.<br/>id:  The API's identifier. This identifier is unique across all of your APIs in API Gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>types</i> A list of endpoint types of an API (<a>RestApi</a>) or its custom domain name (<a>DomainName</a>). For an edge-optimized API and its custom domain name, the endpoint type is <code>"EDGE"</code>. For a regional API and its custom domain name, the endpoint type is <code>REGIONAL</code>. For a private API, the endpoint type is <code>PRIVATE</code>.<br/>        """
        pass

    def update_stage(self, stageName: str, restApiId: str, patchOperations: List[PatchOperation]=None) -> Stage:
        """Changes information about a <a>Stage</a> resource.

        :param self:
        :param patchOperations:  A list of update operations to be applied to the specified resource and in the order specified in this list.<br/>stageName:  [Required] The name of the <a>Stage</a> resource to change information about.<br/>restApiId:  [Required] The string identifier of the associated <a>RestApi</a>.<br/>
        :return: lastUpdatedDate:  The timestamp when the stage last updated.<br/>createdDate:  The timestamp when the stage was created.<br/>tags:  The collection of tags. Each tag element is associated with a given resource.<br/>webAclArn:  The ARN of the WebAcl associated with the <a>Stage</a>.<br/>tracingEnabled:  Specifies whether active tracing with X-ray is enabled for the <a>Stage</a>.<br/>canarySettings:  Settings for the canary deployment in this stage.:<br/>accessLogSettings:  Settings for logging access in this stage.:<br/>documentationVersion:  The version of the associated API documentation.<br/>variables:  A map that defines the stage variables for a <a>Stage</a> resource. Variable names can have alphanumeric and underscore characters, and the values must match <code>[A-Za-z0-9-._~:/?#&amp;=,]+</code>.<br/>methodSettings:  A map that defines the method settings for a <a>Stage</a> resource. Keys (designated as <code>/{method_setting_key</code> below) are method paths defined as <code>{resource_path}/{http_method}</code> for an individual method override, or <code>/\*/\*</code> for overriding all methods in the stage. <br/>cacheClusterStatus:  The status of the cache cluster for the stage, if enabled.<br/>cacheClusterSize:  The size of the cache cluster for the stage, if enabled.<br/>cacheClusterEnabled:  Specifies whether a cache cluster is enabled for the stage.<br/>description:  The stage's description.<br/>stageName:  The name of the stage is the first path segment in the Uniform Resource Identifier (URI) of a call to API Gateway. Stage names can only contain alphanumeric characters, hyphens, and underscores. Maximum length is 128 characters.<br/>clientCertificateId:  The identifier of a client certificate for an API stage.<br/>deploymentId:  The identifier of the <a>Deployment</a> that the stage points to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>destinationArn</i> The ARN of the CloudWatch Logs log group to receive access logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>format</i> A single line format of the access logs of data, as specified by selected <a href="https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-mapping-template-reference.html#context-variable-reference">$context variables</a>. The format must include at least <code>$context.requestId</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>useStageCache</i> A Boolean flag to indicate whether the canary deployment uses the stage cache or not.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stageVariableOverrides</i> Stage variables overridden for a canary release deployment, including new stage variables introduced in the canary. These stage variables are represented as a string-to-string map between stage variable names and their values.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentId</i> The ID of the canary deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>percentTraffic</i> The percent (0-100) of traffic diverted to a canary deployment.<br/>        """
        pass

    def update_usage(self, keyId: str, usagePlanId: str, patchOperations: List[PatchOperation]=None) -> Usage:
        """Grants a temporary extension to the remaining quota of a usage plan associated with a specified API key.

        :param self:
        :param patchOperations:  A list of update operations to be applied to the specified resource and in the order specified in this list.<br/>keyId:  [Required] The identifier of the API key associated with the usage plan in which a temporary extension is granted to the remaining quota.<br/>usagePlanId:  [Required] The Id of the usage plan associated with the usage data.<br/>
        :return: items:  The usage data, as daily logs of used and remaining quotas, over the specified time interval indexed over the API keys in a usage plan. For example, <code>{..., "values" : { "{api_key}" : [ [0, 100], [10, 90], [100, 10]]}</code>, where <code>{api_key}</code> stands for an API key value and the daily log entry is of the format <code>[used quota, remaining quota]</code>.<br/>position:  <br/>endDate:  The ending date of the usage data.<br/>startDate:  The starting date of the usage data.<br/>usagePlanId:  The plan Id associated with this usage data.<br/>        """
        pass

    def update_usage_plan(self, usagePlanId: str, patchOperations: List[PatchOperation]=None) -> UsagePlan:
        """Updates a usage plan of a given plan Id.

        :param self:
        :param patchOperations:  A list of update operations to be applied to the specified resource and in the order specified in this list.<br/>usagePlanId:  [Required] The Id of the to-be-updated usage plan.<br/>
        :return: tags:  The collection of tags. Each tag element is associated with a given resource.<br/>productCode:  The AWS Markeplace product identifier to associate with the usage plan as a SaaS product on AWS Marketplace.<br/>quota:  The maximum number of permitted requests per a given unit time interval.:<br/>throttle:  The request throttle limits of a usage plan.:<br/>apiStages:  The associated API stages of a usage plan.<br/>description:  The description of a usage plan.<br/>name:  The name of a usage plan.<br/>id:  The identifier of a <a>UsagePlan</a> resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>rateLimit</i> The API request steady-state rate limit.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>burstLimit</i> The API request burst limit, the maximum rate limit over a time ranging from one to a few seconds, depending upon whether the underlying token bucket is at its full capacity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>period</i> The time period in which the limit applies. Valid values are "DAY", "WEEK" or "MONTH".<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>offset</i> The number of requests subtracted from the given limit in the initial time period.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>limit</i> The maximum number of requests that can be made in a given time period.<br/>        """
        pass

    def update_vpc_link(self, vpcLinkId: str, patchOperations: List[PatchOperation]=None) -> VpcLink:
        """Updates an existing <a>VpcLink</a> of a specified identifier.

        :param self:
        :param patchOperations:  A list of update operations to be applied to the specified resource and in the order specified in this list.<br/>vpcLinkId:  [Required] The identifier of the <a>VpcLink</a>. It is used in an <a>Integration</a> to reference this <a>VpcLink</a>.<br/>
        :return: tags:  The collection of tags. Each tag element is associated with a given resource.<br/>statusMessage:  A description about the VPC link status.<br/>status:  The status of the VPC link. The valid values are <code>AVAILABLE</code>, <code>PENDING</code>, <code>DELETING</code>, or <code>FAILED</code>. Deploying an API will wait if the status is <code>PENDING</code> and will fail if the status is <code>DELETING</code>. <br/>targetArns:  The ARNs of network load balancers of the VPC targeted by the VPC link. The network load balancers must be owned by the same AWS account of the API owner.<br/>description:  The description of the VPC link.<br/>name:  The name used to label and identify the VPC link.<br/>id:  The identifier of the <a>VpcLink</a>. It is used in an <a>Integration</a> to reference this <a>VpcLink</a>.<br/>        """
        pass



class ApiGatewayManagementApi(BaseClient):
    """The Amazon API Gateway Management API allows you to directly manage runtime aspects of your deployed APIs. To use it, you must explicitly set the SDK's endpoint to point to the endpoint of your deployed API. The endpoint will be of the form https://{api-id}.execute-api.{region}.amazonaws.com/{stage}, or will be the endpoint corresponding to your API's custom domain and base path, if applicable."""




    class PostToConnectionRequest(Mapping):
        pass
    
    class Data(object):
        pass
    
    def post_to_connection(self, ConnectionId: str, Data: Data):
        """Sends the provided data to the specified connection.

        :param self:
        :param ConnectionId:  The identifier of the connection that a specific client is using.<br/>Data:  The data to be sent to the client specified by its connection id.<br/>
        :return:         """
        pass



class ApiGatewayV2(BaseClient):
    """Amazon API Gateway V2"""


    class GetApisPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_apis` for valid parameters"""
            pass
    class GetAuthorizersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_authorizers` for valid parameters"""
            pass
    class GetDeploymentsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_deployments` for valid parameters"""
            pass
    class GetDomainNamesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_domain_names` for valid parameters"""
            pass
    class GetIntegrationResponsesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_integration_responses` for valid parameters"""
            pass
    class GetIntegrationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_integrations` for valid parameters"""
            pass
    class GetModelsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_models` for valid parameters"""
            pass
    class GetRouteResponsesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_route_responses` for valid parameters"""
            pass
    class GetRoutesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_routes` for valid parameters"""
            pass
    class GetStagesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_stages` for valid parameters"""
            pass


    class CreateApiRequest(Mapping):
        pass
    
    class CreateApiResponse(Mapping):
        pass
    
    class __boolean(object):
        pass
    
    class Tags(object):
        pass
    
    class CreateApiMappingRequest(Mapping):
        pass
    
    class CreateApiMappingResponse(Mapping):
        pass
    
    class CreateAuthorizerRequest(Mapping):
        pass
    
    class CreateAuthorizerResponse(Mapping):
        pass
    
    class __string(object):
        pass
    
    class Arn(object):
        pass
    
    class CreateDeploymentRequest(Mapping):
        pass
    
    class CreateDeploymentResponse(Mapping):
        pass
    
    class CreateDomainNameRequest(Mapping):
        pass
    
    class CreateDomainNameResponse(Mapping):
        pass
    
    class DomainNameConfiguration(Mapping):
        pass
    
    class CreateIntegrationRequest(Mapping):
        pass
    
    class CreateIntegrationResponse(Mapping):
        pass
    
    class IntegrationParameters(object):
        pass
    
    class TemplateMap(object):
        pass
    
    class CreateIntegrationResponseRequest(Mapping):
        pass
    
    class CreateIntegrationResponseResponse(Mapping):
        pass
    
    class CreateModelRequest(Mapping):
        pass
    
    class CreateModelResponse(Mapping):
        pass
    
    class CreateRouteRequest(Mapping):
        pass
    
    class CreateRouteResponse(Mapping):
        pass
    
    class StringWithLengthBetween1And64(object):
        pass
    
    class RouteModels(object):
        pass
    
    class RouteParameters(object):
        pass
    
    class CreateRouteResponseRequest(Mapping):
        pass
    
    class CreateRouteResponseResponse(Mapping):
        pass
    
    class CreateStageRequest(Mapping):
        pass
    
    class CreateStageResponse(Mapping):
        pass
    
    class AccessLogSettings(Mapping):
        pass
    
    class RouteSettings(Mapping):
        pass
    
    class RouteSettingsMap(object):
        pass
    
    class StageVariablesMap(object):
        pass
    
    class DeleteApiRequest(Mapping):
        pass
    
    class DeleteApiMappingRequest(Mapping):
        pass
    
    class DeleteAuthorizerRequest(Mapping):
        pass
    
    class DeleteDeploymentRequest(Mapping):
        pass
    
    class DeleteDomainNameRequest(Mapping):
        pass
    
    class DeleteIntegrationRequest(Mapping):
        pass
    
    class DeleteIntegrationResponseRequest(Mapping):
        pass
    
    class DeleteModelRequest(Mapping):
        pass
    
    class DeleteRouteRequest(Mapping):
        pass
    
    class DeleteRouteResponseRequest(Mapping):
        pass
    
    class DeleteStageRequest(Mapping):
        pass
    
    class GetApiRequest(Mapping):
        pass
    
    class GetApiResponse(Mapping):
        pass
    
    class GetApiMappingRequest(Mapping):
        pass
    
    class GetApiMappingResponse(Mapping):
        pass
    
    class GetApiMappingsRequest(Mapping):
        pass
    
    class GetApiMappingsResponse(Mapping):
        pass
    
    class GetApisRequest(Mapping):
        pass
    
    class GetApisResponse(Mapping):
        pass
    
    class GetAuthorizerRequest(Mapping):
        pass
    
    class GetAuthorizerResponse(Mapping):
        pass
    
    class GetAuthorizersRequest(Mapping):
        pass
    
    class GetAuthorizersResponse(Mapping):
        pass
    
    class GetDeploymentRequest(Mapping):
        pass
    
    class GetDeploymentResponse(Mapping):
        pass
    
    class GetDeploymentsRequest(Mapping):
        pass
    
    class GetDeploymentsResponse(Mapping):
        pass
    
    class GetDomainNameRequest(Mapping):
        pass
    
    class GetDomainNameResponse(Mapping):
        pass
    
    class GetDomainNamesRequest(Mapping):
        pass
    
    class GetDomainNamesResponse(Mapping):
        pass
    
    class GetIntegrationRequest(Mapping):
        pass
    
    class GetIntegrationResponse(Mapping):
        pass
    
    class GetIntegrationResponseRequest(Mapping):
        pass
    
    class GetIntegrationResponseResponse(Mapping):
        pass
    
    class GetIntegrationResponsesRequest(Mapping):
        pass
    
    class GetIntegrationResponsesResponse(Mapping):
        pass
    
    class GetIntegrationsRequest(Mapping):
        pass
    
    class GetIntegrationsResponse(Mapping):
        pass
    
    class GetModelRequest(Mapping):
        pass
    
    class GetModelResponse(Mapping):
        pass
    
    class GetModelTemplateRequest(Mapping):
        pass
    
    class GetModelTemplateResponse(Mapping):
        pass
    
    class GetModelsRequest(Mapping):
        pass
    
    class GetModelsResponse(Mapping):
        pass
    
    class GetRouteRequest(Mapping):
        pass
    
    class GetRouteResponse(Mapping):
        pass
    
    class GetRouteResponseRequest(Mapping):
        pass
    
    class GetRouteResponseResponse(Mapping):
        pass
    
    class GetRouteResponsesRequest(Mapping):
        pass
    
    class GetRouteResponsesResponse(Mapping):
        pass
    
    class GetRoutesRequest(Mapping):
        pass
    
    class GetRoutesResponse(Mapping):
        pass
    
    class GetStageRequest(Mapping):
        pass
    
    class GetStageResponse(Mapping):
        pass
    
    class GetStagesRequest(Mapping):
        pass
    
    class GetStagesResponse(Mapping):
        pass
    
    class GetTagsRequest(Mapping):
        pass
    
    class GetTagsResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UpdateApiRequest(Mapping):
        pass
    
    class UpdateApiResponse(Mapping):
        pass
    
    class UpdateApiMappingRequest(Mapping):
        pass
    
    class UpdateApiMappingResponse(Mapping):
        pass
    
    class UpdateAuthorizerRequest(Mapping):
        pass
    
    class UpdateAuthorizerResponse(Mapping):
        pass
    
    class UpdateDeploymentRequest(Mapping):
        pass
    
    class UpdateDeploymentResponse(Mapping):
        pass
    
    class UpdateDomainNameRequest(Mapping):
        pass
    
    class UpdateDomainNameResponse(Mapping):
        pass
    
    class UpdateIntegrationRequest(Mapping):
        pass
    
    class UpdateIntegrationResponse(Mapping):
        pass
    
    class UpdateIntegrationResponseRequest(Mapping):
        pass
    
    class UpdateIntegrationResponseResponse(Mapping):
        pass
    
    class UpdateModelRequest(Mapping):
        pass
    
    class UpdateModelResponse(Mapping):
        pass
    
    class UpdateRouteRequest(Mapping):
        pass
    
    class UpdateRouteResponse(Mapping):
        pass
    
    class UpdateRouteResponseRequest(Mapping):
        pass
    
    class UpdateRouteResponseResponse(Mapping):
        pass
    
    class UpdateStageRequest(Mapping):
        pass
    
    class UpdateStageResponse(Mapping):
        pass
    
    def create_api(self, RouteSelectionExpression: str, ProtocolType: str, Name: str, ApiKeySelectionExpression: str=None, Description: str=None, DisableSchemaValidation: __boolean=None, Version: str=None, Tags: Tags=None) -> CreateApiResponse:
        """Creates an Api resource.

        :param self:
        :param Tags:  The key-value map of strings. The valid character set is [a-zA-Z+-=._:/]. The tag key can be up to 128 characters and must not start with aws:. The tag value can be up to 256 characters..<br/>Version:  A version identifier for the API.<br/>RouteSelectionExpression:  The route selection expression for the API.<br/>ProtocolType:  The API protocol: Currently only WEBSOCKET is supported.<br/>Name:  The name of the API.<br/>DisableSchemaValidation:  Avoid validating models when creating a deployment.<br/>Description:  The description of the API.<br/>ApiKeySelectionExpression:  An API key selection expression. See <a href="https://docs.aws.amazon.com/apigateway/latest/developerguide/apigateway-websocket-api-selection-expressions.html#apigateway-websocket-api-apikey-selection-expressions">API Key Selection Expressions</a>.<br/>
        :return: Tags:  The key-value map of strings. The valid character set is [a-zA-Z+-=._:/]. The tag key can be up to 128 characters and must not start with aws:. The tag value can be up to 256 characters..<br/>Warnings:  The warning messages reported when failonwarnings is turned on during
 API import.<br/>Version:  A version identifier for the API.<br/>RouteSelectionExpression:  The route selection expression for the API.<br/>ProtocolType:  The API protocol: Currently only WEBSOCKET is supported.<br/>Name:  The name of the API.<br/>DisableSchemaValidation:  Avoid validating models when creating a deployment.<br/>Description:  The description of the API.<br/>CreatedDate:  The timestamp when the API was created.<br/>ApiKeySelectionExpression:  An API key selection expression. See <a href="https://docs.aws.amazon.com/apigateway/latest/developerguide/apigateway-websocket-api-selection-expressions.html#apigateway-websocket-api-apikey-selection-expressions">API Key Selection Expressions</a>.<br/>ApiId:  The API ID.<br/>ApiEndpoint:  The URI of the API, of the form {api-id}.execute-api.{region}.amazonaws.com. The
 stage name is typically appended to this URI to form a complete path to a deployed
 API stage.<br/>        """
        pass

    def create_api_mapping(self, Stage: str, DomainName: str, ApiId: str, ApiMappingKey: str=None) -> CreateApiMappingResponse:
        """Creates an API mapping.

        :param self:
        :param Stage:  The API stage.<br/>DomainName:  The domain name.<br/>ApiMappingKey:  <br/>ApiId:  The API identifier.<br/>
        :return: Stage:  The API stage.<br/>ApiMappingKey:  The API mapping key.<br/>ApiMappingId:  The API mapping identifier.<br/>ApiId:  The API identifier.<br/>        """
        pass

    def create_authorizer(self, Name: str, IdentitySource: List[__string], AuthorizerUri: str, AuthorizerType: str, ApiId: str, AuthorizerCredentialsArn: str=None, AuthorizerResultTtlInSeconds: int=None, IdentityValidationExpression: str=None, ProviderArns: List[Arn]=None) -> CreateAuthorizerResponse:
        """Creates an Authorizer for an API.

        :param self:
        :param ProviderArns:  For
 REQUEST authorizer, this is not
 defined.<br/>Name:  The name of the authorizer.<br/>IdentityValidationExpression:  The
 validation expression does not apply to the REQUEST authorizer.<br/>IdentitySource:  The identity source for which authorization is requested.For the REQUEST authorizer, this is required when authorization
 caching is enabled. The value is a comma-separated string of one or more mapping
 expressions of the specified request parameters. For example, if an Auth
 header and a Name query string parameters are defined as identity
 sources, this value is method.request.header.Auth,
 method.request.querystring.Name. These parameters will be used to
 derive the authorization caching key and to perform runtime validation of the
 REQUEST authorizer by verifying all of the identity-related request
 parameters are present, not null, and non-empty. Only when this is true does the
 authorizer invoke the authorizer Lambda function, otherwise, it returns a 401
 Unauthorized response without calling the Lambda function. The valid value
 is a string of comma-separated mapping expressions of the specified request
 parameters. When the authorization caching is not enabled, this property is
 optional.<br/>AuthorizerUri:  The authorizer's Uniform Resource Identifier (URI). For
 REQUEST authorizers, this must be a
 well-formed Lambda function URI, for example,
 arn:aws:apigateway:us-west-2:lambda:path/2015-03-31/functions/arn:aws:lambda:us-west-2:{account_id}:function:{lambda_function_name}/invocations.
 In general, the URI has this form:
 arn:aws:apigateway:{region}:lambda:path/{service_api}
 , where {region} is the same as the region hosting the Lambda
 function, path indicates that the remaining substring in the URI should be treated as
 the path to the resource, including the initial /. For Lambda functions,
 this is usually of the form
 /2015-03-31/functions/[FunctionARN]/invocations.<br/>AuthorizerType:  The authorizer type. Currently the only valid value is REQUEST, for a
 Lambda function using incoming request parameters.<br/>AuthorizerResultTtlInSeconds:  The time to live (TTL), in seconds, of cached authorizer results. If it equals 0,
 authorization caching is disabled. If it is greater than 0, API Gateway will cache
 authorizer responses. If this field is not set, the default value is 300. The maximum
 value is 3600, or 1 hour.<br/>AuthorizerCredentialsArn:  Specifies the required credentials as an IAM role for API Gateway to invoke the
 authorizer. To specify an IAM role for API Gateway to assume, use the role's Amazon
 Resource Name (ARN). To use resource-based permissions on the Lambda function,
 specify null.<br/>ApiId:  The API identifier.<br/>
        :return: ProviderArns:  For
 REQUEST authorizer, this is not
 defined.<br/>Name:  The name of the authorizer.<br/>IdentityValidationExpression:  The
 validation expression does not apply to the REQUEST authorizer.<br/>IdentitySource:  The identity source for which authorization is requested.For the REQUEST authorizer, this is required when authorization
 caching is enabled. The value is a comma-separated string of one or more mapping
 expressions of the specified request parameters. For example, if an Auth
 header and a Name query string parameters are defined as identity
 sources, this value is method.request.header.Auth,
 method.request.querystring.Name. These parameters will be used to
 derive the authorization caching key and to perform runtime validation of the
 REQUEST authorizer by verifying all of the identity-related request
 parameters are present, not null, and non-empty. Only when this is true does the
 authorizer invoke the authorizer Lambda function, otherwise, it returns a 401
 Unauthorized response without calling the Lambda function. The valid value
 is a string of comma-separated mapping expressions of the specified request
 parameters. When the authorization caching is not enabled, this property is
 optional.<br/>AuthorizerUri:  The authorizer's Uniform Resource Identifier (URI).
 ForREQUEST authorizers, this must be a
 well-formed Lambda function URI, for example,
 arn:aws:apigateway:us-west-2:lambda:path/2015-03-31/functions/arn:aws:lambda:us-west-2:{account_id}:function:{lambda_function_name}/invocations.
 In general, the URI has this form:
 arn:aws:apigateway:{region}:lambda:path/{service_api}
 , where {region} is the same as the region hosting the Lambda
 function, path indicates that the remaining substring in the URI should be treated as
 the path to the resource, including the initial /. For Lambda functions,
 this is usually of the form
 /2015-03-31/functions/[FunctionARN]/invocations.<br/>AuthorizerType:  The authorizer type. Currently the only valid value is REQUEST, for a
 Lambda function using incoming request parameters.<br/>AuthorizerResultTtlInSeconds:  The time to live (TTL), in seconds, of cached authorizer results. If it equals 0,
 authorization caching is disabled. If it is greater than 0, API Gateway will cache
 authorizer responses. If this field is not set, the default value is 300. The maximum
 value is 3600, or 1 hour.<br/>AuthorizerId:  The authorizer identifier.<br/>AuthorizerCredentialsArn:  Specifies the required credentials as an IAM role for API Gateway to invoke the
 authorizer. To specify an IAM role for API Gateway to assume, use the role's Amazon
 Resource Name (ARN). To use resource-based permissions on the Lambda function,
 specify null.<br/>        """
        pass

    def create_deployment(self, ApiId: str, Description: str=None, StageName: str=None) -> CreateDeploymentResponse:
        """Creates a Deployment for an API.

        :param self:
        :param StageName:  The name of the Stage resource for the Deployment
 resource to create.<br/>Description:  The description for the deployment resource.<br/>ApiId:  The API identifier.<br/>
        :return: Description:  The description for the deployment.<br/>DeploymentStatusMessage:  May contain additional feedback on the status of an API deployment.<br/>DeploymentStatus:  The status of the deployment: PENDING, FAILED, or
 SUCCEEDED.<br/>DeploymentId:  The identifier for the deployment.<br/>CreatedDate:  The date and time when the Deployment resource was created.<br/>        """
        pass

    def create_domain_name(self, DomainName: str, DomainNameConfigurations: List[DomainNameConfiguration]=None, Tags: Tags=None) -> CreateDomainNameResponse:
        """Creates a domain name.

        :param self:
        :param Tags:  The key-value map of strings. The valid character set is [a-zA-Z+-=._:/]. The tag key can be up to 128 characters and must not start with aws:. The tag value can be up to 256 characters..<br/>DomainNameConfigurations:  The domain name configurations.<br/>DomainName:  The domain name.<br/>
        :return: Tags:  The key-value map of strings. The valid character set is [a-zA-Z+-=._:/]. The tag key can be up to 128 characters and must not start with aws:. The tag value can be up to 256 characters..<br/>DomainNameConfigurations:  The domain name configurations.<br/>DomainName:  The name of the DomainName resource.<br/>ApiMappingSelectionExpression:  The API mapping selection expression.<br/>        """
        pass

    def create_integration(self, IntegrationType: str, ApiId: str, ConnectionId: str=None, ConnectionType: str=None, ContentHandlingStrategy: str=None, CredentialsArn: str=None, Description: str=None, IntegrationMethod: str=None, IntegrationUri: str=None, PassthroughBehavior: str=None, RequestParameters: IntegrationParameters=None, RequestTemplates: TemplateMap=None, TemplateSelectionExpression: str=None, TimeoutInMillis: int=None) -> CreateIntegrationResponse:
        """Creates an Integration.

        :param self:
        :param TimeoutInMillis:  Custom timeout between 50 and 29,000 milliseconds. The default value is 29,000
 milliseconds or 29 seconds.<br/>TemplateSelectionExpression:  The template selection expression for the integration.<br/>RequestTemplates:  Represents a map of Velocity templates that are applied on the request payload
 based on the value of the Content-Type header sent by the client. The content type
 value is the key in this map, and the template (as a String) is the value.<br/>RequestParameters:  A key-value map specifying request parameters that are passed from the method
 request to the backend. The key is an integration request parameter name and the
 associated value is a method request parameter value or static value that must be
 enclosed within single quotes and pre-encoded as required by the backend. The method
 request parameter value must match the pattern of
 method.request.{location}.{name}
 , where 
 {location}
  is querystring, path, or header; and 
 {name}
  must be a valid and unique method request parameter name.<br/>PassthroughBehavior:  Specifies the pass-through behavior for incoming requests based on the
 Content-Type header in the request, and the available mapping
 templates specified as the requestTemplates property on the
 Integration resource. There are three valid values:
 WHEN_NO_MATCH, WHEN_NO_TEMPLATES, and
 NEVER.
 WHEN_NO_MATCH passes the request body for unmapped content types through
 to the integration backend without transformation.
 NEVER rejects unmapped content types with an HTTP 415 Unsupported
 Media Type response.
 WHEN_NO_TEMPLATES allows pass-through when the integration has no
 content types mapped to templates. However, if there is at least one content type
 defined, unmapped content types will be rejected with the same HTTP 415
 Unsupported Media Type response.<br/>IntegrationUri:  For a Lambda proxy integration, this is the URI of the Lambda function.<br/>IntegrationType:  The integration type of an integration. One of the following:
 AWS: for integrating the route or method request with an AWS service
 action, including the Lambda function-invoking action. With the Lambda
 function-invoking action, this is referred to as the Lambda custom integration. With
 any other AWS service action, this is known as AWS integration.
 AWS_PROXY: for integrating the route or method request with the Lambda
 function-invoking action with the client request passed through as-is. This
 integration is also referred to as Lambda proxy integration.
 HTTP: for integrating the route or method request with an HTTP
 endpoint. This
 integration is also referred to as HTTP custom integration.
 HTTP_PROXY: for integrating route or method request with an HTTP
 endpoint, with the client
 request passed through as-is. This is also referred to as HTTP proxy
 integration.
 MOCK: for integrating the route or method request with API Gateway as a
 "loopback" endpoint without invoking any backend.<br/>IntegrationMethod:  Specifies the integration's HTTP method type.<br/>Description:  The description of the integration.<br/>CredentialsArn:  Specifies the credentials required for the integration, if any. For AWS
 integrations, three options are available. To specify an IAM Role for API Gateway to
 assume, use the role's Amazon Resource Name (ARN). To require that the caller's
 identity be passed through from the request, specify the string
 arn:aws:iam::*:user/*. To use resource-based permissions on supported
 AWS services, specify null.<br/>ContentHandlingStrategy:  Specifies how to handle response payload content type conversions. Supported
 values are CONVERT_TO_BINARY and CONVERT_TO_TEXT, with the
 following behaviors:
 CONVERT_TO_BINARY: Converts a response payload from a Base64-encoded
 string to the corresponding binary blob.
 CONVERT_TO_TEXT: Converts a response payload from a binary blob to a
 Base64-encoded string.If this property is not defined, the response payload will be passed through from
 the integration response to the route response or method response without
 modification.<br/>ConnectionType:  The type of the network connection to the integration endpoint. Currently the only
 valid value is INTERNET, for connections through the public routable
 internet.<br/>ConnectionId:  The connection ID.<br/>ApiId:  The API identifier.<br/>
        :return: TimeoutInMillis:  Custom timeout between 50 and 29,000 milliseconds. The default value is 29,000
 milliseconds or 29 seconds.<br/>TemplateSelectionExpression:  The template selection expression for the integration.<br/>RequestTemplates:  Represents a map of Velocity templates that are applied on the request payload
 based on the value of the Content-Type header sent by the client. The content type
 value is the key in this map, and the template (as a String) is the value.<br/>RequestParameters:  A key-value map specifying request parameters that are passed from the method
 request to the backend. The key is an integration request parameter name and the
 associated value is a method request parameter value or static value that must be
 enclosed within single quotes and pre-encoded as required by the backend. The method
 request parameter value must match the pattern of
 method.request.{location}.{name}
 , where 
 {location}
  is querystring, path, or header; and 
 {name}
  must be a valid and unique method request parameter name.<br/>PassthroughBehavior:  Specifies the pass-through behavior for incoming requests based on the
 Content-Type header in the request, and the available mapping
 templates specified as the requestTemplates property on the
 Integration resource. There are three valid values:
 WHEN_NO_MATCH, WHEN_NO_TEMPLATES, and
 NEVER.
 WHEN_NO_MATCH passes the request body for unmapped content types through
 to the integration backend without transformation.
 NEVER rejects unmapped content types with an HTTP 415 Unsupported
 Media Type response.
 WHEN_NO_TEMPLATES allows pass-through when the integration has no
 content types mapped to templates. However, if there is at least one content type
 defined, unmapped content types will be rejected with the same HTTP 415
 Unsupported Media Type response.<br/>IntegrationUri:  For a Lambda proxy integration, this is the URI of the Lambda function.<br/>IntegrationType:  The integration type of an integration. One of the following:
 AWS: for integrating the route or method request with an AWS service
 action, including the Lambda function-invoking action. With the Lambda
 function-invoking action, this is referred to as the Lambda custom integration. With
 any other AWS service action, this is known as AWS integration.
 AWS_PROXY: for integrating the route or method request with the Lambda
 function-invoking action with the client request passed through as-is. This
 integration is also referred to as Lambda proxy integration.
 HTTP: for integrating the route or method request with an HTTP
 endpoint. This
 integration is also referred to as the HTTP custom integration.
 HTTP_PROXY: for integrating route or method request with an HTTP
 endpoint, with the client
 request passed through as-is. This is also referred to as HTTP proxy
 integration.
 MOCK: for integrating the route or method request with API Gateway as a
 "loopback" endpoint without invoking any backend.<br/>IntegrationResponseSelectionExpression:  The integration response selection expression for the integration. See <a href="https://docs.aws.amazon.com/apigateway/latest/developerguide/apigateway-websocket-api-selection-expressions.html#apigateway-websocket-api-integration-response-selection-expressions">Integration Response Selection Expressions</a>.<br/>IntegrationMethod:  Specifies the integration's HTTP method type.<br/>IntegrationId:  Represents the identifier of an integration.<br/>Description:  Represents the description of an integration.<br/>CredentialsArn:  Specifies the credentials required for the integration, if any. For AWS
 integrations, three options are available. To specify an IAM Role for API Gateway to
 assume, use the role's Amazon Resource Name (ARN). To require that the caller's
 identity be passed through from the request, specify the string
 arn:aws:iam::*:user/*. To use resource-based permissions on supported
 AWS services, specify null.<br/>ContentHandlingStrategy:  Specifies how to handle response payload content type conversions. Supported
 values are CONVERT_TO_BINARY and CONVERT_TO_TEXT, with the
 following behaviors:
 CONVERT_TO_BINARY: Converts a response payload from a Base64-encoded
 string to the corresponding binary blob.
 CONVERT_TO_TEXT: Converts a response payload from a binary blob to a
 Base64-encoded string.If this property is not defined, the response payload will be passed through from
 the integration response to the route response or method response without
 modification.<br/>ConnectionType:  The type of the network connection to the integration endpoint. Currently the only
 valid value is INTERNET, for connections through the public routable
 internet.<br/>ConnectionId:  The connection ID.<br/>        """
        pass

    def create_integration_response(self, IntegrationResponseKey: str, IntegrationId: str, ApiId: str, ContentHandlingStrategy: str=None, ResponseParameters: IntegrationParameters=None, ResponseTemplates: TemplateMap=None, TemplateSelectionExpression: str=None) -> CreateIntegrationResponseResponse:
        """Creates an IntegrationResponses.

        :param self:
        :param TemplateSelectionExpression:  The template selection expression for the integration response.<br/>ResponseTemplates:  The collection of response templates for the integration response as a
 string-to-string map of key-value pairs. Response templates are represented as a
 key/value map, with a content-type as the key and a template as the value.<br/>ResponseParameters:  A key-value map specifying response parameters that are passed to the method
 response from the backend. The key is a method response header parameter name and the
 mapped value is an integration response header value, a static value enclosed within
 a pair of single quotes, or a JSON expression from the integration response body. The
 mapping key must match the pattern of method.response.header.{name},
 where {name} is a valid and unique header name. The mapped non-static
 value must match the pattern of integration.response.header.{name} or
 integration.response.body.{JSON-expression}, where
 {name} is a valid and unique response header name and
 {JSON-expression} is a valid JSON expression without the $
 prefix.<br/>IntegrationResponseKey:  The integration response key.<br/>IntegrationId:  The integration ID.<br/>ContentHandlingStrategy:  Specifies how to handle response payload content type conversions. Supported
 values are CONVERT_TO_BINARY and CONVERT_TO_TEXT, with the
 following behaviors:
 CONVERT_TO_BINARY: Converts a response payload from a Base64-encoded
 string to the corresponding binary blob.
 CONVERT_TO_TEXT: Converts a response payload from a binary blob to a
 Base64-encoded string.If this property is not defined, the response payload will be passed through from
 the integration response to the route response or method response without
 modification.<br/>ApiId:  The API identifier.<br/>
        :return: TemplateSelectionExpression:  The template selection expressions for the integration response.<br/>ResponseTemplates:  The collection of response templates for the integration response as a
 string-to-string map of key-value pairs. Response templates are represented as a
 key/value map, with a content-type as the key and a template as the value.<br/>ResponseParameters:  A key-value map specifying response parameters that are passed to the method
 response from the backend. The key is a method response header parameter name and the
 mapped value is an integration response header value, a static value enclosed within
 a pair of single quotes, or a JSON expression from the integration response body. The
 mapping key must match the pattern of method.response.header.{name}, where name is a
 valid and unique header name. The mapped non-static value must match the pattern of
 integration.response.header.{name} or integration.response.body.{JSON-expression},
 where name is a valid and unique response header name and JSON-expression is a valid
 JSON expression without the $ prefix.<br/>IntegrationResponseKey:  The integration response key.<br/>IntegrationResponseId:  The integration response ID.<br/>ContentHandlingStrategy:  Specifies how to handle response payload content type conversions. Supported
 values are CONVERT_TO_BINARY and CONVERT_TO_TEXT, with the
 following behaviors:
 CONVERT_TO_BINARY: Converts a response payload from a Base64-encoded
 string to the corresponding binary blob.
 CONVERT_TO_TEXT: Converts a response payload from a binary blob to a
 Base64-encoded string.If this property is not defined, the response payload will be passed through from
 the integration response to the route response or method response without
 modification.<br/>        """
        pass

    def create_model(self, Schema: str, Name: str, ApiId: str, ContentType: str=None, Description: str=None) -> CreateModelResponse:
        """Creates a Model for an API.

        :param self:
        :param Schema:  The schema for the model. For application/json models, this should be JSON schema
 draft 4 model.<br/>Name:  The name of the model. Must be alphanumeric.<br/>Description:  The description of the model.<br/>ContentType:  The content-type for the model, for example, "application/json".<br/>ApiId:  The API identifier.<br/>
        :return: Schema:  The schema for the model. For application/json models, this should be JSON schema
 draft 4 model.<br/>Name:  The name of the model. Must be alphanumeric.<br/>ModelId:  The model identifier.<br/>Description:  The description of the model.<br/>ContentType:  The content-type for the model, for example, "application/json".<br/>        """
        pass

    def create_route(self, RouteKey: str, ApiId: str, ApiKeyRequired: __boolean=None, AuthorizationScopes: List[StringWithLengthBetween1And64]=None, AuthorizationType: str=None, AuthorizerId: str=None, ModelSelectionExpression: str=None, OperationName: str=None, RequestModels: RouteModels=None, RequestParameters: RouteParameters=None, RouteResponseSelectionExpression: str=None, Target: str=None) -> CreateRouteResponse:
        """Creates a Route for an API.

        :param self:
        :param Target:  The target for the route.<br/>RouteResponseSelectionExpression:  The route response selection expression for the route.<br/>RouteKey:  The route key for the route.<br/>RequestParameters:  The request parameters for the route.<br/>RequestModels:  The request models for the route.<br/>OperationName:  The operation name for the route.<br/>ModelSelectionExpression:  The model selection expression for the route.<br/>AuthorizerId:  The identifier of the Authorizer resource to be associated with this
 route, if the authorizationType is CUSTOM
 . The authorizer identifier is generated by API Gateway
 when you created the authorizer.<br/>AuthorizationType:  The authorization type for the route. Valid values are NONE for open
 access, AWS_IAM for using AWS IAM permissions, and CUSTOM
 for using a Lambda
 authorizer.<br/>AuthorizationScopes:  The authorization scopes supported by this
 route.<br/>ApiKeyRequired:  Specifies whether an API key is required for the route.<br/>ApiId:  The API identifier.<br/>
        :return: Target:  The target for the route.<br/>RouteResponseSelectionExpression:  The route response selection expression for the route.<br/>RouteKey:  The route key for the route.<br/>RouteId:  The route ID.<br/>RequestParameters:  The request parameters for the route.<br/>RequestModels:  The request models for the route.<br/>OperationName:  The operation name for the route.<br/>ModelSelectionExpression:  The model selection expression for the route.<br/>AuthorizerId:  The identifier of the Authorizer resource to be associated with this
 route, if the authorizationType is CUSTOM
 . The authorizer identifier is generated by API Gateway
 when you created the authorizer.<br/>AuthorizationType:  The authorization type for the route. Valid values are NONE for open
 access, AWS_IAM for using AWS IAM permissions, and CUSTOM
 for using a Lambda
 authorizer<br/>AuthorizationScopes:  A list of authorization scopes configured on a route. The scopes are used with a
 COGNITO_USER_POOLS authorizer to authorize the method invocation. The authorization
 works by matching the route scopes against the scopes parsed from the access token in
 the incoming request. The method invocation is authorized if any route scope matches
 a claimed scope in the access token. Otherwise, the invocation is not authorized.
 When the route scope is configured, the client must provide an access token instead
 of an identity token for authorization purposes.<br/>ApiKeyRequired:  Specifies whether an API key is required for this route.<br/>        """
        pass

    def create_route_response(self, RouteResponseKey: str, RouteId: str, ApiId: str, ModelSelectionExpression: str=None, ResponseModels: RouteModels=None, ResponseParameters: RouteParameters=None) -> CreateRouteResponseResponse:
        """Creates a RouteResponse for a Route.

        :param self:
        :param RouteResponseKey:  The route response key.<br/>RouteId:  The route ID.<br/>ResponseParameters:  The route response parameters.<br/>ResponseModels:  The response models for the route response.<br/>ModelSelectionExpression:  The model selection expression for the route response.<br/>ApiId:  The API identifier.<br/>
        :return: RouteResponseKey:  Represents the route response key of a route response.<br/>RouteResponseId:  Represents the identifier of a route response.<br/>ResponseParameters:  Represents the response parameters of a route response.<br/>ResponseModels:  Represents the response models of a route response.<br/>ModelSelectionExpression:  Represents the model selection expression of a route response.<br/>        """
        pass

    def create_stage(self, StageName: str, ApiId: str, AccessLogSettings: AccessLogSettings=None, ClientCertificateId: str=None, DefaultRouteSettings: RouteSettings=None, DeploymentId: str=None, Description: str=None, RouteSettings: RouteSettingsMap=None, StageVariables: StageVariablesMap=None, Tags: Tags=None) -> CreateStageResponse:
        """Creates a Stage for an API.

        :param self:
        :param Tags:  The key-value map of strings. The valid character set is [a-zA-Z+-=._:/]. The tag key can be up to 128 characters and must not start with aws:. The tag value can be up to 256 characters..<br/>StageVariables:  A map that defines the stage variables for a Stage. Variable names
 can have alphanumeric and underscore characters, and the values must match
 [A-Za-z0-9-._~:/?#&=,]+.<br/>StageName:  The name of the stage.<br/>RouteSettings:  Route settings for the stage.<br/>Description:  The description for the API stage.<br/>DeploymentId:  The deployment identifier of the API stage.<br/>DefaultRouteSettings:  The default route settings for the stage.:<br/>ClientCertificateId:  The identifier of a client certificate for a Stage.<br/>ApiId:  The API identifier.<br/>AccessLogSettings:  Settings for logging access in this stage.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> A single line format of the access logs of data, as specified by selected $context
 variables. The format must include at least $context.requestId.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationArn</i> The ARN of the CloudWatch Logs log group to receive access logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThrottlingRateLimit</i> Specifies the throttling rate limit.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThrottlingBurstLimit</i> Specifies the throttling burst limit.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LoggingLevel</i> Specifies the logging level for this route: DEBUG, INFO,
 or WARN. This property affects the log entries pushed to Amazon
 CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DetailedMetricsEnabled</i> Specifies whether detailed metrics are enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTraceEnabled</i> Specifies whether (true) or not (false) data trace
 logging is enabled for this route. This property affects the log entries pushed to
 Amazon CloudWatch Logs.<br/>
        :return: Tags:  The key-value map of strings. The valid character set is [a-zA-Z+-=._:/]. The tag key can be up to 128 characters and must not start with aws:. The tag value can be up to 256 characters..<br/>StageVariables:  A map that defines the stage variables for a stage resource. Variable names can
 have alphanumeric and underscore characters, and the values must match
 [A-Za-z0-9-._~:/?#&=,]+.<br/>StageName:  The name of the stage.<br/>RouteSettings:  Route settings for the stage.<br/>LastUpdatedDate:  The timestamp when the stage was last updated.<br/>Description:  The description of the stage.<br/>DeploymentId:  The identifier of the Deployment that the Stage is
 associated with.<br/>DefaultRouteSettings:  Default route settings for the stage.:<br/>CreatedDate:  The timestamp when the stage was created.<br/>ClientCertificateId:  The identifier of a client certificate for a Stage.<br/>AccessLogSettings:  Settings for logging access in this stage.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> A single line format of the access logs of data, as specified by selected $context
 variables. The format must include at least $context.requestId.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationArn</i> The ARN of the CloudWatch Logs log group to receive access logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThrottlingRateLimit</i> Specifies the throttling rate limit.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThrottlingBurstLimit</i> Specifies the throttling burst limit.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LoggingLevel</i> Specifies the logging level for this route: DEBUG, INFO,
 or WARN. This property affects the log entries pushed to Amazon
 CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DetailedMetricsEnabled</i> Specifies whether detailed metrics are enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTraceEnabled</i> Specifies whether (true) or not (false) data trace
 logging is enabled for this route. This property affects the log entries pushed to
 Amazon CloudWatch Logs.<br/>        """
        pass

    def delete_api(self, ApiId: str):
        """Deletes an Api resource.

        :param self:
        :param ApiId:  The API identifier.<br/>
        :return:         """
        pass

    def delete_api_mapping(self, DomainName: str, ApiMappingId: str):
        """Deletes an API mapping.

        :param self:
        :param DomainName:  The domain name.<br/>ApiMappingId:  The API mapping identifier.<br/>
        :return:         """
        pass

    def delete_authorizer(self, AuthorizerId: str, ApiId: str):
        """Deletes an Authorizer.

        :param self:
        :param AuthorizerId:  The authorizer identifier.<br/>ApiId:  The API identifier.<br/>
        :return:         """
        pass

    def delete_deployment(self, DeploymentId: str, ApiId: str):
        """Deletes a Deployment.

        :param self:
        :param DeploymentId:  The deployment ID.<br/>ApiId:  The API identifier.<br/>
        :return:         """
        pass

    def delete_domain_name(self, DomainName: str):
        """Deletes a domain name.

        :param self:
        :param DomainName:  The domain name.<br/>
        :return:         """
        pass

    def delete_integration(self, IntegrationId: str, ApiId: str):
        """Deletes an Integration.

        :param self:
        :param IntegrationId:  The integration ID.<br/>ApiId:  The API identifier.<br/>
        :return:         """
        pass

    def delete_integration_response(self, IntegrationResponseId: str, IntegrationId: str, ApiId: str):
        """Deletes an IntegrationResponses.

        :param self:
        :param IntegrationResponseId:  The integration response ID.<br/>IntegrationId:  The integration ID.<br/>ApiId:  The API identifier.<br/>
        :return:         """
        pass

    def delete_model(self, ModelId: str, ApiId: str):
        """Deletes a Model.

        :param self:
        :param ModelId:  The model ID.<br/>ApiId:  The API identifier.<br/>
        :return:         """
        pass

    def delete_route(self, RouteId: str, ApiId: str):
        """Deletes a Route.

        :param self:
        :param RouteId:  The route ID.<br/>ApiId:  The API identifier.<br/>
        :return:         """
        pass

    def delete_route_response(self, RouteResponseId: str, RouteId: str, ApiId: str):
        """Deletes a RouteResponse.

        :param self:
        :param RouteResponseId:  The route response ID.<br/>RouteId:  The route ID.<br/>ApiId:  The API identifier.<br/>
        :return:         """
        pass

    def delete_stage(self, StageName: str, ApiId: str):
        """Deletes a Stage.

        :param self:
        :param StageName:  The stage name.<br/>ApiId:  The API identifier.<br/>
        :return:         """
        pass

    def get_api(self, ApiId: str) -> GetApiResponse:
        """Gets an Api resource.

        :param self:
        :param ApiId:  The API identifier.<br/>
        :return: Tags:  The key-value map of strings. The valid character set is [a-zA-Z+-=._:/]. The tag key can be up to 128 characters and must not start with aws:. The tag value can be up to 256 characters..<br/>Warnings:  The warning messages reported when failonwarnings is turned on during
 API import.<br/>Version:  A version identifier for the API.<br/>RouteSelectionExpression:  The route selection expression for the API.<br/>ProtocolType:  The API protocol: Currently only WEBSOCKET is supported.<br/>Name:  The name of the API.<br/>DisableSchemaValidation:  Avoid validating models when creating a deployment.<br/>Description:  The description of the API.<br/>CreatedDate:  The timestamp when the API was created.<br/>ApiKeySelectionExpression:  An API key selection expression. See <a href="https://docs.aws.amazon.com/apigateway/latest/developerguide/apigateway-websocket-api-selection-expressions.html#apigateway-websocket-api-apikey-selection-expressions">API Key Selection Expressions</a>.<br/>ApiId:  The API ID.<br/>ApiEndpoint:  The URI of the API, of the form {api-id}.execute-api.{region}.amazonaws.com. The
 stage name is typically appended to this URI to form a complete path to a deployed
 API stage.<br/>        """
        pass

    def get_api_mapping(self, DomainName: str, ApiMappingId: str) -> GetApiMappingResponse:
        """The API mapping.

        :param self:
        :param DomainName:  The domain name.<br/>ApiMappingId:  The API mapping identifier.<br/>
        :return: Stage:  The API stage.<br/>ApiMappingKey:  The API mapping key.<br/>ApiMappingId:  The API mapping identifier.<br/>ApiId:  The API identifier.<br/>        """
        pass

    def get_api_mappings(self, DomainName: str, MaxResults: str=None, NextToken: str=None) -> GetApiMappingsResponse:
        """The API mappings.

        :param self:
        :param NextToken:  The next page of elements from this collection. Not valid for the last element of
 the collection.<br/>MaxResults:  The maximum number of elements to be returned for this resource.<br/>DomainName:  The domain name.<br/>
        :return: NextToken:  The next page of elements from this collection. Not valid for the last element of
 the collection.<br/>Items:  The elements from this collection.<br/>        """
        pass

    def get_apis(self, MaxResults: str=None, NextToken: str=None) -> GetApisResponse:
        """Gets a collection of Api resources.

        :param self:
        :param NextToken:  The next page of elements from this collection. Not valid for the last element of
 the collection.<br/>MaxResults:  The maximum number of elements to be returned for this resource.<br/>
        :return: NextToken:  The next page of elements from this collection. Not valid for the last element of
 the collection.<br/>Items:  The elements from this collection.<br/>        """
        pass

    def get_authorizer(self, AuthorizerId: str, ApiId: str) -> GetAuthorizerResponse:
        """Gets an Authorizer.

        :param self:
        :param AuthorizerId:  The authorizer identifier.<br/>ApiId:  The API identifier.<br/>
        :return: ProviderArns:  For
 REQUEST authorizer, this is not
 defined.<br/>Name:  The name of the authorizer.<br/>IdentityValidationExpression:  The
 validation expression does not apply to the REQUEST authorizer.<br/>IdentitySource:  The identity source for which authorization is requested.For the REQUEST authorizer, this is required when authorization
 caching is enabled. The value is a comma-separated string of one or more mapping
 expressions of the specified request parameters. For example, if an Auth
 header and a Name query string parameters are defined as identity
 sources, this value is method.request.header.Auth,
 method.request.querystring.Name. These parameters will be used to
 derive the authorization caching key and to perform runtime validation of the
 REQUEST authorizer by verifying all of the identity-related request
 parameters are present, not null, and non-empty. Only when this is true does the
 authorizer invoke the authorizer Lambda function, otherwise, it returns a 401
 Unauthorized response without calling the Lambda function. The valid value
 is a string of comma-separated mapping expressions of the specified request
 parameters. When the authorization caching is not enabled, this property is
 optional.<br/>AuthorizerUri:  The authorizer's Uniform Resource Identifier (URI).
 ForREQUEST authorizers, this must be a
 well-formed Lambda function URI, for example,
 arn:aws:apigateway:us-west-2:lambda:path/2015-03-31/functions/arn:aws:lambda:us-west-2:{account_id}:function:{lambda_function_name}/invocations.
 In general, the URI has this form:
 arn:aws:apigateway:{region}:lambda:path/{service_api}
 , where {region} is the same as the region hosting the Lambda
 function, path indicates that the remaining substring in the URI should be treated as
 the path to the resource, including the initial /. For Lambda functions,
 this is usually of the form
 /2015-03-31/functions/[FunctionARN]/invocations.<br/>AuthorizerType:  The authorizer type. Currently the only valid value is REQUEST, for a
 Lambda function using incoming request parameters.<br/>AuthorizerResultTtlInSeconds:  The time to live (TTL), in seconds, of cached authorizer results. If it equals 0,
 authorization caching is disabled. If it is greater than 0, API Gateway will cache
 authorizer responses. If this field is not set, the default value is 300. The maximum
 value is 3600, or 1 hour.<br/>AuthorizerId:  The authorizer identifier.<br/>AuthorizerCredentialsArn:  Specifies the required credentials as an IAM role for API Gateway to invoke the
 authorizer. To specify an IAM role for API Gateway to assume, use the role's Amazon
 Resource Name (ARN). To use resource-based permissions on the Lambda function,
 specify null.<br/>        """
        pass

    def get_authorizers(self, ApiId: str, MaxResults: str=None, NextToken: str=None) -> GetAuthorizersResponse:
        """Gets the Authorizers for an API.

        :param self:
        :param NextToken:  The next page of elements from this collection. Not valid for the last element of
 the collection.<br/>MaxResults:  The maximum number of elements to be returned for this resource.<br/>ApiId:  The API identifier.<br/>
        :return: NextToken:  The next page of elements from this collection. Not valid for the last element of
 the collection.<br/>Items:  The elements from this collection.<br/>        """
        pass

    def get_deployment(self, DeploymentId: str, ApiId: str) -> GetDeploymentResponse:
        """Gets a Deployment.

        :param self:
        :param DeploymentId:  The deployment ID.<br/>ApiId:  The API identifier.<br/>
        :return: Description:  The description for the deployment.<br/>DeploymentStatusMessage:  May contain additional feedback on the status of an API deployment.<br/>DeploymentStatus:  The status of the deployment: PENDING, FAILED, or
 SUCCEEDED.<br/>DeploymentId:  The identifier for the deployment.<br/>CreatedDate:  The date and time when the Deployment resource was created.<br/>        """
        pass

    def get_deployments(self, ApiId: str, MaxResults: str=None, NextToken: str=None) -> GetDeploymentsResponse:
        """Gets the Deployments for an API.

        :param self:
        :param NextToken:  The next page of elements from this collection. Not valid for the last element of
 the collection.<br/>MaxResults:  The maximum number of elements to be returned for this resource.<br/>ApiId:  The API identifier.<br/>
        :return: NextToken:  The next page of elements from this collection. Not valid for the last element of
 the collection.<br/>Items:  The elements from this collection.<br/>        """
        pass

    def get_domain_name(self, DomainName: str) -> GetDomainNameResponse:
        """Gets a domain name.

        :param self:
        :param DomainName:  The domain name.<br/>
        :return: Tags:  The key-value map of strings. The valid character set is [a-zA-Z+-=._:/]. The tag key can be up to 128 characters and must not start with aws:. The tag value can be up to 256 characters..<br/>DomainNameConfigurations:  The domain name configurations.<br/>DomainName:  The name of the DomainName resource.<br/>ApiMappingSelectionExpression:  The API mapping selection expression.<br/>        """
        pass

    def get_domain_names(self, MaxResults: str=None, NextToken: str=None) -> GetDomainNamesResponse:
        """Gets the domain names for an AWS account.

        :param self:
        :param NextToken:  The next page of elements from this collection. Not valid for the last element of
 the collection.<br/>MaxResults:  The maximum number of elements to be returned for this resource.<br/>
        :return: NextToken:  The next page of elements from this collection. Not valid for the last element of
 the collection.<br/>Items:  The elements from this collection.<br/>        """
        pass

    def get_integration(self, IntegrationId: str, ApiId: str) -> GetIntegrationResponse:
        """Gets an Integration.

        :param self:
        :param IntegrationId:  The integration ID.<br/>ApiId:  The API identifier.<br/>
        :return: TimeoutInMillis:  Custom timeout between 50 and 29,000 milliseconds. The default value is 29,000
 milliseconds or 29 seconds.<br/>TemplateSelectionExpression:  The template selection expression for the integration.<br/>RequestTemplates:  Represents a map of Velocity templates that are applied on the request payload
 based on the value of the Content-Type header sent by the client. The content type
 value is the key in this map, and the template (as a String) is the value.<br/>RequestParameters:  A key-value map specifying request parameters that are passed from the method
 request to the backend. The key is an integration request parameter name and the
 associated value is a method request parameter value or static value that must be
 enclosed within single quotes and pre-encoded as required by the backend. The method
 request parameter value must match the pattern of
 method.request.{location}.{name}
 , where 
 {location}
  is querystring, path, or header; and 
 {name}
  must be a valid and unique method request parameter name.<br/>PassthroughBehavior:  Specifies the pass-through behavior for incoming requests based on the
 Content-Type header in the request, and the available mapping
 templates specified as the requestTemplates property on the
 Integration resource. There are three valid values:
 WHEN_NO_MATCH, WHEN_NO_TEMPLATES, and
 NEVER.
 WHEN_NO_MATCH passes the request body for unmapped content types through
 to the integration backend without transformation.
 NEVER rejects unmapped content types with an HTTP 415 Unsupported
 Media Type response.
 WHEN_NO_TEMPLATES allows pass-through when the integration has no
 content types mapped to templates. However, if there is at least one content type
 defined, unmapped content types will be rejected with the same HTTP 415
 Unsupported Media Type response.<br/>IntegrationUri:  For a Lambda proxy integration, this is the URI of the Lambda function.<br/>IntegrationType:  The integration type of an integration. One of the following:
 AWS: for integrating the route or method request with an AWS service
 action, including the Lambda function-invoking action. With the Lambda
 function-invoking action, this is referred to as the Lambda custom integration. With
 any other AWS service action, this is known as AWS integration.
 AWS_PROXY: for integrating the route or method request with the Lambda
 function-invoking action with the client request passed through as-is. This
 integration is also referred to as Lambda proxy integration.
 HTTP: for integrating the route or method request with an HTTP
 endpoint. This
 integration is also referred to as the HTTP custom integration.
 HTTP_PROXY: for integrating route or method request with an HTTP
 endpoint, with the client
 request passed through as-is. This is also referred to as HTTP proxy
 integration.
 MOCK: for integrating the route or method request with API Gateway as a
 "loopback" endpoint without invoking any backend.<br/>IntegrationResponseSelectionExpression:  The integration response selection expression for the integration. See <a href="https://docs.aws.amazon.com/apigateway/latest/developerguide/apigateway-websocket-api-selection-expressions.html#apigateway-websocket-api-integration-response-selection-expressions">Integration Response Selection Expressions</a>.<br/>IntegrationMethod:  Specifies the integration's HTTP method type.<br/>IntegrationId:  Represents the identifier of an integration.<br/>Description:  Represents the description of an integration.<br/>CredentialsArn:  Specifies the credentials required for the integration, if any. For AWS
 integrations, three options are available. To specify an IAM Role for API Gateway to
 assume, use the role's Amazon Resource Name (ARN). To require that the caller's
 identity be passed through from the request, specify the string
 arn:aws:iam::*:user/*. To use resource-based permissions on supported
 AWS services, specify null.<br/>ContentHandlingStrategy:  Specifies how to handle response payload content type conversions. Supported
 values are CONVERT_TO_BINARY and CONVERT_TO_TEXT, with the
 following behaviors:
 CONVERT_TO_BINARY: Converts a response payload from a Base64-encoded
 string to the corresponding binary blob.
 CONVERT_TO_TEXT: Converts a response payload from a binary blob to a
 Base64-encoded string.If this property is not defined, the response payload will be passed through from
 the integration response to the route response or method response without
 modification.<br/>ConnectionType:  The type of the network connection to the integration endpoint. Currently the only
 valid value is INTERNET, for connections through the public routable
 internet.<br/>ConnectionId:  The connection ID.<br/>        """
        pass

    def get_integration_response(self, IntegrationResponseId: str, IntegrationId: str, ApiId: str) -> GetIntegrationResponseResponse:
        """Gets an IntegrationResponses.

        :param self:
        :param IntegrationResponseId:  The integration response ID.<br/>IntegrationId:  The integration ID.<br/>ApiId:  The API identifier.<br/>
        :return: TemplateSelectionExpression:  The template selection expressions for the integration response.<br/>ResponseTemplates:  The collection of response templates for the integration response as a
 string-to-string map of key-value pairs. Response templates are represented as a
 key/value map, with a content-type as the key and a template as the value.<br/>ResponseParameters:  A key-value map specifying response parameters that are passed to the method
 response from the backend. The key is a method response header parameter name and the
 mapped value is an integration response header value, a static value enclosed within
 a pair of single quotes, or a JSON expression from the integration response body. The
 mapping key must match the pattern of method.response.header.{name}, where name is a
 valid and unique header name. The mapped non-static value must match the pattern of
 integration.response.header.{name} or integration.response.body.{JSON-expression},
 where name is a valid and unique response header name and JSON-expression is a valid
 JSON expression without the $ prefix.<br/>IntegrationResponseKey:  The integration response key.<br/>IntegrationResponseId:  The integration response ID.<br/>ContentHandlingStrategy:  Specifies how to handle response payload content type conversions. Supported
 values are CONVERT_TO_BINARY and CONVERT_TO_TEXT, with the
 following behaviors:
 CONVERT_TO_BINARY: Converts a response payload from a Base64-encoded
 string to the corresponding binary blob.
 CONVERT_TO_TEXT: Converts a response payload from a binary blob to a
 Base64-encoded string.If this property is not defined, the response payload will be passed through from
 the integration response to the route response or method response without
 modification.<br/>        """
        pass

    def get_integration_responses(self, IntegrationId: str, ApiId: str, MaxResults: str=None, NextToken: str=None) -> GetIntegrationResponsesResponse:
        """Gets the IntegrationResponses for an Integration.

        :param self:
        :param NextToken:  The next page of elements from this collection. Not valid for the last element of
 the collection.<br/>MaxResults:  The maximum number of elements to be returned for this resource.<br/>IntegrationId:  The integration ID.<br/>ApiId:  The API identifier.<br/>
        :return: NextToken:  The next page of elements from this collection. Not valid for the last element of
 the collection.<br/>Items:  The elements from this collection.<br/>        """
        pass

    def get_integrations(self, ApiId: str, MaxResults: str=None, NextToken: str=None) -> GetIntegrationsResponse:
        """Gets the Integrations for an API.

        :param self:
        :param NextToken:  The next page of elements from this collection. Not valid for the last element of
 the collection.<br/>MaxResults:  The maximum number of elements to be returned for this resource.<br/>ApiId:  The API identifier.<br/>
        :return: NextToken:  The next page of elements from this collection. Not valid for the last element of
 the collection.<br/>Items:  The elements from this collection.<br/>        """
        pass

    def get_model(self, ModelId: str, ApiId: str) -> GetModelResponse:
        """Gets a Model.

        :param self:
        :param ModelId:  The model ID.<br/>ApiId:  The API identifier.<br/>
        :return: Schema:  The schema for the model. For application/json models, this should be JSON schema
 draft 4 model.<br/>Name:  The name of the model. Must be alphanumeric.<br/>ModelId:  The model identifier.<br/>Description:  The description of the model.<br/>ContentType:  The content-type for the model, for example, "application/json".<br/>        """
        pass

    def get_model_template(self, ModelId: str, ApiId: str) -> GetModelTemplateResponse:
        """Gets a model template.

        :param self:
        :param ModelId:  The model ID.<br/>ApiId:  The API identifier.<br/>
        :return: Value:  The template value.<br/>        """
        pass

    def get_models(self, ApiId: str, MaxResults: str=None, NextToken: str=None) -> GetModelsResponse:
        """Gets the Models for an API.

        :param self:
        :param NextToken:  The next page of elements from this collection. Not valid for the last element of
 the collection.<br/>MaxResults:  The maximum number of elements to be returned for this resource.<br/>ApiId:  The API identifier.<br/>
        :return: NextToken:  The next page of elements from this collection. Not valid for the last element of
 the collection.<br/>Items:  The elements from this collection.<br/>        """
        pass

    def get_route(self, RouteId: str, ApiId: str) -> GetRouteResponse:
        """Gets a Route.

        :param self:
        :param RouteId:  The route ID.<br/>ApiId:  The API identifier.<br/>
        :return: Target:  The target for the route.<br/>RouteResponseSelectionExpression:  The route response selection expression for the route.<br/>RouteKey:  The route key for the route.<br/>RouteId:  The route ID.<br/>RequestParameters:  The request parameters for the route.<br/>RequestModels:  The request models for the route.<br/>OperationName:  The operation name for the route.<br/>ModelSelectionExpression:  The model selection expression for the route.<br/>AuthorizerId:  The identifier of the Authorizer resource to be associated with this
 route, if the authorizationType is CUSTOM
 . The authorizer identifier is generated by API Gateway
 when you created the authorizer.<br/>AuthorizationType:  The authorization type for the route. Valid values are NONE for open
 access, AWS_IAM for using AWS IAM permissions, and CUSTOM
 for using a Lambda
 authorizer<br/>AuthorizationScopes:  A list of authorization scopes configured on a route. The scopes are used with a
 COGNITO_USER_POOLS authorizer to authorize the method invocation. The authorization
 works by matching the route scopes against the scopes parsed from the access token in
 the incoming request. The method invocation is authorized if any route scope matches
 a claimed scope in the access token. Otherwise, the invocation is not authorized.
 When the route scope is configured, the client must provide an access token instead
 of an identity token for authorization purposes.<br/>ApiKeyRequired:  Specifies whether an API key is required for this route.<br/>        """
        pass

    def get_route_response(self, RouteResponseId: str, RouteId: str, ApiId: str) -> GetRouteResponseResponse:
        """Gets a RouteResponse.

        :param self:
        :param RouteResponseId:  The route response ID.<br/>RouteId:  The route ID.<br/>ApiId:  The API identifier.<br/>
        :return: RouteResponseKey:  Represents the route response key of a route response.<br/>RouteResponseId:  Represents the identifier of a route response.<br/>ResponseParameters:  Represents the response parameters of a route response.<br/>ResponseModels:  Represents the response models of a route response.<br/>ModelSelectionExpression:  Represents the model selection expression of a route response.<br/>        """
        pass

    def get_route_responses(self, RouteId: str, ApiId: str, MaxResults: str=None, NextToken: str=None) -> GetRouteResponsesResponse:
        """Gets the RouteResponses for a Route.

        :param self:
        :param RouteId:  The route ID.<br/>NextToken:  The next page of elements from this collection. Not valid for the last element of
 the collection.<br/>MaxResults:  The maximum number of elements to be returned for this resource.<br/>ApiId:  The API identifier.<br/>
        :return: NextToken:  The next page of elements from this collection. Not valid for the last element of
 the collection.<br/>Items:  The elements from this collection.<br/>        """
        pass

    def get_routes(self, ApiId: str, MaxResults: str=None, NextToken: str=None) -> GetRoutesResponse:
        """Gets the Routes for an API.

        :param self:
        :param NextToken:  The next page of elements from this collection. Not valid for the last element of
 the collection.<br/>MaxResults:  The maximum number of elements to be returned for this resource.<br/>ApiId:  The API identifier.<br/>
        :return: NextToken:  The next page of elements from this collection. Not valid for the last element of
 the collection.<br/>Items:  The elements from this collection.<br/>        """
        pass

    def get_stage(self, StageName: str, ApiId: str) -> GetStageResponse:
        """Gets a Stage.

        :param self:
        :param StageName:  The stage name.<br/>ApiId:  The API identifier.<br/>
        :return: Tags:  The key-value map of strings. The valid character set is [a-zA-Z+-=._:/]. The tag key can be up to 128 characters and must not start with aws:. The tag value can be up to 256 characters..<br/>StageVariables:  A map that defines the stage variables for a stage resource. Variable names can
 have alphanumeric and underscore characters, and the values must match
 [A-Za-z0-9-._~:/?#&=,]+.<br/>StageName:  The name of the stage.<br/>RouteSettings:  Route settings for the stage.<br/>LastUpdatedDate:  The timestamp when the stage was last updated.<br/>Description:  The description of the stage.<br/>DeploymentId:  The identifier of the Deployment that the Stage is
 associated with.<br/>DefaultRouteSettings:  Default route settings for the stage.:<br/>CreatedDate:  The timestamp when the stage was created.<br/>ClientCertificateId:  The identifier of a client certificate for a Stage.<br/>AccessLogSettings:  Settings for logging access in this stage.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> A single line format of the access logs of data, as specified by selected $context
 variables. The format must include at least $context.requestId.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationArn</i> The ARN of the CloudWatch Logs log group to receive access logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThrottlingRateLimit</i> Specifies the throttling rate limit.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThrottlingBurstLimit</i> Specifies the throttling burst limit.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LoggingLevel</i> Specifies the logging level for this route: DEBUG, INFO,
 or WARN. This property affects the log entries pushed to Amazon
 CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DetailedMetricsEnabled</i> Specifies whether detailed metrics are enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTraceEnabled</i> Specifies whether (true) or not (false) data trace
 logging is enabled for this route. This property affects the log entries pushed to
 Amazon CloudWatch Logs.<br/>        """
        pass

    def get_stages(self, ApiId: str, MaxResults: str=None, NextToken: str=None) -> GetStagesResponse:
        """Gets the Stages for an API.

        :param self:
        :param NextToken:  The next page of elements from this collection. Not valid for the last element of
 the collection.<br/>MaxResults:  The maximum number of elements to be returned for this resource.<br/>ApiId:  The API identifier.<br/>
        :return: NextToken:  The next page of elements from this collection. Not valid for the last element of
 the collection.<br/>Items:  The elements from this collection.<br/>        """
        pass

    def get_tags(self, ResourceArn: str) -> GetTagsResponse:
        """Gets the Tags for an API.

        :param self:
        :param ResourceArn:  <br/>
        :return: Tags:  <br/>        """
        pass

    def tag_resource(self, ResourceArn: str, Tags: Tags=None) -> TagResourceResponse:
        """Tag an APIGW resource

        :param self:
        :param Tags:  The key-value map of strings. The valid character set is [a-zA-Z+-=._:/]. The tag key can be up to 128 characters and must not start with aws:. The tag value can be up to 256 characters..<br/>ResourceArn:  AWS resource arn <br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[__string], ResourceArn: str):
        """Untag an APIGW resource

        :param self:
        :param TagKeys:  The Tag keys to delete<br/>ResourceArn:  AWS resource arn <br/>
        :return:         """
        pass

    def update_api(self, ApiId: str, ApiKeySelectionExpression: str=None, Description: str=None, DisableSchemaValidation: __boolean=None, Name: str=None, RouteSelectionExpression: str=None, Version: str=None) -> UpdateApiResponse:
        """Updates an Api resource.

        :param self:
        :param Version:  A version identifier for the API.<br/>RouteSelectionExpression:  The route selection expression for the API.<br/>Name:  The name of the API.<br/>DisableSchemaValidation:  Avoid validating models when creating a deployment.<br/>Description:  The description of the API.<br/>ApiKeySelectionExpression:  An API key selection expression. See <a href="https://docs.aws.amazon.com/apigateway/latest/developerguide/apigateway-websocket-api-selection-expressions.html#apigateway-websocket-api-apikey-selection-expressions">API Key Selection Expressions</a>.<br/>ApiId:  The API identifier.<br/>
        :return: Tags:  The key-value map of strings. The valid character set is [a-zA-Z+-=._:/]. The tag key can be up to 128 characters and must not start with aws:. The tag value can be up to 256 characters..<br/>Warnings:  The warning messages reported when failonwarnings is turned on during
 API import.<br/>Version:  A version identifier for the API.<br/>RouteSelectionExpression:  The route selection expression for the API.<br/>ProtocolType:  The API protocol: Currently only WEBSOCKET is supported.<br/>Name:  The name of the API.<br/>DisableSchemaValidation:  Avoid validating models when creating a deployment.<br/>Description:  The description of the API.<br/>CreatedDate:  The timestamp when the API was created.<br/>ApiKeySelectionExpression:  An API key selection expression. See <a href="https://docs.aws.amazon.com/apigateway/latest/developerguide/apigateway-websocket-api-selection-expressions.html#apigateway-websocket-api-apikey-selection-expressions">API Key Selection Expressions</a>.<br/>ApiId:  The API ID.<br/>ApiEndpoint:  The URI of the API, of the form {api-id}.execute-api.{region}.amazonaws.com. The
 stage name is typically appended to this URI to form a complete path to a deployed
 API stage.<br/>        """
        pass

    def update_api_mapping(self, DomainName: str, ApiMappingId: str, ApiId: str, ApiMappingKey: str=None, Stage: str=None) -> UpdateApiMappingResponse:
        """The API mapping.

        :param self:
        :param Stage:  The API stage.<br/>DomainName:  The domain name.<br/>ApiMappingKey:  The API mapping key.<br/>ApiMappingId:  The API mapping identifier.<br/>ApiId:  The API identifier.<br/>
        :return: Stage:  The API stage.<br/>ApiMappingKey:  The API mapping key.<br/>ApiMappingId:  The API mapping identifier.<br/>ApiId:  The API identifier.<br/>        """
        pass

    def update_authorizer(self, AuthorizerId: str, ApiId: str, AuthorizerCredentialsArn: str=None, AuthorizerResultTtlInSeconds: int=None, AuthorizerType: str=None, AuthorizerUri: str=None, IdentitySource: List[__string]=None, IdentityValidationExpression: str=None, Name: str=None, ProviderArns: List[Arn]=None) -> UpdateAuthorizerResponse:
        """Updates an Authorizer.

        :param self:
        :param ProviderArns:  For
 REQUEST authorizer, this is not
 defined.<br/>Name:  The name of the authorizer.<br/>IdentityValidationExpression:  The
 validation expression does not apply to the REQUEST authorizer.<br/>IdentitySource:  The identity source for which authorization is requested.For the REQUEST authorizer, this is required when authorization
 caching is enabled. The value is a comma-separated string of one or more mapping
 expressions of the specified request parameters. For example, if an Auth header, a
 Name query string parameter are defined as identity sources, this value is
 $method.request.header.Auth, $method.request.querystring.Name. These
 parameters will be used to derive the authorization caching key and to perform
 runtime validation of the REQUEST authorizer by verifying all of the
 identity-related request parameters are present, not null and non-empty. Only when
 this is true does the authorizer invoke the authorizer Lambda function, otherwise, it
 returns a 401 Unauthorized response without calling the Lambda function.
 The valid value is a string of comma-separated mapping expressions of the specified
 request parameters. When the authorization caching is not enabled, this property is
 optional.<br/>AuthorizerUri:  The authorizer's Uniform Resource Identifier (URI). For
 REQUEST authorizers, this must be a
 well-formed Lambda function URI, for example,
 arn:aws:apigateway:us-west-2:lambda:path/2015-03-31/functions/arn:aws:lambda:us-west-2:{account_id}:function:{lambda_function_name}/invocations.
 In general, the URI has this form:
 arn:aws:apigateway:{region}:lambda:path/{service_api}
 , where {region} is the same as the region hosting the Lambda
 function, path indicates that the remaining substring in the URI should be treated as
 the path to the resource, including the initial /. For Lambda functions,
 this is usually of the form
 /2015-03-31/functions/[FunctionARN]/invocations.<br/>AuthorizerType:  The authorizer type. Currently the only valid value is REQUEST, for a
 Lambda function using incoming request parameters.<br/>AuthorizerResultTtlInSeconds:  The time to live (TTL), in seconds, of cached authorizer results. If it is zero,
 authorization caching is disabled. If it is greater than zero, API Gateway will cache
 authorizer responses. If this field is not set, the default value is 300. The maximum
 value is 3600, or 1 hour.<br/>AuthorizerId:  The authorizer identifier.<br/>AuthorizerCredentialsArn:  Specifies the required credentials as an IAM role for API Gateway to invoke the
 authorizer. To specify an IAM role for API Gateway to assume, use the role's Amazon
 Resource Name (ARN). To use resource-based permissions on the Lambda function,
 specify null.<br/>ApiId:  The API identifier.<br/>
        :return: ProviderArns:  For
 REQUEST authorizer, this is not
 defined.<br/>Name:  The name of the authorizer.<br/>IdentityValidationExpression:  The
 validation expression does not apply to the REQUEST authorizer.<br/>IdentitySource:  The identity source for which authorization is requested.For the REQUEST authorizer, this is required when authorization
 caching is enabled. The value is a comma-separated string of one or more mapping
 expressions of the specified request parameters. For example, if an Auth
 header and a Name query string parameters are defined as identity
 sources, this value is method.request.header.Auth,
 method.request.querystring.Name. These parameters will be used to
 derive the authorization caching key and to perform runtime validation of the
 REQUEST authorizer by verifying all of the identity-related request
 parameters are present, not null, and non-empty. Only when this is true does the
 authorizer invoke the authorizer Lambda function, otherwise, it returns a 401
 Unauthorized response without calling the Lambda function. The valid value
 is a string of comma-separated mapping expressions of the specified request
 parameters. When the authorization caching is not enabled, this property is
 optional.<br/>AuthorizerUri:  The authorizer's Uniform Resource Identifier (URI).
 ForREQUEST authorizers, this must be a
 well-formed Lambda function URI, for example,
 arn:aws:apigateway:us-west-2:lambda:path/2015-03-31/functions/arn:aws:lambda:us-west-2:{account_id}:function:{lambda_function_name}/invocations.
 In general, the URI has this form:
 arn:aws:apigateway:{region}:lambda:path/{service_api}
 , where {region} is the same as the region hosting the Lambda
 function, path indicates that the remaining substring in the URI should be treated as
 the path to the resource, including the initial /. For Lambda functions,
 this is usually of the form
 /2015-03-31/functions/[FunctionARN]/invocations.<br/>AuthorizerType:  The authorizer type. Currently the only valid value is REQUEST, for a
 Lambda function using incoming request parameters.<br/>AuthorizerResultTtlInSeconds:  The time to live (TTL), in seconds, of cached authorizer results. If it equals 0,
 authorization caching is disabled. If it is greater than 0, API Gateway will cache
 authorizer responses. If this field is not set, the default value is 300. The maximum
 value is 3600, or 1 hour.<br/>AuthorizerId:  The authorizer identifier.<br/>AuthorizerCredentialsArn:  Specifies the required credentials as an IAM role for API Gateway to invoke the
 authorizer. To specify an IAM role for API Gateway to assume, use the role's Amazon
 Resource Name (ARN). To use resource-based permissions on the Lambda function,
 specify null.<br/>        """
        pass

    def update_deployment(self, DeploymentId: str, ApiId: str, Description: str=None) -> UpdateDeploymentResponse:
        """Updates a Deployment.

        :param self:
        :param Description:  The description for the deployment resource.<br/>DeploymentId:  The deployment ID.<br/>ApiId:  The API identifier.<br/>
        :return: Description:  The description for the deployment.<br/>DeploymentStatusMessage:  May contain additional feedback on the status of an API deployment.<br/>DeploymentStatus:  The status of the deployment: PENDING, FAILED, or
 SUCCEEDED.<br/>DeploymentId:  The identifier for the deployment.<br/>CreatedDate:  The date and time when the Deployment resource was created.<br/>        """
        pass

    def update_domain_name(self, DomainName: str, DomainNameConfigurations: List[DomainNameConfiguration]=None) -> UpdateDomainNameResponse:
        """Updates a domain name.

        :param self:
        :param DomainNameConfigurations:  The domain name configurations.<br/>DomainName:  The domain name.<br/>
        :return: Tags:  The key-value map of strings. The valid character set is [a-zA-Z+-=._:/]. The tag key can be up to 128 characters and must not start with aws:. The tag value can be up to 256 characters..<br/>DomainNameConfigurations:  The domain name configurations.<br/>DomainName:  The name of the DomainName resource.<br/>ApiMappingSelectionExpression:  The API mapping selection expression.<br/>        """
        pass

    def update_integration(self, IntegrationId: str, ApiId: str, ConnectionId: str=None, ConnectionType: str=None, ContentHandlingStrategy: str=None, CredentialsArn: str=None, Description: str=None, IntegrationMethod: str=None, IntegrationType: str=None, IntegrationUri: str=None, PassthroughBehavior: str=None, RequestParameters: IntegrationParameters=None, RequestTemplates: TemplateMap=None, TemplateSelectionExpression: str=None, TimeoutInMillis: int=None) -> UpdateIntegrationResponse:
        """Updates an Integration.

        :param self:
        :param TimeoutInMillis:  Custom timeout between 50 and 29,000 milliseconds. The default value is 29,000
 milliseconds or 29 seconds.<br/>TemplateSelectionExpression:  The template selection expression for the integration.<br/>RequestTemplates:  Represents a map of Velocity templates that are applied on the request payload
 based on the value of the Content-Type header sent by the client. The content type
 value is the key in this map, and the template (as a String) is the value.<br/>RequestParameters:  A key-value map specifying request parameters that are passed from the method
 request to the backend. The key is an integration request parameter name and the
 associated value is a method request parameter value or static value that must be
 enclosed within single quotes and pre-encoded as required by the backend. The method
 request parameter value must match the pattern of
 method.request.{location}.{name}
 , where 
 {location}
  is querystring, path, or header; and 
 {name}
  must be a valid and unique method request parameter name.<br/>PassthroughBehavior:  Specifies the pass-through behavior for incoming requests based on the
 Content-Type header in the request, and the available mapping
 templates specified as the requestTemplates property on the
 Integration resource. There are three valid values:
 WHEN_NO_MATCH, WHEN_NO_TEMPLATES, and
 NEVER.
 WHEN_NO_MATCH passes the request body for unmapped content types through
 to the integration backend without transformation.
 NEVER rejects unmapped content types with an HTTP 415 Unsupported
 Media Type response.
 WHEN_NO_TEMPLATES allows pass-through when the integration has no
 content types mapped to templates. However, if there is at least one content type
 defined, unmapped content types will be rejected with the same HTTP 415
 Unsupported Media Type response.<br/>IntegrationUri:  For a Lambda proxy integration, this is the URI of the Lambda function.<br/>IntegrationType:  The integration type of an integration. One of the following:
 AWS: for integrating the route or method request with an AWS service
 action, including the Lambda function-invoking action. With the Lambda
 function-invoking action, this is referred to as the Lambda custom integration. With
 any other AWS service action, this is known as AWS integration.
 AWS_PROXY: for integrating the route or method request with the Lambda
 function-invoking action with the client request passed through as-is. This
 integration is also referred to as Lambda proxy integration.
 HTTP: for integrating the route or method request with an HTTP
 endpoint. This
 integration is also referred to as the HTTP custom integration.
 HTTP_PROXY: for integrating route or method request with an HTTP
 endpoint, with the client
 request passed through as-is. This is also referred to as HTTP proxy
 integration.
 MOCK: for integrating the route or method request with API Gateway as a
 "loopback" endpoint without invoking any backend.<br/>IntegrationMethod:  Specifies the integration's HTTP method type.<br/>IntegrationId:  The integration ID.<br/>Description:  The description of the integration<br/>CredentialsArn:  Specifies the credentials required for the integration, if any. For AWS
 integrations, three options are available. To specify an IAM Role for API Gateway to
 assume, use the role's Amazon Resource Name (ARN). To require that the caller's
 identity be passed through from the request, specify the string
 arn:aws:iam::*:user/*. To use resource-based permissions on supported
 AWS services, specify null.<br/>ContentHandlingStrategy:  Specifies how to handle response payload content type conversions. Supported
 values are CONVERT_TO_BINARY and CONVERT_TO_TEXT, with the
 following behaviors:
 CONVERT_TO_BINARY: Converts a response payload from a Base64-encoded
 string to the corresponding binary blob.
 CONVERT_TO_TEXT: Converts a response payload from a binary blob to a
 Base64-encoded string.If this property is not defined, the response payload will be passed through from
 the integration response to the route response or method response without
 modification.<br/>ConnectionType:  The type of the network connection to the integration endpoint. Currently the only
 valid value is INTERNET, for connections through the public routable
 internet.<br/>ConnectionId:  The connection ID.<br/>ApiId:  The API identifier.<br/>
        :return: TimeoutInMillis:  Custom timeout between 50 and 29,000 milliseconds. The default value is 29,000
 milliseconds or 29 seconds.<br/>TemplateSelectionExpression:  The template selection expression for the integration.<br/>RequestTemplates:  Represents a map of Velocity templates that are applied on the request payload
 based on the value of the Content-Type header sent by the client. The content type
 value is the key in this map, and the template (as a String) is the value.<br/>RequestParameters:  A key-value map specifying request parameters that are passed from the method
 request to the backend. The key is an integration request parameter name and the
 associated value is a method request parameter value or static value that must be
 enclosed within single quotes and pre-encoded as required by the backend. The method
 request parameter value must match the pattern of
 method.request.{location}.{name}
 , where 
 {location}
  is querystring, path, or header; and 
 {name}
  must be a valid and unique method request parameter name.<br/>PassthroughBehavior:  Specifies the pass-through behavior for incoming requests based on the
 Content-Type header in the request, and the available mapping
 templates specified as the requestTemplates property on the
 Integration resource. There are three valid values:
 WHEN_NO_MATCH, WHEN_NO_TEMPLATES, and
 NEVER.
 WHEN_NO_MATCH passes the request body for unmapped content types through
 to the integration backend without transformation.
 NEVER rejects unmapped content types with an HTTP 415 Unsupported
 Media Type response.
 WHEN_NO_TEMPLATES allows pass-through when the integration has no
 content types mapped to templates. However, if there is at least one content type
 defined, unmapped content types will be rejected with the same HTTP 415
 Unsupported Media Type response.<br/>IntegrationUri:  For a Lambda proxy integration, this is the URI of the Lambda function.<br/>IntegrationType:  The integration type of an integration. One of the following:
 AWS: for integrating the route or method request with an AWS service
 action, including the Lambda function-invoking action. With the Lambda
 function-invoking action, this is referred to as the Lambda custom integration. With
 any other AWS service action, this is known as AWS integration.
 AWS_PROXY: for integrating the route or method request with the Lambda
 function-invoking action with the client request passed through as-is. This
 integration is also referred to as Lambda proxy integration.
 HTTP: for integrating the route or method request with an HTTP
 endpoint. This
 integration is also referred to as the HTTP custom integration.
 HTTP_PROXY: for integrating route or method request with an HTTP
 endpoint, with the client
 request passed through as-is. This is also referred to as HTTP proxy
 integration.
 MOCK: for integrating the route or method request with API Gateway as a
 "loopback" endpoint without invoking any backend.<br/>IntegrationResponseSelectionExpression:  The integration response selection expression for the integration. See <a href="https://docs.aws.amazon.com/apigateway/latest/developerguide/apigateway-websocket-api-selection-expressions.html#apigateway-websocket-api-integration-response-selection-expressions">Integration Response Selection Expressions</a>.<br/>IntegrationMethod:  Specifies the integration's HTTP method type.<br/>IntegrationId:  Represents the identifier of an integration.<br/>Description:  Represents the description of an integration.<br/>CredentialsArn:  Specifies the credentials required for the integration, if any. For AWS
 integrations, three options are available. To specify an IAM Role for API Gateway to
 assume, use the role's Amazon Resource Name (ARN). To require that the caller's
 identity be passed through from the request, specify the string
 arn:aws:iam::*:user/*. To use resource-based permissions on supported
 AWS services, specify null.<br/>ContentHandlingStrategy:  Specifies how to handle response payload content type conversions. Supported
 values are CONVERT_TO_BINARY and CONVERT_TO_TEXT, with the
 following behaviors:
 CONVERT_TO_BINARY: Converts a response payload from a Base64-encoded
 string to the corresponding binary blob.
 CONVERT_TO_TEXT: Converts a response payload from a binary blob to a
 Base64-encoded string.If this property is not defined, the response payload will be passed through from
 the integration response to the route response or method response without
 modification.<br/>ConnectionType:  The type of the network connection to the integration endpoint. Currently the only
 valid value is INTERNET, for connections through the public routable
 internet.<br/>ConnectionId:  The connection ID.<br/>        """
        pass

    def update_integration_response(self, IntegrationResponseId: str, IntegrationId: str, ApiId: str, ContentHandlingStrategy: str=None, IntegrationResponseKey: str=None, ResponseParameters: IntegrationParameters=None, ResponseTemplates: TemplateMap=None, TemplateSelectionExpression: str=None) -> UpdateIntegrationResponseResponse:
        """Updates an IntegrationResponses.

        :param self:
        :param TemplateSelectionExpression:  The template selection expression for the integration response.<br/>ResponseTemplates:  The collection of response templates for the integration response as a
 string-to-string map of key-value pairs. Response templates are represented as a
 key/value map, with a content-type as the key and a template as the value.<br/>ResponseParameters:  A key-value map specifying response parameters that are passed to the method
 response from the backend. The key is a method response header parameter name and the
 mapped value is an integration response header value, a static value enclosed within
 a pair of single quotes, or a JSON expression from the integration response body. The
 mapping key must match the pattern of
 method.response.header.{name}
 , where name is a valid and unique header name. The mapped non-static value
 must match the pattern of
 integration.response.header.{name}
  or
 integration.response.body.{JSON-expression}
 , where 
 {name}
  is a valid and unique response header name and 
 {JSON-expression}
  is a valid JSON expression without the $ prefix.<br/>IntegrationResponseKey:  The integration response key.<br/>IntegrationResponseId:  The integration response ID.<br/>IntegrationId:  The integration ID.<br/>ContentHandlingStrategy:  Specifies how to handle response payload content type conversions. Supported
 values are CONVERT_TO_BINARY and CONVERT_TO_TEXT, with the
 following behaviors:
 CONVERT_TO_BINARY: Converts a response payload from a Base64-encoded
 string to the corresponding binary blob.
 CONVERT_TO_TEXT: Converts a response payload from a binary blob to a
 Base64-encoded string.If this property is not defined, the response payload will be passed through from
 the integration response to the route response or method response without
 modification.<br/>ApiId:  The API identifier.<br/>
        :return: TemplateSelectionExpression:  The template selection expressions for the integration response.<br/>ResponseTemplates:  The collection of response templates for the integration response as a
 string-to-string map of key-value pairs. Response templates are represented as a
 key/value map, with a content-type as the key and a template as the value.<br/>ResponseParameters:  A key-value map specifying response parameters that are passed to the method
 response from the backend. The key is a method response header parameter name and the
 mapped value is an integration response header value, a static value enclosed within
 a pair of single quotes, or a JSON expression from the integration response body. The
 mapping key must match the pattern of method.response.header.{name}, where name is a
 valid and unique header name. The mapped non-static value must match the pattern of
 integration.response.header.{name} or integration.response.body.{JSON-expression},
 where name is a valid and unique response header name and JSON-expression is a valid
 JSON expression without the $ prefix.<br/>IntegrationResponseKey:  The integration response key.<br/>IntegrationResponseId:  The integration response ID.<br/>ContentHandlingStrategy:  Specifies how to handle response payload content type conversions. Supported
 values are CONVERT_TO_BINARY and CONVERT_TO_TEXT, with the
 following behaviors:
 CONVERT_TO_BINARY: Converts a response payload from a Base64-encoded
 string to the corresponding binary blob.
 CONVERT_TO_TEXT: Converts a response payload from a binary blob to a
 Base64-encoded string.If this property is not defined, the response payload will be passed through from
 the integration response to the route response or method response without
 modification.<br/>        """
        pass

    def update_model(self, ModelId: str, ApiId: str, ContentType: str=None, Description: str=None, Name: str=None, Schema: str=None) -> UpdateModelResponse:
        """Updates a Model.

        :param self:
        :param Schema:  The schema for the model. For application/json models, this should be JSON schema
 draft 4 model.<br/>Name:  The name of the model.<br/>ModelId:  The model ID.<br/>Description:  The description of the model.<br/>ContentType:  The content-type for the model, for example, "application/json".<br/>ApiId:  The API identifier.<br/>
        :return: Schema:  The schema for the model. For application/json models, this should be JSON schema
 draft 4 model.<br/>Name:  The name of the model. Must be alphanumeric.<br/>ModelId:  The model identifier.<br/>Description:  The description of the model.<br/>ContentType:  The content-type for the model, for example, "application/json".<br/>        """
        pass

    def update_route(self, RouteId: str, ApiId: str, ApiKeyRequired: __boolean=None, AuthorizationScopes: List[StringWithLengthBetween1And64]=None, AuthorizationType: str=None, AuthorizerId: str=None, ModelSelectionExpression: str=None, OperationName: str=None, RequestModels: RouteModels=None, RequestParameters: RouteParameters=None, RouteKey: str=None, RouteResponseSelectionExpression: str=None, Target: str=None) -> UpdateRouteResponse:
        """Updates a Route.

        :param self:
        :param Target:  The target for the route.<br/>RouteResponseSelectionExpression:  The route response selection expression for the route.<br/>RouteKey:  The route key for the route.<br/>RouteId:  The route ID.<br/>RequestParameters:  The request parameters for the route.<br/>RequestModels:  The request models for the route.<br/>OperationName:  The operation name for the route.<br/>ModelSelectionExpression:  The model selection expression for the route.<br/>AuthorizerId:  The identifier of the Authorizer resource to be associated with this
 route, if the authorizationType is CUSTOM
 . The authorizer identifier is generated by API Gateway
 when you created the authorizer.<br/>AuthorizationType:  The authorization type for the route. Valid values are NONE for open
 access, AWS_IAM for using AWS IAM permissions, and CUSTOM
 for using a Lambda
 authorizer.<br/>AuthorizationScopes:  The authorization scopes supported by this
 route.<br/>ApiKeyRequired:  Specifies whether an API key is required for the route.<br/>ApiId:  The API identifier.<br/>
        :return: Target:  The target for the route.<br/>RouteResponseSelectionExpression:  The route response selection expression for the route.<br/>RouteKey:  The route key for the route.<br/>RouteId:  The route ID.<br/>RequestParameters:  The request parameters for the route.<br/>RequestModels:  The request models for the route.<br/>OperationName:  The operation name for the route.<br/>ModelSelectionExpression:  The model selection expression for the route.<br/>AuthorizerId:  The identifier of the Authorizer resource to be associated with this
 route, if the authorizationType is CUSTOM
 . The authorizer identifier is generated by API Gateway
 when you created the authorizer.<br/>AuthorizationType:  The authorization type for the route. Valid values are NONE for open
 access, AWS_IAM for using AWS IAM permissions, and CUSTOM
 for using a Lambda
 authorizer<br/>AuthorizationScopes:  A list of authorization scopes configured on a route. The scopes are used with a
 COGNITO_USER_POOLS authorizer to authorize the method invocation. The authorization
 works by matching the route scopes against the scopes parsed from the access token in
 the incoming request. The method invocation is authorized if any route scope matches
 a claimed scope in the access token. Otherwise, the invocation is not authorized.
 When the route scope is configured, the client must provide an access token instead
 of an identity token for authorization purposes.<br/>ApiKeyRequired:  Specifies whether an API key is required for this route.<br/>        """
        pass

    def update_route_response(self, RouteResponseId: str, RouteId: str, ApiId: str, ModelSelectionExpression: str=None, ResponseModels: RouteModels=None, ResponseParameters: RouteParameters=None, RouteResponseKey: str=None) -> UpdateRouteResponseResponse:
        """Updates a RouteResponse.

        :param self:
        :param RouteResponseKey:  The route response key.<br/>RouteResponseId:  The route response ID.<br/>RouteId:  The route ID.<br/>ResponseParameters:  The route response parameters.<br/>ResponseModels:  The response models for the route response.<br/>ModelSelectionExpression:  The model selection expression for the route response.<br/>ApiId:  The API identifier.<br/>
        :return: RouteResponseKey:  Represents the route response key of a route response.<br/>RouteResponseId:  Represents the identifier of a route response.<br/>ResponseParameters:  Represents the response parameters of a route response.<br/>ResponseModels:  Represents the response models of a route response.<br/>ModelSelectionExpression:  Represents the model selection expression of a route response.<br/>        """
        pass

    def update_stage(self, StageName: str, ApiId: str, AccessLogSettings: AccessLogSettings=None, ClientCertificateId: str=None, DefaultRouteSettings: RouteSettings=None, DeploymentId: str=None, Description: str=None, RouteSettings: RouteSettingsMap=None, StageVariables: StageVariablesMap=None) -> UpdateStageResponse:
        """Updates a Stage.

        :param self:
        :param StageVariables:  A map that defines the stage variables for a Stage. Variable names
 can have alphanumeric and underscore characters, and the values must match
 [A-Za-z0-9-._~:/?#&=,]+.<br/>StageName:  The stage name.<br/>RouteSettings:  Route settings for the stage.<br/>Description:  The description for the API stage.<br/>DeploymentId:  The deployment identifier for the API stage.<br/>DefaultRouteSettings:  The default route settings for the stage.:<br/>ClientCertificateId:  The identifier of a client certificate for a Stage.<br/>ApiId:  The API identifier.<br/>AccessLogSettings:  Settings for logging access in this stage.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> A single line format of the access logs of data, as specified by selected $context
 variables. The format must include at least $context.requestId.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationArn</i> The ARN of the CloudWatch Logs log group to receive access logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThrottlingRateLimit</i> Specifies the throttling rate limit.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThrottlingBurstLimit</i> Specifies the throttling burst limit.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LoggingLevel</i> Specifies the logging level for this route: DEBUG, INFO,
 or WARN. This property affects the log entries pushed to Amazon
 CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DetailedMetricsEnabled</i> Specifies whether detailed metrics are enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTraceEnabled</i> Specifies whether (true) or not (false) data trace
 logging is enabled for this route. This property affects the log entries pushed to
 Amazon CloudWatch Logs.<br/>
        :return: Tags:  The key-value map of strings. The valid character set is [a-zA-Z+-=._:/]. The tag key can be up to 128 characters and must not start with aws:. The tag value can be up to 256 characters..<br/>StageVariables:  A map that defines the stage variables for a stage resource. Variable names can
 have alphanumeric and underscore characters, and the values must match
 [A-Za-z0-9-._~:/?#&=,]+.<br/>StageName:  The name of the stage.<br/>RouteSettings:  Route settings for the stage.<br/>LastUpdatedDate:  The timestamp when the stage was last updated.<br/>Description:  The description of the stage.<br/>DeploymentId:  The identifier of the Deployment that the Stage is
 associated with.<br/>DefaultRouteSettings:  Default route settings for the stage.:<br/>CreatedDate:  The timestamp when the stage was created.<br/>ClientCertificateId:  The identifier of a client certificate for a Stage.<br/>AccessLogSettings:  Settings for logging access in this stage.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> A single line format of the access logs of data, as specified by selected $context
 variables. The format must include at least $context.requestId.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationArn</i> The ARN of the CloudWatch Logs log group to receive access logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThrottlingRateLimit</i> Specifies the throttling rate limit.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThrottlingBurstLimit</i> Specifies the throttling burst limit.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LoggingLevel</i> Specifies the logging level for this route: DEBUG, INFO,
 or WARN. This property affects the log entries pushed to Amazon
 CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DetailedMetricsEnabled</i> Specifies whether detailed metrics are enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTraceEnabled</i> Specifies whether (true) or not (false) data trace
 logging is enabled for this route. This property affects the log entries pushed to
 Amazon CloudWatch Logs.<br/>        """
        pass



class ApplicationAutoScaling(BaseClient):
    """With Application Auto Scaling, you can configure automatic scaling for your scalable resources. You can use Application Auto Scaling to accomplish the following tasks: <ul> <li> Define scaling policies to automatically scale your AWS or custom resources </li> <li> Scale your resources in response to CloudWatch alarms </li> <li> Schedule one-time or recurring scaling actions </li> <li> View the history of your scaling events </li> </ul> Application Auto Scaling can scale the following resources: <ul> <li> Amazon ECS services. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/service-auto-scaling.html">Service Auto Scaling</a> in the <i>Amazon Elastic Container Service Developer Guide</i>. </li> <li> Amazon EC2 Spot fleets. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/fleet-auto-scaling.html">Automatic Scaling for Spot Fleet</a> in the <i>Amazon EC2 User Guide</i>. </li> <li> Amazon EMR clusters. For more information, see <a href="https://docs.aws.amazon.com/ElasticMapReduce/latest/ManagementGuide/emr-automatic-scaling.html">Using Automatic Scaling in Amazon EMR</a> in the <i>Amazon EMR Management Guide</i>. </li> <li> AppStream 2.0 fleets. For more information, see <a href="https://docs.aws.amazon.com/appstream2/latest/developerguide/autoscaling.html">Fleet Auto Scaling for Amazon AppStream 2.0</a> in the <i>Amazon AppStream 2.0 Developer Guide</i>. </li> <li> Provisioned read and write capacity for Amazon DynamoDB tables and global secondary indexes. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/AutoScaling.html">Managing Throughput Capacity Automatically with DynamoDB Auto Scaling</a> in the <i>Amazon DynamoDB Developer Guide</i>. </li> <li> Amazon Aurora Replicas. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/Aurora.Integrating.AutoScaling.html">Using Amazon Aurora Auto Scaling with Aurora Replicas</a>. </li> <li> Amazon SageMaker endpoint variants. For more information, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/endpoint-auto-scaling.html">Automatically Scaling Amazon SageMaker Models</a>. </li> <li> Custom resources provided by your own applications or services. More information is available in our <a href="https://github.com/aws/aws-auto-scaling-custom-resource">GitHub repository</a>.  </li> </ul> To learn more about Application Auto Scaling, including information about granting IAM users required permissions for Application Auto Scaling actions, see the <a href="https://docs.aws.amazon.com/autoscaling/application/userguide/what-is-application-auto-scaling.html">Application Auto Scaling User Guide</a>."""


    class DescribeScalableTargetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_scalable_targets` for valid parameters"""
            pass
    class DescribeScalingActivitiesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_scaling_activities` for valid parameters"""
            pass
    class DescribeScalingPoliciesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_scaling_policies` for valid parameters"""
            pass
    class DescribeScheduledActionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_scheduled_actions` for valid parameters"""
            pass


    class DeleteScalingPolicyRequest(Mapping):
        pass
    
    class DeleteScalingPolicyResponse(Mapping):
        pass
    
    class DeleteScheduledActionRequest(Mapping):
        pass
    
    class DeleteScheduledActionResponse(Mapping):
        pass
    
    class DeregisterScalableTargetRequest(Mapping):
        pass
    
    class DeregisterScalableTargetResponse(Mapping):
        pass
    
    class DescribeScalableTargetsRequest(Mapping):
        pass
    
    class DescribeScalableTargetsResponse(Mapping):
        pass
    
    class ResourceIdMaxLen1600(object):
        pass
    
    class DescribeScalingActivitiesRequest(Mapping):
        pass
    
    class DescribeScalingActivitiesResponse(Mapping):
        pass
    
    class DescribeScalingPoliciesRequest(Mapping):
        pass
    
    class DescribeScalingPoliciesResponse(Mapping):
        pass
    
    class DescribeScheduledActionsRequest(Mapping):
        pass
    
    class DescribeScheduledActionsResponse(Mapping):
        pass
    
    class PutScalingPolicyRequest(Mapping):
        pass
    
    class PutScalingPolicyResponse(Mapping):
        pass
    
    class StepScalingPolicyConfiguration(Mapping):
        pass
    
    class TargetTrackingScalingPolicyConfiguration(Mapping):
        pass
    
    class PutScheduledActionRequest(Mapping):
        pass
    
    class PutScheduledActionResponse(Mapping):
        pass
    
    class TimestampType(object):
        pass
    
    class ScalableTargetAction(Mapping):
        pass
    
    class RegisterScalableTargetRequest(Mapping):
        pass
    
    class RegisterScalableTargetResponse(Mapping):
        pass
    
    def delete_scaling_policy(self, ScalableDimension: str, ResourceId: str, ServiceNamespace: str, PolicyName: str) -> DeleteScalingPolicyResponse:
        """Deletes the specified Application Auto Scaling scaling policy. Deleting a policy deletes the underlying alarm action, but does not delete the CloudWatch alarm associated with the scaling policy, even if it no longer has an associated action. To create a scaling policy or update an existing one, see <a>PutScalingPolicy</a>.

        :param self:
        :param ScalableDimension:  The scalable dimension. This string consists of the service namespace, resource type, and scaling property. <ul> <li>  <code>ecs:service:DesiredCount</code> - The desired task count of an ECS service. </li> <li>  <code>ec2:spot-fleet-request:TargetCapacity</code> - The target capacity of a Spot fleet request. </li> <li>  <code>elasticmapreduce:instancegroup:InstanceCount</code> - The instance count of an EMR Instance Group. </li> <li>  <code>appstream:fleet:DesiredCapacity</code> - The desired capacity of an AppStream 2.0 fleet. </li> <li>  <code>dynamodb:table:ReadCapacityUnits</code> - The provisioned read capacity for a DynamoDB table. </li> <li>  <code>dynamodb:table:WriteCapacityUnits</code> - The provisioned write capacity for a DynamoDB table. </li> <li>  <code>dynamodb:index:ReadCapacityUnits</code> - The provisioned read capacity for a DynamoDB global secondary index. </li> <li>  <code>dynamodb:index:WriteCapacityUnits</code> - The provisioned write capacity for a DynamoDB global secondary index. </li> <li>  <code>rds:cluster:ReadReplicaCount</code> - The count of Aurora Replicas in an Aurora DB cluster. Available for Aurora MySQL-compatible edition and Aurora PostgreSQL-compatible edition. </li> <li>  <code>sagemaker:variant:DesiredInstanceCount</code> - The number of EC2 instances for an Amazon SageMaker model endpoint variant. </li> <li>  <code>custom-resource:ResourceType:Property</code> - The scalable dimension for a custom resource provided by your own application or service. </li> </ul><br/>ResourceId:  The identifier of the resource associated with the scalable target. This string consists of the resource type and unique identifier. <ul> <li> ECS service - The resource type is <code>service</code> and the unique identifier is the cluster name and service name. Example: <code>service/default/sample-webapp</code>. </li> <li> Spot fleet request - The resource type is <code>spot-fleet-request</code> and the unique identifier is the Spot fleet request ID. Example: <code>spot-fleet-request/sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE</code>. </li> <li> EMR cluster - The resource type is <code>instancegroup</code> and the unique identifier is the cluster ID and instance group ID. Example: <code>instancegroup/j-2EEZNYKUA1NTV/ig-1791Y4E1L8YI0</code>. </li> <li> AppStream 2.0 fleet - The resource type is <code>fleet</code> and the unique identifier is the fleet name. Example: <code>fleet/sample-fleet</code>. </li> <li> DynamoDB table - The resource type is <code>table</code> and the unique identifier is the resource ID. Example: <code>table/my-table</code>. </li> <li> DynamoDB global secondary index - The resource type is <code>index</code> and the unique identifier is the resource ID. Example: <code>table/my-table/index/my-table-index</code>. </li> <li> Aurora DB cluster - The resource type is <code>cluster</code> and the unique identifier is the cluster name. Example: <code>cluster:my-db-cluster</code>. </li> <li> Amazon SageMaker endpoint variants - The resource type is <code>variant</code> and the unique identifier is the resource ID. Example: <code>endpoint/my-end-point/variant/KMeansClustering</code>. </li> <li> Custom resources are not supported with a resource type. This parameter must specify the <code>OutputValue</code> from the CloudFormation template stack used to access the resources. The unique identifier is defined by the service provider. More information is available in our <a href="https://github.com/aws/aws-auto-scaling-custom-resource">GitHub repository</a>. </li> </ul><br/>ServiceNamespace:  The namespace of the AWS service that provides the resource or <code>custom-resource</code> for a resource provided by your own application or service. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#genref-aws-service-namespaces">AWS Service Namespaces</a> in the <i>Amazon Web Services General Reference</i>.<br/>PolicyName:  The name of the scaling policy.<br/>
        :return:         """
        pass

    def delete_scheduled_action(self, ScalableDimension: str, ResourceId: str, ScheduledActionName: str, ServiceNamespace: str) -> DeleteScheduledActionResponse:
        """Deletes the specified Application Auto Scaling scheduled action.

        :param self:
        :param ScalableDimension:  The scalable dimension. This string consists of the service namespace, resource type, and scaling property. <ul> <li>  <code>ecs:service:DesiredCount</code> - The desired task count of an ECS service. </li> <li>  <code>ec2:spot-fleet-request:TargetCapacity</code> - The target capacity of a Spot fleet request. </li> <li>  <code>elasticmapreduce:instancegroup:InstanceCount</code> - The instance count of an EMR Instance Group. </li> <li>  <code>appstream:fleet:DesiredCapacity</code> - The desired capacity of an AppStream 2.0 fleet. </li> <li>  <code>dynamodb:table:ReadCapacityUnits</code> - The provisioned read capacity for a DynamoDB table. </li> <li>  <code>dynamodb:table:WriteCapacityUnits</code> - The provisioned write capacity for a DynamoDB table. </li> <li>  <code>dynamodb:index:ReadCapacityUnits</code> - The provisioned read capacity for a DynamoDB global secondary index. </li> <li>  <code>dynamodb:index:WriteCapacityUnits</code> - The provisioned write capacity for a DynamoDB global secondary index. </li> <li>  <code>rds:cluster:ReadReplicaCount</code> - The count of Aurora Replicas in an Aurora DB cluster. Available for Aurora MySQL-compatible edition and Aurora PostgreSQL-compatible edition. </li> <li>  <code>sagemaker:variant:DesiredInstanceCount</code> - The number of EC2 instances for an Amazon SageMaker model endpoint variant. </li> <li>  <code>custom-resource:ResourceType:Property</code> - The scalable dimension for a custom resource provided by your own application or service. </li> </ul><br/>ResourceId:  The identifier of the resource associated with the scheduled action. This string consists of the resource type and unique identifier. <ul> <li> ECS service - The resource type is <code>service</code> and the unique identifier is the cluster name and service name. Example: <code>service/default/sample-webapp</code>. </li> <li> Spot fleet request - The resource type is <code>spot-fleet-request</code> and the unique identifier is the Spot fleet request ID. Example: <code>spot-fleet-request/sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE</code>. </li> <li> EMR cluster - The resource type is <code>instancegroup</code> and the unique identifier is the cluster ID and instance group ID. Example: <code>instancegroup/j-2EEZNYKUA1NTV/ig-1791Y4E1L8YI0</code>. </li> <li> AppStream 2.0 fleet - The resource type is <code>fleet</code> and the unique identifier is the fleet name. Example: <code>fleet/sample-fleet</code>. </li> <li> DynamoDB table - The resource type is <code>table</code> and the unique identifier is the resource ID. Example: <code>table/my-table</code>. </li> <li> DynamoDB global secondary index - The resource type is <code>index</code> and the unique identifier is the resource ID. Example: <code>table/my-table/index/my-table-index</code>. </li> <li> Aurora DB cluster - The resource type is <code>cluster</code> and the unique identifier is the cluster name. Example: <code>cluster:my-db-cluster</code>. </li> <li> Amazon SageMaker endpoint variants - The resource type is <code>variant</code> and the unique identifier is the resource ID. Example: <code>endpoint/my-end-point/variant/KMeansClustering</code>. </li> <li> Custom resources are not supported with a resource type. This parameter must specify the <code>OutputValue</code> from the CloudFormation template stack used to access the resources. The unique identifier is defined by the service provider. More information is available in our <a href="https://github.com/aws/aws-auto-scaling-custom-resource">GitHub repository</a>. </li> </ul><br/>ScheduledActionName:  The name of the scheduled action.<br/>ServiceNamespace:  The namespace of the AWS service that provides the resource or <code>custom-resource</code> for a resource provided by your own application or service. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#genref-aws-service-namespaces">AWS Service Namespaces</a> in the <i>Amazon Web Services General Reference</i>.<br/>
        :return:         """
        pass

    def deregister_scalable_target(self, ScalableDimension: str, ResourceId: str, ServiceNamespace: str) -> DeregisterScalableTargetResponse:
        """Deregisters a scalable target. Deregistering a scalable target deletes the scaling policies that are associated with it. To create a scalable target or update an existing one, see <a>RegisterScalableTarget</a>. 

        :param self:
        :param ScalableDimension:  The scalable dimension associated with the scalable target. This string consists of the service namespace, resource type, and scaling property. <ul> <li>  <code>ecs:service:DesiredCount</code> - The desired task count of an ECS service. </li> <li>  <code>ec2:spot-fleet-request:TargetCapacity</code> - The target capacity of a Spot fleet request. </li> <li>  <code>elasticmapreduce:instancegroup:InstanceCount</code> - The instance count of an EMR Instance Group. </li> <li>  <code>appstream:fleet:DesiredCapacity</code> - The desired capacity of an AppStream 2.0 fleet. </li> <li>  <code>dynamodb:table:ReadCapacityUnits</code> - The provisioned read capacity for a DynamoDB table. </li> <li>  <code>dynamodb:table:WriteCapacityUnits</code> - The provisioned write capacity for a DynamoDB table. </li> <li>  <code>dynamodb:index:ReadCapacityUnits</code> - The provisioned read capacity for a DynamoDB global secondary index. </li> <li>  <code>dynamodb:index:WriteCapacityUnits</code> - The provisioned write capacity for a DynamoDB global secondary index. </li> <li>  <code>rds:cluster:ReadReplicaCount</code> - The count of Aurora Replicas in an Aurora DB cluster. Available for Aurora MySQL-compatible edition and Aurora PostgreSQL-compatible edition. </li> <li>  <code>sagemaker:variant:DesiredInstanceCount</code> - The number of EC2 instances for an Amazon SageMaker model endpoint variant. </li> <li>  <code>custom-resource:ResourceType:Property</code> - The scalable dimension for a custom resource provided by your own application or service. </li> </ul><br/>ResourceId:  The identifier of the resource associated with the scalable target. This string consists of the resource type and unique identifier. <ul> <li> ECS service - The resource type is <code>service</code> and the unique identifier is the cluster name and service name. Example: <code>service/default/sample-webapp</code>. </li> <li> Spot fleet request - The resource type is <code>spot-fleet-request</code> and the unique identifier is the Spot fleet request ID. Example: <code>spot-fleet-request/sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE</code>. </li> <li> EMR cluster - The resource type is <code>instancegroup</code> and the unique identifier is the cluster ID and instance group ID. Example: <code>instancegroup/j-2EEZNYKUA1NTV/ig-1791Y4E1L8YI0</code>. </li> <li> AppStream 2.0 fleet - The resource type is <code>fleet</code> and the unique identifier is the fleet name. Example: <code>fleet/sample-fleet</code>. </li> <li> DynamoDB table - The resource type is <code>table</code> and the unique identifier is the resource ID. Example: <code>table/my-table</code>. </li> <li> DynamoDB global secondary index - The resource type is <code>index</code> and the unique identifier is the resource ID. Example: <code>table/my-table/index/my-table-index</code>. </li> <li> Aurora DB cluster - The resource type is <code>cluster</code> and the unique identifier is the cluster name. Example: <code>cluster:my-db-cluster</code>. </li> <li> Amazon SageMaker endpoint variants - The resource type is <code>variant</code> and the unique identifier is the resource ID. Example: <code>endpoint/my-end-point/variant/KMeansClustering</code>. </li> <li> Custom resources are not supported with a resource type. This parameter must specify the <code>OutputValue</code> from the CloudFormation template stack used to access the resources. The unique identifier is defined by the service provider. More information is available in our <a href="https://github.com/aws/aws-auto-scaling-custom-resource">GitHub repository</a>. </li> </ul><br/>ServiceNamespace:  The namespace of the AWS service that provides the resource or <code>custom-resource</code> for a resource provided by your own application or service. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#genref-aws-service-namespaces">AWS Service Namespaces</a> in the <i>Amazon Web Services General Reference</i>.<br/>
        :return:         """
        pass

    def describe_scalable_targets(self, ServiceNamespace: str, ResourceIds: List[ResourceIdMaxLen1600]=None, ScalableDimension: str=None, MaxResults: int=None, NextToken: str=None) -> DescribeScalableTargetsResponse:
        """Gets information about the scalable targets in the specified namespace. You can filter the results using the <code>ResourceIds</code> and <code>ScalableDimension</code> parameters. To create a scalable target or update an existing one, see <a>RegisterScalableTarget</a>. If you are no longer using a scalable target, you can deregister it using <a>DeregisterScalableTarget</a>.

        :param self:
        :param NextToken:  The token for the next set of results.<br/>MaxResults:  The maximum number of scalable targets. This value can be between 1 and 50. The default value is 50. If this parameter is used, the operation returns up to <code>MaxResults</code> results at a time, along with a <code>NextToken</code> value. To get the next set of results, include the <code>NextToken</code> value in a subsequent call. If this parameter is not used, the operation returns up to 50 results and a <code>NextToken</code> value, if applicable.<br/>ScalableDimension:  The scalable dimension associated with the scalable target. This string consists of the service namespace, resource type, and scaling property. If you specify a scalable dimension, you must also specify a resource ID. <ul> <li>  <code>ecs:service:DesiredCount</code> - The desired task count of an ECS service. </li> <li>  <code>ec2:spot-fleet-request:TargetCapacity</code> - The target capacity of a Spot fleet request. </li> <li>  <code>elasticmapreduce:instancegroup:InstanceCount</code> - The instance count of an EMR Instance Group. </li> <li>  <code>appstream:fleet:DesiredCapacity</code> - The desired capacity of an AppStream 2.0 fleet. </li> <li>  <code>dynamodb:table:ReadCapacityUnits</code> - The provisioned read capacity for a DynamoDB table. </li> <li>  <code>dynamodb:table:WriteCapacityUnits</code> - The provisioned write capacity for a DynamoDB table. </li> <li>  <code>dynamodb:index:ReadCapacityUnits</code> - The provisioned read capacity for a DynamoDB global secondary index. </li> <li>  <code>dynamodb:index:WriteCapacityUnits</code> - The provisioned write capacity for a DynamoDB global secondary index. </li> <li>  <code>rds:cluster:ReadReplicaCount</code> - The count of Aurora Replicas in an Aurora DB cluster. Available for Aurora MySQL-compatible edition and Aurora PostgreSQL-compatible edition. </li> <li>  <code>sagemaker:variant:DesiredInstanceCount</code> - The number of EC2 instances for an Amazon SageMaker model endpoint variant. </li> <li>  <code>custom-resource:ResourceType:Property</code> - The scalable dimension for a custom resource provided by your own application or service. </li> </ul><br/>ResourceIds:  The identifier of the resource associated with the scalable target. This string consists of the resource type and unique identifier. If you specify a scalable dimension, you must also specify a resource ID. <ul> <li> ECS service - The resource type is <code>service</code> and the unique identifier is the cluster name and service name. Example: <code>service/default/sample-webapp</code>. </li> <li> Spot fleet request - The resource type is <code>spot-fleet-request</code> and the unique identifier is the Spot fleet request ID. Example: <code>spot-fleet-request/sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE</code>. </li> <li> EMR cluster - The resource type is <code>instancegroup</code> and the unique identifier is the cluster ID and instance group ID. Example: <code>instancegroup/j-2EEZNYKUA1NTV/ig-1791Y4E1L8YI0</code>. </li> <li> AppStream 2.0 fleet - The resource type is <code>fleet</code> and the unique identifier is the fleet name. Example: <code>fleet/sample-fleet</code>. </li> <li> DynamoDB table - The resource type is <code>table</code> and the unique identifier is the resource ID. Example: <code>table/my-table</code>. </li> <li> DynamoDB global secondary index - The resource type is <code>index</code> and the unique identifier is the resource ID. Example: <code>table/my-table/index/my-table-index</code>. </li> <li> Aurora DB cluster - The resource type is <code>cluster</code> and the unique identifier is the cluster name. Example: <code>cluster:my-db-cluster</code>. </li> <li> Amazon SageMaker endpoint variants - The resource type is <code>variant</code> and the unique identifier is the resource ID. Example: <code>endpoint/my-end-point/variant/KMeansClustering</code>. </li> <li> Custom resources are not supported with a resource type. This parameter must specify the <code>OutputValue</code> from the CloudFormation template stack used to access the resources. The unique identifier is defined by the service provider. More information is available in our <a href="https://github.com/aws/aws-auto-scaling-custom-resource">GitHub repository</a>. </li> </ul><br/>ServiceNamespace:  The namespace of the AWS service that provides the resource or <code>custom-resource</code> for a resource provided by your own application or service. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#genref-aws-service-namespaces">AWS Service Namespaces</a> in the <i>Amazon Web Services General Reference</i>.<br/>
        :return: NextToken:  The token required to get the next set of results. This value is <code>null</code> if there are no more results to return.<br/>ScalableTargets:  The scalable targets that match the request parameters.<br/>        """
        pass

    def describe_scaling_activities(self, ServiceNamespace: str, ResourceId: str=None, ScalableDimension: str=None, MaxResults: int=None, NextToken: str=None) -> DescribeScalingActivitiesResponse:
        """Provides descriptive information about the scaling activities in the specified namespace from the previous six weeks. You can filter the results using the <code>ResourceId</code> and <code>ScalableDimension</code> parameters. Scaling activities are triggered by CloudWatch alarms that are associated with scaling policies. To view the scaling policies for a service namespace, see <a>DescribeScalingPolicies</a>. To create a scaling policy or update an existing one, see <a>PutScalingPolicy</a>.

        :param self:
        :param NextToken:  The token for the next set of results.<br/>MaxResults:  The maximum number of scalable targets. This value can be between 1 and 50. The default value is 50. If this parameter is used, the operation returns up to <code>MaxResults</code> results at a time, along with a <code>NextToken</code> value. To get the next set of results, include the <code>NextToken</code> value in a subsequent call. If this parameter is not used, the operation returns up to 50 results and a <code>NextToken</code> value, if applicable.<br/>ScalableDimension:  The scalable dimension. This string consists of the service namespace, resource type, and scaling property. If you specify a scalable dimension, you must also specify a resource ID. <ul> <li>  <code>ecs:service:DesiredCount</code> - The desired task count of an ECS service. </li> <li>  <code>ec2:spot-fleet-request:TargetCapacity</code> - The target capacity of a Spot fleet request. </li> <li>  <code>elasticmapreduce:instancegroup:InstanceCount</code> - The instance count of an EMR Instance Group. </li> <li>  <code>appstream:fleet:DesiredCapacity</code> - The desired capacity of an AppStream 2.0 fleet. </li> <li>  <code>dynamodb:table:ReadCapacityUnits</code> - The provisioned read capacity for a DynamoDB table. </li> <li>  <code>dynamodb:table:WriteCapacityUnits</code> - The provisioned write capacity for a DynamoDB table. </li> <li>  <code>dynamodb:index:ReadCapacityUnits</code> - The provisioned read capacity for a DynamoDB global secondary index. </li> <li>  <code>dynamodb:index:WriteCapacityUnits</code> - The provisioned write capacity for a DynamoDB global secondary index. </li> <li>  <code>rds:cluster:ReadReplicaCount</code> - The count of Aurora Replicas in an Aurora DB cluster. Available for Aurora MySQL-compatible edition and Aurora PostgreSQL-compatible edition. </li> <li>  <code>sagemaker:variant:DesiredInstanceCount</code> - The number of EC2 instances for an Amazon SageMaker model endpoint variant. </li> <li>  <code>custom-resource:ResourceType:Property</code> - The scalable dimension for a custom resource provided by your own application or service. </li> </ul><br/>ResourceId:  The identifier of the resource associated with the scaling activity. This string consists of the resource type and unique identifier. If you specify a scalable dimension, you must also specify a resource ID. <ul> <li> ECS service - The resource type is <code>service</code> and the unique identifier is the cluster name and service name. Example: <code>service/default/sample-webapp</code>. </li> <li> Spot fleet request - The resource type is <code>spot-fleet-request</code> and the unique identifier is the Spot fleet request ID. Example: <code>spot-fleet-request/sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE</code>. </li> <li> EMR cluster - The resource type is <code>instancegroup</code> and the unique identifier is the cluster ID and instance group ID. Example: <code>instancegroup/j-2EEZNYKUA1NTV/ig-1791Y4E1L8YI0</code>. </li> <li> AppStream 2.0 fleet - The resource type is <code>fleet</code> and the unique identifier is the fleet name. Example: <code>fleet/sample-fleet</code>. </li> <li> DynamoDB table - The resource type is <code>table</code> and the unique identifier is the resource ID. Example: <code>table/my-table</code>. </li> <li> DynamoDB global secondary index - The resource type is <code>index</code> and the unique identifier is the resource ID. Example: <code>table/my-table/index/my-table-index</code>. </li> <li> Aurora DB cluster - The resource type is <code>cluster</code> and the unique identifier is the cluster name. Example: <code>cluster:my-db-cluster</code>. </li> <li> Amazon SageMaker endpoint variants - The resource type is <code>variant</code> and the unique identifier is the resource ID. Example: <code>endpoint/my-end-point/variant/KMeansClustering</code>. </li> <li> Custom resources are not supported with a resource type. This parameter must specify the <code>OutputValue</code> from the CloudFormation template stack used to access the resources. The unique identifier is defined by the service provider. More information is available in our <a href="https://github.com/aws/aws-auto-scaling-custom-resource">GitHub repository</a>. </li> </ul><br/>ServiceNamespace:  The namespace of the AWS service that provides the resource or <code>custom-resource</code> for a resource provided by your own application or service. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#genref-aws-service-namespaces">AWS Service Namespaces</a> in the <i>Amazon Web Services General Reference</i>.<br/>
        :return: NextToken:  The token required to get the next set of results. This value is <code>null</code> if there are no more results to return.<br/>ScalingActivities:  A list of scaling activity objects.<br/>        """
        pass

    def describe_scaling_policies(self, ServiceNamespace: str, PolicyNames: List[ResourceIdMaxLen1600]=None, ResourceId: str=None, ScalableDimension: str=None, MaxResults: int=None, NextToken: str=None) -> DescribeScalingPoliciesResponse:
        """Describes the scaling policies for the specified service namespace. You can filter the results using the <code>ResourceId</code>, <code>ScalableDimension</code>, and <code>PolicyNames</code> parameters. To create a scaling policy or update an existing one, see <a>PutScalingPolicy</a>. If you are no longer using a scaling policy, you can delete it using <a>DeleteScalingPolicy</a>.

        :param self:
        :param NextToken:  The token for the next set of results.<br/>MaxResults:  The maximum number of scalable targets. This value can be between 1 and 50. The default value is 50. If this parameter is used, the operation returns up to <code>MaxResults</code> results at a time, along with a <code>NextToken</code> value. To get the next set of results, include the <code>NextToken</code> value in a subsequent call. If this parameter is not used, the operation returns up to 50 results and a <code>NextToken</code> value, if applicable.<br/>ScalableDimension:  The scalable dimension. This string consists of the service namespace, resource type, and scaling property. If you specify a scalable dimension, you must also specify a resource ID. <ul> <li>  <code>ecs:service:DesiredCount</code> - The desired task count of an ECS service. </li> <li>  <code>ec2:spot-fleet-request:TargetCapacity</code> - The target capacity of a Spot fleet request. </li> <li>  <code>elasticmapreduce:instancegroup:InstanceCount</code> - The instance count of an EMR Instance Group. </li> <li>  <code>appstream:fleet:DesiredCapacity</code> - The desired capacity of an AppStream 2.0 fleet. </li> <li>  <code>dynamodb:table:ReadCapacityUnits</code> - The provisioned read capacity for a DynamoDB table. </li> <li>  <code>dynamodb:table:WriteCapacityUnits</code> - The provisioned write capacity for a DynamoDB table. </li> <li>  <code>dynamodb:index:ReadCapacityUnits</code> - The provisioned read capacity for a DynamoDB global secondary index. </li> <li>  <code>dynamodb:index:WriteCapacityUnits</code> - The provisioned write capacity for a DynamoDB global secondary index. </li> <li>  <code>rds:cluster:ReadReplicaCount</code> - The count of Aurora Replicas in an Aurora DB cluster. Available for Aurora MySQL-compatible edition and Aurora PostgreSQL-compatible edition. </li> <li>  <code>sagemaker:variant:DesiredInstanceCount</code> - The number of EC2 instances for an Amazon SageMaker model endpoint variant. </li> <li>  <code>custom-resource:ResourceType:Property</code> - The scalable dimension for a custom resource provided by your own application or service. </li> </ul><br/>ResourceId:  The identifier of the resource associated with the scaling policy. This string consists of the resource type and unique identifier. If you specify a scalable dimension, you must also specify a resource ID. <ul> <li> ECS service - The resource type is <code>service</code> and the unique identifier is the cluster name and service name. Example: <code>service/default/sample-webapp</code>. </li> <li> Spot fleet request - The resource type is <code>spot-fleet-request</code> and the unique identifier is the Spot fleet request ID. Example: <code>spot-fleet-request/sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE</code>. </li> <li> EMR cluster - The resource type is <code>instancegroup</code> and the unique identifier is the cluster ID and instance group ID. Example: <code>instancegroup/j-2EEZNYKUA1NTV/ig-1791Y4E1L8YI0</code>. </li> <li> AppStream 2.0 fleet - The resource type is <code>fleet</code> and the unique identifier is the fleet name. Example: <code>fleet/sample-fleet</code>. </li> <li> DynamoDB table - The resource type is <code>table</code> and the unique identifier is the resource ID. Example: <code>table/my-table</code>. </li> <li> DynamoDB global secondary index - The resource type is <code>index</code> and the unique identifier is the resource ID. Example: <code>table/my-table/index/my-table-index</code>. </li> <li> Aurora DB cluster - The resource type is <code>cluster</code> and the unique identifier is the cluster name. Example: <code>cluster:my-db-cluster</code>. </li> <li> Amazon SageMaker endpoint variants - The resource type is <code>variant</code> and the unique identifier is the resource ID. Example: <code>endpoint/my-end-point/variant/KMeansClustering</code>. </li> <li> Custom resources are not supported with a resource type. This parameter must specify the <code>OutputValue</code> from the CloudFormation template stack used to access the resources. The unique identifier is defined by the service provider. More information is available in our <a href="https://github.com/aws/aws-auto-scaling-custom-resource">GitHub repository</a>. </li> </ul><br/>ServiceNamespace:  The namespace of the AWS service that provides the resource or <code>custom-resource</code> for a resource provided by your own application or service. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#genref-aws-service-namespaces">AWS Service Namespaces</a> in the <i>Amazon Web Services General Reference</i>.<br/>PolicyNames:  The names of the scaling policies to describe.<br/>
        :return: NextToken:  The token required to get the next set of results. This value is <code>null</code> if there are no more results to return.<br/>ScalingPolicies:  Information about the scaling policies.<br/>        """
        pass

    def describe_scheduled_actions(self, ServiceNamespace: str, ScheduledActionNames: List[ResourceIdMaxLen1600]=None, ResourceId: str=None, ScalableDimension: str=None, MaxResults: int=None, NextToken: str=None) -> DescribeScheduledActionsResponse:
        """Describes the scheduled actions for the specified service namespace. You can filter the results using the <code>ResourceId</code>, <code>ScalableDimension</code>, and <code>ScheduledActionNames</code> parameters. To create a scheduled action or update an existing one, see <a>PutScheduledAction</a>. If you are no longer using a scheduled action, you can delete it using <a>DeleteScheduledAction</a>.

        :param self:
        :param NextToken:  The token for the next set of results.<br/>MaxResults:  The maximum number of scheduled action results. This value can be between 1 and 50. The default value is 50. If this parameter is used, the operation returns up to <code>MaxResults</code> results at a time, along with a <code>NextToken</code> value. To get the next set of results, include the <code>NextToken</code> value in a subsequent call. If this parameter is not used, the operation returns up to 50 results and a <code>NextToken</code> value, if applicable.<br/>ScalableDimension:  The scalable dimension. This string consists of the service namespace, resource type, and scaling property. If you specify a scalable dimension, you must also specify a resource ID. <ul> <li>  <code>ecs:service:DesiredCount</code> - The desired task count of an ECS service. </li> <li>  <code>ec2:spot-fleet-request:TargetCapacity</code> - The target capacity of a Spot fleet request. </li> <li>  <code>elasticmapreduce:instancegroup:InstanceCount</code> - The instance count of an EMR Instance Group. </li> <li>  <code>appstream:fleet:DesiredCapacity</code> - The desired capacity of an AppStream 2.0 fleet. </li> <li>  <code>dynamodb:table:ReadCapacityUnits</code> - The provisioned read capacity for a DynamoDB table. </li> <li>  <code>dynamodb:table:WriteCapacityUnits</code> - The provisioned write capacity for a DynamoDB table. </li> <li>  <code>dynamodb:index:ReadCapacityUnits</code> - The provisioned read capacity for a DynamoDB global secondary index. </li> <li>  <code>dynamodb:index:WriteCapacityUnits</code> - The provisioned write capacity for a DynamoDB global secondary index. </li> <li>  <code>rds:cluster:ReadReplicaCount</code> - The count of Aurora Replicas in an Aurora DB cluster. Available for Aurora MySQL-compatible edition and Aurora PostgreSQL-compatible edition. </li> <li>  <code>sagemaker:variant:DesiredInstanceCount</code> - The number of EC2 instances for an Amazon SageMaker model endpoint variant. </li> <li>  <code>custom-resource:ResourceType:Property</code> - The scalable dimension for a custom resource provided by your own application or service. </li> </ul><br/>ResourceId:  The identifier of the resource associated with the scheduled action. This string consists of the resource type and unique identifier. If you specify a scalable dimension, you must also specify a resource ID. <ul> <li> ECS service - The resource type is <code>service</code> and the unique identifier is the cluster name and service name. Example: <code>service/default/sample-webapp</code>. </li> <li> Spot fleet request - The resource type is <code>spot-fleet-request</code> and the unique identifier is the Spot fleet request ID. Example: <code>spot-fleet-request/sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE</code>. </li> <li> EMR cluster - The resource type is <code>instancegroup</code> and the unique identifier is the cluster ID and instance group ID. Example: <code>instancegroup/j-2EEZNYKUA1NTV/ig-1791Y4E1L8YI0</code>. </li> <li> AppStream 2.0 fleet - The resource type is <code>fleet</code> and the unique identifier is the fleet name. Example: <code>fleet/sample-fleet</code>. </li> <li> DynamoDB table - The resource type is <code>table</code> and the unique identifier is the resource ID. Example: <code>table/my-table</code>. </li> <li> DynamoDB global secondary index - The resource type is <code>index</code> and the unique identifier is the resource ID. Example: <code>table/my-table/index/my-table-index</code>. </li> <li> Aurora DB cluster - The resource type is <code>cluster</code> and the unique identifier is the cluster name. Example: <code>cluster:my-db-cluster</code>. </li> <li> Amazon SageMaker endpoint variants - The resource type is <code>variant</code> and the unique identifier is the resource ID. Example: <code>endpoint/my-end-point/variant/KMeansClustering</code>. </li> <li> Custom resources are not supported with a resource type. This parameter must specify the <code>OutputValue</code> from the CloudFormation template stack used to access the resources. The unique identifier is defined by the service provider. More information is available in our <a href="https://github.com/aws/aws-auto-scaling-custom-resource">GitHub repository</a>. </li> </ul><br/>ServiceNamespace:  The namespace of the AWS service that provides the resource or <code>custom-resource</code> for a resource provided by your own application or service. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#genref-aws-service-namespaces">AWS Service Namespaces</a> in the <i>Amazon Web Services General Reference</i>.<br/>ScheduledActionNames:  The names of the scheduled actions to describe.<br/>
        :return: NextToken:  The token required to get the next set of results. This value is <code>null</code> if there are no more results to return.<br/>ScheduledActions:  Information about the scheduled actions.<br/>        """
        pass

    def put_scaling_policy(self, ScalableDimension: str, ResourceId: str, ServiceNamespace: str, PolicyName: str, PolicyType: str=None, StepScalingPolicyConfiguration: StepScalingPolicyConfiguration=None, TargetTrackingScalingPolicyConfiguration: TargetTrackingScalingPolicyConfiguration=None) -> PutScalingPolicyResponse:
        """Creates or updates a policy for an Application Auto Scaling scalable target. Each scalable target is identified by a service namespace, resource ID, and scalable dimension. A scaling policy applies to the scalable target identified by those three attributes. You cannot create a scaling policy until you have registered the resource as a scalable target using <a>RegisterScalableTarget</a>. To update a policy, specify its policy name and the parameters that you want to change. Any parameters that you don't specify are not changed by this update request. You can view the scaling policies for a service namespace using <a>DescribeScalingPolicies</a>. If you are no longer using a scaling policy, you can delete it using <a>DeleteScalingPolicy</a>. Multiple scaling policies can be in force at the same time for the same scalable target. You can have one or more target tracking scaling policies, one or more step scaling policies, or both. However, there is a chance that multiple policies could conflict, instructing the scalable target to scale out or in at the same time. Application Auto Scaling gives precedence to the policy that provides the largest capacity for both scale in and scale out. For example, if one policy increases capacity by 3, another policy increases capacity by 200 percent, and the current capacity is 10, Application Auto Scaling uses the policy with the highest calculated capacity (200% of 10 = 20) and scales out to 30.  Learn more about how to work with scaling policies in the <a href="https://docs.aws.amazon.com/autoscaling/application/userguide/what-is-application-auto-scaling.html">Application Auto Scaling User Guide</a>.

        :param self:
        :param TargetTrackingScalingPolicyConfiguration:  A target tracking scaling policy. Includes support for predefined or customized metrics. This parameter is required if you are creating a policy and the policy type is <code>TargetTrackingScaling</code>.:<br/>StepScalingPolicyConfiguration:  A step scaling policy. This parameter is required if you are creating a policy and the policy type is <code>StepScaling</code>.:<br/>PolicyType:  The policy type. This parameter is required if you are creating a scaling policy. For information on which services do not support <code>StepScaling</code> or <code>TargetTrackingScaling</code>, see the information about <b>Limits</b> in <a href="https://docs.aws.amazon.com/autoscaling/application/userguide/application-auto-scaling-step-scaling-policies.html">Step Scaling Policies</a> and <a href="https://docs.aws.amazon.com/autoscaling/application/userguide/application-auto-scaling-target-tracking.html">Target Tracking Scaling Policies</a> in the <i>Application Auto Scaling User Guide</i>.<br/>ScalableDimension:  The scalable dimension. This string consists of the service namespace, resource type, and scaling property. <ul> <li>  <code>ecs:service:DesiredCount</code> - The desired task count of an ECS service. </li> <li>  <code>ec2:spot-fleet-request:TargetCapacity</code> - The target capacity of a Spot fleet request. </li> <li>  <code>elasticmapreduce:instancegroup:InstanceCount</code> - The instance count of an EMR Instance Group. </li> <li>  <code>appstream:fleet:DesiredCapacity</code> - The desired capacity of an AppStream 2.0 fleet. </li> <li>  <code>dynamodb:table:ReadCapacityUnits</code> - The provisioned read capacity for a DynamoDB table. </li> <li>  <code>dynamodb:table:WriteCapacityUnits</code> - The provisioned write capacity for a DynamoDB table. </li> <li>  <code>dynamodb:index:ReadCapacityUnits</code> - The provisioned read capacity for a DynamoDB global secondary index. </li> <li>  <code>dynamodb:index:WriteCapacityUnits</code> - The provisioned write capacity for a DynamoDB global secondary index. </li> <li>  <code>rds:cluster:ReadReplicaCount</code> - The count of Aurora Replicas in an Aurora DB cluster. Available for Aurora MySQL-compatible edition and Aurora PostgreSQL-compatible edition. </li> <li>  <code>sagemaker:variant:DesiredInstanceCount</code> - The number of EC2 instances for an Amazon SageMaker model endpoint variant. </li> <li>  <code>custom-resource:ResourceType:Property</code> - The scalable dimension for a custom resource provided by your own application or service. </li> </ul><br/>ResourceId:  The identifier of the resource associated with the scaling policy. This string consists of the resource type and unique identifier. <ul> <li> ECS service - The resource type is <code>service</code> and the unique identifier is the cluster name and service name. Example: <code>service/default/sample-webapp</code>. </li> <li> Spot fleet request - The resource type is <code>spot-fleet-request</code> and the unique identifier is the Spot fleet request ID. Example: <code>spot-fleet-request/sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE</code>. </li> <li> EMR cluster - The resource type is <code>instancegroup</code> and the unique identifier is the cluster ID and instance group ID. Example: <code>instancegroup/j-2EEZNYKUA1NTV/ig-1791Y4E1L8YI0</code>. </li> <li> AppStream 2.0 fleet - The resource type is <code>fleet</code> and the unique identifier is the fleet name. Example: <code>fleet/sample-fleet</code>. </li> <li> DynamoDB table - The resource type is <code>table</code> and the unique identifier is the resource ID. Example: <code>table/my-table</code>. </li> <li> DynamoDB global secondary index - The resource type is <code>index</code> and the unique identifier is the resource ID. Example: <code>table/my-table/index/my-table-index</code>. </li> <li> Aurora DB cluster - The resource type is <code>cluster</code> and the unique identifier is the cluster name. Example: <code>cluster:my-db-cluster</code>. </li> <li> Amazon SageMaker endpoint variants - The resource type is <code>variant</code> and the unique identifier is the resource ID. Example: <code>endpoint/my-end-point/variant/KMeansClustering</code>. </li> <li> Custom resources are not supported with a resource type. This parameter must specify the <code>OutputValue</code> from the CloudFormation template stack used to access the resources. The unique identifier is defined by the service provider. More information is available in our <a href="https://github.com/aws/aws-auto-scaling-custom-resource">GitHub repository</a>. </li> </ul><br/>ServiceNamespace:  The namespace of the AWS service that provides the resource or <code>custom-resource</code> for a resource provided by your own application or service. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#genref-aws-service-namespaces">AWS Service Namespaces</a> in the <i>Amazon Web Services General Reference</i>.<br/>PolicyName:  The name of the scaling policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricAggregationType</i> The aggregation type for the CloudWatch metrics. Valid values are <code>Minimum</code>, <code>Maximum</code>, and <code>Average</code>. If the aggregation type is null, the value is treated as <code>Average</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Cooldown</i> The amount of time, in seconds, after a scaling activity completes where previous trigger-related scaling activities can influence future scaling events. For scale-out policies, while the cooldown period is in effect, the capacity that has been added by the previous scale-out event that initiated the cooldown is calculated as part of the desired capacity for the next scale out. The intention is to continuously (but not excessively) scale out. For example, an alarm triggers a step scaling policy to scale out an Amazon ECS service by 2 tasks, the scaling activity completes successfully, and a cooldown period of 5 minutes starts. During the cooldown period, if the alarm triggers the same policy again but at a more aggressive step adjustment to scale out the service by 3 tasks, the 2 tasks that were added in the previous scale-out event are considered part of that capacity and only 1 additional task is added to the desired count. For scale-in policies, the cooldown period is used to block subsequent scale-in requests until it has expired. The intention is to scale in conservatively to protect your application's availability. However, if another alarm triggers a scale-out policy during the cooldown period after a scale-in, Application Auto Scaling scales out your scalable target immediately.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MinAdjustmentMagnitude</i> The minimum number to adjust your scalable dimension as a result of a scaling activity. If the adjustment type is <code>PercentChangeInCapacity</code>, the scaling policy changes the scalable dimension of the scalable target by this amount. For example, suppose that you create a step scaling policy to scale out an Amazon ECS service by 25 percent and you specify a <code>MinAdjustmentMagnitude</code> of 2. If the service has 4 tasks and the scaling policy is performed, 25 percent of 4 is 1. However, because you specified a <code>MinAdjustmentMagnitude</code> of 2, Application Auto Scaling scales out the service by 2 tasks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StepAdjustments</i> A set of adjustments that enable you to scale based on the size of the alarm breach.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdjustmentType</i> The adjustment type, which specifies how the <code>ScalingAdjustment</code> parameter in a <a>StepAdjustment</a> is interpreted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisableScaleIn</i> Indicates whether scale in by the target tracking scaling policy is disabled. If the value is <code>true</code>, scale in is disabled and the target tracking scaling policy won't remove capacity from the scalable resource. Otherwise, scale in is enabled and the target tracking scaling policy can remove capacity from the scalable resource. The default value is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ScaleInCooldown</i> The amount of time, in seconds, after a scale-in activity completes before another scale in activity can start. The cooldown period is used to block subsequent scale-in requests until it has expired. The intention is to scale in conservatively to protect your application's availability. However, if another alarm triggers a scale-out policy during the cooldown period after a scale-in, Application Auto Scaling scales out your scalable target immediately.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ScaleOutCooldown</i> The amount of time, in seconds, after a scale-out activity completes before another scale-out activity can start. While the cooldown period is in effect, the capacity that has been added by the previous scale-out event that initiated the cooldown is calculated as part of the desired capacity for the next scale out. The intention is to continuously (but not excessively) scale out.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomizedMetricSpecification</i> A customized metric. You can specify either a predefined metric or a customized metric.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CustomizedMetricSpecification</b>: A customized metric. You can specify either a predefined metric or a customized metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PredefinedMetricSpecification</i> A predefined metric. You can specify either a predefined metric or a customized metric.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PredefinedMetricSpecification</b>: A predefined metric. You can specify either a predefined metric or a customized metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetValue</i> The target value for the metric. The range is 8.515920e-109 to 1.174271e+108 (Base 10) or 2e-360 to 2e360 (Base 2).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceLabel</i> Identifies the resource associated with the metric type. You can't specify a resource label unless the metric type is <code>ALBRequestCountPerTarget</code> and there is a target group attached to the Spot fleet request or ECS service. The format is app/&lt;load-balancer-name&gt;/&lt;load-balancer-id&gt;/targetgroup/&lt;target-group-name&gt;/&lt;target-group-id&gt;, where: <ul> <li> app/&lt;load-balancer-name&gt;/&lt;load-balancer-id&gt; is the final portion of the load balancer ARN </li> <li> targetgroup/&lt;target-group-name&gt;/&lt;target-group-id&gt; is the final portion of the target group ARN. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PredefinedMetricType</i> The metric type. The <code>ALBRequestCountPerTarget</code> metric type applies only to Spot fleet requests and ECS services.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Unit</i> The unit of the metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Statistic</i> The statistic of the metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Dimensions</i> The dimensions of the metric.  Conditional: If you published your metric with dimensions, you must specify the same dimensions in your scaling policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Namespace</i> The namespace of the metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricName</i> The name of the metric. <br/>
        :return: Alarms:  The CloudWatch alarms created for the target tracking scaling policy.<br/>PolicyARN:  The Amazon Resource Name (ARN) of the resulting scaling policy.<br/>        """
        pass

    def put_scheduled_action(self, ScalableDimension: str, ResourceId: str, ScheduledActionName: str, ServiceNamespace: str, Schedule: str=None, StartTime: TimestampType=None, EndTime: TimestampType=None, ScalableTargetAction: ScalableTargetAction=None) -> PutScheduledActionResponse:
        """Creates or updates a scheduled action for an Application Auto Scaling scalable target. Each scalable target is identified by a service namespace, resource ID, and scalable dimension. A scheduled action applies to the scalable target identified by those three attributes. You cannot create a scheduled action until you have registered the resource as a scalable target using <a>RegisterScalableTarget</a>.  To update an action, specify its name and the parameters that you want to change. If you don't specify start and end times, the old values are deleted. Any other parameters that you don't specify are not changed by this update request. You can view the scheduled actions using <a>DescribeScheduledActions</a>. If you are no longer using a scheduled action, you can delete it using <a>DeleteScheduledAction</a>. Learn more about how to work with scheduled actions in the <a href="https://docs.aws.amazon.com/autoscaling/application/userguide/what-is-application-auto-scaling.html">Application Auto Scaling User Guide</a>.

        :param self:
        :param ScalableTargetAction:  The new minimum and maximum capacity. You can set both values or just one. During the scheduled time, if the current capacity is below the minimum capacity, Application Auto Scaling scales out to the minimum capacity. If the current capacity is above the maximum capacity, Application Auto Scaling scales in to the maximum capacity.:<br/>EndTime:  The date and time for the scheduled action to end.<br/>StartTime:  The date and time for the scheduled action to start.<br/>ScalableDimension:  The scalable dimension. This string consists of the service namespace, resource type, and scaling property. <ul> <li>  <code>ecs:service:DesiredCount</code> - The desired task count of an ECS service. </li> <li>  <code>ec2:spot-fleet-request:TargetCapacity</code> - The target capacity of a Spot fleet request. </li> <li>  <code>elasticmapreduce:instancegroup:InstanceCount</code> - The instance count of an EMR Instance Group. </li> <li>  <code>appstream:fleet:DesiredCapacity</code> - The desired capacity of an AppStream 2.0 fleet. </li> <li>  <code>dynamodb:table:ReadCapacityUnits</code> - The provisioned read capacity for a DynamoDB table. </li> <li>  <code>dynamodb:table:WriteCapacityUnits</code> - The provisioned write capacity for a DynamoDB table. </li> <li>  <code>dynamodb:index:ReadCapacityUnits</code> - The provisioned read capacity for a DynamoDB global secondary index. </li> <li>  <code>dynamodb:index:WriteCapacityUnits</code> - The provisioned write capacity for a DynamoDB global secondary index. </li> <li>  <code>rds:cluster:ReadReplicaCount</code> - The count of Aurora Replicas in an Aurora DB cluster. Available for Aurora MySQL-compatible edition and Aurora PostgreSQL-compatible edition. </li> <li>  <code>sagemaker:variant:DesiredInstanceCount</code> - The number of EC2 instances for an Amazon SageMaker model endpoint variant. </li> <li>  <code>custom-resource:ResourceType:Property</code> - The scalable dimension for a custom resource provided by your own application or service. </li> </ul><br/>ResourceId:  The identifier of the resource associated with the scheduled action. This string consists of the resource type and unique identifier. <ul> <li> ECS service - The resource type is <code>service</code> and the unique identifier is the cluster name and service name. Example: <code>service/default/sample-webapp</code>. </li> <li> Spot fleet request - The resource type is <code>spot-fleet-request</code> and the unique identifier is the Spot fleet request ID. Example: <code>spot-fleet-request/sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE</code>. </li> <li> EMR cluster - The resource type is <code>instancegroup</code> and the unique identifier is the cluster ID and instance group ID. Example: <code>instancegroup/j-2EEZNYKUA1NTV/ig-1791Y4E1L8YI0</code>. </li> <li> AppStream 2.0 fleet - The resource type is <code>fleet</code> and the unique identifier is the fleet name. Example: <code>fleet/sample-fleet</code>. </li> <li> DynamoDB table - The resource type is <code>table</code> and the unique identifier is the resource ID. Example: <code>table/my-table</code>. </li> <li> DynamoDB global secondary index - The resource type is <code>index</code> and the unique identifier is the resource ID. Example: <code>table/my-table/index/my-table-index</code>. </li> <li> Aurora DB cluster - The resource type is <code>cluster</code> and the unique identifier is the cluster name. Example: <code>cluster:my-db-cluster</code>. </li> <li> Amazon SageMaker endpoint variants - The resource type is <code>variant</code> and the unique identifier is the resource ID. Example: <code>endpoint/my-end-point/variant/KMeansClustering</code>. </li> <li> Custom resources are not supported with a resource type. This parameter must specify the <code>OutputValue</code> from the CloudFormation template stack used to access the resources. The unique identifier is defined by the service provider. More information is available in our <a href="https://github.com/aws/aws-auto-scaling-custom-resource">GitHub repository</a>. </li> </ul><br/>ScheduledActionName:  The name of the scheduled action.<br/>Schedule:  The schedule for this action. The following formats are supported: <ul> <li> At expressions - "<code>at(<i>yyyy</i>-<i>mm</i>-<i>dd</i>T<i>hh</i>:<i>mm</i>:<i>ss</i>)</code>" </li> <li> Rate expressions - "<code>rate(<i>value</i> <i>unit</i>)</code>" </li> <li> Cron expressions - "<code>cron(<i>fields</i>)</code>" </li> </ul> At expressions are useful for one-time schedules. Specify the time, in UTC. For rate expressions, <i>value</i> is a positive integer and <i>unit</i> is <code>minute</code> | <code>minutes</code> | <code>hour</code> | <code>hours</code> | <code>day</code> | <code>days</code>. For more information about cron expressions, see <a href="https://docs.aws.amazon.com/AmazonCloudWatch/latest/events/ScheduledEvents.html#CronExpressions">Cron Expressions</a> in the <i>Amazon CloudWatch Events User Guide</i>.<br/>ServiceNamespace:  The namespace of the AWS service that provides the resource or <code>custom-resource</code> for a resource provided by your own application or service. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#genref-aws-service-namespaces">AWS Service Namespaces</a> in the <i>Amazon Web Services General Reference</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxCapacity</i> The maximum capacity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MinCapacity</i> The minimum capacity.<br/>
        :return:         """
        pass

    def register_scalable_target(self, ScalableDimension: str, ResourceId: str, ServiceNamespace: str, MinCapacity: int=None, MaxCapacity: int=None, RoleARN: str=None) -> RegisterScalableTargetResponse:
        """Registers or updates a scalable target. A scalable target is a resource that Application Auto Scaling can scale out and scale in. Each scalable target has a resource ID, scalable dimension, and namespace, as well as values for minimum and maximum capacity.  After you register a scalable target, you do not need to register it again to use other Application Auto Scaling operations. To see which resources have been registered, use <a>DescribeScalableTargets</a>. You can also view the scaling policies for a service namespace using <a>DescribeScalableTargets</a>.  If you no longer need a scalable target, you can deregister it using <a>DeregisterScalableTarget</a>.

        :param self:
        :param RoleARN:  Application Auto Scaling creates a service-linked role that grants it permissions to modify the scalable target on your behalf. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/application/userguide/application-auto-scaling-service-linked-roles.html">Service-Linked Roles for Application Auto Scaling</a>. For resources that are not supported using a service-linked role, this parameter is required and must specify the ARN of an IAM role that allows Application Auto Scaling to modify the scalable target on your behalf.<br/>MaxCapacity:  The maximum value to scale to in response to a scale-out event. This parameter is required to register a scalable target.<br/>MinCapacity:  The minimum value to scale to in response to a scale-in event. This parameter is required to register a scalable target.<br/>ScalableDimension:  The scalable dimension associated with the scalable target. This string consists of the service namespace, resource type, and scaling property. <ul> <li>  <code>ecs:service:DesiredCount</code> - The desired task count of an ECS service. </li> <li>  <code>ec2:spot-fleet-request:TargetCapacity</code> - The target capacity of a Spot fleet request. </li> <li>  <code>elasticmapreduce:instancegroup:InstanceCount</code> - The instance count of an EMR Instance Group. </li> <li>  <code>appstream:fleet:DesiredCapacity</code> - The desired capacity of an AppStream 2.0 fleet. </li> <li>  <code>dynamodb:table:ReadCapacityUnits</code> - The provisioned read capacity for a DynamoDB table. </li> <li>  <code>dynamodb:table:WriteCapacityUnits</code> - The provisioned write capacity for a DynamoDB table. </li> <li>  <code>dynamodb:index:ReadCapacityUnits</code> - The provisioned read capacity for a DynamoDB global secondary index. </li> <li>  <code>dynamodb:index:WriteCapacityUnits</code> - The provisioned write capacity for a DynamoDB global secondary index. </li> <li>  <code>rds:cluster:ReadReplicaCount</code> - The count of Aurora Replicas in an Aurora DB cluster. Available for Aurora MySQL-compatible edition and Aurora PostgreSQL-compatible edition. </li> <li>  <code>sagemaker:variant:DesiredInstanceCount</code> - The number of EC2 instances for an Amazon SageMaker model endpoint variant. </li> <li>  <code>custom-resource:ResourceType:Property</code> - The scalable dimension for a custom resource provided by your own application or service. </li> </ul><br/>ResourceId:  The identifier of the resource associated with the scalable target. This string consists of the resource type and unique identifier. <ul> <li> ECS service - The resource type is <code>service</code> and the unique identifier is the cluster name and service name. Example: <code>service/default/sample-webapp</code>. </li> <li> Spot fleet request - The resource type is <code>spot-fleet-request</code> and the unique identifier is the Spot fleet request ID. Example: <code>spot-fleet-request/sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE</code>. </li> <li> EMR cluster - The resource type is <code>instancegroup</code> and the unique identifier is the cluster ID and instance group ID. Example: <code>instancegroup/j-2EEZNYKUA1NTV/ig-1791Y4E1L8YI0</code>. </li> <li> AppStream 2.0 fleet - The resource type is <code>fleet</code> and the unique identifier is the fleet name. Example: <code>fleet/sample-fleet</code>. </li> <li> DynamoDB table - The resource type is <code>table</code> and the unique identifier is the resource ID. Example: <code>table/my-table</code>. </li> <li> DynamoDB global secondary index - The resource type is <code>index</code> and the unique identifier is the resource ID. Example: <code>table/my-table/index/my-table-index</code>. </li> <li> Aurora DB cluster - The resource type is <code>cluster</code> and the unique identifier is the cluster name. Example: <code>cluster:my-db-cluster</code>. </li> <li> Amazon SageMaker endpoint variants - The resource type is <code>variant</code> and the unique identifier is the resource ID. Example: <code>endpoint/my-end-point/variant/KMeansClustering</code>. </li> <li> Custom resources are not supported with a resource type. This parameter must specify the <code>OutputValue</code> from the CloudFormation template stack used to access the resources. The unique identifier is defined by the service provider. More information is available in our <a href="https://github.com/aws/aws-auto-scaling-custom-resource">GitHub repository</a>. </li> </ul><br/>ServiceNamespace:  The namespace of the AWS service that provides the resource or <code>custom-resource</code> for a resource provided by your own application or service. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#genref-aws-service-namespaces">AWS Service Namespaces</a> in the <i>Amazon Web Services General Reference</i>.<br/>
        :return:         """
        pass



class ApplicationInsights(BaseClient):
    """<fullname>Amazon CloudWatch Application Insights for .NET and SQL Server</fullname>  Amazon CloudWatch Application Insights for .NET and SQL Server is a service that helps you detect common problems with your .NET and SQL Server-based applications. It enables you to pinpoint the source of issues in your applications (built with technologies such as Microsoft IIS, .NET, and Microsoft SQL Server), by providing key insights into detected problems. After you onboard your application, CloudWatch Application Insights for .NET and SQL Server identifies, recommends, and sets up metrics and logs. It continuously analyzes and correlates your metrics and logs for unusual behavior to surface actionable problems with your application. For example, if your application is slow and unresponsive and leading to HTTP 500 errors in your Application Load Balancer (ALB), Application Insights informs you that a memory pressure problem with your SQL Server database is occurring. It bases this analysis on impactful metrics and log errors. """




    class CreateApplicationRequest(Mapping):
        pass
    
    class CreateApplicationResponse(Mapping):
        pass
    
    class CreateComponentRequest(Mapping):
        pass
    
    class CreateComponentResponse(Mapping):
        pass
    
    class ResourceARN(object):
        pass
    
    class DeleteApplicationRequest(Mapping):
        pass
    
    class DeleteApplicationResponse(Mapping):
        pass
    
    class DeleteComponentRequest(Mapping):
        pass
    
    class DeleteComponentResponse(Mapping):
        pass
    
    class DescribeApplicationRequest(Mapping):
        pass
    
    class DescribeApplicationResponse(Mapping):
        pass
    
    class DescribeComponentRequest(Mapping):
        pass
    
    class DescribeComponentResponse(Mapping):
        pass
    
    class DescribeComponentConfigurationRequest(Mapping):
        pass
    
    class DescribeComponentConfigurationResponse(Mapping):
        pass
    
    class DescribeComponentConfigurationRecommendationRequest(Mapping):
        pass
    
    class DescribeComponentConfigurationRecommendationResponse(Mapping):
        pass
    
    class DescribeObservationRequest(Mapping):
        pass
    
    class DescribeObservationResponse(Mapping):
        pass
    
    class DescribeProblemRequest(Mapping):
        pass
    
    class DescribeProblemResponse(Mapping):
        pass
    
    class DescribeProblemObservationsRequest(Mapping):
        pass
    
    class DescribeProblemObservationsResponse(Mapping):
        pass
    
    class ListApplicationsRequest(Mapping):
        pass
    
    class ListApplicationsResponse(Mapping):
        pass
    
    class ListComponentsRequest(Mapping):
        pass
    
    class ListComponentsResponse(Mapping):
        pass
    
    class ListProblemsRequest(Mapping):
        pass
    
    class ListProblemsResponse(Mapping):
        pass
    
    class StartTime(object):
        pass
    
    class EndTime(object):
        pass
    
    class UpdateComponentRequest(Mapping):
        pass
    
    class UpdateComponentResponse(Mapping):
        pass
    
    class UpdateComponentConfigurationRequest(Mapping):
        pass
    
    class UpdateComponentConfigurationResponse(Mapping):
        pass
    
    class Monitor(object):
        pass
    
    def create_application(self, ResourceGroupName: str) -> CreateApplicationResponse:
        """Adds an application that is created from a resource group.

        :param self:
        :param ResourceGroupName:  The name of the resource group.<br/>
        :return: ApplicationInfo:  Information about the application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Remarks</i> The issues on the user side that are blocking Application Insights from fully monitoring the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LifeCycle</i> The lifecycle of the application. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceGroupName</i> The name of the resource group used for the application.<br/>        """
        pass

    def create_component(self, ResourceList: List[ResourceARN], ComponentName: str, ResourceGroupName: str) -> CreateComponentResponse:
        """Creates a custom component by grouping similar standalone instances to monitor.

        :param self:
        :param ResourceList:  The list of resource ARNs that belong to the component.<br/>ComponentName:  The name of the component.<br/>ResourceGroupName:  The name of the resource group.<br/>
        :return:         """
        pass

    def delete_application(self, ResourceGroupName: str) -> DeleteApplicationResponse:
        """Removes the specified application from monitoring. Does not delete the application.

        :param self:
        :param ResourceGroupName:  The name of the resource group.<br/>
        :return:         """
        pass

    def delete_component(self, ComponentName: str, ResourceGroupName: str) -> DeleteComponentResponse:
        """Ungroups a custom component. When you ungroup custom components, all applicable monitors that are set up for the component are removed and the instances revert to their standalone status.

        :param self:
        :param ComponentName:  The name of the component.<br/>ResourceGroupName:  The name of the resource group.<br/>
        :return:         """
        pass

    def describe_application(self, ResourceGroupName: str) -> DescribeApplicationResponse:
        """Describes the application.

        :param self:
        :param ResourceGroupName:  The name of the resource group.<br/>
        :return: ApplicationInfo:  Information about the application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Remarks</i> The issues on the user side that are blocking Application Insights from fully monitoring the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LifeCycle</i> The lifecycle of the application. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceGroupName</i> The name of the resource group used for the application.<br/>        """
        pass

    def describe_component(self, ComponentName: str, ResourceGroupName: str) -> DescribeComponentResponse:
        """Describes a component and lists the resources that are grouped together in a component.

        :param self:
        :param ComponentName:  The name of the component.<br/>ResourceGroupName:  The name of the resource group.<br/>
        :return: ResourceList:  The list of resource ARNs that belong to the component.<br/>ApplicationComponent:  Describes a standalone resource or similarly grouped resources that the application is made up of.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Monitor</i> Indicates whether the application component is monitored. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tier</i> The stack tier of the application component.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceType</i> The resource type. Supported resource types include EC2 instances, Auto Scaling group, Classic ELB, Application ELB, and SQS Queue.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ComponentName</i> The name of the component.<br/>        """
        pass

    def describe_component_configuration(self, ComponentName: str, ResourceGroupName: str) -> DescribeComponentConfigurationResponse:
        """Describes the monitoring configuration of the component.

        :param self:
        :param ComponentName:  The name of the component.<br/>ResourceGroupName:  The name of the resource group.<br/>
        :return: ComponentConfiguration:  The configuration settings of the component. The value is the escaped JSON of the configuration.<br/>Tier:  The tier of the application component. Supported tiers include <code>DOT_NET_WORKER</code>, <code>DOT_NET_WEB_TIER</code>, <code>SQL_SERVER</code>, and <code>DEFAULT</code> <br/>Monitor:  Indicates whether the application component is monitored.<br/>        """
        pass

    def describe_component_configuration_recommendation(self, Tier: str, ComponentName: str, ResourceGroupName: str) -> DescribeComponentConfigurationRecommendationResponse:
        """Describes the recommended monitoring configuration of the component.

        :param self:
        :param Tier:  The tier of the application component. Supported tiers include <code>DOT_NET_WORKER</code>, <code>DOT_NET_WEB_TIER</code>, <code>SQL_SERVER</code>, and <code>DEFAULT</code>.<br/>ComponentName:  The name of the component.<br/>ResourceGroupName:  The name of the resource group.<br/>
        :return: ComponentConfiguration:  The recommended configuration settings of the component. The value is the escaped JSON of the configuration.<br/>        """
        pass

    def describe_observation(self, ObservationId: str) -> DescribeObservationResponse:
        """Describes an anomaly or error with the application.

        :param self:
        :param ObservationId:  The ID of the observation.<br/>
        :return: Observation:  Information about the observation.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The value of the source observation metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Unit</i> The unit of the source observation metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricName</i> The name of the observation metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricNamespace</i> The namespace of the observation metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogFilter</i> The log filter of the observation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogText</i> The log text of the observation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LineTime</i> The timestamp in the CloudWatch Logs that specifies when the matched line occurred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogGroup</i> The log group name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceARN</i> The source resource ARN of the observation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceType</i> The source type of the observation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> The time when the observation ended, in epoch seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTime</i> The time when the observation was first detected, in epoch seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the observation type.<br/>        """
        pass

    def describe_problem(self, ProblemId: str) -> DescribeProblemResponse:
        """Describes an application problem.

        :param self:
        :param ProblemId:  The ID of the problem.<br/>
        :return: Problem:  Information about the problem. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Feedback</i> Feedback provided by the user about the problem.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceGroupName</i> The name of the resource group affected by the problem.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SeverityLevel</i> A measure of the level of impact of the problem.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> The time when the problem ended, in epoch seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTime</i> The time when the problem started, in epoch seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AffectedResource</i> The resource affected by the problem.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the problem.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Insights</i> A detailed analysis of the problem using machine learning.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Title</i> The name of the problem.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the problem.<br/>        """
        pass

    def describe_problem_observations(self, ProblemId: str) -> DescribeProblemObservationsResponse:
        """Describes the anomalies or errors associated with the problem.

        :param self:
        :param ProblemId:  The ID of the problem.<br/>
        :return: RelatedObservations:  Observations related to the problem.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ObservationList</i> The list of observations related to the problem.<br/>        """
        pass

    def list_applications(self, MaxResults: int=None, NextToken: str=None) -> ListApplicationsResponse:
        """Lists the IDs of the applications that you are monitoring. 

        :param self:
        :param NextToken:  The token to request the next page of results.<br/>MaxResults:  The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value.<br/>
        :return: NextToken:  The token used to retrieve the next page of results. This value is <code>null</code> when there are no more results to return. <br/>ApplicationInfoList:  The list of applications.<br/>        """
        pass

    def list_components(self, ResourceGroupName: str, MaxResults: int=None, NextToken: str=None) -> ListComponentsResponse:
        """Lists the auto-grouped, standalone, and custom components of the application.

        :param self:
        :param NextToken:  The token to request the next page of results.<br/>MaxResults:  The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value.<br/>ResourceGroupName:  The name of the resource group.<br/>
        :return: NextToken:  The token to request the next page of results.<br/>ApplicationComponentList:  The list of application components.<br/>        """
        pass

    def list_problems(self, ResourceGroupName: str=None, StartTime: StartTime=None, EndTime: EndTime=None, MaxResults: int=None, NextToken: str=None) -> ListProblemsResponse:
        """Lists the problems with your application.

        :param self:
        :param NextToken:  The token to request the next page of results.<br/>MaxResults:  The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value.<br/>EndTime:  The time when the problem ended, in epoch seconds. If not specified, problems within the past seven days are returned.<br/>StartTime:  The time when the problem was detected, in epoch seconds. If you don't specify a time frame for the request, problems within the past seven days are returned.<br/>ResourceGroupName:  The name of the resource group.<br/>
        :return: NextToken:  The token used to retrieve the next page of results. This value is <code>null</code> when there are no more results to return. <br/>ProblemList:  The list of problems. <br/>        """
        pass

    def update_component(self, ComponentName: str, ResourceGroupName: str, NewComponentName: str=None, ResourceList: List[ResourceARN]=None) -> UpdateComponentResponse:
        """Updates the custom component name and/or the list of resources that make up the component.

        :param self:
        :param ResourceList:  The list of resource ARNs that belong to the component.<br/>NewComponentName:  The new name of the component.<br/>ComponentName:  The name of the component.<br/>ResourceGroupName:  The name of the resource group.<br/>
        :return:         """
        pass

    def update_component_configuration(self, ComponentName: str, ResourceGroupName: str, Monitor: Monitor=None, Tier: str=None, ComponentConfiguration: str=None) -> UpdateComponentConfigurationResponse:
        """Updates the monitoring configurations for the component. The configuration input parameter is an escaped JSON of the configuration and should match the schema of what is returned by <code>DescribeComponentConfigurationRecommendation</code>.

        :param self:
        :param ComponentConfiguration:  The configuration settings of the component. The value is the escaped JSON of the configuration. For more information about the JSON format, see <a href="https://docs.aws.amazon.com/sdk-for-javascript/v2/developer-guide/working-with-json.html">Working with JSON</a>. You can send a request to <code>DescribeComponentConfigurationRecommendation</code> to see the recommended configuration for a component.<br/>Tier:  The tier of the application component. Supported tiers include <code>DOT_NET_WORKER</code>, <code>DOT_NET_WEB_TIER</code>, <code>SQL_SERVER</code>, and <code>DEFAULT</code>.<br/>Monitor:  Indicates whether the application component is monitored.<br/>ComponentName:  The name of the component.<br/>ResourceGroupName:  The name of the resource group.<br/>
        :return:         """
        pass



class AppMesh(BaseClient):
    """AWS App Mesh is a service mesh based on the Envoy proxy that makes it easy to monitor and
         control microservices. App Mesh standardizes how your microservices communicate, giving you
         end-to-end visibility and helping to ensure high availability for your applications.
         App Mesh gives you consistent visibility and network traffic controls for every
         microservice in an application. You can use App Mesh with AWS Fargate, Amazon ECS, Amazon EKS,
         Kubernetes on AWS, and Amazon EC2.
         <note>
            App Mesh supports microservice applications that use service discovery naming for their
            components. For more information about service discovery on Amazon ECS, see <a href="http://docs.aws.amazon.com/AmazonECS/latest/developerguide/service-discovery.html">Service Discovery</a> in the
               <i>Amazon Elastic Container Service Developer Guide</i>. Kubernetes <code>kube-dns</code> and
               <code>coredns</code> are supported. For more information, see <a href="https://kubernetes.io/docs/concepts/services-networking/dns-pod-service/">DNS
               for Services and Pods</a> in the Kubernetes documentation.
         </note>"""


    class ListMeshesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_meshes` for valid parameters"""
            pass
    class ListRoutesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_routes` for valid parameters"""
            pass
    class ListVirtualNodesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_virtual_nodes` for valid parameters"""
            pass
    class ListVirtualRoutersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_virtual_routers` for valid parameters"""
            pass
    class ListVirtualServicesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_virtual_services` for valid parameters"""
            pass
    class ListTagsForResourcePaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_tags_for_resource` for valid parameters"""
            pass


    class CreateMeshInput(Mapping):
        pass
    
    class CreateMeshOutput(Mapping):
        pass
    
    class MeshSpec(Mapping):
        pass
    
    class TagRef(Mapping):
        pass
    
    class CreateRouteInput(Mapping):
        pass
    
    class CreateRouteOutput(Mapping):
        pass
    
    class RouteSpec(Mapping):
        pass
    
    class CreateVirtualNodeInput(Mapping):
        pass
    
    class CreateVirtualNodeOutput(Mapping):
        pass
    
    class VirtualNodeSpec(Mapping):
        pass
    
    class CreateVirtualRouterInput(Mapping):
        pass
    
    class CreateVirtualRouterOutput(Mapping):
        pass
    
    class VirtualRouterSpec(Mapping):
        pass
    
    class CreateVirtualServiceInput(Mapping):
        pass
    
    class CreateVirtualServiceOutput(Mapping):
        pass
    
    class VirtualServiceSpec(Mapping):
        pass
    
    class DeleteMeshInput(Mapping):
        pass
    
    class DeleteMeshOutput(Mapping):
        pass
    
    class DeleteRouteInput(Mapping):
        pass
    
    class DeleteRouteOutput(Mapping):
        pass
    
    class DeleteVirtualNodeInput(Mapping):
        pass
    
    class DeleteVirtualNodeOutput(Mapping):
        pass
    
    class DeleteVirtualRouterInput(Mapping):
        pass
    
    class DeleteVirtualRouterOutput(Mapping):
        pass
    
    class DeleteVirtualServiceInput(Mapping):
        pass
    
    class DeleteVirtualServiceOutput(Mapping):
        pass
    
    class DescribeMeshInput(Mapping):
        pass
    
    class DescribeMeshOutput(Mapping):
        pass
    
    class DescribeRouteInput(Mapping):
        pass
    
    class DescribeRouteOutput(Mapping):
        pass
    
    class DescribeVirtualNodeInput(Mapping):
        pass
    
    class DescribeVirtualNodeOutput(Mapping):
        pass
    
    class DescribeVirtualRouterInput(Mapping):
        pass
    
    class DescribeVirtualRouterOutput(Mapping):
        pass
    
    class DescribeVirtualServiceInput(Mapping):
        pass
    
    class DescribeVirtualServiceOutput(Mapping):
        pass
    
    class ListMeshesInput(Mapping):
        pass
    
    class ListMeshesOutput(Mapping):
        pass
    
    class ListRoutesInput(Mapping):
        pass
    
    class ListRoutesOutput(Mapping):
        pass
    
    class ListTagsForResourceInput(Mapping):
        pass
    
    class ListTagsForResourceOutput(Mapping):
        pass
    
    class ListVirtualNodesInput(Mapping):
        pass
    
    class ListVirtualNodesOutput(Mapping):
        pass
    
    class ListVirtualRoutersInput(Mapping):
        pass
    
    class ListVirtualRoutersOutput(Mapping):
        pass
    
    class ListVirtualServicesInput(Mapping):
        pass
    
    class ListVirtualServicesOutput(Mapping):
        pass
    
    class TagResourceInput(Mapping):
        pass
    
    class TagResourceOutput(Mapping):
        pass
    
    class UntagResourceInput(Mapping):
        pass
    
    class UntagResourceOutput(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateMeshInput(Mapping):
        pass
    
    class UpdateMeshOutput(Mapping):
        pass
    
    class UpdateRouteInput(Mapping):
        pass
    
    class UpdateRouteOutput(Mapping):
        pass
    
    class UpdateVirtualNodeInput(Mapping):
        pass
    
    class UpdateVirtualNodeOutput(Mapping):
        pass
    
    class UpdateVirtualRouterInput(Mapping):
        pass
    
    class UpdateVirtualRouterOutput(Mapping):
        pass
    
    class UpdateVirtualServiceInput(Mapping):
        pass
    
    class UpdateVirtualServiceOutput(Mapping):
        pass
    
    def create_mesh(self, meshName: str, clientToken: str=None, spec: MeshSpec=None, tags: List[TagRef]=None) -> CreateMeshOutput:
        """Creates a service mesh. A service mesh is a logical boundary for network traffic between
         the services that reside within it.
         After you create your service mesh, you can create virtual services, virtual nodes,
         virtual routers, and routes to distribute traffic between the applications in your
         mesh.

        :param self:
        :param tags:  Optional metadata that you can apply to the service mesh to assist with categorization
         and organization. Each tag consists of a key and an optional value, both of which you
         define. Tag keys can have a maximum character length of 128 characters, and tag values can have
            a maximum length of 256 characters.<br/>spec:  The service mesh specification to apply.:<br/>meshName:  The name to use for the service mesh.<br/>clientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the
request. Up to 36 letters, numbers, hyphens, and underscores are allowed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>egressFilter</i> The egress filter rules for the service mesh.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>egressFilter</b>: The egress filter rules for the service mesh.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The egress filter type. By default, the type is <code>DROP_ALL</code>, which allows
         egress only from virtual nodes to other defined resources in the service mesh (and any
         traffic to <code>*.amazonaws.com</code> for AWS API calls). You can set the egress filter
         type to <code>ALLOW_ALL</code> to allow egress to any endpoint inside or outside of the
         service mesh.<br/>
        :return: mesh:  The full description of your service mesh following the create call.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the service mesh.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>status</b>: The status of the service mesh.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>spec</i> The associated specification for the service mesh.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>spec</b>: The associated specification for the service mesh.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>metadata</i> The associated metadata for the service mesh.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>metadata</b>: The associated metadata for the service mesh.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>meshName</i> The name of the service mesh.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the resource. Resources are created at version 1, and this version is
         incremented each time that they're updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uid</i> The unique identifier for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedAt</i> The Unix epoch timestamp in seconds for when the resource was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix epoch timestamp in seconds for when the resource was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The full Amazon Resource Name (ARN) for the resource.<br/>The egress filter rules for the service mesh.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>egressFilter</b>: The egress filter rules for the service mesh.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current mesh status.<br/>        """
        pass

    def create_route(self, virtualRouterName: str, spec: RouteSpec, routeName: str, meshName: str, clientToken: str=None, tags: List[TagRef]=None) -> CreateRouteOutput:
        """Creates a route that is associated with a virtual router.
         You can use the <code>prefix</code> parameter in your route specification for path-based
         routing of requests. For example, if your virtual service name is
            <code>my-service.local</code> and you want the route to match requests to
            <code>my-service.local/metrics</code>, your prefix should be
         <code>/metrics</code>.
         If your route matches a request, you can distribute traffic to one or more target
         virtual nodes with relative weighting.

        :param self:
        :param virtualRouterName:  The name of the virtual router in which to create the route.<br/>tags:  Optional metadata that you can apply to the route to assist with categorization and
         organization. Each tag consists of a key and an optional value, both of which you define.
         Tag keys can have a maximum character length of 128 characters, and tag values can have
            a maximum length of 256 characters.<br/>spec:  The route specification to apply.:<br/>routeName:  The name to use for the route.<br/>meshName:  The name of the service mesh to create the route in.<br/>clientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the
request. Up to 36 letters, numbers, hyphens, and underscores are allowed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tcpRoute</i> The TCP routing information for the route.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>tcpRoute</b>: The TCP routing information for the route.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>httpRoute</i> The HTTP routing information for the route.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>httpRoute</b>: The HTTP routing information for the route.<br/>The action to take if a match is determined.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>action</b>: The action to take if a match is determined.The action to take if a match is determined.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>action</b>: The action to take if a match is determined.
        :return: route:  The full description of your mesh following the create call.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualRouterName</i> The virtual router that the route is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the route.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>status</b>: The status of the route.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>spec</i> The specifications of the route.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>spec</b>: The specifications of the route.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>routeName</i> The name of the route.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>metadata</i> The associated metadata for the route.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>metadata</b>: The associated metadata for the route.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>meshName</i> The name of the service mesh that the route resides in.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the resource. Resources are created at version 1, and this version is
         incremented each time that they're updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uid</i> The unique identifier for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedAt</i> The Unix epoch timestamp in seconds for when the resource was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix epoch timestamp in seconds for when the resource was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The full Amazon Resource Name (ARN) for the resource.<br/>The HTTP routing information for the route.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>httpRoute</b>: The HTTP routing information for the route.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status for the route.<br/>        """
        pass

    def create_virtual_node(self, virtualNodeName: str, spec: VirtualNodeSpec, meshName: str, clientToken: str=None, tags: List[TagRef]=None) -> CreateVirtualNodeOutput:
        """Creates a virtual node within a service mesh.
         A virtual node acts as a logical pointer to a particular task group, such as an Amazon ECS
         service or a Kubernetes deployment. When you create a virtual node, you can specify the
         service discovery information for your task group.
         Any inbound traffic that your virtual node expects should be specified as a
            <code>listener</code>. Any outbound traffic that your virtual node expects to reach
         should be specified as a <code>backend</code>.
         The response metadata for your new virtual node contains the <code>arn</code> that is
         associated with the virtual node. Set this value (either the full ARN or the truncated
         resource name: for example, <code>mesh/default/virtualNode/simpleapp</code>) as the
            <code>APPMESH_VIRTUAL_NODE_NAME</code> environment variable for your task group's Envoy
         proxy container in your task definition or pod spec. This is then mapped to the
            <code>node.id</code> and <code>node.cluster</code> Envoy parameters.
         <note>
            If you require your Envoy stats or tracing to use a different name, you can override
            the <code>node.cluster</code> value that is set by
               <code>APPMESH_VIRTUAL_NODE_NAME</code> with the
               <code>APPMESH_VIRTUAL_NODE_CLUSTER</code> environment variable.
         </note>

        :param self:
        :param virtualNodeName:  The name to use for the virtual node.<br/>tags:  Optional metadata that you can apply to the virtual node to assist with categorization
         and organization. Each tag consists of a key and an optional value, both of which you
         define. Tag keys can have a maximum character length of 128 characters, and tag values can have
            a maximum length of 256 characters.<br/>spec:  The virtual node specification to apply.:<br/>meshName:  The name of the service mesh to create the virtual node in.<br/>clientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the
request. Up to 36 letters, numbers, hyphens, and underscores are allowed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceDiscovery</i> The service discovery information for the virtual node. If your virtual node does not
         expect ingress traffic, you can omit this parameter.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>serviceDiscovery</b>: The service discovery information for the virtual node. If your virtual node does not
         expect ingress traffic, you can omit this parameter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>logging</i> The inbound and outbound access logging information for the virtual node.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>logging</b>: The inbound and outbound access logging information for the virtual node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>listeners</i> The listeners that the virtual node is expected to receive inbound traffic from.
         Currently only one listener is supported per virtual node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>backends</i> The backends that the virtual node is expected to send outbound traffic to.<br/>The access log configuration for a virtual node.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>accessLog</b>: The access log configuration for a virtual node.Specifies any AWS Cloud Map information for the virtual node.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>awsCloudMap</b>: Specifies any AWS Cloud Map information for the virtual node.
        :return: virtualNode:  The full description of your virtual node following the create call.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualNodeName</i> The name of the virtual node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status for the virtual node.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>status</b>: The current status for the virtual node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>spec</i> The specifications of the virtual node.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>spec</b>: The specifications of the virtual node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>metadata</i> The associated metadata for the virtual node.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>metadata</b>: The associated metadata for the virtual node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>meshName</i> The name of the service mesh that the virtual node resides in.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the resource. Resources are created at version 1, and this version is
         incremented each time that they're updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uid</i> The unique identifier for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedAt</i> The Unix epoch timestamp in seconds for when the resource was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix epoch timestamp in seconds for when the resource was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The full Amazon Resource Name (ARN) for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>listeners</i> The listeners that the virtual node is expected to receive inbound traffic from.
         Currently only one listener is supported per virtual node.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>backends</i> The backends that the virtual node is expected to send outbound traffic to.<br/>The inbound and outbound access logging information for the virtual node.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>logging</b>: The inbound and outbound access logging information for the virtual node.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the virtual node.<br/>        """
        pass

    def create_virtual_router(self, virtualRouterName: str, spec: VirtualRouterSpec, meshName: str, clientToken: str=None, tags: List[TagRef]=None) -> CreateVirtualRouterOutput:
        """Creates a virtual router within a service mesh.
         Any inbound traffic that your virtual router expects should be specified as a
            <code>listener</code>. 
         Virtual routers handle traffic for one or more virtual services within your mesh. After
         you create your virtual router, create and associate routes for your virtual router that
         direct incoming requests to different virtual nodes.

        :param self:
        :param virtualRouterName:  The name to use for the virtual router.<br/>tags:  Optional metadata that you can apply to the virtual router to assist with categorization
         and organization. Each tag consists of a key and an optional value, both of which you
         define. Tag keys can have a maximum character length of 128 characters, and tag values can have
            a maximum length of 256 characters.<br/>spec:  The virtual router specification to apply.:<br/>meshName:  The name of the service mesh to create the virtual router in.<br/>clientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the
request. Up to 36 letters, numbers, hyphens, and underscores are allowed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>listeners</i> The listeners that the virtual router is expected to receive inbound traffic from.
         Currently only one listener is supported per virtual router.<br/>
        :return: virtualRouter:  The full description of your virtual router following the create call.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualRouterName</i> The name of the virtual router.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the virtual router.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>status</b>: The current status of the virtual router.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>spec</i> The specifications of the virtual router.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>spec</b>: The specifications of the virtual router.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>metadata</i> The associated metadata for the virtual router.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>metadata</b>: The associated metadata for the virtual router.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>meshName</i> The name of the service mesh that the virtual router resides in.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the resource. Resources are created at version 1, and this version is
         incremented each time that they're updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uid</i> The unique identifier for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedAt</i> The Unix epoch timestamp in seconds for when the resource was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix epoch timestamp in seconds for when the resource was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The full Amazon Resource Name (ARN) for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>listeners</i> The listeners that the virtual router is expected to receive inbound traffic from.
         Currently only one listener is supported per virtual router.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the virtual router.<br/>        """
        pass

    def create_virtual_service(self, virtualServiceName: str, spec: VirtualServiceSpec, meshName: str, clientToken: str=None, tags: List[TagRef]=None) -> CreateVirtualServiceOutput:
        """Creates a virtual service within a service mesh.
         A virtual service is an abstraction of a real service that is provided by a virtual node
         directly or indirectly by means of a virtual router. Dependent services call your virtual
         service by its <code>virtualServiceName</code>, and those requests are routed to the
         virtual node or virtual router that is specified as the provider for the virtual
         service.

        :param self:
        :param virtualServiceName:  The name to use for the virtual service.<br/>tags:  Optional metadata that you can apply to the virtual service to assist with
         categorization and organization. Each tag consists of a key and an optional value, both of
         which you define. Tag keys can have a maximum character length of 128 characters, and tag values can have
            a maximum length of 256 characters.<br/>spec:  The virtual service specification to apply.:<br/>meshName:  The name of the service mesh to create the virtual service in.<br/>clientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the
request. Up to 36 letters, numbers, hyphens, and underscores are allowed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>provider</i> The App Mesh object that is acting as the provider for a virtual service. You can specify
         a single virtual node or virtual router.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>provider</b>: The App Mesh object that is acting as the provider for a virtual service. You can specify
         a single virtual node or virtual router.<br/>The virtual node associated with a virtual service.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>virtualNode</b>: The virtual node associated with a virtual service.
        :return: virtualService:  The full description of your virtual service following the create call.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualServiceName</i> The name of the virtual service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the virtual service.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>status</b>: The current status of the virtual service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>spec</i> The specifications of the virtual service.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>spec</b>: The specifications of the virtual service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>metadata</i> An object representing metadata for a resource.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>metadata</b>: An object representing metadata for a resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>meshName</i> The name of the service mesh that the virtual service resides in.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the resource. Resources are created at version 1, and this version is
         incremented each time that they're updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uid</i> The unique identifier for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedAt</i> The Unix epoch timestamp in seconds for when the resource was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix epoch timestamp in seconds for when the resource was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The full Amazon Resource Name (ARN) for the resource.<br/>The App Mesh object that is acting as the provider for a virtual service. You can specify
         a single virtual node or virtual router.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>provider</b>: The App Mesh object that is acting as the provider for a virtual service. You can specify
         a single virtual node or virtual router.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the virtual service.<br/>        """
        pass

    def delete_mesh(self, meshName: str) -> DeleteMeshOutput:
        """Deletes an existing service mesh.
         You must delete all resources (virtual services, routes, virtual routers, and virtual
         nodes) in the service mesh before you can delete the mesh itself.

        :param self:
        :param meshName:  The name of the service mesh to delete.<br/>
        :return: mesh:  The service mesh that was deleted.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the service mesh.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>status</b>: The status of the service mesh.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>spec</i> The associated specification for the service mesh.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>spec</b>: The associated specification for the service mesh.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>metadata</i> The associated metadata for the service mesh.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>metadata</b>: The associated metadata for the service mesh.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>meshName</i> The name of the service mesh.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the resource. Resources are created at version 1, and this version is
         incremented each time that they're updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uid</i> The unique identifier for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedAt</i> The Unix epoch timestamp in seconds for when the resource was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix epoch timestamp in seconds for when the resource was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The full Amazon Resource Name (ARN) for the resource.<br/>The egress filter rules for the service mesh.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>egressFilter</b>: The egress filter rules for the service mesh.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current mesh status.<br/>        """
        pass

    def delete_route(self, virtualRouterName: str, routeName: str, meshName: str) -> DeleteRouteOutput:
        """Deletes an existing route.

        :param self:
        :param virtualRouterName:  The name of the virtual router to delete the route in.<br/>routeName:  The name of the route to delete.<br/>meshName:  The name of the service mesh to delete the route in.<br/>
        :return: route:  The route that was deleted.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualRouterName</i> The virtual router that the route is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the route.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>status</b>: The status of the route.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>spec</i> The specifications of the route.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>spec</b>: The specifications of the route.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>routeName</i> The name of the route.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>metadata</i> The associated metadata for the route.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>metadata</b>: The associated metadata for the route.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>meshName</i> The name of the service mesh that the route resides in.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the resource. Resources are created at version 1, and this version is
         incremented each time that they're updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uid</i> The unique identifier for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedAt</i> The Unix epoch timestamp in seconds for when the resource was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix epoch timestamp in seconds for when the resource was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The full Amazon Resource Name (ARN) for the resource.<br/>The HTTP routing information for the route.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>httpRoute</b>: The HTTP routing information for the route.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status for the route.<br/>        """
        pass

    def delete_virtual_node(self, virtualNodeName: str, meshName: str) -> DeleteVirtualNodeOutput:
        """Deletes an existing virtual node.
         You must delete any virtual services that list a virtual node as a service provider
         before you can delete the virtual node itself.

        :param self:
        :param virtualNodeName:  The name of the virtual node to delete.<br/>meshName:  The name of the service mesh to delete the virtual node in.<br/>
        :return: virtualNode:  The virtual node that was deleted.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualNodeName</i> The name of the virtual node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status for the virtual node.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>status</b>: The current status for the virtual node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>spec</i> The specifications of the virtual node.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>spec</b>: The specifications of the virtual node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>metadata</i> The associated metadata for the virtual node.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>metadata</b>: The associated metadata for the virtual node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>meshName</i> The name of the service mesh that the virtual node resides in.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the resource. Resources are created at version 1, and this version is
         incremented each time that they're updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uid</i> The unique identifier for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedAt</i> The Unix epoch timestamp in seconds for when the resource was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix epoch timestamp in seconds for when the resource was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The full Amazon Resource Name (ARN) for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>listeners</i> The listeners that the virtual node is expected to receive inbound traffic from.
         Currently only one listener is supported per virtual node.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>backends</i> The backends that the virtual node is expected to send outbound traffic to.<br/>The inbound and outbound access logging information for the virtual node.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>logging</b>: The inbound and outbound access logging information for the virtual node.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the virtual node.<br/>        """
        pass

    def delete_virtual_router(self, virtualRouterName: str, meshName: str) -> DeleteVirtualRouterOutput:
        """Deletes an existing virtual router.
         You must delete any routes associated with the virtual router before you can delete the
         router itself.

        :param self:
        :param virtualRouterName:  The name of the virtual router to delete.<br/>meshName:  The name of the service mesh to delete the virtual router in.<br/>
        :return: virtualRouter:  The virtual router that was deleted.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualRouterName</i> The name of the virtual router.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the virtual router.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>status</b>: The current status of the virtual router.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>spec</i> The specifications of the virtual router.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>spec</b>: The specifications of the virtual router.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>metadata</i> The associated metadata for the virtual router.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>metadata</b>: The associated metadata for the virtual router.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>meshName</i> The name of the service mesh that the virtual router resides in.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the resource. Resources are created at version 1, and this version is
         incremented each time that they're updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uid</i> The unique identifier for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedAt</i> The Unix epoch timestamp in seconds for when the resource was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix epoch timestamp in seconds for when the resource was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The full Amazon Resource Name (ARN) for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>listeners</i> The listeners that the virtual router is expected to receive inbound traffic from.
         Currently only one listener is supported per virtual router.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the virtual router.<br/>        """
        pass

    def delete_virtual_service(self, virtualServiceName: str, meshName: str) -> DeleteVirtualServiceOutput:
        """Deletes an existing virtual service.

        :param self:
        :param virtualServiceName:  The name of the virtual service to delete.<br/>meshName:  The name of the service mesh to delete the virtual service in.<br/>
        :return: virtualService:  The virtual service that was deleted.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualServiceName</i> The name of the virtual service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the virtual service.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>status</b>: The current status of the virtual service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>spec</i> The specifications of the virtual service.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>spec</b>: The specifications of the virtual service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>metadata</i> An object representing metadata for a resource.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>metadata</b>: An object representing metadata for a resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>meshName</i> The name of the service mesh that the virtual service resides in.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the resource. Resources are created at version 1, and this version is
         incremented each time that they're updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uid</i> The unique identifier for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedAt</i> The Unix epoch timestamp in seconds for when the resource was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix epoch timestamp in seconds for when the resource was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The full Amazon Resource Name (ARN) for the resource.<br/>The App Mesh object that is acting as the provider for a virtual service. You can specify
         a single virtual node or virtual router.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>provider</b>: The App Mesh object that is acting as the provider for a virtual service. You can specify
         a single virtual node or virtual router.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the virtual service.<br/>        """
        pass

    def describe_mesh(self, meshName: str) -> DescribeMeshOutput:
        """Describes an existing service mesh.

        :param self:
        :param meshName:  The name of the service mesh to describe.<br/>
        :return: mesh:  The full description of your service mesh.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the service mesh.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>status</b>: The status of the service mesh.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>spec</i> The associated specification for the service mesh.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>spec</b>: The associated specification for the service mesh.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>metadata</i> The associated metadata for the service mesh.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>metadata</b>: The associated metadata for the service mesh.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>meshName</i> The name of the service mesh.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the resource. Resources are created at version 1, and this version is
         incremented each time that they're updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uid</i> The unique identifier for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedAt</i> The Unix epoch timestamp in seconds for when the resource was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix epoch timestamp in seconds for when the resource was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The full Amazon Resource Name (ARN) for the resource.<br/>The egress filter rules for the service mesh.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>egressFilter</b>: The egress filter rules for the service mesh.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current mesh status.<br/>        """
        pass

    def describe_route(self, virtualRouterName: str, routeName: str, meshName: str) -> DescribeRouteOutput:
        """Describes an existing route.

        :param self:
        :param virtualRouterName:  The name of the virtual router that the route is associated with.<br/>routeName:  The name of the route to describe.<br/>meshName:  The name of the service mesh that the route resides in.<br/>
        :return: route:  The full description of your route.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualRouterName</i> The virtual router that the route is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the route.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>status</b>: The status of the route.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>spec</i> The specifications of the route.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>spec</b>: The specifications of the route.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>routeName</i> The name of the route.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>metadata</i> The associated metadata for the route.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>metadata</b>: The associated metadata for the route.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>meshName</i> The name of the service mesh that the route resides in.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the resource. Resources are created at version 1, and this version is
         incremented each time that they're updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uid</i> The unique identifier for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedAt</i> The Unix epoch timestamp in seconds for when the resource was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix epoch timestamp in seconds for when the resource was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The full Amazon Resource Name (ARN) for the resource.<br/>The HTTP routing information for the route.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>httpRoute</b>: The HTTP routing information for the route.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status for the route.<br/>        """
        pass

    def describe_virtual_node(self, virtualNodeName: str, meshName: str) -> DescribeVirtualNodeOutput:
        """Describes an existing virtual node.

        :param self:
        :param virtualNodeName:  The name of the virtual node to describe.<br/>meshName:  The name of the service mesh that the virtual node resides in.<br/>
        :return: virtualNode:  The full description of your virtual node.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualNodeName</i> The name of the virtual node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status for the virtual node.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>status</b>: The current status for the virtual node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>spec</i> The specifications of the virtual node.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>spec</b>: The specifications of the virtual node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>metadata</i> The associated metadata for the virtual node.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>metadata</b>: The associated metadata for the virtual node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>meshName</i> The name of the service mesh that the virtual node resides in.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the resource. Resources are created at version 1, and this version is
         incremented each time that they're updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uid</i> The unique identifier for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedAt</i> The Unix epoch timestamp in seconds for when the resource was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix epoch timestamp in seconds for when the resource was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The full Amazon Resource Name (ARN) for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>listeners</i> The listeners that the virtual node is expected to receive inbound traffic from.
         Currently only one listener is supported per virtual node.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>backends</i> The backends that the virtual node is expected to send outbound traffic to.<br/>The inbound and outbound access logging information for the virtual node.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>logging</b>: The inbound and outbound access logging information for the virtual node.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the virtual node.<br/>        """
        pass

    def describe_virtual_router(self, virtualRouterName: str, meshName: str) -> DescribeVirtualRouterOutput:
        """Describes an existing virtual router.

        :param self:
        :param virtualRouterName:  The name of the virtual router to describe.<br/>meshName:  The name of the service mesh that the virtual router resides in.<br/>
        :return: virtualRouter:  The full description of your virtual router.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualRouterName</i> The name of the virtual router.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the virtual router.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>status</b>: The current status of the virtual router.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>spec</i> The specifications of the virtual router.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>spec</b>: The specifications of the virtual router.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>metadata</i> The associated metadata for the virtual router.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>metadata</b>: The associated metadata for the virtual router.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>meshName</i> The name of the service mesh that the virtual router resides in.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the resource. Resources are created at version 1, and this version is
         incremented each time that they're updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uid</i> The unique identifier for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedAt</i> The Unix epoch timestamp in seconds for when the resource was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix epoch timestamp in seconds for when the resource was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The full Amazon Resource Name (ARN) for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>listeners</i> The listeners that the virtual router is expected to receive inbound traffic from.
         Currently only one listener is supported per virtual router.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the virtual router.<br/>        """
        pass

    def describe_virtual_service(self, virtualServiceName: str, meshName: str) -> DescribeVirtualServiceOutput:
        """Describes an existing virtual service.

        :param self:
        :param virtualServiceName:  The name of the virtual service to describe.<br/>meshName:  The name of the service mesh that the virtual service resides in.<br/>
        :return: virtualService:  The full description of your virtual service.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualServiceName</i> The name of the virtual service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the virtual service.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>status</b>: The current status of the virtual service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>spec</i> The specifications of the virtual service.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>spec</b>: The specifications of the virtual service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>metadata</i> An object representing metadata for a resource.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>metadata</b>: An object representing metadata for a resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>meshName</i> The name of the service mesh that the virtual service resides in.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the resource. Resources are created at version 1, and this version is
         incremented each time that they're updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uid</i> The unique identifier for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedAt</i> The Unix epoch timestamp in seconds for when the resource was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix epoch timestamp in seconds for when the resource was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The full Amazon Resource Name (ARN) for the resource.<br/>The App Mesh object that is acting as the provider for a virtual service. You can specify
         a single virtual node or virtual router.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>provider</b>: The App Mesh object that is acting as the provider for a virtual service. You can specify
         a single virtual node or virtual router.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the virtual service.<br/>        """
        pass

    def list_meshes(self, limit: int=None, nextToken: str=None) -> ListMeshesOutput:
        """Returns a list of existing service meshes.

        :param self:
        :param nextToken:  The <code>nextToken</code> value returned from a previous paginated
            <code>ListMeshes</code> request where <code>limit</code> was used and the results
         exceeded the value of that parameter. Pagination continues from the end of the previous
         results that returned the <code>nextToken</code> value. 
         <note>
            This token should be treated as an opaque identifier that is used only to
                retrieve the next items in a list and not for other programmatic purposes.
        </note><br/>limit:  The maximum number of results returned by <code>ListMeshes</code> in paginated output.
         When you use this parameter, <code>ListMeshes</code> returns only <code>limit</code>
         results in a single page along with a <code>nextToken</code> response element. You can see
         the remaining results of the initial request by sending another <code>ListMeshes</code>
         request with the returned <code>nextToken</code> value. This value can be between
         1 and 100. If you don't use this parameter,
            <code>ListMeshes</code> returns up to 100 results and a
            <code>nextToken</code> value if applicable.<br/>
        :return: nextToken:  The <code>nextToken</code> value to include in a future <code>ListMeshes</code> request.
         When the results of a <code>ListMeshes</code> request exceed <code>limit</code>, you can
         use this value to retrieve the next page of results. This value is <code>null</code> when
         there are no more results to return.<br/>meshes:  The list of existing service meshes.<br/>        """
        pass

    def list_routes(self, virtualRouterName: str, meshName: str, limit: int=None, nextToken: str=None) -> ListRoutesOutput:
        """Returns a list of existing routes in a service mesh.

        :param self:
        :param virtualRouterName:  The name of the virtual router to list routes in.<br/>nextToken:  The <code>nextToken</code> value returned from a previous paginated
            <code>ListRoutes</code> request where <code>limit</code> was used and the results
         exceeded the value of that parameter. Pagination continues from the end of the previous
         results that returned the <code>nextToken</code> value.<br/>meshName:  The name of the service mesh to list routes in.<br/>limit:  The maximum number of results returned by <code>ListRoutes</code> in paginated output.
         When you use this parameter, <code>ListRoutes</code> returns only <code>limit</code>
         results in a single page along with a <code>nextToken</code> response element. You can see
         the remaining results of the initial request by sending another <code>ListRoutes</code>
         request with the returned <code>nextToken</code> value. This value can be between
         1 and 100. If you don't use this parameter,
            <code>ListRoutes</code> returns up to 100 results and a
            <code>nextToken</code> value if applicable.<br/>
        :return: routes:  The list of existing routes for the specified service mesh and virtual router.<br/>nextToken:  The <code>nextToken</code> value to include in a future <code>ListRoutes</code> request.
         When the results of a <code>ListRoutes</code> request exceed <code>limit</code>, you can
         use this value to retrieve the next page of results. This value is <code>null</code> when
         there are no more results to return.<br/>        """
        pass

    def list_tags_for_resource(self, resourceArn: str, limit: int=None, nextToken: str=None) -> ListTagsForResourceOutput:
        """List the tags for an App Mesh resource.

        :param self:
        :param resourceArn:  The Amazon Resource Name (ARN) that identifies the resource to list the tags for.<br/>nextToken:  The <code>nextToken</code> value returned from a previous paginated
            <code>ListTagsForResource</code> request where <code>limit</code> was used and the
         results exceeded the value of that parameter. Pagination continues from the end of the
         previous results that returned the <code>nextToken</code> value.<br/>limit:  The maximum number of tag results returned by <code>ListTagsForResource</code> in
         paginated output. When this parameter is used, <code>ListTagsForResource</code> returns
         only <code>limit</code> results in a single page along with a <code>nextToken</code>
         response element. You can see the remaining results of the initial request by sending
         another <code>ListTagsForResource</code> request with the returned <code>nextToken</code>
         value. This value can be between 1 and 100. If you don't use
         this parameter, <code>ListTagsForResource</code> returns up to 100
         results and a <code>nextToken</code> value if applicable.<br/>
        :return: tags:  The tags for the resource.<br/>nextToken:  The <code>nextToken</code> value to include in a future <code>ListTagsForResource</code>
         request. When the results of a <code>ListTagsForResource</code> request exceed
            <code>limit</code>, you can use this value to retrieve the next page of results. This
         value is <code>null</code> when there are no more results to return.<br/>        """
        pass

    def list_virtual_nodes(self, meshName: str, limit: int=None, nextToken: str=None) -> ListVirtualNodesOutput:
        """Returns a list of existing virtual nodes.

        :param self:
        :param nextToken:  The <code>nextToken</code> value returned from a previous paginated
            <code>ListVirtualNodes</code> request where <code>limit</code> was used and the results
         exceeded the value of that parameter. Pagination continues from the end of the previous
         results that returned the <code>nextToken</code> value.<br/>meshName:  The name of the service mesh to list virtual nodes in.<br/>limit:  The maximum number of results returned by <code>ListVirtualNodes</code> in paginated
         output. When you use this parameter, <code>ListVirtualNodes</code> returns only
            <code>limit</code> results in a single page along with a <code>nextToken</code> response
         element. You can see the remaining results of the initial request by sending another
            <code>ListVirtualNodes</code> request with the returned <code>nextToken</code> value.
         This value can be between 1 and 100. If you don't use this
         parameter, <code>ListVirtualNodes</code> returns up to 100 results and a
            <code>nextToken</code> value if applicable.<br/>
        :return: virtualNodes:  The list of existing virtual nodes for the specified service mesh.<br/>nextToken:  The <code>nextToken</code> value to include in a future <code>ListVirtualNodes</code>
         request. When the results of a <code>ListVirtualNodes</code> request exceed
            <code>limit</code>, you can use this value to retrieve the next page of results. This
         value is <code>null</code> when there are no more results to return.<br/>        """
        pass

    def list_virtual_routers(self, meshName: str, limit: int=None, nextToken: str=None) -> ListVirtualRoutersOutput:
        """Returns a list of existing virtual routers in a service mesh.

        :param self:
        :param nextToken:  The <code>nextToken</code> value returned from a previous paginated
            <code>ListVirtualRouters</code> request where <code>limit</code> was used and the
         results exceeded the value of that parameter. Pagination continues from the end of the
         previous results that returned the <code>nextToken</code> value.<br/>meshName:  The name of the service mesh to list virtual routers in.<br/>limit:  The maximum number of results returned by <code>ListVirtualRouters</code> in paginated
         output. When you use this parameter, <code>ListVirtualRouters</code> returns only
            <code>limit</code> results in a single page along with a <code>nextToken</code> response
         element. You can see the remaining results of the initial request by sending another
            <code>ListVirtualRouters</code> request with the returned <code>nextToken</code> value.
         This value can be between 1 and 100. If you don't use this
         parameter, <code>ListVirtualRouters</code> returns up to 100 results and
         a <code>nextToken</code> value if applicable.<br/>
        :return: virtualRouters:  The list of existing virtual routers for the specified service mesh.<br/>nextToken:  The <code>nextToken</code> value to include in a future <code>ListVirtualRouters</code>
         request. When the results of a <code>ListVirtualRouters</code> request exceed
            <code>limit</code>, you can use this value to retrieve the next page of results. This
         value is <code>null</code> when there are no more results to return.<br/>        """
        pass

    def list_virtual_services(self, meshName: str, limit: int=None, nextToken: str=None) -> ListVirtualServicesOutput:
        """Returns a list of existing virtual services in a service mesh.

        :param self:
        :param nextToken:  The <code>nextToken</code> value returned from a previous paginated
            <code>ListVirtualServices</code> request where <code>limit</code> was used and the
         results exceeded the value of that parameter. Pagination continues from the end of the
         previous results that returned the <code>nextToken</code> value.<br/>meshName:  The name of the service mesh to list virtual services in.<br/>limit:  The maximum number of results returned by <code>ListVirtualServices</code> in paginated
         output. When you use this parameter, <code>ListVirtualServices</code> returns only
            <code>limit</code> results in a single page along with a <code>nextToken</code> response
         element. You can see the remaining results of the initial request by sending another
            <code>ListVirtualServices</code> request with the returned <code>nextToken</code> value.
         This value can be between 1 and 100. If you don't use this
         parameter, <code>ListVirtualServices</code> returns up to 100 results and
         a <code>nextToken</code> value if applicable.<br/>
        :return: virtualServices:  The list of existing virtual services for the specified service mesh.<br/>nextToken:  The <code>nextToken</code> value to include in a future <code>ListVirtualServices</code>
         request. When the results of a <code>ListVirtualServices</code> request exceed
            <code>limit</code>, you can use this value to retrieve the next page of results. This
         value is <code>null</code> when there are no more results to return.<br/>        """
        pass

    def tag_resource(self, tags: List[TagRef], resourceArn: str) -> TagResourceOutput:
        """Associates the specified tags to a resource with the specified <code>resourceArn</code>.
         If existing tags on a resource aren't specified in the request parameters, they aren't
         changed. When a resource is deleted, the tags associated with that resource are also
         deleted.

        :param self:
        :param tags:  The tags to add to the resource. A tag is an array of key-value pairs.
         Tag keys can have a maximum character length of 128 characters, and tag values can have
            a maximum length of 256 characters.<br/>resourceArn:  The Amazon Resource Name (ARN) of the resource to add tags to.<br/>
        :return:         """
        pass

    def untag_resource(self, tagKeys: List[TagKey], resourceArn: str) -> UntagResourceOutput:
        """Deletes specified tags from a resource.

        :param self:
        :param tagKeys:  The keys of the tags to be removed.<br/>resourceArn:  The Amazon Resource Name (ARN) of the resource to delete tags from.<br/>
        :return:         """
        pass

    def update_mesh(self, meshName: str, clientToken: str=None, spec: MeshSpec=None) -> UpdateMeshOutput:
        """Updates an existing service mesh.

        :param self:
        :param spec:  The service mesh specification to apply.:<br/>meshName:  The name of the service mesh to update.<br/>clientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the
request. Up to 36 letters, numbers, hyphens, and underscores are allowed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>egressFilter</i> The egress filter rules for the service mesh.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>egressFilter</b>: The egress filter rules for the service mesh.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The egress filter type. By default, the type is <code>DROP_ALL</code>, which allows
         egress only from virtual nodes to other defined resources in the service mesh (and any
         traffic to <code>*.amazonaws.com</code> for AWS API calls). You can set the egress filter
         type to <code>ALLOW_ALL</code> to allow egress to any endpoint inside or outside of the
         service mesh.<br/>
        :return: mesh:  An object representing a service mesh returned by a describe operation.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the service mesh.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>status</b>: The status of the service mesh.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>spec</i> The associated specification for the service mesh.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>spec</b>: The associated specification for the service mesh.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>metadata</i> The associated metadata for the service mesh.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>metadata</b>: The associated metadata for the service mesh.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>meshName</i> The name of the service mesh.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the resource. Resources are created at version 1, and this version is
         incremented each time that they're updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uid</i> The unique identifier for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedAt</i> The Unix epoch timestamp in seconds for when the resource was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix epoch timestamp in seconds for when the resource was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The full Amazon Resource Name (ARN) for the resource.<br/>The egress filter rules for the service mesh.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>egressFilter</b>: The egress filter rules for the service mesh.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current mesh status.<br/>        """
        pass

    def update_route(self, virtualRouterName: str, spec: RouteSpec, routeName: str, meshName: str, clientToken: str=None) -> UpdateRouteOutput:
        """Updates an existing route for a specified service mesh and virtual router.

        :param self:
        :param virtualRouterName:  The name of the virtual router that the route is associated with.<br/>spec:  The new route specification to apply. This overwrites the existing data.:<br/>routeName:  The name of the route to update.<br/>meshName:  The name of the service mesh that the route resides in.<br/>clientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the
request. Up to 36 letters, numbers, hyphens, and underscores are allowed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tcpRoute</i> The TCP routing information for the route.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>tcpRoute</b>: The TCP routing information for the route.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>httpRoute</i> The HTTP routing information for the route.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>httpRoute</b>: The HTTP routing information for the route.<br/>The action to take if a match is determined.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>action</b>: The action to take if a match is determined.The action to take if a match is determined.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>action</b>: The action to take if a match is determined.
        :return: route:  A full description of the route that was updated.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualRouterName</i> The virtual router that the route is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the route.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>status</b>: The status of the route.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>spec</i> The specifications of the route.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>spec</b>: The specifications of the route.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>routeName</i> The name of the route.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>metadata</i> The associated metadata for the route.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>metadata</b>: The associated metadata for the route.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>meshName</i> The name of the service mesh that the route resides in.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the resource. Resources are created at version 1, and this version is
         incremented each time that they're updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uid</i> The unique identifier for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedAt</i> The Unix epoch timestamp in seconds for when the resource was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix epoch timestamp in seconds for when the resource was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The full Amazon Resource Name (ARN) for the resource.<br/>The HTTP routing information for the route.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>httpRoute</b>: The HTTP routing information for the route.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status for the route.<br/>        """
        pass

    def update_virtual_node(self, virtualNodeName: str, spec: VirtualNodeSpec, meshName: str, clientToken: str=None) -> UpdateVirtualNodeOutput:
        """Updates an existing virtual node in a specified service mesh.

        :param self:
        :param virtualNodeName:  The name of the virtual node to update.<br/>spec:  The new virtual node specification to apply. This overwrites the existing data.:<br/>meshName:  The name of the service mesh that the virtual node resides in.<br/>clientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the
request. Up to 36 letters, numbers, hyphens, and underscores are allowed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceDiscovery</i> The service discovery information for the virtual node. If your virtual node does not
         expect ingress traffic, you can omit this parameter.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>serviceDiscovery</b>: The service discovery information for the virtual node. If your virtual node does not
         expect ingress traffic, you can omit this parameter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>logging</i> The inbound and outbound access logging information for the virtual node.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>logging</b>: The inbound and outbound access logging information for the virtual node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>listeners</i> The listeners that the virtual node is expected to receive inbound traffic from.
         Currently only one listener is supported per virtual node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>backends</i> The backends that the virtual node is expected to send outbound traffic to.<br/>The access log configuration for a virtual node.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>accessLog</b>: The access log configuration for a virtual node.Specifies any AWS Cloud Map information for the virtual node.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>awsCloudMap</b>: Specifies any AWS Cloud Map information for the virtual node.
        :return: virtualNode:  A full description of the virtual node that was updated.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualNodeName</i> The name of the virtual node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status for the virtual node.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>status</b>: The current status for the virtual node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>spec</i> The specifications of the virtual node.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>spec</b>: The specifications of the virtual node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>metadata</i> The associated metadata for the virtual node.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>metadata</b>: The associated metadata for the virtual node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>meshName</i> The name of the service mesh that the virtual node resides in.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the resource. Resources are created at version 1, and this version is
         incremented each time that they're updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uid</i> The unique identifier for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedAt</i> The Unix epoch timestamp in seconds for when the resource was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix epoch timestamp in seconds for when the resource was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The full Amazon Resource Name (ARN) for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>listeners</i> The listeners that the virtual node is expected to receive inbound traffic from.
         Currently only one listener is supported per virtual node.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>backends</i> The backends that the virtual node is expected to send outbound traffic to.<br/>The inbound and outbound access logging information for the virtual node.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>logging</b>: The inbound and outbound access logging information for the virtual node.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the virtual node.<br/>        """
        pass

    def update_virtual_router(self, virtualRouterName: str, spec: VirtualRouterSpec, meshName: str, clientToken: str=None) -> UpdateVirtualRouterOutput:
        """Updates an existing virtual router in a specified service mesh.

        :param self:
        :param virtualRouterName:  The name of the virtual router to update.<br/>spec:  The new virtual router specification to apply. This overwrites the existing data.:<br/>meshName:  The name of the service mesh that the virtual router resides in.<br/>clientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the
request. Up to 36 letters, numbers, hyphens, and underscores are allowed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>listeners</i> The listeners that the virtual router is expected to receive inbound traffic from.
         Currently only one listener is supported per virtual router.<br/>
        :return: virtualRouter:  A full description of the virtual router that was updated.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualRouterName</i> The name of the virtual router.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the virtual router.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>status</b>: The current status of the virtual router.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>spec</i> The specifications of the virtual router.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>spec</b>: The specifications of the virtual router.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>metadata</i> The associated metadata for the virtual router.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>metadata</b>: The associated metadata for the virtual router.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>meshName</i> The name of the service mesh that the virtual router resides in.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the resource. Resources are created at version 1, and this version is
         incremented each time that they're updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uid</i> The unique identifier for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedAt</i> The Unix epoch timestamp in seconds for when the resource was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix epoch timestamp in seconds for when the resource was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The full Amazon Resource Name (ARN) for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>listeners</i> The listeners that the virtual router is expected to receive inbound traffic from.
         Currently only one listener is supported per virtual router.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the virtual router.<br/>        """
        pass

    def update_virtual_service(self, virtualServiceName: str, spec: VirtualServiceSpec, meshName: str, clientToken: str=None) -> UpdateVirtualServiceOutput:
        """Updates an existing virtual service in a specified service mesh.

        :param self:
        :param virtualServiceName:  The name of the virtual service to update.<br/>spec:  The new virtual service specification to apply. This overwrites the existing
         data.:<br/>meshName:  The name of the service mesh that the virtual service resides in.<br/>clientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the
request. Up to 36 letters, numbers, hyphens, and underscores are allowed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>provider</i> The App Mesh object that is acting as the provider for a virtual service. You can specify
         a single virtual node or virtual router.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>provider</b>: The App Mesh object that is acting as the provider for a virtual service. You can specify
         a single virtual node or virtual router.<br/>The virtual node associated with a virtual service.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>virtualNode</b>: The virtual node associated with a virtual service.
        :return: virtualService:  A full description of the virtual service that was updated.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualServiceName</i> The name of the virtual service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the virtual service.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>status</b>: The current status of the virtual service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>spec</i> The specifications of the virtual service.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>spec</b>: The specifications of the virtual service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>metadata</i> An object representing metadata for a resource.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>metadata</b>: An object representing metadata for a resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>meshName</i> The name of the service mesh that the virtual service resides in.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the resource. Resources are created at version 1, and this version is
         incremented each time that they're updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uid</i> The unique identifier for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedAt</i> The Unix epoch timestamp in seconds for when the resource was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix epoch timestamp in seconds for when the resource was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The full Amazon Resource Name (ARN) for the resource.<br/>The App Mesh object that is acting as the provider for a virtual service. You can specify
         a single virtual node or virtual router.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>provider</b>: The App Mesh object that is acting as the provider for a virtual service. You can specify
         a single virtual node or virtual router.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the virtual service.<br/>        """
        pass



class AppStream(BaseClient):
    """<fullname>Amazon AppStream 2.0</fullname> This is the <i>Amazon AppStream 2.0 API Reference</i>. This documentation provides descriptions and syntax for each of the actions and data types in AppStream 2.0. AppStream 2.0 is a fully managed, secure application streaming service that lets you stream desktop applications to users without rewriting applications. AppStream 2.0 manages the AWS resources that are required to host and run your applications, scales automatically, and provides access to your users on demand. To learn more about AppStream 2.0, see the following resources: <ul> <li>  <a href="http://aws.amazon.com/appstream2">Amazon AppStream 2.0 product page</a>  </li> <li>  <a href="http://aws.amazon.com/documentation/appstream2">Amazon AppStream 2.0 documentation</a>  </li> </ul>"""

    class FleetStartedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_fleets` for valid parameters"""
            pass
    class FleetStoppedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_fleets` for valid parameters"""
            pass


    class DescribeDirectoryConfigsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_directory_configs` for valid parameters"""
            pass
    class DescribeFleetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_fleets` for valid parameters"""
            pass
    class DescribeImageBuildersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_image_builders` for valid parameters"""
            pass
    class DescribeImagesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_images` for valid parameters"""
            pass
    class DescribeSessionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_sessions` for valid parameters"""
            pass
    class DescribeStacksPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_stacks` for valid parameters"""
            pass
    class DescribeUserStackAssociationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_user_stack_associations` for valid parameters"""
            pass
    class DescribeUsersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_users` for valid parameters"""
            pass
    class ListAssociatedFleetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_associated_fleets` for valid parameters"""
            pass
    class ListAssociatedStacksPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_associated_stacks` for valid parameters"""
            pass


    class AssociateFleetRequest(Mapping):
        pass
    
    class AssociateFleetResult(Mapping):
        pass
    
    class BatchAssociateUserStackRequest(Mapping):
        pass
    
    class BatchAssociateUserStackResult(Mapping):
        pass
    
    class UserStackAssociation(Mapping):
        pass
    
    class BatchDisassociateUserStackRequest(Mapping):
        pass
    
    class BatchDisassociateUserStackResult(Mapping):
        pass
    
    class CopyImageRequest(Mapping):
        pass
    
    class CopyImageResponse(Mapping):
        pass
    
    class CreateDirectoryConfigRequest(Mapping):
        pass
    
    class CreateDirectoryConfigResult(Mapping):
        pass
    
    class OrganizationalUnitDistinguishedName(object):
        pass
    
    class ServiceAccountCredentials(Mapping):
        pass
    
    class CreateFleetRequest(Mapping):
        pass
    
    class CreateFleetResult(Mapping):
        pass
    
    class ComputeCapacity(Mapping):
        pass
    
    class VpcConfig(Mapping):
        pass
    
    class BooleanObject(object):
        pass
    
    class DomainJoinInfo(Mapping):
        pass
    
    class Tags(object):
        pass
    
    class CreateImageBuilderRequest(Mapping):
        pass
    
    class CreateImageBuilderResult(Mapping):
        pass
    
    class CreateImageBuilderStreamingURLRequest(Mapping):
        pass
    
    class CreateImageBuilderStreamingURLResult(Mapping):
        pass
    
    class Long(object):
        pass
    
    class CreateStackRequest(Mapping):
        pass
    
    class CreateStackResult(Mapping):
        pass
    
    class StorageConnector(Mapping):
        pass
    
    class UserSetting(Mapping):
        pass
    
    class ApplicationSettings(Mapping):
        pass
    
    class CreateStreamingURLRequest(Mapping):
        pass
    
    class CreateStreamingURLResult(Mapping):
        pass
    
    class CreateUsageReportSubscriptionRequest(Mapping):
        pass
    
    class CreateUsageReportSubscriptionResult(Mapping):
        pass
    
    class CreateUserRequest(Mapping):
        pass
    
    class CreateUserResult(Mapping):
        pass
    
    class DeleteDirectoryConfigRequest(Mapping):
        pass
    
    class DeleteDirectoryConfigResult(Mapping):
        pass
    
    class DeleteFleetRequest(Mapping):
        pass
    
    class DeleteFleetResult(Mapping):
        pass
    
    class DeleteImageRequest(Mapping):
        pass
    
    class DeleteImageResult(Mapping):
        pass
    
    class DeleteImageBuilderRequest(Mapping):
        pass
    
    class DeleteImageBuilderResult(Mapping):
        pass
    
    class DeleteImagePermissionsRequest(Mapping):
        pass
    
    class DeleteImagePermissionsResult(Mapping):
        pass
    
    class DeleteStackRequest(Mapping):
        pass
    
    class DeleteStackResult(Mapping):
        pass
    
    class DeleteUsageReportSubscriptionRequest(Mapping):
        pass
    
    class DeleteUsageReportSubscriptionResult(Mapping):
        pass
    
    class DeleteUserRequest(Mapping):
        pass
    
    class DeleteUserResult(Mapping):
        pass
    
    class DescribeDirectoryConfigsRequest(Mapping):
        pass
    
    class DescribeDirectoryConfigsResult(Mapping):
        pass
    
    class DirectoryName(object):
        pass
    
    class DescribeFleetsRequest(Mapping):
        pass
    
    class DescribeFleetsResult(Mapping):
        pass
    
    class String(object):
        pass
    
    class DescribeImageBuildersRequest(Mapping):
        pass
    
    class DescribeImageBuildersResult(Mapping):
        pass
    
    class DescribeImagePermissionsRequest(Mapping):
        pass
    
    class DescribeImagePermissionsResult(Mapping):
        pass
    
    class AwsAccountId(object):
        pass
    
    class DescribeImagesRequest(Mapping):
        pass
    
    class DescribeImagesResult(Mapping):
        pass
    
    class Arn(object):
        pass
    
    class DescribeSessionsRequest(Mapping):
        pass
    
    class DescribeSessionsResult(Mapping):
        pass
    
    class DescribeStacksRequest(Mapping):
        pass
    
    class DescribeStacksResult(Mapping):
        pass
    
    class DescribeUsageReportSubscriptionsRequest(Mapping):
        pass
    
    class DescribeUsageReportSubscriptionsResult(Mapping):
        pass
    
    class DescribeUserStackAssociationsRequest(Mapping):
        pass
    
    class DescribeUserStackAssociationsResult(Mapping):
        pass
    
    class DescribeUsersRequest(Mapping):
        pass
    
    class DescribeUsersResult(Mapping):
        pass
    
    class DisableUserRequest(Mapping):
        pass
    
    class DisableUserResult(Mapping):
        pass
    
    class DisassociateFleetRequest(Mapping):
        pass
    
    class DisassociateFleetResult(Mapping):
        pass
    
    class EnableUserRequest(Mapping):
        pass
    
    class EnableUserResult(Mapping):
        pass
    
    class ExpireSessionRequest(Mapping):
        pass
    
    class ExpireSessionResult(Mapping):
        pass
    
    class ListAssociatedFleetsRequest(Mapping):
        pass
    
    class ListAssociatedFleetsResult(Mapping):
        pass
    
    class ListAssociatedStacksRequest(Mapping):
        pass
    
    class ListAssociatedStacksResult(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class StartFleetRequest(Mapping):
        pass
    
    class StartFleetResult(Mapping):
        pass
    
    class StartImageBuilderRequest(Mapping):
        pass
    
    class StartImageBuilderResult(Mapping):
        pass
    
    class StopFleetRequest(Mapping):
        pass
    
    class StopFleetResult(Mapping):
        pass
    
    class StopImageBuilderRequest(Mapping):
        pass
    
    class StopImageBuilderResult(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateDirectoryConfigRequest(Mapping):
        pass
    
    class UpdateDirectoryConfigResult(Mapping):
        pass
    
    class UpdateFleetRequest(Mapping):
        pass
    
    class UpdateFleetResult(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class FleetAttribute(object):
        pass
    
    class UpdateImagePermissionsRequest(Mapping):
        pass
    
    class UpdateImagePermissionsResult(Mapping):
        pass
    
    class ImagePermissions(Mapping):
        pass
    
    class UpdateStackRequest(Mapping):
        pass
    
    class UpdateStackResult(Mapping):
        pass
    
    class StackAttribute(object):
        pass
    
    def associate_fleet(self, StackName: str, FleetName: str) -> AssociateFleetResult:
        """Associates the specified fleet with the specified stack.

        :param self:
        :param StackName:  The name of the stack.<br/>FleetName:  The name of the fleet. <br/>
        :return:         """
        pass

    def batch_associate_user_stack(self, UserStackAssociations: List[UserStackAssociation]) -> BatchAssociateUserStackResult:
        """Associates the specified users with the specified stacks. Users in a user pool cannot be assigned to stacks with fleets that are joined to an Active Directory domain.

        :param self:
        :param UserStackAssociations:  The list of UserStackAssociation objects.<br/>
        :return: errors:  The list of UserStackAssociationError objects.<br/>        """
        pass

    def batch_disassociate_user_stack(self, UserStackAssociations: List[UserStackAssociation]) -> BatchDisassociateUserStackResult:
        """Disassociates the specified users from the specified stacks.

        :param self:
        :param UserStackAssociations:  The list of UserStackAssociation objects.<br/>
        :return: errors:  The list of UserStackAssociationError objects.<br/>        """
        pass

    def copy_image(self, DestinationRegion: str, DestinationImageName: str, SourceImageName: str, DestinationImageDescription: str=None) -> CopyImageResponse:
        """Copies the image within the same region or to a new region within the same AWS account. Note that any tags you added to the image will not be copied.

        :param self:
        :param DestinationImageDescription:  The description that the image will have when it is copied to the destination.<br/>DestinationRegion:  The destination region to which the image will be copied. This parameter is required, even if you are copying an image within the same region.<br/>DestinationImageName:  The name that the image will have when it is copied to the destination.<br/>SourceImageName:  The name of the image to copy.<br/>
        :return: DestinationImageName:  The name of the destination image.<br/>        """
        pass

    def create_directory_config(self, ServiceAccountCredentials: ServiceAccountCredentials, OrganizationalUnitDistinguishedNames: List[OrganizationalUnitDistinguishedName], DirectoryName: str) -> CreateDirectoryConfigResult:
        """Creates a Directory Config object in AppStream 2.0. This object includes the configuration information required to join fleets and image builders to Microsoft Active Directory domains.

        :param self:
        :param ServiceAccountCredentials:  The credentials for the service account used by the fleet or image builder to connect to the directory.:<br/>OrganizationalUnitDistinguishedNames:  The distinguished names of the organizational units for computer accounts.<br/>DirectoryName:  The fully qualified name of the directory (for example, corp.example.com).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountPassword</i> The password for the account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountName</i> The user name of the account. This account must have the following privileges: create computer objects, join computers to the domain, and change/reset the password on descendant computer objects for the organizational units specified.<br/>
        :return: DirectoryConfig:  Information about the directory configuration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The time the directory configuration was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccountCredentials</i> The credentials for the service account used by the fleet or image builder to connect to the directory.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ServiceAccountCredentials</b>: The credentials for the service account used by the fleet or image builder to connect to the directory.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationalUnitDistinguishedNames</i> The distinguished names of the organizational units for computer accounts.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DirectoryName</i> The fully qualified name of the directory (for example, corp.example.com).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountPassword</i> The password for the account.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountName</i> The user name of the account. This account must have the following privileges: create computer objects, join computers to the domain, and change/reset the password on descendant computer objects for the organizational units specified.<br/>        """
        pass

    def create_fleet(self, ComputeCapacity: ComputeCapacity, InstanceType: str, Name: str, ImageName: str=None, ImageArn: str=None, FleetType: str=None, VpcConfig: VpcConfig=None, MaxUserDurationInSeconds: int=None, DisconnectTimeoutInSeconds: int=None, Description: str=None, DisplayName: str=None, EnableDefaultInternetAccess: BooleanObject=None, DomainJoinInfo: DomainJoinInfo=None, Tags: Tags=None, IdleDisconnectTimeoutInSeconds: int=None) -> CreateFleetResult:
        """Creates a fleet. A fleet consists of streaming instances that run a specified image.

        :param self:
        :param IdleDisconnectTimeoutInSeconds:  The amount of time that users can be idle (inactive) before they are disconnected from their streaming session and the <code>DisconnectTimeoutInSeconds</code> time interval begins. Users are notified before they are disconnected due to inactivity. If they try to reconnect to the streaming session before the time interval specified in <code>DisconnectTimeoutInSeconds</code> elapses, they are connected to their previous session. Users are considered idle when they stop providing keyboard or mouse input during their streaming session. File uploads and downloads, audio in, audio out, and pixels changing do not qualify as user activity. If users continue to be idle after the time interval in <code>IdleDisconnectTimeoutInSeconds</code> elapses, they are disconnected. To prevent users from being disconnected due to inactivity, specify a value of 0. Otherwise, specify a value between 60 and 3600. The default value is 0. <note> If you enable this feature, we recommend that you specify a value that corresponds exactly to a whole number of minutes (for example, 60, 120, and 180). If you don't do this, the value is rounded to the nearest minute. For example, if you specify a value of 70, users are disconnected after 1 minute of inactivity. If you specify a value that is at the midpoint between two different minutes, the value is rounded up. For example, if you specify a value of 90, users are disconnected after 2 minutes of inactivity.  </note><br/>Tags:  The tags to associate with the fleet. A tag is a key-value pair, and the value is optional. For example, Environment=Test. If you do not specify a value, Environment=.  If you do not specify a value, the value is set to an empty string. Generally allowed characters are: letters, numbers, and spaces representable in UTF-8, and the following special characters:  _ . : / = + \ - @ For more information, see <a href="https://docs.aws.amazon.com/appstream2/latest/developerguide/tagging-basic.html">Tagging Your Resources</a> in the <i>Amazon AppStream 2.0 Developer Guide</i>.<br/>DomainJoinInfo:  The name of the directory and organizational unit (OU) to use to join the fleet to a Microsoft Active Directory domain. :<br/>EnableDefaultInternetAccess:  Enables or disables default internet access for the fleet.<br/>DisplayName:  The fleet name to display.<br/>Description:  The description to display.<br/>DisconnectTimeoutInSeconds:  The amount of time that a streaming session remains active after users disconnect. If users try to reconnect to the streaming session after a disconnection or network interruption within this time interval, they are connected to their previous session. Otherwise, they are connected to a new session with a new streaming instance.  Specify a value between 60 and 360000.<br/>MaxUserDurationInSeconds:  The maximum amount of time that a streaming session can remain active, in seconds. If users are still connected to a streaming instance five minutes before this limit is reached, they are prompted to save any open documents before being disconnected. After this time elapses, the instance is terminated and replaced by a new instance. Specify a value between 600 and 360000.<br/>VpcConfig:  The VPC configuration for the fleet.:<br/>ComputeCapacity:  The desired capacity for the fleet.:<br/>FleetType:  The fleet type. <dl> <dt>ALWAYS_ON</dt> <dd> Provides users with instant-on access to their apps. You are charged for all running instances in your fleet, even if no users are streaming apps. </dd> <dt>ON_DEMAND</dt> <dd> Provide users with access to applications after they connect, which takes one to two minutes. You are charged for instance streaming when users are connected and a small hourly fee for instances that are not streaming apps. </dd> </dl><br/>InstanceType:  The instance type to use when launching fleet instances. The following instance types are available: <ul> <li> stream.standard.medium </li> <li> stream.standard.large </li> <li> stream.compute.large </li> <li> stream.compute.xlarge </li> <li> stream.compute.2xlarge </li> <li> stream.compute.4xlarge </li> <li> stream.compute.8xlarge </li> <li> stream.memory.large </li> <li> stream.memory.xlarge </li> <li> stream.memory.2xlarge </li> <li> stream.memory.4xlarge </li> <li> stream.memory.8xlarge </li> <li> stream.graphics-design.large </li> <li> stream.graphics-design.xlarge </li> <li> stream.graphics-design.2xlarge </li> <li> stream.graphics-design.4xlarge </li> <li> stream.graphics-desktop.2xlarge </li> <li> stream.graphics-pro.4xlarge </li> <li> stream.graphics-pro.8xlarge </li> <li> stream.graphics-pro.16xlarge </li> </ul><br/>ImageArn:  The ARN of the public, private, or shared image to use.<br/>ImageName:  The name of the image used to create the fleet.<br/>Name:  A unique name for the fleet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DesiredInstances</i> The desired number of streaming instances.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The identifiers of the security groups for the fleet or image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> The identifiers of the subnets to which a network interface is attached from the fleet instance or image builder instance. Fleet instances use one or more subnets. Image builder instances use one subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationalUnitDistinguishedName</i> The distinguished name of the organizational unit for computer accounts.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DirectoryName</i> The fully qualified name of the directory (for example, corp.example.com).<br/>
        :return: Fleet:  Information about the fleet.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IdleDisconnectTimeoutInSeconds</i> The amount of time that users can be idle (inactive) before they are disconnected from their streaming session and the <code>DisconnectTimeoutInSeconds</code> time interval begins. Users are notified before they are disconnected due to inactivity. If users try to reconnect to the streaming session before the time interval specified in <code>DisconnectTimeoutInSeconds</code> elapses, they are connected to their previous session. Users are considered idle when they stop providing keyboard or mouse input during their streaming session. File uploads and downloads, audio in, audio out, and pixels changing do not qualify as user activity. If users continue to be idle after the time interval in <code>IdleDisconnectTimeoutInSeconds</code> elapses, they are disconnected. To prevent users from being disconnected due to inactivity, specify a value of 0. Otherwise, specify a value between 60 and 3600. The default value is 0. <note> If you enable this feature, we recommend that you specify a value that corresponds exactly to a whole number of minutes (for example, 60, 120, and 180). If you don't do this, the value is rounded to the nearest minute. For example, if you specify a value of 70, users are disconnected after 1 minute of inactivity. If you specify a value that is at the midpoint between two different minutes, the value is rounded up. For example, if you specify a value of 90, users are disconnected after 2 minutes of inactivity.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainJoinInfo</i> The name of the directory and organizational unit (OU) to use to join the fleet to a Microsoft Active Directory domain. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DomainJoinInfo</b>: The name of the directory and organizational unit (OU) to use to join the fleet to a Microsoft Active Directory domain. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnableDefaultInternetAccess</i> Indicates whether default internet access is enabled for the fleet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FleetErrors</i> The fleet errors.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The time the fleet was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcConfig</i> The VPC configuration for the fleet.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VpcConfig</b>: The VPC configuration for the fleet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The current state for the fleet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisconnectTimeoutInSeconds</i> The amount of time that a streaming session remains active after users disconnect. If they try to reconnect to the streaming session after a disconnection or network interruption within this time interval, they are connected to their previous session. Otherwise, they are connected to a new session with a new streaming instance. Specify a value between 60 and 360000.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxUserDurationInSeconds</i> The maximum amount of time that a streaming session can remain active, in seconds. If users are still connected to a streaming instance five minutes before this limit is reached, they are prompted to save any open documents before being disconnected. After this time elapses, the instance is terminated and replaced by a new instance.  Specify a value between 600 and 360000.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ComputeCapacityStatus</i> The capacity status for the fleet.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ComputeCapacityStatus</b>: The capacity status for the fleet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FleetType</i> The fleet type. <dl> <dt>ALWAYS_ON</dt> <dd> Provides users with instant-on access to their apps. You are charged for all running instances in your fleet, even if no users are streaming apps. </dd> <dt>ON_DEMAND</dt> <dd> Provide users with access to applications after they connect, which takes one to two minutes. You are charged for instance streaming when users are connected and a small hourly fee for instances that are not streaming apps. </dd> </dl><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i> The instance type to use when launching fleet instances.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImageArn</i> The ARN for the public, private, or shared image.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImageName</i> The name of the image used to create the fleet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description to display.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayName</i> The fleet name to display.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the fleet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN for the fleet.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Available</i> The number of currently available instances that can be used to stream sessions.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InUse</i> The number of instances in use for streaming.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Running</i> The total number of simultaneous streaming instances that are running.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Desired</i> The desired number of streaming instances.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The identifiers of the security groups for the fleet or image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> The identifiers of the subnets to which a network interface is attached from the fleet instance or image builder instance. Fleet instances use one or more subnets. Image builder instances use one subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationalUnitDistinguishedName</i> The distinguished name of the organizational unit for computer accounts.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DirectoryName</i> The fully qualified name of the directory (for example, corp.example.com).<br/>        """
        pass

    def create_image_builder(self, InstanceType: str, Name: str, ImageName: str=None, ImageArn: str=None, Description: str=None, DisplayName: str=None, VpcConfig: VpcConfig=None, EnableDefaultInternetAccess: BooleanObject=None, DomainJoinInfo: DomainJoinInfo=None, AppstreamAgentVersion: str=None, Tags: Tags=None) -> CreateImageBuilderResult:
        """Creates an image builder. An image builder is a virtual machine that is used to create an image. The initial state of the builder is <code>PENDING</code>. When it is ready, the state is <code>RUNNING</code>.

        :param self:
        :param Tags:  The tags to associate with the image builder. A tag is a key-value pair, and the value is optional. For example, Environment=Test. If you do not specify a value, Environment=.  Generally allowed characters are: letters, numbers, and spaces representable in UTF-8, and the following special characters:  _ . : / = + \ - @ If you do not specify a value, the value is set to an empty string. For more information about tags, see <a href="https://docs.aws.amazon.com/appstream2/latest/developerguide/tagging-basic.html">Tagging Your Resources</a> in the <i>Amazon AppStream 2.0 Developer Guide</i>.<br/>AppstreamAgentVersion:  The version of the AppStream 2.0 agent to use for this image builder. To use the latest version of the AppStream 2.0 agent, specify [LATEST]. <br/>DomainJoinInfo:  The name of the directory and organizational unit (OU) to use to join the image builder to a Microsoft Active Directory domain. :<br/>EnableDefaultInternetAccess:  Enables or disables default internet access for the image builder.<br/>VpcConfig:  The VPC configuration for the image builder. You can specify only one subnet.:<br/>DisplayName:  The image builder name to display.<br/>Description:  The description to display.<br/>InstanceType:  The instance type to use when launching the image builder.<br/>ImageArn:  The ARN of the public, private, or shared image to use.<br/>ImageName:  The name of the image used to create the image builder.<br/>Name:  A unique name for the image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The identifiers of the security groups for the fleet or image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> The identifiers of the subnets to which a network interface is attached from the fleet instance or image builder instance. Fleet instances use one or more subnets. Image builder instances use one subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationalUnitDistinguishedName</i> The distinguished name of the organizational unit for computer accounts.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DirectoryName</i> The fully qualified name of the directory (for example, corp.example.com).<br/>
        :return: ImageBuilder:  Information about the image builder.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AppstreamAgentVersion</i> The version of the AppStream 2.0 agent that is currently being used by the image builder. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImageBuilderErrors</i> The image builder errors.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkAccessConfiguration</i> Describes the network details of the fleet or image builder instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NetworkAccessConfiguration</b>: Describes the network details of the fleet or image builder instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainJoinInfo</i> The name of the directory and organizational unit (OU) to use to join the image builder to a Microsoft Active Directory domain. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DomainJoinInfo</b>: The name of the directory and organizational unit (OU) to use to join the image builder to a Microsoft Active Directory domain. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnableDefaultInternetAccess</i> Enables or disables default internet access for the image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The time stamp when the image builder was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StateChangeReason</i> The reason why the last state change occurred.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StateChangeReason</b>: The reason why the last state change occurred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The operating system platform of the image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i> The instance type for the image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcConfig</i> The VPC configuration of the image builder.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VpcConfig</b>: The VPC configuration of the image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayName</i> The image builder name to display.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description to display.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImageArn</i> The ARN of the image from which this builder was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN for the image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The identifiers of the security groups for the fleet or image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> The identifiers of the subnets to which a network interface is attached from the fleet instance or image builder instance. Fleet instances use one or more subnets. Image builder instances use one subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> The state change reason message.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Code</i> The state change reason code.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationalUnitDistinguishedName</i> The distinguished name of the organizational unit for computer accounts.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DirectoryName</i> The fully qualified name of the directory (for example, corp.example.com).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EniId</i> The resource identifier of the elastic network interface that is attached to instances in your VPC. All network interfaces have the eni-xxxxxxxx resource identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EniPrivateIpAddress</i> The private IP address of the elastic network interface that is attached to instances in your VPC.<br/>        """
        pass

    def create_image_builder_streaming_url(self, Name: str, Validity: Long=None) -> CreateImageBuilderStreamingURLResult:
        """Creates a URL to start an image builder streaming session.

        :param self:
        :param Validity:  The time that the streaming URL will be valid, in seconds. Specify a value between 1 and 604800 seconds. The default is 3600 seconds.<br/>Name:  The name of the image builder.<br/>
        :return: Expires:  The elapsed time, in seconds after the Unix epoch, when this URL expires.<br/>StreamingURL:  The URL to start the AppStream 2.0 streaming session.<br/>        """
        pass

    def create_stack(self, Name: str, Description: str=None, DisplayName: str=None, StorageConnectors: List[StorageConnector]=None, RedirectURL: str=None, FeedbackURL: str=None, UserSettings: List[UserSetting]=None, ApplicationSettings: ApplicationSettings=None, Tags: Tags=None) -> CreateStackResult:
        """Creates a stack to start streaming applications to users. A stack consists of an associated fleet, user access policies, and storage configurations. 

        :param self:
        :param Tags:  The tags to associate with the stack. A tag is a key-value pair, and the value is optional. For example, Environment=Test. If you do not specify a value, Environment=.  If you do not specify a value, the value is set to an empty string. Generally allowed characters are: letters, numbers, and spaces representable in UTF-8, and the following special characters:  _ . : / = + \ - @ For more information about tags, see <a href="https://docs.aws.amazon.com/appstream2/latest/developerguide/tagging-basic.html">Tagging Your Resources</a> in the <i>Amazon AppStream 2.0 Developer Guide</i>.<br/>ApplicationSettings:  The persistent application settings for users of a stack. When these settings are enabled, changes that users make to applications and Windows settings are automatically saved after each session and applied to the next session.:<br/>UserSettings:  The actions that are enabled or disabled for users during their streaming sessions. By default, these actions are enabled. <br/>FeedbackURL:  The URL that users are redirected to after they click the Send Feedback link. If no URL is specified, no Send Feedback link is displayed.<br/>RedirectURL:  The URL that users are redirected to after their streaming session ends.<br/>StorageConnectors:  The storage connectors to enable.<br/>DisplayName:  The stack name to display.<br/>Description:  The description to display.<br/>Name:  The name of the stack.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SettingsGroup</i> The path prefix for the S3 bucket where users’ persistent application settings are stored. You can allow the same persistent application settings to be used across multiple stacks by specifying the same settings group for each stack. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Enables or disables persistent application settings for users during their streaming sessions. <br/>
        :return: Stack:  Information about the stack.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationSettings</i> The persistent application settings for users of the stack.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ApplicationSettings</b>: The persistent application settings for users of the stack.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserSettings</i> The actions that are enabled or disabled for users during their streaming sessions. By default these actions are enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StackErrors</i> The errors for the stack.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FeedbackURL</i> The URL that users are redirected to after they click the Send Feedback link. If no URL is specified, no Send Feedback link is displayed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RedirectURL</i> The URL that users are redirected to after their streaming session ends.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageConnectors</i> The storage connectors to enable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The time the stack was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayName</i> The stack name to display.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description to display.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the stack.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN of the stack.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3BucketName</i> The S3 bucket where users’ persistent application settings are stored. When persistent application settings are enabled for the first time for an account in an AWS Region, an S3 bucket is created. The bucket is unique to the AWS account and the Region. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SettingsGroup</i> The path prefix for the S3 bucket where users’ persistent application settings are stored.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether persistent application settings are enabled for users during their streaming sessions.<br/>        """
        pass

    def create_streaming_url(self, UserId: str, FleetName: str, StackName: str, ApplicationId: str=None, Validity: Long=None, SessionContext: str=None) -> CreateStreamingURLResult:
        """Creates a temporary URL to start an AppStream 2.0 streaming session for the specified user. A streaming URL enables application streaming to be tested without user setup. 

        :param self:
        :param SessionContext:  The session context. For more information, see <a href="https://docs.aws.amazon.com/appstream2/latest/developerguide/managing-stacks-fleets.html#managing-stacks-fleets-parameters">Session Context</a> in the <i>Amazon AppStream 2.0 Developer Guide</i>.<br/>Validity:  The time that the streaming URL will be valid, in seconds. Specify a value between 1 and 604800 seconds. The default is 60 seconds.<br/>ApplicationId:  The name of the application to launch after the session starts. This is the name that you specified as <b>Name</b> in the Image Assistant.<br/>UserId:  The identifier of the user.<br/>FleetName:  The name of the fleet.<br/>StackName:  The name of the stack.<br/>
        :return: Expires:  The elapsed time, in seconds after the Unix epoch, when this URL expires.<br/>StreamingURL:  The URL to start the AppStream 2.0 streaming session.<br/>        """
        pass

    def create_usage_report_subscription(self) -> CreateUsageReportSubscriptionResult:
        """Creates a usage report subscription. Usage reports are generated daily.

        :param self:
        :param 
        :return: Schedule:  The schedule for generating usage reports.<br/>S3BucketName:  The Amazon S3 bucket where generated reports are stored. If you enabled on-instance session scripts and Amazon S3 logging for your session script configuration, AppStream 2.0 created an S3 bucket to store the script output. The bucket is unique to your account and Region. When you enable usage reporting in this case, AppStream 2.0 uses the same bucket to store your usage reports. If you haven't already enabled on-instance session scripts, when you enable usage reports, AppStream 2.0 creates a new S3 bucket.<br/>        """
        pass

    def create_user(self, AuthenticationType: str, UserName: str, MessageAction: str=None, FirstName: str=None, LastName: str=None) -> CreateUserResult:
        """Creates a new user in the user pool.

        :param self:
        :param AuthenticationType:  The authentication type for the user. You must specify USERPOOL. <br/>LastName:  The last name, or surname, of the user.<br/>FirstName:  The first name, or given name, of the user.<br/>MessageAction:  The action to take for the welcome email that is sent to a user after the user is created in the user pool. If you specify SUPPRESS, no email is sent. If you specify RESEND, do not specify the first name or last name of the user. If the value is null, the email is sent.  <note> The temporary password in the welcome email is valid for only 7 days. If users don’t set their passwords within 7 days, you must send them a new welcome email. </note><br/>UserName:  The email address of the user. <note> Users' email addresses are case-sensitive. During login, if they specify an email address that doesn't use the same capitalization as the email address specified when their user pool account was created, a "user does not exist" error message displays. </note><br/>
        :return:         """
        pass

    def delete_directory_config(self, DirectoryName: str) -> DeleteDirectoryConfigResult:
        """Deletes the specified Directory Config object from AppStream 2.0. This object includes the information required to join streaming instances to an Active Directory domain.

        :param self:
        :param DirectoryName:  The name of the directory configuration.<br/>
        :return:         """
        pass

    def delete_fleet(self, Name: str) -> DeleteFleetResult:
        """Deletes the specified fleet.

        :param self:
        :param Name:  The name of the fleet.<br/>
        :return:         """
        pass

    def delete_image(self, Name: str) -> DeleteImageResult:
        """Deletes the specified image. You cannot delete an image when it is in use. After you delete an image, you cannot provision new capacity using the image.

        :param self:
        :param Name:  The name of the image.<br/>
        :return: Image:  Information about the image.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImagePermissions</i> The permissions to provide to the destination AWS account for the specified image.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ImagePermissions</b>: The permissions to provide to the destination AWS account for the specified image.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AppstreamAgentVersion</i> The version of the AppStream 2.0 agent to use for instances that are launched from this image. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PublicBaseImageReleasedDate</i> The release date of the public base image. For private images, this date is the release date of the base image from which the image was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The time the image was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Applications</i> The applications associated with the image.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StateChangeReason</i> The reason why the last state change occurred.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StateChangeReason</b>: The reason why the last state change occurred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description to display.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The operating system platform of the image.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImageBuilderName</i> The name of the image builder that was used to create the private image. If the image is shared, this value is null.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImageBuilderSupported</i> Indicates whether an image builder can be launched from this image.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Visibility</i> Indicates whether the image is public or private.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The image starts in the <code>PENDING</code> state. If image creation succeeds, the state is <code>AVAILABLE</code>. If image creation fails, the state is <code>FAILED</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayName</i> The image name to display.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BaseImageArn</i> The ARN of the image from which this image was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN of the image.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the image.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> The state change reason message.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Code</i> The state change reason code.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>allowImageBuilder</i> Indicates whether the image can be used for an image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>allowFleet</i> Indicates whether the image can be used for a fleet.<br/>        """
        pass

    def delete_image_builder(self, Name: str) -> DeleteImageBuilderResult:
        """Deletes the specified image builder and releases the capacity.

        :param self:
        :param Name:  The name of the image builder.<br/>
        :return: ImageBuilder:  Information about the image builder.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AppstreamAgentVersion</i> The version of the AppStream 2.0 agent that is currently being used by the image builder. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImageBuilderErrors</i> The image builder errors.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkAccessConfiguration</i> Describes the network details of the fleet or image builder instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NetworkAccessConfiguration</b>: Describes the network details of the fleet or image builder instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainJoinInfo</i> The name of the directory and organizational unit (OU) to use to join the image builder to a Microsoft Active Directory domain. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DomainJoinInfo</b>: The name of the directory and organizational unit (OU) to use to join the image builder to a Microsoft Active Directory domain. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnableDefaultInternetAccess</i> Enables or disables default internet access for the image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The time stamp when the image builder was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StateChangeReason</i> The reason why the last state change occurred.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StateChangeReason</b>: The reason why the last state change occurred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The operating system platform of the image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i> The instance type for the image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcConfig</i> The VPC configuration of the image builder.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VpcConfig</b>: The VPC configuration of the image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayName</i> The image builder name to display.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description to display.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImageArn</i> The ARN of the image from which this builder was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN for the image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The identifiers of the security groups for the fleet or image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> The identifiers of the subnets to which a network interface is attached from the fleet instance or image builder instance. Fleet instances use one or more subnets. Image builder instances use one subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> The state change reason message.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Code</i> The state change reason code.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationalUnitDistinguishedName</i> The distinguished name of the organizational unit for computer accounts.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DirectoryName</i> The fully qualified name of the directory (for example, corp.example.com).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EniId</i> The resource identifier of the elastic network interface that is attached to instances in your VPC. All network interfaces have the eni-xxxxxxxx resource identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EniPrivateIpAddress</i> The private IP address of the elastic network interface that is attached to instances in your VPC.<br/>        """
        pass

    def delete_image_permissions(self, SharedAccountId: str, Name: str) -> DeleteImagePermissionsResult:
        """Deletes permissions for the specified private image. After you delete permissions for an image, AWS accounts to which you previously granted these permissions can no longer use the image.

        :param self:
        :param SharedAccountId:  The 12-digit identifier of the AWS account for which to delete image permissions.<br/>Name:  The name of the private image.<br/>
        :return:         """
        pass

    def delete_stack(self, Name: str) -> DeleteStackResult:
        """Deletes the specified stack. After the stack is deleted, the application streaming environment provided by the stack is no longer available to users. Also, any reservations made for application streaming sessions for the stack are released.

        :param self:
        :param Name:  The name of the stack.<br/>
        :return:         """
        pass

    def delete_usage_report_subscription(self) -> DeleteUsageReportSubscriptionResult:
        """Disables usage report generation.

        :param self:
        :param 
        :return:         """
        pass

    def delete_user(self, AuthenticationType: str, UserName: str) -> DeleteUserResult:
        """Deletes a user from the user pool.

        :param self:
        :param AuthenticationType:  The authentication type for the user. You must specify USERPOOL.<br/>UserName:  The email address of the user. <note> Users' email addresses are case-sensitive. </note><br/>
        :return:         """
        pass

    def describe_directory_configs(self, DirectoryNames: List[DirectoryName]=None, MaxResults: int=None, NextToken: str=None) -> DescribeDirectoryConfigsResult:
        """Retrieves a list that describes one or more specified Directory Config objects for AppStream 2.0, if the names for these objects are provided. Otherwise, all Directory Config objects in the account are described. These objects include the configuration information required to join fleets and image builders to Microsoft Active Directory domains.  Although the response syntax in this topic includes the account password, this password is not returned in the actual response.

        :param self:
        :param NextToken:  The pagination token to use to retrieve the next page of results for this operation. If this value is null, it retrieves the first page.<br/>MaxResults:  The maximum size of each page of results.<br/>DirectoryNames:  The directory names.<br/>
        :return: NextToken:  The pagination token to use to retrieve the next page of results for this operation. If there are no more pages, this value is null.<br/>DirectoryConfigs:  Information about the directory configurations. Note that although the response syntax in this topic includes the account password, this password is not returned in the actual response. <br/>        """
        pass

    def describe_fleets(self, Names: List[String]=None, NextToken: str=None) -> DescribeFleetsResult:
        """Retrieves a list that describes one or more specified fleets, if the fleet names are provided. Otherwise, all fleets in the account are described.

        :param self:
        :param NextToken:  The pagination token to use to retrieve the next page of results for this operation. If this value is null, it retrieves the first page.<br/>Names:  The names of the fleets to describe.<br/>
        :return: NextToken:  The pagination token to use to retrieve the next page of results for this operation. If there are no more pages, this value is null.<br/>Fleets:  Information about the fleets.<br/>        """
        pass

    def describe_image_builders(self, Names: List[String]=None, MaxResults: int=None, NextToken: str=None) -> DescribeImageBuildersResult:
        """Retrieves a list that describes one or more specified image builders, if the image builder names are provided. Otherwise, all image builders in the account are described.

        :param self:
        :param NextToken:  The pagination token to use to retrieve the next page of results for this operation. If this value is null, it retrieves the first page.<br/>MaxResults:  The maximum size of each page of results.<br/>Names:  The names of the image builders to describe.<br/>
        :return: NextToken:  The pagination token to use to retrieve the next page of results for this operation. If there are no more pages, this value is null.<br/>ImageBuilders:  Information about the image builders.<br/>        """
        pass

    def describe_image_permissions(self, Name: str, MaxResults: int=None, SharedAwsAccountIds: List[AwsAccountId]=None, NextToken: str=None) -> DescribeImagePermissionsResult:
        """Retrieves a list that describes the permissions for shared AWS account IDs on a private image that you own. 

        :param self:
        :param NextToken:  The pagination token to use to retrieve the next page of results for this operation. If this value is null, it retrieves the first page.<br/>SharedAwsAccountIds:  The 12-digit identifier of one or more AWS accounts with which the image is shared.<br/>MaxResults:  The maximum size of each page of results.<br/>Name:  The name of the private image for which to describe permissions. The image must be one that you own. <br/>
        :return: NextToken:  The pagination token to use to retrieve the next page of results for this operation. If there are no more pages, this value is null.<br/>SharedImagePermissionsList:  The permissions for a private image that you own. <br/>Name:  The name of the private image.<br/>        """
        pass

    def describe_images(self, Names: List[String]=None, Arns: List[Arn]=None, Type: str=None, NextToken: str=None, MaxResults: int=None) -> DescribeImagesResult:
        """Retrieves a list that describes one or more specified images, if the image names or image ARNs are provided. Otherwise, all images in the account are described.

        :param self:
        :param MaxResults:  The maximum size of each page of results.<br/>NextToken:  The pagination token to use to retrieve the next page of results for this operation. If this value is null, it retrieves the first page.<br/>Type:  The type of image (public, private, or shared) to describe. <br/>Arns:  The ARNs of the public, private, and shared images to describe.<br/>Names:  The names of the public or private images to describe.<br/>
        :return: NextToken:  The pagination token to use to retrieve the next page of results for this operation. If there are no more pages, this value is null.<br/>Images:  Information about the images.<br/>        """
        pass

    def describe_sessions(self, FleetName: str, StackName: str, UserId: str=None, NextToken: str=None, Limit: int=None, AuthenticationType: str=None) -> DescribeSessionsResult:
        """Retrieves a list that describes the streaming sessions for a specified stack and fleet. If a UserId is provided for the stack and fleet, only streaming sessions for that user are described. If an authentication type is not provided, the default is to authenticate users using a streaming URL.

        :param self:
        :param AuthenticationType:  The authentication method. Specify <code>API</code> for a user authenticated using a streaming URL or <code>SAML</code> for a SAML federated user. The default is to authenticate users using a streaming URL.<br/>Limit:  The size of each page of results. The default value is 20 and the maximum value is 50.<br/>NextToken:  The pagination token to use to retrieve the next page of results for this operation. If this value is null, it retrieves the first page.<br/>UserId:  The user identifier.<br/>FleetName:  The name of the fleet. This value is case-sensitive.<br/>StackName:  The name of the stack. This value is case-sensitive.<br/>
        :return: NextToken:  The pagination token to use to retrieve the next page of results for this operation. If there are no more pages, this value is null.<br/>Sessions:  Information about the streaming sessions.<br/>        """
        pass

    def describe_stacks(self, Names: List[String]=None, NextToken: str=None) -> DescribeStacksResult:
        """Retrieves a list that describes one or more specified stacks, if the stack names are provided. Otherwise, all stacks in the account are described.

        :param self:
        :param NextToken:  The pagination token to use to retrieve the next page of results for this operation. If this value is null, it retrieves the first page.<br/>Names:  The names of the stacks to describe.<br/>
        :return: NextToken:  The pagination token to use to retrieve the next page of results for this operation. If there are no more pages, this value is null.<br/>Stacks:  Information about the stacks.<br/>        """
        pass

    def describe_usage_report_subscriptions(self, MaxResults: int=None, NextToken: str=None) -> DescribeUsageReportSubscriptionsResult:
        """Retrieves a list that describes one or more usage report subscriptions.

        :param self:
        :param NextToken:  The pagination token to use to retrieve the next page of results for this operation. If this value is null, it retrieves the first page.<br/>MaxResults:  The maximum size of each page of results.<br/>
        :return: NextToken:  The pagination token to use to retrieve the next page of results for this operation. If there are no more pages, this value is null.<br/>UsageReportSubscriptions:  Information about the usage report subscription.<br/>        """
        pass

    def describe_user_stack_associations(self, StackName: str=None, UserName: str=None, AuthenticationType: str=None, MaxResults: int=None, NextToken: str=None) -> DescribeUserStackAssociationsResult:
        """Retrieves a list that describes the UserStackAssociation objects. You must specify either or both of the following: <ul> <li> The stack name </li> <li> The user name (email address of the user associated with the stack) and the authentication type for the user </li> </ul>

        :param self:
        :param NextToken:  The pagination token to use to retrieve the next page of results for this operation. If this value is null, it retrieves the first page.<br/>MaxResults:  The maximum size of each page of results.<br/>AuthenticationType:  The authentication type for the user who is associated with the stack. You must specify USERPOOL.<br/>UserName:  The email address of the user who is associated with the stack. <note> Users' email addresses are case-sensitive. </note><br/>StackName:  The name of the stack that is associated with the user.<br/>
        :return: NextToken:  The pagination token to use to retrieve the next page of results for this operation. If there are no more pages, this value is null.<br/>UserStackAssociations:  The UserStackAssociation objects.<br/>        """
        pass

    def describe_users(self, AuthenticationType: str, MaxResults: int=None, NextToken: str=None) -> DescribeUsersResult:
        """Retrieves a list that describes one or more specified users in the user pool.

        :param self:
        :param NextToken:  The pagination token to use to retrieve the next page of results for this operation. If this value is null, it retrieves the first page.<br/>MaxResults:  The maximum size of each page of results.<br/>AuthenticationType:  The authentication type for the users in the user pool to describe. You must specify USERPOOL.<br/>
        :return: NextToken:  The pagination token to use to retrieve the next page of results for this operation. If there are no more pages, this value is null.<br/>Users:  Information about users in the user pool.<br/>        """
        pass

    def disable_user(self, AuthenticationType: str, UserName: str) -> DisableUserResult:
        """Disables the specified user in the user pool. Users can't sign in to AppStream 2.0 until they are re-enabled. This action does not delete the user. 

        :param self:
        :param AuthenticationType:  The authentication type for the user. You must specify USERPOOL.<br/>UserName:  The email address of the user. <note> Users' email addresses are case-sensitive. </note><br/>
        :return:         """
        pass

    def disassociate_fleet(self, StackName: str, FleetName: str) -> DisassociateFleetResult:
        """Disassociates the specified fleet from the specified stack.

        :param self:
        :param StackName:  The name of the stack.<br/>FleetName:  The name of the fleet.<br/>
        :return:         """
        pass

    def enable_user(self, AuthenticationType: str, UserName: str) -> EnableUserResult:
        """Enables a user in the user pool. After being enabled, users can sign in to AppStream 2.0 and open applications from the stacks to which they are assigned.

        :param self:
        :param AuthenticationType:  The authentication type for the user. You must specify USERPOOL.<br/>UserName:  The email address of the user. <note> Users' email addresses are case-sensitive. During login, if they specify an email address that doesn't use the same capitalization as the email address specified when their user pool account was created, a "user does not exist" error message displays.  </note><br/>
        :return:         """
        pass

    def expire_session(self, SessionId: str) -> ExpireSessionResult:
        """Immediately stops the specified streaming session.

        :param self:
        :param SessionId:  The identifier of the streaming session.<br/>
        :return:         """
        pass

    def list_associated_fleets(self, StackName: str, NextToken: str=None) -> ListAssociatedFleetsResult:
        """Retrieves the name of the fleet that is associated with the specified stack.

        :param self:
        :param NextToken:  The pagination token to use to retrieve the next page of results for this operation. If this value is null, it retrieves the first page.<br/>StackName:  The name of the stack.<br/>
        :return: NextToken:  The pagination token to use to retrieve the next page of results for this operation. If there are no more pages, this value is null.<br/>Names:  The name of the fleet.<br/>        """
        pass

    def list_associated_stacks(self, FleetName: str, NextToken: str=None) -> ListAssociatedStacksResult:
        """Retrieves the name of the stack with which the specified fleet is associated.

        :param self:
        :param NextToken:  The pagination token to use to retrieve the next page of results for this operation. If this value is null, it retrieves the first page.<br/>FleetName:  The name of the fleet.<br/>
        :return: NextToken:  The pagination token to use to retrieve the next page of results for this operation. If there are no more pages, this value is null.<br/>Names:  The name of the stack.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceArn: str) -> ListTagsForResourceResponse:
        """Retrieves a list of all tags for the specified AppStream 2.0 resource. You can tag AppStream 2.0 image builders, images, fleets, and stacks. For more information about tags, see <a href="https://docs.aws.amazon.com/appstream2/latest/developerguide/tagging-basic.html">Tagging Your Resources</a> in the <i>Amazon AppStream 2.0 Developer Guide</i>.

        :param self:
        :param ResourceArn:  The Amazon Resource Name (ARN) of the resource.<br/>
        :return: Tags:  The information about the tags.<br/>        """
        pass

    def start_fleet(self, Name: str) -> StartFleetResult:
        """Starts the specified fleet.

        :param self:
        :param Name:  The name of the fleet.<br/>
        :return:         """
        pass

    def start_image_builder(self, Name: str, AppstreamAgentVersion: str=None) -> StartImageBuilderResult:
        """Starts the specified image builder.

        :param self:
        :param AppstreamAgentVersion:  The version of the AppStream 2.0 agent to use for this image builder. To use the latest version of the AppStream 2.0 agent, specify [LATEST]. <br/>Name:  The name of the image builder.<br/>
        :return: ImageBuilder:  Information about the image builder.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AppstreamAgentVersion</i> The version of the AppStream 2.0 agent that is currently being used by the image builder. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImageBuilderErrors</i> The image builder errors.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkAccessConfiguration</i> Describes the network details of the fleet or image builder instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NetworkAccessConfiguration</b>: Describes the network details of the fleet or image builder instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainJoinInfo</i> The name of the directory and organizational unit (OU) to use to join the image builder to a Microsoft Active Directory domain. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DomainJoinInfo</b>: The name of the directory and organizational unit (OU) to use to join the image builder to a Microsoft Active Directory domain. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnableDefaultInternetAccess</i> Enables or disables default internet access for the image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The time stamp when the image builder was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StateChangeReason</i> The reason why the last state change occurred.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StateChangeReason</b>: The reason why the last state change occurred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The operating system platform of the image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i> The instance type for the image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcConfig</i> The VPC configuration of the image builder.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VpcConfig</b>: The VPC configuration of the image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayName</i> The image builder name to display.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description to display.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImageArn</i> The ARN of the image from which this builder was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN for the image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The identifiers of the security groups for the fleet or image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> The identifiers of the subnets to which a network interface is attached from the fleet instance or image builder instance. Fleet instances use one or more subnets. Image builder instances use one subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> The state change reason message.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Code</i> The state change reason code.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationalUnitDistinguishedName</i> The distinguished name of the organizational unit for computer accounts.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DirectoryName</i> The fully qualified name of the directory (for example, corp.example.com).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EniId</i> The resource identifier of the elastic network interface that is attached to instances in your VPC. All network interfaces have the eni-xxxxxxxx resource identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EniPrivateIpAddress</i> The private IP address of the elastic network interface that is attached to instances in your VPC.<br/>        """
        pass

    def stop_fleet(self, Name: str) -> StopFleetResult:
        """Stops the specified fleet.

        :param self:
        :param Name:  The name of the fleet.<br/>
        :return:         """
        pass

    def stop_image_builder(self, Name: str) -> StopImageBuilderResult:
        """Stops the specified image builder.

        :param self:
        :param Name:  The name of the image builder.<br/>
        :return: ImageBuilder:  Information about the image builder.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AppstreamAgentVersion</i> The version of the AppStream 2.0 agent that is currently being used by the image builder. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImageBuilderErrors</i> The image builder errors.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkAccessConfiguration</i> Describes the network details of the fleet or image builder instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NetworkAccessConfiguration</b>: Describes the network details of the fleet or image builder instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainJoinInfo</i> The name of the directory and organizational unit (OU) to use to join the image builder to a Microsoft Active Directory domain. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DomainJoinInfo</b>: The name of the directory and organizational unit (OU) to use to join the image builder to a Microsoft Active Directory domain. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnableDefaultInternetAccess</i> Enables or disables default internet access for the image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The time stamp when the image builder was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StateChangeReason</i> The reason why the last state change occurred.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StateChangeReason</b>: The reason why the last state change occurred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The operating system platform of the image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i> The instance type for the image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcConfig</i> The VPC configuration of the image builder.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VpcConfig</b>: The VPC configuration of the image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayName</i> The image builder name to display.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description to display.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImageArn</i> The ARN of the image from which this builder was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN for the image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The identifiers of the security groups for the fleet or image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> The identifiers of the subnets to which a network interface is attached from the fleet instance or image builder instance. Fleet instances use one or more subnets. Image builder instances use one subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> The state change reason message.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Code</i> The state change reason code.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationalUnitDistinguishedName</i> The distinguished name of the organizational unit for computer accounts.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DirectoryName</i> The fully qualified name of the directory (for example, corp.example.com).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EniId</i> The resource identifier of the elastic network interface that is attached to instances in your VPC. All network interfaces have the eni-xxxxxxxx resource identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EniPrivateIpAddress</i> The private IP address of the elastic network interface that is attached to instances in your VPC.<br/>        """
        pass

    def tag_resource(self, Tags: Tags, ResourceArn: str) -> TagResourceResponse:
        """Adds or overwrites one or more tags for the specified AppStream 2.0 resource. You can tag AppStream 2.0 image builders, images, fleets, and stacks. Each tag consists of a key and an optional value. If a resource already has a tag with the same key, this operation updates its value. To list the current tags for your resources, use <a>ListTagsForResource</a>. To disassociate tags from your resources, use <a>UntagResource</a>. For more information about tags, see <a href="https://docs.aws.amazon.com/appstream2/latest/developerguide/tagging-basic.html">Tagging Your Resources</a> in the <i>Amazon AppStream 2.0 Developer Guide</i>.

        :param self:
        :param Tags:  The tags to associate. A tag is a key-value pair, and the value is optional. For example, Environment=Test. If you do not specify a value, Environment=.  If you do not specify a value, the value is set to an empty string. Generally allowed characters are: letters, numbers, and spaces representable in UTF-8, and the following special characters:  _ . : / = + \ - @<br/>ResourceArn:  The Amazon Resource Name (ARN) of the resource.<br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[TagKey], ResourceArn: str) -> UntagResourceResponse:
        """Disassociates one or more specified tags from the specified AppStream 2.0 resource. To list the current tags for your resources, use <a>ListTagsForResource</a>. For more information about tags, see <a href="https://docs.aws.amazon.com/appstream2/latest/developerguide/tagging-basic.html">Tagging Your Resources</a> in the <i>Amazon AppStream 2.0 Developer Guide</i>.

        :param self:
        :param TagKeys:  The tag keys for the tags to disassociate.<br/>ResourceArn:  The Amazon Resource Name (ARN) of the resource.<br/>
        :return:         """
        pass

    def update_directory_config(self, DirectoryName: str, OrganizationalUnitDistinguishedNames: List[OrganizationalUnitDistinguishedName]=None, ServiceAccountCredentials: ServiceAccountCredentials=None) -> UpdateDirectoryConfigResult:
        """Updates the specified Directory Config object in AppStream 2.0. This object includes the configuration information required to join fleets and image builders to Microsoft Active Directory domains.

        :param self:
        :param ServiceAccountCredentials:  The credentials for the service account used by the fleet or image builder to connect to the directory.:<br/>OrganizationalUnitDistinguishedNames:  The distinguished names of the organizational units for computer accounts.<br/>DirectoryName:  The name of the Directory Config object.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountPassword</i> The password for the account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountName</i> The user name of the account. This account must have the following privileges: create computer objects, join computers to the domain, and change/reset the password on descendant computer objects for the organizational units specified.<br/>
        :return: DirectoryConfig:  Information about the Directory Config object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The time the directory configuration was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccountCredentials</i> The credentials for the service account used by the fleet or image builder to connect to the directory.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ServiceAccountCredentials</b>: The credentials for the service account used by the fleet or image builder to connect to the directory.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationalUnitDistinguishedNames</i> The distinguished names of the organizational units for computer accounts.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DirectoryName</i> The fully qualified name of the directory (for example, corp.example.com).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountPassword</i> The password for the account.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountName</i> The user name of the account. This account must have the following privileges: create computer objects, join computers to the domain, and change/reset the password on descendant computer objects for the organizational units specified.<br/>        """
        pass

    def update_fleet(self, ImageName: str=None, ImageArn: str=None, Name: str=None, InstanceType: str=None, ComputeCapacity: ComputeCapacity=None, VpcConfig: VpcConfig=None, MaxUserDurationInSeconds: int=None, DisconnectTimeoutInSeconds: int=None, DeleteVpcConfig: Boolean=None, Description: str=None, DisplayName: str=None, EnableDefaultInternetAccess: BooleanObject=None, DomainJoinInfo: DomainJoinInfo=None, IdleDisconnectTimeoutInSeconds: int=None, AttributesToDelete: List[FleetAttribute]=None) -> UpdateFleetResult:
        """Updates the specified fleet. If the fleet is in the <code>STOPPED</code> state, you can update any attribute except the fleet name. If the fleet is in the <code>RUNNING</code> state, you can update the <code>DisplayName</code>, <code>ComputeCapacity</code>, <code>ImageARN</code>, <code>ImageName</code>, and <code>DisconnectTimeoutInSeconds</code> attributes. If the fleet is in the <code>STARTING</code> or <code>STOPPING</code> state, you can't update it.

        :param self:
        :param AttributesToDelete:  The fleet attributes to delete.<br/>IdleDisconnectTimeoutInSeconds:  The amount of time that users can be idle (inactive) before they are disconnected from their streaming session and the <code>DisconnectTimeoutInSeconds</code> time interval begins. Users are notified before they are disconnected due to inactivity. If users try to reconnect to the streaming session before the time interval specified in <code>DisconnectTimeoutInSeconds</code> elapses, they are connected to their previous session. Users are considered idle when they stop providing keyboard or mouse input during their streaming session. File uploads and downloads, audio in, audio out, and pixels changing do not qualify as user activity. If users continue to be idle after the time interval in <code>IdleDisconnectTimeoutInSeconds</code> elapses, they are disconnected.  To prevent users from being disconnected due to inactivity, specify a value of 0. Otherwise, specify a value between 60 and 3600. The default value is 0. <note> If you enable this feature, we recommend that you specify a value that corresponds exactly to a whole number of minutes (for example, 60, 120, and 180). If you don't do this, the value is rounded to the nearest minute. For example, if you specify a value of 70, users are disconnected after 1 minute of inactivity. If you specify a value that is at the midpoint between two different minutes, the value is rounded up. For example, if you specify a value of 90, users are disconnected after 2 minutes of inactivity.  </note><br/>DomainJoinInfo:  The name of the directory and organizational unit (OU) to use to join the fleet to a Microsoft Active Directory domain. :<br/>EnableDefaultInternetAccess:  Enables or disables default internet access for the fleet.<br/>DisplayName:  The fleet name to display.<br/>Description:  The description to display.<br/>DeleteVpcConfig:  Deletes the VPC association for the specified fleet.<br/>DisconnectTimeoutInSeconds:  The amount of time that a streaming session remains active after users disconnect. If users try to reconnect to the streaming session after a disconnection or network interruption within this time interval, they are connected to their previous session. Otherwise, they are connected to a new session with a new streaming instance.  Specify a value between 60 and 360000.<br/>MaxUserDurationInSeconds:  The maximum amount of time that a streaming session can remain active, in seconds. If users are still connected to a streaming instance five minutes before this limit is reached, they are prompted to save any open documents before being disconnected. After this time elapses, the instance is terminated and replaced by a new instance. Specify a value between 600 and 360000.<br/>VpcConfig:  The VPC configuration for the fleet.:<br/>ComputeCapacity:  The desired capacity for the fleet.:<br/>InstanceType:  The instance type to use when launching fleet instances. The following instance types are available: <ul> <li> stream.standard.medium </li> <li> stream.standard.large </li> <li> stream.compute.large </li> <li> stream.compute.xlarge </li> <li> stream.compute.2xlarge </li> <li> stream.compute.4xlarge </li> <li> stream.compute.8xlarge </li> <li> stream.memory.large </li> <li> stream.memory.xlarge </li> <li> stream.memory.2xlarge </li> <li> stream.memory.4xlarge </li> <li> stream.memory.8xlarge </li> <li> stream.graphics-design.large </li> <li> stream.graphics-design.xlarge </li> <li> stream.graphics-design.2xlarge </li> <li> stream.graphics-design.4xlarge </li> <li> stream.graphics-desktop.2xlarge </li> <li> stream.graphics-pro.4xlarge </li> <li> stream.graphics-pro.8xlarge </li> <li> stream.graphics-pro.16xlarge </li> </ul><br/>Name:  A unique name for the fleet.<br/>ImageArn:  The ARN of the public, private, or shared image to use.<br/>ImageName:  The name of the image used to create the fleet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DesiredInstances</i> The desired number of streaming instances.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The identifiers of the security groups for the fleet or image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> The identifiers of the subnets to which a network interface is attached from the fleet instance or image builder instance. Fleet instances use one or more subnets. Image builder instances use one subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationalUnitDistinguishedName</i> The distinguished name of the organizational unit for computer accounts.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DirectoryName</i> The fully qualified name of the directory (for example, corp.example.com).<br/>
        :return: Fleet:  Information about the fleet.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IdleDisconnectTimeoutInSeconds</i> The amount of time that users can be idle (inactive) before they are disconnected from their streaming session and the <code>DisconnectTimeoutInSeconds</code> time interval begins. Users are notified before they are disconnected due to inactivity. If users try to reconnect to the streaming session before the time interval specified in <code>DisconnectTimeoutInSeconds</code> elapses, they are connected to their previous session. Users are considered idle when they stop providing keyboard or mouse input during their streaming session. File uploads and downloads, audio in, audio out, and pixels changing do not qualify as user activity. If users continue to be idle after the time interval in <code>IdleDisconnectTimeoutInSeconds</code> elapses, they are disconnected. To prevent users from being disconnected due to inactivity, specify a value of 0. Otherwise, specify a value between 60 and 3600. The default value is 0. <note> If you enable this feature, we recommend that you specify a value that corresponds exactly to a whole number of minutes (for example, 60, 120, and 180). If you don't do this, the value is rounded to the nearest minute. For example, if you specify a value of 70, users are disconnected after 1 minute of inactivity. If you specify a value that is at the midpoint between two different minutes, the value is rounded up. For example, if you specify a value of 90, users are disconnected after 2 minutes of inactivity.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainJoinInfo</i> The name of the directory and organizational unit (OU) to use to join the fleet to a Microsoft Active Directory domain. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DomainJoinInfo</b>: The name of the directory and organizational unit (OU) to use to join the fleet to a Microsoft Active Directory domain. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnableDefaultInternetAccess</i> Indicates whether default internet access is enabled for the fleet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FleetErrors</i> The fleet errors.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The time the fleet was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcConfig</i> The VPC configuration for the fleet.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VpcConfig</b>: The VPC configuration for the fleet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The current state for the fleet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisconnectTimeoutInSeconds</i> The amount of time that a streaming session remains active after users disconnect. If they try to reconnect to the streaming session after a disconnection or network interruption within this time interval, they are connected to their previous session. Otherwise, they are connected to a new session with a new streaming instance. Specify a value between 60 and 360000.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxUserDurationInSeconds</i> The maximum amount of time that a streaming session can remain active, in seconds. If users are still connected to a streaming instance five minutes before this limit is reached, they are prompted to save any open documents before being disconnected. After this time elapses, the instance is terminated and replaced by a new instance.  Specify a value between 600 and 360000.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ComputeCapacityStatus</i> The capacity status for the fleet.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ComputeCapacityStatus</b>: The capacity status for the fleet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FleetType</i> The fleet type. <dl> <dt>ALWAYS_ON</dt> <dd> Provides users with instant-on access to their apps. You are charged for all running instances in your fleet, even if no users are streaming apps. </dd> <dt>ON_DEMAND</dt> <dd> Provide users with access to applications after they connect, which takes one to two minutes. You are charged for instance streaming when users are connected and a small hourly fee for instances that are not streaming apps. </dd> </dl><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i> The instance type to use when launching fleet instances.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImageArn</i> The ARN for the public, private, or shared image.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImageName</i> The name of the image used to create the fleet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description to display.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayName</i> The fleet name to display.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the fleet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN for the fleet.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Available</i> The number of currently available instances that can be used to stream sessions.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InUse</i> The number of instances in use for streaming.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Running</i> The total number of simultaneous streaming instances that are running.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Desired</i> The desired number of streaming instances.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The identifiers of the security groups for the fleet or image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> The identifiers of the subnets to which a network interface is attached from the fleet instance or image builder instance. Fleet instances use one or more subnets. Image builder instances use one subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationalUnitDistinguishedName</i> The distinguished name of the organizational unit for computer accounts.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DirectoryName</i> The fully qualified name of the directory (for example, corp.example.com).<br/>        """
        pass

    def update_image_permissions(self, ImagePermissions: ImagePermissions, SharedAccountId: str, Name: str) -> UpdateImagePermissionsResult:
        """Adds or updates permissions for the specified private image. 

        :param self:
        :param ImagePermissions:  The permissions for the image.:<br/>SharedAccountId:  The 12-digit identifier of the AWS account for which you want add or update image permissions.<br/>Name:  The name of the private image.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>allowImageBuilder</i> Indicates whether the image can be used for an image builder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>allowFleet</i> Indicates whether the image can be used for a fleet.<br/>
        :return:         """
        pass

    def update_stack(self, Name: str, DisplayName: str=None, Description: str=None, StorageConnectors: List[StorageConnector]=None, DeleteStorageConnectors: Boolean=None, RedirectURL: str=None, FeedbackURL: str=None, AttributesToDelete: List[StackAttribute]=None, UserSettings: List[UserSetting]=None, ApplicationSettings: ApplicationSettings=None) -> UpdateStackResult:
        """Updates the specified fields for the specified stack.

        :param self:
        :param ApplicationSettings:  The persistent application settings for users of a stack. When these settings are enabled, changes that users make to applications and Windows settings are automatically saved after each session and applied to the next session.:<br/>UserSettings:  The actions that are enabled or disabled for users during their streaming sessions. By default, these actions are enabled.<br/>AttributesToDelete:  The stack attributes to delete.<br/>FeedbackURL:  The URL that users are redirected to after they choose the Send Feedback link. If no URL is specified, no Send Feedback link is displayed.<br/>RedirectURL:  The URL that users are redirected to after their streaming session ends.<br/>DeleteStorageConnectors:  Deletes the storage connectors currently enabled for the stack.<br/>StorageConnectors:  The storage connectors to enable.<br/>Name:  The name of the stack.<br/>Description:  The description to display.<br/>DisplayName:  The stack name to display.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SettingsGroup</i> The path prefix for the S3 bucket where users’ persistent application settings are stored. You can allow the same persistent application settings to be used across multiple stacks by specifying the same settings group for each stack. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Enables or disables persistent application settings for users during their streaming sessions. <br/>
        :return: Stack:  Information about the stack.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationSettings</i> The persistent application settings for users of the stack.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ApplicationSettings</b>: The persistent application settings for users of the stack.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserSettings</i> The actions that are enabled or disabled for users during their streaming sessions. By default these actions are enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StackErrors</i> The errors for the stack.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FeedbackURL</i> The URL that users are redirected to after they click the Send Feedback link. If no URL is specified, no Send Feedback link is displayed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RedirectURL</i> The URL that users are redirected to after their streaming session ends.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageConnectors</i> The storage connectors to enable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The time the stack was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayName</i> The stack name to display.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description to display.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the stack.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN of the stack.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3BucketName</i> The S3 bucket where users’ persistent application settings are stored. When persistent application settings are enabled for the first time for an account in an AWS Region, an S3 bucket is created. The bucket is unique to the AWS account and the Region. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SettingsGroup</i> The path prefix for the S3 bucket where users’ persistent application settings are stored.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether persistent application settings are enabled for users during their streaming sessions.<br/>        """
        pass



class AppSync(BaseClient):
    """AWS AppSync provides API actions for creating and interacting with data sources using GraphQL from your application."""


    class ListApiKeysPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_api_keys` for valid parameters"""
            pass
    class ListDataSourcesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_data_sources` for valid parameters"""
            pass
    class ListFunctionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_functions` for valid parameters"""
            pass
    class ListGraphqlApisPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_graphql_apis` for valid parameters"""
            pass
    class ListResolversPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_resolvers` for valid parameters"""
            pass
    class ListResolversByFunctionPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_resolvers_by_function` for valid parameters"""
            pass
    class ListTypesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_types` for valid parameters"""
            pass


    class CreateApiKeyRequest(Mapping):
        pass
    
    class CreateApiKeyResponse(Mapping):
        pass
    
    class Long(object):
        pass
    
    class CreateDataSourceRequest(Mapping):
        pass
    
    class CreateDataSourceResponse(Mapping):
        pass
    
    class DynamodbDataSourceConfig(Mapping):
        pass
    
    class LambdaDataSourceConfig(Mapping):
        pass
    
    class ElasticsearchDataSourceConfig(Mapping):
        pass
    
    class HttpDataSourceConfig(Mapping):
        pass
    
    class RelationalDatabaseDataSourceConfig(Mapping):
        pass
    
    class CreateFunctionRequest(Mapping):
        pass
    
    class CreateFunctionResponse(Mapping):
        pass
    
    class CreateGraphqlApiRequest(Mapping):
        pass
    
    class CreateGraphqlApiResponse(Mapping):
        pass
    
    class LogConfig(Mapping):
        pass
    
    class UserPoolConfig(Mapping):
        pass
    
    class OpenIDConnectConfig(Mapping):
        pass
    
    class TagMap(object):
        pass
    
    class AdditionalAuthenticationProvider(Mapping):
        pass
    
    class CreateResolverRequest(Mapping):
        pass
    
    class CreateResolverResponse(Mapping):
        pass
    
    class PipelineConfig(Mapping):
        pass
    
    class CreateTypeRequest(Mapping):
        pass
    
    class CreateTypeResponse(Mapping):
        pass
    
    class DeleteApiKeyRequest(Mapping):
        pass
    
    class DeleteApiKeyResponse(Mapping):
        pass
    
    class DeleteDataSourceRequest(Mapping):
        pass
    
    class DeleteDataSourceResponse(Mapping):
        pass
    
    class DeleteFunctionRequest(Mapping):
        pass
    
    class DeleteFunctionResponse(Mapping):
        pass
    
    class DeleteGraphqlApiRequest(Mapping):
        pass
    
    class DeleteGraphqlApiResponse(Mapping):
        pass
    
    class DeleteResolverRequest(Mapping):
        pass
    
    class DeleteResolverResponse(Mapping):
        pass
    
    class DeleteTypeRequest(Mapping):
        pass
    
    class DeleteTypeResponse(Mapping):
        pass
    
    class GetDataSourceRequest(Mapping):
        pass
    
    class GetDataSourceResponse(Mapping):
        pass
    
    class GetFunctionRequest(Mapping):
        pass
    
    class GetFunctionResponse(Mapping):
        pass
    
    class GetGraphqlApiRequest(Mapping):
        pass
    
    class GetGraphqlApiResponse(Mapping):
        pass
    
    class GetIntrospectionSchemaRequest(Mapping):
        pass
    
    class GetIntrospectionSchemaResponse(Mapping):
        pass
    
    class BooleanValue(object):
        pass
    
    class GetResolverRequest(Mapping):
        pass
    
    class GetResolverResponse(Mapping):
        pass
    
    class GetSchemaCreationStatusRequest(Mapping):
        pass
    
    class GetSchemaCreationStatusResponse(Mapping):
        pass
    
    class GetTypeRequest(Mapping):
        pass
    
    class GetTypeResponse(Mapping):
        pass
    
    class ListApiKeysRequest(Mapping):
        pass
    
    class ListApiKeysResponse(Mapping):
        pass
    
    class ListDataSourcesRequest(Mapping):
        pass
    
    class ListDataSourcesResponse(Mapping):
        pass
    
    class ListFunctionsRequest(Mapping):
        pass
    
    class ListFunctionsResponse(Mapping):
        pass
    
    class ListGraphqlApisRequest(Mapping):
        pass
    
    class ListGraphqlApisResponse(Mapping):
        pass
    
    class ListResolversRequest(Mapping):
        pass
    
    class ListResolversResponse(Mapping):
        pass
    
    class ListResolversByFunctionRequest(Mapping):
        pass
    
    class ListResolversByFunctionResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class ListTypesRequest(Mapping):
        pass
    
    class ListTypesResponse(Mapping):
        pass
    
    class StartSchemaCreationRequest(Mapping):
        pass
    
    class StartSchemaCreationResponse(Mapping):
        pass
    
    class Blob(object):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateApiKeyRequest(Mapping):
        pass
    
    class UpdateApiKeyResponse(Mapping):
        pass
    
    class UpdateDataSourceRequest(Mapping):
        pass
    
    class UpdateDataSourceResponse(Mapping):
        pass
    
    class UpdateFunctionRequest(Mapping):
        pass
    
    class UpdateFunctionResponse(Mapping):
        pass
    
    class UpdateGraphqlApiRequest(Mapping):
        pass
    
    class UpdateGraphqlApiResponse(Mapping):
        pass
    
    class UpdateResolverRequest(Mapping):
        pass
    
    class UpdateResolverResponse(Mapping):
        pass
    
    class UpdateTypeRequest(Mapping):
        pass
    
    class UpdateTypeResponse(Mapping):
        pass
    
    def create_api_key(self, apiId: str, description: str=None, expires: Long=None) -> CreateApiKeyResponse:
        """Creates a unique key that you can distribute to clients who are executing your API.

        :param self:
        :param expires:  The time from creation time after which the API key expires. The date is represented as seconds since the epoch, rounded down to the nearest hour. The default value for this parameter is 7 days from creation time. For more information, see .<br/>description:  A description of the purpose of the API key.<br/>apiId:  The ID for your GraphQL API.<br/>
        :return: apiKey:  The API key.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>expires</i> The time after which the API key expires. The date is represented as seconds since the epoch, rounded down to the nearest hour.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> A description of the purpose of the API key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The API key ID.<br/>        """
        pass

    def create_data_source(self, type: str, name: str, apiId: str, description: str=None, serviceRoleArn: str=None, dynamodbConfig: DynamodbDataSourceConfig=None, lambdaConfig: LambdaDataSourceConfig=None, elasticsearchConfig: ElasticsearchDataSourceConfig=None, httpConfig: HttpDataSourceConfig=None, relationalDatabaseConfig: RelationalDatabaseDataSourceConfig=None) -> CreateDataSourceResponse:
        """Creates a <code>DataSource</code> object.

        :param self:
        :param relationalDatabaseConfig:  Relational database settings.:<br/>httpConfig:  HTTP endpoint settings.:<br/>elasticsearchConfig:  Amazon Elasticsearch Service settings.:<br/>lambdaConfig:  AWS Lambda settings.:<br/>dynamodbConfig:  Amazon DynamoDB settings.:<br/>serviceRoleArn:  The AWS IAM service role ARN for the data source. The system assumes this role when accessing the data source.<br/>type:  The type of the <code>DataSource</code>.<br/>description:  A description of the <code>DataSource</code>.<br/>name:  A user-supplied name for the <code>DataSource</code>.<br/>apiId:  The API ID for the GraphQL API for the <code>DataSource</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>useCallerCredentials</i> Set to TRUE to use Amazon Cognito credentials with this data source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>awsRegion</i> The AWS Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tableName</i> The table name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lambdaFunctionArn</i> The ARN for the Lambda function.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>awsRegion</i> The AWS Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>endpoint</i> The endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authorizationConfig</i> The authorization config in case the HTTP endpoint requires authorization.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>authorizationConfig</b>: The authorization config in case the HTTP endpoint requires authorization.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>endpoint</i> The HTTP URL endpoint. You can either specify the domain name or IP, and port combination, and the URL scheme must be HTTP or HTTPS. If the port is not specified, AWS AppSync uses the default port 80 for the HTTP endpoint and port 443 for HTTPS endpoints.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>authorizationType</i> The authorization type required by the HTTP endpoint. <ul> <li>  <b>AWS_IAM</b>: The authorization type is Sigv4. </li> </ul><br/>The AWS IAM settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>awsIamConfig</b>: The AWS IAM settings.&nbsp;&nbsp;&nbsp;&nbsp; <i>rdsHttpEndpointConfig</i> Amazon RDS HTTP endpoint settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>rdsHttpEndpointConfig</b>: Amazon RDS HTTP endpoint settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>relationalDatabaseSourceType</i> Source type for the relational database. <ul> <li>  <b>RDS_HTTP_ENDPOINT</b>: The relational database source type is an Amazon RDS HTTP endpoint. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>awsSecretStoreArn</i> AWS secret store ARN for database credentials.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>schema</i> Logical schema name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>databaseName</i> Logical database name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>dbClusterIdentifier</i> Amazon RDS cluster identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>awsRegion</i> AWS Region for RDS HTTP endpoint.<br/>
        :return: dataSource:  The <code>DataSource</code> object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>relationalDatabaseConfig</i> Relational database settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>relationalDatabaseConfig</b>: Relational database settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>httpConfig</i> HTTP endpoint settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>httpConfig</b>: HTTP endpoint settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>elasticsearchConfig</i> Amazon Elasticsearch Service settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>elasticsearchConfig</b>: Amazon Elasticsearch Service settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lambdaConfig</i> AWS Lambda settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>lambdaConfig</b>: AWS Lambda settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>dynamodbConfig</i> Amazon DynamoDB settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>dynamodbConfig</b>: Amazon DynamoDB settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceRoleArn</i> The AWS IAM service role ARN for the data source. The system assumes this role when accessing the data source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of the data source. <ul> <li>  <b>AMAZON_DYNAMODB</b>: The data source is an Amazon DynamoDB table. </li> <li>  <b>AMAZON_ELASTICSEARCH</b>: The data source is an Amazon Elasticsearch Service domain. </li> <li>  <b>AWS_LAMBDA</b>: The data source is an AWS Lambda function. </li> <li>  <b>NONE</b>: There is no data source. This type is used when you wish to invoke a GraphQL operation without connecting to a data source, such as performing data transformation with resolvers or triggering a subscription to be invoked from a mutation. </li> <li>  <b>HTTP</b>: The data source is an HTTP endpoint. </li> <li>  <b>RELATIONAL_DATABASE</b>: The data source is a relational database. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The description of the data source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the data source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>dataSourceArn</i> The data source ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>useCallerCredentials</i> Set to TRUE to use Amazon Cognito credentials with this data source.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>awsRegion</i> The AWS Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>tableName</i> The table name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lambdaFunctionArn</i> The ARN for the Lambda function.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>awsRegion</i> The AWS Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>endpoint</i> The endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>endpoint</i> The HTTP URL endpoint. You can either specify the domain name or IP, and port combination, and the URL scheme must be HTTP or HTTPS. If the port is not specified, AWS AppSync uses the default port 80 for the HTTP endpoint and port 443 for HTTPS endpoints.<br/>The authorization config in case the HTTP endpoint requires authorization.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>authorizationConfig</b>: The authorization config in case the HTTP endpoint requires authorization.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>relationalDatabaseSourceType</i> Source type for the relational database. <ul> <li>  <b>RDS_HTTP_ENDPOINT</b>: The relational database source type is an Amazon RDS HTTP endpoint. </li> </ul><br/>Amazon RDS HTTP endpoint settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>rdsHttpEndpointConfig</b>: Amazon RDS HTTP endpoint settings.        """
        pass

    def create_function(self, functionVersion: str, requestMappingTemplate: str, dataSourceName: str, name: str, apiId: str, description: str=None, responseMappingTemplate: str=None) -> CreateFunctionResponse:
        """Creates a <code>Function</code> object. A function is a reusable entity. Multiple functions can be used to compose the resolver logic.

        :param self:
        :param functionVersion:  The <code>version</code> of the request mapping template. Currently the supported value is 2018-05-29. <br/>responseMappingTemplate:  The <code>Function</code> response mapping template. <br/>requestMappingTemplate:  The <code>Function</code> request mapping template. Functions support only the 2018-05-29 version of the request mapping template.<br/>dataSourceName:  The <code>Function</code> <code>DataSource</code> name.<br/>description:  The <code>Function</code> description.<br/>name:  The <code>Function</code> name. The function name does not have to be unique.<br/>apiId:  The GraphQL API ID.<br/>
        :return: functionConfiguration:  The <code>Function</code> object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>functionVersion</i> The version of the request mapping template. Currently only the 2018-05-29 version of the template is supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>responseMappingTemplate</i> The <code>Function</code> response mapping template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>requestMappingTemplate</i> The <code>Function</code> request mapping template. Functions support only the 2018-05-29 version of the request mapping template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>dataSourceName</i> The name of the <code>DataSource</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The <code>Function</code> description.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the <code>Function</code> object.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>functionArn</i> The ARN of the <code>Function</code> object.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>functionId</i> A unique ID representing the <code>Function</code> object.<br/>        """
        pass

    def create_graphql_api(self, authenticationType: str, name: str, logConfig: LogConfig=None, userPoolConfig: UserPoolConfig=None, openIDConnectConfig: OpenIDConnectConfig=None, tags: TagMap=None, additionalAuthenticationProviders: List[AdditionalAuthenticationProvider]=None) -> CreateGraphqlApiResponse:
        """Creates a <code>GraphqlApi</code> object.

        :param self:
        :param additionalAuthenticationProviders:  A list of additional authentication providers for the <code>GraphqlApi</code> API.<br/>tags:  A <code>TagMap</code> object.<br/>openIDConnectConfig:  The OpenID Connect configuration.:<br/>userPoolConfig:  The Amazon Cognito user pool configuration.:<br/>authenticationType:  The authentication type: API key, AWS IAM, OIDC, or Amazon Cognito user pools.<br/>logConfig:  The Amazon CloudWatch Logs configuration.:<br/>name:  A user-supplied name for the <code>GraphqlApi</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cloudWatchLogsRoleArn</i> The service role that AWS AppSync will assume to publish to Amazon CloudWatch logs in your account. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fieldLogLevel</i> The field logging level. Values can be NONE, ERROR, or ALL.  <ul> <li>  <b>NONE</b>: No field-level logs are captured. </li> <li>  <b>ERROR</b>: Logs the following information only for the fields that are in error: <ul> <li> The error section in the server response. </li> <li> Field-level errors. </li> <li> The generated request/response functions that got resolved for error fields. </li> </ul> </li> <li>  <b>ALL</b>: The following information is logged for all fields in the query: <ul> <li> Field-level tracing information. </li> <li> The generated request/response functions that got resolved for each field. </li> </ul> </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>appIdClientRegex</i> A regular expression for validating the incoming Amazon Cognito user pool app client ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultAction</i> The action that you want your GraphQL API to take when a request that uses Amazon Cognito user pool authentication doesn't match the Amazon Cognito user pool configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>awsRegion</i> The AWS Region in which the user pool was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>userPoolId</i> The user pool ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authTTL</i> The number of milliseconds a token is valid after being authenticated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>iatTTL</i> The number of milliseconds a token is valid after being issued to a user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clientId</i> The client identifier of the Relying party at the OpenID identity provider. This identifier is typically obtained when the Relying party is registered with the OpenID identity provider. You can specify a regular expression so the AWS AppSync can validate against multiple client identifiers at a time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>issuer</i> The issuer for the OpenID Connect configuration. The issuer returned by discovery must exactly match the value of <code>iss</code> in the ID token.<br/>
        :return: graphqlApi:  The <code>GraphqlApi</code>.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>additionalAuthenticationProviders</i> A list of additional authentication providers for the <code>GraphqlApi</code> API.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> The tags.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>uris</i> The URIs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>openIDConnectConfig</i> The OpenID Connect configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>openIDConnectConfig</b>: The OpenID Connect configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>userPoolConfig</i> The Amazon Cognito user pool configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>userPoolConfig</b>: The Amazon Cognito user pool configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>logConfig</i> The Amazon CloudWatch Logs configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>logConfig</b>: The Amazon CloudWatch Logs configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authenticationType</i> The authentication type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>apiId</i> The API ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The API name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>cloudWatchLogsRoleArn</i> The service role that AWS AppSync will assume to publish to Amazon CloudWatch logs in your account. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>fieldLogLevel</i> The field logging level. Values can be NONE, ERROR, or ALL.  <ul> <li>  <b>NONE</b>: No field-level logs are captured. </li> <li>  <b>ERROR</b>: Logs the following information only for the fields that are in error: <ul> <li> The error section in the server response. </li> <li> Field-level errors. </li> <li> The generated request/response functions that got resolved for error fields. </li> </ul> </li> <li>  <b>ALL</b>: The following information is logged for all fields in the query: <ul> <li> Field-level tracing information. </li> <li> The generated request/response functions that got resolved for each field. </li> </ul> </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>appIdClientRegex</i> A regular expression for validating the incoming Amazon Cognito user pool app client ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultAction</i> The action that you want your GraphQL API to take when a request that uses Amazon Cognito user pool authentication doesn't match the Amazon Cognito user pool configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>awsRegion</i> The AWS Region in which the user pool was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>userPoolId</i> The user pool ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>authTTL</i> The number of milliseconds a token is valid after being authenticated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>iatTTL</i> The number of milliseconds a token is valid after being issued to a user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>clientId</i> The client identifier of the Relying party at the OpenID identity provider. This identifier is typically obtained when the Relying party is registered with the OpenID identity provider. You can specify a regular expression so the AWS AppSync can validate against multiple client identifiers at a time.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>issuer</i> The issuer for the OpenID Connect configuration. The issuer returned by discovery must exactly match the value of <code>iss</code> in the ID token.<br/>        """
        pass

    def create_resolver(self, requestMappingTemplate: str, fieldName: str, typeName: str, apiId: str, dataSourceName: str=None, responseMappingTemplate: str=None, kind: str=None, pipelineConfig: PipelineConfig=None) -> CreateResolverResponse:
        """Creates a <code>Resolver</code> object. A resolver converts incoming requests into a format that a data source can understand and converts the data source's responses into GraphQL.

        :param self:
        :param pipelineConfig:  The <code>PipelineConfig</code>.:<br/>kind:  The resolver type. <ul> <li>  <b>UNIT</b>: A UNIT resolver type. A UNIT resolver is the default resolver type. A UNIT resolver enables you to execute a GraphQL query against a single data source. </li> <li>  <b>PIPELINE</b>: A PIPELINE resolver type. A PIPELINE resolver enables you to execute a series of <code>Function</code> in a serial manner. You can use a pipeline resolver to execute a GraphQL query against multiple data sources. </li> </ul><br/>responseMappingTemplate:  The mapping template to be used for responses from the data source.<br/>requestMappingTemplate:  The mapping template to be used for requests. A resolver uses a request mapping template to convert a GraphQL expression into a format that a data source can understand. Mapping templates are written in Apache Velocity Template Language (VTL).<br/>dataSourceName:  The name of the data source for which the resolver is being created.<br/>fieldName:  The name of the field to attach the resolver to.<br/>typeName:  The name of the <code>Type</code>.<br/>apiId:  The ID for the GraphQL API for which the resolver is being created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>functions</i> A list of <code>Function</code> objects.<br/>
        :return: resolver:  The <code>Resolver</code> object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pipelineConfig</i> The <code>PipelineConfig</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>pipelineConfig</b>: The <code>PipelineConfig</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>kind</i> The resolver type. <ul> <li>  <b>UNIT</b>: A UNIT resolver type. A UNIT resolver is the default resolver type. A UNIT resolver enables you to execute a GraphQL query against a single data source. </li> <li>  <b>PIPELINE</b>: A PIPELINE resolver type. A PIPELINE resolver enables you to execute a series of <code>Function</code> in a serial manner. You can use a pipeline resolver to execute a GraphQL query against multiple data sources. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>responseMappingTemplate</i> The response mapping template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>requestMappingTemplate</i> The request mapping template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resolverArn</i> The resolver ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>dataSourceName</i> The resolver data source name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fieldName</i> The resolver field name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>typeName</i> The resolver type name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>functions</i> A list of <code>Function</code> objects.<br/>        """
        pass

    def create_type(self, format: str, definition: str, apiId: str) -> CreateTypeResponse:
        """Creates a <code>Type</code> object.

        :param self:
        :param format:  The type format: SDL or JSON.<br/>definition:  The type definition, in GraphQL Schema Definition Language (SDL) format. For more information, see the <a href="http://graphql.org/learn/schema/">GraphQL SDL documentation</a>.<br/>apiId:  The API ID.<br/>
        :return: type:  The <code>Type</code> object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>format</i> The type format: SDL or JSON.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>definition</i> The type definition.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The type ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The type description.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The type name.<br/>        """
        pass

    def delete_api_key(self, id: str, apiId: str) -> DeleteApiKeyResponse:
        """Deletes an API key.

        :param self:
        :param id:  The ID for the API key.<br/>apiId:  The API ID.<br/>
        :return:         """
        pass

    def delete_data_source(self, name: str, apiId: str) -> DeleteDataSourceResponse:
        """Deletes a <code>DataSource</code> object.

        :param self:
        :param name:  The name of the data source.<br/>apiId:  The API ID.<br/>
        :return:         """
        pass

    def delete_function(self, functionId: str, apiId: str) -> DeleteFunctionResponse:
        """Deletes a <code>Function</code>.

        :param self:
        :param functionId:  The <code>Function</code> ID.<br/>apiId:  The GraphQL API ID.<br/>
        :return:         """
        pass

    def delete_graphql_api(self, apiId: str) -> DeleteGraphqlApiResponse:
        """Deletes a <code>GraphqlApi</code> object.

        :param self:
        :param apiId:  The API ID.<br/>
        :return:         """
        pass

    def delete_resolver(self, fieldName: str, typeName: str, apiId: str) -> DeleteResolverResponse:
        """Deletes a <code>Resolver</code> object.

        :param self:
        :param fieldName:  The resolver field name.<br/>typeName:  The name of the resolver type.<br/>apiId:  The API ID.<br/>
        :return:         """
        pass

    def delete_type(self, typeName: str, apiId: str) -> DeleteTypeResponse:
        """Deletes a <code>Type</code> object.

        :param self:
        :param typeName:  The type name.<br/>apiId:  The API ID.<br/>
        :return:         """
        pass

    def get_data_source(self, name: str, apiId: str) -> GetDataSourceResponse:
        """Retrieves a <code>DataSource</code> object.

        :param self:
        :param name:  The name of the data source.<br/>apiId:  The API ID.<br/>
        :return: dataSource:  The <code>DataSource</code> object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>relationalDatabaseConfig</i> Relational database settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>relationalDatabaseConfig</b>: Relational database settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>httpConfig</i> HTTP endpoint settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>httpConfig</b>: HTTP endpoint settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>elasticsearchConfig</i> Amazon Elasticsearch Service settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>elasticsearchConfig</b>: Amazon Elasticsearch Service settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lambdaConfig</i> AWS Lambda settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>lambdaConfig</b>: AWS Lambda settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>dynamodbConfig</i> Amazon DynamoDB settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>dynamodbConfig</b>: Amazon DynamoDB settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceRoleArn</i> The AWS IAM service role ARN for the data source. The system assumes this role when accessing the data source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of the data source. <ul> <li>  <b>AMAZON_DYNAMODB</b>: The data source is an Amazon DynamoDB table. </li> <li>  <b>AMAZON_ELASTICSEARCH</b>: The data source is an Amazon Elasticsearch Service domain. </li> <li>  <b>AWS_LAMBDA</b>: The data source is an AWS Lambda function. </li> <li>  <b>NONE</b>: There is no data source. This type is used when you wish to invoke a GraphQL operation without connecting to a data source, such as performing data transformation with resolvers or triggering a subscription to be invoked from a mutation. </li> <li>  <b>HTTP</b>: The data source is an HTTP endpoint. </li> <li>  <b>RELATIONAL_DATABASE</b>: The data source is a relational database. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The description of the data source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the data source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>dataSourceArn</i> The data source ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>useCallerCredentials</i> Set to TRUE to use Amazon Cognito credentials with this data source.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>awsRegion</i> The AWS Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>tableName</i> The table name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lambdaFunctionArn</i> The ARN for the Lambda function.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>awsRegion</i> The AWS Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>endpoint</i> The endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>endpoint</i> The HTTP URL endpoint. You can either specify the domain name or IP, and port combination, and the URL scheme must be HTTP or HTTPS. If the port is not specified, AWS AppSync uses the default port 80 for the HTTP endpoint and port 443 for HTTPS endpoints.<br/>The authorization config in case the HTTP endpoint requires authorization.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>authorizationConfig</b>: The authorization config in case the HTTP endpoint requires authorization.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>relationalDatabaseSourceType</i> Source type for the relational database. <ul> <li>  <b>RDS_HTTP_ENDPOINT</b>: The relational database source type is an Amazon RDS HTTP endpoint. </li> </ul><br/>Amazon RDS HTTP endpoint settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>rdsHttpEndpointConfig</b>: Amazon RDS HTTP endpoint settings.        """
        pass

    def get_function(self, functionId: str, apiId: str) -> GetFunctionResponse:
        """Get a <code>Function</code>.

        :param self:
        :param functionId:  The <code>Function</code> ID.<br/>apiId:  The GraphQL API ID.<br/>
        :return: functionConfiguration:  The <code>Function</code> object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>functionVersion</i> The version of the request mapping template. Currently only the 2018-05-29 version of the template is supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>responseMappingTemplate</i> The <code>Function</code> response mapping template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>requestMappingTemplate</i> The <code>Function</code> request mapping template. Functions support only the 2018-05-29 version of the request mapping template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>dataSourceName</i> The name of the <code>DataSource</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The <code>Function</code> description.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the <code>Function</code> object.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>functionArn</i> The ARN of the <code>Function</code> object.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>functionId</i> A unique ID representing the <code>Function</code> object.<br/>        """
        pass

    def get_graphql_api(self, apiId: str) -> GetGraphqlApiResponse:
        """Retrieves a <code>GraphqlApi</code> object.

        :param self:
        :param apiId:  The API ID for the GraphQL API.<br/>
        :return: graphqlApi:  The <code>GraphqlApi</code> object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>additionalAuthenticationProviders</i> A list of additional authentication providers for the <code>GraphqlApi</code> API.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> The tags.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>uris</i> The URIs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>openIDConnectConfig</i> The OpenID Connect configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>openIDConnectConfig</b>: The OpenID Connect configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>userPoolConfig</i> The Amazon Cognito user pool configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>userPoolConfig</b>: The Amazon Cognito user pool configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>logConfig</i> The Amazon CloudWatch Logs configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>logConfig</b>: The Amazon CloudWatch Logs configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authenticationType</i> The authentication type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>apiId</i> The API ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The API name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>cloudWatchLogsRoleArn</i> The service role that AWS AppSync will assume to publish to Amazon CloudWatch logs in your account. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>fieldLogLevel</i> The field logging level. Values can be NONE, ERROR, or ALL.  <ul> <li>  <b>NONE</b>: No field-level logs are captured. </li> <li>  <b>ERROR</b>: Logs the following information only for the fields that are in error: <ul> <li> The error section in the server response. </li> <li> Field-level errors. </li> <li> The generated request/response functions that got resolved for error fields. </li> </ul> </li> <li>  <b>ALL</b>: The following information is logged for all fields in the query: <ul> <li> Field-level tracing information. </li> <li> The generated request/response functions that got resolved for each field. </li> </ul> </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>appIdClientRegex</i> A regular expression for validating the incoming Amazon Cognito user pool app client ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultAction</i> The action that you want your GraphQL API to take when a request that uses Amazon Cognito user pool authentication doesn't match the Amazon Cognito user pool configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>awsRegion</i> The AWS Region in which the user pool was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>userPoolId</i> The user pool ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>authTTL</i> The number of milliseconds a token is valid after being authenticated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>iatTTL</i> The number of milliseconds a token is valid after being issued to a user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>clientId</i> The client identifier of the Relying party at the OpenID identity provider. This identifier is typically obtained when the Relying party is registered with the OpenID identity provider. You can specify a regular expression so the AWS AppSync can validate against multiple client identifiers at a time.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>issuer</i> The issuer for the OpenID Connect configuration. The issuer returned by discovery must exactly match the value of <code>iss</code> in the ID token.<br/>        """
        pass

    def get_introspection_schema(self, format: str, apiId: str, includeDirectives: BooleanValue=None) -> GetIntrospectionSchemaResponse:
        """Retrieves the introspection schema for a GraphQL API.

        :param self:
        :param includeDirectives:  A flag that specifies whether the schema introspection should contain directives.<br/>format:  The schema format: SDL or JSON.<br/>apiId:  The API ID.<br/>
        :return: schema:  The schema, in GraphQL Schema Definition Language (SDL) format. For more information, see the <a href="http://graphql.org/learn/schema/">GraphQL SDL documentation</a>.<br/>        """
        pass

    def get_resolver(self, fieldName: str, typeName: str, apiId: str) -> GetResolverResponse:
        """Retrieves a <code>Resolver</code> object.

        :param self:
        :param fieldName:  The resolver field name.<br/>typeName:  The resolver type name.<br/>apiId:  The API ID.<br/>
        :return: resolver:  The <code>Resolver</code> object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pipelineConfig</i> The <code>PipelineConfig</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>pipelineConfig</b>: The <code>PipelineConfig</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>kind</i> The resolver type. <ul> <li>  <b>UNIT</b>: A UNIT resolver type. A UNIT resolver is the default resolver type. A UNIT resolver enables you to execute a GraphQL query against a single data source. </li> <li>  <b>PIPELINE</b>: A PIPELINE resolver type. A PIPELINE resolver enables you to execute a series of <code>Function</code> in a serial manner. You can use a pipeline resolver to execute a GraphQL query against multiple data sources. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>responseMappingTemplate</i> The response mapping template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>requestMappingTemplate</i> The request mapping template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resolverArn</i> The resolver ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>dataSourceName</i> The resolver data source name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fieldName</i> The resolver field name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>typeName</i> The resolver type name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>functions</i> A list of <code>Function</code> objects.<br/>        """
        pass

    def get_schema_creation_status(self, apiId: str) -> GetSchemaCreationStatusResponse:
        """Retrieves the current status of a schema creation operation.

        :param self:
        :param apiId:  The API ID.<br/>
        :return: details:  Detailed information about the status of the schema creation operation.<br/>status:  The current state of the schema (PROCESSING, FAILED, SUCCESS, or NOT_APPLICABLE). When the schema is in the ACTIVE state, you can add data.<br/>        """
        pass

    def get_type(self, format: str, typeName: str, apiId: str) -> GetTypeResponse:
        """Retrieves a <code>Type</code> object.

        :param self:
        :param format:  The type format: SDL or JSON.<br/>typeName:  The type name.<br/>apiId:  The API ID.<br/>
        :return: type:  The <code>Type</code> object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>format</i> The type format: SDL or JSON.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>definition</i> The type definition.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The type ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The type description.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The type name.<br/>        """
        pass

    def list_api_keys(self, apiId: str, nextToken: str=None, maxResults: int=None) -> ListApiKeysResponse:
        """Lists the API keys for a given API. <note> API keys are deleted automatically sometime after they expire. However, they may still be included in the response until they have actually been deleted. You can safely call <code>DeleteApiKey</code> to manually delete a key before it's automatically deleted. </note>

        :param self:
        :param maxResults:  The maximum number of results you want the request to return.<br/>nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>apiId:  The API ID.<br/>
        :return: nextToken:  An identifier to be passed in the next request to this operation to return the next set of items in the list.<br/>apiKeys:  The <code>ApiKey</code> objects.<br/>        """
        pass

    def list_data_sources(self, apiId: str, nextToken: str=None, maxResults: int=None) -> ListDataSourcesResponse:
        """Lists the data sources for a given API.

        :param self:
        :param maxResults:  The maximum number of results you want the request to return.<br/>nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list. <br/>apiId:  The API ID.<br/>
        :return: nextToken:  An identifier to be passed in the next request to this operation to return the next set of items in the list.<br/>dataSources:  The <code>DataSource</code> objects.<br/>        """
        pass

    def list_functions(self, apiId: str, nextToken: str=None, maxResults: int=None) -> ListFunctionsResponse:
        """List multiple functions.

        :param self:
        :param maxResults:  The maximum number of results you want the request to return.<br/>nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>apiId:  The GraphQL API ID.<br/>
        :return: nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>functions:  A list of <code>Function</code> objects.<br/>        """
        pass

    def list_graphql_apis(self, nextToken: str=None, maxResults: int=None) -> ListGraphqlApisResponse:
        """Lists your GraphQL APIs.

        :param self:
        :param maxResults:  The maximum number of results you want the request to return.<br/>nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list. <br/>
        :return: nextToken:  An identifier to be passed in the next request to this operation to return the next set of items in the list.<br/>graphqlApis:  The <code>GraphqlApi</code> objects.<br/>        """
        pass

    def list_resolvers(self, typeName: str, apiId: str, nextToken: str=None, maxResults: int=None) -> ListResolversResponse:
        """Lists the resolvers for a given API and type.

        :param self:
        :param maxResults:  The maximum number of results you want the request to return.<br/>nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list. <br/>typeName:  The type name.<br/>apiId:  The API ID.<br/>
        :return: nextToken:  An identifier to be passed in the next request to this operation to return the next set of items in the list.<br/>resolvers:  The <code>Resolver</code> objects.<br/>        """
        pass

    def list_resolvers_by_function(self, functionId: str, apiId: str, nextToken: str=None, maxResults: int=None) -> ListResolversByFunctionResponse:
        """List the resolvers that are associated with a specific function.

        :param self:
        :param maxResults:  The maximum number of results you want the request to return.<br/>nextToken:  An identifier that was returned from the previous call to this operation, which you can use to return the next set of items in the list.<br/>functionId:  The Function ID.<br/>apiId:  The API ID.<br/>
        :return: nextToken:  An identifier that can be used to return the next set of items in the list.<br/>resolvers:  The list of resolvers.<br/>        """
        pass

    def list_tags_for_resource(self, resourceArn: str) -> ListTagsForResourceResponse:
        """Lists the tags for a resource.

        :param self:
        :param resourceArn:  The <code>GraphqlApi</code> ARN.<br/>
        :return: tags:  A <code>TagMap</code> object.<br/>        """
        pass

    def list_types(self, format: str, apiId: str, nextToken: str=None, maxResults: int=None) -> ListTypesResponse:
        """Lists the types for a given API.

        :param self:
        :param maxResults:  The maximum number of results you want the request to return.<br/>nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list. <br/>format:  The type format: SDL or JSON.<br/>apiId:  The API ID.<br/>
        :return: nextToken:  An identifier to be passed in the next request to this operation to return the next set of items in the list.<br/>types:  The <code>Type</code> objects.<br/>        """
        pass

    def start_schema_creation(self, definition: Blob, apiId: str) -> StartSchemaCreationResponse:
        """Adds a new schema to your GraphQL API. This operation is asynchronous. Use to determine when it has completed.

        :param self:
        :param definition:  The schema definition, in GraphQL schema language format.<br/>apiId:  The API ID.<br/>
        :return: status:  The current state of the schema (PROCESSING, FAILED, SUCCESS, or NOT_APPLICABLE). When the schema is in the ACTIVE state, you can add data.<br/>        """
        pass

    def tag_resource(self, tags: TagMap, resourceArn: str) -> TagResourceResponse:
        """Tags a resource with user-supplied tags.

        :param self:
        :param tags:  A <code>TagMap</code> object.<br/>resourceArn:  The <code>GraphqlApi</code> ARN.<br/>
        :return:         """
        pass

    def untag_resource(self, tagKeys: List[TagKey], resourceArn: str) -> UntagResourceResponse:
        """Untags a resource.

        :param self:
        :param tagKeys:  A list of <code>TagKey</code> objects.<br/>resourceArn:  The <code>GraphqlApi</code> ARN.<br/>
        :return:         """
        pass

    def update_api_key(self, id: str, apiId: str, description: str=None, expires: Long=None) -> UpdateApiKeyResponse:
        """Updates an API key.

        :param self:
        :param expires:  The time from update time after which the API key expires. The date is represented as seconds since the epoch. For more information, see .<br/>description:  A description of the purpose of the API key.<br/>id:  The API key ID.<br/>apiId:  The ID for the GraphQL API.<br/>
        :return: apiKey:  The API key.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>expires</i> The time after which the API key expires. The date is represented as seconds since the epoch, rounded down to the nearest hour.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> A description of the purpose of the API key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The API key ID.<br/>        """
        pass

    def update_data_source(self, type: str, name: str, apiId: str, description: str=None, serviceRoleArn: str=None, dynamodbConfig: DynamodbDataSourceConfig=None, lambdaConfig: LambdaDataSourceConfig=None, elasticsearchConfig: ElasticsearchDataSourceConfig=None, httpConfig: HttpDataSourceConfig=None, relationalDatabaseConfig: RelationalDatabaseDataSourceConfig=None) -> UpdateDataSourceResponse:
        """Updates a <code>DataSource</code> object.

        :param self:
        :param relationalDatabaseConfig:  The new relational database configuration.:<br/>httpConfig:  The new HTTP endpoint configuration.:<br/>elasticsearchConfig:  The new Elasticsearch Service configuration.:<br/>lambdaConfig:  The new AWS Lambda configuration.:<br/>dynamodbConfig:  The new Amazon DynamoDB configuration.:<br/>serviceRoleArn:  The new service role ARN for the data source.<br/>type:  The new data source type.<br/>description:  The new description for the data source.<br/>name:  The new name for the data source.<br/>apiId:  The API ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>useCallerCredentials</i> Set to TRUE to use Amazon Cognito credentials with this data source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>awsRegion</i> The AWS Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tableName</i> The table name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lambdaFunctionArn</i> The ARN for the Lambda function.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>awsRegion</i> The AWS Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>endpoint</i> The endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authorizationConfig</i> The authorization config in case the HTTP endpoint requires authorization.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>authorizationConfig</b>: The authorization config in case the HTTP endpoint requires authorization.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>endpoint</i> The HTTP URL endpoint. You can either specify the domain name or IP, and port combination, and the URL scheme must be HTTP or HTTPS. If the port is not specified, AWS AppSync uses the default port 80 for the HTTP endpoint and port 443 for HTTPS endpoints.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>authorizationType</i> The authorization type required by the HTTP endpoint. <ul> <li>  <b>AWS_IAM</b>: The authorization type is Sigv4. </li> </ul><br/>The AWS IAM settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>awsIamConfig</b>: The AWS IAM settings.&nbsp;&nbsp;&nbsp;&nbsp; <i>rdsHttpEndpointConfig</i> Amazon RDS HTTP endpoint settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>rdsHttpEndpointConfig</b>: Amazon RDS HTTP endpoint settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>relationalDatabaseSourceType</i> Source type for the relational database. <ul> <li>  <b>RDS_HTTP_ENDPOINT</b>: The relational database source type is an Amazon RDS HTTP endpoint. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>awsSecretStoreArn</i> AWS secret store ARN for database credentials.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>schema</i> Logical schema name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>databaseName</i> Logical database name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>dbClusterIdentifier</i> Amazon RDS cluster identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>awsRegion</i> AWS Region for RDS HTTP endpoint.<br/>
        :return: dataSource:  The updated <code>DataSource</code> object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>relationalDatabaseConfig</i> Relational database settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>relationalDatabaseConfig</b>: Relational database settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>httpConfig</i> HTTP endpoint settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>httpConfig</b>: HTTP endpoint settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>elasticsearchConfig</i> Amazon Elasticsearch Service settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>elasticsearchConfig</b>: Amazon Elasticsearch Service settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lambdaConfig</i> AWS Lambda settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>lambdaConfig</b>: AWS Lambda settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>dynamodbConfig</i> Amazon DynamoDB settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>dynamodbConfig</b>: Amazon DynamoDB settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceRoleArn</i> The AWS IAM service role ARN for the data source. The system assumes this role when accessing the data source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of the data source. <ul> <li>  <b>AMAZON_DYNAMODB</b>: The data source is an Amazon DynamoDB table. </li> <li>  <b>AMAZON_ELASTICSEARCH</b>: The data source is an Amazon Elasticsearch Service domain. </li> <li>  <b>AWS_LAMBDA</b>: The data source is an AWS Lambda function. </li> <li>  <b>NONE</b>: There is no data source. This type is used when you wish to invoke a GraphQL operation without connecting to a data source, such as performing data transformation with resolvers or triggering a subscription to be invoked from a mutation. </li> <li>  <b>HTTP</b>: The data source is an HTTP endpoint. </li> <li>  <b>RELATIONAL_DATABASE</b>: The data source is a relational database. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The description of the data source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the data source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>dataSourceArn</i> The data source ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>useCallerCredentials</i> Set to TRUE to use Amazon Cognito credentials with this data source.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>awsRegion</i> The AWS Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>tableName</i> The table name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lambdaFunctionArn</i> The ARN for the Lambda function.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>awsRegion</i> The AWS Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>endpoint</i> The endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>endpoint</i> The HTTP URL endpoint. You can either specify the domain name or IP, and port combination, and the URL scheme must be HTTP or HTTPS. If the port is not specified, AWS AppSync uses the default port 80 for the HTTP endpoint and port 443 for HTTPS endpoints.<br/>The authorization config in case the HTTP endpoint requires authorization.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>authorizationConfig</b>: The authorization config in case the HTTP endpoint requires authorization.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>relationalDatabaseSourceType</i> Source type for the relational database. <ul> <li>  <b>RDS_HTTP_ENDPOINT</b>: The relational database source type is an Amazon RDS HTTP endpoint. </li> </ul><br/>Amazon RDS HTTP endpoint settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>rdsHttpEndpointConfig</b>: Amazon RDS HTTP endpoint settings.        """
        pass

    def update_function(self, functionVersion: str, requestMappingTemplate: str, dataSourceName: str, functionId: str, name: str, apiId: str, description: str=None, responseMappingTemplate: str=None) -> UpdateFunctionResponse:
        """Updates a <code>Function</code> object.

        :param self:
        :param functionVersion:  The <code>version</code> of the request mapping template. Currently the supported value is 2018-05-29. <br/>responseMappingTemplate:  The <code>Function</code> request mapping template. <br/>requestMappingTemplate:  The <code>Function</code> request mapping template. Functions support only the 2018-05-29 version of the request mapping template.<br/>dataSourceName:  The <code>Function</code> <code>DataSource</code> name.<br/>functionId:  The function ID.<br/>description:  The <code>Function</code> description.<br/>name:  The <code>Function</code> name.<br/>apiId:  The GraphQL API ID.<br/>
        :return: functionConfiguration:  The <code>Function</code> object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>functionVersion</i> The version of the request mapping template. Currently only the 2018-05-29 version of the template is supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>responseMappingTemplate</i> The <code>Function</code> response mapping template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>requestMappingTemplate</i> The <code>Function</code> request mapping template. Functions support only the 2018-05-29 version of the request mapping template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>dataSourceName</i> The name of the <code>DataSource</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The <code>Function</code> description.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the <code>Function</code> object.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>functionArn</i> The ARN of the <code>Function</code> object.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>functionId</i> A unique ID representing the <code>Function</code> object.<br/>        """
        pass

    def update_graphql_api(self, name: str, apiId: str, logConfig: LogConfig=None, authenticationType: str=None, userPoolConfig: UserPoolConfig=None, openIDConnectConfig: OpenIDConnectConfig=None, additionalAuthenticationProviders: List[AdditionalAuthenticationProvider]=None) -> UpdateGraphqlApiResponse:
        """Updates a <code>GraphqlApi</code> object.

        :param self:
        :param additionalAuthenticationProviders:  A list of additional authentication providers for the <code>GraphqlApi</code> API.<br/>openIDConnectConfig:  The OpenID Connect configuration for the <code>GraphqlApi</code> object.:<br/>userPoolConfig:  The new Amazon Cognito user pool configuration for the <code>GraphqlApi</code> object.:<br/>authenticationType:  The new authentication type for the <code>GraphqlApi</code> object.<br/>logConfig:  The Amazon CloudWatch Logs configuration for the <code>GraphqlApi</code> object.:<br/>name:  The new name for the <code>GraphqlApi</code> object.<br/>apiId:  The API ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cloudWatchLogsRoleArn</i> The service role that AWS AppSync will assume to publish to Amazon CloudWatch logs in your account. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fieldLogLevel</i> The field logging level. Values can be NONE, ERROR, or ALL.  <ul> <li>  <b>NONE</b>: No field-level logs are captured. </li> <li>  <b>ERROR</b>: Logs the following information only for the fields that are in error: <ul> <li> The error section in the server response. </li> <li> Field-level errors. </li> <li> The generated request/response functions that got resolved for error fields. </li> </ul> </li> <li>  <b>ALL</b>: The following information is logged for all fields in the query: <ul> <li> Field-level tracing information. </li> <li> The generated request/response functions that got resolved for each field. </li> </ul> </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>appIdClientRegex</i> A regular expression for validating the incoming Amazon Cognito user pool app client ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultAction</i> The action that you want your GraphQL API to take when a request that uses Amazon Cognito user pool authentication doesn't match the Amazon Cognito user pool configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>awsRegion</i> The AWS Region in which the user pool was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>userPoolId</i> The user pool ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authTTL</i> The number of milliseconds a token is valid after being authenticated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>iatTTL</i> The number of milliseconds a token is valid after being issued to a user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clientId</i> The client identifier of the Relying party at the OpenID identity provider. This identifier is typically obtained when the Relying party is registered with the OpenID identity provider. You can specify a regular expression so the AWS AppSync can validate against multiple client identifiers at a time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>issuer</i> The issuer for the OpenID Connect configuration. The issuer returned by discovery must exactly match the value of <code>iss</code> in the ID token.<br/>
        :return: graphqlApi:  The updated <code>GraphqlApi</code> object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>additionalAuthenticationProviders</i> A list of additional authentication providers for the <code>GraphqlApi</code> API.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> The tags.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>uris</i> The URIs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>openIDConnectConfig</i> The OpenID Connect configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>openIDConnectConfig</b>: The OpenID Connect configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>userPoolConfig</i> The Amazon Cognito user pool configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>userPoolConfig</b>: The Amazon Cognito user pool configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>logConfig</i> The Amazon CloudWatch Logs configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>logConfig</b>: The Amazon CloudWatch Logs configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authenticationType</i> The authentication type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>apiId</i> The API ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The API name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>cloudWatchLogsRoleArn</i> The service role that AWS AppSync will assume to publish to Amazon CloudWatch logs in your account. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>fieldLogLevel</i> The field logging level. Values can be NONE, ERROR, or ALL.  <ul> <li>  <b>NONE</b>: No field-level logs are captured. </li> <li>  <b>ERROR</b>: Logs the following information only for the fields that are in error: <ul> <li> The error section in the server response. </li> <li> Field-level errors. </li> <li> The generated request/response functions that got resolved for error fields. </li> </ul> </li> <li>  <b>ALL</b>: The following information is logged for all fields in the query: <ul> <li> Field-level tracing information. </li> <li> The generated request/response functions that got resolved for each field. </li> </ul> </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>appIdClientRegex</i> A regular expression for validating the incoming Amazon Cognito user pool app client ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultAction</i> The action that you want your GraphQL API to take when a request that uses Amazon Cognito user pool authentication doesn't match the Amazon Cognito user pool configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>awsRegion</i> The AWS Region in which the user pool was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>userPoolId</i> The user pool ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>authTTL</i> The number of milliseconds a token is valid after being authenticated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>iatTTL</i> The number of milliseconds a token is valid after being issued to a user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>clientId</i> The client identifier of the Relying party at the OpenID identity provider. This identifier is typically obtained when the Relying party is registered with the OpenID identity provider. You can specify a regular expression so the AWS AppSync can validate against multiple client identifiers at a time.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>issuer</i> The issuer for the OpenID Connect configuration. The issuer returned by discovery must exactly match the value of <code>iss</code> in the ID token.<br/>        """
        pass

    def update_resolver(self, requestMappingTemplate: str, fieldName: str, typeName: str, apiId: str, dataSourceName: str=None, responseMappingTemplate: str=None, kind: str=None, pipelineConfig: PipelineConfig=None) -> UpdateResolverResponse:
        """Updates a <code>Resolver</code> object.

        :param self:
        :param pipelineConfig:  The <code>PipelineConfig</code>.:<br/>kind:  The resolver type. <ul> <li>  <b>UNIT</b>: A UNIT resolver type. A UNIT resolver is the default resolver type. A UNIT resolver enables you to execute a GraphQL query against a single data source. </li> <li>  <b>PIPELINE</b>: A PIPELINE resolver type. A PIPELINE resolver enables you to execute a series of <code>Function</code> in a serial manner. You can use a pipeline resolver to execute a GraphQL query against multiple data sources. </li> </ul><br/>responseMappingTemplate:  The new response mapping template.<br/>requestMappingTemplate:  The new request mapping template.<br/>dataSourceName:  The new data source name.<br/>fieldName:  The new field name.<br/>typeName:  The new type name.<br/>apiId:  The API ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>functions</i> A list of <code>Function</code> objects.<br/>
        :return: resolver:  The updated <code>Resolver</code> object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pipelineConfig</i> The <code>PipelineConfig</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>pipelineConfig</b>: The <code>PipelineConfig</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>kind</i> The resolver type. <ul> <li>  <b>UNIT</b>: A UNIT resolver type. A UNIT resolver is the default resolver type. A UNIT resolver enables you to execute a GraphQL query against a single data source. </li> <li>  <b>PIPELINE</b>: A PIPELINE resolver type. A PIPELINE resolver enables you to execute a series of <code>Function</code> in a serial manner. You can use a pipeline resolver to execute a GraphQL query against multiple data sources. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>responseMappingTemplate</i> The response mapping template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>requestMappingTemplate</i> The request mapping template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resolverArn</i> The resolver ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>dataSourceName</i> The resolver data source name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fieldName</i> The resolver field name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>typeName</i> The resolver type name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>functions</i> A list of <code>Function</code> objects.<br/>        """
        pass

    def update_type(self, format: str, typeName: str, apiId: str, definition: str=None) -> UpdateTypeResponse:
        """Updates a <code>Type</code> object.

        :param self:
        :param format:  The new type format: SDL or JSON.<br/>definition:  The new definition.<br/>typeName:  The new type name.<br/>apiId:  The API ID.<br/>
        :return: type:  The updated <code>Type</code> object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>format</i> The type format: SDL or JSON.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>definition</i> The type definition.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The type ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The type description.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The type name.<br/>        """
        pass



class Athena(BaseClient):
    """Amazon Athena is an interactive query service that lets you use standard SQL to analyze data directly in Amazon S3. You can point Athena at your data in Amazon S3 and run ad-hoc queries and get results in seconds. Athena is serverless, so there is no infrastructure to set up or manage. You pay only for the queries you run. Athena scales automatically—executing queries in parallel—so results are fast, even with large datasets and complex queries. For more information, see <a href="http://docs.aws.amazon.com/athena/latest/ug/what-is.html">What is Amazon Athena</a> in the <i>Amazon Athena User Guide</i>. If you connect to Athena using the JDBC driver, use version 1.1.0 of the driver or later with the Amazon Athena API. Earlier version drivers do not support the API. For more information and to download the driver, see <a href="https://docs.aws.amazon.com/athena/latest/ug/connect-with-jdbc.html">Accessing Amazon Athena with JDBC</a>. For code samples using the AWS SDK for Java, see <a href="https://docs.aws.amazon.com/athena/latest/ug/code-samples.html">Examples and Code Samples</a> in the <i>Amazon Athena User Guide</i>."""


    class ListNamedQueriesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_named_queries` for valid parameters"""
            pass
    class ListQueryExecutionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_query_executions` for valid parameters"""
            pass
    class GetQueryResultsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_query_results` for valid parameters"""
            pass


    class BatchGetNamedQueryInput(Mapping):
        pass
    
    class BatchGetNamedQueryOutput(Mapping):
        pass
    
    class NamedQueryId(object):
        pass
    
    class BatchGetQueryExecutionInput(Mapping):
        pass
    
    class BatchGetQueryExecutionOutput(Mapping):
        pass
    
    class QueryExecutionId(object):
        pass
    
    class CreateNamedQueryInput(Mapping):
        pass
    
    class CreateNamedQueryOutput(Mapping):
        pass
    
    class CreateWorkGroupInput(Mapping):
        pass
    
    class CreateWorkGroupOutput(Mapping):
        pass
    
    class WorkGroupConfiguration(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class DeleteNamedQueryInput(Mapping):
        pass
    
    class DeleteNamedQueryOutput(Mapping):
        pass
    
    class DeleteWorkGroupInput(Mapping):
        pass
    
    class DeleteWorkGroupOutput(Mapping):
        pass
    
    class BoxedBoolean(object):
        pass
    
    class GetNamedQueryInput(Mapping):
        pass
    
    class GetNamedQueryOutput(Mapping):
        pass
    
    class GetQueryExecutionInput(Mapping):
        pass
    
    class GetQueryExecutionOutput(Mapping):
        pass
    
    class GetQueryResultsInput(Mapping):
        pass
    
    class GetQueryResultsOutput(Mapping):
        pass
    
    class GetWorkGroupInput(Mapping):
        pass
    
    class GetWorkGroupOutput(Mapping):
        pass
    
    class ListNamedQueriesInput(Mapping):
        pass
    
    class ListNamedQueriesOutput(Mapping):
        pass
    
    class ListQueryExecutionsInput(Mapping):
        pass
    
    class ListQueryExecutionsOutput(Mapping):
        pass
    
    class ListTagsForResourceInput(Mapping):
        pass
    
    class ListTagsForResourceOutput(Mapping):
        pass
    
    class ListWorkGroupsInput(Mapping):
        pass
    
    class ListWorkGroupsOutput(Mapping):
        pass
    
    class StartQueryExecutionInput(Mapping):
        pass
    
    class StartQueryExecutionOutput(Mapping):
        pass
    
    class QueryExecutionContext(Mapping):
        pass
    
    class ResultConfiguration(Mapping):
        pass
    
    class StopQueryExecutionInput(Mapping):
        pass
    
    class StopQueryExecutionOutput(Mapping):
        pass
    
    class TagResourceInput(Mapping):
        pass
    
    class TagResourceOutput(Mapping):
        pass
    
    class UntagResourceInput(Mapping):
        pass
    
    class UntagResourceOutput(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateWorkGroupInput(Mapping):
        pass
    
    class UpdateWorkGroupOutput(Mapping):
        pass
    
    class WorkGroupConfigurationUpdates(Mapping):
        pass
    
    def batch_get_named_query(self, NamedQueryIds: List[NamedQueryId]) -> BatchGetNamedQueryOutput:
        """Returns the details of a single named query or a list of up to 50 queries, which you provide as an array of query ID strings. Requires you to have access to the workgroup in which the queries were saved. Use <a>ListNamedQueriesInput</a> to get the list of named query IDs in the specified workgroup. If information could not be retrieved for a submitted query ID, information about the query ID submitted is listed under <a>UnprocessedNamedQueryId</a>. Named queries differ from executed queries. Use <a>BatchGetQueryExecutionInput</a> to get details about each unique query execution, and <a>ListQueryExecutionsInput</a> to get a list of query execution IDs.

        :param self:
        :param NamedQueryIds:  An array of query IDs.<br/>
        :return: UnprocessedNamedQueryIds:  Information about provided query IDs.<br/>NamedQueries:  Information about the named query IDs submitted.<br/>        """
        pass

    def batch_get_query_execution(self, QueryExecutionIds: List[QueryExecutionId]) -> BatchGetQueryExecutionOutput:
        """Returns the details of a single query execution or a list of up to 50 query executions, which you provide as an array of query execution ID strings. Requires you to have access to the workgroup in which the queries ran. To get a list of query execution IDs, use <a>ListQueryExecutionsInput$WorkGroup</a>. Query executions differ from named (saved) queries. Use <a>BatchGetNamedQueryInput</a> to get details about named queries.

        :param self:
        :param QueryExecutionIds:  An array of query execution IDs.<br/>
        :return: UnprocessedQueryExecutionIds:  Information about the query executions that failed to run.<br/>QueryExecutions:  Information about a query execution.<br/>        """
        pass

    def create_named_query(self, QueryString: str, Database: str, Name: str, Description: str=None, ClientRequestToken: str=None, WorkGroup: str=None) -> CreateNamedQueryOutput:
        """Creates a named query in the specified workgroup. Requires that you have access to the workgroup. For code samples using the AWS SDK for Java, see <a href="http://docs.aws.amazon.com/athena/latest/ug/code-samples.html">Examples and Code Samples</a> in the <i>Amazon Athena User Guide</i>.

        :param self:
        :param WorkGroup:  The name of the workgroup in which the named query is being created.<br/>ClientRequestToken:  A unique case-sensitive string used to ensure the request to create the query is idempotent (executes only once). If another <code>CreateNamedQuery</code> request is received, the same response is returned and another query is not created. If a parameter has changed, for example, the <code>QueryString</code>, an error is returned. <important> This token is listed as not required because AWS SDKs (for example the AWS SDK for Java) auto-generate the token for users. If you are not using the AWS SDK or the AWS CLI, you must provide this token or the action will fail. </important><br/>QueryString:  The contents of the query with all query statements.<br/>Database:  The database to which the query belongs.<br/>Description:  The query description.<br/>Name:  The query name.<br/>
        :return: NamedQueryId:  The unique ID of the query.<br/>        """
        pass

    def create_work_group(self, Name: str, Configuration: WorkGroupConfiguration=None, Description: str=None, Tags: List[Tag]=None) -> CreateWorkGroupOutput:
        """Creates a workgroup with the specified name.

        :param self:
        :param Tags:  One or more tags, separated by commas, that you want to attach to the workgroup as you create it.<br/>Description:  The workgroup description.<br/>Configuration:  The configuration for the workgroup, which includes the location in Amazon S3 where query results are stored, the encryption configuration, if any, used for encrypting query results, whether the Amazon CloudWatch Metrics are enabled for the workgroup, the limit for the amount of bytes scanned (cutoff) per query, if it is specified, and whether workgroup's settings (specified with EnforceWorkGroupConfiguration) in the WorkGroupConfiguration override client-side settings. See <a>WorkGroupConfiguration$EnforceWorkGroupConfiguration</a>.:<br/>Name:  The workgroup name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BytesScannedCutoffPerQuery</i> The upper data usage limit (cutoff) for the amount of bytes a single query in a workgroup is allowed to scan.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PublishCloudWatchMetricsEnabled</i> Indicates that the Amazon CloudWatch metrics are enabled for the workgroup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnforceWorkGroupConfiguration</i> If set to "true", the settings for the workgroup override client-side settings. If set to "false", client-side settings are used. For more information, see <a href="https://docs.aws.amazon.com/athena/latest/ug/workgroups-settings-override.html">Workgroup Settings Override Client-Side Settings</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResultConfiguration</i> The configuration for the workgroup, which includes the location in Amazon S3 where query results are stored and the encryption option, if any, used for query results.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResultConfiguration</b>: The configuration for the workgroup, which includes the location in Amazon S3 where query results are stored and the encryption option, if any, used for query results.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputLocation</i> The location in Amazon S3 where your query results are stored, such as <code>s3://path/to/query/bucket/</code>. For more information, see <a href="https://docs.aws.amazon.com/athena/latest/ug/querying.html">Queries and Query Result Files.</a> If workgroup settings override client-side settings, then the query uses the location for the query results and the encryption configuration that are specified for the workgroup. The "workgroup settings override" is specified in EnforceWorkGroupConfiguration (true/false) in the WorkGroupConfiguration. See <a>WorkGroupConfiguration$EnforceWorkGroupConfiguration</a>.<br/>If query results are encrypted in Amazon S3, indicates the encryption option used (for example, <code>SSE-KMS</code> or <code>CSE-KMS</code>) and key information. This is a client-side setting. If workgroup settings override client-side settings, then the query uses the encryption configuration that is specified for the workgroup, and also uses the location for storing query results specified in the workgroup. See <a>WorkGroupConfiguration$EnforceWorkGroupConfiguration</a> and <a href="https://docs.aws.amazon.com/athena/latest/ug/workgroups-settings-override.html">Workgroup Settings Override Client-Side Settings</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncryptionConfiguration</b>: If query results are encrypted in Amazon S3, indicates the encryption option used (for example, <code>SSE-KMS</code> or <code>CSE-KMS</code>) and key information. This is a client-side setting. If workgroup settings override client-side settings, then the query uses the encryption configuration that is specified for the workgroup, and also uses the location for storing query results specified in the workgroup. See <a>WorkGroupConfiguration$EnforceWorkGroupConfiguration</a> and <a href="https://docs.aws.amazon.com/athena/latest/ug/workgroups-settings-override.html">Workgroup Settings Override Client-Side Settings</a>.
        :return:         """
        pass

    def delete_named_query(self, NamedQueryId: str) -> DeleteNamedQueryOutput:
        """Deletes the named query if you have access to the workgroup in which the query was saved. For code samples using the AWS SDK for Java, see <a href="http://docs.aws.amazon.com/athena/latest/ug/code-samples.html">Examples and Code Samples</a> in the <i>Amazon Athena User Guide</i>.

        :param self:
        :param NamedQueryId:  The unique ID of the query to delete.<br/>
        :return:         """
        pass

    def delete_work_group(self, WorkGroup: str, RecursiveDeleteOption: BoxedBoolean=None) -> DeleteWorkGroupOutput:
        """Deletes the workgroup with the specified name. The primary workgroup cannot be deleted.

        :param self:
        :param RecursiveDeleteOption:  The option to delete the workgroup and its contents even if the workgroup contains any named queries.<br/>WorkGroup:  The unique name of the workgroup to delete.<br/>
        :return:         """
        pass

    def get_named_query(self, NamedQueryId: str) -> GetNamedQueryOutput:
        """Returns information about a single query. Requires that you have access to the workgroup in which the query was saved.

        :param self:
        :param NamedQueryId:  The unique ID of the query. Use <a>ListNamedQueries</a> to get query IDs.<br/>
        :return: NamedQuery:  Information about the query.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WorkGroup</i> The name of the workgroup that contains the named query.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NamedQueryId</i> The unique identifier of the query.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QueryString</i> The SQL query statements that comprise the query.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Database</i> The database to which the query belongs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The query description.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The query name.<br/>        """
        pass

    def get_query_execution(self, QueryExecutionId: str) -> GetQueryExecutionOutput:
        """Returns information about a single execution of a query if you have access to the workgroup in which the query ran. Each time a query executes, information about the query execution is saved with a unique ID.

        :param self:
        :param QueryExecutionId:  The unique ID of the query execution.<br/>
        :return: QueryExecution:  Information about the query execution.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WorkGroup</i> The name of the workgroup in which the query ran.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Statistics</i> The amount of data scanned during the query execution and the amount of time that it took to execute, and the type of statement that was run.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Statistics</b>: The amount of data scanned during the query execution and the amount of time that it took to execute, and the type of statement that was run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The completion date, current state, submission time, and state change reason (if applicable) for the query execution.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>: The completion date, current state, submission time, and state change reason (if applicable) for the query execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QueryExecutionContext</i> The database in which the query execution occurred.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>QueryExecutionContext</b>: The database in which the query execution occurred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResultConfiguration</i> The location in Amazon S3 where query results were stored and the encryption option, if any, used for query results. These are known as "client-side settings". If workgroup settings override client-side settings, then the query uses the location for the query results and the encryption configuration that are specified for the workgroup.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResultConfiguration</b>: The location in Amazon S3 where query results were stored and the encryption option, if any, used for query results. These are known as "client-side settings". If workgroup settings override client-side settings, then the query uses the location for the query results and the encryption configuration that are specified for the workgroup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatementType</i> The type of query statement that was run. <code>DDL</code> indicates DDL query statements. <code>DML</code> indicates DML (Data Manipulation Language) query statements, such as <code>CREATE TABLE AS SELECT</code>. <code>UTILITY</code> indicates query statements other than DDL and DML, such as <code>SHOW CREATE TABLE</code>, or <code>DESCRIBE &lt;table&gt;</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Query</i> The SQL query statements which the query execution ran.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QueryExecutionId</i> The unique identifier for each query execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputLocation</i> The location in Amazon S3 where your query results are stored, such as <code>s3://path/to/query/bucket/</code>. For more information, see <a href="https://docs.aws.amazon.com/athena/latest/ug/querying.html">Queries and Query Result Files.</a> If workgroup settings override client-side settings, then the query uses the location for the query results and the encryption configuration that are specified for the workgroup. The "workgroup settings override" is specified in EnforceWorkGroupConfiguration (true/false) in the WorkGroupConfiguration. See <a>WorkGroupConfiguration$EnforceWorkGroupConfiguration</a>.<br/>If query results are encrypted in Amazon S3, indicates the encryption option used (for example, <code>SSE-KMS</code> or <code>CSE-KMS</code>) and key information. This is a client-side setting. If workgroup settings override client-side settings, then the query uses the encryption configuration that is specified for the workgroup, and also uses the location for storing query results specified in the workgroup. See <a>WorkGroupConfiguration$EnforceWorkGroupConfiguration</a> and <a href="https://docs.aws.amazon.com/athena/latest/ug/workgroups-settings-override.html">Workgroup Settings Override Client-Side Settings</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncryptionConfiguration</b>: If query results are encrypted in Amazon S3, indicates the encryption option used (for example, <code>SSE-KMS</code> or <code>CSE-KMS</code>) and key information. This is a client-side setting. If workgroup settings override client-side settings, then the query uses the encryption configuration that is specified for the workgroup, and also uses the location for storing query results specified in the workgroup. See <a>WorkGroupConfiguration$EnforceWorkGroupConfiguration</a> and <a href="https://docs.aws.amazon.com/athena/latest/ug/workgroups-settings-override.html">Workgroup Settings Override Client-Side Settings</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Database</i> The name of the database.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CompletionDateTime</i> The date and time that the query completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmissionDateTime</i> The date and time that the query was submitted.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StateChangeReason</i> Further detail about the status of the query.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of query execution. <code>QUEUED</code> state is listed but is not used by Athena and is reserved for future use. <code>RUNNING</code> indicates that the query has been submitted to the service, and Athena will execute the query as soon as resources are available. <code>SUCCEEDED</code> indicates that the query completed without errors. <code>FAILED</code> indicates that the query experienced an error and did not complete processing. <code>CANCELLED</code> indicates that a user input interrupted query execution. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DataScannedInBytes</i> The number of bytes in the data that was queried.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineExecutionTimeInMillis</i> The number of milliseconds that the query took to execute.<br/>        """
        pass

    def get_query_results(self, QueryExecutionId: str, NextToken: str=None, MaxResults: int=None) -> GetQueryResultsOutput:
        """Returns the results of a single query execution specified by <code>QueryExecutionId</code> if you have access to the workgroup in which the query ran. This request does not execute the query but returns results. Use <a>StartQueryExecution</a> to run a query.

        :param self:
        :param MaxResults:  The maximum number of results (rows) to return in this request.<br/>NextToken:  The token that specifies where to start pagination if a previous request was truncated.<br/>QueryExecutionId:  The unique ID of the query execution.<br/>
        :return: NextToken:  A token to be used by the next request if this request is truncated.<br/>ResultSet:  The results of the query execution.:<br/>UpdateCount:  The number of rows inserted with a CREATE TABLE AS SELECT statement. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResultSetMetadata</i> The metadata that describes the column structure and data types of a table of query results.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResultSetMetadata</b>: The metadata that describes the column structure and data types of a table of query results.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Rows</i> The rows in the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ColumnInfo</i> Information about the columns returned in a query result metadata.<br/>        """
        pass

    def get_work_group(self, WorkGroup: str) -> GetWorkGroupOutput:
        """Returns information about the workgroup with the specified name.

        :param self:
        :param WorkGroup:  The name of the workgroup.<br/>
        :return: WorkGroup:  Information about the workgroup.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The date and time the workgroup was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The workgroup description.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Configuration</i> The configuration of the workgroup, which includes the location in Amazon S3 where query results are stored, the encryption configuration, if any, used for query results; whether the Amazon CloudWatch Metrics are enabled for the workgroup; whether workgroup settings override client-side settings; and the data usage limit for the amount of data scanned per query, if it is specified. The workgroup settings override is specified in EnforceWorkGroupConfiguration (true/false) in the WorkGroupConfiguration. See <a>WorkGroupConfiguration$EnforceWorkGroupConfiguration</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Configuration</b>: The configuration of the workgroup, which includes the location in Amazon S3 where query results are stored, the encryption configuration, if any, used for query results; whether the Amazon CloudWatch Metrics are enabled for the workgroup; whether workgroup settings override client-side settings; and the data usage limit for the amount of data scanned per query, if it is specified. The workgroup settings override is specified in EnforceWorkGroupConfiguration (true/false) in the WorkGroupConfiguration. See <a>WorkGroupConfiguration$EnforceWorkGroupConfiguration</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the workgroup: ENABLED or DISABLED.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The workgroup name.<br/>The configuration for the workgroup, which includes the location in Amazon S3 where query results are stored and the encryption option, if any, used for query results.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResultConfiguration</b>: The configuration for the workgroup, which includes the location in Amazon S3 where query results are stored and the encryption option, if any, used for query results.        """
        pass

    def list_named_queries(self, NextToken: str=None, MaxResults: int=None, WorkGroup: str=None) -> ListNamedQueriesOutput:
        """Provides a list of available query IDs only for queries saved in the specified workgroup. Requires that you have access to the workgroup. For code samples using the AWS SDK for Java, see <a href="http://docs.aws.amazon.com/athena/latest/ug/code-samples.html">Examples and Code Samples</a> in the <i>Amazon Athena User Guide</i>.

        :param self:
        :param WorkGroup:  The name of the workgroup from which the named queries are being returned.<br/>MaxResults:  The maximum number of queries to return in this request.<br/>NextToken:  The token that specifies where to start pagination if a previous request was truncated.<br/>
        :return: NextToken:  A token to be used by the next request if this request is truncated.<br/>NamedQueryIds:  The list of unique query IDs.<br/>        """
        pass

    def list_query_executions(self, NextToken: str=None, MaxResults: int=None, WorkGroup: str=None) -> ListQueryExecutionsOutput:
        """Provides a list of available query execution IDs for the queries in the specified workgroup. Requires you to have access to the workgroup in which the queries ran. For code samples using the AWS SDK for Java, see <a href="http://docs.aws.amazon.com/athena/latest/ug/code-samples.html">Examples and Code Samples</a> in the <i>Amazon Athena User Guide</i>.

        :param self:
        :param WorkGroup:  The name of the workgroup from which queries are being returned.<br/>MaxResults:  The maximum number of query executions to return in this request.<br/>NextToken:  The token that specifies where to start pagination if a previous request was truncated.<br/>
        :return: NextToken:  A token to be used by the next request if this request is truncated.<br/>QueryExecutionIds:  The unique IDs of each query execution as an array of strings.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceARN: str, NextToken: str=None, MaxResults: int=None) -> ListTagsForResourceOutput:
        """Lists the tags associated with this workgroup.

        :param self:
        :param MaxResults:  The maximum number of results to be returned per request that lists the tags for the workgroup resource.<br/>NextToken:  The token for the next set of results, or null if there are no additional results for this request, where the request lists the tags for the workgroup resource with the specified ARN.<br/>ResourceARN:  Lists the tags for the workgroup resource with the specified ARN.<br/>
        :return: NextToken:  A token to be used by the next request if this request is truncated.<br/>Tags:  The list of tags associated with this workgroup.<br/>        """
        pass

    def list_work_groups(self, NextToken: str=None, MaxResults: int=None) -> ListWorkGroupsOutput:
        """Lists available workgroups for the account.

        :param self:
        :param MaxResults:  The maximum number of workgroups to return in this request.<br/>NextToken:  A token to be used by the next request if this request is truncated.<br/>
        :return: NextToken:  A token to be used by the next request if this request is truncated.<br/>WorkGroups:  The list of workgroups, including their names, descriptions, creation times, and states.<br/>        """
        pass

    def start_query_execution(self, QueryString: str, ClientRequestToken: str=None, QueryExecutionContext: QueryExecutionContext=None, ResultConfiguration: ResultConfiguration=None, WorkGroup: str=None) -> StartQueryExecutionOutput:
        """Runs the SQL query statements contained in the <code>Query</code>. Requires you to have access to the workgroup in which the query ran. For code samples using the AWS SDK for Java, see <a href="http://docs.aws.amazon.com/athena/latest/ug/code-samples.html">Examples and Code Samples</a> in the <i>Amazon Athena User Guide</i>.

        :param self:
        :param WorkGroup:  The name of the workgroup in which the query is being started.<br/>ResultConfiguration:  Specifies information about where and how to save the results of the query execution. If the query runs in a workgroup, then workgroup's settings may override query settings. This affects the query results location. The workgroup settings override is specified in EnforceWorkGroupConfiguration (true/false) in the WorkGroupConfiguration. See <a>WorkGroupConfiguration$EnforceWorkGroupConfiguration</a>.:<br/>QueryExecutionContext:  The database within which the query executes.:<br/>ClientRequestToken:  A unique case-sensitive string used to ensure the request to create the query is idempotent (executes only once). If another <code>StartQueryExecution</code> request is received, the same response is returned and another query is not created. If a parameter has changed, for example, the <code>QueryString</code>, an error is returned. <important> This token is listed as not required because AWS SDKs (for example the AWS SDK for Java) auto-generate the token for users. If you are not using the AWS SDK or the AWS CLI, you must provide this token or the action will fail. </important><br/>QueryString:  The SQL query statements to be executed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Database</i> The name of the database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionConfiguration</i> If query results are encrypted in Amazon S3, indicates the encryption option used (for example, <code>SSE-KMS</code> or <code>CSE-KMS</code>) and key information. This is a client-side setting. If workgroup settings override client-side settings, then the query uses the encryption configuration that is specified for the workgroup, and also uses the location for storing query results specified in the workgroup. See <a>WorkGroupConfiguration$EnforceWorkGroupConfiguration</a> and <a href="https://docs.aws.amazon.com/athena/latest/ug/workgroups-settings-override.html">Workgroup Settings Override Client-Side Settings</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncryptionConfiguration</b>: If query results are encrypted in Amazon S3, indicates the encryption option used (for example, <code>SSE-KMS</code> or <code>CSE-KMS</code>) and key information. This is a client-side setting. If workgroup settings override client-side settings, then the query uses the encryption configuration that is specified for the workgroup, and also uses the location for storing query results specified in the workgroup. See <a>WorkGroupConfiguration$EnforceWorkGroupConfiguration</a> and <a href="https://docs.aws.amazon.com/athena/latest/ug/workgroups-settings-override.html">Workgroup Settings Override Client-Side Settings</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputLocation</i> The location in Amazon S3 where your query results are stored, such as <code>s3://path/to/query/bucket/</code>. For more information, see <a href="https://docs.aws.amazon.com/athena/latest/ug/querying.html">Queries and Query Result Files.</a> If workgroup settings override client-side settings, then the query uses the location for the query results and the encryption configuration that are specified for the workgroup. The "workgroup settings override" is specified in EnforceWorkGroupConfiguration (true/false) in the WorkGroupConfiguration. See <a>WorkGroupConfiguration$EnforceWorkGroupConfiguration</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKey</i> For <code>SSE-KMS</code> and <code>CSE-KMS</code>, this is the KMS key ARN or ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionOption</i> Indicates whether Amazon S3 server-side encryption with Amazon S3-managed keys (<code>SSE-S3</code>), server-side encryption with KMS-managed keys (<code>SSE-KMS</code>), or client-side encryption with KMS-managed keys (CSE-KMS) is used. If a query runs in a workgroup and the workgroup overrides client-side settings, then the workgroup's setting for encryption is used. It specifies whether query results must be encrypted, for all queries that run in this workgroup. <br/>
        :return: QueryExecutionId:  The unique ID of the query that ran as a result of this request.<br/>        """
        pass

    def stop_query_execution(self, QueryExecutionId: str) -> StopQueryExecutionOutput:
        """Stops a query execution. Requires you to have access to the workgroup in which the query ran. For code samples using the AWS SDK for Java, see <a href="http://docs.aws.amazon.com/athena/latest/ug/code-samples.html">Examples and Code Samples</a> in the <i>Amazon Athena User Guide</i>.

        :param self:
        :param QueryExecutionId:  The unique ID of the query execution to stop.<br/>
        :return:         """
        pass

    def tag_resource(self, Tags: List[Tag], ResourceARN: str) -> TagResourceOutput:
        """Adds one or more tags to the resource, such as a workgroup. A tag is a label that you assign to an AWS Athena resource (a workgroup). Each tag consists of a key and an optional value, both of which you define. Tags enable you to categorize resources (workgroups) in Athena, for example, by purpose, owner, or environment. Use a consistent set of tag keys to make it easier to search and filter workgroups in your account. For best practices, see <a href="https://aws.amazon.com/answers/account-management/aws-tagging-strategies/">AWS Tagging Strategies</a>. The key length is from 1 (minimum) to 128 (maximum) Unicode characters in UTF-8. The tag value length is from 0 (minimum) to 256 (maximum) Unicode characters in UTF-8. You can use letters and numbers representable in UTF-8, and the following characters: + - = . _ : / @. Tag keys and values are case-sensitive. Tag keys must be unique per resource. If you specify more than one, separate them by commas.

        :param self:
        :param Tags:  One or more tags, separated by commas, to be added to the resource, such as a workgroup.<br/>ResourceARN:  Requests that one or more tags are added to the resource (such as a workgroup) for the specified ARN.<br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[TagKey], ResourceARN: str) -> UntagResourceOutput:
        """Removes one or more tags from the workgroup resource. Takes as an input a list of TagKey Strings separated by commas, and removes their tags at the same time.

        :param self:
        :param TagKeys:  Removes the tags associated with one or more tag keys from the workgroup resource.<br/>ResourceARN:  Removes one or more tags from the workgroup resource for the specified ARN.<br/>
        :return:         """
        pass

    def update_work_group(self, WorkGroup: str, Description: str=None, ConfigurationUpdates: WorkGroupConfigurationUpdates=None, State: str=None) -> UpdateWorkGroupOutput:
        """Updates the workgroup with the specified name. The workgroup's name cannot be changed.

        :param self:
        :param State:  The workgroup state that will be updated for the given workgroup.<br/>ConfigurationUpdates:  The workgroup configuration that will be updated for the given workgroup.:<br/>Description:  The workgroup description.<br/>WorkGroup:  The specified workgroup that will be updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RemoveBytesScannedCutoffPerQuery</i> Indicates that the data usage control limit per query is removed. <a>WorkGroupConfiguration$BytesScannedCutoffPerQuery</a> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BytesScannedCutoffPerQuery</i> The upper limit (cutoff) for the amount of bytes a single query in a workgroup is allowed to scan.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PublishCloudWatchMetricsEnabled</i> Indicates whether this workgroup enables publishing metrics to Amazon CloudWatch.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResultConfigurationUpdates</i> The result configuration information about the queries in this workgroup that will be updated. Includes the updated results location and an updated option for encrypting query results.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResultConfigurationUpdates</b>: The result configuration information about the queries in this workgroup that will be updated. Includes the updated results location and an updated option for encrypting query results.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnforceWorkGroupConfiguration</i> If set to "true", the settings for the workgroup override client-side settings. If set to "false" client-side settings are used. For more information, see <a href="https://docs.aws.amazon.com/athena/latest/ug/workgroups-settings-override.html">Workgroup Settings Override Client-Side Settings</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RemoveOutputLocation</i> If set to "true", indicates that the previously-specified query results location (also known as a client-side setting) for queries in this workgroup should be ignored and set to null. If set to "false" or not set, and a value is present in the OutputLocation in ResultConfigurationUpdates (the client-side setting), the OutputLocation in the workgroup's ResultConfiguration will be updated with the new value. For more information, see <a href="https://docs.aws.amazon.com/athena/latest/ug/workgroups-settings-override.html">Workgroup Settings Override Client-Side Settings</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputLocation</i> The location in Amazon S3 where your query results are stored, such as <code>s3://path/to/query/bucket/</code>. For more information, see <a href="https://docs.aws.amazon.com/athena/latest/ug/querying.html">Queries and Query Result Files.</a> If workgroup settings override client-side settings, then the query uses the location for the query results and the encryption configuration that are specified for the workgroup. The "workgroup settings override" is specified in EnforceWorkGroupConfiguration (true/false) in the WorkGroupConfiguration. See <a>WorkGroupConfiguration$EnforceWorkGroupConfiguration</a>.<br/>The encryption configuration for the query results.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncryptionConfiguration</b>: The encryption configuration for the query results.
        :return:         """
        pass



class AutoScaling(BaseClient):
    """<fullname>Amazon EC2 Auto Scaling</fullname> Amazon EC2 Auto Scaling is designed to automatically launch or terminate EC2 instances based on user-defined policies, schedules, and health checks. Use this service with AWS Auto Scaling, Amazon CloudWatch, and Elastic Load Balancing. For more information, including information about granting IAM users required permissions for Amazon EC2 Auto Scaling actions, see the <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/what-is-amazon-ec2-auto-scaling.html">Amazon EC2 Auto Scaling User Guide</a>."""


    class DescribeAutoScalingGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_auto_scaling_groups` for valid parameters"""
            pass
    class DescribeAutoScalingInstancesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_auto_scaling_instances` for valid parameters"""
            pass
    class DescribeLaunchConfigurationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_launch_configurations` for valid parameters"""
            pass
    class DescribeNotificationConfigurationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_notification_configurations` for valid parameters"""
            pass
    class DescribePoliciesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_policies` for valid parameters"""
            pass
    class DescribeScalingActivitiesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_scaling_activities` for valid parameters"""
            pass
    class DescribeScheduledActionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_scheduled_actions` for valid parameters"""
            pass
    class DescribeTagsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_tags` for valid parameters"""
            pass
    class DescribeLoadBalancerTargetGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_load_balancer_target_groups` for valid parameters"""
            pass
    class DescribeLoadBalancersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_load_balancers` for valid parameters"""
            pass


    class AttachInstancesQuery(Mapping):
        pass
    
    class XmlStringMaxLen19(object):
        pass
    
    class AttachLoadBalancerTargetGroupsType(Mapping):
        pass
    
    class AttachLoadBalancerTargetGroupsResultType(Mapping):
        pass
    
    class XmlStringMaxLen511(object):
        pass
    
    class AttachLoadBalancersType(Mapping):
        pass
    
    class AttachLoadBalancersResultType(Mapping):
        pass
    
    class XmlStringMaxLen255(object):
        pass
    
    class BatchDeleteScheduledActionType(Mapping):
        pass
    
    class BatchDeleteScheduledActionAnswer(Mapping):
        pass
    
    class ResourceName(object):
        pass
    
    class BatchPutScheduledUpdateGroupActionType(Mapping):
        pass
    
    class BatchPutScheduledUpdateGroupActionAnswer(Mapping):
        pass
    
    class ScheduledUpdateGroupActionRequest(Mapping):
        pass
    
    class CompleteLifecycleActionType(Mapping):
        pass
    
    class CompleteLifecycleActionAnswer(Mapping):
        pass
    
    class CreateAutoScalingGroupType(Mapping):
        pass
    
    class LaunchTemplateSpecification(Mapping):
        pass
    
    class MixedInstancesPolicy(Mapping):
        pass
    
    class XmlStringMaxLen1600(object):
        pass
    
    class InstanceProtected(object):
        pass
    
    class LifecycleHookSpecification(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreateLaunchConfigurationType(Mapping):
        pass
    
    class XmlString(object):
        pass
    
    class BlockDeviceMapping(Mapping):
        pass
    
    class InstanceMonitoring(Mapping):
        pass
    
    class EbsOptimized(object):
        pass
    
    class AssociatePublicIpAddress(object):
        pass
    
    class CreateOrUpdateTagsType(Mapping):
        pass
    
    class DeleteAutoScalingGroupType(Mapping):
        pass
    
    class ForceDelete(object):
        pass
    
    class LaunchConfigurationNameType(Mapping):
        pass
    
    class DeleteLifecycleHookType(Mapping):
        pass
    
    class DeleteLifecycleHookAnswer(Mapping):
        pass
    
    class DeleteNotificationConfigurationType(Mapping):
        pass
    
    class DeletePolicyType(Mapping):
        pass
    
    class DeleteScheduledActionType(Mapping):
        pass
    
    class DeleteTagsType(Mapping):
        pass
    
    class DescribeAccountLimitsAnswer(Mapping):
        pass
    
    class DescribeAdjustmentTypesAnswer(Mapping):
        pass
    
    class AutoScalingGroupNamesType(Mapping):
        pass
    
    class AutoScalingGroupsType(Mapping):
        pass
    
    class DescribeAutoScalingInstancesType(Mapping):
        pass
    
    class AutoScalingInstancesType(Mapping):
        pass
    
    class DescribeAutoScalingNotificationTypesAnswer(Mapping):
        pass
    
    class LaunchConfigurationNamesType(Mapping):
        pass
    
    class LaunchConfigurationsType(Mapping):
        pass
    
    class DescribeLifecycleHookTypesAnswer(Mapping):
        pass
    
    class DescribeLifecycleHooksType(Mapping):
        pass
    
    class DescribeLifecycleHooksAnswer(Mapping):
        pass
    
    class AsciiStringMaxLen255(object):
        pass
    
    class DescribeLoadBalancerTargetGroupsRequest(Mapping):
        pass
    
    class DescribeLoadBalancerTargetGroupsResponse(Mapping):
        pass
    
    class DescribeLoadBalancersRequest(Mapping):
        pass
    
    class DescribeLoadBalancersResponse(Mapping):
        pass
    
    class DescribeMetricCollectionTypesAnswer(Mapping):
        pass
    
    class DescribeNotificationConfigurationsType(Mapping):
        pass
    
    class DescribeNotificationConfigurationsAnswer(Mapping):
        pass
    
    class DescribePoliciesType(Mapping):
        pass
    
    class PoliciesType(Mapping):
        pass
    
    class XmlStringMaxLen64(object):
        pass
    
    class DescribeScalingActivitiesType(Mapping):
        pass
    
    class ActivitiesType(Mapping):
        pass
    
    class ProcessesType(Mapping):
        pass
    
    class DescribeScheduledActionsType(Mapping):
        pass
    
    class ScheduledActionsType(Mapping):
        pass
    
    class TimestampType(object):
        pass
    
    class DescribeTagsType(Mapping):
        pass
    
    class TagsType(Mapping):
        pass
    
    class Filter(Mapping):
        pass
    
    class DescribeTerminationPolicyTypesAnswer(Mapping):
        pass
    
    class DetachInstancesQuery(Mapping):
        pass
    
    class DetachInstancesAnswer(Mapping):
        pass
    
    class ShouldDecrementDesiredCapacity(object):
        pass
    
    class DetachLoadBalancerTargetGroupsType(Mapping):
        pass
    
    class DetachLoadBalancerTargetGroupsResultType(Mapping):
        pass
    
    class DetachLoadBalancersType(Mapping):
        pass
    
    class DetachLoadBalancersResultType(Mapping):
        pass
    
    class DisableMetricsCollectionQuery(Mapping):
        pass
    
    class EnableMetricsCollectionQuery(Mapping):
        pass
    
    class EnterStandbyQuery(Mapping):
        pass
    
    class EnterStandbyAnswer(Mapping):
        pass
    
    class ExecutePolicyType(Mapping):
        pass
    
    class HonorCooldown(object):
        pass
    
    class MetricScale(object):
        pass
    
    class ExitStandbyQuery(Mapping):
        pass
    
    class ExitStandbyAnswer(Mapping):
        pass
    
    class PutLifecycleHookType(Mapping):
        pass
    
    class PutLifecycleHookAnswer(Mapping):
        pass
    
    class PutNotificationConfigurationType(Mapping):
        pass
    
    class PutScalingPolicyType(Mapping):
        pass
    
    class PolicyARNType(Mapping):
        pass
    
    class StepAdjustment(Mapping):
        pass
    
    class TargetTrackingConfiguration(Mapping):
        pass
    
    class PutScheduledUpdateGroupActionType(Mapping):
        pass
    
    class RecordLifecycleActionHeartbeatType(Mapping):
        pass
    
    class RecordLifecycleActionHeartbeatAnswer(Mapping):
        pass
    
    class ScalingProcessQuery(Mapping):
        pass
    
    class SetDesiredCapacityType(Mapping):
        pass
    
    class SetInstanceHealthQuery(Mapping):
        pass
    
    class ShouldRespectGracePeriod(object):
        pass
    
    class SetInstanceProtectionQuery(Mapping):
        pass
    
    class SetInstanceProtectionAnswer(Mapping):
        pass
    
    class ProtectedFromScaleIn(object):
        pass
    
    class TerminateInstanceInAutoScalingGroupType(Mapping):
        pass
    
    class ActivityType(Mapping):
        pass
    
    class UpdateAutoScalingGroupType(Mapping):
        pass
    
    def attach_instances(self, AutoScalingGroupName: str, InstanceIds: List[XmlStringMaxLen19]=None):
        """Attaches one or more EC2 instances to the specified Auto Scaling group. When you attach instances, Amazon EC2 Auto Scaling increases the desired capacity of the group by the number of instances being attached. If the number of instances being attached plus the desired capacity of the group exceeds the maximum size of the group, the operation fails. If there is a Classic Load Balancer attached to your Auto Scaling group, the instances are also registered with the load balancer. If there are target groups attached to your Auto Scaling group, the instances are also registered with the target groups. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/attach-instance-asg.html">Attach EC2 Instances to Your Auto Scaling Group</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.

        :param self:
        :param AutoScalingGroupName:  The name of the Auto Scaling group.<br/>InstanceIds:  The IDs of the instances. You can specify up to 20 instances.<br/>
        :return:         """
        pass

    def attach_load_balancer_target_groups(self, TargetGroupARNs: List[XmlStringMaxLen511], AutoScalingGroupName: str) -> AttachLoadBalancerTargetGroupsResultType:
        """Attaches one or more target groups to the specified Auto Scaling group.  To describe the target groups for an Auto Scaling group, use <a>DescribeLoadBalancerTargetGroups</a>. To detach the target group from the Auto Scaling group, use <a>DetachLoadBalancerTargetGroups</a>. With Application Load Balancers and Network Load Balancers, instances are registered as targets with a target group. With Classic Load Balancers, instances are registered with the load balancer. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/attach-load-balancer-asg.html">Attaching a Load Balancer to Your Auto Scaling Group</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.

        :param self:
        :param TargetGroupARNs:  The Amazon Resource Names (ARN) of the target groups. You can specify up to 10 target groups.<br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>
        :return:         """
        pass

    def attach_load_balancers(self, LoadBalancerNames: List[XmlStringMaxLen255], AutoScalingGroupName: str) -> AttachLoadBalancersResultType:
        """Attaches one or more Classic Load Balancers to the specified Auto Scaling group. To attach an Application Load Balancer or a Network Load Balancer instead, see <a>AttachLoadBalancerTargetGroups</a>. To describe the load balancers for an Auto Scaling group, use <a>DescribeLoadBalancers</a>. To detach the load balancer from the Auto Scaling group, use <a>DetachLoadBalancers</a>. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/attach-load-balancer-asg.html">Attaching a Load Balancer to Your Auto Scaling Group</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.

        :param self:
        :param LoadBalancerNames:  The names of the load balancers. You can specify up to 10 load balancers.<br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>
        :return:         """
        pass

    def batch_delete_scheduled_action(self, ScheduledActionNames: List[ResourceName], AutoScalingGroupName: str) -> BatchDeleteScheduledActionAnswer:
        """Deletes one or more scheduled actions for the specified Auto Scaling group.

        :param self:
        :param ScheduledActionNames:  The names of the scheduled actions to delete. The maximum number allowed is 50. <br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>
        :return: FailedScheduledActions:  The names of the scheduled actions that could not be deleted, including an error message. <br/>        """
        pass

    def batch_put_scheduled_update_group_action(self, ScheduledUpdateGroupActions: List[ScheduledUpdateGroupActionRequest], AutoScalingGroupName: str) -> BatchPutScheduledUpdateGroupActionAnswer:
        """Creates or updates one or more scheduled scaling actions for an Auto Scaling group. If you leave a parameter unspecified when updating a scheduled scaling action, the corresponding value remains unchanged.

        :param self:
        :param ScheduledUpdateGroupActions:  One or more scheduled actions. The maximum number allowed is 50. <br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>
        :return: FailedScheduledUpdateGroupActions:  The names of the scheduled actions that could not be created or updated, including an error message.<br/>        """
        pass

    def complete_lifecycle_action(self, LifecycleActionResult: str, AutoScalingGroupName: str, LifecycleHookName: str, LifecycleActionToken: str=None, InstanceId: str=None) -> CompleteLifecycleActionAnswer:
        """Completes the lifecycle action for the specified token or instance with the specified result. This step is a part of the procedure for adding a lifecycle hook to an Auto Scaling group: <ol> <li> (Optional) Create a Lambda function and a rule that allows CloudWatch Events to invoke your Lambda function when Amazon EC2 Auto Scaling launches or terminates instances. </li> <li> (Optional) Create a notification target and an IAM role. The target can be either an Amazon SQS queue or an Amazon SNS topic. The role allows Amazon EC2 Auto Scaling to publish lifecycle notifications to the target. </li> <li> Create the lifecycle hook. Specify whether the hook is used when the instances launch or terminate. </li> <li> If you need more time, record the lifecycle action heartbeat to keep the instance in a pending state. </li> <li>  <b>If you finish before the timeout period ends, complete the lifecycle action.</b>  </li> </ol> For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/lifecycle-hooks.html">Amazon EC2 Auto Scaling Lifecycle Hooks</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.

        :param self:
        :param InstanceId:  The ID of the instance.<br/>LifecycleActionResult:  The action for the group to take. This parameter can be either <code>CONTINUE</code> or <code>ABANDON</code>.<br/>LifecycleActionToken:  A universally unique identifier (UUID) that identifies a specific lifecycle action associated with an instance. Amazon EC2 Auto Scaling sends this token to the notification target you specified when you created the lifecycle hook.<br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>LifecycleHookName:  The name of the lifecycle hook.<br/>
        :return:         """
        pass

    def create_auto_scaling_group(self, MaxSize: int, MinSize: int, AutoScalingGroupName: str, LaunchConfigurationName: str=None, LaunchTemplate: LaunchTemplateSpecification=None, MixedInstancesPolicy: MixedInstancesPolicy=None, InstanceId: str=None, DesiredCapacity: int=None, DefaultCooldown: int=None, AvailabilityZones: List[XmlStringMaxLen255]=None, LoadBalancerNames: List[XmlStringMaxLen255]=None, TargetGroupARNs: List[XmlStringMaxLen511]=None, HealthCheckType: str=None, HealthCheckGracePeriod: int=None, PlacementGroup: str=None, VPCZoneIdentifier: str=None, TerminationPolicies: List[XmlStringMaxLen1600]=None, NewInstancesProtectedFromScaleIn: InstanceProtected=None, LifecycleHookSpecificationList: List[LifecycleHookSpecification]=None, Tags: List[Tag]=None, ServiceLinkedRoleARN: str=None):
        """Creates an Auto Scaling group with the specified name and attributes. If you exceed your maximum limit of Auto Scaling groups, the call fails. For information about viewing this limit, see <a>DescribeAccountLimits</a>. For information about updating this limit, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/as-account-limits.html">Amazon EC2 Auto Scaling Limits</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.

        :param self:
        :param ServiceLinkedRoleARN:  The Amazon Resource Name (ARN) of the service-linked role that the Auto Scaling group uses to call other AWS services on your behalf. By default, Amazon EC2 Auto Scaling uses a service-linked role named AWSServiceRoleForAutoScaling, which it creates if it does not exist. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/autoscaling-service-linked-role.html">Service-Linked Roles</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.<br/>Tags:  One or more tags. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/autoscaling-tagging.html">Tagging Auto Scaling Groups and Instances</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.<br/>LifecycleHookSpecificationList:  One or more lifecycle hooks.<br/>NewInstancesProtectedFromScaleIn:  Indicates whether newly launched instances are protected from termination by Amazon EC2 Auto Scaling when scaling in. For more information about preventing instances from terminating on scale in, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/as-instance-termination.html#instance-protection">Instance Protection</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.<br/>TerminationPolicies:  One or more termination policies used to select the instance to terminate. These policies are executed in the order that they are listed. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/as-instance-termination.html">Controlling Which Instances Auto Scaling Terminates During Scale In</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.<br/>VPCZoneIdentifier:  A comma-separated list of subnet IDs for your virtual private cloud (VPC). If you specify <code>VPCZoneIdentifier</code> with <code>AvailabilityZones</code>, the subnets that you specify for this parameter must reside in those Availability Zones. Conditional: If your account supports EC2-Classic and VPC, this parameter is required to launch instances into a VPC.<br/>PlacementGroup:  The name of the placement group into which to launch your instances, if any. A placement group is a logical grouping of instances within a single Availability Zone. You cannot specify multiple Availability Zones and a placement group. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/placement-groups.html">Placement Groups</a> in the <i>Amazon EC2 User Guide for Linux Instances</i>.<br/>HealthCheckGracePeriod:  The amount of time, in seconds, that Amazon EC2 Auto Scaling waits before checking the health status of an EC2 instance that has come into service. During this time, any health check failures for the instance are ignored. The default value is <code>0</code>. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/healthcheck.html">Health Checks for Auto Scaling Instances</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>. Conditional: This parameter is required if you are adding an <code>ELB</code> health check.<br/>HealthCheckType:  The service to use for the health checks. The valid values are <code>EC2</code> and <code>ELB</code>. The default value is <code>EC2</code>. If you configure an Auto Scaling group to use ELB health checks, it considers the instance unhealthy if it fails either the EC2 status checks or the load balancer health checks. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/healthcheck.html">Health Checks for Auto Scaling Instances</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.<br/>TargetGroupARNs:  The Amazon Resource Names (ARN) of the target groups to associate with the Auto Scaling group. Instances are registered as targets in a target group, and traffic is routed to the target group. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/autoscaling-load-balancer.html">Using a Load Balancer with an Auto Scaling Group</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.<br/>LoadBalancerNames:  A list of Classic Load Balancers associated with this Auto Scaling group. For Application Load Balancers and Network Load Balancers, specify a list of target groups using the <code>TargetGroupARNs</code> property instead. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/autoscaling-load-balancer.html">Using a Load Balancer with an Auto Scaling Group</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.<br/>AvailabilityZones:  One or more Availability Zones for the group. This parameter is optional if you specify one or more subnets for <code>VPCZoneIdentifier</code>. Conditional: If your account supports EC2-Classic and VPC, this parameter is required to launch instances into EC2-Classic.<br/>DefaultCooldown:  The amount of time, in seconds, after a scaling activity completes before another scaling activity can start. The default value is <code>300</code>. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/Cooldown.html">Scaling Cooldowns</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.<br/>DesiredCapacity:  The number of Amazon EC2 instances that the Auto Scaling group attempts to maintain. This number must be greater than or equal to the minimum size of the group and less than or equal to the maximum size of the group. If you do not specify a desired capacity, the default is the minimum size of the group.<br/>MaxSize:  The maximum size of the group.<br/>MinSize:  The minimum size of the group.<br/>InstanceId:  The ID of the instance used to create a launch configuration for the group. When you specify an ID of an instance, Amazon EC2 Auto Scaling creates a new launch configuration and associates it with the group. This launch configuration derives its attributes from the specified instance, except for the block device mapping. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/create-asg-from-instance.html">Create an Auto Scaling Group Using an EC2 Instance</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>. You must specify one of the following parameters in your request: <code>LaunchConfigurationName</code>, <code>LaunchTemplate</code>, <code>InstanceId</code>, or <code>MixedInstancesPolicy</code>.<br/>MixedInstancesPolicy:  An embedded object that specifies a mixed instances policy. The required parameters must be specified. If optional parameters are unspecified, their default values are used. The policy includes parameters that not only define the distribution of On-Demand Instances and Spot Instances, the maximum price to pay for Spot instances, and how the Auto Scaling group allocates instance types to fulfill On-Demand and Spot capacity, but also the parameters that specify the instance configuration information—the launch template and instance types. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/asg-purchase-options.html">Auto Scaling Groups with Multiple Instance Types and Purchase Options</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>. You must specify one of the following parameters in your request: <code>LaunchConfigurationName</code>, <code>LaunchTemplate</code>, <code>InstanceId</code>, or <code>MixedInstancesPolicy</code>.:<br/>LaunchTemplate:  The launch template to use to launch instances. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/create-asg-launch-template.html">Creating an Auto Scaling Group Using a Launch Template</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>. If you do not specify <code>LaunchTemplate</code>, you must specify one of the following parameters: <code>InstanceId</code>, <code>LaunchConfigurationName</code>, or <code>MixedInstancesPolicy</code>.:<br/>LaunchConfigurationName:  The name of the launch configuration. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/create-asg.html">Creating an Auto Scaling Group Using a Launch Configuration</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>. If you do not specify <code>LaunchConfigurationName</code>, you must specify one of the following parameters: <code>InstanceId</code>, <code>LaunchTemplate</code>, or <code>MixedInstancesPolicy</code>.<br/>AutoScalingGroupName:  The name of the Auto Scaling group. This name must be unique per Region per account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version number, <code>$Latest</code>, or <code>$Default</code>. If the value is <code>$Latest</code>, Amazon EC2 Auto Scaling selects the latest version of the launch template when launching instances. If the value is <code>$Default</code>, Amazon EC2 Auto Scaling selects the default version of the launch template when launching instances. The default value is <code>$Default</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LaunchTemplateName</i> The name of the launch template. You must specify either a template name or a template ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LaunchTemplateId</i> The ID of the launch template. You must specify either a template ID or a template name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstancesDistribution</i> The instances distribution to use. If you leave this parameter unspecified when creating a mixed instances policy, the default values are used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InstancesDistribution</b>: The instances distribution to use. If you leave this parameter unspecified when creating a mixed instances policy, the default values are used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LaunchTemplate</i> The launch template and instance types (overrides). This parameter must be specified when creating a mixed instances policy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LaunchTemplate</b>: The launch template and instance types (overrides). This parameter must be specified when creating a mixed instances policy.<br/>The launch template to use. You must specify either the launch template ID or launch template name in the request. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LaunchTemplateSpecification</b>: The launch template to use. You must specify either the launch template ID or launch template name in the request. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SpotMaxPrice</i> The maximum price per unit hour that you are willing to pay for a Spot Instance. If you leave the value of this parameter blank (which is the default), the maximum Spot price is set at the On-Demand price. To remove a value that you previously set, include the parameter but leave the value blank.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SpotInstancePools</i> The number of Spot pools to use to allocate your Spot capacity. The Spot pools are determined from the different instance types in the Overrides array of <a>LaunchTemplate</a>. The range is 1–20. The default value is <code>2</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SpotAllocationStrategy</i> Indicates how to allocate Spot capacity across Spot pools. The only valid value is <code>lowest-price</code>, which is also the default value. The Auto Scaling group selects the cheapest Spot pools and evenly allocates your Spot capacity across the number of Spot pools that you specify. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OnDemandPercentageAboveBaseCapacity</i> Controls the percentages of On-Demand Instances and Spot Instances for your additional capacity beyond <code>OnDemandBaseCapacity</code>. The range is 0–100. The default value is <code>100</code>. If you leave this parameter set to <code>100</code>, the percentages are 100% for On-Demand Instances and 0% for Spot Instances. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OnDemandBaseCapacity</i> The minimum amount of the Auto Scaling group's capacity that must be fulfilled by On-Demand Instances. This base portion is provisioned first as your group scales. The default value is <code>0</code>. If you leave this parameter set to <code>0</code>, On-Demand Instances are launched as a percentage of the Auto Scaling group's desired capacity, per the <code>OnDemandPercentageAboveBaseCapacity</code> setting.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OnDemandAllocationStrategy</i> Indicates how to allocate instance types to fulfill On-Demand capacity.  The only valid value is <code>prioritized</code>, which is also the default value. This strategy uses the order of instance type overrides for the <a>LaunchTemplate</a> to define the launch priority of each instance type. The first instance type in the array is prioritized higher than the last. If all your On-Demand capacity cannot be fulfilled using your highest priority instance, then the Auto Scaling groups launches the remaining capacity using the second priority instance type, and so on. <br/>
        :return:         """
        pass

    def create_launch_configuration(self, LaunchConfigurationName: str, ImageId: str=None, KeyName: str=None, SecurityGroups: List[XmlString]=None, ClassicLinkVPCId: str=None, ClassicLinkVPCSecurityGroups: List[XmlStringMaxLen255]=None, UserData: str=None, InstanceId: str=None, InstanceType: str=None, KernelId: str=None, RamdiskId: str=None, BlockDeviceMappings: List[BlockDeviceMapping]=None, InstanceMonitoring: InstanceMonitoring=None, SpotPrice: str=None, IamInstanceProfile: str=None, EbsOptimized: EbsOptimized=None, AssociatePublicIpAddress: AssociatePublicIpAddress=None, PlacementTenancy: str=None):
        """Creates a launch configuration. If you exceed your maximum limit of launch configurations, the call fails. For information about viewing this limit, see <a>DescribeAccountLimits</a>. For information about updating this limit, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/as-account-limits.html">Amazon EC2 Auto Scaling Limits</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/LaunchConfiguration.html">Launch Configurations</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.

        :param self:
        :param PlacementTenancy:  The tenancy of the instance. An instance with a tenancy of <code>dedicated</code> runs on single-tenant hardware and can only be launched into a VPC. To launch Dedicated Instances into a shared tenancy VPC (a VPC with the instance placement tenancy attribute set to <code>default</code>), you must set the value of this parameter to <code>dedicated</code>. If you specify <code>PlacementTenancy</code>, be sure to specify at least one subnet when you create your group. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/asg-in-vpc.html#as-vpc-tenancy">Instance Placement Tenancy</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>. Valid values: <code>default</code> | <code>dedicated</code> <br/>AssociatePublicIpAddress:  Used for groups that launch instances into a virtual private cloud (VPC). Specifies whether to assign a public IP address to each instance. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/asg-in-vpc.html">Launching Auto Scaling Instances in a VPC</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>. If you specify this parameter, be sure to specify at least one subnet when you create your group. Default: If the instance is launched into a default subnet, the default is to assign a public IP address. If the instance is launched into a nondefault subnet, the default is not to assign a public IP address.<br/>EbsOptimized:  Indicates whether the instance is optimized for Amazon EBS I/O. By default, the instance is not optimized for EBS I/O. The optimization provides dedicated throughput to Amazon EBS and an optimized configuration stack to provide optimal I/O performance. This optimization is not available with all instance types. Additional usage charges apply. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSOptimized.html">Amazon EBS-Optimized Instances</a> in the <i>Amazon EC2 User Guide for Linux Instances</i>.<br/>IamInstanceProfile:  The name or the Amazon Resource Name (ARN) of the instance profile associated with the IAM role for the instance. EC2 instances launched with an IAM role automatically have AWS security credentials available. You can use IAM roles with Amazon EC2 Auto Scaling to automatically enable applications running on your EC2 instances to securely access other AWS resources. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/us-iam-role.html">IAM Role for Applications That Run on Amazon EC2 Instances</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.<br/>SpotPrice:  The maximum hourly price to be paid for any Spot Instance launched to fulfill the request. Spot Instances are launched when the price you specify exceeds the current Spot market price. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/asg-launch-spot-instances.html">Launching Spot Instances in Your Auto Scaling Group</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.<br/>InstanceMonitoring:  Enables detailed monitoring (<code>true</code>) or basic monitoring (<code>false</code>) for the Auto Scaling instances. The default value is <code>true</code>.:<br/>BlockDeviceMappings:  One or more mappings that specify how block devices are exposed to the instance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/block-device-mapping-concepts.html">Block Device Mapping</a> in the <i>Amazon EC2 User Guide for Linux Instances</i>.<br/>RamdiskId:  The ID of the RAM disk associated with the AMI.<br/>KernelId:  The ID of the kernel associated with the AMI.<br/>InstanceType:  The instance type of the EC2 instance. For information about available instance types, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html#AvailableInstanceTypes">Available Instance Types</a> in the <i>Amazon EC2 User Guide for Linux Instances.</i>  If you do not specify <code>InstanceId</code>, you must specify <code>InstanceType</code>.<br/>InstanceId:  The ID of the instance to use to create the launch configuration. The new launch configuration derives attributes from the instance, except for the block device mapping. To create a launch configuration with a block device mapping or override any other instance attributes, specify them as part of the same request. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/create-lc-with-instanceID.html">Create a Launch Configuration Using an EC2 Instance</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>. If you do not specify <code>InstanceId</code>, you must specify both <code>ImageId</code> and <code>InstanceType</code>.<br/>UserData:  The user data to make available to the launched EC2 instances. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-instance-metadata.html">Instance Metadata and User Data</a> in the <i>Amazon EC2 User Guide for Linux Instances</i>.<br/>ClassicLinkVPCSecurityGroups:  The IDs of one or more security groups for the specified ClassicLink-enabled VPC. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/vpc-classiclink.html">ClassicLink</a> in the <i>Amazon EC2 User Guide for Linux Instances</i> and <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/asg-in-vpc.html#as-ClassicLink">Linking EC2-Classic Instances to a VPC</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>. Conditional: This parameter is required if you specify a ClassicLink-enabled VPC, and is not supported otherwise.<br/>ClassicLinkVPCId:  The ID of a ClassicLink-enabled VPC to link your EC2-Classic instances to. This parameter is supported only if you are launching EC2-Classic instances. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/vpc-classiclink.html">ClassicLink</a> in the <i>Amazon EC2 User Guide for Linux Instances</i> and <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/asg-in-vpc.html#as-ClassicLink">Linking EC2-Classic Instances to a VPC</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.<br/>SecurityGroups:  One or more security groups with which to associate the instances. If your instances are launched in EC2-Classic, you can either specify security group names or the security group IDs. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/using-network-security.html">Amazon EC2 Security Groups</a> in the <i>Amazon EC2 User Guide for Linux Instances</i>. If your instances are launched into a VPC, specify security group IDs. For more information, see <a href="https://docs.aws.amazon.com/AmazonVPC/latest/UserGuide/VPC_SecurityGroups.html">Security Groups for Your VPC</a> in the <i>Amazon Virtual Private Cloud User Guide</i>.<br/>KeyName:  The name of the key pair. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-key-pairs.html">Amazon EC2 Key Pairs</a> in the <i>Amazon EC2 User Guide for Linux Instances</i>.<br/>ImageId:  The ID of the Amazon Machine Image (AMI) to use to launch your EC2 instances. If you do not specify <code>InstanceId</code>, you must specify <code>ImageId</code>. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/finding-an-ami.html">Finding an AMI</a> in the <i>Amazon EC2 User Guide for Linux Instances</i>.<br/>LaunchConfigurationName:  The name of the launch configuration. This name must be unique per Region per account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> If <code>true</code>, detailed monitoring is enabled. Otherwise, basic monitoring is enabled.<br/>
        :return:         """
        pass

    def create_or_update_tags(self, Tags: List[Tag]):
        """Creates or updates tags for the specified Auto Scaling group. When you specify a tag with a key that already exists, the operation overwrites the previous tag definition, and you do not get an error message. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/autoscaling-tagging.html">Tagging Auto Scaling Groups and Instances</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.

        :param self:
        :param Tags:  One or more tags.<br/>
        :return:         """
        pass

    def delete_auto_scaling_group(self, AutoScalingGroupName: str, ForceDelete: ForceDelete=None):
        """Deletes the specified Auto Scaling group. If the group has instances or scaling activities in progress, you must specify the option to force the deletion in order for it to succeed. If the group has policies, deleting the group deletes the policies, the underlying alarm actions, and any alarm that no longer has an associated action. To remove instances from the Auto Scaling group before deleting it, call <a>DetachInstances</a> with the list of instances and the option to decrement the desired capacity. This ensures that Amazon EC2 Auto Scaling does not launch replacement instances. To terminate all instances before deleting the Auto Scaling group, call <a>UpdateAutoScalingGroup</a> and set the minimum size and desired capacity of the Auto Scaling group to zero.

        :param self:
        :param ForceDelete:  Specifies that the group is to be deleted along with all instances associated with the group, without waiting for all instances to be terminated. This parameter also deletes any lifecycle actions associated with the group.<br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>
        :return:         """
        pass

    def delete_launch_configuration(self, LaunchConfigurationName: str):
        """Deletes the specified launch configuration. The launch configuration must not be attached to an Auto Scaling group. When this call completes, the launch configuration is no longer available for use.

        :param self:
        :param LaunchConfigurationName:  The name of the launch configuration.<br/>
        :return:         """
        pass

    def delete_lifecycle_hook(self, AutoScalingGroupName: str, LifecycleHookName: str) -> DeleteLifecycleHookAnswer:
        """Deletes the specified lifecycle hook. If there are any outstanding lifecycle actions, they are completed first (<code>ABANDON</code> for launching instances, <code>CONTINUE</code> for terminating instances).

        :param self:
        :param AutoScalingGroupName:  The name of the Auto Scaling group.<br/>LifecycleHookName:  The name of the lifecycle hook.<br/>
        :return:         """
        pass

    def delete_notification_configuration(self, TopicARN: str, AutoScalingGroupName: str):
        """Deletes the specified notification.

        :param self:
        :param TopicARN:  The Amazon Resource Name (ARN) of the Amazon Simple Notification Service (Amazon SNS) topic.<br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>
        :return:         """
        pass

    def delete_policy(self, PolicyName: str, AutoScalingGroupName: str=None):
        """Deletes the specified scaling policy. Deleting either a step scaling policy or a simple scaling policy deletes the underlying alarm action, but does not delete the alarm, even if it no longer has an associated action. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/deleting-scaling-policy.html">Deleting a Scaling Policy</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.

        :param self:
        :param PolicyName:  The name or Amazon Resource Name (ARN) of the policy.<br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>
        :return:         """
        pass

    def delete_scheduled_action(self, ScheduledActionName: str, AutoScalingGroupName: str):
        """Deletes the specified scheduled action.

        :param self:
        :param ScheduledActionName:  The name of the action to delete.<br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>
        :return:         """
        pass

    def delete_tags(self, Tags: List[Tag]):
        """Deletes the specified tags.

        :param self:
        :param Tags:  One or more tags.<br/>
        :return:         """
        pass

    def describe_account_limits(self) -> DescribeAccountLimitsAnswer:
        """Describes the current Amazon EC2 Auto Scaling resource limits for your AWS account. For information about requesting an increase in these limits, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/as-account-limits.html">Amazon EC2 Auto Scaling Limits</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.

        :param self:
        :param 
        :return: NumberOfLaunchConfigurations:  The current number of launch configurations for your AWS account.<br/>NumberOfAutoScalingGroups:  The current number of groups for your AWS account.<br/>MaxNumberOfLaunchConfigurations:  The maximum number of launch configurations allowed for your AWS account. The default limit is 200 per region.<br/>MaxNumberOfAutoScalingGroups:  The maximum number of groups allowed for your AWS account. The default limit is 200 per region.<br/>        """
        pass

    def describe_adjustment_types(self) -> DescribeAdjustmentTypesAnswer:
        """Describes the policy adjustment types for use with <a>PutScalingPolicy</a>.

        :param self:
        :param 
        :return: AdjustmentTypes:  The policy adjustment types.<br/>        """
        pass

    def describe_auto_scaling_groups(self, AutoScalingGroupNames: List[ResourceName]=None, NextToken: str=None, MaxRecords: int=None) -> AutoScalingGroupsType:
        """Describes one or more Auto Scaling groups.

        :param self:
        :param MaxRecords:  The maximum number of items to return with this call. The default value is <code>50</code> and the maximum value is <code>100</code>. <br/>NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>AutoScalingGroupNames:  The names of the Auto Scaling groups. Each name can be a maximum of 1600 characters. By default, you can only specify up to 50 names. You can optionally increase this limit using the <code>MaxRecords</code> parameter.  If you omit this parameter, all Auto Scaling groups are described.<br/>
        :return: NextToken:  A string that indicates that the response contains more items than can be returned in a single response. To receive additional items, specify this string for the <code>NextToken</code> value when requesting the next set of items. This value is null when there are no more items to return.<br/>AutoScalingGroups:  The groups.<br/>        """
        pass

    def describe_auto_scaling_instances(self, InstanceIds: List[XmlStringMaxLen19]=None, MaxRecords: int=None, NextToken: str=None) -> AutoScalingInstancesType:
        """Describes one or more Auto Scaling instances.

        :param self:
        :param NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>MaxRecords:  The maximum number of items to return with this call. The default value is <code>50</code> and the maximum value is <code>50</code>.<br/>InstanceIds:  The IDs of the instances. You can specify up to <code>MaxRecords</code> IDs. If you omit this parameter, all Auto Scaling instances are described. If you specify an ID that does not exist, it is ignored with no error.<br/>
        :return: NextToken:  A string that indicates that the response contains more items than can be returned in a single response. To receive additional items, specify this string for the <code>NextToken</code> value when requesting the next set of items. This value is null when there are no more items to return.<br/>AutoScalingInstances:  The instances.<br/>        """
        pass

    def describe_auto_scaling_notification_types(self) -> DescribeAutoScalingNotificationTypesAnswer:
        """Describes the notification types that are supported by Amazon EC2 Auto Scaling.

        :param self:
        :param 
        :return: AutoScalingNotificationTypes:  The notification types.<br/>        """
        pass

    def describe_launch_configurations(self, LaunchConfigurationNames: List[ResourceName]=None, NextToken: str=None, MaxRecords: int=None) -> LaunchConfigurationsType:
        """Describes one or more launch configurations.

        :param self:
        :param MaxRecords:  The maximum number of items to return with this call. The default value is <code>50</code> and the maximum value is <code>100</code>.<br/>NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>LaunchConfigurationNames:  The launch configuration names. If you omit this parameter, all launch configurations are described.<br/>
        :return: NextToken:  A string that indicates that the response contains more items than can be returned in a single response. To receive additional items, specify this string for the <code>NextToken</code> value when requesting the next set of items. This value is null when there are no more items to return.<br/>LaunchConfigurations:  The launch configurations.<br/>        """
        pass

    def describe_lifecycle_hook_types(self) -> DescribeLifecycleHookTypesAnswer:
        """Describes the available types of lifecycle hooks. The following hook types are supported: <ul> <li> autoscaling:EC2_INSTANCE_LAUNCHING </li> <li> autoscaling:EC2_INSTANCE_TERMINATING </li> </ul>

        :param self:
        :param 
        :return: LifecycleHookTypes:  The lifecycle hook types.<br/>        """
        pass

    def describe_lifecycle_hooks(self, AutoScalingGroupName: str, LifecycleHookNames: List[AsciiStringMaxLen255]=None) -> DescribeLifecycleHooksAnswer:
        """Describes the lifecycle hooks for the specified Auto Scaling group.

        :param self:
        :param LifecycleHookNames:  The names of one or more lifecycle hooks. If you omit this parameter, all lifecycle hooks are described.<br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>
        :return: LifecycleHooks:  The lifecycle hooks for the specified group.<br/>        """
        pass

    def describe_load_balancer_target_groups(self, AutoScalingGroupName: str, NextToken: str=None, MaxRecords: int=None) -> DescribeLoadBalancerTargetGroupsResponse:
        """Describes the target groups for the specified Auto Scaling group.

        :param self:
        :param MaxRecords:  The maximum number of items to return with this call. The default value is <code>100</code> and the maximum value is <code>100</code>.<br/>NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>
        :return: NextToken:  A string that indicates that the response contains more items than can be returned in a single response. To receive additional items, specify this string for the <code>NextToken</code> value when requesting the next set of items. This value is null when there are no more items to return.<br/>LoadBalancerTargetGroups:  Information about the target groups.<br/>        """
        pass

    def describe_load_balancers(self, AutoScalingGroupName: str, NextToken: str=None, MaxRecords: int=None) -> DescribeLoadBalancersResponse:
        """Describes the load balancers for the specified Auto Scaling group. This operation describes only Classic Load Balancers. If you have Application Load Balancers or Network Load Balancers, use <a>DescribeLoadBalancerTargetGroups</a> instead.

        :param self:
        :param MaxRecords:  The maximum number of items to return with this call. The default value is <code>100</code> and the maximum value is <code>100</code>.<br/>NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>
        :return: NextToken:  A string that indicates that the response contains more items than can be returned in a single response. To receive additional items, specify this string for the <code>NextToken</code> value when requesting the next set of items. This value is null when there are no more items to return.<br/>LoadBalancers:  The load balancers.<br/>        """
        pass

    def describe_metric_collection_types(self) -> DescribeMetricCollectionTypesAnswer:
        """Describes the available CloudWatch metrics for Amazon EC2 Auto Scaling. The <code>GroupStandbyInstances</code> metric is not returned by default. You must explicitly request this metric when calling <a>EnableMetricsCollection</a>.

        :param self:
        :param 
        :return: Granularities:  The granularities for the metrics.<br/>Metrics:  One or more metrics.<br/>        """
        pass

    def describe_notification_configurations(self, AutoScalingGroupNames: List[ResourceName]=None, NextToken: str=None, MaxRecords: int=None) -> DescribeNotificationConfigurationsAnswer:
        """Describes the notification actions associated with the specified Auto Scaling group.

        :param self:
        :param MaxRecords:  The maximum number of items to return with this call. The default value is <code>50</code> and the maximum value is <code>100</code>.<br/>NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>AutoScalingGroupNames:  The name of the Auto Scaling group.<br/>
        :return: NextToken:  A string that indicates that the response contains more items than can be returned in a single response. To receive additional items, specify this string for the <code>NextToken</code> value when requesting the next set of items. This value is null when there are no more items to return.<br/>NotificationConfigurations:  The notification configurations.<br/>        """
        pass

    def describe_policies(self, AutoScalingGroupName: str=None, PolicyNames: List[ResourceName]=None, PolicyTypes: List[XmlStringMaxLen64]=None, NextToken: str=None, MaxRecords: int=None) -> PoliciesType:
        """Describes the policies for the specified Auto Scaling group.

        :param self:
        :param MaxRecords:  The maximum number of items to be returned with each call. The default value is <code>50</code> and the maximum value is <code>100</code>.<br/>NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>PolicyTypes:  One or more policy types. The valid values are <code>SimpleScaling</code>, <code>StepScaling</code>, and <code>TargetTrackingScaling</code>.<br/>PolicyNames:  The names of one or more policies. If you omit this parameter, all policies are described. If a group name is provided, the results are limited to that group. This list is limited to 50 items. If you specify an unknown policy name, it is ignored with no error.<br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>
        :return: NextToken:  A string that indicates that the response contains more items than can be returned in a single response. To receive additional items, specify this string for the <code>NextToken</code> value when requesting the next set of items. This value is null when there are no more items to return.<br/>ScalingPolicies:  The scaling policies.<br/>        """
        pass

    def describe_scaling_activities(self, ActivityIds: List[XmlString]=None, AutoScalingGroupName: str=None, MaxRecords: int=None, NextToken: str=None) -> ActivitiesType:
        """Describes one or more scaling activities for the specified Auto Scaling group.

        :param self:
        :param NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>MaxRecords:  The maximum number of items to return with this call. The default value is <code>100</code> and the maximum value is <code>100</code>.<br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>ActivityIds:  The activity IDs of the desired scaling activities. You can specify up to 50 IDs. If you omit this parameter, all activities for the past six weeks are described. If unknown activities are requested, they are ignored with no error. If you specify an Auto Scaling group, the results are limited to that group.<br/>
        :return: NextToken:  A string that indicates that the response contains more items than can be returned in a single response. To receive additional items, specify this string for the <code>NextToken</code> value when requesting the next set of items. This value is null when there are no more items to return.<br/>Activities:  The scaling activities. Activities are sorted by start time. Activities still in progress are described first.<br/>        """
        pass

    def describe_scaling_process_types(self) -> ProcessesType:
        """Describes the scaling process types for use with <a>ResumeProcesses</a> and <a>SuspendProcesses</a>.

        :param self:
        :param 
        :return: Processes:  The names of the process types.<br/>        """
        pass

    def describe_scheduled_actions(self, AutoScalingGroupName: str=None, ScheduledActionNames: List[ResourceName]=None, StartTime: TimestampType=None, EndTime: TimestampType=None, NextToken: str=None, MaxRecords: int=None) -> ScheduledActionsType:
        """Describes the actions scheduled for your Auto Scaling group that haven't run or that have not reached their end time. To describe the actions that have already run, use <a>DescribeScalingActivities</a>. 

        :param self:
        :param MaxRecords:  The maximum number of items to return with this call. The default value is <code>50</code> and the maximum value is <code>100</code>.<br/>NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>EndTime:  The latest scheduled start time to return. If scheduled action names are provided, this parameter is ignored.<br/>StartTime:  The earliest scheduled start time to return. If scheduled action names are provided, this parameter is ignored.<br/>ScheduledActionNames:  The names of one or more scheduled actions. You can specify up to 50 actions. If you omit this parameter, all scheduled actions are described. If you specify an unknown scheduled action, it is ignored with no error.<br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>
        :return: NextToken:  A string that indicates that the response contains more items than can be returned in a single response. To receive additional items, specify this string for the <code>NextToken</code> value when requesting the next set of items. This value is null when there are no more items to return.<br/>ScheduledUpdateGroupActions:  The scheduled actions.<br/>        """
        pass

    def describe_tags(self, Filters: List[Filter]=None, NextToken: str=None, MaxRecords: int=None) -> TagsType:
        """Describes the specified tags. You can use filters to limit the results. For example, you can query for the tags for a specific Auto Scaling group. You can specify multiple values for a filter. A tag must match at least one of the specified values for it to be included in the results. You can also specify multiple filters. The result includes information for a particular tag only if it matches all the filters. If there's no match, no special message is returned.

        :param self:
        :param MaxRecords:  The maximum number of items to return with this call. The default value is <code>50</code> and the maximum value is <code>100</code>.<br/>NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>Filters:  One or more filters to scope the tags to return. The maximum number of filters per filter type (for example, <code>auto-scaling-group</code>) is 1000.<br/>
        :return: NextToken:  A string that indicates that the response contains more items than can be returned in a single response. To receive additional items, specify this string for the <code>NextToken</code> value when requesting the next set of items. This value is null when there are no more items to return.<br/>Tags:  One or more tags.<br/>        """
        pass

    def describe_termination_policy_types(self) -> DescribeTerminationPolicyTypesAnswer:
        """Describes the termination policies supported by Amazon EC2 Auto Scaling. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/as-instance-termination.html">Controlling Which Auto Scaling Instances Terminate During Scale In</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.

        :param self:
        :param 
        :return: TerminationPolicyTypes:  The termination policies supported by Amazon EC2 Auto Scaling: <code>OldestInstance</code>, <code>OldestLaunchConfiguration</code>, <code>NewestInstance</code>, <code>ClosestToNextInstanceHour</code>, <code>Default</code>, <code>OldestLaunchTemplate</code>, and <code>AllocationStrategy</code>.<br/>        """
        pass

    def detach_instances(self, ShouldDecrementDesiredCapacity: ShouldDecrementDesiredCapacity, AutoScalingGroupName: str, InstanceIds: List[XmlStringMaxLen19]=None) -> DetachInstancesAnswer:
        """Removes one or more instances from the specified Auto Scaling group. After the instances are detached, you can manage them independent of the Auto Scaling group. If you do not specify the option to decrement the desired capacity, Amazon EC2 Auto Scaling launches instances to replace the ones that are detached. If there is a Classic Load Balancer attached to the Auto Scaling group, the instances are deregistered from the load balancer. If there are target groups attached to the Auto Scaling group, the instances are deregistered from the target groups. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/detach-instance-asg.html">Detach EC2 Instances from Your Auto Scaling Group</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.

        :param self:
        :param ShouldDecrementDesiredCapacity:  Indicates whether the Auto Scaling group decrements the desired capacity value by the number of instances detached.<br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>InstanceIds:  The IDs of the instances. You can specify up to 20 instances.<br/>
        :return: Activities:  The activities related to detaching the instances from the Auto Scaling group.<br/>        """
        pass

    def detach_load_balancer_target_groups(self, TargetGroupARNs: List[XmlStringMaxLen511], AutoScalingGroupName: str) -> DetachLoadBalancerTargetGroupsResultType:
        """Detaches one or more target groups from the specified Auto Scaling group.

        :param self:
        :param TargetGroupARNs:  The Amazon Resource Names (ARN) of the target groups. You can specify up to 10 target groups.<br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>
        :return:         """
        pass

    def detach_load_balancers(self, LoadBalancerNames: List[XmlStringMaxLen255], AutoScalingGroupName: str) -> DetachLoadBalancersResultType:
        """Detaches one or more Classic Load Balancers from the specified Auto Scaling group. This operation detaches only Classic Load Balancers. If you have Application Load Balancers or Network Load Balancers, use <a>DetachLoadBalancerTargetGroups</a> instead. When you detach a load balancer, it enters the <code>Removing</code> state while deregistering the instances in the group. When all instances are deregistered, then you can no longer describe the load balancer using <a>DescribeLoadBalancers</a>. The instances remain running.

        :param self:
        :param LoadBalancerNames:  The names of the load balancers. You can specify up to 10 load balancers.<br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>
        :return:         """
        pass

    def disable_metrics_collection(self, AutoScalingGroupName: str, Metrics: List[XmlStringMaxLen255]=None):
        """Disables group metrics for the specified Auto Scaling group.

        :param self:
        :param Metrics:  One or more of the following metrics. If you omit this parameter, all metrics are disabled. <ul> <li>  <code>GroupMinSize</code>  </li> <li>  <code>GroupMaxSize</code>  </li> <li>  <code>GroupDesiredCapacity</code>  </li> <li>  <code>GroupInServiceInstances</code>  </li> <li>  <code>GroupPendingInstances</code>  </li> <li>  <code>GroupStandbyInstances</code>  </li> <li>  <code>GroupTerminatingInstances</code>  </li> <li>  <code>GroupTotalInstances</code>  </li> </ul><br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>
        :return:         """
        pass

    def enable_metrics_collection(self, Granularity: str, AutoScalingGroupName: str, Metrics: List[XmlStringMaxLen255]=None):
        """Enables group metrics for the specified Auto Scaling group. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/as-instance-monitoring.html">Monitoring Your Auto Scaling Groups and Instances</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.

        :param self:
        :param Granularity:  The granularity to associate with the metrics to collect. The only valid value is <code>1Minute</code>.<br/>Metrics:  One or more of the following metrics. If you omit this parameter, all metrics are enabled. <ul> <li>  <code>GroupMinSize</code>  </li> <li>  <code>GroupMaxSize</code>  </li> <li>  <code>GroupDesiredCapacity</code>  </li> <li>  <code>GroupInServiceInstances</code>  </li> <li>  <code>GroupPendingInstances</code>  </li> <li>  <code>GroupStandbyInstances</code>  </li> <li>  <code>GroupTerminatingInstances</code>  </li> <li>  <code>GroupTotalInstances</code>  </li> </ul><br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>
        :return:         """
        pass

    def enter_standby(self, ShouldDecrementDesiredCapacity: ShouldDecrementDesiredCapacity, AutoScalingGroupName: str, InstanceIds: List[XmlStringMaxLen19]=None) -> EnterStandbyAnswer:
        """Moves the specified instances into the standby state. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/as-enter-exit-standby.html">Temporarily Removing Instances from Your Auto Scaling Group</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.

        :param self:
        :param ShouldDecrementDesiredCapacity:  Indicates whether to decrement the desired capacity of the Auto Scaling group by the number of instances moved to <code>Standby</code> mode.<br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>InstanceIds:  The IDs of the instances. You can specify up to 20 instances.<br/>
        :return: Activities:  The activities related to moving instances into <code>Standby</code> mode.<br/>        """
        pass

    def execute_policy(self, PolicyName: str, AutoScalingGroupName: str=None, HonorCooldown: HonorCooldown=None, MetricValue: MetricScale=None, BreachThreshold: MetricScale=None):
        """Executes the specified policy.

        :param self:
        :param BreachThreshold:  The breach threshold for the alarm. Conditional: This parameter is required if the policy type is <code>StepScaling</code> and not supported otherwise.<br/>MetricValue:  The metric value to compare to <code>BreachThreshold</code>. This enables you to execute a policy of type <code>StepScaling</code> and determine which step adjustment to use. For example, if the breach threshold is 50 and you want to use a step adjustment with a lower bound of 0 and an upper bound of 10, you can set the metric value to 59. If you specify a metric value that doesn't correspond to a step adjustment for the policy, the call returns an error. Conditional: This parameter is required if the policy type is <code>StepScaling</code> and not supported otherwise.<br/>HonorCooldown:  Indicates whether Amazon EC2 Auto Scaling waits for the cooldown period to complete before executing the policy. This parameter is not supported if the policy type is <code>StepScaling</code> or <code>TargetTrackingScaling</code>. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/Cooldown.html">Scaling Cooldowns</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.<br/>PolicyName:  The name or ARN of the policy.<br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>
        :return:         """
        pass

    def exit_standby(self, AutoScalingGroupName: str, InstanceIds: List[XmlStringMaxLen19]=None) -> ExitStandbyAnswer:
        """Moves the specified instances out of the standby state. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/as-enter-exit-standby.html">Temporarily Removing Instances from Your Auto Scaling Group</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.

        :param self:
        :param AutoScalingGroupName:  The name of the Auto Scaling group.<br/>InstanceIds:  The IDs of the instances. You can specify up to 20 instances.<br/>
        :return: Activities:  The activities related to moving instances out of <code>Standby</code> mode.<br/>        """
        pass

    def put_lifecycle_hook(self, AutoScalingGroupName: str, LifecycleHookName: str, LifecycleTransition: str=None, RoleARN: str=None, NotificationTargetARN: str=None, NotificationMetadata: str=None, HeartbeatTimeout: int=None, DefaultResult: str=None) -> PutLifecycleHookAnswer:
        """Creates or updates a lifecycle hook for the specified Auto Scaling group. A lifecycle hook tells Amazon EC2 Auto Scaling to perform an action on an instance when the instance launches (before it is put into service) or as the instance terminates (before it is fully terminated). This step is a part of the procedure for adding a lifecycle hook to an Auto Scaling group: <ol> <li> (Optional) Create a Lambda function and a rule that allows CloudWatch Events to invoke your Lambda function when Amazon EC2 Auto Scaling launches or terminates instances. </li> <li> (Optional) Create a notification target and an IAM role. The target can be either an Amazon SQS queue or an Amazon SNS topic. The role allows Amazon EC2 Auto Scaling to publish lifecycle notifications to the target. </li> <li>  <b>Create the lifecycle hook. Specify whether the hook is used when the instances launch or terminate.</b>  </li> <li> If you need more time, record the lifecycle action heartbeat to keep the instance in a pending state using using <a>RecordLifecycleActionHeartbeat</a>. </li> <li> If you finish before the timeout period ends, complete the lifecycle action using <a>CompleteLifecycleAction</a>. </li> </ol> For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/lifecycle-hooks.html">Amazon EC2 Auto Scaling Lifecycle Hooks</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>. If you exceed your maximum limit of lifecycle hooks, which by default is 50 per Auto Scaling group, the call fails. You can view the lifecycle hooks for an Auto Scaling group using <a>DescribeLifecycleHooks</a>. If you are no longer using a lifecycle hook, you can delete it using <a>DeleteLifecycleHook</a>.

        :param self:
        :param DefaultResult:  Defines the action the Auto Scaling group should take when the lifecycle hook timeout elapses or if an unexpected failure occurs. This parameter can be either <code>CONTINUE</code> or <code>ABANDON</code>. The default value is <code>ABANDON</code>.<br/>HeartbeatTimeout:  The maximum time, in seconds, that can elapse before the lifecycle hook times out. The range is from <code>30</code> to <code>7200</code> seconds. The default value is <code>3600</code> seconds (1 hour). If the lifecycle hook times out, Amazon EC2 Auto Scaling performs the action that you specified in the <code>DefaultResult</code> parameter. You can prevent the lifecycle hook from timing out by calling <a>RecordLifecycleActionHeartbeat</a>.<br/>NotificationMetadata:  Additional information that you want to include any time Amazon EC2 Auto Scaling sends a message to the notification target.<br/>NotificationTargetARN:  The ARN of the notification target that Amazon EC2 Auto Scaling uses to notify you when an instance is in the transition state for the lifecycle hook. This target can be either an SQS queue or an SNS topic. If you specify an empty string, this overrides the current ARN. This operation uses the JSON format when sending notifications to an Amazon SQS queue, and an email key-value pair format when sending notifications to an Amazon SNS topic. When you specify a notification target, Amazon EC2 Auto Scaling sends it a test message. Test messages contain the following additional key-value pair: <code>"Event": "autoscaling:TEST_NOTIFICATION"</code>.<br/>RoleARN:  The ARN of the IAM role that allows the Auto Scaling group to publish to the specified notification target, for example, an Amazon SNS topic or an Amazon SQS queue. Conditional: This parameter is required for new lifecycle hooks, but optional when updating existing hooks.<br/>LifecycleTransition:  The instance state to which you want to attach the lifecycle hook. The valid values are: <ul> <li> autoscaling:EC2_INSTANCE_LAUNCHING </li> <li> autoscaling:EC2_INSTANCE_TERMINATING </li> </ul> Conditional: This parameter is required for new lifecycle hooks, but optional when updating existing hooks.<br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>LifecycleHookName:  The name of the lifecycle hook.<br/>
        :return:         """
        pass

    def put_notification_configuration(self, NotificationTypes: List[XmlStringMaxLen255], TopicARN: str, AutoScalingGroupName: str):
        """Configures an Auto Scaling group to send notifications when specified events take place. Subscribers to the specified topic can have messages delivered to an endpoint such as a web server or an email address. This configuration overwrites any existing configuration. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/ASGettingNotifications.html">Getting Amazon SNS Notifications When Your Auto Scaling Group Scales</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.

        :param self:
        :param NotificationTypes:  The type of event that causes the notification to be sent. For more information about notification types supported by Amazon EC2 Auto Scaling, see <a>DescribeAutoScalingNotificationTypes</a>.<br/>TopicARN:  The Amazon Resource Name (ARN) of the Amazon Simple Notification Service (Amazon SNS) topic.<br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>
        :return:         """
        pass

    def put_scaling_policy(self, PolicyName: str, AutoScalingGroupName: str, PolicyType: str=None, AdjustmentType: str=None, MinAdjustmentStep: int=None, MinAdjustmentMagnitude: int=None, ScalingAdjustment: int=None, Cooldown: int=None, MetricAggregationType: str=None, StepAdjustments: List[StepAdjustment]=None, EstimatedInstanceWarmup: int=None, TargetTrackingConfiguration: TargetTrackingConfiguration=None) -> PolicyARNType:
        """Creates or updates a policy for an Auto Scaling group. To update an existing policy, use the existing policy name and set the parameters to change. Any existing parameter not changed in an update to an existing policy is not changed in this update request.

        :param self:
        :param TargetTrackingConfiguration:  A target tracking scaling policy. Includes support for predefined or customized metrics. Conditional: This parameter is required if the policy type is <code>TargetTrackingScaling</code> and not supported otherwise.:<br/>EstimatedInstanceWarmup:  The estimated time, in seconds, until a newly launched instance can contribute to the CloudWatch metrics. The default is to use the value specified for the default cooldown period for the group. This parameter is supported if the policy type is <code>StepScaling</code> or <code>TargetTrackingScaling</code>.<br/>StepAdjustments:  A set of adjustments that enable you to scale based on the size of the alarm breach. Conditional: This parameter is required if the policy type is <code>StepScaling</code> and not supported otherwise.<br/>MetricAggregationType:  The aggregation type for the CloudWatch metrics. The valid values are <code>Minimum</code>, <code>Maximum</code>, and <code>Average</code>. If the aggregation type is null, the value is treated as <code>Average</code>. This parameter is supported if the policy type is <code>StepScaling</code>.<br/>Cooldown:  The amount of time, in seconds, after a scaling activity completes and before the next scaling activity can start. If this parameter is not specified, the default cooldown period for the group applies. This parameter is supported if the policy type is <code>SimpleScaling</code>. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/Cooldown.html">Scaling Cooldowns</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.<br/>ScalingAdjustment:  The amount by which to scale, based on the specified adjustment type. A positive value adds to the current capacity while a negative number removes from the current capacity. Conditional: This parameter is required if the policy type is <code>SimpleScaling</code> and not supported otherwise.<br/>MinAdjustmentMagnitude:  The minimum number of instances to scale. If the value of <code>AdjustmentType</code> is <code>PercentChangeInCapacity</code>, the scaling policy changes the <code>DesiredCapacity</code> of the Auto Scaling group by at least this many instances. Otherwise, the error is <code>ValidationError</code>. This parameter is supported if the policy type is <code>SimpleScaling</code> or <code>StepScaling</code>.<br/>MinAdjustmentStep:  Available for backward compatibility. Use <code>MinAdjustmentMagnitude</code> instead.<br/>AdjustmentType:  The adjustment type. The valid values are <code>ChangeInCapacity</code>, <code>ExactCapacity</code>, and <code>PercentChangeInCapacity</code>. This parameter is supported if the policy type is <code>SimpleScaling</code> or <code>StepScaling</code>. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/as-scale-based-on-demand.html">Dynamic Scaling</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.<br/>PolicyType:  The policy type. The valid values are <code>SimpleScaling</code>, <code>StepScaling</code>, and <code>TargetTrackingScaling</code>. If the policy type is null, the value is treated as <code>SimpleScaling</code>.<br/>PolicyName:  The name of the policy.<br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisableScaleIn</i> Indicates whether scaling in by the target tracking scaling policy is disabled. If scaling in is disabled, the target tracking scaling policy doesn't remove instances from the Auto Scaling group. Otherwise, the target tracking scaling policy can remove instances from the Auto Scaling group. The default is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetValue</i> The target value for the metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomizedMetricSpecification</i> A customized metric. You must specify either a predefined metric or a customized metric.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CustomizedMetricSpecification</b>: A customized metric. You must specify either a predefined metric or a customized metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PredefinedMetricSpecification</i> A predefined metric. You must specify either a predefined metric or a customized metric.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PredefinedMetricSpecification</b>: A predefined metric. You must specify either a predefined metric or a customized metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceLabel</i> Identifies the resource associated with the metric type. The following predefined metrics are available: <ul> <li>  <code>ASGAverageCPUUtilization</code> - Average CPU utilization of the Auto Scaling group. </li> <li>  <code>ASGAverageNetworkIn</code> - Average number of bytes received on all network interfaces by the Auto Scaling group. </li> <li>  <code>ASGAverageNetworkOut</code> - Average number of bytes sent out on all network interfaces by the Auto Scaling group. </li> <li>  <code>ALBRequestCountPerTarget</code> - Number of requests completed per target in an Application Load Balancer target group. </li> </ul> For predefined metric types <code>ASGAverageCPUUtilization</code>, <code>ASGAverageNetworkIn</code>, and <code>ASGAverageNetworkOut</code>, the parameter must not be specified as the resource associated with the metric type is the Auto Scaling group. For predefined metric type <code>ALBRequestCountPerTarget</code>, the parameter must be specified in the format: <code>app/<i>load-balancer-name</i>/<i>load-balancer-id</i>/targetgroup/<i>target-group-name</i>/<i>target-group-id</i> </code>, where <code>app/<i>load-balancer-name</i>/<i>load-balancer-id</i> </code> is the final portion of the load balancer ARN, and <code>targetgroup/<i>target-group-name</i>/<i>target-group-id</i> </code> is the final portion of the target group ARN. The target group must be attached to the Auto Scaling group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PredefinedMetricType</i> The metric type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Unit</i> The unit of the metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Statistic</i> The statistic of the metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Dimensions</i> The dimensions of the metric. Conditional: If you published your metric with dimensions, you must specify the same dimensions in your scaling policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Namespace</i> The namespace of the metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricName</i> The name of the metric.<br/>
        :return: Alarms:  The CloudWatch alarms created for the target tracking scaling policy.<br/>PolicyARN:  The Amazon Resource Name (ARN) of the policy.<br/>        """
        pass

    def put_scheduled_update_group_action(self, ScheduledActionName: str, AutoScalingGroupName: str, Time: TimestampType=None, StartTime: TimestampType=None, EndTime: TimestampType=None, Recurrence: str=None, MinSize: int=None, MaxSize: int=None, DesiredCapacity: int=None):
        """Creates or updates a scheduled scaling action for an Auto Scaling group. If you leave a parameter unspecified when updating a scheduled scaling action, the corresponding value remains unchanged. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/schedule_time.html">Scheduled Scaling</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.

        :param self:
        :param DesiredCapacity:  The number of EC2 instances that should be running in the Auto Scaling group.<br/>MaxSize:  The maximum number of instances in the Auto Scaling group.<br/>MinSize:  The minimum number of instances in the Auto Scaling group.<br/>Recurrence:  The recurring schedule for this action, in Unix cron syntax format. This format consists of five fields separated by white spaces: [Minute] [Hour] [Day_of_Month] [Month_of_Year] [Day_of_Week]. The value must be in quotes (for example, <code>"30 0 1 1,6,12 *"</code>). For more information about this format, see <a href="http://crontab.org">Crontab</a>. When <code>StartTime</code> and <code>EndTime</code> are specified with <code>Recurrence</code>, they form the boundaries of when the recurring action starts and stops.<br/>EndTime:  The date and time for the recurring schedule to end. Amazon EC2 Auto Scaling does not perform the action after this time. <br/>StartTime:  The date and time for this action to start, in YYYY-MM-DDThh:mm:ssZ format in UTC/GMT only and in quotes (for example, <code>"2019-06-01T00:00:00Z"</code>). If you specify <code>Recurrence</code> and <code>StartTime</code>, Amazon EC2 Auto Scaling performs the action at this time, and then performs the action based on the specified recurrence. If you try to schedule your action in the past, Amazon EC2 Auto Scaling returns an error message.<br/>Time:  This parameter is deprecated.<br/>ScheduledActionName:  The name of this scaling action.<br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>
        :return:         """
        pass

    def record_lifecycle_action_heartbeat(self, AutoScalingGroupName: str, LifecycleHookName: str, LifecycleActionToken: str=None, InstanceId: str=None) -> RecordLifecycleActionHeartbeatAnswer:
        """Records a heartbeat for the lifecycle action associated with the specified token or instance. This extends the timeout by the length of time defined using <a>PutLifecycleHook</a>. This step is a part of the procedure for adding a lifecycle hook to an Auto Scaling group: <ol> <li> (Optional) Create a Lambda function and a rule that allows CloudWatch Events to invoke your Lambda function when Amazon EC2 Auto Scaling launches or terminates instances. </li> <li> (Optional) Create a notification target and an IAM role. The target can be either an Amazon SQS queue or an Amazon SNS topic. The role allows Amazon EC2 Auto Scaling to publish lifecycle notifications to the target. </li> <li> Create the lifecycle hook. Specify whether the hook is used when the instances launch or terminate. </li> <li>  <b>If you need more time, record the lifecycle action heartbeat to keep the instance in a pending state.</b>  </li> <li> If you finish before the timeout period ends, complete the lifecycle action. </li> </ol> For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/AutoScalingGroupLifecycle.html">Auto Scaling Lifecycle</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.

        :param self:
        :param InstanceId:  The ID of the instance.<br/>LifecycleActionToken:  A token that uniquely identifies a specific lifecycle action associated with an instance. Amazon EC2 Auto Scaling sends this token to the notification target that you specified when you created the lifecycle hook.<br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>LifecycleHookName:  The name of the lifecycle hook.<br/>
        :return:         """
        pass

    def resume_processes(self, AutoScalingGroupName: str, ScalingProcesses: List[XmlStringMaxLen255]=None):
        """Resumes the specified suspended automatic scaling processes, or all suspended process, for the specified Auto Scaling group. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/as-suspend-resume-processes.html">Suspending and Resuming Scaling Processes</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.

        :param self:
        :param ScalingProcesses:  One or more of the following processes. If you omit this parameter, all processes are specified. <ul> <li>  <code>Launch</code>  </li> <li>  <code>Terminate</code>  </li> <li>  <code>HealthCheck</code>  </li> <li>  <code>ReplaceUnhealthy</code>  </li> <li>  <code>AZRebalance</code>  </li> <li>  <code>AlarmNotification</code>  </li> <li>  <code>ScheduledActions</code>  </li> <li>  <code>AddToLoadBalancer</code>  </li> </ul><br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>
        :return:         """
        pass

    def set_desired_capacity(self, DesiredCapacity: int, AutoScalingGroupName: str, HonorCooldown: HonorCooldown=None):
        """Sets the size of the specified Auto Scaling group. For more information about desired capacity, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/what-is-amazon-ec2-auto-scaling.html">What Is Amazon EC2 Auto Scaling?</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.

        :param self:
        :param HonorCooldown:  Indicates whether Amazon EC2 Auto Scaling waits for the cooldown period to complete before initiating a scaling activity to set your Auto Scaling group to its new capacity. By default, Amazon EC2 Auto Scaling does not honor the cooldown period during manual scaling activities.<br/>DesiredCapacity:  The number of EC2 instances that should be running in the Auto Scaling group.<br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>
        :return:         """
        pass

    def set_instance_health(self, HealthStatus: str, InstanceId: str, ShouldRespectGracePeriod: ShouldRespectGracePeriod=None):
        """Sets the health status of the specified instance. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/healthcheck.html">Health Checks for Auto Scaling Instances</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.

        :param self:
        :param ShouldRespectGracePeriod:  If the Auto Scaling group of the specified instance has a <code>HealthCheckGracePeriod</code> specified for the group, by default, this call respects the grace period. Set this to <code>False</code>, to have the call not respect the grace period associated with the group. For more information about the health check grace period, see <a>CreateAutoScalingGroup</a>.<br/>HealthStatus:  The health status of the instance. Set to <code>Healthy</code> to have the instance remain in service. Set to <code>Unhealthy</code> to have the instance be out of service. Amazon EC2 Auto Scaling terminates and replaces the unhealthy instance.<br/>InstanceId:  The ID of the instance.<br/>
        :return:         """
        pass

    def set_instance_protection(self, ProtectedFromScaleIn: ProtectedFromScaleIn, AutoScalingGroupName: str, InstanceIds: List[XmlStringMaxLen19]) -> SetInstanceProtectionAnswer:
        """Updates the instance protection settings of the specified instances. For more information about preventing instances that are part of an Auto Scaling group from terminating on scale in, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/as-instance-termination.html#instance-protection">Instance Protection</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.

        :param self:
        :param ProtectedFromScaleIn:  Indicates whether the instance is protected from termination by Amazon EC2 Auto Scaling when scaling in.<br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>InstanceIds:  One or more instance IDs.<br/>
        :return:         """
        pass

    def suspend_processes(self, AutoScalingGroupName: str, ScalingProcesses: List[XmlStringMaxLen255]=None):
        """Suspends the specified automatic scaling processes, or all processes, for the specified Auto Scaling group. If you suspend either the <code>Launch</code> or <code>Terminate</code> process types, it can prevent other process types from functioning properly. To resume processes that have been suspended, use <a>ResumeProcesses</a>. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/as-suspend-resume-processes.html">Suspending and Resuming Scaling Processes</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.

        :param self:
        :param ScalingProcesses:  One or more of the following processes. If you omit this parameter, all processes are specified. <ul> <li>  <code>Launch</code>  </li> <li>  <code>Terminate</code>  </li> <li>  <code>HealthCheck</code>  </li> <li>  <code>ReplaceUnhealthy</code>  </li> <li>  <code>AZRebalance</code>  </li> <li>  <code>AlarmNotification</code>  </li> <li>  <code>ScheduledActions</code>  </li> <li>  <code>AddToLoadBalancer</code>  </li> </ul><br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>
        :return:         """
        pass

    def terminate_instance_in_auto_scaling_group(self, ShouldDecrementDesiredCapacity: ShouldDecrementDesiredCapacity, InstanceId: str) -> ActivityType:
        """Terminates the specified instance and optionally adjusts the desired group size. This call simply makes a termination request. The instance is not terminated immediately.

        :param self:
        :param ShouldDecrementDesiredCapacity:  Indicates whether terminating the instance also decrements the size of the Auto Scaling group.<br/>InstanceId:  The ID of the instance.<br/>
        :return: Activity:  A scaling activity.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Details</i> The details about the activity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Progress</i> A value between 0 and 100 that indicates the progress of the activity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> A friendly, more verbose description of the activity status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusCode</i> The current status of the activity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> The end time of the activity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTime</i> The start time of the activity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Cause</i> The reason the activity began.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A friendly, more verbose description of the activity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoScalingGroupName</i> The name of the Auto Scaling group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityId</i> The ID of the activity.<br/>        """
        pass

    def update_auto_scaling_group(self, AutoScalingGroupName: str, LaunchConfigurationName: str=None, LaunchTemplate: LaunchTemplateSpecification=None, MixedInstancesPolicy: MixedInstancesPolicy=None, MinSize: int=None, MaxSize: int=None, DesiredCapacity: int=None, DefaultCooldown: int=None, AvailabilityZones: List[XmlStringMaxLen255]=None, HealthCheckType: str=None, HealthCheckGracePeriod: int=None, PlacementGroup: str=None, VPCZoneIdentifier: str=None, TerminationPolicies: List[XmlStringMaxLen1600]=None, NewInstancesProtectedFromScaleIn: InstanceProtected=None, ServiceLinkedRoleARN: str=None):
        """Updates the configuration for the specified Auto Scaling group. To update an Auto Scaling group, specify the name of the group and the parameter that you want to change. Any parameters that you don't specify are not changed by this update request. The new settings take effect on any scaling activities after this call returns. Scaling activities that are currently in progress aren't affected. If you associate a new launch configuration or template with an Auto Scaling group, all new instances will get the updated configuration, but existing instances continue to run with the configuration that they were originally launched with. When you update a group to specify a mixed instances policy instead of a launch configuration or template, existing instances may be replaced to match the new purchasing options that you specified in the policy. For example, if the group currently has 100% On-Demand capacity and the policy specifies 50% Spot capacity, this means that half of your instances will be gradually terminated and relaunched as Spot Instances. When replacing instances, Amazon EC2 Auto Scaling launches new instances before terminating the old ones, so that updating your group does not compromise the performance or availability of your application. Note the following about changing <code>DesiredCapacity</code>, <code>MaxSize</code>, or <code>MinSize</code>: <ul> <li> If a scale-in event occurs as a result of a new <code>DesiredCapacity</code> value that is lower than the current size of the group, the Auto Scaling group uses its termination policy to determine which instances to terminate. </li> <li> If you specify a new value for <code>MinSize</code> without specifying a value for <code>DesiredCapacity</code>, and the new <code>MinSize</code> is larger than the current size of the group, this sets the group's <code>DesiredCapacity</code> to the new <code>MinSize</code> value. </li> <li> If you specify a new value for <code>MaxSize</code> without specifying a value for <code>DesiredCapacity</code>, and the new <code>MaxSize</code> is smaller than the current size of the group, this sets the group's <code>DesiredCapacity</code> to the new <code>MaxSize</code> value. </li> </ul> To see which parameters have been set, use <a>DescribeAutoScalingGroups</a>. You can also view the scaling policies for an Auto Scaling group using <a>DescribePolicies</a>. If the group has scaling policies, you can update them using <a>PutScalingPolicy</a>.

        :param self:
        :param ServiceLinkedRoleARN:  The Amazon Resource Name (ARN) of the service-linked role that the Auto Scaling group uses to call other AWS services on your behalf. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/autoscaling-service-linked-role.html">Service-Linked Roles</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.<br/>NewInstancesProtectedFromScaleIn:  Indicates whether newly launched instances are protected from termination by Amazon EC2 Auto Scaling when scaling in. For more information about preventing instances from terminating on scale in, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/as-instance-termination.html#instance-protection">Instance Protection</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.<br/>TerminationPolicies:  A standalone termination policy or a list of termination policies used to select the instance to terminate. The policies are executed in the order that they are listed. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/as-instance-termination.html">Controlling Which Instances Auto Scaling Terminates During Scale In</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.<br/>VPCZoneIdentifier:  A comma-separated list of subnet IDs for virtual private cloud (VPC). If you specify <code>VPCZoneIdentifier</code> with <code>AvailabilityZones</code>, the subnets that you specify for this parameter must reside in those Availability Zones.<br/>PlacementGroup:  The name of the placement group into which to launch your instances, if any. A placement group is a logical grouping of instances within a single Availability Zone. You cannot specify multiple Availability Zones and a placement group. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/placement-groups.html">Placement Groups</a> in the <i>Amazon EC2 User Guide for Linux Instances</i>.<br/>HealthCheckGracePeriod:  The amount of time, in seconds, that Amazon EC2 Auto Scaling waits before checking the health status of an EC2 instance that has come into service. The default value is <code>0</code>. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/healthcheck.html">Health Checks for Auto Scaling Instances</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>. Conditional: This parameter is required if you are adding an <code>ELB</code> health check.<br/>HealthCheckType:  The service to use for the health checks. The valid values are <code>EC2</code> and <code>ELB</code>. If you configure an Auto Scaling group to use ELB health checks, it considers the instance unhealthy if it fails either the EC2 status checks or the load balancer health checks.<br/>AvailabilityZones:  One or more Availability Zones for the group.<br/>DefaultCooldown:  The amount of time, in seconds, after a scaling activity completes before another scaling activity can start. The default value is <code>300</code>. This cooldown period is not used when a scaling-specific cooldown is specified. Cooldown periods are not supported for target tracking scaling policies, step scaling policies, or scheduled scaling. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/Cooldown.html">Scaling Cooldowns</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.<br/>DesiredCapacity:  The number of EC2 instances that should be running in the Auto Scaling group. This number must be greater than or equal to the minimum size of the group and less than or equal to the maximum size of the group.<br/>MaxSize:  The maximum size of the Auto Scaling group.<br/>MinSize:  The minimum size of the Auto Scaling group.<br/>MixedInstancesPolicy:  An embedded object that specifies a mixed instances policy. In your call to <code>UpdateAutoScalingGroup</code>, you can make changes to the policy that is specified. All optional parameters are left unchanged if not specified. For more information, see <a href="https://docs.aws.amazon.com/autoscaling/ec2/userguide/asg-purchase-options.html">Auto Scaling Groups with Multiple Instance Types and Purchase Options</a> in the <i>Amazon EC2 Auto Scaling User Guide</i>.:<br/>LaunchTemplate:  The launch template and version to use to specify the updates. If you specify <code>LaunchTemplate</code> in your update request, you can't specify <code>LaunchConfigurationName</code> or <code>MixedInstancesPolicy</code>.:<br/>LaunchConfigurationName:  The name of the launch configuration. If you specify <code>LaunchConfigurationName</code> in your update request, you can't specify <code>LaunchTemplate</code> or <code>MixedInstancesPolicy</code>. <note> To update an Auto Scaling group with a launch configuration with <code>InstanceMonitoring</code> set to <code>false</code>, you must first disable the collection of group metrics. Otherwise, you get an error. If you have previously enabled the collection of group metrics, you can disable it using <a>DisableMetricsCollection</a>. </note><br/>AutoScalingGroupName:  The name of the Auto Scaling group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version number, <code>$Latest</code>, or <code>$Default</code>. If the value is <code>$Latest</code>, Amazon EC2 Auto Scaling selects the latest version of the launch template when launching instances. If the value is <code>$Default</code>, Amazon EC2 Auto Scaling selects the default version of the launch template when launching instances. The default value is <code>$Default</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LaunchTemplateName</i> The name of the launch template. You must specify either a template name or a template ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LaunchTemplateId</i> The ID of the launch template. You must specify either a template ID or a template name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstancesDistribution</i> The instances distribution to use. If you leave this parameter unspecified when creating a mixed instances policy, the default values are used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InstancesDistribution</b>: The instances distribution to use. If you leave this parameter unspecified when creating a mixed instances policy, the default values are used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LaunchTemplate</i> The launch template and instance types (overrides). This parameter must be specified when creating a mixed instances policy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LaunchTemplate</b>: The launch template and instance types (overrides). This parameter must be specified when creating a mixed instances policy.<br/>The launch template to use. You must specify either the launch template ID or launch template name in the request. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LaunchTemplateSpecification</b>: The launch template to use. You must specify either the launch template ID or launch template name in the request. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SpotMaxPrice</i> The maximum price per unit hour that you are willing to pay for a Spot Instance. If you leave the value of this parameter blank (which is the default), the maximum Spot price is set at the On-Demand price. To remove a value that you previously set, include the parameter but leave the value blank.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SpotInstancePools</i> The number of Spot pools to use to allocate your Spot capacity. The Spot pools are determined from the different instance types in the Overrides array of <a>LaunchTemplate</a>. The range is 1–20. The default value is <code>2</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SpotAllocationStrategy</i> Indicates how to allocate Spot capacity across Spot pools. The only valid value is <code>lowest-price</code>, which is also the default value. The Auto Scaling group selects the cheapest Spot pools and evenly allocates your Spot capacity across the number of Spot pools that you specify. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OnDemandPercentageAboveBaseCapacity</i> Controls the percentages of On-Demand Instances and Spot Instances for your additional capacity beyond <code>OnDemandBaseCapacity</code>. The range is 0–100. The default value is <code>100</code>. If you leave this parameter set to <code>100</code>, the percentages are 100% for On-Demand Instances and 0% for Spot Instances. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OnDemandBaseCapacity</i> The minimum amount of the Auto Scaling group's capacity that must be fulfilled by On-Demand Instances. This base portion is provisioned first as your group scales. The default value is <code>0</code>. If you leave this parameter set to <code>0</code>, On-Demand Instances are launched as a percentage of the Auto Scaling group's desired capacity, per the <code>OnDemandPercentageAboveBaseCapacity</code> setting.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OnDemandAllocationStrategy</i> Indicates how to allocate instance types to fulfill On-Demand capacity.  The only valid value is <code>prioritized</code>, which is also the default value. This strategy uses the order of instance type overrides for the <a>LaunchTemplate</a> to define the launch priority of each instance type. The first instance type in the array is prioritized higher than the last. If all your On-Demand capacity cannot be fulfilled using your highest priority instance, then the Auto Scaling groups launches the remaining capacity using the second priority instance type, and so on. <br/>
        :return:         """
        pass



class AutoScalingPlans(BaseClient):
    """<fullname>AWS Auto Scaling</fullname> Use AWS Auto Scaling to quickly discover all the scalable AWS resources for your application and configure dynamic scaling and predictive scaling for your resources using scaling plans. Use this service in conjunction with the Amazon EC2 Auto Scaling, Application Auto Scaling, Amazon CloudWatch, and AWS CloudFormation services.  Currently, predictive scaling is only available for Amazon EC2 Auto Scaling groups. For more information about AWS Auto Scaling, including information about granting IAM users required permissions for AWS Auto Scaling actions, see the <a href="https://docs.aws.amazon.com/autoscaling/plans/userguide/what-is-aws-auto-scaling.html">AWS Auto Scaling User Guide</a>."""


    class DescribeScalingPlanResourcesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_scaling_plan_resources` for valid parameters"""
            pass
    class DescribeScalingPlansPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_scaling_plans` for valid parameters"""
            pass


    class CreateScalingPlanRequest(Mapping):
        pass
    
    class CreateScalingPlanResponse(Mapping):
        pass
    
    class ApplicationSource(Mapping):
        pass
    
    class ScalingInstruction(Mapping):
        pass
    
    class DeleteScalingPlanRequest(Mapping):
        pass
    
    class DeleteScalingPlanResponse(Mapping):
        pass
    
    class ScalingPlanVersion(object):
        pass
    
    class DescribeScalingPlanResourcesRequest(Mapping):
        pass
    
    class DescribeScalingPlanResourcesResponse(Mapping):
        pass
    
    class DescribeScalingPlansRequest(Mapping):
        pass
    
    class DescribeScalingPlansResponse(Mapping):
        pass
    
    class ScalingPlanName(object):
        pass
    
    class GetScalingPlanResourceForecastDataRequest(Mapping):
        pass
    
    class GetScalingPlanResourceForecastDataResponse(Mapping):
        pass
    
    class TimestampType(object):
        pass
    
    class UpdateScalingPlanRequest(Mapping):
        pass
    
    class UpdateScalingPlanResponse(Mapping):
        pass
    
    def create_scaling_plan(self, ScalingInstructions: List[ScalingInstruction], ApplicationSource: ApplicationSource, ScalingPlanName: str) -> CreateScalingPlanResponse:
        """Creates a scaling plan.

        :param self:
        :param ScalingInstructions:  The scaling instructions.<br/>ApplicationSource:  A CloudFormation stack or set of tags. You can create one scaling plan per application source.:<br/>ScalingPlanName:  The name of the scaling plan. Names cannot contain vertical bars, colons, or forward slashes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TagFilters</i> A set of tags (up to 50).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudFormationStackARN</i> The Amazon Resource Name (ARN) of a AWS CloudFormation stack.<br/>
        :return: ScalingPlanVersion:  The version number of the scaling plan. This value is always 1. Currently, you cannot specify multiple scaling plan versions.<br/>        """
        pass

    def delete_scaling_plan(self, ScalingPlanVersion: ScalingPlanVersion, ScalingPlanName: str) -> DeleteScalingPlanResponse:
        """Deletes the specified scaling plan. Deleting a scaling plan deletes the underlying <a>ScalingInstruction</a> for all of the scalable resources that are covered by the plan. If the plan has launched resources or has scaling activities in progress, you must delete those resources separately.

        :param self:
        :param ScalingPlanVersion:  The version number of the scaling plan.<br/>ScalingPlanName:  The name of the scaling plan.<br/>
        :return:         """
        pass

    def describe_scaling_plan_resources(self, ScalingPlanVersion: ScalingPlanVersion, ScalingPlanName: str, MaxResults: int=None, NextToken: str=None) -> DescribeScalingPlanResourcesResponse:
        """Describes the scalable resources in the specified scaling plan.

        :param self:
        :param NextToken:  The token for the next set of results.<br/>MaxResults:  The maximum number of scalable resources to return. The value must be between 1 and 50. The default value is 50.<br/>ScalingPlanVersion:  The version number of the scaling plan.<br/>ScalingPlanName:  The name of the scaling plan.<br/>
        :return: NextToken:  The token required to get the next set of results. This value is <code>null</code> if there are no more results to return.<br/>ScalingPlanResources:  Information about the scalable resources.<br/>        """
        pass

    def describe_scaling_plans(self, ScalingPlanNames: List[ScalingPlanName]=None, ScalingPlanVersion: ScalingPlanVersion=None, ApplicationSources: List[ApplicationSource]=None, MaxResults: int=None, NextToken: str=None) -> DescribeScalingPlansResponse:
        """Describes one or more of your scaling plans.

        :param self:
        :param NextToken:  The token for the next set of results.<br/>MaxResults:  The maximum number of scalable resources to return. This value can be between 1 and 50. The default value is 50.<br/>ApplicationSources:  The sources for the applications (up to 10). If you specify scaling plan names, you cannot specify application sources.<br/>ScalingPlanVersion:  The version number of the scaling plan. If you specify a scaling plan version, you must also specify a scaling plan name.<br/>ScalingPlanNames:  The names of the scaling plans (up to 10). If you specify application sources, you cannot specify scaling plan names.<br/>
        :return: NextToken:  The token required to get the next set of results. This value is <code>null</code> if there are no more results to return.<br/>ScalingPlans:  Information about the scaling plans.<br/>        """
        pass

    def get_scaling_plan_resource_forecast_data(self, EndTime: TimestampType, StartTime: TimestampType, ForecastDataType: str, ScalableDimension: str, ResourceId: str, ServiceNamespace: str, ScalingPlanVersion: ScalingPlanVersion, ScalingPlanName: str) -> GetScalingPlanResourceForecastDataResponse:
        """Retrieves the forecast data for a scalable resource. Capacity forecasts are represented as predicted values, or data points, that are calculated using historical data points from a specified CloudWatch load metric. Data points are available for up to 56 days. 

        :param self:
        :param EndTime:  The exclusive end time of the time range for the forecast data to get. The maximum time duration between the start and end time is seven days.  Although this parameter can accept a date and time that is more than two days in the future, the availability of forecast data has limits. AWS Auto Scaling only issues forecasts for periods of two days in advance.<br/>StartTime:  The inclusive start time of the time range for the forecast data to get. The date and time can be at most 56 days before the current date and time. <br/>ForecastDataType:  The type of forecast data to get. <ul> <li>  <code>LoadForecast</code>: The load metric forecast.  </li> <li>  <code>CapacityForecast</code>: The capacity forecast.  </li> <li>  <code>ScheduledActionMinCapacity</code>: The minimum capacity for each scheduled scaling action. This data is calculated as the larger of two values: the capacity forecast or the minimum capacity in the scaling instruction. </li> <li>  <code>ScheduledActionMaxCapacity</code>: The maximum capacity for each scheduled scaling action. The calculation used is determined by the predictive scaling maximum capacity behavior setting in the scaling instruction. </li> </ul><br/>ScalableDimension:  The scalable dimension for the resource.<br/>ResourceId:  The ID of the resource. This string consists of the resource type and unique identifier.  <ul> <li> Auto Scaling group - The resource type is <code>autoScalingGroup</code> and the unique identifier is the name of the Auto Scaling group. Example: <code>autoScalingGroup/my-asg</code>. </li> <li> ECS service - The resource type is <code>service</code> and the unique identifier is the cluster name and service name. Example: <code>service/default/sample-webapp</code>. </li> <li> Spot Fleet request - The resource type is <code>spot-fleet-request</code> and the unique identifier is the Spot Fleet request ID. Example: <code>spot-fleet-request/sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE</code>. </li> <li> DynamoDB table - The resource type is <code>table</code> and the unique identifier is the resource ID. Example: <code>table/my-table</code>. </li> <li> DynamoDB global secondary index - The resource type is <code>index</code> and the unique identifier is the resource ID. Example: <code>table/my-table/index/my-table-index</code>. </li> <li> Aurora DB cluster - The resource type is <code>cluster</code> and the unique identifier is the cluster name. Example: <code>cluster:my-db-cluster</code>. </li> </ul><br/>ServiceNamespace:  The namespace of the AWS service.<br/>ScalingPlanVersion:  The version number of the scaling plan.<br/>ScalingPlanName:  The name of the scaling plan.<br/>
        :return: Datapoints:  The data points to return.<br/>        """
        pass

    def update_scaling_plan(self, ScalingPlanVersion: ScalingPlanVersion, ScalingPlanName: str, ApplicationSource: ApplicationSource=None, ScalingInstructions: List[ScalingInstruction]=None) -> UpdateScalingPlanResponse:
        """Updates the specified scaling plan. You cannot update a scaling plan if it is in the process of being created, updated, or deleted.

        :param self:
        :param ScalingInstructions:  The scaling instructions.<br/>ApplicationSource:  A CloudFormation stack or set of tags.:<br/>ScalingPlanVersion:  The version number of the scaling plan.<br/>ScalingPlanName:  The name of the scaling plan.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TagFilters</i> A set of tags (up to 50).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudFormationStackARN</i> The Amazon Resource Name (ARN) of a AWS CloudFormation stack.<br/>
        :return:         """
        pass



class Backup(BaseClient):
    """<fullname>AWS Backup</fullname> AWS Backup is a unified backup service designed to protect AWS services and their associated data. AWS Backup simplifies the creation, migration, restoration, and deletion of backups, while also providing reporting and auditing."""




    class CreateBackupPlanInput(Mapping):
        pass
    
    class CreateBackupPlanOutput(Mapping):
        pass
    
    class BackupPlanInput(Mapping):
        pass
    
    class Tags(object):
        pass
    
    class CreateBackupSelectionInput(Mapping):
        pass
    
    class CreateBackupSelectionOutput(Mapping):
        pass
    
    class BackupSelection(Mapping):
        pass
    
    class CreateBackupVaultInput(Mapping):
        pass
    
    class CreateBackupVaultOutput(Mapping):
        pass
    
    class DeleteBackupPlanInput(Mapping):
        pass
    
    class DeleteBackupPlanOutput(Mapping):
        pass
    
    class DeleteBackupSelectionInput(Mapping):
        pass
    
    class DeleteBackupVaultInput(Mapping):
        pass
    
    class DeleteBackupVaultAccessPolicyInput(Mapping):
        pass
    
    class DeleteBackupVaultNotificationsInput(Mapping):
        pass
    
    class DeleteRecoveryPointInput(Mapping):
        pass
    
    class DescribeBackupJobInput(Mapping):
        pass
    
    class DescribeBackupJobOutput(Mapping):
        pass
    
    class DescribeBackupVaultInput(Mapping):
        pass
    
    class DescribeBackupVaultOutput(Mapping):
        pass
    
    class DescribeProtectedResourceInput(Mapping):
        pass
    
    class DescribeProtectedResourceOutput(Mapping):
        pass
    
    class DescribeRecoveryPointInput(Mapping):
        pass
    
    class DescribeRecoveryPointOutput(Mapping):
        pass
    
    class DescribeRestoreJobInput(Mapping):
        pass
    
    class DescribeRestoreJobOutput(Mapping):
        pass
    
    class ExportBackupPlanTemplateInput(Mapping):
        pass
    
    class ExportBackupPlanTemplateOutput(Mapping):
        pass
    
    class GetBackupPlanInput(Mapping):
        pass
    
    class GetBackupPlanOutput(Mapping):
        pass
    
    class GetBackupPlanFromJSONInput(Mapping):
        pass
    
    class GetBackupPlanFromJSONOutput(Mapping):
        pass
    
    class GetBackupPlanFromTemplateInput(Mapping):
        pass
    
    class GetBackupPlanFromTemplateOutput(Mapping):
        pass
    
    class GetBackupSelectionInput(Mapping):
        pass
    
    class GetBackupSelectionOutput(Mapping):
        pass
    
    class GetBackupVaultAccessPolicyInput(Mapping):
        pass
    
    class GetBackupVaultAccessPolicyOutput(Mapping):
        pass
    
    class GetBackupVaultNotificationsInput(Mapping):
        pass
    
    class GetBackupVaultNotificationsOutput(Mapping):
        pass
    
    class GetRecoveryPointRestoreMetadataInput(Mapping):
        pass
    
    class GetRecoveryPointRestoreMetadataOutput(Mapping):
        pass
    
    class GetSupportedResourceTypesOutput(Mapping):
        pass
    
    class ListBackupJobsInput(Mapping):
        pass
    
    class ListBackupJobsOutput(Mapping):
        pass
    
    class timestamp(object):
        pass
    
    class ListBackupPlanTemplatesInput(Mapping):
        pass
    
    class ListBackupPlanTemplatesOutput(Mapping):
        pass
    
    class ListBackupPlanVersionsInput(Mapping):
        pass
    
    class ListBackupPlanVersionsOutput(Mapping):
        pass
    
    class ListBackupPlansInput(Mapping):
        pass
    
    class ListBackupPlansOutput(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class ListBackupSelectionsInput(Mapping):
        pass
    
    class ListBackupSelectionsOutput(Mapping):
        pass
    
    class ListBackupVaultsInput(Mapping):
        pass
    
    class ListBackupVaultsOutput(Mapping):
        pass
    
    class ListProtectedResourcesInput(Mapping):
        pass
    
    class ListProtectedResourcesOutput(Mapping):
        pass
    
    class ListRecoveryPointsByBackupVaultInput(Mapping):
        pass
    
    class ListRecoveryPointsByBackupVaultOutput(Mapping):
        pass
    
    class ListRecoveryPointsByResourceInput(Mapping):
        pass
    
    class ListRecoveryPointsByResourceOutput(Mapping):
        pass
    
    class ListRestoreJobsInput(Mapping):
        pass
    
    class ListRestoreJobsOutput(Mapping):
        pass
    
    class ListTagsInput(Mapping):
        pass
    
    class ListTagsOutput(Mapping):
        pass
    
    class PutBackupVaultAccessPolicyInput(Mapping):
        pass
    
    class PutBackupVaultNotificationsInput(Mapping):
        pass
    
    class BackupVaultEvent(object):
        pass
    
    class StartBackupJobInput(Mapping):
        pass
    
    class StartBackupJobOutput(Mapping):
        pass
    
    class WindowMinutes(object):
        pass
    
    class Lifecycle(Mapping):
        pass
    
    class StartRestoreJobInput(Mapping):
        pass
    
    class StartRestoreJobOutput(Mapping):
        pass
    
    class Metadata(object):
        pass
    
    class StopBackupJobInput(Mapping):
        pass
    
    class TagResourceInput(Mapping):
        pass
    
    class UntagResourceInput(Mapping):
        pass
    
    class string(object):
        pass
    
    class UpdateBackupPlanInput(Mapping):
        pass
    
    class UpdateBackupPlanOutput(Mapping):
        pass
    
    class UpdateRecoveryPointLifecycleInput(Mapping):
        pass
    
    class UpdateRecoveryPointLifecycleOutput(Mapping):
        pass
    
    def create_backup_plan(self, BackupPlan: BackupPlanInput, BackupPlanTags: Tags=None, CreatorRequestId: str=None) -> CreateBackupPlanOutput:
        """Backup plans are documents that contain information that AWS Backup uses to schedule tasks that create recovery points of resources. If you call <code>CreateBackupPlan</code> with a plan that already exists, the existing <code>backupPlanId</code> is returned.

        :param self:
        :param CreatorRequestId:  Identifies the request and allows failed requests to be retried without the risk of executing the operation twice. If the request includes a <code>CreatorRequestId</code> that matches an existing backup plan, that plan is returned. This parameter is optional.<br/>BackupPlanTags:  To help organize your resources, you can assign your own metadata to the resources that you create. Each tag is a key-value pair. The specified tags are assigned to all backups created with this plan.<br/>BackupPlan:  Specifies the body of a backup plan. Includes a <code>BackupPlanName</code> and one or more sets of <code>Rules</code>.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Rules</i> An array of <code>BackupRule</code> objects, each of which specifies a scheduled task that is used to back up a selection of resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupPlanName</i> The display name of a backup plan.<br/>
        :return: VersionId:  Unique, randomly generated, Unicode, UTF-8 encoded strings that are at most 1024 bytes long. They cannot be edited.<br/>CreationDate:  The date and time that a backup plan is created, in Unix format and Coordinated Universal Time (UTC). The value of <code>CreationDate</code> is accurate to milliseconds. For example, the value 1516925490.087 represents Friday, January 26, 2018 12:11:30.087 AM.<br/>BackupPlanArn:  An Amazon Resource Name (ARN) that uniquely identifies a backup plan; for example, <code>arn:aws:backup:us-east-1:123456789012:plan:8F81F553-3A74-4A3F-B93D-B3360DC80C50</code>.<br/>BackupPlanId:  Uniquely identifies a backup plan.<br/>        """
        pass

    def create_backup_selection(self, BackupSelection: BackupSelection, BackupPlanId: str, CreatorRequestId: str=None) -> CreateBackupSelectionOutput:
        """Creates a JSON document that specifies a set of resources to assign to a backup plan. Resources can be included by specifying patterns for a <code>ListOfTags</code> and selected <code>Resources</code>.  For example, consider the following patterns: <ul> <li>  <code>Resources: "arn:aws:ec2:region:account-id:volume/volume-id"</code>  </li> <li>  <code>ConditionKey:"department"</code>   <code>ConditionValue:"finance"</code>   <code>ConditionType:"StringEquals"</code>  </li> <li>  <code>ConditionKey:"importance"</code>   <code>ConditionValue:"critical"</code>   <code>ConditionType:"StringEquals"</code>  </li> </ul> Using these patterns would back up all Amazon Elastic Block Store (Amazon EBS) volumes that are tagged as <code>"department=finance"</code>, <code>"importance=critical"</code>, in addition to an EBS volume with the specified volume Id. Resources and conditions are additive in that all resources that match the pattern are selected. This shouldn't be confused with a logical AND, where all conditions must match. The matching patterns are logically 'put together using the OR operator. In other words, all patterns that match are selected for backup.

        :param self:
        :param CreatorRequestId:  A unique string that identifies the request and allows failed requests to be retried without the risk of executing the operation twice.<br/>BackupSelection:  Specifies the body of a request to assign a set of resources to a backup plan. It includes an array of resources, an optional array of patterns to exclude resources, an optional role to provide access to the AWS service the resource belongs to, and an optional array of tags used to identify a set of resources.:<br/>BackupPlanId:  Uniquely identifies the backup plan to be associated with the selection of resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ListOfTags</i> An array of conditions used to specify a set of resources to assign to a backup plan; for example, <code>"StringEquals": {"ec2:ResourceTag/Department": "accounting"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Resources</i> An array of strings that either contain Amazon Resource Names (ARNs) or match patterns such as "<code>arn:aws:ec2:us-east-1:123456789012:volume/*</code>" of resources to assign to a backup plan.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoleArn</i> The ARN of the IAM role that AWS Backup uses to authenticate when restoring the target resource; for example, <code>arn:aws:iam::123456789012:role/S3Access</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SelectionName</i> The display name of a resource selection document.<br/>
        :return: CreationDate:  The date and time a backup selection is created, in Unix format and Coordinated Universal Time (UTC). The value of <code>CreationDate</code> is accurate to milliseconds. For example, the value 1516925490.087 represents Friday, January 26, 2018 12:11:30.087 AM.<br/>BackupPlanId:  Uniquely identifies a backup plan.<br/>SelectionId:  Uniquely identifies the body of a request to assign a set of resources to a backup plan.<br/>        """
        pass

    def create_backup_vault(self, BackupVaultName: str, BackupVaultTags: Tags=None, EncryptionKeyArn: str=None, CreatorRequestId: str=None) -> CreateBackupVaultOutput:
        """Creates a logical container where backups are stored. A <code>CreateBackupVault</code> request includes a name, optionally one or more resource tags, an encryption key, and a request ID. <note> Sensitive data, such as passport numbers, should not be included the name of a backup vault. </note>

        :param self:
        :param CreatorRequestId:  A unique string that identifies the request and allows failed requests to be retried without the risk of executing the operation twice.<br/>EncryptionKeyArn:  The server-side encryption key that is used to protect your backups; for example, <code>arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>.<br/>BackupVaultTags:  Metadata that you can assign to help organize the resources that you create. Each tag is a key-value pair.<br/>BackupVaultName:  The name of a logical container where backups are stored. Backup vaults are identified by names that are unique to the account used to create them and the AWS Region where they are created. They consist of lowercase letters, numbers, and hyphens.<br/>
        :return: CreationDate:  The date and time a backup vault is created, in Unix format and Coordinated Universal Time (UTC). The value of <code>CreationDate</code> is accurate to milliseconds. For example, the value 1516925490.087 represents Friday, January 26, 2018 12:11:30.087 AM.<br/>BackupVaultArn:  An Amazon Resource Name (ARN) that uniquely identifies a backup vault; for example, <code>arn:aws:backup:us-east-1:123456789012:vault:aBackupVault</code>.<br/>BackupVaultName:  The name of a logical container where backups are stored. Backup vaults are identified by names that are unique to the account used to create them and the Region where they are created. They consist of lowercase letters, numbers, and hyphens.<br/>        """
        pass

    def delete_backup_plan(self, BackupPlanId: str) -> DeleteBackupPlanOutput:
        """Deletes a backup plan. A backup plan can only be deleted after all associated selections of resources have been deleted. Deleting a backup plan deletes the current version of a backup plan. Previous versions, if any, will still exist.

        :param self:
        :param BackupPlanId:  Uniquely identifies a backup plan.<br/>
        :return: VersionId:  Unique, randomly generated, Unicode, UTF-8 encoded strings that are at most 1,024 bytes long. Version Ids cannot be edited.<br/>DeletionDate:  The date and time a backup plan is deleted, in Unix format and Coordinated Universal Time (UTC). The value of <code>CreationDate</code> is accurate to milliseconds. For example, the value 1516925490.087 represents Friday, January 26, 2018 12:11:30.087 AM.<br/>BackupPlanArn:  An Amazon Resource Name (ARN) that uniquely identifies a backup plan; for example, <code>arn:aws:backup:us-east-1:123456789012:plan:8F81F553-3A74-4A3F-B93D-B3360DC80C50</code>.<br/>BackupPlanId:  Uniquely identifies a backup plan.<br/>        """
        pass

    def delete_backup_selection(self, SelectionId: str, BackupPlanId: str):
        """Deletes the resource selection associated with a backup plan that is specified by the <code>SelectionId</code>.

        :param self:
        :param SelectionId:  Uniquely identifies the body of a request to assign a set of resources to a backup plan.<br/>BackupPlanId:  Uniquely identifies a backup plan.<br/>
        :return:         """
        pass

    def delete_backup_vault(self, BackupVaultName: str):
        """Deletes the backup vault identified by its name. A vault can be deleted only if it is empty.

        :param self:
        :param BackupVaultName:  The name of a logical container where backups are stored. Backup vaults are identified by names that are unique to the account used to create them and theAWS Region where they are created. They consist of lowercase letters, numbers, and hyphens.<br/>
        :return:         """
        pass

    def delete_backup_vault_access_policy(self, BackupVaultName: str):
        """Deletes the policy document that manages permissions on a backup vault.

        :param self:
        :param BackupVaultName:  The name of a logical container where backups are stored. Backup vaults are identified by names that are unique to the account used to create them and the AWS Region where they are created. They consist of lowercase letters, numbers, and hyphens.<br/>
        :return:         """
        pass

    def delete_backup_vault_notifications(self, BackupVaultName: str):
        """Deletes event notifications for the specified backup vault.

        :param self:
        :param BackupVaultName:  The name of a logical container where backups are stored. Backup vaults are identified by names that are unique to the account used to create them and the Region where they are created. They consist of lowercase letters, numbers, and hyphens.<br/>
        :return:         """
        pass

    def delete_recovery_point(self, RecoveryPointArn: str, BackupVaultName: str):
        """Deletes the recovery point specified by a recovery point ID.

        :param self:
        :param RecoveryPointArn:  An Amazon Resource Name (ARN) that uniquely identifies a recovery point; for example, <code>arn:aws:backup:us-east-1:123456789012:recovery-point:1EB3B5E7-9EB0-435A-A80B-108B488B0D45</code>.<br/>BackupVaultName:  The name of a logical container where backups are stored. Backup vaults are identified by names that are unique to the account used to create them and the AWS Region where they are created. They consist of lowercase letters, numbers, and hyphens.<br/>
        :return:         """
        pass

    def describe_backup_job(self, BackupJobId: str) -> DescribeBackupJobOutput:
        """Returns metadata associated with creating a backup of a resource.

        :param self:
        :param BackupJobId:  Uniquely identifies a request to AWS Backup to back up a resource.<br/>
        :return: StartBy:  Specifies the time in Unix format and Coordinated Universal Time (UTC) when a backup job must be started before it is canceled. The value is calculated by adding the start window to the scheduled time. So if the scheduled time were 6:00 PM and the start window is 2 hours, the <code>StartBy</code> time would be 8:00 PM on the date specified. The value of <code>StartBy</code> is accurate to milliseconds. For example, the value 1516925490.087 represents Friday, January 26, 2018 12:11:30.087 AM.<br/>ExpectedCompletionDate:  The date and time that a job to back up resources is expected to be completed, in Unix format and Coordinated Universal Time (UTC). The value of <code>ExpectedCompletionDate</code> is accurate to milliseconds. For example, the value 1516925490.087 represents Friday, January 26, 2018 12:11:30.087 AM.<br/>BytesTransferred:  The size in bytes transferred to a backup vault at the time that the job status was queried.<br/>ResourceType:  The type of AWS resource to be backed-up; for example, an Amazon Elastic Block Store (Amazon EBS) volume or an Amazon Relational Database Service (Amazon RDS) database.<br/>CreatedBy:  Contains identifying information about the creation of a backup job, including the <code>BackupPlanArn</code>, <code>BackupPlanId</code>, <code>BackupPlanVersion</code>, and <code>BackupRuleId</code> of the backup plan that is used to create it.:<br/>IamRoleArn:  Specifies the IAM role ARN used to create the target recovery point; for example, <code>arn:aws:iam::123456789012:role/S3Access</code>.<br/>BackupSizeInBytes:  The size, in bytes, of a backup.<br/>PercentDone:  Contains an estimated percentage that is complete of a job at the time the job status was queried.<br/>StatusMessage:  A detailed message explaining the status of the job to back up a resource.<br/>State:  The current state of a resource recovery point.<br/>CompletionDate:  The date and time that a job to create a backup job is completed, in Unix format and Coordinated Universal Time (UTC). The value of <code>CreationDate</code> is accurate to milliseconds. For example, the value 1516925490.087 represents Friday, January 26, 2018 12:11:30.087 AM.<br/>CreationDate:  The date and time that a backup job is created, in Unix format and Coordinated Universal Time (UTC). The value of <code>CreationDate</code> is accurate to milliseconds. For example, the value 1516925490.087 represents Friday, January 26, 2018 12:11:30.087 AM.<br/>ResourceArn:  An ARN that uniquely identifies a saved resource. The format of the ARN depends on the resource type.<br/>RecoveryPointArn:  An ARN that uniquely identifies a recovery point; for example, <code>arn:aws:backup:us-east-1:123456789012:recovery-point:1EB3B5E7-9EB0-435A-A80B-108B488B0D45</code>.<br/>BackupVaultArn:  An Amazon Resource Name (ARN) that uniquely identifies a backup vault; for example, <code>arn:aws:backup:us-east-1:123456789012:vault:aBackupVault</code>.<br/>BackupVaultName:  The name of a logical container where backups are stored. Backup vaults are identified by names that are unique to the account used to create them and the AWS Region where they are created. They consist of lowercase letters, numbers, and hyphens.<br/>BackupJobId:  Uniquely identifies a request to AWS Backup to back up a resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRuleId</i> Uniquely identifies a rule used to schedule the backup of a selection of resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupPlanVersion</i> Version IDs are unique, randomly generated, Unicode, UTF-8 encoded strings that are at most 1,024 bytes long. They cannot be edited.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupPlanArn</i> An Amazon Resource Name (ARN) that uniquely identifies a backup plan; for example, <code>arn:aws:backup:us-east-1:123456789012:plan:8F81F553-3A74-4A3F-B93D-B3360DC80C50</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupPlanId</i> Uniquely identifies a backup plan.<br/>        """
        pass

    def describe_backup_vault(self, BackupVaultName: str) -> DescribeBackupVaultOutput:
        """Returns metadata about a backup vault specified by its name.

        :param self:
        :param BackupVaultName:  The name of a logical container where backups are stored. Backup vaults are identified by names that are unique to the account used to create them and the AWS Region where they are created. They consist of lowercase letters, numbers, and hyphens.<br/>
        :return: NumberOfRecoveryPoints:  The number of recovery points that are stored in a backup vault.<br/>CreatorRequestId:  A unique string that identifies the request and allows failed requests to be retried without the risk of executing the operation twice.<br/>CreationDate:  The date and time that a backup vault is created, in Unix format and Coordinated Universal Time (UTC). The value of <code>CreationDate</code> is accurate to milliseconds. For example, the value 1516925490.087 represents Friday, January 26, 2018 12:11:30.087 AM.<br/>EncryptionKeyArn:  The server-side encryption key that is used to protect your backups; for example, <code>arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>.<br/>BackupVaultArn:  An Amazon Resource Name (ARN) that uniquely identifies a backup vault; for example, <code>arn:aws:backup:us-east-1:123456789012:vault:aBackupVault</code>.<br/>BackupVaultName:  The name of a logical container where backups are stored. Backup vaults are identified by names that are unique to the account used to create them and the Region where they are created. They consist of lowercase letters, numbers, and hyphens.<br/>        """
        pass

    def describe_protected_resource(self, ResourceArn: str) -> DescribeProtectedResourceOutput:
        """Returns information about a saved resource, including the last time it was backed-up, its Amazon Resource Name (ARN), and the AWS service type of the saved resource.

        :param self:
        :param ResourceArn:  An Amazon Resource Name (ARN) that uniquely identifies a resource. The format of the ARN depends on the resource type.<br/>
        :return: LastBackupTime:  The date and time that a resource was last backed up, in Unix format and Coordinated Universal Time (UTC). The value of <code>LastBackupTime</code> is accurate to milliseconds. For example, the value 1516925490.087 represents Friday, January 26, 2018 12:11:30.087 AM.<br/>ResourceType:  The type of AWS resource saved as a recovery point; for example, an EBS volume or an Amazon RDS database.<br/>ResourceArn:  An ARN that uniquely identifies a resource. The format of the ARN depends on the resource type.<br/>        """
        pass

    def describe_recovery_point(self, RecoveryPointArn: str, BackupVaultName: str) -> DescribeRecoveryPointOutput:
        """Returns metadata associated with a recovery point, including ID, status, encryption, and lifecycle.

        :param self:
        :param RecoveryPointArn:  An Amazon Resource Name (ARN) that uniquely identifies a recovery point; for example, <code>arn:aws:backup:us-east-1:123456789012:recovery-point:1EB3B5E7-9EB0-435A-A80B-108B488B0D45</code>.<br/>BackupVaultName:  The name of a logical container where backups are stored. Backup vaults are identified by names that are unique to the account used to create them and the AWS Region where they are created. They consist of lowercase letters, numbers, and hyphens.<br/>
        :return: LastRestoreTime:  The date and time that a recovery point was last restored, in Unix format and Coordinated Universal Time (UTC). The value of <code>LastRestoreTime</code> is accurate to milliseconds. For example, the value 1516925490.087 represents Friday, January 26, 2018 12:11:30.087 AM.<br/>StorageClass:  Specifies the storage class of the recovery point. Valid values are <code>WARM</code> or <code>COLD</code>.<br/>IsEncrypted:  A Boolean value that is returned as <code>TRUE</code> if the specified recovery point is encrypted, or <code>FALSE</code> if the recovery point is not encrypted.<br/>EncryptionKeyArn:  The server-side encryption key used to protect your backups; for example, <code>arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>.<br/>Lifecycle:  The lifecycle defines when a protected resource is transitioned to cold storage and when it expires. AWS Backup transitions and expires backups automatically according to the lifecycle that you define.  Backups that are transitioned to cold storage must be stored in cold storage for a minimum of 90 days. Therefore, the “expire after days” setting must be 90 days greater than the “transition to cold after days” setting. The “transition to cold after days” setting cannot be changed after a backup has been transitioned to cold. :<br/>CalculatedLifecycle:  A <code>CalculatedLifecycle</code> object containing <code>DeleteAt</code> and <code>MoveToColdStorageAt</code> timestamps.:<br/>BackupSizeInBytes:  The size, in bytes, of a backup.<br/>CompletionDate:  The date and time that a job to create a recovery point is completed, in Unix format and Coordinated Universal Time (UTC). The value of <code>CompletionDate</code> is accurate to milliseconds. For example, the value 1516925490.087 represents Friday, January 26, 2018 12:11:30.087 AM.<br/>CreationDate:  The date and time that a recovery point is created, in Unix format and Coordinated Universal Time (UTC). The value of <code>CreationDate</code> is accurate to milliseconds. For example, the value 1516925490.087 represents Friday, January 26, 2018 12:11:30.087 AM.<br/>Status:  A status code specifying the state of the recovery point. <note> A partial status indicates that the recovery point was not successfully re-created and must be retried. </note><br/>IamRoleArn:  Specifies the IAM role ARN used to create the target recovery point; for example, <code>arn:aws:iam::123456789012:role/S3Access</code>.<br/>CreatedBy:  Contains identifying information about the creation of a recovery point, including the <code>BackupPlanArn</code>, <code>BackupPlanId</code>, <code>BackupPlanVersion</code>, and <code>BackupRuleId</code> of the backup plan used to create it.:<br/>ResourceType:  The type of AWS resource to save as a recovery point; for example, an Amazon Elastic Block Store (Amazon EBS) volume or an Amazon Relational Database Service (Amazon RDS) database.<br/>ResourceArn:  An ARN that uniquely identifies a saved resource. The format of the ARN depends on the resource type.<br/>BackupVaultArn:  An ARN that uniquely identifies a backup vault; for example, <code>arn:aws:backup:us-east-1:123456789012:vault:aBackupVault</code>.<br/>BackupVaultName:  The name of a logical container where backups are stored. Backup vaults are identified by names that are unique to the account used to create them and the Region where they are created. They consist of lowercase letters, numbers, and hyphens.<br/>RecoveryPointArn:  An ARN that uniquely identifies a recovery point; for example, <code>arn:aws:backup:us-east-1:123456789012:recovery-point:1EB3B5E7-9EB0-435A-A80B-108B488B0D45</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRuleId</i> Uniquely identifies a rule used to schedule the backup of a selection of resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupPlanVersion</i> Version IDs are unique, randomly generated, Unicode, UTF-8 encoded strings that are at most 1,024 bytes long. They cannot be edited.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupPlanArn</i> An Amazon Resource Name (ARN) that uniquely identifies a backup plan; for example, <code>arn:aws:backup:us-east-1:123456789012:plan:8F81F553-3A74-4A3F-B93D-B3360DC80C50</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupPlanId</i> Uniquely identifies a backup plan.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeleteAt</i> A timestamp that specifies when to delete a recovery point.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MoveToColdStorageAt</i> A timestamp that specifies when to transition a recovery point to cold storage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeleteAfterDays</i> Specifies the number of days after creation that a recovery point is deleted. Must be greater than <code>MoveToColdStorageAfterDays</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MoveToColdStorageAfterDays</i> Specifies the number of days after creation that a recovery point is moved to cold storage.<br/>        """
        pass

    def describe_restore_job(self, RestoreJobId: str) -> DescribeRestoreJobOutput:
        """Returns metadata associated with a restore job that is specified by a job ID.

        :param self:
        :param RestoreJobId:  Uniquely identifies the job that restores a recovery point.<br/>
        :return: CreatedResourceArn:  An Amazon Resource Name (ARN) that uniquely identifies a resource whose recovery point is being restored. The format of the ARN depends on the resource type of the backed-up resource.<br/>ExpectedCompletionTimeMinutes:  The amount of time in minutes that a job restoring a recovery point is expected to take.<br/>IamRoleArn:  Specifies the IAM role ARN used to create the target recovery point; for example, <code>arn:aws:iam::123456789012:role/S3Access</code>.<br/>BackupSizeInBytes:  The size, in bytes, of the restored resource.<br/>PercentDone:  Contains an estimated percentage that is complete of a job at the time the job status was queried.<br/>StatusMessage:  A detailed message explaining the status of a job to restore a recovery point.<br/>Status:  Status code specifying the state of the job that is initiated by AWS Backup to restore a recovery point.<br/>CompletionDate:  The date and time that a job to restore a recovery point is completed, in Unix format and Coordinated Universal Time (UTC). The value of <code>CompletionDate</code> is accurate to milliseconds. For example, the value 1516925490.087 represents Friday, January 26, 2018 12:11:30.087 AM.<br/>CreationDate:  The date and time that a restore job is created, in Unix format and Coordinated Universal Time (UTC). The value of <code>CreationDate</code> is accurate to milliseconds. For example, the value 1516925490.087 represents Friday, January 26, 2018 12:11:30.087 AM.<br/>RecoveryPointArn:  An ARN that uniquely identifies a recovery point; for example, <code>arn:aws:backup:us-east-1:123456789012:recovery-point:1EB3B5E7-9EB0-435A-A80B-108B488B0D45</code>.<br/>RestoreJobId:  Uniquely identifies the job that restores a recovery point.<br/>        """
        pass

    def export_backup_plan_template(self, BackupPlanId: str) -> ExportBackupPlanTemplateOutput:
        """Returns the backup plan that is specified by the plan ID as a backup template.

        :param self:
        :param BackupPlanId:  Uniquely identifies a backup plan.<br/>
        :return: BackupPlanTemplateJson:  The body of a backup plan template in JSON format. <note> This is a signed JSON document that cannot be modified before being passed to <code>GetBackupPlanFromJSON.</code>  </note><br/>        """
        pass

    def get_backup_plan(self, BackupPlanId: str, VersionId: str=None) -> GetBackupPlanOutput:
        """Returns the body of a backup plan in JSON format, in addition to plan metadata.

        :param self:
        :param VersionId:  Unique, randomly generated, Unicode, UTF-8 encoded strings that are at most 1,024 bytes long. Version IDs cannot be edited.<br/>BackupPlanId:  Uniquely identifies a backup plan.<br/>
        :return: LastExecutionDate:  The last time a job to back up resources was executed with this backup plan. A date and time, in Unix format and Coordinated Universal Time (UTC). The value of <code>LastExecutionDate</code> is accurate to milliseconds. For example, the value 1516925490.087 represents Friday, January 26, 2018 12:11:30.087 AM.<br/>DeletionDate:  The date and time that a backup plan is deleted, in Unix format and Coordinated Universal Time (UTC). The value of <code>CreationDate</code> is accurate to milliseconds. For example, the value 1516925490.087 represents Friday, January 26, 2018 12:11:30.087 AM.<br/>CreationDate:  The date and time that a backup plan is created, in Unix format and Coordinated Universal Time (UTC). The value of <code>CreationDate</code> is accurate to milliseconds. For example, the value 1516925490.087 represents Friday, January 26, 2018 12:11:30.087 AM.<br/>CreatorRequestId:  A unique string that identifies the request and allows failed requests to be retried without the risk of executing the operation twice.<br/>VersionId:  Unique, randomly generated, Unicode, UTF-8 encoded strings that are at most 1,024 bytes long. Version IDs cannot be edited.<br/>BackupPlanArn:  An Amazon Resource Name (ARN) that uniquely identifies a backup plan; for example, <code>arn:aws:backup:us-east-1:123456789012:plan:8F81F553-3A74-4A3F-B93D-B3360DC80C50</code>.<br/>BackupPlanId:  Uniquely identifies a backup plan.<br/>BackupPlan:  Specifies the body of a backup plan. Includes a <code>BackupPlanName</code> and one or more sets of <code>Rules</code>.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Rules</i> An array of <code>BackupRule</code> objects, each of which specifies a scheduled task that is used to back up a selection of resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupPlanName</i> The display name of a backup plan.<br/>        """
        pass

    def get_backup_plan_from_json(self, BackupPlanTemplateJson: str) -> GetBackupPlanFromJSONOutput:
        """Returns a valid JSON document specifying a backup plan or an error.

        :param self:
        :param BackupPlanTemplateJson:  A customer-supplied backup plan document in JSON format.<br/>
        :return: BackupPlan:  Specifies the body of a backup plan. Includes a <code>BackupPlanName</code> and one or more sets of <code>Rules</code>.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Rules</i> An array of <code>BackupRule</code> objects, each of which specifies a scheduled task that is used to back up a selection of resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupPlanName</i> The display name of a backup plan.<br/>        """
        pass

    def get_backup_plan_from_template(self, BackupPlanTemplateId: str) -> GetBackupPlanFromTemplateOutput:
        """Returns the template specified by its <code>templateId</code> as a backup plan.

        :param self:
        :param BackupPlanTemplateId:  Uniquely identifies a stored backup plan template.<br/>
        :return: BackupPlanDocument:  Returns the body of a backup plan based on the target template, including the name, rules, and backup vault of the plan.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Rules</i> An array of <code>BackupRule</code> objects, each of which specifies a scheduled task that is used to back up a selection of resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupPlanName</i> The display name of a backup plan.<br/>        """
        pass

    def get_backup_selection(self, SelectionId: str, BackupPlanId: str) -> GetBackupSelectionOutput:
        """Returns selection metadata and a document in JSON format that specifies a list of resources that are associated with a backup plan.

        :param self:
        :param SelectionId:  Uniquely identifies the body of a request to assign a set of resources to a backup plan.<br/>BackupPlanId:  Uniquely identifies a backup plan.<br/>
        :return: CreatorRequestId:  A unique string that identifies the request and allows failed requests to be retried without the risk of executing the operation twice.<br/>CreationDate:  The date and time a backup selection is created, in Unix format and Coordinated Universal Time (UTC). The value of <code>CreationDate</code> is accurate to milliseconds. For example, the value 1516925490.087 represents Friday, January 26, 2018 12:11:30.087 AM.<br/>BackupPlanId:  Uniquely identifies a backup plan.<br/>SelectionId:  Uniquely identifies the body of a request to assign a set of resources to a backup plan.<br/>BackupSelection:  Specifies the body of a request to assign a set of resources to a backup plan. It includes an array of resources, an optional array of patterns to exclude resources, an optional role to provide access to the AWS service that the resource belongs to, and an optional array of tags used to identify a set of resources.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ListOfTags</i> An array of conditions used to specify a set of resources to assign to a backup plan; for example, <code>"StringEquals": {"ec2:ResourceTag/Department": "accounting"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Resources</i> An array of strings that either contain Amazon Resource Names (ARNs) or match patterns such as "<code>arn:aws:ec2:us-east-1:123456789012:volume/*</code>" of resources to assign to a backup plan.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoleArn</i> The ARN of the IAM role that AWS Backup uses to authenticate when restoring the target resource; for example, <code>arn:aws:iam::123456789012:role/S3Access</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SelectionName</i> The display name of a resource selection document.<br/>        """
        pass

    def get_backup_vault_access_policy(self, BackupVaultName: str) -> GetBackupVaultAccessPolicyOutput:
        """Returns the access policy document that is associated with the named backup vault.

        :param self:
        :param BackupVaultName:  The name of a logical container where backups are stored. Backup vaults are identified by names that are unique to the account used to create them and the AWS Region where they are created. They consist of lowercase letters, numbers, and hyphens.<br/>
        :return: Policy:  The backup vault access policy document in JSON format.<br/>BackupVaultArn:  An Amazon Resource Name (ARN) that uniquely identifies a backup vault; for example, <code>arn:aws:backup:us-east-1:123456789012:vault:aBackupVault</code>.<br/>BackupVaultName:  The name of a logical container where backups are stored. Backup vaults are identified by names that are unique to the account used to create them and the Region where they are created. They consist of lowercase letters, numbers, and hyphens.<br/>        """
        pass

    def get_backup_vault_notifications(self, BackupVaultName: str) -> GetBackupVaultNotificationsOutput:
        """Returns event notifications for the specified backup vault.

        :param self:
        :param BackupVaultName:  The name of a logical container where backups are stored. Backup vaults are identified by names that are unique to the account used to create them and the AWS Region where they are created. They consist of lowercase letters, numbers, and hyphens.<br/>
        :return: BackupVaultEvents:  An array of events that indicate the status of jobs to back up resources to the backup vault.<br/>SNSTopicArn:  An ARN that uniquely identifies an Amazon Simple Notification Service (Amazon SNS) topic; for example, <code>arn:aws:sns:us-west-2:111122223333:MyTopic</code>.<br/>BackupVaultArn:  An Amazon Resource Name (ARN) that uniquely identifies a backup vault; for example, <code>arn:aws:backup:us-east-1:123456789012:vault:aBackupVault</code>.<br/>BackupVaultName:  The name of a logical container where backups are stored. Backup vaults are identified by names that are unique to the account used to create them and the Region where they are created. They consist of lowercase letters, numbers, and hyphens.<br/>        """
        pass

    def get_recovery_point_restore_metadata(self, RecoveryPointArn: str, BackupVaultName: str) -> GetRecoveryPointRestoreMetadataOutput:
        """Returns two sets of metadata key-value pairs. The first set lists the metadata that the recovery point was created with. The second set lists the metadata key-value pairs that are required to restore the recovery point. These sets can be the same, or the restore metadata set can contain different values if the target service to be restored has changed since the recovery point was created and now requires additional or different information in order to be restored.

        :param self:
        :param RecoveryPointArn:  An Amazon Resource Name (ARN) that uniquely identifies a recovery point; for example, <code>arn:aws:backup:us-east-1:123456789012:recovery-point:1EB3B5E7-9EB0-435A-A80B-108B488B0D45</code>.<br/>BackupVaultName:  The name of a logical container where backups are stored. Backup vaults are identified by names that are unique to the account used to create them and the AWS Region where they are created. They consist of lowercase letters, numbers, and hyphens.<br/>
        :return: RestoreMetadata:  A set of metadata key-value pairs that lists the metadata key-value pairs that are required to restore the recovery point.<br/>RecoveryPointArn:  An ARN that uniquely identifies a recovery point; for example, <code>arn:aws:backup:us-east-1:123456789012:recovery-point:1EB3B5E7-9EB0-435A-A80B-108B488B0D45</code>.<br/>BackupVaultArn:  An ARN that uniquely identifies a backup vault; for example, <code>arn:aws:backup:us-east-1:123456789012:vault:aBackupVault</code>.<br/>        """
        pass

    def get_supported_resource_types(self) -> GetSupportedResourceTypesOutput:
        """Returns the AWS resource types supported by AWS Backup.

        :param self:
        :param 
        :return: ResourceTypes:  Contains a string with the supported AWS resource types: <ul> <li>  <code>EBS</code> for Amazon Elastic Block Store </li> <li>  <code>SGW</code> for AWS Storage Gateway </li> <li>  <code>RDS</code> for Amazon Relational Database Service </li> <li>  <code>DDB</code> for Amazon DynamoDB </li> <li>  <code>EFS</code> for Amazon Elastic File System </li> </ul><br/>        """
        pass

    def list_backup_jobs(self, NextToken: str=None, MaxResults: int=None, ByResourceArn: str=None, ByState: str=None, ByBackupVaultName: str=None, ByCreatedBefore: timestamp=None, ByCreatedAfter: timestamp=None, ByResourceType: str=None) -> ListBackupJobsOutput:
        """Returns metadata about your backup jobs.

        :param self:
        :param ByResourceType:  Returns only backup jobs for the specified resources: <ul> <li>  <code>EBS</code> for Amazon Elastic Block Store </li> <li>  <code>SGW</code> for AWS Storage Gateway </li> <li>  <code>RDS</code> for Amazon Relational Database Service </li> <li>  <code>DDB</code> for Amazon DynamoDB </li> <li>  <code>EFS</code> for Amazon Elastic File System </li> </ul><br/>ByCreatedAfter:  Returns only backup jobs that were created after the specified date.<br/>ByCreatedBefore:  Returns only backup jobs that were created before the specified date.<br/>ByBackupVaultName:  Returns only backup jobs that will be stored in the specified backup vault. Backup vaults are identified by names that are unique to the account used to create them and the AWS Region where they are created. They consist of lowercase letters, numbers, and hyphens.<br/>ByState:  Returns only backup jobs that are in the specified state.<br/>ByResourceArn:  Returns only backup jobs that match the specified resource Amazon Resource Name (ARN).<br/>MaxResults:  The maximum number of items to be returned.<br/>NextToken:  The next item following a partial list of returned items. For example, if a request is made to return <code>maxResults</code> number of items, <code>NextToken</code> allows you to return more items in your list starting at the location pointed to by the next token.<br/>
        :return: NextToken:  The next item following a partial list of returned items. For example, if a request is made to return <code>maxResults</code> number of items, <code>NextToken</code> allows you to return more items in your list starting at the location pointed to by the next token.<br/>BackupJobs:  An array of structures containing metadata about your backup jobs returned in JSON format.<br/>        """
        pass

    def list_backup_plan_templates(self, NextToken: str=None, MaxResults: int=None) -> ListBackupPlanTemplatesOutput:
        """Returns metadata of your saved backup plan templates, including the template ID, name, and the creation and deletion dates.

        :param self:
        :param MaxResults:  The maximum number of items to be returned.<br/>NextToken:  The next item following a partial list of returned items. For example, if a request is made to return <code>maxResults</code> number of items, <code>NextToken</code> allows you to return more items in your list starting at the location pointed to by the next token.<br/>
        :return: BackupPlanTemplatesList:  An array of template list items containing metadata about your saved templates.<br/>NextToken:  The next item following a partial list of returned items. For example, if a request is made to return <code>maxResults</code> number of items, <code>NextToken</code> allows you to return more items in your list starting at the location pointed to by the next token.<br/>        """
        pass

    def list_backup_plan_versions(self, BackupPlanId: str, NextToken: str=None, MaxResults: int=None) -> ListBackupPlanVersionsOutput:
        """Returns version metadata of your backup plans, including Amazon Resource Names (ARNs), backup plan IDs, creation and deletion dates, plan names, and version IDs.

        :param self:
        :param MaxResults:  The maximum number of items to be returned.<br/>NextToken:  The next item following a partial list of returned items. For example, if a request is made to return <code>maxResults</code> number of items, <code>NextToken</code> allows you to return more items in your list starting at the location pointed to by the next token.<br/>BackupPlanId:  Uniquely identifies a backup plan.<br/>
        :return: BackupPlanVersionsList:  An array of version list items containing metadata about your backup plans.<br/>NextToken:  The next item following a partial list of returned items. For example, if a request is made to return <code>maxResults</code> number of items, <code>NextToken</code> allows you to return more items in your list starting at the location pointed to by the next token.<br/>        """
        pass

    def list_backup_plans(self, NextToken: str=None, MaxResults: int=None, IncludeDeleted: Boolean=None) -> ListBackupPlansOutput:
        """Returns metadata of your saved backup plans, including Amazon Resource Names (ARNs), plan IDs, creation and deletion dates, version IDs, plan names, and creator request IDs.

        :param self:
        :param IncludeDeleted:  A Boolean value with a default value of <code>FALSE</code> that returns deleted backup plans when set to <code>TRUE</code>.<br/>MaxResults:  The maximum number of items to be returned.<br/>NextToken:  The next item following a partial list of returned items. For example, if a request is made to return <code>maxResults</code> number of items, <code>NextToken</code> allows you to return more items in your list starting at the location pointed to by the next token.<br/>
        :return: BackupPlansList:  An array of backup plan list items containing metadata about your saved backup plans.<br/>NextToken:  The next item following a partial list of returned items. For example, if a request is made to return <code>maxResults</code> number of items, <code>NextToken</code> allows you to return more items in your list starting at the location pointed to by the next token.<br/>        """
        pass

    def list_backup_selections(self, BackupPlanId: str, NextToken: str=None, MaxResults: int=None) -> ListBackupSelectionsOutput:
        """Returns an array containing metadata of the resources associated with the target backup plan.

        :param self:
        :param MaxResults:  The maximum number of items to be returned.<br/>NextToken:  The next item following a partial list of returned items. For example, if a request is made to return <code>maxResults</code> number of items, <code>NextToken</code> allows you to return more items in your list starting at the location pointed to by the next token.<br/>BackupPlanId:  Uniquely identifies a backup plan.<br/>
        :return: BackupSelectionsList:  An array of backup selection list items containing metadata about each resource in the list.<br/>NextToken:  The next item following a partial list of returned items. For example, if a request is made to return <code>maxResults</code> number of items, <code>NextToken</code> allows you to return more items in your list starting at the location pointed to by the next token.<br/>        """
        pass

    def list_backup_vaults(self, NextToken: str=None, MaxResults: int=None) -> ListBackupVaultsOutput:
        """Returns a list of recovery point storage containers along with information about them.

        :param self:
        :param MaxResults:  The maximum number of items to be returned.<br/>NextToken:  The next item following a partial list of returned items. For example, if a request is made to return <code>maxResults</code> number of items, <code>NextToken</code> allows you to return more items in your list starting at the location pointed to by the next token.<br/>
        :return: NextToken:  The next item following a partial list of returned items. For example, if a request is made to return <code>maxResults</code> number of items, <code>NextToken</code> allows you to return more items in your list starting at the location pointed to by the next token.<br/>BackupVaultList:  An array of backup vault list members containing vault metadata, including Amazon Resource Name (ARN), display name, creation date, number of saved recovery points, and encryption information if the resources saved in the backup vault are encrypted.<br/>        """
        pass

    def list_protected_resources(self, NextToken: str=None, MaxResults: int=None) -> ListProtectedResourcesOutput:
        """Returns an array of resources successfully backed up by AWS Backup, including the time the resource was saved, an Amazon Resource Name (ARN) of the resource, and a resource type.

        :param self:
        :param MaxResults:  The maximum number of items to be returned.<br/>NextToken:  The next item following a partial list of returned items. For example, if a request is made to return <code>maxResults</code> number of items, <code>NextToken</code> allows you to return more items in your list starting at the location pointed to by the next token.<br/>
        :return: NextToken:  The next item following a partial list of returned items. For example, if a request is made to return <code>maxResults</code> number of items, <code>NextToken</code> allows you to return more items in your list starting at the location pointed to by the next token.<br/>Results:  An array of resources successfully backed up by AWS Backup including the time the resource was saved, an Amazon Resource Name (ARN) of the resource, and a resource type.<br/>        """
        pass

    def list_recovery_points_by_backup_vault(self, BackupVaultName: str, NextToken: str=None, MaxResults: int=None, ByResourceArn: str=None, ByResourceType: str=None, ByBackupPlanId: str=None, ByCreatedBefore: timestamp=None, ByCreatedAfter: timestamp=None) -> ListRecoveryPointsByBackupVaultOutput:
        """Returns detailed information about the recovery points stored in a backup vault.

        :param self:
        :param ByCreatedAfter:  Returns only recovery points that were created after the specified timestamp.<br/>ByCreatedBefore:  Returns only recovery points that were created before the specified timestamp.<br/>ByBackupPlanId:  Returns only recovery points that match the specified backup plan ID.<br/>ByResourceType:  Returns only recovery points that match the specified resource type.<br/>ByResourceArn:  Returns only recovery points that match the specified resource Amazon Resource Name (ARN).<br/>MaxResults:  The maximum number of items to be returned.<br/>NextToken:  The next item following a partial list of returned items. For example, if a request is made to return <code>maxResults</code> number of items, <code>NextToken</code> allows you to return more items in your list starting at the location pointed to by the next token.<br/>BackupVaultName:  The name of a logical container where backups are stored. Backup vaults are identified by names that are unique to the account used to create them and the AWS Region where they are created. They consist of lowercase letters, numbers, and hyphens.<br/>
        :return: RecoveryPoints:  An array of objects that contain detailed information about recovery points saved in a backup vault.<br/>NextToken:  The next item following a partial list of returned items. For example, if a request is made to return <code>maxResults</code> number of items, <code>NextToken</code> allows you to return more items in your list starting at the location pointed to by the next token.<br/>        """
        pass

    def list_recovery_points_by_resource(self, ResourceArn: str, NextToken: str=None, MaxResults: int=None) -> ListRecoveryPointsByResourceOutput:
        """Returns detailed information about recovery points of the type specified by a resource Amazon Resource Name (ARN).

        :param self:
        :param MaxResults:  The maximum number of items to be returned.<br/>NextToken:  The next item following a partial list of returned items. For example, if a request is made to return <code>maxResults</code> number of items, <code>NextToken</code> allows you to return more items in your list starting at the location pointed to by the next token.<br/>ResourceArn:  An ARN that uniquely identifies a resource. The format of the ARN depends on the resource type.<br/>
        :return: RecoveryPoints:  An array of objects that contain detailed information about recovery points of the specified resource type.<br/>NextToken:  The next item following a partial list of returned items. For example, if a request is made to return <code>maxResults</code> number of items, <code>NextToken</code> allows you to return more items in your list starting at the location pointed to by the next token.<br/>        """
        pass

    def list_restore_jobs(self, NextToken: str=None, MaxResults: int=None) -> ListRestoreJobsOutput:
        """Returns a list of jobs that AWS Backup initiated to restore a saved resource, including metadata about the recovery process.

        :param self:
        :param MaxResults:  The maximum number of items to be returned.<br/>NextToken:  The next item following a partial list of returned items. For example, if a request is made to return <code>maxResults</code> number of items, <code>NextToken</code> allows you to return more items in your list starting at the location pointed to by the next token.<br/>
        :return: NextToken:  The next item following a partial list of returned items. For example, if a request is made to return <code>maxResults</code> number of items, <code>NextToken</code> allows you to return more items in your list starting at the location pointed to by the next token.<br/>RestoreJobs:  An array of objects that contain detailed information about jobs to restore saved resources.<br/>        """
        pass

    def list_tags(self, ResourceArn: str, NextToken: str=None, MaxResults: int=None) -> ListTagsOutput:
        """Returns a list of key-value pairs assigned to a target recovery point, backup plan, or backup vault.

        :param self:
        :param MaxResults:  The maximum number of items to be returned.<br/>NextToken:  The next item following a partial list of returned items. For example, if a request is made to return <code>maxResults</code> number of items, <code>NextToken</code> allows you to return more items in your list starting at the location pointed to by the next token.<br/>ResourceArn:  An Amazon Resource Name (ARN) that uniquely identifies a resource. The format of the ARN depends on the type of resource. Valid targets for <code>ListTags</code> are recovery points, backup plans, and backup vaults.<br/>
        :return: Tags:  To help organize your resources, you can assign your own metadata to the resources you create. Each tag is a key-value pair.<br/>NextToken:  The next item following a partial list of returned items. For example, if a request is made to return <code>maxResults</code> number of items, <code>NextToken</code> allows you to return more items in your list starting at the location pointed to by the next token.<br/>        """
        pass

    def put_backup_vault_access_policy(self, BackupVaultName: str, Policy: str=None):
        """Sets a resource-based policy that is used to manage access permissions on the target backup vault. Requires a backup vault name and an access policy document in JSON format.

        :param self:
        :param Policy:  The backup vault access policy document in JSON format.<br/>BackupVaultName:  The name of a logical container where backups are stored. Backup vaults are identified by names that are unique to the account used to create them and the AWS Region where they are created. They consist of lowercase letters, numbers, and hyphens.<br/>
        :return:         """
        pass

    def put_backup_vault_notifications(self, BackupVaultEvents: List[BackupVaultEvent], SNSTopicArn: str, BackupVaultName: str):
        """Turns on notifications on a backup vault for the specified topic and events.

        :param self:
        :param BackupVaultEvents:  An array of events that indicate the status of jobs to back up resources to the backup vault.<br/>SNSTopicArn:  The Amazon Resource Name (ARN) that specifies the topic for a backup vault’s events; for example, <code>arn:aws:sns:us-west-2:111122223333:MyVaultTopic</code>.<br/>BackupVaultName:  The name of a logical container where backups are stored. Backup vaults are identified by names that are unique to the account used to create them and the AWS Region where they are created. They consist of lowercase letters, numbers, and hyphens.<br/>
        :return:         """
        pass

    def start_backup_job(self, IamRoleArn: str, ResourceArn: str, BackupVaultName: str, IdempotencyToken: str=None, StartWindowMinutes: WindowMinutes=None, CompleteWindowMinutes: WindowMinutes=None, Lifecycle: Lifecycle=None, RecoveryPointTags: Tags=None) -> StartBackupJobOutput:
        """Starts a job to create a one-time backup of the specified resource.

        :param self:
        :param RecoveryPointTags:  To help organize your resources, you can assign your own metadata to the resources that you create. Each tag is a key-value pair.<br/>Lifecycle:  The lifecycle defines when a protected resource is transitioned to cold storage and when it expires. AWS Backup will transition and expire backups automatically according to the lifecycle that you define.  Backups transitioned to cold storage must be stored in cold storage for a minimum of 90 days. Therefore, the “expire after days” setting must be 90 days greater than the “transition to cold after days” setting. The “transition to cold after days” setting cannot be changed after a backup has been transitioned to cold. :<br/>CompleteWindowMinutes:  The amount of time AWS Backup attempts a backup before canceling the job and returning an error.<br/>StartWindowMinutes:  The amount of time in minutes before beginning a backup.<br/>IdempotencyToken:  A customer chosen string that can be used to distinguish between calls to <code>StartBackupJob</code>. Idempotency tokens time out after one hour. Therefore, if you call <code>StartBackupJob</code> multiple times with the same idempotency token within one hour, AWS Backup recognizes that you are requesting only one backup job and initiates only one. If you change the idempotency token for each call, AWS Backup recognizes that you are requesting to start multiple backups.<br/>IamRoleArn:  Specifies the IAM role ARN used to create the target recovery point; for example, <code>arn:aws:iam::123456789012:role/S3Access</code>.<br/>ResourceArn:  An Amazon Resource Name (ARN) that uniquely identifies a resource. The format of the ARN depends on the resource type.<br/>BackupVaultName:  The name of a logical container where backups are stored. Backup vaults are identified by names that are unique to the account used to create them and the AWS Region where they are created. They consist of lowercase letters, numbers, and hyphens.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeleteAfterDays</i> Specifies the number of days after creation that a recovery point is deleted. Must be greater than <code>MoveToColdStorageAfterDays</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MoveToColdStorageAfterDays</i> Specifies the number of days after creation that a recovery point is moved to cold storage.<br/>
        :return: CreationDate:  The date and time that a backup job is started, in Unix format and Coordinated Universal Time (UTC). The value of <code>CreationDate</code> is accurate to milliseconds. For example, the value 1516925490.087 represents Friday, January 26, 2018 12:11:30.087 AM.<br/>RecoveryPointArn:  An ARN that uniquely identifies a recovery point; for example, <code>arn:aws:backup:us-east-1:123456789012:recovery-point:1EB3B5E7-9EB0-435A-A80B-108B488B0D45</code>.<br/>BackupJobId:  Uniquely identifies a request to AWS Backup to back up a resource.<br/>        """
        pass

    def start_restore_job(self, IamRoleArn: str, Metadata: Metadata, RecoveryPointArn: str, IdempotencyToken: str=None, ResourceType: str=None) -> StartRestoreJobOutput:
        """Recovers the saved resource identified by an Amazon Resource Name (ARN).  If the resource ARN is included in the request, then the last complete backup of that resource is recovered. If the ARN of a recovery point is supplied, then that recovery point is restored.

        :param self:
        :param ResourceType:  Starts a job to restore a recovery point for one of the following resources: <ul> <li>  <code>EBS</code> for Amazon Elastic Block Store </li> <li>  <code>SGW</code> for AWS Storage Gateway </li> <li>  <code>RDS</code> for Amazon Relational Database Service </li> <li>  <code>DDB</code> for Amazon DynamoDB </li> <li>  <code>EFS</code> for Amazon Elastic File System </li> </ul><br/>IdempotencyToken:  A customer chosen string that can be used to distinguish between calls to <code>StartRestoreJob</code>. Idempotency tokens time out after one hour. Therefore, if you call <code>StartRestoreJob</code> multiple times with the same idempotency token within one hour, AWS Backup recognizes that you are requesting only one restore job and initiates only one. If you change the idempotency token for each call, AWS Backup recognizes that you are requesting to start multiple restores. <br/>IamRoleArn:  The Amazon Resource Name (ARN) of the IAM role that AWS Backup uses to create the target recovery point; for example, <code>arn:aws:iam::123456789012:role/S3Access</code>.<br/>Metadata:  A set of metadata key-value pairs. Lists the metadata that the recovery point was created with.<br/>RecoveryPointArn:  An ARN that uniquely identifies a recovery point; for example, <code>arn:aws:backup:us-east-1:123456789012:recovery-point:1EB3B5E7-9EB0-435A-A80B-108B488B0D45</code>.<br/>
        :return: RestoreJobId:  Uniquely identifies the job that restores a recovery point.<br/>        """
        pass

    def stop_backup_job(self, BackupJobId: str):
        """Attempts to cancel a job to create a one-time backup of a resource.

        :param self:
        :param BackupJobId:  Uniquely identifies a request to AWS Backup to back up a resource.<br/>
        :return:         """
        pass

    def tag_resource(self, Tags: Tags, ResourceArn: str):
        """Assigns a set of key-value pairs to a recovery point, backup plan, or backup vault identified by an Amazon Resource Name (ARN).

        :param self:
        :param Tags:  Key-value pairs that are used to help organize your resources. You can assign your own metadata to the resources you create. <br/>ResourceArn:  An ARN that uniquely identifies a resource. The format of the ARN depends on the type of the tagged resource.<br/>
        :return:         """
        pass

    def untag_resource(self, TagKeyList: List[string], ResourceArn: str):
        """Removes a set of key-value pairs from a recovery point, backup plan, or backup vault identified by an Amazon Resource Name (ARN)

        :param self:
        :param TagKeyList:  A list of keys to identify which key-value tags to remove from a resource.<br/>ResourceArn:  An ARN that uniquely identifies a resource. The format of the ARN depends on the type of the tagged resource.<br/>
        :return:         """
        pass

    def update_backup_plan(self, BackupPlan: BackupPlanInput, BackupPlanId: str) -> UpdateBackupPlanOutput:
        """Replaces the body of a saved backup plan identified by its <code>backupPlanId</code> with the input document in JSON format. The new version is uniquely identified by a <code>VersionId</code>.

        :param self:
        :param BackupPlan:  Specifies the body of a backup plan. Includes a <code>BackupPlanName</code> and one or more sets of <code>Rules</code>.:<br/>BackupPlanId:  Uniquely identifies a backup plan.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Rules</i> An array of <code>BackupRule</code> objects, each of which specifies a scheduled task that is used to back up a selection of resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupPlanName</i> The display name of a backup plan.<br/>
        :return: VersionId:  Unique, randomly generated, Unicode, UTF-8 encoded strings that are at most 1,024 bytes long. Version Ids cannot be edited.<br/>CreationDate:  The date and time a backup plan is updated, in Unix format and Coordinated Universal Time (UTC). The value of <code>CreationDate</code> is accurate to milliseconds. For example, the value 1516925490.087 represents Friday, January 26, 2018 12:11:30.087 AM.<br/>BackupPlanArn:  An Amazon Resource Name (ARN) that uniquely identifies a backup plan; for example, <code>arn:aws:backup:us-east-1:123456789012:plan:8F81F553-3A74-4A3F-B93D-B3360DC80C50</code>.<br/>BackupPlanId:  Uniquely identifies a backup plan.<br/>        """
        pass

    def update_recovery_point_lifecycle(self, RecoveryPointArn: str, BackupVaultName: str, Lifecycle: Lifecycle=None) -> UpdateRecoveryPointLifecycleOutput:
        """Sets the transition lifecycle of a recovery point. The lifecycle defines when a protected resource is transitioned to cold storage and when it expires. AWS Backup transitions and expires backups automatically according to the lifecycle that you define.  Backups transitioned to cold storage must be stored in cold storage for a minimum of 90 days. Therefore, the “expire after days” setting must be 90 days greater than the “transition to cold after days” setting. The “transition to cold after days” setting cannot be changed after a backup has been transitioned to cold. 

        :param self:
        :param Lifecycle:  The lifecycle defines when a protected resource is transitioned to cold storage and when it expires. AWS Backup transitions and expires backups automatically according to the lifecycle that you define.  Backups transitioned to cold storage must be stored in cold storage for a minimum of 90 days. Therefore, the “expire after days” setting must be 90 days greater than the “transition to cold after days” setting. The “transition to cold after days” setting cannot be changed after a backup has been transitioned to cold. :<br/>RecoveryPointArn:  An Amazon Resource Name (ARN) that uniquely identifies a recovery point; for example, <code>arn:aws:backup:us-east-1:123456789012:recovery-point:1EB3B5E7-9EB0-435A-A80B-108B488B0D45</code>.<br/>BackupVaultName:  The name of a logical container where backups are stored. Backup vaults are identified by names that are unique to the account used to create them and the AWS Region where they are created. They consist of lowercase letters, numbers, and hyphens.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeleteAfterDays</i> Specifies the number of days after creation that a recovery point is deleted. Must be greater than <code>MoveToColdStorageAfterDays</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MoveToColdStorageAfterDays</i> Specifies the number of days after creation that a recovery point is moved to cold storage.<br/>
        :return: CalculatedLifecycle:  A <code>CalculatedLifecycle</code> object containing <code>DeleteAt</code> and <code>MoveToColdStorageAt</code> timestamps.:<br/>Lifecycle:  The lifecycle defines when a protected resource is transitioned to cold storage and when it expires. AWS Backup transitions and expires backups automatically according to the lifecycle that you define.  Backups transitioned to cold storage must be stored in cold storage for a minimum of 90 days. Therefore, the “expire after days” setting must be 90 days greater than the “transition to cold after days” setting. The “transition to cold after days” setting cannot be changed after a backup has been transitioned to cold. :<br/>RecoveryPointArn:  An Amazon Resource Name (ARN) that uniquely identifies a recovery point; for example, <code>arn:aws:backup:us-east-1:123456789012:recovery-point:1EB3B5E7-9EB0-435A-A80B-108B488B0D45</code>.<br/>BackupVaultArn:  An ARN that uniquely identifies a backup vault; for example, <code>arn:aws:backup:us-east-1:123456789012:vault:aBackupVault</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeleteAfterDays</i> Specifies the number of days after creation that a recovery point is deleted. Must be greater than <code>MoveToColdStorageAfterDays</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MoveToColdStorageAfterDays</i> Specifies the number of days after creation that a recovery point is moved to cold storage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeleteAt</i> A timestamp that specifies when to delete a recovery point.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MoveToColdStorageAt</i> A timestamp that specifies when to transition a recovery point to cold storage.<br/>        """
        pass



class Batch(BaseClient):
    """AWS Batch enables you to run batch computing workloads on the AWS Cloud. Batch computing is a common way for developers, scientists, and engineers to access large amounts of compute resources, and AWS Batch removes the undifferentiated heavy lifting of configuring and managing the required infrastructure. AWS Batch will be familiar to users of traditional batch computing software. This service can efficiently provision resources in response to jobs submitted in order to eliminate capacity constraints, reduce compute costs, and deliver results quickly. As a fully managed service, AWS Batch enables developers, scientists, and engineers to run batch computing workloads of any scale. AWS Batch automatically provisions compute resources and optimizes the workload distribution based on the quantity and scale of the workloads. With AWS Batch, there is no need to install or manage batch computing software, which allows you to focus on analyzing results and solving problems. AWS Batch reduces operational complexities, saves time, and reduces costs, which makes it easy for developers, scientists, and engineers to run their batch jobs in the AWS Cloud."""


    class DescribeComputeEnvironmentsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_compute_environments` for valid parameters"""
            pass
    class DescribeJobDefinitionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_job_definitions` for valid parameters"""
            pass
    class DescribeJobQueuesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_job_queues` for valid parameters"""
            pass
    class ListJobsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_jobs` for valid parameters"""
            pass


    class CancelJobRequest(Mapping):
        pass
    
    class CancelJobResponse(Mapping):
        pass
    
    class CreateComputeEnvironmentRequest(Mapping):
        pass
    
    class CreateComputeEnvironmentResponse(Mapping):
        pass
    
    class ComputeResource(Mapping):
        pass
    
    class CreateJobQueueRequest(Mapping):
        pass
    
    class CreateJobQueueResponse(Mapping):
        pass
    
    class ComputeEnvironmentOrder(Mapping):
        pass
    
    class DeleteComputeEnvironmentRequest(Mapping):
        pass
    
    class DeleteComputeEnvironmentResponse(Mapping):
        pass
    
    class DeleteJobQueueRequest(Mapping):
        pass
    
    class DeleteJobQueueResponse(Mapping):
        pass
    
    class DeregisterJobDefinitionRequest(Mapping):
        pass
    
    class DeregisterJobDefinitionResponse(Mapping):
        pass
    
    class DescribeComputeEnvironmentsRequest(Mapping):
        pass
    
    class DescribeComputeEnvironmentsResponse(Mapping):
        pass
    
    class String(object):
        pass
    
    class DescribeJobDefinitionsRequest(Mapping):
        pass
    
    class DescribeJobDefinitionsResponse(Mapping):
        pass
    
    class DescribeJobQueuesRequest(Mapping):
        pass
    
    class DescribeJobQueuesResponse(Mapping):
        pass
    
    class DescribeJobsRequest(Mapping):
        pass
    
    class DescribeJobsResponse(Mapping):
        pass
    
    class ListJobsRequest(Mapping):
        pass
    
    class ListJobsResponse(Mapping):
        pass
    
    class RegisterJobDefinitionRequest(Mapping):
        pass
    
    class RegisterJobDefinitionResponse(Mapping):
        pass
    
    class ParametersMap(object):
        pass
    
    class ContainerProperties(Mapping):
        pass
    
    class NodeProperties(Mapping):
        pass
    
    class RetryStrategy(Mapping):
        pass
    
    class JobTimeout(Mapping):
        pass
    
    class SubmitJobRequest(Mapping):
        pass
    
    class SubmitJobResponse(Mapping):
        pass
    
    class ArrayProperties(Mapping):
        pass
    
    class JobDependency(Mapping):
        pass
    
    class ContainerOverrides(Mapping):
        pass
    
    class NodeOverrides(Mapping):
        pass
    
    class TerminateJobRequest(Mapping):
        pass
    
    class TerminateJobResponse(Mapping):
        pass
    
    class UpdateComputeEnvironmentRequest(Mapping):
        pass
    
    class UpdateComputeEnvironmentResponse(Mapping):
        pass
    
    class ComputeResourceUpdate(Mapping):
        pass
    
    class UpdateJobQueueRequest(Mapping):
        pass
    
    class UpdateJobQueueResponse(Mapping):
        pass
    
    def cancel_job(self, reason: str, jobId: str) -> CancelJobResponse:
        """Cancels a job in an AWS Batch job queue. Jobs that are in the <code>SUBMITTED</code>, <code>PENDING</code>, or <code>RUNNABLE</code> state are cancelled. Jobs that have progressed to <code>STARTING</code> or <code>RUNNING</code> are not cancelled (but the API operation still succeeds, even if no job is cancelled); these jobs must be terminated with the <a>TerminateJob</a> operation.

        :param self:
        :param reason:  A message to attach to the job that explains the reason for canceling it. This message is returned by future <a>DescribeJobs</a> operations on the job. This message is also recorded in the AWS Batch activity logs. <br/>jobId:  The AWS Batch job ID of the job to cancel.<br/>
        :return:         """
        pass

    def create_compute_environment(self, serviceRole: str, type: str, computeEnvironmentName: str, state: str=None, computeResources: ComputeResource=None) -> CreateComputeEnvironmentResponse:
        """Creates an AWS Batch compute environment. You can create <code>MANAGED</code> or <code>UNMANAGED</code> compute environments. In a managed compute environment, AWS Batch manages the capacity and instance types of the compute resources within the environment. This is based on the compute resource specification that you define or the <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-launch-templates.html">launch template</a> that you specify when you create the compute environment. You can choose to use Amazon EC2 On-Demand Instances or Spot Instances in your managed compute environment. You can optionally set a maximum price so that Spot Instances only launch when the Spot Instance price is below a specified percentage of the On-Demand price. <note> Multi-node parallel jobs are not supported on Spot Instances. </note> In an unmanaged compute environment, you can manage your own compute resources. This provides more compute resource configuration options, such as using a custom AMI, but you must ensure that your AMI meets the Amazon ECS container instance AMI specification. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/container_instance_AMIs.html">Container Instance AMIs</a> in the <i>Amazon Elastic Container Service Developer Guide</i>. After you have created your unmanaged compute environment, you can use the <a>DescribeComputeEnvironments</a> operation to find the Amazon ECS cluster that is associated with it. Then, manually launch your container instances into that Amazon ECS cluster. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/launch_container_instance.html">Launching an Amazon ECS Container Instance</a> in the <i>Amazon Elastic Container Service Developer Guide</i>. <note> AWS Batch does not upgrade the AMIs in a compute environment after it is created (for example, when a newer version of the Amazon ECS-optimized AMI is available). You are responsible for the management of the guest operating system (including updates and security patches) and any additional application software or utilities that you install on the compute resources. To use a new AMI for your AWS Batch jobs: <ol> <li> Create a new compute environment with the new AMI. </li> <li> Add the compute environment to an existing job queue. </li> <li> Remove the old compute environment from your job queue. </li> <li> Delete the old compute environment. </li> </ol> </note>

        :param self:
        :param serviceRole:  The full Amazon Resource Name (ARN) of the IAM role that allows AWS Batch to make calls to other AWS services on your behalf. If your specified role has a path other than <code>/</code>, then you must either specify the full role ARN (this is recommended) or prefix the role name with the path. <note> Depending on how you created your AWS Batch service role, its ARN may contain the <code>service-role</code> path prefix. When you only specify the name of the service role, AWS Batch assumes that your ARN does not use the <code>service-role</code> path prefix. Because of this, we recommend that you specify the full ARN of your service role when you create compute environments. </note><br/>computeResources:  Details of the compute resources managed by the compute environment. This parameter is required for managed compute environments. For more information, see <a href="https://docs.aws.amazon.com/batch/latest/userguide/compute_environments.html">Compute Environments</a> in the <i>AWS Batch User Guide</i>.:<br/>state:  The state of the compute environment. If the state is <code>ENABLED</code>, then the compute environment accepts jobs from a queue and can scale out automatically based on queues.<br/>type:  The type of the compute environment. For more information, see <a href="https://docs.aws.amazon.com/batch/latest/userguide/compute_environments.html">Compute Environments</a> in the <i>AWS Batch User Guide</i>.<br/>computeEnvironmentName:  The name for your compute environment. Up to 128 letters (uppercase and lowercase), numbers, hyphens, and underscores are allowed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>launchTemplate</i> The launch template to use for your compute resources. Any other compute resource parameters that you specify in a <a>CreateComputeEnvironment</a> API operation override the same parameters in the launch template. You must specify either the launch template ID or launch template name in the request, but not both. For more information, see <a href="https://docs.aws.amazon.com/batch/latest/userguide/launch-templates.html">Launch Template Support</a> in the <i>AWS Batch User Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>launchTemplate</b>: The launch template to use for your compute resources. Any other compute resource parameters that you specify in a <a>CreateComputeEnvironment</a> API operation override the same parameters in the launch template. You must specify either the launch template ID or launch template name in the request, but not both. For more information, see <a href="https://docs.aws.amazon.com/batch/latest/userguide/launch-templates.html">Launch Template Support</a> in the <i>AWS Batch User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>spotIamFleetRole</i> The Amazon Resource Name (ARN) of the Amazon EC2 Spot Fleet IAM role applied to a <code>SPOT</code> compute environment. For more information, see <a href="https://docs.aws.amazon.com/batch/latest/userguide/spot_fleet_IAM_role.html">Amazon EC2 Spot Fleet Role</a> in the <i>AWS Batch User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>bidPercentage</i> The maximum percentage that a Spot Instance price can be when compared with the On-Demand price for that instance type before instances are launched. For example, if your maximum percentage is 20%, then the Spot price must be below 20% of the current On-Demand price for that EC2 instance. You always pay the lowest (market) price and never more than your maximum percentage. If you leave this field empty, the default value is 100% of the On-Demand price.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>placementGroup</i> The Amazon EC2 placement group to associate with your compute resources. If you intend to submit multi-node parallel jobs to your compute environment, you should consider creating a cluster placement group and associate it with your compute resources. This keeps your multi-node parallel job on a logical grouping of instances within a single Availability Zone with high network flow potential. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/placement-groups.html">Placement Groups</a> in the <i>Amazon EC2 User Guide for Linux Instances</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> Key-value pair tags to be applied to resources that are launched in the compute environment. For AWS Batch, these take the form of "String1": "String2", where String1 is the tag key and String2 is the tag value—for example, { "Name": "AWS Batch Instance - C4OnDemand" }.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>instanceRole</i> The Amazon ECS instance profile applied to Amazon EC2 instances in a compute environment. You can specify the short name or full Amazon Resource Name (ARN) of an instance profile. For example, <code> <i>ecsInstanceRole</i> </code> or <code>arn:aws:iam::<i>&lt;aws_account_id&gt;</i>:instance-profile/<i>ecsInstanceRole</i> </code>. For more information, see <a href="https://docs.aws.amazon.com/batch/latest/userguide/instance_IAM_role.html">Amazon ECS Instance Role</a> in the <i>AWS Batch User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ec2KeyPair</i> The EC2 key pair that is used for instances launched in the compute environment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>securityGroupIds</i> The EC2 security group that is associated with instances launched in the compute environment. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>subnets</i> The VPC subnets into which the compute resources are launched. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>imageId</i> The Amazon Machine Image (AMI) ID used for instances launched in the compute environment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>instanceTypes</i> The instances types that may be launched. You can specify instance families to launch any instance type within those families (for example, <code>c4</code> or <code>p3</code>), or you can specify specific sizes within a family (such as <code>c4.8xlarge</code>). You can also choose <code>optimal</code> to pick instance types (from the C, M, and R instance families) on the fly that match the demand of your job queues.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>desiredvCpus</i> The desired number of EC2 vCPUS in the compute environment. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>maxvCpus</i> The maximum number of EC2 vCPUs that an environment can reach. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>minvCpus</i> The minimum number of EC2 vCPUs that an environment should maintain (even if the compute environment is <code>DISABLED</code>). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of compute environment: EC2 or SPOT.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version number of the launch template. Default: The default version of the launch template.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>launchTemplateName</i> The name of the launch template.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>launchTemplateId</i> The ID of the launch template.<br/>
        :return: computeEnvironmentArn:  The Amazon Resource Name (ARN) of the compute environment. <br/>computeEnvironmentName:  The name of the compute environment.<br/>        """
        pass

    def create_job_queue(self, computeEnvironmentOrder: List[ComputeEnvironmentOrder], priority: int, jobQueueName: str, state: str=None) -> CreateJobQueueResponse:
        """Creates an AWS Batch job queue. When you create a job queue, you associate one or more compute environments to the queue and assign an order of preference for the compute environments. You also set a priority to the job queue that determines the order in which the AWS Batch scheduler places jobs onto its associated compute environments. For example, if a compute environment is associated with more than one job queue, the job queue with a higher priority is given preference for scheduling jobs to that compute environment.

        :param self:
        :param computeEnvironmentOrder:  The set of compute environments mapped to a job queue and their order relative to each other. The job scheduler uses this parameter to determine which compute environment should execute a given job. Compute environments must be in the <code>VALID</code> state before you can associate them with a job queue. You can associate up to three compute environments with a job queue.<br/>priority:  The priority of the job queue. Job queues with a higher priority (or a higher integer value for the <code>priority</code> parameter) are evaluated first when associated with the same compute environment. Priority is determined in descending order, for example, a job queue with a priority value of <code>10</code> is given scheduling preference over a job queue with a priority value of <code>1</code>.<br/>state:  The state of the job queue. If the job queue state is <code>ENABLED</code>, it is able to accept jobs.<br/>jobQueueName:  The name of the job queue.<br/>
        :return: jobQueueArn:  The Amazon Resource Name (ARN) of the job queue.<br/>jobQueueName:  The name of the job queue.<br/>        """
        pass

    def delete_compute_environment(self, computeEnvironment: str) -> DeleteComputeEnvironmentResponse:
        """Deletes an AWS Batch compute environment. Before you can delete a compute environment, you must set its state to <code>DISABLED</code> with the <a>UpdateComputeEnvironment</a> API operation and disassociate it from any job queues with the <a>UpdateJobQueue</a> API operation.

        :param self:
        :param computeEnvironment:  The name or Amazon Resource Name (ARN) of the compute environment to delete. <br/>
        :return:         """
        pass

    def delete_job_queue(self, jobQueue: str) -> DeleteJobQueueResponse:
        """Deletes the specified job queue. You must first disable submissions for a queue with the <a>UpdateJobQueue</a> operation. All jobs in the queue are terminated when you delete a job queue. It is not necessary to disassociate compute environments from a queue before submitting a <code>DeleteJobQueue</code> request. 

        :param self:
        :param jobQueue:  The short name or full Amazon Resource Name (ARN) of the queue to delete. <br/>
        :return:         """
        pass

    def deregister_job_definition(self, jobDefinition: str) -> DeregisterJobDefinitionResponse:
        """Deregisters an AWS Batch job definition.

        :param self:
        :param jobDefinition:  The name and revision (<code>name:revision</code>) or full Amazon Resource Name (ARN) of the job definition to deregister. <br/>
        :return:         """
        pass

    def describe_compute_environments(self, computeEnvironments: List[String]=None, maxResults: int=None, nextToken: str=None) -> DescribeComputeEnvironmentsResponse:
        """Describes one or more of your compute environments. If you are using an unmanaged compute environment, you can use the <code>DescribeComputeEnvironment</code> operation to determine the <code>ecsClusterArn</code> that you should launch your Amazon ECS container instances into.

        :param self:
        :param nextToken:  The <code>nextToken</code> value returned from a previous paginated <code>DescribeComputeEnvironments</code> request where <code>maxResults</code> was used and the results exceeded the value of that parameter. Pagination continues from the end of the previous results that returned the <code>nextToken</code> value. This value is <code>null</code> when there are no more results to return. <note> This token should be treated as an opaque identifier that is only used to retrieve the next items in a list and not for other programmatic purposes. </note><br/>maxResults:  The maximum number of cluster results returned by <code>DescribeComputeEnvironments</code> in paginated output. When this parameter is used, <code>DescribeComputeEnvironments</code> only returns <code>maxResults</code> results in a single page along with a <code>nextToken</code> response element. The remaining results of the initial request can be seen by sending another <code>DescribeComputeEnvironments</code> request with the returned <code>nextToken</code> value. This value can be between 1 and 100. If this parameter is not used, then <code>DescribeComputeEnvironments</code> returns up to 100 results and a <code>nextToken</code> value if applicable.<br/>computeEnvironments:  A list of up to 100 compute environment names or full Amazon Resource Name (ARN) entries. <br/>
        :return: nextToken:  The <code>nextToken</code> value to include in a future <code>DescribeComputeEnvironments</code> request. When the results of a <code>DescribeJobDefinitions</code> request exceed <code>maxResults</code>, this value can be used to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>computeEnvironments:  The list of compute environments.<br/>        """
        pass

    def describe_job_definitions(self, jobDefinitions: List[String]=None, maxResults: int=None, jobDefinitionName: str=None, status: str=None, nextToken: str=None) -> DescribeJobDefinitionsResponse:
        """Describes a list of job definitions. You can specify a <code>status</code> (such as <code>ACTIVE</code>) to only return job definitions that match that status.

        :param self:
        :param nextToken:  The <code>nextToken</code> value returned from a previous paginated <code>DescribeJobDefinitions</code> request where <code>maxResults</code> was used and the results exceeded the value of that parameter. Pagination continues from the end of the previous results that returned the <code>nextToken</code> value. This value is <code>null</code> when there are no more results to return. <note> This token should be treated as an opaque identifier that is only used to retrieve the next items in a list and not for other programmatic purposes. </note><br/>status:  The status with which to filter job definitions.<br/>jobDefinitionName:  The name of the job definition to describe.<br/>maxResults:  The maximum number of results returned by <code>DescribeJobDefinitions</code> in paginated output. When this parameter is used, <code>DescribeJobDefinitions</code> only returns <code>maxResults</code> results in a single page along with a <code>nextToken</code> response element. The remaining results of the initial request can be seen by sending another <code>DescribeJobDefinitions</code> request with the returned <code>nextToken</code> value. This value can be between 1 and 100. If this parameter is not used, then <code>DescribeJobDefinitions</code> returns up to 100 results and a <code>nextToken</code> value if applicable.<br/>jobDefinitions:  A list of up to 100 job definition names or full Amazon Resource Name (ARN) entries.<br/>
        :return: nextToken:  The <code>nextToken</code> value to include in a future <code>DescribeJobDefinitions</code> request. When the results of a <code>DescribeJobDefinitions</code> request exceed <code>maxResults</code>, this value can be used to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>jobDefinitions:  The list of job definitions. <br/>        """
        pass

    def describe_job_queues(self, jobQueues: List[String]=None, maxResults: int=None, nextToken: str=None) -> DescribeJobQueuesResponse:
        """Describes one or more of your job queues.

        :param self:
        :param nextToken:  The <code>nextToken</code> value returned from a previous paginated <code>DescribeJobQueues</code> request where <code>maxResults</code> was used and the results exceeded the value of that parameter. Pagination continues from the end of the previous results that returned the <code>nextToken</code> value. This value is <code>null</code> when there are no more results to return. <note> This token should be treated as an opaque identifier that is only used to retrieve the next items in a list and not for other programmatic purposes. </note><br/>maxResults:  The maximum number of results returned by <code>DescribeJobQueues</code> in paginated output. When this parameter is used, <code>DescribeJobQueues</code> only returns <code>maxResults</code> results in a single page along with a <code>nextToken</code> response element. The remaining results of the initial request can be seen by sending another <code>DescribeJobQueues</code> request with the returned <code>nextToken</code> value. This value can be between 1 and 100. If this parameter is not used, then <code>DescribeJobQueues</code> returns up to 100 results and a <code>nextToken</code> value if applicable.<br/>jobQueues:  A list of up to 100 queue names or full queue Amazon Resource Name (ARN) entries.<br/>
        :return: nextToken:  The <code>nextToken</code> value to include in a future <code>DescribeJobQueues</code> request. When the results of a <code>DescribeJobQueues</code> request exceed <code>maxResults</code>, this value can be used to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>jobQueues:  The list of job queues. <br/>        """
        pass

    def describe_jobs(self, jobs: List[String]) -> DescribeJobsResponse:
        """Describes a list of AWS Batch jobs.

        :param self:
        :param jobs:  A list of up to 100 job IDs.<br/>
        :return: jobs:  The list of jobs. <br/>        """
        pass

    def list_jobs(self, jobQueue: str=None, arrayJobId: str=None, multiNodeJobId: str=None, jobStatus: str=None, maxResults: int=None, nextToken: str=None) -> ListJobsResponse:
        """Returns a list of AWS Batch jobs. You must specify only one of the following: <ul> <li> a job queue ID to return a list of jobs in that job queue </li> <li> a multi-node parallel job ID to return a list of that job's nodes </li> <li> an array job ID to return a list of that job's children </li> </ul> You can filter the results by job status with the <code>jobStatus</code> parameter. If you do not specify a status, only <code>RUNNING</code> jobs are returned.

        :param self:
        :param nextToken:  The <code>nextToken</code> value returned from a previous paginated <code>ListJobs</code> request where <code>maxResults</code> was used and the results exceeded the value of that parameter. Pagination continues from the end of the previous results that returned the <code>nextToken</code> value. This value is <code>null</code> when there are no more results to return. <note> This token should be treated as an opaque identifier that is only used to retrieve the next items in a list and not for other programmatic purposes. </note><br/>maxResults:  The maximum number of results returned by <code>ListJobs</code> in paginated output. When this parameter is used, <code>ListJobs</code> only returns <code>maxResults</code> results in a single page along with a <code>nextToken</code> response element. The remaining results of the initial request can be seen by sending another <code>ListJobs</code> request with the returned <code>nextToken</code> value. This value can be between 1 and 100. If this parameter is not used, then <code>ListJobs</code> returns up to 100 results and a <code>nextToken</code> value if applicable.<br/>jobStatus:  The job status with which to filter jobs in the specified queue. If you do not specify a status, only <code>RUNNING</code> jobs are returned.<br/>multiNodeJobId:  The job ID for a multi-node parallel job. Specifying a multi-node parallel job ID with this parameter lists all nodes that are associated with the specified job.<br/>arrayJobId:  The job ID for an array job. Specifying an array job ID with this parameter lists all child jobs from within the specified array.<br/>jobQueue:  The name or full Amazon Resource Name (ARN) of the job queue with which to list jobs.<br/>
        :return: nextToken:  The <code>nextToken</code> value to include in a future <code>ListJobs</code> request. When the results of a <code>ListJobs</code> request exceed <code>maxResults</code>, this value can be used to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>jobSummaryList:  A list of job summaries that match the request.<br/>        """
        pass

    def register_job_definition(self, type: str, jobDefinitionName: str, parameters: ParametersMap=None, containerProperties: ContainerProperties=None, nodeProperties: NodeProperties=None, retryStrategy: RetryStrategy=None, timeout: JobTimeout=None) -> RegisterJobDefinitionResponse:
        """Registers an AWS Batch job definition. 

        :param self:
        :param timeout:  The timeout configuration for jobs that are submitted with this job definition, after which AWS Batch terminates your jobs if they have not finished. If a job is terminated due to a timeout, it is not retried. The minimum value for the timeout is 60 seconds. Any timeout configuration that is specified during a <a>SubmitJob</a> operation overrides the timeout configuration defined here. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/job_timeouts.html">Job Timeouts</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.:<br/>retryStrategy:  The retry strategy to use for failed jobs that are submitted with this job definition. Any retry strategy that is specified during a <a>SubmitJob</a> operation overrides the retry strategy defined here. If a job is terminated due to a timeout, it is not retried. :<br/>nodeProperties:  An object with various properties specific to multi-node parallel jobs. If you specify node properties for a job, it becomes a multi-node parallel job. For more information, see <a href="https://docs.aws.amazon.com/batch/latest/userguide/multi-node-parallel-jobs.html">Multi-node Parallel Jobs</a> in the <i>AWS Batch User Guide</i>. If the job definition's <code>type</code> parameter is <code>container</code>, then you must specify either <code>containerProperties</code> or <code>nodeProperties</code>.:<br/>containerProperties:  An object with various properties specific to single-node container-based jobs. If the job definition's <code>type</code> parameter is <code>container</code>, then you must specify either <code>containerProperties</code> or <code>nodeProperties</code>.:<br/>parameters:  Default parameter substitution placeholders to set in the job definition. Parameters are specified as a key-value pair mapping. Parameters in a <code>SubmitJob</code> request override any corresponding parameter defaults from the job definition.<br/>type:  The type of job definition.<br/>jobDefinitionName:  The name of the job definition to register. Up to 128 letters (uppercase and lowercase), numbers, hyphens, and underscores are allowed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceRequirements</i> The type and amount of a resource to assign to a container. Currently, the only supported resource is <code>GPU</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>instanceType</i> The instance type to use for a multi-node parallel job. Currently all node groups in a multi-node parallel job must use the same instance type. This parameter is not valid for single-node container jobs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>user</i> The user name to use inside the container. This parameter maps to <code>User</code> in the <a href="https://docs.docker.com/engine/api/v1.23/#create-a-container">Create a container</a> section of the <a href="https://docs.docker.com/engine/api/v1.23/">Docker Remote API</a> and the <code>--user</code> option to <a href="https://docs.docker.com/engine/reference/run/">docker run</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ulimits</i> A list of <code>ulimits</code> to set in the container. This parameter maps to <code>Ulimits</code> in the <a href="https://docs.docker.com/engine/api/v1.23/#create-a-container">Create a container</a> section of the <a href="https://docs.docker.com/engine/api/v1.23/">Docker Remote API</a> and the <code>--ulimit</code> option to <a href="https://docs.docker.com/engine/reference/run/">docker run</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>privileged</i> When this parameter is true, the container is given elevated privileges on the host container instance (similar to the <code>root</code> user). This parameter maps to <code>Privileged</code> in the <a href="https://docs.docker.com/engine/api/v1.23/#create-a-container">Create a container</a> section of the <a href="https://docs.docker.com/engine/api/v1.23/">Docker Remote API</a> and the <code>--privileged</code> option to <a href="https://docs.docker.com/engine/reference/run/">docker run</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>readonlyRootFilesystem</i> When this parameter is true, the container is given read-only access to its root file system. This parameter maps to <code>ReadonlyRootfs</code> in the <a href="https://docs.docker.com/engine/api/v1.23/#create-a-container">Create a container</a> section of the <a href="https://docs.docker.com/engine/api/v1.23/">Docker Remote API</a> and the <code>--read-only</code> option to <code>docker run</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>mountPoints</i> The mount points for data volumes in your container. This parameter maps to <code>Volumes</code> in the <a href="https://docs.docker.com/engine/api/v1.23/#create-a-container">Create a container</a> section of the <a href="https://docs.docker.com/engine/api/v1.23/">Docker Remote API</a> and the <code>--volume</code> option to <a href="https://docs.docker.com/engine/reference/run/">docker run</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>environment</i> The environment variables to pass to a container. This parameter maps to <code>Env</code> in the <a href="https://docs.docker.com/engine/api/v1.23/#create-a-container">Create a container</a> section of the <a href="https://docs.docker.com/engine/api/v1.23/">Docker Remote API</a> and the <code>--env</code> option to <a href="https://docs.docker.com/engine/reference/run/">docker run</a>. <important> We do not recommend using plaintext environment variables for sensitive information, such as credential data. </important> <note> Environment variables must not start with <code>AWS_BATCH</code>; this naming convention is reserved for variables that are set by the AWS Batch service. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>volumes</i> A list of data volumes used in a job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jobRoleArn</i> The Amazon Resource Name (ARN) of the IAM role that the container can assume for AWS permissions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>command</i> The command that is passed to the container. This parameter maps to <code>Cmd</code> in the <a href="https://docs.docker.com/engine/api/v1.23/#create-a-container">Create a container</a> section of the <a href="https://docs.docker.com/engine/api/v1.23/">Docker Remote API</a> and the <code>COMMAND</code> parameter to <a href="https://docs.docker.com/engine/reference/run/">docker run</a>. For more information, see <a href="https://docs.docker.com/engine/reference/builder/#cmd">https://docs.docker.com/engine/reference/builder/#cmd</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>memory</i> The hard limit (in MiB) of memory to present to the container. If your container attempts to exceed the memory specified here, the container is killed. This parameter maps to <code>Memory</code> in the <a href="https://docs.docker.com/engine/api/v1.23/#create-a-container">Create a container</a> section of the <a href="https://docs.docker.com/engine/api/v1.23/">Docker Remote API</a> and the <code>--memory</code> option to <a href="https://docs.docker.com/engine/reference/run/">docker run</a>. You must specify at least 4 MiB of memory for a job. <note> If you are trying to maximize your resource utilization by providing your jobs as much memory as possible for a particular instance type, see <a href="https://docs.aws.amazon.com/batch/latest/userguide/memory-management.html">Memory Management</a> in the <i>AWS Batch User Guide</i>. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vcpus</i> The number of vCPUs reserved for the container. This parameter maps to <code>CpuShares</code> in the <a href="https://docs.docker.com/engine/api/v1.23/#create-a-container">Create a container</a> section of the <a href="https://docs.docker.com/engine/api/v1.23/">Docker Remote API</a> and the <code>--cpu-shares</code> option to <a href="https://docs.docker.com/engine/reference/run/">docker run</a>. Each vCPU is equivalent to 1,024 CPU shares. You must specify at least one vCPU.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>image</i> The image used to start a container. This string is passed directly to the Docker daemon. Images in the Docker Hub registry are available by default. Other repositories are specified with <code> <i>repository-url</i>/<i>image</i>:<i>tag</i> </code>. Up to 255 letters (uppercase and lowercase), numbers, hyphens, underscores, colons, periods, forward slashes, and number signs are allowed. This parameter maps to <code>Image</code> in the <a href="https://docs.docker.com/engine/api/v1.23/#create-a-container">Create a container</a> section of the <a href="https://docs.docker.com/engine/api/v1.23/">Docker Remote API</a> and the <code>IMAGE</code> parameter of <a href="https://docs.docker.com/engine/reference/run/">docker run</a>. <ul> <li> Images in Amazon ECR repositories use the full registry and repository URI (for example, <code>012345678910.dkr.ecr.&lt;region-name&gt;.amazonaws.com/&lt;repository-name&gt;</code>). </li> <li> Images in official repositories on Docker Hub use a single name (for example, <code>ubuntu</code> or <code>mongo</code>). </li> <li> Images in other repositories on Docker Hub are qualified with an organization name (for example, <code>amazon/amazon-ecs-agent</code>). </li> <li> Images in other online repositories are qualified further by a domain name (for example, <code>quay.io/assemblyline/ubuntu</code>). </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>nodeRangeProperties</i> A list of node ranges and their properties associated with a multi-node parallel job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>mainNode</i> Specifies the node index for the main node of a multi-node parallel job. This node index value must be fewer than the number of nodes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>numNodes</i> The number of nodes associated with a multi-node parallel job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>attempts</i> The number of times to move a job to the <code>RUNNABLE</code> status. You may specify between 1 and 10 attempts. If the value of <code>attempts</code> is greater than one, the job is retried on failure the same number of attempts as the value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>attemptDurationSeconds</i> The time duration in seconds (measured from the job attempt's <code>startedAt</code> timestamp) after which AWS Batch terminates your jobs if they have not finished.<br/>
        :return: revision:  The revision of the job definition.<br/>jobDefinitionArn:  The Amazon Resource Name (ARN) of the job definition. <br/>jobDefinitionName:  The name of the job definition.<br/>        """
        pass

    def submit_job(self, jobDefinition: str, jobQueue: str, jobName: str, arrayProperties: ArrayProperties=None, dependsOn: List[JobDependency]=None, parameters: ParametersMap=None, containerOverrides: ContainerOverrides=None, nodeOverrides: NodeOverrides=None, retryStrategy: RetryStrategy=None, timeout: JobTimeout=None) -> SubmitJobResponse:
        """Submits an AWS Batch job from a job definition. Parameters specified during <a>SubmitJob</a> override parameters defined in the job definition. 

        :param self:
        :param timeout:  The timeout configuration for this <a>SubmitJob</a> operation. You can specify a timeout duration after which AWS Batch terminates your jobs if they have not finished. If a job is terminated due to a timeout, it is not retried. The minimum value for the timeout is 60 seconds. This configuration overrides any timeout configuration specified in the job definition. For array jobs, child jobs have the same timeout configuration as the parent job. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/job_timeouts.html">Job Timeouts</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.:<br/>retryStrategy:  The retry strategy to use for failed jobs from this <a>SubmitJob</a> operation. When a retry strategy is specified here, it overrides the retry strategy defined in the job definition.:<br/>nodeOverrides:  A list of node overrides in JSON format that specify the node range to target and the container overrides for that node range.:<br/>containerOverrides:  A list of container overrides in JSON format that specify the name of a container in the specified job definition and the overrides it should receive. You can override the default command for a container (that is specified in the job definition or the Docker image) with a <code>command</code> override. You can also override existing environment variables (that are specified in the job definition or Docker image) on a container or add new environment variables to it with an <code>environment</code> override.:<br/>parameters:  Additional parameters passed to the job that replace parameter substitution placeholders that are set in the job definition. Parameters are specified as a key and value pair mapping. Parameters in a <code>SubmitJob</code> request override any corresponding parameter defaults from the job definition.<br/>jobDefinition:  The job definition used by this job. This value can be either a <code>name:revision</code> or the Amazon Resource Name (ARN) for the job definition.<br/>dependsOn:  A list of dependencies for the job. A job can depend upon a maximum of 20 jobs. You can specify a <code>SEQUENTIAL</code> type dependency without specifying a job ID for array jobs so that each child array job completes sequentially, starting at index 0. You can also specify an <code>N_TO_N</code> type dependency with a job ID for array jobs. In that case, each index child of this job must wait for the corresponding index child of each dependency to complete before it can begin.<br/>arrayProperties:  The array properties for the submitted job, such as the size of the array. The array size can be between 2 and 10,000. If you specify array properties for a job, it becomes an array job. For more information, see <a href="https://docs.aws.amazon.com/batch/latest/userguide/array_jobs.html">Array Jobs</a> in the <i>AWS Batch User Guide</i>.:<br/>jobQueue:  The job queue into which the job is submitted. You can specify either the name or the Amazon Resource Name (ARN) of the queue. <br/>jobName:  The name of the job. The first character must be alphanumeric, and up to 128 letters (uppercase and lowercase), numbers, hyphens, and underscores are allowed. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>size</i> The size of the array job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceRequirements</i> The type and amount of a resource to assign to a container. This value overrides the value set in the job definition. Currently, the only supported resource is <code>GPU</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>environment</i> The environment variables to send to the container. You can add new environment variables, which are added to the container at launch, or you can override the existing environment variables from the Docker image or the job definition. <note> Environment variables must not start with <code>AWS_BATCH</code>; this naming convention is reserved for variables that are set by the AWS Batch service. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>instanceType</i> The instance type to use for a multi-node parallel job. This parameter is not valid for single-node container jobs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>command</i> The command to send to the container that overrides the default command from the Docker image or the job definition.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>memory</i> The number of MiB of memory reserved for the job. This value overrides the value set in the job definition.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vcpus</i> The number of vCPUs to reserve for the container. This value overrides the value set in the job definition.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>nodePropertyOverrides</i> The node property overrides for the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>numNodes</i> The number of nodes to use with a multi-node parallel job. This value overrides the number of nodes that are specified in the job definition. To use this override: <ul> <li> There must be at least one node range in your job definition that has an open upper boundary (such as <code>:</code> or <code>n:</code>). </li> <li> The lower boundary of the node range specified in the job definition must be fewer than the number of nodes specified in the override. </li> <li> The main node index specified in the job definition must be fewer than the number of nodes specified in the override. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>attempts</i> The number of times to move a job to the <code>RUNNABLE</code> status. You may specify between 1 and 10 attempts. If the value of <code>attempts</code> is greater than one, the job is retried on failure the same number of attempts as the value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>attemptDurationSeconds</i> The time duration in seconds (measured from the job attempt's <code>startedAt</code> timestamp) after which AWS Batch terminates your jobs if they have not finished.<br/>
        :return: jobId:  The unique identifier for the job.<br/>jobName:  The name of the job. <br/>        """
        pass

    def terminate_job(self, reason: str, jobId: str) -> TerminateJobResponse:
        """Terminates a job in a job queue. Jobs that are in the <code>STARTING</code> or <code>RUNNING</code> state are terminated, which causes them to transition to <code>FAILED</code>. Jobs that have not progressed to the <code>STARTING</code> state are cancelled.

        :param self:
        :param reason:  A message to attach to the job that explains the reason for canceling it. This message is returned by future <a>DescribeJobs</a> operations on the job. This message is also recorded in the AWS Batch activity logs. <br/>jobId:  The AWS Batch job ID of the job to terminate.<br/>
        :return:         """
        pass

    def update_compute_environment(self, computeEnvironment: str, state: str=None, computeResources: ComputeResourceUpdate=None, serviceRole: str=None) -> UpdateComputeEnvironmentResponse:
        """Updates an AWS Batch compute environment.

        :param self:
        :param serviceRole:  The full Amazon Resource Name (ARN) of the IAM role that allows AWS Batch to make calls to other AWS services on your behalf. If your specified role has a path other than <code>/</code>, then you must either specify the full role ARN (this is recommended) or prefix the role name with the path. <note> Depending on how you created your AWS Batch service role, its ARN may contain the <code>service-role</code> path prefix. When you only specify the name of the service role, AWS Batch assumes that your ARN does not use the <code>service-role</code> path prefix. Because of this, we recommend that you specify the full ARN of your service role when you create compute environments. </note><br/>computeResources:  Details of the compute resources managed by the compute environment. Required for a managed compute environment.:<br/>state:  The state of the compute environment. Compute environments in the <code>ENABLED</code> state can accept jobs from a queue and scale in or out automatically based on the workload demand of its associated queues.<br/>computeEnvironment:  The name or full Amazon Resource Name (ARN) of the compute environment to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>desiredvCpus</i> The desired number of EC2 vCPUS in the compute environment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>maxvCpus</i> The maximum number of EC2 vCPUs that an environment can reach.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>minvCpus</i> The minimum number of EC2 vCPUs that an environment should maintain.<br/>
        :return: computeEnvironmentArn:  The Amazon Resource Name (ARN) of the compute environment. <br/>computeEnvironmentName:  The name of the compute environment.<br/>        """
        pass

    def update_job_queue(self, jobQueue: str, state: str=None, priority: int=None, computeEnvironmentOrder: List[ComputeEnvironmentOrder]=None) -> UpdateJobQueueResponse:
        """Updates a job queue.

        :param self:
        :param computeEnvironmentOrder:  Details the set of compute environments mapped to a job queue and their order relative to each other. This is one of the parameters used by the job scheduler to determine which compute environment should execute a given job. <br/>priority:  The priority of the job queue. Job queues with a higher priority (or a higher integer value for the <code>priority</code> parameter) are evaluated first when associated with the same compute environment. Priority is determined in descending order, for example, a job queue with a priority value of <code>10</code> is given scheduling preference over a job queue with a priority value of <code>1</code>.<br/>state:  Describes the queue's ability to accept new jobs.<br/>jobQueue:  The name or the Amazon Resource Name (ARN) of the job queue.<br/>
        :return: jobQueueArn:  The Amazon Resource Name (ARN) of the job queue.<br/>jobQueueName:  The name of the job queue.<br/>        """
        pass



class Budgets(BaseClient):
    """The AWS Budgets API enables you to use AWS Budgets to plan your service usage, service costs, and instance reservations. The API reference provides descriptions, syntax, and usage examples for each of the actions and data types for AWS Budgets.  Budgets provide you with a way to see the following information: <ul> <li> How close your plan is to your budgeted amount or to the free tier limits </li> <li> Your usage-to-date, including how much you've used of your Reserved Instances (RIs) </li> <li> Your current estimated charges from AWS, and how much your predicted usage will accrue in charges by the end of the month </li> <li> How much of your budget has been used </li> </ul> AWS updates your budget status several times a day. Budgets track your unblended costs, subscriptions, refunds, and RIs. You can create the following types of budgets: <ul> <li>  <b>Cost budgets</b> - Plan how much you want to spend on a service. </li> <li>  <b>Usage budgets</b> - Plan how much you want to use one or more services. </li> <li>  <b>RI utilization budgets</b> - Define a utilization threshold, and receive alerts when your RI usage falls below that threshold. This lets you see if your RIs are unused or under-utilized. </li> <li>  <b>RI coverage budgets</b> - Define a coverage threshold, and receive alerts when the number of your instance hours that are covered by RIs fall below that threshold. This lets you see how much of your instance usage is covered by a reservation. </li> </ul> Service Endpoint The AWS Budgets API provides the following endpoint: <ul> <li> https://budgets.amazonaws.com </li> </ul> For information about costs that are associated with the AWS Budgets API, see <a href="https://aws.amazon.com/aws-cost-management/pricing/">AWS Cost Management Pricing</a>."""


    class DescribeBudgetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_budgets` for valid parameters"""
            pass
    class DescribeNotificationsForBudgetPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_notifications_for_budget` for valid parameters"""
            pass
    class DescribeSubscribersForNotificationPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_subscribers_for_notification` for valid parameters"""
            pass


    class CreateBudgetRequest(Mapping):
        pass
    
    class CreateBudgetResponse(Mapping):
        pass
    
    class Budget(Mapping):
        pass
    
    class NotificationWithSubscribers(Mapping):
        pass
    
    class CreateNotificationRequest(Mapping):
        pass
    
    class CreateNotificationResponse(Mapping):
        pass
    
    class Notification(Mapping):
        pass
    
    class Subscriber(Mapping):
        pass
    
    class CreateSubscriberRequest(Mapping):
        pass
    
    class CreateSubscriberResponse(Mapping):
        pass
    
    class DeleteBudgetRequest(Mapping):
        pass
    
    class DeleteBudgetResponse(Mapping):
        pass
    
    class DeleteNotificationRequest(Mapping):
        pass
    
    class DeleteNotificationResponse(Mapping):
        pass
    
    class DeleteSubscriberRequest(Mapping):
        pass
    
    class DeleteSubscriberResponse(Mapping):
        pass
    
    class DescribeBudgetRequest(Mapping):
        pass
    
    class DescribeBudgetResponse(Mapping):
        pass
    
    class DescribeBudgetPerformanceHistoryRequest(Mapping):
        pass
    
    class DescribeBudgetPerformanceHistoryResponse(Mapping):
        pass
    
    class TimePeriod(Mapping):
        pass
    
    class DescribeBudgetsRequest(Mapping):
        pass
    
    class DescribeBudgetsResponse(Mapping):
        pass
    
    class DescribeNotificationsForBudgetRequest(Mapping):
        pass
    
    class DescribeNotificationsForBudgetResponse(Mapping):
        pass
    
    class DescribeSubscribersForNotificationRequest(Mapping):
        pass
    
    class DescribeSubscribersForNotificationResponse(Mapping):
        pass
    
    class UpdateBudgetRequest(Mapping):
        pass
    
    class UpdateBudgetResponse(Mapping):
        pass
    
    class UpdateNotificationRequest(Mapping):
        pass
    
    class UpdateNotificationResponse(Mapping):
        pass
    
    class UpdateSubscriberRequest(Mapping):
        pass
    
    class UpdateSubscriberResponse(Mapping):
        pass
    
    def create_budget(self, Budget: Budget, AccountId: str, NotificationsWithSubscribers: List[NotificationWithSubscribers]=None) -> CreateBudgetResponse:
        """Creates a budget and, if included, notifications and subscribers.  <important> Only one of <code>BudgetLimit</code> or <code>PlannedBudgetLimits</code> can be present in the syntax at one time. Use the syntax that matches your case. The Request Syntax section shows the <code>BudgetLimit</code> syntax. For <code>PlannedBudgetLimits</code>, see the <a href="https://docs.aws.amazon.com/aws-cost-management/latest/APIReference/API_budgets_CreateBudget.html#API_CreateBudget_Examples">Examples</a> section.  </important>

        :param self:
        :param NotificationsWithSubscribers:  A notification that you want to associate with a budget. A budget can have up to five notifications, and each notification can have one SNS subscriber and up to 10 email subscribers. If you include notifications and subscribers in your <code>CreateBudget</code> call, AWS creates the notifications and subscribers for you.<br/>Budget:  The budget object that you want to create.:<br/>AccountId:  The <code>accountId</code> that is associated with the budget.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdatedTime</i> The last time that you updated this budget.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BudgetType</i> Whether this budget tracks costs, usage, RI utilization, or RI coverage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CalculatedSpend</i> The actual and forecasted cost or usage that the budget tracks.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CalculatedSpend</b>: The actual and forecasted cost or usage that the budget tracks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimePeriod</i> The period of time that is covered by a budget. The period has a start date and an end date. The start date must come before the end date. The end date must come before <code>06/15/87 00:00 UTC</code>.  If you create your budget and don't specify a start date, AWS defaults to the start of your chosen time period (DAILY, MONTHLY, QUARTERLY, or ANNUALLY). For example, if you created your budget on January 24, 2018, chose <code>DAILY</code>, and didn't set a start date, AWS set your start date to <code>01/24/18 00:00 UTC</code>. If you chose <code>MONTHLY</code>, AWS set your start date to <code>01/01/18 00:00 UTC</code>. If you didn't specify an end date, AWS set your end date to <code>06/15/87 00:00 UTC</code>. The defaults are the same for the AWS Billing and Cost Management console and the API.  You can change either date with the <code>UpdateBudget</code> operation. After the end date, AWS deletes the budget and all associated notifications and subscribers.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TimePeriod</b>: The period of time that is covered by a budget. The period has a start date and an end date. The start date must come before the end date. The end date must come before <code>06/15/87 00:00 UTC</code>.  If you create your budget and don't specify a start date, AWS defaults to the start of your chosen time period (DAILY, MONTHLY, QUARTERLY, or ANNUALLY). For example, if you created your budget on January 24, 2018, chose <code>DAILY</code>, and didn't set a start date, AWS set your start date to <code>01/24/18 00:00 UTC</code>. If you chose <code>MONTHLY</code>, AWS set your start date to <code>01/01/18 00:00 UTC</code>. If you didn't specify an end date, AWS set your end date to <code>06/15/87 00:00 UTC</code>. The defaults are the same for the AWS Billing and Cost Management console and the API.  You can change either date with the <code>UpdateBudget</code> operation. After the end date, AWS deletes the budget and all associated notifications and subscribers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeUnit</i> The length of time until a budget resets the actual and forecasted spend. <code>DAILY</code> is available only for <code>RI_UTILIZATION</code> and <code>RI_COVERAGE</code> budgets.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CostTypes</i> The types of costs that are included in this <code>COST</code> budget.  <code>USAGE</code>, <code>RI_UTILIZATION</code>, and <code>RI_COVERAGE</code> budgets do not have <code>CostTypes</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CostTypes</b>: The types of costs that are included in this <code>COST</code> budget.  <code>USAGE</code>, <code>RI_UTILIZATION</code>, and <code>RI_COVERAGE</code> budgets do not have <code>CostTypes</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CostFilters</i> The cost filters, such as service or tag, that are applied to a budget. AWS Budgets supports the following services as a filter for RI budgets: <ul> <li> Amazon Elastic Compute Cloud - Compute </li> <li> Amazon Redshift </li> <li> Amazon Relational Database Service </li> <li> Amazon ElastiCache </li> <li> Amazon Elasticsearch Service </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlannedBudgetLimits</i> A map containing multiple <code>BudgetLimit</code>, including current or future limits.  <code>PlannedBudgetLimits</code> is available for cost or usage budget and supports monthly and quarterly <code>TimeUnit</code>.  For monthly budgets, provide 12 months of <code>PlannedBudgetLimits</code> values. This must start from the current month and include the next 11 months. The <code>key</code> is the start of the month, <code>UTC</code> in epoch seconds.  For quarterly budgets, provide 4 quarters of <code>PlannedBudgetLimits</code> value entries in standard calendar quarter increments. This must start from the current quarter and include the next 3 quarters. The <code>key</code> is the start of the quarter, <code>UTC</code> in epoch seconds.  If the planned budget expires before 12 months for monthly or 4 quarters for quarterly, provide the <code>PlannedBudgetLimits</code> values only for the remaining periods. If the budget begins at a date in the future, provide <code>PlannedBudgetLimits</code> values from the start date of the budget.  After all of the <code>BudgetLimit</code> values in <code>PlannedBudgetLimits</code> are used, the budget continues to use the last limit as the <code>BudgetLimit</code>. At that point, the planned budget provides the same experience as a fixed budget.   <code>DescribeBudget</code> and <code>DescribeBudgets</code> response along with <code>PlannedBudgetLimits</code> will also contain <code>BudgetLimit</code> representing the current month or quarter limit present in <code>PlannedBudgetLimits</code>. This only applies to budgets created with <code>PlannedBudgetLimits</code>. Budgets created without <code>PlannedBudgetLimits</code> will only contain <code>BudgetLimit</code>, and no <code>PlannedBudgetLimits</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BudgetLimit</i> The total amount of cost, usage, RI utilization, or RI coverage that you want to track with your budget.  <code>BudgetLimit</code> is required for cost or usage budgets, but optional for RI utilization or coverage budgets. RI utilization or coverage budgets default to <code>100</code>, which is the only valid value for RI utilization or coverage budgets. You can't use <code>BudgetLimit</code> with <code>PlannedBudgetLimits</code> for <code>CreateBudget</code> and <code>UpdateBudget</code> actions. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BudgetLimit</b>: The total amount of cost, usage, RI utilization, or RI coverage that you want to track with your budget.  <code>BudgetLimit</code> is required for cost or usage budgets, but optional for RI utilization or coverage budgets. RI utilization or coverage budgets default to <code>100</code>, which is the only valid value for RI utilization or coverage budgets. You can't use <code>BudgetLimit</code> with <code>PlannedBudgetLimits</code> for <code>CreateBudget</code> and <code>UpdateBudget</code> actions. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BudgetName</i> The name of a budget. The name must be unique within an account. The <code>:</code> and <code>\</code> characters aren't allowed in <code>BudgetName</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Unit</i> The unit of measurement that is used for the budget forecast, actual spend, or budget threshold, such as dollars or GB.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Amount</i> The cost or usage amount that is associated with a budget forecast, actual spend, or budget threshold.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UseAmortized</i> Specifies whether a budget uses the amortized rate. The default value is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeDiscount</i> Specifies whether a budget includes discounts. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeSupport</i> Specifies whether a budget includes support subscription fees. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeOtherSubscription</i> Specifies whether a budget includes non-RI subscription costs. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeRecurring</i> Specifies whether a budget includes recurring fees such as monthly RI fees. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeUpfront</i> Specifies whether a budget includes upfront RI costs. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeCredit</i> Specifies whether a budget includes credits. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeRefund</i> Specifies whether a budget includes refunds. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UseBlended</i> Specifies whether a budget uses a blended rate. The default value is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeSubscription</i> Specifies whether a budget includes subscriptions. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeTax</i> Specifies whether a budget includes taxes. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>End</i> The end date for a budget. If you didn't specify an end date, AWS set your end date to <code>06/15/87 00:00 UTC</code>. The defaults are the same for the AWS Billing and Cost Management console and the API. After the end date, AWS deletes the budget and all associated notifications and subscribers. You can change your end date with the <code>UpdateBudget</code> operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Start</i> The start date for a budget. If you created your budget and didn't specify a start date, AWS defaults to the start of your chosen time period (DAILY, MONTHLY, QUARTERLY, or ANNUALLY). For example, if you created your budget on January 24, 2018, chose <code>DAILY</code>, and didn't set a start date, AWS set your start date to <code>01/24/18 00:00 UTC</code>. If you chose <code>MONTHLY</code>, AWS set your start date to <code>01/01/18 00:00 UTC</code>. The defaults are the same for the AWS Billing and Cost Management console and the API. You can change your start date with the <code>UpdateBudget</code> operation.<br/>The amount of cost, usage, or RI units that you have used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ActualSpend</b>: The amount of cost, usage, or RI units that you have used.
        :return:         """
        pass

    def create_notification(self, Subscribers: List[Subscriber], Notification: Notification, BudgetName: str, AccountId: str) -> CreateNotificationResponse:
        """Creates a notification. You must create the budget before you create the associated notification.

        :param self:
        :param Subscribers:  A list of subscribers that you want to associate with the notification. Each notification can have one SNS subscriber and up to 10 email subscribers.<br/>Notification:  The notification that you want to create.:<br/>BudgetName:  The name of the budget that you want AWS to notify you about. Budget names must be unique within an account.<br/>AccountId:  The <code>accountId</code> that is associated with the budget that you want to create a notification for.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationState</i> Whether this notification is in alarm. If a budget notification is in the <code>ALARM</code> state, you have passed the set threshold for the budget.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThresholdType</i> The type of threshold for a notification. For <code>ABSOLUTE_VALUE</code> thresholds, AWS notifies you when you go over or are forecasted to go over your total cost threshold. For <code>PERCENTAGE</code> thresholds, AWS notifies you when you go over or are forecasted to go over a certain percentage of your forecasted spend. For example, if you have a budget for 200 dollars and you have a <code>PERCENTAGE</code> threshold of 80%, AWS notifies you when you go over 160 dollars.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Threshold</i> The threshold that is associated with a notification. Thresholds are always a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ComparisonOperator</i> The comparison that is used for this notification.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationType</i> Whether the notification is for how much you have spent (<code>ACTUAL</code>) or for how much you're forecasted to spend (<code>FORECASTED</code>).<br/>
        :return:         """
        pass

    def create_subscriber(self, Subscriber: Subscriber, Notification: Notification, BudgetName: str, AccountId: str) -> CreateSubscriberResponse:
        """Creates a subscriber. You must create the associated budget and notification before you create the subscriber.

        :param self:
        :param Subscriber:  The subscriber that you want to associate with a budget notification.:<br/>Notification:  The notification that you want to create a subscriber for.:<br/>BudgetName:  The name of the budget that you want to subscribe to. Budget names must be unique within an account.<br/>AccountId:  The <code>accountId</code> that is associated with the budget that you want to create a subscriber for.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationState</i> Whether this notification is in alarm. If a budget notification is in the <code>ALARM</code> state, you have passed the set threshold for the budget.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThresholdType</i> The type of threshold for a notification. For <code>ABSOLUTE_VALUE</code> thresholds, AWS notifies you when you go over or are forecasted to go over your total cost threshold. For <code>PERCENTAGE</code> thresholds, AWS notifies you when you go over or are forecasted to go over a certain percentage of your forecasted spend. For example, if you have a budget for 200 dollars and you have a <code>PERCENTAGE</code> threshold of 80%, AWS notifies you when you go over 160 dollars.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Threshold</i> The threshold that is associated with a notification. Thresholds are always a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ComparisonOperator</i> The comparison that is used for this notification.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationType</i> Whether the notification is for how much you have spent (<code>ACTUAL</code>) or for how much you're forecasted to spend (<code>FORECASTED</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The address that AWS sends budget notifications to, either an SNS topic or an email. AWS validates the address for a <code>CreateSubscriber</code> request with the <code>.*</code> regex.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubscriptionType</i> The type of notification that AWS sends to a subscriber.<br/>
        :return:         """
        pass

    def delete_budget(self, BudgetName: str, AccountId: str) -> DeleteBudgetResponse:
        """Deletes a budget. You can delete your budget at any time. <important> Deleting a budget also deletes the notifications and subscribers that are associated with that budget. </important>

        :param self:
        :param BudgetName:  The name of the budget that you want to delete.<br/>AccountId:  The <code>accountId</code> that is associated with the budget that you want to delete.<br/>
        :return:         """
        pass

    def delete_notification(self, Notification: Notification, BudgetName: str, AccountId: str) -> DeleteNotificationResponse:
        """Deletes a notification. <important> Deleting a notification also deletes the subscribers that are associated with the notification. </important>

        :param self:
        :param Notification:  The notification that you want to delete.:<br/>BudgetName:  The name of the budget whose notification you want to delete.<br/>AccountId:  The <code>accountId</code> that is associated with the budget whose notification you want to delete.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationState</i> Whether this notification is in alarm. If a budget notification is in the <code>ALARM</code> state, you have passed the set threshold for the budget.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThresholdType</i> The type of threshold for a notification. For <code>ABSOLUTE_VALUE</code> thresholds, AWS notifies you when you go over or are forecasted to go over your total cost threshold. For <code>PERCENTAGE</code> thresholds, AWS notifies you when you go over or are forecasted to go over a certain percentage of your forecasted spend. For example, if you have a budget for 200 dollars and you have a <code>PERCENTAGE</code> threshold of 80%, AWS notifies you when you go over 160 dollars.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Threshold</i> The threshold that is associated with a notification. Thresholds are always a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ComparisonOperator</i> The comparison that is used for this notification.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationType</i> Whether the notification is for how much you have spent (<code>ACTUAL</code>) or for how much you're forecasted to spend (<code>FORECASTED</code>).<br/>
        :return:         """
        pass

    def delete_subscriber(self, Subscriber: Subscriber, Notification: Notification, BudgetName: str, AccountId: str) -> DeleteSubscriberResponse:
        """Deletes a subscriber. <important> Deleting the last subscriber to a notification also deletes the notification. </important>

        :param self:
        :param Subscriber:  The subscriber that you want to delete.:<br/>Notification:  The notification whose subscriber you want to delete.:<br/>BudgetName:  The name of the budget whose subscriber you want to delete.<br/>AccountId:  The <code>accountId</code> that is associated with the budget whose subscriber you want to delete.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationState</i> Whether this notification is in alarm. If a budget notification is in the <code>ALARM</code> state, you have passed the set threshold for the budget.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThresholdType</i> The type of threshold for a notification. For <code>ABSOLUTE_VALUE</code> thresholds, AWS notifies you when you go over or are forecasted to go over your total cost threshold. For <code>PERCENTAGE</code> thresholds, AWS notifies you when you go over or are forecasted to go over a certain percentage of your forecasted spend. For example, if you have a budget for 200 dollars and you have a <code>PERCENTAGE</code> threshold of 80%, AWS notifies you when you go over 160 dollars.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Threshold</i> The threshold that is associated with a notification. Thresholds are always a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ComparisonOperator</i> The comparison that is used for this notification.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationType</i> Whether the notification is for how much you have spent (<code>ACTUAL</code>) or for how much you're forecasted to spend (<code>FORECASTED</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The address that AWS sends budget notifications to, either an SNS topic or an email. AWS validates the address for a <code>CreateSubscriber</code> request with the <code>.*</code> regex.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubscriptionType</i> The type of notification that AWS sends to a subscriber.<br/>
        :return:         """
        pass

    def describe_budget(self, BudgetName: str, AccountId: str) -> DescribeBudgetResponse:
        """Describes a budget. <important> The Request Syntax section shows the <code>BudgetLimit</code> syntax. For <code>PlannedBudgetLimits</code>, see the <a href="https://docs.aws.amazon.com/aws-cost-management/latest/APIReference/API_budgets_DescribeBudget.html#API_DescribeBudget_Examples">Examples</a> section.  </important>

        :param self:
        :param BudgetName:  The name of the budget that you want a description of.<br/>AccountId:  The <code>accountId</code> that is associated with the budget that you want a description of.<br/>
        :return: Budget:  The description of the budget.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdatedTime</i> The last time that you updated this budget.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BudgetType</i> Whether this budget tracks costs, usage, RI utilization, or RI coverage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CalculatedSpend</i> The actual and forecasted cost or usage that the budget tracks.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CalculatedSpend</b>: The actual and forecasted cost or usage that the budget tracks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimePeriod</i> The period of time that is covered by a budget. The period has a start date and an end date. The start date must come before the end date. The end date must come before <code>06/15/87 00:00 UTC</code>.  If you create your budget and don't specify a start date, AWS defaults to the start of your chosen time period (DAILY, MONTHLY, QUARTERLY, or ANNUALLY). For example, if you created your budget on January 24, 2018, chose <code>DAILY</code>, and didn't set a start date, AWS set your start date to <code>01/24/18 00:00 UTC</code>. If you chose <code>MONTHLY</code>, AWS set your start date to <code>01/01/18 00:00 UTC</code>. If you didn't specify an end date, AWS set your end date to <code>06/15/87 00:00 UTC</code>. The defaults are the same for the AWS Billing and Cost Management console and the API.  You can change either date with the <code>UpdateBudget</code> operation. After the end date, AWS deletes the budget and all associated notifications and subscribers.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TimePeriod</b>: The period of time that is covered by a budget. The period has a start date and an end date. The start date must come before the end date. The end date must come before <code>06/15/87 00:00 UTC</code>.  If you create your budget and don't specify a start date, AWS defaults to the start of your chosen time period (DAILY, MONTHLY, QUARTERLY, or ANNUALLY). For example, if you created your budget on January 24, 2018, chose <code>DAILY</code>, and didn't set a start date, AWS set your start date to <code>01/24/18 00:00 UTC</code>. If you chose <code>MONTHLY</code>, AWS set your start date to <code>01/01/18 00:00 UTC</code>. If you didn't specify an end date, AWS set your end date to <code>06/15/87 00:00 UTC</code>. The defaults are the same for the AWS Billing and Cost Management console and the API.  You can change either date with the <code>UpdateBudget</code> operation. After the end date, AWS deletes the budget and all associated notifications and subscribers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeUnit</i> The length of time until a budget resets the actual and forecasted spend. <code>DAILY</code> is available only for <code>RI_UTILIZATION</code> and <code>RI_COVERAGE</code> budgets.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CostTypes</i> The types of costs that are included in this <code>COST</code> budget.  <code>USAGE</code>, <code>RI_UTILIZATION</code>, and <code>RI_COVERAGE</code> budgets do not have <code>CostTypes</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CostTypes</b>: The types of costs that are included in this <code>COST</code> budget.  <code>USAGE</code>, <code>RI_UTILIZATION</code>, and <code>RI_COVERAGE</code> budgets do not have <code>CostTypes</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CostFilters</i> The cost filters, such as service or tag, that are applied to a budget. AWS Budgets supports the following services as a filter for RI budgets: <ul> <li> Amazon Elastic Compute Cloud - Compute </li> <li> Amazon Redshift </li> <li> Amazon Relational Database Service </li> <li> Amazon ElastiCache </li> <li> Amazon Elasticsearch Service </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlannedBudgetLimits</i> A map containing multiple <code>BudgetLimit</code>, including current or future limits.  <code>PlannedBudgetLimits</code> is available for cost or usage budget and supports monthly and quarterly <code>TimeUnit</code>.  For monthly budgets, provide 12 months of <code>PlannedBudgetLimits</code> values. This must start from the current month and include the next 11 months. The <code>key</code> is the start of the month, <code>UTC</code> in epoch seconds.  For quarterly budgets, provide 4 quarters of <code>PlannedBudgetLimits</code> value entries in standard calendar quarter increments. This must start from the current quarter and include the next 3 quarters. The <code>key</code> is the start of the quarter, <code>UTC</code> in epoch seconds.  If the planned budget expires before 12 months for monthly or 4 quarters for quarterly, provide the <code>PlannedBudgetLimits</code> values only for the remaining periods. If the budget begins at a date in the future, provide <code>PlannedBudgetLimits</code> values from the start date of the budget.  After all of the <code>BudgetLimit</code> values in <code>PlannedBudgetLimits</code> are used, the budget continues to use the last limit as the <code>BudgetLimit</code>. At that point, the planned budget provides the same experience as a fixed budget.   <code>DescribeBudget</code> and <code>DescribeBudgets</code> response along with <code>PlannedBudgetLimits</code> will also contain <code>BudgetLimit</code> representing the current month or quarter limit present in <code>PlannedBudgetLimits</code>. This only applies to budgets created with <code>PlannedBudgetLimits</code>. Budgets created without <code>PlannedBudgetLimits</code> will only contain <code>BudgetLimit</code>, and no <code>PlannedBudgetLimits</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BudgetLimit</i> The total amount of cost, usage, RI utilization, or RI coverage that you want to track with your budget.  <code>BudgetLimit</code> is required for cost or usage budgets, but optional for RI utilization or coverage budgets. RI utilization or coverage budgets default to <code>100</code>, which is the only valid value for RI utilization or coverage budgets. You can't use <code>BudgetLimit</code> with <code>PlannedBudgetLimits</code> for <code>CreateBudget</code> and <code>UpdateBudget</code> actions. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BudgetLimit</b>: The total amount of cost, usage, RI utilization, or RI coverage that you want to track with your budget.  <code>BudgetLimit</code> is required for cost or usage budgets, but optional for RI utilization or coverage budgets. RI utilization or coverage budgets default to <code>100</code>, which is the only valid value for RI utilization or coverage budgets. You can't use <code>BudgetLimit</code> with <code>PlannedBudgetLimits</code> for <code>CreateBudget</code> and <code>UpdateBudget</code> actions. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BudgetName</i> The name of a budget. The name must be unique within an account. The <code>:</code> and <code>\</code> characters aren't allowed in <code>BudgetName</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Unit</i> The unit of measurement that is used for the budget forecast, actual spend, or budget threshold, such as dollars or GB.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Amount</i> The cost or usage amount that is associated with a budget forecast, actual spend, or budget threshold.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UseAmortized</i> Specifies whether a budget uses the amortized rate. The default value is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeDiscount</i> Specifies whether a budget includes discounts. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeSupport</i> Specifies whether a budget includes support subscription fees. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeOtherSubscription</i> Specifies whether a budget includes non-RI subscription costs. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeRecurring</i> Specifies whether a budget includes recurring fees such as monthly RI fees. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeUpfront</i> Specifies whether a budget includes upfront RI costs. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeCredit</i> Specifies whether a budget includes credits. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeRefund</i> Specifies whether a budget includes refunds. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UseBlended</i> Specifies whether a budget uses a blended rate. The default value is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeSubscription</i> Specifies whether a budget includes subscriptions. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeTax</i> Specifies whether a budget includes taxes. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>End</i> The end date for a budget. If you didn't specify an end date, AWS set your end date to <code>06/15/87 00:00 UTC</code>. The defaults are the same for the AWS Billing and Cost Management console and the API. After the end date, AWS deletes the budget and all associated notifications and subscribers. You can change your end date with the <code>UpdateBudget</code> operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Start</i> The start date for a budget. If you created your budget and didn't specify a start date, AWS defaults to the start of your chosen time period (DAILY, MONTHLY, QUARTERLY, or ANNUALLY). For example, if you created your budget on January 24, 2018, chose <code>DAILY</code>, and didn't set a start date, AWS set your start date to <code>01/24/18 00:00 UTC</code>. If you chose <code>MONTHLY</code>, AWS set your start date to <code>01/01/18 00:00 UTC</code>. The defaults are the same for the AWS Billing and Cost Management console and the API. You can change your start date with the <code>UpdateBudget</code> operation.<br/>The amount of cost, usage, or RI units that you have used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ActualSpend</b>: The amount of cost, usage, or RI units that you have used.        """
        pass

    def describe_budget_performance_history(self, BudgetName: str, AccountId: str, TimePeriod: TimePeriod=None, MaxResults: int=None, NextToken: str=None) -> DescribeBudgetPerformanceHistoryResponse:
        """Describes the history for <code>DAILY</code>, <code>MONTHLY</code>, and <code>QUARTERLY</code> budgets. Budget history isn't available for <code>ANNUAL</code> budgets.

        :param self:
        :param NextToken:   A generic string.<br/>MaxResults:   An integer that represents how many entries a paginated response contains. The maximum is 100.<br/>TimePeriod:  Retrieves how often the budget went into an <code>ALARM</code> state for the specified time period.:<br/>BudgetName:   A string that represents the budget name. The ":" and "\" characters aren't allowed.<br/>AccountId:  The account ID of the user. It should be a 12-digit number.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>End</i> The end date for a budget. If you didn't specify an end date, AWS set your end date to <code>06/15/87 00:00 UTC</code>. The defaults are the same for the AWS Billing and Cost Management console and the API. After the end date, AWS deletes the budget and all associated notifications and subscribers. You can change your end date with the <code>UpdateBudget</code> operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Start</i> The start date for a budget. If you created your budget and didn't specify a start date, AWS defaults to the start of your chosen time period (DAILY, MONTHLY, QUARTERLY, or ANNUALLY). For example, if you created your budget on January 24, 2018, chose <code>DAILY</code>, and didn't set a start date, AWS set your start date to <code>01/24/18 00:00 UTC</code>. If you chose <code>MONTHLY</code>, AWS set your start date to <code>01/01/18 00:00 UTC</code>. The defaults are the same for the AWS Billing and Cost Management console and the API. You can change your start date with the <code>UpdateBudget</code> operation.<br/>
        :return: NextToken:   A generic string.<br/>BudgetPerformanceHistory:  The history of how often the budget has gone into an <code>ALARM</code> state. For <code>DAILY</code> budgets, the history saves the state of the budget for the last 60 days. For <code>MONTHLY</code> budgets, the history saves the state of the budget for the current month plus the last 12 months. For <code>QUARTERLY</code> budgets, the history saves the state of the budget for the last four quarters.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BudgetedAndActualAmountsList</i> A list of amounts of cost or usage that you created budgets for, compared to your actual costs or usage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeUnit</i>  The time unit of the budget, such as MONTHLY or QUARTERLY.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CostTypes</i> The history of the cost types for a budget during the specified time period.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CostTypes</b>: The history of the cost types for a budget during the specified time period.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CostFilters</i> The history of the cost filters for a budget during the specified time period.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BudgetType</i>  The type of a budget. It must be one of the following types:   <code>COST</code>, <code>USAGE</code>, <code>RI_UTILIZATION</code>, or <code>RI_COVERAGE</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BudgetName</i>  A string that represents the budget name. The ":" and "\" characters aren't allowed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UseAmortized</i> Specifies whether a budget uses the amortized rate. The default value is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeDiscount</i> Specifies whether a budget includes discounts. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeSupport</i> Specifies whether a budget includes support subscription fees. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeOtherSubscription</i> Specifies whether a budget includes non-RI subscription costs. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeRecurring</i> Specifies whether a budget includes recurring fees such as monthly RI fees. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeUpfront</i> Specifies whether a budget includes upfront RI costs. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeCredit</i> Specifies whether a budget includes credits. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeRefund</i> Specifies whether a budget includes refunds. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UseBlended</i> Specifies whether a budget uses a blended rate. The default value is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeSubscription</i> Specifies whether a budget includes subscriptions. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeTax</i> Specifies whether a budget includes taxes. The default value is <code>true</code>.<br/>        """
        pass

    def describe_budgets(self, AccountId: str, MaxResults: int=None, NextToken: str=None) -> DescribeBudgetsResponse:
        """Lists the budgets that are associated with an account. <important> The Request Syntax section shows the <code>BudgetLimit</code> syntax. For <code>PlannedBudgetLimits</code>, see the <a href="https://docs.aws.amazon.com/aws-cost-management/latest/APIReference/API_budgets_DescribeBudgets.html#API_DescribeBudgets_Examples">Examples</a> section.  </important>

        :param self:
        :param NextToken:  The pagination token that you include in your request to indicate the next set of results that you want to retrieve.<br/>MaxResults:  An optional integer that represents how many entries a paginated response contains. The maximum is 100.<br/>AccountId:  The <code>accountId</code> that is associated with the budgets that you want descriptions of.<br/>
        :return: NextToken:  The pagination token in the service response that indicates the next set of results that you can retrieve.<br/>Budgets:  A list of budgets.<br/>        """
        pass

    def describe_notifications_for_budget(self, BudgetName: str, AccountId: str, MaxResults: int=None, NextToken: str=None) -> DescribeNotificationsForBudgetResponse:
        """Lists the notifications that are associated with a budget.

        :param self:
        :param NextToken:  The pagination token that you include in your request to indicate the next set of results that you want to retrieve.<br/>MaxResults:  An optional integer that represents how many entries a paginated response contains. The maximum is 100.<br/>BudgetName:  The name of the budget whose notifications you want descriptions of.<br/>AccountId:  The <code>accountId</code> that is associated with the budget whose notifications you want descriptions of.<br/>
        :return: NextToken:  The pagination token in the service response that indicates the next set of results that you can retrieve.<br/>Notifications:  A list of notifications that are associated with a budget.<br/>        """
        pass

    def describe_subscribers_for_notification(self, Notification: Notification, BudgetName: str, AccountId: str, MaxResults: int=None, NextToken: str=None) -> DescribeSubscribersForNotificationResponse:
        """Lists the subscribers that are associated with a notification.

        :param self:
        :param NextToken:  The pagination token that you include in your request to indicate the next set of results that you want to retrieve.<br/>MaxResults:  An optional integer that represents how many entries a paginated response contains. The maximum is 100.<br/>Notification:  The notification whose subscribers you want to list.:<br/>BudgetName:  The name of the budget whose subscribers you want descriptions of.<br/>AccountId:  The <code>accountId</code> that is associated with the budget whose subscribers you want descriptions of.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationState</i> Whether this notification is in alarm. If a budget notification is in the <code>ALARM</code> state, you have passed the set threshold for the budget.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThresholdType</i> The type of threshold for a notification. For <code>ABSOLUTE_VALUE</code> thresholds, AWS notifies you when you go over or are forecasted to go over your total cost threshold. For <code>PERCENTAGE</code> thresholds, AWS notifies you when you go over or are forecasted to go over a certain percentage of your forecasted spend. For example, if you have a budget for 200 dollars and you have a <code>PERCENTAGE</code> threshold of 80%, AWS notifies you when you go over 160 dollars.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Threshold</i> The threshold that is associated with a notification. Thresholds are always a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ComparisonOperator</i> The comparison that is used for this notification.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationType</i> Whether the notification is for how much you have spent (<code>ACTUAL</code>) or for how much you're forecasted to spend (<code>FORECASTED</code>).<br/>
        :return: NextToken:  The pagination token in the service response that indicates the next set of results that you can retrieve.<br/>Subscribers:  A list of subscribers that are associated with a notification.<br/>        """
        pass

    def update_budget(self, NewBudget: Budget, AccountId: str) -> UpdateBudgetResponse:
        """Updates a budget. You can change every part of a budget except for the <code>budgetName</code> and the <code>calculatedSpend</code>. When you modify a budget, the <code>calculatedSpend</code> drops to zero until AWS has new usage data to use for forecasting. <important> Only one of <code>BudgetLimit</code> or <code>PlannedBudgetLimits</code> can be present in the syntax at one time. Use the syntax that matches your case. The Request Syntax section shows the <code>BudgetLimit</code> syntax. For <code>PlannedBudgetLimits</code>, see the <a href="https://docs.aws.amazon.com/aws-cost-management/latest/APIReference/API_budgets_UpdateBudget.html#API_UpdateBudget_Examples">Examples</a> section.  </important>

        :param self:
        :param NewBudget:  The budget that you want to update your budget to.:<br/>AccountId:  The <code>accountId</code> that is associated with the budget that you want to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdatedTime</i> The last time that you updated this budget.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BudgetType</i> Whether this budget tracks costs, usage, RI utilization, or RI coverage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CalculatedSpend</i> The actual and forecasted cost or usage that the budget tracks.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CalculatedSpend</b>: The actual and forecasted cost or usage that the budget tracks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimePeriod</i> The period of time that is covered by a budget. The period has a start date and an end date. The start date must come before the end date. The end date must come before <code>06/15/87 00:00 UTC</code>.  If you create your budget and don't specify a start date, AWS defaults to the start of your chosen time period (DAILY, MONTHLY, QUARTERLY, or ANNUALLY). For example, if you created your budget on January 24, 2018, chose <code>DAILY</code>, and didn't set a start date, AWS set your start date to <code>01/24/18 00:00 UTC</code>. If you chose <code>MONTHLY</code>, AWS set your start date to <code>01/01/18 00:00 UTC</code>. If you didn't specify an end date, AWS set your end date to <code>06/15/87 00:00 UTC</code>. The defaults are the same for the AWS Billing and Cost Management console and the API.  You can change either date with the <code>UpdateBudget</code> operation. After the end date, AWS deletes the budget and all associated notifications and subscribers.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TimePeriod</b>: The period of time that is covered by a budget. The period has a start date and an end date. The start date must come before the end date. The end date must come before <code>06/15/87 00:00 UTC</code>.  If you create your budget and don't specify a start date, AWS defaults to the start of your chosen time period (DAILY, MONTHLY, QUARTERLY, or ANNUALLY). For example, if you created your budget on January 24, 2018, chose <code>DAILY</code>, and didn't set a start date, AWS set your start date to <code>01/24/18 00:00 UTC</code>. If you chose <code>MONTHLY</code>, AWS set your start date to <code>01/01/18 00:00 UTC</code>. If you didn't specify an end date, AWS set your end date to <code>06/15/87 00:00 UTC</code>. The defaults are the same for the AWS Billing and Cost Management console and the API.  You can change either date with the <code>UpdateBudget</code> operation. After the end date, AWS deletes the budget and all associated notifications and subscribers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeUnit</i> The length of time until a budget resets the actual and forecasted spend. <code>DAILY</code> is available only for <code>RI_UTILIZATION</code> and <code>RI_COVERAGE</code> budgets.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CostTypes</i> The types of costs that are included in this <code>COST</code> budget.  <code>USAGE</code>, <code>RI_UTILIZATION</code>, and <code>RI_COVERAGE</code> budgets do not have <code>CostTypes</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CostTypes</b>: The types of costs that are included in this <code>COST</code> budget.  <code>USAGE</code>, <code>RI_UTILIZATION</code>, and <code>RI_COVERAGE</code> budgets do not have <code>CostTypes</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CostFilters</i> The cost filters, such as service or tag, that are applied to a budget. AWS Budgets supports the following services as a filter for RI budgets: <ul> <li> Amazon Elastic Compute Cloud - Compute </li> <li> Amazon Redshift </li> <li> Amazon Relational Database Service </li> <li> Amazon ElastiCache </li> <li> Amazon Elasticsearch Service </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlannedBudgetLimits</i> A map containing multiple <code>BudgetLimit</code>, including current or future limits.  <code>PlannedBudgetLimits</code> is available for cost or usage budget and supports monthly and quarterly <code>TimeUnit</code>.  For monthly budgets, provide 12 months of <code>PlannedBudgetLimits</code> values. This must start from the current month and include the next 11 months. The <code>key</code> is the start of the month, <code>UTC</code> in epoch seconds.  For quarterly budgets, provide 4 quarters of <code>PlannedBudgetLimits</code> value entries in standard calendar quarter increments. This must start from the current quarter and include the next 3 quarters. The <code>key</code> is the start of the quarter, <code>UTC</code> in epoch seconds.  If the planned budget expires before 12 months for monthly or 4 quarters for quarterly, provide the <code>PlannedBudgetLimits</code> values only for the remaining periods. If the budget begins at a date in the future, provide <code>PlannedBudgetLimits</code> values from the start date of the budget.  After all of the <code>BudgetLimit</code> values in <code>PlannedBudgetLimits</code> are used, the budget continues to use the last limit as the <code>BudgetLimit</code>. At that point, the planned budget provides the same experience as a fixed budget.   <code>DescribeBudget</code> and <code>DescribeBudgets</code> response along with <code>PlannedBudgetLimits</code> will also contain <code>BudgetLimit</code> representing the current month or quarter limit present in <code>PlannedBudgetLimits</code>. This only applies to budgets created with <code>PlannedBudgetLimits</code>. Budgets created without <code>PlannedBudgetLimits</code> will only contain <code>BudgetLimit</code>, and no <code>PlannedBudgetLimits</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BudgetLimit</i> The total amount of cost, usage, RI utilization, or RI coverage that you want to track with your budget.  <code>BudgetLimit</code> is required for cost or usage budgets, but optional for RI utilization or coverage budgets. RI utilization or coverage budgets default to <code>100</code>, which is the only valid value for RI utilization or coverage budgets. You can't use <code>BudgetLimit</code> with <code>PlannedBudgetLimits</code> for <code>CreateBudget</code> and <code>UpdateBudget</code> actions. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BudgetLimit</b>: The total amount of cost, usage, RI utilization, or RI coverage that you want to track with your budget.  <code>BudgetLimit</code> is required for cost or usage budgets, but optional for RI utilization or coverage budgets. RI utilization or coverage budgets default to <code>100</code>, which is the only valid value for RI utilization or coverage budgets. You can't use <code>BudgetLimit</code> with <code>PlannedBudgetLimits</code> for <code>CreateBudget</code> and <code>UpdateBudget</code> actions. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BudgetName</i> The name of a budget. The name must be unique within an account. The <code>:</code> and <code>\</code> characters aren't allowed in <code>BudgetName</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Unit</i> The unit of measurement that is used for the budget forecast, actual spend, or budget threshold, such as dollars or GB.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Amount</i> The cost or usage amount that is associated with a budget forecast, actual spend, or budget threshold.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UseAmortized</i> Specifies whether a budget uses the amortized rate. The default value is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeDiscount</i> Specifies whether a budget includes discounts. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeSupport</i> Specifies whether a budget includes support subscription fees. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeOtherSubscription</i> Specifies whether a budget includes non-RI subscription costs. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeRecurring</i> Specifies whether a budget includes recurring fees such as monthly RI fees. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeUpfront</i> Specifies whether a budget includes upfront RI costs. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeCredit</i> Specifies whether a budget includes credits. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeRefund</i> Specifies whether a budget includes refunds. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UseBlended</i> Specifies whether a budget uses a blended rate. The default value is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeSubscription</i> Specifies whether a budget includes subscriptions. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeTax</i> Specifies whether a budget includes taxes. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>End</i> The end date for a budget. If you didn't specify an end date, AWS set your end date to <code>06/15/87 00:00 UTC</code>. The defaults are the same for the AWS Billing and Cost Management console and the API. After the end date, AWS deletes the budget and all associated notifications and subscribers. You can change your end date with the <code>UpdateBudget</code> operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Start</i> The start date for a budget. If you created your budget and didn't specify a start date, AWS defaults to the start of your chosen time period (DAILY, MONTHLY, QUARTERLY, or ANNUALLY). For example, if you created your budget on January 24, 2018, chose <code>DAILY</code>, and didn't set a start date, AWS set your start date to <code>01/24/18 00:00 UTC</code>. If you chose <code>MONTHLY</code>, AWS set your start date to <code>01/01/18 00:00 UTC</code>. The defaults are the same for the AWS Billing and Cost Management console and the API. You can change your start date with the <code>UpdateBudget</code> operation.<br/>The amount of cost, usage, or RI units that you have used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ActualSpend</b>: The amount of cost, usage, or RI units that you have used.
        :return:         """
        pass

    def update_notification(self, NewNotification: Notification, OldNotification: Notification, BudgetName: str, AccountId: str) -> UpdateNotificationResponse:
        """Updates a notification.

        :param self:
        :param NewNotification:  The updated notification to be associated with a budget.:<br/>OldNotification:  The previous notification that is associated with a budget.:<br/>BudgetName:  The name of the budget whose notification you want to update.<br/>AccountId:  The <code>accountId</code> that is associated with the budget whose notification you want to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationState</i> Whether this notification is in alarm. If a budget notification is in the <code>ALARM</code> state, you have passed the set threshold for the budget.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThresholdType</i> The type of threshold for a notification. For <code>ABSOLUTE_VALUE</code> thresholds, AWS notifies you when you go over or are forecasted to go over your total cost threshold. For <code>PERCENTAGE</code> thresholds, AWS notifies you when you go over or are forecasted to go over a certain percentage of your forecasted spend. For example, if you have a budget for 200 dollars and you have a <code>PERCENTAGE</code> threshold of 80%, AWS notifies you when you go over 160 dollars.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Threshold</i> The threshold that is associated with a notification. Thresholds are always a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ComparisonOperator</i> The comparison that is used for this notification.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationType</i> Whether the notification is for how much you have spent (<code>ACTUAL</code>) or for how much you're forecasted to spend (<code>FORECASTED</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationState</i> Whether this notification is in alarm. If a budget notification is in the <code>ALARM</code> state, you have passed the set threshold for the budget.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThresholdType</i> The type of threshold for a notification. For <code>ABSOLUTE_VALUE</code> thresholds, AWS notifies you when you go over or are forecasted to go over your total cost threshold. For <code>PERCENTAGE</code> thresholds, AWS notifies you when you go over or are forecasted to go over a certain percentage of your forecasted spend. For example, if you have a budget for 200 dollars and you have a <code>PERCENTAGE</code> threshold of 80%, AWS notifies you when you go over 160 dollars.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Threshold</i> The threshold that is associated with a notification. Thresholds are always a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ComparisonOperator</i> The comparison that is used for this notification.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationType</i> Whether the notification is for how much you have spent (<code>ACTUAL</code>) or for how much you're forecasted to spend (<code>FORECASTED</code>).<br/>
        :return:         """
        pass

    def update_subscriber(self, NewSubscriber: Subscriber, OldSubscriber: Subscriber, Notification: Notification, BudgetName: str, AccountId: str) -> UpdateSubscriberResponse:
        """Updates a subscriber.

        :param self:
        :param NewSubscriber:  The updated subscriber that is associated with a budget notification.:<br/>OldSubscriber:  The previous subscriber that is associated with a budget notification.:<br/>Notification:  The notification whose subscriber you want to update.:<br/>BudgetName:  The name of the budget whose subscriber you want to update.<br/>AccountId:  The <code>accountId</code> that is associated with the budget whose subscriber you want to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationState</i> Whether this notification is in alarm. If a budget notification is in the <code>ALARM</code> state, you have passed the set threshold for the budget.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThresholdType</i> The type of threshold for a notification. For <code>ABSOLUTE_VALUE</code> thresholds, AWS notifies you when you go over or are forecasted to go over your total cost threshold. For <code>PERCENTAGE</code> thresholds, AWS notifies you when you go over or are forecasted to go over a certain percentage of your forecasted spend. For example, if you have a budget for 200 dollars and you have a <code>PERCENTAGE</code> threshold of 80%, AWS notifies you when you go over 160 dollars.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Threshold</i> The threshold that is associated with a notification. Thresholds are always a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ComparisonOperator</i> The comparison that is used for this notification.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationType</i> Whether the notification is for how much you have spent (<code>ACTUAL</code>) or for how much you're forecasted to spend (<code>FORECASTED</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The address that AWS sends budget notifications to, either an SNS topic or an email. AWS validates the address for a <code>CreateSubscriber</code> request with the <code>.*</code> regex.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubscriptionType</i> The type of notification that AWS sends to a subscriber.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The address that AWS sends budget notifications to, either an SNS topic or an email. AWS validates the address for a <code>CreateSubscriber</code> request with the <code>.*</code> regex.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubscriptionType</i> The type of notification that AWS sends to a subscriber.<br/>
        :return:         """
        pass



class CostExplorer(BaseClient):
    """The Cost Explorer API enables you to programmatically query your cost and usage data. You can query for aggregated data such as total monthly costs or total daily usage. You can also query for granular data, such as the number of daily write operations for Amazon DynamoDB database tables in your production environment.  Service Endpoint The Cost Explorer API provides the following endpoint: <ul> <li>  <code>https://ce.us-east-1.amazonaws.com</code>  </li> </ul> For information about costs associated with the Cost Explorer API, see <a href="https://aws.amazon.com/aws-cost-management/pricing/">AWS Cost Management Pricing</a>."""




    class GetCostAndUsageRequest(Mapping):
        pass
    
    class GetCostAndUsageResponse(Mapping):
        pass
    
    class DateInterval(Mapping):
        pass
    
    class Expression(Mapping):
        pass
    
    class MetricName(object):
        pass
    
    class GroupDefinition(Mapping):
        pass
    
    class GetCostForecastRequest(Mapping):
        pass
    
    class GetCostForecastResponse(Mapping):
        pass
    
    class GetDimensionValuesRequest(Mapping):
        pass
    
    class GetDimensionValuesResponse(Mapping):
        pass
    
    class GetReservationCoverageRequest(Mapping):
        pass
    
    class GetReservationCoverageResponse(Mapping):
        pass
    
    class GetReservationPurchaseRecommendationRequest(Mapping):
        pass
    
    class GetReservationPurchaseRecommendationResponse(Mapping):
        pass
    
    class ServiceSpecification(Mapping):
        pass
    
    class GetReservationUtilizationRequest(Mapping):
        pass
    
    class GetReservationUtilizationResponse(Mapping):
        pass
    
    class GetTagsRequest(Mapping):
        pass
    
    class GetTagsResponse(Mapping):
        pass
    
    class GetUsageForecastRequest(Mapping):
        pass
    
    class GetUsageForecastResponse(Mapping):
        pass
    
    def get_cost_and_usage(self, TimePeriod: DateInterval, Granularity: str=None, Filter: Expression=None, Metrics: List[MetricName]=None, GroupBy: List[GroupDefinition]=None, NextPageToken: str=None) -> GetCostAndUsageResponse:
        """Retrieves cost and usage metrics for your account. You can specify which cost and usage-related metric, such as <code>BlendedCosts</code> or <code>UsageQuantity</code>, that you want the request to return. You can also filter and group your data by various dimensions, such as <code>SERVICE</code> or <code>AZ</code>, in a specific time range. For a complete list of valid dimensions, see the <a href="http://docs.aws.amazon.com/aws-cost-management/latest/APIReference/API_GetDimensionValues.html">GetDimensionValues</a> operation. Master accounts in an organization in AWS Organizations have access to all member accounts.

        :param self:
        :param NextPageToken:  The token to retrieve the next set of results. AWS provides the token when the response from a previous call has more results than the maximum page size.<br/>GroupBy:  You can group AWS costs using up to two different groups, either dimensions, tag keys, or both. When you group by tag key, you get all tag values, including empty strings. Valid values are <code>AZ</code>, <code>INSTANCE_TYPE</code>, <code>LEGAL_ENTITY_NAME</code>, <code>LINKED_ACCOUNT</code>, <code>OPERATION</code>, <code>PLATFORM</code>, <code>PURCHASE_TYPE</code>, <code>SERVICE</code>, <code>TAGS</code>, <code>TENANCY</code>, and <code>USAGE_TYPE</code>.<br/>Metrics:  Which metrics are returned in the query. For more information about blended and unblended rates, see <a href="https://aws.amazon.com/premiumsupport/knowledge-center/blended-rates-intro/">Why does the "blended" annotation appear on some line items in my bill?</a>.  Valid values are <code>AmortizedCost</code>, <code>BlendedCost</code>, <code>NetAmortizedCost</code>, <code>NetUnblendedCost</code>, <code>NormalizedUsageAmount</code>, <code>UnblendedCost</code>, and <code>UsageQuantity</code>.  <note> If you return the <code>UsageQuantity</code> metric, the service aggregates all usage numbers without taking into account the units. For example, if you aggregate <code>usageQuantity</code> across all of Amazon EC2, the results aren't meaningful because Amazon EC2 compute hours and data transfer are measured in different units (for example, hours vs. GB). To get more meaningful <code>UsageQuantity</code> metrics, filter by <code>UsageType</code> or <code>UsageTypeGroups</code>.  </note>  <code>Metrics</code> is required for <code>GetCostAndUsage</code> requests.<br/>Filter:  Filters AWS costs by different dimensions. For example, you can specify <code>SERVICE</code> and <code>LINKED_ACCOUNT</code> and get the costs that are associated with that account's usage of that service. You can nest <code>Expression</code> objects to define any combination of dimension filters. For more information, see <a href="http://docs.aws.amazon.com/aws-cost-management/latest/APIReference/API_Expression.html">Expression</a>. :<br/>Granularity:  Sets the AWS cost granularity to <code>MONTHLY</code> or <code>DAILY</code>. If <code>Granularity</code> isn't set, the response object doesn't include the <code>Granularity</code>, either <code>MONTHLY</code> or <code>DAILY</code>.  The <code>GetCostAndUsageRequest</code> operation supports only <code>DAILY</code> and <code>MONTHLY</code> granularities.<br/>TimePeriod:  Sets the start and end dates for retrieving AWS costs. The start date is inclusive, but the end date is exclusive. For example, if <code>start</code> is <code>2017-01-01</code> and <code>end</code> is <code>2017-05-01</code>, then the cost and usage data is retrieved from <code>2017-01-01</code> up to and including <code>2017-04-30</code> but not including <code>2017-05-01</code>.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>End</i> The end of the time period that you want the usage and costs for. The end date is exclusive. For example, if <code>end</code> is <code>2017-05-01</code>, AWS retrieves cost and usage data from the start date up to, but not including, <code>2017-05-01</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Start</i> The beginning of the time period that you want the usage and costs for. The start date is inclusive. For example, if <code>start</code> is <code>2017-01-01</code>, AWS retrieves cost and usage data starting at <code>2017-01-01</code> up to the end date.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The specific <code>Tag</code> to use for <code>Expression</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Tags</b>: The specific <code>Tag</code> to use for <code>Expression</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Dimensions</i> The specific <code>Dimension</code> to use for <code>Expression</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Dimensions</b>: The specific <code>Dimension</code> to use for <code>Expression</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Not</i> Return results that don't match a <code>Dimension</code> object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Not</b>: Return results that don't match a <code>Dimension</code> object.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>And</i> Return results that match both <code>Dimension</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Or</i> Return results that match either <code>Dimension</code> object.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>And</i> Return results that match both <code>Dimension</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Or</i> Return results that match either <code>Dimension</code> object.<br/>Return results that don't match a <code>Dimension</code> object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Not</b>: Return results that don't match a <code>Dimension</code> object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Values</i> The metadata values that you can use to filter and group your results. You can use <code>GetDimensionValues</code> to find specific values. Valid values for the <code>SERVICE</code> dimension are <code>Amazon Elastic Compute Cloud - Compute</code>, <code>Amazon Elasticsearch Service</code>, <code>Amazon ElastiCache</code>, <code>Amazon Redshift</code>, and <code>Amazon Relational Database Service</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> The names of the metadata types that you can use to filter and group your results. For example, <code>AZ</code> returns a list of Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Values</i> The specific value of the tag.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> The key for the tag.<br/>
        :return: ResultsByTime:  The time period that is covered by the results in the response.<br/>GroupDefinitions:  The groups that are specified by the <code>Filter</code> or <code>GroupBy</code> parameters in the request.<br/>NextPageToken:  The token for the next set of retrievable results. AWS provides the token when the response from a previous call has more results than the maximum page size.<br/>        """
        pass

    def get_cost_forecast(self, Granularity: str, Metric: str, TimePeriod: DateInterval, Filter: Expression=None, PredictionIntervalLevel: int=None) -> GetCostForecastResponse:
        """Retrieves a forecast for how much Amazon Web Services predicts that you will spend over the forecast time period that you select, based on your past costs. 

        :param self:
        :param PredictionIntervalLevel:  Cost Explorer always returns the mean forecast as a single point. You can request a prediction interval around the mean by specifying a confidence level. The higher the confidence level, the more confident Cost Explorer is about the actual value falling in the prediction interval. Higher confidence levels result in wider prediction intervals.<br/>Filter:  The filters that you want to use to filter your forecast. Cost Explorer API supports all of the Cost Explorer filters.:<br/>Granularity:  How granular you want the forecast to be. You can get 3 months of <code>DAILY</code> forecasts or 12 months of <code>MONTHLY</code> forecasts. The <code>GetCostForecast</code> operation supports only <code>DAILY</code> and <code>MONTHLY</code> granularities.<br/>Metric:  Which metric Cost Explorer uses to create your forecast. For more information about blended and unblended rates, see <a href="https://aws.amazon.com/premiumsupport/knowledge-center/blended-rates-intro/">Why does the "blended" annotation appear on some line items in my bill?</a>.  Valid values for a <code>GetCostForecast</code> call are the following: <ul> <li> AMORTIZED_COST </li> <li> BLENDED_COST </li> <li> NET_AMORTIZED_COST </li> <li> NET_UNBLENDED_COST </li> <li> UNBLENDED_COST </li> </ul><br/>TimePeriod:  The period of time that you want the forecast to cover.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>End</i> The end of the time period that you want the usage and costs for. The end date is exclusive. For example, if <code>end</code> is <code>2017-05-01</code>, AWS retrieves cost and usage data from the start date up to, but not including, <code>2017-05-01</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Start</i> The beginning of the time period that you want the usage and costs for. The start date is inclusive. For example, if <code>start</code> is <code>2017-01-01</code>, AWS retrieves cost and usage data starting at <code>2017-01-01</code> up to the end date.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The specific <code>Tag</code> to use for <code>Expression</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Tags</b>: The specific <code>Tag</code> to use for <code>Expression</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Dimensions</i> The specific <code>Dimension</code> to use for <code>Expression</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Dimensions</b>: The specific <code>Dimension</code> to use for <code>Expression</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Not</i> Return results that don't match a <code>Dimension</code> object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Not</b>: Return results that don't match a <code>Dimension</code> object.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>And</i> Return results that match both <code>Dimension</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Or</i> Return results that match either <code>Dimension</code> object.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>And</i> Return results that match both <code>Dimension</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Or</i> Return results that match either <code>Dimension</code> object.<br/>Return results that don't match a <code>Dimension</code> object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Not</b>: Return results that don't match a <code>Dimension</code> object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Values</i> The metadata values that you can use to filter and group your results. You can use <code>GetDimensionValues</code> to find specific values. Valid values for the <code>SERVICE</code> dimension are <code>Amazon Elastic Compute Cloud - Compute</code>, <code>Amazon Elasticsearch Service</code>, <code>Amazon ElastiCache</code>, <code>Amazon Redshift</code>, and <code>Amazon Relational Database Service</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> The names of the metadata types that you can use to filter and group your results. For example, <code>AZ</code> returns a list of Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Values</i> The specific value of the tag.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> The key for the tag.<br/>
        :return: ForecastResultsByTime:  The forecasts for your query, in order. For <code>DAILY</code> forecasts, this is a list of days. For <code>MONTHLY</code> forecasts, this is a list of months.<br/>Total:  How much you are forecasted to spend over the forecast period, in <code>USD</code>.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Unit</i> The unit that the metric is given in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Amount</i> The actual number that represents the metric.<br/>        """
        pass

    def get_dimension_values(self, Dimension: str, TimePeriod: DateInterval, SearchString: str=None, Context: str=None, NextPageToken: str=None) -> GetDimensionValuesResponse:
        """Retrieves all available filter values for a specified filter over a period of time. You can search the dimension values for an arbitrary string. 

        :param self:
        :param NextPageToken:  The token to retrieve the next set of results. AWS provides the token when the response from a previous call has more results than the maximum page size.<br/>Context:  The context for the call to <code>GetDimensionValues</code>. This can be <code>RESERVATIONS</code> or <code>COST_AND_USAGE</code>. The default value is <code>COST_AND_USAGE</code>. If the context is set to <code>RESERVATIONS</code>, the resulting dimension values can be used in the <code>GetReservationUtilization</code> operation. If the context is set to <code>COST_AND_USAGE</code>, the resulting dimension values can be used in the <code>GetCostAndUsage</code> operation. If you set the context to <code>COST_AND_USAGE</code>, you can use the following dimensions for searching: <ul> <li> AZ - The Availability Zone. An example is <code>us-east-1a</code>. </li> <li> DATABASE_ENGINE - The Amazon Relational Database Service database. Examples are Aurora or MySQL. </li> <li> INSTANCE_TYPE - The type of Amazon EC2 instance. An example is <code>m4.xlarge</code>. </li> <li> LEGAL_ENTITY_NAME - The name of the organization that sells you AWS services, such as Amazon Web Services. </li> <li> LINKED_ACCOUNT - The description in the attribute map that includes the full name of the member account. The value field contains the AWS ID of the member account. </li> <li> OPERATING_SYSTEM - The operating system. Examples are Windows or Linux. </li> <li> OPERATION - The action performed. Examples include <code>RunInstance</code> and <code>CreateBucket</code>. </li> <li> PLATFORM - The Amazon EC2 operating system. Examples are Windows or Linux. </li> <li> PURCHASE_TYPE - The reservation type of the purchase to which this usage is related. Examples include On-Demand Instances and Standard Reserved Instances. </li> <li> SERVICE - The AWS service such as Amazon DynamoDB. </li> <li> USAGE_TYPE - The type of usage. An example is DataTransfer-In-Bytes. The response for the <code>GetDimensionValues</code> operation includes a unit attribute. Examples include GB and Hrs. </li> <li> USAGE_TYPE_GROUP - The grouping of common usage types. An example is Amazon EC2: CloudWatch – Alarms. The response for this operation includes a unit attribute. </li> <li> RECORD_TYPE - The different types of charges such as RI fees, usage costs, tax refunds, and credits. </li> </ul> If you set the context to <code>RESERVATIONS</code>, you can use the following dimensions for searching: <ul> <li> AZ - The Availability Zone. An example is <code>us-east-1a</code>. </li> <li> CACHE_ENGINE - The Amazon ElastiCache operating system. Examples are Windows or Linux. </li> <li> DEPLOYMENT_OPTION - The scope of Amazon Relational Database Service deployments. Valid values are <code>SingleAZ</code> and <code>MultiAZ</code>. </li> <li> INSTANCE_TYPE - The type of Amazon EC2 instance. An example is <code>m4.xlarge</code>. </li> <li> LINKED_ACCOUNT - The description in the attribute map that includes the full name of the member account. The value field contains the AWS ID of the member account. </li> <li> PLATFORM - The Amazon EC2 operating system. Examples are Windows or Linux. </li> <li> REGION - The AWS Region. </li> <li> SCOPE (Utilization only) - The scope of a Reserved Instance (RI). Values are regional or a single Availability Zone. </li> <li> TAG (Coverage only) - The tags that are associated with a Reserved Instance (RI). </li> <li> TENANCY - The tenancy of a resource. Examples are shared or dedicated. </li> </ul><br/>Dimension:  The name of the dimension. Each <code>Dimension</code> is available for a different <code>Context</code>. For more information, see <code>Context</code>.<br/>TimePeriod:  The start and end dates for retrieving the dimension values. The start date is inclusive, but the end date is exclusive. For example, if <code>start</code> is <code>2017-01-01</code> and <code>end</code> is <code>2017-05-01</code>, then the cost and usage data is retrieved from <code>2017-01-01</code> up to and including <code>2017-04-30</code> but not including <code>2017-05-01</code>.:<br/>SearchString:  The value that you want to search the filter values for.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>End</i> The end of the time period that you want the usage and costs for. The end date is exclusive. For example, if <code>end</code> is <code>2017-05-01</code>, AWS retrieves cost and usage data from the start date up to, but not including, <code>2017-05-01</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Start</i> The beginning of the time period that you want the usage and costs for. The start date is inclusive. For example, if <code>start</code> is <code>2017-01-01</code>, AWS retrieves cost and usage data starting at <code>2017-01-01</code> up to the end date.<br/>
        :return: NextPageToken:  The token for the next set of retrievable results. AWS provides the token when the response from a previous call has more results than the maximum page size.<br/>TotalSize:  The total number of search results.<br/>ReturnSize:  The number of results that AWS returned at one time.<br/>DimensionValues:  The filters that you used to filter your request. Some dimensions are available only for a specific context. If you set the context to <code>COST_AND_USAGE</code>, you can use the following dimensions for searching: <ul> <li> AZ - The Availability Zone. An example is <code>us-east-1a</code>. </li> <li> DATABASE_ENGINE - The Amazon Relational Database Service database. Examples are Aurora or MySQL. </li> <li> INSTANCE_TYPE - The type of Amazon EC2 instance. An example is <code>m4.xlarge</code>. </li> <li> LEGAL_ENTITY_NAME - The name of the organization that sells you AWS services, such as Amazon Web Services. </li> <li> LINKED_ACCOUNT - The description in the attribute map that includes the full name of the member account. The value field contains the AWS ID of the member account. </li> <li> OPERATING_SYSTEM - The operating system. Examples are Windows or Linux. </li> <li> OPERATION - The action performed. Examples include <code>RunInstance</code> and <code>CreateBucket</code>. </li> <li> PLATFORM - The Amazon EC2 operating system. Examples are Windows or Linux. </li> <li> PURCHASE_TYPE - The reservation type of the purchase to which this usage is related. Examples include On-Demand Instances and Standard Reserved Instances. </li> <li> SERVICE - The AWS service such as Amazon DynamoDB. </li> <li> USAGE_TYPE - The type of usage. An example is DataTransfer-In-Bytes. The response for the <code>GetDimensionValues</code> operation includes a unit attribute. Examples include GB and Hrs. </li> <li> USAGE_TYPE_GROUP - The grouping of common usage types. An example is Amazon EC2: CloudWatch – Alarms. The response for this operation includes a unit attribute. </li> <li> RECORD_TYPE - The different types of charges such as RI fees, usage costs, tax refunds, and credits. </li> </ul> If you set the context to <code>RESERVATIONS</code>, you can use the following dimensions for searching: <ul> <li> AZ - The Availability Zone. An example is <code>us-east-1a</code>. </li> <li> CACHE_ENGINE - The Amazon ElastiCache operating system. Examples are Windows or Linux. </li> <li> DEPLOYMENT_OPTION - The scope of Amazon Relational Database Service deployments. Valid values are <code>SingleAZ</code> and <code>MultiAZ</code>. </li> <li> INSTANCE_TYPE - The type of Amazon EC2 instance. An example is <code>m4.xlarge</code>. </li> <li> LINKED_ACCOUNT - The description in the attribute map that includes the full name of the member account. The value field contains the AWS ID of the member account. </li> <li> PLATFORM - The Amazon EC2 operating system. Examples are Windows or Linux. </li> <li> REGION - The AWS Region. </li> <li> SCOPE (Utilization only) - The scope of a Reserved Instance (RI). Values are regional or a single Availability Zone. </li> <li> TAG (Coverage only) - The tags that are associated with a Reserved Instance (RI). </li> <li> TENANCY - The tenancy of a resource. Examples are shared or dedicated. </li> </ul><br/>        """
        pass

    def get_reservation_coverage(self, TimePeriod: DateInterval, GroupBy: List[GroupDefinition]=None, Granularity: str=None, Filter: Expression=None, Metrics: List[MetricName]=None, NextPageToken: str=None) -> GetReservationCoverageResponse:
        """Retrieves the reservation coverage for your account. This enables you to see how much of your Amazon Elastic Compute Cloud, Amazon ElastiCache, Amazon Relational Database Service, or Amazon Redshift usage is covered by a reservation. An organization's master account can see the coverage of the associated member accounts. For any time period, you can filter data about reservation usage by the following dimensions: <ul> <li> AZ </li> <li> CACHE_ENGINE </li> <li> DATABASE_ENGINE </li> <li> DEPLOYMENT_OPTION </li> <li> INSTANCE_TYPE </li> <li> LINKED_ACCOUNT </li> <li> OPERATING_SYSTEM </li> <li> PLATFORM </li> <li> REGION </li> <li> SERVICE </li> <li> TAG </li> <li> TENANCY </li> </ul> To determine valid values for a dimension, use the <code>GetDimensionValues</code> operation. 

        :param self:
        :param NextPageToken:  The token to retrieve the next set of results. AWS provides the token when the response from a previous call has more results than the maximum page size.<br/>Metrics:  The measurement that you want your reservation coverage reported in. Valid values are <code>Hour</code>, <code>Unit</code>, and <code>Cost</code>. You can use multiple values in a request.<br/>Filter:  Filters utilization data by dimensions. You can filter by the following dimensions: <ul> <li> AZ </li> <li> CACHE_ENGINE </li> <li> DATABASE_ENGINE </li> <li> DEPLOYMENT_OPTION </li> <li> INSTANCE_TYPE </li> <li> LINKED_ACCOUNT </li> <li> OPERATING_SYSTEM </li> <li> PLATFORM </li> <li> REGION </li> <li> SERVICE </li> <li> TAG </li> <li> TENANCY </li> </ul>  <code>GetReservationCoverage</code> uses the same <a href="http://docs.aws.amazon.com/aws-cost-management/latest/APIReference/API_Expression.html">Expression</a> object as the other operations, but only <code>AND</code> is supported among each dimension. You can nest only one level deep. If there are multiple values for a dimension, they are OR'd together. If you don't provide a <code>SERVICE</code> filter, Cost Explorer defaults to EC2.:<br/>Granularity:  The granularity of the AWS cost data for the reservation. Valid values are <code>MONTHLY</code> and <code>DAILY</code>. If <code>GroupBy</code> is set, <code>Granularity</code> can't be set. If <code>Granularity</code> isn't set, the response object doesn't include <code>Granularity</code>, either <code>MONTHLY</code> or <code>DAILY</code>. The <code>GetReservationCoverage</code> operation supports only <code>DAILY</code> and <code>MONTHLY</code> granularities.<br/>GroupBy:  You can group the data by the following attributes: <ul> <li> AZ </li> <li> CACHE_ENGINE </li> <li> DATABASE_ENGINE </li> <li> DEPLOYMENT_OPTION </li> <li> INSTANCE_TYPE </li> <li> LINKED_ACCOUNT </li> <li> OPERATING_SYSTEM </li> <li> PLATFORM </li> <li> REGION </li> <li> TENANCY </li> </ul><br/>TimePeriod:  The start and end dates of the period that you want to retrieve data about reservation coverage for. You can retrieve data for a maximum of 13 months: the last 12 months and the current month. The start date is inclusive, but the end date is exclusive. For example, if <code>start</code> is <code>2017-01-01</code> and <code>end</code> is <code>2017-05-01</code>, then the cost and usage data is retrieved from <code>2017-01-01</code> up to and including <code>2017-04-30</code> but not including <code>2017-05-01</code>. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>End</i> The end of the time period that you want the usage and costs for. The end date is exclusive. For example, if <code>end</code> is <code>2017-05-01</code>, AWS retrieves cost and usage data from the start date up to, but not including, <code>2017-05-01</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Start</i> The beginning of the time period that you want the usage and costs for. The start date is inclusive. For example, if <code>start</code> is <code>2017-01-01</code>, AWS retrieves cost and usage data starting at <code>2017-01-01</code> up to the end date.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The specific <code>Tag</code> to use for <code>Expression</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Tags</b>: The specific <code>Tag</code> to use for <code>Expression</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Dimensions</i> The specific <code>Dimension</code> to use for <code>Expression</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Dimensions</b>: The specific <code>Dimension</code> to use for <code>Expression</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Not</i> Return results that don't match a <code>Dimension</code> object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Not</b>: Return results that don't match a <code>Dimension</code> object.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>And</i> Return results that match both <code>Dimension</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Or</i> Return results that match either <code>Dimension</code> object.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>And</i> Return results that match both <code>Dimension</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Or</i> Return results that match either <code>Dimension</code> object.<br/>Return results that don't match a <code>Dimension</code> object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Not</b>: Return results that don't match a <code>Dimension</code> object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Values</i> The metadata values that you can use to filter and group your results. You can use <code>GetDimensionValues</code> to find specific values. Valid values for the <code>SERVICE</code> dimension are <code>Amazon Elastic Compute Cloud - Compute</code>, <code>Amazon Elasticsearch Service</code>, <code>Amazon ElastiCache</code>, <code>Amazon Redshift</code>, and <code>Amazon Relational Database Service</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> The names of the metadata types that you can use to filter and group your results. For example, <code>AZ</code> returns a list of Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Values</i> The specific value of the tag.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> The key for the tag.<br/>
        :return: NextPageToken:  The token for the next set of retrievable results. AWS provides the token when the response from a previous call has more results than the maximum page size.<br/>Total:  The total amount of instance usage that a reservation covered.:<br/>CoveragesByTime:  The amount of time that your reservations covered.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CoverageCost</i> The amount of cost that the reservation covered.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CoverageCost</b>: The amount of cost that the reservation covered.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CoverageNormalizedUnits</i> The amount of instance usage that the reservation covered, in normalized units.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CoverageNormalizedUnits</b>: The amount of instance usage that the reservation covered, in normalized units.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CoverageHours</i> The amount of instance usage that the reservation covered, in hours.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CoverageHours</b>: The amount of instance usage that the reservation covered, in hours.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CoverageHoursPercentage</i> The percentage of instance hours that a reservation covered.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalRunningHours</i> The total instance usage, in hours.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservedHours</i> The number of instance running hours that reservations covered.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OnDemandHours</i> The number of instance running hours that On-Demand Instances covered.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CoverageNormalizedUnitsPercentage</i> The percentage of your used instance normalized units that a reservation covers.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalRunningNormalizedUnits</i> The total number of normalized units that you used.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservedNormalizedUnits</i> The number of normalized units that a reservation covers.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OnDemandNormalizedUnits</i> The number of normalized units that are covered by On-Demand Instances instead of a reservation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OnDemandCost</i> How much an On-Demand instance cost.<br/>        """
        pass

    def get_reservation_purchase_recommendation(self, Service: str, AccountId: str=None, AccountScope: str=None, LookbackPeriodInDays: str=None, TermInYears: str=None, PaymentOption: str=None, ServiceSpecification: ServiceSpecification=None, PageSize: int=None, NextPageToken: str=None) -> GetReservationPurchaseRecommendationResponse:
        """Gets recommendations for which reservations to purchase. These recommendations could help you reduce your costs. Reservations provide a discounted hourly rate (up to 75%) compared to On-Demand pricing. AWS generates your recommendations by identifying your On-Demand usage during a specific time period and collecting your usage into categories that are eligible for a reservation. After AWS has these categories, it simulates every combination of reservations in each category of usage to identify the best number of each type of RI to purchase to maximize your estimated savings.  For example, AWS automatically aggregates your Amazon EC2 Linux, shared tenancy, and c4 family usage in the US West (Oregon) Region and recommends that you buy size-flexible regional reservations to apply to the c4 family usage. AWS recommends the smallest size instance in an instance family. This makes it easier to purchase a size-flexible RI. AWS also shows the equal number of normalized units so that you can purchase any instance size that you want. For this example, your RI recommendation would be for <code>c4.large</code> because that is the smallest size instance in the c4 instance family.

        :param self:
        :param NextPageToken:  The pagination token that indicates the next set of results that you want to retrieve.<br/>PageSize:  The number of recommendations that you want returned in a single response object.<br/>ServiceSpecification:  The hardware specifications for the service instances that you want recommendations for, such as standard or convertible Amazon EC2 instances.:<br/>PaymentOption:  The reservation purchase option that you want recommendations for.<br/>TermInYears:  The reservation term that you want recommendations for.<br/>LookbackPeriodInDays:  The number of previous days that you want AWS to consider when it calculates your recommendations.<br/>AccountScope:  The account scope that you want recommendations for. <code>PAYER</code> means that AWS includes the master account and any member accounts when it calculates its recommendations. <code>LINKED</code> means that AWS includes only member accounts when it calculates its recommendations. Valid values are <code>PAYER</code> and <code>LINKED</code>.<br/>Service:  The specific service that you want recommendations for.<br/>AccountId:  The account ID that is associated with the recommendation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EC2Specification</i> The Amazon EC2 hardware specifications that you want AWS to provide recommendations for.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EC2Specification</b>: The Amazon EC2 hardware specifications that you want AWS to provide recommendations for.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OfferingClass</i> Whether you want a recommendation for standard or convertible reservations.<br/>
        :return: NextPageToken:  The pagination token for the next set of retrievable results.<br/>Recommendations:  Recommendations for reservations to purchase.<br/>Metadata:  Information about this specific recommendation call, such as the time stamp for when Cost Explorer generated this recommendation.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GenerationTimestamp</i> The time stamp for when AWS made this recommendation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecommendationId</i> The ID for this specific recommendation.<br/>        """
        pass

    def get_reservation_utilization(self, TimePeriod: DateInterval, GroupBy: List[GroupDefinition]=None, Granularity: str=None, Filter: Expression=None, NextPageToken: str=None) -> GetReservationUtilizationResponse:
        """Retrieves the reservation utilization for your account. Master accounts in an organization have access to member accounts. You can filter data by dimensions in a time period. You can use <code>GetDimensionValues</code> to determine the possible dimension values. Currently, you can group only by <code>SUBSCRIPTION_ID</code>. 

        :param self:
        :param NextPageToken:  The token to retrieve the next set of results. AWS provides the token when the response from a previous call has more results than the maximum page size.<br/>Filter:  Filters utilization data by dimensions. You can filter by the following dimensions: <ul> <li> AZ </li> <li> CACHE_ENGINE </li> <li> DATABASE_ENGINE </li> <li> DEPLOYMENT_OPTION </li> <li> INSTANCE_TYPE </li> <li> LINKED_ACCOUNT </li> <li> OPERATING_SYSTEM </li> <li> PLATFORM </li> <li> REGION </li> <li> SERVICE </li> <li> SCOPE </li> <li> TENANCY </li> </ul>  <code>GetReservationUtilization</code> uses the same <a href="http://docs.aws.amazon.com/aws-cost-management/latest/APIReference/API_Expression.html">Expression</a> object as the other operations, but only <code>AND</code> is supported among each dimension, and nesting is supported up to only one level deep. If there are multiple values for a dimension, they are OR'd together.:<br/>Granularity:  If <code>GroupBy</code> is set, <code>Granularity</code> can't be set. If <code>Granularity</code> isn't set, the response object doesn't include <code>Granularity</code>, either <code>MONTHLY</code> or <code>DAILY</code>. If both <code>GroupBy</code> and <code>Granularity</code> aren't set, <code>GetReservationUtilization</code> defaults to <code>DAILY</code>. The <code>GetReservationUtilization</code> operation supports only <code>DAILY</code> and <code>MONTHLY</code> granularities.<br/>GroupBy:  Groups only by <code>SUBSCRIPTION_ID</code>. Metadata is included.<br/>TimePeriod:  Sets the start and end dates for retrieving RI utilization. The start date is inclusive, but the end date is exclusive. For example, if <code>start</code> is <code>2017-01-01</code> and <code>end</code> is <code>2017-05-01</code>, then the cost and usage data is retrieved from <code>2017-01-01</code> up to and including <code>2017-04-30</code> but not including <code>2017-05-01</code>. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>End</i> The end of the time period that you want the usage and costs for. The end date is exclusive. For example, if <code>end</code> is <code>2017-05-01</code>, AWS retrieves cost and usage data from the start date up to, but not including, <code>2017-05-01</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Start</i> The beginning of the time period that you want the usage and costs for. The start date is inclusive. For example, if <code>start</code> is <code>2017-01-01</code>, AWS retrieves cost and usage data starting at <code>2017-01-01</code> up to the end date.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The specific <code>Tag</code> to use for <code>Expression</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Tags</b>: The specific <code>Tag</code> to use for <code>Expression</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Dimensions</i> The specific <code>Dimension</code> to use for <code>Expression</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Dimensions</b>: The specific <code>Dimension</code> to use for <code>Expression</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Not</i> Return results that don't match a <code>Dimension</code> object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Not</b>: Return results that don't match a <code>Dimension</code> object.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>And</i> Return results that match both <code>Dimension</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Or</i> Return results that match either <code>Dimension</code> object.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>And</i> Return results that match both <code>Dimension</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Or</i> Return results that match either <code>Dimension</code> object.<br/>Return results that don't match a <code>Dimension</code> object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Not</b>: Return results that don't match a <code>Dimension</code> object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Values</i> The metadata values that you can use to filter and group your results. You can use <code>GetDimensionValues</code> to find specific values. Valid values for the <code>SERVICE</code> dimension are <code>Amazon Elastic Compute Cloud - Compute</code>, <code>Amazon Elasticsearch Service</code>, <code>Amazon ElastiCache</code>, <code>Amazon Redshift</code>, and <code>Amazon Relational Database Service</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> The names of the metadata types that you can use to filter and group your results. For example, <code>AZ</code> returns a list of Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Values</i> The specific value of the tag.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> The key for the tag.<br/>
        :return: NextPageToken:  The token for the next set of retrievable results. AWS provides the token when the response from a previous call has more results than the maximum page size.<br/>Total:  The total amount of time that you used your RIs.:<br/>UtilizationsByTime:  The amount of time that you used your RIs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalAmortizedFee</i> The total cost of your reservation, amortized over the reservation period.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AmortizedRecurringFee</i> The monthly cost of your reservation, amortized over the reservation period.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AmortizedUpfrontFee</i> The upfront cost of your reservation, amortized over the reservation period.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalPotentialRISavings</i> How much you could save if you use your entire reservation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetRISavings</i> How much you saved due to purchasing and utilizing reservation. AWS calculates this by subtracting <code>TotalAmortizedFee</code> from <code>OnDemandCostOfRIHoursUsed</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OnDemandCostOfRIHoursUsed</i> How much your reservation would cost if charged On-Demand rates.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UnusedUnits</i> The number of Amazon EC2 reservation hours that you didn't use, converted to normalized units. Normalized units are available only for Amazon EC2 usage after November 11, 2017.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UnusedHours</i> The number of reservation hours that you didn't use.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalActualUnits</i> The total number of Amazon EC2 reservation hours that you used, converted to normalized units. Normalized units are available only for Amazon EC2 usage after November 11, 2017.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalActualHours</i> The total number of reservation hours that you used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PurchasedUnits</i> How many Amazon EC2 reservation hours that you purchased, converted to normalized units. Normalized units are available only for Amazon EC2 usage after November 11, 2017.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PurchasedHours</i> How many reservation hours that you purchased.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UtilizationPercentageInUnits</i> The percentage of Amazon EC2 reservation time that you used, converted to normalized units. Normalized units are available only for Amazon EC2 usage after November 11, 2017.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UtilizationPercentage</i> The percentage of reservation time that you used.<br/>        """
        pass

    def get_tags(self, TimePeriod: DateInterval, SearchString: str=None, TagKey: str=None, NextPageToken: str=None) -> GetTagsResponse:
        """Queries for available tag keys and tag values for a specified period. You can search the tag values for an arbitrary string. 

        :param self:
        :param NextPageToken:  The token to retrieve the next set of results. AWS provides the token when the response from a previous call has more results than the maximum page size.<br/>TagKey:  The key of the tag that you want to return values for.<br/>TimePeriod:  The start and end dates for retrieving the dimension values. The start date is inclusive, but the end date is exclusive. For example, if <code>start</code> is <code>2017-01-01</code> and <code>end</code> is <code>2017-05-01</code>, then the cost and usage data is retrieved from <code>2017-01-01</code> up to and including <code>2017-04-30</code> but not including <code>2017-05-01</code>.:<br/>SearchString:  The value that you want to search for.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>End</i> The end of the time period that you want the usage and costs for. The end date is exclusive. For example, if <code>end</code> is <code>2017-05-01</code>, AWS retrieves cost and usage data from the start date up to, but not including, <code>2017-05-01</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Start</i> The beginning of the time period that you want the usage and costs for. The start date is inclusive. For example, if <code>start</code> is <code>2017-01-01</code>, AWS retrieves cost and usage data starting at <code>2017-01-01</code> up to the end date.<br/>
        :return: TotalSize:  The total number of query results.<br/>ReturnSize:  The number of query results that AWS returns at a time.<br/>Tags:  The tags that match your request.<br/>NextPageToken:  The token for the next set of retrievable results. AWS provides the token when the response from a previous call has more results than the maximum page size.<br/>        """
        pass

    def get_usage_forecast(self, Granularity: str, Metric: str, TimePeriod: DateInterval, Filter: Expression=None, PredictionIntervalLevel: int=None) -> GetUsageForecastResponse:
        """Retrieves a forecast for how much Amazon Web Services predicts that you will use over the forecast time period that you select, based on your past usage. 

        :param self:
        :param PredictionIntervalLevel:  Cost Explorer always returns the mean forecast as a single point. You can request a prediction interval around the mean by specifying a confidence level. The higher the confidence level, the more confident Cost Explorer is about the actual value falling in the prediction interval. Higher confidence levels result in wider prediction intervals.<br/>Filter:  The filters that you want to use to filter your forecast. Cost Explorer API supports all of the Cost Explorer filters.:<br/>Granularity:  How granular you want the forecast to be. You can get 3 months of <code>DAILY</code> forecasts or 12 months of <code>MONTHLY</code> forecasts. The <code>GetUsageForecast</code> operation supports only <code>DAILY</code> and <code>MONTHLY</code> granularities.<br/>Metric:  Which metric Cost Explorer uses to create your forecast. Valid values for a <code>GetUsageForecast</code> call are the following: <ul> <li> USAGE_QUANTITY </li> <li> NORMALIZED_USAGE_AMOUNT </li> </ul><br/>TimePeriod:  The start and end dates of the period that you want to retrieve usage forecast for. The start date is inclusive, but the end date is exclusive. For example, if <code>start</code> is <code>2017-01-01</code> and <code>end</code> is <code>2017-05-01</code>, then the cost and usage data is retrieved from <code>2017-01-01</code> up to and including <code>2017-04-30</code> but not including <code>2017-05-01</code>.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>End</i> The end of the time period that you want the usage and costs for. The end date is exclusive. For example, if <code>end</code> is <code>2017-05-01</code>, AWS retrieves cost and usage data from the start date up to, but not including, <code>2017-05-01</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Start</i> The beginning of the time period that you want the usage and costs for. The start date is inclusive. For example, if <code>start</code> is <code>2017-01-01</code>, AWS retrieves cost and usage data starting at <code>2017-01-01</code> up to the end date.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The specific <code>Tag</code> to use for <code>Expression</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Tags</b>: The specific <code>Tag</code> to use for <code>Expression</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Dimensions</i> The specific <code>Dimension</code> to use for <code>Expression</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Dimensions</b>: The specific <code>Dimension</code> to use for <code>Expression</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Not</i> Return results that don't match a <code>Dimension</code> object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Not</b>: Return results that don't match a <code>Dimension</code> object.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>And</i> Return results that match both <code>Dimension</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Or</i> Return results that match either <code>Dimension</code> object.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>And</i> Return results that match both <code>Dimension</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Or</i> Return results that match either <code>Dimension</code> object.<br/>Return results that don't match a <code>Dimension</code> object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Not</b>: Return results that don't match a <code>Dimension</code> object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Values</i> The metadata values that you can use to filter and group your results. You can use <code>GetDimensionValues</code> to find specific values. Valid values for the <code>SERVICE</code> dimension are <code>Amazon Elastic Compute Cloud - Compute</code>, <code>Amazon Elasticsearch Service</code>, <code>Amazon ElastiCache</code>, <code>Amazon Redshift</code>, and <code>Amazon Relational Database Service</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> The names of the metadata types that you can use to filter and group your results. For example, <code>AZ</code> returns a list of Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Values</i> The specific value of the tag.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> The key for the tag.<br/>
        :return: ForecastResultsByTime:  The forecasts for your query, in order. For <code>DAILY</code> forecasts, this is a list of days. For <code>MONTHLY</code> forecasts, this is a list of months.<br/>Total:  How much you're forecasted to use over the forecast period.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Unit</i> The unit that the metric is given in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Amount</i> The actual number that represents the metric.<br/>        """
        pass



class Chime(BaseClient):
    """The Amazon Chime API (application programming interface) is designed for administrators to use to perform key tasks, such as creating and managing Amazon Chime accounts and users. This guide provides detailed information about the Amazon Chime API, including operations, types, inputs and outputs, and error codes. You can use an AWS SDK, the AWS Command Line Interface (AWS CLI), or the REST API to make API calls. We recommend using an AWS SDK or the AWS CLI. Each API operation includes links to information about using it with a language-specific AWS SDK or the AWS CLI. <dl> <dt>Using an AWS SDK</dt> <dd> You don't need to write code to calculate a signature for request authentication. The SDK clients authenticate your requests by using access keys that you provide. For more information about AWS SDKs, see the <a href="http://aws.amazon.com/developer/">AWS Developer Center</a>. </dd> <dt>Using the AWS CLI</dt> <dd> Use your access keys with the AWS CLI to make API calls. For information about setting up the AWS CLI, see <a href="https://docs.aws.amazon.com/cli/latest/userguide/installing.html">Installing the AWS Command Line Interface</a> in the <i>AWS Command Line Interface User Guide</i>. For a list of available Amazon Chime commands, see the <a href="https://docs.aws.amazon.com/cli/latest/reference/chime/index.html">Amazon Chime commands</a> in the <i>AWS CLI Command Reference</i>. </dd> <dt>Using REST API</dt> <dd> If you use REST to make API calls, you must authenticate your request by providing a signature. Amazon Chime supports signature version 4. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/signature-version-4.html">Signature Version 4 Signing Process</a> in the <i>Amazon Web Services General Reference</i>. When making REST API calls, use the service name <code>chime</code> and REST endpoint <code>https://service.chime.aws.amazon.com</code>. </dd> </dl> Administrative permissions are controlled using AWS Identity and Access Management (IAM). For more information, see <a href="https://docs.aws.amazon.com/chime/latest/ag/control-access.html">Control Access to the Amazon Chime Console</a> in the <i>Amazon Chime Administration Guide</i>."""


    class ListAccountsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_accounts` for valid parameters"""
            pass
    class ListUsersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_users` for valid parameters"""
            pass


    class AssociatePhoneNumberWithUserRequest(Mapping):
        pass
    
    class AssociatePhoneNumberWithUserResponse(Mapping):
        pass
    
    class AssociatePhoneNumbersWithVoiceConnectorRequest(Mapping):
        pass
    
    class AssociatePhoneNumbersWithVoiceConnectorResponse(Mapping):
        pass
    
    class E164PhoneNumber(object):
        pass
    
    class BatchDeletePhoneNumberRequest(Mapping):
        pass
    
    class BatchDeletePhoneNumberResponse(Mapping):
        pass
    
    class String(object):
        pass
    
    class BatchSuspendUserRequest(Mapping):
        pass
    
    class BatchSuspendUserResponse(Mapping):
        pass
    
    class NonEmptyString(object):
        pass
    
    class BatchUnsuspendUserRequest(Mapping):
        pass
    
    class BatchUnsuspendUserResponse(Mapping):
        pass
    
    class BatchUpdatePhoneNumberRequest(Mapping):
        pass
    
    class BatchUpdatePhoneNumberResponse(Mapping):
        pass
    
    class UpdatePhoneNumberRequestItem(Mapping):
        pass
    
    class BatchUpdateUserRequest(Mapping):
        pass
    
    class BatchUpdateUserResponse(Mapping):
        pass
    
    class UpdateUserRequestItem(Mapping):
        pass
    
    class CreateAccountRequest(Mapping):
        pass
    
    class CreateAccountResponse(Mapping):
        pass
    
    class CreateBotRequest(Mapping):
        pass
    
    class CreateBotResponse(Mapping):
        pass
    
    class CreatePhoneNumberOrderRequest(Mapping):
        pass
    
    class CreatePhoneNumberOrderResponse(Mapping):
        pass
    
    class CreateVoiceConnectorRequest(Mapping):
        pass
    
    class CreateVoiceConnectorResponse(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class DeleteAccountRequest(Mapping):
        pass
    
    class DeleteAccountResponse(Mapping):
        pass
    
    class DeleteEventsConfigurationRequest(Mapping):
        pass
    
    class DeletePhoneNumberRequest(Mapping):
        pass
    
    class DeleteVoiceConnectorRequest(Mapping):
        pass
    
    class DeleteVoiceConnectorOriginationRequest(Mapping):
        pass
    
    class DeleteVoiceConnectorTerminationRequest(Mapping):
        pass
    
    class DeleteVoiceConnectorTerminationCredentialsRequest(Mapping):
        pass
    
    class SensitiveString(object):
        pass
    
    class DisassociatePhoneNumberFromUserRequest(Mapping):
        pass
    
    class DisassociatePhoneNumberFromUserResponse(Mapping):
        pass
    
    class DisassociatePhoneNumbersFromVoiceConnectorRequest(Mapping):
        pass
    
    class DisassociatePhoneNumbersFromVoiceConnectorResponse(Mapping):
        pass
    
    class GetAccountRequest(Mapping):
        pass
    
    class GetAccountResponse(Mapping):
        pass
    
    class GetAccountSettingsRequest(Mapping):
        pass
    
    class GetAccountSettingsResponse(Mapping):
        pass
    
    class GetBotRequest(Mapping):
        pass
    
    class GetBotResponse(Mapping):
        pass
    
    class GetEventsConfigurationRequest(Mapping):
        pass
    
    class GetEventsConfigurationResponse(Mapping):
        pass
    
    class GetGlobalSettingsResponse(Mapping):
        pass
    
    class GetPhoneNumberRequest(Mapping):
        pass
    
    class GetPhoneNumberResponse(Mapping):
        pass
    
    class GetPhoneNumberOrderRequest(Mapping):
        pass
    
    class GetPhoneNumberOrderResponse(Mapping):
        pass
    
    class GetUserRequest(Mapping):
        pass
    
    class GetUserResponse(Mapping):
        pass
    
    class GetUserSettingsRequest(Mapping):
        pass
    
    class GetUserSettingsResponse(Mapping):
        pass
    
    class GetVoiceConnectorRequest(Mapping):
        pass
    
    class GetVoiceConnectorResponse(Mapping):
        pass
    
    class GetVoiceConnectorOriginationRequest(Mapping):
        pass
    
    class GetVoiceConnectorOriginationResponse(Mapping):
        pass
    
    class GetVoiceConnectorTerminationRequest(Mapping):
        pass
    
    class GetVoiceConnectorTerminationResponse(Mapping):
        pass
    
    class GetVoiceConnectorTerminationHealthRequest(Mapping):
        pass
    
    class GetVoiceConnectorTerminationHealthResponse(Mapping):
        pass
    
    class InviteUsersRequest(Mapping):
        pass
    
    class InviteUsersResponse(Mapping):
        pass
    
    class EmailAddress(object):
        pass
    
    class ListAccountsRequest(Mapping):
        pass
    
    class ListAccountsResponse(Mapping):
        pass
    
    class ListBotsRequest(Mapping):
        pass
    
    class ListBotsResponse(Mapping):
        pass
    
    class ListPhoneNumberOrdersRequest(Mapping):
        pass
    
    class ListPhoneNumberOrdersResponse(Mapping):
        pass
    
    class ListPhoneNumbersRequest(Mapping):
        pass
    
    class ListPhoneNumbersResponse(Mapping):
        pass
    
    class ListUsersRequest(Mapping):
        pass
    
    class ListUsersResponse(Mapping):
        pass
    
    class ListVoiceConnectorTerminationCredentialsRequest(Mapping):
        pass
    
    class ListVoiceConnectorTerminationCredentialsResponse(Mapping):
        pass
    
    class ListVoiceConnectorsRequest(Mapping):
        pass
    
    class ListVoiceConnectorsResponse(Mapping):
        pass
    
    class LogoutUserRequest(Mapping):
        pass
    
    class LogoutUserResponse(Mapping):
        pass
    
    class PutEventsConfigurationRequest(Mapping):
        pass
    
    class PutEventsConfigurationResponse(Mapping):
        pass
    
    class PutVoiceConnectorOriginationRequest(Mapping):
        pass
    
    class PutVoiceConnectorOriginationResponse(Mapping):
        pass
    
    class Origination(Mapping):
        pass
    
    class PutVoiceConnectorTerminationRequest(Mapping):
        pass
    
    class PutVoiceConnectorTerminationResponse(Mapping):
        pass
    
    class Termination(Mapping):
        pass
    
    class PutVoiceConnectorTerminationCredentialsRequest(Mapping):
        pass
    
    class Credential(Mapping):
        pass
    
    class RegenerateSecurityTokenRequest(Mapping):
        pass
    
    class RegenerateSecurityTokenResponse(Mapping):
        pass
    
    class ResetPersonalPINRequest(Mapping):
        pass
    
    class ResetPersonalPINResponse(Mapping):
        pass
    
    class RestorePhoneNumberRequest(Mapping):
        pass
    
    class RestorePhoneNumberResponse(Mapping):
        pass
    
    class SearchAvailablePhoneNumbersRequest(Mapping):
        pass
    
    class SearchAvailablePhoneNumbersResponse(Mapping):
        pass
    
    class UpdateAccountRequest(Mapping):
        pass
    
    class UpdateAccountResponse(Mapping):
        pass
    
    class UpdateAccountSettingsRequest(Mapping):
        pass
    
    class UpdateAccountSettingsResponse(Mapping):
        pass
    
    class AccountSettings(Mapping):
        pass
    
    class UpdateBotRequest(Mapping):
        pass
    
    class UpdateBotResponse(Mapping):
        pass
    
    class NullableBoolean(object):
        pass
    
    class UpdateGlobalSettingsRequest(Mapping):
        pass
    
    class BusinessCallingSettings(Mapping):
        pass
    
    class VoiceConnectorSettings(Mapping):
        pass
    
    class UpdatePhoneNumberRequest(Mapping):
        pass
    
    class UpdatePhoneNumberResponse(Mapping):
        pass
    
    class UpdateUserRequest(Mapping):
        pass
    
    class UpdateUserResponse(Mapping):
        pass
    
    class UpdateUserSettingsRequest(Mapping):
        pass
    
    class UserSettings(Mapping):
        pass
    
    class UpdateVoiceConnectorRequest(Mapping):
        pass
    
    class UpdateVoiceConnectorResponse(Mapping):
        pass
    
    def associate_phone_number_with_user(self, E164PhoneNumber: str, UserId: str, AccountId: str) -> AssociatePhoneNumberWithUserResponse:
        """Associates a phone number with the specified Amazon Chime user.

        :param self:
        :param E164PhoneNumber:  The phone number, in E.164 format.<br/>UserId:  The user ID.<br/>AccountId:  The Amazon Chime account ID.<br/>
        :return:         """
        pass

    def associate_phone_numbers_with_voice_connector(self, VoiceConnectorId: str, E164PhoneNumbers: List[E164PhoneNumber]=None) -> AssociatePhoneNumbersWithVoiceConnectorResponse:
        """Associates a phone number with the specified Amazon Chime Voice Connector.

        :param self:
        :param E164PhoneNumbers:  List of phone numbers, in E.164 format.<br/>VoiceConnectorId:  The Amazon Chime Voice Connector ID.<br/>
        :return: PhoneNumberErrors:  If the action fails for one or more of the phone numbers in the request, a list of the phone numbers is returned, along with error codes and error messages.<br/>        """
        pass

    def batch_delete_phone_number(self, PhoneNumberIds: List[String]) -> BatchDeletePhoneNumberResponse:
        """Moves phone numbers into the <b>Deletion queue</b>. Phone numbers must be disassociated from any users or Amazon Chime Voice Connectors before they can be deleted. Phone numbers remain in the <b>Deletion queue</b> for 7 days before they are deleted permanently.

        :param self:
        :param PhoneNumberIds:  List of phone number IDs.<br/>
        :return: PhoneNumberErrors:  If the action fails for one or more of the phone numbers in the request, a list of the phone numbers is returned, along with error codes and error messages.<br/>        """
        pass

    def batch_suspend_user(self, UserIdList: List[NonEmptyString], AccountId: str) -> BatchSuspendUserResponse:
        """Suspends up to 50 users from a <code>Team</code> or <code>EnterpriseLWA</code> Amazon Chime account. For more information about different account types, see <a href="https://docs.aws.amazon.com/chime/latest/ag/manage-chime-account.html">Managing Your Amazon Chime Accounts</a> in the <i>Amazon Chime Administration Guide</i>. Users suspended from a <code>Team</code> account are dissasociated from the account, but they can continue to use Amazon Chime as free users. To remove the suspension from suspended <code>Team</code> account users, invite them to the <code>Team</code> account again. You can use the <a>InviteUsers</a> action to do so. Users suspended from an <code>EnterpriseLWA</code> account are immediately signed out of Amazon Chime and can no longer sign in. To remove the suspension from suspended <code>EnterpriseLWA</code> account users, use the <a>BatchUnsuspendUser</a> action.  To sign out users without suspending them, use the <a>LogoutUser</a> action.

        :param self:
        :param UserIdList:  The request containing the user IDs to suspend.<br/>AccountId:  The Amazon Chime account ID.<br/>
        :return: UserErrors:  If the <a>BatchSuspendUser</a> action fails for one or more of the user IDs in the request, a list of the user IDs is returned, along with error codes and error messages.<br/>        """
        pass

    def batch_unsuspend_user(self, UserIdList: List[NonEmptyString], AccountId: str) -> BatchUnsuspendUserResponse:
        """Removes the suspension from up to 50 previously suspended users for the specified Amazon Chime <code>EnterpriseLWA</code> account. Only users on <code>EnterpriseLWA</code> accounts can be unsuspended using this action. For more information about different account types, see <a href="https://docs.aws.amazon.com/chime/latest/ag/manage-chime-account.html">Managing Your Amazon Chime Accounts</a> in the <i>Amazon Chime Administration Guide</i>. Previously suspended users who are unsuspended using this action are returned to <code>Registered</code> status. Users who are not previously suspended are ignored.

        :param self:
        :param UserIdList:  The request containing the user IDs to unsuspend.<br/>AccountId:  The Amazon Chime account ID.<br/>
        :return: UserErrors:  If the <a>BatchUnsuspendUser</a> action fails for one or more of the user IDs in the request, a list of the user IDs is returned, along with error codes and error messages.<br/>        """
        pass

    def batch_update_phone_number(self, UpdatePhoneNumberRequestItems: List[UpdatePhoneNumberRequestItem]) -> BatchUpdatePhoneNumberResponse:
        """Updates phone number product types. Choose from Amazon Chime Business Calling and Amazon Chime Voice Connector product types. For toll-free numbers, you can use only the Amazon Chime Voice Connector product type.

        :param self:
        :param UpdatePhoneNumberRequestItems:  The request containing the phone number IDs and product types to update.<br/>
        :return: PhoneNumberErrors:  If the action fails for one or more of the phone numbers in the request, a list of the phone numbers is returned, along with error codes and error messages.<br/>        """
        pass

    def batch_update_user(self, UpdateUserRequestItems: List[UpdateUserRequestItem], AccountId: str) -> BatchUpdateUserResponse:
        """Updates user details within the <a>UpdateUserRequestItem</a> object for up to 20 users for the specified Amazon Chime account. Currently, only <code>LicenseType</code> updates are supported for this action.

        :param self:
        :param UpdateUserRequestItems:  The request containing the user IDs and details to update.<br/>AccountId:  The Amazon Chime account ID.<br/>
        :return: UserErrors:  If the <a>BatchUpdateUser</a> action fails for one or more of the user IDs in the request, a list of the user IDs is returned, along with error codes and error messages.<br/>        """
        pass

    def create_account(self, Name: str) -> CreateAccountResponse:
        """Creates an Amazon Chime account under the administrator's AWS account. Only <code>Team</code> account types are currently supported for this action. For more information about different account types, see <a href="https://docs.aws.amazon.com/chime/latest/ag/manage-chime-account.html">Managing Your Amazon Chime Accounts</a> in the <i>Amazon Chime Administration Guide</i>.

        :param self:
        :param Name:  The name of the Amazon Chime account.<br/>
        :return: Account:  The Amazon Chime account details.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedLicenses</i> Supported licenses for the Amazon Chime account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultLicense</i> The default license for the Amazon Chime account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The Amazon Chime account creation timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountType</i> The Amazon Chime account type. For more information about different account types, see <a href="https://docs.aws.amazon.com/chime/latest/ag/manage-chime-account.html">Managing Your Amazon Chime Accounts</a> in the <i>Amazon Chime Administration Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The Amazon Chime account name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountId</i> The Amazon Chime account ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AwsAccountId</i> The AWS account ID.<br/>        """
        pass

    def create_bot(self, DisplayName: str, AccountId: str, Domain: str=None) -> CreateBotResponse:
        """Creates a bot for an Amazon Chime Enterprise account.

        :param self:
        :param Domain:  The domain of the Amazon Chime Enterprise account.<br/>DisplayName:  The bot display name.<br/>AccountId:  The Amazon Chime account ID.<br/>
        :return: Bot:  The bot details.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityToken</i> The security token used to authenticate Amazon Chime with the outgoing event endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BotEmail</i> The bot email address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdatedTimestamp</i> The updated bot timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The bot creation timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Disabled</i> When true, the bot is stopped from running in your account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BotType</i> The bot type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayName</i> The bot display name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserId</i> The unique ID for the bot user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BotId</i> The bot ID.<br/>        """
        pass

    def create_phone_number_order(self, E164PhoneNumbers: List[E164PhoneNumber], ProductType: str) -> CreatePhoneNumberOrderResponse:
        """Creates an order for phone numbers to be provisioned. Choose from Amazon Chime Business Calling and Amazon Chime Voice Connector product types. For toll-free numbers, you can use only the Amazon Chime Voice Connector product type.

        :param self:
        :param E164PhoneNumbers:  List of phone numbers, in E.164 format.<br/>ProductType:  The phone number product type.<br/>
        :return: PhoneNumberOrder:  The phone number order details.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdatedTimestamp</i> The updated phone number order timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The phone number order creation timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OrderedPhoneNumbers</i> The ordered phone number details, such as the phone number in E.164 format and the phone number status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the phone number order.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductType</i> The phone number order product type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneNumberOrderId</i> The phone number order ID.<br/>        """
        pass

    def create_voice_connector(self, RequireEncryption: Boolean, Name: str) -> CreateVoiceConnectorResponse:
        """Creates an Amazon Chime Voice Connector under the administrator's AWS account. Enabling <a>CreateVoiceConnectorRequest$RequireEncryption</a> configures your Amazon Chime Voice Connector to use TLS transport for SIP signaling and Secure RTP (SRTP) for media. Inbound calls use TLS transport, and unencrypted outbound calls are blocked.

        :param self:
        :param RequireEncryption:  When enabled, requires encryption for the Amazon Chime Voice Connector.<br/>Name:  The name of the Amazon Chime Voice Connector.<br/>
        :return: VoiceConnector:  The Amazon Chime Voice Connector details.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdatedTimestamp</i> The updated Amazon Chime Voice Connector timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The Amazon Chime Voice Connector creation timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequireEncryption</i> Designates whether encryption is required for the Amazon Chime Voice Connector.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutboundHostName</i> The outbound host name for the Amazon Chime Voice Connector.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the Amazon Chime Voice Connector.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VoiceConnectorId</i> The Amazon Chime Voice Connector ID.<br/>        """
        pass

    def delete_account(self, AccountId: str) -> DeleteAccountResponse:
        """Deletes the specified Amazon Chime account. You must suspend all users before deleting a <code>Team</code> account. You can use the <a>BatchSuspendUser</a> action to do so. For <code>EnterpriseLWA</code> and <code>EnterpriseAD</code> accounts, you must release the claimed domains for your Amazon Chime account before deletion. As soon as you release the domain, all users under that account are suspended. Deleted accounts appear in your <code>Disabled</code> accounts list for 90 days. To restore a deleted account from your <code>Disabled</code> accounts list, you must contact AWS Support. After 90 days, deleted accounts are permanently removed from your <code>Disabled</code> accounts list.

        :param self:
        :param AccountId:  The Amazon Chime account ID.<br/>
        :return:         """
        pass

    def delete_events_configuration(self, BotId: str, AccountId: str):
        """Deletes the events configuration that allows a bot to receive outgoing events.

        :param self:
        :param BotId:  The bot ID.<br/>AccountId:  The Amazon Chime account ID.<br/>
        :return:         """
        pass

    def delete_phone_number(self, PhoneNumberId: str):
        """Moves the specified phone number into the <b>Deletion queue</b>. A phone number must be disassociated from any users or Amazon Chime Voice Connectors before it can be deleted. Deleted phone numbers remain in the <b>Deletion queue</b> for 7 days before they are deleted permanently.

        :param self:
        :param PhoneNumberId:  The phone number ID.<br/>
        :return:         """
        pass

    def delete_voice_connector(self, VoiceConnectorId: str):
        """Deletes the specified Amazon Chime Voice Connector. Any phone numbers assigned to the Amazon Chime Voice Connector must be unassigned from it before it can be deleted.

        :param self:
        :param VoiceConnectorId:  The Amazon Chime Voice Connector ID.<br/>
        :return:         """
        pass

    def delete_voice_connector_origination(self, VoiceConnectorId: str):
        """Deletes the origination settings for the specified Amazon Chime Voice Connector.

        :param self:
        :param VoiceConnectorId:  The Amazon Chime Voice Connector ID.<br/>
        :return:         """
        pass

    def delete_voice_connector_termination(self, VoiceConnectorId: str):
        """Deletes the termination settings for the specified Amazon Chime Voice Connector.

        :param self:
        :param VoiceConnectorId:  The Amazon Chime Voice Connector ID.<br/>
        :return:         """
        pass

    def delete_voice_connector_termination_credentials(self, VoiceConnectorId: str, Usernames: List[SensitiveString]=None):
        """Deletes the specified SIP credentials used by your equipment to authenticate during call termination.

        :param self:
        :param Usernames:  The RFC2617 compliant username associated with the SIP credentials, in US-ASCII format.<br/>VoiceConnectorId:  The Amazon Chime Voice Connector ID.<br/>
        :return:         """
        pass

    def disassociate_phone_number_from_user(self, UserId: str, AccountId: str) -> DisassociatePhoneNumberFromUserResponse:
        """Disassociates the primary provisioned phone number from the specified Amazon Chime user.

        :param self:
        :param UserId:  The user ID.<br/>AccountId:  The Amazon Chime account ID.<br/>
        :return:         """
        pass

    def disassociate_phone_numbers_from_voice_connector(self, VoiceConnectorId: str, E164PhoneNumbers: List[E164PhoneNumber]=None) -> DisassociatePhoneNumbersFromVoiceConnectorResponse:
        """Disassociates the specified phone number from the specified Amazon Chime Voice Connector.

        :param self:
        :param E164PhoneNumbers:  List of phone numbers, in E.164 format.<br/>VoiceConnectorId:  The Amazon Chime Voice Connector ID.<br/>
        :return: PhoneNumberErrors:  If the action fails for one or more of the phone numbers in the request, a list of the phone numbers is returned, along with error codes and error messages.<br/>        """
        pass

    def get_account(self, AccountId: str) -> GetAccountResponse:
        """Retrieves details for the specified Amazon Chime account, such as account type and supported licenses.

        :param self:
        :param AccountId:  The Amazon Chime account ID.<br/>
        :return: Account:  The Amazon Chime account details.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedLicenses</i> Supported licenses for the Amazon Chime account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultLicense</i> The default license for the Amazon Chime account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The Amazon Chime account creation timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountType</i> The Amazon Chime account type. For more information about different account types, see <a href="https://docs.aws.amazon.com/chime/latest/ag/manage-chime-account.html">Managing Your Amazon Chime Accounts</a> in the <i>Amazon Chime Administration Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The Amazon Chime account name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountId</i> The Amazon Chime account ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AwsAccountId</i> The AWS account ID.<br/>        """
        pass

    def get_account_settings(self, AccountId: str) -> GetAccountSettingsResponse:
        """Retrieves account settings for the specified Amazon Chime account ID, such as remote control and dial out settings. For more information about these settings, see <a href="https://docs.aws.amazon.com/chime/latest/ag/policies.html">Use the Policies Page</a> in the <i>Amazon Chime Administration Guide</i>.

        :param self:
        :param AccountId:  The Amazon Chime account ID.<br/>
        :return: AccountSettings:  The Amazon Chime account settings.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnableDialOut</i> Setting that allows meeting participants to choose the <b>Call me at a phone number</b> option. For more information, see <a href="https://docs.aws.amazon.com/chime/latest/ug/chime-join-meeting.html">Join a Meeting without the Amazon Chime App</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisableRemoteControl</i> Setting that stops or starts remote control of shared screens during meetings.<br/>        """
        pass

    def get_bot(self, BotId: str, AccountId: str) -> GetBotResponse:
        """Retrieves details for the specified bot, such as bot email address, bot type, status, and display name.

        :param self:
        :param BotId:  The bot ID.<br/>AccountId:  The Amazon Chime account ID.<br/>
        :return: Bot:  The chat bot details.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityToken</i> The security token used to authenticate Amazon Chime with the outgoing event endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BotEmail</i> The bot email address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdatedTimestamp</i> The updated bot timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The bot creation timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Disabled</i> When true, the bot is stopped from running in your account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BotType</i> The bot type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayName</i> The bot display name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserId</i> The unique ID for the bot user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BotId</i> The bot ID.<br/>        """
        pass

    def get_events_configuration(self, BotId: str, AccountId: str) -> GetEventsConfigurationResponse:
        """Gets details for an events configuration that allows a bot to receive outgoing events, such as an HTTPS endpoint or Lambda function ARN. 

        :param self:
        :param BotId:  The bot ID.<br/>AccountId:  The Amazon Chime account ID.<br/>
        :return: EventsConfiguration:  The events configuration details.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LambdaFunctionArn</i> Lambda function ARN that allows a bot to receive outgoing events.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutboundEventsHTTPSEndpoint</i> HTTPS endpoint that allows a bot to receive outgoing events.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BotId</i> The bot ID.<br/>        """
        pass

    def get_global_settings(self) -> GetGlobalSettingsResponse:
        """Retrieves global settings for the administrator's AWS account, such as Amazon Chime Business Calling and Amazon Chime Voice Connector settings.

        :param self:
        :param 
        :return: VoiceConnector:  The Amazon Chime Voice Connector settings.:<br/>BusinessCalling:  The Amazon Chime Business Calling settings.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CdrBucket</i> The Amazon S3 bucket designated for call detail record storage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CdrBucket</i> The Amazon S3 bucket designated for call detail record storage.<br/>        """
        pass

    def get_phone_number(self, PhoneNumberId: str) -> GetPhoneNumberResponse:
        """Retrieves details for the specified phone number ID, such as associations, capabilities, and product type.

        :param self:
        :param PhoneNumberId:  The phone number ID.<br/>
        :return: PhoneNumber:  The phone number details.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionTimestamp</i> The deleted phone number timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdatedTimestamp</i> The updated phone number timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The phone number creation timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Associations</i> The phone number associations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Capabilities</i> The phone number capabilities.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Capabilities</b>: The phone number capabilities.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The phone number status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductType</i> The phone number product type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The phone number type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>E164PhoneNumber</i> The phone number, in E.164 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneNumberId</i> The phone number ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OutboundMMS</i> Allows or denies outbound MMS messaging for the specified phone number.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InboundMMS</i> Allows or denies inbound MMS messaging for the specified phone number.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OutboundSMS</i> Allows or denies outbound SMS messaging for the specified phone number.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InboundSMS</i> Allows or denies inbound SMS messaging for the specified phone number.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OutboundCall</i> Allows or denies outbound calling for the specified phone number.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InboundCall</i> Allows or denies inbound calling for the specified phone number.<br/>        """
        pass

    def get_phone_number_order(self, PhoneNumberOrderId: str) -> GetPhoneNumberOrderResponse:
        """Retrieves details for the specified phone number order, such as order creation timestamp, phone numbers in E.164 format, product type, and order status.

        :param self:
        :param PhoneNumberOrderId:  The ID for the phone number order.<br/>
        :return: PhoneNumberOrder:  The phone number order details.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdatedTimestamp</i> The updated phone number order timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The phone number order creation timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OrderedPhoneNumbers</i> The ordered phone number details, such as the phone number in E.164 format and the phone number status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the phone number order.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductType</i> The phone number order product type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneNumberOrderId</i> The phone number order ID.<br/>        """
        pass

    def get_user(self, UserId: str, AccountId: str) -> GetUserResponse:
        """Retrieves details for the specified user ID, such as primary email address, license type, and personal meeting PIN. To retrieve user details with an email address instead of a user ID, use the <a>ListUsers</a> action, and then filter by email address.

        :param self:
        :param UserId:  The user ID.<br/>AccountId:  The Amazon Chime account ID.<br/>
        :return: User:  The user details.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PersonalPIN</i> The user's personal meeting PIN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InvitedOn</i> Date and time when the user is invited to the Amazon Chime account, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RegisteredOn</i> Date and time when the user is registered, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserInvitationStatus</i> The user invite status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserRegistrationStatus</i> The user registration status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseType</i> The license type for the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayName</i> The display name of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrimaryProvisionedNumber</i> The primary phone number associated with the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrimaryEmail</i> The primary email address of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountId</i> The Amazon Chime account ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserId</i> The user ID.<br/>        """
        pass

    def get_user_settings(self, UserId: str, AccountId: str) -> GetUserSettingsResponse:
        """Retrieves settings for the specified user ID, such as any associated phone number settings.

        :param self:
        :param UserId:  The user ID.<br/>AccountId:  The Amazon Chime account ID.<br/>
        :return: UserSettings:  The user settings.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Telephony</i> The telephony settings associated with the user.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Telephony</b>: The telephony settings associated with the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SMS</i> Allows or denies SMS messaging.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OutboundCalling</i> Allows or denies outbound calling.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InboundCalling</i> Allows or denies inbound calling.<br/>        """
        pass

    def get_voice_connector(self, VoiceConnectorId: str) -> GetVoiceConnectorResponse:
        """Retrieves details for the specified Amazon Chime Voice Connector, such as timestamps, name, outbound host, and encryption requirements.

        :param self:
        :param VoiceConnectorId:  The Amazon Chime Voice Connector ID.<br/>
        :return: VoiceConnector:  The Amazon Chime Voice Connector details.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdatedTimestamp</i> The updated Amazon Chime Voice Connector timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The Amazon Chime Voice Connector creation timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequireEncryption</i> Designates whether encryption is required for the Amazon Chime Voice Connector.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutboundHostName</i> The outbound host name for the Amazon Chime Voice Connector.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the Amazon Chime Voice Connector.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VoiceConnectorId</i> The Amazon Chime Voice Connector ID.<br/>        """
        pass

    def get_voice_connector_origination(self, VoiceConnectorId: str) -> GetVoiceConnectorOriginationResponse:
        """Retrieves origination setting details for the specified Amazon Chime Voice Connector.

        :param self:
        :param VoiceConnectorId:  The Amazon Chime Voice Connector ID.<br/>
        :return: Origination:  The origination setting details.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Disabled</i> When origination settings are disabled, inbound calls are not enabled for your Amazon Chime Voice Connector.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Routes</i> The call distribution properties defined for your SIP hosts. Valid range: Minimum value of 1. Maximum value of 20.<br/>        """
        pass

    def get_voice_connector_termination(self, VoiceConnectorId: str) -> GetVoiceConnectorTerminationResponse:
        """Retrieves termination setting details for the specified Amazon Chime Voice Connector.

        :param self:
        :param VoiceConnectorId:  The Amazon Chime Voice Connector ID.<br/>
        :return: Termination:  The termination setting details.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Disabled</i> When termination settings are disabled, outbound calls can not be made.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CidrAllowedList</i> The IP addresses allowed to make calls, in CIDR format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallingRegions</i> The countries to which calls are allowed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultPhoneNumber</i> The default caller ID phone number.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CpsLimit</i> The limit on calls per second. Max value based on account service limit. Default value of 1.<br/>        """
        pass

    def get_voice_connector_termination_health(self, VoiceConnectorId: str) -> GetVoiceConnectorTerminationHealthResponse:
        """Retrieves information about the last time a SIP <code>OPTIONS</code> ping was received from your SIP infrastructure for the specified Amazon Chime Voice Connector.

        :param self:
        :param VoiceConnectorId:  The Amazon Chime Voice Connector ID.<br/>
        :return: TerminationHealth:  The termination health details.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Source</i> The source IP address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timestamp</i> The timestamp, in ISO 8601 format.<br/>        """
        pass

    def invite_users(self, UserEmailList: List[EmailAddress], AccountId: str) -> InviteUsersResponse:
        """Sends email invites to as many as 50 users, inviting them to the specified Amazon Chime <code>Team</code> account. Only <code>Team</code> account types are currently supported for this action. 

        :param self:
        :param UserEmailList:  The user email addresses to which to send the invite.<br/>AccountId:  The Amazon Chime account ID.<br/>
        :return: Invites:  The invite details.<br/>        """
        pass

    def list_accounts(self, Name: str=None, UserEmail: str=None, NextToken: str=None, MaxResults: int=None) -> ListAccountsResponse:
        """Lists the Amazon Chime accounts under the administrator's AWS account. You can filter accounts by account name prefix. To find out which Amazon Chime account a user belongs to, you can filter by the user's email address, which returns one account result.

        :param self:
        :param MaxResults:  The maximum number of results to return in a single call. Defaults to 100.<br/>NextToken:  The token to use to retrieve the next page of results.<br/>UserEmail:  User email address with which to filter results.<br/>Name:  Amazon Chime account name prefix with which to filter results.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results.<br/>Accounts:  List of Amazon Chime accounts and account details.<br/>        """
        pass

    def list_bots(self, AccountId: str, MaxResults: int=None, NextToken: str=None) -> ListBotsResponse:
        """Lists the bots associated with the administrator's Amazon Chime Enterprise account ID.

        :param self:
        :param NextToken:  The token to use to retrieve the next page of results.<br/>MaxResults:  The maximum number of results to return in a single call. Default is 10.<br/>AccountId:  The Amazon Chime account ID.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results.<br/>Bots:  List of bots and bot details.<br/>        """
        pass

    def list_phone_number_orders(self, NextToken: str=None, MaxResults: int=None) -> ListPhoneNumberOrdersResponse:
        """Lists the phone number orders for the administrator's Amazon Chime account.

        :param self:
        :param MaxResults:  The maximum number of results to return in a single call.<br/>NextToken:  The token to use to retrieve the next page of results.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results.<br/>PhoneNumberOrders:  The phone number order details.<br/>        """
        pass

    def list_phone_numbers(self, Status: str=None, ProductType: str=None, FilterName: str=None, FilterValue: str=None, MaxResults: int=None, NextToken: str=None) -> ListPhoneNumbersResponse:
        """Lists the phone numbers for the specified Amazon Chime account, Amazon Chime user, or Amazon Chime Voice Connector.

        :param self:
        :param NextToken:  The token to use to retrieve the next page of results.<br/>MaxResults:  The maximum number of results to return in a single call.<br/>FilterValue:  The value to use for the filter.<br/>FilterName:  The filter to use to limit the number of results.<br/>ProductType:  The phone number product type.<br/>Status:  The phone number status.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results.<br/>PhoneNumbers:  The phone number details.<br/>        """
        pass

    def list_users(self, AccountId: str, UserEmail: str=None, MaxResults: int=None, NextToken: str=None) -> ListUsersResponse:
        """Lists the users that belong to the specified Amazon Chime account. You can specify an email address to list only the user that the email address belongs to.

        :param self:
        :param NextToken:  The token to use to retrieve the next page of results.<br/>MaxResults:  The maximum number of results to return in a single call. Defaults to 100.<br/>UserEmail:  Optional. The user email address used to filter results. Maximum 1.<br/>AccountId:  The Amazon Chime account ID.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results.<br/>Users:  List of users and user details.<br/>        """
        pass

    def list_voice_connector_termination_credentials(self, VoiceConnectorId: str) -> ListVoiceConnectorTerminationCredentialsResponse:
        """Lists the SIP credentials for the specified Amazon Chime Voice Connector.

        :param self:
        :param VoiceConnectorId:  The Amazon Chime Voice Connector ID.<br/>
        :return: Usernames:  A list of user names.<br/>        """
        pass

    def list_voice_connectors(self, NextToken: str=None, MaxResults: int=None) -> ListVoiceConnectorsResponse:
        """Lists the Amazon Chime Voice Connectors for the administrator's AWS account.

        :param self:
        :param MaxResults:  The maximum number of results to return in a single call.<br/>NextToken:  The token to use to retrieve the next page of results.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results.<br/>VoiceConnectors:  The details of the Amazon Chime Voice Connectors.<br/>        """
        pass

    def logout_user(self, UserId: str, AccountId: str) -> LogoutUserResponse:
        """Logs out the specified user from all of the devices they are currently logged into.

        :param self:
        :param UserId:  The user ID.<br/>AccountId:  The Amazon Chime account ID.<br/>
        :return:         """
        pass

    def put_events_configuration(self, BotId: str, AccountId: str, OutboundEventsHTTPSEndpoint: str=None, LambdaFunctionArn: str=None) -> PutEventsConfigurationResponse:
        """Creates an events configuration that allows a bot to receive outgoing events sent by Amazon Chime. Choose either an HTTPS endpoint or a Lambda function ARN. For more information, see <a>Bot</a>.

        :param self:
        :param LambdaFunctionArn:  Lambda function ARN that allows the bot to receive outgoing events.<br/>OutboundEventsHTTPSEndpoint:  HTTPS endpoint that allows the bot to receive outgoing events.<br/>BotId:  The bot ID.<br/>AccountId:  The Amazon Chime account ID.<br/>
        :return: EventsConfiguration:  The configuration that allows a bot to receive outgoing events. Can be either an HTTPS endpoint or a Lambda function ARN.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LambdaFunctionArn</i> Lambda function ARN that allows a bot to receive outgoing events.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutboundEventsHTTPSEndpoint</i> HTTPS endpoint that allows a bot to receive outgoing events.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BotId</i> The bot ID.<br/>        """
        pass

    def put_voice_connector_origination(self, Origination: Origination, VoiceConnectorId: str) -> PutVoiceConnectorOriginationResponse:
        """Adds origination settings for the specified Amazon Chime Voice Connector.

        :param self:
        :param Origination:  The origination setting details to add.:<br/>VoiceConnectorId:  The Amazon Chime Voice Connector ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Disabled</i> When origination settings are disabled, inbound calls are not enabled for your Amazon Chime Voice Connector.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Routes</i> The call distribution properties defined for your SIP hosts. Valid range: Minimum value of 1. Maximum value of 20.<br/>
        :return: Origination:  The updated origination setting details.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Disabled</i> When origination settings are disabled, inbound calls are not enabled for your Amazon Chime Voice Connector.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Routes</i> The call distribution properties defined for your SIP hosts. Valid range: Minimum value of 1. Maximum value of 20.<br/>        """
        pass

    def put_voice_connector_termination(self, Termination: Termination, VoiceConnectorId: str) -> PutVoiceConnectorTerminationResponse:
        """Adds termination settings for the specified Amazon Chime Voice Connector.

        :param self:
        :param Termination:  The termination setting details to add.:<br/>VoiceConnectorId:  The Amazon Chime Voice Connector ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Disabled</i> When termination settings are disabled, outbound calls can not be made.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CidrAllowedList</i> The IP addresses allowed to make calls, in CIDR format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallingRegions</i> The countries to which calls are allowed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultPhoneNumber</i> The default caller ID phone number.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CpsLimit</i> The limit on calls per second. Max value based on account service limit. Default value of 1.<br/>
        :return: Termination:  The updated termination setting details.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Disabled</i> When termination settings are disabled, outbound calls can not be made.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CidrAllowedList</i> The IP addresses allowed to make calls, in CIDR format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallingRegions</i> The countries to which calls are allowed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultPhoneNumber</i> The default caller ID phone number.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CpsLimit</i> The limit on calls per second. Max value based on account service limit. Default value of 1.<br/>        """
        pass

    def put_voice_connector_termination_credentials(self, VoiceConnectorId: str, Credentials: List[Credential]=None):
        """Adds termination SIP credentials for the specified Amazon Chime Voice Connector.

        :param self:
        :param Credentials:  The termination SIP credentials.<br/>VoiceConnectorId:  The Amazon Chime Voice Connector ID.<br/>
        :return:         """
        pass

    def regenerate_security_token(self, BotId: str, AccountId: str) -> RegenerateSecurityTokenResponse:
        """Regenerates the security token for a bot.

        :param self:
        :param BotId:  The bot ID.<br/>AccountId:  The Amazon Chime account ID.<br/>
        :return: Bot:  A resource that allows Enterprise account administrators to configure an interface to receive events from Amazon Chime.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityToken</i> The security token used to authenticate Amazon Chime with the outgoing event endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BotEmail</i> The bot email address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdatedTimestamp</i> The updated bot timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The bot creation timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Disabled</i> When true, the bot is stopped from running in your account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BotType</i> The bot type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayName</i> The bot display name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserId</i> The unique ID for the bot user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BotId</i> The bot ID.<br/>        """
        pass

    def reset_personal_pin(self, UserId: str, AccountId: str) -> ResetPersonalPINResponse:
        """Resets the personal meeting PIN for the specified user on an Amazon Chime account. Returns the <a>User</a> object with the updated personal meeting PIN.

        :param self:
        :param UserId:  The user ID.<br/>AccountId:  The Amazon Chime account ID.<br/>
        :return: User:  The user details and new personal meeting PIN.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PersonalPIN</i> The user's personal meeting PIN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InvitedOn</i> Date and time when the user is invited to the Amazon Chime account, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RegisteredOn</i> Date and time when the user is registered, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserInvitationStatus</i> The user invite status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserRegistrationStatus</i> The user registration status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseType</i> The license type for the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayName</i> The display name of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrimaryProvisionedNumber</i> The primary phone number associated with the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrimaryEmail</i> The primary email address of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountId</i> The Amazon Chime account ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserId</i> The user ID.<br/>        """
        pass

    def restore_phone_number(self, PhoneNumberId: str) -> RestorePhoneNumberResponse:
        """Moves a phone number from the <b>Deletion queue</b> back into the phone number <b>Inventory</b>.

        :param self:
        :param PhoneNumberId:  The phone number.<br/>
        :return: PhoneNumber:  The phone number details.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionTimestamp</i> The deleted phone number timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdatedTimestamp</i> The updated phone number timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The phone number creation timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Associations</i> The phone number associations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Capabilities</i> The phone number capabilities.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Capabilities</b>: The phone number capabilities.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The phone number status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductType</i> The phone number product type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The phone number type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>E164PhoneNumber</i> The phone number, in E.164 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneNumberId</i> The phone number ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OutboundMMS</i> Allows or denies outbound MMS messaging for the specified phone number.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InboundMMS</i> Allows or denies inbound MMS messaging for the specified phone number.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OutboundSMS</i> Allows or denies outbound SMS messaging for the specified phone number.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InboundSMS</i> Allows or denies inbound SMS messaging for the specified phone number.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OutboundCall</i> Allows or denies outbound calling for the specified phone number.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InboundCall</i> Allows or denies inbound calling for the specified phone number.<br/>        """
        pass

    def search_available_phone_numbers(self, AreaCode: str=None, City: str=None, Country: str=None, State: str=None, TollFreePrefix: str=None, MaxResults: int=None, NextToken: str=None) -> SearchAvailablePhoneNumbersResponse:
        """Searches phone numbers that can be ordered.

        :param self:
        :param NextToken:  The token to use to retrieve the next page of results.<br/>MaxResults:  The maximum number of results to return in a single call.<br/>TollFreePrefix:  The toll-free prefix that you use to filter results.<br/>State:  The state used to filter results.<br/>Country:  The country used to filter results.<br/>City:  The city used to filter results.<br/>AreaCode:  The area code used to filter results.<br/>
        :return: E164PhoneNumbers:  List of phone numbers, in E.164 format.<br/>        """
        pass

    def update_account(self, AccountId: str, Name: str=None) -> UpdateAccountResponse:
        """Updates account details for the specified Amazon Chime account. Currently, only account name updates are supported for this action.

        :param self:
        :param Name:  The new name for the specified Amazon Chime account.<br/>AccountId:  The Amazon Chime account ID.<br/>
        :return: Account:  The updated Amazon Chime account details.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedLicenses</i> Supported licenses for the Amazon Chime account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultLicense</i> The default license for the Amazon Chime account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The Amazon Chime account creation timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountType</i> The Amazon Chime account type. For more information about different account types, see <a href="https://docs.aws.amazon.com/chime/latest/ag/manage-chime-account.html">Managing Your Amazon Chime Accounts</a> in the <i>Amazon Chime Administration Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The Amazon Chime account name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountId</i> The Amazon Chime account ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AwsAccountId</i> The AWS account ID.<br/>        """
        pass

    def update_account_settings(self, AccountSettings: AccountSettings, AccountId: str) -> UpdateAccountSettingsResponse:
        """Updates the settings for the specified Amazon Chime account. You can update settings for remote control of shared screens, or for the dial-out option. For more information about these settings, see <a href="https://docs.aws.amazon.com/chime/latest/ag/policies.html">Use the Policies Page</a> in the <i>Amazon Chime Administration Guide</i>.

        :param self:
        :param AccountSettings:  The Amazon Chime account settings to update.:<br/>AccountId:  The Amazon Chime account ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnableDialOut</i> Setting that allows meeting participants to choose the <b>Call me at a phone number</b> option. For more information, see <a href="https://docs.aws.amazon.com/chime/latest/ug/chime-join-meeting.html">Join a Meeting without the Amazon Chime App</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisableRemoteControl</i> Setting that stops or starts remote control of shared screens during meetings.<br/>
        :return:         """
        pass

    def update_bot(self, BotId: str, AccountId: str, Disabled: NullableBoolean=None) -> UpdateBotResponse:
        """Updates the status of the specified bot, such as starting or stopping the bot from running in your Amazon Chime Enterprise account.

        :param self:
        :param Disabled:  When true, stops the specified bot from running in your account.<br/>BotId:  The bot ID.<br/>AccountId:  The Amazon Chime account ID.<br/>
        :return: Bot:  The updated bot details.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityToken</i> The security token used to authenticate Amazon Chime with the outgoing event endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BotEmail</i> The bot email address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdatedTimestamp</i> The updated bot timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The bot creation timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Disabled</i> When true, the bot is stopped from running in your account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BotType</i> The bot type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayName</i> The bot display name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserId</i> The unique ID for the bot user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BotId</i> The bot ID.<br/>        """
        pass

    def update_global_settings(self, VoiceConnector: VoiceConnectorSettings, BusinessCalling: BusinessCallingSettings):
        """Updates global settings for the administrator's AWS account, such as Amazon Chime Business Calling and Amazon Chime Voice Connector settings.

        :param self:
        :param VoiceConnector:  The Amazon Chime Voice Connector settings.:<br/>BusinessCalling:  The Amazon Chime Business Calling settings.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CdrBucket</i> The Amazon S3 bucket designated for call detail record storage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CdrBucket</i> The Amazon S3 bucket designated for call detail record storage.<br/>
        :return:         """
        pass

    def update_phone_number(self, PhoneNumberId: str, ProductType: str=None) -> UpdatePhoneNumberResponse:
        """Updates phone number details, such as product type, for the specified phone number ID. For toll-free numbers, you can use only the Amazon Chime Voice Connector product type.

        :param self:
        :param ProductType:  The product type.<br/>PhoneNumberId:  The phone number ID.<br/>
        :return: PhoneNumber:  The updated phone number details.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionTimestamp</i> The deleted phone number timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdatedTimestamp</i> The updated phone number timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The phone number creation timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Associations</i> The phone number associations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Capabilities</i> The phone number capabilities.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Capabilities</b>: The phone number capabilities.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The phone number status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductType</i> The phone number product type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The phone number type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>E164PhoneNumber</i> The phone number, in E.164 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneNumberId</i> The phone number ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OutboundMMS</i> Allows or denies outbound MMS messaging for the specified phone number.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InboundMMS</i> Allows or denies inbound MMS messaging for the specified phone number.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OutboundSMS</i> Allows or denies outbound SMS messaging for the specified phone number.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InboundSMS</i> Allows or denies inbound SMS messaging for the specified phone number.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OutboundCall</i> Allows or denies outbound calling for the specified phone number.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InboundCall</i> Allows or denies inbound calling for the specified phone number.<br/>        """
        pass

    def update_user(self, UserId: str, AccountId: str, LicenseType: str=None) -> UpdateUserResponse:
        """Updates user details for a specified user ID. Currently, only <code>LicenseType</code> updates are supported for this action.

        :param self:
        :param LicenseType:  The user license type to update. This must be a supported license type for the Amazon Chime account that the user belongs to.<br/>UserId:  The user ID.<br/>AccountId:  The Amazon Chime account ID.<br/>
        :return: User:  The updated user details.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PersonalPIN</i> The user's personal meeting PIN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InvitedOn</i> Date and time when the user is invited to the Amazon Chime account, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RegisteredOn</i> Date and time when the user is registered, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserInvitationStatus</i> The user invite status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserRegistrationStatus</i> The user registration status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseType</i> The license type for the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayName</i> The display name of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrimaryProvisionedNumber</i> The primary phone number associated with the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrimaryEmail</i> The primary email address of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountId</i> The Amazon Chime account ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserId</i> The user ID.<br/>        """
        pass

    def update_user_settings(self, UserSettings: UserSettings, UserId: str, AccountId: str):
        """Updates the settings for the specified user, such as phone number settings.

        :param self:
        :param UserSettings:  The user settings to update.:<br/>UserId:  The user ID.<br/>AccountId:  The Amazon Chime account ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Telephony</i> The telephony settings associated with the user.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Telephony</b>: The telephony settings associated with the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SMS</i> Allows or denies SMS messaging.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OutboundCalling</i> Allows or denies outbound calling.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InboundCalling</i> Allows or denies inbound calling.<br/>
        :return:         """
        pass

    def update_voice_connector(self, RequireEncryption: Boolean, Name: str, VoiceConnectorId: str) -> UpdateVoiceConnectorResponse:
        """Updates details for the specified Amazon Chime Voice Connector.

        :param self:
        :param RequireEncryption:  When enabled, requires encryption for the Amazon Chime Voice Connector.<br/>Name:  The name of the Amazon Chime Voice Connector.<br/>VoiceConnectorId:  The Amazon Chime Voice Connector ID.<br/>
        :return: VoiceConnector:  The Amazon Chime Voice Connector details.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdatedTimestamp</i> The updated Amazon Chime Voice Connector timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The Amazon Chime Voice Connector creation timestamp, in ISO 8601 format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequireEncryption</i> Designates whether encryption is required for the Amazon Chime Voice Connector.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutboundHostName</i> The outbound host name for the Amazon Chime Voice Connector.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the Amazon Chime Voice Connector.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VoiceConnectorId</i> The Amazon Chime Voice Connector ID.<br/>        """
        pass



class Cloud9(BaseClient):
    """<fullname>AWS Cloud9</fullname> AWS Cloud9 is a collection of tools that you can use to code, build, run, test, debug, and release software in the cloud. For more information about AWS Cloud9, see the <a href="https://docs.aws.amazon.com/cloud9/latest/user-guide">AWS Cloud9 User Guide</a>. AWS Cloud9 supports these operations: <ul> <li>  <code>CreateEnvironmentEC2</code>: Creates an AWS Cloud9 development environment, launches an Amazon EC2 instance, and then connects from the instance to the environment. </li> <li>  <code>CreateEnvironmentMembership</code>: Adds an environment member to an environment. </li> <li>  <code>DeleteEnvironment</code>: Deletes an environment. If an Amazon EC2 instance is connected to the environment, also terminates the instance. </li> <li>  <code>DeleteEnvironmentMembership</code>: Deletes an environment member from an environment. </li> <li>  <code>DescribeEnvironmentMemberships</code>: Gets information about environment members for an environment. </li> <li>  <code>DescribeEnvironments</code>: Gets information about environments. </li> <li>  <code>DescribeEnvironmentStatus</code>: Gets status information for an environment. </li> <li>  <code>ListEnvironments</code>: Gets a list of environment identifiers. </li> <li>  <code>UpdateEnvironment</code>: Changes the settings of an existing environment. </li> <li>  <code>UpdateEnvironmentMembership</code>: Changes the settings of an existing environment member for an environment. </li> </ul>"""


    class DescribeEnvironmentMembershipsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_environment_memberships` for valid parameters"""
            pass
    class ListEnvironmentsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_environments` for valid parameters"""
            pass


    class CreateEnvironmentEC2Request(Mapping):
        pass
    
    class CreateEnvironmentEC2Result(Mapping):
        pass
    
    class CreateEnvironmentMembershipRequest(Mapping):
        pass
    
    class CreateEnvironmentMembershipResult(Mapping):
        pass
    
    class DeleteEnvironmentRequest(Mapping):
        pass
    
    class DeleteEnvironmentResult(Mapping):
        pass
    
    class DeleteEnvironmentMembershipRequest(Mapping):
        pass
    
    class DeleteEnvironmentMembershipResult(Mapping):
        pass
    
    class DescribeEnvironmentMembershipsRequest(Mapping):
        pass
    
    class DescribeEnvironmentMembershipsResult(Mapping):
        pass
    
    class Permissions(object):
        pass
    
    class DescribeEnvironmentStatusRequest(Mapping):
        pass
    
    class DescribeEnvironmentStatusResult(Mapping):
        pass
    
    class DescribeEnvironmentsRequest(Mapping):
        pass
    
    class DescribeEnvironmentsResult(Mapping):
        pass
    
    class EnvironmentId(object):
        pass
    
    class ListEnvironmentsRequest(Mapping):
        pass
    
    class ListEnvironmentsResult(Mapping):
        pass
    
    class UpdateEnvironmentRequest(Mapping):
        pass
    
    class UpdateEnvironmentResult(Mapping):
        pass
    
    class UpdateEnvironmentMembershipRequest(Mapping):
        pass
    
    class UpdateEnvironmentMembershipResult(Mapping):
        pass
    
    def create_environment_ec2(self, instanceType: str, name: str, description: str=None, clientRequestToken: str=None, subnetId: str=None, automaticStopTimeMinutes: int=None, ownerArn: str=None) -> CreateEnvironmentEC2Result:
        """Creates an AWS Cloud9 development environment, launches an Amazon Elastic Compute Cloud (Amazon EC2) instance, and then connects from the instance to the environment.

        :param self:
        :param ownerArn:  The Amazon Resource Name (ARN) of the environment owner. This ARN can be the ARN of any AWS IAM principal. If this value is not specified, the ARN defaults to this environment's creator.<br/>automaticStopTimeMinutes:  The number of minutes until the running instance is shut down after the environment has last been used.<br/>subnetId:  The ID of the subnet in Amazon VPC that AWS Cloud9 will use to communicate with the Amazon EC2 instance.<br/>instanceType:  The type of instance to connect to the environment (for example, <code>t2.micro</code>).<br/>clientRequestToken:  A unique, case-sensitive string that helps AWS Cloud9 to ensure this operation completes no more than one time. For more information, see <a href="http://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">Client Tokens</a> in the <i>Amazon EC2 API Reference</i>.<br/>description:  The description of the environment to create.<br/>name:  The name of the environment to create. This name is visible to other AWS IAM users in the same AWS account.<br/>
        :return: environmentId:  The ID of the environment that was created.<br/>        """
        pass

    def create_environment_membership(self, permissions: str, userArn: str, environmentId: str) -> CreateEnvironmentMembershipResult:
        """Adds an environment member to an AWS Cloud9 development environment.

        :param self:
        :param permissions:  The type of environment member permissions you want to associate with this environment member. Available values include: <ul> <li>  <code>read-only</code>: Has read-only access to the environment. </li> <li>  <code>read-write</code>: Has read-write access to the environment. </li> </ul><br/>userArn:  The Amazon Resource Name (ARN) of the environment member you want to add.<br/>environmentId:  The ID of the environment that contains the environment member you want to add.<br/>
        :return: membership:  Information about the environment member that was added.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastAccess</i> The time, expressed in epoch time format, when the environment member last opened the environment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>environmentId</i> The ID of the environment for the environment member.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>userArn</i> The Amazon Resource Name (ARN) of the environment member.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>userId</i> The user ID in AWS Identity and Access Management (AWS IAM) of the environment member.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>permissions</i> The type of environment member permissions associated with this environment member. Available values include: <ul> <li>  <code>owner</code>: Owns the environment. </li> <li>  <code>read-only</code>: Has read-only access to the environment. </li> <li>  <code>read-write</code>: Has read-write access to the environment. </li> </ul><br/>        """
        pass

    def delete_environment(self, environmentId: str) -> DeleteEnvironmentResult:
        """Deletes an AWS Cloud9 development environment. If an Amazon EC2 instance is connected to the environment, also terminates the instance.

        :param self:
        :param environmentId:  The ID of the environment to delete.<br/>
        :return:         """
        pass

    def delete_environment_membership(self, userArn: str, environmentId: str) -> DeleteEnvironmentMembershipResult:
        """Deletes an environment member from an AWS Cloud9 development environment.

        :param self:
        :param userArn:  The Amazon Resource Name (ARN) of the environment member to delete from the environment.<br/>environmentId:  The ID of the environment to delete the environment member from.<br/>
        :return:         """
        pass

    def describe_environment_memberships(self, userArn: str=None, environmentId: str=None, permissions: List[Permissions]=None, nextToken: str=None, maxResults: int=None) -> DescribeEnvironmentMembershipsResult:
        """Gets information about environment members for an AWS Cloud9 development environment.

        :param self:
        :param maxResults:  The maximum number of environment members to get information about.<br/>nextToken:  During a previous call, if there are more than 25 items in the list, only the first 25 items are returned, along with a unique string called a <i>next token</i>. To get the next batch of items in the list, call this operation again, adding the next token to the call. To get all of the items in the list, keep calling this operation with each subsequent next token that is returned, until no more next tokens are returned.<br/>permissions:  The type of environment member permissions to get information about. Available values include: <ul> <li>  <code>owner</code>: Owns the environment. </li> <li>  <code>read-only</code>: Has read-only access to the environment. </li> <li>  <code>read-write</code>: Has read-write access to the environment. </li> </ul> If no value is specified, information about all environment members are returned.<br/>environmentId:  The ID of the environment to get environment member information about.<br/>userArn:  The Amazon Resource Name (ARN) of an individual environment member to get information about. If no value is specified, information about all environment members are returned.<br/>
        :return: nextToken:  If there are more than 25 items in the list, only the first 25 items are returned, along with a unique string called a <i>next token</i>. To get the next batch of items in the list, call this operation again, adding the next token to the call.<br/>memberships:  Information about the environment members for the environment.<br/>        """
        pass

    def describe_environment_status(self, environmentId: str) -> DescribeEnvironmentStatusResult:
        """Gets status information for an AWS Cloud9 development environment.

        :param self:
        :param environmentId:  The ID of the environment to get status information about.<br/>
        :return: message:  Any informational message about the status of the environment.<br/>status:  The status of the environment. Available values include: <ul> <li>  <code>connecting</code>: The environment is connecting. </li> <li>  <code>creating</code>: The environment is being created. </li> <li>  <code>deleting</code>: The environment is being deleted. </li> <li>  <code>error</code>: The environment is in an error state. </li> <li>  <code>ready</code>: The environment is ready. </li> <li>  <code>stopped</code>: The environment is stopped. </li> <li>  <code>stopping</code>: The environment is stopping. </li> </ul><br/>        """
        pass

    def describe_environments(self, environmentIds: List[EnvironmentId]) -> DescribeEnvironmentsResult:
        """Gets information about AWS Cloud9 development environments.

        :param self:
        :param environmentIds:  The IDs of individual environments to get information about.<br/>
        :return: environments:  Information about the environments that are returned.<br/>        """
        pass

    def list_environments(self, nextToken: str=None, maxResults: int=None) -> ListEnvironmentsResult:
        """Gets a list of AWS Cloud9 development environment identifiers.

        :param self:
        :param maxResults:  The maximum number of environments to get identifiers for.<br/>nextToken:  During a previous call, if there are more than 25 items in the list, only the first 25 items are returned, along with a unique string called a <i>next token</i>. To get the next batch of items in the list, call this operation again, adding the next token to the call. To get all of the items in the list, keep calling this operation with each subsequent next token that is returned, until no more next tokens are returned.<br/>
        :return: environmentIds:  The list of environment identifiers.<br/>nextToken:  If there are more than 25 items in the list, only the first 25 items are returned, along with a unique string called a <i>next token</i>. To get the next batch of items in the list, call this operation again, adding the next token to the call.<br/>        """
        pass

    def update_environment(self, environmentId: str, name: str=None, description: str=None) -> UpdateEnvironmentResult:
        """Changes the settings of an existing AWS Cloud9 development environment.

        :param self:
        :param description:  Any new or replacement description for the environment.<br/>name:  A replacement name for the environment.<br/>environmentId:  The ID of the environment to change settings.<br/>
        :return:         """
        pass

    def update_environment_membership(self, permissions: str, userArn: str, environmentId: str) -> UpdateEnvironmentMembershipResult:
        """Changes the settings of an existing environment member for an AWS Cloud9 development environment.

        :param self:
        :param permissions:  The replacement type of environment member permissions you want to associate with this environment member. Available values include: <ul> <li>  <code>read-only</code>: Has read-only access to the environment. </li> <li>  <code>read-write</code>: Has read-write access to the environment. </li> </ul><br/>userArn:  The Amazon Resource Name (ARN) of the environment member whose settings you want to change.<br/>environmentId:  The ID of the environment for the environment member whose settings you want to change.<br/>
        :return: membership:  Information about the environment member whose settings were changed.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastAccess</i> The time, expressed in epoch time format, when the environment member last opened the environment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>environmentId</i> The ID of the environment for the environment member.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>userArn</i> The Amazon Resource Name (ARN) of the environment member.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>userId</i> The user ID in AWS Identity and Access Management (AWS IAM) of the environment member.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>permissions</i> The type of environment member permissions associated with this environment member. Available values include: <ul> <li>  <code>owner</code>: Owns the environment. </li> <li>  <code>read-only</code>: Has read-only access to the environment. </li> <li>  <code>read-write</code>: Has read-write access to the environment. </li> </ul><br/>        """
        pass



class CloudDirectory(BaseClient):
    """<fullname>Amazon Cloud Directory</fullname> Amazon Cloud Directory is a component of the AWS Directory Service that simplifies the development and management of cloud-scale web, mobile, and IoT applications. This guide describes the Cloud Directory operations that you can call programmatically and includes detailed information on data types and errors. For information about Cloud Directory features, see <a href="https://aws.amazon.com/directoryservice/">AWS Directory Service</a> and the <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/what_is_cloud_directory.html">Amazon Cloud Directory Developer Guide</a>."""


    class ListObjectParentPathsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_object_parent_paths` for valid parameters"""
            pass
    class ListFacetNamesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_facet_names` for valid parameters"""
            pass
    class ListPublishedSchemaArnsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_published_schema_arns` for valid parameters"""
            pass
    class ListDirectoriesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_directories` for valid parameters"""
            pass
    class ListDevelopmentSchemaArnsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_development_schema_arns` for valid parameters"""
            pass
    class ListTypedLinkFacetNamesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_typed_link_facet_names` for valid parameters"""
            pass
    class ListIndexPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_index` for valid parameters"""
            pass
    class ListFacetAttributesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_facet_attributes` for valid parameters"""
            pass
    class ListObjectPoliciesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_object_policies` for valid parameters"""
            pass
    class ListTagsForResourcePaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_tags_for_resource` for valid parameters"""
            pass
    class ListAttachedIndicesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_attached_indices` for valid parameters"""
            pass
    class LookupPolicyPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `lookup_policy` for valid parameters"""
            pass
    class ListPolicyAttachmentsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_policy_attachments` for valid parameters"""
            pass
    class ListObjectAttributesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_object_attributes` for valid parameters"""
            pass
    class ListAppliedSchemaArnsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_applied_schema_arns` for valid parameters"""
            pass
    class ListTypedLinkFacetAttributesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_typed_link_facet_attributes` for valid parameters"""
            pass
    class ListIncomingTypedLinksPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_incoming_typed_links` for valid parameters"""
            pass
    class ListManagedSchemaArnsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_managed_schema_arns` for valid parameters"""
            pass
    class ListOutgoingTypedLinksPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_outgoing_typed_links` for valid parameters"""
            pass


    class AddFacetToObjectRequest(Mapping):
        pass
    
    class AddFacetToObjectResponse(Mapping):
        pass
    
    class SchemaFacet(Mapping):
        pass
    
    class AttributeKeyAndValue(Mapping):
        pass
    
    class ObjectReference(Mapping):
        pass
    
    class ApplySchemaRequest(Mapping):
        pass
    
    class ApplySchemaResponse(Mapping):
        pass
    
    class AttachObjectRequest(Mapping):
        pass
    
    class AttachObjectResponse(Mapping):
        pass
    
    class AttachPolicyRequest(Mapping):
        pass
    
    class AttachPolicyResponse(Mapping):
        pass
    
    class AttachToIndexRequest(Mapping):
        pass
    
    class AttachToIndexResponse(Mapping):
        pass
    
    class AttachTypedLinkRequest(Mapping):
        pass
    
    class AttachTypedLinkResponse(Mapping):
        pass
    
    class TypedLinkSchemaAndFacetName(Mapping):
        pass
    
    class AttributeNameAndValue(Mapping):
        pass
    
    class BatchReadRequest(Mapping):
        pass
    
    class BatchReadResponse(Mapping):
        pass
    
    class BatchReadOperation(Mapping):
        pass
    
    class BatchWriteRequest(Mapping):
        pass
    
    class BatchWriteResponse(Mapping):
        pass
    
    class BatchWriteOperation(Mapping):
        pass
    
    class CreateDirectoryRequest(Mapping):
        pass
    
    class CreateDirectoryResponse(Mapping):
        pass
    
    class CreateFacetRequest(Mapping):
        pass
    
    class CreateFacetResponse(Mapping):
        pass
    
    class FacetAttribute(Mapping):
        pass
    
    class CreateIndexRequest(Mapping):
        pass
    
    class CreateIndexResponse(Mapping):
        pass
    
    class AttributeKey(Mapping):
        pass
    
    class Bool(object):
        pass
    
    class CreateObjectRequest(Mapping):
        pass
    
    class CreateObjectResponse(Mapping):
        pass
    
    class CreateSchemaRequest(Mapping):
        pass
    
    class CreateSchemaResponse(Mapping):
        pass
    
    class CreateTypedLinkFacetRequest(Mapping):
        pass
    
    class CreateTypedLinkFacetResponse(Mapping):
        pass
    
    class TypedLinkFacet(Mapping):
        pass
    
    class DeleteDirectoryRequest(Mapping):
        pass
    
    class DeleteDirectoryResponse(Mapping):
        pass
    
    class DeleteFacetRequest(Mapping):
        pass
    
    class DeleteFacetResponse(Mapping):
        pass
    
    class DeleteObjectRequest(Mapping):
        pass
    
    class DeleteObjectResponse(Mapping):
        pass
    
    class DeleteSchemaRequest(Mapping):
        pass
    
    class DeleteSchemaResponse(Mapping):
        pass
    
    class DeleteTypedLinkFacetRequest(Mapping):
        pass
    
    class DeleteTypedLinkFacetResponse(Mapping):
        pass
    
    class DetachFromIndexRequest(Mapping):
        pass
    
    class DetachFromIndexResponse(Mapping):
        pass
    
    class DetachObjectRequest(Mapping):
        pass
    
    class DetachObjectResponse(Mapping):
        pass
    
    class DetachPolicyRequest(Mapping):
        pass
    
    class DetachPolicyResponse(Mapping):
        pass
    
    class DetachTypedLinkRequest(Mapping):
        pass
    
    class TypedLinkSpecifier(Mapping):
        pass
    
    class DisableDirectoryRequest(Mapping):
        pass
    
    class DisableDirectoryResponse(Mapping):
        pass
    
    class EnableDirectoryRequest(Mapping):
        pass
    
    class EnableDirectoryResponse(Mapping):
        pass
    
    class GetAppliedSchemaVersionRequest(Mapping):
        pass
    
    class GetAppliedSchemaVersionResponse(Mapping):
        pass
    
    class GetDirectoryRequest(Mapping):
        pass
    
    class GetDirectoryResponse(Mapping):
        pass
    
    class GetFacetRequest(Mapping):
        pass
    
    class GetFacetResponse(Mapping):
        pass
    
    class GetLinkAttributesRequest(Mapping):
        pass
    
    class GetLinkAttributesResponse(Mapping):
        pass
    
    class AttributeName(object):
        pass
    
    class GetObjectAttributesRequest(Mapping):
        pass
    
    class GetObjectAttributesResponse(Mapping):
        pass
    
    class GetObjectInformationRequest(Mapping):
        pass
    
    class GetObjectInformationResponse(Mapping):
        pass
    
    class GetSchemaAsJsonRequest(Mapping):
        pass
    
    class GetSchemaAsJsonResponse(Mapping):
        pass
    
    class GetTypedLinkFacetInformationRequest(Mapping):
        pass
    
    class GetTypedLinkFacetInformationResponse(Mapping):
        pass
    
    class ListAppliedSchemaArnsRequest(Mapping):
        pass
    
    class ListAppliedSchemaArnsResponse(Mapping):
        pass
    
    class ListAttachedIndicesRequest(Mapping):
        pass
    
    class ListAttachedIndicesResponse(Mapping):
        pass
    
    class ListDevelopmentSchemaArnsRequest(Mapping):
        pass
    
    class ListDevelopmentSchemaArnsResponse(Mapping):
        pass
    
    class ListDirectoriesRequest(Mapping):
        pass
    
    class ListDirectoriesResponse(Mapping):
        pass
    
    class ListFacetAttributesRequest(Mapping):
        pass
    
    class ListFacetAttributesResponse(Mapping):
        pass
    
    class ListFacetNamesRequest(Mapping):
        pass
    
    class ListFacetNamesResponse(Mapping):
        pass
    
    class ListIncomingTypedLinksRequest(Mapping):
        pass
    
    class ListIncomingTypedLinksResponse(Mapping):
        pass
    
    class TypedLinkAttributeRange(Mapping):
        pass
    
    class ListIndexRequest(Mapping):
        pass
    
    class ListIndexResponse(Mapping):
        pass
    
    class ObjectAttributeRange(Mapping):
        pass
    
    class ListManagedSchemaArnsRequest(Mapping):
        pass
    
    class ListManagedSchemaArnsResponse(Mapping):
        pass
    
    class ListObjectAttributesRequest(Mapping):
        pass
    
    class ListObjectAttributesResponse(Mapping):
        pass
    
    class ListObjectChildrenRequest(Mapping):
        pass
    
    class ListObjectChildrenResponse(Mapping):
        pass
    
    class ListObjectParentPathsRequest(Mapping):
        pass
    
    class ListObjectParentPathsResponse(Mapping):
        pass
    
    class ListObjectParentsRequest(Mapping):
        pass
    
    class ListObjectParentsResponse(Mapping):
        pass
    
    class ListObjectPoliciesRequest(Mapping):
        pass
    
    class ListObjectPoliciesResponse(Mapping):
        pass
    
    class ListOutgoingTypedLinksRequest(Mapping):
        pass
    
    class ListOutgoingTypedLinksResponse(Mapping):
        pass
    
    class ListPolicyAttachmentsRequest(Mapping):
        pass
    
    class ListPolicyAttachmentsResponse(Mapping):
        pass
    
    class ListPublishedSchemaArnsRequest(Mapping):
        pass
    
    class ListPublishedSchemaArnsResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class ListTypedLinkFacetAttributesRequest(Mapping):
        pass
    
    class ListTypedLinkFacetAttributesResponse(Mapping):
        pass
    
    class ListTypedLinkFacetNamesRequest(Mapping):
        pass
    
    class ListTypedLinkFacetNamesResponse(Mapping):
        pass
    
    class LookupPolicyRequest(Mapping):
        pass
    
    class LookupPolicyResponse(Mapping):
        pass
    
    class PublishSchemaRequest(Mapping):
        pass
    
    class PublishSchemaResponse(Mapping):
        pass
    
    class PutSchemaFromJsonRequest(Mapping):
        pass
    
    class PutSchemaFromJsonResponse(Mapping):
        pass
    
    class RemoveFacetFromObjectRequest(Mapping):
        pass
    
    class RemoveFacetFromObjectResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateFacetRequest(Mapping):
        pass
    
    class UpdateFacetResponse(Mapping):
        pass
    
    class FacetAttributeUpdate(Mapping):
        pass
    
    class UpdateLinkAttributesRequest(Mapping):
        pass
    
    class UpdateLinkAttributesResponse(Mapping):
        pass
    
    class LinkAttributeUpdate(Mapping):
        pass
    
    class UpdateObjectAttributesRequest(Mapping):
        pass
    
    class UpdateObjectAttributesResponse(Mapping):
        pass
    
    class ObjectAttributeUpdate(Mapping):
        pass
    
    class UpdateSchemaRequest(Mapping):
        pass
    
    class UpdateSchemaResponse(Mapping):
        pass
    
    class UpdateTypedLinkFacetRequest(Mapping):
        pass
    
    class UpdateTypedLinkFacetResponse(Mapping):
        pass
    
    class TypedLinkFacetAttributeUpdate(Mapping):
        pass
    
    class UpgradeAppliedSchemaRequest(Mapping):
        pass
    
    class UpgradeAppliedSchemaResponse(Mapping):
        pass
    
    class UpgradePublishedSchemaRequest(Mapping):
        pass
    
    class UpgradePublishedSchemaResponse(Mapping):
        pass
    
    def add_facet_to_object(self, ObjectReference: ObjectReference, SchemaFacet: SchemaFacet, DirectoryArn: str, ObjectAttributeList: List[AttributeKeyAndValue]=None) -> AddFacetToObjectResponse:
        """Adds a new <a>Facet</a> to an object. An object can have more than one facet applied on it.

        :param self:
        :param ObjectReference:  A reference to the object you are adding the specified facet to.:<br/>ObjectAttributeList:  Attributes on the facet that you are adding to the object.<br/>SchemaFacet:  Identifiers for the facet that you are adding to the object. See <a>SchemaFacet</a> for details.:<br/>DirectoryArn:  The Amazon Resource Name (ARN) that is associated with the <a>Directory</a> where the object resides. For more information, see <a>arns</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FacetName</i> The name of the facet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SchemaArn</i> The ARN of the schema that contains the facet with no minor component. See <a>arns</a> and <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/schemas_inplaceschemaupgrade.html">In-Place Schema Upgrade</a> for a description of when to provide minor versions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>
        :return:         """
        pass

    def apply_schema(self, DirectoryArn: str, PublishedSchemaArn: str) -> ApplySchemaResponse:
        """Copies the input published schema, at the specified version, into the <a>Directory</a> with the same name and version as that of the published schema.

        :param self:
        :param DirectoryArn:  The Amazon Resource Name (ARN) that is associated with the <a>Directory</a> into which the schema is copied. For more information, see <a>arns</a>.<br/>PublishedSchemaArn:  Published schema Amazon Resource Name (ARN) that needs to be copied. For more information, see <a>arns</a>.<br/>
        :return: DirectoryArn:  The ARN that is associated with the <a>Directory</a>. For more information, see <a>arns</a>.<br/>AppliedSchemaArn:  The applied schema ARN that is associated with the copied schema in the <a>Directory</a>. You can use this ARN to describe the schema information applied on this directory. For more information, see <a>arns</a>.<br/>        """
        pass

    def attach_object(self, LinkName: str, ChildReference: ObjectReference, ParentReference: ObjectReference, DirectoryArn: str) -> AttachObjectResponse:
        """Attaches an existing object to another object. An object can be accessed in two ways: <ol> <li> Using the path </li> <li> Using <code>ObjectIdentifier</code>  </li> </ol>

        :param self:
        :param LinkName:  The link name with which the child object is attached to the parent.<br/>ChildReference:  The child object reference to be attached to the object.:<br/>ParentReference:  The parent object reference.:<br/>DirectoryArn:  Amazon Resource Name (ARN) that is associated with the <a>Directory</a> where both objects reside. For more information, see <a>arns</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>
        :return: AttachedObjectIdentifier:  The attached <code>ObjectIdentifier</code>, which is the child <code>ObjectIdentifier</code>.<br/>        """
        pass

    def attach_policy(self, ObjectReference: ObjectReference, PolicyReference: ObjectReference, DirectoryArn: str) -> AttachPolicyResponse:
        """Attaches a policy object to a regular object. An object can have a limited number of attached policies.

        :param self:
        :param ObjectReference:  The reference that identifies the object to which the policy will be attached.:<br/>PolicyReference:  The reference that is associated with the policy object.:<br/>DirectoryArn:  The Amazon Resource Name (ARN) that is associated with the <a>Directory</a> where both objects reside. For more information, see <a>arns</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>
        :return:         """
        pass

    def attach_to_index(self, TargetReference: ObjectReference, IndexReference: ObjectReference, DirectoryArn: str) -> AttachToIndexResponse:
        """Attaches the specified object to the specified index.

        :param self:
        :param TargetReference:  A reference to the object that you are attaching to the index.:<br/>IndexReference:  A reference to the index that you are attaching the object to.:<br/>DirectoryArn:  The Amazon Resource Name (ARN) of the directory where the object and index exist.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>
        :return: AttachedObjectIdentifier:  The <code>ObjectIdentifier</code> of the object that was attached to the index.<br/>        """
        pass

    def attach_typed_link(self, Attributes: List[AttributeNameAndValue], TypedLinkFacet: TypedLinkSchemaAndFacetName, TargetObjectReference: ObjectReference, SourceObjectReference: ObjectReference, DirectoryArn: str) -> AttachTypedLinkResponse:
        """Attaches a typed link to a specified source and target object. For more information, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_links.html#directory_objects_links_typedlink">Typed Links</a>.

        :param self:
        :param Attributes:  A set of attributes that are associated with the typed link.<br/>TypedLinkFacet:  Identifies the typed link facet that is associated with the typed link.:<br/>TargetObjectReference:  Identifies the target object that the typed link will attach to.:<br/>SourceObjectReference:  Identifies the source object that the typed link will attach to.:<br/>DirectoryArn:  The Amazon Resource Name (ARN) of the directory where you want to attach the typed link.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TypedLinkName</i> The unique name of the typed link facet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SchemaArn</i> The Amazon Resource Name (ARN) that is associated with the schema. For more information, see <a>arns</a>.<br/>
        :return: TypedLinkSpecifier:  Returns a typed link specifier as output.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IdentityAttributeValues</i> Identifies the attribute value to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetObjectReference</i> Identifies the target object that the typed link will attach to.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TargetObjectReference</b>: Identifies the target object that the typed link will attach to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceObjectReference</i> Identifies the source object that the typed link will attach to.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SourceObjectReference</b>: Identifies the source object that the typed link will attach to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TypedLinkFacet</i> Identifies the typed link facet that is associated with the typed link.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TypedLinkFacet</b>: Identifies the typed link facet that is associated with the typed link.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TypedLinkName</i> The unique name of the typed link facet.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SchemaArn</i> The Amazon Resource Name (ARN) that is associated with the schema. For more information, see <a>arns</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>        """
        pass

    def batch_read(self, Operations: List[BatchReadOperation], DirectoryArn: str, ConsistencyLevel: str=None) -> BatchReadResponse:
        """Performs all the read operations in a batch. 

        :param self:
        :param ConsistencyLevel:  Represents the manner and timing in which the successful write or update of an object is reflected in a subsequent read operation of that same object.<br/>Operations:  A list of operations that are part of the batch.<br/>DirectoryArn:  The Amazon Resource Name (ARN) that is associated with the <a>Directory</a>. For more information, see <a>arns</a>.<br/>
        :return: Responses:  A list of all the responses for each batch read.<br/>        """
        pass

    def batch_write(self, Operations: List[BatchWriteOperation], DirectoryArn: str) -> BatchWriteResponse:
        """Performs all the write operations in a batch. Either all the operations succeed or none.

        :param self:
        :param Operations:  A list of operations that are part of the batch.<br/>DirectoryArn:  The Amazon Resource Name (ARN) that is associated with the <a>Directory</a>. For more information, see <a>arns</a>.<br/>
        :return: Responses:  A list of all the responses for each batch write.<br/>        """
        pass

    def create_directory(self, SchemaArn: str, Name: str) -> CreateDirectoryResponse:
        """Creates a <a>Directory</a> by copying the published schema into the directory. A directory cannot be created without a schema. You can also quickly create a directory using a managed schema, called the <code>QuickStartSchema</code>. For more information, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/schemas_managed.html">Managed Schema</a> in the <i>Amazon Cloud Directory Developer Guide</i>.

        :param self:
        :param SchemaArn:  The Amazon Resource Name (ARN) of the published schema that will be copied into the data <a>Directory</a>. For more information, see <a>arns</a>.<br/>Name:  The name of the <a>Directory</a>. Should be unique per account, per region.<br/>
        :return: AppliedSchemaArn:  The ARN of the published schema in the <a>Directory</a>. Once a published schema is copied into the directory, it has its own ARN, which is referred to applied schema ARN. For more information, see <a>arns</a>.<br/>ObjectIdentifier:  The root object node of the created directory.<br/>Name:  The name of the <a>Directory</a>.<br/>DirectoryArn:  The ARN that is associated with the <a>Directory</a>. For more information, see <a>arns</a>.<br/>        """
        pass

    def create_facet(self, Name: str, SchemaArn: str, Attributes: List[FacetAttribute]=None, ObjectType: str=None, FacetStyle: str=None) -> CreateFacetResponse:
        """Creates a new <a>Facet</a> in a schema. Facet creation is allowed only in development or applied schemas.

        :param self:
        :param FacetStyle:  There are two different styles that you can define on any given facet, <code>Static</code> and <code>Dynamic</code>. For static facets, all attributes must be defined in the schema. For dynamic facets, attributes can be defined during data plane operations.<br/>ObjectType:  Specifies whether a given object created from this facet is of type node, leaf node, policy or index. <ul> <li> Node: Can have multiple children but one parent. </li> </ul> <ul> <li> Leaf node: Cannot have children but can have multiple parents. </li> </ul> <ul> <li> Policy: Allows you to store a policy document and policy type. For more information, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/key_concepts_directory.html#key_concepts_policies">Policies</a>. </li> </ul> <ul> <li> Index: Can be created with the Index API. </li> </ul><br/>Attributes:  The attributes that are associated with the <a>Facet</a>.<br/>Name:  The name of the <a>Facet</a>, which is unique for a given schema.<br/>SchemaArn:  The schema ARN in which the new <a>Facet</a> will be created. For more information, see <a>arns</a>.<br/>
        :return:         """
        pass

    def create_index(self, IsUnique: Bool, OrderedIndexedAttributeList: List[AttributeKey], DirectoryArn: str, ParentReference: ObjectReference=None, LinkName: str=None) -> CreateIndexResponse:
        """Creates an index object. See <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/indexing_search.html">Indexing and search</a> for more information.

        :param self:
        :param LinkName:  The name of the link between the parent object and the index object.<br/>ParentReference:  A reference to the parent object that contains the index object.:<br/>IsUnique:  Indicates whether the attribute that is being indexed has unique values or not.<br/>OrderedIndexedAttributeList:  Specifies the attributes that should be indexed on. Currently only a single attribute is supported.<br/>DirectoryArn:  The ARN of the directory where the index should be created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>
        :return: ObjectIdentifier:  The <code>ObjectIdentifier</code> of the index created by this operation.<br/>        """
        pass

    def create_object(self, SchemaFacets: List[SchemaFacet], DirectoryArn: str, ObjectAttributeList: List[AttributeKeyAndValue]=None, ParentReference: ObjectReference=None, LinkName: str=None) -> CreateObjectResponse:
        """Creates an object in a <a>Directory</a>. Additionally attaches the object to a parent, if a parent reference and <code>LinkName</code> is specified. An object is simply a collection of <a>Facet</a> attributes. You can also use this API call to create a policy object, if the facet from which you create the object is a policy facet. 

        :param self:
        :param LinkName:  The name of link that is used to attach this object to a parent.<br/>ParentReference:  If specified, the parent reference to which this object will be attached.:<br/>ObjectAttributeList:  The attribute map whose attribute ARN contains the key and attribute value as the map value.<br/>SchemaFacets:  A list of schema facets to be associated with the object. Do not provide minor version components. See <a>SchemaFacet</a> for details.<br/>DirectoryArn:  The Amazon Resource Name (ARN) that is associated with the <a>Directory</a> in which the object will be created. For more information, see <a>arns</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>
        :return: ObjectIdentifier:  The identifier that is associated with the object.<br/>        """
        pass

    def create_schema(self, Name: str) -> CreateSchemaResponse:
        """Creates a new schema in a development state. A schema can exist in three phases: <ul> <li>  <i>Development:</i> This is a mutable phase of the schema. All new schemas are in the development phase. Once the schema is finalized, it can be published. </li> <li>  <i>Published:</i> Published schemas are immutable and have a version associated with them. </li> <li>  <i>Applied:</i> Applied schemas are mutable in a way that allows you to add new schema facets. You can also add new, nonrequired attributes to existing schema facets. You can apply only published schemas to directories.  </li> </ul>

        :param self:
        :param Name:  The name that is associated with the schema. This is unique to each account and in each region.<br/>
        :return: SchemaArn:  The Amazon Resource Name (ARN) that is associated with the schema. For more information, see <a>arns</a>.<br/>        """
        pass

    def create_typed_link_facet(self, Facet: TypedLinkFacet, SchemaArn: str) -> CreateTypedLinkFacetResponse:
        """Creates a <a>TypedLinkFacet</a>. For more information, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_links.html#directory_objects_links_typedlink">Typed Links</a>.

        :param self:
        :param Facet:   <a>Facet</a> structure that is associated with the typed link facet.:<br/>SchemaArn:  The Amazon Resource Name (ARN) that is associated with the schema. For more information, see <a>arns</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IdentityAttributeOrder</i> The set of attributes that distinguish links made from this facet from each other, in the order of significance. Listing typed links can filter on the values of these attributes. See <a>ListOutgoingTypedLinks</a> and <a>ListIncomingTypedLinks</a> for details.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Attributes</i> A set of key-value pairs associated with the typed link. Typed link attributes are used when you have data values that are related to the link itself, and not to one of the two objects being linked. Identity attributes also serve to distinguish the link from others of the same type between the same objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The unique name of the typed link facet.<br/>
        :return:         """
        pass

    def delete_directory(self, DirectoryArn: str) -> DeleteDirectoryResponse:
        """Deletes a directory. Only disabled directories can be deleted. A deleted directory cannot be undone. Exercise extreme caution when deleting directories.

        :param self:
        :param DirectoryArn:  The ARN of the directory to delete.<br/>
        :return: DirectoryArn:  The ARN of the deleted directory.<br/>        """
        pass

    def delete_facet(self, Name: str, SchemaArn: str) -> DeleteFacetResponse:
        """Deletes a given <a>Facet</a>. All attributes and <a>Rule</a>s that are associated with the facet will be deleted. Only development schema facets are allowed deletion.

        :param self:
        :param Name:  The name of the facet to delete.<br/>SchemaArn:  The Amazon Resource Name (ARN) that is associated with the <a>Facet</a>. For more information, see <a>arns</a>.<br/>
        :return:         """
        pass

    def delete_object(self, ObjectReference: ObjectReference, DirectoryArn: str) -> DeleteObjectResponse:
        """Deletes an object and its associated attributes. Only objects with no children and no parents can be deleted. The maximum number of attributes that can be deleted during an object deletion is 30. For more information, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/limits.html">Amazon Cloud Directory Limits</a>.

        :param self:
        :param ObjectReference:  A reference that identifies the object.:<br/>DirectoryArn:  The Amazon Resource Name (ARN) that is associated with the <a>Directory</a> where the object resides. For more information, see <a>arns</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>
        :return:         """
        pass

    def delete_schema(self, SchemaArn: str) -> DeleteSchemaResponse:
        """Deletes a given schema. Schemas in a development and published state can only be deleted. 

        :param self:
        :param SchemaArn:  The Amazon Resource Name (ARN) of the development schema. For more information, see <a>arns</a>.<br/>
        :return: SchemaArn:  The input ARN that is returned as part of the response. For more information, see <a>arns</a>.<br/>        """
        pass

    def delete_typed_link_facet(self, Name: str, SchemaArn: str) -> DeleteTypedLinkFacetResponse:
        """Deletes a <a>TypedLinkFacet</a>. For more information, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_links.html#directory_objects_links_typedlink">Typed Links</a>.

        :param self:
        :param Name:  The unique name of the typed link facet.<br/>SchemaArn:  The Amazon Resource Name (ARN) that is associated with the schema. For more information, see <a>arns</a>.<br/>
        :return:         """
        pass

    def detach_from_index(self, TargetReference: ObjectReference, IndexReference: ObjectReference, DirectoryArn: str) -> DetachFromIndexResponse:
        """Detaches the specified object from the specified index.

        :param self:
        :param TargetReference:  A reference to the object being detached from the index.:<br/>IndexReference:  A reference to the index object.:<br/>DirectoryArn:  The Amazon Resource Name (ARN) of the directory the index and object exist in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>
        :return: DetachedObjectIdentifier:  The <code>ObjectIdentifier</code> of the object that was detached from the index.<br/>        """
        pass

    def detach_object(self, LinkName: str, ParentReference: ObjectReference, DirectoryArn: str) -> DetachObjectResponse:
        """Detaches a given object from the parent object. The object that is to be detached from the parent is specified by the link name.

        :param self:
        :param LinkName:  The link name associated with the object that needs to be detached.<br/>ParentReference:  The parent reference from which the object with the specified link name is detached.:<br/>DirectoryArn:  The Amazon Resource Name (ARN) that is associated with the <a>Directory</a> where objects reside. For more information, see <a>arns</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>
        :return: DetachedObjectIdentifier:  The <code>ObjectIdentifier</code> that was detached from the object.<br/>        """
        pass

    def detach_policy(self, ObjectReference: ObjectReference, PolicyReference: ObjectReference, DirectoryArn: str) -> DetachPolicyResponse:
        """Detaches a policy from an object.

        :param self:
        :param ObjectReference:  Reference that identifies the object whose policy object will be detached.:<br/>PolicyReference:  Reference that identifies the policy object.:<br/>DirectoryArn:  The Amazon Resource Name (ARN) that is associated with the <a>Directory</a> where both objects reside. For more information, see <a>arns</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>
        :return:         """
        pass

    def detach_typed_link(self, TypedLinkSpecifier: TypedLinkSpecifier, DirectoryArn: str):
        """Detaches a typed link from a specified source and target object. For more information, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_links.html#directory_objects_links_typedlink">Typed Links</a>.

        :param self:
        :param TypedLinkSpecifier:  Used to accept a typed link specifier as input.:<br/>DirectoryArn:  The Amazon Resource Name (ARN) of the directory where you want to detach the typed link.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IdentityAttributeValues</i> Identifies the attribute value to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetObjectReference</i> Identifies the target object that the typed link will attach to.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TargetObjectReference</b>: Identifies the target object that the typed link will attach to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceObjectReference</i> Identifies the source object that the typed link will attach to.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SourceObjectReference</b>: Identifies the source object that the typed link will attach to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TypedLinkFacet</i> Identifies the typed link facet that is associated with the typed link.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TypedLinkFacet</b>: Identifies the typed link facet that is associated with the typed link.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TypedLinkName</i> The unique name of the typed link facet.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SchemaArn</i> The Amazon Resource Name (ARN) that is associated with the schema. For more information, see <a>arns</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>
        :return:         """
        pass

    def disable_directory(self, DirectoryArn: str) -> DisableDirectoryResponse:
        """Disables the specified directory. Disabled directories cannot be read or written to. Only enabled directories can be disabled. Disabled directories may be reenabled.

        :param self:
        :param DirectoryArn:  The ARN of the directory to disable.<br/>
        :return: DirectoryArn:  The ARN of the directory that has been disabled.<br/>        """
        pass

    def enable_directory(self, DirectoryArn: str) -> EnableDirectoryResponse:
        """Enables the specified directory. Only disabled directories can be enabled. Once enabled, the directory can then be read and written to.

        :param self:
        :param DirectoryArn:  The ARN of the directory to enable.<br/>
        :return: DirectoryArn:  The ARN of the enabled directory.<br/>        """
        pass

    def get_applied_schema_version(self, SchemaArn: str) -> GetAppliedSchemaVersionResponse:
        """Returns current applied schema version ARN, including the minor version in use.

        :param self:
        :param SchemaArn:  The ARN of the applied schema.<br/>
        :return: AppliedSchemaArn:  Current applied schema ARN, including the minor version in use if one was provided.<br/>        """
        pass

    def get_directory(self, DirectoryArn: str) -> GetDirectoryResponse:
        """Retrieves metadata about a directory.

        :param self:
        :param DirectoryArn:  The ARN of the directory.<br/>
        :return: Directory:  Metadata about the directory.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDateTime</i> The date and time when the directory was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the directory. Can be either <code>Enabled</code>, <code>Disabled</code>, or <code>Deleted</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DirectoryArn</i> The Amazon Resource Name (ARN) that is associated with the directory. For more information, see <a>arns</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the directory.<br/>        """
        pass

    def get_facet(self, Name: str, SchemaArn: str) -> GetFacetResponse:
        """Gets details of the <a>Facet</a>, such as facet name, attributes, <a>Rule</a>s, or <code>ObjectType</code>. You can call this on all kinds of schema facets -- published, development, or applied.

        :param self:
        :param Name:  The name of the facet to retrieve.<br/>SchemaArn:  The Amazon Resource Name (ARN) that is associated with the <a>Facet</a>. For more information, see <a>arns</a>.<br/>
        :return: Facet:  The <a>Facet</a> structure that is associated with the facet.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FacetStyle</i> There are two different styles that you can define on any given facet, <code>Static</code> and <code>Dynamic</code>. For static facets, all attributes must be defined in the schema. For dynamic facets, attributes can be defined during data plane operations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ObjectType</i> The object type that is associated with the facet. See <a>CreateFacetRequest$ObjectType</a> for more details.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the <a>Facet</a>.<br/>        """
        pass

    def get_link_attributes(self, AttributeNames: List[AttributeName], TypedLinkSpecifier: TypedLinkSpecifier, DirectoryArn: str, ConsistencyLevel: str=None) -> GetLinkAttributesResponse:
        """Retrieves attributes that are associated with a typed link.

        :param self:
        :param ConsistencyLevel:  The consistency level at which to retrieve the attributes on a typed link.<br/>AttributeNames:  A list of attribute names whose values will be retrieved.<br/>TypedLinkSpecifier:  Allows a typed link specifier to be accepted as input.:<br/>DirectoryArn:  The Amazon Resource Name (ARN) that is associated with the Directory where the typed link resides. For more information, see <a>arns</a> or <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_links.html#directory_objects_links_typedlink">Typed Links</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IdentityAttributeValues</i> Identifies the attribute value to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetObjectReference</i> Identifies the target object that the typed link will attach to.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TargetObjectReference</b>: Identifies the target object that the typed link will attach to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceObjectReference</i> Identifies the source object that the typed link will attach to.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SourceObjectReference</b>: Identifies the source object that the typed link will attach to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TypedLinkFacet</i> Identifies the typed link facet that is associated with the typed link.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TypedLinkFacet</b>: Identifies the typed link facet that is associated with the typed link.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TypedLinkName</i> The unique name of the typed link facet.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SchemaArn</i> The Amazon Resource Name (ARN) that is associated with the schema. For more information, see <a>arns</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>
        :return: Attributes:  The attributes that are associated with the typed link.<br/>        """
        pass

    def get_object_attributes(self, AttributeNames: List[AttributeName], SchemaFacet: SchemaFacet, ObjectReference: ObjectReference, DirectoryArn: str, ConsistencyLevel: str=None) -> GetObjectAttributesResponse:
        """Retrieves attributes within a facet that are associated with an object.

        :param self:
        :param AttributeNames:  List of attribute names whose values will be retrieved.<br/>SchemaFacet:  Identifier for the facet whose attributes will be retrieved. See <a>SchemaFacet</a> for details.:<br/>ConsistencyLevel:  The consistency level at which to retrieve the attributes on an object.<br/>ObjectReference:  Reference that identifies the object whose attributes will be retrieved.:<br/>DirectoryArn:  The Amazon Resource Name (ARN) that is associated with the <a>Directory</a> where the object resides.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FacetName</i> The name of the facet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SchemaArn</i> The ARN of the schema that contains the facet with no minor component. See <a>arns</a> and <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/schemas_inplaceschemaupgrade.html">In-Place Schema Upgrade</a> for a description of when to provide minor versions.<br/>
        :return: Attributes:  The attributes that are associated with the object.<br/>        """
        pass

    def get_object_information(self, ObjectReference: ObjectReference, DirectoryArn: str, ConsistencyLevel: str=None) -> GetObjectInformationResponse:
        """Retrieves metadata about an object.

        :param self:
        :param ConsistencyLevel:  The consistency level at which to retrieve the object information.<br/>ObjectReference:  A reference to the object.:<br/>DirectoryArn:  The ARN of the directory being retrieved.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>
        :return: ObjectIdentifier:  The <code>ObjectIdentifier</code> of the specified object.<br/>SchemaFacets:  The facets attached to the specified object. Although the response does not include minor version information, the most recently applied minor version of each Facet is in effect. See <a>GetAppliedSchemaVersion</a> for details.<br/>        """
        pass

    def get_schema_as_json(self, SchemaArn: str) -> GetSchemaAsJsonResponse:
        """Retrieves a JSON representation of the schema. See <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/schemas_jsonformat.html#schemas_json">JSON Schema Format</a> for more information.

        :param self:
        :param SchemaArn:  The ARN of the schema to retrieve.<br/>
        :return: Document:  The JSON representation of the schema document.<br/>Name:  The name of the retrieved schema.<br/>        """
        pass

    def get_typed_link_facet_information(self, Name: str, SchemaArn: str) -> GetTypedLinkFacetInformationResponse:
        """Returns the identity attribute order for a specific <a>TypedLinkFacet</a>. For more information, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_links.html#directory_objects_links_typedlink">Typed Links</a>.

        :param self:
        :param Name:  The unique name of the typed link facet.<br/>SchemaArn:  The Amazon Resource Name (ARN) that is associated with the schema. For more information, see <a>arns</a>.<br/>
        :return: IdentityAttributeOrder:  The order of identity attributes for the facet, from most significant to least significant. The ability to filter typed links considers the order that the attributes are defined on the typed link facet. When providing ranges to typed link selection, any inexact ranges must be specified at the end. Any attributes that do not have a range specified are presumed to match the entire range. Filters are interpreted in the order of the attributes on the typed link facet, not the order in which they are supplied to any API calls. For more information about identity attributes, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_links.html#directory_objects_links_typedlink">Typed Links</a>.<br/>        """
        pass

    def list_applied_schema_arns(self, DirectoryArn: str, SchemaArn: str=None, NextToken: str=None, MaxResults: int=None) -> ListAppliedSchemaArnsResponse:
        """Lists schema major versions applied to a directory. If <code>SchemaArn</code> is provided, lists the minor version.

        :param self:
        :param MaxResults:  The maximum number of results to retrieve.<br/>NextToken:  The pagination token.<br/>SchemaArn:  The response for <code>ListAppliedSchemaArns</code> when this parameter is used will list all minor version ARNs for a major version.<br/>DirectoryArn:  The ARN of the directory you are listing.<br/>
        :return: NextToken:  The pagination token.<br/>SchemaArns:  The ARNs of schemas that are applied to the directory.<br/>        """
        pass

    def list_attached_indices(self, TargetReference: ObjectReference, DirectoryArn: str, NextToken: str=None, MaxResults: int=None, ConsistencyLevel: str=None) -> ListAttachedIndicesResponse:
        """Lists indices attached to the specified object.

        :param self:
        :param ConsistencyLevel:  The consistency level to use for this operation.<br/>MaxResults:  The maximum number of results to retrieve.<br/>NextToken:  The pagination token.<br/>TargetReference:  A reference to the object that has indices attached.:<br/>DirectoryArn:  The ARN of the directory.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>
        :return: NextToken:  The pagination token.<br/>IndexAttachments:  The indices attached to the specified object.<br/>        """
        pass

    def list_development_schema_arns(self, NextToken: str=None, MaxResults: int=None) -> ListDevelopmentSchemaArnsResponse:
        """Retrieves each Amazon Resource Name (ARN) of schemas in the development state.

        :param self:
        :param MaxResults:  The maximum number of results to retrieve.<br/>NextToken:  The pagination token.<br/>
        :return: NextToken:  The pagination token.<br/>SchemaArns:  The ARNs of retrieved development schemas.<br/>        """
        pass

    def list_directories(self, NextToken: str=None, MaxResults: int=None, state: str=None) -> ListDirectoriesResponse:
        """Lists directories created within an account.

        :param self:
        :param state:  The state of the directories in the list. Can be either Enabled, Disabled, or Deleted.<br/>MaxResults:  The maximum number of results to retrieve.<br/>NextToken:  The pagination token.<br/>
        :return: NextToken:  The pagination token.<br/>Directories:  Lists all directories that are associated with your account in pagination fashion.<br/>        """
        pass

    def list_facet_attributes(self, Name: str, SchemaArn: str, NextToken: str=None, MaxResults: int=None) -> ListFacetAttributesResponse:
        """Retrieves attributes attached to the facet.

        :param self:
        :param MaxResults:  The maximum number of results to retrieve.<br/>NextToken:  The pagination token.<br/>Name:  The name of the facet whose attributes will be retrieved.<br/>SchemaArn:  The ARN of the schema where the facet resides.<br/>
        :return: NextToken:  The pagination token.<br/>Attributes:  The attributes attached to the facet.<br/>        """
        pass

    def list_facet_names(self, SchemaArn: str, NextToken: str=None, MaxResults: int=None) -> ListFacetNamesResponse:
        """Retrieves the names of facets that exist in a schema.

        :param self:
        :param MaxResults:  The maximum number of results to retrieve.<br/>NextToken:  The pagination token.<br/>SchemaArn:  The Amazon Resource Name (ARN) to retrieve facet names from.<br/>
        :return: NextToken:  The pagination token.<br/>FacetNames:  The names of facets that exist within the schema.<br/>        """
        pass

    def list_incoming_typed_links(self, ObjectReference: ObjectReference, DirectoryArn: str, FilterAttributeRanges: List[TypedLinkAttributeRange]=None, FilterTypedLink: TypedLinkSchemaAndFacetName=None, NextToken: str=None, MaxResults: int=None, ConsistencyLevel: str=None) -> ListIncomingTypedLinksResponse:
        """Returns a paginated list of all the incoming <a>TypedLinkSpecifier</a> information for an object. It also supports filtering by typed link facet and identity attributes. For more information, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_links.html#directory_objects_links_typedlink">Typed Links</a>.

        :param self:
        :param ConsistencyLevel:  The consistency level to execute the request at.<br/>MaxResults:  The maximum number of results to retrieve.<br/>NextToken:  The pagination token.<br/>FilterTypedLink:  Filters are interpreted in the order of the attributes on the typed link facet, not the order in which they are supplied to any API calls.:<br/>FilterAttributeRanges:  Provides range filters for multiple attributes. When providing ranges to typed link selection, any inexact ranges must be specified at the end. Any attributes that do not have a range specified are presumed to match the entire range.<br/>ObjectReference:  Reference that identifies the object whose attributes will be listed.:<br/>DirectoryArn:  The Amazon Resource Name (ARN) of the directory where you want to list the typed links.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TypedLinkName</i> The unique name of the typed link facet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SchemaArn</i> The Amazon Resource Name (ARN) that is associated with the schema. For more information, see <a>arns</a>.<br/>
        :return: NextToken:  The pagination token.<br/>LinkSpecifiers:  Returns one or more typed link specifiers as output.<br/>        """
        pass

    def list_index(self, IndexReference: ObjectReference, DirectoryArn: str, RangesOnIndexedValues: List[ObjectAttributeRange]=None, MaxResults: int=None, NextToken: str=None, ConsistencyLevel: str=None) -> ListIndexResponse:
        """Lists objects attached to the specified index.

        :param self:
        :param ConsistencyLevel:  The consistency level to execute the request at.<br/>NextToken:  The pagination token.<br/>MaxResults:  The maximum number of objects in a single page to retrieve from the index during a request. For more information, see <a href="http://docs.aws.amazon.com/clouddirectory/latest/developerguide/limits.html">Amazon Cloud Directory Limits</a>.<br/>IndexReference:  The reference to the index to list.:<br/>RangesOnIndexedValues:  Specifies the ranges of indexed values that you want to query.<br/>DirectoryArn:  The ARN of the directory that the index exists in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>
        :return: NextToken:  The pagination token.<br/>IndexAttachments:  The objects and indexed values attached to the index.<br/>        """
        pass

    def list_managed_schema_arns(self, SchemaArn: str=None, NextToken: str=None, MaxResults: int=None) -> ListManagedSchemaArnsResponse:
        """Lists the major version families of each managed schema. If a major version ARN is provided as SchemaArn, the minor version revisions in that family are listed instead.

        :param self:
        :param MaxResults:  The maximum number of results to retrieve.<br/>NextToken:  The pagination token.<br/>SchemaArn:  The response for ListManagedSchemaArns. When this parameter is used, all minor version ARNs for a major version are listed.<br/>
        :return: NextToken:  The pagination token.<br/>SchemaArns:  The ARNs for all AWS managed schemas.<br/>        """
        pass

    def list_object_attributes(self, ObjectReference: ObjectReference, DirectoryArn: str, NextToken: str=None, MaxResults: int=None, ConsistencyLevel: str=None, FacetFilter: SchemaFacet=None) -> ListObjectAttributesResponse:
        """Lists all attributes that are associated with an object. 

        :param self:
        :param FacetFilter:  Used to filter the list of object attributes that are associated with a certain facet.:<br/>ConsistencyLevel:  Represents the manner and timing in which the successful write or update of an object is reflected in a subsequent read operation of that same object.<br/>MaxResults:  The maximum number of items to be retrieved in a single call. This is an approximate number.<br/>NextToken:  The pagination token.<br/>ObjectReference:  The reference that identifies the object whose attributes will be listed.:<br/>DirectoryArn:  The Amazon Resource Name (ARN) that is associated with the <a>Directory</a> where the object resides. For more information, see <a>arns</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FacetName</i> The name of the facet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SchemaArn</i> The ARN of the schema that contains the facet with no minor component. See <a>arns</a> and <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/schemas_inplaceschemaupgrade.html">In-Place Schema Upgrade</a> for a description of when to provide minor versions.<br/>
        :return: NextToken:  The pagination token.<br/>Attributes:  Attributes map that is associated with the object. <code>AttributeArn</code> is the key, and attribute value is the value.<br/>        """
        pass

    def list_object_children(self, ObjectReference: ObjectReference, DirectoryArn: str, NextToken: str=None, MaxResults: int=None, ConsistencyLevel: str=None) -> ListObjectChildrenResponse:
        """Returns a paginated list of child objects that are associated with a given object.

        :param self:
        :param ConsistencyLevel:  Represents the manner and timing in which the successful write or update of an object is reflected in a subsequent read operation of that same object.<br/>MaxResults:  The maximum number of items to be retrieved in a single call. This is an approximate number.<br/>NextToken:  The pagination token.<br/>ObjectReference:  The reference that identifies the object for which child objects are being listed.:<br/>DirectoryArn:  The Amazon Resource Name (ARN) that is associated with the <a>Directory</a> where the object resides. For more information, see <a>arns</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>
        :return: NextToken:  The pagination token.<br/>Children:  Children structure, which is a map with key as the <code>LinkName</code> and <code>ObjectIdentifier</code> as the value.<br/>        """
        pass

    def list_object_parent_paths(self, ObjectReference: ObjectReference, DirectoryArn: str, NextToken: str=None, MaxResults: int=None) -> ListObjectParentPathsResponse:
        """Retrieves all available parent paths for any object type such as node, leaf node, policy node, and index node objects. For more information about objects, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/key_concepts_directorystructure.html">Directory Structure</a>. Use this API to evaluate all parents for an object. The call returns all objects from the root of the directory up to the requested object. The API returns the number of paths based on user-defined <code>MaxResults</code>, in case there are multiple paths to the parent. The order of the paths and nodes returned is consistent among multiple API calls unless the objects are deleted or moved. Paths not leading to the directory root are ignored from the target object.

        :param self:
        :param MaxResults:  The maximum number of items to be retrieved in a single call. This is an approximate number.<br/>NextToken:  The pagination token.<br/>ObjectReference:  The reference that identifies the object whose parent paths are listed.:<br/>DirectoryArn:  The ARN of the directory to which the parent path applies.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>
        :return: NextToken:  The pagination token.<br/>PathToObjectIdentifiersList:  Returns the path to the <code>ObjectIdentifiers</code> that are associated with the directory.<br/>        """
        pass

    def list_object_parents(self, ObjectReference: ObjectReference, DirectoryArn: str, NextToken: str=None, MaxResults: int=None, ConsistencyLevel: str=None, IncludeAllLinksToEachParent: Bool=None) -> ListObjectParentsResponse:
        """Lists parent objects that are associated with a given object in pagination fashion.

        :param self:
        :param IncludeAllLinksToEachParent:  When set to True, returns all <a>ListObjectParentsResponse$ParentLinks</a>. There could be multiple links between a parent-child pair.<br/>ConsistencyLevel:  Represents the manner and timing in which the successful write or update of an object is reflected in a subsequent read operation of that same object.<br/>MaxResults:  The maximum number of items to be retrieved in a single call. This is an approximate number.<br/>NextToken:  The pagination token.<br/>ObjectReference:  The reference that identifies the object for which parent objects are being listed.:<br/>DirectoryArn:  The Amazon Resource Name (ARN) that is associated with the <a>Directory</a> where the object resides. For more information, see <a>arns</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>
        :return: ParentLinks:  Returns a list of parent reference and LinkName Tuples.<br/>NextToken:  The pagination token.<br/>Parents:  The parent structure, which is a map with key as the <code>ObjectIdentifier</code> and LinkName as the value.<br/>        """
        pass

    def list_object_policies(self, ObjectReference: ObjectReference, DirectoryArn: str, NextToken: str=None, MaxResults: int=None, ConsistencyLevel: str=None) -> ListObjectPoliciesResponse:
        """Returns policies attached to an object in pagination fashion.

        :param self:
        :param ConsistencyLevel:  Represents the manner and timing in which the successful write or update of an object is reflected in a subsequent read operation of that same object.<br/>MaxResults:  The maximum number of items to be retrieved in a single call. This is an approximate number.<br/>NextToken:  The pagination token.<br/>ObjectReference:  Reference that identifies the object for which policies will be listed.:<br/>DirectoryArn:  The Amazon Resource Name (ARN) that is associated with the <a>Directory</a> where objects reside. For more information, see <a>arns</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>
        :return: NextToken:  The pagination token.<br/>AttachedPolicyIds:  A list of policy <code>ObjectIdentifiers</code>, that are attached to the object.<br/>        """
        pass

    def list_outgoing_typed_links(self, ObjectReference: ObjectReference, DirectoryArn: str, FilterAttributeRanges: List[TypedLinkAttributeRange]=None, FilterTypedLink: TypedLinkSchemaAndFacetName=None, NextToken: str=None, MaxResults: int=None, ConsistencyLevel: str=None) -> ListOutgoingTypedLinksResponse:
        """Returns a paginated list of all the outgoing <a>TypedLinkSpecifier</a> information for an object. It also supports filtering by typed link facet and identity attributes. For more information, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_links.html#directory_objects_links_typedlink">Typed Links</a>.

        :param self:
        :param ConsistencyLevel:  The consistency level to execute the request at.<br/>MaxResults:  The maximum number of results to retrieve.<br/>NextToken:  The pagination token.<br/>FilterTypedLink:  Filters are interpreted in the order of the attributes defined on the typed link facet, not the order they are supplied to any API calls.:<br/>FilterAttributeRanges:  Provides range filters for multiple attributes. When providing ranges to typed link selection, any inexact ranges must be specified at the end. Any attributes that do not have a range specified are presumed to match the entire range.<br/>ObjectReference:  A reference that identifies the object whose attributes will be listed.:<br/>DirectoryArn:  The Amazon Resource Name (ARN) of the directory where you want to list the typed links.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TypedLinkName</i> The unique name of the typed link facet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SchemaArn</i> The Amazon Resource Name (ARN) that is associated with the schema. For more information, see <a>arns</a>.<br/>
        :return: NextToken:  The pagination token.<br/>TypedLinkSpecifiers:  Returns a typed link specifier as output.<br/>        """
        pass

    def list_policy_attachments(self, PolicyReference: ObjectReference, DirectoryArn: str, NextToken: str=None, MaxResults: int=None, ConsistencyLevel: str=None) -> ListPolicyAttachmentsResponse:
        """Returns all of the <code>ObjectIdentifiers</code> to which a given policy is attached.

        :param self:
        :param ConsistencyLevel:  Represents the manner and timing in which the successful write or update of an object is reflected in a subsequent read operation of that same object.<br/>MaxResults:  The maximum number of items to be retrieved in a single call. This is an approximate number.<br/>NextToken:  The pagination token.<br/>PolicyReference:  The reference that identifies the policy object.:<br/>DirectoryArn:  The Amazon Resource Name (ARN) that is associated with the <a>Directory</a> where objects reside. For more information, see <a>arns</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>
        :return: NextToken:  The pagination token.<br/>ObjectIdentifiers:  A list of <code>ObjectIdentifiers</code> to which the policy is attached.<br/>        """
        pass

    def list_published_schema_arns(self, SchemaArn: str=None, NextToken: str=None, MaxResults: int=None) -> ListPublishedSchemaArnsResponse:
        """Lists the major version families of each published schema. If a major version ARN is provided as <code>SchemaArn</code>, the minor version revisions in that family are listed instead.

        :param self:
        :param MaxResults:  The maximum number of results to retrieve.<br/>NextToken:  The pagination token.<br/>SchemaArn:  The response for <code>ListPublishedSchemaArns</code> when this parameter is used will list all minor version ARNs for a major version.<br/>
        :return: NextToken:  The pagination token.<br/>SchemaArns:  The ARNs of published schemas.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceArn: str, NextToken: str=None, MaxResults: int=None) -> ListTagsForResourceResponse:
        """Returns tags for a resource. Tagging is currently supported only for directories with a limit of 50 tags per directory. All 50 tags are returned for a given directory with this API call.

        :param self:
        :param MaxResults:  The <code>MaxResults</code> parameter sets the maximum number of results returned in a single page. This is for future use and is not supported currently.<br/>NextToken:  The pagination token. This is for future use. Currently pagination is not supported for tagging.<br/>ResourceArn:  The Amazon Resource Name (ARN) of the resource. Tagging is only supported for directories.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is null when there are no more results to return.<br/>Tags:  A list of tag key value pairs that are associated with the response.<br/>        """
        pass

    def list_typed_link_facet_attributes(self, Name: str, SchemaArn: str, NextToken: str=None, MaxResults: int=None) -> ListTypedLinkFacetAttributesResponse:
        """Returns a paginated list of all attribute definitions for a particular <a>TypedLinkFacet</a>. For more information, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_links.html#directory_objects_links_typedlink">Typed Links</a>.

        :param self:
        :param MaxResults:  The maximum number of results to retrieve.<br/>NextToken:  The pagination token.<br/>Name:  The unique name of the typed link facet.<br/>SchemaArn:  The Amazon Resource Name (ARN) that is associated with the schema. For more information, see <a>arns</a>.<br/>
        :return: NextToken:  The pagination token.<br/>Attributes:  An ordered set of attributes associate with the typed link.<br/>        """
        pass

    def list_typed_link_facet_names(self, SchemaArn: str, NextToken: str=None, MaxResults: int=None) -> ListTypedLinkFacetNamesResponse:
        """Returns a paginated list of <code>TypedLink</code> facet names for a particular schema. For more information, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_links.html#directory_objects_links_typedlink">Typed Links</a>.

        :param self:
        :param MaxResults:  The maximum number of results to retrieve.<br/>NextToken:  The pagination token.<br/>SchemaArn:  The Amazon Resource Name (ARN) that is associated with the schema. For more information, see <a>arns</a>.<br/>
        :return: NextToken:  The pagination token.<br/>FacetNames:  The names of typed link facets that exist within the schema.<br/>        """
        pass

    def lookup_policy(self, ObjectReference: ObjectReference, DirectoryArn: str, NextToken: str=None, MaxResults: int=None) -> LookupPolicyResponse:
        """Lists all policies from the root of the <a>Directory</a> to the object specified. If there are no policies present, an empty list is returned. If policies are present, and if some objects don't have the policies attached, it returns the <code>ObjectIdentifier</code> for such objects. If policies are present, it returns <code>ObjectIdentifier</code>, <code>policyId</code>, and <code>policyType</code>. Paths that don't lead to the root from the target object are ignored. For more information, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/key_concepts_directory.html#key_concepts_policies">Policies</a>.

        :param self:
        :param MaxResults:  The maximum number of items to be retrieved in a single call. This is an approximate number.<br/>NextToken:  The token to request the next page of results.<br/>ObjectReference:  Reference that identifies the object whose policies will be looked up.:<br/>DirectoryArn:  The Amazon Resource Name (ARN) that is associated with the <a>Directory</a>. For more information, see <a>arns</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>
        :return: NextToken:  The pagination token.<br/>PolicyToPathList:  Provides list of path to policies. Policies contain <code>PolicyId</code>, <code>ObjectIdentifier</code>, and <code>PolicyType</code>. For more information, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/key_concepts_directory.html#key_concepts_policies">Policies</a>.<br/>        """
        pass

    def publish_schema(self, Version: str, DevelopmentSchemaArn: str, MinorVersion: str=None, Name: str=None) -> PublishSchemaResponse:
        """Publishes a development schema with a major version and a recommended minor version.

        :param self:
        :param Name:  The new name under which the schema will be published. If this is not provided, the development schema is considered.<br/>MinorVersion:  The minor version under which the schema will be published. This parameter is recommended. Schemas have both a major and minor version associated with them.<br/>Version:  The major version under which the schema will be published. Schemas have both a major and minor version associated with them.<br/>DevelopmentSchemaArn:  The Amazon Resource Name (ARN) that is associated with the development schema. For more information, see <a>arns</a>.<br/>
        :return: PublishedSchemaArn:  The ARN that is associated with the published schema. For more information, see <a>arns</a>.<br/>        """
        pass

    def put_schema_from_json(self, Document: str, SchemaArn: str) -> PutSchemaFromJsonResponse:
        """Allows a schema to be updated using JSON upload. Only available for development schemas. See <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/schemas_jsonformat.html#schemas_json">JSON Schema Format</a> for more information.

        :param self:
        :param Document:  The replacement JSON schema.<br/>SchemaArn:  The ARN of the schema to update.<br/>
        :return: Arn:  The ARN of the schema to update.<br/>        """
        pass

    def remove_facet_from_object(self, ObjectReference: ObjectReference, SchemaFacet: SchemaFacet, DirectoryArn: str) -> RemoveFacetFromObjectResponse:
        """Removes the specified facet from the specified object.

        :param self:
        :param ObjectReference:  A reference to the object to remove the facet from.:<br/>SchemaFacet:  The facet to remove. See <a>SchemaFacet</a> for details.:<br/>DirectoryArn:  The ARN of the directory in which the object resides.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FacetName</i> The name of the facet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SchemaArn</i> The ARN of the schema that contains the facet with no minor component. See <a>arns</a> and <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/schemas_inplaceschemaupgrade.html">In-Place Schema Upgrade</a> for a description of when to provide minor versions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>
        :return:         """
        pass

    def tag_resource(self, Tags: List[Tag], ResourceArn: str) -> TagResourceResponse:
        """An API operation for adding tags to a resource.

        :param self:
        :param Tags:  A list of tag key-value pairs.<br/>ResourceArn:  The Amazon Resource Name (ARN) of the resource. Tagging is only supported for directories.<br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[TagKey], ResourceArn: str) -> UntagResourceResponse:
        """An API operation for removing tags from a resource.

        :param self:
        :param TagKeys:  Keys of the tag that need to be removed from the resource.<br/>ResourceArn:  The Amazon Resource Name (ARN) of the resource. Tagging is only supported for directories.<br/>
        :return:         """
        pass

    def update_facet(self, Name: str, SchemaArn: str, AttributeUpdates: List[FacetAttributeUpdate]=None, ObjectType: str=None) -> UpdateFacetResponse:
        """Does the following: <ol> <li> Adds new <code>Attributes</code>, <code>Rules</code>, or <code>ObjectTypes</code>. </li> <li> Updates existing <code>Attributes</code>, <code>Rules</code>, or <code>ObjectTypes</code>. </li> <li> Deletes existing <code>Attributes</code>, <code>Rules</code>, or <code>ObjectTypes</code>. </li> </ol>

        :param self:
        :param ObjectType:  The object type that is associated with the facet. See <a>CreateFacetRequest$ObjectType</a> for more details.<br/>AttributeUpdates:  List of attributes that need to be updated in a given schema <a>Facet</a>. Each attribute is followed by <code>AttributeAction</code>, which specifies the type of update operation to perform. <br/>Name:  The name of the facet.<br/>SchemaArn:  The Amazon Resource Name (ARN) that is associated with the <a>Facet</a>. For more information, see <a>arns</a>.<br/>
        :return:         """
        pass

    def update_link_attributes(self, AttributeUpdates: List[LinkAttributeUpdate], TypedLinkSpecifier: TypedLinkSpecifier, DirectoryArn: str) -> UpdateLinkAttributesResponse:
        """Updates a given typed link’s attributes. Attributes to be updated must not contribute to the typed link’s identity, as defined by its <code>IdentityAttributeOrder</code>.

        :param self:
        :param AttributeUpdates:  The attributes update structure.<br/>TypedLinkSpecifier:  Allows a typed link specifier to be accepted as input.:<br/>DirectoryArn:  The Amazon Resource Name (ARN) that is associated with the Directory where the updated typed link resides. For more information, see <a>arns</a> or <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_links.html#directory_objects_links_typedlink">Typed Links</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IdentityAttributeValues</i> Identifies the attribute value to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetObjectReference</i> Identifies the target object that the typed link will attach to.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TargetObjectReference</b>: Identifies the target object that the typed link will attach to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceObjectReference</i> Identifies the source object that the typed link will attach to.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SourceObjectReference</b>: Identifies the source object that the typed link will attach to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TypedLinkFacet</i> Identifies the typed link facet that is associated with the typed link.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TypedLinkFacet</b>: Identifies the typed link facet that is associated with the typed link.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TypedLinkName</i> The unique name of the typed link facet.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SchemaArn</i> The Amazon Resource Name (ARN) that is associated with the schema. For more information, see <a>arns</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>
        :return:         """
        pass

    def update_object_attributes(self, AttributeUpdates: List[ObjectAttributeUpdate], ObjectReference: ObjectReference, DirectoryArn: str) -> UpdateObjectAttributesResponse:
        """Updates a given object's attributes.

        :param self:
        :param AttributeUpdates:  The attributes update structure.<br/>ObjectReference:  The reference that identifies the object.:<br/>DirectoryArn:  The Amazon Resource Name (ARN) that is associated with the <a>Directory</a> where the object resides. For more information, see <a>arns</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> A path selector supports easy selection of an object by the parent/child links leading to it from the directory root. Use the link names from each parent/child link to construct the path. Path selectors start with a slash (/) and link names are separated by slashes. For more information about paths, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_access_objects.html">Access Objects</a>. You can identify an object in one of the following ways: <ul> <li>  <i>$ObjectIdentifier</i> - An object identifier is an opaque string provided by Amazon Cloud Directory. When creating objects, the system will provide you with the identifier of the created object. An object’s identifier is immutable and no two objects will ever share the same object identifier </li> <li>  <i>/some/path</i> - Identifies the object based on path </li> <li>  <i>#SomeBatchReference</i> - Identifies the object in a batch call </li> </ul><br/>
        :return: ObjectIdentifier:  The <code>ObjectIdentifier</code> of the updated object.<br/>        """
        pass

    def update_schema(self, Name: str, SchemaArn: str) -> UpdateSchemaResponse:
        """Updates the schema name with a new name. Only development schema names can be updated.

        :param self:
        :param Name:  The name of the schema.<br/>SchemaArn:  The Amazon Resource Name (ARN) of the development schema. For more information, see <a>arns</a>.<br/>
        :return: SchemaArn:  The ARN that is associated with the updated schema. For more information, see <a>arns</a>.<br/>        """
        pass

    def update_typed_link_facet(self, IdentityAttributeOrder: List[AttributeName], AttributeUpdates: List[TypedLinkFacetAttributeUpdate], Name: str, SchemaArn: str) -> UpdateTypedLinkFacetResponse:
        """Updates a <a>TypedLinkFacet</a>. For more information, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_links.html#directory_objects_links_typedlink">Typed Links</a>.

        :param self:
        :param IdentityAttributeOrder:  The order of identity attributes for the facet, from most significant to least significant. The ability to filter typed links considers the order that the attributes are defined on the typed link facet. When providing ranges to a typed link selection, any inexact ranges must be specified at the end. Any attributes that do not have a range specified are presumed to match the entire range. Filters are interpreted in the order of the attributes on the typed link facet, not the order in which they are supplied to any API calls. For more information about identity attributes, see <a href="https://docs.aws.amazon.com/clouddirectory/latest/developerguide/directory_objects_links.html#directory_objects_links_typedlink">Typed Links</a>.<br/>AttributeUpdates:  Attributes update structure.<br/>Name:  The unique name of the typed link facet.<br/>SchemaArn:  The Amazon Resource Name (ARN) that is associated with the schema. For more information, see <a>arns</a>.<br/>
        :return:         """
        pass

    def upgrade_applied_schema(self, DirectoryArn: str, PublishedSchemaArn: str, DryRun: Bool=None) -> UpgradeAppliedSchemaResponse:
        """Upgrades a single directory in-place using the <code>PublishedSchemaArn</code> with schema updates found in <code>MinorVersion</code>. Backwards-compatible minor version upgrades are instantaneously available for readers on all objects in the directory. Note: This is a synchronous API call and upgrades only one schema on a given directory per call. To upgrade multiple directories from one schema, you would need to call this API on each directory.

        :param self:
        :param DryRun:  Used for testing whether the major version schemas are backward compatible or not. If schema compatibility fails, an exception would be thrown else the call would succeed but no changes will be saved. This parameter is optional.<br/>DirectoryArn:  The ARN for the directory to which the upgraded schema will be applied.<br/>PublishedSchemaArn:  The revision of the published schema to upgrade the directory to.<br/>
        :return: DirectoryArn:  The ARN of the directory that is returned as part of the response.<br/>UpgradedSchemaArn:  The ARN of the upgraded schema that is returned as part of the response.<br/>        """
        pass

    def upgrade_published_schema(self, MinorVersion: str, PublishedSchemaArn: str, DevelopmentSchemaArn: str, DryRun: Bool=None) -> UpgradePublishedSchemaResponse:
        """Upgrades a published schema under a new minor version revision using the current contents of <code>DevelopmentSchemaArn</code>.

        :param self:
        :param DryRun:  Used for testing whether the Development schema provided is backwards compatible, or not, with the publish schema provided by the user to be upgraded. If schema compatibility fails, an exception would be thrown else the call would succeed. This parameter is optional and defaults to false.<br/>MinorVersion:  Identifies the minor version of the published schema that will be created. This parameter is NOT optional.<br/>PublishedSchemaArn:  The ARN of the published schema to be upgraded.<br/>DevelopmentSchemaArn:  The ARN of the development schema with the changes used for the upgrade.<br/>
        :return: UpgradedSchemaArn:  The ARN of the upgraded schema that is returned as part of the response.<br/>        """
        pass



class CloudFormation(BaseClient):
    """<fullname>AWS CloudFormation</fullname> AWS CloudFormation allows you to create and manage AWS infrastructure deployments predictably and repeatedly. You can use AWS CloudFormation to leverage AWS products, such as Amazon Elastic Compute Cloud, Amazon Elastic Block Store, Amazon Simple Notification Service, Elastic Load Balancing, and Auto Scaling to build highly-reliable, highly scalable, cost-effective applications without creating or configuring the underlying AWS infrastructure. With AWS CloudFormation, you declare all of your resources and dependencies in a template file. The template defines a collection of resources as a single unit called a stack. AWS CloudFormation creates and deletes all member resources of the stack together and manages all dependencies between the resources for you. For more information about AWS CloudFormation, see the <a href="http://aws.amazon.com/cloudformation/">AWS CloudFormation Product Page</a>. Amazon CloudFormation makes use of other AWS products. If you need additional technical information about a specific AWS product, you can find the product's technical documentation at <a href="https://docs.aws.amazon.com/">docs.aws.amazon.com</a>."""

    class ChangeSetCreateCompleteWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_change_set` for valid parameters"""
            pass
    class StackCreateCompleteWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_stacks` for valid parameters"""
            pass
    class StackDeleteCompleteWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_stacks` for valid parameters"""
            pass
    class StackExistsWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_stacks` for valid parameters"""
            pass
    class StackUpdateCompleteWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_stacks` for valid parameters"""
            pass


    class DescribeAccountLimitsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_account_limits` for valid parameters"""
            pass
    class DescribeChangeSetPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_change_set` for valid parameters"""
            pass
    class DescribeStackEventsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_stack_events` for valid parameters"""
            pass
    class DescribeStacksPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_stacks` for valid parameters"""
            pass
    class ListChangeSetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_change_sets` for valid parameters"""
            pass
    class ListStackInstancesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_stack_instances` for valid parameters"""
            pass
    class ListStackResourcesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_stack_resources` for valid parameters"""
            pass
    class ListStacksPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_stacks` for valid parameters"""
            pass
    class ListStackSetOperationResultsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_stack_set_operation_results` for valid parameters"""
            pass
    class ListStackSetOperationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_stack_set_operations` for valid parameters"""
            pass
    class ListStackSetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_stack_sets` for valid parameters"""
            pass
    class ListExportsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_exports` for valid parameters"""
            pass
    class ListImportsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_imports` for valid parameters"""
            pass


    class CancelUpdateStackInput(Mapping):
        pass
    
    class ContinueUpdateRollbackInput(Mapping):
        pass
    
    class ContinueUpdateRollbackOutput(Mapping):
        pass
    
    class ResourceToSkip(object):
        pass
    
    class CreateChangeSetInput(Mapping):
        pass
    
    class CreateChangeSetOutput(Mapping):
        pass
    
    class UsePreviousTemplate(object):
        pass
    
    class Parameter(Mapping):
        pass
    
    class Capability(object):
        pass
    
    class ResourceType(object):
        pass
    
    class RollbackConfiguration(Mapping):
        pass
    
    class NotificationARN(object):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreateStackInput(Mapping):
        pass
    
    class CreateStackOutput(Mapping):
        pass
    
    class DisableRollback(object):
        pass
    
    class EnableTerminationProtection(object):
        pass
    
    class CreateStackInstancesInput(Mapping):
        pass
    
    class CreateStackInstancesOutput(Mapping):
        pass
    
    class Account(object):
        pass
    
    class Region(object):
        pass
    
    class StackSetOperationPreferences(Mapping):
        pass
    
    class CreateStackSetInput(Mapping):
        pass
    
    class CreateStackSetOutput(Mapping):
        pass
    
    class DeleteChangeSetInput(Mapping):
        pass
    
    class DeleteChangeSetOutput(Mapping):
        pass
    
    class DeleteStackInput(Mapping):
        pass
    
    class LogicalResourceId(object):
        pass
    
    class DeleteStackInstancesInput(Mapping):
        pass
    
    class DeleteStackInstancesOutput(Mapping):
        pass
    
    class RetainStacks(object):
        pass
    
    class DeleteStackSetInput(Mapping):
        pass
    
    class DeleteStackSetOutput(Mapping):
        pass
    
    class DescribeAccountLimitsInput(Mapping):
        pass
    
    class DescribeAccountLimitsOutput(Mapping):
        pass
    
    class DescribeChangeSetInput(Mapping):
        pass
    
    class DescribeChangeSetOutput(Mapping):
        pass
    
    class DescribeStackDriftDetectionStatusInput(Mapping):
        pass
    
    class DescribeStackDriftDetectionStatusOutput(Mapping):
        pass
    
    class DescribeStackEventsInput(Mapping):
        pass
    
    class DescribeStackEventsOutput(Mapping):
        pass
    
    class DescribeStackInstanceInput(Mapping):
        pass
    
    class DescribeStackInstanceOutput(Mapping):
        pass
    
    class DescribeStackResourceInput(Mapping):
        pass
    
    class DescribeStackResourceOutput(Mapping):
        pass
    
    class DescribeStackResourceDriftsInput(Mapping):
        pass
    
    class DescribeStackResourceDriftsOutput(Mapping):
        pass
    
    class StackResourceDriftStatus(object):
        pass
    
    class DescribeStackResourcesInput(Mapping):
        pass
    
    class DescribeStackResourcesOutput(Mapping):
        pass
    
    class DescribeStackSetInput(Mapping):
        pass
    
    class DescribeStackSetOutput(Mapping):
        pass
    
    class DescribeStackSetOperationInput(Mapping):
        pass
    
    class DescribeStackSetOperationOutput(Mapping):
        pass
    
    class DescribeStacksInput(Mapping):
        pass
    
    class DescribeStacksOutput(Mapping):
        pass
    
    class DetectStackDriftInput(Mapping):
        pass
    
    class DetectStackDriftOutput(Mapping):
        pass
    
    class DetectStackResourceDriftInput(Mapping):
        pass
    
    class DetectStackResourceDriftOutput(Mapping):
        pass
    
    class EstimateTemplateCostInput(Mapping):
        pass
    
    class EstimateTemplateCostOutput(Mapping):
        pass
    
    class ExecuteChangeSetInput(Mapping):
        pass
    
    class ExecuteChangeSetOutput(Mapping):
        pass
    
    class GetStackPolicyInput(Mapping):
        pass
    
    class GetStackPolicyOutput(Mapping):
        pass
    
    class GetTemplateInput(Mapping):
        pass
    
    class GetTemplateOutput(Mapping):
        pass
    
    class GetTemplateSummaryInput(Mapping):
        pass
    
    class GetTemplateSummaryOutput(Mapping):
        pass
    
    class ListChangeSetsInput(Mapping):
        pass
    
    class ListChangeSetsOutput(Mapping):
        pass
    
    class ListExportsInput(Mapping):
        pass
    
    class ListExportsOutput(Mapping):
        pass
    
    class ListImportsInput(Mapping):
        pass
    
    class ListImportsOutput(Mapping):
        pass
    
    class ListStackInstancesInput(Mapping):
        pass
    
    class ListStackInstancesOutput(Mapping):
        pass
    
    class ListStackResourcesInput(Mapping):
        pass
    
    class ListStackResourcesOutput(Mapping):
        pass
    
    class ListStackSetOperationResultsInput(Mapping):
        pass
    
    class ListStackSetOperationResultsOutput(Mapping):
        pass
    
    class ListStackSetOperationsInput(Mapping):
        pass
    
    class ListStackSetOperationsOutput(Mapping):
        pass
    
    class ListStackSetsInput(Mapping):
        pass
    
    class ListStackSetsOutput(Mapping):
        pass
    
    class ListStacksInput(Mapping):
        pass
    
    class ListStacksOutput(Mapping):
        pass
    
    class StackStatus(object):
        pass
    
    class SetStackPolicyInput(Mapping):
        pass
    
    class SignalResourceInput(Mapping):
        pass
    
    class StopStackSetOperationInput(Mapping):
        pass
    
    class StopStackSetOperationOutput(Mapping):
        pass
    
    class UpdateStackInput(Mapping):
        pass
    
    class UpdateStackOutput(Mapping):
        pass
    
    class UpdateStackInstancesInput(Mapping):
        pass
    
    class UpdateStackInstancesOutput(Mapping):
        pass
    
    class UpdateStackSetInput(Mapping):
        pass
    
    class UpdateStackSetOutput(Mapping):
        pass
    
    class UpdateTerminationProtectionInput(Mapping):
        pass
    
    class UpdateTerminationProtectionOutput(Mapping):
        pass
    
    class ValidateTemplateInput(Mapping):
        pass
    
    class ValidateTemplateOutput(Mapping):
        pass
    
    def cancel_update_stack(self, StackName: str, ClientRequestToken: str=None):
        """Cancels an update on the specified stack. If the call completes successfully, the stack rolls back the update and reverts to the previous stack configuration. <note> You can cancel only stacks that are in the UPDATE_IN_PROGRESS state. </note>

        :param self:
        :param ClientRequestToken:  A unique identifier for this <code>CancelUpdateStack</code> request. Specify this token if you plan to retry requests so that AWS CloudFormation knows that you're not attempting to cancel an update on a stack with the same name. You might retry <code>CancelUpdateStack</code> requests to ensure that AWS CloudFormation successfully received them.<br/>StackName:  The name or the unique stack ID that is associated with the stack.<br/>
        :return:         """
        pass

    def continue_update_rollback(self, StackName: str, RoleARN: str=None, ResourcesToSkip: List[ResourceToSkip]=None, ClientRequestToken: str=None) -> ContinueUpdateRollbackOutput:
        """For a specified stack that is in the <code>UPDATE_ROLLBACK_FAILED</code> state, continues rolling it back to the <code>UPDATE_ROLLBACK_COMPLETE</code> state. Depending on the cause of the failure, you can manually <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/troubleshooting.html#troubleshooting-errors-update-rollback-failed"> fix the error</a> and continue the rollback. By continuing the rollback, you can return your stack to a working state (the <code>UPDATE_ROLLBACK_COMPLETE</code> state), and then try to update the stack again. A stack goes into the <code>UPDATE_ROLLBACK_FAILED</code> state when AWS CloudFormation cannot roll back all changes after a failed stack update. For example, you might have a stack that is rolling back to an old database instance that was deleted outside of AWS CloudFormation. Because AWS CloudFormation doesn't know the database was deleted, it assumes that the database instance still exists and attempts to roll back to it, causing the update rollback to fail.

        :param self:
        :param ClientRequestToken:  A unique identifier for this <code>ContinueUpdateRollback</code> request. Specify this token if you plan to retry requests so that AWS CloudFormation knows that you're not attempting to continue the rollback to a stack with the same name. You might retry <code>ContinueUpdateRollback</code> requests to ensure that AWS CloudFormation successfully received them.<br/>ResourcesToSkip:  A list of the logical IDs of the resources that AWS CloudFormation skips during the continue update rollback operation. You can specify only resources that are in the <code>UPDATE_FAILED</code> state because a rollback failed. You can't specify resources that are in the <code>UPDATE_FAILED</code> state for other reasons, for example, because an update was cancelled. To check why a resource update failed, use the <a>DescribeStackResources</a> action, and view the resource status reason.  <important> Specify this property to skip rolling back resources that AWS CloudFormation can't successfully roll back. We recommend that you <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/troubleshooting.html#troubleshooting-errors-update-rollback-failed"> troubleshoot</a> resources before skipping them. AWS CloudFormation sets the status of the specified resources to <code>UPDATE_COMPLETE</code> and continues to roll back the stack. After the rollback is complete, the state of the skipped resources will be inconsistent with the state of the resources in the stack template. Before performing another stack update, you must update the stack or resources to be consistent with each other. If you don't, subsequent stack updates might fail, and the stack will become unrecoverable.  </important> Specify the minimum number of resources required to successfully roll back your stack. For example, a failed resource update might cause dependent resources to fail. In this case, it might not be necessary to skip the dependent resources.  To skip resources that are part of nested stacks, use the following format: <code>NestedStackName.ResourceLogicalID</code>. If you want to specify the logical ID of a stack resource (<code>Type: AWS::CloudFormation::Stack</code>) in the <code>ResourcesToSkip</code> list, then its corresponding embedded stack must be in one of the following states: <code>DELETE_IN_PROGRESS</code>, <code>DELETE_COMPLETE</code>, or <code>DELETE_FAILED</code>.  <note> Don't confuse a child stack's name with its corresponding logical ID defined in the parent stack. For an example of a continue update rollback operation with nested stacks, see <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-cfn-updating-stacks-continueupdaterollback.html#nested-stacks">Using ResourcesToSkip to recover a nested stacks hierarchy</a>.  </note><br/>RoleARN:  The Amazon Resource Name (ARN) of an AWS Identity and Access Management (IAM) role that AWS CloudFormation assumes to roll back the stack. AWS CloudFormation uses the role's credentials to make calls on your behalf. AWS CloudFormation always uses this role for all future operations on the stack. As long as users have permission to operate on the stack, AWS CloudFormation uses this role even if the users don't have permission to pass it. Ensure that the role grants least privilege. If you don't specify a value, AWS CloudFormation uses the role that was previously associated with the stack. If no role is available, AWS CloudFormation uses a temporary session that is generated from your user credentials.<br/>StackName:  The name or the unique ID of the stack that you want to continue rolling back. <note> Don't specify the name of a nested stack (a stack that was created by using the <code>AWS::CloudFormation::Stack</code> resource). Instead, use this operation on the parent stack (the stack that contains the <code>AWS::CloudFormation::Stack</code> resource). </note><br/>
        :return:         """
        pass

    def create_change_set(self, ChangeSetName: str, StackName: str, TemplateBody: str=None, TemplateURL: str=None, UsePreviousTemplate: UsePreviousTemplate=None, Parameters: List[Parameter]=None, Capabilities: List[Capability]=None, ResourceTypes: List[ResourceType]=None, RoleARN: str=None, RollbackConfiguration: RollbackConfiguration=None, NotificationARNs: List[NotificationARN]=None, Tags: List[Tag]=None, ClientToken: str=None, Description: str=None, ChangeSetType: str=None) -> CreateChangeSetOutput:
        """Creates a list of changes that will be applied to a stack so that you can review the changes before executing them. You can create a change set for a stack that doesn't exist or an existing stack. If you create a change set for a stack that doesn't exist, the change set shows all of the resources that AWS CloudFormation will create. If you create a change set for an existing stack, AWS CloudFormation compares the stack's information with the information that you submit in the change set and lists the differences. Use change sets to understand which resources AWS CloudFormation will create or change, and how it will change resources in an existing stack, before you create or update a stack. To create a change set for a stack that doesn't exist, for the <code>ChangeSetType</code> parameter, specify <code>CREATE</code>. To create a change set for an existing stack, specify <code>UPDATE</code> for the <code>ChangeSetType</code> parameter. After the <code>CreateChangeSet</code> call successfully completes, AWS CloudFormation starts creating the change set. To check the status of the change set or to review it, use the <a>DescribeChangeSet</a> action. When you are satisfied with the changes the change set will make, execute the change set by using the <a>ExecuteChangeSet</a> action. AWS CloudFormation doesn't make changes until you execute the change set.

        :param self:
        :param ChangeSetType:  The type of change set operation. To create a change set for a new stack, specify <code>CREATE</code>. To create a change set for an existing stack, specify <code>UPDATE</code>. If you create a change set for a new stack, AWS Cloudformation creates a stack with a unique stack ID, but no template or resources. The stack will be in the <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-cfn-describing-stacks.html#d0e11995"> <code>REVIEW_IN_PROGRESS</code> </a> state until you execute the change set. By default, AWS CloudFormation specifies <code>UPDATE</code>. You can't use the <code>UPDATE</code> type to create a change set for a new stack or the <code>CREATE</code> type to create a change set for an existing stack.<br/>Description:  A description to help you identify this change set.<br/>ClientToken:  A unique identifier for this <code>CreateChangeSet</code> request. Specify this token if you plan to retry requests so that AWS CloudFormation knows that you're not attempting to create another change set with the same name. You might retry <code>CreateChangeSet</code> requests to ensure that AWS CloudFormation successfully received them.<br/>ChangeSetName:  The name of the change set. The name must be unique among all change sets that are associated with the specified stack. A change set name can contain only alphanumeric, case sensitive characters and hyphens. It must start with an alphabetic character and cannot exceed 128 characters.<br/>Tags:  Key-value pairs to associate with this stack. AWS CloudFormation also propagates these tags to resources in the stack. You can specify a maximum of 50 tags.<br/>NotificationARNs:  The Amazon Resource Names (ARNs) of Amazon Simple Notification Service (Amazon SNS) topics that AWS CloudFormation associates with the stack. To remove all associated notification topics, specify an empty list.<br/>RollbackConfiguration:  The rollback triggers for AWS CloudFormation to monitor during stack creation and updating operations, and for the specified monitoring period afterwards.:<br/>RoleARN:  The Amazon Resource Name (ARN) of an AWS Identity and Access Management (IAM) role that AWS CloudFormation assumes when executing the change set. AWS CloudFormation uses the role's credentials to make calls on your behalf. AWS CloudFormation uses this role for all future operations on the stack. As long as users have permission to operate on the stack, AWS CloudFormation uses this role even if the users don't have permission to pass it. Ensure that the role grants least privilege. If you don't specify a value, AWS CloudFormation uses the role that was previously associated with the stack. If no role is available, AWS CloudFormation uses a temporary session that is generated from your user credentials.<br/>ResourceTypes:  The template resource types that you have permissions to work with if you execute this change set, such as <code>AWS::EC2::Instance</code>, <code>AWS::EC2::*</code>, or <code>Custom::MyCustomInstance</code>. If the list of resource types doesn't include a resource type that you're updating, the stack update fails. By default, AWS CloudFormation grants permissions to all resource types. AWS Identity and Access Management (IAM) uses this parameter for condition keys in IAM policies for AWS CloudFormation. For more information, see <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-iam-template.html">Controlling Access with AWS Identity and Access Management</a> in the AWS CloudFormation User Guide.<br/>Capabilities:  In some cases, you must explicity acknowledge that your stack template contains certain capabilities in order for AWS CloudFormation to create the stack. <ul> <li>  <code>CAPABILITY_IAM</code> and <code>CAPABILITY_NAMED_IAM</code>  Some stack templates might include resources that can affect permissions in your AWS account; for example, by creating new AWS Identity and Access Management (IAM) users. For those stacks, you must explicitly acknowledge this by specifying one of these capabilities. The following IAM resources require you to specify either the <code>CAPABILITY_IAM</code> or <code>CAPABILITY_NAMED_IAM</code> capability. <ul> <li> If you have IAM resources, you can specify either capability.  </li> <li> If you have IAM resources with custom names, you <i>must</i> specify <code>CAPABILITY_NAMED_IAM</code>.  </li> <li> If you don't specify either of these capabilities, AWS CloudFormation returns an <code>InsufficientCapabilities</code> error. </li> </ul> If your stack template contains these resources, we recommend that you review all permissions associated with them and edit their permissions if necessary. <ul> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-accesskey.html"> AWS::IAM::AccessKey</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-group.html"> AWS::IAM::Group</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-instanceprofile.html"> AWS::IAM::InstanceProfile</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-policy.html"> AWS::IAM::Policy</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-role.html"> AWS::IAM::Role</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-user.html"> AWS::IAM::User</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-addusertogroup.html"> AWS::IAM::UserToGroupAddition</a>  </li> </ul> For more information, see <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-iam-template.html#capabilities">Acknowledging IAM Resources in AWS CloudFormation Templates</a>. </li> <li>  <code>CAPABILITY_AUTO_EXPAND</code>  Some template contain macros. Macros perform custom processing on templates; this can include simple actions like find-and-replace operations, all the way to extensive transformations of entire templates. Because of this, users typically create a change set from the processed template, so that they can review the changes resulting from the macros before actually creating the stack. If your stack template contains one or more macros, and you choose to create a stack directly from the processed template, without first reviewing the resulting changes in a change set, you must acknowledge this capability. This includes the <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/create-reusable-transform-function-snippets-and-add-to-your-template-with-aws-include-transform.html">AWS::Include</a> and <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/transform-aws-serverless.html">AWS::Serverless</a> transforms, which are macros hosted by AWS CloudFormation. <note> This capacity does not apply to creating change sets, and specifying it when creating change sets has no effect. Also, change sets do not currently support nested stacks. If you want to create a stack from a stack template that contains macros <i>and</i> nested stacks, you must create or update the stack directly from the template using the <a>CreateStack</a> or <a>UpdateStack</a> action, and specifying this capability. </note> For more information on macros, see <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/template-macros.html">Using AWS CloudFormation Macros to Perform Custom Processing on Templates</a>. </li> </ul><br/>Parameters:  A list of <code>Parameter</code> structures that specify input parameters for the change set. For more information, see the <a>Parameter</a> data type.<br/>UsePreviousTemplate:  Whether to reuse the template that is associated with the stack to create the change set.<br/>TemplateURL:  The location of the file that contains the revised template. The URL must point to a template (max size: 460,800 bytes) that is located in an S3 bucket. AWS CloudFormation generates the change set by comparing this template with the stack that you specified. Conditional: You must specify only <code>TemplateBody</code> or <code>TemplateURL</code>.<br/>TemplateBody:  A structure that contains the body of the revised template, with a minimum length of 1 byte and a maximum length of 51,200 bytes. AWS CloudFormation generates the change set by comparing this template with the template of the stack that you specified. Conditional: You must specify only <code>TemplateBody</code> or <code>TemplateURL</code>.<br/>StackName:  The name or the unique ID of the stack for which you are creating a change set. AWS CloudFormation generates the change set by comparing this stack's information with the information that you submit, such as a modified template or different parameter input values.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringTimeInMinutes</i> The amount of time, in minutes, during which CloudFormation should monitor all the rollback triggers after the stack creation or update operation deploys all necessary resources. The default is 0 minutes. If you specify a monitoring period but do not specify any rollback triggers, CloudFormation still waits the specified period of time before cleaning up old resources after update operations. You can use this monitoring period to perform any manual stack validation desired, and manually cancel the stack creation or update (using <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/APIReference/API_CancelUpdateStack.html">CancelUpdateStack</a>, for example) as necessary. If you specify 0 for this parameter, CloudFormation still monitors the specified rollback triggers during stack creation and update operations. Then, for update operations, it begins disposing of old resources immediately once the operation completes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RollbackTriggers</i> The triggers to monitor during stack creation or update actions.  By default, AWS CloudFormation saves the rollback triggers specified for a stack and applies them to any subsequent update operations for the stack, unless you specify otherwise. If you do specify rollback triggers for this parameter, those triggers replace any list of triggers previously specified for the stack. This means: <ul> <li> To use the rollback triggers previously specified for this stack, if any, don't specify this parameter. </li> <li> To specify new or updated rollback triggers, you must specify <i>all</i> the triggers that you want used for this stack, even triggers you've specifed before (for example, when creating the stack or during a previous stack update). Any triggers that you don't include in the updated list of triggers are no longer applied to the stack. </li> <li> To remove all currently specified triggers, specify an empty list for this parameter. </li> </ul> If a specified trigger is missing, the entire stack operation fails and is rolled back. <br/>
        :return: StackId:  The unique ID of the stack.<br/>Id:  The Amazon Resource Name (ARN) of the change set.<br/>        """
        pass

    def create_stack(self, StackName: str, TemplateBody: str=None, TemplateURL: str=None, Parameters: List[Parameter]=None, DisableRollback: DisableRollback=None, RollbackConfiguration: RollbackConfiguration=None, TimeoutInMinutes: int=None, NotificationARNs: List[NotificationARN]=None, Capabilities: List[Capability]=None, ResourceTypes: List[ResourceType]=None, RoleARN: str=None, OnFailure: str=None, StackPolicyBody: str=None, StackPolicyURL: str=None, Tags: List[Tag]=None, ClientRequestToken: str=None, EnableTerminationProtection: EnableTerminationProtection=None) -> CreateStackOutput:
        """Creates a stack as specified in the template. After the call completes successfully, the stack creation starts. You can check the status of the stack via the <a>DescribeStacks</a> API.

        :param self:
        :param EnableTerminationProtection:  Whether to enable termination protection on the specified stack. If a user attempts to delete a stack with termination protection enabled, the operation fails and the stack remains unchanged. For more information, see <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-cfn-protect-stacks.html">Protecting a Stack From Being Deleted</a> in the <i>AWS CloudFormation User Guide</i>. Termination protection is disabled on stacks by default.   For <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-cfn-nested-stacks.html">nested stacks</a>, termination protection is set on the root stack and cannot be changed directly on the nested stack.<br/>ClientRequestToken:  A unique identifier for this <code>CreateStack</code> request. Specify this token if you plan to retry requests so that AWS CloudFormation knows that you're not attempting to create a stack with the same name. You might retry <code>CreateStack</code> requests to ensure that AWS CloudFormation successfully received them. All events triggered by a given stack operation are assigned the same client request token, which you can use to track operations. For example, if you execute a <code>CreateStack</code> operation with the token <code>token1</code>, then all the <code>StackEvents</code> generated by that operation will have <code>ClientRequestToken</code> set as <code>token1</code>. In the console, stack operations display the client request token on the Events tab. Stack operations that are initiated from the console use the token format <i>Console-StackOperation-ID</i>, which helps you easily identify the stack operation . For example, if you create a stack using the console, each stack event would be assigned the same token in the following format: <code>Console-CreateStack-7f59c3cf-00d2-40c7-b2ff-e75db0987002</code>. <br/>Tags:  Key-value pairs to associate with this stack. AWS CloudFormation also propagates these tags to the resources created in the stack. A maximum number of 50 tags can be specified.<br/>StackPolicyURL:  Location of a file containing the stack policy. The URL must point to a policy (maximum size: 16 KB) located in an S3 bucket in the same region as the stack. You can specify either the <code>StackPolicyBody</code> or the <code>StackPolicyURL</code> parameter, but not both.<br/>StackPolicyBody:  Structure containing the stack policy body. For more information, go to <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/protect-stack-resources.html"> Prevent Updates to Stack Resources</a> in the <i>AWS CloudFormation User Guide</i>. You can specify either the <code>StackPolicyBody</code> or the <code>StackPolicyURL</code> parameter, but not both.<br/>OnFailure:  Determines what action will be taken if stack creation fails. This must be one of: DO_NOTHING, ROLLBACK, or DELETE. You can specify either <code>OnFailure</code> or <code>DisableRollback</code>, but not both. Default: <code>ROLLBACK</code> <br/>RoleARN:  The Amazon Resource Name (ARN) of an AWS Identity and Access Management (IAM) role that AWS CloudFormation assumes to create the stack. AWS CloudFormation uses the role's credentials to make calls on your behalf. AWS CloudFormation always uses this role for all future operations on the stack. As long as users have permission to operate on the stack, AWS CloudFormation uses this role even if the users don't have permission to pass it. Ensure that the role grants least privilege. If you don't specify a value, AWS CloudFormation uses the role that was previously associated with the stack. If no role is available, AWS CloudFormation uses a temporary session that is generated from your user credentials.<br/>ResourceTypes:  The template resource types that you have permissions to work with for this create stack action, such as <code>AWS::EC2::Instance</code>, <code>AWS::EC2::*</code>, or <code>Custom::MyCustomInstance</code>. Use the following syntax to describe template resource types: <code>AWS::*</code> (for all AWS resource), <code>Custom::*</code> (for all custom resources), <code>Custom::<i>logical_ID</i> </code> (for a specific custom resource), <code>AWS::<i>service_name</i>::*</code> (for all resources of a particular AWS service), and <code>AWS::<i>service_name</i>::<i>resource_logical_ID</i> </code> (for a specific AWS resource). If the list of resource types doesn't include a resource that you're creating, the stack creation fails. By default, AWS CloudFormation grants permissions to all resource types. AWS Identity and Access Management (IAM) uses this parameter for AWS CloudFormation-specific condition keys in IAM policies. For more information, see <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-iam-template.html">Controlling Access with AWS Identity and Access Management</a>.<br/>Capabilities:  In some cases, you must explicity acknowledge that your stack template contains certain capabilities in order for AWS CloudFormation to create the stack. <ul> <li>  <code>CAPABILITY_IAM</code> and <code>CAPABILITY_NAMED_IAM</code>  Some stack templates might include resources that can affect permissions in your AWS account; for example, by creating new AWS Identity and Access Management (IAM) users. For those stacks, you must explicitly acknowledge this by specifying one of these capabilities. The following IAM resources require you to specify either the <code>CAPABILITY_IAM</code> or <code>CAPABILITY_NAMED_IAM</code> capability. <ul> <li> If you have IAM resources, you can specify either capability.  </li> <li> If you have IAM resources with custom names, you <i>must</i> specify <code>CAPABILITY_NAMED_IAM</code>.  </li> <li> If you don't specify either of these capabilities, AWS CloudFormation returns an <code>InsufficientCapabilities</code> error. </li> </ul> If your stack template contains these resources, we recommend that you review all permissions associated with them and edit their permissions if necessary. <ul> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-accesskey.html"> AWS::IAM::AccessKey</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-group.html"> AWS::IAM::Group</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-instanceprofile.html"> AWS::IAM::InstanceProfile</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-policy.html"> AWS::IAM::Policy</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-role.html"> AWS::IAM::Role</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-user.html"> AWS::IAM::User</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-addusertogroup.html"> AWS::IAM::UserToGroupAddition</a>  </li> </ul> For more information, see <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-iam-template.html#capabilities">Acknowledging IAM Resources in AWS CloudFormation Templates</a>. </li> <li>  <code>CAPABILITY_AUTO_EXPAND</code>  Some template contain macros. Macros perform custom processing on templates; this can include simple actions like find-and-replace operations, all the way to extensive transformations of entire templates. Because of this, users typically create a change set from the processed template, so that they can review the changes resulting from the macros before actually creating the stack. If your stack template contains one or more macros, and you choose to create a stack directly from the processed template, without first reviewing the resulting changes in a change set, you must acknowledge this capability. This includes the <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/create-reusable-transform-function-snippets-and-add-to-your-template-with-aws-include-transform.html">AWS::Include</a> and <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/transform-aws-serverless.html">AWS::Serverless</a> transforms, which are macros hosted by AWS CloudFormation. Change sets do not currently support nested stacks. If you want to create a stack from a stack template that contains macros <i>and</i> nested stacks, you must create the stack directly from the template using this capability. <important> You should only create stacks directly from a stack template that contains macros if you know what processing the macro performs. Each macro relies on an underlying Lambda service function for processing stack templates. Be aware that the Lambda function owner can update the function operation without AWS CloudFormation being notified. </important> For more information, see <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/template-macros.html">Using AWS CloudFormation Macros to Perform Custom Processing on Templates</a>. </li> </ul><br/>NotificationARNs:  The Simple Notification Service (SNS) topic ARNs to publish stack related events. You can find your SNS topic ARNs using the SNS console or your Command Line Interface (CLI).<br/>TimeoutInMinutes:  The amount of time that can pass before the stack status becomes CREATE_FAILED; if <code>DisableRollback</code> is not set or is set to <code>false</code>, the stack will be rolled back.<br/>RollbackConfiguration:  The rollback triggers for AWS CloudFormation to monitor during stack creation and updating operations, and for the specified monitoring period afterwards.:<br/>DisableRollback:  Set to <code>true</code> to disable rollback of the stack if stack creation failed. You can specify either <code>DisableRollback</code> or <code>OnFailure</code>, but not both. Default: <code>false</code> <br/>Parameters:  A list of <code>Parameter</code> structures that specify input parameters for the stack. For more information, see the <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/APIReference/API_Parameter.html">Parameter</a> data type.<br/>TemplateURL:  Location of file containing the template body. The URL must point to a template (max size: 460,800 bytes) that is located in an Amazon S3 bucket. For more information, go to the <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/template-anatomy.html">Template Anatomy</a> in the AWS CloudFormation User Guide. Conditional: You must specify either the <code>TemplateBody</code> or the <code>TemplateURL</code> parameter, but not both.<br/>TemplateBody:  Structure containing the template body with a minimum length of 1 byte and a maximum length of 51,200 bytes. For more information, go to <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/template-anatomy.html">Template Anatomy</a> in the AWS CloudFormation User Guide. Conditional: You must specify either the <code>TemplateBody</code> or the <code>TemplateURL</code> parameter, but not both.<br/>StackName:  The name that is associated with the stack. The name must be unique in the region in which you are creating the stack. <note> A stack name can contain only alphanumeric characters (case sensitive) and hyphens. It must start with an alphabetic character and cannot be longer than 128 characters. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringTimeInMinutes</i> The amount of time, in minutes, during which CloudFormation should monitor all the rollback triggers after the stack creation or update operation deploys all necessary resources. The default is 0 minutes. If you specify a monitoring period but do not specify any rollback triggers, CloudFormation still waits the specified period of time before cleaning up old resources after update operations. You can use this monitoring period to perform any manual stack validation desired, and manually cancel the stack creation or update (using <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/APIReference/API_CancelUpdateStack.html">CancelUpdateStack</a>, for example) as necessary. If you specify 0 for this parameter, CloudFormation still monitors the specified rollback triggers during stack creation and update operations. Then, for update operations, it begins disposing of old resources immediately once the operation completes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RollbackTriggers</i> The triggers to monitor during stack creation or update actions.  By default, AWS CloudFormation saves the rollback triggers specified for a stack and applies them to any subsequent update operations for the stack, unless you specify otherwise. If you do specify rollback triggers for this parameter, those triggers replace any list of triggers previously specified for the stack. This means: <ul> <li> To use the rollback triggers previously specified for this stack, if any, don't specify this parameter. </li> <li> To specify new or updated rollback triggers, you must specify <i>all</i> the triggers that you want used for this stack, even triggers you've specifed before (for example, when creating the stack or during a previous stack update). Any triggers that you don't include in the updated list of triggers are no longer applied to the stack. </li> <li> To remove all currently specified triggers, specify an empty list for this parameter. </li> </ul> If a specified trigger is missing, the entire stack operation fails and is rolled back. <br/>
        :return: StackId:  Unique identifier of the stack.<br/>        """
        pass

    def create_stack_instances(self, Regions: List[Region], Accounts: List[Account], StackSetName: str, ParameterOverrides: List[Parameter]=None, OperationPreferences: StackSetOperationPreferences=None, OperationId: str=None) -> CreateStackInstancesOutput:
        """Creates stack instances for the specified accounts, within the specified regions. A stack instance refers to a stack in a specific account and region. <code>Accounts</code> and <code>Regions</code> are required parameters—you must specify at least one account and one region. 

        :param self:
        :param OperationId:  The unique identifier for this stack set operation.  The operation ID also functions as an idempotency token, to ensure that AWS CloudFormation performs the stack set operation only once, even if you retry the request multiple times. You might retry stack set operation requests to ensure that AWS CloudFormation successfully received them. If you don't specify an operation ID, the SDK generates one automatically.  Repeating this stack set operation with a new operation ID retries all stack instances whose status is <code>OUTDATED</code>. <br/>OperationPreferences:  Preferences for how AWS CloudFormation performs this stack set operation.:<br/>ParameterOverrides:  A list of stack set parameters whose values you want to override in the selected stack instances. Any overridden parameter values will be applied to all stack instances in the specified accounts and regions. When specifying parameters and their values, be aware of how AWS CloudFormation sets parameter values during stack instance operations: <ul> <li> To override the current value for a parameter, include the parameter and specify its value. </li> <li> To leave a parameter set to its present value, you can do one of the following: <ul> <li> Do not include the parameter in the list. </li> <li> Include the parameter and specify <code>UsePreviousValue</code> as <code>true</code>. (You cannot specify both a value and set <code>UsePreviousValue</code> to <code>true</code>.) </li> </ul> </li> <li> To set all overridden parameter back to the values specified in the stack set, specify a parameter list but do not include any parameters. </li> <li> To leave all parameters set to their present values, do not specify this property at all. </li> </ul> During stack set updates, any parameter values overridden for a stack instance are not updated, but retain their overridden value. You can only override the parameter <i>values</i> that are specified in the stack set; to add or delete a parameter itself, use <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/APIReference/API_UpdateStackSet.html">UpdateStackSet</a> to update the stack set template.<br/>Regions:  The names of one or more regions where you want to create stack instances using the specified AWS account(s). <br/>Accounts:  The names of one or more AWS accounts that you want to create stack instances in the specified region(s) for.<br/>StackSetName:  The name or unique ID of the stack set that you want to create stack instances from.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxConcurrentPercentage</i> The maximum percentage of accounts in which to perform this operation at one time. When calculating the number of accounts based on the specified percentage, AWS CloudFormation rounds down to the next whole number. This is true except in cases where rounding down would result is zero. In this case, CloudFormation sets the number as one instead. Note that this setting lets you specify the <i>maximum</i> for operations. For large deployments, under certain circumstances the actual number of accounts acted upon concurrently may be lower due to service throttling. Conditional: You must specify either <code>MaxConcurrentCount</code> or <code>MaxConcurrentPercentage</code>, but not both.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxConcurrentCount</i> The maximum number of accounts in which to perform this operation at one time. This is dependent on the value of <code>FailureToleranceCount</code>—<code>MaxConcurrentCount</code> is at most one more than the <code>FailureToleranceCount</code> . Note that this setting lets you specify the <i>maximum</i> for operations. For large deployments, under certain circumstances the actual number of accounts acted upon concurrently may be lower due to service throttling. Conditional: You must specify either <code>MaxConcurrentCount</code> or <code>MaxConcurrentPercentage</code>, but not both.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureTolerancePercentage</i> The percentage of accounts, per region, for which this stack operation can fail before AWS CloudFormation stops the operation in that region. If the operation is stopped in a region, AWS CloudFormation doesn't attempt the operation in any subsequent regions. When calculating the number of accounts based on the specified percentage, AWS CloudFormation rounds <i>down</i> to the next whole number. Conditional: You must specify either <code>FailureToleranceCount</code> or <code>FailureTolerancePercentage</code>, but not both.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureToleranceCount</i> The number of accounts, per region, for which this operation can fail before AWS CloudFormation stops the operation in that region. If the operation is stopped in a region, AWS CloudFormation doesn't attempt the operation in any subsequent regions. Conditional: You must specify either <code>FailureToleranceCount</code> or <code>FailureTolerancePercentage</code> (but not both).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RegionOrder</i> The order of the regions in where you want to perform the stack operation.<br/>
        :return: OperationId:  The unique identifier for this stack set operation.<br/>        """
        pass

    def create_stack_set(self, StackSetName: str, Description: str=None, TemplateBody: str=None, TemplateURL: str=None, Parameters: List[Parameter]=None, Capabilities: List[Capability]=None, Tags: List[Tag]=None, AdministrationRoleARN: str=None, ExecutionRoleName: str=None, ClientRequestToken: str=None) -> CreateStackSetOutput:
        """Creates a stack set.

        :param self:
        :param ClientRequestToken:  A unique identifier for this <code>CreateStackSet</code> request. Specify this token if you plan to retry requests so that AWS CloudFormation knows that you're not attempting to create another stack set with the same name. You might retry <code>CreateStackSet</code> requests to ensure that AWS CloudFormation successfully received them. If you don't specify an operation ID, the SDK generates one automatically. <br/>ExecutionRoleName:  The name of the IAM execution role to use to create the stack set. If you do not specify an execution role, AWS CloudFormation uses the <code>AWSCloudFormationStackSetExecutionRole</code> role for the stack set operation. Specify an IAM role only if you are using customized execution roles to control which stack resources users and groups can include in their stack sets. <br/>AdministrationRoleARN:  The Amazon Resource Number (ARN) of the IAM role to use to create this stack set.  Specify an IAM role only if you are using customized administrator roles to control which users or groups can manage specific stack sets within the same administrator account. For more information, see <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/stacksets-prereqs.html">Prerequisites: Granting Permissions for Stack Set Operations</a> in the <i>AWS CloudFormation User Guide</i>.<br/>Tags:  The key-value pairs to associate with this stack set and the stacks created from it. AWS CloudFormation also propagates these tags to supported resources that are created in the stacks. A maximum number of 50 tags can be specified. If you specify tags as part of a <code>CreateStackSet</code> action, AWS CloudFormation checks to see if you have the required IAM permission to tag resources. If you don't, the entire <code>CreateStackSet</code> action fails with an <code>access denied</code> error, and the stack set is not created.<br/>Capabilities:  In some cases, you must explicity acknowledge that your stack set template contains certain capabilities in order for AWS CloudFormation to create the stack set and related stack instances. <ul> <li>  <code>CAPABILITY_IAM</code> and <code>CAPABILITY_NAMED_IAM</code>  Some stack templates might include resources that can affect permissions in your AWS account; for example, by creating new AWS Identity and Access Management (IAM) users. For those stack sets, you must explicitly acknowledge this by specifying one of these capabilities. The following IAM resources require you to specify either the <code>CAPABILITY_IAM</code> or <code>CAPABILITY_NAMED_IAM</code> capability. <ul> <li> If you have IAM resources, you can specify either capability.  </li> <li> If you have IAM resources with custom names, you <i>must</i> specify <code>CAPABILITY_NAMED_IAM</code>.  </li> <li> If you don't specify either of these capabilities, AWS CloudFormation returns an <code>InsufficientCapabilities</code> error. </li> </ul> If your stack template contains these resources, we recommend that you review all permissions associated with them and edit their permissions if necessary. <ul> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-accesskey.html"> AWS::IAM::AccessKey</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-group.html"> AWS::IAM::Group</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-instanceprofile.html"> AWS::IAM::InstanceProfile</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-policy.html"> AWS::IAM::Policy</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-role.html"> AWS::IAM::Role</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-user.html"> AWS::IAM::User</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-addusertogroup.html"> AWS::IAM::UserToGroupAddition</a>  </li> </ul> For more information, see <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-iam-template.html#capabilities">Acknowledging IAM Resources in AWS CloudFormation Templates</a>. </li> <li>  <code>CAPABILITY_AUTO_EXPAND</code>  Some templates contain macros. If your stack template contains one or more macros, and you choose to create a stack directly from the processed template, without first reviewing the resulting changes in a change set, you must acknowledge this capability. For more information, see <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/template-macros.html">Using AWS CloudFormation Macros to Perform Custom Processing on Templates</a>. <note> Stack sets do not currently support macros in stack templates. (This includes the <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/create-reusable-transform-function-snippets-and-add-to-your-template-with-aws-include-transform.html">AWS::Include</a> and <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/transform-aws-serverless.html">AWS::Serverless</a> transforms, which are macros hosted by AWS CloudFormation.) Even if you specify this capability, if you include a macro in your template the stack set operation will fail. </note> </li> </ul><br/>Parameters:  The input parameters for the stack set template. <br/>TemplateURL:  The location of the file that contains the template body. The URL must point to a template (maximum size: 460,800 bytes) that's located in an Amazon S3 bucket. For more information, see <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/template-anatomy.html">Template Anatomy</a> in the AWS CloudFormation User Guide. Conditional: You must specify either the TemplateBody or the TemplateURL parameter, but not both.<br/>TemplateBody:  The structure that contains the template body, with a minimum length of 1 byte and a maximum length of 51,200 bytes. For more information, see <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/template-anatomy.html">Template Anatomy</a> in the AWS CloudFormation User Guide. Conditional: You must specify either the TemplateBody or the TemplateURL parameter, but not both.<br/>Description:  A description of the stack set. You can use the description to identify the stack set's purpose or other important information.<br/>StackSetName:  The name to associate with the stack set. The name must be unique in the region where you create your stack set. <note> A stack name can contain only alphanumeric characters (case-sensitive) and hyphens. It must start with an alphabetic character and can't be longer than 128 characters. </note><br/>
        :return: StackSetId:  The ID of the stack set that you're creating.<br/>        """
        pass

    def delete_change_set(self, ChangeSetName: str, StackName: str=None) -> DeleteChangeSetOutput:
        """Deletes the specified change set. Deleting change sets ensures that no one executes the wrong change set. If the call successfully completes, AWS CloudFormation successfully deleted the change set.

        :param self:
        :param StackName:  If you specified the name of a change set to delete, specify the stack name or ID (ARN) that is associated with it.<br/>ChangeSetName:  The name or Amazon Resource Name (ARN) of the change set that you want to delete.<br/>
        :return:         """
        pass

    def delete_stack(self, StackName: str, RetainResources: List[LogicalResourceId]=None, RoleARN: str=None, ClientRequestToken: str=None):
        """Deletes a specified stack. Once the call completes successfully, stack deletion starts. Deleted stacks do not show up in the <a>DescribeStacks</a> API if the deletion has been completed successfully.

        :param self:
        :param ClientRequestToken:  A unique identifier for this <code>DeleteStack</code> request. Specify this token if you plan to retry requests so that AWS CloudFormation knows that you're not attempting to delete a stack with the same name. You might retry <code>DeleteStack</code> requests to ensure that AWS CloudFormation successfully received them. All events triggered by a given stack operation are assigned the same client request token, which you can use to track operations. For example, if you execute a <code>CreateStack</code> operation with the token <code>token1</code>, then all the <code>StackEvents</code> generated by that operation will have <code>ClientRequestToken</code> set as <code>token1</code>. In the console, stack operations display the client request token on the Events tab. Stack operations that are initiated from the console use the token format <i>Console-StackOperation-ID</i>, which helps you easily identify the stack operation . For example, if you create a stack using the console, each stack event would be assigned the same token in the following format: <code>Console-CreateStack-7f59c3cf-00d2-40c7-b2ff-e75db0987002</code>. <br/>RoleARN:  The Amazon Resource Name (ARN) of an AWS Identity and Access Management (IAM) role that AWS CloudFormation assumes to delete the stack. AWS CloudFormation uses the role's credentials to make calls on your behalf. If you don't specify a value, AWS CloudFormation uses the role that was previously associated with the stack. If no role is available, AWS CloudFormation uses a temporary session that is generated from your user credentials.<br/>RetainResources:  For stacks in the <code>DELETE_FAILED</code> state, a list of resource logical IDs that are associated with the resources you want to retain. During deletion, AWS CloudFormation deletes the stack but does not delete the retained resources. Retaining resources is useful when you cannot delete a resource, such as a non-empty S3 bucket, but you want to delete the stack.<br/>StackName:  The name or the unique stack ID that is associated with the stack.<br/>
        :return:         """
        pass

    def delete_stack_instances(self, RetainStacks: RetainStacks, Regions: List[Region], Accounts: List[Account], StackSetName: str, OperationPreferences: StackSetOperationPreferences=None, OperationId: str=None) -> DeleteStackInstancesOutput:
        """Deletes stack instances for the specified accounts, in the specified regions. 

        :param self:
        :param OperationId:  The unique identifier for this stack set operation.  If you don't specify an operation ID, the SDK generates one automatically.  The operation ID also functions as an idempotency token, to ensure that AWS CloudFormation performs the stack set operation only once, even if you retry the request multiple times. You can retry stack set operation requests to ensure that AWS CloudFormation successfully received them. Repeating this stack set operation with a new operation ID retries all stack instances whose status is <code>OUTDATED</code>. <br/>RetainStacks:  Removes the stack instances from the specified stack set, but doesn't delete the stacks. You can't reassociate a retained stack or add an existing, saved stack to a new stack set. For more information, see <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/stacksets-concepts.html#stackset-ops-options">Stack set operation options</a>.<br/>OperationPreferences:  Preferences for how AWS CloudFormation performs this stack set operation.:<br/>Regions:  The regions where you want to delete stack set instances. <br/>Accounts:  The names of the AWS accounts that you want to delete stack instances for.<br/>StackSetName:  The name or unique ID of the stack set that you want to delete stack instances for.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxConcurrentPercentage</i> The maximum percentage of accounts in which to perform this operation at one time. When calculating the number of accounts based on the specified percentage, AWS CloudFormation rounds down to the next whole number. This is true except in cases where rounding down would result is zero. In this case, CloudFormation sets the number as one instead. Note that this setting lets you specify the <i>maximum</i> for operations. For large deployments, under certain circumstances the actual number of accounts acted upon concurrently may be lower due to service throttling. Conditional: You must specify either <code>MaxConcurrentCount</code> or <code>MaxConcurrentPercentage</code>, but not both.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxConcurrentCount</i> The maximum number of accounts in which to perform this operation at one time. This is dependent on the value of <code>FailureToleranceCount</code>—<code>MaxConcurrentCount</code> is at most one more than the <code>FailureToleranceCount</code> . Note that this setting lets you specify the <i>maximum</i> for operations. For large deployments, under certain circumstances the actual number of accounts acted upon concurrently may be lower due to service throttling. Conditional: You must specify either <code>MaxConcurrentCount</code> or <code>MaxConcurrentPercentage</code>, but not both.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureTolerancePercentage</i> The percentage of accounts, per region, for which this stack operation can fail before AWS CloudFormation stops the operation in that region. If the operation is stopped in a region, AWS CloudFormation doesn't attempt the operation in any subsequent regions. When calculating the number of accounts based on the specified percentage, AWS CloudFormation rounds <i>down</i> to the next whole number. Conditional: You must specify either <code>FailureToleranceCount</code> or <code>FailureTolerancePercentage</code>, but not both.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureToleranceCount</i> The number of accounts, per region, for which this operation can fail before AWS CloudFormation stops the operation in that region. If the operation is stopped in a region, AWS CloudFormation doesn't attempt the operation in any subsequent regions. Conditional: You must specify either <code>FailureToleranceCount</code> or <code>FailureTolerancePercentage</code> (but not both).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RegionOrder</i> The order of the regions in where you want to perform the stack operation.<br/>
        :return: OperationId:  The unique identifier for this stack set operation.<br/>        """
        pass

    def delete_stack_set(self, StackSetName: str) -> DeleteStackSetOutput:
        """Deletes a stack set. Before you can delete a stack set, all of its member stack instances must be deleted. For more information about how to do this, see <a>DeleteStackInstances</a>. 

        :param self:
        :param StackSetName:  The name or unique ID of the stack set that you're deleting. You can obtain this value by running <a>ListStackSets</a>.<br/>
        :return:         """
        pass

    def describe_account_limits(self, NextToken: str=None) -> DescribeAccountLimitsOutput:
        """Retrieves your account's AWS CloudFormation limits, such as the maximum number of stacks that you can create in your account. For more information about account limits, see <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/cloudformation-limits.html">AWS CloudFormation Limits</a> in the <i>AWS CloudFormation User Guide</i>.

        :param self:
        :param NextToken:  A string that identifies the next page of limits that you want to retrieve.<br/>
        :return: NextToken:  If the output exceeds 1 MB in size, a string that identifies the next page of limits. If no additional page exists, this value is null.<br/>AccountLimits:  An account limit structure that contain a list of AWS CloudFormation account limits and their values.<br/>        """
        pass

    def describe_change_set(self, ChangeSetName: str, StackName: str=None, NextToken: str=None) -> DescribeChangeSetOutput:
        """Returns the inputs for the change set and a list of changes that AWS CloudFormation will make if you execute the change set. For more information, see <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-cfn-updating-stacks-changesets.html">Updating Stacks Using Change Sets</a> in the AWS CloudFormation User Guide.

        :param self:
        :param NextToken:  A string (provided by the <a>DescribeChangeSet</a> response output) that identifies the next page of information that you want to retrieve.<br/>StackName:  If you specified the name of a change set, specify the stack name or ID (ARN) of the change set you want to describe.<br/>ChangeSetName:  The name or Amazon Resource Name (ARN) of the change set that you want to describe.<br/>
        :return: NextToken:  If the output exceeds 1 MB, a string that identifies the next page of changes. If there is no additional page, this value is null.<br/>Changes:  A list of <code>Change</code> structures that describes the resources AWS CloudFormation changes if you execute the change set.<br/>Tags:  If you execute the change set, the tags that will be associated with the stack.<br/>Capabilities:  If you execute the change set, the list of capabilities that were explicitly acknowledged when the change set was created.<br/>RollbackConfiguration:  The rollback triggers for AWS CloudFormation to monitor during stack creation and updating operations, and for the specified monitoring period afterwards.:<br/>NotificationARNs:  The ARNs of the Amazon Simple Notification Service (Amazon SNS) topics that will be associated with the stack if you execute the change set.<br/>StatusReason:  A description of the change set's status. For example, if your attempt to create a change set failed, AWS CloudFormation shows the error message.<br/>Status:  The current status of the change set, such as <code>CREATE_IN_PROGRESS</code>, <code>CREATE_COMPLETE</code>, or <code>FAILED</code>.<br/>ExecutionStatus:  If the change set execution status is <code>AVAILABLE</code>, you can execute the change set. If you can’t execute the change set, the status indicates why. For example, a change set might be in an <code>UNAVAILABLE</code> state because AWS CloudFormation is still creating it or in an <code>OBSOLETE</code> state because the stack was already updated.<br/>CreationTime:  The start time when the change set was created, in UTC.<br/>Parameters:  A list of <code>Parameter</code> structures that describes the input parameters and their values used to create the change set. For more information, see the <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/APIReference/API_Parameter.html">Parameter</a> data type.<br/>Description:  Information about the change set.<br/>StackName:  The name of the stack that is associated with the change set.<br/>StackId:  The ARN of the stack that is associated with the change set.<br/>ChangeSetId:  The ARN of the change set.<br/>ChangeSetName:  The name of the change set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringTimeInMinutes</i> The amount of time, in minutes, during which CloudFormation should monitor all the rollback triggers after the stack creation or update operation deploys all necessary resources. The default is 0 minutes. If you specify a monitoring period but do not specify any rollback triggers, CloudFormation still waits the specified period of time before cleaning up old resources after update operations. You can use this monitoring period to perform any manual stack validation desired, and manually cancel the stack creation or update (using <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/APIReference/API_CancelUpdateStack.html">CancelUpdateStack</a>, for example) as necessary. If you specify 0 for this parameter, CloudFormation still monitors the specified rollback triggers during stack creation and update operations. Then, for update operations, it begins disposing of old resources immediately once the operation completes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RollbackTriggers</i> The triggers to monitor during stack creation or update actions.  By default, AWS CloudFormation saves the rollback triggers specified for a stack and applies them to any subsequent update operations for the stack, unless you specify otherwise. If you do specify rollback triggers for this parameter, those triggers replace any list of triggers previously specified for the stack. This means: <ul> <li> To use the rollback triggers previously specified for this stack, if any, don't specify this parameter. </li> <li> To specify new or updated rollback triggers, you must specify <i>all</i> the triggers that you want used for this stack, even triggers you've specifed before (for example, when creating the stack or during a previous stack update). Any triggers that you don't include in the updated list of triggers are no longer applied to the stack. </li> <li> To remove all currently specified triggers, specify an empty list for this parameter. </li> </ul> If a specified trigger is missing, the entire stack operation fails and is rolled back. <br/>        """
        pass

    def describe_stack_drift_detection_status(self, StackDriftDetectionId: str) -> DescribeStackDriftDetectionStatusOutput:
        """Returns information about a stack drift detection operation. A stack drift detection operation detects whether a stack's actual configuration differs, or has <i>drifted</i>, from it's expected configuration, as defined in the stack template and any values specified as template parameters. A stack is considered to have drifted if one or more of its resources have drifted. For more information on stack and resource drift, see <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-cfn-stack-drift.html">Detecting Unregulated Configuration Changes to Stacks and Resources</a>. Use <a>DetectStackDrift</a> to initiate a stack drift detection operation. <code>DetectStackDrift</code> returns a <code>StackDriftDetectionId</code> you can use to monitor the progress of the operation using <code>DescribeStackDriftDetectionStatus</code>. Once the drift detection operation has completed, use <a>DescribeStackResourceDrifts</a> to return drift information about the stack and its resources.

        :param self:
        :param StackDriftDetectionId:  The ID of the drift detection results of this operation.  AWS CloudFormation generates new results, with a new drift detection ID, each time this operation is run. However, the number of drift results AWS CloudFormation retains for any given stack, and for how long, may vary. <br/>
        :return: Timestamp:  Time at which the stack drift detection operation was initiated.<br/>DriftedStackResourceCount:  Total number of stack resources that have drifted. This is NULL until the drift detection operation reaches a status of <code>DETECTION_COMPLETE</code>. This value will be 0 for stacks whose drift status is <code>IN_SYNC</code>.<br/>DetectionStatusReason:  The reason the stack drift detection operation has its current status.<br/>DetectionStatus:  The status of the stack drift detection operation. <ul> <li>  <code>DETECTION_COMPLETE</code>: The stack drift detection operation has successfully completed for all resources in the stack that support drift detection. (Resources that do not currently support stack detection remain unchecked.) If you specified logical resource IDs for AWS CloudFormation to use as a filter for the stack drift detection operation, only the resources with those logical IDs are checked for drift. </li> <li>  <code>DETECTION_FAILED</code>: The stack drift detection operation has failed for at least one resource in the stack. Results will be available for resources on which AWS CloudFormation successfully completed drift detection. </li> <li>  <code>DETECTION_IN_PROGRESS</code>: The stack drift detection operation is currently in progress. </li> </ul><br/>StackDriftStatus:  Status of the stack's actual configuration compared to its expected configuration.  <ul> <li>  <code>DRIFTED</code>: The stack differs from its expected template configuration. A stack is considered to have drifted if one or more of its resources have drifted. </li> <li>  <code>NOT_CHECKED</code>: AWS CloudFormation has not checked if the stack differs from its expected template configuration. </li> <li>  <code>IN_SYNC</code>: The stack's actual configuration matches its expected template configuration. </li> <li>  <code>UNKNOWN</code>: This value is reserved for future use. </li> </ul><br/>StackDriftDetectionId:  The ID of the drift detection results of this operation.  AWS CloudFormation generates new results, with a new drift detection ID, each time this operation is run. However, the number of reports AWS CloudFormation retains for any given stack, and for how long, may vary.<br/>StackId:  The ID of the stack.<br/>        """
        pass

    def describe_stack_events(self, StackName: str=None, NextToken: str=None) -> DescribeStackEventsOutput:
        """Returns all stack related events for a specified stack in reverse chronological order. For more information about a stack's event history, go to <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/concept-stack.html">Stacks</a> in the AWS CloudFormation User Guide. <note> You can list events for stacks that have failed to create or have been deleted by specifying the unique stack identifier (stack ID). </note>

        :param self:
        :param NextToken:  A string that identifies the next page of events that you want to retrieve.<br/>StackName:  The name or the unique stack ID that is associated with the stack, which are not always interchangeable: <ul> <li> Running stacks: You can specify either the stack's name or its unique stack ID. </li> <li> Deleted stacks: You must specify the unique stack ID. </li> </ul> Default: There is no default value.<br/>
        :return: NextToken:  If the output exceeds 1 MB in size, a string that identifies the next page of events. If no additional page exists, this value is null.<br/>StackEvents:  A list of <code>StackEvents</code> structures.<br/>        """
        pass

    def describe_stack_instance(self, StackInstanceRegion: str, StackInstanceAccount: str, StackSetName: str) -> DescribeStackInstanceOutput:
        """Returns the stack instance that's associated with the specified stack set, AWS account, and region. For a list of stack instances that are associated with a specific stack set, use <a>ListStackInstances</a>.

        :param self:
        :param StackInstanceRegion:  The name of a region that's associated with this stack instance.<br/>StackInstanceAccount:  The ID of an AWS account that's associated with this stack instance.<br/>StackSetName:  The name or the unique stack ID of the stack set that you want to get stack instance information for.<br/>
        :return: StackInstance:  The stack instance that matches the specified request parameters.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusReason</i> The explanation for the specific status code that is assigned to this stack instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the stack instance, in terms of its synchronization with its associated stack set. <ul> <li>  <code>INOPERABLE</code>: A <code>DeleteStackInstances</code> operation has failed and left the stack in an unstable state. Stacks in this state are excluded from further <code>UpdateStackSet</code> operations. You might need to perform a <code>DeleteStackInstances</code> operation, with <code>RetainStacks</code> set to <code>true</code>, to delete the stack instance, and then delete the stack manually. </li> <li>  <code>OUTDATED</code>: The stack isn't currently up to date with the stack set because: <ul> <li> The associated stack failed during a <code>CreateStackSet</code> or <code>UpdateStackSet</code> operation.  </li> <li> The stack was part of a <code>CreateStackSet</code> or <code>UpdateStackSet</code> operation that failed or was stopped before the stack was created or updated.  </li> </ul> </li> <li>  <code>CURRENT</code>: The stack is currently up to date with the stack set. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterOverrides</i> A list of parameters from the stack set template whose values have been overridden in this stack instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StackId</i> The ID of the stack instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Account</i> The name of the AWS account that the stack instance is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Region</i> The name of the AWS region that the stack instance is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StackSetId</i> The name or unique ID of the stack set that the stack instance is associated with.<br/>        """
        pass

    def describe_stack_resource(self, LogicalResourceId: str, StackName: str) -> DescribeStackResourceOutput:
        """Returns a description of the specified resource in the specified stack. For deleted stacks, DescribeStackResource returns resource information for up to 90 days after the stack has been deleted.

        :param self:
        :param LogicalResourceId:  The logical name of the resource as specified in the template. Default: There is no default value.<br/>StackName:  The name or the unique stack ID that is associated with the stack, which are not always interchangeable: <ul> <li> Running stacks: You can specify either the stack's name or its unique stack ID. </li> <li> Deleted stacks: You must specify the unique stack ID. </li> </ul> Default: There is no default value.<br/>
        :return: StackResourceDetail:  A <code>StackResourceDetail</code> structure containing the description of the specified resource in the specified stack.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DriftInformation</i> Information about whether the resource's actual configuration differs, or has <i>drifted</i>, from its expected configuration, as defined in the stack template and any values specified as template parameters. For more information, see <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-cfn-stack-drift.html">Detecting Unregulated Configuration Changes to Stacks and Resources</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DriftInformation</b>: Information about whether the resource's actual configuration differs, or has <i>drifted</i>, from its expected configuration, as defined in the stack template and any values specified as template parameters. For more information, see <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-cfn-stack-drift.html">Detecting Unregulated Configuration Changes to Stacks and Resources</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Metadata</i> The content of the <code>Metadata</code> attribute declared for the resource. For more information, see <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-attribute-metadata.html">Metadata Attribute</a> in the AWS CloudFormation User Guide.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> User defined description associated with the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceStatusReason</i> Success/failure message associated with the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceStatus</i> Current status of the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdatedTimestamp</i> Time the status was updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceType</i> Type of resource. ((For more information, go to <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-template-resource-type-ref.html"> AWS Resource Types Reference</a> in the AWS CloudFormation User Guide.)<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhysicalResourceId</i> The name or unique identifier that corresponds to a physical instance ID of a resource supported by AWS CloudFormation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogicalResourceId</i> The logical name of the resource specified in the template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StackId</i> Unique identifier of the stack.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StackName</i> The name associated with the stack.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LastCheckTimestamp</i> When AWS CloudFormation last checked if the resource had drifted from its expected configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StackResourceDriftStatus</i> Status of the resource's actual configuration compared to its expected configuration <ul> <li>  <code>DELETED</code>: The resource differs from its expected configuration in that it has been deleted. </li> <li>  <code>MODIFIED</code>: The resource differs from its expected configuration. </li> <li>  <code>NOT_CHECKED</code>: AWS CloudFormation has not checked if the resource differs from its expected configuration. Any resources that do not currently support drift detection have a status of <code>NOT_CHECKED</code>. For more information, see <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-cfn-stack-drift-resource-list.html">Resources that Support Drift Detection</a>.  </li> <li>  <code>IN_SYNC</code>: The resources's actual configuration matches its expected configuration. </li> </ul><br/>        """
        pass

    def describe_stack_resource_drifts(self, StackName: str, StackResourceDriftStatusFilters: List[StackResourceDriftStatus]=None, NextToken: str=None, MaxResults: int=None) -> DescribeStackResourceDriftsOutput:
        """Returns drift information for the resources that have been checked for drift in the specified stack. This includes actual and expected configuration values for resources where AWS CloudFormation detects configuration drift. For a given stack, there will be one <code>StackResourceDrift</code> for each stack resource that has been checked for drift. Resources that have not yet been checked for drift are not included. Resources that do not currently support drift detection are not checked, and so not included. For a list of resources that support drift detection, see <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-cfn-stack-drift-resource-list.html">Resources that Support Drift Detection</a>. Use <a>DetectStackResourceDrift</a> to detect drift on individual resources, or <a>DetectStackDrift</a> to detect drift on all supported resources for a given stack.

        :param self:
        :param MaxResults:  The maximum number of results to be returned with a single call. If the number of available results exceeds this maximum, the response includes a <code>NextToken</code> value that you can assign to the <code>NextToken</code> request parameter to get the next set of results.<br/>NextToken:  A string that identifies the next page of stack resource drift results.<br/>StackResourceDriftStatusFilters:  The resource drift status values to use as filters for the resource drift results returned. <ul> <li>  <code>DELETED</code>: The resource differs from its expected template configuration in that the resource has been deleted. </li> <li>  <code>MODIFIED</code>: One or more resource properties differ from their expected template values. </li> <li>  <code>IN_SYNC</code>: The resources's actual configuration matches its expected template configuration. </li> <li>  <code>NOT_CHECKED</code>: AWS CloudFormation does not currently return this value. </li> </ul><br/>StackName:  The name of the stack for which you want drift information.<br/>
        :return: NextToken:  If the request doesn't return all of the remaining results, <code>NextToken</code> is set to a token. To retrieve the next set of results, call <code>DescribeStackResourceDrifts</code> again and assign that token to the request object's <code>NextToken</code> parameter. If the request returns all results, <code>NextToken</code> is set to <code>null</code>.<br/>StackResourceDrifts:  Drift information for the resources that have been checked for drift in the specified stack. This includes actual and expected configuration values for resources where AWS CloudFormation detects drift. For a given stack, there will be one <code>StackResourceDrift</code> for each stack resource that has been checked for drift. Resources that have not yet been checked for drift are not included. Resources that do not currently support drift detection are not checked, and so not included. For a list of resources that support drift detection, see <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-cfn-stack-drift-resource-list.html">Resources that Support Drift Detection</a>.<br/>        """
        pass

    def describe_stack_resources(self, StackName: str=None, LogicalResourceId: str=None, PhysicalResourceId: str=None) -> DescribeStackResourcesOutput:
        """Returns AWS resource descriptions for running and deleted stacks. If <code>StackName</code> is specified, all the associated resources that are part of the stack are returned. If <code>PhysicalResourceId</code> is specified, the associated resources of the stack that the resource belongs to are returned. <note> Only the first 100 resources will be returned. If your stack has more resources than this, you should use <code>ListStackResources</code> instead. </note> For deleted stacks, <code>DescribeStackResources</code> returns resource information for up to 90 days after the stack has been deleted. You must specify either <code>StackName</code> or <code>PhysicalResourceId</code>, but not both. In addition, you can specify <code>LogicalResourceId</code> to filter the returned result. For more information about resources, the <code>LogicalResourceId</code> and <code>PhysicalResourceId</code>, go to the <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/">AWS CloudFormation User Guide</a>. <note> A <code>ValidationError</code> is returned if you specify both <code>StackName</code> and <code>PhysicalResourceId</code> in the same request. </note>

        :param self:
        :param PhysicalResourceId:  The name or unique identifier that corresponds to a physical instance ID of a resource supported by AWS CloudFormation. For example, for an Amazon Elastic Compute Cloud (EC2) instance, <code>PhysicalResourceId</code> corresponds to the <code>InstanceId</code>. You can pass the EC2 <code>InstanceId</code> to <code>DescribeStackResources</code> to find which stack the instance belongs to and what other resources are part of the stack. Required: Conditional. If you do not specify <code>PhysicalResourceId</code>, you must specify <code>StackName</code>. Default: There is no default value.<br/>LogicalResourceId:  The logical name of the resource as specified in the template. Default: There is no default value.<br/>StackName:  The name or the unique stack ID that is associated with the stack, which are not always interchangeable: <ul> <li> Running stacks: You can specify either the stack's name or its unique stack ID. </li> <li> Deleted stacks: You must specify the unique stack ID. </li> </ul> Default: There is no default value. Required: Conditional. If you do not specify <code>StackName</code>, you must specify <code>PhysicalResourceId</code>.<br/>
        :return: StackResources:  A list of <code>StackResource</code> structures.<br/>        """
        pass

    def describe_stack_set(self, StackSetName: str) -> DescribeStackSetOutput:
        """Returns the description of the specified stack set. 

        :param self:
        :param StackSetName:  The name or unique ID of the stack set whose description you want.<br/>
        :return: StackSet:  The specified stack set.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExecutionRoleName</i> The name of the IAM execution role used to create or update the stack set.  Use customized execution roles to control which stack resources users and groups can include in their stack sets. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdministrationRoleARN</i> The Amazon Resource Number (ARN) of the IAM role used to create or update the stack set. Use customized administrator roles to control which users or groups can manage specific stack sets within the same administrator account. For more information, see <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/stacksets-prereqs.html">Prerequisites: Granting Permissions for Stack Set Operations</a> in the <i>AWS CloudFormation User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StackSetARN</i> The Amazon Resource Number (ARN) of the stack set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> A list of tags that specify information about the stack set. A maximum number of 50 tags can be specified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Capabilities</i> The capabilities that are allowed in the stack set. Some stack set templates might include resources that can affect permissions in your AWS account—for example, by creating new AWS Identity and Access Management (IAM) users. For more information, see <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-iam-template.html#capabilities">Acknowledging IAM Resources in AWS CloudFormation Templates.</a> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> A list of input parameters for a stack set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TemplateBody</i> The structure that contains the body of the template that was used to create or update the stack set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the stack set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of the stack set that you specify when the stack set is created or updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StackSetId</i> The ID of the stack set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StackSetName</i> The name that's associated with the stack set.<br/>        """
        pass

    def describe_stack_set_operation(self, OperationId: str, StackSetName: str) -> DescribeStackSetOperationOutput:
        """Returns the description of the specified stack set operation. 

        :param self:
        :param OperationId:  The unique ID of the stack set operation. <br/>StackSetName:  The name or the unique stack ID of the stack set for the stack operation.<br/>
        :return: StackSetOperation:  The specified stack set operation.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTimestamp</i> The time at which the stack set operation ended, across all accounts and regions specified. Note that this doesn't necessarily mean that the stack set operation was successful, or even attempted, in each account or region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTimestamp</i> The time at which the operation was initiated. Note that the creation times for the stack set operation might differ from the creation time of the individual stacks themselves. This is because AWS CloudFormation needs to perform preparatory work for the operation, such as dispatching the work to the requested regions, before actually creating the first stacks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExecutionRoleName</i> The name of the IAM execution role used to create or update the stack set. Use customized execution roles to control which stack resources users and groups can include in their stack sets. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdministrationRoleARN</i> The Amazon Resource Number (ARN) of the IAM role used to perform this stack set operation.  Use customized administrator roles to control which users or groups can manage specific stack sets within the same administrator account. For more information, see <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/stacksets-prereqs.html">Define Permissions for Multiple Administrators</a> in the <i>AWS CloudFormation User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RetainStacks</i> For stack set operations of action type <code>DELETE</code>, specifies whether to remove the stack instances from the specified stack set, but doesn't delete the stacks. You can't reassociate a retained stack, or add an existing, saved stack to a new stack set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OperationPreferences</i> The preferences for how AWS CloudFormation performs this stack set operation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>OperationPreferences</b>: The preferences for how AWS CloudFormation performs this stack set operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the operation.  <ul> <li>  <code>FAILED</code>: The operation exceeded the specified failure tolerance. The failure tolerance value that you've set for an operation is applied for each region during stack create and update operations. If the number of failed stacks within a region exceeds the failure tolerance, the status of the operation in the region is set to <code>FAILED</code>. This in turn sets the status of the operation as a whole to <code>FAILED</code>, and AWS CloudFormation cancels the operation in any remaining regions. </li> <li>  <code>RUNNING</code>: The operation is currently being performed. </li> <li>  <code>STOPPED</code>: The user has cancelled the operation. </li> <li>  <code>STOPPING</code>: The operation is in the process of stopping, at user request.  </li> <li>  <code>SUCCEEDED</code>: The operation completed creating or updating all the specified stacks without exceeding the failure tolerance for the operation. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Action</i> The type of stack set operation: <code>CREATE</code>, <code>UPDATE</code>, or <code>DELETE</code>. Create and delete operations affect only the specified stack set instances that are associated with the specified stack set. Update operations affect both the stack set itself, as well as <i>all</i> associated stack set instances.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StackSetId</i> The ID of the stack set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OperationId</i> The unique ID of a stack set operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxConcurrentPercentage</i> The maximum percentage of accounts in which to perform this operation at one time. When calculating the number of accounts based on the specified percentage, AWS CloudFormation rounds down to the next whole number. This is true except in cases where rounding down would result is zero. In this case, CloudFormation sets the number as one instead. Note that this setting lets you specify the <i>maximum</i> for operations. For large deployments, under certain circumstances the actual number of accounts acted upon concurrently may be lower due to service throttling. Conditional: You must specify either <code>MaxConcurrentCount</code> or <code>MaxConcurrentPercentage</code>, but not both.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxConcurrentCount</i> The maximum number of accounts in which to perform this operation at one time. This is dependent on the value of <code>FailureToleranceCount</code>—<code>MaxConcurrentCount</code> is at most one more than the <code>FailureToleranceCount</code> . Note that this setting lets you specify the <i>maximum</i> for operations. For large deployments, under certain circumstances the actual number of accounts acted upon concurrently may be lower due to service throttling. Conditional: You must specify either <code>MaxConcurrentCount</code> or <code>MaxConcurrentPercentage</code>, but not both.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureTolerancePercentage</i> The percentage of accounts, per region, for which this stack operation can fail before AWS CloudFormation stops the operation in that region. If the operation is stopped in a region, AWS CloudFormation doesn't attempt the operation in any subsequent regions. When calculating the number of accounts based on the specified percentage, AWS CloudFormation rounds <i>down</i> to the next whole number. Conditional: You must specify either <code>FailureToleranceCount</code> or <code>FailureTolerancePercentage</code>, but not both.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureToleranceCount</i> The number of accounts, per region, for which this operation can fail before AWS CloudFormation stops the operation in that region. If the operation is stopped in a region, AWS CloudFormation doesn't attempt the operation in any subsequent regions. Conditional: You must specify either <code>FailureToleranceCount</code> or <code>FailureTolerancePercentage</code> (but not both).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RegionOrder</i> The order of the regions in where you want to perform the stack operation.<br/>        """
        pass

    def describe_stacks(self, StackName: str=None, NextToken: str=None) -> DescribeStacksOutput:
        """Returns the description for the specified stack; if no stack name was specified, then it returns the description for all the stacks created. <note> If the stack does not exist, an <code>AmazonCloudFormationException</code> is returned. </note>

        :param self:
        :param NextToken:  A string that identifies the next page of stacks that you want to retrieve.<br/>StackName:  The name or the unique stack ID that is associated with the stack, which are not always interchangeable: <ul> <li> Running stacks: You can specify either the stack's name or its unique stack ID. </li> <li> Deleted stacks: You must specify the unique stack ID. </li> </ul> Default: There is no default value.<br/>
        :return: NextToken:  If the output exceeds 1 MB in size, a string that identifies the next page of stacks. If no additional page exists, this value is null.<br/>Stacks:  A list of stack structures.<br/>        """
        pass

    def detect_stack_drift(self, StackName: str, LogicalResourceIds: List[LogicalResourceId]=None) -> DetectStackDriftOutput:
        """Detects whether a stack's actual configuration differs, or has <i>drifted</i>, from it's expected configuration, as defined in the stack template and any values specified as template parameters. For each resource in the stack that supports drift detection, AWS CloudFormation compares the actual configuration of the resource with its expected template configuration. Only resource properties explicitly defined in the stack template are checked for drift. A stack is considered to have drifted if one or more of its resources differ from their expected template configurations. For more information, see <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-cfn-stack-drift.html">Detecting Unregulated Configuration Changes to Stacks and Resources</a>. Use <code>DetectStackDrift</code> to detect drift on all supported resources for a given stack, or <a>DetectStackResourceDrift</a> to detect drift on individual resources. For a list of stack resources that currently support drift detection, see <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-cfn-stack-drift-resource-list.html">Resources that Support Drift Detection</a>.  <code>DetectStackDrift</code> can take up to several minutes, depending on the number of resources contained within the stack. Use <a>DescribeStackDriftDetectionStatus</a> to monitor the progress of a detect stack drift operation. Once the drift detection operation has completed, use <a>DescribeStackResourceDrifts</a> to return drift information about the stack and its resources. When detecting drift on a stack, AWS CloudFormation does not detect drift on any nested stacks belonging to that stack. Perform <code>DetectStackDrift</code> directly on the nested stack itself.

        :param self:
        :param LogicalResourceIds:  The logical names of any resources you want to use as filters.<br/>StackName:  The name of the stack for which you want to detect drift. <br/>
        :return: StackDriftDetectionId:  The ID of the drift detection results of this operation.  AWS CloudFormation generates new results, with a new drift detection ID, each time this operation is run. However, the number of drift results AWS CloudFormation retains for any given stack, and for how long, may vary. <br/>        """
        pass

    def detect_stack_resource_drift(self, LogicalResourceId: str, StackName: str) -> DetectStackResourceDriftOutput:
        """Returns information about whether a resource's actual configuration differs, or has <i>drifted</i>, from it's expected configuration, as defined in the stack template and any values specified as template parameters. This information includes actual and expected property values for resources in which AWS CloudFormation detects drift. Only resource properties explicitly defined in the stack template are checked for drift. For more information about stack and resource drift, see <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-cfn-stack-drift.html">Detecting Unregulated Configuration Changes to Stacks and Resources</a>. Use <code>DetectStackResourceDrift</code> to detect drift on individual resources, or <a>DetectStackDrift</a> to detect drift on all resources in a given stack that support drift detection. Resources that do not currently support drift detection cannot be checked. For a list of resources that support drift detection, see <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-cfn-stack-drift-resource-list.html">Resources that Support Drift Detection</a>.

        :param self:
        :param LogicalResourceId:  The logical name of the resource for which to return drift information.<br/>StackName:  The name of the stack to which the resource belongs.<br/>
        :return: StackResourceDrift:  Information about whether the resource's actual configuration has drifted from its expected template configuration, including actual and expected property values and any differences detected.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timestamp</i> Time at which AWS CloudFormation performed drift detection on the stack resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StackResourceDriftStatus</i> Status of the resource's actual configuration compared to its expected configuration <ul> <li>  <code>DELETED</code>: The resource differs from its expected template configuration because the resource has been deleted. </li> <li>  <code>MODIFIED</code>: One or more resource properties differ from their expected values (as defined in the stack template and any values specified as template parameters). </li> <li>  <code>IN_SYNC</code>: The resources's actual configuration matches its expected template configuration. </li> <li>  <code>NOT_CHECKED</code>: AWS CloudFormation does not currently return this value. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PropertyDifferences</i> A collection of the resource properties whose actual values differ from their expected values. These will be present only for resources whose <code>StackResourceDriftStatus</code> is <code>MODIFIED</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActualProperties</i> A JSON structure containing the actual property values of the stack resource. For resources whose <code>StackResourceDriftStatus</code> is <code>DELETED</code>, this structure will not be present. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpectedProperties</i> A JSON structure containing the expected property values of the stack resource, as defined in the stack template and any values specified as template parameters.  For resources whose <code>StackResourceDriftStatus</code> is <code>DELETED</code>, this structure will not be present. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceType</i> The type of the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhysicalResourceIdContext</i> Context information that enables AWS CloudFormation to uniquely identify a resource. AWS CloudFormation uses context key-value pairs in cases where a resource's logical and physical IDs are not enough to uniquely identify that resource. Each context key-value pair specifies a unique resource that contains the targeted resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhysicalResourceId</i> The name or unique identifier that corresponds to a physical instance ID of a resource supported by AWS CloudFormation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogicalResourceId</i> The logical name of the resource specified in the template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StackId</i> The ID of the stack.<br/>        """
        pass

    def estimate_template_cost(self, TemplateBody: str=None, TemplateURL: str=None, Parameters: List[Parameter]=None) -> EstimateTemplateCostOutput:
        """Returns the estimated monthly cost of a template. The return value is an AWS Simple Monthly Calculator URL with a query string that describes the resources required to run the template.

        :param self:
        :param Parameters:  A list of <code>Parameter</code> structures that specify input parameters.<br/>TemplateURL:  Location of file containing the template body. The URL must point to a template that is located in an Amazon S3 bucket. For more information, go to <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/template-anatomy.html">Template Anatomy</a> in the AWS CloudFormation User Guide. Conditional: You must pass <code>TemplateURL</code> or <code>TemplateBody</code>. If both are passed, only <code>TemplateBody</code> is used.<br/>TemplateBody:  Structure containing the template body with a minimum length of 1 byte and a maximum length of 51,200 bytes. (For more information, go to <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/template-anatomy.html">Template Anatomy</a> in the AWS CloudFormation User Guide.) Conditional: You must pass <code>TemplateBody</code> or <code>TemplateURL</code>. If both are passed, only <code>TemplateBody</code> is used.<br/>
        :return: Url:  An AWS Simple Monthly Calculator URL with a query string that describes the resources required to run the template.<br/>        """
        pass

    def execute_change_set(self, ChangeSetName: str, StackName: str=None, ClientRequestToken: str=None) -> ExecuteChangeSetOutput:
        """Updates a stack using the input information that was provided when the specified change set was created. After the call successfully completes, AWS CloudFormation starts updating the stack. Use the <a>DescribeStacks</a> action to view the status of the update. When you execute a change set, AWS CloudFormation deletes all other change sets associated with the stack because they aren't valid for the updated stack. If a stack policy is associated with the stack, AWS CloudFormation enforces the policy during the update. You can't specify a temporary stack policy that overrides the current policy.

        :param self:
        :param ClientRequestToken:  A unique identifier for this <code>ExecuteChangeSet</code> request. Specify this token if you plan to retry requests so that AWS CloudFormation knows that you're not attempting to execute a change set to update a stack with the same name. You might retry <code>ExecuteChangeSet</code> requests to ensure that AWS CloudFormation successfully received them.<br/>StackName:  If you specified the name of a change set, specify the stack name or ID (ARN) that is associated with the change set you want to execute.<br/>ChangeSetName:  The name or ARN of the change set that you want use to update the specified stack.<br/>
        :return:         """
        pass

    def get_stack_policy(self, StackName: str) -> GetStackPolicyOutput:
        """Returns the stack policy for a specified stack. If a stack doesn't have a policy, a null value is returned.

        :param self:
        :param StackName:  The name or unique stack ID that is associated with the stack whose policy you want to get.<br/>
        :return: StackPolicyBody:  Structure containing the stack policy body. (For more information, go to <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/protect-stack-resources.html"> Prevent Updates to Stack Resources</a> in the AWS CloudFormation User Guide.)<br/>        """
        pass

    def get_template(self, StackName: str=None, ChangeSetName: str=None, TemplateStage: str=None) -> GetTemplateOutput:
        """Returns the template body for a specified stack. You can get the template for running or deleted stacks. For deleted stacks, GetTemplate returns the template for up to 90 days after the stack has been deleted. <note>  If the template does not exist, a <code>ValidationError</code> is returned.  </note>

        :param self:
        :param TemplateStage:  For templates that include transforms, the stage of the template that AWS CloudFormation returns. To get the user-submitted template, specify <code>Original</code>. To get the template after AWS CloudFormation has processed all transforms, specify <code>Processed</code>.  If the template doesn't include transforms, <code>Original</code> and <code>Processed</code> return the same template. By default, AWS CloudFormation specifies <code>Original</code>. <br/>ChangeSetName:  The name or Amazon Resource Name (ARN) of a change set for which AWS CloudFormation returns the associated template. If you specify a name, you must also specify the <code>StackName</code>.<br/>StackName:  The name or the unique stack ID that is associated with the stack, which are not always interchangeable: <ul> <li> Running stacks: You can specify either the stack's name or its unique stack ID. </li> <li> Deleted stacks: You must specify the unique stack ID. </li> </ul> Default: There is no default value.<br/>
        :return: StagesAvailable:  The stage of the template that you can retrieve. For stacks, the <code>Original</code> and <code>Processed</code> templates are always available. For change sets, the <code>Original</code> template is always available. After AWS CloudFormation finishes creating the change set, the <code>Processed</code> template becomes available.<br/>TemplateBody:  Structure containing the template body. (For more information, go to <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/template-anatomy.html">Template Anatomy</a> in the AWS CloudFormation User Guide.) AWS CloudFormation returns the same template that was used when the stack was created.<br/>        """
        pass

    def get_template_summary(self, TemplateBody: str=None, TemplateURL: str=None, StackName: str=None, StackSetName: str=None) -> GetTemplateSummaryOutput:
        """Returns information about a new or existing template. The <code>GetTemplateSummary</code> action is useful for viewing parameter information, such as default parameter values and parameter types, before you create or update a stack or stack set. You can use the <code>GetTemplateSummary</code> action when you submit a template, or you can get template information for a stack set, or a running or deleted stack. For deleted stacks, <code>GetTemplateSummary</code> returns the template information for up to 90 days after the stack has been deleted. If the template does not exist, a <code>ValidationError</code> is returned.

        :param self:
        :param StackSetName:  The name or unique ID of the stack set from which the stack was created. Conditional: You must specify only one of the following parameters: <code>StackName</code>, <code>StackSetName</code>, <code>TemplateBody</code>, or <code>TemplateURL</code>.<br/>StackName:  The name or the stack ID that is associated with the stack, which are not always interchangeable. For running stacks, you can specify either the stack's name or its unique stack ID. For deleted stack, you must specify the unique stack ID. Conditional: You must specify only one of the following parameters: <code>StackName</code>, <code>StackSetName</code>, <code>TemplateBody</code>, or <code>TemplateURL</code>.<br/>TemplateURL:  Location of file containing the template body. The URL must point to a template (max size: 460,800 bytes) that is located in an Amazon S3 bucket. For more information about templates, see <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/template-anatomy.html">Template Anatomy</a> in the AWS CloudFormation User Guide. Conditional: You must specify only one of the following parameters: <code>StackName</code>, <code>StackSetName</code>, <code>TemplateBody</code>, or <code>TemplateURL</code>.<br/>TemplateBody:  Structure containing the template body with a minimum length of 1 byte and a maximum length of 51,200 bytes. For more information about templates, see <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/template-anatomy.html">Template Anatomy</a> in the AWS CloudFormation User Guide. Conditional: You must specify only one of the following parameters: <code>StackName</code>, <code>StackSetName</code>, <code>TemplateBody</code>, or <code>TemplateURL</code>.<br/>
        :return: DeclaredTransforms:  A list of the transforms that are declared in the template.<br/>Metadata:  The value that is defined for the <code>Metadata</code> property of the template.<br/>Version:  The AWS template format version, which identifies the capabilities of the template.<br/>ResourceTypes:  A list of all the template resource types that are defined in the template, such as <code>AWS::EC2::Instance</code>, <code>AWS::Dynamo::Table</code>, and <code>Custom::MyCustomInstance</code>.<br/>CapabilitiesReason:  The list of resources that generated the values in the <code>Capabilities</code> response element.<br/>Capabilities:  The capabilities found within the template. If your template contains IAM resources, you must specify the CAPABILITY_IAM or CAPABILITY_NAMED_IAM value for this parameter when you use the <a>CreateStack</a> or <a>UpdateStack</a> actions with your template; otherwise, those actions return an InsufficientCapabilities error. For more information, see <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-iam-template.html#capabilities">Acknowledging IAM Resources in AWS CloudFormation Templates</a>.<br/>Description:  The value that is defined in the <code>Description</code> property of the template.<br/>Parameters:  A list of parameter declarations that describe various properties for each parameter.<br/>        """
        pass

    def list_change_sets(self, StackName: str, NextToken: str=None) -> ListChangeSetsOutput:
        """Returns the ID and status of each active change set for a stack. For example, AWS CloudFormation lists change sets that are in the <code>CREATE_IN_PROGRESS</code> or <code>CREATE_PENDING</code> state.

        :param self:
        :param NextToken:  A string (provided by the <a>ListChangeSets</a> response output) that identifies the next page of change sets that you want to retrieve.<br/>StackName:  The name or the Amazon Resource Name (ARN) of the stack for which you want to list change sets.<br/>
        :return: NextToken:  If the output exceeds 1 MB, a string that identifies the next page of change sets. If there is no additional page, this value is null.<br/>Summaries:  A list of <code>ChangeSetSummary</code> structures that provides the ID and status of each change set for the specified stack.<br/>        """
        pass

    def list_exports(self, NextToken: str=None) -> ListExportsOutput:
        """Lists all exported output values in the account and region in which you call this action. Use this action to see the exported output values that you can import into other stacks. To import values, use the <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/intrinsic-function-reference-importvalue.html"> <code>Fn::ImportValue</code> </a> function.  For more information, see <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-cfn-stack-exports.html"> AWS CloudFormation Export Stack Output Values</a>.

        :param self:
        :param NextToken:  A string (provided by the <a>ListExports</a> response output) that identifies the next page of exported output values that you asked to retrieve.<br/>
        :return: NextToken:  If the output exceeds 100 exported output values, a string that identifies the next page of exports. If there is no additional page, this value is null.<br/>Exports:  The output for the <a>ListExports</a> action.<br/>        """
        pass

    def list_imports(self, ExportName: str, NextToken: str=None) -> ListImportsOutput:
        """Lists all stacks that are importing an exported output value. To modify or remove an exported output value, first use this action to see which stacks are using it. To see the exported output values in your account, see <a>ListExports</a>.  For more information about importing an exported output value, see the <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/intrinsic-function-reference-importvalue.html"> <code>Fn::ImportValue</code> </a> function. 

        :param self:
        :param NextToken:  A string (provided by the <a>ListImports</a> response output) that identifies the next page of stacks that are importing the specified exported output value. <br/>ExportName:  The name of the exported output value. AWS CloudFormation returns the stack names that are importing this value. <br/>
        :return: NextToken:  A string that identifies the next page of exports. If there is no additional page, this value is null.<br/>Imports:  A list of stack names that are importing the specified exported output value. <br/>        """
        pass

    def list_stack_instances(self, StackSetName: str, NextToken: str=None, MaxResults: int=None, StackInstanceAccount: str=None, StackInstanceRegion: str=None) -> ListStackInstancesOutput:
        """Returns summary information about stack instances that are associated with the specified stack set. You can filter for stack instances that are associated with a specific AWS account name or region.

        :param self:
        :param StackInstanceRegion:  The name of the region where you want to list stack instances. <br/>StackInstanceAccount:  The name of the AWS account that you want to list stack instances for.<br/>MaxResults:  The maximum number of results to be returned with a single call. If the number of available results exceeds this maximum, the response includes a <code>NextToken</code> value that you can assign to the <code>NextToken</code> request parameter to get the next set of results.<br/>NextToken:  If the previous request didn't return all of the remaining results, the response's <code>NextToken</code> parameter value is set to a token. To retrieve the next set of results, call <code>ListStackInstances</code> again and assign that token to the request object's <code>NextToken</code> parameter. If there are no remaining results, the previous response object's <code>NextToken</code> parameter is set to <code>null</code>.<br/>StackSetName:  The name or unique ID of the stack set that you want to list stack instances for.<br/>
        :return: NextToken:  If the request doesn't return all of the remaining results, <code>NextToken</code> is set to a token. To retrieve the next set of results, call <code>ListStackInstances</code> again and assign that token to the request object's <code>NextToken</code> parameter. If the request returns all results, <code>NextToken</code> is set to <code>null</code>.<br/>Summaries:  A list of <code>StackInstanceSummary</code> structures that contain information about the specified stack instances.<br/>        """
        pass

    def list_stack_resources(self, StackName: str, NextToken: str=None) -> ListStackResourcesOutput:
        """Returns descriptions of all resources of the specified stack. For deleted stacks, ListStackResources returns resource information for up to 90 days after the stack has been deleted.

        :param self:
        :param NextToken:  A string that identifies the next page of stack resources that you want to retrieve.<br/>StackName:  The name or the unique stack ID that is associated with the stack, which are not always interchangeable: <ul> <li> Running stacks: You can specify either the stack's name or its unique stack ID. </li> <li> Deleted stacks: You must specify the unique stack ID. </li> </ul> Default: There is no default value.<br/>
        :return: NextToken:  If the output exceeds 1 MB, a string that identifies the next page of stack resources. If no additional page exists, this value is null.<br/>StackResourceSummaries:  A list of <code>StackResourceSummary</code> structures.<br/>        """
        pass

    def list_stack_set_operation_results(self, OperationId: str, StackSetName: str, NextToken: str=None, MaxResults: int=None) -> ListStackSetOperationResultsOutput:
        """Returns summary information about the results of a stack set operation. 

        :param self:
        :param MaxResults:  The maximum number of results to be returned with a single call. If the number of available results exceeds this maximum, the response includes a <code>NextToken</code> value that you can assign to the <code>NextToken</code> request parameter to get the next set of results.<br/>NextToken:  If the previous request didn't return all of the remaining results, the response object's <code>NextToken</code> parameter value is set to a token. To retrieve the next set of results, call <code>ListStackSetOperationResults</code> again and assign that token to the request object's <code>NextToken</code> parameter. If there are no remaining results, the previous response object's <code>NextToken</code> parameter is set to <code>null</code>.<br/>OperationId:  The ID of the stack set operation.<br/>StackSetName:  The name or unique ID of the stack set that you want to get operation results for.<br/>
        :return: NextToken:  If the request doesn't return all results, <code>NextToken</code> is set to a token. To retrieve the next set of results, call <code>ListOperationResults</code> again and assign that token to the request object's <code>NextToken</code> parameter. If there are no remaining results, <code>NextToken</code> is set to <code>null</code>.<br/>Summaries:  A list of <code>StackSetOperationResultSummary</code> structures that contain information about the specified operation results, for accounts and regions that are included in the operation.<br/>        """
        pass

    def list_stack_set_operations(self, StackSetName: str, NextToken: str=None, MaxResults: int=None) -> ListStackSetOperationsOutput:
        """Returns summary information about operations performed on a stack set. 

        :param self:
        :param MaxResults:  The maximum number of results to be returned with a single call. If the number of available results exceeds this maximum, the response includes a <code>NextToken</code> value that you can assign to the <code>NextToken</code> request parameter to get the next set of results.<br/>NextToken:  If the previous paginated request didn't return all of the remaining results, the response object's <code>NextToken</code> parameter value is set to a token. To retrieve the next set of results, call <code>ListStackSetOperations</code> again and assign that token to the request object's <code>NextToken</code> parameter. If there are no remaining results, the previous response object's <code>NextToken</code> parameter is set to <code>null</code>.<br/>StackSetName:  The name or unique ID of the stack set that you want to get operation summaries for.<br/>
        :return: NextToken:  If the request doesn't return all results, <code>NextToken</code> is set to a token. To retrieve the next set of results, call <code>ListOperationResults</code> again and assign that token to the request object's <code>NextToken</code> parameter. If there are no remaining results, <code>NextToken</code> is set to <code>null</code>.<br/>Summaries:  A list of <code>StackSetOperationSummary</code> structures that contain summary information about operations for the specified stack set.<br/>        """
        pass

    def list_stack_sets(self, NextToken: str=None, MaxResults: int=None, Status: str=None) -> ListStackSetsOutput:
        """Returns summary information about stack sets that are associated with the user.

        :param self:
        :param Status:  The status of the stack sets that you want to get summary information about.<br/>MaxResults:  The maximum number of results to be returned with a single call. If the number of available results exceeds this maximum, the response includes a <code>NextToken</code> value that you can assign to the <code>NextToken</code> request parameter to get the next set of results.<br/>NextToken:  If the previous paginated request didn't return all of the remaining results, the response object's <code>NextToken</code> parameter value is set to a token. To retrieve the next set of results, call <code>ListStackSets</code> again and assign that token to the request object's <code>NextToken</code> parameter. If there are no remaining results, the previous response object's <code>NextToken</code> parameter is set to <code>null</code>.<br/>
        :return: NextToken:  If the request doesn't return all of the remaining results, <code>NextToken</code> is set to a token. To retrieve the next set of results, call <code>ListStackInstances</code> again and assign that token to the request object's <code>NextToken</code> parameter. If the request returns all results, <code>NextToken</code> is set to <code>null</code>.<br/>Summaries:  A list of <code>StackSetSummary</code> structures that contain information about the user's stack sets.<br/>        """
        pass

    def list_stacks(self, NextToken: str=None, StackStatusFilter: List[StackStatus]=None) -> ListStacksOutput:
        """Returns the summary information for stacks whose status matches the specified StackStatusFilter. Summary information for stacks that have been deleted is kept for 90 days after the stack is deleted. If no StackStatusFilter is specified, summary information for all stacks is returned (including existing stacks and stacks that have been deleted).

        :param self:
        :param StackStatusFilter:  Stack status to use as a filter. Specify one or more stack status codes to list only stacks with the specified status codes. For a complete list of stack status codes, see the <code>StackStatus</code> parameter of the <a>Stack</a> data type.<br/>NextToken:  A string that identifies the next page of stacks that you want to retrieve.<br/>
        :return: NextToken:  If the output exceeds 1 MB in size, a string that identifies the next page of stacks. If no additional page exists, this value is null.<br/>StackSummaries:  A list of <code>StackSummary</code> structures containing information about the specified stacks.<br/>        """
        pass

    def set_stack_policy(self, StackName: str, StackPolicyBody: str=None, StackPolicyURL: str=None):
        """Sets a stack policy for a specified stack.

        :param self:
        :param StackPolicyURL:  Location of a file containing the stack policy. The URL must point to a policy (maximum size: 16 KB) located in an S3 bucket in the same region as the stack. You can specify either the <code>StackPolicyBody</code> or the <code>StackPolicyURL</code> parameter, but not both.<br/>StackPolicyBody:  Structure containing the stack policy body. For more information, go to <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/protect-stack-resources.html"> Prevent Updates to Stack Resources</a> in the AWS CloudFormation User Guide. You can specify either the <code>StackPolicyBody</code> or the <code>StackPolicyURL</code> parameter, but not both.<br/>StackName:  The name or unique stack ID that you want to associate a policy with.<br/>
        :return:         """
        pass

    def signal_resource(self, Status: str, UniqueId: str, LogicalResourceId: str, StackName: str):
        """Sends a signal to the specified resource with a success or failure status. You can use the SignalResource API in conjunction with a creation policy or update policy. AWS CloudFormation doesn't proceed with a stack creation or update until resources receive the required number of signals or the timeout period is exceeded. The SignalResource API is useful in cases where you want to send signals from anywhere other than an Amazon EC2 instance.

        :param self:
        :param Status:  The status of the signal, which is either success or failure. A failure signal causes AWS CloudFormation to immediately fail the stack creation or update.<br/>UniqueId:  A unique ID of the signal. When you signal Amazon EC2 instances or Auto Scaling groups, specify the instance ID that you are signaling as the unique ID. If you send multiple signals to a single resource (such as signaling a wait condition), each signal requires a different unique ID.<br/>LogicalResourceId:  The logical ID of the resource that you want to signal. The logical ID is the name of the resource that given in the template.<br/>StackName:  The stack name or unique stack ID that includes the resource that you want to signal.<br/>
        :return:         """
        pass

    def stop_stack_set_operation(self, OperationId: str, StackSetName: str) -> StopStackSetOperationOutput:
        """Stops an in-progress operation on a stack set and its associated stack instances. 

        :param self:
        :param OperationId:  The ID of the stack operation. <br/>StackSetName:  The name or unique ID of the stack set that you want to stop the operation for.<br/>
        :return:         """
        pass

    def update_stack(self, StackName: str, TemplateBody: str=None, TemplateURL: str=None, UsePreviousTemplate: UsePreviousTemplate=None, StackPolicyDuringUpdateBody: str=None, StackPolicyDuringUpdateURL: str=None, Parameters: List[Parameter]=None, Capabilities: List[Capability]=None, ResourceTypes: List[ResourceType]=None, RoleARN: str=None, RollbackConfiguration: RollbackConfiguration=None, StackPolicyBody: str=None, StackPolicyURL: str=None, NotificationARNs: List[NotificationARN]=None, Tags: List[Tag]=None, ClientRequestToken: str=None) -> UpdateStackOutput:
        """Updates a stack as specified in the template. After the call completes successfully, the stack update starts. You can check the status of the stack via the <a>DescribeStacks</a> action. To get a copy of the template for an existing stack, you can use the <a>GetTemplate</a> action. For more information about creating an update template, updating a stack, and monitoring the progress of the update, see <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-cfn-updating-stacks.html">Updating a Stack</a>.

        :param self:
        :param ClientRequestToken:  A unique identifier for this <code>UpdateStack</code> request. Specify this token if you plan to retry requests so that AWS CloudFormation knows that you're not attempting to update a stack with the same name. You might retry <code>UpdateStack</code> requests to ensure that AWS CloudFormation successfully received them. All events triggered by a given stack operation are assigned the same client request token, which you can use to track operations. For example, if you execute a <code>CreateStack</code> operation with the token <code>token1</code>, then all the <code>StackEvents</code> generated by that operation will have <code>ClientRequestToken</code> set as <code>token1</code>. In the console, stack operations display the client request token on the Events tab. Stack operations that are initiated from the console use the token format <i>Console-StackOperation-ID</i>, which helps you easily identify the stack operation . For example, if you create a stack using the console, each stack event would be assigned the same token in the following format: <code>Console-CreateStack-7f59c3cf-00d2-40c7-b2ff-e75db0987002</code>. <br/>Tags:  Key-value pairs to associate with this stack. AWS CloudFormation also propagates these tags to supported resources in the stack. You can specify a maximum number of 50 tags. If you don't specify this parameter, AWS CloudFormation doesn't modify the stack's tags. If you specify an empty value, AWS CloudFormation removes all associated tags.<br/>NotificationARNs:  Amazon Simple Notification Service topic Amazon Resource Names (ARNs) that AWS CloudFormation associates with the stack. Specify an empty list to remove all notification topics.<br/>StackPolicyURL:  Location of a file containing the updated stack policy. The URL must point to a policy (max size: 16KB) located in an S3 bucket in the same region as the stack. You can specify either the <code>StackPolicyBody</code> or the <code>StackPolicyURL</code> parameter, but not both. You might update the stack policy, for example, in order to protect a new resource that you created during a stack update. If you do not specify a stack policy, the current policy that is associated with the stack is unchanged.<br/>StackPolicyBody:  Structure containing a new stack policy body. You can specify either the <code>StackPolicyBody</code> or the <code>StackPolicyURL</code> parameter, but not both. You might update the stack policy, for example, in order to protect a new resource that you created during a stack update. If you do not specify a stack policy, the current policy that is associated with the stack is unchanged.<br/>RollbackConfiguration:  The rollback triggers for AWS CloudFormation to monitor during stack creation and updating operations, and for the specified monitoring period afterwards.:<br/>RoleARN:  The Amazon Resource Name (ARN) of an AWS Identity and Access Management (IAM) role that AWS CloudFormation assumes to update the stack. AWS CloudFormation uses the role's credentials to make calls on your behalf. AWS CloudFormation always uses this role for all future operations on the stack. As long as users have permission to operate on the stack, AWS CloudFormation uses this role even if the users don't have permission to pass it. Ensure that the role grants least privilege. If you don't specify a value, AWS CloudFormation uses the role that was previously associated with the stack. If no role is available, AWS CloudFormation uses a temporary session that is generated from your user credentials.<br/>ResourceTypes:  The template resource types that you have permissions to work with for this update stack action, such as <code>AWS::EC2::Instance</code>, <code>AWS::EC2::*</code>, or <code>Custom::MyCustomInstance</code>. If the list of resource types doesn't include a resource that you're updating, the stack update fails. By default, AWS CloudFormation grants permissions to all resource types. AWS Identity and Access Management (IAM) uses this parameter for AWS CloudFormation-specific condition keys in IAM policies. For more information, see <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-iam-template.html">Controlling Access with AWS Identity and Access Management</a>.<br/>Capabilities:  In some cases, you must explicity acknowledge that your stack template contains certain capabilities in order for AWS CloudFormation to update the stack. <ul> <li>  <code>CAPABILITY_IAM</code> and <code>CAPABILITY_NAMED_IAM</code>  Some stack templates might include resources that can affect permissions in your AWS account; for example, by creating new AWS Identity and Access Management (IAM) users. For those stacks, you must explicitly acknowledge this by specifying one of these capabilities. The following IAM resources require you to specify either the <code>CAPABILITY_IAM</code> or <code>CAPABILITY_NAMED_IAM</code> capability. <ul> <li> If you have IAM resources, you can specify either capability.  </li> <li> If you have IAM resources with custom names, you <i>must</i> specify <code>CAPABILITY_NAMED_IAM</code>.  </li> <li> If you don't specify either of these capabilities, AWS CloudFormation returns an <code>InsufficientCapabilities</code> error. </li> </ul> If your stack template contains these resources, we recommend that you review all permissions associated with them and edit their permissions if necessary. <ul> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-accesskey.html"> AWS::IAM::AccessKey</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-group.html"> AWS::IAM::Group</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-instanceprofile.html"> AWS::IAM::InstanceProfile</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-policy.html"> AWS::IAM::Policy</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-role.html"> AWS::IAM::Role</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-user.html"> AWS::IAM::User</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-addusertogroup.html"> AWS::IAM::UserToGroupAddition</a>  </li> </ul> For more information, see <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-iam-template.html#capabilities">Acknowledging IAM Resources in AWS CloudFormation Templates</a>. </li> <li>  <code>CAPABILITY_AUTO_EXPAND</code>  Some template contain macros. Macros perform custom processing on templates; this can include simple actions like find-and-replace operations, all the way to extensive transformations of entire templates. Because of this, users typically create a change set from the processed template, so that they can review the changes resulting from the macros before actually updating the stack. If your stack template contains one or more macros, and you choose to update a stack directly from the processed template, without first reviewing the resulting changes in a change set, you must acknowledge this capability. This includes the <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/create-reusable-transform-function-snippets-and-add-to-your-template-with-aws-include-transform.html">AWS::Include</a> and <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/transform-aws-serverless.html">AWS::Serverless</a> transforms, which are macros hosted by AWS CloudFormation. Change sets do not currently support nested stacks. If you want to update a stack from a stack template that contains macros <i>and</i> nested stacks, you must update the stack directly from the template using this capability. <important> You should only update stacks directly from a stack template that contains macros if you know what processing the macro performs. Each macro relies on an underlying Lambda service function for processing stack templates. Be aware that the Lambda function owner can update the function operation without AWS CloudFormation being notified. </important> For more information, see <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/template-macros.html">Using AWS CloudFormation Macros to Perform Custom Processing on Templates</a>. </li> </ul><br/>Parameters:  A list of <code>Parameter</code> structures that specify input parameters for the stack. For more information, see the <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/APIReference/API_Parameter.html">Parameter</a> data type.<br/>StackPolicyDuringUpdateURL:  Location of a file containing the temporary overriding stack policy. The URL must point to a policy (max size: 16KB) located in an S3 bucket in the same region as the stack. You can specify either the <code>StackPolicyDuringUpdateBody</code> or the <code>StackPolicyDuringUpdateURL</code> parameter, but not both. If you want to update protected resources, specify a temporary overriding stack policy during this update. If you do not specify a stack policy, the current policy that is associated with the stack will be used.<br/>StackPolicyDuringUpdateBody:  Structure containing the temporary overriding stack policy body. You can specify either the <code>StackPolicyDuringUpdateBody</code> or the <code>StackPolicyDuringUpdateURL</code> parameter, but not both. If you want to update protected resources, specify a temporary overriding stack policy during this update. If you do not specify a stack policy, the current policy that is associated with the stack will be used.<br/>UsePreviousTemplate:  Reuse the existing template that is associated with the stack that you are updating. Conditional: You must specify only one of the following parameters: <code>TemplateBody</code>, <code>TemplateURL</code>, or set the <code>UsePreviousTemplate</code> to <code>true</code>.<br/>TemplateURL:  Location of file containing the template body. The URL must point to a template that is located in an Amazon S3 bucket. For more information, go to <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/template-anatomy.html">Template Anatomy</a> in the AWS CloudFormation User Guide. Conditional: You must specify only one of the following parameters: <code>TemplateBody</code>, <code>TemplateURL</code>, or set the <code>UsePreviousTemplate</code> to <code>true</code>.<br/>TemplateBody:  Structure containing the template body with a minimum length of 1 byte and a maximum length of 51,200 bytes. (For more information, go to <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/template-anatomy.html">Template Anatomy</a> in the AWS CloudFormation User Guide.) Conditional: You must specify only one of the following parameters: <code>TemplateBody</code>, <code>TemplateURL</code>, or set the <code>UsePreviousTemplate</code> to <code>true</code>.<br/>StackName:  The name or unique stack ID of the stack to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringTimeInMinutes</i> The amount of time, in minutes, during which CloudFormation should monitor all the rollback triggers after the stack creation or update operation deploys all necessary resources. The default is 0 minutes. If you specify a monitoring period but do not specify any rollback triggers, CloudFormation still waits the specified period of time before cleaning up old resources after update operations. You can use this monitoring period to perform any manual stack validation desired, and manually cancel the stack creation or update (using <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/APIReference/API_CancelUpdateStack.html">CancelUpdateStack</a>, for example) as necessary. If you specify 0 for this parameter, CloudFormation still monitors the specified rollback triggers during stack creation and update operations. Then, for update operations, it begins disposing of old resources immediately once the operation completes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RollbackTriggers</i> The triggers to monitor during stack creation or update actions.  By default, AWS CloudFormation saves the rollback triggers specified for a stack and applies them to any subsequent update operations for the stack, unless you specify otherwise. If you do specify rollback triggers for this parameter, those triggers replace any list of triggers previously specified for the stack. This means: <ul> <li> To use the rollback triggers previously specified for this stack, if any, don't specify this parameter. </li> <li> To specify new or updated rollback triggers, you must specify <i>all</i> the triggers that you want used for this stack, even triggers you've specifed before (for example, when creating the stack or during a previous stack update). Any triggers that you don't include in the updated list of triggers are no longer applied to the stack. </li> <li> To remove all currently specified triggers, specify an empty list for this parameter. </li> </ul> If a specified trigger is missing, the entire stack operation fails and is rolled back. <br/>
        :return: StackId:  Unique identifier of the stack.<br/>        """
        pass

    def update_stack_instances(self, Regions: List[Region], Accounts: List[Account], StackSetName: str, ParameterOverrides: List[Parameter]=None, OperationPreferences: StackSetOperationPreferences=None, OperationId: str=None) -> UpdateStackInstancesOutput:
        """Updates the parameter values for stack instances for the specified accounts, within the specified regions. A stack instance refers to a stack in a specific account and region.  You can only update stack instances in regions and accounts where they already exist; to create additional stack instances, use <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/APIReference/API_CreateStackInstances.html">CreateStackInstances</a>.  During stack set updates, any parameters overridden for a stack instance are not updated, but retain their overridden value. You can only update the parameter <i>values</i> that are specified in the stack set; to add or delete a parameter itself, use <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/APIReference/API_UpdateStackSet.html">UpdateStackSet</a> to update the stack set template. If you add a parameter to a template, before you can override the parameter value specified in the stack set you must first use <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/APIReference/API_UpdateStackSet.html">UpdateStackSet</a> to update all stack instances with the updated template and parameter value specified in the stack set. Once a stack instance has been updated with the new parameter, you can then override the parameter value using <code>UpdateStackInstances</code>.

        :param self:
        :param OperationId:  The unique identifier for this stack set operation.  The operation ID also functions as an idempotency token, to ensure that AWS CloudFormation performs the stack set operation only once, even if you retry the request multiple times. You might retry stack set operation requests to ensure that AWS CloudFormation successfully received them. If you don't specify an operation ID, the SDK generates one automatically. <br/>OperationPreferences:  Preferences for how AWS CloudFormation performs this stack set operation.:<br/>ParameterOverrides:   A list of input parameters whose values you want to update for the specified stack instances.  Any overridden parameter values will be applied to all stack instances in the specified accounts and regions. When specifying parameters and their values, be aware of how AWS CloudFormation sets parameter values during stack instance update operations: <ul> <li> To override the current value for a parameter, include the parameter and specify its value. </li> <li> To leave a parameter set to its present value, you can do one of the following: <ul> <li> Do not include the parameter in the list. </li> <li> Include the parameter and specify <code>UsePreviousValue</code> as <code>true</code>. (You cannot specify both a value and set <code>UsePreviousValue</code> to <code>true</code>.) </li> </ul> </li> <li> To set all overridden parameter back to the values specified in the stack set, specify a parameter list but do not include any parameters. </li> <li> To leave all parameters set to their present values, do not specify this property at all. </li> </ul> During stack set updates, any parameter values overridden for a stack instance are not updated, but retain their overridden value. You can only override the parameter <i>values</i> that are specified in the stack set; to add or delete a parameter itself, use <code>UpdateStackSet</code> to update the stack set template. If you add a parameter to a template, before you can override the parameter value specified in the stack set you must first use <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/APIReference/API_UpdateStackSet.html">UpdateStackSet</a> to update all stack instances with the updated template and parameter value specified in the stack set. Once a stack instance has been updated with the new parameter, you can then override the parameter value using <code>UpdateStackInstances</code>.<br/>Regions:  The names of one or more regions in which you want to update parameter values for stack instances. The overridden parameter values will be applied to all stack instances in the specified accounts and regions.<br/>Accounts:  The names of one or more AWS accounts for which you want to update parameter values for stack instances. The overridden parameter values will be applied to all stack instances in the specified accounts and regions.<br/>StackSetName:  The name or unique ID of the stack set associated with the stack instances.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxConcurrentPercentage</i> The maximum percentage of accounts in which to perform this operation at one time. When calculating the number of accounts based on the specified percentage, AWS CloudFormation rounds down to the next whole number. This is true except in cases where rounding down would result is zero. In this case, CloudFormation sets the number as one instead. Note that this setting lets you specify the <i>maximum</i> for operations. For large deployments, under certain circumstances the actual number of accounts acted upon concurrently may be lower due to service throttling. Conditional: You must specify either <code>MaxConcurrentCount</code> or <code>MaxConcurrentPercentage</code>, but not both.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxConcurrentCount</i> The maximum number of accounts in which to perform this operation at one time. This is dependent on the value of <code>FailureToleranceCount</code>—<code>MaxConcurrentCount</code> is at most one more than the <code>FailureToleranceCount</code> . Note that this setting lets you specify the <i>maximum</i> for operations. For large deployments, under certain circumstances the actual number of accounts acted upon concurrently may be lower due to service throttling. Conditional: You must specify either <code>MaxConcurrentCount</code> or <code>MaxConcurrentPercentage</code>, but not both.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureTolerancePercentage</i> The percentage of accounts, per region, for which this stack operation can fail before AWS CloudFormation stops the operation in that region. If the operation is stopped in a region, AWS CloudFormation doesn't attempt the operation in any subsequent regions. When calculating the number of accounts based on the specified percentage, AWS CloudFormation rounds <i>down</i> to the next whole number. Conditional: You must specify either <code>FailureToleranceCount</code> or <code>FailureTolerancePercentage</code>, but not both.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureToleranceCount</i> The number of accounts, per region, for which this operation can fail before AWS CloudFormation stops the operation in that region. If the operation is stopped in a region, AWS CloudFormation doesn't attempt the operation in any subsequent regions. Conditional: You must specify either <code>FailureToleranceCount</code> or <code>FailureTolerancePercentage</code> (but not both).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RegionOrder</i> The order of the regions in where you want to perform the stack operation.<br/>
        :return: OperationId:  The unique identifier for this stack set operation. <br/>        """
        pass

    def update_stack_set(self, StackSetName: str, Description: str=None, TemplateBody: str=None, TemplateURL: str=None, UsePreviousTemplate: UsePreviousTemplate=None, Parameters: List[Parameter]=None, Capabilities: List[Capability]=None, Tags: List[Tag]=None, OperationPreferences: StackSetOperationPreferences=None, AdministrationRoleARN: str=None, ExecutionRoleName: str=None, OperationId: str=None, Accounts: List[Account]=None, Regions: List[Region]=None) -> UpdateStackSetOutput:
        """Updates the stack set, and associated stack instances in the specified accounts and regions. Even if the stack set operation created by updating the stack set fails (completely or partially, below or above a specified failure tolerance), the stack set is updated with your changes. Subsequent <a>CreateStackInstances</a> calls on the specified stack set use the updated stack set.

        :param self:
        :param Regions:  The regions in which to update associated stack instances. If you specify regions, you must also specify accounts in which to update stack set instances. To update <i>all</i> the stack instances associated with this stack set, do not specify the <code>Accounts</code> or <code>Regions</code> properties. If the stack set update includes changes to the template (that is, if the <code>TemplateBody</code> or <code>TemplateURL</code> properties are specified), or the <code>Parameters</code> property, AWS CloudFormation marks all stack instances with a status of <code>OUTDATED</code> prior to updating the stack instances in the specified accounts and regions. If the stack set update does not include changes to the template or parameters, AWS CloudFormation updates the stack instances in the specified accounts and regions, while leaving all other stack instances with their existing stack instance status. <br/>Accounts:  The accounts in which to update associated stack instances. If you specify accounts, you must also specify the regions in which to update stack set instances. To update <i>all</i> the stack instances associated with this stack set, do not specify the <code>Accounts</code> or <code>Regions</code> properties. If the stack set update includes changes to the template (that is, if the <code>TemplateBody</code> or <code>TemplateURL</code> properties are specified), or the <code>Parameters</code> property, AWS CloudFormation marks all stack instances with a status of <code>OUTDATED</code> prior to updating the stack instances in the specified accounts and regions. If the stack set update does not include changes to the template or parameters, AWS CloudFormation updates the stack instances in the specified accounts and regions, while leaving all other stack instances with their existing stack instance status. <br/>OperationId:  The unique ID for this stack set operation.  The operation ID also functions as an idempotency token, to ensure that AWS CloudFormation performs the stack set operation only once, even if you retry the request multiple times. You might retry stack set operation requests to ensure that AWS CloudFormation successfully received them. If you don't specify an operation ID, AWS CloudFormation generates one automatically. Repeating this stack set operation with a new operation ID retries all stack instances whose status is <code>OUTDATED</code>. <br/>ExecutionRoleName:  The name of the IAM execution role to use to update the stack set. If you do not specify an execution role, AWS CloudFormation uses the <code>AWSCloudFormationStackSetExecutionRole</code> role for the stack set operation. Specify an IAM role only if you are using customized execution roles to control which stack resources users and groups can include in their stack sets.   If you specify a customized execution role, AWS CloudFormation uses that role to update the stack. If you do not specify a customized execution role, AWS CloudFormation performs the update using the role previously associated with the stack set, so long as you have permissions to perform operations on the stack set.<br/>AdministrationRoleARN:  The Amazon Resource Number (ARN) of the IAM role to use to update this stack set. Specify an IAM role only if you are using customized administrator roles to control which users or groups can manage specific stack sets within the same administrator account. For more information, see <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/stacksets-prereqs.html">Granting Permissions for Stack Set Operations</a> in the <i>AWS CloudFormation User Guide</i>. If you specified a customized administrator role when you created the stack set, you must specify a customized administrator role, even if it is the same customized administrator role used with this stack set previously.<br/>OperationPreferences:  Preferences for how AWS CloudFormation performs this stack set operation.:<br/>Tags:  The key-value pairs to associate with this stack set and the stacks created from it. AWS CloudFormation also propagates these tags to supported resources that are created in the stacks. You can specify a maximum number of 50 tags. If you specify tags for this parameter, those tags replace any list of tags that are currently associated with this stack set. This means: <ul> <li> If you don't specify this parameter, AWS CloudFormation doesn't modify the stack's tags.  </li> <li> If you specify <i>any</i> tags using this parameter, you must specify <i>all</i> the tags that you want associated with this stack set, even tags you've specifed before (for example, when creating the stack set or during a previous update of the stack set.). Any tags that you don't include in the updated list of tags are removed from the stack set, and therefore from the stacks and resources as well.  </li> <li> If you specify an empty value, AWS CloudFormation removes all currently associated tags. </li> </ul> If you specify new tags as part of an <code>UpdateStackSet</code> action, AWS CloudFormation checks to see if you have the required IAM permission to tag resources. If you omit tags that are currently associated with the stack set from the list of tags you specify, AWS CloudFormation assumes that you want to remove those tags from the stack set, and checks to see if you have permission to untag resources. If you don't have the necessary permission(s), the entire <code>UpdateStackSet</code> action fails with an <code>access denied</code> error, and the stack set is not updated.<br/>Capabilities:  In some cases, you must explicity acknowledge that your stack template contains certain capabilities in order for AWS CloudFormation to update the stack set and its associated stack instances. <ul> <li>  <code>CAPABILITY_IAM</code> and <code>CAPABILITY_NAMED_IAM</code>  Some stack templates might include resources that can affect permissions in your AWS account; for example, by creating new AWS Identity and Access Management (IAM) users. For those stacks sets, you must explicitly acknowledge this by specifying one of these capabilities. The following IAM resources require you to specify either the <code>CAPABILITY_IAM</code> or <code>CAPABILITY_NAMED_IAM</code> capability. <ul> <li> If you have IAM resources, you can specify either capability.  </li> <li> If you have IAM resources with custom names, you <i>must</i> specify <code>CAPABILITY_NAMED_IAM</code>.  </li> <li> If you don't specify either of these capabilities, AWS CloudFormation returns an <code>InsufficientCapabilities</code> error. </li> </ul> If your stack template contains these resources, we recommend that you review all permissions associated with them and edit their permissions if necessary. <ul> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-accesskey.html"> AWS::IAM::AccessKey</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-group.html"> AWS::IAM::Group</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-instanceprofile.html"> AWS::IAM::InstanceProfile</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-policy.html"> AWS::IAM::Policy</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-role.html"> AWS::IAM::Role</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-user.html"> AWS::IAM::User</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-addusertogroup.html"> AWS::IAM::UserToGroupAddition</a>  </li> </ul> For more information, see <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-iam-template.html#capabilities">Acknowledging IAM Resources in AWS CloudFormation Templates</a>. </li> <li>  <code>CAPABILITY_AUTO_EXPAND</code>  Some templates contain macros. If your stack template contains one or more macros, and you choose to update a stack directly from the processed template, without first reviewing the resulting changes in a change set, you must acknowledge this capability. For more information, see <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/template-macros.html">Using AWS CloudFormation Macros to Perform Custom Processing on Templates</a>. <important> Stack sets do not currently support macros in stack templates. (This includes the <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/create-reusable-transform-function-snippets-and-add-to-your-template-with-aws-include-transform.html">AWS::Include</a> and <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/transform-aws-serverless.html">AWS::Serverless</a> transforms, which are macros hosted by AWS CloudFormation.) Even if you specify this capability, if you include a macro in your template the stack set operation will fail. </important> </li> </ul><br/>Parameters:  A list of input parameters for the stack set template. <br/>UsePreviousTemplate:  Use the existing template that's associated with the stack set that you're updating. Conditional: You must specify only one of the following parameters: <code>TemplateBody</code> or <code>TemplateURL</code>—or set <code>UsePreviousTemplate</code> to true. <br/>TemplateURL:  The location of the file that contains the template body. The URL must point to a template (maximum size: 460,800 bytes) that is located in an Amazon S3 bucket. For more information, see <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/template-anatomy.html">Template Anatomy</a> in the AWS CloudFormation User Guide. Conditional: You must specify only one of the following parameters: <code>TemplateBody</code> or <code>TemplateURL</code>—or set <code>UsePreviousTemplate</code> to true. <br/>TemplateBody:  The structure that contains the template body, with a minimum length of 1 byte and a maximum length of 51,200 bytes. For more information, see <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/template-anatomy.html">Template Anatomy</a> in the AWS CloudFormation User Guide. Conditional: You must specify only one of the following parameters: <code>TemplateBody</code> or <code>TemplateURL</code>—or set <code>UsePreviousTemplate</code> to true.<br/>Description:  A brief description of updates that you are making.<br/>StackSetName:  The name or unique ID of the stack set that you want to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxConcurrentPercentage</i> The maximum percentage of accounts in which to perform this operation at one time. When calculating the number of accounts based on the specified percentage, AWS CloudFormation rounds down to the next whole number. This is true except in cases where rounding down would result is zero. In this case, CloudFormation sets the number as one instead. Note that this setting lets you specify the <i>maximum</i> for operations. For large deployments, under certain circumstances the actual number of accounts acted upon concurrently may be lower due to service throttling. Conditional: You must specify either <code>MaxConcurrentCount</code> or <code>MaxConcurrentPercentage</code>, but not both.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxConcurrentCount</i> The maximum number of accounts in which to perform this operation at one time. This is dependent on the value of <code>FailureToleranceCount</code>—<code>MaxConcurrentCount</code> is at most one more than the <code>FailureToleranceCount</code> . Note that this setting lets you specify the <i>maximum</i> for operations. For large deployments, under certain circumstances the actual number of accounts acted upon concurrently may be lower due to service throttling. Conditional: You must specify either <code>MaxConcurrentCount</code> or <code>MaxConcurrentPercentage</code>, but not both.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureTolerancePercentage</i> The percentage of accounts, per region, for which this stack operation can fail before AWS CloudFormation stops the operation in that region. If the operation is stopped in a region, AWS CloudFormation doesn't attempt the operation in any subsequent regions. When calculating the number of accounts based on the specified percentage, AWS CloudFormation rounds <i>down</i> to the next whole number. Conditional: You must specify either <code>FailureToleranceCount</code> or <code>FailureTolerancePercentage</code>, but not both.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureToleranceCount</i> The number of accounts, per region, for which this operation can fail before AWS CloudFormation stops the operation in that region. If the operation is stopped in a region, AWS CloudFormation doesn't attempt the operation in any subsequent regions. Conditional: You must specify either <code>FailureToleranceCount</code> or <code>FailureTolerancePercentage</code> (but not both).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RegionOrder</i> The order of the regions in where you want to perform the stack operation.<br/>
        :return: OperationId:  The unique ID for this stack set operation.<br/>        """
        pass

    def update_termination_protection(self, StackName: str, EnableTerminationProtection: EnableTerminationProtection) -> UpdateTerminationProtectionOutput:
        """Updates termination protection for the specified stack. If a user attempts to delete a stack with termination protection enabled, the operation fails and the stack remains unchanged. For more information, see <a href="AWSCloudFormation/latest/UserGuide/using-cfn-protect-stacks.html">Protecting a Stack From Being Deleted</a> in the <i>AWS CloudFormation User Guide</i>.  For <a href="AWSCloudFormation/latest/UserGuide/using-cfn-nested-stacks.html">nested stacks</a>, termination protection is set on the root stack and cannot be changed directly on the nested stack.

        :param self:
        :param StackName:  The name or unique ID of the stack for which you want to set termination protection.<br/>EnableTerminationProtection:  Whether to enable termination protection on the specified stack.<br/>
        :return: StackId:  The unique ID of the stack.<br/>        """
        pass

    def validate_template(self, TemplateBody: str=None, TemplateURL: str=None) -> ValidateTemplateOutput:
        """Validates a specified template. AWS CloudFormation first checks if the template is valid JSON. If it isn't, AWS CloudFormation checks if the template is valid YAML. If both these checks fail, AWS CloudFormation returns a template validation error.

        :param self:
        :param TemplateURL:  Location of file containing the template body. The URL must point to a template (max size: 460,800 bytes) that is located in an Amazon S3 bucket. For more information, go to <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/template-anatomy.html">Template Anatomy</a> in the AWS CloudFormation User Guide. Conditional: You must pass <code>TemplateURL</code> or <code>TemplateBody</code>. If both are passed, only <code>TemplateBody</code> is used.<br/>TemplateBody:  Structure containing the template body with a minimum length of 1 byte and a maximum length of 51,200 bytes. For more information, go to <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/template-anatomy.html">Template Anatomy</a> in the AWS CloudFormation User Guide. Conditional: You must pass <code>TemplateURL</code> or <code>TemplateBody</code>. If both are passed, only <code>TemplateBody</code> is used.<br/>
        :return: DeclaredTransforms:  A list of the transforms that are declared in the template.<br/>CapabilitiesReason:  The list of resources that generated the values in the <code>Capabilities</code> response element.<br/>Capabilities:  The capabilities found within the template. If your template contains IAM resources, you must specify the CAPABILITY_IAM or CAPABILITY_NAMED_IAM value for this parameter when you use the <a>CreateStack</a> or <a>UpdateStack</a> actions with your template; otherwise, those actions return an InsufficientCapabilities error. For more information, see <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/using-iam-template.html#capabilities">Acknowledging IAM Resources in AWS CloudFormation Templates</a>.<br/>Description:  The description found within the template.<br/>Parameters:  A list of <code>TemplateParameter</code> structures.<br/>        """
        pass

    class CloudformationResource:
        
        class Event:
            """Creates a Event resource.::

  event = cloudformation.Event('id')

:type id: string
:param id: The Event's id identifier. This **must** be set.

:rtype: :py:class:`CloudFormation.Event`
:returns: A Event resource"""

            def __init__(self, id):
                pass
            
            class Timestamp(object):
                pass
        
            stack_id: str=None
            event_id: str=None
            stack_name: str=None
            logical_resource_id: str=None
            physical_resource_id: str=None
            resource_type: str=None
            timestamp: Timestamp=None
            resource_status: str=None
            resource_status_reason: str=None
            resource_properties: str=None
            client_request_token: str=None
            

        
        class Stack:
            """Creates a Stack resource.::

  stack = cloudformation.Stack('name')

:type name: string
:param name: The Stack's name identifier. This **must** be set.

:rtype: :py:class:`CloudFormation.Stack`
:returns: A Stack resource"""

            def __init__(self, name):
                pass
            
            class DeletionTime(object):
                pass

            class Capability(object):
                pass

            class EnableTerminationProtection(object):
                pass

            class StackDriftInformation(Mapping):
                pass

            class Output(Mapping):
                pass

            class RollbackConfiguration(Mapping):
                pass

            class NotificationARN(object):
                pass

            class CreationTime(object):
                pass

            class Parameter(Mapping):
                pass

            class LastUpdatedTime(object):
                pass

            class Tag(Mapping):
                pass

            class DisableRollback(object):
                pass
        
            stack_id: str=None
            stack_name: str=None
            change_set_id: str=None
            description: str=None
            parameters: List[Parameter]=None
            creation_time: CreationTime=None
            deletion_time: DeletionTime=None
            last_updated_time: LastUpdatedTime=None
            rollback_configuration: RollbackConfiguration=None
            stack_status: str=None
            stack_status_reason: str=None
            disable_rollback: DisableRollback=None
            notification_arns: List[NotificationARN]=None
            timeout_in_minutes: int=None
            capabilities: List[Capability]=None
            outputs: List[Output]=None
            role_arn: str=None
            tags: List[Tag]=None
            enable_termination_protection: EnableTerminationProtection=None
            parent_id: str=None
            root_id: str=None
            drift_information: StackDriftInformation=None
            
            def cancel_update(self, **kwargs):
                # type: (object) -> dict
                return {}

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def update(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class StackResource:
            """Creates a StackResource resource.::

  stack_resource = cloudformation.StackResource('stack_name','logical_id')

:type stack_name: string
:param stack_name: The StackResource's stack_name identifier. This **must** be set.
:type logical_id: string
:param logical_id: The StackResource's logical_id identifier. This **must** be set.

:rtype: :py:class:`CloudFormation.StackResource`
:returns: A StackResource resource"""

            def __init__(self, stack_name, logical_id):
                pass
            
            class StackResourceDriftInformation(Mapping):
                pass

            class Timestamp(object):
                pass
        
            stack_name: str=None
            stack_id: str=None
            logical_resource_id: str=None
            physical_resource_id: str=None
            resource_type: str=None
            last_updated_timestamp: Timestamp=None
            resource_status: str=None
            resource_status_reason: str=None
            description: str=None
            metadata: str=None
            drift_information: StackResourceDriftInformation=None
            

        
        class StackResourceSummary:
            """Creates a StackResourceSummary resource.::

  stack_resource_summary = cloudformation.StackResourceSummary('stack_name','logical_id')

:type stack_name: string
:param stack_name: The StackResourceSummary's stack_name identifier. This **must** be set.
:type logical_id: string
:param logical_id: The StackResourceSummary's logical_id identifier. This **must** be set.

:rtype: :py:class:`CloudFormation.StackResourceSummary`
:returns: A StackResourceSummary resource"""

            def __init__(self, stack_name, logical_id):
                pass
            
            class StackResourceDriftInformationSummary(Mapping):
                pass

            class Timestamp(object):
                pass
        
            logical_resource_id: str=None
            physical_resource_id: str=None
            resource_type: str=None
            last_updated_timestamp: Timestamp=None
            resource_status: str=None
            resource_status_reason: str=None
            drift_information: StackResourceDriftInformationSummary=None
            

        def create_stack(self, **kwargs):
            # type: (object) -> Stack
            pass

        def get_available_subresources(self) -> List[str]:
            pass

        class StackResourceCollection(List[Stack], ResourceCollection):
            pass

        stacks: StackResourceCollection = None




class CloudFront(BaseClient):
    """<fullname>Amazon CloudFront</fullname> This is the <i>Amazon CloudFront API Reference</i>. This guide is for developers who need detailed information about CloudFront API actions, data types, and errors. For detailed information about CloudFront features, see the <i>Amazon CloudFront Developer Guide</i>."""

    class DistributionDeployedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `get_distribution` for valid parameters"""
            pass
    class InvalidationCompletedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `get_invalidation` for valid parameters"""
            pass
    class StreamingDistributionDeployedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `get_streaming_distribution` for valid parameters"""
            pass


    class ListCloudFrontOriginAccessIdentitiesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_cloud_front_origin_access_identities` for valid parameters"""
            pass
    class ListDistributionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_distributions` for valid parameters"""
            pass
    class ListInvalidationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_invalidations` for valid parameters"""
            pass
    class ListStreamingDistributionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_streaming_distributions` for valid parameters"""
            pass


    class CreateCloudFrontOriginAccessIdentityRequest(Mapping):
        pass
    
    class CreateCloudFrontOriginAccessIdentityResult(Mapping):
        pass
    
    class CloudFrontOriginAccessIdentityConfig(Mapping):
        pass
    
    class CreateDistributionRequest(Mapping):
        pass
    
    class CreateDistributionResult(Mapping):
        pass
    
    class DistributionConfig(Mapping):
        pass
    
    class CreateDistributionWithTagsRequest(Mapping):
        pass
    
    class CreateDistributionWithTagsResult(Mapping):
        pass
    
    class DistributionConfigWithTags(Mapping):
        pass
    
    class CreateFieldLevelEncryptionConfigRequest(Mapping):
        pass
    
    class CreateFieldLevelEncryptionConfigResult(Mapping):
        pass
    
    class FieldLevelEncryptionConfig(Mapping):
        pass
    
    class CreateFieldLevelEncryptionProfileRequest(Mapping):
        pass
    
    class CreateFieldLevelEncryptionProfileResult(Mapping):
        pass
    
    class FieldLevelEncryptionProfileConfig(Mapping):
        pass
    
    class CreateInvalidationRequest(Mapping):
        pass
    
    class CreateInvalidationResult(Mapping):
        pass
    
    class InvalidationBatch(Mapping):
        pass
    
    class CreatePublicKeyRequest(Mapping):
        pass
    
    class CreatePublicKeyResult(Mapping):
        pass
    
    class PublicKeyConfig(Mapping):
        pass
    
    class CreateStreamingDistributionRequest(Mapping):
        pass
    
    class CreateStreamingDistributionResult(Mapping):
        pass
    
    class StreamingDistributionConfig(Mapping):
        pass
    
    class CreateStreamingDistributionWithTagsRequest(Mapping):
        pass
    
    class CreateStreamingDistributionWithTagsResult(Mapping):
        pass
    
    class StreamingDistributionConfigWithTags(Mapping):
        pass
    
    class DeleteCloudFrontOriginAccessIdentityRequest(Mapping):
        pass
    
    class DeleteDistributionRequest(Mapping):
        pass
    
    class DeleteFieldLevelEncryptionConfigRequest(Mapping):
        pass
    
    class DeleteFieldLevelEncryptionProfileRequest(Mapping):
        pass
    
    class DeletePublicKeyRequest(Mapping):
        pass
    
    class DeleteStreamingDistributionRequest(Mapping):
        pass
    
    class GetCloudFrontOriginAccessIdentityRequest(Mapping):
        pass
    
    class GetCloudFrontOriginAccessIdentityResult(Mapping):
        pass
    
    class GetCloudFrontOriginAccessIdentityConfigRequest(Mapping):
        pass
    
    class GetCloudFrontOriginAccessIdentityConfigResult(Mapping):
        pass
    
    class GetDistributionRequest(Mapping):
        pass
    
    class GetDistributionResult(Mapping):
        pass
    
    class GetDistributionConfigRequest(Mapping):
        pass
    
    class GetDistributionConfigResult(Mapping):
        pass
    
    class GetFieldLevelEncryptionRequest(Mapping):
        pass
    
    class GetFieldLevelEncryptionResult(Mapping):
        pass
    
    class GetFieldLevelEncryptionConfigRequest(Mapping):
        pass
    
    class GetFieldLevelEncryptionConfigResult(Mapping):
        pass
    
    class GetFieldLevelEncryptionProfileRequest(Mapping):
        pass
    
    class GetFieldLevelEncryptionProfileResult(Mapping):
        pass
    
    class GetFieldLevelEncryptionProfileConfigRequest(Mapping):
        pass
    
    class GetFieldLevelEncryptionProfileConfigResult(Mapping):
        pass
    
    class GetInvalidationRequest(Mapping):
        pass
    
    class GetInvalidationResult(Mapping):
        pass
    
    class GetPublicKeyRequest(Mapping):
        pass
    
    class GetPublicKeyResult(Mapping):
        pass
    
    class GetPublicKeyConfigRequest(Mapping):
        pass
    
    class GetPublicKeyConfigResult(Mapping):
        pass
    
    class GetStreamingDistributionRequest(Mapping):
        pass
    
    class GetStreamingDistributionResult(Mapping):
        pass
    
    class GetStreamingDistributionConfigRequest(Mapping):
        pass
    
    class GetStreamingDistributionConfigResult(Mapping):
        pass
    
    class ListCloudFrontOriginAccessIdentitiesRequest(Mapping):
        pass
    
    class ListCloudFrontOriginAccessIdentitiesResult(Mapping):
        pass
    
    class ListDistributionsRequest(Mapping):
        pass
    
    class ListDistributionsResult(Mapping):
        pass
    
    class ListDistributionsByWebACLIdRequest(Mapping):
        pass
    
    class ListDistributionsByWebACLIdResult(Mapping):
        pass
    
    class ListFieldLevelEncryptionConfigsRequest(Mapping):
        pass
    
    class ListFieldLevelEncryptionConfigsResult(Mapping):
        pass
    
    class ListFieldLevelEncryptionProfilesRequest(Mapping):
        pass
    
    class ListFieldLevelEncryptionProfilesResult(Mapping):
        pass
    
    class ListInvalidationsRequest(Mapping):
        pass
    
    class ListInvalidationsResult(Mapping):
        pass
    
    class ListPublicKeysRequest(Mapping):
        pass
    
    class ListPublicKeysResult(Mapping):
        pass
    
    class ListStreamingDistributionsRequest(Mapping):
        pass
    
    class ListStreamingDistributionsResult(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResult(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class Tags(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class TagKeys(Mapping):
        pass
    
    class UpdateCloudFrontOriginAccessIdentityRequest(Mapping):
        pass
    
    class UpdateCloudFrontOriginAccessIdentityResult(Mapping):
        pass
    
    class UpdateDistributionRequest(Mapping):
        pass
    
    class UpdateDistributionResult(Mapping):
        pass
    
    class UpdateFieldLevelEncryptionConfigRequest(Mapping):
        pass
    
    class UpdateFieldLevelEncryptionConfigResult(Mapping):
        pass
    
    class UpdateFieldLevelEncryptionProfileRequest(Mapping):
        pass
    
    class UpdateFieldLevelEncryptionProfileResult(Mapping):
        pass
    
    class UpdatePublicKeyRequest(Mapping):
        pass
    
    class UpdatePublicKeyResult(Mapping):
        pass
    
    class UpdateStreamingDistributionRequest(Mapping):
        pass
    
    class UpdateStreamingDistributionResult(Mapping):
        pass
    
    def create_cloud_front_origin_access_identity(self, CloudFrontOriginAccessIdentityConfig: CloudFrontOriginAccessIdentityConfig) -> CreateCloudFrontOriginAccessIdentityResult:
        """Creates a new origin access identity. If you're using Amazon S3 for your origin, you can use an origin access identity to require users to access your content using a CloudFront URL instead of the Amazon S3 URL. For more information about how to use origin access identities, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/PrivateContent.html">Serving Private Content through CloudFront</a> in the <i>Amazon CloudFront Developer Guide</i>.

        :param self:
        :param CloudFrontOriginAccessIdentityConfig:  The current configuration information for the identity.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> Any comments you want to include about the origin access identity. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique value (for example, a date-time stamp) that ensures that the request can't be replayed. If the value of <code>CallerReference</code> is new (regardless of the content of the <code>CloudFrontOriginAccessIdentityConfig</code> object), a new origin access identity is created. If the <code>CallerReference</code> is a value already sent in a previous identity request, and the content of the <code>CloudFrontOriginAccessIdentityConfig</code> is identical to the original request (ignoring white space), the response includes the same information returned to the original request.  If the <code>CallerReference</code> is a value you already sent in a previous request to create an identity, but the content of the <code>CloudFrontOriginAccessIdentityConfig</code> is different from the original request, CloudFront returns a <code>CloudFrontOriginAccessIdentityAlreadyExists</code> error. <br/>
        :return: ETag:  The current version of the origin access identity created.<br/>Location:  The fully qualified URI of the new origin access identity just created. For example: <code>https://cloudfront.amazonaws.com/2010-11-01/origin-access-identity/cloudfront/E74FTE3AJFJ256A</code>.<br/>CloudFrontOriginAccessIdentity:  The origin access identity's information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudFrontOriginAccessIdentityConfig</i> The current configuration information for the identity. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CloudFrontOriginAccessIdentityConfig</b>: The current configuration information for the identity. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3CanonicalUserId</i> The Amazon S3 canonical user ID for the origin access identity, used when giving the origin access identity read permission to an object in Amazon S3. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID for the origin access identity, for example, <code>E74FTE3AJFJ256A</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> Any comments you want to include about the origin access identity. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique value (for example, a date-time stamp) that ensures that the request can't be replayed. If the value of <code>CallerReference</code> is new (regardless of the content of the <code>CloudFrontOriginAccessIdentityConfig</code> object), a new origin access identity is created. If the <code>CallerReference</code> is a value already sent in a previous identity request, and the content of the <code>CloudFrontOriginAccessIdentityConfig</code> is identical to the original request (ignoring white space), the response includes the same information returned to the original request.  If the <code>CallerReference</code> is a value you already sent in a previous request to create an identity, but the content of the <code>CloudFrontOriginAccessIdentityConfig</code> is different from the original request, CloudFront returns a <code>CloudFrontOriginAccessIdentityAlreadyExists</code> error. <br/>        """
        pass

    def create_distribution(self, DistributionConfig: DistributionConfig) -> CreateDistributionResult:
        """Creates a new web distribution. You create a CloudFront distribution to tell CloudFront where you want content to be delivered from, and the details about how to track and manage content delivery. Send a <code>POST</code> request to the <code>/<i>CloudFront API version</i>/distribution</code>/<code>distribution ID</code> resource. <important> When you update a distribution, there are more required fields than when you create a distribution. When you update your distribution by using <a href="https://docs.aws.amazon.com/cloudfront/latest/APIReference/API_UpdateDistribution.html">UpdateDistribution</a>, follow the steps included in the documentation to get the current configuration and then make your updates. This helps to make sure that you include all of the required fields. To view a summary, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/distribution-overview-required-fields.html">Required Fields for Create Distribution and Update Distribution</a> in the <i>Amazon CloudFront Developer Guide</i>. </important>

        :param self:
        :param DistributionConfig:  The distribution's configuration information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsIPV6Enabled</i> If you want CloudFront to respond to IPv6 DNS requests with an IPv6 address for your distribution, specify <code>true</code>. If you specify <code>false</code>, CloudFront responds to IPv6 DNS requests with the DNS response code <code>NOERROR</code> and with no IP addresses. This allows viewers to submit a second request, for an IPv4 address for your distribution.  In general, you should enable IPv6 if you have users on IPv6 networks who want to access your content. However, if you're using signed URLs or signed cookies to restrict access to your content, and if you're using a custom policy that includes the <code>IpAddress</code> parameter to restrict the IP addresses that can access your content, don't enable IPv6. If you want to restrict access to some content by IP address and not restrict access to other content (or restrict access but not by IP address), you can create two distributions. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/private-content-creating-signed-url-custom-policy.html">Creating a Signed URL Using a Custom Policy</a> in the <i>Amazon CloudFront Developer Guide</i>. If you're using an Amazon Route 53 alias resource record set to route traffic to your CloudFront distribution, you need to create a second alias resource record set when both of the following are true: <ul> <li> You enable IPv6 for the distribution </li> <li> You're using alternate domain names in the URLs for your objects </li> </ul> For more information, see <a href="https://docs.aws.amazon.com/Route53/latest/DeveloperGuide/routing-to-cloudfront-distribution.html">Routing Traffic to an Amazon CloudFront Web Distribution by Using Your Domain Name</a> in the <i>Amazon Route 53 Developer Guide</i>. If you created a CNAME resource record set, either with Amazon Route 53 or with another DNS service, you don't need to make any changes. A CNAME record will route traffic to your distribution regardless of the IP address format of the viewer request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HttpVersion</i> (Optional) Specify the maximum HTTP version that you want viewers to use to communicate with CloudFront. The default value for new web distributions is http2. Viewers that don't support HTTP/2 automatically use an earlier HTTP version. For viewers and CloudFront to use HTTP/2, viewers must support TLS 1.2 or later, and must support Server Name Identification (SNI). In general, configuring CloudFront to communicate with viewers using HTTP/2 reduces latency. You can improve performance by optimizing for HTTP/2. For more information, do an Internet search for "http/2 optimization." <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WebACLId</i> A unique identifier that specifies the AWS WAF web ACL, if any, to associate with this distribution. AWS WAF is a web application firewall that lets you monitor the HTTP and HTTPS requests that are forwarded to CloudFront, and lets you control access to your content. Based on conditions that you specify, such as the IP addresses that requests originate from or the values of query strings, CloudFront responds to requests either with the requested content or with an HTTP 403 status code (Forbidden). You can also configure CloudFront to return a custom error page when a request is blocked. For more information about AWS WAF, see the <a href="http://docs.aws.amazon.com/waf/latest/developerguide/what-is-aws-waf.html">AWS WAF Developer Guide</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Restrictions</i> A complex type that identifies ways in which you want to restrict distribution of your content.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Restrictions</b>: A complex type that identifies ways in which you want to restrict distribution of your content.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ViewerCertificate</i> A complex type that specifies whether you want viewers to use HTTP or HTTPS to request your objects, whether you're using an alternate domain name with HTTPS, and if so, if you're using AWS Certificate Manager (ACM) or a third-party certificate authority.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ViewerCertificate</b>: A complex type that specifies whether you want viewers to use HTTP or HTTPS to request your objects, whether you're using an alternate domain name with HTTPS, and if so, if you're using AWS Certificate Manager (ACM) or a third-party certificate authority.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> From this field, you can enable or disable the selected distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PriceClass</i> The price class that corresponds with the maximum price that you want to pay for CloudFront service. If you specify <code>PriceClass_All</code>, CloudFront responds to requests for your objects from all CloudFront edge locations. If you specify a price class other than <code>PriceClass_All</code>, CloudFront serves your objects from the CloudFront edge location that has the lowest latency among the edge locations in your price class. Viewers who are in or near regions that are excluded from your specified price class may encounter slower performance. For more information about price classes, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/PriceClass.html">Choosing the Price Class for a CloudFront Distribution</a> in the <i>Amazon CloudFront Developer Guide</i>. For information about CloudFront pricing, including how price classes (such as Price Class 100) map to CloudFront regions, see <a href="http://aws.amazon.com/cloudfront/pricing/">Amazon CloudFront Pricing</a>. For price class information, scroll down to see the table at the bottom of the page.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Logging</i> A complex type that controls whether access logs are written for the distribution. For more information about logging, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/AccessLogs.html">Access Logs</a> in the <i>Amazon CloudFront Developer Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Logging</b>: A complex type that controls whether access logs are written for the distribution. For more information about logging, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/AccessLogs.html">Access Logs</a> in the <i>Amazon CloudFront Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> Any comments you want to include about the distribution. If you don't want to specify a comment, include an empty <code>Comment</code> element. To delete an existing comment, update the distribution configuration and include an empty <code>Comment</code> element. To add or change a comment, update the distribution configuration and specify the new comment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomErrorResponses</i> A complex type that controls the following: <ul> <li> Whether CloudFront replaces HTTP status codes in the 4xx and 5xx range with custom error messages before returning the response to the viewer. </li> <li> How long CloudFront caches HTTP status codes in the 4xx and 5xx range. </li> </ul> For more information about custom error pages, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/custom-error-pages.html">Customizing Error Responses</a> in the <i>Amazon CloudFront Developer Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CustomErrorResponses</b>: A complex type that controls the following: <ul> <li> Whether CloudFront replaces HTTP status codes in the 4xx and 5xx range with custom error messages before returning the response to the viewer. </li> <li> How long CloudFront caches HTTP status codes in the 4xx and 5xx range. </li> </ul> For more information about custom error pages, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/custom-error-pages.html">Customizing Error Responses</a> in the <i>Amazon CloudFront Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheBehaviors</i> A complex type that contains zero or more <code>CacheBehavior</code> elements. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CacheBehaviors</b>: A complex type that contains zero or more <code>CacheBehavior</code> elements. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultCacheBehavior</i> A complex type that describes the default cache behavior if you don't specify a <code>CacheBehavior</code> element or if files don't match any of the values of <code>PathPattern</code> in <code>CacheBehavior</code> elements. You must create exactly one default cache behavior.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DefaultCacheBehavior</b>: A complex type that describes the default cache behavior if you don't specify a <code>CacheBehavior</code> element or if files don't match any of the values of <code>PathPattern</code> in <code>CacheBehavior</code> elements. You must create exactly one default cache behavior.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OriginGroups</i>  A complex type that contains information about origin groups for this distribution.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>OriginGroups</b>:  A complex type that contains information about origin groups for this distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Origins</i> A complex type that contains information about origins for this distribution. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Origins</b>: A complex type that contains information about origins for this distribution. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultRootObject</i> The object that you want CloudFront to request from your origin (for example, <code>index.html</code>) when a viewer requests the root URL for your distribution (<code>http://www.example.com</code>) instead of an object in your distribution (<code>http://www.example.com/product-description.html</code>). Specifying a default root object avoids exposing the contents of your distribution. Specify only the object name, for example, <code>index.html</code>. Don't add a <code>/</code> before the object name. If you don't want to specify a default root object when you create a distribution, include an empty <code>DefaultRootObject</code> element. To delete the default root object from an existing distribution, update the distribution configuration and include an empty <code>DefaultRootObject</code> element. To replace the default root object, update the distribution configuration and specify the new object. For more information about the default root object, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/DefaultRootObject.html">Creating a Default Root Object</a> in the <i>Amazon CloudFront Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Aliases</i> A complex type that contains information about CNAMEs (alternate domain names), if any, for this distribution.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Aliases</b>: A complex type that contains information about CNAMEs (alternate domain names), if any, for this distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique value (for example, a date-time stamp) that ensures that the request can't be replayed. If the value of <code>CallerReference</code> is new (regardless of the content of the <code>DistributionConfig</code> object), CloudFront creates a new distribution. If <code>CallerReference</code> is a value that you already sent in a previous request to create a distribution, CloudFront returns a <code>DistributionAlreadyExists</code> error.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> A complex type that contains the CNAME aliases, if any, that you want to associate with this distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of CNAME aliases, if any, that you want to associate with this distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> A complex type that contains origins or origin groups for this distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of origins or origin groups for this distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> The items (origin groups) in a distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of origin groups.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetOriginId</i> The value of <code>ID</code> for the origin that you want CloudFront to route requests to when a request matches the path pattern either for a cache behavior or for the default cache behavior in your distribution.<br/>A complex type that specifies how CloudFront handles query strings and cookies.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ForwardedValues</b>: A complex type that specifies how CloudFront handles query strings and cookies.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> Optional: A complex type that contains cache behaviors for this distribution. If <code>Quantity</code> is <code>0</code>, you can omit <code>Items</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of cache behaviors for this distribution. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> A complex type that contains a <code>CustomErrorResponse</code> element for each HTTP status code for which you want to specify a custom error page and/or a caching duration. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of HTTP status codes for which you want to specify a custom error page and/or a caching duration. If <code>Quantity</code> is <code>0</code>, you can omit <code>Items</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> An optional string that you want CloudFront to prefix to the access log <code>filenames</code> for this distribution, for example, <code>myprefix/</code>. If you want to enable logging, but you don't want to specify a prefix, you still must include an empty <code>Prefix</code> element in the <code>Logging</code> element.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> The Amazon S3 bucket to store the access logs in, for example, <code>myawslogbucket.s3.amazonaws.com</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeCookies</i> Specifies whether you want CloudFront to include cookies in access logs, specify <code>true</code> for <code>IncludeCookies</code>. If you choose to include cookies in logs, CloudFront logs all cookies regardless of how you configure the cache behaviors for this distribution. If you don't want to include cookies when you create a distribution or if you want to disable include cookies for an existing distribution, specify <code>false</code> for <code>IncludeCookies</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether you want CloudFront to save access logs to an Amazon S3 bucket. If you don't want to enable logging when you create a distribution or if you want to disable logging for an existing distribution, specify <code>false</code> for <code>Enabled</code>, and specify empty <code>Bucket</code> and <code>Prefix</code> elements. If you specify <code>false</code> for <code>Enabled</code> but you specify values for <code>Bucket</code>, <code>prefix</code>, and <code>IncludeCookies</code>, the values are automatically deleted.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateSource</i> This field is no longer used. Use one of the following fields instead: <ul> <li>  <a href="https://docs.aws.amazon.com/cloudfront/latest/APIReference/API_ViewerCertificate.html#cloudfront-Type-ViewerCertificate-ACMCertificateArn">ACMCertificateArn</a>  </li> <li>  <a href="https://docs.aws.amazon.com/cloudfront/latest/APIReference/API_ViewerCertificate.html#cloudfront-Type-ViewerCertificate-IAMCertificateId">IAMCertificateId</a>  </li> <li>  <a href="https://docs.aws.amazon.com/cloudfront/latest/APIReference/API_ViewerCertificate.html#cloudfront-Type-ViewerCertificate-CloudFrontDefaultCertificate">CloudFrontDefaultCertificate</a>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Certificate</i> This field is no longer used. Use one of the following fields instead: <ul> <li>  <a href="https://docs.aws.amazon.com/cloudfront/latest/APIReference/API_ViewerCertificate.html#cloudfront-Type-ViewerCertificate-ACMCertificateArn">ACMCertificateArn</a>  </li> <li>  <a href="https://docs.aws.amazon.com/cloudfront/latest/APIReference/API_ViewerCertificate.html#cloudfront-Type-ViewerCertificate-IAMCertificateId">IAMCertificateId</a>  </li> <li>  <a href="https://docs.aws.amazon.com/cloudfront/latest/APIReference/API_ViewerCertificate.html#cloudfront-Type-ViewerCertificate-CloudFrontDefaultCertificate">CloudFrontDefaultCertificate</a>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinimumProtocolVersion</i> Specify the security policy that you want CloudFront to use for HTTPS connections. A security policy determines two settings: <ul> <li> The minimum SSL/TLS protocol that CloudFront uses to communicate with viewers </li> <li> The cipher that CloudFront uses to encrypt the content that it returns to viewers </li> </ul> <note> On the CloudFront console, this setting is called <b>Security policy</b>. </note> We recommend that you specify <code>TLSv1.1_2016</code> unless your users are using browsers or devices that do not support TLSv1.1 or later. When both of the following are true, you must specify <code>TLSv1</code> or later for the security policy:  <ul> <li> You're using a custom certificate: you specified a value for <code>ACMCertificateArn</code> or for <code>IAMCertificateId</code>  </li> <li> You're using SNI: you specified <code>sni-only</code> for <code>SSLSupportMethod</code>  </li> </ul> If you specify <code>true</code> for <code>CloudFrontDefaultCertificate</code>, CloudFront automatically sets the security policy to <code>TLSv1</code> regardless of the value that you specify for <code>MinimumProtocolVersion</code>. For information about the relationship between the security policy that you choose and the protocols and ciphers that CloudFront uses to communicate with viewers, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/secure-connections-supported-viewer-protocols-ciphers.html#secure-connections-supported-ciphers"> Supported SSL/TLS Protocols and Ciphers for Communication Between Viewers and CloudFront</a> in the <i>Amazon CloudFront Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SSLSupportMethod</i> If you specify a value for <a href="https://docs.aws.amazon.com/cloudfront/latest/APIReference/API_ViewerCertificate.html#cloudfront-Type-ViewerCertificate-ACMCertificateArn">ACMCertificateArn</a> or for <a href="https://docs.aws.amazon.com/cloudfront/latest/APIReference/API_ViewerCertificate.html#cloudfront-Type-ViewerCertificate-IAMCertificateId">IAMCertificateId</a>, you must also specify how you want CloudFront to serve HTTPS requests: using a method that works for browsers and clients released after 2010 or one that works for all clients. <ul> <li>  <code>sni-only</code>: CloudFront can respond to HTTPS requests from viewers that support Server Name Indication (SNI). All modern browsers support SNI, but there are a few that don't. For a current list of the browsers that support SNI, see the <a href="http://en.wikipedia.org/wiki/Server_Name_Indication">Wikipedia entry Server Name Indication</a>. To learn about options to explore if you have users with browsers that don't include SNI support, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/cnames-https-dedicated-ip-or-sni.html">Choosing How CloudFront Serves HTTPS Requests</a> in the <i>Amazon CloudFront Developer Guide</i>. </li> <li>  <code>vip</code>: CloudFront uses dedicated IP addresses for your content and can respond to HTTPS requests from any viewer. However, there are additional monthly charges. For details, including specific pricing information, see <a href="http://aws.amazon.com/cloudfront/custom-ssl-domains/">Custom SSL options for Amazon CloudFront</a> on the AWS marketing site. </li> </ul> Don't specify a value for <code>SSLSupportMethod</code> if you specified <code>&lt;CloudFrontDefaultCertificate&gt;true&lt;CloudFrontDefaultCertificate&gt;</code>. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/cnames-https-dedicated-ip-or-sni.html">Choosing How CloudFront Serves HTTPS Requests</a> in the <i>Amazon CloudFront Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ACMCertificateArn</i> If you want viewers to use HTTPS to request your objects and you're using an alternate domain name, you must choose the type of certificate that you want to use. Specify the following value if ACM provided your certificate: <ul> <li>  <code>&lt;ACMCertificateArn&gt;<i>ARN for ACM SSL/TLS certificate</i>&lt;ACMCertificateArn&gt;</code> where <code> <i>ARN for ACM SSL/TLS certificate</i> </code> is the ARN for the ACM SSL/TLS certificate that you want to use for this distribution. </li> </ul> If you specify <code>ACMCertificateArn</code>, you must also specify a value for <code>SSLSupportMethod</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMCertificateId</i> If you want viewers to use HTTPS to request your objects and you're using an alternate domain name, you must choose the type of certificate that you want to use. Specify the following value if you purchased your certificate from a third-party certificate authority: <ul> <li>  <code>&lt;IAMCertificateId&gt;<i>IAM certificate ID</i>&lt;IAMCertificateId&gt;</code> where <code> <i>IAM certificate ID</i> </code> is the ID that IAM returned when you added the certificate to the IAM certificate store. </li> </ul> If you specify <code>IAMCertificateId</code>, you must also specify a value for <code>SSLSupportMethod</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudFrontDefaultCertificate</i> If you're using the CloudFront domain name for your distribution, such as <code>d111111abcdef8.cloudfront.net</code>, specify the following value: <ul> <li>  <code>&lt;CloudFrontDefaultCertificate&gt;true&lt;CloudFrontDefaultCertificate&gt; </code>  </li> </ul><br/>A complex type that controls the countries in which your content is distributed. CloudFront determines the location of your users using <code>MaxMind</code> GeoIP databases.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>GeoRestriction</b>: A complex type that controls the countries in which your content is distributed. CloudFront determines the location of your users using <code>MaxMind</code> GeoIP databases.
        :return: ETag:  The current version of the distribution created.<br/>Location:  The fully qualified URI of the new distribution resource just created. For example: <code>https://cloudfront.amazonaws.com/2010-11-01/distribution/EDFDVBD632BHDS5</code>.<br/>Distribution:  The distribution's information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AliasICPRecordals</i> AWS services in China customers must file for an Internet Content Provider (ICP) recordal if they want to serve content publicly on an alternate domain name, also known as a CNAME, that they've added to CloudFront. AliasICPRecordal provides the ICP recordal status for CNAMEs associated with distributions. For more information about ICP recordals, see <a href="https://docs.amazonaws.cn/en_us/aws/latest/userguide/accounts-and-credentials.html"> Signup, Accounts, and Credentials</a> in <i>Getting Started with AWS services in China</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DistributionConfig</i> The current configuration information for the distribution. Send a <code>GET</code> request to the <code>/<i>CloudFront API version</i>/distribution ID/config</code> resource.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DistributionConfig</b>: The current configuration information for the distribution. Send a <code>GET</code> request to the <code>/<i>CloudFront API version</i>/distribution ID/config</code> resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActiveTrustedSigners</i> CloudFront automatically adds this element to the response only if you've set up the distribution to serve private content with signed URLs. The element lists the key pair IDs that CloudFront is aware of for each trusted signer. The <code>Signer</code> child element lists the AWS account number of the trusted signer (or an empty <code>Self</code> element if the signer is you). The <code>Signer</code> element also includes the IDs of any active key pairs associated with the trusted signer's AWS account. If no <code>KeyPairId</code> element appears for a <code>Signer</code>, that signer can't create working signed URLs.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ActiveTrustedSigners</b>: CloudFront automatically adds this element to the response only if you've set up the distribution to serve private content with signed URLs. The element lists the key pair IDs that CloudFront is aware of for each trusted signer. The <code>Signer</code> child element lists the AWS account number of the trusted signer (or an empty <code>Self</code> element if the signer is you). The <code>Signer</code> element also includes the IDs of any active key pairs associated with the trusted signer's AWS account. If no <code>KeyPairId</code> element appears for a <code>Signer</code>, that signer can't create working signed URLs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainName</i> The domain name corresponding to the distribution, for example, <code>d111111abcdef8.cloudfront.net</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InProgressInvalidationBatches</i> The number of invalidation batches currently in progress. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedTime</i> The date and time the distribution was last modified. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> This response element indicates the current status of the distribution. When the status is <code>Deployed</code>, the distribution's information is fully propagated to all CloudFront edge locations. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ARN</i> The ARN (Amazon Resource Name) for the distribution. For example: <code>arn:aws:cloudfront::123456789012:distribution/EDFDVBD632BHDS5</code>, where <code>123456789012</code> is your AWS account ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier for the distribution. For example: <code>EDFDVBD632BHDS5</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> A complex type that contains one <code>Signer</code> complex type for each trusted signer that is specified in the <code>TrustedSigners</code> complex type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of trusted signers specified in the <code>TrustedSigners</code> complex type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Enabled is <code>true</code> if any of the AWS accounts listed in the <code>TrustedSigners</code> complex type for this distribution have active CloudFront key pairs. If not, <code>Enabled</code> is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique value (for example, a date-time stamp) that ensures that the request can't be replayed. If the value of <code>CallerReference</code> is new (regardless of the content of the <code>DistributionConfig</code> object), CloudFront creates a new distribution. If <code>CallerReference</code> is a value that you already sent in a previous request to create a distribution, CloudFront returns a <code>DistributionAlreadyExists</code> error.<br/>A complex type that contains information about CNAMEs (alternate domain names), if any, for this distribution.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Aliases</b>: A complex type that contains information about CNAMEs (alternate domain names), if any, for this distribution.        """
        pass

    def create_distribution_with_tags(self, DistributionConfigWithTags: DistributionConfigWithTags) -> CreateDistributionWithTagsResult:
        """Create a new distribution with tags.

        :param self:
        :param DistributionConfigWithTags:  The distribution's configuration information. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> A complex type that contains zero or more <code>Tag</code> elements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Tags</b>: A complex type that contains zero or more <code>Tag</code> elements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DistributionConfig</i> A distribution configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DistributionConfig</b>: A distribution configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique value (for example, a date-time stamp) that ensures that the request can't be replayed. If the value of <code>CallerReference</code> is new (regardless of the content of the <code>DistributionConfig</code> object), CloudFront creates a new distribution. If <code>CallerReference</code> is a value that you already sent in a previous request to create a distribution, CloudFront returns a <code>DistributionAlreadyExists</code> error.<br/>A complex type that contains information about CNAMEs (alternate domain names), if any, for this distribution.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Aliases</b>: A complex type that contains information about CNAMEs (alternate domain names), if any, for this distribution.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i>  A complex type that contains <code>Tag</code> elements.<br/>
        :return: ETag:  The current version of the distribution created.<br/>Location:  The fully qualified URI of the new distribution resource just created. For example: <code>https://cloudfront.amazonaws.com/2010-11-01/distribution/EDFDVBD632BHDS5</code>. <br/>Distribution:  The distribution's information. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AliasICPRecordals</i> AWS services in China customers must file for an Internet Content Provider (ICP) recordal if they want to serve content publicly on an alternate domain name, also known as a CNAME, that they've added to CloudFront. AliasICPRecordal provides the ICP recordal status for CNAMEs associated with distributions. For more information about ICP recordals, see <a href="https://docs.amazonaws.cn/en_us/aws/latest/userguide/accounts-and-credentials.html"> Signup, Accounts, and Credentials</a> in <i>Getting Started with AWS services in China</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DistributionConfig</i> The current configuration information for the distribution. Send a <code>GET</code> request to the <code>/<i>CloudFront API version</i>/distribution ID/config</code> resource.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DistributionConfig</b>: The current configuration information for the distribution. Send a <code>GET</code> request to the <code>/<i>CloudFront API version</i>/distribution ID/config</code> resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActiveTrustedSigners</i> CloudFront automatically adds this element to the response only if you've set up the distribution to serve private content with signed URLs. The element lists the key pair IDs that CloudFront is aware of for each trusted signer. The <code>Signer</code> child element lists the AWS account number of the trusted signer (or an empty <code>Self</code> element if the signer is you). The <code>Signer</code> element also includes the IDs of any active key pairs associated with the trusted signer's AWS account. If no <code>KeyPairId</code> element appears for a <code>Signer</code>, that signer can't create working signed URLs.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ActiveTrustedSigners</b>: CloudFront automatically adds this element to the response only if you've set up the distribution to serve private content with signed URLs. The element lists the key pair IDs that CloudFront is aware of for each trusted signer. The <code>Signer</code> child element lists the AWS account number of the trusted signer (or an empty <code>Self</code> element if the signer is you). The <code>Signer</code> element also includes the IDs of any active key pairs associated with the trusted signer's AWS account. If no <code>KeyPairId</code> element appears for a <code>Signer</code>, that signer can't create working signed URLs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainName</i> The domain name corresponding to the distribution, for example, <code>d111111abcdef8.cloudfront.net</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InProgressInvalidationBatches</i> The number of invalidation batches currently in progress. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedTime</i> The date and time the distribution was last modified. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> This response element indicates the current status of the distribution. When the status is <code>Deployed</code>, the distribution's information is fully propagated to all CloudFront edge locations. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ARN</i> The ARN (Amazon Resource Name) for the distribution. For example: <code>arn:aws:cloudfront::123456789012:distribution/EDFDVBD632BHDS5</code>, where <code>123456789012</code> is your AWS account ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier for the distribution. For example: <code>EDFDVBD632BHDS5</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> A complex type that contains one <code>Signer</code> complex type for each trusted signer that is specified in the <code>TrustedSigners</code> complex type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of trusted signers specified in the <code>TrustedSigners</code> complex type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Enabled is <code>true</code> if any of the AWS accounts listed in the <code>TrustedSigners</code> complex type for this distribution have active CloudFront key pairs. If not, <code>Enabled</code> is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique value (for example, a date-time stamp) that ensures that the request can't be replayed. If the value of <code>CallerReference</code> is new (regardless of the content of the <code>DistributionConfig</code> object), CloudFront creates a new distribution. If <code>CallerReference</code> is a value that you already sent in a previous request to create a distribution, CloudFront returns a <code>DistributionAlreadyExists</code> error.<br/>A complex type that contains information about CNAMEs (alternate domain names), if any, for this distribution.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Aliases</b>: A complex type that contains information about CNAMEs (alternate domain names), if any, for this distribution.        """
        pass

    def create_field_level_encryption_config(self, FieldLevelEncryptionConfig: FieldLevelEncryptionConfig) -> CreateFieldLevelEncryptionConfigResult:
        """Create a new field-level encryption configuration.

        :param self:
        :param FieldLevelEncryptionConfig:  The request to create a new field-level encryption configuration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContentTypeProfileConfig</i> A complex data type that specifies when to forward content if a content type isn't recognized and profiles to use as by default in a request if a query argument doesn't specify a profile to use.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ContentTypeProfileConfig</b>: A complex data type that specifies when to forward content if a content type isn't recognized and profiles to use as by default in a request if a query argument doesn't specify a profile to use.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QueryArgProfileConfig</i> A complex data type that specifies when to forward content if a profile isn't found and the profile that can be provided as a query argument in a request.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>QueryArgProfileConfig</b>: A complex data type that specifies when to forward content if a profile isn't found and the profile that can be provided as a query argument in a request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> An optional comment about the configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique number that ensures the request can't be replayed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ForwardWhenQueryArgProfileIsUnknown</i> Flag to set if you want a request to be forwarded to the origin even if the profile specified by the field-level encryption query argument, fle-profile, is unknown.<br/>Profiles specified for query argument-profile mapping for field-level encryption.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>QueryArgProfiles</b>: Profiles specified for query argument-profile mapping for field-level encryption.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ForwardWhenContentTypeIsUnknown</i> The setting in a field-level encryption content type-profile mapping that specifies what to do when an unknown content type is provided for the profile. If true, content is forwarded without being encrypted when the content type is unknown. If false (the default), an error is returned when the content type is unknown. <br/>The configuration for a field-level encryption content type-profile. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ContentTypeProfiles</b>: The configuration for a field-level encryption content type-profile. 
        :return: ETag:  The current version of the field level encryption configuration. For example: <code>E2QWRUHAPOMQZL</code>.<br/>Location:  The fully qualified URI of the new configuration resource just created. For example: <code>https://cloudfront.amazonaws.com/2010-11-01/field-level-encryption-config/EDFDVBD632BHDS5</code>.<br/>FieldLevelEncryption:  Returned when you create a new field-level encryption configuration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FieldLevelEncryptionConfig</i> A complex data type that includes the profile configurations specified for field-level encryption. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>FieldLevelEncryptionConfig</b>: A complex data type that includes the profile configurations specified for field-level encryption. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedTime</i> The last time the field-level encryption configuration was changed. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The configuration ID for a field-level encryption configuration which includes a set of profiles that specify certain selected data fields to be encrypted by specific public keys.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> An optional comment about the configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique number that ensures the request can't be replayed.<br/>A complex data type that specifies when to forward content if a profile isn't found and the profile that can be provided as a query argument in a request.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>QueryArgProfileConfig</b>: A complex data type that specifies when to forward content if a profile isn't found and the profile that can be provided as a query argument in a request.        """
        pass

    def create_field_level_encryption_profile(self, FieldLevelEncryptionProfileConfig: FieldLevelEncryptionProfileConfig) -> CreateFieldLevelEncryptionProfileResult:
        """Create a field-level encryption profile.

        :param self:
        :param FieldLevelEncryptionProfileConfig:  The request to create a field-level encryption profile.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionEntities</i> A complex data type of encryption entities for the field-level encryption profile that include the public key ID, provider, and field patterns for specifying which fields to encrypt with this key.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncryptionEntities</b>: A complex data type of encryption entities for the field-level encryption profile that include the public key ID, provider, and field patterns for specifying which fields to encrypt with this key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> An optional comment for the field-level encryption profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique number that ensures that the request can't be replayed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Profile name for the field-level encryption profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> An array of field patterns in a field-level encryption content type-profile mapping. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> Number of field pattern items in a field-level encryption content type-profile mapping. <br/>
        :return: ETag:  The current version of the field level encryption profile. For example: <code>E2QWRUHAPOMQZL</code>.<br/>Location:  The fully qualified URI of the new profile resource just created. For example: <code>https://cloudfront.amazonaws.com/2010-11-01/field-level-encryption-profile/EDFDVBD632BHDS5</code>.<br/>FieldLevelEncryptionProfile:  Returned when you create a new field-level encryption profile.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FieldLevelEncryptionProfileConfig</i> A complex data type that includes the profile name and the encryption entities for the field-level encryption profile.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>FieldLevelEncryptionProfileConfig</b>: A complex data type that includes the profile name and the encryption entities for the field-level encryption profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedTime</i> The last time the field-level encryption profile was updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID for a field-level encryption profile configuration which includes a set of profiles that specify certain selected data fields to be encrypted by specific public keys.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> An optional comment for the field-level encryption profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique number that ensures that the request can't be replayed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Profile name for the field-level encryption profile.<br/>A complex data type of encryption entities for the field-level encryption profile that include the public key ID, provider, and field patterns for specifying which fields to encrypt with this key.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncryptionEntities</b>: A complex data type of encryption entities for the field-level encryption profile that include the public key ID, provider, and field patterns for specifying which fields to encrypt with this key.        """
        pass

    def create_invalidation(self, InvalidationBatch: InvalidationBatch, DistributionId: str) -> CreateInvalidationResult:
        """Create a new invalidation. 

        :param self:
        :param InvalidationBatch:  The batch information for the invalidation.:<br/>DistributionId:  The distribution's id.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A value that you specify to uniquely identify an invalidation request. CloudFront uses the value to prevent you from accidentally resubmitting an identical request. Whenever you create a new invalidation request, you must specify a new value for <code>CallerReference</code> and change other values in the request as applicable. One way to ensure that the value of <code>CallerReference</code> is unique is to use a <code>timestamp</code>, for example, <code>20120301090000</code>. If you make a second invalidation request with the same value for <code>CallerReference</code>, and if the rest of the request is the same, CloudFront doesn't create a new invalidation request. Instead, CloudFront returns information about the invalidation request that you previously created with the same <code>CallerReference</code>. If <code>CallerReference</code> is a value you already sent in a previous invalidation batch request but the content of any <code>Path</code> is different from the original request, CloudFront returns an <code>InvalidationBatchAlreadyExists</code> error.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Paths</i> A complex type that contains information about the objects that you want to invalidate. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/Invalidation.html#invalidation-specifying-objects">Specifying the Objects to Invalidate</a> in the <i>Amazon CloudFront Developer Guide</i>. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Paths</b>: A complex type that contains information about the objects that you want to invalidate. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/Invalidation.html#invalidation-specifying-objects">Specifying the Objects to Invalidate</a> in the <i>Amazon CloudFront Developer Guide</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> A complex type that contains a list of the paths that you want to invalidate.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of invalidation paths specified for the objects that you want to invalidate.<br/>
        :return: Invalidation:  The invalidation's information.:<br/>Location:  The fully qualified URI of the distribution and invalidation batch request, including the <code>Invalidation ID</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InvalidationBatch</i> The current invalidation information for the batch request. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InvalidationBatch</b>: The current invalidation information for the batch request. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateTime</i> The date and time the invalidation request was first made. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the invalidation request. When the invalidation batch is finished, the status is <code>Completed</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier for the invalidation request. For example: <code>IDFDVBD632BHDS5</code>.<br/>A complex type that contains information about the objects that you want to invalidate. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/Invalidation.html#invalidation-specifying-objects">Specifying the Objects to Invalidate</a> in the <i>Amazon CloudFront Developer Guide</i>. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Paths</b>: A complex type that contains information about the objects that you want to invalidate. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/Invalidation.html#invalidation-specifying-objects">Specifying the Objects to Invalidate</a> in the <i>Amazon CloudFront Developer Guide</i>.         """
        pass

    def create_public_key(self, PublicKeyConfig: PublicKeyConfig) -> CreatePublicKeyResult:
        """Add a new public key to CloudFront to use, for example, for field-level encryption. You can add a maximum of 10 public keys with one AWS account.

        :param self:
        :param PublicKeyConfig:  The request to add a public key to CloudFront.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> An optional comment about a public key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncodedKey</i> The encoded public key that you want to add to CloudFront to use with features like field-level encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name for a public key you add to CloudFront to use with features like field-level encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique number that ensures that the request can't be replayed.<br/>
        :return: ETag:  The current version of the public key. For example: <code>E2QWRUHAPOMQZL</code>.<br/>Location:  The fully qualified URI of the new public key resource just created. For example: <code>https://cloudfront.amazonaws.com/2010-11-01/cloudfront-public-key/EDFDVBD632BHDS5</code>.<br/>PublicKey:  Returned when you add a public key.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PublicKeyConfig</i> A complex data type for a public key you add to CloudFront to use with features like field-level encryption.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PublicKeyConfig</b>: A complex data type for a public key you add to CloudFront to use with features like field-level encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> A time you added a public key to CloudFront.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> A unique ID assigned to a public key you've added to CloudFront.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> An optional comment about a public key.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncodedKey</i> The encoded public key that you want to add to CloudFront to use with features like field-level encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name for a public key you add to CloudFront to use with features like field-level encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique number that ensures that the request can't be replayed.<br/>        """
        pass

    def create_streaming_distribution(self, StreamingDistributionConfig: StreamingDistributionConfig) -> CreateStreamingDistributionResult:
        """Creates a new RTMP distribution. An RTMP distribution is similar to a web distribution, but an RTMP distribution streams media files using the Adobe Real-Time Messaging Protocol (RTMP) instead of serving files using HTTP.  To create a new distribution, submit a <code>POST</code> request to the <i>CloudFront API version</i>/distribution resource. The request body must include a document with a <i>StreamingDistributionConfig</i> element. The response echoes the <code>StreamingDistributionConfig</code> element and returns other information about the RTMP distribution. To get the status of your request, use the <i>GET StreamingDistribution</i> API action. When the value of <code>Enabled</code> is <code>true</code> and the value of <code>Status</code> is <code>Deployed</code>, your distribution is ready. A distribution usually deploys in less than 15 minutes. For more information about web distributions, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/distribution-rtmp.html">Working with RTMP Distributions</a> in the <i>Amazon CloudFront Developer Guide</i>. <important> Beginning with the 2012-05-05 version of the CloudFront API, we made substantial changes to the format of the XML document that you include in the request body when you create or update a web distribution or an RTMP distribution, and when you invalidate objects. With previous versions of the API, we discovered that it was too easy to accidentally delete one or more values for an element that accepts multiple values, for example, CNAMEs and trusted signers. Our changes for the 2012-05-05 release are intended to prevent these accidental deletions and to notify you when there's a mismatch between the number of values you say you're specifying in the <code>Quantity</code> element and the number of values specified. </important>

        :param self:
        :param StreamingDistributionConfig:  The streaming distribution's configuration information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Whether the streaming distribution is enabled to accept user requests for content.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PriceClass</i> A complex type that contains information about price class for this streaming distribution. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrustedSigners</i> A complex type that specifies any AWS accounts that you want to permit to create signed URLs for private content. If you want the distribution to use signed URLs, include this element; if you want the distribution to use public URLs, remove this element. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/PrivateContent.html">Serving Private Content through CloudFront</a> in the <i>Amazon CloudFront Developer Guide</i>. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TrustedSigners</b>: A complex type that specifies any AWS accounts that you want to permit to create signed URLs for private content. If you want the distribution to use signed URLs, include this element; if you want the distribution to use public URLs, remove this element. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/PrivateContent.html">Serving Private Content through CloudFront</a> in the <i>Amazon CloudFront Developer Guide</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Logging</i> A complex type that controls whether access logs are written for the streaming distribution. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Logging</b>: A complex type that controls whether access logs are written for the streaming distribution. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> Any comments you want to include about the streaming distribution. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Aliases</i> A complex type that contains information about CNAMEs (alternate domain names), if any, for this streaming distribution. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Aliases</b>: A complex type that contains information about CNAMEs (alternate domain names), if any, for this streaming distribution. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Origin</i> A complex type that contains information about the Amazon S3 bucket from which you want CloudFront to get your media files for distribution. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Origin</b>: A complex type that contains information about the Amazon S3 bucket from which you want CloudFront to get your media files for distribution. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique value (for example, a date-time stamp) that ensures that the request can't be replayed. If the value of <code>CallerReference</code> is new (regardless of the content of the <code>StreamingDistributionConfig</code> object), CloudFront creates a new distribution. If <code>CallerReference</code> is a value that you already sent in a previous request to create a distribution, CloudFront returns a <code>DistributionAlreadyExists</code> error.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OriginAccessIdentity</i> The CloudFront origin access identity to associate with the distribution. Use an origin access identity to configure the distribution so that end users can only access objects in an Amazon S3 bucket through CloudFront. If you want end users to be able to access objects using either the CloudFront URL or the Amazon S3 URL, specify an empty <code>OriginAccessIdentity</code> element. To delete the origin access identity from an existing distribution, update the distribution configuration and include an empty <code>OriginAccessIdentity</code> element. To replace the origin access identity, update the distribution configuration and specify the new origin access identity. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/private-content-restricting-access-to-s3.html">Using an Origin Access Identity to Restrict Access to Your Amazon S3 Content</a> in the <i> Amazon CloudFront Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainName</i> The DNS name of the Amazon S3 origin. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> A complex type that contains the CNAME aliases, if any, that you want to associate with this distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of CNAME aliases, if any, that you want to associate with this distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> An optional string that you want CloudFront to prefix to the access log filenames for this streaming distribution, for example, <code>myprefix/</code>. If you want to enable logging, but you don't want to specify a prefix, you still must include an empty <code>Prefix</code> element in the <code>Logging</code> element.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> The Amazon S3 bucket to store the access logs in, for example, <code>myawslogbucket.s3.amazonaws.com</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether you want CloudFront to save access logs to an Amazon S3 bucket. If you don't want to enable logging when you create a streaming distribution or if you want to disable logging for an existing streaming distribution, specify <code>false</code> for <code>Enabled</code>, and specify <code>empty Bucket</code> and <code>Prefix</code> elements. If you specify <code>false</code> for <code>Enabled</code> but you specify values for <code>Bucket</code> and <code>Prefix</code>, the values are automatically deleted. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i>  <b>Optional</b>: A complex type that contains trusted signers for this cache behavior. If <code>Quantity</code> is <code>0</code>, you can omit <code>Items</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of trusted signers for this cache behavior.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether you want to require viewers to use signed URLs to access the files specified by <code>PathPattern</code> and <code>TargetOriginId</code>.<br/>
        :return: ETag:  The current version of the streaming distribution created.<br/>Location:  The fully qualified URI of the new streaming distribution resource just created. For example: <code>https://cloudfront.amazonaws.com/2010-11-01/streaming-distribution/EGTXBD79H29TRA8</code>.<br/>StreamingDistribution:  The streaming distribution's information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamingDistributionConfig</i> The current configuration information for the RTMP distribution.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamingDistributionConfig</b>: The current configuration information for the RTMP distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActiveTrustedSigners</i> A complex type that lists the AWS accounts, if any, that you included in the <code>TrustedSigners</code> complex type for this distribution. These are the accounts that you want to allow to create signed URLs for private content. The <code>Signer</code> complex type lists the AWS account number of the trusted signer or <code>self</code> if the signer is the AWS account that created the distribution. The <code>Signer</code> element also includes the IDs of any active CloudFront key pairs that are associated with the trusted signer's AWS account. If no <code>KeyPairId</code> element appears for a <code>Signer</code>, that signer can't create signed URLs. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/PrivateContent.html">Serving Private Content through CloudFront</a> in the <i>Amazon CloudFront Developer Guide</i>. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ActiveTrustedSigners</b>: A complex type that lists the AWS accounts, if any, that you included in the <code>TrustedSigners</code> complex type for this distribution. These are the accounts that you want to allow to create signed URLs for private content. The <code>Signer</code> complex type lists the AWS account number of the trusted signer or <code>self</code> if the signer is the AWS account that created the distribution. The <code>Signer</code> element also includes the IDs of any active CloudFront key pairs that are associated with the trusted signer's AWS account. If no <code>KeyPairId</code> element appears for a <code>Signer</code>, that signer can't create signed URLs. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/PrivateContent.html">Serving Private Content through CloudFront</a> in the <i>Amazon CloudFront Developer Guide</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainName</i> The domain name that corresponds to the streaming distribution, for example, <code>s5c39gqb8ow64r.cloudfront.net</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedTime</i> The date and time that the distribution was last modified. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current status of the RTMP distribution. When the status is <code>Deployed</code>, the distribution's information is propagated to all CloudFront edge locations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ARN</i> The ARN (Amazon Resource Name) for the distribution. For example: <code>arn:aws:cloudfront::123456789012:distribution/EDFDVBD632BHDS5</code>, where <code>123456789012</code> is your AWS account ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier for the RTMP distribution. For example: <code>EGTXBD79EXAMPLE</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> A complex type that contains one <code>Signer</code> complex type for each trusted signer that is specified in the <code>TrustedSigners</code> complex type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of trusted signers specified in the <code>TrustedSigners</code> complex type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Enabled is <code>true</code> if any of the AWS accounts listed in the <code>TrustedSigners</code> complex type for this distribution have active CloudFront key pairs. If not, <code>Enabled</code> is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique value (for example, a date-time stamp) that ensures that the request can't be replayed. If the value of <code>CallerReference</code> is new (regardless of the content of the <code>StreamingDistributionConfig</code> object), CloudFront creates a new distribution. If <code>CallerReference</code> is a value that you already sent in a previous request to create a distribution, CloudFront returns a <code>DistributionAlreadyExists</code> error.<br/>A complex type that contains information about the Amazon S3 bucket from which you want CloudFront to get your media files for distribution. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Origin</b>: A complex type that contains information about the Amazon S3 bucket from which you want CloudFront to get your media files for distribution.         """
        pass

    def create_streaming_distribution_with_tags(self, StreamingDistributionConfigWithTags: StreamingDistributionConfigWithTags) -> CreateStreamingDistributionWithTagsResult:
        """Create a new streaming distribution with tags.

        :param self:
        :param StreamingDistributionConfigWithTags:   The streaming distribution's configuration information. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> A complex type that contains zero or more <code>Tag</code> elements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Tags</b>: A complex type that contains zero or more <code>Tag</code> elements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamingDistributionConfig</i> A streaming distribution Configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamingDistributionConfig</b>: A streaming distribution Configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique value (for example, a date-time stamp) that ensures that the request can't be replayed. If the value of <code>CallerReference</code> is new (regardless of the content of the <code>StreamingDistributionConfig</code> object), CloudFront creates a new distribution. If <code>CallerReference</code> is a value that you already sent in a previous request to create a distribution, CloudFront returns a <code>DistributionAlreadyExists</code> error.<br/>A complex type that contains information about the Amazon S3 bucket from which you want CloudFront to get your media files for distribution. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Origin</b>: A complex type that contains information about the Amazon S3 bucket from which you want CloudFront to get your media files for distribution. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i>  A complex type that contains <code>Tag</code> elements.<br/>
        :return: ETag:  The current version of the distribution created.<br/>Location:  The fully qualified URI of the new streaming distribution resource just created. For example:<code> https://cloudfront.amazonaws.com/2010-11-01/streaming-distribution/EGTXBD79H29TRA8</code>.<br/>StreamingDistribution:  The streaming distribution's information. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamingDistributionConfig</i> The current configuration information for the RTMP distribution.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamingDistributionConfig</b>: The current configuration information for the RTMP distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActiveTrustedSigners</i> A complex type that lists the AWS accounts, if any, that you included in the <code>TrustedSigners</code> complex type for this distribution. These are the accounts that you want to allow to create signed URLs for private content. The <code>Signer</code> complex type lists the AWS account number of the trusted signer or <code>self</code> if the signer is the AWS account that created the distribution. The <code>Signer</code> element also includes the IDs of any active CloudFront key pairs that are associated with the trusted signer's AWS account. If no <code>KeyPairId</code> element appears for a <code>Signer</code>, that signer can't create signed URLs. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/PrivateContent.html">Serving Private Content through CloudFront</a> in the <i>Amazon CloudFront Developer Guide</i>. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ActiveTrustedSigners</b>: A complex type that lists the AWS accounts, if any, that you included in the <code>TrustedSigners</code> complex type for this distribution. These are the accounts that you want to allow to create signed URLs for private content. The <code>Signer</code> complex type lists the AWS account number of the trusted signer or <code>self</code> if the signer is the AWS account that created the distribution. The <code>Signer</code> element also includes the IDs of any active CloudFront key pairs that are associated with the trusted signer's AWS account. If no <code>KeyPairId</code> element appears for a <code>Signer</code>, that signer can't create signed URLs. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/PrivateContent.html">Serving Private Content through CloudFront</a> in the <i>Amazon CloudFront Developer Guide</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainName</i> The domain name that corresponds to the streaming distribution, for example, <code>s5c39gqb8ow64r.cloudfront.net</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedTime</i> The date and time that the distribution was last modified. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current status of the RTMP distribution. When the status is <code>Deployed</code>, the distribution's information is propagated to all CloudFront edge locations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ARN</i> The ARN (Amazon Resource Name) for the distribution. For example: <code>arn:aws:cloudfront::123456789012:distribution/EDFDVBD632BHDS5</code>, where <code>123456789012</code> is your AWS account ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier for the RTMP distribution. For example: <code>EGTXBD79EXAMPLE</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> A complex type that contains one <code>Signer</code> complex type for each trusted signer that is specified in the <code>TrustedSigners</code> complex type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of trusted signers specified in the <code>TrustedSigners</code> complex type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Enabled is <code>true</code> if any of the AWS accounts listed in the <code>TrustedSigners</code> complex type for this distribution have active CloudFront key pairs. If not, <code>Enabled</code> is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique value (for example, a date-time stamp) that ensures that the request can't be replayed. If the value of <code>CallerReference</code> is new (regardless of the content of the <code>StreamingDistributionConfig</code> object), CloudFront creates a new distribution. If <code>CallerReference</code> is a value that you already sent in a previous request to create a distribution, CloudFront returns a <code>DistributionAlreadyExists</code> error.<br/>A complex type that contains information about the Amazon S3 bucket from which you want CloudFront to get your media files for distribution. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Origin</b>: A complex type that contains information about the Amazon S3 bucket from which you want CloudFront to get your media files for distribution.         """
        pass

    def delete_cloud_front_origin_access_identity(self, Id: str, IfMatch: str=None):
        """Delete an origin access identity. 

        :param self:
        :param IfMatch:  The value of the <code>ETag</code> header you received from a previous <code>GET</code> or <code>PUT</code> request. For example: <code>E2QWRUHAPOMQZL</code>.<br/>Id:  The origin access identity's ID.<br/>
        :return:         """
        pass

    def delete_distribution(self, Id: str, IfMatch: str=None):
        """Delete a distribution. 

        :param self:
        :param IfMatch:  The value of the <code>ETag</code> header that you received when you disabled the distribution. For example: <code>E2QWRUHAPOMQZL</code>. <br/>Id:  The distribution ID. <br/>
        :return:         """
        pass

    def delete_field_level_encryption_config(self, Id: str, IfMatch: str=None):
        """Remove a field-level encryption configuration.

        :param self:
        :param IfMatch:  The value of the <code>ETag</code> header that you received when retrieving the configuration identity to delete. For example: <code>E2QWRUHAPOMQZL</code>.<br/>Id:  The ID of the configuration you want to delete from CloudFront.<br/>
        :return:         """
        pass

    def delete_field_level_encryption_profile(self, Id: str, IfMatch: str=None):
        """Remove a field-level encryption profile.

        :param self:
        :param IfMatch:  The value of the <code>ETag</code> header that you received when retrieving the profile to delete. For example: <code>E2QWRUHAPOMQZL</code>.<br/>Id:  Request the ID of the profile you want to delete from CloudFront.<br/>
        :return:         """
        pass

    def delete_public_key(self, Id: str, IfMatch: str=None):
        """Remove a public key you previously added to CloudFront.

        :param self:
        :param IfMatch:  The value of the <code>ETag</code> header that you received when retrieving the public key identity to delete. For example: <code>E2QWRUHAPOMQZL</code>.<br/>Id:  The ID of the public key you want to remove from CloudFront.<br/>
        :return:         """
        pass

    def delete_streaming_distribution(self, Id: str, IfMatch: str=None):
        """Delete a streaming distribution. To delete an RTMP distribution using the CloudFront API, perform the following steps.  <b>To delete an RTMP distribution using the CloudFront API</b>: <ol> <li> Disable the RTMP distribution. </li> <li> Submit a <code>GET Streaming Distribution Config</code> request to get the current configuration and the <code>Etag</code> header for the distribution.  </li> <li> Update the XML document that was returned in the response to your <code>GET Streaming Distribution Config</code> request to change the value of <code>Enabled</code> to <code>false</code>. </li> <li> Submit a <code>PUT Streaming Distribution Config</code> request to update the configuration for your distribution. In the request body, include the XML document that you updated in Step 3. Then set the value of the HTTP <code>If-Match</code> header to the value of the <code>ETag</code> header that CloudFront returned when you submitted the <code>GET Streaming Distribution Config</code> request in Step 2. </li> <li> Review the response to the <code>PUT Streaming Distribution Config</code> request to confirm that the distribution was successfully disabled. </li> <li> Submit a <code>GET Streaming Distribution Config</code> request to confirm that your changes have propagated. When propagation is complete, the value of <code>Status</code> is <code>Deployed</code>. </li> <li> Submit a <code>DELETE Streaming Distribution</code> request. Set the value of the HTTP <code>If-Match</code> header to the value of the <code>ETag</code> header that CloudFront returned when you submitted the <code>GET Streaming Distribution Config</code> request in Step 2. </li> <li> Review the response to your <code>DELETE Streaming Distribution</code> request to confirm that the distribution was successfully deleted. </li> </ol> For information about deleting a distribution using the CloudFront console, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/HowToDeleteDistribution.html">Deleting a Distribution</a> in the <i>Amazon CloudFront Developer Guide</i>.

        :param self:
        :param IfMatch:  The value of the <code>ETag</code> header that you received when you disabled the streaming distribution. For example: <code>E2QWRUHAPOMQZL</code>.<br/>Id:  The distribution ID. <br/>
        :return:         """
        pass

    def get_cloud_front_origin_access_identity(self, Id: str) -> GetCloudFrontOriginAccessIdentityResult:
        """Get the information about an origin access identity. 

        :param self:
        :param Id:  The identity's ID.<br/>
        :return: ETag:  The current version of the origin access identity's information. For example: <code>E2QWRUHAPOMQZL</code>.<br/>CloudFrontOriginAccessIdentity:  The origin access identity's information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudFrontOriginAccessIdentityConfig</i> The current configuration information for the identity. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CloudFrontOriginAccessIdentityConfig</b>: The current configuration information for the identity. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3CanonicalUserId</i> The Amazon S3 canonical user ID for the origin access identity, used when giving the origin access identity read permission to an object in Amazon S3. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID for the origin access identity, for example, <code>E74FTE3AJFJ256A</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> Any comments you want to include about the origin access identity. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique value (for example, a date-time stamp) that ensures that the request can't be replayed. If the value of <code>CallerReference</code> is new (regardless of the content of the <code>CloudFrontOriginAccessIdentityConfig</code> object), a new origin access identity is created. If the <code>CallerReference</code> is a value already sent in a previous identity request, and the content of the <code>CloudFrontOriginAccessIdentityConfig</code> is identical to the original request (ignoring white space), the response includes the same information returned to the original request.  If the <code>CallerReference</code> is a value you already sent in a previous request to create an identity, but the content of the <code>CloudFrontOriginAccessIdentityConfig</code> is different from the original request, CloudFront returns a <code>CloudFrontOriginAccessIdentityAlreadyExists</code> error. <br/>        """
        pass

    def get_cloud_front_origin_access_identity_config(self, Id: str) -> GetCloudFrontOriginAccessIdentityConfigResult:
        """Get the configuration information about an origin access identity. 

        :param self:
        :param Id:  The identity's ID. <br/>
        :return: ETag:  The current version of the configuration. For example: <code>E2QWRUHAPOMQZL</code>.<br/>CloudFrontOriginAccessIdentityConfig:  The origin access identity's configuration information. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> Any comments you want to include about the origin access identity. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique value (for example, a date-time stamp) that ensures that the request can't be replayed. If the value of <code>CallerReference</code> is new (regardless of the content of the <code>CloudFrontOriginAccessIdentityConfig</code> object), a new origin access identity is created. If the <code>CallerReference</code> is a value already sent in a previous identity request, and the content of the <code>CloudFrontOriginAccessIdentityConfig</code> is identical to the original request (ignoring white space), the response includes the same information returned to the original request.  If the <code>CallerReference</code> is a value you already sent in a previous request to create an identity, but the content of the <code>CloudFrontOriginAccessIdentityConfig</code> is different from the original request, CloudFront returns a <code>CloudFrontOriginAccessIdentityAlreadyExists</code> error. <br/>        """
        pass

    def get_distribution(self, Id: str) -> GetDistributionResult:
        """Get the information about a distribution.

        :param self:
        :param Id:  The distribution's ID. If the ID is empty, an empty distribution configuration is returned.<br/>
        :return: ETag:  The current version of the distribution's information. For example: <code>E2QWRUHAPOMQZL</code>.<br/>Distribution:  The distribution's information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AliasICPRecordals</i> AWS services in China customers must file for an Internet Content Provider (ICP) recordal if they want to serve content publicly on an alternate domain name, also known as a CNAME, that they've added to CloudFront. AliasICPRecordal provides the ICP recordal status for CNAMEs associated with distributions. For more information about ICP recordals, see <a href="https://docs.amazonaws.cn/en_us/aws/latest/userguide/accounts-and-credentials.html"> Signup, Accounts, and Credentials</a> in <i>Getting Started with AWS services in China</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DistributionConfig</i> The current configuration information for the distribution. Send a <code>GET</code> request to the <code>/<i>CloudFront API version</i>/distribution ID/config</code> resource.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DistributionConfig</b>: The current configuration information for the distribution. Send a <code>GET</code> request to the <code>/<i>CloudFront API version</i>/distribution ID/config</code> resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActiveTrustedSigners</i> CloudFront automatically adds this element to the response only if you've set up the distribution to serve private content with signed URLs. The element lists the key pair IDs that CloudFront is aware of for each trusted signer. The <code>Signer</code> child element lists the AWS account number of the trusted signer (or an empty <code>Self</code> element if the signer is you). The <code>Signer</code> element also includes the IDs of any active key pairs associated with the trusted signer's AWS account. If no <code>KeyPairId</code> element appears for a <code>Signer</code>, that signer can't create working signed URLs.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ActiveTrustedSigners</b>: CloudFront automatically adds this element to the response only if you've set up the distribution to serve private content with signed URLs. The element lists the key pair IDs that CloudFront is aware of for each trusted signer. The <code>Signer</code> child element lists the AWS account number of the trusted signer (or an empty <code>Self</code> element if the signer is you). The <code>Signer</code> element also includes the IDs of any active key pairs associated with the trusted signer's AWS account. If no <code>KeyPairId</code> element appears for a <code>Signer</code>, that signer can't create working signed URLs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainName</i> The domain name corresponding to the distribution, for example, <code>d111111abcdef8.cloudfront.net</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InProgressInvalidationBatches</i> The number of invalidation batches currently in progress. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedTime</i> The date and time the distribution was last modified. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> This response element indicates the current status of the distribution. When the status is <code>Deployed</code>, the distribution's information is fully propagated to all CloudFront edge locations. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ARN</i> The ARN (Amazon Resource Name) for the distribution. For example: <code>arn:aws:cloudfront::123456789012:distribution/EDFDVBD632BHDS5</code>, where <code>123456789012</code> is your AWS account ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier for the distribution. For example: <code>EDFDVBD632BHDS5</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> A complex type that contains one <code>Signer</code> complex type for each trusted signer that is specified in the <code>TrustedSigners</code> complex type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of trusted signers specified in the <code>TrustedSigners</code> complex type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Enabled is <code>true</code> if any of the AWS accounts listed in the <code>TrustedSigners</code> complex type for this distribution have active CloudFront key pairs. If not, <code>Enabled</code> is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique value (for example, a date-time stamp) that ensures that the request can't be replayed. If the value of <code>CallerReference</code> is new (regardless of the content of the <code>DistributionConfig</code> object), CloudFront creates a new distribution. If <code>CallerReference</code> is a value that you already sent in a previous request to create a distribution, CloudFront returns a <code>DistributionAlreadyExists</code> error.<br/>A complex type that contains information about CNAMEs (alternate domain names), if any, for this distribution.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Aliases</b>: A complex type that contains information about CNAMEs (alternate domain names), if any, for this distribution.        """
        pass

    def get_distribution_config(self, Id: str) -> GetDistributionConfigResult:
        """Get the configuration information about a distribution. 

        :param self:
        :param Id:  The distribution's ID. If the ID is empty, an empty distribution configuration is returned.<br/>
        :return: ETag:  The current version of the configuration. For example: <code>E2QWRUHAPOMQZL</code>.<br/>DistributionConfig:  The distribution's configuration information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsIPV6Enabled</i> If you want CloudFront to respond to IPv6 DNS requests with an IPv6 address for your distribution, specify <code>true</code>. If you specify <code>false</code>, CloudFront responds to IPv6 DNS requests with the DNS response code <code>NOERROR</code> and with no IP addresses. This allows viewers to submit a second request, for an IPv4 address for your distribution.  In general, you should enable IPv6 if you have users on IPv6 networks who want to access your content. However, if you're using signed URLs or signed cookies to restrict access to your content, and if you're using a custom policy that includes the <code>IpAddress</code> parameter to restrict the IP addresses that can access your content, don't enable IPv6. If you want to restrict access to some content by IP address and not restrict access to other content (or restrict access but not by IP address), you can create two distributions. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/private-content-creating-signed-url-custom-policy.html">Creating a Signed URL Using a Custom Policy</a> in the <i>Amazon CloudFront Developer Guide</i>. If you're using an Amazon Route 53 alias resource record set to route traffic to your CloudFront distribution, you need to create a second alias resource record set when both of the following are true: <ul> <li> You enable IPv6 for the distribution </li> <li> You're using alternate domain names in the URLs for your objects </li> </ul> For more information, see <a href="https://docs.aws.amazon.com/Route53/latest/DeveloperGuide/routing-to-cloudfront-distribution.html">Routing Traffic to an Amazon CloudFront Web Distribution by Using Your Domain Name</a> in the <i>Amazon Route 53 Developer Guide</i>. If you created a CNAME resource record set, either with Amazon Route 53 or with another DNS service, you don't need to make any changes. A CNAME record will route traffic to your distribution regardless of the IP address format of the viewer request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HttpVersion</i> (Optional) Specify the maximum HTTP version that you want viewers to use to communicate with CloudFront. The default value for new web distributions is http2. Viewers that don't support HTTP/2 automatically use an earlier HTTP version. For viewers and CloudFront to use HTTP/2, viewers must support TLS 1.2 or later, and must support Server Name Identification (SNI). In general, configuring CloudFront to communicate with viewers using HTTP/2 reduces latency. You can improve performance by optimizing for HTTP/2. For more information, do an Internet search for "http/2 optimization." <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WebACLId</i> A unique identifier that specifies the AWS WAF web ACL, if any, to associate with this distribution. AWS WAF is a web application firewall that lets you monitor the HTTP and HTTPS requests that are forwarded to CloudFront, and lets you control access to your content. Based on conditions that you specify, such as the IP addresses that requests originate from or the values of query strings, CloudFront responds to requests either with the requested content or with an HTTP 403 status code (Forbidden). You can also configure CloudFront to return a custom error page when a request is blocked. For more information about AWS WAF, see the <a href="http://docs.aws.amazon.com/waf/latest/developerguide/what-is-aws-waf.html">AWS WAF Developer Guide</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Restrictions</i> A complex type that identifies ways in which you want to restrict distribution of your content.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Restrictions</b>: A complex type that identifies ways in which you want to restrict distribution of your content.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ViewerCertificate</i> A complex type that specifies whether you want viewers to use HTTP or HTTPS to request your objects, whether you're using an alternate domain name with HTTPS, and if so, if you're using AWS Certificate Manager (ACM) or a third-party certificate authority.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ViewerCertificate</b>: A complex type that specifies whether you want viewers to use HTTP or HTTPS to request your objects, whether you're using an alternate domain name with HTTPS, and if so, if you're using AWS Certificate Manager (ACM) or a third-party certificate authority.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> From this field, you can enable or disable the selected distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PriceClass</i> The price class that corresponds with the maximum price that you want to pay for CloudFront service. If you specify <code>PriceClass_All</code>, CloudFront responds to requests for your objects from all CloudFront edge locations. If you specify a price class other than <code>PriceClass_All</code>, CloudFront serves your objects from the CloudFront edge location that has the lowest latency among the edge locations in your price class. Viewers who are in or near regions that are excluded from your specified price class may encounter slower performance. For more information about price classes, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/PriceClass.html">Choosing the Price Class for a CloudFront Distribution</a> in the <i>Amazon CloudFront Developer Guide</i>. For information about CloudFront pricing, including how price classes (such as Price Class 100) map to CloudFront regions, see <a href="http://aws.amazon.com/cloudfront/pricing/">Amazon CloudFront Pricing</a>. For price class information, scroll down to see the table at the bottom of the page.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Logging</i> A complex type that controls whether access logs are written for the distribution. For more information about logging, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/AccessLogs.html">Access Logs</a> in the <i>Amazon CloudFront Developer Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Logging</b>: A complex type that controls whether access logs are written for the distribution. For more information about logging, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/AccessLogs.html">Access Logs</a> in the <i>Amazon CloudFront Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> Any comments you want to include about the distribution. If you don't want to specify a comment, include an empty <code>Comment</code> element. To delete an existing comment, update the distribution configuration and include an empty <code>Comment</code> element. To add or change a comment, update the distribution configuration and specify the new comment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomErrorResponses</i> A complex type that controls the following: <ul> <li> Whether CloudFront replaces HTTP status codes in the 4xx and 5xx range with custom error messages before returning the response to the viewer. </li> <li> How long CloudFront caches HTTP status codes in the 4xx and 5xx range. </li> </ul> For more information about custom error pages, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/custom-error-pages.html">Customizing Error Responses</a> in the <i>Amazon CloudFront Developer Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CustomErrorResponses</b>: A complex type that controls the following: <ul> <li> Whether CloudFront replaces HTTP status codes in the 4xx and 5xx range with custom error messages before returning the response to the viewer. </li> <li> How long CloudFront caches HTTP status codes in the 4xx and 5xx range. </li> </ul> For more information about custom error pages, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/custom-error-pages.html">Customizing Error Responses</a> in the <i>Amazon CloudFront Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheBehaviors</i> A complex type that contains zero or more <code>CacheBehavior</code> elements. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CacheBehaviors</b>: A complex type that contains zero or more <code>CacheBehavior</code> elements. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultCacheBehavior</i> A complex type that describes the default cache behavior if you don't specify a <code>CacheBehavior</code> element or if files don't match any of the values of <code>PathPattern</code> in <code>CacheBehavior</code> elements. You must create exactly one default cache behavior.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DefaultCacheBehavior</b>: A complex type that describes the default cache behavior if you don't specify a <code>CacheBehavior</code> element or if files don't match any of the values of <code>PathPattern</code> in <code>CacheBehavior</code> elements. You must create exactly one default cache behavior.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OriginGroups</i>  A complex type that contains information about origin groups for this distribution.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>OriginGroups</b>:  A complex type that contains information about origin groups for this distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Origins</i> A complex type that contains information about origins for this distribution. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Origins</b>: A complex type that contains information about origins for this distribution. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultRootObject</i> The object that you want CloudFront to request from your origin (for example, <code>index.html</code>) when a viewer requests the root URL for your distribution (<code>http://www.example.com</code>) instead of an object in your distribution (<code>http://www.example.com/product-description.html</code>). Specifying a default root object avoids exposing the contents of your distribution. Specify only the object name, for example, <code>index.html</code>. Don't add a <code>/</code> before the object name. If you don't want to specify a default root object when you create a distribution, include an empty <code>DefaultRootObject</code> element. To delete the default root object from an existing distribution, update the distribution configuration and include an empty <code>DefaultRootObject</code> element. To replace the default root object, update the distribution configuration and specify the new object. For more information about the default root object, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/DefaultRootObject.html">Creating a Default Root Object</a> in the <i>Amazon CloudFront Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Aliases</i> A complex type that contains information about CNAMEs (alternate domain names), if any, for this distribution.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Aliases</b>: A complex type that contains information about CNAMEs (alternate domain names), if any, for this distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique value (for example, a date-time stamp) that ensures that the request can't be replayed. If the value of <code>CallerReference</code> is new (regardless of the content of the <code>DistributionConfig</code> object), CloudFront creates a new distribution. If <code>CallerReference</code> is a value that you already sent in a previous request to create a distribution, CloudFront returns a <code>DistributionAlreadyExists</code> error.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> A complex type that contains the CNAME aliases, if any, that you want to associate with this distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of CNAME aliases, if any, that you want to associate with this distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> A complex type that contains origins or origin groups for this distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of origins or origin groups for this distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> The items (origin groups) in a distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of origin groups.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetOriginId</i> The value of <code>ID</code> for the origin that you want CloudFront to route requests to when a request matches the path pattern either for a cache behavior or for the default cache behavior in your distribution.<br/>A complex type that specifies how CloudFront handles query strings and cookies.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ForwardedValues</b>: A complex type that specifies how CloudFront handles query strings and cookies.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> Optional: A complex type that contains cache behaviors for this distribution. If <code>Quantity</code> is <code>0</code>, you can omit <code>Items</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of cache behaviors for this distribution. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> A complex type that contains a <code>CustomErrorResponse</code> element for each HTTP status code for which you want to specify a custom error page and/or a caching duration. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of HTTP status codes for which you want to specify a custom error page and/or a caching duration. If <code>Quantity</code> is <code>0</code>, you can omit <code>Items</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> An optional string that you want CloudFront to prefix to the access log <code>filenames</code> for this distribution, for example, <code>myprefix/</code>. If you want to enable logging, but you don't want to specify a prefix, you still must include an empty <code>Prefix</code> element in the <code>Logging</code> element.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> The Amazon S3 bucket to store the access logs in, for example, <code>myawslogbucket.s3.amazonaws.com</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeCookies</i> Specifies whether you want CloudFront to include cookies in access logs, specify <code>true</code> for <code>IncludeCookies</code>. If you choose to include cookies in logs, CloudFront logs all cookies regardless of how you configure the cache behaviors for this distribution. If you don't want to include cookies when you create a distribution or if you want to disable include cookies for an existing distribution, specify <code>false</code> for <code>IncludeCookies</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether you want CloudFront to save access logs to an Amazon S3 bucket. If you don't want to enable logging when you create a distribution or if you want to disable logging for an existing distribution, specify <code>false</code> for <code>Enabled</code>, and specify empty <code>Bucket</code> and <code>Prefix</code> elements. If you specify <code>false</code> for <code>Enabled</code> but you specify values for <code>Bucket</code>, <code>prefix</code>, and <code>IncludeCookies</code>, the values are automatically deleted.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateSource</i> This field is no longer used. Use one of the following fields instead: <ul> <li>  <a href="https://docs.aws.amazon.com/cloudfront/latest/APIReference/API_ViewerCertificate.html#cloudfront-Type-ViewerCertificate-ACMCertificateArn">ACMCertificateArn</a>  </li> <li>  <a href="https://docs.aws.amazon.com/cloudfront/latest/APIReference/API_ViewerCertificate.html#cloudfront-Type-ViewerCertificate-IAMCertificateId">IAMCertificateId</a>  </li> <li>  <a href="https://docs.aws.amazon.com/cloudfront/latest/APIReference/API_ViewerCertificate.html#cloudfront-Type-ViewerCertificate-CloudFrontDefaultCertificate">CloudFrontDefaultCertificate</a>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Certificate</i> This field is no longer used. Use one of the following fields instead: <ul> <li>  <a href="https://docs.aws.amazon.com/cloudfront/latest/APIReference/API_ViewerCertificate.html#cloudfront-Type-ViewerCertificate-ACMCertificateArn">ACMCertificateArn</a>  </li> <li>  <a href="https://docs.aws.amazon.com/cloudfront/latest/APIReference/API_ViewerCertificate.html#cloudfront-Type-ViewerCertificate-IAMCertificateId">IAMCertificateId</a>  </li> <li>  <a href="https://docs.aws.amazon.com/cloudfront/latest/APIReference/API_ViewerCertificate.html#cloudfront-Type-ViewerCertificate-CloudFrontDefaultCertificate">CloudFrontDefaultCertificate</a>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinimumProtocolVersion</i> Specify the security policy that you want CloudFront to use for HTTPS connections. A security policy determines two settings: <ul> <li> The minimum SSL/TLS protocol that CloudFront uses to communicate with viewers </li> <li> The cipher that CloudFront uses to encrypt the content that it returns to viewers </li> </ul> <note> On the CloudFront console, this setting is called <b>Security policy</b>. </note> We recommend that you specify <code>TLSv1.1_2016</code> unless your users are using browsers or devices that do not support TLSv1.1 or later. When both of the following are true, you must specify <code>TLSv1</code> or later for the security policy:  <ul> <li> You're using a custom certificate: you specified a value for <code>ACMCertificateArn</code> or for <code>IAMCertificateId</code>  </li> <li> You're using SNI: you specified <code>sni-only</code> for <code>SSLSupportMethod</code>  </li> </ul> If you specify <code>true</code> for <code>CloudFrontDefaultCertificate</code>, CloudFront automatically sets the security policy to <code>TLSv1</code> regardless of the value that you specify for <code>MinimumProtocolVersion</code>. For information about the relationship between the security policy that you choose and the protocols and ciphers that CloudFront uses to communicate with viewers, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/secure-connections-supported-viewer-protocols-ciphers.html#secure-connections-supported-ciphers"> Supported SSL/TLS Protocols and Ciphers for Communication Between Viewers and CloudFront</a> in the <i>Amazon CloudFront Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SSLSupportMethod</i> If you specify a value for <a href="https://docs.aws.amazon.com/cloudfront/latest/APIReference/API_ViewerCertificate.html#cloudfront-Type-ViewerCertificate-ACMCertificateArn">ACMCertificateArn</a> or for <a href="https://docs.aws.amazon.com/cloudfront/latest/APIReference/API_ViewerCertificate.html#cloudfront-Type-ViewerCertificate-IAMCertificateId">IAMCertificateId</a>, you must also specify how you want CloudFront to serve HTTPS requests: using a method that works for browsers and clients released after 2010 or one that works for all clients. <ul> <li>  <code>sni-only</code>: CloudFront can respond to HTTPS requests from viewers that support Server Name Indication (SNI). All modern browsers support SNI, but there are a few that don't. For a current list of the browsers that support SNI, see the <a href="http://en.wikipedia.org/wiki/Server_Name_Indication">Wikipedia entry Server Name Indication</a>. To learn about options to explore if you have users with browsers that don't include SNI support, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/cnames-https-dedicated-ip-or-sni.html">Choosing How CloudFront Serves HTTPS Requests</a> in the <i>Amazon CloudFront Developer Guide</i>. </li> <li>  <code>vip</code>: CloudFront uses dedicated IP addresses for your content and can respond to HTTPS requests from any viewer. However, there are additional monthly charges. For details, including specific pricing information, see <a href="http://aws.amazon.com/cloudfront/custom-ssl-domains/">Custom SSL options for Amazon CloudFront</a> on the AWS marketing site. </li> </ul> Don't specify a value for <code>SSLSupportMethod</code> if you specified <code>&lt;CloudFrontDefaultCertificate&gt;true&lt;CloudFrontDefaultCertificate&gt;</code>. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/cnames-https-dedicated-ip-or-sni.html">Choosing How CloudFront Serves HTTPS Requests</a> in the <i>Amazon CloudFront Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ACMCertificateArn</i> If you want viewers to use HTTPS to request your objects and you're using an alternate domain name, you must choose the type of certificate that you want to use. Specify the following value if ACM provided your certificate: <ul> <li>  <code>&lt;ACMCertificateArn&gt;<i>ARN for ACM SSL/TLS certificate</i>&lt;ACMCertificateArn&gt;</code> where <code> <i>ARN for ACM SSL/TLS certificate</i> </code> is the ARN for the ACM SSL/TLS certificate that you want to use for this distribution. </li> </ul> If you specify <code>ACMCertificateArn</code>, you must also specify a value for <code>SSLSupportMethod</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMCertificateId</i> If you want viewers to use HTTPS to request your objects and you're using an alternate domain name, you must choose the type of certificate that you want to use. Specify the following value if you purchased your certificate from a third-party certificate authority: <ul> <li>  <code>&lt;IAMCertificateId&gt;<i>IAM certificate ID</i>&lt;IAMCertificateId&gt;</code> where <code> <i>IAM certificate ID</i> </code> is the ID that IAM returned when you added the certificate to the IAM certificate store. </li> </ul> If you specify <code>IAMCertificateId</code>, you must also specify a value for <code>SSLSupportMethod</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudFrontDefaultCertificate</i> If you're using the CloudFront domain name for your distribution, such as <code>d111111abcdef8.cloudfront.net</code>, specify the following value: <ul> <li>  <code>&lt;CloudFrontDefaultCertificate&gt;true&lt;CloudFrontDefaultCertificate&gt; </code>  </li> </ul><br/>A complex type that controls the countries in which your content is distributed. CloudFront determines the location of your users using <code>MaxMind</code> GeoIP databases.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>GeoRestriction</b>: A complex type that controls the countries in which your content is distributed. CloudFront determines the location of your users using <code>MaxMind</code> GeoIP databases.        """
        pass

    def get_field_level_encryption(self, Id: str) -> GetFieldLevelEncryptionResult:
        """Get the field-level encryption configuration information.

        :param self:
        :param Id:  Request the ID for the field-level encryption configuration information.<br/>
        :return: ETag:  The current version of the field level encryption configuration. For example: <code>E2QWRUHAPOMQZL</code>.<br/>FieldLevelEncryption:  Return the field-level encryption configuration information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FieldLevelEncryptionConfig</i> A complex data type that includes the profile configurations specified for field-level encryption. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>FieldLevelEncryptionConfig</b>: A complex data type that includes the profile configurations specified for field-level encryption. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedTime</i> The last time the field-level encryption configuration was changed. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The configuration ID for a field-level encryption configuration which includes a set of profiles that specify certain selected data fields to be encrypted by specific public keys.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> An optional comment about the configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique number that ensures the request can't be replayed.<br/>A complex data type that specifies when to forward content if a profile isn't found and the profile that can be provided as a query argument in a request.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>QueryArgProfileConfig</b>: A complex data type that specifies when to forward content if a profile isn't found and the profile that can be provided as a query argument in a request.        """
        pass

    def get_field_level_encryption_config(self, Id: str) -> GetFieldLevelEncryptionConfigResult:
        """Get the field-level encryption configuration information.

        :param self:
        :param Id:  Request the ID for the field-level encryption configuration information.<br/>
        :return: ETag:  The current version of the field level encryption configuration. For example: <code>E2QWRUHAPOMQZL</code>.<br/>FieldLevelEncryptionConfig:  Return the field-level encryption configuration information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContentTypeProfileConfig</i> A complex data type that specifies when to forward content if a content type isn't recognized and profiles to use as by default in a request if a query argument doesn't specify a profile to use.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ContentTypeProfileConfig</b>: A complex data type that specifies when to forward content if a content type isn't recognized and profiles to use as by default in a request if a query argument doesn't specify a profile to use.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QueryArgProfileConfig</i> A complex data type that specifies when to forward content if a profile isn't found and the profile that can be provided as a query argument in a request.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>QueryArgProfileConfig</b>: A complex data type that specifies when to forward content if a profile isn't found and the profile that can be provided as a query argument in a request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> An optional comment about the configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique number that ensures the request can't be replayed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ForwardWhenQueryArgProfileIsUnknown</i> Flag to set if you want a request to be forwarded to the origin even if the profile specified by the field-level encryption query argument, fle-profile, is unknown.<br/>Profiles specified for query argument-profile mapping for field-level encryption.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>QueryArgProfiles</b>: Profiles specified for query argument-profile mapping for field-level encryption.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ForwardWhenContentTypeIsUnknown</i> The setting in a field-level encryption content type-profile mapping that specifies what to do when an unknown content type is provided for the profile. If true, content is forwarded without being encrypted when the content type is unknown. If false (the default), an error is returned when the content type is unknown. <br/>The configuration for a field-level encryption content type-profile. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ContentTypeProfiles</b>: The configuration for a field-level encryption content type-profile.         """
        pass

    def get_field_level_encryption_profile(self, Id: str) -> GetFieldLevelEncryptionProfileResult:
        """Get the field-level encryption profile information.

        :param self:
        :param Id:  Get the ID for the field-level encryption profile information.<br/>
        :return: ETag:  The current version of the field level encryption profile. For example: <code>E2QWRUHAPOMQZL</code>.<br/>FieldLevelEncryptionProfile:  Return the field-level encryption profile information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FieldLevelEncryptionProfileConfig</i> A complex data type that includes the profile name and the encryption entities for the field-level encryption profile.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>FieldLevelEncryptionProfileConfig</b>: A complex data type that includes the profile name and the encryption entities for the field-level encryption profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedTime</i> The last time the field-level encryption profile was updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID for a field-level encryption profile configuration which includes a set of profiles that specify certain selected data fields to be encrypted by specific public keys.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> An optional comment for the field-level encryption profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique number that ensures that the request can't be replayed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Profile name for the field-level encryption profile.<br/>A complex data type of encryption entities for the field-level encryption profile that include the public key ID, provider, and field patterns for specifying which fields to encrypt with this key.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncryptionEntities</b>: A complex data type of encryption entities for the field-level encryption profile that include the public key ID, provider, and field patterns for specifying which fields to encrypt with this key.        """
        pass

    def get_field_level_encryption_profile_config(self, Id: str) -> GetFieldLevelEncryptionProfileConfigResult:
        """Get the field-level encryption profile configuration information.

        :param self:
        :param Id:  Get the ID for the field-level encryption profile configuration information.<br/>
        :return: ETag:  The current version of the field-level encryption profile configuration result. For example: <code>E2QWRUHAPOMQZL</code>.<br/>FieldLevelEncryptionProfileConfig:  Return the field-level encryption profile configuration information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionEntities</i> A complex data type of encryption entities for the field-level encryption profile that include the public key ID, provider, and field patterns for specifying which fields to encrypt with this key.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncryptionEntities</b>: A complex data type of encryption entities for the field-level encryption profile that include the public key ID, provider, and field patterns for specifying which fields to encrypt with this key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> An optional comment for the field-level encryption profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique number that ensures that the request can't be replayed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Profile name for the field-level encryption profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> An array of field patterns in a field-level encryption content type-profile mapping. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> Number of field pattern items in a field-level encryption content type-profile mapping. <br/>        """
        pass

    def get_invalidation(self, Id: str, DistributionId: str) -> GetInvalidationResult:
        """Get the information about an invalidation. 

        :param self:
        :param Id:  The identifier for the invalidation request, for example, <code>IDFDVBD632BHDS5</code>.<br/>DistributionId:  The distribution's ID.<br/>
        :return: Invalidation:  The invalidation's information. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/InvalidationDatatype.html">Invalidation Complex Type</a>. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InvalidationBatch</i> The current invalidation information for the batch request. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InvalidationBatch</b>: The current invalidation information for the batch request. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateTime</i> The date and time the invalidation request was first made. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the invalidation request. When the invalidation batch is finished, the status is <code>Completed</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier for the invalidation request. For example: <code>IDFDVBD632BHDS5</code>.<br/>A complex type that contains information about the objects that you want to invalidate. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/Invalidation.html#invalidation-specifying-objects">Specifying the Objects to Invalidate</a> in the <i>Amazon CloudFront Developer Guide</i>. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Paths</b>: A complex type that contains information about the objects that you want to invalidate. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/Invalidation.html#invalidation-specifying-objects">Specifying the Objects to Invalidate</a> in the <i>Amazon CloudFront Developer Guide</i>.         """
        pass

    def get_public_key(self, Id: str) -> GetPublicKeyResult:
        """Get the public key information.

        :param self:
        :param Id:  Request the ID for the public key.<br/>
        :return: ETag:  The current version of the public key. For example: <code>E2QWRUHAPOMQZL</code>.<br/>PublicKey:  Return the public key.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PublicKeyConfig</i> A complex data type for a public key you add to CloudFront to use with features like field-level encryption.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PublicKeyConfig</b>: A complex data type for a public key you add to CloudFront to use with features like field-level encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> A time you added a public key to CloudFront.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> A unique ID assigned to a public key you've added to CloudFront.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> An optional comment about a public key.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncodedKey</i> The encoded public key that you want to add to CloudFront to use with features like field-level encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name for a public key you add to CloudFront to use with features like field-level encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique number that ensures that the request can't be replayed.<br/>        """
        pass

    def get_public_key_config(self, Id: str) -> GetPublicKeyConfigResult:
        """Return public key configuration informaation

        :param self:
        :param Id:  Request the ID for the public key configuration.<br/>
        :return: ETag:  The current version of the public key configuration. For example: <code>E2QWRUHAPOMQZL</code>.<br/>PublicKeyConfig:  Return the result for the public key configuration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> An optional comment about a public key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncodedKey</i> The encoded public key that you want to add to CloudFront to use with features like field-level encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name for a public key you add to CloudFront to use with features like field-level encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique number that ensures that the request can't be replayed.<br/>        """
        pass

    def get_streaming_distribution(self, Id: str) -> GetStreamingDistributionResult:
        """Gets information about a specified RTMP distribution, including the distribution configuration.

        :param self:
        :param Id:  The streaming distribution's ID.<br/>
        :return: ETag:  The current version of the streaming distribution's information. For example: <code>E2QWRUHAPOMQZL</code>.<br/>StreamingDistribution:  The streaming distribution's information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamingDistributionConfig</i> The current configuration information for the RTMP distribution.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamingDistributionConfig</b>: The current configuration information for the RTMP distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActiveTrustedSigners</i> A complex type that lists the AWS accounts, if any, that you included in the <code>TrustedSigners</code> complex type for this distribution. These are the accounts that you want to allow to create signed URLs for private content. The <code>Signer</code> complex type lists the AWS account number of the trusted signer or <code>self</code> if the signer is the AWS account that created the distribution. The <code>Signer</code> element also includes the IDs of any active CloudFront key pairs that are associated with the trusted signer's AWS account. If no <code>KeyPairId</code> element appears for a <code>Signer</code>, that signer can't create signed URLs. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/PrivateContent.html">Serving Private Content through CloudFront</a> in the <i>Amazon CloudFront Developer Guide</i>. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ActiveTrustedSigners</b>: A complex type that lists the AWS accounts, if any, that you included in the <code>TrustedSigners</code> complex type for this distribution. These are the accounts that you want to allow to create signed URLs for private content. The <code>Signer</code> complex type lists the AWS account number of the trusted signer or <code>self</code> if the signer is the AWS account that created the distribution. The <code>Signer</code> element also includes the IDs of any active CloudFront key pairs that are associated with the trusted signer's AWS account. If no <code>KeyPairId</code> element appears for a <code>Signer</code>, that signer can't create signed URLs. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/PrivateContent.html">Serving Private Content through CloudFront</a> in the <i>Amazon CloudFront Developer Guide</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainName</i> The domain name that corresponds to the streaming distribution, for example, <code>s5c39gqb8ow64r.cloudfront.net</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedTime</i> The date and time that the distribution was last modified. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current status of the RTMP distribution. When the status is <code>Deployed</code>, the distribution's information is propagated to all CloudFront edge locations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ARN</i> The ARN (Amazon Resource Name) for the distribution. For example: <code>arn:aws:cloudfront::123456789012:distribution/EDFDVBD632BHDS5</code>, where <code>123456789012</code> is your AWS account ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier for the RTMP distribution. For example: <code>EGTXBD79EXAMPLE</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> A complex type that contains one <code>Signer</code> complex type for each trusted signer that is specified in the <code>TrustedSigners</code> complex type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of trusted signers specified in the <code>TrustedSigners</code> complex type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Enabled is <code>true</code> if any of the AWS accounts listed in the <code>TrustedSigners</code> complex type for this distribution have active CloudFront key pairs. If not, <code>Enabled</code> is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique value (for example, a date-time stamp) that ensures that the request can't be replayed. If the value of <code>CallerReference</code> is new (regardless of the content of the <code>StreamingDistributionConfig</code> object), CloudFront creates a new distribution. If <code>CallerReference</code> is a value that you already sent in a previous request to create a distribution, CloudFront returns a <code>DistributionAlreadyExists</code> error.<br/>A complex type that contains information about the Amazon S3 bucket from which you want CloudFront to get your media files for distribution. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Origin</b>: A complex type that contains information about the Amazon S3 bucket from which you want CloudFront to get your media files for distribution.         """
        pass

    def get_streaming_distribution_config(self, Id: str) -> GetStreamingDistributionConfigResult:
        """Get the configuration information about a streaming distribution. 

        :param self:
        :param Id:  The streaming distribution's ID.<br/>
        :return: ETag:  The current version of the configuration. For example: <code>E2QWRUHAPOMQZL</code>. <br/>StreamingDistributionConfig:  The streaming distribution's configuration information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Whether the streaming distribution is enabled to accept user requests for content.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PriceClass</i> A complex type that contains information about price class for this streaming distribution. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrustedSigners</i> A complex type that specifies any AWS accounts that you want to permit to create signed URLs for private content. If you want the distribution to use signed URLs, include this element; if you want the distribution to use public URLs, remove this element. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/PrivateContent.html">Serving Private Content through CloudFront</a> in the <i>Amazon CloudFront Developer Guide</i>. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TrustedSigners</b>: A complex type that specifies any AWS accounts that you want to permit to create signed URLs for private content. If you want the distribution to use signed URLs, include this element; if you want the distribution to use public URLs, remove this element. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/PrivateContent.html">Serving Private Content through CloudFront</a> in the <i>Amazon CloudFront Developer Guide</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Logging</i> A complex type that controls whether access logs are written for the streaming distribution. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Logging</b>: A complex type that controls whether access logs are written for the streaming distribution. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> Any comments you want to include about the streaming distribution. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Aliases</i> A complex type that contains information about CNAMEs (alternate domain names), if any, for this streaming distribution. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Aliases</b>: A complex type that contains information about CNAMEs (alternate domain names), if any, for this streaming distribution. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Origin</i> A complex type that contains information about the Amazon S3 bucket from which you want CloudFront to get your media files for distribution. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Origin</b>: A complex type that contains information about the Amazon S3 bucket from which you want CloudFront to get your media files for distribution. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique value (for example, a date-time stamp) that ensures that the request can't be replayed. If the value of <code>CallerReference</code> is new (regardless of the content of the <code>StreamingDistributionConfig</code> object), CloudFront creates a new distribution. If <code>CallerReference</code> is a value that you already sent in a previous request to create a distribution, CloudFront returns a <code>DistributionAlreadyExists</code> error.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OriginAccessIdentity</i> The CloudFront origin access identity to associate with the distribution. Use an origin access identity to configure the distribution so that end users can only access objects in an Amazon S3 bucket through CloudFront. If you want end users to be able to access objects using either the CloudFront URL or the Amazon S3 URL, specify an empty <code>OriginAccessIdentity</code> element. To delete the origin access identity from an existing distribution, update the distribution configuration and include an empty <code>OriginAccessIdentity</code> element. To replace the origin access identity, update the distribution configuration and specify the new origin access identity. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/private-content-restricting-access-to-s3.html">Using an Origin Access Identity to Restrict Access to Your Amazon S3 Content</a> in the <i> Amazon CloudFront Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainName</i> The DNS name of the Amazon S3 origin. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> A complex type that contains the CNAME aliases, if any, that you want to associate with this distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of CNAME aliases, if any, that you want to associate with this distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> An optional string that you want CloudFront to prefix to the access log filenames for this streaming distribution, for example, <code>myprefix/</code>. If you want to enable logging, but you don't want to specify a prefix, you still must include an empty <code>Prefix</code> element in the <code>Logging</code> element.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> The Amazon S3 bucket to store the access logs in, for example, <code>myawslogbucket.s3.amazonaws.com</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether you want CloudFront to save access logs to an Amazon S3 bucket. If you don't want to enable logging when you create a streaming distribution or if you want to disable logging for an existing streaming distribution, specify <code>false</code> for <code>Enabled</code>, and specify <code>empty Bucket</code> and <code>Prefix</code> elements. If you specify <code>false</code> for <code>Enabled</code> but you specify values for <code>Bucket</code> and <code>Prefix</code>, the values are automatically deleted. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i>  <b>Optional</b>: A complex type that contains trusted signers for this cache behavior. If <code>Quantity</code> is <code>0</code>, you can omit <code>Items</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of trusted signers for this cache behavior.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether you want to require viewers to use signed URLs to access the files specified by <code>PathPattern</code> and <code>TargetOriginId</code>.<br/>        """
        pass

    def list_cloud_front_origin_access_identities(self, Marker: str=None, MaxItems: str=None) -> ListCloudFrontOriginAccessIdentitiesResult:
        """Lists origin access identities.

        :param self:
        :param MaxItems:  The maximum number of origin access identities you want in the response body. <br/>Marker:  Use this when paginating results to indicate where to begin in your list of origin access identities. The results include identities in the list that occur after the marker. To get the next page of results, set the <code>Marker</code> to the value of the <code>NextMarker</code> from the current page's response (which is also the ID of the last identity on that page).<br/>
        :return: CloudFrontOriginAccessIdentityList:  The <code>CloudFrontOriginAccessIdentityList</code> type. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> A complex type that contains one <code>CloudFrontOriginAccessIdentitySummary</code> element for each origin access identity that was created by the current AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of CloudFront origin access identities that were created by the current AWS account. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsTruncated</i> A flag that indicates whether more origin access identities remain to be listed. If your results were truncated, you can make a follow-up pagination request using the <code>Marker</code> request parameter to retrieve more items in the list.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxItems</i> The maximum number of origin access identities you want in the response body. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NextMarker</i> If <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value you can use for the <code>Marker</code> request parameter to continue listing your origin access identities where they left off. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Marker</i> Use this when paginating results to indicate where to begin in your list of origin access identities. The results include identities in the list that occur after the marker. To get the next page of results, set the <code>Marker</code> to the value of the <code>NextMarker</code> from the current page's response (which is also the ID of the last identity on that page). <br/>        """
        pass

    def list_distributions(self, Marker: str=None, MaxItems: str=None) -> ListDistributionsResult:
        """List CloudFront distributions.

        :param self:
        :param MaxItems:  The maximum number of distributions you want in the response body.<br/>Marker:  Use this when paginating results to indicate where to begin in your list of distributions. The results include distributions in the list that occur after the marker. To get the next page of results, set the <code>Marker</code> to the value of the <code>NextMarker</code> from the current page's response (which is also the ID of the last distribution on that page).<br/>
        :return: DistributionList:  The <code>DistributionList</code> type. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> A complex type that contains one <code>DistributionSummary</code> element for each distribution that was created by the current AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of distributions that were created by the current AWS account. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsTruncated</i> A flag that indicates whether more distributions remain to be listed. If your results were truncated, you can make a follow-up pagination request using the <code>Marker</code> request parameter to retrieve more distributions in the list.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxItems</i> The value you provided for the <code>MaxItems</code> request parameter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NextMarker</i> If <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value you can use for the <code>Marker</code> request parameter to continue listing your distributions where they left off. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Marker</i> The value you provided for the <code>Marker</code> request parameter.<br/>        """
        pass

    def list_distributions_by_web_acl_id(self, WebACLId: str, Marker: str=None, MaxItems: str=None) -> ListDistributionsByWebACLIdResult:
        """List the distributions that are associated with a specified AWS WAF web ACL. 

        :param self:
        :param WebACLId:  The ID of the AWS WAF web ACL that you want to list the associated distributions. If you specify "null" for the ID, the request returns a list of the distributions that aren't associated with a web ACL. <br/>MaxItems:  The maximum number of distributions that you want CloudFront to return in the response body. The maximum and default values are both 100.<br/>Marker:  Use <code>Marker</code> and <code>MaxItems</code> to control pagination of results. If you have more than <code>MaxItems</code> distributions that satisfy the request, the response includes a <code>NextMarker</code> element. To get the next page of results, submit another request. For the value of <code>Marker</code>, specify the value of <code>NextMarker</code> from the last response. (For the first request, omit <code>Marker</code>.) <br/>
        :return: DistributionList:  The <code>DistributionList</code> type. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> A complex type that contains one <code>DistributionSummary</code> element for each distribution that was created by the current AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of distributions that were created by the current AWS account. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsTruncated</i> A flag that indicates whether more distributions remain to be listed. If your results were truncated, you can make a follow-up pagination request using the <code>Marker</code> request parameter to retrieve more distributions in the list.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxItems</i> The value you provided for the <code>MaxItems</code> request parameter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NextMarker</i> If <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value you can use for the <code>Marker</code> request parameter to continue listing your distributions where they left off. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Marker</i> The value you provided for the <code>Marker</code> request parameter.<br/>        """
        pass

    def list_field_level_encryption_configs(self, Marker: str=None, MaxItems: str=None) -> ListFieldLevelEncryptionConfigsResult:
        """List all field-level encryption configurations that have been created in CloudFront for this account.

        :param self:
        :param MaxItems:  The maximum number of field-level encryption configurations you want in the response body. <br/>Marker:  Use this when paginating results to indicate where to begin in your list of configurations. The results include configurations in the list that occur after the marker. To get the next page of results, set the <code>Marker</code> to the value of the <code>NextMarker</code> from the current page's response (which is also the ID of the last configuration on that page). <br/>
        :return: FieldLevelEncryptionList:  Returns a list of all field-level encryption configurations that have been created in CloudFront for this account.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> An array of field-level encryption items.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of field-level encryption items.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxItems</i> The maximum number of elements you want in the response body. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NextMarker</i> If there are more elements to be listed, this element is present and contains the value that you can use for the <code>Marker</code> request parameter to continue listing your configurations where you left off.<br/>        """
        pass

    def list_field_level_encryption_profiles(self, Marker: str=None, MaxItems: str=None) -> ListFieldLevelEncryptionProfilesResult:
        """Request a list of field-level encryption profiles that have been created in CloudFront for this account.

        :param self:
        :param MaxItems:  The maximum number of field-level encryption profiles you want in the response body. <br/>Marker:  Use this when paginating results to indicate where to begin in your list of profiles. The results include profiles in the list that occur after the marker. To get the next page of results, set the <code>Marker</code> to the value of the <code>NextMarker</code> from the current page's response (which is also the ID of the last profile on that page). <br/>
        :return: FieldLevelEncryptionProfileList:  Returns a list of the field-level encryption profiles that have been created in CloudFront for this account.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> The field-level encryption profile items.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of field-level encryption profiles.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxItems</i> The maximum number of field-level encryption profiles you want in the response body. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NextMarker</i> If there are more elements to be listed, this element is present and contains the value that you can use for the <code>Marker</code> request parameter to continue listing your profiles where you left off.<br/>        """
        pass

    def list_invalidations(self, DistributionId: str, Marker: str=None, MaxItems: str=None) -> ListInvalidationsResult:
        """Lists invalidation batches. 

        :param self:
        :param MaxItems:  The maximum number of invalidation batches that you want in the response body.<br/>Marker:  Use this parameter when paginating results to indicate where to begin in your list of invalidation batches. Because the results are returned in decreasing order from most recent to oldest, the most recent results are on the first page, the second page will contain earlier results, and so on. To get the next page of results, set <code>Marker</code> to the value of the <code>NextMarker</code> from the current page's response. This value is the same as the ID of the last invalidation batch on that page. <br/>DistributionId:  The distribution's ID.<br/>
        :return: InvalidationList:  Information about invalidation batches. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> A complex type that contains one <code>InvalidationSummary</code> element for each invalidation batch created by the current AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of invalidation batches that were created by the current AWS account. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsTruncated</i> A flag that indicates whether more invalidation batch requests remain to be listed. If your results were truncated, you can make a follow-up pagination request using the <code>Marker</code> request parameter to retrieve more invalidation batches in the list.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxItems</i> The value that you provided for the <code>MaxItems</code> request parameter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NextMarker</i> If <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value that you can use for the <code>Marker</code> request parameter to continue listing your invalidation batches where they left off.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Marker</i> The value that you provided for the <code>Marker</code> request parameter.<br/>        """
        pass

    def list_public_keys(self, Marker: str=None, MaxItems: str=None) -> ListPublicKeysResult:
        """List all public keys that have been added to CloudFront for this account.

        :param self:
        :param MaxItems:  The maximum number of public keys you want in the response body. <br/>Marker:  Use this when paginating results to indicate where to begin in your list of public keys. The results include public keys in the list that occur after the marker. To get the next page of results, set the <code>Marker</code> to the value of the <code>NextMarker</code> from the current page's response (which is also the ID of the last public key on that page). <br/>
        :return: PublicKeyList:  Returns a list of all public keys that have been added to CloudFront for this account.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> An array of information about a public key you add to CloudFront to use with features like field-level encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of public keys you added to CloudFront to use with features like field-level encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxItems</i> The maximum number of public keys you want in the response body. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NextMarker</i> If there are more elements to be listed, this element is present and contains the value that you can use for the <code>Marker</code> request parameter to continue listing your public keys where you left off.<br/>        """
        pass

    def list_streaming_distributions(self, Marker: str=None, MaxItems: str=None) -> ListStreamingDistributionsResult:
        """List streaming distributions. 

        :param self:
        :param MaxItems:  The value that you provided for the <code>MaxItems</code> request parameter.<br/>Marker:  The value that you provided for the <code>Marker</code> request parameter.<br/>
        :return: StreamingDistributionList:  The <code>StreamingDistributionList</code> type. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> A complex type that contains one <code>StreamingDistributionSummary</code> element for each distribution that was created by the current AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of streaming distributions that were created by the current AWS account. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsTruncated</i> A flag that indicates whether more streaming distributions remain to be listed. If your results were truncated, you can make a follow-up pagination request using the <code>Marker</code> request parameter to retrieve more distributions in the list. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxItems</i> The value you provided for the <code>MaxItems</code> request parameter. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NextMarker</i> If <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value you can use for the <code>Marker</code> request parameter to continue listing your RTMP distributions where they left off. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Marker</i> The value you provided for the <code>Marker</code> request parameter. <br/>        """
        pass

    def list_tags_for_resource(self, Resource: str) -> ListTagsForResourceResult:
        """List tags for a CloudFront resource.

        :param self:
        :param Resource:   An ARN of a CloudFront resource.<br/>
        :return: Tags:   A complex type that contains zero or more <code>Tag</code> elements.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i>  A complex type that contains <code>Tag</code> elements.<br/>        """
        pass

    def tag_resource(self, Tags: Tags, Resource: str):
        """Add tags to a CloudFront resource.

        :param self:
        :param Tags:   A complex type that contains zero or more <code>Tag</code> elements.:<br/>Resource:   An ARN of a CloudFront resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i>  A complex type that contains <code>Tag</code> elements.<br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: TagKeys, Resource: str):
        """Remove tags from a CloudFront resource.

        :param self:
        :param TagKeys:   A complex type that contains zero or more <code>Tag</code> key elements.:<br/>Resource:   An ARN of a CloudFront resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i>  A complex type that contains <code>Tag</code> key elements.<br/>
        :return:         """
        pass

    def update_cloud_front_origin_access_identity(self, Id: str, CloudFrontOriginAccessIdentityConfig: CloudFrontOriginAccessIdentityConfig, IfMatch: str=None) -> UpdateCloudFrontOriginAccessIdentityResult:
        """Update an origin access identity. 

        :param self:
        :param IfMatch:  The value of the <code>ETag</code> header that you received when retrieving the identity's configuration. For example: <code>E2QWRUHAPOMQZL</code>.<br/>Id:  The identity's id.<br/>CloudFrontOriginAccessIdentityConfig:  The identity's configuration information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> Any comments you want to include about the origin access identity. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique value (for example, a date-time stamp) that ensures that the request can't be replayed. If the value of <code>CallerReference</code> is new (regardless of the content of the <code>CloudFrontOriginAccessIdentityConfig</code> object), a new origin access identity is created. If the <code>CallerReference</code> is a value already sent in a previous identity request, and the content of the <code>CloudFrontOriginAccessIdentityConfig</code> is identical to the original request (ignoring white space), the response includes the same information returned to the original request.  If the <code>CallerReference</code> is a value you already sent in a previous request to create an identity, but the content of the <code>CloudFrontOriginAccessIdentityConfig</code> is different from the original request, CloudFront returns a <code>CloudFrontOriginAccessIdentityAlreadyExists</code> error. <br/>
        :return: ETag:  The current version of the configuration. For example: <code>E2QWRUHAPOMQZL</code>.<br/>CloudFrontOriginAccessIdentity:  The origin access identity's information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudFrontOriginAccessIdentityConfig</i> The current configuration information for the identity. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CloudFrontOriginAccessIdentityConfig</b>: The current configuration information for the identity. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3CanonicalUserId</i> The Amazon S3 canonical user ID for the origin access identity, used when giving the origin access identity read permission to an object in Amazon S3. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID for the origin access identity, for example, <code>E74FTE3AJFJ256A</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> Any comments you want to include about the origin access identity. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique value (for example, a date-time stamp) that ensures that the request can't be replayed. If the value of <code>CallerReference</code> is new (regardless of the content of the <code>CloudFrontOriginAccessIdentityConfig</code> object), a new origin access identity is created. If the <code>CallerReference</code> is a value already sent in a previous identity request, and the content of the <code>CloudFrontOriginAccessIdentityConfig</code> is identical to the original request (ignoring white space), the response includes the same information returned to the original request.  If the <code>CallerReference</code> is a value you already sent in a previous request to create an identity, but the content of the <code>CloudFrontOriginAccessIdentityConfig</code> is different from the original request, CloudFront returns a <code>CloudFrontOriginAccessIdentityAlreadyExists</code> error. <br/>        """
        pass

    def update_distribution(self, Id: str, DistributionConfig: DistributionConfig, IfMatch: str=None) -> UpdateDistributionResult:
        """Updates the configuration for a web distribution.  <important> When you update a distribution, there are more required fields than when you create a distribution. When you update your distribution by using this API action, follow the steps here to get the current configuration and then make your updates, to make sure that you include all of the required fields. To view a summary, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/distribution-overview-required-fields.html">Required Fields for Create Distribution and Update Distribution</a> in the <i>Amazon CloudFront Developer Guide</i>. </important> The update process includes getting the current distribution configuration, updating the XML document that is returned to make your changes, and then submitting an <code>UpdateDistribution</code> request to make the updates. For information about updating a distribution using the CloudFront console instead, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/distribution-web-creating-console.html">Creating a Distribution</a> in the <i>Amazon CloudFront Developer Guide</i>.  <b>To update a web distribution using the CloudFront API</b>  <ol> <li> Submit a <a href="https://docs.aws.amazon.com/cloudfront/latest/APIReference/API_GetDistributionConfig.html">GetDistributionConfig</a> request to get the current configuration and an <code>Etag</code> header for the distribution. <note> If you update the distribution again, you must get a new <code>Etag</code> header. </note> </li> <li> Update the XML document that was returned in the response to your <code>GetDistributionConfig</code> request to include your changes.  <important> When you edit the XML file, be aware of the following: <ul> <li> You must strip out the ETag parameter that is returned. </li> <li> Additional fields are required when you update a distribution. There may be fields included in the XML file for features that you haven't configured for your distribution. This is expected and required to successfully update the distribution. </li> <li> You can't change the value of <code>CallerReference</code>. If you try to change this value, CloudFront returns an <code>IllegalUpdate</code> error.  </li> <li> The new configuration replaces the existing configuration; the values that you specify in an <code>UpdateDistribution</code> request are not merged into your existing configuration. When you add, delete, or replace values in an element that allows multiple values (for example, <code>CNAME</code>), you must specify all of the values that you want to appear in the updated distribution. In addition, you must update the corresponding <code>Quantity</code> element. </li> </ul> </important> </li> <li> Submit an <code>UpdateDistribution</code> request to update the configuration for your distribution: <ul> <li> In the request body, include the XML document that you updated in Step 2. The request body must include an XML document with a <code>DistributionConfig</code> element. </li> <li> Set the value of the HTTP <code>If-Match</code> header to the value of the <code>ETag</code> header that CloudFront returned when you submitted the <code>GetDistributionConfig</code> request in Step 1. </li> </ul> </li> <li> Review the response to the <code>UpdateDistribution</code> request to confirm that the configuration was successfully updated. </li> <li> Optional: Submit a <a href="https://docs.aws.amazon.com/cloudfront/latest/APIReference/API_GetDistribution.html">GetDistribution</a> request to confirm that your changes have propagated. When propagation is complete, the value of <code>Status</code> is <code>Deployed</code>. </li> </ol>

        :param self:
        :param IfMatch:  The value of the <code>ETag</code> header that you received when retrieving the distribution's configuration. For example: <code>E2QWRUHAPOMQZL</code>.<br/>Id:  The distribution's id.<br/>DistributionConfig:  The distribution's configuration information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsIPV6Enabled</i> If you want CloudFront to respond to IPv6 DNS requests with an IPv6 address for your distribution, specify <code>true</code>. If you specify <code>false</code>, CloudFront responds to IPv6 DNS requests with the DNS response code <code>NOERROR</code> and with no IP addresses. This allows viewers to submit a second request, for an IPv4 address for your distribution.  In general, you should enable IPv6 if you have users on IPv6 networks who want to access your content. However, if you're using signed URLs or signed cookies to restrict access to your content, and if you're using a custom policy that includes the <code>IpAddress</code> parameter to restrict the IP addresses that can access your content, don't enable IPv6. If you want to restrict access to some content by IP address and not restrict access to other content (or restrict access but not by IP address), you can create two distributions. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/private-content-creating-signed-url-custom-policy.html">Creating a Signed URL Using a Custom Policy</a> in the <i>Amazon CloudFront Developer Guide</i>. If you're using an Amazon Route 53 alias resource record set to route traffic to your CloudFront distribution, you need to create a second alias resource record set when both of the following are true: <ul> <li> You enable IPv6 for the distribution </li> <li> You're using alternate domain names in the URLs for your objects </li> </ul> For more information, see <a href="https://docs.aws.amazon.com/Route53/latest/DeveloperGuide/routing-to-cloudfront-distribution.html">Routing Traffic to an Amazon CloudFront Web Distribution by Using Your Domain Name</a> in the <i>Amazon Route 53 Developer Guide</i>. If you created a CNAME resource record set, either with Amazon Route 53 or with another DNS service, you don't need to make any changes. A CNAME record will route traffic to your distribution regardless of the IP address format of the viewer request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HttpVersion</i> (Optional) Specify the maximum HTTP version that you want viewers to use to communicate with CloudFront. The default value for new web distributions is http2. Viewers that don't support HTTP/2 automatically use an earlier HTTP version. For viewers and CloudFront to use HTTP/2, viewers must support TLS 1.2 or later, and must support Server Name Identification (SNI). In general, configuring CloudFront to communicate with viewers using HTTP/2 reduces latency. You can improve performance by optimizing for HTTP/2. For more information, do an Internet search for "http/2 optimization." <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WebACLId</i> A unique identifier that specifies the AWS WAF web ACL, if any, to associate with this distribution. AWS WAF is a web application firewall that lets you monitor the HTTP and HTTPS requests that are forwarded to CloudFront, and lets you control access to your content. Based on conditions that you specify, such as the IP addresses that requests originate from or the values of query strings, CloudFront responds to requests either with the requested content or with an HTTP 403 status code (Forbidden). You can also configure CloudFront to return a custom error page when a request is blocked. For more information about AWS WAF, see the <a href="http://docs.aws.amazon.com/waf/latest/developerguide/what-is-aws-waf.html">AWS WAF Developer Guide</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Restrictions</i> A complex type that identifies ways in which you want to restrict distribution of your content.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Restrictions</b>: A complex type that identifies ways in which you want to restrict distribution of your content.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ViewerCertificate</i> A complex type that specifies whether you want viewers to use HTTP or HTTPS to request your objects, whether you're using an alternate domain name with HTTPS, and if so, if you're using AWS Certificate Manager (ACM) or a third-party certificate authority.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ViewerCertificate</b>: A complex type that specifies whether you want viewers to use HTTP or HTTPS to request your objects, whether you're using an alternate domain name with HTTPS, and if so, if you're using AWS Certificate Manager (ACM) or a third-party certificate authority.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> From this field, you can enable or disable the selected distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PriceClass</i> The price class that corresponds with the maximum price that you want to pay for CloudFront service. If you specify <code>PriceClass_All</code>, CloudFront responds to requests for your objects from all CloudFront edge locations. If you specify a price class other than <code>PriceClass_All</code>, CloudFront serves your objects from the CloudFront edge location that has the lowest latency among the edge locations in your price class. Viewers who are in or near regions that are excluded from your specified price class may encounter slower performance. For more information about price classes, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/PriceClass.html">Choosing the Price Class for a CloudFront Distribution</a> in the <i>Amazon CloudFront Developer Guide</i>. For information about CloudFront pricing, including how price classes (such as Price Class 100) map to CloudFront regions, see <a href="http://aws.amazon.com/cloudfront/pricing/">Amazon CloudFront Pricing</a>. For price class information, scroll down to see the table at the bottom of the page.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Logging</i> A complex type that controls whether access logs are written for the distribution. For more information about logging, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/AccessLogs.html">Access Logs</a> in the <i>Amazon CloudFront Developer Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Logging</b>: A complex type that controls whether access logs are written for the distribution. For more information about logging, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/AccessLogs.html">Access Logs</a> in the <i>Amazon CloudFront Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> Any comments you want to include about the distribution. If you don't want to specify a comment, include an empty <code>Comment</code> element. To delete an existing comment, update the distribution configuration and include an empty <code>Comment</code> element. To add or change a comment, update the distribution configuration and specify the new comment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomErrorResponses</i> A complex type that controls the following: <ul> <li> Whether CloudFront replaces HTTP status codes in the 4xx and 5xx range with custom error messages before returning the response to the viewer. </li> <li> How long CloudFront caches HTTP status codes in the 4xx and 5xx range. </li> </ul> For more information about custom error pages, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/custom-error-pages.html">Customizing Error Responses</a> in the <i>Amazon CloudFront Developer Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CustomErrorResponses</b>: A complex type that controls the following: <ul> <li> Whether CloudFront replaces HTTP status codes in the 4xx and 5xx range with custom error messages before returning the response to the viewer. </li> <li> How long CloudFront caches HTTP status codes in the 4xx and 5xx range. </li> </ul> For more information about custom error pages, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/custom-error-pages.html">Customizing Error Responses</a> in the <i>Amazon CloudFront Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheBehaviors</i> A complex type that contains zero or more <code>CacheBehavior</code> elements. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CacheBehaviors</b>: A complex type that contains zero or more <code>CacheBehavior</code> elements. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultCacheBehavior</i> A complex type that describes the default cache behavior if you don't specify a <code>CacheBehavior</code> element or if files don't match any of the values of <code>PathPattern</code> in <code>CacheBehavior</code> elements. You must create exactly one default cache behavior.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DefaultCacheBehavior</b>: A complex type that describes the default cache behavior if you don't specify a <code>CacheBehavior</code> element or if files don't match any of the values of <code>PathPattern</code> in <code>CacheBehavior</code> elements. You must create exactly one default cache behavior.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OriginGroups</i>  A complex type that contains information about origin groups for this distribution.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>OriginGroups</b>:  A complex type that contains information about origin groups for this distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Origins</i> A complex type that contains information about origins for this distribution. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Origins</b>: A complex type that contains information about origins for this distribution. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultRootObject</i> The object that you want CloudFront to request from your origin (for example, <code>index.html</code>) when a viewer requests the root URL for your distribution (<code>http://www.example.com</code>) instead of an object in your distribution (<code>http://www.example.com/product-description.html</code>). Specifying a default root object avoids exposing the contents of your distribution. Specify only the object name, for example, <code>index.html</code>. Don't add a <code>/</code> before the object name. If you don't want to specify a default root object when you create a distribution, include an empty <code>DefaultRootObject</code> element. To delete the default root object from an existing distribution, update the distribution configuration and include an empty <code>DefaultRootObject</code> element. To replace the default root object, update the distribution configuration and specify the new object. For more information about the default root object, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/DefaultRootObject.html">Creating a Default Root Object</a> in the <i>Amazon CloudFront Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Aliases</i> A complex type that contains information about CNAMEs (alternate domain names), if any, for this distribution.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Aliases</b>: A complex type that contains information about CNAMEs (alternate domain names), if any, for this distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique value (for example, a date-time stamp) that ensures that the request can't be replayed. If the value of <code>CallerReference</code> is new (regardless of the content of the <code>DistributionConfig</code> object), CloudFront creates a new distribution. If <code>CallerReference</code> is a value that you already sent in a previous request to create a distribution, CloudFront returns a <code>DistributionAlreadyExists</code> error.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> A complex type that contains the CNAME aliases, if any, that you want to associate with this distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of CNAME aliases, if any, that you want to associate with this distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> A complex type that contains origins or origin groups for this distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of origins or origin groups for this distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> The items (origin groups) in a distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of origin groups.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetOriginId</i> The value of <code>ID</code> for the origin that you want CloudFront to route requests to when a request matches the path pattern either for a cache behavior or for the default cache behavior in your distribution.<br/>A complex type that specifies how CloudFront handles query strings and cookies.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ForwardedValues</b>: A complex type that specifies how CloudFront handles query strings and cookies.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> Optional: A complex type that contains cache behaviors for this distribution. If <code>Quantity</code> is <code>0</code>, you can omit <code>Items</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of cache behaviors for this distribution. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> A complex type that contains a <code>CustomErrorResponse</code> element for each HTTP status code for which you want to specify a custom error page and/or a caching duration. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of HTTP status codes for which you want to specify a custom error page and/or a caching duration. If <code>Quantity</code> is <code>0</code>, you can omit <code>Items</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> An optional string that you want CloudFront to prefix to the access log <code>filenames</code> for this distribution, for example, <code>myprefix/</code>. If you want to enable logging, but you don't want to specify a prefix, you still must include an empty <code>Prefix</code> element in the <code>Logging</code> element.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> The Amazon S3 bucket to store the access logs in, for example, <code>myawslogbucket.s3.amazonaws.com</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeCookies</i> Specifies whether you want CloudFront to include cookies in access logs, specify <code>true</code> for <code>IncludeCookies</code>. If you choose to include cookies in logs, CloudFront logs all cookies regardless of how you configure the cache behaviors for this distribution. If you don't want to include cookies when you create a distribution or if you want to disable include cookies for an existing distribution, specify <code>false</code> for <code>IncludeCookies</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether you want CloudFront to save access logs to an Amazon S3 bucket. If you don't want to enable logging when you create a distribution or if you want to disable logging for an existing distribution, specify <code>false</code> for <code>Enabled</code>, and specify empty <code>Bucket</code> and <code>Prefix</code> elements. If you specify <code>false</code> for <code>Enabled</code> but you specify values for <code>Bucket</code>, <code>prefix</code>, and <code>IncludeCookies</code>, the values are automatically deleted.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateSource</i> This field is no longer used. Use one of the following fields instead: <ul> <li>  <a href="https://docs.aws.amazon.com/cloudfront/latest/APIReference/API_ViewerCertificate.html#cloudfront-Type-ViewerCertificate-ACMCertificateArn">ACMCertificateArn</a>  </li> <li>  <a href="https://docs.aws.amazon.com/cloudfront/latest/APIReference/API_ViewerCertificate.html#cloudfront-Type-ViewerCertificate-IAMCertificateId">IAMCertificateId</a>  </li> <li>  <a href="https://docs.aws.amazon.com/cloudfront/latest/APIReference/API_ViewerCertificate.html#cloudfront-Type-ViewerCertificate-CloudFrontDefaultCertificate">CloudFrontDefaultCertificate</a>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Certificate</i> This field is no longer used. Use one of the following fields instead: <ul> <li>  <a href="https://docs.aws.amazon.com/cloudfront/latest/APIReference/API_ViewerCertificate.html#cloudfront-Type-ViewerCertificate-ACMCertificateArn">ACMCertificateArn</a>  </li> <li>  <a href="https://docs.aws.amazon.com/cloudfront/latest/APIReference/API_ViewerCertificate.html#cloudfront-Type-ViewerCertificate-IAMCertificateId">IAMCertificateId</a>  </li> <li>  <a href="https://docs.aws.amazon.com/cloudfront/latest/APIReference/API_ViewerCertificate.html#cloudfront-Type-ViewerCertificate-CloudFrontDefaultCertificate">CloudFrontDefaultCertificate</a>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinimumProtocolVersion</i> Specify the security policy that you want CloudFront to use for HTTPS connections. A security policy determines two settings: <ul> <li> The minimum SSL/TLS protocol that CloudFront uses to communicate with viewers </li> <li> The cipher that CloudFront uses to encrypt the content that it returns to viewers </li> </ul> <note> On the CloudFront console, this setting is called <b>Security policy</b>. </note> We recommend that you specify <code>TLSv1.1_2016</code> unless your users are using browsers or devices that do not support TLSv1.1 or later. When both of the following are true, you must specify <code>TLSv1</code> or later for the security policy:  <ul> <li> You're using a custom certificate: you specified a value for <code>ACMCertificateArn</code> or for <code>IAMCertificateId</code>  </li> <li> You're using SNI: you specified <code>sni-only</code> for <code>SSLSupportMethod</code>  </li> </ul> If you specify <code>true</code> for <code>CloudFrontDefaultCertificate</code>, CloudFront automatically sets the security policy to <code>TLSv1</code> regardless of the value that you specify for <code>MinimumProtocolVersion</code>. For information about the relationship between the security policy that you choose and the protocols and ciphers that CloudFront uses to communicate with viewers, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/secure-connections-supported-viewer-protocols-ciphers.html#secure-connections-supported-ciphers"> Supported SSL/TLS Protocols and Ciphers for Communication Between Viewers and CloudFront</a> in the <i>Amazon CloudFront Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SSLSupportMethod</i> If you specify a value for <a href="https://docs.aws.amazon.com/cloudfront/latest/APIReference/API_ViewerCertificate.html#cloudfront-Type-ViewerCertificate-ACMCertificateArn">ACMCertificateArn</a> or for <a href="https://docs.aws.amazon.com/cloudfront/latest/APIReference/API_ViewerCertificate.html#cloudfront-Type-ViewerCertificate-IAMCertificateId">IAMCertificateId</a>, you must also specify how you want CloudFront to serve HTTPS requests: using a method that works for browsers and clients released after 2010 or one that works for all clients. <ul> <li>  <code>sni-only</code>: CloudFront can respond to HTTPS requests from viewers that support Server Name Indication (SNI). All modern browsers support SNI, but there are a few that don't. For a current list of the browsers that support SNI, see the <a href="http://en.wikipedia.org/wiki/Server_Name_Indication">Wikipedia entry Server Name Indication</a>. To learn about options to explore if you have users with browsers that don't include SNI support, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/cnames-https-dedicated-ip-or-sni.html">Choosing How CloudFront Serves HTTPS Requests</a> in the <i>Amazon CloudFront Developer Guide</i>. </li> <li>  <code>vip</code>: CloudFront uses dedicated IP addresses for your content and can respond to HTTPS requests from any viewer. However, there are additional monthly charges. For details, including specific pricing information, see <a href="http://aws.amazon.com/cloudfront/custom-ssl-domains/">Custom SSL options for Amazon CloudFront</a> on the AWS marketing site. </li> </ul> Don't specify a value for <code>SSLSupportMethod</code> if you specified <code>&lt;CloudFrontDefaultCertificate&gt;true&lt;CloudFrontDefaultCertificate&gt;</code>. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/cnames-https-dedicated-ip-or-sni.html">Choosing How CloudFront Serves HTTPS Requests</a> in the <i>Amazon CloudFront Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ACMCertificateArn</i> If you want viewers to use HTTPS to request your objects and you're using an alternate domain name, you must choose the type of certificate that you want to use. Specify the following value if ACM provided your certificate: <ul> <li>  <code>&lt;ACMCertificateArn&gt;<i>ARN for ACM SSL/TLS certificate</i>&lt;ACMCertificateArn&gt;</code> where <code> <i>ARN for ACM SSL/TLS certificate</i> </code> is the ARN for the ACM SSL/TLS certificate that you want to use for this distribution. </li> </ul> If you specify <code>ACMCertificateArn</code>, you must also specify a value for <code>SSLSupportMethod</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMCertificateId</i> If you want viewers to use HTTPS to request your objects and you're using an alternate domain name, you must choose the type of certificate that you want to use. Specify the following value if you purchased your certificate from a third-party certificate authority: <ul> <li>  <code>&lt;IAMCertificateId&gt;<i>IAM certificate ID</i>&lt;IAMCertificateId&gt;</code> where <code> <i>IAM certificate ID</i> </code> is the ID that IAM returned when you added the certificate to the IAM certificate store. </li> </ul> If you specify <code>IAMCertificateId</code>, you must also specify a value for <code>SSLSupportMethod</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudFrontDefaultCertificate</i> If you're using the CloudFront domain name for your distribution, such as <code>d111111abcdef8.cloudfront.net</code>, specify the following value: <ul> <li>  <code>&lt;CloudFrontDefaultCertificate&gt;true&lt;CloudFrontDefaultCertificate&gt; </code>  </li> </ul><br/>A complex type that controls the countries in which your content is distributed. CloudFront determines the location of your users using <code>MaxMind</code> GeoIP databases.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>GeoRestriction</b>: A complex type that controls the countries in which your content is distributed. CloudFront determines the location of your users using <code>MaxMind</code> GeoIP databases.
        :return: ETag:  The current version of the configuration. For example: <code>E2QWRUHAPOMQZL</code>.<br/>Distribution:  The distribution's information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AliasICPRecordals</i> AWS services in China customers must file for an Internet Content Provider (ICP) recordal if they want to serve content publicly on an alternate domain name, also known as a CNAME, that they've added to CloudFront. AliasICPRecordal provides the ICP recordal status for CNAMEs associated with distributions. For more information about ICP recordals, see <a href="https://docs.amazonaws.cn/en_us/aws/latest/userguide/accounts-and-credentials.html"> Signup, Accounts, and Credentials</a> in <i>Getting Started with AWS services in China</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DistributionConfig</i> The current configuration information for the distribution. Send a <code>GET</code> request to the <code>/<i>CloudFront API version</i>/distribution ID/config</code> resource.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DistributionConfig</b>: The current configuration information for the distribution. Send a <code>GET</code> request to the <code>/<i>CloudFront API version</i>/distribution ID/config</code> resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActiveTrustedSigners</i> CloudFront automatically adds this element to the response only if you've set up the distribution to serve private content with signed URLs. The element lists the key pair IDs that CloudFront is aware of for each trusted signer. The <code>Signer</code> child element lists the AWS account number of the trusted signer (or an empty <code>Self</code> element if the signer is you). The <code>Signer</code> element also includes the IDs of any active key pairs associated with the trusted signer's AWS account. If no <code>KeyPairId</code> element appears for a <code>Signer</code>, that signer can't create working signed URLs.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ActiveTrustedSigners</b>: CloudFront automatically adds this element to the response only if you've set up the distribution to serve private content with signed URLs. The element lists the key pair IDs that CloudFront is aware of for each trusted signer. The <code>Signer</code> child element lists the AWS account number of the trusted signer (or an empty <code>Self</code> element if the signer is you). The <code>Signer</code> element also includes the IDs of any active key pairs associated with the trusted signer's AWS account. If no <code>KeyPairId</code> element appears for a <code>Signer</code>, that signer can't create working signed URLs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainName</i> The domain name corresponding to the distribution, for example, <code>d111111abcdef8.cloudfront.net</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InProgressInvalidationBatches</i> The number of invalidation batches currently in progress. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedTime</i> The date and time the distribution was last modified. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> This response element indicates the current status of the distribution. When the status is <code>Deployed</code>, the distribution's information is fully propagated to all CloudFront edge locations. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ARN</i> The ARN (Amazon Resource Name) for the distribution. For example: <code>arn:aws:cloudfront::123456789012:distribution/EDFDVBD632BHDS5</code>, where <code>123456789012</code> is your AWS account ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier for the distribution. For example: <code>EDFDVBD632BHDS5</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> A complex type that contains one <code>Signer</code> complex type for each trusted signer that is specified in the <code>TrustedSigners</code> complex type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of trusted signers specified in the <code>TrustedSigners</code> complex type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Enabled is <code>true</code> if any of the AWS accounts listed in the <code>TrustedSigners</code> complex type for this distribution have active CloudFront key pairs. If not, <code>Enabled</code> is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique value (for example, a date-time stamp) that ensures that the request can't be replayed. If the value of <code>CallerReference</code> is new (regardless of the content of the <code>DistributionConfig</code> object), CloudFront creates a new distribution. If <code>CallerReference</code> is a value that you already sent in a previous request to create a distribution, CloudFront returns a <code>DistributionAlreadyExists</code> error.<br/>A complex type that contains information about CNAMEs (alternate domain names), if any, for this distribution.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Aliases</b>: A complex type that contains information about CNAMEs (alternate domain names), if any, for this distribution.        """
        pass

    def update_field_level_encryption_config(self, Id: str, FieldLevelEncryptionConfig: FieldLevelEncryptionConfig, IfMatch: str=None) -> UpdateFieldLevelEncryptionConfigResult:
        """Update a field-level encryption configuration. 

        :param self:
        :param IfMatch:  The value of the <code>ETag</code> header that you received when retrieving the configuration identity to update. For example: <code>E2QWRUHAPOMQZL</code>.<br/>Id:  The ID of the configuration you want to update.<br/>FieldLevelEncryptionConfig:  Request to update a field-level encryption configuration. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContentTypeProfileConfig</i> A complex data type that specifies when to forward content if a content type isn't recognized and profiles to use as by default in a request if a query argument doesn't specify a profile to use.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ContentTypeProfileConfig</b>: A complex data type that specifies when to forward content if a content type isn't recognized and profiles to use as by default in a request if a query argument doesn't specify a profile to use.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QueryArgProfileConfig</i> A complex data type that specifies when to forward content if a profile isn't found and the profile that can be provided as a query argument in a request.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>QueryArgProfileConfig</b>: A complex data type that specifies when to forward content if a profile isn't found and the profile that can be provided as a query argument in a request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> An optional comment about the configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique number that ensures the request can't be replayed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ForwardWhenQueryArgProfileIsUnknown</i> Flag to set if you want a request to be forwarded to the origin even if the profile specified by the field-level encryption query argument, fle-profile, is unknown.<br/>Profiles specified for query argument-profile mapping for field-level encryption.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>QueryArgProfiles</b>: Profiles specified for query argument-profile mapping for field-level encryption.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ForwardWhenContentTypeIsUnknown</i> The setting in a field-level encryption content type-profile mapping that specifies what to do when an unknown content type is provided for the profile. If true, content is forwarded without being encrypted when the content type is unknown. If false (the default), an error is returned when the content type is unknown. <br/>The configuration for a field-level encryption content type-profile. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ContentTypeProfiles</b>: The configuration for a field-level encryption content type-profile. 
        :return: ETag:  The value of the <code>ETag</code> header that you received when updating the configuration. For example: <code>E2QWRUHAPOMQZL</code>.<br/>FieldLevelEncryption:  Return the results of updating the configuration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FieldLevelEncryptionConfig</i> A complex data type that includes the profile configurations specified for field-level encryption. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>FieldLevelEncryptionConfig</b>: A complex data type that includes the profile configurations specified for field-level encryption. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedTime</i> The last time the field-level encryption configuration was changed. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The configuration ID for a field-level encryption configuration which includes a set of profiles that specify certain selected data fields to be encrypted by specific public keys.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> An optional comment about the configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique number that ensures the request can't be replayed.<br/>A complex data type that specifies when to forward content if a profile isn't found and the profile that can be provided as a query argument in a request.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>QueryArgProfileConfig</b>: A complex data type that specifies when to forward content if a profile isn't found and the profile that can be provided as a query argument in a request.        """
        pass

    def update_field_level_encryption_profile(self, Id: str, FieldLevelEncryptionProfileConfig: FieldLevelEncryptionProfileConfig, IfMatch: str=None) -> UpdateFieldLevelEncryptionProfileResult:
        """Update a field-level encryption profile. 

        :param self:
        :param IfMatch:  The value of the <code>ETag</code> header that you received when retrieving the profile identity to update. For example: <code>E2QWRUHAPOMQZL</code>.<br/>Id:  The ID of the field-level encryption profile request. <br/>FieldLevelEncryptionProfileConfig:  Request to update a field-level encryption profile. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionEntities</i> A complex data type of encryption entities for the field-level encryption profile that include the public key ID, provider, and field patterns for specifying which fields to encrypt with this key.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncryptionEntities</b>: A complex data type of encryption entities for the field-level encryption profile that include the public key ID, provider, and field patterns for specifying which fields to encrypt with this key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> An optional comment for the field-level encryption profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique number that ensures that the request can't be replayed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Profile name for the field-level encryption profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> An array of field patterns in a field-level encryption content type-profile mapping. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> Number of field pattern items in a field-level encryption content type-profile mapping. <br/>
        :return: ETag:  The result of the field-level encryption profile request. <br/>FieldLevelEncryptionProfile:  Return the results of updating the profile.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FieldLevelEncryptionProfileConfig</i> A complex data type that includes the profile name and the encryption entities for the field-level encryption profile.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>FieldLevelEncryptionProfileConfig</b>: A complex data type that includes the profile name and the encryption entities for the field-level encryption profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedTime</i> The last time the field-level encryption profile was updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID for a field-level encryption profile configuration which includes a set of profiles that specify certain selected data fields to be encrypted by specific public keys.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> An optional comment for the field-level encryption profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique number that ensures that the request can't be replayed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Profile name for the field-level encryption profile.<br/>A complex data type of encryption entities for the field-level encryption profile that include the public key ID, provider, and field patterns for specifying which fields to encrypt with this key.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncryptionEntities</b>: A complex data type of encryption entities for the field-level encryption profile that include the public key ID, provider, and field patterns for specifying which fields to encrypt with this key.        """
        pass

    def update_public_key(self, Id: str, PublicKeyConfig: PublicKeyConfig, IfMatch: str=None) -> UpdatePublicKeyResult:
        """Update public key information. Note that the only value you can change is the comment.

        :param self:
        :param IfMatch:  The value of the <code>ETag</code> header that you received when retrieving the public key to update. For example: <code>E2QWRUHAPOMQZL</code>.<br/>Id:  ID of the public key to be updated.<br/>PublicKeyConfig:  Request to update public key information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> An optional comment about a public key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncodedKey</i> The encoded public key that you want to add to CloudFront to use with features like field-level encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name for a public key you add to CloudFront to use with features like field-level encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique number that ensures that the request can't be replayed.<br/>
        :return: ETag:  The current version of the update public key result. For example: <code>E2QWRUHAPOMQZL</code>.<br/>PublicKey:  Return the results of updating the public key.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PublicKeyConfig</i> A complex data type for a public key you add to CloudFront to use with features like field-level encryption.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PublicKeyConfig</b>: A complex data type for a public key you add to CloudFront to use with features like field-level encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> A time you added a public key to CloudFront.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> A unique ID assigned to a public key you've added to CloudFront.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> An optional comment about a public key.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncodedKey</i> The encoded public key that you want to add to CloudFront to use with features like field-level encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name for a public key you add to CloudFront to use with features like field-level encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique number that ensures that the request can't be replayed.<br/>        """
        pass

    def update_streaming_distribution(self, Id: str, StreamingDistributionConfig: StreamingDistributionConfig, IfMatch: str=None) -> UpdateStreamingDistributionResult:
        """Update a streaming distribution. 

        :param self:
        :param IfMatch:  The value of the <code>ETag</code> header that you received when retrieving the streaming distribution's configuration. For example: <code>E2QWRUHAPOMQZL</code>.<br/>Id:  The streaming distribution's id.<br/>StreamingDistributionConfig:  The streaming distribution's configuration information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Whether the streaming distribution is enabled to accept user requests for content.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PriceClass</i> A complex type that contains information about price class for this streaming distribution. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrustedSigners</i> A complex type that specifies any AWS accounts that you want to permit to create signed URLs for private content. If you want the distribution to use signed URLs, include this element; if you want the distribution to use public URLs, remove this element. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/PrivateContent.html">Serving Private Content through CloudFront</a> in the <i>Amazon CloudFront Developer Guide</i>. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TrustedSigners</b>: A complex type that specifies any AWS accounts that you want to permit to create signed URLs for private content. If you want the distribution to use signed URLs, include this element; if you want the distribution to use public URLs, remove this element. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/PrivateContent.html">Serving Private Content through CloudFront</a> in the <i>Amazon CloudFront Developer Guide</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Logging</i> A complex type that controls whether access logs are written for the streaming distribution. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Logging</b>: A complex type that controls whether access logs are written for the streaming distribution. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> Any comments you want to include about the streaming distribution. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Aliases</i> A complex type that contains information about CNAMEs (alternate domain names), if any, for this streaming distribution. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Aliases</b>: A complex type that contains information about CNAMEs (alternate domain names), if any, for this streaming distribution. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Origin</i> A complex type that contains information about the Amazon S3 bucket from which you want CloudFront to get your media files for distribution. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Origin</b>: A complex type that contains information about the Amazon S3 bucket from which you want CloudFront to get your media files for distribution. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique value (for example, a date-time stamp) that ensures that the request can't be replayed. If the value of <code>CallerReference</code> is new (regardless of the content of the <code>StreamingDistributionConfig</code> object), CloudFront creates a new distribution. If <code>CallerReference</code> is a value that you already sent in a previous request to create a distribution, CloudFront returns a <code>DistributionAlreadyExists</code> error.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OriginAccessIdentity</i> The CloudFront origin access identity to associate with the distribution. Use an origin access identity to configure the distribution so that end users can only access objects in an Amazon S3 bucket through CloudFront. If you want end users to be able to access objects using either the CloudFront URL or the Amazon S3 URL, specify an empty <code>OriginAccessIdentity</code> element. To delete the origin access identity from an existing distribution, update the distribution configuration and include an empty <code>OriginAccessIdentity</code> element. To replace the origin access identity, update the distribution configuration and specify the new origin access identity. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/private-content-restricting-access-to-s3.html">Using an Origin Access Identity to Restrict Access to Your Amazon S3 Content</a> in the <i> Amazon CloudFront Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainName</i> The DNS name of the Amazon S3 origin. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> A complex type that contains the CNAME aliases, if any, that you want to associate with this distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of CNAME aliases, if any, that you want to associate with this distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> An optional string that you want CloudFront to prefix to the access log filenames for this streaming distribution, for example, <code>myprefix/</code>. If you want to enable logging, but you don't want to specify a prefix, you still must include an empty <code>Prefix</code> element in the <code>Logging</code> element.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> The Amazon S3 bucket to store the access logs in, for example, <code>myawslogbucket.s3.amazonaws.com</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether you want CloudFront to save access logs to an Amazon S3 bucket. If you don't want to enable logging when you create a streaming distribution or if you want to disable logging for an existing streaming distribution, specify <code>false</code> for <code>Enabled</code>, and specify <code>empty Bucket</code> and <code>Prefix</code> elements. If you specify <code>false</code> for <code>Enabled</code> but you specify values for <code>Bucket</code> and <code>Prefix</code>, the values are automatically deleted. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i>  <b>Optional</b>: A complex type that contains trusted signers for this cache behavior. If <code>Quantity</code> is <code>0</code>, you can omit <code>Items</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of trusted signers for this cache behavior.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether you want to require viewers to use signed URLs to access the files specified by <code>PathPattern</code> and <code>TargetOriginId</code>.<br/>
        :return: ETag:  The current version of the configuration. For example: <code>E2QWRUHAPOMQZL</code>.<br/>StreamingDistribution:  The streaming distribution's information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamingDistributionConfig</i> The current configuration information for the RTMP distribution.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamingDistributionConfig</b>: The current configuration information for the RTMP distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActiveTrustedSigners</i> A complex type that lists the AWS accounts, if any, that you included in the <code>TrustedSigners</code> complex type for this distribution. These are the accounts that you want to allow to create signed URLs for private content. The <code>Signer</code> complex type lists the AWS account number of the trusted signer or <code>self</code> if the signer is the AWS account that created the distribution. The <code>Signer</code> element also includes the IDs of any active CloudFront key pairs that are associated with the trusted signer's AWS account. If no <code>KeyPairId</code> element appears for a <code>Signer</code>, that signer can't create signed URLs. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/PrivateContent.html">Serving Private Content through CloudFront</a> in the <i>Amazon CloudFront Developer Guide</i>. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ActiveTrustedSigners</b>: A complex type that lists the AWS accounts, if any, that you included in the <code>TrustedSigners</code> complex type for this distribution. These are the accounts that you want to allow to create signed URLs for private content. The <code>Signer</code> complex type lists the AWS account number of the trusted signer or <code>self</code> if the signer is the AWS account that created the distribution. The <code>Signer</code> element also includes the IDs of any active CloudFront key pairs that are associated with the trusted signer's AWS account. If no <code>KeyPairId</code> element appears for a <code>Signer</code>, that signer can't create signed URLs. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/PrivateContent.html">Serving Private Content through CloudFront</a> in the <i>Amazon CloudFront Developer Guide</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainName</i> The domain name that corresponds to the streaming distribution, for example, <code>s5c39gqb8ow64r.cloudfront.net</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedTime</i> The date and time that the distribution was last modified. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current status of the RTMP distribution. When the status is <code>Deployed</code>, the distribution's information is propagated to all CloudFront edge locations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ARN</i> The ARN (Amazon Resource Name) for the distribution. For example: <code>arn:aws:cloudfront::123456789012:distribution/EDFDVBD632BHDS5</code>, where <code>123456789012</code> is your AWS account ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier for the RTMP distribution. For example: <code>EGTXBD79EXAMPLE</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Items</i> A complex type that contains one <code>Signer</code> complex type for each trusted signer that is specified in the <code>TrustedSigners</code> complex type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Quantity</i> The number of trusted signers specified in the <code>TrustedSigners</code> complex type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Enabled is <code>true</code> if any of the AWS accounts listed in the <code>TrustedSigners</code> complex type for this distribution have active CloudFront key pairs. If not, <code>Enabled</code> is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique value (for example, a date-time stamp) that ensures that the request can't be replayed. If the value of <code>CallerReference</code> is new (regardless of the content of the <code>StreamingDistributionConfig</code> object), CloudFront creates a new distribution. If <code>CallerReference</code> is a value that you already sent in a previous request to create a distribution, CloudFront returns a <code>DistributionAlreadyExists</code> error.<br/>A complex type that contains information about the Amazon S3 bucket from which you want CloudFront to get your media files for distribution. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Origin</b>: A complex type that contains information about the Amazon S3 bucket from which you want CloudFront to get your media files for distribution.         """
        pass



class CloudHSM(BaseClient):
    """<fullname>AWS CloudHSM Service</fullname> This is documentation for <b>AWS CloudHSM Classic</b>. For more information, see <a href="http://aws.amazon.com/cloudhsm/faqs-classic/">AWS CloudHSM Classic FAQs</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/classic/userguide/">AWS CloudHSM Classic User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/classic/APIReference/">AWS CloudHSM Classic API Reference</a>.  <b>For information about the current version of AWS CloudHSM</b>, see <a href="http://aws.amazon.com/cloudhsm/">AWS CloudHSM</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/latest/userguide/">AWS CloudHSM User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/latest/APIReference/">AWS CloudHSM API Reference</a>."""


    class ListHapgsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_hapgs` for valid parameters"""
            pass
    class ListHsmsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_hsms` for valid parameters"""
            pass
    class ListLunaClientsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_luna_clients` for valid parameters"""
            pass


    class AddTagsToResourceRequest(Mapping):
        pass
    
    class AddTagsToResourceResponse(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreateHapgRequest(Mapping):
        pass
    
    class CreateHapgResponse(Mapping):
        pass
    
    class CreateHsmRequest(Mapping):
        pass
    
    class CreateHsmResponse(Mapping):
        pass
    
    class CreateLunaClientRequest(Mapping):
        pass
    
    class CreateLunaClientResponse(Mapping):
        pass
    
    class DeleteHapgRequest(Mapping):
        pass
    
    class DeleteHapgResponse(Mapping):
        pass
    
    class DeleteHsmRequest(Mapping):
        pass
    
    class DeleteHsmResponse(Mapping):
        pass
    
    class DeleteLunaClientRequest(Mapping):
        pass
    
    class DeleteLunaClientResponse(Mapping):
        pass
    
    class DescribeHapgRequest(Mapping):
        pass
    
    class DescribeHapgResponse(Mapping):
        pass
    
    class DescribeHsmRequest(Mapping):
        pass
    
    class DescribeHsmResponse(Mapping):
        pass
    
    class DescribeLunaClientRequest(Mapping):
        pass
    
    class DescribeLunaClientResponse(Mapping):
        pass
    
    class GetConfigRequest(Mapping):
        pass
    
    class GetConfigResponse(Mapping):
        pass
    
    class HapgArn(object):
        pass
    
    class ListAvailableZonesRequest(Mapping):
        pass
    
    class ListAvailableZonesResponse(Mapping):
        pass
    
    class ListHapgsRequest(Mapping):
        pass
    
    class ListHapgsResponse(Mapping):
        pass
    
    class ListHsmsRequest(Mapping):
        pass
    
    class ListHsmsResponse(Mapping):
        pass
    
    class ListLunaClientsRequest(Mapping):
        pass
    
    class ListLunaClientsResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class ModifyHapgRequest(Mapping):
        pass
    
    class ModifyHapgResponse(Mapping):
        pass
    
    class PartitionSerial(object):
        pass
    
    class ModifyHsmRequest(Mapping):
        pass
    
    class ModifyHsmResponse(Mapping):
        pass
    
    class ModifyLunaClientRequest(Mapping):
        pass
    
    class ModifyLunaClientResponse(Mapping):
        pass
    
    class RemoveTagsFromResourceRequest(Mapping):
        pass
    
    class RemoveTagsFromResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    def add_tags_to_resource(self, TagList: List[Tag], ResourceArn: str) -> AddTagsToResourceResponse:
        """This is documentation for <b>AWS CloudHSM Classic</b>. For more information, see <a href="http://aws.amazon.com/cloudhsm/faqs-classic/">AWS CloudHSM Classic FAQs</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/classic/userguide/">AWS CloudHSM Classic User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/classic/APIReference/">AWS CloudHSM Classic API Reference</a>.  <b>For information about the current version of AWS CloudHSM</b>, see <a href="http://aws.amazon.com/cloudhsm/">AWS CloudHSM</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/latest/userguide/">AWS CloudHSM User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/latest/APIReference/">AWS CloudHSM API Reference</a>. Adds or overwrites one or more tags for the specified AWS CloudHSM resource. Each tag consists of a key and a value. Tag keys must be unique to each resource.

        :param self:
        :param TagList:  One or more tags.<br/>ResourceArn:  The Amazon Resource Name (ARN) of the AWS CloudHSM resource to tag.<br/>
        :return: Status:  The status of the operation.<br/>        """
        pass

    def create_hapg(self, Label: str) -> CreateHapgResponse:
        """This is documentation for <b>AWS CloudHSM Classic</b>. For more information, see <a href="http://aws.amazon.com/cloudhsm/faqs-classic/">AWS CloudHSM Classic FAQs</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/classic/userguide/">AWS CloudHSM Classic User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/classic/APIReference/">AWS CloudHSM Classic API Reference</a>.  <b>For information about the current version of AWS CloudHSM</b>, see <a href="http://aws.amazon.com/cloudhsm/">AWS CloudHSM</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/latest/userguide/">AWS CloudHSM User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/latest/APIReference/">AWS CloudHSM API Reference</a>. Creates a high-availability partition group. A high-availability partition group is a group of partitions that spans multiple physical HSMs.

        :param self:
        :param Label:  The label of the new high-availability partition group.<br/>
        :return: HapgArn:  The ARN of the high-availability partition group.<br/>        """
        pass

    def create_hsm(self, SubscriptionType: str, IamRoleArn: str, SshKey: str, SubnetId: str, EniIp: str=None, ExternalId: str=None, ClientToken: str=None, SyslogIp: str=None) -> CreateHsmResponse:
        """This is documentation for <b>AWS CloudHSM Classic</b>. For more information, see <a href="http://aws.amazon.com/cloudhsm/faqs-classic/">AWS CloudHSM Classic FAQs</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/classic/userguide/">AWS CloudHSM Classic User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/classic/APIReference/">AWS CloudHSM Classic API Reference</a>.  <b>For information about the current version of AWS CloudHSM</b>, see <a href="http://aws.amazon.com/cloudhsm/">AWS CloudHSM</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/latest/userguide/">AWS CloudHSM User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/latest/APIReference/">AWS CloudHSM API Reference</a>. Creates an uninitialized HSM instance. There is an upfront fee charged for each HSM instance that you create with the <code>CreateHsm</code> operation. If you accidentally provision an HSM and want to request a refund, delete the instance using the <a>DeleteHsm</a> operation, go to the <a href="https://console.aws.amazon.com/support/home">AWS Support Center</a>, create a new case, and select <b>Account and Billing Support</b>. <important> It can take up to 20 minutes to create and provision an HSM. You can monitor the status of the HSM with the <a>DescribeHsm</a> operation. The HSM is ready to be initialized when the status changes to <code>RUNNING</code>. </important>

        :param self:
        :param SyslogIp:  The IP address for the syslog monitoring server. The AWS CloudHSM service only supports one syslog monitoring server.<br/>ClientToken:  A user-defined token to ensure idempotence. Subsequent calls to this operation with the same token will be ignored.<br/>SubscriptionType:  Specifies the type of subscription for the HSM. <ul> <li>  <b>PRODUCTION</b> - The HSM is being used in a production environment. </li> <li>  <b>TRIAL</b> - The HSM is being used in a product trial. </li> </ul><br/>ExternalId:  The external ID from <code>IamRoleArn</code>, if present.<br/>IamRoleArn:  The ARN of an IAM role to enable the AWS CloudHSM service to allocate an ENI on your behalf.<br/>EniIp:  The IP address to assign to the HSM's ENI. If an IP address is not specified, an IP address will be randomly chosen from the CIDR range of the subnet.<br/>SshKey:  The SSH public key to install on the HSM.<br/>SubnetId:  The identifier of the subnet in your VPC in which to place the HSM.<br/>
        :return: HsmArn:  The ARN of the HSM.<br/>        """
        pass

    def create_luna_client(self, Certificate: str, Label: str=None) -> CreateLunaClientResponse:
        """This is documentation for <b>AWS CloudHSM Classic</b>. For more information, see <a href="http://aws.amazon.com/cloudhsm/faqs-classic/">AWS CloudHSM Classic FAQs</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/classic/userguide/">AWS CloudHSM Classic User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/classic/APIReference/">AWS CloudHSM Classic API Reference</a>.  <b>For information about the current version of AWS CloudHSM</b>, see <a href="http://aws.amazon.com/cloudhsm/">AWS CloudHSM</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/latest/userguide/">AWS CloudHSM User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/latest/APIReference/">AWS CloudHSM API Reference</a>. Creates an HSM client.

        :param self:
        :param Certificate:  The contents of a Base64-Encoded X.509 v3 certificate to be installed on the HSMs used by this client.<br/>Label:  The label for the client.<br/>
        :return: ClientArn:  The ARN of the client.<br/>        """
        pass

    def delete_hapg(self, HapgArn: str) -> DeleteHapgResponse:
        """This is documentation for <b>AWS CloudHSM Classic</b>. For more information, see <a href="http://aws.amazon.com/cloudhsm/faqs-classic/">AWS CloudHSM Classic FAQs</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/classic/userguide/">AWS CloudHSM Classic User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/classic/APIReference/">AWS CloudHSM Classic API Reference</a>.  <b>For information about the current version of AWS CloudHSM</b>, see <a href="http://aws.amazon.com/cloudhsm/">AWS CloudHSM</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/latest/userguide/">AWS CloudHSM User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/latest/APIReference/">AWS CloudHSM API Reference</a>. Deletes a high-availability partition group.

        :param self:
        :param HapgArn:  The ARN of the high-availability partition group to delete.<br/>
        :return: Status:  The status of the action.<br/>        """
        pass

    def delete_hsm(self, HsmArn: str) -> DeleteHsmResponse:
        """This is documentation for <b>AWS CloudHSM Classic</b>. For more information, see <a href="http://aws.amazon.com/cloudhsm/faqs-classic/">AWS CloudHSM Classic FAQs</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/classic/userguide/">AWS CloudHSM Classic User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/classic/APIReference/">AWS CloudHSM Classic API Reference</a>.  <b>For information about the current version of AWS CloudHSM</b>, see <a href="http://aws.amazon.com/cloudhsm/">AWS CloudHSM</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/latest/userguide/">AWS CloudHSM User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/latest/APIReference/">AWS CloudHSM API Reference</a>. Deletes an HSM. After completion, this operation cannot be undone and your key material cannot be recovered.

        :param self:
        :param HsmArn:  The ARN of the HSM to delete.<br/>
        :return: Status:  The status of the operation.<br/>        """
        pass

    def delete_luna_client(self, ClientArn: str) -> DeleteLunaClientResponse:
        """This is documentation for <b>AWS CloudHSM Classic</b>. For more information, see <a href="http://aws.amazon.com/cloudhsm/faqs-classic/">AWS CloudHSM Classic FAQs</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/classic/userguide/">AWS CloudHSM Classic User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/classic/APIReference/">AWS CloudHSM Classic API Reference</a>.  <b>For information about the current version of AWS CloudHSM</b>, see <a href="http://aws.amazon.com/cloudhsm/">AWS CloudHSM</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/latest/userguide/">AWS CloudHSM User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/latest/APIReference/">AWS CloudHSM API Reference</a>. Deletes a client.

        :param self:
        :param ClientArn:  The ARN of the client to delete.<br/>
        :return: Status:  The status of the action.<br/>        """
        pass

    def describe_hapg(self, HapgArn: str) -> DescribeHapgResponse:
        """This is documentation for <b>AWS CloudHSM Classic</b>. For more information, see <a href="http://aws.amazon.com/cloudhsm/faqs-classic/">AWS CloudHSM Classic FAQs</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/classic/userguide/">AWS CloudHSM Classic User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/classic/APIReference/">AWS CloudHSM Classic API Reference</a>.  <b>For information about the current version of AWS CloudHSM</b>, see <a href="http://aws.amazon.com/cloudhsm/">AWS CloudHSM</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/latest/userguide/">AWS CloudHSM User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/latest/APIReference/">AWS CloudHSM API Reference</a>. Retrieves information about a high-availability partition group.

        :param self:
        :param HapgArn:  The ARN of the high-availability partition group to describe.<br/>
        :return: State:  The state of the high-availability partition group.<br/>PartitionSerialList:  The list of partition serial numbers that belong to the high-availability partition group.<br/>LastModifiedTimestamp:  The date and time the high-availability partition group was last modified.<br/>Label:  The label for the high-availability partition group.<br/>HsmsPendingRegistration:  <p/><br/>HsmsPendingDeletion:  <p/><br/>HsmsLastActionFailed:  <p/><br/>HapgSerial:  The serial number of the high-availability partition group.<br/>HapgArn:  The ARN of the high-availability partition group.<br/>        """
        pass

    def describe_hsm(self, HsmArn: str=None, HsmSerialNumber: str=None) -> DescribeHsmResponse:
        """This is documentation for <b>AWS CloudHSM Classic</b>. For more information, see <a href="http://aws.amazon.com/cloudhsm/faqs-classic/">AWS CloudHSM Classic FAQs</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/classic/userguide/">AWS CloudHSM Classic User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/classic/APIReference/">AWS CloudHSM Classic API Reference</a>.  <b>For information about the current version of AWS CloudHSM</b>, see <a href="http://aws.amazon.com/cloudhsm/">AWS CloudHSM</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/latest/userguide/">AWS CloudHSM User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/latest/APIReference/">AWS CloudHSM API Reference</a>. Retrieves information about an HSM. You can identify the HSM by its ARN or its serial number.

        :param self:
        :param HsmSerialNumber:  The serial number of the HSM. Either the <code>HsmArn</code> or the <code>HsmSerialNumber</code> parameter must be specified.<br/>HsmArn:  The ARN of the HSM. Either the <code>HsmArn</code> or the <code>SerialNumber</code> parameter must be specified.<br/>
        :return: Partitions:  The list of partitions on the HSM.<br/>ServerCertLastUpdated:  The date and time that the server certificate was last updated.<br/>ServerCertUri:  The URI of the certificate server.<br/>SshKeyLastUpdated:  The date and time that the SSH key was last updated.<br/>SshPublicKey:  The public SSH key.<br/>SoftwareVersion:  The HSM software version.<br/>HsmType:  The HSM model type.<br/>VendorName:  The name of the HSM vendor.<br/>SerialNumber:  The serial number of the HSM.<br/>IamRoleArn:  The ARN of the IAM role assigned to the HSM.<br/>SubnetId:  The identifier of the subnet that the HSM is in.<br/>VpcId:  The identifier of the VPC that the HSM is in.<br/>SubscriptionEndDate:  The subscription end date.<br/>SubscriptionStartDate:  The subscription start date.<br/>SubscriptionType:  Specifies the type of subscription for the HSM. <ul> <li>  <b>PRODUCTION</b> - The HSM is being used in a production environment. </li> <li>  <b>TRIAL</b> - The HSM is being used in a product trial. </li> </ul><br/>EniIp:  The IP address assigned to the HSM's ENI.<br/>EniId:  The identifier of the elastic network interface (ENI) attached to the HSM.<br/>AvailabilityZone:  The Availability Zone that the HSM is in.<br/>StatusDetails:  Contains additional information about the status of the HSM.<br/>Status:  The status of the HSM.<br/>HsmArn:  The ARN of the HSM.<br/>        """
        pass

    def describe_luna_client(self, ClientArn: str=None, CertificateFingerprint: str=None) -> DescribeLunaClientResponse:
        """This is documentation for <b>AWS CloudHSM Classic</b>. For more information, see <a href="http://aws.amazon.com/cloudhsm/faqs-classic/">AWS CloudHSM Classic FAQs</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/classic/userguide/">AWS CloudHSM Classic User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/classic/APIReference/">AWS CloudHSM Classic API Reference</a>.  <b>For information about the current version of AWS CloudHSM</b>, see <a href="http://aws.amazon.com/cloudhsm/">AWS CloudHSM</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/latest/userguide/">AWS CloudHSM User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/latest/APIReference/">AWS CloudHSM API Reference</a>. Retrieves information about an HSM client.

        :param self:
        :param CertificateFingerprint:  The certificate fingerprint.<br/>ClientArn:  The ARN of the client.<br/>
        :return: Label:  The label of the client.<br/>LastModifiedTimestamp:  The date and time the client was last modified.<br/>CertificateFingerprint:  The certificate fingerprint.<br/>Certificate:  The certificate installed on the HSMs used by this client.<br/>ClientArn:  The ARN of the client.<br/>        """
        pass

    def get_config(self, HapgList: List[HapgArn], ClientVersion: str, ClientArn: str) -> GetConfigResponse:
        """This is documentation for <b>AWS CloudHSM Classic</b>. For more information, see <a href="http://aws.amazon.com/cloudhsm/faqs-classic/">AWS CloudHSM Classic FAQs</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/classic/userguide/">AWS CloudHSM Classic User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/classic/APIReference/">AWS CloudHSM Classic API Reference</a>.  <b>For information about the current version of AWS CloudHSM</b>, see <a href="http://aws.amazon.com/cloudhsm/">AWS CloudHSM</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/latest/userguide/">AWS CloudHSM User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/latest/APIReference/">AWS CloudHSM API Reference</a>. Gets the configuration files necessary to connect to all high availability partition groups the client is associated with.

        :param self:
        :param HapgList:  A list of ARNs that identify the high-availability partition groups that are associated with the client.<br/>ClientVersion:  The client version.<br/>ClientArn:  The ARN of the client.<br/>
        :return: ConfigCred:  The certificate file containing the server.pem files of the HSMs.<br/>ConfigFile:  The chrystoki.conf configuration file.<br/>ConfigType:  The type of credentials.<br/>        """
        pass

    def list_available_zones(self) -> ListAvailableZonesResponse:
        """This is documentation for <b>AWS CloudHSM Classic</b>. For more information, see <a href="http://aws.amazon.com/cloudhsm/faqs-classic/">AWS CloudHSM Classic FAQs</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/classic/userguide/">AWS CloudHSM Classic User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/classic/APIReference/">AWS CloudHSM Classic API Reference</a>.  <b>For information about the current version of AWS CloudHSM</b>, see <a href="http://aws.amazon.com/cloudhsm/">AWS CloudHSM</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/latest/userguide/">AWS CloudHSM User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/latest/APIReference/">AWS CloudHSM API Reference</a>. Lists the Availability Zones that have available AWS CloudHSM capacity.

        :param self:
        :param 
        :return: AZList:  The list of Availability Zones that have available AWS CloudHSM capacity.<br/>        """
        pass

    def list_hapgs(self, NextToken: str=None) -> ListHapgsResponse:
        """This is documentation for <b>AWS CloudHSM Classic</b>. For more information, see <a href="http://aws.amazon.com/cloudhsm/faqs-classic/">AWS CloudHSM Classic FAQs</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/classic/userguide/">AWS CloudHSM Classic User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/classic/APIReference/">AWS CloudHSM Classic API Reference</a>.  <b>For information about the current version of AWS CloudHSM</b>, see <a href="http://aws.amazon.com/cloudhsm/">AWS CloudHSM</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/latest/userguide/">AWS CloudHSM User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/latest/APIReference/">AWS CloudHSM API Reference</a>. Lists the high-availability partition groups for the account. This operation supports pagination with the use of the <code>NextToken</code> member. If more results are available, the <code>NextToken</code> member of the response contains a token that you pass in the next call to <code>ListHapgs</code> to retrieve the next set of items.

        :param self:
        :param NextToken:  The <code>NextToken</code> value from a previous call to <code>ListHapgs</code>. Pass null if this is the first call.<br/>
        :return: NextToken:  If not null, more results are available. Pass this value to <code>ListHapgs</code> to retrieve the next set of items.<br/>HapgList:  The list of high-availability partition groups.<br/>        """
        pass

    def list_hsms(self, NextToken: str=None) -> ListHsmsResponse:
        """This is documentation for <b>AWS CloudHSM Classic</b>. For more information, see <a href="http://aws.amazon.com/cloudhsm/faqs-classic/">AWS CloudHSM Classic FAQs</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/classic/userguide/">AWS CloudHSM Classic User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/classic/APIReference/">AWS CloudHSM Classic API Reference</a>.  <b>For information about the current version of AWS CloudHSM</b>, see <a href="http://aws.amazon.com/cloudhsm/">AWS CloudHSM</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/latest/userguide/">AWS CloudHSM User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/latest/APIReference/">AWS CloudHSM API Reference</a>. Retrieves the identifiers of all of the HSMs provisioned for the current customer. This operation supports pagination with the use of the <code>NextToken</code> member. If more results are available, the <code>NextToken</code> member of the response contains a token that you pass in the next call to <code>ListHsms</code> to retrieve the next set of items.

        :param self:
        :param NextToken:  The <code>NextToken</code> value from a previous call to <code>ListHsms</code>. Pass null if this is the first call.<br/>
        :return: NextToken:  If not null, more results are available. Pass this value to <code>ListHsms</code> to retrieve the next set of items.<br/>HsmList:  The list of ARNs that identify the HSMs.<br/>        """
        pass

    def list_luna_clients(self, NextToken: str=None) -> ListLunaClientsResponse:
        """This is documentation for <b>AWS CloudHSM Classic</b>. For more information, see <a href="http://aws.amazon.com/cloudhsm/faqs-classic/">AWS CloudHSM Classic FAQs</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/classic/userguide/">AWS CloudHSM Classic User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/classic/APIReference/">AWS CloudHSM Classic API Reference</a>.  <b>For information about the current version of AWS CloudHSM</b>, see <a href="http://aws.amazon.com/cloudhsm/">AWS CloudHSM</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/latest/userguide/">AWS CloudHSM User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/latest/APIReference/">AWS CloudHSM API Reference</a>. Lists all of the clients. This operation supports pagination with the use of the <code>NextToken</code> member. If more results are available, the <code>NextToken</code> member of the response contains a token that you pass in the next call to <code>ListLunaClients</code> to retrieve the next set of items.

        :param self:
        :param NextToken:  The <code>NextToken</code> value from a previous call to <code>ListLunaClients</code>. Pass null if this is the first call.<br/>
        :return: NextToken:  If not null, more results are available. Pass this to <code>ListLunaClients</code> to retrieve the next set of items.<br/>ClientList:  The list of clients.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceArn: str) -> ListTagsForResourceResponse:
        """This is documentation for <b>AWS CloudHSM Classic</b>. For more information, see <a href="http://aws.amazon.com/cloudhsm/faqs-classic/">AWS CloudHSM Classic FAQs</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/classic/userguide/">AWS CloudHSM Classic User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/classic/APIReference/">AWS CloudHSM Classic API Reference</a>.  <b>For information about the current version of AWS CloudHSM</b>, see <a href="http://aws.amazon.com/cloudhsm/">AWS CloudHSM</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/latest/userguide/">AWS CloudHSM User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/latest/APIReference/">AWS CloudHSM API Reference</a>. Returns a list of all tags for the specified AWS CloudHSM resource.

        :param self:
        :param ResourceArn:  The Amazon Resource Name (ARN) of the AWS CloudHSM resource.<br/>
        :return: TagList:  One or more tags.<br/>        """
        pass

    def modify_hapg(self, HapgArn: str, Label: str=None, PartitionSerialList: List[PartitionSerial]=None) -> ModifyHapgResponse:
        """This is documentation for <b>AWS CloudHSM Classic</b>. For more information, see <a href="http://aws.amazon.com/cloudhsm/faqs-classic/">AWS CloudHSM Classic FAQs</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/classic/userguide/">AWS CloudHSM Classic User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/classic/APIReference/">AWS CloudHSM Classic API Reference</a>.  <b>For information about the current version of AWS CloudHSM</b>, see <a href="http://aws.amazon.com/cloudhsm/">AWS CloudHSM</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/latest/userguide/">AWS CloudHSM User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/latest/APIReference/">AWS CloudHSM API Reference</a>. Modifies an existing high-availability partition group.

        :param self:
        :param PartitionSerialList:  The list of partition serial numbers to make members of the high-availability partition group.<br/>Label:  The new label for the high-availability partition group.<br/>HapgArn:  The ARN of the high-availability partition group to modify.<br/>
        :return: HapgArn:  The ARN of the high-availability partition group.<br/>        """
        pass

    def modify_hsm(self, HsmArn: str, SubnetId: str=None, EniIp: str=None, IamRoleArn: str=None, ExternalId: str=None, SyslogIp: str=None) -> ModifyHsmResponse:
        """This is documentation for <b>AWS CloudHSM Classic</b>. For more information, see <a href="http://aws.amazon.com/cloudhsm/faqs-classic/">AWS CloudHSM Classic FAQs</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/classic/userguide/">AWS CloudHSM Classic User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/classic/APIReference/">AWS CloudHSM Classic API Reference</a>.  <b>For information about the current version of AWS CloudHSM</b>, see <a href="http://aws.amazon.com/cloudhsm/">AWS CloudHSM</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/latest/userguide/">AWS CloudHSM User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/latest/APIReference/">AWS CloudHSM API Reference</a>. Modifies an HSM. <important> This operation can result in the HSM being offline for up to 15 minutes while the AWS CloudHSM service is reconfigured. If you are modifying a production HSM, you should ensure that your AWS CloudHSM service is configured for high availability, and consider executing this operation during a maintenance window. </important>

        :param self:
        :param SyslogIp:  The new IP address for the syslog monitoring server. The AWS CloudHSM service only supports one syslog monitoring server.<br/>ExternalId:  The new external ID.<br/>IamRoleArn:  The new IAM role ARN.<br/>EniIp:  The new IP address for the elastic network interface (ENI) attached to the HSM. If the HSM is moved to a different subnet, and an IP address is not specified, an IP address will be randomly chosen from the CIDR range of the new subnet.<br/>SubnetId:  The new identifier of the subnet that the HSM is in. The new subnet must be in the same Availability Zone as the current subnet.<br/>HsmArn:  The ARN of the HSM to modify.<br/>
        :return: HsmArn:  The ARN of the HSM.<br/>        """
        pass

    def modify_luna_client(self, Certificate: str, ClientArn: str) -> ModifyLunaClientResponse:
        """This is documentation for <b>AWS CloudHSM Classic</b>. For more information, see <a href="http://aws.amazon.com/cloudhsm/faqs-classic/">AWS CloudHSM Classic FAQs</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/classic/userguide/">AWS CloudHSM Classic User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/classic/APIReference/">AWS CloudHSM Classic API Reference</a>.  <b>For information about the current version of AWS CloudHSM</b>, see <a href="http://aws.amazon.com/cloudhsm/">AWS CloudHSM</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/latest/userguide/">AWS CloudHSM User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/latest/APIReference/">AWS CloudHSM API Reference</a>. Modifies the certificate used by the client. This action can potentially start a workflow to install the new certificate on the client's HSMs.

        :param self:
        :param Certificate:  The new certificate for the client.<br/>ClientArn:  The ARN of the client.<br/>
        :return: ClientArn:  The ARN of the client.<br/>        """
        pass

    def remove_tags_from_resource(self, TagKeyList: List[TagKey], ResourceArn: str) -> RemoveTagsFromResourceResponse:
        """This is documentation for <b>AWS CloudHSM Classic</b>. For more information, see <a href="http://aws.amazon.com/cloudhsm/faqs-classic/">AWS CloudHSM Classic FAQs</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/classic/userguide/">AWS CloudHSM Classic User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/classic/APIReference/">AWS CloudHSM Classic API Reference</a>.  <b>For information about the current version of AWS CloudHSM</b>, see <a href="http://aws.amazon.com/cloudhsm/">AWS CloudHSM</a>, the <a href="http://docs.aws.amazon.com/cloudhsm/latest/userguide/">AWS CloudHSM User Guide</a>, and the <a href="http://docs.aws.amazon.com/cloudhsm/latest/APIReference/">AWS CloudHSM API Reference</a>. Removes one or more tags from the specified AWS CloudHSM resource. To remove a tag, specify only the tag key to remove (not the value). To overwrite the value for an existing tag, use <a>AddTagsToResource</a>.

        :param self:
        :param TagKeyList:  The tag key or keys to remove. Specify only the tag key to remove (not the value). To overwrite the value for an existing tag, use <a>AddTagsToResource</a>.<br/>ResourceArn:  The Amazon Resource Name (ARN) of the AWS CloudHSM resource.<br/>
        :return: Status:  The status of the operation.<br/>        """
        pass



class CloudHSMV2(BaseClient):
    """For more information about AWS CloudHSM, see <a href="http://aws.amazon.com/cloudhsm/">AWS CloudHSM</a> and the <a href="http://docs.aws.amazon.com/cloudhsm/latest/userguide/">AWS CloudHSM User Guide</a>."""


    class DescribeBackupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_backups` for valid parameters"""
            pass
    class DescribeClustersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_clusters` for valid parameters"""
            pass
    class ListTagsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_tags` for valid parameters"""
            pass


    class CopyBackupToRegionRequest(Mapping):
        pass
    
    class CopyBackupToRegionResponse(Mapping):
        pass
    
    class CreateClusterRequest(Mapping):
        pass
    
    class CreateClusterResponse(Mapping):
        pass
    
    class SubnetId(object):
        pass
    
    class CreateHsmRequest(Mapping):
        pass
    
    class CreateHsmResponse(Mapping):
        pass
    
    class DeleteBackupRequest(Mapping):
        pass
    
    class DeleteBackupResponse(Mapping):
        pass
    
    class DeleteClusterRequest(Mapping):
        pass
    
    class DeleteClusterResponse(Mapping):
        pass
    
    class DeleteHsmRequest(Mapping):
        pass
    
    class DeleteHsmResponse(Mapping):
        pass
    
    class DescribeBackupsRequest(Mapping):
        pass
    
    class DescribeBackupsResponse(Mapping):
        pass
    
    class Filters(object):
        pass
    
    class Boolean(object):
        pass
    
    class DescribeClustersRequest(Mapping):
        pass
    
    class DescribeClustersResponse(Mapping):
        pass
    
    class InitializeClusterRequest(Mapping):
        pass
    
    class InitializeClusterResponse(Mapping):
        pass
    
    class ListTagsRequest(Mapping):
        pass
    
    class ListTagsResponse(Mapping):
        pass
    
    class RestoreBackupRequest(Mapping):
        pass
    
    class RestoreBackupResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    def copy_backup_to_region(self, BackupId: str, DestinationRegion: str) -> CopyBackupToRegionResponse:
        """Copy an AWS CloudHSM cluster backup to a different region.

        :param self:
        :param BackupId:  The ID of the backup that will be copied to the destination region. <br/>DestinationRegion:  The AWS region that will contain your copied CloudHSM cluster backup.<br/>
        :return: DestinationBackup:  Information on the backup that will be copied to the destination region, including CreateTimestamp, SourceBackup, SourceCluster, and Source Region. CreateTimestamp of the destination backup will be the same as that of the source backup. You will need to use the <code>sourceBackupID</code> returned in this operation to use the <a>DescribeBackups</a> operation on the backup that will be copied to the destination region.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceCluster</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceBackup</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceRegion</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateTimestamp</i> <br/>        """
        pass

    def create_cluster(self, HsmType: str, SubnetIds: List[SubnetId], SourceBackupId: str=None) -> CreateClusterResponse:
        """Creates a new AWS CloudHSM cluster.

        :param self:
        :param SourceBackupId:  The identifier (ID) of the cluster backup to restore. Use this value to restore the cluster from a backup instead of creating a new cluster. To find the backup ID, use <a>DescribeBackups</a>.<br/>HsmType:  The type of HSM to use in the cluster. Currently the only allowed value is <code>hsm1.medium</code>.<br/>SubnetIds:  The identifiers (IDs) of the subnets where you are creating the cluster. You must specify at least one subnet. If you specify multiple subnets, they must meet the following criteria: <ul> <li> All subnets must be in the same virtual private cloud (VPC). </li> <li> You can specify only one subnet per Availability Zone. </li> </ul><br/>
        :return: Cluster:  Information about the cluster that was created.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Certificates</i> Contains one or more certificates or a certificate signing request (CSR).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Certificates</b>: Contains one or more certificates or a certificate signing request (CSR).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The identifier (ID) of the virtual private cloud (VPC) that contains the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetMapping</i> A map of the cluster's subnets and their corresponding Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StateMessage</i> A description of the cluster's state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The cluster's state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceBackupId</i> The identifier (ID) of the backup used to create the cluster. This value exists only when the cluster was created from a backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroup</i> The identifier (ID) of the cluster's security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreCoPassword</i> The default password for the cluster's Pre-Crypto Officer (PRECO) user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmType</i> The type of HSM that the cluster contains.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Hsms</i> Contains information about the HSMs in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateTimestamp</i> The date and time when the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterId</i> The cluster's identifier (ID).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupPolicy</i> The cluster's backup policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCertificate</i> The cluster certificate issued (signed) by the issuing certificate authority (CA) of the cluster's owner.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ManufacturerHardwareCertificate</i> The HSM hardware certificate issued (signed) by the hardware manufacturer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AwsHardwareCertificate</i> The HSM hardware certificate issued (signed) by AWS CloudHSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmCertificate</i> The HSM certificate issued (signed) by the HSM hardware.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCsr</i> The cluster's certificate signing request (CSR). The CSR exists only when the cluster's state is <code>UNINITIALIZED</code>.<br/>        """
        pass

    def create_hsm(self, AvailabilityZone: str, ClusterId: str, IpAddress: str=None) -> CreateHsmResponse:
        """Creates a new hardware security module (HSM) in the specified AWS CloudHSM cluster.

        :param self:
        :param IpAddress:  The HSM's IP address. If you specify an IP address, use an available address from the subnet that maps to the Availability Zone where you are creating the HSM. If you don't specify an IP address, one is chosen for you from that subnet.<br/>AvailabilityZone:  The Availability Zone where you are creating the HSM. To find the cluster's Availability Zones, use <a>DescribeClusters</a>.<br/>ClusterId:  The identifier (ID) of the HSM's cluster. To find the cluster ID, use <a>DescribeClusters</a>.<br/>
        :return: Hsm:  Information about the HSM that was created.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StateMessage</i> A description of the HSM's state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The HSM's state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmId</i> The HSM's identifier (ID).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EniIp</i> The IP address of the HSM's elastic network interface (ENI).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EniId</i> The identifier (ID) of the HSM's elastic network interface (ENI).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetId</i> The subnet that contains the HSM's elastic network interface (ENI).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterId</i> The identifier (ID) of the cluster that contains the HSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone that contains the HSM.<br/>        """
        pass

    def delete_backup(self, BackupId: str) -> DeleteBackupResponse:
        """Deletes a specified AWS CloudHSM backup. A backup can be restored up to 7 days after the DeleteBackup request. For more information on restoring a backup, see <a>RestoreBackup</a> 

        :param self:
        :param BackupId:  The ID of the backup to be deleted. To find the ID of a backup, use the <a>DescribeBackups</a> operation.<br/>
        :return: Backup:  Information on the <code>Backup</code> object deleted.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeleteTimestamp</i> The date and time when the backup will be permanently deleted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceCluster</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceBackup</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceRegion</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyTimestamp</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateTimestamp</i> The date and time when the backup was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterId</i> The identifier (ID) of the cluster that was backed up.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupState</i> The state of the backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupId</i> The identifier (ID) of the backup.<br/>        """
        pass

    def delete_cluster(self, ClusterId: str) -> DeleteClusterResponse:
        """Deletes the specified AWS CloudHSM cluster. Before you can delete a cluster, you must delete all HSMs in the cluster. To see if the cluster contains any HSMs, use <a>DescribeClusters</a>. To delete an HSM, use <a>DeleteHsm</a>.

        :param self:
        :param ClusterId:  The identifier (ID) of the cluster that you are deleting. To find the cluster ID, use <a>DescribeClusters</a>.<br/>
        :return: Cluster:  Information about the cluster that was deleted.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Certificates</i> Contains one or more certificates or a certificate signing request (CSR).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Certificates</b>: Contains one or more certificates or a certificate signing request (CSR).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The identifier (ID) of the virtual private cloud (VPC) that contains the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetMapping</i> A map of the cluster's subnets and their corresponding Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StateMessage</i> A description of the cluster's state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The cluster's state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceBackupId</i> The identifier (ID) of the backup used to create the cluster. This value exists only when the cluster was created from a backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroup</i> The identifier (ID) of the cluster's security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreCoPassword</i> The default password for the cluster's Pre-Crypto Officer (PRECO) user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmType</i> The type of HSM that the cluster contains.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Hsms</i> Contains information about the HSMs in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateTimestamp</i> The date and time when the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterId</i> The cluster's identifier (ID).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupPolicy</i> The cluster's backup policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCertificate</i> The cluster certificate issued (signed) by the issuing certificate authority (CA) of the cluster's owner.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ManufacturerHardwareCertificate</i> The HSM hardware certificate issued (signed) by the hardware manufacturer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AwsHardwareCertificate</i> The HSM hardware certificate issued (signed) by AWS CloudHSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmCertificate</i> The HSM certificate issued (signed) by the HSM hardware.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCsr</i> The cluster's certificate signing request (CSR). The CSR exists only when the cluster's state is <code>UNINITIALIZED</code>.<br/>        """
        pass

    def delete_hsm(self, ClusterId: str, HsmId: str=None, EniId: str=None, EniIp: str=None) -> DeleteHsmResponse:
        """Deletes the specified HSM. To specify an HSM, you can use its identifier (ID), the IP address of the HSM's elastic network interface (ENI), or the ID of the HSM's ENI. You need to specify only one of these values. To find these values, use <a>DescribeClusters</a>.

        :param self:
        :param EniIp:  The IP address of the elastic network interface (ENI) of the HSM that you are deleting.<br/>EniId:  The identifier (ID) of the elastic network interface (ENI) of the HSM that you are deleting.<br/>HsmId:  The identifier (ID) of the HSM that you are deleting.<br/>ClusterId:  The identifier (ID) of the cluster that contains the HSM that you are deleting.<br/>
        :return: HsmId:  The identifier (ID) of the HSM that was deleted.<br/>        """
        pass

    def describe_backups(self, NextToken: str=None, MaxResults: int=None, Filters: Filters=None, SortAscending: Boolean=None) -> DescribeBackupsResponse:
        """Gets information about backups of AWS CloudHSM clusters. This is a paginated operation, which means that each response might contain only a subset of all the backups. When the response contains only a subset of backups, it includes a <code>NextToken</code> value. Use this value in a subsequent <code>DescribeBackups</code> request to get more backups. When you receive a response with no <code>NextToken</code> (or an empty or null value), that means there are no more backups to get.

        :param self:
        :param SortAscending:  <br/>Filters:  One or more filters to limit the items returned in the response. Use the <code>backupIds</code> filter to return only the specified backups. Specify backups by their backup identifier (ID). Use the <code>sourceBackupIds</code> filter to return only the backups created from a source backup. The <code>sourceBackupID</code> of a source backup is returned by the <a>CopyBackupToRegion</a> operation. Use the <code>clusterIds</code> filter to return only the backups for the specified clusters. Specify clusters by their cluster identifier (ID). Use the <code>states</code> filter to return only backups that match the specified state.<br/>MaxResults:  The maximum number of backups to return in the response. When there are more backups than the number you specify, the response contains a <code>NextToken</code> value.<br/>NextToken:  The <code>NextToken</code> value that you received in the previous response. Use this value to get more backups.<br/>
        :return: NextToken:  An opaque string that indicates that the response contains only a subset of backups. Use this value in a subsequent <code>DescribeBackups</code> request to get more backups.<br/>Backups:  A list of backups.<br/>        """
        pass

    def describe_clusters(self, Filters: Filters=None, NextToken: str=None, MaxResults: int=None) -> DescribeClustersResponse:
        """Gets information about AWS CloudHSM clusters. This is a paginated operation, which means that each response might contain only a subset of all the clusters. When the response contains only a subset of clusters, it includes a <code>NextToken</code> value. Use this value in a subsequent <code>DescribeClusters</code> request to get more clusters. When you receive a response with no <code>NextToken</code> (or an empty or null value), that means there are no more clusters to get.

        :param self:
        :param MaxResults:  The maximum number of clusters to return in the response. When there are more clusters than the number you specify, the response contains a <code>NextToken</code> value.<br/>NextToken:  The <code>NextToken</code> value that you received in the previous response. Use this value to get more clusters.<br/>Filters:  One or more filters to limit the items returned in the response. Use the <code>clusterIds</code> filter to return only the specified clusters. Specify clusters by their cluster identifier (ID). Use the <code>vpcIds</code> filter to return only the clusters in the specified virtual private clouds (VPCs). Specify VPCs by their VPC identifier (ID). Use the <code>states</code> filter to return only clusters that match the specified state.<br/>
        :return: NextToken:  An opaque string that indicates that the response contains only a subset of clusters. Use this value in a subsequent <code>DescribeClusters</code> request to get more clusters.<br/>Clusters:  A list of clusters.<br/>        """
        pass

    def initialize_cluster(self, TrustAnchor: str, SignedCert: str, ClusterId: str) -> InitializeClusterResponse:
        """Claims an AWS CloudHSM cluster by submitting the cluster certificate issued by your issuing certificate authority (CA) and the CA's root certificate. Before you can claim a cluster, you must sign the cluster's certificate signing request (CSR) with your issuing CA. To get the cluster's CSR, use <a>DescribeClusters</a>.

        :param self:
        :param TrustAnchor:  The issuing certificate of the issuing certificate authority (CA) that issued (signed) the cluster certificate. This can be a root (self-signed) certificate or a certificate chain that begins with the certificate that issued the cluster certificate and ends with a root certificate. The certificate or certificate chain must be in PEM format and can contain a maximum of 5000 characters.<br/>SignedCert:  The cluster certificate issued (signed) by your issuing certificate authority (CA). The certificate must be in PEM format and can contain a maximum of 5000 characters.<br/>ClusterId:  The identifier (ID) of the cluster that you are claiming. To find the cluster ID, use <a>DescribeClusters</a>.<br/>
        :return: StateMessage:  A description of the cluster's state.<br/>State:  The cluster's state.<br/>        """
        pass

    def list_tags(self, ResourceId: str, NextToken: str=None, MaxResults: int=None) -> ListTagsResponse:
        """Gets a list of tags for the specified AWS CloudHSM cluster. This is a paginated operation, which means that each response might contain only a subset of all the tags. When the response contains only a subset of tags, it includes a <code>NextToken</code> value. Use this value in a subsequent <code>ListTags</code> request to get more tags. When you receive a response with no <code>NextToken</code> (or an empty or null value), that means there are no more tags to get.

        :param self:
        :param MaxResults:  The maximum number of tags to return in the response. When there are more tags than the number you specify, the response contains a <code>NextToken</code> value.<br/>NextToken:  The <code>NextToken</code> value that you received in the previous response. Use this value to get more tags.<br/>ResourceId:  The cluster identifier (ID) for the cluster whose tags you are getting. To find the cluster ID, use <a>DescribeClusters</a>.<br/>
        :return: NextToken:  An opaque string that indicates that the response contains only a subset of tags. Use this value in a subsequent <code>ListTags</code> request to get more tags.<br/>TagList:  A list of tags.<br/>        """
        pass

    def restore_backup(self, BackupId: str) -> RestoreBackupResponse:
        """Restores a specified AWS CloudHSM backup that is in the <code>PENDING_DELETION</code> state. For more information on deleting a backup, see <a>DeleteBackup</a>.

        :param self:
        :param BackupId:  The ID of the backup to be restored. To find the ID of a backup, use the <a>DescribeBackups</a> operation.<br/>
        :return: Backup:  Information on the <code>Backup</code> object created.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeleteTimestamp</i> The date and time when the backup will be permanently deleted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceCluster</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceBackup</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceRegion</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyTimestamp</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateTimestamp</i> The date and time when the backup was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterId</i> The identifier (ID) of the cluster that was backed up.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupState</i> The state of the backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupId</i> The identifier (ID) of the backup.<br/>        """
        pass

    def tag_resource(self, TagList: List[Tag], ResourceId: str) -> TagResourceResponse:
        """Adds or overwrites one or more tags for the specified AWS CloudHSM cluster.

        :param self:
        :param TagList:  A list of one or more tags.<br/>ResourceId:  The cluster identifier (ID) for the cluster that you are tagging. To find the cluster ID, use <a>DescribeClusters</a>.<br/>
        :return:         """
        pass

    def untag_resource(self, TagKeyList: List[TagKey], ResourceId: str) -> UntagResourceResponse:
        """Removes the specified tag or tags from the specified AWS CloudHSM cluster.

        :param self:
        :param TagKeyList:  A list of one or more tag keys for the tags that you are removing. Specify only the tag keys, not the tag values.<br/>ResourceId:  The cluster identifier (ID) for the cluster whose tags you are removing. To find the cluster ID, use <a>DescribeClusters</a>.<br/>
        :return:         """
        pass



class CloudSearch(BaseClient):
    """<fullname>Amazon CloudSearch Configuration Service</fullname> You use the Amazon CloudSearch configuration service to create, configure, and manage search domains. Configuration service requests are submitted using the AWS Query protocol. AWS Query requests are HTTP or HTTPS requests submitted via HTTP GET or POST with a query parameter named Action. The endpoint for configuration service requests is region-specific: cloudsearch.<i>region</i>.amazonaws.com. For example, cloudsearch.us-east-1.amazonaws.com. For a current list of supported regions and endpoints, see <a href="http://docs.aws.amazon.com/general/latest/gr/rande.html#cloudsearch_region" target="_blank">Regions and Endpoints</a>."""




    class BuildSuggestersRequest(Mapping):
        pass
    
    class BuildSuggestersResponse(Mapping):
        pass
    
    class CreateDomainRequest(Mapping):
        pass
    
    class CreateDomainResponse(Mapping):
        pass
    
    class DefineAnalysisSchemeRequest(Mapping):
        pass
    
    class DefineAnalysisSchemeResponse(Mapping):
        pass
    
    class AnalysisScheme(Mapping):
        pass
    
    class DefineExpressionRequest(Mapping):
        pass
    
    class DefineExpressionResponse(Mapping):
        pass
    
    class Expression(Mapping):
        pass
    
    class DefineIndexFieldRequest(Mapping):
        pass
    
    class DefineIndexFieldResponse(Mapping):
        pass
    
    class IndexField(Mapping):
        pass
    
    class DefineSuggesterRequest(Mapping):
        pass
    
    class DefineSuggesterResponse(Mapping):
        pass
    
    class Suggester(Mapping):
        pass
    
    class DeleteAnalysisSchemeRequest(Mapping):
        pass
    
    class DeleteAnalysisSchemeResponse(Mapping):
        pass
    
    class DeleteDomainRequest(Mapping):
        pass
    
    class DeleteDomainResponse(Mapping):
        pass
    
    class DeleteExpressionRequest(Mapping):
        pass
    
    class DeleteExpressionResponse(Mapping):
        pass
    
    class DeleteIndexFieldRequest(Mapping):
        pass
    
    class DeleteIndexFieldResponse(Mapping):
        pass
    
    class DeleteSuggesterRequest(Mapping):
        pass
    
    class DeleteSuggesterResponse(Mapping):
        pass
    
    class DescribeAnalysisSchemesRequest(Mapping):
        pass
    
    class DescribeAnalysisSchemesResponse(Mapping):
        pass
    
    class StandardName(object):
        pass
    
    class Boolean(object):
        pass
    
    class DescribeAvailabilityOptionsRequest(Mapping):
        pass
    
    class DescribeAvailabilityOptionsResponse(Mapping):
        pass
    
    class DescribeDomainsRequest(Mapping):
        pass
    
    class DescribeDomainsResponse(Mapping):
        pass
    
    class DomainName(object):
        pass
    
    class DescribeExpressionsRequest(Mapping):
        pass
    
    class DescribeExpressionsResponse(Mapping):
        pass
    
    class DescribeIndexFieldsRequest(Mapping):
        pass
    
    class DescribeIndexFieldsResponse(Mapping):
        pass
    
    class DynamicFieldName(object):
        pass
    
    class DescribeScalingParametersRequest(Mapping):
        pass
    
    class DescribeScalingParametersResponse(Mapping):
        pass
    
    class DescribeServiceAccessPoliciesRequest(Mapping):
        pass
    
    class DescribeServiceAccessPoliciesResponse(Mapping):
        pass
    
    class DescribeSuggestersRequest(Mapping):
        pass
    
    class DescribeSuggestersResponse(Mapping):
        pass
    
    class IndexDocumentsRequest(Mapping):
        pass
    
    class IndexDocumentsResponse(Mapping):
        pass
    
    class ListDomainNamesResponse(Mapping):
        pass
    
    class UpdateAvailabilityOptionsRequest(Mapping):
        pass
    
    class UpdateAvailabilityOptionsResponse(Mapping):
        pass
    
    class UpdateScalingParametersRequest(Mapping):
        pass
    
    class UpdateScalingParametersResponse(Mapping):
        pass
    
    class ScalingParameters(Mapping):
        pass
    
    class UpdateServiceAccessPoliciesRequest(Mapping):
        pass
    
    class UpdateServiceAccessPoliciesResponse(Mapping):
        pass
    
    def build_suggesters(self, DomainName: str) -> BuildSuggestersResponse:
        """Indexes the search suggestions. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/getting-suggestions.html#configuring-suggesters">Configuring Suggesters</a> in the <i>Amazon CloudSearch Developer Guide</i>.

        :param self:
        :param DomainName:  A string that represents the name of a domain. Domain names are unique across the domains owned by an account within an AWS region. Domain names start with a letter or number and can contain the following characters: a-z (lowercase), 0-9, and - (hyphen).<br/>
        :return: FieldNames:  A list of field names.<br/>        """
        pass

    def create_domain(self, DomainName: str) -> CreateDomainResponse:
        """Creates a new search domain. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/creating-domains.html" target="_blank">Creating a Search Domain</a> in the <i>Amazon CloudSearch Developer Guide</i>.

        :param self:
        :param DomainName:  A name for the domain you are creating. Allowed characters are a-z (lower-case letters), 0-9, and hyphen (-). Domain names must start with a letter or number and be at least 3 and no more than 28 characters long.<br/>
        :return: DomainStatus:  The current status of the search domain.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Limits</i> &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Limits</b>: <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SearchInstanceCount</i> The number of search instances that are available to process search requests.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SearchPartitionCount</i> The number of partitions across which the search index is spread.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SearchInstanceType</i> The instance type that is being used to process search requests.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Processing</i> True if processing is being done to activate the current domain configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequiresIndexDocuments</i> True if <a>IndexDocuments</a> needs to be called to activate the current domain configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SearchService</i> The service endpoint for requesting search results from a search domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SearchService</b>: The service endpoint for requesting search results from a search domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DocService</i> The service endpoint for updating documents in a search domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DocService</b>: The service endpoint for updating documents in a search domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Deleted</i> True if the search domain has been deleted. The system must clean up resources dedicated to the search domain when <a>DeleteDomain</a> is called. Newly deleted search domains are returned from <a>DescribeDomains</a> with a true value for IsDeleted for several minutes until resource cleanup is complete.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Created</i> True if the search domain is created. It can take several minutes to initialize a domain when <a>CreateDomain</a> is called. Newly created search domains are returned from <a>DescribeDomains</a> with a false value for Created until domain creation is complete.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ARN</i> The Amazon Resource Name (ARN) of the search domain. See <a href="http://docs.aws.amazon.com/IAM/latest/UserGuide/index.html?Using_Identifiers.html" target="_blank">Identifiers for IAM Entities</a> in <i>Using AWS Identity and Access Management</i> for more information.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainName</i> A string that represents the name of a domain. Domain names are unique across the domains owned by an account within an AWS region. Domain names start with a letter or number and can contain the following characters: a-z (lowercase), 0-9, and - (hyphen).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainId</i> An internally generated unique identifier for a domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> The endpoint to which service requests can be submitted. For example, <code>search-imdb-movies-oopcnjfn6ugofer3zx5iadxxca.eu-west-1.cloudsearch.amazonaws.com</code> or <code>doc-imdb-movies-oopcnjfn6ugofer3zx5iadxxca.eu-west-1.cloudsearch.amazonaws.com</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> The endpoint to which service requests can be submitted. For example, <code>search-imdb-movies-oopcnjfn6ugofer3zx5iadxxca.eu-west-1.cloudsearch.amazonaws.com</code> or <code>doc-imdb-movies-oopcnjfn6ugofer3zx5iadxxca.eu-west-1.cloudsearch.amazonaws.com</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumPartitionCount</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumReplicationCount</i> <br/>        """
        pass

    def define_analysis_scheme(self, AnalysisScheme: AnalysisScheme, DomainName: str) -> DefineAnalysisSchemeResponse:
        """Configures an analysis scheme that can be applied to a <code>text</code> or <code>text-array</code> field to define language-specific text processing options. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/configuring-analysis-schemes.html" target="_blank">Configuring Analysis Schemes</a> in the <i>Amazon CloudSearch Developer Guide</i>.

        :param self:
        :param AnalysisScheme:  Configuration information for an analysis scheme. Each analysis scheme has a unique name and specifies the language of the text to be processed. The following options can be configured for an analysis scheme: <code>Synonyms</code>, <code>Stopwords</code>, <code>StemmingDictionary</code>, <code>JapaneseTokenizationDictionary</code> and <code>AlgorithmicStemming</code>.:<br/>DomainName:  A string that represents the name of a domain. Domain names are unique across the domains owned by an account within an AWS region. Domain names start with a letter or number and can contain the following characters: a-z (lowercase), 0-9, and - (hyphen).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AnalysisOptions</i> Synonyms, stopwords, and stemming options for an analysis scheme. Includes tokenization dictionary for Japanese.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AnalysisOptions</b>: Synonyms, stopwords, and stemming options for an analysis scheme. Includes tokenization dictionary for Japanese.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AnalysisSchemeLanguage</i> An <a href="http://tools.ietf.org/html/rfc4646" target="_blank">IETF RFC 4646</a> language code or <code>mul</code> for multiple languages.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AnalysisSchemeName</i> Names must begin with a letter and can contain the following characters: a-z (lowercase), 0-9, and _ (underscore).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AlgorithmicStemming</i> The level of algorithmic stemming to perform: <code>none</code>, <code>minimal</code>, <code>light</code>, or <code>full</code>. The available levels vary depending on the language. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/text-processing.html#text-processing-settings" target="_blank">Language Specific Text Processing Settings</a> in the <i>Amazon CloudSearch Developer Guide</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>JapaneseTokenizationDictionary</i> A JSON array that contains a collection of terms, tokens, readings and part of speech for Japanese Tokenizaiton. The Japanese tokenization dictionary enables you to override the default tokenization for selected terms. This is only valid for Japanese language fields.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StemmingDictionary</i> A JSON object that contains a collection of string:value pairs that each map a term to its stem. For example, <code>{"term1": "stem1", "term2": "stem2", "term3": "stem3"}</code>. The stemming dictionary is applied in addition to any algorithmic stemming. This enables you to override the results of the algorithmic stemming to correct specific cases of overstemming or understemming. The maximum size of a stemming dictionary is 500 KB.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Stopwords</i> A JSON array of terms to ignore during indexing and searching. For example, <code>["a", "an", "the", "of"]</code>. The stopwords dictionary must explicitly list each word you want to ignore. Wildcards and regular expressions are not supported. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Synonyms</i> A JSON object that defines synonym groups and aliases. A synonym group is an array of arrays, where each sub-array is a group of terms where each term in the group is considered a synonym of every other term in the group. The aliases value is an object that contains a collection of string:value pairs where the string specifies a term and the array of values specifies each of the aliases for that term. An alias is considered a synonym of the specified term, but the term is not considered a synonym of the alias. For more information about specifying synonyms, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/configuring-analysis-schemes.html#synonyms">Synonyms</a> in the <i>Amazon CloudSearch Developer Guide</i>.<br/>
        :return: AnalysisScheme:  The status and configuration of an <code>AnalysisScheme</code>.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of domain configuration option.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>: The status of domain configuration option.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> Configuration information for an analysis scheme. Each analysis scheme has a unique name and specifies the language of the text to be processed. The following options can be configured for an analysis scheme: <code>Synonyms</code>, <code>Stopwords</code>, <code>StemmingDictionary</code>, <code>JapaneseTokenizationDictionary</code> and <code>AlgorithmicStemming</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>: Configuration information for an analysis scheme. Each analysis scheme has a unique name and specifies the language of the text to be processed. The following options can be configured for an analysis scheme: <code>Synonyms</code>, <code>Stopwords</code>, <code>StemmingDictionary</code>, <code>JapaneseTokenizationDictionary</code> and <code>AlgorithmicStemming</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AnalysisSchemeLanguage</i> An <a href="http://tools.ietf.org/html/rfc4646" target="_blank">IETF RFC 4646</a> language code or <code>mul</code> for multiple languages.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AnalysisSchemeName</i> Names must begin with a letter and can contain the following characters: a-z (lowercase), 0-9, and _ (underscore).<br/>Synonyms, stopwords, and stemming options for an analysis scheme. Includes tokenization dictionary for Japanese.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AnalysisOptions</b>: Synonyms, stopwords, and stemming options for an analysis scheme. Includes tokenization dictionary for Japanese.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingDeletion</i> Indicates that the option will be deleted once processing is complete.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of processing a change to an option. Possible values: <ul> <li> <code>RequiresIndexDocuments</code>: the option's latest value will not be deployed until <a>IndexDocuments</a> has been called and indexing is complete.</li> <li> <code>Processing</code>: the option's latest value is in the process of being activated. </li> <li> <code>Active</code>: the option's latest value is completely deployed.</li> <li> <code>FailedToValidate</code>: the option value is not compatible with the domain's data and cannot be used to index the data. You must either modify the option value or update or remove the incompatible documents.</li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateVersion</i> A unique integer that indicates when this option was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateDate</i> A timestamp for when this option was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> A timestamp for when this option was created.<br/>        """
        pass

    def define_expression(self, Expression: Expression, DomainName: str) -> DefineExpressionResponse:
        """Configures an <code><a>Expression</a></code> for the search domain. Used to create new expressions and modify existing ones. If the expression exists, the new configuration replaces the old one. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/configuring-expressions.html" target="_blank">Configuring Expressions</a> in the <i>Amazon CloudSearch Developer Guide</i>.

        :param self:
        :param Expression:  A named expression that can be evaluated at search time. Can be used to sort the search results, define other expressions, or return computed information in the search results. :<br/>DomainName:  A string that represents the name of a domain. Domain names are unique across the domains owned by an account within an AWS region. Domain names start with a letter or number and can contain the following characters: a-z (lowercase), 0-9, and - (hyphen).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpressionValue</i> The expression to evaluate for sorting while processing a search request. The <code>Expression</code> syntax is based on JavaScript expressions. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/configuring-expressions.html" target="_blank">Configuring Expressions</a> in the <i>Amazon CloudSearch Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpressionName</i> Names must begin with a letter and can contain the following characters: a-z (lowercase), 0-9, and _ (underscore).<br/>
        :return: Expression:  The value of an <code>Expression</code> and its current status.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of domain configuration option.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>: The status of domain configuration option.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> The expression that is evaluated for sorting while processing a search request.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>: The expression that is evaluated for sorting while processing a search request.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpressionValue</i> The expression to evaluate for sorting while processing a search request. The <code>Expression</code> syntax is based on JavaScript expressions. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/configuring-expressions.html" target="_blank">Configuring Expressions</a> in the <i>Amazon CloudSearch Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpressionName</i> Names must begin with a letter and can contain the following characters: a-z (lowercase), 0-9, and _ (underscore).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingDeletion</i> Indicates that the option will be deleted once processing is complete.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of processing a change to an option. Possible values: <ul> <li> <code>RequiresIndexDocuments</code>: the option's latest value will not be deployed until <a>IndexDocuments</a> has been called and indexing is complete.</li> <li> <code>Processing</code>: the option's latest value is in the process of being activated. </li> <li> <code>Active</code>: the option's latest value is completely deployed.</li> <li> <code>FailedToValidate</code>: the option value is not compatible with the domain's data and cannot be used to index the data. You must either modify the option value or update or remove the incompatible documents.</li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateVersion</i> A unique integer that indicates when this option was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateDate</i> A timestamp for when this option was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> A timestamp for when this option was created.<br/>        """
        pass

    def define_index_field(self, IndexField: IndexField, DomainName: str) -> DefineIndexFieldResponse:
        """Configures an <code><a>IndexField</a></code> for the search domain. Used to create new fields and modify existing ones. You must specify the name of the domain you are configuring and an index field configuration. The index field configuration specifies a unique name, the index field type, and the options you want to configure for the field. The options you can specify depend on the <code><a>IndexFieldType</a></code>. If the field exists, the new configuration replaces the old one. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/configuring-index-fields.html" target="_blank">Configuring Index Fields</a> in the <i>Amazon CloudSearch Developer Guide</i>. 

        :param self:
        :param IndexField:  The index field and field options you want to configure. :<br/>DomainName:  A string that represents the name of a domain. Domain names are unique across the domains owned by an account within an AWS region. Domain names start with a letter or number and can contain the following characters: a-z (lowercase), 0-9, and - (hyphen).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DateArrayOptions</i> Options for a field that contains an array of dates. Present if <code>IndexFieldType</code> specifies the field is of type <code>date-array</code>. All options are enabled by default.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DateArrayOptions</b>: Options for a field that contains an array of dates. Present if <code>IndexFieldType</code> specifies the field is of type <code>date-array</code>. All options are enabled by default.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TextArrayOptions</i> Options for a field that contains an array of text strings. Present if <code>IndexFieldType</code> specifies the field is of type <code>text-array</code>. A <code>text-array</code> field is always searchable. All options are enabled by default.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TextArrayOptions</b>: Options for a field that contains an array of text strings. Present if <code>IndexFieldType</code> specifies the field is of type <code>text-array</code>. A <code>text-array</code> field is always searchable. All options are enabled by default.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LiteralArrayOptions</i> Options for a field that contains an array of literal strings. Present if <code>IndexFieldType</code> specifies the field is of type <code>literal-array</code>. All options are enabled by default.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LiteralArrayOptions</b>: Options for a field that contains an array of literal strings. Present if <code>IndexFieldType</code> specifies the field is of type <code>literal-array</code>. All options are enabled by default.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DoubleArrayOptions</i> Options for a field that contains an array of double-precision 64-bit floating point values. Present if <code>IndexFieldType</code> specifies the field is of type <code>double-array</code>. All options are enabled by default.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DoubleArrayOptions</b>: Options for a field that contains an array of double-precision 64-bit floating point values. Present if <code>IndexFieldType</code> specifies the field is of type <code>double-array</code>. All options are enabled by default.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IntArrayOptions</i> Options for a field that contains an array of 64-bit signed integers. Present if <code>IndexFieldType</code> specifies the field is of type <code>int-array</code>. All options are enabled by default.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>IntArrayOptions</b>: Options for a field that contains an array of 64-bit signed integers. Present if <code>IndexFieldType</code> specifies the field is of type <code>int-array</code>. All options are enabled by default.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatLonOptions</i> Options for a latlon field. A latlon field contains a location stored as a latitude and longitude value pair. Present if <code>IndexFieldType</code> specifies the field is of type <code>latlon</code>. All options are enabled by default.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LatLonOptions</b>: Options for a latlon field. A latlon field contains a location stored as a latitude and longitude value pair. Present if <code>IndexFieldType</code> specifies the field is of type <code>latlon</code>. All options are enabled by default.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DateOptions</i> Options for a date field. Dates and times are specified in UTC (Coordinated Universal Time) according to IETF RFC3339: yyyy-mm-ddT00:00:00Z. Present if <code>IndexFieldType</code> specifies the field is of type <code>date</code>. All options are enabled by default.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DateOptions</b>: Options for a date field. Dates and times are specified in UTC (Coordinated Universal Time) according to IETF RFC3339: yyyy-mm-ddT00:00:00Z. Present if <code>IndexFieldType</code> specifies the field is of type <code>date</code>. All options are enabled by default.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TextOptions</i> Options for text field. Present if <code>IndexFieldType</code> specifies the field is of type <code>text</code>. A <code>text</code> field is always searchable. All options are enabled by default.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TextOptions</b>: Options for text field. Present if <code>IndexFieldType</code> specifies the field is of type <code>text</code>. A <code>text</code> field is always searchable. All options are enabled by default.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LiteralOptions</i> Options for literal field. Present if <code>IndexFieldType</code> specifies the field is of type <code>literal</code>. All options are enabled by default.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LiteralOptions</b>: Options for literal field. Present if <code>IndexFieldType</code> specifies the field is of type <code>literal</code>. All options are enabled by default.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DoubleOptions</i> Options for a double-precision 64-bit floating point field. Present if <code>IndexFieldType</code> specifies the field is of type <code>double</code>. All options are enabled by default.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DoubleOptions</b>: Options for a double-precision 64-bit floating point field. Present if <code>IndexFieldType</code> specifies the field is of type <code>double</code>. All options are enabled by default.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IntOptions</i> Options for a 64-bit signed integer field. Present if <code>IndexFieldType</code> specifies the field is of type <code>int</code>. All options are enabled by default.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>IntOptions</b>: Options for a 64-bit signed integer field. Present if <code>IndexFieldType</code> specifies the field is of type <code>int</code>. All options are enabled by default.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IndexFieldType</i> The type of field. The valid options for a field depend on the field type. For more information about the supported field types, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/configuring-index-fields.html" target="_blank">Configuring Index Fields</a> in the <i>Amazon CloudSearch Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IndexFieldName</i> A string that represents the name of an index field. CloudSearch supports regular index fields as well as dynamic fields. A dynamic field's name defines a pattern that begins or ends with a wildcard. Any document fields that don't map to a regular index field but do match a dynamic field's pattern are configured with the dynamic field's indexing options.  Regular field names begin with a letter and can contain the following characters: a-z (lowercase), 0-9, and _ (underscore). Dynamic field names must begin or end with a wildcard (*). The wildcard can also be the only character in a dynamic field name. Multiple wildcards, and wildcards embedded within a string are not supported.  The name <code>score</code> is reserved and cannot be used as a field name. To reference a document's ID, you can use the name <code>_id</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SortEnabled</i> Whether the field can be used to sort the search results.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReturnEnabled</i> Whether the contents of the field can be returned in the search results.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SearchEnabled</i> Whether the contents of the field are searchable.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FacetEnabled</i> Whether facet information can be returned for the field.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceField</i> The name of the source field to map to the field. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultValue</i> A value to use for the field if the field isn't specified for a document. This can be important if you are using the field in an expression and that field is not present in every document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SortEnabled</i> Whether the field can be used to sort the search results.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReturnEnabled</i> Whether the contents of the field can be returned in the search results.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SearchEnabled</i> Whether the contents of the field are searchable.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FacetEnabled</i> Whether facet information can be returned for the field.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceField</i> The name of the source field to map to the field. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultValue</i> A value to use for the field if the field isn't specified for a document. This can be important if you are using the field in an expression and that field is not present in every document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SortEnabled</i> Whether the field can be used to sort the search results.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReturnEnabled</i> Whether the contents of the field can be returned in the search results.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SearchEnabled</i> Whether the contents of the field are searchable.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FacetEnabled</i> Whether facet information can be returned for the field.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceField</i> A string that represents the name of an index field. CloudSearch supports regular index fields as well as dynamic fields. A dynamic field's name defines a pattern that begins or ends with a wildcard. Any document fields that don't map to a regular index field but do match a dynamic field's pattern are configured with the dynamic field's indexing options.  Regular field names begin with a letter and can contain the following characters: a-z (lowercase), 0-9, and _ (underscore). Dynamic field names must begin or end with a wildcard (*). The wildcard can also be the only character in a dynamic field name. Multiple wildcards, and wildcards embedded within a string are not supported.  The name <code>score</code> is reserved and cannot be used as a field name. To reference a document's ID, you can use the name <code>_id</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultValue</i> A value to use for the field if the field isn't specified for a document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AnalysisScheme</i> The name of an analysis scheme for a <code>text</code> field.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HighlightEnabled</i> Whether highlights can be returned for the field.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SortEnabled</i> Whether the field can be used to sort the search results.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReturnEnabled</i> Whether the contents of the field can be returned in the search results.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceField</i> A string that represents the name of an index field. CloudSearch supports regular index fields as well as dynamic fields. A dynamic field's name defines a pattern that begins or ends with a wildcard. Any document fields that don't map to a regular index field but do match a dynamic field's pattern are configured with the dynamic field's indexing options.  Regular field names begin with a letter and can contain the following characters: a-z (lowercase), 0-9, and _ (underscore). Dynamic field names must begin or end with a wildcard (*). The wildcard can also be the only character in a dynamic field name. Multiple wildcards, and wildcards embedded within a string are not supported.  The name <code>score</code> is reserved and cannot be used as a field name. To reference a document's ID, you can use the name <code>_id</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultValue</i> A value to use for the field if the field isn't specified for a document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SortEnabled</i> Whether the field can be used to sort the search results.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReturnEnabled</i> Whether the contents of the field can be returned in the search results.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SearchEnabled</i> Whether the contents of the field are searchable.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FacetEnabled</i> Whether facet information can be returned for the field.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceField</i> A string that represents the name of an index field. CloudSearch supports regular index fields as well as dynamic fields. A dynamic field's name defines a pattern that begins or ends with a wildcard. Any document fields that don't map to a regular index field but do match a dynamic field's pattern are configured with the dynamic field's indexing options.  Regular field names begin with a letter and can contain the following characters: a-z (lowercase), 0-9, and _ (underscore). Dynamic field names must begin or end with a wildcard (*). The wildcard can also be the only character in a dynamic field name. Multiple wildcards, and wildcards embedded within a string are not supported.  The name <code>score</code> is reserved and cannot be used as a field name. To reference a document's ID, you can use the name <code>_id</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultValue</i> A value to use for the field if the field isn't specified for a document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SortEnabled</i> Whether the field can be used to sort the search results.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReturnEnabled</i> Whether the contents of the field can be returned in the search results.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SearchEnabled</i> Whether the contents of the field are searchable.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FacetEnabled</i> Whether facet information can be returned for the field.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceField</i> A string that represents the name of an index field. CloudSearch supports regular index fields as well as dynamic fields. A dynamic field's name defines a pattern that begins or ends with a wildcard. Any document fields that don't map to a regular index field but do match a dynamic field's pattern are configured with the dynamic field's indexing options.  Regular field names begin with a letter and can contain the following characters: a-z (lowercase), 0-9, and _ (underscore). Dynamic field names must begin or end with a wildcard (*). The wildcard can also be the only character in a dynamic field name. Multiple wildcards, and wildcards embedded within a string are not supported.  The name <code>score</code> is reserved and cannot be used as a field name. To reference a document's ID, you can use the name <code>_id</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultValue</i> A value to use for the field if the field isn't specified for a document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReturnEnabled</i> Whether the contents of the field can be returned in the search results.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SearchEnabled</i> Whether the contents of the field are searchable.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FacetEnabled</i> Whether facet information can be returned for the field.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceFields</i> A list of source fields to map to the field. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultValue</i> A value to use for the field if the field isn't specified for a document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReturnEnabled</i> Whether the contents of the field can be returned in the search results.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SearchEnabled</i> Whether the contents of the field are searchable.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FacetEnabled</i> Whether facet information can be returned for the field.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceFields</i> A list of source fields to map to the field. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultValue</i> A value to use for the field if the field isn't specified for a document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReturnEnabled</i> Whether the contents of the field can be returned in the search results.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SearchEnabled</i> Whether the contents of the field are searchable.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FacetEnabled</i> Whether facet information can be returned for the field.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceFields</i> A list of source fields to map to the field. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultValue</i> A value to use for the field if the field isn't specified for a document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AnalysisScheme</i> The name of an analysis scheme for a <code>text-array</code> field.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HighlightEnabled</i> Whether highlights can be returned for the field.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReturnEnabled</i> Whether the contents of the field can be returned in the search results.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceFields</i> A list of source fields to map to the field. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultValue</i> A value to use for the field if the field isn't specified for a document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReturnEnabled</i> Whether the contents of the field can be returned in the search results.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SearchEnabled</i> Whether the contents of the field are searchable.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FacetEnabled</i> Whether facet information can be returned for the field.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceFields</i> A list of source fields to map to the field. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultValue</i> A value to use for the field if the field isn't specified for a document.<br/>
        :return: IndexField:  The value of an <code>IndexField</code> and its current status.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of domain configuration option.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>: The status of domain configuration option.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> Configuration information for a field in the index, including its name, type, and options. The supported options depend on the <code><a>IndexFieldType</a></code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>: Configuration information for a field in the index, including its name, type, and options. The supported options depend on the <code><a>IndexFieldType</a></code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IndexFieldType</i> The type of field. The valid options for a field depend on the field type. For more information about the supported field types, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/configuring-index-fields.html" target="_blank">Configuring Index Fields</a> in the <i>Amazon CloudSearch Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IndexFieldName</i> A string that represents the name of an index field. CloudSearch supports regular index fields as well as dynamic fields. A dynamic field's name defines a pattern that begins or ends with a wildcard. Any document fields that don't map to a regular index field but do match a dynamic field's pattern are configured with the dynamic field's indexing options.  Regular field names begin with a letter and can contain the following characters: a-z (lowercase), 0-9, and _ (underscore). Dynamic field names must begin or end with a wildcard (*). The wildcard can also be the only character in a dynamic field name. Multiple wildcards, and wildcards embedded within a string are not supported.  The name <code>score</code> is reserved and cannot be used as a field name. To reference a document's ID, you can use the name <code>_id</code>. <br/>Options for a 64-bit signed integer field. Present if <code>IndexFieldType</code> specifies the field is of type <code>int</code>. All options are enabled by default.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>IntOptions</b>: Options for a 64-bit signed integer field. Present if <code>IndexFieldType</code> specifies the field is of type <code>int</code>. All options are enabled by default.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingDeletion</i> Indicates that the option will be deleted once processing is complete.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of processing a change to an option. Possible values: <ul> <li> <code>RequiresIndexDocuments</code>: the option's latest value will not be deployed until <a>IndexDocuments</a> has been called and indexing is complete.</li> <li> <code>Processing</code>: the option's latest value is in the process of being activated. </li> <li> <code>Active</code>: the option's latest value is completely deployed.</li> <li> <code>FailedToValidate</code>: the option value is not compatible with the domain's data and cannot be used to index the data. You must either modify the option value or update or remove the incompatible documents.</li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateVersion</i> A unique integer that indicates when this option was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateDate</i> A timestamp for when this option was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> A timestamp for when this option was created.<br/>        """
        pass

    def define_suggester(self, Suggester: Suggester, DomainName: str) -> DefineSuggesterResponse:
        """Configures a suggester for a domain. A suggester enables you to display possible matches before users finish typing their queries. When you configure a suggester, you must specify the name of the text field you want to search for possible matches and a unique name for the suggester. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/getting-suggestions.html" target="_blank">Getting Search Suggestions</a> in the <i>Amazon CloudSearch Developer Guide</i>.

        :param self:
        :param Suggester:  Configuration information for a search suggester. Each suggester has a unique name and specifies the text field you want to use for suggestions. The following options can be configured for a suggester: <code>FuzzyMatching</code>, <code>SortExpression</code>. :<br/>DomainName:  A string that represents the name of a domain. Domain names are unique across the domains owned by an account within an AWS region. Domain names start with a letter or number and can contain the following characters: a-z (lowercase), 0-9, and - (hyphen).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentSuggesterOptions</i> Options for a search suggester.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DocumentSuggesterOptions</b>: Options for a search suggester.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SuggesterName</i> Names must begin with a letter and can contain the following characters: a-z (lowercase), 0-9, and _ (underscore).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SortExpression</i> An expression that computes a score for each suggestion to control how they are sorted. The scores are rounded to the nearest integer, with a floor of 0 and a ceiling of 2^31-1. A document's relevance score is not computed for suggestions, so sort expressions cannot reference the <code>_score</code> value. To sort suggestions using a numeric field or existing expression, simply specify the name of the field or expression. If no expression is configured for the suggester, the suggestions are sorted with the closest matches listed first.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FuzzyMatching</i> The level of fuzziness allowed when suggesting matches for a string: <code>none</code>, <code>low</code>, or <code>high</code>. With none, the specified string is treated as an exact prefix. With low, suggestions must differ from the specified string by no more than one character. With high, suggestions can differ by up to two characters. The default is none. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceField</i> The name of the index field you want to use for suggestions. <br/>
        :return: Suggester:  The value of a <code>Suggester</code> and its current status.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of domain configuration option.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>: The status of domain configuration option.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> Configuration information for a search suggester. Each suggester has a unique name and specifies the text field you want to use for suggestions. The following options can be configured for a suggester: <code>FuzzyMatching</code>, <code>SortExpression</code>. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>: Configuration information for a search suggester. Each suggester has a unique name and specifies the text field you want to use for suggestions. The following options can be configured for a suggester: <code>FuzzyMatching</code>, <code>SortExpression</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SuggesterName</i> Names must begin with a letter and can contain the following characters: a-z (lowercase), 0-9, and _ (underscore).<br/>Options for a search suggester.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DocumentSuggesterOptions</b>: Options for a search suggester.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingDeletion</i> Indicates that the option will be deleted once processing is complete.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of processing a change to an option. Possible values: <ul> <li> <code>RequiresIndexDocuments</code>: the option's latest value will not be deployed until <a>IndexDocuments</a> has been called and indexing is complete.</li> <li> <code>Processing</code>: the option's latest value is in the process of being activated. </li> <li> <code>Active</code>: the option's latest value is completely deployed.</li> <li> <code>FailedToValidate</code>: the option value is not compatible with the domain's data and cannot be used to index the data. You must either modify the option value or update or remove the incompatible documents.</li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateVersion</i> A unique integer that indicates when this option was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateDate</i> A timestamp for when this option was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> A timestamp for when this option was created.<br/>        """
        pass

    def delete_analysis_scheme(self, AnalysisSchemeName: str, DomainName: str) -> DeleteAnalysisSchemeResponse:
        """Deletes an analysis scheme. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/configuring-analysis-schemes.html" target="_blank">Configuring Analysis Schemes</a> in the <i>Amazon CloudSearch Developer Guide</i>. 

        :param self:
        :param AnalysisSchemeName:  The name of the analysis scheme you want to delete.<br/>DomainName:  A string that represents the name of a domain. Domain names are unique across the domains owned by an account within an AWS region. Domain names start with a letter or number and can contain the following characters: a-z (lowercase), 0-9, and - (hyphen).<br/>
        :return: AnalysisScheme:  The status of the analysis scheme being deleted.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of domain configuration option.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>: The status of domain configuration option.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> Configuration information for an analysis scheme. Each analysis scheme has a unique name and specifies the language of the text to be processed. The following options can be configured for an analysis scheme: <code>Synonyms</code>, <code>Stopwords</code>, <code>StemmingDictionary</code>, <code>JapaneseTokenizationDictionary</code> and <code>AlgorithmicStemming</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>: Configuration information for an analysis scheme. Each analysis scheme has a unique name and specifies the language of the text to be processed. The following options can be configured for an analysis scheme: <code>Synonyms</code>, <code>Stopwords</code>, <code>StemmingDictionary</code>, <code>JapaneseTokenizationDictionary</code> and <code>AlgorithmicStemming</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AnalysisSchemeLanguage</i> An <a href="http://tools.ietf.org/html/rfc4646" target="_blank">IETF RFC 4646</a> language code or <code>mul</code> for multiple languages.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AnalysisSchemeName</i> Names must begin with a letter and can contain the following characters: a-z (lowercase), 0-9, and _ (underscore).<br/>Synonyms, stopwords, and stemming options for an analysis scheme. Includes tokenization dictionary for Japanese.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AnalysisOptions</b>: Synonyms, stopwords, and stemming options for an analysis scheme. Includes tokenization dictionary for Japanese.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingDeletion</i> Indicates that the option will be deleted once processing is complete.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of processing a change to an option. Possible values: <ul> <li> <code>RequiresIndexDocuments</code>: the option's latest value will not be deployed until <a>IndexDocuments</a> has been called and indexing is complete.</li> <li> <code>Processing</code>: the option's latest value is in the process of being activated. </li> <li> <code>Active</code>: the option's latest value is completely deployed.</li> <li> <code>FailedToValidate</code>: the option value is not compatible with the domain's data and cannot be used to index the data. You must either modify the option value or update or remove the incompatible documents.</li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateVersion</i> A unique integer that indicates when this option was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateDate</i> A timestamp for when this option was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> A timestamp for when this option was created.<br/>        """
        pass

    def delete_domain(self, DomainName: str) -> DeleteDomainResponse:
        """Permanently deletes a search domain and all of its data. Once a domain has been deleted, it cannot be recovered. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/deleting-domains.html" target="_blank">Deleting a Search Domain</a> in the <i>Amazon CloudSearch Developer Guide</i>. 

        :param self:
        :param DomainName:  The name of the domain you want to permanently delete.<br/>
        :return: DomainStatus:  The current status of the search domain.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Limits</i> &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Limits</b>: <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SearchInstanceCount</i> The number of search instances that are available to process search requests.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SearchPartitionCount</i> The number of partitions across which the search index is spread.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SearchInstanceType</i> The instance type that is being used to process search requests.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Processing</i> True if processing is being done to activate the current domain configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequiresIndexDocuments</i> True if <a>IndexDocuments</a> needs to be called to activate the current domain configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SearchService</i> The service endpoint for requesting search results from a search domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SearchService</b>: The service endpoint for requesting search results from a search domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DocService</i> The service endpoint for updating documents in a search domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DocService</b>: The service endpoint for updating documents in a search domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Deleted</i> True if the search domain has been deleted. The system must clean up resources dedicated to the search domain when <a>DeleteDomain</a> is called. Newly deleted search domains are returned from <a>DescribeDomains</a> with a true value for IsDeleted for several minutes until resource cleanup is complete.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Created</i> True if the search domain is created. It can take several minutes to initialize a domain when <a>CreateDomain</a> is called. Newly created search domains are returned from <a>DescribeDomains</a> with a false value for Created until domain creation is complete.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ARN</i> The Amazon Resource Name (ARN) of the search domain. See <a href="http://docs.aws.amazon.com/IAM/latest/UserGuide/index.html?Using_Identifiers.html" target="_blank">Identifiers for IAM Entities</a> in <i>Using AWS Identity and Access Management</i> for more information.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainName</i> A string that represents the name of a domain. Domain names are unique across the domains owned by an account within an AWS region. Domain names start with a letter or number and can contain the following characters: a-z (lowercase), 0-9, and - (hyphen).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainId</i> An internally generated unique identifier for a domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> The endpoint to which service requests can be submitted. For example, <code>search-imdb-movies-oopcnjfn6ugofer3zx5iadxxca.eu-west-1.cloudsearch.amazonaws.com</code> or <code>doc-imdb-movies-oopcnjfn6ugofer3zx5iadxxca.eu-west-1.cloudsearch.amazonaws.com</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> The endpoint to which service requests can be submitted. For example, <code>search-imdb-movies-oopcnjfn6ugofer3zx5iadxxca.eu-west-1.cloudsearch.amazonaws.com</code> or <code>doc-imdb-movies-oopcnjfn6ugofer3zx5iadxxca.eu-west-1.cloudsearch.amazonaws.com</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumPartitionCount</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumReplicationCount</i> <br/>        """
        pass

    def delete_expression(self, ExpressionName: str, DomainName: str) -> DeleteExpressionResponse:
        """Removes an <code><a>Expression</a></code> from the search domain. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/configuring-expressions.html" target="_blank">Configuring Expressions</a> in the <i>Amazon CloudSearch Developer Guide</i>.

        :param self:
        :param ExpressionName:  The name of the <code><a>Expression</a></code> to delete.<br/>DomainName:  A string that represents the name of a domain. Domain names are unique across the domains owned by an account within an AWS region. Domain names start with a letter or number and can contain the following characters: a-z (lowercase), 0-9, and - (hyphen).<br/>
        :return: Expression:  The status of the expression being deleted.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of domain configuration option.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>: The status of domain configuration option.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> The expression that is evaluated for sorting while processing a search request.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>: The expression that is evaluated for sorting while processing a search request.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpressionValue</i> The expression to evaluate for sorting while processing a search request. The <code>Expression</code> syntax is based on JavaScript expressions. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/configuring-expressions.html" target="_blank">Configuring Expressions</a> in the <i>Amazon CloudSearch Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpressionName</i> Names must begin with a letter and can contain the following characters: a-z (lowercase), 0-9, and _ (underscore).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingDeletion</i> Indicates that the option will be deleted once processing is complete.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of processing a change to an option. Possible values: <ul> <li> <code>RequiresIndexDocuments</code>: the option's latest value will not be deployed until <a>IndexDocuments</a> has been called and indexing is complete.</li> <li> <code>Processing</code>: the option's latest value is in the process of being activated. </li> <li> <code>Active</code>: the option's latest value is completely deployed.</li> <li> <code>FailedToValidate</code>: the option value is not compatible with the domain's data and cannot be used to index the data. You must either modify the option value or update or remove the incompatible documents.</li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateVersion</i> A unique integer that indicates when this option was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateDate</i> A timestamp for when this option was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> A timestamp for when this option was created.<br/>        """
        pass

    def delete_index_field(self, IndexFieldName: str, DomainName: str) -> DeleteIndexFieldResponse:
        """Removes an <code><a>IndexField</a></code> from the search domain. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/configuring-index-fields.html" target="_blank">Configuring Index Fields</a> in the <i>Amazon CloudSearch Developer Guide</i>.

        :param self:
        :param IndexFieldName:  The name of the index field your want to remove from the domain's indexing options.<br/>DomainName:  A string that represents the name of a domain. Domain names are unique across the domains owned by an account within an AWS region. Domain names start with a letter or number and can contain the following characters: a-z (lowercase), 0-9, and - (hyphen).<br/>
        :return: IndexField:  The status of the index field being deleted.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of domain configuration option.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>: The status of domain configuration option.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> Configuration information for a field in the index, including its name, type, and options. The supported options depend on the <code><a>IndexFieldType</a></code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>: Configuration information for a field in the index, including its name, type, and options. The supported options depend on the <code><a>IndexFieldType</a></code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IndexFieldType</i> The type of field. The valid options for a field depend on the field type. For more information about the supported field types, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/configuring-index-fields.html" target="_blank">Configuring Index Fields</a> in the <i>Amazon CloudSearch Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IndexFieldName</i> A string that represents the name of an index field. CloudSearch supports regular index fields as well as dynamic fields. A dynamic field's name defines a pattern that begins or ends with a wildcard. Any document fields that don't map to a regular index field but do match a dynamic field's pattern are configured with the dynamic field's indexing options.  Regular field names begin with a letter and can contain the following characters: a-z (lowercase), 0-9, and _ (underscore). Dynamic field names must begin or end with a wildcard (*). The wildcard can also be the only character in a dynamic field name. Multiple wildcards, and wildcards embedded within a string are not supported.  The name <code>score</code> is reserved and cannot be used as a field name. To reference a document's ID, you can use the name <code>_id</code>. <br/>Options for a 64-bit signed integer field. Present if <code>IndexFieldType</code> specifies the field is of type <code>int</code>. All options are enabled by default.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>IntOptions</b>: Options for a 64-bit signed integer field. Present if <code>IndexFieldType</code> specifies the field is of type <code>int</code>. All options are enabled by default.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingDeletion</i> Indicates that the option will be deleted once processing is complete.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of processing a change to an option. Possible values: <ul> <li> <code>RequiresIndexDocuments</code>: the option's latest value will not be deployed until <a>IndexDocuments</a> has been called and indexing is complete.</li> <li> <code>Processing</code>: the option's latest value is in the process of being activated. </li> <li> <code>Active</code>: the option's latest value is completely deployed.</li> <li> <code>FailedToValidate</code>: the option value is not compatible with the domain's data and cannot be used to index the data. You must either modify the option value or update or remove the incompatible documents.</li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateVersion</i> A unique integer that indicates when this option was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateDate</i> A timestamp for when this option was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> A timestamp for when this option was created.<br/>        """
        pass

    def delete_suggester(self, SuggesterName: str, DomainName: str) -> DeleteSuggesterResponse:
        """Deletes a suggester. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/getting-suggestions.html" target="_blank">Getting Search Suggestions</a> in the <i>Amazon CloudSearch Developer Guide</i>.

        :param self:
        :param SuggesterName:  Specifies the name of the suggester you want to delete.<br/>DomainName:  A string that represents the name of a domain. Domain names are unique across the domains owned by an account within an AWS region. Domain names start with a letter or number and can contain the following characters: a-z (lowercase), 0-9, and - (hyphen).<br/>
        :return: Suggester:  The status of the suggester being deleted.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of domain configuration option.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>: The status of domain configuration option.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> Configuration information for a search suggester. Each suggester has a unique name and specifies the text field you want to use for suggestions. The following options can be configured for a suggester: <code>FuzzyMatching</code>, <code>SortExpression</code>. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>: Configuration information for a search suggester. Each suggester has a unique name and specifies the text field you want to use for suggestions. The following options can be configured for a suggester: <code>FuzzyMatching</code>, <code>SortExpression</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SuggesterName</i> Names must begin with a letter and can contain the following characters: a-z (lowercase), 0-9, and _ (underscore).<br/>Options for a search suggester.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DocumentSuggesterOptions</b>: Options for a search suggester.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingDeletion</i> Indicates that the option will be deleted once processing is complete.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of processing a change to an option. Possible values: <ul> <li> <code>RequiresIndexDocuments</code>: the option's latest value will not be deployed until <a>IndexDocuments</a> has been called and indexing is complete.</li> <li> <code>Processing</code>: the option's latest value is in the process of being activated. </li> <li> <code>Active</code>: the option's latest value is completely deployed.</li> <li> <code>FailedToValidate</code>: the option value is not compatible with the domain's data and cannot be used to index the data. You must either modify the option value or update or remove the incompatible documents.</li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateVersion</i> A unique integer that indicates when this option was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateDate</i> A timestamp for when this option was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> A timestamp for when this option was created.<br/>        """
        pass

    def describe_analysis_schemes(self, DomainName: str, AnalysisSchemeNames: List[StandardName]=None, Deployed: Boolean=None) -> DescribeAnalysisSchemesResponse:
        """Gets the analysis schemes configured for a domain. An analysis scheme defines language-specific text processing options for a <code>text</code> field. Can be limited to specific analysis schemes by name. By default, shows all analysis schemes and includes any pending changes to the configuration. Set the <code>Deployed</code> option to <code>true</code> to show the active configuration and exclude pending changes. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/configuring-analysis-schemes.html" target="_blank">Configuring Analysis Schemes</a> in the <i>Amazon CloudSearch Developer Guide</i>.

        :param self:
        :param Deployed:  Whether to display the deployed configuration (<code>true</code>) or include any pending changes (<code>false</code>). Defaults to <code>false</code>.<br/>AnalysisSchemeNames:  The analysis schemes you want to describe.<br/>DomainName:  The name of the domain you want to describe.<br/>
        :return: AnalysisSchemes:  The analysis scheme descriptions.<br/>        """
        pass

    def describe_availability_options(self, DomainName: str, Deployed: Boolean=None) -> DescribeAvailabilityOptionsResponse:
        """Gets the availability options configured for a domain. By default, shows the configuration with any pending changes. Set the <code>Deployed</code> option to <code>true</code> to show the active configuration and exclude pending changes. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/configuring-availability-options.html" target="_blank">Configuring Availability Options</a> in the <i>Amazon CloudSearch Developer Guide</i>.

        :param self:
        :param Deployed:  Whether to display the deployed configuration (<code>true</code>) or include any pending changes (<code>false</code>). Defaults to <code>false</code>.<br/>DomainName:  The name of the domain you want to describe.<br/>
        :return: AvailabilityOptions:  The availability options configured for the domain. Indicates whether Multi-AZ is enabled for the domain. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of domain configuration option.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>: The status of domain configuration option.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> The availability options configured for the domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingDeletion</i> Indicates that the option will be deleted once processing is complete.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of processing a change to an option. Possible values: <ul> <li> <code>RequiresIndexDocuments</code>: the option's latest value will not be deployed until <a>IndexDocuments</a> has been called and indexing is complete.</li> <li> <code>Processing</code>: the option's latest value is in the process of being activated. </li> <li> <code>Active</code>: the option's latest value is completely deployed.</li> <li> <code>FailedToValidate</code>: the option value is not compatible with the domain's data and cannot be used to index the data. You must either modify the option value or update or remove the incompatible documents.</li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateVersion</i> A unique integer that indicates when this option was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateDate</i> A timestamp for when this option was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> A timestamp for when this option was created.<br/>        """
        pass

    def describe_domains(self, DomainNames: List[DomainName]=None) -> DescribeDomainsResponse:
        """Gets information about the search domains owned by this account. Can be limited to specific domains. Shows all domains by default. To get the number of searchable documents in a domain, use the console or submit a <code>matchall</code> request to your domain's search endpoint: <code>q=matchall&amp;amp;q.parser=structured&amp;amp;size=0</code>. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/getting-domain-info.html" target="_blank">Getting Information about a Search Domain</a> in the <i>Amazon CloudSearch Developer Guide</i>.

        :param self:
        :param DomainNames:  The names of the domains you want to include in the response.<br/>
        :return: DomainStatusList:  A list that contains the status of each requested domain.<br/>        """
        pass

    def describe_expressions(self, DomainName: str, ExpressionNames: List[StandardName]=None, Deployed: Boolean=None) -> DescribeExpressionsResponse:
        """Gets the expressions configured for the search domain. Can be limited to specific expressions by name. By default, shows all expressions and includes any pending changes to the configuration. Set the <code>Deployed</code> option to <code>true</code> to show the active configuration and exclude pending changes. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/configuring-expressions.html" target="_blank">Configuring Expressions</a> in the <i>Amazon CloudSearch Developer Guide</i>.

        :param self:
        :param Deployed:  Whether to display the deployed configuration (<code>true</code>) or include any pending changes (<code>false</code>). Defaults to <code>false</code>.<br/>ExpressionNames:  Limits the <code><a>DescribeExpressions</a></code> response to the specified expressions. If not specified, all expressions are shown.<br/>DomainName:  The name of the domain you want to describe.<br/>
        :return: Expressions:  The expressions configured for the domain.<br/>        """
        pass

    def describe_index_fields(self, DomainName: str, FieldNames: List[DynamicFieldName]=None, Deployed: Boolean=None) -> DescribeIndexFieldsResponse:
        """Gets information about the index fields configured for the search domain. Can be limited to specific fields by name. By default, shows all fields and includes any pending changes to the configuration. Set the <code>Deployed</code> option to <code>true</code> to show the active configuration and exclude pending changes. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/getting-domain-info.html" target="_blank">Getting Domain Information</a> in the <i>Amazon CloudSearch Developer Guide</i>.

        :param self:
        :param Deployed:  Whether to display the deployed configuration (<code>true</code>) or include any pending changes (<code>false</code>). Defaults to <code>false</code>.<br/>FieldNames:  A list of the index fields you want to describe. If not specified, information is returned for all configured index fields.<br/>DomainName:  The name of the domain you want to describe.<br/>
        :return: IndexFields:  The index fields configured for the domain.<br/>        """
        pass

    def describe_scaling_parameters(self, DomainName: str) -> DescribeScalingParametersResponse:
        """Gets the scaling parameters configured for a domain. A domain's scaling parameters specify the desired search instance type and replication count. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/configuring-scaling-options.html" target="_blank">Configuring Scaling Options</a> in the <i>Amazon CloudSearch Developer Guide</i>.

        :param self:
        :param DomainName:  A string that represents the name of a domain. Domain names are unique across the domains owned by an account within an AWS region. Domain names start with a letter or number and can contain the following characters: a-z (lowercase), 0-9, and - (hyphen).<br/>
        :return: ScalingParameters:  The status and configuration of a search domain's scaling parameters. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of domain configuration option.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>: The status of domain configuration option.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> The desired instance type and desired number of replicas of each index partition.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>: The desired instance type and desired number of replicas of each index partition.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DesiredPartitionCount</i> The number of partitions you want to preconfigure for your domain. Only valid when you select <code>m2.2xlarge</code> as the desired instance type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DesiredReplicationCount</i> The number of replicas you want to preconfigure for each index partition.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DesiredInstanceType</i> The instance type that you want to preconfigure for your domain. For example, <code>search.m1.small</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingDeletion</i> Indicates that the option will be deleted once processing is complete.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of processing a change to an option. Possible values: <ul> <li> <code>RequiresIndexDocuments</code>: the option's latest value will not be deployed until <a>IndexDocuments</a> has been called and indexing is complete.</li> <li> <code>Processing</code>: the option's latest value is in the process of being activated. </li> <li> <code>Active</code>: the option's latest value is completely deployed.</li> <li> <code>FailedToValidate</code>: the option value is not compatible with the domain's data and cannot be used to index the data. You must either modify the option value or update or remove the incompatible documents.</li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateVersion</i> A unique integer that indicates when this option was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateDate</i> A timestamp for when this option was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> A timestamp for when this option was created.<br/>        """
        pass

    def describe_service_access_policies(self, DomainName: str, Deployed: Boolean=None) -> DescribeServiceAccessPoliciesResponse:
        """Gets information about the access policies that control access to the domain's document and search endpoints. By default, shows the configuration with any pending changes. Set the <code>Deployed</code> option to <code>true</code> to show the active configuration and exclude pending changes. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/configuring-access.html" target="_blank">Configuring Access for a Search Domain</a> in the <i>Amazon CloudSearch Developer Guide</i>.

        :param self:
        :param Deployed:  Whether to display the deployed configuration (<code>true</code>) or include any pending changes (<code>false</code>). Defaults to <code>false</code>.<br/>DomainName:  The name of the domain you want to describe.<br/>
        :return: AccessPolicies:  The access rules configured for the domain specified in the request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of domain configuration option.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>: The status of domain configuration option.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> Access rules for a domain's document or search service endpoints. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/configuring-access.html" target="_blank">Configuring Access for a Search Domain</a> in the <i>Amazon CloudSearch Developer Guide</i>. The maximum size of a policy document is 100 KB.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingDeletion</i> Indicates that the option will be deleted once processing is complete.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of processing a change to an option. Possible values: <ul> <li> <code>RequiresIndexDocuments</code>: the option's latest value will not be deployed until <a>IndexDocuments</a> has been called and indexing is complete.</li> <li> <code>Processing</code>: the option's latest value is in the process of being activated. </li> <li> <code>Active</code>: the option's latest value is completely deployed.</li> <li> <code>FailedToValidate</code>: the option value is not compatible with the domain's data and cannot be used to index the data. You must either modify the option value or update or remove the incompatible documents.</li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateVersion</i> A unique integer that indicates when this option was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateDate</i> A timestamp for when this option was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> A timestamp for when this option was created.<br/>        """
        pass

    def describe_suggesters(self, DomainName: str, SuggesterNames: List[StandardName]=None, Deployed: Boolean=None) -> DescribeSuggestersResponse:
        """Gets the suggesters configured for a domain. A suggester enables you to display possible matches before users finish typing their queries. Can be limited to specific suggesters by name. By default, shows all suggesters and includes any pending changes to the configuration. Set the <code>Deployed</code> option to <code>true</code> to show the active configuration and exclude pending changes. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/getting-suggestions.html" target="_blank">Getting Search Suggestions</a> in the <i>Amazon CloudSearch Developer Guide</i>.

        :param self:
        :param Deployed:  Whether to display the deployed configuration (<code>true</code>) or include any pending changes (<code>false</code>). Defaults to <code>false</code>.<br/>SuggesterNames:  The suggesters you want to describe.<br/>DomainName:  The name of the domain you want to describe.<br/>
        :return: Suggesters:  The suggesters configured for the domain specified in the request.<br/>        """
        pass

    def index_documents(self, DomainName: str) -> IndexDocumentsResponse:
        """Tells the search domain to start indexing its documents using the latest indexing options. This operation must be invoked to activate options whose <a>OptionStatus</a> is <code>RequiresIndexDocuments</code>.

        :param self:
        :param DomainName:  A string that represents the name of a domain. Domain names are unique across the domains owned by an account within an AWS region. Domain names start with a letter or number and can contain the following characters: a-z (lowercase), 0-9, and - (hyphen).<br/>
        :return: FieldNames:  The names of the fields that are currently being indexed.<br/>        """
        pass

    def list_domain_names(self) -> ListDomainNamesResponse:
        """Lists all search domains owned by an account.

        :param self:
        :param 
        :return: DomainNames:  The names of the search domains owned by an account.<br/>        """
        pass

    def update_availability_options(self, MultiAZ: Boolean, DomainName: str) -> UpdateAvailabilityOptionsResponse:
        """Configures the availability options for a domain. Enabling the Multi-AZ option expands an Amazon CloudSearch domain to an additional Availability Zone in the same Region to increase fault tolerance in the event of a service disruption. Changes to the Multi-AZ option can take about half an hour to become active. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/configuring-availability-options.html" target="_blank">Configuring Availability Options</a> in the <i>Amazon CloudSearch Developer Guide</i>.

        :param self:
        :param MultiAZ:  You expand an existing search domain to a second Availability Zone by setting the Multi-AZ option to true. Similarly, you can turn off the Multi-AZ option to downgrade the domain to a single Availability Zone by setting the Multi-AZ option to <code>false</code>. <br/>DomainName:  A string that represents the name of a domain. Domain names are unique across the domains owned by an account within an AWS region. Domain names start with a letter or number and can contain the following characters: a-z (lowercase), 0-9, and - (hyphen).<br/>
        :return: AvailabilityOptions:  The newly-configured availability options. Indicates whether Multi-AZ is enabled for the domain. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of domain configuration option.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>: The status of domain configuration option.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> The availability options configured for the domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingDeletion</i> Indicates that the option will be deleted once processing is complete.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of processing a change to an option. Possible values: <ul> <li> <code>RequiresIndexDocuments</code>: the option's latest value will not be deployed until <a>IndexDocuments</a> has been called and indexing is complete.</li> <li> <code>Processing</code>: the option's latest value is in the process of being activated. </li> <li> <code>Active</code>: the option's latest value is completely deployed.</li> <li> <code>FailedToValidate</code>: the option value is not compatible with the domain's data and cannot be used to index the data. You must either modify the option value or update or remove the incompatible documents.</li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateVersion</i> A unique integer that indicates when this option was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateDate</i> A timestamp for when this option was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> A timestamp for when this option was created.<br/>        """
        pass

    def update_scaling_parameters(self, ScalingParameters: ScalingParameters, DomainName: str) -> UpdateScalingParametersResponse:
        """Configures scaling parameters for a domain. A domain's scaling parameters specify the desired search instance type and replication count. Amazon CloudSearch will still automatically scale your domain based on the volume of data and traffic, but not below the desired instance type and replication count. If the Multi-AZ option is enabled, these values control the resources used per Availability Zone. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/configuring-scaling-options.html" target="_blank">Configuring Scaling Options</a> in the <i>Amazon CloudSearch Developer Guide</i>. 

        :param self:
        :param ScalingParameters:  The desired instance type and desired number of replicas of each index partition.:<br/>DomainName:  A string that represents the name of a domain. Domain names are unique across the domains owned by an account within an AWS region. Domain names start with a letter or number and can contain the following characters: a-z (lowercase), 0-9, and - (hyphen).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DesiredPartitionCount</i> The number of partitions you want to preconfigure for your domain. Only valid when you select <code>m2.2xlarge</code> as the desired instance type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DesiredReplicationCount</i> The number of replicas you want to preconfigure for each index partition.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DesiredInstanceType</i> The instance type that you want to preconfigure for your domain. For example, <code>search.m1.small</code>.<br/>
        :return: ScalingParameters:  The status and configuration of a search domain's scaling parameters. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of domain configuration option.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>: The status of domain configuration option.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> The desired instance type and desired number of replicas of each index partition.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>: The desired instance type and desired number of replicas of each index partition.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DesiredPartitionCount</i> The number of partitions you want to preconfigure for your domain. Only valid when you select <code>m2.2xlarge</code> as the desired instance type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DesiredReplicationCount</i> The number of replicas you want to preconfigure for each index partition.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DesiredInstanceType</i> The instance type that you want to preconfigure for your domain. For example, <code>search.m1.small</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingDeletion</i> Indicates that the option will be deleted once processing is complete.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of processing a change to an option. Possible values: <ul> <li> <code>RequiresIndexDocuments</code>: the option's latest value will not be deployed until <a>IndexDocuments</a> has been called and indexing is complete.</li> <li> <code>Processing</code>: the option's latest value is in the process of being activated. </li> <li> <code>Active</code>: the option's latest value is completely deployed.</li> <li> <code>FailedToValidate</code>: the option value is not compatible with the domain's data and cannot be used to index the data. You must either modify the option value or update or remove the incompatible documents.</li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateVersion</i> A unique integer that indicates when this option was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateDate</i> A timestamp for when this option was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> A timestamp for when this option was created.<br/>        """
        pass

    def update_service_access_policies(self, AccessPolicies: str, DomainName: str) -> UpdateServiceAccessPoliciesResponse:
        """Configures the access rules that control access to the domain's document and search endpoints. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/configuring-access.html" target="_blank"> Configuring Access for an Amazon CloudSearch Domain</a>.

        :param self:
        :param AccessPolicies:  The access rules you want to configure. These rules replace any existing rules. <br/>DomainName:  A string that represents the name of a domain. Domain names are unique across the domains owned by an account within an AWS region. Domain names start with a letter or number and can contain the following characters: a-z (lowercase), 0-9, and - (hyphen).<br/>
        :return: AccessPolicies:  The access rules configured for the domain.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of domain configuration option.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>: The status of domain configuration option.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> Access rules for a domain's document or search service endpoints. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/configuring-access.html" target="_blank">Configuring Access for a Search Domain</a> in the <i>Amazon CloudSearch Developer Guide</i>. The maximum size of a policy document is 100 KB.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingDeletion</i> Indicates that the option will be deleted once processing is complete.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of processing a change to an option. Possible values: <ul> <li> <code>RequiresIndexDocuments</code>: the option's latest value will not be deployed until <a>IndexDocuments</a> has been called and indexing is complete.</li> <li> <code>Processing</code>: the option's latest value is in the process of being activated. </li> <li> <code>Active</code>: the option's latest value is completely deployed.</li> <li> <code>FailedToValidate</code>: the option value is not compatible with the domain's data and cannot be used to index the data. You must either modify the option value or update or remove the incompatible documents.</li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateVersion</i> A unique integer that indicates when this option was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateDate</i> A timestamp for when this option was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> A timestamp for when this option was created.<br/>        """
        pass



class CloudSearchDomain(BaseClient):
    """You use the AmazonCloudSearch2013 API to upload documents to a search domain and search those documents.  The endpoints for submitting <code>UploadDocuments</code>, <code>Search</code>, and <code>Suggest</code> requests are domain-specific. To get the endpoints for your domain, use the Amazon CloudSearch configuration service <code>DescribeDomains</code> action. The domain endpoints are also displayed on the domain dashboard in the Amazon CloudSearch console. You submit suggest requests to the search endpoint.  For more information, see the <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide">Amazon CloudSearch Developer Guide</a>."""



    class SearchRequest(Mapping):
        pass
    
    class SearchResponse(Mapping):
        pass
    
    class Partial(object):
        pass
    
    class Size(object):
        pass
    
    class Start(object):
        pass
    
    class SuggestRequest(Mapping):
        pass
    
    class SuggestResponse(Mapping):
        pass
    
    class SuggestionsSize(object):
        pass
    
    class UploadDocumentsRequest(Mapping):
        pass
    
    class UploadDocumentsResponse(Mapping):
        pass
    
    class Blob(object):
        pass
    
    def search(self, query: str, cursor: str=None, expr: str=None, facet: str=None, filterQuery: str=None, highlight: str=None, partial: Partial=None, queryOptions: str=None, queryParser: str=None, return_: str=None, size: Size=None, sort: str=None, start: Start=None, stats: str=None) -> SearchResponse:
        """Retrieves a list of documents that match the specified search criteria. How you specify the search criteria depends on which query parser you use. Amazon CloudSearch supports four query parsers: <ul> <li><code>simple</code>: search all <code>text</code> and <code>text-array</code> fields for the specified string. Search for phrases, individual terms, and prefixes. </li> <li><code>structured</code>: search specific fields, construct compound queries using Boolean operators, and use advanced features such as term boosting and proximity searching.</li> <li><code>lucene</code>: specify search criteria using the Apache Lucene query parser syntax.</li> <li><code>dismax</code>: specify search criteria using the simplified subset of the Apache Lucene query parser syntax defined by the DisMax query parser.</li> </ul> For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/searching.html">Searching Your Data</a> in the <i>Amazon CloudSearch Developer Guide</i>. The endpoint for submitting <code>Search</code> requests is domain-specific. You submit search requests to a domain's search endpoint. To get the search endpoint for your domain, use the Amazon CloudSearch configuration service <code>DescribeDomains</code> action. A domain's endpoints are also displayed on the domain dashboard in the Amazon CloudSearch console. 

        :param self:
        :param stats:  Specifies one or more fields for which to get statistics information. Each specified field must be facet-enabled in the domain configuration. The fields are specified in JSON using the form: <code>{"FIELD-A":{},"FIELD-B":{}}</code> There are currently no options supported for statistics.<br/>start:  Specifies the offset of the first search hit you want to return. Note that the result set is zero-based; the first result is at index 0. You can specify either the <code>start</code> or <code>cursor</code> parameter in a request, they are mutually exclusive.  For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/paginating-results.html">Paginating Results</a> in the <i>Amazon CloudSearch Developer Guide</i>.<br/>sort:  Specifies the fields or custom expressions to use to sort the search results. Multiple fields or expressions are specified as a comma-separated list. You must specify the sort direction (<code>asc</code> or <code>desc</code>) for each field; for example, <code>year desc,title asc</code>. To use a field to sort results, the field must be sort-enabled in the domain configuration. Array type fields cannot be used for sorting. If no <code>sort</code> parameter is specified, results are sorted by their default relevance scores in descending order: <code>_score desc</code>. You can also sort by document ID (<code>_id asc</code>) and version (<code>_version desc</code>). For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/sorting-results.html">Sorting Results</a> in the <i>Amazon CloudSearch Developer Guide</i>.<br/>size:  Specifies the maximum number of search hits to include in the response. <br/>return:  Specifies the field and expression values to include in the response. Multiple fields or expressions are specified as a comma-separated list. By default, a search response includes all return enabled fields (<code>_all_fields</code>). To return only the document IDs for the matching documents, specify <code>_no_fields</code>. To retrieve the relevance score calculated for each document, specify <code>_score</code>. <br/>queryParser:  Specifies which query parser to use to process the request. If <code>queryParser</code> is not specified, Amazon CloudSearch uses the <code>simple</code> query parser.  Amazon CloudSearch supports four query parsers: <ul> <li> <code>simple</code>: perform simple searches of <code>text</code> and <code>text-array</code> fields. By default, the <code>simple</code> query parser searches all <code>text</code> and <code>text-array</code> fields. You can specify which fields to search by with the <code>queryOptions</code> parameter. If you prefix a search term with a plus sign (+) documents must contain the term to be considered a match. (This is the default, unless you configure the default operator with the <code>queryOptions</code> parameter.) You can use the <code>-</code> (NOT), <code>|</code> (OR), and <code>*</code> (wildcard) operators to exclude particular terms, find results that match any of the specified terms, or search for a prefix. To search for a phrase rather than individual terms, enclose the phrase in double quotes. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/searching-text.html">Searching for Text</a> in the <i>Amazon CloudSearch Developer Guide</i>. </li> <li> <code>structured</code>: perform advanced searches by combining multiple expressions to define the search criteria. You can also search within particular fields, search for values and ranges of values, and use advanced options such as term boosting, <code>matchall</code>, and <code>near</code>. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/searching-compound-queries.html">Constructing Compound Queries</a> in the <i>Amazon CloudSearch Developer Guide</i>. </li> <li> <code>lucene</code>: search using the Apache Lucene query parser syntax. For more information, see <a href="http://lucene.apache.org/core/4_6_0/queryparser/org/apache/lucene/queryparser/classic/package-summary.html#package_description">Apache Lucene Query Parser Syntax</a>. </li> <li> <code>dismax</code>: search using the simplified subset of the Apache Lucene query parser syntax defined by the DisMax query parser. For more information, see <a href="http://wiki.apache.org/solr/DisMaxQParserPlugin#Query_Syntax">DisMax Query Parser Syntax</a>. </li> </ul><br/>queryOptions:  Configures options for the query parser specified in the <code>queryParser</code> parameter. You specify the options in JSON using the following form <code>{"OPTION1":"VALUE1","OPTION2":VALUE2"..."OPTIONN":"VALUEN"}.</code> The options you can configure vary according to which parser you use: <ul> <li><code>defaultOperator</code>: The default operator used to combine individual terms in the search string. For example: <code>defaultOperator: 'or'</code>. For the <code>dismax</code> parser, you specify a percentage that represents the percentage of terms in the search string (rounded down) that must match, rather than a default operator. A value of <code>0%</code> is the equivalent to OR, and a value of <code>100%</code> is equivalent to AND. The percentage must be specified as a value in the range 0-100 followed by the percent (%) symbol. For example, <code>defaultOperator: 50%</code>. Valid values: <code>and</code>, <code>or</code>, a percentage in the range 0%-100% (<code>dismax</code>). Default: <code>and</code> (<code>simple</code>, <code>structured</code>, <code>lucene</code>) or <code>100</code> (<code>dismax</code>). Valid for: <code>simple</code>, <code>structured</code>, <code>lucene</code>, and <code>dismax</code>.</li> <li><code>fields</code>: An array of the fields to search when no fields are specified in a search. If no fields are specified in a search and this option is not specified, all text and text-array fields are searched. You can specify a weight for each field to control the relative importance of each field when Amazon CloudSearch calculates relevance scores. To specify a field weight, append a caret (<code>^</code>) symbol and the weight to the field name. For example, to boost the importance of the <code>title</code> field over the <code>description</code> field you could specify: <code>"fields":["title^5","description"]</code>. Valid values: The name of any configured field and an optional numeric value greater than zero. Default: All <code>text</code> and <code>text-array</code> fields. Valid for: <code>simple</code>, <code>structured</code>, <code>lucene</code>, and <code>dismax</code>.</li> <li><code>operators</code>: An array of the operators or special characters you want to disable for the simple query parser. If you disable the <code>and</code>, <code>or</code>, or <code>not</code> operators, the corresponding operators (<code>+</code>, <code>|</code>, <code>-</code>) have no special meaning and are dropped from the search string. Similarly, disabling <code>prefix</code> disables the wildcard operator (<code>*</code>) and disabling <code>phrase</code> disables the ability to search for phrases by enclosing phrases in double quotes. Disabling precedence disables the ability to control order of precedence using parentheses. Disabling <code>near</code> disables the ability to use the ~ operator to perform a sloppy phrase search. Disabling the <code>fuzzy</code> operator disables the ability to use the ~ operator to perform a fuzzy search. <code>escape</code> disables the ability to use a backslash (<code>\</code>) to escape special characters within the search string. Disabling whitespace is an advanced option that prevents the parser from tokenizing on whitespace, which can be useful for Vietnamese. (It prevents Vietnamese words from being split incorrectly.) For example, you could disable all operators other than the phrase operator to support just simple term and phrase queries: <code>"operators":["and","not","or", "prefix"]</code>. Valid values: <code>and</code>, <code>escape</code>, <code>fuzzy</code>, <code>near</code>, <code>not</code>, <code>or</code>, <code>phrase</code>, <code>precedence</code>, <code>prefix</code>, <code>whitespace</code>. Default: All operators and special characters are enabled. Valid for: <code>simple</code>.</li> <li><code>phraseFields</code>: An array of the <code>text</code> or <code>text-array</code> fields you want to use for phrase searches. When the terms in the search string appear in close proximity within a field, the field scores higher. You can specify a weight for each field to boost that score. The <code>phraseSlop</code> option controls how much the matches can deviate from the search string and still be boosted. To specify a field weight, append a caret (<code>^</code>) symbol and the weight to the field name. For example, to boost phrase matches in the <code>title</code> field over the <code>abstract</code> field, you could specify: <code>"phraseFields":["title^3", "plot"]</code> Valid values: The name of any <code>text</code> or <code>text-array</code> field and an optional numeric value greater than zero. Default: No fields. If you don't specify any fields with <code>phraseFields</code>, proximity scoring is disabled even if <code>phraseSlop</code> is specified. Valid for: <code>dismax</code>.</li> <li><code>phraseSlop</code>: An integer value that specifies how much matches can deviate from the search phrase and still be boosted according to the weights specified in the <code>phraseFields</code> option; for example, <code>phraseSlop: 2</code>. You must also specify <code>phraseFields</code> to enable proximity scoring. Valid values: positive integers. Default: 0. Valid for: <code>dismax</code>.</li> <li><code>explicitPhraseSlop</code>: An integer value that specifies how much a match can deviate from the search phrase when the phrase is enclosed in double quotes in the search string. (Phrases that exceed this proximity distance are not considered a match.) For example, to specify a slop of three for dismax phrase queries, you would specify <code>"explicitPhraseSlop":3</code>. Valid values: positive integers. Default: 0. Valid for: <code>dismax</code>.</li> <li><code>tieBreaker</code>: When a term in the search string is found in a document's field, a score is calculated for that field based on how common the word is in that field compared to other documents. If the term occurs in multiple fields within a document, by default only the highest scoring field contributes to the document's overall score. You can specify a <code>tieBreaker</code> value to enable the matches in lower-scoring fields to contribute to the document's score. That way, if two documents have the same max field score for a particular term, the score for the document that has matches in more fields will be higher. The formula for calculating the score with a tieBreaker is <code>(max field score) + (tieBreaker) * (sum of the scores for the rest of the matching fields)</code>. Set <code>tieBreaker</code> to 0 to disregard all but the highest scoring field (pure max): <code>"tieBreaker":0</code>. Set to 1 to sum the scores from all fields (pure sum): <code>"tieBreaker":1</code>. Valid values: 0.0 to 1.0. Default: 0.0. Valid for: <code>dismax</code>. </li> </ul><br/>query:  Specifies the search criteria for the request. How you specify the search criteria depends on the query parser used for the request and the parser options specified in the <code>queryOptions</code> parameter. By default, the <code>simple</code> query parser is used to process requests. To use the <code>structured</code>, <code>lucene</code>, or <code>dismax</code> query parser, you must also specify the <code>queryParser</code> parameter.  For more information about specifying search criteria, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/searching.html">Searching Your Data</a> in the <i>Amazon CloudSearch Developer Guide</i>.<br/>partial:  Enables partial results to be returned if one or more index partitions are unavailable. When your search index is partitioned across multiple search instances, by default Amazon CloudSearch only returns results if every partition can be queried. This means that the failure of a single search instance can result in 5xx (internal server) errors. When you enable partial results, Amazon CloudSearch returns whatever results are available and includes the percentage of documents searched in the search results (percent-searched). This enables you to more gracefully degrade your users' search experience. For example, rather than displaying no results, you could display the partial results and a message indicating that the results might be incomplete due to a temporary system outage.<br/>highlight:  Retrieves highlights for matches in the specified <code>text</code> or <code>text-array</code> fields. Each specified field must be highlight enabled in the domain configuration. The fields and options are specified in JSON using the form <code>{"FIELD":{"OPTION":VALUE,"OPTION:"STRING"},"FIELD":{"OPTION":VALUE,"OPTION":"STRING"}}</code>. You can specify the following highlight options: <ul> <li> <code>format</code>: specifies the format of the data in the text field: <code>text</code> or <code>html</code>. When data is returned as HTML, all non-alphanumeric characters are encoded. The default is <code>html</code>. </li> <li> <code>max_phrases</code>: specifies the maximum number of occurrences of the search term(s) you want to highlight. By default, the first occurrence is highlighted. </li> <li> <code>pre_tag</code>: specifies the string to prepend to an occurrence of a search term. The default for HTML highlights is <code>&amp;lt;em&amp;gt;</code>. The default for text highlights is <code>*</code>. </li> <li> <code>post_tag</code>: specifies the string to append to an occurrence of a search term. The default for HTML highlights is <code>&amp;lt;/em&amp;gt;</code>. The default for text highlights is <code>*</code>. </li> </ul> If no highlight options are specified for a field, the returned field text is treated as HTML and the first match is highlighted with emphasis tags: <code>&amp;lt;em&gt;search-term&amp;lt;/em&amp;gt;</code>. For example, the following request retrieves highlights for the <code>actors</code> and <code>title</code> fields.  <code>{ "actors": {}, "title": {"format": "text","max_phrases": 2,"pre_tag": "<b>","post_tag": "</b>"} }</code><br/>filterQuery:  Specifies a structured query that filters the results of a search without affecting how the results are scored and sorted. You use <code>filterQuery</code> in conjunction with the <code>query</code> parameter to filter the documents that match the constraints specified in the <code>query</code> parameter. Specifying a filter controls only which matching documents are included in the results, it has no effect on how they are scored and sorted. The <code>filterQuery</code> parameter supports the full structured query syntax.  For more information about using filters, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/filtering-results.html">Filtering Matching Documents</a> in the <i>Amazon CloudSearch Developer Guide</i>.<br/>facet:  Specifies one or more fields for which to get facet information, and options that control how the facet information is returned. Each specified field must be facet-enabled in the domain configuration. The fields and options are specified in JSON using the form <code>{"FIELD":{"OPTION":VALUE,"OPTION:"STRING"},"FIELD":{"OPTION":VALUE,"OPTION":"STRING"}}</code>. You can specify the following faceting options: <ul> <li> <code>buckets</code> specifies an array of the facet values or ranges to count. Ranges are specified using the same syntax that you use to search for a range of values. For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/searching-ranges.html"> Searching for a Range of Values</a> in the <i>Amazon CloudSearch Developer Guide</i>. Buckets are returned in the order they are specified in the request. The <code>sort</code> and <code>size</code> options are not valid if you specify <code>buckets</code>. </li> <li> <code>size</code> specifies the maximum number of facets to include in the results. By default, Amazon CloudSearch returns counts for the top 10. The <code>size</code> parameter is only valid when you specify the <code>sort</code> option; it cannot be used in conjunction with <code>buckets</code>. </li> <li> <code>sort</code> specifies how you want to sort the facets in the results: <code>bucket</code> or <code>count</code>. Specify <code>bucket</code> to sort alphabetically or numerically by facet value (in ascending order). Specify <code>count</code> to sort by the facet counts computed for each facet value (in descending order). To retrieve facet counts for particular values or ranges of values, use the <code>buckets</code> option instead of <code>sort</code>.  </li> </ul> If no facet options are specified, facet counts are computed for all field values, the facets are sorted by facet count, and the top 10 facets are returned in the results. To count particular buckets of values, use the <code>buckets</code> option. For example, the following request uses the <code>buckets</code> option to calculate and return facet counts by decade. <code> {"year":{"buckets":["[1970,1979]","[1980,1989]","[1990,1999]","[2000,2009]","[2010,}"]}} </code> To sort facets by facet count, use the <code>count</code> option. For example, the following request sets the <code>sort</code> option to <code>count</code> to sort the facet values by facet count, with the facet values that have the most matching documents listed first. Setting the <code>size</code> option to 3 returns only the top three facet values. <code> {"year":{"sort":"count","size":3}} </code> To sort the facets by value, use the <code>bucket</code> option. For example, the following request sets the <code>sort</code> option to <code>bucket</code> to sort the facet values numerically by year, with earliest year listed first.  <code> {"year":{"sort":"bucket"}} </code> For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/faceting.html">Getting and Using Facet Information</a> in the <i>Amazon CloudSearch Developer Guide</i>.<br/>expr:  Defines one or more numeric expressions that can be used to sort results or specify search or filter criteria. You can also specify expressions as return fields.  You specify the expressions in JSON using the form <code>{"EXPRESSIONNAME":"EXPRESSION"}</code>. You can define and use multiple expressions in a search request. For example: <code> {"expression1":"_score*rating", "expression2":"(1/rank)*year"} </code>  For information about the variables, operators, and functions you can use in expressions, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/configuring-expressions.html#writing-expressions">Writing Expressions</a> in the <i>Amazon CloudSearch Developer Guide</i>.<br/>cursor:  Retrieves a cursor value you can use to page through large result sets. Use the <code>size</code> parameter to control the number of hits to include in each response. You can specify either the <code>cursor</code> or <code>start</code> parameter in a request; they are mutually exclusive. To get the first cursor, set the cursor value to <code>initial</code>. In subsequent requests, specify the cursor value returned in the hits section of the response.  For more information, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/paginating-results.html">Paginating Results</a> in the <i>Amazon CloudSearch Developer Guide</i>.<br/>
        :return: stats:  The requested field statistics information.<br/>facets:  The requested facet information.<br/>hits:  The documents that match the search criteria.:<br/>status:  The status information returned for the search request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>rid</i> The encrypted resource ID for the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>timems</i> How long it took to process the request, in milliseconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>hit</i> A document that matches the search request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cursor</i> A cursor that can be used to retrieve the next set of matching documents when you want to page through a large result set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>start</i> The index of the first matching document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>found</i> The total number of documents that match the search request.<br/>        """
        pass

    def suggest(self, suggester: str, query: str, size: SuggestionsSize=None) -> SuggestResponse:
        """Retrieves autocomplete suggestions for a partial query string. You can use suggestions enable you to display likely matches before users finish typing. In Amazon CloudSearch, suggestions are based on the contents of a particular text field. When you request suggestions, Amazon CloudSearch finds all of the documents whose values in the suggester field start with the specified query string. The beginning of the field must match the query string to be considered a match.  For more information about configuring suggesters and retrieving suggestions, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/getting-suggestions.html">Getting Suggestions</a> in the <i>Amazon CloudSearch Developer Guide</i>.  The endpoint for submitting <code>Suggest</code> requests is domain-specific. You submit suggest requests to a domain's search endpoint. To get the search endpoint for your domain, use the Amazon CloudSearch configuration service <code>DescribeDomains</code> action. A domain's endpoints are also displayed on the domain dashboard in the Amazon CloudSearch console. 

        :param self:
        :param size:  Specifies the maximum number of suggestions to return. <br/>suggester:  Specifies the name of the suggester to use to find suggested matches.<br/>query:  Specifies the string for which you want to get suggestions.<br/>
        :return: suggest:  Container for the matching search suggestion information.:<br/>status:  The status of a <code>SuggestRequest</code>. Contains the resource ID (<code>rid</code>) and how long it took to process the request (<code>timems</code>).:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>rid</i> The encrypted resource ID for the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>timems</i> How long it took to process the request, in milliseconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>suggestions</i> The documents that match the query string.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>found</i> The number of documents that were found to match the query string.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>query</i> The query string specified in the suggest request.<br/>        """
        pass

    def upload_documents(self, contentType: str, documents: Blob) -> UploadDocumentsResponse:
        """Posts a batch of documents to a search domain for indexing. A document batch is a collection of add and delete operations that represent the documents you want to add, update, or delete from your domain. Batches can be described in either JSON or XML. Each item that you want Amazon CloudSearch to return as a search result (such as a product) is represented as a document. Every document has a unique ID and one or more fields that contain the data that you want to search and return in results. Individual documents cannot contain more than 1 MB of data. The entire batch cannot exceed 5 MB. To get the best possible upload performance, group add and delete operations in batches that are close the 5 MB limit. Submitting a large volume of single-document batches can overload a domain's document service.  The endpoint for submitting <code>UploadDocuments</code> requests is domain-specific. To get the document endpoint for your domain, use the Amazon CloudSearch configuration service <code>DescribeDomains</code> action. A domain's endpoints are also displayed on the domain dashboard in the Amazon CloudSearch console.  For more information about formatting your data for Amazon CloudSearch, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/preparing-data.html">Preparing Your Data</a> in the <i>Amazon CloudSearch Developer Guide</i>. For more information about uploading data for indexing, see <a href="http://docs.aws.amazon.com/cloudsearch/latest/developerguide/uploading-data.html">Uploading Data</a> in the <i>Amazon CloudSearch Developer Guide</i>. 

        :param self:
        :param contentType:  The format of the batch you are uploading. Amazon CloudSearch supports two document batch formats: <ul> <li>application/json</li> <li>application/xml</li> </ul><br/>documents:  A batch of documents formatted in JSON or HTML.<br/>
        :return: warnings:  Any warnings returned by the document service about the documents being uploaded.<br/>deletes:  The number of documents that were deleted from the search domain.<br/>adds:  The number of documents that were added to the search domain.<br/>status:  The status of an <code>UploadDocumentsRequest</code>.<br/>        """
        pass



class CloudTrail(BaseClient):
    """<fullname>AWS CloudTrail</fullname> This is the CloudTrail API Reference. It provides descriptions of actions, data types, common parameters, and common errors for CloudTrail. CloudTrail is a web service that records AWS API calls for your AWS account and delivers log files to an Amazon S3 bucket. The recorded information includes the identity of the user, the start time of the AWS API call, the source IP address, the request parameters, and the response elements returned by the service. <note> As an alternative to the API, you can use one of the AWS SDKs, which consist of libraries and sample code for various programming languages and platforms (Java, Ruby, .NET, iOS, Android, etc.). The SDKs provide a convenient way to create programmatic access to AWSCloudTrail. For example, the SDKs take care of cryptographically signing requests, managing errors, and retrying requests automatically. For information about the AWS SDKs, including how to download and install them, see the <a href="http://aws.amazon.com/tools/">Tools for Amazon Web Services page</a>. </note> See the <a href="http://docs.aws.amazon.com/awscloudtrail/latest/userguide/cloudtrail-user-guide.html">AWS CloudTrail User Guide</a> for information about the data that is included with each AWS API call listed in the log files."""


    class LookupEventsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `lookup_events` for valid parameters"""
            pass
    class ListPublicKeysPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_public_keys` for valid parameters"""
            pass
    class ListTagsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_tags` for valid parameters"""
            pass


    class AddTagsRequest(Mapping):
        pass
    
    class AddTagsResponse(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreateTrailRequest(Mapping):
        pass
    
    class CreateTrailResponse(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class DeleteTrailRequest(Mapping):
        pass
    
    class DeleteTrailResponse(Mapping):
        pass
    
    class DescribeTrailsRequest(Mapping):
        pass
    
    class DescribeTrailsResponse(Mapping):
        pass
    
    class String(object):
        pass
    
    class GetEventSelectorsRequest(Mapping):
        pass
    
    class GetEventSelectorsResponse(Mapping):
        pass
    
    class GetTrailStatusRequest(Mapping):
        pass
    
    class GetTrailStatusResponse(Mapping):
        pass
    
    class ListPublicKeysRequest(Mapping):
        pass
    
    class ListPublicKeysResponse(Mapping):
        pass
    
    class Date(object):
        pass
    
    class ListTagsRequest(Mapping):
        pass
    
    class ListTagsResponse(Mapping):
        pass
    
    class LookupEventsRequest(Mapping):
        pass
    
    class LookupEventsResponse(Mapping):
        pass
    
    class LookupAttribute(Mapping):
        pass
    
    class PutEventSelectorsRequest(Mapping):
        pass
    
    class PutEventSelectorsResponse(Mapping):
        pass
    
    class EventSelector(Mapping):
        pass
    
    class RemoveTagsRequest(Mapping):
        pass
    
    class RemoveTagsResponse(Mapping):
        pass
    
    class StartLoggingRequest(Mapping):
        pass
    
    class StartLoggingResponse(Mapping):
        pass
    
    class StopLoggingRequest(Mapping):
        pass
    
    class StopLoggingResponse(Mapping):
        pass
    
    class UpdateTrailRequest(Mapping):
        pass
    
    class UpdateTrailResponse(Mapping):
        pass
    
    def add_tags(self, ResourceId: str, TagsList: List[Tag]=None) -> AddTagsResponse:
        """Adds one or more tags to a trail, up to a limit of 50. Tags must be unique per trail. Overwrites an existing tag's value when a new value is specified for an existing tag key. If you specify a key without a value, the tag will be created with the specified key and a value of null. You can tag a trail that applies to all regions only from the region in which the trail was created (that is, from its home region).

        :param self:
        :param TagsList:  Contains a list of CloudTrail tags, up to a limit of 50<br/>ResourceId:  Specifies the ARN of the trail to which one or more tags will be added. The format of a trail ARN is:  <code>arn:aws:cloudtrail:us-east-2:123456789012:trail/MyTrail</code> <br/>
        :return:         """
        pass

    def create_trail(self, S3BucketName: str, Name: str, S3KeyPrefix: str=None, SnsTopicName: str=None, IncludeGlobalServiceEvents: Boolean=None, IsMultiRegionTrail: Boolean=None, EnableLogFileValidation: Boolean=None, CloudWatchLogsLogGroupArn: str=None, CloudWatchLogsRoleArn: str=None, KmsKeyId: str=None, IsOrganizationTrail: Boolean=None) -> CreateTrailResponse:
        """Creates a trail that specifies the settings for delivery of log data to an Amazon S3 bucket. A maximum of five trails can exist in a region, irrespective of the region in which they were created.

        :param self:
        :param IsOrganizationTrail:  Specifies whether the trail is created for all accounts in an organization in AWS Organizations, or only for the current AWS account. The default is false, and cannot be true unless the call is made on behalf of an AWS account that is the master account for an organization in AWS Organizations.<br/>KmsKeyId:  Specifies the KMS key ID to use to encrypt the logs delivered by CloudTrail. The value can be an alias name prefixed by "alias/", a fully specified ARN to an alias, a fully specified ARN to a key, or a globally unique identifier. Examples: <ul> <li> alias/MyAliasName </li> <li> arn:aws:kms:us-east-2:123456789012:alias/MyAliasName </li> <li> arn:aws:kms:us-east-2:123456789012:key/12345678-1234-1234-1234-123456789012 </li> <li> 12345678-1234-1234-1234-123456789012 </li> </ul><br/>CloudWatchLogsRoleArn:  Specifies the role for the CloudWatch Logs endpoint to assume to write to a user's log group.<br/>CloudWatchLogsLogGroupArn:  Specifies a log group name using an Amazon Resource Name (ARN), a unique identifier that represents the log group to which CloudTrail logs will be delivered. Not required unless you specify CloudWatchLogsRoleArn.<br/>EnableLogFileValidation:  Specifies whether log file integrity validation is enabled. The default is false. <note> When you disable log file integrity validation, the chain of digest files is broken after one hour. CloudTrail will not create digest files for log files that were delivered during a period in which log file integrity validation was disabled. For example, if you enable log file integrity validation at noon on January 1, disable it at noon on January 2, and re-enable it at noon on January 10, digest files will not be created for the log files delivered from noon on January 2 to noon on January 10. The same applies whenever you stop CloudTrail logging or delete a trail. </note><br/>IsMultiRegionTrail:  Specifies whether the trail is created in the current region or in all regions. The default is false.<br/>IncludeGlobalServiceEvents:  Specifies whether the trail is publishing events from global services such as IAM to the log files.<br/>SnsTopicName:  Specifies the name of the Amazon SNS topic defined for notification of log file delivery. The maximum length is 256 characters.<br/>S3KeyPrefix:  Specifies the Amazon S3 key prefix that comes after the name of the bucket you have designated for log file delivery. For more information, see <a href="http://docs.aws.amazon.com/awscloudtrail/latest/userguide/cloudtrail-find-log-files.html">Finding Your CloudTrail Log Files</a>. The maximum length is 200 characters.<br/>S3BucketName:  Specifies the name of the Amazon S3 bucket designated for publishing log files. See <a href="http://docs.aws.amazon.com/awscloudtrail/latest/userguide/create_trail_naming_policy.html">Amazon S3 Bucket Naming Requirements</a>.<br/>Name:  Specifies the name of the trail. The name must meet the following requirements: <ul> <li> Contain only ASCII letters (a-z, A-Z), numbers (0-9), periods (.), underscores (_), or dashes (-) </li> <li> Start with a letter or number, and end with a letter or number </li> <li> Be between 3 and 128 characters </li> <li> Have no adjacent periods, underscores or dashes. Names like <code>my-_namespace</code> and <code>my--namespace</code> are invalid. </li> <li> Not be in IP address format (for example, 192.168.5.4) </li> </ul><br/>
        :return: IsOrganizationTrail:  Specifies whether the trail is an organization trail.<br/>KmsKeyId:  Specifies the KMS key ID that encrypts the logs delivered by CloudTrail. The value is a fully specified ARN to a KMS key in the format:  <code>arn:aws:kms:us-east-2:123456789012:key/12345678-1234-1234-1234-123456789012</code> <br/>CloudWatchLogsRoleArn:  Specifies the role for the CloudWatch Logs endpoint to assume to write to a user's log group.<br/>CloudWatchLogsLogGroupArn:  Specifies the Amazon Resource Name (ARN) of the log group to which CloudTrail logs will be delivered.<br/>LogFileValidationEnabled:  Specifies whether log file integrity validation is enabled.<br/>TrailARN:  Specifies the ARN of the trail that was created. The format of a trail ARN is:  <code>arn:aws:cloudtrail:us-east-2:123456789012:trail/MyTrail</code> <br/>IsMultiRegionTrail:  Specifies whether the trail exists in one region or in all regions.<br/>IncludeGlobalServiceEvents:  Specifies whether the trail is publishing events from global services such as IAM to the log files.<br/>SnsTopicARN:  Specifies the ARN of the Amazon SNS topic that CloudTrail uses to send notifications when log files are delivered. The format of a topic ARN is:  <code>arn:aws:sns:us-east-2:123456789012:MyTopic</code> <br/>SnsTopicName:  This field is deprecated. Use SnsTopicARN.<br/>S3KeyPrefix:  Specifies the Amazon S3 key prefix that comes after the name of the bucket you have designated for log file delivery. For more information, see <a href="http://docs.aws.amazon.com/awscloudtrail/latest/userguide/cloudtrail-find-log-files.html">Finding Your CloudTrail Log Files</a>.<br/>S3BucketName:  Specifies the name of the Amazon S3 bucket designated for publishing log files.<br/>Name:  Specifies the name of the trail.<br/>        """
        pass

    def delete_trail(self, Name: str) -> DeleteTrailResponse:
        """Deletes a trail. This operation must be called from the region in which the trail was created. <code>DeleteTrail</code> cannot be called on the shadow trails (replicated trails in other regions) of a trail that is enabled in all regions.

        :param self:
        :param Name:  Specifies the name or the CloudTrail ARN of the trail to be deleted. The format of a trail ARN is: <code>arn:aws:cloudtrail:us-east-2:123456789012:trail/MyTrail</code> <br/>
        :return:         """
        pass

    def describe_trails(self, trailNameList: List[String]=None, includeShadowTrails: Boolean=None) -> DescribeTrailsResponse:
        """Retrieves settings for the trail associated with the current region for your account.

        :param self:
        :param includeShadowTrails:  Specifies whether to include shadow trails in the response. A shadow trail is the replication in a region of a trail that was created in a different region, or in the case of an organization trail, the replication of an organization trail in member accounts. If you do not include shadow trails, organization trails in a member account and region replication trails will not be returned. The default is true.<br/>trailNameList:  Specifies a list of trail names, trail ARNs, or both, of the trails to describe. The format of a trail ARN is:  <code>arn:aws:cloudtrail:us-east-2:123456789012:trail/MyTrail</code>  If an empty list is specified, information for the trail in the current region is returned. <ul> <li> If an empty list is specified and <code>IncludeShadowTrails</code> is false, then information for all trails in the current region is returned. </li> <li> If an empty list is specified and IncludeShadowTrails is null or true, then information for all trails in the current region and any associated shadow trails in other regions is returned. </li> </ul> <note> If one or more trail names are specified, information is returned only if the names match the names of trails belonging only to the current region. To return information about a trail in another region, you must specify its trail ARN. </note><br/>
        :return: trailList:  The list of trail objects.<br/>        """
        pass

    def get_event_selectors(self, TrailName: str) -> GetEventSelectorsResponse:
        """Describes the settings for the event selectors that you configured for your trail. The information returned for your event selectors includes the following: <ul> <li> If your event selector includes read-only events, write-only events, or all events. This applies to both management events and data events. </li> <li> If your event selector includes management events. </li> <li> If your event selector includes data events, the Amazon S3 objects or AWS Lambda functions that you are logging for data events. </li> </ul> For more information, see <a href="http://docs.aws.amazon.com/awscloudtrail/latest/userguide/logging-management-and-data-events-with-cloudtrail.html">Logging Data and Management Events for Trails </a> in the <i>AWS CloudTrail User Guide</i>.

        :param self:
        :param TrailName:  Specifies the name of the trail or trail ARN. If you specify a trail name, the string must meet the following requirements: <ul> <li> Contain only ASCII letters (a-z, A-Z), numbers (0-9), periods (.), underscores (_), or dashes (-) </li> <li> Start with a letter or number, and end with a letter or number </li> <li> Be between 3 and 128 characters </li> <li> Have no adjacent periods, underscores or dashes. Names like <code>my-_namespace</code> and <code>my--namespace</code> are not valid. </li> <li> Not be in IP address format (for example, 192.168.5.4) </li> </ul> If you specify a trail ARN, it must be in the format:  <code>arn:aws:cloudtrail:us-east-2:123456789012:trail/MyTrail</code> <br/>
        :return: EventSelectors:  The event selectors that are configured for the trail.<br/>TrailARN:  The specified trail ARN that has the event selectors.<br/>        """
        pass

    def get_trail_status(self, Name: str) -> GetTrailStatusResponse:
        """Returns a JSON-formatted list of information about the specified trail. Fields include information on delivery errors, Amazon SNS and Amazon S3 errors, and start and stop logging times for each trail. This operation returns trail status from a single region. To return trail status from all regions, you must call the operation on each region.

        :param self:
        :param Name:  Specifies the name or the CloudTrail ARN of the trail for which you are requesting status. To get the status of a shadow trail (a replication of the trail in another region), you must specify its ARN. The format of a trail ARN is:  <code>arn:aws:cloudtrail:us-east-2:123456789012:trail/MyTrail</code> <br/>
        :return: TimeLoggingStopped:  This field is deprecated.<br/>TimeLoggingStarted:  This field is deprecated.<br/>LatestDeliveryAttemptSucceeded:  This field is deprecated.<br/>LatestNotificationAttemptSucceeded:  This field is deprecated.<br/>LatestNotificationAttemptTime:  This field is deprecated.<br/>LatestDeliveryAttemptTime:  This field is deprecated.<br/>LatestDigestDeliveryError:  Displays any Amazon S3 error that CloudTrail encountered when attempting to deliver a digest file to the designated bucket. For more information see the topic <a href="http://docs.aws.amazon.com/AmazonS3/latest/API/ErrorResponses.html">Error Responses</a> in the Amazon S3 API Reference.  <note> This error occurs only when there is a problem with the destination S3 bucket and will not occur for timeouts. To resolve the issue, create a new bucket and call <code>UpdateTrail</code> to specify the new bucket, or fix the existing objects so that CloudTrail can again write to the bucket. </note><br/>LatestDigestDeliveryTime:  Specifies the date and time that CloudTrail last delivered a digest file to an account's Amazon S3 bucket.<br/>LatestCloudWatchLogsDeliveryTime:  Displays the most recent date and time when CloudTrail delivered logs to CloudWatch Logs.<br/>LatestCloudWatchLogsDeliveryError:  Displays any CloudWatch Logs error that CloudTrail encountered when attempting to deliver logs to CloudWatch Logs.<br/>StopLoggingTime:  Specifies the most recent date and time when CloudTrail stopped recording API calls for an AWS account.<br/>StartLoggingTime:  Specifies the most recent date and time when CloudTrail started recording API calls for an AWS account.<br/>LatestNotificationTime:  Specifies the date and time of the most recent Amazon SNS notification that CloudTrail has written a new log file to an account's Amazon S3 bucket.<br/>LatestDeliveryTime:  Specifies the date and time that CloudTrail last delivered log files to an account's Amazon S3 bucket.<br/>LatestNotificationError:  Displays any Amazon SNS error that CloudTrail encountered when attempting to send a notification. For more information about Amazon SNS errors, see the <a href="http://docs.aws.amazon.com/sns/latest/dg/welcome.html">Amazon SNS Developer Guide</a>. <br/>LatestDeliveryError:  Displays any Amazon S3 error that CloudTrail encountered when attempting to deliver log files to the designated bucket. For more information see the topic <a href="http://docs.aws.amazon.com/AmazonS3/latest/API/ErrorResponses.html">Error Responses</a> in the Amazon S3 API Reference.  <note> This error occurs only when there is a problem with the destination S3 bucket and will not occur for timeouts. To resolve the issue, create a new bucket and call <code>UpdateTrail</code> to specify the new bucket, or fix the existing objects so that CloudTrail can again write to the bucket. </note><br/>IsLogging:  Whether the CloudTrail is currently logging AWS API calls.<br/>        """
        pass

    def list_public_keys(self, StartTime: Date=None, EndTime: Date=None, NextToken: str=None) -> ListPublicKeysResponse:
        """Returns all public keys whose private keys were used to sign the digest files within the specified time range. The public key is needed to validate digest files that were signed with its corresponding private key. <note> CloudTrail uses different private/public key pairs per region. Each digest file is signed with a private key unique to its region. Therefore, when you validate a digest file from a particular region, you must look in the same region for its corresponding public key. </note>

        :param self:
        :param NextToken:  Reserved for future use.<br/>EndTime:  Optionally specifies, in UTC, the end of the time range to look up public keys for CloudTrail digest files. If not specified, the current time is used.<br/>StartTime:  Optionally specifies, in UTC, the start of the time range to look up public keys for CloudTrail digest files. If not specified, the current time is used, and the current public key is returned.<br/>
        :return: NextToken:  Reserved for future use.<br/>PublicKeyList:  Contains an array of PublicKey objects. <note> The returned public keys may have validity time ranges that overlap. </note><br/>        """
        pass

    def list_tags(self, ResourceIdList: List[String], NextToken: str=None) -> ListTagsResponse:
        """Lists the tags for the trail in the current region.

        :param self:
        :param NextToken:  Reserved for future use.<br/>ResourceIdList:  Specifies a list of trail ARNs whose tags will be listed. The list has a limit of 20 ARNs. The format of a trail ARN is:  <code>arn:aws:cloudtrail:us-east-2:123456789012:trail/MyTrail</code> <br/>
        :return: NextToken:  Reserved for future use.<br/>ResourceTagList:  A list of resource tags.<br/>        """
        pass

    def lookup_events(self, LookupAttributes: List[LookupAttribute]=None, StartTime: Date=None, EndTime: Date=None, MaxResults: int=None, NextToken: str=None) -> LookupEventsResponse:
        """Looks up <a href="https://docs.aws.amazon.com/awscloudtrail/latest/userguide/cloudtrail-concepts.html#cloudtrail-concepts-management-events">management events</a> captured by CloudTrail. Events for a region can be looked up in that region during the last 90 days. Lookup supports the following attributes: <ul> <li> AWS access key </li> <li> Event ID </li> <li> Event name </li> <li> Event source </li> <li> Read only </li> <li> Resource name </li> <li> Resource type </li> <li> User name </li> </ul> All attributes are optional. The default number of results returned is 50, with a maximum of 50 possible. The response includes a token that you can use to get the next page of results. <important> The rate of lookup requests is limited to one per second per account. If this limit is exceeded, a throttling error occurs. </important> <important> Events that occurred during the selected time range will not be available for lookup if CloudTrail logging was not enabled when the events occurred. </important>

        :param self:
        :param NextToken:  The token to use to get the next page of results after a previous API call. This token must be passed in with the same parameters that were specified in the the original call. For example, if the original call specified an AttributeKey of 'Username' with a value of 'root', the call with NextToken should include those same parameters.<br/>MaxResults:  The number of events to return. Possible values are 1 through 50. The default is 50.<br/>EndTime:  Specifies that only events that occur before or at the specified time are returned. If the specified end time is before the specified start time, an error is returned.<br/>StartTime:  Specifies that only events that occur after or at the specified time are returned. If the specified start time is after the specified end time, an error is returned.<br/>LookupAttributes:  Contains a list of lookup attributes. Currently the list can contain only one item.<br/>
        :return: NextToken:  The token to use to get the next page of results after a previous API call. If the token does not appear, there are no more results to return. The token must be passed in with the same parameters as the previous call. For example, if the original call specified an AttributeKey of 'Username' with a value of 'root', the call with NextToken should include those same parameters.<br/>Events:  A list of events returned based on the lookup attributes specified and the CloudTrail event. The events list is sorted by time. The most recent event is listed first.<br/>        """
        pass

    def put_event_selectors(self, EventSelectors: List[EventSelector], TrailName: str) -> PutEventSelectorsResponse:
        """Configures an event selector for your trail. Use event selectors to further specify the management and data event settings for your trail. By default, trails created without specific event selectors will be configured to log all read and write management events, and no data events.  When an event occurs in your account, CloudTrail evaluates the event selectors in all trails. For each trail, if the event matches any event selector, the trail processes and logs the event. If the event doesn't match any event selector, the trail doesn't log the event.  Example <ol> <li> You create an event selector for a trail and specify that you want write-only events. </li> <li> The EC2 <code>GetConsoleOutput</code> and <code>RunInstances</code> API operations occur in your account. </li> <li> CloudTrail evaluates whether the events match your event selectors. </li> <li> The <code>RunInstances</code> is a write-only event and it matches your event selector. The trail logs the event. </li> <li> The <code>GetConsoleOutput</code> is a read-only event but it doesn't match your event selector. The trail doesn't log the event.  </li> </ol> The <code>PutEventSelectors</code> operation must be called from the region in which the trail was created; otherwise, an <code>InvalidHomeRegionException</code> is thrown. You can configure up to five event selectors for each trail. For more information, see <a href="http://docs.aws.amazon.com/awscloudtrail/latest/userguide/logging-management-and-data-events-with-cloudtrail.html">Logging Data and Management Events for Trails </a> and <a href="https://docs.aws.amazon.com/awscloudtrail/latest/userguide/WhatIsCloudTrail-Limits.html">Limits in AWS CloudTrail</a> in the <i>AWS CloudTrail User Guide</i>.

        :param self:
        :param EventSelectors:  Specifies the settings for your event selectors. You can configure up to five event selectors for a trail.<br/>TrailName:  Specifies the name of the trail or trail ARN. If you specify a trail name, the string must meet the following requirements: <ul> <li> Contain only ASCII letters (a-z, A-Z), numbers (0-9), periods (.), underscores (_), or dashes (-) </li> <li> Start with a letter or number, and end with a letter or number </li> <li> Be between 3 and 128 characters </li> <li> Have no adjacent periods, underscores or dashes. Names like <code>my-_namespace</code> and <code>my--namespace</code> are invalid. </li> <li> Not be in IP address format (for example, 192.168.5.4) </li> </ul> If you specify a trail ARN, it must be in the format:  <code>arn:aws:cloudtrail:us-east-2:123456789012:trail/MyTrail</code> <br/>
        :return: EventSelectors:  Specifies the event selectors configured for your trail.<br/>TrailARN:  Specifies the ARN of the trail that was updated with event selectors. The format of a trail ARN is:  <code>arn:aws:cloudtrail:us-east-2:123456789012:trail/MyTrail</code> <br/>        """
        pass

    def remove_tags(self, ResourceId: str, TagsList: List[Tag]=None) -> RemoveTagsResponse:
        """Removes the specified tags from a trail.

        :param self:
        :param TagsList:  Specifies a list of tags to be removed.<br/>ResourceId:  Specifies the ARN of the trail from which tags should be removed. The format of a trail ARN is:  <code>arn:aws:cloudtrail:us-east-2:123456789012:trail/MyTrail</code> <br/>
        :return:         """
        pass

    def start_logging(self, Name: str) -> StartLoggingResponse:
        """Starts the recording of AWS API calls and log file delivery for a trail. For a trail that is enabled in all regions, this operation must be called from the region in which the trail was created. This operation cannot be called on the shadow trails (replicated trails in other regions) of a trail that is enabled in all regions.

        :param self:
        :param Name:  Specifies the name or the CloudTrail ARN of the trail for which CloudTrail logs AWS API calls. The format of a trail ARN is:  <code>arn:aws:cloudtrail:us-east-2:123456789012:trail/MyTrail</code> <br/>
        :return:         """
        pass

    def stop_logging(self, Name: str) -> StopLoggingResponse:
        """Suspends the recording of AWS API calls and log file delivery for the specified trail. Under most circumstances, there is no need to use this action. You can update a trail without stopping it first. This action is the only way to stop recording. For a trail enabled in all regions, this operation must be called from the region in which the trail was created, or an <code>InvalidHomeRegionException</code> will occur. This operation cannot be called on the shadow trails (replicated trails in other regions) of a trail enabled in all regions.

        :param self:
        :param Name:  Specifies the name or the CloudTrail ARN of the trail for which CloudTrail will stop logging AWS API calls. The format of a trail ARN is:  <code>arn:aws:cloudtrail:us-east-2:123456789012:trail/MyTrail</code> <br/>
        :return:         """
        pass

    def update_trail(self, Name: str, S3BucketName: str=None, S3KeyPrefix: str=None, SnsTopicName: str=None, IncludeGlobalServiceEvents: Boolean=None, IsMultiRegionTrail: Boolean=None, EnableLogFileValidation: Boolean=None, CloudWatchLogsLogGroupArn: str=None, CloudWatchLogsRoleArn: str=None, KmsKeyId: str=None, IsOrganizationTrail: Boolean=None) -> UpdateTrailResponse:
        """Updates the settings that specify delivery of log files. Changes to a trail do not require stopping the CloudTrail service. Use this action to designate an existing bucket for log delivery. If the existing bucket has previously been a target for CloudTrail log files, an IAM policy exists for the bucket. <code>UpdateTrail</code> must be called from the region in which the trail was created; otherwise, an <code>InvalidHomeRegionException</code> is thrown.

        :param self:
        :param IsOrganizationTrail:  Specifies whether the trail is applied to all accounts in an organization in AWS Organizations, or only for the current AWS account. The default is false, and cannot be true unless the call is made on behalf of an AWS account that is the master account for an organization in AWS Organizations. If the trail is not an organization trail and this is set to true, the trail will be created in all AWS accounts that belong to the organization. If the trail is an organization trail and this is set to false, the trail will remain in the current AWS account but be deleted from all member accounts in the organization.<br/>KmsKeyId:  Specifies the KMS key ID to use to encrypt the logs delivered by CloudTrail. The value can be an alias name prefixed by "alias/", a fully specified ARN to an alias, a fully specified ARN to a key, or a globally unique identifier. Examples: <ul> <li> alias/MyAliasName </li> <li> arn:aws:kms:us-east-2:123456789012:alias/MyAliasName </li> <li> arn:aws:kms:us-east-2:123456789012:key/12345678-1234-1234-1234-123456789012 </li> <li> 12345678-1234-1234-1234-123456789012 </li> </ul><br/>CloudWatchLogsRoleArn:  Specifies the role for the CloudWatch Logs endpoint to assume to write to a user's log group.<br/>CloudWatchLogsLogGroupArn:  Specifies a log group name using an Amazon Resource Name (ARN), a unique identifier that represents the log group to which CloudTrail logs will be delivered. Not required unless you specify CloudWatchLogsRoleArn.<br/>EnableLogFileValidation:  Specifies whether log file validation is enabled. The default is false. <note> When you disable log file integrity validation, the chain of digest files is broken after one hour. CloudTrail will not create digest files for log files that were delivered during a period in which log file integrity validation was disabled. For example, if you enable log file integrity validation at noon on January 1, disable it at noon on January 2, and re-enable it at noon on January 10, digest files will not be created for the log files delivered from noon on January 2 to noon on January 10. The same applies whenever you stop CloudTrail logging or delete a trail. </note><br/>IsMultiRegionTrail:  Specifies whether the trail applies only to the current region or to all regions. The default is false. If the trail exists only in the current region and this value is set to true, shadow trails (replications of the trail) will be created in the other regions. If the trail exists in all regions and this value is set to false, the trail will remain in the region where it was created, and its shadow trails in other regions will be deleted.<br/>IncludeGlobalServiceEvents:  Specifies whether the trail is publishing events from global services such as IAM to the log files.<br/>SnsTopicName:  Specifies the name of the Amazon SNS topic defined for notification of log file delivery. The maximum length is 256 characters.<br/>S3KeyPrefix:  Specifies the Amazon S3 key prefix that comes after the name of the bucket you have designated for log file delivery. For more information, see <a href="http://docs.aws.amazon.com/awscloudtrail/latest/userguide/cloudtrail-find-log-files.html">Finding Your CloudTrail Log Files</a>. The maximum length is 200 characters.<br/>S3BucketName:  Specifies the name of the Amazon S3 bucket designated for publishing log files. See <a href="http://docs.aws.amazon.com/awscloudtrail/latest/userguide/create_trail_naming_policy.html">Amazon S3 Bucket Naming Requirements</a>.<br/>Name:  Specifies the name of the trail or trail ARN. If <code>Name</code> is a trail name, the string must meet the following requirements: <ul> <li> Contain only ASCII letters (a-z, A-Z), numbers (0-9), periods (.), underscores (_), or dashes (-) </li> <li> Start with a letter or number, and end with a letter or number </li> <li> Be between 3 and 128 characters </li> <li> Have no adjacent periods, underscores or dashes. Names like <code>my-_namespace</code> and <code>my--namespace</code> are invalid. </li> <li> Not be in IP address format (for example, 192.168.5.4) </li> </ul> If <code>Name</code> is a trail ARN, it must be in the format:  <code>arn:aws:cloudtrail:us-east-2:123456789012:trail/MyTrail</code> <br/>
        :return: IsOrganizationTrail:  Specifies whether the trail is an organization trail.<br/>KmsKeyId:  Specifies the KMS key ID that encrypts the logs delivered by CloudTrail. The value is a fully specified ARN to a KMS key in the format:  <code>arn:aws:kms:us-east-2:123456789012:key/12345678-1234-1234-1234-123456789012</code> <br/>CloudWatchLogsRoleArn:  Specifies the role for the CloudWatch Logs endpoint to assume to write to a user's log group.<br/>CloudWatchLogsLogGroupArn:  Specifies the Amazon Resource Name (ARN) of the log group to which CloudTrail logs will be delivered.<br/>LogFileValidationEnabled:  Specifies whether log file integrity validation is enabled.<br/>TrailARN:  Specifies the ARN of the trail that was updated. The format of a trail ARN is:  <code>arn:aws:cloudtrail:us-east-2:123456789012:trail/MyTrail</code> <br/>IsMultiRegionTrail:  Specifies whether the trail exists in one region or in all regions.<br/>IncludeGlobalServiceEvents:  Specifies whether the trail is publishing events from global services such as IAM to the log files.<br/>SnsTopicARN:  Specifies the ARN of the Amazon SNS topic that CloudTrail uses to send notifications when log files are delivered. The format of a topic ARN is:  <code>arn:aws:sns:us-east-2:123456789012:MyTopic</code> <br/>SnsTopicName:  This field is deprecated. Use SnsTopicARN.<br/>S3KeyPrefix:  Specifies the Amazon S3 key prefix that comes after the name of the bucket you have designated for log file delivery. For more information, see <a href="http://docs.aws.amazon.com/awscloudtrail/latest/userguide/cloudtrail-find-log-files.html">Finding Your CloudTrail Log Files</a>.<br/>S3BucketName:  Specifies the name of the Amazon S3 bucket designated for publishing log files.<br/>Name:  Specifies the name of the trail.<br/>        """
        pass



class CloudWatch(BaseClient):
    """Amazon CloudWatch monitors your Amazon Web Services (AWS) resources and the applications you run on AWS in real time. You can use CloudWatch to collect and track metrics, which are the variables you want to measure for your resources and applications. CloudWatch alarms send notifications or automatically change the resources you are monitoring based on rules that you define. For example, you can monitor the CPU usage and disk reads and writes of your Amazon EC2 instances. Then, use this data to determine whether you should launch additional instances to handle increased load. You can also use this data to stop under-used instances to save money. In addition to monitoring the built-in metrics that come with AWS, you can monitor your own custom metrics. With CloudWatch, you gain system-wide visibility into resource utilization, application performance, and operational health."""

    class AlarmExistsWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_alarms` for valid parameters"""
            pass


    class DescribeAlarmHistoryPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_alarm_history` for valid parameters"""
            pass
    class DescribeAlarmsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_alarms` for valid parameters"""
            pass
    class ListDashboardsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_dashboards` for valid parameters"""
            pass
    class ListMetricsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_metrics` for valid parameters"""
            pass
    class GetMetricDataPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_metric_data` for valid parameters"""
            pass


    class DeleteAlarmsInput(Mapping):
        pass
    
    class AlarmName(object):
        pass
    
    class DeleteAnomalyDetectorInput(Mapping):
        pass
    
    class DeleteAnomalyDetectorOutput(Mapping):
        pass
    
    class Dimension(Mapping):
        pass
    
    class DeleteDashboardsInput(Mapping):
        pass
    
    class DeleteDashboardsOutput(Mapping):
        pass
    
    class DashboardName(object):
        pass
    
    class DescribeAlarmHistoryInput(Mapping):
        pass
    
    class DescribeAlarmHistoryOutput(Mapping):
        pass
    
    class Timestamp(object):
        pass
    
    class DescribeAlarmsInput(Mapping):
        pass
    
    class DescribeAlarmsOutput(Mapping):
        pass
    
    class DescribeAlarmsForMetricInput(Mapping):
        pass
    
    class DescribeAlarmsForMetricOutput(Mapping):
        pass
    
    class DescribeAnomalyDetectorsInput(Mapping):
        pass
    
    class DescribeAnomalyDetectorsOutput(Mapping):
        pass
    
    class DisableAlarmActionsInput(Mapping):
        pass
    
    class EnableAlarmActionsInput(Mapping):
        pass
    
    class GetDashboardInput(Mapping):
        pass
    
    class GetDashboardOutput(Mapping):
        pass
    
    class GetMetricDataInput(Mapping):
        pass
    
    class GetMetricDataOutput(Mapping):
        pass
    
    class MetricDataQuery(Mapping):
        pass
    
    class GetMetricStatisticsInput(Mapping):
        pass
    
    class GetMetricStatisticsOutput(Mapping):
        pass
    
    class Statistic(object):
        pass
    
    class ExtendedStatistic(object):
        pass
    
    class GetMetricWidgetImageInput(Mapping):
        pass
    
    class GetMetricWidgetImageOutput(Mapping):
        pass
    
    class ListDashboardsInput(Mapping):
        pass
    
    class ListDashboardsOutput(Mapping):
        pass
    
    class ListMetricsInput(Mapping):
        pass
    
    class ListMetricsOutput(Mapping):
        pass
    
    class DimensionFilter(Mapping):
        pass
    
    class ListTagsForResourceInput(Mapping):
        pass
    
    class ListTagsForResourceOutput(Mapping):
        pass
    
    class PutAnomalyDetectorInput(Mapping):
        pass
    
    class PutAnomalyDetectorOutput(Mapping):
        pass
    
    class AnomalyDetectorConfiguration(Mapping):
        pass
    
    class PutDashboardInput(Mapping):
        pass
    
    class PutDashboardOutput(Mapping):
        pass
    
    class PutMetricAlarmInput(Mapping):
        pass
    
    class ActionsEnabled(object):
        pass
    
    class ResourceName(object):
        pass
    
    class Threshold(object):
        pass
    
    class Tag(Mapping):
        pass
    
    class PutMetricDataInput(Mapping):
        pass
    
    class MetricDatum(Mapping):
        pass
    
    class SetAlarmStateInput(Mapping):
        pass
    
    class TagResourceInput(Mapping):
        pass
    
    class TagResourceOutput(Mapping):
        pass
    
    class UntagResourceInput(Mapping):
        pass
    
    class UntagResourceOutput(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    def delete_alarms(self, AlarmNames: List[AlarmName]):
        """Deletes the specified alarms. In the event of an error, no alarms are deleted.

        :param self:
        :param AlarmNames:  The alarms to be deleted.<br/>
        :return:         """
        pass

    def delete_anomaly_detector(self, Stat: str, MetricName: str, Namespace: str, Dimensions: List[Dimension]=None) -> DeleteAnomalyDetectorOutput:
        """Deletes the specified anomaly detection model from your account.

        :param self:
        :param Stat:  The statistic associated with the anomaly detection model to delete.<br/>Dimensions:  The metric dimensions associated with the anomaly detection model to delete.<br/>MetricName:  The metric name associated with the anomaly detection model to delete.<br/>Namespace:  The namespace associated with the anomaly detection model to delete.<br/>
        :return:         """
        pass

    def delete_dashboards(self, DashboardNames: List[DashboardName]) -> DeleteDashboardsOutput:
        """Deletes all dashboards that you specify. You may specify up to 100 dashboards to delete. If there is an error during this call, no dashboards are deleted.

        :param self:
        :param DashboardNames:  The dashboards to be deleted. This parameter is required.<br/>
        :return:         """
        pass

    def describe_alarm_history(self, AlarmName: str=None, HistoryItemType: str=None, StartDate: Timestamp=None, EndDate: Timestamp=None, MaxRecords: int=None, NextToken: str=None) -> DescribeAlarmHistoryOutput:
        """Retrieves the history for the specified alarm. You can filter the results by date range or item type. If an alarm name is not specified, the histories for all alarms are returned. CloudWatch retains the history of an alarm even if you delete the alarm.

        :param self:
        :param NextToken:  The token returned by a previous call to indicate that there is more data available.<br/>MaxRecords:  The maximum number of alarm history records to retrieve.<br/>EndDate:  The ending date to retrieve alarm history.<br/>StartDate:  The starting date to retrieve alarm history.<br/>HistoryItemType:  The type of alarm histories to retrieve.<br/>AlarmName:  The name of the alarm.<br/>
        :return: NextToken:  The token that marks the start of the next batch of returned results.<br/>AlarmHistoryItems:  The alarm histories, in JSON format.<br/>        """
        pass

    def describe_alarms(self, AlarmNames: List[AlarmName]=None, AlarmNamePrefix: str=None, StateValue: str=None, ActionPrefix: str=None, MaxRecords: int=None, NextToken: str=None) -> DescribeAlarmsOutput:
        """Retrieves the specified alarms. If no alarms are specified, all alarms are returned. Alarms can be retrieved by using only a prefix for the alarm name, the alarm state, or a prefix for any action.

        :param self:
        :param NextToken:  The token returned by a previous call to indicate that there is more data available.<br/>MaxRecords:  The maximum number of alarm descriptions to retrieve.<br/>ActionPrefix:  The action name prefix.<br/>StateValue:  The state value to be used in matching alarms.<br/>AlarmNamePrefix:  The alarm name prefix. If this parameter is specified, you cannot specify <code>AlarmNames</code>.<br/>AlarmNames:  The names of the alarms.<br/>
        :return: NextToken:  The token that marks the start of the next batch of returned results.<br/>MetricAlarms:  The information for the specified alarms.<br/>        """
        pass

    def describe_alarms_for_metric(self, Namespace: str, MetricName: str, Statistic: str=None, ExtendedStatistic: str=None, Dimensions: List[Dimension]=None, Period: int=None, Unit: str=None) -> DescribeAlarmsForMetricOutput:
        """Retrieves the alarms for the specified metric. To filter the results, specify a statistic, period, or unit.

        :param self:
        :param Unit:  The unit for the metric.<br/>Period:  The period, in seconds, over which the statistic is applied.<br/>Dimensions:  The dimensions associated with the metric. If the metric has any associated dimensions, you must specify them in order for the call to succeed.<br/>ExtendedStatistic:  The percentile statistic for the metric. Specify a value between p0.0 and p100.<br/>Statistic:  The statistic for the metric, other than percentiles. For percentile statistics, use <code>ExtendedStatistics</code>.<br/>Namespace:  The namespace of the metric.<br/>MetricName:  The name of the metric.<br/>
        :return: MetricAlarms:  The information for each alarm with the specified metric.<br/>        """
        pass

    def describe_anomaly_detectors(self, NextToken: str=None, MaxResults: int=None, Namespace: str=None, MetricName: str=None, Dimensions: List[Dimension]=None) -> DescribeAnomalyDetectorsOutput:
        """Lists the anomaly detection models that you have created in your account. You can list all models in your account or filter the results to only the models that are related to a certain namespace, metric name, or metric dimension.

        :param self:
        :param Dimensions:  Limits the results to only the anomaly detection models that are associated with the specified metric dimensions. If there are multiple metrics that have these dimensions and have anomaly detection models associated, they're all returned.<br/>MetricName:  Limits the results to only the anomaly detection models that are associated with the specified metric name. If there are multiple metrics with this name in different namespaces that have anomaly detection models, they're all returned.<br/>Namespace:  Limits the results to only the anomaly detection models that are associated with the specified namespace.<br/>MaxResults:  The maximum number of results to return in one operation. The maximum value you can specify is 10. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value. <br/>NextToken:  Use the token returned by the previous operation to request the next page of results.<br/>
        :return: NextToken:  A token that you can use in a subsequent operation to retrieve the next set of results.<br/>AnomalyDetectors:  The list of anomaly detection models returned by the operation.<br/>        """
        pass

    def disable_alarm_actions(self, AlarmNames: List[AlarmName]):
        """Disables the actions for the specified alarms. When an alarm's actions are disabled, the alarm actions do not execute when the alarm state changes.

        :param self:
        :param AlarmNames:  The names of the alarms.<br/>
        :return:         """
        pass

    def enable_alarm_actions(self, AlarmNames: List[AlarmName]):
        """Enables the actions for the specified alarms.

        :param self:
        :param AlarmNames:  The names of the alarms.<br/>
        :return:         """
        pass

    def get_dashboard(self, DashboardName: str) -> GetDashboardOutput:
        """Displays the details of the dashboard that you specify. To copy an existing dashboard, use <code>GetDashboard</code>, and then use the data returned within <code>DashboardBody</code> as the template for the new dashboard when you call <code>PutDashboard</code> to create the copy.

        :param self:
        :param DashboardName:  The name of the dashboard to be described.<br/>
        :return: DashboardName:  The name of the dashboard.<br/>DashboardBody:  The detailed information about the dashboard, including what widgets are included and their location on the dashboard. For more information about the <code>DashboardBody</code> syntax, see <a>CloudWatch-Dashboard-Body-Structure</a>. <br/>DashboardArn:  The Amazon Resource Name (ARN) of the dashboard.<br/>        """
        pass

    def get_metric_data(self, EndTime: Timestamp, StartTime: Timestamp, MetricDataQueries: List[MetricDataQuery], NextToken: str=None, ScanBy: str=None, MaxDatapoints: int=None) -> GetMetricDataOutput:
        """You can use the <code>GetMetricData</code> API to retrieve as many as 100 different metrics in a single request, with a total of as many as 100,800 datapoints. You can also optionally perform math expressions on the values of the returned statistics, to create new time series that represent new insights into your data. For example, using Lambda metrics, you could divide the Errors metric by the Invocations metric to get an error rate time series. For more information about metric math expressions, see <a href="https://docs.aws.amazon.com/AmazonCloudWatch/latest/monitoring/using-metric-math.html#metric-math-syntax">Metric Math Syntax and Functions</a> in the <i>Amazon CloudWatch User Guide</i>. Calls to the <code>GetMetricData</code> API have a different pricing structure than calls to <code>GetMetricStatistics</code>. For more information about pricing, see <a href="https://aws.amazon.com/cloudwatch/pricing/">Amazon CloudWatch Pricing</a>. Amazon CloudWatch retains metric data as follows: <ul> <li> Data points with a period of less than 60 seconds are available for 3 hours. These data points are high-resolution metrics and are available only for custom metrics that have been defined with a <code>StorageResolution</code> of 1. </li> <li> Data points with a period of 60 seconds (1-minute) are available for 15 days. </li> <li> Data points with a period of 300 seconds (5-minute) are available for 63 days. </li> <li> Data points with a period of 3600 seconds (1 hour) are available for 455 days (15 months). </li> </ul> Data points that are initially published with a shorter period are aggregated together for long-term storage. For example, if you collect data using a period of 1 minute, the data remains available for 15 days with 1-minute resolution. After 15 days, this data is still available, but is aggregated and retrievable only with a resolution of 5 minutes. After 63 days, the data is further aggregated and is available with a resolution of 1 hour.

        :param self:
        :param MaxDatapoints:  The maximum number of data points the request should return before paginating. If you omit this, the default of 100,800 is used.<br/>ScanBy:  The order in which data points should be returned. <code>TimestampDescending</code> returns the newest data first and paginates when the <code>MaxDatapoints</code> limit is reached. <code>TimestampAscending</code> returns the oldest data first and paginates when the <code>MaxDatapoints</code> limit is reached.<br/>NextToken:  Include this value, if it was returned by the previous call, to get the next set of data points.<br/>EndTime:  The time stamp indicating the latest data to be returned. For better performance, specify <code>StartTime</code> and <code>EndTime</code> values that align with the value of the metric's <code>Period</code> and sync up with the beginning and end of an hour. For example, if the <code>Period</code> of a metric is 5 minutes, specifying 12:05 or 12:30 as <code>EndTime</code> can get a faster response from CloudWatch than setting 12:07 or 12:29 as the <code>EndTime</code>.<br/>StartTime:  The time stamp indicating the earliest data to be returned. For better performance, specify <code>StartTime</code> and <code>EndTime</code> values that align with the value of the metric's <code>Period</code> and sync up with the beginning and end of an hour. For example, if the <code>Period</code> of a metric is 5 minutes, specifying 12:05 or 12:30 as <code>StartTime</code> can get a faster response from CloudWatch than setting 12:07 or 12:29 as the <code>StartTime</code>.<br/>MetricDataQueries:  The metric queries to be returned. A single <code>GetMetricData</code> call can include as many as 100 <code>MetricDataQuery</code> structures. Each of these structures can specify either a metric to retrieve, or a math expression to perform on retrieved data. <br/>
        :return: Messages:  Contains a message about this <code>GetMetricData</code> operation, if the operation results in such a message. An example of a message that may be returned is <code>Maximum number of allowed metrics exceeded</code>. If there is a message, as much of the operation as possible is still executed. A message appears here only if it is related to the global <code>GetMetricData</code> operation. Any message about a specific metric returned by the operation appears in the <code>MetricDataResult</code> object returned for that metric.<br/>NextToken:  A token that marks the next batch of returned results.<br/>MetricDataResults:  The metrics that are returned, including the metric name, namespace, and dimensions.<br/>        """
        pass

    def get_metric_statistics(self, Period: int, EndTime: Timestamp, StartTime: Timestamp, MetricName: str, Namespace: str, Dimensions: List[Dimension]=None, Statistics: List[Statistic]=None, ExtendedStatistics: List[ExtendedStatistic]=None, Unit: str=None) -> GetMetricStatisticsOutput:
        """Gets statistics for the specified metric. The maximum number of data points returned from a single call is 1,440. If you request more than 1,440 data points, CloudWatch returns an error. To reduce the number of data points, you can narrow the specified time range and make multiple requests across adjacent time ranges, or you can increase the specified period. Data points are not returned in chronological order. CloudWatch aggregates data points based on the length of the period that you specify. For example, if you request statistics with a one-hour period, CloudWatch aggregates all data points with time stamps that fall within each one-hour period. Therefore, the number of values aggregated by CloudWatch is larger than the number of data points returned. CloudWatch needs raw data points to calculate percentile statistics. If you publish data using a statistic set instead, you can only retrieve percentile statistics for this data if one of the following conditions is true: <ul> <li> The SampleCount value of the statistic set is 1. </li> <li> The Min and the Max values of the statistic set are equal. </li> </ul> Percentile statistics are not available for metrics when any of the metric values are negative numbers. Amazon CloudWatch retains metric data as follows: <ul> <li> Data points with a period of less than 60 seconds are available for 3 hours. These data points are high-resolution metrics and are available only for custom metrics that have been defined with a <code>StorageResolution</code> of 1. </li> <li> Data points with a period of 60 seconds (1-minute) are available for 15 days. </li> <li> Data points with a period of 300 seconds (5-minute) are available for 63 days. </li> <li> Data points with a period of 3600 seconds (1 hour) are available for 455 days (15 months). </li> </ul> Data points that are initially published with a shorter period are aggregated together for long-term storage. For example, if you collect data using a period of 1 minute, the data remains available for 15 days with 1-minute resolution. After 15 days, this data is still available, but is aggregated and retrievable only with a resolution of 5 minutes. After 63 days, the data is further aggregated and is available with a resolution of 1 hour. CloudWatch started retaining 5-minute and 1-hour metric data as of July 9, 2016. For information about metrics and dimensions supported by AWS services, see the <a href="https://docs.aws.amazon.com/AmazonCloudWatch/latest/monitoring/CW_Support_For_AWS.html">Amazon CloudWatch Metrics and Dimensions Reference</a> in the <i>Amazon CloudWatch User Guide</i>.

        :param self:
        :param Unit:  The unit for a given metric. Metrics may be reported in multiple units. Not supplying a unit results in all units being returned. If you specify only a unit that the metric does not report, the results of the call are null.<br/>ExtendedStatistics:  The percentile statistics. Specify values between p0.0 and p100. When calling <code>GetMetricStatistics</code>, you must specify either <code>Statistics</code> or <code>ExtendedStatistics</code>, but not both. Percentile statistics are not available for metrics when any of the metric values are negative numbers.<br/>Statistics:  The metric statistics, other than percentile. For percentile statistics, use <code>ExtendedStatistics</code>. When calling <code>GetMetricStatistics</code>, you must specify either <code>Statistics</code> or <code>ExtendedStatistics</code>, but not both.<br/>Period:  The granularity, in seconds, of the returned data points. For metrics with regular resolution, a period can be as short as one minute (60 seconds) and must be a multiple of 60. For high-resolution metrics that are collected at intervals of less than one minute, the period can be 1, 5, 10, 30, 60, or any multiple of 60. High-resolution metrics are those metrics stored by a <code>PutMetricData</code> call that includes a <code>StorageResolution</code> of 1 second. If the <code>StartTime</code> parameter specifies a time stamp that is greater than 3 hours ago, you must specify the period as follows or no data points in that time range is returned: <ul> <li> Start time between 3 hours and 15 days ago - Use a multiple of 60 seconds (1 minute). </li> <li> Start time between 15 and 63 days ago - Use a multiple of 300 seconds (5 minutes). </li> <li> Start time greater than 63 days ago - Use a multiple of 3600 seconds (1 hour). </li> </ul><br/>EndTime:  The time stamp that determines the last data point to return. The value specified is exclusive; results include data points up to the specified time stamp. The time stamp must be in ISO 8601 UTC format (for example, 2016-10-10T23:00:00Z).<br/>StartTime:  The time stamp that determines the first data point to return. Start times are evaluated relative to the time that CloudWatch receives the request. The value specified is inclusive; results include data points with the specified time stamp. The time stamp must be in ISO 8601 UTC format (for example, 2016-10-03T23:00:00Z). CloudWatch rounds the specified time stamp as follows: <ul> <li> Start time less than 15 days ago - Round down to the nearest whole minute. For example, 12:32:34 is rounded down to 12:32:00. </li> <li> Start time between 15 and 63 days ago - Round down to the nearest 5-minute clock interval. For example, 12:32:34 is rounded down to 12:30:00. </li> <li> Start time greater than 63 days ago - Round down to the nearest 1-hour clock interval. For example, 12:32:34 is rounded down to 12:00:00. </li> </ul> If you set <code>Period</code> to 5, 10, or 30, the start time of your request is rounded down to the nearest time that corresponds to even 5-, 10-, or 30-second divisions of a minute. For example, if you make a query at (HH:mm:ss) 01:05:23 for the previous 10-second period, the start time of your request is rounded down and you receive data from 01:05:10 to 01:05:20. If you make a query at 15:07:17 for the previous 5 minutes of data, using a period of 5 seconds, you receive data timestamped between 15:02:15 and 15:07:15. <br/>Dimensions:  The dimensions. If the metric contains multiple dimensions, you must include a value for each dimension. CloudWatch treats each unique combination of dimensions as a separate metric. If a specific combination of dimensions was not published, you can't retrieve statistics for it. You must specify the same dimensions that were used when the metrics were created. For an example, see <a href="https://docs.aws.amazon.com/AmazonCloudWatch/latest/monitoring/cloudwatch_concepts.html#dimension-combinations">Dimension Combinations</a> in the <i>Amazon CloudWatch User Guide</i>. For more information about specifying dimensions, see <a href="https://docs.aws.amazon.com/AmazonCloudWatch/latest/monitoring/publishingMetrics.html">Publishing Metrics</a> in the <i>Amazon CloudWatch User Guide</i>.<br/>MetricName:  The name of the metric, with or without spaces.<br/>Namespace:  The namespace of the metric, with or without spaces.<br/>
        :return: Datapoints:  The data points for the specified metric.<br/>Label:  A label for the specified metric.<br/>        """
        pass

    def get_metric_widget_image(self, MetricWidget: str, OutputFormat: str=None) -> GetMetricWidgetImageOutput:
        """You can use the <code>GetMetricWidgetImage</code> API to retrieve a snapshot graph of one or more Amazon CloudWatch metrics as a bitmap image. You can then embed this image into your services and products, such as wiki pages, reports, and documents. You could also retrieve images regularly, such as every minute, and create your own custom live dashboard. The graph you retrieve can include all CloudWatch metric graph features, including metric math and horizontal and vertical annotations. There is a limit of 20 transactions per second for this API. Each <code>GetMetricWidgetImage</code> action has the following limits: <ul> <li> As many as 100 metrics in the graph. </li> <li> Up to 100 KB uncompressed payload. </li> </ul>

        :param self:
        :param OutputFormat:  The format of the resulting image. Only PNG images are supported. The default is <code>png</code>. If you specify <code>png</code>, the API returns an HTTP response with the content-type set to <code>text/xml</code>. The image data is in a <code>MetricWidgetImage</code> field. For example:  <code> &lt;GetMetricWidgetImageResponse xmlns=&lt;URLstring&gt;&gt;</code>   <code> &lt;GetMetricWidgetImageResult&gt;</code>   <code> &lt;MetricWidgetImage&gt;</code>   <code> iVBORw0KGgoAAAANSUhEUgAAAlgAAAGQEAYAAAAip...</code>   <code> &lt;/MetricWidgetImage&gt;</code>   <code> &lt;/GetMetricWidgetImageResult&gt;</code>   <code> &lt;ResponseMetadata&gt;</code>   <code> &lt;RequestId&gt;6f0d4192-4d42-11e8-82c1-f539a07e0e3b&lt;/RequestId&gt;</code>   <code> &lt;/ResponseMetadata&gt;</code>   <code>&lt;/GetMetricWidgetImageResponse&gt;</code>  The <code>image/png</code> setting is intended only for custom HTTP requests. For most use cases, and all actions using an AWS SDK, you should use <code>png</code>. If you specify <code>image/png</code>, the HTTP response has a content-type set to <code>image/png</code>, and the body of the response is a PNG image. <br/>MetricWidget:  A JSON string that defines the bitmap graph to be retrieved. The string includes the metrics to include in the graph, statistics, annotations, title, axis limits, and so on. You can include only one <code>MetricWidget</code> parameter in each <code>GetMetricWidgetImage</code> call. For more information about the syntax of <code>MetricWidget</code> see <a>CloudWatch-Metric-Widget-Structure</a>. If any metric on the graph could not load all the requested data points, an orange triangle with an exclamation point appears next to the graph legend.<br/>
        :return: MetricWidgetImage:  The image of the graph, in the output format specified.<br/>        """
        pass

    def list_dashboards(self, DashboardNamePrefix: str=None, NextToken: str=None) -> ListDashboardsOutput:
        """Returns a list of the dashboards for your account. If you include <code>DashboardNamePrefix</code>, only those dashboards with names starting with the prefix are listed. Otherwise, all dashboards in your account are listed.   <code>ListDashboards</code> returns up to 1000 results on one page. If there are more than 1000 dashboards, you can call <code>ListDashboards</code> again and include the value you received for <code>NextToken</code> in the first call, to receive the next 1000 results.

        :param self:
        :param NextToken:  The token returned by a previous call to indicate that there is more data available.<br/>DashboardNamePrefix:  If you specify this parameter, only the dashboards with names starting with the specified string are listed. The maximum length is 255, and valid characters are A-Z, a-z, 0-9, ".", "-", and "_". <br/>
        :return: NextToken:  The token that marks the start of the next batch of returned results.<br/>DashboardEntries:  The list of matching dashboards.<br/>        """
        pass

    def list_metrics(self, Namespace: str=None, MetricName: str=None, Dimensions: List[DimensionFilter]=None, NextToken: str=None) -> ListMetricsOutput:
        """List the specified metrics. You can use the returned metrics with <a>GetMetricData</a> or <a>GetMetricStatistics</a> to obtain statistical data. Up to 500 results are returned for any one call. To retrieve additional results, use the returned token with subsequent calls. After you create a metric, allow up to fifteen minutes before the metric appears. Statistics about the metric, however, are available sooner using <a>GetMetricData</a> or <a>GetMetricStatistics</a>.

        :param self:
        :param NextToken:  The token returned by a previous call to indicate that there is more data available.<br/>Dimensions:  The dimensions to filter against.<br/>MetricName:  The name of the metric to filter against.<br/>Namespace:  The namespace to filter against.<br/>
        :return: NextToken:  The token that marks the start of the next batch of returned results.<br/>Metrics:  The metrics.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceARN: str) -> ListTagsForResourceOutput:
        """Displays the tags associated with a CloudWatch resource. Alarms support tagging.

        :param self:
        :param ResourceARN:  The ARN of the CloudWatch resource that you want to view tags for. For more information on ARN format, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#arn-syntax-cloudwatch">Example ARNs</a> in the <i>Amazon Web Services General Reference</i>.<br/>
        :return: Tags:  The list of tag keys and values associated with the resource you specified.<br/>        """
        pass

    def put_anomaly_detector(self, Stat: str, MetricName: str, Namespace: str, Dimensions: List[Dimension]=None, Configuration: AnomalyDetectorConfiguration=None) -> PutAnomalyDetectorOutput:
        """Creates an anomaly detection model for a CloudWatch metric. You can use the model to display a band of expected normal values when the metric is graphed. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudWatch/latest/monitoring/CloudWatch_Anomaly_Detection.html">CloudWatch Anomaly Detection</a>.

        :param self:
        :param Configuration:  The configuration specifies details about how the anomaly detection model is to be trained, including time ranges to exclude when training and updating the model. You can specify as many as 10 time ranges. The configuration can also include the time zone to use for the metric. You can in:<br/>Stat:  The statistic to use for the metric and the anomaly detection model.<br/>Dimensions:  The metric dimensions to create the anomaly detection model for.<br/>MetricName:  The name of the metric to create the anomaly detection model for.<br/>Namespace:  The namespace of the metric to create the anomaly detection model for.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricTimezone</i> The time zone to use for the metric. This is useful to enable the model to automatically account for daylight savings time changes if the metric is sensitive to such time changes. To specify a time zone, use the name of the time zone as specified in the standard tz database. For more information, see <a href="https://en.wikipedia.org/wiki/Tz_database">tz database</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExcludedTimeRanges</i> An array of time ranges to exclude from use when the anomaly detection model is trained. Use this to make sure that events that could cause unusual values for the metric, such as deployments, aren't used when CloudWatch creates the model.<br/>
        :return:         """
        pass

    def put_dashboard(self, DashboardBody: str, DashboardName: str) -> PutDashboardOutput:
        """Creates a dashboard if it does not already exist, or updates an existing dashboard. If you update a dashboard, the entire contents are replaced with what you specify here. All dashboards in your account are global, not region-specific. A simple way to create a dashboard using <code>PutDashboard</code> is to copy an existing dashboard. To copy an existing dashboard using the console, you can load the dashboard and then use the View/edit source command in the Actions menu to display the JSON block for that dashboard. Another way to copy a dashboard is to use <code>GetDashboard</code>, and then use the data returned within <code>DashboardBody</code> as the template for the new dashboard when you call <code>PutDashboard</code>. When you create a dashboard with <code>PutDashboard</code>, a good practice is to add a text widget at the top of the dashboard with a message that the dashboard was created by script and should not be changed in the console. This message could also point console users to the location of the <code>DashboardBody</code> script or the CloudFormation template used to create the dashboard.

        :param self:
        :param DashboardBody:  The detailed information about the dashboard in JSON format, including the widgets to include and their location on the dashboard. This parameter is required. For more information about the syntax, see <a>CloudWatch-Dashboard-Body-Structure</a>.<br/>DashboardName:  The name of the dashboard. If a dashboard with this name already exists, this call modifies that dashboard, replacing its current contents. Otherwise, a new dashboard is created. The maximum length is 255, and valid characters are A-Z, a-z, 0-9, "-", and "_". This parameter is required.<br/>
        :return: DashboardValidationMessages:  If the input for <code>PutDashboard</code> was correct and the dashboard was successfully created or modified, this result is empty. If this result includes only warning messages, then the input was valid enough for the dashboard to be created or modified, but some elements of the dashboard may not render. If this result includes error messages, the input was not valid and the operation failed.<br/>        """
        pass

    def put_metric_alarm(self, ComparisonOperator: str, EvaluationPeriods: int, AlarmName: str, AlarmDescription: str=None, ActionsEnabled: ActionsEnabled=None, OKActions: List[ResourceName]=None, AlarmActions: List[ResourceName]=None, InsufficientDataActions: List[ResourceName]=None, MetricName: str=None, Namespace: str=None, Statistic: str=None, ExtendedStatistic: str=None, Dimensions: List[Dimension]=None, Period: int=None, Unit: str=None, DatapointsToAlarm: int=None, Threshold: Threshold=None, TreatMissingData: str=None, EvaluateLowSampleCountPercentile: str=None, Metrics: List[MetricDataQuery]=None, Tags: List[Tag]=None, ThresholdMetricId: str=None):
        """Creates or updates an alarm and associates it with the specified metric, metric math expression, or anomaly detection model. Alarms based on anomaly detection models cannot have Auto Scaling actions. When this operation creates an alarm, the alarm state is immediately set to <code>INSUFFICIENT_DATA</code>. The alarm is then evaluated and its state is set appropriately. Any actions associated with the new state are then executed. When you update an existing alarm, its state is left unchanged, but the update completely overwrites the previous configuration of the alarm. If you are an IAM user, you must have Amazon EC2 permissions for some alarm operations: <ul> <li>  <code>iam:CreateServiceLinkedRole</code> for all alarms with EC2 actions </li> <li>  <code>ec2:DescribeInstanceStatus</code> and <code>ec2:DescribeInstances</code> for all alarms on EC2 instance status metrics </li> <li>  <code>ec2:StopInstances</code> for alarms with stop actions </li> <li>  <code>ec2:TerminateInstances</code> for alarms with terminate actions </li> <li> No specific permissions are needed for alarms with recover actions </li> </ul> If you have read/write permissions for Amazon CloudWatch but not for Amazon EC2, you can still create an alarm, but the stop or terminate actions are not performed. However, if you are later granted the required permissions, the alarm actions that you created earlier are performed. If you are using an IAM role (for example, an EC2 instance profile), you cannot stop or terminate the instance using alarm actions. However, you can still see the alarm state and perform any other actions such as Amazon SNS notifications or Auto Scaling policies. If you are using temporary security credentials granted using AWS STS, you cannot stop or terminate an EC2 instance using alarm actions. The first time you create an alarm in the AWS Management Console, the CLI, or by using the PutMetricAlarm API, CloudWatch creates the necessary service-linked role for you. The service-linked role is called <code>AWSServiceRoleForCloudWatchEvents</code>. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_terms-and-concepts.html#iam-term-service-linked-role">AWS service-linked role</a>.

        :param self:
        :param ThresholdMetricId:  If this is an alarm based on an anomaly detection model, make this value match the ID of the <code>ANOMALY_DETECTION_BAND</code> function. For an example of how to use this parameter, see the <b>Anomaly Detection Model Alarm</b> example on this page. If your alarm uses this parameter, it cannot have Auto Scaling actions.<br/>Tags:  A list of key-value pairs to associate with the alarm. You can associate as many as 50 tags with an alarm. Tags can help you organize and categorize your resources. You can also use them to scope user permissions, by granting a user permission to access or change only resources with certain tag values.<br/>Metrics:  An array of <code>MetricDataQuery</code> structures that enable you to create an alarm based on the result of a metric math expression. Each item in the <code>Metrics</code> array either retrieves a metric or performs a math expression. One item in the <code>Metrics</code> array is the expression that the alarm watches. You designate this expression by setting <code>ReturnValue</code> to true for this object in the array. For more information, see <a>MetricDataQuery</a>. If you use the <code>Metrics</code> parameter, you cannot include the <code>MetricName</code>, <code>Dimensions</code>, <code>Period</code>, <code>Namespace</code>, <code>Statistic</code>, or <code>ExtendedStatistic</code> parameters of <code>PutMetricAlarm</code> in the same operation. Instead, you retrieve the metrics you are using in your math expression as part of the <code>Metrics</code> array.<br/>EvaluateLowSampleCountPercentile:   Used only for alarms based on percentiles. If you specify <code>ignore</code>, the alarm state does not change during periods with too few data points to be statistically significant. If you specify <code>evaluate</code> or omit this parameter, the alarm is always evaluated and possibly changes state no matter how many data points are available. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudWatch/latest/monitoring/AlarmThatSendsEmail.html#percentiles-with-low-samples">Percentile-Based CloudWatch Alarms and Low Data Samples</a>. Valid Values: <code>evaluate | ignore</code> <br/>TreatMissingData:   Sets how this alarm is to handle missing data points. If <code>TreatMissingData</code> is omitted, the default behavior of <code>missing</code> is used. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudWatch/latest/monitoring/AlarmThatSendsEmail.html#alarms-and-missing-data">Configuring How CloudWatch Alarms Treats Missing Data</a>. Valid Values: <code>breaching | notBreaching | ignore | missing</code> <br/>ComparisonOperator:   The arithmetic operation to use when comparing the specified statistic and threshold. The specified statistic value is used as the first operand. The values <code>LessThanLowerOrGreaterThanUpperThreshold</code>, <code>LessThanLowerThreshold</code>, and <code>GreaterThanUpperThreshold</code> are used only for alarms based on anomaly detection models.<br/>Threshold:  The value against which the specified statistic is compared.<br/>DatapointsToAlarm:  The number of datapoints that must be breaching to trigger the alarm. This is used only if you are setting an "M out of N" alarm. In that case, this value is the M. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudWatch/latest/monitoring/AlarmThatSendsEmail.html#alarm-evaluation">Evaluating an Alarm</a> in the <i>Amazon CloudWatch User Guide</i>.<br/>EvaluationPeriods:  The number of periods over which data is compared to the specified threshold. If you are setting an alarm that requires that a number of consecutive data points be breaching to trigger the alarm, this value specifies that number. If you are setting an "M out of N" alarm, this value is the N. An alarm's total current evaluation period can be no longer than one day, so this number multiplied by <code>Period</code> cannot be more than 86,400 seconds.<br/>Unit:  The unit of measure for the statistic. For example, the units for the Amazon EC2 NetworkIn metric are Bytes because NetworkIn tracks the number of bytes that an instance receives on all network interfaces. You can also specify a unit when you create a custom metric. Units help provide conceptual meaning to your data. Metric data points that specify a unit of measure, such as Percent, are aggregated separately. If you specify a unit, you must use a unit that is appropriate for the metric. Otherwise, the CloudWatch alarm can get stuck in the <code>INSUFFICIENT DATA</code> state. <br/>Period:  The length, in seconds, used each time the metric specified in <code>MetricName</code> is evaluated. Valid values are 10, 30, and any multiple of 60. Be sure to specify 10 or 30 only for metrics that are stored by a <code>PutMetricData</code> call with a <code>StorageResolution</code> of 1. If you specify a period of 10 or 30 for a metric that does not have sub-minute resolution, the alarm still attempts to gather data at the period rate that you specify. In this case, it does not receive data for the attempts that do not correspond to a one-minute data resolution, and the alarm may often lapse into INSUFFICENT_DATA status. Specifying 10 or 30 also sets this alarm as a high-resolution alarm, which has a higher charge than other alarms. For more information about pricing, see <a href="https://aws.amazon.com/cloudwatch/pricing/">Amazon CloudWatch Pricing</a>. An alarm's total current evaluation period can be no longer than one day, so <code>Period</code> multiplied by <code>EvaluationPeriods</code> cannot be more than 86,400 seconds.<br/>Dimensions:  The dimensions for the metric specified in <code>MetricName</code>.<br/>ExtendedStatistic:  The percentile statistic for the metric specified in <code>MetricName</code>. Specify a value between p0.0 and p100. When you call <code>PutMetricAlarm</code> and specify a <code>MetricName</code>, you must specify either <code>Statistic</code> or <code>ExtendedStatistic,</code> but not both.<br/>Statistic:  The statistic for the metric specified in <code>MetricName</code>, other than percentile. For percentile statistics, use <code>ExtendedStatistic</code>. When you call <code>PutMetricAlarm</code> and specify a <code>MetricName</code>, you must specify either <code>Statistic</code> or <code>ExtendedStatistic,</code> but not both.<br/>Namespace:  The namespace for the metric associated specified in <code>MetricName</code>.<br/>MetricName:  The name for the metric associated with the alarm. If you are creating an alarm based on a math expression, you cannot specify this parameter, or any of the <code>Dimensions</code>, <code>Period</code>, <code>Namespace</code>, <code>Statistic</code>, or <code>ExtendedStatistic</code> parameters. Instead, you specify all this information in the <code>Metrics</code> array.<br/>InsufficientDataActions:  The actions to execute when this alarm transitions to the <code>INSUFFICIENT_DATA</code> state from any other state. Each action is specified as an Amazon Resource Name (ARN). Valid Values: <code>arn:aws:automate:<i>region</i>:ec2:stop</code> | <code>arn:aws:automate:<i>region</i>:ec2:terminate</code> | <code>arn:aws:automate:<i>region</i>:ec2:recover</code> | <code>arn:aws:automate:<i>region</i>:ec2:reboot</code> | <code>arn:aws:sns:<i>region</i>:<i>account-id</i>:<i>sns-topic-name</i> </code> | <code>arn:aws:autoscaling:<i>region</i>:<i>account-id</i>:scalingPolicy:<i>policy-id</i>autoScalingGroupName/<i>group-friendly-name</i>:policyName/<i>policy-friendly-name</i> </code>  Valid Values (for use with IAM roles): <code>&gt;arn:aws:swf:<i>region</i>:<i>account-id</i>:action/actions/AWS_EC2.InstanceId.Stop/1.0</code> | <code>arn:aws:swf:<i>region</i>:<i>account-id</i>:action/actions/AWS_EC2.InstanceId.Terminate/1.0</code> | <code>arn:aws:swf:<i>region</i>:<i>account-id</i>:action/actions/AWS_EC2.InstanceId.Reboot/1.0</code> <br/>AlarmActions:  The actions to execute when this alarm transitions to the <code>ALARM</code> state from any other state. Each action is specified as an Amazon Resource Name (ARN). Valid Values: <code>arn:aws:automate:<i>region</i>:ec2:stop</code> | <code>arn:aws:automate:<i>region</i>:ec2:terminate</code> | <code>arn:aws:automate:<i>region</i>:ec2:recover</code> | <code>arn:aws:automate:<i>region</i>:ec2:reboot</code> | <code>arn:aws:sns:<i>region</i>:<i>account-id</i>:<i>sns-topic-name</i> </code> | <code>arn:aws:autoscaling:<i>region</i>:<i>account-id</i>:scalingPolicy:<i>policy-id</i>autoScalingGroupName/<i>group-friendly-name</i>:policyName/<i>policy-friendly-name</i> </code>  Valid Values (for use with IAM roles): <code>arn:aws:swf:<i>region</i>:<i>account-id</i>:action/actions/AWS_EC2.InstanceId.Stop/1.0</code> | <code>arn:aws:swf:<i>region</i>:<i>account-id</i>:action/actions/AWS_EC2.InstanceId.Terminate/1.0</code> | <code>arn:aws:swf:<i>region</i>:<i>account-id</i>:action/actions/AWS_EC2.InstanceId.Reboot/1.0</code> <br/>OKActions:  The actions to execute when this alarm transitions to an <code>OK</code> state from any other state. Each action is specified as an Amazon Resource Name (ARN). Valid Values: <code>arn:aws:automate:<i>region</i>:ec2:stop</code> | <code>arn:aws:automate:<i>region</i>:ec2:terminate</code> | <code>arn:aws:automate:<i>region</i>:ec2:recover</code> | <code>arn:aws:automate:<i>region</i>:ec2:reboot</code> | <code>arn:aws:sns:<i>region</i>:<i>account-id</i>:<i>sns-topic-name</i> </code> | <code>arn:aws:autoscaling:<i>region</i>:<i>account-id</i>:scalingPolicy:<i>policy-id</i>autoScalingGroupName/<i>group-friendly-name</i>:policyName/<i>policy-friendly-name</i> </code>  Valid Values (for use with IAM roles): <code>arn:aws:swf:<i>region</i>:<i>account-id</i>:action/actions/AWS_EC2.InstanceId.Stop/1.0</code> | <code>arn:aws:swf:<i>region</i>:<i>account-id</i>:action/actions/AWS_EC2.InstanceId.Terminate/1.0</code> | <code>arn:aws:swf:<i>region</i>:<i>account-id</i>:action/actions/AWS_EC2.InstanceId.Reboot/1.0</code> <br/>ActionsEnabled:  Indicates whether actions should be executed during any changes to the alarm state. The default is TRUE.<br/>AlarmDescription:  The description for the alarm.<br/>AlarmName:  The name for the alarm. This name must be unique within your AWS account.<br/>
        :return:         """
        pass

    def put_metric_data(self, MetricData: List[MetricDatum], Namespace: str):
        """Publishes metric data points to Amazon CloudWatch. CloudWatch associates the data points with the specified metric. If the specified metric does not exist, CloudWatch creates the metric. When CloudWatch creates a metric, it can take up to fifteen minutes for the metric to appear in calls to <a>ListMetrics</a>. You can publish either individual data points in the <code>Value</code> field, or arrays of values and the number of times each value occurred during the period by using the <code>Values</code> and <code>Counts</code> fields in the <code>MetricDatum</code> structure. Using the <code>Values</code> and <code>Counts</code> method enables you to publish up to 150 values per metric with one <code>PutMetricData</code> request, and supports retrieving percentile statistics on this data. Each <code>PutMetricData</code> request is limited to 40 KB in size for HTTP POST requests. You can send a payload compressed by gzip. Each request is also limited to no more than 20 different metrics. Although the <code>Value</code> parameter accepts numbers of type <code>Double</code>, CloudWatch rejects values that are either too small or too large. Values must be in the range of 8.515920e-109 to 1.174271e+108 (Base 10) or 2e-360 to 2e360 (Base 2). In addition, special values (for example, NaN, +Infinity, -Infinity) are not supported. You can use up to 10 dimensions per metric to further clarify what data the metric collects. Each dimension consists of a Name and Value pair. For more information about specifying dimensions, see <a href="https://docs.aws.amazon.com/AmazonCloudWatch/latest/monitoring/publishingMetrics.html">Publishing Metrics</a> in the <i>Amazon CloudWatch User Guide</i>. Data points with time stamps from 24 hours ago or longer can take at least 48 hours to become available for <a>GetMetricData</a> or <a>GetMetricStatistics</a> from the time they are submitted. CloudWatch needs raw data points to calculate percentile statistics. If you publish data using a statistic set instead, you can only retrieve percentile statistics for this data if one of the following conditions is true: <ul> <li> The <code>SampleCount</code> value of the statistic set is 1 and <code>Min</code>, <code>Max</code>, and <code>Sum</code> are all equal. </li> <li> The <code>Min</code> and <code>Max</code> are equal, and <code>Sum</code> is equal to <code>Min</code> multiplied by <code>SampleCount</code>. </li> </ul>

        :param self:
        :param MetricData:  The data for the metric. The array can include no more than 20 metrics per call.<br/>Namespace:  The namespace for the metric data. You cannot specify a namespace that begins with "AWS/". Namespaces that begin with "AWS/" are reserved for use by Amazon Web Services products.<br/>
        :return:         """
        pass

    def set_alarm_state(self, StateReason: str, StateValue: str, AlarmName: str, StateReasonData: str=None):
        """Temporarily sets the state of an alarm for testing purposes. When the updated state differs from the previous value, the action configured for the appropriate state is invoked. For example, if your alarm is configured to send an Amazon SNS message when an alarm is triggered, temporarily changing the alarm state to <code>ALARM</code> sends an SNS message. The alarm returns to its actual state (often within seconds). Because the alarm state change happens quickly, it is typically only visible in the alarm's <b>History</b> tab in the Amazon CloudWatch console or through <a>DescribeAlarmHistory</a>.

        :param self:
        :param StateReasonData:  The reason that this alarm is set to this specific state, in JSON format.<br/>StateReason:  The reason that this alarm is set to this specific state, in text format.<br/>StateValue:  The value of the state.<br/>AlarmName:  The name for the alarm. This name must be unique within the AWS account. The maximum length is 255 characters.<br/>
        :return:         """
        pass

    def tag_resource(self, Tags: List[Tag], ResourceARN: str) -> TagResourceOutput:
        """Assigns one or more tags (key-value pairs) to the specified CloudWatch resource. Tags can help you organize and categorize your resources. You can also use them to scope user permissions, by granting a user permission to access or change only resources with certain tag values. In CloudWatch, alarms can be tagged. Tags don't have any semantic meaning to AWS and are interpreted strictly as strings of characters. You can use the <code>TagResource</code> action with a resource that already has tags. If you specify a new tag key for the resource, this tag is appended to the list of tags associated with the resource. If you specify a tag key that is already associated with the resource, the new tag value that you specify replaces the previous value for that tag. You can associate as many as 50 tags with a resource.

        :param self:
        :param Tags:  The list of key-value pairs to associate with the resource.<br/>ResourceARN:  The ARN of the CloudWatch resource that you're adding tags to. For more information on ARN format, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#arn-syntax-cloudwatch">Example ARNs</a> in the <i>Amazon Web Services General Reference</i>.<br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[TagKey], ResourceARN: str) -> UntagResourceOutput:
        """Removes one or more tags from the specified resource.

        :param self:
        :param TagKeys:  The list of tag keys to remove from the resource.<br/>ResourceARN:  The ARN of the CloudWatch resource that you're removing tags from. For more information on ARN format, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#arn-syntax-cloudwatch">Example ARNs</a> in the <i>Amazon Web Services General Reference</i>.<br/>
        :return:         """
        pass

    class CloudwatchResource:
        
        class Alarm:
            """Creates a Alarm resource.::

  alarm = cloudwatch.Alarm('name')

:type name: string
:param name: The Alarm's name identifier. This **must** be set.

:rtype: :py:class:`CloudWatch.Alarm`
:returns: A Alarm resource"""

            def __init__(self, name):
                pass
            
            class ActionsEnabled(object):
                pass

            class ResourceName(object):
                pass

            class Threshold(object):
                pass

            class Dimension(Mapping):
                pass

            class MetricDataQuery(Mapping):
                pass

            class Timestamp(object):
                pass
        
            alarm_name: str=None
            alarm_arn: str=None
            alarm_description: str=None
            alarm_configuration_updated_timestamp: Timestamp=None
            actions_enabled: ActionsEnabled=None
            ok_actions: List[ResourceName]=None
            alarm_actions: List[ResourceName]=None
            insufficient_data_actions: List[ResourceName]=None
            state_value: str=None
            state_reason: str=None
            state_reason_data: str=None
            state_updated_timestamp: Timestamp=None
            metric_name: str=None
            namespace: str=None
            statistic: str=None
            extended_statistic: str=None
            dimensions: List[Dimension]=None
            period: int=None
            unit: str=None
            evaluation_periods: int=None
            datapoints_to_alarm: int=None
            threshold: Threshold=None
            comparison_operator: str=None
            treat_missing_data: str=None
            evaluate_low_sample_count_percentile: str=None
            metrics: List[MetricDataQuery]=None
            threshold_metric_id: str=None
            
            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def describe_history(self, **kwargs):
                # type: (object) -> dict
                return {}

            def disable_actions(self, **kwargs):
                # type: (object) -> dict
                return {}

            def enable_actions(self, **kwargs):
                # type: (object) -> dict
                return {}

            def set_state(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class Metric:
            """Creates a Metric resource.::

  metric = cloudwatch.Metric('namespace','name')

:type namespace: string
:param namespace: The Metric's namespace identifier. This **must** be set.
:type name: string
:param name: The Metric's name identifier. This **must** be set.

:rtype: :py:class:`CloudWatch.Metric`
:returns: A Metric resource"""

            def __init__(self, namespace, name):
                pass
            
            class Dimension(Mapping):
                pass
        
            namespace: str=None
            metric_name: str=None
            dimensions: List[Dimension]=None
            
            def get_statistics(self, **kwargs):
                # type: (object) -> dict
                return {}

            def put_alarm(self, **kwargs):
                # type: (object) -> Alarm
                return Alarm()

            def put_data(self, **kwargs):
                # type: (object) -> dict
                return {}


        def get_available_subresources(self) -> List[str]:
            pass

        class AlarmResourceCollection(List[Alarm], ResourceCollection):
            pass

        alarms: AlarmResourceCollection = None
        class MetricResourceCollection(List[Metric], ResourceCollection):
            pass

        metrics: MetricResourceCollection = None




class CodeBuild(BaseClient):
    """<fullname>AWS CodeBuild</fullname> AWS CodeBuild is a fully managed build service in the cloud. AWS CodeBuild compiles your source code, runs unit tests, and produces artifacts that are ready to deploy. AWS CodeBuild eliminates the need to provision, manage, and scale your own build servers. It provides prepackaged build environments for the most popular programming languages and build tools, such as Apache Maven, Gradle, and more. You can also fully customize build environments in AWS CodeBuild to use your own build tools. AWS CodeBuild scales automatically to meet peak build requests. You pay only for the build time you consume. For more information about AWS CodeBuild, see the <i>AWS CodeBuild User Guide</i>. AWS CodeBuild supports these operations: <ul> <li>  <code>BatchDeleteBuilds</code>: Deletes one or more builds. </li> <li>  <code>BatchGetProjects</code>: Gets information about one or more build projects. A <i>build project</i> defines how AWS CodeBuild runs a build. This includes information such as where to get the source code to build, the build environment to use, the build commands to run, and where to store the build output. A <i>build environment</i> is a representation of operating system, programming language runtime, and tools that AWS CodeBuild uses to run a build. You can add tags to build projects to help manage your resources and costs. </li> <li>  <code>CreateProject</code>: Creates a build project. </li> <li>  <code>CreateWebhook</code>: For an existing AWS CodeBuild build project that has its source code stored in a GitHub or Bitbucket repository, enables AWS CodeBuild to start rebuilding the source code every time a code change is pushed to the repository. </li> <li>  <code>UpdateWebhook</code>: Changes the settings of an existing webhook. </li> <li>  <code>DeleteProject</code>: Deletes a build project. </li> <li>  <code>DeleteWebhook</code>: For an existing AWS CodeBuild build project that has its source code stored in a GitHub or Bitbucket repository, stops AWS CodeBuild from rebuilding the source code every time a code change is pushed to the repository. </li> <li>  <code>ListProjects</code>: Gets a list of build project names, with each build project name representing a single build project. </li> <li>  <code>UpdateProject</code>: Changes the settings of an existing build project. </li> <li>  <code>BatchGetBuilds</code>: Gets information about one or more builds. </li> <li>  <code>ListBuilds</code>: Gets a list of build IDs, with each build ID representing a single build. </li> <li>  <code>ListBuildsForProject</code>: Gets a list of build IDs for the specified build project, with each build ID representing a single build. </li> <li>  <code>StartBuild</code>: Starts running a build. </li> <li>  <code>StopBuild</code>: Attempts to stop running a build. </li> <li>  <code>ListCuratedEnvironmentImages</code>: Gets information about Docker images that are managed by AWS CodeBuild. </li> <li>  <code>DeleteSourceCredentials</code>: Deletes a set of GitHub, GitHub Enterprise, or Bitbucket source credentials. </li> <li>  <code>ImportSourceCredentials</code>: Imports the source repository credentials for an AWS CodeBuild project that has its source code stored in a GitHub, GitHub Enterprise, or Bitbucket repository. </li> <li>  <code>ListSourceCredentials</code>: Returns a list of <code>SourceCredentialsInfo</code> objects. Each <code>SourceCredentialsInfo</code> object includes the authentication type, token ARN, and type of source provider for one set of credentials. </li> </ul>"""


    class ListBuildsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_builds` for valid parameters"""
            pass
    class ListProjectsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_projects` for valid parameters"""
            pass
    class ListBuildsForProjectPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_builds_for_project` for valid parameters"""
            pass


    class BatchDeleteBuildsInput(Mapping):
        pass
    
    class BatchDeleteBuildsOutput(Mapping):
        pass
    
    class NonEmptyString(object):
        pass
    
    class BatchGetBuildsInput(Mapping):
        pass
    
    class BatchGetBuildsOutput(Mapping):
        pass
    
    class BatchGetProjectsInput(Mapping):
        pass
    
    class BatchGetProjectsOutput(Mapping):
        pass
    
    class CreateProjectInput(Mapping):
        pass
    
    class CreateProjectOutput(Mapping):
        pass
    
    class ProjectSource(Mapping):
        pass
    
    class ProjectSourceVersion(Mapping):
        pass
    
    class ProjectArtifacts(Mapping):
        pass
    
    class ProjectCache(Mapping):
        pass
    
    class ProjectEnvironment(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class VpcConfig(Mapping):
        pass
    
    class WrapperBoolean(object):
        pass
    
    class LogsConfig(Mapping):
        pass
    
    class CreateWebhookInput(Mapping):
        pass
    
    class CreateWebhookOutput(Mapping):
        pass
    
    class FilterGroup(object):
        pass
    
    class DeleteProjectInput(Mapping):
        pass
    
    class DeleteProjectOutput(Mapping):
        pass
    
    class DeleteSourceCredentialsInput(Mapping):
        pass
    
    class DeleteSourceCredentialsOutput(Mapping):
        pass
    
    class DeleteWebhookInput(Mapping):
        pass
    
    class DeleteWebhookOutput(Mapping):
        pass
    
    class ImportSourceCredentialsInput(Mapping):
        pass
    
    class ImportSourceCredentialsOutput(Mapping):
        pass
    
    class InvalidateProjectCacheInput(Mapping):
        pass
    
    class InvalidateProjectCacheOutput(Mapping):
        pass
    
    class ListBuildsInput(Mapping):
        pass
    
    class ListBuildsOutput(Mapping):
        pass
    
    class ListBuildsForProjectInput(Mapping):
        pass
    
    class ListBuildsForProjectOutput(Mapping):
        pass
    
    class ListCuratedEnvironmentImagesInput(Mapping):
        pass
    
    class ListCuratedEnvironmentImagesOutput(Mapping):
        pass
    
    class ListProjectsInput(Mapping):
        pass
    
    class ListProjectsOutput(Mapping):
        pass
    
    class ListSourceCredentialsInput(Mapping):
        pass
    
    class ListSourceCredentialsOutput(Mapping):
        pass
    
    class StartBuildInput(Mapping):
        pass
    
    class StartBuildOutput(Mapping):
        pass
    
    class EnvironmentVariable(Mapping):
        pass
    
    class SourceAuth(Mapping):
        pass
    
    class GitSubmodulesConfig(Mapping):
        pass
    
    class RegistryCredential(Mapping):
        pass
    
    class StopBuildInput(Mapping):
        pass
    
    class StopBuildOutput(Mapping):
        pass
    
    class UpdateProjectInput(Mapping):
        pass
    
    class UpdateProjectOutput(Mapping):
        pass
    
    class UpdateWebhookInput(Mapping):
        pass
    
    class UpdateWebhookOutput(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    def batch_delete_builds(self, ids: List[NonEmptyString]) -> BatchDeleteBuildsOutput:
        """Deletes one or more builds.

        :param self:
        :param ids:  The IDs of the builds to delete.<br/>
        :return: buildsNotDeleted:  Information about any builds that could not be successfully deleted.<br/>buildsDeleted:  The IDs of the builds that were successfully deleted.<br/>        """
        pass

    def batch_get_builds(self, ids: List[NonEmptyString]) -> BatchGetBuildsOutput:
        """Gets information about builds.

        :param self:
        :param ids:  The IDs of the builds.<br/>
        :return: buildsNotFound:  The IDs of builds for which information could not be found.<br/>builds:  Information about the requested builds.<br/>        """
        pass

    def batch_get_projects(self, names: List[NonEmptyString]) -> BatchGetProjectsOutput:
        """Gets information about build projects.

        :param self:
        :param names:  The names of the build projects.<br/>
        :return: projectsNotFound:  The names of build projects for which information could not be found.<br/>projects:  Information about the requested build projects.<br/>        """
        pass

    def create_project(self, serviceRole: str, environment: ProjectEnvironment, artifacts: ProjectArtifacts, source: ProjectSource, name: str, description: str=None, secondarySources: List[ProjectSource]=None, sourceVersion: str=None, secondarySourceVersions: List[ProjectSourceVersion]=None, secondaryArtifacts: List[ProjectArtifacts]=None, cache: ProjectCache=None, timeoutInMinutes: int=None, queuedTimeoutInMinutes: int=None, encryptionKey: str=None, tags: List[Tag]=None, vpcConfig: VpcConfig=None, badgeEnabled: WrapperBoolean=None, logsConfig: LogsConfig=None) -> CreateProjectOutput:
        """Creates a build project.

        :param self:
        :param logsConfig:   Information about logs for the build project. These can be logs in Amazon CloudWatch Logs, logs uploaded to a specified S3 bucket, or both. :<br/>badgeEnabled:  Set this to true to generate a publicly accessible URL for your project's build badge.<br/>vpcConfig:  VpcConfig enables AWS CodeBuild to access resources in an Amazon VPC.:<br/>tags:  A set of tags for this build project. These tags are available for use by AWS services that support AWS CodeBuild build project tags.<br/>encryptionKey:  The AWS Key Management Service (AWS KMS) customer master key (CMK) to be used for encrypting the build output artifacts. <note>  You can use a cross-account KMS key to encrypt the build output artifacts if your service role has permission to that key.  </note> You can specify either the Amazon Resource Name (ARN) of the CMK or, if available, the CMK's alias (using the format <code>alias/<i>alias-name</i> </code>).<br/>queuedTimeoutInMinutes:   The number of minutes a build is allowed to be queued before it times out. <br/>timeoutInMinutes:  How long, in minutes, from 5 to 480 (8 hours), for AWS CodeBuild to wait before it times out any build that has not been marked as completed. The default is 60 minutes.<br/>serviceRole:  The ARN of the AWS Identity and Access Management (IAM) role that enables AWS CodeBuild to interact with dependent AWS services on behalf of the AWS account.<br/>environment:  Information about the build environment for the build project.:<br/>cache:  Stores recently used information so that it can be quickly accessed at a later time.:<br/>secondaryArtifacts:   An array of <code>ProjectArtifacts</code> objects. <br/>artifacts:  Information about the build output artifacts for the build project.:<br/>secondarySourceVersions:   An array of <code>ProjectSourceVersion</code> objects. If <code>secondarySourceVersions</code> is specified at the build level, then they take precedence over these <code>secondarySourceVersions</code> (at the project level). <br/>sourceVersion:   A version of the build input to be built for this project. If not specified, the latest version is used. If specified, it must be one of:  <ul> <li> For AWS CodeCommit: the commit ID to use. </li> <li> For GitHub: the commit ID, pull request ID, branch name, or tag name that corresponds to the version of the source code you want to build. If a pull request ID is specified, it must use the format <code>pr/pull-request-ID</code> (for example <code>pr/25</code>). If a branch name is specified, the branch's HEAD commit ID is used. If not specified, the default branch's HEAD commit ID is used. </li> <li> For Bitbucket: the commit ID, branch name, or tag name that corresponds to the version of the source code you want to build. If a branch name is specified, the branch's HEAD commit ID is used. If not specified, the default branch's HEAD commit ID is used. </li> <li> For Amazon Simple Storage Service (Amazon S3): the version ID of the object that represents the build input ZIP file to use. </li> </ul>  If <code>sourceVersion</code> is specified at the build level, then that version takes precedence over this <code>sourceVersion</code> (at the project level).   For more information, see <a href="https://docs.aws.amazon.com/codebuild/latest/userguide/sample-source-version.html">Source Version Sample with CodeBuild</a> in the <i>AWS CodeBuild User Guide</i>. <br/>secondarySources:   An array of <code>ProjectSource</code> objects. <br/>source:  Information about the build input source code for the build project.:<br/>description:  A description that makes the build project easy to identify.<br/>name:  The name of the build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>sourceIdentifier</i>  An identifier for this project source. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>insecureSsl</i> Enable this flag to ignore SSL warnings while connecting to the project source code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>reportBuildStatus</i>  Set to true to report the status of a build's start and finish to your source provider. This option is valid only when your source provider is GitHub, GitHub Enterprise, or Bitbucket. If this is set and you use a different source provider, an invalidInputException is thrown. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>auth</i> Information about the authorization settings for AWS CodeBuild to access the source code to be built. This information is for the AWS CodeBuild console's use only. Your code should not get or set this information directly.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>auth</b>: Information about the authorization settings for AWS CodeBuild to access the source code to be built. This information is for the AWS CodeBuild console's use only. Your code should not get or set this information directly.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>buildspec</i> The build spec declaration to use for the builds in this build project. If this value is not specified, a build spec must be included along with the source code to be built.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>gitSubmodulesConfig</i>  Information about the Git submodules configuration for the build project. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>gitSubmodulesConfig</b>:  Information about the Git submodules configuration for the build project. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>gitCloneDepth</i> Information about the Git clone depth for the build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> Information about the location of the source code to be built. Valid values include: <ul> <li> For source code settings that are specified in the source action of a pipeline in AWS CodePipeline, <code>location</code> should not be specified. If it is specified, AWS CodePipeline ignores it. This is because AWS CodePipeline uses the settings in a pipeline's source action instead of this value. </li> <li> For source code in an AWS CodeCommit repository, the HTTPS clone URL to the repository that contains the source code and the build spec (for example, <code>https://git-codecommit.<i>region-ID</i>.amazonaws.com/v1/repos/<i>repo-name</i> </code>). </li> <li> For source code in an Amazon Simple Storage Service (Amazon S3) input bucket, one of the following.  <ul> <li>  The path to the ZIP file that contains the source code (for example, <code> <i>bucket-name</i>/<i>path</i>/<i>to</i>/<i>object-name</i>.zip</code>).  </li> <li>  The path to the folder that contains the source code (for example, <code> <i>bucket-name</i>/<i>path</i>/<i>to</i>/<i>source-code</i>/<i>folder</i>/</code>).  </li> </ul> </li> <li> For source code in a GitHub repository, the HTTPS clone URL to the repository that contains the source and the build spec. You must connect your AWS account to your GitHub account. Use the AWS CodeBuild console to start creating a build project. When you use the console to connect (or reconnect) with GitHub, on the GitHub <b>Authorize application</b> page, for <b>Organization access</b>, choose <b>Request access</b> next to each repository you want to allow AWS CodeBuild to have access to, and then choose <b>Authorize application</b>. (After you have connected to your GitHub account, you do not need to finish creating the build project. You can leave the AWS CodeBuild console.) To instruct AWS CodeBuild to use this connection, in the <code>source</code> object, set the <code>auth</code> object's <code>type</code> value to <code>OAUTH</code>. </li> <li> For source code in a Bitbucket repository, the HTTPS clone URL to the repository that contains the source and the build spec. You must connect your AWS account to your Bitbucket account. Use the AWS CodeBuild console to start creating a build project. When you use the console to connect (or reconnect) with Bitbucket, on the Bitbucket <b>Confirm access to your account</b> page, choose <b>Grant access</b>. (After you have connected to your Bitbucket account, you do not need to finish creating the build project. You can leave the AWS CodeBuild console.) To instruct AWS CodeBuild to use this connection, in the <code>source</code> object, set the <code>auth</code> object's <code>type</code> value to <code>OAUTH</code>. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of repository that contains the source code to be built. Valid values include: <ul> <li>  <code>BITBUCKET</code>: The source code is in a Bitbucket repository. </li> <li>  <code>CODECOMMIT</code>: The source code is in an AWS CodeCommit repository. </li> <li>  <code>CODEPIPELINE</code>: The source code settings are specified in the source action of a pipeline in AWS CodePipeline. </li> <li>  <code>GITHUB</code>: The source code is in a GitHub repository. </li> <li>  <code>NO_SOURCE</code>: The project does not have input source code. </li> <li>  <code>S3</code>: The source code is in an Amazon Simple Storage Service (Amazon S3) input bucket. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>fetchSubmodules</i>  Set to true to fetch Git submodules for your AWS CodeBuild build project. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>resource</i> The resource value that applies to the specified authorization type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> <note>  This data type is deprecated and is no longer accurate or used.  </note> The authorization type to use. The only valid value is <code>OAUTH</code>, which represents the OAuth authorization type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>artifactIdentifier</i>  An identifier for this artifact definition. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>encryptionDisabled</i>  Set to true if you do not want your output artifacts encrypted. This option is valid only if your artifacts type is Amazon Simple Storage Service (Amazon S3). If this is set with another artifacts type, an invalidInputException is thrown. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>overrideArtifactName</i>  If this flag is set, a name specified in the build spec file overrides the artifact name. The name specified in a build spec file is calculated at build time and uses the Shell Command Language. For example, you can append a date and time to your artifact name so that it is always unique. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>packaging</i> The type of build output artifact to create: <ul> <li> If <code>type</code> is set to <code>CODEPIPELINE</code>, AWS CodePipeline ignores this value if specified. This is because AWS CodePipeline manages its build output artifacts instead of AWS CodeBuild. </li> <li> If <code>type</code> is set to <code>NO_ARTIFACTS</code>, this value is ignored if specified, because no build output is produced. </li> <li> If <code>type</code> is set to <code>S3</code>, valid values include: <ul> <li>  <code>NONE</code>: AWS CodeBuild creates in the output bucket a folder that contains the build output. This is the default if <code>packaging</code> is not specified. </li> <li>  <code>ZIP</code>: AWS CodeBuild creates in the output bucket a ZIP file that contains the build output. </li> </ul> </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> Along with <code>path</code> and <code>namespaceType</code>, the pattern that AWS CodeBuild uses to name and store the output artifact: <ul> <li> If <code>type</code> is set to <code>CODEPIPELINE</code>, AWS CodePipeline ignores this value if specified. This is because AWS CodePipeline manages its build output names instead of AWS CodeBuild. </li> <li> If <code>type</code> is set to <code>NO_ARTIFACTS</code>, this value is ignored if specified, because no build output is produced. </li> <li> If <code>type</code> is set to <code>S3</code>, this is the name of the output artifact object. If you set the name to be a forward slash ("/"), the artifact is stored in the root of the output bucket. </li> </ul> For example: <ul> <li>  If <code>path</code> is set to <code>MyArtifacts</code>, <code>namespaceType</code> is set to <code>BUILD_ID</code>, and <code>name</code> is set to <code>MyArtifact.zip</code>, then the output artifact is stored in <code>MyArtifacts/<i>build-ID</i>/MyArtifact.zip</code>.  </li> <li>  If <code>path</code> is empty, <code>namespaceType</code> is set to <code>NONE</code>, and <code>name</code> is set to "<code>/</code>", the output artifact is stored in the root of the output bucket.  </li> <li>  If <code>path</code> is set to <code>MyArtifacts</code>, <code>namespaceType</code> is set to <code>BUILD_ID</code>, and <code>name</code> is set to "<code>/</code>", the output artifact is stored in <code>MyArtifacts/<i>build-ID</i> </code>.  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>namespaceType</i> Along with <code>path</code> and <code>name</code>, the pattern that AWS CodeBuild uses to determine the name and location to store the output artifact: <ul> <li> If <code>type</code> is set to <code>CODEPIPELINE</code>, AWS CodePipeline ignores this value if specified. This is because AWS CodePipeline manages its build output names instead of AWS CodeBuild. </li> <li> If <code>type</code> is set to <code>NO_ARTIFACTS</code>, this value is ignored if specified, because no build output is produced. </li> <li> If <code>type</code> is set to <code>S3</code>, valid values include: <ul> <li>  <code>BUILD_ID</code>: Include the build ID in the location of the build output artifact. </li> <li>  <code>NONE</code>: Do not include the build ID. This is the default if <code>namespaceType</code> is not specified. </li> </ul> </li> </ul> For example, if <code>path</code> is set to <code>MyArtifacts</code>, <code>namespaceType</code> is set to <code>BUILD_ID</code>, and <code>name</code> is set to <code>MyArtifact.zip</code>, the output artifact is stored in <code>MyArtifacts/<i>build-ID</i>/MyArtifact.zip</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>path</i> Along with <code>namespaceType</code> and <code>name</code>, the pattern that AWS CodeBuild uses to name and store the output artifact: <ul> <li> If <code>type</code> is set to <code>CODEPIPELINE</code>, AWS CodePipeline ignores this value if specified. This is because AWS CodePipeline manages its build output names instead of AWS CodeBuild. </li> <li> If <code>type</code> is set to <code>NO_ARTIFACTS</code>, this value is ignored if specified, because no build output is produced. </li> <li> If <code>type</code> is set to <code>S3</code>, this is the path to the output artifact. If <code>path</code> is not specified, <code>path</code> is not used. </li> </ul> For example, if <code>path</code> is set to <code>MyArtifacts</code>, <code>namespaceType</code> is set to <code>NONE</code>, and <code>name</code> is set to <code>MyArtifact.zip</code>, the output artifact is stored in the output bucket at <code>MyArtifacts/MyArtifact.zip</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> Information about the build output artifact location: <ul> <li> If <code>type</code> is set to <code>CODEPIPELINE</code>, AWS CodePipeline ignores this value if specified. This is because AWS CodePipeline manages its build output locations instead of AWS CodeBuild. </li> <li> If <code>type</code> is set to <code>NO_ARTIFACTS</code>, this value is ignored if specified, because no build output is produced. </li> <li> If <code>type</code> is set to <code>S3</code>, this is the name of the output bucket. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of build output artifact. Valid values include: <ul> <li>  <code>CODEPIPELINE</code>: The build project has build output generated through AWS CodePipeline. </li> <li>  <code>NO_ARTIFACTS</code>: The build project does not produce any build output. </li> <li>  <code>S3</code>: The build project stores build output in Amazon Simple Storage Service (Amazon S3). </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>modes</i>  If you use a <code>LOCAL</code> cache, the local cache mode. You can use one or more local cache modes at the same time.  <ul> <li>  <code>LOCAL_SOURCE_CACHE</code> mode caches Git metadata for primary and secondary sources. After the cache is created, subsequent builds pull only the change between commits. This mode is a good choice for projects with a clean working directory and a source that is a large Git repository. If you choose this option and your project does not use a Git repository (GitHub, GitHub Enterprise, or Bitbucket), the option is ignored.  </li> <li>  <code>LOCAL_DOCKER_LAYER_CACHE</code> mode caches existing Docker layers. This mode is a good choice for projects that build or pull large Docker images. It can prevent the performance issues caused by pulling large Docker images down from the network.  <note> <ul> <li>  You can use a Docker layer cache in the Linux environment only.  </li> <li>  The <code>privileged</code> flag must be set so that your project has the required Docker permissions.  </li> <li>  You should consider the security implications before you use a Docker layer cache.  </li> </ul> </note> </li> </ul> <ul> <li>  <code>LOCAL_CUSTOM_CACHE</code> mode caches directories you specify in the buildspec file. This mode is a good choice if your build scenario is not suited to one of the other three local cache modes. If you use a custom cache:  <ul> <li>  Only directories can be specified for caching. You cannot specify individual files.  </li> <li>  Symlinks are used to reference cached directories.  </li> <li>  Cached directories are linked to your build before it downloads its project sources. Cached items are overriden if a source item has the same name. Directories are specified using cache paths in the buildspec file.  </li> </ul> </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> Information about the cache location:  <ul> <li>  <code>NO_CACHE</code> or <code>LOCAL</code>: This value is ignored. </li> <li>  <code>S3</code>: This is the S3 bucket name/prefix. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of cache used by the build project. Valid values include: <ul> <li>  <code>NO_CACHE</code>: The build project does not use any cache. </li> <li>  <code>S3</code>: The build project reads and writes from and to S3. </li> <li>  <code>LOCAL</code>: The build project stores a cache locally on a build host that is only available to that build host. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>imagePullCredentialsType</i>  The type of credentials AWS CodeBuild uses to pull images in your build. There are two valid values:  <ul> <li>  <code>CODEBUILD</code> specifies that AWS CodeBuild uses its own credentials. This requires that you modify your ECR repository policy to trust AWS CodeBuild's service principal.  </li> <li>  <code>SERVICE_ROLE</code> specifies that AWS CodeBuild uses your build project's service role.  </li> </ul>  When you use a cross-account or private registry image, you must use SERVICE_ROLE credentials. When you use an AWS CodeBuild curated image, you must use CODEBUILD credentials. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>registryCredential</i>  The credentials for access to a private registry.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>registryCredential</b>:  The credentials for access to a private registry.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>certificate</i> The certificate to use with this build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>privilegedMode</i> Enables running the Docker daemon inside a Docker container. Set to true only if the build project is be used to build Docker images, and the specified build environment image is not provided by AWS CodeBuild with Docker support. Otherwise, all associated builds that attempt to interact with the Docker daemon fail. You must also start the Docker daemon so that builds can interact with it. One way to do this is to initialize the Docker daemon during the install phase of your build spec by running the following build commands. (Do not run these commands if the specified build environment image is provided by AWS CodeBuild with Docker support.) If the operating system's base image is Ubuntu Linux:  <code>- nohup /usr/local/bin/dockerd --host=unix:///var/run/docker.sock --host=tcp://0.0.0.0:2375 --storage-driver=overlay&amp;</code>   <code>- timeout 15 sh -c "until docker info; do echo .; sleep 1; done"</code>  If the operating system's base image is Alpine Linux and the previous command does not work, add the <code>-t</code> argument to <code>timeout</code>:  <code>- nohup /usr/local/bin/dockerd --host=unix:///var/run/docker.sock --host=tcp://0.0.0.0:2375 --storage-driver=overlay&amp;</code>   <code>- timeout -t 15 sh -c "until docker info; do echo .; sleep 1; done"</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>environmentVariables</i> A set of environment variables to make available to builds for this build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>computeType</i> Information about the compute resources the build project uses. Available values include: <ul> <li>  <code>BUILD_GENERAL1_SMALL</code>: Use up to 3 GB memory and 2 vCPUs for builds. </li> <li>  <code>BUILD_GENERAL1_MEDIUM</code>: Use up to 7 GB memory and 4 vCPUs for builds. </li> <li>  <code>BUILD_GENERAL1_LARGE</code>: Use up to 15 GB memory and 8 vCPUs for builds. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>image</i> The image tag or image digest that identifies the Docker image to use for this build project. Use the following formats: <ul> <li> For an image tag: <code>registry/repository:tag</code>. For example, to specify an image with the tag "latest," use <code>registry/repository:latest</code>. </li> <li> For an image digest: <code>registry/repository@digest</code>. For example, to specify an image with the digest "sha256:cbbf2f9a99b47fc460d422812b6a5adff7dfee951d8fa2e4a98caa0382cfbdbf," use <code>registry/repository@sha256:cbbf2f9a99b47fc460d422812b6a5adff7dfee951d8fa2e4a98caa0382cfbdbf</code>. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of build environment to use for related builds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>credentialProvider</i>  The service that created the credentials to access a private Docker registry. The valid value, SECRETS_MANAGER, is for AWS Secrets Manager. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>credential</i>  The Amazon Resource Name (ARN) or name of credentials created using AWS Secrets Manager.  <note>  The <code>credential</code> can use the name of the credentials only if they exist in your current region.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>securityGroupIds</i> A list of one or more security groups IDs in your Amazon VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>subnets</i> A list of one or more subnet IDs in your Amazon VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vpcId</i> The ID of the Amazon VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>s3Logs</i>  Information about logs built to an S3 bucket for a build project. S3 logs are not enabled by default. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>s3Logs</b>:  Information about logs built to an S3 bucket for a build project. S3 logs are not enabled by default. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cloudWatchLogs</i>  Information about Amazon CloudWatch Logs for a build project. Amazon CloudWatch Logs are enabled by default. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>cloudWatchLogs</b>:  Information about Amazon CloudWatch Logs for a build project. Amazon CloudWatch Logs are enabled by default. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>streamName</i>  The prefix of the stream name of the Amazon CloudWatch Logs. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudWatch/latest/logs/Working-with-log-groups-and-streams.html">Working with Log Groups and Log Streams</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>groupName</i>  The group name of the logs in Amazon CloudWatch Logs. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudWatch/latest/logs/Working-with-log-groups-and-streams.html">Working with Log Groups and Log Streams</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the logs in Amazon CloudWatch Logs for a build project. Valid values are: <ul> <li>  <code>ENABLED</code>: Amazon CloudWatch Logs are enabled for this build project. </li> <li>  <code>DISABLED</code>: Amazon CloudWatch Logs are not enabled for this build project. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>encryptionDisabled</i>  Set to true if you do not want your S3 build log output encrypted. By default S3 build logs are encrypted. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i>  The ARN of an S3 bucket and the path prefix for S3 logs. If your Amazon S3 bucket name is <code>my-bucket</code>, and your path prefix is <code>build-log</code>, then acceptable formats are <code>my-bucket/build-log</code> or <code>arn:aws:s3:::my-bucket/build-log</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the S3 build logs. Valid values are: <ul> <li>  <code>ENABLED</code>: S3 build logs are enabled for this build project. </li> <li>  <code>DISABLED</code>: S3 build logs are not enabled for this build project. </li> </ul><br/>
        :return: project:  Information about the build project that was created.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>logsConfig</i>  Information about logs for the build project. A project can create logs in Amazon CloudWatch Logs, an S3 bucket, or both. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>logsConfig</b>:  Information about logs for the build project. A project can create logs in Amazon CloudWatch Logs, an S3 bucket, or both. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>badge</i> Information about the build badge for the build project.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>badge</b>: Information about the build badge for the build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vpcConfig</i> Information about the VPC configuration that AWS CodeBuild accesses.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>vpcConfig</b>: Information about the VPC configuration that AWS CodeBuild accesses.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>webhook</i> Information about a webhook that connects repository events to a build project in AWS CodeBuild.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>webhook</b>: Information about a webhook that connects repository events to a build project in AWS CodeBuild.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastModified</i> When the build project's settings were last modified, expressed in Unix time format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>created</i> When the build project was created, expressed in Unix time format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> The tags for this build project. These tags are available for use by AWS services that support AWS CodeBuild build project tags.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>encryptionKey</i> The AWS Key Management Service (AWS KMS) customer master key (CMK) to be used for encrypting the build output artifacts. <note>  You can use a cross-account KMS key to encrypt the build output artifacts if your service role has permission to that key.  </note> You can specify either the Amazon Resource Name (ARN) of the CMK or, if available, the CMK's alias (using the format <code>alias/<i>alias-name</i> </code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>queuedTimeoutInMinutes</i>  The number of minutes a build is allowed to be queued before it times out. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>timeoutInMinutes</i> How long, in minutes, from 5 to 480 (8 hours), for AWS CodeBuild to wait before timing out any related build that did not get marked as completed. The default is 60 minutes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceRole</i> The ARN of the AWS Identity and Access Management (IAM) role that enables AWS CodeBuild to interact with dependent AWS services on behalf of the AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>environment</i> Information about the build environment for this build project.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>environment</b>: Information about the build environment for this build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cache</i> Information about the cache for the build project.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>cache</b>: Information about the cache for the build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>secondaryArtifacts</i>  An array of <code>ProjectArtifacts</code> objects. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>artifacts</i> Information about the build output artifacts for the build project.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>artifacts</b>: Information about the build output artifacts for the build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>secondarySourceVersions</i>  An array of <code>ProjectSourceVersion</code> objects. If <code>secondarySourceVersions</code> is specified at the build level, then they take over these <code>secondarySourceVersions</code> (at the project level). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>sourceVersion</i> A version of the build input to be built for this project. If not specified, the latest version is used. If specified, it must be one of: <ul> <li> For AWS CodeCommit: the commit ID to use. </li> <li> For GitHub: the commit ID, pull request ID, branch name, or tag name that corresponds to the version of the source code you want to build. If a pull request ID is specified, it must use the format <code>pr/pull-request-ID</code> (for example <code>pr/25</code>). If a branch name is specified, the branch's HEAD commit ID is used. If not specified, the default branch's HEAD commit ID is used. </li> <li> For Bitbucket: the commit ID, branch name, or tag name that corresponds to the version of the source code you want to build. If a branch name is specified, the branch's HEAD commit ID is used. If not specified, the default branch's HEAD commit ID is used. </li> <li> For Amazon Simple Storage Service (Amazon S3): the version ID of the object that represents the build input ZIP file to use. </li> </ul>  If <code>sourceVersion</code> is specified at the build level, then that version takes precedence over this <code>sourceVersion</code> (at the project level).   For more information, see <a href="https://docs.aws.amazon.com/codebuild/latest/userguide/sample-source-version.html">Source Version Sample with CodeBuild</a> in the <i>AWS CodeBuild User Guide</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>secondarySources</i>  An array of <code>ProjectSource</code> objects. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>source</i> Information about the build input source code for this build project.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>source</b>: Information about the build input source code for this build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> A description that makes the build project easy to identify.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>gitCloneDepth</i> Information about the Git clone depth for the build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> Information about the location of the source code to be built. Valid values include: <ul> <li> For source code settings that are specified in the source action of a pipeline in AWS CodePipeline, <code>location</code> should not be specified. If it is specified, AWS CodePipeline ignores it. This is because AWS CodePipeline uses the settings in a pipeline's source action instead of this value. </li> <li> For source code in an AWS CodeCommit repository, the HTTPS clone URL to the repository that contains the source code and the build spec (for example, <code>https://git-codecommit.<i>region-ID</i>.amazonaws.com/v1/repos/<i>repo-name</i> </code>). </li> <li> For source code in an Amazon Simple Storage Service (Amazon S3) input bucket, one of the following.  <ul> <li>  The path to the ZIP file that contains the source code (for example, <code> <i>bucket-name</i>/<i>path</i>/<i>to</i>/<i>object-name</i>.zip</code>).  </li> <li>  The path to the folder that contains the source code (for example, <code> <i>bucket-name</i>/<i>path</i>/<i>to</i>/<i>source-code</i>/<i>folder</i>/</code>).  </li> </ul> </li> <li> For source code in a GitHub repository, the HTTPS clone URL to the repository that contains the source and the build spec. You must connect your AWS account to your GitHub account. Use the AWS CodeBuild console to start creating a build project. When you use the console to connect (or reconnect) with GitHub, on the GitHub <b>Authorize application</b> page, for <b>Organization access</b>, choose <b>Request access</b> next to each repository you want to allow AWS CodeBuild to have access to, and then choose <b>Authorize application</b>. (After you have connected to your GitHub account, you do not need to finish creating the build project. You can leave the AWS CodeBuild console.) To instruct AWS CodeBuild to use this connection, in the <code>source</code> object, set the <code>auth</code> object's <code>type</code> value to <code>OAUTH</code>. </li> <li> For source code in a Bitbucket repository, the HTTPS clone URL to the repository that contains the source and the build spec. You must connect your AWS account to your Bitbucket account. Use the AWS CodeBuild console to start creating a build project. When you use the console to connect (or reconnect) with Bitbucket, on the Bitbucket <b>Confirm access to your account</b> page, choose <b>Grant access</b>. (After you have connected to your Bitbucket account, you do not need to finish creating the build project. You can leave the AWS CodeBuild console.) To instruct AWS CodeBuild to use this connection, in the <code>source</code> object, set the <code>auth</code> object's <code>type</code> value to <code>OAUTH</code>. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of repository that contains the source code to be built. Valid values include: <ul> <li>  <code>BITBUCKET</code>: The source code is in a Bitbucket repository. </li> <li>  <code>CODECOMMIT</code>: The source code is in an AWS CodeCommit repository. </li> <li>  <code>CODEPIPELINE</code>: The source code settings are specified in the source action of a pipeline in AWS CodePipeline. </li> <li>  <code>GITHUB</code>: The source code is in a GitHub repository. </li> <li>  <code>NO_SOURCE</code>: The project does not have input source code. </li> <li>  <code>S3</code>: The source code is in an Amazon Simple Storage Service (Amazon S3) input bucket. </li> </ul><br/> Information about the Git submodules configuration for the build project. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>gitSubmodulesConfig</b>:  Information about the Git submodules configuration for the build project. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>artifactIdentifier</i>  An identifier for this artifact definition. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>encryptionDisabled</i>  Set to true if you do not want your output artifacts encrypted. This option is valid only if your artifacts type is Amazon Simple Storage Service (Amazon S3). If this is set with another artifacts type, an invalidInputException is thrown. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>overrideArtifactName</i>  If this flag is set, a name specified in the build spec file overrides the artifact name. The name specified in a build spec file is calculated at build time and uses the Shell Command Language. For example, you can append a date and time to your artifact name so that it is always unique. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>packaging</i> The type of build output artifact to create: <ul> <li> If <code>type</code> is set to <code>CODEPIPELINE</code>, AWS CodePipeline ignores this value if specified. This is because AWS CodePipeline manages its build output artifacts instead of AWS CodeBuild. </li> <li> If <code>type</code> is set to <code>NO_ARTIFACTS</code>, this value is ignored if specified, because no build output is produced. </li> <li> If <code>type</code> is set to <code>S3</code>, valid values include: <ul> <li>  <code>NONE</code>: AWS CodeBuild creates in the output bucket a folder that contains the build output. This is the default if <code>packaging</code> is not specified. </li> <li>  <code>ZIP</code>: AWS CodeBuild creates in the output bucket a ZIP file that contains the build output. </li> </ul> </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> Along with <code>path</code> and <code>namespaceType</code>, the pattern that AWS CodeBuild uses to name and store the output artifact: <ul> <li> If <code>type</code> is set to <code>CODEPIPELINE</code>, AWS CodePipeline ignores this value if specified. This is because AWS CodePipeline manages its build output names instead of AWS CodeBuild. </li> <li> If <code>type</code> is set to <code>NO_ARTIFACTS</code>, this value is ignored if specified, because no build output is produced. </li> <li> If <code>type</code> is set to <code>S3</code>, this is the name of the output artifact object. If you set the name to be a forward slash ("/"), the artifact is stored in the root of the output bucket. </li> </ul> For example: <ul> <li>  If <code>path</code> is set to <code>MyArtifacts</code>, <code>namespaceType</code> is set to <code>BUILD_ID</code>, and <code>name</code> is set to <code>MyArtifact.zip</code>, then the output artifact is stored in <code>MyArtifacts/<i>build-ID</i>/MyArtifact.zip</code>.  </li> <li>  If <code>path</code> is empty, <code>namespaceType</code> is set to <code>NONE</code>, and <code>name</code> is set to "<code>/</code>", the output artifact is stored in the root of the output bucket.  </li> <li>  If <code>path</code> is set to <code>MyArtifacts</code>, <code>namespaceType</code> is set to <code>BUILD_ID</code>, and <code>name</code> is set to "<code>/</code>", the output artifact is stored in <code>MyArtifacts/<i>build-ID</i> </code>.  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>namespaceType</i> Along with <code>path</code> and <code>name</code>, the pattern that AWS CodeBuild uses to determine the name and location to store the output artifact: <ul> <li> If <code>type</code> is set to <code>CODEPIPELINE</code>, AWS CodePipeline ignores this value if specified. This is because AWS CodePipeline manages its build output names instead of AWS CodeBuild. </li> <li> If <code>type</code> is set to <code>NO_ARTIFACTS</code>, this value is ignored if specified, because no build output is produced. </li> <li> If <code>type</code> is set to <code>S3</code>, valid values include: <ul> <li>  <code>BUILD_ID</code>: Include the build ID in the location of the build output artifact. </li> <li>  <code>NONE</code>: Do not include the build ID. This is the default if <code>namespaceType</code> is not specified. </li> </ul> </li> </ul> For example, if <code>path</code> is set to <code>MyArtifacts</code>, <code>namespaceType</code> is set to <code>BUILD_ID</code>, and <code>name</code> is set to <code>MyArtifact.zip</code>, the output artifact is stored in <code>MyArtifacts/<i>build-ID</i>/MyArtifact.zip</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>path</i> Along with <code>namespaceType</code> and <code>name</code>, the pattern that AWS CodeBuild uses to name and store the output artifact: <ul> <li> If <code>type</code> is set to <code>CODEPIPELINE</code>, AWS CodePipeline ignores this value if specified. This is because AWS CodePipeline manages its build output names instead of AWS CodeBuild. </li> <li> If <code>type</code> is set to <code>NO_ARTIFACTS</code>, this value is ignored if specified, because no build output is produced. </li> <li> If <code>type</code> is set to <code>S3</code>, this is the path to the output artifact. If <code>path</code> is not specified, <code>path</code> is not used. </li> </ul> For example, if <code>path</code> is set to <code>MyArtifacts</code>, <code>namespaceType</code> is set to <code>NONE</code>, and <code>name</code> is set to <code>MyArtifact.zip</code>, the output artifact is stored in the output bucket at <code>MyArtifacts/MyArtifact.zip</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> Information about the build output artifact location: <ul> <li> If <code>type</code> is set to <code>CODEPIPELINE</code>, AWS CodePipeline ignores this value if specified. This is because AWS CodePipeline manages its build output locations instead of AWS CodeBuild. </li> <li> If <code>type</code> is set to <code>NO_ARTIFACTS</code>, this value is ignored if specified, because no build output is produced. </li> <li> If <code>type</code> is set to <code>S3</code>, this is the name of the output bucket. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of build output artifact. Valid values include: <ul> <li>  <code>CODEPIPELINE</code>: The build project has build output generated through AWS CodePipeline. </li> <li>  <code>NO_ARTIFACTS</code>: The build project does not produce any build output. </li> <li>  <code>S3</code>: The build project stores build output in Amazon Simple Storage Service (Amazon S3). </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>modes</i>  If you use a <code>LOCAL</code> cache, the local cache mode. You can use one or more local cache modes at the same time.  <ul> <li>  <code>LOCAL_SOURCE_CACHE</code> mode caches Git metadata for primary and secondary sources. After the cache is created, subsequent builds pull only the change between commits. This mode is a good choice for projects with a clean working directory and a source that is a large Git repository. If you choose this option and your project does not use a Git repository (GitHub, GitHub Enterprise, or Bitbucket), the option is ignored.  </li> <li>  <code>LOCAL_DOCKER_LAYER_CACHE</code> mode caches existing Docker layers. This mode is a good choice for projects that build or pull large Docker images. It can prevent the performance issues caused by pulling large Docker images down from the network.  <note> <ul> <li>  You can use a Docker layer cache in the Linux environment only.  </li> <li>  The <code>privileged</code> flag must be set so that your project has the required Docker permissions.  </li> <li>  You should consider the security implications before you use a Docker layer cache.  </li> </ul> </note> </li> </ul> <ul> <li>  <code>LOCAL_CUSTOM_CACHE</code> mode caches directories you specify in the buildspec file. This mode is a good choice if your build scenario is not suited to one of the other three local cache modes. If you use a custom cache:  <ul> <li>  Only directories can be specified for caching. You cannot specify individual files.  </li> <li>  Symlinks are used to reference cached directories.  </li> <li>  Cached directories are linked to your build before it downloads its project sources. Cached items are overriden if a source item has the same name. Directories are specified using cache paths in the buildspec file.  </li> </ul> </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> Information about the cache location:  <ul> <li>  <code>NO_CACHE</code> or <code>LOCAL</code>: This value is ignored. </li> <li>  <code>S3</code>: This is the S3 bucket name/prefix. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of cache used by the build project. Valid values include: <ul> <li>  <code>NO_CACHE</code>: The build project does not use any cache. </li> <li>  <code>S3</code>: The build project reads and writes from and to S3. </li> <li>  <code>LOCAL</code>: The build project stores a cache locally on a build host that is only available to that build host. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>certificate</i> The certificate to use with this build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>privilegedMode</i> Enables running the Docker daemon inside a Docker container. Set to true only if the build project is be used to build Docker images, and the specified build environment image is not provided by AWS CodeBuild with Docker support. Otherwise, all associated builds that attempt to interact with the Docker daemon fail. You must also start the Docker daemon so that builds can interact with it. One way to do this is to initialize the Docker daemon during the install phase of your build spec by running the following build commands. (Do not run these commands if the specified build environment image is provided by AWS CodeBuild with Docker support.) If the operating system's base image is Ubuntu Linux:  <code>- nohup /usr/local/bin/dockerd --host=unix:///var/run/docker.sock --host=tcp://0.0.0.0:2375 --storage-driver=overlay&amp;</code>   <code>- timeout 15 sh -c "until docker info; do echo .; sleep 1; done"</code>  If the operating system's base image is Alpine Linux and the previous command does not work, add the <code>-t</code> argument to <code>timeout</code>:  <code>- nohup /usr/local/bin/dockerd --host=unix:///var/run/docker.sock --host=tcp://0.0.0.0:2375 --storage-driver=overlay&amp;</code>   <code>- timeout -t 15 sh -c "until docker info; do echo .; sleep 1; done"</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>environmentVariables</i> A set of environment variables to make available to builds for this build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>computeType</i> Information about the compute resources the build project uses. Available values include: <ul> <li>  <code>BUILD_GENERAL1_SMALL</code>: Use up to 3 GB memory and 2 vCPUs for builds. </li> <li>  <code>BUILD_GENERAL1_MEDIUM</code>: Use up to 7 GB memory and 4 vCPUs for builds. </li> <li>  <code>BUILD_GENERAL1_LARGE</code>: Use up to 15 GB memory and 8 vCPUs for builds. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>image</i> The image tag or image digest that identifies the Docker image to use for this build project. Use the following formats: <ul> <li> For an image tag: <code>registry/repository:tag</code>. For example, to specify an image with the tag "latest," use <code>registry/repository:latest</code>. </li> <li> For an image digest: <code>registry/repository@digest</code>. For example, to specify an image with the digest "sha256:cbbf2f9a99b47fc460d422812b6a5adff7dfee951d8fa2e4a98caa0382cfbdbf," use <code>registry/repository@sha256:cbbf2f9a99b47fc460d422812b6a5adff7dfee951d8fa2e4a98caa0382cfbdbf</code>. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of build environment to use for related builds.<br/> The credentials for access to a private registry.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>registryCredential</b>:  The credentials for access to a private registry.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastModifiedSecret</i>  A timestamp that indicates the last time a repository's secret token was modified. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>filterGroups</i>  An array of arrays of <code>WebhookFilter</code> objects used to determine which webhooks are triggered. At least one <code>WebhookFilter</code> in the array must specify <code>EVENT</code> as its <code>type</code>.   For a build to be triggered, at least one filter group in the <code>filterGroups</code> array must pass. For a filter group to pass, each of its filters must pass. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>branchFilter</i> A regular expression used to determine which repository branches are built when a webhook is triggered. If the name of a branch matches the regular expression, then it is built. If <code>branchFilter</code> is empty, then all branches are built. <note>  It is recommended that you use <code>filterGroups</code> instead of <code>branchFilter</code>.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>secret</i>  The secret token of the associated repository.  <note>  A Bitbucket webhook does not support <code>secret</code>.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>payloadUrl</i>  The AWS CodeBuild endpoint where webhook events are sent.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>url</i> The URL to the webhook.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>securityGroupIds</i> A list of one or more security groups IDs in your Amazon VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>subnets</i> A list of one or more subnet IDs in your Amazon VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>vpcId</i> The ID of the Amazon VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>badgeRequestUrl</i> The publicly-accessible URL through which you can access the build badge for your project.  The publicly accessible URL through which you can access the build badge for your project. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>badgeEnabled</i> Set this to true to generate a publicly accessible URL for your project's build badge.<br/> Information about Amazon CloudWatch Logs for a build project. Amazon CloudWatch Logs are enabled by default. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>cloudWatchLogs</b>:  Information about Amazon CloudWatch Logs for a build project. Amazon CloudWatch Logs are enabled by default.         """
        pass

    def create_webhook(self, projectName: str, branchFilter: str=None, filterGroups: List[FilterGroup]=None) -> CreateWebhookOutput:
        """For an existing AWS CodeBuild build project that has its source code stored in a GitHub or Bitbucket repository, enables AWS CodeBuild to start rebuilding the source code every time a code change is pushed to the repository. <important> If you enable webhooks for an AWS CodeBuild project, and the project is used as a build step in AWS CodePipeline, then two identical builds are created for each commit. One build is triggered through webhooks, and one through AWS CodePipeline. Because billing is on a per-build basis, you are billed for both builds. Therefore, if you are using AWS CodePipeline, we recommend that you disable webhooks in AWS CodeBuild. In the AWS CodeBuild console, clear the Webhook box. For more information, see step 5 in <a href="https://docs.aws.amazon.com/codebuild/latest/userguide/change-project.html#change-project-console">Change a Build Project's Settings</a>. </important>

        :param self:
        :param filterGroups:   An array of arrays of <code>WebhookFilter</code> objects used to determine which webhooks are triggered. At least one <code>WebhookFilter</code> in the array must specify <code>EVENT</code> as its <code>type</code>.   For a build to be triggered, at least one filter group in the <code>filterGroups</code> array must pass. For a filter group to pass, each of its filters must pass. <br/>branchFilter:  A regular expression used to determine which repository branches are built when a webhook is triggered. If the name of a branch matches the regular expression, then it is built. If <code>branchFilter</code> is empty, then all branches are built. <note>  It is recommended that you use <code>filterGroups</code> instead of <code>branchFilter</code>.  </note><br/>projectName:  The name of the AWS CodeBuild project.<br/>
        :return: webhook:  Information about a webhook that connects repository events to a build project in AWS CodeBuild.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastModifiedSecret</i>  A timestamp that indicates the last time a repository's secret token was modified. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>filterGroups</i>  An array of arrays of <code>WebhookFilter</code> objects used to determine which webhooks are triggered. At least one <code>WebhookFilter</code> in the array must specify <code>EVENT</code> as its <code>type</code>.   For a build to be triggered, at least one filter group in the <code>filterGroups</code> array must pass. For a filter group to pass, each of its filters must pass. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>branchFilter</i> A regular expression used to determine which repository branches are built when a webhook is triggered. If the name of a branch matches the regular expression, then it is built. If <code>branchFilter</code> is empty, then all branches are built. <note>  It is recommended that you use <code>filterGroups</code> instead of <code>branchFilter</code>.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>secret</i>  The secret token of the associated repository.  <note>  A Bitbucket webhook does not support <code>secret</code>.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>payloadUrl</i>  The AWS CodeBuild endpoint where webhook events are sent.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>url</i> The URL to the webhook.<br/>        """
        pass

    def delete_project(self, name: str) -> DeleteProjectOutput:
        """Deletes a build project.

        :param self:
        :param name:  The name of the build project.<br/>
        :return:         """
        pass

    def delete_source_credentials(self, arn: str) -> DeleteSourceCredentialsOutput:
        """ Deletes a set of GitHub, GitHub Enterprise, or Bitbucket source credentials. 

        :param self:
        :param arn:   The Amazon Resource Name (ARN) of the token.<br/>
        :return: arn:   The Amazon Resource Name (ARN) of the token. <br/>        """
        pass

    def delete_webhook(self, projectName: str) -> DeleteWebhookOutput:
        """For an existing AWS CodeBuild build project that has its source code stored in a GitHub or Bitbucket repository, stops AWS CodeBuild from rebuilding the source code every time a code change is pushed to the repository.

        :param self:
        :param projectName:  The name of the AWS CodeBuild project.<br/>
        :return:         """
        pass

    def import_source_credentials(self, authType: str, serverType: str, token: str, username: str=None) -> ImportSourceCredentialsOutput:
        """ Imports the source repository credentials for an AWS CodeBuild project that has its source code stored in a GitHub, GitHub Enterprise, or Bitbucket repository. 

        :param self:
        :param authType:   The type of authentication used to connect to a GitHub, GitHub Enterprise, or Bitbucket repository. An OAUTH connection is not supported by the API and must be created using the AWS CodeBuild console. <br/>serverType:   The source provider used for this project. <br/>token:   For GitHub or GitHub Enterprise, this is the personal access token. For Bitbucket, this is the app password. <br/>username:   The Bitbucket username when the <code>authType</code> is BASIC_AUTH. This parameter is not valid for other types of source providers or connections. <br/>
        :return: arn:   The Amazon Resource Name (ARN) of the token. <br/>        """
        pass

    def invalidate_project_cache(self, projectName: str) -> InvalidateProjectCacheOutput:
        """Resets the cache for a project.

        :param self:
        :param projectName:  The name of the AWS CodeBuild build project that the cache is reset for.<br/>
        :return:         """
        pass

    def list_builds(self, sortOrder: str=None, nextToken: str=None) -> ListBuildsOutput:
        """Gets a list of build IDs, with each build ID representing a single build.

        :param self:
        :param nextToken:  During a previous call, if there are more than 100 items in the list, only the first 100 items are returned, along with a unique string called a <i>next token</i>. To get the next batch of items in the list, call this operation again, adding the next token to the call. To get all of the items in the list, keep calling this operation with each subsequent next token that is returned, until no more next tokens are returned.<br/>sortOrder:  The order to list build IDs. Valid values include: <ul> <li>  <code>ASCENDING</code>: List the build IDs in ascending order by build ID. </li> <li>  <code>DESCENDING</code>: List the build IDs in descending order by build ID. </li> </ul><br/>
        :return: nextToken:  If there are more than 100 items in the list, only the first 100 items are returned, along with a unique string called a <i>next token</i>. To get the next batch of items in the list, call this operation again, adding the next token to the call.<br/>ids:  A list of build IDs, with each build ID representing a single build.<br/>        """
        pass

    def list_builds_for_project(self, projectName: str, sortOrder: str=None, nextToken: str=None) -> ListBuildsForProjectOutput:
        """Gets a list of build IDs for the specified build project, with each build ID representing a single build.

        :param self:
        :param nextToken:  During a previous call, if there are more than 100 items in the list, only the first 100 items are returned, along with a unique string called a <i>next token</i>. To get the next batch of items in the list, call this operation again, adding the next token to the call. To get all of the items in the list, keep calling this operation with each subsequent next token that is returned, until no more next tokens are returned.<br/>sortOrder:  The order to list build IDs. Valid values include: <ul> <li>  <code>ASCENDING</code>: List the build IDs in ascending order by build ID. </li> <li>  <code>DESCENDING</code>: List the build IDs in descending order by build ID. </li> </ul><br/>projectName:  The name of the AWS CodeBuild project.<br/>
        :return: nextToken:  If there are more than 100 items in the list, only the first 100 items are returned, along with a unique string called a <i>next token</i>. To get the next batch of items in the list, call this operation again, adding the next token to the call.<br/>ids:  A list of build IDs for the specified build project, with each build ID representing a single build.<br/>        """
        pass

    def list_curated_environment_images(self) -> ListCuratedEnvironmentImagesOutput:
        """Gets information about Docker images that are managed by AWS CodeBuild.

        :param self:
        :param 
        :return: platforms:  Information about supported platforms for Docker images that are managed by AWS CodeBuild.<br/>        """
        pass

    def list_projects(self, sortBy: str=None, sortOrder: str=None, nextToken: str=None) -> ListProjectsOutput:
        """Gets a list of build project names, with each build project name representing a single build project.

        :param self:
        :param nextToken:  During a previous call, if there are more than 100 items in the list, only the first 100 items are returned, along with a unique string called a <i>next token</i>. To get the next batch of items in the list, call this operation again, adding the next token to the call. To get all of the items in the list, keep calling this operation with each subsequent next token that is returned, until no more next tokens are returned.<br/>sortOrder:  The order in which to list build projects. Valid values include: <ul> <li>  <code>ASCENDING</code>: List in ascending order. </li> <li>  <code>DESCENDING</code>: List in descending order. </li> </ul> Use <code>sortBy</code> to specify the criterion to be used to list build project names.<br/>sortBy:  The criterion to be used to list build project names. Valid values include: <ul> <li>  <code>CREATED_TIME</code>: List based on when each build project was created. </li> <li>  <code>LAST_MODIFIED_TIME</code>: List based on when information about each build project was last changed. </li> <li>  <code>NAME</code>: List based on each build project's name. </li> </ul> Use <code>sortOrder</code> to specify in what order to list the build project names based on the preceding criteria.<br/>
        :return: projects:  The list of build project names, with each build project name representing a single build project.<br/>nextToken:  If there are more than 100 items in the list, only the first 100 items are returned, along with a unique string called a <i>next token</i>. To get the next batch of items in the list, call this operation again, adding the next token to the call.<br/>        """
        pass

    def list_source_credentials(self) -> ListSourceCredentialsOutput:
        """ Returns a list of <code>SourceCredentialsInfo</code> objects. 

        :param self:
        :param 
        :return: sourceCredentialsInfos:   A list of <code>SourceCredentialsInfo</code> objects. Each <code>SourceCredentialsInfo</code> object includes the authentication type, token ARN, and type of source provider for one set of credentials. <br/>        """
        pass

    def start_build(self, projectName: str, secondarySourcesOverride: List[ProjectSource]=None, secondarySourcesVersionOverride: List[ProjectSourceVersion]=None, sourceVersion: str=None, artifactsOverride: ProjectArtifacts=None, secondaryArtifactsOverride: List[ProjectArtifacts]=None, environmentVariablesOverride: List[EnvironmentVariable]=None, sourceTypeOverride: str=None, sourceLocationOverride: str=None, sourceAuthOverride: SourceAuth=None, gitCloneDepthOverride: int=None, gitSubmodulesConfigOverride: GitSubmodulesConfig=None, buildspecOverride: str=None, insecureSslOverride: WrapperBoolean=None, reportBuildStatusOverride: WrapperBoolean=None, environmentTypeOverride: str=None, imageOverride: str=None, computeTypeOverride: str=None, certificateOverride: str=None, cacheOverride: ProjectCache=None, serviceRoleOverride: str=None, privilegedModeOverride: WrapperBoolean=None, timeoutInMinutesOverride: int=None, queuedTimeoutInMinutesOverride: int=None, idempotencyToken: str=None, logsConfigOverride: LogsConfig=None, registryCredentialOverride: RegistryCredential=None, imagePullCredentialsTypeOverride: str=None) -> StartBuildOutput:
        """Starts running a build.

        :param self:
        :param imagePullCredentialsTypeOverride:   The type of credentials AWS CodeBuild uses to pull images in your build. There are two valid values:  <ul> <li>  <code>CODEBUILD</code> specifies that AWS CodeBuild uses its own credentials. This requires that you modify your ECR repository policy to trust AWS CodeBuild's service principal. </li> <li>  <code>SERVICE_ROLE</code> specifies that AWS CodeBuild uses your build project's service role.  </li> </ul>  When using a cross-account or private registry image, you must use SERVICE_ROLE credentials. When using an AWS CodeBuild curated image, you must use CODEBUILD credentials. <br/>registryCredentialOverride:   The credentials for access to a private registry. :<br/>logsConfigOverride:   Log settings for this build that override the log settings defined in the build project. :<br/>idempotencyToken:  A unique, case sensitive identifier you provide to ensure the idempotency of the StartBuild request. The token is included in the StartBuild request and is valid for 12 hours. If you repeat the StartBuild request with the same token, but change a parameter, AWS CodeBuild returns a parameter mismatch error. <br/>queuedTimeoutInMinutesOverride:   The number of minutes a build is allowed to be queued before it times out. <br/>timeoutInMinutesOverride:  The number of build timeout minutes, from 5 to 480 (8 hours), that overrides, for this build only, the latest setting already defined in the build project.<br/>privilegedModeOverride:  Enable this flag to override privileged mode in the build project.<br/>serviceRoleOverride:  The name of a service role for this build that overrides the one specified in the build project.<br/>cacheOverride:  A ProjectCache object specified for this build that overrides the one defined in the build project.:<br/>certificateOverride:  The name of a certificate for this build that overrides the one specified in the build project.<br/>computeTypeOverride:  The name of a compute type for this build that overrides the one specified in the build project.<br/>imageOverride:  The name of an image for this build that overrides the one specified in the build project.<br/>environmentTypeOverride:  A container type for this build that overrides the one specified in the build project.<br/>reportBuildStatusOverride:   Set to true to report to your source provider the status of a build's start and completion. If you use this option with a source provider other than GitHub, GitHub Enterprise, or Bitbucket, an invalidInputException is thrown. <br/>insecureSslOverride:  Enable this flag to override the insecure SSL setting that is specified in the build project. The insecure SSL setting determines whether to ignore SSL warnings while connecting to the project source code. This override applies only if the build's source is GitHub Enterprise.<br/>buildspecOverride:  A build spec declaration that overrides, for this build only, the latest one already defined in the build project.<br/>gitSubmodulesConfigOverride:   Information about the Git submodules configuration for this build of an AWS CodeBuild build project. :<br/>gitCloneDepthOverride:  The user-defined depth of history, with a minimum value of 0, that overrides, for this build only, any previous depth of history defined in the build project.<br/>sourceAuthOverride:  An authorization type for this build that overrides the one defined in the build project. This override applies only if the build project's source is BitBucket or GitHub.:<br/>sourceLocationOverride:  A location that overrides, for this build, the source location for the one defined in the build project.<br/>sourceTypeOverride:  A source input type, for this build, that overrides the source input defined in the build project.<br/>environmentVariablesOverride:  A set of environment variables that overrides, for this build only, the latest ones already defined in the build project.<br/>secondaryArtifactsOverride:   An array of <code>ProjectArtifacts</code> objects. <br/>artifactsOverride:  Build output artifact settings that override, for this build only, the latest ones already defined in the build project.:<br/>sourceVersion:  A version of the build input to be built, for this build only. If not specified, the latest version is used. If specified, must be one of: <ul> <li> For AWS CodeCommit: the commit ID to use. </li> <li> For GitHub: the commit ID, pull request ID, branch name, or tag name that corresponds to the version of the source code you want to build. If a pull request ID is specified, it must use the format <code>pr/pull-request-ID</code> (for example <code>pr/25</code>). If a branch name is specified, the branch's HEAD commit ID is used. If not specified, the default branch's HEAD commit ID is used. </li> <li> For Bitbucket: the commit ID, branch name, or tag name that corresponds to the version of the source code you want to build. If a branch name is specified, the branch's HEAD commit ID is used. If not specified, the default branch's HEAD commit ID is used. </li> <li> For Amazon Simple Storage Service (Amazon S3): the version ID of the object that represents the build input ZIP file to use. </li> </ul>  If <code>sourceVersion</code> is specified at the project level, then this <code>sourceVersion</code> (at the build level) takes precedence.   For more information, see <a href="https://docs.aws.amazon.com/codebuild/latest/userguide/sample-source-version.html">Source Version Sample with CodeBuild</a> in the <i>AWS CodeBuild User Guide</i>. <br/>secondarySourcesVersionOverride:   An array of <code>ProjectSourceVersion</code> objects that specify one or more versions of the project's secondary sources to be used for this build only. <br/>secondarySourcesOverride:   An array of <code>ProjectSource</code> objects. <br/>projectName:  The name of the AWS CodeBuild build project to start running a build.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>artifactIdentifier</i>  An identifier for this artifact definition. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>encryptionDisabled</i>  Set to true if you do not want your output artifacts encrypted. This option is valid only if your artifacts type is Amazon Simple Storage Service (Amazon S3). If this is set with another artifacts type, an invalidInputException is thrown. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>overrideArtifactName</i>  If this flag is set, a name specified in the build spec file overrides the artifact name. The name specified in a build spec file is calculated at build time and uses the Shell Command Language. For example, you can append a date and time to your artifact name so that it is always unique. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>packaging</i> The type of build output artifact to create: <ul> <li> If <code>type</code> is set to <code>CODEPIPELINE</code>, AWS CodePipeline ignores this value if specified. This is because AWS CodePipeline manages its build output artifacts instead of AWS CodeBuild. </li> <li> If <code>type</code> is set to <code>NO_ARTIFACTS</code>, this value is ignored if specified, because no build output is produced. </li> <li> If <code>type</code> is set to <code>S3</code>, valid values include: <ul> <li>  <code>NONE</code>: AWS CodeBuild creates in the output bucket a folder that contains the build output. This is the default if <code>packaging</code> is not specified. </li> <li>  <code>ZIP</code>: AWS CodeBuild creates in the output bucket a ZIP file that contains the build output. </li> </ul> </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> Along with <code>path</code> and <code>namespaceType</code>, the pattern that AWS CodeBuild uses to name and store the output artifact: <ul> <li> If <code>type</code> is set to <code>CODEPIPELINE</code>, AWS CodePipeline ignores this value if specified. This is because AWS CodePipeline manages its build output names instead of AWS CodeBuild. </li> <li> If <code>type</code> is set to <code>NO_ARTIFACTS</code>, this value is ignored if specified, because no build output is produced. </li> <li> If <code>type</code> is set to <code>S3</code>, this is the name of the output artifact object. If you set the name to be a forward slash ("/"), the artifact is stored in the root of the output bucket. </li> </ul> For example: <ul> <li>  If <code>path</code> is set to <code>MyArtifacts</code>, <code>namespaceType</code> is set to <code>BUILD_ID</code>, and <code>name</code> is set to <code>MyArtifact.zip</code>, then the output artifact is stored in <code>MyArtifacts/<i>build-ID</i>/MyArtifact.zip</code>.  </li> <li>  If <code>path</code> is empty, <code>namespaceType</code> is set to <code>NONE</code>, and <code>name</code> is set to "<code>/</code>", the output artifact is stored in the root of the output bucket.  </li> <li>  If <code>path</code> is set to <code>MyArtifacts</code>, <code>namespaceType</code> is set to <code>BUILD_ID</code>, and <code>name</code> is set to "<code>/</code>", the output artifact is stored in <code>MyArtifacts/<i>build-ID</i> </code>.  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>namespaceType</i> Along with <code>path</code> and <code>name</code>, the pattern that AWS CodeBuild uses to determine the name and location to store the output artifact: <ul> <li> If <code>type</code> is set to <code>CODEPIPELINE</code>, AWS CodePipeline ignores this value if specified. This is because AWS CodePipeline manages its build output names instead of AWS CodeBuild. </li> <li> If <code>type</code> is set to <code>NO_ARTIFACTS</code>, this value is ignored if specified, because no build output is produced. </li> <li> If <code>type</code> is set to <code>S3</code>, valid values include: <ul> <li>  <code>BUILD_ID</code>: Include the build ID in the location of the build output artifact. </li> <li>  <code>NONE</code>: Do not include the build ID. This is the default if <code>namespaceType</code> is not specified. </li> </ul> </li> </ul> For example, if <code>path</code> is set to <code>MyArtifacts</code>, <code>namespaceType</code> is set to <code>BUILD_ID</code>, and <code>name</code> is set to <code>MyArtifact.zip</code>, the output artifact is stored in <code>MyArtifacts/<i>build-ID</i>/MyArtifact.zip</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>path</i> Along with <code>namespaceType</code> and <code>name</code>, the pattern that AWS CodeBuild uses to name and store the output artifact: <ul> <li> If <code>type</code> is set to <code>CODEPIPELINE</code>, AWS CodePipeline ignores this value if specified. This is because AWS CodePipeline manages its build output names instead of AWS CodeBuild. </li> <li> If <code>type</code> is set to <code>NO_ARTIFACTS</code>, this value is ignored if specified, because no build output is produced. </li> <li> If <code>type</code> is set to <code>S3</code>, this is the path to the output artifact. If <code>path</code> is not specified, <code>path</code> is not used. </li> </ul> For example, if <code>path</code> is set to <code>MyArtifacts</code>, <code>namespaceType</code> is set to <code>NONE</code>, and <code>name</code> is set to <code>MyArtifact.zip</code>, the output artifact is stored in the output bucket at <code>MyArtifacts/MyArtifact.zip</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> Information about the build output artifact location: <ul> <li> If <code>type</code> is set to <code>CODEPIPELINE</code>, AWS CodePipeline ignores this value if specified. This is because AWS CodePipeline manages its build output locations instead of AWS CodeBuild. </li> <li> If <code>type</code> is set to <code>NO_ARTIFACTS</code>, this value is ignored if specified, because no build output is produced. </li> <li> If <code>type</code> is set to <code>S3</code>, this is the name of the output bucket. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of build output artifact. Valid values include: <ul> <li>  <code>CODEPIPELINE</code>: The build project has build output generated through AWS CodePipeline. </li> <li>  <code>NO_ARTIFACTS</code>: The build project does not produce any build output. </li> <li>  <code>S3</code>: The build project stores build output in Amazon Simple Storage Service (Amazon S3). </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resource</i> The resource value that applies to the specified authorization type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> <note>  This data type is deprecated and is no longer accurate or used.  </note> The authorization type to use. The only valid value is <code>OAUTH</code>, which represents the OAuth authorization type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fetchSubmodules</i>  Set to true to fetch Git submodules for your AWS CodeBuild build project. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>modes</i>  If you use a <code>LOCAL</code> cache, the local cache mode. You can use one or more local cache modes at the same time.  <ul> <li>  <code>LOCAL_SOURCE_CACHE</code> mode caches Git metadata for primary and secondary sources. After the cache is created, subsequent builds pull only the change between commits. This mode is a good choice for projects with a clean working directory and a source that is a large Git repository. If you choose this option and your project does not use a Git repository (GitHub, GitHub Enterprise, or Bitbucket), the option is ignored.  </li> <li>  <code>LOCAL_DOCKER_LAYER_CACHE</code> mode caches existing Docker layers. This mode is a good choice for projects that build or pull large Docker images. It can prevent the performance issues caused by pulling large Docker images down from the network.  <note> <ul> <li>  You can use a Docker layer cache in the Linux environment only.  </li> <li>  The <code>privileged</code> flag must be set so that your project has the required Docker permissions.  </li> <li>  You should consider the security implications before you use a Docker layer cache.  </li> </ul> </note> </li> </ul> <ul> <li>  <code>LOCAL_CUSTOM_CACHE</code> mode caches directories you specify in the buildspec file. This mode is a good choice if your build scenario is not suited to one of the other three local cache modes. If you use a custom cache:  <ul> <li>  Only directories can be specified for caching. You cannot specify individual files.  </li> <li>  Symlinks are used to reference cached directories.  </li> <li>  Cached directories are linked to your build before it downloads its project sources. Cached items are overriden if a source item has the same name. Directories are specified using cache paths in the buildspec file.  </li> </ul> </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> Information about the cache location:  <ul> <li>  <code>NO_CACHE</code> or <code>LOCAL</code>: This value is ignored. </li> <li>  <code>S3</code>: This is the S3 bucket name/prefix. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of cache used by the build project. Valid values include: <ul> <li>  <code>NO_CACHE</code>: The build project does not use any cache. </li> <li>  <code>S3</code>: The build project reads and writes from and to S3. </li> <li>  <code>LOCAL</code>: The build project stores a cache locally on a build host that is only available to that build host. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>s3Logs</i>  Information about logs built to an S3 bucket for a build project. S3 logs are not enabled by default. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>s3Logs</b>:  Information about logs built to an S3 bucket for a build project. S3 logs are not enabled by default. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cloudWatchLogs</i>  Information about Amazon CloudWatch Logs for a build project. Amazon CloudWatch Logs are enabled by default. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>cloudWatchLogs</b>:  Information about Amazon CloudWatch Logs for a build project. Amazon CloudWatch Logs are enabled by default. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>streamName</i>  The prefix of the stream name of the Amazon CloudWatch Logs. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudWatch/latest/logs/Working-with-log-groups-and-streams.html">Working with Log Groups and Log Streams</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>groupName</i>  The group name of the logs in Amazon CloudWatch Logs. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudWatch/latest/logs/Working-with-log-groups-and-streams.html">Working with Log Groups and Log Streams</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the logs in Amazon CloudWatch Logs for a build project. Valid values are: <ul> <li>  <code>ENABLED</code>: Amazon CloudWatch Logs are enabled for this build project. </li> <li>  <code>DISABLED</code>: Amazon CloudWatch Logs are not enabled for this build project. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>encryptionDisabled</i>  Set to true if you do not want your S3 build log output encrypted. By default S3 build logs are encrypted. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i>  The ARN of an S3 bucket and the path prefix for S3 logs. If your Amazon S3 bucket name is <code>my-bucket</code>, and your path prefix is <code>build-log</code>, then acceptable formats are <code>my-bucket/build-log</code> or <code>arn:aws:s3:::my-bucket/build-log</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the S3 build logs. Valid values are: <ul> <li>  <code>ENABLED</code>: S3 build logs are enabled for this build project. </li> <li>  <code>DISABLED</code>: S3 build logs are not enabled for this build project. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>credentialProvider</i>  The service that created the credentials to access a private Docker registry. The valid value, SECRETS_MANAGER, is for AWS Secrets Manager. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>credential</i>  The Amazon Resource Name (ARN) or name of credentials created using AWS Secrets Manager.  <note>  The <code>credential</code> can use the name of the credentials only if they exist in your current region.  </note><br/>
        :return: build:  Information about the build to be run.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>encryptionKey</i> The AWS Key Management Service (AWS KMS) customer master key (CMK) to be used for encrypting the build output artifacts. <note>  You can use a cross-account KMS key to encrypt the build output artifacts if your service role has permission to that key.  </note> You can specify either the Amazon Resource Name (ARN) of the CMK or, if available, the CMK's alias (using the format <code>alias/<i>alias-name</i> </code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>networkInterface</i> Describes a network interface.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>networkInterface</b>: Describes a network interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vpcConfig</i> If your AWS CodeBuild project accesses resources in an Amazon VPC, you provide this parameter that identifies the VPC ID and the list of security group IDs and subnet IDs. The security groups and subnets must belong to the same VPC. You must provide at least one security group and one subnet ID.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>vpcConfig</b>: If your AWS CodeBuild project accesses resources in an Amazon VPC, you provide this parameter that identifies the VPC ID and the list of security group IDs and subnet IDs. The security groups and subnets must belong to the same VPC. You must provide at least one security group and one subnet ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>initiator</i> The entity that started the build. Valid values include: <ul> <li> If AWS CodePipeline started the build, the pipeline's name (for example, <code>codepipeline/my-demo-pipeline</code>). </li> <li> If an AWS Identity and Access Management (IAM) user started the build, the user's name (for example, <code>MyUserName</code>). </li> <li> If the Jenkins plugin for AWS CodeBuild started the build, the string <code>CodeBuild-Jenkins-Plugin</code>. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>buildComplete</i> Whether the build is complete. True if complete; otherwise, false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>queuedTimeoutInMinutes</i>  The number of minutes a build is allowed to be queued before it times out. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>timeoutInMinutes</i> How long, in minutes, for AWS CodeBuild to wait before timing out this build if it does not get marked as completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>logs</i> Information about the build's logs in Amazon CloudWatch Logs.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>logs</b>: Information about the build's logs in Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceRole</i> The name of a service role used for this build.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>environment</i> Information about the build environment for this build.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>environment</b>: Information about the build environment for this build.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cache</i> Information about the cache for the build.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>cache</b>: Information about the cache for the build.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>secondaryArtifacts</i>  An array of <code>ProjectArtifacts</code> objects. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>artifacts</i> Information about the output artifacts for the build.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>artifacts</b>: Information about the output artifacts for the build.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>secondarySourceVersions</i>  An array of <code>ProjectSourceVersion</code> objects. Each <code>ProjectSourceVersion</code> must be one of:  <ul> <li> For AWS CodeCommit: the commit ID to use. </li> <li> For GitHub: the commit ID, pull request ID, branch name, or tag name that corresponds to the version of the source code you want to build. If a pull request ID is specified, it must use the format <code>pr/pull-request-ID</code> (for example, <code>pr/25</code>). If a branch name is specified, the branch's HEAD commit ID is used. If not specified, the default branch's HEAD commit ID is used. </li> <li> For Bitbucket: the commit ID, branch name, or tag name that corresponds to the version of the source code you want to build. If a branch name is specified, the branch's HEAD commit ID is used. If not specified, the default branch's HEAD commit ID is used. </li> <li> For Amazon Simple Storage Service (Amazon S3): the version ID of the object that represents the build input ZIP file to use. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>secondarySources</i>  An array of <code>ProjectSource</code> objects. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>source</i> Information about the source code to be built.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>source</b>: Information about the source code to be built.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>phases</i> Information about all previous build phases that are complete and information about any current build phase that is not yet complete.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>projectName</i> The name of the AWS CodeBuild project.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resolvedSourceVersion</i>  An identifier for the version of this build's source code.  <ul> <li>  For AWS CodeCommit, GitHub, GitHub Enterprise, and BitBucket, the commit ID.  </li> <li>  For AWS CodePipeline, the source revision provided by AWS CodePipeline.  </li> <li>  For Amazon Simple Storage Service (Amazon S3), this does not apply.  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>sourceVersion</i> Any version identifier for the version of the source code to be built. If <code>sourceVersion</code> is specified at the project level, then this <code>sourceVersion</code> (at the build level) takes precedence.   For more information, see <a href="https://docs.aws.amazon.com/codebuild/latest/userguide/sample-source-version.html">Source Version Sample with CodeBuild</a> in the <i>AWS CodeBuild User Guide</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>buildStatus</i> The current status of the build. Valid values include: <ul> <li>  <code>FAILED</code>: The build failed. </li> <li>  <code>FAULT</code>: The build faulted. </li> <li>  <code>IN_PROGRESS</code>: The build is still in progress. </li> <li>  <code>STOPPED</code>: The build stopped. </li> <li>  <code>SUCCEEDED</code>: The build succeeded. </li> <li>  <code>TIMED_OUT</code>: The build timed out. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>currentPhase</i> The current build phase.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>endTime</i> When the build process ended, expressed in Unix time format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>startTime</i> When the build process started, expressed in Unix time format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the build.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The unique ID for the build.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>gitCloneDepth</i> Information about the Git clone depth for the build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> Information about the location of the source code to be built. Valid values include: <ul> <li> For source code settings that are specified in the source action of a pipeline in AWS CodePipeline, <code>location</code> should not be specified. If it is specified, AWS CodePipeline ignores it. This is because AWS CodePipeline uses the settings in a pipeline's source action instead of this value. </li> <li> For source code in an AWS CodeCommit repository, the HTTPS clone URL to the repository that contains the source code and the build spec (for example, <code>https://git-codecommit.<i>region-ID</i>.amazonaws.com/v1/repos/<i>repo-name</i> </code>). </li> <li> For source code in an Amazon Simple Storage Service (Amazon S3) input bucket, one of the following.  <ul> <li>  The path to the ZIP file that contains the source code (for example, <code> <i>bucket-name</i>/<i>path</i>/<i>to</i>/<i>object-name</i>.zip</code>).  </li> <li>  The path to the folder that contains the source code (for example, <code> <i>bucket-name</i>/<i>path</i>/<i>to</i>/<i>source-code</i>/<i>folder</i>/</code>).  </li> </ul> </li> <li> For source code in a GitHub repository, the HTTPS clone URL to the repository that contains the source and the build spec. You must connect your AWS account to your GitHub account. Use the AWS CodeBuild console to start creating a build project. When you use the console to connect (or reconnect) with GitHub, on the GitHub <b>Authorize application</b> page, for <b>Organization access</b>, choose <b>Request access</b> next to each repository you want to allow AWS CodeBuild to have access to, and then choose <b>Authorize application</b>. (After you have connected to your GitHub account, you do not need to finish creating the build project. You can leave the AWS CodeBuild console.) To instruct AWS CodeBuild to use this connection, in the <code>source</code> object, set the <code>auth</code> object's <code>type</code> value to <code>OAUTH</code>. </li> <li> For source code in a Bitbucket repository, the HTTPS clone URL to the repository that contains the source and the build spec. You must connect your AWS account to your Bitbucket account. Use the AWS CodeBuild console to start creating a build project. When you use the console to connect (or reconnect) with Bitbucket, on the Bitbucket <b>Confirm access to your account</b> page, choose <b>Grant access</b>. (After you have connected to your Bitbucket account, you do not need to finish creating the build project. You can leave the AWS CodeBuild console.) To instruct AWS CodeBuild to use this connection, in the <code>source</code> object, set the <code>auth</code> object's <code>type</code> value to <code>OAUTH</code>. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of repository that contains the source code to be built. Valid values include: <ul> <li>  <code>BITBUCKET</code>: The source code is in a Bitbucket repository. </li> <li>  <code>CODECOMMIT</code>: The source code is in an AWS CodeCommit repository. </li> <li>  <code>CODEPIPELINE</code>: The source code settings are specified in the source action of a pipeline in AWS CodePipeline. </li> <li>  <code>GITHUB</code>: The source code is in a GitHub repository. </li> <li>  <code>NO_SOURCE</code>: The project does not have input source code. </li> <li>  <code>S3</code>: The source code is in an Amazon Simple Storage Service (Amazon S3) input bucket. </li> </ul><br/> Information about the Git submodules configuration for the build project. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>gitSubmodulesConfig</b>:  Information about the Git submodules configuration for the build project. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>artifactIdentifier</i>  An identifier for this artifact definition. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>encryptionDisabled</i>  Information that tells you if encryption for build artifacts is disabled. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>overrideArtifactName</i>  If this flag is set, a name specified in the build spec file overrides the artifact name. The name specified in a build spec file is calculated at build time and uses the Shell Command Language. For example, you can append a date and time to your artifact name so that it is always unique. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>md5sum</i> The MD5 hash of the build artifact. You can use this hash along with a checksum tool to confirm file integrity and authenticity. <note> This value is available only if the build project's <code>packaging</code> value is set to <code>ZIP</code>. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>sha256sum</i> The SHA-256 hash of the build artifact. You can use this hash along with a checksum tool to confirm file integrity and authenticity. <note> This value is available only if the build project's <code>packaging</code> value is set to <code>ZIP</code>. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> Information about the location of the build artifacts.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>modes</i>  If you use a <code>LOCAL</code> cache, the local cache mode. You can use one or more local cache modes at the same time.  <ul> <li>  <code>LOCAL_SOURCE_CACHE</code> mode caches Git metadata for primary and secondary sources. After the cache is created, subsequent builds pull only the change between commits. This mode is a good choice for projects with a clean working directory and a source that is a large Git repository. If you choose this option and your project does not use a Git repository (GitHub, GitHub Enterprise, or Bitbucket), the option is ignored.  </li> <li>  <code>LOCAL_DOCKER_LAYER_CACHE</code> mode caches existing Docker layers. This mode is a good choice for projects that build or pull large Docker images. It can prevent the performance issues caused by pulling large Docker images down from the network.  <note> <ul> <li>  You can use a Docker layer cache in the Linux environment only.  </li> <li>  The <code>privileged</code> flag must be set so that your project has the required Docker permissions.  </li> <li>  You should consider the security implications before you use a Docker layer cache.  </li> </ul> </note> </li> </ul> <ul> <li>  <code>LOCAL_CUSTOM_CACHE</code> mode caches directories you specify in the buildspec file. This mode is a good choice if your build scenario is not suited to one of the other three local cache modes. If you use a custom cache:  <ul> <li>  Only directories can be specified for caching. You cannot specify individual files.  </li> <li>  Symlinks are used to reference cached directories.  </li> <li>  Cached directories are linked to your build before it downloads its project sources. Cached items are overriden if a source item has the same name. Directories are specified using cache paths in the buildspec file.  </li> </ul> </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> Information about the cache location:  <ul> <li>  <code>NO_CACHE</code> or <code>LOCAL</code>: This value is ignored. </li> <li>  <code>S3</code>: This is the S3 bucket name/prefix. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of cache used by the build project. Valid values include: <ul> <li>  <code>NO_CACHE</code>: The build project does not use any cache. </li> <li>  <code>S3</code>: The build project reads and writes from and to S3. </li> <li>  <code>LOCAL</code>: The build project stores a cache locally on a build host that is only available to that build host. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>certificate</i> The certificate to use with this build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>privilegedMode</i> Enables running the Docker daemon inside a Docker container. Set to true only if the build project is be used to build Docker images, and the specified build environment image is not provided by AWS CodeBuild with Docker support. Otherwise, all associated builds that attempt to interact with the Docker daemon fail. You must also start the Docker daemon so that builds can interact with it. One way to do this is to initialize the Docker daemon during the install phase of your build spec by running the following build commands. (Do not run these commands if the specified build environment image is provided by AWS CodeBuild with Docker support.) If the operating system's base image is Ubuntu Linux:  <code>- nohup /usr/local/bin/dockerd --host=unix:///var/run/docker.sock --host=tcp://0.0.0.0:2375 --storage-driver=overlay&amp;</code>   <code>- timeout 15 sh -c "until docker info; do echo .; sleep 1; done"</code>  If the operating system's base image is Alpine Linux and the previous command does not work, add the <code>-t</code> argument to <code>timeout</code>:  <code>- nohup /usr/local/bin/dockerd --host=unix:///var/run/docker.sock --host=tcp://0.0.0.0:2375 --storage-driver=overlay&amp;</code>   <code>- timeout -t 15 sh -c "until docker info; do echo .; sleep 1; done"</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>environmentVariables</i> A set of environment variables to make available to builds for this build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>computeType</i> Information about the compute resources the build project uses. Available values include: <ul> <li>  <code>BUILD_GENERAL1_SMALL</code>: Use up to 3 GB memory and 2 vCPUs for builds. </li> <li>  <code>BUILD_GENERAL1_MEDIUM</code>: Use up to 7 GB memory and 4 vCPUs for builds. </li> <li>  <code>BUILD_GENERAL1_LARGE</code>: Use up to 15 GB memory and 8 vCPUs for builds. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>image</i> The image tag or image digest that identifies the Docker image to use for this build project. Use the following formats: <ul> <li> For an image tag: <code>registry/repository:tag</code>. For example, to specify an image with the tag "latest," use <code>registry/repository:latest</code>. </li> <li> For an image digest: <code>registry/repository@digest</code>. For example, to specify an image with the digest "sha256:cbbf2f9a99b47fc460d422812b6a5adff7dfee951d8fa2e4a98caa0382cfbdbf," use <code>registry/repository@sha256:cbbf2f9a99b47fc460d422812b6a5adff7dfee951d8fa2e4a98caa0382cfbdbf</code>. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of build environment to use for related builds.<br/> The credentials for access to a private registry.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>registryCredential</b>:  The credentials for access to a private registry.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>s3DeepLink</i>  The URL to a build log in an S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>deepLink</i> The URL to an individual build log in Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>streamName</i> The name of the Amazon CloudWatch Logs stream for the build logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>groupName</i> The name of the Amazon CloudWatch Logs group for the build logs.<br/> Information about Amazon CloudWatch Logs for a build project. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>cloudWatchLogs</b>:  Information about Amazon CloudWatch Logs for a build project. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>securityGroupIds</i> A list of one or more security groups IDs in your Amazon VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>subnets</i> A list of one or more subnet IDs in your Amazon VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>vpcId</i> The ID of the Amazon VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>networkInterfaceId</i> The ID of the network interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>subnetId</i> The ID of the subnet.<br/>        """
        pass

    def stop_build(self, id: str) -> StopBuildOutput:
        """Attempts to stop running a build.

        :param self:
        :param id:  The ID of the build.<br/>
        :return: build:  Information about the build.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>encryptionKey</i> The AWS Key Management Service (AWS KMS) customer master key (CMK) to be used for encrypting the build output artifacts. <note>  You can use a cross-account KMS key to encrypt the build output artifacts if your service role has permission to that key.  </note> You can specify either the Amazon Resource Name (ARN) of the CMK or, if available, the CMK's alias (using the format <code>alias/<i>alias-name</i> </code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>networkInterface</i> Describes a network interface.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>networkInterface</b>: Describes a network interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vpcConfig</i> If your AWS CodeBuild project accesses resources in an Amazon VPC, you provide this parameter that identifies the VPC ID and the list of security group IDs and subnet IDs. The security groups and subnets must belong to the same VPC. You must provide at least one security group and one subnet ID.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>vpcConfig</b>: If your AWS CodeBuild project accesses resources in an Amazon VPC, you provide this parameter that identifies the VPC ID and the list of security group IDs and subnet IDs. The security groups and subnets must belong to the same VPC. You must provide at least one security group and one subnet ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>initiator</i> The entity that started the build. Valid values include: <ul> <li> If AWS CodePipeline started the build, the pipeline's name (for example, <code>codepipeline/my-demo-pipeline</code>). </li> <li> If an AWS Identity and Access Management (IAM) user started the build, the user's name (for example, <code>MyUserName</code>). </li> <li> If the Jenkins plugin for AWS CodeBuild started the build, the string <code>CodeBuild-Jenkins-Plugin</code>. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>buildComplete</i> Whether the build is complete. True if complete; otherwise, false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>queuedTimeoutInMinutes</i>  The number of minutes a build is allowed to be queued before it times out. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>timeoutInMinutes</i> How long, in minutes, for AWS CodeBuild to wait before timing out this build if it does not get marked as completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>logs</i> Information about the build's logs in Amazon CloudWatch Logs.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>logs</b>: Information about the build's logs in Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceRole</i> The name of a service role used for this build.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>environment</i> Information about the build environment for this build.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>environment</b>: Information about the build environment for this build.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cache</i> Information about the cache for the build.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>cache</b>: Information about the cache for the build.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>secondaryArtifacts</i>  An array of <code>ProjectArtifacts</code> objects. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>artifacts</i> Information about the output artifacts for the build.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>artifacts</b>: Information about the output artifacts for the build.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>secondarySourceVersions</i>  An array of <code>ProjectSourceVersion</code> objects. Each <code>ProjectSourceVersion</code> must be one of:  <ul> <li> For AWS CodeCommit: the commit ID to use. </li> <li> For GitHub: the commit ID, pull request ID, branch name, or tag name that corresponds to the version of the source code you want to build. If a pull request ID is specified, it must use the format <code>pr/pull-request-ID</code> (for example, <code>pr/25</code>). If a branch name is specified, the branch's HEAD commit ID is used. If not specified, the default branch's HEAD commit ID is used. </li> <li> For Bitbucket: the commit ID, branch name, or tag name that corresponds to the version of the source code you want to build. If a branch name is specified, the branch's HEAD commit ID is used. If not specified, the default branch's HEAD commit ID is used. </li> <li> For Amazon Simple Storage Service (Amazon S3): the version ID of the object that represents the build input ZIP file to use. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>secondarySources</i>  An array of <code>ProjectSource</code> objects. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>source</i> Information about the source code to be built.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>source</b>: Information about the source code to be built.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>phases</i> Information about all previous build phases that are complete and information about any current build phase that is not yet complete.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>projectName</i> The name of the AWS CodeBuild project.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resolvedSourceVersion</i>  An identifier for the version of this build's source code.  <ul> <li>  For AWS CodeCommit, GitHub, GitHub Enterprise, and BitBucket, the commit ID.  </li> <li>  For AWS CodePipeline, the source revision provided by AWS CodePipeline.  </li> <li>  For Amazon Simple Storage Service (Amazon S3), this does not apply.  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>sourceVersion</i> Any version identifier for the version of the source code to be built. If <code>sourceVersion</code> is specified at the project level, then this <code>sourceVersion</code> (at the build level) takes precedence.   For more information, see <a href="https://docs.aws.amazon.com/codebuild/latest/userguide/sample-source-version.html">Source Version Sample with CodeBuild</a> in the <i>AWS CodeBuild User Guide</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>buildStatus</i> The current status of the build. Valid values include: <ul> <li>  <code>FAILED</code>: The build failed. </li> <li>  <code>FAULT</code>: The build faulted. </li> <li>  <code>IN_PROGRESS</code>: The build is still in progress. </li> <li>  <code>STOPPED</code>: The build stopped. </li> <li>  <code>SUCCEEDED</code>: The build succeeded. </li> <li>  <code>TIMED_OUT</code>: The build timed out. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>currentPhase</i> The current build phase.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>endTime</i> When the build process ended, expressed in Unix time format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>startTime</i> When the build process started, expressed in Unix time format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the build.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The unique ID for the build.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>gitCloneDepth</i> Information about the Git clone depth for the build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> Information about the location of the source code to be built. Valid values include: <ul> <li> For source code settings that are specified in the source action of a pipeline in AWS CodePipeline, <code>location</code> should not be specified. If it is specified, AWS CodePipeline ignores it. This is because AWS CodePipeline uses the settings in a pipeline's source action instead of this value. </li> <li> For source code in an AWS CodeCommit repository, the HTTPS clone URL to the repository that contains the source code and the build spec (for example, <code>https://git-codecommit.<i>region-ID</i>.amazonaws.com/v1/repos/<i>repo-name</i> </code>). </li> <li> For source code in an Amazon Simple Storage Service (Amazon S3) input bucket, one of the following.  <ul> <li>  The path to the ZIP file that contains the source code (for example, <code> <i>bucket-name</i>/<i>path</i>/<i>to</i>/<i>object-name</i>.zip</code>).  </li> <li>  The path to the folder that contains the source code (for example, <code> <i>bucket-name</i>/<i>path</i>/<i>to</i>/<i>source-code</i>/<i>folder</i>/</code>).  </li> </ul> </li> <li> For source code in a GitHub repository, the HTTPS clone URL to the repository that contains the source and the build spec. You must connect your AWS account to your GitHub account. Use the AWS CodeBuild console to start creating a build project. When you use the console to connect (or reconnect) with GitHub, on the GitHub <b>Authorize application</b> page, for <b>Organization access</b>, choose <b>Request access</b> next to each repository you want to allow AWS CodeBuild to have access to, and then choose <b>Authorize application</b>. (After you have connected to your GitHub account, you do not need to finish creating the build project. You can leave the AWS CodeBuild console.) To instruct AWS CodeBuild to use this connection, in the <code>source</code> object, set the <code>auth</code> object's <code>type</code> value to <code>OAUTH</code>. </li> <li> For source code in a Bitbucket repository, the HTTPS clone URL to the repository that contains the source and the build spec. You must connect your AWS account to your Bitbucket account. Use the AWS CodeBuild console to start creating a build project. When you use the console to connect (or reconnect) with Bitbucket, on the Bitbucket <b>Confirm access to your account</b> page, choose <b>Grant access</b>. (After you have connected to your Bitbucket account, you do not need to finish creating the build project. You can leave the AWS CodeBuild console.) To instruct AWS CodeBuild to use this connection, in the <code>source</code> object, set the <code>auth</code> object's <code>type</code> value to <code>OAUTH</code>. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of repository that contains the source code to be built. Valid values include: <ul> <li>  <code>BITBUCKET</code>: The source code is in a Bitbucket repository. </li> <li>  <code>CODECOMMIT</code>: The source code is in an AWS CodeCommit repository. </li> <li>  <code>CODEPIPELINE</code>: The source code settings are specified in the source action of a pipeline in AWS CodePipeline. </li> <li>  <code>GITHUB</code>: The source code is in a GitHub repository. </li> <li>  <code>NO_SOURCE</code>: The project does not have input source code. </li> <li>  <code>S3</code>: The source code is in an Amazon Simple Storage Service (Amazon S3) input bucket. </li> </ul><br/> Information about the Git submodules configuration for the build project. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>gitSubmodulesConfig</b>:  Information about the Git submodules configuration for the build project. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>artifactIdentifier</i>  An identifier for this artifact definition. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>encryptionDisabled</i>  Information that tells you if encryption for build artifacts is disabled. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>overrideArtifactName</i>  If this flag is set, a name specified in the build spec file overrides the artifact name. The name specified in a build spec file is calculated at build time and uses the Shell Command Language. For example, you can append a date and time to your artifact name so that it is always unique. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>md5sum</i> The MD5 hash of the build artifact. You can use this hash along with a checksum tool to confirm file integrity and authenticity. <note> This value is available only if the build project's <code>packaging</code> value is set to <code>ZIP</code>. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>sha256sum</i> The SHA-256 hash of the build artifact. You can use this hash along with a checksum tool to confirm file integrity and authenticity. <note> This value is available only if the build project's <code>packaging</code> value is set to <code>ZIP</code>. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> Information about the location of the build artifacts.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>modes</i>  If you use a <code>LOCAL</code> cache, the local cache mode. You can use one or more local cache modes at the same time.  <ul> <li>  <code>LOCAL_SOURCE_CACHE</code> mode caches Git metadata for primary and secondary sources. After the cache is created, subsequent builds pull only the change between commits. This mode is a good choice for projects with a clean working directory and a source that is a large Git repository. If you choose this option and your project does not use a Git repository (GitHub, GitHub Enterprise, or Bitbucket), the option is ignored.  </li> <li>  <code>LOCAL_DOCKER_LAYER_CACHE</code> mode caches existing Docker layers. This mode is a good choice for projects that build or pull large Docker images. It can prevent the performance issues caused by pulling large Docker images down from the network.  <note> <ul> <li>  You can use a Docker layer cache in the Linux environment only.  </li> <li>  The <code>privileged</code> flag must be set so that your project has the required Docker permissions.  </li> <li>  You should consider the security implications before you use a Docker layer cache.  </li> </ul> </note> </li> </ul> <ul> <li>  <code>LOCAL_CUSTOM_CACHE</code> mode caches directories you specify in the buildspec file. This mode is a good choice if your build scenario is not suited to one of the other three local cache modes. If you use a custom cache:  <ul> <li>  Only directories can be specified for caching. You cannot specify individual files.  </li> <li>  Symlinks are used to reference cached directories.  </li> <li>  Cached directories are linked to your build before it downloads its project sources. Cached items are overriden if a source item has the same name. Directories are specified using cache paths in the buildspec file.  </li> </ul> </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> Information about the cache location:  <ul> <li>  <code>NO_CACHE</code> or <code>LOCAL</code>: This value is ignored. </li> <li>  <code>S3</code>: This is the S3 bucket name/prefix. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of cache used by the build project. Valid values include: <ul> <li>  <code>NO_CACHE</code>: The build project does not use any cache. </li> <li>  <code>S3</code>: The build project reads and writes from and to S3. </li> <li>  <code>LOCAL</code>: The build project stores a cache locally on a build host that is only available to that build host. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>certificate</i> The certificate to use with this build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>privilegedMode</i> Enables running the Docker daemon inside a Docker container. Set to true only if the build project is be used to build Docker images, and the specified build environment image is not provided by AWS CodeBuild with Docker support. Otherwise, all associated builds that attempt to interact with the Docker daemon fail. You must also start the Docker daemon so that builds can interact with it. One way to do this is to initialize the Docker daemon during the install phase of your build spec by running the following build commands. (Do not run these commands if the specified build environment image is provided by AWS CodeBuild with Docker support.) If the operating system's base image is Ubuntu Linux:  <code>- nohup /usr/local/bin/dockerd --host=unix:///var/run/docker.sock --host=tcp://0.0.0.0:2375 --storage-driver=overlay&amp;</code>   <code>- timeout 15 sh -c "until docker info; do echo .; sleep 1; done"</code>  If the operating system's base image is Alpine Linux and the previous command does not work, add the <code>-t</code> argument to <code>timeout</code>:  <code>- nohup /usr/local/bin/dockerd --host=unix:///var/run/docker.sock --host=tcp://0.0.0.0:2375 --storage-driver=overlay&amp;</code>   <code>- timeout -t 15 sh -c "until docker info; do echo .; sleep 1; done"</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>environmentVariables</i> A set of environment variables to make available to builds for this build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>computeType</i> Information about the compute resources the build project uses. Available values include: <ul> <li>  <code>BUILD_GENERAL1_SMALL</code>: Use up to 3 GB memory and 2 vCPUs for builds. </li> <li>  <code>BUILD_GENERAL1_MEDIUM</code>: Use up to 7 GB memory and 4 vCPUs for builds. </li> <li>  <code>BUILD_GENERAL1_LARGE</code>: Use up to 15 GB memory and 8 vCPUs for builds. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>image</i> The image tag or image digest that identifies the Docker image to use for this build project. Use the following formats: <ul> <li> For an image tag: <code>registry/repository:tag</code>. For example, to specify an image with the tag "latest," use <code>registry/repository:latest</code>. </li> <li> For an image digest: <code>registry/repository@digest</code>. For example, to specify an image with the digest "sha256:cbbf2f9a99b47fc460d422812b6a5adff7dfee951d8fa2e4a98caa0382cfbdbf," use <code>registry/repository@sha256:cbbf2f9a99b47fc460d422812b6a5adff7dfee951d8fa2e4a98caa0382cfbdbf</code>. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of build environment to use for related builds.<br/> The credentials for access to a private registry.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>registryCredential</b>:  The credentials for access to a private registry.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>s3DeepLink</i>  The URL to a build log in an S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>deepLink</i> The URL to an individual build log in Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>streamName</i> The name of the Amazon CloudWatch Logs stream for the build logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>groupName</i> The name of the Amazon CloudWatch Logs group for the build logs.<br/> Information about Amazon CloudWatch Logs for a build project. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>cloudWatchLogs</b>:  Information about Amazon CloudWatch Logs for a build project. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>securityGroupIds</i> A list of one or more security groups IDs in your Amazon VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>subnets</i> A list of one or more subnet IDs in your Amazon VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>vpcId</i> The ID of the Amazon VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>networkInterfaceId</i> The ID of the network interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>subnetId</i> The ID of the subnet.<br/>        """
        pass

    def update_project(self, name: str, description: str=None, source: ProjectSource=None, secondarySources: List[ProjectSource]=None, sourceVersion: str=None, secondarySourceVersions: List[ProjectSourceVersion]=None, artifacts: ProjectArtifacts=None, secondaryArtifacts: List[ProjectArtifacts]=None, cache: ProjectCache=None, environment: ProjectEnvironment=None, serviceRole: str=None, timeoutInMinutes: int=None, queuedTimeoutInMinutes: int=None, encryptionKey: str=None, tags: List[Tag]=None, vpcConfig: VpcConfig=None, badgeEnabled: WrapperBoolean=None, logsConfig: LogsConfig=None) -> UpdateProjectOutput:
        """Changes the settings of a build project.

        :param self:
        :param logsConfig:   Information about logs for the build project. A project can create logs in Amazon CloudWatch Logs, logs in an S3 bucket, or both. :<br/>badgeEnabled:  Set this to true to generate a publicly accessible URL for your project's build badge.<br/>vpcConfig:  VpcConfig enables AWS CodeBuild to access resources in an Amazon VPC.:<br/>tags:  The replacement set of tags for this build project. These tags are available for use by AWS services that support AWS CodeBuild build project tags.<br/>encryptionKey:  The AWS Key Management Service (AWS KMS) customer master key (CMK) to be used for encrypting the build output artifacts. <note>  You can use a cross-account KMS key to encrypt the build output artifacts if your service role has permission to that key.  </note> You can specify either the Amazon Resource Name (ARN) of the CMK or, if available, the CMK's alias (using the format <code>alias/<i>alias-name</i> </code>).<br/>queuedTimeoutInMinutes:   The number of minutes a build is allowed to be queued before it times out. <br/>timeoutInMinutes:  The replacement value in minutes, from 5 to 480 (8 hours), for AWS CodeBuild to wait before timing out any related build that did not get marked as completed.<br/>serviceRole:  The replacement ARN of the AWS Identity and Access Management (IAM) role that enables AWS CodeBuild to interact with dependent AWS services on behalf of the AWS account.<br/>environment:  Information to be changed about the build environment for the build project.:<br/>cache:  Stores recently used information so that it can be quickly accessed at a later time.:<br/>secondaryArtifacts:   An array of <code>ProjectSource</code> objects. <br/>artifacts:  Information to be changed about the build output artifacts for the build project.:<br/>secondarySourceVersions:   An array of <code>ProjectSourceVersion</code> objects. If <code>secondarySourceVersions</code> is specified at the build level, then they take over these <code>secondarySourceVersions</code> (at the project level). <br/>sourceVersion:   A version of the build input to be built for this project. If not specified, the latest version is used. If specified, it must be one of:  <ul> <li> For AWS CodeCommit: the commit ID to use. </li> <li> For GitHub: the commit ID, pull request ID, branch name, or tag name that corresponds to the version of the source code you want to build. If a pull request ID is specified, it must use the format <code>pr/pull-request-ID</code> (for example <code>pr/25</code>). If a branch name is specified, the branch's HEAD commit ID is used. If not specified, the default branch's HEAD commit ID is used. </li> <li> For Bitbucket: the commit ID, branch name, or tag name that corresponds to the version of the source code you want to build. If a branch name is specified, the branch's HEAD commit ID is used. If not specified, the default branch's HEAD commit ID is used. </li> <li> For Amazon Simple Storage Service (Amazon S3): the version ID of the object that represents the build input ZIP file to use. </li> </ul>  If <code>sourceVersion</code> is specified at the build level, then that version takes precedence over this <code>sourceVersion</code> (at the project level).   For more information, see <a href="https://docs.aws.amazon.com/codebuild/latest/userguide/sample-source-version.html">Source Version Sample with CodeBuild</a> in the <i>AWS CodeBuild User Guide</i>. <br/>secondarySources:   An array of <code>ProjectSource</code> objects. <br/>source:  Information to be changed about the build input source code for the build project.:<br/>description:  A new or replacement description of the build project.<br/>name:  The name of the build project. <note> You cannot change a build project's name. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>sourceIdentifier</i>  An identifier for this project source. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>insecureSsl</i> Enable this flag to ignore SSL warnings while connecting to the project source code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>reportBuildStatus</i>  Set to true to report the status of a build's start and finish to your source provider. This option is valid only when your source provider is GitHub, GitHub Enterprise, or Bitbucket. If this is set and you use a different source provider, an invalidInputException is thrown. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>auth</i> Information about the authorization settings for AWS CodeBuild to access the source code to be built. This information is for the AWS CodeBuild console's use only. Your code should not get or set this information directly.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>auth</b>: Information about the authorization settings for AWS CodeBuild to access the source code to be built. This information is for the AWS CodeBuild console's use only. Your code should not get or set this information directly.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>buildspec</i> The build spec declaration to use for the builds in this build project. If this value is not specified, a build spec must be included along with the source code to be built.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>gitSubmodulesConfig</i>  Information about the Git submodules configuration for the build project. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>gitSubmodulesConfig</b>:  Information about the Git submodules configuration for the build project. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>gitCloneDepth</i> Information about the Git clone depth for the build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> Information about the location of the source code to be built. Valid values include: <ul> <li> For source code settings that are specified in the source action of a pipeline in AWS CodePipeline, <code>location</code> should not be specified. If it is specified, AWS CodePipeline ignores it. This is because AWS CodePipeline uses the settings in a pipeline's source action instead of this value. </li> <li> For source code in an AWS CodeCommit repository, the HTTPS clone URL to the repository that contains the source code and the build spec (for example, <code>https://git-codecommit.<i>region-ID</i>.amazonaws.com/v1/repos/<i>repo-name</i> </code>). </li> <li> For source code in an Amazon Simple Storage Service (Amazon S3) input bucket, one of the following.  <ul> <li>  The path to the ZIP file that contains the source code (for example, <code> <i>bucket-name</i>/<i>path</i>/<i>to</i>/<i>object-name</i>.zip</code>).  </li> <li>  The path to the folder that contains the source code (for example, <code> <i>bucket-name</i>/<i>path</i>/<i>to</i>/<i>source-code</i>/<i>folder</i>/</code>).  </li> </ul> </li> <li> For source code in a GitHub repository, the HTTPS clone URL to the repository that contains the source and the build spec. You must connect your AWS account to your GitHub account. Use the AWS CodeBuild console to start creating a build project. When you use the console to connect (or reconnect) with GitHub, on the GitHub <b>Authorize application</b> page, for <b>Organization access</b>, choose <b>Request access</b> next to each repository you want to allow AWS CodeBuild to have access to, and then choose <b>Authorize application</b>. (After you have connected to your GitHub account, you do not need to finish creating the build project. You can leave the AWS CodeBuild console.) To instruct AWS CodeBuild to use this connection, in the <code>source</code> object, set the <code>auth</code> object's <code>type</code> value to <code>OAUTH</code>. </li> <li> For source code in a Bitbucket repository, the HTTPS clone URL to the repository that contains the source and the build spec. You must connect your AWS account to your Bitbucket account. Use the AWS CodeBuild console to start creating a build project. When you use the console to connect (or reconnect) with Bitbucket, on the Bitbucket <b>Confirm access to your account</b> page, choose <b>Grant access</b>. (After you have connected to your Bitbucket account, you do not need to finish creating the build project. You can leave the AWS CodeBuild console.) To instruct AWS CodeBuild to use this connection, in the <code>source</code> object, set the <code>auth</code> object's <code>type</code> value to <code>OAUTH</code>. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of repository that contains the source code to be built. Valid values include: <ul> <li>  <code>BITBUCKET</code>: The source code is in a Bitbucket repository. </li> <li>  <code>CODECOMMIT</code>: The source code is in an AWS CodeCommit repository. </li> <li>  <code>CODEPIPELINE</code>: The source code settings are specified in the source action of a pipeline in AWS CodePipeline. </li> <li>  <code>GITHUB</code>: The source code is in a GitHub repository. </li> <li>  <code>NO_SOURCE</code>: The project does not have input source code. </li> <li>  <code>S3</code>: The source code is in an Amazon Simple Storage Service (Amazon S3) input bucket. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>fetchSubmodules</i>  Set to true to fetch Git submodules for your AWS CodeBuild build project. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>resource</i> The resource value that applies to the specified authorization type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> <note>  This data type is deprecated and is no longer accurate or used.  </note> The authorization type to use. The only valid value is <code>OAUTH</code>, which represents the OAuth authorization type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>artifactIdentifier</i>  An identifier for this artifact definition. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>encryptionDisabled</i>  Set to true if you do not want your output artifacts encrypted. This option is valid only if your artifacts type is Amazon Simple Storage Service (Amazon S3). If this is set with another artifacts type, an invalidInputException is thrown. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>overrideArtifactName</i>  If this flag is set, a name specified in the build spec file overrides the artifact name. The name specified in a build spec file is calculated at build time and uses the Shell Command Language. For example, you can append a date and time to your artifact name so that it is always unique. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>packaging</i> The type of build output artifact to create: <ul> <li> If <code>type</code> is set to <code>CODEPIPELINE</code>, AWS CodePipeline ignores this value if specified. This is because AWS CodePipeline manages its build output artifacts instead of AWS CodeBuild. </li> <li> If <code>type</code> is set to <code>NO_ARTIFACTS</code>, this value is ignored if specified, because no build output is produced. </li> <li> If <code>type</code> is set to <code>S3</code>, valid values include: <ul> <li>  <code>NONE</code>: AWS CodeBuild creates in the output bucket a folder that contains the build output. This is the default if <code>packaging</code> is not specified. </li> <li>  <code>ZIP</code>: AWS CodeBuild creates in the output bucket a ZIP file that contains the build output. </li> </ul> </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> Along with <code>path</code> and <code>namespaceType</code>, the pattern that AWS CodeBuild uses to name and store the output artifact: <ul> <li> If <code>type</code> is set to <code>CODEPIPELINE</code>, AWS CodePipeline ignores this value if specified. This is because AWS CodePipeline manages its build output names instead of AWS CodeBuild. </li> <li> If <code>type</code> is set to <code>NO_ARTIFACTS</code>, this value is ignored if specified, because no build output is produced. </li> <li> If <code>type</code> is set to <code>S3</code>, this is the name of the output artifact object. If you set the name to be a forward slash ("/"), the artifact is stored in the root of the output bucket. </li> </ul> For example: <ul> <li>  If <code>path</code> is set to <code>MyArtifacts</code>, <code>namespaceType</code> is set to <code>BUILD_ID</code>, and <code>name</code> is set to <code>MyArtifact.zip</code>, then the output artifact is stored in <code>MyArtifacts/<i>build-ID</i>/MyArtifact.zip</code>.  </li> <li>  If <code>path</code> is empty, <code>namespaceType</code> is set to <code>NONE</code>, and <code>name</code> is set to "<code>/</code>", the output artifact is stored in the root of the output bucket.  </li> <li>  If <code>path</code> is set to <code>MyArtifacts</code>, <code>namespaceType</code> is set to <code>BUILD_ID</code>, and <code>name</code> is set to "<code>/</code>", the output artifact is stored in <code>MyArtifacts/<i>build-ID</i> </code>.  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>namespaceType</i> Along with <code>path</code> and <code>name</code>, the pattern that AWS CodeBuild uses to determine the name and location to store the output artifact: <ul> <li> If <code>type</code> is set to <code>CODEPIPELINE</code>, AWS CodePipeline ignores this value if specified. This is because AWS CodePipeline manages its build output names instead of AWS CodeBuild. </li> <li> If <code>type</code> is set to <code>NO_ARTIFACTS</code>, this value is ignored if specified, because no build output is produced. </li> <li> If <code>type</code> is set to <code>S3</code>, valid values include: <ul> <li>  <code>BUILD_ID</code>: Include the build ID in the location of the build output artifact. </li> <li>  <code>NONE</code>: Do not include the build ID. This is the default if <code>namespaceType</code> is not specified. </li> </ul> </li> </ul> For example, if <code>path</code> is set to <code>MyArtifacts</code>, <code>namespaceType</code> is set to <code>BUILD_ID</code>, and <code>name</code> is set to <code>MyArtifact.zip</code>, the output artifact is stored in <code>MyArtifacts/<i>build-ID</i>/MyArtifact.zip</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>path</i> Along with <code>namespaceType</code> and <code>name</code>, the pattern that AWS CodeBuild uses to name and store the output artifact: <ul> <li> If <code>type</code> is set to <code>CODEPIPELINE</code>, AWS CodePipeline ignores this value if specified. This is because AWS CodePipeline manages its build output names instead of AWS CodeBuild. </li> <li> If <code>type</code> is set to <code>NO_ARTIFACTS</code>, this value is ignored if specified, because no build output is produced. </li> <li> If <code>type</code> is set to <code>S3</code>, this is the path to the output artifact. If <code>path</code> is not specified, <code>path</code> is not used. </li> </ul> For example, if <code>path</code> is set to <code>MyArtifacts</code>, <code>namespaceType</code> is set to <code>NONE</code>, and <code>name</code> is set to <code>MyArtifact.zip</code>, the output artifact is stored in the output bucket at <code>MyArtifacts/MyArtifact.zip</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> Information about the build output artifact location: <ul> <li> If <code>type</code> is set to <code>CODEPIPELINE</code>, AWS CodePipeline ignores this value if specified. This is because AWS CodePipeline manages its build output locations instead of AWS CodeBuild. </li> <li> If <code>type</code> is set to <code>NO_ARTIFACTS</code>, this value is ignored if specified, because no build output is produced. </li> <li> If <code>type</code> is set to <code>S3</code>, this is the name of the output bucket. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of build output artifact. Valid values include: <ul> <li>  <code>CODEPIPELINE</code>: The build project has build output generated through AWS CodePipeline. </li> <li>  <code>NO_ARTIFACTS</code>: The build project does not produce any build output. </li> <li>  <code>S3</code>: The build project stores build output in Amazon Simple Storage Service (Amazon S3). </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>modes</i>  If you use a <code>LOCAL</code> cache, the local cache mode. You can use one or more local cache modes at the same time.  <ul> <li>  <code>LOCAL_SOURCE_CACHE</code> mode caches Git metadata for primary and secondary sources. After the cache is created, subsequent builds pull only the change between commits. This mode is a good choice for projects with a clean working directory and a source that is a large Git repository. If you choose this option and your project does not use a Git repository (GitHub, GitHub Enterprise, or Bitbucket), the option is ignored.  </li> <li>  <code>LOCAL_DOCKER_LAYER_CACHE</code> mode caches existing Docker layers. This mode is a good choice for projects that build or pull large Docker images. It can prevent the performance issues caused by pulling large Docker images down from the network.  <note> <ul> <li>  You can use a Docker layer cache in the Linux environment only.  </li> <li>  The <code>privileged</code> flag must be set so that your project has the required Docker permissions.  </li> <li>  You should consider the security implications before you use a Docker layer cache.  </li> </ul> </note> </li> </ul> <ul> <li>  <code>LOCAL_CUSTOM_CACHE</code> mode caches directories you specify in the buildspec file. This mode is a good choice if your build scenario is not suited to one of the other three local cache modes. If you use a custom cache:  <ul> <li>  Only directories can be specified for caching. You cannot specify individual files.  </li> <li>  Symlinks are used to reference cached directories.  </li> <li>  Cached directories are linked to your build before it downloads its project sources. Cached items are overriden if a source item has the same name. Directories are specified using cache paths in the buildspec file.  </li> </ul> </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> Information about the cache location:  <ul> <li>  <code>NO_CACHE</code> or <code>LOCAL</code>: This value is ignored. </li> <li>  <code>S3</code>: This is the S3 bucket name/prefix. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of cache used by the build project. Valid values include: <ul> <li>  <code>NO_CACHE</code>: The build project does not use any cache. </li> <li>  <code>S3</code>: The build project reads and writes from and to S3. </li> <li>  <code>LOCAL</code>: The build project stores a cache locally on a build host that is only available to that build host. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>imagePullCredentialsType</i>  The type of credentials AWS CodeBuild uses to pull images in your build. There are two valid values:  <ul> <li>  <code>CODEBUILD</code> specifies that AWS CodeBuild uses its own credentials. This requires that you modify your ECR repository policy to trust AWS CodeBuild's service principal.  </li> <li>  <code>SERVICE_ROLE</code> specifies that AWS CodeBuild uses your build project's service role.  </li> </ul>  When you use a cross-account or private registry image, you must use SERVICE_ROLE credentials. When you use an AWS CodeBuild curated image, you must use CODEBUILD credentials. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>registryCredential</i>  The credentials for access to a private registry.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>registryCredential</b>:  The credentials for access to a private registry.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>certificate</i> The certificate to use with this build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>privilegedMode</i> Enables running the Docker daemon inside a Docker container. Set to true only if the build project is be used to build Docker images, and the specified build environment image is not provided by AWS CodeBuild with Docker support. Otherwise, all associated builds that attempt to interact with the Docker daemon fail. You must also start the Docker daemon so that builds can interact with it. One way to do this is to initialize the Docker daemon during the install phase of your build spec by running the following build commands. (Do not run these commands if the specified build environment image is provided by AWS CodeBuild with Docker support.) If the operating system's base image is Ubuntu Linux:  <code>- nohup /usr/local/bin/dockerd --host=unix:///var/run/docker.sock --host=tcp://0.0.0.0:2375 --storage-driver=overlay&amp;</code>   <code>- timeout 15 sh -c "until docker info; do echo .; sleep 1; done"</code>  If the operating system's base image is Alpine Linux and the previous command does not work, add the <code>-t</code> argument to <code>timeout</code>:  <code>- nohup /usr/local/bin/dockerd --host=unix:///var/run/docker.sock --host=tcp://0.0.0.0:2375 --storage-driver=overlay&amp;</code>   <code>- timeout -t 15 sh -c "until docker info; do echo .; sleep 1; done"</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>environmentVariables</i> A set of environment variables to make available to builds for this build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>computeType</i> Information about the compute resources the build project uses. Available values include: <ul> <li>  <code>BUILD_GENERAL1_SMALL</code>: Use up to 3 GB memory and 2 vCPUs for builds. </li> <li>  <code>BUILD_GENERAL1_MEDIUM</code>: Use up to 7 GB memory and 4 vCPUs for builds. </li> <li>  <code>BUILD_GENERAL1_LARGE</code>: Use up to 15 GB memory and 8 vCPUs for builds. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>image</i> The image tag or image digest that identifies the Docker image to use for this build project. Use the following formats: <ul> <li> For an image tag: <code>registry/repository:tag</code>. For example, to specify an image with the tag "latest," use <code>registry/repository:latest</code>. </li> <li> For an image digest: <code>registry/repository@digest</code>. For example, to specify an image with the digest "sha256:cbbf2f9a99b47fc460d422812b6a5adff7dfee951d8fa2e4a98caa0382cfbdbf," use <code>registry/repository@sha256:cbbf2f9a99b47fc460d422812b6a5adff7dfee951d8fa2e4a98caa0382cfbdbf</code>. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of build environment to use for related builds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>credentialProvider</i>  The service that created the credentials to access a private Docker registry. The valid value, SECRETS_MANAGER, is for AWS Secrets Manager. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>credential</i>  The Amazon Resource Name (ARN) or name of credentials created using AWS Secrets Manager.  <note>  The <code>credential</code> can use the name of the credentials only if they exist in your current region.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>securityGroupIds</i> A list of one or more security groups IDs in your Amazon VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>subnets</i> A list of one or more subnet IDs in your Amazon VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vpcId</i> The ID of the Amazon VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>s3Logs</i>  Information about logs built to an S3 bucket for a build project. S3 logs are not enabled by default. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>s3Logs</b>:  Information about logs built to an S3 bucket for a build project. S3 logs are not enabled by default. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cloudWatchLogs</i>  Information about Amazon CloudWatch Logs for a build project. Amazon CloudWatch Logs are enabled by default. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>cloudWatchLogs</b>:  Information about Amazon CloudWatch Logs for a build project. Amazon CloudWatch Logs are enabled by default. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>streamName</i>  The prefix of the stream name of the Amazon CloudWatch Logs. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudWatch/latest/logs/Working-with-log-groups-and-streams.html">Working with Log Groups and Log Streams</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>groupName</i>  The group name of the logs in Amazon CloudWatch Logs. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudWatch/latest/logs/Working-with-log-groups-and-streams.html">Working with Log Groups and Log Streams</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the logs in Amazon CloudWatch Logs for a build project. Valid values are: <ul> <li>  <code>ENABLED</code>: Amazon CloudWatch Logs are enabled for this build project. </li> <li>  <code>DISABLED</code>: Amazon CloudWatch Logs are not enabled for this build project. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>encryptionDisabled</i>  Set to true if you do not want your S3 build log output encrypted. By default S3 build logs are encrypted. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i>  The ARN of an S3 bucket and the path prefix for S3 logs. If your Amazon S3 bucket name is <code>my-bucket</code>, and your path prefix is <code>build-log</code>, then acceptable formats are <code>my-bucket/build-log</code> or <code>arn:aws:s3:::my-bucket/build-log</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the S3 build logs. Valid values are: <ul> <li>  <code>ENABLED</code>: S3 build logs are enabled for this build project. </li> <li>  <code>DISABLED</code>: S3 build logs are not enabled for this build project. </li> </ul><br/>
        :return: project:  Information about the build project that was changed.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>logsConfig</i>  Information about logs for the build project. A project can create logs in Amazon CloudWatch Logs, an S3 bucket, or both. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>logsConfig</b>:  Information about logs for the build project. A project can create logs in Amazon CloudWatch Logs, an S3 bucket, or both. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>badge</i> Information about the build badge for the build project.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>badge</b>: Information about the build badge for the build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vpcConfig</i> Information about the VPC configuration that AWS CodeBuild accesses.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>vpcConfig</b>: Information about the VPC configuration that AWS CodeBuild accesses.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>webhook</i> Information about a webhook that connects repository events to a build project in AWS CodeBuild.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>webhook</b>: Information about a webhook that connects repository events to a build project in AWS CodeBuild.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastModified</i> When the build project's settings were last modified, expressed in Unix time format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>created</i> When the build project was created, expressed in Unix time format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> The tags for this build project. These tags are available for use by AWS services that support AWS CodeBuild build project tags.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>encryptionKey</i> The AWS Key Management Service (AWS KMS) customer master key (CMK) to be used for encrypting the build output artifacts. <note>  You can use a cross-account KMS key to encrypt the build output artifacts if your service role has permission to that key.  </note> You can specify either the Amazon Resource Name (ARN) of the CMK or, if available, the CMK's alias (using the format <code>alias/<i>alias-name</i> </code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>queuedTimeoutInMinutes</i>  The number of minutes a build is allowed to be queued before it times out. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>timeoutInMinutes</i> How long, in minutes, from 5 to 480 (8 hours), for AWS CodeBuild to wait before timing out any related build that did not get marked as completed. The default is 60 minutes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceRole</i> The ARN of the AWS Identity and Access Management (IAM) role that enables AWS CodeBuild to interact with dependent AWS services on behalf of the AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>environment</i> Information about the build environment for this build project.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>environment</b>: Information about the build environment for this build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cache</i> Information about the cache for the build project.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>cache</b>: Information about the cache for the build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>secondaryArtifacts</i>  An array of <code>ProjectArtifacts</code> objects. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>artifacts</i> Information about the build output artifacts for the build project.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>artifacts</b>: Information about the build output artifacts for the build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>secondarySourceVersions</i>  An array of <code>ProjectSourceVersion</code> objects. If <code>secondarySourceVersions</code> is specified at the build level, then they take over these <code>secondarySourceVersions</code> (at the project level). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>sourceVersion</i> A version of the build input to be built for this project. If not specified, the latest version is used. If specified, it must be one of: <ul> <li> For AWS CodeCommit: the commit ID to use. </li> <li> For GitHub: the commit ID, pull request ID, branch name, or tag name that corresponds to the version of the source code you want to build. If a pull request ID is specified, it must use the format <code>pr/pull-request-ID</code> (for example <code>pr/25</code>). If a branch name is specified, the branch's HEAD commit ID is used. If not specified, the default branch's HEAD commit ID is used. </li> <li> For Bitbucket: the commit ID, branch name, or tag name that corresponds to the version of the source code you want to build. If a branch name is specified, the branch's HEAD commit ID is used. If not specified, the default branch's HEAD commit ID is used. </li> <li> For Amazon Simple Storage Service (Amazon S3): the version ID of the object that represents the build input ZIP file to use. </li> </ul>  If <code>sourceVersion</code> is specified at the build level, then that version takes precedence over this <code>sourceVersion</code> (at the project level).   For more information, see <a href="https://docs.aws.amazon.com/codebuild/latest/userguide/sample-source-version.html">Source Version Sample with CodeBuild</a> in the <i>AWS CodeBuild User Guide</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>secondarySources</i>  An array of <code>ProjectSource</code> objects. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>source</i> Information about the build input source code for this build project.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>source</b>: Information about the build input source code for this build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> A description that makes the build project easy to identify.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>gitCloneDepth</i> Information about the Git clone depth for the build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> Information about the location of the source code to be built. Valid values include: <ul> <li> For source code settings that are specified in the source action of a pipeline in AWS CodePipeline, <code>location</code> should not be specified. If it is specified, AWS CodePipeline ignores it. This is because AWS CodePipeline uses the settings in a pipeline's source action instead of this value. </li> <li> For source code in an AWS CodeCommit repository, the HTTPS clone URL to the repository that contains the source code and the build spec (for example, <code>https://git-codecommit.<i>region-ID</i>.amazonaws.com/v1/repos/<i>repo-name</i> </code>). </li> <li> For source code in an Amazon Simple Storage Service (Amazon S3) input bucket, one of the following.  <ul> <li>  The path to the ZIP file that contains the source code (for example, <code> <i>bucket-name</i>/<i>path</i>/<i>to</i>/<i>object-name</i>.zip</code>).  </li> <li>  The path to the folder that contains the source code (for example, <code> <i>bucket-name</i>/<i>path</i>/<i>to</i>/<i>source-code</i>/<i>folder</i>/</code>).  </li> </ul> </li> <li> For source code in a GitHub repository, the HTTPS clone URL to the repository that contains the source and the build spec. You must connect your AWS account to your GitHub account. Use the AWS CodeBuild console to start creating a build project. When you use the console to connect (or reconnect) with GitHub, on the GitHub <b>Authorize application</b> page, for <b>Organization access</b>, choose <b>Request access</b> next to each repository you want to allow AWS CodeBuild to have access to, and then choose <b>Authorize application</b>. (After you have connected to your GitHub account, you do not need to finish creating the build project. You can leave the AWS CodeBuild console.) To instruct AWS CodeBuild to use this connection, in the <code>source</code> object, set the <code>auth</code> object's <code>type</code> value to <code>OAUTH</code>. </li> <li> For source code in a Bitbucket repository, the HTTPS clone URL to the repository that contains the source and the build spec. You must connect your AWS account to your Bitbucket account. Use the AWS CodeBuild console to start creating a build project. When you use the console to connect (or reconnect) with Bitbucket, on the Bitbucket <b>Confirm access to your account</b> page, choose <b>Grant access</b>. (After you have connected to your Bitbucket account, you do not need to finish creating the build project. You can leave the AWS CodeBuild console.) To instruct AWS CodeBuild to use this connection, in the <code>source</code> object, set the <code>auth</code> object's <code>type</code> value to <code>OAUTH</code>. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of repository that contains the source code to be built. Valid values include: <ul> <li>  <code>BITBUCKET</code>: The source code is in a Bitbucket repository. </li> <li>  <code>CODECOMMIT</code>: The source code is in an AWS CodeCommit repository. </li> <li>  <code>CODEPIPELINE</code>: The source code settings are specified in the source action of a pipeline in AWS CodePipeline. </li> <li>  <code>GITHUB</code>: The source code is in a GitHub repository. </li> <li>  <code>NO_SOURCE</code>: The project does not have input source code. </li> <li>  <code>S3</code>: The source code is in an Amazon Simple Storage Service (Amazon S3) input bucket. </li> </ul><br/> Information about the Git submodules configuration for the build project. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>gitSubmodulesConfig</b>:  Information about the Git submodules configuration for the build project. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>artifactIdentifier</i>  An identifier for this artifact definition. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>encryptionDisabled</i>  Set to true if you do not want your output artifacts encrypted. This option is valid only if your artifacts type is Amazon Simple Storage Service (Amazon S3). If this is set with another artifacts type, an invalidInputException is thrown. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>overrideArtifactName</i>  If this flag is set, a name specified in the build spec file overrides the artifact name. The name specified in a build spec file is calculated at build time and uses the Shell Command Language. For example, you can append a date and time to your artifact name so that it is always unique. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>packaging</i> The type of build output artifact to create: <ul> <li> If <code>type</code> is set to <code>CODEPIPELINE</code>, AWS CodePipeline ignores this value if specified. This is because AWS CodePipeline manages its build output artifacts instead of AWS CodeBuild. </li> <li> If <code>type</code> is set to <code>NO_ARTIFACTS</code>, this value is ignored if specified, because no build output is produced. </li> <li> If <code>type</code> is set to <code>S3</code>, valid values include: <ul> <li>  <code>NONE</code>: AWS CodeBuild creates in the output bucket a folder that contains the build output. This is the default if <code>packaging</code> is not specified. </li> <li>  <code>ZIP</code>: AWS CodeBuild creates in the output bucket a ZIP file that contains the build output. </li> </ul> </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> Along with <code>path</code> and <code>namespaceType</code>, the pattern that AWS CodeBuild uses to name and store the output artifact: <ul> <li> If <code>type</code> is set to <code>CODEPIPELINE</code>, AWS CodePipeline ignores this value if specified. This is because AWS CodePipeline manages its build output names instead of AWS CodeBuild. </li> <li> If <code>type</code> is set to <code>NO_ARTIFACTS</code>, this value is ignored if specified, because no build output is produced. </li> <li> If <code>type</code> is set to <code>S3</code>, this is the name of the output artifact object. If you set the name to be a forward slash ("/"), the artifact is stored in the root of the output bucket. </li> </ul> For example: <ul> <li>  If <code>path</code> is set to <code>MyArtifacts</code>, <code>namespaceType</code> is set to <code>BUILD_ID</code>, and <code>name</code> is set to <code>MyArtifact.zip</code>, then the output artifact is stored in <code>MyArtifacts/<i>build-ID</i>/MyArtifact.zip</code>.  </li> <li>  If <code>path</code> is empty, <code>namespaceType</code> is set to <code>NONE</code>, and <code>name</code> is set to "<code>/</code>", the output artifact is stored in the root of the output bucket.  </li> <li>  If <code>path</code> is set to <code>MyArtifacts</code>, <code>namespaceType</code> is set to <code>BUILD_ID</code>, and <code>name</code> is set to "<code>/</code>", the output artifact is stored in <code>MyArtifacts/<i>build-ID</i> </code>.  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>namespaceType</i> Along with <code>path</code> and <code>name</code>, the pattern that AWS CodeBuild uses to determine the name and location to store the output artifact: <ul> <li> If <code>type</code> is set to <code>CODEPIPELINE</code>, AWS CodePipeline ignores this value if specified. This is because AWS CodePipeline manages its build output names instead of AWS CodeBuild. </li> <li> If <code>type</code> is set to <code>NO_ARTIFACTS</code>, this value is ignored if specified, because no build output is produced. </li> <li> If <code>type</code> is set to <code>S3</code>, valid values include: <ul> <li>  <code>BUILD_ID</code>: Include the build ID in the location of the build output artifact. </li> <li>  <code>NONE</code>: Do not include the build ID. This is the default if <code>namespaceType</code> is not specified. </li> </ul> </li> </ul> For example, if <code>path</code> is set to <code>MyArtifacts</code>, <code>namespaceType</code> is set to <code>BUILD_ID</code>, and <code>name</code> is set to <code>MyArtifact.zip</code>, the output artifact is stored in <code>MyArtifacts/<i>build-ID</i>/MyArtifact.zip</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>path</i> Along with <code>namespaceType</code> and <code>name</code>, the pattern that AWS CodeBuild uses to name and store the output artifact: <ul> <li> If <code>type</code> is set to <code>CODEPIPELINE</code>, AWS CodePipeline ignores this value if specified. This is because AWS CodePipeline manages its build output names instead of AWS CodeBuild. </li> <li> If <code>type</code> is set to <code>NO_ARTIFACTS</code>, this value is ignored if specified, because no build output is produced. </li> <li> If <code>type</code> is set to <code>S3</code>, this is the path to the output artifact. If <code>path</code> is not specified, <code>path</code> is not used. </li> </ul> For example, if <code>path</code> is set to <code>MyArtifacts</code>, <code>namespaceType</code> is set to <code>NONE</code>, and <code>name</code> is set to <code>MyArtifact.zip</code>, the output artifact is stored in the output bucket at <code>MyArtifacts/MyArtifact.zip</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> Information about the build output artifact location: <ul> <li> If <code>type</code> is set to <code>CODEPIPELINE</code>, AWS CodePipeline ignores this value if specified. This is because AWS CodePipeline manages its build output locations instead of AWS CodeBuild. </li> <li> If <code>type</code> is set to <code>NO_ARTIFACTS</code>, this value is ignored if specified, because no build output is produced. </li> <li> If <code>type</code> is set to <code>S3</code>, this is the name of the output bucket. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of build output artifact. Valid values include: <ul> <li>  <code>CODEPIPELINE</code>: The build project has build output generated through AWS CodePipeline. </li> <li>  <code>NO_ARTIFACTS</code>: The build project does not produce any build output. </li> <li>  <code>S3</code>: The build project stores build output in Amazon Simple Storage Service (Amazon S3). </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>modes</i>  If you use a <code>LOCAL</code> cache, the local cache mode. You can use one or more local cache modes at the same time.  <ul> <li>  <code>LOCAL_SOURCE_CACHE</code> mode caches Git metadata for primary and secondary sources. After the cache is created, subsequent builds pull only the change between commits. This mode is a good choice for projects with a clean working directory and a source that is a large Git repository. If you choose this option and your project does not use a Git repository (GitHub, GitHub Enterprise, or Bitbucket), the option is ignored.  </li> <li>  <code>LOCAL_DOCKER_LAYER_CACHE</code> mode caches existing Docker layers. This mode is a good choice for projects that build or pull large Docker images. It can prevent the performance issues caused by pulling large Docker images down from the network.  <note> <ul> <li>  You can use a Docker layer cache in the Linux environment only.  </li> <li>  The <code>privileged</code> flag must be set so that your project has the required Docker permissions.  </li> <li>  You should consider the security implications before you use a Docker layer cache.  </li> </ul> </note> </li> </ul> <ul> <li>  <code>LOCAL_CUSTOM_CACHE</code> mode caches directories you specify in the buildspec file. This mode is a good choice if your build scenario is not suited to one of the other three local cache modes. If you use a custom cache:  <ul> <li>  Only directories can be specified for caching. You cannot specify individual files.  </li> <li>  Symlinks are used to reference cached directories.  </li> <li>  Cached directories are linked to your build before it downloads its project sources. Cached items are overriden if a source item has the same name. Directories are specified using cache paths in the buildspec file.  </li> </ul> </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> Information about the cache location:  <ul> <li>  <code>NO_CACHE</code> or <code>LOCAL</code>: This value is ignored. </li> <li>  <code>S3</code>: This is the S3 bucket name/prefix. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of cache used by the build project. Valid values include: <ul> <li>  <code>NO_CACHE</code>: The build project does not use any cache. </li> <li>  <code>S3</code>: The build project reads and writes from and to S3. </li> <li>  <code>LOCAL</code>: The build project stores a cache locally on a build host that is only available to that build host. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>certificate</i> The certificate to use with this build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>privilegedMode</i> Enables running the Docker daemon inside a Docker container. Set to true only if the build project is be used to build Docker images, and the specified build environment image is not provided by AWS CodeBuild with Docker support. Otherwise, all associated builds that attempt to interact with the Docker daemon fail. You must also start the Docker daemon so that builds can interact with it. One way to do this is to initialize the Docker daemon during the install phase of your build spec by running the following build commands. (Do not run these commands if the specified build environment image is provided by AWS CodeBuild with Docker support.) If the operating system's base image is Ubuntu Linux:  <code>- nohup /usr/local/bin/dockerd --host=unix:///var/run/docker.sock --host=tcp://0.0.0.0:2375 --storage-driver=overlay&amp;</code>   <code>- timeout 15 sh -c "until docker info; do echo .; sleep 1; done"</code>  If the operating system's base image is Alpine Linux and the previous command does not work, add the <code>-t</code> argument to <code>timeout</code>:  <code>- nohup /usr/local/bin/dockerd --host=unix:///var/run/docker.sock --host=tcp://0.0.0.0:2375 --storage-driver=overlay&amp;</code>   <code>- timeout -t 15 sh -c "until docker info; do echo .; sleep 1; done"</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>environmentVariables</i> A set of environment variables to make available to builds for this build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>computeType</i> Information about the compute resources the build project uses. Available values include: <ul> <li>  <code>BUILD_GENERAL1_SMALL</code>: Use up to 3 GB memory and 2 vCPUs for builds. </li> <li>  <code>BUILD_GENERAL1_MEDIUM</code>: Use up to 7 GB memory and 4 vCPUs for builds. </li> <li>  <code>BUILD_GENERAL1_LARGE</code>: Use up to 15 GB memory and 8 vCPUs for builds. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>image</i> The image tag or image digest that identifies the Docker image to use for this build project. Use the following formats: <ul> <li> For an image tag: <code>registry/repository:tag</code>. For example, to specify an image with the tag "latest," use <code>registry/repository:latest</code>. </li> <li> For an image digest: <code>registry/repository@digest</code>. For example, to specify an image with the digest "sha256:cbbf2f9a99b47fc460d422812b6a5adff7dfee951d8fa2e4a98caa0382cfbdbf," use <code>registry/repository@sha256:cbbf2f9a99b47fc460d422812b6a5adff7dfee951d8fa2e4a98caa0382cfbdbf</code>. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of build environment to use for related builds.<br/> The credentials for access to a private registry.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>registryCredential</b>:  The credentials for access to a private registry.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastModifiedSecret</i>  A timestamp that indicates the last time a repository's secret token was modified. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>filterGroups</i>  An array of arrays of <code>WebhookFilter</code> objects used to determine which webhooks are triggered. At least one <code>WebhookFilter</code> in the array must specify <code>EVENT</code> as its <code>type</code>.   For a build to be triggered, at least one filter group in the <code>filterGroups</code> array must pass. For a filter group to pass, each of its filters must pass. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>branchFilter</i> A regular expression used to determine which repository branches are built when a webhook is triggered. If the name of a branch matches the regular expression, then it is built. If <code>branchFilter</code> is empty, then all branches are built. <note>  It is recommended that you use <code>filterGroups</code> instead of <code>branchFilter</code>.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>secret</i>  The secret token of the associated repository.  <note>  A Bitbucket webhook does not support <code>secret</code>.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>payloadUrl</i>  The AWS CodeBuild endpoint where webhook events are sent.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>url</i> The URL to the webhook.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>securityGroupIds</i> A list of one or more security groups IDs in your Amazon VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>subnets</i> A list of one or more subnet IDs in your Amazon VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>vpcId</i> The ID of the Amazon VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>badgeRequestUrl</i> The publicly-accessible URL through which you can access the build badge for your project.  The publicly accessible URL through which you can access the build badge for your project. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>badgeEnabled</i> Set this to true to generate a publicly accessible URL for your project's build badge.<br/> Information about Amazon CloudWatch Logs for a build project. Amazon CloudWatch Logs are enabled by default. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>cloudWatchLogs</b>:  Information about Amazon CloudWatch Logs for a build project. Amazon CloudWatch Logs are enabled by default.         """
        pass

    def update_webhook(self, projectName: str, branchFilter: str=None, rotateSecret: Boolean=None, filterGroups: List[FilterGroup]=None) -> UpdateWebhookOutput:
        """ Updates the webhook associated with an AWS CodeBuild build project.  <note>  If you use Bitbucket for your repository, <code>rotateSecret</code> is ignored.  </note>

        :param self:
        :param filterGroups:   An array of arrays of <code>WebhookFilter</code> objects used to determine if a webhook event can trigger a build. A filter group must pcontain at least one <code>EVENT</code> <code>WebhookFilter</code>. <br/>rotateSecret:   A boolean value that specifies whether the associated GitHub repository's secret token should be updated. If you use Bitbucket for your repository, <code>rotateSecret</code> is ignored. <br/>branchFilter:  A regular expression used to determine which repository branches are built when a webhook is triggered. If the name of a branch matches the regular expression, then it is built. If <code>branchFilter</code> is empty, then all branches are built. <note>  It is recommended that you use <code>filterGroups</code> instead of <code>branchFilter</code>.  </note><br/>projectName:  The name of the AWS CodeBuild project.<br/>
        :return: webhook:   Information about a repository's webhook that is associated with a project in AWS CodeBuild. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastModifiedSecret</i>  A timestamp that indicates the last time a repository's secret token was modified. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>filterGroups</i>  An array of arrays of <code>WebhookFilter</code> objects used to determine which webhooks are triggered. At least one <code>WebhookFilter</code> in the array must specify <code>EVENT</code> as its <code>type</code>.   For a build to be triggered, at least one filter group in the <code>filterGroups</code> array must pass. For a filter group to pass, each of its filters must pass. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>branchFilter</i> A regular expression used to determine which repository branches are built when a webhook is triggered. If the name of a branch matches the regular expression, then it is built. If <code>branchFilter</code> is empty, then all branches are built. <note>  It is recommended that you use <code>filterGroups</code> instead of <code>branchFilter</code>.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>secret</i>  The secret token of the associated repository.  <note>  A Bitbucket webhook does not support <code>secret</code>.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>payloadUrl</i>  The AWS CodeBuild endpoint where webhook events are sent.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>url</i> The URL to the webhook.<br/>        """
        pass



class CodeCommit(BaseClient):
    """<fullname>AWS CodeCommit</fullname> This is the <i>AWS CodeCommit API Reference</i>. This reference provides descriptions of the operations and data types for AWS CodeCommit API along with usage examples. You can use the AWS CodeCommit API to work with the following objects: Repositories, by calling the following: <ul> <li>  <a>BatchGetRepositories</a>, which returns information about one or more repositories associated with your AWS account. </li> <li>  <a>CreateRepository</a>, which creates an AWS CodeCommit repository. </li> <li>  <a>DeleteRepository</a>, which deletes an AWS CodeCommit repository. </li> <li>  <a>GetRepository</a>, which returns information about a specified repository. </li> <li>  <a>ListRepositories</a>, which lists all AWS CodeCommit repositories associated with your AWS account. </li> <li>  <a>UpdateRepositoryDescription</a>, which sets or updates the description of the repository. </li> <li>  <a>UpdateRepositoryName</a>, which changes the name of the repository. If you change the name of a repository, no other users of that repository will be able to access it until you send them the new HTTPS or SSH URL to use. </li> </ul> Branches, by calling the following: <ul> <li>  <a>CreateBranch</a>, which creates a new branch in a specified repository. </li> <li>  <a>DeleteBranch</a>, which deletes the specified branch in a repository unless it is the default branch. </li> <li>  <a>GetBranch</a>, which returns information about a specified branch. </li> <li>  <a>ListBranches</a>, which lists all branches for a specified repository. </li> <li>  <a>UpdateDefaultBranch</a>, which changes the default branch for a repository. </li> </ul> Files, by calling the following: <ul> <li>  <a>DeleteFile</a>, which deletes the content of a specified file from a specified branch. </li> <li>  <a>GetBlob</a>, which returns the base-64 encoded content of an individual Git blob object within a repository. </li> <li>  <a>GetFile</a>, which returns the base-64 encoded content of a specified file. </li> <li>  <a>GetFolder</a>, which returns the contents of a specified folder or directory. </li> <li>  <a>PutFile</a>, which adds or modifies a single file in a specified repository and branch. </li> </ul> Commits, by calling the following: <ul> <li>  <a>CreateCommit</a>, which creates a commit for changes to a repository. </li> <li>  <a>GetCommit</a>, which returns information about a commit, including commit messages and author and committer information. </li> <li>  <a>GetDifferences</a>, which returns information about the differences in a valid commit specifier (such as a branch, tag, HEAD, commit ID or other fully qualified reference). </li> </ul> Merges, by calling the following: <ul> <li>  <a>BatchDescribeMergeConflicts</a>, which returns information about conflicts in a merge between commits in a repository. </li> <li>  <a>CreateUnreferencedMergeCommit</a>, which creates an unreferenced commit between two branches or commits for the purpose of comparing them and identifying any potential conflicts. </li> <li>  <a>DescribeMergeConflicts</a>, which returns information about merge conflicts between the base, source, and destination versions of a file in a potential merge. </li> <li>  <a>GetMergeCommit</a>, which returns information about the merge between a source and destination commit.  </li> <li>  <a>GetMergeConflicts</a>, which returns information about merge conflicts between the source and destination branch in a pull request. </li> <li>  <a>GetMergeOptions</a>, which returns information about the available merge options between two branches or commit specifiers. </li> <li>  <a>MergeBranchesByFastForward</a>, which merges two branches using the fast-forward merge option. </li> <li>  <a>MergeBranchesBySquash</a>, which merges two branches using the squash merge option. </li> <li>  <a>MergeBranchesByThreeWay</a>, which merges two branches using the three-way merge option. </li> </ul> Pull requests, by calling the following: <ul> <li>  <a>CreatePullRequest</a>, which creates a pull request in a specified repository. </li> <li>  <a>DescribePullRequestEvents</a>, which returns information about one or more pull request events. </li> <li>  <a>GetCommentsForPullRequest</a>, which returns information about comments on a specified pull request. </li> <li>  <a>GetPullRequest</a>, which returns information about a specified pull request. </li> <li>  <a>ListPullRequests</a>, which lists all pull requests for a repository. </li> <li>  <a>MergePullRequestByFastForward</a>, which merges the source destination branch of a pull request into the specified destination branch for that pull request using the fast-forward merge option. </li> <li>  <a>MergePullRequestBySquash</a>, which merges the source destination branch of a pull request into the specified destination branch for that pull request using the squash merge option. </li> <li>  <a>MergePullRequestByThreeWay</a>. which merges the source destination branch of a pull request into the specified destination branch for that pull request using the three-way merge option. </li> <li>  <a>PostCommentForPullRequest</a>, which posts a comment to a pull request at the specified line, file, or request. </li> <li>  <a>UpdatePullRequestDescription</a>, which updates the description of a pull request. </li> <li>  <a>UpdatePullRequestStatus</a>, which updates the status of a pull request. </li> <li>  <a>UpdatePullRequestTitle</a>, which updates the title of a pull request. </li> </ul> Comments in a repository, by calling the following: <ul> <li>  <a>DeleteCommentContent</a>, which deletes the content of a comment on a commit in a repository. </li> <li>  <a>GetComment</a>, which returns information about a comment on a commit. </li> <li>  <a>GetCommentsForComparedCommit</a>, which returns information about comments on the comparison between two commit specifiers in a repository. </li> <li>  <a>PostCommentForComparedCommit</a>, which creates a comment on the comparison between two commit specifiers in a repository. </li> <li>  <a>PostCommentReply</a>, which creates a reply to a comment. </li> <li>  <a>UpdateComment</a>, which updates the content of a comment on a commit in a repository. </li> </ul> Tags used to tag resources in AWS CodeCommit (not Git tags), by calling the following: <ul> <li>  <a>ListTagsForResource</a>, which gets information about AWS tags for a specified Amazon Resource Name (ARN) in AWS CodeCommit. </li> <li>  <a>TagResource</a>, which adds or updates tags for a resource in AWS CodeCommit. </li> <li>  <a>UntagResource</a>, which removes tags for a resource in AWS CodeCommit. </li> </ul> Triggers, by calling the following: <ul> <li>  <a>GetRepositoryTriggers</a>, which returns information about triggers configured for a repository. </li> <li>  <a>PutRepositoryTriggers</a>, which replaces all triggers for a repository and can be used to create or delete triggers. </li> <li>  <a>TestRepositoryTriggers</a>, which tests the functionality of a repository trigger by sending data to the trigger target. </li> </ul> For information about how to use AWS CodeCommit, see the <a href="https://docs.aws.amazon.com/codecommit/latest/userguide/welcome.html">AWS CodeCommit User Guide</a>."""


    class ListBranchesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_branches` for valid parameters"""
            pass
    class ListRepositoriesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_repositories` for valid parameters"""
            pass
    class GetCommentsForComparedCommitPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_comments_for_compared_commit` for valid parameters"""
            pass
    class DescribePullRequestEventsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_pull_request_events` for valid parameters"""
            pass
    class GetCommentsForPullRequestPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_comments_for_pull_request` for valid parameters"""
            pass
    class ListPullRequestsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_pull_requests` for valid parameters"""
            pass
    class GetDifferencesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_differences` for valid parameters"""
            pass


    class BatchDescribeMergeConflictsInput(Mapping):
        pass
    
    class BatchDescribeMergeConflictsOutput(Mapping):
        pass
    
    class Path(object):
        pass
    
    class BatchGetRepositoriesInput(Mapping):
        pass
    
    class BatchGetRepositoriesOutput(Mapping):
        pass
    
    class RepositoryName(object):
        pass
    
    class CreateBranchInput(Mapping):
        pass
    
    class CreateCommitInput(Mapping):
        pass
    
    class CreateCommitOutput(Mapping):
        pass
    
    class KeepEmptyFolders(object):
        pass
    
    class PutFileEntry(Mapping):
        pass
    
    class DeleteFileEntry(Mapping):
        pass
    
    class SetFileModeEntry(Mapping):
        pass
    
    class CreatePullRequestInput(Mapping):
        pass
    
    class CreatePullRequestOutput(Mapping):
        pass
    
    class Target(Mapping):
        pass
    
    class CreateRepositoryInput(Mapping):
        pass
    
    class CreateRepositoryOutput(Mapping):
        pass
    
    class TagsMap(object):
        pass
    
    class CreateUnreferencedMergeCommitInput(Mapping):
        pass
    
    class CreateUnreferencedMergeCommitOutput(Mapping):
        pass
    
    class ConflictResolution(Mapping):
        pass
    
    class DeleteBranchInput(Mapping):
        pass
    
    class DeleteBranchOutput(Mapping):
        pass
    
    class DeleteCommentContentInput(Mapping):
        pass
    
    class DeleteCommentContentOutput(Mapping):
        pass
    
    class DeleteFileInput(Mapping):
        pass
    
    class DeleteFileOutput(Mapping):
        pass
    
    class DeleteRepositoryInput(Mapping):
        pass
    
    class DeleteRepositoryOutput(Mapping):
        pass
    
    class DescribeMergeConflictsInput(Mapping):
        pass
    
    class DescribeMergeConflictsOutput(Mapping):
        pass
    
    class DescribePullRequestEventsInput(Mapping):
        pass
    
    class DescribePullRequestEventsOutput(Mapping):
        pass
    
    class GetBlobInput(Mapping):
        pass
    
    class GetBlobOutput(Mapping):
        pass
    
    class GetBranchInput(Mapping):
        pass
    
    class GetBranchOutput(Mapping):
        pass
    
    class GetCommentInput(Mapping):
        pass
    
    class GetCommentOutput(Mapping):
        pass
    
    class GetCommentsForComparedCommitInput(Mapping):
        pass
    
    class GetCommentsForComparedCommitOutput(Mapping):
        pass
    
    class GetCommentsForPullRequestInput(Mapping):
        pass
    
    class GetCommentsForPullRequestOutput(Mapping):
        pass
    
    class GetCommitInput(Mapping):
        pass
    
    class GetCommitOutput(Mapping):
        pass
    
    class GetDifferencesInput(Mapping):
        pass
    
    class GetDifferencesOutput(Mapping):
        pass
    
    class GetFileInput(Mapping):
        pass
    
    class GetFileOutput(Mapping):
        pass
    
    class GetFolderInput(Mapping):
        pass
    
    class GetFolderOutput(Mapping):
        pass
    
    class GetMergeCommitInput(Mapping):
        pass
    
    class GetMergeCommitOutput(Mapping):
        pass
    
    class GetMergeConflictsInput(Mapping):
        pass
    
    class GetMergeConflictsOutput(Mapping):
        pass
    
    class GetMergeOptionsInput(Mapping):
        pass
    
    class GetMergeOptionsOutput(Mapping):
        pass
    
    class GetPullRequestInput(Mapping):
        pass
    
    class GetPullRequestOutput(Mapping):
        pass
    
    class GetRepositoryInput(Mapping):
        pass
    
    class GetRepositoryOutput(Mapping):
        pass
    
    class GetRepositoryTriggersInput(Mapping):
        pass
    
    class GetRepositoryTriggersOutput(Mapping):
        pass
    
    class ListBranchesInput(Mapping):
        pass
    
    class ListBranchesOutput(Mapping):
        pass
    
    class ListPullRequestsInput(Mapping):
        pass
    
    class ListPullRequestsOutput(Mapping):
        pass
    
    class ListRepositoriesInput(Mapping):
        pass
    
    class ListRepositoriesOutput(Mapping):
        pass
    
    class ListTagsForResourceInput(Mapping):
        pass
    
    class ListTagsForResourceOutput(Mapping):
        pass
    
    class MergeBranchesByFastForwardInput(Mapping):
        pass
    
    class MergeBranchesByFastForwardOutput(Mapping):
        pass
    
    class MergeBranchesBySquashInput(Mapping):
        pass
    
    class MergeBranchesBySquashOutput(Mapping):
        pass
    
    class MergeBranchesByThreeWayInput(Mapping):
        pass
    
    class MergeBranchesByThreeWayOutput(Mapping):
        pass
    
    class MergePullRequestByFastForwardInput(Mapping):
        pass
    
    class MergePullRequestByFastForwardOutput(Mapping):
        pass
    
    class MergePullRequestBySquashInput(Mapping):
        pass
    
    class MergePullRequestBySquashOutput(Mapping):
        pass
    
    class MergePullRequestByThreeWayInput(Mapping):
        pass
    
    class MergePullRequestByThreeWayOutput(Mapping):
        pass
    
    class PostCommentForComparedCommitInput(Mapping):
        pass
    
    class PostCommentForComparedCommitOutput(Mapping):
        pass
    
    class Location(Mapping):
        pass
    
    class PostCommentForPullRequestInput(Mapping):
        pass
    
    class PostCommentForPullRequestOutput(Mapping):
        pass
    
    class PostCommentReplyInput(Mapping):
        pass
    
    class PostCommentReplyOutput(Mapping):
        pass
    
    class PutFileInput(Mapping):
        pass
    
    class PutFileOutput(Mapping):
        pass
    
    class FileContent(object):
        pass
    
    class PutRepositoryTriggersInput(Mapping):
        pass
    
    class PutRepositoryTriggersOutput(Mapping):
        pass
    
    class RepositoryTrigger(Mapping):
        pass
    
    class TagResourceInput(Mapping):
        pass
    
    class TestRepositoryTriggersInput(Mapping):
        pass
    
    class TestRepositoryTriggersOutput(Mapping):
        pass
    
    class UntagResourceInput(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateCommentInput(Mapping):
        pass
    
    class UpdateCommentOutput(Mapping):
        pass
    
    class UpdateDefaultBranchInput(Mapping):
        pass
    
    class UpdatePullRequestDescriptionInput(Mapping):
        pass
    
    class UpdatePullRequestDescriptionOutput(Mapping):
        pass
    
    class UpdatePullRequestStatusInput(Mapping):
        pass
    
    class UpdatePullRequestStatusOutput(Mapping):
        pass
    
    class UpdatePullRequestTitleInput(Mapping):
        pass
    
    class UpdatePullRequestTitleOutput(Mapping):
        pass
    
    class UpdateRepositoryDescriptionInput(Mapping):
        pass
    
    class UpdateRepositoryNameInput(Mapping):
        pass
    
    def batch_describe_merge_conflicts(self, mergeOption: str, sourceCommitSpecifier: str, destinationCommitSpecifier: str, repositoryName: str, maxMergeHunks: int=None, maxConflictFiles: int=None, filePaths: List[Path]=None, conflictDetailLevel: str=None, conflictResolutionStrategy: str=None, nextToken: str=None) -> BatchDescribeMergeConflictsOutput:
        """Returns information about one or more merge conflicts in the attempted merge of two commit specifiers using the squash or three-way merge strategy.

        :param self:
        :param nextToken:  An enumeration token that when provided in a request, returns the next batch of the results.<br/>conflictResolutionStrategy:  Specifies which branch to use when resolving conflicts, or whether to attempt automatically merging two versions of a file. The default is NONE, which requires any conflicts to be resolved manually before the merge operation will be successful.<br/>conflictDetailLevel:  The level of conflict detail to use. If unspecified, the default FILE_LEVEL is used, which will return a not mergeable result if the same file has differences in both branches. If LINE_LEVEL is specified, a conflict will be considered not mergeable if the same file in both branches has differences on the same line.<br/>filePaths:  The path of the target files used to describe the conflicts. If not specified, the default is all conflict files.<br/>maxConflictFiles:  The maximum number of files to include in the output.<br/>maxMergeHunks:  The maximum number of merge hunks to include in the output.<br/>mergeOption:  The merge option or strategy you want to use to merge the code.<br/>sourceCommitSpecifier:  The branch, tag, HEAD, or other fully qualified reference used to identify a commit. For example, a branch name or a full commit ID.<br/>destinationCommitSpecifier:  The branch, tag, HEAD, or other fully qualified reference used to identify a commit. For example, a branch name or a full commit ID.<br/>repositoryName:  The name of the repository that contains the merge conflicts you want to review.<br/>
        :return: baseCommitId:  The commit ID of the merge base.<br/>sourceCommitId:  The commit ID of the source commit specifier that was used in the merge evaluation.<br/>destinationCommitId:  The commit ID of the destination commit specifier that was used in the merge evaluation.<br/>errors:  A list of any errors returned while describing the merge conflicts for each file.<br/>nextToken:  An enumeration token that can be used in a request to return the next batch of the results.<br/>conflicts:  A list of conflicts for each file, including the conflict metadata and the hunks of the differences between the files.<br/>        """
        pass

    def batch_get_repositories(self, repositoryNames: List[RepositoryName]) -> BatchGetRepositoriesOutput:
        """Returns information about one or more repositories. <note> The description field for a repository accepts all HTML characters and all valid Unicode characters. Applications that do not HTML-encode the description and display it in a web page could expose users to potentially malicious code. Make sure that you HTML-encode the description field in any application that uses this API to display the repository description on a web page. </note>

        :param self:
        :param repositoryNames:  The names of the repositories to get information about.<br/>
        :return: repositoriesNotFound:  Returns a list of repository names for which information could not be found.<br/>repositories:  A list of repositories returned by the batch get repositories operation.<br/>        """
        pass

    def create_branch(self, commitId: str, branchName: str, repositoryName: str):
        """Creates a new branch in a repository and points the branch to a commit. <note> Calling the create branch operation does not set a repository's default branch. To do this, call the update default branch operation. </note>

        :param self:
        :param commitId:  The ID of the commit to point the new branch to.<br/>branchName:  The name of the new branch to create.<br/>repositoryName:  The name of the repository in which you want to create the new branch.<br/>
        :return:         """
        pass

    def create_commit(self, branchName: str, repositoryName: str, parentCommitId: str=None, authorName: str=None, email: str=None, commitMessage: str=None, keepEmptyFolders: KeepEmptyFolders=None, putFiles: List[PutFileEntry]=None, deleteFiles: List[DeleteFileEntry]=None, setFileModes: List[SetFileModeEntry]=None) -> CreateCommitOutput:
        """Creates a commit for a repository on the tip of a specified branch.

        :param self:
        :param setFileModes:  The file modes to update for files in this commit.<br/>deleteFiles:  The files to delete in this commit. These files will still exist in prior commits.<br/>putFiles:  The files to add or update in this commit.<br/>keepEmptyFolders:  If the commit contains deletions, whether to keep a folder or folder structure if the changes leave the folders empty. If this is specified as true, a .gitkeep file will be created for empty folders. The default is false.<br/>commitMessage:  The commit message you want to include as part of creating the commit. Commit messages are limited to 256 KB. If no message is specified, a default message will be used.<br/>email:  The email address of the person who created the commit.<br/>authorName:  The name of the author who created the commit. This information will be used as both the author and committer for the commit.<br/>parentCommitId:  The ID of the commit that is the parent of the commit you will create. If this is an empty repository, this is not required.<br/>branchName:  The name of the branch where you will create the commit.<br/>repositoryName:  The name of the repository where you will create the commit.<br/>
        :return: filesDeleted:  The files deleted as part of the committed file changes.<br/>filesUpdated:  The files updated as part of the commited file changes.<br/>filesAdded:  The files added as part of the committed file changes.<br/>treeId:  The full SHA-1 pointer of the tree information for the commit that contains the commited file changes.<br/>commitId:  The full commit ID of the commit that contains your committed file changes.<br/>        """
        pass

    def create_pull_request(self, targets: List[Target], title: str, description: str=None, clientRequestToken: str=None) -> CreatePullRequestOutput:
        """Creates a pull request in the specified repository.

        :param self:
        :param clientRequestToken:  A unique, client-generated idempotency token that when provided in a request, ensures the request cannot be repeated with a changed parameter. If a request is received with the same parameters and a token is included, the request will return information about the initial request that used that token. <note> The AWS SDKs prepopulate client request tokens. If using an AWS SDK, you do not have to generate an idempotency token, as this will be done for you. </note><br/>targets:  The targets for the pull request, including the source of the code to be reviewed (the source branch), and the destination where the creator of the pull request intends the code to be merged after the pull request is closed (the destination branch).<br/>description:  A description of the pull request.<br/>title:  The title of the pull request. This title will be used to identify the pull request to other users in the repository.<br/>
        :return: pullRequest:  Information about the newly created pull request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clientRequestToken</i> A unique, client-generated idempotency token that when provided in a request, ensures the request cannot be repeated with a changed parameter. If a request is received with the same parameters and a token is included, the request will return information about the initial request that used that token.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pullRequestTargets</i> The targets of the pull request, including the source branch and destination branch for the pull request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authorArn</i> The Amazon Resource Name (ARN) of the user who created the pull request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pullRequestStatus</i> The status of the pull request. Pull request status can only change from <code>OPEN</code> to <code>CLOSED</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDate</i> The date and time the pull request was originally created, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastActivityDate</i> The day and time of the last user or system activity on the pull request, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The user-defined description of the pull request. This description can be used to clarify what should be reviewed and other details of the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>title</i> The user-defined title of the pull request. This title is displayed in the list of pull requests to other users of the repository.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pullRequestId</i> The system-generated ID of the pull request. <br/>        """
        pass

    def create_repository(self, repositoryName: str, repositoryDescription: str=None, tags: TagsMap=None) -> CreateRepositoryOutput:
        """Creates a new, empty repository.

        :param self:
        :param tags:  One or more tag key-value pairs to use when tagging this repository.<br/>repositoryDescription:  A comment or description about the new repository. <note> The description field for a repository accepts all HTML characters and all valid Unicode characters. Applications that do not HTML-encode the description and display it in a web page could expose users to potentially malicious code. Make sure that you HTML-encode the description field in any application that uses this API to display the repository description on a web page. </note><br/>repositoryName:  The name of the new repository to be created. <note> The repository name must be unique across the calling AWS account. In addition, repository names are limited to 100 alphanumeric, dash, and underscore characters, and cannot include certain characters. For a full description of the limits on repository names, see <a href="https://docs.aws.amazon.com/codecommit/latest/userguide/limits.html">Limits</a> in the AWS CodeCommit User Guide. The suffix ".git" is prohibited. </note><br/>
        :return: repositoryMetadata:  Information about the newly created repository.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the repository.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cloneUrlSsh</i> The URL to use for cloning the repository over SSH.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cloneUrlHttp</i> The URL to use for cloning the repository over HTTPS.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDate</i> The date and time the repository was created, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastModifiedDate</i> The date and time the repository was last modified, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultBranch</i> The repository's default branch name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>repositoryDescription</i> A comment or description about the repository.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>repositoryName</i> The repository's name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>repositoryId</i> The ID of the repository.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>accountId</i> The ID of the AWS account associated with the repository.<br/>        """
        pass

    def create_unreferenced_merge_commit(self, mergeOption: str, destinationCommitSpecifier: str, sourceCommitSpecifier: str, repositoryName: str, conflictDetailLevel: str=None, conflictResolutionStrategy: str=None, authorName: str=None, email: str=None, commitMessage: str=None, keepEmptyFolders: KeepEmptyFolders=None, conflictResolution: ConflictResolution=None) -> CreateUnreferencedMergeCommitOutput:
        """Creates an unreferenced commit that represents the result of merging two branches using a specified merge strategy. This can help you determine the outcome of a potential merge. This API cannot be used with the fast-forward merge strategy, as that strategy does not create a merge commit. <note> This unreferenced merge commit can only be accessed using the GetCommit API or through git commands such as git fetch. To retrieve this commit, you must specify its commit ID or otherwise reference it. </note>

        :param self:
        :param conflictResolution:  A list of inputs to use when resolving conflicts during a merge if AUTOMERGE is chosen as the conflict resolution strategy.:<br/>keepEmptyFolders:  If the commit contains deletions, whether to keep a folder or folder structure if the changes leave the folders empty. If this is specified as true, a .gitkeep file will be created for empty folders. The default is false.<br/>commitMessage:  The commit message for the unreferenced commit.<br/>email:  The email address for the person who created the unreferenced commit.<br/>authorName:  The name of the author who created the unreferenced commit. This information will be used as both the author and committer for the commit.<br/>conflictResolutionStrategy:  Specifies which branch to use when resolving conflicts, or whether to attempt automatically merging two versions of a file. The default is NONE, which requires any conflicts to be resolved manually before the merge operation will be successful.<br/>conflictDetailLevel:  The level of conflict detail to use. If unspecified, the default FILE_LEVEL is used, which will return a not mergeable result if the same file has differences in both branches. If LINE_LEVEL is specified, a conflict will be considered not mergeable if the same file in both branches has differences on the same line.<br/>mergeOption:  The merge option or strategy you want to use to merge the code.<br/>destinationCommitSpecifier:  The branch, tag, HEAD, or other fully qualified reference used to identify a commit. For example, a branch name or a full commit ID.<br/>sourceCommitSpecifier:  The branch, tag, HEAD, or other fully qualified reference used to identify a commit. For example, a branch name or a full commit ID.<br/>repositoryName:  The name of the repository where you want to create the unreferenced merge commit.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>setFileModes</i> File modes that will be set as part of the merge conflict resolution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deleteFiles</i> Files that will be deleted as part of the merge conflict resolution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>replaceContents</i> Files that will have content replaced as part of the merge conflict resolution.<br/>
        :return: treeId:  The full SHA-1 pointer of the tree information for the commit that contains the merge results.<br/>commitId:  The full commit ID of the commit that contains your merge results.<br/>        """
        pass

    def delete_branch(self, branchName: str, repositoryName: str) -> DeleteBranchOutput:
        """Deletes a branch from a repository, unless that branch is the default branch for the repository. 

        :param self:
        :param branchName:  The name of the branch to delete.<br/>repositoryName:  The name of the repository that contains the branch to be deleted.<br/>
        :return: deletedBranch:  Information about the branch deleted by the operation, including the branch name and the commit ID that was the tip of the branch.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>commitId</i> The ID of the last commit made to the branch.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>branchName</i> The name of the branch.<br/>        """
        pass

    def delete_comment_content(self, commentId: str) -> DeleteCommentContentOutput:
        """Deletes the content of a comment made on a change, file, or commit in a repository.

        :param self:
        :param commentId:  The unique, system-generated ID of the comment. To get this ID, use <a>GetCommentsForComparedCommit</a> or <a>GetCommentsForPullRequest</a>.<br/>
        :return: comment:  Information about the comment you just deleted.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clientRequestToken</i> A unique, client-generated idempotency token that when provided in a request, ensures the request cannot be repeated with a changed parameter. If a request is received with the same parameters and a token is included, the request will return information about the initial request that used that token.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deleted</i> A Boolean value indicating whether the comment has been deleted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authorArn</i> The Amazon Resource Name (ARN) of the person who posted the comment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastModifiedDate</i> The date and time the comment was most recently modified, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDate</i> The date and time the comment was created, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>inReplyTo</i> The ID of the comment for which this comment is a reply, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>content</i> The content of the comment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>commentId</i> The system-generated comment ID.<br/>        """
        pass

    def delete_file(self, parentCommitId: str, filePath: str, branchName: str, repositoryName: str, keepEmptyFolders: KeepEmptyFolders=None, commitMessage: str=None, name: str=None, email: str=None) -> DeleteFileOutput:
        """Deletes a specified file from a specified branch. A commit is created on the branch that contains the revision. The file will still exist in the commits prior to the commit that contains the deletion.

        :param self:
        :param email:  The email address for the commit that deletes the file. If no email address is specified, the email address will be left blank.<br/>name:  The name of the author of the commit that deletes the file. If no name is specified, the user's ARN will be used as the author name and committer name.<br/>commitMessage:  The commit message you want to include as part of deleting the file. Commit messages are limited to 256 KB. If no message is specified, a default message will be used.<br/>keepEmptyFolders:  Specifies whether to delete the folder or directory that contains the file you want to delete if that file is the only object in the folder or directory. By default, empty folders will be deleted. This includes empty folders that are part of the directory structure. For example, if the path to a file is dir1/dir2/dir3/dir4, and dir2 and dir3 are empty, deleting the last file in dir4 will also delete the empty folders dir4, dir3, and dir2.<br/>parentCommitId:  The ID of the commit that is the tip of the branch where you want to create the commit that will delete the file. This must be the HEAD commit for the branch. The commit that deletes the file will be created from this commit ID.<br/>filePath:  The fully-qualified path to the file that will be deleted, including the full name and extension of that file. For example, /examples/file.md is a fully qualified path to a file named file.md in a folder named examples.<br/>branchName:  The name of the branch where the commit will be made deleting the file.<br/>repositoryName:  The name of the repository that contains the file to delete.<br/>
        :return: filePath:  The fully-qualified path to the file that will be deleted, including the full name and extension of that file.<br/>treeId:  The full SHA-1 pointer of the tree information for the commit that contains the delete file change.<br/>blobId:  The blob ID removed from the tree as part of deleting the file.<br/>commitId:  The full commit ID of the commit that contains the change that deletes the file.<br/>        """
        pass

    def delete_repository(self, repositoryName: str) -> DeleteRepositoryOutput:
        """Deletes a repository. If a specified repository was already deleted, a null repository ID will be returned. <important> Deleting a repository also deletes all associated objects and metadata. After a repository is deleted, all future push calls to the deleted repository will fail. </important>

        :param self:
        :param repositoryName:  The name of the repository to delete.<br/>
        :return: repositoryId:  The ID of the repository that was deleted.<br/>        """
        pass

    def describe_merge_conflicts(self, filePath: str, mergeOption: str, sourceCommitSpecifier: str, destinationCommitSpecifier: str, repositoryName: str, maxMergeHunks: int=None, conflictDetailLevel: str=None, conflictResolutionStrategy: str=None, nextToken: str=None) -> DescribeMergeConflictsOutput:
        """Returns information about one or more merge conflicts in the attempted merge of two commit specifiers using the squash or three-way merge strategy. If the merge option for the attempted merge is specified as FAST_FORWARD_MERGE, an exception will be thrown.

        :param self:
        :param nextToken:  An enumeration token that when provided in a request, returns the next batch of the results.<br/>conflictResolutionStrategy:  Specifies which branch to use when resolving conflicts, or whether to attempt automatically merging two versions of a file. The default is NONE, which requires any conflicts to be resolved manually before the merge operation will be successful.<br/>conflictDetailLevel:  The level of conflict detail to use. If unspecified, the default FILE_LEVEL is used, which will return a not mergeable result if the same file has differences in both branches. If LINE_LEVEL is specified, a conflict will be considered not mergeable if the same file in both branches has differences on the same line.<br/>filePath:  The path of the target files used to describe the conflicts. <br/>maxMergeHunks:  The maximum number of merge hunks to include in the output.<br/>mergeOption:  The merge option or strategy you want to use to merge the code.<br/>sourceCommitSpecifier:  The branch, tag, HEAD, or other fully qualified reference used to identify a commit. For example, a branch name or a full commit ID.<br/>destinationCommitSpecifier:  The branch, tag, HEAD, or other fully qualified reference used to identify a commit. For example, a branch name or a full commit ID.<br/>repositoryName:  The name of the repository where you want to get information about a merge conflict.<br/>
        :return: baseCommitId:  The commit ID of the merge base.<br/>sourceCommitId:  The commit ID of the source commit specifier that was used in the merge evaluation.<br/>destinationCommitId:  The commit ID of the destination commit specifier that was used in the merge evaluation.<br/>nextToken:  An enumeration token that can be used in a request to return the next batch of the results.<br/>mergeHunks:  A list of merge hunks of the differences between the files or lines.<br/>conflictMetadata:  Contains metadata about the conflicts found in the merge.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>mergeOperations</i> Whether an add, modify, or delete operation caused the conflict between the source and destination of the merge.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>mergeOperations</b>: Whether an add, modify, or delete operation caused the conflict between the source and destination of the merge.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>objectTypeConflict</i> A boolean value (true or false) indicating whether there are conflicts between the branches in the object type of a file, folder, or submodule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fileModeConflict</i> A boolean value indicating whether there are conflicts in the file mode of a file.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>contentConflict</i> A boolean value indicating whether there are conflicts in the content of a file.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>isBinaryFile</i> A boolean value (true or false) indicating whether the file is binary or textual in the source, destination, and base of the merge.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>isBinaryFile</b>: A boolean value (true or false) indicating whether the file is binary or textual in the source, destination, and base of the merge.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>numberOfConflicts</i> The number of conflicts, including both hunk conflicts and metadata conflicts.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>objectTypes</i> Information about any object type conflicts in a merge operation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>objectTypes</b>: Information about any object type conflicts in a merge operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fileModes</i> The file modes of the file in the source, destination, and base of the merge.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>fileModes</b>: The file modes of the file in the source, destination, and base of the merge.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fileSizes</i> The file sizes of the file in the source, destination, and base of the merge.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>fileSizes</b>: The file sizes of the file in the source, destination, and base of the merge.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>filePath</i> The path of the file that contains conflicts.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>base</i> The size of a file in the base of a merge or pull request.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>destination</i> The size of a file in the destination of a merge or pull request.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>source</i> The size of a file in the source of a merge or pull request.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>base</i> The file mode of a file in the base of a merge or pull request.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>destination</i> The file mode of a file in the destination of a merge or pull request.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>source</i> The file mode of a file in the source of a merge or pull request.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>base</i> The type of the object in the base commit of the merge.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>destination</i> The type of the object in the destination branch.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>source</i> The type of the object in the source branch.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>base</i> The binary or non-binary status of a file in the base of a merge or pull request.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>destination</i> The binary or non-binary status of a file in the destination of a merge or pull request.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>source</i> The binary or non-binary status of file in the source of a merge or pull request.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>destination</i> The operation on a file in the destination of a merge or pull request.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>source</i> The operation on a file (add, modify, or delete) of a file in the source of a merge or pull request.<br/>        """
        pass

    def describe_pull_request_events(self, pullRequestId: str, pullRequestEventType: str=None, actorArn: str=None, nextToken: str=None, maxResults: int=None) -> DescribePullRequestEventsOutput:
        """Returns information about one or more pull request events.

        :param self:
        :param maxResults:  A non-negative integer used to limit the number of returned results. The default is 100 events, which is also the maximum number of events that can be returned in a result.<br/>nextToken:  An enumeration token that when provided in a request, returns the next batch of the results.<br/>actorArn:  The Amazon Resource Name (ARN) of the user whose actions resulted in the event. Examples include updating the pull request with additional commits or changing the status of a pull request.<br/>pullRequestEventType:  Optional. The pull request event type about which you want to return information.<br/>pullRequestId:  The system-generated ID of the pull request. To get this ID, use <a>ListPullRequests</a>.<br/>
        :return: nextToken:  An enumeration token that can be used in a request to return the next batch of the results.<br/>pullRequestEvents:  Information about the pull request events.<br/>        """
        pass

    def get_blob(self, blobId: str, repositoryName: str) -> GetBlobOutput:
        """Returns the base-64 encoded content of an individual blob within a repository.

        :param self:
        :param blobId:  The ID of the blob, which is its SHA-1 pointer.<br/>repositoryName:  The name of the repository that contains the blob.<br/>
        :return: content:  The content of the blob, usually a file.<br/>        """
        pass

    def get_branch(self, repositoryName: str=None, branchName: str=None) -> GetBranchOutput:
        """Returns information about a repository branch, including its name and the last commit ID.

        :param self:
        :param branchName:  The name of the branch for which you want to retrieve information.<br/>repositoryName:  The name of the repository that contains the branch for which you want to retrieve information.<br/>
        :return: branch:  The name of the branch.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>commitId</i> The ID of the last commit made to the branch.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>branchName</i> The name of the branch.<br/>        """
        pass

    def get_comment(self, commentId: str) -> GetCommentOutput:
        """Returns the content of a comment made on a change, file, or commit in a repository.

        :param self:
        :param commentId:  The unique, system-generated ID of the comment. To get this ID, use <a>GetCommentsForComparedCommit</a> or <a>GetCommentsForPullRequest</a>.<br/>
        :return: comment:  The contents of the comment.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clientRequestToken</i> A unique, client-generated idempotency token that when provided in a request, ensures the request cannot be repeated with a changed parameter. If a request is received with the same parameters and a token is included, the request will return information about the initial request that used that token.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deleted</i> A Boolean value indicating whether the comment has been deleted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authorArn</i> The Amazon Resource Name (ARN) of the person who posted the comment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastModifiedDate</i> The date and time the comment was most recently modified, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDate</i> The date and time the comment was created, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>inReplyTo</i> The ID of the comment for which this comment is a reply, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>content</i> The content of the comment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>commentId</i> The system-generated comment ID.<br/>        """
        pass

    def get_comments_for_compared_commit(self, afterCommitId: str, repositoryName: str, beforeCommitId: str=None, nextToken: str=None, maxResults: int=None) -> GetCommentsForComparedCommitOutput:
        """Returns information about comments made on the comparison between two commits.

        :param self:
        :param maxResults:  A non-negative integer used to limit the number of returned results. The default is 100 comments, and is configurable up to 500.<br/>nextToken:  An enumeration token that when provided in a request, returns the next batch of the results. <br/>afterCommitId:  To establish the directionality of the comparison, the full commit ID of the 'after' commit.<br/>beforeCommitId:  To establish the directionality of the comparison, the full commit ID of the 'before' commit.<br/>repositoryName:  The name of the repository where you want to compare commits.<br/>
        :return: nextToken:  An enumeration token that can be used in a request to return the next batch of the results.<br/>commentsForComparedCommitData:  A list of comment objects on the compared commit.<br/>        """
        pass

    def get_comments_for_pull_request(self, pullRequestId: str, repositoryName: str=None, beforeCommitId: str=None, afterCommitId: str=None, nextToken: str=None, maxResults: int=None) -> GetCommentsForPullRequestOutput:
        """Returns comments made on a pull request.

        :param self:
        :param maxResults:  A non-negative integer used to limit the number of returned results. The default is 100 comments. You can return up to 500 comments with a single request.<br/>nextToken:  An enumeration token that when provided in a request, returns the next batch of the results.<br/>afterCommitId:  The full commit ID of the commit in the source branch that was the tip of the branch at the time the comment was made.<br/>beforeCommitId:  The full commit ID of the commit in the destination branch that was the tip of the branch at the time the pull request was created.<br/>repositoryName:  The name of the repository that contains the pull request.<br/>pullRequestId:  The system-generated ID of the pull request. To get this ID, use <a>ListPullRequests</a>.<br/>
        :return: nextToken:  An enumeration token that can be used in a request to return the next batch of the results.<br/>commentsForPullRequestData:  An array of comment objects on the pull request.<br/>        """
        pass

    def get_commit(self, commitId: str, repositoryName: str) -> GetCommitOutput:
        """Returns information about a commit, including commit message and committer information.

        :param self:
        :param commitId:  The commit ID. Commit IDs are the full SHA of the commit.<br/>repositoryName:  The name of the repository to which the commit was made.<br/>
        :return: commit:  A commit data type object that contains information about the specified commit.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>additionalData</i> Any additional data associated with the specified commit.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>committer</i> Information about the person who committed the specified commit, also known as the committer. Information includes the date in timestamp format with GMT offset, the name of the committer, and the email address for the committer, as configured in Git. For more information about the difference between an author and a committer in Git, see <a href="http://git-scm.com/book/ch2-3.html">Viewing the Commit History</a> in Pro Git by Scott Chacon and Ben Straub.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>committer</b>: Information about the person who committed the specified commit, also known as the committer. Information includes the date in timestamp format with GMT offset, the name of the committer, and the email address for the committer, as configured in Git. For more information about the difference between an author and a committer in Git, see <a href="http://git-scm.com/book/ch2-3.html">Viewing the Commit History</a> in Pro Git by Scott Chacon and Ben Straub.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>author</i> Information about the author of the specified commit. Information includes the date in timestamp format with GMT offset, the name of the author, and the email address for the author, as configured in Git.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>author</b>: Information about the author of the specified commit. Information includes the date in timestamp format with GMT offset, the name of the author, and the email address for the author, as configured in Git.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>message</i> The commit message associated with the specified commit.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>parents</i> A list of parent commits for the specified commit. Each parent commit ID is the full commit ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>treeId</i> Tree information for the specified commit.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>commitId</i> The full SHA of the specified commit. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>date</i> The date when the specified commit was commited, in timestamp format with GMT offset.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>email</i> The email address associated with the user who made the commit, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the user who made the specified commit.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>date</i> The date when the specified commit was commited, in timestamp format with GMT offset.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>email</i> The email address associated with the user who made the commit, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the user who made the specified commit.<br/>        """
        pass

    def get_differences(self, afterCommitSpecifier: str, repositoryName: str, beforeCommitSpecifier: str=None, beforePath: str=None, afterPath: str=None, MaxResults: int=None, NextToken: str=None) -> GetDifferencesOutput:
        """Returns information about the differences in a valid commit specifier (such as a branch, tag, HEAD, commit ID or other fully qualified reference). Results can be limited to a specified path.

        :param self:
        :param NextToken:  An enumeration token that when provided in a request, returns the next batch of the results.<br/>MaxResults:  A non-negative integer used to limit the number of returned results.<br/>afterPath:  The file path in which to check differences. Limits the results to this path. Can also be used to specify the changed name of a directory or folder, if it has changed. If not specified, differences will be shown for all paths.<br/>beforePath:  The file path in which to check for differences. Limits the results to this path. Can also be used to specify the previous name of a directory or folder. If <code>beforePath</code> and <code>afterPath</code> are not specified, differences will be shown for all paths.<br/>afterCommitSpecifier:  The branch, tag, HEAD, or other fully qualified reference used to identify a commit.<br/>beforeCommitSpecifier:  The branch, tag, HEAD, or other fully qualified reference used to identify a commit. For example, the full commit ID. Optional. If not specified, all changes prior to the <code>afterCommitSpecifier</code> value will be shown. If you do not use <code>beforeCommitSpecifier</code> in your request, consider limiting the results with <code>maxResults</code>.<br/>repositoryName:  The name of the repository where you want to get differences.<br/>
        :return: NextToken:  An enumeration token that can be used in a request to return the next batch of the results.<br/>differences:  A differences data type object that contains information about the differences, including whether the difference is added, modified, or deleted (A, D, M).<br/>        """
        pass

    def get_file(self, filePath: str, repositoryName: str, commitSpecifier: str=None) -> GetFileOutput:
        """Returns the base-64 encoded contents of a specified file and its metadata.

        :param self:
        :param filePath:  The fully-qualified path to the file, including the full name and extension of the file. For example, /examples/file.md is the fully-qualified path to a file named file.md in a folder named examples.<br/>commitSpecifier:  The fully-quaified reference that identifies the commit that contains the file. For example, you could specify a full commit ID, a tag, a branch name, or a reference such as refs/heads/master. If none is provided, then the head commit will be used.<br/>repositoryName:  The name of the repository that contains the file.<br/>
        :return: fileContent:  The base-64 encoded binary data object that represents the content of the file.<br/>fileSize:  The size of the contents of the file, in bytes.<br/>fileMode:  The extrapolated file mode permissions of the blob. Valid values include strings such as EXECUTABLE and not numeric values. <note> The file mode permissions returned by this API are not the standard file mode permission values, such as 100644, but rather extrapolated values. See below for a full list of supported return values. </note><br/>filePath:  The fully qualified path to the specified file. This returns the name and extension of the file.<br/>blobId:  The blob ID of the object that represents the file content.<br/>commitId:  The full commit ID of the commit that contains the content returned by GetFile.<br/>        """
        pass

    def get_folder(self, folderPath: str, repositoryName: str, commitSpecifier: str=None) -> GetFolderOutput:
        """Returns the contents of a specified folder in a repository.

        :param self:
        :param folderPath:  The fully-qualified path to the folder whose contents will be returned, including the folder name. For example, /examples is a fully-qualified path to a folder named examples that was created off of the root directory (/) of a repository. <br/>commitSpecifier:  A fully-qualified reference used to identify a commit that contains the version of the folder's content to return. A fully-qualified reference can be a commit ID, branch name, tag, or reference such as HEAD. If no specifier is provided, the folder content will be returned as it exists in the HEAD commit.<br/>repositoryName:  The name of the repository.<br/>
        :return: subModules:  The list of submodules that exist in the specified folder, if any.<br/>symbolicLinks:  The list of symbolic links to other files and folders that exist in the specified folder, if any.<br/>files:  The list of files that exist in the specified folder, if any.<br/>subFolders:  The list of folders that exist beneath the specified folder, if any.<br/>treeId:  The full SHA-1 pointer of the tree information for the commit that contains the folder.<br/>folderPath:  The fully-qualified path of the folder whose contents are returned.<br/>commitId:  The full commit ID used as a reference for which version of the folder content is returned.<br/>        """
        pass

    def get_merge_commit(self, destinationCommitSpecifier: str, sourceCommitSpecifier: str, repositoryName: str, conflictDetailLevel: str=None, conflictResolutionStrategy: str=None) -> GetMergeCommitOutput:
        """Returns information about a specified merge commit.

        :param self:
        :param conflictResolutionStrategy:  Specifies which branch to use when resolving conflicts, or whether to attempt automatically merging two versions of a file. The default is NONE, which requires any conflicts to be resolved manually before the merge operation will be successful.<br/>conflictDetailLevel:  The level of conflict detail to use. If unspecified, the default FILE_LEVEL is used, which will return a not mergeable result if the same file has differences in both branches. If LINE_LEVEL is specified, a conflict will be considered not mergeable if the same file in both branches has differences on the same line.<br/>destinationCommitSpecifier:  The branch, tag, HEAD, or other fully qualified reference used to identify a commit. For example, a branch name or a full commit ID.<br/>sourceCommitSpecifier:  The branch, tag, HEAD, or other fully qualified reference used to identify a commit. For example, a branch name or a full commit ID.<br/>repositoryName:  The name of the repository that contains the merge commit about which you want to get information.<br/>
        :return: mergedCommitId:  The commit ID for the merge commit created when the source branch was merged into the destination branch. If the fast-forward merge strategy was used, no merge commit exists.<br/>baseCommitId:  The commit ID of the merge base.<br/>destinationCommitId:  The commit ID of the destination commit specifier that was used in the merge evaluation.<br/>sourceCommitId:  The commit ID of the source commit specifier that was used in the merge evaluation.<br/>        """
        pass

    def get_merge_conflicts(self, mergeOption: str, sourceCommitSpecifier: str, destinationCommitSpecifier: str, repositoryName: str, conflictDetailLevel: str=None, maxConflictFiles: int=None, conflictResolutionStrategy: str=None, nextToken: str=None) -> GetMergeConflictsOutput:
        """Returns information about merge conflicts between the before and after commit IDs for a pull request in a repository.

        :param self:
        :param nextToken:  An enumeration token that when provided in a request, returns the next batch of the results.<br/>conflictResolutionStrategy:  Specifies which branch to use when resolving conflicts, or whether to attempt automatically merging two versions of a file. The default is NONE, which requires any conflicts to be resolved manually before the merge operation will be successful.<br/>maxConflictFiles:  The maximum number of files to include in the output.<br/>conflictDetailLevel:  The level of conflict detail to use. If unspecified, the default FILE_LEVEL is used, which will return a not mergeable result if the same file has differences in both branches. If LINE_LEVEL is specified, a conflict will be considered not mergeable if the same file in both branches has differences on the same line.<br/>mergeOption:  The merge option or strategy you want to use to merge the code. <br/>sourceCommitSpecifier:  The branch, tag, HEAD, or other fully qualified reference used to identify a commit. For example, a branch name or a full commit ID.<br/>destinationCommitSpecifier:  The branch, tag, HEAD, or other fully qualified reference used to identify a commit. For example, a branch name or a full commit ID.<br/>repositoryName:  The name of the repository where the pull request was created.<br/>
        :return: nextToken:  An enumeration token that can be used in a request to return the next batch of the results.<br/>conflictMetadataList:  A list of metadata for any conflicting files. If the specified merge strategy is FAST_FORWARD_MERGE, this list will always be empty.<br/>baseCommitId:  The commit ID of the merge base.<br/>sourceCommitId:  The commit ID of the source commit specifier that was used in the merge evaluation.<br/>destinationCommitId:  The commit ID of the destination commit specifier that was used in the merge evaluation.<br/>mergeable:  A Boolean value that indicates whether the code is mergeable by the specified merge option.<br/>        """
        pass

    def get_merge_options(self, destinationCommitSpecifier: str, sourceCommitSpecifier: str, repositoryName: str, conflictDetailLevel: str=None, conflictResolutionStrategy: str=None) -> GetMergeOptionsOutput:
        """Returns information about the merge options available for merging two specified branches. For details about why a particular merge option is not available, use GetMergeConflicts or DescribeMergeConflicts.

        :param self:
        :param conflictResolutionStrategy:  Specifies which branch to use when resolving conflicts, or whether to attempt automatically merging two versions of a file. The default is NONE, which requires any conflicts to be resolved manually before the merge operation will be successful.<br/>conflictDetailLevel:  The level of conflict detail to use. If unspecified, the default FILE_LEVEL is used, which will return a not mergeable result if the same file has differences in both branches. If LINE_LEVEL is specified, a conflict will be considered not mergeable if the same file in both branches has differences on the same line.<br/>destinationCommitSpecifier:  The branch, tag, HEAD, or other fully qualified reference used to identify a commit. For example, a branch name or a full commit ID.<br/>sourceCommitSpecifier:  The branch, tag, HEAD, or other fully qualified reference used to identify a commit. For example, a branch name or a full commit ID.<br/>repositoryName:  The name of the repository that contains the commits about which you want to get merge options.<br/>
        :return: baseCommitId:  The commit ID of the merge base.<br/>destinationCommitId:  The commit ID of the destination commit specifier that was used in the merge evaluation.<br/>sourceCommitId:  The commit ID of the source commit specifier that was used in the merge evaluation.<br/>mergeOptions:  The merge option or strategy used to merge the code.<br/>        """
        pass

    def get_pull_request(self, pullRequestId: str) -> GetPullRequestOutput:
        """Gets information about a pull request in a specified repository.

        :param self:
        :param pullRequestId:  The system-generated ID of the pull request. To get this ID, use <a>ListPullRequests</a>.<br/>
        :return: pullRequest:  Information about the specified pull request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clientRequestToken</i> A unique, client-generated idempotency token that when provided in a request, ensures the request cannot be repeated with a changed parameter. If a request is received with the same parameters and a token is included, the request will return information about the initial request that used that token.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pullRequestTargets</i> The targets of the pull request, including the source branch and destination branch for the pull request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authorArn</i> The Amazon Resource Name (ARN) of the user who created the pull request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pullRequestStatus</i> The status of the pull request. Pull request status can only change from <code>OPEN</code> to <code>CLOSED</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDate</i> The date and time the pull request was originally created, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastActivityDate</i> The day and time of the last user or system activity on the pull request, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The user-defined description of the pull request. This description can be used to clarify what should be reviewed and other details of the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>title</i> The user-defined title of the pull request. This title is displayed in the list of pull requests to other users of the repository.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pullRequestId</i> The system-generated ID of the pull request. <br/>        """
        pass

    def get_repository(self, repositoryName: str) -> GetRepositoryOutput:
        """Returns information about a repository. <note> The description field for a repository accepts all HTML characters and all valid Unicode characters. Applications that do not HTML-encode the description and display it in a web page could expose users to potentially malicious code. Make sure that you HTML-encode the description field in any application that uses this API to display the repository description on a web page. </note>

        :param self:
        :param repositoryName:  The name of the repository to get information about.<br/>
        :return: repositoryMetadata:  Information about the repository.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the repository.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cloneUrlSsh</i> The URL to use for cloning the repository over SSH.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cloneUrlHttp</i> The URL to use for cloning the repository over HTTPS.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDate</i> The date and time the repository was created, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastModifiedDate</i> The date and time the repository was last modified, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultBranch</i> The repository's default branch name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>repositoryDescription</i> A comment or description about the repository.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>repositoryName</i> The repository's name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>repositoryId</i> The ID of the repository.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>accountId</i> The ID of the AWS account associated with the repository.<br/>        """
        pass

    def get_repository_triggers(self, repositoryName: str) -> GetRepositoryTriggersOutput:
        """Gets information about triggers configured for a repository.

        :param self:
        :param repositoryName:  The name of the repository for which the trigger is configured.<br/>
        :return: triggers:  The JSON block of configuration information for each trigger.<br/>configurationId:  The system-generated unique ID for the trigger.<br/>        """
        pass

    def list_branches(self, repositoryName: str, nextToken: str=None) -> ListBranchesOutput:
        """Gets information about one or more branches in a repository.

        :param self:
        :param nextToken:  An enumeration token that allows the operation to batch the results.<br/>repositoryName:  The name of the repository that contains the branches.<br/>
        :return: nextToken:  An enumeration token that returns the batch of the results.<br/>branches:  The list of branch names.<br/>        """
        pass

    def list_pull_requests(self, repositoryName: str, authorArn: str=None, pullRequestStatus: str=None, nextToken: str=None, maxResults: int=None) -> ListPullRequestsOutput:
        """Returns a list of pull requests for a specified repository. The return list can be refined by pull request status or pull request author ARN.

        :param self:
        :param maxResults:  A non-negative integer used to limit the number of returned results.<br/>nextToken:  An enumeration token that when provided in a request, returns the next batch of the results.<br/>pullRequestStatus:  Optional. The status of the pull request. If used, this refines the results to the pull requests that match the specified status.<br/>authorArn:  Optional. The Amazon Resource Name (ARN) of the user who created the pull request. If used, this filters the results to pull requests created by that user.<br/>repositoryName:  The name of the repository for which you want to list pull requests.<br/>
        :return: nextToken:  An enumeration token that when provided in a request, returns the next batch of the results.<br/>pullRequestIds:  The system-generated IDs of the pull requests.<br/>        """
        pass

    def list_repositories(self, nextToken: str=None, sortBy: str=None, order: str=None) -> ListRepositoriesOutput:
        """Gets information about one or more repositories.

        :param self:
        :param order:  The order in which to sort the results of a list repositories operation.<br/>sortBy:  The criteria used to sort the results of a list repositories operation.<br/>nextToken:  An enumeration token that allows the operation to batch the results of the operation. Batch sizes are 1,000 for list repository operations. When the client sends the token back to AWS CodeCommit, another page of 1,000 records is retrieved.<br/>
        :return: nextToken:  An enumeration token that allows the operation to batch the results of the operation. Batch sizes are 1,000 for list repository operations. When the client sends the token back to AWS CodeCommit, another page of 1,000 records is retrieved.<br/>repositories:  Lists the repositories called by the list repositories operation.<br/>        """
        pass

    def list_tags_for_resource(self, resourceArn: str, nextToken: str=None) -> ListTagsForResourceOutput:
        """Gets information about AWS tags for a specified Amazon Resource Name (ARN) in AWS CodeCommit. For a list of valid resources in AWS CodeCommit, see <a href="https://docs.aws.amazon.com/codecommit/latest/userguide/auth-and-access-control-iam-access-control-identity-based.html#arn-formats">CodeCommit Resources and Operations</a> in the AWS CodeCommit User Guide.

        :param self:
        :param nextToken:  An enumeration token that when provided in a request, returns the next batch of the results.<br/>resourceArn:  The Amazon Resource Name (ARN) of the resource for which you want to get information about tags, if any.<br/>
        :return: nextToken:  An enumeration token that allows the operation to batch the next results of the operation.<br/>tags:  A list of tag key and value pairs associated with the specified resource.<br/>        """
        pass

    def merge_branches_by_fast_forward(self, destinationCommitSpecifier: str, sourceCommitSpecifier: str, repositoryName: str, targetBranch: str=None) -> MergeBranchesByFastForwardOutput:
        """Merges two branches using the fast-forward merge strategy.

        :param self:
        :param targetBranch:  The branch where the merge will be applied.<br/>destinationCommitSpecifier:  The branch, tag, HEAD, or other fully qualified reference used to identify a commit. For example, a branch name or a full commit ID.<br/>sourceCommitSpecifier:  The branch, tag, HEAD, or other fully qualified reference used to identify a commit. For example, a branch name or a full commit ID.<br/>repositoryName:  The name of the repository where you want to merge two branches.<br/>
        :return: treeId:  The tree ID of the merge in the destination or target branch.<br/>commitId:  The commit ID of the merge in the destination or target branch.<br/>        """
        pass

    def merge_branches_by_squash(self, destinationCommitSpecifier: str, sourceCommitSpecifier: str, repositoryName: str, targetBranch: str=None, conflictDetailLevel: str=None, conflictResolutionStrategy: str=None, authorName: str=None, email: str=None, commitMessage: str=None, keepEmptyFolders: KeepEmptyFolders=None, conflictResolution: ConflictResolution=None) -> MergeBranchesBySquashOutput:
        """Merges two branches using the squash merge strategy.

        :param self:
        :param conflictResolution:  A list of inputs to use when resolving conflicts during a merge if AUTOMERGE is chosen as the conflict resolution strategy.:<br/>keepEmptyFolders:  If the commit contains deletions, whether to keep a folder or folder structure if the changes leave the folders empty. If this is specified as true, a .gitkeep file will be created for empty folders. The default is false.<br/>commitMessage:  The commit message for the merge.<br/>email:  The email address of the person merging the branches. This information will be used in the commit information for the merge.<br/>authorName:  The name of the author who created the commit. This information will be used as both the author and committer for the commit.<br/>conflictResolutionStrategy:  Specifies which branch to use when resolving conflicts, or whether to attempt automatically merging two versions of a file. The default is NONE, which requires any conflicts to be resolved manually before the merge operation will be successful.<br/>conflictDetailLevel:  The level of conflict detail to use. If unspecified, the default FILE_LEVEL is used, which will return a not mergeable result if the same file has differences in both branches. If LINE_LEVEL is specified, a conflict will be considered not mergeable if the same file in both branches has differences on the same line.<br/>targetBranch:  The branch where the merge will be applied. <br/>destinationCommitSpecifier:  The branch, tag, HEAD, or other fully qualified reference used to identify a commit. For example, a branch name or a full commit ID.<br/>sourceCommitSpecifier:  The branch, tag, HEAD, or other fully qualified reference used to identify a commit. For example, a branch name or a full commit ID.<br/>repositoryName:  The name of the repository where you want to merge two branches.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>setFileModes</i> File modes that will be set as part of the merge conflict resolution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deleteFiles</i> Files that will be deleted as part of the merge conflict resolution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>replaceContents</i> Files that will have content replaced as part of the merge conflict resolution.<br/>
        :return: treeId:  The tree ID of the merge in the destination or target branch.<br/>commitId:  The commit ID of the merge in the destination or target branch.<br/>        """
        pass

    def merge_branches_by_three_way(self, destinationCommitSpecifier: str, sourceCommitSpecifier: str, repositoryName: str, targetBranch: str=None, conflictDetailLevel: str=None, conflictResolutionStrategy: str=None, authorName: str=None, email: str=None, commitMessage: str=None, keepEmptyFolders: KeepEmptyFolders=None, conflictResolution: ConflictResolution=None) -> MergeBranchesByThreeWayOutput:
        """Merges two specified branches using the three-way merge strategy.

        :param self:
        :param conflictResolution:  A list of inputs to use when resolving conflicts during a merge if AUTOMERGE is chosen as the conflict resolution strategy.:<br/>keepEmptyFolders:  If the commit contains deletions, whether to keep a folder or folder structure if the changes leave the folders empty. If this is specified as true, a .gitkeep file will be created for empty folders. The default is false.<br/>commitMessage:  The commit message to include in the commit information for the merge.<br/>email:  The email address of the person merging the branches. This information will be used in the commit information for the merge.<br/>authorName:  The name of the author who created the commit. This information will be used as both the author and committer for the commit.<br/>conflictResolutionStrategy:  Specifies which branch to use when resolving conflicts, or whether to attempt automatically merging two versions of a file. The default is NONE, which requires any conflicts to be resolved manually before the merge operation will be successful.<br/>conflictDetailLevel:  The level of conflict detail to use. If unspecified, the default FILE_LEVEL is used, which will return a not mergeable result if the same file has differences in both branches. If LINE_LEVEL is specified, a conflict will be considered not mergeable if the same file in both branches has differences on the same line.<br/>targetBranch:  The branch where the merge will be applied. <br/>destinationCommitSpecifier:  The branch, tag, HEAD, or other fully qualified reference used to identify a commit. For example, a branch name or a full commit ID.<br/>sourceCommitSpecifier:  The branch, tag, HEAD, or other fully qualified reference used to identify a commit. For example, a branch name or a full commit ID.<br/>repositoryName:  The name of the repository where you want to merge two branches.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>setFileModes</i> File modes that will be set as part of the merge conflict resolution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deleteFiles</i> Files that will be deleted as part of the merge conflict resolution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>replaceContents</i> Files that will have content replaced as part of the merge conflict resolution.<br/>
        :return: treeId:  The tree ID of the merge in the destination or target branch.<br/>commitId:  The commit ID of the merge in the destination or target branch.<br/>        """
        pass

    def merge_pull_request_by_fast_forward(self, repositoryName: str, pullRequestId: str, sourceCommitId: str=None) -> MergePullRequestByFastForwardOutput:
        """Attempts to merge the source commit of a pull request into the specified destination branch for that pull request at the specified commit using the fast-forward merge strategy. If the merge is successful, it closes the pull request.

        :param self:
        :param sourceCommitId:  The full commit ID of the original or updated commit in the pull request source branch. Pass this value if you want an exception thrown if the current commit ID of the tip of the source branch does not match this commit ID.<br/>repositoryName:  The name of the repository where the pull request was created.<br/>pullRequestId:  The system-generated ID of the pull request. To get this ID, use <a>ListPullRequests</a>.<br/>
        :return: pullRequest:  Information about the specified pull request, including information about the merge.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clientRequestToken</i> A unique, client-generated idempotency token that when provided in a request, ensures the request cannot be repeated with a changed parameter. If a request is received with the same parameters and a token is included, the request will return information about the initial request that used that token.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pullRequestTargets</i> The targets of the pull request, including the source branch and destination branch for the pull request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authorArn</i> The Amazon Resource Name (ARN) of the user who created the pull request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pullRequestStatus</i> The status of the pull request. Pull request status can only change from <code>OPEN</code> to <code>CLOSED</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDate</i> The date and time the pull request was originally created, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastActivityDate</i> The day and time of the last user or system activity on the pull request, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The user-defined description of the pull request. This description can be used to clarify what should be reviewed and other details of the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>title</i> The user-defined title of the pull request. This title is displayed in the list of pull requests to other users of the repository.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pullRequestId</i> The system-generated ID of the pull request. <br/>        """
        pass

    def merge_pull_request_by_squash(self, repositoryName: str, pullRequestId: str, sourceCommitId: str=None, conflictDetailLevel: str=None, conflictResolutionStrategy: str=None, commitMessage: str=None, authorName: str=None, email: str=None, keepEmptyFolders: KeepEmptyFolders=None, conflictResolution: ConflictResolution=None) -> MergePullRequestBySquashOutput:
        """Attempts to merge the source commit of a pull request into the specified destination branch for that pull request at the specified commit using the squash merge strategy. If the merge is successful, it closes the pull request.

        :param self:
        :param conflictResolution:  A list of inputs to use when resolving conflicts during a merge if AUTOMERGE is chosen as the conflict resolution strategy.:<br/>keepEmptyFolders:  If the commit contains deletions, whether to keep a folder or folder structure if the changes leave the folders empty. If this is specified as true, a .gitkeep file will be created for empty folders. The default is false.<br/>email:  The email address of the person merging the branches. This information will be used in the commit information for the merge.<br/>authorName:  The name of the author who created the commit. This information will be used as both the author and committer for the commit.<br/>commitMessage:  The commit message to include in the commit information for the merge.<br/>conflictResolutionStrategy:  Specifies which branch to use when resolving conflicts, or whether to attempt automatically merging two versions of a file. The default is NONE, which requires any conflicts to be resolved manually before the merge operation will be successful.<br/>conflictDetailLevel:  The level of conflict detail to use. If unspecified, the default FILE_LEVEL is used, which will return a not mergeable result if the same file has differences in both branches. If LINE_LEVEL is specified, a conflict will be considered not mergeable if the same file in both branches has differences on the same line.<br/>sourceCommitId:  The full commit ID of the original or updated commit in the pull request source branch. Pass this value if you want an exception thrown if the current commit ID of the tip of the source branch does not match this commit ID.<br/>repositoryName:  The name of the repository where the pull request was created.<br/>pullRequestId:  The system-generated ID of the pull request. To get this ID, use <a>ListPullRequests</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>setFileModes</i> File modes that will be set as part of the merge conflict resolution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deleteFiles</i> Files that will be deleted as part of the merge conflict resolution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>replaceContents</i> Files that will have content replaced as part of the merge conflict resolution.<br/>
        :return: pullRequest:  Returns information about a pull request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clientRequestToken</i> A unique, client-generated idempotency token that when provided in a request, ensures the request cannot be repeated with a changed parameter. If a request is received with the same parameters and a token is included, the request will return information about the initial request that used that token.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pullRequestTargets</i> The targets of the pull request, including the source branch and destination branch for the pull request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authorArn</i> The Amazon Resource Name (ARN) of the user who created the pull request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pullRequestStatus</i> The status of the pull request. Pull request status can only change from <code>OPEN</code> to <code>CLOSED</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDate</i> The date and time the pull request was originally created, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastActivityDate</i> The day and time of the last user or system activity on the pull request, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The user-defined description of the pull request. This description can be used to clarify what should be reviewed and other details of the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>title</i> The user-defined title of the pull request. This title is displayed in the list of pull requests to other users of the repository.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pullRequestId</i> The system-generated ID of the pull request. <br/>        """
        pass

    def merge_pull_request_by_three_way(self, repositoryName: str, pullRequestId: str, sourceCommitId: str=None, conflictDetailLevel: str=None, conflictResolutionStrategy: str=None, commitMessage: str=None, authorName: str=None, email: str=None, keepEmptyFolders: KeepEmptyFolders=None, conflictResolution: ConflictResolution=None) -> MergePullRequestByThreeWayOutput:
        """Attempts to merge the source commit of a pull request into the specified destination branch for that pull request at the specified commit using the three-way merge strategy. If the merge is successful, it closes the pull request.

        :param self:
        :param conflictResolution:  A list of inputs to use when resolving conflicts during a merge if AUTOMERGE is chosen as the conflict resolution strategy.:<br/>keepEmptyFolders:  If the commit contains deletions, whether to keep a folder or folder structure if the changes leave the folders empty. If this is specified as true, a .gitkeep file will be created for empty folders. The default is false.<br/>email:  The email address of the person merging the branches. This information will be used in the commit information for the merge.<br/>authorName:  The name of the author who created the commit. This information will be used as both the author and committer for the commit.<br/>commitMessage:  The commit message to include in the commit information for the merge.<br/>conflictResolutionStrategy:  Specifies which branch to use when resolving conflicts, or whether to attempt automatically merging two versions of a file. The default is NONE, which requires any conflicts to be resolved manually before the merge operation will be successful.<br/>conflictDetailLevel:  The level of conflict detail to use. If unspecified, the default FILE_LEVEL is used, which will return a not mergeable result if the same file has differences in both branches. If LINE_LEVEL is specified, a conflict will be considered not mergeable if the same file in both branches has differences on the same line.<br/>sourceCommitId:  The full commit ID of the original or updated commit in the pull request source branch. Pass this value if you want an exception thrown if the current commit ID of the tip of the source branch does not match this commit ID.<br/>repositoryName:  The name of the repository where the pull request was created.<br/>pullRequestId:  The system-generated ID of the pull request. To get this ID, use <a>ListPullRequests</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>setFileModes</i> File modes that will be set as part of the merge conflict resolution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deleteFiles</i> Files that will be deleted as part of the merge conflict resolution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>replaceContents</i> Files that will have content replaced as part of the merge conflict resolution.<br/>
        :return: pullRequest:  Returns information about a pull request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clientRequestToken</i> A unique, client-generated idempotency token that when provided in a request, ensures the request cannot be repeated with a changed parameter. If a request is received with the same parameters and a token is included, the request will return information about the initial request that used that token.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pullRequestTargets</i> The targets of the pull request, including the source branch and destination branch for the pull request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authorArn</i> The Amazon Resource Name (ARN) of the user who created the pull request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pullRequestStatus</i> The status of the pull request. Pull request status can only change from <code>OPEN</code> to <code>CLOSED</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDate</i> The date and time the pull request was originally created, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastActivityDate</i> The day and time of the last user or system activity on the pull request, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The user-defined description of the pull request. This description can be used to clarify what should be reviewed and other details of the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>title</i> The user-defined title of the pull request. This title is displayed in the list of pull requests to other users of the repository.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pullRequestId</i> The system-generated ID of the pull request. <br/>        """
        pass

    def post_comment_for_compared_commit(self, content: str, afterCommitId: str, repositoryName: str, beforeCommitId: str=None, location: Location=None, clientRequestToken: str=None) -> PostCommentForComparedCommitOutput:
        """Posts a comment on the comparison between two commits.

        :param self:
        :param clientRequestToken:  A unique, client-generated idempotency token that when provided in a request, ensures the request cannot be repeated with a changed parameter. If a request is received with the same parameters and a token is included, the request will return information about the initial request that used that token.<br/>content:  The content of the comment you want to make.<br/>location:  The location of the comparison where you want to comment.:<br/>afterCommitId:  To establish the directionality of the comparison, the full commit ID of the 'after' commit.<br/>beforeCommitId:  To establish the directionality of the comparison, the full commit ID of the 'before' commit. <note> This is required for commenting on any commit unless that commit is the initial commit. </note><br/>repositoryName:  The name of the repository where you want to post a comment on the comparison between commits.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>relativeFileVersion</i> In a comparison of commits or a pull request, whether the change is in the 'before' or 'after' of that comparison.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>filePosition</i> The position of a change within a compared file, in line number format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>filePath</i> The name of the file being compared, including its extension and subdirectory, if any.<br/>
        :return: comment:  The content of the comment you posted.:<br/>location:  The location of the comment in the comparison between the two commits.:<br/>afterBlobId:  In the directionality you established, the blob ID of the 'after' blob.<br/>beforeBlobId:  In the directionality you established, the blob ID of the 'before' blob.<br/>afterCommitId:  In the directionality you established, the full commit ID of the 'after' commit.<br/>beforeCommitId:  In the directionality you established, the full commit ID of the 'before' commit.<br/>repositoryName:  The name of the repository where you posted a comment on the comparison between commits.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>relativeFileVersion</i> In a comparison of commits or a pull request, whether the change is in the 'before' or 'after' of that comparison.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>filePosition</i> The position of a change within a compared file, in line number format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>filePath</i> The name of the file being compared, including its extension and subdirectory, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clientRequestToken</i> A unique, client-generated idempotency token that when provided in a request, ensures the request cannot be repeated with a changed parameter. If a request is received with the same parameters and a token is included, the request will return information about the initial request that used that token.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deleted</i> A Boolean value indicating whether the comment has been deleted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authorArn</i> The Amazon Resource Name (ARN) of the person who posted the comment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastModifiedDate</i> The date and time the comment was most recently modified, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDate</i> The date and time the comment was created, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>inReplyTo</i> The ID of the comment for which this comment is a reply, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>content</i> The content of the comment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>commentId</i> The system-generated comment ID.<br/>        """
        pass

    def post_comment_for_pull_request(self, content: str, afterCommitId: str, beforeCommitId: str, repositoryName: str, pullRequestId: str, location: Location=None, clientRequestToken: str=None) -> PostCommentForPullRequestOutput:
        """Posts a comment on a pull request.

        :param self:
        :param clientRequestToken:  A unique, client-generated idempotency token that when provided in a request, ensures the request cannot be repeated with a changed parameter. If a request is received with the same parameters and a token is included, the request will return information about the initial request that used that token.<br/>content:  The content of your comment on the change.<br/>location:  The location of the change where you want to post your comment. If no location is provided, the comment will be posted as a general comment on the pull request difference between the before commit ID and the after commit ID.:<br/>afterCommitId:  The full commit ID of the commit in the source branch that is the current tip of the branch for the pull request when you post the comment.<br/>beforeCommitId:  The full commit ID of the commit in the destination branch that was the tip of the branch at the time the pull request was created.<br/>repositoryName:  The name of the repository where you want to post a comment on a pull request.<br/>pullRequestId:  The system-generated ID of the pull request. To get this ID, use <a>ListPullRequests</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>relativeFileVersion</i> In a comparison of commits or a pull request, whether the change is in the 'before' or 'after' of that comparison.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>filePosition</i> The position of a change within a compared file, in line number format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>filePath</i> The name of the file being compared, including its extension and subdirectory, if any.<br/>
        :return: comment:  The content of the comment you posted.:<br/>location:  The location of the change where you posted your comment.:<br/>afterBlobId:  In the directionality of the pull request, the blob ID of the 'after' blob.<br/>beforeBlobId:  In the directionality of the pull request, the blob ID of the 'before' blob.<br/>afterCommitId:  The full commit ID of the commit in the destination branch where the pull request will be merged.<br/>beforeCommitId:  The full commit ID of the commit in the source branch used to create the pull request, or in the case of an updated pull request, the full commit ID of the commit used to update the pull request.<br/>pullRequestId:  The system-generated ID of the pull request. <br/>repositoryName:  The name of the repository where you posted a comment on a pull request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>relativeFileVersion</i> In a comparison of commits or a pull request, whether the change is in the 'before' or 'after' of that comparison.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>filePosition</i> The position of a change within a compared file, in line number format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>filePath</i> The name of the file being compared, including its extension and subdirectory, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clientRequestToken</i> A unique, client-generated idempotency token that when provided in a request, ensures the request cannot be repeated with a changed parameter. If a request is received with the same parameters and a token is included, the request will return information about the initial request that used that token.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deleted</i> A Boolean value indicating whether the comment has been deleted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authorArn</i> The Amazon Resource Name (ARN) of the person who posted the comment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastModifiedDate</i> The date and time the comment was most recently modified, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDate</i> The date and time the comment was created, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>inReplyTo</i> The ID of the comment for which this comment is a reply, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>content</i> The content of the comment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>commentId</i> The system-generated comment ID.<br/>        """
        pass

    def post_comment_reply(self, content: str, inReplyTo: str, clientRequestToken: str=None) -> PostCommentReplyOutput:
        """Posts a comment in reply to an existing comment on a comparison between commits or a pull request.

        :param self:
        :param content:  The contents of your reply to a comment.<br/>clientRequestToken:  A unique, client-generated idempotency token that when provided in a request, ensures the request cannot be repeated with a changed parameter. If a request is received with the same parameters and a token is included, the request will return information about the initial request that used that token.<br/>inReplyTo:  The system-generated ID of the comment to which you want to reply. To get this ID, use <a>GetCommentsForComparedCommit</a> or <a>GetCommentsForPullRequest</a>.<br/>
        :return: comment:  Information about the reply to a comment.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clientRequestToken</i> A unique, client-generated idempotency token that when provided in a request, ensures the request cannot be repeated with a changed parameter. If a request is received with the same parameters and a token is included, the request will return information about the initial request that used that token.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deleted</i> A Boolean value indicating whether the comment has been deleted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authorArn</i> The Amazon Resource Name (ARN) of the person who posted the comment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastModifiedDate</i> The date and time the comment was most recently modified, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDate</i> The date and time the comment was created, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>inReplyTo</i> The ID of the comment for which this comment is a reply, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>content</i> The content of the comment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>commentId</i> The system-generated comment ID.<br/>        """
        pass

    def put_file(self, filePath: str, fileContent: FileContent, branchName: str, repositoryName: str, fileMode: str=None, parentCommitId: str=None, commitMessage: str=None, name: str=None, email: str=None) -> PutFileOutput:
        """Adds or updates a file in a branch in an AWS CodeCommit repository, and generates a commit for the addition in the specified branch.

        :param self:
        :param email:  An email address for the person adding or updating the file.<br/>name:  The name of the person adding or updating the file. While optional, adding a name is strongly encouraged in order to provide a more useful commit history for your repository.<br/>commitMessage:  A message about why this file was added or updated. While optional, adding a message is strongly encouraged in order to provide a more useful commit history for your repository.<br/>parentCommitId:  The full commit ID of the head commit in the branch where you want to add or update the file. If this is an empty repository, no commit ID is required. If this is not an empty repository, a commit ID is required.  The commit ID must match the ID of the head commit at the time of the operation, or an error will occur, and the file will not be added or updated.<br/>fileMode:  The file mode permissions of the blob. Valid file mode permissions are listed below.<br/>filePath:  The name of the file you want to add or update, including the relative path to the file in the repository. <note> If the path does not currently exist in the repository, the path will be created as part of adding the file. </note><br/>fileContent:  The content of the file, in binary object format. <br/>branchName:  The name of the branch where you want to add or update the file. If this is an empty repository, this branch will be created.<br/>repositoryName:  The name of the repository where you want to add or update the file.<br/>
        :return: treeId:  The full SHA-1 pointer of the tree information for the commit that contains this file change.<br/>blobId:  The ID of the blob, which is its SHA-1 pointer.<br/>commitId:  The full SHA of the commit that contains this file change.<br/>        """
        pass

    def put_repository_triggers(self, triggers: List[RepositoryTrigger], repositoryName: str) -> PutRepositoryTriggersOutput:
        """Replaces all triggers for a repository. This can be used to create or delete triggers.

        :param self:
        :param triggers:  The JSON block of configuration information for each trigger.<br/>repositoryName:  The name of the repository where you want to create or update the trigger.<br/>
        :return: configurationId:  The system-generated unique ID for the create or update operation.<br/>        """
        pass

    def tag_resource(self, tags: TagsMap, resourceArn: str):
        """Adds or updates tags for a resource in AWS CodeCommit. For a list of valid resources in AWS CodeCommit, see <a href="https://docs.aws.amazon.com/codecommit/latest/userguide/auth-and-access-control-iam-access-control-identity-based.html#arn-formats">CodeCommit Resources and Operations</a> in the AWS CodeCommit User Guide.

        :param self:
        :param tags:  The key-value pair to use when tagging this repository.<br/>resourceArn:  The Amazon Resource Name (ARN) of the resource to which you want to add or update tags.<br/>
        :return:         """
        pass

    def test_repository_triggers(self, triggers: List[RepositoryTrigger], repositoryName: str) -> TestRepositoryTriggersOutput:
        """Tests the functionality of repository triggers by sending information to the trigger target. If real data is available in the repository, the test will send data from the last commit. If no data is available, sample data will be generated.

        :param self:
        :param triggers:  The list of triggers to test.<br/>repositoryName:  The name of the repository in which to test the triggers.<br/>
        :return: failedExecutions:  The list of triggers that were not able to be tested. This list provides the names of the triggers that could not be tested, separated by commas.<br/>successfulExecutions:  The list of triggers that were successfully tested. This list provides the names of the triggers that were successfully tested, separated by commas.<br/>        """
        pass

    def untag_resource(self, tagKeys: List[TagKey], resourceArn: str):
        """Removes tags for a resource in AWS CodeCommit. For a list of valid resources in AWS CodeCommit, see <a href="https://docs.aws.amazon.com/codecommit/latest/userguide/auth-and-access-control-iam-access-control-identity-based.html#arn-formats">CodeCommit Resources and Operations</a> in the AWS CodeCommit User Guide.

        :param self:
        :param tagKeys:  The tag key for each tag that you want to remove from the resource.<br/>resourceArn:  The Amazon Resource Name (ARN) of the resource to which you want to remove tags.<br/>
        :return:         """
        pass

    def update_comment(self, content: str, commentId: str) -> UpdateCommentOutput:
        """Replaces the contents of a comment.

        :param self:
        :param content:  The updated content with which you want to replace the existing content of the comment.<br/>commentId:  The system-generated ID of the comment you want to update. To get this ID, use <a>GetCommentsForComparedCommit</a> or <a>GetCommentsForPullRequest</a>.<br/>
        :return: comment:  Information about the updated comment.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clientRequestToken</i> A unique, client-generated idempotency token that when provided in a request, ensures the request cannot be repeated with a changed parameter. If a request is received with the same parameters and a token is included, the request will return information about the initial request that used that token.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deleted</i> A Boolean value indicating whether the comment has been deleted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authorArn</i> The Amazon Resource Name (ARN) of the person who posted the comment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastModifiedDate</i> The date and time the comment was most recently modified, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDate</i> The date and time the comment was created, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>inReplyTo</i> The ID of the comment for which this comment is a reply, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>content</i> The content of the comment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>commentId</i> The system-generated comment ID.<br/>        """
        pass

    def update_default_branch(self, defaultBranchName: str, repositoryName: str):
        """Sets or changes the default branch name for the specified repository. <note> If you use this operation to change the default branch name to the current default branch name, a success message is returned even though the default branch did not change. </note>

        :param self:
        :param defaultBranchName:  The name of the branch to set as the default.<br/>repositoryName:  The name of the repository to set or change the default branch for.<br/>
        :return:         """
        pass

    def update_pull_request_description(self, description: str, pullRequestId: str) -> UpdatePullRequestDescriptionOutput:
        """Replaces the contents of the description of a pull request.

        :param self:
        :param description:  The updated content of the description for the pull request. This content will replace the existing description.<br/>pullRequestId:  The system-generated ID of the pull request. To get this ID, use <a>ListPullRequests</a>.<br/>
        :return: pullRequest:  Information about the updated pull request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clientRequestToken</i> A unique, client-generated idempotency token that when provided in a request, ensures the request cannot be repeated with a changed parameter. If a request is received with the same parameters and a token is included, the request will return information about the initial request that used that token.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pullRequestTargets</i> The targets of the pull request, including the source branch and destination branch for the pull request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authorArn</i> The Amazon Resource Name (ARN) of the user who created the pull request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pullRequestStatus</i> The status of the pull request. Pull request status can only change from <code>OPEN</code> to <code>CLOSED</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDate</i> The date and time the pull request was originally created, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastActivityDate</i> The day and time of the last user or system activity on the pull request, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The user-defined description of the pull request. This description can be used to clarify what should be reviewed and other details of the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>title</i> The user-defined title of the pull request. This title is displayed in the list of pull requests to other users of the repository.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pullRequestId</i> The system-generated ID of the pull request. <br/>        """
        pass

    def update_pull_request_status(self, pullRequestStatus: str, pullRequestId: str) -> UpdatePullRequestStatusOutput:
        """Updates the status of a pull request. 

        :param self:
        :param pullRequestStatus:  The status of the pull request. The only valid operations are to update the status from <code>OPEN</code> to <code>OPEN</code>, <code>OPEN</code> to <code>CLOSED</code> or from from <code>CLOSED</code> to <code>CLOSED</code>.<br/>pullRequestId:  The system-generated ID of the pull request. To get this ID, use <a>ListPullRequests</a>.<br/>
        :return: pullRequest:  Information about the pull request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clientRequestToken</i> A unique, client-generated idempotency token that when provided in a request, ensures the request cannot be repeated with a changed parameter. If a request is received with the same parameters and a token is included, the request will return information about the initial request that used that token.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pullRequestTargets</i> The targets of the pull request, including the source branch and destination branch for the pull request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authorArn</i> The Amazon Resource Name (ARN) of the user who created the pull request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pullRequestStatus</i> The status of the pull request. Pull request status can only change from <code>OPEN</code> to <code>CLOSED</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDate</i> The date and time the pull request was originally created, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastActivityDate</i> The day and time of the last user or system activity on the pull request, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The user-defined description of the pull request. This description can be used to clarify what should be reviewed and other details of the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>title</i> The user-defined title of the pull request. This title is displayed in the list of pull requests to other users of the repository.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pullRequestId</i> The system-generated ID of the pull request. <br/>        """
        pass

    def update_pull_request_title(self, title: str, pullRequestId: str) -> UpdatePullRequestTitleOutput:
        """Replaces the title of a pull request.

        :param self:
        :param title:  The updated title of the pull request. This will replace the existing title.<br/>pullRequestId:  The system-generated ID of the pull request. To get this ID, use <a>ListPullRequests</a>.<br/>
        :return: pullRequest:  Information about the updated pull request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clientRequestToken</i> A unique, client-generated idempotency token that when provided in a request, ensures the request cannot be repeated with a changed parameter. If a request is received with the same parameters and a token is included, the request will return information about the initial request that used that token.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pullRequestTargets</i> The targets of the pull request, including the source branch and destination branch for the pull request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authorArn</i> The Amazon Resource Name (ARN) of the user who created the pull request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pullRequestStatus</i> The status of the pull request. Pull request status can only change from <code>OPEN</code> to <code>CLOSED</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDate</i> The date and time the pull request was originally created, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastActivityDate</i> The day and time of the last user or system activity on the pull request, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The user-defined description of the pull request. This description can be used to clarify what should be reviewed and other details of the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>title</i> The user-defined title of the pull request. This title is displayed in the list of pull requests to other users of the repository.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pullRequestId</i> The system-generated ID of the pull request. <br/>        """
        pass

    def update_repository_description(self, repositoryName: str, repositoryDescription: str=None):
        """Sets or changes the comment or description for a repository. <note> The description field for a repository accepts all HTML characters and all valid Unicode characters. Applications that do not HTML-encode the description and display it in a web page could expose users to potentially malicious code. Make sure that you HTML-encode the description field in any application that uses this API to display the repository description on a web page. </note>

        :param self:
        :param repositoryDescription:  The new comment or description for the specified repository. Repository descriptions are limited to 1,000 characters.<br/>repositoryName:  The name of the repository to set or change the comment or description for.<br/>
        :return:         """
        pass

    def update_repository_name(self, newName: str, oldName: str):
        """Renames a repository. The repository name must be unique across the calling AWS account. In addition, repository names are limited to 100 alphanumeric, dash, and underscore characters, and cannot include certain characters. The suffix ".git" is prohibited. For a full description of the limits on repository names, see <a href="https://docs.aws.amazon.com/codecommit/latest/userguide/limits.html">Limits</a> in the AWS CodeCommit User Guide.

        :param self:
        :param newName:  The new name for the repository.<br/>oldName:  The existing name of the repository.<br/>
        :return:         """
        pass



class CodeDeploy(BaseClient):
    """<fullname>AWS CodeDeploy</fullname> AWS CodeDeploy is a deployment service that automates application deployments to Amazon EC2 instances, on-premises instances running in your own facility, serverless AWS Lambda functions, or applications in an Amazon ECS service. You can deploy a nearly unlimited variety of application content, such as an updated Lambda function, updated applications in an Amazon ECS service, code, web and configuration files, executables, packages, scripts, multimedia files, and so on. AWS CodeDeploy can deploy application content stored in Amazon S3 buckets, GitHub repositories, or Bitbucket repositories. You do not need to make changes to your existing code before you can use AWS CodeDeploy. AWS CodeDeploy makes it easier for you to rapidly release new features, helps you avoid downtime during application deployment, and handles the complexity of updating your applications, without many of the risks associated with error-prone manual deployments.  <b>AWS CodeDeploy Components</b>  Use the information in this guide to help you work with the following AWS CodeDeploy components: <ul> <li>  <b>Application</b>: A name that uniquely identifies the application you want to deploy. AWS CodeDeploy uses this name, which functions as a container, to ensure the correct combination of revision, deployment configuration, and deployment group are referenced during a deployment. </li> <li>  <b>Deployment group</b>: A set of individual instances, CodeDeploy Lambda deployment configuration settings, or an Amazon ECS service and network details. A Lambda deployment group specifies how to route traffic to a new version of a Lambda function. An Amazon ECS deployment group specifies the service created in Amazon ECS to deploy, a load balancer, and a listener to reroute production traffic to an updated containerized application. An EC2/On-premises deployment group contains individually tagged instances, Amazon EC2 instances in Amazon EC2 Auto Scaling groups, or both. All deployment groups can specify optional trigger, alarm, and rollback settings. </li> <li>  <b>Deployment configuration</b>: A set of deployment rules and deployment success and failure conditions used by AWS CodeDeploy during a deployment. </li> <li>  <b>Deployment</b>: The process and the components used when updating a Lambda function, a containerized application in an Amazon ECS service, or of installing content on one or more instances.  </li> <li>  <b>Application revisions</b>: For an AWS Lambda deployment, this is an AppSpec file that specifies the Lambda function to be updated and one or more functions to validate deployment lifecycle events. For an Amazon ECS deployment, this is an AppSpec file that specifies the Amazon ECS task definition, container, and port where production traffic is rerouted. For an EC2/On-premises deployment, this is an archive file that contains source content—source code, webpages, executable files, and deployment scripts—along with an AppSpec file. Revisions are stored in Amazon S3 buckets or GitHub repositories. For Amazon S3, a revision is uniquely identified by its Amazon S3 object key and its ETag, version, or both. For GitHub, a revision is uniquely identified by its commit ID. </li> </ul> This guide also contains information to help you get details about the instances in your deployments, to make on-premises instances available for AWS CodeDeploy deployments, to get details about a Lambda function deployment, and to get details about Amazon ECS service deployments.  <b>AWS CodeDeploy Information Resources</b>  <ul> <li>  <a href="https://docs.aws.amazon.com/codedeploy/latest/userguide">AWS CodeDeploy User Guide</a>  </li> <li>  <a href="https://docs.aws.amazon.com/codedeploy/latest/APIReference/">AWS CodeDeploy API Reference Guide</a>  </li> <li>  <a href="https://docs.aws.amazon.com/cli/latest/reference/deploy/index.html">AWS CLI Reference for AWS CodeDeploy</a>  </li> <li>  <a href="https://forums.aws.amazon.com/forum.jspa?forumID=179">AWS CodeDeploy Developer Forum</a>  </li> </ul>"""

    class DeploymentSuccessfulWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `get_deployment` for valid parameters"""
            pass


    class ListApplicationRevisionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_application_revisions` for valid parameters"""
            pass
    class ListApplicationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_applications` for valid parameters"""
            pass
    class ListDeploymentConfigsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_deployment_configs` for valid parameters"""
            pass
    class ListDeploymentGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_deployment_groups` for valid parameters"""
            pass
    class ListDeploymentInstancesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_deployment_instances` for valid parameters"""
            pass
    class ListDeploymentsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_deployments` for valid parameters"""
            pass
    class ListDeploymentTargetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_deployment_targets` for valid parameters"""
            pass
    class ListGitHubAccountTokenNamesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_git_hub_account_token_names` for valid parameters"""
            pass
    class ListOnPremisesInstancesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_on_premises_instances` for valid parameters"""
            pass


    class AddTagsToOnPremisesInstancesInput(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class InstanceName(object):
        pass
    
    class BatchGetApplicationRevisionsInput(Mapping):
        pass
    
    class BatchGetApplicationRevisionsOutput(Mapping):
        pass
    
    class RevisionLocation(Mapping):
        pass
    
    class BatchGetApplicationsInput(Mapping):
        pass
    
    class BatchGetApplicationsOutput(Mapping):
        pass
    
    class ApplicationName(object):
        pass
    
    class BatchGetDeploymentGroupsInput(Mapping):
        pass
    
    class BatchGetDeploymentGroupsOutput(Mapping):
        pass
    
    class DeploymentGroupName(object):
        pass
    
    class BatchGetDeploymentInstancesInput(Mapping):
        pass
    
    class BatchGetDeploymentInstancesOutput(Mapping):
        pass
    
    class InstanceId(object):
        pass
    
    class BatchGetDeploymentTargetsInput(Mapping):
        pass
    
    class BatchGetDeploymentTargetsOutput(Mapping):
        pass
    
    class TargetId(object):
        pass
    
    class BatchGetDeploymentsInput(Mapping):
        pass
    
    class BatchGetDeploymentsOutput(Mapping):
        pass
    
    class DeploymentId(object):
        pass
    
    class BatchGetOnPremisesInstancesInput(Mapping):
        pass
    
    class BatchGetOnPremisesInstancesOutput(Mapping):
        pass
    
    class ContinueDeploymentInput(Mapping):
        pass
    
    class CreateApplicationInput(Mapping):
        pass
    
    class CreateApplicationOutput(Mapping):
        pass
    
    class CreateDeploymentInput(Mapping):
        pass
    
    class CreateDeploymentOutput(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class TargetInstances(Mapping):
        pass
    
    class AutoRollbackConfiguration(Mapping):
        pass
    
    class CreateDeploymentConfigInput(Mapping):
        pass
    
    class CreateDeploymentConfigOutput(Mapping):
        pass
    
    class MinimumHealthyHosts(Mapping):
        pass
    
    class TrafficRoutingConfig(Mapping):
        pass
    
    class CreateDeploymentGroupInput(Mapping):
        pass
    
    class CreateDeploymentGroupOutput(Mapping):
        pass
    
    class EC2TagFilter(Mapping):
        pass
    
    class TagFilter(Mapping):
        pass
    
    class AutoScalingGroupName(object):
        pass
    
    class TriggerConfig(Mapping):
        pass
    
    class AlarmConfiguration(Mapping):
        pass
    
    class DeploymentStyle(Mapping):
        pass
    
    class BlueGreenDeploymentConfiguration(Mapping):
        pass
    
    class LoadBalancerInfo(Mapping):
        pass
    
    class EC2TagSet(Mapping):
        pass
    
    class ECSService(Mapping):
        pass
    
    class OnPremisesTagSet(Mapping):
        pass
    
    class DeleteApplicationInput(Mapping):
        pass
    
    class DeleteDeploymentConfigInput(Mapping):
        pass
    
    class DeleteDeploymentGroupInput(Mapping):
        pass
    
    class DeleteDeploymentGroupOutput(Mapping):
        pass
    
    class DeleteGitHubAccountTokenInput(Mapping):
        pass
    
    class DeleteGitHubAccountTokenOutput(Mapping):
        pass
    
    class DeregisterOnPremisesInstanceInput(Mapping):
        pass
    
    class GetApplicationInput(Mapping):
        pass
    
    class GetApplicationOutput(Mapping):
        pass
    
    class GetApplicationRevisionInput(Mapping):
        pass
    
    class GetApplicationRevisionOutput(Mapping):
        pass
    
    class GetDeploymentInput(Mapping):
        pass
    
    class GetDeploymentOutput(Mapping):
        pass
    
    class GetDeploymentConfigInput(Mapping):
        pass
    
    class GetDeploymentConfigOutput(Mapping):
        pass
    
    class GetDeploymentGroupInput(Mapping):
        pass
    
    class GetDeploymentGroupOutput(Mapping):
        pass
    
    class GetDeploymentInstanceInput(Mapping):
        pass
    
    class GetDeploymentInstanceOutput(Mapping):
        pass
    
    class GetDeploymentTargetInput(Mapping):
        pass
    
    class GetDeploymentTargetOutput(Mapping):
        pass
    
    class GetOnPremisesInstanceInput(Mapping):
        pass
    
    class GetOnPremisesInstanceOutput(Mapping):
        pass
    
    class ListApplicationRevisionsInput(Mapping):
        pass
    
    class ListApplicationRevisionsOutput(Mapping):
        pass
    
    class ListApplicationsInput(Mapping):
        pass
    
    class ListApplicationsOutput(Mapping):
        pass
    
    class ListDeploymentConfigsInput(Mapping):
        pass
    
    class ListDeploymentConfigsOutput(Mapping):
        pass
    
    class ListDeploymentGroupsInput(Mapping):
        pass
    
    class ListDeploymentGroupsOutput(Mapping):
        pass
    
    class ListDeploymentInstancesInput(Mapping):
        pass
    
    class ListDeploymentInstancesOutput(Mapping):
        pass
    
    class InstanceStatus(object):
        pass
    
    class InstanceType(object):
        pass
    
    class ListDeploymentTargetsInput(Mapping):
        pass
    
    class ListDeploymentTargetsOutput(Mapping):
        pass
    
    class TargetFilters(object):
        pass
    
    class ListDeploymentsInput(Mapping):
        pass
    
    class ListDeploymentsOutput(Mapping):
        pass
    
    class DeploymentStatus(object):
        pass
    
    class TimeRange(Mapping):
        pass
    
    class ListGitHubAccountTokenNamesInput(Mapping):
        pass
    
    class ListGitHubAccountTokenNamesOutput(Mapping):
        pass
    
    class ListOnPremisesInstancesInput(Mapping):
        pass
    
    class ListOnPremisesInstancesOutput(Mapping):
        pass
    
    class ListTagsForResourceInput(Mapping):
        pass
    
    class ListTagsForResourceOutput(Mapping):
        pass
    
    class PutLifecycleEventHookExecutionStatusInput(Mapping):
        pass
    
    class PutLifecycleEventHookExecutionStatusOutput(Mapping):
        pass
    
    class RegisterApplicationRevisionInput(Mapping):
        pass
    
    class RegisterOnPremisesInstanceInput(Mapping):
        pass
    
    class RemoveTagsFromOnPremisesInstancesInput(Mapping):
        pass
    
    class SkipWaitTimeForInstanceTerminationInput(Mapping):
        pass
    
    class StopDeploymentInput(Mapping):
        pass
    
    class StopDeploymentOutput(Mapping):
        pass
    
    class NullableBoolean(object):
        pass
    
    class TagResourceInput(Mapping):
        pass
    
    class TagResourceOutput(Mapping):
        pass
    
    class UntagResourceInput(Mapping):
        pass
    
    class UntagResourceOutput(Mapping):
        pass
    
    class Key(object):
        pass
    
    class UpdateApplicationInput(Mapping):
        pass
    
    class UpdateDeploymentGroupInput(Mapping):
        pass
    
    class UpdateDeploymentGroupOutput(Mapping):
        pass
    
    def add_tags_to_on_premises_instances(self, instanceNames: List[InstanceName], tags: List[Tag]):
        """Adds tags to on-premises instances.

        :param self:
        :param instanceNames:  The names of the on-premises instances to which to add tags.<br/>tags:  The tag key-value pairs to add to the on-premises instances. Keys and values are both required. Keys cannot be null or empty strings. Value-only tags are not allowed.<br/>
        :return:         """
        pass

    def batch_get_application_revisions(self, revisions: List[RevisionLocation], applicationName: str) -> BatchGetApplicationRevisionsOutput:
        """Gets information about one or more application revisions. The maximum number of application revisions that can be returned is 25.

        :param self:
        :param revisions:  An array of <code>RevisionLocation</code> objects that specify information to get about the application revisions, including type and location. The maximum number of <code>RevisionLocation</code> objects you can specify is 25.<br/>applicationName:  The name of an AWS CodeDeploy application about which to get revision information.<br/>
        :return: revisions:  Additional information about the revisions, including the type and location.<br/>errorMessage:  Information about errors that might have occurred during the API call.<br/>applicationName:  The name of the application that corresponds to the revisions.<br/>        """
        pass

    def batch_get_applications(self, applicationNames: List[ApplicationName]) -> BatchGetApplicationsOutput:
        """Gets information about one or more applications. The maximum number of applications that can be returned is 25.

        :param self:
        :param applicationNames:  A list of application names separated by spaces. The maximum number of application names you can specify is 25.<br/>
        :return: applicationsInfo:  Information about the applications.<br/>        """
        pass

    def batch_get_deployment_groups(self, deploymentGroupNames: List[DeploymentGroupName], applicationName: str) -> BatchGetDeploymentGroupsOutput:
        """Gets information about one or more deployment groups.

        :param self:
        :param deploymentGroupNames:  The names of the deployment groups.<br/>applicationName:  The name of an AWS CodeDeploy application associated with the applicable IAM user or AWS account.<br/>
        :return: errorMessage:  Information about errors that might have occurred during the API call.<br/>deploymentGroupsInfo:  Information about the deployment groups.<br/>        """
        pass

    def batch_get_deployment_instances(self, instanceIds: List[InstanceId], deploymentId: str) -> BatchGetDeploymentInstancesOutput:
        """<note>  This method works, but is deprecated. Use <code>BatchGetDeploymentTargets</code> instead.  </note>  Returns an array of one or more instances associated with a deployment. This method works with EC2/On-premises and AWS Lambda compute platforms. The newer <code>BatchGetDeploymentTargets</code> works with all compute platforms. The maximum number of instances that can be returned is 25.

        :param self:
        :param instanceIds:  The unique IDs of instances used in the deployment. The maximum number of instance IDs you can specify is 25.<br/>deploymentId:   The unique ID of a deployment. <br/>
        :return: errorMessage:  Information about errors that might have occurred during the API call.<br/>instancesSummary:  Information about the instance.<br/>        """
        pass

    def batch_get_deployment_targets(self, deploymentId: str=None, targetIds: List[TargetId]=None) -> BatchGetDeploymentTargetsOutput:
        """ Returns an array of one or more targets associated with a deployment. This method works with all compute types and should be used instead of the deprecated <code>BatchGetDeploymentInstances</code>. The maximum number of targets that can be returned is 25.  The type of targets returned depends on the deployment's compute platform:  <ul> <li>  <b>EC2/On-premises</b>: Information about EC2 instance targets.  </li> <li>  <b>AWS Lambda</b>: Information about Lambda functions targets.  </li> <li>  <b>Amazon ECS</b>: Information about Amazon ECS service targets.  </li> </ul>

        :param self:
        :param targetIds:   The unique IDs of the deployment targets. The compute platform of the deployment determines the type of the targets and their formats. The maximum number of deployment target IDs you can specify is 25. <ul> <li>  For deployments that use the EC2/On-premises compute platform, the target IDs are EC2 or on-premises instances IDs, and their target type is <code>instanceTarget</code>.  </li> <li>  For deployments that use the AWS Lambda compute platform, the target IDs are the names of Lambda functions, and their target type is <code>instanceTarget</code>.  </li> <li>  For deployments that use the Amazon ECS compute platform, the target IDs are pairs of Amazon ECS clusters and services specified using the format <code>&lt;clustername&gt;:&lt;servicename&gt;</code>. Their target type is <code>ecsTarget</code>.  </li> </ul><br/>deploymentId:   The unique ID of a deployment. <br/>
        :return: deploymentTargets:   A list of target objects for a deployment. Each target object contains details about the target, such as its status and lifecycle events. The type of the target objects depends on the deployment' compute platform.  <ul> <li>  <b>EC2/On-premises</b>: Each target object is an EC2 or on-premises instance.  </li> <li>  <b>AWS Lambda</b>: The target object is a specific version of an AWS Lambda function.  </li> <li>  <b>Amazon ECS</b>: The target object is an Amazon ECS service.  </li> </ul><br/>        """
        pass

    def batch_get_deployments(self, deploymentIds: List[DeploymentId]) -> BatchGetDeploymentsOutput:
        """Gets information about one or more deployments. The maximum number of deployments that can be returned is 25.

        :param self:
        :param deploymentIds:   A list of deployment IDs, separated by spaces. The maximum number of deployment IDs you can specify is 25.<br/>
        :return: deploymentsInfo:   Information about the deployments. <br/>        """
        pass

    def batch_get_on_premises_instances(self, instanceNames: List[InstanceName]) -> BatchGetOnPremisesInstancesOutput:
        """Gets information about one or more on-premises instances. The maximum number of on-premises instances that can be returned is 25.

        :param self:
        :param instanceNames:  The names of the on-premises instances about which to get information. The maximum number of instance names you can specify is 25.<br/>
        :return: instanceInfos:  Information about the on-premises instances.<br/>        """
        pass

    def continue_deployment(self, deploymentId: str=None, deploymentWaitType: str=None):
        """For a blue/green deployment, starts the process of rerouting traffic from instances in the original environment to instances in the replacement environment without waiting for a specified wait time to elapse. (Traffic rerouting, which is achieved by registering instances in the replacement environment with the load balancer, can start as soon as all instances have a status of Ready.) 

        :param self:
        :param deploymentWaitType:   The status of the deployment's waiting period. READY_WAIT indicates the deployment is ready to start shifting traffic. TERMINATION_WAIT indicates the traffic is shifted, but the original target is not terminated. <br/>deploymentId:   The unique ID of a blue/green deployment for which you want to start rerouting traffic to the replacement environment. <br/>
        :return:         """
        pass

    def create_application(self, applicationName: str, computePlatform: str=None, tags: List[Tag]=None) -> CreateApplicationOutput:
        """Creates an application.

        :param self:
        :param tags:   The metadata that you apply to CodeDeploy applications to help you organize and categorize them. Each tag consists of a key and an optional value, both of which you define. <br/>computePlatform:   The destination platform type for the deployment (<code>Lambda</code>, <code>Server</code>, or <code>ECS</code>).<br/>applicationName:  The name of the application. This name must be unique with the applicable IAM user or AWS account.<br/>
        :return: applicationId:  A unique application ID.<br/>        """
        pass

    def create_deployment(self, applicationName: str, deploymentGroupName: str=None, revision: RevisionLocation=None, deploymentConfigName: str=None, description: str=None, ignoreApplicationStopFailures: Boolean=None, targetInstances: TargetInstances=None, autoRollbackConfiguration: AutoRollbackConfiguration=None, updateOutdatedInstancesOnly: Boolean=None, fileExistsBehavior: str=None) -> CreateDeploymentOutput:
        """Deploys an application revision through the specified deployment group.

        :param self:
        :param fileExistsBehavior:  Information about how AWS CodeDeploy handles files that already exist in a deployment target location but weren't part of the previous successful deployment. The fileExistsBehavior parameter takes any of the following values: <ul> <li> DISALLOW: The deployment fails. This is also the default behavior if no option is specified. </li> <li> OVERWRITE: The version of the file from the application revision currently being deployed replaces the version already on the instance. </li> <li> RETAIN: The version of the file already on the instance is kept and used as part of the new deployment. </li> </ul><br/>updateOutdatedInstancesOnly:   Indicates whether to deploy to all instances or only to instances that are not running the latest application revision. <br/>autoRollbackConfiguration:  Configuration information for an automatic rollback that is added when a deployment is created.:<br/>targetInstances:   Information about the instances that belong to the replacement environment in a blue/green deployment. :<br/>ignoreApplicationStopFailures:   If true, then if an ApplicationStop, BeforeBlockTraffic, or AfterBlockTraffic deployment lifecycle event to an instance fails, then the deployment continues to the next deployment lifecycle event. For example, if ApplicationStop fails, the deployment continues with DownloadBundle. If BeforeBlockTraffic fails, the deployment continues with BlockTraffic. If AfterBlockTraffic fails, the deployment continues with ApplicationStop.   If false or not specified, then if a lifecycle event fails during a deployment to an instance, that deployment fails. If deployment to that instance is part of an overall deployment and the number of healthy hosts is not less than the minimum number of healthy hosts, then a deployment to the next instance is attempted.   During a deployment, the AWS CodeDeploy agent runs the scripts specified for ApplicationStop, BeforeBlockTraffic, and AfterBlockTraffic in the AppSpec file from the previous successful deployment. (All other scripts are run from the AppSpec file in the current deployment.) If one of these scripts contains an error and does not run successfully, the deployment can fail.   If the cause of the failure is a script from the last successful deployment that will never run successfully, create a new deployment and use <code>ignoreApplicationStopFailures</code> to specify that the ApplicationStop, BeforeBlockTraffic, and AfterBlockTraffic failures should be ignored. <br/>description:  A comment about the deployment.<br/>deploymentConfigName:  The name of a deployment configuration associated with the IAM user or AWS account. If not specified, the value configured in the deployment group is used as the default. If the deployment group does not have a deployment configuration associated with it, CodeDeployDefault.OneAtATime is used by default.<br/>revision:   The type and location of the revision to deploy. :<br/>deploymentGroupName:  The name of the deployment group.<br/>applicationName:  The name of an AWS CodeDeploy application associated with the IAM user or AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>appSpecContent</i>  The content of an AppSpec file for an AWS Lambda or Amazon ECS deployment. The content is formatted as JSON or YAML and stored as a RawString. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>appSpecContent</b>:  The content of an AppSpec file for an AWS Lambda or Amazon ECS deployment. The content is formatted as JSON or YAML and stored as a RawString. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>string</i> Information about the location of an AWS Lambda deployment revision stored as a RawString.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>string</b>: Information about the location of an AWS Lambda deployment revision stored as a RawString.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>gitHubLocation</i> Information about the location of application artifacts stored in GitHub.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>gitHubLocation</b>: Information about the location of application artifacts stored in GitHub.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>s3Location</i> Information about the location of a revision stored in Amazon S3. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>s3Location</b>: Information about the location of a revision stored in Amazon S3. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>revisionType</i> The type of application revision: <ul> <li> S3: An application revision stored in Amazon S3. </li> <li> GitHub: An application revision stored in GitHub (EC2/On-premises deployments only). </li> <li> String: A YAML-formatted or JSON-formatted string (AWS Lambda deployments only). </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>eTag</i> The ETag of the Amazon S3 object that represents the bundled artifacts for the application revision. If the ETag is not specified as an input parameter, ETag validation of the object is skipped.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> A specific version of the Amazon S3 object that represents the bundled artifacts for the application revision. If the version is not specified, the system uses the most recent version by default.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>bundleType</i> The file type of the application revision. Must be one of the following: <ul> <li> tar: A tar archive file. </li> <li> tgz: A compressed tar archive file. </li> <li> zip: A zip archive file. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>key</i> The name of the Amazon S3 object that represents the bundled artifacts for the application revision.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>bucket</i> The name of the Amazon S3 bucket where the application revision is stored.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>commitId</i> The SHA1 commit ID of the GitHub commit that represents the bundled artifacts for the application revision.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>repository</i> The GitHub account and repository pair that stores a reference to the commit that represents the bundled artifacts for the application revision.  Specified as account/repository.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>sha256</i> The SHA256 hash value of the revision content.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>content</i> The YAML-formatted or JSON-formatted revision string. It includes information about which Lambda function to update and optional Lambda functions that validate deployment lifecycle events.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>sha256</i>  The SHA256 hash value of the revision content. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>content</i>  The YAML-formatted or JSON-formatted revision string.   For an AWS Lambda deployment, the content includes a Lambda function name, the alias for its original version, and the alias for its replacement version. The deployment shifts traffic from the original version of the Lambda function to the replacement version.   For an Amazon ECS deployment, the content includes the task name, information about the load balancer that serves traffic to the container, and more.   For both types of deployments, the content can specify Lambda functions that run at specified hooks, such as <code>BeforeInstall</code>, during a deployment. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ec2TagSet</i> Information about the groups of EC2 instance tags that an instance must be identified by in order for it to be included in the replacement environment for a blue/green deployment. Cannot be used in the same call as tagFilters.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ec2TagSet</b>: Information about the groups of EC2 instance tags that an instance must be identified by in order for it to be included in the replacement environment for a blue/green deployment. Cannot be used in the same call as tagFilters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>autoScalingGroups</i> The names of one or more Auto Scaling groups to identify a replacement environment for a blue/green deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tagFilters</i> The tag filter key, type, and value used to identify Amazon EC2 instances in a replacement environment for a blue/green deployment. Cannot be used in the same call as ec2TagSet.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ec2TagSetList</i> A list that contains other lists of EC2 instance tag groups. For an instance to be included in the deployment group, it must be identified by all of the tag groups in the list.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>events</i> The event type or types that trigger a rollback.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enabled</i> Indicates whether a defined automatic rollback configuration is currently enabled.<br/>
        :return: deploymentId:   The unique ID of a deployment. <br/>        """
        pass

    def create_deployment_config(self, deploymentConfigName: str, minimumHealthyHosts: MinimumHealthyHosts=None, trafficRoutingConfig: TrafficRoutingConfig=None, computePlatform: str=None) -> CreateDeploymentConfigOutput:
        """ Creates a deployment configuration. 

        :param self:
        :param computePlatform:  The destination platform type for the deployment (<code>Lambda</code>, <code>Server</code>, or <code>ECS</code>).<br/>trafficRoutingConfig:  The configuration that specifies how the deployment traffic is routed.:<br/>minimumHealthyHosts:  The minimum number of healthy instances that should be available at any time during the deployment. There are two parameters expected in the input: type and value. The type parameter takes either of the following values: <ul> <li> HOST_COUNT: The value parameter represents the minimum number of healthy instances as an absolute value. </li> <li> FLEET_PERCENT: The value parameter represents the minimum number of healthy instances as a percentage of the total number of instances in the deployment. If you specify FLEET_PERCENT, at the start of the deployment, AWS CodeDeploy converts the percentage to the equivalent number of instance and rounds up fractional instances. </li> </ul> The value parameter takes an integer. For example, to set a minimum of 95% healthy instance, specify a type of FLEET_PERCENT and a value of 95.:<br/>deploymentConfigName:  The name of the deployment configuration to create.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The minimum healthy instance type: <ul> <li> HOST_COUNT: The minimum number of healthy instance as an absolute value. </li> <li> FLEET_PERCENT: The minimum number of healthy instance as a percentage of the total number of instance in the deployment. </li> </ul> In an example of nine instance, if a HOST_COUNT of six is specified, deploy to up to three instances at a time. The deployment is successful if six or more instances are deployed to successfully. Otherwise, the deployment fails. If a FLEET_PERCENT of 40 is specified, deploy to up to five instance at a time. The deployment is successful if four or more instance are deployed to successfully. Otherwise, the deployment fails. <note> In a call to the <code>GetDeploymentConfig</code>, CodeDeployDefault.OneAtATime returns a minimum healthy instance type of MOST_CONCURRENCY and a value of 1. This means a deployment to only one instance at a time. (You cannot set the type to MOST_CONCURRENCY, only to HOST_COUNT or FLEET_PERCENT.) In addition, with CodeDeployDefault.OneAtATime, AWS CodeDeploy attempts to ensure that all instances but one are kept in a healthy state during the deployment. Although this allows one instance at a time to be taken offline for a new deployment, it also means that if the deployment to the last instance fails, the overall deployment is still successful. </note> For more information, see <a href="https://docs.aws.amazon.com/codedeploy/latest/userguide/instances-health.html">AWS CodeDeploy Instance Health</a> in the <i>AWS CodeDeploy User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>value</i> The minimum healthy instance value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>timeBasedLinear</i> A configuration that shifts traffic from one version of a Lambda function to another in equal increments, with an equal number of minutes between each increment. The original and target Lambda function versions are specified in the deployment's AppSpec file.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>timeBasedLinear</b>: A configuration that shifts traffic from one version of a Lambda function to another in equal increments, with an equal number of minutes between each increment. The original and target Lambda function versions are specified in the deployment's AppSpec file.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>timeBasedCanary</i> A configuration that shifts traffic from one version of a Lambda function to another in two increments. The original and target Lambda function versions are specified in the deployment's AppSpec file.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>timeBasedCanary</b>: A configuration that shifts traffic from one version of a Lambda function to another in two increments. The original and target Lambda function versions are specified in the deployment's AppSpec file.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of traffic shifting (<code>TimeBasedCanary</code> or <code>TimeBasedLinear</code>) used by a deployment configuration .<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>canaryInterval</i> The number of minutes between the first and second traffic shifts of a <code>TimeBasedCanary</code> deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>canaryPercentage</i> The percentage of traffic to shift in the first increment of a <code>TimeBasedCanary</code> deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>linearInterval</i> The number of minutes between each incremental traffic shift of a <code>TimeBasedLinear</code> deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>linearPercentage</i> The percentage of traffic that is shifted at the start of each increment of a <code>TimeBasedLinear</code> deployment.<br/>
        :return: deploymentConfigId:  A unique deployment configuration ID.<br/>        """
        pass

    def create_deployment_group(self, serviceRoleArn: str, deploymentGroupName: str, applicationName: str, deploymentConfigName: str=None, ec2TagFilters: List[EC2TagFilter]=None, onPremisesInstanceTagFilters: List[TagFilter]=None, autoScalingGroups: List[AutoScalingGroupName]=None, triggerConfigurations: List[TriggerConfig]=None, alarmConfiguration: AlarmConfiguration=None, autoRollbackConfiguration: AutoRollbackConfiguration=None, deploymentStyle: DeploymentStyle=None, blueGreenDeploymentConfiguration: BlueGreenDeploymentConfiguration=None, loadBalancerInfo: LoadBalancerInfo=None, ec2TagSet: EC2TagSet=None, ecsServices: List[ECSService]=None, onPremisesTagSet: OnPremisesTagSet=None, tags: List[Tag]=None) -> CreateDeploymentGroupOutput:
        """Creates a deployment group to which application revisions are deployed.

        :param self:
        :param tags:   The metadata that you apply to CodeDeploy deployment groups to help you organize and categorize them. Each tag consists of a key and an optional value, both of which you define. <br/>onPremisesTagSet:  Information about groups of tags applied to on-premises instances. The deployment group includes only on-premises instances identified by all of the tag groups. Cannot be used in the same call as onPremisesInstanceTagFilters.:<br/>ecsServices:   The target Amazon ECS services in the deployment group. This applies only to deployment groups that use the Amazon ECS compute platform. A target Amazon ECS service is specified as an Amazon ECS cluster and service name pair using the format <code>&lt;clustername&gt;:&lt;servicename&gt;</code>. <br/>ec2TagSet:  Information about groups of tags applied to EC2 instances. The deployment group includes only EC2 instances identified by all the tag groups. Cannot be used in the same call as ec2TagFilters.:<br/>loadBalancerInfo:  Information about the load balancer used in a deployment.:<br/>blueGreenDeploymentConfiguration:  Information about blue/green deployment options for a deployment group.:<br/>deploymentStyle:  Information about the type of deployment, in-place or blue/green, that you want to run and whether to route deployment traffic behind a load balancer.:<br/>autoRollbackConfiguration:  Configuration information for an automatic rollback that is added when a deployment group is created.:<br/>alarmConfiguration:  Information to add about Amazon CloudWatch alarms when the deployment group is created.:<br/>triggerConfigurations:  Information about triggers to create when the deployment group is created. For examples, see <a href="https://docs.aws.amazon.com/codedeploy/latest/userguide/how-to-notify-sns.html">Create a Trigger for an AWS CodeDeploy Event</a> in the AWS CodeDeploy User Guide.<br/>serviceRoleArn:  A service role ARN that allows AWS CodeDeploy to act on the user's behalf when interacting with AWS services.<br/>autoScalingGroups:  A list of associated Amazon EC2 Auto Scaling groups.<br/>onPremisesInstanceTagFilters:  The on-premises instance tags on which to filter. The deployment group includes on-premises instances with any of the specified tags. Cannot be used in the same call as OnPremisesTagSet.<br/>ec2TagFilters:  The Amazon EC2 tags on which to filter. The deployment group includes EC2 instances with any of the specified tags. Cannot be used in the same call as ec2TagSet.<br/>deploymentConfigName:  If specified, the deployment configuration name can be either one of the predefined configurations provided with AWS CodeDeploy or a custom deployment configuration that you create by calling the create deployment configuration operation. CodeDeployDefault.OneAtATime is the default deployment configuration. It is used if a configuration isn't specified for the deployment or deployment group. For more information about the predefined deployment configurations in AWS CodeDeploy, see <a href="https://docs.aws.amazon.com/codedeploy/latest/userguide/deployment-configurations.html">Working with Deployment Groups in AWS CodeDeploy</a> in the AWS CodeDeploy User Guide.<br/>deploymentGroupName:  The name of a new deployment group for the specified application.<br/>applicationName:  The name of an AWS CodeDeploy application associated with the IAM user or AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>alarms</i> A list of alarms configured for the deployment group. A maximum of 10 alarms can be added to a deployment group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ignorePollAlarmFailure</i> Indicates whether a deployment should continue if information about the current state of alarms cannot be retrieved from Amazon CloudWatch. The default value is false. <ul> <li> true: The deployment proceeds even if alarm status information can't be retrieved from Amazon CloudWatch. </li> <li> false: The deployment stops if alarm status information can't be retrieved from Amazon CloudWatch. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enabled</i> Indicates whether the alarm configuration is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>events</i> The event type or types that trigger a rollback.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enabled</i> Indicates whether a defined automatic rollback configuration is currently enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentOption</i> Indicates whether to route deployment traffic behind a load balancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentType</i> Indicates whether to run an in-place deployment or a blue/green deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>greenFleetProvisioningOption</i> Information about how instances are provisioned for a replacement environment in a blue/green deployment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>greenFleetProvisioningOption</b>: Information about how instances are provisioned for a replacement environment in a blue/green deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentReadyOption</i> Information about the action to take when newly provisioned instances are ready to receive traffic in a blue/green deployment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>deploymentReadyOption</b>: Information about the action to take when newly provisioned instances are ready to receive traffic in a blue/green deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>terminateBlueInstancesOnDeploymentSuccess</i> Information about whether to terminate instances in the original fleet during a blue/green deployment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>terminateBlueInstancesOnDeploymentSuccess</b>: Information about whether to terminate instances in the original fleet during a blue/green deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>terminationWaitTimeInMinutes</i> For an Amazon EC2 deployment, the number of minutes to wait after a successful blue/green deployment before terminating instances from the original environment.  For an Amazon ECS deployment, the number of minutes before deleting the original (blue) task set. During an Amazon ECS deployment, CodeDeploy shifts traffic from the original (blue) task set to a replacement (green) task set.   The maximum setting is 2880 minutes (2 days). <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>action</i> The action to take on instances in the original environment after a successful blue/green deployment. <ul> <li> TERMINATE: Instances are terminated after a specified wait time. </li> <li> KEEP_ALIVE: Instances are left running after they are deregistered from the load balancer and removed from the deployment group. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>waitTimeInMinutes</i> The number of minutes to wait before the status of a blue/green deployment is changed to Stopped if rerouting is not started manually. Applies only to the STOP_DEPLOYMENT option for actionOnTimeout<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>actionOnTimeout</i> Information about when to reroute traffic from an original environment to a replacement environment in a blue/green deployment. <ul> <li> CONTINUE_DEPLOYMENT: Register new instances with the load balancer immediately after the new application revision is installed on the instances in the replacement environment. </li> <li> STOP_DEPLOYMENT: Do not register new instances with a load balancer unless traffic rerouting is started using <a>ContinueDeployment</a>. If traffic rerouting is not started before the end of the specified wait period, the deployment status is changed to Stopped. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>action</i> The method used to add instances to a replacement environment. <ul> <li> DISCOVER_EXISTING: Use instances that already exist or will be created manually. </li> <li> COPY_AUTO_SCALING_GROUP: Use settings from a specified Auto Scaling group to define and create instances in a new Auto Scaling group. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>targetGroupPairInfoList</i>  The target group pair information. This is an array of <code>TargeGroupPairInfo</code> objects with a maximum size of one. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>targetGroupInfoList</i> An array that contains information about the target group to use for load balancing in a deployment. In Elastic Load Balancing, target groups are used with Application Load Balancers. <note>  Adding more than one target group to the array is not supported.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>elbInfoList</i> An array that contains information about the load balancer to use for load balancing in a deployment. In Elastic Load Balancing, load balancers are used with Classic Load Balancers. <note>  Adding more than one load balancer to the array is not supported.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ec2TagSetList</i> A list that contains other lists of EC2 instance tag groups. For an instance to be included in the deployment group, it must be identified by all of the tag groups in the list.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>onPremisesTagSetList</i> A list that contains other lists of on-premises instance tag groups. For an instance to be included in the deployment group, it must be identified by all of the tag groups in the list.<br/>
        :return: deploymentGroupId:  A unique deployment group ID.<br/>        """
        pass

    def delete_application(self, applicationName: str):
        """Deletes an application.

        :param self:
        :param applicationName:  The name of an AWS CodeDeploy application associated with the IAM user or AWS account.<br/>
        :return:         """
        pass

    def delete_deployment_config(self, deploymentConfigName: str):
        """Deletes a deployment configuration. <note> A deployment configuration cannot be deleted if it is currently in use. Predefined configurations cannot be deleted. </note>

        :param self:
        :param deploymentConfigName:  The name of a deployment configuration associated with the IAM user or AWS account.<br/>
        :return:         """
        pass

    def delete_deployment_group(self, deploymentGroupName: str, applicationName: str) -> DeleteDeploymentGroupOutput:
        """Deletes a deployment group.

        :param self:
        :param deploymentGroupName:  The name of a deployment group for the specified application.<br/>applicationName:  The name of an AWS CodeDeploy application associated with the IAM user or AWS account.<br/>
        :return: hooksNotCleanedUp:  If the output contains no data, and the corresponding deployment group contained at least one Auto Scaling group, AWS CodeDeploy successfully removed all corresponding Auto Scaling lifecycle event hooks from the Amazon EC2 instances in the Auto Scaling group. If the output contains data, AWS CodeDeploy could not remove some Auto Scaling lifecycle event hooks from the Amazon EC2 instances in the Auto Scaling group.<br/>        """
        pass

    def delete_git_hub_account_token(self, tokenName: str=None) -> DeleteGitHubAccountTokenOutput:
        """Deletes a GitHub account connection.

        :param self:
        :param tokenName:  The name of the GitHub account connection to delete.<br/>
        :return: tokenName:  The name of the GitHub account connection that was deleted.<br/>        """
        pass

    def deregister_on_premises_instance(self, instanceName: str):
        """Deregisters an on-premises instance.

        :param self:
        :param instanceName:  The name of the on-premises instance to deregister.<br/>
        :return:         """
        pass

    def get_application(self, applicationName: str) -> GetApplicationOutput:
        """Gets information about an application.

        :param self:
        :param applicationName:  The name of an AWS CodeDeploy application associated with the IAM user or AWS account.<br/>
        :return: application:  Information about the application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>computePlatform</i> The destination platform type for deployment of the application (<code>Lambda</code> or <code>Server</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>gitHubAccountName</i> The name for a connection to a GitHub account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>linkedToGitHub</i> True if the user has authenticated with GitHub for the specified application. Otherwise, false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createTime</i> The time at which the application was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>applicationName</i> The application name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>applicationId</i> The application ID.<br/>        """
        pass

    def get_application_revision(self, revision: RevisionLocation, applicationName: str) -> GetApplicationRevisionOutput:
        """Gets information about an application revision.

        :param self:
        :param revision:  Information about the application revision to get, including type and location.:<br/>applicationName:  The name of the application that corresponds to the revision.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>appSpecContent</i>  The content of an AppSpec file for an AWS Lambda or Amazon ECS deployment. The content is formatted as JSON or YAML and stored as a RawString. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>appSpecContent</b>:  The content of an AppSpec file for an AWS Lambda or Amazon ECS deployment. The content is formatted as JSON or YAML and stored as a RawString. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>string</i> Information about the location of an AWS Lambda deployment revision stored as a RawString.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>string</b>: Information about the location of an AWS Lambda deployment revision stored as a RawString.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>gitHubLocation</i> Information about the location of application artifacts stored in GitHub.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>gitHubLocation</b>: Information about the location of application artifacts stored in GitHub.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>s3Location</i> Information about the location of a revision stored in Amazon S3. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>s3Location</b>: Information about the location of a revision stored in Amazon S3. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>revisionType</i> The type of application revision: <ul> <li> S3: An application revision stored in Amazon S3. </li> <li> GitHub: An application revision stored in GitHub (EC2/On-premises deployments only). </li> <li> String: A YAML-formatted or JSON-formatted string (AWS Lambda deployments only). </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>eTag</i> The ETag of the Amazon S3 object that represents the bundled artifacts for the application revision. If the ETag is not specified as an input parameter, ETag validation of the object is skipped.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> A specific version of the Amazon S3 object that represents the bundled artifacts for the application revision. If the version is not specified, the system uses the most recent version by default.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>bundleType</i> The file type of the application revision. Must be one of the following: <ul> <li> tar: A tar archive file. </li> <li> tgz: A compressed tar archive file. </li> <li> zip: A zip archive file. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>key</i> The name of the Amazon S3 object that represents the bundled artifacts for the application revision.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>bucket</i> The name of the Amazon S3 bucket where the application revision is stored.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>commitId</i> The SHA1 commit ID of the GitHub commit that represents the bundled artifacts for the application revision.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>repository</i> The GitHub account and repository pair that stores a reference to the commit that represents the bundled artifacts for the application revision.  Specified as account/repository.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>sha256</i> The SHA256 hash value of the revision content.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>content</i> The YAML-formatted or JSON-formatted revision string. It includes information about which Lambda function to update and optional Lambda functions that validate deployment lifecycle events.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>sha256</i>  The SHA256 hash value of the revision content. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>content</i>  The YAML-formatted or JSON-formatted revision string.   For an AWS Lambda deployment, the content includes a Lambda function name, the alias for its original version, and the alias for its replacement version. The deployment shifts traffic from the original version of the Lambda function to the replacement version.   For an Amazon ECS deployment, the content includes the task name, information about the load balancer that serves traffic to the container, and more.   For both types of deployments, the content can specify Lambda functions that run at specified hooks, such as <code>BeforeInstall</code>, during a deployment. <br/>
        :return: revisionInfo:  General information about the revision.:<br/>revision:  Additional information about the revision, including type and location.:<br/>applicationName:  The name of the application that corresponds to the revision.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>appSpecContent</i>  The content of an AppSpec file for an AWS Lambda or Amazon ECS deployment. The content is formatted as JSON or YAML and stored as a RawString. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>appSpecContent</b>:  The content of an AppSpec file for an AWS Lambda or Amazon ECS deployment. The content is formatted as JSON or YAML and stored as a RawString. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>string</i> Information about the location of an AWS Lambda deployment revision stored as a RawString.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>string</b>: Information about the location of an AWS Lambda deployment revision stored as a RawString.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>gitHubLocation</i> Information about the location of application artifacts stored in GitHub.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>gitHubLocation</b>: Information about the location of application artifacts stored in GitHub.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>s3Location</i> Information about the location of a revision stored in Amazon S3. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>s3Location</b>: Information about the location of a revision stored in Amazon S3. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>revisionType</i> The type of application revision: <ul> <li> S3: An application revision stored in Amazon S3. </li> <li> GitHub: An application revision stored in GitHub (EC2/On-premises deployments only). </li> <li> String: A YAML-formatted or JSON-formatted string (AWS Lambda deployments only). </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>eTag</i> The ETag of the Amazon S3 object that represents the bundled artifacts for the application revision. If the ETag is not specified as an input parameter, ETag validation of the object is skipped.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> A specific version of the Amazon S3 object that represents the bundled artifacts for the application revision. If the version is not specified, the system uses the most recent version by default.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>bundleType</i> The file type of the application revision. Must be one of the following: <ul> <li> tar: A tar archive file. </li> <li> tgz: A compressed tar archive file. </li> <li> zip: A zip archive file. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>key</i> The name of the Amazon S3 object that represents the bundled artifacts for the application revision.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>bucket</i> The name of the Amazon S3 bucket where the application revision is stored.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>commitId</i> The SHA1 commit ID of the GitHub commit that represents the bundled artifacts for the application revision.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>repository</i> The GitHub account and repository pair that stores a reference to the commit that represents the bundled artifacts for the application revision.  Specified as account/repository.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>sha256</i> The SHA256 hash value of the revision content.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>content</i> The YAML-formatted or JSON-formatted revision string. It includes information about which Lambda function to update and optional Lambda functions that validate deployment lifecycle events.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>sha256</i>  The SHA256 hash value of the revision content. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>content</i>  The YAML-formatted or JSON-formatted revision string.   For an AWS Lambda deployment, the content includes a Lambda function name, the alias for its original version, and the alias for its replacement version. The deployment shifts traffic from the original version of the Lambda function to the replacement version.   For an Amazon ECS deployment, the content includes the task name, information about the load balancer that serves traffic to the container, and more.   For both types of deployments, the content can specify Lambda functions that run at specified hooks, such as <code>BeforeInstall</code>, during a deployment. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>registerTime</i> When the revision was registered with AWS CodeDeploy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUsedTime</i> When the revision was last used by AWS CodeDeploy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>firstUsedTime</i> When the revision was first used by AWS CodeDeploy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentGroups</i> The deployment groups for which this is the current target revision.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> A comment about the revision.<br/>        """
        pass

    def get_deployment(self, deploymentId: str) -> GetDeploymentOutput:
        """Gets information about a deployment. <note>  The <code>content</code> property of the <code>appSpecContent</code> object in the returned revision is always null. Use <code>GetApplicationRevision</code> and the <code>sha256</code> property of the returned <code>appSpecContent</code> object to get the content of the deployment’s AppSpec file.  </note>

        :param self:
        :param deploymentId:   The unique ID of a deployment associated with the IAM user or AWS account. <br/>
        :return: deploymentInfo:  Information about the deployment.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>computePlatform</i> The destination platform type for the deployment (<code>Lambda</code>, <code>Server</code>, or <code>ECS</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentStatusMessages</i> Messages that contain information about the status of a deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fileExistsBehavior</i> Information about how AWS CodeDeploy handles files that already exist in a deployment target location but weren't part of the previous successful deployment. <ul> <li> DISALLOW: The deployment fails. This is also the default behavior if no option is specified. </li> <li> OVERWRITE: The version of the file from the application revision currently being deployed replaces the version already on the instance. </li> <li> RETAIN: The version of the file already on the instance is kept and used as part of the new deployment. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>additionalDeploymentStatusInfo</i> Provides information about the results of a deployment, such as whether instances in the original environment in a blue/green deployment were not terminated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>loadBalancerInfo</i> Information about the load balancer used in the deployment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>loadBalancerInfo</b>: Information about the load balancer used in the deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>blueGreenDeploymentConfiguration</i> Information about blue/green deployment options for this deployment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>blueGreenDeploymentConfiguration</b>: Information about blue/green deployment options for this deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>instanceTerminationWaitTimeStarted</i> Indicates whether the wait period set for the termination of instances in the original environment has started. Status is 'false' if the KEEP_ALIVE option is specified. Otherwise, 'true' as soon as the termination wait period starts.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>targetInstances</i> Information about the instances that belong to the replacement environment in a blue/green deployment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>targetInstances</b>: Information about the instances that belong to the replacement environment in a blue/green deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentStyle</i> Information about the type of deployment, either in-place or blue/green, you want to run and whether to route deployment traffic behind a load balancer.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>deploymentStyle</b>: Information about the type of deployment, either in-place or blue/green, you want to run and whether to route deployment traffic behind a load balancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>rollbackInfo</i> Information about a deployment rollback.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>rollbackInfo</b>: Information about a deployment rollback.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>updateOutdatedInstancesOnly</i> Indicates whether only instances that are not running the latest application revision are to be deployed to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>autoRollbackConfiguration</i> Information about the automatic rollback configuration associated with the deployment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>autoRollbackConfiguration</b>: Information about the automatic rollback configuration associated with the deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ignoreApplicationStopFailures</i>  If true, then if an <code>ApplicationStop</code>, <code>BeforeBlockTraffic</code>, or <code>AfterBlockTraffic</code> deployment lifecycle event to an instance fails, then the deployment continues to the next deployment lifecycle event. For example, if <code>ApplicationStop</code> fails, the deployment continues with DownloadBundle. If <code>BeforeBlockTraffic</code> fails, the deployment continues with <code>BlockTraffic</code>. If <code>AfterBlockTraffic</code> fails, the deployment continues with <code>ApplicationStop</code>.   If false or not specified, then if a lifecycle event fails during a deployment to an instance, that deployment fails. If deployment to that instance is part of an overall deployment and the number of healthy hosts is not less than the minimum number of healthy hosts, then a deployment to the next instance is attempted.   During a deployment, the AWS CodeDeploy agent runs the scripts specified for <code>ApplicationStop</code>, <code>BeforeBlockTraffic</code>, and <code>AfterBlockTraffic</code> in the AppSpec file from the previous successful deployment. (All other scripts are run from the AppSpec file in the current deployment.) If one of these scripts contains an error and does not run successfully, the deployment can fail.   If the cause of the failure is a script from the last successful deployment that will never run successfully, create a new deployment and use <code>ignoreApplicationStopFailures</code> to specify that the <code>ApplicationStop</code>, <code>BeforeBlockTraffic</code>, and <code>AfterBlockTraffic</code> failures should be ignored. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creator</i> The means by which the deployment was created: <ul> <li> user: A user created the deployment. </li> <li> autoscaling: Amazon EC2 Auto Scaling created the deployment. </li> <li> codeDeployRollback: A rollback process created the deployment. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> A comment about the deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentOverview</i> A summary of the deployment status of the instances in the deployment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>deploymentOverview</b>: A summary of the deployment status of the instances in the deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>completeTime</i> A timestamp that indicates when the deployment was complete.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>startTime</i> A timestamp that indicates when the deployment was deployed to the deployment group. In some cases, the reported value of the start time might be later than the complete time. This is due to differences in the clock settings of backend servers that participate in the deployment process.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createTime</i> A timestamp that indicates when the deployment was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorInformation</i> Information about any error associated with this deployment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>errorInformation</b>: Information about any error associated with this deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current state of the deployment as a whole.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>revision</i> Information about the location of stored application artifacts and the service from which to retrieve them.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>revision</b>: Information about the location of stored application artifacts and the service from which to retrieve them.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>previousRevision</i> Information about the application revision that was deployed to the deployment group before the most recent successful deployment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>previousRevision</b>: Information about the application revision that was deployed to the deployment group before the most recent successful deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentId</i>  The unique ID of a deployment. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentConfigName</i>  The deployment configuration name. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentGroupName</i>  The deployment group name. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>applicationName</i> The application name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>revisionType</i> The type of application revision: <ul> <li> S3: An application revision stored in Amazon S3. </li> <li> GitHub: An application revision stored in GitHub (EC2/On-premises deployments only). </li> <li> String: A YAML-formatted or JSON-formatted string (AWS Lambda deployments only). </li> </ul><br/>Information about the location of a revision stored in Amazon S3. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>s3Location</b>: Information about the location of a revision stored in Amazon S3. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>revisionType</i> The type of application revision: <ul> <li> S3: An application revision stored in Amazon S3. </li> <li> GitHub: An application revision stored in GitHub (EC2/On-premises deployments only). </li> <li> String: A YAML-formatted or JSON-formatted string (AWS Lambda deployments only). </li> </ul><br/>Information about the location of a revision stored in Amazon S3. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>s3Location</b>: Information about the location of a revision stored in Amazon S3. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>message</i> An accompanying error message.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>code</i> For more information, see <a href="https://docs.aws.amazon.com/codedeploy/latest/userguide/error-codes.html">Error Codes for AWS CodeDeploy</a> in the <a href="https://docs.aws.amazon.com/codedeploy/latest/userguide">AWS CodeDeploy User Guide</a>. The error code: <ul> <li> APPLICATION_MISSING: The application was missing. This error code is most likely raised if the application is deleted after the deployment is created, but before it is started. </li> <li> DEPLOYMENT_GROUP_MISSING: The deployment group was missing. This error code is most likely raised if the deployment group is deleted after the deployment is created, but before it is started. </li> <li> HEALTH_CONSTRAINTS: The deployment failed on too many instances to be successfully deployed within the instance health constraints specified. </li> <li> HEALTH_CONSTRAINTS_INVALID: The revision cannot be successfully deployed within the instance health constraints specified. </li> <li> IAM_ROLE_MISSING: The service role cannot be accessed. </li> <li> IAM_ROLE_PERMISSIONS: The service role does not have the correct permissions. </li> <li> INTERNAL_ERROR: There was an internal error. </li> <li> NO_EC2_SUBSCRIPTION: The calling account is not subscribed to Amazon EC2. </li> <li> NO_INSTANCES: No instances were specified, or no instances can be found. </li> <li> OVER_MAX_INSTANCES: The maximum number of instances was exceeded. </li> <li> THROTTLED: The operation was throttled because the calling account exceeded the throttling limits of one or more AWS services. </li> <li> TIMEOUT: The deployment has timed out. </li> <li> REVISION_MISSING: The revision ID was missing. This error code is most likely raised if the revision is deleted after the deployment is created, but before it is started. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Ready</i> The number of instances in a replacement environment ready to receive traffic in a blue/green deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Skipped</i> The number of instances in the deployment in a skipped state.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Failed</i> The number of instances in the deployment in a failed state.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Succeeded</i> The number of instances in the deployment to which revisions have been successfully deployed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InProgress</i> The number of instances in which the deployment is in progress.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Pending</i> The number of instances in the deployment in a pending state.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>events</i> The event type or types that trigger a rollback.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>enabled</i> Indicates whether a defined automatic rollback configuration is currently enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>rollbackMessage</i> Information that describes the status of a deployment rollback (for example, whether the deployment can't be rolled back, is in progress, failed, or succeeded). <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>rollbackTriggeringDeploymentId</i> The deployment ID of the deployment that was underway and triggered a rollback deployment because it failed or was stopped.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>rollbackDeploymentId</i> The ID of the deployment rollback.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentOption</i> Indicates whether to route deployment traffic behind a load balancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentType</i> Indicates whether to run an in-place deployment or a blue/green deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>autoScalingGroups</i> The names of one or more Auto Scaling groups to identify a replacement environment for a blue/green deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>tagFilters</i> The tag filter key, type, and value used to identify Amazon EC2 instances in a replacement environment for a blue/green deployment. Cannot be used in the same call as ec2TagSet.<br/>Information about the groups of EC2 instance tags that an instance must be identified by in order for it to be included in the replacement environment for a blue/green deployment. Cannot be used in the same call as tagFilters.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ec2TagSet</b>: Information about the groups of EC2 instance tags that an instance must be identified by in order for it to be included in the replacement environment for a blue/green deployment. Cannot be used in the same call as tagFilters.Information about whether to terminate instances in the original fleet during a blue/green deployment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>terminateBlueInstancesOnDeploymentSuccess</b>: Information about whether to terminate instances in the original fleet during a blue/green deployment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>targetGroupPairInfoList</i>  The target group pair information. This is an array of <code>TargeGroupPairInfo</code> objects with a maximum size of one. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>targetGroupInfoList</i> An array that contains information about the target group to use for load balancing in a deployment. In Elastic Load Balancing, target groups are used with Application Load Balancers. <note>  Adding more than one target group to the array is not supported.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>elbInfoList</i> An array that contains information about the load balancer to use for load balancing in a deployment. In Elastic Load Balancing, load balancers are used with Classic Load Balancers. <note>  Adding more than one load balancer to the array is not supported.  </note><br/>        """
        pass

    def get_deployment_config(self, deploymentConfigName: str) -> GetDeploymentConfigOutput:
        """Gets information about a deployment configuration.

        :param self:
        :param deploymentConfigName:  The name of a deployment configuration associated with the IAM user or AWS account.<br/>
        :return: deploymentConfigInfo:  Information about the deployment configuration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>trafficRoutingConfig</i> The configuration that specifies how the deployment traffic is routed. Only deployments with a Lambda compute platform can specify this.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>trafficRoutingConfig</b>: The configuration that specifies how the deployment traffic is routed. Only deployments with a Lambda compute platform can specify this.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>computePlatform</i> The destination platform type for the deployment (<code>Lambda</code>, <code>Server</code>, or <code>ECS</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createTime</i> The time at which the deployment configuration was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>minimumHealthyHosts</i> Information about the number or percentage of minimum healthy instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>minimumHealthyHosts</b>: Information about the number or percentage of minimum healthy instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentConfigName</i> The deployment configuration name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentConfigId</i> The deployment configuration ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The minimum healthy instance type: <ul> <li> HOST_COUNT: The minimum number of healthy instance as an absolute value. </li> <li> FLEET_PERCENT: The minimum number of healthy instance as a percentage of the total number of instance in the deployment. </li> </ul> In an example of nine instance, if a HOST_COUNT of six is specified, deploy to up to three instances at a time. The deployment is successful if six or more instances are deployed to successfully. Otherwise, the deployment fails. If a FLEET_PERCENT of 40 is specified, deploy to up to five instance at a time. The deployment is successful if four or more instance are deployed to successfully. Otherwise, the deployment fails. <note> In a call to the <code>GetDeploymentConfig</code>, CodeDeployDefault.OneAtATime returns a minimum healthy instance type of MOST_CONCURRENCY and a value of 1. This means a deployment to only one instance at a time. (You cannot set the type to MOST_CONCURRENCY, only to HOST_COUNT or FLEET_PERCENT.) In addition, with CodeDeployDefault.OneAtATime, AWS CodeDeploy attempts to ensure that all instances but one are kept in a healthy state during the deployment. Although this allows one instance at a time to be taken offline for a new deployment, it also means that if the deployment to the last instance fails, the overall deployment is still successful. </note> For more information, see <a href="https://docs.aws.amazon.com/codedeploy/latest/userguide/instances-health.html">AWS CodeDeploy Instance Health</a> in the <i>AWS CodeDeploy User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>value</i> The minimum healthy instance value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of traffic shifting (<code>TimeBasedCanary</code> or <code>TimeBasedLinear</code>) used by a deployment configuration .<br/>A configuration that shifts traffic from one version of a Lambda function to another in two increments. The original and target Lambda function versions are specified in the deployment's AppSpec file.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>timeBasedCanary</b>: A configuration that shifts traffic from one version of a Lambda function to another in two increments. The original and target Lambda function versions are specified in the deployment's AppSpec file.        """
        pass

    def get_deployment_group(self, deploymentGroupName: str, applicationName: str) -> GetDeploymentGroupOutput:
        """Gets information about a deployment group.

        :param self:
        :param deploymentGroupName:  The name of a deployment group for the specified application.<br/>applicationName:  The name of an AWS CodeDeploy application associated with the IAM user or AWS account.<br/>
        :return: deploymentGroupInfo:  Information about the deployment group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ecsServices</i>  The target Amazon ECS services in the deployment group. This applies only to deployment groups that use the Amazon ECS compute platform. A target Amazon ECS service is specified as an Amazon ECS cluster and service name pair using the format <code>&lt;clustername&gt;:&lt;servicename&gt;</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>computePlatform</i> The destination platform type for the deployment (<code>Lambda</code>, <code>Server</code>, or <code>ECS</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>onPremisesTagSet</i> Information about groups of tags applied to an on-premises instance. The deployment group includes only on-premises instances identified by all the tag groups. Cannot be used in the same call as onPremisesInstanceTagFilters.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>onPremisesTagSet</b>: Information about groups of tags applied to an on-premises instance. The deployment group includes only on-premises instances identified by all the tag groups. Cannot be used in the same call as onPremisesInstanceTagFilters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ec2TagSet</i> Information about groups of tags applied to an EC2 instance. The deployment group includes only EC2 instances identified by all of the tag groups. Cannot be used in the same call as ec2TagFilters.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ec2TagSet</b>: Information about groups of tags applied to an EC2 instance. The deployment group includes only EC2 instances identified by all of the tag groups. Cannot be used in the same call as ec2TagFilters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastAttemptedDeployment</i> Information about the most recent attempted deployment to the deployment group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>lastAttemptedDeployment</b>: Information about the most recent attempted deployment to the deployment group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastSuccessfulDeployment</i> Information about the most recent successful deployment to the deployment group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>lastSuccessfulDeployment</b>: Information about the most recent successful deployment to the deployment group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>loadBalancerInfo</i> Information about the load balancer to use in a deployment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>loadBalancerInfo</b>: Information about the load balancer to use in a deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>blueGreenDeploymentConfiguration</i> Information about blue/green deployment options for a deployment group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>blueGreenDeploymentConfiguration</b>: Information about blue/green deployment options for a deployment group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentStyle</i> Information about the type of deployment, either in-place or blue/green, you want to run and whether to route deployment traffic behind a load balancer.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>deploymentStyle</b>: Information about the type of deployment, either in-place or blue/green, you want to run and whether to route deployment traffic behind a load balancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>autoRollbackConfiguration</i> Information about the automatic rollback configuration associated with the deployment group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>autoRollbackConfiguration</b>: Information about the automatic rollback configuration associated with the deployment group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>alarmConfiguration</i> A list of alarms associated with the deployment group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>alarmConfiguration</b>: A list of alarms associated with the deployment group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>triggerConfigurations</i> Information about triggers associated with the deployment group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>targetRevision</i> Information about the deployment group's target revision, including type and location.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>targetRevision</b>: Information about the deployment group's target revision, including type and location.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceRoleArn</i> A service role Amazon Resource Name (ARN) that grants CodeDeploy permission to make calls to AWS services on your behalf. For more information, see <a href="https://docs.aws.amazon.com/codedeploy/latest/userguide/getting-started-create-service-role.html">Create a Service Role for AWS CodeDeploy</a> in the <i>AWS CodeDeploy User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>autoScalingGroups</i> A list of associated Auto Scaling groups.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>onPremisesInstanceTagFilters</i> The on-premises instance tags on which to filter. The deployment group includes on-premises instances with any of the specified tags.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ec2TagFilters</i> The Amazon EC2 tags on which to filter. The deployment group includes EC2 instances with any of the specified tags.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentConfigName</i> The deployment configuration name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentGroupName</i> The deployment group name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentGroupId</i> The deployment group ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>applicationName</i> The application name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>revisionType</i> The type of application revision: <ul> <li> S3: An application revision stored in Amazon S3. </li> <li> GitHub: An application revision stored in GitHub (EC2/On-premises deployments only). </li> <li> String: A YAML-formatted or JSON-formatted string (AWS Lambda deployments only). </li> </ul><br/>Information about the location of a revision stored in Amazon S3. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>s3Location</b>: Information about the location of a revision stored in Amazon S3. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>alarms</i> A list of alarms configured for the deployment group. A maximum of 10 alarms can be added to a deployment group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ignorePollAlarmFailure</i> Indicates whether a deployment should continue if information about the current state of alarms cannot be retrieved from Amazon CloudWatch. The default value is false. <ul> <li> true: The deployment proceeds even if alarm status information can't be retrieved from Amazon CloudWatch. </li> <li> false: The deployment stops if alarm status information can't be retrieved from Amazon CloudWatch. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>enabled</i> Indicates whether the alarm configuration is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>events</i> The event type or types that trigger a rollback.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>enabled</i> Indicates whether a defined automatic rollback configuration is currently enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentOption</i> Indicates whether to route deployment traffic behind a load balancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentType</i> Indicates whether to run an in-place deployment or a blue/green deployment.<br/>Information about whether to terminate instances in the original fleet during a blue/green deployment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>terminateBlueInstancesOnDeploymentSuccess</b>: Information about whether to terminate instances in the original fleet during a blue/green deployment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>targetGroupPairInfoList</i>  The target group pair information. This is an array of <code>TargeGroupPairInfo</code> objects with a maximum size of one. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>targetGroupInfoList</i> An array that contains information about the target group to use for load balancing in a deployment. In Elastic Load Balancing, target groups are used with Application Load Balancers. <note>  Adding more than one target group to the array is not supported.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>elbInfoList</i> An array that contains information about the load balancer to use for load balancing in a deployment. In Elastic Load Balancing, load balancers are used with Classic Load Balancers. <note>  Adding more than one load balancer to the array is not supported.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>createTime</i> A timestamp that indicates when the most recent deployment to the deployment group started.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>endTime</i> A timestamp that indicates when the most recent deployment to the deployment group was complete.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the most recent deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentId</i>  The unique ID of a deployment. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>createTime</i> A timestamp that indicates when the most recent deployment to the deployment group started.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>endTime</i> A timestamp that indicates when the most recent deployment to the deployment group was complete.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the most recent deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentId</i>  The unique ID of a deployment. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ec2TagSetList</i> A list that contains other lists of EC2 instance tag groups. For an instance to be included in the deployment group, it must be identified by all of the tag groups in the list.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>onPremisesTagSetList</i> A list that contains other lists of on-premises instance tag groups. For an instance to be included in the deployment group, it must be identified by all of the tag groups in the list.<br/>        """
        pass

    def get_deployment_instance(self, instanceId: str, deploymentId: str) -> GetDeploymentInstanceOutput:
        """Gets information about an instance as part of a deployment.

        :param self:
        :param instanceId:   The unique ID of an instance in the deployment group. <br/>deploymentId:   The unique ID of a deployment. <br/>
        :return: instanceSummary:   Information about the instance. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>instanceType</i> Information about which environment an instance belongs to in a blue/green deployment. <ul> <li> BLUE: The instance is part of the original environment. </li> <li> GREEN: The instance is part of the replacement environment. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lifecycleEvents</i> A list of lifecycle events for this instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedAt</i> A timestamp that indicaties when the instance information was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The deployment status for this instance: <ul> <li> Pending: The deployment is pending for this instance. </li> <li> In Progress: The deployment is in progress for this instance. </li> <li> Succeeded: The deployment has succeeded for this instance. </li> <li> Failed: The deployment has failed for this instance. </li> <li> Skipped: The deployment has been skipped for this instance. </li> <li> Unknown: The deployment status is unknown for this instance. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>instanceId</i> The instance ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentId</i>  The unique ID of a deployment. <br/>        """
        pass

    def get_deployment_target(self, deploymentId: str=None, targetId: str=None) -> GetDeploymentTargetOutput:
        """ Returns information about a deployment target. 

        :param self:
        :param targetId:   The unique ID of a deployment target. <br/>deploymentId:   The unique ID of a deployment. <br/>
        :return: deploymentTarget:   A deployment target that contains information about a deployment such as its status, lifecyle events, and when it was last updated. It also contains metadata about the deployment target. The deployment target metadata depends on the deployment target's type (<code>instanceTarget</code>, <code>lambdaTarget</code>, or <code>ecsTarget</code>). :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ecsTarget</i>  Information about the target for a deployment that uses the Amazon ECS compute platform. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ecsTarget</b>:  Information about the target for a deployment that uses the Amazon ECS compute platform. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lambdaTarget</i>  Information about the target for a deployment that uses the AWS Lambda compute platform. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>lambdaTarget</b>:  Information about the target for a deployment that uses the AWS Lambda compute platform. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>instanceTarget</i>  Information about the target for a deployment that uses the EC2/On-premises compute platform. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>instanceTarget</b>:  Information about the target for a deployment that uses the EC2/On-premises compute platform. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentTargetType</i>  The deployment type that is specific to the deployment's compute platform. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>instanceLabel</i>  A label that identifies whether the instance is an original target (<code>BLUE</code>) or a replacement target (<code>GREEN</code>). <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lifecycleEvents</i>  The lifecycle events of the deployment to this target instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedAt</i>  The date and time when the target instance was updated by a deployment. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i>  The status an EC2/On-premises deployment's target instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>targetArn</i>  The ARN of the target. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>targetId</i>  The unique ID of a deployment target that has a type of <code>instanceTarget</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentId</i>  The unique ID of a deployment. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lifecycleEvents</i>  The lifecycle events of the deployment to this target Lambda function. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedAt</i>  The date and time when the target Lambda function was updated by a deployment. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i>  The status an AWS Lambda deployment's target Lambda function. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>targetArn</i>  The ARN of the target. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>targetId</i>  The unique ID of a deployment target that has a type of <code>lambdaTarget</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentId</i>  The unique ID of a deployment. <br/> A <code>LambdaFunctionInfo</code> object that describes a target Lambda function. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>lambdaFunctionInfo</b>:  A <code>LambdaFunctionInfo</code> object that describes a target Lambda function. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>taskSetsInfo</i>  The <code>ECSTaskSet</code> objects associated with the ECS target. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i>  The status an Amazon ECS deployment's target ECS application. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lifecycleEvents</i>  The lifecycle events of the deployment to this target Amazon ECS application. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedAt</i>  The date and time when the target Amazon ECS application was updated by a deployment. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>targetArn</i>  The ARN of the target. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>targetId</i>  The unique ID of a deployment target that has a type of <code>ecsTarget</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentId</i>  The unique ID of a deployment. <br/>        """
        pass

    def get_on_premises_instance(self, instanceName: str) -> GetOnPremisesInstanceOutput:
        """ Gets information about an on-premises instance. 

        :param self:
        :param instanceName:   The name of the on-premises instance about which to get information. <br/>
        :return: instanceInfo:   Information about the on-premises instance. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> The tags currently associated with the on-premises instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deregisterTime</i> If the on-premises instance was deregistered, the time at which the on-premises instance was deregistered.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>registerTime</i> The time at which the on-premises instance was registered.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>instanceArn</i> The ARN of the on-premises instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>iamUserArn</i> The IAM user ARN associated with the on-premises instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>iamSessionArn</i> The ARN of the IAM session associated with the on-premises instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>instanceName</i> The name of the on-premises instance.<br/>        """
        pass

    def list_application_revisions(self, applicationName: str, sortBy: str=None, sortOrder: str=None, s3Bucket: str=None, s3KeyPrefix: str=None, deployed: str=None, nextToken: str=None) -> ListApplicationRevisionsOutput:
        """Lists information about revisions for an application.

        :param self:
        :param nextToken:  An identifier returned from the previous <code>ListApplicationRevisions</code> call. It can be used to return the next set of applications in the list.<br/>deployed:   Whether to list revisions based on whether the revision is the target revision of an deployment group:  <ul> <li> include: List revisions that are target revisions of a deployment group. </li> <li> exclude: Do not list revisions that are target revisions of a deployment group. </li> <li> ignore: List all revisions. </li> </ul><br/>s3KeyPrefix:   A key prefix for the set of Amazon S3 objects to limit the search for revisions. <br/>s3Bucket:   An Amazon S3 bucket name to limit the search for revisions.   If set to null, all of the user's buckets are searched. <br/>sortOrder:   The order in which to sort the list results:  <ul> <li> ascending: ascending order. </li> <li> descending: descending order. </li> </ul> If not specified, the results are sorted in ascending order. If set to null, the results are sorted in an arbitrary order.<br/>sortBy:  The column name to use to sort the list results: <ul> <li> registerTime: Sort by the time the revisions were registered with AWS CodeDeploy. </li> <li> firstUsedTime: Sort by the time the revisions were first used in a deployment. </li> <li> lastUsedTime: Sort by the time the revisions were last used in a deployment. </li> </ul>  If not specified or set to null, the results are returned in an arbitrary order. <br/>applicationName:   The name of an AWS CodeDeploy application associated with the IAM user or AWS account. <br/>
        :return: nextToken:  If a large amount of information is returned, an identifier is also returned. It can be used in a subsequent list application revisions call to return the next set of application revisions in the list.<br/>revisions:  A list of locations that contain the matching revisions.<br/>        """
        pass

    def list_applications(self, nextToken: str=None) -> ListApplicationsOutput:
        """Lists the applications registered with the IAM user or AWS account.

        :param self:
        :param nextToken:  An identifier returned from the previous list applications call. It can be used to return the next set of applications in the list.<br/>
        :return: nextToken:  If a large amount of information is returned, an identifier is also returned. It can be used in a subsequent list applications call to return the next set of applications in the list.<br/>applications:  A list of application names.<br/>        """
        pass

    def list_deployment_configs(self, nextToken: str=None) -> ListDeploymentConfigsOutput:
        """Lists the deployment configurations with the IAM user or AWS account.

        :param self:
        :param nextToken:  An identifier returned from the previous <code>ListDeploymentConfigs</code> call. It can be used to return the next set of deployment configurations in the list. <br/>
        :return: nextToken:  If a large amount of information is returned, an identifier is also returned. It can be used in a subsequent list deployment configurations call to return the next set of deployment configurations in the list.<br/>deploymentConfigsList:  A list of deployment configurations, including built-in configurations such as CodeDeployDefault.OneAtATime.<br/>        """
        pass

    def list_deployment_groups(self, applicationName: str, nextToken: str=None) -> ListDeploymentGroupsOutput:
        """Lists the deployment groups for an application registered with the IAM user or AWS account.

        :param self:
        :param nextToken:  An identifier returned from the previous list deployment groups call. It can be used to return the next set of deployment groups in the list.<br/>applicationName:  The name of an AWS CodeDeploy application associated with the IAM user or AWS account.<br/>
        :return: nextToken:  If a large amount of information is returned, an identifier is also returned. It can be used in a subsequent list deployment groups call to return the next set of deployment groups in the list.<br/>deploymentGroups:  A list of deployment group names.<br/>applicationName:  The application name.<br/>        """
        pass

    def list_deployment_instances(self, deploymentId: str, nextToken: str=None, instanceStatusFilter: List[InstanceStatus]=None, instanceTypeFilter: List[InstanceType]=None) -> ListDeploymentInstancesOutput:
        """<note>  The newer BatchGetDeploymentTargets should be used instead because it works with all compute types. <code>ListDeploymentInstances</code> throws an exception if it is used with a compute platform other than EC2/On-premises or AWS Lambda.  </note>  Lists the instance for a deployment associated with the IAM user or AWS account. 

        :param self:
        :param instanceTypeFilter:  The set of instances in a blue/green deployment, either those in the original environment ("BLUE") or those in the replacement environment ("GREEN"), for which you want to view instance information.<br/>instanceStatusFilter:  A subset of instances to list by status: <ul> <li> Pending: Include those instances with pending deployments. </li> <li> InProgress: Include those instances where deployments are still in progress. </li> <li> Succeeded: Include those instances with successful deployments. </li> <li> Failed: Include those instances with failed deployments. </li> <li> Skipped: Include those instances with skipped deployments. </li> <li> Unknown: Include those instances with deployments in an unknown state. </li> </ul><br/>nextToken:  An identifier returned from the previous list deployment instances call. It can be used to return the next set of deployment instances in the list.<br/>deploymentId:   The unique ID of a deployment. <br/>
        :return: nextToken:  If a large amount of information is returned, an identifier is also returned. It can be used in a subsequent list deployment instances call to return the next set of deployment instances in the list.<br/>instancesList:  A list of instance IDs.<br/>        """
        pass

    def list_deployment_targets(self, deploymentId: str=None, nextToken: str=None, targetFilters: TargetFilters=None) -> ListDeploymentTargetsOutput:
        """ Returns an array of target IDs that are associated a deployment. 

        :param self:
        :param targetFilters:   A key used to filter the returned targets. The two valid values are: <ul> <li>  <code>TargetStatus</code> - A <code>TargetStatus</code> filter string can be <code>Failed</code>, <code>InProgress</code>, <code>Pending</code>, <code>Ready</code>, <code>Skipped</code>, <code>Succeeded</code>, or <code>Unknown</code>.  </li> <li>  <code>ServerInstanceLabel</code> - A <code>ServerInstanceLabel</code> filter string can be <code>Blue</code> or <code>Green</code>.  </li> </ul><br/>nextToken:   A token identifier returned from the previous <code>ListDeploymentTargets</code> call. It can be used to return the next set of deployment targets in the list. <br/>deploymentId:   The unique ID of a deployment. <br/>
        :return: nextToken:   If a large amount of information is returned, a token identifier is also returned. It can be used in a subsequent <code>ListDeploymentTargets</code> call to return the next set of deployment targets in the list. <br/>targetIds:   The unique IDs of deployment targets. <br/>        """
        pass

    def list_deployments(self, applicationName: str=None, deploymentGroupName: str=None, includeOnlyStatuses: List[DeploymentStatus]=None, createTimeRange: TimeRange=None, nextToken: str=None) -> ListDeploymentsOutput:
        """Lists the deployments in a deployment group for an application registered with the IAM user or AWS account.

        :param self:
        :param nextToken:  An identifier returned from the previous list deployments call. It can be used to return the next set of deployments in the list.<br/>createTimeRange:  A time range (start and end) for returning a subset of the list of deployments.:<br/>includeOnlyStatuses:  A subset of deployments to list by status: <ul> <li> Created: Include created deployments in the resulting list. </li> <li> Queued: Include queued deployments in the resulting list. </li> <li> In Progress: Include in-progress deployments in the resulting list. </li> <li> Succeeded: Include successful deployments in the resulting list. </li> <li> Failed: Include failed deployments in the resulting list. </li> <li> Stopped: Include stopped deployments in the resulting list. </li> </ul><br/>deploymentGroupName:  The name of a deployment group for the specified application. <note> If <code>deploymentGroupName</code> is specified, then <code>applicationName</code> must be specified. If it is not specified, then <code>applicationName</code> must not be specified.  </note><br/>applicationName:  The name of an AWS CodeDeploy application associated with the IAM user or AWS account. <note> If <code>applicationName</code> is specified, then <code>deploymentGroupName</code> must be specified. If it is not specified, then <code>deploymentGroupName</code> must not be specified.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>end</i> The end time of the time range. <note> Specify null to leave the end time open-ended. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>start</i> The start time of the time range. <note> Specify null to leave the start time open-ended. </note><br/>
        :return: nextToken:  If a large amount of information is returned, an identifier is also returned. It can be used in a subsequent list deployments call to return the next set of deployments in the list.<br/>deployments:  A list of deployment IDs.<br/>        """
        pass

    def list_git_hub_account_token_names(self, nextToken: str=None) -> ListGitHubAccountTokenNamesOutput:
        """Lists the names of stored connections to GitHub accounts.

        :param self:
        :param nextToken:  An identifier returned from the previous ListGitHubAccountTokenNames call. It can be used to return the next set of names in the list. <br/>
        :return: nextToken:  If a large amount of information is returned, an identifier is also returned. It can be used in a subsequent ListGitHubAccountTokenNames call to return the next set of names in the list. <br/>tokenNameList:  A list of names of connections to GitHub accounts.<br/>        """
        pass

    def list_on_premises_instances(self, registrationStatus: str=None, tagFilters: List[TagFilter]=None, nextToken: str=None) -> ListOnPremisesInstancesOutput:
        """Gets a list of names for one or more on-premises instances. Unless otherwise specified, both registered and deregistered on-premises instance names are listed. To list only registered or deregistered on-premises instance names, use the registration status parameter.

        :param self:
        :param nextToken:  An identifier returned from the previous list on-premises instances call. It can be used to return the next set of on-premises instances in the list.<br/>tagFilters:  The on-premises instance tags that are used to restrict the on-premises instance names returned.<br/>registrationStatus:  The registration status of the on-premises instances: <ul> <li> Deregistered: Include deregistered on-premises instances in the resulting list. </li> <li> Registered: Include registered on-premises instances in the resulting list. </li> </ul><br/>
        :return: nextToken:  If a large amount of information is returned, an identifier is also returned. It can be used in a subsequent list on-premises instances call to return the next set of on-premises instances in the list.<br/>instanceNames:  The list of matching on-premises instance names.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceArn: str, NextToken: str=None) -> ListTagsForResourceOutput:
        """ Returns a list of tags for the resource identified by a specified ARN. Tags are used to organize and categorize your CodeDeploy resources. 

        :param self:
        :param NextToken:  An identifier returned from the previous <code>ListTagsForResource</code> call. It can be used to return the next set of applications in the list.<br/>ResourceArn:   The ARN of a CodeDeploy resource. <code>ListTagsForResource</code> returns all the tags associated with the resource that is identified by the <code>ResourceArn</code>. <br/>
        :return: NextToken:  If a large amount of information is returned, an identifier is also returned. It can be used in a subsequent list application revisions call to return the next set of application revisions in the list.<br/>Tags:   A list of tags returned by <code>ListTagsForResource</code>. The tags are associated with the resource identified by the input <code>ResourceArn</code> parameter. <br/>        """
        pass

    def put_lifecycle_event_hook_execution_status(self, deploymentId: str=None, lifecycleEventHookExecutionId: str=None, status: str=None) -> PutLifecycleEventHookExecutionStatusOutput:
        """ Sets the result of a Lambda validation function. The function validates one or both lifecycle events (<code>BeforeAllowTraffic</code> and <code>AfterAllowTraffic</code>) and returns <code>Succeeded</code> or <code>Failed</code>. 

        :param self:
        :param status:  The result of a Lambda function that validates a deployment lifecycle event (<code>Succeeded</code> or <code>Failed</code>).<br/>lifecycleEventHookExecutionId:   The execution ID of a deployment's lifecycle hook. A deployment lifecycle hook is specified in the <code>hooks</code> section of the AppSpec file. <br/>deploymentId:   The unique ID of a deployment. Pass this ID to a Lambda function that validates a deployment lifecycle event. <br/>
        :return: lifecycleEventHookExecutionId:  The execution ID of the lifecycle event hook. A hook is specified in the <code>hooks</code> section of the deployment's AppSpec file.<br/>        """
        pass

    def register_application_revision(self, revision: RevisionLocation, applicationName: str, description: str=None):
        """Registers with AWS CodeDeploy a revision for the specified application.

        :param self:
        :param revision:  Information about the application revision to register, including type and location.:<br/>description:  A comment about the revision.<br/>applicationName:  The name of an AWS CodeDeploy application associated with the IAM user or AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>appSpecContent</i>  The content of an AppSpec file for an AWS Lambda or Amazon ECS deployment. The content is formatted as JSON or YAML and stored as a RawString. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>appSpecContent</b>:  The content of an AppSpec file for an AWS Lambda or Amazon ECS deployment. The content is formatted as JSON or YAML and stored as a RawString. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>string</i> Information about the location of an AWS Lambda deployment revision stored as a RawString.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>string</b>: Information about the location of an AWS Lambda deployment revision stored as a RawString.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>gitHubLocation</i> Information about the location of application artifacts stored in GitHub.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>gitHubLocation</b>: Information about the location of application artifacts stored in GitHub.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>s3Location</i> Information about the location of a revision stored in Amazon S3. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>s3Location</b>: Information about the location of a revision stored in Amazon S3. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>revisionType</i> The type of application revision: <ul> <li> S3: An application revision stored in Amazon S3. </li> <li> GitHub: An application revision stored in GitHub (EC2/On-premises deployments only). </li> <li> String: A YAML-formatted or JSON-formatted string (AWS Lambda deployments only). </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>eTag</i> The ETag of the Amazon S3 object that represents the bundled artifacts for the application revision. If the ETag is not specified as an input parameter, ETag validation of the object is skipped.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> A specific version of the Amazon S3 object that represents the bundled artifacts for the application revision. If the version is not specified, the system uses the most recent version by default.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>bundleType</i> The file type of the application revision. Must be one of the following: <ul> <li> tar: A tar archive file. </li> <li> tgz: A compressed tar archive file. </li> <li> zip: A zip archive file. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>key</i> The name of the Amazon S3 object that represents the bundled artifacts for the application revision.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>bucket</i> The name of the Amazon S3 bucket where the application revision is stored.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>commitId</i> The SHA1 commit ID of the GitHub commit that represents the bundled artifacts for the application revision.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>repository</i> The GitHub account and repository pair that stores a reference to the commit that represents the bundled artifacts for the application revision.  Specified as account/repository.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>sha256</i> The SHA256 hash value of the revision content.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>content</i> The YAML-formatted or JSON-formatted revision string. It includes information about which Lambda function to update and optional Lambda functions that validate deployment lifecycle events.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>sha256</i>  The SHA256 hash value of the revision content. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>content</i>  The YAML-formatted or JSON-formatted revision string.   For an AWS Lambda deployment, the content includes a Lambda function name, the alias for its original version, and the alias for its replacement version. The deployment shifts traffic from the original version of the Lambda function to the replacement version.   For an Amazon ECS deployment, the content includes the task name, information about the load balancer that serves traffic to the container, and more.   For both types of deployments, the content can specify Lambda functions that run at specified hooks, such as <code>BeforeInstall</code>, during a deployment. <br/>
        :return:         """
        pass

    def register_on_premises_instance(self, instanceName: str, iamSessionArn: str=None, iamUserArn: str=None):
        """Registers an on-premises instance. <note> Only one IAM ARN (an IAM session ARN or IAM user ARN) is supported in the request. You cannot use both. </note>

        :param self:
        :param iamUserArn:  The ARN of the IAM user to associate with the on-premises instance.<br/>iamSessionArn:  The ARN of the IAM session to associate with the on-premises instance.<br/>instanceName:  The name of the on-premises instance to register.<br/>
        :return:         """
        pass

    def remove_tags_from_on_premises_instances(self, instanceNames: List[InstanceName], tags: List[Tag]):
        """Removes one or more tags from one or more on-premises instances.

        :param self:
        :param instanceNames:  The names of the on-premises instances from which to remove tags.<br/>tags:  The tag key-value pairs to remove from the on-premises instances.<br/>
        :return:         """
        pass

    def skip_wait_time_for_instance_termination(self, deploymentId: str=None):
        """In a blue/green deployment, overrides any specified wait time and starts terminating instances immediately after the traffic routing is complete.

        :param self:
        :param deploymentId:   The unique ID of a blue/green deployment for which you want to skip the instance termination wait time. <br/>
        :return:         """
        pass

    def stop_deployment(self, deploymentId: str, autoRollbackEnabled: NullableBoolean=None) -> StopDeploymentOutput:
        """Attempts to stop an ongoing deployment.

        :param self:
        :param autoRollbackEnabled:   Indicates, when a deployment is stopped, whether instances that have been updated should be rolled back to the previous version of the application revision. <br/>deploymentId:   The unique ID of a deployment. <br/>
        :return: statusMessage:  An accompanying status message.<br/>status:  The status of the stop deployment operation: <ul> <li> Pending: The stop operation is pending. </li> <li> Succeeded: The stop operation was successful. </li> </ul><br/>        """
        pass

    def tag_resource(self, Tags: List[Tag], ResourceArn: str) -> TagResourceOutput:
        """ Associates the list of tags in the input <code>Tags</code> parameter with the resource identified by the <code>ResourceArn</code> input parameter. 

        :param self:
        :param Tags:   A list of tags that <code>TagResource</code> associates with a resource. The resource is identified by the <code>ResourceArn</code> input parameter. <br/>ResourceArn:   The ARN of a resource, such as a CodeDeploy application or deployment group. <br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[Key], ResourceArn: str) -> UntagResourceOutput:
        """ Disassociates a resource from a list of tags. The resource is identified by the <code>ResourceArn</code> input parameter. The tags are identfied by the list of keys in the <code>TagKeys</code> input parameter. 

        :param self:
        :param TagKeys:   A list of keys of <code>Tag</code> objects. The <code>Tag</code> objects identified by the keys are disassociated from the resource specified by the <code>ResourceArn</code> input parameter. <br/>ResourceArn:   The ARN that specifies from which resource to disassociate the tags with the keys in the <code>TagKeys</code> input paramter. <br/>
        :return:         """
        pass

    def update_application(self, applicationName: str=None, newApplicationName: str=None):
        """Changes the name of an application.

        :param self:
        :param newApplicationName:  The new name to give the application.<br/>applicationName:  The current name of the application you want to change.<br/>
        :return:         """
        pass

    def update_deployment_group(self, currentDeploymentGroupName: str, applicationName: str, newDeploymentGroupName: str=None, deploymentConfigName: str=None, ec2TagFilters: List[EC2TagFilter]=None, onPremisesInstanceTagFilters: List[TagFilter]=None, autoScalingGroups: List[AutoScalingGroupName]=None, serviceRoleArn: str=None, triggerConfigurations: List[TriggerConfig]=None, alarmConfiguration: AlarmConfiguration=None, autoRollbackConfiguration: AutoRollbackConfiguration=None, deploymentStyle: DeploymentStyle=None, blueGreenDeploymentConfiguration: BlueGreenDeploymentConfiguration=None, loadBalancerInfo: LoadBalancerInfo=None, ec2TagSet: EC2TagSet=None, ecsServices: List[ECSService]=None, onPremisesTagSet: OnPremisesTagSet=None) -> UpdateDeploymentGroupOutput:
        """Changes information about a deployment group.

        :param self:
        :param onPremisesTagSet:  Information about an on-premises instance tag set. The deployment group includes only on-premises instances identified by all the tag groups.:<br/>ecsServices:   The target Amazon ECS services in the deployment group. This applies only to deployment groups that use the Amazon ECS compute platform. A target Amazon ECS service is specified as an Amazon ECS cluster and service name pair using the format <code>&lt;clustername&gt;:&lt;servicename&gt;</code>. <br/>ec2TagSet:  Information about groups of tags applied to on-premises instances. The deployment group includes only EC2 instances identified by all the tag groups.:<br/>loadBalancerInfo:  Information about the load balancer used in a deployment.:<br/>blueGreenDeploymentConfiguration:  Information about blue/green deployment options for a deployment group.:<br/>deploymentStyle:  Information about the type of deployment, either in-place or blue/green, you want to run and whether to route deployment traffic behind a load balancer.:<br/>autoRollbackConfiguration:  Information for an automatic rollback configuration that is added or changed when a deployment group is updated.:<br/>alarmConfiguration:  Information to add or change about Amazon CloudWatch alarms when the deployment group is updated.:<br/>triggerConfigurations:  Information about triggers to change when the deployment group is updated. For examples, see <a href="https://docs.aws.amazon.com/codedeploy/latest/userguide/how-to-notify-edit.html">Modify Triggers in an AWS CodeDeploy Deployment Group</a> in the AWS CodeDeploy User Guide.<br/>serviceRoleArn:  A replacement ARN for the service role, if you want to change it.<br/>autoScalingGroups:  The replacement list of Auto Scaling groups to be included in the deployment group, if you want to change them. To keep the Auto Scaling groups, enter their names. To remove Auto Scaling groups, do not enter any Auto Scaling group names.<br/>onPremisesInstanceTagFilters:  The replacement set of on-premises instance tags on which to filter, if you want to change them. To keep the existing tags, enter their names. To remove tags, do not enter any tag names.<br/>ec2TagFilters:  The replacement set of Amazon EC2 tags on which to filter, if you want to change them. To keep the existing tags, enter their names. To remove tags, do not enter any tag names.<br/>deploymentConfigName:  The replacement deployment configuration name to use, if you want to change it.<br/>newDeploymentGroupName:  The new name of the deployment group, if you want to change it.<br/>currentDeploymentGroupName:  The current name of the deployment group.<br/>applicationName:  The application name that corresponds to the deployment group to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>alarms</i> A list of alarms configured for the deployment group. A maximum of 10 alarms can be added to a deployment group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ignorePollAlarmFailure</i> Indicates whether a deployment should continue if information about the current state of alarms cannot be retrieved from Amazon CloudWatch. The default value is false. <ul> <li> true: The deployment proceeds even if alarm status information can't be retrieved from Amazon CloudWatch. </li> <li> false: The deployment stops if alarm status information can't be retrieved from Amazon CloudWatch. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enabled</i> Indicates whether the alarm configuration is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>events</i> The event type or types that trigger a rollback.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enabled</i> Indicates whether a defined automatic rollback configuration is currently enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentOption</i> Indicates whether to route deployment traffic behind a load balancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentType</i> Indicates whether to run an in-place deployment or a blue/green deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>greenFleetProvisioningOption</i> Information about how instances are provisioned for a replacement environment in a blue/green deployment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>greenFleetProvisioningOption</b>: Information about how instances are provisioned for a replacement environment in a blue/green deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentReadyOption</i> Information about the action to take when newly provisioned instances are ready to receive traffic in a blue/green deployment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>deploymentReadyOption</b>: Information about the action to take when newly provisioned instances are ready to receive traffic in a blue/green deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>terminateBlueInstancesOnDeploymentSuccess</i> Information about whether to terminate instances in the original fleet during a blue/green deployment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>terminateBlueInstancesOnDeploymentSuccess</b>: Information about whether to terminate instances in the original fleet during a blue/green deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>terminationWaitTimeInMinutes</i> For an Amazon EC2 deployment, the number of minutes to wait after a successful blue/green deployment before terminating instances from the original environment.  For an Amazon ECS deployment, the number of minutes before deleting the original (blue) task set. During an Amazon ECS deployment, CodeDeploy shifts traffic from the original (blue) task set to a replacement (green) task set.   The maximum setting is 2880 minutes (2 days). <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>action</i> The action to take on instances in the original environment after a successful blue/green deployment. <ul> <li> TERMINATE: Instances are terminated after a specified wait time. </li> <li> KEEP_ALIVE: Instances are left running after they are deregistered from the load balancer and removed from the deployment group. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>waitTimeInMinutes</i> The number of minutes to wait before the status of a blue/green deployment is changed to Stopped if rerouting is not started manually. Applies only to the STOP_DEPLOYMENT option for actionOnTimeout<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>actionOnTimeout</i> Information about when to reroute traffic from an original environment to a replacement environment in a blue/green deployment. <ul> <li> CONTINUE_DEPLOYMENT: Register new instances with the load balancer immediately after the new application revision is installed on the instances in the replacement environment. </li> <li> STOP_DEPLOYMENT: Do not register new instances with a load balancer unless traffic rerouting is started using <a>ContinueDeployment</a>. If traffic rerouting is not started before the end of the specified wait period, the deployment status is changed to Stopped. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>action</i> The method used to add instances to a replacement environment. <ul> <li> DISCOVER_EXISTING: Use instances that already exist or will be created manually. </li> <li> COPY_AUTO_SCALING_GROUP: Use settings from a specified Auto Scaling group to define and create instances in a new Auto Scaling group. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>targetGroupPairInfoList</i>  The target group pair information. This is an array of <code>TargeGroupPairInfo</code> objects with a maximum size of one. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>targetGroupInfoList</i> An array that contains information about the target group to use for load balancing in a deployment. In Elastic Load Balancing, target groups are used with Application Load Balancers. <note>  Adding more than one target group to the array is not supported.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>elbInfoList</i> An array that contains information about the load balancer to use for load balancing in a deployment. In Elastic Load Balancing, load balancers are used with Classic Load Balancers. <note>  Adding more than one load balancer to the array is not supported.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ec2TagSetList</i> A list that contains other lists of EC2 instance tag groups. For an instance to be included in the deployment group, it must be identified by all of the tag groups in the list.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>onPremisesTagSetList</i> A list that contains other lists of on-premises instance tag groups. For an instance to be included in the deployment group, it must be identified by all of the tag groups in the list.<br/>
        :return: hooksNotCleanedUp:  If the output contains no data, and the corresponding deployment group contained at least one Auto Scaling group, AWS CodeDeploy successfully removed all corresponding Auto Scaling lifecycle event hooks from the AWS account. If the output contains data, AWS CodeDeploy could not remove some Auto Scaling lifecycle event hooks from the AWS account.<br/>        """
        pass



class CodePipeline(BaseClient):
    """<fullname>AWS CodePipeline</fullname>  <b>Overview</b>  This is the AWS CodePipeline API Reference. This guide provides descriptions of the actions and data types for AWS CodePipeline. Some functionality for your pipeline is only configurable through the API. For additional information, see the <a href="https://docs.aws.amazon.com/codepipeline/latest/userguide/welcome.html">AWS CodePipeline User Guide</a>. You can use the AWS CodePipeline API to work with pipelines, stages, actions, and transitions, as described below.  <i>Pipelines</i> are models of automated release processes. Each pipeline is uniquely named, and consists of stages, actions, and transitions.  You can work with pipelines by calling: <ul> <li>  <a>CreatePipeline</a>, which creates a uniquely-named pipeline. </li> <li>  <a>DeletePipeline</a>, which deletes the specified pipeline. </li> <li>  <a>GetPipeline</a>, which returns information about the pipeline structure and pipeline metadata, including the pipeline Amazon Resource Name (ARN). </li> <li>  <a>GetPipelineExecution</a>, which returns information about a specific execution of a pipeline. </li> <li>  <a>GetPipelineState</a>, which returns information about the current state of the stages and actions of a pipeline. </li> <li>  <a>ListActionExecutions</a>, which returns action-level details for past executions. The details include full stage and action-level details, including individual action duration, status, any errors which occurred during the execution, and input and output artifact location details. </li> <li>  <a>ListPipelines</a>, which gets a summary of all of the pipelines associated with your account. </li> <li>  <a>ListPipelineExecutions</a>, which gets a summary of the most recent executions for a pipeline. </li> <li>  <a>StartPipelineExecution</a>, which runs the the most recent revision of an artifact through the pipeline. </li> <li>  <a>UpdatePipeline</a>, which updates a pipeline with edits or changes to the structure of the pipeline. </li> </ul> Pipelines include <i>stages</i>. Each stage contains one or more actions that must complete before the next stage begins. A stage will result in success or failure. If a stage fails, then the pipeline stops at that stage and will remain stopped until either a new version of an artifact appears in the source location, or a user takes action to re-run the most recent artifact through the pipeline. You can call <a>GetPipelineState</a>, which displays the status of a pipeline, including the status of stages in the pipeline, or <a>GetPipeline</a>, which returns the entire structure of the pipeline, including the stages of that pipeline. For more information about the structure of stages and actions, also refer to the <a href="https://docs.aws.amazon.com/codepipeline/latest/userguide/pipeline-structure.html">AWS CodePipeline Pipeline Structure Reference</a>. Pipeline stages include <i>actions</i>, which are categorized into categories such as source or build actions performed within a stage of a pipeline. For example, you can use a source action to import artifacts into a pipeline from a source such as Amazon S3. Like stages, you do not work with actions directly in most cases, but you do define and interact with actions when working with pipeline operations such as <a>CreatePipeline</a> and <a>GetPipelineState</a>. Valid action categories are: <ul> <li> Source </li> <li> Build </li> <li> Test </li> <li> Deploy </li> <li> Approval </li> <li> Invoke </li> </ul> Pipelines also include <i>transitions</i>, which allow the transition of artifacts from one stage to the next in a pipeline after the actions in one stage complete. You can work with transitions by calling: <ul> <li>  <a>DisableStageTransition</a>, which prevents artifacts from transitioning to the next stage in a pipeline. </li> <li>  <a>EnableStageTransition</a>, which enables transition of artifacts between stages in a pipeline.  </li> </ul>  <b>Using the API to integrate with AWS CodePipeline</b>  For third-party integrators or developers who want to create their own integrations with AWS CodePipeline, the expected sequence varies from the standard API user. In order to integrate with AWS CodePipeline, developers will need to work with the following items:  <b>Jobs</b>, which are instances of an action. For example, a job for a source action might import a revision of an artifact from a source.  You can work with jobs by calling: <ul> <li>  <a>AcknowledgeJob</a>, which confirms whether a job worker has received the specified job, </li> <li>  <a>GetJobDetails</a>, which returns the details of a job, </li> <li>  <a>PollForJobs</a>, which determines whether there are any jobs to act upon,  </li> <li>  <a>PutJobFailureResult</a>, which provides details of a job failure, and </li> <li>  <a>PutJobSuccessResult</a>, which provides details of a job success. </li> </ul>  <b>Third party jobs</b>, which are instances of an action created by a partner action and integrated into AWS CodePipeline. Partner actions are created by members of the AWS Partner Network. You can work with third party jobs by calling: <ul> <li>  <a>AcknowledgeThirdPartyJob</a>, which confirms whether a job worker has received the specified job, </li> <li>  <a>GetThirdPartyJobDetails</a>, which requests the details of a job for a partner action, </li> <li>  <a>PollForThirdPartyJobs</a>, which determines whether there are any jobs to act upon,  </li> <li>  <a>PutThirdPartyJobFailureResult</a>, which provides details of a job failure, and </li> <li>  <a>PutThirdPartyJobSuccessResult</a>, which provides details of a job success. </li> </ul>"""


    class ListActionTypesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_action_types` for valid parameters"""
            pass
    class ListPipelineExecutionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_pipeline_executions` for valid parameters"""
            pass
    class ListPipelinesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_pipelines` for valid parameters"""
            pass
    class ListWebhooksPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_webhooks` for valid parameters"""
            pass
    class ListActionExecutionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_action_executions` for valid parameters"""
            pass
    class ListTagsForResourcePaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_tags_for_resource` for valid parameters"""
            pass


    class AcknowledgeJobInput(Mapping):
        pass
    
    class AcknowledgeJobOutput(Mapping):
        pass
    
    class AcknowledgeThirdPartyJobInput(Mapping):
        pass
    
    class AcknowledgeThirdPartyJobOutput(Mapping):
        pass
    
    class CreateCustomActionTypeInput(Mapping):
        pass
    
    class CreateCustomActionTypeOutput(Mapping):
        pass
    
    class ActionTypeSettings(Mapping):
        pass
    
    class ActionConfigurationProperty(Mapping):
        pass
    
    class ArtifactDetails(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreatePipelineInput(Mapping):
        pass
    
    class CreatePipelineOutput(Mapping):
        pass
    
    class PipelineDeclaration(Mapping):
        pass
    
    class DeleteCustomActionTypeInput(Mapping):
        pass
    
    class DeletePipelineInput(Mapping):
        pass
    
    class DeleteWebhookInput(Mapping):
        pass
    
    class DeleteWebhookOutput(Mapping):
        pass
    
    class DeregisterWebhookWithThirdPartyInput(Mapping):
        pass
    
    class DeregisterWebhookWithThirdPartyOutput(Mapping):
        pass
    
    class DisableStageTransitionInput(Mapping):
        pass
    
    class EnableStageTransitionInput(Mapping):
        pass
    
    class GetJobDetailsInput(Mapping):
        pass
    
    class GetJobDetailsOutput(Mapping):
        pass
    
    class GetPipelineInput(Mapping):
        pass
    
    class GetPipelineOutput(Mapping):
        pass
    
    class GetPipelineExecutionInput(Mapping):
        pass
    
    class GetPipelineExecutionOutput(Mapping):
        pass
    
    class GetPipelineStateInput(Mapping):
        pass
    
    class GetPipelineStateOutput(Mapping):
        pass
    
    class GetThirdPartyJobDetailsInput(Mapping):
        pass
    
    class GetThirdPartyJobDetailsOutput(Mapping):
        pass
    
    class ListActionExecutionsInput(Mapping):
        pass
    
    class ListActionExecutionsOutput(Mapping):
        pass
    
    class ActionExecutionFilter(Mapping):
        pass
    
    class ListActionTypesInput(Mapping):
        pass
    
    class ListActionTypesOutput(Mapping):
        pass
    
    class ListPipelineExecutionsInput(Mapping):
        pass
    
    class ListPipelineExecutionsOutput(Mapping):
        pass
    
    class ListPipelinesInput(Mapping):
        pass
    
    class ListPipelinesOutput(Mapping):
        pass
    
    class ListTagsForResourceInput(Mapping):
        pass
    
    class ListTagsForResourceOutput(Mapping):
        pass
    
    class ListWebhooksInput(Mapping):
        pass
    
    class ListWebhooksOutput(Mapping):
        pass
    
    class PollForJobsInput(Mapping):
        pass
    
    class PollForJobsOutput(Mapping):
        pass
    
    class ActionTypeId(Mapping):
        pass
    
    class QueryParamMap(object):
        pass
    
    class PollForThirdPartyJobsInput(Mapping):
        pass
    
    class PollForThirdPartyJobsOutput(Mapping):
        pass
    
    class PutActionRevisionInput(Mapping):
        pass
    
    class PutActionRevisionOutput(Mapping):
        pass
    
    class ActionRevision(Mapping):
        pass
    
    class PutApprovalResultInput(Mapping):
        pass
    
    class PutApprovalResultOutput(Mapping):
        pass
    
    class ApprovalResult(Mapping):
        pass
    
    class PutJobFailureResultInput(Mapping):
        pass
    
    class FailureDetails(Mapping):
        pass
    
    class PutJobSuccessResultInput(Mapping):
        pass
    
    class CurrentRevision(Mapping):
        pass
    
    class ExecutionDetails(Mapping):
        pass
    
    class PutThirdPartyJobFailureResultInput(Mapping):
        pass
    
    class PutThirdPartyJobSuccessResultInput(Mapping):
        pass
    
    class PutWebhookInput(Mapping):
        pass
    
    class PutWebhookOutput(Mapping):
        pass
    
    class WebhookDefinition(Mapping):
        pass
    
    class RegisterWebhookWithThirdPartyInput(Mapping):
        pass
    
    class RegisterWebhookWithThirdPartyOutput(Mapping):
        pass
    
    class RetryStageExecutionInput(Mapping):
        pass
    
    class RetryStageExecutionOutput(Mapping):
        pass
    
    class StartPipelineExecutionInput(Mapping):
        pass
    
    class StartPipelineExecutionOutput(Mapping):
        pass
    
    class TagResourceInput(Mapping):
        pass
    
    class TagResourceOutput(Mapping):
        pass
    
    class UntagResourceInput(Mapping):
        pass
    
    class UntagResourceOutput(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdatePipelineInput(Mapping):
        pass
    
    class UpdatePipelineOutput(Mapping):
        pass
    
    def acknowledge_job(self, nonce: str, jobId: str) -> AcknowledgeJobOutput:
        """Returns information about a specified job and whether that job has been received by the job worker. Only used for custom actions.

        :param self:
        :param nonce:  A system-generated random number that AWS CodePipeline uses to ensure that the job is being worked on by only one job worker. Get this number from the response of the <a>PollForJobs</a> request that returned this job.<br/>jobId:  The unique system-generated ID of the job for which you want to confirm receipt.<br/>
        :return: status:  Whether the job worker has received the specified job.<br/>        """
        pass

    def acknowledge_third_party_job(self, clientToken: str, nonce: str, jobId: str) -> AcknowledgeThirdPartyJobOutput:
        """Confirms a job worker has received the specified job. Only used for partner actions.

        :param self:
        :param clientToken:  The clientToken portion of the clientId and clientToken pair used to verify that the calling entity is allowed access to the job and its details.<br/>nonce:  A system-generated random number that AWS CodePipeline uses to ensure that the job is being worked on by only one job worker. Get this number from the response to a <a>GetThirdPartyJobDetails</a> request.<br/>jobId:  The unique system-generated ID of the job.<br/>
        :return: status:  The status information for the third party job, if any.<br/>        """
        pass

    def create_custom_action_type(self, outputArtifactDetails: ArtifactDetails, inputArtifactDetails: ArtifactDetails, version: str, provider: str, category: str, settings: ActionTypeSettings=None, configurationProperties: List[ActionConfigurationProperty]=None, tags: List[Tag]=None) -> CreateCustomActionTypeOutput:
        """Creates a new custom action that can be used in all pipelines associated with the AWS account. Only used for custom actions.

        :param self:
        :param tags:  The tags for the custom action.<br/>outputArtifactDetails:  The details of the output artifact of the action, such as its commit ID.:<br/>inputArtifactDetails:  The details of the input artifact for the action, such as its commit ID.:<br/>configurationProperties:  The configuration properties for the custom action. <note> You can refer to a name in the configuration properties of the custom action within the URL templates by following the format of {Config:name}, as long as the configuration property is both required and not secret. For more information, see <a href="https://docs.aws.amazon.com/codepipeline/latest/userguide/how-to-create-custom-action.html">Create a Custom Action for a Pipeline</a>. </note><br/>settings:  URLs that provide users information about this custom action.:<br/>version:  The version identifier of the custom action.<br/>provider:  The provider of the service used in the custom action, such as AWS CodeDeploy.<br/>category:  The category of the custom action, such as a build action or a test action. <note> Although <code>Source</code> and <code>Approval</code> are listed as valid values, they are not currently functional. These values are reserved for future use. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>revisionUrlTemplate</i> The URL returned to the AWS CodePipeline console that contains a link to the page where customers can update or change the configuration of the external action.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>executionUrlTemplate</i> The URL returned to the AWS CodePipeline console that contains a link to the top-level landing page for the external system, such as console page for AWS CodeDeploy. This link is shown on the pipeline view page in the AWS CodePipeline console and provides a link to the execution entity of the external action.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>entityUrlTemplate</i> The URL returned to the AWS CodePipeline console that provides a deep link to the resources of the external system, such as the configuration page for an AWS CodeDeploy deployment group. This link is provided as part of the action display within the pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>thirdPartyConfigurationUrl</i> The URL of a sign-up page where users can sign up for an external service and perform initial configuration of the action provided by that service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>maximumCount</i> The maximum number of artifacts allowed for the action type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>minimumCount</i> The minimum number of artifacts allowed for the action type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>maximumCount</i> The maximum number of artifacts allowed for the action type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>minimumCount</i> The minimum number of artifacts allowed for the action type.<br/>
        :return: tags:  Specifies the tags applied to the custom action.<br/>actionType:  Returns information about the details of an action type.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>outputArtifactDetails</i> The details of the output artifact of the action, such as its commit ID.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>outputArtifactDetails</b>: The details of the output artifact of the action, such as its commit ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>inputArtifactDetails</i> The details of the input artifact for the action, such as its commit ID.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>inputArtifactDetails</b>: The details of the input artifact for the action, such as its commit ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>actionConfigurationProperties</i> The configuration properties for the action type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>settings</i> The settings for the action type.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>settings</b>: The settings for the action type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> Represents information about an action type.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>id</b>: Represents information about an action type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> A string that describes the action version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>provider</i> The provider of the service being called by the action. Valid providers are determined by the action category. For example, an action in the Deploy category type might have a provider of AWS CodeDeploy, which would be specified as CodeDeploy. To reference a list of action providers by action type, see <a href="https://docs.aws.amazon.com/codepipeline/latest/userguide/reference-pipeline-structure.html#actions-valid-providers">Valid Action Types and Providers in CodePipeline</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>owner</i> The creator of the action being called.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>category</i> A category defines what kind of action can be taken in the stage, and constrains the provider type for the action. Valid categories are limited to one of the values below.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>revisionUrlTemplate</i> The URL returned to the AWS CodePipeline console that contains a link to the page where customers can update or change the configuration of the external action.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>executionUrlTemplate</i> The URL returned to the AWS CodePipeline console that contains a link to the top-level landing page for the external system, such as console page for AWS CodeDeploy. This link is shown on the pipeline view page in the AWS CodePipeline console and provides a link to the execution entity of the external action.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>entityUrlTemplate</i> The URL returned to the AWS CodePipeline console that provides a deep link to the resources of the external system, such as the configuration page for an AWS CodeDeploy deployment group. This link is provided as part of the action display within the pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>thirdPartyConfigurationUrl</i> The URL of a sign-up page where users can sign up for an external service and perform initial configuration of the action provided by that service.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>maximumCount</i> The maximum number of artifacts allowed for the action type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>minimumCount</i> The minimum number of artifacts allowed for the action type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>maximumCount</i> The maximum number of artifacts allowed for the action type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>minimumCount</i> The minimum number of artifacts allowed for the action type.<br/>        """
        pass

    def create_pipeline(self, pipeline: PipelineDeclaration, tags: List[Tag]=None) -> CreatePipelineOutput:
        """Creates a pipeline.

        :param self:
        :param tags:  The tags for the pipeline.<br/>pipeline:  Represents the structure of actions and stages to be performed in the pipeline. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version number of the pipeline. A new pipeline always has a version number of 1. This number is automatically incremented when a pipeline is updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stages</i> The stage in which to perform the action.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>artifactStores</i> A mapping of <code>artifactStore</code> objects and their corresponding regions. There must be an artifact store for the pipeline region and for each cross-region action within the pipeline. You can only use either <code>artifactStore</code> or <code>artifactStores</code>, not both. If you create a cross-region action in your pipeline, you must use <code>artifactStores</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>artifactStore</i> Represents information about the Amazon S3 bucket where artifacts are stored for the pipeline. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>artifactStore</b>: Represents information about the Amazon S3 bucket where artifacts are stored for the pipeline. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The Amazon Resource Name (ARN) for AWS CodePipeline to use to either perform actions with no <code>actionRoleArn</code>, or to use to assume roles for actions with an <code>actionRoleArn</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the action to be performed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The Amazon S3 bucket used for storing the artifacts for a pipeline. You can specify the name of an S3 bucket but not a folder within the bucket. A folder to contain the pipeline artifacts is created for you based on the name of the pipeline. You can use any Amazon S3 bucket in the same AWS Region as the pipeline to store your pipeline artifacts.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of the artifact store, such as S3.<br/>The encryption key used to encrypt the data in the artifact store, such as an AWS Key Management Service (AWS KMS) key. If this is undefined, the default key for Amazon S3 is used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>encryptionKey</b>: The encryption key used to encrypt the data in the artifact store, such as an AWS Key Management Service (AWS KMS) key. If this is undefined, the default key for Amazon S3 is used.
        :return: tags:  Specifies the tags applied to the pipeline.<br/>pipeline:  Represents the structure of actions and stages to be performed in the pipeline. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version number of the pipeline. A new pipeline always has a version number of 1. This number is automatically incremented when a pipeline is updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stages</i> The stage in which to perform the action.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>artifactStores</i> A mapping of <code>artifactStore</code> objects and their corresponding regions. There must be an artifact store for the pipeline region and for each cross-region action within the pipeline. You can only use either <code>artifactStore</code> or <code>artifactStores</code>, not both. If you create a cross-region action in your pipeline, you must use <code>artifactStores</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>artifactStore</i> Represents information about the Amazon S3 bucket where artifacts are stored for the pipeline. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>artifactStore</b>: Represents information about the Amazon S3 bucket where artifacts are stored for the pipeline. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The Amazon Resource Name (ARN) for AWS CodePipeline to use to either perform actions with no <code>actionRoleArn</code>, or to use to assume roles for actions with an <code>actionRoleArn</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the action to be performed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The Amazon S3 bucket used for storing the artifacts for a pipeline. You can specify the name of an S3 bucket but not a folder within the bucket. A folder to contain the pipeline artifacts is created for you based on the name of the pipeline. You can use any Amazon S3 bucket in the same AWS Region as the pipeline to store your pipeline artifacts.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of the artifact store, such as S3.<br/>The encryption key used to encrypt the data in the artifact store, such as an AWS Key Management Service (AWS KMS) key. If this is undefined, the default key for Amazon S3 is used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>encryptionKey</b>: The encryption key used to encrypt the data in the artifact store, such as an AWS Key Management Service (AWS KMS) key. If this is undefined, the default key for Amazon S3 is used.        """
        pass

    def delete_custom_action_type(self, version: str, provider: str, category: str):
        """Marks a custom action as deleted. <code>PollForJobs</code> for the custom action will fail after the action is marked for deletion. Only used for custom actions. <important> To re-create a custom action after it has been deleted you must use a string in the version field that has never been used before. This string can be an incremented version number, for example. To restore a deleted custom action, use a JSON file that is identical to the deleted action, including the original string in the version field. </important>

        :param self:
        :param version:  The version of the custom action to delete.<br/>provider:  The provider of the service used in the custom action, such as AWS CodeDeploy.<br/>category:  The category of the custom action that you want to delete, such as source or deploy.<br/>
        :return:         """
        pass

    def delete_pipeline(self, name: str):
        """Deletes the specified pipeline.

        :param self:
        :param name:  The name of the pipeline to be deleted.<br/>
        :return:         """
        pass

    def delete_webhook(self, name: str) -> DeleteWebhookOutput:
        """Deletes a previously created webhook by name. Deleting the webhook stops AWS CodePipeline from starting a pipeline every time an external event occurs. The API will return successfully when trying to delete a webhook that is already deleted. If a deleted webhook is re-created by calling PutWebhook with the same name, it will have a different URL.

        :param self:
        :param name:  The name of the webhook you want to delete.<br/>
        :return:         """
        pass

    def deregister_webhook_with_third_party(self, webhookName: str=None) -> DeregisterWebhookWithThirdPartyOutput:
        """Removes the connection between the webhook that was created by CodePipeline and the external tool with events to be detected. Currently only supported for webhooks that target an action type of GitHub.

        :param self:
        :param webhookName:  The name of the webhook you want to deregister.<br/>
        :return:         """
        pass

    def disable_stage_transition(self, reason: str, transitionType: str, stageName: str, pipelineName: str):
        """Prevents artifacts in a pipeline from transitioning to the next stage in the pipeline.

        :param self:
        :param reason:  The reason given to the user why a stage is disabled, such as waiting for manual approval or manual tests. This message is displayed in the pipeline console UI.<br/>transitionType:  Specifies whether artifacts will be prevented from transitioning into the stage and being processed by the actions in that stage (inbound), or prevented from transitioning from the stage after they have been processed by the actions in that stage (outbound).<br/>stageName:  The name of the stage where you want to disable the inbound or outbound transition of artifacts.<br/>pipelineName:  The name of the pipeline in which you want to disable the flow of artifacts from one stage to another.<br/>
        :return:         """
        pass

    def enable_stage_transition(self, transitionType: str, stageName: str, pipelineName: str):
        """Enables artifacts in a pipeline to transition to a stage in a pipeline.

        :param self:
        :param transitionType:  Specifies whether artifacts will be allowed to enter the stage and be processed by the actions in that stage (inbound) or whether already-processed artifacts will be allowed to transition to the next stage (outbound).<br/>stageName:  The name of the stage where you want to enable the transition of artifacts, either into the stage (inbound) or from that stage to the next stage (outbound).<br/>pipelineName:  The name of the pipeline in which you want to enable the flow of artifacts from one stage to another.<br/>
        :return:         """
        pass

    def get_job_details(self, jobId: str) -> GetJobDetailsOutput:
        """Returns information about a job. Only used for custom actions. <important> When this API is called, AWS CodePipeline returns temporary credentials for the Amazon S3 bucket used to store artifacts for the pipeline, if the action requires access to that Amazon S3 bucket for input or output artifacts. Additionally, this API returns any secret values defined for the action. </important>

        :param self:
        :param jobId:  The unique system-generated ID for the job.<br/>
        :return: jobDetails:  The details of the job. <note> If AWSSessionCredentials is used, a long-running job can call <code>GetJobDetails</code> again to obtain new credentials. </note>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>accountId</i> The AWS account ID associated with the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>data</i> Represents additional information about a job required for a job worker to complete the job. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>data</b>: Represents additional information about a job required for a job worker to complete the job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The unique system-generated ID of the job.<br/>Represents information about an action type.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>actionTypeId</b>: Represents information about an action type.        """
        pass

    def get_pipeline(self, name: str, version: int=None) -> GetPipelineOutput:
        """Returns the metadata, structure, stages, and actions of a pipeline. Can be used to return the entire structure of a pipeline in JSON format, which can then be modified and used to update the pipeline structure with <a>UpdatePipeline</a>.

        :param self:
        :param version:  The version number of the pipeline. If you do not specify a version, defaults to the most current version.<br/>name:  The name of the pipeline for which you want to get information. Pipeline names must be unique under an Amazon Web Services (AWS) user account.<br/>
        :return: metadata:  Represents the pipeline metadata information returned as part of the output of a <code>GetPipeline</code> action.:<br/>pipeline:  Represents the structure of actions and stages to be performed in the pipeline. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version number of the pipeline. A new pipeline always has a version number of 1. This number is automatically incremented when a pipeline is updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stages</i> The stage in which to perform the action.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>artifactStores</i> A mapping of <code>artifactStore</code> objects and their corresponding regions. There must be an artifact store for the pipeline region and for each cross-region action within the pipeline. You can only use either <code>artifactStore</code> or <code>artifactStores</code>, not both. If you create a cross-region action in your pipeline, you must use <code>artifactStores</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>artifactStore</i> Represents information about the Amazon S3 bucket where artifacts are stored for the pipeline. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>artifactStore</b>: Represents information about the Amazon S3 bucket where artifacts are stored for the pipeline. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The Amazon Resource Name (ARN) for AWS CodePipeline to use to either perform actions with no <code>actionRoleArn</code>, or to use to assume roles for actions with an <code>actionRoleArn</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the action to be performed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The Amazon S3 bucket used for storing the artifacts for a pipeline. You can specify the name of an S3 bucket but not a folder within the bucket. A folder to contain the pipeline artifacts is created for you based on the name of the pipeline. You can use any Amazon S3 bucket in the same AWS Region as the pipeline to store your pipeline artifacts.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of the artifact store, such as S3.<br/>The encryption key used to encrypt the data in the artifact store, such as an AWS Key Management Service (AWS KMS) key. If this is undefined, the default key for Amazon S3 is used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>encryptionKey</b>: The encryption key used to encrypt the data in the artifact store, such as an AWS Key Management Service (AWS KMS) key. If this is undefined, the default key for Amazon S3 is used.&nbsp;&nbsp;&nbsp;&nbsp; <i>updated</i> The date and time the pipeline was last updated, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>created</i> The date and time the pipeline was created, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pipelineArn</i> The Amazon Resource Name (ARN) of the pipeline.<br/>        """
        pass

    def get_pipeline_execution(self, pipelineExecutionId: str, pipelineName: str) -> GetPipelineExecutionOutput:
        """Returns information about an execution of a pipeline, including details about artifacts, the pipeline execution ID, and the name, version, and status of the pipeline.

        :param self:
        :param pipelineExecutionId:  The ID of the pipeline execution about which you want to get execution details.<br/>pipelineName:  The name of the pipeline about which you want to get execution details.<br/>
        :return: pipelineExecution:  Represents information about the execution of a pipeline.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>artifactRevisions</i> A list of <code>ArtifactRevision</code> objects included in a pipeline execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the pipeline execution. <ul> <li> InProgress: The pipeline execution is currently running. </li> <li> Succeeded: The pipeline execution was completed successfully.  </li> <li> Superseded: While this pipeline execution was waiting for the next stage to be completed, a newer pipeline execution advanced and continued through the pipeline instead.  </li> <li> Failed: The pipeline execution was not completed successfully. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pipelineExecutionId</i> The ID of the pipeline execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pipelineVersion</i> The version number of the pipeline that was executed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pipelineName</i> The name of the pipeline that was executed.<br/>        """
        pass

    def get_pipeline_state(self, name: str) -> GetPipelineStateOutput:
        """Returns information about the state of a pipeline, including the stages and actions. <note> Values returned in the <code>revisionId</code> and <code>revisionUrl</code> fields indicate the source revision information, such as the commit ID, for the current state. </note>

        :param self:
        :param name:  The name of the pipeline about which you want to get information.<br/>
        :return: updated:  The date and time the pipeline was last updated, in timestamp format.<br/>created:  The date and time the pipeline was created, in timestamp format.<br/>stageStates:  A list of the pipeline stage output information, including stage name, state, most recent run details, whether the stage is disabled, and other data.<br/>pipelineVersion:  The version number of the pipeline. <note> A newly-created pipeline is always assigned a version number of <code>1</code>. </note><br/>pipelineName:  The name of the pipeline for which you want to get the state.<br/>        """
        pass

    def get_third_party_job_details(self, clientToken: str, jobId: str) -> GetThirdPartyJobDetailsOutput:
        """Requests the details of a job for a third party action. Only used for partner actions. <important> When this API is called, AWS CodePipeline returns temporary credentials for the Amazon S3 bucket used to store artifacts for the pipeline, if the action requires access to that Amazon S3 bucket for input or output artifacts. Additionally, this API returns any secret values defined for the action. </important>

        :param self:
        :param clientToken:  The clientToken portion of the clientId and clientToken pair used to verify that the calling entity is allowed access to the job and its details.<br/>jobId:  The unique system-generated ID used for identifying the job.<br/>
        :return: jobDetails:  The details of the job, including any protected values defined for the job.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>nonce</i> A system-generated random number that AWS CodePipeline uses to ensure that the job is being worked on by only one job worker. Use this number in an <a>AcknowledgeThirdPartyJob</a> request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>data</i> The data to be returned by the third party job worker.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>data</b>: The data to be returned by the third party job worker.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The identifier used to identify the job details in AWS CodePipeline.<br/>Represents information about an action type.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>actionTypeId</b>: Represents information about an action type.        """
        pass

    def list_action_executions(self, pipelineName: str, filter: ActionExecutionFilter=None, maxResults: int=None, nextToken: str=None) -> ListActionExecutionsOutput:
        """Lists the action executions that have occurred in a pipeline.

        :param self:
        :param nextToken:  The token that was returned from the previous <code>ListActionExecutions</code> call, which can be used to return the next set of action executions in the list.<br/>maxResults:  The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned nextToken value. Action execution history is retained for up to 12 months, based on action execution start times. Default value is 100.  <note> Detailed execution history is available for executions run on or after February 21, 2019. </note><br/>filter:  Input information used to filter action execution history.:<br/>pipelineName:   The name of the pipeline for which you want to list action execution history.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pipelineExecutionId</i> The pipeline execution ID used to filter action execution history.<br/>
        :return: nextToken:  If the amount of returned information is significantly large, an identifier is also returned and can be used in a subsequent <code>ListActionExecutions</code> call to return the next set of action executions in the list.<br/>actionExecutionDetails:  The details for a list of recent executions, such as action execution ID.<br/>        """
        pass

    def list_action_types(self, actionOwnerFilter: str=None, nextToken: str=None) -> ListActionTypesOutput:
        """Gets a summary of all AWS CodePipeline action types associated with your account.

        :param self:
        :param nextToken:  An identifier that was returned from the previous list action types call, which can be used to return the next set of action types in the list.<br/>actionOwnerFilter:  Filters the list of action types to those created by a specified entity.<br/>
        :return: nextToken:  If the amount of returned information is significantly large, an identifier is also returned which can be used in a subsequent list action types call to return the next set of action types in the list.<br/>actionTypes:  Provides details of the action types.<br/>        """
        pass

    def list_pipeline_executions(self, pipelineName: str, maxResults: int=None, nextToken: str=None) -> ListPipelineExecutionsOutput:
        """Gets a summary of the most recent executions for a pipeline.

        :param self:
        :param nextToken:  The token that was returned from the previous <code>ListPipelineExecutions</code> call, which can be used to return the next set of pipeline executions in the list.<br/>maxResults:  The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned nextToken value. Pipeline history is limited to the most recent 12 months, based on pipeline execution start times. Default value is 100.<br/>pipelineName:  The name of the pipeline for which you want to get execution summary information.<br/>
        :return: nextToken:  A token that can be used in the next <code>ListPipelineExecutions</code> call. To view all items in the list, continue to call this operation with each subsequent token until no more nextToken values are returned.<br/>pipelineExecutionSummaries:  A list of executions in the history of a pipeline.<br/>        """
        pass

    def list_pipelines(self, nextToken: str=None) -> ListPipelinesOutput:
        """Gets a summary of all of the pipelines associated with your account.

        :param self:
        :param nextToken:  An identifier that was returned from the previous list pipelines call, which can be used to return the next set of pipelines in the list.<br/>
        :return: nextToken:  If the amount of returned information is significantly large, an identifier is also returned which can be used in a subsequent list pipelines call to return the next set of pipelines in the list.<br/>pipelines:  The list of pipelines.<br/>        """
        pass

    def list_tags_for_resource(self, resourceArn: str, nextToken: str=None, maxResults: int=None) -> ListTagsForResourceOutput:
        """Gets the set of key/value pairs (metadata) that are used to manage the resource.

        :param self:
        :param maxResults:  The maximum number of results to return in a single call.<br/>nextToken:  The token that was returned from the previous API call, which would be used to return the next page of the list. However, the ListTagsforResource call lists all available tags in one call and does not use pagination.<br/>resourceArn:  The Amazon Resource Name (ARN) of the resource to get tags for.<br/>
        :return: nextToken:  If the amount of returned information is significantly large, an identifier is also returned and can be used in a subsequent API call to return the next page of the list. However, the ListTagsforResource call lists all available tags in one call and does not use pagination.<br/>tags:  The tags for the resource.<br/>        """
        pass

    def list_webhooks(self, NextToken: str=None, MaxResults: int=None) -> ListWebhooksOutput:
        """Gets a listing of all the webhooks in this region for this account. The output lists all webhooks and includes the webhook URL and ARN, as well the configuration for each webhook.

        :param self:
        :param MaxResults:  The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned nextToken value.<br/>NextToken:  The token that was returned from the previous ListWebhooks call, which can be used to return the next set of webhooks in the list.<br/>
        :return: NextToken:  If the amount of returned information is significantly large, an identifier is also returned and can be used in a subsequent ListWebhooks call to return the next set of webhooks in the list. <br/>webhooks:  The JSON detail returned for each webhook in the list output for the ListWebhooks call.<br/>        """
        pass

    def poll_for_jobs(self, actionTypeId: ActionTypeId, maxBatchSize: int=None, queryParam: QueryParamMap=None) -> PollForJobsOutput:
        """Returns information about any jobs for AWS CodePipeline to act upon. <code>PollForJobs</code> is only valid for action types with "Custom" in the owner field. If the action type contains "AWS" or "ThirdParty" in the owner field, the <code>PollForJobs</code> action returns an error. <important> When this API is called, AWS CodePipeline returns temporary credentials for the Amazon S3 bucket used to store artifacts for the pipeline, if the action requires access to that Amazon S3 bucket for input or output artifacts. Additionally, this API returns any secret values defined for the action. </important>

        :param self:
        :param queryParam:  A map of property names and values. For an action type with no queryable properties, this value must be null or an empty map. For an action type with a queryable property, you must supply that property as a key in the map. Only jobs whose action configuration matches the mapped value will be returned.<br/>maxBatchSize:  The maximum number of jobs to return in a poll for jobs call.<br/>actionTypeId:  Represents information about an action type.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> A string that describes the action version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>provider</i> The provider of the service being called by the action. Valid providers are determined by the action category. For example, an action in the Deploy category type might have a provider of AWS CodeDeploy, which would be specified as CodeDeploy. To reference a list of action providers by action type, see <a href="https://docs.aws.amazon.com/codepipeline/latest/userguide/reference-pipeline-structure.html#actions-valid-providers">Valid Action Types and Providers in CodePipeline</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>owner</i> The creator of the action being called.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>category</i> A category defines what kind of action can be taken in the stage, and constrains the provider type for the action. Valid categories are limited to one of the values below.<br/>
        :return: jobs:  Information about the jobs to take action on.<br/>        """
        pass

    def poll_for_third_party_jobs(self, actionTypeId: ActionTypeId, maxBatchSize: int=None) -> PollForThirdPartyJobsOutput:
        """Determines whether there are any third party jobs for a job worker to act on. Only used for partner actions. <important> When this API is called, AWS CodePipeline returns temporary credentials for the Amazon S3 bucket used to store artifacts for the pipeline, if the action requires access to that Amazon S3 bucket for input or output artifacts. </important>

        :param self:
        :param maxBatchSize:  The maximum number of jobs to return in a poll for jobs call.<br/>actionTypeId:  Represents information about an action type.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> A string that describes the action version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>provider</i> The provider of the service being called by the action. Valid providers are determined by the action category. For example, an action in the Deploy category type might have a provider of AWS CodeDeploy, which would be specified as CodeDeploy. To reference a list of action providers by action type, see <a href="https://docs.aws.amazon.com/codepipeline/latest/userguide/reference-pipeline-structure.html#actions-valid-providers">Valid Action Types and Providers in CodePipeline</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>owner</i> The creator of the action being called.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>category</i> A category defines what kind of action can be taken in the stage, and constrains the provider type for the action. Valid categories are limited to one of the values below.<br/>
        :return: jobs:  Information about the jobs to take action on.<br/>        """
        pass

    def put_action_revision(self, actionRevision: ActionRevision, actionName: str, stageName: str, pipelineName: str) -> PutActionRevisionOutput:
        """Provides information to AWS CodePipeline about new revisions to a source.

        :param self:
        :param actionRevision:  Represents information about the version (or revision) of an action.:<br/>actionName:  The name of the action that will process the revision.<br/>stageName:  The name of the stage that contains the action that will act upon the revision.<br/>pipelineName:  The name of the pipeline that will start processing the revision to the source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>created</i> The date and time when the most recent version of the action was created, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>revisionChangeId</i> The unique identifier of the change that set the state to this revision, for example a deployment ID or timestamp.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>revisionId</i> The system-generated unique ID that identifies the revision number of the action.<br/>
        :return: pipelineExecutionId:  The ID of the current workflow state of the pipeline.<br/>newRevision:  Indicates whether the artifact revision was previously used in an execution of the specified pipeline.<br/>        """
        pass

    def put_approval_result(self, token: str, result: ApprovalResult, actionName: str, stageName: str, pipelineName: str) -> PutApprovalResultOutput:
        """Provides the response to a manual approval request to AWS CodePipeline. Valid responses include Approved and Rejected.

        :param self:
        :param token:  The system-generated token used to identify a unique approval request. The token for each open approval request can be obtained using the <a>GetPipelineState</a> action and is used to validate that the approval request corresponding to this token is still valid.<br/>result:  Represents information about the result of the approval request.:<br/>actionName:  The name of the action for which approval is requested.<br/>stageName:  The name of the stage that contains the action.<br/>pipelineName:  The name of the pipeline that contains the action. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The response submitted by a reviewer assigned to an approval action request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>summary</i> The summary of the current status of the approval request.<br/>
        :return: approvedAt:  The timestamp showing when the approval or rejection was submitted.<br/>        """
        pass

    def put_job_failure_result(self, failureDetails: FailureDetails, jobId: str):
        """Represents the failure of a job as returned to the pipeline by a job worker. Only used for custom actions.

        :param self:
        :param failureDetails:  The details about the failure of a job.:<br/>jobId:  The unique system-generated ID of the job that failed. This is the same ID returned from <code>PollForJobs</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>externalExecutionId</i> The external ID of the run of the action that failed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>message</i> The message about the failure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of the failure.<br/>
        :return:         """
        pass

    def put_job_success_result(self, jobId: str, currentRevision: CurrentRevision=None, continuationToken: str=None, executionDetails: ExecutionDetails=None):
        """Represents the success of a job as returned to the pipeline by a job worker. Only used for custom actions.

        :param self:
        :param executionDetails:  The execution details of the successful job, such as the actions taken by the job worker.:<br/>continuationToken:  A token generated by a job worker, such as an AWS CodeDeploy deployment ID, that a successful job provides to identify a custom action in progress. Future jobs will use this token in order to identify the running instance of the action. It can be reused to return additional information about the progress of the custom action. When the action is complete, no continuation token should be supplied.<br/>currentRevision:  The ID of the current revision of the artifact successfully worked upon by the job.:<br/>jobId:  The unique system-generated ID of the job that succeeded. This is the same ID returned from <code>PollForJobs</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>revisionSummary</i> The summary of the most recent revision of the artifact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>created</i> The date and time when the most recent revision of the artifact was created, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>changeIdentifier</i> The change identifier for the current revision.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>revision</i> The revision ID of the current version of an artifact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>percentComplete</i> The percentage of work completed on the action, represented on a scale of zero to one hundred percent.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>externalExecutionId</i> The system-generated unique ID of this action used to identify this job worker in any external systems, such as AWS CodeDeploy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>summary</i> The summary of the current status of the actions.<br/>
        :return:         """
        pass

    def put_third_party_job_failure_result(self, failureDetails: FailureDetails, clientToken: str, jobId: str):
        """Represents the failure of a third party job as returned to the pipeline by a job worker. Only used for partner actions.

        :param self:
        :param failureDetails:  Represents information about failure details.:<br/>clientToken:  The clientToken portion of the clientId and clientToken pair used to verify that the calling entity is allowed access to the job and its details.<br/>jobId:  The ID of the job that failed. This is the same ID returned from <code>PollForThirdPartyJobs</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>externalExecutionId</i> The external ID of the run of the action that failed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>message</i> The message about the failure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of the failure.<br/>
        :return:         """
        pass

    def put_third_party_job_success_result(self, clientToken: str, jobId: str, currentRevision: CurrentRevision=None, continuationToken: str=None, executionDetails: ExecutionDetails=None):
        """Represents the success of a third party job as returned to the pipeline by a job worker. Only used for partner actions.

        :param self:
        :param executionDetails:  The details of the actions taken and results produced on an artifact as it passes through stages in the pipeline. :<br/>continuationToken:  A token generated by a job worker, such as an AWS CodeDeploy deployment ID, that a successful job provides to identify a partner action in progress. Future jobs will use this token in order to identify the running instance of the action. It can be reused to return additional information about the progress of the partner action. When the action is complete, no continuation token should be supplied.<br/>currentRevision:  Represents information about a current revision.:<br/>clientToken:  The clientToken portion of the clientId and clientToken pair used to verify that the calling entity is allowed access to the job and its details.<br/>jobId:  The ID of the job that successfully completed. This is the same ID returned from <code>PollForThirdPartyJobs</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>revisionSummary</i> The summary of the most recent revision of the artifact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>created</i> The date and time when the most recent revision of the artifact was created, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>changeIdentifier</i> The change identifier for the current revision.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>revision</i> The revision ID of the current version of an artifact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>percentComplete</i> The percentage of work completed on the action, represented on a scale of zero to one hundred percent.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>externalExecutionId</i> The system-generated unique ID of this action used to identify this job worker in any external systems, such as AWS CodeDeploy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>summary</i> The summary of the current status of the actions.<br/>
        :return:         """
        pass

    def put_webhook(self, webhook: WebhookDefinition, tags: List[Tag]=None) -> PutWebhookOutput:
        """Defines a webhook and returns a unique webhook URL generated by CodePipeline. This URL can be supplied to third party source hosting providers to call every time there's a code change. When CodePipeline receives a POST request on this URL, the pipeline defined in the webhook is started as long as the POST request satisfied the authentication and filtering requirements supplied when defining the webhook. RegisterWebhookWithThirdParty and DeregisterWebhookWithThirdParty APIs can be used to automatically configure supported third parties to call the generated webhook URL.

        :param self:
        :param tags:  The tags for the webhook.<br/>webhook:  The detail provided in an input file to create the webhook, such as the webhook name, the pipeline name, and the action name. Give the webhook a unique name which identifies the webhook being defined. You may choose to name the webhook after the pipeline and action it targets so that you can easily recognize what it's used for later.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authenticationConfiguration</i> Properties that configure the authentication applied to incoming webhook trigger requests. The required properties depend on the authentication type. For GITHUB_HMAC, only the <code>SecretToken </code>property must be set. For IP, only the <code>AllowedIPRange </code>property must be set to a valid CIDR range. For UNAUTHENTICATED, no properties can be set.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>authenticationConfiguration</b>: Properties that configure the authentication applied to incoming webhook trigger requests. The required properties depend on the authentication type. For GITHUB_HMAC, only the <code>SecretToken </code>property must be set. For IP, only the <code>AllowedIPRange </code>property must be set to a valid CIDR range. For UNAUTHENTICATED, no properties can be set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authentication</i> Supported options are GITHUB_HMAC, IP and UNAUTHENTICATED. <ul> <li> For information about the authentication scheme implemented by GITHUB_HMAC, see <a href="https://developer.github.com/webhooks/securing/">Securing your webhooks</a> on the GitHub Developer website. </li> <li>  IP will reject webhooks trigger requests unless they originate from an IP within the IP range whitelisted in the authentication configuration. </li> <li>  UNAUTHENTICATED will accept all webhook trigger requests regardless of origin. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>filters</i> A list of rules applied to the body/payload sent in the POST request to a webhook URL. All defined rules must pass for the request to be accepted and the pipeline started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>targetAction</i> The name of the action in a pipeline you want to connect to the webhook. The action must be from the source (first) stage of the pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>targetPipeline</i> The name of the pipeline you want to connect to the webhook.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the webhook.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecretToken</i> The property used to configure GitHub authentication. For GITHUB_HMAC, only the <code>SecretToken</code> property must be set.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowedIPRange</i> The property used to configure acceptance of webhooks within a specific IP range. For IP, only the <code>AllowedIPRange</code> property must be set, and this property must be set to a valid CIDR range.<br/>
        :return: webhook:  The detail returned from creating the webhook, such as the webhook name, webhook URL, and webhook ARN.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> Specifies the tags applied to the webhook.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the webhook.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastTriggered</i> The date and time a webhook was last successfully triggered, in timestamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorCode</i> The number code of the error.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorMessage</i> The text of the error message about the webhook.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>url</i> A unique URL generated by CodePipeline. When a POST request is made to this URL, the defined pipeline is started as long as the body of the post request satisfies the defined authentication and filtering conditions. Deleting and re-creating a webhook will make the old URL invalid and generate a new URL.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>definition</i> The detail returned for each webhook, such as the webhook authentication type and filter rules.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>definition</b>: The detail returned for each webhook, such as the webhook authentication type and filter rules.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>authentication</i> Supported options are GITHUB_HMAC, IP and UNAUTHENTICATED. <ul> <li> For information about the authentication scheme implemented by GITHUB_HMAC, see <a href="https://developer.github.com/webhooks/securing/">Securing your webhooks</a> on the GitHub Developer website. </li> <li>  IP will reject webhooks trigger requests unless they originate from an IP within the IP range whitelisted in the authentication configuration. </li> <li>  UNAUTHENTICATED will accept all webhook trigger requests regardless of origin. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>filters</i> A list of rules applied to the body/payload sent in the POST request to a webhook URL. All defined rules must pass for the request to be accepted and the pipeline started.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>targetAction</i> The name of the action in a pipeline you want to connect to the webhook. The action must be from the source (first) stage of the pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>targetPipeline</i> The name of the pipeline you want to connect to the webhook.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the webhook.<br/>Properties that configure the authentication applied to incoming webhook trigger requests. The required properties depend on the authentication type. For GITHUB_HMAC, only the <code>SecretToken </code>property must be set. For IP, only the <code>AllowedIPRange </code>property must be set to a valid CIDR range. For UNAUTHENTICATED, no properties can be set.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>authenticationConfiguration</b>: Properties that configure the authentication applied to incoming webhook trigger requests. The required properties depend on the authentication type. For GITHUB_HMAC, only the <code>SecretToken </code>property must be set. For IP, only the <code>AllowedIPRange </code>property must be set to a valid CIDR range. For UNAUTHENTICATED, no properties can be set.        """
        pass

    def register_webhook_with_third_party(self, webhookName: str=None) -> RegisterWebhookWithThirdPartyOutput:
        """Configures a connection between the webhook that was created and the external tool with events to be detected.

        :param self:
        :param webhookName:  The name of an existing webhook created with PutWebhook to register with a supported third party. <br/>
        :return:         """
        pass

    def retry_stage_execution(self, retryMode: str, pipelineExecutionId: str, stageName: str, pipelineName: str) -> RetryStageExecutionOutput:
        """Resumes the pipeline execution by retrying the last failed actions in a stage.

        :param self:
        :param retryMode:  The scope of the retry attempt. Currently, the only supported value is FAILED_ACTIONS.<br/>pipelineExecutionId:  The ID of the pipeline execution in the failed stage to be retried. Use the <a>GetPipelineState</a> action to retrieve the current pipelineExecutionId of the failed stage<br/>stageName:  The name of the failed stage to be retried.<br/>pipelineName:  The name of the pipeline that contains the failed stage.<br/>
        :return: pipelineExecutionId:  The ID of the current workflow execution in the failed stage.<br/>        """
        pass

    def start_pipeline_execution(self, name: str, clientRequestToken: str=None) -> StartPipelineExecutionOutput:
        """Starts the specified pipeline. Specifically, it begins processing the latest commit to the source location specified as part of the pipeline.

        :param self:
        :param clientRequestToken:  The system-generated unique ID used to identify a unique execution request.<br/>name:  The name of the pipeline to start.<br/>
        :return: pipelineExecutionId:  The unique system-generated ID of the pipeline execution that was started.<br/>        """
        pass

    def tag_resource(self, tags: List[Tag], resourceArn: str) -> TagResourceOutput:
        """Adds to or modifies the tags of the given resource. Tags are metadata that can be used to manage a resource. 

        :param self:
        :param tags:  The tags you want to modify or add to the resource.<br/>resourceArn:  The Amazon Resource Name (ARN) of the resource you want to add tags to.<br/>
        :return:         """
        pass

    def untag_resource(self, tagKeys: List[TagKey], resourceArn: str) -> UntagResourceOutput:
        """Removes tags from an AWS resource.

        :param self:
        :param tagKeys:  The list of keys for the tags to be removed from the resource.<br/>resourceArn:   The Amazon Resource Name (ARN) of the resource to remove tags from.<br/>
        :return:         """
        pass

    def update_pipeline(self, pipeline: PipelineDeclaration) -> UpdatePipelineOutput:
        """Updates a specified pipeline with edits or changes to its structure. Use a JSON file with the pipeline structure in conjunction with <code>UpdatePipeline</code> to provide the full structure of the pipeline. Updating the pipeline increases the version number of the pipeline by 1.

        :param self:
        :param pipeline:  The name of the pipeline to be updated.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version number of the pipeline. A new pipeline always has a version number of 1. This number is automatically incremented when a pipeline is updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stages</i> The stage in which to perform the action.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>artifactStores</i> A mapping of <code>artifactStore</code> objects and their corresponding regions. There must be an artifact store for the pipeline region and for each cross-region action within the pipeline. You can only use either <code>artifactStore</code> or <code>artifactStores</code>, not both. If you create a cross-region action in your pipeline, you must use <code>artifactStores</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>artifactStore</i> Represents information about the Amazon S3 bucket where artifacts are stored for the pipeline. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>artifactStore</b>: Represents information about the Amazon S3 bucket where artifacts are stored for the pipeline. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The Amazon Resource Name (ARN) for AWS CodePipeline to use to either perform actions with no <code>actionRoleArn</code>, or to use to assume roles for actions with an <code>actionRoleArn</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the action to be performed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The Amazon S3 bucket used for storing the artifacts for a pipeline. You can specify the name of an S3 bucket but not a folder within the bucket. A folder to contain the pipeline artifacts is created for you based on the name of the pipeline. You can use any Amazon S3 bucket in the same AWS Region as the pipeline to store your pipeline artifacts.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of the artifact store, such as S3.<br/>The encryption key used to encrypt the data in the artifact store, such as an AWS Key Management Service (AWS KMS) key. If this is undefined, the default key for Amazon S3 is used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>encryptionKey</b>: The encryption key used to encrypt the data in the artifact store, such as an AWS Key Management Service (AWS KMS) key. If this is undefined, the default key for Amazon S3 is used.
        :return: pipeline:  The structure of the updated pipeline.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version number of the pipeline. A new pipeline always has a version number of 1. This number is automatically incremented when a pipeline is updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stages</i> The stage in which to perform the action.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>artifactStores</i> A mapping of <code>artifactStore</code> objects and their corresponding regions. There must be an artifact store for the pipeline region and for each cross-region action within the pipeline. You can only use either <code>artifactStore</code> or <code>artifactStores</code>, not both. If you create a cross-region action in your pipeline, you must use <code>artifactStores</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>artifactStore</i> Represents information about the Amazon S3 bucket where artifacts are stored for the pipeline. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>artifactStore</b>: Represents information about the Amazon S3 bucket where artifacts are stored for the pipeline. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The Amazon Resource Name (ARN) for AWS CodePipeline to use to either perform actions with no <code>actionRoleArn</code>, or to use to assume roles for actions with an <code>actionRoleArn</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the action to be performed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The Amazon S3 bucket used for storing the artifacts for a pipeline. You can specify the name of an S3 bucket but not a folder within the bucket. A folder to contain the pipeline artifacts is created for you based on the name of the pipeline. You can use any Amazon S3 bucket in the same AWS Region as the pipeline to store your pipeline artifacts.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of the artifact store, such as S3.<br/>The encryption key used to encrypt the data in the artifact store, such as an AWS Key Management Service (AWS KMS) key. If this is undefined, the default key for Amazon S3 is used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>encryptionKey</b>: The encryption key used to encrypt the data in the artifact store, such as an AWS Key Management Service (AWS KMS) key. If this is undefined, the default key for Amazon S3 is used.        """
        pass



class CodeStar(BaseClient):
    """<fullname>AWS CodeStar</fullname> This is the API reference for AWS CodeStar. This reference provides descriptions of the operations and data types for the AWS CodeStar API along with usage examples. You can use the AWS CodeStar API to work with: Projects and their resources, by calling the following: <ul> <li>  <code>DeleteProject</code>, which deletes a project. </li> <li>  <code>DescribeProject</code>, which lists the attributes of a project. </li> <li>  <code>ListProjects</code>, which lists all projects associated with your AWS account. </li> <li>  <code>ListResources</code>, which lists the resources associated with a project. </li> <li>  <code>ListTagsForProject</code>, which lists the tags associated with a project. </li> <li>  <code>TagProject</code>, which adds tags to a project. </li> <li>  <code>UntagProject</code>, which removes tags from a project. </li> <li>  <code>UpdateProject</code>, which updates the attributes of a project. </li> </ul> Teams and team members, by calling the following: <ul> <li>  <code>AssociateTeamMember</code>, which adds an IAM user to the team for a project. </li> <li>  <code>DisassociateTeamMember</code>, which removes an IAM user from the team for a project. </li> <li>  <code>ListTeamMembers</code>, which lists all the IAM users in the team for a project, including their roles and attributes. </li> <li>  <code>UpdateTeamMember</code>, which updates a team member's attributes in a project. </li> </ul> Users, by calling the following: <ul> <li>  <code>CreateUserProfile</code>, which creates a user profile that contains data associated with the user across all projects. </li> <li>  <code>DeleteUserProfile</code>, which deletes all user profile information across all projects. </li> <li>  <code>DescribeUserProfile</code>, which describes the profile of a user. </li> <li>  <code>ListUserProfiles</code>, which lists all user profiles. </li> <li>  <code>UpdateUserProfile</code>, which updates the profile for a user. </li> </ul>"""


    class ListProjectsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_projects` for valid parameters"""
            pass
    class ListResourcesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_resources` for valid parameters"""
            pass
    class ListTeamMembersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_team_members` for valid parameters"""
            pass
    class ListUserProfilesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_user_profiles` for valid parameters"""
            pass


    class AssociateTeamMemberRequest(Mapping):
        pass
    
    class AssociateTeamMemberResult(Mapping):
        pass
    
    class RemoteAccessAllowed(object):
        pass
    
    class CreateProjectRequest(Mapping):
        pass
    
    class CreateProjectResult(Mapping):
        pass
    
    class Code(Mapping):
        pass
    
    class Toolchain(Mapping):
        pass
    
    class Tags(object):
        pass
    
    class CreateUserProfileRequest(Mapping):
        pass
    
    class CreateUserProfileResult(Mapping):
        pass
    
    class DeleteProjectRequest(Mapping):
        pass
    
    class DeleteProjectResult(Mapping):
        pass
    
    class DeleteStack(object):
        pass
    
    class DeleteUserProfileRequest(Mapping):
        pass
    
    class DeleteUserProfileResult(Mapping):
        pass
    
    class DescribeProjectRequest(Mapping):
        pass
    
    class DescribeProjectResult(Mapping):
        pass
    
    class DescribeUserProfileRequest(Mapping):
        pass
    
    class DescribeUserProfileResult(Mapping):
        pass
    
    class DisassociateTeamMemberRequest(Mapping):
        pass
    
    class DisassociateTeamMemberResult(Mapping):
        pass
    
    class ListProjectsRequest(Mapping):
        pass
    
    class ListProjectsResult(Mapping):
        pass
    
    class ListResourcesRequest(Mapping):
        pass
    
    class ListResourcesResult(Mapping):
        pass
    
    class ListTagsForProjectRequest(Mapping):
        pass
    
    class ListTagsForProjectResult(Mapping):
        pass
    
    class ListTeamMembersRequest(Mapping):
        pass
    
    class ListTeamMembersResult(Mapping):
        pass
    
    class ListUserProfilesRequest(Mapping):
        pass
    
    class ListUserProfilesResult(Mapping):
        pass
    
    class TagProjectRequest(Mapping):
        pass
    
    class TagProjectResult(Mapping):
        pass
    
    class UntagProjectRequest(Mapping):
        pass
    
    class UntagProjectResult(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateProjectRequest(Mapping):
        pass
    
    class UpdateProjectResult(Mapping):
        pass
    
    class UpdateTeamMemberRequest(Mapping):
        pass
    
    class UpdateTeamMemberResult(Mapping):
        pass
    
    class UpdateUserProfileRequest(Mapping):
        pass
    
    class UpdateUserProfileResult(Mapping):
        pass
    
    def associate_team_member(self, projectRole: str, userArn: str, projectId: str, clientRequestToken: str=None, remoteAccessAllowed: RemoteAccessAllowed=None) -> AssociateTeamMemberResult:
        """Adds an IAM user to the team for an AWS CodeStar project.

        :param self:
        :param remoteAccessAllowed:  Whether the team member is allowed to use an SSH public/private key pair to remotely access project resources, for example Amazon EC2 instances.<br/>projectRole:  The AWS CodeStar project role that will apply to this user. This role determines what actions a user can take in an AWS CodeStar project.<br/>userArn:  The Amazon Resource Name (ARN) for the IAM user you want to add to the AWS CodeStar project.<br/>clientRequestToken:  A user- or system-generated token that identifies the entity that requested the team member association to the project. This token can be used to repeat the request.<br/>projectId:  The ID of the project to which you will add the IAM user.<br/>
        :return: clientRequestToken:  The user- or system-generated token from the initial request that can be used to repeat the request.<br/>        """
        pass

    def create_project(self, id: str, name: str, description: str=None, clientRequestToken: str=None, sourceCode: List[Code]=None, toolchain: Toolchain=None, tags: Tags=None) -> CreateProjectResult:
        """Creates a project, including project resources. This action creates a project based on a submitted project request. A set of source code files and a toolchain template file can be included with the project request. If these are not provided, an empty project is created.

        :param self:
        :param tags:  The tags created for the project.<br/>toolchain:  The name of the toolchain template file submitted with the project request. If this parameter is specified, the request must also include the sourceCode parameter.:<br/>sourceCode:  A list of the Code objects submitted with the project request. If this parameter is specified, the request must also include the toolchain parameter.<br/>clientRequestToken:  A user- or system-generated token that identifies the entity that requested project creation. This token can be used to repeat the request.<br/>description:  The description of the project, if any.<br/>id:  The ID of the project to be created in AWS CodeStar.<br/>name:  The display name for the project to be created in AWS CodeStar.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stackParameters</i> The list of parameter overrides to be passed into the toolchain template during stack provisioning, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The service role ARN for AWS CodeStar to use for the toolchain template during stack provisioning.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>source</i> The Amazon S3 location where the toolchain template file provided with the project request is stored. AWS CodeStar retrieves the file during project creation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>source</b>: The Amazon S3 location where the toolchain template file provided with the project request is stored. AWS CodeStar retrieves the file during project creation.<br/>The Amazon S3 bucket where the toolchain template file provided with the project request is stored.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>s3</b>: The Amazon S3 bucket where the toolchain template file provided with the project request is stored.
        :return: projectTemplateId:  Reserved for future use.<br/>clientRequestToken:  A user- or system-generated token that identifies the entity that requested project creation.<br/>arn:  The Amazon Resource Name (ARN) of the created project.<br/>id:  The ID of the project.<br/>        """
        pass

    def create_user_profile(self, emailAddress: str, displayName: str, userArn: str, sshPublicKey: str=None) -> CreateUserProfileResult:
        """Creates a profile for a user that includes user preferences, such as the display name and email address assocciated with the user, in AWS CodeStar. The user profile is not project-specific. Information in the user profile is displayed wherever the user's information appears to other users in AWS CodeStar.

        :param self:
        :param sshPublicKey:  The SSH public key associated with the user in AWS CodeStar. If a project owner allows the user remote access to project resources, this public key will be used along with the user's private key for SSH access.<br/>emailAddress:  The email address that will be displayed as part of the user's profile in AWS CodeStar.<br/>displayName:  The name that will be displayed as the friendly name for the user in AWS CodeStar. <br/>userArn:  The Amazon Resource Name (ARN) of the user in IAM.<br/>
        :return: lastModifiedTimestamp:  The date the user profile was last modified, in timestamp format.<br/>createdTimestamp:  The date the user profile was created, in timestamp format.<br/>sshPublicKey:  The SSH public key associated with the user in AWS CodeStar. This is the public portion of the public/private keypair the user can use to access project resources if a project owner allows the user remote access to those resources.<br/>emailAddress:  The email address that is displayed as part of the user's profile in AWS CodeStar.<br/>displayName:  The name that is displayed as the friendly name for the user in AWS CodeStar.<br/>userArn:  The Amazon Resource Name (ARN) of the user in IAM.<br/>        """
        pass

    def delete_project(self, id: str, clientRequestToken: str=None, deleteStack: DeleteStack=None) -> DeleteProjectResult:
        """Deletes a project, including project resources. Does not delete users associated with the project, but does delete the IAM roles that allowed access to the project.

        :param self:
        :param deleteStack:  Whether to send a delete request for the primary stack in AWS CloudFormation originally used to generate the project and its resources. This option will delete all AWS resources for the project (except for any buckets in Amazon S3) as well as deleting the project itself. Recommended for most use cases.<br/>clientRequestToken:  A user- or system-generated token that identifies the entity that requested project deletion. This token can be used to repeat the request. <br/>id:  The ID of the project to be deleted in AWS CodeStar.<br/>
        :return: projectArn:  The Amazon Resource Name (ARN) of the deleted project.<br/>stackId:  The ID of the primary stack in AWS CloudFormation that will be deleted as part of deleting the project and its resources.<br/>        """
        pass

    def delete_user_profile(self, userArn: str) -> DeleteUserProfileResult:
        """Deletes a user profile in AWS CodeStar, including all personal preference data associated with that profile, such as display name and email address. It does not delete the history of that user, for example the history of commits made by that user.

        :param self:
        :param userArn:  The Amazon Resource Name (ARN) of the user to delete from AWS CodeStar.<br/>
        :return: userArn:  The Amazon Resource Name (ARN) of the user deleted from AWS CodeStar.<br/>        """
        pass

    def describe_project(self, id: str) -> DescribeProjectResult:
        """Describes a project and its resources.

        :param self:
        :param id:  The ID of the project.<br/>
        :return: status:  The project creation or deletion status.:<br/>projectTemplateId:  The ID for the AWS CodeStar project template used to create the project.<br/>stackId:  The ID of the primary stack in AWS CloudFormation used to generate resources for the project.<br/>createdTimeStamp:  The date and time the project was created, in timestamp format.<br/>clientRequestToken:  A user- or system-generated token that identifies the entity that requested project creation. <br/>description:  The description of the project, if any.<br/>arn:  The Amazon Resource Name (ARN) for the project.<br/>id:  The ID of the project.<br/>name:  The display name for the project.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>reason</i> In the case of a project creation or deletion failure, a reason for the failure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>state</i> The phase of completion for a project creation or deletion.<br/>        """
        pass

    def describe_user_profile(self, userArn: str) -> DescribeUserProfileResult:
        """Describes a user in AWS CodeStar and the user attributes across all projects.

        :param self:
        :param userArn:  The Amazon Resource Name (ARN) of the user.<br/>
        :return: lastModifiedTimestamp:  The date and time when the user profile was last modified, in timestamp format.<br/>createdTimestamp:  The date and time when the user profile was created in AWS CodeStar, in timestamp format.<br/>sshPublicKey:  The SSH public key associated with the user. This SSH public key is associated with the user profile, and can be used in conjunction with the associated private key for access to project resources, such as Amazon EC2 instances, if a project owner grants remote access to those resources.<br/>emailAddress:  The email address for the user. Optional.<br/>displayName:  The display name shown for the user in AWS CodeStar projects. For example, this could be set to both first and last name ("Mary Major") or a single name ("Mary"). The display name is also used to generate the initial icon associated with the user in AWS CodeStar projects. If spaces are included in the display name, the first character that appears after the space will be used as the second character in the user initial icon. The initial icon displays a maximum of two characters, so a display name with more than one space (for example "Mary Jane Major") would generate an initial icon using the first character and the first character after the space ("MJ", not "MM").<br/>userArn:  The Amazon Resource Name (ARN) of the user.<br/>        """
        pass

    def disassociate_team_member(self, userArn: str, projectId: str) -> DisassociateTeamMemberResult:
        """Removes a user from a project. Removing a user from a project also removes the IAM policies from that user that allowed access to the project and its resources. Disassociating a team member does not remove that user's profile from AWS CodeStar. It does not remove the user from IAM.

        :param self:
        :param userArn:  The Amazon Resource Name (ARN) of the IAM user or group whom you want to remove from the project.<br/>projectId:  The ID of the AWS CodeStar project from which you want to remove a team member.<br/>
        :return:         """
        pass

    def list_projects(self, nextToken: str=None, maxResults: int=None) -> ListProjectsResult:
        """Lists all projects in AWS CodeStar associated with your AWS account.

        :param self:
        :param maxResults:  The maximum amount of data that can be contained in a single set of results.<br/>nextToken:  The continuation token to be used to return the next set of results, if the results cannot be returned in one response.<br/>
        :return: nextToken:  The continuation token to use when requesting the next set of results, if there are more results to be returned.<br/>projects:  A list of projects.<br/>        """
        pass

    def list_resources(self, projectId: str, nextToken: str=None, maxResults: int=None) -> ListResourcesResult:
        """Lists resources associated with a project in AWS CodeStar.

        :param self:
        :param maxResults:  The maximum amount of data that can be contained in a single set of results.<br/>nextToken:  The continuation token for the next set of results, if the results cannot be returned in one response.<br/>projectId:  The ID of the project.<br/>
        :return: nextToken:  The continuation token to use when requesting the next set of results, if there are more results to be returned.<br/>resources:  An array of resources associated with the project. <br/>        """
        pass

    def list_tags_for_project(self, id: str, nextToken: str=None, maxResults: int=None) -> ListTagsForProjectResult:
        """Gets the tags for a project.

        :param self:
        :param maxResults:  Reserved for future use.<br/>nextToken:  Reserved for future use.<br/>id:  The ID of the project to get tags for.<br/>
        :return: nextToken:  Reserved for future use.<br/>tags:  The tags for the project.<br/>        """
        pass

    def list_team_members(self, projectId: str, nextToken: str=None, maxResults: int=None) -> ListTeamMembersResult:
        """Lists all team members associated with a project.

        :param self:
        :param maxResults:  The maximum number of team members you want returned in a response.<br/>nextToken:  The continuation token for the next set of results, if the results cannot be returned in one response.<br/>projectId:  The ID of the project for which you want to list team members.<br/>
        :return: nextToken:  The continuation token to use when requesting the next set of results, if there are more results to be returned.<br/>teamMembers:  A list of team member objects for the project.<br/>        """
        pass

    def list_user_profiles(self, nextToken: str=None, maxResults: int=None) -> ListUserProfilesResult:
        """Lists all the user profiles configured for your AWS account in AWS CodeStar.

        :param self:
        :param maxResults:  The maximum number of results to return in a response.<br/>nextToken:  The continuation token for the next set of results, if the results cannot be returned in one response.<br/>
        :return: nextToken:  The continuation token to use when requesting the next set of results, if there are more results to be returned.<br/>userProfiles:  All the user profiles configured in AWS CodeStar for an AWS account.<br/>        """
        pass

    def tag_project(self, tags: Tags, id: str) -> TagProjectResult:
        """Adds tags to a project.

        :param self:
        :param tags:  The tags you want to add to the project.<br/>id:  The ID of the project you want to add a tag to.<br/>
        :return: tags:  The tags for the project.<br/>        """
        pass

    def untag_project(self, tags: List[TagKey], id: str) -> UntagProjectResult:
        """Removes tags from a project.

        :param self:
        :param tags:  The tags to remove from the project.<br/>id:  The ID of the project to remove tags from.<br/>
        :return:         """
        pass

    def update_project(self, id: str, name: str=None, description: str=None) -> UpdateProjectResult:
        """Updates a project in AWS CodeStar.

        :param self:
        :param description:  The description of the project, if any.<br/>name:  The name of the project you want to update.<br/>id:  The ID of the project you want to update.<br/>
        :return:         """
        pass

    def update_team_member(self, userArn: str, projectId: str, projectRole: str=None, remoteAccessAllowed: RemoteAccessAllowed=None) -> UpdateTeamMemberResult:
        """Updates a team member's attributes in an AWS CodeStar project. For example, you can change a team member's role in the project, or change whether they have remote access to project resources.

        :param self:
        :param remoteAccessAllowed:  Whether a team member is allowed to remotely access project resources using the SSH public key associated with the user's profile. Even if this is set to True, the user must associate a public key with their profile before the user can access resources.<br/>projectRole:  The role assigned to the user in the project. Project roles have different levels of access. For more information, see <a href="http://docs.aws.amazon.com/codestar/latest/userguide/working-with-teams.html">Working with Teams</a> in the <i>AWS CodeStar User Guide</i>.<br/>userArn:  The Amazon Resource Name (ARN) of the user for whom you want to change team membership attributes.<br/>projectId:  The ID of the project.<br/>
        :return: remoteAccessAllowed:  Whether a team member is allowed to remotely access project resources using the SSH public key associated with the user's profile.<br/>projectRole:  The project role granted to the user.<br/>userArn:  The Amazon Resource Name (ARN) of the user whose team membership attributes were updated.<br/>        """
        pass

    def update_user_profile(self, userArn: str, displayName: str=None, emailAddress: str=None, sshPublicKey: str=None) -> UpdateUserProfileResult:
        """Updates a user's profile in AWS CodeStar. The user profile is not project-specific. Information in the user profile is displayed wherever the user's information appears to other users in AWS CodeStar. 

        :param self:
        :param sshPublicKey:  The SSH public key associated with the user in AWS CodeStar. If a project owner allows the user remote access to project resources, this public key will be used along with the user's private key for SSH access.<br/>emailAddress:  The email address that is displayed as part of the user's profile in AWS CodeStar.<br/>displayName:  The name that is displayed as the friendly name for the user in AWS CodeStar.<br/>userArn:  The name that will be displayed as the friendly name for the user in AWS CodeStar.<br/>
        :return: lastModifiedTimestamp:  The date the user profile was last modified, in timestamp format.<br/>createdTimestamp:  The date the user profile was created, in timestamp format.<br/>sshPublicKey:  The SSH public key associated with the user in AWS CodeStar. This is the public portion of the public/private keypair the user can use to access project resources if a project owner allows the user remote access to those resources.<br/>emailAddress:  The email address that is displayed as part of the user's profile in AWS CodeStar.<br/>displayName:  The name that is displayed as the friendly name for the user in AWS CodeStar.<br/>userArn:  The Amazon Resource Name (ARN) of the user in IAM.<br/>        """
        pass



class CognitoIdentity(BaseClient):
    """<fullname>Amazon Cognito Federated Identities</fullname> Amazon Cognito Federated Identities is a web service that delivers scoped temporary credentials to mobile devices and other untrusted environments. It uniquely identifies a device and supplies the user with a consistent identity over the lifetime of an application. Using Amazon Cognito Federated Identities, you can enable authentication with one or more third-party identity providers (Facebook, Google, or Login with Amazon) or an Amazon Cognito user pool, and you can also choose to support unauthenticated access from your app. Cognito delivers a unique identifier for each user and acts as an OpenID token provider trusted by AWS Security Token Service (STS) to access temporary, limited-privilege AWS credentials. For a description of the authentication flow from the Amazon Cognito Developer Guide see <a href="https://docs.aws.amazon.com/cognito/latest/developerguide/authentication-flow.html">Authentication Flow</a>. For more information see <a href="https://docs.aws.amazon.com/cognito/latest/developerguide/cognito-identity.html">Amazon Cognito Federated Identities</a>."""


    class ListIdentityPoolsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_identity_pools` for valid parameters"""
            pass


    class CreateIdentityPoolInput(Mapping):
        pass
    
    class IdentityPool(Mapping):
        pass
    
    class IdentityPoolUnauthenticated(object):
        pass
    
    class IdentityProviders(object):
        pass
    
    class ARNString(object):
        pass
    
    class CognitoIdentityProvider(Mapping):
        pass
    
    class IdentityPoolTagsType(object):
        pass
    
    class DeleteIdentitiesInput(Mapping):
        pass
    
    class DeleteIdentitiesResponse(Mapping):
        pass
    
    class IdentityId(object):
        pass
    
    class DeleteIdentityPoolInput(Mapping):
        pass
    
    class DescribeIdentityInput(Mapping):
        pass
    
    class IdentityDescription(Mapping):
        pass
    
    class DescribeIdentityPoolInput(Mapping):
        pass
    
    class GetCredentialsForIdentityInput(Mapping):
        pass
    
    class GetCredentialsForIdentityResponse(Mapping):
        pass
    
    class LoginsMap(object):
        pass
    
    class GetIdInput(Mapping):
        pass
    
    class GetIdResponse(Mapping):
        pass
    
    class GetIdentityPoolRolesInput(Mapping):
        pass
    
    class GetIdentityPoolRolesResponse(Mapping):
        pass
    
    class GetOpenIdTokenInput(Mapping):
        pass
    
    class GetOpenIdTokenResponse(Mapping):
        pass
    
    class GetOpenIdTokenForDeveloperIdentityInput(Mapping):
        pass
    
    class GetOpenIdTokenForDeveloperIdentityResponse(Mapping):
        pass
    
    class TokenDuration(object):
        pass
    
    class ListIdentitiesInput(Mapping):
        pass
    
    class ListIdentitiesResponse(Mapping):
        pass
    
    class HideDisabled(object):
        pass
    
    class ListIdentityPoolsInput(Mapping):
        pass
    
    class ListIdentityPoolsResponse(Mapping):
        pass
    
    class ListTagsForResourceInput(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class LookupDeveloperIdentityInput(Mapping):
        pass
    
    class LookupDeveloperIdentityResponse(Mapping):
        pass
    
    class MergeDeveloperIdentitiesInput(Mapping):
        pass
    
    class MergeDeveloperIdentitiesResponse(Mapping):
        pass
    
    class SetIdentityPoolRolesInput(Mapping):
        pass
    
    class RolesMap(object):
        pass
    
    class RoleMappingMap(object):
        pass
    
    class TagResourceInput(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UnlinkDeveloperIdentityInput(Mapping):
        pass
    
    class UnlinkIdentityInput(Mapping):
        pass
    
    class IdentityProviderName(object):
        pass
    
    class UntagResourceInput(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKeysType(object):
        pass
    
    def create_identity_pool(self, AllowUnauthenticatedIdentities: IdentityPoolUnauthenticated, IdentityPoolName: str, SupportedLoginProviders: IdentityProviders=None, DeveloperProviderName: str=None, OpenIdConnectProviderARNs: List[ARNString]=None, CognitoIdentityProviders: List[CognitoIdentityProvider]=None, SamlProviderARNs: List[ARNString]=None, IdentityPoolTags: IdentityPoolTagsType=None) -> IdentityPool:
        """Creates a new identity pool. The identity pool is a store of user identity information that is specific to your AWS account. The limit on identity pools is 60 per account. The keys for <code>SupportedLoginProviders</code> are as follows: <ul> <li> Facebook: <code>graph.facebook.com</code>  </li> <li> Google: <code>accounts.google.com</code>  </li> <li> Amazon: <code>www.amazon.com</code>  </li> <li> Twitter: <code>api.twitter.com</code>  </li> <li> Digits: <code>www.digits.com</code>  </li> </ul> You must use AWS Developer credentials to call this API.

        :param self:
        :param IdentityPoolTags:  Tags to assign to the identity pool. A tag is a label that you can apply to identity pools to categorize and manage them in different ways, such as by purpose, owner, environment, or other criteria.<br/>SamlProviderARNs:  An array of Amazon Resource Names (ARNs) of the SAML provider for your identity pool.<br/>CognitoIdentityProviders:  An array of Amazon Cognito user pools and their client IDs.<br/>OpenIdConnectProviderARNs:  A list of OpendID Connect provider ARNs.<br/>DeveloperProviderName:  The "domain" by which Cognito will refer to your users. This name acts as a placeholder that allows your backend and the Cognito service to communicate about the developer provider. For the <code>DeveloperProviderName</code>, you can use letters as well as period (<code>.</code>), underscore (<code>_</code>), and dash (<code>-</code>). Once you have set a developer provider name, you cannot change it. Please take care in setting this parameter.<br/>SupportedLoginProviders:  Optional key:value pairs mapping provider names to provider app IDs.<br/>AllowUnauthenticatedIdentities:  TRUE if the identity pool supports unauthenticated logins.<br/>IdentityPoolName:  A string that you provide.<br/>
        :return: IdentityPoolTags:  The tags that are assigned to the identity pool. A tag is a label that you can apply to identity pools to categorize and manage them in different ways, such as by purpose, owner, environment, or other criteria.<br/>SamlProviderARNs:  An array of Amazon Resource Names (ARNs) of the SAML provider for your identity pool.<br/>CognitoIdentityProviders:  A list representing an Amazon Cognito user pool and its client ID.<br/>OpenIdConnectProviderARNs:  A list of OpendID Connect provider ARNs.<br/>DeveloperProviderName:  The "domain" by which Cognito will refer to your users.<br/>SupportedLoginProviders:  Optional key:value pairs mapping provider names to provider app IDs.<br/>AllowUnauthenticatedIdentities:  TRUE if the identity pool supports unauthenticated logins.<br/>IdentityPoolName:  A string that you provide.<br/>IdentityPoolId:  An identity pool ID in the format REGION:GUID.<br/>        """
        pass

    def delete_identities(self, IdentityIdsToDelete: List[IdentityId]) -> DeleteIdentitiesResponse:
        """Deletes identities from an identity pool. You can specify a list of 1-60 identities that you want to delete. You must use AWS Developer credentials to call this API.

        :param self:
        :param IdentityIdsToDelete:  A list of 1-60 identities that you want to delete.<br/>
        :return: UnprocessedIdentityIds:  An array of UnprocessedIdentityId objects, each of which contains an ErrorCode and IdentityId.<br/>        """
        pass

    def delete_identity_pool(self, IdentityPoolId: str):
        """Deletes an identity pool. Once a pool is deleted, users will not be able to authenticate with the pool. You must use AWS Developer credentials to call this API.

        :param self:
        :param IdentityPoolId:  An identity pool ID in the format REGION:GUID.<br/>
        :return:         """
        pass

    def describe_identity(self, IdentityId: str) -> IdentityDescription:
        """Returns metadata related to the given identity, including when the identity was created and any associated linked logins. You must use AWS Developer credentials to call this API.

        :param self:
        :param IdentityId:  A unique identifier in the format REGION:GUID.<br/>
        :return: LastModifiedDate:  Date on which the identity was last modified.<br/>CreationDate:  Date on which the identity was created.<br/>Logins:  The provider names.<br/>IdentityId:  A unique identifier in the format REGION:GUID.<br/>        """
        pass

    def describe_identity_pool(self, IdentityPoolId: str) -> IdentityPool:
        """Gets details about a particular identity pool, including the pool name, ID description, creation date, and current number of users. You must use AWS Developer credentials to call this API.

        :param self:
        :param IdentityPoolId:  An identity pool ID in the format REGION:GUID.<br/>
        :return: IdentityPoolTags:  The tags that are assigned to the identity pool. A tag is a label that you can apply to identity pools to categorize and manage them in different ways, such as by purpose, owner, environment, or other criteria.<br/>SamlProviderARNs:  An array of Amazon Resource Names (ARNs) of the SAML provider for your identity pool.<br/>CognitoIdentityProviders:  A list representing an Amazon Cognito user pool and its client ID.<br/>OpenIdConnectProviderARNs:  A list of OpendID Connect provider ARNs.<br/>DeveloperProviderName:  The "domain" by which Cognito will refer to your users.<br/>SupportedLoginProviders:  Optional key:value pairs mapping provider names to provider app IDs.<br/>AllowUnauthenticatedIdentities:  TRUE if the identity pool supports unauthenticated logins.<br/>IdentityPoolName:  A string that you provide.<br/>IdentityPoolId:  An identity pool ID in the format REGION:GUID.<br/>        """
        pass

    def get_credentials_for_identity(self, IdentityId: str, Logins: LoginsMap=None, CustomRoleArn: str=None) -> GetCredentialsForIdentityResponse:
        """Returns credentials for the provided identity ID. Any provided logins will be validated against supported login providers. If the token is for cognito-identity.amazonaws.com, it will be passed through to AWS Security Token Service with the appropriate role for the token. This is a public API. You do not need any credentials to call this API.

        :param self:
        :param CustomRoleArn:  The Amazon Resource Name (ARN) of the role to be assumed when multiple roles were received in the token from the identity provider. For example, a SAML-based identity provider. This parameter is optional for identity providers that do not support role customization.<br/>Logins:  A set of optional name-value pairs that map provider names to provider tokens. The name-value pair will follow the syntax "provider_name": "provider_user_identifier". Logins should not be specified when trying to get credentials for an unauthenticated identity. The Logins parameter is required when using identities associated with external identity providers such as FaceBook. For examples of <code>Logins</code> maps, see the code examples in the <a href="http://docs.aws.amazon.com/cognito/latest/developerguide/external-identity-providers.html">External Identity Providers</a> section of the Amazon Cognito Developer Guide.<br/>IdentityId:  A unique identifier in the format REGION:GUID.<br/>
        :return: Credentials:  Credentials for the provided identity ID.:<br/>IdentityId:  A unique identifier in the format REGION:GUID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Expiration</i> The date at which these credentials will expire.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SessionToken</i> The Session Token portion of the credentials<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecretKey</i> The Secret Access Key portion of the credentials<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccessKeyId</i> The Access Key portion of the credentials.<br/>        """
        pass

    def get_id(self, IdentityPoolId: str, AccountId: str=None, Logins: LoginsMap=None) -> GetIdResponse:
        """Generates (or retrieves) a Cognito ID. Supplying multiple logins will create an implicit linked account. This is a public API. You do not need any credentials to call this API.

        :param self:
        :param Logins:  A set of optional name-value pairs that map provider names to provider tokens. The available provider names for <code>Logins</code> are as follows: <ul> <li> Facebook: <code>graph.facebook.com</code>  </li> <li> Amazon Cognito user pool: <code>cognito-idp.&lt;region&gt;.amazonaws.com/&lt;YOUR_USER_POOL_ID&gt;</code>, for example, <code>cognito-idp.us-east-1.amazonaws.com/us-east-1_123456789</code>.  </li> <li> Google: <code>accounts.google.com</code>  </li> <li> Amazon: <code>www.amazon.com</code>  </li> <li> Twitter: <code>api.twitter.com</code>  </li> <li> Digits: <code>www.digits.com</code>  </li> </ul><br/>IdentityPoolId:  An identity pool ID in the format REGION:GUID.<br/>AccountId:  A standard AWS account ID (9+ digits).<br/>
        :return: IdentityId:  A unique identifier in the format REGION:GUID.<br/>        """
        pass

    def get_identity_pool_roles(self, IdentityPoolId: str) -> GetIdentityPoolRolesResponse:
        """Gets the roles for an identity pool. You must use AWS Developer credentials to call this API.

        :param self:
        :param IdentityPoolId:  An identity pool ID in the format REGION:GUID.<br/>
        :return: RoleMappings:  How users for a specific identity provider are to mapped to roles. This is a String-to-<a>RoleMapping</a> object map. The string identifies the identity provider, for example, "graph.facebook.com" or "cognito-idp.us-east-1.amazonaws.com/us-east-1_abcdefghi:app_client_id".<br/>Roles:  The map of roles associated with this pool. Currently only authenticated and unauthenticated roles are supported.<br/>IdentityPoolId:  An identity pool ID in the format REGION:GUID.<br/>        """
        pass

    def get_open_id_token(self, IdentityId: str, Logins: LoginsMap=None) -> GetOpenIdTokenResponse:
        """Gets an OpenID token, using a known Cognito ID. This known Cognito ID is returned by <a>GetId</a>. You can optionally add additional logins for the identity. Supplying multiple logins creates an implicit link. The OpenId token is valid for 10 minutes. This is a public API. You do not need any credentials to call this API.

        :param self:
        :param Logins:  A set of optional name-value pairs that map provider names to provider tokens. When using graph.facebook.com and www.amazon.com, supply the access_token returned from the provider's authflow. For accounts.google.com, an Amazon Cognito user pool provider, or any other OpenId Connect provider, always include the <code>id_token</code>.<br/>IdentityId:  A unique identifier in the format REGION:GUID.<br/>
        :return: Token:  An OpenID token, valid for 10 minutes.<br/>IdentityId:  A unique identifier in the format REGION:GUID. Note that the IdentityId returned may not match the one passed on input.<br/>        """
        pass

    def get_open_id_token_for_developer_identity(self, Logins: LoginsMap, IdentityPoolId: str, IdentityId: str=None, TokenDuration: TokenDuration=None) -> GetOpenIdTokenForDeveloperIdentityResponse:
        """Registers (or retrieves) a Cognito <code>IdentityId</code> and an OpenID Connect token for a user authenticated by your backend authentication process. Supplying multiple logins will create an implicit linked account. You can only specify one developer provider as part of the <code>Logins</code> map, which is linked to the identity pool. The developer provider is the "domain" by which Cognito will refer to your users. You can use <code>GetOpenIdTokenForDeveloperIdentity</code> to create a new identity and to link new logins (that is, user credentials issued by a public provider or developer provider) to an existing identity. When you want to create a new identity, the <code>IdentityId</code> should be null. When you want to associate a new login with an existing authenticated/unauthenticated identity, you can do so by providing the existing <code>IdentityId</code>. This API will create the identity in the specified <code>IdentityPoolId</code>. You must use AWS Developer credentials to call this API.

        :param self:
        :param TokenDuration:  The expiration time of the token, in seconds. You can specify a custom expiration time for the token so that you can cache it. If you don't provide an expiration time, the token is valid for 15 minutes. You can exchange the token with Amazon STS for temporary AWS credentials, which are valid for a maximum of one hour. The maximum token duration you can set is 24 hours. You should take care in setting the expiration time for a token, as there are significant security implications: an attacker could use a leaked token to access your AWS resources for the token's duration.<br/>Logins:  A set of optional name-value pairs that map provider names to provider tokens. Each name-value pair represents a user from a public provider or developer provider. If the user is from a developer provider, the name-value pair will follow the syntax <code>"developer_provider_name": "developer_user_identifier"</code>. The developer provider is the "domain" by which Cognito will refer to your users; you provided this domain while creating/updating the identity pool. The developer user identifier is an identifier from your backend that uniquely identifies a user. When you create an identity pool, you can specify the supported logins.<br/>IdentityId:  A unique identifier in the format REGION:GUID.<br/>IdentityPoolId:  An identity pool ID in the format REGION:GUID.<br/>
        :return: Token:  An OpenID token.<br/>IdentityId:  A unique identifier in the format REGION:GUID.<br/>        """
        pass

    def list_identities(self, MaxResults: int, IdentityPoolId: str, NextToken: str=None, HideDisabled: HideDisabled=None) -> ListIdentitiesResponse:
        """Lists the identities in an identity pool. You must use AWS Developer credentials to call this API.

        :param self:
        :param HideDisabled:  An optional boolean parameter that allows you to hide disabled identities. If omitted, the ListIdentities API will include disabled identities in the response.<br/>NextToken:  A pagination token.<br/>MaxResults:  The maximum number of identities to return.<br/>IdentityPoolId:  An identity pool ID in the format REGION:GUID.<br/>
        :return: NextToken:  A pagination token.<br/>Identities:  An object containing a set of identities and associated mappings.<br/>IdentityPoolId:  An identity pool ID in the format REGION:GUID.<br/>        """
        pass

    def list_identity_pools(self, MaxResults: int, NextToken: str=None) -> ListIdentityPoolsResponse:
        """Lists all of the Cognito identity pools registered for your account. You must use AWS Developer credentials to call this API.

        :param self:
        :param NextToken:  A pagination token.<br/>MaxResults:  The maximum number of identities to return.<br/>
        :return: NextToken:  A pagination token.<br/>IdentityPools:  The identity pools returned by the ListIdentityPools action.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceArn: str) -> ListTagsForResourceResponse:
        """Lists the tags that are assigned to an Amazon Cognito identity pool. A tag is a label that you can apply to identity pools to categorize and manage them in different ways, such as by purpose, owner, environment, or other criteria. You can use this action up to 10 times per second, per account.

        :param self:
        :param ResourceArn:  The Amazon Resource Name (ARN) of the identity pool that the tags are assigned to.<br/>
        :return: Tags:  The tags that are assigned to the identity pool.<br/>        """
        pass

    def lookup_developer_identity(self, IdentityPoolId: str, IdentityId: str=None, DeveloperUserIdentifier: str=None, MaxResults: int=None, NextToken: str=None) -> LookupDeveloperIdentityResponse:
        """Retrieves the <code>IdentityID</code> associated with a <code>DeveloperUserIdentifier</code> or the list of <code>DeveloperUserIdentifier</code> values associated with an <code>IdentityId</code> for an existing identity. Either <code>IdentityID</code> or <code>DeveloperUserIdentifier</code> must not be null. If you supply only one of these values, the other value will be searched in the database and returned as a part of the response. If you supply both, <code>DeveloperUserIdentifier</code> will be matched against <code>IdentityID</code>. If the values are verified against the database, the response returns both values and is the same as the request. Otherwise a <code>ResourceConflictException</code> is thrown.  <code>LookupDeveloperIdentity</code> is intended for low-throughput control plane operations: for example, to enable customer service to locate an identity ID by username. If you are using it for higher-volume operations such as user authentication, your requests are likely to be throttled. <a>GetOpenIdTokenForDeveloperIdentity</a> is a better option for higher-volume operations for user authentication. You must use AWS Developer credentials to call this API.

        :param self:
        :param NextToken:  A pagination token. The first call you make will have <code>NextToken</code> set to null. After that the service will return <code>NextToken</code> values as needed. For example, let's say you make a request with <code>MaxResults</code> set to 10, and there are 20 matches in the database. The service will return a pagination token as a part of the response. This token can be used to call the API again and get results starting from the 11th match.<br/>MaxResults:  The maximum number of identities to return.<br/>DeveloperUserIdentifier:  A unique ID used by your backend authentication process to identify a user. Typically, a developer identity provider would issue many developer user identifiers, in keeping with the number of users.<br/>IdentityId:  A unique identifier in the format REGION:GUID.<br/>IdentityPoolId:  An identity pool ID in the format REGION:GUID.<br/>
        :return: NextToken:  A pagination token. The first call you make will have <code>NextToken</code> set to null. After that the service will return <code>NextToken</code> values as needed. For example, let's say you make a request with <code>MaxResults</code> set to 10, and there are 20 matches in the database. The service will return a pagination token as a part of the response. This token can be used to call the API again and get results starting from the 11th match.<br/>DeveloperUserIdentifierList:  This is the list of developer user identifiers associated with an identity ID. Cognito supports the association of multiple developer user identifiers with an identity ID.<br/>IdentityId:  A unique identifier in the format REGION:GUID.<br/>        """
        pass

    def merge_developer_identities(self, IdentityPoolId: str, DeveloperProviderName: str, DestinationUserIdentifier: str, SourceUserIdentifier: str) -> MergeDeveloperIdentitiesResponse:
        """Merges two users having different <code>IdentityId</code>s, existing in the same identity pool, and identified by the same developer provider. You can use this action to request that discrete users be merged and identified as a single user in the Cognito environment. Cognito associates the given source user (<code>SourceUserIdentifier</code>) with the <code>IdentityId</code> of the <code>DestinationUserIdentifier</code>. Only developer-authenticated users can be merged. If the users to be merged are associated with the same public provider, but as two different users, an exception will be thrown. The number of linked logins is limited to 20. So, the number of linked logins for the source user, <code>SourceUserIdentifier</code>, and the destination user, <code>DestinationUserIdentifier</code>, together should not be larger than 20. Otherwise, an exception will be thrown. You must use AWS Developer credentials to call this API.

        :param self:
        :param IdentityPoolId:  An identity pool ID in the format REGION:GUID.<br/>DeveloperProviderName:  The "domain" by which Cognito will refer to your users. This is a (pseudo) domain name that you provide while creating an identity pool. This name acts as a placeholder that allows your backend and the Cognito service to communicate about the developer provider. For the <code>DeveloperProviderName</code>, you can use letters as well as period (.), underscore (_), and dash (-).<br/>DestinationUserIdentifier:  User identifier for the destination user. The value should be a <code>DeveloperUserIdentifier</code>.<br/>SourceUserIdentifier:  User identifier for the source user. The value should be a <code>DeveloperUserIdentifier</code>.<br/>
        :return: IdentityId:  A unique identifier in the format REGION:GUID.<br/>        """
        pass

    def set_identity_pool_roles(self, Roles: RolesMap, IdentityPoolId: str, RoleMappings: RoleMappingMap=None):
        """Sets the roles for an identity pool. These roles are used when making calls to <a>GetCredentialsForIdentity</a> action. You must use AWS Developer credentials to call this API.

        :param self:
        :param RoleMappings:  How users for a specific identity provider are to mapped to roles. This is a string to <a>RoleMapping</a> object map. The string identifies the identity provider, for example, "graph.facebook.com" or "cognito-idp-east-1.amazonaws.com/us-east-1_abcdefghi:app_client_id". Up to 25 rules can be specified per identity provider.<br/>Roles:  The map of roles associated with this pool. For a given role, the key will be either "authenticated" or "unauthenticated" and the value will be the Role ARN.<br/>IdentityPoolId:  An identity pool ID in the format REGION:GUID.<br/>
        :return:         """
        pass

    def tag_resource(self, ResourceArn: str, Tags: IdentityPoolTagsType=None) -> TagResourceResponse:
        """Assigns a set of tags to an Amazon Cognito identity pool. A tag is a label that you can use to categorize and manage identity pools in different ways, such as by purpose, owner, environment, or other criteria. Each tag consists of a key and value, both of which you define. A key is a general category for more specific values. For example, if you have two versions of an identity pool, one for testing and another for production, you might assign an <code>Environment</code> tag key to both identity pools. The value of this key might be <code>Test</code> for one identity pool and <code>Production</code> for the other. Tags are useful for cost tracking and access control. You can activate your tags so that they appear on the Billing and Cost Management console, where you can track the costs associated with your identity pools. In an IAM policy, you can constrain permissions for identity pools based on specific tags or tag values. You can use this action up to 5 times per second, per account. An identity pool can have as many as 50 tags.

        :param self:
        :param Tags:  The tags to assign to the identity pool.<br/>ResourceArn:  The Amazon Resource Name (ARN) of the identity pool to assign the tags to.<br/>
        :return:         """
        pass

    def unlink_developer_identity(self, DeveloperUserIdentifier: str, DeveloperProviderName: str, IdentityPoolId: str, IdentityId: str):
        """Unlinks a <code>DeveloperUserIdentifier</code> from an existing identity. Unlinked developer users will be considered new identities next time they are seen. If, for a given Cognito identity, you remove all federated identities as well as the developer user identifier, the Cognito identity becomes inaccessible. You must use AWS Developer credentials to call this API.

        :param self:
        :param DeveloperUserIdentifier:  A unique ID used by your backend authentication process to identify a user.<br/>DeveloperProviderName:  The "domain" by which Cognito will refer to your users.<br/>IdentityPoolId:  An identity pool ID in the format REGION:GUID.<br/>IdentityId:  A unique identifier in the format REGION:GUID.<br/>
        :return:         """
        pass

    def unlink_identity(self, LoginsToRemove: List[IdentityProviderName], Logins: LoginsMap, IdentityId: str):
        """Unlinks a federated identity from an existing account. Unlinked logins will be considered new identities next time they are seen. Removing the last linked login will make this identity inaccessible. This is a public API. You do not need any credentials to call this API.

        :param self:
        :param LoginsToRemove:  Provider names to unlink from this identity.<br/>Logins:  A set of optional name-value pairs that map provider names to provider tokens.<br/>IdentityId:  A unique identifier in the format REGION:GUID.<br/>
        :return:         """
        pass

    def untag_resource(self, ResourceArn: str, TagKeys: List[TagKeysType]=None) -> UntagResourceResponse:
        """Removes the specified tags from an Amazon Cognito identity pool. You can use this action up to 5 times per second, per account

        :param self:
        :param TagKeys:  The keys of the tags to remove from the user pool.<br/>ResourceArn:  The Amazon Resource Name (ARN) of the identity pool that the tags are assigned to.<br/>
        :return:         """
        pass

    def update_identity_pool(self, AllowUnauthenticatedIdentities: IdentityPoolUnauthenticated, IdentityPoolName: str, IdentityPoolId: str, SupportedLoginProviders: IdentityProviders=None, DeveloperProviderName: str=None, OpenIdConnectProviderARNs: List[ARNString]=None, CognitoIdentityProviders: List[CognitoIdentityProvider]=None, SamlProviderARNs: List[ARNString]=None, IdentityPoolTags: IdentityPoolTagsType=None) -> IdentityPool:
        """Updates an identity pool. You must use AWS Developer credentials to call this API.

        :param self:
        :param IdentityPoolTags:  The tags that are assigned to the identity pool. A tag is a label that you can apply to identity pools to categorize and manage them in different ways, such as by purpose, owner, environment, or other criteria.<br/>SamlProviderARNs:  An array of Amazon Resource Names (ARNs) of the SAML provider for your identity pool.<br/>CognitoIdentityProviders:  A list representing an Amazon Cognito user pool and its client ID.<br/>OpenIdConnectProviderARNs:  A list of OpendID Connect provider ARNs.<br/>DeveloperProviderName:  The "domain" by which Cognito will refer to your users.<br/>SupportedLoginProviders:  Optional key:value pairs mapping provider names to provider app IDs.<br/>AllowUnauthenticatedIdentities:  TRUE if the identity pool supports unauthenticated logins.<br/>IdentityPoolName:  A string that you provide.<br/>IdentityPoolId:  An identity pool ID in the format REGION:GUID.<br/>
        :return: IdentityPoolTags:  The tags that are assigned to the identity pool. A tag is a label that you can apply to identity pools to categorize and manage them in different ways, such as by purpose, owner, environment, or other criteria.<br/>SamlProviderARNs:  An array of Amazon Resource Names (ARNs) of the SAML provider for your identity pool.<br/>CognitoIdentityProviders:  A list representing an Amazon Cognito user pool and its client ID.<br/>OpenIdConnectProviderARNs:  A list of OpendID Connect provider ARNs.<br/>DeveloperProviderName:  The "domain" by which Cognito will refer to your users.<br/>SupportedLoginProviders:  Optional key:value pairs mapping provider names to provider app IDs.<br/>AllowUnauthenticatedIdentities:  TRUE if the identity pool supports unauthenticated logins.<br/>IdentityPoolName:  A string that you provide.<br/>IdentityPoolId:  An identity pool ID in the format REGION:GUID.<br/>        """
        pass



class CognitoIdentityProvider(BaseClient):
    """Using the Amazon Cognito User Pools API, you can create a user pool to manage directories and users. You can authenticate a user to obtain tokens related to user identity and access policies. This API reference provides information about user pools in Amazon Cognito User Pools. For more information, see the Amazon Cognito Documentation."""


    class AdminListGroupsForUserPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `admin_list_groups_for_user` for valid parameters"""
            pass
    class AdminListUserAuthEventsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `admin_list_user_auth_events` for valid parameters"""
            pass
    class ListGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_groups` for valid parameters"""
            pass
    class ListIdentityProvidersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_identity_providers` for valid parameters"""
            pass
    class ListResourceServersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_resource_servers` for valid parameters"""
            pass
    class ListUserPoolClientsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_user_pool_clients` for valid parameters"""
            pass
    class ListUserPoolsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_user_pools` for valid parameters"""
            pass
    class ListUsersInGroupPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_users_in_group` for valid parameters"""
            pass


    class AddCustomAttributesRequest(Mapping):
        pass
    
    class AddCustomAttributesResponse(Mapping):
        pass
    
    class SchemaAttributeType(Mapping):
        pass
    
    class AdminAddUserToGroupRequest(Mapping):
        pass
    
    class AdminConfirmSignUpRequest(Mapping):
        pass
    
    class AdminConfirmSignUpResponse(Mapping):
        pass
    
    class AdminCreateUserRequest(Mapping):
        pass
    
    class AdminCreateUserResponse(Mapping):
        pass
    
    class AttributeType(Mapping):
        pass
    
    class ForceAliasCreation(object):
        pass
    
    class DeliveryMediumType(object):
        pass
    
    class AdminDeleteUserRequest(Mapping):
        pass
    
    class AdminDeleteUserAttributesRequest(Mapping):
        pass
    
    class AdminDeleteUserAttributesResponse(Mapping):
        pass
    
    class AttributeNameType(object):
        pass
    
    class AdminDisableProviderForUserRequest(Mapping):
        pass
    
    class AdminDisableProviderForUserResponse(Mapping):
        pass
    
    class ProviderUserIdentifierType(Mapping):
        pass
    
    class AdminDisableUserRequest(Mapping):
        pass
    
    class AdminDisableUserResponse(Mapping):
        pass
    
    class AdminEnableUserRequest(Mapping):
        pass
    
    class AdminEnableUserResponse(Mapping):
        pass
    
    class AdminForgetDeviceRequest(Mapping):
        pass
    
    class AdminGetDeviceRequest(Mapping):
        pass
    
    class AdminGetDeviceResponse(Mapping):
        pass
    
    class AdminGetUserRequest(Mapping):
        pass
    
    class AdminGetUserResponse(Mapping):
        pass
    
    class AdminInitiateAuthRequest(Mapping):
        pass
    
    class AdminInitiateAuthResponse(Mapping):
        pass
    
    class AuthParametersType(object):
        pass
    
    class ClientMetadataType(object):
        pass
    
    class AnalyticsMetadataType(Mapping):
        pass
    
    class ContextDataType(Mapping):
        pass
    
    class AdminLinkProviderForUserRequest(Mapping):
        pass
    
    class AdminLinkProviderForUserResponse(Mapping):
        pass
    
    class AdminListDevicesRequest(Mapping):
        pass
    
    class AdminListDevicesResponse(Mapping):
        pass
    
    class AdminListGroupsForUserRequest(Mapping):
        pass
    
    class AdminListGroupsForUserResponse(Mapping):
        pass
    
    class AdminListUserAuthEventsRequest(Mapping):
        pass
    
    class AdminListUserAuthEventsResponse(Mapping):
        pass
    
    class AdminRemoveUserFromGroupRequest(Mapping):
        pass
    
    class AdminResetUserPasswordRequest(Mapping):
        pass
    
    class AdminResetUserPasswordResponse(Mapping):
        pass
    
    class AdminRespondToAuthChallengeRequest(Mapping):
        pass
    
    class AdminRespondToAuthChallengeResponse(Mapping):
        pass
    
    class ChallengeResponsesType(object):
        pass
    
    class AdminSetUserMFAPreferenceRequest(Mapping):
        pass
    
    class AdminSetUserMFAPreferenceResponse(Mapping):
        pass
    
    class SMSMfaSettingsType(Mapping):
        pass
    
    class SoftwareTokenMfaSettingsType(Mapping):
        pass
    
    class AdminSetUserPasswordRequest(Mapping):
        pass
    
    class AdminSetUserPasswordResponse(Mapping):
        pass
    
    class BooleanType(object):
        pass
    
    class AdminSetUserSettingsRequest(Mapping):
        pass
    
    class AdminSetUserSettingsResponse(Mapping):
        pass
    
    class MFAOptionType(Mapping):
        pass
    
    class AdminUpdateAuthEventFeedbackRequest(Mapping):
        pass
    
    class AdminUpdateAuthEventFeedbackResponse(Mapping):
        pass
    
    class AdminUpdateDeviceStatusRequest(Mapping):
        pass
    
    class AdminUpdateDeviceStatusResponse(Mapping):
        pass
    
    class AdminUpdateUserAttributesRequest(Mapping):
        pass
    
    class AdminUpdateUserAttributesResponse(Mapping):
        pass
    
    class AdminUserGlobalSignOutRequest(Mapping):
        pass
    
    class AdminUserGlobalSignOutResponse(Mapping):
        pass
    
    class AssociateSoftwareTokenRequest(Mapping):
        pass
    
    class AssociateSoftwareTokenResponse(Mapping):
        pass
    
    class ChangePasswordRequest(Mapping):
        pass
    
    class ChangePasswordResponse(Mapping):
        pass
    
    class ConfirmDeviceRequest(Mapping):
        pass
    
    class ConfirmDeviceResponse(Mapping):
        pass
    
    class DeviceSecretVerifierConfigType(Mapping):
        pass
    
    class ConfirmForgotPasswordRequest(Mapping):
        pass
    
    class ConfirmForgotPasswordResponse(Mapping):
        pass
    
    class UserContextDataType(Mapping):
        pass
    
    class ConfirmSignUpRequest(Mapping):
        pass
    
    class ConfirmSignUpResponse(Mapping):
        pass
    
    class CreateGroupRequest(Mapping):
        pass
    
    class CreateGroupResponse(Mapping):
        pass
    
    class CreateIdentityProviderRequest(Mapping):
        pass
    
    class CreateIdentityProviderResponse(Mapping):
        pass
    
    class ProviderDetailsType(object):
        pass
    
    class AttributeMappingType(object):
        pass
    
    class IdpIdentifierType(object):
        pass
    
    class CreateResourceServerRequest(Mapping):
        pass
    
    class CreateResourceServerResponse(Mapping):
        pass
    
    class ResourceServerScopeType(Mapping):
        pass
    
    class CreateUserImportJobRequest(Mapping):
        pass
    
    class CreateUserImportJobResponse(Mapping):
        pass
    
    class CreateUserPoolRequest(Mapping):
        pass
    
    class CreateUserPoolResponse(Mapping):
        pass
    
    class UserPoolPolicyType(Mapping):
        pass
    
    class LambdaConfigType(Mapping):
        pass
    
    class VerifiedAttributeType(object):
        pass
    
    class AliasAttributeType(object):
        pass
    
    class UsernameAttributeType(object):
        pass
    
    class VerificationMessageTemplateType(Mapping):
        pass
    
    class DeviceConfigurationType(Mapping):
        pass
    
    class EmailConfigurationType(Mapping):
        pass
    
    class SmsConfigurationType(Mapping):
        pass
    
    class UserPoolTagsType(object):
        pass
    
    class AdminCreateUserConfigType(Mapping):
        pass
    
    class UserPoolAddOnsType(Mapping):
        pass
    
    class CreateUserPoolClientRequest(Mapping):
        pass
    
    class CreateUserPoolClientResponse(Mapping):
        pass
    
    class GenerateSecret(object):
        pass
    
    class ClientPermissionType(object):
        pass
    
    class ExplicitAuthFlowsType(object):
        pass
    
    class ProviderNameType(object):
        pass
    
    class RedirectUrlType(object):
        pass
    
    class OAuthFlowType(object):
        pass
    
    class ScopeType(object):
        pass
    
    class AnalyticsConfigurationType(Mapping):
        pass
    
    class CreateUserPoolDomainRequest(Mapping):
        pass
    
    class CreateUserPoolDomainResponse(Mapping):
        pass
    
    class CustomDomainConfigType(Mapping):
        pass
    
    class DeleteGroupRequest(Mapping):
        pass
    
    class DeleteIdentityProviderRequest(Mapping):
        pass
    
    class DeleteResourceServerRequest(Mapping):
        pass
    
    class DeleteUserRequest(Mapping):
        pass
    
    class DeleteUserAttributesRequest(Mapping):
        pass
    
    class DeleteUserAttributesResponse(Mapping):
        pass
    
    class DeleteUserPoolRequest(Mapping):
        pass
    
    class DeleteUserPoolClientRequest(Mapping):
        pass
    
    class DeleteUserPoolDomainRequest(Mapping):
        pass
    
    class DeleteUserPoolDomainResponse(Mapping):
        pass
    
    class DescribeIdentityProviderRequest(Mapping):
        pass
    
    class DescribeIdentityProviderResponse(Mapping):
        pass
    
    class DescribeResourceServerRequest(Mapping):
        pass
    
    class DescribeResourceServerResponse(Mapping):
        pass
    
    class DescribeRiskConfigurationRequest(Mapping):
        pass
    
    class DescribeRiskConfigurationResponse(Mapping):
        pass
    
    class DescribeUserImportJobRequest(Mapping):
        pass
    
    class DescribeUserImportJobResponse(Mapping):
        pass
    
    class DescribeUserPoolRequest(Mapping):
        pass
    
    class DescribeUserPoolResponse(Mapping):
        pass
    
    class DescribeUserPoolClientRequest(Mapping):
        pass
    
    class DescribeUserPoolClientResponse(Mapping):
        pass
    
    class DescribeUserPoolDomainRequest(Mapping):
        pass
    
    class DescribeUserPoolDomainResponse(Mapping):
        pass
    
    class ForgetDeviceRequest(Mapping):
        pass
    
    class ForgotPasswordRequest(Mapping):
        pass
    
    class ForgotPasswordResponse(Mapping):
        pass
    
    class GetCSVHeaderRequest(Mapping):
        pass
    
    class GetCSVHeaderResponse(Mapping):
        pass
    
    class GetDeviceRequest(Mapping):
        pass
    
    class GetDeviceResponse(Mapping):
        pass
    
    class GetGroupRequest(Mapping):
        pass
    
    class GetGroupResponse(Mapping):
        pass
    
    class GetIdentityProviderByIdentifierRequest(Mapping):
        pass
    
    class GetIdentityProviderByIdentifierResponse(Mapping):
        pass
    
    class GetSigningCertificateRequest(Mapping):
        pass
    
    class GetSigningCertificateResponse(Mapping):
        pass
    
    class GetUICustomizationRequest(Mapping):
        pass
    
    class GetUICustomizationResponse(Mapping):
        pass
    
    class GetUserRequest(Mapping):
        pass
    
    class GetUserResponse(Mapping):
        pass
    
    class GetUserAttributeVerificationCodeRequest(Mapping):
        pass
    
    class GetUserAttributeVerificationCodeResponse(Mapping):
        pass
    
    class GetUserPoolMfaConfigRequest(Mapping):
        pass
    
    class GetUserPoolMfaConfigResponse(Mapping):
        pass
    
    class GlobalSignOutRequest(Mapping):
        pass
    
    class GlobalSignOutResponse(Mapping):
        pass
    
    class InitiateAuthRequest(Mapping):
        pass
    
    class InitiateAuthResponse(Mapping):
        pass
    
    class ListDevicesRequest(Mapping):
        pass
    
    class ListDevicesResponse(Mapping):
        pass
    
    class ListGroupsRequest(Mapping):
        pass
    
    class ListGroupsResponse(Mapping):
        pass
    
    class ListIdentityProvidersRequest(Mapping):
        pass
    
    class ListIdentityProvidersResponse(Mapping):
        pass
    
    class ListResourceServersRequest(Mapping):
        pass
    
    class ListResourceServersResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class ListUserImportJobsRequest(Mapping):
        pass
    
    class ListUserImportJobsResponse(Mapping):
        pass
    
    class ListUserPoolClientsRequest(Mapping):
        pass
    
    class ListUserPoolClientsResponse(Mapping):
        pass
    
    class ListUserPoolsRequest(Mapping):
        pass
    
    class ListUserPoolsResponse(Mapping):
        pass
    
    class ListUsersRequest(Mapping):
        pass
    
    class ListUsersResponse(Mapping):
        pass
    
    class ListUsersInGroupRequest(Mapping):
        pass
    
    class ListUsersInGroupResponse(Mapping):
        pass
    
    class ResendConfirmationCodeRequest(Mapping):
        pass
    
    class ResendConfirmationCodeResponse(Mapping):
        pass
    
    class RespondToAuthChallengeRequest(Mapping):
        pass
    
    class RespondToAuthChallengeResponse(Mapping):
        pass
    
    class SetRiskConfigurationRequest(Mapping):
        pass
    
    class SetRiskConfigurationResponse(Mapping):
        pass
    
    class CompromisedCredentialsRiskConfigurationType(Mapping):
        pass
    
    class AccountTakeoverRiskConfigurationType(Mapping):
        pass
    
    class RiskExceptionConfigurationType(Mapping):
        pass
    
    class SetUICustomizationRequest(Mapping):
        pass
    
    class SetUICustomizationResponse(Mapping):
        pass
    
    class ImageFileType(object):
        pass
    
    class SetUserMFAPreferenceRequest(Mapping):
        pass
    
    class SetUserMFAPreferenceResponse(Mapping):
        pass
    
    class SetUserPoolMfaConfigRequest(Mapping):
        pass
    
    class SetUserPoolMfaConfigResponse(Mapping):
        pass
    
    class SmsMfaConfigType(Mapping):
        pass
    
    class SoftwareTokenMfaConfigType(Mapping):
        pass
    
    class SetUserSettingsRequest(Mapping):
        pass
    
    class SetUserSettingsResponse(Mapping):
        pass
    
    class SignUpRequest(Mapping):
        pass
    
    class SignUpResponse(Mapping):
        pass
    
    class StartUserImportJobRequest(Mapping):
        pass
    
    class StartUserImportJobResponse(Mapping):
        pass
    
    class StopUserImportJobRequest(Mapping):
        pass
    
    class StopUserImportJobResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKeysType(object):
        pass
    
    class UpdateAuthEventFeedbackRequest(Mapping):
        pass
    
    class UpdateAuthEventFeedbackResponse(Mapping):
        pass
    
    class UpdateDeviceStatusRequest(Mapping):
        pass
    
    class UpdateDeviceStatusResponse(Mapping):
        pass
    
    class UpdateGroupRequest(Mapping):
        pass
    
    class UpdateGroupResponse(Mapping):
        pass
    
    class UpdateIdentityProviderRequest(Mapping):
        pass
    
    class UpdateIdentityProviderResponse(Mapping):
        pass
    
    class UpdateResourceServerRequest(Mapping):
        pass
    
    class UpdateResourceServerResponse(Mapping):
        pass
    
    class UpdateUserAttributesRequest(Mapping):
        pass
    
    class UpdateUserAttributesResponse(Mapping):
        pass
    
    class UpdateUserPoolRequest(Mapping):
        pass
    
    class UpdateUserPoolResponse(Mapping):
        pass
    
    class UpdateUserPoolClientRequest(Mapping):
        pass
    
    class UpdateUserPoolClientResponse(Mapping):
        pass
    
    class UpdateUserPoolDomainRequest(Mapping):
        pass
    
    class UpdateUserPoolDomainResponse(Mapping):
        pass
    
    class VerifySoftwareTokenRequest(Mapping):
        pass
    
    class VerifySoftwareTokenResponse(Mapping):
        pass
    
    class VerifyUserAttributeRequest(Mapping):
        pass
    
    class VerifyUserAttributeResponse(Mapping):
        pass
    
    def add_custom_attributes(self, CustomAttributes: List[SchemaAttributeType], UserPoolId: str) -> AddCustomAttributesResponse:
        """Adds additional user attributes to the user pool schema.

        :param self:
        :param CustomAttributes:  An array of custom attributes, such as Mutable and Name.<br/>UserPoolId:  The user pool ID for the user pool where you want to add custom attributes.<br/>
        :return:         """
        pass

    def admin_add_user_to_group(self, GroupName: str, Username: str, UserPoolId: str):
        """Adds the specified user to the specified group. Requires developer credentials.

        :param self:
        :param GroupName:  The group name.<br/>Username:  The username for the user.<br/>UserPoolId:  The user pool ID for the user pool.<br/>
        :return:         """
        pass

    def admin_confirm_sign_up(self, Username: str, UserPoolId: str) -> AdminConfirmSignUpResponse:
        """Confirms user registration as an admin without using a confirmation code. Works on any user. Requires developer credentials.

        :param self:
        :param Username:  The user name for which you want to confirm user registration.<br/>UserPoolId:  The user pool ID for which you want to confirm user registration.<br/>
        :return:         """
        pass

    def admin_create_user(self, Username: str, UserPoolId: str, UserAttributes: List[AttributeType]=None, ValidationData: List[AttributeType]=None, TemporaryPassword: str=None, ForceAliasCreation: ForceAliasCreation=None, MessageAction: str=None, DesiredDeliveryMediums: List[DeliveryMediumType]=None) -> AdminCreateUserResponse:
        """Creates a new user in the specified user pool. If <code>MessageAction</code> is not set, the default is to send a welcome message via email or phone (SMS). <note> This message is based on a template that you configured in your call to or . This template includes your custom sign-up instructions and placeholders for user name and temporary password. </note> Alternatively, you can call AdminCreateUser with “SUPPRESS” for the <code>MessageAction</code> parameter, and Amazon Cognito will not send any email.  In either case, the user will be in the <code>FORCE_CHANGE_PASSWORD</code> state until they sign in and change their password. AdminCreateUser requires developer credentials.

        :param self:
        :param DesiredDeliveryMediums:  Specify <code>"EMAIL"</code> if email will be used to send the welcome message. Specify <code>"SMS"</code> if the phone number will be used. The default value is <code>"SMS"</code>. More than one value can be specified.<br/>MessageAction:  Set to <code>"RESEND"</code> to resend the invitation message to a user that already exists and reset the expiration limit on the user's account. Set to <code>"SUPPRESS"</code> to suppress sending the message. Only one value can be specified.<br/>ForceAliasCreation:  This parameter is only used if the <code>phone_number_verified</code> or <code>email_verified</code> attribute is set to <code>True</code>. Otherwise, it is ignored. If this parameter is set to <code>True</code> and the phone number or email address specified in the UserAttributes parameter already exists as an alias with a different user, the API call will migrate the alias from the previous user to the newly created user. The previous user will no longer be able to log in using that alias. If this parameter is set to <code>False</code>, the API throws an <code>AliasExistsException</code> error if the alias already exists. The default value is <code>False</code>.<br/>TemporaryPassword:  The user's temporary password. This password must conform to the password policy that you specified when you created the user pool. The temporary password is valid only once. To complete the Admin Create User flow, the user must enter the temporary password in the sign-in page along with a new password to be used in all future sign-ins. This parameter is not required. If you do not specify a value, Amazon Cognito generates one for you. The temporary password can only be used until the user account expiration limit that you specified when you created the user pool. To reset the account after that time limit, you must call <code>AdminCreateUser</code> again, specifying <code>"RESEND"</code> for the <code>MessageAction</code> parameter.<br/>ValidationData:  The user's validation data. This is an array of name-value pairs that contain user attributes and attribute values that you can use for custom validation, such as restricting the types of user accounts that can be registered. For example, you might choose to allow or disallow user sign-up based on the user's domain. To configure custom validation, you must create a Pre Sign-up Lambda trigger for the user pool as described in the Amazon Cognito Developer Guide. The Lambda trigger receives the validation data and uses it in the validation process. The user's validation data is not persisted.<br/>UserAttributes:  An array of name-value pairs that contain user attributes and attribute values to be set for the user to be created. You can create a user without specifying any attributes other than <code>Username</code>. However, any attributes that you specify as required (in or in the <b>Attributes</b> tab of the console) must be supplied either by you (in your call to <code>AdminCreateUser</code>) or by the user (when he or she signs up in response to your welcome message). For custom attributes, you must prepend the <code>custom:</code> prefix to the attribute name. To send a message inviting the user to sign up, you must specify the user's email address or phone number. This can be done in your call to AdminCreateUser or in the <b>Users</b> tab of the Amazon Cognito console for managing your user pools. In your call to <code>AdminCreateUser</code>, you can set the <code>email_verified</code> attribute to <code>True</code>, and you can set the <code>phone_number_verified</code> attribute to <code>True</code>. (You can also do this by calling .) <ul> <li>  <b>email</b>: The email address of the user to whom the message that contains the code and username will be sent. Required if the <code>email_verified</code> attribute is set to <code>True</code>, or if <code>"EMAIL"</code> is specified in the <code>DesiredDeliveryMediums</code> parameter. </li> <li>  <b>phone_number</b>: The phone number of the user to whom the message that contains the code and username will be sent. Required if the <code>phone_number_verified</code> attribute is set to <code>True</code>, or if <code>"SMS"</code> is specified in the <code>DesiredDeliveryMediums</code> parameter. </li> </ul><br/>Username:  The username for the user. Must be unique within the user pool. Must be a UTF-8 string between 1 and 128 characters. After the user is created, the username cannot be changed.<br/>UserPoolId:  The user pool ID for the user pool where the user will be created.<br/>
        :return: User:  The newly created user.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MFAOptions</i> The MFA options for the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserStatus</i> The user status. Can be one of the following: <ul> <li> UNCONFIRMED - User has been created but not confirmed. </li> <li> CONFIRMED - User has been confirmed. </li> <li> ARCHIVED - User is no longer active. </li> <li> COMPROMISED - User is disabled due to a potential security threat. </li> <li> UNKNOWN - User status is not known. </li> <li> RESET_REQUIRED - User is confirmed, but the user must request a code and reset his or her password before he or she can sign in. </li> <li> FORCE_CHANGE_PASSWORD - The user is confirmed and the user can sign in using a temporary password, but on first sign-in, the user must change his or her password to a new value before doing anything else.  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the user is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserLastModifiedDate</i> The last modified date of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserCreateDate</i> The creation date of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Attributes</i> A container with information about the user type attributes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> The user name of the user you wish to describe.<br/>        """
        pass

    def admin_delete_user(self, Username: str, UserPoolId: str):
        """Deletes a user as an administrator. Works on any user. Requires developer credentials.

        :param self:
        :param Username:  The user name of the user you wish to delete.<br/>UserPoolId:  The user pool ID for the user pool where you want to delete the user.<br/>
        :return:         """
        pass

    def admin_delete_user_attributes(self, UserAttributeNames: List[AttributeNameType], Username: str, UserPoolId: str) -> AdminDeleteUserAttributesResponse:
        """Deletes the user attributes in a user pool as an administrator. Works on any user. Requires developer credentials.

        :param self:
        :param UserAttributeNames:  An array of strings representing the user attribute names you wish to delete. For custom attributes, you must prepend the <code>custom:</code> prefix to the attribute name.<br/>Username:  The user name of the user from which you would like to delete attributes.<br/>UserPoolId:  The user pool ID for the user pool where you want to delete user attributes.<br/>
        :return:         """
        pass

    def admin_disable_provider_for_user(self, User: ProviderUserIdentifierType, UserPoolId: str) -> AdminDisableProviderForUserResponse:
        """Disables the user from signing in with the specified external (SAML or social) identity provider. If the user to disable is a Cognito User Pools native username + password user, they are not permitted to use their password to sign-in. If the user to disable is a linked external IdP user, any link between that user and an existing user is removed. The next time the external user (no longer attached to the previously linked <code>DestinationUser</code>) signs in, they must create a new user account. See . This action is enabled only for admin access and requires developer credentials. The <code>ProviderName</code> must match the value specified when creating an IdP for the pool.  To disable a native username + password user, the <code>ProviderName</code> value must be <code>Cognito</code> and the <code>ProviderAttributeName</code> must be <code>Cognito_Subject</code>, with the <code>ProviderAttributeValue</code> being the name that is used in the user pool for the user. The <code>ProviderAttributeName</code> must always be <code>Cognito_Subject</code> for social identity providers. The <code>ProviderAttributeValue</code> must always be the exact subject that was used when the user was originally linked as a source user. For de-linking a SAML identity, there are two scenarios. If the linked identity has not yet been used to sign-in, the <code>ProviderAttributeName</code> and <code>ProviderAttributeValue</code> must be the same values that were used for the <code>SourceUser</code> when the identities were originally linked in the call. (If the linking was done with <code>ProviderAttributeName</code> set to <code>Cognito_Subject</code>, the same applies here). However, if the user has already signed in, the <code>ProviderAttributeName</code> must be <code>Cognito_Subject</code> and <code>ProviderAttributeValue</code> must be the subject of the SAML assertion.

        :param self:
        :param User:  The user to be disabled.:<br/>UserPoolId:  The user pool ID for the user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProviderAttributeValue</i> The value of the provider attribute to link to, for example, <code>xxxxx_account</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProviderAttributeName</i> The name of the provider attribute to link to, for example, <code>NameID</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProviderName</i> The name of the provider, for example, Facebook, Google, or Login with Amazon.<br/>
        :return:         """
        pass

    def admin_disable_user(self, Username: str, UserPoolId: str) -> AdminDisableUserResponse:
        """Disables the specified user as an administrator. Works on any user. Requires developer credentials.

        :param self:
        :param Username:  The user name of the user you wish to disable.<br/>UserPoolId:  The user pool ID for the user pool where you want to disable the user.<br/>
        :return:         """
        pass

    def admin_enable_user(self, Username: str, UserPoolId: str) -> AdminEnableUserResponse:
        """Enables the specified user as an administrator. Works on any user. Requires developer credentials.

        :param self:
        :param Username:  The user name of the user you wish to enable.<br/>UserPoolId:  The user pool ID for the user pool where you want to enable the user.<br/>
        :return:         """
        pass

    def admin_forget_device(self, DeviceKey: str, Username: str, UserPoolId: str):
        """Forgets the device, as an administrator. Requires developer credentials.

        :param self:
        :param DeviceKey:  The device key.<br/>Username:  The user name.<br/>UserPoolId:  The user pool ID.<br/>
        :return:         """
        pass

    def admin_get_device(self, Username: str, UserPoolId: str, DeviceKey: str) -> AdminGetDeviceResponse:
        """Gets the device, as an administrator. Requires developer credentials.

        :param self:
        :param Username:  The user name.<br/>UserPoolId:  The user pool ID.<br/>DeviceKey:  The device key.<br/>
        :return: Device:  The device.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceLastAuthenticatedDate</i> The date in which the device was last authenticated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceLastModifiedDate</i> The last modified date of the device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceCreateDate</i> The creation date of the device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceAttributes</i> The device attributes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceKey</i> The device key.<br/>        """
        pass

    def admin_get_user(self, Username: str, UserPoolId: str) -> AdminGetUserResponse:
        """Gets the specified user by user name in a user pool as an administrator. Works on any user. Requires developer credentials.

        :param self:
        :param Username:  The user name of the user you wish to retrieve.<br/>UserPoolId:  The user pool ID for the user pool where you want to get information about the user.<br/>
        :return: UserMFASettingList:  The list of the user's MFA settings.<br/>PreferredMfaSetting:  The user's preferred MFA setting.<br/>MFAOptions:  Specifies the options for MFA (e.g., email or phone number).<br/>UserStatus:  The user status. Can be one of the following: <ul> <li> UNCONFIRMED - User has been created but not confirmed. </li> <li> CONFIRMED - User has been confirmed. </li> <li> ARCHIVED - User is no longer active. </li> <li> COMPROMISED - User is disabled due to a potential security threat. </li> <li> UNKNOWN - User status is not known. </li> <li> RESET_REQUIRED - User is confirmed, but the user must request a code and reset his or her password before he or she can sign in. </li> <li> FORCE_CHANGE_PASSWORD - The user is confirmed and the user can sign in using a temporary password, but on first sign-in, the user must change his or her password to a new value before doing anything else.  </li> </ul><br/>Enabled:  Indicates that the status is enabled.<br/>UserLastModifiedDate:  The date the user was last modified.<br/>UserCreateDate:  The date the user was created.<br/>UserAttributes:  An array of name-value pairs representing user attributes.<br/>Username:  The user name of the user about whom you are receiving information.<br/>        """
        pass

    def admin_initiate_auth(self, AuthFlow: str, ClientId: str, UserPoolId: str, AuthParameters: AuthParametersType=None, ClientMetadata: ClientMetadataType=None, AnalyticsMetadata: AnalyticsMetadataType=None, ContextData: ContextDataType=None) -> AdminInitiateAuthResponse:
        """Initiates the authentication flow, as an administrator. Requires developer credentials.

        :param self:
        :param ContextData:  Contextual data such as the user's device fingerprint, IP address, or location used for evaluating the risk of an unexpected event by Amazon Cognito advanced security.:<br/>AnalyticsMetadata:  The analytics metadata for collecting Amazon Pinpoint metrics for <code>AdminInitiateAuth</code> calls.:<br/>ClientMetadata:  This is a random key-value pair map which can contain any key and will be passed to your PreAuthentication Lambda trigger as-is. It can be used to implement additional validations around authentication.<br/>AuthParameters:  The authentication parameters. These are inputs corresponding to the <code>AuthFlow</code> that you are invoking. The required values depend on the value of <code>AuthFlow</code>: <ul> <li> For <code>USER_SRP_AUTH</code>: <code>USERNAME</code> (required), <code>SRP_A</code> (required), <code>SECRET_HASH</code> (required if the app client is configured with a client secret), <code>DEVICE_KEY</code>  </li> <li> For <code>REFRESH_TOKEN_AUTH/REFRESH_TOKEN</code>: <code>REFRESH_TOKEN</code> (required), <code>SECRET_HASH</code> (required if the app client is configured with a client secret), <code>DEVICE_KEY</code>  </li> <li> For <code>ADMIN_NO_SRP_AUTH</code>: <code>USERNAME</code> (required), <code>SECRET_HASH</code> (if app client is configured with client secret), <code>PASSWORD</code> (required), <code>DEVICE_KEY</code>  </li> <li> For <code>CUSTOM_AUTH</code>: <code>USERNAME</code> (required), <code>SECRET_HASH</code> (if app client is configured with client secret), <code>DEVICE_KEY</code>  </li> </ul><br/>AuthFlow:  The authentication flow for this call to execute. The API action will depend on this value. For example: <ul> <li>  <code>REFRESH_TOKEN_AUTH</code> will take in a valid refresh token and return new tokens. </li> <li>  <code>USER_SRP_AUTH</code> will take in <code>USERNAME</code> and <code>SRP_A</code> and return the SRP variables to be used for next challenge execution. </li> <li>  <code>USER_PASSWORD_AUTH</code> will take in <code>USERNAME</code> and <code>PASSWORD</code> and return the next challenge or tokens. </li> </ul> Valid values include: <ul> <li>  <code>USER_SRP_AUTH</code>: Authentication flow for the Secure Remote Password (SRP) protocol. </li> <li>  <code>REFRESH_TOKEN_AUTH</code>/<code>REFRESH_TOKEN</code>: Authentication flow for refreshing the access token and ID token by supplying a valid refresh token. </li> <li>  <code>CUSTOM_AUTH</code>: Custom authentication flow. </li> <li>  <code>ADMIN_NO_SRP_AUTH</code>: Non-SRP authentication flow; you can pass in the USERNAME and PASSWORD directly if the flow is enabled for calling the app client. </li> <li>  <code>USER_PASSWORD_AUTH</code>: Non-SRP authentication flow; USERNAME and PASSWORD are passed directly. If a user migration Lambda trigger is set, this flow will invoke the user migration Lambda if the USERNAME is not found in the user pool.  </li> </ul><br/>ClientId:  The app client ID.<br/>UserPoolId:  The ID of the Amazon Cognito user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AnalyticsEndpointId</i> The endpoint ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncodedData</i> Encoded data containing device fingerprinting details, collected using the Amazon Cognito context data collection library.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HttpHeaders</i> HttpHeaders received on your server in same order.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerPath</i> Your server path where this API is invoked. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerName</i> Your server endpoint where this API is invoked.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IpAddress</i> Source IP address of your user.<br/>
        :return: AuthenticationResult:  The result of the authentication response. This is only returned if the caller does not need to pass another challenge. If the caller does need to pass another challenge before it gets tokens, <code>ChallengeName</code>, <code>ChallengeParameters</code>, and <code>Session</code> are returned.:<br/>ChallengeParameters:  The challenge parameters. These are returned to you in the <code>AdminInitiateAuth</code> response if you need to pass another challenge. The responses in this parameter should be used to compute inputs to the next call (<code>AdminRespondToAuthChallenge</code>). All challenges require <code>USERNAME</code> and <code>SECRET_HASH</code> (if applicable). The value of the <code>USER_ID_FOR_SRP</code> attribute will be the user's actual username, not an alias (such as email address or phone number), even if you specified an alias in your call to <code>AdminInitiateAuth</code>. This is because, in the <code>AdminRespondToAuthChallenge</code> API <code>ChallengeResponses</code>, the <code>USERNAME</code> attribute cannot be an alias.<br/>Session:  The session which should be passed both ways in challenge-response calls to the service. If <code>AdminInitiateAuth</code> or <code>AdminRespondToAuthChallenge</code> API call determines that the caller needs to go through another challenge, they return a session with other challenge parameters. This session should be passed as it is to the next <code>AdminRespondToAuthChallenge</code> API call.<br/>ChallengeName:  The name of the challenge which you are responding to with this call. This is returned to you in the <code>AdminInitiateAuth</code> response if you need to pass another challenge. <ul> <li>  <code>MFA_SETUP</code>: If MFA is required, users who do not have at least one of the MFA methods set up are presented with an <code>MFA_SETUP</code> challenge. The user must set up at least one MFA type to continue to authenticate. </li> <li>  <code>SELECT_MFA_TYPE</code>: Selects the MFA type. Valid MFA options are <code>SMS_MFA</code> for text SMS MFA, and <code>SOFTWARE_TOKEN_MFA</code> for TOTP software token MFA. </li> <li>  <code>SMS_MFA</code>: Next challenge is to supply an <code>SMS_MFA_CODE</code>, delivered via SMS. </li> <li>  <code>PASSWORD_VERIFIER</code>: Next challenge is to supply <code>PASSWORD_CLAIM_SIGNATURE</code>, <code>PASSWORD_CLAIM_SECRET_BLOCK</code>, and <code>TIMESTAMP</code> after the client-side SRP calculations. </li> <li>  <code>CUSTOM_CHALLENGE</code>: This is returned if your custom authentication flow determines that the user should pass another challenge before tokens are issued. </li> <li>  <code>DEVICE_SRP_AUTH</code>: If device tracking was enabled on your user pool and the previous challenges were passed, this challenge is returned so that Amazon Cognito can start tracking this device. </li> <li>  <code>DEVICE_PASSWORD_VERIFIER</code>: Similar to <code>PASSWORD_VERIFIER</code>, but for devices only. </li> <li>  <code>ADMIN_NO_SRP_AUTH</code>: This is returned if you need to authenticate with <code>USERNAME</code> and <code>PASSWORD</code> directly. An app client must be enabled to use this flow. </li> <li>  <code>NEW_PASSWORD_REQUIRED</code>: For users which are required to change their passwords after successful first login. This challenge should be passed with <code>NEW_PASSWORD</code> and any other required attributes. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NewDeviceMetadata</i> The new device metadata from an authentication result.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NewDeviceMetadata</b>: The new device metadata from an authentication result.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IdToken</i> The ID token.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RefreshToken</i> The refresh token.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TokenType</i> The token type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpiresIn</i> The expiration period of the authentication result in seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccessToken</i> The access token.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceGroupKey</i> The device group key.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceKey</i> The device key.<br/>        """
        pass

    def admin_link_provider_for_user(self, SourceUser: ProviderUserIdentifierType, DestinationUser: ProviderUserIdentifierType, UserPoolId: str) -> AdminLinkProviderForUserResponse:
        """Links an existing user account in a user pool (<code>DestinationUser</code>) to an identity from an external identity provider (<code>SourceUser</code>) based on a specified attribute name and value from the external identity provider. This allows you to create a link from the existing user account to an external federated user identity that has not yet been used to sign in, so that the federated user identity can be used to sign in as the existing user account.   For example, if there is an existing user with a username and password, this API links that user to a federated user identity, so that when the federated user identity is used, the user signs in as the existing user account.  <important> Because this API allows a user with an external federated identity to sign in as an existing user in the user pool, it is critical that it only be used with external identity providers and provider attributes that have been trusted by the application owner. </important> See also . This action is enabled only for admin access and requires developer credentials.

        :param self:
        :param SourceUser:  An external identity provider account for a user who does not currently exist yet in the user pool. This user must be a federated user (for example, a SAML or Facebook user), not another native user. If the <code>SourceUser</code> is a federated social identity provider user (Facebook, Google, or Login with Amazon), you must set the <code>ProviderAttributeName</code> to <code>Cognito_Subject</code>. For social identity providers, the <code>ProviderName</code> will be <code>Facebook</code>, <code>Google</code>, or <code>LoginWithAmazon</code>, and Cognito will automatically parse the Facebook, Google, and Login with Amazon tokens for <code>id</code>, <code>sub</code>, and <code>user_id</code>, respectively. The <code>ProviderAttributeValue</code> for the user must be the same value as the <code>id</code>, <code>sub</code>, or <code>user_id</code> value found in the social identity provider token. <p/> For SAML, the <code>ProviderAttributeName</code> can be any value that matches a claim in the SAML assertion. If you wish to link SAML users based on the subject of the SAML assertion, you should map the subject to a claim through the SAML identity provider and submit that claim name as the <code>ProviderAttributeName</code>. If you set <code>ProviderAttributeName</code> to <code>Cognito_Subject</code>, Cognito will automatically parse the default unique identifier found in the subject from the SAML token.:<br/>DestinationUser:  The existing user in the user pool to be linked to the external identity provider user account. Can be a native (Username + Password) Cognito User Pools user or a federated user (for example, a SAML or Facebook user). If the user doesn't exist, an exception is thrown. This is the user that is returned when the new user (with the linked identity provider attribute) signs in. For a native username + password user, the <code>ProviderAttributeValue</code> for the <code>DestinationUser</code> should be the username in the user pool. For a federated user, it should be the provider-specific <code>user_id</code>. The <code>ProviderAttributeName</code> of the <code>DestinationUser</code> is ignored. The <code>ProviderName</code> should be set to <code>Cognito</code> for users in Cognito user pools.:<br/>UserPoolId:  The user pool ID for the user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProviderAttributeValue</i> The value of the provider attribute to link to, for example, <code>xxxxx_account</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProviderAttributeName</i> The name of the provider attribute to link to, for example, <code>NameID</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProviderName</i> The name of the provider, for example, Facebook, Google, or Login with Amazon.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProviderAttributeValue</i> The value of the provider attribute to link to, for example, <code>xxxxx_account</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProviderAttributeName</i> The name of the provider attribute to link to, for example, <code>NameID</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProviderName</i> The name of the provider, for example, Facebook, Google, or Login with Amazon.<br/>
        :return:         """
        pass

    def admin_list_devices(self, Username: str, UserPoolId: str, Limit: int=None, PaginationToken: str=None) -> AdminListDevicesResponse:
        """Lists devices, as an administrator. Requires developer credentials.

        :param self:
        :param PaginationToken:  The pagination token.<br/>Limit:  The limit of the devices request.<br/>Username:  The user name.<br/>UserPoolId:  The user pool ID.<br/>
        :return: PaginationToken:  The pagination token.<br/>Devices:  The devices in the list of devices response.<br/>        """
        pass

    def admin_list_groups_for_user(self, UserPoolId: str, Username: str, Limit: int=None, NextToken: str=None) -> AdminListGroupsForUserResponse:
        """Lists the groups that the user belongs to. Requires developer credentials.

        :param self:
        :param NextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>Limit:  The limit of the request to list groups.<br/>UserPoolId:  The user pool ID for the user pool.<br/>Username:  The username for the user.<br/>
        :return: NextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>Groups:  The groups that the user belongs to.<br/>        """
        pass

    def admin_list_user_auth_events(self, Username: str, UserPoolId: str, MaxResults: int=None, NextToken: str=None) -> AdminListUserAuthEventsResponse:
        """Lists a history of user activity and any risks detected as part of Amazon Cognito advanced security.

        :param self:
        :param NextToken:  A pagination token.<br/>MaxResults:  The maximum number of authentication events to return.<br/>Username:  The user pool username or an alias.<br/>UserPoolId:  The user pool ID.<br/>
        :return: NextToken:  A pagination token.<br/>AuthEvents:  The response object. It includes the <code>EventID</code>, <code>EventType</code>, <code>CreationDate</code>, <code>EventRisk</code>, and <code>EventResponse</code>.<br/>        """
        pass

    def admin_remove_user_from_group(self, GroupName: str, Username: str, UserPoolId: str):
        """Removes the specified user from the specified group. Requires developer credentials.

        :param self:
        :param GroupName:  The group name.<br/>Username:  The username for the user.<br/>UserPoolId:  The user pool ID for the user pool.<br/>
        :return:         """
        pass

    def admin_reset_user_password(self, Username: str, UserPoolId: str) -> AdminResetUserPasswordResponse:
        """Resets the specified user's password in a user pool as an administrator. Works on any user. When a developer calls this API, the current password is invalidated, so it must be changed. If a user tries to sign in after the API is called, the app will get a PasswordResetRequiredException exception back and should direct the user down the flow to reset the password, which is the same as the forgot password flow. In addition, if the user pool has phone verification selected and a verified phone number exists for the user, or if email verification is selected and a verified email exists for the user, calling this API will also result in sending a message to the end user with the code to change their password. Requires developer credentials.

        :param self:
        :param Username:  The user name of the user whose password you wish to reset.<br/>UserPoolId:  The user pool ID for the user pool where you want to reset the user's password.<br/>
        :return:         """
        pass

    def admin_respond_to_auth_challenge(self, ChallengeName: str, ClientId: str, UserPoolId: str, ChallengeResponses: ChallengeResponsesType=None, Session: str=None, AnalyticsMetadata: AnalyticsMetadataType=None, ContextData: ContextDataType=None) -> AdminRespondToAuthChallengeResponse:
        """Responds to an authentication challenge, as an administrator. Requires developer credentials.

        :param self:
        :param ContextData:  Contextual data such as the user's device fingerprint, IP address, or location used for evaluating the risk of an unexpected event by Amazon Cognito advanced security.:<br/>AnalyticsMetadata:  The analytics metadata for collecting Amazon Pinpoint metrics for <code>AdminRespondToAuthChallenge</code> calls.:<br/>Session:  The session which should be passed both ways in challenge-response calls to the service. If <code>InitiateAuth</code> or <code>RespondToAuthChallenge</code> API call determines that the caller needs to go through another challenge, they return a session with other challenge parameters. This session should be passed as it is to the next <code>RespondToAuthChallenge</code> API call.<br/>ChallengeResponses:  The challenge responses. These are inputs corresponding to the value of <code>ChallengeName</code>, for example: <ul> <li>  <code>SMS_MFA</code>: <code>SMS_MFA_CODE</code>, <code>USERNAME</code>, <code>SECRET_HASH</code> (if app client is configured with client secret). </li> <li>  <code>PASSWORD_VERIFIER</code>: <code>PASSWORD_CLAIM_SIGNATURE</code>, <code>PASSWORD_CLAIM_SECRET_BLOCK</code>, <code>TIMESTAMP</code>, <code>USERNAME</code>, <code>SECRET_HASH</code> (if app client is configured with client secret). </li> <li>  <code>ADMIN_NO_SRP_AUTH</code>: <code>PASSWORD</code>, <code>USERNAME</code>, <code>SECRET_HASH</code> (if app client is configured with client secret).  </li> <li>  <code>NEW_PASSWORD_REQUIRED</code>: <code>NEW_PASSWORD</code>, any other required attributes, <code>USERNAME</code>, <code>SECRET_HASH</code> (if app client is configured with client secret).  </li> </ul> The value of the <code>USERNAME</code> attribute must be the user's actual username, not an alias (such as email address or phone number). To make this easier, the <code>AdminInitiateAuth</code> response includes the actual username value in the <code>USERNAMEUSER_ID_FOR_SRP</code> attribute, even if you specified an alias in your call to <code>AdminInitiateAuth</code>.<br/>ChallengeName:  The challenge name. For more information, see .<br/>ClientId:  The app client ID.<br/>UserPoolId:  The ID of the Amazon Cognito user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AnalyticsEndpointId</i> The endpoint ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncodedData</i> Encoded data containing device fingerprinting details, collected using the Amazon Cognito context data collection library.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HttpHeaders</i> HttpHeaders received on your server in same order.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerPath</i> Your server path where this API is invoked. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerName</i> Your server endpoint where this API is invoked.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IpAddress</i> Source IP address of your user.<br/>
        :return: AuthenticationResult:  The result returned by the server in response to the authentication request.:<br/>ChallengeParameters:  The challenge parameters. For more information, see .<br/>Session:  The session which should be passed both ways in challenge-response calls to the service. If the or API call determines that the caller needs to go through another challenge, they return a session with other challenge parameters. This session should be passed as it is to the next <code>RespondToAuthChallenge</code> API call.<br/>ChallengeName:  The name of the challenge. For more information, see .<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NewDeviceMetadata</i> The new device metadata from an authentication result.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NewDeviceMetadata</b>: The new device metadata from an authentication result.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IdToken</i> The ID token.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RefreshToken</i> The refresh token.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TokenType</i> The token type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpiresIn</i> The expiration period of the authentication result in seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccessToken</i> The access token.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceGroupKey</i> The device group key.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceKey</i> The device key.<br/>        """
        pass

    def admin_set_user_mfa_preference(self, UserPoolId: str, Username: str, SMSMfaSettings: SMSMfaSettingsType=None, SoftwareTokenMfaSettings: SoftwareTokenMfaSettingsType=None) -> AdminSetUserMFAPreferenceResponse:
        """Sets the user's multi-factor authentication (MFA) preference.

        :param self:
        :param UserPoolId:  The user pool ID.<br/>Username:  The user pool username or alias.<br/>SoftwareTokenMfaSettings:  The time-based one-time password software token MFA settings.:<br/>SMSMfaSettings:  The SMS text message MFA settings.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMfa</i> The preferred MFA method.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether SMS text message MFA is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMfa</i> The preferred MFA method.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether software token MFA is enabled.<br/>
        :return:         """
        pass

    def admin_set_user_password(self, Password: str, Username: str, UserPoolId: str, Permanent: BooleanType=None) -> AdminSetUserPasswordResponse:
        """

        :param self:
        :param Permanent:  <br/>Password:  <br/>Username:  <br/>UserPoolId:  <br/>
        :return:         """
        pass

    def admin_set_user_settings(self, MFAOptions: List[MFAOptionType], Username: str, UserPoolId: str) -> AdminSetUserSettingsResponse:
        """Sets all the user settings for a specified user name. Works on any user. Requires developer credentials.

        :param self:
        :param MFAOptions:  Specifies the options for MFA (e.g., email or phone number).<br/>Username:  The user name of the user for whom you wish to set user settings.<br/>UserPoolId:  The user pool ID for the user pool where you want to set the user's settings, such as MFA options.<br/>
        :return:         """
        pass

    def admin_update_auth_event_feedback(self, FeedbackValue: str, EventId: str, Username: str, UserPoolId: str) -> AdminUpdateAuthEventFeedbackResponse:
        """Provides feedback for an authentication event as to whether it was from a valid user. This feedback is used for improving the risk evaluation decision for the user pool as part of Amazon Cognito advanced security.

        :param self:
        :param FeedbackValue:  The authentication event feedback value.<br/>EventId:  The authentication event ID.<br/>Username:  The user pool username.<br/>UserPoolId:  The user pool ID.<br/>
        :return:         """
        pass

    def admin_update_device_status(self, DeviceKey: str, Username: str, UserPoolId: str, DeviceRememberedStatus: str=None) -> AdminUpdateDeviceStatusResponse:
        """Updates the device status as an administrator. Requires developer credentials.

        :param self:
        :param DeviceRememberedStatus:  The status indicating whether a device has been remembered or not.<br/>DeviceKey:  The device key.<br/>Username:  The user name.<br/>UserPoolId:  The user pool ID.<br/>
        :return:         """
        pass

    def admin_update_user_attributes(self, UserAttributes: List[AttributeType], Username: str, UserPoolId: str) -> AdminUpdateUserAttributesResponse:
        """Updates the specified user's attributes, including developer attributes, as an administrator. Works on any user. For custom attributes, you must prepend the <code>custom:</code> prefix to the attribute name. In addition to updating user attributes, this API can also be used to mark phone and email as verified. Requires developer credentials.

        :param self:
        :param UserAttributes:  An array of name-value pairs representing user attributes. For custom attributes, you must prepend the <code>custom:</code> prefix to the attribute name.<br/>Username:  The user name of the user for whom you want to update user attributes.<br/>UserPoolId:  The user pool ID for the user pool where you want to update user attributes.<br/>
        :return:         """
        pass

    def admin_user_global_sign_out(self, Username: str, UserPoolId: str) -> AdminUserGlobalSignOutResponse:
        """Signs out users from all devices, as an administrator. Requires developer credentials.

        :param self:
        :param Username:  The user name.<br/>UserPoolId:  The user pool ID.<br/>
        :return:         """
        pass

    def associate_software_token(self, AccessToken: str=None, Session: str=None) -> AssociateSoftwareTokenResponse:
        """Returns a unique generated shared secret key code for the user account. The request takes an access token or a session string, but not both.

        :param self:
        :param Session:  The session which should be passed both ways in challenge-response calls to the service. This allows authentication of the user as part of the MFA setup process.<br/>AccessToken:  The access token.<br/>
        :return: Session:  The session which should be passed both ways in challenge-response calls to the service. This allows authentication of the user as part of the MFA setup process.<br/>SecretCode:  A unique generated shared secret code that is used in the TOTP algorithm to generate a one time code.<br/>        """
        pass

    def change_password(self, AccessToken: str, ProposedPassword: str, PreviousPassword: str) -> ChangePasswordResponse:
        """Changes the password for a specified user in a user pool.

        :param self:
        :param AccessToken:  The access token.<br/>ProposedPassword:  The new password.<br/>PreviousPassword:  The old password.<br/>
        :return:         """
        pass

    def confirm_device(self, DeviceKey: str, AccessToken: str, DeviceSecretVerifierConfig: DeviceSecretVerifierConfigType=None, DeviceName: str=None) -> ConfirmDeviceResponse:
        """Confirms tracking of the device. This API call is the call that begins device tracking.

        :param self:
        :param DeviceName:  The device name.<br/>DeviceSecretVerifierConfig:  The configuration of the device secret verifier.:<br/>DeviceKey:  The device key.<br/>AccessToken:  The access token.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Salt</i> The salt.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PasswordVerifier</i> The password verifier.<br/>
        :return: UserConfirmationNecessary:  Indicates whether the user confirmation is necessary to confirm the device response.<br/>        """
        pass

    def confirm_forgot_password(self, Password: str, ConfirmationCode: str, Username: str, ClientId: str, SecretHash: str=None, AnalyticsMetadata: AnalyticsMetadataType=None, UserContextData: UserContextDataType=None) -> ConfirmForgotPasswordResponse:
        """Allows a user to enter a confirmation code to reset a forgotten password.

        :param self:
        :param UserContextData:  Contextual data such as the user's device fingerprint, IP address, or location used for evaluating the risk of an unexpected event by Amazon Cognito advanced security.:<br/>AnalyticsMetadata:  The Amazon Pinpoint analytics metadata for collecting metrics for <code>ConfirmForgotPassword</code> calls.:<br/>Password:  The password sent by a user's request to retrieve a forgotten password.<br/>ConfirmationCode:  The confirmation code sent by a user's request to retrieve a forgotten password. For more information, see <br/>Username:  The user name of the user for whom you want to enter a code to retrieve a forgotten password.<br/>SecretHash:  A keyed-hash message authentication code (HMAC) calculated using the secret key of a user pool client and username plus the client ID in the message.<br/>ClientId:  The app client ID of the app associated with the user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AnalyticsEndpointId</i> The endpoint ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncodedData</i> Contextual data such as the user's device fingerprint, IP address, or location used for evaluating the risk of an unexpected event by Amazon Cognito advanced security.<br/>
        :return:         """
        pass

    def confirm_sign_up(self, ConfirmationCode: str, Username: str, ClientId: str, SecretHash: str=None, ForceAliasCreation: ForceAliasCreation=None, AnalyticsMetadata: AnalyticsMetadataType=None, UserContextData: UserContextDataType=None) -> ConfirmSignUpResponse:
        """Confirms registration of a user and handles the existing alias from a previous user.

        :param self:
        :param UserContextData:  Contextual data such as the user's device fingerprint, IP address, or location used for evaluating the risk of an unexpected event by Amazon Cognito advanced security.:<br/>AnalyticsMetadata:  The Amazon Pinpoint analytics metadata for collecting metrics for <code>ConfirmSignUp</code> calls.:<br/>ForceAliasCreation:  Boolean to be specified to force user confirmation irrespective of existing alias. By default set to <code>False</code>. If this parameter is set to <code>True</code> and the phone number/email used for sign up confirmation already exists as an alias with a different user, the API call will migrate the alias from the previous user to the newly created user being confirmed. If set to <code>False</code>, the API will throw an <b>AliasExistsException</b> error.<br/>ConfirmationCode:  The confirmation code sent by a user's request to confirm registration.<br/>Username:  The user name of the user whose registration you wish to confirm.<br/>SecretHash:  A keyed-hash message authentication code (HMAC) calculated using the secret key of a user pool client and username plus the client ID in the message.<br/>ClientId:  The ID of the app client associated with the user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AnalyticsEndpointId</i> The endpoint ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncodedData</i> Contextual data such as the user's device fingerprint, IP address, or location used for evaluating the risk of an unexpected event by Amazon Cognito advanced security.<br/>
        :return:         """
        pass

    def create_group(self, UserPoolId: str, GroupName: str, Description: str=None, RoleArn: str=None, Precedence: int=None) -> CreateGroupResponse:
        """Creates a new group in the specified user pool. Requires developer credentials.

        :param self:
        :param Precedence:  A nonnegative integer value that specifies the precedence of this group relative to the other groups that a user can belong to in the user pool. Zero is the highest precedence value. Groups with lower <code>Precedence</code> values take precedence over groups with higher or null <code>Precedence</code> values. If a user belongs to two or more groups, it is the group with the lowest precedence value whose role ARN will be used in the <code>cognito:roles</code> and <code>cognito:preferred_role</code> claims in the user's tokens. Two groups can have the same <code>Precedence</code> value. If this happens, neither group takes precedence over the other. If two groups with the same <code>Precedence</code> have the same role ARN, that role is used in the <code>cognito:preferred_role</code> claim in tokens for users in each group. If the two groups have different role ARNs, the <code>cognito:preferred_role</code> claim is not set in users' tokens. The default <code>Precedence</code> value is null.<br/>RoleArn:  The role ARN for the group.<br/>Description:  A string containing the description of the group.<br/>UserPoolId:  The user pool ID for the user pool.<br/>GroupName:  The name of the group. Must be unique.<br/>
        :return: Group:  The group object for the group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date the group was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date the group was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Precedence</i> A nonnegative integer value that specifies the precedence of this group relative to the other groups that a user can belong to in the user pool. If a user belongs to two or more groups, it is the group with the highest precedence whose role ARN will be used in the <code>cognito:roles</code> and <code>cognito:preferred_role</code> claims in the user's tokens. Groups with higher <code>Precedence</code> values take precedence over groups with lower <code>Precedence</code> values or with null <code>Precedence</code> values. Two groups can have the same <code>Precedence</code> value. If this happens, neither group takes precedence over the other. If two groups with the same <code>Precedence</code> have the same role ARN, that role is used in the <code>cognito:preferred_role</code> claim in tokens for users in each group. If the two groups have different role ARNs, the <code>cognito:preferred_role</code> claim is not set in users' tokens. The default <code>Precedence</code> value is null.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The role ARN for the group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A string containing the description of the group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolId</i> The user pool ID for the user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupName</i> The name of the group.<br/>        """
        pass

    def create_identity_provider(self, ProviderDetails: ProviderDetailsType, ProviderType: str, ProviderName: str, UserPoolId: str, AttributeMapping: AttributeMappingType=None, IdpIdentifiers: List[IdpIdentifierType]=None) -> CreateIdentityProviderResponse:
        """Creates an identity provider for a user pool.

        :param self:
        :param IdpIdentifiers:  A list of identity provider identifiers.<br/>AttributeMapping:  A mapping of identity provider attributes to standard and custom user pool attributes.<br/>ProviderDetails:  The identity provider details, such as <code>MetadataURL</code> and <code>MetadataFile</code>.<br/>ProviderType:  The identity provider type.<br/>ProviderName:  The identity provider name.<br/>UserPoolId:  The user pool ID.<br/>
        :return: IdentityProvider:  The newly created identity provider object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date the identity provider was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date the identity provider was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IdpIdentifiers</i> A list of identity provider identifiers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttributeMapping</i> A mapping of identity provider attributes to standard and custom user pool attributes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProviderDetails</i> The identity provider details, such as <code>MetadataURL</code> and <code>MetadataFile</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProviderType</i> The identity provider type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProviderName</i> The identity provider name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolId</i> The user pool ID.<br/>        """
        pass

    def create_resource_server(self, Name: str, Identifier: str, UserPoolId: str, Scopes: List[ResourceServerScopeType]=None) -> CreateResourceServerResponse:
        """Creates a new OAuth2.0 resource server and defines custom scopes in it.

        :param self:
        :param Scopes:  A list of scopes. Each scope is map, where the keys are <code>name</code> and <code>description</code>.<br/>Name:  A friendly name for the resource server.<br/>Identifier:  A unique resource server identifier for the resource server. This could be an HTTPS endpoint where the resource server is located. For example, <code>https://my-weather-api.example.com</code>.<br/>UserPoolId:  The user pool ID for the user pool.<br/>
        :return: ResourceServer:  The newly created resource server.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Scopes</i> A list of scopes that are defined for the resource server.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the resource server.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Identifier</i> The identifier for the resource server.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolId</i> The user pool ID for the user pool that hosts the resource server.<br/>        """
        pass

    def create_user_import_job(self, CloudWatchLogsRoleArn: str, UserPoolId: str, JobName: str) -> CreateUserImportJobResponse:
        """Creates the user import job.

        :param self:
        :param CloudWatchLogsRoleArn:  The role ARN for the Amazon CloudWatch Logging role for the user import job.<br/>UserPoolId:  The user pool ID for the user pool that the users are being imported into.<br/>JobName:  The job name for the user import job.<br/>
        :return: UserImportJob:  The job object that represents the user import job.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompletionMessage</i> The message returned when the user import job is completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailedUsers</i> The number of users that could not be imported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SkippedUsers</i> The number of users that were skipped.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImportedUsers</i> The number of users that were successfully imported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchLogsRoleArn</i> The role ARN for the Amazon CloudWatch Logging role for the user import job. For more information, see "Creating the CloudWatch Logs IAM Role" in the Amazon Cognito Developer Guide.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the user import job. One of the following: <ul> <li>  <code>Created</code> - The job was created but not started. </li> <li>  <code>Pending</code> - A transition state. You have started the job, but it has not begun importing users yet. </li> <li>  <code>InProgress</code> - The job has started, and users are being imported. </li> <li>  <code>Stopping</code> - You have stopped the job, but the job has not stopped importing users yet. </li> <li>  <code>Stopped</code> - You have stopped the job, and the job has stopped importing users. </li> <li>  <code>Succeeded</code> - The job has completed successfully. </li> <li>  <code>Failed</code> - The job has stopped due to an error. </li> <li>  <code>Expired</code> - You created a job, but did not start the job within 24-48 hours. All data associated with the job was deleted, and the job cannot be started. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompletionDate</i> The date when the user import job was completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartDate</i> The date when the user import job was started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date the user import job was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreSignedUrl</i> The pre-signed URL to be used to upload the <code>.csv</code> file.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolId</i> The user pool ID for the user pool that the users are being imported into.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobId</i> The job ID for the user import job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobName</i> The job name for the user import job.<br/>        """
        pass

    def create_user_pool(self, PoolName: str, Policies: UserPoolPolicyType=None, LambdaConfig: LambdaConfigType=None, AutoVerifiedAttributes: List[VerifiedAttributeType]=None, AliasAttributes: List[AliasAttributeType]=None, UsernameAttributes: List[UsernameAttributeType]=None, SmsVerificationMessage: str=None, EmailVerificationMessage: str=None, EmailVerificationSubject: str=None, VerificationMessageTemplate: VerificationMessageTemplateType=None, SmsAuthenticationMessage: str=None, MfaConfiguration: str=None, DeviceConfiguration: DeviceConfigurationType=None, EmailConfiguration: EmailConfigurationType=None, SmsConfiguration: SmsConfigurationType=None, UserPoolTags: UserPoolTagsType=None, AdminCreateUserConfig: AdminCreateUserConfigType=None, Schema: List[SchemaAttributeType]=None, UserPoolAddOns: UserPoolAddOnsType=None) -> CreateUserPoolResponse:
        """Creates a new Amazon Cognito user pool and sets the password policy for the pool.

        :param self:
        :param UserPoolAddOns:  Used to enable advanced security risk detection. Set the key <code>AdvancedSecurityMode</code> to the value "AUDIT".:<br/>Schema:  An array of schema attributes for the new user pool. These attributes can be standard or custom attributes.<br/>AdminCreateUserConfig:  The configuration for <code>AdminCreateUser</code> requests.:<br/>UserPoolTags:  The tag keys and values to assign to the user pool. A tag is a label that you can use to categorize and manage user pools in different ways, such as by purpose, owner, environment, or other criteria.<br/>SmsConfiguration:  The SMS configuration.:<br/>EmailConfiguration:  The email configuration.:<br/>DeviceConfiguration:  The device configuration.:<br/>MfaConfiguration:  Specifies MFA configuration details.<br/>SmsAuthenticationMessage:  A string representing the SMS authentication message.<br/>VerificationMessageTemplate:  The template for the verification message that the user sees when the app requests permission to access the user's information.:<br/>EmailVerificationSubject:  A string representing the email verification subject.<br/>EmailVerificationMessage:  A string representing the email verification message.<br/>SmsVerificationMessage:  A string representing the SMS verification message.<br/>UsernameAttributes:  Specifies whether email addresses or phone numbers can be specified as usernames when a user signs up.<br/>AliasAttributes:  Attributes supported as an alias for this user pool. Possible values: <b>phone_number</b>, <b>email</b>, or <b>preferred_username</b>.<br/>AutoVerifiedAttributes:  The attributes to be auto-verified. Possible values: <b>email</b>, <b>phone_number</b>.<br/>LambdaConfig:  The Lambda trigger configuration information for the new user pool. <note> In a push model, event sources (such as Amazon S3 and custom applications) need permission to invoke a function. So you will need to make an extra call to add permission for these event sources to invoke your Lambda function. <p/> For more information on using the Lambda API to add permission, see <a href="https://docs.aws.amazon.com/lambda/latest/dg/API_AddPermission.html"> AddPermission </a>.  For adding permission using the AWS CLI, see <a href="https://docs.aws.amazon.com/cli/latest/reference/lambda/add-permission.html"> add-permission </a>. </note>:<br/>Policies:  The policies associated with the new user pool.:<br/>PoolName:  A string used to name the user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PasswordPolicy</i> The password policy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PasswordPolicy</b>: The password policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TemporaryPasswordValidityDays</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RequireSymbols</i> In the password policy that you have set, refers to whether you have required users to use at least one symbol in their password.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RequireNumbers</i> In the password policy that you have set, refers to whether you have required users to use at least one number in their password.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RequireLowercase</i> In the password policy that you have set, refers to whether you have required users to use at least one lowercase letter in their password.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RequireUppercase</i> In the password policy that you have set, refers to whether you have required users to use at least one uppercase letter in their password.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinimumLength</i> The minimum length of the password policy that you have set. Cannot be less than 6.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserMigration</i> The user migration Lambda config type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreTokenGeneration</i> A Lambda trigger that is invoked before token generation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VerifyAuthChallengeResponse</i> Verifies the authentication challenge response.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateAuthChallenge</i> Creates an authentication challenge.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefineAuthChallenge</i> Defines the authentication challenge.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PostAuthentication</i> A post-authentication AWS Lambda trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreAuthentication</i> A pre-authentication AWS Lambda trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PostConfirmation</i> A post-confirmation AWS Lambda trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomMessage</i> A custom Message AWS Lambda trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreSignUp</i> A pre-registration AWS Lambda trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultEmailOption</i> The default email option.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailSubjectByLink</i> The subject line for the email message template for sending a confirmation link to the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailMessageByLink</i> The email message template for sending a confirmation link to the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailSubject</i> The subject line for the email message template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailMessage</i> The email message template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SmsMessage</i> The SMS message template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceOnlyRememberedOnUserPrompt</i> If true, a device is only remembered on user prompt.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ChallengeRequiredOnNewDevice</i> Indicates whether a challenge is required on a new device. Only applicable to a new device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailSendingAccount</i> Specifies whether Amazon Cognito emails your users by using its built-in email functionality or your Amazon SES email configuration. Specify one of the following values: <dl> <dt>COGNITO_DEFAULT</dt> <dd> When Amazon Cognito emails your users, it uses its built-in email functionality. When you use the default option, Amazon Cognito allows only a limited number of emails each day for your user pool. For typical production environments, the default email limit is below the required delivery volume. To achieve a higher delivery volume, specify DEVELOPER to use your Amazon SES email configuration. To look up the email delivery limit for the default option, see <a href="https://docs.aws.amazon.com/cognito/latest/developerguide/limits.html">Limits in Amazon Cognito</a> in the <i>Amazon Cognito Developer Guide</i>. The default FROM address is no-reply@verificationemail.com. To customize the FROM address, provide the ARN of an Amazon SES verified email address for the <code>SourceArn</code> parameter. </dd> <dt>DEVELOPER</dt> <dd> When Amazon Cognito emails your users, it uses your Amazon SES configuration. Amazon Cognito calls Amazon SES on your behalf to send email from your verified email address. When you use this option, the email delivery limits are the same limits that apply to your Amazon SES verified email address in your AWS account. If you use this option, you must provide the ARN of an Amazon SES verified email address for the <code>SourceArn</code> parameter. Before Amazon Cognito can email your users, it requires additional permissions to call Amazon SES on your behalf. When you update your user pool with this option, Amazon Cognito creates a <i>service-linked role</i>, which is a type of IAM role, in your AWS account. This role contains the permissions that allow Amazon Cognito to access Amazon SES and send email messages with your address. For more information about the service-linked role that Amazon Cognito creates, see <a href="https://docs.aws.amazon.com/cognito/latest/developerguide/using-service-linked-roles.html">Using Service-Linked Roles for Amazon Cognito</a> in the <i>Amazon Cognito Developer Guide</i>. </dd> </dl><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplyToEmailAddress</i> The destination to which the receiver of the email should reply to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceArn</i> The Amazon Resource Name (ARN) of a verified email address in Amazon SES. This email address is used in one of the following ways, depending on the value that you specify for the <code>EmailSendingAccount</code> parameter: <ul> <li> If you specify <code>COGNITO_DEFAULT</code>, Amazon Cognito uses this address as the custom FROM address when it emails your users by using its built-in email account. </li> <li> If you specify <code>DEVELOPER</code>, Amazon Cognito emails your users with this address by calling Amazon SES on your behalf. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalId</i> The external ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsCallerArn</i> The Amazon Resource Name (ARN) of the Amazon Simple Notification Service (SNS) caller.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InviteMessageTemplate</i> The message template to be used for the welcome message to new users. See also <a href="http://docs.aws.amazon.com/cognito/latest/developerguide/cognito-user-pool-settings-message-customizations.html#cognito-user-pool-settings-user-invitation-message-customization">Customizing User Invitation Messages</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InviteMessageTemplate</b>: The message template to be used for the welcome message to new users. See also <a href="http://docs.aws.amazon.com/cognito/latest/developerguide/cognito-user-pool-settings-message-customizations.html#cognito-user-pool-settings-user-invitation-message-customization">Customizing User Invitation Messages</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UnusedAccountValidityDays</i> The user account expiration limit, in days, after which the account is no longer usable. To reset the account after that time limit, you must call <code>AdminCreateUser</code> again, specifying <code>"RESEND"</code> for the <code>MessageAction</code> parameter. The default value for this parameter is 7.  <note> If you set a value for <code>TemporaryPasswordValidityDays</code> in <code>PasswordPolicy</code>, that value will be used and <code>UnusedAccountValidityDays</code> will be deprecated for that user pool.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowAdminCreateUserOnly</i> Set to <code>True</code> if only the administrator is allowed to create user profiles. Set to <code>False</code> if users can sign themselves up via an app.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailSubject</i> The subject line for email messages.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailMessage</i> The message template for email messages.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SMSMessage</i> The message template for SMS messages.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdvancedSecurityMode</i> The advanced security mode.<br/>
        :return: UserPool:  A container for the user pool details.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) for the user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolAddOns</i> The user pool add-ons.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>UserPoolAddOns</b>: The user pool add-ons.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdminCreateUserConfig</i> The configuration for <code>AdminCreateUser</code> requests.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AdminCreateUserConfig</b>: The configuration for <code>AdminCreateUser</code> requests.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomDomain</i> A custom domain name that you provide to Amazon Cognito. This parameter applies only if you use a custom domain to host the sign-up and sign-in pages for your application. For example: <code>auth.example.com</code>. For more information about adding a custom domain to your user pool, see <a href="https://docs.aws.amazon.com/cognito/latest/developerguide/cognito-user-pools-add-custom-domain.html">Using Your Own Domain for the Hosted UI</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Domain</i> Holds the domain prefix if the user pool has a domain associated with it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailConfigurationFailure</i> The reason why the email configuration cannot send the messages to your users.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SmsConfigurationFailure</i> The reason why the SMS configuration cannot send the messages to your users.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolTags</i> The tags that are assigned to the user pool. A tag is a label that you can apply to user pools to categorize and manage them in different ways, such as by purpose, owner, environment, or other criteria.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SmsConfiguration</i> The SMS configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SmsConfiguration</b>: The SMS configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailConfiguration</i> The email configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EmailConfiguration</b>: The email configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedNumberOfUsers</i> A number estimating the size of the user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceConfiguration</i> The device configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DeviceConfiguration</b>: The device configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MfaConfiguration</i> Can be one of the following values: <ul> <li>  <code>OFF</code> - MFA tokens are not required and cannot be specified during user registration. </li> <li>  <code>ON</code> - MFA tokens are required for all user registrations. You can only specify required when you are initially creating a user pool. </li> <li>  <code>OPTIONAL</code> - Users have the option when registering to create an MFA token. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SmsAuthenticationMessage</i> The contents of the SMS authentication message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VerificationMessageTemplate</i> The template for verification messages.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VerificationMessageTemplate</b>: The template for verification messages.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailVerificationSubject</i> The subject of the email verification message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailVerificationMessage</i> The contents of the email verification message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SmsVerificationMessage</i> The contents of the SMS verification message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UsernameAttributes</i> Specifies whether email addresses or phone numbers can be specified as usernames when a user signs up.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AliasAttributes</i> Specifies the attributes that are aliased in a user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoVerifiedAttributes</i> Specifies the attributes that are auto-verified in a user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SchemaAttributes</i> A container with the schema attributes of a user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date the user pool was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date the user pool was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of a user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LambdaConfig</i> The AWS Lambda triggers associated with the user pool.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LambdaConfig</b>: The AWS Lambda triggers associated with the user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Policies</i> The policies associated with the user pool.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Policies</b>: The policies associated with the user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the user pool.<br/>The password policy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PasswordPolicy</b>: The password policy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UserMigration</i> The user migration Lambda config type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PreTokenGeneration</i> A Lambda trigger that is invoked before token generation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VerifyAuthChallengeResponse</i> Verifies the authentication challenge response.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateAuthChallenge</i> Creates an authentication challenge.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DefineAuthChallenge</i> Defines the authentication challenge.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PostAuthentication</i> A post-authentication AWS Lambda trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PreAuthentication</i> A pre-authentication AWS Lambda trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PostConfirmation</i> A post-confirmation AWS Lambda trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomMessage</i> A custom Message AWS Lambda trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PreSignUp</i> A pre-registration AWS Lambda trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultEmailOption</i> The default email option.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailSubjectByLink</i> The subject line for the email message template for sending a confirmation link to the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailMessageByLink</i> The email message template for sending a confirmation link to the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailSubject</i> The subject line for the email message template.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailMessage</i> The email message template.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SmsMessage</i> The SMS message template.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceOnlyRememberedOnUserPrompt</i> If true, a device is only remembered on user prompt.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ChallengeRequiredOnNewDevice</i> Indicates whether a challenge is required on a new device. Only applicable to a new device.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailSendingAccount</i> Specifies whether Amazon Cognito emails your users by using its built-in email functionality or your Amazon SES email configuration. Specify one of the following values: <dl> <dt>COGNITO_DEFAULT</dt> <dd> When Amazon Cognito emails your users, it uses its built-in email functionality. When you use the default option, Amazon Cognito allows only a limited number of emails each day for your user pool. For typical production environments, the default email limit is below the required delivery volume. To achieve a higher delivery volume, specify DEVELOPER to use your Amazon SES email configuration. To look up the email delivery limit for the default option, see <a href="https://docs.aws.amazon.com/cognito/latest/developerguide/limits.html">Limits in Amazon Cognito</a> in the <i>Amazon Cognito Developer Guide</i>. The default FROM address is no-reply@verificationemail.com. To customize the FROM address, provide the ARN of an Amazon SES verified email address for the <code>SourceArn</code> parameter. </dd> <dt>DEVELOPER</dt> <dd> When Amazon Cognito emails your users, it uses your Amazon SES configuration. Amazon Cognito calls Amazon SES on your behalf to send email from your verified email address. When you use this option, the email delivery limits are the same limits that apply to your Amazon SES verified email address in your AWS account. If you use this option, you must provide the ARN of an Amazon SES verified email address for the <code>SourceArn</code> parameter. Before Amazon Cognito can email your users, it requires additional permissions to call Amazon SES on your behalf. When you update your user pool with this option, Amazon Cognito creates a <i>service-linked role</i>, which is a type of IAM role, in your AWS account. This role contains the permissions that allow Amazon Cognito to access Amazon SES and send email messages with your address. For more information about the service-linked role that Amazon Cognito creates, see <a href="https://docs.aws.amazon.com/cognito/latest/developerguide/using-service-linked-roles.html">Using Service-Linked Roles for Amazon Cognito</a> in the <i>Amazon Cognito Developer Guide</i>. </dd> </dl><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplyToEmailAddress</i> The destination to which the receiver of the email should reply to.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceArn</i> The Amazon Resource Name (ARN) of a verified email address in Amazon SES. This email address is used in one of the following ways, depending on the value that you specify for the <code>EmailSendingAccount</code> parameter: <ul> <li> If you specify <code>COGNITO_DEFAULT</code>, Amazon Cognito uses this address as the custom FROM address when it emails your users by using its built-in email account. </li> <li> If you specify <code>DEVELOPER</code>, Amazon Cognito emails your users with this address by calling Amazon SES on your behalf. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalId</i> The external ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsCallerArn</i> The Amazon Resource Name (ARN) of the Amazon Simple Notification Service (SNS) caller.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UnusedAccountValidityDays</i> The user account expiration limit, in days, after which the account is no longer usable. To reset the account after that time limit, you must call <code>AdminCreateUser</code> again, specifying <code>"RESEND"</code> for the <code>MessageAction</code> parameter. The default value for this parameter is 7.  <note> If you set a value for <code>TemporaryPasswordValidityDays</code> in <code>PasswordPolicy</code>, that value will be used and <code>UnusedAccountValidityDays</code> will be deprecated for that user pool.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowAdminCreateUserOnly</i> Set to <code>True</code> if only the administrator is allowed to create user profiles. Set to <code>False</code> if users can sign themselves up via an app.<br/>The message template to be used for the welcome message to new users. See also <a href="http://docs.aws.amazon.com/cognito/latest/developerguide/cognito-user-pool-settings-message-customizations.html#cognito-user-pool-settings-user-invitation-message-customization">Customizing User Invitation Messages</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InviteMessageTemplate</b>: The message template to be used for the welcome message to new users. See also <a href="http://docs.aws.amazon.com/cognito/latest/developerguide/cognito-user-pool-settings-message-customizations.html#cognito-user-pool-settings-user-invitation-message-customization">Customizing User Invitation Messages</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AdvancedSecurityMode</i> The advanced security mode.<br/>        """
        pass

    def create_user_pool_client(self, ClientName: str, UserPoolId: str, GenerateSecret: GenerateSecret=None, RefreshTokenValidity: int=None, ReadAttributes: List[ClientPermissionType]=None, WriteAttributes: List[ClientPermissionType]=None, ExplicitAuthFlows: List[ExplicitAuthFlowsType]=None, SupportedIdentityProviders: List[ProviderNameType]=None, CallbackURLs: List[RedirectUrlType]=None, LogoutURLs: List[RedirectUrlType]=None, DefaultRedirectURI: str=None, AllowedOAuthFlows: List[OAuthFlowType]=None, AllowedOAuthScopes: List[ScopeType]=None, AllowedOAuthFlowsUserPoolClient: BooleanType=None, AnalyticsConfiguration: AnalyticsConfigurationType=None) -> CreateUserPoolClientResponse:
        """Creates the user pool client.

        :param self:
        :param AnalyticsConfiguration:  The Amazon Pinpoint analytics configuration for collecting metrics for this user pool.:<br/>AllowedOAuthFlowsUserPoolClient:  Set to <code>True</code> if the client is allowed to follow the OAuth protocol when interacting with Cognito user pools.<br/>AllowedOAuthScopes:  A list of allowed <code>OAuth</code> scopes. Currently supported values are <code>"phone"</code>, <code>"email"</code>, <code>"openid"</code>, and <code>"Cognito"</code>.<br/>AllowedOAuthFlows:  Set to <code>code</code> to initiate a code grant flow, which provides an authorization code as the response. This code can be exchanged for access tokens with the token endpoint. Set to <code>token</code> to specify that the client should get the access token (and, optionally, ID token, based on scopes) directly.<br/>DefaultRedirectURI:  The default redirect URI. Must be in the <code>CallbackURLs</code> list. A redirect URI must: <ul> <li> Be an absolute URI. </li> <li> Be registered with the authorization server. </li> <li> Not include a fragment component. </li> </ul> See <a href="https://tools.ietf.org/html/rfc6749#section-3.1.2">OAuth 2.0 - Redirection Endpoint</a>. Amazon Cognito requires HTTPS over HTTP except for http://localhost for testing purposes only. App callback URLs such as myapp://example are also supported.<br/>LogoutURLs:  A list of allowed logout URLs for the identity providers.<br/>CallbackURLs:  A list of allowed redirect (callback) URLs for the identity providers. A redirect URI must: <ul> <li> Be an absolute URI. </li> <li> Be registered with the authorization server. </li> <li> Not include a fragment component. </li> </ul> See <a href="https://tools.ietf.org/html/rfc6749#section-3.1.2">OAuth 2.0 - Redirection Endpoint</a>. Amazon Cognito requires HTTPS over HTTP except for http://localhost for testing purposes only. App callback URLs such as myapp://example are also supported.<br/>SupportedIdentityProviders:  A list of provider names for the identity providers that are supported on this client. The following are supported: <code>COGNITO</code>, <code>Facebook</code>, <code>Google</code> and <code>LoginWithAmazon</code>.<br/>ExplicitAuthFlows:  The explicit authentication flows.<br/>WriteAttributes:  The user pool attributes that the app client can write to. If your app client allows users to sign in through an identity provider, this array must include all attributes that are mapped to identity provider attributes. Amazon Cognito updates mapped attributes when users sign in to your application through an identity provider. If your app client lacks write access to a mapped attribute, Amazon Cognito throws an error when it attempts to update the attribute. For more information, see <a href="https://docs.aws.amazon.com/cognito/latest/developerguide/cognito-user-pools-specifying-attribute-mapping.html">Specifying Identity Provider Attribute Mappings for Your User Pool</a>.<br/>ReadAttributes:  The read attributes.<br/>RefreshTokenValidity:  The time limit, in days, after which the refresh token is no longer valid and cannot be used.<br/>GenerateSecret:  Boolean to specify whether you want to generate a secret for the user pool client being created.<br/>ClientName:  The client name for the user pool client you would like to create.<br/>UserPoolId:  The user pool ID for the user pool where you want to create a user pool client.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserDataShared</i> If <code>UserDataShared</code> is <code>true</code>, Amazon Cognito will include user data in the events it publishes to Amazon Pinpoint analytics.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalId</i> The external ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The ARN of an IAM role that authorizes Amazon Cognito to publish events to Amazon Pinpoint analytics.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The application ID for an Amazon Pinpoint application.<br/>
        :return: UserPoolClient:  The user pool client that was just created.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AnalyticsConfiguration</i> The Amazon Pinpoint analytics configuration for the user pool client.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AnalyticsConfiguration</b>: The Amazon Pinpoint analytics configuration for the user pool client.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowedOAuthFlowsUserPoolClient</i> Set to TRUE if the client is allowed to follow the OAuth protocol when interacting with Cognito user pools.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowedOAuthScopes</i> A list of allowed <code>OAuth</code> scopes. Currently supported values are <code>"phone"</code>, <code>"email"</code>, <code>"openid"</code>, and <code>"Cognito"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowedOAuthFlows</i> Set to <code>code</code> to initiate a code grant flow, which provides an authorization code as the response. This code can be exchanged for access tokens with the token endpoint. Set to <code>token</code> to specify that the client should get the access token (and, optionally, ID token, based on scopes) directly.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultRedirectURI</i> The default redirect URI. Must be in the <code>CallbackURLs</code> list. A redirect URI must: <ul> <li> Be an absolute URI. </li> <li> Be registered with the authorization server. </li> <li> Not include a fragment component. </li> </ul> See <a href="https://tools.ietf.org/html/rfc6749#section-3.1.2">OAuth 2.0 - Redirection Endpoint</a>. Amazon Cognito requires HTTPS over HTTP except for http://localhost for testing purposes only. App callback URLs such as myapp://example are also supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogoutURLs</i> A list of allowed logout URLs for the identity providers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallbackURLs</i> A list of allowed redirect (callback) URLs for the identity providers. A redirect URI must: <ul> <li> Be an absolute URI. </li> <li> Be registered with the authorization server. </li> <li> Not include a fragment component. </li> </ul> See <a href="https://tools.ietf.org/html/rfc6749#section-3.1.2">OAuth 2.0 - Redirection Endpoint</a>. Amazon Cognito requires HTTPS over HTTP except for http://localhost for testing purposes only. App callback URLs such as myapp://example are also supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedIdentityProviders</i> A list of provider names for the identity providers that are supported on this client.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExplicitAuthFlows</i> The explicit authentication flows.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WriteAttributes</i> The writeable attributes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadAttributes</i> The Read-only attributes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RefreshTokenValidity</i> The time limit, in days, after which the refresh token is no longer valid and cannot be used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date the user pool client was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date the user pool client was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientSecret</i> The client secret from the user pool request of the client type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientId</i> The ID of the client associated with the user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientName</i> The client name from the user pool request of the client type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolId</i> The user pool ID for the user pool client.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UserDataShared</i> If <code>UserDataShared</code> is <code>true</code>, Amazon Cognito will include user data in the events it publishes to Amazon Pinpoint analytics.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalId</i> The external ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The ARN of an IAM role that authorizes Amazon Cognito to publish events to Amazon Pinpoint analytics.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The application ID for an Amazon Pinpoint application.<br/>        """
        pass

    def create_user_pool_domain(self, UserPoolId: str, Domain: str, CustomDomainConfig: CustomDomainConfigType=None) -> CreateUserPoolDomainResponse:
        """Creates a new domain for a user pool.

        :param self:
        :param CustomDomainConfig:  The configuration for a custom domain that hosts the sign-up and sign-in webpages for your application. Provide this parameter only if you want to use a custom domain for your user pool. Otherwise, you can exclude this parameter and use the Amazon Cognito hosted domain instead. For more information about the hosted domain and custom domains, see <a href="https://docs.aws.amazon.com/cognito/latest/developerguide/cognito-user-pools-assign-domain.html">Configuring a User Pool Domain</a>.:<br/>UserPoolId:  The user pool ID.<br/>Domain:  The domain string.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateArn</i> The Amazon Resource Name (ARN) of an AWS Certificate Manager SSL certificate. You use this certificate for the subdomain of your custom domain.<br/>
        :return: CloudFrontDomain:  The Amazon CloudFront endpoint that you use as the target of the alias that you set up with your Domain Name Service (DNS) provider.<br/>        """
        pass

    def delete_group(self, UserPoolId: str, GroupName: str):
        """Deletes a group. Currently only groups with no members can be deleted. Requires developer credentials.

        :param self:
        :param UserPoolId:  The user pool ID for the user pool.<br/>GroupName:  The name of the group.<br/>
        :return:         """
        pass

    def delete_identity_provider(self, ProviderName: str, UserPoolId: str):
        """Deletes an identity provider for a user pool.

        :param self:
        :param ProviderName:  The identity provider name.<br/>UserPoolId:  The user pool ID.<br/>
        :return:         """
        pass

    def delete_resource_server(self, Identifier: str, UserPoolId: str):
        """Deletes a resource server.

        :param self:
        :param Identifier:  The identifier for the resource server.<br/>UserPoolId:  The user pool ID for the user pool that hosts the resource server.<br/>
        :return:         """
        pass

    def delete_user(self, AccessToken: str):
        """Allows a user to delete himself or herself.

        :param self:
        :param AccessToken:  The access token from a request to delete a user.<br/>
        :return:         """
        pass

    def delete_user_attributes(self, AccessToken: str, UserAttributeNames: List[AttributeNameType]) -> DeleteUserAttributesResponse:
        """Deletes the attributes for a user.

        :param self:
        :param AccessToken:  The access token used in the request to delete user attributes.<br/>UserAttributeNames:  An array of strings representing the user attribute names you wish to delete. For custom attributes, you must prepend the <code>custom:</code> prefix to the attribute name.<br/>
        :return:         """
        pass

    def delete_user_pool(self, UserPoolId: str):
        """Deletes the specified Amazon Cognito user pool.

        :param self:
        :param UserPoolId:  The user pool ID for the user pool you want to delete.<br/>
        :return:         """
        pass

    def delete_user_pool_client(self, ClientId: str, UserPoolId: str):
        """Allows the developer to delete the user pool client.

        :param self:
        :param ClientId:  The app client ID of the app associated with the user pool.<br/>UserPoolId:  The user pool ID for the user pool where you want to delete the client.<br/>
        :return:         """
        pass

    def delete_user_pool_domain(self, UserPoolId: str, Domain: str) -> DeleteUserPoolDomainResponse:
        """Deletes a domain for a user pool.

        :param self:
        :param UserPoolId:  The user pool ID.<br/>Domain:  The domain string.<br/>
        :return:         """
        pass

    def describe_identity_provider(self, ProviderName: str, UserPoolId: str) -> DescribeIdentityProviderResponse:
        """Gets information about a specific identity provider.

        :param self:
        :param ProviderName:  The identity provider name.<br/>UserPoolId:  The user pool ID.<br/>
        :return: IdentityProvider:  The identity provider that was deleted.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date the identity provider was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date the identity provider was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IdpIdentifiers</i> A list of identity provider identifiers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttributeMapping</i> A mapping of identity provider attributes to standard and custom user pool attributes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProviderDetails</i> The identity provider details, such as <code>MetadataURL</code> and <code>MetadataFile</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProviderType</i> The identity provider type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProviderName</i> The identity provider name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolId</i> The user pool ID.<br/>        """
        pass

    def describe_resource_server(self, Identifier: str, UserPoolId: str) -> DescribeResourceServerResponse:
        """Describes a resource server.

        :param self:
        :param Identifier:  The identifier for the resource server<br/>UserPoolId:  The user pool ID for the user pool that hosts the resource server.<br/>
        :return: ResourceServer:  The resource server.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Scopes</i> A list of scopes that are defined for the resource server.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the resource server.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Identifier</i> The identifier for the resource server.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolId</i> The user pool ID for the user pool that hosts the resource server.<br/>        """
        pass

    def describe_risk_configuration(self, UserPoolId: str, ClientId: str=None) -> DescribeRiskConfigurationResponse:
        """Describes the risk configuration.

        :param self:
        :param ClientId:  The app client ID.<br/>UserPoolId:  The user pool ID.<br/>
        :return: RiskConfiguration:  The risk configuration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The last modified date.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RiskExceptionConfiguration</i> The configuration to override the risk decision.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RiskExceptionConfiguration</b>: The configuration to override the risk decision.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountTakeoverRiskConfiguration</i> The account takeover risk configuration object including the <code>NotifyConfiguration</code> object and <code>Actions</code> to take in the case of an account takeover.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AccountTakeoverRiskConfiguration</b>: The account takeover risk configuration object including the <code>NotifyConfiguration</code> object and <code>Actions</code> to take in the case of an account takeover.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompromisedCredentialsRiskConfiguration</i> The compromised credentials risk configuration object including the <code>EventFilter</code> and the <code>EventAction</code> &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CompromisedCredentialsRiskConfiguration</b>: The compromised credentials risk configuration object including the <code>EventFilter</code> and the <code>EventAction</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientId</i> The app client ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolId</i> The user pool ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EventFilter</i> Perform the action for these events. The default is to perform all events if no event filter is specified.<br/>The compromised credentials risk configuration actions.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Actions</b>: The compromised credentials risk configuration actions.The notify configuration used to construct email notifications.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NotifyConfiguration</b>: The notify configuration used to construct email notifications.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SkippedIPRangeList</i> Risk detection is not performed on the IP addresses in the range list. The IP range is in CIDR notation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BlockedIPRangeList</i> Overrides the risk decision to always block the pre-authentication requests. The IP range is in CIDR notation: a compact representation of an IP address and its associated routing prefix.<br/>        """
        pass

    def describe_user_import_job(self, JobId: str, UserPoolId: str) -> DescribeUserImportJobResponse:
        """Describes the user import job.

        :param self:
        :param JobId:  The job ID for the user import job.<br/>UserPoolId:  The user pool ID for the user pool that the users are being imported into.<br/>
        :return: UserImportJob:  The job object that represents the user import job.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompletionMessage</i> The message returned when the user import job is completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailedUsers</i> The number of users that could not be imported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SkippedUsers</i> The number of users that were skipped.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImportedUsers</i> The number of users that were successfully imported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchLogsRoleArn</i> The role ARN for the Amazon CloudWatch Logging role for the user import job. For more information, see "Creating the CloudWatch Logs IAM Role" in the Amazon Cognito Developer Guide.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the user import job. One of the following: <ul> <li>  <code>Created</code> - The job was created but not started. </li> <li>  <code>Pending</code> - A transition state. You have started the job, but it has not begun importing users yet. </li> <li>  <code>InProgress</code> - The job has started, and users are being imported. </li> <li>  <code>Stopping</code> - You have stopped the job, but the job has not stopped importing users yet. </li> <li>  <code>Stopped</code> - You have stopped the job, and the job has stopped importing users. </li> <li>  <code>Succeeded</code> - The job has completed successfully. </li> <li>  <code>Failed</code> - The job has stopped due to an error. </li> <li>  <code>Expired</code> - You created a job, but did not start the job within 24-48 hours. All data associated with the job was deleted, and the job cannot be started. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompletionDate</i> The date when the user import job was completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartDate</i> The date when the user import job was started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date the user import job was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreSignedUrl</i> The pre-signed URL to be used to upload the <code>.csv</code> file.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolId</i> The user pool ID for the user pool that the users are being imported into.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobId</i> The job ID for the user import job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobName</i> The job name for the user import job.<br/>        """
        pass

    def describe_user_pool(self, UserPoolId: str) -> DescribeUserPoolResponse:
        """Returns the configuration information and metadata of the specified user pool.

        :param self:
        :param UserPoolId:  The user pool ID for the user pool you want to describe.<br/>
        :return: UserPool:  The container of metadata returned by the server to describe the pool.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) for the user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolAddOns</i> The user pool add-ons.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>UserPoolAddOns</b>: The user pool add-ons.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdminCreateUserConfig</i> The configuration for <code>AdminCreateUser</code> requests.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AdminCreateUserConfig</b>: The configuration for <code>AdminCreateUser</code> requests.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomDomain</i> A custom domain name that you provide to Amazon Cognito. This parameter applies only if you use a custom domain to host the sign-up and sign-in pages for your application. For example: <code>auth.example.com</code>. For more information about adding a custom domain to your user pool, see <a href="https://docs.aws.amazon.com/cognito/latest/developerguide/cognito-user-pools-add-custom-domain.html">Using Your Own Domain for the Hosted UI</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Domain</i> Holds the domain prefix if the user pool has a domain associated with it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailConfigurationFailure</i> The reason why the email configuration cannot send the messages to your users.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SmsConfigurationFailure</i> The reason why the SMS configuration cannot send the messages to your users.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolTags</i> The tags that are assigned to the user pool. A tag is a label that you can apply to user pools to categorize and manage them in different ways, such as by purpose, owner, environment, or other criteria.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SmsConfiguration</i> The SMS configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SmsConfiguration</b>: The SMS configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailConfiguration</i> The email configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EmailConfiguration</b>: The email configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedNumberOfUsers</i> A number estimating the size of the user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceConfiguration</i> The device configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DeviceConfiguration</b>: The device configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MfaConfiguration</i> Can be one of the following values: <ul> <li>  <code>OFF</code> - MFA tokens are not required and cannot be specified during user registration. </li> <li>  <code>ON</code> - MFA tokens are required for all user registrations. You can only specify required when you are initially creating a user pool. </li> <li>  <code>OPTIONAL</code> - Users have the option when registering to create an MFA token. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SmsAuthenticationMessage</i> The contents of the SMS authentication message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VerificationMessageTemplate</i> The template for verification messages.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VerificationMessageTemplate</b>: The template for verification messages.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailVerificationSubject</i> The subject of the email verification message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailVerificationMessage</i> The contents of the email verification message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SmsVerificationMessage</i> The contents of the SMS verification message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UsernameAttributes</i> Specifies whether email addresses or phone numbers can be specified as usernames when a user signs up.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AliasAttributes</i> Specifies the attributes that are aliased in a user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoVerifiedAttributes</i> Specifies the attributes that are auto-verified in a user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SchemaAttributes</i> A container with the schema attributes of a user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date the user pool was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date the user pool was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of a user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LambdaConfig</i> The AWS Lambda triggers associated with the user pool.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LambdaConfig</b>: The AWS Lambda triggers associated with the user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Policies</i> The policies associated with the user pool.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Policies</b>: The policies associated with the user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the user pool.<br/>The password policy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PasswordPolicy</b>: The password policy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UserMigration</i> The user migration Lambda config type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PreTokenGeneration</i> A Lambda trigger that is invoked before token generation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VerifyAuthChallengeResponse</i> Verifies the authentication challenge response.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateAuthChallenge</i> Creates an authentication challenge.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DefineAuthChallenge</i> Defines the authentication challenge.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PostAuthentication</i> A post-authentication AWS Lambda trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PreAuthentication</i> A pre-authentication AWS Lambda trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PostConfirmation</i> A post-confirmation AWS Lambda trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomMessage</i> A custom Message AWS Lambda trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PreSignUp</i> A pre-registration AWS Lambda trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultEmailOption</i> The default email option.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailSubjectByLink</i> The subject line for the email message template for sending a confirmation link to the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailMessageByLink</i> The email message template for sending a confirmation link to the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailSubject</i> The subject line for the email message template.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailMessage</i> The email message template.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SmsMessage</i> The SMS message template.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceOnlyRememberedOnUserPrompt</i> If true, a device is only remembered on user prompt.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ChallengeRequiredOnNewDevice</i> Indicates whether a challenge is required on a new device. Only applicable to a new device.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailSendingAccount</i> Specifies whether Amazon Cognito emails your users by using its built-in email functionality or your Amazon SES email configuration. Specify one of the following values: <dl> <dt>COGNITO_DEFAULT</dt> <dd> When Amazon Cognito emails your users, it uses its built-in email functionality. When you use the default option, Amazon Cognito allows only a limited number of emails each day for your user pool. For typical production environments, the default email limit is below the required delivery volume. To achieve a higher delivery volume, specify DEVELOPER to use your Amazon SES email configuration. To look up the email delivery limit for the default option, see <a href="https://docs.aws.amazon.com/cognito/latest/developerguide/limits.html">Limits in Amazon Cognito</a> in the <i>Amazon Cognito Developer Guide</i>. The default FROM address is no-reply@verificationemail.com. To customize the FROM address, provide the ARN of an Amazon SES verified email address for the <code>SourceArn</code> parameter. </dd> <dt>DEVELOPER</dt> <dd> When Amazon Cognito emails your users, it uses your Amazon SES configuration. Amazon Cognito calls Amazon SES on your behalf to send email from your verified email address. When you use this option, the email delivery limits are the same limits that apply to your Amazon SES verified email address in your AWS account. If you use this option, you must provide the ARN of an Amazon SES verified email address for the <code>SourceArn</code> parameter. Before Amazon Cognito can email your users, it requires additional permissions to call Amazon SES on your behalf. When you update your user pool with this option, Amazon Cognito creates a <i>service-linked role</i>, which is a type of IAM role, in your AWS account. This role contains the permissions that allow Amazon Cognito to access Amazon SES and send email messages with your address. For more information about the service-linked role that Amazon Cognito creates, see <a href="https://docs.aws.amazon.com/cognito/latest/developerguide/using-service-linked-roles.html">Using Service-Linked Roles for Amazon Cognito</a> in the <i>Amazon Cognito Developer Guide</i>. </dd> </dl><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplyToEmailAddress</i> The destination to which the receiver of the email should reply to.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceArn</i> The Amazon Resource Name (ARN) of a verified email address in Amazon SES. This email address is used in one of the following ways, depending on the value that you specify for the <code>EmailSendingAccount</code> parameter: <ul> <li> If you specify <code>COGNITO_DEFAULT</code>, Amazon Cognito uses this address as the custom FROM address when it emails your users by using its built-in email account. </li> <li> If you specify <code>DEVELOPER</code>, Amazon Cognito emails your users with this address by calling Amazon SES on your behalf. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalId</i> The external ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsCallerArn</i> The Amazon Resource Name (ARN) of the Amazon Simple Notification Service (SNS) caller.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UnusedAccountValidityDays</i> The user account expiration limit, in days, after which the account is no longer usable. To reset the account after that time limit, you must call <code>AdminCreateUser</code> again, specifying <code>"RESEND"</code> for the <code>MessageAction</code> parameter. The default value for this parameter is 7.  <note> If you set a value for <code>TemporaryPasswordValidityDays</code> in <code>PasswordPolicy</code>, that value will be used and <code>UnusedAccountValidityDays</code> will be deprecated for that user pool.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowAdminCreateUserOnly</i> Set to <code>True</code> if only the administrator is allowed to create user profiles. Set to <code>False</code> if users can sign themselves up via an app.<br/>The message template to be used for the welcome message to new users. See also <a href="http://docs.aws.amazon.com/cognito/latest/developerguide/cognito-user-pool-settings-message-customizations.html#cognito-user-pool-settings-user-invitation-message-customization">Customizing User Invitation Messages</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InviteMessageTemplate</b>: The message template to be used for the welcome message to new users. See also <a href="http://docs.aws.amazon.com/cognito/latest/developerguide/cognito-user-pool-settings-message-customizations.html#cognito-user-pool-settings-user-invitation-message-customization">Customizing User Invitation Messages</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AdvancedSecurityMode</i> The advanced security mode.<br/>        """
        pass

    def describe_user_pool_client(self, ClientId: str, UserPoolId: str) -> DescribeUserPoolClientResponse:
        """Client method for returning the configuration information and metadata of the specified user pool app client.

        :param self:
        :param ClientId:  The app client ID of the app associated with the user pool.<br/>UserPoolId:  The user pool ID for the user pool you want to describe.<br/>
        :return: UserPoolClient:  The user pool client from a server response to describe the user pool client.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AnalyticsConfiguration</i> The Amazon Pinpoint analytics configuration for the user pool client.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AnalyticsConfiguration</b>: The Amazon Pinpoint analytics configuration for the user pool client.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowedOAuthFlowsUserPoolClient</i> Set to TRUE if the client is allowed to follow the OAuth protocol when interacting with Cognito user pools.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowedOAuthScopes</i> A list of allowed <code>OAuth</code> scopes. Currently supported values are <code>"phone"</code>, <code>"email"</code>, <code>"openid"</code>, and <code>"Cognito"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowedOAuthFlows</i> Set to <code>code</code> to initiate a code grant flow, which provides an authorization code as the response. This code can be exchanged for access tokens with the token endpoint. Set to <code>token</code> to specify that the client should get the access token (and, optionally, ID token, based on scopes) directly.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultRedirectURI</i> The default redirect URI. Must be in the <code>CallbackURLs</code> list. A redirect URI must: <ul> <li> Be an absolute URI. </li> <li> Be registered with the authorization server. </li> <li> Not include a fragment component. </li> </ul> See <a href="https://tools.ietf.org/html/rfc6749#section-3.1.2">OAuth 2.0 - Redirection Endpoint</a>. Amazon Cognito requires HTTPS over HTTP except for http://localhost for testing purposes only. App callback URLs such as myapp://example are also supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogoutURLs</i> A list of allowed logout URLs for the identity providers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallbackURLs</i> A list of allowed redirect (callback) URLs for the identity providers. A redirect URI must: <ul> <li> Be an absolute URI. </li> <li> Be registered with the authorization server. </li> <li> Not include a fragment component. </li> </ul> See <a href="https://tools.ietf.org/html/rfc6749#section-3.1.2">OAuth 2.0 - Redirection Endpoint</a>. Amazon Cognito requires HTTPS over HTTP except for http://localhost for testing purposes only. App callback URLs such as myapp://example are also supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedIdentityProviders</i> A list of provider names for the identity providers that are supported on this client.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExplicitAuthFlows</i> The explicit authentication flows.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WriteAttributes</i> The writeable attributes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadAttributes</i> The Read-only attributes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RefreshTokenValidity</i> The time limit, in days, after which the refresh token is no longer valid and cannot be used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date the user pool client was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date the user pool client was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientSecret</i> The client secret from the user pool request of the client type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientId</i> The ID of the client associated with the user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientName</i> The client name from the user pool request of the client type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolId</i> The user pool ID for the user pool client.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UserDataShared</i> If <code>UserDataShared</code> is <code>true</code>, Amazon Cognito will include user data in the events it publishes to Amazon Pinpoint analytics.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalId</i> The external ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The ARN of an IAM role that authorizes Amazon Cognito to publish events to Amazon Pinpoint analytics.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The application ID for an Amazon Pinpoint application.<br/>        """
        pass

    def describe_user_pool_domain(self, Domain: str) -> DescribeUserPoolDomainResponse:
        """Gets information about a domain.

        :param self:
        :param Domain:  The domain string.<br/>
        :return: DomainDescription:  A domain description object containing information about the domain.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomDomainConfig</i> The configuration for a custom domain that hosts the sign-up and sign-in webpages for your application.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CustomDomainConfig</b>: The configuration for a custom domain that hosts the sign-up and sign-in webpages for your application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The domain status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The app version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudFrontDistribution</i> The ARN of the CloudFront distribution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Bucket</i> The S3 bucket where the static files for this domain are stored.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Domain</i> The domain string.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AWSAccountId</i> The AWS account ID for the user pool owner.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolId</i> The user pool ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateArn</i> The Amazon Resource Name (ARN) of an AWS Certificate Manager SSL certificate. You use this certificate for the subdomain of your custom domain.<br/>        """
        pass

    def forget_device(self, DeviceKey: str, AccessToken: str=None):
        """Forgets the specified device.

        :param self:
        :param DeviceKey:  The device key.<br/>AccessToken:  The access token for the forgotten device request.<br/>
        :return:         """
        pass

    def forgot_password(self, Username: str, ClientId: str, SecretHash: str=None, UserContextData: UserContextDataType=None, AnalyticsMetadata: AnalyticsMetadataType=None) -> ForgotPasswordResponse:
        """Calling this API causes a message to be sent to the end user with a confirmation code that is required to change the user's password. For the <code>Username</code> parameter, you can use the username or user alias. If a verified phone number exists for the user, the confirmation code is sent to the phone number. Otherwise, if a verified email exists, the confirmation code is sent to the email. If neither a verified phone number nor a verified email exists, <code>InvalidParameterException</code> is thrown. To use the confirmation code for resetting the password, call .

        :param self:
        :param AnalyticsMetadata:  The Amazon Pinpoint analytics metadata for collecting metrics for <code>ForgotPassword</code> calls.:<br/>Username:  The user name of the user for whom you want to enter a code to reset a forgotten password.<br/>UserContextData:  Contextual data such as the user's device fingerprint, IP address, or location used for evaluating the risk of an unexpected event by Amazon Cognito advanced security.:<br/>SecretHash:  A keyed-hash message authentication code (HMAC) calculated using the secret key of a user pool client and username plus the client ID in the message.<br/>ClientId:  The ID of the client associated with the user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncodedData</i> Contextual data such as the user's device fingerprint, IP address, or location used for evaluating the risk of an unexpected event by Amazon Cognito advanced security.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AnalyticsEndpointId</i> The endpoint ID.<br/>
        :return: CodeDeliveryDetails:  The code delivery details returned by the server in response to the request to reset a password.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttributeName</i> The attribute name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeliveryMedium</i> The delivery medium (email message or phone number).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Destination</i> The destination for the code delivery details.<br/>        """
        pass

    def get_csv_header(self, UserPoolId: str) -> GetCSVHeaderResponse:
        """Gets the header information for the .csv file to be used as input for the user import job.

        :param self:
        :param UserPoolId:  The user pool ID for the user pool that the users are to be imported into.<br/>
        :return: CSVHeader:  The header information for the .csv file for the user import job.<br/>UserPoolId:  The user pool ID for the user pool that the users are to be imported into.<br/>        """
        pass

    def get_device(self, DeviceKey: str, AccessToken: str=None) -> GetDeviceResponse:
        """Gets the device.

        :param self:
        :param AccessToken:  The access token.<br/>DeviceKey:  The device key.<br/>
        :return: Device:  The device.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceLastAuthenticatedDate</i> The date in which the device was last authenticated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceLastModifiedDate</i> The last modified date of the device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceCreateDate</i> The creation date of the device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceAttributes</i> The device attributes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceKey</i> The device key.<br/>        """
        pass

    def get_group(self, UserPoolId: str, GroupName: str) -> GetGroupResponse:
        """Gets a group. Requires developer credentials.

        :param self:
        :param UserPoolId:  The user pool ID for the user pool.<br/>GroupName:  The name of the group.<br/>
        :return: Group:  The group object for the group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date the group was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date the group was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Precedence</i> A nonnegative integer value that specifies the precedence of this group relative to the other groups that a user can belong to in the user pool. If a user belongs to two or more groups, it is the group with the highest precedence whose role ARN will be used in the <code>cognito:roles</code> and <code>cognito:preferred_role</code> claims in the user's tokens. Groups with higher <code>Precedence</code> values take precedence over groups with lower <code>Precedence</code> values or with null <code>Precedence</code> values. Two groups can have the same <code>Precedence</code> value. If this happens, neither group takes precedence over the other. If two groups with the same <code>Precedence</code> have the same role ARN, that role is used in the <code>cognito:preferred_role</code> claim in tokens for users in each group. If the two groups have different role ARNs, the <code>cognito:preferred_role</code> claim is not set in users' tokens. The default <code>Precedence</code> value is null.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The role ARN for the group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A string containing the description of the group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolId</i> The user pool ID for the user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupName</i> The name of the group.<br/>        """
        pass

    def get_identity_provider_by_identifier(self, IdpIdentifier: str, UserPoolId: str) -> GetIdentityProviderByIdentifierResponse:
        """Gets the specified identity provider.

        :param self:
        :param IdpIdentifier:  The identity provider ID.<br/>UserPoolId:  The user pool ID.<br/>
        :return: IdentityProvider:  The identity provider object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date the identity provider was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date the identity provider was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IdpIdentifiers</i> A list of identity provider identifiers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttributeMapping</i> A mapping of identity provider attributes to standard and custom user pool attributes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProviderDetails</i> The identity provider details, such as <code>MetadataURL</code> and <code>MetadataFile</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProviderType</i> The identity provider type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProviderName</i> The identity provider name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolId</i> The user pool ID.<br/>        """
        pass

    def get_signing_certificate(self, UserPoolId: str) -> GetSigningCertificateResponse:
        """This method takes a user pool ID, and returns the signing certificate.

        :param self:
        :param UserPoolId:  The user pool ID.<br/>
        :return: Certificate:  The signing certificate.<br/>        """
        pass

    def get_ui_customization(self, UserPoolId: str, ClientId: str=None) -> GetUICustomizationResponse:
        """Gets the UI Customization information for a particular app client's app UI, if there is something set. If nothing is set for the particular client, but there is an existing pool level customization (app <code>clientId</code> will be <code>ALL</code>), then that is returned. If nothing is present, then an empty shape is returned.

        :param self:
        :param ClientId:  The client ID for the client app.<br/>UserPoolId:  The user pool ID for the user pool.<br/>
        :return: UICustomization:  The UI customization information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The creation date for the UI customization.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The last-modified date for the UI customization.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CSSVersion</i> The CSS version number.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CSS</i> The CSS values in the UI customization.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImageUrl</i> The logo image for the UI customization.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientId</i> The client ID for the client app.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolId</i> The user pool ID for the user pool.<br/>        """
        pass

    def get_user(self, AccessToken: str) -> GetUserResponse:
        """Gets the user attributes and metadata for a user.

        :param self:
        :param AccessToken:  The access token returned by the server response to get information about the user.<br/>
        :return: UserMFASettingList:  The list of the user's MFA settings.<br/>PreferredMfaSetting:  The user's preferred MFA setting.<br/>MFAOptions:  Specifies the options for MFA (e.g., email or phone number).<br/>UserAttributes:  An array of name-value pairs representing user attributes. For custom attributes, you must prepend the <code>custom:</code> prefix to the attribute name.<br/>Username:  The user name of the user you wish to retrieve from the get user request.<br/>        """
        pass

    def get_user_attribute_verification_code(self, AttributeName: str, AccessToken: str) -> GetUserAttributeVerificationCodeResponse:
        """Gets the user attribute verification code for the specified attribute name.

        :param self:
        :param AttributeName:  The attribute name returned by the server response to get the user attribute verification code.<br/>AccessToken:  The access token returned by the server response to get the user attribute verification code.<br/>
        :return: CodeDeliveryDetails:  The code delivery details returned by the server in response to the request to get the user attribute verification code.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttributeName</i> The attribute name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeliveryMedium</i> The delivery medium (email message or phone number).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Destination</i> The destination for the code delivery details.<br/>        """
        pass

    def get_user_pool_mfa_config(self, UserPoolId: str) -> GetUserPoolMfaConfigResponse:
        """Gets the user pool multi-factor authentication (MFA) configuration.

        :param self:
        :param UserPoolId:  The user pool ID.<br/>
        :return: MfaConfiguration:  The multi-factor (MFA) configuration.<br/>SoftwareTokenMfaConfiguration:  The software token multi-factor (MFA) configuration.:<br/>SmsMfaConfiguration:  The SMS text message multi-factor (MFA) configuration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SmsConfiguration</i> The SMS configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SmsConfiguration</b>: The SMS configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SmsAuthenticationMessage</i> The SMS authentication message.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalId</i> The external ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsCallerArn</i> The Amazon Resource Name (ARN) of the Amazon Simple Notification Service (SNS) caller.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether software token MFA is enabled.<br/>        """
        pass

    def global_sign_out(self, AccessToken: str) -> GlobalSignOutResponse:
        """Signs out users from all devices.

        :param self:
        :param AccessToken:  The access token.<br/>
        :return:         """
        pass

    def initiate_auth(self, ClientId: str, AuthFlow: str, AuthParameters: AuthParametersType=None, ClientMetadata: ClientMetadataType=None, AnalyticsMetadata: AnalyticsMetadataType=None, UserContextData: UserContextDataType=None) -> InitiateAuthResponse:
        """Initiates the authentication flow.

        :param self:
        :param UserContextData:  Contextual data such as the user's device fingerprint, IP address, or location used for evaluating the risk of an unexpected event by Amazon Cognito advanced security.:<br/>AnalyticsMetadata:  The Amazon Pinpoint analytics metadata for collecting metrics for <code>InitiateAuth</code> calls.:<br/>ClientId:  The app client ID.<br/>ClientMetadata:  This is a random key-value pair map which can contain any key and will be passed to your PreAuthentication Lambda trigger as-is. It can be used to implement additional validations around authentication.<br/>AuthParameters:  The authentication parameters. These are inputs corresponding to the <code>AuthFlow</code> that you are invoking. The required values depend on the value of <code>AuthFlow</code>: <ul> <li> For <code>USER_SRP_AUTH</code>: <code>USERNAME</code> (required), <code>SRP_A</code> (required), <code>SECRET_HASH</code> (required if the app client is configured with a client secret), <code>DEVICE_KEY</code>  </li> <li> For <code>REFRESH_TOKEN_AUTH/REFRESH_TOKEN</code>: <code>REFRESH_TOKEN</code> (required), <code>SECRET_HASH</code> (required if the app client is configured with a client secret), <code>DEVICE_KEY</code>  </li> <li> For <code>CUSTOM_AUTH</code>: <code>USERNAME</code> (required), <code>SECRET_HASH</code> (if app client is configured with client secret), <code>DEVICE_KEY</code>  </li> </ul><br/>AuthFlow:  The authentication flow for this call to execute. The API action will depend on this value. For example:  <ul> <li>  <code>REFRESH_TOKEN_AUTH</code> will take in a valid refresh token and return new tokens. </li> <li>  <code>USER_SRP_AUTH</code> will take in <code>USERNAME</code> and <code>SRP_A</code> and return the SRP variables to be used for next challenge execution. </li> <li>  <code>USER_PASSWORD_AUTH</code> will take in <code>USERNAME</code> and <code>PASSWORD</code> and return the next challenge or tokens. </li> </ul> Valid values include: <ul> <li>  <code>USER_SRP_AUTH</code>: Authentication flow for the Secure Remote Password (SRP) protocol. </li> <li>  <code>REFRESH_TOKEN_AUTH</code>/<code>REFRESH_TOKEN</code>: Authentication flow for refreshing the access token and ID token by supplying a valid refresh token. </li> <li>  <code>CUSTOM_AUTH</code>: Custom authentication flow. </li> <li>  <code>USER_PASSWORD_AUTH</code>: Non-SRP authentication flow; USERNAME and PASSWORD are passed directly. If a user migration Lambda trigger is set, this flow will invoke the user migration Lambda if the USERNAME is not found in the user pool.  </li> </ul>  <code>ADMIN_NO_SRP_AUTH</code> is not a valid value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AnalyticsEndpointId</i> The endpoint ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncodedData</i> Contextual data such as the user's device fingerprint, IP address, or location used for evaluating the risk of an unexpected event by Amazon Cognito advanced security.<br/>
        :return: AuthenticationResult:  The result of the authentication response. This is only returned if the caller does not need to pass another challenge. If the caller does need to pass another challenge before it gets tokens, <code>ChallengeName</code>, <code>ChallengeParameters</code>, and <code>Session</code> are returned.:<br/>ChallengeParameters:  The challenge parameters. These are returned to you in the <code>InitiateAuth</code> response if you need to pass another challenge. The responses in this parameter should be used to compute inputs to the next call (<code>RespondToAuthChallenge</code>).  All challenges require <code>USERNAME</code> and <code>SECRET_HASH</code> (if applicable).<br/>Session:  The session which should be passed both ways in challenge-response calls to the service. If the or API call determines that the caller needs to go through another challenge, they return a session with other challenge parameters. This session should be passed as it is to the next <code>RespondToAuthChallenge</code> API call.<br/>ChallengeName:  The name of the challenge which you are responding to with this call. This is returned to you in the <code>AdminInitiateAuth</code> response if you need to pass another challenge. Valid values include the following. Note that all of these challenges require <code>USERNAME</code> and <code>SECRET_HASH</code> (if applicable) in the parameters. <ul> <li>  <code>SMS_MFA</code>: Next challenge is to supply an <code>SMS_MFA_CODE</code>, delivered via SMS. </li> <li>  <code>PASSWORD_VERIFIER</code>: Next challenge is to supply <code>PASSWORD_CLAIM_SIGNATURE</code>, <code>PASSWORD_CLAIM_SECRET_BLOCK</code>, and <code>TIMESTAMP</code> after the client-side SRP calculations. </li> <li>  <code>CUSTOM_CHALLENGE</code>: This is returned if your custom authentication flow determines that the user should pass another challenge before tokens are issued. </li> <li>  <code>DEVICE_SRP_AUTH</code>: If device tracking was enabled on your user pool and the previous challenges were passed, this challenge is returned so that Amazon Cognito can start tracking this device. </li> <li>  <code>DEVICE_PASSWORD_VERIFIER</code>: Similar to <code>PASSWORD_VERIFIER</code>, but for devices only. </li> <li>  <code>NEW_PASSWORD_REQUIRED</code>: For users which are required to change their passwords after successful first login. This challenge should be passed with <code>NEW_PASSWORD</code> and any other required attributes. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NewDeviceMetadata</i> The new device metadata from an authentication result.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NewDeviceMetadata</b>: The new device metadata from an authentication result.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IdToken</i> The ID token.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RefreshToken</i> The refresh token.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TokenType</i> The token type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpiresIn</i> The expiration period of the authentication result in seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccessToken</i> The access token.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceGroupKey</i> The device group key.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceKey</i> The device key.<br/>        """
        pass

    def list_devices(self, AccessToken: str, Limit: int=None, PaginationToken: str=None) -> ListDevicesResponse:
        """Lists the devices.

        :param self:
        :param PaginationToken:  The pagination token for the list request.<br/>Limit:  The limit of the device request.<br/>AccessToken:  The access tokens for the request to list devices.<br/>
        :return: PaginationToken:  The pagination token for the list device response.<br/>Devices:  The devices returned in the list devices response.<br/>        """
        pass

    def list_groups(self, UserPoolId: str, Limit: int=None, NextToken: str=None) -> ListGroupsResponse:
        """Lists the groups associated with a user pool. Requires developer credentials.

        :param self:
        :param NextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>Limit:  The limit of the request to list groups.<br/>UserPoolId:  The user pool ID for the user pool.<br/>
        :return: NextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>Groups:  The group objects for the groups.<br/>        """
        pass

    def list_identity_providers(self, UserPoolId: str, MaxResults: int=None, NextToken: str=None) -> ListIdentityProvidersResponse:
        """Lists information about all identity providers for a user pool.

        :param self:
        :param NextToken:  A pagination token.<br/>MaxResults:  The maximum number of identity providers to return.<br/>UserPoolId:  The user pool ID.<br/>
        :return: NextToken:  A pagination token.<br/>Providers:  A list of identity provider objects.<br/>        """
        pass

    def list_resource_servers(self, UserPoolId: str, MaxResults: int=None, NextToken: str=None) -> ListResourceServersResponse:
        """Lists the resource servers for a user pool.

        :param self:
        :param NextToken:  A pagination token.<br/>MaxResults:  The maximum number of resource servers to return.<br/>UserPoolId:  The user pool ID for the user pool.<br/>
        :return: NextToken:  A pagination token.<br/>ResourceServers:  The resource servers.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceArn: str) -> ListTagsForResourceResponse:
        """Lists the tags that are assigned to an Amazon Cognito user pool. A tag is a label that you can apply to user pools to categorize and manage them in different ways, such as by purpose, owner, environment, or other criteria. You can use this action up to 10 times per second, per account.

        :param self:
        :param ResourceArn:  The Amazon Resource Name (ARN) of the user pool that the tags are assigned to.<br/>
        :return: Tags:  The tags that are assigned to the user pool.<br/>        """
        pass

    def list_user_import_jobs(self, MaxResults: int, UserPoolId: str, PaginationToken: str=None) -> ListUserImportJobsResponse:
        """Lists the user import jobs.

        :param self:
        :param PaginationToken:  An identifier that was returned from the previous call to <code>ListUserImportJobs</code>, which can be used to return the next set of import jobs in the list.<br/>MaxResults:  The maximum number of import jobs you want the request to return.<br/>UserPoolId:  The user pool ID for the user pool that the users are being imported into.<br/>
        :return: PaginationToken:  An identifier that can be used to return the next set of user import jobs in the list.<br/>UserImportJobs:  The user import jobs.<br/>        """
        pass

    def list_user_pool_clients(self, UserPoolId: str, MaxResults: int=None, NextToken: str=None) -> ListUserPoolClientsResponse:
        """Lists the clients that have been created for the specified user pool.

        :param self:
        :param NextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>MaxResults:  The maximum number of results you want the request to return when listing the user pool clients.<br/>UserPoolId:  The user pool ID for the user pool where you want to list user pool clients.<br/>
        :return: NextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>UserPoolClients:  The user pool clients in the response that lists user pool clients.<br/>        """
        pass

    def list_user_pools(self, MaxResults: int, NextToken: str=None) -> ListUserPoolsResponse:
        """Lists the user pools associated with an AWS account.

        :param self:
        :param MaxResults:  The maximum number of results you want the request to return when listing the user pools.<br/>NextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>
        :return: NextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>UserPools:  The user pools from the response to list users.<br/>        """
        pass

    def list_users(self, UserPoolId: str, AttributesToGet: List[AttributeNameType]=None, Limit: int=None, PaginationToken: str=None, Filter: str=None) -> ListUsersResponse:
        """Lists the users in the Amazon Cognito user pool.

        :param self:
        :param Filter:  A filter string of the form "<i>AttributeName</i> <i>Filter-Type</i> "<i>AttributeValue</i>"". Quotation marks within the filter string must be escaped using the backslash (\) character. For example, "<code>family_name</code> = \"Reddy\"". <ul> <li>  <i>AttributeName</i>: The name of the attribute to search for. You can only search for one attribute at a time. </li> <li>  <i>Filter-Type</i>: For an exact match, use =, for example, "<code>given_name</code> = \"Jon\"". For a prefix ("starts with") match, use ^=, for example, "<code>given_name</code> ^= \"Jon\"".  </li> <li>  <i>AttributeValue</i>: The attribute value that must be matched for each user. </li> </ul> If the filter string is empty, <code>ListUsers</code> returns all users in the user pool. You can only search for the following standard attributes: <ul> <li>  <code>username</code> (case-sensitive) </li> <li>  <code>email</code>  </li> <li>  <code>phone_number</code>  </li> <li>  <code>name</code>  </li> <li>  <code>given_name</code>  </li> <li>  <code>family_name</code>  </li> <li>  <code>preferred_username</code>  </li> <li>  <code>cognito:user_status</code> (called <b>Status</b> in the Console) (case-insensitive) </li> <li>  <code>status (called <b>Enabled</b> in the Console) (case-sensitive)</code>  </li> <li>  <code>sub</code>  </li> </ul> Custom attributes are not searchable. For more information, see <a href="http://docs.aws.amazon.com/cognito/latest/developerguide/how-to-manage-user-accounts.html#cognito-user-pools-searching-for-users-using-listusers-api">Searching for Users Using the ListUsers API</a> and <a href="http://docs.aws.amazon.com/cognito/latest/developerguide/how-to-manage-user-accounts.html#cognito-user-pools-searching-for-users-listusers-api-examples">Examples of Using the ListUsers API</a> in the <i>Amazon Cognito Developer Guide</i>.<br/>PaginationToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>Limit:  Maximum number of users to be returned.<br/>AttributesToGet:  An array of strings, where each string is the name of a user attribute to be returned for each user in the search results. If the array is null, all attributes are returned.<br/>UserPoolId:  The user pool ID for the user pool on which the search should be performed.<br/>
        :return: PaginationToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>Users:  The users returned in the request to list users.<br/>        """
        pass

    def list_users_in_group(self, GroupName: str, UserPoolId: str, Limit: int=None, NextToken: str=None) -> ListUsersInGroupResponse:
        """Lists the users in the specified group. Requires developer credentials.

        :param self:
        :param NextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>Limit:  The limit of the request to list users.<br/>GroupName:  The name of the group.<br/>UserPoolId:  The user pool ID for the user pool.<br/>
        :return: NextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>Users:  The users returned in the request to list users.<br/>        """
        pass

    def resend_confirmation_code(self, Username: str, ClientId: str, SecretHash: str=None, UserContextData: UserContextDataType=None, AnalyticsMetadata: AnalyticsMetadataType=None) -> ResendConfirmationCodeResponse:
        """Resends the confirmation (for confirmation of registration) to a specific user in the user pool.

        :param self:
        :param AnalyticsMetadata:  The Amazon Pinpoint analytics metadata for collecting metrics for <code>ResendConfirmationCode</code> calls.:<br/>Username:  The user name of the user to whom you wish to resend a confirmation code.<br/>UserContextData:  Contextual data such as the user's device fingerprint, IP address, or location used for evaluating the risk of an unexpected event by Amazon Cognito advanced security.:<br/>SecretHash:  A keyed-hash message authentication code (HMAC) calculated using the secret key of a user pool client and username plus the client ID in the message.<br/>ClientId:  The ID of the client associated with the user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncodedData</i> Contextual data such as the user's device fingerprint, IP address, or location used for evaluating the risk of an unexpected event by Amazon Cognito advanced security.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AnalyticsEndpointId</i> The endpoint ID.<br/>
        :return: CodeDeliveryDetails:  The code delivery details returned by the server in response to the request to resend the confirmation code.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttributeName</i> The attribute name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeliveryMedium</i> The delivery medium (email message or phone number).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Destination</i> The destination for the code delivery details.<br/>        """
        pass

    def respond_to_auth_challenge(self, ChallengeName: str, ClientId: str, Session: str=None, ChallengeResponses: ChallengeResponsesType=None, AnalyticsMetadata: AnalyticsMetadataType=None, UserContextData: UserContextDataType=None) -> RespondToAuthChallengeResponse:
        """Responds to the authentication challenge.

        :param self:
        :param UserContextData:  Contextual data such as the user's device fingerprint, IP address, or location used for evaluating the risk of an unexpected event by Amazon Cognito advanced security.:<br/>AnalyticsMetadata:  The Amazon Pinpoint analytics metadata for collecting metrics for <code>RespondToAuthChallenge</code> calls.:<br/>ChallengeResponses:  The challenge responses. These are inputs corresponding to the value of <code>ChallengeName</code>, for example: <ul> <li>  <code>SMS_MFA</code>: <code>SMS_MFA_CODE</code>, <code>USERNAME</code>, <code>SECRET_HASH</code> (if app client is configured with client secret). </li> <li>  <code>PASSWORD_VERIFIER</code>: <code>PASSWORD_CLAIM_SIGNATURE</code>, <code>PASSWORD_CLAIM_SECRET_BLOCK</code>, <code>TIMESTAMP</code>, <code>USERNAME</code>, <code>SECRET_HASH</code> (if app client is configured with client secret). </li> <li>  <code>NEW_PASSWORD_REQUIRED</code>: <code>NEW_PASSWORD</code>, any other required attributes, <code>USERNAME</code>, <code>SECRET_HASH</code> (if app client is configured with client secret).  </li> </ul><br/>Session:  The session which should be passed both ways in challenge-response calls to the service. If <code>InitiateAuth</code> or <code>RespondToAuthChallenge</code> API call determines that the caller needs to go through another challenge, they return a session with other challenge parameters. This session should be passed as it is to the next <code>RespondToAuthChallenge</code> API call.<br/>ChallengeName:  The challenge name. For more information, see .  <code>ADMIN_NO_SRP_AUTH</code> is not a valid value.<br/>ClientId:  The app client ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AnalyticsEndpointId</i> The endpoint ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncodedData</i> Contextual data such as the user's device fingerprint, IP address, or location used for evaluating the risk of an unexpected event by Amazon Cognito advanced security.<br/>
        :return: AuthenticationResult:  The result returned by the server in response to the request to respond to the authentication challenge.:<br/>ChallengeParameters:  The challenge parameters. For more information, see .<br/>Session:  The session which should be passed both ways in challenge-response calls to the service. If the or API call determines that the caller needs to go through another challenge, they return a session with other challenge parameters. This session should be passed as it is to the next <code>RespondToAuthChallenge</code> API call.<br/>ChallengeName:  The challenge name. For more information, see .<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NewDeviceMetadata</i> The new device metadata from an authentication result.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NewDeviceMetadata</b>: The new device metadata from an authentication result.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IdToken</i> The ID token.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RefreshToken</i> The refresh token.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TokenType</i> The token type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpiresIn</i> The expiration period of the authentication result in seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccessToken</i> The access token.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceGroupKey</i> The device group key.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceKey</i> The device key.<br/>        """
        pass

    def set_risk_configuration(self, UserPoolId: str, ClientId: str=None, CompromisedCredentialsRiskConfiguration: CompromisedCredentialsRiskConfigurationType=None, AccountTakeoverRiskConfiguration: AccountTakeoverRiskConfigurationType=None, RiskExceptionConfiguration: RiskExceptionConfigurationType=None) -> SetRiskConfigurationResponse:
        """Configures actions on detected risks. To delete the risk configuration for <code>UserPoolId</code> or <code>ClientId</code>, pass null values for all four configuration types. To enable Amazon Cognito advanced security features, update the user pool to include the <code>UserPoolAddOns</code> key<code>AdvancedSecurityMode</code>. See .

        :param self:
        :param RiskExceptionConfiguration:  The configuration to override the risk decision.:<br/>AccountTakeoverRiskConfiguration:  The account takeover risk configuration.:<br/>CompromisedCredentialsRiskConfiguration:  The compromised credentials risk configuration.:<br/>ClientId:  The app client ID. If <code>ClientId</code> is null, then the risk configuration is mapped to <code>userPoolId</code>. When the client ID is null, the same risk configuration is applied to all the clients in the userPool. Otherwise, <code>ClientId</code> is mapped to the client. When the client ID is not null, the user pool configuration is overridden and the risk configuration for the client is used instead.<br/>UserPoolId:  The user pool ID. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Actions</i> The compromised credentials risk configuration actions.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Actions</b>: The compromised credentials risk configuration actions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EventFilter</i> Perform the action for these events. The default is to perform all events if no event filter is specified.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EventAction</i> The event action.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Actions</i> Account takeover risk configuration actions&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Actions</b>: Account takeover risk configuration actions<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotifyConfiguration</i> The notify configuration used to construct email notifications.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NotifyConfiguration</b>: The notify configuration used to construct email notifications.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceArn</i> The Amazon Resource Name (ARN) of the identity that is associated with the sending authorization policy. It permits Amazon Cognito to send for the email address specified in the <code>From</code> parameter.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplyTo</i> The destination to which the receiver of an email should reply to.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>From</i> The email address that is sending the email. It must be either individually verified with Amazon SES, or from a domain that has been verified with Amazon SES.<br/>Email template used when a detected risk event is blocked.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BlockEmail</b>: Email template used when a detected risk event is blocked.Action to take for a low risk.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LowAction</b>: Action to take for a low risk.&nbsp;&nbsp;&nbsp;&nbsp; <i>SkippedIPRangeList</i> Risk detection is not performed on the IP addresses in the range list. The IP range is in CIDR notation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BlockedIPRangeList</i> Overrides the risk decision to always block the pre-authentication requests. The IP range is in CIDR notation: a compact representation of an IP address and its associated routing prefix.<br/>
        :return: RiskConfiguration:  The risk configuration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The last modified date.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RiskExceptionConfiguration</i> The configuration to override the risk decision.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RiskExceptionConfiguration</b>: The configuration to override the risk decision.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountTakeoverRiskConfiguration</i> The account takeover risk configuration object including the <code>NotifyConfiguration</code> object and <code>Actions</code> to take in the case of an account takeover.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AccountTakeoverRiskConfiguration</b>: The account takeover risk configuration object including the <code>NotifyConfiguration</code> object and <code>Actions</code> to take in the case of an account takeover.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompromisedCredentialsRiskConfiguration</i> The compromised credentials risk configuration object including the <code>EventFilter</code> and the <code>EventAction</code> &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CompromisedCredentialsRiskConfiguration</b>: The compromised credentials risk configuration object including the <code>EventFilter</code> and the <code>EventAction</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientId</i> The app client ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolId</i> The user pool ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EventFilter</i> Perform the action for these events. The default is to perform all events if no event filter is specified.<br/>The compromised credentials risk configuration actions.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Actions</b>: The compromised credentials risk configuration actions.The notify configuration used to construct email notifications.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NotifyConfiguration</b>: The notify configuration used to construct email notifications.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SkippedIPRangeList</i> Risk detection is not performed on the IP addresses in the range list. The IP range is in CIDR notation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BlockedIPRangeList</i> Overrides the risk decision to always block the pre-authentication requests. The IP range is in CIDR notation: a compact representation of an IP address and its associated routing prefix.<br/>        """
        pass

    def set_ui_customization(self, UserPoolId: str, ClientId: str=None, CSS: str=None, ImageFile: ImageFileType=None) -> SetUICustomizationResponse:
        """Sets the UI customization information for a user pool's built-in app UI. You can specify app UI customization settings for a single client (with a specific <code>clientId</code>) or for all clients (by setting the <code>clientId</code> to <code>ALL</code>). If you specify <code>ALL</code>, the default configuration will be used for every client that has no UI customization set previously. If you specify UI customization settings for a particular client, it will no longer fall back to the <code>ALL</code> configuration.  <note> To use this API, your user pool must have a domain associated with it. Otherwise, there is no place to host the app's pages, and the service will throw an error. </note>

        :param self:
        :param ImageFile:  The uploaded logo image for the UI customization.<br/>CSS:  The CSS values in the UI customization.<br/>ClientId:  The client ID for the client app.<br/>UserPoolId:  The user pool ID for the user pool.<br/>
        :return: UICustomization:  The UI customization information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The creation date for the UI customization.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The last-modified date for the UI customization.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CSSVersion</i> The CSS version number.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CSS</i> The CSS values in the UI customization.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImageUrl</i> The logo image for the UI customization.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientId</i> The client ID for the client app.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolId</i> The user pool ID for the user pool.<br/>        """
        pass

    def set_user_mfa_preference(self, AccessToken: str, SMSMfaSettings: SMSMfaSettingsType=None, SoftwareTokenMfaSettings: SoftwareTokenMfaSettingsType=None) -> SetUserMFAPreferenceResponse:
        """Set the user's multi-factor authentication (MFA) method preference.

        :param self:
        :param AccessToken:  The access token.<br/>SoftwareTokenMfaSettings:  The time-based one-time password software token MFA settings.:<br/>SMSMfaSettings:  The SMS text message multi-factor authentication (MFA) settings.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMfa</i> The preferred MFA method.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether SMS text message MFA is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMfa</i> The preferred MFA method.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether software token MFA is enabled.<br/>
        :return:         """
        pass

    def set_user_pool_mfa_config(self, UserPoolId: str, SmsMfaConfiguration: SmsMfaConfigType=None, SoftwareTokenMfaConfiguration: SoftwareTokenMfaConfigType=None, MfaConfiguration: str=None) -> SetUserPoolMfaConfigResponse:
        """Set the user pool MFA configuration.

        :param self:
        :param MfaConfiguration:  The MFA configuration.<br/>SoftwareTokenMfaConfiguration:  The software token MFA configuration.:<br/>SmsMfaConfiguration:  The SMS text message MFA configuration.:<br/>UserPoolId:  The user pool ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SmsConfiguration</i> The SMS configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SmsConfiguration</b>: The SMS configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SmsAuthenticationMessage</i> The SMS authentication message.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalId</i> The external ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsCallerArn</i> The Amazon Resource Name (ARN) of the Amazon Simple Notification Service (SNS) caller.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether software token MFA is enabled.<br/>
        :return: MfaConfiguration:  The MFA configuration.<br/>SoftwareTokenMfaConfiguration:  The software token MFA configuration.:<br/>SmsMfaConfiguration:  The SMS text message MFA configuration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SmsConfiguration</i> The SMS configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SmsConfiguration</b>: The SMS configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SmsAuthenticationMessage</i> The SMS authentication message.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalId</i> The external ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsCallerArn</i> The Amazon Resource Name (ARN) of the Amazon Simple Notification Service (SNS) caller.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether software token MFA is enabled.<br/>        """
        pass

    def set_user_settings(self, MFAOptions: List[MFAOptionType], AccessToken: str) -> SetUserSettingsResponse:
        """Sets the user settings like multi-factor authentication (MFA). If MFA is to be removed for a particular attribute pass the attribute with code delivery as null. If null list is passed, all MFA options are removed.

        :param self:
        :param MFAOptions:  Specifies the options for MFA (e.g., email or phone number).<br/>AccessToken:  The access token for the set user settings request.<br/>
        :return:         """
        pass

    def sign_up(self, Password: str, Username: str, ClientId: str, SecretHash: str=None, UserAttributes: List[AttributeType]=None, ValidationData: List[AttributeType]=None, AnalyticsMetadata: AnalyticsMetadataType=None, UserContextData: UserContextDataType=None) -> SignUpResponse:
        """Registers the user in the specified user pool and creates a user name, password, and user attributes.

        :param self:
        :param UserContextData:  Contextual data such as the user's device fingerprint, IP address, or location used for evaluating the risk of an unexpected event by Amazon Cognito advanced security.:<br/>AnalyticsMetadata:  The Amazon Pinpoint analytics metadata for collecting metrics for <code>SignUp</code> calls.:<br/>ValidationData:  The validation data in the request to register a user.<br/>UserAttributes:  An array of name-value pairs representing user attributes. For custom attributes, you must prepend the <code>custom:</code> prefix to the attribute name.<br/>Password:  The password of the user you wish to register.<br/>Username:  The user name of the user you wish to register.<br/>SecretHash:  A keyed-hash message authentication code (HMAC) calculated using the secret key of a user pool client and username plus the client ID in the message.<br/>ClientId:  The ID of the client associated with the user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AnalyticsEndpointId</i> The endpoint ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncodedData</i> Contextual data such as the user's device fingerprint, IP address, or location used for evaluating the risk of an unexpected event by Amazon Cognito advanced security.<br/>
        :return: UserSub:  The UUID of the authenticated user. This is not the same as <code>username</code>.<br/>CodeDeliveryDetails:  The code delivery details returned by the server response to the user registration request.:<br/>UserConfirmed:  A response from the server indicating that a user registration has been confirmed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttributeName</i> The attribute name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeliveryMedium</i> The delivery medium (email message or phone number).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Destination</i> The destination for the code delivery details.<br/>        """
        pass

    def start_user_import_job(self, JobId: str, UserPoolId: str) -> StartUserImportJobResponse:
        """Starts the user import.

        :param self:
        :param JobId:  The job ID for the user import job.<br/>UserPoolId:  The user pool ID for the user pool that the users are being imported into.<br/>
        :return: UserImportJob:  The job object that represents the user import job.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompletionMessage</i> The message returned when the user import job is completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailedUsers</i> The number of users that could not be imported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SkippedUsers</i> The number of users that were skipped.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImportedUsers</i> The number of users that were successfully imported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchLogsRoleArn</i> The role ARN for the Amazon CloudWatch Logging role for the user import job. For more information, see "Creating the CloudWatch Logs IAM Role" in the Amazon Cognito Developer Guide.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the user import job. One of the following: <ul> <li>  <code>Created</code> - The job was created but not started. </li> <li>  <code>Pending</code> - A transition state. You have started the job, but it has not begun importing users yet. </li> <li>  <code>InProgress</code> - The job has started, and users are being imported. </li> <li>  <code>Stopping</code> - You have stopped the job, but the job has not stopped importing users yet. </li> <li>  <code>Stopped</code> - You have stopped the job, and the job has stopped importing users. </li> <li>  <code>Succeeded</code> - The job has completed successfully. </li> <li>  <code>Failed</code> - The job has stopped due to an error. </li> <li>  <code>Expired</code> - You created a job, but did not start the job within 24-48 hours. All data associated with the job was deleted, and the job cannot be started. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompletionDate</i> The date when the user import job was completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartDate</i> The date when the user import job was started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date the user import job was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreSignedUrl</i> The pre-signed URL to be used to upload the <code>.csv</code> file.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolId</i> The user pool ID for the user pool that the users are being imported into.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobId</i> The job ID for the user import job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobName</i> The job name for the user import job.<br/>        """
        pass

    def stop_user_import_job(self, JobId: str, UserPoolId: str) -> StopUserImportJobResponse:
        """Stops the user import job.

        :param self:
        :param JobId:  The job ID for the user import job.<br/>UserPoolId:  The user pool ID for the user pool that the users are being imported into.<br/>
        :return: UserImportJob:  The job object that represents the user import job.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompletionMessage</i> The message returned when the user import job is completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailedUsers</i> The number of users that could not be imported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SkippedUsers</i> The number of users that were skipped.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImportedUsers</i> The number of users that were successfully imported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchLogsRoleArn</i> The role ARN for the Amazon CloudWatch Logging role for the user import job. For more information, see "Creating the CloudWatch Logs IAM Role" in the Amazon Cognito Developer Guide.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the user import job. One of the following: <ul> <li>  <code>Created</code> - The job was created but not started. </li> <li>  <code>Pending</code> - A transition state. You have started the job, but it has not begun importing users yet. </li> <li>  <code>InProgress</code> - The job has started, and users are being imported. </li> <li>  <code>Stopping</code> - You have stopped the job, but the job has not stopped importing users yet. </li> <li>  <code>Stopped</code> - You have stopped the job, and the job has stopped importing users. </li> <li>  <code>Succeeded</code> - The job has completed successfully. </li> <li>  <code>Failed</code> - The job has stopped due to an error. </li> <li>  <code>Expired</code> - You created a job, but did not start the job within 24-48 hours. All data associated with the job was deleted, and the job cannot be started. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompletionDate</i> The date when the user import job was completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartDate</i> The date when the user import job was started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date the user import job was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreSignedUrl</i> The pre-signed URL to be used to upload the <code>.csv</code> file.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolId</i> The user pool ID for the user pool that the users are being imported into.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobId</i> The job ID for the user import job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobName</i> The job name for the user import job.<br/>        """
        pass

    def tag_resource(self, ResourceArn: str, Tags: UserPoolTagsType=None) -> TagResourceResponse:
        """Assigns a set of tags to an Amazon Cognito user pool. A tag is a label that you can use to categorize and manage user pools in different ways, such as by purpose, owner, environment, or other criteria. Each tag consists of a key and value, both of which you define. A key is a general category for more specific values. For example, if you have two versions of a user pool, one for testing and another for production, you might assign an <code>Environment</code> tag key to both user pools. The value of this key might be <code>Test</code> for one user pool and <code>Production</code> for the other. Tags are useful for cost tracking and access control. You can activate your tags so that they appear on the Billing and Cost Management console, where you can track the costs associated with your user pools. In an IAM policy, you can constrain permissions for user pools based on specific tags or tag values. You can use this action up to 5 times per second, per account. A user pool can have as many as 50 tags.

        :param self:
        :param Tags:  The tags to assign to the user pool.<br/>ResourceArn:  The Amazon Resource Name (ARN) of the user pool to assign the tags to.<br/>
        :return:         """
        pass

    def untag_resource(self, ResourceArn: str, TagKeys: List[TagKeysType]=None) -> UntagResourceResponse:
        """Removes the specified tags from an Amazon Cognito user pool. You can use this action up to 5 times per second, per account

        :param self:
        :param TagKeys:  The keys of the tags to remove from the user pool.<br/>ResourceArn:  The Amazon Resource Name (ARN) of the user pool that the tags are assigned to.<br/>
        :return:         """
        pass

    def update_auth_event_feedback(self, FeedbackValue: str, FeedbackToken: str, EventId: str, Username: str, UserPoolId: str) -> UpdateAuthEventFeedbackResponse:
        """Provides the feedback for an authentication event whether it was from a valid user or not. This feedback is used for improving the risk evaluation decision for the user pool as part of Amazon Cognito advanced security.

        :param self:
        :param FeedbackValue:  The authentication event feedback value.<br/>FeedbackToken:  The feedback token.<br/>EventId:  The event ID.<br/>Username:  The user pool username.<br/>UserPoolId:  The user pool ID.<br/>
        :return:         """
        pass

    def update_device_status(self, DeviceKey: str, AccessToken: str, DeviceRememberedStatus: str=None) -> UpdateDeviceStatusResponse:
        """Updates the device status.

        :param self:
        :param DeviceRememberedStatus:  The status of whether a device is remembered.<br/>DeviceKey:  The device key.<br/>AccessToken:  The access token.<br/>
        :return:         """
        pass

    def update_group(self, UserPoolId: str, GroupName: str, Description: str=None, RoleArn: str=None, Precedence: int=None) -> UpdateGroupResponse:
        """Updates the specified group with the specified attributes. Requires developer credentials.

        :param self:
        :param Precedence:  The new precedence value for the group. For more information about this parameter, see .<br/>RoleArn:  The new role ARN for the group. This is used for setting the <code>cognito:roles</code> and <code>cognito:preferred_role</code> claims in the token.<br/>Description:  A string containing the new description of the group.<br/>UserPoolId:  The user pool ID for the user pool.<br/>GroupName:  The name of the group.<br/>
        :return: Group:  The group object for the group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date the group was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date the group was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Precedence</i> A nonnegative integer value that specifies the precedence of this group relative to the other groups that a user can belong to in the user pool. If a user belongs to two or more groups, it is the group with the highest precedence whose role ARN will be used in the <code>cognito:roles</code> and <code>cognito:preferred_role</code> claims in the user's tokens. Groups with higher <code>Precedence</code> values take precedence over groups with lower <code>Precedence</code> values or with null <code>Precedence</code> values. Two groups can have the same <code>Precedence</code> value. If this happens, neither group takes precedence over the other. If two groups with the same <code>Precedence</code> have the same role ARN, that role is used in the <code>cognito:preferred_role</code> claim in tokens for users in each group. If the two groups have different role ARNs, the <code>cognito:preferred_role</code> claim is not set in users' tokens. The default <code>Precedence</code> value is null.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The role ARN for the group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A string containing the description of the group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolId</i> The user pool ID for the user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupName</i> The name of the group.<br/>        """
        pass

    def update_identity_provider(self, ProviderName: str, UserPoolId: str, ProviderDetails: ProviderDetailsType=None, AttributeMapping: AttributeMappingType=None, IdpIdentifiers: List[IdpIdentifierType]=None) -> UpdateIdentityProviderResponse:
        """Updates identity provider information for a user pool.

        :param self:
        :param IdpIdentifiers:  A list of identity provider identifiers.<br/>AttributeMapping:  The identity provider attribute mapping to be changed.<br/>ProviderDetails:  The identity provider details to be updated, such as <code>MetadataURL</code> and <code>MetadataFile</code>.<br/>ProviderName:  The identity provider name.<br/>UserPoolId:  The user pool ID.<br/>
        :return: IdentityProvider:  The identity provider object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date the identity provider was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date the identity provider was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IdpIdentifiers</i> A list of identity provider identifiers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttributeMapping</i> A mapping of identity provider attributes to standard and custom user pool attributes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProviderDetails</i> The identity provider details, such as <code>MetadataURL</code> and <code>MetadataFile</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProviderType</i> The identity provider type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProviderName</i> The identity provider name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolId</i> The user pool ID.<br/>        """
        pass

    def update_resource_server(self, Name: str, Identifier: str, UserPoolId: str, Scopes: List[ResourceServerScopeType]=None) -> UpdateResourceServerResponse:
        """Updates the name and scopes of resource server. All other fields are read-only.

        :param self:
        :param Scopes:  The scope values to be set for the resource server.<br/>Name:  The name of the resource server.<br/>Identifier:  The identifier for the resource server.<br/>UserPoolId:  The user pool ID for the user pool.<br/>
        :return: ResourceServer:  The resource server.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Scopes</i> A list of scopes that are defined for the resource server.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the resource server.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Identifier</i> The identifier for the resource server.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolId</i> The user pool ID for the user pool that hosts the resource server.<br/>        """
        pass

    def update_user_attributes(self, AccessToken: str, UserAttributes: List[AttributeType]) -> UpdateUserAttributesResponse:
        """Allows a user to update a specific attribute (one at a time).

        :param self:
        :param AccessToken:  The access token for the request to update user attributes.<br/>UserAttributes:  An array of name-value pairs representing user attributes. For custom attributes, you must prepend the <code>custom:</code> prefix to the attribute name.<br/>
        :return: CodeDeliveryDetailsList:  The code delivery details list from the server for the request to update user attributes.<br/>        """
        pass

    def update_user_pool(self, UserPoolId: str, Policies: UserPoolPolicyType=None, LambdaConfig: LambdaConfigType=None, AutoVerifiedAttributes: List[VerifiedAttributeType]=None, SmsVerificationMessage: str=None, EmailVerificationMessage: str=None, EmailVerificationSubject: str=None, VerificationMessageTemplate: VerificationMessageTemplateType=None, SmsAuthenticationMessage: str=None, MfaConfiguration: str=None, DeviceConfiguration: DeviceConfigurationType=None, EmailConfiguration: EmailConfigurationType=None, SmsConfiguration: SmsConfigurationType=None, UserPoolTags: UserPoolTagsType=None, AdminCreateUserConfig: AdminCreateUserConfigType=None, UserPoolAddOns: UserPoolAddOnsType=None) -> UpdateUserPoolResponse:
        """Updates the specified user pool with the specified attributes. If you don't provide a value for an attribute, it will be set to the default value. You can get a list of the current user pool settings with .

        :param self:
        :param UserPoolAddOns:  Used to enable advanced security risk detection. Set the key <code>AdvancedSecurityMode</code> to the value "AUDIT".:<br/>AdminCreateUserConfig:  The configuration for <code>AdminCreateUser</code> requests.:<br/>UserPoolTags:  The tag keys and values to assign to the user pool. A tag is a label that you can use to categorize and manage user pools in different ways, such as by purpose, owner, environment, or other criteria.<br/>SmsConfiguration:  SMS configuration.:<br/>EmailConfiguration:  Email configuration.:<br/>DeviceConfiguration:  Device configuration.:<br/>MfaConfiguration:  Can be one of the following values: <ul> <li>  <code>OFF</code> - MFA tokens are not required and cannot be specified during user registration. </li> <li>  <code>ON</code> - MFA tokens are required for all user registrations. You can only specify required when you are initially creating a user pool. </li> <li>  <code>OPTIONAL</code> - Users have the option when registering to create an MFA token. </li> </ul><br/>SmsAuthenticationMessage:  The contents of the SMS authentication message.<br/>VerificationMessageTemplate:  The template for verification messages.:<br/>EmailVerificationSubject:  The subject of the email verification message.<br/>EmailVerificationMessage:  The contents of the email verification message.<br/>SmsVerificationMessage:  A container with information about the SMS verification message.<br/>AutoVerifiedAttributes:  The attributes that are automatically verified when the Amazon Cognito service makes a request to update user pools.<br/>LambdaConfig:  The AWS Lambda configuration information from the request to update the user pool.:<br/>Policies:  A container with the policies you wish to update in a user pool.:<br/>UserPoolId:  The user pool ID for the user pool you want to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PasswordPolicy</i> The password policy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PasswordPolicy</b>: The password policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TemporaryPasswordValidityDays</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RequireSymbols</i> In the password policy that you have set, refers to whether you have required users to use at least one symbol in their password.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RequireNumbers</i> In the password policy that you have set, refers to whether you have required users to use at least one number in their password.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RequireLowercase</i> In the password policy that you have set, refers to whether you have required users to use at least one lowercase letter in their password.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RequireUppercase</i> In the password policy that you have set, refers to whether you have required users to use at least one uppercase letter in their password.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinimumLength</i> The minimum length of the password policy that you have set. Cannot be less than 6.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserMigration</i> The user migration Lambda config type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreTokenGeneration</i> A Lambda trigger that is invoked before token generation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VerifyAuthChallengeResponse</i> Verifies the authentication challenge response.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateAuthChallenge</i> Creates an authentication challenge.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefineAuthChallenge</i> Defines the authentication challenge.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PostAuthentication</i> A post-authentication AWS Lambda trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreAuthentication</i> A pre-authentication AWS Lambda trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PostConfirmation</i> A post-confirmation AWS Lambda trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomMessage</i> A custom Message AWS Lambda trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreSignUp</i> A pre-registration AWS Lambda trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultEmailOption</i> The default email option.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailSubjectByLink</i> The subject line for the email message template for sending a confirmation link to the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailMessageByLink</i> The email message template for sending a confirmation link to the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailSubject</i> The subject line for the email message template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailMessage</i> The email message template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SmsMessage</i> The SMS message template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceOnlyRememberedOnUserPrompt</i> If true, a device is only remembered on user prompt.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ChallengeRequiredOnNewDevice</i> Indicates whether a challenge is required on a new device. Only applicable to a new device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailSendingAccount</i> Specifies whether Amazon Cognito emails your users by using its built-in email functionality or your Amazon SES email configuration. Specify one of the following values: <dl> <dt>COGNITO_DEFAULT</dt> <dd> When Amazon Cognito emails your users, it uses its built-in email functionality. When you use the default option, Amazon Cognito allows only a limited number of emails each day for your user pool. For typical production environments, the default email limit is below the required delivery volume. To achieve a higher delivery volume, specify DEVELOPER to use your Amazon SES email configuration. To look up the email delivery limit for the default option, see <a href="https://docs.aws.amazon.com/cognito/latest/developerguide/limits.html">Limits in Amazon Cognito</a> in the <i>Amazon Cognito Developer Guide</i>. The default FROM address is no-reply@verificationemail.com. To customize the FROM address, provide the ARN of an Amazon SES verified email address for the <code>SourceArn</code> parameter. </dd> <dt>DEVELOPER</dt> <dd> When Amazon Cognito emails your users, it uses your Amazon SES configuration. Amazon Cognito calls Amazon SES on your behalf to send email from your verified email address. When you use this option, the email delivery limits are the same limits that apply to your Amazon SES verified email address in your AWS account. If you use this option, you must provide the ARN of an Amazon SES verified email address for the <code>SourceArn</code> parameter. Before Amazon Cognito can email your users, it requires additional permissions to call Amazon SES on your behalf. When you update your user pool with this option, Amazon Cognito creates a <i>service-linked role</i>, which is a type of IAM role, in your AWS account. This role contains the permissions that allow Amazon Cognito to access Amazon SES and send email messages with your address. For more information about the service-linked role that Amazon Cognito creates, see <a href="https://docs.aws.amazon.com/cognito/latest/developerguide/using-service-linked-roles.html">Using Service-Linked Roles for Amazon Cognito</a> in the <i>Amazon Cognito Developer Guide</i>. </dd> </dl><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplyToEmailAddress</i> The destination to which the receiver of the email should reply to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceArn</i> The Amazon Resource Name (ARN) of a verified email address in Amazon SES. This email address is used in one of the following ways, depending on the value that you specify for the <code>EmailSendingAccount</code> parameter: <ul> <li> If you specify <code>COGNITO_DEFAULT</code>, Amazon Cognito uses this address as the custom FROM address when it emails your users by using its built-in email account. </li> <li> If you specify <code>DEVELOPER</code>, Amazon Cognito emails your users with this address by calling Amazon SES on your behalf. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalId</i> The external ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsCallerArn</i> The Amazon Resource Name (ARN) of the Amazon Simple Notification Service (SNS) caller.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InviteMessageTemplate</i> The message template to be used for the welcome message to new users. See also <a href="http://docs.aws.amazon.com/cognito/latest/developerguide/cognito-user-pool-settings-message-customizations.html#cognito-user-pool-settings-user-invitation-message-customization">Customizing User Invitation Messages</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InviteMessageTemplate</b>: The message template to be used for the welcome message to new users. See also <a href="http://docs.aws.amazon.com/cognito/latest/developerguide/cognito-user-pool-settings-message-customizations.html#cognito-user-pool-settings-user-invitation-message-customization">Customizing User Invitation Messages</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UnusedAccountValidityDays</i> The user account expiration limit, in days, after which the account is no longer usable. To reset the account after that time limit, you must call <code>AdminCreateUser</code> again, specifying <code>"RESEND"</code> for the <code>MessageAction</code> parameter. The default value for this parameter is 7.  <note> If you set a value for <code>TemporaryPasswordValidityDays</code> in <code>PasswordPolicy</code>, that value will be used and <code>UnusedAccountValidityDays</code> will be deprecated for that user pool.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowAdminCreateUserOnly</i> Set to <code>True</code> if only the administrator is allowed to create user profiles. Set to <code>False</code> if users can sign themselves up via an app.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailSubject</i> The subject line for email messages.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailMessage</i> The message template for email messages.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SMSMessage</i> The message template for SMS messages.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdvancedSecurityMode</i> The advanced security mode.<br/>
        :return:         """
        pass

    def update_user_pool_client(self, ClientId: str, UserPoolId: str, ClientName: str=None, RefreshTokenValidity: int=None, ReadAttributes: List[ClientPermissionType]=None, WriteAttributes: List[ClientPermissionType]=None, ExplicitAuthFlows: List[ExplicitAuthFlowsType]=None, SupportedIdentityProviders: List[ProviderNameType]=None, CallbackURLs: List[RedirectUrlType]=None, LogoutURLs: List[RedirectUrlType]=None, DefaultRedirectURI: str=None, AllowedOAuthFlows: List[OAuthFlowType]=None, AllowedOAuthScopes: List[ScopeType]=None, AllowedOAuthFlowsUserPoolClient: BooleanType=None, AnalyticsConfiguration: AnalyticsConfigurationType=None) -> UpdateUserPoolClientResponse:
        """Updates the specified user pool app client with the specified attributes. If you don't provide a value for an attribute, it will be set to the default value. You can get a list of the current user pool app client settings with .

        :param self:
        :param AnalyticsConfiguration:  The Amazon Pinpoint analytics configuration for collecting metrics for this user pool.:<br/>AllowedOAuthFlowsUserPoolClient:  Set to TRUE if the client is allowed to follow the OAuth protocol when interacting with Cognito user pools.<br/>AllowedOAuthScopes:  A list of allowed <code>OAuth</code> scopes. Currently supported values are <code>"phone"</code>, <code>"email"</code>, <code>"openid"</code>, and <code>"Cognito"</code>.<br/>AllowedOAuthFlows:  Set to <code>code</code> to initiate a code grant flow, which provides an authorization code as the response. This code can be exchanged for access tokens with the token endpoint.<br/>DefaultRedirectURI:  The default redirect URI. Must be in the <code>CallbackURLs</code> list. A redirect URI must: <ul> <li> Be an absolute URI. </li> <li> Be registered with the authorization server. </li> <li> Not include a fragment component. </li> </ul> See <a href="https://tools.ietf.org/html/rfc6749#section-3.1.2">OAuth 2.0 - Redirection Endpoint</a>. Amazon Cognito requires HTTPS over HTTP except for http://localhost for testing purposes only. App callback URLs such as myapp://example are also supported.<br/>LogoutURLs:  A list of allowed logout URLs for the identity providers.<br/>CallbackURLs:  A list of allowed redirect (callback) URLs for the identity providers. A redirect URI must: <ul> <li> Be an absolute URI. </li> <li> Be registered with the authorization server. </li> <li> Not include a fragment component. </li> </ul> See <a href="https://tools.ietf.org/html/rfc6749#section-3.1.2">OAuth 2.0 - Redirection Endpoint</a>. Amazon Cognito requires HTTPS over HTTP except for http://localhost for testing purposes only. App callback URLs such as myapp://example are also supported.<br/>SupportedIdentityProviders:  A list of provider names for the identity providers that are supported on this client.<br/>ExplicitAuthFlows:  Explicit authentication flows.<br/>WriteAttributes:  The writeable attributes of the user pool.<br/>ReadAttributes:  The read-only attributes of the user pool.<br/>RefreshTokenValidity:  The time limit, in days, after which the refresh token is no longer valid and cannot be used.<br/>ClientName:  The client name from the update user pool client request.<br/>ClientId:  The ID of the client associated with the user pool.<br/>UserPoolId:  The user pool ID for the user pool where you want to update the user pool client.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserDataShared</i> If <code>UserDataShared</code> is <code>true</code>, Amazon Cognito will include user data in the events it publishes to Amazon Pinpoint analytics.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalId</i> The external ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The ARN of an IAM role that authorizes Amazon Cognito to publish events to Amazon Pinpoint analytics.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The application ID for an Amazon Pinpoint application.<br/>
        :return: UserPoolClient:  The user pool client value from the response from the server when an update user pool client request is made.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AnalyticsConfiguration</i> The Amazon Pinpoint analytics configuration for the user pool client.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AnalyticsConfiguration</b>: The Amazon Pinpoint analytics configuration for the user pool client.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowedOAuthFlowsUserPoolClient</i> Set to TRUE if the client is allowed to follow the OAuth protocol when interacting with Cognito user pools.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowedOAuthScopes</i> A list of allowed <code>OAuth</code> scopes. Currently supported values are <code>"phone"</code>, <code>"email"</code>, <code>"openid"</code>, and <code>"Cognito"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowedOAuthFlows</i> Set to <code>code</code> to initiate a code grant flow, which provides an authorization code as the response. This code can be exchanged for access tokens with the token endpoint. Set to <code>token</code> to specify that the client should get the access token (and, optionally, ID token, based on scopes) directly.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultRedirectURI</i> The default redirect URI. Must be in the <code>CallbackURLs</code> list. A redirect URI must: <ul> <li> Be an absolute URI. </li> <li> Be registered with the authorization server. </li> <li> Not include a fragment component. </li> </ul> See <a href="https://tools.ietf.org/html/rfc6749#section-3.1.2">OAuth 2.0 - Redirection Endpoint</a>. Amazon Cognito requires HTTPS over HTTP except for http://localhost for testing purposes only. App callback URLs such as myapp://example are also supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogoutURLs</i> A list of allowed logout URLs for the identity providers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallbackURLs</i> A list of allowed redirect (callback) URLs for the identity providers. A redirect URI must: <ul> <li> Be an absolute URI. </li> <li> Be registered with the authorization server. </li> <li> Not include a fragment component. </li> </ul> See <a href="https://tools.ietf.org/html/rfc6749#section-3.1.2">OAuth 2.0 - Redirection Endpoint</a>. Amazon Cognito requires HTTPS over HTTP except for http://localhost for testing purposes only. App callback URLs such as myapp://example are also supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedIdentityProviders</i> A list of provider names for the identity providers that are supported on this client.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExplicitAuthFlows</i> The explicit authentication flows.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WriteAttributes</i> The writeable attributes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadAttributes</i> The Read-only attributes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RefreshTokenValidity</i> The time limit, in days, after which the refresh token is no longer valid and cannot be used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date the user pool client was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date the user pool client was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientSecret</i> The client secret from the user pool request of the client type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientId</i> The ID of the client associated with the user pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientName</i> The client name from the user pool request of the client type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolId</i> The user pool ID for the user pool client.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UserDataShared</i> If <code>UserDataShared</code> is <code>true</code>, Amazon Cognito will include user data in the events it publishes to Amazon Pinpoint analytics.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalId</i> The external ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The ARN of an IAM role that authorizes Amazon Cognito to publish events to Amazon Pinpoint analytics.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The application ID for an Amazon Pinpoint application.<br/>        """
        pass

    def update_user_pool_domain(self, CustomDomainConfig: CustomDomainConfigType, UserPoolId: str, Domain: str) -> UpdateUserPoolDomainResponse:
        """Updates the Secure Sockets Layer (SSL) certificate for the custom domain for your user pool. You can use this operation to provide the Amazon Resource Name (ARN) of a new certificate to Amazon Cognito. You cannot use it to change the domain for a user pool. A custom domain is used to host the Amazon Cognito hosted UI, which provides sign-up and sign-in pages for your application. When you set up a custom domain, you provide a certificate that you manage with AWS Certificate Manager (ACM). When necessary, you can use this operation to change the certificate that you applied to your custom domain. Usually, this is unnecessary following routine certificate renewal with ACM. When you renew your existing certificate in ACM, the ARN for your certificate remains the same, and your custom domain uses the new certificate automatically. However, if you replace your existing certificate with a new one, ACM gives the new certificate a new ARN. To apply the new certificate to your custom domain, you must provide this ARN to Amazon Cognito. When you add your new certificate in ACM, you must choose US East (N. Virginia) as the AWS Region. After you submit your request, Amazon Cognito requires up to 1 hour to distribute your new certificate to your custom domain. For more information about adding a custom domain to your user pool, see <a href="https://docs.aws.amazon.com/cognito/latest/developerguide/cognito-user-pools-add-custom-domain.html">Using Your Own Domain for the Hosted UI</a>.

        :param self:
        :param CustomDomainConfig:  The configuration for a custom domain that hosts the sign-up and sign-in pages for your application. Use this object to specify an SSL certificate that is managed by ACM.:<br/>UserPoolId:  The ID of the user pool that is associated with the custom domain that you are updating the certificate for.<br/>Domain:  The domain name for the custom domain that hosts the sign-up and sign-in pages for your application. For example: <code>auth.example.com</code>.  This string can include only lowercase letters, numbers, and hyphens. Do not use a hyphen for the first or last character. Use periods to separate subdomain names.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateArn</i> The Amazon Resource Name (ARN) of an AWS Certificate Manager SSL certificate. You use this certificate for the subdomain of your custom domain.<br/>
        :return: CloudFrontDomain:  The Amazon CloudFront endpoint that Amazon Cognito set up when you added the custom domain to your user pool.<br/>        """
        pass

    def verify_software_token(self, UserCode: str, AccessToken: str=None, Session: str=None, FriendlyDeviceName: str=None) -> VerifySoftwareTokenResponse:
        """Use this API to register a user's entered TOTP code and mark the user's software token MFA status as "verified" if successful. The request takes an access token or a session string, but not both.

        :param self:
        :param FriendlyDeviceName:  The friendly device name.<br/>UserCode:  The one time password computed using the secret code returned by <br/>Session:  The session which should be passed both ways in challenge-response calls to the service.<br/>AccessToken:  The access token.<br/>
        :return: Session:  The session which should be passed both ways in challenge-response calls to the service.<br/>Status:  The status of the verify software token.<br/>        """
        pass

    def verify_user_attribute(self, Code: str, AttributeName: str, AccessToken: str) -> VerifyUserAttributeResponse:
        """Verifies the specified user attributes in the user pool.

        :param self:
        :param Code:  The verification code in the request to verify user attributes.<br/>AttributeName:  The attribute name in the request to verify user attributes.<br/>AccessToken:  Represents the access token of the request to verify user attributes.<br/>
        :return:         """
        pass



class CognitoSync(BaseClient):
    """<fullname>Amazon Cognito Sync</fullname> Amazon Cognito Sync provides an AWS service and client library that enable cross-device syncing of application-related user data. High-level client libraries are available for both iOS and Android. You can use these libraries to persist data locally so that it's available even if the device is offline. Developer credentials don't need to be stored on the mobile device to access the service. You can use Amazon Cognito to obtain a normalized user ID and credentials. User data is persisted in a dataset that can store up to 1 MB of key-value pairs, and you can have up to 20 datasets per user identity. With Amazon Cognito Sync, the data stored for each identity is accessible only to credentials assigned to that identity. In order to use the Cognito Sync service, you need to make API calls using credentials retrieved with <a href="http://docs.aws.amazon.com/cognitoidentity/latest/APIReference/Welcome.html">Amazon Cognito Identity service</a>. If you want to use Cognito Sync in an Android or iOS application, you will probably want to make API calls via the AWS Mobile SDK. To learn more, see the <a href="http://docs.aws.amazon.com/mobile/sdkforandroid/developerguide/cognito-sync.html">Developer Guide for Android</a> and the <a href="http://docs.aws.amazon.com/mobile/sdkforios/developerguide/cognito-sync.html">Developer Guide for iOS</a>."""



    class BulkPublishRequest(Mapping):
        pass
    
    class BulkPublishResponse(Mapping):
        pass
    
    class DeleteDatasetRequest(Mapping):
        pass
    
    class DeleteDatasetResponse(Mapping):
        pass
    
    class DescribeDatasetRequest(Mapping):
        pass
    
    class DescribeDatasetResponse(Mapping):
        pass
    
    class DescribeIdentityPoolUsageRequest(Mapping):
        pass
    
    class DescribeIdentityPoolUsageResponse(Mapping):
        pass
    
    class DescribeIdentityUsageRequest(Mapping):
        pass
    
    class DescribeIdentityUsageResponse(Mapping):
        pass
    
    class GetBulkPublishDetailsRequest(Mapping):
        pass
    
    class GetBulkPublishDetailsResponse(Mapping):
        pass
    
    class GetCognitoEventsRequest(Mapping):
        pass
    
    class GetCognitoEventsResponse(Mapping):
        pass
    
    class GetIdentityPoolConfigurationRequest(Mapping):
        pass
    
    class GetIdentityPoolConfigurationResponse(Mapping):
        pass
    
    class ListDatasetsRequest(Mapping):
        pass
    
    class ListDatasetsResponse(Mapping):
        pass
    
    class ListIdentityPoolUsageRequest(Mapping):
        pass
    
    class ListIdentityPoolUsageResponse(Mapping):
        pass
    
    class ListRecordsRequest(Mapping):
        pass
    
    class ListRecordsResponse(Mapping):
        pass
    
    class Long(object):
        pass
    
    class RegisterDeviceRequest(Mapping):
        pass
    
    class RegisterDeviceResponse(Mapping):
        pass
    
    class SetCognitoEventsRequest(Mapping):
        pass
    
    class Events(object):
        pass
    
    class SetIdentityPoolConfigurationRequest(Mapping):
        pass
    
    class SetIdentityPoolConfigurationResponse(Mapping):
        pass
    
    class PushSync(Mapping):
        pass
    
    class CognitoStreams(Mapping):
        pass
    
    class SubscribeToDatasetRequest(Mapping):
        pass
    
    class SubscribeToDatasetResponse(Mapping):
        pass
    
    class UnsubscribeFromDatasetRequest(Mapping):
        pass
    
    class UnsubscribeFromDatasetResponse(Mapping):
        pass
    
    class UpdateRecordsRequest(Mapping):
        pass
    
    class UpdateRecordsResponse(Mapping):
        pass
    
    class RecordPatch(Mapping):
        pass
    
    def bulk_publish(self, IdentityPoolId: str) -> BulkPublishResponse:
        """Initiates a bulk publish of all existing datasets for an Identity Pool to the configured stream. Customers are limited to one successful bulk publish per 24 hours. Bulk publish is an asynchronous request, customers can see the status of the request via the GetBulkPublishDetails operation. This API can only be called with developer credentials. You cannot call this API with the temporary user credentials provided by Cognito Identity.

        :param self:
        :param IdentityPoolId:  A name-spaced GUID (for example, us-east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon Cognito. GUID generation is unique within a region.<br/>
        :return: IdentityPoolId:  A name-spaced GUID (for example, us-east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon Cognito. GUID generation is unique within a region.<br/>        """
        pass

    def delete_dataset(self, DatasetName: str, IdentityId: str, IdentityPoolId: str) -> DeleteDatasetResponse:
        """Deletes the specific dataset. The dataset will be deleted permanently, and the action can't be undone. Datasets that this dataset was merged with will no longer report the merge. Any subsequent operation on this dataset will result in a ResourceNotFoundException. This API can be called with temporary user credentials provided by Cognito Identity or with developer credentials.

        :param self:
        :param DatasetName:  A string of up to 128 characters. Allowed characters are a-z, A-Z, 0-9, '_' (underscore), '-' (dash), and '.' (dot).<br/>IdentityId:  A name-spaced GUID (for example, us-east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon Cognito. GUID generation is unique within a region.<br/>IdentityPoolId:  A name-spaced GUID (for example, us-east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon Cognito. GUID generation is unique within a region.<br/>
        :return: Dataset:  A collection of data for an identity pool. An identity pool can have multiple datasets. A dataset is per identity and can be general or associated with a particular entity in an application (like a saved game). Datasets are automatically created if they don't exist. Data is synced by dataset, and a dataset can hold up to 1MB of key-value pairs.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumRecords</i> Number of records in this dataset.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataStorage</i> Total size in bytes of the records in this dataset.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The device that made the last change to this dataset.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> Date when the dataset was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> Date on which the dataset was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatasetName</i> A string of up to 128 characters. Allowed characters are a-z, A-Z, 0-9, '_' (underscore), '-' (dash), and '.' (dot).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IdentityId</i> A name-spaced GUID (for example, us-east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon Cognito. GUID generation is unique within a region.<br/>        """
        pass

    def describe_dataset(self, DatasetName: str, IdentityId: str, IdentityPoolId: str) -> DescribeDatasetResponse:
        """Gets meta data about a dataset by identity and dataset name. With Amazon Cognito Sync, each identity has access only to its own data. Thus, the credentials used to make this API call need to have access to the identity data. This API can be called with temporary user credentials provided by Cognito Identity or with developer credentials. You should use Cognito Identity credentials to make this API call.

        :param self:
        :param DatasetName:  A string of up to 128 characters. Allowed characters are a-z, A-Z, 0-9, '_' (underscore), '-' (dash), and '.' (dot).<br/>IdentityId:  A name-spaced GUID (for example, us-east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon Cognito. GUID generation is unique within a region.<br/>IdentityPoolId:  A name-spaced GUID (for example, us-east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon Cognito. GUID generation is unique within a region.<br/>
        :return: Dataset:  Meta data for a collection of data for an identity. An identity can have multiple datasets. A dataset can be general or associated with a particular entity in an application (like a saved game). Datasets are automatically created if they don't exist. Data is synced by dataset, and a dataset can hold up to 1MB of key-value pairs.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumRecords</i> Number of records in this dataset.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataStorage</i> Total size in bytes of the records in this dataset.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The device that made the last change to this dataset.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> Date when the dataset was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> Date on which the dataset was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatasetName</i> A string of up to 128 characters. Allowed characters are a-z, A-Z, 0-9, '_' (underscore), '-' (dash), and '.' (dot).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IdentityId</i> A name-spaced GUID (for example, us-east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon Cognito. GUID generation is unique within a region.<br/>        """
        pass

    def describe_identity_pool_usage(self, IdentityPoolId: str) -> DescribeIdentityPoolUsageResponse:
        """Gets usage details (for example, data storage) about a particular identity pool. This API can only be called with developer credentials. You cannot call this API with the temporary user credentials provided by Cognito Identity.

        :param self:
        :param IdentityPoolId:  A name-spaced GUID (for example, us-east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon Cognito. GUID generation is unique within a region.<br/>
        :return: IdentityPoolUsage:  Information about the usage of the identity pool.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> Date on which the identity pool was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataStorage</i> Data storage information for the identity pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SyncSessionsCount</i> Number of sync sessions for the identity pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IdentityPoolId</i> A name-spaced GUID (for example, us-east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon Cognito. GUID generation is unique within a region.<br/>        """
        pass

    def describe_identity_usage(self, IdentityId: str, IdentityPoolId: str) -> DescribeIdentityUsageResponse:
        """Gets usage information for an identity, including number of datasets and data usage. This API can be called with temporary user credentials provided by Cognito Identity or with developer credentials.

        :param self:
        :param IdentityId:  A name-spaced GUID (for example, us-east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon Cognito. GUID generation is unique within a region.<br/>IdentityPoolId:  A name-spaced GUID (for example, us-east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon Cognito. GUID generation is unique within a region.<br/>
        :return: IdentityUsage:  Usage information for the identity.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataStorage</i> Total data storage for this identity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatasetCount</i> Number of datasets for the identity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> Date on which the identity was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IdentityPoolId</i> A name-spaced GUID (for example, us-east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon Cognito. GUID generation is unique within a region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IdentityId</i> A name-spaced GUID (for example, us-east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon Cognito. GUID generation is unique within a region.<br/>        """
        pass

    def get_bulk_publish_details(self, IdentityPoolId: str) -> GetBulkPublishDetailsResponse:
        """Get the status of the last BulkPublish operation for an identity pool. This API can only be called with developer credentials. You cannot call this API with the temporary user credentials provided by Cognito Identity.

        :param self:
        :param IdentityPoolId:  A name-spaced GUID (for example, us-east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon Cognito. GUID generation is unique within a region.<br/>
        :return: FailureMessage:  If BulkPublishStatus is FAILED this field will contain the error message that caused the bulk publish to fail.<br/>BulkPublishStatus:  Status of the last bulk publish operation, valid values are: NOT_STARTED - No bulk publish has been requested for this identity pool IN_PROGRESS - Data is being published to the configured stream SUCCEEDED - All data for the identity pool has been published to the configured stream FAILED - Some portion of the data has failed to publish, check FailureMessage for the cause.<br/>BulkPublishCompleteTime:  If BulkPublishStatus is SUCCEEDED, the time the last bulk publish operation completed.<br/>BulkPublishStartTime:  The date/time at which the last bulk publish was initiated.<br/>IdentityPoolId:  A name-spaced GUID (for example, us-east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon Cognito. GUID generation is unique within a region.<br/>        """
        pass

    def get_cognito_events(self, IdentityPoolId: str) -> GetCognitoEventsResponse:
        """Gets the events and the corresponding Lambda functions associated with an identity pool. This API can only be called with developer credentials. You cannot call this API with the temporary user credentials provided by Cognito Identity.

        :param self:
        :param IdentityPoolId:  The Cognito Identity Pool ID for the request<br/>
        :return: Events:  The Cognito Events returned from the GetCognitoEvents request<br/>        """
        pass

    def get_identity_pool_configuration(self, IdentityPoolId: str) -> GetIdentityPoolConfigurationResponse:
        """Gets the configuration settings of an identity pool. This API can only be called with developer credentials. You cannot call this API with the temporary user credentials provided by Cognito Identity.

        :param self:
        :param IdentityPoolId:  A name-spaced GUID (for example, us-east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon Cognito. This is the ID of the pool for which to return a configuration.<br/>
        :return: CognitoStreams:  Options to apply to this identity pool for Amazon Cognito streams.:<br/>PushSync:  Options to apply to this identity pool for push synchronization.:<br/>IdentityPoolId:  A name-spaced GUID (for example, us-east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon Cognito.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> A role configured to allow Cognito to call SNS on behalf of the developer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationArns</i> List of SNS platform application ARNs that could be used by clients.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamingStatus</i> Status of the Cognito streams. Valid values are: ENABLED - Streaming of updates to identity pool is enabled. DISABLED - Streaming of updates to identity pool is disabled. Bulk publish will also fail if StreamingStatus is DISABLED.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The ARN of the role Amazon Cognito can assume in order to publish to the stream. This role must grant access to Amazon Cognito (cognito-sync) to invoke PutRecord on your Cognito stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamName</i> The name of the Cognito stream to receive updates. This stream must be in the developers account and in the same region as the identity pool.<br/>        """
        pass

    def list_datasets(self, IdentityId: str, IdentityPoolId: str, NextToken: str=None, MaxResults: int=None) -> ListDatasetsResponse:
        """Lists datasets for an identity. With Amazon Cognito Sync, each identity has access only to its own data. Thus, the credentials used to make this API call need to have access to the identity data. ListDatasets can be called with temporary user credentials provided by Cognito Identity or with developer credentials. You should use the Cognito Identity credentials to make this API call.

        :param self:
        :param MaxResults:  The maximum number of results to be returned.<br/>NextToken:  A pagination token for obtaining the next page of results.<br/>IdentityId:  A name-spaced GUID (for example, us-east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon Cognito. GUID generation is unique within a region.<br/>IdentityPoolId:  A name-spaced GUID (for example, us-east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon Cognito. GUID generation is unique within a region.<br/>
        :return: NextToken:  A pagination token for obtaining the next page of results.<br/>Count:  Number of datasets returned.<br/>Datasets:  A set of datasets.<br/>        """
        pass

    def list_identity_pool_usage(self, NextToken: str=None, MaxResults: int=None) -> ListIdentityPoolUsageResponse:
        """Gets a list of identity pools registered with Cognito. ListIdentityPoolUsage can only be called with developer credentials. You cannot make this API call with the temporary user credentials provided by Cognito Identity.

        :param self:
        :param MaxResults:  The maximum number of results to be returned.<br/>NextToken:  A pagination token for obtaining the next page of results.<br/>
        :return: NextToken:  A pagination token for obtaining the next page of results.<br/>Count:  Total number of identities for the identity pool.<br/>MaxResults:  The maximum number of results to be returned.<br/>IdentityPoolUsages:  Usage information for the identity pools.<br/>        """
        pass

    def list_records(self, DatasetName: str, IdentityId: str, IdentityPoolId: str, LastSyncCount: Long=None, NextToken: str=None, MaxResults: int=None, SyncSessionToken: str=None) -> ListRecordsResponse:
        """Gets paginated records, optionally changed after a particular sync count for a dataset and identity. With Amazon Cognito Sync, each identity has access only to its own data. Thus, the credentials used to make this API call need to have access to the identity data. ListRecords can be called with temporary user credentials provided by Cognito Identity or with developer credentials. You should use Cognito Identity credentials to make this API call.

        :param self:
        :param SyncSessionToken:  A token containing a session ID, identity ID, and expiration.<br/>MaxResults:  The maximum number of results to be returned.<br/>NextToken:  A pagination token for obtaining the next page of results.<br/>LastSyncCount:  The last server sync count for this record.<br/>DatasetName:  A string of up to 128 characters. Allowed characters are a-z, A-Z, 0-9, '_' (underscore), '-' (dash), and '.' (dot).<br/>IdentityId:  A name-spaced GUID (for example, us-east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon Cognito. GUID generation is unique within a region.<br/>IdentityPoolId:  A name-spaced GUID (for example, us-east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon Cognito. GUID generation is unique within a region.<br/>
        :return: SyncSessionToken:  A token containing a session ID, identity ID, and expiration.<br/>DatasetDeletedAfterRequestedSyncCount:  A boolean value specifying whether to delete the dataset locally.<br/>DatasetExists:  Indicates whether the dataset exists.<br/>MergedDatasetNames:  Names of merged datasets.<br/>LastModifiedBy:  The user/device that made the last change to this record.<br/>DatasetSyncCount:  Server sync count for this dataset.<br/>Count:  Total number of records.<br/>NextToken:  A pagination token for obtaining the next page of results.<br/>Records:  A list of all records.<br/>        """
        pass

    def register_device(self, Token: str, Platform: str, IdentityId: str, IdentityPoolId: str) -> RegisterDeviceResponse:
        """Registers a device to receive push sync notifications. This API can only be called with temporary credentials provided by Cognito Identity. You cannot call this API with developer credentials.

        :param self:
        :param Token:  The push token.<br/>Platform:  The SNS platform type (e.g. GCM, SDM, APNS, APNS_SANDBOX).<br/>IdentityId:  The unique ID for this identity.<br/>IdentityPoolId:  A name-spaced GUID (for example, us-east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon Cognito. Here, the ID of the pool that the identity belongs to.<br/>
        :return: DeviceId:  The unique ID generated for this device by Cognito.<br/>        """
        pass

    def set_cognito_events(self, Events: Events, IdentityPoolId: str):
        """Sets the AWS Lambda function for a given event type for an identity pool. This request only updates the key/value pair specified. Other key/values pairs are not updated. To remove a key value pair, pass a empty value for the particular key. This API can only be called with developer credentials. You cannot call this API with the temporary user credentials provided by Cognito Identity.

        :param self:
        :param Events:  The events to configure<br/>IdentityPoolId:  The Cognito Identity Pool to use when configuring Cognito Events<br/>
        :return:         """
        pass

    def set_identity_pool_configuration(self, IdentityPoolId: str, PushSync: PushSync=None, CognitoStreams: CognitoStreams=None) -> SetIdentityPoolConfigurationResponse:
        """Sets the necessary configuration for push sync. This API can only be called with developer credentials. You cannot call this API with the temporary user credentials provided by Cognito Identity.

        :param self:
        :param CognitoStreams:  Options to apply to this identity pool for Amazon Cognito streams.:<br/>PushSync:  Options to apply to this identity pool for push synchronization.:<br/>IdentityPoolId:  A name-spaced GUID (for example, us-east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon Cognito. This is the ID of the pool to modify.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> A role configured to allow Cognito to call SNS on behalf of the developer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationArns</i> List of SNS platform application ARNs that could be used by clients.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamingStatus</i> Status of the Cognito streams. Valid values are: ENABLED - Streaming of updates to identity pool is enabled. DISABLED - Streaming of updates to identity pool is disabled. Bulk publish will also fail if StreamingStatus is DISABLED.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The ARN of the role Amazon Cognito can assume in order to publish to the stream. This role must grant access to Amazon Cognito (cognito-sync) to invoke PutRecord on your Cognito stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamName</i> The name of the Cognito stream to receive updates. This stream must be in the developers account and in the same region as the identity pool.<br/>
        :return: CognitoStreams:  Options to apply to this identity pool for Amazon Cognito streams.:<br/>PushSync:  Options to apply to this identity pool for push synchronization.:<br/>IdentityPoolId:  A name-spaced GUID (for example, us-east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon Cognito.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> A role configured to allow Cognito to call SNS on behalf of the developer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationArns</i> List of SNS platform application ARNs that could be used by clients.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamingStatus</i> Status of the Cognito streams. Valid values are: ENABLED - Streaming of updates to identity pool is enabled. DISABLED - Streaming of updates to identity pool is disabled. Bulk publish will also fail if StreamingStatus is DISABLED.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The ARN of the role Amazon Cognito can assume in order to publish to the stream. This role must grant access to Amazon Cognito (cognito-sync) to invoke PutRecord on your Cognito stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamName</i> The name of the Cognito stream to receive updates. This stream must be in the developers account and in the same region as the identity pool.<br/>        """
        pass

    def subscribe_to_dataset(self, DeviceId: str, DatasetName: str, IdentityId: str, IdentityPoolId: str) -> SubscribeToDatasetResponse:
        """Subscribes to receive notifications when a dataset is modified by another device. This API can only be called with temporary credentials provided by Cognito Identity. You cannot call this API with developer credentials.

        :param self:
        :param DeviceId:  The unique ID generated for this device by Cognito.<br/>DatasetName:  The name of the dataset to subcribe to.<br/>IdentityId:  Unique ID for this identity.<br/>IdentityPoolId:  A name-spaced GUID (for example, us-east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon Cognito. The ID of the pool to which the identity belongs.<br/>
        :return:         """
        pass

    def unsubscribe_from_dataset(self, DeviceId: str, DatasetName: str, IdentityId: str, IdentityPoolId: str) -> UnsubscribeFromDatasetResponse:
        """Unsubscribes from receiving notifications when a dataset is modified by another device. This API can only be called with temporary credentials provided by Cognito Identity. You cannot call this API with developer credentials.

        :param self:
        :param DeviceId:  The unique ID generated for this device by Cognito.<br/>DatasetName:  The name of the dataset from which to unsubcribe.<br/>IdentityId:  Unique ID for this identity.<br/>IdentityPoolId:  A name-spaced GUID (for example, us-east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon Cognito. The ID of the pool to which this identity belongs.<br/>
        :return:         """
        pass

    def update_records(self, SyncSessionToken: str, DatasetName: str, IdentityId: str, IdentityPoolId: str, DeviceId: str=None, RecordPatches: List[RecordPatch]=None, ClientContext: str=None) -> UpdateRecordsResponse:
        """Posts updates to records and adds and deletes records for a dataset and user. The sync count in the record patch is your last known sync count for that record. The server will reject an UpdateRecords request with a ResourceConflictException if you try to patch a record with a new value but a stale sync count. For example, if the sync count on the server is 5 for a key called highScore and you try and submit a new highScore with sync count of 4, the request will be rejected. To obtain the current sync count for a record, call ListRecords. On a successful update of the record, the response returns the new sync count for that record. You should present that sync count the next time you try to update that same record. When the record does not exist, specify the sync count as 0. This API can be called with temporary user credentials provided by Cognito Identity or with developer credentials.

        :param self:
        :param ClientContext:  Intended to supply a device ID that will populate the lastModifiedBy field referenced in other methods. The ClientContext field is not yet implemented.<br/>SyncSessionToken:  The SyncSessionToken returned by a previous call to ListRecords for this dataset and identity.<br/>RecordPatches:  A list of patch operations.<br/>DeviceId:  The unique ID generated for this device by Cognito.<br/>DatasetName:  A string of up to 128 characters. Allowed characters are a-z, A-Z, 0-9, '_' (underscore), '-' (dash), and '.' (dot).<br/>IdentityId:  A name-spaced GUID (for example, us-east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon Cognito. GUID generation is unique within a region.<br/>IdentityPoolId:  A name-spaced GUID (for example, us-east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon Cognito. GUID generation is unique within a region.<br/>
        :return: Records:  A list of records that have been updated.<br/>        """
        pass



class Comprehend(BaseClient):
    """Amazon Comprehend is an AWS service for gaining insight into the content of documents. Use these actions to determine the topics contained in your documents, the topics they discuss, the predominant sentiment expressed in them, the predominant language used, and more."""


    class ListTopicsDetectionJobsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_topics_detection_jobs` for valid parameters"""
            pass
    class ListDocumentClassificationJobsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_document_classification_jobs` for valid parameters"""
            pass
    class ListDocumentClassifiersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_document_classifiers` for valid parameters"""
            pass
    class ListDominantLanguageDetectionJobsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_dominant_language_detection_jobs` for valid parameters"""
            pass
    class ListEntitiesDetectionJobsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_entities_detection_jobs` for valid parameters"""
            pass
    class ListEntityRecognizersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_entity_recognizers` for valid parameters"""
            pass
    class ListKeyPhrasesDetectionJobsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_key_phrases_detection_jobs` for valid parameters"""
            pass
    class ListSentimentDetectionJobsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_sentiment_detection_jobs` for valid parameters"""
            pass


    class BatchDetectDominantLanguageRequest(Mapping):
        pass
    
    class BatchDetectDominantLanguageResponse(Mapping):
        pass
    
    class String(object):
        pass
    
    class BatchDetectEntitiesRequest(Mapping):
        pass
    
    class BatchDetectEntitiesResponse(Mapping):
        pass
    
    class BatchDetectKeyPhrasesRequest(Mapping):
        pass
    
    class BatchDetectKeyPhrasesResponse(Mapping):
        pass
    
    class BatchDetectSentimentRequest(Mapping):
        pass
    
    class BatchDetectSentimentResponse(Mapping):
        pass
    
    class BatchDetectSyntaxRequest(Mapping):
        pass
    
    class BatchDetectSyntaxResponse(Mapping):
        pass
    
    class CreateDocumentClassifierRequest(Mapping):
        pass
    
    class CreateDocumentClassifierResponse(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class DocumentClassifierInputDataConfig(Mapping):
        pass
    
    class DocumentClassifierOutputDataConfig(Mapping):
        pass
    
    class VpcConfig(Mapping):
        pass
    
    class CreateEntityRecognizerRequest(Mapping):
        pass
    
    class CreateEntityRecognizerResponse(Mapping):
        pass
    
    class EntityRecognizerInputDataConfig(Mapping):
        pass
    
    class DeleteDocumentClassifierRequest(Mapping):
        pass
    
    class DeleteDocumentClassifierResponse(Mapping):
        pass
    
    class DeleteEntityRecognizerRequest(Mapping):
        pass
    
    class DeleteEntityRecognizerResponse(Mapping):
        pass
    
    class DescribeDocumentClassificationJobRequest(Mapping):
        pass
    
    class DescribeDocumentClassificationJobResponse(Mapping):
        pass
    
    class DescribeDocumentClassifierRequest(Mapping):
        pass
    
    class DescribeDocumentClassifierResponse(Mapping):
        pass
    
    class DescribeDominantLanguageDetectionJobRequest(Mapping):
        pass
    
    class DescribeDominantLanguageDetectionJobResponse(Mapping):
        pass
    
    class DescribeEntitiesDetectionJobRequest(Mapping):
        pass
    
    class DescribeEntitiesDetectionJobResponse(Mapping):
        pass
    
    class DescribeEntityRecognizerRequest(Mapping):
        pass
    
    class DescribeEntityRecognizerResponse(Mapping):
        pass
    
    class DescribeKeyPhrasesDetectionJobRequest(Mapping):
        pass
    
    class DescribeKeyPhrasesDetectionJobResponse(Mapping):
        pass
    
    class DescribeSentimentDetectionJobRequest(Mapping):
        pass
    
    class DescribeSentimentDetectionJobResponse(Mapping):
        pass
    
    class DescribeTopicsDetectionJobRequest(Mapping):
        pass
    
    class DescribeTopicsDetectionJobResponse(Mapping):
        pass
    
    class DetectDominantLanguageRequest(Mapping):
        pass
    
    class DetectDominantLanguageResponse(Mapping):
        pass
    
    class DetectEntitiesRequest(Mapping):
        pass
    
    class DetectEntitiesResponse(Mapping):
        pass
    
    class DetectKeyPhrasesRequest(Mapping):
        pass
    
    class DetectKeyPhrasesResponse(Mapping):
        pass
    
    class DetectSentimentRequest(Mapping):
        pass
    
    class DetectSentimentResponse(Mapping):
        pass
    
    class DetectSyntaxRequest(Mapping):
        pass
    
    class DetectSyntaxResponse(Mapping):
        pass
    
    class ListDocumentClassificationJobsRequest(Mapping):
        pass
    
    class ListDocumentClassificationJobsResponse(Mapping):
        pass
    
    class DocumentClassificationJobFilter(Mapping):
        pass
    
    class ListDocumentClassifiersRequest(Mapping):
        pass
    
    class ListDocumentClassifiersResponse(Mapping):
        pass
    
    class DocumentClassifierFilter(Mapping):
        pass
    
    class ListDominantLanguageDetectionJobsRequest(Mapping):
        pass
    
    class ListDominantLanguageDetectionJobsResponse(Mapping):
        pass
    
    class DominantLanguageDetectionJobFilter(Mapping):
        pass
    
    class ListEntitiesDetectionJobsRequest(Mapping):
        pass
    
    class ListEntitiesDetectionJobsResponse(Mapping):
        pass
    
    class EntitiesDetectionJobFilter(Mapping):
        pass
    
    class ListEntityRecognizersRequest(Mapping):
        pass
    
    class ListEntityRecognizersResponse(Mapping):
        pass
    
    class EntityRecognizerFilter(Mapping):
        pass
    
    class ListKeyPhrasesDetectionJobsRequest(Mapping):
        pass
    
    class ListKeyPhrasesDetectionJobsResponse(Mapping):
        pass
    
    class KeyPhrasesDetectionJobFilter(Mapping):
        pass
    
    class ListSentimentDetectionJobsRequest(Mapping):
        pass
    
    class ListSentimentDetectionJobsResponse(Mapping):
        pass
    
    class SentimentDetectionJobFilter(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class ListTopicsDetectionJobsRequest(Mapping):
        pass
    
    class ListTopicsDetectionJobsResponse(Mapping):
        pass
    
    class TopicsDetectionJobFilter(Mapping):
        pass
    
    class StartDocumentClassificationJobRequest(Mapping):
        pass
    
    class StartDocumentClassificationJobResponse(Mapping):
        pass
    
    class InputDataConfig(Mapping):
        pass
    
    class OutputDataConfig(Mapping):
        pass
    
    class StartDominantLanguageDetectionJobRequest(Mapping):
        pass
    
    class StartDominantLanguageDetectionJobResponse(Mapping):
        pass
    
    class StartEntitiesDetectionJobRequest(Mapping):
        pass
    
    class StartEntitiesDetectionJobResponse(Mapping):
        pass
    
    class StartKeyPhrasesDetectionJobRequest(Mapping):
        pass
    
    class StartKeyPhrasesDetectionJobResponse(Mapping):
        pass
    
    class StartSentimentDetectionJobRequest(Mapping):
        pass
    
    class StartSentimentDetectionJobResponse(Mapping):
        pass
    
    class StartTopicsDetectionJobRequest(Mapping):
        pass
    
    class StartTopicsDetectionJobResponse(Mapping):
        pass
    
    class StopDominantLanguageDetectionJobRequest(Mapping):
        pass
    
    class StopDominantLanguageDetectionJobResponse(Mapping):
        pass
    
    class StopEntitiesDetectionJobRequest(Mapping):
        pass
    
    class StopEntitiesDetectionJobResponse(Mapping):
        pass
    
    class StopKeyPhrasesDetectionJobRequest(Mapping):
        pass
    
    class StopKeyPhrasesDetectionJobResponse(Mapping):
        pass
    
    class StopSentimentDetectionJobRequest(Mapping):
        pass
    
    class StopSentimentDetectionJobResponse(Mapping):
        pass
    
    class StopTrainingDocumentClassifierRequest(Mapping):
        pass
    
    class StopTrainingDocumentClassifierResponse(Mapping):
        pass
    
    class StopTrainingEntityRecognizerRequest(Mapping):
        pass
    
    class StopTrainingEntityRecognizerResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    def batch_detect_dominant_language(self, TextList: List[String]) -> BatchDetectDominantLanguageResponse:
        """Determines the dominant language of the input text for a batch of documents. For a list of languages that Amazon Comprehend can detect, see <a href="https://docs.aws.amazon.com/comprehend/latest/dg/how-languages.html">Amazon Comprehend Supported Languages</a>. 

        :param self:
        :param TextList:  A list containing the text of the input documents. The list can contain a maximum of 25 documents. Each document should contain at least 20 characters and must contain fewer than 5,000 bytes of UTF-8 encoded characters.<br/>
        :return: ErrorList:  A list containing one object for each document that contained an error. The results are sorted in ascending order by the <code>Index</code> field and match the order of the documents in the input list. If there are no errors in the batch, the <code>ErrorList</code> is empty.<br/>ResultList:  A list of objects containing the results of the operation. The results are sorted in ascending order by the <code>Index</code> field and match the order of the documents in the input list. If all of the documents contain an error, the <code>ResultList</code> is empty.<br/>        """
        pass

    def batch_detect_entities(self, LanguageCode: str, TextList: List[String]) -> BatchDetectEntitiesResponse:
        """Inspects the text of a batch of documents for named entities and returns information about them. For more information about named entities, see <a>how-entities</a> 

        :param self:
        :param LanguageCode:  The language of the input documents. You can specify any of the primary languages supported by Amazon Comprehend: German ("de"), English ("en"), Spanish ("es"), French ("fr"), Italian ("it"), or Portuguese ("pt"). All documents must be in the same language.<br/>TextList:  A list containing the text of the input documents. The list can contain a maximum of 25 documents. Each document must contain fewer than 5,000 bytes of UTF-8 encoded characters.<br/>
        :return: ErrorList:  A list containing one object for each document that contained an error. The results are sorted in ascending order by the <code>Index</code> field and match the order of the documents in the input list. If there are no errors in the batch, the <code>ErrorList</code> is empty.<br/>ResultList:  A list of objects containing the results of the operation. The results are sorted in ascending order by the <code>Index</code> field and match the order of the documents in the input list. If all of the documents contain an error, the <code>ResultList</code> is empty.<br/>        """
        pass

    def batch_detect_key_phrases(self, LanguageCode: str, TextList: List[String]) -> BatchDetectKeyPhrasesResponse:
        """Detects the key noun phrases found in a batch of documents.

        :param self:
        :param LanguageCode:  The language of the input documents. You can specify any of the primary languages supported by Amazon Comprehend: German ("de"), English ("en"), Spanish ("es"), French ("fr"), Italian ("it"), or Portuguese ("pt"). All documents must be in the same language.<br/>TextList:  A list containing the text of the input documents. The list can contain a maximum of 25 documents. Each document must contain fewer that 5,000 bytes of UTF-8 encoded characters.<br/>
        :return: ErrorList:  A list containing one object for each document that contained an error. The results are sorted in ascending order by the <code>Index</code> field and match the order of the documents in the input list. If there are no errors in the batch, the <code>ErrorList</code> is empty.<br/>ResultList:  A list of objects containing the results of the operation. The results are sorted in ascending order by the <code>Index</code> field and match the order of the documents in the input list. If all of the documents contain an error, the <code>ResultList</code> is empty.<br/>        """
        pass

    def batch_detect_sentiment(self, LanguageCode: str, TextList: List[String]) -> BatchDetectSentimentResponse:
        """Inspects a batch of documents and returns an inference of the prevailing sentiment, <code>POSITIVE</code>, <code>NEUTRAL</code>, <code>MIXED</code>, or <code>NEGATIVE</code>, in each one.

        :param self:
        :param LanguageCode:  The language of the input documents. You can specify any of the primary languages supported by Amazon Comprehend: German ("de"), English ("en"), Spanish ("es"), French ("fr"), Italian ("it"), or Portuguese ("pt"). All documents must be in the same language.<br/>TextList:  A list containing the text of the input documents. The list can contain a maximum of 25 documents. Each document must contain fewer that 5,000 bytes of UTF-8 encoded characters.<br/>
        :return: ErrorList:  A list containing one object for each document that contained an error. The results are sorted in ascending order by the <code>Index</code> field and match the order of the documents in the input list. If there are no errors in the batch, the <code>ErrorList</code> is empty.<br/>ResultList:  A list of objects containing the results of the operation. The results are sorted in ascending order by the <code>Index</code> field and match the order of the documents in the input list. If all of the documents contain an error, the <code>ResultList</code> is empty.<br/>        """
        pass

    def batch_detect_syntax(self, LanguageCode: str, TextList: List[String]) -> BatchDetectSyntaxResponse:
        """Inspects the text of a batch of documents for the syntax and part of speech of the words in the document and returns information about them. For more information, see <a>how-syntax</a>.

        :param self:
        :param LanguageCode:  The language of the input documents. You can specify any of the primary languages supported by Amazon Comprehend: German ("de"), English ("en"), Spanish ("es"), French ("fr"), Italian ("it"), or Portuguese ("pt"). All documents must be in the same language.<br/>TextList:  A list containing the text of the input documents. The list can contain a maximum of 25 documents. Each document must contain fewer that 5,000 bytes of UTF-8 encoded characters.<br/>
        :return: ErrorList:  A list containing one object for each document that contained an error. The results are sorted in ascending order by the <code>Index</code> field and match the order of the documents in the input list. If there are no errors in the batch, the <code>ErrorList</code> is empty.<br/>ResultList:  A list of objects containing the results of the operation. The results are sorted in ascending order by the <code>Index</code> field and match the order of the documents in the input list. If all of the documents contain an error, the <code>ResultList</code> is empty.<br/>        """
        pass

    def create_document_classifier(self, LanguageCode: str, InputDataConfig: DocumentClassifierInputDataConfig, DataAccessRoleArn: str, DocumentClassifierName: str, Tags: List[Tag]=None, OutputDataConfig: DocumentClassifierOutputDataConfig=None, ClientRequestToken: str=None, VolumeKmsKeyId: str=None, VpcConfig: VpcConfig=None) -> CreateDocumentClassifierResponse:
        """Creates a new document classifier that you can use to categorize documents. To create a classifier you provide a set of training documents that labeled with the categories that you want to use. After the classifier is trained you can use it to categorize a set of labeled documents into the categories. For more information, see <a>how-document-classification</a>.

        :param self:
        :param VpcConfig:  Configuration parameters for an optional private Virtual Private Cloud (VPC) containing the resources you are using for your custom classifier. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/what-is-amazon-vpc.html">Amazon VPC</a>. :<br/>VolumeKmsKeyId:  ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt data on the storage volume attached to the ML compute instance(s) that process the analysis job. The VolumeKmsKeyId can be either of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> </ul><br/>LanguageCode:  The language of the input documents. You can specify any of the primary languages supported by Amazon Comprehend: German ("de"), English ("en"), Spanish ("es"), French ("fr"), Italian ("it"), or Portuguese ("pt"). All documents must be in the same language.<br/>ClientRequestToken:  A unique identifier for the request. If you don't set the client request token, Amazon Comprehend generates one.<br/>OutputDataConfig:  Enables the addition of output results configuration parameters for custom classifier jobs.:<br/>InputDataConfig:  Specifies the format and location of the input data for the job.:<br/>Tags:  Tags to be associated with the document classifier being created. A tag is a key-value pair that adds as a metadata to a resource used by Amazon Comprehend. For example, a tag with "Sales" as the key might be added to a resource to indicate its use by the sales department. <br/>DataAccessRoleArn:  The Amazon Resource Name (ARN) of the AWS Identity and Management (IAM) role that grants Amazon Comprehend read access to your input data.<br/>DocumentClassifierName:  The name of the document classifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> The Amazon S3 URI for the input data. The S3 bucket must be in the same region as the API endpoint that you are calling. The URI can point to a single input file or it can provide the prefix for a collection of input files. For example, if you use the URI <code>S3://bucketName/prefix</code>, if the prefix is a single file, Amazon Comprehend uses that file as input. If more than one file begins with the prefix, Amazon Comprehend uses all of them as input.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt the output results from an analysis job. The KmsKeyId can be one of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> KMS Key Alias: <code>"alias/ExampleAlias"</code>  </li> <li> ARN of a KMS Key Alias: <code>"arn:aws:kms:us-west-2:111122223333:alias/ExampleAlias"</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> When you use the <code>OutputDataConfig</code> object while creating a custom classifier, you specify the Amazon S3 location where you want to write the confusion matrix. The URI must be in the same region as the API endpoint that you are calling. The location is used as the prefix for the actual location of this output file. When the custom classifier job is finished, the service creates the output file in a directory specific to the job. The <code>S3Uri</code> field contains the location of the output file, called <code>output.tar.gz</code>. It is a compressed archive that contains the confusion matrix.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> The ID for each subnet being used in your private VPC. This subnet is a subset of the a range of IPv4 addresses used by the VPC and is specific to a given availability zone in the VPC’s region. This ID number is preceded by "subnet-", for instance: "subnet-04ccf456919e69055". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Subnets.html">VPCs and Subnets</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The ID number for a security group on an instance of your private VPC. Security groups on your VPC function serve as a virtual firewall to control inbound and outbound traffic and provides security for the resources that you’ll be accessing on the VPC. This ID number is preceded by "sg-", for instance: "sg-03b388029b0a285ea". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_SecurityGroups.html">Security Groups for your VPC</a>. <br/>
        :return: DocumentClassifierArn:  The Amazon Resource Name (ARN) that identifies the document classifier.<br/>        """
        pass

    def create_entity_recognizer(self, LanguageCode: str, InputDataConfig: EntityRecognizerInputDataConfig, DataAccessRoleArn: str, RecognizerName: str, Tags: List[Tag]=None, ClientRequestToken: str=None, VolumeKmsKeyId: str=None, VpcConfig: VpcConfig=None) -> CreateEntityRecognizerResponse:
        """Creates an entity recognizer using submitted files. After your <code>CreateEntityRecognizer</code> request is submitted, you can check job status using the API. 

        :param self:
        :param VpcConfig:  Configuration parameters for an optional private Virtual Private Cloud (VPC) containing the resources you are using for your custom entity recognizer. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/what-is-amazon-vpc.html">Amazon VPC</a>. :<br/>VolumeKmsKeyId:  ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt data on the storage volume attached to the ML compute instance(s) that process the analysis job. The VolumeKmsKeyId can be either of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> </ul><br/>LanguageCode:   The language of the input documents. All documents must be in the same language. Only English ("en") is currently supported. <br/>ClientRequestToken:   A unique identifier for the request. If you don't set the client request token, Amazon Comprehend generates one.<br/>InputDataConfig:  Specifies the format and location of the input data. The S3 bucket containing the input data must be located in the same region as the entity recognizer being created. :<br/>Tags:  Tags to be associated with the entity recognizer being created. A tag is a key-value pair that adds as a metadata to a resource used by Amazon Comprehend. For example, a tag with "Sales" as the key might be added to a resource to indicate its use by the sales department. <br/>DataAccessRoleArn:  The Amazon Resource Name (ARN) of the AWS Identity and Management (IAM) role that grants Amazon Comprehend read access to your input data.<br/>RecognizerName:  The name given to the newly created recognizer. Recognizer names can be a maximum of 256 characters. Alphanumeric characters, hyphens (-) and underscores (_) are allowed. The name must be unique in the account/region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EntityList</i> S3 location of the entity list for an entity recognizer.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EntityList</b>: S3 location of the entity list for an entity recognizer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Annotations</i> S3 location of the annotations file for an entity recognizer.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Annotations</b>: S3 location of the annotations file for an entity recognizer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Documents</i> S3 location of the documents folder for an entity recognizer&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Documents</b>: S3 location of the documents folder for an entity recognizer<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EntityTypes</i> The entity types in the input data for an entity recognizer. A maximum of 12 entity types can be used at one time to train an entity recognizer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i>  Specifies the Amazon S3 location where the training documents for an entity recognizer are located. The URI must be in the same region as the API endpoint that you are calling.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i>  Specifies the Amazon S3 location where the annotations for an entity recognizer are located. The URI must be in the same region as the API endpoint that you are calling.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> Specifies the Amazon S3 location where the entity list is located. The URI must be in the same region as the API endpoint that you are calling.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> The ID for each subnet being used in your private VPC. This subnet is a subset of the a range of IPv4 addresses used by the VPC and is specific to a given availability zone in the VPC’s region. This ID number is preceded by "subnet-", for instance: "subnet-04ccf456919e69055". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Subnets.html">VPCs and Subnets</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The ID number for a security group on an instance of your private VPC. Security groups on your VPC function serve as a virtual firewall to control inbound and outbound traffic and provides security for the resources that you’ll be accessing on the VPC. This ID number is preceded by "sg-", for instance: "sg-03b388029b0a285ea". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_SecurityGroups.html">Security Groups for your VPC</a>. <br/>
        :return: EntityRecognizerArn:  The Amazon Resource Name (ARN) that identifies the entity recognizer.<br/>        """
        pass

    def delete_document_classifier(self, DocumentClassifierArn: str) -> DeleteDocumentClassifierResponse:
        """Deletes a previously created document classifier Only those classifiers that are in terminated states (IN_ERROR, TRAINED) will be deleted. If an active inference job is using the model, a <code>ResourceInUseException</code> will be returned. This is an asynchronous action that puts the classifier into a DELETING state, and it is then removed by a background job. Once removed, the classifier disappears from your account and is no longer available for use. 

        :param self:
        :param DocumentClassifierArn:  The Amazon Resource Name (ARN) that identifies the document classifier. <br/>
        :return:         """
        pass

    def delete_entity_recognizer(self, EntityRecognizerArn: str) -> DeleteEntityRecognizerResponse:
        """Deletes an entity recognizer. Only those recognizers that are in terminated states (IN_ERROR, TRAINED) will be deleted. If an active inference job is using the model, a <code>ResourceInUseException</code> will be returned. This is an asynchronous action that puts the recognizer into a DELETING state, and it is then removed by a background job. Once removed, the recognizer disappears from your account and is no longer available for use. 

        :param self:
        :param EntityRecognizerArn:  The Amazon Resource Name (ARN) that identifies the entity recognizer.<br/>
        :return:         """
        pass

    def describe_document_classification_job(self, JobId: str) -> DescribeDocumentClassificationJobResponse:
        """Gets the properties associated with a document classification job. Use this operation to get the status of a classification job.

        :param self:
        :param JobId:  The identifier that Amazon Comprehend generated for the job. The operation returns this identifier in its response.<br/>
        :return: DocumentClassificationJobProperties:  An object that describes the properties associated with the document classification job.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcConfig</i>  Configuration parameters for a private Virtual Private Cloud (VPC) containing the resources you are using for your document classification job. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/what-is-amazon-vpc.html">Amazon VPC</a>. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VpcConfig</b>:  Configuration parameters for a private Virtual Private Cloud (VPC) containing the resources you are using for your document classification job. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/what-is-amazon-vpc.html">Amazon VPC</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeKmsKeyId</i> ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt data on the storage volume attached to the ML compute instance(s) that process the analysis job. The VolumeKmsKeyId can be either of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataAccessRoleArn</i> The Amazon Resource Name (ARN) of the AWS identity and Access Management (IAM) role that grants Amazon Comprehend read access to your input data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputDataConfig</i> The output data configuration that you supplied when you created the document classification job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>OutputDataConfig</b>: The output data configuration that you supplied when you created the document classification job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputDataConfig</i> The input data configuration that you supplied when you created the document classification job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputDataConfig</b>: The input data configuration that you supplied when you created the document classification job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentClassifierArn</i> The Amazon Resource Name (ARN) that identifies the document classifier. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> The time that the document classification job completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmitTime</i> The time that the document classification job was submitted for processing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> A description of the status of the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobStatus</i> The current status of the document classification job. If the status is <code>FAILED</code>, the <code>Message</code> field shows the reason for the failure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobName</i> The name that you assigned to the document classification job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobId</i> The identifier assigned to the document classification job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InputFormat</i> Specifies how the text in an input file should be processed: <ul> <li>  <code>ONE_DOC_PER_FILE</code> - Each file is considered a separate document. Use this option when you are processing large documents, such as newspaper articles or scientific papers. </li> <li>  <code>ONE_DOC_PER_LINE</code> - Each line in a file is considered a separate document. Use this option when you are processing many short documents, such as text messages. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> The Amazon S3 URI for the input data. The URI must be in same region as the API endpoint that you are calling. The URI can point to a single input file or it can provide the prefix for a collection of data files.  For example, if you use the URI <code>S3://bucketName/prefix</code>, if the prefix is a single file, Amazon Comprehend uses that file as input. If more than one file begins with the prefix, Amazon Comprehend uses all of them as input.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt the output results from an analysis job. The KmsKeyId can be one of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> KMS Key Alias: <code>"alias/ExampleAlias"</code>  </li> <li> ARN of a KMS Key Alias: <code>"arn:aws:kms:us-west-2:111122223333:alias/ExampleAlias"</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> When you use the <code>OutputDataConfig</code> object with asynchronous operations, you specify the Amazon S3 location where you want to write the output data. The URI must be in the same region as the API endpoint that you are calling. The location is used as the prefix for the actual location of the output file. When the topic detection job is finished, the service creates an output file in a directory specific to the job. The <code>S3Uri</code> field contains the location of the output file, called <code>output.tar.gz</code>. It is a compressed archive that contains the ouput of the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> The ID for each subnet being used in your private VPC. This subnet is a subset of the a range of IPv4 addresses used by the VPC and is specific to a given availability zone in the VPC’s region. This ID number is preceded by "subnet-", for instance: "subnet-04ccf456919e69055". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Subnets.html">VPCs and Subnets</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The ID number for a security group on an instance of your private VPC. Security groups on your VPC function serve as a virtual firewall to control inbound and outbound traffic and provides security for the resources that you’ll be accessing on the VPC. This ID number is preceded by "sg-", for instance: "sg-03b388029b0a285ea". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_SecurityGroups.html">Security Groups for your VPC</a>. <br/>        """
        pass

    def describe_document_classifier(self, DocumentClassifierArn: str) -> DescribeDocumentClassifierResponse:
        """Gets the properties associated with a document classifier.

        :param self:
        :param DocumentClassifierArn:  The Amazon Resource Name (ARN) that identifies the document classifier. The operation returns this identifier in its response.<br/>
        :return: DocumentClassifierProperties:  An object that contains the properties associated with a document classifier.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcConfig</i>  Configuration parameters for a private Virtual Private Cloud (VPC) containing the resources you are using for your custom classifier. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/what-is-amazon-vpc.html">Amazon VPC</a>. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VpcConfig</b>:  Configuration parameters for a private Virtual Private Cloud (VPC) containing the resources you are using for your custom classifier. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/what-is-amazon-vpc.html">Amazon VPC</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeKmsKeyId</i> ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt data on the storage volume attached to the ML compute instance(s) that process the analysis job. The VolumeKmsKeyId can be either of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataAccessRoleArn</i> The Amazon Resource Name (ARN) of the AWS Identity and Management (IAM) role that grants Amazon Comprehend read access to your input data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClassifierMetadata</i> Information about the document classifier, including the number of documents used for training the classifier, the number of documents used for test the classifier, and an accuracy rating.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ClassifierMetadata</b>: Information about the document classifier, including the number of documents used for training the classifier, the number of documents used for test the classifier, and an accuracy rating.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputDataConfig</i>  Provides output results configuration parameters for custom classifier jobs.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>OutputDataConfig</b>:  Provides output results configuration parameters for custom classifier jobs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputDataConfig</i> The input data configuration that you supplied when you created the document classifier for training.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputDataConfig</b>: The input data configuration that you supplied when you created the document classifier for training.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingEndTime</i> The time that training of the document classifier was completed. Indicates the time when the training completes on documentation classifiers. You are billed for the time interval between this time and the value of TrainingStartTime.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingStartTime</i> Indicates the time when the training starts on documentation classifiers. You are billed for the time interval between this time and the value of TrainingEndTime. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> The time that training the document classifier completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmitTime</i> The time that the document classifier was submitted for training.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> Additional information about the status of the classifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the document classifier. If the status is <code>TRAINED</code> the classifier is ready to use. If the status is <code>FAILED</code> you can see additional information about why the classifier wasn't trained in the <code>Message</code> field.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LanguageCode</i> The language code for the language of the documents that the classifier was trained on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentClassifierArn</i> The Amazon Resource Name (ARN) that identifies the document classifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> The Amazon S3 URI for the input data. The S3 bucket must be in the same region as the API endpoint that you are calling. The URI can point to a single input file or it can provide the prefix for a collection of input files. For example, if you use the URI <code>S3://bucketName/prefix</code>, if the prefix is a single file, Amazon Comprehend uses that file as input. If more than one file begins with the prefix, Amazon Comprehend uses all of them as input.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt the output results from an analysis job. The KmsKeyId can be one of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> KMS Key Alias: <code>"alias/ExampleAlias"</code>  </li> <li> ARN of a KMS Key Alias: <code>"arn:aws:kms:us-west-2:111122223333:alias/ExampleAlias"</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> When you use the <code>OutputDataConfig</code> object while creating a custom classifier, you specify the Amazon S3 location where you want to write the confusion matrix. The URI must be in the same region as the API endpoint that you are calling. The location is used as the prefix for the actual location of this output file. When the custom classifier job is finished, the service creates the output file in a directory specific to the job. The <code>S3Uri</code> field contains the location of the output file, called <code>output.tar.gz</code>. It is a compressed archive that contains the confusion matrix.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfTestDocuments</i> The number of documents in the input data that were used to test the classifier. Typically this is 10 to 20 percent of the input documents.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfTrainedDocuments</i> The number of documents in the input data that were used to train the classifier. Typically this is 80 to 90 percent of the input documents.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfLabels</i> The number of labels in the input data. <br/> Describes the result metrics for the test data associated with an documentation classifier.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EvaluationMetrics</b>:  Describes the result metrics for the test data associated with an documentation classifier.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> The ID for each subnet being used in your private VPC. This subnet is a subset of the a range of IPv4 addresses used by the VPC and is specific to a given availability zone in the VPC’s region. This ID number is preceded by "subnet-", for instance: "subnet-04ccf456919e69055". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Subnets.html">VPCs and Subnets</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The ID number for a security group on an instance of your private VPC. Security groups on your VPC function serve as a virtual firewall to control inbound and outbound traffic and provides security for the resources that you’ll be accessing on the VPC. This ID number is preceded by "sg-", for instance: "sg-03b388029b0a285ea". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_SecurityGroups.html">Security Groups for your VPC</a>. <br/>        """
        pass

    def describe_dominant_language_detection_job(self, JobId: str) -> DescribeDominantLanguageDetectionJobResponse:
        """Gets the properties associated with a dominant language detection job. Use this operation to get the status of a detection job.

        :param self:
        :param JobId:  The identifier that Amazon Comprehend generated for the job. The operation returns this identifier in its response.<br/>
        :return: DominantLanguageDetectionJobProperties:  An object that contains the properties associated with a dominant language detection job.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcConfig</i>  Configuration parameters for a private Virtual Private Cloud (VPC) containing the resources you are using for your dominant language detection job. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/what-is-amazon-vpc.html">Amazon VPC</a>. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VpcConfig</b>:  Configuration parameters for a private Virtual Private Cloud (VPC) containing the resources you are using for your dominant language detection job. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/what-is-amazon-vpc.html">Amazon VPC</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeKmsKeyId</i> ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt data on the storage volume attached to the ML compute instance(s) that process the analysis job. The VolumeKmsKeyId can be either of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataAccessRoleArn</i> The Amazon Resource Name (ARN) that gives Amazon Comprehend read access to your input data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputDataConfig</i> The output data configuration that you supplied when you created the dominant language detection job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>OutputDataConfig</b>: The output data configuration that you supplied when you created the dominant language detection job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputDataConfig</i> The input data configuration that you supplied when you created the dominant language detection job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputDataConfig</b>: The input data configuration that you supplied when you created the dominant language detection job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> The time that the dominant language detection job completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmitTime</i> The time that the dominant language detection job was submitted for processing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> A description for the status of a job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobStatus</i> The current status of the dominant language detection job. If the status is <code>FAILED</code>, the <code>Message</code> field shows the reason for the failure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobName</i> The name that you assigned to the dominant language detection job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobId</i> The identifier assigned to the dominant language detection job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InputFormat</i> Specifies how the text in an input file should be processed: <ul> <li>  <code>ONE_DOC_PER_FILE</code> - Each file is considered a separate document. Use this option when you are processing large documents, such as newspaper articles or scientific papers. </li> <li>  <code>ONE_DOC_PER_LINE</code> - Each line in a file is considered a separate document. Use this option when you are processing many short documents, such as text messages. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> The Amazon S3 URI for the input data. The URI must be in same region as the API endpoint that you are calling. The URI can point to a single input file or it can provide the prefix for a collection of data files.  For example, if you use the URI <code>S3://bucketName/prefix</code>, if the prefix is a single file, Amazon Comprehend uses that file as input. If more than one file begins with the prefix, Amazon Comprehend uses all of them as input.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt the output results from an analysis job. The KmsKeyId can be one of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> KMS Key Alias: <code>"alias/ExampleAlias"</code>  </li> <li> ARN of a KMS Key Alias: <code>"arn:aws:kms:us-west-2:111122223333:alias/ExampleAlias"</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> When you use the <code>OutputDataConfig</code> object with asynchronous operations, you specify the Amazon S3 location where you want to write the output data. The URI must be in the same region as the API endpoint that you are calling. The location is used as the prefix for the actual location of the output file. When the topic detection job is finished, the service creates an output file in a directory specific to the job. The <code>S3Uri</code> field contains the location of the output file, called <code>output.tar.gz</code>. It is a compressed archive that contains the ouput of the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> The ID for each subnet being used in your private VPC. This subnet is a subset of the a range of IPv4 addresses used by the VPC and is specific to a given availability zone in the VPC’s region. This ID number is preceded by "subnet-", for instance: "subnet-04ccf456919e69055". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Subnets.html">VPCs and Subnets</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The ID number for a security group on an instance of your private VPC. Security groups on your VPC function serve as a virtual firewall to control inbound and outbound traffic and provides security for the resources that you’ll be accessing on the VPC. This ID number is preceded by "sg-", for instance: "sg-03b388029b0a285ea". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_SecurityGroups.html">Security Groups for your VPC</a>. <br/>        """
        pass

    def describe_entities_detection_job(self, JobId: str) -> DescribeEntitiesDetectionJobResponse:
        """Gets the properties associated with an entities detection job. Use this operation to get the status of a detection job.

        :param self:
        :param JobId:  The identifier that Amazon Comprehend generated for the job. The operation returns this identifier in its response.<br/>
        :return: EntitiesDetectionJobProperties:  An object that contains the properties associated with an entities detection job.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcConfig</i>  Configuration parameters for a private Virtual Private Cloud (VPC) containing the resources you are using for your entity detection job. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/what-is-amazon-vpc.html">Amazon VPC</a>. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VpcConfig</b>:  Configuration parameters for a private Virtual Private Cloud (VPC) containing the resources you are using for your entity detection job. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/what-is-amazon-vpc.html">Amazon VPC</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeKmsKeyId</i> ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt data on the storage volume attached to the ML compute instance(s) that process the analysis job. The VolumeKmsKeyId can be either of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataAccessRoleArn</i> The Amazon Resource Name (ARN) that gives Amazon Comprehend read access to your input data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LanguageCode</i> The language code of the input documents.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputDataConfig</i> The output data configuration that you supplied when you created the entities detection job. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>OutputDataConfig</b>: The output data configuration that you supplied when you created the entities detection job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputDataConfig</i> The input data configuration that you supplied when you created the entities detection job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputDataConfig</b>: The input data configuration that you supplied when you created the entities detection job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EntityRecognizerArn</i> The Amazon Resource Name (ARN) that identifies the entity recognizer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> The time that the entities detection job completed<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmitTime</i> The time that the entities detection job was submitted for processing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> A description of the status of a job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobStatus</i> The current status of the entities detection job. If the status is <code>FAILED</code>, the <code>Message</code> field shows the reason for the failure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobName</i> The name that you assigned the entities detection job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobId</i> The identifier assigned to the entities detection job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InputFormat</i> Specifies how the text in an input file should be processed: <ul> <li>  <code>ONE_DOC_PER_FILE</code> - Each file is considered a separate document. Use this option when you are processing large documents, such as newspaper articles or scientific papers. </li> <li>  <code>ONE_DOC_PER_LINE</code> - Each line in a file is considered a separate document. Use this option when you are processing many short documents, such as text messages. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> The Amazon S3 URI for the input data. The URI must be in same region as the API endpoint that you are calling. The URI can point to a single input file or it can provide the prefix for a collection of data files.  For example, if you use the URI <code>S3://bucketName/prefix</code>, if the prefix is a single file, Amazon Comprehend uses that file as input. If more than one file begins with the prefix, Amazon Comprehend uses all of them as input.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt the output results from an analysis job. The KmsKeyId can be one of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> KMS Key Alias: <code>"alias/ExampleAlias"</code>  </li> <li> ARN of a KMS Key Alias: <code>"arn:aws:kms:us-west-2:111122223333:alias/ExampleAlias"</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> When you use the <code>OutputDataConfig</code> object with asynchronous operations, you specify the Amazon S3 location where you want to write the output data. The URI must be in the same region as the API endpoint that you are calling. The location is used as the prefix for the actual location of the output file. When the topic detection job is finished, the service creates an output file in a directory specific to the job. The <code>S3Uri</code> field contains the location of the output file, called <code>output.tar.gz</code>. It is a compressed archive that contains the ouput of the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> The ID for each subnet being used in your private VPC. This subnet is a subset of the a range of IPv4 addresses used by the VPC and is specific to a given availability zone in the VPC’s region. This ID number is preceded by "subnet-", for instance: "subnet-04ccf456919e69055". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Subnets.html">VPCs and Subnets</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The ID number for a security group on an instance of your private VPC. Security groups on your VPC function serve as a virtual firewall to control inbound and outbound traffic and provides security for the resources that you’ll be accessing on the VPC. This ID number is preceded by "sg-", for instance: "sg-03b388029b0a285ea". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_SecurityGroups.html">Security Groups for your VPC</a>. <br/>        """
        pass

    def describe_entity_recognizer(self, EntityRecognizerArn: str) -> DescribeEntityRecognizerResponse:
        """Provides details about an entity recognizer including status, S3 buckets containing training data, recognizer metadata, metrics, and so on.

        :param self:
        :param EntityRecognizerArn:  The Amazon Resource Name (ARN) that identifies the entity recognizer.<br/>
        :return: EntityRecognizerProperties:  Describes information associated with an entity recognizer.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcConfig</i>  Configuration parameters for a private Virtual Private Cloud (VPC) containing the resources you are using for your custom entity recognizer. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/what-is-amazon-vpc.html">Amazon VPC</a>. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VpcConfig</b>:  Configuration parameters for a private Virtual Private Cloud (VPC) containing the resources you are using for your custom entity recognizer. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/what-is-amazon-vpc.html">Amazon VPC</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeKmsKeyId</i> ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt data on the storage volume attached to the ML compute instance(s) that process the analysis job. The VolumeKmsKeyId can be either of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataAccessRoleArn</i>  The Amazon Resource Name (ARN) of the AWS Identity and Management (IAM) role that grants Amazon Comprehend read access to your input data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecognizerMetadata</i>  Provides information about an entity recognizer.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RecognizerMetadata</b>:  Provides information about an entity recognizer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputDataConfig</i> The input data properties of an entity recognizer.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputDataConfig</b>: The input data properties of an entity recognizer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingEndTime</i> The time that training of the entity recognizer was completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingStartTime</i> The time that training of the entity recognizer started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> The time that the recognizer creation completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmitTime</i> The time that the recognizer was submitted for processing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i>  A description of the status of the recognizer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Provides the status of the entity recognizer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LanguageCode</i>  The language of the input documents. All documents must be in the same language. Only English ("en") is currently supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EntityRecognizerArn</i> The Amazon Resource Name (ARN) that identifies the entity recognizer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EntityTypes</i> The entity types in the input data for an entity recognizer. A maximum of 12 entity types can be used at one time to train an entity recognizer.<br/>S3 location of the documents folder for an entity recognizer&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Documents</b>: S3 location of the documents folder for an entity recognizer&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfTestDocuments</i>  The number of documents in the input data that were used to test the entity recognizer. Typically this is 10 to 20 percent of the input documents.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfTrainedDocuments</i>  The number of documents in the input data that were used to train the entity recognizer. Typically this is 80 to 90 percent of the input documents.<br/>Detailed information about the accuracy of an entity recognizer.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EvaluationMetrics</b>: Detailed information about the accuracy of an entity recognizer.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> The ID for each subnet being used in your private VPC. This subnet is a subset of the a range of IPv4 addresses used by the VPC and is specific to a given availability zone in the VPC’s region. This ID number is preceded by "subnet-", for instance: "subnet-04ccf456919e69055". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Subnets.html">VPCs and Subnets</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The ID number for a security group on an instance of your private VPC. Security groups on your VPC function serve as a virtual firewall to control inbound and outbound traffic and provides security for the resources that you’ll be accessing on the VPC. This ID number is preceded by "sg-", for instance: "sg-03b388029b0a285ea". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_SecurityGroups.html">Security Groups for your VPC</a>. <br/>        """
        pass

    def describe_key_phrases_detection_job(self, JobId: str) -> DescribeKeyPhrasesDetectionJobResponse:
        """Gets the properties associated with a key phrases detection job. Use this operation to get the status of a detection job.

        :param self:
        :param JobId:  The identifier that Amazon Comprehend generated for the job. The operation returns this identifier in its response.<br/>
        :return: KeyPhrasesDetectionJobProperties:  An object that contains the properties associated with a key phrases detection job. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcConfig</i>  Configuration parameters for a private Virtual Private Cloud (VPC) containing the resources you are using for your key phrases detection job. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/what-is-amazon-vpc.html">Amazon VPC</a>. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VpcConfig</b>:  Configuration parameters for a private Virtual Private Cloud (VPC) containing the resources you are using for your key phrases detection job. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/what-is-amazon-vpc.html">Amazon VPC</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeKmsKeyId</i> ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt data on the storage volume attached to the ML compute instance(s) that process the analysis job. The VolumeKmsKeyId can be either of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataAccessRoleArn</i> The Amazon Resource Name (ARN) that gives Amazon Comprehend read access to your input data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LanguageCode</i> The language code of the input documents.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputDataConfig</i> The output data configuration that you supplied when you created the key phrases detection job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>OutputDataConfig</b>: The output data configuration that you supplied when you created the key phrases detection job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputDataConfig</i> The input data configuration that you supplied when you created the key phrases detection job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputDataConfig</b>: The input data configuration that you supplied when you created the key phrases detection job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> The time that the key phrases detection job completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmitTime</i> The time that the key phrases detection job was submitted for processing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> A description of the status of a job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobStatus</i> The current status of the key phrases detection job. If the status is <code>FAILED</code>, the <code>Message</code> field shows the reason for the failure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobName</i> The name that you assigned the key phrases detection job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobId</i> The identifier assigned to the key phrases detection job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InputFormat</i> Specifies how the text in an input file should be processed: <ul> <li>  <code>ONE_DOC_PER_FILE</code> - Each file is considered a separate document. Use this option when you are processing large documents, such as newspaper articles or scientific papers. </li> <li>  <code>ONE_DOC_PER_LINE</code> - Each line in a file is considered a separate document. Use this option when you are processing many short documents, such as text messages. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> The Amazon S3 URI for the input data. The URI must be in same region as the API endpoint that you are calling. The URI can point to a single input file or it can provide the prefix for a collection of data files.  For example, if you use the URI <code>S3://bucketName/prefix</code>, if the prefix is a single file, Amazon Comprehend uses that file as input. If more than one file begins with the prefix, Amazon Comprehend uses all of them as input.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt the output results from an analysis job. The KmsKeyId can be one of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> KMS Key Alias: <code>"alias/ExampleAlias"</code>  </li> <li> ARN of a KMS Key Alias: <code>"arn:aws:kms:us-west-2:111122223333:alias/ExampleAlias"</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> When you use the <code>OutputDataConfig</code> object with asynchronous operations, you specify the Amazon S3 location where you want to write the output data. The URI must be in the same region as the API endpoint that you are calling. The location is used as the prefix for the actual location of the output file. When the topic detection job is finished, the service creates an output file in a directory specific to the job. The <code>S3Uri</code> field contains the location of the output file, called <code>output.tar.gz</code>. It is a compressed archive that contains the ouput of the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> The ID for each subnet being used in your private VPC. This subnet is a subset of the a range of IPv4 addresses used by the VPC and is specific to a given availability zone in the VPC’s region. This ID number is preceded by "subnet-", for instance: "subnet-04ccf456919e69055". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Subnets.html">VPCs and Subnets</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The ID number for a security group on an instance of your private VPC. Security groups on your VPC function serve as a virtual firewall to control inbound and outbound traffic and provides security for the resources that you’ll be accessing on the VPC. This ID number is preceded by "sg-", for instance: "sg-03b388029b0a285ea". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_SecurityGroups.html">Security Groups for your VPC</a>. <br/>        """
        pass

    def describe_sentiment_detection_job(self, JobId: str) -> DescribeSentimentDetectionJobResponse:
        """Gets the properties associated with a sentiment detection job. Use this operation to get the status of a detection job.

        :param self:
        :param JobId:  The identifier that Amazon Comprehend generated for the job. The operation returns this identifier in its response.<br/>
        :return: SentimentDetectionJobProperties:  An object that contains the properties associated with a sentiment detection job.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcConfig</i>  Configuration parameters for a private Virtual Private Cloud (VPC) containing the resources you are using for your sentiment detection job. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/what-is-amazon-vpc.html">Amazon VPC</a>. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VpcConfig</b>:  Configuration parameters for a private Virtual Private Cloud (VPC) containing the resources you are using for your sentiment detection job. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/what-is-amazon-vpc.html">Amazon VPC</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeKmsKeyId</i> ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt data on the storage volume attached to the ML compute instance(s) that process the analysis job. The VolumeKmsKeyId can be either of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataAccessRoleArn</i> The Amazon Resource Name (ARN) that gives Amazon Comprehend read access to your input data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LanguageCode</i> The language code of the input documents.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputDataConfig</i> The output data configuration that you supplied when you created the sentiment detection job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>OutputDataConfig</b>: The output data configuration that you supplied when you created the sentiment detection job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputDataConfig</i> The input data configuration that you supplied when you created the sentiment detection job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputDataConfig</b>: The input data configuration that you supplied when you created the sentiment detection job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> The time that the sentiment detection job ended.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmitTime</i> The time that the sentiment detection job was submitted for processing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> A description of the status of a job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobStatus</i> The current status of the sentiment detection job. If the status is <code>FAILED</code>, the <code>Messages</code> field shows the reason for the failure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobName</i> The name that you assigned to the sentiment detection job<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobId</i> The identifier assigned to the sentiment detection job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InputFormat</i> Specifies how the text in an input file should be processed: <ul> <li>  <code>ONE_DOC_PER_FILE</code> - Each file is considered a separate document. Use this option when you are processing large documents, such as newspaper articles or scientific papers. </li> <li>  <code>ONE_DOC_PER_LINE</code> - Each line in a file is considered a separate document. Use this option when you are processing many short documents, such as text messages. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> The Amazon S3 URI for the input data. The URI must be in same region as the API endpoint that you are calling. The URI can point to a single input file or it can provide the prefix for a collection of data files.  For example, if you use the URI <code>S3://bucketName/prefix</code>, if the prefix is a single file, Amazon Comprehend uses that file as input. If more than one file begins with the prefix, Amazon Comprehend uses all of them as input.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt the output results from an analysis job. The KmsKeyId can be one of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> KMS Key Alias: <code>"alias/ExampleAlias"</code>  </li> <li> ARN of a KMS Key Alias: <code>"arn:aws:kms:us-west-2:111122223333:alias/ExampleAlias"</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> When you use the <code>OutputDataConfig</code> object with asynchronous operations, you specify the Amazon S3 location where you want to write the output data. The URI must be in the same region as the API endpoint that you are calling. The location is used as the prefix for the actual location of the output file. When the topic detection job is finished, the service creates an output file in a directory specific to the job. The <code>S3Uri</code> field contains the location of the output file, called <code>output.tar.gz</code>. It is a compressed archive that contains the ouput of the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> The ID for each subnet being used in your private VPC. This subnet is a subset of the a range of IPv4 addresses used by the VPC and is specific to a given availability zone in the VPC’s region. This ID number is preceded by "subnet-", for instance: "subnet-04ccf456919e69055". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Subnets.html">VPCs and Subnets</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The ID number for a security group on an instance of your private VPC. Security groups on your VPC function serve as a virtual firewall to control inbound and outbound traffic and provides security for the resources that you’ll be accessing on the VPC. This ID number is preceded by "sg-", for instance: "sg-03b388029b0a285ea". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_SecurityGroups.html">Security Groups for your VPC</a>. <br/>        """
        pass

    def describe_topics_detection_job(self, JobId: str) -> DescribeTopicsDetectionJobResponse:
        """Gets the properties associated with a topic detection job. Use this operation to get the status of a detection job.

        :param self:
        :param JobId:  The identifier assigned by the user to the detection job.<br/>
        :return: TopicsDetectionJobProperties:  The list of properties for the requested job.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcConfig</i> Configuration parameters for a private Virtual Private Cloud (VPC) containing the resources you are using for your topic detection job. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/what-is-amazon-vpc.html">Amazon VPC</a>. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VpcConfig</b>: Configuration parameters for a private Virtual Private Cloud (VPC) containing the resources you are using for your topic detection job. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/what-is-amazon-vpc.html">Amazon VPC</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeKmsKeyId</i> ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt data on the storage volume attached to the ML compute instance(s) that process the analysis job. The VolumeKmsKeyId can be either of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataAccessRoleArn</i> The Amazon Resource Name (ARN) of the AWS Identity and Management (IAM) role that grants Amazon Comprehend read access to your job data. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfTopics</i> The number of topics to detect supplied when you created the topic detection job. The default is 10. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputDataConfig</i> The output data configuration supplied when you created the topic detection job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>OutputDataConfig</b>: The output data configuration supplied when you created the topic detection job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputDataConfig</i> The input data configuration supplied when you created the topic detection job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputDataConfig</b>: The input data configuration supplied when you created the topic detection job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> The time that the topic detection job was completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmitTime</i> The time that the topic detection job was submitted for processing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> A description for the status of a job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobStatus</i> The current status of the topic detection job. If the status is <code>Failed</code>, the reason for the failure is shown in the <code>Message</code> field.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobName</i> The name of the topic detection job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobId</i> The identifier assigned to the topic detection job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InputFormat</i> Specifies how the text in an input file should be processed: <ul> <li>  <code>ONE_DOC_PER_FILE</code> - Each file is considered a separate document. Use this option when you are processing large documents, such as newspaper articles or scientific papers. </li> <li>  <code>ONE_DOC_PER_LINE</code> - Each line in a file is considered a separate document. Use this option when you are processing many short documents, such as text messages. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> The Amazon S3 URI for the input data. The URI must be in same region as the API endpoint that you are calling. The URI can point to a single input file or it can provide the prefix for a collection of data files.  For example, if you use the URI <code>S3://bucketName/prefix</code>, if the prefix is a single file, Amazon Comprehend uses that file as input. If more than one file begins with the prefix, Amazon Comprehend uses all of them as input.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt the output results from an analysis job. The KmsKeyId can be one of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> KMS Key Alias: <code>"alias/ExampleAlias"</code>  </li> <li> ARN of a KMS Key Alias: <code>"arn:aws:kms:us-west-2:111122223333:alias/ExampleAlias"</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> When you use the <code>OutputDataConfig</code> object with asynchronous operations, you specify the Amazon S3 location where you want to write the output data. The URI must be in the same region as the API endpoint that you are calling. The location is used as the prefix for the actual location of the output file. When the topic detection job is finished, the service creates an output file in a directory specific to the job. The <code>S3Uri</code> field contains the location of the output file, called <code>output.tar.gz</code>. It is a compressed archive that contains the ouput of the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> The ID for each subnet being used in your private VPC. This subnet is a subset of the a range of IPv4 addresses used by the VPC and is specific to a given availability zone in the VPC’s region. This ID number is preceded by "subnet-", for instance: "subnet-04ccf456919e69055". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Subnets.html">VPCs and Subnets</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The ID number for a security group on an instance of your private VPC. Security groups on your VPC function serve as a virtual firewall to control inbound and outbound traffic and provides security for the resources that you’ll be accessing on the VPC. This ID number is preceded by "sg-", for instance: "sg-03b388029b0a285ea". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_SecurityGroups.html">Security Groups for your VPC</a>. <br/>        """
        pass

    def detect_dominant_language(self, Text: str) -> DetectDominantLanguageResponse:
        """Determines the dominant language of the input text. For a list of languages that Amazon Comprehend can detect, see <a href="https://docs.aws.amazon.com/comprehend/latest/dg/how-languages.html">Amazon Comprehend Supported Languages</a>. 

        :param self:
        :param Text:  A UTF-8 text string. Each string should contain at least 20 characters and must contain fewer that 5,000 bytes of UTF-8 encoded characters.<br/>
        :return: Languages:  The languages that Amazon Comprehend detected in the input text. For each language, the response returns the RFC 5646 language code and the level of confidence that Amazon Comprehend has in the accuracy of its inference. For more information about RFC 5646, see <a href="https://tools.ietf.org/html/rfc5646">Tags for Identifying Languages</a> on the <i>IETF Tools</i> web site.<br/>        """
        pass

    def detect_entities(self, LanguageCode: str, Text: str) -> DetectEntitiesResponse:
        """Inspects text for named entities, and returns information about them. For more information, about named entities, see <a>how-entities</a>. 

        :param self:
        :param LanguageCode:  The language of the input documents. You can specify any of the primary languages supported by Amazon Comprehend: German ("de"), English ("en"), Spanish ("es"), French ("fr"), Italian ("it"), or Portuguese ("pt"). All documents must be in the same language.<br/>Text:  A UTF-8 text string. Each string must contain fewer that 5,000 bytes of UTF-8 encoded characters.<br/>
        :return: Entities:  A collection of entities identified in the input text. For each entity, the response provides the entity text, entity type, where the entity text begins and ends, and the level of confidence that Amazon Comprehend has in the detection. For a list of entity types, see <a>how-entities</a>. <br/>        """
        pass

    def detect_key_phrases(self, LanguageCode: str, Text: str) -> DetectKeyPhrasesResponse:
        """Detects the key noun phrases found in the text. 

        :param self:
        :param LanguageCode:  The language of the input documents. You can specify any of the primary languages supported by Amazon Comprehend: German ("de"), English ("en"), Spanish ("es"), French ("fr"), Italian ("it"), or Portuguese ("pt"). All documents must be in the same language.<br/>Text:  A UTF-8 text string. Each string must contain fewer that 5,000 bytes of UTF-8 encoded characters.<br/>
        :return: KeyPhrases:  A collection of key phrases that Amazon Comprehend identified in the input text. For each key phrase, the response provides the text of the key phrase, where the key phrase begins and ends, and the level of confidence that Amazon Comprehend has in the accuracy of the detection. <br/>        """
        pass

    def detect_sentiment(self, LanguageCode: str, Text: str) -> DetectSentimentResponse:
        """Inspects text and returns an inference of the prevailing sentiment (<code>POSITIVE</code>, <code>NEUTRAL</code>, <code>MIXED</code>, or <code>NEGATIVE</code>). 

        :param self:
        :param LanguageCode:  The language of the input documents. You can specify any of the primary languages supported by Amazon Comprehend: German ("de"), English ("en"), Spanish ("es"), French ("fr"), Italian ("it"), or Portuguese ("pt"). All documents must be in the same language.<br/>Text:  A UTF-8 text string. Each string must contain fewer that 5,000 bytes of UTF-8 encoded characters.<br/>
        :return: SentimentScore:  An object that lists the sentiments, and their corresponding confidence levels.:<br/>Sentiment:  The inferred sentiment that Amazon Comprehend has the highest level of confidence in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Mixed</i> The level of confidence that Amazon Comprehend has in the accuracy of its detection of the <code>MIXED</code> sentiment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Neutral</i> The level of confidence that Amazon Comprehend has in the accuracy of its detection of the <code>NEUTRAL</code> sentiment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Negative</i> The level of confidence that Amazon Comprehend has in the accuracy of its detection of the <code>NEGATIVE</code> sentiment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Positive</i> The level of confidence that Amazon Comprehend has in the accuracy of its detection of the <code>POSITIVE</code> sentiment.<br/>        """
        pass

    def detect_syntax(self, LanguageCode: str, Text: str) -> DetectSyntaxResponse:
        """Inspects text for syntax and the part of speech of words in the document. For more information, <a>how-syntax</a>.

        :param self:
        :param LanguageCode:  The language code of the input documents. You can specify any of the primary languages supported by Amazon Comprehend: German ("de"), English ("en"), Spanish ("es"), French ("fr"), Italian ("it"), or Portuguese ("pt").<br/>Text:  A UTF-8 string. Each string must contain fewer that 5,000 bytes of UTF encoded characters.<br/>
        :return: SyntaxTokens:  A collection of syntax tokens describing the text. For each token, the response provides the text, the token type, where the text begins and ends, and the level of confidence that Amazon Comprehend has that the token is correct. For a list of token types, see <a>how-syntax</a>.<br/>        """
        pass

    def list_document_classification_jobs(self, Filter: DocumentClassificationJobFilter=None, NextToken: str=None, MaxResults: int=None) -> ListDocumentClassificationJobsResponse:
        """Gets a list of the documentation classification jobs that you have submitted.

        :param self:
        :param MaxResults:  The maximum number of results to return in each page. The default is 100.<br/>NextToken:  Identifies the next page of results to return.<br/>Filter:  Filters the jobs that are returned. You can filter jobs on their names, status, or the date and time that they were submitted. You can only set one filter at a time.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmitTimeAfter</i> Filters the list of jobs based on the time that the job was submitted for processing. Returns only jobs submitted before the specified time. Jobs are returned in descending order, newest to oldest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmitTimeBefore</i> Filters the list of jobs based on the time that the job was submitted for processing. Returns only jobs submitted after the specified time. Jobs are returned in ascending order, oldest to newest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobStatus</i> Filters the list based on job status. Returns only jobs with the specified status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobName</i> Filters on the name of the job.<br/>
        :return: NextToken:  Identifies the next page of results to return.<br/>DocumentClassificationJobPropertiesList:  A list containing the properties of each job returned.<br/>        """
        pass

    def list_document_classifiers(self, Filter: DocumentClassifierFilter=None, NextToken: str=None, MaxResults: int=None) -> ListDocumentClassifiersResponse:
        """Gets a list of the document classifiers that you have created.

        :param self:
        :param MaxResults:  The maximum number of results to return in each page. The default is 100.<br/>NextToken:  Identifies the next page of results to return.<br/>Filter:  Filters the jobs that are returned. You can filter jobs on their name, status, or the date and time that they were submitted. You can only set one filter at a time.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmitTimeAfter</i> Filters the list of classifiers based on the time that the classifier was submitted for processing. Returns only classifiers submitted after the specified time. Classifiers are returned in descending order, newest to oldest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmitTimeBefore</i> Filters the list of classifiers based on the time that the classifier was submitted for processing. Returns only classifiers submitted before the specified time. Classifiers are returned in ascending order, oldest to newest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Filters the list of classifiers based on status. <br/>
        :return: NextToken:  Identifies the next page of results to return.<br/>DocumentClassifierPropertiesList:  A list containing the properties of each job returned.<br/>        """
        pass

    def list_dominant_language_detection_jobs(self, Filter: DominantLanguageDetectionJobFilter=None, NextToken: str=None, MaxResults: int=None) -> ListDominantLanguageDetectionJobsResponse:
        """Gets a list of the dominant language detection jobs that you have submitted.

        :param self:
        :param MaxResults:  The maximum number of results to return in each page. The default is 100.<br/>NextToken:  Identifies the next page of results to return.<br/>Filter:  Filters that jobs that are returned. You can filter jobs on their name, status, or the date and time that they were submitted. You can only set one filter at a time.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmitTimeAfter</i> Filters the list of jobs based on the time that the job was submitted for processing. Returns only jobs submitted after the specified time. Jobs are returned in descending order, newest to oldest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmitTimeBefore</i> Filters the list of jobs based on the time that the job was submitted for processing. Returns only jobs submitted before the specified time. Jobs are returned in ascending order, oldest to newest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobStatus</i> Filters the list of jobs based on job status. Returns only jobs with the specified status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobName</i> Filters on the name of the job.<br/>
        :return: NextToken:  Identifies the next page of results to return.<br/>DominantLanguageDetectionJobPropertiesList:  A list containing the properties of each job that is returned.<br/>        """
        pass

    def list_entities_detection_jobs(self, Filter: EntitiesDetectionJobFilter=None, NextToken: str=None, MaxResults: int=None) -> ListEntitiesDetectionJobsResponse:
        """Gets a list of the entity detection jobs that you have submitted.

        :param self:
        :param MaxResults:  The maximum number of results to return in each page. The default is 100.<br/>NextToken:  Identifies the next page of results to return.<br/>Filter:  Filters the jobs that are returned. You can filter jobs on their name, status, or the date and time that they were submitted. You can only set one filter at a time.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmitTimeAfter</i> Filters the list of jobs based on the time that the job was submitted for processing. Returns only jobs submitted after the specified time. Jobs are returned in descending order, newest to oldest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmitTimeBefore</i> Filters the list of jobs based on the time that the job was submitted for processing. Returns only jobs submitted before the specified time. Jobs are returned in ascending order, oldest to newest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobStatus</i> Filters the list of jobs based on job status. Returns only jobs with the specified status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobName</i> Filters on the name of the job.<br/>
        :return: NextToken:  Identifies the next page of results to return.<br/>EntitiesDetectionJobPropertiesList:  A list containing the properties of each job that is returned.<br/>        """
        pass

    def list_entity_recognizers(self, Filter: EntityRecognizerFilter=None, NextToken: str=None, MaxResults: int=None) -> ListEntityRecognizersResponse:
        """Gets a list of the properties of all entity recognizers that you created, including recognizers currently in training. Allows you to filter the list of recognizers based on criteria such as status and submission time. This call returns up to 500 entity recognizers in the list, with a default number of 100 recognizers in the list. The results of this list are not in any particular order. Please get the list and sort locally if needed.

        :param self:
        :param MaxResults:   The maximum number of results to return on each page. The default is 100.<br/>NextToken:  Identifies the next page of results to return.<br/>Filter:  Filters the list of entities returned. You can filter on <code>Status</code>, <code>SubmitTimeBefore</code>, or <code>SubmitTimeAfter</code>. You can only set one filter at a time.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmitTimeAfter</i> Filters the list of entities based on the time that the list was submitted for processing. Returns only jobs submitted after the specified time. Jobs are returned in ascending order, oldest to newest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmitTimeBefore</i> Filters the list of entities based on the time that the list was submitted for processing. Returns only jobs submitted before the specified time. Jobs are returned in descending order, newest to oldest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of an entity recognizer.<br/>
        :return: NextToken:  Identifies the next page of results to return.<br/>EntityRecognizerPropertiesList:  The list of properties of an entity recognizer.<br/>        """
        pass

    def list_key_phrases_detection_jobs(self, Filter: KeyPhrasesDetectionJobFilter=None, NextToken: str=None, MaxResults: int=None) -> ListKeyPhrasesDetectionJobsResponse:
        """Get a list of key phrase detection jobs that you have submitted.

        :param self:
        :param MaxResults:  The maximum number of results to return in each page. The default is 100.<br/>NextToken:  Identifies the next page of results to return.<br/>Filter:  Filters the jobs that are returned. You can filter jobs on their name, status, or the date and time that they were submitted. You can only set one filter at a time.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmitTimeAfter</i> Filters the list of jobs based on the time that the job was submitted for processing. Returns only jobs submitted after the specified time. Jobs are returned in descending order, newest to oldest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmitTimeBefore</i> Filters the list of jobs based on the time that the job was submitted for processing. Returns only jobs submitted before the specified time. Jobs are returned in ascending order, oldest to newest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobStatus</i> Filters the list of jobs based on job status. Returns only jobs with the specified status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobName</i> Filters on the name of the job.<br/>
        :return: NextToken:  Identifies the next page of results to return.<br/>KeyPhrasesDetectionJobPropertiesList:  A list containing the properties of each job that is returned.<br/>        """
        pass

    def list_sentiment_detection_jobs(self, Filter: SentimentDetectionJobFilter=None, NextToken: str=None, MaxResults: int=None) -> ListSentimentDetectionJobsResponse:
        """Gets a list of sentiment detection jobs that you have submitted.

        :param self:
        :param MaxResults:  The maximum number of results to return in each page. The default is 100.<br/>NextToken:  Identifies the next page of results to return.<br/>Filter:  Filters the jobs that are returned. You can filter jobs on their name, status, or the date and time that they were submitted. You can only set one filter at a time.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmitTimeAfter</i> Filters the list of jobs based on the time that the job was submitted for processing. Returns only jobs submitted after the specified time. Jobs are returned in descending order, newest to oldest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmitTimeBefore</i> Filters the list of jobs based on the time that the job was submitted for processing. Returns only jobs submitted before the specified time. Jobs are returned in ascending order, oldest to newest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobStatus</i> Filters the list of jobs based on job status. Returns only jobs with the specified status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobName</i> Filters on the name of the job.<br/>
        :return: NextToken:  Identifies the next page of results to return.<br/>SentimentDetectionJobPropertiesList:  A list containing the properties of each job that is returned.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceArn: str) -> ListTagsForResourceResponse:
        """Lists all tags associated with a given Amazon Comprehend resource. 

        :param self:
        :param ResourceArn:  The Amazon Resource Name (ARN) of the given Amazon Comprehend resource you are querying. <br/>
        :return: Tags:  Tags associated with the Amazon Comprehend resource being queried. A tag is a key-value pair that adds as a metadata to a resource used by Amazon Comprehend. For example, a tag with "Sales" as the key might be added to a resource to indicate its use by the sales department. <br/>ResourceArn:  The Amazon Resource Name (ARN) of the given Amazon Comprehend resource you are querying.<br/>        """
        pass

    def list_topics_detection_jobs(self, Filter: TopicsDetectionJobFilter=None, NextToken: str=None, MaxResults: int=None) -> ListTopicsDetectionJobsResponse:
        """Gets a list of the topic detection jobs that you have submitted.

        :param self:
        :param MaxResults:  The maximum number of results to return in each page. The default is 100.<br/>NextToken:  Identifies the next page of results to return.<br/>Filter:  Filters the jobs that are returned. Jobs can be filtered on their name, status, or the date and time that they were submitted. You can set only one filter at a time.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmitTimeAfter</i> Filters the list of jobs based on the time that the job was submitted for processing. Only returns jobs submitted after the specified time. Jobs are returned in ascending order, oldest to newest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmitTimeBefore</i> Filters the list of jobs based on the time that the job was submitted for processing. Only returns jobs submitted before the specified time. Jobs are returned in descending order, newest to oldest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobStatus</i> Filters the list of topic detection jobs based on job status. Returns only jobs with the specified status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobName</i> <p/><br/>
        :return: NextToken:  Identifies the next page of results to return.<br/>TopicsDetectionJobPropertiesList:  A list containing the properties of each job that is returned.<br/>        """
        pass

    def start_document_classification_job(self, DataAccessRoleArn: str, OutputDataConfig: OutputDataConfig, InputDataConfig: InputDataConfig, DocumentClassifierArn: str, JobName: str=None, ClientRequestToken: str=None, VolumeKmsKeyId: str=None, VpcConfig: VpcConfig=None) -> StartDocumentClassificationJobResponse:
        """Starts an asynchronous document classification job. Use the operation to track the progress of the job.

        :param self:
        :param VpcConfig:  Configuration parameters for an optional private Virtual Private Cloud (VPC) containing the resources you are using for your document classification job. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/what-is-amazon-vpc.html">Amazon VPC</a>. :<br/>VolumeKmsKeyId:  ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt data on the storage volume attached to the ML compute instance(s) that process the analysis job. The VolumeKmsKeyId can be either of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> </ul><br/>ClientRequestToken:  A unique identifier for the request. If you do not set the client request token, Amazon Comprehend generates one.<br/>DataAccessRoleArn:  The Amazon Resource Name (ARN) of the AWS Identity and Access Management (IAM) role that grants Amazon Comprehend read access to your input data.<br/>OutputDataConfig:  Specifies where to send the output files.:<br/>InputDataConfig:  Specifies the format and location of the input data for the job.:<br/>DocumentClassifierArn:  The Amazon Resource Name (ARN) of the document classifier to use to process the job.<br/>JobName:  The identifier of the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputFormat</i> Specifies how the text in an input file should be processed: <ul> <li>  <code>ONE_DOC_PER_FILE</code> - Each file is considered a separate document. Use this option when you are processing large documents, such as newspaper articles or scientific papers. </li> <li>  <code>ONE_DOC_PER_LINE</code> - Each line in a file is considered a separate document. Use this option when you are processing many short documents, such as text messages. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> The Amazon S3 URI for the input data. The URI must be in same region as the API endpoint that you are calling. The URI can point to a single input file or it can provide the prefix for a collection of data files.  For example, if you use the URI <code>S3://bucketName/prefix</code>, if the prefix is a single file, Amazon Comprehend uses that file as input. If more than one file begins with the prefix, Amazon Comprehend uses all of them as input.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt the output results from an analysis job. The KmsKeyId can be one of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> KMS Key Alias: <code>"alias/ExampleAlias"</code>  </li> <li> ARN of a KMS Key Alias: <code>"arn:aws:kms:us-west-2:111122223333:alias/ExampleAlias"</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> When you use the <code>OutputDataConfig</code> object with asynchronous operations, you specify the Amazon S3 location where you want to write the output data. The URI must be in the same region as the API endpoint that you are calling. The location is used as the prefix for the actual location of the output file. When the topic detection job is finished, the service creates an output file in a directory specific to the job. The <code>S3Uri</code> field contains the location of the output file, called <code>output.tar.gz</code>. It is a compressed archive that contains the ouput of the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> The ID for each subnet being used in your private VPC. This subnet is a subset of the a range of IPv4 addresses used by the VPC and is specific to a given availability zone in the VPC’s region. This ID number is preceded by "subnet-", for instance: "subnet-04ccf456919e69055". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Subnets.html">VPCs and Subnets</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The ID number for a security group on an instance of your private VPC. Security groups on your VPC function serve as a virtual firewall to control inbound and outbound traffic and provides security for the resources that you’ll be accessing on the VPC. This ID number is preceded by "sg-", for instance: "sg-03b388029b0a285ea". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_SecurityGroups.html">Security Groups for your VPC</a>. <br/>
        :return: JobStatus:  The status of the job: <ul> <li> SUBMITTED - The job has been received and queued for processing. </li> <li> IN_PROGRESS - Amazon Comprehend is processing the job. </li> <li> COMPLETED - The job was successfully completed and the output is available. </li> <li> FAILED - The job did not complete. For details, use the operation. </li> <li> STOP_REQUESTED - Amazon Comprehend has received a stop request for the job and is processing the request. </li> <li> STOPPED - The job was successfully stopped without completing. </li> </ul><br/>JobId:  The identifier generated for the job. To get the status of the job, use this identifier with the operation.<br/>        """
        pass

    def start_dominant_language_detection_job(self, DataAccessRoleArn: str, OutputDataConfig: OutputDataConfig, InputDataConfig: InputDataConfig, JobName: str=None, ClientRequestToken: str=None, VolumeKmsKeyId: str=None, VpcConfig: VpcConfig=None) -> StartDominantLanguageDetectionJobResponse:
        """Starts an asynchronous dominant language detection job for a collection of documents. Use the operation to track the status of a job.

        :param self:
        :param VpcConfig:  Configuration parameters for an optional private Virtual Private Cloud (VPC) containing the resources you are using for your dominant language detection job. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/what-is-amazon-vpc.html">Amazon VPC</a>. :<br/>VolumeKmsKeyId:  ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt data on the storage volume attached to the ML compute instance(s) that process the analysis job. The VolumeKmsKeyId can be either of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> </ul><br/>ClientRequestToken:  A unique identifier for the request. If you do not set the client request token, Amazon Comprehend generates one.<br/>JobName:  An identifier for the job.<br/>DataAccessRoleArn:  The Amazon Resource Name (ARN) of the AWS Identity and Access Management (IAM) role that grants Amazon Comprehend read access to your input data. For more information, see <a href="https://docs.aws.amazon.com/comprehend/latest/dg/access-control-managing-permissions.html#auth-role-permissions">https://docs.aws.amazon.com/comprehend/latest/dg/access-control-managing-permissions.html#auth-role-permissions</a>.<br/>OutputDataConfig:  Specifies where to send the output files.:<br/>InputDataConfig:  Specifies the format and location of the input data for the job.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputFormat</i> Specifies how the text in an input file should be processed: <ul> <li>  <code>ONE_DOC_PER_FILE</code> - Each file is considered a separate document. Use this option when you are processing large documents, such as newspaper articles or scientific papers. </li> <li>  <code>ONE_DOC_PER_LINE</code> - Each line in a file is considered a separate document. Use this option when you are processing many short documents, such as text messages. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> The Amazon S3 URI for the input data. The URI must be in same region as the API endpoint that you are calling. The URI can point to a single input file or it can provide the prefix for a collection of data files.  For example, if you use the URI <code>S3://bucketName/prefix</code>, if the prefix is a single file, Amazon Comprehend uses that file as input. If more than one file begins with the prefix, Amazon Comprehend uses all of them as input.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt the output results from an analysis job. The KmsKeyId can be one of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> KMS Key Alias: <code>"alias/ExampleAlias"</code>  </li> <li> ARN of a KMS Key Alias: <code>"arn:aws:kms:us-west-2:111122223333:alias/ExampleAlias"</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> When you use the <code>OutputDataConfig</code> object with asynchronous operations, you specify the Amazon S3 location where you want to write the output data. The URI must be in the same region as the API endpoint that you are calling. The location is used as the prefix for the actual location of the output file. When the topic detection job is finished, the service creates an output file in a directory specific to the job. The <code>S3Uri</code> field contains the location of the output file, called <code>output.tar.gz</code>. It is a compressed archive that contains the ouput of the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> The ID for each subnet being used in your private VPC. This subnet is a subset of the a range of IPv4 addresses used by the VPC and is specific to a given availability zone in the VPC’s region. This ID number is preceded by "subnet-", for instance: "subnet-04ccf456919e69055". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Subnets.html">VPCs and Subnets</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The ID number for a security group on an instance of your private VPC. Security groups on your VPC function serve as a virtual firewall to control inbound and outbound traffic and provides security for the resources that you’ll be accessing on the VPC. This ID number is preceded by "sg-", for instance: "sg-03b388029b0a285ea". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_SecurityGroups.html">Security Groups for your VPC</a>. <br/>
        :return: JobStatus:  The status of the job.  <ul> <li> SUBMITTED - The job has been received and is queued for processing. </li> <li> IN_PROGRESS - Amazon Comprehend is processing the job. </li> <li> COMPLETED - The job was successfully completed and the output is available. </li> <li> FAILED - The job did not complete. To get details, use the operation. </li> </ul><br/>JobId:  The identifier generated for the job. To get the status of a job, use this identifier with the operation.<br/>        """
        pass

    def start_entities_detection_job(self, LanguageCode: str, DataAccessRoleArn: str, OutputDataConfig: OutputDataConfig, InputDataConfig: InputDataConfig, JobName: str=None, EntityRecognizerArn: str=None, ClientRequestToken: str=None, VolumeKmsKeyId: str=None, VpcConfig: VpcConfig=None) -> StartEntitiesDetectionJobResponse:
        """Starts an asynchronous entity detection job for a collection of documents. Use the operation to track the status of a job. This API can be used for either standard entity detection or custom entity recognition. In order to be used for custom entity recognition, the optional <code>EntityRecognizerArn</code> must be used in order to provide access to the recognizer being used to detect the custom entity.

        :param self:
        :param VpcConfig:  Configuration parameters for an optional private Virtual Private Cloud (VPC) containing the resources you are using for your entity detection job. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/what-is-amazon-vpc.html">Amazon VPC</a>. :<br/>VolumeKmsKeyId:  ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt data on the storage volume attached to the ML compute instance(s) that process the analysis job. The VolumeKmsKeyId can be either of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> </ul><br/>ClientRequestToken:  A unique identifier for the request. If you don't set the client request token, Amazon Comprehend generates one.<br/>LanguageCode:  The language of the input documents. All documents must be in the same language. You can specify any of the languages supported by Amazon Comprehend: English ("en"), Spanish ("es"), French ("fr"), German ("de"), Italian ("it"), or Portuguese ("pt"). If custom entities recognition is used, this parameter is ignored and the language used for training the model is used instead.<br/>EntityRecognizerArn:  The Amazon Resource Name (ARN) that identifies the specific entity recognizer to be used by the <code>StartEntitiesDetectionJob</code>. This ARN is optional and is only used for a custom entity recognition job.<br/>JobName:  The identifier of the job.<br/>DataAccessRoleArn:  The Amazon Resource Name (ARN) of the AWS Identity and Access Management (IAM) role that grants Amazon Comprehend read access to your input data. For more information, see <a href="https://docs.aws.amazon.com/comprehend/latest/dg/access-control-managing-permissions.html#auth-role-permissions">https://docs.aws.amazon.com/comprehend/latest/dg/access-control-managing-permissions.html#auth-role-permissions</a>.<br/>OutputDataConfig:  Specifies where to send the output files.:<br/>InputDataConfig:  Specifies the format and location of the input data for the job.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputFormat</i> Specifies how the text in an input file should be processed: <ul> <li>  <code>ONE_DOC_PER_FILE</code> - Each file is considered a separate document. Use this option when you are processing large documents, such as newspaper articles or scientific papers. </li> <li>  <code>ONE_DOC_PER_LINE</code> - Each line in a file is considered a separate document. Use this option when you are processing many short documents, such as text messages. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> The Amazon S3 URI for the input data. The URI must be in same region as the API endpoint that you are calling. The URI can point to a single input file or it can provide the prefix for a collection of data files.  For example, if you use the URI <code>S3://bucketName/prefix</code>, if the prefix is a single file, Amazon Comprehend uses that file as input. If more than one file begins with the prefix, Amazon Comprehend uses all of them as input.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt the output results from an analysis job. The KmsKeyId can be one of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> KMS Key Alias: <code>"alias/ExampleAlias"</code>  </li> <li> ARN of a KMS Key Alias: <code>"arn:aws:kms:us-west-2:111122223333:alias/ExampleAlias"</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> When you use the <code>OutputDataConfig</code> object with asynchronous operations, you specify the Amazon S3 location where you want to write the output data. The URI must be in the same region as the API endpoint that you are calling. The location is used as the prefix for the actual location of the output file. When the topic detection job is finished, the service creates an output file in a directory specific to the job. The <code>S3Uri</code> field contains the location of the output file, called <code>output.tar.gz</code>. It is a compressed archive that contains the ouput of the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> The ID for each subnet being used in your private VPC. This subnet is a subset of the a range of IPv4 addresses used by the VPC and is specific to a given availability zone in the VPC’s region. This ID number is preceded by "subnet-", for instance: "subnet-04ccf456919e69055". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Subnets.html">VPCs and Subnets</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The ID number for a security group on an instance of your private VPC. Security groups on your VPC function serve as a virtual firewall to control inbound and outbound traffic and provides security for the resources that you’ll be accessing on the VPC. This ID number is preceded by "sg-", for instance: "sg-03b388029b0a285ea". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_SecurityGroups.html">Security Groups for your VPC</a>. <br/>
        :return: JobStatus:  The status of the job.  <ul> <li> SUBMITTED - The job has been received and is queued for processing. </li> <li> IN_PROGRESS - Amazon Comprehend is processing the job. </li> <li> COMPLETED - The job was successfully completed and the output is available. </li> <li> FAILED - The job did not complete. To get details, use the operation. </li> <li> STOP_REQUESTED - Amazon Comprehend has received a stop request for the job and is processing the request. </li> <li> STOPPED - The job was successfully stopped without completing. </li> </ul><br/>JobId:  The identifier generated for the job. To get the status of job, use this identifier with the operation.<br/>        """
        pass

    def start_key_phrases_detection_job(self, LanguageCode: str, DataAccessRoleArn: str, OutputDataConfig: OutputDataConfig, InputDataConfig: InputDataConfig, JobName: str=None, ClientRequestToken: str=None, VolumeKmsKeyId: str=None, VpcConfig: VpcConfig=None) -> StartKeyPhrasesDetectionJobResponse:
        """Starts an asynchronous key phrase detection job for a collection of documents. Use the operation to track the status of a job.

        :param self:
        :param VpcConfig:   Configuration parameters for an optional private Virtual Private Cloud (VPC) containing the resources you are using for your key phrases detection job. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/what-is-amazon-vpc.html">Amazon VPC</a>. :<br/>VolumeKmsKeyId:  ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt data on the storage volume attached to the ML compute instance(s) that process the analysis job. The VolumeKmsKeyId can be either of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> </ul><br/>ClientRequestToken:  A unique identifier for the request. If you don't set the client request token, Amazon Comprehend generates one.<br/>LanguageCode:  The language of the input documents. You can specify any of the primary languages supported by Amazon Comprehend: German ("de"), English ("en"), Spanish ("es"), French ("fr"), Italian ("it"), or Portuguese ("pt"). All documents must be in the same language.<br/>JobName:  The identifier of the job.<br/>DataAccessRoleArn:  The Amazon Resource Name (ARN) of the AWS Identity and Access Management (IAM) role that grants Amazon Comprehend read access to your input data. For more information, see <a href="https://docs.aws.amazon.com/comprehend/latest/dg/access-control-managing-permissions.html#auth-role-permissions">https://docs.aws.amazon.com/comprehend/latest/dg/access-control-managing-permissions.html#auth-role-permissions</a>.<br/>OutputDataConfig:  Specifies where to send the output files.:<br/>InputDataConfig:  Specifies the format and location of the input data for the job.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputFormat</i> Specifies how the text in an input file should be processed: <ul> <li>  <code>ONE_DOC_PER_FILE</code> - Each file is considered a separate document. Use this option when you are processing large documents, such as newspaper articles or scientific papers. </li> <li>  <code>ONE_DOC_PER_LINE</code> - Each line in a file is considered a separate document. Use this option when you are processing many short documents, such as text messages. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> The Amazon S3 URI for the input data. The URI must be in same region as the API endpoint that you are calling. The URI can point to a single input file or it can provide the prefix for a collection of data files.  For example, if you use the URI <code>S3://bucketName/prefix</code>, if the prefix is a single file, Amazon Comprehend uses that file as input. If more than one file begins with the prefix, Amazon Comprehend uses all of them as input.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt the output results from an analysis job. The KmsKeyId can be one of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> KMS Key Alias: <code>"alias/ExampleAlias"</code>  </li> <li> ARN of a KMS Key Alias: <code>"arn:aws:kms:us-west-2:111122223333:alias/ExampleAlias"</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> When you use the <code>OutputDataConfig</code> object with asynchronous operations, you specify the Amazon S3 location where you want to write the output data. The URI must be in the same region as the API endpoint that you are calling. The location is used as the prefix for the actual location of the output file. When the topic detection job is finished, the service creates an output file in a directory specific to the job. The <code>S3Uri</code> field contains the location of the output file, called <code>output.tar.gz</code>. It is a compressed archive that contains the ouput of the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> The ID for each subnet being used in your private VPC. This subnet is a subset of the a range of IPv4 addresses used by the VPC and is specific to a given availability zone in the VPC’s region. This ID number is preceded by "subnet-", for instance: "subnet-04ccf456919e69055". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Subnets.html">VPCs and Subnets</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The ID number for a security group on an instance of your private VPC. Security groups on your VPC function serve as a virtual firewall to control inbound and outbound traffic and provides security for the resources that you’ll be accessing on the VPC. This ID number is preceded by "sg-", for instance: "sg-03b388029b0a285ea". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_SecurityGroups.html">Security Groups for your VPC</a>. <br/>
        :return: JobStatus:  The status of the job.  <ul> <li> SUBMITTED - The job has been received and is queued for processing. </li> <li> IN_PROGRESS - Amazon Comprehend is processing the job. </li> <li> COMPLETED - The job was successfully completed and the output is available. </li> <li> FAILED - The job did not complete. To get details, use the operation. </li> </ul><br/>JobId:  The identifier generated for the job. To get the status of a job, use this identifier with the operation.<br/>        """
        pass

    def start_sentiment_detection_job(self, LanguageCode: str, DataAccessRoleArn: str, OutputDataConfig: OutputDataConfig, InputDataConfig: InputDataConfig, JobName: str=None, ClientRequestToken: str=None, VolumeKmsKeyId: str=None, VpcConfig: VpcConfig=None) -> StartSentimentDetectionJobResponse:
        """Starts an asynchronous sentiment detection job for a collection of documents. use the operation to track the status of a job.

        :param self:
        :param VpcConfig:  Configuration parameters for an optional private Virtual Private Cloud (VPC) containing the resources you are using for your sentiment detection job. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/what-is-amazon-vpc.html">Amazon VPC</a>. :<br/>VolumeKmsKeyId:  ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt data on the storage volume attached to the ML compute instance(s) that process the analysis job. The VolumeKmsKeyId can be either of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> </ul><br/>ClientRequestToken:  A unique identifier for the request. If you don't set the client request token, Amazon Comprehend generates one.<br/>LanguageCode:  The language of the input documents. You can specify any of the primary languages supported by Amazon Comprehend: German ("de"), English ("en"), Spanish ("es"), French ("fr"), Italian ("it"), or Portuguese ("pt"). All documents must be in the same language.<br/>JobName:  The identifier of the job.<br/>DataAccessRoleArn:  The Amazon Resource Name (ARN) of the AWS Identity and Access Management (IAM) role that grants Amazon Comprehend read access to your input data. For more information, see <a href="https://docs.aws.amazon.com/comprehend/latest/dg/access-control-managing-permissions.html#auth-role-permissions">https://docs.aws.amazon.com/comprehend/latest/dg/access-control-managing-permissions.html#auth-role-permissions</a>.<br/>OutputDataConfig:  Specifies where to send the output files. :<br/>InputDataConfig:  Specifies the format and location of the input data for the job.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputFormat</i> Specifies how the text in an input file should be processed: <ul> <li>  <code>ONE_DOC_PER_FILE</code> - Each file is considered a separate document. Use this option when you are processing large documents, such as newspaper articles or scientific papers. </li> <li>  <code>ONE_DOC_PER_LINE</code> - Each line in a file is considered a separate document. Use this option when you are processing many short documents, such as text messages. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> The Amazon S3 URI for the input data. The URI must be in same region as the API endpoint that you are calling. The URI can point to a single input file or it can provide the prefix for a collection of data files.  For example, if you use the URI <code>S3://bucketName/prefix</code>, if the prefix is a single file, Amazon Comprehend uses that file as input. If more than one file begins with the prefix, Amazon Comprehend uses all of them as input.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt the output results from an analysis job. The KmsKeyId can be one of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> KMS Key Alias: <code>"alias/ExampleAlias"</code>  </li> <li> ARN of a KMS Key Alias: <code>"arn:aws:kms:us-west-2:111122223333:alias/ExampleAlias"</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> When you use the <code>OutputDataConfig</code> object with asynchronous operations, you specify the Amazon S3 location where you want to write the output data. The URI must be in the same region as the API endpoint that you are calling. The location is used as the prefix for the actual location of the output file. When the topic detection job is finished, the service creates an output file in a directory specific to the job. The <code>S3Uri</code> field contains the location of the output file, called <code>output.tar.gz</code>. It is a compressed archive that contains the ouput of the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> The ID for each subnet being used in your private VPC. This subnet is a subset of the a range of IPv4 addresses used by the VPC and is specific to a given availability zone in the VPC’s region. This ID number is preceded by "subnet-", for instance: "subnet-04ccf456919e69055". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Subnets.html">VPCs and Subnets</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The ID number for a security group on an instance of your private VPC. Security groups on your VPC function serve as a virtual firewall to control inbound and outbound traffic and provides security for the resources that you’ll be accessing on the VPC. This ID number is preceded by "sg-", for instance: "sg-03b388029b0a285ea". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_SecurityGroups.html">Security Groups for your VPC</a>. <br/>
        :return: JobStatus:  The status of the job.  <ul> <li> SUBMITTED - The job has been received and is queued for processing. </li> <li> IN_PROGRESS - Amazon Comprehend is processing the job. </li> <li> COMPLETED - The job was successfully completed and the output is available. </li> <li> FAILED - The job did not complete. To get details, use the operation. </li> </ul><br/>JobId:  The identifier generated for the job. To get the status of a job, use this identifier with the operation.<br/>        """
        pass

    def start_topics_detection_job(self, DataAccessRoleArn: str, OutputDataConfig: OutputDataConfig, InputDataConfig: InputDataConfig, JobName: str=None, NumberOfTopics: int=None, ClientRequestToken: str=None, VolumeKmsKeyId: str=None, VpcConfig: VpcConfig=None) -> StartTopicsDetectionJobResponse:
        """Starts an asynchronous topic detection job. Use the <code>DescribeTopicDetectionJob</code> operation to track the status of a job.

        :param self:
        :param VpcConfig:  Configuration parameters for an optional private Virtual Private Cloud (VPC) containing the resources you are using for your topic detection job. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/what-is-amazon-vpc.html">Amazon VPC</a>. :<br/>VolumeKmsKeyId:  ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt data on the storage volume attached to the ML compute instance(s) that process the analysis job. The VolumeKmsKeyId can be either of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> </ul><br/>ClientRequestToken:  A unique identifier for the request. If you do not set the client request token, Amazon Comprehend generates one.<br/>NumberOfTopics:  The number of topics to detect.<br/>JobName:  The identifier of the job.<br/>DataAccessRoleArn:  The Amazon Resource Name (ARN) of the AWS Identity and Access Management (IAM) role that grants Amazon Comprehend read access to your input data. For more information, see <a href="https://docs.aws.amazon.com/comprehend/latest/dg/access-control-managing-permissions.html#auth-role-permissions">https://docs.aws.amazon.com/comprehend/latest/dg/access-control-managing-permissions.html#auth-role-permissions</a>.<br/>OutputDataConfig:  Specifies where to send the output files. The output is a compressed archive with two files, <code>topic-terms.csv</code> that lists the terms associated with each topic, and <code>doc-topics.csv</code> that lists the documents associated with each topic:<br/>InputDataConfig:  Specifies the format and location of the input data for the job.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputFormat</i> Specifies how the text in an input file should be processed: <ul> <li>  <code>ONE_DOC_PER_FILE</code> - Each file is considered a separate document. Use this option when you are processing large documents, such as newspaper articles or scientific papers. </li> <li>  <code>ONE_DOC_PER_LINE</code> - Each line in a file is considered a separate document. Use this option when you are processing many short documents, such as text messages. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> The Amazon S3 URI for the input data. The URI must be in same region as the API endpoint that you are calling. The URI can point to a single input file or it can provide the prefix for a collection of data files.  For example, if you use the URI <code>S3://bucketName/prefix</code>, if the prefix is a single file, Amazon Comprehend uses that file as input. If more than one file begins with the prefix, Amazon Comprehend uses all of them as input.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> ID for the AWS Key Management Service (KMS) key that Amazon Comprehend uses to encrypt the output results from an analysis job. The KmsKeyId can be one of the following formats: <ul> <li> KMS Key ID: <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> Amazon Resource Name (ARN) of a KMS Key: <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> KMS Key Alias: <code>"alias/ExampleAlias"</code>  </li> <li> ARN of a KMS Key Alias: <code>"arn:aws:kms:us-west-2:111122223333:alias/ExampleAlias"</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> When you use the <code>OutputDataConfig</code> object with asynchronous operations, you specify the Amazon S3 location where you want to write the output data. The URI must be in the same region as the API endpoint that you are calling. The location is used as the prefix for the actual location of the output file. When the topic detection job is finished, the service creates an output file in a directory specific to the job. The <code>S3Uri</code> field contains the location of the output file, called <code>output.tar.gz</code>. It is a compressed archive that contains the ouput of the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> The ID for each subnet being used in your private VPC. This subnet is a subset of the a range of IPv4 addresses used by the VPC and is specific to a given availability zone in the VPC’s region. This ID number is preceded by "subnet-", for instance: "subnet-04ccf456919e69055". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Subnets.html">VPCs and Subnets</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The ID number for a security group on an instance of your private VPC. Security groups on your VPC function serve as a virtual firewall to control inbound and outbound traffic and provides security for the resources that you’ll be accessing on the VPC. This ID number is preceded by "sg-", for instance: "sg-03b388029b0a285ea". For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_SecurityGroups.html">Security Groups for your VPC</a>. <br/>
        :return: JobStatus:  The status of the job:  <ul> <li> SUBMITTED - The job has been received and is queued for processing. </li> <li> IN_PROGRESS - Amazon Comprehend is processing the job. </li> <li> COMPLETED - The job was successfully completed and the output is available. </li> <li> FAILED - The job did not complete. To get details, use the <code>DescribeTopicDetectionJob</code> operation. </li> </ul><br/>JobId:  The identifier generated for the job. To get the status of the job, use this identifier with the <code>DescribeTopicDetectionJob</code> operation.<br/>        """
        pass

    def stop_dominant_language_detection_job(self, JobId: str) -> StopDominantLanguageDetectionJobResponse:
        """Stops a dominant language detection job in progress. If the job state is <code>IN_PROGRESS</code> the job is marked for termination and put into the <code>STOP_REQUESTED</code> state. If the job completes before it can be stopped, it is put into the <code>COMPLETED</code> state; otherwise the job is stopped and put into the <code>STOPPED</code> state. If the job is in the <code>COMPLETED</code> or <code>FAILED</code> state when you call the <code>StopDominantLanguageDetectionJob</code> operation, the operation returns a 400 Internal Request Exception.  When a job is stopped, any documents already processed are written to the output location.

        :param self:
        :param JobId:  The identifier of the dominant language detection job to stop.<br/>
        :return: JobStatus:  Either <code>STOP_REQUESTED</code> if the job is currently running, or <code>STOPPED</code> if the job was previously stopped with the <code>StopDominantLanguageDetectionJob</code> operation.<br/>JobId:  The identifier of the dominant language detection job to stop.<br/>        """
        pass

    def stop_entities_detection_job(self, JobId: str) -> StopEntitiesDetectionJobResponse:
        """Stops an entities detection job in progress. If the job state is <code>IN_PROGRESS</code> the job is marked for termination and put into the <code>STOP_REQUESTED</code> state. If the job completes before it can be stopped, it is put into the <code>COMPLETED</code> state; otherwise the job is stopped and put into the <code>STOPPED</code> state. If the job is in the <code>COMPLETED</code> or <code>FAILED</code> state when you call the <code>StopDominantLanguageDetectionJob</code> operation, the operation returns a 400 Internal Request Exception.  When a job is stopped, any documents already processed are written to the output location.

        :param self:
        :param JobId:  The identifier of the entities detection job to stop.<br/>
        :return: JobStatus:  Either <code>STOP_REQUESTED</code> if the job is currently running, or <code>STOPPED</code> if the job was previously stopped with the <code>StopEntitiesDetectionJob</code> operation.<br/>JobId:  The identifier of the entities detection job to stop.<br/>        """
        pass

    def stop_key_phrases_detection_job(self, JobId: str) -> StopKeyPhrasesDetectionJobResponse:
        """Stops a key phrases detection job in progress. If the job state is <code>IN_PROGRESS</code> the job is marked for termination and put into the <code>STOP_REQUESTED</code> state. If the job completes before it can be stopped, it is put into the <code>COMPLETED</code> state; otherwise the job is stopped and put into the <code>STOPPED</code> state. If the job is in the <code>COMPLETED</code> or <code>FAILED</code> state when you call the <code>StopDominantLanguageDetectionJob</code> operation, the operation returns a 400 Internal Request Exception.  When a job is stopped, any documents already processed are written to the output location.

        :param self:
        :param JobId:  The identifier of the key phrases detection job to stop.<br/>
        :return: JobStatus:  Either <code>STOP_REQUESTED</code> if the job is currently running, or <code>STOPPED</code> if the job was previously stopped with the <code>StopKeyPhrasesDetectionJob</code> operation.<br/>JobId:  The identifier of the key phrases detection job to stop.<br/>        """
        pass

    def stop_sentiment_detection_job(self, JobId: str) -> StopSentimentDetectionJobResponse:
        """Stops a sentiment detection job in progress. If the job state is <code>IN_PROGRESS</code> the job is marked for termination and put into the <code>STOP_REQUESTED</code> state. If the job completes before it can be stopped, it is put into the <code>COMPLETED</code> state; otherwise the job is be stopped and put into the <code>STOPPED</code> state. If the job is in the <code>COMPLETED</code> or <code>FAILED</code> state when you call the <code>StopDominantLanguageDetectionJob</code> operation, the operation returns a 400 Internal Request Exception.  When a job is stopped, any documents already processed are written to the output location.

        :param self:
        :param JobId:  The identifier of the sentiment detection job to stop.<br/>
        :return: JobStatus:  Either <code>STOP_REQUESTED</code> if the job is currently running, or <code>STOPPED</code> if the job was previously stopped with the <code>StopSentimentDetectionJob</code> operation.<br/>JobId:  The identifier of the sentiment detection job to stop.<br/>        """
        pass

    def stop_training_document_classifier(self, DocumentClassifierArn: str) -> StopTrainingDocumentClassifierResponse:
        """Stops a document classifier training job while in progress. If the training job state is <code>TRAINING</code>, the job is marked for termination and put into the <code>STOP_REQUESTED</code> state. If the training job completes before it can be stopped, it is put into the <code>TRAINED</code>; otherwise the training job is stopped and put into the <code>STOPPED</code> state and the service sends back an HTTP 200 response with an empty HTTP body. 

        :param self:
        :param DocumentClassifierArn:  The Amazon Resource Name (ARN) that identifies the document classifier currently being trained.<br/>
        :return:         """
        pass

    def stop_training_entity_recognizer(self, EntityRecognizerArn: str) -> StopTrainingEntityRecognizerResponse:
        """Stops an entity recognizer training job while in progress. If the training job state is <code>TRAINING</code>, the job is marked for termination and put into the <code>STOP_REQUESTED</code> state. If the training job completes before it can be stopped, it is put into the <code>TRAINED</code>; otherwise the training job is stopped and putted into the <code>STOPPED</code> state and the service sends back an HTTP 200 response with an empty HTTP body.

        :param self:
        :param EntityRecognizerArn:  The Amazon Resource Name (ARN) that identifies the entity recognizer currently being trained.<br/>
        :return:         """
        pass

    def tag_resource(self, Tags: List[Tag], ResourceArn: str) -> TagResourceResponse:
        """Associates a specific tag with an Amazon Comprehend resource. A tag is a key-value pair that adds as a metadata to a resource used by Amazon Comprehend. For example, a tag with "Sales" as the key might be added to a resource to indicate its use by the sales department. 

        :param self:
        :param Tags:  Tags being associated with a specific Amazon Comprehend resource. There can be a maximum of 50 tags (both existing and pending) associated with a specific resource. <br/>ResourceArn:  The Amazon Resource Name (ARN) of the given Amazon Comprehend resource to which you want to associate the tags. <br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[TagKey], ResourceArn: str) -> UntagResourceResponse:
        """Removes a specific tag associated with an Amazon Comprehend resource. 

        :param self:
        :param TagKeys:  The initial part of a key-value pair that forms a tag being removed from a given resource. For example, a tag with "Sales" as the key might be added to a resource to indicate its use by the sales department. Keys must be unique and cannot be duplicated for a particular resource. <br/>ResourceArn:   The Amazon Resource Name (ARN) of the given Amazon Comprehend resource from which you want to remove the tags. <br/>
        :return:         """
        pass



class ComprehendMedical(BaseClient):
    """ Comprehend Medical extracts structured information from unstructured clinical text. Use these actions to gain insight in your documents. """




    class DetectEntitiesRequest(Mapping):
        pass
    
    class DetectEntitiesResponse(Mapping):
        pass
    
    class DetectPHIRequest(Mapping):
        pass
    
    class DetectPHIResponse(Mapping):
        pass
    
    def detect_entities(self, Text: str) -> DetectEntitiesResponse:
        """ Inspects the clinical text for a variety of medical entities and returns specific information about them such as entity category, location, and confidence score on that information .

        :param self:
        :param Text:   A UTF-8 text string containing the clinical content being examined for entities. Each string must contain fewer than 20,000 bytes of characters.<br/>
        :return: PaginationToken:   If the result of the previous request to DetectEntities was truncated, include the Paginationtoken to fetch the next page of entities.<br/>UnmappedAttributes:   Attributes extracted from the input text that we were unable to relate to an entity.<br/>Entities:   The collection of medical entities extracted from the input text and their associated information. For each entity, the response provides the entity text, the entity category, where the entity text begins and ends, and the level of confidence that Comprehend Medical has in the detection and analysis. Attributes and traits of the entity are also returned.<br/>        """
        pass

    def detect_phi(self, Text: str) -> DetectPHIResponse:
        """ Inspects the clinical text for personal health information (PHI) entities and entity category, location, and confidence score on that information.

        :param self:
        :param Text:   A UTF-8 text string containing the clinical content being examined for PHI entities. Each string must contain fewer than 20,000 bytes of characters. <br/>
        :return: PaginationToken:   If the result of the previous request to DetectPHI was truncated, include the Paginationtoken to fetch the next page of PHI entities. <br/>Entities:   The collection of PHI entities extracted from the input text and their associated information. For each entity, the response provides the entity text, the entity category, where the entity text begins and ends, and the level of confidence that Comprehend Medical has in its detection. <br/>        """
        pass



class ConfigService(BaseClient):
    """<fullname>AWS Config</fullname> AWS Config provides a way to keep track of the configurations of all the AWS resources associated with your AWS account. You can use AWS Config to get the current and historical configurations of each AWS resource and also to get information about the relationship between the resources. An AWS resource can be an Amazon Compute Cloud (Amazon EC2) instance, an Elastic Block Store (EBS) volume, an elastic network Interface (ENI), or a security group. For a complete list of resources currently supported by AWS Config, see <a href="https://docs.aws.amazon.com/config/latest/developerguide/resource-config-reference.html#supported-resources">Supported AWS Resources</a>. You can access and manage AWS Config through the AWS Management Console, the AWS Command Line Interface (AWS CLI), the AWS Config API, or the AWS SDKs for AWS Config. This reference guide contains documentation for the AWS Config API and the AWS CLI commands that you can use to manage AWS Config. The AWS Config API uses the Signature Version 4 protocol for signing requests. For more information about how to sign a request with this protocol, see <a href="https://docs.aws.amazon.com/general/latest/gr/signature-version-4.html">Signature Version 4 Signing Process</a>. For detailed information about AWS Config features and their associated actions or commands, as well as how to work with AWS Management Console, see <a href="https://docs.aws.amazon.com/config/latest/developerguide/WhatIsConfig.html">What Is AWS Config</a> in the <i>AWS Config Developer Guide</i>."""


    class DescribeComplianceByConfigRulePaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_compliance_by_config_rule` for valid parameters"""
            pass
    class DescribeComplianceByResourcePaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_compliance_by_resource` for valid parameters"""
            pass
    class DescribeConfigRulesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_config_rules` for valid parameters"""
            pass
    class GetComplianceDetailsByConfigRulePaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_compliance_details_by_config_rule` for valid parameters"""
            pass
    class GetComplianceDetailsByResourcePaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_compliance_details_by_resource` for valid parameters"""
            pass
    class GetResourceConfigHistoryPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_resource_config_history` for valid parameters"""
            pass
    class ListDiscoveredResourcesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_discovered_resources` for valid parameters"""
            pass
    class DescribeAggregateComplianceByConfigRulesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_aggregate_compliance_by_config_rules` for valid parameters"""
            pass
    class DescribeAggregationAuthorizationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_aggregation_authorizations` for valid parameters"""
            pass
    class DescribeConfigRuleEvaluationStatusPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_config_rule_evaluation_status` for valid parameters"""
            pass
    class DescribeConfigurationAggregatorSourcesStatusPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_configuration_aggregator_sources_status` for valid parameters"""
            pass
    class DescribeConfigurationAggregatorsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_configuration_aggregators` for valid parameters"""
            pass
    class DescribePendingAggregationRequestsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_pending_aggregation_requests` for valid parameters"""
            pass
    class DescribeRetentionConfigurationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_retention_configurations` for valid parameters"""
            pass
    class GetAggregateComplianceDetailsByConfigRulePaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_aggregate_compliance_details_by_config_rule` for valid parameters"""
            pass
    class ListAggregateDiscoveredResourcesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_aggregate_discovered_resources` for valid parameters"""
            pass
    class DescribeRemediationExecutionStatusPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_remediation_execution_status` for valid parameters"""
            pass


    class BatchGetAggregateResourceConfigRequest(Mapping):
        pass
    
    class BatchGetAggregateResourceConfigResponse(Mapping):
        pass
    
    class AggregateResourceIdentifier(Mapping):
        pass
    
    class BatchGetResourceConfigRequest(Mapping):
        pass
    
    class BatchGetResourceConfigResponse(Mapping):
        pass
    
    class ResourceKey(Mapping):
        pass
    
    class DeleteAggregationAuthorizationRequest(Mapping):
        pass
    
    class DeleteConfigRuleRequest(Mapping):
        pass
    
    class DeleteConfigurationAggregatorRequest(Mapping):
        pass
    
    class DeleteConfigurationRecorderRequest(Mapping):
        pass
    
    class DeleteDeliveryChannelRequest(Mapping):
        pass
    
    class DeleteEvaluationResultsRequest(Mapping):
        pass
    
    class DeleteEvaluationResultsResponse(Mapping):
        pass
    
    class DeleteOrganizationConfigRuleRequest(Mapping):
        pass
    
    class DeletePendingAggregationRequestRequest(Mapping):
        pass
    
    class DeleteRemediationConfigurationRequest(Mapping):
        pass
    
    class DeleteRemediationConfigurationResponse(Mapping):
        pass
    
    class DeleteRetentionConfigurationRequest(Mapping):
        pass
    
    class DeliverConfigSnapshotRequest(Mapping):
        pass
    
    class DeliverConfigSnapshotResponse(Mapping):
        pass
    
    class DescribeAggregateComplianceByConfigRulesRequest(Mapping):
        pass
    
    class DescribeAggregateComplianceByConfigRulesResponse(Mapping):
        pass
    
    class ConfigRuleComplianceFilters(Mapping):
        pass
    
    class DescribeAggregationAuthorizationsRequest(Mapping):
        pass
    
    class DescribeAggregationAuthorizationsResponse(Mapping):
        pass
    
    class DescribeComplianceByConfigRuleRequest(Mapping):
        pass
    
    class DescribeComplianceByConfigRuleResponse(Mapping):
        pass
    
    class StringWithCharLimit64(object):
        pass
    
    class ComplianceType(object):
        pass
    
    class DescribeComplianceByResourceRequest(Mapping):
        pass
    
    class DescribeComplianceByResourceResponse(Mapping):
        pass
    
    class DescribeConfigRuleEvaluationStatusRequest(Mapping):
        pass
    
    class DescribeConfigRuleEvaluationStatusResponse(Mapping):
        pass
    
    class DescribeConfigRulesRequest(Mapping):
        pass
    
    class DescribeConfigRulesResponse(Mapping):
        pass
    
    class DescribeConfigurationAggregatorSourcesStatusRequest(Mapping):
        pass
    
    class DescribeConfigurationAggregatorSourcesStatusResponse(Mapping):
        pass
    
    class AggregatedSourceStatusType(object):
        pass
    
    class DescribeConfigurationAggregatorsRequest(Mapping):
        pass
    
    class DescribeConfigurationAggregatorsResponse(Mapping):
        pass
    
    class ConfigurationAggregatorName(object):
        pass
    
    class DescribeConfigurationRecorderStatusRequest(Mapping):
        pass
    
    class DescribeConfigurationRecorderStatusResponse(Mapping):
        pass
    
    class RecorderName(object):
        pass
    
    class DescribeConfigurationRecordersRequest(Mapping):
        pass
    
    class DescribeConfigurationRecordersResponse(Mapping):
        pass
    
    class DescribeDeliveryChannelStatusRequest(Mapping):
        pass
    
    class DescribeDeliveryChannelStatusResponse(Mapping):
        pass
    
    class ChannelName(object):
        pass
    
    class DescribeDeliveryChannelsRequest(Mapping):
        pass
    
    class DescribeDeliveryChannelsResponse(Mapping):
        pass
    
    class DescribeOrganizationConfigRuleStatusesRequest(Mapping):
        pass
    
    class DescribeOrganizationConfigRuleStatusesResponse(Mapping):
        pass
    
    class DescribeOrganizationConfigRulesRequest(Mapping):
        pass
    
    class DescribeOrganizationConfigRulesResponse(Mapping):
        pass
    
    class DescribePendingAggregationRequestsRequest(Mapping):
        pass
    
    class DescribePendingAggregationRequestsResponse(Mapping):
        pass
    
    class DescribeRemediationConfigurationsRequest(Mapping):
        pass
    
    class DescribeRemediationConfigurationsResponse(Mapping):
        pass
    
    class DescribeRemediationExecutionStatusRequest(Mapping):
        pass
    
    class DescribeRemediationExecutionStatusResponse(Mapping):
        pass
    
    class DescribeRetentionConfigurationsRequest(Mapping):
        pass
    
    class DescribeRetentionConfigurationsResponse(Mapping):
        pass
    
    class RetentionConfigurationName(object):
        pass
    
    class GetAggregateComplianceDetailsByConfigRuleRequest(Mapping):
        pass
    
    class GetAggregateComplianceDetailsByConfigRuleResponse(Mapping):
        pass
    
    class GetAggregateConfigRuleComplianceSummaryRequest(Mapping):
        pass
    
    class GetAggregateConfigRuleComplianceSummaryResponse(Mapping):
        pass
    
    class ConfigRuleComplianceSummaryFilters(Mapping):
        pass
    
    class GetAggregateDiscoveredResourceCountsRequest(Mapping):
        pass
    
    class GetAggregateDiscoveredResourceCountsResponse(Mapping):
        pass
    
    class ResourceCountFilters(Mapping):
        pass
    
    class GetAggregateResourceConfigRequest(Mapping):
        pass
    
    class GetAggregateResourceConfigResponse(Mapping):
        pass
    
    class GetComplianceDetailsByConfigRuleRequest(Mapping):
        pass
    
    class GetComplianceDetailsByConfigRuleResponse(Mapping):
        pass
    
    class GetComplianceDetailsByResourceRequest(Mapping):
        pass
    
    class GetComplianceDetailsByResourceResponse(Mapping):
        pass
    
    class GetComplianceSummaryByConfigRuleResponse(Mapping):
        pass
    
    class GetComplianceSummaryByResourceTypeRequest(Mapping):
        pass
    
    class GetComplianceSummaryByResourceTypeResponse(Mapping):
        pass
    
    class StringWithCharLimit256(object):
        pass
    
    class GetDiscoveredResourceCountsRequest(Mapping):
        pass
    
    class GetDiscoveredResourceCountsResponse(Mapping):
        pass
    
    class GetOrganizationConfigRuleDetailedStatusRequest(Mapping):
        pass
    
    class GetOrganizationConfigRuleDetailedStatusResponse(Mapping):
        pass
    
    class StatusDetailFilters(Mapping):
        pass
    
    class GetResourceConfigHistoryRequest(Mapping):
        pass
    
    class GetResourceConfigHistoryResponse(Mapping):
        pass
    
    class LaterTime(object):
        pass
    
    class EarlierTime(object):
        pass
    
    class ListAggregateDiscoveredResourcesRequest(Mapping):
        pass
    
    class ListAggregateDiscoveredResourcesResponse(Mapping):
        pass
    
    class ResourceFilters(Mapping):
        pass
    
    class ListDiscoveredResourcesRequest(Mapping):
        pass
    
    class ListDiscoveredResourcesResponse(Mapping):
        pass
    
    class ResourceId(object):
        pass
    
    class Boolean(object):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class PutAggregationAuthorizationRequest(Mapping):
        pass
    
    class PutAggregationAuthorizationResponse(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class PutConfigRuleRequest(Mapping):
        pass
    
    class ConfigRule(Mapping):
        pass
    
    class PutConfigurationAggregatorRequest(Mapping):
        pass
    
    class PutConfigurationAggregatorResponse(Mapping):
        pass
    
    class AccountAggregationSource(Mapping):
        pass
    
    class OrganizationAggregationSource(Mapping):
        pass
    
    class PutConfigurationRecorderRequest(Mapping):
        pass
    
    class ConfigurationRecorder(Mapping):
        pass
    
    class PutDeliveryChannelRequest(Mapping):
        pass
    
    class DeliveryChannel(Mapping):
        pass
    
    class PutEvaluationsRequest(Mapping):
        pass
    
    class PutEvaluationsResponse(Mapping):
        pass
    
    class Evaluation(Mapping):
        pass
    
    class PutOrganizationConfigRuleRequest(Mapping):
        pass
    
    class PutOrganizationConfigRuleResponse(Mapping):
        pass
    
    class OrganizationManagedRuleMetadata(Mapping):
        pass
    
    class OrganizationCustomRuleMetadata(Mapping):
        pass
    
    class AccountId(object):
        pass
    
    class PutRemediationConfigurationsRequest(Mapping):
        pass
    
    class PutRemediationConfigurationsResponse(Mapping):
        pass
    
    class RemediationConfiguration(Mapping):
        pass
    
    class PutRetentionConfigurationRequest(Mapping):
        pass
    
    class PutRetentionConfigurationResponse(Mapping):
        pass
    
    class SelectResourceConfigRequest(Mapping):
        pass
    
    class SelectResourceConfigResponse(Mapping):
        pass
    
    class StartConfigRulesEvaluationRequest(Mapping):
        pass
    
    class StartConfigRulesEvaluationResponse(Mapping):
        pass
    
    class StartConfigurationRecorderRequest(Mapping):
        pass
    
    class StartRemediationExecutionRequest(Mapping):
        pass
    
    class StartRemediationExecutionResponse(Mapping):
        pass
    
    class StopConfigurationRecorderRequest(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    def batch_get_aggregate_resource_config(self, ResourceIdentifiers: List[AggregateResourceIdentifier], ConfigurationAggregatorName: str) -> BatchGetAggregateResourceConfigResponse:
        """Returns the current configuration items for resources that are present in your AWS Config aggregator. The operation also returns a list of resources that are not processed in the current request. If there are no unprocessed resources, the operation returns an empty <code>unprocessedResourceIdentifiers</code> list.  <note> <ul> <li> The API does not return results for deleted resources. </li> <li>  The API does not return tags and relationships. </li> </ul> </note>

        :param self:
        :param ResourceIdentifiers:  A list of aggregate ResourceIdentifiers objects. <br/>ConfigurationAggregatorName:  The name of the configuration aggregator.<br/>
        :return: UnprocessedResourceIdentifiers:  A list of resource identifiers that were not processed with current scope. The list is empty if all the resources are processed.<br/>BaseConfigurationItems:  A list that contains the current configuration of one or more resources.<br/>        """
        pass

    def batch_get_resource_config(self, resourceKeys: List[ResourceKey]) -> BatchGetResourceConfigResponse:
        """Returns the current configuration for one or more requested resources. The operation also returns a list of resources that are not processed in the current request. If there are no unprocessed resources, the operation returns an empty unprocessedResourceKeys list.  <note> <ul> <li> The API does not return results for deleted resources. </li> <li>  The API does not return any tags for the requested resources. This information is filtered out of the supplementaryConfiguration section of the API response. </li> </ul> </note>

        :param self:
        :param resourceKeys:  A list of resource keys to be processed with the current request. Each element in the list consists of the resource type and resource ID.<br/>
        :return: unprocessedResourceKeys:  A list of resource keys that were not processed with the current response. The unprocessesResourceKeys value is in the same form as ResourceKeys, so the value can be directly provided to a subsequent BatchGetResourceConfig operation. If there are no unprocessed resource keys, the response contains an empty unprocessedResourceKeys list. <br/>baseConfigurationItems:  A list that contains the current configuration of one or more resources.<br/>        """
        pass

    def delete_aggregation_authorization(self, AuthorizedAwsRegion: str, AuthorizedAccountId: str):
        """Deletes the authorization granted to the specified configuration aggregator account in a specified region.

        :param self:
        :param AuthorizedAwsRegion:  The region authorized to collect aggregated data.<br/>AuthorizedAccountId:  The 12-digit account ID of the account authorized to aggregate data.<br/>
        :return:         """
        pass

    def delete_config_rule(self, ConfigRuleName: str):
        """Deletes the specified AWS Config rule and all of its evaluation results. AWS Config sets the state of a rule to <code>DELETING</code> until the deletion is complete. You cannot update a rule while it is in this state. If you make a <code>PutConfigRule</code> or <code>DeleteConfigRule</code> request for the rule, you will receive a <code>ResourceInUseException</code>. You can check the state of a rule by using the <code>DescribeConfigRules</code> request.

        :param self:
        :param ConfigRuleName:  The name of the AWS Config rule that you want to delete.<br/>
        :return:         """
        pass

    def delete_configuration_aggregator(self, ConfigurationAggregatorName: str):
        """Deletes the specified configuration aggregator and the aggregated data associated with the aggregator.

        :param self:
        :param ConfigurationAggregatorName:  The name of the configuration aggregator.<br/>
        :return:         """
        pass

    def delete_configuration_recorder(self, ConfigurationRecorderName: str):
        """Deletes the configuration recorder. After the configuration recorder is deleted, AWS Config will not record resource configuration changes until you create a new configuration recorder. This action does not delete the configuration information that was previously recorded. You will be able to access the previously recorded information by using the <code>GetResourceConfigHistory</code> action, but you will not be able to access this information in the AWS Config console until you create a new configuration recorder.

        :param self:
        :param ConfigurationRecorderName:  The name of the configuration recorder to be deleted. You can retrieve the name of your configuration recorder by using the <code>DescribeConfigurationRecorders</code> action.<br/>
        :return:         """
        pass

    def delete_delivery_channel(self, DeliveryChannelName: str):
        """Deletes the delivery channel. Before you can delete the delivery channel, you must stop the configuration recorder by using the <a>StopConfigurationRecorder</a> action.

        :param self:
        :param DeliveryChannelName:  The name of the delivery channel to delete.<br/>
        :return:         """
        pass

    def delete_evaluation_results(self, ConfigRuleName: str) -> DeleteEvaluationResultsResponse:
        """Deletes the evaluation results for the specified AWS Config rule. You can specify one AWS Config rule per request. After you delete the evaluation results, you can call the <a>StartConfigRulesEvaluation</a> API to start evaluating your AWS resources against the rule.

        :param self:
        :param ConfigRuleName:  The name of the AWS Config rule for which you want to delete the evaluation results.<br/>
        :return:         """
        pass

    def delete_organization_config_rule(self, OrganizationConfigRuleName: str):
        """

        :param self:
        :param OrganizationConfigRuleName:  <br/>
        :return:         """
        pass

    def delete_pending_aggregation_request(self, RequesterAwsRegion: str, RequesterAccountId: str):
        """Deletes pending authorization requests for a specified aggregator account in a specified region.

        :param self:
        :param RequesterAwsRegion:  The region requesting to aggregate data.<br/>RequesterAccountId:  The 12-digit account ID of the account requesting to aggregate data.<br/>
        :return:         """
        pass

    def delete_remediation_configuration(self, ConfigRuleName: str, ResourceType: str=None) -> DeleteRemediationConfigurationResponse:
        """Deletes the remediation configuration.

        :param self:
        :param ResourceType:  The type of a resource.<br/>ConfigRuleName:  The name of the AWS Config rule for which you want to delete remediation configuration.<br/>
        :return:         """
        pass

    def delete_retention_configuration(self, RetentionConfigurationName: str):
        """Deletes the retention configuration.

        :param self:
        :param RetentionConfigurationName:  The name of the retention configuration to delete.<br/>
        :return:         """
        pass

    def deliver_config_snapshot(self, deliveryChannelName: str) -> DeliverConfigSnapshotResponse:
        """Schedules delivery of a configuration snapshot to the Amazon S3 bucket in the specified delivery channel. After the delivery has started, AWS Config sends the following notifications using an Amazon SNS topic that you have specified. <ul> <li> Notification of the start of the delivery. </li> <li> Notification of the completion of the delivery, if the delivery was successfully completed. </li> <li> Notification of delivery failure, if the delivery failed. </li> </ul>

        :param self:
        :param deliveryChannelName:  The name of the delivery channel through which the snapshot is delivered.<br/>
        :return: configSnapshotId:  The ID of the snapshot that is being created.<br/>        """
        pass

    def describe_aggregate_compliance_by_config_rules(self, ConfigurationAggregatorName: str, Filters: ConfigRuleComplianceFilters=None, Limit: int=None, NextToken: str=None) -> DescribeAggregateComplianceByConfigRulesResponse:
        """Returns a list of compliant and noncompliant rules with the number of resources for compliant and noncompliant rules.  <note> The results can return an empty result page, but if you have a nextToken, the results are displayed on the next page. </note>

        :param self:
        :param NextToken:  The nextToken string returned on a previous page that you use to get the next page of results in a paginated response.<br/>Limit:  The maximum number of evaluation results returned on each page. The default is maximum. If you specify 0, AWS Config uses the default.<br/>Filters:  Filters the results by ConfigRuleComplianceFilters object. :<br/>ConfigurationAggregatorName:  The name of the configuration aggregator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AwsRegion</i> The source region where the data is aggregated. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountId</i> The 12-digit account ID of the source account. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ComplianceType</i> The rule compliance status. For the <code>ConfigRuleComplianceFilters</code> data type, AWS Config supports only <code>COMPLIANT</code> and <code>NON_COMPLIANT</code>. AWS Config does not support the <code>NOT_APPLICABLE</code> and the <code>INSUFFICIENT_DATA</code> values.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigRuleName</i> The name of the AWS Config rule.<br/>
        :return: NextToken:  The nextToken string returned on a previous page that you use to get the next page of results in a paginated response.<br/>AggregateComplianceByConfigRules:  Returns a list of AggregateComplianceByConfigRule object.<br/>        """
        pass

    def describe_aggregation_authorizations(self, Limit: int=None, NextToken: str=None) -> DescribeAggregationAuthorizationsResponse:
        """Returns a list of authorizations granted to various aggregator accounts and regions.

        :param self:
        :param NextToken:  The nextToken string returned on a previous page that you use to get the next page of results in a paginated response.<br/>Limit:  The maximum number of AggregationAuthorizations returned on each page. The default is maximum. If you specify 0, AWS Config uses the default.<br/>
        :return: NextToken:  The nextToken string returned on a previous page that you use to get the next page of results in a paginated response.<br/>AggregationAuthorizations:  Returns a list of authorizations granted to various aggregator accounts and regions.<br/>        """
        pass

    def describe_compliance_by_config_rule(self, ConfigRuleNames: List[StringWithCharLimit64]=None, ComplianceTypes: List[ComplianceType]=None, NextToken: str=None) -> DescribeComplianceByConfigRuleResponse:
        """Indicates whether the specified AWS Config rules are compliant. If a rule is noncompliant, this action returns the number of AWS resources that do not comply with the rule. A rule is compliant if all of the evaluated resources comply with it. It is noncompliant if any of these resources do not comply. If AWS Config has no current evaluation results for the rule, it returns <code>INSUFFICIENT_DATA</code>. This result might indicate one of the following conditions: <ul> <li> AWS Config has never invoked an evaluation for the rule. To check whether it has, use the <code>DescribeConfigRuleEvaluationStatus</code> action to get the <code>LastSuccessfulInvocationTime</code> and <code>LastFailedInvocationTime</code>. </li> <li> The rule's AWS Lambda function is failing to send evaluation results to AWS Config. Verify that the role you assigned to your configuration recorder includes the <code>config:PutEvaluations</code> permission. If the rule is a custom rule, verify that the AWS Lambda execution role includes the <code>config:PutEvaluations</code> permission. </li> <li> The rule's AWS Lambda function has returned <code>NOT_APPLICABLE</code> for all evaluation results. This can occur if the resources were deleted or removed from the rule's scope. </li> </ul>

        :param self:
        :param NextToken:  The <code>nextToken</code> string returned on a previous page that you use to get the next page of results in a paginated response.<br/>ComplianceTypes:  Filters the results by compliance. The allowed values are <code>COMPLIANT</code> and <code>NON_COMPLIANT</code>.<br/>ConfigRuleNames:  Specify one or more AWS Config rule names to filter the results by rule.<br/>
        :return: NextToken:  The string that you use in a subsequent request to get the next page of results in a paginated response.<br/>ComplianceByConfigRules:  Indicates whether each of the specified AWS Config rules is compliant.<br/>        """
        pass

    def describe_compliance_by_resource(self, ResourceType: str=None, ResourceId: str=None, ComplianceTypes: List[ComplianceType]=None, Limit: int=None, NextToken: str=None) -> DescribeComplianceByResourceResponse:
        """Indicates whether the specified AWS resources are compliant. If a resource is noncompliant, this action returns the number of AWS Config rules that the resource does not comply with. A resource is compliant if it complies with all the AWS Config rules that evaluate it. It is noncompliant if it does not comply with one or more of these rules. If AWS Config has no current evaluation results for the resource, it returns <code>INSUFFICIENT_DATA</code>. This result might indicate one of the following conditions about the rules that evaluate the resource: <ul> <li> AWS Config has never invoked an evaluation for the rule. To check whether it has, use the <code>DescribeConfigRuleEvaluationStatus</code> action to get the <code>LastSuccessfulInvocationTime</code> and <code>LastFailedInvocationTime</code>. </li> <li> The rule's AWS Lambda function is failing to send evaluation results to AWS Config. Verify that the role that you assigned to your configuration recorder includes the <code>config:PutEvaluations</code> permission. If the rule is a custom rule, verify that the AWS Lambda execution role includes the <code>config:PutEvaluations</code> permission. </li> <li> The rule's AWS Lambda function has returned <code>NOT_APPLICABLE</code> for all evaluation results. This can occur if the resources were deleted or removed from the rule's scope. </li> </ul>

        :param self:
        :param NextToken:  The <code>nextToken</code> string returned on a previous page that you use to get the next page of results in a paginated response.<br/>Limit:  The maximum number of evaluation results returned on each page. The default is 10. You cannot specify a number greater than 100. If you specify 0, AWS Config uses the default.<br/>ComplianceTypes:  Filters the results by compliance. The allowed values are <code>COMPLIANT</code>, <code>NON_COMPLIANT</code>, and <code>INSUFFICIENT_DATA</code>.<br/>ResourceId:  The ID of the AWS resource for which you want compliance information. You can specify only one resource ID. If you specify a resource ID, you must also specify a type for <code>ResourceType</code>.<br/>ResourceType:  The types of AWS resources for which you want compliance information (for example, <code>AWS::EC2::Instance</code>). For this action, you can specify that the resource type is an AWS account by specifying <code>AWS::::Account</code>.<br/>
        :return: NextToken:  The string that you use in a subsequent request to get the next page of results in a paginated response.<br/>ComplianceByResources:  Indicates whether the specified AWS resource complies with all of the AWS Config rules that evaluate it.<br/>        """
        pass

    def describe_config_rule_evaluation_status(self, ConfigRuleNames: List[StringWithCharLimit64]=None, NextToken: str=None, Limit: int=None) -> DescribeConfigRuleEvaluationStatusResponse:
        """Returns status information for each of your AWS managed Config rules. The status includes information such as the last time AWS Config invoked the rule, the last time AWS Config failed to invoke the rule, and the related error for the last failure.

        :param self:
        :param Limit:  The number of rule evaluation results that you want returned. This parameter is required if the rule limit for your account is more than the default of 150 rules. For information about requesting a rule limit increase, see <a href="http://docs.aws.amazon.com/general/latest/gr/aws_service_limits.html#limits_config">AWS Config Limits</a> in the <i>AWS General Reference Guide</i>.<br/>NextToken:  The <code>nextToken</code> string returned on a previous page that you use to get the next page of results in a paginated response.<br/>ConfigRuleNames:  The name of the AWS managed Config rules for which you want status information. If you do not specify any names, AWS Config returns status information for all AWS managed Config rules that you use.<br/>
        :return: NextToken:  The string that you use in a subsequent request to get the next page of results in a paginated response.<br/>ConfigRulesEvaluationStatus:  Status information about your AWS managed Config rules.<br/>        """
        pass

    def describe_config_rules(self, ConfigRuleNames: List[StringWithCharLimit64]=None, NextToken: str=None) -> DescribeConfigRulesResponse:
        """Returns details about your AWS Config rules.

        :param self:
        :param NextToken:  The <code>nextToken</code> string returned on a previous page that you use to get the next page of results in a paginated response.<br/>ConfigRuleNames:  The names of the AWS Config rules for which you want details. If you do not specify any names, AWS Config returns details for all your rules.<br/>
        :return: NextToken:  The string that you use in a subsequent request to get the next page of results in a paginated response.<br/>ConfigRules:  The details about your AWS Config rules.<br/>        """
        pass

    def describe_configuration_aggregator_sources_status(self, ConfigurationAggregatorName: str, UpdateStatus: List[AggregatedSourceStatusType]=None, NextToken: str=None, Limit: int=None) -> DescribeConfigurationAggregatorSourcesStatusResponse:
        """Returns status information for sources within an aggregator. The status includes information about the last time AWS Config verified authorization between the source account and an aggregator account. In case of a failure, the status contains the related error code or message. 

        :param self:
        :param Limit:  The maximum number of AggregatorSourceStatus returned on each page. The default is maximum. If you specify 0, AWS Config uses the default.<br/>NextToken:  The nextToken string returned on a previous page that you use to get the next page of results in a paginated response.<br/>UpdateStatus:  Filters the status type. <ul> <li> Valid value FAILED indicates errors while moving data. </li> <li> Valid value SUCCEEDED indicates the data was successfully moved. </li> <li> Valid value OUTDATED indicates the data is not the most recent. </li> </ul><br/>ConfigurationAggregatorName:  The name of the configuration aggregator.<br/>
        :return: NextToken:  The nextToken string returned on a previous page that you use to get the next page of results in a paginated response.<br/>AggregatedSourceStatusList:  Returns an AggregatedSourceStatus object. <br/>        """
        pass

    def describe_configuration_aggregators(self, ConfigurationAggregatorNames: List[ConfigurationAggregatorName]=None, NextToken: str=None, Limit: int=None) -> DescribeConfigurationAggregatorsResponse:
        """Returns the details of one or more configuration aggregators. If the configuration aggregator is not specified, this action returns the details for all the configuration aggregators associated with the account. 

        :param self:
        :param Limit:  The maximum number of configuration aggregators returned on each page. The default is maximum. If you specify 0, AWS Config uses the default.<br/>NextToken:  The nextToken string returned on a previous page that you use to get the next page of results in a paginated response.<br/>ConfigurationAggregatorNames:  The name of the configuration aggregators.<br/>
        :return: NextToken:  The nextToken string returned on a previous page that you use to get the next page of results in a paginated response.<br/>ConfigurationAggregators:  Returns a ConfigurationAggregators object.<br/>        """
        pass

    def describe_configuration_recorder_status(self, ConfigurationRecorderNames: List[RecorderName]=None) -> DescribeConfigurationRecorderStatusResponse:
        """Returns the current status of the specified configuration recorder. If a configuration recorder is not specified, this action returns the status of all configuration recorders associated with the account. <note> Currently, you can specify only one configuration recorder per region in your account. </note>

        :param self:
        :param ConfigurationRecorderNames:  The name(s) of the configuration recorder. If the name is not specified, the action returns the current status of all the configuration recorders associated with the account.<br/>
        :return: ConfigurationRecordersStatus:  A list that contains status of the specified recorders.<br/>        """
        pass

    def describe_configuration_recorders(self, ConfigurationRecorderNames: List[RecorderName]=None) -> DescribeConfigurationRecordersResponse:
        """Returns the details for the specified configuration recorders. If the configuration recorder is not specified, this action returns the details for all configuration recorders associated with the account. <note> Currently, you can specify only one configuration recorder per region in your account. </note>

        :param self:
        :param ConfigurationRecorderNames:  A list of configuration recorder names.<br/>
        :return: ConfigurationRecorders:  A list that contains the descriptions of the specified configuration recorders.<br/>        """
        pass

    def describe_delivery_channel_status(self, DeliveryChannelNames: List[ChannelName]=None) -> DescribeDeliveryChannelStatusResponse:
        """Returns the current status of the specified delivery channel. If a delivery channel is not specified, this action returns the current status of all delivery channels associated with the account. <note> Currently, you can specify only one delivery channel per region in your account. </note>

        :param self:
        :param DeliveryChannelNames:  A list of delivery channel names.<br/>
        :return: DeliveryChannelsStatus:  A list that contains the status of a specified delivery channel.<br/>        """
        pass

    def describe_delivery_channels(self, DeliveryChannelNames: List[ChannelName]=None) -> DescribeDeliveryChannelsResponse:
        """Returns details about the specified delivery channel. If a delivery channel is not specified, this action returns the details of all delivery channels associated with the account. <note> Currently, you can specify only one delivery channel per region in your account. </note>

        :param self:
        :param DeliveryChannelNames:  A list of delivery channel names.<br/>
        :return: DeliveryChannels:  A list that contains the descriptions of the specified delivery channel.<br/>        """
        pass

    def describe_organization_config_rule_statuses(self, OrganizationConfigRuleNames: List[StringWithCharLimit64]=None, Limit: int=None, NextToken: str=None) -> DescribeOrganizationConfigRuleStatusesResponse:
        """

        :param self:
        :param NextToken:  <br/>Limit:  <br/>OrganizationConfigRuleNames:  <br/>
        :return: NextToken:  <br/>OrganizationConfigRuleStatuses:  <br/>        """
        pass

    def describe_organization_config_rules(self, OrganizationConfigRuleNames: List[StringWithCharLimit64]=None, Limit: int=None, NextToken: str=None) -> DescribeOrganizationConfigRulesResponse:
        """

        :param self:
        :param NextToken:  <br/>Limit:  <br/>OrganizationConfigRuleNames:  <br/>
        :return: NextToken:  <br/>OrganizationConfigRules:  <br/>        """
        pass

    def describe_pending_aggregation_requests(self, Limit: int=None, NextToken: str=None) -> DescribePendingAggregationRequestsResponse:
        """Returns a list of all pending aggregation requests.

        :param self:
        :param NextToken:  The nextToken string returned on a previous page that you use to get the next page of results in a paginated response.<br/>Limit:  The maximum number of evaluation results returned on each page. The default is maximum. If you specify 0, AWS Config uses the default.<br/>
        :return: NextToken:  The nextToken string returned on a previous page that you use to get the next page of results in a paginated response.<br/>PendingAggregationRequests:  Returns a PendingAggregationRequests object.<br/>        """
        pass

    def describe_remediation_configurations(self, ConfigRuleNames: List[StringWithCharLimit64]) -> DescribeRemediationConfigurationsResponse:
        """Returns the details of one or more remediation configurations.

        :param self:
        :param ConfigRuleNames:  A list of AWS Config rule names of remediation configurations for which you want details. <br/>
        :return: RemediationConfigurations:  Returns a remediation configuration object.<br/>        """
        pass

    def describe_remediation_execution_status(self, ConfigRuleName: str, ResourceKeys: List[ResourceKey]=None, Limit: int=None, NextToken: str=None) -> DescribeRemediationExecutionStatusResponse:
        """Provides a detailed view of a Remediation Execution for a set of resources including state, timestamps for when steps for the remediation execution occur, and any error messages for steps that have failed. When you specify the limit and the next token, you receive a paginated response.

        :param self:
        :param NextToken:  The <code>nextToken</code> string returned on a previous page that you use to get the next page of results in a paginated response.<br/>Limit:  The maximum number of RemediationExecutionStatuses returned on each page. The default is maximum. If you specify 0, AWS Config uses the default. <br/>ResourceKeys:  A list of resource keys to be processed with the current request. Each element in the list consists of the resource type and resource ID. <br/>ConfigRuleName:  A list of AWS Config rule names.<br/>
        :return: NextToken:  The <code>nextToken</code> string returned on a previous page that you use to get the next page of results in a paginated response.<br/>RemediationExecutionStatuses:  Returns a list of remediation execution statuses objects.<br/>        """
        pass

    def describe_retention_configurations(self, RetentionConfigurationNames: List[RetentionConfigurationName]=None, NextToken: str=None) -> DescribeRetentionConfigurationsResponse:
        """Returns the details of one or more retention configurations. If the retention configuration name is not specified, this action returns the details for all the retention configurations for that account. <note> Currently, AWS Config supports only one retention configuration per region in your account. </note>

        :param self:
        :param NextToken:  The <code>nextToken</code> string returned on a previous page that you use to get the next page of results in a paginated response. <br/>RetentionConfigurationNames:  A list of names of retention configurations for which you want details. If you do not specify a name, AWS Config returns details for all the retention configurations for that account. <note> Currently, AWS Config supports only one retention configuration per region in your account. </note><br/>
        :return: NextToken:  The <code>nextToken</code> string returned on a previous page that you use to get the next page of results in a paginated response. <br/>RetentionConfigurations:  Returns a retention configuration object.<br/>        """
        pass

    def get_aggregate_compliance_details_by_config_rule(self, AwsRegion: str, AccountId: str, ConfigRuleName: str, ConfigurationAggregatorName: str, ComplianceType: str=None, Limit: int=None, NextToken: str=None) -> GetAggregateComplianceDetailsByConfigRuleResponse:
        """Returns the evaluation results for the specified AWS Config rule for a specific resource in a rule. The results indicate which AWS resources were evaluated by the rule, when each resource was last evaluated, and whether each resource complies with the rule.  <note> The results can return an empty result page. But if you have a nextToken, the results are displayed on the next page. </note>

        :param self:
        :param NextToken:  The nextToken string returned on a previous page that you use to get the next page of results in a paginated response.<br/>Limit:  The maximum number of evaluation results returned on each page. The default is 50. You cannot specify a number greater than 100. If you specify 0, AWS Config uses the default.<br/>ComplianceType:  The resource compliance status. <note> For the <code>GetAggregateComplianceDetailsByConfigRuleRequest</code> data type, AWS Config supports only the <code>COMPLIANT</code> and <code>NON_COMPLIANT</code>. AWS Config does not support the <code>NOT_APPLICABLE</code> and <code>INSUFFICIENT_DATA</code> values. </note><br/>AwsRegion:  The source region from where the data is aggregated.<br/>AccountId:  The 12-digit account ID of the source account.<br/>ConfigRuleName:  The name of the AWS Config rule for which you want compliance information.<br/>ConfigurationAggregatorName:  The name of the configuration aggregator.<br/>
        :return: NextToken:  The nextToken string returned on a previous page that you use to get the next page of results in a paginated response.<br/>AggregateEvaluationResults:  Returns an AggregateEvaluationResults object.<br/>        """
        pass

    def get_aggregate_config_rule_compliance_summary(self, ConfigurationAggregatorName: str, Filters: ConfigRuleComplianceSummaryFilters=None, GroupByKey: str=None, Limit: int=None, NextToken: str=None) -> GetAggregateConfigRuleComplianceSummaryResponse:
        """Returns the number of compliant and noncompliant rules for one or more accounts and regions in an aggregator. <note> The results can return an empty result page, but if you have a nextToken, the results are displayed on the next page. </note>

        :param self:
        :param NextToken:  The nextToken string returned on a previous page that you use to get the next page of results in a paginated response.<br/>Limit:  The maximum number of evaluation results returned on each page. The default is 1000. You cannot specify a number greater than 1000. If you specify 0, AWS Config uses the default.<br/>GroupByKey:  Groups the result based on ACCOUNT_ID or AWS_REGION.<br/>Filters:  Filters the results based on the ConfigRuleComplianceSummaryFilters object.:<br/>ConfigurationAggregatorName:  The name of the configuration aggregator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AwsRegion</i> The source region where the data is aggregated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountId</i> The 12-digit account ID of the source account.<br/>
        :return: NextToken:  The nextToken string returned on a previous page that you use to get the next page of results in a paginated response.<br/>AggregateComplianceCounts:  Returns a list of AggregateComplianceCounts object.<br/>GroupByKey:  Groups the result based on ACCOUNT_ID or AWS_REGION.<br/>        """
        pass

    def get_aggregate_discovered_resource_counts(self, ConfigurationAggregatorName: str, Filters: ResourceCountFilters=None, GroupByKey: str=None, Limit: int=None, NextToken: str=None) -> GetAggregateDiscoveredResourceCountsResponse:
        """Returns the resource counts across accounts and regions that are present in your AWS Config aggregator. You can request the resource counts by providing filters and GroupByKey. For example, if the input contains accountID 12345678910 and region us-east-1 in filters, the API returns the count of resources in account ID 12345678910 and region us-east-1. If the input contains ACCOUNT_ID as a GroupByKey, the API returns resource counts for all source accounts that are present in your aggregator.

        :param self:
        :param NextToken:  The <code>nextToken</code> string returned on a previous page that you use to get the next page of results in a paginated response. <br/>Limit:  The maximum number of <a>GroupedResourceCount</a> objects returned on each page. The default is 1000. You cannot specify a number greater than 1000. If you specify 0, AWS Config uses the default.<br/>GroupByKey:  The key to group the resource counts.<br/>Filters:  Filters the results based on the <code>ResourceCountFilters</code> object.:<br/>ConfigurationAggregatorName:  The name of the configuration aggregator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Region</i> The region where the account is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountId</i> The 12-digit ID of the account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceType</i> The type of the AWS resource.<br/>
        :return: NextToken:  The <code>nextToken</code> string returned on a previous page that you use to get the next page of results in a paginated response.<br/>GroupedResourceCounts:  Returns a list of GroupedResourceCount objects.<br/>GroupByKey:  The key passed into the request object. If <code>GroupByKey</code> is not provided, the result will be empty.<br/>TotalDiscoveredResources:  The total number of resources that are present in an aggregator with the filters that you provide.<br/>        """
        pass

    def get_aggregate_resource_config(self, ResourceIdentifier: AggregateResourceIdentifier, ConfigurationAggregatorName: str) -> GetAggregateResourceConfigResponse:
        """Returns configuration item that is aggregated for your specific resource in a specific source account and region.

        :param self:
        :param ResourceIdentifier:  An object that identifies aggregate resource.:<br/>ConfigurationAggregatorName:  The name of the configuration aggregator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceName</i> The name of the AWS resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceType</i> The type of the AWS resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceId</i> The ID of the AWS resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceRegion</i> The source region where data is aggregated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceAccountId</i> The 12-digit account ID of the source account.<br/>
        :return: ConfigurationItem:  Returns a <code>ConfigurationItem</code> object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>supplementaryConfiguration</i> Configuration attributes that AWS Config returns for certain resource types to supplement the information returned for the <code>configuration</code> parameter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>configuration</i> The description of the resource configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>relationships</i> A list of related AWS resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>relatedEvents</i> A list of CloudTrail event IDs. A populated field indicates that the current configuration was initiated by the events recorded in the CloudTrail log. For more information about CloudTrail, see <a href="https://docs.aws.amazon.com/awscloudtrail/latest/userguide/what_is_cloud_trail_top_level.html">What Is AWS CloudTrail</a>. An empty field indicates that the current configuration was not initiated by any event.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> A mapping of key value tags associated with the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceCreationTime</i> The time stamp when the resource was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>availabilityZone</i> The Availability Zone associated with the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>awsRegion</i> The region where the resource resides.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceName</i> The custom name of the resource, if available.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceId</i> The ID of the resource (for example, <code>sg-xxxxxx</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceType</i> The type of AWS resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> accoun<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>configurationItemMD5Hash</i> Unique MD5 hash that represents the configuration item's state. You can use MD5 hash to compare the states of two or more configuration items that are associated with the same resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>configurationStateId</i> An identifier that indicates the ordering of the configuration items of a resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>configurationItemStatus</i> The configuration item status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>configurationItemCaptureTime</i> The time when the configuration recording was initiated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>accountId</i> The 12-digit AWS account ID associated with the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version number of the resource configuration.<br/>        """
        pass

    def get_compliance_details_by_config_rule(self, ConfigRuleName: str, ComplianceTypes: List[ComplianceType]=None, Limit: int=None, NextToken: str=None) -> GetComplianceDetailsByConfigRuleResponse:
        """Returns the evaluation results for the specified AWS Config rule. The results indicate which AWS resources were evaluated by the rule, when each resource was last evaluated, and whether each resource complies with the rule.

        :param self:
        :param NextToken:  The <code>nextToken</code> string returned on a previous page that you use to get the next page of results in a paginated response.<br/>Limit:  The maximum number of evaluation results returned on each page. The default is 10. You cannot specify a number greater than 100. If you specify 0, AWS Config uses the default.<br/>ComplianceTypes:  Filters the results by compliance. The allowed values are <code>COMPLIANT</code>, <code>NON_COMPLIANT</code>, and <code>NOT_APPLICABLE</code>.<br/>ConfigRuleName:  The name of the AWS Config rule for which you want compliance information.<br/>
        :return: NextToken:  The string that you use in a subsequent request to get the next page of results in a paginated response.<br/>EvaluationResults:  Indicates whether the AWS resource complies with the specified AWS Config rule.<br/>        """
        pass

    def get_compliance_details_by_resource(self, ResourceId: str, ResourceType: str, ComplianceTypes: List[ComplianceType]=None, NextToken: str=None) -> GetComplianceDetailsByResourceResponse:
        """Returns the evaluation results for the specified AWS resource. The results indicate which AWS Config rules were used to evaluate the resource, when each rule was last used, and whether the resource complies with each rule.

        :param self:
        :param NextToken:  The <code>nextToken</code> string returned on a previous page that you use to get the next page of results in a paginated response.<br/>ComplianceTypes:  Filters the results by compliance. The allowed values are <code>COMPLIANT</code>, <code>NON_COMPLIANT</code>, and <code>NOT_APPLICABLE</code>.<br/>ResourceId:  The ID of the AWS resource for which you want compliance information.<br/>ResourceType:  The type of the AWS resource for which you want compliance information.<br/>
        :return: NextToken:  The string that you use in a subsequent request to get the next page of results in a paginated response.<br/>EvaluationResults:  Indicates whether the specified AWS resource complies each AWS Config rule.<br/>        """
        pass

    def get_compliance_summary_by_config_rule(self) -> GetComplianceSummaryByConfigRuleResponse:
        """Returns the number of AWS Config rules that are compliant and noncompliant, up to a maximum of 25 for each.

        :param self:
        :param 
        :return: ComplianceSummary:  The number of AWS Config rules that are compliant and the number that are noncompliant, up to a maximum of 25 for each.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ComplianceSummaryTimestamp</i> The time that AWS Config created the compliance summary.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NonCompliantResourceCount</i> The number of AWS Config rules or AWS resources that are noncompliant, up to a maximum of 25 for rules and 100 for resources.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NonCompliantResourceCount</b>: The number of AWS Config rules or AWS resources that are noncompliant, up to a maximum of 25 for rules and 100 for resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompliantResourceCount</i> The number of AWS Config rules or AWS resources that are compliant, up to a maximum of 25 for rules and 100 for resources.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CompliantResourceCount</b>: The number of AWS Config rules or AWS resources that are compliant, up to a maximum of 25 for rules and 100 for resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CapExceeded</i> Indicates whether the maximum count is reached.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CappedCount</i> The number of AWS resources or AWS Config rules responsible for the current compliance of the item.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CapExceeded</i> Indicates whether the maximum count is reached.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CappedCount</i> The number of AWS resources or AWS Config rules responsible for the current compliance of the item.<br/>        """
        pass

    def get_compliance_summary_by_resource_type(self, ResourceTypes: List[StringWithCharLimit256]=None) -> GetComplianceSummaryByResourceTypeResponse:
        """Returns the number of resources that are compliant and the number that are noncompliant. You can specify one or more resource types to get these numbers for each resource type. The maximum number returned is 100.

        :param self:
        :param ResourceTypes:  Specify one or more resource types to get the number of resources that are compliant and the number that are noncompliant for each resource type. For this request, you can specify an AWS resource type such as <code>AWS::EC2::Instance</code>. You can specify that the resource type is an AWS account by specifying <code>AWS::::Account</code>.<br/>
        :return: ComplianceSummariesByResourceType:  The number of resources that are compliant and the number that are noncompliant. If one or more resource types were provided with the request, the numbers are returned for each resource type. The maximum number returned is 100.<br/>        """
        pass

    def get_discovered_resource_counts(self, resourceTypes: List[StringWithCharLimit256]=None, limit: int=None, nextToken: str=None) -> GetDiscoveredResourceCountsResponse:
        """Returns the resource types, the number of each resource type, and the total number of resources that AWS Config is recording in this region for your AWS account.  <p class="title"> <b>Example</b>  <ol> <li> AWS Config is recording three resource types in the US East (Ohio) Region for your account: 25 EC2 instances, 20 IAM users, and 15 S3 buckets. </li> <li> You make a call to the <code>GetDiscoveredResourceCounts</code> action and specify that you want all resource types.  </li> <li> AWS Config returns the following: <ul> <li> The resource types (EC2 instances, IAM users, and S3 buckets). </li> <li> The number of each resource type (25, 20, and 15). </li> <li> The total number of all resources (60). </li> </ul> </li> </ol> The response is paginated. By default, AWS Config lists 100 <a>ResourceCount</a> objects on each page. You can customize this number with the <code>limit</code> parameter. The response includes a <code>nextToken</code> string. To get the next page of results, run the request again and specify the string for the <code>nextToken</code> parameter. <note> If you make a call to the <a>GetDiscoveredResourceCounts</a> action, you might not immediately receive resource counts in the following situations: <ul> <li> You are a new AWS Config customer. </li> <li> You just enabled resource recording. </li> </ul> It might take a few minutes for AWS Config to record and count your resources. Wait a few minutes and then retry the <a>GetDiscoveredResourceCounts</a> action.  </note>

        :param self:
        :param nextToken:  The <code>nextToken</code> string returned on a previous page that you use to get the next page of results in a paginated response.<br/>limit:  The maximum number of <a>ResourceCount</a> objects returned on each page. The default is 100. You cannot specify a number greater than 100. If you specify 0, AWS Config uses the default.<br/>resourceTypes:  The comma-separated list that specifies the resource types that you want AWS Config to return (for example, <code>"AWS::EC2::Instance"</code>, <code>"AWS::IAM::User"</code>). If a value for <code>resourceTypes</code> is not specified, AWS Config returns all resource types that AWS Config is recording in the region for your account. <note> If the configuration recorder is turned off, AWS Config returns an empty list of <a>ResourceCount</a> objects. If the configuration recorder is not recording a specific resource type (for example, S3 buckets), that resource type is not returned in the list of <a>ResourceCount</a> objects. </note><br/>
        :return: nextToken:  The string that you use in a subsequent request to get the next page of results in a paginated response.<br/>resourceCounts:  The list of <code>ResourceCount</code> objects. Each object is listed in descending order by the number of resources.<br/>totalDiscoveredResources:  The total number of resources that AWS Config is recording in the region for your account. If you specify resource types in the request, AWS Config returns only the total number of resources for those resource types. <p class="title"> <b>Example</b>  <ol> <li> AWS Config is recording three resource types in the US East (Ohio) Region for your account: 25 EC2 instances, 20 IAM users, and 15 S3 buckets, for a total of 60 resources. </li> <li> You make a call to the <code>GetDiscoveredResourceCounts</code> action and specify the resource type, <code>"AWS::EC2::Instances"</code>, in the request. </li> <li> AWS Config returns 25 for <code>totalDiscoveredResources</code>. </li> </ol><br/>        """
        pass

    def get_organization_config_rule_detailed_status(self, OrganizationConfigRuleName: str, Filters: StatusDetailFilters=None, Limit: int=None, NextToken: str=None) -> GetOrganizationConfigRuleDetailedStatusResponse:
        """

        :param self:
        :param NextToken:  <br/>Limit:  <br/>Filters:  :<br/>OrganizationConfigRuleName:  <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MemberAccountRuleStatus</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountId</i> <br/>
        :return: NextToken:  <br/>OrganizationConfigRuleDetailedStatus:  <br/>        """
        pass

    def get_resource_config_history(self, resourceId: str, resourceType: str, laterTime: LaterTime=None, earlierTime: EarlierTime=None, chronologicalOrder: str=None, limit: int=None, nextToken: str=None) -> GetResourceConfigHistoryResponse:
        """Returns a list of configuration items for the specified resource. The list contains details about each state of the resource during the specified time interval. If you specified a retention period to retain your <code>ConfigurationItems</code> between a minimum of 30 days and a maximum of 7 years (2557 days), AWS Config returns the <code>ConfigurationItems</code> for the specified retention period.  The response is paginated. By default, AWS Config returns a limit of 10 configuration items per page. You can customize this number with the <code>limit</code> parameter. The response includes a <code>nextToken</code> string. To get the next page of results, run the request again and specify the string for the <code>nextToken</code> parameter. <note> Each call to the API is limited to span a duration of seven days. It is likely that the number of records returned is smaller than the specified <code>limit</code>. In such cases, you can make another call, using the <code>nextToken</code>. </note>

        :param self:
        :param nextToken:  The <code>nextToken</code> string returned on a previous page that you use to get the next page of results in a paginated response.<br/>limit:  The maximum number of configuration items returned on each page. The default is 10. You cannot specify a number greater than 100. If you specify 0, AWS Config uses the default.<br/>chronologicalOrder:  The chronological order for configuration items listed. By default, the results are listed in reverse chronological order.<br/>earlierTime:  The time stamp that indicates an earlier time. If not specified, the action returns paginated results that contain configuration items that start when the first configuration item was recorded.<br/>laterTime:  The time stamp that indicates a later time. If not specified, current time is taken.<br/>resourceId:  The ID of the resource (for example., <code>sg-xxxxxx</code>).<br/>resourceType:  The resource type.<br/>
        :return: nextToken:  The string that you use in a subsequent request to get the next page of results in a paginated response.<br/>configurationItems:  A list that contains the configuration history of one or more resources.<br/>        """
        pass

    def list_aggregate_discovered_resources(self, ResourceType: str, ConfigurationAggregatorName: str, Filters: ResourceFilters=None, Limit: int=None, NextToken: str=None) -> ListAggregateDiscoveredResourcesResponse:
        """Accepts a resource type and returns a list of resource identifiers that are aggregated for a specific resource type across accounts and regions. A resource identifier includes the resource type, ID, (if available) the custom resource name, source account, and source region. You can narrow the results to include only resources that have specific resource IDs, or a resource name, or source account ID, or source region. For example, if the input consists of accountID 12345678910 and the region is us-east-1 for resource type <code>AWS::EC2::Instance</code> then the API returns all the EC2 instance identifiers of accountID 12345678910 and region us-east-1.

        :param self:
        :param NextToken:  The <code>nextToken</code> string returned on a previous page that you use to get the next page of results in a paginated response.<br/>Limit:  The maximum number of resource identifiers returned on each page. The default is 100. You cannot specify a number greater than 100. If you specify 0, AWS Config uses the default.<br/>Filters:  Filters the results based on the <code>ResourceFilters</code> object.:<br/>ResourceType:  The type of resources that you want AWS Config to list in the response.<br/>ConfigurationAggregatorName:  The name of the configuration aggregator. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Region</i> The source region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceName</i> The name of the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceId</i> The ID of the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountId</i> The 12-digit source account ID.<br/>
        :return: NextToken:  The <code>nextToken</code> string returned on a previous page that you use to get the next page of results in a paginated response.<br/>ResourceIdentifiers:  Returns a list of <code>ResourceIdentifiers</code> objects.<br/>        """
        pass

    def list_discovered_resources(self, resourceType: str, resourceIds: List[ResourceId]=None, resourceName: str=None, limit: int=None, includeDeletedResources: Boolean=None, nextToken: str=None) -> ListDiscoveredResourcesResponse:
        """Accepts a resource type and returns a list of resource identifiers for the resources of that type. A resource identifier includes the resource type, ID, and (if available) the custom resource name. The results consist of resources that AWS Config has discovered, including those that AWS Config is not currently recording. You can narrow the results to include only resources that have specific resource IDs or a resource name. <note> You can specify either resource IDs or a resource name, but not both, in the same request. </note> The response is paginated. By default, AWS Config lists 100 resource identifiers on each page. You can customize this number with the <code>limit</code> parameter. The response includes a <code>nextToken</code> string. To get the next page of results, run the request again and specify the string for the <code>nextToken</code> parameter.

        :param self:
        :param nextToken:  The <code>nextToken</code> string returned on a previous page that you use to get the next page of results in a paginated response.<br/>includeDeletedResources:  Specifies whether AWS Config includes deleted resources in the results. By default, deleted resources are not included.<br/>limit:  The maximum number of resource identifiers returned on each page. The default is 100. You cannot specify a number greater than 100. If you specify 0, AWS Config uses the default.<br/>resourceName:  The custom name of only those resources that you want AWS Config to list in the response. If you do not specify this parameter, AWS Config lists all resources of the specified type that it has discovered.<br/>resourceIds:  The IDs of only those resources that you want AWS Config to list in the response. If you do not specify this parameter, AWS Config lists all resources of the specified type that it has discovered.<br/>resourceType:  The type of resources that you want AWS Config to list in the response.<br/>
        :return: nextToken:  The string that you use in a subsequent request to get the next page of results in a paginated response.<br/>resourceIdentifiers:  The details that identify a resource that is discovered by AWS Config, including the resource type, ID, and (if available) the custom resource name.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceArn: str, Limit: int=None, NextToken: str=None) -> ListTagsForResourceResponse:
        """List the tags for AWS Config resource.

        :param self:
        :param NextToken:  The nextToken string returned on a previous page that you use to get the next page of results in a paginated response. <br/>Limit:  The maximum number of tags returned on each page. The limit maximum is 50. You cannot specify a number greater than 50. If you specify 0, AWS Config uses the default. <br/>ResourceArn:  The Amazon Resource Name (ARN) that identifies the resource for which to list the tags. Currently, the supported resources are <code>ConfigRule</code>, <code>ConfigurationAggregator</code> and <code>AggregatorAuthorization</code>.<br/>
        :return: NextToken:  The nextToken string returned on a previous page that you use to get the next page of results in a paginated response. <br/>Tags:  The tags for the resource.<br/>        """
        pass

    def put_aggregation_authorization(self, AuthorizedAwsRegion: str, AuthorizedAccountId: str, Tags: List[Tag]=None) -> PutAggregationAuthorizationResponse:
        """Authorizes the aggregator account and region to collect data from the source account and region. 

        :param self:
        :param Tags:  <br/>AuthorizedAwsRegion:  The region authorized to collect aggregated data.<br/>AuthorizedAccountId:  The 12-digit account ID of the account authorized to aggregate data.<br/>
        :return: AggregationAuthorization:  Returns an AggregationAuthorization object. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The time stamp when the aggregation authorization was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthorizedAwsRegion</i> The region authorized to collect aggregated data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthorizedAccountId</i> The 12-digit account ID of the account authorized to aggregate data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AggregationAuthorizationArn</i> The Amazon Resource Name (ARN) of the aggregation object.<br/>        """
        pass

    def put_config_rule(self, ConfigRule: ConfigRule, Tags: List[Tag]=None):
        """Adds or updates an AWS Config rule for evaluating whether your AWS resources comply with your desired configurations. You can use this action for custom AWS Config rules and AWS managed Config rules. A custom AWS Config rule is a rule that you develop and maintain. An AWS managed Config rule is a customizable, predefined rule that AWS Config provides. If you are adding a new custom AWS Config rule, you must first create the AWS Lambda function that the rule invokes to evaluate your resources. When you use the <code>PutConfigRule</code> action to add the rule to AWS Config, you must specify the Amazon Resource Name (ARN) that AWS Lambda assigns to the function. Specify the ARN for the <code>SourceIdentifier</code> key. This key is part of the <code>Source</code> object, which is part of the <code>ConfigRule</code> object.  If you are adding an AWS managed Config rule, specify the rule's identifier for the <code>SourceIdentifier</code> key. To reference AWS managed Config rule identifiers, see <a href="https://docs.aws.amazon.com/config/latest/developerguide/evaluate-config_use-managed-rules.html">About AWS Managed Config Rules</a>. For any new rule that you add, specify the <code>ConfigRuleName</code> in the <code>ConfigRule</code> object. Do not specify the <code>ConfigRuleArn</code> or the <code>ConfigRuleId</code>. These values are generated by AWS Config for new rules. If you are updating a rule that you added previously, you can specify the rule by <code>ConfigRuleName</code>, <code>ConfigRuleId</code>, or <code>ConfigRuleArn</code> in the <code>ConfigRule</code> data type that you use in this request. The maximum number of rules that AWS Config supports is 150. For information about requesting a rule limit increase, see <a href="http://docs.aws.amazon.com/general/latest/gr/aws_service_limits.html#limits_config">AWS Config Limits</a> in the <i>AWS General Reference Guide</i>. For more information about developing and using AWS Config rules, see <a href="https://docs.aws.amazon.com/config/latest/developerguide/evaluate-config.html">Evaluating AWS Resource Configurations with AWS Config</a> in the <i>AWS Config Developer Guide</i>.

        :param self:
        :param Tags:  <br/>ConfigRule:  The rule that you want to add to your account.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedBy</i> Service principal name of the service that created the rule. <note> The field is populated only if the service linked rule is created by a service. The field is empty if you create your own rule. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigRuleState</i> Indicates whether the AWS Config rule is active or is currently being deleted by AWS Config. It can also indicate the evaluation status for the AWS Config rule. AWS Config sets the state of the rule to <code>EVALUATING</code> temporarily after you use the <code>StartConfigRulesEvaluation</code> request to evaluate your resources against the AWS Config rule. AWS Config sets the state of the rule to <code>DELETING_RESULTS</code> temporarily after you use the <code>DeleteEvaluationResults</code> request to delete the current evaluation results for the AWS Config rule. AWS Config temporarily sets the state of a rule to <code>DELETING</code> after you use the <code>DeleteConfigRule</code> request to delete the rule. After AWS Config deletes the rule, the rule and all of its evaluations are erased and are no longer available.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumExecutionFrequency</i> The maximum frequency with which AWS Config runs evaluations for a rule. You can specify a value for <code>MaximumExecutionFrequency</code> when: <ul> <li> You are using an AWS managed rule that is triggered at a periodic frequency. </li> <li> Your custom rule is triggered when AWS Config delivers the configuration snapshot. For more information, see <a>ConfigSnapshotDeliveryProperties</a>. </li> </ul> <note> By default, rules with a periodic trigger are evaluated every 24 hours. To change the frequency, specify a valid value for the <code>MaximumExecutionFrequency</code> parameter. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputParameters</i> A string, in JSON format, that is passed to the AWS Config rule Lambda function.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Source</i> Provides the rule owner (AWS or customer), the rule identifier, and the notifications that cause the function to evaluate your AWS resources.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Source</b>: Provides the rule owner (AWS or customer), the rule identifier, and the notifications that cause the function to evaluate your AWS resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Scope</i> Defines which resources can trigger an evaluation for the rule. The scope can include one or more resource types, a combination of one resource type and one resource ID, or a combination of a tag key and value. Specify a scope to constrain the resources that can trigger an evaluation for the rule. If you do not specify a scope, evaluations are triggered when any resource in the recording group changes.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Scope</b>: Defines which resources can trigger an evaluation for the rule. The scope can include one or more resource types, a combination of one resource type and one resource ID, or a combination of a tag key and value. Specify a scope to constrain the resources that can trigger an evaluation for the rule. If you do not specify a scope, evaluations are triggered when any resource in the recording group changes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description that you provide for the AWS Config rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigRuleId</i> The ID of the AWS Config rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigRuleArn</i> The Amazon Resource Name (ARN) of the AWS Config rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigRuleName</i> The name that you assign to the AWS Config rule. The name is required if you are adding a new rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ComplianceResourceId</i> The ID of the only AWS resource that you want to trigger an evaluation for the rule. If you specify a resource ID, you must specify one resource type for <code>ComplianceResourceTypes</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TagValue</i> The tag value applied to only those AWS resources that you want to trigger an evaluation for the rule. If you specify a value for <code>TagValue</code>, you must also specify a value for <code>TagKey</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TagKey</i> The tag key that is applied to only those AWS resources that you want to trigger an evaluation for the rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ComplianceResourceTypes</i> The resource types of only those AWS resources that you want to trigger an evaluation for the rule. You can only specify one type if you also specify a resource ID for <code>ComplianceResourceId</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceDetails</i> Provides the source and type of the event that causes AWS Config to evaluate your AWS resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceIdentifier</i> For AWS Config managed rules, a predefined identifier from a list. For example, <code>IAM_PASSWORD_POLICY</code> is a managed rule. To reference a managed rule, see <a href="https://docs.aws.amazon.com/config/latest/developerguide/evaluate-config_use-managed-rules.html">Using AWS Managed Config Rules</a>. For custom rules, the identifier is the Amazon Resource Name (ARN) of the rule's AWS Lambda function, such as <code>arn:aws:lambda:us-east-2:123456789012:function:custom_rule_name</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Owner</i> Indicates whether AWS or the customer owns and manages the AWS Config rule.<br/>
        :return:         """
        pass

    def put_configuration_aggregator(self, ConfigurationAggregatorName: str, AccountAggregationSources: List[AccountAggregationSource]=None, OrganizationAggregationSource: OrganizationAggregationSource=None, Tags: List[Tag]=None) -> PutConfigurationAggregatorResponse:
        """Creates and updates the configuration aggregator with the selected source accounts and regions. The source account can be individual account(s) or an organization. <note> AWS Config should be enabled in source accounts and regions you want to aggregate. If your source type is an organization, you must be signed in to the master account and all features must be enabled in your organization. AWS Config calls <code>EnableAwsServiceAccess</code> API to enable integration between AWS Config and AWS Organizations.  </note>

        :param self:
        :param Tags:  <br/>OrganizationAggregationSource:  An OrganizationAggregationSource object.:<br/>AccountAggregationSources:  A list of AccountAggregationSource object. <br/>ConfigurationAggregatorName:  The name of the configuration aggregator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllAwsRegions</i> If true, aggregate existing AWS Config regions and future regions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AwsRegions</i> The source regions being aggregated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> ARN of the IAM role used to retrieve AWS Organization details associated with the aggregator account.<br/>
        :return: ConfigurationAggregator:  Returns a ConfigurationAggregator object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdatedTime</i> The time of the last update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The time stamp when the configuration aggregator was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationAggregationSource</i> Provides an organization and list of regions to be aggregated.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>OrganizationAggregationSource</b>: Provides an organization and list of regions to be aggregated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountAggregationSources</i> Provides a list of source accounts and regions to be aggregated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigurationAggregatorArn</i> The Amazon Resource Name (ARN) of the aggregator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigurationAggregatorName</i> The name of the aggregator.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllAwsRegions</i> If true, aggregate existing AWS Config regions and future regions.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AwsRegions</i> The source regions being aggregated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> ARN of the IAM role used to retrieve AWS Organization details associated with the aggregator account.<br/>        """
        pass

    def put_configuration_recorder(self, ConfigurationRecorder: ConfigurationRecorder):
        """Creates a new configuration recorder to record the selected resource configurations. You can use this action to change the role <code>roleARN</code> or the <code>recordingGroup</code> of an existing recorder. To change the role, call the action on the existing configuration recorder and specify a role. <note> Currently, you can specify only one configuration recorder per region in your account. If <code>ConfigurationRecorder</code> does not have the <b>recordingGroup</b> parameter specified, the default is to record all supported resource types. </note>

        :param self:
        :param ConfigurationRecorder:  The configuration recorder object that records each configuration change made to the resources.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>recordingGroup</i> Specifies the types of AWS resources for which AWS Config records configuration changes.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>recordingGroup</b>: Specifies the types of AWS resources for which AWS Config records configuration changes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleARN</i> Amazon Resource Name (ARN) of the IAM role used to describe the AWS resources associated with the account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the recorder. By default, AWS Config automatically assigns the name "default" when creating the configuration recorder. You cannot change the assigned name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceTypes</i> A comma-separated list that specifies the types of AWS resources for which AWS Config records configuration changes (for example, <code>AWS::EC2::Instance</code> or <code>AWS::CloudTrail::Trail</code>). Before you can set this option to <code>true</code>, you must set the <code>allSupported</code> option to <code>false</code>. If you set this option to <code>true</code>, when AWS Config adds support for a new type of resource, it will not record resources of that type unless you manually add that type to your recording group. For a list of valid <code>resourceTypes</code> values, see the <b>resourceType Value</b> column in <a href="https://docs.aws.amazon.com/config/latest/developerguide/resource-config-reference.html#supported-resources">Supported AWS Resource Types</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>includeGlobalResourceTypes</i> Specifies whether AWS Config includes all supported types of global resources (for example, IAM resources) with the resources that it records. Before you can set this option to <code>true</code>, you must set the <code>allSupported</code> option to <code>true</code>. If you set this option to <code>true</code>, when AWS Config adds support for a new type of global resource, it starts recording resources of that type automatically. The configuration details for any global resource are the same in all regions. To prevent duplicate configuration items, you should consider customizing AWS Config in only one region to record global resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>allSupported</i> Specifies whether AWS Config records configuration changes for every supported type of regional resource. If you set this option to <code>true</code>, when AWS Config adds support for a new type of regional resource, it starts recording resources of that type automatically. If you set this option to <code>true</code>, you cannot enumerate a list of <code>resourceTypes</code>.<br/>
        :return:         """
        pass

    def put_delivery_channel(self, DeliveryChannel: DeliveryChannel):
        """Creates a delivery channel object to deliver configuration information to an Amazon S3 bucket and Amazon SNS topic. Before you can create a delivery channel, you must create a configuration recorder. You can use this action to change the Amazon S3 bucket or an Amazon SNS topic of the existing delivery channel. To change the Amazon S3 bucket or an Amazon SNS topic, call this action and specify the changed values for the S3 bucket and the SNS topic. If you specify a different value for either the S3 bucket or the SNS topic, this action will keep the existing value for the parameter that is not changed. <note> You can have only one delivery channel per region in your account. </note>

        :param self:
        :param DeliveryChannel:  The configuration delivery channel object that delivers the configuration information to an Amazon S3 bucket and to an Amazon SNS topic.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>configSnapshotDeliveryProperties</i> The options for how often AWS Config delivers configuration snapshots to the Amazon S3 bucket.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>configSnapshotDeliveryProperties</b>: The options for how often AWS Config delivers configuration snapshots to the Amazon S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>snsTopicARN</i> The Amazon Resource Name (ARN) of the Amazon SNS topic to which AWS Config sends notifications about configuration changes. If you choose a topic from another account, the topic must have policies that grant access permissions to AWS Config. For more information, see <a href="https://docs.aws.amazon.com/config/latest/developerguide/sns-topic-policy.html">Permissions for the Amazon SNS Topic</a> in the AWS Config Developer Guide.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>s3KeyPrefix</i> The prefix for the specified Amazon S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>s3BucketName</i> The name of the Amazon S3 bucket to which AWS Config delivers configuration snapshots and configuration history files. If you specify a bucket that belongs to another AWS account, that bucket must have policies that grant access permissions to AWS Config. For more information, see <a href="https://docs.aws.amazon.com/config/latest/developerguide/s3-bucket-policy.html">Permissions for the Amazon S3 Bucket</a> in the AWS Config Developer Guide.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the delivery channel. By default, AWS Config assigns the name "default" when creating the delivery channel. To change the delivery channel name, you must use the DeleteDeliveryChannel action to delete your current delivery channel, and then you must use the PutDeliveryChannel command to create a delivery channel that has the desired name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>deliveryFrequency</i> The frequency with which AWS Config delivers configuration snapshots.<br/>
        :return:         """
        pass

    def put_evaluations(self, ResultToken: str, Evaluations: List[Evaluation]=None, TestMode: Boolean=None) -> PutEvaluationsResponse:
        """Used by an AWS Lambda function to deliver evaluation results to AWS Config. This action is required in every AWS Lambda function that is invoked by an AWS Config rule.

        :param self:
        :param TestMode:  Use this parameter to specify a test run for <code>PutEvaluations</code>. You can verify whether your AWS Lambda function will deliver evaluation results to AWS Config. No updates occur to your existing evaluations, and evaluation results are not sent to AWS Config. <note> When <code>TestMode</code> is <code>true</code>, <code>PutEvaluations</code> doesn't require a valid value for the <code>ResultToken</code> parameter, but the value cannot be null. </note><br/>ResultToken:  An encrypted token that associates an evaluation with an AWS Config rule. Identifies the rule and the event that triggered the evaluation.<br/>Evaluations:  The assessments that the AWS Lambda function performs. Each evaluation identifies an AWS resource and indicates whether it complies with the AWS Config rule that invokes the AWS Lambda function.<br/>
        :return: FailedEvaluations:  Requests that failed because of a client or server error.<br/>        """
        pass

    def put_organization_config_rule(self, OrganizationConfigRuleName: str, OrganizationManagedRuleMetadata: OrganizationManagedRuleMetadata=None, OrganizationCustomRuleMetadata: OrganizationCustomRuleMetadata=None, ExcludedAccounts: List[AccountId]=None) -> PutOrganizationConfigRuleResponse:
        """

        :param self:
        :param ExcludedAccounts:  <br/>OrganizationCustomRuleMetadata:  :<br/>OrganizationManagedRuleMetadata:  :<br/>OrganizationConfigRuleName:  <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TagValueScope</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TagKeyScope</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceIdScope</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceTypesScope</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumExecutionFrequency</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputParameters</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleIdentifier</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TagValueScope</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TagKeyScope</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceIdScope</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceTypesScope</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumExecutionFrequency</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputParameters</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationConfigRuleTriggerTypes</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LambdaFunctionArn</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> <br/>
        :return: OrganizationConfigRuleArn:  <br/>        """
        pass

    def put_remediation_configurations(self, RemediationConfigurations: List[RemediationConfiguration]) -> PutRemediationConfigurationsResponse:
        """Adds or updates the remediation configuration with a specific AWS Config rule with the selected target or action. The API creates the <code>RemediationConfiguration</code> object for the AWS Config rule. The AWS Config rule must already exist for you to add a remediation configuration. The target (SSM document) must exist and have permissions to use the target. 

        :param self:
        :param RemediationConfigurations:  A list of remediation configuration objects.<br/>
        :return: FailedBatches:  Returns a list of failed remediation batch objects.<br/>        """
        pass

    def put_retention_configuration(self, RetentionPeriodInDays: int) -> PutRetentionConfigurationResponse:
        """Creates and updates the retention configuration with details about retention period (number of days) that AWS Config stores your historical information. The API creates the <code>RetentionConfiguration</code> object and names the object as <b>default</b>. When you have a <code>RetentionConfiguration</code> object named <b>default</b>, calling the API modifies the default object.  <note> Currently, AWS Config supports only one retention configuration per region in your account. </note>

        :param self:
        :param RetentionPeriodInDays:  Number of days AWS Config stores your historical information. <note> Currently, only applicable to the configuration item history. </note><br/>
        :return: RetentionConfiguration:  Returns a retention configuration object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RetentionPeriodInDays</i> Number of days AWS Config stores your historical information. <note> Currently, only applicable to the configuration item history. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the retention configuration object.<br/>        """
        pass

    def select_resource_config(self, Expression: str, Limit: int=None, NextToken: str=None) -> SelectResourceConfigResponse:
        """Accepts a structured query language (SQL) <code>SELECT</code> command, performs the corresponding search, and returns resource configurations matching the properties. For more information about query components, see the <a href="https://docs.aws.amazon.com/config/latest/developerguide/query-components.html"> <b>Query Components</b> </a> section in the AWS Config Developer Guide.

        :param self:
        :param NextToken:  The <code>nextToken</code> string returned in a previous request that you use to request the next page of results in a paginated response. <br/>Limit:  The maximum number of query results returned on each page. <br/>Expression:  The SQL query <code>SELECT</code> command.<br/>
        :return: NextToken:  The <code>nextToken</code> string returned in a previous request that you use to request the next page of results in a paginated response. <br/>QueryInfo:  Returns the <code>QueryInfo</code> object.:<br/>Results:  Returns the results for the SQL query.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SelectFields</i> Returns a <code>FieldInfo</code> object.<br/>        """
        pass

    def start_config_rules_evaluation(self, ConfigRuleNames: List[StringWithCharLimit64]=None) -> StartConfigRulesEvaluationResponse:
        """Runs an on-demand evaluation for the specified AWS Config rules against the last known configuration state of the resources. Use <code>StartConfigRulesEvaluation</code> when you want to test that a rule you updated is working as expected. <code>StartConfigRulesEvaluation</code> does not re-record the latest configuration state for your resources. It re-runs an evaluation against the last known state of your resources.  You can specify up to 25 AWS Config rules per request.  An existing <code>StartConfigRulesEvaluation</code> call for the specified rules must complete before you can call the API again. If you chose to have AWS Config stream to an Amazon SNS topic, you will receive a <code>ConfigRuleEvaluationStarted</code> notification when the evaluation starts. <note> You don't need to call the <code>StartConfigRulesEvaluation</code> API to run an evaluation for a new rule. When you create a rule, AWS Config evaluates your resources against the rule automatically.  </note> The <code>StartConfigRulesEvaluation</code> API is useful if you want to run on-demand evaluations, such as the following example: <ol> <li> You have a custom rule that evaluates your IAM resources every 24 hours. </li> <li> You update your Lambda function to add additional conditions to your rule. </li> <li> Instead of waiting for the next periodic evaluation, you call the <code>StartConfigRulesEvaluation</code> API. </li> <li> AWS Config invokes your Lambda function and evaluates your IAM resources. </li> <li> Your custom rule will still run periodic evaluations every 24 hours. </li> </ol>

        :param self:
        :param ConfigRuleNames:  The list of names of AWS Config rules that you want to run evaluations for.<br/>
        :return:         """
        pass

    def start_configuration_recorder(self, ConfigurationRecorderName: str):
        """Starts recording configurations of the AWS resources you have selected to record in your AWS account. You must have created at least one delivery channel to successfully start the configuration recorder.

        :param self:
        :param ConfigurationRecorderName:  The name of the recorder object that records each configuration change made to the resources.<br/>
        :return:         """
        pass

    def start_remediation_execution(self, ResourceKeys: List[ResourceKey], ConfigRuleName: str) -> StartRemediationExecutionResponse:
        """Runs an on-demand remediation for the specified AWS Config rules against the last known remediation configuration. It runs an execution against the current state of your resources. Remediation execution is asynchronous. You can specify up to 100 resource keys per request. An existing StartRemediationExecution call for the specified resource keys must complete before you can call the API again.

        :param self:
        :param ResourceKeys:  A list of resource keys to be processed with the current request. Each element in the list consists of the resource type and resource ID. <br/>ConfigRuleName:  The list of names of AWS Config rules that you want to run remediation execution for.<br/>
        :return: FailedItems:  For resources that have failed to start execution, the API returns a resource key object.<br/>FailureMessage:  Returns a failure message. For example, the resource is already compliant.<br/>        """
        pass

    def stop_configuration_recorder(self, ConfigurationRecorderName: str):
        """Stops recording configurations of the AWS resources you have selected to record in your AWS account.

        :param self:
        :param ConfigurationRecorderName:  The name of the recorder object that records each configuration change made to the resources.<br/>
        :return:         """
        pass

    def tag_resource(self, Tags: List[Tag], ResourceArn: str):
        """Associates the specified tags to a resource with the specified resourceArn. If existing tags on a resource are not specified in the request parameters, they are not changed. When a resource is deleted, the tags associated with that resource are deleted as well.

        :param self:
        :param Tags:  An array of tag object.<br/>ResourceArn:  The Amazon Resource Name (ARN) that identifies the resource for which to list the tags. Currently, the supported resources are <code>ConfigRule</code>, <code>ConfigurationAggregator</code> and <code>AggregatorAuthorization</code>.<br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[TagKey], ResourceArn: str):
        """Deletes specified tags from a resource.

        :param self:
        :param TagKeys:  The keys of the tags to be removed.<br/>ResourceArn:  The Amazon Resource Name (ARN) that identifies the resource for which to list the tags. Currently, the supported resources are <code>ConfigRule</code>, <code>ConfigurationAggregator</code> and <code>AggregatorAuthorization</code>.<br/>
        :return:         """
        pass



class Connect(BaseClient):
    """The Amazon Connect API Reference provides descriptions, syntax, and usage examples for each of the Amazon Connect actions, data types, parameters, and errors. Amazon Connect is a cloud-based contact center solution that makes it easy to set up and manage a customer contact center and provide reliable customer engagement at any scale. Throttling limits for the Amazon Connect API operations: For the <code>GetMetricData</code> and <code>GetCurrentMetricData</code> operations, a RateLimit of 5 per second, and a BurstLimit of 8 per second. For all other operations, a RateLimit of 2 per second, and a BurstLimit of 5 per second. You can request an increase to the throttling limits by submitting a <a href="https://console.aws.amazon.com/support/home#/case/create?issueType=service-limit-increase">Amazon Connect service limits increase form</a>. You must be signed in to your AWS account to access the form."""


    class GetMetricDataPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_metric_data` for valid parameters"""
            pass
    class ListRoutingProfilesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_routing_profiles` for valid parameters"""
            pass
    class ListSecurityProfilesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_security_profiles` for valid parameters"""
            pass
    class ListUserHierarchyGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_user_hierarchy_groups` for valid parameters"""
            pass
    class ListUsersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_users` for valid parameters"""
            pass


    class CreateUserRequest(Mapping):
        pass
    
    class CreateUserResponse(Mapping):
        pass
    
    class UserIdentityInfo(Mapping):
        pass
    
    class UserPhoneConfig(Mapping):
        pass
    
    class SecurityProfileId(object):
        pass
    
    class DeleteUserRequest(Mapping):
        pass
    
    class DescribeUserRequest(Mapping):
        pass
    
    class DescribeUserResponse(Mapping):
        pass
    
    class DescribeUserHierarchyGroupRequest(Mapping):
        pass
    
    class DescribeUserHierarchyGroupResponse(Mapping):
        pass
    
    class DescribeUserHierarchyStructureRequest(Mapping):
        pass
    
    class DescribeUserHierarchyStructureResponse(Mapping):
        pass
    
    class GetContactAttributesRequest(Mapping):
        pass
    
    class GetContactAttributesResponse(Mapping):
        pass
    
    class GetCurrentMetricDataRequest(Mapping):
        pass
    
    class GetCurrentMetricDataResponse(Mapping):
        pass
    
    class Filters(Mapping):
        pass
    
    class Grouping(object):
        pass
    
    class CurrentMetric(Mapping):
        pass
    
    class GetFederationTokenRequest(Mapping):
        pass
    
    class GetFederationTokenResponse(Mapping):
        pass
    
    class GetMetricDataRequest(Mapping):
        pass
    
    class GetMetricDataResponse(Mapping):
        pass
    
    class timestamp(object):
        pass
    
    class HistoricalMetric(Mapping):
        pass
    
    class ListRoutingProfilesRequest(Mapping):
        pass
    
    class ListRoutingProfilesResponse(Mapping):
        pass
    
    class ListSecurityProfilesRequest(Mapping):
        pass
    
    class ListSecurityProfilesResponse(Mapping):
        pass
    
    class ListUserHierarchyGroupsRequest(Mapping):
        pass
    
    class ListUserHierarchyGroupsResponse(Mapping):
        pass
    
    class ListUsersRequest(Mapping):
        pass
    
    class ListUsersResponse(Mapping):
        pass
    
    class StartOutboundVoiceContactRequest(Mapping):
        pass
    
    class StartOutboundVoiceContactResponse(Mapping):
        pass
    
    class Attributes(object):
        pass
    
    class StopContactRequest(Mapping):
        pass
    
    class StopContactResponse(Mapping):
        pass
    
    class UpdateContactAttributesRequest(Mapping):
        pass
    
    class UpdateContactAttributesResponse(Mapping):
        pass
    
    class UpdateUserHierarchyRequest(Mapping):
        pass
    
    class UpdateUserIdentityInfoRequest(Mapping):
        pass
    
    class UpdateUserPhoneConfigRequest(Mapping):
        pass
    
    class UpdateUserRoutingProfileRequest(Mapping):
        pass
    
    class UpdateUserSecurityProfilesRequest(Mapping):
        pass
    
    def create_user(self, InstanceId: str, RoutingProfileId: str, SecurityProfileIds: List[SecurityProfileId], PhoneConfig: UserPhoneConfig, Username: str, Password: str=None, IdentityInfo: UserIdentityInfo=None, DirectoryUserId: str=None, HierarchyGroupId: str=None) -> CreateUserResponse:
        """Creates a new user account in your Amazon Connect instance.

        :param self:
        :param InstanceId:  The identifier for your Amazon Connect instance. To find the ID of your instance, open the AWS console and select Amazon Connect. Select the alias of the instance in the Instance alias column. The instance ID is displayed in the Overview section of your instance settings. For example, the instance ID is the set of characters at the end of the instance ARN, after instance/, such as 10a4c4eb-f57e-4d4c-b602-bf39176ced07.<br/>HierarchyGroupId:  The unique identifier for the hierarchy group to assign to the user created.<br/>RoutingProfileId:  The unique identifier for the routing profile to assign to the user created.<br/>SecurityProfileIds:  The unique identifier of the security profile to assign to the user created.<br/>DirectoryUserId:  The unique identifier for the user account in the directory service directory used for identity management. If Amazon Connect is unable to access the existing directory, you can use the <code>DirectoryUserId</code> to authenticate users. If you include the parameter, it is assumed that Amazon Connect cannot access the directory. If the parameter is not included, the <code>UserIdentityInfo</code> is used to authenticate users from your existing directory. This parameter is required if you are using an existing directory for identity management in Amazon Connect when Amazon Connect cannot access your directory to authenticate users. If you are using SAML for identity management and include this parameter, an <code>InvalidRequestException</code> is returned.<br/>PhoneConfig:  Specifies the phone settings for the user, including <code>AfterContactWorkTimeLimit</code>, <code>AutoAccept</code>, <code>DeskPhoneNumber</code>, and <code>PhoneType</code>.:<br/>IdentityInfo:  Information about the user, including email address, first name, and last name.:<br/>Password:  The password for the user account to create. This is required if you are using Amazon Connect for identity management. If you are using SAML for identity management and include this parameter, an <code>InvalidRequestException</code> is returned.<br/>Username:  The user name in Amazon Connect for the account to create. If you are using SAML for identity management in your Amazon Connect, the value for <code>Username</code> can include up to 64 characters from [a-zA-Z0-9_-.\@]+.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Email</i> The email address added to the user account. If you are using SAML for identity management and include this parameter, an <code>InvalidRequestException</code> is returned.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastName</i> The last name used in the user account. This is required if you are using Amazon Connect or SAML for identity management.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FirstName</i> The first name used in the user account. This is required if you are using Amazon Connect or SAML for identity management.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeskPhoneNumber</i> The phone number for the user's desk phone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AfterContactWorkTimeLimit</i> The After Call Work (ACW) timeout setting, in seconds, for the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoAccept</i> The Auto accept setting for the user, Yes or No.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneType</i> The phone type selected for the user, either Soft phone or Desk phone.<br/>
        :return: UserArn:  The Amazon Resource Name (ARN) of the user account created.<br/>UserId:  The unique identifier for the user account in Amazon Connect<br/>        """
        pass

    def delete_user(self, UserId: str, InstanceId: str):
        """Deletes a user account from Amazon Connect.

        :param self:
        :param UserId:  The unique identifier of the user to delete.<br/>InstanceId:  The identifier for your Amazon Connect instance. To find the ID of your instance, open the AWS console and select Amazon Connect. Select the alias of the instance in the Instance alias column. The instance ID is displayed in the Overview section of your instance settings. For example, the instance ID is the set of characters at the end of the instance ARN, after instance/, such as 10a4c4eb-f57e-4d4c-b602-bf39176ced07.<br/>
        :return:         """
        pass

    def describe_user(self, InstanceId: str, UserId: str) -> DescribeUserResponse:
        """Returns a <code>User</code> object that contains information about the user account specified by the <code>UserId</code>.

        :param self:
        :param InstanceId:  The identifier for your Amazon Connect instance. To find the ID of your instance, open the AWS console and select Amazon Connect. Select the alias of the instance in the Instance alias column. The instance ID is displayed in the Overview section of your instance settings. For example, the instance ID is the set of characters at the end of the instance ARN, after instance/, such as 10a4c4eb-f57e-4d4c-b602-bf39176ced07.<br/>UserId:  Unique identifier for the user account to return.<br/>
        :return: User:  A <code>User</code> object that contains information about the user account and configuration settings.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HierarchyGroupId</i> The identifier for the hierarchy group assigned to the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoutingProfileId</i> The identifier of the routing profile assigned to the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityProfileIds</i> The identifier(s) for the security profile assigned to the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DirectoryUserId</i> The directory Id for the user account in the existing directory used for identity management.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneConfig</i> A <code>UserPhoneConfig</code> object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PhoneConfig</b>: A <code>UserPhoneConfig</code> object.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IdentityInfo</i> A <code>UserIdentityInfo</code> object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>IdentityInfo</b>: A <code>UserIdentityInfo</code> object.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> The user name assigned to the user account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN of the user account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier of the user account.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Email</i> The email address added to the user account. If you are using SAML for identity management and include this parameter, an <code>InvalidRequestException</code> is returned.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LastName</i> The last name used in the user account. This is required if you are using Amazon Connect or SAML for identity management.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FirstName</i> The first name used in the user account. This is required if you are using Amazon Connect or SAML for identity management.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DeskPhoneNumber</i> The phone number for the user's desk phone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AfterContactWorkTimeLimit</i> The After Call Work (ACW) timeout setting, in seconds, for the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoAccept</i> The Auto accept setting for the user, Yes or No.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneType</i> The phone type selected for the user, either Soft phone or Desk phone.<br/>        """
        pass

    def describe_user_hierarchy_group(self, InstanceId: str, HierarchyGroupId: str) -> DescribeUserHierarchyGroupResponse:
        """Returns a <code>HierarchyGroup</code> object that includes information about a hierarchy group in your instance.

        :param self:
        :param InstanceId:  The identifier for your Amazon Connect instance. To find the ID of your instance, open the AWS console and select Amazon Connect. Select the alias of the instance in the Instance alias column. The instance ID is displayed in the Overview section of your instance settings. For example, the instance ID is the set of characters at the end of the instance ARN, after instance/, such as 10a4c4eb-f57e-4d4c-b602-bf39176ced07.<br/>HierarchyGroupId:  The identifier for the hierarchy group to return.<br/>
        :return: HierarchyGroup:  Returns a <code>HierarchyGroup</code> object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HierarchyPath</i> A <code>HierarchyPath</code> object that contains information about the levels in the hierarchy group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>HierarchyPath</b>: A <code>HierarchyPath</code> object that contains information about the levels in the hierarchy group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LevelId</i> The identifier for the level in the hierarchy group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the hierarchy group in your instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) for the hierarchy group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier for the hierarchy group.<br/>A <code>HierarchyGroupSummary</code> object that contains information about the level of the hierarchy group, including ARN, Id, and Name.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LevelOne</b>: A <code>HierarchyGroupSummary</code> object that contains information about the level of the hierarchy group, including ARN, Id, and Name.        """
        pass

    def describe_user_hierarchy_structure(self, InstanceId: str) -> DescribeUserHierarchyStructureResponse:
        """Returns a <code>HiearchyGroupStructure</code> object, which contains data about the levels in the agent hierarchy.

        :param self:
        :param InstanceId:  The identifier for your Amazon Connect instance. To find the ID of your instance, open the AWS console and select Amazon Connect. Select the alias of the instance in the Instance alias column. The instance ID is displayed in the Overview section of your instance settings. For example, the instance ID is the set of characters at the end of the instance ARN, after instance/, such as 10a4c4eb-f57e-4d4c-b602-bf39176ced07.<br/>
        :return: HierarchyStructure:  A <code>HierarchyStructure</code> object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LevelFive</i> A <code>HierarchyLevel</code> object that contains information about the hierarchy group level.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LevelFive</b>: A <code>HierarchyLevel</code> object that contains information about the hierarchy group level.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LevelFour</i> A <code>HierarchyLevel</code> object that contains information about the hierarchy group level.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LevelFour</b>: A <code>HierarchyLevel</code> object that contains information about the hierarchy group level.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LevelThree</i> A <code>HierarchyLevel</code> object that contains information about the hierarchy group level.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LevelThree</b>: A <code>HierarchyLevel</code> object that contains information about the hierarchy group level.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LevelTwo</i> A <code>HierarchyLevel</code> object that contains information about the hierarchy group level.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LevelTwo</b>: A <code>HierarchyLevel</code> object that contains information about the hierarchy group level.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LevelOne</i> A <code>HierarchyLevel</code> object that contains information about the hierarchy group level.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LevelOne</b>: A <code>HierarchyLevel</code> object that contains information about the hierarchy group level.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the hierarchy group level.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN for the hierarchy group level.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier for the hierarchy group level.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the hierarchy group level.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN for the hierarchy group level.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier for the hierarchy group level.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the hierarchy group level.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN for the hierarchy group level.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier for the hierarchy group level.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the hierarchy group level.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN for the hierarchy group level.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier for the hierarchy group level.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the hierarchy group level.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN for the hierarchy group level.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier for the hierarchy group level.<br/>        """
        pass

    def get_contact_attributes(self, InitialContactId: str, InstanceId: str) -> GetContactAttributesResponse:
        """Retrieves the contact attributes associated with a contact.

        :param self:
        :param InitialContactId:  The ID for the initial contact in Amazon Connect associated with the attributes to update.<br/>InstanceId:  The instance ID for the instance from which to retrieve contact attributes.<br/>
        :return: Attributes:  The attributes to update.<br/>        """
        pass

    def get_current_metric_data(self, CurrentMetrics: List[CurrentMetric], Filters: Filters, InstanceId: str, Groupings: List[Grouping]=None, NextToken: str=None, MaxResults: int=None) -> GetCurrentMetricDataResponse:
        """The <code>GetCurrentMetricData</code> operation retrieves current metric data from your Amazon Connect instance. If you are using an IAM account, it must have permission to the <code>connect:GetCurrentMetricData</code> action.

        :param self:
        :param MaxResults:   <code>MaxResults</code> indicates the maximum number of results to return per page in the response, between 1 and 100.<br/>NextToken:  The token for the next set of results. Use the value returned in the previous response in the next request to retrieve the next set of results. The token expires after 5 minutes from the time it is created. Subsequent requests that use the <a href="">NextToken</a> must use the same request parameters as the request that generated the token.<br/>CurrentMetrics:  A list of <code>CurrentMetric</code> objects for the metrics to retrieve. Each <code>CurrentMetric</code> includes a name of a metric to retrieve and the unit to use for it. You must list each metric to retrieve data for in the request. The following metrics are available: <dl> <dt>AGENTS_AVAILABLE</dt> <dd> Unit: COUNT </dd> <dt>AGENTS_ONLINE</dt> <dd> Unit: COUNT </dd> <dt>AGENTS_ON_CALL</dt> <dd> Unit: COUNT </dd> <dt>AGENTS_STAFFED</dt> <dd> Unit: COUNT </dd> <dt>AGENTS_AFTER_CONTACT_WORK</dt> <dd> Unit: COUNT </dd> <dt>AGENTS_NON_PRODUCTIVE</dt> <dd> Unit: COUNT </dd> <dt>AGENTS_ERROR</dt> <dd> Unit: COUNT </dd> <dt>CONTACTS_IN_QUEUE</dt> <dd> Unit: COUNT </dd> <dt>OLDEST_CONTACT_AGE</dt> <dd> Unit: SECONDS </dd> <dt>CONTACTS_SCHEDULED</dt> <dd> Unit: COUNT </dd> </dl><br/>Groupings:  The grouping applied to the metrics returned. For example, when grouped by QUEUE, the metrics returned apply to each queue rather than aggregated for all queues. If you group by CHANNEL, you should include a Channels filter. The only supported channel is VOICE. If no <code>Grouping</code> is included in the request, a summary of <code>CurrentMetrics</code> is returned.<br/>Filters:  A <code>Filters</code> object that contains a list of queue IDs or queue ARNs, up to 100, or list of Channels to use to filter the metrics returned in the response. Metric data is retrieved only for the resources associated with the queue IDs, ARNs, or Channels included in the filter. You can include both IDs and ARNs in the same request. To retrieve metrics for all queues, add the queue ID or ARN for each queue in your instance. Only VOICE is supported for Channels. To find the ARN for a queue, open the queue you want to use in the Amazon Connect Queue editor. The ARN for the queue is displayed in the address bar as part of the URL. For example, the queue ARN is the set of characters at the end of the URL, after 'id=' such as <code>arn:aws:connect:us-east-1:270923740243:instance/78fb859d-1b7d-44b1-8aa3-12f0835c5855/queue/1d1a4575-9618-40ab-bbeb-81e45795fe61</code>. The queue ID is also included in the URL, and is the string after 'queue/'.:<br/>InstanceId:  The identifier for your Amazon Connect instance. To find the ID of your instance, open the AWS console and select Amazon Connect. Select the alias of the instance in the Instance alias column. The instance ID is displayed in the Overview section of your instance settings. For example, the instance ID is the set of characters at the end of the instance ARN, after instance/, such as 10a4c4eb-f57e-4d4c-b602-bf39176ced07.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Channels</i> The Channel to use as a filter for the metrics returned. Only VOICE is supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Queues</i> A list of up to 100 queue IDs or queue ARNs to use to filter the metrics retrieved. You can include both IDs and ARNs in a request.<br/>
        :return: DataSnapshotTime:  The time at which <code>CurrentMetricData</code> was retrieved and cached for pagination.<br/>MetricResults:  A list of <code>CurrentMetricResult</code> objects organized by <code>Dimensions</code> combining with <code>CurrentMetricDataCollections</code>.  <code>Dimensions</code> is the resourceId specified in the <code>Filters</code> of the request.   <code>Collections</code> is a list of <code>CurrentMetricData</code> objects with corresponding values to the <code>CurrentMetrics</code> specified in the request. If no <code>Grouping</code> is specified in the request, <code>Collections</code> is a summary for the <code>CurrentMetric</code> returned.<br/>NextToken:  A string returned in the response. Use the value returned in the response as the value of the NextToken in a subsequent request to retrieve the next set of results. The token expires after 5 minutes from the time it is created. Subsequent requests that use the NextToken must use the same request parameters as the request that generated the token. <br/>        """
        pass

    def get_federation_token(self, InstanceId: str) -> GetFederationTokenResponse:
        """Retrieves a token for federation.

        :param self:
        :param InstanceId:  The identifier for your Amazon Connect instance. To find the ID of your instance, open the AWS console and select Amazon Connect. Select the alias of the instance in the Instance alias column. The instance ID is displayed in the Overview section of your instance settings. For example, the instance ID is the set of characters at the end of the instance ARN, after instance/, such as 10a4c4eb-f57e-4d4c-b602-bf39176ced07.<br/>
        :return: Credentials:  The credentials to use for federation.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RefreshTokenExpiration</i> Renews the expiration timer for a generated token.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RefreshToken</i> Renews a token generated for a user to access the Amazon Connect instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccessTokenExpiration</i> A token generated with an expiration time for the session a user is logged in to Amazon Connect<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccessToken</i> An access token generated for a federated user to access Amazon Connect<br/>        """
        pass

    def get_metric_data(self, HistoricalMetrics: List[HistoricalMetric], Filters: Filters, EndTime: timestamp, StartTime: timestamp, InstanceId: str, Groupings: List[Grouping]=None, NextToken: str=None, MaxResults: int=None) -> GetMetricDataResponse:
        """The <code>GetMetricData</code> operation retrieves historical metrics data from your Amazon Connect instance. If you are using an IAM account, it must have permission to the <code>connect:GetMetricData</code> action.

        :param self:
        :param MaxResults:  Indicates the maximum number of results to return per page in the response, between 1-100.<br/>NextToken:  The token for the next set of results. Use the value returned in the previous response in the next request to retrieve the next set of results.<br/>HistoricalMetrics:  A list of <code>HistoricalMetric</code> objects that contain the metrics to retrieve with the request. A <code>HistoricalMetric</code> object contains: <code>HistoricalMetricName</code>, <code>Statistic</code>, <code>Threshold</code>, and <code>Unit</code>. You must list each metric to retrieve data for in the request. For each historical metric you include in the request, you must include a <code>Unit</code> and a <code>Statistic</code>.  The following historical metrics are available: <dl> <dt>CONTACTS_QUEUED</dt> <dd> Unit: COUNT Statistic: SUM </dd> <dt>CONTACTS_HANDLED</dt> <dd> Unit: COUNT Statistics: SUM </dd> <dt>CONTACTS_ABANDONED</dt> <dd> Unit: COUNT Statistics: SUM </dd> <dt>CONTACTS_CONSULTED</dt> <dd> Unit: COUNT Statistics: SUM </dd> <dt>CONTACTS_AGENT_HUNG_UP_FIRST</dt> <dd> Unit: COUNT Statistics: SUM </dd> <dt>CONTACTS_HANDLED_INCOMING</dt> <dd> Unit: COUNT Statistics: SUM </dd> <dt>CONTACTS_HANDLED_OUTBOUND</dt> <dd> Unit: COUNT Statistics: SUM </dd> <dt>CONTACTS_HOLD_ABANDONS</dt> <dd> Unit: COUNT Statistics: SUM </dd> <dt>CONTACTS_TRANSFERRED_IN</dt> <dd> Unit: COUNT Statistics: SUM </dd> <dt>CONTACTS_TRANSFERRED_OUT</dt> <dd> Unit: COUNT Statistics: SUM </dd> <dt>CONTACTS_TRANSFERRED_IN_FROM_QUEUE</dt> <dd> Unit: COUNT Statistics: SUM </dd> <dt>CONTACTS_TRANSFERRED_OUT_FROM_QUEUE</dt> <dd> Unit: COUNT Statistics: SUM </dd> <dt>CALLBACK_CONTACTS_HANDLED</dt> <dd> Unit: COUNT Statistics: SUM </dd> <dt>CALLBACK_CONTACTS_HANDLED</dt> <dd> Unit: COUNT Statistics: SUM </dd> <dt>API_CONTACTS_HANDLED</dt> <dd> Unit: COUNT Statistics: SUM </dd> <dt>CONTACTS_MISSED</dt> <dd> Unit: COUNT Statistics: SUM </dd> <dt>OCCUPANCY</dt> <dd> Unit: PERCENT Statistics: AVG </dd> <dt>HANDLE_TIME</dt> <dd> Unit: SECONDS Statistics: AVG </dd> <dt>AFTER_CONTACT_WORK_TIME</dt> <dd> Unit: SECONDS Statistics: AVG </dd> <dt>QUEUED_TIME</dt> <dd> Unit: SECONDS Statistics: MAX </dd> <dt>ABANDON_TIME</dt> <dd> Unit: COUNT Statistics: SUM </dd> <dt>QUEUE_ANSWER_TIME</dt> <dd> Unit: SECONDS Statistics: AVG </dd> <dt>HOLD_TIME</dt> <dd> Unit: SECONDS Statistics: AVG </dd> <dt>INTERACTION_TIME</dt> <dd> Unit: SECONDS Statistics: AVG </dd> <dt>INTERACTION_AND_HOLD_TIME</dt> <dd> Unit: SECONDS Statistics: AVG </dd> <dt>SERVICE_LEVEL</dt> <dd> Unit: PERCENT Statistics: AVG Threshold: Only "Less than" comparisons are supported, with the following service level thresholds: 15, 20, 25, 30, 45, 60, 90, 120, 180, 240, 300, 600 </dd> </dl><br/>Groupings:  The grouping applied to the metrics returned. For example, when results are grouped by queueId, the metrics returned are grouped by queue. The values returned apply to the metrics for each queue rather than aggregated for all queues. The current version supports grouping by Queue If no <code>Grouping</code> is included in the request, a summary of <code>HistoricalMetrics</code> for all queues is returned.<br/>Filters:  A <code>Filters</code> object that contains a list of queue IDs or queue ARNs, up to 100, or a list of Channels to use to filter the metrics returned in the response. Metric data is retrieved only for the resources associated with the IDs, ARNs, or Channels included in the filter. You can use both IDs and ARNs together in a request. Only VOICE is supported for Channel. To find the ARN for a queue, open the queue you want to use in the Amazon Connect Queue editor. The ARN for the queue is displayed in the address bar as part of the URL. For example, the queue ARN is the set of characters at the end of the URL, after 'id=' such as <code>arn:aws:connect:us-east-1:270923740243:instance/78fb859d-1b7d-44b1-8aa3-12f0835c5855/queue/1d1a4575-9618-40ab-bbeb-81e45795fe61</code>. The queue ID is also included in the URL, and is the string after 'queue/'.:<br/>EndTime:  The timestamp, in UNIX Epoch time format, at which to end the reporting interval for the retrieval of historical metrics data. The time must be specified using an interval of 5 minutes, such as 11:00, 11:05, 11:10, and must be later than the <code>StartTime</code> timestamp. The time range between <code>StartTime</code> and <code>EndTime</code> must be less than 24 hours.<br/>StartTime:  The timestamp, in UNIX Epoch time format, at which to start the reporting interval for the retrieval of historical metrics data. The time must be specified using a multiple of 5 minutes, such as 10:05, 10:10, 10:15.  <code>StartTime</code> cannot be earlier than 24 hours before the time of the request. Historical metrics are available in Amazon Connect only for 24 hours.<br/>InstanceId:  The identifier for your Amazon Connect instance. To find the ID of your instance, open the AWS console and select Amazon Connect. Select the alias of the instance in the Instance alias column. The instance ID is displayed in the Overview section of your instance settings. For example, the instance ID is the set of characters at the end of the instance ARN, after instance/, such as 10a4c4eb-f57e-4d4c-b602-bf39176ced07.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Channels</i> The Channel to use as a filter for the metrics returned. Only VOICE is supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Queues</i> A list of up to 100 queue IDs or queue ARNs to use to filter the metrics retrieved. You can include both IDs and ARNs in a request.<br/>
        :return: MetricResults:  A list of <code>HistoricalMetricResult</code> objects, organized by <code>Dimensions</code>, which is the ID of the resource specified in the <code>Filters</code> used for the request. The metrics are combined with the metrics included in <code>Collections</code>, which is a list of <code>HisotricalMetricData</code> objects. If no <code>Grouping</code> is specified in the request, <code>Collections</code> includes summary data for the <code>HistoricalMetrics</code>.<br/>NextToken:  A string returned in the response. Use the value returned in the response as the value of the NextToken in a subsequent request to retrieve the next set of results. The token expires after 5 minutes from the time it is created. Subsequent requests that use the NextToken must use the same request parameters as the request that generated the token. <br/>        """
        pass

    def list_routing_profiles(self, InstanceId: str, NextToken: str=None, MaxResults: int=None) -> ListRoutingProfilesResponse:
        """Returns an array of <code>RoutingProfileSummary</code> objects that includes information about the routing profiles in your instance.

        :param self:
        :param MaxResults:  The maximum number of routing profiles to return in the response.<br/>NextToken:  The token for the next set of results. Use the value returned in the previous response in the next request to retrieve the next set of results.<br/>InstanceId:  The identifier for your Amazon Connect instance. To find the ID of your instance, open the AWS console and select Amazon Connect. Select the alias of the instance in the Instance alias column. The instance ID is displayed in the Overview section of your instance settings. For example, the instance ID is the set of characters at the end of the instance ARN, after instance/, such as 10a4c4eb-f57e-4d4c-b602-bf39176ced07.<br/>
        :return: NextToken:  A string returned in the response. Use the value returned in the response as the value of the NextToken in a subsequent request to retrieve the next set of results.<br/>RoutingProfileSummaryList:  An array of <code>RoutingProfileSummary</code> objects that include the ARN, Id, and Name of the routing profile.<br/>        """
        pass

    def list_security_profiles(self, InstanceId: str, NextToken: str=None, MaxResults: int=None) -> ListSecurityProfilesResponse:
        """Returns an array of SecurityProfileSummary objects that contain information about the security profiles in your instance, including the ARN, Id, and Name of the security profile.

        :param self:
        :param MaxResults:  The maximum number of security profiles to return.<br/>NextToken:  The token for the next set of results. Use the value returned in the previous response in the next request to retrieve the next set of results.<br/>InstanceId:  The identifier for your Amazon Connect instance. To find the ID of your instance, open the AWS console and select Amazon Connect. Select the alias of the instance in the Instance alias column. The instance ID is displayed in the Overview section of your instance settings. For example, the instance ID is the set of characters at the end of the instance ARN, after instance/, such as 10a4c4eb-f57e-4d4c-b602-bf39176ced07.<br/>
        :return: NextToken:  A string returned in the response. Use the value returned in the response as the value of the NextToken in a subsequent request to retrieve the next set of results.<br/>SecurityProfileSummaryList:  An array of <code>SecurityProfileSummary</code> objects.<br/>        """
        pass

    def list_user_hierarchy_groups(self, InstanceId: str, NextToken: str=None, MaxResults: int=None) -> ListUserHierarchyGroupsResponse:
        """Returns a <code>UserHierarchyGroupSummaryList</code>, which is an array of <code>HierarchyGroupSummary</code> objects that contain information about the hierarchy groups in your instance.

        :param self:
        :param MaxResults:  The maximum number of hierarchy groups to return.<br/>NextToken:  The token for the next set of results. Use the value returned in the previous response in the next request to retrieve the next set of results.<br/>InstanceId:  The identifier for your Amazon Connect instance. To find the ID of your instance, open the AWS console and select Amazon Connect. Select the alias of the instance in the Instance alias column. The instance ID is displayed in the Overview section of your instance settings. For example, the instance ID is the set of characters at the end of the instance ARN, after instance/, such as 10a4c4eb-f57e-4d4c-b602-bf39176ced07.<br/>
        :return: NextToken:  A string returned in the response. Use the value returned in the response as the value of the NextToken in a subsequent request to retrieve the next set of results.<br/>UserHierarchyGroupSummaryList:  An array of <code>HierarchyGroupSummary</code> objects.<br/>        """
        pass

    def list_users(self, InstanceId: str, NextToken: str=None, MaxResults: int=None) -> ListUsersResponse:
        """Returns a <code>UserSummaryList</code>, which is an array of <code>UserSummary</code> objects.

        :param self:
        :param MaxResults:  The maximum number of results to return in the response.<br/>NextToken:  The token for the next set of results. Use the value returned in the previous response in the next request to retrieve the next set of results.<br/>InstanceId:  The identifier for your Amazon Connect instance. To find the ID of your instance, open the AWS console and select Amazon Connect. Select the alias of the instance in the Instance alias column. The instance ID is displayed in the Overview section of your instance settings. For example, the instance ID is the set of characters at the end of the instance ARN, after instance/, such as 10a4c4eb-f57e-4d4c-b602-bf39176ced07.<br/>
        :return: NextToken:  A string returned in the response. Use the value returned in the response as the value of the NextToken in a subsequent request to retrieve the next set of results.<br/>UserSummaryList:  An array of <code>UserSummary</code> objects that contain information about the users in your instance.<br/>        """
        pass

    def start_outbound_voice_contact(self, InstanceId: str, ContactFlowId: str, DestinationPhoneNumber: str, ClientToken: str=None, SourcePhoneNumber: str=None, QueueId: str=None, Attributes: Attributes=None) -> StartOutboundVoiceContactResponse:
        """The <code>StartOutboundVoiceContact</code> operation initiates a contact flow to place an outbound call to a customer. If you are using an IAM account, it must have permission to the <code>connect:StartOutboundVoiceContact</code> action. There is a 60 second dialing timeout for this operation. If the call is not connected after 60 seconds, the call fails.

        :param self:
        :param Attributes:  Specify a custom key-value pair using an attribute map. The attributes are standard Amazon Connect attributes, and can be accessed in contact flows just like any other contact attributes. There can be up to 32,768 UTF-8 bytes across all key-value pairs per contact. Attribute keys can include only alphanumeric, dash, and underscore characters. For example, if you want play a greeting when the customer answers the call, you can pass the customer name in attributes similar to the following:<br/>QueueId:  The queue to add the call to. If you specify a queue, the phone displayed for caller ID is the phone number specified in the queue. If you do not specify a queue, the queue used will be the queue defined in the contact flow. To find the <code>QueueId</code>, open the queue you want to use in the Amazon Connect Queue editor. The ID for the queue is displayed in the address bar as part of the URL. For example, the queue ID is the set of characters at the end of the URL, after 'queue/' such as <code>queue/aeg40574-2d01-51c3-73d6-bf8624d2168c</code>.<br/>SourcePhoneNumber:  The phone number, in E.164 format, associated with your Amazon Connect instance to use for the outbound call.<br/>ClientToken:  A unique, case-sensitive identifier that you provide to ensure the idempotency of the request. The token is valid for 7 days after creation. If a contact is already started, the contact ID is returned. If the contact is disconnected, a new contact is started.<br/>InstanceId:  The identifier for your Amazon Connect instance. To find the ID of your instance, open the AWS console and select Amazon Connect. Select the alias of the instance in the Instance alias column. The instance ID is displayed in the Overview section of your instance settings. For example, the instance ID is the set of characters at the end of the instance ARN, after instance/, such as 10a4c4eb-f57e-4d4c-b602-bf39176ced07.<br/>ContactFlowId:  The identifier for the contact flow to connect the outbound call to. To find the <code>ContactFlowId</code>, open the contact flow you want to use in the Amazon Connect contact flow editor. The ID for the contact flow is displayed in the address bar as part of the URL. For example, the contact flow ID is the set of characters at the end of the URL, after 'contact-flow/' such as <code>78ea8fd5-2659-4f2b-b528-699760ccfc1b</code>.<br/>DestinationPhoneNumber:  The phone number of the customer in E.164 format.<br/>
        :return: ContactId:  The unique identifier of this contact within your Amazon Connect instance.<br/>        """
        pass

    def stop_contact(self, InstanceId: str, ContactId: str) -> StopContactResponse:
        """Ends the contact initiated by the <code>StartOutboundVoiceContact</code> operation. If you are using an IAM account, it must have permission to the <code>connect:StopContact</code> action.

        :param self:
        :param InstanceId:  The identifier for your Amazon Connect instance. To find the ID of your instance, open the AWS console and select Amazon Connect. Select the alias of the instance in the Instance alias column. The instance ID is displayed in the Overview section of your instance settings. For example, the instance ID is the set of characters at the end of the instance ARN, after instance/, such as 10a4c4eb-f57e-4d4c-b602-bf39176ced07.<br/>ContactId:  The unique identifier of the contact to end.<br/>
        :return:         """
        pass

    def update_contact_attributes(self, Attributes: Attributes, InstanceId: str, InitialContactId: str) -> UpdateContactAttributesResponse:
        """The <code>UpdateContactAttributes</code> operation lets you programmatically create new, or update existing, contact attributes associated with a contact. You can use the operation to add or update attributes for both ongoing and completed contacts. For example, you can update the customer's name or the reason the customer called while the call is active, or add notes about steps that the agent took during the call that are displayed to the next agent that takes the call. You can also use the <code>UpdateContactAttributes</code> operation to update attributes for a contact using data from your CRM application and save the data with the contact in Amazon Connect. You could also flag calls for additional analysis, such as legal review or identifying abusive callers. Contact attributes are available in Amazon Connect for 24 months, and are then deleted.  <i>Important:</i>  You cannot use the operation to update attributes for contacts that occurred prior to the release of the API, September 12, 2018. You can update attributes only for contacts that started after the release of the API. If you attempt to update attributes for a contact that occurred prior to the release of the API, a 400 error is returned. This applies also to queued callbacks that were initiated prior to the release of the API but are still active in your instance.

        :param self:
        :param Attributes:  Specify a custom key-value pair using an attribute map. The attributes are standard Amazon Connect attributes, and can be accessed in contact flows just like any other contact attributes. There can be up to 32,768 UTF-8 bytes across all key-value pairs per contact. Attribute keys can include only alphanumeric, dash, and underscore characters.<br/>InstanceId:  The identifier for your Amazon Connect instance. To find the ID of your instance, open the AWS console and select Amazon Connect. Select the alias of the instance in the Instance alias column. The instance ID is displayed in the Overview section of your instance settings. For example, the instance ID is the set of characters at the end of the instance ARN, after instance/, such as 10a4c4eb-f57e-4d4c-b602-bf39176ced07.<br/>InitialContactId:  The unique identifier of the contact for which to update attributes. This is the identifier for the contact associated with the first interaction with the contact center.<br/>
        :return:         """
        pass

    def update_user_hierarchy(self, InstanceId: str, UserId: str, HierarchyGroupId: str=None):
        """Assigns the specified hierarchy group to the user.

        :param self:
        :param InstanceId:  The identifier for your Amazon Connect instance. To find the ID of your instance, open the AWS console and select Amazon Connect. Select the alias of the instance in the Instance alias column. The instance ID is displayed in the Overview section of your instance settings. For example, the instance ID is the set of characters at the end of the instance ARN, after instance/, such as 10a4c4eb-f57e-4d4c-b602-bf39176ced07.<br/>UserId:  The identifier of the user account to assign the hierarchy group to.<br/>HierarchyGroupId:  The identifier for the hierarchy group to assign to the user.<br/>
        :return:         """
        pass

    def update_user_identity_info(self, InstanceId: str, UserId: str, IdentityInfo: UserIdentityInfo):
        """Updates the identity information for the specified user in a <code>UserIdentityInfo</code> object, including email, first name, and last name.

        :param self:
        :param InstanceId:  The identifier for your Amazon Connect instance. To find the ID of your instance, open the AWS console and select Amazon Connect. Select the alias of the instance in the Instance alias column. The instance ID is displayed in the Overview section of your instance settings. For example, the instance ID is the set of characters at the end of the instance ARN, after instance/, such as 10a4c4eb-f57e-4d4c-b602-bf39176ced07.<br/>UserId:  The identifier for the user account to update identity information for.<br/>IdentityInfo:  A <code>UserIdentityInfo</code> object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Email</i> The email address added to the user account. If you are using SAML for identity management and include this parameter, an <code>InvalidRequestException</code> is returned.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastName</i> The last name used in the user account. This is required if you are using Amazon Connect or SAML for identity management.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FirstName</i> The first name used in the user account. This is required if you are using Amazon Connect or SAML for identity management.<br/>
        :return:         """
        pass

    def update_user_phone_config(self, InstanceId: str, UserId: str, PhoneConfig: UserPhoneConfig):
        """Updates the phone configuration settings in the <code>UserPhoneConfig</code> object for the specified user.

        :param self:
        :param InstanceId:  The identifier for your Amazon Connect instance. To find the ID of your instance, open the AWS console and select Amazon Connect. Select the alias of the instance in the Instance alias column. The instance ID is displayed in the Overview section of your instance settings. For example, the instance ID is the set of characters at the end of the instance ARN, after instance/, such as 10a4c4eb-f57e-4d4c-b602-bf39176ced07.<br/>UserId:  The identifier for the user account to change phone settings for.<br/>PhoneConfig:  A <code>UserPhoneConfig</code> object that contains settings for <code>AfterContactWorkTimeLimit</code>, <code>AutoAccept</code>, <code>DeskPhoneNumber</code>, and <code>PhoneType</code> to assign to the user.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeskPhoneNumber</i> The phone number for the user's desk phone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AfterContactWorkTimeLimit</i> The After Call Work (ACW) timeout setting, in seconds, for the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoAccept</i> The Auto accept setting for the user, Yes or No.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneType</i> The phone type selected for the user, either Soft phone or Desk phone.<br/>
        :return:         """
        pass

    def update_user_routing_profile(self, InstanceId: str, UserId: str, RoutingProfileId: str):
        """Assigns the specified routing profile to a user.

        :param self:
        :param InstanceId:  The identifier for your Amazon Connect instance. To find the ID of your instance, open the AWS console and select Amazon Connect. Select the alias of the instance in the Instance alias column. The instance ID is displayed in the Overview section of your instance settings. For example, the instance ID is the set of characters at the end of the instance ARN, after instance/, such as 10a4c4eb-f57e-4d4c-b602-bf39176ced07.<br/>UserId:  The identifier for the user account to assign the routing profile to.<br/>RoutingProfileId:  The identifier of the routing profile to assign to the user.<br/>
        :return:         """
        pass

    def update_user_security_profiles(self, InstanceId: str, UserId: str, SecurityProfileIds: List[SecurityProfileId]):
        """Updates the security profiles assigned to the user.

        :param self:
        :param InstanceId:  The identifier for your Amazon Connect instance. To find the ID of your instance, open the AWS console and select Amazon Connect. Select the alias of the instance in the Instance alias column. The instance ID is displayed in the Overview section of your instance settings. For example, the instance ID is the set of characters at the end of the instance ARN, after instance/, such as 10a4c4eb-f57e-4d4c-b602-bf39176ced07.<br/>UserId:  The identifier of the user account to assign the security profiles.<br/>SecurityProfileIds:  The identifiers for the security profiles to assign to the user.<br/>
        :return:         """
        pass



class CostandUsageReportService(BaseClient):
    """The AWS Cost and Usage Report API enables you to programmatically create, query, and delete AWS Cost and Usage report definitions. AWS Cost and Usage reports track the monthly AWS costs and usage associated with your AWS account. The report contains line items for each unique combination of AWS product, usage type, and operation that your AWS account uses. You can configure the AWS Cost and Usage report to show only the data that you want, using the AWS Cost and Usage API. Service Endpoint The AWS Cost and Usage Report API provides the following endpoint: <ul> <li> cur.us-east-1.amazonaws.com </li> </ul>"""


    class DescribeReportDefinitionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_report_definitions` for valid parameters"""
            pass


    class DeleteReportDefinitionRequest(Mapping):
        pass
    
    class DeleteReportDefinitionResponse(Mapping):
        pass
    
    class DescribeReportDefinitionsRequest(Mapping):
        pass
    
    class DescribeReportDefinitionsResponse(Mapping):
        pass
    
    class PutReportDefinitionRequest(Mapping):
        pass
    
    class PutReportDefinitionResponse(Mapping):
        pass
    
    class ReportDefinition(Mapping):
        pass
    
    def delete_report_definition(self, ReportName: str=None) -> DeleteReportDefinitionResponse:
        """Deletes the specified report.

        :param self:
        :param ReportName:  The name of the report that you want to create. The name must be unique, is case sensitive, and can't include spaces. <br/>
        :return: ResponseMessage:  Whether the deletion was successful or not.<br/>        """
        pass

    def describe_report_definitions(self, MaxResults: int=None, NextToken: str=None) -> DescribeReportDefinitionsResponse:
        """Lists the AWS Cost and Usage reports available to this account.

        :param self:
        :param NextToken:  A generic string.<br/>MaxResults:  The maximum number of results that AWS returns for the operation.<br/>
        :return: NextToken:  A generic string.<br/>ReportDefinitions:  A list of AWS Cost and Usage reports owned by the account.<br/>        """
        pass

    def put_report_definition(self, ReportDefinition: ReportDefinition) -> PutReportDefinitionResponse:
        """Creates a new report using the description that you provide.

        :param self:
        :param ReportDefinition:  Represents the output of the PutReportDefinition operation. The content consists of the detailed metadata and data file information. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReportVersioning</i> Whether you want Amazon Web Services to overwrite the previous version of each report or to deliver the report in addition to the previous versions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RefreshClosedReports</i> Whether you want Amazon Web Services to update your reports after they have been finalized if Amazon Web Services detects charges related to previous months. These charges can include refunds, credits, or support fees.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdditionalArtifacts</i> A list of manifests that you want Amazon Web Services to create for this report.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Region</i> The region of the S3 bucket that AWS delivers the report into.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Prefix</i> The prefix that AWS adds to the report name when AWS delivers the report. Your prefix can't include spaces.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Bucket</i> The S3 bucket where AWS delivers the report.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdditionalSchemaElements</i> A list of strings that indicate additional content that Amazon Web Services includes in the report, such as individual resource IDs. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Compression</i> The compression format that AWS uses for the report.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> The format that AWS saves the report in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeUnit</i> The length of time covered by the report. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReportName</i> The name of the report that you want to create. The name must be unique, is case sensitive, and can't include spaces. <br/>
        :return:         """
        pass



class DataPipeline(BaseClient):
    """AWS Data Pipeline configures and manages a data-driven workflow called a pipeline. AWS Data Pipeline handles the details of scheduling and ensuring that data dependencies are met so that your application can focus on processing the data. AWS Data Pipeline provides a JAR implementation of a task runner called AWS Data Pipeline Task Runner. AWS Data Pipeline Task Runner provides logic for common data management scenarios, such as performing database queries and running data analysis using Amazon Elastic MapReduce (Amazon EMR). You can use AWS Data Pipeline Task Runner as your task runner, or you can write your own task runner to provide custom data management. AWS Data Pipeline implements two main sets of functionality. Use the first set to create a pipeline and define data sources, schedules, dependencies, and the transforms to be performed on the data. Use the second set in your task runner application to receive the next task ready for processing. The logic for performing the task, such as querying the data, running data analysis, or converting the data from one format to another, is contained within the task runner. The task runner performs the task assigned to it by the web service, reporting progress to the web service as it does so. When the task is done, the task runner reports the final success or failure of the task to the web service."""


    class ListPipelinesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_pipelines` for valid parameters"""
            pass
    class DescribeObjectsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_objects` for valid parameters"""
            pass
    class QueryObjectsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `query_objects` for valid parameters"""
            pass


    class ActivatePipelineInput(Mapping):
        pass
    
    class ActivatePipelineOutput(Mapping):
        pass
    
    class ParameterValue(Mapping):
        pass
    
    class timestamp(object):
        pass
    
    class AddTagsInput(Mapping):
        pass
    
    class AddTagsOutput(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreatePipelineInput(Mapping):
        pass
    
    class CreatePipelineOutput(Mapping):
        pass
    
    class DeactivatePipelineInput(Mapping):
        pass
    
    class DeactivatePipelineOutput(Mapping):
        pass
    
    class cancelActive(object):
        pass
    
    class DeletePipelineInput(Mapping):
        pass
    
    class DescribeObjectsInput(Mapping):
        pass
    
    class DescribeObjectsOutput(Mapping):
        pass
    
    class id(object):
        pass
    
    class boolean(object):
        pass
    
    class DescribePipelinesInput(Mapping):
        pass
    
    class DescribePipelinesOutput(Mapping):
        pass
    
    class EvaluateExpressionInput(Mapping):
        pass
    
    class EvaluateExpressionOutput(Mapping):
        pass
    
    class GetPipelineDefinitionInput(Mapping):
        pass
    
    class GetPipelineDefinitionOutput(Mapping):
        pass
    
    class ListPipelinesInput(Mapping):
        pass
    
    class ListPipelinesOutput(Mapping):
        pass
    
    class PollForTaskInput(Mapping):
        pass
    
    class PollForTaskOutput(Mapping):
        pass
    
    class InstanceIdentity(Mapping):
        pass
    
    class PutPipelineDefinitionInput(Mapping):
        pass
    
    class PutPipelineDefinitionOutput(Mapping):
        pass
    
    class PipelineObject(Mapping):
        pass
    
    class ParameterObject(Mapping):
        pass
    
    class QueryObjectsInput(Mapping):
        pass
    
    class QueryObjectsOutput(Mapping):
        pass
    
    class Query(Mapping):
        pass
    
    class RemoveTagsInput(Mapping):
        pass
    
    class RemoveTagsOutput(Mapping):
        pass
    
    class string(object):
        pass
    
    class ReportTaskProgressInput(Mapping):
        pass
    
    class ReportTaskProgressOutput(Mapping):
        pass
    
    class Field(Mapping):
        pass
    
    class ReportTaskRunnerHeartbeatInput(Mapping):
        pass
    
    class ReportTaskRunnerHeartbeatOutput(Mapping):
        pass
    
    class SetStatusInput(Mapping):
        pass
    
    class SetTaskStatusInput(Mapping):
        pass
    
    class SetTaskStatusOutput(Mapping):
        pass
    
    class ValidatePipelineDefinitionInput(Mapping):
        pass
    
    class ValidatePipelineDefinitionOutput(Mapping):
        pass
    
    def activate_pipeline(self, pipelineId: str, parameterValues: List[ParameterValue]=None, startTimestamp: timestamp=None) -> ActivatePipelineOutput:
        """Validates the specified pipeline and starts processing pipeline tasks. If the pipeline does not pass validation, activation fails. If you need to pause the pipeline to investigate an issue with a component, such as a data source or script, call <a>DeactivatePipeline</a>. To activate a finished pipeline, modify the end date for the pipeline and then activate it.

        :param self:
        :param startTimestamp:  The date and time to resume the pipeline. By default, the pipeline resumes from the last completed execution.<br/>parameterValues:  A list of parameter values to pass to the pipeline at activation.<br/>pipelineId:  The ID of the pipeline.<br/>
        :return:         """
        pass

    def add_tags(self, tags: List[Tag], pipelineId: str) -> AddTagsOutput:
        """Adds or modifies tags for the specified pipeline.

        :param self:
        :param tags:  The tags to add, as key/value pairs.<br/>pipelineId:  The ID of the pipeline.<br/>
        :return:         """
        pass

    def create_pipeline(self, uniqueId: str, name: str, description: str=None, tags: List[Tag]=None) -> CreatePipelineOutput:
        """Creates a new, empty pipeline. Use <a>PutPipelineDefinition</a> to populate the pipeline.

        :param self:
        :param tags:  A list of tags to associate with the pipeline at creation. Tags let you control access to pipelines. For more information, see <a href="http://docs.aws.amazon.com/datapipeline/latest/DeveloperGuide/dp-control-access.html">Controlling User Access to Pipelines</a> in the <i>AWS Data Pipeline Developer Guide</i>.<br/>description:  The description for the pipeline.<br/>uniqueId:  A unique identifier. This identifier is not the same as the pipeline identifier assigned by AWS Data Pipeline. You are responsible for defining the format and ensuring the uniqueness of this identifier. You use this parameter to ensure idempotency during repeated calls to <code>CreatePipeline</code>. For example, if the first call to <code>CreatePipeline</code> does not succeed, you can pass in the same unique identifier and pipeline name combination on a subsequent call to <code>CreatePipeline</code>. <code>CreatePipeline</code> ensures that if a pipeline already exists with the same name and unique identifier, a new pipeline is not created. Instead, you'll receive the pipeline identifier from the previous attempt. The uniqueness of the name and unique identifier combination is scoped to the AWS account or IAM user credentials.<br/>name:  The name for the pipeline. You can use the same name for multiple pipelines associated with your AWS account, because AWS Data Pipeline assigns each pipeline a unique pipeline identifier.<br/>
        :return: pipelineId:  The ID that AWS Data Pipeline assigns the newly created pipeline. For example, <code>df-06372391ZG65EXAMPLE</code>.<br/>        """
        pass

    def deactivate_pipeline(self, pipelineId: str, cancelActive: cancelActive=None) -> DeactivatePipelineOutput:
        """Deactivates the specified running pipeline. The pipeline is set to the <code>DEACTIVATING</code> state until the deactivation process completes. To resume a deactivated pipeline, use <a>ActivatePipeline</a>. By default, the pipeline resumes from the last completed execution. Optionally, you can specify the date and time to resume the pipeline.

        :param self:
        :param cancelActive:  Indicates whether to cancel any running objects. The default is true, which sets the state of any running objects to <code>CANCELED</code>. If this value is false, the pipeline is deactivated after all running objects finish.<br/>pipelineId:  The ID of the pipeline.<br/>
        :return:         """
        pass

    def delete_pipeline(self, pipelineId: str):
        """Deletes a pipeline, its pipeline definition, and its run history. AWS Data Pipeline attempts to cancel instances associated with the pipeline that are currently being processed by task runners. Deleting a pipeline cannot be undone. You cannot query or restore a deleted pipeline. To temporarily pause a pipeline instead of deleting it, call <a>SetStatus</a> with the status set to <code>PAUSE</code> on individual components. Components that are paused by <a>SetStatus</a> can be resumed.

        :param self:
        :param pipelineId:  The ID of the pipeline.<br/>
        :return:         """
        pass

    def describe_objects(self, objectIds: List[id], pipelineId: str, evaluateExpressions: boolean=None, marker: str=None) -> DescribeObjectsOutput:
        """Gets the object definitions for a set of objects associated with the pipeline. Object definitions are composed of a set of fields that define the properties of the object.

        :param self:
        :param marker:  The starting point for the results to be returned. For the first call, this value should be empty. As long as there are more results, continue to call <code>DescribeObjects</code> with the marker value from the previous call to retrieve the next set of results.<br/>evaluateExpressions:  Indicates whether any expressions in the object should be evaluated when the object descriptions are returned.<br/>objectIds:  The IDs of the pipeline objects that contain the definitions to be described. You can pass as many as 25 identifiers in a single call to <code>DescribeObjects</code>.<br/>pipelineId:  The ID of the pipeline that contains the object definitions.<br/>
        :return: hasMoreResults:  Indicates whether there are more results to return.<br/>marker:  The starting point for the next page of results. To view the next page of results, call <code>DescribeObjects</code> again with this marker value. If the value is null, there are no more results.<br/>pipelineObjects:  An array of object definitions.<br/>        """
        pass

    def describe_pipelines(self, pipelineIds: List[id]) -> DescribePipelinesOutput:
        """Retrieves metadata about one or more pipelines. The information retrieved includes the name of the pipeline, the pipeline identifier, its current state, and the user account that owns the pipeline. Using account credentials, you can retrieve metadata about pipelines that you or your IAM users have created. If you are using an IAM user account, you can retrieve metadata about only those pipelines for which you have read permissions. To retrieve the full pipeline definition instead of metadata about the pipeline, call <a>GetPipelineDefinition</a>.

        :param self:
        :param pipelineIds:  The IDs of the pipelines to describe. You can pass as many as 25 identifiers in a single call. To obtain pipeline IDs, call <a>ListPipelines</a>.<br/>
        :return: pipelineDescriptionList:  An array of descriptions for the specified pipelines.<br/>        """
        pass

    def evaluate_expression(self, expression: str, objectId: str, pipelineId: str) -> EvaluateExpressionOutput:
        """Task runners call <code>EvaluateExpression</code> to evaluate a string in the context of the specified object. For example, a task runner can evaluate SQL queries stored in Amazon S3.

        :param self:
        :param expression:  The expression to evaluate.<br/>objectId:  The ID of the object.<br/>pipelineId:  The ID of the pipeline.<br/>
        :return: evaluatedExpression:  The evaluated expression.<br/>        """
        pass

    def get_pipeline_definition(self, pipelineId: str, version: str=None) -> GetPipelineDefinitionOutput:
        """Gets the definition of the specified pipeline. You can call <code>GetPipelineDefinition</code> to retrieve the pipeline definition that you provided using <a>PutPipelineDefinition</a>.

        :param self:
        :param version:  The version of the pipeline definition to retrieve. Set this parameter to <code>latest</code> (default) to use the last definition saved to the pipeline or <code>active</code> to use the last definition that was activated.<br/>pipelineId:  The ID of the pipeline.<br/>
        :return: parameterValues:  The parameter values used in the pipeline definition.<br/>parameterObjects:  The parameter objects used in the pipeline definition.<br/>pipelineObjects:  The objects defined in the pipeline.<br/>        """
        pass

    def list_pipelines(self, marker: str=None) -> ListPipelinesOutput:
        """Lists the pipeline identifiers for all active pipelines that you have permission to access.

        :param self:
        :param marker:  The starting point for the results to be returned. For the first call, this value should be empty. As long as there are more results, continue to call <code>ListPipelines</code> with the marker value from the previous call to retrieve the next set of results.<br/>
        :return: hasMoreResults:  Indicates whether there are more results that can be obtained by a subsequent call.<br/>marker:  The starting point for the next page of results. To view the next page of results, call <code>ListPipelinesOutput</code> again with this marker value. If the value is null, there are no more results.<br/>pipelineIdList:  The pipeline identifiers. If you require additional information about the pipelines, you can use these identifiers to call <a>DescribePipelines</a> and <a>GetPipelineDefinition</a>.<br/>        """
        pass

    def poll_for_task(self, workerGroup: str, hostname: str=None, instanceIdentity: InstanceIdentity=None) -> PollForTaskOutput:
        """Task runners call <code>PollForTask</code> to receive a task to perform from AWS Data Pipeline. The task runner specifies which tasks it can perform by setting a value for the <code>workerGroup</code> parameter. The task returned can come from any of the pipelines that match the <code>workerGroup</code> value passed in by the task runner and that was launched using the IAM user credentials specified by the task runner. If tasks are ready in the work queue, <code>PollForTask</code> returns a response immediately. If no tasks are available in the queue, <code>PollForTask</code> uses long-polling and holds on to a poll connection for up to a 90 seconds, during which time the first newly scheduled task is handed to the task runner. To accomodate this, set the socket timeout in your task runner to 90 seconds. The task runner should not call <code>PollForTask</code> again on the same <code>workerGroup</code> until it receives a response, and this can take up to 90 seconds. 

        :param self:
        :param instanceIdentity:  Identity information for the EC2 instance that is hosting the task runner. You can get this value from the instance using <code>http://169.254.169.254/latest/meta-data/instance-id</code>. For more information, see <a href="http://docs.aws.amazon.com/AWSEC2/latest/UserGuide/AESDG-chapter-instancedata.html">Instance Metadata</a> in the <i>Amazon Elastic Compute Cloud User Guide.</i> Passing in this value proves that your task runner is running on an EC2 instance, and ensures the proper AWS Data Pipeline service charges are applied to your pipeline.:<br/>hostname:  The public DNS name of the calling task runner.<br/>workerGroup:  The type of task the task runner is configured to accept and process. The worker group is set as a field on objects in the pipeline when they are created. You can only specify a single value for <code>workerGroup</code> in the call to <code>PollForTask</code>. There are no wildcard values permitted in <code>workerGroup</code>; the string must be an exact, case-sensitive, match.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>signature</i> A signature which can be used to verify the accuracy and authenticity of the information provided in the instance identity document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>document</i> A description of an EC2 instance that is generated when the instance is launched and exposed to the instance via the instance metadata service in the form of a JSON representation of an object.<br/>
        :return: taskObject:  The information needed to complete the task that is being assigned to the task runner. One of the fields returned in this object is <code>taskId</code>, which contains an identifier for the task being assigned. The calling task runner uses <code>taskId</code> in subsequent calls to <a>ReportTaskProgress</a> and <a>SetTaskStatus</a>.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>objects</i> Connection information for the location where the task runner will publish the output of the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>attemptId</i> The ID of the pipeline task attempt object. AWS Data Pipeline uses this value to track how many times a task is attempted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pipelineId</i> The ID of the pipeline that provided the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>taskId</i> An internal identifier for the task. This ID is passed to the <a>SetTaskStatus</a> and <a>ReportTaskProgress</a> actions.<br/>        """
        pass

    def put_pipeline_definition(self, pipelineObjects: List[PipelineObject], pipelineId: str, parameterObjects: List[ParameterObject]=None, parameterValues: List[ParameterValue]=None) -> PutPipelineDefinitionOutput:
        """Adds tasks, schedules, and preconditions to the specified pipeline. You can use <code>PutPipelineDefinition</code> to populate a new pipeline.  <code>PutPipelineDefinition</code> also validates the configuration as it adds it to the pipeline. Changes to the pipeline are saved unless one of the following three validation errors exists in the pipeline.  <ol> <li>An object is missing a name or identifier field.</li> <li>A string or reference field is empty.</li> <li>The number of objects in the pipeline exceeds the maximum allowed objects.</li> <li>The pipeline is in a FINISHED state.</li> </ol>  Pipeline object definitions are passed to the <code>PutPipelineDefinition</code> action and returned by the <a>GetPipelineDefinition</a> action. 

        :param self:
        :param parameterValues:  The parameter values used with the pipeline.<br/>parameterObjects:  The parameter objects used with the pipeline.<br/>pipelineObjects:  The objects that define the pipeline. These objects overwrite the existing pipeline definition.<br/>pipelineId:  The ID of the pipeline.<br/>
        :return: errored:  Indicates whether there were validation errors, and the pipeline definition is stored but cannot be activated until you correct the pipeline and call <code>PutPipelineDefinition</code> to commit the corrected pipeline.<br/>validationWarnings:  The validation warnings that are associated with the objects defined in <code>pipelineObjects</code>.<br/>validationErrors:  The validation errors that are associated with the objects defined in <code>pipelineObjects</code>.<br/>        """
        pass

    def query_objects(self, sphere: str, pipelineId: str, query: Query=None, marker: str=None, limit: int=None) -> QueryObjectsOutput:
        """Queries the specified pipeline for the names of objects that match the specified set of conditions.

        :param self:
        :param limit:  The maximum number of object names that <code>QueryObjects</code> will return in a single call. The default value is 100. <br/>marker:  The starting point for the results to be returned. For the first call, this value should be empty. As long as there are more results, continue to call <code>QueryObjects</code> with the marker value from the previous call to retrieve the next set of results.<br/>sphere:  Indicates whether the query applies to components or instances. The possible values are: <code>COMPONENT</code>, <code>INSTANCE</code>, and <code>ATTEMPT</code>.<br/>query:  The query that defines the objects to be returned. The <code>Query</code> object can contain a maximum of ten selectors. The conditions in the query are limited to top-level String fields in the object. These filters can be applied to components, instances, and attempts.:<br/>pipelineId:  The ID of the pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>selectors</i> List of selectors that define the query. An object must satisfy all of the selectors to match the query.<br/>
        :return: hasMoreResults:  Indicates whether there are more results that can be obtained by a subsequent call.<br/>marker:  The starting point for the next page of results. To view the next page of results, call <code>QueryObjects</code> again with this marker value. If the value is null, there are no more results.<br/>ids:  The identifiers that match the query selectors.<br/>        """
        pass

    def remove_tags(self, tagKeys: List[string], pipelineId: str) -> RemoveTagsOutput:
        """Removes existing tags from the specified pipeline.

        :param self:
        :param tagKeys:  The keys of the tags to remove.<br/>pipelineId:  The ID of the pipeline.<br/>
        :return:         """
        pass

    def report_task_progress(self, taskId: str, fields: List[Field]=None) -> ReportTaskProgressOutput:
        """Task runners call <code>ReportTaskProgress</code> when assigned a task to acknowledge that it has the task. If the web service does not receive this acknowledgement within 2 minutes, it assigns the task in a subsequent <a>PollForTask</a> call. After this initial acknowledgement, the task runner only needs to report progress every 15 minutes to maintain its ownership of the task. You can change this reporting time from 15 minutes by specifying a <code>reportProgressTimeout</code> field in your pipeline. If a task runner does not report its status after 5 minutes, AWS Data Pipeline assumes that the task runner is unable to process the task and reassigns the task in a subsequent response to <a>PollForTask</a>. Task runners should call <code>ReportTaskProgress</code> every 60 seconds.

        :param self:
        :param fields:  Key-value pairs that define the properties of the ReportTaskProgressInput object.<br/>taskId:  The ID of the task assigned to the task runner. This value is provided in the response for <a>PollForTask</a>.<br/>
        :return: canceled:  If true, the calling task runner should cancel processing of the task. The task runner does not need to call <a>SetTaskStatus</a> for canceled tasks.<br/>        """
        pass

    def report_task_runner_heartbeat(self, taskrunnerId: str, workerGroup: str=None, hostname: str=None) -> ReportTaskRunnerHeartbeatOutput:
        """Task runners call <code>ReportTaskRunnerHeartbeat</code> every 15 minutes to indicate that they are operational. If the AWS Data Pipeline Task Runner is launched on a resource managed by AWS Data Pipeline, the web service can use this call to detect when the task runner application has failed and restart a new instance.

        :param self:
        :param hostname:  The public DNS name of the task runner.<br/>workerGroup:  The type of task the task runner is configured to accept and process. The worker group is set as a field on objects in the pipeline when they are created. You can only specify a single value for <code>workerGroup</code>. There are no wildcard values permitted in <code>workerGroup</code>; the string must be an exact, case-sensitive, match.<br/>taskrunnerId:  The ID of the task runner. This value should be unique across your AWS account. In the case of AWS Data Pipeline Task Runner launched on a resource managed by AWS Data Pipeline, the web service provides a unique identifier when it launches the application. If you have written a custom task runner, you should assign a unique identifier for the task runner.<br/>
        :return: terminate:  Indicates whether the calling task runner should terminate.<br/>        """
        pass

    def set_status(self, status: str, objectIds: List[id], pipelineId: str):
        """Requests that the status of the specified physical or logical pipeline objects be updated in the specified pipeline. This update might not occur immediately, but is eventually consistent. The status that can be set depends on the type of object (for example, DataNode or Activity). You cannot perform this operation on <code>FINISHED</code> pipelines and attempting to do so returns <code>InvalidRequestException</code>.

        :param self:
        :param status:  The status to be set on all the objects specified in <code>objectIds</code>. For components, use <code>PAUSE</code> or <code>RESUME</code>. For instances, use <code>TRY_CANCEL</code>, <code>RERUN</code>, or <code>MARK_FINISHED</code>.<br/>objectIds:  The IDs of the objects. The corresponding objects can be either physical or components, but not a mix of both types.<br/>pipelineId:  The ID of the pipeline that contains the objects.<br/>
        :return:         """
        pass

    def set_task_status(self, taskStatus: str, taskId: str, errorId: str=None, errorMessage: str=None, errorStackTrace: str=None) -> SetTaskStatusOutput:
        """Task runners call <code>SetTaskStatus</code> to notify AWS Data Pipeline that a task is completed and provide information about the final status. A task runner makes this call regardless of whether the task was sucessful. A task runner does not need to call <code>SetTaskStatus</code> for tasks that are canceled by the web service during a call to <a>ReportTaskProgress</a>.

        :param self:
        :param errorStackTrace:  If an error occurred during the task, this value specifies the stack trace associated with the error. This value is set on the physical attempt object. It is used to display error information to the user. The web service does not parse this value.<br/>errorMessage:  If an error occurred during the task, this value specifies a text description of the error. This value is set on the physical attempt object. It is used to display error information to the user. The web service does not parse this value.<br/>errorId:  If an error occurred during the task, this value specifies the error code. This value is set on the physical attempt object. It is used to display error information to the user. It should not start with string "Service_" which is reserved by the system.<br/>taskStatus:  If <code>FINISHED</code>, the task successfully completed. If <code>FAILED</code>, the task ended unsuccessfully. Preconditions use false.<br/>taskId:  The ID of the task assigned to the task runner. This value is provided in the response for <a>PollForTask</a>.<br/>
        :return:         """
        pass

    def validate_pipeline_definition(self, pipelineObjects: List[PipelineObject], pipelineId: str, parameterObjects: List[ParameterObject]=None, parameterValues: List[ParameterValue]=None) -> ValidatePipelineDefinitionOutput:
        """Validates the specified pipeline definition to ensure that it is well formed and can be run without error.

        :param self:
        :param parameterValues:  The parameter values used with the pipeline.<br/>parameterObjects:  The parameter objects used with the pipeline.<br/>pipelineObjects:  The objects that define the pipeline changes to validate against the pipeline.<br/>pipelineId:  The ID of the pipeline.<br/>
        :return: errored:  Indicates whether there were validation errors.<br/>validationWarnings:  Any validation warnings that were found.<br/>validationErrors:  Any validation errors that were found.<br/>        """
        pass



class DataSync(BaseClient):
    """<fullname>AWS DataSync</fullname> AWS DataSync is a managed data transfer service that makes it simpler for you to automate moving data between on-premises storage and Amazon Simple Storage Service (Amazon S3) or Amazon Elastic File System (Amazon EFS).  This API interface reference for AWS DataSync contains documentation for a programming interface that you can use to manage AWS DataSync."""


    class ListAgentsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_agents` for valid parameters"""
            pass
    class ListLocationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_locations` for valid parameters"""
            pass
    class ListTagsForResourcePaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_tags_for_resource` for valid parameters"""
            pass
    class ListTaskExecutionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_task_executions` for valid parameters"""
            pass
    class ListTasksPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_tasks` for valid parameters"""
            pass


    class CancelTaskExecutionRequest(Mapping):
        pass
    
    class CancelTaskExecutionResponse(Mapping):
        pass
    
    class CreateAgentRequest(Mapping):
        pass
    
    class CreateAgentResponse(Mapping):
        pass
    
    class TagListEntry(Mapping):
        pass
    
    class CreateLocationEfsRequest(Mapping):
        pass
    
    class CreateLocationEfsResponse(Mapping):
        pass
    
    class Ec2Config(Mapping):
        pass
    
    class CreateLocationNfsRequest(Mapping):
        pass
    
    class CreateLocationNfsResponse(Mapping):
        pass
    
    class OnPremConfig(Mapping):
        pass
    
    class NfsMountOptions(Mapping):
        pass
    
    class CreateLocationS3Request(Mapping):
        pass
    
    class CreateLocationS3Response(Mapping):
        pass
    
    class S3Config(Mapping):
        pass
    
    class CreateTaskRequest(Mapping):
        pass
    
    class CreateTaskResponse(Mapping):
        pass
    
    class Options(Mapping):
        pass
    
    class FilterRule(Mapping):
        pass
    
    class DeleteAgentRequest(Mapping):
        pass
    
    class DeleteAgentResponse(Mapping):
        pass
    
    class DeleteLocationRequest(Mapping):
        pass
    
    class DeleteLocationResponse(Mapping):
        pass
    
    class DeleteTaskRequest(Mapping):
        pass
    
    class DeleteTaskResponse(Mapping):
        pass
    
    class DescribeAgentRequest(Mapping):
        pass
    
    class DescribeAgentResponse(Mapping):
        pass
    
    class DescribeLocationEfsRequest(Mapping):
        pass
    
    class DescribeLocationEfsResponse(Mapping):
        pass
    
    class DescribeLocationNfsRequest(Mapping):
        pass
    
    class DescribeLocationNfsResponse(Mapping):
        pass
    
    class DescribeLocationS3Request(Mapping):
        pass
    
    class DescribeLocationS3Response(Mapping):
        pass
    
    class DescribeTaskRequest(Mapping):
        pass
    
    class DescribeTaskResponse(Mapping):
        pass
    
    class DescribeTaskExecutionRequest(Mapping):
        pass
    
    class DescribeTaskExecutionResponse(Mapping):
        pass
    
    class ListAgentsRequest(Mapping):
        pass
    
    class ListAgentsResponse(Mapping):
        pass
    
    class ListLocationsRequest(Mapping):
        pass
    
    class ListLocationsResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class ListTaskExecutionsRequest(Mapping):
        pass
    
    class ListTaskExecutionsResponse(Mapping):
        pass
    
    class ListTasksRequest(Mapping):
        pass
    
    class ListTasksResponse(Mapping):
        pass
    
    class StartTaskExecutionRequest(Mapping):
        pass
    
    class StartTaskExecutionResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateAgentRequest(Mapping):
        pass
    
    class UpdateAgentResponse(Mapping):
        pass
    
    class UpdateTaskRequest(Mapping):
        pass
    
    class UpdateTaskResponse(Mapping):
        pass
    
    def cancel_task_execution(self, TaskExecutionArn: str) -> CancelTaskExecutionResponse:
        """Cancels execution of a task.  When you cancel a task execution, the transfer of some files are abruptly interrupted. The contents of files that are transferred to the destination might be incomplete or inconsistent with the source files. However, if you start a new task execution on the same task and you allow the task execution to complete, file content on the destination is complete and consistent. This applies to other unexpected failures that interrupt a task execution. In all of these cases, AWS DataSync successfully complete the transfer when you start the next task execution.

        :param self:
        :param TaskExecutionArn:  The Amazon Resource Name (ARN) of the task execution to cancel.<br/>
        :return:         """
        pass

    def create_agent(self, ActivationKey: str, AgentName: str=None, Tags: List[TagListEntry]=None) -> CreateAgentResponse:
        """Activates an AWS DataSync agent that you have deployed on your host. The activation process associates your agent with your account. In the activation process, you specify information such as the AWS Region that you want to activate the agent in. You activate the agent in the AWS Region where your target locations (in Amazon S3 or Amazon EFS) reside. Your tasks are created in this AWS Region.  You can use an agent for more than one location. If a task uses multiple agents, all of them need to have status AVAILABLE for the task to run. If you use multiple agents for a source location, the status of all the agents must be AVAILABLE for the task to run.  For more information, see "https://docs.aws.amazon.com/datasync/latest/userguide/working-with-agents.html#activating-agent" (Activating an Agent) in the <i>AWS DataSync User Guide.</i>  Agents are automatically updated by AWS on a regular basis, using a mechanism that ensures minimal interruption to your tasks. <p/>

        :param self:
        :param Tags:  The key-value pair that represents the tag that you want to associate with the agent. The value can be an empty string. This value helps you manage, filter, and search for your agents. <note> Valid characters for key and value are letters, spaces, and numbers representable in UTF-8 format, and the following special characters: + - = . _ : / @.  </note><br/>AgentName:  The name you configured for your agent. This value is a text reference that is used to identify the agent in the console.<br/>ActivationKey:  Your agent activation key. You can get the activation key either by sending an HTTP GET request with redirects that enable you to get the agent IP address (port 80). Alternatively, you can get it from the AWS DataSync console.  The redirect URL returned in the response provides you the activation key for your agent in the query string parameter <code>activationKey</code>. It might also include other activation-related parameters; however, these are merely defaults. The arguments you pass to this API call determine the actual configuration of your agent. For more information, see "https://docs.aws.amazon.com/datasync/latest/userguide/working-with-agents.html#activating-agent" (Activating a Agent) in the <i>AWS DataSync User Guide.</i> <br/>
        :return: AgentArn:  The Amazon Resource Name (ARN) of the agent. Use the <code>ListAgents</code> operation to return a list of agents for your account and AWS Region.<br/>        """
        pass

    def create_location_efs(self, Ec2Config: Ec2Config, EfsFilesystemArn: str, Subdirectory: str=None, Tags: List[TagListEntry]=None) -> CreateLocationEfsResponse:
        """Creates an endpoint for an Amazon EFS file system.

        :param self:
        :param Tags:  The key-value pair that represents a tag that you want to add to the resource. The value can be an empty string. This value helps you manage, filter, and search for your resources. We recommend that you create a name tag for your location.<br/>Ec2Config:  The subnet and security group that the Amazon EFS file system uses. The security group that you provide needs to be able to communicate with the security group on the mount target in the subnet specified. The exact relationship between security group M (of the mount target) and security group S (which you provide for DataSync to use at this stage) is as follows:  <ul> <li>  Security group M (which you associate with the mount target) must allow inbound access for the Transmission Control Protocol (TCP) on the NFS port (2049) from security group S. You can enable inbound connections either by IP address (CIDR range) or security group.  </li> <li> Security group S (provided to DataSync to access EFS) should have a rule that enables outbound connections to the NFS port on one of the file system’s mount targets. You can enable outbound connections either by IP address (CIDR range) or security group. For information about security groups and mount targets, see "https://docs.aws.amazon.com/efs/latest/ug/security-considerations.html#network-access" (Security Groups for Amazon EC2 Instances and Mount Targets) in the <i>Amazon EFS User Guide</i>. </li> </ul>:<br/>EfsFilesystemArn:  The Amazon Resource Name (ARN) for the Amazon EFS file system.<br/>Subdirectory:  A subdirectory in the location’s path. This subdirectory in the EFS file system is used to read data from the EFS source location or write data to the EFS destination. By default, AWS DataSync uses the root directory.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupArns</i> The Amazon Resource Names (ARNs) of the security groups that are configured for the Amazon EC2 resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetArn</i> The ARN of the subnet and the security group that DataSync uses to access the target EFS file system.<br/>
        :return: LocationArn:  The Amazon Resource Name (ARN) of the Amazon EFS file system location that is created.<br/>        """
        pass

    def create_location_nfs(self, OnPremConfig: OnPremConfig, ServerHostname: str, Subdirectory: str, MountOptions: NfsMountOptions=None, Tags: List[TagListEntry]=None) -> CreateLocationNfsResponse:
        """Creates an endpoint for a Network File System (NFS) file system.

        :param self:
        :param Tags:  The key-value pair that represents the tag that you want to add to the location. The value can be an empty string. We recommend using tags to name your resources.<br/>MountOptions:  The NFS mount options that DataSync can use to mount your NFS share.:<br/>OnPremConfig:  Contains a list of Amazon Resource Names (ARNs) of agents that are used to connect to an NFS server.:<br/>ServerHostname:  The name of the NFS server. This value is the IP address or Domain Name Service (DNS) name of the NFS server. An agent that is installed on-premises uses this host name to mount the NFS server in a network.  <note> This name must either be DNS-compliant or must be an IP version 4 (IPv4) address. </note><br/>Subdirectory:  The subdirectory in the NFS file system that is used to read data from the NFS source location or write data to the NFS destination. The NFS path should be a path that's exported by the NFS server, or a subdirectory of that path. The path should be such that it can be mounted by other NFS clients in your network.  To see all the paths exported by your NFS server. run "<code>showmount -e nfs-server-name</code>" from an NFS client that has access to your server. You can specify any directory that appears in the results, and any subdirectory of that directory. Ensure that the NFS export is accessible without Kerberos authentication.  To transfer all the data in the folder you specified, DataSync needs to have permissions to read all the data. To ensure this, either configure the NFS export with <code>no_root_squash,</code> or ensure that the permissions for all of the files that you want DataSync allow read access for all users. Doing either enables the agent to read the files. For the agent to access directories, you must additionally enable all execute access. For information about NFS export configuration, see "http://web.mit.edu/rhel-doc/5/RHEL-5-manual/Deployment_Guide-en-US/s1-nfs-server-config-exports.html" (18.7. The /etc/exports Configuration File).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AgentArns</i> ARNs)of the agents to use for an NFS location.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The specific NFS version that you want DataSync to use to mount your NFS share. If you don't specify a version, DataSync defaults to <code>AUTOMATIC</code>. That is, DataSync automatically selects a version based on negotiation with the NFS server.<br/>
        :return: LocationArn:  The Amazon Resource Name (ARN) of the source NFS file system location that is created.<br/>        """
        pass

    def create_location_s3(self, S3Config: S3Config, S3BucketArn: str, Subdirectory: str=None, Tags: List[TagListEntry]=None) -> CreateLocationS3Response:
        """Creates an endpoint for an Amazon S3 bucket. For AWS DataSync to access a destination S3 bucket, it needs an AWS Identity and Access Management (IAM) role that has the required permissions. You can set up the required permissions by creating an IAM policy that grants the required permissions and attaching the policy to the role. An example of such a policy is shown in the examples section. For more information, see "https://docs.aws.amazon.com/datasync/latest/userguide/working-with-locations.html#create-s3-location" (Configuring Amazon S3 Location Settings) in the <i>AWS DataSync User Guide</i>.

        :param self:
        :param Tags:  The key-value pair that represents the tag that you want to add to the location. The value can be an empty string. We recommend using tags to name your resources.<br/>S3Config:  The Amazon Resource Name (ARN) of the AWS Identity and Access Management (IAM) role that is used to access an Amazon S3 bucket. For detailed information about using such a role, see "https://docs.aws.amazon.com/datasync/latest/userguide/working-with-locations.html#create-s3-location" (Creating a Location for Amazon S3) in the <i>AWS DataSync User Guide</i>.:<br/>S3BucketArn:  The Amazon Resource Name (ARN) of the Amazon S3 bucket.<br/>Subdirectory:  A subdirectory in the Amazon S3 bucket. This subdirectory in Amazon S3 is used to read data from the S3 source location or write data to the S3 destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketAccessRoleArn</i> The Amazon S3 bucket to access. This bucket is used as a parameter in the <a>CreateLocationS3</a> operation. <br/>
        :return: LocationArn:  The Amazon Resource Name (ARN) of the source Amazon S3 bucket location that is created.<br/>        """
        pass

    def create_task(self, DestinationLocationArn: str, SourceLocationArn: str, CloudWatchLogGroupArn: str=None, Name: str=None, Options: Options=None, Excludes: List[FilterRule]=None, Tags: List[TagListEntry]=None) -> CreateTaskResponse:
        """Creates a task. A task is a set of two locations (source and destination) and a set of Options that you use to control the behavior of a task. If you don't specify Options when you create a task, AWS DataSync populates them with service defaults. When you create a task, it first enters the CREATING state. During CREATING AWS DataSync attempts to mount the on-premises Network File System (NFS) location. The task transitions to the AVAILABLE state without waiting for the AWS location to become mounted. If required, AWS DataSync mounts the AWS location before each task execution. If an agent that is associated with a source (NFS) location goes offline, the task transitions to the UNAVAILABLE status. If the status of the task remains in the CREATING status for more than a few minutes, it means that your agent might be having trouble mounting the source NFS file system. Check the task's ErrorCode and ErrorDetail. Mount issues are often caused by either a misconfigured firewall or a mistyped NFS server host name.

        :param self:
        :param Tags:  The key-value pair that represents the tag that you want to add to the resource. The value can be an empty string. <br/>Excludes:  <p/> A filter that determines which files to exclude from a task based on the specified pattern. Transfers all files in the task’s subdirectory, except files that match the filter that is set. <br/>Options:  The set of configuration options that control the behavior of a single execution of the task that occurs when you call <code>StartTaskExecution</code>. You can configure these options to preserve metadata such as user ID (UID) and group ID (GID), file permissions, data integrity verification, and so on. For each individual task execution, you can override these options by specifying the <code>OverrideOptions</code> before starting a the task execution. For more information, see the operation. :<br/>Name:  The name of a task. This value is a text reference that is used to identify the task in the console. <br/>CloudWatchLogGroupArn:  The Amazon Resource Name (ARN) of the Amazon CloudWatch log group that is used to monitor and log events in the task.  For more information on these groups, see "https://docs.aws.amazon.com/AmazonCloudWatch/latest/logs/Working-with-log-groups-and-streams.html" (Working with Log Groups and Log Streams) in the <i>Amazon CloudWatch User Guide</i>. For more information about how to useCloudWatchLogs with DataSync, see "https://docs.aws.amazon.com/datasync/latest/userguide/monitor-datasync.html" (Monitoring Your Task)<br/>DestinationLocationArn:  The Amazon Resource Name (ARN) of an AWS storage resource's location. <br/>SourceLocationArn:  The Amazon Resource Name (ARN) of the source location for the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BytesPerSecond</i> A value that limits the bandwidth used by AWS DataSync. For example, if you want AWS DataSync to use a maximum of 1 MB, set this value to <code>1048576</code> (<code>=1024*1024</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PosixPermissions</i> A value that determines which users or groups can access a file for a specific purpose such as reading, writing, or execution of the file.  Default value: PRESERVE. PRESERVE: Preserve POSIX-style permissions (recommended). NONE: Ignore permissions.  <note> AWS DataSync can preserve extant permissions of a source location. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreserveDevices</i> A value that determines whether AWS DataSync should preserve the metadata of block and character devices in the source file system, and recreate the files with that device name and metadata on the destination. <note> AWS DataSync can't sync the actual contents of such devices, because they are nonterminal and don't return an end-of-file (EOF) marker. </note> Default value: NONE. NONE: Ignore special devices (recommended).  PRESERVE: Preserve character and block device metadata. This option isn't currently supported for Amazon EFS. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreserveDeletedFiles</i> A value that specifies whether files in the destination that don't exist in the source file system should be preserved.  Default value: PRESERVE. PRESERVE: Ignore such destination files (recommended).  REMOVE: Delete destination files that aren’t present in the source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Gid</i> The group ID (GID) of the file's owners.  Default value: INT_VALUE. This preserves the integer value of the ID. INT_VALUE: Preserve the integer value of user ID (UID) and GID (recommended). NONE: Ignore UID and GID. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Uid</i> The user ID (UID) of the file's owner.  Default value: INT_VALUE. This preserves the integer value of the ID. INT_VALUE: Preserve the integer value of UID and group ID (GID) (recommended). NONE: Ignore UID and GID. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Mtime</i> A value that indicates the last time that a file was modified (that is, a file was written to) before the PREPARING phase.  Default value: PRESERVE.  PRESERVE: Preserve original <code>Mtime</code> (recommended)  NONE: Ignore <code>Mtime</code>.  <note> If <code>Mtime</code> is set to PRESERVE, <code>Atime</code> must be set to BEST_EFFORT. If <code>Mtime</code> is set to NONE, <code>Atime</code> must also be set to NONE.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Atime</i> A file metadata value that shows the last time a file was accessed (that is, when the file was read or written to). If you set <code>Atime</code> to BEST_EFFORT, DataSync attempts to preserve the original <code>Atime</code> attribute on all source files (that is, the version before the PREPARING phase). However, <code>Atime</code>'s behavior is not fully standard across platforms, so AWS DataSync can only do this on a best-effort basis.  Default value: BEST_EFFORT. BEST_EFFORT: Attempt to preserve the per-file <code>Atime</code> value (recommended). NONE: Ignore <code>Atime</code>. <note> If <code>Atime</code> is set to BEST_EFFORT, <code>Mtime</code> must be set to PRESERVE.  If <code>Atime</code> is set to NONE, <code>Mtime</code> must also be NONE.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VerifyMode</i> A value that determines whether a data integrity verification should be performed at the end of a task execution after all data and metadata have been transferred.  Default value: POINT_IN_TIME_CONSISTENT. POINT_IN_TIME_CONSISTENT: Perform verification (recommended).  NONE: Skip verification.<br/>
        :return: TaskArn:  The Amazon Resource Name (ARN) of the task.<br/>        """
        pass

    def delete_agent(self, AgentArn: str) -> DeleteAgentResponse:
        """Deletes an agent. To specify which agent to delete, use the Amazon Resource Name (ARN) of the agent in your request. The operation disassociates the agent from your AWS account. However, it doesn't delete the agent virtual machine (VM) from your on-premises environment.

        :param self:
        :param AgentArn:  The Amazon Resource Name (ARN) of the agent to delete. Use the <code>ListAgents</code> operation to return a list of agents for your account and AWS Region.<br/>
        :return:         """
        pass

    def delete_location(self, LocationArn: str) -> DeleteLocationResponse:
        """Deletes the configuration of a location used by AWS DataSync. 

        :param self:
        :param LocationArn:  The Amazon Resource Name (ARN) of the location to delete.<br/>
        :return:         """
        pass

    def delete_task(self, TaskArn: str) -> DeleteTaskResponse:
        """Deletes a task.

        :param self:
        :param TaskArn:  The Amazon Resource Name (ARN) of the task to delete.<br/>
        :return:         """
        pass

    def describe_agent(self, AgentArn: str) -> DescribeAgentResponse:
        """Returns metadata such as the name, the network interfaces, and the status (that is, whether the agent is running or not) for an agent. To specify which agent to describe, use the Amazon Resource Name (ARN) of the agent in your request. 

        :param self:
        :param AgentArn:  The Amazon Resource Name (ARN) of the agent to describe.<br/>
        :return: PrivateLinkConfig:  :<br/>EndpointOptions:  :<br/>CreationTime:  The time that the agent was activated (that is, created in your account).<br/>LastConnectionTime:  The time that the agent last connected to DataSyc.<br/>Status:  The status of the agent. If the status is ONLINE, then the agent is configured properly and is available to use. The Running status is the normal running status for an agent. If the status is OFFLINE, the agent's VM is turned off or the agent is in an unhealthy state. When the issue that caused the unhealthy state is resolved, the agent returns to ONLINE status.<br/>Name:  The name of the agent.<br/>AgentArn:  The Amazon Resource Name (ARN) of the agent.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrivateLink</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Fips</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupArns</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetArns</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrivateLinkEndpoint</i> <br/>        """
        pass

    def describe_location_efs(self, LocationArn: str) -> DescribeLocationEfsResponse:
        """Returns metadata, such as the path information about an Amazon EFS location.

        :param self:
        :param LocationArn:  The Amazon Resource Name (ARN) of the EFS location to describe.<br/>
        :return: CreationTime:  The time that the EFS location was created.<br/>Ec2Config:  The subnet and the security group that DataSync uses to access target EFS file system. The subnet must have at least one mount target for that file system. The security group that you provide needs to be able to communicate with the security group on the mount target in the subnet specified. :<br/>LocationUri:  The URL of the EFS location that was described.<br/>LocationArn:  The Amazon resource Name (ARN) of the EFS location that was described.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupArns</i> The Amazon Resource Names (ARNs) of the security groups that are configured for the Amazon EC2 resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetArn</i> The ARN of the subnet and the security group that DataSync uses to access the target EFS file system.<br/>        """
        pass

    def describe_location_nfs(self, LocationArn: str) -> DescribeLocationNfsResponse:
        """Returns metadata, such as the path information, about a NFS location.

        :param self:
        :param LocationArn:  The Amazon resource Name (ARN) of the NFS location to describe.<br/>
        :return: CreationTime:  The time that the NFS location was created.<br/>MountOptions:  The NFS mount options that DataSync used to mount your NFS share.:<br/>OnPremConfig:  A list of Amazon Resource Names (ARNs) of agents to use for a Network File System (NFS) location.:<br/>LocationUri:  The URL of the source NFS location that was described.<br/>LocationArn:  The Amazon resource Name (ARN) of the NFS location that was described.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AgentArns</i> ARNs)of the agents to use for an NFS location.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The specific NFS version that you want DataSync to use to mount your NFS share. If you don't specify a version, DataSync defaults to <code>AUTOMATIC</code>. That is, DataSync automatically selects a version based on negotiation with the NFS server.<br/>        """
        pass

    def describe_location_s3(self, LocationArn: str) -> DescribeLocationS3Response:
        """Returns metadata, such as bucket name, about an Amazon S3 bucket location.

        :param self:
        :param LocationArn:  The Amazon Resource Name (ARN) of the Amazon S3 bucket location to describe.<br/>
        :return: CreationTime:  The time that the Amazon S3 bucket location was created.<br/>S3Config:  The Amazon Resource Name (ARN) of the AWS Identity and Access Management (IAM) role that is used to access an Amazon S3 bucket. For detailed information about using such a role, see "https://docs.aws.amazon.com/datasync/latest/userguide/working-with-locations.html#create-s3-location" (Creating a Location for Amazon S3) in the <i>AWS DataSync User Guide</i>.:<br/>LocationUri:  The URL of the Amazon S3 location that was described.<br/>LocationArn:  The Amazon Resource Name (ARN) of the Amazon S3 bucket location.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketAccessRoleArn</i> The Amazon S3 bucket to access. This bucket is used as a parameter in the <a>CreateLocationS3</a> operation. <br/>        """
        pass

    def describe_task(self, TaskArn: str) -> DescribeTaskResponse:
        """Returns metadata about a task.

        :param self:
        :param TaskArn:  The Amazon Resource Name (ARN) of the task to describe.<br/>
        :return: CreationTime:  The time that the task was created.<br/>ErrorDetail:  Detailed description of an error that was encountered during the task execution. You can use this information to help troubleshoot issues. <br/>ErrorCode:  Errors that AWS DataSync encountered during execution of the task. You can use this error code to help troubleshoot issues.<br/>Excludes:  <p/> Specifies that the task excludes files in the transfer based on the specified pattern in the filter. Transfers all files in the task’s subdirectory, except files that match the filter that is set. <br/>Options:  The set of configuration options that control the behavior of a single execution of the task that occurs when you call <code>StartTaskExecution</code>. You can configure these options to preserve metadata such as user ID (UID) and group (GID), file permissions, data integrity verification, and so on. For each individual task execution, you can override these options by specifying the overriding <code>OverrideOptions</code> value to operation. :<br/>CloudWatchLogGroupArn:  The Amazon Resource Name (ARN) of the Amazon CloudWatch log group that was used to monitor and log events in the task. For more information on these groups, see "https://docs.aws.amazon.com/AmazonCloudWatch/latest/logs/Working-with-log-groups-and-streams.html" (Working with Log Groups and Log Streams) in the <i>Amazon CloudWatch UserGuide</i>.<br/>DestinationLocationArn:  The Amazon Resource Name (ARN) of the AWS storage resource's location.<br/>SourceLocationArn:  The Amazon Resource Name (ARN) of the source file system's location.<br/>CurrentTaskExecutionArn:  The Amazon Resource Name (ARN) of the task execution that is syncing files.<br/>Name:  The name of the task that was described.<br/>Status:  The status of the task that was described. For detailed information about task execution statuses, see "https://docs.aws.amazon.com/datasync/latest/userguide/working-with-tasks.html#understand-task-creation-statuses" (Understanding Task Statuses).<br/>TaskArn:  The Amazon Resource Name (ARN) of the task that was described.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BytesPerSecond</i> A value that limits the bandwidth used by AWS DataSync. For example, if you want AWS DataSync to use a maximum of 1 MB, set this value to <code>1048576</code> (<code>=1024*1024</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PosixPermissions</i> A value that determines which users or groups can access a file for a specific purpose such as reading, writing, or execution of the file.  Default value: PRESERVE. PRESERVE: Preserve POSIX-style permissions (recommended). NONE: Ignore permissions.  <note> AWS DataSync can preserve extant permissions of a source location. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreserveDevices</i> A value that determines whether AWS DataSync should preserve the metadata of block and character devices in the source file system, and recreate the files with that device name and metadata on the destination. <note> AWS DataSync can't sync the actual contents of such devices, because they are nonterminal and don't return an end-of-file (EOF) marker. </note> Default value: NONE. NONE: Ignore special devices (recommended).  PRESERVE: Preserve character and block device metadata. This option isn't currently supported for Amazon EFS. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreserveDeletedFiles</i> A value that specifies whether files in the destination that don't exist in the source file system should be preserved.  Default value: PRESERVE. PRESERVE: Ignore such destination files (recommended).  REMOVE: Delete destination files that aren’t present in the source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Gid</i> The group ID (GID) of the file's owners.  Default value: INT_VALUE. This preserves the integer value of the ID. INT_VALUE: Preserve the integer value of user ID (UID) and GID (recommended). NONE: Ignore UID and GID. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Uid</i> The user ID (UID) of the file's owner.  Default value: INT_VALUE. This preserves the integer value of the ID. INT_VALUE: Preserve the integer value of UID and group ID (GID) (recommended). NONE: Ignore UID and GID. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Mtime</i> A value that indicates the last time that a file was modified (that is, a file was written to) before the PREPARING phase.  Default value: PRESERVE.  PRESERVE: Preserve original <code>Mtime</code> (recommended)  NONE: Ignore <code>Mtime</code>.  <note> If <code>Mtime</code> is set to PRESERVE, <code>Atime</code> must be set to BEST_EFFORT. If <code>Mtime</code> is set to NONE, <code>Atime</code> must also be set to NONE.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Atime</i> A file metadata value that shows the last time a file was accessed (that is, when the file was read or written to). If you set <code>Atime</code> to BEST_EFFORT, DataSync attempts to preserve the original <code>Atime</code> attribute on all source files (that is, the version before the PREPARING phase). However, <code>Atime</code>'s behavior is not fully standard across platforms, so AWS DataSync can only do this on a best-effort basis.  Default value: BEST_EFFORT. BEST_EFFORT: Attempt to preserve the per-file <code>Atime</code> value (recommended). NONE: Ignore <code>Atime</code>. <note> If <code>Atime</code> is set to BEST_EFFORT, <code>Mtime</code> must be set to PRESERVE.  If <code>Atime</code> is set to NONE, <code>Mtime</code> must also be NONE.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VerifyMode</i> A value that determines whether a data integrity verification should be performed at the end of a task execution after all data and metadata have been transferred.  Default value: POINT_IN_TIME_CONSISTENT. POINT_IN_TIME_CONSISTENT: Perform verification (recommended).  NONE: Skip verification.<br/>        """
        pass

    def describe_task_execution(self, TaskExecutionArn: str) -> DescribeTaskExecutionResponse:
        """Returns detailed metadata about a task that is being executed.

        :param self:
        :param TaskExecutionArn:  The Amazon Resource Name (ARN) of the task that is being executed.<br/>
        :return: Result:  The result of the task execution.:<br/>BytesTransferred:  The physical number of bytes transferred over the network.<br/>BytesWritten:  The number of logical bytes written to the destination AWS storage resource.<br/>FilesTransferred:  The actual number of files that was transferred over the network. This value is calculated and updated on an ongoing basis during the TRANSFERRING phase. It's updated periodically when each file is read from the source and sent over the network.  If failures occur during a transfer, this value can be less than <code>EstimatedFilesToTransfer</code>. This value can also be greater than <code>EstimatedFilesTransferred</code> in some cases. This element is implementation-specific for some location types, so don't use it as an indicator for a correct file number or to monitor your task execution.<br/>EstimatedBytesToTransfer:  The estimated physical number of bytes that is to be transferred over the network.<br/>EstimatedFilesToTransfer:  The expected number of files that is to be transferred over the network. This value is calculated during the PREPARING phase, before the TRANSFERRING phase. This value is the expected number of files to be transferred. It's calculated based on comparing the content of the source and destination locations and finding the delta that needs to be transferred. <br/>StartTime:  The time that the task execution was started.<br/>Includes:  <p/> Specifies that the task execution excludes files in the transfer based on the specified pattern in the filter. When multiple include filters are set, they are interpreted as an OR. <br/>Excludes:  <p/> Specifies that the task execution excludes files from the transfer based on the specified pattern in the filter. Transfers all files in the task’s subdirectory, except files that match the filter that is set. <br/>Options:  Represents the options that are available to control the behavior of a <a>StartTaskExecution</a> operation. Behavior includes preserving metadata such as user ID (UID), group ID (GID), and file permissions, and also overwriting files in the destination, data integrity verification, and so on. A task has a set of default options associated with it. If you don't specify an option in <a>StartTaskExecution</a>, the default value is used. You can override the defaults options on each task execution by specifying an overriding <code>Options</code> value to <a>StartTaskExecution</a>.:<br/>Status:  The status of the task execution.  For detailed information about task execution statuses, see "https://docs.aws.amazon.com/datasync/latest/userguide/working-with-tasks.html#understand-task-creation-statuses" (Understanding Task Statuses).<br/>TaskExecutionArn:  The Amazon Resource Name (ARN) of the task execution that was described. <code>TaskExecutionArn</code> is hierarchical and includes <code>TaskArn</code> for the task that was executed.  For example, a <code>TaskExecution</code> value with the ARN <code>arn:aws:datasync:us-east-1:111222333444:task/task-0208075f79cedf4a2/execution/exec-08ef1e88ec491019b</code> executed the task with the ARN <code>arn:aws:datasync:us-east-1:111222333444:task/task-0208075f79cedf4a2</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BytesPerSecond</i> A value that limits the bandwidth used by AWS DataSync. For example, if you want AWS DataSync to use a maximum of 1 MB, set this value to <code>1048576</code> (<code>=1024*1024</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PosixPermissions</i> A value that determines which users or groups can access a file for a specific purpose such as reading, writing, or execution of the file.  Default value: PRESERVE. PRESERVE: Preserve POSIX-style permissions (recommended). NONE: Ignore permissions.  <note> AWS DataSync can preserve extant permissions of a source location. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreserveDevices</i> A value that determines whether AWS DataSync should preserve the metadata of block and character devices in the source file system, and recreate the files with that device name and metadata on the destination. <note> AWS DataSync can't sync the actual contents of such devices, because they are nonterminal and don't return an end-of-file (EOF) marker. </note> Default value: NONE. NONE: Ignore special devices (recommended).  PRESERVE: Preserve character and block device metadata. This option isn't currently supported for Amazon EFS. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreserveDeletedFiles</i> A value that specifies whether files in the destination that don't exist in the source file system should be preserved.  Default value: PRESERVE. PRESERVE: Ignore such destination files (recommended).  REMOVE: Delete destination files that aren’t present in the source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Gid</i> The group ID (GID) of the file's owners.  Default value: INT_VALUE. This preserves the integer value of the ID. INT_VALUE: Preserve the integer value of user ID (UID) and GID (recommended). NONE: Ignore UID and GID. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Uid</i> The user ID (UID) of the file's owner.  Default value: INT_VALUE. This preserves the integer value of the ID. INT_VALUE: Preserve the integer value of UID and group ID (GID) (recommended). NONE: Ignore UID and GID. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Mtime</i> A value that indicates the last time that a file was modified (that is, a file was written to) before the PREPARING phase.  Default value: PRESERVE.  PRESERVE: Preserve original <code>Mtime</code> (recommended)  NONE: Ignore <code>Mtime</code>.  <note> If <code>Mtime</code> is set to PRESERVE, <code>Atime</code> must be set to BEST_EFFORT. If <code>Mtime</code> is set to NONE, <code>Atime</code> must also be set to NONE.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Atime</i> A file metadata value that shows the last time a file was accessed (that is, when the file was read or written to). If you set <code>Atime</code> to BEST_EFFORT, DataSync attempts to preserve the original <code>Atime</code> attribute on all source files (that is, the version before the PREPARING phase). However, <code>Atime</code>'s behavior is not fully standard across platforms, so AWS DataSync can only do this on a best-effort basis.  Default value: BEST_EFFORT. BEST_EFFORT: Attempt to preserve the per-file <code>Atime</code> value (recommended). NONE: Ignore <code>Atime</code>. <note> If <code>Atime</code> is set to BEST_EFFORT, <code>Mtime</code> must be set to PRESERVE.  If <code>Atime</code> is set to NONE, <code>Mtime</code> must also be NONE.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VerifyMode</i> A value that determines whether a data integrity verification should be performed at the end of a task execution after all data and metadata have been transferred.  Default value: POINT_IN_TIME_CONSISTENT. POINT_IN_TIME_CONSISTENT: Perform verification (recommended).  NONE: Skip verification.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorDetail</i> Detailed description of an error that was encountered during the task execution. You can use this information to help troubleshoot issues. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorCode</i> Errors that AWS DataSync encountered during execution of the task. You can use this error code to help troubleshoot issues.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VerifyStatus</i> The status of the VERIFYING Phase.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VerifyDuration</i> The total time in milliseconds that AWS DataSync spent in the VERIFYING phase.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransferStatus</i> The status of the TRANSFERRING Phase.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransferDuration</i> The total time in milliseconds that AWS DataSync spent in the TRANSFERRING phase.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrepareStatus</i> The status of the PREPARING phase.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrepareDuration</i> The total time in milliseconds that AWS DataSync spent in the PREPARING phase. <br/>        """
        pass

    def list_agents(self, MaxResults: int=None, NextToken: str=None) -> ListAgentsResponse:
        """Returns a list of agents owned by an AWS account in the AWS Region specified in the request. The returned list is ordered by agent Amazon Resource Name (ARN). By default, this operation returns a maximum of 100 agents. This operation supports pagination that enables you to optionally reduce the number of agents returned in a response. If you have more agents than are returned in a response (that is, the response returns only a truncated list of your agents), the response contains a marker that you can specify in your next request to fetch the next page of agents.

        :param self:
        :param NextToken:  An opaque string that indicates the position at which to begin the next list of agents.<br/>MaxResults:  The maximum number of agents to list.<br/>
        :return: NextToken:  An opaque string that indicates the position at which to begin returning the next list of agents.<br/>Agents:  A list of agents in your account.<br/>        """
        pass

    def list_locations(self, MaxResults: int=None, NextToken: str=None) -> ListLocationsResponse:
        """Returns a lists of source and destination locations. If you have more locations than are returned in a response (that is, the response returns only a truncated list of your agents), the response contains a token that you can specify in your next request to fetch the next page of locations.

        :param self:
        :param NextToken:  An opaque string that indicates the position at which to begin the next list of locations.<br/>MaxResults:  The maximum number of locations to return.<br/>
        :return: NextToken:  An opaque string that indicates the position at which to begin returning the next list of locations.<br/>Locations:  An array that contains a list of locations.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceArn: str, MaxResults: int=None, NextToken: str=None) -> ListTagsForResourceResponse:
        """Returns all the tags associated with a specified resources. 

        :param self:
        :param NextToken:  An opaque string that indicates the position at which to begin the next list of locations.<br/>MaxResults:  The maximum number of locations to return.<br/>ResourceArn:  The Amazon Resource Name (ARN) of the resource whose tags to list.<br/>
        :return: NextToken:  An opaque string that indicates the position at which to begin returning the next list of resource tags.<br/>Tags:  Array of resource tags.<br/>        """
        pass

    def list_task_executions(self, TaskArn: str=None, MaxResults: int=None, NextToken: str=None) -> ListTaskExecutionsResponse:
        """Returns a list of executed tasks.

        :param self:
        :param NextToken:  An opaque string that indicates the position at which to begin the next list of the executed tasks.<br/>MaxResults:  The maximum number of executed tasks to list.<br/>TaskArn:  The Amazon Resource Name (ARN) of the task whose tasks you want to list.<br/>
        :return: NextToken:  An opaque string that indicates the position at which to begin returning the next list of executed tasks.<br/>TaskExecutions:  A list of executed tasks.<br/>        """
        pass

    def list_tasks(self, MaxResults: int=None, NextToken: str=None) -> ListTasksResponse:
        """Returns a list of all the tasks.

        :param self:
        :param NextToken:  An opaque string that indicates the position at which to begin the next list of tasks.<br/>MaxResults:  The maximum number of tasks to return.<br/>
        :return: NextToken:  An opaque string that indicates the position at which to begin returning the next list of tasks.<br/>Tasks:  A list of all the tasks that are returned.<br/>        """
        pass

    def start_task_execution(self, TaskArn: str, OverrideOptions: Options=None, Includes: List[FilterRule]=None) -> StartTaskExecutionResponse:
        """Starts a specific invocation of a task. A <code>TaskExecution</code> value represents an individual run of a task. Each task can have at most one <code>TaskExecution</code> at a time.  <code>TaskExecution</code> has the following transition phases: INITIALIZING | PREPARING | TRANSFERRING | VERIFYING | SUCCESS/FAILURE.  For detailed information, see <i>Task Execution</i> in "https://docs.aws.amazon.com/datasync/latest/userguide/how-datasync-works.html#terminology" (Components and Terminology) in the <i>AWS DataSync User Guide</i>.

        :param self:
        :param Includes:  <p/> A filter that determines which files to include in the transfer during a task execution based on the specified pattern in the filter. When multiple include filters are set, they are interpreted as an OR.<br/>OverrideOptions:  Represents the options that are available to control the behavior of a <a>StartTaskExecution</a> operation. Behavior includes preserving metadata such as user ID (UID), group ID (GID), and file permissions, and also overwriting files in the destination, data integrity verification, and so on. A task has a set of default options associated with it. If you don't specify an option in <a>StartTaskExecution</a>, the default value is used. You can override the defaults options on each task execution by specifying an overriding <code>Options</code> value to <a>StartTaskExecution</a>.:<br/>TaskArn:  The Amazon Resource Name (ARN) of the task to start.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BytesPerSecond</i> A value that limits the bandwidth used by AWS DataSync. For example, if you want AWS DataSync to use a maximum of 1 MB, set this value to <code>1048576</code> (<code>=1024*1024</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PosixPermissions</i> A value that determines which users or groups can access a file for a specific purpose such as reading, writing, or execution of the file.  Default value: PRESERVE. PRESERVE: Preserve POSIX-style permissions (recommended). NONE: Ignore permissions.  <note> AWS DataSync can preserve extant permissions of a source location. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreserveDevices</i> A value that determines whether AWS DataSync should preserve the metadata of block and character devices in the source file system, and recreate the files with that device name and metadata on the destination. <note> AWS DataSync can't sync the actual contents of such devices, because they are nonterminal and don't return an end-of-file (EOF) marker. </note> Default value: NONE. NONE: Ignore special devices (recommended).  PRESERVE: Preserve character and block device metadata. This option isn't currently supported for Amazon EFS. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreserveDeletedFiles</i> A value that specifies whether files in the destination that don't exist in the source file system should be preserved.  Default value: PRESERVE. PRESERVE: Ignore such destination files (recommended).  REMOVE: Delete destination files that aren’t present in the source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Gid</i> The group ID (GID) of the file's owners.  Default value: INT_VALUE. This preserves the integer value of the ID. INT_VALUE: Preserve the integer value of user ID (UID) and GID (recommended). NONE: Ignore UID and GID. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Uid</i> The user ID (UID) of the file's owner.  Default value: INT_VALUE. This preserves the integer value of the ID. INT_VALUE: Preserve the integer value of UID and group ID (GID) (recommended). NONE: Ignore UID and GID. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Mtime</i> A value that indicates the last time that a file was modified (that is, a file was written to) before the PREPARING phase.  Default value: PRESERVE.  PRESERVE: Preserve original <code>Mtime</code> (recommended)  NONE: Ignore <code>Mtime</code>.  <note> If <code>Mtime</code> is set to PRESERVE, <code>Atime</code> must be set to BEST_EFFORT. If <code>Mtime</code> is set to NONE, <code>Atime</code> must also be set to NONE.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Atime</i> A file metadata value that shows the last time a file was accessed (that is, when the file was read or written to). If you set <code>Atime</code> to BEST_EFFORT, DataSync attempts to preserve the original <code>Atime</code> attribute on all source files (that is, the version before the PREPARING phase). However, <code>Atime</code>'s behavior is not fully standard across platforms, so AWS DataSync can only do this on a best-effort basis.  Default value: BEST_EFFORT. BEST_EFFORT: Attempt to preserve the per-file <code>Atime</code> value (recommended). NONE: Ignore <code>Atime</code>. <note> If <code>Atime</code> is set to BEST_EFFORT, <code>Mtime</code> must be set to PRESERVE.  If <code>Atime</code> is set to NONE, <code>Mtime</code> must also be NONE.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VerifyMode</i> A value that determines whether a data integrity verification should be performed at the end of a task execution after all data and metadata have been transferred.  Default value: POINT_IN_TIME_CONSISTENT. POINT_IN_TIME_CONSISTENT: Perform verification (recommended).  NONE: Skip verification.<br/>
        :return: TaskExecutionArn:  The Amazon Resource Name (ARN) of the specific task execution that was started.<br/>        """
        pass

    def tag_resource(self, Tags: List[TagListEntry], ResourceArn: str) -> TagResourceResponse:
        """Applies a key-value pair to an AWS resource.

        :param self:
        :param Tags:  The tags to apply.<br/>ResourceArn:  The Amazon Resource Name (ARN) of the resource to apply the tag to.<br/>
        :return:         """
        pass

    def untag_resource(self, Keys: List[TagKey], ResourceArn: str) -> UntagResourceResponse:
        """Removes a tag from an AWS resource.

        :param self:
        :param Keys:  The keys in the key-value pair in the tag to remove.<br/>ResourceArn:  The Amazon Resource Name (ARN) of the resource to remove the tag from.<br/>
        :return:         """
        pass

    def update_agent(self, AgentArn: str, Name: str=None) -> UpdateAgentResponse:
        """Updates the name of an agent.

        :param self:
        :param Name:  The name that you want to use to configure the agent.<br/>AgentArn:  The Amazon Resource Name (ARN) of the agent to update.<br/>
        :return:         """
        pass

    def update_task(self, TaskArn: str, Options: Options=None, Excludes: List[FilterRule]=None, Name: str=None, CloudWatchLogGroupArn: str=None) -> UpdateTaskResponse:
        """Updates the metadata associated with a task.

        :param self:
        :param CloudWatchLogGroupArn:  The Amazon Resource Name (ARN) of the resource name of the CloudWatch LogGroup.<br/>Name:  The name of the task to update.<br/>Excludes:  <p/> A filter that determines which files to exclude from a task based on the specified pattern in the filter. Transfers all files in the task’s subdirectory, except files that match the filter that is set. <br/>Options:  Represents the options that are available to control the behavior of a <a>StartTaskExecution</a> operation. Behavior includes preserving metadata such as user ID (UID), group ID (GID), and file permissions, and also overwriting files in the destination, data integrity verification, and so on. A task has a set of default options associated with it. If you don't specify an option in <a>StartTaskExecution</a>, the default value is used. You can override the defaults options on each task execution by specifying an overriding <code>Options</code> value to <a>StartTaskExecution</a>.:<br/>TaskArn:  The Amazon Resource Name (ARN) of the resource name of the task to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BytesPerSecond</i> A value that limits the bandwidth used by AWS DataSync. For example, if you want AWS DataSync to use a maximum of 1 MB, set this value to <code>1048576</code> (<code>=1024*1024</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PosixPermissions</i> A value that determines which users or groups can access a file for a specific purpose such as reading, writing, or execution of the file.  Default value: PRESERVE. PRESERVE: Preserve POSIX-style permissions (recommended). NONE: Ignore permissions.  <note> AWS DataSync can preserve extant permissions of a source location. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreserveDevices</i> A value that determines whether AWS DataSync should preserve the metadata of block and character devices in the source file system, and recreate the files with that device name and metadata on the destination. <note> AWS DataSync can't sync the actual contents of such devices, because they are nonterminal and don't return an end-of-file (EOF) marker. </note> Default value: NONE. NONE: Ignore special devices (recommended).  PRESERVE: Preserve character and block device metadata. This option isn't currently supported for Amazon EFS. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreserveDeletedFiles</i> A value that specifies whether files in the destination that don't exist in the source file system should be preserved.  Default value: PRESERVE. PRESERVE: Ignore such destination files (recommended).  REMOVE: Delete destination files that aren’t present in the source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Gid</i> The group ID (GID) of the file's owners.  Default value: INT_VALUE. This preserves the integer value of the ID. INT_VALUE: Preserve the integer value of user ID (UID) and GID (recommended). NONE: Ignore UID and GID. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Uid</i> The user ID (UID) of the file's owner.  Default value: INT_VALUE. This preserves the integer value of the ID. INT_VALUE: Preserve the integer value of UID and group ID (GID) (recommended). NONE: Ignore UID and GID. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Mtime</i> A value that indicates the last time that a file was modified (that is, a file was written to) before the PREPARING phase.  Default value: PRESERVE.  PRESERVE: Preserve original <code>Mtime</code> (recommended)  NONE: Ignore <code>Mtime</code>.  <note> If <code>Mtime</code> is set to PRESERVE, <code>Atime</code> must be set to BEST_EFFORT. If <code>Mtime</code> is set to NONE, <code>Atime</code> must also be set to NONE.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Atime</i> A file metadata value that shows the last time a file was accessed (that is, when the file was read or written to). If you set <code>Atime</code> to BEST_EFFORT, DataSync attempts to preserve the original <code>Atime</code> attribute on all source files (that is, the version before the PREPARING phase). However, <code>Atime</code>'s behavior is not fully standard across platforms, so AWS DataSync can only do this on a best-effort basis.  Default value: BEST_EFFORT. BEST_EFFORT: Attempt to preserve the per-file <code>Atime</code> value (recommended). NONE: Ignore <code>Atime</code>. <note> If <code>Atime</code> is set to BEST_EFFORT, <code>Mtime</code> must be set to PRESERVE.  If <code>Atime</code> is set to NONE, <code>Mtime</code> must also be NONE.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VerifyMode</i> A value that determines whether a data integrity verification should be performed at the end of a task execution after all data and metadata have been transferred.  Default value: POINT_IN_TIME_CONSISTENT. POINT_IN_TIME_CONSISTENT: Perform verification (recommended).  NONE: Skip verification.<br/>
        :return:         """
        pass



class DAX(BaseClient):
    """DAX is a managed caching service engineered for Amazon DynamoDB. DAX dramatically speeds up database reads by caching frequently-accessed data from DynamoDB, so applications can access that data with sub-millisecond latency. You can create a DAX cluster easily, using the AWS Management Console. With a few simple modifications to your code, your application can begin taking advantage of the DAX cluster and realize significant improvements in read performance."""


    class DescribeClustersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_clusters` for valid parameters"""
            pass
    class DescribeDefaultParametersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_default_parameters` for valid parameters"""
            pass
    class DescribeEventsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_events` for valid parameters"""
            pass
    class DescribeParameterGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_parameter_groups` for valid parameters"""
            pass
    class DescribeParametersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_parameters` for valid parameters"""
            pass
    class DescribeSubnetGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_subnet_groups` for valid parameters"""
            pass
    class ListTagsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_tags` for valid parameters"""
            pass


    class CreateClusterRequest(Mapping):
        pass
    
    class CreateClusterResponse(Mapping):
        pass
    
    class String(object):
        pass
    
    class Tag(Mapping):
        pass
    
    class SSESpecification(Mapping):
        pass
    
    class CreateParameterGroupRequest(Mapping):
        pass
    
    class CreateParameterGroupResponse(Mapping):
        pass
    
    class CreateSubnetGroupRequest(Mapping):
        pass
    
    class CreateSubnetGroupResponse(Mapping):
        pass
    
    class DecreaseReplicationFactorRequest(Mapping):
        pass
    
    class DecreaseReplicationFactorResponse(Mapping):
        pass
    
    class DeleteClusterRequest(Mapping):
        pass
    
    class DeleteClusterResponse(Mapping):
        pass
    
    class DeleteParameterGroupRequest(Mapping):
        pass
    
    class DeleteParameterGroupResponse(Mapping):
        pass
    
    class DeleteSubnetGroupRequest(Mapping):
        pass
    
    class DeleteSubnetGroupResponse(Mapping):
        pass
    
    class DescribeClustersRequest(Mapping):
        pass
    
    class DescribeClustersResponse(Mapping):
        pass
    
    class DescribeDefaultParametersRequest(Mapping):
        pass
    
    class DescribeDefaultParametersResponse(Mapping):
        pass
    
    class DescribeEventsRequest(Mapping):
        pass
    
    class DescribeEventsResponse(Mapping):
        pass
    
    class TStamp(object):
        pass
    
    class DescribeParameterGroupsRequest(Mapping):
        pass
    
    class DescribeParameterGroupsResponse(Mapping):
        pass
    
    class DescribeParametersRequest(Mapping):
        pass
    
    class DescribeParametersResponse(Mapping):
        pass
    
    class DescribeSubnetGroupsRequest(Mapping):
        pass
    
    class DescribeSubnetGroupsResponse(Mapping):
        pass
    
    class IncreaseReplicationFactorRequest(Mapping):
        pass
    
    class IncreaseReplicationFactorResponse(Mapping):
        pass
    
    class ListTagsRequest(Mapping):
        pass
    
    class ListTagsResponse(Mapping):
        pass
    
    class RebootNodeRequest(Mapping):
        pass
    
    class RebootNodeResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class UpdateClusterRequest(Mapping):
        pass
    
    class UpdateClusterResponse(Mapping):
        pass
    
    class UpdateParameterGroupRequest(Mapping):
        pass
    
    class UpdateParameterGroupResponse(Mapping):
        pass
    
    class ParameterNameValue(Mapping):
        pass
    
    class UpdateSubnetGroupRequest(Mapping):
        pass
    
    class UpdateSubnetGroupResponse(Mapping):
        pass
    
    def create_cluster(self, IamRoleArn: str, ReplicationFactor: int, NodeType: str, ClusterName: str, Description: str=None, AvailabilityZones: List[String]=None, SubnetGroupName: str=None, SecurityGroupIds: List[String]=None, PreferredMaintenanceWindow: str=None, NotificationTopicArn: str=None, ParameterGroupName: str=None, Tags: List[Tag]=None, SSESpecification: SSESpecification=None) -> CreateClusterResponse:
        """Creates a DAX cluster. All nodes in the cluster run the same DAX caching software.

        :param self:
        :param SSESpecification:  Represents the settings used to enable server-side encryption on the cluster.:<br/>Tags:  A set of tags to associate with the DAX cluster. <br/>ParameterGroupName:  The parameter group to be associated with the DAX cluster.<br/>IamRoleArn:  A valid Amazon Resource Name (ARN) that identifies an IAM role. At runtime, DAX will assume this role and use the role's permissions to access DynamoDB on your behalf.<br/>NotificationTopicArn:  The Amazon Resource Name (ARN) of the Amazon SNS topic to which notifications will be sent. <note> The Amazon SNS topic owner must be same as the DAX cluster owner. </note><br/>PreferredMaintenanceWindow:  Specifies the weekly time range during which maintenance on the DAX cluster is performed. It is specified as a range in the format ddd:hh24:mi-ddd:hh24:mi (24H Clock UTC). The minimum maintenance window is a 60 minute period. Valid values for <code>ddd</code> are: <ul> <li>  <code>sun</code>  </li> <li>  <code>mon</code>  </li> <li>  <code>tue</code>  </li> <li>  <code>wed</code>  </li> <li>  <code>thu</code>  </li> <li>  <code>fri</code>  </li> <li>  <code>sat</code>  </li> </ul> Example: <code>sun:05:00-sun:09:00</code>  <note> If you don't specify a preferred maintenance window when you create or modify a cache cluster, DAX assigns a 60-minute maintenance window on a randomly selected day of the week. </note><br/>SecurityGroupIds:  A list of security group IDs to be assigned to each node in the DAX cluster. (Each of the security group ID is system-generated.) If this parameter is not specified, DAX assigns the default VPC security group to each node.<br/>SubnetGroupName:  The name of the subnet group to be used for the replication group. <important> DAX clusters can only run in an Amazon VPC environment. All of the subnets that you specify in a subnet group must exist in the same VPC. </important><br/>AvailabilityZones:  The Availability Zones (AZs) in which the cluster nodes will be created. All nodes belonging to the cluster are placed in these Availability Zones. Use this parameter if you want to distribute the nodes across multiple AZs.<br/>ReplicationFactor:  The number of nodes in the DAX cluster. A replication factor of 1 will create a single-node cluster, without any read replicas. For additional fault tolerance, you can create a multiple node cluster with one or more read replicas. To do this, set <i>ReplicationFactor</i> to 2 or more. <note> AWS recommends that you have at least two read replicas per cluster. </note><br/>Description:  A description of the cluster.<br/>NodeType:  The compute and memory capacity of the nodes in the cluster.<br/>ClusterName:  The cluster identifier. This parameter is stored as a lowercase string.  <b>Constraints:</b>  <ul> <li> A name must contain from 1 to 20 alphanumeric characters or hyphens. </li> <li> The first character must be a letter. </li> <li> A name cannot end with a hyphen or contain two consecutive hyphens. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Indicates whether server-side encryption is enabled (true) or disabled (false) on the cluster.<br/>
        :return: Cluster:  A description of the DAX cluster that you have created.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SSEDescription</i> The description of the server-side encryption status on the specified DAX cluster.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SSEDescription</b>: The description of the server-side encryption status on the specified DAX cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterGroup</i> The parameter group being used by nodes in the cluster.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ParameterGroup</b>: The parameter group being used by nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoleArn</i> A valid Amazon Resource Name (ARN) that identifies an IAM role. At runtime, DAX will assume this role and use the role's permissions to access DynamoDB on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroups</i> A list of security groups, and the status of each, for the nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroup</i> The subnet group where the DAX cluster is running.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationConfiguration</i> Describes a notification topic and its status. Notification topics are used for publishing DAX events to subscribers using Amazon Simple Notification Service (SNS).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NotificationConfiguration</b>: Describes a notification topic and its status. Notification topics are used for publishing DAX events to subscribers using Amazon Simple Notification Service (SNS).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> A range of time when maintenance of DAX cluster software will be performed. For example: <code>sun:01:00-sun:09:00</code>. Cluster maintenance normally takes less than 30 minutes, and is performed automatically within the maintenance window.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Nodes</i> A list of nodes that are currently in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeIdsToRemove</i> A list of nodes to be removed from the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterDiscoveryEndpoint</i> The configuration endpoint for this DAX cluster, consisting of a DNS name and a port number. Client applications can specify this endpoint, rather than an individual node endpoint, and allow the DAX client software to intelligently route requests and responses to nodes in the DAX cluster.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ClusterDiscoveryEndpoint</b>: The configuration endpoint for this DAX cluster, consisting of a DNS name and a port number. Client applications can specify this endpoint, rather than an individual node endpoint, and allow the DAX client software to intelligently route requests and responses to nodes in the DAX cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current status of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The node type for the nodes in the cluster. (All nodes in a DAX cluster are of the same type.)<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActiveNodes</i> The number of nodes in the cluster that are active (i.e., capable of serving requests).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalNodes</i> The total number of nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterArn</i> The Amazon Resource Name (ARN) that uniquely identifies the cluster. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterName</i> The name of the DAX cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port number that applications should use to connect to the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS hostname of the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicStatus</i> The current state of the topic.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicArn</i> The Amazon Resource Name (ARN) that identifies the topic. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeIdsToReboot</i> The node IDs of one or more nodes to be rebooted.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterApplyStatus</i> The status of parameter updates. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterGroupName</i> The name of the parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current state of server-side encryption: <ul> <li>  <code>ENABLING</code> - Server-side encryption is being enabled. </li> <li>  <code>ENABLED</code> - Server-side encryption is enabled. </li> <li>  <code>DISABLING</code> - Server-side encryption is being disabled. </li> <li>  <code>DISABLED</code> - Server-side encryption is disabled. </li> </ul><br/>        """
        pass

    def create_parameter_group(self, ParameterGroupName: str, Description: str=None) -> CreateParameterGroupResponse:
        """Creates a new parameter group. A parameter group is a collection of parameters that you apply to all of the nodes in a DAX cluster.

        :param self:
        :param Description:  A description of the parameter group.<br/>ParameterGroupName:  The name of the parameter group to apply to all of the clusters in this replication group.<br/>
        :return: ParameterGroup:  Represents the output of a <i>CreateParameterGroup</i> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of the parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterGroupName</i> The name of the parameter group.<br/>        """
        pass

    def create_subnet_group(self, SubnetIds: List[String], SubnetGroupName: str, Description: str=None) -> CreateSubnetGroupResponse:
        """Creates a new subnet group.

        :param self:
        :param SubnetIds:  A list of VPC subnet IDs for the subnet group.<br/>Description:  A description for the subnet group<br/>SubnetGroupName:  A name for the subnet group. This value is stored as a lowercase string. <br/>
        :return: SubnetGroup:  Represents the output of a <i>CreateSubnetGroup</i> operation.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> A list of subnets associated with the subnet group. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The Amazon Virtual Private Cloud identifier (VPC ID) of the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupName</i> The name of the subnet group.<br/>        """
        pass

    def decrease_replication_factor(self, NewReplicationFactor: int, ClusterName: str, AvailabilityZones: List[String]=None, NodeIdsToRemove: List[String]=None) -> DecreaseReplicationFactorResponse:
        """Removes one or more nodes from a DAX cluster. <note> You cannot use <code>DecreaseReplicationFactor</code> to remove the last node in a DAX cluster. If you need to do this, use <code>DeleteCluster</code> instead. </note>

        :param self:
        :param NodeIdsToRemove:  The unique identifiers of the nodes to be removed from the cluster.<br/>AvailabilityZones:  The Availability Zone(s) from which to remove nodes.<br/>NewReplicationFactor:  The new number of nodes for the DAX cluster.<br/>ClusterName:  The name of the DAX cluster from which you want to remove nodes.<br/>
        :return: Cluster:  A description of the DAX cluster, after you have decreased its replication factor.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SSEDescription</i> The description of the server-side encryption status on the specified DAX cluster.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SSEDescription</b>: The description of the server-side encryption status on the specified DAX cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterGroup</i> The parameter group being used by nodes in the cluster.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ParameterGroup</b>: The parameter group being used by nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoleArn</i> A valid Amazon Resource Name (ARN) that identifies an IAM role. At runtime, DAX will assume this role and use the role's permissions to access DynamoDB on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroups</i> A list of security groups, and the status of each, for the nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroup</i> The subnet group where the DAX cluster is running.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationConfiguration</i> Describes a notification topic and its status. Notification topics are used for publishing DAX events to subscribers using Amazon Simple Notification Service (SNS).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NotificationConfiguration</b>: Describes a notification topic and its status. Notification topics are used for publishing DAX events to subscribers using Amazon Simple Notification Service (SNS).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> A range of time when maintenance of DAX cluster software will be performed. For example: <code>sun:01:00-sun:09:00</code>. Cluster maintenance normally takes less than 30 minutes, and is performed automatically within the maintenance window.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Nodes</i> A list of nodes that are currently in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeIdsToRemove</i> A list of nodes to be removed from the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterDiscoveryEndpoint</i> The configuration endpoint for this DAX cluster, consisting of a DNS name and a port number. Client applications can specify this endpoint, rather than an individual node endpoint, and allow the DAX client software to intelligently route requests and responses to nodes in the DAX cluster.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ClusterDiscoveryEndpoint</b>: The configuration endpoint for this DAX cluster, consisting of a DNS name and a port number. Client applications can specify this endpoint, rather than an individual node endpoint, and allow the DAX client software to intelligently route requests and responses to nodes in the DAX cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current status of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The node type for the nodes in the cluster. (All nodes in a DAX cluster are of the same type.)<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActiveNodes</i> The number of nodes in the cluster that are active (i.e., capable of serving requests).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalNodes</i> The total number of nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterArn</i> The Amazon Resource Name (ARN) that uniquely identifies the cluster. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterName</i> The name of the DAX cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port number that applications should use to connect to the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS hostname of the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicStatus</i> The current state of the topic.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicArn</i> The Amazon Resource Name (ARN) that identifies the topic. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeIdsToReboot</i> The node IDs of one or more nodes to be rebooted.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterApplyStatus</i> The status of parameter updates. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterGroupName</i> The name of the parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current state of server-side encryption: <ul> <li>  <code>ENABLING</code> - Server-side encryption is being enabled. </li> <li>  <code>ENABLED</code> - Server-side encryption is enabled. </li> <li>  <code>DISABLING</code> - Server-side encryption is being disabled. </li> <li>  <code>DISABLED</code> - Server-side encryption is disabled. </li> </ul><br/>        """
        pass

    def delete_cluster(self, ClusterName: str) -> DeleteClusterResponse:
        """Deletes a previously provisioned DAX cluster. <i>DeleteCluster</i> deletes all associated nodes, node endpoints and the DAX cluster itself. When you receive a successful response from this action, DAX immediately begins deleting the cluster; you cannot cancel or revert this action.

        :param self:
        :param ClusterName:  The name of the cluster to be deleted.<br/>
        :return: Cluster:  A description of the DAX cluster that is being deleted.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SSEDescription</i> The description of the server-side encryption status on the specified DAX cluster.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SSEDescription</b>: The description of the server-side encryption status on the specified DAX cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterGroup</i> The parameter group being used by nodes in the cluster.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ParameterGroup</b>: The parameter group being used by nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoleArn</i> A valid Amazon Resource Name (ARN) that identifies an IAM role. At runtime, DAX will assume this role and use the role's permissions to access DynamoDB on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroups</i> A list of security groups, and the status of each, for the nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroup</i> The subnet group where the DAX cluster is running.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationConfiguration</i> Describes a notification topic and its status. Notification topics are used for publishing DAX events to subscribers using Amazon Simple Notification Service (SNS).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NotificationConfiguration</b>: Describes a notification topic and its status. Notification topics are used for publishing DAX events to subscribers using Amazon Simple Notification Service (SNS).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> A range of time when maintenance of DAX cluster software will be performed. For example: <code>sun:01:00-sun:09:00</code>. Cluster maintenance normally takes less than 30 minutes, and is performed automatically within the maintenance window.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Nodes</i> A list of nodes that are currently in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeIdsToRemove</i> A list of nodes to be removed from the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterDiscoveryEndpoint</i> The configuration endpoint for this DAX cluster, consisting of a DNS name and a port number. Client applications can specify this endpoint, rather than an individual node endpoint, and allow the DAX client software to intelligently route requests and responses to nodes in the DAX cluster.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ClusterDiscoveryEndpoint</b>: The configuration endpoint for this DAX cluster, consisting of a DNS name and a port number. Client applications can specify this endpoint, rather than an individual node endpoint, and allow the DAX client software to intelligently route requests and responses to nodes in the DAX cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current status of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The node type for the nodes in the cluster. (All nodes in a DAX cluster are of the same type.)<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActiveNodes</i> The number of nodes in the cluster that are active (i.e., capable of serving requests).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalNodes</i> The total number of nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterArn</i> The Amazon Resource Name (ARN) that uniquely identifies the cluster. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterName</i> The name of the DAX cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port number that applications should use to connect to the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS hostname of the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicStatus</i> The current state of the topic.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicArn</i> The Amazon Resource Name (ARN) that identifies the topic. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeIdsToReboot</i> The node IDs of one or more nodes to be rebooted.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterApplyStatus</i> The status of parameter updates. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterGroupName</i> The name of the parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current state of server-side encryption: <ul> <li>  <code>ENABLING</code> - Server-side encryption is being enabled. </li> <li>  <code>ENABLED</code> - Server-side encryption is enabled. </li> <li>  <code>DISABLING</code> - Server-side encryption is being disabled. </li> <li>  <code>DISABLED</code> - Server-side encryption is disabled. </li> </ul><br/>        """
        pass

    def delete_parameter_group(self, ParameterGroupName: str) -> DeleteParameterGroupResponse:
        """Deletes the specified parameter group. You cannot delete a parameter group if it is associated with any DAX clusters.

        :param self:
        :param ParameterGroupName:  The name of the parameter group to delete.<br/>
        :return: DeletionMessage:  A user-specified message for this action (i.e., a reason for deleting the parameter group).<br/>        """
        pass

    def delete_subnet_group(self, SubnetGroupName: str) -> DeleteSubnetGroupResponse:
        """Deletes a subnet group. <note> You cannot delete a subnet group if it is associated with any DAX clusters. </note>

        :param self:
        :param SubnetGroupName:  The name of the subnet group to delete.<br/>
        :return: DeletionMessage:  A user-specified message for this action (i.e., a reason for deleting the subnet group).<br/>        """
        pass

    def describe_clusters(self, ClusterNames: List[String]=None, MaxResults: int=None, NextToken: str=None) -> DescribeClustersResponse:
        """Returns information about all provisioned DAX clusters if no cluster identifier is specified, or about a specific DAX cluster if a cluster identifier is supplied. If the cluster is in the CREATING state, only cluster level information will be displayed until all of the nodes are successfully provisioned. If the cluster is in the DELETING state, only cluster level information will be displayed. If nodes are currently being added to the DAX cluster, node endpoint information and creation time for the additional nodes will not be displayed until they are completely provisioned. When the DAX cluster state is <i>available</i>, the cluster is ready for use. If nodes are currently being removed from the DAX cluster, no endpoint information for the removed nodes is displayed.

        :param self:
        :param NextToken:  An optional token returned from a prior request. Use this token for pagination of results from this action. If this parameter is specified, the response includes only results beyond the token, up to the value specified by <code>MaxResults</code>.<br/>MaxResults:  The maximum number of results to include in the response. If more results exist than the specified <code>MaxResults</code> value, a token is included in the response so that the remaining results can be retrieved. The value for <code>MaxResults</code> must be between 20 and 100.<br/>ClusterNames:  The names of the DAX clusters being described.<br/>
        :return: Clusters:  The descriptions of your DAX clusters, in response to a <i>DescribeClusters</i> request.<br/>NextToken:  Provides an identifier to allow retrieval of paginated results.<br/>        """
        pass

    def describe_default_parameters(self, MaxResults: int=None, NextToken: str=None) -> DescribeDefaultParametersResponse:
        """Returns the default system parameter information for the DAX caching software.

        :param self:
        :param NextToken:  An optional token returned from a prior request. Use this token for pagination of results from this action. If this parameter is specified, the response includes only results beyond the token, up to the value specified by <code>MaxResults</code>.<br/>MaxResults:  The maximum number of results to include in the response. If more results exist than the specified <code>MaxResults</code> value, a token is included in the response so that the remaining results can be retrieved. The value for <code>MaxResults</code> must be between 20 and 100.<br/>
        :return: Parameters:  A list of parameters. Each element in the list represents one parameter.<br/>NextToken:  Provides an identifier to allow retrieval of paginated results.<br/>        """
        pass

    def describe_events(self, SourceName: str=None, SourceType: str=None, StartTime: TStamp=None, EndTime: TStamp=None, Duration: int=None, MaxResults: int=None, NextToken: str=None) -> DescribeEventsResponse:
        """Returns events related to DAX clusters and parameter groups. You can obtain events specific to a particular DAX cluster or parameter group by providing the name as a parameter. By default, only the events occurring within the last hour are returned; however, you can retrieve up to 14 days' worth of events if necessary.

        :param self:
        :param NextToken:  An optional token returned from a prior request. Use this token for pagination of results from this action. If this parameter is specified, the response includes only results beyond the token, up to the value specified by <code>MaxResults</code>.<br/>MaxResults:  The maximum number of results to include in the response. If more results exist than the specified <code>MaxResults</code> value, a token is included in the response so that the remaining results can be retrieved. The value for <code>MaxResults</code> must be between 20 and 100.<br/>Duration:  The number of minutes' worth of events to retrieve.<br/>EndTime:  The end of the time interval for which to retrieve events, specified in ISO 8601 format.<br/>StartTime:  The beginning of the time interval to retrieve events for, specified in ISO 8601 format.<br/>SourceType:  The event source to retrieve events for. If no value is specified, all events are returned.<br/>SourceName:  The identifier of the event source for which events will be returned. If not specified, then all sources are included in the response.<br/>
        :return: Events:  An array of events. Each element in the array represents one event.<br/>NextToken:  Provides an identifier to allow retrieval of paginated results.<br/>        """
        pass

    def describe_parameter_groups(self, ParameterGroupNames: List[String]=None, MaxResults: int=None, NextToken: str=None) -> DescribeParameterGroupsResponse:
        """Returns a list of parameter group descriptions. If a parameter group name is specified, the list will contain only the descriptions for that group.

        :param self:
        :param NextToken:  An optional token returned from a prior request. Use this token for pagination of results from this action. If this parameter is specified, the response includes only results beyond the token, up to the value specified by <code>MaxResults</code>.<br/>MaxResults:  The maximum number of results to include in the response. If more results exist than the specified <code>MaxResults</code> value, a token is included in the response so that the remaining results can be retrieved. The value for <code>MaxResults</code> must be between 20 and 100.<br/>ParameterGroupNames:  The names of the parameter groups.<br/>
        :return: ParameterGroups:  An array of parameter groups. Each element in the array represents one parameter group.<br/>NextToken:  Provides an identifier to allow retrieval of paginated results.<br/>        """
        pass

    def describe_parameters(self, ParameterGroupName: str, Source: str=None, MaxResults: int=None, NextToken: str=None) -> DescribeParametersResponse:
        """Returns the detailed parameter list for a particular parameter group.

        :param self:
        :param NextToken:  An optional token returned from a prior request. Use this token for pagination of results from this action. If this parameter is specified, the response includes only results beyond the token, up to the value specified by <code>MaxResults</code>.<br/>MaxResults:  The maximum number of results to include in the response. If more results exist than the specified <code>MaxResults</code> value, a token is included in the response so that the remaining results can be retrieved. The value for <code>MaxResults</code> must be between 20 and 100.<br/>Source:  How the parameter is defined. For example, <code>system</code> denotes a system-defined parameter.<br/>ParameterGroupName:  The name of the parameter group.<br/>
        :return: Parameters:  A list of parameters within a parameter group. Each element in the list represents one parameter.<br/>NextToken:  Provides an identifier to allow retrieval of paginated results.<br/>        """
        pass

    def describe_subnet_groups(self, SubnetGroupNames: List[String]=None, MaxResults: int=None, NextToken: str=None) -> DescribeSubnetGroupsResponse:
        """Returns a list of subnet group descriptions. If a subnet group name is specified, the list will contain only the description of that group.

        :param self:
        :param NextToken:  An optional token returned from a prior request. Use this token for pagination of results from this action. If this parameter is specified, the response includes only results beyond the token, up to the value specified by <code>MaxResults</code>.<br/>MaxResults:  The maximum number of results to include in the response. If more results exist than the specified <code>MaxResults</code> value, a token is included in the response so that the remaining results can be retrieved. The value for <code>MaxResults</code> must be between 20 and 100.<br/>SubnetGroupNames:  The name of the subnet group.<br/>
        :return: SubnetGroups:  An array of subnet groups. Each element in the array represents a single subnet group.<br/>NextToken:  Provides an identifier to allow retrieval of paginated results.<br/>        """
        pass

    def increase_replication_factor(self, NewReplicationFactor: int, ClusterName: str, AvailabilityZones: List[String]=None) -> IncreaseReplicationFactorResponse:
        """Adds one or more nodes to a DAX cluster.

        :param self:
        :param AvailabilityZones:  The Availability Zones (AZs) in which the cluster nodes will be created. All nodes belonging to the cluster are placed in these Availability Zones. Use this parameter if you want to distribute the nodes across multiple AZs.<br/>NewReplicationFactor:  The new number of nodes for the DAX cluster.<br/>ClusterName:  The name of the DAX cluster that will receive additional nodes.<br/>
        :return: Cluster:  A description of the DAX cluster. with its new replication factor.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SSEDescription</i> The description of the server-side encryption status on the specified DAX cluster.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SSEDescription</b>: The description of the server-side encryption status on the specified DAX cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterGroup</i> The parameter group being used by nodes in the cluster.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ParameterGroup</b>: The parameter group being used by nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoleArn</i> A valid Amazon Resource Name (ARN) that identifies an IAM role. At runtime, DAX will assume this role and use the role's permissions to access DynamoDB on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroups</i> A list of security groups, and the status of each, for the nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroup</i> The subnet group where the DAX cluster is running.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationConfiguration</i> Describes a notification topic and its status. Notification topics are used for publishing DAX events to subscribers using Amazon Simple Notification Service (SNS).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NotificationConfiguration</b>: Describes a notification topic and its status. Notification topics are used for publishing DAX events to subscribers using Amazon Simple Notification Service (SNS).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> A range of time when maintenance of DAX cluster software will be performed. For example: <code>sun:01:00-sun:09:00</code>. Cluster maintenance normally takes less than 30 minutes, and is performed automatically within the maintenance window.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Nodes</i> A list of nodes that are currently in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeIdsToRemove</i> A list of nodes to be removed from the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterDiscoveryEndpoint</i> The configuration endpoint for this DAX cluster, consisting of a DNS name and a port number. Client applications can specify this endpoint, rather than an individual node endpoint, and allow the DAX client software to intelligently route requests and responses to nodes in the DAX cluster.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ClusterDiscoveryEndpoint</b>: The configuration endpoint for this DAX cluster, consisting of a DNS name and a port number. Client applications can specify this endpoint, rather than an individual node endpoint, and allow the DAX client software to intelligently route requests and responses to nodes in the DAX cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current status of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The node type for the nodes in the cluster. (All nodes in a DAX cluster are of the same type.)<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActiveNodes</i> The number of nodes in the cluster that are active (i.e., capable of serving requests).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalNodes</i> The total number of nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterArn</i> The Amazon Resource Name (ARN) that uniquely identifies the cluster. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterName</i> The name of the DAX cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port number that applications should use to connect to the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS hostname of the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicStatus</i> The current state of the topic.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicArn</i> The Amazon Resource Name (ARN) that identifies the topic. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeIdsToReboot</i> The node IDs of one or more nodes to be rebooted.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterApplyStatus</i> The status of parameter updates. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterGroupName</i> The name of the parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current state of server-side encryption: <ul> <li>  <code>ENABLING</code> - Server-side encryption is being enabled. </li> <li>  <code>ENABLED</code> - Server-side encryption is enabled. </li> <li>  <code>DISABLING</code> - Server-side encryption is being disabled. </li> <li>  <code>DISABLED</code> - Server-side encryption is disabled. </li> </ul><br/>        """
        pass

    def list_tags(self, ResourceName: str, NextToken: str=None) -> ListTagsResponse:
        """List all of the tags for a DAX cluster. You can call <code>ListTags</code> up to 10 times per second, per account.

        :param self:
        :param NextToken:  An optional token returned from a prior request. Use this token for pagination of results from this action. If this parameter is specified, the response includes only results beyond the token.<br/>ResourceName:  The name of the DAX resource to which the tags belong.<br/>
        :return: NextToken:  If this value is present, there are additional results to be displayed. To retrieve them, call <code>ListTags</code> again, with <code>NextToken</code> set to this value.<br/>Tags:  A list of tags currently associated with the DAX cluster.<br/>        """
        pass

    def reboot_node(self, NodeId: str, ClusterName: str) -> RebootNodeResponse:
        """Reboots a single node of a DAX cluster. The reboot action takes place as soon as possible. During the reboot, the node status is set to REBOOTING.

        :param self:
        :param NodeId:  The system-assigned ID of the node to be rebooted.<br/>ClusterName:  The name of the DAX cluster containing the node to be rebooted.<br/>
        :return: Cluster:  A description of the DAX cluster after a node has been rebooted.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SSEDescription</i> The description of the server-side encryption status on the specified DAX cluster.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SSEDescription</b>: The description of the server-side encryption status on the specified DAX cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterGroup</i> The parameter group being used by nodes in the cluster.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ParameterGroup</b>: The parameter group being used by nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoleArn</i> A valid Amazon Resource Name (ARN) that identifies an IAM role. At runtime, DAX will assume this role and use the role's permissions to access DynamoDB on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroups</i> A list of security groups, and the status of each, for the nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroup</i> The subnet group where the DAX cluster is running.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationConfiguration</i> Describes a notification topic and its status. Notification topics are used for publishing DAX events to subscribers using Amazon Simple Notification Service (SNS).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NotificationConfiguration</b>: Describes a notification topic and its status. Notification topics are used for publishing DAX events to subscribers using Amazon Simple Notification Service (SNS).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> A range of time when maintenance of DAX cluster software will be performed. For example: <code>sun:01:00-sun:09:00</code>. Cluster maintenance normally takes less than 30 minutes, and is performed automatically within the maintenance window.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Nodes</i> A list of nodes that are currently in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeIdsToRemove</i> A list of nodes to be removed from the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterDiscoveryEndpoint</i> The configuration endpoint for this DAX cluster, consisting of a DNS name and a port number. Client applications can specify this endpoint, rather than an individual node endpoint, and allow the DAX client software to intelligently route requests and responses to nodes in the DAX cluster.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ClusterDiscoveryEndpoint</b>: The configuration endpoint for this DAX cluster, consisting of a DNS name and a port number. Client applications can specify this endpoint, rather than an individual node endpoint, and allow the DAX client software to intelligently route requests and responses to nodes in the DAX cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current status of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The node type for the nodes in the cluster. (All nodes in a DAX cluster are of the same type.)<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActiveNodes</i> The number of nodes in the cluster that are active (i.e., capable of serving requests).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalNodes</i> The total number of nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterArn</i> The Amazon Resource Name (ARN) that uniquely identifies the cluster. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterName</i> The name of the DAX cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port number that applications should use to connect to the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS hostname of the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicStatus</i> The current state of the topic.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicArn</i> The Amazon Resource Name (ARN) that identifies the topic. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeIdsToReboot</i> The node IDs of one or more nodes to be rebooted.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterApplyStatus</i> The status of parameter updates. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterGroupName</i> The name of the parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current state of server-side encryption: <ul> <li>  <code>ENABLING</code> - Server-side encryption is being enabled. </li> <li>  <code>ENABLED</code> - Server-side encryption is enabled. </li> <li>  <code>DISABLING</code> - Server-side encryption is being disabled. </li> <li>  <code>DISABLED</code> - Server-side encryption is disabled. </li> </ul><br/>        """
        pass

    def tag_resource(self, Tags: List[Tag], ResourceName: str) -> TagResourceResponse:
        """Associates a set of tags with a DAX resource. You can call <code>TagResource</code> up to 5 times per second, per account. 

        :param self:
        :param Tags:  The tags to be assigned to the DAX resource. <br/>ResourceName:  The name of the DAX resource to which tags should be added.<br/>
        :return: Tags:  The list of tags that are associated with the DAX resource.<br/>        """
        pass

    def untag_resource(self, TagKeys: List[String], ResourceName: str) -> UntagResourceResponse:
        """Removes the association of tags from a DAX resource. You can call <code>UntagResource</code> up to 5 times per second, per account. 

        :param self:
        :param TagKeys:  A list of tag keys. If the DAX cluster has any tags with these keys, then the tags are removed from the cluster.<br/>ResourceName:  The name of the DAX resource from which the tags should be removed.<br/>
        :return: Tags:  The tag keys that have been removed from the cluster.<br/>        """
        pass

    def update_cluster(self, ClusterName: str, Description: str=None, PreferredMaintenanceWindow: str=None, NotificationTopicArn: str=None, NotificationTopicStatus: str=None, ParameterGroupName: str=None, SecurityGroupIds: List[String]=None) -> UpdateClusterResponse:
        """Modifies the settings for a DAX cluster. You can use this action to change one or more cluster configuration parameters by specifying the parameters and the new values.

        :param self:
        :param SecurityGroupIds:  A list of user-specified security group IDs to be assigned to each node in the DAX cluster. If this parameter is not specified, DAX assigns the default VPC security group to each node.<br/>ParameterGroupName:  The name of a parameter group for this cluster.<br/>NotificationTopicStatus:  The current state of the topic.<br/>NotificationTopicArn:  The Amazon Resource Name (ARN) that identifies the topic.<br/>PreferredMaintenanceWindow:  A range of time when maintenance of DAX cluster software will be performed. For example: <code>sun:01:00-sun:09:00</code>. Cluster maintenance normally takes less than 30 minutes, and is performed automatically within the maintenance window.<br/>Description:  A description of the changes being made to the cluster.<br/>ClusterName:  The name of the DAX cluster to be modified.<br/>
        :return: Cluster:  A description of the DAX cluster, after it has been modified.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SSEDescription</i> The description of the server-side encryption status on the specified DAX cluster.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SSEDescription</b>: The description of the server-side encryption status on the specified DAX cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterGroup</i> The parameter group being used by nodes in the cluster.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ParameterGroup</b>: The parameter group being used by nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoleArn</i> A valid Amazon Resource Name (ARN) that identifies an IAM role. At runtime, DAX will assume this role and use the role's permissions to access DynamoDB on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroups</i> A list of security groups, and the status of each, for the nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroup</i> The subnet group where the DAX cluster is running.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationConfiguration</i> Describes a notification topic and its status. Notification topics are used for publishing DAX events to subscribers using Amazon Simple Notification Service (SNS).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NotificationConfiguration</b>: Describes a notification topic and its status. Notification topics are used for publishing DAX events to subscribers using Amazon Simple Notification Service (SNS).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> A range of time when maintenance of DAX cluster software will be performed. For example: <code>sun:01:00-sun:09:00</code>. Cluster maintenance normally takes less than 30 minutes, and is performed automatically within the maintenance window.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Nodes</i> A list of nodes that are currently in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeIdsToRemove</i> A list of nodes to be removed from the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterDiscoveryEndpoint</i> The configuration endpoint for this DAX cluster, consisting of a DNS name and a port number. Client applications can specify this endpoint, rather than an individual node endpoint, and allow the DAX client software to intelligently route requests and responses to nodes in the DAX cluster.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ClusterDiscoveryEndpoint</b>: The configuration endpoint for this DAX cluster, consisting of a DNS name and a port number. Client applications can specify this endpoint, rather than an individual node endpoint, and allow the DAX client software to intelligently route requests and responses to nodes in the DAX cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current status of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The node type for the nodes in the cluster. (All nodes in a DAX cluster are of the same type.)<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActiveNodes</i> The number of nodes in the cluster that are active (i.e., capable of serving requests).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalNodes</i> The total number of nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterArn</i> The Amazon Resource Name (ARN) that uniquely identifies the cluster. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterName</i> The name of the DAX cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port number that applications should use to connect to the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS hostname of the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicStatus</i> The current state of the topic.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicArn</i> The Amazon Resource Name (ARN) that identifies the topic. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeIdsToReboot</i> The node IDs of one or more nodes to be rebooted.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterApplyStatus</i> The status of parameter updates. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterGroupName</i> The name of the parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current state of server-side encryption: <ul> <li>  <code>ENABLING</code> - Server-side encryption is being enabled. </li> <li>  <code>ENABLED</code> - Server-side encryption is enabled. </li> <li>  <code>DISABLING</code> - Server-side encryption is being disabled. </li> <li>  <code>DISABLED</code> - Server-side encryption is disabled. </li> </ul><br/>        """
        pass

    def update_parameter_group(self, ParameterNameValues: List[ParameterNameValue], ParameterGroupName: str) -> UpdateParameterGroupResponse:
        """Modifies the parameters of a parameter group. You can modify up to 20 parameters in a single request by submitting a list parameter name and value pairs.

        :param self:
        :param ParameterNameValues:  An array of name-value pairs for the parameters in the group. Each element in the array represents a single parameter.<br/>ParameterGroupName:  The name of the parameter group.<br/>
        :return: ParameterGroup:  The parameter group that has been modified.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of the parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterGroupName</i> The name of the parameter group.<br/>        """
        pass

    def update_subnet_group(self, SubnetGroupName: str, Description: str=None, SubnetIds: List[String]=None) -> UpdateSubnetGroupResponse:
        """Modifies an existing subnet group.

        :param self:
        :param SubnetIds:  A list of subnet IDs in the subnet group.<br/>Description:  A description of the subnet group.<br/>SubnetGroupName:  The name of the subnet group.<br/>
        :return: SubnetGroup:  The subnet group that has been modified.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> A list of subnets associated with the subnet group. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The Amazon Virtual Private Cloud identifier (VPC ID) of the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupName</i> The name of the subnet group.<br/>        """
        pass



class DeviceFarm(BaseClient):
    """AWS Device Farm is a service that enables mobile app developers to test Android, iOS, and Fire OS apps on physical phones, tablets, and other devices in the cloud."""


    class ListArtifactsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_artifacts` for valid parameters"""
            pass
    class ListDevicePoolsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_device_pools` for valid parameters"""
            pass
    class ListDevicesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_devices` for valid parameters"""
            pass
    class ListJobsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_jobs` for valid parameters"""
            pass
    class ListProjectsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_projects` for valid parameters"""
            pass
    class ListRunsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_runs` for valid parameters"""
            pass
    class ListSamplesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_samples` for valid parameters"""
            pass
    class ListSuitesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_suites` for valid parameters"""
            pass
    class ListTestsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_tests` for valid parameters"""
            pass
    class ListUniqueProblemsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_unique_problems` for valid parameters"""
            pass
    class ListUploadsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_uploads` for valid parameters"""
            pass
    class GetOfferingStatusPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_offering_status` for valid parameters"""
            pass
    class ListOfferingTransactionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_offering_transactions` for valid parameters"""
            pass
    class ListOfferingsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_offerings` for valid parameters"""
            pass
    class ListDeviceInstancesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_device_instances` for valid parameters"""
            pass
    class ListInstanceProfilesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_instance_profiles` for valid parameters"""
            pass
    class ListNetworkProfilesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_network_profiles` for valid parameters"""
            pass
    class ListOfferingPromotionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_offering_promotions` for valid parameters"""
            pass
    class ListRemoteAccessSessionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_remote_access_sessions` for valid parameters"""
            pass
    class ListVPCEConfigurationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_vpce_configurations` for valid parameters"""
            pass


    class CreateDevicePoolRequest(Mapping):
        pass
    
    class CreateDevicePoolResult(Mapping):
        pass
    
    class Rule(Mapping):
        pass
    
    class CreateInstanceProfileRequest(Mapping):
        pass
    
    class CreateInstanceProfileResult(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class String(object):
        pass
    
    class CreateNetworkProfileRequest(Mapping):
        pass
    
    class CreateNetworkProfileResult(Mapping):
        pass
    
    class Long(object):
        pass
    
    class CreateProjectRequest(Mapping):
        pass
    
    class CreateProjectResult(Mapping):
        pass
    
    class CreateRemoteAccessSessionRequest(Mapping):
        pass
    
    class CreateRemoteAccessSessionResult(Mapping):
        pass
    
    class CreateRemoteAccessSessionConfiguration(Mapping):
        pass
    
    class CreateUploadRequest(Mapping):
        pass
    
    class CreateUploadResult(Mapping):
        pass
    
    class CreateVPCEConfigurationRequest(Mapping):
        pass
    
    class CreateVPCEConfigurationResult(Mapping):
        pass
    
    class DeleteDevicePoolRequest(Mapping):
        pass
    
    class DeleteDevicePoolResult(Mapping):
        pass
    
    class DeleteInstanceProfileRequest(Mapping):
        pass
    
    class DeleteInstanceProfileResult(Mapping):
        pass
    
    class DeleteNetworkProfileRequest(Mapping):
        pass
    
    class DeleteNetworkProfileResult(Mapping):
        pass
    
    class DeleteProjectRequest(Mapping):
        pass
    
    class DeleteProjectResult(Mapping):
        pass
    
    class DeleteRemoteAccessSessionRequest(Mapping):
        pass
    
    class DeleteRemoteAccessSessionResult(Mapping):
        pass
    
    class DeleteRunRequest(Mapping):
        pass
    
    class DeleteRunResult(Mapping):
        pass
    
    class DeleteUploadRequest(Mapping):
        pass
    
    class DeleteUploadResult(Mapping):
        pass
    
    class DeleteVPCEConfigurationRequest(Mapping):
        pass
    
    class DeleteVPCEConfigurationResult(Mapping):
        pass
    
    class GetAccountSettingsRequest(Mapping):
        pass
    
    class GetAccountSettingsResult(Mapping):
        pass
    
    class GetDeviceRequest(Mapping):
        pass
    
    class GetDeviceResult(Mapping):
        pass
    
    class GetDeviceInstanceRequest(Mapping):
        pass
    
    class GetDeviceInstanceResult(Mapping):
        pass
    
    class GetDevicePoolRequest(Mapping):
        pass
    
    class GetDevicePoolResult(Mapping):
        pass
    
    class GetDevicePoolCompatibilityRequest(Mapping):
        pass
    
    class GetDevicePoolCompatibilityResult(Mapping):
        pass
    
    class ScheduleRunTest(Mapping):
        pass
    
    class ScheduleRunConfiguration(Mapping):
        pass
    
    class GetInstanceProfileRequest(Mapping):
        pass
    
    class GetInstanceProfileResult(Mapping):
        pass
    
    class GetJobRequest(Mapping):
        pass
    
    class GetJobResult(Mapping):
        pass
    
    class GetNetworkProfileRequest(Mapping):
        pass
    
    class GetNetworkProfileResult(Mapping):
        pass
    
    class GetOfferingStatusRequest(Mapping):
        pass
    
    class GetOfferingStatusResult(Mapping):
        pass
    
    class GetProjectRequest(Mapping):
        pass
    
    class GetProjectResult(Mapping):
        pass
    
    class GetRemoteAccessSessionRequest(Mapping):
        pass
    
    class GetRemoteAccessSessionResult(Mapping):
        pass
    
    class GetRunRequest(Mapping):
        pass
    
    class GetRunResult(Mapping):
        pass
    
    class GetSuiteRequest(Mapping):
        pass
    
    class GetSuiteResult(Mapping):
        pass
    
    class GetTestRequest(Mapping):
        pass
    
    class GetTestResult(Mapping):
        pass
    
    class GetUploadRequest(Mapping):
        pass
    
    class GetUploadResult(Mapping):
        pass
    
    class GetVPCEConfigurationRequest(Mapping):
        pass
    
    class GetVPCEConfigurationResult(Mapping):
        pass
    
    class InstallToRemoteAccessSessionRequest(Mapping):
        pass
    
    class InstallToRemoteAccessSessionResult(Mapping):
        pass
    
    class ListArtifactsRequest(Mapping):
        pass
    
    class ListArtifactsResult(Mapping):
        pass
    
    class ListDeviceInstancesRequest(Mapping):
        pass
    
    class ListDeviceInstancesResult(Mapping):
        pass
    
    class ListDevicePoolsRequest(Mapping):
        pass
    
    class ListDevicePoolsResult(Mapping):
        pass
    
    class ListDevicesRequest(Mapping):
        pass
    
    class ListDevicesResult(Mapping):
        pass
    
    class DeviceFilter(Mapping):
        pass
    
    class ListInstanceProfilesRequest(Mapping):
        pass
    
    class ListInstanceProfilesResult(Mapping):
        pass
    
    class ListJobsRequest(Mapping):
        pass
    
    class ListJobsResult(Mapping):
        pass
    
    class ListNetworkProfilesRequest(Mapping):
        pass
    
    class ListNetworkProfilesResult(Mapping):
        pass
    
    class ListOfferingPromotionsRequest(Mapping):
        pass
    
    class ListOfferingPromotionsResult(Mapping):
        pass
    
    class ListOfferingTransactionsRequest(Mapping):
        pass
    
    class ListOfferingTransactionsResult(Mapping):
        pass
    
    class ListOfferingsRequest(Mapping):
        pass
    
    class ListOfferingsResult(Mapping):
        pass
    
    class ListProjectsRequest(Mapping):
        pass
    
    class ListProjectsResult(Mapping):
        pass
    
    class ListRemoteAccessSessionsRequest(Mapping):
        pass
    
    class ListRemoteAccessSessionsResult(Mapping):
        pass
    
    class ListRunsRequest(Mapping):
        pass
    
    class ListRunsResult(Mapping):
        pass
    
    class ListSamplesRequest(Mapping):
        pass
    
    class ListSamplesResult(Mapping):
        pass
    
    class ListSuitesRequest(Mapping):
        pass
    
    class ListSuitesResult(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class ListTestsRequest(Mapping):
        pass
    
    class ListTestsResult(Mapping):
        pass
    
    class ListUniqueProblemsRequest(Mapping):
        pass
    
    class ListUniqueProblemsResult(Mapping):
        pass
    
    class ListUploadsRequest(Mapping):
        pass
    
    class ListUploadsResult(Mapping):
        pass
    
    class ListVPCEConfigurationsRequest(Mapping):
        pass
    
    class ListVPCEConfigurationsResult(Mapping):
        pass
    
    class PurchaseOfferingRequest(Mapping):
        pass
    
    class PurchaseOfferingResult(Mapping):
        pass
    
    class RenewOfferingRequest(Mapping):
        pass
    
    class RenewOfferingResult(Mapping):
        pass
    
    class ScheduleRunRequest(Mapping):
        pass
    
    class ScheduleRunResult(Mapping):
        pass
    
    class DeviceSelectionConfiguration(Mapping):
        pass
    
    class ExecutionConfiguration(Mapping):
        pass
    
    class StopJobRequest(Mapping):
        pass
    
    class StopJobResult(Mapping):
        pass
    
    class StopRemoteAccessSessionRequest(Mapping):
        pass
    
    class StopRemoteAccessSessionResult(Mapping):
        pass
    
    class StopRunRequest(Mapping):
        pass
    
    class StopRunResult(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateDeviceInstanceRequest(Mapping):
        pass
    
    class UpdateDeviceInstanceResult(Mapping):
        pass
    
    class UpdateDevicePoolRequest(Mapping):
        pass
    
    class UpdateDevicePoolResult(Mapping):
        pass
    
    class UpdateInstanceProfileRequest(Mapping):
        pass
    
    class UpdateInstanceProfileResult(Mapping):
        pass
    
    class UpdateNetworkProfileRequest(Mapping):
        pass
    
    class UpdateNetworkProfileResult(Mapping):
        pass
    
    class UpdateProjectRequest(Mapping):
        pass
    
    class UpdateProjectResult(Mapping):
        pass
    
    class UpdateUploadRequest(Mapping):
        pass
    
    class UpdateUploadResult(Mapping):
        pass
    
    class UpdateVPCEConfigurationRequest(Mapping):
        pass
    
    class UpdateVPCEConfigurationResult(Mapping):
        pass
    
    def create_device_pool(self, rules: List[Rule], name: str, projectArn: str, description: str=None, maxDevices: int=None) -> CreateDevicePoolResult:
        """Creates a device pool.

        :param self:
        :param maxDevices:  The number of devices that Device Farm can add to your device pool. Device Farm adds devices that are available and that meet the criteria that you assign for the <code>rules</code> parameter. Depending on how many devices meet these constraints, your device pool might contain fewer devices than the value for this parameter. By specifying the maximum number of devices, you can control the costs that you incur by running tests.<br/>rules:  The device pool's rules.<br/>description:  The device pool's description.<br/>name:  The device pool's name.<br/>projectArn:  The ARN of the project for the device pool.<br/>
        :return: devicePool:  The newly created device pool.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>maxDevices</i> The number of devices that Device Farm can add to your device pool. Device Farm adds devices that are available and that meet the criteria that you assign for the <code>rules</code> parameter. Depending on how many devices meet these constraints, your device pool might contain fewer devices than the value for this parameter. By specifying the maximum number of devices, you can control the costs that you incur by running tests.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>rules</i> Information about the device pool's rules.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The device pool's type. Allowed values include: <ul> <li> CURATED: A device pool that is created and managed by AWS Device Farm. </li> <li> PRIVATE: A device pool that is created and managed by the device pool developer. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The device pool's description.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The device pool's name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The device pool's ARN.<br/>        """
        pass

    def create_instance_profile(self, name: str, description: str=None, packageCleanup: Boolean=None, excludeAppPackagesFromCleanup: List[String]=None, rebootAfterUse: Boolean=None) -> CreateInstanceProfileResult:
        """Creates a profile that can be applied to one or more private fleet device instances.

        :param self:
        :param rebootAfterUse:  When set to <code>true</code>, Device Farm will reboot the instance after a test run. The default value is <code>true</code>.<br/>excludeAppPackagesFromCleanup:  An array of strings specifying the list of app packages that should not be cleaned up from the device after a test run is over. The list of packages is only considered if you set <code>packageCleanup</code> to <code>true</code>.<br/>packageCleanup:  When set to <code>true</code>, Device Farm will remove app packages after a test run. The default value is <code>false</code> for private devices.<br/>description:  The description of your instance profile.<br/>name:  The name of your instance profile.<br/>
        :return: instanceProfile:  An object containing information about your instance profile.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The description of the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>rebootAfterUse</i> When set to <code>true</code>, Device Farm will reboot the instance after a test run. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>excludeAppPackagesFromCleanup</i> An array of strings specifying the list of app packages that should not be cleaned up from the device after a test run is over. The list of packages is only considered if you set <code>packageCleanup</code> to <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>packageCleanup</i> When set to <code>true</code>, Device Farm will remove app packages after a test run. The default value is <code>false</code> for private devices.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the instance profile.<br/>        """
        pass

    def create_network_profile(self, name: str, projectArn: str, description: str=None, type: str=None, uplinkBandwidthBits: Long=None, downlinkBandwidthBits: Long=None, uplinkDelayMs: Long=None, downlinkDelayMs: Long=None, uplinkJitterMs: Long=None, downlinkJitterMs: Long=None, uplinkLossPercent: int=None, downlinkLossPercent: int=None) -> CreateNetworkProfileResult:
        """Creates a network profile.

        :param self:
        :param downlinkLossPercent:  Proportion of received packets that fail to arrive from 0 to 100 percent.<br/>uplinkLossPercent:  Proportion of transmitted packets that fail to arrive from 0 to 100 percent.<br/>downlinkJitterMs:  Time variation in the delay of received packets in milliseconds as an integer from 0 to 2000.<br/>uplinkJitterMs:  Time variation in the delay of received packets in milliseconds as an integer from 0 to 2000.<br/>downlinkDelayMs:  Delay time for all packets to destination in milliseconds as an integer from 0 to 2000.<br/>uplinkDelayMs:  Delay time for all packets to destination in milliseconds as an integer from 0 to 2000.<br/>downlinkBandwidthBits:  The data throughput rate in bits per second, as an integer from 0 to 104857600.<br/>uplinkBandwidthBits:  The data throughput rate in bits per second, as an integer from 0 to 104857600.<br/>type:  The type of network profile you wish to create. Valid values are listed below.<br/>description:  The description of the network profile.<br/>name:  The name you wish to specify for the new network profile.<br/>projectArn:  The Amazon Resource Name (ARN) of the project for which you want to create a network profile.<br/>
        :return: networkProfile:  The network profile that is returned by the create network profile request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>downlinkLossPercent</i> Proportion of received packets that fail to arrive from 0 to 100 percent.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>uplinkLossPercent</i> Proportion of transmitted packets that fail to arrive from 0 to 100 percent.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>downlinkJitterMs</i> Time variation in the delay of received packets in milliseconds as an integer from 0 to 2000.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>uplinkJitterMs</i> Time variation in the delay of received packets in milliseconds as an integer from 0 to 2000.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>downlinkDelayMs</i> Delay time for all packets to destination in milliseconds as an integer from 0 to 2000.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>uplinkDelayMs</i> Delay time for all packets to destination in milliseconds as an integer from 0 to 2000.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>downlinkBandwidthBits</i> The data throughput rate in bits per second, as an integer from 0 to 104857600.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>uplinkBandwidthBits</i> The data throughput rate in bits per second, as an integer from 0 to 104857600.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of network profile. Valid values are listed below.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The description of the network profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the network profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the network profile.<br/>        """
        pass

    def create_project(self, name: str, defaultJobTimeoutMinutes: int=None) -> CreateProjectResult:
        """Creates a new project.

        :param self:
        :param defaultJobTimeoutMinutes:  Sets the execution timeout value (in minutes) for a project. All test runs in this project will use the specified execution timeout value unless overridden when scheduling a run.<br/>name:  The project's name.<br/>
        :return: project:  The newly created project.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>created</i> When the project was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultJobTimeoutMinutes</i> The default number of minutes (at the project level) a test run will execute before it times out. The default value is 150 minutes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The project's name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The project's ARN.<br/>        """
        pass

    def create_remote_access_session(self, deviceArn: str, projectArn: str, instanceArn: str=None, sshPublicKey: str=None, remoteDebugEnabled: Boolean=None, remoteRecordEnabled: Boolean=None, remoteRecordAppArn: str=None, name: str=None, clientId: str=None, configuration: CreateRemoteAccessSessionConfiguration=None, interactionMode: str=None, skipAppResign: Boolean=None) -> CreateRemoteAccessSessionResult:
        """Specifies and starts a remote access session.

        :param self:
        :param skipAppResign:  When set to <code>true</code>, for private devices, Device Farm will not sign your app again. For public devices, Device Farm always signs your apps again and this parameter has no effect. For more information about how Device Farm re-signs your app(s), see <a href="https://aws.amazon.com/device-farm/faq/">Do you modify my app?</a> in the <i>AWS Device Farm FAQs</i>.<br/>interactionMode:  The interaction mode of the remote access session. Valid values are: <ul> <li> INTERACTIVE: You can interact with the iOS device by viewing, touching, and rotating the screen. You <b>cannot</b> run XCUITest framework-based tests in this mode. </li> <li> NO_VIDEO: You are connected to the device but cannot interact with it or view the screen. This mode has the fastest test execution speed. You <b>can</b> run XCUITest framework-based tests in this mode. </li> <li> VIDEO_ONLY: You can view the screen but cannot touch or rotate it. You <b>can</b> run XCUITest framework-based tests and watch the screen in this mode. </li> </ul><br/>configuration:  The configuration information for the remote access session request.:<br/>clientId:  Unique identifier for the client. If you want access to multiple devices on the same client, you should pass the same <code>clientId</code> value in each call to <code>CreateRemoteAccessSession</code>. This is required only if <code>remoteDebugEnabled</code> is set to <code>true</code>.<br/>name:  The name of the remote access session that you wish to create.<br/>remoteRecordAppArn:  The Amazon Resource Name (ARN) for the app to be recorded in the remote access session.<br/>remoteRecordEnabled:  Set to <code>true</code> to enable remote recording for the remote access session.<br/>remoteDebugEnabled:  Set to <code>true</code> if you want to access devices remotely for debugging in your remote access session.<br/>sshPublicKey:  The public key of the <code>ssh</code> key pair you want to use for connecting to remote devices in your remote debugging session. This is only required if <code>remoteDebugEnabled</code> is set to <code>true</code>.<br/>instanceArn:  The Amazon Resource Name (ARN) of the device instance for which you want to create a remote access session.<br/>deviceArn:  The Amazon Resource Name (ARN) of the device for which you want to create a remote access session.<br/>projectArn:  The Amazon Resource Name (ARN) of the project for which you want to create a remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vpceConfigurationArns</i> An array of Amazon Resource Names (ARNs) included in the VPC endpoint configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>billingMethod</i> The billing method for the remote access session.<br/>
        :return: remoteAccessSession:  A container that describes the remote access session when the request to create a remote access session is sent.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>skipAppResign</i> When set to <code>true</code>, for private devices, Device Farm will not sign your app again. For public devices, Device Farm always signs your apps again and this parameter has no effect. For more information about how Device Farm re-signs your app(s), see <a href="https://aws.amazon.com/device-farm/faq/">Do you modify my app?</a> in the <i>AWS Device Farm FAQs</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>interactionMode</i> The interaction mode of the remote access session. Valid values are: <ul> <li> INTERACTIVE: You can interact with the iOS device by viewing, touching, and rotating the screen. You <b>cannot</b> run XCUITest framework-based tests in this mode. </li> <li> NO_VIDEO: You are connected to the device but cannot interact with it or view the screen. This mode has the fastest test execution speed. You <b>can</b> run XCUITest framework-based tests in this mode. </li> <li> VIDEO_ONLY: You can view the screen but cannot touch or rotate it. You <b>can</b> run XCUITest framework-based tests and watch the screen in this mode. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deviceUdid</i> Unique device identifier for the remote device. Only returned if remote debugging is enabled for the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>endpoint</i> The endpoint for the remote access sesssion.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deviceMinutes</i> The number of minutes a device is used in a remote access session (including setup and teardown minutes).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>deviceMinutes</b>: The number of minutes a device is used in a remote access session (including setup and teardown minutes).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>billingMethod</i> The billing method of the remote access session. Possible values include <code>METERED</code> or <code>UNMETERED</code>. For more information about metered devices, see <a href="https://docs.aws.amazon.com/devicefarm/latest/developerguide/welcome.html#welcome-terminology">AWS Device Farm terminology</a>."<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clientId</i> Unique identifier of your client for the remote access session. Only returned if remote debugging is enabled for the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>hostAddress</i> IP address of the EC2 host where you need to connect to remotely debug devices. Only returned if remote debugging is enabled for the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>remoteRecordAppArn</i> The Amazon Resource Name (ARN) for the app to be recorded in the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>remoteRecordEnabled</i> This flag is set to <code>true</code> if remote recording is enabled for the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>remoteDebugEnabled</i> This flag is set to <code>true</code> if remote debugging is enabled for the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>instanceArn</i> The Amazon Resource Name (ARN) of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>device</i> The device (phone or tablet) used in the remote access session.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>device</b>: The device (phone or tablet) used in the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stopped</i> The date and time the remote access session was stopped.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>started</i> The date and time the remote access session was started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>message</i> A message about the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>result</i> The result of the remote access session. Can be any of the following: <ul> <li> PENDING: A pending condition. </li> <li> PASSED: A passing condition. </li> <li> WARNED: A warning condition. </li> <li> FAILED: A failed condition. </li> <li> SKIPPED: A skipped condition. </li> <li> ERRORED: An error condition. </li> <li> STOPPED: A stopped condition. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the remote access session. Can be any of the following: <ul> <li> PENDING: A pending status. </li> <li> PENDING_CONCURRENCY: A pending concurrency status. </li> <li> PENDING_DEVICE: A pending device status. </li> <li> PROCESSING: A processing status. </li> <li> SCHEDULING: A scheduling status. </li> <li> PREPARING: A preparing status. </li> <li> RUNNING: A running status. </li> <li> COMPLETED: A completed status. </li> <li> STOPPING: A stopping status. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>created</i> The date and time the remote access session was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>os</i> The device's operating system type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>platform</i> The device's platform. Allowed values include: <ul> <li> ANDROID: The Android platform. </li> <li> IOS: The iOS platform. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>formFactor</i> The device's form factor. Allowed values include: <ul> <li> PHONE: The phone form factor. </li> <li> TABLET: The tablet form factor. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>modelId</i> The device's model ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>model</i> The device's model name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>manufacturer</i> The device's manufacturer name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The device's display name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The device's ARN.<br/>Information about the device's CPU.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>cpu</b>: Information about the device's CPU.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>unmetered</i> When specified, represents only the sum of unmetered minutes used by the resource to run tests.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>metered</i> When specified, represents only the sum of metered minutes used by the resource to run tests.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>total</i> When specified, represents the total minutes used by the resource to run tests.<br/>        """
        pass

    def create_upload(self, type: str, name: str, projectArn: str, contentType: str=None) -> CreateUploadResult:
        """Uploads an app or test scripts.

        :param self:
        :param contentType:  The upload's content type (for example, "application/octet-stream").<br/>type:  The upload's upload type. Must be one of the following values: <ul> <li> ANDROID_APP: An Android upload. </li> <li> IOS_APP: An iOS upload. </li> <li> WEB_APP: A web application upload. </li> <li> EXTERNAL_DATA: An external data upload. </li> <li> APPIUM_JAVA_JUNIT_TEST_PACKAGE: An Appium Java JUnit test package upload. </li> <li> APPIUM_JAVA_TESTNG_TEST_PACKAGE: An Appium Java TestNG test package upload. </li> <li> APPIUM_PYTHON_TEST_PACKAGE: An Appium Python test package upload. </li> <li> APPIUM_NODE_TEST_PACKAGE: An Appium Node.js test package upload. </li> <li> APPIUM_RUBY_TEST_PACKAGE: An Appium Ruby test package upload. </li> <li> APPIUM_WEB_JAVA_JUNIT_TEST_PACKAGE: An Appium Java JUnit test package upload for a web app. </li> <li> APPIUM_WEB_JAVA_TESTNG_TEST_PACKAGE: An Appium Java TestNG test package upload for a web app. </li> <li> APPIUM_WEB_PYTHON_TEST_PACKAGE: An Appium Python test package upload for a web app. </li> <li> APPIUM_WEB_NODE_TEST_PACKAGE: An Appium Node.js test package upload for a web app. </li> <li> APPIUM_WEB_RUBY_TEST_PACKAGE: An Appium Ruby test package upload for a web app. </li> <li> CALABASH_TEST_PACKAGE: A Calabash test package upload. </li> <li> INSTRUMENTATION_TEST_PACKAGE: An instrumentation upload. </li> <li> UIAUTOMATION_TEST_PACKAGE: A uiautomation test package upload. </li> <li> UIAUTOMATOR_TEST_PACKAGE: A uiautomator test package upload. </li> <li> XCTEST_TEST_PACKAGE: An Xcode test package upload. </li> <li> XCTEST_UI_TEST_PACKAGE: An Xcode UI test package upload. </li> <li> APPIUM_JAVA_JUNIT_TEST_SPEC: An Appium Java JUnit test spec upload. </li> <li> APPIUM_JAVA_TESTNG_TEST_SPEC: An Appium Java TestNG test spec upload. </li> <li> APPIUM_PYTHON_TEST_SPEC: An Appium Python test spec upload. </li> <li> APPIUM_NODE_TEST_SPEC: An Appium Node.js test spec upload. </li> <li> APPIUM_RUBY_TEST_SPEC: An Appium Ruby test spec upload. </li> <li> APPIUM_WEB_JAVA_JUNIT_TEST_SPEC: An Appium Java JUnit test spec upload for a web app. </li> <li> APPIUM_WEB_JAVA_TESTNG_TEST_SPEC: An Appium Java TestNG test spec upload for a web app. </li> <li> APPIUM_WEB_PYTHON_TEST_SPEC: An Appium Python test spec upload for a web app. </li> <li> APPIUM_WEB_NODE_TEST_SPEC: An Appium Node.js test spec upload for a web app. </li> <li> APPIUM_WEB_RUBY_TEST_SPEC: An Appium Ruby test spec upload for a web app. </li> <li> INSTRUMENTATION_TEST_SPEC: An instrumentation test spec upload. </li> <li> XCTEST_UI_TEST_SPEC: An Xcode UI test spec upload. </li> </ul>  <b>Note</b> If you call <code>CreateUpload</code> with <code>WEB_APP</code> specified, AWS Device Farm throws an <code>ArgumentException</code> error.<br/>name:  The upload's file name. The name should not contain the '/' character. If uploading an iOS app, the file name needs to end with the <code>.ipa</code> extension. If uploading an Android app, the file name needs to end with the <code>.apk</code> extension. For all others, the file name must end with the <code>.zip</code> file extension.<br/>projectArn:  The ARN of the project for the upload.<br/>
        :return: upload:  The newly created upload.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>category</i> The upload's category. Allowed values include: <ul> <li> CURATED: An upload managed by AWS Device Farm. </li> <li> PRIVATE: An upload managed by the AWS Device Farm customer. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>message</i> A message about the upload's result.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>contentType</i> The upload's content type (for example, "application/octet-stream").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>metadata</i> The upload's metadata. For example, for Android, this contains information that is parsed from the manifest and is displayed in the AWS Device Farm console after the associated app is uploaded.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>url</i> The pre-signed Amazon S3 URL that was used to store a file through a corresponding PUT request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The upload's status. Must be one of the following values: <ul> <li> FAILED: A failed status. </li> <li> INITIALIZED: An initialized status. </li> <li> PROCESSING: A processing status. </li> <li> SUCCEEDED: A succeeded status. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The upload's type. Must be one of the following values: <ul> <li> ANDROID_APP: An Android upload. </li> <li> IOS_APP: An iOS upload. </li> <li> WEB_APP: A web application upload. </li> <li> EXTERNAL_DATA: An external data upload. </li> <li> APPIUM_JAVA_JUNIT_TEST_PACKAGE: An Appium Java JUnit test package upload. </li> <li> APPIUM_JAVA_TESTNG_TEST_PACKAGE: An Appium Java TestNG test package upload. </li> <li> APPIUM_PYTHON_TEST_PACKAGE: An Appium Python test package upload. </li> <li> APPIUM_NODE_TEST_PACKAGE: An Appium Node.js test package upload. </li> <li> APPIUM_RUBY_TEST_PACKAGE: An Appium Ruby test package upload. </li> <li> APPIUM_WEB_JAVA_JUNIT_TEST_PACKAGE: An Appium Java JUnit test package upload for web apps. </li> <li> APPIUM_WEB_JAVA_TESTNG_TEST_PACKAGE: An Appium Java TestNG test package upload for web apps. </li> <li> APPIUM_WEB_PYTHON_TEST_PACKAGE: An Appium Python test package upload for web apps. </li> <li> APPIUM_WEB_NODE_TEST_PACKAGE: An Appium Node.js test package upload for web apps. </li> <li> APPIUM_WEB_RUBY_TEST_PACKAGE: An Appium Ruby test package upload for web apps. </li> <li> CALABASH_TEST_PACKAGE: A Calabash test package upload. </li> <li> INSTRUMENTATION_TEST_PACKAGE: An instrumentation upload. </li> <li> UIAUTOMATION_TEST_PACKAGE: A uiautomation test package upload. </li> <li> UIAUTOMATOR_TEST_PACKAGE: A uiautomator test package upload. </li> <li> XCTEST_TEST_PACKAGE: An Xcode test package upload. </li> <li> XCTEST_UI_TEST_PACKAGE: An Xcode UI test package upload. </li> <li> APPIUM_JAVA_JUNIT_TEST_SPEC: An Appium Java JUnit test spec upload. </li> <li> APPIUM_JAVA_TESTNG_TEST_SPEC: An Appium Java TestNG test spec upload. </li> <li> APPIUM_PYTHON_TEST_SPEC: An Appium Python test spec upload. </li> <li> APPIUM_NODE_TEST_SPEC: An Appium Node.js test spec upload. </li> <li> APPIUM_RUBY_TEST_SPEC: An Appium Ruby test spec upload. </li> <li> APPIUM_WEB_JAVA_JUNIT_TEST_SPEC: An Appium Java JUnit test spec upload for a web app. </li> <li> APPIUM_WEB_JAVA_TESTNG_TEST_SPEC: An Appium Java TestNG test spec upload for a web app. </li> <li> APPIUM_WEB_PYTHON_TEST_SPEC: An Appium Python test spec upload for a web app. </li> <li> APPIUM_WEB_NODE_TEST_SPEC: An Appium Node.js test spec upload for a web app. </li> <li> APPIUM_WEB_RUBY_TEST_SPEC: An Appium Ruby test spec upload for a web app. </li> <li> INSTRUMENTATION_TEST_SPEC: An instrumentation test spec upload. </li> <li> XCTEST_UI_TEST_SPEC: An Xcode UI test spec upload. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>created</i> When the upload was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The upload's file name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The upload's ARN.<br/>        """
        pass

    def create_vpce_configuration(self, serviceDnsName: str, vpceServiceName: str, vpceConfigurationName: str, vpceConfigurationDescription: str=None) -> CreateVPCEConfigurationResult:
        """Creates a configuration record in Device Farm for your Amazon Virtual Private Cloud (VPC) endpoint.

        :param self:
        :param vpceConfigurationDescription:  An optional description, providing more details about your VPC endpoint configuration.<br/>serviceDnsName:  The DNS name of the service running in your VPC that you want Device Farm to test.<br/>vpceServiceName:  The name of the VPC endpoint service running inside your AWS account that you want Device Farm to test.<br/>vpceConfigurationName:  The friendly name you give to your VPC endpoint configuration, to manage your configurations more easily.<br/>
        :return: vpceConfiguration:  An object containing information about your VPC endpoint configuration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vpceConfigurationDescription</i> An optional description, providing more details about your VPC endpoint configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceDnsName</i> The DNS name that maps to the private IP address of the service you want to access.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vpceServiceName</i> The name of the VPC endpoint service running inside your AWS account that you want Device Farm to test.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vpceConfigurationName</i> The friendly name you give to your VPC endpoint configuration, to manage your configurations more easily.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the VPC endpoint configuration.<br/>        """
        pass

    def delete_device_pool(self, arn: str) -> DeleteDevicePoolResult:
        """Deletes a device pool given the pool ARN. Does not allow deletion of curated pools owned by the system.

        :param self:
        :param arn:  Represents the Amazon Resource Name (ARN) of the Device Farm device pool you wish to delete.<br/>
        :return:         """
        pass

    def delete_instance_profile(self, arn: str) -> DeleteInstanceProfileResult:
        """Deletes a profile that can be applied to one or more private device instances.

        :param self:
        :param arn:  The Amazon Resource Name (ARN) of the instance profile you are requesting to delete.<br/>
        :return:         """
        pass

    def delete_network_profile(self, arn: str) -> DeleteNetworkProfileResult:
        """Deletes a network profile.

        :param self:
        :param arn:  The Amazon Resource Name (ARN) of the network profile you want to delete.<br/>
        :return:         """
        pass

    def delete_project(self, arn: str) -> DeleteProjectResult:
        """Deletes an AWS Device Farm project, given the project ARN.  <b>Note</b> Deleting this resource does not stop an in-progress run.

        :param self:
        :param arn:  Represents the Amazon Resource Name (ARN) of the Device Farm project you wish to delete.<br/>
        :return:         """
        pass

    def delete_remote_access_session(self, arn: str) -> DeleteRemoteAccessSessionResult:
        """Deletes a completed remote access session and its results.

        :param self:
        :param arn:  The Amazon Resource Name (ARN) of the session for which you want to delete remote access.<br/>
        :return:         """
        pass

    def delete_run(self, arn: str) -> DeleteRunResult:
        """Deletes the run, given the run ARN.  <b>Note</b> Deleting this resource does not stop an in-progress run.

        :param self:
        :param arn:  The Amazon Resource Name (ARN) for the run you wish to delete.<br/>
        :return:         """
        pass

    def delete_upload(self, arn: str) -> DeleteUploadResult:
        """Deletes an upload given the upload ARN.

        :param self:
        :param arn:  Represents the Amazon Resource Name (ARN) of the Device Farm upload you wish to delete.<br/>
        :return:         """
        pass

    def delete_vpce_configuration(self, arn: str) -> DeleteVPCEConfigurationResult:
        """Deletes a configuration for your Amazon Virtual Private Cloud (VPC) endpoint.

        :param self:
        :param arn:  The Amazon Resource Name (ARN) of the VPC endpoint configuration you want to delete.<br/>
        :return:         """
        pass

    def get_account_settings(self) -> GetAccountSettingsResult:
        """Returns the number of unmetered iOS and/or unmetered Android devices that have been purchased by the account.

        :param self:
        :param 
        :return: accountSettings:  The account settings.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>skipAppResign</i> When set to <code>true</code>, for private devices, Device Farm will not sign your app again. For public devices, Device Farm always signs your apps again and this parameter has no effect. For more information about how Device Farm re-signs your app(s), see <a href="https://aws.amazon.com/device-farm/faq/">Do you modify my app?</a> in the <i>AWS Device Farm FAQs</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultJobTimeoutMinutes</i> The default number of minutes (at the account level) a test run will execute before it times out. The default value is 150 minutes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>maxSlots</i> The maximum number of device slots that the AWS account can purchase. Each maximum is expressed as an <code>offering-id:number</code> pair, where the <code>offering-id</code> represents one of the IDs returned by the <code>ListOfferings</code> command.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>trialMinutes</i> Information about an AWS account's usage of free trial device minutes.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>trialMinutes</b>: Information about an AWS account's usage of free trial device minutes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>maxJobTimeoutMinutes</i> The maximum number of minutes a test run will execute before it times out.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>unmeteredRemoteAccessDevices</i> Returns the unmetered remote access devices you have purchased or want to purchase.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>unmeteredDevices</i> Returns the unmetered devices you have purchased or want to purchase.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>awsAccountNumber</i> The AWS account number specified in the <code>AccountSettings</code> container.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>remaining</i> The number of free trial minutes remaining in the account.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>total</i> The total number of free trial minutes that the account started with.<br/>        """
        pass

    def get_device(self, arn: str) -> GetDeviceResult:
        """Gets information about a unique device type.

        :param self:
        :param arn:  The device type's ARN.<br/>
        :return: device:  An object containing information about the requested device.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>availability</i> Reflects how likely a device will be available for a test run. It is currently available in the ListDevices and GetDevice API methods.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>instances</i> The instances belonging to this device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fleetName</i> The name of the fleet to which this device belongs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fleetType</i> The type of fleet to which this device belongs. Possible values for fleet type are PRIVATE and PUBLIC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>remoteDebugEnabled</i> This flag is set to <code>true</code> if remote debugging is enabled for the device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>remoteAccessEnabled</i> Specifies whether remote access has been enabled for the specified device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>radio</i> The device's radio.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>carrier</i> The device's carrier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>image</i> The device's image name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>memory</i> The device's total memory size, expressed in bytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>heapSize</i> The device's heap size, expressed in bytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resolution</i> The resolution of the device.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>resolution</b>: The resolution of the device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cpu</i> Information about the device's CPU.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>cpu</b>: Information about the device's CPU.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>os</i> The device's operating system type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>platform</i> The device's platform. Allowed values include: <ul> <li> ANDROID: The Android platform. </li> <li> IOS: The iOS platform. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>formFactor</i> The device's form factor. Allowed values include: <ul> <li> PHONE: The phone form factor. </li> <li> TABLET: The tablet form factor. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>modelId</i> The device's model ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>model</i> The device's model name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>manufacturer</i> The device's manufacturer name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The device's display name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The device's ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>clock</i> The clock speed of the device's CPU, expressed in hertz (Hz). For example, a 1.2 GHz CPU is expressed as 1200000000.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>architecture</i> The CPU's architecture, for example x86 or ARM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>frequency</i> The CPU's frequency.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>height</i> The screen resolution's height, expressed in pixels.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>width</i> The screen resolution's width, expressed in pixels.<br/>        """
        pass

    def get_device_instance(self, arn: str) -> GetDeviceInstanceResult:
        """Returns information about a device instance belonging to a private device fleet.

        :param self:
        :param arn:  The Amazon Resource Name (ARN) of the instance you're requesting information about.<br/>
        :return: deviceInstance:  An object containing information about your device instance.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>instanceProfile</i> A object containing information about the instance profile.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>instanceProfile</b>: A object containing information about the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>udid</i> Unique device identifier for the device instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the device instance. Valid values are listed below.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>labels</i> An array of strings describing the device instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deviceArn</i> The Amazon Resource Name (ARN) of the device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the device instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The description of the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>rebootAfterUse</i> When set to <code>true</code>, Device Farm will reboot the instance after a test run. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>excludeAppPackagesFromCleanup</i> An array of strings specifying the list of app packages that should not be cleaned up from the device after a test run is over. The list of packages is only considered if you set <code>packageCleanup</code> to <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>packageCleanup</i> When set to <code>true</code>, Device Farm will remove app packages after a test run. The default value is <code>false</code> for private devices.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the instance profile.<br/>        """
        pass

    def get_device_pool(self, arn: str) -> GetDevicePoolResult:
        """Gets information about a device pool.

        :param self:
        :param arn:  The device pool's ARN.<br/>
        :return: devicePool:  An object containing information about the requested device pool.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>maxDevices</i> The number of devices that Device Farm can add to your device pool. Device Farm adds devices that are available and that meet the criteria that you assign for the <code>rules</code> parameter. Depending on how many devices meet these constraints, your device pool might contain fewer devices than the value for this parameter. By specifying the maximum number of devices, you can control the costs that you incur by running tests.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>rules</i> Information about the device pool's rules.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The device pool's type. Allowed values include: <ul> <li> CURATED: A device pool that is created and managed by AWS Device Farm. </li> <li> PRIVATE: A device pool that is created and managed by the device pool developer. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The device pool's description.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The device pool's name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The device pool's ARN.<br/>        """
        pass

    def get_device_pool_compatibility(self, devicePoolArn: str, appArn: str=None, testType: str=None, test: ScheduleRunTest=None, configuration: ScheduleRunConfiguration=None) -> GetDevicePoolCompatibilityResult:
        """Gets information about compatibility with a device pool.

        :param self:
        :param configuration:  An object containing information about the settings for a run.:<br/>test:  Information about the uploaded test to be run against the device pool.:<br/>testType:  The test type for the specified device pool. Allowed values include the following: <ul> <li> BUILTIN_FUZZ: The built-in fuzz type. </li> <li> BUILTIN_EXPLORER: For Android, an app explorer that will traverse an Android app, interacting with it and capturing screenshots at the same time. </li> <li> APPIUM_JAVA_JUNIT: The Appium Java JUnit type. </li> <li> APPIUM_JAVA_TESTNG: The Appium Java TestNG type. </li> <li> APPIUM_PYTHON: The Appium Python type. </li> <li> APPIUM_NODE: The Appium Node.js type. </li> <li> APPIUM_RUBY: The Appium Ruby type. </li> <li> APPIUM_WEB_JAVA_JUNIT: The Appium Java JUnit type for web apps. </li> <li> APPIUM_WEB_JAVA_TESTNG: The Appium Java TestNG type for web apps. </li> <li> APPIUM_WEB_PYTHON: The Appium Python type for web apps. </li> <li> APPIUM_WEB_NODE: The Appium Node.js type for web apps. </li> <li> APPIUM_WEB_RUBY: The Appium Ruby type for web apps. </li> <li> CALABASH: The Calabash type. </li> <li> INSTRUMENTATION: The Instrumentation type. </li> <li> UIAUTOMATION: The uiautomation type. </li> <li> UIAUTOMATOR: The uiautomator type. </li> <li> XCTEST: The Xcode test type. </li> <li> XCTEST_UI: The Xcode UI test type. </li> </ul><br/>appArn:  The ARN of the app that is associated with the specified device pool.<br/>devicePoolArn:  The device pool's ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>parameters</i> The test's parameters, such as test framework parameters and fixture settings. Parameters are represented by name-value pairs of strings. For all tests: <ul> <li> app_performance_monitoring: Performance monitoring is enabled by default. Set this parameter to "false" to disable it. </li> </ul> For Calabash tests: <ul> <li> profile: A cucumber profile, for example, "my_profile_name". </li> <li> tags: You can limit execution to features or scenarios that have (or don't have) certain tags, for example, "@smoke" or "@smoke,~@wip". </li> </ul> For Appium tests (all types): <ul> <li> appium_version: The Appium version. Currently supported values are "1.6.5" (and higher), "latest", and "default". <ul> <li> “latest” will run the latest Appium version supported by Device Farm (1.9.1). </li> <li> For “default”, Device Farm will choose a compatible version of Appium for the device. The current behavior is to run 1.7.2 on Android devices and iOS 9 and earlier, 1.7.2 for iOS 10 and later. </li> <li> This behavior is subject to change. </li> </ul> </li> </ul> For Fuzz tests (Android only): <ul> <li> event_count: The number of events, between 1 and 10000, that the UI fuzz test should perform. </li> <li> throttle: The time, in ms, between 0 and 1000, that the UI fuzz test should wait between events. </li> <li> seed: A seed to use for randomizing the UI fuzz test. Using the same seed value between tests ensures identical event sequences. </li> </ul> For Explorer tests: <ul> <li> username: A username to use if the Explorer encounters a login form. If not supplied, no username will be inserted. </li> <li> password: A password to use if the Explorer encounters a login form. If not supplied, no password will be inserted. </li> </ul> For Instrumentation: <ul> <li> filter: A test filter string. Examples: <ul> <li> Running a single test case: "com.android.abc.Test1" </li> <li> Running a single test: "com.android.abc.Test1#smoke" </li> <li> Running multiple tests: "com.android.abc.Test1,com.android.abc.Test2" </li> </ul> </li> </ul> For XCTest and XCTestUI: <ul> <li> filter: A test filter string. Examples: <ul> <li> Running a single test class: "LoginTests" </li> <li> Running a multiple test classes: "LoginTests,SmokeTests" </li> <li> Running a single test: "LoginTests/testValid" </li> <li> Running multiple tests: "LoginTests/testValid,LoginTests/testInvalid" </li> </ul> </li> </ul> For UIAutomator: <ul> <li> filter: A test filter string. Examples: <ul> <li> Running a single test case: "com.android.abc.Test1" </li> <li> Running a single test: "com.android.abc.Test1#smoke" </li> <li> Running multiple tests: "com.android.abc.Test1,com.android.abc.Test2" </li> </ul> </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>filter</i> The test's filter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>testSpecArn</i> The ARN of the YAML-formatted test specification.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>testPackageArn</i> The ARN of the uploaded test that will be run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The test's type. Must be one of the following values: <ul> <li> BUILTIN_FUZZ: The built-in fuzz type. </li> <li> BUILTIN_EXPLORER: For Android, an app explorer that will traverse an Android app, interacting with it and capturing screenshots at the same time. </li> <li> APPIUM_JAVA_JUNIT: The Appium Java JUnit type. </li> <li> APPIUM_JAVA_TESTNG: The Appium Java TestNG type. </li> <li> APPIUM_PYTHON: The Appium Python type. </li> <li> APPIUM_NODE: The Appium Node.js type. </li> <li> APPIUM_RUBY: The Appium Ruby type. </li> <li> APPIUM_WEB_JAVA_JUNIT: The Appium Java JUnit type for web apps. </li> <li> APPIUM_WEB_JAVA_TESTNG: The Appium Java TestNG type for web apps. </li> <li> APPIUM_WEB_PYTHON: The Appium Python type for web apps. </li> <li> APPIUM_WEB_NODE: The Appium Node.js type for web apps. </li> <li> APPIUM_WEB_RUBY: The Appium Ruby type for web apps. </li> <li> CALABASH: The Calabash type. </li> <li> INSTRUMENTATION: The Instrumentation type. </li> <li> UIAUTOMATION: The uiautomation type. </li> <li> UIAUTOMATOR: The uiautomator type. </li> <li> XCTEST: The Xcode test type. </li> <li> XCTEST_UI: The Xcode UI test type. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>billingMethod</i> Specifies the billing method for a test run: <code>metered</code> or <code>unmetered</code>. If the parameter is not specified, the default value is <code>metered</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>auxiliaryApps</i> A list of auxiliary apps for the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>radios</i> Information about the radio states for the run.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>radios</b>: Information about the radio states for the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customerArtifactPaths</i> Input <code>CustomerArtifactPaths</code> object for the scheduled run configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>customerArtifactPaths</b>: Input <code>CustomerArtifactPaths</code> object for the scheduled run configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vpceConfigurationArns</i> An array of Amazon Resource Names (ARNs) for your VPC endpoint configurations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> Information about the location that is used for the run.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>location</b>: Information about the location that is used for the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>locale</i> Information about the locale that is used for the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>networkProfileArn</i> Reserved for internal use.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>extraDataPackageArn</i> The ARN of the extra data for the run. The extra data is a .zip file that AWS Device Farm will extract to external data for Android or the app's sandbox for iOS.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>longitude</i> The longitude.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>latitude</i> The latitude.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>deviceHostPaths</i> Comma-separated list of paths in the test execution environment where the artifacts generated by the customer's tests will be pulled from.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>androidPaths</i> Comma-separated list of paths on the Android device where the artifacts generated by the customer's tests will be pulled from.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>iosPaths</i> Comma-separated list of paths on the iOS device where the artifacts generated by the customer's tests will be pulled from.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>gps</i> True if GPS is enabled at the beginning of the test; otherwise, false.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>nfc</i> True if NFC is enabled at the beginning of the test; otherwise, false.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>bluetooth</i> True if Bluetooth is enabled at the beginning of the test; otherwise, false.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>wifi</i> True if Wi-Fi is enabled at the beginning of the test; otherwise, false.<br/>
        :return: incompatibleDevices:  Information about incompatible devices.<br/>compatibleDevices:  Information about compatible devices.<br/>        """
        pass

    def get_instance_profile(self, arn: str) -> GetInstanceProfileResult:
        """Returns information about the specified instance profile.

        :param self:
        :param arn:  The Amazon Resource Name (ARN) of your instance profile.<br/>
        :return: instanceProfile:  An object containing information about your instance profile.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The description of the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>rebootAfterUse</i> When set to <code>true</code>, Device Farm will reboot the instance after a test run. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>excludeAppPackagesFromCleanup</i> An array of strings specifying the list of app packages that should not be cleaned up from the device after a test run is over. The list of packages is only considered if you set <code>packageCleanup</code> to <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>packageCleanup</i> When set to <code>true</code>, Device Farm will remove app packages after a test run. The default value is <code>false</code> for private devices.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the instance profile.<br/>        """
        pass

    def get_job(self, arn: str) -> GetJobResult:
        """Gets information about a job.

        :param self:
        :param arn:  The job's ARN.<br/>
        :return: job:  An object containing information about the requested job.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>videoCapture</i> This value is set to true if video capture is enabled; otherwise, it is set to false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>videoEndpoint</i> The endpoint for streaming device video.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deviceMinutes</i> Represents the total (metered or unmetered) minutes used by the job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>deviceMinutes</b>: Represents the total (metered or unmetered) minutes used by the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>instanceArn</i> The Amazon Resource Name (ARN) of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>device</i> The device (phone or tablet).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>device</b>: The device (phone or tablet).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>message</i> A message about the job's result.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>counters</i> The job's result counters.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>counters</b>: The job's result counters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stopped</i> The job's stop time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>started</i> The job's start time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>result</i> The job's result. Allowed values include: <ul> <li> PENDING: A pending condition. </li> <li> PASSED: A passing condition. </li> <li> WARNED: A warning condition. </li> <li> FAILED: A failed condition. </li> <li> SKIPPED: A skipped condition. </li> <li> ERRORED: An error condition. </li> <li> STOPPED: A stopped condition. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The job's status. Allowed values include: <ul> <li> PENDING: A pending status. </li> <li> PENDING_CONCURRENCY: A pending concurrency status. </li> <li> PENDING_DEVICE: A pending device status. </li> <li> PROCESSING: A processing status. </li> <li> SCHEDULING: A scheduling status. </li> <li> PREPARING: A preparing status. </li> <li> RUNNING: A running status. </li> <li> COMPLETED: A completed status. </li> <li> STOPPING: A stopping status. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>created</i> When the job was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The job's type. Allowed values include the following: <ul> <li> BUILTIN_FUZZ: The built-in fuzz type. </li> <li> BUILTIN_EXPLORER: For Android, an app explorer that will traverse an Android app, interacting with it and capturing screenshots at the same time. </li> <li> APPIUM_JAVA_JUNIT: The Appium Java JUnit type. </li> <li> APPIUM_JAVA_TESTNG: The Appium Java TestNG type. </li> <li> APPIUM_PYTHON: The Appium Python type. </li> <li> APPIUM_NODE: The Appium Node.js type. </li> <li> APPIUM_RUBY: The Appium Ruby type. </li> <li> APPIUM_WEB_JAVA_JUNIT: The Appium Java JUnit type for web apps. </li> <li> APPIUM_WEB_JAVA_TESTNG: The Appium Java TestNG type for web apps. </li> <li> APPIUM_WEB_PYTHON: The Appium Python type for web apps. </li> <li> APPIUM_WEB_NODE: The Appium Node.js type for web apps. </li> <li> APPIUM_WEB_RUBY: The Appium Ruby test type for web apps. </li> <li> CALABASH: The Calabash type. </li> <li> INSTRUMENTATION: The Instrumentation type. </li> <li> UIAUTOMATION: The uiautomation type. </li> <li> UIAUTOMATOR: The uiautomator type. </li> <li> XCTEST: The Xcode test type. </li> <li> XCTEST_UI: The Xcode UI test type. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The job's name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The job's ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>skipped</i> The number of skipped entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>stopped</i> The number of stopped entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>errored</i> The number of errored entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>warned</i> The number of warned entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>failed</i> The number of failed entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>passed</i> The number of passed entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>total</i> The total number of entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>os</i> The device's operating system type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>platform</i> The device's platform. Allowed values include: <ul> <li> ANDROID: The Android platform. </li> <li> IOS: The iOS platform. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>formFactor</i> The device's form factor. Allowed values include: <ul> <li> PHONE: The phone form factor. </li> <li> TABLET: The tablet form factor. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>modelId</i> The device's model ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>model</i> The device's model name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>manufacturer</i> The device's manufacturer name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The device's display name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The device's ARN.<br/>Information about the device's CPU.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>cpu</b>: Information about the device's CPU.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>unmetered</i> When specified, represents only the sum of unmetered minutes used by the resource to run tests.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>metered</i> When specified, represents only the sum of metered minutes used by the resource to run tests.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>total</i> When specified, represents the total minutes used by the resource to run tests.<br/>        """
        pass

    def get_network_profile(self, arn: str) -> GetNetworkProfileResult:
        """Returns information about a network profile.

        :param self:
        :param arn:  The Amazon Resource Name (ARN) of the network profile you want to return information about.<br/>
        :return: networkProfile:  The network profile.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>downlinkLossPercent</i> Proportion of received packets that fail to arrive from 0 to 100 percent.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>uplinkLossPercent</i> Proportion of transmitted packets that fail to arrive from 0 to 100 percent.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>downlinkJitterMs</i> Time variation in the delay of received packets in milliseconds as an integer from 0 to 2000.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>uplinkJitterMs</i> Time variation in the delay of received packets in milliseconds as an integer from 0 to 2000.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>downlinkDelayMs</i> Delay time for all packets to destination in milliseconds as an integer from 0 to 2000.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>uplinkDelayMs</i> Delay time for all packets to destination in milliseconds as an integer from 0 to 2000.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>downlinkBandwidthBits</i> The data throughput rate in bits per second, as an integer from 0 to 104857600.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>uplinkBandwidthBits</i> The data throughput rate in bits per second, as an integer from 0 to 104857600.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of network profile. Valid values are listed below.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The description of the network profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the network profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the network profile.<br/>        """
        pass

    def get_offering_status(self, nextToken: str=None) -> GetOfferingStatusResult:
        """Gets the current status and future status of all offerings purchased by an AWS account. The response indicates how many offerings are currently available and the offerings that will be available in the next period. The API returns a <code>NotEligible</code> error if the user is not permitted to invoke the operation. Please contact <a href="mailto:aws-devicefarm-support@amazon.com">aws-devicefarm-support@amazon.com</a> if you believe that you should be able to invoke this operation.

        :param self:
        :param nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>
        :return: nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>nextPeriod:  When specified, gets the offering status for the next period.<br/>current:  When specified, gets the offering status for the current period.<br/>        """
        pass

    def get_project(self, arn: str) -> GetProjectResult:
        """Gets information about a project.

        :param self:
        :param arn:  The project's ARN.<br/>
        :return: project:  The project you wish to get information about.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>created</i> When the project was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultJobTimeoutMinutes</i> The default number of minutes (at the project level) a test run will execute before it times out. The default value is 150 minutes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The project's name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The project's ARN.<br/>        """
        pass

    def get_remote_access_session(self, arn: str) -> GetRemoteAccessSessionResult:
        """Returns a link to a currently running remote access session.

        :param self:
        :param arn:  The Amazon Resource Name (ARN) of the remote access session about which you want to get session information.<br/>
        :return: remoteAccessSession:  A container that lists detailed information about the remote access session.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>skipAppResign</i> When set to <code>true</code>, for private devices, Device Farm will not sign your app again. For public devices, Device Farm always signs your apps again and this parameter has no effect. For more information about how Device Farm re-signs your app(s), see <a href="https://aws.amazon.com/device-farm/faq/">Do you modify my app?</a> in the <i>AWS Device Farm FAQs</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>interactionMode</i> The interaction mode of the remote access session. Valid values are: <ul> <li> INTERACTIVE: You can interact with the iOS device by viewing, touching, and rotating the screen. You <b>cannot</b> run XCUITest framework-based tests in this mode. </li> <li> NO_VIDEO: You are connected to the device but cannot interact with it or view the screen. This mode has the fastest test execution speed. You <b>can</b> run XCUITest framework-based tests in this mode. </li> <li> VIDEO_ONLY: You can view the screen but cannot touch or rotate it. You <b>can</b> run XCUITest framework-based tests and watch the screen in this mode. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deviceUdid</i> Unique device identifier for the remote device. Only returned if remote debugging is enabled for the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>endpoint</i> The endpoint for the remote access sesssion.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deviceMinutes</i> The number of minutes a device is used in a remote access session (including setup and teardown minutes).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>deviceMinutes</b>: The number of minutes a device is used in a remote access session (including setup and teardown minutes).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>billingMethod</i> The billing method of the remote access session. Possible values include <code>METERED</code> or <code>UNMETERED</code>. For more information about metered devices, see <a href="https://docs.aws.amazon.com/devicefarm/latest/developerguide/welcome.html#welcome-terminology">AWS Device Farm terminology</a>."<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clientId</i> Unique identifier of your client for the remote access session. Only returned if remote debugging is enabled for the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>hostAddress</i> IP address of the EC2 host where you need to connect to remotely debug devices. Only returned if remote debugging is enabled for the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>remoteRecordAppArn</i> The Amazon Resource Name (ARN) for the app to be recorded in the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>remoteRecordEnabled</i> This flag is set to <code>true</code> if remote recording is enabled for the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>remoteDebugEnabled</i> This flag is set to <code>true</code> if remote debugging is enabled for the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>instanceArn</i> The Amazon Resource Name (ARN) of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>device</i> The device (phone or tablet) used in the remote access session.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>device</b>: The device (phone or tablet) used in the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stopped</i> The date and time the remote access session was stopped.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>started</i> The date and time the remote access session was started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>message</i> A message about the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>result</i> The result of the remote access session. Can be any of the following: <ul> <li> PENDING: A pending condition. </li> <li> PASSED: A passing condition. </li> <li> WARNED: A warning condition. </li> <li> FAILED: A failed condition. </li> <li> SKIPPED: A skipped condition. </li> <li> ERRORED: An error condition. </li> <li> STOPPED: A stopped condition. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the remote access session. Can be any of the following: <ul> <li> PENDING: A pending status. </li> <li> PENDING_CONCURRENCY: A pending concurrency status. </li> <li> PENDING_DEVICE: A pending device status. </li> <li> PROCESSING: A processing status. </li> <li> SCHEDULING: A scheduling status. </li> <li> PREPARING: A preparing status. </li> <li> RUNNING: A running status. </li> <li> COMPLETED: A completed status. </li> <li> STOPPING: A stopping status. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>created</i> The date and time the remote access session was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>os</i> The device's operating system type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>platform</i> The device's platform. Allowed values include: <ul> <li> ANDROID: The Android platform. </li> <li> IOS: The iOS platform. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>formFactor</i> The device's form factor. Allowed values include: <ul> <li> PHONE: The phone form factor. </li> <li> TABLET: The tablet form factor. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>modelId</i> The device's model ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>model</i> The device's model name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>manufacturer</i> The device's manufacturer name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The device's display name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The device's ARN.<br/>Information about the device's CPU.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>cpu</b>: Information about the device's CPU.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>unmetered</i> When specified, represents only the sum of unmetered minutes used by the resource to run tests.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>metered</i> When specified, represents only the sum of metered minutes used by the resource to run tests.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>total</i> When specified, represents the total minutes used by the resource to run tests.<br/>        """
        pass

    def get_run(self, arn: str) -> GetRunResult:
        """Gets information about a run.

        :param self:
        :param arn:  The run's ARN.<br/>
        :return: run:  The run you wish to get results from.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deviceSelectionResult</i> The results of a device filter used to select the devices for a test run.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>deviceSelectionResult</b>: The results of a device filter used to select the devices for a test run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>testSpecArn</i> The ARN of the YAML-formatted test specification for the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>skipAppResign</i> When set to <code>true</code>, for private devices, Device Farm will not sign your app again. For public devices, Device Farm always signs your apps again and this parameter has no effect. For more information about how Device Farm re-signs your app(s), see <a href="https://aws.amazon.com/device-farm/faq/">Do you modify my app?</a> in the <i>AWS Device Farm FAQs</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>webUrl</i> The Device Farm console URL for the recording of the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customerArtifactPaths</i> Output <code>CustomerArtifactPaths</code> object for the test run.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>customerArtifactPaths</b>: Output <code>CustomerArtifactPaths</code> object for the test run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> Information about the location that is used for the run.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>location</b>: Information about the location that is used for the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>radios</i> Information about the radio states for the run.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>radios</b>: Information about the radio states for the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>locale</i> Information about the locale that is used for the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>devicePoolArn</i> The ARN of the device pool for the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jobTimeoutMinutes</i> The number of minutes the job will execute before it times out.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>eventCount</i> For fuzz tests, this is the number of events, between 1 and 10000, that the UI fuzz test should perform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>appUpload</i> An app to upload or that has been uploaded.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>seed</i> For fuzz tests, this is a seed to use for randomizing the UI fuzz test. Using the same seed value between tests ensures identical event sequences.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resultCode</i> Supporting field for the result field. Set only if <code>result</code> is <code>SKIPPED</code>. <code>PARSING_FAILED</code> if the result is skipped because of test package parsing failure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>parsingResultUrl</i> Read-only URL for an object in S3 bucket where you can get the parsing results of the test package. If the test package doesn't parse, the reason why it doesn't parse appears in the file that this URL points to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>networkProfile</i> The network profile being used for a test run.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>networkProfile</b>: The network profile being used for a test run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deviceMinutes</i> Represents the total (metered or unmetered) minutes used by the test run.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>deviceMinutes</b>: Represents the total (metered or unmetered) minutes used by the test run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>billingMethod</i> Specifies the billing method for a test run: <code>metered</code> or <code>unmetered</code>. If the parameter is not specified, the default value is <code>metered</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>completedJobs</i> The total number of completed jobs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>totalJobs</i> The total number of jobs for the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>message</i> A message about the run's result.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>counters</i> The run's result counters.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>counters</b>: The run's result counters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stopped</i> The run's stop time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>started</i> The run's start time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>result</i> The run's result. Allowed values include: <ul> <li> PENDING: A pending condition. </li> <li> PASSED: A passing condition. </li> <li> WARNED: A warning condition. </li> <li> FAILED: A failed condition. </li> <li> SKIPPED: A skipped condition. </li> <li> ERRORED: An error condition. </li> <li> STOPPED: A stopped condition. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The run's status. Allowed values include: <ul> <li> PENDING: A pending status. </li> <li> PENDING_CONCURRENCY: A pending concurrency status. </li> <li> PENDING_DEVICE: A pending device status. </li> <li> PROCESSING: A processing status. </li> <li> SCHEDULING: A scheduling status. </li> <li> PREPARING: A preparing status. </li> <li> RUNNING: A running status. </li> <li> COMPLETED: A completed status. </li> <li> STOPPING: A stopping status. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>created</i> When the run was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>platform</i> The run's platform. Allowed values include: <ul> <li> ANDROID: The Android platform. </li> <li> IOS: The iOS platform. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The run's type. Must be one of the following values: <ul> <li> BUILTIN_FUZZ: The built-in fuzz type. </li> <li> BUILTIN_EXPLORER: For Android, an app explorer that will traverse an Android app, interacting with it and capturing screenshots at the same time. </li> <li> APPIUM_JAVA_JUNIT: The Appium Java JUnit type. </li> <li> APPIUM_JAVA_TESTNG: The Appium Java TestNG type. </li> <li> APPIUM_PYTHON: The Appium Python type. </li> <li> APPIUM_NODE: The Appium Node.js type. </li> <li> APPIUM_RUBY: The Appium Ruby type. </li> <li> APPIUM_WEB_JAVA_JUNIT: The Appium Java JUnit type for web apps. </li> <li> APPIUM_WEB_JAVA_TESTNG: The Appium Java TestNG type for web apps. </li> <li> APPIUM_WEB_PYTHON: The Appium Python type for web apps. </li> <li> APPIUM_WEB_NODE: The Appium Node.js type for web apps. </li> <li> APPIUM_WEB_RUBY: The Appium Ruby type for web apps. </li> <li> CALABASH: The Calabash type. </li> <li> INSTRUMENTATION: The Instrumentation type. </li> <li> UIAUTOMATION: The uiautomation type. </li> <li> UIAUTOMATOR: The uiautomator type. </li> <li> XCTEST: The Xcode test type. </li> <li> XCTEST_UI: The Xcode UI test type. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The run's name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The run's ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>skipped</i> The number of skipped entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>stopped</i> The number of stopped entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>errored</i> The number of errored entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>warned</i> The number of warned entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>failed</i> The number of failed entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>passed</i> The number of passed entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>total</i> The total number of entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>unmetered</i> When specified, represents only the sum of unmetered minutes used by the resource to run tests.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>metered</i> When specified, represents only the sum of metered minutes used by the resource to run tests.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>total</i> When specified, represents the total minutes used by the resource to run tests.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>downlinkLossPercent</i> Proportion of received packets that fail to arrive from 0 to 100 percent.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uplinkLossPercent</i> Proportion of transmitted packets that fail to arrive from 0 to 100 percent.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>downlinkJitterMs</i> Time variation in the delay of received packets in milliseconds as an integer from 0 to 2000.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uplinkJitterMs</i> Time variation in the delay of received packets in milliseconds as an integer from 0 to 2000.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>downlinkDelayMs</i> Delay time for all packets to destination in milliseconds as an integer from 0 to 2000.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uplinkDelayMs</i> Delay time for all packets to destination in milliseconds as an integer from 0 to 2000.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>downlinkBandwidthBits</i> The data throughput rate in bits per second, as an integer from 0 to 104857600.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uplinkBandwidthBits</i> The data throughput rate in bits per second, as an integer from 0 to 104857600.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of network profile. Valid values are listed below.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The description of the network profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the network profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the network profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>gps</i> True if GPS is enabled at the beginning of the test; otherwise, false.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>nfc</i> True if NFC is enabled at the beginning of the test; otherwise, false.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>bluetooth</i> True if Bluetooth is enabled at the beginning of the test; otherwise, false.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>wifi</i> True if Wi-Fi is enabled at the beginning of the test; otherwise, false.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>longitude</i> The longitude.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>latitude</i> The latitude.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>deviceHostPaths</i> Comma-separated list of paths in the test execution environment where the artifacts generated by the customer's tests will be pulled from.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>androidPaths</i> Comma-separated list of paths on the Android device where the artifacts generated by the customer's tests will be pulled from.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>iosPaths</i> Comma-separated list of paths on the iOS device where the artifacts generated by the customer's tests will be pulled from.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>maxDevices</i> The maximum number of devices to be selected by a device filter and included in a test run.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>matchedDevicesCount</i> The number of devices that matched the device filter selection criteria.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>filters</i> The filters in a device selection result.<br/>        """
        pass

    def get_suite(self, arn: str) -> GetSuiteResult:
        """Gets information about a suite.

        :param self:
        :param arn:  The suite's ARN.<br/>
        :return: suite:  A collection of one or more tests.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deviceMinutes</i> Represents the total (metered or unmetered) minutes used by the test suite.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>deviceMinutes</b>: Represents the total (metered or unmetered) minutes used by the test suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>message</i> A message about the suite's result.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>counters</i> The suite's result counters.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>counters</b>: The suite's result counters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stopped</i> The suite's stop time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>started</i> The suite's start time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>result</i> The suite's result. Allowed values include: <ul> <li> PENDING: A pending condition. </li> <li> PASSED: A passing condition. </li> <li> WARNED: A warning condition. </li> <li> FAILED: A failed condition. </li> <li> SKIPPED: A skipped condition. </li> <li> ERRORED: An error condition. </li> <li> STOPPED: A stopped condition. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The suite's status. Allowed values include: <ul> <li> PENDING: A pending status. </li> <li> PENDING_CONCURRENCY: A pending concurrency status. </li> <li> PENDING_DEVICE: A pending device status. </li> <li> PROCESSING: A processing status. </li> <li> SCHEDULING: A scheduling status. </li> <li> PREPARING: A preparing status. </li> <li> RUNNING: A running status. </li> <li> COMPLETED: A completed status. </li> <li> STOPPING: A stopping status. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>created</i> When the suite was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The suite's type. Must be one of the following values: <ul> <li> BUILTIN_FUZZ: The built-in fuzz type. </li> <li> BUILTIN_EXPLORER: For Android, an app explorer that will traverse an Android app, interacting with it and capturing screenshots at the same time. </li> <li> APPIUM_JAVA_JUNIT: The Appium Java JUnit type. </li> <li> APPIUM_JAVA_TESTNG: The Appium Java TestNG type. </li> <li> APPIUM_PYTHON: The Appium Python type. </li> <li> APPIUM_NODE: The Appium Node.js type. </li> <li> APPIUM_RUBY: The Appium Ruby type. </li> <li> APPIUM_WEB_JAVA_JUNIT: The Appium Java JUnit type for web apps. </li> <li> APPIUM_WEB_JAVA_TESTNG: The Appium Java TestNG type for web apps. </li> <li> APPIUM_WEB_PYTHON: The Appium Python type for web apps. </li> <li> APPIUM_WEB_NODE: The Appium Node.js type for web apps. </li> <li> APPIUM_WEB_RUBY: The Appium Ruby type for web apps. </li> <li> CALABASH: The Calabash type. </li> <li> INSTRUMENTATION: The Instrumentation type. </li> <li> UIAUTOMATION: The uiautomation type. </li> <li> UIAUTOMATOR: The uiautomator type. </li> <li> XCTEST: The Xcode test type. </li> <li> XCTEST_UI: The Xcode UI test type. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The suite's name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The suite's ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>skipped</i> The number of skipped entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>stopped</i> The number of stopped entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>errored</i> The number of errored entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>warned</i> The number of warned entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>failed</i> The number of failed entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>passed</i> The number of passed entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>total</i> The total number of entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>unmetered</i> When specified, represents only the sum of unmetered minutes used by the resource to run tests.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>metered</i> When specified, represents only the sum of metered minutes used by the resource to run tests.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>total</i> When specified, represents the total minutes used by the resource to run tests.<br/>        """
        pass

    def get_test(self, arn: str) -> GetTestResult:
        """Gets information about a test.

        :param self:
        :param arn:  The test's ARN.<br/>
        :return: test:  A test condition that is evaluated.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deviceMinutes</i> Represents the total (metered or unmetered) minutes used by the test.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>deviceMinutes</b>: Represents the total (metered or unmetered) minutes used by the test.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>message</i> A message about the test's result.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>counters</i> The test's result counters.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>counters</b>: The test's result counters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stopped</i> The test's stop time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>started</i> The test's start time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>result</i> The test's result. Allowed values include: <ul> <li> PENDING: A pending condition. </li> <li> PASSED: A passing condition. </li> <li> WARNED: A warning condition. </li> <li> FAILED: A failed condition. </li> <li> SKIPPED: A skipped condition. </li> <li> ERRORED: An error condition. </li> <li> STOPPED: A stopped condition. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The test's status. Allowed values include: <ul> <li> PENDING: A pending status. </li> <li> PENDING_CONCURRENCY: A pending concurrency status. </li> <li> PENDING_DEVICE: A pending device status. </li> <li> PROCESSING: A processing status. </li> <li> SCHEDULING: A scheduling status. </li> <li> PREPARING: A preparing status. </li> <li> RUNNING: A running status. </li> <li> COMPLETED: A completed status. </li> <li> STOPPING: A stopping status. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>created</i> When the test was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The test's type. Must be one of the following values: <ul> <li> BUILTIN_FUZZ: The built-in fuzz type. </li> <li> BUILTIN_EXPLORER: For Android, an app explorer that will traverse an Android app, interacting with it and capturing screenshots at the same time. </li> <li> APPIUM_JAVA_JUNIT: The Appium Java JUnit type. </li> <li> APPIUM_JAVA_TESTNG: The Appium Java TestNG type. </li> <li> APPIUM_PYTHON: The Appium Python type. </li> <li> APPIUM_NODE: The Appium Node.js type. </li> <li> APPIUM_RUBY: The Appium Ruby type. </li> <li> APPIUM_WEB_JAVA_JUNIT: The Appium Java JUnit type for web apps. </li> <li> APPIUM_WEB_JAVA_TESTNG: The Appium Java TestNG type for web apps. </li> <li> APPIUM_WEB_PYTHON: The Appium Python type for web apps. </li> <li> APPIUM_WEB_NODE: The Appium Node.js type for web apps. </li> <li> APPIUM_WEB_RUBY: The Appium Ruby type for web apps. </li> <li> CALABASH: The Calabash type. </li> <li> INSTRUMENTATION: The Instrumentation type. </li> <li> UIAUTOMATION: The uiautomation type. </li> <li> UIAUTOMATOR: The uiautomator type. </li> <li> XCTEST: The Xcode test type. </li> <li> XCTEST_UI: The Xcode UI test type. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The test's name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The test's ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>skipped</i> The number of skipped entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>stopped</i> The number of stopped entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>errored</i> The number of errored entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>warned</i> The number of warned entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>failed</i> The number of failed entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>passed</i> The number of passed entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>total</i> The total number of entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>unmetered</i> When specified, represents only the sum of unmetered minutes used by the resource to run tests.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>metered</i> When specified, represents only the sum of metered minutes used by the resource to run tests.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>total</i> When specified, represents the total minutes used by the resource to run tests.<br/>        """
        pass

    def get_upload(self, arn: str) -> GetUploadResult:
        """Gets information about an upload.

        :param self:
        :param arn:  The upload's ARN.<br/>
        :return: upload:  An app or a set of one or more tests to upload or that have been uploaded.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>category</i> The upload's category. Allowed values include: <ul> <li> CURATED: An upload managed by AWS Device Farm. </li> <li> PRIVATE: An upload managed by the AWS Device Farm customer. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>message</i> A message about the upload's result.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>contentType</i> The upload's content type (for example, "application/octet-stream").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>metadata</i> The upload's metadata. For example, for Android, this contains information that is parsed from the manifest and is displayed in the AWS Device Farm console after the associated app is uploaded.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>url</i> The pre-signed Amazon S3 URL that was used to store a file through a corresponding PUT request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The upload's status. Must be one of the following values: <ul> <li> FAILED: A failed status. </li> <li> INITIALIZED: An initialized status. </li> <li> PROCESSING: A processing status. </li> <li> SUCCEEDED: A succeeded status. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The upload's type. Must be one of the following values: <ul> <li> ANDROID_APP: An Android upload. </li> <li> IOS_APP: An iOS upload. </li> <li> WEB_APP: A web application upload. </li> <li> EXTERNAL_DATA: An external data upload. </li> <li> APPIUM_JAVA_JUNIT_TEST_PACKAGE: An Appium Java JUnit test package upload. </li> <li> APPIUM_JAVA_TESTNG_TEST_PACKAGE: An Appium Java TestNG test package upload. </li> <li> APPIUM_PYTHON_TEST_PACKAGE: An Appium Python test package upload. </li> <li> APPIUM_NODE_TEST_PACKAGE: An Appium Node.js test package upload. </li> <li> APPIUM_RUBY_TEST_PACKAGE: An Appium Ruby test package upload. </li> <li> APPIUM_WEB_JAVA_JUNIT_TEST_PACKAGE: An Appium Java JUnit test package upload for web apps. </li> <li> APPIUM_WEB_JAVA_TESTNG_TEST_PACKAGE: An Appium Java TestNG test package upload for web apps. </li> <li> APPIUM_WEB_PYTHON_TEST_PACKAGE: An Appium Python test package upload for web apps. </li> <li> APPIUM_WEB_NODE_TEST_PACKAGE: An Appium Node.js test package upload for web apps. </li> <li> APPIUM_WEB_RUBY_TEST_PACKAGE: An Appium Ruby test package upload for web apps. </li> <li> CALABASH_TEST_PACKAGE: A Calabash test package upload. </li> <li> INSTRUMENTATION_TEST_PACKAGE: An instrumentation upload. </li> <li> UIAUTOMATION_TEST_PACKAGE: A uiautomation test package upload. </li> <li> UIAUTOMATOR_TEST_PACKAGE: A uiautomator test package upload. </li> <li> XCTEST_TEST_PACKAGE: An Xcode test package upload. </li> <li> XCTEST_UI_TEST_PACKAGE: An Xcode UI test package upload. </li> <li> APPIUM_JAVA_JUNIT_TEST_SPEC: An Appium Java JUnit test spec upload. </li> <li> APPIUM_JAVA_TESTNG_TEST_SPEC: An Appium Java TestNG test spec upload. </li> <li> APPIUM_PYTHON_TEST_SPEC: An Appium Python test spec upload. </li> <li> APPIUM_NODE_TEST_SPEC: An Appium Node.js test spec upload. </li> <li> APPIUM_RUBY_TEST_SPEC: An Appium Ruby test spec upload. </li> <li> APPIUM_WEB_JAVA_JUNIT_TEST_SPEC: An Appium Java JUnit test spec upload for a web app. </li> <li> APPIUM_WEB_JAVA_TESTNG_TEST_SPEC: An Appium Java TestNG test spec upload for a web app. </li> <li> APPIUM_WEB_PYTHON_TEST_SPEC: An Appium Python test spec upload for a web app. </li> <li> APPIUM_WEB_NODE_TEST_SPEC: An Appium Node.js test spec upload for a web app. </li> <li> APPIUM_WEB_RUBY_TEST_SPEC: An Appium Ruby test spec upload for a web app. </li> <li> INSTRUMENTATION_TEST_SPEC: An instrumentation test spec upload. </li> <li> XCTEST_UI_TEST_SPEC: An Xcode UI test spec upload. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>created</i> When the upload was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The upload's file name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The upload's ARN.<br/>        """
        pass

    def get_vpce_configuration(self, arn: str) -> GetVPCEConfigurationResult:
        """Returns information about the configuration settings for your Amazon Virtual Private Cloud (VPC) endpoint.

        :param self:
        :param arn:  The Amazon Resource Name (ARN) of the VPC endpoint configuration you want to describe.<br/>
        :return: vpceConfiguration:  An object containing information about your VPC endpoint configuration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vpceConfigurationDescription</i> An optional description, providing more details about your VPC endpoint configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceDnsName</i> The DNS name that maps to the private IP address of the service you want to access.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vpceServiceName</i> The name of the VPC endpoint service running inside your AWS account that you want Device Farm to test.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vpceConfigurationName</i> The friendly name you give to your VPC endpoint configuration, to manage your configurations more easily.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the VPC endpoint configuration.<br/>        """
        pass

    def install_to_remote_access_session(self, appArn: str, remoteAccessSessionArn: str) -> InstallToRemoteAccessSessionResult:
        """Installs an application to the device in a remote access session. For Android applications, the file must be in .apk format. For iOS applications, the file must be in .ipa format.

        :param self:
        :param appArn:  The Amazon Resource Name (ARN) of the app about which you are requesting information.<br/>remoteAccessSessionArn:  The Amazon Resource Name (ARN) of the remote access session about which you are requesting information.<br/>
        :return: appUpload:  An app to upload or that has been uploaded.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>category</i> The upload's category. Allowed values include: <ul> <li> CURATED: An upload managed by AWS Device Farm. </li> <li> PRIVATE: An upload managed by the AWS Device Farm customer. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>message</i> A message about the upload's result.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>contentType</i> The upload's content type (for example, "application/octet-stream").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>metadata</i> The upload's metadata. For example, for Android, this contains information that is parsed from the manifest and is displayed in the AWS Device Farm console after the associated app is uploaded.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>url</i> The pre-signed Amazon S3 URL that was used to store a file through a corresponding PUT request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The upload's status. Must be one of the following values: <ul> <li> FAILED: A failed status. </li> <li> INITIALIZED: An initialized status. </li> <li> PROCESSING: A processing status. </li> <li> SUCCEEDED: A succeeded status. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The upload's type. Must be one of the following values: <ul> <li> ANDROID_APP: An Android upload. </li> <li> IOS_APP: An iOS upload. </li> <li> WEB_APP: A web application upload. </li> <li> EXTERNAL_DATA: An external data upload. </li> <li> APPIUM_JAVA_JUNIT_TEST_PACKAGE: An Appium Java JUnit test package upload. </li> <li> APPIUM_JAVA_TESTNG_TEST_PACKAGE: An Appium Java TestNG test package upload. </li> <li> APPIUM_PYTHON_TEST_PACKAGE: An Appium Python test package upload. </li> <li> APPIUM_NODE_TEST_PACKAGE: An Appium Node.js test package upload. </li> <li> APPIUM_RUBY_TEST_PACKAGE: An Appium Ruby test package upload. </li> <li> APPIUM_WEB_JAVA_JUNIT_TEST_PACKAGE: An Appium Java JUnit test package upload for web apps. </li> <li> APPIUM_WEB_JAVA_TESTNG_TEST_PACKAGE: An Appium Java TestNG test package upload for web apps. </li> <li> APPIUM_WEB_PYTHON_TEST_PACKAGE: An Appium Python test package upload for web apps. </li> <li> APPIUM_WEB_NODE_TEST_PACKAGE: An Appium Node.js test package upload for web apps. </li> <li> APPIUM_WEB_RUBY_TEST_PACKAGE: An Appium Ruby test package upload for web apps. </li> <li> CALABASH_TEST_PACKAGE: A Calabash test package upload. </li> <li> INSTRUMENTATION_TEST_PACKAGE: An instrumentation upload. </li> <li> UIAUTOMATION_TEST_PACKAGE: A uiautomation test package upload. </li> <li> UIAUTOMATOR_TEST_PACKAGE: A uiautomator test package upload. </li> <li> XCTEST_TEST_PACKAGE: An Xcode test package upload. </li> <li> XCTEST_UI_TEST_PACKAGE: An Xcode UI test package upload. </li> <li> APPIUM_JAVA_JUNIT_TEST_SPEC: An Appium Java JUnit test spec upload. </li> <li> APPIUM_JAVA_TESTNG_TEST_SPEC: An Appium Java TestNG test spec upload. </li> <li> APPIUM_PYTHON_TEST_SPEC: An Appium Python test spec upload. </li> <li> APPIUM_NODE_TEST_SPEC: An Appium Node.js test spec upload. </li> <li> APPIUM_RUBY_TEST_SPEC: An Appium Ruby test spec upload. </li> <li> APPIUM_WEB_JAVA_JUNIT_TEST_SPEC: An Appium Java JUnit test spec upload for a web app. </li> <li> APPIUM_WEB_JAVA_TESTNG_TEST_SPEC: An Appium Java TestNG test spec upload for a web app. </li> <li> APPIUM_WEB_PYTHON_TEST_SPEC: An Appium Python test spec upload for a web app. </li> <li> APPIUM_WEB_NODE_TEST_SPEC: An Appium Node.js test spec upload for a web app. </li> <li> APPIUM_WEB_RUBY_TEST_SPEC: An Appium Ruby test spec upload for a web app. </li> <li> INSTRUMENTATION_TEST_SPEC: An instrumentation test spec upload. </li> <li> XCTEST_UI_TEST_SPEC: An Xcode UI test spec upload. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>created</i> When the upload was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The upload's file name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The upload's ARN.<br/>        """
        pass

    def list_artifacts(self, type: str, arn: str, nextToken: str=None) -> ListArtifactsResult:
        """Gets information about artifacts.

        :param self:
        :param nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>type:  The artifacts' type. Allowed values include: <ul> <li> FILE: The artifacts are files. </li> <li> LOG: The artifacts are logs. </li> <li> SCREENSHOT: The artifacts are screenshots. </li> </ul><br/>arn:  The Run, Job, Suite, or Test ARN.<br/>
        :return: nextToken:  If the number of items that are returned is significantly large, this is an identifier that is also returned, which can be used in a subsequent call to this operation to return the next set of items in the list.<br/>artifacts:  Information about the artifacts.<br/>        """
        pass

    def list_device_instances(self, maxResults: int=None, nextToken: str=None) -> ListDeviceInstancesResult:
        """Returns information about the private device instances associated with one or more AWS accounts.

        :param self:
        :param nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>maxResults:  An integer specifying the maximum number of items you want to return in the API response.<br/>
        :return: nextToken:  An identifier that can be used in the next call to this operation to return the next set of items in the list.<br/>deviceInstances:  An object containing information about your device instances.<br/>        """
        pass

    def list_device_pools(self, arn: str, type: str=None, nextToken: str=None) -> ListDevicePoolsResult:
        """Gets information about device pools.

        :param self:
        :param nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>type:  The device pools' type. Allowed values include: <ul> <li> CURATED: A device pool that is created and managed by AWS Device Farm. </li> <li> PRIVATE: A device pool that is created and managed by the device pool developer. </li> </ul><br/>arn:  The project ARN.<br/>
        :return: nextToken:  If the number of items that are returned is significantly large, this is an identifier that is also returned, which can be used in a subsequent call to this operation to return the next set of items in the list.<br/>devicePools:  Information about the device pools.<br/>        """
        pass

    def list_devices(self, arn: str=None, nextToken: str=None, filters: List[DeviceFilter]=None) -> ListDevicesResult:
        """Gets information about unique device types.

        :param self:
        :param filters:  Used to select a set of devices. A filter is made up of an attribute, an operator, and one or more values. <ul> <li> Attribute: The aspect of a device such as platform or model used as the selection criteria in a device filter. Allowed values include: <ul> <li> ARN: The Amazon Resource Name (ARN) of the device. For example, "arn:aws:devicefarm:us-west-2::device:12345Example". </li> <li> PLATFORM: The device platform. Valid values are "ANDROID" or "IOS". </li> <li> OS_VERSION: The operating system version. For example, "10.3.2". </li> <li> MODEL: The device model. For example, "iPad 5th Gen". </li> <li> AVAILABILITY: The current availability of the device. Valid values are "AVAILABLE", "HIGHLY_AVAILABLE", "BUSY", or "TEMPORARY_NOT_AVAILABLE". </li> <li> FORM_FACTOR: The device form factor. Valid values are "PHONE" or "TABLET". </li> <li> MANUFACTURER: The device manufacturer. For example, "Apple". </li> <li> REMOTE_ACCESS_ENABLED: Whether the device is enabled for remote access. Valid values are "TRUE" or "FALSE". </li> <li> REMOTE_DEBUG_ENABLED: Whether the device is enabled for remote debugging. Valid values are "TRUE" or "FALSE". </li> <li> INSTANCE_ARN: The Amazon Resource Name (ARN) of the device instance. </li> <li> INSTANCE_LABELS: The label of the device instance. </li> <li> FLEET_TYPE: The fleet type. Valid values are "PUBLIC" or "PRIVATE". </li> </ul> </li> <li> Operator: The filter operator. <ul> <li> The EQUALS operator is available for every attribute except INSTANCE_LABELS. </li> <li> The CONTAINS operator is available for the INSTANCE_LABELS and MODEL attributes. </li> <li> The IN and NOT_IN operators are available for the ARN, OS_VERSION, MODEL, MANUFACTURER, and INSTANCE_ARN attributes. </li> <li> The LESS_THAN, GREATER_THAN, LESS_THAN_OR_EQUALS, and GREATER_THAN_OR_EQUALS operators are also available for the OS_VERSION attribute. </li> </ul> </li> <li> Values: An array of one or more filter values. <ul> <li> The IN and NOT_IN operators take a values array that has one or more elements. </li> <li> The other operators require an array with a single element. </li> <li> In a request, the AVAILABILITY attribute takes "AVAILABLE", "HIGHLY_AVAILABLE", "BUSY", or "TEMPORARY_NOT_AVAILABLE" as values. </li> </ul> </li> </ul><br/>nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>arn:  The Amazon Resource Name (ARN) of the project.<br/>
        :return: nextToken:  If the number of items that are returned is significantly large, this is an identifier that is also returned, which can be used in a subsequent call to this operation to return the next set of items in the list.<br/>devices:  Information about the devices.<br/>        """
        pass

    def list_instance_profiles(self, maxResults: int=None, nextToken: str=None) -> ListInstanceProfilesResult:
        """Returns information about all the instance profiles in an AWS account.

        :param self:
        :param nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>maxResults:  An integer specifying the maximum number of items you want to return in the API response.<br/>
        :return: nextToken:  An identifier that can be used in the next call to this operation to return the next set of items in the list.<br/>instanceProfiles:  An object containing information about your instance profiles.<br/>        """
        pass

    def list_jobs(self, arn: str, nextToken: str=None) -> ListJobsResult:
        """Gets information about jobs for a given test run.

        :param self:
        :param nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>arn:  The run's Amazon Resource Name (ARN).<br/>
        :return: nextToken:  If the number of items that are returned is significantly large, this is an identifier that is also returned, which can be used in a subsequent call to this operation to return the next set of items in the list.<br/>jobs:  Information about the jobs.<br/>        """
        pass

    def list_network_profiles(self, arn: str, type: str=None, nextToken: str=None) -> ListNetworkProfilesResult:
        """Returns the list of available network profiles.

        :param self:
        :param nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>type:  The type of network profile you wish to return information about. Valid values are listed below.<br/>arn:  The Amazon Resource Name (ARN) of the project for which you want to list network profiles.<br/>
        :return: nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>networkProfiles:  A list of the available network profiles.<br/>        """
        pass

    def list_offering_promotions(self, nextToken: str=None) -> ListOfferingPromotionsResult:
        """Returns a list of offering promotions. Each offering promotion record contains the ID and description of the promotion. The API returns a <code>NotEligible</code> error if the caller is not permitted to invoke the operation. Contact <a href="mailto:aws-devicefarm-support@amazon.com">aws-devicefarm-support@amazon.com</a> if you believe that you should be able to invoke this operation.

        :param self:
        :param nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>
        :return: nextToken:  An identifier to be used in the next call to this operation, to return the next set of items in the list.<br/>offeringPromotions:  Information about the offering promotions.<br/>        """
        pass

    def list_offering_transactions(self, nextToken: str=None) -> ListOfferingTransactionsResult:
        """Returns a list of all historical purchases, renewals, and system renewal transactions for an AWS account. The list is paginated and ordered by a descending timestamp (most recent transactions are first). The API returns a <code>NotEligible</code> error if the user is not permitted to invoke the operation. Please contact <a href="mailto:aws-devicefarm-support@amazon.com">aws-devicefarm-support@amazon.com</a> if you believe that you should be able to invoke this operation.

        :param self:
        :param nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>
        :return: nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>offeringTransactions:  The audit log of subscriptions you have purchased and modified through AWS Device Farm.<br/>        """
        pass

    def list_offerings(self, nextToken: str=None) -> ListOfferingsResult:
        """Returns a list of products or offerings that the user can manage through the API. Each offering record indicates the recurring price per unit and the frequency for that offering. The API returns a <code>NotEligible</code> error if the user is not permitted to invoke the operation. Please contact <a href="mailto:aws-devicefarm-support@amazon.com">aws-devicefarm-support@amazon.com</a> if you believe that you should be able to invoke this operation.

        :param self:
        :param nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>
        :return: nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>offerings:  A value representing the list offering results.<br/>        """
        pass

    def list_projects(self, arn: str=None, nextToken: str=None) -> ListProjectsResult:
        """Gets information about projects.

        :param self:
        :param nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>arn:  Optional. If no Amazon Resource Name (ARN) is specified, then AWS Device Farm returns a list of all projects for the AWS account. You can also specify a project ARN.<br/>
        :return: nextToken:  If the number of items that are returned is significantly large, this is an identifier that is also returned, which can be used in a subsequent call to this operation to return the next set of items in the list.<br/>projects:  Information about the projects.<br/>        """
        pass

    def list_remote_access_sessions(self, arn: str, nextToken: str=None) -> ListRemoteAccessSessionsResult:
        """Returns a list of all currently running remote access sessions.

        :param self:
        :param nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>arn:  The Amazon Resource Name (ARN) of the remote access session about which you are requesting information.<br/>
        :return: nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>remoteAccessSessions:  A container representing the metadata from the service about each remote access session you are requesting.<br/>        """
        pass

    def list_runs(self, arn: str, nextToken: str=None) -> ListRunsResult:
        """Gets information about runs, given an AWS Device Farm project ARN.

        :param self:
        :param nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>arn:  The Amazon Resource Name (ARN) of the project for which you want to list runs.<br/>
        :return: nextToken:  If the number of items that are returned is significantly large, this is an identifier that is also returned, which can be used in a subsequent call to this operation to return the next set of items in the list.<br/>runs:  Information about the runs.<br/>        """
        pass

    def list_samples(self, arn: str, nextToken: str=None) -> ListSamplesResult:
        """Gets information about samples, given an AWS Device Farm job ARN.

        :param self:
        :param nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>arn:  The Amazon Resource Name (ARN) of the job used to list samples.<br/>
        :return: nextToken:  If the number of items that are returned is significantly large, this is an identifier that is also returned, which can be used in a subsequent call to this operation to return the next set of items in the list.<br/>samples:  Information about the samples.<br/>        """
        pass

    def list_suites(self, arn: str, nextToken: str=None) -> ListSuitesResult:
        """Gets information about test suites for a given job.

        :param self:
        :param nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>arn:  The job's Amazon Resource Name (ARN).<br/>
        :return: nextToken:  If the number of items that are returned is significantly large, this is an identifier that is also returned, which can be used in a subsequent call to this operation to return the next set of items in the list.<br/>suites:  Information about the suites.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceARN: str) -> ListTagsForResourceResponse:
        """List the tags for an AWS Device Farm resource.

        :param self:
        :param ResourceARN:  The Amazon Resource Name (ARN) of the resource(s) for which to list tags. You can associate tags with the following Device Farm resources: <code>PROJECT</code>, <code>RUN</code>, <code>NETWORK_PROFILE</code>, <code>INSTANCE_PROFILE</code>, <code>DEVICE_INSTANCE</code>, <code>SESSION</code>, <code>DEVICE_POOL</code>, <code>DEVICE</code>, and <code>VPCE_CONFIGURATION</code>.<br/>
        :return: Tags:  The tags to add to the resource. A tag is an array of key-value pairs. Tag keys can have a maximum character length of 128 characters, and tag values can have a maximum length of 256 characters.<br/>        """
        pass

    def list_tests(self, arn: str, nextToken: str=None) -> ListTestsResult:
        """Gets information about tests in a given test suite.

        :param self:
        :param nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>arn:  The test suite's Amazon Resource Name (ARN).<br/>
        :return: nextToken:  If the number of items that are returned is significantly large, this is an identifier that is also returned, which can be used in a subsequent call to this operation to return the next set of items in the list.<br/>tests:  Information about the tests.<br/>        """
        pass

    def list_unique_problems(self, arn: str, nextToken: str=None) -> ListUniqueProblemsResult:
        """Gets information about unique problems.

        :param self:
        :param nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>arn:  The unique problems' ARNs.<br/>
        :return: nextToken:  If the number of items that are returned is significantly large, this is an identifier that is also returned, which can be used in a subsequent call to this operation to return the next set of items in the list.<br/>uniqueProblems:  Information about the unique problems. Allowed values include: <ul> <li> PENDING: A pending condition. </li> <li> PASSED: A passing condition. </li> <li> WARNED: A warning condition. </li> <li> FAILED: A failed condition. </li> <li> SKIPPED: A skipped condition. </li> <li> ERRORED: An error condition. </li> <li> STOPPED: A stopped condition. </li> </ul><br/>        """
        pass

    def list_uploads(self, arn: str, type: str=None, nextToken: str=None) -> ListUploadsResult:
        """Gets information about uploads, given an AWS Device Farm project ARN.

        :param self:
        :param nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>type:  The type of upload. Must be one of the following values: <ul> <li> ANDROID_APP: An Android upload. </li> <li> IOS_APP: An iOS upload. </li> <li> WEB_APP: A web application upload. </li> <li> EXTERNAL_DATA: An external data upload. </li> <li> APPIUM_JAVA_JUNIT_TEST_PACKAGE: An Appium Java JUnit test package upload. </li> <li> APPIUM_JAVA_TESTNG_TEST_PACKAGE: An Appium Java TestNG test package upload. </li> <li> APPIUM_PYTHON_TEST_PACKAGE: An Appium Python test package upload. </li> <li> APPIUM_NODE_TEST_PACKAGE: An Appium Node.js test package upload. </li> <li> APPIUM_RUBY_TEST_PACKAGE: An Appium Ruby test package upload. </li> <li> APPIUM_WEB_JAVA_JUNIT_TEST_PACKAGE: An Appium Java JUnit test package upload for a web app. </li> <li> APPIUM_WEB_JAVA_TESTNG_TEST_PACKAGE: An Appium Java TestNG test package upload for a web app. </li> <li> APPIUM_WEB_PYTHON_TEST_PACKAGE: An Appium Python test package upload for a web app. </li> <li> APPIUM_WEB_NODE_TEST_PACKAGE: An Appium Node.js test package upload for a web app. </li> <li> APPIUM_WEB_RUBY_TEST_PACKAGE: An Appium Ruby test package upload for a web app. </li> <li> CALABASH_TEST_PACKAGE: A Calabash test package upload. </li> <li> INSTRUMENTATION_TEST_PACKAGE: An instrumentation upload. </li> <li> UIAUTOMATION_TEST_PACKAGE: A uiautomation test package upload. </li> <li> UIAUTOMATOR_TEST_PACKAGE: A uiautomator test package upload. </li> <li> XCTEST_TEST_PACKAGE: An Xcode test package upload. </li> <li> XCTEST_UI_TEST_PACKAGE: An Xcode UI test package upload. </li> <li> APPIUM_JAVA_JUNIT_TEST_SPEC: An Appium Java JUnit test spec upload. </li> <li> APPIUM_JAVA_TESTNG_TEST_SPEC: An Appium Java TestNG test spec upload. </li> <li> APPIUM_PYTHON_TEST_SPEC: An Appium Python test spec upload. </li> <li> APPIUM_NODE_TEST_SPEC: An Appium Node.js test spec upload. </li> <li>  APPIUM_RUBY_TEST_SPEC: An Appium Ruby test spec upload. </li> <li> APPIUM_WEB_JAVA_JUNIT_TEST_SPEC: An Appium Java JUnit test spec upload for a web app. </li> <li> APPIUM_WEB_JAVA_TESTNG_TEST_SPEC: An Appium Java TestNG test spec upload for a web app. </li> <li> APPIUM_WEB_PYTHON_TEST_SPEC: An Appium Python test spec upload for a web app. </li> <li> APPIUM_WEB_NODE_TEST_SPEC: An Appium Node.js test spec upload for a web app. </li> <li> APPIUM_WEB_RUBY_TEST_SPEC: An Appium Ruby test spec upload for a web app. </li> <li> INSTRUMENTATION_TEST_SPEC: An instrumentation test spec upload. </li> <li> XCTEST_UI_TEST_SPEC: An Xcode UI test spec upload. </li> </ul><br/>arn:  The Amazon Resource Name (ARN) of the project for which you want to list uploads.<br/>
        :return: nextToken:  If the number of items that are returned is significantly large, this is an identifier that is also returned, which can be used in a subsequent call to this operation to return the next set of items in the list.<br/>uploads:  Information about the uploads.<br/>        """
        pass

    def list_vpce_configurations(self, maxResults: int=None, nextToken: str=None) -> ListVPCEConfigurationsResult:
        """Returns information about all Amazon Virtual Private Cloud (VPC) endpoint configurations in the AWS account.

        :param self:
        :param nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>maxResults:  An integer specifying the maximum number of items you want to return in the API response.<br/>
        :return: nextToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>vpceConfigurations:  An array of <code>VPCEConfiguration</code> objects containing information about your VPC endpoint configuration.<br/>        """
        pass

    def purchase_offering(self, offeringId: str=None, quantity: int=None, offeringPromotionId: str=None) -> PurchaseOfferingResult:
        """Immediately purchases offerings for an AWS account. Offerings renew with the latest total purchased quantity for an offering, unless the renewal was overridden. The API returns a <code>NotEligible</code> error if the user is not permitted to invoke the operation. Please contact <a href="mailto:aws-devicefarm-support@amazon.com">aws-devicefarm-support@amazon.com</a> if you believe that you should be able to invoke this operation.

        :param self:
        :param offeringPromotionId:  The ID of the offering promotion to be applied to the purchase.<br/>quantity:  The number of device slots you wish to purchase in an offering request.<br/>offeringId:  The ID of the offering.<br/>
        :return: offeringTransaction:  Represents the offering transaction for the purchase result.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cost</i> The cost of an offering transaction.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>cost</b>: The cost of an offering transaction.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdOn</i> The date on which an offering transaction was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>offeringPromotionId</i> The ID that corresponds to a device offering promotion.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>transactionId</i> The transaction ID of the offering transaction.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>offeringStatus</i> The status of an offering transaction.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>offeringStatus</b>: The status of an offering transaction.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type specified for the offering status.<br/>Represents the metadata of an offering status.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>offering</b>: Represents the metadata of an offering status.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>currencyCode</i> The currency code of a monetary amount. For example, <code>USD</code> means "U.S. dollars."<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>amount</i> The numerical amount of an offering or transaction.<br/>        """
        pass

    def renew_offering(self, offeringId: str=None, quantity: int=None) -> RenewOfferingResult:
        """Explicitly sets the quantity of devices to renew for an offering, starting from the <code>effectiveDate</code> of the next period. The API returns a <code>NotEligible</code> error if the user is not permitted to invoke the operation. Please contact <a href="mailto:aws-devicefarm-support@amazon.com">aws-devicefarm-support@amazon.com</a> if you believe that you should be able to invoke this operation.

        :param self:
        :param quantity:  The quantity requested in an offering renewal.<br/>offeringId:  The ID of a request to renew an offering.<br/>
        :return: offeringTransaction:  Represents the status of the offering transaction for the renewal.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cost</i> The cost of an offering transaction.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>cost</b>: The cost of an offering transaction.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdOn</i> The date on which an offering transaction was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>offeringPromotionId</i> The ID that corresponds to a device offering promotion.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>transactionId</i> The transaction ID of the offering transaction.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>offeringStatus</i> The status of an offering transaction.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>offeringStatus</b>: The status of an offering transaction.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type specified for the offering status.<br/>Represents the metadata of an offering status.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>offering</b>: Represents the metadata of an offering status.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>currencyCode</i> The currency code of a monetary amount. For example, <code>USD</code> means "U.S. dollars."<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>amount</i> The numerical amount of an offering or transaction.<br/>        """
        pass

    def schedule_run(self, test: ScheduleRunTest, projectArn: str, appArn: str=None, devicePoolArn: str=None, deviceSelectionConfiguration: DeviceSelectionConfiguration=None, name: str=None, configuration: ScheduleRunConfiguration=None, executionConfiguration: ExecutionConfiguration=None) -> ScheduleRunResult:
        """Schedules a run.

        :param self:
        :param executionConfiguration:  Specifies configuration information about a test run, such as the execution timeout (in minutes).:<br/>configuration:  Information about the settings for the run to be scheduled.:<br/>test:  Information about the test for the run to be scheduled.:<br/>name:  The name for the run to be scheduled.<br/>deviceSelectionConfiguration:  The filter criteria used to dynamically select a set of devices for a test run, as well as the maximum number of devices to be included in the run. Either <b> <code>devicePoolArn</code> </b> or <b> <code>deviceSelectionConfiguration</code> </b> is required in a request.:<br/>devicePoolArn:  The ARN of the device pool for the run to be scheduled.<br/>appArn:  The ARN of the app to schedule a run.<br/>projectArn:  The ARN of the project for the run to be scheduled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>maxDevices</i> The maximum number of devices to be included in a test run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>filters</i> Used to dynamically select a set of devices for a test run. A filter is made up of an attribute, an operator, and one or more values. <ul> <li>  <b>Attribute</b>  The aspect of a device such as platform or model used as the selection criteria in a device filter. Allowed values include: <ul> <li> ARN: The Amazon Resource Name (ARN) of the device. For example, "arn:aws:devicefarm:us-west-2::device:12345Example". </li> <li> PLATFORM: The device platform. Valid values are "ANDROID" or "IOS". </li> <li> OS_VERSION: The operating system version. For example, "10.3.2". </li> <li> MODEL: The device model. For example, "iPad 5th Gen". </li> <li> AVAILABILITY: The current availability of the device. Valid values are "AVAILABLE", "HIGHLY_AVAILABLE", "BUSY", or "TEMPORARY_NOT_AVAILABLE". </li> <li> FORM_FACTOR: The device form factor. Valid values are "PHONE" or "TABLET". </li> <li> MANUFACTURER: The device manufacturer. For example, "Apple". </li> <li> REMOTE_ACCESS_ENABLED: Whether the device is enabled for remote access. Valid values are "TRUE" or "FALSE". </li> <li> REMOTE_DEBUG_ENABLED: Whether the device is enabled for remote debugging. Valid values are "TRUE" or "FALSE". </li> <li> INSTANCE_ARN: The Amazon Resource Name (ARN) of the device instance. </li> <li> INSTANCE_LABELS: The label of the device instance. </li> <li> FLEET_TYPE: The fleet type. Valid values are "PUBLIC" or "PRIVATE". </li> </ul> </li> <li>  <b>Operator</b>  The filter operator. <ul> <li> The EQUALS operator is available for every attribute except INSTANCE_LABELS. </li> <li> The CONTAINS operator is available for the INSTANCE_LABELS and MODEL attributes. </li> <li> The IN and NOT_IN operators are available for the ARN, OS_VERSION, MODEL, MANUFACTURER, and INSTANCE_ARN attributes. </li> <li> The LESS_THAN, GREATER_THAN, LESS_THAN_OR_EQUALS, and GREATER_THAN_OR_EQUALS operators are also available for the OS_VERSION attribute. </li> </ul> </li> <li>  <b>Values</b>  An array of one or more filter values. <p class="title"> <b>Operator Values</b>  <ul> <li> The IN and NOT_IN operators can take a values array that has more than one element. </li> <li> The other operators require an array with a single element. </li> </ul> <p class="title"> <b>Attribute Values</b>  <ul> <li> The PLATFORM attribute can be set to "ANDROID" or "IOS". </li> <li> The AVAILABILITY attribute can be set to "AVAILABLE", "HIGHLY_AVAILABLE", "BUSY", or "TEMPORARY_NOT_AVAILABLE". </li> <li> The FORM_FACTOR attribute can be set to "PHONE" or "TABLET". </li> <li> The FLEET_TYPE attribute can be set to "PUBLIC" or "PRIVATE". </li> </ul> </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>parameters</i> The test's parameters, such as test framework parameters and fixture settings. Parameters are represented by name-value pairs of strings. For all tests: <ul> <li> app_performance_monitoring: Performance monitoring is enabled by default. Set this parameter to "false" to disable it. </li> </ul> For Calabash tests: <ul> <li> profile: A cucumber profile, for example, "my_profile_name". </li> <li> tags: You can limit execution to features or scenarios that have (or don't have) certain tags, for example, "@smoke" or "@smoke,~@wip". </li> </ul> For Appium tests (all types): <ul> <li> appium_version: The Appium version. Currently supported values are "1.6.5" (and higher), "latest", and "default". <ul> <li> “latest” will run the latest Appium version supported by Device Farm (1.9.1). </li> <li> For “default”, Device Farm will choose a compatible version of Appium for the device. The current behavior is to run 1.7.2 on Android devices and iOS 9 and earlier, 1.7.2 for iOS 10 and later. </li> <li> This behavior is subject to change. </li> </ul> </li> </ul> For Fuzz tests (Android only): <ul> <li> event_count: The number of events, between 1 and 10000, that the UI fuzz test should perform. </li> <li> throttle: The time, in ms, between 0 and 1000, that the UI fuzz test should wait between events. </li> <li> seed: A seed to use for randomizing the UI fuzz test. Using the same seed value between tests ensures identical event sequences. </li> </ul> For Explorer tests: <ul> <li> username: A username to use if the Explorer encounters a login form. If not supplied, no username will be inserted. </li> <li> password: A password to use if the Explorer encounters a login form. If not supplied, no password will be inserted. </li> </ul> For Instrumentation: <ul> <li> filter: A test filter string. Examples: <ul> <li> Running a single test case: "com.android.abc.Test1" </li> <li> Running a single test: "com.android.abc.Test1#smoke" </li> <li> Running multiple tests: "com.android.abc.Test1,com.android.abc.Test2" </li> </ul> </li> </ul> For XCTest and XCTestUI: <ul> <li> filter: A test filter string. Examples: <ul> <li> Running a single test class: "LoginTests" </li> <li> Running a multiple test classes: "LoginTests,SmokeTests" </li> <li> Running a single test: "LoginTests/testValid" </li> <li> Running multiple tests: "LoginTests/testValid,LoginTests/testInvalid" </li> </ul> </li> </ul> For UIAutomator: <ul> <li> filter: A test filter string. Examples: <ul> <li> Running a single test case: "com.android.abc.Test1" </li> <li> Running a single test: "com.android.abc.Test1#smoke" </li> <li> Running multiple tests: "com.android.abc.Test1,com.android.abc.Test2" </li> </ul> </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>filter</i> The test's filter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>testSpecArn</i> The ARN of the YAML-formatted test specification.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>testPackageArn</i> The ARN of the uploaded test that will be run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The test's type. Must be one of the following values: <ul> <li> BUILTIN_FUZZ: The built-in fuzz type. </li> <li> BUILTIN_EXPLORER: For Android, an app explorer that will traverse an Android app, interacting with it and capturing screenshots at the same time. </li> <li> APPIUM_JAVA_JUNIT: The Appium Java JUnit type. </li> <li> APPIUM_JAVA_TESTNG: The Appium Java TestNG type. </li> <li> APPIUM_PYTHON: The Appium Python type. </li> <li> APPIUM_NODE: The Appium Node.js type. </li> <li> APPIUM_RUBY: The Appium Ruby type. </li> <li> APPIUM_WEB_JAVA_JUNIT: The Appium Java JUnit type for web apps. </li> <li> APPIUM_WEB_JAVA_TESTNG: The Appium Java TestNG type for web apps. </li> <li> APPIUM_WEB_PYTHON: The Appium Python type for web apps. </li> <li> APPIUM_WEB_NODE: The Appium Node.js type for web apps. </li> <li> APPIUM_WEB_RUBY: The Appium Ruby type for web apps. </li> <li> CALABASH: The Calabash type. </li> <li> INSTRUMENTATION: The Instrumentation type. </li> <li> UIAUTOMATION: The uiautomation type. </li> <li> UIAUTOMATOR: The uiautomator type. </li> <li> XCTEST: The Xcode test type. </li> <li> XCTEST_UI: The Xcode UI test type. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>billingMethod</i> Specifies the billing method for a test run: <code>metered</code> or <code>unmetered</code>. If the parameter is not specified, the default value is <code>metered</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>auxiliaryApps</i> A list of auxiliary apps for the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>radios</i> Information about the radio states for the run.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>radios</b>: Information about the radio states for the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customerArtifactPaths</i> Input <code>CustomerArtifactPaths</code> object for the scheduled run configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>customerArtifactPaths</b>: Input <code>CustomerArtifactPaths</code> object for the scheduled run configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vpceConfigurationArns</i> An array of Amazon Resource Names (ARNs) for your VPC endpoint configurations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> Information about the location that is used for the run.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>location</b>: Information about the location that is used for the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>locale</i> Information about the locale that is used for the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>networkProfileArn</i> Reserved for internal use.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>extraDataPackageArn</i> The ARN of the extra data for the run. The extra data is a .zip file that AWS Device Farm will extract to external data for Android or the app's sandbox for iOS.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>longitude</i> The longitude.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>latitude</i> The latitude.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>deviceHostPaths</i> Comma-separated list of paths in the test execution environment where the artifacts generated by the customer's tests will be pulled from.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>androidPaths</i> Comma-separated list of paths on the Android device where the artifacts generated by the customer's tests will be pulled from.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>iosPaths</i> Comma-separated list of paths on the iOS device where the artifacts generated by the customer's tests will be pulled from.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>gps</i> True if GPS is enabled at the beginning of the test; otherwise, false.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>nfc</i> True if NFC is enabled at the beginning of the test; otherwise, false.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>bluetooth</i> True if Bluetooth is enabled at the beginning of the test; otherwise, false.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>wifi</i> True if Wi-Fi is enabled at the beginning of the test; otherwise, false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>skipAppResign</i> When set to <code>true</code>, for private devices, Device Farm will not sign your app again. For public devices, Device Farm always signs your apps again and this parameter has no effect. For more information about how Device Farm re-signs your app(s), see <a href="https://aws.amazon.com/device-farm/faq/">Do you modify my app?</a> in the <i>AWS Device Farm FAQs</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>videoCapture</i> Set to true to enable video capture; otherwise, set to false. The default is true.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>appPackagesCleanup</i> True if app package cleanup is enabled at the beginning of the test; otherwise, false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>accountsCleanup</i> True if account cleanup is enabled at the beginning of the test; otherwise, false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jobTimeoutMinutes</i> The number of minutes a test run will execute before it times out.<br/>
        :return: run:  Information about the scheduled run.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deviceSelectionResult</i> The results of a device filter used to select the devices for a test run.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>deviceSelectionResult</b>: The results of a device filter used to select the devices for a test run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>testSpecArn</i> The ARN of the YAML-formatted test specification for the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>skipAppResign</i> When set to <code>true</code>, for private devices, Device Farm will not sign your app again. For public devices, Device Farm always signs your apps again and this parameter has no effect. For more information about how Device Farm re-signs your app(s), see <a href="https://aws.amazon.com/device-farm/faq/">Do you modify my app?</a> in the <i>AWS Device Farm FAQs</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>webUrl</i> The Device Farm console URL for the recording of the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customerArtifactPaths</i> Output <code>CustomerArtifactPaths</code> object for the test run.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>customerArtifactPaths</b>: Output <code>CustomerArtifactPaths</code> object for the test run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> Information about the location that is used for the run.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>location</b>: Information about the location that is used for the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>radios</i> Information about the radio states for the run.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>radios</b>: Information about the radio states for the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>locale</i> Information about the locale that is used for the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>devicePoolArn</i> The ARN of the device pool for the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jobTimeoutMinutes</i> The number of minutes the job will execute before it times out.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>eventCount</i> For fuzz tests, this is the number of events, between 1 and 10000, that the UI fuzz test should perform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>appUpload</i> An app to upload or that has been uploaded.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>seed</i> For fuzz tests, this is a seed to use for randomizing the UI fuzz test. Using the same seed value between tests ensures identical event sequences.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resultCode</i> Supporting field for the result field. Set only if <code>result</code> is <code>SKIPPED</code>. <code>PARSING_FAILED</code> if the result is skipped because of test package parsing failure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>parsingResultUrl</i> Read-only URL for an object in S3 bucket where you can get the parsing results of the test package. If the test package doesn't parse, the reason why it doesn't parse appears in the file that this URL points to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>networkProfile</i> The network profile being used for a test run.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>networkProfile</b>: The network profile being used for a test run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deviceMinutes</i> Represents the total (metered or unmetered) minutes used by the test run.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>deviceMinutes</b>: Represents the total (metered or unmetered) minutes used by the test run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>billingMethod</i> Specifies the billing method for a test run: <code>metered</code> or <code>unmetered</code>. If the parameter is not specified, the default value is <code>metered</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>completedJobs</i> The total number of completed jobs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>totalJobs</i> The total number of jobs for the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>message</i> A message about the run's result.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>counters</i> The run's result counters.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>counters</b>: The run's result counters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stopped</i> The run's stop time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>started</i> The run's start time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>result</i> The run's result. Allowed values include: <ul> <li> PENDING: A pending condition. </li> <li> PASSED: A passing condition. </li> <li> WARNED: A warning condition. </li> <li> FAILED: A failed condition. </li> <li> SKIPPED: A skipped condition. </li> <li> ERRORED: An error condition. </li> <li> STOPPED: A stopped condition. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The run's status. Allowed values include: <ul> <li> PENDING: A pending status. </li> <li> PENDING_CONCURRENCY: A pending concurrency status. </li> <li> PENDING_DEVICE: A pending device status. </li> <li> PROCESSING: A processing status. </li> <li> SCHEDULING: A scheduling status. </li> <li> PREPARING: A preparing status. </li> <li> RUNNING: A running status. </li> <li> COMPLETED: A completed status. </li> <li> STOPPING: A stopping status. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>created</i> When the run was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>platform</i> The run's platform. Allowed values include: <ul> <li> ANDROID: The Android platform. </li> <li> IOS: The iOS platform. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The run's type. Must be one of the following values: <ul> <li> BUILTIN_FUZZ: The built-in fuzz type. </li> <li> BUILTIN_EXPLORER: For Android, an app explorer that will traverse an Android app, interacting with it and capturing screenshots at the same time. </li> <li> APPIUM_JAVA_JUNIT: The Appium Java JUnit type. </li> <li> APPIUM_JAVA_TESTNG: The Appium Java TestNG type. </li> <li> APPIUM_PYTHON: The Appium Python type. </li> <li> APPIUM_NODE: The Appium Node.js type. </li> <li> APPIUM_RUBY: The Appium Ruby type. </li> <li> APPIUM_WEB_JAVA_JUNIT: The Appium Java JUnit type for web apps. </li> <li> APPIUM_WEB_JAVA_TESTNG: The Appium Java TestNG type for web apps. </li> <li> APPIUM_WEB_PYTHON: The Appium Python type for web apps. </li> <li> APPIUM_WEB_NODE: The Appium Node.js type for web apps. </li> <li> APPIUM_WEB_RUBY: The Appium Ruby type for web apps. </li> <li> CALABASH: The Calabash type. </li> <li> INSTRUMENTATION: The Instrumentation type. </li> <li> UIAUTOMATION: The uiautomation type. </li> <li> UIAUTOMATOR: The uiautomator type. </li> <li> XCTEST: The Xcode test type. </li> <li> XCTEST_UI: The Xcode UI test type. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The run's name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The run's ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>skipped</i> The number of skipped entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>stopped</i> The number of stopped entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>errored</i> The number of errored entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>warned</i> The number of warned entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>failed</i> The number of failed entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>passed</i> The number of passed entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>total</i> The total number of entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>unmetered</i> When specified, represents only the sum of unmetered minutes used by the resource to run tests.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>metered</i> When specified, represents only the sum of metered minutes used by the resource to run tests.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>total</i> When specified, represents the total minutes used by the resource to run tests.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>downlinkLossPercent</i> Proportion of received packets that fail to arrive from 0 to 100 percent.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uplinkLossPercent</i> Proportion of transmitted packets that fail to arrive from 0 to 100 percent.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>downlinkJitterMs</i> Time variation in the delay of received packets in milliseconds as an integer from 0 to 2000.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uplinkJitterMs</i> Time variation in the delay of received packets in milliseconds as an integer from 0 to 2000.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>downlinkDelayMs</i> Delay time for all packets to destination in milliseconds as an integer from 0 to 2000.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uplinkDelayMs</i> Delay time for all packets to destination in milliseconds as an integer from 0 to 2000.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>downlinkBandwidthBits</i> The data throughput rate in bits per second, as an integer from 0 to 104857600.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uplinkBandwidthBits</i> The data throughput rate in bits per second, as an integer from 0 to 104857600.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of network profile. Valid values are listed below.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The description of the network profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the network profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the network profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>gps</i> True if GPS is enabled at the beginning of the test; otherwise, false.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>nfc</i> True if NFC is enabled at the beginning of the test; otherwise, false.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>bluetooth</i> True if Bluetooth is enabled at the beginning of the test; otherwise, false.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>wifi</i> True if Wi-Fi is enabled at the beginning of the test; otherwise, false.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>longitude</i> The longitude.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>latitude</i> The latitude.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>deviceHostPaths</i> Comma-separated list of paths in the test execution environment where the artifacts generated by the customer's tests will be pulled from.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>androidPaths</i> Comma-separated list of paths on the Android device where the artifacts generated by the customer's tests will be pulled from.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>iosPaths</i> Comma-separated list of paths on the iOS device where the artifacts generated by the customer's tests will be pulled from.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>maxDevices</i> The maximum number of devices to be selected by a device filter and included in a test run.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>matchedDevicesCount</i> The number of devices that matched the device filter selection criteria.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>filters</i> The filters in a device selection result.<br/>        """
        pass

    def stop_job(self, arn: str) -> StopJobResult:
        """Initiates a stop request for the current job. AWS Device Farm will immediately stop the job on the device where tests have not started executing, and you will not be billed for this device. On the device where tests have started executing, Setup Suite and Teardown Suite tests will run to completion before stopping execution on the device. You will be billed for Setup, Teardown, and any tests that were in progress or already completed.

        :param self:
        :param arn:  Represents the Amazon Resource Name (ARN) of the Device Farm job you wish to stop.<br/>
        :return: job:  The job that was stopped.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>videoCapture</i> This value is set to true if video capture is enabled; otherwise, it is set to false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>videoEndpoint</i> The endpoint for streaming device video.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deviceMinutes</i> Represents the total (metered or unmetered) minutes used by the job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>deviceMinutes</b>: Represents the total (metered or unmetered) minutes used by the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>instanceArn</i> The Amazon Resource Name (ARN) of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>device</i> The device (phone or tablet).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>device</b>: The device (phone or tablet).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>message</i> A message about the job's result.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>counters</i> The job's result counters.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>counters</b>: The job's result counters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stopped</i> The job's stop time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>started</i> The job's start time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>result</i> The job's result. Allowed values include: <ul> <li> PENDING: A pending condition. </li> <li> PASSED: A passing condition. </li> <li> WARNED: A warning condition. </li> <li> FAILED: A failed condition. </li> <li> SKIPPED: A skipped condition. </li> <li> ERRORED: An error condition. </li> <li> STOPPED: A stopped condition. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The job's status. Allowed values include: <ul> <li> PENDING: A pending status. </li> <li> PENDING_CONCURRENCY: A pending concurrency status. </li> <li> PENDING_DEVICE: A pending device status. </li> <li> PROCESSING: A processing status. </li> <li> SCHEDULING: A scheduling status. </li> <li> PREPARING: A preparing status. </li> <li> RUNNING: A running status. </li> <li> COMPLETED: A completed status. </li> <li> STOPPING: A stopping status. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>created</i> When the job was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The job's type. Allowed values include the following: <ul> <li> BUILTIN_FUZZ: The built-in fuzz type. </li> <li> BUILTIN_EXPLORER: For Android, an app explorer that will traverse an Android app, interacting with it and capturing screenshots at the same time. </li> <li> APPIUM_JAVA_JUNIT: The Appium Java JUnit type. </li> <li> APPIUM_JAVA_TESTNG: The Appium Java TestNG type. </li> <li> APPIUM_PYTHON: The Appium Python type. </li> <li> APPIUM_NODE: The Appium Node.js type. </li> <li> APPIUM_RUBY: The Appium Ruby type. </li> <li> APPIUM_WEB_JAVA_JUNIT: The Appium Java JUnit type for web apps. </li> <li> APPIUM_WEB_JAVA_TESTNG: The Appium Java TestNG type for web apps. </li> <li> APPIUM_WEB_PYTHON: The Appium Python type for web apps. </li> <li> APPIUM_WEB_NODE: The Appium Node.js type for web apps. </li> <li> APPIUM_WEB_RUBY: The Appium Ruby test type for web apps. </li> <li> CALABASH: The Calabash type. </li> <li> INSTRUMENTATION: The Instrumentation type. </li> <li> UIAUTOMATION: The uiautomation type. </li> <li> UIAUTOMATOR: The uiautomator type. </li> <li> XCTEST: The Xcode test type. </li> <li> XCTEST_UI: The Xcode UI test type. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The job's name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The job's ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>skipped</i> The number of skipped entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>stopped</i> The number of stopped entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>errored</i> The number of errored entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>warned</i> The number of warned entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>failed</i> The number of failed entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>passed</i> The number of passed entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>total</i> The total number of entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>os</i> The device's operating system type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>platform</i> The device's platform. Allowed values include: <ul> <li> ANDROID: The Android platform. </li> <li> IOS: The iOS platform. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>formFactor</i> The device's form factor. Allowed values include: <ul> <li> PHONE: The phone form factor. </li> <li> TABLET: The tablet form factor. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>modelId</i> The device's model ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>model</i> The device's model name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>manufacturer</i> The device's manufacturer name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The device's display name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The device's ARN.<br/>Information about the device's CPU.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>cpu</b>: Information about the device's CPU.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>unmetered</i> When specified, represents only the sum of unmetered minutes used by the resource to run tests.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>metered</i> When specified, represents only the sum of metered minutes used by the resource to run tests.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>total</i> When specified, represents the total minutes used by the resource to run tests.<br/>        """
        pass

    def stop_remote_access_session(self, arn: str) -> StopRemoteAccessSessionResult:
        """Ends a specified remote access session.

        :param self:
        :param arn:  The Amazon Resource Name (ARN) of the remote access session you wish to stop.<br/>
        :return: remoteAccessSession:  A container representing the metadata from the service about the remote access session you are stopping.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>skipAppResign</i> When set to <code>true</code>, for private devices, Device Farm will not sign your app again. For public devices, Device Farm always signs your apps again and this parameter has no effect. For more information about how Device Farm re-signs your app(s), see <a href="https://aws.amazon.com/device-farm/faq/">Do you modify my app?</a> in the <i>AWS Device Farm FAQs</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>interactionMode</i> The interaction mode of the remote access session. Valid values are: <ul> <li> INTERACTIVE: You can interact with the iOS device by viewing, touching, and rotating the screen. You <b>cannot</b> run XCUITest framework-based tests in this mode. </li> <li> NO_VIDEO: You are connected to the device but cannot interact with it or view the screen. This mode has the fastest test execution speed. You <b>can</b> run XCUITest framework-based tests in this mode. </li> <li> VIDEO_ONLY: You can view the screen but cannot touch or rotate it. You <b>can</b> run XCUITest framework-based tests and watch the screen in this mode. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deviceUdid</i> Unique device identifier for the remote device. Only returned if remote debugging is enabled for the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>endpoint</i> The endpoint for the remote access sesssion.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deviceMinutes</i> The number of minutes a device is used in a remote access session (including setup and teardown minutes).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>deviceMinutes</b>: The number of minutes a device is used in a remote access session (including setup and teardown minutes).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>billingMethod</i> The billing method of the remote access session. Possible values include <code>METERED</code> or <code>UNMETERED</code>. For more information about metered devices, see <a href="https://docs.aws.amazon.com/devicefarm/latest/developerguide/welcome.html#welcome-terminology">AWS Device Farm terminology</a>."<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clientId</i> Unique identifier of your client for the remote access session. Only returned if remote debugging is enabled for the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>hostAddress</i> IP address of the EC2 host where you need to connect to remotely debug devices. Only returned if remote debugging is enabled for the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>remoteRecordAppArn</i> The Amazon Resource Name (ARN) for the app to be recorded in the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>remoteRecordEnabled</i> This flag is set to <code>true</code> if remote recording is enabled for the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>remoteDebugEnabled</i> This flag is set to <code>true</code> if remote debugging is enabled for the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>instanceArn</i> The Amazon Resource Name (ARN) of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>device</i> The device (phone or tablet) used in the remote access session.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>device</b>: The device (phone or tablet) used in the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stopped</i> The date and time the remote access session was stopped.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>started</i> The date and time the remote access session was started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>message</i> A message about the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>result</i> The result of the remote access session. Can be any of the following: <ul> <li> PENDING: A pending condition. </li> <li> PASSED: A passing condition. </li> <li> WARNED: A warning condition. </li> <li> FAILED: A failed condition. </li> <li> SKIPPED: A skipped condition. </li> <li> ERRORED: An error condition. </li> <li> STOPPED: A stopped condition. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the remote access session. Can be any of the following: <ul> <li> PENDING: A pending status. </li> <li> PENDING_CONCURRENCY: A pending concurrency status. </li> <li> PENDING_DEVICE: A pending device status. </li> <li> PROCESSING: A processing status. </li> <li> SCHEDULING: A scheduling status. </li> <li> PREPARING: A preparing status. </li> <li> RUNNING: A running status. </li> <li> COMPLETED: A completed status. </li> <li> STOPPING: A stopping status. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>created</i> The date and time the remote access session was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the remote access session.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>os</i> The device's operating system type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>platform</i> The device's platform. Allowed values include: <ul> <li> ANDROID: The Android platform. </li> <li> IOS: The iOS platform. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>formFactor</i> The device's form factor. Allowed values include: <ul> <li> PHONE: The phone form factor. </li> <li> TABLET: The tablet form factor. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>modelId</i> The device's model ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>model</i> The device's model name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>manufacturer</i> The device's manufacturer name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The device's display name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The device's ARN.<br/>Information about the device's CPU.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>cpu</b>: Information about the device's CPU.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>unmetered</i> When specified, represents only the sum of unmetered minutes used by the resource to run tests.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>metered</i> When specified, represents only the sum of metered minutes used by the resource to run tests.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>total</i> When specified, represents the total minutes used by the resource to run tests.<br/>        """
        pass

    def stop_run(self, arn: str) -> StopRunResult:
        """Initiates a stop request for the current test run. AWS Device Farm will immediately stop the run on devices where tests have not started executing, and you will not be billed for these devices. On devices where tests have started executing, Setup Suite and Teardown Suite tests will run to completion before stopping execution on those devices. You will be billed for Setup, Teardown, and any tests that were in progress or already completed.

        :param self:
        :param arn:  Represents the Amazon Resource Name (ARN) of the Device Farm run you wish to stop.<br/>
        :return: run:  The run that was stopped.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deviceSelectionResult</i> The results of a device filter used to select the devices for a test run.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>deviceSelectionResult</b>: The results of a device filter used to select the devices for a test run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>testSpecArn</i> The ARN of the YAML-formatted test specification for the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>skipAppResign</i> When set to <code>true</code>, for private devices, Device Farm will not sign your app again. For public devices, Device Farm always signs your apps again and this parameter has no effect. For more information about how Device Farm re-signs your app(s), see <a href="https://aws.amazon.com/device-farm/faq/">Do you modify my app?</a> in the <i>AWS Device Farm FAQs</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>webUrl</i> The Device Farm console URL for the recording of the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customerArtifactPaths</i> Output <code>CustomerArtifactPaths</code> object for the test run.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>customerArtifactPaths</b>: Output <code>CustomerArtifactPaths</code> object for the test run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> Information about the location that is used for the run.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>location</b>: Information about the location that is used for the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>radios</i> Information about the radio states for the run.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>radios</b>: Information about the radio states for the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>locale</i> Information about the locale that is used for the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>devicePoolArn</i> The ARN of the device pool for the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jobTimeoutMinutes</i> The number of minutes the job will execute before it times out.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>eventCount</i> For fuzz tests, this is the number of events, between 1 and 10000, that the UI fuzz test should perform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>appUpload</i> An app to upload or that has been uploaded.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>seed</i> For fuzz tests, this is a seed to use for randomizing the UI fuzz test. Using the same seed value between tests ensures identical event sequences.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resultCode</i> Supporting field for the result field. Set only if <code>result</code> is <code>SKIPPED</code>. <code>PARSING_FAILED</code> if the result is skipped because of test package parsing failure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>parsingResultUrl</i> Read-only URL for an object in S3 bucket where you can get the parsing results of the test package. If the test package doesn't parse, the reason why it doesn't parse appears in the file that this URL points to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>networkProfile</i> The network profile being used for a test run.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>networkProfile</b>: The network profile being used for a test run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deviceMinutes</i> Represents the total (metered or unmetered) minutes used by the test run.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>deviceMinutes</b>: Represents the total (metered or unmetered) minutes used by the test run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>billingMethod</i> Specifies the billing method for a test run: <code>metered</code> or <code>unmetered</code>. If the parameter is not specified, the default value is <code>metered</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>completedJobs</i> The total number of completed jobs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>totalJobs</i> The total number of jobs for the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>message</i> A message about the run's result.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>counters</i> The run's result counters.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>counters</b>: The run's result counters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stopped</i> The run's stop time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>started</i> The run's start time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>result</i> The run's result. Allowed values include: <ul> <li> PENDING: A pending condition. </li> <li> PASSED: A passing condition. </li> <li> WARNED: A warning condition. </li> <li> FAILED: A failed condition. </li> <li> SKIPPED: A skipped condition. </li> <li> ERRORED: An error condition. </li> <li> STOPPED: A stopped condition. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The run's status. Allowed values include: <ul> <li> PENDING: A pending status. </li> <li> PENDING_CONCURRENCY: A pending concurrency status. </li> <li> PENDING_DEVICE: A pending device status. </li> <li> PROCESSING: A processing status. </li> <li> SCHEDULING: A scheduling status. </li> <li> PREPARING: A preparing status. </li> <li> RUNNING: A running status. </li> <li> COMPLETED: A completed status. </li> <li> STOPPING: A stopping status. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>created</i> When the run was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>platform</i> The run's platform. Allowed values include: <ul> <li> ANDROID: The Android platform. </li> <li> IOS: The iOS platform. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The run's type. Must be one of the following values: <ul> <li> BUILTIN_FUZZ: The built-in fuzz type. </li> <li> BUILTIN_EXPLORER: For Android, an app explorer that will traverse an Android app, interacting with it and capturing screenshots at the same time. </li> <li> APPIUM_JAVA_JUNIT: The Appium Java JUnit type. </li> <li> APPIUM_JAVA_TESTNG: The Appium Java TestNG type. </li> <li> APPIUM_PYTHON: The Appium Python type. </li> <li> APPIUM_NODE: The Appium Node.js type. </li> <li> APPIUM_RUBY: The Appium Ruby type. </li> <li> APPIUM_WEB_JAVA_JUNIT: The Appium Java JUnit type for web apps. </li> <li> APPIUM_WEB_JAVA_TESTNG: The Appium Java TestNG type for web apps. </li> <li> APPIUM_WEB_PYTHON: The Appium Python type for web apps. </li> <li> APPIUM_WEB_NODE: The Appium Node.js type for web apps. </li> <li> APPIUM_WEB_RUBY: The Appium Ruby type for web apps. </li> <li> CALABASH: The Calabash type. </li> <li> INSTRUMENTATION: The Instrumentation type. </li> <li> UIAUTOMATION: The uiautomation type. </li> <li> UIAUTOMATOR: The uiautomator type. </li> <li> XCTEST: The Xcode test type. </li> <li> XCTEST_UI: The Xcode UI test type. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The run's name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The run's ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>skipped</i> The number of skipped entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>stopped</i> The number of stopped entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>errored</i> The number of errored entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>warned</i> The number of warned entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>failed</i> The number of failed entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>passed</i> The number of passed entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>total</i> The total number of entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>unmetered</i> When specified, represents only the sum of unmetered minutes used by the resource to run tests.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>metered</i> When specified, represents only the sum of metered minutes used by the resource to run tests.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>total</i> When specified, represents the total minutes used by the resource to run tests.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>downlinkLossPercent</i> Proportion of received packets that fail to arrive from 0 to 100 percent.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uplinkLossPercent</i> Proportion of transmitted packets that fail to arrive from 0 to 100 percent.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>downlinkJitterMs</i> Time variation in the delay of received packets in milliseconds as an integer from 0 to 2000.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uplinkJitterMs</i> Time variation in the delay of received packets in milliseconds as an integer from 0 to 2000.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>downlinkDelayMs</i> Delay time for all packets to destination in milliseconds as an integer from 0 to 2000.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uplinkDelayMs</i> Delay time for all packets to destination in milliseconds as an integer from 0 to 2000.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>downlinkBandwidthBits</i> The data throughput rate in bits per second, as an integer from 0 to 104857600.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uplinkBandwidthBits</i> The data throughput rate in bits per second, as an integer from 0 to 104857600.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of network profile. Valid values are listed below.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The description of the network profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the network profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the network profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>gps</i> True if GPS is enabled at the beginning of the test; otherwise, false.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>nfc</i> True if NFC is enabled at the beginning of the test; otherwise, false.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>bluetooth</i> True if Bluetooth is enabled at the beginning of the test; otherwise, false.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>wifi</i> True if Wi-Fi is enabled at the beginning of the test; otherwise, false.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>longitude</i> The longitude.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>latitude</i> The latitude.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>deviceHostPaths</i> Comma-separated list of paths in the test execution environment where the artifacts generated by the customer's tests will be pulled from.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>androidPaths</i> Comma-separated list of paths on the Android device where the artifacts generated by the customer's tests will be pulled from.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>iosPaths</i> Comma-separated list of paths on the iOS device where the artifacts generated by the customer's tests will be pulled from.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>maxDevices</i> The maximum number of devices to be selected by a device filter and included in a test run.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>matchedDevicesCount</i> The number of devices that matched the device filter selection criteria.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>filters</i> The filters in a device selection result.<br/>        """
        pass

    def tag_resource(self, Tags: List[Tag], ResourceARN: str) -> TagResourceResponse:
        """Associates the specified tags to a resource with the specified <code>resourceArn</code>. If existing tags on a resource are not specified in the request parameters, they are not changed. When a resource is deleted, the tags associated with that resource are deleted as well.

        :param self:
        :param Tags:  The tags to add to the resource. A tag is an array of key-value pairs. Tag keys can have a maximum character length of 128 characters, and tag values can have a maximum length of 256 characters.<br/>ResourceARN:  The Amazon Resource Name (ARN) of the resource(s) to which to add tags. You can associate tags with the following Device Farm resources: <code>PROJECT</code>, <code>RUN</code>, <code>NETWORK_PROFILE</code>, <code>INSTANCE_PROFILE</code>, <code>DEVICE_INSTANCE</code>, <code>SESSION</code>, <code>DEVICE_POOL</code>, <code>DEVICE</code>, and <code>VPCE_CONFIGURATION</code>.<br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[TagKey], ResourceARN: str) -> UntagResourceResponse:
        """Deletes the specified tags from a resource.

        :param self:
        :param TagKeys:  The keys of the tags to be removed.<br/>ResourceARN:  The Amazon Resource Name (ARN) of the resource(s) from which to delete tags. You can associate tags with the following Device Farm resources: <code>PROJECT</code>, <code>RUN</code>, <code>NETWORK_PROFILE</code>, <code>INSTANCE_PROFILE</code>, <code>DEVICE_INSTANCE</code>, <code>SESSION</code>, <code>DEVICE_POOL</code>, <code>DEVICE</code>, and <code>VPCE_CONFIGURATION</code>.<br/>
        :return:         """
        pass

    def update_device_instance(self, arn: str, profileArn: str=None, labels: List[String]=None) -> UpdateDeviceInstanceResult:
        """Updates information about an existing private device instance.

        :param self:
        :param labels:  An array of strings that you want to associate with the device instance.<br/>profileArn:  The Amazon Resource Name (ARN) of the profile that you want to associate with the device instance.<br/>arn:  The Amazon Resource Name (ARN) of the device instance.<br/>
        :return: deviceInstance:  An object containing information about your device instance.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>instanceProfile</i> A object containing information about the instance profile.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>instanceProfile</b>: A object containing information about the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>udid</i> Unique device identifier for the device instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the device instance. Valid values are listed below.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>labels</i> An array of strings describing the device instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deviceArn</i> The Amazon Resource Name (ARN) of the device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the device instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The description of the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>rebootAfterUse</i> When set to <code>true</code>, Device Farm will reboot the instance after a test run. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>excludeAppPackagesFromCleanup</i> An array of strings specifying the list of app packages that should not be cleaned up from the device after a test run is over. The list of packages is only considered if you set <code>packageCleanup</code> to <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>packageCleanup</i> When set to <code>true</code>, Device Farm will remove app packages after a test run. The default value is <code>false</code> for private devices.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the instance profile.<br/>        """
        pass

    def update_device_pool(self, arn: str, name: str=None, description: str=None, rules: List[Rule]=None, maxDevices: int=None, clearMaxDevices: Boolean=None) -> UpdateDevicePoolResult:
        """Modifies the name, description, and rules in a device pool given the attributes and the pool ARN. Rule updates are all-or-nothing, meaning they can only be updated as a whole (or not at all).

        :param self:
        :param clearMaxDevices:  Sets whether the <code>maxDevices</code> parameter applies to your device pool. If you set this parameter to <code>true</code>, the <code>maxDevices</code> parameter does not apply, and Device Farm does not limit the number of devices that it adds to your device pool. In this case, Device Farm adds all available devices that meet the criteria that are specified for the <code>rules</code> parameter. If you use this parameter in your request, you cannot use the <code>maxDevices</code> parameter in the same request.<br/>maxDevices:  The number of devices that Device Farm can add to your device pool. Device Farm adds devices that are available and that meet the criteria that you assign for the <code>rules</code> parameter. Depending on how many devices meet these constraints, your device pool might contain fewer devices than the value for this parameter. By specifying the maximum number of devices, you can control the costs that you incur by running tests. If you use this parameter in your request, you cannot use the <code>clearMaxDevices</code> parameter in the same request.<br/>rules:  Represents the rules you wish to modify for the device pool. Updating rules is optional; however, if you choose to update rules for your request, the update will replace the existing rules.<br/>description:  A description of the device pool you wish to update.<br/>name:  A string representing the name of the device pool you wish to update.<br/>arn:  The Amazon Resource Name (ARN) of the Device Farm device pool you wish to update.<br/>
        :return: devicePool:  The device pool you just updated.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>maxDevices</i> The number of devices that Device Farm can add to your device pool. Device Farm adds devices that are available and that meet the criteria that you assign for the <code>rules</code> parameter. Depending on how many devices meet these constraints, your device pool might contain fewer devices than the value for this parameter. By specifying the maximum number of devices, you can control the costs that you incur by running tests.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>rules</i> Information about the device pool's rules.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The device pool's type. Allowed values include: <ul> <li> CURATED: A device pool that is created and managed by AWS Device Farm. </li> <li> PRIVATE: A device pool that is created and managed by the device pool developer. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The device pool's description.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The device pool's name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The device pool's ARN.<br/>        """
        pass

    def update_instance_profile(self, arn: str, name: str=None, description: str=None, packageCleanup: Boolean=None, excludeAppPackagesFromCleanup: List[String]=None, rebootAfterUse: Boolean=None) -> UpdateInstanceProfileResult:
        """Updates information about an existing private device instance profile.

        :param self:
        :param rebootAfterUse:  The updated choice for whether you want to reboot the device after use. The default value is <code>true</code>.<br/>excludeAppPackagesFromCleanup:  An array of strings specifying the list of app packages that should not be cleaned up from the device after a test run is over. The list of packages is only considered if you set <code>packageCleanup</code> to <code>true</code>.<br/>packageCleanup:  The updated choice for whether you want to specify package cleanup. The default value is <code>false</code> for private devices.<br/>description:  The updated description for your instance profile.<br/>name:  The updated name for your instance profile.<br/>arn:  The Amazon Resource Name (ARN) of the instance profile.<br/>
        :return: instanceProfile:  An object containing information about your instance profile.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The description of the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>rebootAfterUse</i> When set to <code>true</code>, Device Farm will reboot the instance after a test run. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>excludeAppPackagesFromCleanup</i> An array of strings specifying the list of app packages that should not be cleaned up from the device after a test run is over. The list of packages is only considered if you set <code>packageCleanup</code> to <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>packageCleanup</i> When set to <code>true</code>, Device Farm will remove app packages after a test run. The default value is <code>false</code> for private devices.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the instance profile.<br/>        """
        pass

    def update_network_profile(self, arn: str, name: str=None, description: str=None, type: str=None, uplinkBandwidthBits: Long=None, downlinkBandwidthBits: Long=None, uplinkDelayMs: Long=None, downlinkDelayMs: Long=None, uplinkJitterMs: Long=None, downlinkJitterMs: Long=None, uplinkLossPercent: int=None, downlinkLossPercent: int=None) -> UpdateNetworkProfileResult:
        """Updates the network profile with specific settings.

        :param self:
        :param downlinkLossPercent:  Proportion of received packets that fail to arrive from 0 to 100 percent.<br/>uplinkLossPercent:  Proportion of transmitted packets that fail to arrive from 0 to 100 percent.<br/>downlinkJitterMs:  Time variation in the delay of received packets in milliseconds as an integer from 0 to 2000.<br/>uplinkJitterMs:  Time variation in the delay of received packets in milliseconds as an integer from 0 to 2000.<br/>downlinkDelayMs:  Delay time for all packets to destination in milliseconds as an integer from 0 to 2000.<br/>uplinkDelayMs:  Delay time for all packets to destination in milliseconds as an integer from 0 to 2000.<br/>downlinkBandwidthBits:  The data throughput rate in bits per second, as an integer from 0 to 104857600.<br/>uplinkBandwidthBits:  The data throughput rate in bits per second, as an integer from 0 to 104857600.<br/>type:  The type of network profile you wish to return information about. Valid values are listed below.<br/>description:  The description of the network profile about which you are returning information.<br/>name:  The name of the network profile about which you are returning information.<br/>arn:  The Amazon Resource Name (ARN) of the project for which you want to update network profile settings.<br/>
        :return: networkProfile:  A list of the available network profiles.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>downlinkLossPercent</i> Proportion of received packets that fail to arrive from 0 to 100 percent.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>uplinkLossPercent</i> Proportion of transmitted packets that fail to arrive from 0 to 100 percent.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>downlinkJitterMs</i> Time variation in the delay of received packets in milliseconds as an integer from 0 to 2000.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>uplinkJitterMs</i> Time variation in the delay of received packets in milliseconds as an integer from 0 to 2000.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>downlinkDelayMs</i> Delay time for all packets to destination in milliseconds as an integer from 0 to 2000.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>uplinkDelayMs</i> Delay time for all packets to destination in milliseconds as an integer from 0 to 2000.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>downlinkBandwidthBits</i> The data throughput rate in bits per second, as an integer from 0 to 104857600.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>uplinkBandwidthBits</i> The data throughput rate in bits per second, as an integer from 0 to 104857600.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of network profile. Valid values are listed below.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The description of the network profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the network profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the network profile.<br/>        """
        pass

    def update_project(self, arn: str, name: str=None, defaultJobTimeoutMinutes: int=None) -> UpdateProjectResult:
        """Modifies the specified project name, given the project ARN and a new name.

        :param self:
        :param defaultJobTimeoutMinutes:  The number of minutes a test run in the project will execute before it times out.<br/>name:  A string representing the new name of the project that you are updating.<br/>arn:  The Amazon Resource Name (ARN) of the project whose name you wish to update.<br/>
        :return: project:  The project you wish to update.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>created</i> When the project was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultJobTimeoutMinutes</i> The default number of minutes (at the project level) a test run will execute before it times out. The default value is 150 minutes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The project's name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The project's ARN.<br/>        """
        pass

    def update_upload(self, arn: str, name: str=None, contentType: str=None, editContent: Boolean=None) -> UpdateUploadResult:
        """Update an uploaded test specification (test spec).

        :param self:
        :param editContent:  Set to true if the YAML file has changed and needs to be updated; otherwise, set to false.<br/>contentType:  The upload's content type (for example, "application/x-yaml").<br/>name:  The upload's test spec file name. The name should not contain the '/' character. The test spec file name must end with the <code>.yaml</code> or <code>.yml</code> file extension.<br/>arn:  The Amazon Resource Name (ARN) of the uploaded test spec.<br/>
        :return: upload:  A test spec uploaded to Device Farm.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>category</i> The upload's category. Allowed values include: <ul> <li> CURATED: An upload managed by AWS Device Farm. </li> <li> PRIVATE: An upload managed by the AWS Device Farm customer. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>message</i> A message about the upload's result.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>contentType</i> The upload's content type (for example, "application/octet-stream").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>metadata</i> The upload's metadata. For example, for Android, this contains information that is parsed from the manifest and is displayed in the AWS Device Farm console after the associated app is uploaded.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>url</i> The pre-signed Amazon S3 URL that was used to store a file through a corresponding PUT request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The upload's status. Must be one of the following values: <ul> <li> FAILED: A failed status. </li> <li> INITIALIZED: An initialized status. </li> <li> PROCESSING: A processing status. </li> <li> SUCCEEDED: A succeeded status. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The upload's type. Must be one of the following values: <ul> <li> ANDROID_APP: An Android upload. </li> <li> IOS_APP: An iOS upload. </li> <li> WEB_APP: A web application upload. </li> <li> EXTERNAL_DATA: An external data upload. </li> <li> APPIUM_JAVA_JUNIT_TEST_PACKAGE: An Appium Java JUnit test package upload. </li> <li> APPIUM_JAVA_TESTNG_TEST_PACKAGE: An Appium Java TestNG test package upload. </li> <li> APPIUM_PYTHON_TEST_PACKAGE: An Appium Python test package upload. </li> <li> APPIUM_NODE_TEST_PACKAGE: An Appium Node.js test package upload. </li> <li> APPIUM_RUBY_TEST_PACKAGE: An Appium Ruby test package upload. </li> <li> APPIUM_WEB_JAVA_JUNIT_TEST_PACKAGE: An Appium Java JUnit test package upload for web apps. </li> <li> APPIUM_WEB_JAVA_TESTNG_TEST_PACKAGE: An Appium Java TestNG test package upload for web apps. </li> <li> APPIUM_WEB_PYTHON_TEST_PACKAGE: An Appium Python test package upload for web apps. </li> <li> APPIUM_WEB_NODE_TEST_PACKAGE: An Appium Node.js test package upload for web apps. </li> <li> APPIUM_WEB_RUBY_TEST_PACKAGE: An Appium Ruby test package upload for web apps. </li> <li> CALABASH_TEST_PACKAGE: A Calabash test package upload. </li> <li> INSTRUMENTATION_TEST_PACKAGE: An instrumentation upload. </li> <li> UIAUTOMATION_TEST_PACKAGE: A uiautomation test package upload. </li> <li> UIAUTOMATOR_TEST_PACKAGE: A uiautomator test package upload. </li> <li> XCTEST_TEST_PACKAGE: An Xcode test package upload. </li> <li> XCTEST_UI_TEST_PACKAGE: An Xcode UI test package upload. </li> <li> APPIUM_JAVA_JUNIT_TEST_SPEC: An Appium Java JUnit test spec upload. </li> <li> APPIUM_JAVA_TESTNG_TEST_SPEC: An Appium Java TestNG test spec upload. </li> <li> APPIUM_PYTHON_TEST_SPEC: An Appium Python test spec upload. </li> <li> APPIUM_NODE_TEST_SPEC: An Appium Node.js test spec upload. </li> <li> APPIUM_RUBY_TEST_SPEC: An Appium Ruby test spec upload. </li> <li> APPIUM_WEB_JAVA_JUNIT_TEST_SPEC: An Appium Java JUnit test spec upload for a web app. </li> <li> APPIUM_WEB_JAVA_TESTNG_TEST_SPEC: An Appium Java TestNG test spec upload for a web app. </li> <li> APPIUM_WEB_PYTHON_TEST_SPEC: An Appium Python test spec upload for a web app. </li> <li> APPIUM_WEB_NODE_TEST_SPEC: An Appium Node.js test spec upload for a web app. </li> <li> APPIUM_WEB_RUBY_TEST_SPEC: An Appium Ruby test spec upload for a web app. </li> <li> INSTRUMENTATION_TEST_SPEC: An instrumentation test spec upload. </li> <li> XCTEST_UI_TEST_SPEC: An Xcode UI test spec upload. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>created</i> When the upload was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The upload's file name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The upload's ARN.<br/>        """
        pass

    def update_vpce_configuration(self, arn: str, vpceConfigurationName: str=None, vpceServiceName: str=None, serviceDnsName: str=None, vpceConfigurationDescription: str=None) -> UpdateVPCEConfigurationResult:
        """Updates information about an existing Amazon Virtual Private Cloud (VPC) endpoint configuration.

        :param self:
        :param vpceConfigurationDescription:  An optional description, providing more details about your VPC endpoint configuration.<br/>serviceDnsName:  The DNS (domain) name used to connect to your private service in your Amazon VPC. The DNS name must not already be in use on the Internet.<br/>vpceServiceName:  The name of the VPC endpoint service running inside your AWS account that you want Device Farm to test.<br/>vpceConfigurationName:  The friendly name you give to your VPC endpoint configuration, to manage your configurations more easily.<br/>arn:  The Amazon Resource Name (ARN) of the VPC endpoint configuration you want to update.<br/>
        :return: vpceConfiguration:  An object containing information about your VPC endpoint configuration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vpceConfigurationDescription</i> An optional description, providing more details about your VPC endpoint configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceDnsName</i> The DNS name that maps to the private IP address of the service you want to access.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vpceServiceName</i> The name of the VPC endpoint service running inside your AWS account that you want Device Farm to test.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vpceConfigurationName</i> The friendly name you give to your VPC endpoint configuration, to manage your configurations more easily.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the VPC endpoint configuration.<br/>        """
        pass



class DirectConnect(BaseClient):
    """AWS Direct Connect links your internal network to an AWS Direct Connect location over a standard Ethernet fiber-optic cable. One end of the cable is connected to your router, the other to an AWS Direct Connect router. With this connection in place, you can create virtual interfaces directly to the AWS cloud (for example, to Amazon EC2 and Amazon S3) and to Amazon VPC, bypassing Internet service providers in your network path. A connection provides access to all AWS Regions except the China (Beijing) and (China) Ningxia Regions. AWS resources in the China Regions can only be accessed through locations associated with those Regions."""


    class DescribeDirectConnectGatewayAssociationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_direct_connect_gateway_associations` for valid parameters"""
            pass
    class DescribeDirectConnectGatewayAttachmentsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_direct_connect_gateway_attachments` for valid parameters"""
            pass
    class DescribeDirectConnectGatewaysPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_direct_connect_gateways` for valid parameters"""
            pass


    class AcceptDirectConnectGatewayAssociationProposalRequest(Mapping):
        pass
    
    class AcceptDirectConnectGatewayAssociationProposalResult(Mapping):
        pass
    
    class RouteFilterPrefix(Mapping):
        pass
    
    class AllocateConnectionOnInterconnectRequest(Mapping):
        pass
    
    class Connection(Mapping):
        pass
    
    class AllocateHostedConnectionRequest(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class AllocatePrivateVirtualInterfaceRequest(Mapping):
        pass
    
    class VirtualInterface(Mapping):
        pass
    
    class NewPrivateVirtualInterfaceAllocation(Mapping):
        pass
    
    class AllocatePublicVirtualInterfaceRequest(Mapping):
        pass
    
    class NewPublicVirtualInterfaceAllocation(Mapping):
        pass
    
    class AllocateTransitVirtualInterfaceRequest(Mapping):
        pass
    
    class AllocateTransitVirtualInterfaceResult(Mapping):
        pass
    
    class NewTransitVirtualInterfaceAllocation(Mapping):
        pass
    
    class AssociateConnectionWithLagRequest(Mapping):
        pass
    
    class AssociateHostedConnectionRequest(Mapping):
        pass
    
    class AssociateVirtualInterfaceRequest(Mapping):
        pass
    
    class ConfirmConnectionRequest(Mapping):
        pass
    
    class ConfirmConnectionResponse(Mapping):
        pass
    
    class ConfirmPrivateVirtualInterfaceRequest(Mapping):
        pass
    
    class ConfirmPrivateVirtualInterfaceResponse(Mapping):
        pass
    
    class ConfirmPublicVirtualInterfaceRequest(Mapping):
        pass
    
    class ConfirmPublicVirtualInterfaceResponse(Mapping):
        pass
    
    class ConfirmTransitVirtualInterfaceRequest(Mapping):
        pass
    
    class ConfirmTransitVirtualInterfaceResponse(Mapping):
        pass
    
    class CreateBGPPeerRequest(Mapping):
        pass
    
    class CreateBGPPeerResponse(Mapping):
        pass
    
    class NewBGPPeer(Mapping):
        pass
    
    class CreateConnectionRequest(Mapping):
        pass
    
    class CreateDirectConnectGatewayRequest(Mapping):
        pass
    
    class CreateDirectConnectGatewayResult(Mapping):
        pass
    
    class LongAsn(object):
        pass
    
    class CreateDirectConnectGatewayAssociationRequest(Mapping):
        pass
    
    class CreateDirectConnectGatewayAssociationResult(Mapping):
        pass
    
    class CreateDirectConnectGatewayAssociationProposalRequest(Mapping):
        pass
    
    class CreateDirectConnectGatewayAssociationProposalResult(Mapping):
        pass
    
    class CreateInterconnectRequest(Mapping):
        pass
    
    class Interconnect(Mapping):
        pass
    
    class CreateLagRequest(Mapping):
        pass
    
    class Lag(Mapping):
        pass
    
    class CreatePrivateVirtualInterfaceRequest(Mapping):
        pass
    
    class NewPrivateVirtualInterface(Mapping):
        pass
    
    class CreatePublicVirtualInterfaceRequest(Mapping):
        pass
    
    class NewPublicVirtualInterface(Mapping):
        pass
    
    class CreateTransitVirtualInterfaceRequest(Mapping):
        pass
    
    class CreateTransitVirtualInterfaceResult(Mapping):
        pass
    
    class NewTransitVirtualInterface(Mapping):
        pass
    
    class DeleteBGPPeerRequest(Mapping):
        pass
    
    class DeleteBGPPeerResponse(Mapping):
        pass
    
    class DeleteConnectionRequest(Mapping):
        pass
    
    class DeleteDirectConnectGatewayRequest(Mapping):
        pass
    
    class DeleteDirectConnectGatewayResult(Mapping):
        pass
    
    class DeleteDirectConnectGatewayAssociationRequest(Mapping):
        pass
    
    class DeleteDirectConnectGatewayAssociationResult(Mapping):
        pass
    
    class DeleteDirectConnectGatewayAssociationProposalRequest(Mapping):
        pass
    
    class DeleteDirectConnectGatewayAssociationProposalResult(Mapping):
        pass
    
    class DeleteInterconnectRequest(Mapping):
        pass
    
    class DeleteInterconnectResponse(Mapping):
        pass
    
    class DeleteLagRequest(Mapping):
        pass
    
    class DeleteVirtualInterfaceRequest(Mapping):
        pass
    
    class DeleteVirtualInterfaceResponse(Mapping):
        pass
    
    class DescribeConnectionLoaRequest(Mapping):
        pass
    
    class DescribeConnectionLoaResponse(Mapping):
        pass
    
    class DescribeConnectionsRequest(Mapping):
        pass
    
    class Connections(Mapping):
        pass
    
    class DescribeConnectionsOnInterconnectRequest(Mapping):
        pass
    
    class DescribeDirectConnectGatewayAssociationProposalsRequest(Mapping):
        pass
    
    class DescribeDirectConnectGatewayAssociationProposalsResult(Mapping):
        pass
    
    class DescribeDirectConnectGatewayAssociationsRequest(Mapping):
        pass
    
    class DescribeDirectConnectGatewayAssociationsResult(Mapping):
        pass
    
    class DescribeDirectConnectGatewayAttachmentsRequest(Mapping):
        pass
    
    class DescribeDirectConnectGatewayAttachmentsResult(Mapping):
        pass
    
    class DescribeDirectConnectGatewaysRequest(Mapping):
        pass
    
    class DescribeDirectConnectGatewaysResult(Mapping):
        pass
    
    class DescribeHostedConnectionsRequest(Mapping):
        pass
    
    class DescribeInterconnectLoaRequest(Mapping):
        pass
    
    class DescribeInterconnectLoaResponse(Mapping):
        pass
    
    class DescribeInterconnectsRequest(Mapping):
        pass
    
    class Interconnects(Mapping):
        pass
    
    class DescribeLagsRequest(Mapping):
        pass
    
    class Lags(Mapping):
        pass
    
    class DescribeLoaRequest(Mapping):
        pass
    
    class Loa(Mapping):
        pass
    
    class Locations(Mapping):
        pass
    
    class DescribeTagsRequest(Mapping):
        pass
    
    class DescribeTagsResponse(Mapping):
        pass
    
    class ResourceArn(object):
        pass
    
    class VirtualGateways(Mapping):
        pass
    
    class DescribeVirtualInterfacesRequest(Mapping):
        pass
    
    class VirtualInterfaces(Mapping):
        pass
    
    class DisassociateConnectionFromLagRequest(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateDirectConnectGatewayAssociationRequest(Mapping):
        pass
    
    class UpdateDirectConnectGatewayAssociationResult(Mapping):
        pass
    
    class UpdateLagRequest(Mapping):
        pass
    
    class UpdateVirtualInterfaceAttributesRequest(Mapping):
        pass
    
    def accept_direct_connect_gateway_association_proposal(self, associatedGatewayOwnerAccount: str, proposalId: str, directConnectGatewayId: str, overrideAllowedPrefixesToDirectConnectGateway: List[RouteFilterPrefix]=None) -> AcceptDirectConnectGatewayAssociationProposalResult:
        """Accepts a proposal request to attach a virtual private gateway or transit gateway to a Direct Connect gateway.

        :param self:
        :param overrideAllowedPrefixesToDirectConnectGateway:  Overrides the Amazon VPC prefixes advertised to the Direct Connect gateway. For information about how to set the prefixes, see <a href="https://docs.aws.amazon.com/directconnect/latest/UserGuide/multi-account-associate-vgw.html#allowed-prefixes">Allowed Prefixes</a> in the <i>AWS Direct Connect User Guide</i>.<br/>associatedGatewayOwnerAccount:  The ID of the AWS account that owns the virtual private gateway or transit gateway.<br/>proposalId:  The ID of the request proposal.<br/>directConnectGatewayId:  The ID of the Direct Connect gateway.<br/>
        :return: directConnectGatewayAssociation:  Information about an association between a Direct Connect gateway and a virtual private gateway or transit gateway.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualGatewayOwnerAccount</i> The ID of the AWS account that owns the virtual private gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualGatewayRegion</i> The AWS Region where the virtual private gateway is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualGatewayId</i> The ID of the virtual private gateway. Applies only to private virtual interfaces.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>allowedPrefixesToDirectConnectGateway</i> The Amazon VPC prefixes to advertise to the Direct Connect gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>associationId</i> The ID of the Direct Connect gateway association.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>associatedGateway</i> Information about the associated gateway.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>associatedGateway</b>: Information about the associated gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stateChangeError</i> The error message if the state of an object failed to advance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>associationState</i> The state of the association. The following are the possible values: <ul> <li>  <code>associating</code>: The initial state after calling <a>CreateDirectConnectGatewayAssociation</a>. </li> <li>  <code>associated</code>: The Direct Connect gateway and virtual private gateway or transit gateway are successfully associated and ready to pass traffic. </li> <li>  <code>disassociating</code>: The initial state after calling <a>DeleteDirectConnectGatewayAssociation</a>. </li> <li>  <code>disassociated</code>: The virtual private gateway or transit gateway is disassociated from the Direct Connect gateway. Traffic flow between the Direct Connect gateway and virtual private gateway or transit gateway is stopped. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>directConnectGatewayOwnerAccount</i> The ID of the AWS account that owns the associated gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>directConnectGatewayId</i> The ID of the Direct Connect gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>region</i> The Region where the associated gateway is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ownerAccount</i> The ID of the AWS account that owns the associated virtual private gateway or transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of associated gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the associated gateway.<br/>        """
        pass

    def allocate_connection_on_interconnect(self, vlan: int, interconnectId: str, ownerAccount: str, connectionName: str, bandwidth: str) -> Connection:
        """Deprecated. Use <a>AllocateHostedConnection</a> instead. Creates a hosted connection on an interconnect. Allocates a VLAN number and a specified amount of bandwidth for use by a hosted connection on the specified interconnect. <note> Intended for use by AWS Direct Connect Partners only. </note>

        :param self:
        :param vlan:  The dedicated VLAN provisioned to the connection.<br/>interconnectId:  The ID of the interconnect on which the connection will be provisioned.<br/>ownerAccount:  The ID of the AWS account of the customer for whom the connection will be provisioned.<br/>connectionName:  The name of the provisioned connection.<br/>bandwidth:  The bandwidth of the connection. The possible values are 50Mbps, 100Mbps, 200Mbps, 300Mbps, 400Mbps, 500Mbps, 1Gbps, 2Gbps, 5Gbps, and 10Gbps. Note that only those AWS Direct Connect Partners who have met specific requirements are allowed to create a 1Gbps, 2Gbps, 5Gbps or 10Gbps hosted connection.<br/>
        :return: tags:  Any tags assigned to the connection.<br/>hasLogicalRedundancy:  Indicates whether the connection supports a secondary BGP peer in the same address family (IPv4/IPv6).<br/>awsDeviceV2:  The Direct Connect endpoint on which the physical connection terminates.<br/>jumboFrameCapable:  Indicates whether jumbo frames (9001 MTU) are supported.<br/>awsDevice:  The Direct Connect endpoint on which the physical connection terminates.<br/>lagId:  The ID of the LAG.<br/>loaIssueTime:  The time of the most recent call to <a>DescribeLoa</a> for this connection.<br/>partnerName:  The name of the AWS Direct Connect service provider associated with the connection.<br/>vlan:  The ID of the VLAN.<br/>bandwidth:  The bandwidth of the connection.<br/>location:  The location of the connection.<br/>region:  The AWS Region where the connection is located.<br/>connectionState:  The state of the connection. The following are the possible values: <ul> <li>  <code>ordering</code>: The initial state of a hosted connection provisioned on an interconnect. The connection stays in the ordering state until the owner of the hosted connection confirms or declines the connection order. </li> <li>  <code>requested</code>: The initial state of a standard connection. The connection stays in the requested state until the Letter of Authorization (LOA) is sent to the customer. </li> <li>  <code>pending</code>: The connection has been approved and is being initialized. </li> <li>  <code>available</code>: The network link is up and the connection is ready for use. </li> <li>  <code>down</code>: The network link is down. </li> <li>  <code>deleting</code>: The connection is being deleted. </li> <li>  <code>deleted</code>: The connection has been deleted. </li> <li>  <code>rejected</code>: A hosted connection in the <code>ordering</code> state enters the <code>rejected</code> state if it is deleted by the customer. </li> <li>  <code>unknown</code>: The state of the connection is not available. </li> </ul><br/>connectionName:  The name of the connection.<br/>connectionId:  The ID of the connection.<br/>ownerAccount:  The ID of the AWS account that owns the connection.<br/>        """
        pass

    def allocate_hosted_connection(self, vlan: int, connectionName: str, bandwidth: str, ownerAccount: str, connectionId: str, tags: List[Tag]=None) -> Connection:
        """Creates a hosted connection on the specified interconnect or a link aggregation group (LAG) of interconnects. Allocates a VLAN number and a specified amount of capacity (bandwidth) for use by a hosted connection on the specified interconnect or LAG of interconnects. AWS polices the hosted connection for the specified capacity and the AWS Direct Connect Partner must also police the hosted connection for the specified capacity. <note> Intended for use by AWS Direct Connect Partners only. </note>

        :param self:
        :param tags:  The tags to assign to the hosted connection.<br/>vlan:  The dedicated VLAN provisioned to the hosted connection.<br/>connectionName:  The name of the hosted connection.<br/>bandwidth:  The bandwidth of the connection. The possible values are 50Mbps, 100Mbps, 200Mbps, 300Mbps, 400Mbps, 500Mbps, 1Gbps, 2Gbps, 5Gbps, and 10Gbps. Note that only those AWS Direct Connect Partners who have met specific requirements are allowed to create a 1Gbps, 2Gbps, 5Gbps or 10Gbps hosted connection. <br/>ownerAccount:  The ID of the AWS account ID of the customer for the connection.<br/>connectionId:  The ID of the interconnect or LAG.<br/>
        :return: tags:  Any tags assigned to the connection.<br/>hasLogicalRedundancy:  Indicates whether the connection supports a secondary BGP peer in the same address family (IPv4/IPv6).<br/>awsDeviceV2:  The Direct Connect endpoint on which the physical connection terminates.<br/>jumboFrameCapable:  Indicates whether jumbo frames (9001 MTU) are supported.<br/>awsDevice:  The Direct Connect endpoint on which the physical connection terminates.<br/>lagId:  The ID of the LAG.<br/>loaIssueTime:  The time of the most recent call to <a>DescribeLoa</a> for this connection.<br/>partnerName:  The name of the AWS Direct Connect service provider associated with the connection.<br/>vlan:  The ID of the VLAN.<br/>bandwidth:  The bandwidth of the connection.<br/>location:  The location of the connection.<br/>region:  The AWS Region where the connection is located.<br/>connectionState:  The state of the connection. The following are the possible values: <ul> <li>  <code>ordering</code>: The initial state of a hosted connection provisioned on an interconnect. The connection stays in the ordering state until the owner of the hosted connection confirms or declines the connection order. </li> <li>  <code>requested</code>: The initial state of a standard connection. The connection stays in the requested state until the Letter of Authorization (LOA) is sent to the customer. </li> <li>  <code>pending</code>: The connection has been approved and is being initialized. </li> <li>  <code>available</code>: The network link is up and the connection is ready for use. </li> <li>  <code>down</code>: The network link is down. </li> <li>  <code>deleting</code>: The connection is being deleted. </li> <li>  <code>deleted</code>: The connection has been deleted. </li> <li>  <code>rejected</code>: A hosted connection in the <code>ordering</code> state enters the <code>rejected</code> state if it is deleted by the customer. </li> <li>  <code>unknown</code>: The state of the connection is not available. </li> </ul><br/>connectionName:  The name of the connection.<br/>connectionId:  The ID of the connection.<br/>ownerAccount:  The ID of the AWS account that owns the connection.<br/>        """
        pass

    def allocate_private_virtual_interface(self, newPrivateVirtualInterfaceAllocation: NewPrivateVirtualInterfaceAllocation, ownerAccount: str, connectionId: str) -> VirtualInterface:
        """Provisions a private virtual interface to be owned by the specified AWS account. Virtual interfaces created using this action must be confirmed by the owner using <a>ConfirmPrivateVirtualInterface</a>. Until then, the virtual interface is in the <code>Confirming</code> state and is not available to handle traffic.

        :param self:
        :param newPrivateVirtualInterfaceAllocation:  Information about the private virtual interface.:<br/>ownerAccount:  The ID of the AWS account that owns the virtual private interface.<br/>connectionId:  The ID of the connection on which the private virtual interface is provisioned.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> Any tags assigned to the private virtual interface to be provisioned on a connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customerAddress</i> The IP address assigned to the customer interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>addressFamily</i> The address family for the BGP peer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>amazonAddress</i> The IP address assigned to the Amazon interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authKey</i> The authentication key for BGP configuration. This string has a minimum length of 6 characters and and a maximun lenth of 80 characters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>mtu</i> The maximum transmission unit (MTU), in bytes. The supported values are 1500 and 9001. The default value is 1500.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>asn</i> The autonomous system (AS) number for Border Gateway Protocol (BGP) configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vlan</i> The ID of the VLAN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualInterfaceName</i> The name of the virtual interface assigned by the customer network.<br/>
        :return: tags:  Any tags assigned to the virtual interface.<br/>awsDeviceV2:  The Direct Connect endpoint on which the virtual interface terminates.<br/>region:  The AWS Region where the virtual interface is located.<br/>bgpPeers:  The BGP peers configured on this virtual interface.<br/>routeFilterPrefixes:  The routes to be advertised to the AWS network in this Region. Applies to public virtual interfaces.<br/>directConnectGatewayId:  The ID of the Direct Connect gateway.<br/>virtualGatewayId:  The ID of the virtual private gateway. Applies only to private virtual interfaces.<br/>jumboFrameCapable:  Indicates whether jumbo frames (9001 MTU) are supported.<br/>mtu:  The maximum transmission unit (MTU), in bytes. The supported values are 1500 and 9001. The default value is 1500.<br/>customerRouterConfig:  The customer router configuration.<br/>virtualInterfaceState:  The state of the virtual interface. The following are the possible values: <ul> <li>  <code>confirming</code>: The creation of the virtual interface is pending confirmation from the virtual interface owner. If the owner of the virtual interface is different from the owner of the connection on which it is provisioned, then the virtual interface will remain in this state until it is confirmed by the virtual interface owner. </li> <li>  <code>verifying</code>: This state only applies to public virtual interfaces. Each public virtual interface needs validation before the virtual interface can be created. </li> <li>  <code>pending</code>: A virtual interface is in this state from the time that it is created until the virtual interface is ready to forward traffic. </li> <li>  <code>available</code>: A virtual interface that is able to forward traffic. </li> <li>  <code>down</code>: A virtual interface that is BGP down. </li> <li>  <code>deleting</code>: A virtual interface is in this state immediately after calling <a>DeleteVirtualInterface</a> until it can no longer forward traffic. </li> <li>  <code>deleted</code>: A virtual interface that cannot forward traffic. </li> <li>  <code>rejected</code>: The virtual interface owner has declined creation of the virtual interface. If a virtual interface in the <code>Confirming</code> state is deleted by the virtual interface owner, the virtual interface enters the <code>Rejected</code> state. </li> <li>  <code>unknown</code>: The state of the virtual interface is not available. </li> </ul><br/>addressFamily:  The address family for the BGP peer.<br/>customerAddress:  The IP address assigned to the customer interface.<br/>amazonAddress:  The IP address assigned to the Amazon interface.<br/>authKey:  The authentication key for BGP configuration. This string has a minimum length of 6 characters and and a maximun lenth of 80 characters.<br/>amazonSideAsn:  The autonomous system number (ASN) for the Amazon side of the connection.<br/>asn:  The autonomous system (AS) number for Border Gateway Protocol (BGP) configuration.<br/>vlan:  The ID of the VLAN.<br/>virtualInterfaceName:  The name of the virtual interface assigned by the customer network.<br/>virtualInterfaceType:  The type of virtual interface. The possible values are <code>private</code> and <code>public</code>.<br/>connectionId:  The ID of the connection.<br/>location:  The location of the connection.<br/>virtualInterfaceId:  The ID of the virtual interface.<br/>ownerAccount:  The ID of the AWS account that owns the virtual interface.<br/>        """
        pass

    def allocate_public_virtual_interface(self, newPublicVirtualInterfaceAllocation: NewPublicVirtualInterfaceAllocation, ownerAccount: str, connectionId: str) -> VirtualInterface:
        """Provisions a public virtual interface to be owned by the specified AWS account. The owner of a connection calls this function to provision a public virtual interface to be owned by the specified AWS account. Virtual interfaces created using this function must be confirmed by the owner using <a>ConfirmPublicVirtualInterface</a>. Until this step has been completed, the virtual interface is in the <code>confirming</code> state and is not available to handle traffic. When creating an IPv6 public virtual interface, omit the Amazon address and customer address. IPv6 addresses are automatically assigned from the Amazon pool of IPv6 addresses; you cannot specify custom IPv6 addresses.

        :param self:
        :param newPublicVirtualInterfaceAllocation:  Information about the public virtual interface.:<br/>ownerAccount:  The ID of the AWS account that owns the public virtual interface.<br/>connectionId:  The ID of the connection on which the public virtual interface is provisioned.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> Any tags assigned to the public virtual interface to be provisioned on a connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>routeFilterPrefixes</i> The routes to be advertised to the AWS network in this Region. Applies to public virtual interfaces.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>addressFamily</i> The address family for the BGP peer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customerAddress</i> The IP address assigned to the customer interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>amazonAddress</i> The IP address assigned to the Amazon interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authKey</i> The authentication key for BGP configuration. This string has a minimum length of 6 characters and and a maximun lenth of 80 characters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>asn</i> The autonomous system (AS) number for Border Gateway Protocol (BGP) configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vlan</i> The ID of the VLAN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualInterfaceName</i> The name of the virtual interface assigned by the customer network.<br/>
        :return: tags:  Any tags assigned to the virtual interface.<br/>awsDeviceV2:  The Direct Connect endpoint on which the virtual interface terminates.<br/>region:  The AWS Region where the virtual interface is located.<br/>bgpPeers:  The BGP peers configured on this virtual interface.<br/>routeFilterPrefixes:  The routes to be advertised to the AWS network in this Region. Applies to public virtual interfaces.<br/>directConnectGatewayId:  The ID of the Direct Connect gateway.<br/>virtualGatewayId:  The ID of the virtual private gateway. Applies only to private virtual interfaces.<br/>jumboFrameCapable:  Indicates whether jumbo frames (9001 MTU) are supported.<br/>mtu:  The maximum transmission unit (MTU), in bytes. The supported values are 1500 and 9001. The default value is 1500.<br/>customerRouterConfig:  The customer router configuration.<br/>virtualInterfaceState:  The state of the virtual interface. The following are the possible values: <ul> <li>  <code>confirming</code>: The creation of the virtual interface is pending confirmation from the virtual interface owner. If the owner of the virtual interface is different from the owner of the connection on which it is provisioned, then the virtual interface will remain in this state until it is confirmed by the virtual interface owner. </li> <li>  <code>verifying</code>: This state only applies to public virtual interfaces. Each public virtual interface needs validation before the virtual interface can be created. </li> <li>  <code>pending</code>: A virtual interface is in this state from the time that it is created until the virtual interface is ready to forward traffic. </li> <li>  <code>available</code>: A virtual interface that is able to forward traffic. </li> <li>  <code>down</code>: A virtual interface that is BGP down. </li> <li>  <code>deleting</code>: A virtual interface is in this state immediately after calling <a>DeleteVirtualInterface</a> until it can no longer forward traffic. </li> <li>  <code>deleted</code>: A virtual interface that cannot forward traffic. </li> <li>  <code>rejected</code>: The virtual interface owner has declined creation of the virtual interface. If a virtual interface in the <code>Confirming</code> state is deleted by the virtual interface owner, the virtual interface enters the <code>Rejected</code> state. </li> <li>  <code>unknown</code>: The state of the virtual interface is not available. </li> </ul><br/>addressFamily:  The address family for the BGP peer.<br/>customerAddress:  The IP address assigned to the customer interface.<br/>amazonAddress:  The IP address assigned to the Amazon interface.<br/>authKey:  The authentication key for BGP configuration. This string has a minimum length of 6 characters and and a maximun lenth of 80 characters.<br/>amazonSideAsn:  The autonomous system number (ASN) for the Amazon side of the connection.<br/>asn:  The autonomous system (AS) number for Border Gateway Protocol (BGP) configuration.<br/>vlan:  The ID of the VLAN.<br/>virtualInterfaceName:  The name of the virtual interface assigned by the customer network.<br/>virtualInterfaceType:  The type of virtual interface. The possible values are <code>private</code> and <code>public</code>.<br/>connectionId:  The ID of the connection.<br/>location:  The location of the connection.<br/>virtualInterfaceId:  The ID of the virtual interface.<br/>ownerAccount:  The ID of the AWS account that owns the virtual interface.<br/>        """
        pass

    def allocate_transit_virtual_interface(self, newTransitVirtualInterfaceAllocation: NewTransitVirtualInterfaceAllocation, ownerAccount: str, connectionId: str) -> AllocateTransitVirtualInterfaceResult:
        """Provisions a transit virtual interface to be owned by the specified AWS account. Use this type of interface to connect a transit gateway to your Direct Connect gateway. The owner of a connection provisions a transit virtual interface to be owned by the specified AWS account. After you create a transit virtual interface, it must be confirmed by the owner using <a>ConfirmTransitVirtualInterface</a>. Until this step has been completed, the transit virtual interface is in the <code>requested</code> state and is not available to handle traffic.

        :param self:
        :param newTransitVirtualInterfaceAllocation:  Information about the transit virtual interface.:<br/>ownerAccount:  The ID of the AWS account that owns the transit virtual interface.<br/>connectionId:  The ID of the connection on which the transit virtual interface is provisioned.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> Any tags assigned to the transit virtual interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>addressFamily</i> The address family for the BGP peer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customerAddress</i> The IP address assigned to the customer interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>amazonAddress</i> The IP address assigned to the Amazon interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authKey</i> The authentication key for BGP configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>mtu</i> The maximum transmission unit (MTU), in bytes. The supported values are 1500 and 8500. The default value is 1500. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>asn</i> The autonomous system (AS) number for Border Gateway Protocol (BGP) configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vlan</i> The ID of the VLAN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualInterfaceName</i> The name of the virtual interface assigned by the customer network.<br/>
        :return: virtualInterface:  Information about a virtual interface.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> Any tags assigned to the virtual interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>awsDeviceV2</i> The Direct Connect endpoint on which the virtual interface terminates.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>region</i> The AWS Region where the virtual interface is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>bgpPeers</i> The BGP peers configured on this virtual interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>routeFilterPrefixes</i> The routes to be advertised to the AWS network in this Region. Applies to public virtual interfaces.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>directConnectGatewayId</i> The ID of the Direct Connect gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualGatewayId</i> The ID of the virtual private gateway. Applies only to private virtual interfaces.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jumboFrameCapable</i> Indicates whether jumbo frames (9001 MTU) are supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>mtu</i> The maximum transmission unit (MTU), in bytes. The supported values are 1500 and 9001. The default value is 1500.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customerRouterConfig</i> The customer router configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualInterfaceState</i> The state of the virtual interface. The following are the possible values: <ul> <li>  <code>confirming</code>: The creation of the virtual interface is pending confirmation from the virtual interface owner. If the owner of the virtual interface is different from the owner of the connection on which it is provisioned, then the virtual interface will remain in this state until it is confirmed by the virtual interface owner. </li> <li>  <code>verifying</code>: This state only applies to public virtual interfaces. Each public virtual interface needs validation before the virtual interface can be created. </li> <li>  <code>pending</code>: A virtual interface is in this state from the time that it is created until the virtual interface is ready to forward traffic. </li> <li>  <code>available</code>: A virtual interface that is able to forward traffic. </li> <li>  <code>down</code>: A virtual interface that is BGP down. </li> <li>  <code>deleting</code>: A virtual interface is in this state immediately after calling <a>DeleteVirtualInterface</a> until it can no longer forward traffic. </li> <li>  <code>deleted</code>: A virtual interface that cannot forward traffic. </li> <li>  <code>rejected</code>: The virtual interface owner has declined creation of the virtual interface. If a virtual interface in the <code>Confirming</code> state is deleted by the virtual interface owner, the virtual interface enters the <code>Rejected</code> state. </li> <li>  <code>unknown</code>: The state of the virtual interface is not available. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>addressFamily</i> The address family for the BGP peer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customerAddress</i> The IP address assigned to the customer interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>amazonAddress</i> The IP address assigned to the Amazon interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authKey</i> The authentication key for BGP configuration. This string has a minimum length of 6 characters and and a maximun lenth of 80 characters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>amazonSideAsn</i> The autonomous system number (ASN) for the Amazon side of the connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>asn</i> The autonomous system (AS) number for Border Gateway Protocol (BGP) configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vlan</i> The ID of the VLAN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualInterfaceName</i> The name of the virtual interface assigned by the customer network.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualInterfaceType</i> The type of virtual interface. The possible values are <code>private</code> and <code>public</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>connectionId</i> The ID of the connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The location of the connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualInterfaceId</i> The ID of the virtual interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ownerAccount</i> The ID of the AWS account that owns the virtual interface.<br/>        """
        pass

    def associate_connection_with_lag(self, lagId: str, connectionId: str) -> Connection:
        """Associates an existing connection with a link aggregation group (LAG). The connection is interrupted and re-established as a member of the LAG (connectivity to AWS is interrupted). The connection must be hosted on the same AWS Direct Connect endpoint as the LAG, and its bandwidth must match the bandwidth for the LAG. You can re-associate a connection that's currently associated with a different LAG; however, if removing the connection would cause the original LAG to fall below its setting for minimum number of operational connections, the request fails. Any virtual interfaces that are directly associated with the connection are automatically re-associated with the LAG. If the connection was originally associated with a different LAG, the virtual interfaces remain associated with the original LAG. For interconnects, any hosted connections are automatically re-associated with the LAG. If the interconnect was originally associated with a different LAG, the hosted connections remain associated with the original LAG.

        :param self:
        :param lagId:  The ID of the LAG with which to associate the connection.<br/>connectionId:  The ID of the connection.<br/>
        :return: tags:  Any tags assigned to the connection.<br/>hasLogicalRedundancy:  Indicates whether the connection supports a secondary BGP peer in the same address family (IPv4/IPv6).<br/>awsDeviceV2:  The Direct Connect endpoint on which the physical connection terminates.<br/>jumboFrameCapable:  Indicates whether jumbo frames (9001 MTU) are supported.<br/>awsDevice:  The Direct Connect endpoint on which the physical connection terminates.<br/>lagId:  The ID of the LAG.<br/>loaIssueTime:  The time of the most recent call to <a>DescribeLoa</a> for this connection.<br/>partnerName:  The name of the AWS Direct Connect service provider associated with the connection.<br/>vlan:  The ID of the VLAN.<br/>bandwidth:  The bandwidth of the connection.<br/>location:  The location of the connection.<br/>region:  The AWS Region where the connection is located.<br/>connectionState:  The state of the connection. The following are the possible values: <ul> <li>  <code>ordering</code>: The initial state of a hosted connection provisioned on an interconnect. The connection stays in the ordering state until the owner of the hosted connection confirms or declines the connection order. </li> <li>  <code>requested</code>: The initial state of a standard connection. The connection stays in the requested state until the Letter of Authorization (LOA) is sent to the customer. </li> <li>  <code>pending</code>: The connection has been approved and is being initialized. </li> <li>  <code>available</code>: The network link is up and the connection is ready for use. </li> <li>  <code>down</code>: The network link is down. </li> <li>  <code>deleting</code>: The connection is being deleted. </li> <li>  <code>deleted</code>: The connection has been deleted. </li> <li>  <code>rejected</code>: A hosted connection in the <code>ordering</code> state enters the <code>rejected</code> state if it is deleted by the customer. </li> <li>  <code>unknown</code>: The state of the connection is not available. </li> </ul><br/>connectionName:  The name of the connection.<br/>connectionId:  The ID of the connection.<br/>ownerAccount:  The ID of the AWS account that owns the connection.<br/>        """
        pass

    def associate_hosted_connection(self, parentConnectionId: str, connectionId: str) -> Connection:
        """Associates a hosted connection and its virtual interfaces with a link aggregation group (LAG) or interconnect. If the target interconnect or LAG has an existing hosted connection with a conflicting VLAN number or IP address, the operation fails. This action temporarily interrupts the hosted connection's connectivity to AWS as it is being migrated. <note> Intended for use by AWS Direct Connect Partners only. </note>

        :param self:
        :param parentConnectionId:  The ID of the interconnect or the LAG.<br/>connectionId:  The ID of the hosted connection.<br/>
        :return: tags:  Any tags assigned to the connection.<br/>hasLogicalRedundancy:  Indicates whether the connection supports a secondary BGP peer in the same address family (IPv4/IPv6).<br/>awsDeviceV2:  The Direct Connect endpoint on which the physical connection terminates.<br/>jumboFrameCapable:  Indicates whether jumbo frames (9001 MTU) are supported.<br/>awsDevice:  The Direct Connect endpoint on which the physical connection terminates.<br/>lagId:  The ID of the LAG.<br/>loaIssueTime:  The time of the most recent call to <a>DescribeLoa</a> for this connection.<br/>partnerName:  The name of the AWS Direct Connect service provider associated with the connection.<br/>vlan:  The ID of the VLAN.<br/>bandwidth:  The bandwidth of the connection.<br/>location:  The location of the connection.<br/>region:  The AWS Region where the connection is located.<br/>connectionState:  The state of the connection. The following are the possible values: <ul> <li>  <code>ordering</code>: The initial state of a hosted connection provisioned on an interconnect. The connection stays in the ordering state until the owner of the hosted connection confirms or declines the connection order. </li> <li>  <code>requested</code>: The initial state of a standard connection. The connection stays in the requested state until the Letter of Authorization (LOA) is sent to the customer. </li> <li>  <code>pending</code>: The connection has been approved and is being initialized. </li> <li>  <code>available</code>: The network link is up and the connection is ready for use. </li> <li>  <code>down</code>: The network link is down. </li> <li>  <code>deleting</code>: The connection is being deleted. </li> <li>  <code>deleted</code>: The connection has been deleted. </li> <li>  <code>rejected</code>: A hosted connection in the <code>ordering</code> state enters the <code>rejected</code> state if it is deleted by the customer. </li> <li>  <code>unknown</code>: The state of the connection is not available. </li> </ul><br/>connectionName:  The name of the connection.<br/>connectionId:  The ID of the connection.<br/>ownerAccount:  The ID of the AWS account that owns the connection.<br/>        """
        pass

    def associate_virtual_interface(self, connectionId: str, virtualInterfaceId: str) -> VirtualInterface:
        """Associates a virtual interface with a specified link aggregation group (LAG) or connection. Connectivity to AWS is temporarily interrupted as the virtual interface is being migrated. If the target connection or LAG has an associated virtual interface with a conflicting VLAN number or a conflicting IP address, the operation fails. Virtual interfaces associated with a hosted connection cannot be associated with a LAG; hosted connections must be migrated along with their virtual interfaces using <a>AssociateHostedConnection</a>. To reassociate a virtual interface to a new connection or LAG, the requester must own either the virtual interface itself or the connection to which the virtual interface is currently associated. Additionally, the requester must own the connection or LAG for the association.

        :param self:
        :param connectionId:  The ID of the LAG or connection.<br/>virtualInterfaceId:  The ID of the virtual interface.<br/>
        :return: tags:  Any tags assigned to the virtual interface.<br/>awsDeviceV2:  The Direct Connect endpoint on which the virtual interface terminates.<br/>region:  The AWS Region where the virtual interface is located.<br/>bgpPeers:  The BGP peers configured on this virtual interface.<br/>routeFilterPrefixes:  The routes to be advertised to the AWS network in this Region. Applies to public virtual interfaces.<br/>directConnectGatewayId:  The ID of the Direct Connect gateway.<br/>virtualGatewayId:  The ID of the virtual private gateway. Applies only to private virtual interfaces.<br/>jumboFrameCapable:  Indicates whether jumbo frames (9001 MTU) are supported.<br/>mtu:  The maximum transmission unit (MTU), in bytes. The supported values are 1500 and 9001. The default value is 1500.<br/>customerRouterConfig:  The customer router configuration.<br/>virtualInterfaceState:  The state of the virtual interface. The following are the possible values: <ul> <li>  <code>confirming</code>: The creation of the virtual interface is pending confirmation from the virtual interface owner. If the owner of the virtual interface is different from the owner of the connection on which it is provisioned, then the virtual interface will remain in this state until it is confirmed by the virtual interface owner. </li> <li>  <code>verifying</code>: This state only applies to public virtual interfaces. Each public virtual interface needs validation before the virtual interface can be created. </li> <li>  <code>pending</code>: A virtual interface is in this state from the time that it is created until the virtual interface is ready to forward traffic. </li> <li>  <code>available</code>: A virtual interface that is able to forward traffic. </li> <li>  <code>down</code>: A virtual interface that is BGP down. </li> <li>  <code>deleting</code>: A virtual interface is in this state immediately after calling <a>DeleteVirtualInterface</a> until it can no longer forward traffic. </li> <li>  <code>deleted</code>: A virtual interface that cannot forward traffic. </li> <li>  <code>rejected</code>: The virtual interface owner has declined creation of the virtual interface. If a virtual interface in the <code>Confirming</code> state is deleted by the virtual interface owner, the virtual interface enters the <code>Rejected</code> state. </li> <li>  <code>unknown</code>: The state of the virtual interface is not available. </li> </ul><br/>addressFamily:  The address family for the BGP peer.<br/>customerAddress:  The IP address assigned to the customer interface.<br/>amazonAddress:  The IP address assigned to the Amazon interface.<br/>authKey:  The authentication key for BGP configuration. This string has a minimum length of 6 characters and and a maximun lenth of 80 characters.<br/>amazonSideAsn:  The autonomous system number (ASN) for the Amazon side of the connection.<br/>asn:  The autonomous system (AS) number for Border Gateway Protocol (BGP) configuration.<br/>vlan:  The ID of the VLAN.<br/>virtualInterfaceName:  The name of the virtual interface assigned by the customer network.<br/>virtualInterfaceType:  The type of virtual interface. The possible values are <code>private</code> and <code>public</code>.<br/>connectionId:  The ID of the connection.<br/>location:  The location of the connection.<br/>virtualInterfaceId:  The ID of the virtual interface.<br/>ownerAccount:  The ID of the AWS account that owns the virtual interface.<br/>        """
        pass

    def confirm_connection(self, connectionId: str) -> ConfirmConnectionResponse:
        """Confirms the creation of the specified hosted connection on an interconnect. Upon creation, the hosted connection is initially in the <code>Ordering</code> state, and remains in this state until the owner confirms creation of the hosted connection.

        :param self:
        :param connectionId:  The ID of the hosted connection.<br/>
        :return: connectionState:  The state of the connection. The following are the possible values: <ul> <li>  <code>ordering</code>: The initial state of a hosted connection provisioned on an interconnect. The connection stays in the ordering state until the owner of the hosted connection confirms or declines the connection order. </li> <li>  <code>requested</code>: The initial state of a standard connection. The connection stays in the requested state until the Letter of Authorization (LOA) is sent to the customer. </li> <li>  <code>pending</code>: The connection has been approved and is being initialized. </li> <li>  <code>available</code>: The network link is up and the connection is ready for use. </li> <li>  <code>down</code>: The network link is down. </li> <li>  <code>deleting</code>: The connection is being deleted. </li> <li>  <code>deleted</code>: The connection has been deleted. </li> <li>  <code>rejected</code>: A hosted connection in the <code>ordering</code> state enters the <code>rejected</code> state if it is deleted by the customer. </li> <li>  <code>unknown</code>: The state of the connection is not available. </li> </ul><br/>        """
        pass

    def confirm_private_virtual_interface(self, virtualInterfaceId: str, virtualGatewayId: str=None, directConnectGatewayId: str=None) -> ConfirmPrivateVirtualInterfaceResponse:
        """Accepts ownership of a private virtual interface created by another AWS account. After the virtual interface owner makes this call, the virtual interface is created and attached to the specified virtual private gateway or Direct Connect gateway, and is made available to handle traffic.

        :param self:
        :param directConnectGatewayId:  The ID of the Direct Connect gateway.<br/>virtualGatewayId:  The ID of the virtual private gateway.<br/>virtualInterfaceId:  The ID of the virtual interface.<br/>
        :return: virtualInterfaceState:  The state of the virtual interface. The following are the possible values: <ul> <li>  <code>confirming</code>: The creation of the virtual interface is pending confirmation from the virtual interface owner. If the owner of the virtual interface is different from the owner of the connection on which it is provisioned, then the virtual interface will remain in this state until it is confirmed by the virtual interface owner. </li> <li>  <code>verifying</code>: This state only applies to public virtual interfaces. Each public virtual interface needs validation before the virtual interface can be created. </li> <li>  <code>pending</code>: A virtual interface is in this state from the time that it is created until the virtual interface is ready to forward traffic. </li> <li>  <code>available</code>: A virtual interface that is able to forward traffic. </li> <li>  <code>down</code>: A virtual interface that is BGP down. </li> <li>  <code>deleting</code>: A virtual interface is in this state immediately after calling <a>DeleteVirtualInterface</a> until it can no longer forward traffic. </li> <li>  <code>deleted</code>: A virtual interface that cannot forward traffic. </li> <li>  <code>rejected</code>: The virtual interface owner has declined creation of the virtual interface. If a virtual interface in the <code>Confirming</code> state is deleted by the virtual interface owner, the virtual interface enters the <code>Rejected</code> state. </li> <li>  <code>unknown</code>: The state of the virtual interface is not available. </li> </ul><br/>        """
        pass

    def confirm_public_virtual_interface(self, virtualInterfaceId: str) -> ConfirmPublicVirtualInterfaceResponse:
        """Accepts ownership of a public virtual interface created by another AWS account. After the virtual interface owner makes this call, the specified virtual interface is created and made available to handle traffic.

        :param self:
        :param virtualInterfaceId:  The ID of the virtual interface.<br/>
        :return: virtualInterfaceState:  The state of the virtual interface. The following are the possible values: <ul> <li>  <code>confirming</code>: The creation of the virtual interface is pending confirmation from the virtual interface owner. If the owner of the virtual interface is different from the owner of the connection on which it is provisioned, then the virtual interface will remain in this state until it is confirmed by the virtual interface owner. </li> <li>  <code>verifying</code>: This state only applies to public virtual interfaces. Each public virtual interface needs validation before the virtual interface can be created. </li> <li>  <code>pending</code>: A virtual interface is in this state from the time that it is created until the virtual interface is ready to forward traffic. </li> <li>  <code>available</code>: A virtual interface that is able to forward traffic. </li> <li>  <code>down</code>: A virtual interface that is BGP down. </li> <li>  <code>deleting</code>: A virtual interface is in this state immediately after calling <a>DeleteVirtualInterface</a> until it can no longer forward traffic. </li> <li>  <code>deleted</code>: A virtual interface that cannot forward traffic. </li> <li>  <code>rejected</code>: The virtual interface owner has declined creation of the virtual interface. If a virtual interface in the <code>Confirming</code> state is deleted by the virtual interface owner, the virtual interface enters the <code>Rejected</code> state. </li> <li>  <code>unknown</code>: The state of the virtual interface is not available. </li> </ul><br/>        """
        pass

    def confirm_transit_virtual_interface(self, directConnectGatewayId: str, virtualInterfaceId: str) -> ConfirmTransitVirtualInterfaceResponse:
        """Accepts ownership of a transit virtual interface created by another AWS account.  After the owner of the transit virtual interface makes this call, the specified transit virtual interface is created and made available to handle traffic.

        :param self:
        :param directConnectGatewayId:  The ID of the Direct Connect gateway.<br/>virtualInterfaceId:  The ID of the virtual interface.<br/>
        :return: virtualInterfaceState:  The state of the virtual interface. The following are the possible values: <ul> <li>  <code>confirming</code>: The creation of the virtual interface is pending confirmation from the virtual interface owner. If the owner of the virtual interface is different from the owner of the connection on which it is provisioned, then the virtual interface will remain in this state until it is confirmed by the virtual interface owner. </li> <li>  <code>verifying</code>: This state only applies to public virtual interfaces. Each public virtual interface needs validation before the virtual interface can be created. </li> <li>  <code>pending</code>: A virtual interface is in this state from the time that it is created until the virtual interface is ready to forward traffic. </li> <li>  <code>available</code>: A virtual interface that is able to forward traffic. </li> <li>  <code>down</code>: A virtual interface that is BGP down. </li> <li>  <code>deleting</code>: A virtual interface is in this state immediately after calling <a>DeleteVirtualInterface</a> until it can no longer forward traffic. </li> <li>  <code>deleted</code>: A virtual interface that cannot forward traffic. </li> <li>  <code>rejected</code>: The virtual interface owner has declined creation of the virtual interface. If a virtual interface in the <code>Confirming</code> state is deleted by the virtual interface owner, the virtual interface enters the <code>Rejected</code> state. </li> <li>  <code>unknown</code>: The state of the virtual interface is not available. </li> </ul><br/>        """
        pass

    def create_bgp_peer(self, virtualInterfaceId: str=None, newBGPPeer: NewBGPPeer=None) -> CreateBGPPeerResponse:
        """Creates a BGP peer on the specified virtual interface. You must create a BGP peer for the corresponding address family (IPv4/IPv6) in order to access AWS resources that also use that address family. If logical redundancy is not supported by the connection, interconnect, or LAG, the BGP peer cannot be in the same address family as an existing BGP peer on the virtual interface. When creating a IPv6 BGP peer, omit the Amazon address and customer address. IPv6 addresses are automatically assigned from the Amazon pool of IPv6 addresses; you cannot specify custom IPv6 addresses. For a public virtual interface, the Autonomous System Number (ASN) must be private or already whitelisted for the virtual interface.

        :param self:
        :param newBGPPeer:  Information about the BGP peer.:<br/>virtualInterfaceId:  The ID of the virtual interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customerAddress</i> The IP address assigned to the customer interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>amazonAddress</i> The IP address assigned to the Amazon interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>addressFamily</i> The address family for the BGP peer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authKey</i> The authentication key for BGP configuration. This string has a minimum length of 6 characters and and a maximun lenth of 80 characters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>asn</i> The autonomous system (AS) number for Border Gateway Protocol (BGP) configuration.<br/>
        :return: virtualInterface:  The virtual interface.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> Any tags assigned to the virtual interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>awsDeviceV2</i> The Direct Connect endpoint on which the virtual interface terminates.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>region</i> The AWS Region where the virtual interface is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>bgpPeers</i> The BGP peers configured on this virtual interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>routeFilterPrefixes</i> The routes to be advertised to the AWS network in this Region. Applies to public virtual interfaces.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>directConnectGatewayId</i> The ID of the Direct Connect gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualGatewayId</i> The ID of the virtual private gateway. Applies only to private virtual interfaces.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jumboFrameCapable</i> Indicates whether jumbo frames (9001 MTU) are supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>mtu</i> The maximum transmission unit (MTU), in bytes. The supported values are 1500 and 9001. The default value is 1500.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customerRouterConfig</i> The customer router configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualInterfaceState</i> The state of the virtual interface. The following are the possible values: <ul> <li>  <code>confirming</code>: The creation of the virtual interface is pending confirmation from the virtual interface owner. If the owner of the virtual interface is different from the owner of the connection on which it is provisioned, then the virtual interface will remain in this state until it is confirmed by the virtual interface owner. </li> <li>  <code>verifying</code>: This state only applies to public virtual interfaces. Each public virtual interface needs validation before the virtual interface can be created. </li> <li>  <code>pending</code>: A virtual interface is in this state from the time that it is created until the virtual interface is ready to forward traffic. </li> <li>  <code>available</code>: A virtual interface that is able to forward traffic. </li> <li>  <code>down</code>: A virtual interface that is BGP down. </li> <li>  <code>deleting</code>: A virtual interface is in this state immediately after calling <a>DeleteVirtualInterface</a> until it can no longer forward traffic. </li> <li>  <code>deleted</code>: A virtual interface that cannot forward traffic. </li> <li>  <code>rejected</code>: The virtual interface owner has declined creation of the virtual interface. If a virtual interface in the <code>Confirming</code> state is deleted by the virtual interface owner, the virtual interface enters the <code>Rejected</code> state. </li> <li>  <code>unknown</code>: The state of the virtual interface is not available. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>addressFamily</i> The address family for the BGP peer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customerAddress</i> The IP address assigned to the customer interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>amazonAddress</i> The IP address assigned to the Amazon interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authKey</i> The authentication key for BGP configuration. This string has a minimum length of 6 characters and and a maximun lenth of 80 characters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>amazonSideAsn</i> The autonomous system number (ASN) for the Amazon side of the connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>asn</i> The autonomous system (AS) number for Border Gateway Protocol (BGP) configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vlan</i> The ID of the VLAN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualInterfaceName</i> The name of the virtual interface assigned by the customer network.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualInterfaceType</i> The type of virtual interface. The possible values are <code>private</code> and <code>public</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>connectionId</i> The ID of the connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The location of the connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualInterfaceId</i> The ID of the virtual interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ownerAccount</i> The ID of the AWS account that owns the virtual interface.<br/>        """
        pass

    def create_connection(self, connectionName: str, bandwidth: str, location: str, lagId: str=None, tags: List[Tag]=None) -> Connection:
        """Creates a connection between a customer network and a specific AWS Direct Connect location. A connection links your internal network to an AWS Direct Connect location over a standard Ethernet fiber-optic cable. One end of the cable is connected to your router, the other to an AWS Direct Connect router. To find the locations for your Region, use <a>DescribeLocations</a>. You can automatically add the new connection to a link aggregation group (LAG) by specifying a LAG ID in the request. This ensures that the new connection is allocated on the same AWS Direct Connect endpoint that hosts the specified LAG. If there are no available ports on the endpoint, the request fails and no connection is created.

        :param self:
        :param tags:  The tags to assign to the connection.<br/>lagId:  The ID of the LAG.<br/>connectionName:  The name of the connection.<br/>bandwidth:  The bandwidth of the connection.<br/>location:  The location of the connection.<br/>
        :return: tags:  Any tags assigned to the connection.<br/>hasLogicalRedundancy:  Indicates whether the connection supports a secondary BGP peer in the same address family (IPv4/IPv6).<br/>awsDeviceV2:  The Direct Connect endpoint on which the physical connection terminates.<br/>jumboFrameCapable:  Indicates whether jumbo frames (9001 MTU) are supported.<br/>awsDevice:  The Direct Connect endpoint on which the physical connection terminates.<br/>lagId:  The ID of the LAG.<br/>loaIssueTime:  The time of the most recent call to <a>DescribeLoa</a> for this connection.<br/>partnerName:  The name of the AWS Direct Connect service provider associated with the connection.<br/>vlan:  The ID of the VLAN.<br/>bandwidth:  The bandwidth of the connection.<br/>location:  The location of the connection.<br/>region:  The AWS Region where the connection is located.<br/>connectionState:  The state of the connection. The following are the possible values: <ul> <li>  <code>ordering</code>: The initial state of a hosted connection provisioned on an interconnect. The connection stays in the ordering state until the owner of the hosted connection confirms or declines the connection order. </li> <li>  <code>requested</code>: The initial state of a standard connection. The connection stays in the requested state until the Letter of Authorization (LOA) is sent to the customer. </li> <li>  <code>pending</code>: The connection has been approved and is being initialized. </li> <li>  <code>available</code>: The network link is up and the connection is ready for use. </li> <li>  <code>down</code>: The network link is down. </li> <li>  <code>deleting</code>: The connection is being deleted. </li> <li>  <code>deleted</code>: The connection has been deleted. </li> <li>  <code>rejected</code>: A hosted connection in the <code>ordering</code> state enters the <code>rejected</code> state if it is deleted by the customer. </li> <li>  <code>unknown</code>: The state of the connection is not available. </li> </ul><br/>connectionName:  The name of the connection.<br/>connectionId:  The ID of the connection.<br/>ownerAccount:  The ID of the AWS account that owns the connection.<br/>        """
        pass

    def create_direct_connect_gateway(self, directConnectGatewayName: str, amazonSideAsn: LongAsn=None) -> CreateDirectConnectGatewayResult:
        """Creates a Direct Connect gateway, which is an intermediate object that enables you to connect a set of virtual interfaces and virtual private gateways. A Direct Connect gateway is global and visible in any AWS Region after it is created. The virtual interfaces and virtual private gateways that are connected through a Direct Connect gateway can be in different AWS Regions. This enables you to connect to a VPC in any Region, regardless of the Region in which the virtual interfaces are located, and pass traffic between them.

        :param self:
        :param amazonSideAsn:  The autonomous system number (ASN) for Border Gateway Protocol (BGP) to be configured on the Amazon side of the connection. The ASN must be in the private range of 64,512 to 65,534 or 4,200,000,000 to 4,294,967,294. The default is 64512.<br/>directConnectGatewayName:  The name of the Direct Connect gateway.<br/>
        :return: directConnectGateway:  The Direct Connect gateway.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stateChangeError</i> The error message if the state of an object failed to advance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>directConnectGatewayState</i> The state of the Direct Connect gateway. The following are the possible values: <ul> <li>  <code>pending</code>: The initial state after calling <a>CreateDirectConnectGateway</a>. </li> <li>  <code>available</code>: The Direct Connect gateway is ready for use. </li> <li>  <code>deleting</code>: The initial state after calling <a>DeleteDirectConnectGateway</a>. </li> <li>  <code>deleted</code>: The Direct Connect gateway is deleted and cannot pass traffic. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ownerAccount</i> The ID of the AWS account that owns the Direct Connect gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>amazonSideAsn</i> The autonomous system number (ASN) for the Amazon side of the connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>directConnectGatewayName</i> The name of the Direct Connect gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>directConnectGatewayId</i> The ID of the Direct Connect gateway.<br/>        """
        pass

    def create_direct_connect_gateway_association(self, directConnectGatewayId: str, gatewayId: str=None, addAllowedPrefixesToDirectConnectGateway: List[RouteFilterPrefix]=None, virtualGatewayId: str=None) -> CreateDirectConnectGatewayAssociationResult:
        """Creates an association between a Direct Connect gateway and a virtual private gateway. The virtual private gateway must be attached to a VPC and must not be associated with another Direct Connect gateway.

        :param self:
        :param virtualGatewayId:  The ID of the virtual private gateway.<br/>addAllowedPrefixesToDirectConnectGateway:  The Amazon VPC prefixes to advertise to the Direct Connect gateway For information about how to set the prefixes, see <a href="https://docs.aws.amazon.com/directconnect/latest/UserGuide/multi-account-associate-vgw.html#allowed-prefixes">Allowed Prefixes</a> in the <i>AWS Direct Connect User Guide</i>.<br/>gatewayId:  The ID of the virtual private gateway or transit gateway.<br/>directConnectGatewayId:  The ID of the Direct Connect gateway.<br/>
        :return: directConnectGatewayAssociation:  The association to be created.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualGatewayOwnerAccount</i> The ID of the AWS account that owns the virtual private gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualGatewayRegion</i> The AWS Region where the virtual private gateway is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualGatewayId</i> The ID of the virtual private gateway. Applies only to private virtual interfaces.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>allowedPrefixesToDirectConnectGateway</i> The Amazon VPC prefixes to advertise to the Direct Connect gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>associationId</i> The ID of the Direct Connect gateway association.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>associatedGateway</i> Information about the associated gateway.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>associatedGateway</b>: Information about the associated gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stateChangeError</i> The error message if the state of an object failed to advance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>associationState</i> The state of the association. The following are the possible values: <ul> <li>  <code>associating</code>: The initial state after calling <a>CreateDirectConnectGatewayAssociation</a>. </li> <li>  <code>associated</code>: The Direct Connect gateway and virtual private gateway or transit gateway are successfully associated and ready to pass traffic. </li> <li>  <code>disassociating</code>: The initial state after calling <a>DeleteDirectConnectGatewayAssociation</a>. </li> <li>  <code>disassociated</code>: The virtual private gateway or transit gateway is disassociated from the Direct Connect gateway. Traffic flow between the Direct Connect gateway and virtual private gateway or transit gateway is stopped. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>directConnectGatewayOwnerAccount</i> The ID of the AWS account that owns the associated gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>directConnectGatewayId</i> The ID of the Direct Connect gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>region</i> The Region where the associated gateway is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ownerAccount</i> The ID of the AWS account that owns the associated virtual private gateway or transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of associated gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the associated gateway.<br/>        """
        pass

    def create_direct_connect_gateway_association_proposal(self, gatewayId: str, directConnectGatewayOwnerAccount: str, directConnectGatewayId: str, addAllowedPrefixesToDirectConnectGateway: List[RouteFilterPrefix]=None, removeAllowedPrefixesToDirectConnectGateway: List[RouteFilterPrefix]=None) -> CreateDirectConnectGatewayAssociationProposalResult:
        """Creates a proposal to associate the specified virtual private gateway or transit gateway with the specified Direct Connect gateway. You can only associate a Direct Connect gateway and virtual private gateway or transit gateway when the account that owns the Direct Connect gateway and the account that owns the virtual private gateway or transit gateway have the same AWS Payer ID.

        :param self:
        :param removeAllowedPrefixesToDirectConnectGateway:  The Amazon VPC prefixes to no longer advertise to the Direct Connect gateway.<br/>addAllowedPrefixesToDirectConnectGateway:  The Amazon VPC prefixes to advertise to the Direct Connect gateway.<br/>gatewayId:  The ID of the virtual private gateway or transit gateway.<br/>directConnectGatewayOwnerAccount:  The ID of the AWS account that owns the Direct Connect gateway.<br/>directConnectGatewayId:  The ID of the Direct Connect gateway.<br/>
        :return: directConnectGatewayAssociationProposal:  Information about the Direct Connect gateway proposal.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>requestedAllowedPrefixesToDirectConnectGateway</i> The Amazon VPC prefixes to advertise to the Direct Connect gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>existingAllowedPrefixesToDirectConnectGateway</i> The existing Amazon VPC prefixes advertised to the Direct Connect gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>associatedGateway</i> Information about the associated gateway.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>associatedGateway</b>: Information about the associated gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>proposalState</i> The state of the proposal. The following are possible values: <ul> <li>  <code>accepted</code>: The proposal has been accepted. The Direct Connect gateway association is available to use in this state. </li> <li>  <code>deleted</code>: The proposal has been deleted by the owner that made the proposal. The Direct Connect gateway association cannot be used in this state. </li> <li>  <code>requested</code>: The proposal has been requested. The Direct Connect gateway association cannot be used in this state. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>directConnectGatewayOwnerAccount</i> The ID of the AWS account that owns the Direct Connect gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>directConnectGatewayId</i> The ID of the Direct Connect gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>proposalId</i> The ID of the association proposal.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>region</i> The Region where the associated gateway is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ownerAccount</i> The ID of the AWS account that owns the associated virtual private gateway or transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of associated gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the associated gateway.<br/>        """
        pass

    def create_interconnect(self, location: str, bandwidth: str, interconnectName: str, lagId: str=None, tags: List[Tag]=None) -> Interconnect:
        """Creates an interconnect between an AWS Direct Connect Partner's network and a specific AWS Direct Connect location. An interconnect is a connection that is capable of hosting other connections. The AWS Direct Connect partner can use an interconnect to provide AWS Direct Connect hosted connections to customers through their own network services. Like a standard connection, an interconnect links the partner's network to an AWS Direct Connect location over a standard Ethernet fiber-optic cable. One end is connected to the partner's router, the other to an AWS Direct Connect router. You can automatically add the new interconnect to a link aggregation group (LAG) by specifying a LAG ID in the request. This ensures that the new interconnect is allocated on the same AWS Direct Connect endpoint that hosts the specified LAG. If there are no available ports on the endpoint, the request fails and no interconnect is created. For each end customer, the AWS Direct Connect Partner provisions a connection on their interconnect by calling <a>AllocateHostedConnection</a>. The end customer can then connect to AWS resources by creating a virtual interface on their connection, using the VLAN assigned to them by the AWS Direct Connect Partner. <note> Intended for use by AWS Direct Connect Partners only. </note>

        :param self:
        :param tags:  The tags to assign to the interconnect,<br/>lagId:  The ID of the LAG.<br/>location:  The location of the interconnect.<br/>bandwidth:  The port bandwidth, in Gbps. The possible values are 1 and 10.<br/>interconnectName:  The name of the interconnect.<br/>
        :return: tags:  Any tags assigned to the interconnect.<br/>hasLogicalRedundancy:  Indicates whether the interconnect supports a secondary BGP in the same address family (IPv4/IPv6).<br/>awsDeviceV2:  The Direct Connect endpoint on which the physical connection terminates.<br/>jumboFrameCapable:  Indicates whether jumbo frames (9001 MTU) are supported.<br/>awsDevice:  The Direct Connect endpoint on which the physical connection terminates.<br/>lagId:  The ID of the LAG.<br/>loaIssueTime:  The time of the most recent call to <a>DescribeLoa</a> for this connection.<br/>bandwidth:  The bandwidth of the connection.<br/>location:  The location of the connection.<br/>region:  The AWS Region where the connection is located.<br/>interconnectState:  The state of the interconnect. The following are the possible values: <ul> <li>  <code>requested</code>: The initial state of an interconnect. The interconnect stays in the requested state until the Letter of Authorization (LOA) is sent to the customer. </li> <li>  <code>pending</code>: The interconnect is approved, and is being initialized. </li> <li>  <code>available</code>: The network link is up, and the interconnect is ready for use. </li> <li>  <code>down</code>: The network link is down. </li> <li>  <code>deleting</code>: The interconnect is being deleted. </li> <li>  <code>deleted</code>: The interconnect is deleted. </li> <li>  <code>unknown</code>: The state of the interconnect is not available. </li> </ul><br/>interconnectName:  The name of the interconnect.<br/>interconnectId:  The ID of the interconnect.<br/>        """
        pass

    def create_lag(self, lagName: str, connectionsBandwidth: str, location: str, numberOfConnections: int, connectionId: str=None, tags: List[Tag]=None, childConnectionTags: List[Tag]=None) -> Lag:
        """Creates a link aggregation group (LAG) with the specified number of bundled physical connections between the customer network and a specific AWS Direct Connect location. A LAG is a logical interface that uses the Link Aggregation Control Protocol (LACP) to aggregate multiple interfaces, enabling you to treat them as a single interface. All connections in a LAG must use the same bandwidth and must terminate at the same AWS Direct Connect endpoint. You can have up to 10 connections per LAG. Regardless of this limit, if you request more connections for the LAG than AWS Direct Connect can allocate on a single endpoint, no LAG is created. You can specify an existing physical connection or interconnect to include in the LAG (which counts towards the total number of connections). Doing so interrupts the current physical connection or hosted connections, and re-establishes them as a member of the LAG. The LAG will be created on the same AWS Direct Connect endpoint to which the connection terminates. Any virtual interfaces associated with the connection are automatically disassociated and re-associated with the LAG. The connection ID does not change. If the AWS account used to create a LAG is a registered AWS Direct Connect Partner, the LAG is automatically enabled to host sub-connections. For a LAG owned by a partner, any associated virtual interfaces cannot be directly configured.

        :param self:
        :param childConnectionTags:  The tags to assign to the child connections of the LAG. Only newly created child connections as the result of creating a LAG connection are assigned the provided tags. The tags are not assigned to an existing connection that is provided via the “connectionId” parameter that will be migrated to the LAG.<br/>tags:  The tags to assign to the link aggregation group (LAG).<br/>connectionId:  The ID of an existing connection to migrate to the LAG.<br/>lagName:  The name of the LAG.<br/>connectionsBandwidth:  The bandwidth of the individual physical connections bundled by the LAG. The possible values are 50Mbps, 100Mbps, 200Mbps, 300Mbps, 400Mbps, 500Mbps, 1Gbps, 2Gbps, 5Gbps, and 10Gbps. <br/>location:  The location for the LAG.<br/>numberOfConnections:  The number of physical connections initially provisioned and bundled by the LAG.<br/>
        :return: tags:  Any tags assigned to link aggregation group (LAG).<br/>hasLogicalRedundancy:  Indicates whether the LAG supports a secondary BGP peer in the same address family (IPv4/IPv6).<br/>jumboFrameCapable:  Indicates whether jumbo frames (9001 MTU) are supported.<br/>allowsHostedConnections:  Indicates whether the LAG can host other connections.<br/>connections:  The connections bundled by the LAG.<br/>awsDeviceV2:  The AWS Direct Connect endpoint that hosts the LAG.<br/>awsDevice:  The AWS Direct Connect endpoint that hosts the LAG.<br/>minimumLinks:  The minimum number of physical connections that must be operational for the LAG itself to be operational.<br/>region:  The AWS Region where the connection is located.<br/>location:  The location of the LAG.<br/>lagState:  The state of the LAG. The following are the possible values: <ul> <li>  <code>requested</code>: The initial state of a LAG. The LAG stays in the requested state until the Letter of Authorization (LOA) is available. </li> <li>  <code>pending</code>: The LAG has been approved and is being initialized. </li> <li>  <code>available</code>: The network link is established and the LAG is ready for use. </li> <li>  <code>down</code>: The network link is down. </li> <li>  <code>deleting</code>: The LAG is being deleted. </li> <li>  <code>deleted</code>: The LAG is deleted. </li> <li>  <code>unknown</code>: The state of the LAG is not available. </li> </ul><br/>lagName:  The name of the LAG.<br/>ownerAccount:  The ID of the AWS account that owns the LAG.<br/>lagId:  The ID of the LAG.<br/>numberOfConnections:  The number of physical connections bundled by the LAG, up to a maximum of 10.<br/>connectionsBandwidth:  The individual bandwidth of the physical connections bundled by the LAG. The possible values are 1Gbps and 10Gbps. <br/>        """
        pass

    def create_private_virtual_interface(self, newPrivateVirtualInterface: NewPrivateVirtualInterface, connectionId: str) -> VirtualInterface:
        """Creates a private virtual interface. A virtual interface is the VLAN that transports AWS Direct Connect traffic. A private virtual interface can be connected to either a Direct Connect gateway or a Virtual Private Gateway (VGW). Connecting the private virtual interface to a Direct Connect gateway enables the possibility for connecting to multiple VPCs, including VPCs in different AWS Regions. Connecting the private virtual interface to a VGW only provides access to a single VPC within the same Region.

        :param self:
        :param newPrivateVirtualInterface:  Information about the private virtual interface.:<br/>connectionId:  The ID of the connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> Any tags assigned to the private virtual interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>directConnectGatewayId</i> The ID of the Direct Connect gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualGatewayId</i> The ID of the virtual private gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>addressFamily</i> The address family for the BGP peer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customerAddress</i> The IP address assigned to the customer interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>amazonAddress</i> The IP address assigned to the Amazon interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authKey</i> The authentication key for BGP configuration. This string has a minimum length of 6 characters and and a maximun lenth of 80 characters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>mtu</i> The maximum transmission unit (MTU), in bytes. The supported values are 1500 and 9001. The default value is 1500.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>asn</i> The autonomous system (AS) number for Border Gateway Protocol (BGP) configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vlan</i> The ID of the VLAN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualInterfaceName</i> The name of the virtual interface assigned by the customer network.<br/>
        :return: tags:  Any tags assigned to the virtual interface.<br/>awsDeviceV2:  The Direct Connect endpoint on which the virtual interface terminates.<br/>region:  The AWS Region where the virtual interface is located.<br/>bgpPeers:  The BGP peers configured on this virtual interface.<br/>routeFilterPrefixes:  The routes to be advertised to the AWS network in this Region. Applies to public virtual interfaces.<br/>directConnectGatewayId:  The ID of the Direct Connect gateway.<br/>virtualGatewayId:  The ID of the virtual private gateway. Applies only to private virtual interfaces.<br/>jumboFrameCapable:  Indicates whether jumbo frames (9001 MTU) are supported.<br/>mtu:  The maximum transmission unit (MTU), in bytes. The supported values are 1500 and 9001. The default value is 1500.<br/>customerRouterConfig:  The customer router configuration.<br/>virtualInterfaceState:  The state of the virtual interface. The following are the possible values: <ul> <li>  <code>confirming</code>: The creation of the virtual interface is pending confirmation from the virtual interface owner. If the owner of the virtual interface is different from the owner of the connection on which it is provisioned, then the virtual interface will remain in this state until it is confirmed by the virtual interface owner. </li> <li>  <code>verifying</code>: This state only applies to public virtual interfaces. Each public virtual interface needs validation before the virtual interface can be created. </li> <li>  <code>pending</code>: A virtual interface is in this state from the time that it is created until the virtual interface is ready to forward traffic. </li> <li>  <code>available</code>: A virtual interface that is able to forward traffic. </li> <li>  <code>down</code>: A virtual interface that is BGP down. </li> <li>  <code>deleting</code>: A virtual interface is in this state immediately after calling <a>DeleteVirtualInterface</a> until it can no longer forward traffic. </li> <li>  <code>deleted</code>: A virtual interface that cannot forward traffic. </li> <li>  <code>rejected</code>: The virtual interface owner has declined creation of the virtual interface. If a virtual interface in the <code>Confirming</code> state is deleted by the virtual interface owner, the virtual interface enters the <code>Rejected</code> state. </li> <li>  <code>unknown</code>: The state of the virtual interface is not available. </li> </ul><br/>addressFamily:  The address family for the BGP peer.<br/>customerAddress:  The IP address assigned to the customer interface.<br/>amazonAddress:  The IP address assigned to the Amazon interface.<br/>authKey:  The authentication key for BGP configuration. This string has a minimum length of 6 characters and and a maximun lenth of 80 characters.<br/>amazonSideAsn:  The autonomous system number (ASN) for the Amazon side of the connection.<br/>asn:  The autonomous system (AS) number for Border Gateway Protocol (BGP) configuration.<br/>vlan:  The ID of the VLAN.<br/>virtualInterfaceName:  The name of the virtual interface assigned by the customer network.<br/>virtualInterfaceType:  The type of virtual interface. The possible values are <code>private</code> and <code>public</code>.<br/>connectionId:  The ID of the connection.<br/>location:  The location of the connection.<br/>virtualInterfaceId:  The ID of the virtual interface.<br/>ownerAccount:  The ID of the AWS account that owns the virtual interface.<br/>        """
        pass

    def create_public_virtual_interface(self, newPublicVirtualInterface: NewPublicVirtualInterface, connectionId: str) -> VirtualInterface:
        """Creates a public virtual interface. A virtual interface is the VLAN that transports AWS Direct Connect traffic. A public virtual interface supports sending traffic to public services of AWS such as Amazon S3. When creating an IPv6 public virtual interface (<code>addressFamily</code> is <code>ipv6</code>), leave the <code>customer</code> and <code>amazon</code> address fields blank to use auto-assigned IPv6 space. Custom IPv6 addresses are not supported.

        :param self:
        :param newPublicVirtualInterface:  Information about the public virtual interface.:<br/>connectionId:  The ID of the connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> Any tags assigned to the public virtual interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>routeFilterPrefixes</i> The routes to be advertised to the AWS network in this Region. Applies to public virtual interfaces.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>addressFamily</i> The address family for the BGP peer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customerAddress</i> The IP address assigned to the customer interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>amazonAddress</i> The IP address assigned to the Amazon interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authKey</i> The authentication key for BGP configuration. This string has a minimum length of 6 characters and and a maximun lenth of 80 characters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>asn</i> The autonomous system (AS) number for Border Gateway Protocol (BGP) configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vlan</i> The ID of the VLAN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualInterfaceName</i> The name of the virtual interface assigned by the customer network.<br/>
        :return: tags:  Any tags assigned to the virtual interface.<br/>awsDeviceV2:  The Direct Connect endpoint on which the virtual interface terminates.<br/>region:  The AWS Region where the virtual interface is located.<br/>bgpPeers:  The BGP peers configured on this virtual interface.<br/>routeFilterPrefixes:  The routes to be advertised to the AWS network in this Region. Applies to public virtual interfaces.<br/>directConnectGatewayId:  The ID of the Direct Connect gateway.<br/>virtualGatewayId:  The ID of the virtual private gateway. Applies only to private virtual interfaces.<br/>jumboFrameCapable:  Indicates whether jumbo frames (9001 MTU) are supported.<br/>mtu:  The maximum transmission unit (MTU), in bytes. The supported values are 1500 and 9001. The default value is 1500.<br/>customerRouterConfig:  The customer router configuration.<br/>virtualInterfaceState:  The state of the virtual interface. The following are the possible values: <ul> <li>  <code>confirming</code>: The creation of the virtual interface is pending confirmation from the virtual interface owner. If the owner of the virtual interface is different from the owner of the connection on which it is provisioned, then the virtual interface will remain in this state until it is confirmed by the virtual interface owner. </li> <li>  <code>verifying</code>: This state only applies to public virtual interfaces. Each public virtual interface needs validation before the virtual interface can be created. </li> <li>  <code>pending</code>: A virtual interface is in this state from the time that it is created until the virtual interface is ready to forward traffic. </li> <li>  <code>available</code>: A virtual interface that is able to forward traffic. </li> <li>  <code>down</code>: A virtual interface that is BGP down. </li> <li>  <code>deleting</code>: A virtual interface is in this state immediately after calling <a>DeleteVirtualInterface</a> until it can no longer forward traffic. </li> <li>  <code>deleted</code>: A virtual interface that cannot forward traffic. </li> <li>  <code>rejected</code>: The virtual interface owner has declined creation of the virtual interface. If a virtual interface in the <code>Confirming</code> state is deleted by the virtual interface owner, the virtual interface enters the <code>Rejected</code> state. </li> <li>  <code>unknown</code>: The state of the virtual interface is not available. </li> </ul><br/>addressFamily:  The address family for the BGP peer.<br/>customerAddress:  The IP address assigned to the customer interface.<br/>amazonAddress:  The IP address assigned to the Amazon interface.<br/>authKey:  The authentication key for BGP configuration. This string has a minimum length of 6 characters and and a maximun lenth of 80 characters.<br/>amazonSideAsn:  The autonomous system number (ASN) for the Amazon side of the connection.<br/>asn:  The autonomous system (AS) number for Border Gateway Protocol (BGP) configuration.<br/>vlan:  The ID of the VLAN.<br/>virtualInterfaceName:  The name of the virtual interface assigned by the customer network.<br/>virtualInterfaceType:  The type of virtual interface. The possible values are <code>private</code> and <code>public</code>.<br/>connectionId:  The ID of the connection.<br/>location:  The location of the connection.<br/>virtualInterfaceId:  The ID of the virtual interface.<br/>ownerAccount:  The ID of the AWS account that owns the virtual interface.<br/>        """
        pass

    def create_transit_virtual_interface(self, newTransitVirtualInterface: NewTransitVirtualInterface, connectionId: str) -> CreateTransitVirtualInterfaceResult:
        """Creates a transit virtual interface. A transit virtual interface should be used to access one or more transit gateways associated with Direct Connect gateways. A transit virtual interface enables the connection of multiple VPCs attached to a transit gateway to a Direct Connect gateway. <important> If you associate your transit gateway with one or more Direct Connect gateways, the Autonomous System Number (ASN) used by the transit gateway and the Direct Connect gateway must be different. For example, if you use the default ASN 64512 for both your the transit gateway and Direct Connect gateway, the association request fails. </important>

        :param self:
        :param newTransitVirtualInterface:  Information about the transit virtual interface.:<br/>connectionId:  The ID of the connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> Any tags assigned to the transit virtual interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>directConnectGatewayId</i> The ID of the Direct Connect gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>addressFamily</i> The address family for the BGP peer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customerAddress</i> The IP address assigned to the customer interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>amazonAddress</i> The IP address assigned to the Amazon interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authKey</i> The authentication key for BGP configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>mtu</i> The maximum transmission unit (MTU), in bytes. The supported values are 1500 and 8500. The default value is 1500. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>asn</i> The autonomous system (AS) number for Border Gateway Protocol (BGP) configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vlan</i> The ID of the VLAN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualInterfaceName</i> The name of the virtual interface assigned by the customer network.<br/>
        :return: virtualInterface:  Information about a virtual interface.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> Any tags assigned to the virtual interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>awsDeviceV2</i> The Direct Connect endpoint on which the virtual interface terminates.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>region</i> The AWS Region where the virtual interface is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>bgpPeers</i> The BGP peers configured on this virtual interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>routeFilterPrefixes</i> The routes to be advertised to the AWS network in this Region. Applies to public virtual interfaces.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>directConnectGatewayId</i> The ID of the Direct Connect gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualGatewayId</i> The ID of the virtual private gateway. Applies only to private virtual interfaces.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jumboFrameCapable</i> Indicates whether jumbo frames (9001 MTU) are supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>mtu</i> The maximum transmission unit (MTU), in bytes. The supported values are 1500 and 9001. The default value is 1500.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customerRouterConfig</i> The customer router configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualInterfaceState</i> The state of the virtual interface. The following are the possible values: <ul> <li>  <code>confirming</code>: The creation of the virtual interface is pending confirmation from the virtual interface owner. If the owner of the virtual interface is different from the owner of the connection on which it is provisioned, then the virtual interface will remain in this state until it is confirmed by the virtual interface owner. </li> <li>  <code>verifying</code>: This state only applies to public virtual interfaces. Each public virtual interface needs validation before the virtual interface can be created. </li> <li>  <code>pending</code>: A virtual interface is in this state from the time that it is created until the virtual interface is ready to forward traffic. </li> <li>  <code>available</code>: A virtual interface that is able to forward traffic. </li> <li>  <code>down</code>: A virtual interface that is BGP down. </li> <li>  <code>deleting</code>: A virtual interface is in this state immediately after calling <a>DeleteVirtualInterface</a> until it can no longer forward traffic. </li> <li>  <code>deleted</code>: A virtual interface that cannot forward traffic. </li> <li>  <code>rejected</code>: The virtual interface owner has declined creation of the virtual interface. If a virtual interface in the <code>Confirming</code> state is deleted by the virtual interface owner, the virtual interface enters the <code>Rejected</code> state. </li> <li>  <code>unknown</code>: The state of the virtual interface is not available. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>addressFamily</i> The address family for the BGP peer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customerAddress</i> The IP address assigned to the customer interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>amazonAddress</i> The IP address assigned to the Amazon interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authKey</i> The authentication key for BGP configuration. This string has a minimum length of 6 characters and and a maximun lenth of 80 characters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>amazonSideAsn</i> The autonomous system number (ASN) for the Amazon side of the connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>asn</i> The autonomous system (AS) number for Border Gateway Protocol (BGP) configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vlan</i> The ID of the VLAN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualInterfaceName</i> The name of the virtual interface assigned by the customer network.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualInterfaceType</i> The type of virtual interface. The possible values are <code>private</code> and <code>public</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>connectionId</i> The ID of the connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The location of the connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualInterfaceId</i> The ID of the virtual interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ownerAccount</i> The ID of the AWS account that owns the virtual interface.<br/>        """
        pass

    def delete_bgp_peer(self, virtualInterfaceId: str=None, asn: int=None, customerAddress: str=None, bgpPeerId: str=None) -> DeleteBGPPeerResponse:
        """Deletes the specified BGP peer on the specified virtual interface with the specified customer address and ASN. You cannot delete the last BGP peer from a virtual interface.

        :param self:
        :param bgpPeerId:  The ID of the BGP peer.<br/>customerAddress:  The IP address assigned to the customer interface.<br/>asn:  The autonomous system (AS) number for Border Gateway Protocol (BGP) configuration.<br/>virtualInterfaceId:  The ID of the virtual interface.<br/>
        :return: virtualInterface:  The virtual interface.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> Any tags assigned to the virtual interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>awsDeviceV2</i> The Direct Connect endpoint on which the virtual interface terminates.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>region</i> The AWS Region where the virtual interface is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>bgpPeers</i> The BGP peers configured on this virtual interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>routeFilterPrefixes</i> The routes to be advertised to the AWS network in this Region. Applies to public virtual interfaces.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>directConnectGatewayId</i> The ID of the Direct Connect gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualGatewayId</i> The ID of the virtual private gateway. Applies only to private virtual interfaces.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jumboFrameCapable</i> Indicates whether jumbo frames (9001 MTU) are supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>mtu</i> The maximum transmission unit (MTU), in bytes. The supported values are 1500 and 9001. The default value is 1500.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customerRouterConfig</i> The customer router configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualInterfaceState</i> The state of the virtual interface. The following are the possible values: <ul> <li>  <code>confirming</code>: The creation of the virtual interface is pending confirmation from the virtual interface owner. If the owner of the virtual interface is different from the owner of the connection on which it is provisioned, then the virtual interface will remain in this state until it is confirmed by the virtual interface owner. </li> <li>  <code>verifying</code>: This state only applies to public virtual interfaces. Each public virtual interface needs validation before the virtual interface can be created. </li> <li>  <code>pending</code>: A virtual interface is in this state from the time that it is created until the virtual interface is ready to forward traffic. </li> <li>  <code>available</code>: A virtual interface that is able to forward traffic. </li> <li>  <code>down</code>: A virtual interface that is BGP down. </li> <li>  <code>deleting</code>: A virtual interface is in this state immediately after calling <a>DeleteVirtualInterface</a> until it can no longer forward traffic. </li> <li>  <code>deleted</code>: A virtual interface that cannot forward traffic. </li> <li>  <code>rejected</code>: The virtual interface owner has declined creation of the virtual interface. If a virtual interface in the <code>Confirming</code> state is deleted by the virtual interface owner, the virtual interface enters the <code>Rejected</code> state. </li> <li>  <code>unknown</code>: The state of the virtual interface is not available. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>addressFamily</i> The address family for the BGP peer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customerAddress</i> The IP address assigned to the customer interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>amazonAddress</i> The IP address assigned to the Amazon interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authKey</i> The authentication key for BGP configuration. This string has a minimum length of 6 characters and and a maximun lenth of 80 characters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>amazonSideAsn</i> The autonomous system number (ASN) for the Amazon side of the connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>asn</i> The autonomous system (AS) number for Border Gateway Protocol (BGP) configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vlan</i> The ID of the VLAN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualInterfaceName</i> The name of the virtual interface assigned by the customer network.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualInterfaceType</i> The type of virtual interface. The possible values are <code>private</code> and <code>public</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>connectionId</i> The ID of the connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The location of the connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualInterfaceId</i> The ID of the virtual interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ownerAccount</i> The ID of the AWS account that owns the virtual interface.<br/>        """
        pass

    def delete_connection(self, connectionId: str) -> Connection:
        """Deletes the specified connection. Deleting a connection only stops the AWS Direct Connect port hour and data transfer charges. If you are partnering with any third parties to connect with the AWS Direct Connect location, you must cancel your service with them separately.

        :param self:
        :param connectionId:  The ID of the connection.<br/>
        :return: tags:  Any tags assigned to the connection.<br/>hasLogicalRedundancy:  Indicates whether the connection supports a secondary BGP peer in the same address family (IPv4/IPv6).<br/>awsDeviceV2:  The Direct Connect endpoint on which the physical connection terminates.<br/>jumboFrameCapable:  Indicates whether jumbo frames (9001 MTU) are supported.<br/>awsDevice:  The Direct Connect endpoint on which the physical connection terminates.<br/>lagId:  The ID of the LAG.<br/>loaIssueTime:  The time of the most recent call to <a>DescribeLoa</a> for this connection.<br/>partnerName:  The name of the AWS Direct Connect service provider associated with the connection.<br/>vlan:  The ID of the VLAN.<br/>bandwidth:  The bandwidth of the connection.<br/>location:  The location of the connection.<br/>region:  The AWS Region where the connection is located.<br/>connectionState:  The state of the connection. The following are the possible values: <ul> <li>  <code>ordering</code>: The initial state of a hosted connection provisioned on an interconnect. The connection stays in the ordering state until the owner of the hosted connection confirms or declines the connection order. </li> <li>  <code>requested</code>: The initial state of a standard connection. The connection stays in the requested state until the Letter of Authorization (LOA) is sent to the customer. </li> <li>  <code>pending</code>: The connection has been approved and is being initialized. </li> <li>  <code>available</code>: The network link is up and the connection is ready for use. </li> <li>  <code>down</code>: The network link is down. </li> <li>  <code>deleting</code>: The connection is being deleted. </li> <li>  <code>deleted</code>: The connection has been deleted. </li> <li>  <code>rejected</code>: A hosted connection in the <code>ordering</code> state enters the <code>rejected</code> state if it is deleted by the customer. </li> <li>  <code>unknown</code>: The state of the connection is not available. </li> </ul><br/>connectionName:  The name of the connection.<br/>connectionId:  The ID of the connection.<br/>ownerAccount:  The ID of the AWS account that owns the connection.<br/>        """
        pass

    def delete_direct_connect_gateway(self, directConnectGatewayId: str) -> DeleteDirectConnectGatewayResult:
        """Deletes the specified Direct Connect gateway. You must first delete all virtual interfaces that are attached to the Direct Connect gateway and disassociate all virtual private gateways that are associated with the Direct Connect gateway.

        :param self:
        :param directConnectGatewayId:  The ID of the Direct Connect gateway.<br/>
        :return: directConnectGateway:  The Direct Connect gateway.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stateChangeError</i> The error message if the state of an object failed to advance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>directConnectGatewayState</i> The state of the Direct Connect gateway. The following are the possible values: <ul> <li>  <code>pending</code>: The initial state after calling <a>CreateDirectConnectGateway</a>. </li> <li>  <code>available</code>: The Direct Connect gateway is ready for use. </li> <li>  <code>deleting</code>: The initial state after calling <a>DeleteDirectConnectGateway</a>. </li> <li>  <code>deleted</code>: The Direct Connect gateway is deleted and cannot pass traffic. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ownerAccount</i> The ID of the AWS account that owns the Direct Connect gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>amazonSideAsn</i> The autonomous system number (ASN) for the Amazon side of the connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>directConnectGatewayName</i> The name of the Direct Connect gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>directConnectGatewayId</i> The ID of the Direct Connect gateway.<br/>        """
        pass

    def delete_direct_connect_gateway_association(self, associationId: str=None, directConnectGatewayId: str=None, virtualGatewayId: str=None) -> DeleteDirectConnectGatewayAssociationResult:
        """Deletes the association between the specified Direct Connect gateway and virtual private gateway.

        :param self:
        :param virtualGatewayId:  The ID of the virtual private gateway.<br/>directConnectGatewayId:  The ID of the Direct Connect gateway.<br/>associationId:  The ID of the Direct Connect gateway association.<br/>
        :return: directConnectGatewayAssociation:  Information about the deleted association.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualGatewayOwnerAccount</i> The ID of the AWS account that owns the virtual private gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualGatewayRegion</i> The AWS Region where the virtual private gateway is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualGatewayId</i> The ID of the virtual private gateway. Applies only to private virtual interfaces.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>allowedPrefixesToDirectConnectGateway</i> The Amazon VPC prefixes to advertise to the Direct Connect gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>associationId</i> The ID of the Direct Connect gateway association.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>associatedGateway</i> Information about the associated gateway.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>associatedGateway</b>: Information about the associated gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stateChangeError</i> The error message if the state of an object failed to advance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>associationState</i> The state of the association. The following are the possible values: <ul> <li>  <code>associating</code>: The initial state after calling <a>CreateDirectConnectGatewayAssociation</a>. </li> <li>  <code>associated</code>: The Direct Connect gateway and virtual private gateway or transit gateway are successfully associated and ready to pass traffic. </li> <li>  <code>disassociating</code>: The initial state after calling <a>DeleteDirectConnectGatewayAssociation</a>. </li> <li>  <code>disassociated</code>: The virtual private gateway or transit gateway is disassociated from the Direct Connect gateway. Traffic flow between the Direct Connect gateway and virtual private gateway or transit gateway is stopped. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>directConnectGatewayOwnerAccount</i> The ID of the AWS account that owns the associated gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>directConnectGatewayId</i> The ID of the Direct Connect gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>region</i> The Region where the associated gateway is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ownerAccount</i> The ID of the AWS account that owns the associated virtual private gateway or transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of associated gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the associated gateway.<br/>        """
        pass

    def delete_direct_connect_gateway_association_proposal(self, proposalId: str) -> DeleteDirectConnectGatewayAssociationProposalResult:
        """Deletes the association proposal request between the specified Direct Connect gateway and virtual private gateway or transit gateway.

        :param self:
        :param proposalId:  The ID of the proposal.<br/>
        :return: directConnectGatewayAssociationProposal:  The ID of the associated gateway.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>requestedAllowedPrefixesToDirectConnectGateway</i> The Amazon VPC prefixes to advertise to the Direct Connect gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>existingAllowedPrefixesToDirectConnectGateway</i> The existing Amazon VPC prefixes advertised to the Direct Connect gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>associatedGateway</i> Information about the associated gateway.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>associatedGateway</b>: Information about the associated gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>proposalState</i> The state of the proposal. The following are possible values: <ul> <li>  <code>accepted</code>: The proposal has been accepted. The Direct Connect gateway association is available to use in this state. </li> <li>  <code>deleted</code>: The proposal has been deleted by the owner that made the proposal. The Direct Connect gateway association cannot be used in this state. </li> <li>  <code>requested</code>: The proposal has been requested. The Direct Connect gateway association cannot be used in this state. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>directConnectGatewayOwnerAccount</i> The ID of the AWS account that owns the Direct Connect gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>directConnectGatewayId</i> The ID of the Direct Connect gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>proposalId</i> The ID of the association proposal.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>region</i> The Region where the associated gateway is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ownerAccount</i> The ID of the AWS account that owns the associated virtual private gateway or transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of associated gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the associated gateway.<br/>        """
        pass

    def delete_interconnect(self, interconnectId: str) -> DeleteInterconnectResponse:
        """Deletes the specified interconnect. <note> Intended for use by AWS Direct Connect Partners only. </note>

        :param self:
        :param interconnectId:  The ID of the interconnect.<br/>
        :return: interconnectState:  The state of the interconnect. The following are the possible values: <ul> <li>  <code>requested</code>: The initial state of an interconnect. The interconnect stays in the requested state until the Letter of Authorization (LOA) is sent to the customer. </li> <li>  <code>pending</code>: The interconnect is approved, and is being initialized. </li> <li>  <code>available</code>: The network link is up, and the interconnect is ready for use. </li> <li>  <code>down</code>: The network link is down. </li> <li>  <code>deleting</code>: The interconnect is being deleted. </li> <li>  <code>deleted</code>: The interconnect is deleted. </li> <li>  <code>unknown</code>: The state of the interconnect is not available. </li> </ul><br/>        """
        pass

    def delete_lag(self, lagId: str) -> Lag:
        """Deletes the specified link aggregation group (LAG). You cannot delete a LAG if it has active virtual interfaces or hosted connections.

        :param self:
        :param lagId:  The ID of the LAG.<br/>
        :return: tags:  Any tags assigned to link aggregation group (LAG).<br/>hasLogicalRedundancy:  Indicates whether the LAG supports a secondary BGP peer in the same address family (IPv4/IPv6).<br/>jumboFrameCapable:  Indicates whether jumbo frames (9001 MTU) are supported.<br/>allowsHostedConnections:  Indicates whether the LAG can host other connections.<br/>connections:  The connections bundled by the LAG.<br/>awsDeviceV2:  The AWS Direct Connect endpoint that hosts the LAG.<br/>awsDevice:  The AWS Direct Connect endpoint that hosts the LAG.<br/>minimumLinks:  The minimum number of physical connections that must be operational for the LAG itself to be operational.<br/>region:  The AWS Region where the connection is located.<br/>location:  The location of the LAG.<br/>lagState:  The state of the LAG. The following are the possible values: <ul> <li>  <code>requested</code>: The initial state of a LAG. The LAG stays in the requested state until the Letter of Authorization (LOA) is available. </li> <li>  <code>pending</code>: The LAG has been approved and is being initialized. </li> <li>  <code>available</code>: The network link is established and the LAG is ready for use. </li> <li>  <code>down</code>: The network link is down. </li> <li>  <code>deleting</code>: The LAG is being deleted. </li> <li>  <code>deleted</code>: The LAG is deleted. </li> <li>  <code>unknown</code>: The state of the LAG is not available. </li> </ul><br/>lagName:  The name of the LAG.<br/>ownerAccount:  The ID of the AWS account that owns the LAG.<br/>lagId:  The ID of the LAG.<br/>numberOfConnections:  The number of physical connections bundled by the LAG, up to a maximum of 10.<br/>connectionsBandwidth:  The individual bandwidth of the physical connections bundled by the LAG. The possible values are 1Gbps and 10Gbps. <br/>        """
        pass

    def delete_virtual_interface(self, virtualInterfaceId: str) -> DeleteVirtualInterfaceResponse:
        """Deletes a virtual interface.

        :param self:
        :param virtualInterfaceId:  The ID of the virtual interface.<br/>
        :return: virtualInterfaceState:  The state of the virtual interface. The following are the possible values: <ul> <li>  <code>confirming</code>: The creation of the virtual interface is pending confirmation from the virtual interface owner. If the owner of the virtual interface is different from the owner of the connection on which it is provisioned, then the virtual interface will remain in this state until it is confirmed by the virtual interface owner. </li> <li>  <code>verifying</code>: This state only applies to public virtual interfaces. Each public virtual interface needs validation before the virtual interface can be created. </li> <li>  <code>pending</code>: A virtual interface is in this state from the time that it is created until the virtual interface is ready to forward traffic. </li> <li>  <code>available</code>: A virtual interface that is able to forward traffic. </li> <li>  <code>down</code>: A virtual interface that is BGP down. </li> <li>  <code>deleting</code>: A virtual interface is in this state immediately after calling <a>DeleteVirtualInterface</a> until it can no longer forward traffic. </li> <li>  <code>deleted</code>: A virtual interface that cannot forward traffic. </li> <li>  <code>rejected</code>: The virtual interface owner has declined creation of the virtual interface. If a virtual interface in the <code>Confirming</code> state is deleted by the virtual interface owner, the virtual interface enters the <code>Rejected</code> state. </li> <li>  <code>unknown</code>: The state of the virtual interface is not available. </li> </ul><br/>        """
        pass

    def describe_connection_loa(self, connectionId: str, providerName: str=None, loaContentType: str=None) -> DescribeConnectionLoaResponse:
        """Deprecated. Use <a>DescribeLoa</a> instead. Gets the LOA-CFA for a connection. The Letter of Authorization - Connecting Facility Assignment (LOA-CFA) is a document that your APN partner or service provider uses when establishing your cross connect to AWS at the colocation facility. For more information, see <a href="https://docs.aws.amazon.com/directconnect/latest/UserGuide/Colocation.html">Requesting Cross Connects at AWS Direct Connect Locations</a> in the <i>AWS Direct Connect User Guide</i>.

        :param self:
        :param loaContentType:  The standard media type for the LOA-CFA document. The only supported value is application/pdf.<br/>providerName:  The name of the APN partner or service provider who establishes connectivity on your behalf. If you specify this parameter, the LOA-CFA lists the provider name alongside your company name as the requester of the cross connect.<br/>connectionId:  The ID of the connection.<br/>
        :return: loa:  The Letter of Authorization - Connecting Facility Assignment (LOA-CFA).:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>loaContentType</i> The standard media type for the LOA-CFA document. The only supported value is application/pdf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>loaContent</i> The binary contents of the LOA-CFA document.<br/>        """
        pass

    def describe_connections(self, connectionId: str=None) -> Connections:
        """Displays the specified connection or all connections in this Region.

        :param self:
        :param connectionId:  The ID of the connection.<br/>
        :return: connections:  The connections.<br/>        """
        pass

    def describe_connections_on_interconnect(self, interconnectId: str) -> Connections:
        """Deprecated. Use <a>DescribeHostedConnections</a> instead. Lists the connections that have been provisioned on the specified interconnect. <note> Intended for use by AWS Direct Connect Partners only. </note>

        :param self:
        :param interconnectId:  The ID of the interconnect.<br/>
        :return: connections:  The connections.<br/>        """
        pass

    def describe_direct_connect_gateway_association_proposals(self, directConnectGatewayId: str=None, proposalId: str=None, associatedGatewayId: str=None, maxResults: int=None, nextToken: str=None) -> DescribeDirectConnectGatewayAssociationProposalsResult:
        """Describes one or more association proposals for connection between a virtual private gateway or transit gateway and a Direct Connect gateway. 

        :param self:
        :param nextToken:  The token for the next page of results.<br/>maxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value. If <code>MaxResults</code> is given a value larger than 100, only 100 results are returned.<br/>associatedGatewayId:  The ID of the associated gateway.<br/>proposalId:  The ID of the proposal.<br/>directConnectGatewayId:  The ID of the Direct Connect gateway.<br/>
        :return: nextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>directConnectGatewayAssociationProposals:  Describes the Direct Connect gateway association proposals.<br/>        """
        pass

    def describe_direct_connect_gateway_associations(self, associationId: str=None, associatedGatewayId: str=None, directConnectGatewayId: str=None, maxResults: int=None, nextToken: str=None, virtualGatewayId: str=None) -> DescribeDirectConnectGatewayAssociationsResult:
        """Lists the associations between your Direct Connect gateways and virtual private gateways. You must specify a Direct Connect gateway, a virtual private gateway, or both. If you specify a Direct Connect gateway, the response contains all virtual private gateways associated with the Direct Connect gateway. If you specify a virtual private gateway, the response contains all Direct Connect gateways associated with the virtual private gateway. If you specify both, the response contains the association between the Direct Connect gateway and the virtual private gateway.

        :param self:
        :param virtualGatewayId:  The ID of the virtual private gateway.<br/>nextToken:  The token provided in the previous call to retrieve the next page.<br/>maxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value. If <code>MaxResults</code> is given a value larger than 100, only 100 results are returned.<br/>directConnectGatewayId:  The ID of the Direct Connect gateway.<br/>associatedGatewayId:  The ID of the associated gateway.<br/>associationId:  The ID of the Direct Connect gateway association.<br/>
        :return: nextToken:  The token to retrieve the next page.<br/>directConnectGatewayAssociations:  Information about the associations.<br/>        """
        pass

    def describe_direct_connect_gateway_attachments(self, directConnectGatewayId: str=None, virtualInterfaceId: str=None, maxResults: int=None, nextToken: str=None) -> DescribeDirectConnectGatewayAttachmentsResult:
        """Lists the attachments between your Direct Connect gateways and virtual interfaces. You must specify a Direct Connect gateway, a virtual interface, or both. If you specify a Direct Connect gateway, the response contains all virtual interfaces attached to the Direct Connect gateway. If you specify a virtual interface, the response contains all Direct Connect gateways attached to the virtual interface. If you specify both, the response contains the attachment between the Direct Connect gateway and the virtual interface.

        :param self:
        :param nextToken:  The token provided in the previous call to retrieve the next page.<br/>maxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value. If <code>MaxResults</code> is given a value larger than 100, only 100 results are returned.<br/>virtualInterfaceId:  The ID of the virtual interface.<br/>directConnectGatewayId:  The ID of the Direct Connect gateway.<br/>
        :return: nextToken:  The token to retrieve the next page.<br/>directConnectGatewayAttachments:  The attachments.<br/>        """
        pass

    def describe_direct_connect_gateways(self, directConnectGatewayId: str=None, maxResults: int=None, nextToken: str=None) -> DescribeDirectConnectGatewaysResult:
        """Lists all your Direct Connect gateways or only the specified Direct Connect gateway. Deleted Direct Connect gateways are not returned.

        :param self:
        :param nextToken:  The token provided in the previous call to retrieve the next page.<br/>maxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value. If <code>MaxResults</code> is given a value larger than 100, only 100 results are returned.<br/>directConnectGatewayId:  The ID of the Direct Connect gateway.<br/>
        :return: nextToken:  The token to retrieve the next page.<br/>directConnectGateways:  The Direct Connect gateways.<br/>        """
        pass

    def describe_hosted_connections(self, connectionId: str) -> Connections:
        """Lists the hosted connections that have been provisioned on the specified interconnect or link aggregation group (LAG). <note> Intended for use by AWS Direct Connect Partners only. </note>

        :param self:
        :param connectionId:  The ID of the interconnect or LAG.<br/>
        :return: connections:  The connections.<br/>        """
        pass

    def describe_interconnect_loa(self, interconnectId: str, providerName: str=None, loaContentType: str=None) -> DescribeInterconnectLoaResponse:
        """Deprecated. Use <a>DescribeLoa</a> instead. Gets the LOA-CFA for the specified interconnect. The Letter of Authorization - Connecting Facility Assignment (LOA-CFA) is a document that is used when establishing your cross connect to AWS at the colocation facility. For more information, see <a href="https://docs.aws.amazon.com/directconnect/latest/UserGuide/Colocation.html">Requesting Cross Connects at AWS Direct Connect Locations</a> in the <i>AWS Direct Connect User Guide</i>.

        :param self:
        :param loaContentType:  The standard media type for the LOA-CFA document. The only supported value is application/pdf.<br/>providerName:  The name of the service provider who establishes connectivity on your behalf. If you supply this parameter, the LOA-CFA lists the provider name alongside your company name as the requester of the cross connect.<br/>interconnectId:  The ID of the interconnect.<br/>
        :return: loa:  The Letter of Authorization - Connecting Facility Assignment (LOA-CFA).:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>loaContentType</i> The standard media type for the LOA-CFA document. The only supported value is application/pdf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>loaContent</i> The binary contents of the LOA-CFA document.<br/>        """
        pass

    def describe_interconnects(self, interconnectId: str=None) -> Interconnects:
        """Lists the interconnects owned by the AWS account or only the specified interconnect.

        :param self:
        :param interconnectId:  The ID of the interconnect.<br/>
        :return: interconnects:  The interconnects.<br/>        """
        pass

    def describe_lags(self, lagId: str=None) -> Lags:
        """Describes all your link aggregation groups (LAG) or the specified LAG.

        :param self:
        :param lagId:  The ID of the LAG.<br/>
        :return: lags:  The LAGs.<br/>        """
        pass

    def describe_loa(self, connectionId: str, providerName: str=None, loaContentType: str=None) -> Loa:
        """Gets the LOA-CFA for a connection, interconnect, or link aggregation group (LAG). The Letter of Authorization - Connecting Facility Assignment (LOA-CFA) is a document that is used when establishing your cross connect to AWS at the colocation facility. For more information, see <a href="https://docs.aws.amazon.com/directconnect/latest/UserGuide/Colocation.html">Requesting Cross Connects at AWS Direct Connect Locations</a> in the <i>AWS Direct Connect User Guide</i>.

        :param self:
        :param loaContentType:  The standard media type for the LOA-CFA document. The only supported value is application/pdf.<br/>providerName:  The name of the service provider who establishes connectivity on your behalf. If you specify this parameter, the LOA-CFA lists the provider name alongside your company name as the requester of the cross connect.<br/>connectionId:  The ID of a connection, LAG, or interconnect.<br/>
        :return: loaContentType:  The standard media type for the LOA-CFA document. The only supported value is application/pdf.<br/>loaContent:  The binary contents of the LOA-CFA document.<br/>        """
        pass

    def describe_locations(self) -> Locations:
        """Lists the AWS Direct Connect locations in the current AWS Region. These are the locations that can be selected when calling <a>CreateConnection</a> or <a>CreateInterconnect</a>.

        :param self:
        :param 
        :return: locations:  The locations.<br/>        """
        pass

    def describe_tags(self, resourceArns: List[ResourceArn]) -> DescribeTagsResponse:
        """Describes the tags associated with the specified AWS Direct Connect resources.

        :param self:
        :param resourceArns:  The Amazon Resource Names (ARNs) of the resources.<br/>
        :return: resourceTags:  Information about the tags.<br/>        """
        pass

    def describe_virtual_gateways(self) -> VirtualGateways:
        """Lists the virtual private gateways owned by the AWS account. You can create one or more AWS Direct Connect private virtual interfaces linked to a virtual private gateway.

        :param self:
        :param 
        :return: virtualGateways:  The virtual private gateways.<br/>        """
        pass

    def describe_virtual_interfaces(self, connectionId: str=None, virtualInterfaceId: str=None) -> VirtualInterfaces:
        """Displays all virtual interfaces for an AWS account. Virtual interfaces deleted fewer than 15 minutes before you make the request are also returned. If you specify a connection ID, only the virtual interfaces associated with the connection are returned. If you specify a virtual interface ID, then only a single virtual interface is returned. A virtual interface (VLAN) transmits the traffic between the AWS Direct Connect location and the customer network.

        :param self:
        :param virtualInterfaceId:  The ID of the virtual interface.<br/>connectionId:  The ID of the connection.<br/>
        :return: virtualInterfaces:  The virtual interfaces<br/>        """
        pass

    def disassociate_connection_from_lag(self, lagId: str, connectionId: str) -> Connection:
        """Disassociates a connection from a link aggregation group (LAG). The connection is interrupted and re-established as a standalone connection (the connection is not deleted; to delete the connection, use the <a>DeleteConnection</a> request). If the LAG has associated virtual interfaces or hosted connections, they remain associated with the LAG. A disassociated connection owned by an AWS Direct Connect Partner is automatically converted to an interconnect. If disassociating the connection would cause the LAG to fall below its setting for minimum number of operational connections, the request fails, except when it's the last member of the LAG. If all connections are disassociated, the LAG continues to exist as an empty LAG with no physical connections. 

        :param self:
        :param lagId:  The ID of the LAG.<br/>connectionId:  The ID of the connection.<br/>
        :return: tags:  Any tags assigned to the connection.<br/>hasLogicalRedundancy:  Indicates whether the connection supports a secondary BGP peer in the same address family (IPv4/IPv6).<br/>awsDeviceV2:  The Direct Connect endpoint on which the physical connection terminates.<br/>jumboFrameCapable:  Indicates whether jumbo frames (9001 MTU) are supported.<br/>awsDevice:  The Direct Connect endpoint on which the physical connection terminates.<br/>lagId:  The ID of the LAG.<br/>loaIssueTime:  The time of the most recent call to <a>DescribeLoa</a> for this connection.<br/>partnerName:  The name of the AWS Direct Connect service provider associated with the connection.<br/>vlan:  The ID of the VLAN.<br/>bandwidth:  The bandwidth of the connection.<br/>location:  The location of the connection.<br/>region:  The AWS Region where the connection is located.<br/>connectionState:  The state of the connection. The following are the possible values: <ul> <li>  <code>ordering</code>: The initial state of a hosted connection provisioned on an interconnect. The connection stays in the ordering state until the owner of the hosted connection confirms or declines the connection order. </li> <li>  <code>requested</code>: The initial state of a standard connection. The connection stays in the requested state until the Letter of Authorization (LOA) is sent to the customer. </li> <li>  <code>pending</code>: The connection has been approved and is being initialized. </li> <li>  <code>available</code>: The network link is up and the connection is ready for use. </li> <li>  <code>down</code>: The network link is down. </li> <li>  <code>deleting</code>: The connection is being deleted. </li> <li>  <code>deleted</code>: The connection has been deleted. </li> <li>  <code>rejected</code>: A hosted connection in the <code>ordering</code> state enters the <code>rejected</code> state if it is deleted by the customer. </li> <li>  <code>unknown</code>: The state of the connection is not available. </li> </ul><br/>connectionName:  The name of the connection.<br/>connectionId:  The ID of the connection.<br/>ownerAccount:  The ID of the AWS account that owns the connection.<br/>        """
        pass

    def tag_resource(self, tags: List[Tag], resourceArn: str) -> TagResourceResponse:
        """Adds the specified tags to the specified AWS Direct Connect resource. Each resource can have a maximum of 50 tags. Each tag consists of a key and an optional value. If a tag with the same key is already associated with the resource, this action updates its value.

        :param self:
        :param tags:  The tags to assign.<br/>resourceArn:  The Amazon Resource Name (ARN) of the resource.<br/>
        :return:         """
        pass

    def untag_resource(self, tagKeys: List[TagKey], resourceArn: str) -> UntagResourceResponse:
        """Removes one or more tags from the specified AWS Direct Connect resource.

        :param self:
        :param tagKeys:  The tag keys of the tags to remove.<br/>resourceArn:  The Amazon Resource Name (ARN) of the resource.<br/>
        :return:         """
        pass

    def update_direct_connect_gateway_association(self, associationId: str=None, addAllowedPrefixesToDirectConnectGateway: List[RouteFilterPrefix]=None, removeAllowedPrefixesToDirectConnectGateway: List[RouteFilterPrefix]=None) -> UpdateDirectConnectGatewayAssociationResult:
        """Updates the specified attributes of the Direct Connect gateway association. Add or remove prefixes from the association.

        :param self:
        :param removeAllowedPrefixesToDirectConnectGateway:  The Amazon VPC prefixes to no longer advertise to the Direct Connect gateway.<br/>addAllowedPrefixesToDirectConnectGateway:  The Amazon VPC prefixes to advertise to the Direct Connect gateway.<br/>associationId:  The ID of the Direct Connect gateway association.<br/>
        :return: directConnectGatewayAssociation:  Information about an association between a Direct Connect gateway and a virtual private gateway or transit gateway.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualGatewayOwnerAccount</i> The ID of the AWS account that owns the virtual private gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualGatewayRegion</i> The AWS Region where the virtual private gateway is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>virtualGatewayId</i> The ID of the virtual private gateway. Applies only to private virtual interfaces.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>allowedPrefixesToDirectConnectGateway</i> The Amazon VPC prefixes to advertise to the Direct Connect gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>associationId</i> The ID of the Direct Connect gateway association.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>associatedGateway</i> Information about the associated gateway.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>associatedGateway</b>: Information about the associated gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stateChangeError</i> The error message if the state of an object failed to advance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>associationState</i> The state of the association. The following are the possible values: <ul> <li>  <code>associating</code>: The initial state after calling <a>CreateDirectConnectGatewayAssociation</a>. </li> <li>  <code>associated</code>: The Direct Connect gateway and virtual private gateway or transit gateway are successfully associated and ready to pass traffic. </li> <li>  <code>disassociating</code>: The initial state after calling <a>DeleteDirectConnectGatewayAssociation</a>. </li> <li>  <code>disassociated</code>: The virtual private gateway or transit gateway is disassociated from the Direct Connect gateway. Traffic flow between the Direct Connect gateway and virtual private gateway or transit gateway is stopped. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>directConnectGatewayOwnerAccount</i> The ID of the AWS account that owns the associated gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>directConnectGatewayId</i> The ID of the Direct Connect gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>region</i> The Region where the associated gateway is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ownerAccount</i> The ID of the AWS account that owns the associated virtual private gateway or transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of associated gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the associated gateway.<br/>        """
        pass

    def update_lag(self, lagId: str, lagName: str=None, minimumLinks: int=None) -> Lag:
        """Updates the attributes of the specified link aggregation group (LAG). You can update the following attributes: <ul> <li> The name of the LAG. </li> <li> The value for the minimum number of connections that must be operational for the LAG itself to be operational.  </li> </ul> When you create a LAG, the default value for the minimum number of operational connections is zero (0). If you update this value and the number of operational connections falls below the specified value, the LAG automatically goes down to avoid over-utilization of the remaining connections. Adjust this value with care, as it could force the LAG down if it is set higher than the current number of operational connections.

        :param self:
        :param minimumLinks:  The minimum number of physical connections that must be operational for the LAG itself to be operational.<br/>lagName:  The name of the LAG.<br/>lagId:  The ID of the LAG.<br/>
        :return: tags:  Any tags assigned to link aggregation group (LAG).<br/>hasLogicalRedundancy:  Indicates whether the LAG supports a secondary BGP peer in the same address family (IPv4/IPv6).<br/>jumboFrameCapable:  Indicates whether jumbo frames (9001 MTU) are supported.<br/>allowsHostedConnections:  Indicates whether the LAG can host other connections.<br/>connections:  The connections bundled by the LAG.<br/>awsDeviceV2:  The AWS Direct Connect endpoint that hosts the LAG.<br/>awsDevice:  The AWS Direct Connect endpoint that hosts the LAG.<br/>minimumLinks:  The minimum number of physical connections that must be operational for the LAG itself to be operational.<br/>region:  The AWS Region where the connection is located.<br/>location:  The location of the LAG.<br/>lagState:  The state of the LAG. The following are the possible values: <ul> <li>  <code>requested</code>: The initial state of a LAG. The LAG stays in the requested state until the Letter of Authorization (LOA) is available. </li> <li>  <code>pending</code>: The LAG has been approved and is being initialized. </li> <li>  <code>available</code>: The network link is established and the LAG is ready for use. </li> <li>  <code>down</code>: The network link is down. </li> <li>  <code>deleting</code>: The LAG is being deleted. </li> <li>  <code>deleted</code>: The LAG is deleted. </li> <li>  <code>unknown</code>: The state of the LAG is not available. </li> </ul><br/>lagName:  The name of the LAG.<br/>ownerAccount:  The ID of the AWS account that owns the LAG.<br/>lagId:  The ID of the LAG.<br/>numberOfConnections:  The number of physical connections bundled by the LAG, up to a maximum of 10.<br/>connectionsBandwidth:  The individual bandwidth of the physical connections bundled by the LAG. The possible values are 1Gbps and 10Gbps. <br/>        """
        pass

    def update_virtual_interface_attributes(self, virtualInterfaceId: str, mtu: int=None) -> VirtualInterface:
        """Updates the specified attributes of the specified virtual private interface. Setting the MTU of a virtual interface to 9001 (jumbo frames) can cause an update to the underlying physical connection if it wasn't updated to support jumbo frames. Updating the connection disrupts network connectivity for all virtual interfaces associated with the connection for up to 30 seconds. To check whether your connection supports jumbo frames, call <a>DescribeConnections</a>. To check whether your virtual interface supports jumbo frames, call <a>DescribeVirtualInterfaces</a>.

        :param self:
        :param mtu:  The maximum transmission unit (MTU), in bytes. The supported values are 1500 and 9001. The default value is 1500.<br/>virtualInterfaceId:  The ID of the virtual private interface.<br/>
        :return: tags:  Any tags assigned to the virtual interface.<br/>awsDeviceV2:  The Direct Connect endpoint on which the virtual interface terminates.<br/>region:  The AWS Region where the virtual interface is located.<br/>bgpPeers:  The BGP peers configured on this virtual interface.<br/>routeFilterPrefixes:  The routes to be advertised to the AWS network in this Region. Applies to public virtual interfaces.<br/>directConnectGatewayId:  The ID of the Direct Connect gateway.<br/>virtualGatewayId:  The ID of the virtual private gateway. Applies only to private virtual interfaces.<br/>jumboFrameCapable:  Indicates whether jumbo frames (9001 MTU) are supported.<br/>mtu:  The maximum transmission unit (MTU), in bytes. The supported values are 1500 and 9001. The default value is 1500.<br/>customerRouterConfig:  The customer router configuration.<br/>virtualInterfaceState:  The state of the virtual interface. The following are the possible values: <ul> <li>  <code>confirming</code>: The creation of the virtual interface is pending confirmation from the virtual interface owner. If the owner of the virtual interface is different from the owner of the connection on which it is provisioned, then the virtual interface will remain in this state until it is confirmed by the virtual interface owner. </li> <li>  <code>verifying</code>: This state only applies to public virtual interfaces. Each public virtual interface needs validation before the virtual interface can be created. </li> <li>  <code>pending</code>: A virtual interface is in this state from the time that it is created until the virtual interface is ready to forward traffic. </li> <li>  <code>available</code>: A virtual interface that is able to forward traffic. </li> <li>  <code>down</code>: A virtual interface that is BGP down. </li> <li>  <code>deleting</code>: A virtual interface is in this state immediately after calling <a>DeleteVirtualInterface</a> until it can no longer forward traffic. </li> <li>  <code>deleted</code>: A virtual interface that cannot forward traffic. </li> <li>  <code>rejected</code>: The virtual interface owner has declined creation of the virtual interface. If a virtual interface in the <code>Confirming</code> state is deleted by the virtual interface owner, the virtual interface enters the <code>Rejected</code> state. </li> <li>  <code>unknown</code>: The state of the virtual interface is not available. </li> </ul><br/>addressFamily:  The address family for the BGP peer.<br/>customerAddress:  The IP address assigned to the customer interface.<br/>amazonAddress:  The IP address assigned to the Amazon interface.<br/>authKey:  The authentication key for BGP configuration. This string has a minimum length of 6 characters and and a maximun lenth of 80 characters.<br/>amazonSideAsn:  The autonomous system number (ASN) for the Amazon side of the connection.<br/>asn:  The autonomous system (AS) number for Border Gateway Protocol (BGP) configuration.<br/>vlan:  The ID of the VLAN.<br/>virtualInterfaceName:  The name of the virtual interface assigned by the customer network.<br/>virtualInterfaceType:  The type of virtual interface. The possible values are <code>private</code> and <code>public</code>.<br/>connectionId:  The ID of the connection.<br/>location:  The location of the connection.<br/>virtualInterfaceId:  The ID of the virtual interface.<br/>ownerAccount:  The ID of the AWS account that owns the virtual interface.<br/>        """
        pass



class ApplicationDiscoveryService(BaseClient):
    """<fullname>AWS Application Discovery Service</fullname> AWS Application Discovery Service helps you plan application migration projects by automatically identifying servers, virtual machines (VMs), software, and software dependencies running in your on-premises data centers. Application Discovery Service also collects application performance data, which can help you assess the outcome of your migration. The data collected by Application Discovery Service is securely retained in an AWS-hosted and managed database in the cloud. You can export the data as a CSV or XML file into your preferred visualization tool or cloud-migration solution to plan your migration. For more information, see <a href="http://aws.amazon.com/application-discovery/faqs/">AWS Application Discovery Service FAQ</a>. Application Discovery Service offers two modes of operation: <ul> <li>  <b>Agentless discovery</b> mode is recommended for environments that use VMware vCenter Server. This mode doesn't require you to install an agent on each host. Agentless discovery gathers server information regardless of the operating systems, which minimizes the time required for initial on-premises infrastructure assessment. Agentless discovery doesn't collect information about software and software dependencies. It also doesn't work in non-VMware environments.  </li> <li>  <b>Agent-based discovery</b> mode collects a richer set of data than agentless discovery by using the AWS Application Discovery Agent, which you install on one or more hosts in your data center. The agent captures infrastructure and application information, including an inventory of installed software applications, system and process performance, resource utilization, and network dependencies between workloads. The information collected by agents is secured at rest and in transit to the Application Discovery Service database in the cloud.  </li> </ul> We recommend that you use agent-based discovery for non-VMware environments and to collect information about software and software dependencies. You can also run agent-based and agentless discovery simultaneously. Use agentless discovery to quickly complete the initial infrastructure assessment and then install agents on select hosts. Application Discovery Service integrates with application discovery solutions from AWS Partner Network (APN) partners. Third-party application discovery tools can query Application Discovery Service and write to the Application Discovery Service database using a public API. You can then import the data into either a visualization tool or cloud-migration solution. <important> Application Discovery Service doesn't gather sensitive information. All data is handled according to the <a href="http://aws.amazon.com/privacy/">AWS Privacy Policy</a>. You can operate Application Discovery Service offline to inspect collected data before it is shared with the service. </important> This API reference provides descriptions, syntax, and usage examples for each of the actions and data types for Application Discovery Service. The topic for each action shows the API request parameters and the response. Alternatively, you can use one of the AWS SDKs to access an API that is tailored to the programming language or platform that you're using. For more information, see <a href="http://aws.amazon.com/tools/#SDKs">AWS SDKs</a>. This guide is intended for use with the <a href="http://docs.aws.amazon.com/application-discovery/latest/userguide/"> <i>AWS Application Discovery Service User Guide</i> </a>."""


    class DescribeAgentsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_agents` for valid parameters"""
            pass
    class DescribeContinuousExportsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_continuous_exports` for valid parameters"""
            pass
    class DescribeExportConfigurationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_export_configurations` for valid parameters"""
            pass
    class DescribeExportTasksPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_export_tasks` for valid parameters"""
            pass
    class DescribeTagsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_tags` for valid parameters"""
            pass
    class ListConfigurationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_configurations` for valid parameters"""
            pass


    class AssociateConfigurationItemsToApplicationRequest(Mapping):
        pass
    
    class AssociateConfigurationItemsToApplicationResponse(Mapping):
        pass
    
    class ConfigurationId(object):
        pass
    
    class BatchDeleteImportDataRequest(Mapping):
        pass
    
    class BatchDeleteImportDataResponse(Mapping):
        pass
    
    class ImportTaskIdentifier(object):
        pass
    
    class CreateApplicationRequest(Mapping):
        pass
    
    class CreateApplicationResponse(Mapping):
        pass
    
    class CreateTagsRequest(Mapping):
        pass
    
    class CreateTagsResponse(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class DeleteApplicationsRequest(Mapping):
        pass
    
    class DeleteApplicationsResponse(Mapping):
        pass
    
    class ApplicationId(object):
        pass
    
    class DeleteTagsRequest(Mapping):
        pass
    
    class DeleteTagsResponse(Mapping):
        pass
    
    class DescribeAgentsRequest(Mapping):
        pass
    
    class DescribeAgentsResponse(Mapping):
        pass
    
    class AgentId(object):
        pass
    
    class Filter(Mapping):
        pass
    
    class DescribeConfigurationsRequest(Mapping):
        pass
    
    class DescribeConfigurationsResponse(Mapping):
        pass
    
    class DescribeContinuousExportsRequest(Mapping):
        pass
    
    class DescribeContinuousExportsResponse(Mapping):
        pass
    
    class ConfigurationsExportId(object):
        pass
    
    class DescribeExportConfigurationsRequest(Mapping):
        pass
    
    class DescribeExportConfigurationsResponse(Mapping):
        pass
    
    class DescribeExportTasksRequest(Mapping):
        pass
    
    class DescribeExportTasksResponse(Mapping):
        pass
    
    class ExportFilter(Mapping):
        pass
    
    class DescribeImportTasksRequest(Mapping):
        pass
    
    class DescribeImportTasksResponse(Mapping):
        pass
    
    class ImportTaskFilter(Mapping):
        pass
    
    class DescribeTagsRequest(Mapping):
        pass
    
    class DescribeTagsResponse(Mapping):
        pass
    
    class TagFilter(Mapping):
        pass
    
    class DisassociateConfigurationItemsFromApplicationRequest(Mapping):
        pass
    
    class DisassociateConfigurationItemsFromApplicationResponse(Mapping):
        pass
    
    class ExportConfigurationsResponse(Mapping):
        pass
    
    class GetDiscoverySummaryRequest(Mapping):
        pass
    
    class GetDiscoverySummaryResponse(Mapping):
        pass
    
    class ListConfigurationsRequest(Mapping):
        pass
    
    class ListConfigurationsResponse(Mapping):
        pass
    
    class OrderByElement(Mapping):
        pass
    
    class ListServerNeighborsRequest(Mapping):
        pass
    
    class ListServerNeighborsResponse(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class StartContinuousExportRequest(Mapping):
        pass
    
    class StartContinuousExportResponse(Mapping):
        pass
    
    class StartDataCollectionByAgentIdsRequest(Mapping):
        pass
    
    class StartDataCollectionByAgentIdsResponse(Mapping):
        pass
    
    class StartExportTaskRequest(Mapping):
        pass
    
    class StartExportTaskResponse(Mapping):
        pass
    
    class ExportDataFormat(object):
        pass
    
    class TimeStamp(object):
        pass
    
    class StartImportTaskRequest(Mapping):
        pass
    
    class StartImportTaskResponse(Mapping):
        pass
    
    class StopContinuousExportRequest(Mapping):
        pass
    
    class StopContinuousExportResponse(Mapping):
        pass
    
    class StopDataCollectionByAgentIdsRequest(Mapping):
        pass
    
    class StopDataCollectionByAgentIdsResponse(Mapping):
        pass
    
    class UpdateApplicationRequest(Mapping):
        pass
    
    class UpdateApplicationResponse(Mapping):
        pass
    
    def associate_configuration_items_to_application(self, configurationIds: List[ConfigurationId], applicationConfigurationId: str) -> AssociateConfigurationItemsToApplicationResponse:
        """Associates one or more configuration items with an application.

        :param self:
        :param configurationIds:  The ID of each configuration item to be associated with an application.<br/>applicationConfigurationId:  The configuration ID of an application with which items are to be associated.<br/>
        :return:         """
        pass

    def batch_delete_import_data(self, importTaskIds: List[ImportTaskIdentifier]) -> BatchDeleteImportDataResponse:
        """Deletes one or more import tasks, each identified by their import ID. Each import task has a number of records that can identify servers or applications.  AWS Application Discovery Service has built-in matching logic that will identify when discovered servers match existing entries that you've previously discovered, the information for the already-existing discovered server is updated. When you delete an import task that contains records that were used to match, the information in those matched records that comes from the deleted records will also be deleted.

        :param self:
        :param importTaskIds:  The IDs for the import tasks that you want to delete.<br/>
        :return: errors:  Error messages returned for each import task that you deleted as a response for this command.<br/>        """
        pass

    def create_application(self, name: str, description: str=None) -> CreateApplicationResponse:
        """Creates an application with the given name and description.

        :param self:
        :param description:  Description of the application to be created.<br/>name:  Name of the application to be created.<br/>
        :return: configurationId:  Configuration ID of an application to be created.<br/>        """
        pass

    def create_tags(self, tags: List[Tag], configurationIds: List[ConfigurationId]) -> CreateTagsResponse:
        """Creates one or more tags for configuration items. Tags are metadata that help you categorize IT assets. This API accepts a list of multiple configuration items.

        :param self:
        :param tags:  Tags that you want to associate with one or more configuration items. Specify the tags that you want to create in a <i>key</i>-<i>value</i> format. For example:  <code>{"key": "serverType", "value": "webServer"}</code> <br/>configurationIds:  A list of configuration items that you want to tag.<br/>
        :return:         """
        pass

    def delete_applications(self, configurationIds: List[ApplicationId]) -> DeleteApplicationsResponse:
        """Deletes a list of applications and their associations with configuration items.

        :param self:
        :param configurationIds:  Configuration ID of an application to be deleted.<br/>
        :return:         """
        pass

    def delete_tags(self, configurationIds: List[ConfigurationId], tags: List[Tag]=None) -> DeleteTagsResponse:
        """Deletes the association between configuration items and one or more tags. This API accepts a list of multiple configuration items.

        :param self:
        :param tags:  Tags that you want to delete from one or more configuration items. Specify the tags that you want to delete in a <i>key</i>-<i>value</i> format. For example:  <code>{"key": "serverType", "value": "webServer"}</code> <br/>configurationIds:  A list of configuration items with tags that you want to delete.<br/>
        :return:         """
        pass

    def describe_agents(self, agentIds: List[AgentId]=None, filters: List[Filter]=None, maxResults: int=None, nextToken: str=None) -> DescribeAgentsResponse:
        """Lists agents or connectors as specified by ID or other filters. All agents/connectors associated with your user account can be listed if you call <code>DescribeAgents</code> as is without passing any parameters.

        :param self:
        :param nextToken:  Token to retrieve the next set of results. For example, if you previously specified 100 IDs for <code>DescribeAgentsRequest$agentIds</code> but set <code>DescribeAgentsRequest$maxResults</code> to 10, you received a set of 10 results along with a token. Use that token in this query to get the next set of 10.<br/>maxResults:  The total number of agents/Connectors to return in a single page of output. The maximum value is 100.<br/>filters:  You can filter the request using various logical operators and a <i>key</i>-<i>value</i> format. For example:   <code>{"key": "collectionStatus", "value": "STARTED"}</code> <br/>agentIds:  The agent or the Connector IDs for which you want information. If you specify no IDs, the system returns information about all agents/Connectors associated with your AWS user account.<br/>
        :return: nextToken:  Token to retrieve the next set of results. For example, if you specified 100 IDs for <code>DescribeAgentsRequest$agentIds</code> but set <code>DescribeAgentsRequest$maxResults</code> to 10, you received a set of 10 results along with this token. Use this token in the next query to retrieve the next set of 10.<br/>agentsInfo:  Lists agents or the Connector by ID or lists all agents/Connectors associated with your user account if you did not specify an agent/Connector ID. The output includes agent/Connector IDs, IP addresses, media access control (MAC) addresses, agent/Connector health, host name where the agent/Connector resides, and the version number of each agent/Connector.<br/>        """
        pass

    def describe_configurations(self, configurationIds: List[ConfigurationId]) -> DescribeConfigurationsResponse:
        """Retrieves attributes for a list of configuration item IDs. <note> All of the supplied IDs must be for the same asset type from one of the following: <ul> <li> server </li> <li> application </li> <li> process </li> <li> connection </li> </ul> Output fields are specific to the asset type specified. For example, the output for a <i>server</i> configuration item includes a list of attributes about the server, such as host name, operating system, number of network cards, etc. For a complete list of outputs for each asset type, see <a href="http://docs.aws.amazon.com/application-discovery/latest/APIReference/discovery-api-queries.html#DescribeConfigurations">Using the DescribeConfigurations Action</a>. </note>

        :param self:
        :param configurationIds:  One or more configuration IDs.<br/>
        :return: configurations:  A key in the response map. The value is an array of data.<br/>        """
        pass

    def describe_continuous_exports(self, exportIds: List[ConfigurationsExportId]=None, maxResults: int=None, nextToken: str=None) -> DescribeContinuousExportsResponse:
        """Lists exports as specified by ID. All continuous exports associated with your user account can be listed if you call <code>DescribeContinuousExports</code> as is without passing any parameters.

        :param self:
        :param nextToken:  The token from the previous call to <code>DescribeExportTasks</code>.<br/>maxResults:  A number between 1 and 100 specifying the maximum number of continuous export descriptions returned.<br/>exportIds:  The unique IDs assigned to the exports.<br/>
        :return: nextToken:  The token from the previous call to <code>DescribeExportTasks</code>.<br/>descriptions:  A list of continuous export descriptions.<br/>        """
        pass

    def describe_export_configurations(self, exportIds: List[ConfigurationsExportId]=None, maxResults: int=None, nextToken: str=None) -> DescribeExportConfigurationsResponse:
        """ <code>DescribeExportConfigurations</code> is deprecated. Use <a href="https://docs.aws.amazon.com/application-discovery/latest/APIReference/API_DescribeExportTasks.html">DescribeImportTasks</a>, instead.

        :param self:
        :param nextToken:  The token from the previous call to describe-export-tasks.<br/>maxResults:  A number between 1 and 100 specifying the maximum number of continuous export descriptions returned.<br/>exportIds:  A list of continuous export ids to search for.<br/>
        :return: nextToken:  The token from the previous call to describe-export-tasks.<br/>exportsInfo:  <p/><br/>        """
        pass

    def describe_export_tasks(self, exportIds: List[ConfigurationsExportId]=None, filters: List[ExportFilter]=None, maxResults: int=None, nextToken: str=None) -> DescribeExportTasksResponse:
        """Retrieve status of one or more export tasks. You can retrieve the status of up to 100 export tasks.

        :param self:
        :param nextToken:  The <code>nextToken</code> value returned from a previous paginated <code>DescribeExportTasks</code> request where <code>maxResults</code> was used and the results exceeded the value of that parameter. Pagination continues from the end of the previous results that returned the <code>nextToken</code> value. This value is null when there are no more results to return.<br/>maxResults:  The maximum number of volume results returned by <code>DescribeExportTasks</code> in paginated output. When this parameter is used, <code>DescribeExportTasks</code> only returns <code>maxResults</code> results in a single page along with a <code>nextToken</code> response element.<br/>filters:  One or more filters. <ul> <li>  <code>AgentId</code> - ID of the agent whose collected data will be exported </li> </ul><br/>exportIds:  One or more unique identifiers used to query the status of an export request.<br/>
        :return: nextToken:  The <code>nextToken</code> value to include in a future <code>DescribeExportTasks</code> request. When the results of a <code>DescribeExportTasks</code> request exceed <code>maxResults</code>, this value can be used to retrieve the next page of results. This value is null when there are no more results to return.<br/>exportsInfo:  Contains one or more sets of export request details. When the status of a request is <code>SUCCEEDED</code>, the response includes a URL for an Amazon S3 bucket where you can view the data in a CSV file.<br/>        """
        pass

    def describe_import_tasks(self, filters: List[ImportTaskFilter]=None, maxResults: int=None, nextToken: str=None) -> DescribeImportTasksResponse:
        """Returns an array of import tasks for your account, including status information, times, IDs, the Amazon S3 Object URL for the import file, and more.

        :param self:
        :param nextToken:  The token to request a specific page of results.<br/>maxResults:  The maximum number of results that you want this request to return, up to 100.<br/>filters:  An array of name-value pairs that you provide to filter the results for the <code>DescribeImportTask</code> request to a specific subset of results. Currently, wildcard values aren't supported for filters.<br/>
        :return: tasks:  A returned array of import tasks that match any applied filters, up to the specified number of maximum results.<br/>nextToken:  The token to request the next page of results.<br/>        """
        pass

    def describe_tags(self, filters: List[TagFilter]=None, maxResults: int=None, nextToken: str=None) -> DescribeTagsResponse:
        """Retrieves a list of configuration items that have tags as specified by the key-value pairs, name and value, passed to the optional parameter <code>filters</code>. There are three valid tag filter names: <ul> <li> tagKey </li> <li> tagValue </li> <li> configurationId </li> </ul> Also, all configuration items associated with your user account that have tags can be listed if you call <code>DescribeTags</code> as is without passing any parameters.

        :param self:
        :param nextToken:  A token to start the list. Use this token to get the next set of results.<br/>maxResults:  The total number of items to return in a single page of output. The maximum value is 100.<br/>filters:  You can filter the list using a <i>key</i>-<i>value</i> format. You can separate these items by using logical operators. Allowed filters include <code>tagKey</code>, <code>tagValue</code>, and <code>configurationId</code>. <br/>
        :return: nextToken:  The call returns a token. Use this token to get the next set of results.<br/>tags:  Depending on the input, this is a list of configuration items tagged with a specific tag, or a list of tags for a specific configuration item.<br/>        """
        pass

    def disassociate_configuration_items_from_application(self, configurationIds: List[ConfigurationId], applicationConfigurationId: str) -> DisassociateConfigurationItemsFromApplicationResponse:
        """Disassociates one or more configuration items from an application.

        :param self:
        :param configurationIds:  Configuration ID of each item to be disassociated from an application.<br/>applicationConfigurationId:  Configuration ID of an application from which each item is disassociated.<br/>
        :return:         """
        pass

    def export_configurations(self) -> ExportConfigurationsResponse:
        """Deprecated. Use <code>StartExportTask</code> instead. Exports all discovered configuration data to an Amazon S3 bucket or an application that enables you to view and evaluate the data. Data includes tags and tag associations, processes, connections, servers, and system performance. This API returns an export ID that you can query using the <i>DescribeExportConfigurations</i> API. The system imposes a limit of two configuration exports in six hours.

        :param self:
        :param 
        :return: exportId:  A unique identifier that you can use to query the export status.<br/>        """
        pass

    def get_discovery_summary(self) -> GetDiscoverySummaryResponse:
        """Retrieves a short summary of discovered assets. This API operation takes no request parameters and is called as is at the command prompt as shown in the example.

        :param self:
        :param 
        :return: connectorSummary:  Details about discovered connectors, including connector status and health.:<br/>agentSummary:  Details about discovered agents, including agent status and health.:<br/>serversMappedtoTags:  The number of servers mapped to tags.<br/>serversMappedToApplications:  The number of servers mapped to applications.<br/>applications:  The number of applications discovered.<br/>servers:  The number of servers discovered.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>unknownAgents</i> Number of unknown discovery agents.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>totalAgents</i> Total number of discovery agents.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>unhealthyAgents</i> Number of unhealthy discovery agents.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>shutdownAgents</i> Number of discovery agents with status SHUTDOWN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>blackListedAgents</i> Number of blacklisted discovery agents.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>healthyAgents</i> Number of healthy discovery agents<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>activeAgents</i> Number of active discovery agents.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>unknownConnectors</i> Number of unknown discovery connectors.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>totalConnectors</i> Total number of discovery connectors.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>unhealthyConnectors</i> Number of unhealthy discovery connectors.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>shutdownConnectors</i> Number of discovery connectors with status SHUTDOWN,<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>blackListedConnectors</i> Number of blacklisted discovery connectors.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>healthyConnectors</i> Number of healthy discovery connectors.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>activeConnectors</i> Number of active discovery connectors.<br/>        """
        pass

    def list_configurations(self, configurationType: str, filters: List[Filter]=None, maxResults: int=None, nextToken: str=None, orderBy: List[OrderByElement]=None) -> ListConfigurationsResponse:
        """Retrieves a list of configuration items as specified by the value passed to the required paramater <code>configurationType</code>. Optional filtering may be applied to refine search results.

        :param self:
        :param orderBy:  Certain filter criteria return output that can be sorted in ascending or descending order. For a list of output characteristics for each filter, see <a href="http://docs.aws.amazon.com/application-discovery/latest/APIReference/discovery-api-queries.html#ListConfigurations">Using the ListConfigurations Action</a>.<br/>nextToken:  Token to retrieve the next set of results. For example, if a previous call to ListConfigurations returned 100 items, but you set <code>ListConfigurationsRequest$maxResults</code> to 10, you received a set of 10 results along with a token. Use that token in this query to get the next set of 10.<br/>maxResults:  The total number of items to return. The maximum value is 100.<br/>filters:  You can filter the request using various logical operators and a <i>key</i>-<i>value</i> format. For example:   <code>{"key": "serverType", "value": "webServer"}</code>  For a complete list of filter options and guidance about using them with this action, see <a href="http://docs.aws.amazon.com/application-discovery/latest/APIReference/discovery-api-queries.html#ListConfigurations">Querying Discovered Configuration Items</a>. <br/>configurationType:  A valid configuration identified by Application Discovery Service. <br/>
        :return: nextToken:  Token to retrieve the next set of results. For example, if your call to ListConfigurations returned 100 items, but you set <code>ListConfigurationsRequest$maxResults</code> to 10, you received a set of 10 results along with this token. Use this token in the next query to retrieve the next set of 10.<br/>configurations:  Returns configuration details, including the configuration ID, attribute names, and attribute values.<br/>        """
        pass

    def list_server_neighbors(self, configurationId: str, portInformationNeeded: Boolean=None, neighborConfigurationIds: List[ConfigurationId]=None, maxResults: int=None, nextToken: str=None) -> ListServerNeighborsResponse:
        """Retrieves a list of servers that are one network hop away from a specified server.

        :param self:
        :param nextToken:  Token to retrieve the next set of results. For example, if you previously specified 100 IDs for <code>ListServerNeighborsRequest$neighborConfigurationIds</code> but set <code>ListServerNeighborsRequest$maxResults</code> to 10, you received a set of 10 results along with a token. Use that token in this query to get the next set of 10.<br/>maxResults:  Maximum number of results to return in a single page of output.<br/>neighborConfigurationIds:  List of configuration IDs to test for one-hop-away.<br/>portInformationNeeded:  Flag to indicate if port and protocol information is needed as part of the response.<br/>configurationId:  Configuration ID of the server for which neighbors are being listed.<br/>
        :return: knownDependencyCount:  Count of distinct servers that are one hop away from the given server.<br/>nextToken:  Token to retrieve the next set of results. For example, if you specified 100 IDs for <code>ListServerNeighborsRequest$neighborConfigurationIds</code> but set <code>ListServerNeighborsRequest$maxResults</code> to 10, you received a set of 10 results along with this token. Use this token in the next query to retrieve the next set of 10.<br/>neighbors:  List of distinct servers that are one hop away from the given server.<br/>        """
        pass

    def start_continuous_export(self) -> StartContinuousExportResponse:
        """Start the continuous flow of agent's discovered data into Amazon Athena.

        :param self:
        :param 
        :return: schemaStorageConfig:  A dictionary which describes how the data is stored. <ul> <li>  <code>databaseName</code> - the name of the Glue database used to store the schema. </li> </ul><br/>dataSource:  The type of data collector used to gather this data (currently only offered for AGENT).<br/>startTime:  The timestamp representing when the continuous export was started.<br/>s3Bucket:  The name of the s3 bucket where the export data parquet files are stored.<br/>exportId:  The unique ID assigned to this export.<br/>        """
        pass

    def start_data_collection_by_agent_ids(self, agentIds: List[AgentId]) -> StartDataCollectionByAgentIdsResponse:
        """Instructs the specified agents or connectors to start collecting data.

        :param self:
        :param agentIds:  The IDs of the agents or connectors from which to start collecting data. If you send a request to an agent/connector ID that you do not have permission to contact, according to your AWS account, the service does not throw an exception. Instead, it returns the error in the <i>Description</i> field. If you send a request to multiple agents/connectors and you do not have permission to contact some of those agents/connectors, the system does not throw an exception. Instead, the system shows <code>Failed</code> in the <i>Description</i> field.<br/>
        :return: agentsConfigurationStatus:  Information about agents or the connector that were instructed to start collecting data. Information includes the agent/connector ID, a description of the operation performed, and whether the agent/connector configuration was updated.<br/>        """
        pass

    def start_export_task(self, exportDataFormat: List[ExportDataFormat]=None, filters: List[ExportFilter]=None, startTime: TimeStamp=None, endTime: TimeStamp=None) -> StartExportTaskResponse:
        """ Begins the export of discovered data to an S3 bucket.  If you specify <code>agentIds</code> in a filter, the task exports up to 72 hours of detailed data collected by the identified Application Discovery Agent, including network, process, and performance details. A time range for exported agent data may be set by using <code>startTime</code> and <code>endTime</code>. Export of detailed agent data is limited to five concurrently running exports.   If you do not include an <code>agentIds</code> filter, summary data is exported that includes both AWS Agentless Discovery Connector data and summary data from AWS Discovery Agents. Export of summary data is limited to two exports per day. 

        :param self:
        :param endTime:  The end timestamp for exported data from the single Application Discovery Agent selected in the filters. If no value is specified, exported data includes the most recent data collected by the agent.<br/>startTime:  The start timestamp for exported data from the single Application Discovery Agent selected in the filters. If no value is specified, data is exported starting from the first data collected by the agent.<br/>filters:  If a filter is present, it selects the single <code>agentId</code> of the Application Discovery Agent for which data is exported. The <code>agentId</code> can be found in the results of the <code>DescribeAgents</code> API or CLI. If no filter is present, <code>startTime</code> and <code>endTime</code> are ignored and exported data includes both Agentless Discovery Connector data and summary data from Application Discovery agents. <br/>exportDataFormat:  The file format for the returned export data. Default value is <code>CSV</code>. <b>Note:</b> <i>The</i> <code>GRAPHML</code> <i>option has been deprecated.</i> <br/>
        :return: exportId:  A unique identifier used to query the status of an export request.<br/>        """
        pass

    def start_import_task(self, importUrl: str, name: str, clientRequestToken: str=None) -> StartImportTaskResponse:
        """Starts an import task, which allows you to import details of your on-premises environment directly into AWS without having to use the Application Discovery Service (ADS) tools such as the Discovery Connector or Discovery Agent. This gives you the option to perform migration assessment and planning directly from your imported data, including the ability to group your devices as applications and track their migration status. To start an import request, do this: <ol> <li> Download the specially formatted comma separated value (CSV) import template, which you can find here: <a href="https://s3-us-west-2.amazonaws.com/templates-7cffcf56-bd96-4b1c-b45b-a5b42f282e46/import_template.csv">https://s3-us-west-2.amazonaws.com/templates-7cffcf56-bd96-4b1c-b45b-a5b42f282e46/import_template.csv</a>. </li> <li> Fill out the template with your server and application data. </li> <li> Upload your import file to an Amazon S3 bucket, and make a note of it's Object URL. Your import file must be in the CSV format. </li> <li> Use the console or the <code>StartImportTask</code> command with the AWS CLI or one of the AWS SDKs to import the records from your file. </li> </ol> For more information, including step-by-step procedures, see <a href="https://docs.aws.amazon.com/application-discovery/latest/userguide/discovery-import.html">Migration Hub Import</a> in the <i>AWS Application Discovery Service User Guide</i>. <note> There are limits to the number of import tasks you can create (and delete) in an AWS account. For more information, see <a href="https://docs.aws.amazon.com/application-discovery/latest/userguide/ads_service_limits.html">AWS Application Discovery Service Limits</a> in the <i>AWS Application Discovery Service User Guide</i>. </note>

        :param self:
        :param importUrl:  The URL for your import file that you've uploaded to Amazon S3. <note> If you're using the AWS CLI, this URL is structured as follows: <code>s3://BucketName/ImportFileName.CSV</code>  </note><br/>name:  A descriptive name for this request. You can use this name to filter future requests related to this import task, such as identifying applications and servers that were included in this import task. We recommend that you use a meaningful name for each import task.<br/>clientRequestToken:  Optional. A unique token that you can provide to prevent the same import request from occurring more than once. If you don't provide a token, a token is automatically generated. Sending more than one <code>StartImportTask</code> request with the same client request token will return information about the original import task with that client request token.<br/>
        :return: task:  An array of information related to the import task request including status information, times, IDs, the Amazon S3 Object URL for the import file, and more. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorsAndFailedEntriesZip</i> A link to a compressed archive folder (in the ZIP format) that contains an error log and a file of failed records. You can use these two files to quickly identify records that failed, why they failed, and correct those records. Afterward, you can upload the corrected file to your Amazon S3 bucket and create another import task request. This field also includes authorization information so you can confirm the authenticity of the compressed archive before you download it. If some records failed to be imported we recommend that you correct the records in the failed entries file and then imports that failed entries file. This prevents you from having to correct and update the larger original file and attempt importing it again.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>applicationImportFailure</i> The total number of application records in the import file that failed to be imported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>applicationImportSuccess</i> The total number of application records in the import file that were successfully imported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serverImportFailure</i> The total number of server records in the import file that failed to be imported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serverImportSuccess</i> The total number of server records in the import file that were successfully imported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>importDeletedTime</i> The time that the import task request was deleted, presented in the Unix time stamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>importCompletionTime</i> The time that the import task request finished, presented in the Unix time stamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>importRequestTime</i> The time that the import task request was made, presented in the Unix time stamp format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the import task. An import can have the status of <code>IMPORT_COMPLETE</code> and still have some records fail to import from the overall request. More information can be found in the downloadable archive defined in the <code>errorsAndFailedEntriesZip</code> field, or in the Migration Hub management console.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>importUrl</i> The URL for your import file that you've uploaded to Amazon S3.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> A descriptive name for an import task. You can use this name to filter future requests related to this import task, such as identifying applications and servers that were included in this import task. We recommend that you use a meaningful name for each import task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clientRequestToken</i> A unique token used to prevent the same import request from occurring more than once. If you didn't provide a token, a token was automatically generated when the import task request was sent.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>importTaskId</i> The unique ID for a specific import task. These IDs aren't globally unique, but they are unique within an AWS account.<br/>        """
        pass

    def stop_continuous_export(self, exportId: str) -> StopContinuousExportResponse:
        """Stop the continuous flow of agent's discovered data into Amazon Athena.

        :param self:
        :param exportId:  The unique ID assigned to this export.<br/>
        :return: stopTime:  Timestamp that represents when this continuous export was stopped.<br/>startTime:  Timestamp that represents when this continuous export started collecting data.<br/>        """
        pass

    def stop_data_collection_by_agent_ids(self, agentIds: List[AgentId]) -> StopDataCollectionByAgentIdsResponse:
        """Instructs the specified agents or connectors to stop collecting data.

        :param self:
        :param agentIds:  The IDs of the agents or connectors from which to stop collecting data.<br/>
        :return: agentsConfigurationStatus:  Information about the agents or connector that were instructed to stop collecting data. Information includes the agent/connector ID, a description of the operation performed, and whether the agent/connector configuration was updated.<br/>        """
        pass

    def update_application(self, configurationId: str, name: str=None, description: str=None) -> UpdateApplicationResponse:
        """Updates metadata about an application.

        :param self:
        :param description:  New description of the application to be updated.<br/>name:  New name of the application to be updated.<br/>configurationId:  Configuration ID of the application to be updated.<br/>
        :return:         """
        pass



class DLM(BaseClient):
    """<fullname>Amazon Data Lifecycle Manager</fullname> With Amazon Data Lifecycle Manager, you can manage the lifecycle of your AWS resources. You create lifecycle policies, which are used to automate operations on the specified resources. Amazon DLM supports Amazon EBS volumes and snapshots. For information about using Amazon DLM with Amazon EBS, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/snapshot-lifecycle.html">Automating the Amazon EBS Snapshot Lifecycle</a> in the <i>Amazon EC2 User Guide</i>."""




    class CreateLifecyclePolicyRequest(Mapping):
        pass
    
    class CreateLifecyclePolicyResponse(Mapping):
        pass
    
    class PolicyDetails(Mapping):
        pass
    
    class DeleteLifecyclePolicyRequest(Mapping):
        pass
    
    class DeleteLifecyclePolicyResponse(Mapping):
        pass
    
    class GetLifecyclePoliciesRequest(Mapping):
        pass
    
    class GetLifecyclePoliciesResponse(Mapping):
        pass
    
    class PolicyId(object):
        pass
    
    class ResourceTypeValues(object):
        pass
    
    class TagFilter(object):
        pass
    
    class GetLifecyclePolicyRequest(Mapping):
        pass
    
    class GetLifecyclePolicyResponse(Mapping):
        pass
    
    class UpdateLifecyclePolicyRequest(Mapping):
        pass
    
    class UpdateLifecyclePolicyResponse(Mapping):
        pass
    
    def create_lifecycle_policy(self, PolicyDetails: PolicyDetails, State: str, Description: str, ExecutionRoleArn: str) -> CreateLifecyclePolicyResponse:
        """Creates a policy to manage the lifecycle of the specified AWS resources. You can create up to 100 lifecycle policies.

        :param self:
        :param PolicyDetails:  The configuration details of the lifecycle policy. Target tags cannot be re-used across lifecycle policies.:<br/>State:  The desired activation state of the lifecycle policy after creation.<br/>Description:  A description of the lifecycle policy. The characters ^[0-9A-Za-z _-]+$ are supported.<br/>ExecutionRoleArn:  The Amazon Resource Name (ARN) of the IAM role used to run the operations specified by the lifecycle policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> A set of optional parameters that can be provided by the policy. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Parameters</b>: A set of optional parameters that can be provided by the policy. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Schedules</i> The schedule of policy-defined actions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetTags</i> The single tag that identifies targeted resources for this policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceTypes</i> The resource type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyType</i> This field determines the valid target resource types and actions a policy can manage. This field defaults to EBS_SNAPSHOT_MANAGEMENT if not present.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExcludeBootVolume</i> When executing an EBS Snapshot Management – Instance policy, execute all CreateSnapshots calls with the <code>excludeBootVolume</code> set to the supplied field. Defaults to false. Only valid for EBS Snapshot Management – Instance policies.<br/>
        :return: PolicyId:  The identifier of the lifecycle policy.<br/>        """
        pass

    def delete_lifecycle_policy(self, PolicyId: str) -> DeleteLifecyclePolicyResponse:
        """Deletes the specified lifecycle policy and halts the automated operations that the policy specified.

        :param self:
        :param PolicyId:  The identifier of the lifecycle policy.<br/>
        :return:         """
        pass

    def get_lifecycle_policies(self, PolicyIds: List[PolicyId]=None, State: str=None, ResourceTypes: List[ResourceTypeValues]=None, TargetTags: List[TagFilter]=None, TagsToAdd: List[TagFilter]=None) -> GetLifecyclePoliciesResponse:
        """Gets summary information about all or the specified data lifecycle policies. To get complete information about a policy, use <a>GetLifecyclePolicy</a>.

        :param self:
        :param TagsToAdd:  The tags to add to objects created by the policy. Tags are strings in the format <code>key=value</code>. These user-defined tags are added in addition to the AWS-added lifecycle tags.<br/>TargetTags:  The target tag for a policy. Tags are strings in the format <code>key=value</code>.<br/>ResourceTypes:  The resource type.<br/>State:  The activation state.<br/>PolicyIds:  The identifiers of the data lifecycle policies.<br/>
        :return: Policies:  Summary information about the lifecycle policies.<br/>        """
        pass

    def get_lifecycle_policy(self, PolicyId: str) -> GetLifecyclePolicyResponse:
        """Gets detailed information about the specified lifecycle policy.

        :param self:
        :param PolicyId:  The identifier of the lifecycle policy.<br/>
        :return: Policy:  Detailed information about the lifecycle policy.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyDetails</i> The configuration of the lifecycle policy&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PolicyDetails</b>: The configuration of the lifecycle policy<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DateModified</i> The local date and time when the lifecycle policy was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DateCreated</i> The local date and time when the lifecycle policy was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExecutionRoleArn</i> The Amazon Resource Name (ARN) of the IAM role used to run the operations specified by the lifecycle policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The activation state of the lifecycle policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the lifecycle policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyId</i> The identifier of the lifecycle policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Schedules</i> The schedule of policy-defined actions.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetTags</i> The single tag that identifies targeted resources for this policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceTypes</i> The resource type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyType</i> This field determines the valid target resource types and actions a policy can manage. This field defaults to EBS_SNAPSHOT_MANAGEMENT if not present.<br/>A set of optional parameters that can be provided by the policy. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Parameters</b>: A set of optional parameters that can be provided by the policy.         """
        pass

    def update_lifecycle_policy(self, PolicyId: str, ExecutionRoleArn: str=None, State: str=None, Description: str=None, PolicyDetails: PolicyDetails=None) -> UpdateLifecyclePolicyResponse:
        """Updates the specified lifecycle policy.

        :param self:
        :param PolicyDetails:  The configuration of the lifecycle policy. Target tags cannot be re-used across policies.:<br/>Description:  A description of the lifecycle policy.<br/>State:  The desired activation state of the lifecycle policy after creation.<br/>ExecutionRoleArn:  The Amazon Resource Name (ARN) of the IAM role used to run the operations specified by the lifecycle policy.<br/>PolicyId:  The identifier of the lifecycle policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> A set of optional parameters that can be provided by the policy. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Parameters</b>: A set of optional parameters that can be provided by the policy. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Schedules</i> The schedule of policy-defined actions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetTags</i> The single tag that identifies targeted resources for this policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceTypes</i> The resource type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyType</i> This field determines the valid target resource types and actions a policy can manage. This field defaults to EBS_SNAPSHOT_MANAGEMENT if not present.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExcludeBootVolume</i> When executing an EBS Snapshot Management – Instance policy, execute all CreateSnapshots calls with the <code>excludeBootVolume</code> set to the supplied field. Defaults to false. Only valid for EBS Snapshot Management – Instance policies.<br/>
        :return:         """
        pass



class DatabaseMigrationService(BaseClient):
    """<fullname>AWS Database Migration Service</fullname> AWS Database Migration Service (AWS DMS) can migrate your data to and from the most widely used commercial and open-source databases such as Oracle, PostgreSQL, Microsoft SQL Server, Amazon Redshift, MariaDB, Amazon Aurora, MySQL, and SAP Adaptive Server Enterprise (ASE). The service supports homogeneous migrations such as Oracle to Oracle, as well as heterogeneous migrations between different database platforms, such as Oracle to MySQL or SQL Server to PostgreSQL. For more information about AWS DMS, see <a href="https://docs.aws.amazon.com/dms/latest/userguide/Welcome.html">What Is AWS Database Migration Service?</a> in the <i>AWS Database Migration User Guide.</i> """

    class EndpointDeletedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_endpoints` for valid parameters"""
            pass
    class ReplicationInstanceAvailableWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_replication_instances` for valid parameters"""
            pass
    class ReplicationInstanceDeletedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_replication_instances` for valid parameters"""
            pass
    class ReplicationTaskDeletedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_replication_tasks` for valid parameters"""
            pass
    class ReplicationTaskReadyWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_replication_tasks` for valid parameters"""
            pass
    class ReplicationTaskRunningWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_replication_tasks` for valid parameters"""
            pass
    class ReplicationTaskStoppedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_replication_tasks` for valid parameters"""
            pass
    class TestConnectionSucceedsWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_connections` for valid parameters"""
            pass


    class DescribeSchemasPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_schemas` for valid parameters"""
            pass
    class DescribeCertificatesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_certificates` for valid parameters"""
            pass
    class DescribeEndpointsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_endpoints` for valid parameters"""
            pass
    class DescribeEventSubscriptionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_event_subscriptions` for valid parameters"""
            pass
    class DescribeEndpointTypesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_endpoint_types` for valid parameters"""
            pass
    class DescribeReplicationInstancesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_replication_instances` for valid parameters"""
            pass
    class DescribeTableStatisticsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_table_statistics` for valid parameters"""
            pass
    class DescribeConnectionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_connections` for valid parameters"""
            pass
    class DescribeReplicationTaskAssessmentResultsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_replication_task_assessment_results` for valid parameters"""
            pass
    class DescribeEventsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_events` for valid parameters"""
            pass
    class DescribeOrderableReplicationInstancesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_orderable_replication_instances` for valid parameters"""
            pass
    class DescribeReplicationSubnetGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_replication_subnet_groups` for valid parameters"""
            pass
    class DescribeReplicationTasksPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_replication_tasks` for valid parameters"""
            pass


    class AddTagsToResourceMessage(Mapping):
        pass
    
    class AddTagsToResourceResponse(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class ApplyPendingMaintenanceActionMessage(Mapping):
        pass
    
    class ApplyPendingMaintenanceActionResponse(Mapping):
        pass
    
    class CreateEndpointMessage(Mapping):
        pass
    
    class CreateEndpointResponse(Mapping):
        pass
    
    class DynamoDbSettings(Mapping):
        pass
    
    class S3Settings(Mapping):
        pass
    
    class DmsTransferSettings(Mapping):
        pass
    
    class MongoDbSettings(Mapping):
        pass
    
    class KinesisSettings(Mapping):
        pass
    
    class ElasticsearchSettings(Mapping):
        pass
    
    class RedshiftSettings(Mapping):
        pass
    
    class CreateEventSubscriptionMessage(Mapping):
        pass
    
    class CreateEventSubscriptionResponse(Mapping):
        pass
    
    class String(object):
        pass
    
    class BooleanOptional(object):
        pass
    
    class CreateReplicationInstanceMessage(Mapping):
        pass
    
    class CreateReplicationInstanceResponse(Mapping):
        pass
    
    class CreateReplicationSubnetGroupMessage(Mapping):
        pass
    
    class CreateReplicationSubnetGroupResponse(Mapping):
        pass
    
    class CreateReplicationTaskMessage(Mapping):
        pass
    
    class CreateReplicationTaskResponse(Mapping):
        pass
    
    class TStamp(object):
        pass
    
    class DeleteCertificateMessage(Mapping):
        pass
    
    class DeleteCertificateResponse(Mapping):
        pass
    
    class DeleteEndpointMessage(Mapping):
        pass
    
    class DeleteEndpointResponse(Mapping):
        pass
    
    class DeleteEventSubscriptionMessage(Mapping):
        pass
    
    class DeleteEventSubscriptionResponse(Mapping):
        pass
    
    class DeleteReplicationInstanceMessage(Mapping):
        pass
    
    class DeleteReplicationInstanceResponse(Mapping):
        pass
    
    class DeleteReplicationSubnetGroupMessage(Mapping):
        pass
    
    class DeleteReplicationSubnetGroupResponse(Mapping):
        pass
    
    class DeleteReplicationTaskMessage(Mapping):
        pass
    
    class DeleteReplicationTaskResponse(Mapping):
        pass
    
    class DescribeAccountAttributesMessage(Mapping):
        pass
    
    class DescribeAccountAttributesResponse(Mapping):
        pass
    
    class DescribeCertificatesMessage(Mapping):
        pass
    
    class DescribeCertificatesResponse(Mapping):
        pass
    
    class Filter(Mapping):
        pass
    
    class DescribeConnectionsMessage(Mapping):
        pass
    
    class DescribeConnectionsResponse(Mapping):
        pass
    
    class DescribeEndpointTypesMessage(Mapping):
        pass
    
    class DescribeEndpointTypesResponse(Mapping):
        pass
    
    class DescribeEndpointsMessage(Mapping):
        pass
    
    class DescribeEndpointsResponse(Mapping):
        pass
    
    class DescribeEventCategoriesMessage(Mapping):
        pass
    
    class DescribeEventCategoriesResponse(Mapping):
        pass
    
    class DescribeEventSubscriptionsMessage(Mapping):
        pass
    
    class DescribeEventSubscriptionsResponse(Mapping):
        pass
    
    class DescribeEventsMessage(Mapping):
        pass
    
    class DescribeEventsResponse(Mapping):
        pass
    
    class DescribeOrderableReplicationInstancesMessage(Mapping):
        pass
    
    class DescribeOrderableReplicationInstancesResponse(Mapping):
        pass
    
    class DescribePendingMaintenanceActionsMessage(Mapping):
        pass
    
    class DescribePendingMaintenanceActionsResponse(Mapping):
        pass
    
    class DescribeRefreshSchemasStatusMessage(Mapping):
        pass
    
    class DescribeRefreshSchemasStatusResponse(Mapping):
        pass
    
    class DescribeReplicationInstanceTaskLogsMessage(Mapping):
        pass
    
    class DescribeReplicationInstanceTaskLogsResponse(Mapping):
        pass
    
    class DescribeReplicationInstancesMessage(Mapping):
        pass
    
    class DescribeReplicationInstancesResponse(Mapping):
        pass
    
    class DescribeReplicationSubnetGroupsMessage(Mapping):
        pass
    
    class DescribeReplicationSubnetGroupsResponse(Mapping):
        pass
    
    class DescribeReplicationTaskAssessmentResultsMessage(Mapping):
        pass
    
    class DescribeReplicationTaskAssessmentResultsResponse(Mapping):
        pass
    
    class DescribeReplicationTasksMessage(Mapping):
        pass
    
    class DescribeReplicationTasksResponse(Mapping):
        pass
    
    class DescribeSchemasMessage(Mapping):
        pass
    
    class DescribeSchemasResponse(Mapping):
        pass
    
    class DescribeTableStatisticsMessage(Mapping):
        pass
    
    class DescribeTableStatisticsResponse(Mapping):
        pass
    
    class ImportCertificateMessage(Mapping):
        pass
    
    class ImportCertificateResponse(Mapping):
        pass
    
    class CertificateWallet(object):
        pass
    
    class ListTagsForResourceMessage(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class ModifyEndpointMessage(Mapping):
        pass
    
    class ModifyEndpointResponse(Mapping):
        pass
    
    class ModifyEventSubscriptionMessage(Mapping):
        pass
    
    class ModifyEventSubscriptionResponse(Mapping):
        pass
    
    class ModifyReplicationInstanceMessage(Mapping):
        pass
    
    class ModifyReplicationInstanceResponse(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class ModifyReplicationSubnetGroupMessage(Mapping):
        pass
    
    class ModifyReplicationSubnetGroupResponse(Mapping):
        pass
    
    class ModifyReplicationTaskMessage(Mapping):
        pass
    
    class ModifyReplicationTaskResponse(Mapping):
        pass
    
    class RebootReplicationInstanceMessage(Mapping):
        pass
    
    class RebootReplicationInstanceResponse(Mapping):
        pass
    
    class RefreshSchemasMessage(Mapping):
        pass
    
    class RefreshSchemasResponse(Mapping):
        pass
    
    class ReloadTablesMessage(Mapping):
        pass
    
    class ReloadTablesResponse(Mapping):
        pass
    
    class TableToReload(Mapping):
        pass
    
    class RemoveTagsFromResourceMessage(Mapping):
        pass
    
    class RemoveTagsFromResourceResponse(Mapping):
        pass
    
    class StartReplicationTaskMessage(Mapping):
        pass
    
    class StartReplicationTaskResponse(Mapping):
        pass
    
    class StartReplicationTaskAssessmentMessage(Mapping):
        pass
    
    class StartReplicationTaskAssessmentResponse(Mapping):
        pass
    
    class StopReplicationTaskMessage(Mapping):
        pass
    
    class StopReplicationTaskResponse(Mapping):
        pass
    
    class TestConnectionMessage(Mapping):
        pass
    
    class TestConnectionResponse(Mapping):
        pass
    
    def add_tags_to_resource(self, Tags: List[Tag], ResourceArn: str) -> AddTagsToResourceResponse:
        """Adds metadata tags to an AWS DMS resource, including replication instance, endpoint, security group, and migration task. These tags can also be used with cost allocation reporting to track cost associated with DMS resources, or used in a Condition statement in an IAM policy for DMS.

        :param self:
        :param Tags:  One or more tags to be assigned to the resource.<br/>ResourceArn:  Identifies the AWS DMS resource to which tags should be added. The value for this parameter is an Amazon Resource Name (ARN). For AWS DMS, you can tag a replication instance, an endpoint, or a replication task.<br/>
        :return:         """
        pass

    def apply_pending_maintenance_action(self, OptInType: str, ApplyAction: str, ReplicationInstanceArn: str) -> ApplyPendingMaintenanceActionResponse:
        """Applies a pending maintenance action to a resource (for example, to a replication instance).

        :param self:
        :param OptInType:  A value that specifies the type of opt-in request, or undoes an opt-in request. You can't undo an opt-in request of type <code>immediate</code>. Valid values: <ul> <li>  <code>immediate</code> - Apply the maintenance action immediately. </li> <li>  <code>next-maintenance</code> - Apply the maintenance action during the next maintenance window for the resource. </li> <li>  <code>undo-opt-in</code> - Cancel any existing <code>next-maintenance</code> opt-in requests. </li> </ul><br/>ApplyAction:  The pending maintenance action to apply to this resource.<br/>ReplicationInstanceArn:  The Amazon Resource Name (ARN) of the AWS DMS resource that the pending maintenance action applies to.<br/>
        :return: ResourcePendingMaintenanceActions:  The AWS DMS resource that the pending maintenance action will be applied to.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingMaintenanceActionDetails</i> Detailed information about the pending maintenance action.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceIdentifier</i> The Amazon Resource Name (ARN) of the DMS resource that the pending maintenance action applies to. For information about creating an ARN, see <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Introduction.AWS.ARN.html"> Constructing an Amazon Resource Name (ARN) for AWS DMS</a> in the DMS documentation.<br/>        """
        pass

    def create_endpoint(self, EngineName: str, EndpointType: str, EndpointIdentifier: str, Username: str=None, Password: str=None, ServerName: str=None, Port: int=None, DatabaseName: str=None, ExtraConnectionAttributes: str=None, KmsKeyId: str=None, Tags: List[Tag]=None, CertificateArn: str=None, SslMode: str=None, ServiceAccessRoleArn: str=None, ExternalTableDefinition: str=None, DynamoDbSettings: DynamoDbSettings=None, S3Settings: S3Settings=None, DmsTransferSettings: DmsTransferSettings=None, MongoDbSettings: MongoDbSettings=None, KinesisSettings: KinesisSettings=None, ElasticsearchSettings: ElasticsearchSettings=None, RedshiftSettings: RedshiftSettings=None) -> CreateEndpointResponse:
        """Creates an endpoint using the provided settings.

        :param self:
        :param RedshiftSettings:  <p/>:<br/>ElasticsearchSettings:  Settings in JSON format for the target Elasticsearch endpoint. For more information about the available settings, see <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Target.Elasticsearch.html#CHAP_Target.Elasticsearch.Configuration">Extra Connection Attributes When Using Elasticsearch as a Target for AWS DMS</a> in the <i>AWS Database Migration User Guide.</i> :<br/>KinesisSettings:  Settings in JSON format for the target Amazon Kinesis Data Streams endpoint. For more information about the available settings, see <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Target.Kinesis.html#CHAP_Target.Kinesis.ObjectMapping">Using Object Mapping to Migrate Data to a Kinesis Data Stream</a> in the <i>AWS Database Migration User Guide.</i> :<br/>MongoDbSettings:  Settings in JSON format for the source MongoDB endpoint. For more information about the available settings, see the configuration properties section in <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Source.MongoDB.html"> Using MongoDB as a Target for AWS Database Migration Service</a> in the <i>AWS Database Migration Service User Guide.</i> :<br/>DmsTransferSettings:  The settings in JSON format for the DMS transfer type of source endpoint.  Possible attributes include the following: <ul> <li>  <code>serviceAccessRoleArn</code> - The IAM role that has permission to access the Amazon S3 bucket. </li> <li>  <code>bucketName</code> - The name of the S3 bucket to use. </li> <li>  <code>compressionType</code> - An optional parameter to use GZIP to compress the target files. To use GZIP, set this value to <code>NONE</code> (the default). To keep the files uncompressed, don't use this value. </li> </ul> Shorthand syntax for these attributes is as follows: <code>ServiceAccessRoleArn=string,BucketName=string,CompressionType=string</code>  JSON syntax for these attributes is as follows: <code>{ "ServiceAccessRoleArn": "string", "BucketName": "string", "CompressionType": "none"|"gzip" } </code> :<br/>S3Settings:  Settings in JSON format for the target Amazon S3 endpoint. For more information about the available settings, see <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Target.S3.html#CHAP_Target.S3.Configuring">Extra Connection Attributes When Using Amazon S3 as a Target for AWS DMS</a> in the <i>AWS Database Migration Service User Guide.</i> :<br/>DynamoDbSettings:  Settings in JSON format for the target Amazon DynamoDB endpoint. For more information about the available settings, see <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Target.DynamoDB.html">Using Object Mapping to Migrate Data to DynamoDB</a> in the <i>AWS Database Migration Service User Guide.</i> :<br/>ExternalTableDefinition:  The external table definition. <br/>ServiceAccessRoleArn:   The Amazon Resource Name (ARN) for the service access role that you want to use to create the endpoint. <br/>SslMode:  The Secure Sockets Layer (SSL) mode to use for the SSL connection. The default is <code>none</code> <br/>CertificateArn:  The Amazon Resource Name (ARN) for the certificate.<br/>Tags:  One or more tags to be assigned to the endpoint.<br/>KmsKeyId:  An AWS KMS key identifier that is used to encrypt the connection parameters for the endpoint. If you don't specify a value for the <code>KmsKeyId</code> parameter, then AWS DMS uses your default encryption key. AWS KMS creates the default encryption key for your AWS account. Your AWS account has a different default encryption key for each AWS Region.<br/>ExtraConnectionAttributes:  Additional attributes associated with the connection. Each attribute is specified as a name-value pair associated by an equal sign (=). Multiple attributes are separated by a semicolon (;) with no additional white space. For information on the attributes available for connecting your source or target endpoint, see <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Endpoints.html">Working with AWS DMS Endpoints</a> in the <i>AWS Database Migration Service User Guide.</i> <br/>DatabaseName:  The name of the endpoint database.<br/>Port:  The port used by the endpoint database.<br/>ServerName:  The name of the server where the endpoint database resides.<br/>Password:  The password to be used to log in to the endpoint database.<br/>Username:  The user name to be used to log in to the endpoint database.<br/>EngineName:  The type of engine for the endpoint. Valid values, depending on the <code>EndpointType</code> value, include <code>mysql</code>, <code>oracle</code>, <code>postgres</code>, <code>mariadb</code>, <code>aurora</code>, <code>aurora-postgresql</code>, <code>redshift</code>, <code>s3</code>, <code>db2</code>, <code>azuredb</code>, <code>sybase</code>, <code>dynamodb</code>, <code>mongodb</code>, and <code>sqlserver</code>.<br/>EndpointType:  The type of endpoint. Valid values are <code>source</code> and <code>target</code>.<br/>EndpointIdentifier:  The database endpoint identifier. Identifiers must begin with a letter; must contain only ASCII letters, digits, and hyphens; and must not end with a hyphen or contain two consecutive hyphens.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i>  The Amazon Resource Name (ARN) used by the service access IAM role. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimestampColumnName</i> A value that includes a timestamp column in the Amazon S3 target endpoint data. AWS DMS includes an additional column in the migrated data when you set <code>timestampColumnName</code> to a non-blank value.  <note> AWS DMS supports <code>TimestampColumnName</code> in versions 3.1.4 and later. </note> For a full load, each row of the timestamp column contains a timestamp for when the data was transferred from the source to the target by DMS. For a CDC load, each row of the timestamp column contains the timestamp for the commit of that row in the source database. The format for the timestamp column value is <code>yyyy-MM-dd HH:mm:ss.SSSSSS</code>. For CDC, the microsecond precision depends on the commit timestamp supported by DMS for the source database. When the <code>AddColumnName</code> setting is set to <code>true</code>, DMS also includes the name for the timestamp column that you set as the nonblank value of <code>timestampColumnName</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CdcInsertsOnly</i> A value that enables a change data capture (CDC) load to write only INSERT operations to .csv or columnar storage (.parquet) output files. By default (the <code>false</code> setting), the first field in a .csv or .parquet record contains the letter I (INSERT), U (UPDATE), or D (DELETE). These values indicate whether the row was inserted, updated, or deleted at the source database for a CDC load to the target. If <code>cdcInsertsOnly</code> is set to <code>true</code> or <code>y</code>, only INSERTs from the source database are migrated to the .csv or .parquet file. For .csv format only, how these INSERTs are recorded depends on the value of <code>IncludeOpForFullLoad</code>. If <code>IncludeOpForFullLoad</code> is set to <code>true</code>, the first field of every CDC record is set to I to indicate the INSERT operation at the source. If <code>IncludeOpForFullLoad</code> is set to <code>false</code>, every CDC record is written without a first field to indicate the INSERT operation at the source. For more information about how these settings work together, see <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Target.S3.html#CHAP_Target.S3.Configuring.InsertOps">Indicating Source DB Operations in Migrated S3 Data</a> in the <i>AWS Database Migration Service User Guide.</i>. <note> AWS DMS supports this interaction between <code>CdcInsertsOnly</code> and <code>IncludeOpForFullLoad</code> in versions 3.1.4 and later.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeOpForFullLoad</i> A value that enables a full load to write INSERT operations to the comma-separated value (.csv) output files only to indicate how the rows were added to the source database. <note> AWS DMS supports <code>IncludeOpForFullLoad</code> in versions 3.1.4 and later. </note> For full load, records can only be inserted. By default (the <code>false</code> setting), no information is recorded in these output files for a full load to indicate that the rows were inserted at the source database. If <code>IncludeOpForFullLoad</code> is set to <code>true</code> or <code>y</code>, the INSERT is recorded as an I annotation in the first field of the .csv file. This allows the format of your target records from a full load to be consistent with the target records from a CDC load. <note> This setting works together with <code>CdcInsertsOnly</code> for output to .csv files only. For more information about how these settings work together, see <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Target.S3.html#CHAP_Target.S3.Configuring.InsertOps">Indicating Source DB Operations in Migrated S3 Data</a> in the <i>AWS Database Migration Service User Guide.</i>. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnableStatistics</i> A value that enables statistics for Parquet pages and row groups. Choose <code>true</code> to enable statistics, <code>false</code> to disable. Statistics include <code>NULL</code>, <code>DISTINCT</code>, <code>MAX</code>, and <code>MIN</code> values. This parameter defaults to <code>true</code>. This value is used for .parquet file format only.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParquetVersion</i> The version of the Apache Parquet format that you want to use: <code>parquet_1_0</code> (the default) or <code>parquet_2_0</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataPageSize</i> The size of one data page in bytes. This parameter defaults to 1024 * 1024 bytes (1 MiB). This number is used for .parquet file format only. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RowGroupLength</i> The number of rows in a row group. A smaller row group size provides faster reads. But as the number of row groups grows, the slower writes become. This parameter defaults to 10,000 rows. This number is used for .parquet file format only.  If you choose a value larger than the maximum, <code>RowGroupLength</code> is set to the max row group length in bytes (64 * 1024 * 1024). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DictPageSizeLimit</i> The maximum size of an encoded dictionary page of a column. If the dictionary page exceeds this, this column is stored using an encoding type of <code>PLAIN</code>. This parameter defaults to 1024 * 1024 bytes (1 MiB), the maximum size of a dictionary page before it reverts to <code>PLAIN</code> encoding. This size is used for .parquet file format only. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncodingType</i> The type of encoding you are using:  <ul> <li>  <code>RLE_DICTIONARY</code> uses a combination of bit-packing and run-length encoding to store repeated values more efficiently. This is the default. </li> <li>  <code>PLAIN</code> doesn't use encoding at all. Values are stored as they are. </li> <li>  <code>PLAIN_DICTIONARY</code> builds a dictionary of the values encountered in a given column. The dictionary is stored in a dictionary page for each column chunk. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataFormat</i> The format of the data that you want to use for output. You can choose one of the following:  <ul> <li>  <code>csv</code> : This is a row-based file format with comma-separated values (.csv).  </li> <li>  <code>parquet</code> : Apache Parquet (.parquet) is a columnar storage file format that features efficient compression and provides faster query response.  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerSideEncryptionKmsKeyId</i> If you are using <code>SSE_KMS</code> for the <code>EncryptionMode</code>, provide the AWS KMS key ID. The key that you use needs an attached policy that enables AWS Identity and Access Management (IAM) user permissions and allows use of the key. Here is a CLI example: <code>aws dms create-endpoint --endpoint-identifier <i>value</i> --endpoint-type target --engine-name s3 --s3-settings ServiceAccessRoleArn=<i>value</i>,BucketFolder=<i>value</i>,BucketName=<i>value</i>,EncryptionMode=SSE_KMS,ServerSideEncryptionKmsKeyId=<i>value</i> </code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionMode</i> The type of server-side encryption that you want to use for your data. This encryption type is part of the endpoint settings or the extra connections attributes for Amazon S3. You can choose either <code>SSE_S3</code> (the default) or <code>SSE_KMS</code>. To use <code>SSE_S3</code>, you need an AWS Identity and Access Management (IAM) role with permission to allow <code>"arn:aws:s3:::dms-*"</code> to use the following actions: <ul> <li>  <code>s3:CreateBucket</code>  </li> <li>  <code>s3:ListBucket</code>  </li> <li>  <code>s3:DeleteBucket</code>  </li> <li>  <code>s3:GetBucketLocation</code>  </li> <li>  <code>s3:GetObject</code>  </li> <li>  <code>s3:PutObject</code>  </li> <li>  <code>s3:DeleteObject</code>  </li> <li>  <code>s3:GetObjectVersion</code>  </li> <li>  <code>s3:GetBucketPolicy</code>  </li> <li>  <code>s3:PutBucketPolicy</code>  </li> <li>  <code>s3:DeleteBucketPolicy</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompressionType</i>  An optional parameter to use GZIP to compress the target files. Set to GZIP to compress the target files. Set to NONE (the default) or do not use to leave the files uncompressed. Applies to both .csv and .parquet file formats. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketName</i>  The name of the S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketFolder</i>  An optional parameter to set a folder name in the S3 bucket. If provided, tables are created in the path <code> <i>bucketFolder</i>/<i>schema_name</i>/<i>table_name</i>/</code>. If this parameter is not specified, then the path used is <code> <i>schema_name</i>/<i>table_name</i>/</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CsvDelimiter</i>  The delimiter used to separate columns in the source files. The default is a comma. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CsvRowDelimiter</i>  The delimiter used to separate rows in the source files. The default is a carriage return (<code>\n</code>). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalTableDefinition</i>  The external table definition. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i>  The Amazon Resource Name (ARN) used by the service access IAM role. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketName</i>  The name of the S3 bucket to use. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i>  The IAM role that has permission to access the Amazon S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS KMS key identifier that is used to encrypt the content on the replication instance. If you don't specify a value for the <code>KmsKeyId</code> parameter, then AWS DMS uses your default encryption key. AWS KMS creates the default encryption key for your AWS account. Your AWS account has a different default encryption key for each AWS Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthSource</i>  The MongoDB database name. This attribute is not used when <code>authType=NO</code>.  The default is admin.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DocsToInvestigate</i>  Indicates the number of documents to preview to determine the document organization. Use this attribute when <code>NestingLevel</code> is set to ONE.  Must be a positive value greater than 0. Default value is 1000.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExtractDocId</i>  Specifies the document ID. Use this attribute when <code>NestingLevel</code> is set to NONE.  Default value is false. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NestingLevel</i>  Specifies either document or table mode.  Valid values: NONE, ONE Default value is NONE. Specify NONE to use document mode. Specify ONE to use table mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthMechanism</i>  The authentication mechanism you use to access the MongoDB source endpoint. Valid values: DEFAULT, MONGODB_CR, SCRAM_SHA_1  DEFAULT – For MongoDB version 2.x, use MONGODB_CR. For MongoDB version 3.x, use SCRAM_SHA_1. This attribute is not used when authType=No.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthType</i>  The authentication type you use to access the MongoDB source endpoint. Valid values: NO, PASSWORD  When NO is selected, user name and password parameters are not used and can be empty. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i>  The database name on the MongoDB source endpoint. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i>  The port value for the MongoDB source endpoint. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerName</i>  The name of the server on the MongoDB source endpoint. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Password</i>  The password for the user account you use to access the MongoDB source endpoint. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> The user name you use to access the MongoDB source endpoint. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i> The Amazon Resource Name (ARN) for the IAM role that DMS uses to write to the Amazon Kinesis data stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MessageFormat</i> The output format for the records created on the endpoint. The message format is <code>JSON</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamArn</i> The Amazon Resource Name (ARN) for the Amazon Kinesis Data Streams endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorRetryDuration</i> The maximum number of seconds that DMS retries failed API requests to the Elasticsearch cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FullLoadErrorPercentage</i> The maximum percentage of records that can fail to be written before a full load operation stops. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointUri</i> The endpoint for the Elasticsearch cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i> The Amazon Resource Name (ARN) used by service to access the IAM role.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WriteBufferSize</i> The size of the write buffer to use in rows. Valid values range from 1 through 2,048. The default is 1,024. Use this setting to tune performance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> An Amazon Redshift user name for a registered user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TruncateColumns</i> A value that specifies to truncate data in columns to the appropriate number of characters, so that the data fits in the column. This parameter applies only to columns with a VARCHAR or CHAR data type, and rows with a size of 4 MB or less. Choose <code>true</code> to truncate data. The default is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrimBlanks</i> A value that specifies to remove the trailing white space characters from a VARCHAR string. This parameter applies only to columns with a VARCHAR data type. Choose <code>true</code> to remove unneeded white space. The default is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeFormat</i> The time format that you want to use. Valid values are <code>auto</code> (case-sensitive), <code>'timeformat_string'</code>, <code>'epochsecs'</code>, or <code>'epochmillisecs'</code>. It defaults to 10. Using <code>auto</code> recognizes most strings, even some that aren't supported when you use a time format string.  If your date and time values use formats different from each other, set this parameter to <code>auto</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerSideEncryptionKmsKeyId</i> The AWS KMS key ID. If you are using <code>SSE_KMS</code> for the <code>EncryptionMode</code>, provide this key ID. The key that you use needs an attached policy that enables IAM user permissions and allows use of the key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i> The Amazon Resource Name (ARN) of the IAM role that has access to the Amazon Redshift service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerName</i> The name of the Amazon Redshift cluster you are using.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplaceChars</i> A value that specifies to replaces the invalid characters specified in <code>ReplaceInvalidChars</code>, substituting the specified characters instead. The default is <code>"?"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplaceInvalidChars</i> A list of characters that you want to replace. Use with <code>ReplaceChars</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RemoveQuotes</i> A value that specifies to remove surrounding quotation marks from strings in the incoming data. All characters within the quotation marks, including delimiters, are retained. Choose <code>true</code> to remove quotation marks. The default is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port number for Amazon Redshift. The default value is 5439.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Password</i> The password for the user named in the <code>username</code> property.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxFileSize</i> The maximum size (in KB) of any .csv file used to transfer data to Amazon Redshift. This accepts a value from 1 through 1,048,576. It defaults to 32,768 KB (32 MB).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LoadTimeout</i> The amount of time to wait (in milliseconds) before timing out, beginning from when you begin loading.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FileTransferUploadStreams</i> The number of threads used to upload a single file. This parameter accepts a value from 1 through 64. It defaults to 10.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionMode</i> The type of server-side encryption that you want to use for your data. This encryption type is part of the endpoint settings or the extra connections attributes for Amazon S3. You can choose either <code>SSE_S3</code> (the default) or <code>SSE_KMS</code>. To use <code>SSE_S3</code>, create an AWS Identity and Access Management (IAM) role with a policy that allows <code>"arn:aws:s3:::*"</code> to use the following actions: <code>"s3:PutObject", "s3:ListBucket"</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EmptyAsNull</i> A value that specifies whether AWS DMS should migrate empty CHAR and VARCHAR fields as NULL. A value of <code>true</code> sets empty CHAR and VARCHAR fields to null. The default is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DateFormat</i> The date format that you are using. Valid values are <code>auto</code> (case-sensitive), your date format string enclosed in quotes, or NULL. If this parameter is left unset (NULL), it defaults to a format of 'YYYY-MM-DD'. Using <code>auto</code> recognizes most strings, even some that aren't supported when you use a date format string.  If your date and time values use formats different from each other, set this to <code>auto</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> The name of the Amazon Redshift data warehouse (service) that you are working with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectionTimeout</i> A value that sets the amount of time to wait (in milliseconds) before timing out, beginning from when you initially establish a connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketName</i> The name of the S3 bucket you want to use<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketFolder</i> The location where the comma-separated value (.csv) files are stored before being uploaded to the S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AfterConnectScript</i> Code to run after connecting. This parameter should contain the code itself, not the name of a file containing the code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AcceptAnyDate</i> A value that indicates to allow any date format, including invalid formats such as 00/00/00 00:00:00, to be loaded without generating an error. You can choose <code>true</code> or <code>false</code> (the default). This parameter applies only to TIMESTAMP and DATE columns. Always use ACCEPTANYDATE with the DATEFORMAT parameter. If the date format for the data doesn't match the DATEFORMAT specification, Amazon Redshift inserts a NULL value into that field. <br/>
        :return: Endpoint:  The endpoint that was created.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RedshiftSettings</i> Settings for the Amazon Redshift endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RedshiftSettings</b>: Settings for the Amazon Redshift endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticsearchSettings</i> The settings for the Elasticsearch source endpoint. For more information, see the <code>ElasticsearchSettings</code> structure.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ElasticsearchSettings</b>: The settings for the Elasticsearch source endpoint. For more information, see the <code>ElasticsearchSettings</code> structure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KinesisSettings</i> The settings for the Amazon Kinesis source endpoint. For more information, see the <code>KinesisSettings</code> structure.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KinesisSettings</b>: The settings for the Amazon Kinesis source endpoint. For more information, see the <code>KinesisSettings</code> structure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MongoDbSettings</i> The settings for the MongoDB source endpoint. For more information, see the <code>MongoDbSettings</code> structure.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>MongoDbSettings</b>: The settings for the MongoDB source endpoint. For more information, see the <code>MongoDbSettings</code> structure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DmsTransferSettings</i> The settings in JSON format for the DMS transfer type of source endpoint.  Possible attributes include the following: <ul> <li>  <code>serviceAccessRoleArn</code> - The IAM role that has permission to access the Amazon S3 bucket. </li> <li>  <code>bucketName</code> - The name of the S3 bucket to use. </li> <li>  <code>compressionType</code> - An optional parameter to use GZIP to compress the target files. To use GZIP, set this value to <code>NONE</code> (the default). To keep the files uncompressed, don't use this value. </li> </ul> Shorthand syntax for these attributes is as follows: <code>ServiceAccessRoleArn=string,BucketName=string,CompressionType=string</code>  JSON syntax for these attributes is as follows: <code>{ "ServiceAccessRoleArn": "string", "BucketName": "string", "CompressionType": "none"|"gzip" } </code> &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DmsTransferSettings</b>: The settings in JSON format for the DMS transfer type of source endpoint.  Possible attributes include the following: <ul> <li>  <code>serviceAccessRoleArn</code> - The IAM role that has permission to access the Amazon S3 bucket. </li> <li>  <code>bucketName</code> - The name of the S3 bucket to use. </li> <li>  <code>compressionType</code> - An optional parameter to use GZIP to compress the target files. To use GZIP, set this value to <code>NONE</code> (the default). To keep the files uncompressed, don't use this value. </li> </ul> Shorthand syntax for these attributes is as follows: <code>ServiceAccessRoleArn=string,BucketName=string,CompressionType=string</code>  JSON syntax for these attributes is as follows: <code>{ "ServiceAccessRoleArn": "string", "BucketName": "string", "CompressionType": "none"|"gzip" } </code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Settings</i> The settings for the S3 target endpoint. For more information, see the <code>S3Settings</code> structure.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Settings</b>: The settings for the S3 target endpoint. For more information, see the <code>S3Settings</code> structure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DynamoDbSettings</i> The settings for the target DynamoDB database. For more information, see the <code>DynamoDBSettings</code> structure.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DynamoDbSettings</b>: The settings for the target DynamoDB database. For more information, see the <code>DynamoDBSettings</code> structure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalId</i>  Value returned by a call to CreateEndpoint that can be used for cross-account validation. Use it on a subsequent call to CreateEndpoint to create the endpoint with a cross-account. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalTableDefinition</i> The external table definition.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i> The Amazon Resource Name (ARN) used by the service access IAM role.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SslMode</i> The SSL mode used to connect to the endpoint. The default value is <code>none</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateArn</i> The Amazon Resource Name (ARN) used for SSL connection to the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointArn</i> The Amazon Resource Name (ARN) string that uniquely identifies the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> An AWS KMS key identifier that is used to encrypt the connection parameters for the endpoint. If you don't specify a value for the <code>KmsKeyId</code> parameter, then AWS DMS uses your default encryption key. AWS KMS creates the default encryption key for your AWS account. Your AWS account has a different default encryption key for each AWS Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExtraConnectionAttributes</i> Additional connection attributes used to connect to the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> The name of the database at the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port value used to access the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerName</i> The name of the server at the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> The user name used to connect to the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineDisplayName</i> The expanded name for the engine name. For example, if the <code>EngineName</code> parameter is "aurora," this value would be "Amazon Aurora MySQL."<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineName</i> The database engine name. Valid values, depending on the EndpointType, include mysql, oracle, postgres, mariadb, aurora, aurora-postgresql, redshift, s3, db2, azuredb, sybase, dynamodb, mongodb, and sqlserver.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointType</i> The type of endpoint. Valid values are <code>source</code> and <code>target</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointIdentifier</i> The database endpoint identifier. Identifiers must begin with a letter; must contain only ASCII letters, digits, and hyphens; and must not end with a hyphen or contain two consecutive hyphens.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i>  The Amazon Resource Name (ARN) used by the service access IAM role. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TimestampColumnName</i> A value that includes a timestamp column in the Amazon S3 target endpoint data. AWS DMS includes an additional column in the migrated data when you set <code>timestampColumnName</code> to a non-blank value.  <note> AWS DMS supports <code>TimestampColumnName</code> in versions 3.1.4 and later. </note> For a full load, each row of the timestamp column contains a timestamp for when the data was transferred from the source to the target by DMS. For a CDC load, each row of the timestamp column contains the timestamp for the commit of that row in the source database. The format for the timestamp column value is <code>yyyy-MM-dd HH:mm:ss.SSSSSS</code>. For CDC, the microsecond precision depends on the commit timestamp supported by DMS for the source database. When the <code>AddColumnName</code> setting is set to <code>true</code>, DMS also includes the name for the timestamp column that you set as the nonblank value of <code>timestampColumnName</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CdcInsertsOnly</i> A value that enables a change data capture (CDC) load to write only INSERT operations to .csv or columnar storage (.parquet) output files. By default (the <code>false</code> setting), the first field in a .csv or .parquet record contains the letter I (INSERT), U (UPDATE), or D (DELETE). These values indicate whether the row was inserted, updated, or deleted at the source database for a CDC load to the target. If <code>cdcInsertsOnly</code> is set to <code>true</code> or <code>y</code>, only INSERTs from the source database are migrated to the .csv or .parquet file. For .csv format only, how these INSERTs are recorded depends on the value of <code>IncludeOpForFullLoad</code>. If <code>IncludeOpForFullLoad</code> is set to <code>true</code>, the first field of every CDC record is set to I to indicate the INSERT operation at the source. If <code>IncludeOpForFullLoad</code> is set to <code>false</code>, every CDC record is written without a first field to indicate the INSERT operation at the source. For more information about how these settings work together, see <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Target.S3.html#CHAP_Target.S3.Configuring.InsertOps">Indicating Source DB Operations in Migrated S3 Data</a> in the <i>AWS Database Migration Service User Guide.</i>. <note> AWS DMS supports this interaction between <code>CdcInsertsOnly</code> and <code>IncludeOpForFullLoad</code> in versions 3.1.4 and later.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeOpForFullLoad</i> A value that enables a full load to write INSERT operations to the comma-separated value (.csv) output files only to indicate how the rows were added to the source database. <note> AWS DMS supports <code>IncludeOpForFullLoad</code> in versions 3.1.4 and later. </note> For full load, records can only be inserted. By default (the <code>false</code> setting), no information is recorded in these output files for a full load to indicate that the rows were inserted at the source database. If <code>IncludeOpForFullLoad</code> is set to <code>true</code> or <code>y</code>, the INSERT is recorded as an I annotation in the first field of the .csv file. This allows the format of your target records from a full load to be consistent with the target records from a CDC load. <note> This setting works together with <code>CdcInsertsOnly</code> for output to .csv files only. For more information about how these settings work together, see <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Target.S3.html#CHAP_Target.S3.Configuring.InsertOps">Indicating Source DB Operations in Migrated S3 Data</a> in the <i>AWS Database Migration Service User Guide.</i>. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EnableStatistics</i> A value that enables statistics for Parquet pages and row groups. Choose <code>true</code> to enable statistics, <code>false</code> to disable. Statistics include <code>NULL</code>, <code>DISTINCT</code>, <code>MAX</code>, and <code>MIN</code> values. This parameter defaults to <code>true</code>. This value is used for .parquet file format only.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ParquetVersion</i> The version of the Apache Parquet format that you want to use: <code>parquet_1_0</code> (the default) or <code>parquet_2_0</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DataPageSize</i> The size of one data page in bytes. This parameter defaults to 1024 * 1024 bytes (1 MiB). This number is used for .parquet file format only. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RowGroupLength</i> The number of rows in a row group. A smaller row group size provides faster reads. But as the number of row groups grows, the slower writes become. This parameter defaults to 10,000 rows. This number is used for .parquet file format only.  If you choose a value larger than the maximum, <code>RowGroupLength</code> is set to the max row group length in bytes (64 * 1024 * 1024). <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DictPageSizeLimit</i> The maximum size of an encoded dictionary page of a column. If the dictionary page exceeds this, this column is stored using an encoding type of <code>PLAIN</code>. This parameter defaults to 1024 * 1024 bytes (1 MiB), the maximum size of a dictionary page before it reverts to <code>PLAIN</code> encoding. This size is used for .parquet file format only. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncodingType</i> The type of encoding you are using:  <ul> <li>  <code>RLE_DICTIONARY</code> uses a combination of bit-packing and run-length encoding to store repeated values more efficiently. This is the default. </li> <li>  <code>PLAIN</code> doesn't use encoding at all. Values are stored as they are. </li> <li>  <code>PLAIN_DICTIONARY</code> builds a dictionary of the values encountered in a given column. The dictionary is stored in a dictionary page for each column chunk. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DataFormat</i> The format of the data that you want to use for output. You can choose one of the following:  <ul> <li>  <code>csv</code> : This is a row-based file format with comma-separated values (.csv).  </li> <li>  <code>parquet</code> : Apache Parquet (.parquet) is a columnar storage file format that features efficient compression and provides faster query response.  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerSideEncryptionKmsKeyId</i> If you are using <code>SSE_KMS</code> for the <code>EncryptionMode</code>, provide the AWS KMS key ID. The key that you use needs an attached policy that enables AWS Identity and Access Management (IAM) user permissions and allows use of the key. Here is a CLI example: <code>aws dms create-endpoint --endpoint-identifier <i>value</i> --endpoint-type target --engine-name s3 --s3-settings ServiceAccessRoleArn=<i>value</i>,BucketFolder=<i>value</i>,BucketName=<i>value</i>,EncryptionMode=SSE_KMS,ServerSideEncryptionKmsKeyId=<i>value</i> </code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionMode</i> The type of server-side encryption that you want to use for your data. This encryption type is part of the endpoint settings or the extra connections attributes for Amazon S3. You can choose either <code>SSE_S3</code> (the default) or <code>SSE_KMS</code>. To use <code>SSE_S3</code>, you need an AWS Identity and Access Management (IAM) role with permission to allow <code>"arn:aws:s3:::dms-*"</code> to use the following actions: <ul> <li>  <code>s3:CreateBucket</code>  </li> <li>  <code>s3:ListBucket</code>  </li> <li>  <code>s3:DeleteBucket</code>  </li> <li>  <code>s3:GetBucketLocation</code>  </li> <li>  <code>s3:GetObject</code>  </li> <li>  <code>s3:PutObject</code>  </li> <li>  <code>s3:DeleteObject</code>  </li> <li>  <code>s3:GetObjectVersion</code>  </li> <li>  <code>s3:GetBucketPolicy</code>  </li> <li>  <code>s3:PutBucketPolicy</code>  </li> <li>  <code>s3:DeleteBucketPolicy</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CompressionType</i>  An optional parameter to use GZIP to compress the target files. Set to GZIP to compress the target files. Set to NONE (the default) or do not use to leave the files uncompressed. Applies to both .csv and .parquet file formats. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketName</i>  The name of the S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketFolder</i>  An optional parameter to set a folder name in the S3 bucket. If provided, tables are created in the path <code> <i>bucketFolder</i>/<i>schema_name</i>/<i>table_name</i>/</code>. If this parameter is not specified, then the path used is <code> <i>schema_name</i>/<i>table_name</i>/</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CsvDelimiter</i>  The delimiter used to separate columns in the source files. The default is a comma. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CsvRowDelimiter</i>  The delimiter used to separate rows in the source files. The default is a carriage return (<code>\n</code>). <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalTableDefinition</i>  The external table definition. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i>  The Amazon Resource Name (ARN) used by the service access IAM role. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketName</i>  The name of the S3 bucket to use. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i>  The IAM role that has permission to access the Amazon S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS KMS key identifier that is used to encrypt the content on the replication instance. If you don't specify a value for the <code>KmsKeyId</code> parameter, then AWS DMS uses your default encryption key. AWS KMS creates the default encryption key for your AWS account. Your AWS account has a different default encryption key for each AWS Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthSource</i>  The MongoDB database name. This attribute is not used when <code>authType=NO</code>.  The default is admin.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DocsToInvestigate</i>  Indicates the number of documents to preview to determine the document organization. Use this attribute when <code>NestingLevel</code> is set to ONE.  Must be a positive value greater than 0. Default value is 1000.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExtractDocId</i>  Specifies the document ID. Use this attribute when <code>NestingLevel</code> is set to NONE.  Default value is false. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NestingLevel</i>  Specifies either document or table mode.  Valid values: NONE, ONE Default value is NONE. Specify NONE to use document mode. Specify ONE to use table mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthMechanism</i>  The authentication mechanism you use to access the MongoDB source endpoint. Valid values: DEFAULT, MONGODB_CR, SCRAM_SHA_1  DEFAULT – For MongoDB version 2.x, use MONGODB_CR. For MongoDB version 3.x, use SCRAM_SHA_1. This attribute is not used when authType=No.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthType</i>  The authentication type you use to access the MongoDB source endpoint. Valid values: NO, PASSWORD  When NO is selected, user name and password parameters are not used and can be empty. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i>  The database name on the MongoDB source endpoint. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i>  The port value for the MongoDB source endpoint. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerName</i>  The name of the server on the MongoDB source endpoint. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Password</i>  The password for the user account you use to access the MongoDB source endpoint. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> The user name you use to access the MongoDB source endpoint. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i> The Amazon Resource Name (ARN) for the IAM role that DMS uses to write to the Amazon Kinesis data stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MessageFormat</i> The output format for the records created on the endpoint. The message format is <code>JSON</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamArn</i> The Amazon Resource Name (ARN) for the Amazon Kinesis Data Streams endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorRetryDuration</i> The maximum number of seconds that DMS retries failed API requests to the Elasticsearch cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FullLoadErrorPercentage</i> The maximum percentage of records that can fail to be written before a full load operation stops. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointUri</i> The endpoint for the Elasticsearch cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i> The Amazon Resource Name (ARN) used by service to access the IAM role.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WriteBufferSize</i> The size of the write buffer to use in rows. Valid values range from 1 through 2,048. The default is 1,024. Use this setting to tune performance. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> An Amazon Redshift user name for a registered user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TruncateColumns</i> A value that specifies to truncate data in columns to the appropriate number of characters, so that the data fits in the column. This parameter applies only to columns with a VARCHAR or CHAR data type, and rows with a size of 4 MB or less. Choose <code>true</code> to truncate data. The default is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TrimBlanks</i> A value that specifies to remove the trailing white space characters from a VARCHAR string. This parameter applies only to columns with a VARCHAR data type. Choose <code>true</code> to remove unneeded white space. The default is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeFormat</i> The time format that you want to use. Valid values are <code>auto</code> (case-sensitive), <code>'timeformat_string'</code>, <code>'epochsecs'</code>, or <code>'epochmillisecs'</code>. It defaults to 10. Using <code>auto</code> recognizes most strings, even some that aren't supported when you use a time format string.  If your date and time values use formats different from each other, set this parameter to <code>auto</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerSideEncryptionKmsKeyId</i> The AWS KMS key ID. If you are using <code>SSE_KMS</code> for the <code>EncryptionMode</code>, provide this key ID. The key that you use needs an attached policy that enables IAM user permissions and allows use of the key.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i> The Amazon Resource Name (ARN) of the IAM role that has access to the Amazon Redshift service.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerName</i> The name of the Amazon Redshift cluster you are using.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplaceChars</i> A value that specifies to replaces the invalid characters specified in <code>ReplaceInvalidChars</code>, substituting the specified characters instead. The default is <code>"?"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplaceInvalidChars</i> A list of characters that you want to replace. Use with <code>ReplaceChars</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RemoveQuotes</i> A value that specifies to remove surrounding quotation marks from strings in the incoming data. All characters within the quotation marks, including delimiters, are retained. Choose <code>true</code> to remove quotation marks. The default is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port number for Amazon Redshift. The default value is 5439.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Password</i> The password for the user named in the <code>username</code> property.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxFileSize</i> The maximum size (in KB) of any .csv file used to transfer data to Amazon Redshift. This accepts a value from 1 through 1,048,576. It defaults to 32,768 KB (32 MB).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LoadTimeout</i> The amount of time to wait (in milliseconds) before timing out, beginning from when you begin loading.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FileTransferUploadStreams</i> The number of threads used to upload a single file. This parameter accepts a value from 1 through 64. It defaults to 10.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionMode</i> The type of server-side encryption that you want to use for your data. This encryption type is part of the endpoint settings or the extra connections attributes for Amazon S3. You can choose either <code>SSE_S3</code> (the default) or <code>SSE_KMS</code>. To use <code>SSE_S3</code>, create an AWS Identity and Access Management (IAM) role with a policy that allows <code>"arn:aws:s3:::*"</code> to use the following actions: <code>"s3:PutObject", "s3:ListBucket"</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EmptyAsNull</i> A value that specifies whether AWS DMS should migrate empty CHAR and VARCHAR fields as NULL. A value of <code>true</code> sets empty CHAR and VARCHAR fields to null. The default is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DateFormat</i> The date format that you are using. Valid values are <code>auto</code> (case-sensitive), your date format string enclosed in quotes, or NULL. If this parameter is left unset (NULL), it defaults to a format of 'YYYY-MM-DD'. Using <code>auto</code> recognizes most strings, even some that aren't supported when you use a date format string.  If your date and time values use formats different from each other, set this to <code>auto</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> The name of the Amazon Redshift data warehouse (service) that you are working with.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectionTimeout</i> A value that sets the amount of time to wait (in milliseconds) before timing out, beginning from when you initially establish a connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketName</i> The name of the S3 bucket you want to use<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketFolder</i> The location where the comma-separated value (.csv) files are stored before being uploaded to the S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AfterConnectScript</i> Code to run after connecting. This parameter should contain the code itself, not the name of a file containing the code.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AcceptAnyDate</i> A value that indicates to allow any date format, including invalid formats such as 00/00/00 00:00:00, to be loaded without generating an error. You can choose <code>true</code> or <code>false</code> (the default). This parameter applies only to TIMESTAMP and DATE columns. Always use ACCEPTANYDATE with the DATEFORMAT parameter. If the date format for the data doesn't match the DATEFORMAT specification, Amazon Redshift inserts a NULL value into that field. <br/>        """
        pass

    def create_event_subscription(self, SnsTopicArn: str, SubscriptionName: str, SourceType: str=None, EventCategories: List[String]=None, SourceIds: List[String]=None, Enabled: BooleanOptional=None, Tags: List[Tag]=None) -> CreateEventSubscriptionResponse:
        """ Creates an AWS DMS event notification subscription.  You can specify the type of source (<code>SourceType</code>) you want to be notified of, provide a list of AWS DMS source IDs (<code>SourceIds</code>) that triggers the events, and provide a list of event categories (<code>EventCategories</code>) for events you want to be notified of. If you specify both the <code>SourceType</code> and <code>SourceIds</code>, such as <code>SourceType = replication-instance</code> and <code>SourceIdentifier = my-replinstance</code>, you will be notified of all the replication instance events for the specified source. If you specify a <code>SourceType</code> but don't specify a <code>SourceIdentifier</code>, you receive notice of the events for that source type for all your AWS DMS sources. If you don't specify either <code>SourceType</code> nor <code>SourceIdentifier</code>, you will be notified of events generated from all AWS DMS sources belonging to your customer account. For more information about AWS DMS events, see <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Events.html">Working with Events and Notifications</a> in the <i>AWS Database Migration Service User Guide.</i> 

        :param self:
        :param Tags:  One or more tags to be assigned to the event subscription.<br/>Enabled:   A Boolean value; set to <code>true</code> to activate the subscription, or set to <code>false</code> to create the subscription but not activate it. <br/>SourceIds:  A list of identifiers for which AWS DMS provides notification events. If you don't specify a value, notifications are provided for all sources. If you specify multiple values, they must be of the same type. For example, if you specify a database instance ID, then all of the other values must be database instance IDs.<br/>EventCategories:  A list of event categories for a source type that you want to subscribe to. For more information, see <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Events.html">Working with Events and Notifications</a> in the <i>AWS Database Migration Service User Guide.</i> <br/>SourceType:   The type of AWS DMS resource that generates the events. For example, if you want to be notified of events generated by a replication instance, you set this parameter to <code>replication-instance</code>. If this value is not specified, all events are returned.  Valid values: <code>replication-instance</code> | <code>replication-task</code> <br/>SnsTopicArn:   The Amazon Resource Name (ARN) of the Amazon SNS topic created for event notification. The ARN is created by Amazon SNS when you create a topic and subscribe to it. <br/>SubscriptionName:  The name of the AWS DMS event notification subscription. This name must be less than 255 characters.<br/>
        :return: EventSubscription:  The event subscription that was created.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Boolean value that indicates if the event subscription is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EventCategoriesList</i> A lists of event categories.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceIdsList</i> A list of source Ids for the event subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceType</i>  The type of AWS DMS resource that generates events.  Valid values: replication-instance | replication-server | security-group | replication-task<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubscriptionCreationTime</i> The time the RDS event notification subscription was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the AWS DMS event notification subscription. Constraints: Can be one of the following: creating | modifying | deleting | active | no-permission | topic-not-exist The status "no-permission" indicates that AWS DMS no longer has permission to post to the SNS topic. The status "topic-not-exist" indicates that the topic was deleted after the subscription was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsTopicArn</i> The topic ARN of the AWS DMS event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustSubscriptionId</i> The AWS DMS event notification subscription Id.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomerAwsId</i> The AWS customer account associated with the AWS DMS event notification subscription.<br/>        """
        pass

    def create_replication_instance(self, ReplicationInstanceClass: str, ReplicationInstanceIdentifier: str, AllocatedStorage: int=None, VpcSecurityGroupIds: List[String]=None, AvailabilityZone: str=None, ReplicationSubnetGroupIdentifier: str=None, PreferredMaintenanceWindow: str=None, MultiAZ: BooleanOptional=None, EngineVersion: str=None, AutoMinorVersionUpgrade: BooleanOptional=None, Tags: List[Tag]=None, KmsKeyId: str=None, PubliclyAccessible: BooleanOptional=None, DnsNameServers: str=None) -> CreateReplicationInstanceResponse:
        """Creates the replication instance using the specified parameters.

        :param self:
        :param DnsNameServers:  A list of DNS name servers supported for the replication instance.<br/>PubliclyAccessible:   Specifies the accessibility options for the replication instance. A value of <code>true</code> represents an instance with a public IP address. A value of <code>false</code> represents an instance with a private IP address. The default value is <code>true</code>. <br/>KmsKeyId:  An AWS KMS key identifier that is used to encrypt the data on the replication instance. If you don't specify a value for the <code>KmsKeyId</code> parameter, then AWS DMS uses your default encryption key. AWS KMS creates the default encryption key for your AWS account. Your AWS account has a different default encryption key for each AWS Region.<br/>Tags:  One or more tags to be assigned to the replication instance.<br/>AutoMinorVersionUpgrade:  Indicates whether minor engine upgrades will be applied automatically to the replication instance during the maintenance window. This parameter defaults to <code>true</code>. Default: <code>true</code> <br/>EngineVersion:  The engine version number of the replication instance.<br/>MultiAZ:   Specifies whether the replication instance is a Multi-AZ deployment. You cannot set the <code>AvailabilityZone</code> parameter if the Multi-AZ parameter is set to <code>true</code>. <br/>PreferredMaintenanceWindow:  The weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).  Format: <code>ddd:hh24:mi-ddd:hh24:mi</code>  Default: A 30-minute window selected at random from an 8-hour block of time per AWS Region, occurring on a random day of the week. Valid Days: Mon, Tue, Wed, Thu, Fri, Sat, Sun Constraints: Minimum 30-minute window.<br/>ReplicationSubnetGroupIdentifier:  A subnet group to associate with the replication instance.<br/>AvailabilityZone:  The AWS Availability Zone where the replication instance will be created. The default value is a random, system-chosen Availability Zone in the endpoint's AWS Region, for example: <code>us-east-1d</code> <br/>VpcSecurityGroupIds:   Specifies the VPC security group to be used with the replication instance. The VPC security group must work with the VPC containing the replication instance. <br/>ReplicationInstanceClass:  The compute and memory capacity of the replication instance as specified by the replication instance class.  Valid Values: <code>dms.t2.micro | dms.t2.small | dms.t2.medium | dms.t2.large | dms.c4.large | dms.c4.xlarge | dms.c4.2xlarge | dms.c4.4xlarge </code> <br/>AllocatedStorage:  The amount of storage (in gigabytes) to be initially allocated for the replication instance.<br/>ReplicationInstanceIdentifier:  The replication instance identifier. This parameter is stored as a lowercase string. Constraints: <ul> <li> Must contain from 1 to 63 alphanumeric characters or hyphens. </li> <li> First character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul> Example: <code>myrepinstance</code> <br/>
        :return: ReplicationInstance:  The replication instance that was created.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DnsNameServers</i> The DNS name servers for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FreeUntil</i>  The expiration date of the free replication instance that is part of the Free DMS program. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondaryAvailabilityZone</i> The availability zone of the standby replication instance in a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i>  Specifies the accessibility options for the replication instance. A value of <code>true</code> represents an instance with a public IP address. A value of <code>false</code> represents an instance with a private IP address. The default value is <code>true</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstancePrivateIpAddresses</i> One or more private IP addresses for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstancePublicIpAddresses</i> One or more public IP addresses for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstancePrivateIpAddress</i> The private IP address of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstancePublicIpAddress</i> The public IP address of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceArn</i> The Amazon Resource Name (ARN) of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> An AWS KMS key identifier that is used to encrypt the data on the replication instance. If you don't specify a value for the <code>KmsKeyId</code> parameter, then AWS DMS uses your default encryption key. AWS KMS creates the default encryption key for your AWS account. Your AWS account has a different default encryption key for each AWS Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> Boolean value indicating if minor version upgrades will be automatically applied to the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> The engine version number of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i>  Specifies whether the replication instance is a Multi-AZ deployment. You cannot set the <code>AvailabilityZone</code> parameter if the Multi-AZ parameter is set to <code>true</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> The pending modification values.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: The pending modification values.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> The maintenance window times for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSubnetGroup</i> The subnet group for the replication instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ReplicationSubnetGroup</b>: The subnet group for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> The VPC security group for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCreateTime</i> The time the replication instance was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> The amount of storage (in gigabytes) that is allocated for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceStatus</i> The status of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceClass</i> The compute and memory capacity of the replication instance.  Valid Values: <code>dms.t2.micro | dms.t2.small | dms.t2.medium | dms.t2.large | dms.c4.large | dms.c4.xlarge | dms.c4.2xlarge | dms.c4.4xlarge </code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceIdentifier</i> The replication instance identifier. This parameter is stored as a lowercase string. Constraints: <ul> <li> Must contain from 1 to 63 alphanumeric characters or hyphens. </li> <li> First character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul> Example: <code>myrepinstance</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> The subnets that are in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> The status of the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSubnetGroupDescription</i> A description for the replication subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSubnetGroupIdentifier</i> The identifier of the replication instance subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> The engine version number of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i>  Specifies whether the replication instance is a Multi-AZ deployment. You cannot set the <code>AvailabilityZone</code> parameter if the Multi-AZ parameter is set to <code>true</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> The amount of storage (in gigabytes) that is allocated for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceClass</i> The compute and memory capacity of the replication instance.  Valid Values: <code>dms.t2.micro | dms.t2.small | dms.t2.medium | dms.t2.large | dms.c4.large | dms.c4.xlarge | dms.c4.2xlarge | dms.c4.4xlarge </code> <br/>        """
        pass

    def create_replication_subnet_group(self, SubnetIds: List[String], ReplicationSubnetGroupDescription: str, ReplicationSubnetGroupIdentifier: str, Tags: List[Tag]=None) -> CreateReplicationSubnetGroupResponse:
        """Creates a replication subnet group given a list of the subnet IDs in a VPC.

        :param self:
        :param Tags:  One or more tags to be assigned to the subnet group.<br/>SubnetIds:  One or more subnet IDs to be assigned to the subnet group.<br/>ReplicationSubnetGroupDescription:  The description for the subnet group.<br/>ReplicationSubnetGroupIdentifier:  The name for the replication subnet group. This value is stored as a lowercase string. Constraints: Must contain no more than 255 alphanumeric characters, periods, spaces, underscores, or hyphens. Must not be "default". Example: <code>mySubnetgroup</code> <br/>
        :return: ReplicationSubnetGroup:  The replication subnet group that was created.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> The subnets that are in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> The status of the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSubnetGroupDescription</i> A description for the replication subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSubnetGroupIdentifier</i> The identifier of the replication instance subnet group.<br/>        """
        pass

    def create_replication_task(self, TableMappings: str, MigrationType: str, ReplicationInstanceArn: str, TargetEndpointArn: str, SourceEndpointArn: str, ReplicationTaskIdentifier: str, ReplicationTaskSettings: str=None, CdcStartTime: TStamp=None, CdcStartPosition: str=None, CdcStopPosition: str=None, Tags: List[Tag]=None) -> CreateReplicationTaskResponse:
        """Creates a replication task using the specified parameters.

        :param self:
        :param Tags:  One or more tags to be assigned to the replication task.<br/>CdcStopPosition:  Indicates when you want a change data capture (CDC) operation to stop. The value can be either server time or commit time. Server time example: --cdc-stop-position “server_time:3018-02-09T12:12:12” Commit time example: --cdc-stop-position “commit_time: 3018-02-09T12:12:12 “<br/>CdcStartPosition:  Indicates when you want a change data capture (CDC) operation to start. Use either CdcStartPosition or CdcStartTime to specify when you want a CDC operation to start. Specifying both values results in an error.  The value can be in date, checkpoint, or LSN/SCN format. Date Example: --cdc-start-position “2018-03-08T12:12:12” Checkpoint Example: --cdc-start-position "checkpoint:V1#27#mysql-bin-changelog.157832:1975:-1:2002:677883278264080:mysql-bin-changelog.157832:1876#0#0#*#0#93" LSN Example: --cdc-start-position “mysql-bin-changelog.000024:373”<br/>CdcStartTime:  Indicates the start time for a change data capture (CDC) operation. Use either CdcStartTime or CdcStartPosition to specify when you want a CDC operation to start. Specifying both values results in an error. Timestamp Example: --cdc-start-time “2018-03-08T12:12:12”<br/>ReplicationTaskSettings:  Overall settings for the task, in JSON format. For more information, see <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Tasks.CustomizingTasks.TaskSettings.html">Task Settings</a> in the <i>AWS Database Migration User Guide.</i> <br/>TableMappings:  The table mappings for the task, in JSON format. For more information, see <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Tasks.CustomizingTasks.TableMapping.html">Table Mapping</a> in the <i>AWS Database Migration User Guide.</i> <br/>MigrationType:  The migration type. Valid values: <code>full-load</code> | <code>cdc</code> | <code>full-load-and-cdc</code> <br/>ReplicationInstanceArn:  The Amazon Resource Name (ARN) of a replication instance.<br/>TargetEndpointArn:  An Amazon Resource Name (ARN) that uniquely identifies the target endpoint.<br/>SourceEndpointArn:  An Amazon Resource Name (ARN) that uniquely identifies the source endpoint.<br/>ReplicationTaskIdentifier:  An identifier for the replication task. Constraints: <ul> <li> Must contain from 1 to 255 alphanumeric characters or hyphens. </li> <li> First character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul><br/>
        :return: ReplicationTask:  The replication task that was created.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskStats</i> The statistics for the task, including elapsed time, tables loaded, and table errors.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ReplicationTaskStats</b>: The statistics for the task, including elapsed time, tables loaded, and table errors.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskArn</i> The Amazon Resource Name (ARN) of the replication task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecoveryCheckpoint</i> Indicates the last checkpoint that occurred during a change data capture (CDC) operation. You can provide this value to the <code>CdcStartPosition</code> parameter to start a CDC operation that begins at that checkpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CdcStopPosition</i> Indicates when you want a change data capture (CDC) operation to stop. The value can be either server time or commit time. Server time example: --cdc-stop-position “server_time:3018-02-09T12:12:12” Commit time example: --cdc-stop-position “commit_time: 3018-02-09T12:12:12 “<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CdcStartPosition</i> Indicates when you want a change data capture (CDC) operation to start. Use either <code>CdcStartPosition</code> or <code>CdcStartTime</code> to specify when you want the CDC operation to start. Specifying both values results in an error. The value can be in date, checkpoint, or LSN/SCN format. Date Example: --cdc-start-position “2018-03-08T12:12:12” Checkpoint Example: --cdc-start-position "checkpoint:V1#27#mysql-bin-changelog.157832:1975:-1:2002:677883278264080:mysql-bin-changelog.157832:1876#0#0#*#0#93" LSN Example: --cdc-start-position “mysql-bin-changelog.000024:373”<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskStartDate</i> The date the replication task is scheduled to start.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskCreationDate</i> The date the replication task was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StopReason</i> The reason the replication task was stopped.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastFailureMessage</i> The last error (failure) message generated for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the replication task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskSettings</i> The settings for the replication task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableMappings</i> Table mappings specified in the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MigrationType</i> The type of migration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceArn</i> The Amazon Resource Name (ARN) of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetEndpointArn</i> The Amazon Resource Name (ARN) string that uniquely identifies the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceEndpointArn</i> The Amazon Resource Name (ARN) string that uniquely identifies the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskIdentifier</i> The user-assigned replication task identifier or name. Constraints: <ul> <li> Must contain from 1 to 255 alphanumeric characters or hyphens. </li> <li> First character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TablesErrored</i> The number of errors that have occurred during this task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TablesQueued</i> The number of tables queued for this task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TablesLoading</i> The number of tables currently loading for this task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TablesLoaded</i> The number of tables loaded for this task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeMillis</i> The elapsed time of the task, in milliseconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FullLoadProgressPercent</i> The percent complete for the full load migration task.<br/>        """
        pass

    def delete_certificate(self, CertificateArn: str) -> DeleteCertificateResponse:
        """Deletes the specified certificate. 

        :param self:
        :param CertificateArn:  The Amazon Resource Name (ARN) of the deleted certificate.<br/>
        :return: Certificate:  The Secure Sockets Layer (SSL) certificate.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyLength</i> The key length of the cryptographic algorithm being used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SigningAlgorithm</i> The signing algorithm for the certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ValidToDate</i> The final date that the certificate is valid.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ValidFromDate</i> The beginning date that the certificate is valid.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateOwner</i> The owner of the certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateArn</i> The Amazon Resource Name (ARN) for the certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateWallet</i> The location of an imported Oracle Wallet certificate for use with SSL.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificatePem</i> The contents of a <code>.pem</code> file, which contains an X.509 certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateCreationDate</i> The date that the certificate was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateIdentifier</i> A customer-assigned name for the certificate. Identifiers must begin with a letter; must contain only ASCII letters, digits, and hyphens; and must not end with a hyphen or contain two consecutive hyphens.<br/>        """
        pass

    def delete_endpoint(self, EndpointArn: str) -> DeleteEndpointResponse:
        """Deletes the specified endpoint. <note> All tasks associated with the endpoint must be deleted before you can delete the endpoint. </note> <p/>

        :param self:
        :param EndpointArn:  The Amazon Resource Name (ARN) string that uniquely identifies the endpoint.<br/>
        :return: Endpoint:  The endpoint that was deleted.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RedshiftSettings</i> Settings for the Amazon Redshift endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RedshiftSettings</b>: Settings for the Amazon Redshift endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticsearchSettings</i> The settings for the Elasticsearch source endpoint. For more information, see the <code>ElasticsearchSettings</code> structure.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ElasticsearchSettings</b>: The settings for the Elasticsearch source endpoint. For more information, see the <code>ElasticsearchSettings</code> structure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KinesisSettings</i> The settings for the Amazon Kinesis source endpoint. For more information, see the <code>KinesisSettings</code> structure.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KinesisSettings</b>: The settings for the Amazon Kinesis source endpoint. For more information, see the <code>KinesisSettings</code> structure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MongoDbSettings</i> The settings for the MongoDB source endpoint. For more information, see the <code>MongoDbSettings</code> structure.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>MongoDbSettings</b>: The settings for the MongoDB source endpoint. For more information, see the <code>MongoDbSettings</code> structure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DmsTransferSettings</i> The settings in JSON format for the DMS transfer type of source endpoint.  Possible attributes include the following: <ul> <li>  <code>serviceAccessRoleArn</code> - The IAM role that has permission to access the Amazon S3 bucket. </li> <li>  <code>bucketName</code> - The name of the S3 bucket to use. </li> <li>  <code>compressionType</code> - An optional parameter to use GZIP to compress the target files. To use GZIP, set this value to <code>NONE</code> (the default). To keep the files uncompressed, don't use this value. </li> </ul> Shorthand syntax for these attributes is as follows: <code>ServiceAccessRoleArn=string,BucketName=string,CompressionType=string</code>  JSON syntax for these attributes is as follows: <code>{ "ServiceAccessRoleArn": "string", "BucketName": "string", "CompressionType": "none"|"gzip" } </code> &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DmsTransferSettings</b>: The settings in JSON format for the DMS transfer type of source endpoint.  Possible attributes include the following: <ul> <li>  <code>serviceAccessRoleArn</code> - The IAM role that has permission to access the Amazon S3 bucket. </li> <li>  <code>bucketName</code> - The name of the S3 bucket to use. </li> <li>  <code>compressionType</code> - An optional parameter to use GZIP to compress the target files. To use GZIP, set this value to <code>NONE</code> (the default). To keep the files uncompressed, don't use this value. </li> </ul> Shorthand syntax for these attributes is as follows: <code>ServiceAccessRoleArn=string,BucketName=string,CompressionType=string</code>  JSON syntax for these attributes is as follows: <code>{ "ServiceAccessRoleArn": "string", "BucketName": "string", "CompressionType": "none"|"gzip" } </code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Settings</i> The settings for the S3 target endpoint. For more information, see the <code>S3Settings</code> structure.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Settings</b>: The settings for the S3 target endpoint. For more information, see the <code>S3Settings</code> structure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DynamoDbSettings</i> The settings for the target DynamoDB database. For more information, see the <code>DynamoDBSettings</code> structure.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DynamoDbSettings</b>: The settings for the target DynamoDB database. For more information, see the <code>DynamoDBSettings</code> structure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalId</i>  Value returned by a call to CreateEndpoint that can be used for cross-account validation. Use it on a subsequent call to CreateEndpoint to create the endpoint with a cross-account. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalTableDefinition</i> The external table definition.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i> The Amazon Resource Name (ARN) used by the service access IAM role.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SslMode</i> The SSL mode used to connect to the endpoint. The default value is <code>none</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateArn</i> The Amazon Resource Name (ARN) used for SSL connection to the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointArn</i> The Amazon Resource Name (ARN) string that uniquely identifies the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> An AWS KMS key identifier that is used to encrypt the connection parameters for the endpoint. If you don't specify a value for the <code>KmsKeyId</code> parameter, then AWS DMS uses your default encryption key. AWS KMS creates the default encryption key for your AWS account. Your AWS account has a different default encryption key for each AWS Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExtraConnectionAttributes</i> Additional connection attributes used to connect to the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> The name of the database at the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port value used to access the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerName</i> The name of the server at the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> The user name used to connect to the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineDisplayName</i> The expanded name for the engine name. For example, if the <code>EngineName</code> parameter is "aurora," this value would be "Amazon Aurora MySQL."<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineName</i> The database engine name. Valid values, depending on the EndpointType, include mysql, oracle, postgres, mariadb, aurora, aurora-postgresql, redshift, s3, db2, azuredb, sybase, dynamodb, mongodb, and sqlserver.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointType</i> The type of endpoint. Valid values are <code>source</code> and <code>target</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointIdentifier</i> The database endpoint identifier. Identifiers must begin with a letter; must contain only ASCII letters, digits, and hyphens; and must not end with a hyphen or contain two consecutive hyphens.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i>  The Amazon Resource Name (ARN) used by the service access IAM role. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TimestampColumnName</i> A value that includes a timestamp column in the Amazon S3 target endpoint data. AWS DMS includes an additional column in the migrated data when you set <code>timestampColumnName</code> to a non-blank value.  <note> AWS DMS supports <code>TimestampColumnName</code> in versions 3.1.4 and later. </note> For a full load, each row of the timestamp column contains a timestamp for when the data was transferred from the source to the target by DMS. For a CDC load, each row of the timestamp column contains the timestamp for the commit of that row in the source database. The format for the timestamp column value is <code>yyyy-MM-dd HH:mm:ss.SSSSSS</code>. For CDC, the microsecond precision depends on the commit timestamp supported by DMS for the source database. When the <code>AddColumnName</code> setting is set to <code>true</code>, DMS also includes the name for the timestamp column that you set as the nonblank value of <code>timestampColumnName</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CdcInsertsOnly</i> A value that enables a change data capture (CDC) load to write only INSERT operations to .csv or columnar storage (.parquet) output files. By default (the <code>false</code> setting), the first field in a .csv or .parquet record contains the letter I (INSERT), U (UPDATE), or D (DELETE). These values indicate whether the row was inserted, updated, or deleted at the source database for a CDC load to the target. If <code>cdcInsertsOnly</code> is set to <code>true</code> or <code>y</code>, only INSERTs from the source database are migrated to the .csv or .parquet file. For .csv format only, how these INSERTs are recorded depends on the value of <code>IncludeOpForFullLoad</code>. If <code>IncludeOpForFullLoad</code> is set to <code>true</code>, the first field of every CDC record is set to I to indicate the INSERT operation at the source. If <code>IncludeOpForFullLoad</code> is set to <code>false</code>, every CDC record is written without a first field to indicate the INSERT operation at the source. For more information about how these settings work together, see <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Target.S3.html#CHAP_Target.S3.Configuring.InsertOps">Indicating Source DB Operations in Migrated S3 Data</a> in the <i>AWS Database Migration Service User Guide.</i>. <note> AWS DMS supports this interaction between <code>CdcInsertsOnly</code> and <code>IncludeOpForFullLoad</code> in versions 3.1.4 and later.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeOpForFullLoad</i> A value that enables a full load to write INSERT operations to the comma-separated value (.csv) output files only to indicate how the rows were added to the source database. <note> AWS DMS supports <code>IncludeOpForFullLoad</code> in versions 3.1.4 and later. </note> For full load, records can only be inserted. By default (the <code>false</code> setting), no information is recorded in these output files for a full load to indicate that the rows were inserted at the source database. If <code>IncludeOpForFullLoad</code> is set to <code>true</code> or <code>y</code>, the INSERT is recorded as an I annotation in the first field of the .csv file. This allows the format of your target records from a full load to be consistent with the target records from a CDC load. <note> This setting works together with <code>CdcInsertsOnly</code> for output to .csv files only. For more information about how these settings work together, see <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Target.S3.html#CHAP_Target.S3.Configuring.InsertOps">Indicating Source DB Operations in Migrated S3 Data</a> in the <i>AWS Database Migration Service User Guide.</i>. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EnableStatistics</i> A value that enables statistics for Parquet pages and row groups. Choose <code>true</code> to enable statistics, <code>false</code> to disable. Statistics include <code>NULL</code>, <code>DISTINCT</code>, <code>MAX</code>, and <code>MIN</code> values. This parameter defaults to <code>true</code>. This value is used for .parquet file format only.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ParquetVersion</i> The version of the Apache Parquet format that you want to use: <code>parquet_1_0</code> (the default) or <code>parquet_2_0</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DataPageSize</i> The size of one data page in bytes. This parameter defaults to 1024 * 1024 bytes (1 MiB). This number is used for .parquet file format only. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RowGroupLength</i> The number of rows in a row group. A smaller row group size provides faster reads. But as the number of row groups grows, the slower writes become. This parameter defaults to 10,000 rows. This number is used for .parquet file format only.  If you choose a value larger than the maximum, <code>RowGroupLength</code> is set to the max row group length in bytes (64 * 1024 * 1024). <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DictPageSizeLimit</i> The maximum size of an encoded dictionary page of a column. If the dictionary page exceeds this, this column is stored using an encoding type of <code>PLAIN</code>. This parameter defaults to 1024 * 1024 bytes (1 MiB), the maximum size of a dictionary page before it reverts to <code>PLAIN</code> encoding. This size is used for .parquet file format only. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncodingType</i> The type of encoding you are using:  <ul> <li>  <code>RLE_DICTIONARY</code> uses a combination of bit-packing and run-length encoding to store repeated values more efficiently. This is the default. </li> <li>  <code>PLAIN</code> doesn't use encoding at all. Values are stored as they are. </li> <li>  <code>PLAIN_DICTIONARY</code> builds a dictionary of the values encountered in a given column. The dictionary is stored in a dictionary page for each column chunk. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DataFormat</i> The format of the data that you want to use for output. You can choose one of the following:  <ul> <li>  <code>csv</code> : This is a row-based file format with comma-separated values (.csv).  </li> <li>  <code>parquet</code> : Apache Parquet (.parquet) is a columnar storage file format that features efficient compression and provides faster query response.  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerSideEncryptionKmsKeyId</i> If you are using <code>SSE_KMS</code> for the <code>EncryptionMode</code>, provide the AWS KMS key ID. The key that you use needs an attached policy that enables AWS Identity and Access Management (IAM) user permissions and allows use of the key. Here is a CLI example: <code>aws dms create-endpoint --endpoint-identifier <i>value</i> --endpoint-type target --engine-name s3 --s3-settings ServiceAccessRoleArn=<i>value</i>,BucketFolder=<i>value</i>,BucketName=<i>value</i>,EncryptionMode=SSE_KMS,ServerSideEncryptionKmsKeyId=<i>value</i> </code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionMode</i> The type of server-side encryption that you want to use for your data. This encryption type is part of the endpoint settings or the extra connections attributes for Amazon S3. You can choose either <code>SSE_S3</code> (the default) or <code>SSE_KMS</code>. To use <code>SSE_S3</code>, you need an AWS Identity and Access Management (IAM) role with permission to allow <code>"arn:aws:s3:::dms-*"</code> to use the following actions: <ul> <li>  <code>s3:CreateBucket</code>  </li> <li>  <code>s3:ListBucket</code>  </li> <li>  <code>s3:DeleteBucket</code>  </li> <li>  <code>s3:GetBucketLocation</code>  </li> <li>  <code>s3:GetObject</code>  </li> <li>  <code>s3:PutObject</code>  </li> <li>  <code>s3:DeleteObject</code>  </li> <li>  <code>s3:GetObjectVersion</code>  </li> <li>  <code>s3:GetBucketPolicy</code>  </li> <li>  <code>s3:PutBucketPolicy</code>  </li> <li>  <code>s3:DeleteBucketPolicy</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CompressionType</i>  An optional parameter to use GZIP to compress the target files. Set to GZIP to compress the target files. Set to NONE (the default) or do not use to leave the files uncompressed. Applies to both .csv and .parquet file formats. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketName</i>  The name of the S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketFolder</i>  An optional parameter to set a folder name in the S3 bucket. If provided, tables are created in the path <code> <i>bucketFolder</i>/<i>schema_name</i>/<i>table_name</i>/</code>. If this parameter is not specified, then the path used is <code> <i>schema_name</i>/<i>table_name</i>/</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CsvDelimiter</i>  The delimiter used to separate columns in the source files. The default is a comma. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CsvRowDelimiter</i>  The delimiter used to separate rows in the source files. The default is a carriage return (<code>\n</code>). <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalTableDefinition</i>  The external table definition. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i>  The Amazon Resource Name (ARN) used by the service access IAM role. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketName</i>  The name of the S3 bucket to use. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i>  The IAM role that has permission to access the Amazon S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS KMS key identifier that is used to encrypt the content on the replication instance. If you don't specify a value for the <code>KmsKeyId</code> parameter, then AWS DMS uses your default encryption key. AWS KMS creates the default encryption key for your AWS account. Your AWS account has a different default encryption key for each AWS Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthSource</i>  The MongoDB database name. This attribute is not used when <code>authType=NO</code>.  The default is admin.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DocsToInvestigate</i>  Indicates the number of documents to preview to determine the document organization. Use this attribute when <code>NestingLevel</code> is set to ONE.  Must be a positive value greater than 0. Default value is 1000.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExtractDocId</i>  Specifies the document ID. Use this attribute when <code>NestingLevel</code> is set to NONE.  Default value is false. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NestingLevel</i>  Specifies either document or table mode.  Valid values: NONE, ONE Default value is NONE. Specify NONE to use document mode. Specify ONE to use table mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthMechanism</i>  The authentication mechanism you use to access the MongoDB source endpoint. Valid values: DEFAULT, MONGODB_CR, SCRAM_SHA_1  DEFAULT – For MongoDB version 2.x, use MONGODB_CR. For MongoDB version 3.x, use SCRAM_SHA_1. This attribute is not used when authType=No.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthType</i>  The authentication type you use to access the MongoDB source endpoint. Valid values: NO, PASSWORD  When NO is selected, user name and password parameters are not used and can be empty. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i>  The database name on the MongoDB source endpoint. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i>  The port value for the MongoDB source endpoint. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerName</i>  The name of the server on the MongoDB source endpoint. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Password</i>  The password for the user account you use to access the MongoDB source endpoint. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> The user name you use to access the MongoDB source endpoint. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i> The Amazon Resource Name (ARN) for the IAM role that DMS uses to write to the Amazon Kinesis data stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MessageFormat</i> The output format for the records created on the endpoint. The message format is <code>JSON</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamArn</i> The Amazon Resource Name (ARN) for the Amazon Kinesis Data Streams endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorRetryDuration</i> The maximum number of seconds that DMS retries failed API requests to the Elasticsearch cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FullLoadErrorPercentage</i> The maximum percentage of records that can fail to be written before a full load operation stops. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointUri</i> The endpoint for the Elasticsearch cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i> The Amazon Resource Name (ARN) used by service to access the IAM role.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WriteBufferSize</i> The size of the write buffer to use in rows. Valid values range from 1 through 2,048. The default is 1,024. Use this setting to tune performance. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> An Amazon Redshift user name for a registered user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TruncateColumns</i> A value that specifies to truncate data in columns to the appropriate number of characters, so that the data fits in the column. This parameter applies only to columns with a VARCHAR or CHAR data type, and rows with a size of 4 MB or less. Choose <code>true</code> to truncate data. The default is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TrimBlanks</i> A value that specifies to remove the trailing white space characters from a VARCHAR string. This parameter applies only to columns with a VARCHAR data type. Choose <code>true</code> to remove unneeded white space. The default is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeFormat</i> The time format that you want to use. Valid values are <code>auto</code> (case-sensitive), <code>'timeformat_string'</code>, <code>'epochsecs'</code>, or <code>'epochmillisecs'</code>. It defaults to 10. Using <code>auto</code> recognizes most strings, even some that aren't supported when you use a time format string.  If your date and time values use formats different from each other, set this parameter to <code>auto</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerSideEncryptionKmsKeyId</i> The AWS KMS key ID. If you are using <code>SSE_KMS</code> for the <code>EncryptionMode</code>, provide this key ID. The key that you use needs an attached policy that enables IAM user permissions and allows use of the key.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i> The Amazon Resource Name (ARN) of the IAM role that has access to the Amazon Redshift service.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerName</i> The name of the Amazon Redshift cluster you are using.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplaceChars</i> A value that specifies to replaces the invalid characters specified in <code>ReplaceInvalidChars</code>, substituting the specified characters instead. The default is <code>"?"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplaceInvalidChars</i> A list of characters that you want to replace. Use with <code>ReplaceChars</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RemoveQuotes</i> A value that specifies to remove surrounding quotation marks from strings in the incoming data. All characters within the quotation marks, including delimiters, are retained. Choose <code>true</code> to remove quotation marks. The default is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port number for Amazon Redshift. The default value is 5439.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Password</i> The password for the user named in the <code>username</code> property.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxFileSize</i> The maximum size (in KB) of any .csv file used to transfer data to Amazon Redshift. This accepts a value from 1 through 1,048,576. It defaults to 32,768 KB (32 MB).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LoadTimeout</i> The amount of time to wait (in milliseconds) before timing out, beginning from when you begin loading.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FileTransferUploadStreams</i> The number of threads used to upload a single file. This parameter accepts a value from 1 through 64. It defaults to 10.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionMode</i> The type of server-side encryption that you want to use for your data. This encryption type is part of the endpoint settings or the extra connections attributes for Amazon S3. You can choose either <code>SSE_S3</code> (the default) or <code>SSE_KMS</code>. To use <code>SSE_S3</code>, create an AWS Identity and Access Management (IAM) role with a policy that allows <code>"arn:aws:s3:::*"</code> to use the following actions: <code>"s3:PutObject", "s3:ListBucket"</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EmptyAsNull</i> A value that specifies whether AWS DMS should migrate empty CHAR and VARCHAR fields as NULL. A value of <code>true</code> sets empty CHAR and VARCHAR fields to null. The default is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DateFormat</i> The date format that you are using. Valid values are <code>auto</code> (case-sensitive), your date format string enclosed in quotes, or NULL. If this parameter is left unset (NULL), it defaults to a format of 'YYYY-MM-DD'. Using <code>auto</code> recognizes most strings, even some that aren't supported when you use a date format string.  If your date and time values use formats different from each other, set this to <code>auto</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> The name of the Amazon Redshift data warehouse (service) that you are working with.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectionTimeout</i> A value that sets the amount of time to wait (in milliseconds) before timing out, beginning from when you initially establish a connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketName</i> The name of the S3 bucket you want to use<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketFolder</i> The location where the comma-separated value (.csv) files are stored before being uploaded to the S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AfterConnectScript</i> Code to run after connecting. This parameter should contain the code itself, not the name of a file containing the code.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AcceptAnyDate</i> A value that indicates to allow any date format, including invalid formats such as 00/00/00 00:00:00, to be loaded without generating an error. You can choose <code>true</code> or <code>false</code> (the default). This parameter applies only to TIMESTAMP and DATE columns. Always use ACCEPTANYDATE with the DATEFORMAT parameter. If the date format for the data doesn't match the DATEFORMAT specification, Amazon Redshift inserts a NULL value into that field. <br/>        """
        pass

    def delete_event_subscription(self, SubscriptionName: str) -> DeleteEventSubscriptionResponse:
        """ Deletes an AWS DMS event subscription. 

        :param self:
        :param SubscriptionName:  The name of the DMS event notification subscription to be deleted.<br/>
        :return: EventSubscription:  The event subscription that was deleted.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Boolean value that indicates if the event subscription is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EventCategoriesList</i> A lists of event categories.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceIdsList</i> A list of source Ids for the event subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceType</i>  The type of AWS DMS resource that generates events.  Valid values: replication-instance | replication-server | security-group | replication-task<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubscriptionCreationTime</i> The time the RDS event notification subscription was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the AWS DMS event notification subscription. Constraints: Can be one of the following: creating | modifying | deleting | active | no-permission | topic-not-exist The status "no-permission" indicates that AWS DMS no longer has permission to post to the SNS topic. The status "topic-not-exist" indicates that the topic was deleted after the subscription was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsTopicArn</i> The topic ARN of the AWS DMS event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustSubscriptionId</i> The AWS DMS event notification subscription Id.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomerAwsId</i> The AWS customer account associated with the AWS DMS event notification subscription.<br/>        """
        pass

    def delete_replication_instance(self, ReplicationInstanceArn: str) -> DeleteReplicationInstanceResponse:
        """Deletes the specified replication instance. <note> You must delete any migration tasks that are associated with the replication instance before you can delete it. </note> <p/>

        :param self:
        :param ReplicationInstanceArn:  The Amazon Resource Name (ARN) of the replication instance to be deleted.<br/>
        :return: ReplicationInstance:  The replication instance that was deleted.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DnsNameServers</i> The DNS name servers for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FreeUntil</i>  The expiration date of the free replication instance that is part of the Free DMS program. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondaryAvailabilityZone</i> The availability zone of the standby replication instance in a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i>  Specifies the accessibility options for the replication instance. A value of <code>true</code> represents an instance with a public IP address. A value of <code>false</code> represents an instance with a private IP address. The default value is <code>true</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstancePrivateIpAddresses</i> One or more private IP addresses for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstancePublicIpAddresses</i> One or more public IP addresses for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstancePrivateIpAddress</i> The private IP address of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstancePublicIpAddress</i> The public IP address of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceArn</i> The Amazon Resource Name (ARN) of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> An AWS KMS key identifier that is used to encrypt the data on the replication instance. If you don't specify a value for the <code>KmsKeyId</code> parameter, then AWS DMS uses your default encryption key. AWS KMS creates the default encryption key for your AWS account. Your AWS account has a different default encryption key for each AWS Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> Boolean value indicating if minor version upgrades will be automatically applied to the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> The engine version number of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i>  Specifies whether the replication instance is a Multi-AZ deployment. You cannot set the <code>AvailabilityZone</code> parameter if the Multi-AZ parameter is set to <code>true</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> The pending modification values.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: The pending modification values.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> The maintenance window times for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSubnetGroup</i> The subnet group for the replication instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ReplicationSubnetGroup</b>: The subnet group for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> The VPC security group for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCreateTime</i> The time the replication instance was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> The amount of storage (in gigabytes) that is allocated for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceStatus</i> The status of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceClass</i> The compute and memory capacity of the replication instance.  Valid Values: <code>dms.t2.micro | dms.t2.small | dms.t2.medium | dms.t2.large | dms.c4.large | dms.c4.xlarge | dms.c4.2xlarge | dms.c4.4xlarge </code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceIdentifier</i> The replication instance identifier. This parameter is stored as a lowercase string. Constraints: <ul> <li> Must contain from 1 to 63 alphanumeric characters or hyphens. </li> <li> First character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul> Example: <code>myrepinstance</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> The subnets that are in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> The status of the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSubnetGroupDescription</i> A description for the replication subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSubnetGroupIdentifier</i> The identifier of the replication instance subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> The engine version number of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i>  Specifies whether the replication instance is a Multi-AZ deployment. You cannot set the <code>AvailabilityZone</code> parameter if the Multi-AZ parameter is set to <code>true</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> The amount of storage (in gigabytes) that is allocated for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceClass</i> The compute and memory capacity of the replication instance.  Valid Values: <code>dms.t2.micro | dms.t2.small | dms.t2.medium | dms.t2.large | dms.c4.large | dms.c4.xlarge | dms.c4.2xlarge | dms.c4.4xlarge </code> <br/>        """
        pass

    def delete_replication_subnet_group(self, ReplicationSubnetGroupIdentifier: str) -> DeleteReplicationSubnetGroupResponse:
        """Deletes a subnet group.

        :param self:
        :param ReplicationSubnetGroupIdentifier:  The subnet group name of the replication instance.<br/>
        :return:         """
        pass

    def delete_replication_task(self, ReplicationTaskArn: str) -> DeleteReplicationTaskResponse:
        """Deletes the specified replication task.

        :param self:
        :param ReplicationTaskArn:  The Amazon Resource Name (ARN) of the replication task to be deleted.<br/>
        :return: ReplicationTask:  The deleted replication task.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskStats</i> The statistics for the task, including elapsed time, tables loaded, and table errors.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ReplicationTaskStats</b>: The statistics for the task, including elapsed time, tables loaded, and table errors.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskArn</i> The Amazon Resource Name (ARN) of the replication task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecoveryCheckpoint</i> Indicates the last checkpoint that occurred during a change data capture (CDC) operation. You can provide this value to the <code>CdcStartPosition</code> parameter to start a CDC operation that begins at that checkpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CdcStopPosition</i> Indicates when you want a change data capture (CDC) operation to stop. The value can be either server time or commit time. Server time example: --cdc-stop-position “server_time:3018-02-09T12:12:12” Commit time example: --cdc-stop-position “commit_time: 3018-02-09T12:12:12 “<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CdcStartPosition</i> Indicates when you want a change data capture (CDC) operation to start. Use either <code>CdcStartPosition</code> or <code>CdcStartTime</code> to specify when you want the CDC operation to start. Specifying both values results in an error. The value can be in date, checkpoint, or LSN/SCN format. Date Example: --cdc-start-position “2018-03-08T12:12:12” Checkpoint Example: --cdc-start-position "checkpoint:V1#27#mysql-bin-changelog.157832:1975:-1:2002:677883278264080:mysql-bin-changelog.157832:1876#0#0#*#0#93" LSN Example: --cdc-start-position “mysql-bin-changelog.000024:373”<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskStartDate</i> The date the replication task is scheduled to start.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskCreationDate</i> The date the replication task was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StopReason</i> The reason the replication task was stopped.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastFailureMessage</i> The last error (failure) message generated for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the replication task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskSettings</i> The settings for the replication task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableMappings</i> Table mappings specified in the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MigrationType</i> The type of migration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceArn</i> The Amazon Resource Name (ARN) of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetEndpointArn</i> The Amazon Resource Name (ARN) string that uniquely identifies the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceEndpointArn</i> The Amazon Resource Name (ARN) string that uniquely identifies the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskIdentifier</i> The user-assigned replication task identifier or name. Constraints: <ul> <li> Must contain from 1 to 255 alphanumeric characters or hyphens. </li> <li> First character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TablesErrored</i> The number of errors that have occurred during this task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TablesQueued</i> The number of tables queued for this task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TablesLoading</i> The number of tables currently loading for this task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TablesLoaded</i> The number of tables loaded for this task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeMillis</i> The elapsed time of the task, in milliseconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FullLoadProgressPercent</i> The percent complete for the full load migration task.<br/>        """
        pass

    def describe_account_attributes(self) -> DescribeAccountAttributesResponse:
        """Lists all of the AWS DMS attributes for a customer account. These attributes include AWS DMS quotas for the account and a unique account identifier in a particular DMS region. DMS quotas include a list of resource quotas supported by the account, such as the number of replication instances allowed. The description for each resource quota, includes the quota name, current usage toward that quota, and the quota's maximum value. DMS uses the unique account identifier to name each artifact used by DMS in the given region. This command does not take any parameters.

        :param self:
        :param 
        :return: UniqueAccountIdentifier:  A unique AWS DMS identifier for an account in a particular AWS Region. The value of this identifier has the following format: <code>c99999999999</code>. DMS uses this identifier to name artifacts. For example, DMS uses this identifier to name the default Amazon S3 bucket for storing task assessment reports in a given AWS Region. The format of this S3 bucket name is the following: <code>dms-<i>AccountNumber</i>-<i>UniqueAccountIdentifier</i>.</code> Here is an example name for this default S3 bucket: <code>dms-111122223333-c44445555666</code>. <note> AWS DMS supports <code>UniqueAccountIdentifier</code> in versions 3.1.4 and later. </note><br/>AccountQuotas:  Account quota information.<br/>        """
        pass

    def describe_certificates(self, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DescribeCertificatesResponse:
        """Provides a description of the certificate.

        :param self:
        :param Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the vlue specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 10<br/>Filters:  Filters applied to the certificate described in the form of key-value pairs.<br/>
        :return: Certificates:  The Secure Sockets Layer (SSL) certificates associated with the replication instance.<br/>Marker:  The pagination token.<br/>        """
        pass

    def describe_connections(self, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DescribeConnectionsResponse:
        """Describes the status of the connections that have been made between the replication instance and an endpoint. Connections are created when you test an endpoint.

        :param self:
        :param Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  The filters applied to the connection. Valid filter names: endpoint-arn | replication-instance-arn<br/>
        :return: Connections:  A description of the connections.<br/>Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>        """
        pass

    def describe_endpoint_types(self, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DescribeEndpointTypesResponse:
        """Returns information about the type of endpoints available.

        :param self:
        :param Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  Filters applied to the describe action. Valid filter names: engine-name | endpoint-type<br/>
        :return: SupportedEndpointTypes:  The types of endpoints that are supported.<br/>Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>        """
        pass

    def describe_endpoints(self, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DescribeEndpointsResponse:
        """Returns information about the endpoints for your account in the current region.

        :param self:
        :param Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  Filters applied to the describe action. Valid filter names: endpoint-arn | endpoint-type | endpoint-id | engine-name<br/>
        :return: Endpoints:  Endpoint description.<br/>Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>        """
        pass

    def describe_event_categories(self, SourceType: str=None, Filters: List[Filter]=None) -> DescribeEventCategoriesResponse:
        """Lists categories for all event source types, or, if specified, for a specified source type. You can see a list of the event categories and source types in <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Events.html">Working with Events and Notifications</a> in the <i>AWS Database Migration Service User Guide.</i> 

        :param self:
        :param Filters:  Filters applied to the action.<br/>SourceType:   The type of AWS DMS resource that generates events.  Valid values: replication-instance | replication-task<br/>
        :return: EventCategoryGroupList:  A list of event categories.<br/>        """
        pass

    def describe_event_subscriptions(self, SubscriptionName: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DescribeEventSubscriptionsResponse:
        """Lists all the event subscriptions for a customer account. The description of a subscription includes <code>SubscriptionName</code>, <code>SNSTopicARN</code>, <code>CustomerID</code>, <code>SourceType</code>, <code>SourceID</code>, <code>CreationTime</code>, and <code>Status</code>.  If you specify <code>SubscriptionName</code>, this action lists the description for that subscription.

        :param self:
        :param Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  Filters applied to the action.<br/>SubscriptionName:  The name of the AWS DMS event subscription to be described.<br/>
        :return: EventSubscriptionsList:  A list of event subscriptions.<br/>Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>        """
        pass

    def describe_events(self, SourceIdentifier: str=None, SourceType: str=None, StartTime: TStamp=None, EndTime: TStamp=None, Duration: int=None, EventCategories: List[String]=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DescribeEventsResponse:
        """ Lists events for a given source identifier and source type. You can also specify a start and end time. For more information on AWS DMS events, see <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Events.html">Working with Events and Notifications</a> in the <i>AWS Database Migration User Guide.</i> 

        :param self:
        :param Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  Filters applied to the action.<br/>EventCategories:  A list of event categories for the source type that you've chosen.<br/>Duration:  The duration of the events to be listed.<br/>EndTime:  The end time for the events to be listed.<br/>StartTime:  The start time for the events to be listed.<br/>SourceType:  The type of AWS DMS resource that generates events. Valid values: replication-instance | replication-task<br/>SourceIdentifier:   The identifier of an event source.<br/>
        :return: Events:  The events described.<br/>Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>        """
        pass

    def describe_orderable_replication_instances(self, MaxRecords: int=None, Marker: str=None) -> DescribeOrderableReplicationInstancesResponse:
        """Returns information about the replication instance types that can be created in the specified region.

        :param self:
        :param Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>
        :return: Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>OrderableReplicationInstances:  The order-able replication instances available.<br/>        """
        pass

    def describe_pending_maintenance_actions(self, ReplicationInstanceArn: str=None, Filters: List[Filter]=None, Marker: str=None, MaxRecords: int=None) -> DescribePendingMaintenanceActionsResponse:
        """For internal use only

        :param self:
        :param MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>Filters:  <p/><br/>ReplicationInstanceArn:  The Amazon Resource Name (ARN) of the replication instance.<br/>
        :return: Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>PendingMaintenanceActions:  The pending maintenance action.<br/>        """
        pass

    def describe_refresh_schemas_status(self, EndpointArn: str) -> DescribeRefreshSchemasStatusResponse:
        """Returns the status of the RefreshSchemas operation.

        :param self:
        :param EndpointArn:  The Amazon Resource Name (ARN) string that uniquely identifies the endpoint.<br/>
        :return: RefreshSchemasStatus:  The status of the schema.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastFailureMessage</i> The last failure message for the schema.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastRefreshDate</i> The date the schema was last refreshed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the schema.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceArn</i> The Amazon Resource Name (ARN) of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointArn</i> The Amazon Resource Name (ARN) string that uniquely identifies the endpoint.<br/>        """
        pass

    def describe_replication_instance_task_logs(self, ReplicationInstanceArn: str, MaxRecords: int=None, Marker: str=None) -> DescribeReplicationInstanceTaskLogsResponse:
        """Returns information about the task logs for the specified task.

        :param self:
        :param Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>ReplicationInstanceArn:  The Amazon Resource Name (ARN) of the replication instance.<br/>
        :return: Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>ReplicationInstanceTaskLogs:  An array of replication task log metadata. Each member of the array contains the replication task name, ARN, and task log size (in bytes). <br/>ReplicationInstanceArn:  The Amazon Resource Name (ARN) of the replication instance.<br/>        """
        pass

    def describe_replication_instances(self, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DescribeReplicationInstancesResponse:
        """Returns information about replication instances for your account in the current region.

        :param self:
        :param Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  Filters applied to the describe action. Valid filter names: replication-instance-arn | replication-instance-id | replication-instance-class | engine-version<br/>
        :return: ReplicationInstances:  The replication instances described.<br/>Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>        """
        pass

    def describe_replication_subnet_groups(self, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DescribeReplicationSubnetGroupsResponse:
        """Returns information about the replication subnet groups.

        :param self:
        :param Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  Filters applied to the describe action.<br/>
        :return: ReplicationSubnetGroups:  A description of the replication subnet groups.<br/>Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>        """
        pass

    def describe_replication_task_assessment_results(self, ReplicationTaskArn: str=None, MaxRecords: int=None, Marker: str=None) -> DescribeReplicationTaskAssessmentResultsResponse:
        """Returns the task assessment results from Amazon S3. This action always returns the latest results.

        :param self:
        :param Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>ReplicationTaskArn:  - The Amazon Resource Name (ARN) string that uniquely identifies the task. When this input parameter is specified the API will return only one result and ignore the values of the max-records and marker parameters. <br/>
        :return: ReplicationTaskAssessmentResults:   The task assessment report. <br/>BucketName:  - The Amazon S3 bucket where the task assessment report is located. <br/>Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>        """
        pass

    def describe_replication_tasks(self, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None, WithoutSettings: BooleanOptional=None) -> DescribeReplicationTasksResponse:
        """Returns information about replication tasks for your account in the current region.

        :param self:
        :param WithoutSettings:  An option to set to avoid returning information about settings. Use this to reduce overhead when setting information is too large. To use this option, choose <code>true</code>; otherwise, choose <code>false</code> (the default).<br/>Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  Filters applied to the describe action. Valid filter names: replication-task-arn | replication-task-id | migration-type | endpoint-arn | replication-instance-arn<br/>
        :return: ReplicationTasks:  A description of the replication tasks.<br/>Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>        """
        pass

    def describe_schemas(self, EndpointArn: str, MaxRecords: int=None, Marker: str=None) -> DescribeSchemasResponse:
        """Returns information about the schema for the specified endpoint. <p/>

        :param self:
        :param Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>EndpointArn:  The Amazon Resource Name (ARN) string that uniquely identifies the endpoint.<br/>
        :return: Schemas:  The described schema.<br/>Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>        """
        pass

    def describe_table_statistics(self, ReplicationTaskArn: str, MaxRecords: int=None, Marker: str=None, Filters: List[Filter]=None) -> DescribeTableStatisticsResponse:
        """Returns table statistics on the database migration task, including table name, rows inserted, rows updated, and rows deleted. Note that the "last updated" column the DMS console only indicates the time that AWS DMS last updated the table statistics record for a table. It does not indicate the time of the last update to the table.

        :param self:
        :param Filters:  Filters applied to the describe table statistics action. Valid filter names: schema-name | table-name | table-state A combination of filters creates an AND condition where each record matches all specified filters.<br/>Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 500.<br/>ReplicationTaskArn:  The Amazon Resource Name (ARN) of the replication task.<br/>
        :return: Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>TableStatistics:  The table statistics.<br/>ReplicationTaskArn:  The Amazon Resource Name (ARN) of the replication task.<br/>        """
        pass

    def import_certificate(self, CertificateIdentifier: str, CertificatePem: str=None, CertificateWallet: CertificateWallet=None, Tags: List[Tag]=None) -> ImportCertificateResponse:
        """Uploads the specified certificate.

        :param self:
        :param Tags:  The tags associated with the certificate.<br/>CertificateWallet:  The location of an imported Oracle Wallet certificate for use with SSL.<br/>CertificatePem:  The contents of a <code>.pem</code> file, which contains an X.509 certificate.<br/>CertificateIdentifier:  A customer-assigned name for the certificate. Identifiers must begin with a letter; must contain only ASCII letters, digits, and hyphens; and must not end with a hyphen or contain two consecutive hyphens.<br/>
        :return: Certificate:  The certificate to be uploaded.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyLength</i> The key length of the cryptographic algorithm being used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SigningAlgorithm</i> The signing algorithm for the certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ValidToDate</i> The final date that the certificate is valid.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ValidFromDate</i> The beginning date that the certificate is valid.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateOwner</i> The owner of the certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateArn</i> The Amazon Resource Name (ARN) for the certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateWallet</i> The location of an imported Oracle Wallet certificate for use with SSL.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificatePem</i> The contents of a <code>.pem</code> file, which contains an X.509 certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateCreationDate</i> The date that the certificate was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateIdentifier</i> A customer-assigned name for the certificate. Identifiers must begin with a letter; must contain only ASCII letters, digits, and hyphens; and must not end with a hyphen or contain two consecutive hyphens.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceArn: str) -> ListTagsForResourceResponse:
        """Lists all tags for an AWS DMS resource.

        :param self:
        :param ResourceArn:  The Amazon Resource Name (ARN) string that uniquely identifies the AWS DMS resource.<br/>
        :return: TagList:  A list of tags for the resource.<br/>        """
        pass

    def modify_endpoint(self, EndpointArn: str, EndpointIdentifier: str=None, EndpointType: str=None, EngineName: str=None, Username: str=None, Password: str=None, ServerName: str=None, Port: int=None, DatabaseName: str=None, ExtraConnectionAttributes: str=None, CertificateArn: str=None, SslMode: str=None, ServiceAccessRoleArn: str=None, ExternalTableDefinition: str=None, DynamoDbSettings: DynamoDbSettings=None, S3Settings: S3Settings=None, DmsTransferSettings: DmsTransferSettings=None, MongoDbSettings: MongoDbSettings=None, KinesisSettings: KinesisSettings=None, ElasticsearchSettings: ElasticsearchSettings=None, RedshiftSettings: RedshiftSettings=None) -> ModifyEndpointResponse:
        """Modifies the specified endpoint.

        :param self:
        :param RedshiftSettings:  <p/>:<br/>ElasticsearchSettings:  Settings in JSON format for the target Elasticsearch endpoint. For more information about the available settings, see <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Target.Elasticsearch.html#CHAP_Target.Elasticsearch.Configuration">Extra Connection Attributes When Using Elasticsearch as a Target for AWS DMS</a> in the <i>AWS Database Migration User Guide.</i> :<br/>KinesisSettings:  Settings in JSON format for the target Amazon Kinesis Data Streams endpoint. For more information about the available settings, see <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Target.Kinesis.html#CHAP_Target.Kinesis.ObjectMapping">Using Object Mapping to Migrate Data to a Kinesis Data Stream</a> in the <i>AWS Database Migration User Guide.</i> :<br/>MongoDbSettings:  Settings in JSON format for the source MongoDB endpoint. For more information about the available settings, see the configuration properties section in <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Source.MongoDB.html"> Using MongoDB as a Target for AWS Database Migration Service</a> in the <i>AWS Database Migration Service User Guide.</i> :<br/>DmsTransferSettings:  The settings in JSON format for the DMS transfer type of source endpoint.  Attributes include the following: <ul> <li> serviceAccessRoleArn - The IAM role that has permission to access the Amazon S3 bucket. </li> <li> BucketName - The name of the S3 bucket to use. </li> <li> compressionType - An optional parameter to use GZIP to compress the target files. Set to NONE (the default) or do not use to leave the files uncompressed. </li> </ul> Shorthand syntax: ServiceAccessRoleArn=string ,BucketName=string,CompressionType=string JSON syntax:  { "ServiceAccessRoleArn": "string", "BucketName": "string", "CompressionType": "none"|"gzip" } :<br/>S3Settings:  Settings in JSON format for the target Amazon S3 endpoint. For more information about the available settings, see <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Target.S3.html#CHAP_Target.S3.Configuring">Extra Connection Attributes When Using Amazon S3 as a Target for AWS DMS</a> in the <i>AWS Database Migration Service User Guide.</i> :<br/>DynamoDbSettings:  Settings in JSON format for the target Amazon DynamoDB endpoint. For more information about the available settings, see <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Target.DynamoDB.html">Using Object Mapping to Migrate Data to DynamoDB</a> in the <i>AWS Database Migration Service User Guide.</i> :<br/>ExternalTableDefinition:  The external table definition.<br/>ServiceAccessRoleArn:   The Amazon Resource Name (ARN) for the service access role you want to use to modify the endpoint. <br/>SslMode:  The SSL mode used to connect to the endpoint. The default value is <code>none</code>.<br/>CertificateArn:  The Amazon Resource Name (ARN) of the certificate used for SSL connection.<br/>ExtraConnectionAttributes:  Additional attributes associated with the connection. To reset this parameter, pass the empty string ("") as an argument.<br/>DatabaseName:  The name of the endpoint database.<br/>Port:  The port used by the endpoint database.<br/>ServerName:  The name of the server where the endpoint database resides.<br/>Password:  The password to be used to login to the endpoint database.<br/>Username:  The user name to be used to login to the endpoint database.<br/>EngineName:  The type of engine for the endpoint. Valid values, depending on the EndpointType, include mysql, oracle, postgres, mariadb, aurora, aurora-postgresql, redshift, s3, db2, azuredb, sybase, dynamodb, mongodb, and sqlserver.<br/>EndpointType:  The type of endpoint. Valid values are <code>source</code> and <code>target</code>.<br/>EndpointIdentifier:  The database endpoint identifier. Identifiers must begin with a letter; must contain only ASCII letters, digits, and hyphens; and must not end with a hyphen or contain two consecutive hyphens.<br/>EndpointArn:  The Amazon Resource Name (ARN) string that uniquely identifies the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i>  The Amazon Resource Name (ARN) used by the service access IAM role. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimestampColumnName</i> A value that includes a timestamp column in the Amazon S3 target endpoint data. AWS DMS includes an additional column in the migrated data when you set <code>timestampColumnName</code> to a non-blank value.  <note> AWS DMS supports <code>TimestampColumnName</code> in versions 3.1.4 and later. </note> For a full load, each row of the timestamp column contains a timestamp for when the data was transferred from the source to the target by DMS. For a CDC load, each row of the timestamp column contains the timestamp for the commit of that row in the source database. The format for the timestamp column value is <code>yyyy-MM-dd HH:mm:ss.SSSSSS</code>. For CDC, the microsecond precision depends on the commit timestamp supported by DMS for the source database. When the <code>AddColumnName</code> setting is set to <code>true</code>, DMS also includes the name for the timestamp column that you set as the nonblank value of <code>timestampColumnName</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CdcInsertsOnly</i> A value that enables a change data capture (CDC) load to write only INSERT operations to .csv or columnar storage (.parquet) output files. By default (the <code>false</code> setting), the first field in a .csv or .parquet record contains the letter I (INSERT), U (UPDATE), or D (DELETE). These values indicate whether the row was inserted, updated, or deleted at the source database for a CDC load to the target. If <code>cdcInsertsOnly</code> is set to <code>true</code> or <code>y</code>, only INSERTs from the source database are migrated to the .csv or .parquet file. For .csv format only, how these INSERTs are recorded depends on the value of <code>IncludeOpForFullLoad</code>. If <code>IncludeOpForFullLoad</code> is set to <code>true</code>, the first field of every CDC record is set to I to indicate the INSERT operation at the source. If <code>IncludeOpForFullLoad</code> is set to <code>false</code>, every CDC record is written without a first field to indicate the INSERT operation at the source. For more information about how these settings work together, see <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Target.S3.html#CHAP_Target.S3.Configuring.InsertOps">Indicating Source DB Operations in Migrated S3 Data</a> in the <i>AWS Database Migration Service User Guide.</i>. <note> AWS DMS supports this interaction between <code>CdcInsertsOnly</code> and <code>IncludeOpForFullLoad</code> in versions 3.1.4 and later.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeOpForFullLoad</i> A value that enables a full load to write INSERT operations to the comma-separated value (.csv) output files only to indicate how the rows were added to the source database. <note> AWS DMS supports <code>IncludeOpForFullLoad</code> in versions 3.1.4 and later. </note> For full load, records can only be inserted. By default (the <code>false</code> setting), no information is recorded in these output files for a full load to indicate that the rows were inserted at the source database. If <code>IncludeOpForFullLoad</code> is set to <code>true</code> or <code>y</code>, the INSERT is recorded as an I annotation in the first field of the .csv file. This allows the format of your target records from a full load to be consistent with the target records from a CDC load. <note> This setting works together with <code>CdcInsertsOnly</code> for output to .csv files only. For more information about how these settings work together, see <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Target.S3.html#CHAP_Target.S3.Configuring.InsertOps">Indicating Source DB Operations in Migrated S3 Data</a> in the <i>AWS Database Migration Service User Guide.</i>. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnableStatistics</i> A value that enables statistics for Parquet pages and row groups. Choose <code>true</code> to enable statistics, <code>false</code> to disable. Statistics include <code>NULL</code>, <code>DISTINCT</code>, <code>MAX</code>, and <code>MIN</code> values. This parameter defaults to <code>true</code>. This value is used for .parquet file format only.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParquetVersion</i> The version of the Apache Parquet format that you want to use: <code>parquet_1_0</code> (the default) or <code>parquet_2_0</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataPageSize</i> The size of one data page in bytes. This parameter defaults to 1024 * 1024 bytes (1 MiB). This number is used for .parquet file format only. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RowGroupLength</i> The number of rows in a row group. A smaller row group size provides faster reads. But as the number of row groups grows, the slower writes become. This parameter defaults to 10,000 rows. This number is used for .parquet file format only.  If you choose a value larger than the maximum, <code>RowGroupLength</code> is set to the max row group length in bytes (64 * 1024 * 1024). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DictPageSizeLimit</i> The maximum size of an encoded dictionary page of a column. If the dictionary page exceeds this, this column is stored using an encoding type of <code>PLAIN</code>. This parameter defaults to 1024 * 1024 bytes (1 MiB), the maximum size of a dictionary page before it reverts to <code>PLAIN</code> encoding. This size is used for .parquet file format only. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncodingType</i> The type of encoding you are using:  <ul> <li>  <code>RLE_DICTIONARY</code> uses a combination of bit-packing and run-length encoding to store repeated values more efficiently. This is the default. </li> <li>  <code>PLAIN</code> doesn't use encoding at all. Values are stored as they are. </li> <li>  <code>PLAIN_DICTIONARY</code> builds a dictionary of the values encountered in a given column. The dictionary is stored in a dictionary page for each column chunk. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataFormat</i> The format of the data that you want to use for output. You can choose one of the following:  <ul> <li>  <code>csv</code> : This is a row-based file format with comma-separated values (.csv).  </li> <li>  <code>parquet</code> : Apache Parquet (.parquet) is a columnar storage file format that features efficient compression and provides faster query response.  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerSideEncryptionKmsKeyId</i> If you are using <code>SSE_KMS</code> for the <code>EncryptionMode</code>, provide the AWS KMS key ID. The key that you use needs an attached policy that enables AWS Identity and Access Management (IAM) user permissions and allows use of the key. Here is a CLI example: <code>aws dms create-endpoint --endpoint-identifier <i>value</i> --endpoint-type target --engine-name s3 --s3-settings ServiceAccessRoleArn=<i>value</i>,BucketFolder=<i>value</i>,BucketName=<i>value</i>,EncryptionMode=SSE_KMS,ServerSideEncryptionKmsKeyId=<i>value</i> </code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionMode</i> The type of server-side encryption that you want to use for your data. This encryption type is part of the endpoint settings or the extra connections attributes for Amazon S3. You can choose either <code>SSE_S3</code> (the default) or <code>SSE_KMS</code>. To use <code>SSE_S3</code>, you need an AWS Identity and Access Management (IAM) role with permission to allow <code>"arn:aws:s3:::dms-*"</code> to use the following actions: <ul> <li>  <code>s3:CreateBucket</code>  </li> <li>  <code>s3:ListBucket</code>  </li> <li>  <code>s3:DeleteBucket</code>  </li> <li>  <code>s3:GetBucketLocation</code>  </li> <li>  <code>s3:GetObject</code>  </li> <li>  <code>s3:PutObject</code>  </li> <li>  <code>s3:DeleteObject</code>  </li> <li>  <code>s3:GetObjectVersion</code>  </li> <li>  <code>s3:GetBucketPolicy</code>  </li> <li>  <code>s3:PutBucketPolicy</code>  </li> <li>  <code>s3:DeleteBucketPolicy</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompressionType</i>  An optional parameter to use GZIP to compress the target files. Set to GZIP to compress the target files. Set to NONE (the default) or do not use to leave the files uncompressed. Applies to both .csv and .parquet file formats. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketName</i>  The name of the S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketFolder</i>  An optional parameter to set a folder name in the S3 bucket. If provided, tables are created in the path <code> <i>bucketFolder</i>/<i>schema_name</i>/<i>table_name</i>/</code>. If this parameter is not specified, then the path used is <code> <i>schema_name</i>/<i>table_name</i>/</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CsvDelimiter</i>  The delimiter used to separate columns in the source files. The default is a comma. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CsvRowDelimiter</i>  The delimiter used to separate rows in the source files. The default is a carriage return (<code>\n</code>). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalTableDefinition</i>  The external table definition. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i>  The Amazon Resource Name (ARN) used by the service access IAM role. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketName</i>  The name of the S3 bucket to use. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i>  The IAM role that has permission to access the Amazon S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS KMS key identifier that is used to encrypt the content on the replication instance. If you don't specify a value for the <code>KmsKeyId</code> parameter, then AWS DMS uses your default encryption key. AWS KMS creates the default encryption key for your AWS account. Your AWS account has a different default encryption key for each AWS Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthSource</i>  The MongoDB database name. This attribute is not used when <code>authType=NO</code>.  The default is admin.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DocsToInvestigate</i>  Indicates the number of documents to preview to determine the document organization. Use this attribute when <code>NestingLevel</code> is set to ONE.  Must be a positive value greater than 0. Default value is 1000.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExtractDocId</i>  Specifies the document ID. Use this attribute when <code>NestingLevel</code> is set to NONE.  Default value is false. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NestingLevel</i>  Specifies either document or table mode.  Valid values: NONE, ONE Default value is NONE. Specify NONE to use document mode. Specify ONE to use table mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthMechanism</i>  The authentication mechanism you use to access the MongoDB source endpoint. Valid values: DEFAULT, MONGODB_CR, SCRAM_SHA_1  DEFAULT – For MongoDB version 2.x, use MONGODB_CR. For MongoDB version 3.x, use SCRAM_SHA_1. This attribute is not used when authType=No.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthType</i>  The authentication type you use to access the MongoDB source endpoint. Valid values: NO, PASSWORD  When NO is selected, user name and password parameters are not used and can be empty. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i>  The database name on the MongoDB source endpoint. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i>  The port value for the MongoDB source endpoint. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerName</i>  The name of the server on the MongoDB source endpoint. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Password</i>  The password for the user account you use to access the MongoDB source endpoint. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> The user name you use to access the MongoDB source endpoint. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i> The Amazon Resource Name (ARN) for the IAM role that DMS uses to write to the Amazon Kinesis data stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MessageFormat</i> The output format for the records created on the endpoint. The message format is <code>JSON</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamArn</i> The Amazon Resource Name (ARN) for the Amazon Kinesis Data Streams endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorRetryDuration</i> The maximum number of seconds that DMS retries failed API requests to the Elasticsearch cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FullLoadErrorPercentage</i> The maximum percentage of records that can fail to be written before a full load operation stops. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointUri</i> The endpoint for the Elasticsearch cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i> The Amazon Resource Name (ARN) used by service to access the IAM role.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WriteBufferSize</i> The size of the write buffer to use in rows. Valid values range from 1 through 2,048. The default is 1,024. Use this setting to tune performance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> An Amazon Redshift user name for a registered user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TruncateColumns</i> A value that specifies to truncate data in columns to the appropriate number of characters, so that the data fits in the column. This parameter applies only to columns with a VARCHAR or CHAR data type, and rows with a size of 4 MB or less. Choose <code>true</code> to truncate data. The default is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrimBlanks</i> A value that specifies to remove the trailing white space characters from a VARCHAR string. This parameter applies only to columns with a VARCHAR data type. Choose <code>true</code> to remove unneeded white space. The default is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeFormat</i> The time format that you want to use. Valid values are <code>auto</code> (case-sensitive), <code>'timeformat_string'</code>, <code>'epochsecs'</code>, or <code>'epochmillisecs'</code>. It defaults to 10. Using <code>auto</code> recognizes most strings, even some that aren't supported when you use a time format string.  If your date and time values use formats different from each other, set this parameter to <code>auto</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerSideEncryptionKmsKeyId</i> The AWS KMS key ID. If you are using <code>SSE_KMS</code> for the <code>EncryptionMode</code>, provide this key ID. The key that you use needs an attached policy that enables IAM user permissions and allows use of the key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i> The Amazon Resource Name (ARN) of the IAM role that has access to the Amazon Redshift service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerName</i> The name of the Amazon Redshift cluster you are using.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplaceChars</i> A value that specifies to replaces the invalid characters specified in <code>ReplaceInvalidChars</code>, substituting the specified characters instead. The default is <code>"?"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplaceInvalidChars</i> A list of characters that you want to replace. Use with <code>ReplaceChars</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RemoveQuotes</i> A value that specifies to remove surrounding quotation marks from strings in the incoming data. All characters within the quotation marks, including delimiters, are retained. Choose <code>true</code> to remove quotation marks. The default is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port number for Amazon Redshift. The default value is 5439.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Password</i> The password for the user named in the <code>username</code> property.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxFileSize</i> The maximum size (in KB) of any .csv file used to transfer data to Amazon Redshift. This accepts a value from 1 through 1,048,576. It defaults to 32,768 KB (32 MB).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LoadTimeout</i> The amount of time to wait (in milliseconds) before timing out, beginning from when you begin loading.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FileTransferUploadStreams</i> The number of threads used to upload a single file. This parameter accepts a value from 1 through 64. It defaults to 10.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionMode</i> The type of server-side encryption that you want to use for your data. This encryption type is part of the endpoint settings or the extra connections attributes for Amazon S3. You can choose either <code>SSE_S3</code> (the default) or <code>SSE_KMS</code>. To use <code>SSE_S3</code>, create an AWS Identity and Access Management (IAM) role with a policy that allows <code>"arn:aws:s3:::*"</code> to use the following actions: <code>"s3:PutObject", "s3:ListBucket"</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EmptyAsNull</i> A value that specifies whether AWS DMS should migrate empty CHAR and VARCHAR fields as NULL. A value of <code>true</code> sets empty CHAR and VARCHAR fields to null. The default is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DateFormat</i> The date format that you are using. Valid values are <code>auto</code> (case-sensitive), your date format string enclosed in quotes, or NULL. If this parameter is left unset (NULL), it defaults to a format of 'YYYY-MM-DD'. Using <code>auto</code> recognizes most strings, even some that aren't supported when you use a date format string.  If your date and time values use formats different from each other, set this to <code>auto</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> The name of the Amazon Redshift data warehouse (service) that you are working with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectionTimeout</i> A value that sets the amount of time to wait (in milliseconds) before timing out, beginning from when you initially establish a connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketName</i> The name of the S3 bucket you want to use<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketFolder</i> The location where the comma-separated value (.csv) files are stored before being uploaded to the S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AfterConnectScript</i> Code to run after connecting. This parameter should contain the code itself, not the name of a file containing the code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AcceptAnyDate</i> A value that indicates to allow any date format, including invalid formats such as 00/00/00 00:00:00, to be loaded without generating an error. You can choose <code>true</code> or <code>false</code> (the default). This parameter applies only to TIMESTAMP and DATE columns. Always use ACCEPTANYDATE with the DATEFORMAT parameter. If the date format for the data doesn't match the DATEFORMAT specification, Amazon Redshift inserts a NULL value into that field. <br/>
        :return: Endpoint:  The modified endpoint.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RedshiftSettings</i> Settings for the Amazon Redshift endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RedshiftSettings</b>: Settings for the Amazon Redshift endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticsearchSettings</i> The settings for the Elasticsearch source endpoint. For more information, see the <code>ElasticsearchSettings</code> structure.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ElasticsearchSettings</b>: The settings for the Elasticsearch source endpoint. For more information, see the <code>ElasticsearchSettings</code> structure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KinesisSettings</i> The settings for the Amazon Kinesis source endpoint. For more information, see the <code>KinesisSettings</code> structure.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KinesisSettings</b>: The settings for the Amazon Kinesis source endpoint. For more information, see the <code>KinesisSettings</code> structure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MongoDbSettings</i> The settings for the MongoDB source endpoint. For more information, see the <code>MongoDbSettings</code> structure.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>MongoDbSettings</b>: The settings for the MongoDB source endpoint. For more information, see the <code>MongoDbSettings</code> structure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DmsTransferSettings</i> The settings in JSON format for the DMS transfer type of source endpoint.  Possible attributes include the following: <ul> <li>  <code>serviceAccessRoleArn</code> - The IAM role that has permission to access the Amazon S3 bucket. </li> <li>  <code>bucketName</code> - The name of the S3 bucket to use. </li> <li>  <code>compressionType</code> - An optional parameter to use GZIP to compress the target files. To use GZIP, set this value to <code>NONE</code> (the default). To keep the files uncompressed, don't use this value. </li> </ul> Shorthand syntax for these attributes is as follows: <code>ServiceAccessRoleArn=string,BucketName=string,CompressionType=string</code>  JSON syntax for these attributes is as follows: <code>{ "ServiceAccessRoleArn": "string", "BucketName": "string", "CompressionType": "none"|"gzip" } </code> &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DmsTransferSettings</b>: The settings in JSON format for the DMS transfer type of source endpoint.  Possible attributes include the following: <ul> <li>  <code>serviceAccessRoleArn</code> - The IAM role that has permission to access the Amazon S3 bucket. </li> <li>  <code>bucketName</code> - The name of the S3 bucket to use. </li> <li>  <code>compressionType</code> - An optional parameter to use GZIP to compress the target files. To use GZIP, set this value to <code>NONE</code> (the default). To keep the files uncompressed, don't use this value. </li> </ul> Shorthand syntax for these attributes is as follows: <code>ServiceAccessRoleArn=string,BucketName=string,CompressionType=string</code>  JSON syntax for these attributes is as follows: <code>{ "ServiceAccessRoleArn": "string", "BucketName": "string", "CompressionType": "none"|"gzip" } </code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Settings</i> The settings for the S3 target endpoint. For more information, see the <code>S3Settings</code> structure.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Settings</b>: The settings for the S3 target endpoint. For more information, see the <code>S3Settings</code> structure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DynamoDbSettings</i> The settings for the target DynamoDB database. For more information, see the <code>DynamoDBSettings</code> structure.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DynamoDbSettings</b>: The settings for the target DynamoDB database. For more information, see the <code>DynamoDBSettings</code> structure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalId</i>  Value returned by a call to CreateEndpoint that can be used for cross-account validation. Use it on a subsequent call to CreateEndpoint to create the endpoint with a cross-account. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalTableDefinition</i> The external table definition.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i> The Amazon Resource Name (ARN) used by the service access IAM role.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SslMode</i> The SSL mode used to connect to the endpoint. The default value is <code>none</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateArn</i> The Amazon Resource Name (ARN) used for SSL connection to the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointArn</i> The Amazon Resource Name (ARN) string that uniquely identifies the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> An AWS KMS key identifier that is used to encrypt the connection parameters for the endpoint. If you don't specify a value for the <code>KmsKeyId</code> parameter, then AWS DMS uses your default encryption key. AWS KMS creates the default encryption key for your AWS account. Your AWS account has a different default encryption key for each AWS Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExtraConnectionAttributes</i> Additional connection attributes used to connect to the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> The name of the database at the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port value used to access the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerName</i> The name of the server at the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> The user name used to connect to the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineDisplayName</i> The expanded name for the engine name. For example, if the <code>EngineName</code> parameter is "aurora," this value would be "Amazon Aurora MySQL."<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineName</i> The database engine name. Valid values, depending on the EndpointType, include mysql, oracle, postgres, mariadb, aurora, aurora-postgresql, redshift, s3, db2, azuredb, sybase, dynamodb, mongodb, and sqlserver.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointType</i> The type of endpoint. Valid values are <code>source</code> and <code>target</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointIdentifier</i> The database endpoint identifier. Identifiers must begin with a letter; must contain only ASCII letters, digits, and hyphens; and must not end with a hyphen or contain two consecutive hyphens.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i>  The Amazon Resource Name (ARN) used by the service access IAM role. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TimestampColumnName</i> A value that includes a timestamp column in the Amazon S3 target endpoint data. AWS DMS includes an additional column in the migrated data when you set <code>timestampColumnName</code> to a non-blank value.  <note> AWS DMS supports <code>TimestampColumnName</code> in versions 3.1.4 and later. </note> For a full load, each row of the timestamp column contains a timestamp for when the data was transferred from the source to the target by DMS. For a CDC load, each row of the timestamp column contains the timestamp for the commit of that row in the source database. The format for the timestamp column value is <code>yyyy-MM-dd HH:mm:ss.SSSSSS</code>. For CDC, the microsecond precision depends on the commit timestamp supported by DMS for the source database. When the <code>AddColumnName</code> setting is set to <code>true</code>, DMS also includes the name for the timestamp column that you set as the nonblank value of <code>timestampColumnName</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CdcInsertsOnly</i> A value that enables a change data capture (CDC) load to write only INSERT operations to .csv or columnar storage (.parquet) output files. By default (the <code>false</code> setting), the first field in a .csv or .parquet record contains the letter I (INSERT), U (UPDATE), or D (DELETE). These values indicate whether the row was inserted, updated, or deleted at the source database for a CDC load to the target. If <code>cdcInsertsOnly</code> is set to <code>true</code> or <code>y</code>, only INSERTs from the source database are migrated to the .csv or .parquet file. For .csv format only, how these INSERTs are recorded depends on the value of <code>IncludeOpForFullLoad</code>. If <code>IncludeOpForFullLoad</code> is set to <code>true</code>, the first field of every CDC record is set to I to indicate the INSERT operation at the source. If <code>IncludeOpForFullLoad</code> is set to <code>false</code>, every CDC record is written without a first field to indicate the INSERT operation at the source. For more information about how these settings work together, see <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Target.S3.html#CHAP_Target.S3.Configuring.InsertOps">Indicating Source DB Operations in Migrated S3 Data</a> in the <i>AWS Database Migration Service User Guide.</i>. <note> AWS DMS supports this interaction between <code>CdcInsertsOnly</code> and <code>IncludeOpForFullLoad</code> in versions 3.1.4 and later.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeOpForFullLoad</i> A value that enables a full load to write INSERT operations to the comma-separated value (.csv) output files only to indicate how the rows were added to the source database. <note> AWS DMS supports <code>IncludeOpForFullLoad</code> in versions 3.1.4 and later. </note> For full load, records can only be inserted. By default (the <code>false</code> setting), no information is recorded in these output files for a full load to indicate that the rows were inserted at the source database. If <code>IncludeOpForFullLoad</code> is set to <code>true</code> or <code>y</code>, the INSERT is recorded as an I annotation in the first field of the .csv file. This allows the format of your target records from a full load to be consistent with the target records from a CDC load. <note> This setting works together with <code>CdcInsertsOnly</code> for output to .csv files only. For more information about how these settings work together, see <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Target.S3.html#CHAP_Target.S3.Configuring.InsertOps">Indicating Source DB Operations in Migrated S3 Data</a> in the <i>AWS Database Migration Service User Guide.</i>. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EnableStatistics</i> A value that enables statistics for Parquet pages and row groups. Choose <code>true</code> to enable statistics, <code>false</code> to disable. Statistics include <code>NULL</code>, <code>DISTINCT</code>, <code>MAX</code>, and <code>MIN</code> values. This parameter defaults to <code>true</code>. This value is used for .parquet file format only.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ParquetVersion</i> The version of the Apache Parquet format that you want to use: <code>parquet_1_0</code> (the default) or <code>parquet_2_0</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DataPageSize</i> The size of one data page in bytes. This parameter defaults to 1024 * 1024 bytes (1 MiB). This number is used for .parquet file format only. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RowGroupLength</i> The number of rows in a row group. A smaller row group size provides faster reads. But as the number of row groups grows, the slower writes become. This parameter defaults to 10,000 rows. This number is used for .parquet file format only.  If you choose a value larger than the maximum, <code>RowGroupLength</code> is set to the max row group length in bytes (64 * 1024 * 1024). <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DictPageSizeLimit</i> The maximum size of an encoded dictionary page of a column. If the dictionary page exceeds this, this column is stored using an encoding type of <code>PLAIN</code>. This parameter defaults to 1024 * 1024 bytes (1 MiB), the maximum size of a dictionary page before it reverts to <code>PLAIN</code> encoding. This size is used for .parquet file format only. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncodingType</i> The type of encoding you are using:  <ul> <li>  <code>RLE_DICTIONARY</code> uses a combination of bit-packing and run-length encoding to store repeated values more efficiently. This is the default. </li> <li>  <code>PLAIN</code> doesn't use encoding at all. Values are stored as they are. </li> <li>  <code>PLAIN_DICTIONARY</code> builds a dictionary of the values encountered in a given column. The dictionary is stored in a dictionary page for each column chunk. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DataFormat</i> The format of the data that you want to use for output. You can choose one of the following:  <ul> <li>  <code>csv</code> : This is a row-based file format with comma-separated values (.csv).  </li> <li>  <code>parquet</code> : Apache Parquet (.parquet) is a columnar storage file format that features efficient compression and provides faster query response.  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerSideEncryptionKmsKeyId</i> If you are using <code>SSE_KMS</code> for the <code>EncryptionMode</code>, provide the AWS KMS key ID. The key that you use needs an attached policy that enables AWS Identity and Access Management (IAM) user permissions and allows use of the key. Here is a CLI example: <code>aws dms create-endpoint --endpoint-identifier <i>value</i> --endpoint-type target --engine-name s3 --s3-settings ServiceAccessRoleArn=<i>value</i>,BucketFolder=<i>value</i>,BucketName=<i>value</i>,EncryptionMode=SSE_KMS,ServerSideEncryptionKmsKeyId=<i>value</i> </code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionMode</i> The type of server-side encryption that you want to use for your data. This encryption type is part of the endpoint settings or the extra connections attributes for Amazon S3. You can choose either <code>SSE_S3</code> (the default) or <code>SSE_KMS</code>. To use <code>SSE_S3</code>, you need an AWS Identity and Access Management (IAM) role with permission to allow <code>"arn:aws:s3:::dms-*"</code> to use the following actions: <ul> <li>  <code>s3:CreateBucket</code>  </li> <li>  <code>s3:ListBucket</code>  </li> <li>  <code>s3:DeleteBucket</code>  </li> <li>  <code>s3:GetBucketLocation</code>  </li> <li>  <code>s3:GetObject</code>  </li> <li>  <code>s3:PutObject</code>  </li> <li>  <code>s3:DeleteObject</code>  </li> <li>  <code>s3:GetObjectVersion</code>  </li> <li>  <code>s3:GetBucketPolicy</code>  </li> <li>  <code>s3:PutBucketPolicy</code>  </li> <li>  <code>s3:DeleteBucketPolicy</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CompressionType</i>  An optional parameter to use GZIP to compress the target files. Set to GZIP to compress the target files. Set to NONE (the default) or do not use to leave the files uncompressed. Applies to both .csv and .parquet file formats. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketName</i>  The name of the S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketFolder</i>  An optional parameter to set a folder name in the S3 bucket. If provided, tables are created in the path <code> <i>bucketFolder</i>/<i>schema_name</i>/<i>table_name</i>/</code>. If this parameter is not specified, then the path used is <code> <i>schema_name</i>/<i>table_name</i>/</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CsvDelimiter</i>  The delimiter used to separate columns in the source files. The default is a comma. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CsvRowDelimiter</i>  The delimiter used to separate rows in the source files. The default is a carriage return (<code>\n</code>). <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalTableDefinition</i>  The external table definition. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i>  The Amazon Resource Name (ARN) used by the service access IAM role. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketName</i>  The name of the S3 bucket to use. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i>  The IAM role that has permission to access the Amazon S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS KMS key identifier that is used to encrypt the content on the replication instance. If you don't specify a value for the <code>KmsKeyId</code> parameter, then AWS DMS uses your default encryption key. AWS KMS creates the default encryption key for your AWS account. Your AWS account has a different default encryption key for each AWS Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthSource</i>  The MongoDB database name. This attribute is not used when <code>authType=NO</code>.  The default is admin.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DocsToInvestigate</i>  Indicates the number of documents to preview to determine the document organization. Use this attribute when <code>NestingLevel</code> is set to ONE.  Must be a positive value greater than 0. Default value is 1000.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExtractDocId</i>  Specifies the document ID. Use this attribute when <code>NestingLevel</code> is set to NONE.  Default value is false. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NestingLevel</i>  Specifies either document or table mode.  Valid values: NONE, ONE Default value is NONE. Specify NONE to use document mode. Specify ONE to use table mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthMechanism</i>  The authentication mechanism you use to access the MongoDB source endpoint. Valid values: DEFAULT, MONGODB_CR, SCRAM_SHA_1  DEFAULT – For MongoDB version 2.x, use MONGODB_CR. For MongoDB version 3.x, use SCRAM_SHA_1. This attribute is not used when authType=No.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthType</i>  The authentication type you use to access the MongoDB source endpoint. Valid values: NO, PASSWORD  When NO is selected, user name and password parameters are not used and can be empty. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i>  The database name on the MongoDB source endpoint. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i>  The port value for the MongoDB source endpoint. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerName</i>  The name of the server on the MongoDB source endpoint. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Password</i>  The password for the user account you use to access the MongoDB source endpoint. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> The user name you use to access the MongoDB source endpoint. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i> The Amazon Resource Name (ARN) for the IAM role that DMS uses to write to the Amazon Kinesis data stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MessageFormat</i> The output format for the records created on the endpoint. The message format is <code>JSON</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamArn</i> The Amazon Resource Name (ARN) for the Amazon Kinesis Data Streams endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorRetryDuration</i> The maximum number of seconds that DMS retries failed API requests to the Elasticsearch cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FullLoadErrorPercentage</i> The maximum percentage of records that can fail to be written before a full load operation stops. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointUri</i> The endpoint for the Elasticsearch cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i> The Amazon Resource Name (ARN) used by service to access the IAM role.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WriteBufferSize</i> The size of the write buffer to use in rows. Valid values range from 1 through 2,048. The default is 1,024. Use this setting to tune performance. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> An Amazon Redshift user name for a registered user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TruncateColumns</i> A value that specifies to truncate data in columns to the appropriate number of characters, so that the data fits in the column. This parameter applies only to columns with a VARCHAR or CHAR data type, and rows with a size of 4 MB or less. Choose <code>true</code> to truncate data. The default is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TrimBlanks</i> A value that specifies to remove the trailing white space characters from a VARCHAR string. This parameter applies only to columns with a VARCHAR data type. Choose <code>true</code> to remove unneeded white space. The default is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeFormat</i> The time format that you want to use. Valid values are <code>auto</code> (case-sensitive), <code>'timeformat_string'</code>, <code>'epochsecs'</code>, or <code>'epochmillisecs'</code>. It defaults to 10. Using <code>auto</code> recognizes most strings, even some that aren't supported when you use a time format string.  If your date and time values use formats different from each other, set this parameter to <code>auto</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerSideEncryptionKmsKeyId</i> The AWS KMS key ID. If you are using <code>SSE_KMS</code> for the <code>EncryptionMode</code>, provide this key ID. The key that you use needs an attached policy that enables IAM user permissions and allows use of the key.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessRoleArn</i> The Amazon Resource Name (ARN) of the IAM role that has access to the Amazon Redshift service.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerName</i> The name of the Amazon Redshift cluster you are using.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplaceChars</i> A value that specifies to replaces the invalid characters specified in <code>ReplaceInvalidChars</code>, substituting the specified characters instead. The default is <code>"?"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplaceInvalidChars</i> A list of characters that you want to replace. Use with <code>ReplaceChars</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RemoveQuotes</i> A value that specifies to remove surrounding quotation marks from strings in the incoming data. All characters within the quotation marks, including delimiters, are retained. Choose <code>true</code> to remove quotation marks. The default is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port number for Amazon Redshift. The default value is 5439.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Password</i> The password for the user named in the <code>username</code> property.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxFileSize</i> The maximum size (in KB) of any .csv file used to transfer data to Amazon Redshift. This accepts a value from 1 through 1,048,576. It defaults to 32,768 KB (32 MB).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LoadTimeout</i> The amount of time to wait (in milliseconds) before timing out, beginning from when you begin loading.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FileTransferUploadStreams</i> The number of threads used to upload a single file. This parameter accepts a value from 1 through 64. It defaults to 10.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionMode</i> The type of server-side encryption that you want to use for your data. This encryption type is part of the endpoint settings or the extra connections attributes for Amazon S3. You can choose either <code>SSE_S3</code> (the default) or <code>SSE_KMS</code>. To use <code>SSE_S3</code>, create an AWS Identity and Access Management (IAM) role with a policy that allows <code>"arn:aws:s3:::*"</code> to use the following actions: <code>"s3:PutObject", "s3:ListBucket"</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EmptyAsNull</i> A value that specifies whether AWS DMS should migrate empty CHAR and VARCHAR fields as NULL. A value of <code>true</code> sets empty CHAR and VARCHAR fields to null. The default is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DateFormat</i> The date format that you are using. Valid values are <code>auto</code> (case-sensitive), your date format string enclosed in quotes, or NULL. If this parameter is left unset (NULL), it defaults to a format of 'YYYY-MM-DD'. Using <code>auto</code> recognizes most strings, even some that aren't supported when you use a date format string.  If your date and time values use formats different from each other, set this to <code>auto</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> The name of the Amazon Redshift data warehouse (service) that you are working with.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectionTimeout</i> A value that sets the amount of time to wait (in milliseconds) before timing out, beginning from when you initially establish a connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketName</i> The name of the S3 bucket you want to use<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketFolder</i> The location where the comma-separated value (.csv) files are stored before being uploaded to the S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AfterConnectScript</i> Code to run after connecting. This parameter should contain the code itself, not the name of a file containing the code.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AcceptAnyDate</i> A value that indicates to allow any date format, including invalid formats such as 00/00/00 00:00:00, to be loaded without generating an error. You can choose <code>true</code> or <code>false</code> (the default). This parameter applies only to TIMESTAMP and DATE columns. Always use ACCEPTANYDATE with the DATEFORMAT parameter. If the date format for the data doesn't match the DATEFORMAT specification, Amazon Redshift inserts a NULL value into that field. <br/>        """
        pass

    def modify_event_subscription(self, SubscriptionName: str, SnsTopicArn: str=None, SourceType: str=None, EventCategories: List[String]=None, Enabled: BooleanOptional=None) -> ModifyEventSubscriptionResponse:
        """Modifies an existing AWS DMS event notification subscription. 

        :param self:
        :param Enabled:   A Boolean value; set to <b>true</b> to activate the subscription. <br/>EventCategories:   A list of event categories for a source type that you want to subscribe to. Use the <code>DescribeEventCategories</code> action to see a list of event categories. <br/>SourceType:   The type of AWS DMS resource that generates the events you want to subscribe to.  Valid values: replication-instance | replication-task<br/>SnsTopicArn:   The Amazon Resource Name (ARN) of the Amazon SNS topic created for event notification. The ARN is created by Amazon SNS when you create a topic and subscribe to it.<br/>SubscriptionName:  The name of the AWS DMS event notification subscription to be modified.<br/>
        :return: EventSubscription:  The modified event subscription.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Boolean value that indicates if the event subscription is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EventCategoriesList</i> A lists of event categories.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceIdsList</i> A list of source Ids for the event subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceType</i>  The type of AWS DMS resource that generates events.  Valid values: replication-instance | replication-server | security-group | replication-task<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubscriptionCreationTime</i> The time the RDS event notification subscription was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the AWS DMS event notification subscription. Constraints: Can be one of the following: creating | modifying | deleting | active | no-permission | topic-not-exist The status "no-permission" indicates that AWS DMS no longer has permission to post to the SNS topic. The status "topic-not-exist" indicates that the topic was deleted after the subscription was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsTopicArn</i> The topic ARN of the AWS DMS event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustSubscriptionId</i> The AWS DMS event notification subscription Id.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomerAwsId</i> The AWS customer account associated with the AWS DMS event notification subscription.<br/>        """
        pass

    def modify_replication_instance(self, ReplicationInstanceArn: str, AllocatedStorage: int=None, ApplyImmediately: Boolean=None, ReplicationInstanceClass: str=None, VpcSecurityGroupIds: List[String]=None, PreferredMaintenanceWindow: str=None, MultiAZ: BooleanOptional=None, EngineVersion: str=None, AllowMajorVersionUpgrade: Boolean=None, AutoMinorVersionUpgrade: BooleanOptional=None, ReplicationInstanceIdentifier: str=None) -> ModifyReplicationInstanceResponse:
        """Modifies the replication instance to apply new settings. You can change one or more parameters by specifying these parameters and the new values in the request. Some settings are applied during the maintenance window. <p/>

        :param self:
        :param ReplicationInstanceIdentifier:  The replication instance identifier. This parameter is stored as a lowercase string.<br/>AutoMinorVersionUpgrade:   Indicates that minor version upgrades will be applied automatically to the replication instance during the maintenance window. Changing this parameter does not result in an outage except in the following case and the change is asynchronously applied as soon as possible. An outage will result if this parameter is set to <code>true</code> during the maintenance window, and a newer minor version is available, and AWS DMS has enabled auto patching for that engine version. <br/>AllowMajorVersionUpgrade:  Indicates that major version upgrades are allowed. Changing this parameter does not result in an outage, and the change is asynchronously applied as soon as possible. This parameter must be set to <code>true</code> when specifying a value for the <code>EngineVersion</code> parameter that is a different major version than the replication instance's current version.<br/>EngineVersion:  The engine version number of the replication instance.<br/>MultiAZ:   Specifies whether the replication instance is a Multi-AZ deployment. You cannot set the <code>AvailabilityZone</code> parameter if the Multi-AZ parameter is set to <code>true</code>. <br/>PreferredMaintenanceWindow:  The weekly time range (in UTC) during which system maintenance can occur, which might result in an outage. Changing this parameter does not result in an outage, except in the following situation, and the change is asynchronously applied as soon as possible. If moving this window to the current time, there must be at least 30 minutes between the current time and end of the window to ensure pending changes are applied. Default: Uses existing setting Format: ddd:hh24:mi-ddd:hh24:mi Valid Days: Mon | Tue | Wed | Thu | Fri | Sat | Sun Constraints: Must be at least 30 minutes<br/>VpcSecurityGroupIds:   Specifies the VPC security group to be used with the replication instance. The VPC security group must work with the VPC containing the replication instance. <br/>ReplicationInstanceClass:  The compute and memory capacity of the replication instance.  Valid Values: <code>dms.t2.micro | dms.t2.small | dms.t2.medium | dms.t2.large | dms.c4.large | dms.c4.xlarge | dms.c4.2xlarge | dms.c4.4xlarge </code> <br/>ApplyImmediately:  Indicates whether the changes should be applied immediately or during the next maintenance window.<br/>AllocatedStorage:  The amount of storage (in gigabytes) to be allocated for the replication instance.<br/>ReplicationInstanceArn:  The Amazon Resource Name (ARN) of the replication instance.<br/>
        :return: ReplicationInstance:  The modified replication instance.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DnsNameServers</i> The DNS name servers for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FreeUntil</i>  The expiration date of the free replication instance that is part of the Free DMS program. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondaryAvailabilityZone</i> The availability zone of the standby replication instance in a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i>  Specifies the accessibility options for the replication instance. A value of <code>true</code> represents an instance with a public IP address. A value of <code>false</code> represents an instance with a private IP address. The default value is <code>true</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstancePrivateIpAddresses</i> One or more private IP addresses for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstancePublicIpAddresses</i> One or more public IP addresses for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstancePrivateIpAddress</i> The private IP address of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstancePublicIpAddress</i> The public IP address of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceArn</i> The Amazon Resource Name (ARN) of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> An AWS KMS key identifier that is used to encrypt the data on the replication instance. If you don't specify a value for the <code>KmsKeyId</code> parameter, then AWS DMS uses your default encryption key. AWS KMS creates the default encryption key for your AWS account. Your AWS account has a different default encryption key for each AWS Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> Boolean value indicating if minor version upgrades will be automatically applied to the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> The engine version number of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i>  Specifies whether the replication instance is a Multi-AZ deployment. You cannot set the <code>AvailabilityZone</code> parameter if the Multi-AZ parameter is set to <code>true</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> The pending modification values.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: The pending modification values.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> The maintenance window times for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSubnetGroup</i> The subnet group for the replication instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ReplicationSubnetGroup</b>: The subnet group for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> The VPC security group for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCreateTime</i> The time the replication instance was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> The amount of storage (in gigabytes) that is allocated for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceStatus</i> The status of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceClass</i> The compute and memory capacity of the replication instance.  Valid Values: <code>dms.t2.micro | dms.t2.small | dms.t2.medium | dms.t2.large | dms.c4.large | dms.c4.xlarge | dms.c4.2xlarge | dms.c4.4xlarge </code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceIdentifier</i> The replication instance identifier. This parameter is stored as a lowercase string. Constraints: <ul> <li> Must contain from 1 to 63 alphanumeric characters or hyphens. </li> <li> First character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul> Example: <code>myrepinstance</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> The subnets that are in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> The status of the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSubnetGroupDescription</i> A description for the replication subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSubnetGroupIdentifier</i> The identifier of the replication instance subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> The engine version number of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i>  Specifies whether the replication instance is a Multi-AZ deployment. You cannot set the <code>AvailabilityZone</code> parameter if the Multi-AZ parameter is set to <code>true</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> The amount of storage (in gigabytes) that is allocated for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceClass</i> The compute and memory capacity of the replication instance.  Valid Values: <code>dms.t2.micro | dms.t2.small | dms.t2.medium | dms.t2.large | dms.c4.large | dms.c4.xlarge | dms.c4.2xlarge | dms.c4.4xlarge </code> <br/>        """
        pass

    def modify_replication_subnet_group(self, SubnetIds: List[String], ReplicationSubnetGroupIdentifier: str, ReplicationSubnetGroupDescription: str=None) -> ModifyReplicationSubnetGroupResponse:
        """Modifies the settings for the specified replication subnet group.

        :param self:
        :param SubnetIds:  A list of subnet IDs.<br/>ReplicationSubnetGroupDescription:  A description for the replication instance subnet group.<br/>ReplicationSubnetGroupIdentifier:  The name of the replication instance subnet group.<br/>
        :return: ReplicationSubnetGroup:  The modified replication subnet group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> The subnets that are in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> The status of the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSubnetGroupDescription</i> A description for the replication subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSubnetGroupIdentifier</i> The identifier of the replication instance subnet group.<br/>        """
        pass

    def modify_replication_task(self, ReplicationTaskArn: str, ReplicationTaskIdentifier: str=None, MigrationType: str=None, TableMappings: str=None, ReplicationTaskSettings: str=None, CdcStartTime: TStamp=None, CdcStartPosition: str=None, CdcStopPosition: str=None) -> ModifyReplicationTaskResponse:
        """Modifies the specified replication task. You can't modify the task endpoints. The task must be stopped before you can modify it.  For more information about AWS DMS tasks, see <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Tasks.html">Working with Migration Tasks</a> in the <i>AWS Database Migration Service User Guide</i>.

        :param self:
        :param CdcStopPosition:  Indicates when you want a change data capture (CDC) operation to stop. The value can be either server time or commit time. Server time example: --cdc-stop-position “server_time:3018-02-09T12:12:12” Commit time example: --cdc-stop-position “commit_time: 3018-02-09T12:12:12 “<br/>CdcStartPosition:  Indicates when you want a change data capture (CDC) operation to start. Use either CdcStartPosition or CdcStartTime to specify when you want a CDC operation to start. Specifying both values results in an error.  The value can be in date, checkpoint, or LSN/SCN format. Date Example: --cdc-start-position “2018-03-08T12:12:12” Checkpoint Example: --cdc-start-position "checkpoint:V1#27#mysql-bin-changelog.157832:1975:-1:2002:677883278264080:mysql-bin-changelog.157832:1876#0#0#*#0#93" LSN Example: --cdc-start-position “mysql-bin-changelog.000024:373”<br/>CdcStartTime:  Indicates the start time for a change data capture (CDC) operation. Use either CdcStartTime or CdcStartPosition to specify when you want a CDC operation to start. Specifying both values results in an error. Timestamp Example: --cdc-start-time “2018-03-08T12:12:12”<br/>ReplicationTaskSettings:  JSON file that contains settings for the task, such as target metadata settings.<br/>TableMappings:  When using the AWS CLI or boto3, provide the path of the JSON file that contains the table mappings. Precede the path with <code>file://</code>. When working with the DMS API, provide the JSON as the parameter value, for example: <code>--table-mappings file://mappingfile.json</code> <br/>MigrationType:  The migration type. Valid values: <code>full-load</code> | <code>cdc</code> | <code>full-load-and-cdc</code> <br/>ReplicationTaskIdentifier:  The replication task identifier. Constraints: <ul> <li> Must contain from 1 to 255 alphanumeric characters or hyphens. </li> <li> First character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul><br/>ReplicationTaskArn:  The Amazon Resource Name (ARN) of the replication task.<br/>
        :return: ReplicationTask:  The replication task that was modified.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskStats</i> The statistics for the task, including elapsed time, tables loaded, and table errors.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ReplicationTaskStats</b>: The statistics for the task, including elapsed time, tables loaded, and table errors.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskArn</i> The Amazon Resource Name (ARN) of the replication task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecoveryCheckpoint</i> Indicates the last checkpoint that occurred during a change data capture (CDC) operation. You can provide this value to the <code>CdcStartPosition</code> parameter to start a CDC operation that begins at that checkpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CdcStopPosition</i> Indicates when you want a change data capture (CDC) operation to stop. The value can be either server time or commit time. Server time example: --cdc-stop-position “server_time:3018-02-09T12:12:12” Commit time example: --cdc-stop-position “commit_time: 3018-02-09T12:12:12 “<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CdcStartPosition</i> Indicates when you want a change data capture (CDC) operation to start. Use either <code>CdcStartPosition</code> or <code>CdcStartTime</code> to specify when you want the CDC operation to start. Specifying both values results in an error. The value can be in date, checkpoint, or LSN/SCN format. Date Example: --cdc-start-position “2018-03-08T12:12:12” Checkpoint Example: --cdc-start-position "checkpoint:V1#27#mysql-bin-changelog.157832:1975:-1:2002:677883278264080:mysql-bin-changelog.157832:1876#0#0#*#0#93" LSN Example: --cdc-start-position “mysql-bin-changelog.000024:373”<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskStartDate</i> The date the replication task is scheduled to start.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskCreationDate</i> The date the replication task was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StopReason</i> The reason the replication task was stopped.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastFailureMessage</i> The last error (failure) message generated for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the replication task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskSettings</i> The settings for the replication task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableMappings</i> Table mappings specified in the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MigrationType</i> The type of migration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceArn</i> The Amazon Resource Name (ARN) of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetEndpointArn</i> The Amazon Resource Name (ARN) string that uniquely identifies the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceEndpointArn</i> The Amazon Resource Name (ARN) string that uniquely identifies the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskIdentifier</i> The user-assigned replication task identifier or name. Constraints: <ul> <li> Must contain from 1 to 255 alphanumeric characters or hyphens. </li> <li> First character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TablesErrored</i> The number of errors that have occurred during this task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TablesQueued</i> The number of tables queued for this task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TablesLoading</i> The number of tables currently loading for this task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TablesLoaded</i> The number of tables loaded for this task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeMillis</i> The elapsed time of the task, in milliseconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FullLoadProgressPercent</i> The percent complete for the full load migration task.<br/>        """
        pass

    def reboot_replication_instance(self, ReplicationInstanceArn: str, ForceFailover: BooleanOptional=None) -> RebootReplicationInstanceResponse:
        """Reboots a replication instance. Rebooting results in a momentary outage, until the replication instance becomes available again.

        :param self:
        :param ForceFailover:  If this parameter is <code>true</code>, the reboot is conducted through a Multi-AZ failover. (If the instance isn't configured for Multi-AZ, then you can't specify <code>true</code>.)<br/>ReplicationInstanceArn:  The Amazon Resource Name (ARN) of the replication instance.<br/>
        :return: ReplicationInstance:  The replication instance that is being rebooted. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DnsNameServers</i> The DNS name servers for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FreeUntil</i>  The expiration date of the free replication instance that is part of the Free DMS program. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondaryAvailabilityZone</i> The availability zone of the standby replication instance in a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i>  Specifies the accessibility options for the replication instance. A value of <code>true</code> represents an instance with a public IP address. A value of <code>false</code> represents an instance with a private IP address. The default value is <code>true</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstancePrivateIpAddresses</i> One or more private IP addresses for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstancePublicIpAddresses</i> One or more public IP addresses for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstancePrivateIpAddress</i> The private IP address of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstancePublicIpAddress</i> The public IP address of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceArn</i> The Amazon Resource Name (ARN) of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> An AWS KMS key identifier that is used to encrypt the data on the replication instance. If you don't specify a value for the <code>KmsKeyId</code> parameter, then AWS DMS uses your default encryption key. AWS KMS creates the default encryption key for your AWS account. Your AWS account has a different default encryption key for each AWS Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> Boolean value indicating if minor version upgrades will be automatically applied to the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> The engine version number of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i>  Specifies whether the replication instance is a Multi-AZ deployment. You cannot set the <code>AvailabilityZone</code> parameter if the Multi-AZ parameter is set to <code>true</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> The pending modification values.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: The pending modification values.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> The maintenance window times for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSubnetGroup</i> The subnet group for the replication instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ReplicationSubnetGroup</b>: The subnet group for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> The VPC security group for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCreateTime</i> The time the replication instance was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> The amount of storage (in gigabytes) that is allocated for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceStatus</i> The status of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceClass</i> The compute and memory capacity of the replication instance.  Valid Values: <code>dms.t2.micro | dms.t2.small | dms.t2.medium | dms.t2.large | dms.c4.large | dms.c4.xlarge | dms.c4.2xlarge | dms.c4.4xlarge </code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceIdentifier</i> The replication instance identifier. This parameter is stored as a lowercase string. Constraints: <ul> <li> Must contain from 1 to 63 alphanumeric characters or hyphens. </li> <li> First character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul> Example: <code>myrepinstance</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> The subnets that are in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> The status of the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSubnetGroupDescription</i> A description for the replication subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSubnetGroupIdentifier</i> The identifier of the replication instance subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> The engine version number of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i>  Specifies whether the replication instance is a Multi-AZ deployment. You cannot set the <code>AvailabilityZone</code> parameter if the Multi-AZ parameter is set to <code>true</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> The amount of storage (in gigabytes) that is allocated for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceClass</i> The compute and memory capacity of the replication instance.  Valid Values: <code>dms.t2.micro | dms.t2.small | dms.t2.medium | dms.t2.large | dms.c4.large | dms.c4.xlarge | dms.c4.2xlarge | dms.c4.4xlarge </code> <br/>        """
        pass

    def refresh_schemas(self, ReplicationInstanceArn: str, EndpointArn: str) -> RefreshSchemasResponse:
        """Populates the schema for the specified endpoint. This is an asynchronous operation and can take several minutes. You can check the status of this operation by calling the DescribeRefreshSchemasStatus operation.

        :param self:
        :param ReplicationInstanceArn:  The Amazon Resource Name (ARN) of the replication instance.<br/>EndpointArn:  The Amazon Resource Name (ARN) string that uniquely identifies the endpoint.<br/>
        :return: RefreshSchemasStatus:  The status of the refreshed schema.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastFailureMessage</i> The last failure message for the schema.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastRefreshDate</i> The date the schema was last refreshed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the schema.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceArn</i> The Amazon Resource Name (ARN) of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointArn</i> The Amazon Resource Name (ARN) string that uniquely identifies the endpoint.<br/>        """
        pass

    def reload_tables(self, TablesToReload: List[TableToReload], ReplicationTaskArn: str, ReloadOption: str=None) -> ReloadTablesResponse:
        """Reloads the target database table with the source data. 

        :param self:
        :param ReloadOption:  Options for reload. Specify <code>data-reload</code> to reload the data and re-validate it if validation is enabled. Specify <code>validate-only</code> to re-validate the table. This option applies only when validation is enabled for the task.  Valid values: data-reload, validate-only Default value is data-reload.<br/>TablesToReload:  The name and schema of the table to be reloaded. <br/>ReplicationTaskArn:  The Amazon Resource Name (ARN) of the replication task. <br/>
        :return: ReplicationTaskArn:  The Amazon Resource Name (ARN) of the replication task. <br/>        """
        pass

    def remove_tags_from_resource(self, TagKeys: List[String], ResourceArn: str) -> RemoveTagsFromResourceResponse:
        """Removes metadata tags from a DMS resource.

        :param self:
        :param TagKeys:  The tag key (name) of the tag to be removed.<br/>ResourceArn:  An AWS DMS resource from which you want to remove tag(s). The value for this parameter is an Amazon Resource Name (ARN).<br/>
        :return:         """
        pass

    def start_replication_task(self, StartReplicationTaskType: str, ReplicationTaskArn: str, CdcStartTime: TStamp=None, CdcStartPosition: str=None, CdcStopPosition: str=None) -> StartReplicationTaskResponse:
        """Starts the replication task. For more information about AWS DMS tasks, see <a href="https://docs.aws.amazon.com/dms/latest/userguide/CHAP_Tasks.html">Working with Migration Tasks </a> in the <i>AWS Database Migration Service User Guide.</i> 

        :param self:
        :param CdcStopPosition:  Indicates when you want a change data capture (CDC) operation to stop. The value can be either server time or commit time. Server time example: --cdc-stop-position “server_time:3018-02-09T12:12:12” Commit time example: --cdc-stop-position “commit_time: 3018-02-09T12:12:12 “<br/>CdcStartPosition:  Indicates when you want a change data capture (CDC) operation to start. Use either CdcStartPosition or CdcStartTime to specify when you want a CDC operation to start. Specifying both values results in an error.  The value can be in date, checkpoint, or LSN/SCN format. Date Example: --cdc-start-position “2018-03-08T12:12:12” Checkpoint Example: --cdc-start-position "checkpoint:V1#27#mysql-bin-changelog.157832:1975:-1:2002:677883278264080:mysql-bin-changelog.157832:1876#0#0#*#0#93" LSN Example: --cdc-start-position “mysql-bin-changelog.000024:373”<br/>CdcStartTime:  Indicates the start time for a change data capture (CDC) operation. Use either CdcStartTime or CdcStartPosition to specify when you want a CDC operation to start. Specifying both values results in an error. Timestamp Example: --cdc-start-time “2018-03-08T12:12:12”<br/>StartReplicationTaskType:  The type of replication task.<br/>ReplicationTaskArn:  The Amazon Resource Name (ARN) of the replication task to be started.<br/>
        :return: ReplicationTask:  The replication task started.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskStats</i> The statistics for the task, including elapsed time, tables loaded, and table errors.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ReplicationTaskStats</b>: The statistics for the task, including elapsed time, tables loaded, and table errors.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskArn</i> The Amazon Resource Name (ARN) of the replication task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecoveryCheckpoint</i> Indicates the last checkpoint that occurred during a change data capture (CDC) operation. You can provide this value to the <code>CdcStartPosition</code> parameter to start a CDC operation that begins at that checkpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CdcStopPosition</i> Indicates when you want a change data capture (CDC) operation to stop. The value can be either server time or commit time. Server time example: --cdc-stop-position “server_time:3018-02-09T12:12:12” Commit time example: --cdc-stop-position “commit_time: 3018-02-09T12:12:12 “<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CdcStartPosition</i> Indicates when you want a change data capture (CDC) operation to start. Use either <code>CdcStartPosition</code> or <code>CdcStartTime</code> to specify when you want the CDC operation to start. Specifying both values results in an error. The value can be in date, checkpoint, or LSN/SCN format. Date Example: --cdc-start-position “2018-03-08T12:12:12” Checkpoint Example: --cdc-start-position "checkpoint:V1#27#mysql-bin-changelog.157832:1975:-1:2002:677883278264080:mysql-bin-changelog.157832:1876#0#0#*#0#93" LSN Example: --cdc-start-position “mysql-bin-changelog.000024:373”<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskStartDate</i> The date the replication task is scheduled to start.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskCreationDate</i> The date the replication task was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StopReason</i> The reason the replication task was stopped.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastFailureMessage</i> The last error (failure) message generated for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the replication task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskSettings</i> The settings for the replication task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableMappings</i> Table mappings specified in the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MigrationType</i> The type of migration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceArn</i> The Amazon Resource Name (ARN) of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetEndpointArn</i> The Amazon Resource Name (ARN) string that uniquely identifies the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceEndpointArn</i> The Amazon Resource Name (ARN) string that uniquely identifies the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskIdentifier</i> The user-assigned replication task identifier or name. Constraints: <ul> <li> Must contain from 1 to 255 alphanumeric characters or hyphens. </li> <li> First character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TablesErrored</i> The number of errors that have occurred during this task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TablesQueued</i> The number of tables queued for this task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TablesLoading</i> The number of tables currently loading for this task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TablesLoaded</i> The number of tables loaded for this task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeMillis</i> The elapsed time of the task, in milliseconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FullLoadProgressPercent</i> The percent complete for the full load migration task.<br/>        """
        pass

    def start_replication_task_assessment(self, ReplicationTaskArn: str) -> StartReplicationTaskAssessmentResponse:
        """ Starts the replication task assessment for unsupported data types in the source database. 

        :param self:
        :param ReplicationTaskArn:   The Amazon Resource Name (ARN) of the replication task. <br/>
        :return: ReplicationTask:   The assessed replication task. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskStats</i> The statistics for the task, including elapsed time, tables loaded, and table errors.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ReplicationTaskStats</b>: The statistics for the task, including elapsed time, tables loaded, and table errors.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskArn</i> The Amazon Resource Name (ARN) of the replication task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecoveryCheckpoint</i> Indicates the last checkpoint that occurred during a change data capture (CDC) operation. You can provide this value to the <code>CdcStartPosition</code> parameter to start a CDC operation that begins at that checkpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CdcStopPosition</i> Indicates when you want a change data capture (CDC) operation to stop. The value can be either server time or commit time. Server time example: --cdc-stop-position “server_time:3018-02-09T12:12:12” Commit time example: --cdc-stop-position “commit_time: 3018-02-09T12:12:12 “<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CdcStartPosition</i> Indicates when you want a change data capture (CDC) operation to start. Use either <code>CdcStartPosition</code> or <code>CdcStartTime</code> to specify when you want the CDC operation to start. Specifying both values results in an error. The value can be in date, checkpoint, or LSN/SCN format. Date Example: --cdc-start-position “2018-03-08T12:12:12” Checkpoint Example: --cdc-start-position "checkpoint:V1#27#mysql-bin-changelog.157832:1975:-1:2002:677883278264080:mysql-bin-changelog.157832:1876#0#0#*#0#93" LSN Example: --cdc-start-position “mysql-bin-changelog.000024:373”<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskStartDate</i> The date the replication task is scheduled to start.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskCreationDate</i> The date the replication task was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StopReason</i> The reason the replication task was stopped.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastFailureMessage</i> The last error (failure) message generated for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the replication task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskSettings</i> The settings for the replication task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableMappings</i> Table mappings specified in the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MigrationType</i> The type of migration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceArn</i> The Amazon Resource Name (ARN) of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetEndpointArn</i> The Amazon Resource Name (ARN) string that uniquely identifies the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceEndpointArn</i> The Amazon Resource Name (ARN) string that uniquely identifies the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskIdentifier</i> The user-assigned replication task identifier or name. Constraints: <ul> <li> Must contain from 1 to 255 alphanumeric characters or hyphens. </li> <li> First character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TablesErrored</i> The number of errors that have occurred during this task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TablesQueued</i> The number of tables queued for this task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TablesLoading</i> The number of tables currently loading for this task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TablesLoaded</i> The number of tables loaded for this task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeMillis</i> The elapsed time of the task, in milliseconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FullLoadProgressPercent</i> The percent complete for the full load migration task.<br/>        """
        pass

    def stop_replication_task(self, ReplicationTaskArn: str) -> StopReplicationTaskResponse:
        """Stops the replication task. <p/>

        :param self:
        :param ReplicationTaskArn:  The Amazon Resource Name(ARN) of the replication task to be stopped.<br/>
        :return: ReplicationTask:  The replication task stopped.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskStats</i> The statistics for the task, including elapsed time, tables loaded, and table errors.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ReplicationTaskStats</b>: The statistics for the task, including elapsed time, tables loaded, and table errors.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskArn</i> The Amazon Resource Name (ARN) of the replication task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecoveryCheckpoint</i> Indicates the last checkpoint that occurred during a change data capture (CDC) operation. You can provide this value to the <code>CdcStartPosition</code> parameter to start a CDC operation that begins at that checkpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CdcStopPosition</i> Indicates when you want a change data capture (CDC) operation to stop. The value can be either server time or commit time. Server time example: --cdc-stop-position “server_time:3018-02-09T12:12:12” Commit time example: --cdc-stop-position “commit_time: 3018-02-09T12:12:12 “<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CdcStartPosition</i> Indicates when you want a change data capture (CDC) operation to start. Use either <code>CdcStartPosition</code> or <code>CdcStartTime</code> to specify when you want the CDC operation to start. Specifying both values results in an error. The value can be in date, checkpoint, or LSN/SCN format. Date Example: --cdc-start-position “2018-03-08T12:12:12” Checkpoint Example: --cdc-start-position "checkpoint:V1#27#mysql-bin-changelog.157832:1975:-1:2002:677883278264080:mysql-bin-changelog.157832:1876#0#0#*#0#93" LSN Example: --cdc-start-position “mysql-bin-changelog.000024:373”<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskStartDate</i> The date the replication task is scheduled to start.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskCreationDate</i> The date the replication task was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StopReason</i> The reason the replication task was stopped.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastFailureMessage</i> The last error (failure) message generated for the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the replication task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskSettings</i> The settings for the replication task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableMappings</i> Table mappings specified in the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MigrationType</i> The type of migration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceArn</i> The Amazon Resource Name (ARN) of the replication instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetEndpointArn</i> The Amazon Resource Name (ARN) string that uniquely identifies the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceEndpointArn</i> The Amazon Resource Name (ARN) string that uniquely identifies the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationTaskIdentifier</i> The user-assigned replication task identifier or name. Constraints: <ul> <li> Must contain from 1 to 255 alphanumeric characters or hyphens. </li> <li> First character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TablesErrored</i> The number of errors that have occurred during this task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TablesQueued</i> The number of tables queued for this task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TablesLoading</i> The number of tables currently loading for this task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TablesLoaded</i> The number of tables loaded for this task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeMillis</i> The elapsed time of the task, in milliseconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FullLoadProgressPercent</i> The percent complete for the full load migration task.<br/>        """
        pass

    def test_connection(self, EndpointArn: str, ReplicationInstanceArn: str) -> TestConnectionResponse:
        """Tests the connection between the replication instance and the endpoint.

        :param self:
        :param EndpointArn:  The Amazon Resource Name (ARN) string that uniquely identifies the endpoint.<br/>ReplicationInstanceArn:  The Amazon Resource Name (ARN) of the replication instance.<br/>
        :return: Connection:  The connection tested.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceIdentifier</i> The replication instance identifier. This parameter is stored as a lowercase string.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointIdentifier</i> The identifier of the endpoint. Identifiers must begin with a letter; must contain only ASCII letters, digits, and hyphens; and must not end with a hyphen or contain two consecutive hyphens.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastFailureMessage</i> The error message when the connection last failed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The connection status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointArn</i> The Amazon Resource Name (ARN) string that uniquely identifies the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationInstanceArn</i> The Amazon Resource Name (ARN) of the replication instance.<br/>        """
        pass



class DocDB(BaseClient):
    """Amazon DocumentDB API documentation"""

    class DBInstanceAvailableWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_db_instances` for valid parameters"""
            pass
    class DBInstanceDeletedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_db_instances` for valid parameters"""
            pass


    class DescribeDBClustersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_db_clusters` for valid parameters"""
            pass
    class DescribeDBEngineVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_db_engine_versions` for valid parameters"""
            pass
    class DescribeDBInstancesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_db_instances` for valid parameters"""
            pass
    class DescribeDBSubnetGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_db_subnet_groups` for valid parameters"""
            pass
    class DescribeEventsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_events` for valid parameters"""
            pass
    class DescribeOrderableDBInstanceOptionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_orderable_db_instance_options` for valid parameters"""
            pass


    class AddTagsToResourceMessage(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class ApplyPendingMaintenanceActionMessage(Mapping):
        pass
    
    class ApplyPendingMaintenanceActionResult(Mapping):
        pass
    
    class CopyDBClusterParameterGroupMessage(Mapping):
        pass
    
    class CopyDBClusterParameterGroupResult(Mapping):
        pass
    
    class CopyDBClusterSnapshotMessage(Mapping):
        pass
    
    class CopyDBClusterSnapshotResult(Mapping):
        pass
    
    class BooleanOptional(object):
        pass
    
    class CreateDBClusterMessage(Mapping):
        pass
    
    class CreateDBClusterResult(Mapping):
        pass
    
    class String(object):
        pass
    
    class CreateDBClusterParameterGroupMessage(Mapping):
        pass
    
    class CreateDBClusterParameterGroupResult(Mapping):
        pass
    
    class CreateDBClusterSnapshotMessage(Mapping):
        pass
    
    class CreateDBClusterSnapshotResult(Mapping):
        pass
    
    class CreateDBInstanceMessage(Mapping):
        pass
    
    class CreateDBInstanceResult(Mapping):
        pass
    
    class CreateDBSubnetGroupMessage(Mapping):
        pass
    
    class CreateDBSubnetGroupResult(Mapping):
        pass
    
    class DeleteDBClusterMessage(Mapping):
        pass
    
    class DeleteDBClusterResult(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class DeleteDBClusterParameterGroupMessage(Mapping):
        pass
    
    class DeleteDBClusterSnapshotMessage(Mapping):
        pass
    
    class DeleteDBClusterSnapshotResult(Mapping):
        pass
    
    class DeleteDBInstanceMessage(Mapping):
        pass
    
    class DeleteDBInstanceResult(Mapping):
        pass
    
    class DeleteDBSubnetGroupMessage(Mapping):
        pass
    
    class DescribeDBClusterParameterGroupsMessage(Mapping):
        pass
    
    class DBClusterParameterGroupsMessage(Mapping):
        pass
    
    class Filter(Mapping):
        pass
    
    class DescribeDBClusterParametersMessage(Mapping):
        pass
    
    class DBClusterParameterGroupDetails(Mapping):
        pass
    
    class DescribeDBClusterSnapshotAttributesMessage(Mapping):
        pass
    
    class DescribeDBClusterSnapshotAttributesResult(Mapping):
        pass
    
    class DescribeDBClusterSnapshotsMessage(Mapping):
        pass
    
    class DBClusterSnapshotMessage(Mapping):
        pass
    
    class DescribeDBClustersMessage(Mapping):
        pass
    
    class DBClusterMessage(Mapping):
        pass
    
    class DescribeDBEngineVersionsMessage(Mapping):
        pass
    
    class DBEngineVersionMessage(Mapping):
        pass
    
    class DescribeDBInstancesMessage(Mapping):
        pass
    
    class DBInstanceMessage(Mapping):
        pass
    
    class DescribeDBSubnetGroupsMessage(Mapping):
        pass
    
    class DBSubnetGroupMessage(Mapping):
        pass
    
    class DescribeEngineDefaultClusterParametersMessage(Mapping):
        pass
    
    class DescribeEngineDefaultClusterParametersResult(Mapping):
        pass
    
    class DescribeEventCategoriesMessage(Mapping):
        pass
    
    class EventCategoriesMessage(Mapping):
        pass
    
    class DescribeEventsMessage(Mapping):
        pass
    
    class EventsMessage(Mapping):
        pass
    
    class TStamp(object):
        pass
    
    class DescribeOrderableDBInstanceOptionsMessage(Mapping):
        pass
    
    class OrderableDBInstanceOptionsMessage(Mapping):
        pass
    
    class DescribePendingMaintenanceActionsMessage(Mapping):
        pass
    
    class PendingMaintenanceActionsMessage(Mapping):
        pass
    
    class FailoverDBClusterMessage(Mapping):
        pass
    
    class FailoverDBClusterResult(Mapping):
        pass
    
    class ListTagsForResourceMessage(Mapping):
        pass
    
    class TagListMessage(Mapping):
        pass
    
    class ModifyDBClusterMessage(Mapping):
        pass
    
    class ModifyDBClusterResult(Mapping):
        pass
    
    class CloudwatchLogsExportConfiguration(Mapping):
        pass
    
    class ModifyDBClusterParameterGroupMessage(Mapping):
        pass
    
    class DBClusterParameterGroupNameMessage(Mapping):
        pass
    
    class Parameter(Mapping):
        pass
    
    class ModifyDBClusterSnapshotAttributeMessage(Mapping):
        pass
    
    class ModifyDBClusterSnapshotAttributeResult(Mapping):
        pass
    
    class ModifyDBInstanceMessage(Mapping):
        pass
    
    class ModifyDBInstanceResult(Mapping):
        pass
    
    class ModifyDBSubnetGroupMessage(Mapping):
        pass
    
    class ModifyDBSubnetGroupResult(Mapping):
        pass
    
    class RebootDBInstanceMessage(Mapping):
        pass
    
    class RebootDBInstanceResult(Mapping):
        pass
    
    class RemoveTagsFromResourceMessage(Mapping):
        pass
    
    class ResetDBClusterParameterGroupMessage(Mapping):
        pass
    
    class RestoreDBClusterFromSnapshotMessage(Mapping):
        pass
    
    class RestoreDBClusterFromSnapshotResult(Mapping):
        pass
    
    class RestoreDBClusterToPointInTimeMessage(Mapping):
        pass
    
    class RestoreDBClusterToPointInTimeResult(Mapping):
        pass
    
    class StartDBClusterMessage(Mapping):
        pass
    
    class StartDBClusterResult(Mapping):
        pass
    
    class StopDBClusterMessage(Mapping):
        pass
    
    class StopDBClusterResult(Mapping):
        pass
    
    def add_tags_to_resource(self, Tags: List[Tag], ResourceName: str):
        """Adds metadata tags to an Amazon DocumentDB resource. You can use these tags with cost allocation reporting to track costs that are associated with Amazon DocumentDB resources. or in a <code>Condition</code> statement in an AWS Identity and Access Management (IAM) policy for Amazon DocumentDB.

        :param self:
        :param Tags:  The tags to be assigned to the Amazon DocumentDB resource. <br/>ResourceName:  The Amazon DocumentDB resource that the tags are added to. This value is an Amazon Resource Name (ARN).<br/>
        :return:         """
        pass

    def apply_pending_maintenance_action(self, OptInType: str, ApplyAction: str, ResourceIdentifier: str) -> ApplyPendingMaintenanceActionResult:
        """Applies a pending maintenance action to a resource (for example, to a DB instance).

        :param self:
        :param OptInType:  A value that specifies the type of opt-in request or undoes an opt-in request. An opt-in request of type <code>immediate</code> can't be undone. Valid values: <ul> <li>  <code>immediate</code> - Apply the maintenance action immediately. </li> <li>  <code>next-maintenance</code> - Apply the maintenance action during the next maintenance window for the resource. </li> <li>  <code>undo-opt-in</code> - Cancel any existing <code>next-maintenance</code> opt-in requests. </li> </ul><br/>ApplyAction:  The pending maintenance action to apply to this resource. Valid values: <code>system-update</code>, <code>db-upgrade</code> <br/>ResourceIdentifier:  The Amazon Resource Name (ARN) of the resource that the pending maintenance action applies to.<br/>
        :return: ResourcePendingMaintenanceActions:  Represents the output of <a>ApplyPendingMaintenanceAction</a>.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingMaintenanceActionDetails</i> A list that provides details about the pending maintenance actions for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceIdentifier</i> The Amazon Resource Name (ARN) of the resource that has pending maintenance actions.<br/>        """
        pass

    def copy_db_cluster_parameter_group(self, TargetDBClusterParameterGroupDescription: str, TargetDBClusterParameterGroupIdentifier: str, SourceDBClusterParameterGroupIdentifier: str, Tags: List[Tag]=None) -> CopyDBClusterParameterGroupResult:
        """Copies the specified DB cluster parameter group.

        :param self:
        :param Tags:  The tags that are to be assigned to the parameter group.<br/>TargetDBClusterParameterGroupDescription:  A description for the copied DB cluster parameter group.<br/>TargetDBClusterParameterGroupIdentifier:  The identifier for the copied DB cluster parameter group. Constraints: <ul> <li> Cannot be null, empty, or blank. </li> <li> Must contain from 1 to 255 letters, numbers, or hyphens. </li> <li> The first character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul> Example: <code>my-cluster-param-group1</code> <br/>SourceDBClusterParameterGroupIdentifier:  The identifier or Amazon Resource Name (ARN) for the source DB cluster parameter group. Constraints: <ul> <li> Must specify a valid DB cluster parameter group. </li> <li> If the source DB cluster parameter group is in the same AWS Region as the copy, specify a valid DB parameter group identifier; for example, <code>my-db-cluster-param-group</code>, or a valid ARN. </li> <li> If the source DB parameter group is in a different AWS Region than the copy, specify a valid DB cluster parameter group ARN; for example, <code>arn:aws:rds:us-east-1:123456789012:cluster-pg:custom-cluster-group1</code>. </li> </ul><br/>
        :return: DBClusterParameterGroup:  Detailed information about a DB cluster parameter group. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroupArn</i> The Amazon Resource Name (ARN) for the DB cluster parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> Provides the customer-specified description for this DB cluster parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroupFamily</i> Provides the name of the DB parameter group family that this DB cluster parameter group is compatible with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroupName</i> Provides the name of the DB cluster parameter group.<br/>        """
        pass

    def copy_db_cluster_snapshot(self, TargetDBClusterSnapshotIdentifier: str, SourceDBClusterSnapshotIdentifier: str, KmsKeyId: str=None, PreSignedUrl: str=None, CopyTags: BooleanOptional=None, Tags: List[Tag]=None) -> CopyDBClusterSnapshotResult:
        """Copies a snapshot of a DB cluster. To copy a DB cluster snapshot from a shared manual DB cluster snapshot, <code>SourceDBClusterSnapshotIdentifier</code> must be the Amazon Resource Name (ARN) of the shared DB cluster snapshot. To cancel the copy operation after it is in progress, delete the target DB cluster snapshot identified by <code>TargetDBClusterSnapshotIdentifier</code> while that DB cluster snapshot is in the <i>copying</i> status.

        :param self:
        :param Tags:  The tags to be assigned to the DB cluster snapshot.<br/>CopyTags:  Set to <code>true</code> to copy all tags from the source DB cluster snapshot to the target DB cluster snapshot, and otherwise <code>false</code>. The default is <code>false</code>.<br/>PreSignedUrl:  The URL that contains a Signature Version 4 signed request for the <code>CopyDBClusterSnapshot</code> API action in the AWS Region that contains the source DB cluster snapshot to copy. You must use the <code>PreSignedUrl</code> parameter when copying an encrypted DB cluster snapshot from another AWS Region. The presigned URL must be a valid request for the <code>CopyDBSClusterSnapshot</code> API action that can be executed in the source AWS Region that contains the encrypted DB cluster snapshot to be copied. The presigned URL request must contain the following parameter values: <ul> <li>  <code>KmsKeyId</code> - The AWS KMS key identifier for the key to use to encrypt the copy of the DB cluster snapshot in the destination AWS Region. This is the same identifier for both the <code>CopyDBClusterSnapshot</code> action that is called in the destination AWS Region, and the action contained in the presigned URL. </li> <li>  <code>DestinationRegion</code> - The name of the AWS Region that the DB cluster snapshot will be created in. </li> <li>  <code>SourceDBClusterSnapshotIdentifier</code> - The DB cluster snapshot identifier for the encrypted DB cluster snapshot to be copied. This identifier must be in the Amazon Resource Name (ARN) format for the source AWS Region. For example, if you are copying an encrypted DB cluster snapshot from the us-west-2 AWS Region, then your <code>SourceDBClusterSnapshotIdentifier</code> looks like the following example: <code>arn:aws:rds:us-west-2:123456789012:cluster-snapshot:my-cluster-snapshot-20161115</code>. </li> </ul><br/>KmsKeyId:  The AWS KMS key ID for an encrypted DB cluster snapshot. The AWS KMS key ID is the Amazon Resource Name (ARN), AWS KMS key identifier, or the AWS KMS key alias for the AWS KMS encryption key.  If you copy an encrypted DB cluster snapshot from your AWS account, you can specify a value for <code>KmsKeyId</code> to encrypt the copy with a new AWS KMS encryption key. If you don't specify a value for <code>KmsKeyId</code>, then the copy of the DB cluster snapshot is encrypted with the same AWS KMS key as the source DB cluster snapshot.  If you copy an encrypted DB cluster snapshot that is shared from another AWS account, then you must specify a value for <code>KmsKeyId</code>.  To copy an encrypted DB cluster snapshot to another AWS Region, set <code>KmsKeyId</code> to the AWS KMS key ID that you want to use to encrypt the copy of the DB cluster snapshot in the destination Region. AWS KMS encryption keys are specific to the AWS Region that they are created in, and you can't use encryption keys from one Region in another Region. If you copy an unencrypted DB cluster snapshot and specify a value for the <code>KmsKeyId</code> parameter, an error is returned.<br/>TargetDBClusterSnapshotIdentifier:  The identifier of the new DB cluster snapshot to create from the source DB cluster snapshot. This parameter is not case sensitive. Constraints: <ul> <li> Must contain from 1 to 63 letters, numbers, or hyphens. </li> <li> The first character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul> Example: <code>my-cluster-snapshot2</code> <br/>SourceDBClusterSnapshotIdentifier:  The identifier of the DB cluster snapshot to copy. This parameter is not case sensitive. You can't copy an encrypted, shared DB cluster snapshot from one AWS Region to another. Constraints: <ul> <li> Must specify a valid system snapshot in the "available" state. </li> <li> If the source snapshot is in the same AWS Region as the copy, specify a valid DB snapshot identifier. </li> <li> If the source snapshot is in a different AWS Region than the copy, specify a valid DB cluster snapshot ARN. </li> </ul> Example: <code>my-cluster-snapshot1</code> <br/>
        :return: DBClusterSnapshot:  Detailed information about a DB cluster snapshot. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceDBClusterSnapshotArn</i> If the DB cluster snapshot was copied from a source DB cluster snapshot, the ARN for the source DB cluster snapshot; otherwise, a null value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotArn</i> The Amazon Resource Name (ARN) for the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is <code>true</code>, the AWS KMS key identifier for the encrypted DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster snapshot is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the percentage of the estimated data that has been transferred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotType</i> Provides the type of the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Provides the version of the database engine for this DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Provides the master user name for the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the virtual private cloud (VPC) ID that is associated with the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the DB cluster was listening on at the time of the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the status of this DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Specifies the name of the database engine.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCreateTime</i> Provides the time when the snapshot was taken, in UTC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Specifies the DB cluster identifier of the DB cluster that this DB cluster snapshot was created from.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotIdentifier</i> Specifies the identifier for the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of Amazon EC2 Availability Zones that instances in the DB cluster snapshot can be restored in.<br/>        """
        pass

    def create_db_cluster(self, MasterUserPassword: str, MasterUsername: str, Engine: str, DBClusterIdentifier: str, AvailabilityZones: List[String]=None, BackupRetentionPeriod: int=None, DBClusterParameterGroupName: str=None, VpcSecurityGroupIds: List[String]=None, DBSubnetGroupName: str=None, EngineVersion: str=None, Port: int=None, PreferredBackupWindow: str=None, PreferredMaintenanceWindow: str=None, Tags: List[Tag]=None, StorageEncrypted: BooleanOptional=None, KmsKeyId: str=None, EnableCloudwatchLogsExports: List[String]=None, DeletionProtection: BooleanOptional=None) -> CreateDBClusterResult:
        """Creates a new Amazon DocumentDB DB cluster.

        :param self:
        :param DeletionProtection:  Specifies whether this cluster can be deleted. If <code>DeletionProtection</code> is enabled, the cluster cannot be deleted unless it is modified and <code>DeletionProtection</code> is disabled. <code>DeletionProtection</code> protects clusters from being accidentally deleted.<br/>EnableCloudwatchLogsExports:  A list of log types that need to be enabled for exporting to Amazon CloudWatch Logs.<br/>KmsKeyId:  The AWS KMS key identifier for an encrypted DB cluster. The AWS KMS key identifier is the Amazon Resource Name (ARN) for the AWS KMS encryption key. If you are creating a DB cluster using the same AWS account that owns the AWS KMS encryption key that is used to encrypt the new DB cluster, you can use the AWS KMS key alias instead of the ARN for the AWS KMS encryption key. If an encryption key is not specified in <code>KmsKeyId</code>: <ul> <li> If <code>ReplicationSourceIdentifier</code> identifies an encrypted source, then Amazon DocumentDB uses the encryption key that is used to encrypt the source. Otherwise, Amazon DocumentDB uses your default encryption key.  </li> <li> If the <code>StorageEncrypted</code> parameter is <code>true</code> and <code>ReplicationSourceIdentifier</code> is not specified, Amazon DocumentDB uses your default encryption key. </li> </ul> AWS KMS creates the default encryption key for your AWS account. Your AWS account has a different default encryption key for each AWS Region. If you create a replica of an encrypted DB cluster in another AWS Region, you must set <code>KmsKeyId</code> to a KMS key ID that is valid in the destination AWS Region. This key is used to encrypt the replica in that AWS Region.<br/>StorageEncrypted:  Specifies whether the DB cluster is encrypted.<br/>Tags:  The tags to be assigned to the DB cluster.<br/>PreferredMaintenanceWindow:  The weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC). Format: <code>ddd:hh24:mi-ddd:hh24:mi</code>  The default is a 30-minute window selected at random from an 8-hour block of time for each AWS Region, occurring on a random day of the week. Valid days: Mon, Tue, Wed, Thu, Fri, Sat, Sun Constraints: Minimum 30-minute window.<br/>PreferredBackupWindow:  The daily time range during which automated backups are created if automated backups are enabled using the <code>BackupRetentionPeriod</code> parameter.  The default is a 30-minute window selected at random from an 8-hour block of time for each AWS Region.  Constraints: <ul> <li> Must be in the format <code>hh24:mi-hh24:mi</code>. </li> <li> Must be in Universal Coordinated Time (UTC). </li> <li> Must not conflict with the preferred maintenance window. </li> <li> Must be at least 30 minutes. </li> </ul><br/>MasterUserPassword:  The password for the master database user. This password can contain any printable ASCII character except forward slash (/), double quote ("), or the "at" symbol (@). Constraints: Must contain from 8 to 41 characters.<br/>MasterUsername:  The name of the master user for the DB cluster. Constraints: <ul> <li> Must be from 1 to 16 letters or numbers. </li> <li> The first character must be a letter. </li> <li> Cannot be a reserved word for the chosen database engine. </li> </ul><br/>Port:  The port number on which the instances in the DB cluster accept connections.<br/>EngineVersion:  The version number of the database engine to use.<br/>Engine:  The name of the database engine to be used for this DB cluster. Valid values: <code>docdb</code> <br/>DBSubnetGroupName:  A DB subnet group to associate with this DB cluster. Constraints: Must match the name of an existing <code>DBSubnetGroup</code>. Must not be default. Example: <code>mySubnetgroup</code> <br/>VpcSecurityGroupIds:  A list of EC2 VPC security groups to associate with this DB cluster.<br/>DBClusterParameterGroupName:   The name of the DB cluster parameter group to associate with this DB cluster.<br/>DBClusterIdentifier:  The DB cluster identifier. This parameter is stored as a lowercase string. Constraints: <ul> <li> Must contain from 1 to 63 letters, numbers, or hyphens. </li> <li> The first character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul> Example: <code>my-cluster</code> <br/>BackupRetentionPeriod:  The number of days for which automated backups are retained. You must specify a minimum value of 1. Default: 1 Constraints: <ul> <li> Must be a value from 1 to 35. </li> </ul><br/>AvailabilityZones:  A list of Amazon EC2 Availability Zones that instances in the DB cluster can be created in.<br/>
        :return: DBCluster:  Detailed information about a DB cluster. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i> Specifies whether this cluster can be deleted. If <code>DeletionProtection</code> is enabled, the cluster cannot be deleted unless it is modified and <code>DeletionProtection</code> is disabled. <code>DeletionProtection</code> protects clusters from being accidentally deleted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB cluster is configured to export to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i> Provides a list of the AWS Identity and Access Management (IAM) roles that are associated with the DB cluster. IAM roles that are associated with a DB cluster grant permission for the DB cluster to access other AWS services on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterArn</i> The Amazon Resource Name (ARN) for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbClusterResourceId</i> The AWS Region-unique, immutable identifier for the DB cluster. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB cluster is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is <code>true</code>, the AWS KMS key identifier for the encrypted DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of virtual private cloud (VPC) security groups that the DB cluster belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterMembers</i> Provides the list of instances that make up the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i> Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master user name for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies whether the DB cluster has instances in multiple Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReaderEndpoint</i> The reader endpoint for the DB cluster. The reader endpoint for a DB cluster load balances connections across the Amazon DocumentDB replicas that are available in a DB cluster. As clients request new connections to the reader endpoint, Amazon DocumentDB distributes the connection requests among the Amazon DocumentDB replicas in the DB cluster. This functionality can help balance your read workload across multiple Amazon DocumentDB replicas in your DB cluster.  If a failover occurs, and the Amazon DocumentDB replica that you are connected to is promoted to be the primary instance, your connection is dropped. To continue sending your read workload to other Amazon DocumentDB replicas in the cluster, you can then reconnect to the reader endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint for the primary instance of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestRestorableTime</i> The earliest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the progress of the operation as a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the current state of this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group that is associated with the DB cluster, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroup</i> Specifies the name of the DB cluster parameter group for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Contains a user-supplied DB cluster identifier. This identifier is the unique key that identifies a DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of Amazon EC2 Availability Zones that instances in the DB cluster can be created in.<br/>        """
        pass

    def create_db_cluster_parameter_group(self, Description: str, DBParameterGroupFamily: str, DBClusterParameterGroupName: str, Tags: List[Tag]=None) -> CreateDBClusterParameterGroupResult:
        """Creates a new DB cluster parameter group. Parameters in a DB cluster parameter group apply to all of the instances in a DB cluster. A DB cluster parameter group is initially created with the default parameters for the database engine used by instances in the DB cluster. To provide custom values for any of the parameters, you must modify the group after you create it. After you create a DB cluster parameter group, you must associate it with your DB cluster. For the new DB cluster parameter group and associated settings to take effect, you must then reboot the DB instances in the DB cluster without failover. <important> After you create a DB cluster parameter group, you should wait at least 5 minutes before creating your first DB cluster that uses that DB cluster parameter group as the default parameter group. This allows Amazon DocumentDB to fully complete the create action before the DB cluster parameter group is used as the default for a new DB cluster. This step is especially important for parameters that are critical when creating the default database for a DB cluster, such as the character set for the default database defined by the <code>character_set_database</code> parameter. </important>

        :param self:
        :param Tags:  The tags to be assigned to the DB cluster parameter group.<br/>Description:  The description for the DB cluster parameter group.<br/>DBParameterGroupFamily:  The DB cluster parameter group family name.<br/>DBClusterParameterGroupName:  The name of the DB cluster parameter group. Constraints: <ul> <li> Must match the name of an existing <code>DBClusterParameterGroup</code>. </li> </ul> <note> This value is stored as a lowercase string. </note><br/>
        :return: DBClusterParameterGroup:  Detailed information about a DB cluster parameter group. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroupArn</i> The Amazon Resource Name (ARN) for the DB cluster parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> Provides the customer-specified description for this DB cluster parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroupFamily</i> Provides the name of the DB parameter group family that this DB cluster parameter group is compatible with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroupName</i> Provides the name of the DB cluster parameter group.<br/>        """
        pass

    def create_db_cluster_snapshot(self, DBClusterIdentifier: str, DBClusterSnapshotIdentifier: str, Tags: List[Tag]=None) -> CreateDBClusterSnapshotResult:
        """Creates a snapshot of a DB cluster. 

        :param self:
        :param Tags:  The tags to be assigned to the DB cluster snapshot.<br/>DBClusterIdentifier:  The identifier of the DB cluster to create a snapshot for. This parameter is not case sensitive. Constraints: <ul> <li> Must match the identifier of an existing <code>DBCluster</code>. </li> </ul> Example: <code>my-cluster</code> <br/>DBClusterSnapshotIdentifier:  The identifier of the DB cluster snapshot. This parameter is stored as a lowercase string. Constraints: <ul> <li> Must contain from 1 to 63 letters, numbers, or hyphens. </li> <li> The first character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul> Example: <code>my-cluster-snapshot1</code> <br/>
        :return: DBClusterSnapshot:  Detailed information about a DB cluster snapshot. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceDBClusterSnapshotArn</i> If the DB cluster snapshot was copied from a source DB cluster snapshot, the ARN for the source DB cluster snapshot; otherwise, a null value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotArn</i> The Amazon Resource Name (ARN) for the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is <code>true</code>, the AWS KMS key identifier for the encrypted DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster snapshot is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the percentage of the estimated data that has been transferred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotType</i> Provides the type of the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Provides the version of the database engine for this DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Provides the master user name for the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the virtual private cloud (VPC) ID that is associated with the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the DB cluster was listening on at the time of the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the status of this DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Specifies the name of the database engine.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCreateTime</i> Provides the time when the snapshot was taken, in UTC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Specifies the DB cluster identifier of the DB cluster that this DB cluster snapshot was created from.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotIdentifier</i> Specifies the identifier for the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of Amazon EC2 Availability Zones that instances in the DB cluster snapshot can be restored in.<br/>        """
        pass

    def create_db_instance(self, DBClusterIdentifier: str, Engine: str, DBInstanceClass: str, DBInstanceIdentifier: str, AvailabilityZone: str=None, PreferredMaintenanceWindow: str=None, AutoMinorVersionUpgrade: BooleanOptional=None, Tags: List[Tag]=None, PromotionTier: int=None) -> CreateDBInstanceResult:
        """Creates a new DB instance.

        :param self:
        :param PromotionTier:  A value that specifies the order in which an Amazon DocumentDB replica is promoted to the primary instance after a failure of the existing primary instance. Default: 1 Valid values: 0-15<br/>DBClusterIdentifier:  The identifier of the DB cluster that the instance will belong to.<br/>Tags:  The tags to be assigned to the DB instance.<br/>AutoMinorVersionUpgrade:  Indicates that minor engine upgrades are applied automatically to the DB instance during the maintenance window. Default: <code>true</code> <br/>PreferredMaintenanceWindow:  The time range each week during which system maintenance can occur, in Universal Coordinated Time (UTC).  Format: <code>ddd:hh24:mi-ddd:hh24:mi</code>  The default is a 30-minute window selected at random from an 8-hour block of time for each AWS Region, occurring on a random day of the week.  Valid days: Mon, Tue, Wed, Thu, Fri, Sat, Sun Constraints: Minimum 30-minute window.<br/>AvailabilityZone:   The Amazon EC2 Availability Zone that the DB instance is created in. Default: A random, system-chosen Availability Zone in the endpoint's AWS Region.  Example: <code>us-east-1d</code>   Constraint: The <code>AvailabilityZone</code> parameter can't be specified if the <code>MultiAZ</code> parameter is set to <code>true</code>. The specified Availability Zone must be in the same AWS Region as the current endpoint. <br/>Engine:  The name of the database engine to be used for this instance. Valid value: <code>docdb</code> <br/>DBInstanceClass:  The compute and memory capacity of the DB instance; for example, <code>db.r5.large</code>. <br/>DBInstanceIdentifier:  The DB instance identifier. This parameter is stored as a lowercase string. Constraints: <ul> <li> Must contain from 1 to 63 letters, numbers, or hyphens. </li> <li> The first character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul> Example: <code>mydbinstance</code> <br/>
        :return: DBInstance:  Detailed information about a DB instance. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB instance is configured to export to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceArn</i> The Amazon Resource Name (ARN) for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PromotionTier</i> A value that specifies the order in which an Amazon DocumentDB replica is promoted to the primary instance after a failure of the existing primary instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbiResourceId</i> The AWS Region-unique, immutable identifier for the DB instance. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB instance is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i>  If <code>StorageEncrypted</code> is <code>true</code>, the AWS KMS key identifier for the encrypted DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB instance is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Contains the name of the DB cluster that the DB instance is a member of if the DB instance is a member of a DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusInfos</i> The status of a read replica. If the instance is not a read replica, this is blank.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> Specifies the availability options for the DB instance. A value of <code>true</code> specifies an internet-facing instance with a publicly resolvable DNS name, which resolves to a public IP address. A value of <code>false</code> specifies an internal instance with a DNS name that resolves to a private IP address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> Indicates that minor version patches are applied automatically.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> Specifies that changes to the DB instance are pending. This element is included only when changes are pending. Specific changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: Specifies that changes to the DB instance are pending. This element is included only when changes are pending. Specific changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group that is associated with the DB instance, including the name, description, and subnets in the subnet group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DBSubnetGroup</b>: Specifies information on the subnet group that is associated with the DB instance, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> Specifies the name of the Availability Zone that the DB instance is located in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security group elements that the DB instance belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i>  Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCreateTime</i> Provides the date and time that the DB instance was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: Specifies the connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceStatus</i> Specifies the current state of this database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i> Contains the name of the compute and memory capacity class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i> Contains a user-provided database identifier. This identifier is the unique key that identifies a DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupArn</i> The Amazon Resource Identifier (ARN) for the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> Detailed information about one or more subnets within a DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> Provides the status of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the virtual private cloud (VPC) ID of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupDescription</i> Provides the description of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The name of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The new DB subnet group for the DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> Specifies the identifier of the certificate authority (CA) certificate for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type to be associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i>  Contains the new <code>DBInstanceIdentifier</code> for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the new Provisioned IOPS value for the DB instance that will be applied or is currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> The license model for the DB instance. Valid values: <code>license-included</code>, <code>bring-your-own-license</code>, <code>general-public-license</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Indicates that the Single-AZ DB instance is to change to a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the pending number of days for which automated backups are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the pending port for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> Contains the pending or currently in-progress change of the master credentials for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i>  Contains the new <code>AllocatedStorage</code> size for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i>  Contains the new <code>DBInstanceClass</code> for the DB instance that will be applied or is currently being applied. <br/>A list of the log types whose configuration is still pending. These log types are in the process of being activated or deactivated.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingCloudwatchLogsExports</b>: A list of the log types whose configuration is still pending. These log types are in the process of being activated or deactivated.        """
        pass

    def create_db_subnet_group(self, SubnetIds: List[String], DBSubnetGroupDescription: str, DBSubnetGroupName: str, Tags: List[Tag]=None) -> CreateDBSubnetGroupResult:
        """Creates a new DB subnet group. DB subnet groups must contain at least one subnet in at least two Availability Zones in the AWS Region.

        :param self:
        :param Tags:  The tags to be assigned to the DB subnet group.<br/>SubnetIds:  The Amazon EC2 subnet IDs for the DB subnet group.<br/>DBSubnetGroupDescription:  The description for the DB subnet group.<br/>DBSubnetGroupName:  The name for the DB subnet group. This value is stored as a lowercase string. Constraints: Must contain no more than 255 letters, numbers, periods, underscores, spaces, or hyphens. Must not be default. Example: <code>mySubnetgroup</code> <br/>
        :return: DBSubnetGroup:  Detailed information about a DB subnet group. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupArn</i> The Amazon Resource Identifier (ARN) for the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> Detailed information about one or more subnets within a DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> Provides the status of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the virtual private cloud (VPC) ID of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupDescription</i> Provides the description of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The name of the DB subnet group.<br/>        """
        pass

    def delete_db_cluster(self, DBClusterIdentifier: str, SkipFinalSnapshot: Boolean=None, FinalDBSnapshotIdentifier: str=None) -> DeleteDBClusterResult:
        """Deletes a previously provisioned DB cluster. When you delete a DB cluster, all automated backups for that DB cluster are deleted and can't be recovered. Manual DB cluster snapshots of the specified DB cluster are not deleted. <p/>

        :param self:
        :param FinalDBSnapshotIdentifier:   The DB cluster snapshot identifier of the new DB cluster snapshot created when <code>SkipFinalSnapshot</code> is set to <code>false</code>.  <note>  Specifying this parameter and also setting the <code>SkipFinalShapshot</code> parameter to <code>true</code> results in an error.  </note> Constraints: <ul> <li> Must be from 1 to 255 letters, numbers, or hyphens. </li> <li> The first character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul><br/>SkipFinalSnapshot:   Determines whether a final DB cluster snapshot is created before the DB cluster is deleted. If <code>true</code> is specified, no DB cluster snapshot is created. If <code>false</code> is specified, a DB cluster snapshot is created before the DB cluster is deleted.  <note> If <code>SkipFinalSnapshot</code> is <code>false</code>, you must specify a <code>FinalDBSnapshotIdentifier</code> parameter. </note> Default: <code>false</code> <br/>DBClusterIdentifier:  The DB cluster identifier for the DB cluster to be deleted. This parameter isn't case sensitive. Constraints: <ul> <li> Must match an existing <code>DBClusterIdentifier</code>. </li> </ul><br/>
        :return: DBCluster:  Detailed information about a DB cluster. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i> Specifies whether this cluster can be deleted. If <code>DeletionProtection</code> is enabled, the cluster cannot be deleted unless it is modified and <code>DeletionProtection</code> is disabled. <code>DeletionProtection</code> protects clusters from being accidentally deleted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB cluster is configured to export to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i> Provides a list of the AWS Identity and Access Management (IAM) roles that are associated with the DB cluster. IAM roles that are associated with a DB cluster grant permission for the DB cluster to access other AWS services on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterArn</i> The Amazon Resource Name (ARN) for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbClusterResourceId</i> The AWS Region-unique, immutable identifier for the DB cluster. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB cluster is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is <code>true</code>, the AWS KMS key identifier for the encrypted DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of virtual private cloud (VPC) security groups that the DB cluster belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterMembers</i> Provides the list of instances that make up the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i> Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master user name for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies whether the DB cluster has instances in multiple Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReaderEndpoint</i> The reader endpoint for the DB cluster. The reader endpoint for a DB cluster load balances connections across the Amazon DocumentDB replicas that are available in a DB cluster. As clients request new connections to the reader endpoint, Amazon DocumentDB distributes the connection requests among the Amazon DocumentDB replicas in the DB cluster. This functionality can help balance your read workload across multiple Amazon DocumentDB replicas in your DB cluster.  If a failover occurs, and the Amazon DocumentDB replica that you are connected to is promoted to be the primary instance, your connection is dropped. To continue sending your read workload to other Amazon DocumentDB replicas in the cluster, you can then reconnect to the reader endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint for the primary instance of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestRestorableTime</i> The earliest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the progress of the operation as a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the current state of this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group that is associated with the DB cluster, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroup</i> Specifies the name of the DB cluster parameter group for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Contains a user-supplied DB cluster identifier. This identifier is the unique key that identifies a DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of Amazon EC2 Availability Zones that instances in the DB cluster can be created in.<br/>        """
        pass

    def delete_db_cluster_parameter_group(self, DBClusterParameterGroupName: str):
        """Deletes a specified DB cluster parameter group. The DB cluster parameter group to be deleted can't be associated with any DB clusters.

        :param self:
        :param DBClusterParameterGroupName:  The name of the DB cluster parameter group. Constraints: <ul> <li> Must be the name of an existing DB cluster parameter group. </li> <li> You can't delete a default DB cluster parameter group. </li> <li> Cannot be associated with any DB clusters. </li> </ul><br/>
        :return:         """
        pass

    def delete_db_cluster_snapshot(self, DBClusterSnapshotIdentifier: str) -> DeleteDBClusterSnapshotResult:
        """Deletes a DB cluster snapshot. If the snapshot is being copied, the copy operation is terminated. <note> The DB cluster snapshot must be in the <code>available</code> state to be deleted. </note>

        :param self:
        :param DBClusterSnapshotIdentifier:  The identifier of the DB cluster snapshot to delete. Constraints: Must be the name of an existing DB cluster snapshot in the <code>available</code> state.<br/>
        :return: DBClusterSnapshot:  Detailed information about a DB cluster snapshot. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceDBClusterSnapshotArn</i> If the DB cluster snapshot was copied from a source DB cluster snapshot, the ARN for the source DB cluster snapshot; otherwise, a null value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotArn</i> The Amazon Resource Name (ARN) for the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is <code>true</code>, the AWS KMS key identifier for the encrypted DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster snapshot is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the percentage of the estimated data that has been transferred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotType</i> Provides the type of the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Provides the version of the database engine for this DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Provides the master user name for the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the virtual private cloud (VPC) ID that is associated with the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the DB cluster was listening on at the time of the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the status of this DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Specifies the name of the database engine.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCreateTime</i> Provides the time when the snapshot was taken, in UTC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Specifies the DB cluster identifier of the DB cluster that this DB cluster snapshot was created from.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotIdentifier</i> Specifies the identifier for the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of Amazon EC2 Availability Zones that instances in the DB cluster snapshot can be restored in.<br/>        """
        pass

    def delete_db_instance(self, DBInstanceIdentifier: str) -> DeleteDBInstanceResult:
        """Deletes a previously provisioned DB instance. 

        :param self:
        :param DBInstanceIdentifier:  The DB instance identifier for the DB instance to be deleted. This parameter isn't case sensitive. Constraints: <ul> <li> Must match the name of an existing DB instance. </li> </ul><br/>
        :return: DBInstance:  Detailed information about a DB instance. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB instance is configured to export to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceArn</i> The Amazon Resource Name (ARN) for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PromotionTier</i> A value that specifies the order in which an Amazon DocumentDB replica is promoted to the primary instance after a failure of the existing primary instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbiResourceId</i> The AWS Region-unique, immutable identifier for the DB instance. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB instance is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i>  If <code>StorageEncrypted</code> is <code>true</code>, the AWS KMS key identifier for the encrypted DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB instance is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Contains the name of the DB cluster that the DB instance is a member of if the DB instance is a member of a DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusInfos</i> The status of a read replica. If the instance is not a read replica, this is blank.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> Specifies the availability options for the DB instance. A value of <code>true</code> specifies an internet-facing instance with a publicly resolvable DNS name, which resolves to a public IP address. A value of <code>false</code> specifies an internal instance with a DNS name that resolves to a private IP address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> Indicates that minor version patches are applied automatically.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> Specifies that changes to the DB instance are pending. This element is included only when changes are pending. Specific changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: Specifies that changes to the DB instance are pending. This element is included only when changes are pending. Specific changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group that is associated with the DB instance, including the name, description, and subnets in the subnet group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DBSubnetGroup</b>: Specifies information on the subnet group that is associated with the DB instance, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> Specifies the name of the Availability Zone that the DB instance is located in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security group elements that the DB instance belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i>  Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCreateTime</i> Provides the date and time that the DB instance was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: Specifies the connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceStatus</i> Specifies the current state of this database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i> Contains the name of the compute and memory capacity class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i> Contains a user-provided database identifier. This identifier is the unique key that identifies a DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupArn</i> The Amazon Resource Identifier (ARN) for the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> Detailed information about one or more subnets within a DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> Provides the status of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the virtual private cloud (VPC) ID of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupDescription</i> Provides the description of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The name of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The new DB subnet group for the DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> Specifies the identifier of the certificate authority (CA) certificate for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type to be associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i>  Contains the new <code>DBInstanceIdentifier</code> for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the new Provisioned IOPS value for the DB instance that will be applied or is currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> The license model for the DB instance. Valid values: <code>license-included</code>, <code>bring-your-own-license</code>, <code>general-public-license</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Indicates that the Single-AZ DB instance is to change to a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the pending number of days for which automated backups are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the pending port for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> Contains the pending or currently in-progress change of the master credentials for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i>  Contains the new <code>AllocatedStorage</code> size for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i>  Contains the new <code>DBInstanceClass</code> for the DB instance that will be applied or is currently being applied. <br/>A list of the log types whose configuration is still pending. These log types are in the process of being activated or deactivated.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingCloudwatchLogsExports</b>: A list of the log types whose configuration is still pending. These log types are in the process of being activated or deactivated.        """
        pass

    def delete_db_subnet_group(self, DBSubnetGroupName: str):
        """Deletes a DB subnet group. <note> The specified database subnet group must not be associated with any DB instances. </note>

        :param self:
        :param DBSubnetGroupName:  The name of the database subnet group to delete. <note> You can't delete the default subnet group. </note> Constraints: Must match the name of an existing <code>DBSubnetGroup</code>. Must not be default. Example: <code>mySubnetgroup</code> <br/>
        :return:         """
        pass

    def describe_db_cluster_parameter_groups(self, DBClusterParameterGroupName: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DBClusterParameterGroupsMessage:
        """Returns a list of <code>DBClusterParameterGroup</code> descriptions. If a <code>DBClusterParameterGroupName</code> parameter is specified, the list contains only the description of the specified DB cluster parameter group. 

        :param self:
        :param Marker:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token (marker) is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>DBClusterParameterGroupName:  The name of a specific DB cluster parameter group to return details for. Constraints: <ul> <li> If provided, must match the name of an existing <code>DBClusterParameterGroup</code>. </li> </ul><br/>
        :return: DBClusterParameterGroups:  A list of DB cluster parameter groups.<br/>Marker:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>        """
        pass

    def describe_db_cluster_parameters(self, DBClusterParameterGroupName: str, Source: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DBClusterParameterGroupDetails:
        """Returns the detailed parameter list for a particular DB cluster parameter group.

        :param self:
        :param Marker:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token (marker) is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>Source:   A value that indicates to return only parameters for a specific source. Parameter sources can be <code>engine</code>, <code>service</code>, or <code>customer</code>. <br/>DBClusterParameterGroupName:  The name of a specific DB cluster parameter group to return parameter details for. Constraints: <ul> <li> If provided, must match the name of an existing <code>DBClusterParameterGroup</code>. </li> </ul><br/>
        :return: Marker:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>Parameters:  Provides a list of parameters for the DB cluster parameter group.<br/>        """
        pass

    def describe_db_cluster_snapshot_attributes(self, DBClusterSnapshotIdentifier: str) -> DescribeDBClusterSnapshotAttributesResult:
        """Returns a list of DB cluster snapshot attribute names and values for a manual DB cluster snapshot. When you share snapshots with other AWS accounts, <code>DescribeDBClusterSnapshotAttributes</code> returns the <code>restore</code> attribute and a list of IDs for the AWS accounts that are authorized to copy or restore the manual DB cluster snapshot. If <code>all</code> is included in the list of values for the <code>restore</code> attribute, then the manual DB cluster snapshot is public and can be copied or restored by all AWS accounts.

        :param self:
        :param DBClusterSnapshotIdentifier:  The identifier for the DB cluster snapshot to describe the attributes for.<br/>
        :return: DBClusterSnapshotAttributesResult:  Detailed information about the attributes that are associated with a DB cluster snapshot.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotAttributes</i> The list of attributes and values for the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotIdentifier</i> The identifier of the DB cluster snapshot that the attributes apply to.<br/>        """
        pass

    def describe_db_cluster_snapshots(self, DBClusterIdentifier: str=None, DBClusterSnapshotIdentifier: str=None, SnapshotType: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None, IncludeShared: Boolean=None, IncludePublic: Boolean=None) -> DBClusterSnapshotMessage:
        """Returns information about DB cluster snapshots. This API operation supports pagination.

        :param self:
        :param IncludePublic:  Set to <code>true</code> to include manual DB cluster snapshots that are public and can be copied or restored by any AWS account, and otherwise <code>false</code>. The default is <code>false</code>.<br/>IncludeShared:  Set to <code>true</code> to include shared manual DB cluster snapshots from other AWS accounts that this AWS account has been given permission to copy or restore, and otherwise <code>false</code>. The default is <code>false</code>.<br/>Marker:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token (marker) is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>SnapshotType:  The type of DB cluster snapshots to be returned. You can specify one of the following values: <ul> <li>  <code>automated</code> - Return all DB cluster snapshots that Amazon DocumentDB has automatically created for your AWS account. </li> <li>  <code>manual</code> - Return all DB cluster snapshots that you have manually created for your AWS account. </li> <li>  <code>shared</code> - Return all manual DB cluster snapshots that have been shared to your AWS account. </li> <li>  <code>public</code> - Return all DB cluster snapshots that have been marked as public. </li> </ul> If you don't specify a <code>SnapshotType</code> value, then both automated and manual DB cluster snapshots are returned. You can include shared DB cluster snapshots with these results by setting the <code>IncludeShared</code> parameter to <code>true</code>. You can include public DB cluster snapshots with these results by setting the <code>IncludePublic</code> parameter to <code>true</code>. The <code>IncludeShared</code> and <code>IncludePublic</code> parameters don't apply for <code>SnapshotType</code> values of <code>manual</code> or <code>automated</code>. The <code>IncludePublic</code> parameter doesn't apply when <code>SnapshotType</code> is set to <code>shared</code>. The <code>IncludeShared</code> parameter doesn't apply when <code>SnapshotType</code> is set to <code>public</code>.<br/>DBClusterSnapshotIdentifier:  A specific DB cluster snapshot identifier to describe. This parameter can't be used with the <code>DBClusterIdentifier</code> parameter. This value is stored as a lowercase string.  Constraints: <ul> <li> If provided, must match the identifier of an existing <code>DBClusterSnapshot</code>. </li> <li> If this identifier is for an automated snapshot, the <code>SnapshotType</code> parameter must also be specified. </li> </ul><br/>DBClusterIdentifier:  The ID of the DB cluster to retrieve the list of DB cluster snapshots for. This parameter can't be used with the <code>DBClusterSnapshotIdentifier</code> parameter. This parameter is not case sensitive.  Constraints: <ul> <li> If provided, must match the identifier of an existing <code>DBCluster</code>. </li> </ul><br/>
        :return: DBClusterSnapshots:  Provides a list of DB cluster snapshots.<br/>Marker:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>        """
        pass

    def describe_db_clusters(self, DBClusterIdentifier: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DBClusterMessage:
        """Returns information about provisioned Amazon DocumentDB DB clusters. This API operation supports pagination.

        :param self:
        :param Marker:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token (marker) is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  A filter that specifies one or more DB clusters to describe. Supported filters: <ul> <li>  <code>db-cluster-id</code> - Accepts DB cluster identifiers and DB cluster Amazon Resource Names (ARNs). The results list only includes information about the DB clusters identified by these ARNs. </li> </ul><br/>DBClusterIdentifier:  The user-provided DB cluster identifier. If this parameter is specified, information from only the specific DB cluster is returned. This parameter isn't case sensitive. Constraints: <ul> <li> If provided, must match an existing <code>DBClusterIdentifier</code>. </li> </ul><br/>
        :return: DBClusters:  A list of DB clusters.<br/>Marker:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>        """
        pass

    def describe_db_engine_versions(self, Engine: str=None, EngineVersion: str=None, DBParameterGroupFamily: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None, DefaultOnly: Boolean=None, ListSupportedCharacterSets: BooleanOptional=None, ListSupportedTimezones: BooleanOptional=None) -> DBEngineVersionMessage:
        """Returns a list of the available DB engines.

        :param self:
        :param ListSupportedTimezones:  If this parameter is specified and the requested engine supports the <code>TimeZone</code> parameter for <code>CreateDBInstance</code>, the response includes a list of supported time zones for each engine version. <br/>ListSupportedCharacterSets:  If this parameter is specified and the requested engine supports the <code>CharacterSetName</code> parameter for <code>CreateDBInstance</code>, the response includes a list of supported character sets for each engine version. <br/>DefaultOnly:  Indicates that only the default version of the specified engine or engine and major version combination is returned.<br/>Marker:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token (marker) is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>DBParameterGroupFamily:  The name of a specific DB parameter group family to return details for. Constraints: <ul> <li> If provided, must match an existing <code>DBParameterGroupFamily</code>. </li> </ul><br/>EngineVersion:  The database engine version to return. Example: <code>5.1.49</code> <br/>Engine:  The database engine to return.<br/>
        :return: DBEngineVersions:  Detailed information about one or more DB engine versions.<br/>Marker:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>        """
        pass

    def describe_db_instances(self, DBInstanceIdentifier: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DBInstanceMessage:
        """Returns information about provisioned Amazon DocumentDB instances. This API supports pagination.

        :param self:
        :param Marker:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token (marker) is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  A filter that specifies one or more DB instances to describe. Supported filters: <ul> <li>  <code>db-cluster-id</code> - Accepts DB cluster identifiers and DB cluster Amazon Resource Names (ARNs). The results list includes only the information about the DB instances that are associated with the DB clusters that are identified by these ARNs. </li> <li>  <code>db-instance-id</code> - Accepts DB instance identifiers and DB instance ARNs. The results list includes only the information about the DB instances that are identified by these ARNs. </li> </ul><br/>DBInstanceIdentifier:  The user-provided instance identifier. If this parameter is specified, information from only the specific DB instance is returned. This parameter isn't case sensitive. Constraints: <ul> <li> If provided, must match the identifier of an existing <code>DBInstance</code>. </li> </ul><br/>
        :return: DBInstances:  Detailed information about one or more DB instances. <br/>Marker:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>        """
        pass

    def describe_db_subnet_groups(self, DBSubnetGroupName: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DBSubnetGroupMessage:
        """Returns a list of <code>DBSubnetGroup</code> descriptions. If a <code>DBSubnetGroupName</code> is specified, the list will contain only the descriptions of the specified <code>DBSubnetGroup</code>.

        :param self:
        :param Marker:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token (marker) is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>DBSubnetGroupName:  The name of the DB subnet group to return details for.<br/>
        :return: DBSubnetGroups:  Detailed information about one or more DB subnet groups.<br/>Marker:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>        """
        pass

    def describe_engine_default_cluster_parameters(self, DBParameterGroupFamily: str, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DescribeEngineDefaultClusterParametersResult:
        """Returns the default engine and system parameter information for the cluster database engine.

        :param self:
        :param Marker:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token (marker) is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>DBParameterGroupFamily:  The name of the DB cluster parameter group family to return the engine parameter information for.<br/>
        :return: EngineDefaults:  Contains the result of a successful invocation of the <code>DescribeEngineDefaultClusterParameters</code> operation. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> The parameters of a particular DB cluster parameter group family.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Marker</i> An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroupFamily</i> The name of the DB cluster parameter group family to return the engine parameter information for.<br/>        """
        pass

    def describe_event_categories(self, SourceType: str=None, Filters: List[Filter]=None) -> EventCategoriesMessage:
        """Displays a list of categories for all event source types, or, if specified, for a specified source type. 

        :param self:
        :param Filters:  This parameter is not currently supported.<br/>SourceType:  The type of source that is generating the events. Valid values: <code>db-instance</code>, <code>db-parameter-group</code>, <code>db-security-group</code>, <code>db-snapshot</code> <br/>
        :return: EventCategoriesMapList:  A list of event category maps.<br/>        """
        pass

    def describe_events(self, SourceIdentifier: str=None, SourceType: str=None, StartTime: TStamp=None, EndTime: TStamp=None, Duration: int=None, EventCategories: List[String]=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> EventsMessage:
        """Returns events related to DB instances, DB security groups, DB snapshots, and DB parameter groups for the past 14 days. You can obtain events specific to a particular DB instance, DB security group, DB snapshot, or DB parameter group by providing the name as a parameter. By default, the events of the past hour are returned.

        :param self:
        :param Marker:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token (marker) is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>EventCategories:  A list of event categories that trigger notifications for an event notification subscription.<br/>Duration:  The number of minutes to retrieve events for. Default: 60<br/>EndTime:   The end of the time interval for which to retrieve events, specified in ISO 8601 format.  Example: 2009-07-08T18:00Z<br/>StartTime:   The beginning of the time interval to retrieve events for, specified in ISO 8601 format.  Example: 2009-07-08T18:00Z<br/>SourceType:  The event source to retrieve events for. If no value is specified, all events are returned.<br/>SourceIdentifier:  The identifier of the event source for which events are returned. If not specified, then all sources are included in the response. Constraints: <ul> <li> If <code>SourceIdentifier</code> is provided, <code>SourceType</code> must also be provided. </li> <li> If the source type is <code>DBInstance</code>, a <code>DBInstanceIdentifier</code> must be provided. </li> <li> If the source type is <code>DBSecurityGroup</code>, a <code>DBSecurityGroupName</code> must be provided. </li> <li> If the source type is <code>DBParameterGroup</code>, a <code>DBParameterGroupName</code> must be provided. </li> <li> If the source type is <code>DBSnapshot</code>, a <code>DBSnapshotIdentifier</code> must be provided. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul><br/>
        :return: Events:  Detailed information about one or more events. <br/>Marker:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>        """
        pass

    def describe_orderable_db_instance_options(self, Engine: str, EngineVersion: str=None, DBInstanceClass: str=None, LicenseModel: str=None, Vpc: BooleanOptional=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> OrderableDBInstanceOptionsMessage:
        """Returns a list of orderable DB instance options for the specified engine.

        :param self:
        :param Marker:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token (marker) is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>Vpc:  The virtual private cloud (VPC) filter value. Specify this parameter to show only the available VPC or non-VPC offerings.<br/>LicenseModel:  The license model filter value. Specify this parameter to show only the available offerings that match the specified license model.<br/>DBInstanceClass:  The DB instance class filter value. Specify this parameter to show only the available offerings that match the specified DB instance class.<br/>EngineVersion:  The engine version filter value. Specify this parameter to show only the available offerings that match the specified engine version.<br/>Engine:  The name of the engine to retrieve DB instance options for.<br/>
        :return: Marker:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>OrderableDBInstanceOptions:  The options that are available for a particular orderable DB instance.<br/>        """
        pass

    def describe_pending_maintenance_actions(self, ResourceIdentifier: str=None, Filters: List[Filter]=None, Marker: str=None, MaxRecords: int=None) -> PendingMaintenanceActionsMessage:
        """Returns a list of resources (for example, DB instances) that have at least one pending maintenance action.

        :param self:
        :param MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token (marker) is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: Minimum 20, maximum 100.<br/>Marker:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>Filters:  A filter that specifies one or more resources to return pending maintenance actions for. Supported filters: <ul> <li>  <code>db-cluster-id</code> - Accepts DB cluster identifiers and DB cluster Amazon Resource Names (ARNs). The results list includes only pending maintenance actions for the DB clusters identified by these ARNs. </li> <li>  <code>db-instance-id</code> - Accepts DB instance identifiers and DB instance ARNs. The results list includes only pending maintenance actions for the DB instances identified by these ARNs. </li> </ul><br/>ResourceIdentifier:  The ARN of a resource to return pending maintenance actions for.<br/>
        :return: Marker:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>PendingMaintenanceActions:  The maintenance actions to be applied.<br/>        """
        pass

    def failover_db_cluster(self, DBClusterIdentifier: str=None, TargetDBInstanceIdentifier: str=None) -> FailoverDBClusterResult:
        """Forces a failover for a DB cluster. A failover for a DB cluster promotes one of the Amazon DocumentDB replicas (read-only instances) in the DB cluster to be the primary instance (the cluster writer). If the primary instance fails, Amazon DocumentDB automatically fails over to an Amazon DocumentDB replica, if one exists. You can force a failover when you want to simulate a failure of a primary instance for testing.

        :param self:
        :param TargetDBInstanceIdentifier:  The name of the instance to promote to the primary instance. You must specify the instance identifier for an Amazon DocumentDB replica in the DB cluster. For example, <code>mydbcluster-replica1</code>.<br/>DBClusterIdentifier:  A DB cluster identifier to force a failover for. This parameter is not case sensitive. Constraints: <ul> <li> Must match the identifier of an existing <code>DBCluster</code>. </li> </ul><br/>
        :return: DBCluster:  Detailed information about a DB cluster. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i> Specifies whether this cluster can be deleted. If <code>DeletionProtection</code> is enabled, the cluster cannot be deleted unless it is modified and <code>DeletionProtection</code> is disabled. <code>DeletionProtection</code> protects clusters from being accidentally deleted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB cluster is configured to export to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i> Provides a list of the AWS Identity and Access Management (IAM) roles that are associated with the DB cluster. IAM roles that are associated with a DB cluster grant permission for the DB cluster to access other AWS services on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterArn</i> The Amazon Resource Name (ARN) for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbClusterResourceId</i> The AWS Region-unique, immutable identifier for the DB cluster. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB cluster is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is <code>true</code>, the AWS KMS key identifier for the encrypted DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of virtual private cloud (VPC) security groups that the DB cluster belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterMembers</i> Provides the list of instances that make up the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i> Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master user name for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies whether the DB cluster has instances in multiple Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReaderEndpoint</i> The reader endpoint for the DB cluster. The reader endpoint for a DB cluster load balances connections across the Amazon DocumentDB replicas that are available in a DB cluster. As clients request new connections to the reader endpoint, Amazon DocumentDB distributes the connection requests among the Amazon DocumentDB replicas in the DB cluster. This functionality can help balance your read workload across multiple Amazon DocumentDB replicas in your DB cluster.  If a failover occurs, and the Amazon DocumentDB replica that you are connected to is promoted to be the primary instance, your connection is dropped. To continue sending your read workload to other Amazon DocumentDB replicas in the cluster, you can then reconnect to the reader endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint for the primary instance of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestRestorableTime</i> The earliest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the progress of the operation as a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the current state of this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group that is associated with the DB cluster, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroup</i> Specifies the name of the DB cluster parameter group for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Contains a user-supplied DB cluster identifier. This identifier is the unique key that identifies a DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of Amazon EC2 Availability Zones that instances in the DB cluster can be created in.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceName: str, Filters: List[Filter]=None) -> TagListMessage:
        """Lists all tags on an Amazon DocumentDB resource.

        :param self:
        :param Filters:  This parameter is not currently supported.<br/>ResourceName:  The Amazon DocumentDB resource with tags to be listed. This value is an Amazon Resource Name (ARN).<br/>
        :return: TagList:  A list of one or more tags.<br/>        """
        pass

    def modify_db_cluster(self, DBClusterIdentifier: str, NewDBClusterIdentifier: str=None, ApplyImmediately: Boolean=None, BackupRetentionPeriod: int=None, DBClusterParameterGroupName: str=None, VpcSecurityGroupIds: List[String]=None, Port: int=None, MasterUserPassword: str=None, PreferredBackupWindow: str=None, PreferredMaintenanceWindow: str=None, CloudwatchLogsExportConfiguration: CloudwatchLogsExportConfiguration=None, EngineVersion: str=None, DeletionProtection: BooleanOptional=None) -> ModifyDBClusterResult:
        """Modifies a setting for an Amazon DocumentDB DB cluster. You can change one or more database configuration parameters by specifying these parameters and the new values in the request. 

        :param self:
        :param DeletionProtection:  Specifies whether this cluster can be deleted. If <code>DeletionProtection</code> is enabled, the cluster cannot be deleted unless it is modified and <code>DeletionProtection</code> is disabled. <code>DeletionProtection</code> protects clusters from being accidentally deleted.<br/>EngineVersion:  The version number of the database engine to which you want to upgrade. Changing this parameter results in an outage. The change is applied during the next maintenance window unless the <code>ApplyImmediately</code> parameter is set to <code>true</code>.<br/>CloudwatchLogsExportConfiguration:  The configuration setting for the log types to be enabled for export to Amazon CloudWatch Logs for a specific DB instance or DB cluster. The <code>EnableLogTypes</code> and <code>DisableLogTypes</code> arrays determine which logs are exported (or not exported) to CloudWatch Logs.:<br/>PreferredMaintenanceWindow:  The weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC). Format: <code>ddd:hh24:mi-ddd:hh24:mi</code>  The default is a 30-minute window selected at random from an 8-hour block of time for each AWS Region, occurring on a random day of the week.  Valid days: Mon, Tue, Wed, Thu, Fri, Sat, Sun Constraints: Minimum 30-minute window.<br/>PreferredBackupWindow:  The daily time range during which automated backups are created if automated backups are enabled, using the <code>BackupRetentionPeriod</code> parameter.  The default is a 30-minute window selected at random from an 8-hour block of time for each AWS Region.  Constraints: <ul> <li> Must be in the format <code>hh24:mi-hh24:mi</code>. </li> <li> Must be in Universal Coordinated Time (UTC). </li> <li> Must not conflict with the preferred maintenance window. </li> <li> Must be at least 30 minutes. </li> </ul><br/>MasterUserPassword:  The password for the master database user. This password can contain any printable ASCII character except forward slash (/), double quote ("), or the "at" symbol (@). Constraints: Must contain from 8 to 41 characters.<br/>Port:  The port number on which the DB cluster accepts connections. Constraints: Must be a value from <code>1150</code> to <code>65535</code>.  Default: The same port as the original DB cluster.<br/>VpcSecurityGroupIds:  A list of virtual private cloud (VPC) security groups that the DB cluster will belong to.<br/>DBClusterParameterGroupName:  The name of the DB cluster parameter group to use for the DB cluster.<br/>BackupRetentionPeriod:  The number of days for which automated backups are retained. You must specify a minimum value of 1. Default: 1 Constraints: <ul> <li> Must be a value from 1 to 35. </li> </ul><br/>ApplyImmediately:  A value that specifies whether the changes in this request and any pending changes are asynchronously applied as soon as possible, regardless of the <code>PreferredMaintenanceWindow</code> setting for the DB cluster. If this parameter is set to <code>false</code>, changes to the DB cluster are applied during the next maintenance window. The <code>ApplyImmediately</code> parameter affects only the <code>NewDBClusterIdentifier</code> and <code>MasterUserPassword</code> values. If you set this parameter value to <code>false</code>, the changes to the <code>NewDBClusterIdentifier</code> and <code>MasterUserPassword</code> values are applied during the next maintenance window. All other changes are applied immediately, regardless of the value of the <code>ApplyImmediately</code> parameter. Default: <code>false</code> <br/>NewDBClusterIdentifier:  The new DB cluster identifier for the DB cluster when renaming a DB cluster. This value is stored as a lowercase string. Constraints: <ul> <li> Must contain from 1 to 63 letters, numbers, or hyphens. </li> <li> The first character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul> Example: <code>my-cluster2</code> <br/>DBClusterIdentifier:  The DB cluster identifier for the cluster that is being modified. This parameter is not case sensitive. Constraints: <ul> <li> Must match the identifier of an existing <code>DBCluster</code>. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisableLogTypes</i> The list of log types to disable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnableLogTypes</i> The list of log types to enable.<br/>
        :return: DBCluster:  Detailed information about a DB cluster. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i> Specifies whether this cluster can be deleted. If <code>DeletionProtection</code> is enabled, the cluster cannot be deleted unless it is modified and <code>DeletionProtection</code> is disabled. <code>DeletionProtection</code> protects clusters from being accidentally deleted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB cluster is configured to export to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i> Provides a list of the AWS Identity and Access Management (IAM) roles that are associated with the DB cluster. IAM roles that are associated with a DB cluster grant permission for the DB cluster to access other AWS services on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterArn</i> The Amazon Resource Name (ARN) for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbClusterResourceId</i> The AWS Region-unique, immutable identifier for the DB cluster. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB cluster is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is <code>true</code>, the AWS KMS key identifier for the encrypted DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of virtual private cloud (VPC) security groups that the DB cluster belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterMembers</i> Provides the list of instances that make up the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i> Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master user name for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies whether the DB cluster has instances in multiple Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReaderEndpoint</i> The reader endpoint for the DB cluster. The reader endpoint for a DB cluster load balances connections across the Amazon DocumentDB replicas that are available in a DB cluster. As clients request new connections to the reader endpoint, Amazon DocumentDB distributes the connection requests among the Amazon DocumentDB replicas in the DB cluster. This functionality can help balance your read workload across multiple Amazon DocumentDB replicas in your DB cluster.  If a failover occurs, and the Amazon DocumentDB replica that you are connected to is promoted to be the primary instance, your connection is dropped. To continue sending your read workload to other Amazon DocumentDB replicas in the cluster, you can then reconnect to the reader endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint for the primary instance of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestRestorableTime</i> The earliest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the progress of the operation as a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the current state of this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group that is associated with the DB cluster, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroup</i> Specifies the name of the DB cluster parameter group for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Contains a user-supplied DB cluster identifier. This identifier is the unique key that identifies a DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of Amazon EC2 Availability Zones that instances in the DB cluster can be created in.<br/>        """
        pass

    def modify_db_cluster_parameter_group(self, Parameters: List[Parameter], DBClusterParameterGroupName: str) -> DBClusterParameterGroupNameMessage:
        """ Modifies the parameters of a DB cluster parameter group. To modify more than one parameter, submit a list of the following: <code>ParameterName</code>, <code>ParameterValue</code>, and <code>ApplyMethod</code>. A maximum of 20 parameters can be modified in a single request.  <note> Changes to dynamic parameters are applied immediately. Changes to static parameters require a reboot or maintenance window before the change can take effect. </note> <important> After you create a DB cluster parameter group, you should wait at least 5 minutes before creating your first DB cluster that uses that DB cluster parameter group as the default parameter group. This allows Amazon DocumentDB to fully complete the create action before the parameter group is used as the default for a new DB cluster. This step is especially important for parameters that are critical when creating the default database for a DB cluster, such as the character set for the default database defined by the <code>character_set_database</code> parameter. </important>

        :param self:
        :param Parameters:  A list of parameters in the DB cluster parameter group to modify.<br/>DBClusterParameterGroupName:  The name of the DB cluster parameter group to modify.<br/>
        :return: DBClusterParameterGroupName:  The name of a DB cluster parameter group. Constraints: <ul> <li> Must be from 1 to 255 letters or numbers. </li> <li> The first character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul> <note> This value is stored as a lowercase string. </note><br/>        """
        pass

    def modify_db_cluster_snapshot_attribute(self, AttributeName: str, DBClusterSnapshotIdentifier: str, ValuesToAdd: List[String]=None, ValuesToRemove: List[String]=None) -> ModifyDBClusterSnapshotAttributeResult:
        """Adds an attribute and values to, or removes an attribute and values from, a manual DB cluster snapshot. To share a manual DB cluster snapshot with other AWS accounts, specify <code>restore</code> as the <code>AttributeName</code>, and use the <code>ValuesToAdd</code> parameter to add a list of IDs of the AWS accounts that are authorized to restore the manual DB cluster snapshot. Use the value <code>all</code> to make the manual DB cluster snapshot public, which means that it can be copied or restored by all AWS accounts. Do not add the <code>all</code> value for any manual DB cluster snapshots that contain private information that you don't want available to all AWS accounts. If a manual DB cluster snapshot is encrypted, it can be shared, but only by specifying a list of authorized AWS account IDs for the <code>ValuesToAdd</code> parameter. You can't use <code>all</code> as a value for that parameter in this case.

        :param self:
        :param ValuesToRemove:  A list of DB cluster snapshot attributes to remove from the attribute specified by <code>AttributeName</code>. To remove authorization for other AWS accounts to copy or restore a manual DB cluster snapshot, set this list to include one or more AWS account identifiers. To remove authorization for any AWS account to copy or restore the DB cluster snapshot, set it to <code>all</code> . If you specify <code>all</code>, an AWS account whose account ID is explicitly added to the <code>restore</code> attribute can still copy or restore a manual DB cluster snapshot.<br/>ValuesToAdd:  A list of DB cluster snapshot attributes to add to the attribute specified by <code>AttributeName</code>. To authorize other AWS accounts to copy or restore a manual DB cluster snapshot, set this list to include one or more AWS account IDs. To make the manual DB cluster snapshot restorable by any AWS account, set it to <code>all</code>. Do not add the <code>all</code> value for any manual DB cluster snapshots that contain private information that you don't want to be available to all AWS accounts.<br/>AttributeName:  The name of the DB cluster snapshot attribute to modify. To manage authorization for other AWS accounts to copy or restore a manual DB cluster snapshot, set this value to <code>restore</code>.<br/>DBClusterSnapshotIdentifier:  The identifier for the DB cluster snapshot to modify the attributes for.<br/>
        :return: DBClusterSnapshotAttributesResult:  Detailed information about the attributes that are associated with a DB cluster snapshot.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotAttributes</i> The list of attributes and values for the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotIdentifier</i> The identifier of the DB cluster snapshot that the attributes apply to.<br/>        """
        pass

    def modify_db_instance(self, DBInstanceIdentifier: str, DBInstanceClass: str=None, ApplyImmediately: Boolean=None, PreferredMaintenanceWindow: str=None, AutoMinorVersionUpgrade: BooleanOptional=None, NewDBInstanceIdentifier: str=None, PromotionTier: int=None) -> ModifyDBInstanceResult:
        """Modifies settings for a DB instance. You can change one or more database configuration parameters by specifying these parameters and the new values in the request.

        :param self:
        :param PromotionTier:  A value that specifies the order in which an Amazon DocumentDB replica is promoted to the primary instance after a failure of the existing primary instance. Default: 1 Valid values: 0-15<br/>NewDBInstanceIdentifier:   The new DB instance identifier for the DB instance when renaming a DB instance. When you change the DB instance identifier, an instance reboot occurs immediately if you set <code>Apply Immediately</code> to <code>true</code>. It occurs during the next maintenance window if you set <code>Apply Immediately</code> to <code>false</code>. This value is stored as a lowercase string.  Constraints: <ul> <li> Must contain from 1 to 63 letters, numbers, or hyphens. </li> <li> The first character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul> Example: <code>mydbinstance</code> <br/>AutoMinorVersionUpgrade:  Indicates that minor version upgrades are applied automatically to the DB instance during the maintenance window. Changing this parameter doesn't result in an outage except in the following case, and the change is asynchronously applied as soon as possible. An outage results if this parameter is set to <code>true</code> during the maintenance window, and a newer minor version is available, and Amazon DocumentDB has enabled automatic patching for that engine version. <br/>PreferredMaintenanceWindow:  The weekly time range (in UTC) during which system maintenance can occur, which might result in an outage. Changing this parameter doesn't result in an outage except in the following situation, and the change is asynchronously applied as soon as possible. If there are pending actions that cause a reboot, and the maintenance window is changed to include the current time, changing this parameter causes a reboot of the DB instance. If you are moving this window to the current time, there must be at least 30 minutes between the current time and end of the window to ensure that pending changes are applied. Default: Uses existing setting. Format: <code>ddd:hh24:mi-ddd:hh24:mi</code>  Valid days: Mon, Tue, Wed, Thu, Fri, Sat, Sun Constraints: Must be at least 30 minutes.<br/>ApplyImmediately:  Specifies whether the modifications in this request and any pending modifications are asynchronously applied as soon as possible, regardless of the <code>PreferredMaintenanceWindow</code> setting for the DB instance.   If this parameter is set to <code>false</code>, changes to the DB instance are applied during the next maintenance window. Some parameter changes can cause an outage and are applied on the next reboot. Default: <code>false</code> <br/>DBInstanceClass:  The new compute and memory capacity of the DB instance; for example, <code>db.r5.large</code>. Not all DB instance classes are available in all AWS Regions.  If you modify the DB instance class, an outage occurs during the change. The change is applied during the next maintenance window, unless <code>ApplyImmediately</code> is specified as <code>true</code> for this request.  Default: Uses existing setting.<br/>DBInstanceIdentifier:  The DB instance identifier. This value is stored as a lowercase string. Constraints: <ul> <li> Must match the identifier of an existing <code>DBInstance</code>. </li> </ul><br/>
        :return: DBInstance:  Detailed information about a DB instance. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB instance is configured to export to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceArn</i> The Amazon Resource Name (ARN) for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PromotionTier</i> A value that specifies the order in which an Amazon DocumentDB replica is promoted to the primary instance after a failure of the existing primary instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbiResourceId</i> The AWS Region-unique, immutable identifier for the DB instance. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB instance is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i>  If <code>StorageEncrypted</code> is <code>true</code>, the AWS KMS key identifier for the encrypted DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB instance is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Contains the name of the DB cluster that the DB instance is a member of if the DB instance is a member of a DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusInfos</i> The status of a read replica. If the instance is not a read replica, this is blank.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> Specifies the availability options for the DB instance. A value of <code>true</code> specifies an internet-facing instance with a publicly resolvable DNS name, which resolves to a public IP address. A value of <code>false</code> specifies an internal instance with a DNS name that resolves to a private IP address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> Indicates that minor version patches are applied automatically.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> Specifies that changes to the DB instance are pending. This element is included only when changes are pending. Specific changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: Specifies that changes to the DB instance are pending. This element is included only when changes are pending. Specific changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group that is associated with the DB instance, including the name, description, and subnets in the subnet group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DBSubnetGroup</b>: Specifies information on the subnet group that is associated with the DB instance, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> Specifies the name of the Availability Zone that the DB instance is located in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security group elements that the DB instance belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i>  Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCreateTime</i> Provides the date and time that the DB instance was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: Specifies the connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceStatus</i> Specifies the current state of this database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i> Contains the name of the compute and memory capacity class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i> Contains a user-provided database identifier. This identifier is the unique key that identifies a DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupArn</i> The Amazon Resource Identifier (ARN) for the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> Detailed information about one or more subnets within a DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> Provides the status of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the virtual private cloud (VPC) ID of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupDescription</i> Provides the description of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The name of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The new DB subnet group for the DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> Specifies the identifier of the certificate authority (CA) certificate for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type to be associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i>  Contains the new <code>DBInstanceIdentifier</code> for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the new Provisioned IOPS value for the DB instance that will be applied or is currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> The license model for the DB instance. Valid values: <code>license-included</code>, <code>bring-your-own-license</code>, <code>general-public-license</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Indicates that the Single-AZ DB instance is to change to a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the pending number of days for which automated backups are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the pending port for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> Contains the pending or currently in-progress change of the master credentials for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i>  Contains the new <code>AllocatedStorage</code> size for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i>  Contains the new <code>DBInstanceClass</code> for the DB instance that will be applied or is currently being applied. <br/>A list of the log types whose configuration is still pending. These log types are in the process of being activated or deactivated.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingCloudwatchLogsExports</b>: A list of the log types whose configuration is still pending. These log types are in the process of being activated or deactivated.        """
        pass

    def modify_db_subnet_group(self, SubnetIds: List[String], DBSubnetGroupName: str, DBSubnetGroupDescription: str=None) -> ModifyDBSubnetGroupResult:
        """Modifies an existing DB subnet group. DB subnet groups must contain at least one subnet in at least two Availability Zones in the AWS Region.

        :param self:
        :param SubnetIds:  The Amazon EC2 subnet IDs for the DB subnet group.<br/>DBSubnetGroupDescription:  The description for the DB subnet group.<br/>DBSubnetGroupName:  The name for the DB subnet group. This value is stored as a lowercase string. You can't modify the default subnet group.  Constraints: Must match the name of an existing <code>DBSubnetGroup</code>. Must not be default. Example: <code>mySubnetgroup</code> <br/>
        :return: DBSubnetGroup:  Detailed information about a DB subnet group. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupArn</i> The Amazon Resource Identifier (ARN) for the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> Detailed information about one or more subnets within a DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> Provides the status of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the virtual private cloud (VPC) ID of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupDescription</i> Provides the description of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The name of the DB subnet group.<br/>        """
        pass

    def reboot_db_instance(self, DBInstanceIdentifier: str, ForceFailover: BooleanOptional=None) -> RebootDBInstanceResult:
        """You might need to reboot your DB instance, usually for maintenance reasons. For example, if you make certain changes, or if you change the DB cluster parameter group that is associated with the DB instance, you must reboot the instance for the changes to take effect.  Rebooting a DB instance restarts the database engine service. Rebooting a DB instance results in a momentary outage, during which the DB instance status is set to <i>rebooting</i>. 

        :param self:
        :param ForceFailover:   When <code>true</code>, the reboot is conducted through a Multi-AZ failover.  Constraint: You can't specify <code>true</code> if the instance is not configured for Multi-AZ.<br/>DBInstanceIdentifier:  The DB instance identifier. This parameter is stored as a lowercase string. Constraints: <ul> <li> Must match the identifier of an existing <code>DBInstance</code>. </li> </ul><br/>
        :return: DBInstance:  Detailed information about a DB instance. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB instance is configured to export to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceArn</i> The Amazon Resource Name (ARN) for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PromotionTier</i> A value that specifies the order in which an Amazon DocumentDB replica is promoted to the primary instance after a failure of the existing primary instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbiResourceId</i> The AWS Region-unique, immutable identifier for the DB instance. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB instance is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i>  If <code>StorageEncrypted</code> is <code>true</code>, the AWS KMS key identifier for the encrypted DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB instance is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Contains the name of the DB cluster that the DB instance is a member of if the DB instance is a member of a DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusInfos</i> The status of a read replica. If the instance is not a read replica, this is blank.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> Specifies the availability options for the DB instance. A value of <code>true</code> specifies an internet-facing instance with a publicly resolvable DNS name, which resolves to a public IP address. A value of <code>false</code> specifies an internal instance with a DNS name that resolves to a private IP address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> Indicates that minor version patches are applied automatically.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> Specifies that changes to the DB instance are pending. This element is included only when changes are pending. Specific changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: Specifies that changes to the DB instance are pending. This element is included only when changes are pending. Specific changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group that is associated with the DB instance, including the name, description, and subnets in the subnet group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DBSubnetGroup</b>: Specifies information on the subnet group that is associated with the DB instance, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> Specifies the name of the Availability Zone that the DB instance is located in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security group elements that the DB instance belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i>  Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCreateTime</i> Provides the date and time that the DB instance was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: Specifies the connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceStatus</i> Specifies the current state of this database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i> Contains the name of the compute and memory capacity class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i> Contains a user-provided database identifier. This identifier is the unique key that identifies a DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupArn</i> The Amazon Resource Identifier (ARN) for the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> Detailed information about one or more subnets within a DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> Provides the status of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the virtual private cloud (VPC) ID of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupDescription</i> Provides the description of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The name of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The new DB subnet group for the DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> Specifies the identifier of the certificate authority (CA) certificate for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type to be associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i>  Contains the new <code>DBInstanceIdentifier</code> for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the new Provisioned IOPS value for the DB instance that will be applied or is currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> The license model for the DB instance. Valid values: <code>license-included</code>, <code>bring-your-own-license</code>, <code>general-public-license</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Indicates that the Single-AZ DB instance is to change to a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the pending number of days for which automated backups are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the pending port for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> Contains the pending or currently in-progress change of the master credentials for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i>  Contains the new <code>AllocatedStorage</code> size for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i>  Contains the new <code>DBInstanceClass</code> for the DB instance that will be applied or is currently being applied. <br/>A list of the log types whose configuration is still pending. These log types are in the process of being activated or deactivated.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingCloudwatchLogsExports</b>: A list of the log types whose configuration is still pending. These log types are in the process of being activated or deactivated.        """
        pass

    def remove_tags_from_resource(self, TagKeys: List[String], ResourceName: str):
        """Removes metadata tags from an Amazon DocumentDB resource.

        :param self:
        :param TagKeys:  The tag key (name) of the tag to be removed.<br/>ResourceName:  The Amazon DocumentDB resource that the tags are removed from. This value is an Amazon Resource Name (ARN).<br/>
        :return:         """
        pass

    def reset_db_cluster_parameter_group(self, DBClusterParameterGroupName: str, ResetAllParameters: Boolean=None, Parameters: List[Parameter]=None) -> DBClusterParameterGroupNameMessage:
        """ Modifies the parameters of a DB cluster parameter group to the default value. To reset specific parameters, submit a list of the following: <code>ParameterName</code> and <code>ApplyMethod</code>. To reset the entire DB cluster parameter group, specify the <code>DBClusterParameterGroupName</code> and <code>ResetAllParameters</code> parameters.   When you reset the entire group, dynamic parameters are updated immediately and static parameters are set to <code>pending-reboot</code> to take effect on the next DB instance reboot.

        :param self:
        :param Parameters:  A list of parameter names in the DB cluster parameter group to reset to the default values. You can't use this parameter if the <code>ResetAllParameters</code> parameter is set to <code>true</code>.<br/>ResetAllParameters:  A value that is set to <code>true</code> to reset all parameters in the DB cluster parameter group to their default values, and <code>false</code> otherwise. You can't use this parameter if there is a list of parameter names specified for the <code>Parameters</code> parameter.<br/>DBClusterParameterGroupName:  The name of the DB cluster parameter group to reset.<br/>
        :return: DBClusterParameterGroupName:  The name of a DB cluster parameter group. Constraints: <ul> <li> Must be from 1 to 255 letters or numbers. </li> <li> The first character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul> <note> This value is stored as a lowercase string. </note><br/>        """
        pass

    def restore_db_cluster_from_snapshot(self, Engine: str, SnapshotIdentifier: str, DBClusterIdentifier: str, AvailabilityZones: List[String]=None, EngineVersion: str=None, Port: int=None, DBSubnetGroupName: str=None, VpcSecurityGroupIds: List[String]=None, Tags: List[Tag]=None, KmsKeyId: str=None, EnableCloudwatchLogsExports: List[String]=None, DeletionProtection: BooleanOptional=None) -> RestoreDBClusterFromSnapshotResult:
        """Creates a new DB cluster from a DB snapshot or DB cluster snapshot. If a DB snapshot is specified, the target DB cluster is created from the source DB snapshot with a default configuration and default security group. If a DB cluster snapshot is specified, the target DB cluster is created from the source DB cluster restore point with the same configuration as the original source DB cluster, except that the new DB cluster is created with the default security group.

        :param self:
        :param DeletionProtection:  Specifies whether this cluster can be deleted. If <code>DeletionProtection</code> is enabled, the cluster cannot be deleted unless it is modified and <code>DeletionProtection</code> is disabled. <code>DeletionProtection</code> protects clusters from being accidentally deleted.<br/>EnableCloudwatchLogsExports:  A list of log types that must be enabled for exporting to Amazon CloudWatch Logs.<br/>KmsKeyId:  The AWS KMS key identifier to use when restoring an encrypted DB cluster from a DB snapshot or DB cluster snapshot. The AWS KMS key identifier is the Amazon Resource Name (ARN) for the AWS KMS encryption key. If you are restoring a DB cluster with the same AWS account that owns the AWS KMS encryption key used to encrypt the new DB cluster, then you can use the AWS KMS key alias instead of the ARN for the AWS KMS encryption key. If you do not specify a value for the <code>KmsKeyId</code> parameter, then the following occurs: <ul> <li> If the DB snapshot or DB cluster snapshot in <code>SnapshotIdentifier</code> is encrypted, then the restored DB cluster is encrypted using the AWS KMS key that was used to encrypt the DB snapshot or the DB cluster snapshot. </li> <li> If the DB snapshot or the DB cluster snapshot in <code>SnapshotIdentifier</code> is not encrypted, then the restored DB cluster is not encrypted. </li> </ul><br/>Tags:  The tags to be assigned to the restored DB cluster.<br/>VpcSecurityGroupIds:  A list of virtual private cloud (VPC) security groups that the new DB cluster will belong to.<br/>DBSubnetGroupName:  The name of the DB subnet group to use for the new DB cluster. Constraints: If provided, must match the name of an existing <code>DBSubnetGroup</code>. Example: <code>mySubnetgroup</code> <br/>Port:  The port number on which the new DB cluster accepts connections. Constraints: Must be a value from <code>1150</code> to <code>65535</code>. Default: The same port as the original DB cluster.<br/>EngineVersion:  The version of the database engine to use for the new DB cluster.<br/>Engine:  The database engine to use for the new DB cluster. Default: The same as source. Constraint: Must be compatible with the engine of the source.<br/>SnapshotIdentifier:  The identifier for the DB snapshot or DB cluster snapshot to restore from. You can use either the name or the Amazon Resource Name (ARN) to specify a DB cluster snapshot. However, you can use only the ARN to specify a DB snapshot. Constraints: <ul> <li> Must match the identifier of an existing snapshot. </li> </ul><br/>DBClusterIdentifier:  The name of the DB cluster to create from the DB snapshot or DB cluster snapshot. This parameter isn't case sensitive. Constraints: <ul> <li> Must contain from 1 to 63 letters, numbers, or hyphens. </li> <li> The first character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul> Example: <code>my-snapshot-id</code> <br/>AvailabilityZones:  Provides the list of Amazon EC2 Availability Zones that instances in the restored DB cluster can be created in.<br/>
        :return: DBCluster:  Detailed information about a DB cluster. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i> Specifies whether this cluster can be deleted. If <code>DeletionProtection</code> is enabled, the cluster cannot be deleted unless it is modified and <code>DeletionProtection</code> is disabled. <code>DeletionProtection</code> protects clusters from being accidentally deleted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB cluster is configured to export to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i> Provides a list of the AWS Identity and Access Management (IAM) roles that are associated with the DB cluster. IAM roles that are associated with a DB cluster grant permission for the DB cluster to access other AWS services on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterArn</i> The Amazon Resource Name (ARN) for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbClusterResourceId</i> The AWS Region-unique, immutable identifier for the DB cluster. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB cluster is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is <code>true</code>, the AWS KMS key identifier for the encrypted DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of virtual private cloud (VPC) security groups that the DB cluster belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterMembers</i> Provides the list of instances that make up the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i> Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master user name for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies whether the DB cluster has instances in multiple Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReaderEndpoint</i> The reader endpoint for the DB cluster. The reader endpoint for a DB cluster load balances connections across the Amazon DocumentDB replicas that are available in a DB cluster. As clients request new connections to the reader endpoint, Amazon DocumentDB distributes the connection requests among the Amazon DocumentDB replicas in the DB cluster. This functionality can help balance your read workload across multiple Amazon DocumentDB replicas in your DB cluster.  If a failover occurs, and the Amazon DocumentDB replica that you are connected to is promoted to be the primary instance, your connection is dropped. To continue sending your read workload to other Amazon DocumentDB replicas in the cluster, you can then reconnect to the reader endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint for the primary instance of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestRestorableTime</i> The earliest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the progress of the operation as a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the current state of this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group that is associated with the DB cluster, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroup</i> Specifies the name of the DB cluster parameter group for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Contains a user-supplied DB cluster identifier. This identifier is the unique key that identifies a DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of Amazon EC2 Availability Zones that instances in the DB cluster can be created in.<br/>        """
        pass

    def restore_db_cluster_to_point_in_time(self, SourceDBClusterIdentifier: str, DBClusterIdentifier: str, RestoreToTime: TStamp=None, UseLatestRestorableTime: Boolean=None, Port: int=None, DBSubnetGroupName: str=None, VpcSecurityGroupIds: List[String]=None, Tags: List[Tag]=None, KmsKeyId: str=None, EnableCloudwatchLogsExports: List[String]=None, DeletionProtection: BooleanOptional=None) -> RestoreDBClusterToPointInTimeResult:
        """Restores a DB cluster to an arbitrary point in time. Users can restore to any point in time before <code>LatestRestorableTime</code> for up to <code>BackupRetentionPeriod</code> days. The target DB cluster is created from the source DB cluster with the same configuration as the original DB cluster, except that the new DB cluster is created with the default DB security group. 

        :param self:
        :param DeletionProtection:  Specifies whether this cluster can be deleted. If <code>DeletionProtection</code> is enabled, the cluster cannot be deleted unless it is modified and <code>DeletionProtection</code> is disabled. <code>DeletionProtection</code> protects clusters from being accidentally deleted.<br/>EnableCloudwatchLogsExports:  A list of log types that must be enabled for exporting to Amazon CloudWatch Logs.<br/>KmsKeyId:  The AWS KMS key identifier to use when restoring an encrypted DB cluster from an encrypted DB cluster. The AWS KMS key identifier is the Amazon Resource Name (ARN) for the AWS KMS encryption key. If you are restoring a DB cluster with the same AWS account that owns the AWS KMS encryption key used to encrypt the new DB cluster, then you can use the AWS KMS key alias instead of the ARN for the AWS KMS encryption key. You can restore to a new DB cluster and encrypt the new DB cluster with an AWS KMS key that is different from the AWS KMS key used to encrypt the source DB cluster. The new DB cluster is encrypted with the AWS KMS key identified by the <code>KmsKeyId</code> parameter. If you do not specify a value for the <code>KmsKeyId</code> parameter, then the following occurs: <ul> <li> If the DB cluster is encrypted, then the restored DB cluster is encrypted using the AWS KMS key that was used to encrypt the source DB cluster. </li> <li> If the DB cluster is not encrypted, then the restored DB cluster is not encrypted. </li> </ul> If <code>DBClusterIdentifier</code> refers to a DB cluster that is not encrypted, then the restore request is rejected.<br/>Tags:  The tags to be assigned to the restored DB cluster.<br/>VpcSecurityGroupIds:  A list of VPC security groups that the new DB cluster belongs to.<br/>DBSubnetGroupName:  The DB subnet group name to use for the new DB cluster. Constraints: If provided, must match the name of an existing <code>DBSubnetGroup</code>. Example: <code>mySubnetgroup</code> <br/>Port:  The port number on which the new DB cluster accepts connections. Constraints: Must be a value from <code>1150</code> to <code>65535</code>.  Default: The default port for the engine.<br/>UseLatestRestorableTime:  A value that is set to <code>true</code> to restore the DB cluster to the latest restorable backup time, and <code>false</code> otherwise.  Default: <code>false</code>  Constraints: Cannot be specified if the <code>RestoreToTime</code> parameter is provided.<br/>RestoreToTime:  The date and time to restore the DB cluster to. Valid values: A time in Universal Coordinated Time (UTC) format. Constraints: <ul> <li> Must be before the latest restorable time for the DB instance. </li> <li> Must be specified if the <code>UseLatestRestorableTime</code> parameter is not provided. </li> <li> Cannot be specified if the <code>UseLatestRestorableTime</code> parameter is <code>true</code>. </li> <li> Cannot be specified if the <code>RestoreType</code> parameter is <code>copy-on-write</code>. </li> </ul> Example: <code>2015-03-07T23:45:00Z</code> <br/>SourceDBClusterIdentifier:  The identifier of the source DB cluster from which to restore. Constraints: <ul> <li> Must match the identifier of an existing <code>DBCluster</code>. </li> </ul><br/>DBClusterIdentifier:  The name of the new DB cluster to be created. Constraints: <ul> <li> Must contain from 1 to 63 letters, numbers, or hyphens. </li> <li> The first character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul><br/>
        :return: DBCluster:  Detailed information about a DB cluster. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i> Specifies whether this cluster can be deleted. If <code>DeletionProtection</code> is enabled, the cluster cannot be deleted unless it is modified and <code>DeletionProtection</code> is disabled. <code>DeletionProtection</code> protects clusters from being accidentally deleted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB cluster is configured to export to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i> Provides a list of the AWS Identity and Access Management (IAM) roles that are associated with the DB cluster. IAM roles that are associated with a DB cluster grant permission for the DB cluster to access other AWS services on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterArn</i> The Amazon Resource Name (ARN) for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbClusterResourceId</i> The AWS Region-unique, immutable identifier for the DB cluster. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB cluster is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is <code>true</code>, the AWS KMS key identifier for the encrypted DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of virtual private cloud (VPC) security groups that the DB cluster belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterMembers</i> Provides the list of instances that make up the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i> Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master user name for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies whether the DB cluster has instances in multiple Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReaderEndpoint</i> The reader endpoint for the DB cluster. The reader endpoint for a DB cluster load balances connections across the Amazon DocumentDB replicas that are available in a DB cluster. As clients request new connections to the reader endpoint, Amazon DocumentDB distributes the connection requests among the Amazon DocumentDB replicas in the DB cluster. This functionality can help balance your read workload across multiple Amazon DocumentDB replicas in your DB cluster.  If a failover occurs, and the Amazon DocumentDB replica that you are connected to is promoted to be the primary instance, your connection is dropped. To continue sending your read workload to other Amazon DocumentDB replicas in the cluster, you can then reconnect to the reader endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint for the primary instance of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestRestorableTime</i> The earliest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the progress of the operation as a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the current state of this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group that is associated with the DB cluster, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroup</i> Specifies the name of the DB cluster parameter group for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Contains a user-supplied DB cluster identifier. This identifier is the unique key that identifies a DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of Amazon EC2 Availability Zones that instances in the DB cluster can be created in.<br/>        """
        pass

    def start_db_cluster(self, DBClusterIdentifier: str) -> StartDBClusterResult:
        """Restarts the stopped cluster that is specified by <code>DBClusterIdentifier</code>. For more information, see <a href="https://docs.aws.amazon.com/documentdb/latest/developerguide/db-cluster-stop-start.html">Stopping and Starting an Amazon DocumentDB Cluster</a>.

        :param self:
        :param DBClusterIdentifier:  The identifier of the cluster to restart. Example: <code>docdb-2019-05-28-15-24-52</code> <br/>
        :return: DBCluster:  Detailed information about a DB cluster. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i> Specifies whether this cluster can be deleted. If <code>DeletionProtection</code> is enabled, the cluster cannot be deleted unless it is modified and <code>DeletionProtection</code> is disabled. <code>DeletionProtection</code> protects clusters from being accidentally deleted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB cluster is configured to export to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i> Provides a list of the AWS Identity and Access Management (IAM) roles that are associated with the DB cluster. IAM roles that are associated with a DB cluster grant permission for the DB cluster to access other AWS services on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterArn</i> The Amazon Resource Name (ARN) for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbClusterResourceId</i> The AWS Region-unique, immutable identifier for the DB cluster. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB cluster is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is <code>true</code>, the AWS KMS key identifier for the encrypted DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of virtual private cloud (VPC) security groups that the DB cluster belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterMembers</i> Provides the list of instances that make up the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i> Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master user name for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies whether the DB cluster has instances in multiple Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReaderEndpoint</i> The reader endpoint for the DB cluster. The reader endpoint for a DB cluster load balances connections across the Amazon DocumentDB replicas that are available in a DB cluster. As clients request new connections to the reader endpoint, Amazon DocumentDB distributes the connection requests among the Amazon DocumentDB replicas in the DB cluster. This functionality can help balance your read workload across multiple Amazon DocumentDB replicas in your DB cluster.  If a failover occurs, and the Amazon DocumentDB replica that you are connected to is promoted to be the primary instance, your connection is dropped. To continue sending your read workload to other Amazon DocumentDB replicas in the cluster, you can then reconnect to the reader endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint for the primary instance of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestRestorableTime</i> The earliest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the progress of the operation as a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the current state of this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group that is associated with the DB cluster, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroup</i> Specifies the name of the DB cluster parameter group for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Contains a user-supplied DB cluster identifier. This identifier is the unique key that identifies a DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of Amazon EC2 Availability Zones that instances in the DB cluster can be created in.<br/>        """
        pass

    def stop_db_cluster(self, DBClusterIdentifier: str) -> StopDBClusterResult:
        """Stops the running cluster that is specified by <code>DBClusterIdentifier</code>. The cluster must be in the <i>available</i> state. For more information, see <a href="https://docs.aws.amazon.com/documentdb/latest/developerguide/db-cluster-stop-start.html">Stopping and Starting an Amazon DocumentDB Cluster</a>.

        :param self:
        :param DBClusterIdentifier:  The identifier of the cluster to stop. Example: <code>docdb-2019-05-28-15-24-52</code> <br/>
        :return: DBCluster:  Detailed information about a DB cluster. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i> Specifies whether this cluster can be deleted. If <code>DeletionProtection</code> is enabled, the cluster cannot be deleted unless it is modified and <code>DeletionProtection</code> is disabled. <code>DeletionProtection</code> protects clusters from being accidentally deleted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB cluster is configured to export to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i> Provides a list of the AWS Identity and Access Management (IAM) roles that are associated with the DB cluster. IAM roles that are associated with a DB cluster grant permission for the DB cluster to access other AWS services on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterArn</i> The Amazon Resource Name (ARN) for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbClusterResourceId</i> The AWS Region-unique, immutable identifier for the DB cluster. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB cluster is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is <code>true</code>, the AWS KMS key identifier for the encrypted DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of virtual private cloud (VPC) security groups that the DB cluster belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterMembers</i> Provides the list of instances that make up the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i> Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master user name for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies whether the DB cluster has instances in multiple Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReaderEndpoint</i> The reader endpoint for the DB cluster. The reader endpoint for a DB cluster load balances connections across the Amazon DocumentDB replicas that are available in a DB cluster. As clients request new connections to the reader endpoint, Amazon DocumentDB distributes the connection requests among the Amazon DocumentDB replicas in the DB cluster. This functionality can help balance your read workload across multiple Amazon DocumentDB replicas in your DB cluster.  If a failover occurs, and the Amazon DocumentDB replica that you are connected to is promoted to be the primary instance, your connection is dropped. To continue sending your read workload to other Amazon DocumentDB replicas in the cluster, you can then reconnect to the reader endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint for the primary instance of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestRestorableTime</i> The earliest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the progress of the operation as a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the current state of this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group that is associated with the DB cluster, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroup</i> Specifies the name of the DB cluster parameter group for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Contains a user-supplied DB cluster identifier. This identifier is the unique key that identifies a DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of Amazon EC2 Availability Zones that instances in the DB cluster can be created in.<br/>        """
        pass



class DirectoryService(BaseClient):
    """<fullname>AWS Directory Service</fullname> AWS Directory Service is a web service that makes it easy for you to setup and run directories in the AWS cloud, or connect your AWS resources with an existing on-premises Microsoft Active Directory. This guide provides detailed information about AWS Directory Service operations, data types, parameters, and errors. For information about AWS Directory Services features, see <a href="https://aws.amazon.com/directoryservice/">AWS Directory Service</a> and the <a href="http://docs.aws.amazon.com/directoryservice/latest/admin-guide/what_is.html">AWS Directory Service Administration Guide</a>. <note> AWS provides SDKs that consist of libraries and sample code for various programming languages and platforms (Java, Ruby, .Net, iOS, Android, etc.). The SDKs provide a convenient way to create programmatic access to AWS Directory Service and other AWS services. For more information about the AWS SDKs, including how to download and install them, see <a href="http://aws.amazon.com/tools/">Tools for Amazon Web Services</a>. </note>"""


    class DescribeDomainControllersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_domain_controllers` for valid parameters"""
            pass
    class DescribeDirectoriesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_directories` for valid parameters"""
            pass
    class DescribeSharedDirectoriesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_shared_directories` for valid parameters"""
            pass
    class DescribeSnapshotsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_snapshots` for valid parameters"""
            pass
    class DescribeTrustsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_trusts` for valid parameters"""
            pass
    class ListIpRoutesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_ip_routes` for valid parameters"""
            pass
    class ListLogSubscriptionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_log_subscriptions` for valid parameters"""
            pass
    class ListSchemaExtensionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_schema_extensions` for valid parameters"""
            pass
    class ListTagsForResourcePaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_tags_for_resource` for valid parameters"""
            pass


    class AcceptSharedDirectoryRequest(Mapping):
        pass
    
    class AcceptSharedDirectoryResult(Mapping):
        pass
    
    class AddIpRoutesRequest(Mapping):
        pass
    
    class AddIpRoutesResult(Mapping):
        pass
    
    class IpRoute(Mapping):
        pass
    
    class UpdateSecurityGroupForDirectoryControllers(object):
        pass
    
    class AddTagsToResourceRequest(Mapping):
        pass
    
    class AddTagsToResourceResult(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CancelSchemaExtensionRequest(Mapping):
        pass
    
    class CancelSchemaExtensionResult(Mapping):
        pass
    
    class ConnectDirectoryRequest(Mapping):
        pass
    
    class ConnectDirectoryResult(Mapping):
        pass
    
    class DirectoryConnectSettings(Mapping):
        pass
    
    class CreateAliasRequest(Mapping):
        pass
    
    class CreateAliasResult(Mapping):
        pass
    
    class CreateComputerRequest(Mapping):
        pass
    
    class CreateComputerResult(Mapping):
        pass
    
    class Attribute(Mapping):
        pass
    
    class CreateConditionalForwarderRequest(Mapping):
        pass
    
    class CreateConditionalForwarderResult(Mapping):
        pass
    
    class IpAddr(object):
        pass
    
    class CreateDirectoryRequest(Mapping):
        pass
    
    class CreateDirectoryResult(Mapping):
        pass
    
    class DirectoryVpcSettings(Mapping):
        pass
    
    class CreateLogSubscriptionRequest(Mapping):
        pass
    
    class CreateLogSubscriptionResult(Mapping):
        pass
    
    class CreateMicrosoftADRequest(Mapping):
        pass
    
    class CreateMicrosoftADResult(Mapping):
        pass
    
    class CreateSnapshotRequest(Mapping):
        pass
    
    class CreateSnapshotResult(Mapping):
        pass
    
    class CreateTrustRequest(Mapping):
        pass
    
    class CreateTrustResult(Mapping):
        pass
    
    class DeleteConditionalForwarderRequest(Mapping):
        pass
    
    class DeleteConditionalForwarderResult(Mapping):
        pass
    
    class DeleteDirectoryRequest(Mapping):
        pass
    
    class DeleteDirectoryResult(Mapping):
        pass
    
    class DeleteLogSubscriptionRequest(Mapping):
        pass
    
    class DeleteLogSubscriptionResult(Mapping):
        pass
    
    class DeleteSnapshotRequest(Mapping):
        pass
    
    class DeleteSnapshotResult(Mapping):
        pass
    
    class DeleteTrustRequest(Mapping):
        pass
    
    class DeleteTrustResult(Mapping):
        pass
    
    class DeleteAssociatedConditionalForwarder(object):
        pass
    
    class DeregisterEventTopicRequest(Mapping):
        pass
    
    class DeregisterEventTopicResult(Mapping):
        pass
    
    class DescribeConditionalForwardersRequest(Mapping):
        pass
    
    class DescribeConditionalForwardersResult(Mapping):
        pass
    
    class RemoteDomainName(object):
        pass
    
    class DescribeDirectoriesRequest(Mapping):
        pass
    
    class DescribeDirectoriesResult(Mapping):
        pass
    
    class DirectoryId(object):
        pass
    
    class DescribeDomainControllersRequest(Mapping):
        pass
    
    class DescribeDomainControllersResult(Mapping):
        pass
    
    class DomainControllerId(object):
        pass
    
    class DescribeEventTopicsRequest(Mapping):
        pass
    
    class DescribeEventTopicsResult(Mapping):
        pass
    
    class TopicName(object):
        pass
    
    class DescribeSharedDirectoriesRequest(Mapping):
        pass
    
    class DescribeSharedDirectoriesResult(Mapping):
        pass
    
    class DescribeSnapshotsRequest(Mapping):
        pass
    
    class DescribeSnapshotsResult(Mapping):
        pass
    
    class SnapshotId(object):
        pass
    
    class DescribeTrustsRequest(Mapping):
        pass
    
    class DescribeTrustsResult(Mapping):
        pass
    
    class TrustId(object):
        pass
    
    class DisableRadiusRequest(Mapping):
        pass
    
    class DisableRadiusResult(Mapping):
        pass
    
    class DisableSsoRequest(Mapping):
        pass
    
    class DisableSsoResult(Mapping):
        pass
    
    class EnableRadiusRequest(Mapping):
        pass
    
    class EnableRadiusResult(Mapping):
        pass
    
    class RadiusSettings(Mapping):
        pass
    
    class EnableSsoRequest(Mapping):
        pass
    
    class EnableSsoResult(Mapping):
        pass
    
    class GetDirectoryLimitsRequest(Mapping):
        pass
    
    class GetDirectoryLimitsResult(Mapping):
        pass
    
    class GetSnapshotLimitsRequest(Mapping):
        pass
    
    class GetSnapshotLimitsResult(Mapping):
        pass
    
    class ListIpRoutesRequest(Mapping):
        pass
    
    class ListIpRoutesResult(Mapping):
        pass
    
    class ListLogSubscriptionsRequest(Mapping):
        pass
    
    class ListLogSubscriptionsResult(Mapping):
        pass
    
    class ListSchemaExtensionsRequest(Mapping):
        pass
    
    class ListSchemaExtensionsResult(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResult(Mapping):
        pass
    
    class RegisterEventTopicRequest(Mapping):
        pass
    
    class RegisterEventTopicResult(Mapping):
        pass
    
    class RejectSharedDirectoryRequest(Mapping):
        pass
    
    class RejectSharedDirectoryResult(Mapping):
        pass
    
    class RemoveIpRoutesRequest(Mapping):
        pass
    
    class RemoveIpRoutesResult(Mapping):
        pass
    
    class CidrIp(object):
        pass
    
    class RemoveTagsFromResourceRequest(Mapping):
        pass
    
    class RemoveTagsFromResourceResult(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class ResetUserPasswordRequest(Mapping):
        pass
    
    class ResetUserPasswordResult(Mapping):
        pass
    
    class RestoreFromSnapshotRequest(Mapping):
        pass
    
    class RestoreFromSnapshotResult(Mapping):
        pass
    
    class ShareDirectoryRequest(Mapping):
        pass
    
    class ShareDirectoryResult(Mapping):
        pass
    
    class ShareTarget(Mapping):
        pass
    
    class StartSchemaExtensionRequest(Mapping):
        pass
    
    class StartSchemaExtensionResult(Mapping):
        pass
    
    class CreateSnapshotBeforeSchemaExtension(object):
        pass
    
    class UnshareDirectoryRequest(Mapping):
        pass
    
    class UnshareDirectoryResult(Mapping):
        pass
    
    class UnshareTarget(Mapping):
        pass
    
    class UpdateConditionalForwarderRequest(Mapping):
        pass
    
    class UpdateConditionalForwarderResult(Mapping):
        pass
    
    class UpdateNumberOfDomainControllersRequest(Mapping):
        pass
    
    class UpdateNumberOfDomainControllersResult(Mapping):
        pass
    
    class UpdateRadiusRequest(Mapping):
        pass
    
    class UpdateRadiusResult(Mapping):
        pass
    
    class UpdateTrustRequest(Mapping):
        pass
    
    class UpdateTrustResult(Mapping):
        pass
    
    class VerifyTrustRequest(Mapping):
        pass
    
    class VerifyTrustResult(Mapping):
        pass
    
    def accept_shared_directory(self, SharedDirectoryId: str) -> AcceptSharedDirectoryResult:
        """Accepts a directory sharing request that was sent from the directory owner account.

        :param self:
        :param SharedDirectoryId:  Identifier of the shared directory in the directory consumer account. This identifier is different for each directory owner account. <br/>
        :return: SharedDirectory:  The shared directory in the directory consumer account.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdatedDateTime</i> The date and time that the shared directory was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedDateTime</i> The date and time that the shared directory was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ShareNotes</i> A directory share request that is sent by the directory owner to the directory consumer. The request includes a typed message to help the directory consumer administrator determine whether to approve or reject the share invitation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ShareStatus</i> Current directory status of the shared AWS Managed Microsoft AD directory.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SharedDirectoryId</i> Identifier of the shared directory in the directory consumer account. This identifier is different for each directory owner account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SharedAccountId</i> Identifier of the directory consumer account that has access to the shared directory (<code>OwnerDirectoryId</code>) in the directory owner account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ShareMethod</i> The method used when sharing a directory to determine whether the directory should be shared within your AWS organization (<code>ORGANIZATIONS</code>) or with any AWS account by sending a shared directory request (<code>HANDSHAKE</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerDirectoryId</i> Identifier of the directory in the directory owner account. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerAccountId</i> Identifier of the directory owner account, which contains the directory that has been shared to the consumer account.<br/>        """
        pass

    def add_ip_routes(self, IpRoutes: List[IpRoute], DirectoryId: str, UpdateSecurityGroupForDirectoryControllers: UpdateSecurityGroupForDirectoryControllers=None) -> AddIpRoutesResult:
        """If the DNS server for your on-premises domain uses a publicly addressable IP address, you must add a CIDR address block to correctly route traffic to and from your Microsoft AD on Amazon Web Services. <i>AddIpRoutes</i> adds this address block. You can also use <i>AddIpRoutes</i> to facilitate routing traffic that uses public IP ranges from your Microsoft AD on AWS to a peer VPC.  Before you call <i>AddIpRoutes</i>, ensure that all of the required permissions have been explicitly granted through a policy. For details about what permissions are required to run the <i>AddIpRoutes</i> operation, see <a href="http://docs.aws.amazon.com/directoryservice/latest/admin-guide/UsingWithDS_IAM_ResourcePermissions.html">AWS Directory Service API Permissions: Actions, Resources, and Conditions Reference</a>.

        :param self:
        :param UpdateSecurityGroupForDirectoryControllers:  If set to true, updates the inbound and outbound rules of the security group that has the description: "AWS created security group for <i>directory ID</i> directory controllers." Following are the new rules:  Inbound: <ul> <li> Type: Custom UDP Rule, Protocol: UDP, Range: 88, Source: 0.0.0.0/0 </li> <li> Type: Custom UDP Rule, Protocol: UDP, Range: 123, Source: 0.0.0.0/0 </li> <li> Type: Custom UDP Rule, Protocol: UDP, Range: 138, Source: 0.0.0.0/0 </li> <li> Type: Custom UDP Rule, Protocol: UDP, Range: 389, Source: 0.0.0.0/0 </li> <li> Type: Custom UDP Rule, Protocol: UDP, Range: 464, Source: 0.0.0.0/0 </li> <li> Type: Custom UDP Rule, Protocol: UDP, Range: 445, Source: 0.0.0.0/0 </li> <li> Type: Custom TCP Rule, Protocol: TCP, Range: 88, Source: 0.0.0.0/0 </li> <li> Type: Custom TCP Rule, Protocol: TCP, Range: 135, Source: 0.0.0.0/0 </li> <li> Type: Custom TCP Rule, Protocol: TCP, Range: 445, Source: 0.0.0.0/0 </li> <li> Type: Custom TCP Rule, Protocol: TCP, Range: 464, Source: 0.0.0.0/0 </li> <li> Type: Custom TCP Rule, Protocol: TCP, Range: 636, Source: 0.0.0.0/0 </li> <li> Type: Custom TCP Rule, Protocol: TCP, Range: 1024-65535, Source: 0.0.0.0/0 </li> <li> Type: Custom TCP Rule, Protocol: TCP, Range: 3268-33269, Source: 0.0.0.0/0 </li> <li> Type: DNS (UDP), Protocol: UDP, Range: 53, Source: 0.0.0.0/0 </li> <li> Type: DNS (TCP), Protocol: TCP, Range: 53, Source: 0.0.0.0/0 </li> <li> Type: LDAP, Protocol: TCP, Range: 389, Source: 0.0.0.0/0 </li> <li> Type: All ICMP, Protocol: All, Range: N/A, Source: 0.0.0.0/0 </li> </ul> <p/> Outbound: <ul> <li> Type: All traffic, Protocol: All, Range: All, Destination: 0.0.0.0/0 </li> </ul> These security rules impact an internal network interface that is not exposed publicly.<br/>IpRoutes:  IP address blocks, using CIDR format, of the traffic to route. This is often the IP address block of the DNS server used for your on-premises domain.<br/>DirectoryId:  Identifier (ID) of the directory to which to add the address block.<br/>
        :return:         """
        pass

    def add_tags_to_resource(self, Tags: List[Tag], ResourceId: str) -> AddTagsToResourceResult:
        """Adds or overwrites one or more tags for the specified directory. Each directory can have a maximum of 50 tags. Each tag consists of a key and optional value. Tag keys must be unique to each resource.

        :param self:
        :param Tags:  The tags to be assigned to the directory.<br/>ResourceId:  Identifier (ID) for the directory to which to add the tag.<br/>
        :return:         """
        pass

    def cancel_schema_extension(self, SchemaExtensionId: str, DirectoryId: str) -> CancelSchemaExtensionResult:
        """Cancels an in-progress schema extension to a Microsoft AD directory. Once a schema extension has started replicating to all domain controllers, the task can no longer be canceled. A schema extension can be canceled during any of the following states; <code>Initializing</code>, <code>CreatingSnapshot</code>, and <code>UpdatingSchema</code>.

        :param self:
        :param SchemaExtensionId:  The identifier of the schema extension that will be canceled.<br/>DirectoryId:  The identifier of the directory whose schema extension will be canceled.<br/>
        :return:         """
        pass

    def connect_directory(self, ConnectSettings: DirectoryConnectSettings, Size: str, Password: str, Name: str, ShortName: str=None, Description: str=None, Tags: List[Tag]=None) -> ConnectDirectoryResult:
        """Creates an AD Connector to connect to an on-premises directory. Before you call <code>ConnectDirectory</code>, ensure that all of the required permissions have been explicitly granted through a policy. For details about what permissions are required to run the <code>ConnectDirectory</code> operation, see <a href="http://docs.aws.amazon.com/directoryservice/latest/admin-guide/UsingWithDS_IAM_ResourcePermissions.html">AWS Directory Service API Permissions: Actions, Resources, and Conditions Reference</a>.

        :param self:
        :param Tags:  The tags to be assigned to AD Connector.<br/>ConnectSettings:  A <a>DirectoryConnectSettings</a> object that contains additional information for the operation.:<br/>Size:  The size of the directory.<br/>Description:  A textual description for the directory.<br/>Password:  The password for the on-premises user account.<br/>ShortName:  The NetBIOS name of the on-premises directory, such as <code>CORP</code>.<br/>Name:  The fully qualified name of the on-premises directory, such as <code>corp.example.com</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomerUserName</i> The user name of an account in the on-premises directory that is used to connect to the directory. This account must have the following permissions: <ul> <li> Read users and groups </li> <li> Create computer objects </li> <li> Join computers to the domain </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomerDnsIps</i> A list of one or more IP addresses of DNS servers or domain controllers in the on-premises directory.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> A list of subnet identifiers in the VPC in which the AD Connector is created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The identifier of the VPC in which the AD Connector is created.<br/>
        :return: DirectoryId:  The identifier of the new directory.<br/>        """
        pass

    def create_alias(self, Alias: str, DirectoryId: str) -> CreateAliasResult:
        """Creates an alias for a directory and assigns the alias to the directory. The alias is used to construct the access URL for the directory, such as <code>http://&lt;alias&gt;.awsapps.com</code>. <important> After an alias has been created, it cannot be deleted or reused, so this operation should only be used when absolutely necessary. </important>

        :param self:
        :param Alias:  The requested alias. The alias must be unique amongst all aliases in AWS. This operation throws an <code>EntityAlreadyExistsException</code> error if the alias already exists.<br/>DirectoryId:  The identifier of the directory for which to create the alias.<br/>
        :return: Alias:  The alias for the directory.<br/>DirectoryId:  The identifier of the directory.<br/>        """
        pass

    def create_computer(self, Password: str, ComputerName: str, DirectoryId: str, OrganizationalUnitDistinguishedName: str=None, ComputerAttributes: List[Attribute]=None) -> CreateComputerResult:
        """Creates a computer account in the specified directory, and joins the computer to the directory.

        :param self:
        :param ComputerAttributes:  An array of <a>Attribute</a> objects that contain any LDAP attributes to apply to the computer account.<br/>OrganizationalUnitDistinguishedName:  The fully-qualified distinguished name of the organizational unit to place the computer account in.<br/>Password:  A one-time password that is used to join the computer to the directory. You should generate a random, strong password to use for this parameter.<br/>ComputerName:  The name of the computer account.<br/>DirectoryId:  The identifier of the directory in which to create the computer account.<br/>
        :return: Computer:  A <a>Computer</a> object that represents the computer account.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ComputerAttributes</i> An array of <a>Attribute</a> objects containing the LDAP attributes that belong to the computer account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ComputerName</i> The computer name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ComputerId</i> The identifier of the computer.<br/>        """
        pass

    def create_conditional_forwarder(self, DnsIpAddrs: List[IpAddr], RemoteDomainName: str, DirectoryId: str) -> CreateConditionalForwarderResult:
        """Creates a conditional forwarder associated with your AWS directory. Conditional forwarders are required in order to set up a trust relationship with another domain. The conditional forwarder points to the trusted domain.

        :param self:
        :param DnsIpAddrs:  The IP addresses of the remote DNS server associated with RemoteDomainName.<br/>RemoteDomainName:  The fully qualified domain name (FQDN) of the remote domain with which you will set up a trust relationship.<br/>DirectoryId:  The directory ID of the AWS directory for which you are creating the conditional forwarder.<br/>
        :return:         """
        pass

    def create_directory(self, Size: str, Password: str, Name: str, ShortName: str=None, Description: str=None, VpcSettings: DirectoryVpcSettings=None, Tags: List[Tag]=None) -> CreateDirectoryResult:
        """Creates a Simple AD directory. Before you call <code>CreateDirectory</code>, ensure that all of the required permissions have been explicitly granted through a policy. For details about what permissions are required to run the <code>CreateDirectory</code> operation, see <a href="http://docs.aws.amazon.com/directoryservice/latest/admin-guide/UsingWithDS_IAM_ResourcePermissions.html">AWS Directory Service API Permissions: Actions, Resources, and Conditions Reference</a>.

        :param self:
        :param Tags:  The tags to be assigned to the Simple AD directory.<br/>VpcSettings:  A <a>DirectoryVpcSettings</a> object that contains additional information for the operation.:<br/>Size:  The size of the directory.<br/>Description:  A textual description for the directory.<br/>Password:  The password for the directory administrator. The directory creation process creates a directory administrator account with the user name <code>Administrator</code> and this password.<br/>ShortName:  The short name of the directory, such as <code>CORP</code>.<br/>Name:  The fully qualified name for the directory, such as <code>corp.example.com</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> The identifiers of the subnets for the directory servers. The two subnets must be in different Availability Zones. AWS Directory Service creates a directory server and a DNS server in each of these subnets.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The identifier of the VPC in which to create the directory.<br/>
        :return: DirectoryId:  The identifier of the directory that was created.<br/>        """
        pass

    def create_log_subscription(self, LogGroupName: str, DirectoryId: str) -> CreateLogSubscriptionResult:
        """Creates a subscription to forward real time Directory Service domain controller security logs to the specified CloudWatch log group in your AWS account.

        :param self:
        :param LogGroupName:  The name of the CloudWatch log group where the real-time domain controller logs are forwarded.<br/>DirectoryId:  Identifier (ID) of the directory to which you want to subscribe and receive real-time logs to your specified CloudWatch log group.<br/>
        :return:         """
        pass

    def create_microsoft_ad(self, VpcSettings: DirectoryVpcSettings, Password: str, Name: str, ShortName: str=None, Description: str=None, Edition: str=None, Tags: List[Tag]=None) -> CreateMicrosoftADResult:
        """Creates an AWS Managed Microsoft AD directory. Before you call <i>CreateMicrosoftAD</i>, ensure that all of the required permissions have been explicitly granted through a policy. For details about what permissions are required to run the <i>CreateMicrosoftAD</i> operation, see <a href="http://docs.aws.amazon.com/directoryservice/latest/admin-guide/UsingWithDS_IAM_ResourcePermissions.html">AWS Directory Service API Permissions: Actions, Resources, and Conditions Reference</a>.

        :param self:
        :param Tags:  The tags to be assigned to the AWS Managed Microsoft AD directory.<br/>Edition:  AWS Managed Microsoft AD is available in two editions: Standard and Enterprise. Enterprise is the default.<br/>VpcSettings:  Contains VPC information for the <a>CreateDirectory</a> or <a>CreateMicrosoftAD</a> operation.:<br/>Description:  A textual description for the directory. This label will appear on the AWS console <code>Directory Details</code> page after the directory is created.<br/>Password:  The password for the default administrative user named <code>Admin</code>.<br/>ShortName:  The NetBIOS name for your domain. A short identifier for your domain, such as <code>CORP</code>. If you don't specify a NetBIOS name, it will default to the first part of your directory DNS. For example, <code>CORP</code> for the directory DNS <code>corp.example.com</code>. <br/>Name:  The fully qualified domain name for the directory, such as <code>corp.example.com</code>. This name will resolve inside your VPC only. It does not need to be publicly resolvable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> The identifiers of the subnets for the directory servers. The two subnets must be in different Availability Zones. AWS Directory Service creates a directory server and a DNS server in each of these subnets.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The identifier of the VPC in which to create the directory.<br/>
        :return: DirectoryId:  The identifier of the directory that was created.<br/>        """
        pass

    def create_snapshot(self, DirectoryId: str, Name: str=None) -> CreateSnapshotResult:
        """Creates a snapshot of a Simple AD or Microsoft AD directory in the AWS cloud. <note> You cannot take snapshots of AD Connector directories. </note>

        :param self:
        :param Name:  The descriptive name to apply to the snapshot.<br/>DirectoryId:  The identifier of the directory of which to take a snapshot.<br/>
        :return: SnapshotId:  The identifier of the snapshot that was created.<br/>        """
        pass

    def create_trust(self, TrustDirection: str, TrustPassword: str, RemoteDomainName: str, DirectoryId: str, TrustType: str=None, ConditionalForwarderIpAddrs: List[IpAddr]=None, SelectiveAuth: str=None) -> CreateTrustResult:
        """AWS Directory Service for Microsoft Active Directory allows you to configure trust relationships. For example, you can establish a trust between your AWS Managed Microsoft AD directory, and your existing on-premises Microsoft Active Directory. This would allow you to provide users and groups access to resources in either domain, with a single set of credentials. This action initiates the creation of the AWS side of a trust relationship between an AWS Managed Microsoft AD directory and an external domain. You can create either a forest trust or an external trust.

        :param self:
        :param SelectiveAuth:  Optional parameter to enable selective authentication for the trust.<br/>ConditionalForwarderIpAddrs:  The IP addresses of the remote DNS server associated with RemoteDomainName.<br/>TrustType:  The trust relationship type. <code>Forest</code> is the default.<br/>TrustDirection:  The direction of the trust relationship.<br/>TrustPassword:  The trust password. The must be the same password that was used when creating the trust relationship on the external domain.<br/>RemoteDomainName:  The Fully Qualified Domain Name (FQDN) of the external domain for which to create the trust relationship.<br/>DirectoryId:  The Directory ID of the AWS Managed Microsoft AD directory for which to establish the trust relationship.<br/>
        :return: TrustId:  A unique identifier for the trust relationship that was created.<br/>        """
        pass

    def delete_conditional_forwarder(self, RemoteDomainName: str, DirectoryId: str) -> DeleteConditionalForwarderResult:
        """Deletes a conditional forwarder that has been set up for your AWS directory.

        :param self:
        :param RemoteDomainName:  The fully qualified domain name (FQDN) of the remote domain with which you are deleting the conditional forwarder.<br/>DirectoryId:  The directory ID for which you are deleting the conditional forwarder.<br/>
        :return:         """
        pass

    def delete_directory(self, DirectoryId: str) -> DeleteDirectoryResult:
        """Deletes an AWS Directory Service directory. Before you call <code>DeleteDirectory</code>, ensure that all of the required permissions have been explicitly granted through a policy. For details about what permissions are required to run the <code>DeleteDirectory</code> operation, see <a href="http://docs.aws.amazon.com/directoryservice/latest/admin-guide/UsingWithDS_IAM_ResourcePermissions.html">AWS Directory Service API Permissions: Actions, Resources, and Conditions Reference</a>.

        :param self:
        :param DirectoryId:  The identifier of the directory to delete.<br/>
        :return: DirectoryId:  The directory identifier.<br/>        """
        pass

    def delete_log_subscription(self, DirectoryId: str) -> DeleteLogSubscriptionResult:
        """Deletes the specified log subscription.

        :param self:
        :param DirectoryId:  Identifier (ID) of the directory whose log subscription you want to delete.<br/>
        :return:         """
        pass

    def delete_snapshot(self, SnapshotId: str) -> DeleteSnapshotResult:
        """Deletes a directory snapshot.

        :param self:
        :param SnapshotId:  The identifier of the directory snapshot to be deleted.<br/>
        :return: SnapshotId:  The identifier of the directory snapshot that was deleted.<br/>        """
        pass

    def delete_trust(self, TrustId: str, DeleteAssociatedConditionalForwarder: DeleteAssociatedConditionalForwarder=None) -> DeleteTrustResult:
        """Deletes an existing trust relationship between your AWS Managed Microsoft AD directory and an external domain.

        :param self:
        :param DeleteAssociatedConditionalForwarder:  Delete a conditional forwarder as part of a DeleteTrustRequest.<br/>TrustId:  The Trust ID of the trust relationship to be deleted.<br/>
        :return: TrustId:  The Trust ID of the trust relationship that was deleted.<br/>        """
        pass

    def deregister_event_topic(self, TopicName: str, DirectoryId: str) -> DeregisterEventTopicResult:
        """Removes the specified directory as a publisher to the specified SNS topic.

        :param self:
        :param TopicName:  The name of the SNS topic from which to remove the directory as a publisher.<br/>DirectoryId:  The Directory ID to remove as a publisher. This directory will no longer send messages to the specified SNS topic.<br/>
        :return:         """
        pass

    def describe_conditional_forwarders(self, DirectoryId: str, RemoteDomainNames: List[RemoteDomainName]=None) -> DescribeConditionalForwardersResult:
        """Obtains information about the conditional forwarders for this account. If no input parameters are provided for RemoteDomainNames, this request describes all conditional forwarders for the specified directory ID.

        :param self:
        :param RemoteDomainNames:  The fully qualified domain names (FQDN) of the remote domains for which to get the list of associated conditional forwarders. If this member is null, all conditional forwarders are returned.<br/>DirectoryId:  The directory ID for which to get the list of associated conditional forwarders.<br/>
        :return: ConditionalForwarders:  The list of conditional forwarders that have been created.<br/>        """
        pass

    def describe_directories(self, DirectoryIds: List[DirectoryId]=None, NextToken: str=None, Limit: int=None) -> DescribeDirectoriesResult:
        """Obtains information about the directories that belong to this account. You can retrieve information about specific directories by passing the directory identifiers in the <code>DirectoryIds</code> parameter. Otherwise, all directories that belong to the current account are returned. This operation supports pagination with the use of the <code>NextToken</code> request and response parameters. If more results are available, the <code>DescribeDirectoriesResult.NextToken</code> member contains a token that you pass in the next call to <a>DescribeDirectories</a> to retrieve the next set of items. You can also specify a maximum number of return results with the <code>Limit</code> parameter.

        :param self:
        :param Limit:  The maximum number of items to return. If this value is zero, the maximum number of items is specified by the limitations of the operation.<br/>NextToken:  The <code>DescribeDirectoriesResult.NextToken</code> value from a previous call to <a>DescribeDirectories</a>. Pass null if this is the first call.<br/>DirectoryIds:  A list of identifiers of the directories for which to obtain the information. If this member is null, all directories that belong to the current account are returned. An empty list results in an <code>InvalidParameterException</code> being thrown.<br/>
        :return: NextToken:  If not null, more results are available. Pass this value for the <code>NextToken</code> parameter in a subsequent call to <a>DescribeDirectories</a> to retrieve the next set of items.<br/>DirectoryDescriptions:  The list of <a>DirectoryDescription</a> objects that were retrieved. It is possible that this list contains less than the number of items specified in the <code>Limit</code> member of the request. This occurs if there are less than the requested number of items left to retrieve, or if the limitations of the operation have been exceeded.<br/>        """
        pass

    def describe_domain_controllers(self, DirectoryId: str, DomainControllerIds: List[DomainControllerId]=None, NextToken: str=None, Limit: int=None) -> DescribeDomainControllersResult:
        """Provides information about any domain controllers in your directory.

        :param self:
        :param Limit:  The maximum number of items to return.<br/>NextToken:  The <i>DescribeDomainControllers.NextToken</i> value from a previous call to <a>DescribeDomainControllers</a>. Pass null if this is the first call. <br/>DomainControllerIds:  A list of identifiers for the domain controllers whose information will be provided.<br/>DirectoryId:  Identifier of the directory for which to retrieve the domain controller information.<br/>
        :return: NextToken:  If not null, more results are available. Pass this value for the <code>NextToken</code> parameter in a subsequent call to <a>DescribeDomainControllers</a> retrieve the next set of items.<br/>DomainControllers:  List of the <a>DomainController</a> objects that were retrieved.<br/>        """
        pass

    def describe_event_topics(self, DirectoryId: str=None, TopicNames: List[TopicName]=None) -> DescribeEventTopicsResult:
        """Obtains information about which SNS topics receive status messages from the specified directory. If no input parameters are provided, such as DirectoryId or TopicName, this request describes all of the associations in the account.

        :param self:
        :param TopicNames:  A list of SNS topic names for which to obtain the information. If this member is null, all associations for the specified Directory ID are returned. An empty list results in an <code>InvalidParameterException</code> being thrown.<br/>DirectoryId:  The Directory ID for which to get the list of associated SNS topics. If this member is null, associations for all Directory IDs are returned.<br/>
        :return: EventTopics:  A list of SNS topic names that receive status messages from the specified Directory ID.<br/>        """
        pass

    def describe_shared_directories(self, OwnerDirectoryId: str, SharedDirectoryIds: List[DirectoryId]=None, NextToken: str=None, Limit: int=None) -> DescribeSharedDirectoriesResult:
        """Returns the shared directories in your account. 

        :param self:
        :param Limit:  The number of shared directories to return in the response object.<br/>NextToken:  The <code>DescribeSharedDirectoriesResult.NextToken</code> value from a previous call to <a>DescribeSharedDirectories</a>. Pass null if this is the first call. <br/>SharedDirectoryIds:  A list of identifiers of all shared directories in your account. <br/>OwnerDirectoryId:  Returns the identifier of the directory in the directory owner account. <br/>
        :return: NextToken:  If not null, token that indicates that more results are available. Pass this value for the <code>NextToken</code> parameter in a subsequent call to <a>DescribeSharedDirectories</a> to retrieve the next set of items.<br/>SharedDirectories:  A list of all shared directories in your account.<br/>        """
        pass

    def describe_snapshots(self, DirectoryId: str=None, SnapshotIds: List[SnapshotId]=None, NextToken: str=None, Limit: int=None) -> DescribeSnapshotsResult:
        """Obtains information about the directory snapshots that belong to this account. This operation supports pagination with the use of the <i>NextToken</i> request and response parameters. If more results are available, the <i>DescribeSnapshots.NextToken</i> member contains a token that you pass in the next call to <a>DescribeSnapshots</a> to retrieve the next set of items. You can also specify a maximum number of return results with the <i>Limit</i> parameter.

        :param self:
        :param Limit:  The maximum number of objects to return.<br/>NextToken:  The <i>DescribeSnapshotsResult.NextToken</i> value from a previous call to <a>DescribeSnapshots</a>. Pass null if this is the first call.<br/>SnapshotIds:  A list of identifiers of the snapshots to obtain the information for. If this member is null or empty, all snapshots are returned using the <i>Limit</i> and <i>NextToken</i> members.<br/>DirectoryId:  The identifier of the directory for which to retrieve snapshot information.<br/>
        :return: NextToken:  If not null, more results are available. Pass this value in the <i>NextToken</i> member of a subsequent call to <a>DescribeSnapshots</a>.<br/>Snapshots:  The list of <a>Snapshot</a> objects that were retrieved. It is possible that this list contains less than the number of items specified in the <i>Limit</i> member of the request. This occurs if there are less than the requested number of items left to retrieve, or if the limitations of the operation have been exceeded.<br/>        """
        pass

    def describe_trusts(self, DirectoryId: str=None, TrustIds: List[TrustId]=None, NextToken: str=None, Limit: int=None) -> DescribeTrustsResult:
        """Obtains information about the trust relationships for this account. If no input parameters are provided, such as DirectoryId or TrustIds, this request describes all the trust relationships belonging to the account.

        :param self:
        :param Limit:  The maximum number of objects to return.<br/>NextToken:  The <i>DescribeTrustsResult.NextToken</i> value from a previous call to <a>DescribeTrusts</a>. Pass null if this is the first call.<br/>TrustIds:  A list of identifiers of the trust relationships for which to obtain the information. If this member is null, all trust relationships that belong to the current account are returned. An empty list results in an <code>InvalidParameterException</code> being thrown.<br/>DirectoryId:  The Directory ID of the AWS directory that is a part of the requested trust relationship.<br/>
        :return: NextToken:  If not null, more results are available. Pass this value for the <i>NextToken</i> parameter in a subsequent call to <a>DescribeTrusts</a> to retrieve the next set of items.<br/>Trusts:  The list of Trust objects that were retrieved. It is possible that this list contains less than the number of items specified in the <i>Limit</i> member of the request. This occurs if there are less than the requested number of items left to retrieve, or if the limitations of the operation have been exceeded.<br/>        """
        pass

    def disable_radius(self, DirectoryId: str) -> DisableRadiusResult:
        """Disables multi-factor authentication (MFA) with the Remote Authentication Dial In User Service (RADIUS) server for an AD Connector or Microsoft AD directory.

        :param self:
        :param DirectoryId:  The identifier of the directory for which to disable MFA.<br/>
        :return:         """
        pass

    def disable_sso(self, DirectoryId: str, UserName: str=None, Password: str=None) -> DisableSsoResult:
        """Disables single-sign on for a directory.

        :param self:
        :param Password:  The password of an alternate account to use to disable single-sign on. This is only used for AD Connector directories. For more information, see the <i>UserName</i> parameter.<br/>UserName:  The username of an alternate account to use to disable single-sign on. This is only used for AD Connector directories. This account must have privileges to remove a service principal name. If the AD Connector service account does not have privileges to remove a service principal name, you can specify an alternate account with the <i>UserName</i> and <i>Password</i> parameters. These credentials are only used to disable single sign-on and are not stored by the service. The AD Connector service account is not changed.<br/>DirectoryId:  The identifier of the directory for which to disable single-sign on.<br/>
        :return:         """
        pass

    def enable_radius(self, RadiusSettings: RadiusSettings, DirectoryId: str) -> EnableRadiusResult:
        """Enables multi-factor authentication (MFA) with the Remote Authentication Dial In User Service (RADIUS) server for an AD Connector or Microsoft AD directory.

        :param self:
        :param RadiusSettings:  A <a>RadiusSettings</a> object that contains information about the RADIUS server.:<br/>DirectoryId:  The identifier of the directory for which to enable MFA.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UseSameUsername</i> Not currently used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayLabel</i> Not currently used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthenticationProtocol</i> The protocol specified for your RADIUS endpoints.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SharedSecret</i> Required for enabling RADIUS on the directory.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RadiusRetries</i> The maximum number of times that communication with the RADIUS server is attempted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RadiusTimeout</i> The amount of time, in seconds, to wait for the RADIUS server to respond.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RadiusPort</i> The port that your RADIUS server is using for communications. Your on-premises network must allow inbound traffic over this port from the AWS Directory Service servers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RadiusServers</i> An array of strings that contains the IP addresses of the RADIUS server endpoints, or the IP addresses of your RADIUS server load balancer.<br/>
        :return:         """
        pass

    def enable_sso(self, DirectoryId: str, UserName: str=None, Password: str=None) -> EnableSsoResult:
        """Enables single sign-on for a directory.

        :param self:
        :param Password:  The password of an alternate account to use to enable single-sign on. This is only used for AD Connector directories. For more information, see the <i>UserName</i> parameter.<br/>UserName:  The username of an alternate account to use to enable single-sign on. This is only used for AD Connector directories. This account must have privileges to add a service principal name. If the AD Connector service account does not have privileges to add a service principal name, you can specify an alternate account with the <i>UserName</i> and <i>Password</i> parameters. These credentials are only used to enable single sign-on and are not stored by the service. The AD Connector service account is not changed.<br/>DirectoryId:  The identifier of the directory for which to enable single-sign on.<br/>
        :return:         """
        pass

    def get_directory_limits(self) -> GetDirectoryLimitsResult:
        """Obtains directory limit information for the current region.

        :param self:
        :param 
        :return: DirectoryLimits:  A <a>DirectoryLimits</a> object that contains the directory limits for the current region.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectedDirectoriesLimitReached</i> Indicates if the connected directory limit has been reached.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectedDirectoriesCurrentCount</i> The current number of connected directories in the region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectedDirectoriesLimit</i> The maximum number of connected directories allowed in the region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudOnlyMicrosoftADLimitReached</i> Indicates if the AWS Managed Microsoft AD directory limit has been reached.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudOnlyMicrosoftADCurrentCount</i> The current number of AWS Managed Microsoft AD directories in the region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudOnlyMicrosoftADLimit</i> The maximum number of AWS Managed Microsoft AD directories allowed in the region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudOnlyDirectoriesLimitReached</i> Indicates if the cloud directory limit has been reached.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudOnlyDirectoriesCurrentCount</i> The current number of cloud directories in the region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudOnlyDirectoriesLimit</i> The maximum number of cloud directories allowed in the region.<br/>        """
        pass

    def get_snapshot_limits(self, DirectoryId: str) -> GetSnapshotLimitsResult:
        """Obtains the manual snapshot limits for a directory.

        :param self:
        :param DirectoryId:  Contains the identifier of the directory to obtain the limits for.<br/>
        :return: SnapshotLimits:  A <a>SnapshotLimits</a> object that contains the manual snapshot limits for the specified directory.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotsLimitReached</i> Indicates if the manual snapshot limit has been reached.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotsCurrentCount</i> The current number of manual snapshots of the directory.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotsLimit</i> The maximum number of manual snapshots allowed.<br/>        """
        pass

    def list_ip_routes(self, DirectoryId: str, NextToken: str=None, Limit: int=None) -> ListIpRoutesResult:
        """Lists the address blocks that you have added to a directory.

        :param self:
        :param Limit:  Maximum number of items to return. If this value is zero, the maximum number of items is specified by the limitations of the operation.<br/>NextToken:  The <i>ListIpRoutes.NextToken</i> value from a previous call to <a>ListIpRoutes</a>. Pass null if this is the first call.<br/>DirectoryId:  Identifier (ID) of the directory for which you want to retrieve the IP addresses.<br/>
        :return: NextToken:  If not null, more results are available. Pass this value for the <i>NextToken</i> parameter in a subsequent call to <a>ListIpRoutes</a> to retrieve the next set of items.<br/>IpRoutesInfo:  A list of <a>IpRoute</a>s.<br/>        """
        pass

    def list_log_subscriptions(self, DirectoryId: str=None, NextToken: str=None, Limit: int=None) -> ListLogSubscriptionsResult:
        """Lists the active log subscriptions for the AWS account.

        :param self:
        :param Limit:  The maximum number of items returned.<br/>NextToken:  The token for the next set of items to return.<br/>DirectoryId:  If a <i>DirectoryID</i> is provided, lists only the log subscription associated with that directory. If no <i>DirectoryId</i> is provided, lists all log subscriptions associated with your AWS account. If there are no log subscriptions for the AWS account or the directory, an empty list will be returned.<br/>
        :return: NextToken:  The token for the next set of items to return.<br/>LogSubscriptions:  A list of active <a>LogSubscription</a> objects for calling the AWS account.<br/>        """
        pass

    def list_schema_extensions(self, DirectoryId: str, NextToken: str=None, Limit: int=None) -> ListSchemaExtensionsResult:
        """Lists all schema extensions applied to a Microsoft AD Directory.

        :param self:
        :param Limit:  The maximum number of items to return.<br/>NextToken:  The <code>ListSchemaExtensions.NextToken</code> value from a previous call to <code>ListSchemaExtensions</code>. Pass null if this is the first call.<br/>DirectoryId:  The identifier of the directory from which to retrieve the schema extension information.<br/>
        :return: NextToken:  If not null, more results are available. Pass this value for the <code>NextToken</code> parameter in a subsequent call to <code>ListSchemaExtensions</code> to retrieve the next set of items.<br/>SchemaExtensionsInfo:  Information about the schema extensions applied to the directory.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceId: str, NextToken: str=None, Limit: int=None) -> ListTagsForResourceResult:
        """Lists all tags on a directory.

        :param self:
        :param Limit:  Reserved for future use.<br/>NextToken:  Reserved for future use.<br/>ResourceId:  Identifier (ID) of the directory for which you want to retrieve tags.<br/>
        :return: NextToken:  Reserved for future use.<br/>Tags:  List of tags returned by the ListTagsForResource operation.<br/>        """
        pass

    def register_event_topic(self, TopicName: str, DirectoryId: str) -> RegisterEventTopicResult:
        """Associates a directory with an SNS topic. This establishes the directory as a publisher to the specified SNS topic. You can then receive email or text (SMS) messages when the status of your directory changes. You get notified if your directory goes from an Active status to an Impaired or Inoperable status. You also receive a notification when the directory returns to an Active status.

        :param self:
        :param TopicName:  The SNS topic name to which the directory will publish status messages. This SNS topic must be in the same region as the specified Directory ID.<br/>DirectoryId:  The Directory ID that will publish status messages to the SNS topic.<br/>
        :return:         """
        pass

    def reject_shared_directory(self, SharedDirectoryId: str) -> RejectSharedDirectoryResult:
        """Rejects a directory sharing request that was sent from the directory owner account.

        :param self:
        :param SharedDirectoryId:  Identifier of the shared directory in the directory consumer account. This identifier is different for each directory owner account.<br/>
        :return: SharedDirectoryId:  Identifier of the shared directory in the directory consumer account.<br/>        """
        pass

    def remove_ip_routes(self, CidrIps: List[CidrIp], DirectoryId: str) -> RemoveIpRoutesResult:
        """Removes IP address blocks from a directory.

        :param self:
        :param CidrIps:  IP address blocks that you want to remove.<br/>DirectoryId:  Identifier (ID) of the directory from which you want to remove the IP addresses.<br/>
        :return:         """
        pass

    def remove_tags_from_resource(self, TagKeys: List[TagKey], ResourceId: str) -> RemoveTagsFromResourceResult:
        """Removes tags from a directory.

        :param self:
        :param TagKeys:  The tag key (name) of the tag to be removed.<br/>ResourceId:  Identifier (ID) of the directory from which to remove the tag.<br/>
        :return:         """
        pass

    def reset_user_password(self, NewPassword: str, UserName: str, DirectoryId: str) -> ResetUserPasswordResult:
        """Resets the password for any user in your AWS Managed Microsoft AD or Simple AD directory.

        :param self:
        :param NewPassword:  The new password that will be reset.<br/>UserName:  The user name of the user whose password will be reset.<br/>DirectoryId:  Identifier of the AWS Managed Microsoft AD or Simple AD directory in which the user resides.<br/>
        :return:         """
        pass

    def restore_from_snapshot(self, SnapshotId: str) -> RestoreFromSnapshotResult:
        """Restores a directory using an existing directory snapshot. When you restore a directory from a snapshot, any changes made to the directory after the snapshot date are overwritten. This action returns as soon as the restore operation is initiated. You can monitor the progress of the restore operation by calling the <a>DescribeDirectories</a> operation with the directory identifier. When the <b>DirectoryDescription.Stage</b> value changes to <code>Active</code>, the restore operation is complete.

        :param self:
        :param SnapshotId:  The identifier of the snapshot to restore from.<br/>
        :return:         """
        pass

    def share_directory(self, ShareMethod: str, ShareTarget: ShareTarget, DirectoryId: str, ShareNotes: str=None) -> ShareDirectoryResult:
        """Shares a specified directory (<code>DirectoryId</code>) in your AWS account (directory owner) with another AWS account (directory consumer). With this operation you can use your directory from any AWS account and from any Amazon VPC within an AWS Region. When you share your AWS Managed Microsoft AD directory, AWS Directory Service creates a shared directory in the directory consumer account. This shared directory contains the metadata to provide access to the directory within the directory owner account. The shared directory is visible in all VPCs in the directory consumer account. The <code>ShareMethod</code> parameter determines whether the specified directory can be shared between AWS accounts inside the same AWS organization (<code>ORGANIZATIONS</code>). It also determines whether you can share the directory with any other AWS account either inside or outside of the organization (<code>HANDSHAKE</code>). The <code>ShareNotes</code> parameter is only used when <code>HANDSHAKE</code> is called, which sends a directory sharing request to the directory consumer. 

        :param self:
        :param ShareMethod:  The method used when sharing a directory to determine whether the directory should be shared within your AWS organization (<code>ORGANIZATIONS</code>) or with any AWS account by sending a directory sharing request (<code>HANDSHAKE</code>).<br/>ShareTarget:  Identifier for the directory consumer account with whom the directory is to be shared.:<br/>ShareNotes:  A directory share request that is sent by the directory owner to the directory consumer. The request includes a typed message to help the directory consumer administrator determine whether to approve or reject the share invitation.<br/>DirectoryId:  Identifier of the AWS Managed Microsoft AD directory that you want to share with other AWS accounts.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> Type of identifier to be used in the <code>Id</code> field.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> Identifier of the directory consumer account.<br/>
        :return: SharedDirectoryId:  Identifier of the directory that is stored in the directory consumer account that is shared from the specified directory (<code>DirectoryId</code>).<br/>        """
        pass

    def start_schema_extension(self, Description: str, LdifContent: str, CreateSnapshotBeforeSchemaExtension: CreateSnapshotBeforeSchemaExtension, DirectoryId: str) -> StartSchemaExtensionResult:
        """Applies a schema extension to a Microsoft AD directory.

        :param self:
        :param Description:  A description of the schema extension.<br/>LdifContent:  The LDIF file represented as a string. To construct the LdifContent string, precede each line as it would be formatted in an ldif file with \n. See the example request below for more details. The file size can be no larger than 1MB.<br/>CreateSnapshotBeforeSchemaExtension:  If true, creates a snapshot of the directory before applying the schema extension.<br/>DirectoryId:  The identifier of the directory for which the schema extension will be applied to.<br/>
        :return: SchemaExtensionId:  The identifier of the schema extension that will be applied.<br/>        """
        pass

    def unshare_directory(self, UnshareTarget: UnshareTarget, DirectoryId: str) -> UnshareDirectoryResult:
        """Stops the directory sharing between the directory owner and consumer accounts. 

        :param self:
        :param UnshareTarget:  Identifier for the directory consumer account with whom the directory has to be unshared.:<br/>DirectoryId:  The identifier of the AWS Managed Microsoft AD directory that you want to stop sharing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> Type of identifier to be used in the <i>Id</i> field.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> Identifier of the directory consumer account.<br/>
        :return: SharedDirectoryId:  Identifier of the directory stored in the directory consumer account that is to be unshared from the specified directory (<code>DirectoryId</code>).<br/>        """
        pass

    def update_conditional_forwarder(self, DnsIpAddrs: List[IpAddr], RemoteDomainName: str, DirectoryId: str) -> UpdateConditionalForwarderResult:
        """Updates a conditional forwarder that has been set up for your AWS directory.

        :param self:
        :param DnsIpAddrs:  The updated IP addresses of the remote DNS server associated with the conditional forwarder.<br/>RemoteDomainName:  The fully qualified domain name (FQDN) of the remote domain with which you will set up a trust relationship.<br/>DirectoryId:  The directory ID of the AWS directory for which to update the conditional forwarder.<br/>
        :return:         """
        pass

    def update_number_of_domain_controllers(self, DesiredNumber: int, DirectoryId: str) -> UpdateNumberOfDomainControllersResult:
        """Adds or removes domain controllers to or from the directory. Based on the difference between current value and new value (provided through this API call), domain controllers will be added or removed. It may take up to 45 minutes for any new domain controllers to become fully active once the requested number of domain controllers is updated. During this time, you cannot make another update request.

        :param self:
        :param DesiredNumber:  The number of domain controllers desired in the directory.<br/>DirectoryId:  Identifier of the directory to which the domain controllers will be added or removed.<br/>
        :return:         """
        pass

    def update_radius(self, RadiusSettings: RadiusSettings, DirectoryId: str) -> UpdateRadiusResult:
        """Updates the Remote Authentication Dial In User Service (RADIUS) server information for an AD Connector or Microsoft AD directory.

        :param self:
        :param RadiusSettings:  A <a>RadiusSettings</a> object that contains information about the RADIUS server.:<br/>DirectoryId:  The identifier of the directory for which to update the RADIUS server information.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UseSameUsername</i> Not currently used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayLabel</i> Not currently used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthenticationProtocol</i> The protocol specified for your RADIUS endpoints.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SharedSecret</i> Required for enabling RADIUS on the directory.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RadiusRetries</i> The maximum number of times that communication with the RADIUS server is attempted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RadiusTimeout</i> The amount of time, in seconds, to wait for the RADIUS server to respond.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RadiusPort</i> The port that your RADIUS server is using for communications. Your on-premises network must allow inbound traffic over this port from the AWS Directory Service servers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RadiusServers</i> An array of strings that contains the IP addresses of the RADIUS server endpoints, or the IP addresses of your RADIUS server load balancer.<br/>
        :return:         """
        pass

    def update_trust(self, TrustId: str, SelectiveAuth: str=None) -> UpdateTrustResult:
        """Updates the trust that has been set up between your AWS Managed Microsoft AD directory and an on-premises Active Directory.

        :param self:
        :param SelectiveAuth:  Updates selective authentication for the trust.<br/>TrustId:  Identifier of the trust relationship.<br/>
        :return: TrustId:  Identifier of the trust relationship.<br/>RequestId:  The AWS request identifier.<br/>        """
        pass

    def verify_trust(self, TrustId: str) -> VerifyTrustResult:
        """AWS Directory Service for Microsoft Active Directory allows you to configure and verify trust relationships. This action verifies a trust relationship between your AWS Managed Microsoft AD directory and an external domain.

        :param self:
        :param TrustId:  The unique Trust ID of the trust relationship to verify.<br/>
        :return: TrustId:  The unique Trust ID of the trust relationship that was verified.<br/>        """
        pass



class DynamoDB(BaseClient):
    """<fullname>Amazon DynamoDB</fullname> Amazon DynamoDB is a fully managed NoSQL database service that provides fast and predictable performance with seamless scalability. DynamoDB lets you offload the administrative burdens of operating and scaling a distributed database, so that you don't have to worry about hardware provisioning, setup and configuration, replication, software patching, or cluster scaling. With DynamoDB, you can create database tables that can store and retrieve any amount of data, and serve any level of request traffic. You can scale up or scale down your tables' throughput capacity without downtime or performance degradation, and use the AWS Management Console to monitor resource utilization and performance metrics. DynamoDB automatically spreads the data and traffic for your tables over a sufficient number of servers to handle your throughput and storage requirements, while maintaining consistent and fast performance. All of your data is stored on solid state disks (SSDs) and automatically replicated across multiple Availability Zones in an AWS region, providing built-in high availability and data durability. """

    class TableExistsWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_table` for valid parameters"""
            pass
    class TableNotExistsWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_table` for valid parameters"""
            pass


    class ListBackupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_backups` for valid parameters"""
            pass
    class ListTablesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_tables` for valid parameters"""
            pass
    class QueryPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `query` for valid parameters"""
            pass
    class ScanPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `scan` for valid parameters"""
            pass
    class ListTagsOfResourcePaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_tags_of_resource` for valid parameters"""
            pass


    class BatchGetItemInput(Mapping):
        pass
    
    class BatchGetItemOutput(Mapping):
        pass
    
    class BatchGetRequestMap(object):
        pass
    
    class BatchWriteItemInput(Mapping):
        pass
    
    class BatchWriteItemOutput(Mapping):
        pass
    
    class BatchWriteItemRequestMap(object):
        pass
    
    class CreateBackupInput(Mapping):
        pass
    
    class CreateBackupOutput(Mapping):
        pass
    
    class CreateGlobalTableInput(Mapping):
        pass
    
    class CreateGlobalTableOutput(Mapping):
        pass
    
    class Replica(Mapping):
        pass
    
    class CreateTableInput(Mapping):
        pass
    
    class CreateTableOutput(Mapping):
        pass
    
    class AttributeDefinition(Mapping):
        pass
    
    class KeySchemaElement(Mapping):
        pass
    
    class LocalSecondaryIndex(Mapping):
        pass
    
    class GlobalSecondaryIndex(Mapping):
        pass
    
    class ProvisionedThroughput(Mapping):
        pass
    
    class StreamSpecification(Mapping):
        pass
    
    class SSESpecification(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class DeleteBackupInput(Mapping):
        pass
    
    class DeleteBackupOutput(Mapping):
        pass
    
    class DeleteItemInput(Mapping):
        pass
    
    class DeleteItemOutput(Mapping):
        pass
    
    class Key(object):
        pass
    
    class ExpectedAttributeMap(object):
        pass
    
    class ExpressionAttributeNameMap(object):
        pass
    
    class ExpressionAttributeValueMap(object):
        pass
    
    class DeleteTableInput(Mapping):
        pass
    
    class DeleteTableOutput(Mapping):
        pass
    
    class DescribeBackupInput(Mapping):
        pass
    
    class DescribeBackupOutput(Mapping):
        pass
    
    class DescribeContinuousBackupsInput(Mapping):
        pass
    
    class DescribeContinuousBackupsOutput(Mapping):
        pass
    
    class DescribeEndpointsRequest(Mapping):
        pass
    
    class DescribeEndpointsResponse(Mapping):
        pass
    
    class DescribeGlobalTableInput(Mapping):
        pass
    
    class DescribeGlobalTableOutput(Mapping):
        pass
    
    class DescribeGlobalTableSettingsInput(Mapping):
        pass
    
    class DescribeGlobalTableSettingsOutput(Mapping):
        pass
    
    class DescribeLimitsInput(Mapping):
        pass
    
    class DescribeLimitsOutput(Mapping):
        pass
    
    class DescribeTableInput(Mapping):
        pass
    
    class DescribeTableOutput(Mapping):
        pass
    
    class DescribeTimeToLiveInput(Mapping):
        pass
    
    class DescribeTimeToLiveOutput(Mapping):
        pass
    
    class GetItemInput(Mapping):
        pass
    
    class GetItemOutput(Mapping):
        pass
    
    class AttributeName(object):
        pass
    
    class ConsistentRead(object):
        pass
    
    class ListBackupsInput(Mapping):
        pass
    
    class ListBackupsOutput(Mapping):
        pass
    
    class TimeRangeLowerBound(object):
        pass
    
    class TimeRangeUpperBound(object):
        pass
    
    class ListGlobalTablesInput(Mapping):
        pass
    
    class ListGlobalTablesOutput(Mapping):
        pass
    
    class ListTablesInput(Mapping):
        pass
    
    class ListTablesOutput(Mapping):
        pass
    
    class ListTagsOfResourceInput(Mapping):
        pass
    
    class ListTagsOfResourceOutput(Mapping):
        pass
    
    class PutItemInput(Mapping):
        pass
    
    class PutItemOutput(Mapping):
        pass
    
    class PutItemInputAttributeMap(object):
        pass
    
    class QueryInput(Mapping):
        pass
    
    class QueryOutput(Mapping):
        pass
    
    class KeyConditions(object):
        pass
    
    class FilterConditionMap(object):
        pass
    
    class BooleanObject(object):
        pass
    
    class RestoreTableFromBackupInput(Mapping):
        pass
    
    class RestoreTableFromBackupOutput(Mapping):
        pass
    
    class RestoreTableToPointInTimeInput(Mapping):
        pass
    
    class RestoreTableToPointInTimeOutput(Mapping):
        pass
    
    class Date(object):
        pass
    
    class ScanInput(Mapping):
        pass
    
    class ScanOutput(Mapping):
        pass
    
    class TagResourceInput(Mapping):
        pass
    
    class TransactGetItemsInput(Mapping):
        pass
    
    class TransactGetItemsOutput(Mapping):
        pass
    
    class TransactGetItem(Mapping):
        pass
    
    class TransactWriteItemsInput(Mapping):
        pass
    
    class TransactWriteItemsOutput(Mapping):
        pass
    
    class TransactWriteItem(Mapping):
        pass
    
    class UntagResourceInput(Mapping):
        pass
    
    class TagKeyString(object):
        pass
    
    class UpdateContinuousBackupsInput(Mapping):
        pass
    
    class UpdateContinuousBackupsOutput(Mapping):
        pass
    
    class PointInTimeRecoverySpecification(Mapping):
        pass
    
    class UpdateGlobalTableInput(Mapping):
        pass
    
    class UpdateGlobalTableOutput(Mapping):
        pass
    
    class ReplicaUpdate(Mapping):
        pass
    
    class UpdateGlobalTableSettingsInput(Mapping):
        pass
    
    class UpdateGlobalTableSettingsOutput(Mapping):
        pass
    
    class PositiveLongObject(object):
        pass
    
    class AutoScalingSettingsUpdate(Mapping):
        pass
    
    class GlobalTableGlobalSecondaryIndexSettingsUpdate(Mapping):
        pass
    
    class ReplicaSettingsUpdate(Mapping):
        pass
    
    class UpdateItemInput(Mapping):
        pass
    
    class UpdateItemOutput(Mapping):
        pass
    
    class AttributeUpdates(object):
        pass
    
    class UpdateTableInput(Mapping):
        pass
    
    class UpdateTableOutput(Mapping):
        pass
    
    class GlobalSecondaryIndexUpdate(Mapping):
        pass
    
    class UpdateTimeToLiveInput(Mapping):
        pass
    
    class UpdateTimeToLiveOutput(Mapping):
        pass
    
    class TimeToLiveSpecification(Mapping):
        pass
    
    def batch_get_item(self, RequestItems: BatchGetRequestMap, ReturnConsumedCapacity: str=None) -> BatchGetItemOutput:
        """The <code>BatchGetItem</code> operation returns the attributes of one or more items from one or more tables. You identify requested items by primary key. A single operation can retrieve up to 16 MB of data, which can contain as many as 100 items. <code>BatchGetItem</code> returns a partial result if the response size limit is exceeded, the table's provisioned throughput is exceeded, or an internal processing failure occurs. If a partial result is returned, the operation returns a value for <code>UnprocessedKeys</code>. You can use this value to retry the operation starting with the next item to get. <important> If you request more than 100 items, <code>BatchGetItem</code> returns a <code>ValidationException</code> with the message "Too many items requested for the BatchGetItem call." </important> For example, if you ask to retrieve 100 items, but each individual item is 300 KB in size, the system returns 52 items (so as not to exceed the 16 MB limit). It also returns an appropriate <code>UnprocessedKeys</code> value so you can get the next page of results. If desired, your application can include its own logic to assemble the pages of results into one dataset. If <i>none</i> of the items can be processed due to insufficient provisioned throughput on all of the tables in the request, then <code>BatchGetItem</code> returns a <code>ProvisionedThroughputExceededException</code>. If <i>at least one</i> of the items is successfully processed, then <code>BatchGetItem</code> completes successfully, while returning the keys of the unread items in <code>UnprocessedKeys</code>. <important> If DynamoDB returns any unprocessed items, you should retry the batch operation on those items. However, <i>we strongly recommend that you use an exponential backoff algorithm</i>. If you retry the batch operation immediately, the underlying read or write requests can still fail due to throttling on the individual tables. If you delay the batch operation using exponential backoff, the individual requests in the batch are much more likely to succeed. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/ErrorHandling.html#BatchOperations">Batch Operations and Error Handling</a> in the <i>Amazon DynamoDB Developer Guide</i>. </important> By default, <code>BatchGetItem</code> performs eventually consistent reads on every table in the request. If you want strongly consistent reads instead, you can set <code>ConsistentRead</code> to <code>true</code> for any or all tables. In order to minimize response latency, <code>BatchGetItem</code> retrieves items in parallel. When designing your application, keep in mind that DynamoDB does not return items in any particular order. To help parse the response by item, include the primary key values for the items in your request in the <code>ProjectionExpression</code> parameter. If a requested item does not exist, it is not returned in the result. Requests for nonexistent items consume the minimum read capacity units according to the type of read. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/WorkingWithTables.html#CapacityUnitCalculations">Working with Tables</a> in the <i>Amazon DynamoDB Developer Guide</i>.

        :param self:
        :param ReturnConsumedCapacity:  Determines the level of detail about provisioned throughput consumption that is returned in the response: <ul> <li>  <code>INDEXES</code> - The response includes the aggregate <code>ConsumedCapacity</code> for the operation, together with <code>ConsumedCapacity</code> for each table and secondary index that was accessed. Note that some operations, such as <code>GetItem</code> and <code>BatchGetItem</code>, do not access any indexes at all. In these cases, specifying <code>INDEXES</code> will only return <code>ConsumedCapacity</code> information for table(s). </li> <li>  <code>TOTAL</code> - The response includes only the aggregate <code>ConsumedCapacity</code> for the operation. </li> <li>  <code>NONE</code> - No <code>ConsumedCapacity</code> details are included in the response. </li> </ul><br/>RequestItems:  A map of one or more table names and, for each table, a map that describes one or more items to retrieve from that table. Each table name can be used only once per <code>BatchGetItem</code> request. Each element in the map of items to retrieve consists of the following: <ul> <li>  <code>ConsistentRead</code> - If <code>true</code>, a strongly consistent read is used; if <code>false</code> (the default), an eventually consistent read is used. </li> <li>  <code>ExpressionAttributeNames</code> - One or more substitution tokens for attribute names in the <code>ProjectionExpression</code> parameter. The following are some use cases for using <code>ExpressionAttributeNames</code>: <ul> <li> To access an attribute whose name conflicts with a DynamoDB reserved word. </li> <li> To create a placeholder for repeating occurrences of an attribute name in an expression. </li> <li> To prevent special characters in an attribute name from being misinterpreted in an expression. </li> </ul> Use the <b>#</b> character in an expression to dereference an attribute name. For example, consider the following attribute name: <ul> <li>  <code>Percentile</code>  </li> </ul> The name of this attribute conflicts with a reserved word, so it cannot be used directly in an expression. (For the complete list of reserved words, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/ReservedWords.html">Reserved Words</a> in the <i>Amazon DynamoDB Developer Guide</i>). To work around this, you could specify the following for <code>ExpressionAttributeNames</code>: <ul> <li>  <code>{"#P":"Percentile"}</code>  </li> </ul> You could then use this substitution in an expression, as in this example: <ul> <li>  <code>#P = :val</code>  </li> </ul> <note> Tokens that begin with the <b>:</b> character are <i>expression attribute values</i>, which are placeholders for the actual value at runtime. </note> For more information about expression attribute names, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Expressions.AccessingItemAttributes.html">Accessing Item Attributes</a> in the <i>Amazon DynamoDB Developer Guide</i>. </li> <li>  <code>Keys</code> - An array of primary key attribute values that define specific items in the table. For each primary key, you must provide <i>all</i> of the key attributes. For example, with a simple primary key, you only need to provide the partition key value. For a composite key, you must provide <i>both</i> the partition key value and the sort key value. </li> <li>  <code>ProjectionExpression</code> - A string that identifies one or more attributes to retrieve from the table. These attributes can include scalars, sets, or elements of a JSON document. The attributes in the expression must be separated by commas. If no attribute names are specified, then all attributes are returned. If any of the requested attributes are not found, they do not appear in the result. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Expressions.AccessingItemAttributes.html">Accessing Item Attributes</a> in the <i>Amazon DynamoDB Developer Guide</i>. </li> <li>  <code>AttributesToGet</code> - This is a legacy parameter. Use <code>ProjectionExpression</code> instead. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/LegacyConditionalParameters.AttributesToGet.html">AttributesToGet</a> in the <i>Amazon DynamoDB Developer Guide</i>.  </li> </ul><br/>
        :return: ConsumedCapacity:  The read capacity units consumed by the entire <code>BatchGetItem</code> operation. Each element consists of: <ul> <li>  <code>TableName</code> - The table that consumed the provisioned throughput. </li> <li>  <code>CapacityUnits</code> - The total number of capacity units consumed. </li> </ul><br/>UnprocessedKeys:  A map of tables and their respective keys that were not processed with the current response. The <code>UnprocessedKeys</code> value is in the same form as <code>RequestItems</code>, so the value can be provided directly to a subsequent <code>BatchGetItem</code> operation. For more information, see <code>RequestItems</code> in the Request Parameters section. Each element consists of: <ul> <li>  <code>Keys</code> - An array of primary key attribute values that define specific items in the table. </li> <li>  <code>ProjectionExpression</code> - One or more attributes to be retrieved from the table or index. By default, all attributes are returned. If a requested attribute is not found, it does not appear in the result. </li> <li>  <code>ConsistentRead</code> - The consistency of a read operation. If set to <code>true</code>, then a strongly consistent read is used; otherwise, an eventually consistent read is used. </li> </ul> If there are no unprocessed keys remaining, the response contains an empty <code>UnprocessedKeys</code> map.<br/>Responses:  A map of table name to a list of items. Each object in <code>Responses</code> consists of a table name, along with a map of attribute data consisting of the data type and attribute value.<br/>        """
        pass

    def batch_write_item(self, RequestItems: BatchWriteItemRequestMap, ReturnConsumedCapacity: str=None, ReturnItemCollectionMetrics: str=None) -> BatchWriteItemOutput:
        """The <code>BatchWriteItem</code> operation puts or deletes multiple items in one or more tables. A single call to <code>BatchWriteItem</code> can write up to 16 MB of data, which can comprise as many as 25 put or delete requests. Individual items to be written can be as large as 400 KB. <note>  <code>BatchWriteItem</code> cannot update items. To update items, use the <code>UpdateItem</code> action. </note> The individual <code>PutItem</code> and <code>DeleteItem</code> operations specified in <code>BatchWriteItem</code> are atomic; however <code>BatchWriteItem</code> as a whole is not. If any requested operations fail because the table's provisioned throughput is exceeded or an internal processing failure occurs, the failed operations are returned in the <code>UnprocessedItems</code> response parameter. You can investigate and optionally resend the requests. Typically, you would call <code>BatchWriteItem</code> in a loop. Each iteration would check for unprocessed items and submit a new <code>BatchWriteItem</code> request with those unprocessed items until all items have been processed. If <i>none</i> of the items can be processed due to insufficient provisioned throughput on all of the tables in the request, then <code>BatchWriteItem</code> returns a <code>ProvisionedThroughputExceededException</code>. <important> If DynamoDB returns any unprocessed items, you should retry the batch operation on those items. However, <i>we strongly recommend that you use an exponential backoff algorithm</i>. If you retry the batch operation immediately, the underlying read or write requests can still fail due to throttling on the individual tables. If you delay the batch operation using exponential backoff, the individual requests in the batch are much more likely to succeed. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/ErrorHandling.html#Programming.Errors.BatchOperations">Batch Operations and Error Handling</a> in the <i>Amazon DynamoDB Developer Guide</i>. </important> With <code>BatchWriteItem</code>, you can efficiently write or delete large amounts of data, such as from Amazon EMR, or copy data from another database into DynamoDB. In order to improve performance with these large-scale operations, <code>BatchWriteItem</code> does not behave in the same way as individual <code>PutItem</code> and <code>DeleteItem</code> calls would. For example, you cannot specify conditions on individual put and delete requests, and <code>BatchWriteItem</code> does not return deleted items in the response. If you use a programming language that supports concurrency, you can use threads to write items in parallel. Your application must include the necessary logic to manage the threads. With languages that don't support threading, you must update or delete the specified items one at a time. In both situations, <code>BatchWriteItem</code> performs the specified put and delete operations in parallel, giving you the power of the thread pool approach without having to introduce complexity into your application. Parallel processing reduces latency, but each specified put and delete request consumes the same number of write capacity units whether it is processed in parallel or not. Delete operations on nonexistent items consume one write capacity unit. If one or more of the following is true, DynamoDB rejects the entire batch write operation: <ul> <li> One or more tables specified in the <code>BatchWriteItem</code> request does not exist. </li> <li> Primary key attributes specified on an item in the request do not match those in the corresponding table's primary key schema. </li> <li> You try to perform multiple operations on the same item in the same <code>BatchWriteItem</code> request. For example, you cannot put and delete the same item in the same <code>BatchWriteItem</code> request.  </li> <li>  Your request contains at least two items with identical hash and range keys (which essentially is two put operations).  </li> <li> There are more than 25 requests in the batch. </li> <li> Any individual item in a batch exceeds 400 KB. </li> <li> The total request size exceeds 16 MB. </li> </ul>

        :param self:
        :param ReturnItemCollectionMetrics:  Determines whether item collection metrics are returned. If set to <code>SIZE</code>, the response includes statistics about item collections, if any, that were modified during the operation are returned in the response. If set to <code>NONE</code> (the default), no statistics are returned.<br/>ReturnConsumedCapacity:  Determines the level of detail about provisioned throughput consumption that is returned in the response: <ul> <li>  <code>INDEXES</code> - The response includes the aggregate <code>ConsumedCapacity</code> for the operation, together with <code>ConsumedCapacity</code> for each table and secondary index that was accessed. Note that some operations, such as <code>GetItem</code> and <code>BatchGetItem</code>, do not access any indexes at all. In these cases, specifying <code>INDEXES</code> will only return <code>ConsumedCapacity</code> information for table(s). </li> <li>  <code>TOTAL</code> - The response includes only the aggregate <code>ConsumedCapacity</code> for the operation. </li> <li>  <code>NONE</code> - No <code>ConsumedCapacity</code> details are included in the response. </li> </ul><br/>RequestItems:  A map of one or more table names and, for each table, a list of operations to be performed (<code>DeleteRequest</code> or <code>PutRequest</code>). Each element in the map consists of the following: <ul> <li>  <code>DeleteRequest</code> - Perform a <code>DeleteItem</code> operation on the specified item. The item to be deleted is identified by a <code>Key</code> subelement: <ul> <li>  <code>Key</code> - A map of primary key attribute values that uniquely identify the item. Each entry in this map consists of an attribute name and an attribute value. For each primary key, you must provide <i>all</i> of the key attributes. For example, with a simple primary key, you only need to provide a value for the partition key. For a composite primary key, you must provide values for <i>both</i> the partition key and the sort key. </li> </ul> </li> <li>  <code>PutRequest</code> - Perform a <code>PutItem</code> operation on the specified item. The item to be put is identified by an <code>Item</code> subelement: <ul> <li>  <code>Item</code> - A map of attributes and their values. Each entry in this map consists of an attribute name and an attribute value. Attribute values must not be null; string and binary type attributes must have lengths greater than zero; and set type attributes must not be empty. Requests that contain empty values are rejected with a <code>ValidationException</code> exception. If you specify any attributes that are part of an index key, then the data types for those attributes must match those of the schema in the table's attribute definition. </li> </ul> </li> </ul><br/>
        :return: ConsumedCapacity:  The capacity units consumed by the entire <code>BatchWriteItem</code> operation. Each element consists of: <ul> <li>  <code>TableName</code> - The table that consumed the provisioned throughput. </li> <li>  <code>CapacityUnits</code> - The total number of capacity units consumed. </li> </ul><br/>ItemCollectionMetrics:  A list of tables that were processed by <code>BatchWriteItem</code> and, for each table, information about any item collections that were affected by individual <code>DeleteItem</code> or <code>PutItem</code> operations. Each entry consists of the following subelements: <ul> <li>  <code>ItemCollectionKey</code> - The partition key value of the item collection. This is the same as the partition key value of the item. </li> <li>  <code>SizeEstimateRangeGB</code> - An estimate of item collection size, expressed in GB. This is a two-element array containing a lower bound and an upper bound for the estimate. The estimate includes the size of all the items in the table, plus the size of all attributes projected into all of the local secondary indexes on the table. Use this estimate to measure whether a local secondary index is approaching its size limit. The estimate is subject to change over time; therefore, do not rely on the precision or accuracy of the estimate. </li> </ul><br/>UnprocessedItems:  A map of tables and requests against those tables that were not processed. The <code>UnprocessedItems</code> value is in the same form as <code>RequestItems</code>, so you can provide this value directly to a subsequent <code>BatchGetItem</code> operation. For more information, see <code>RequestItems</code> in the Request Parameters section. Each <code>UnprocessedItems</code> entry consists of a table name and, for that table, a list of operations to perform (<code>DeleteRequest</code> or <code>PutRequest</code>). <ul> <li>  <code>DeleteRequest</code> - Perform a <code>DeleteItem</code> operation on the specified item. The item to be deleted is identified by a <code>Key</code> subelement: <ul> <li>  <code>Key</code> - A map of primary key attribute values that uniquely identify the item. Each entry in this map consists of an attribute name and an attribute value. </li> </ul> </li> <li>  <code>PutRequest</code> - Perform a <code>PutItem</code> operation on the specified item. The item to be put is identified by an <code>Item</code> subelement: <ul> <li>  <code>Item</code> - A map of attributes and their values. Each entry in this map consists of an attribute name and an attribute value. Attribute values must not be null; string and binary type attributes must have lengths greater than zero; and set type attributes must not be empty. Requests that contain empty values will be rejected with a <code>ValidationException</code> exception. If you specify any attributes that are part of an index key, then the data types for those attributes must match those of the schema in the table's attribute definition. </li> </ul> </li> </ul> If there are no unprocessed items remaining, the response contains an empty <code>UnprocessedItems</code> map.<br/>        """
        pass

    def create_backup(self, BackupName: str, TableName: str) -> CreateBackupOutput:
        """Creates a backup for an existing table.  Each time you create an on-demand backup, the entire table data is backed up. There is no limit to the number of on-demand backups that can be taken.   When you create an on-demand backup, a time marker of the request is cataloged, and the backup is created asynchronously, by applying all changes until the time of the request to the last full table snapshot. Backup requests are processed instantaneously and become available for restore within minutes.  You can call <code>CreateBackup</code> at a maximum rate of 50 times per second. All backups in DynamoDB work without consuming any provisioned throughput on the table.  If you submit a backup request on 2018-12-14 at 14:25:00, the backup is guaranteed to contain all data committed to the table up to 14:24:00, and data committed after 14:26:00 will not be. The backup might contain data modifications made between 14:24:00 and 14:26:00. On-demand backup does not support causal consistency.   Along with data, the following are also included on the backups:  <ul> <li> Global secondary indexes (GSIs) </li> <li> Local secondary indexes (LSIs) </li> <li> Streams </li> <li> Provisioned read and write capacity </li> </ul>

        :param self:
        :param BackupName:  Specified name for the backup.<br/>TableName:  The name of the table.<br/>
        :return: BackupDetails:  Contains the details of the backup created for the table.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupExpiryDateTime</i> Time at which the automatic on-demand backup created by DynamoDB will expire. This <code>SYSTEM</code> on-demand backup expires automatically 35 days after its creation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupCreationDateTime</i> Time at which the backup was created. This is the request time of the backup. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupType</i> BackupType: <ul> <li>  <code>USER</code> - You create and manage these using the on-demand backup feature. </li> <li>  <code>SYSTEM</code> - If you delete a table with point-in-time recovery enabled, a <code>SYSTEM</code> backup is automatically created and is retained for 35 days (at no additional cost). System backups allow you to restore the deleted table to the state it was in just before the point of deletion.  </li> <li>  <code>AWS_BACKUP</code> - On-demand backup created by you from AWS Backup service. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupStatus</i> Backup can be in one of the following states: CREATING, ACTIVE, DELETED. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupSizeBytes</i> Size of the backup in bytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupName</i> Name of the requested backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupArn</i> ARN associated with the backup.<br/>        """
        pass

    def create_global_table(self, ReplicationGroup: List[Replica], GlobalTableName: str) -> CreateGlobalTableOutput:
        """Creates a global table from an existing table. A global table creates a replication relationship between two or more DynamoDB tables with the same table name in the provided Regions.  If you want to add a new replica table to a global table, each of the following conditions must be true: <ul> <li> The table must have the same primary key as all of the other replicas. </li> <li> The table must have the same name as all of the other replicas. </li> <li> The table must have DynamoDB Streams enabled, with the stream containing both the new and the old images of the item. </li> <li> None of the replica tables in the global table can contain any data. </li> </ul>  If global secondary indexes are specified, then the following conditions must also be met:  <ul> <li>  The global secondary indexes must have the same name.  </li> <li>  The global secondary indexes must have the same hash key and sort key (if present).  </li> </ul> <important>  Write capacity settings should be set consistently across your replica tables and secondary indexes. DynamoDB strongly recommends enabling auto scaling to manage the write capacity settings for all of your global tables replicas and indexes.   If you prefer to manage write capacity settings manually, you should provision equal replicated write capacity units to your replica tables. You should also provision equal replicated write capacity units to matching secondary indexes across your global table.  </important>

        :param self:
        :param ReplicationGroup:  The Regions where the global table needs to be created.<br/>GlobalTableName:  The global table name.<br/>
        :return: GlobalTableDescription:  Contains the details of the global table.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalTableName</i> The global table name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalTableStatus</i> The current state of the global table: <ul> <li>  <code>CREATING</code> - The global table is being created. </li> <li>  <code>UPDATING</code> - The global table is being updated. </li> <li>  <code>DELETING</code> - The global table is being deleted. </li> <li>  <code>ACTIVE</code> - The global table is ready for use. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDateTime</i> The creation time of the global table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalTableArn</i> The unique identifier of the global table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationGroup</i> The regions where the global table has replicas.<br/>        """
        pass

    def create_table(self, KeySchema: List[KeySchemaElement], TableName: str, AttributeDefinitions: List[AttributeDefinition], LocalSecondaryIndexes: List[LocalSecondaryIndex]=None, GlobalSecondaryIndexes: List[GlobalSecondaryIndex]=None, BillingMode: str=None, ProvisionedThroughput: ProvisionedThroughput=None, StreamSpecification: StreamSpecification=None, SSESpecification: SSESpecification=None, Tags: List[Tag]=None) -> CreateTableOutput:
        """The <code>CreateTable</code> operation adds a new table to your account. In an AWS account, table names must be unique within each Region. That is, you can have two tables with same name if you create the tables in different Regions.  <code>CreateTable</code> is an asynchronous operation. Upon receiving a <code>CreateTable</code> request, DynamoDB immediately returns a response with a <code>TableStatus</code> of <code>CREATING</code>. After the table is created, DynamoDB sets the <code>TableStatus</code> to <code>ACTIVE</code>. You can perform read and write operations only on an <code>ACTIVE</code> table.  You can optionally define secondary indexes on the new table, as part of the <code>CreateTable</code> operation. If you want to create multiple tables with secondary indexes on them, you must create the tables sequentially. Only one table with secondary indexes can be in the <code>CREATING</code> state at any given time. You can use the <code>DescribeTable</code> action to check the table status.

        :param self:
        :param Tags:  A list of key-value pairs to label the table. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Tagging.html">Tagging for DynamoDB</a>.<br/>SSESpecification:  Represents the settings used to enable server-side encryption.:<br/>StreamSpecification:  The settings for DynamoDB Streams on the table. These settings consist of: <ul> <li>  <code>StreamEnabled</code> - Indicates whether DynamoDB Streams is to be enabled (true) or disabled (false). </li> <li>  <code>StreamViewType</code> - When an item in the table is modified, <code>StreamViewType</code> determines what information is written to the table's stream. Valid values for <code>StreamViewType</code> are: <ul> <li>  <code>KEYS_ONLY</code> - Only the key attributes of the modified item are written to the stream. </li> <li>  <code>NEW_IMAGE</code> - The entire item, as it appears after it was modified, is written to the stream. </li> <li>  <code>OLD_IMAGE</code> - The entire item, as it appeared before it was modified, is written to the stream. </li> <li>  <code>NEW_AND_OLD_IMAGES</code> - Both the new and the old item images of the item are written to the stream. </li> </ul> </li> </ul>:<br/>ProvisionedThroughput:  Represents the provisioned throughput settings for a specified table or index. The settings can be modified using the <code>UpdateTable</code> operation.  If you set BillingMode as <code>PROVISIONED</code>, you must specify this property. If you set BillingMode as <code>PAY_PER_REQUEST</code>, you cannot specify this property.  For current minimum and maximum provisioned throughput values, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Limits.html">Limits</a> in the <i>Amazon DynamoDB Developer Guide</i>.:<br/>BillingMode:  Controls how you are charged for read and write throughput and how you manage capacity. This setting can be changed later. <ul> <li>  <code>PROVISIONED</code> - Sets the billing mode to <code>PROVISIONED</code>. We recommend using <code>PROVISIONED</code> for predictable workloads. </li> <li>  <code>PAY_PER_REQUEST</code> - Sets the billing mode to <code>PAY_PER_REQUEST</code>. We recommend using <code>PAY_PER_REQUEST</code> for unpredictable workloads.  </li> </ul><br/>GlobalSecondaryIndexes:  One or more global secondary indexes (the maximum is 20) to be created on the table. Each global secondary index in the array includes the following: <ul> <li>  <code>IndexName</code> - The name of the global secondary index. Must be unique only for this table. <p/> </li> <li>  <code>KeySchema</code> - Specifies the key schema for the global secondary index. </li> <li>  <code>Projection</code> - Specifies attributes that are copied (projected) from the table into the index. These are in addition to the primary key attributes and index key attributes, which are automatically projected. Each attribute specification is composed of: <ul> <li>  <code>ProjectionType</code> - One of the following: <ul> <li>  <code>KEYS_ONLY</code> - Only the index and primary keys are projected into the index. </li> <li>  <code>INCLUDE</code> - Only the specified table attributes are projected into the index. The list of projected attributes is in <code>NonKeyAttributes</code>. </li> <li>  <code>ALL</code> - All of the table attributes are projected into the index. </li> </ul> </li> <li>  <code>NonKeyAttributes</code> - A list of one or more non-key attribute names that are projected into the secondary index. The total count of attributes provided in <code>NonKeyAttributes</code>, summed across all of the secondary indexes, must not exceed 100. If you project the same attribute into two different indexes, this counts as two distinct attributes when determining the total. </li> </ul> </li> <li>  <code>ProvisionedThroughput</code> - The provisioned throughput settings for the global secondary index, consisting of read and write capacity units. </li> </ul><br/>LocalSecondaryIndexes:  One or more local secondary indexes (the maximum is 5) to be created on the table. Each index is scoped to a given partition key value. There is a 10 GB size limit per partition key value; otherwise, the size of a local secondary index is unconstrained. Each local secondary index in the array includes the following: <ul> <li>  <code>IndexName</code> - The name of the local secondary index. Must be unique only for this table. <p/> </li> <li>  <code>KeySchema</code> - Specifies the key schema for the local secondary index. The key schema must begin with the same partition key as the table. </li> <li>  <code>Projection</code> - Specifies attributes that are copied (projected) from the table into the index. These are in addition to the primary key attributes and index key attributes, which are automatically projected. Each attribute specification is composed of: <ul> <li>  <code>ProjectionType</code> - One of the following: <ul> <li>  <code>KEYS_ONLY</code> - Only the index and primary keys are projected into the index. </li> <li>  <code>INCLUDE</code> - Only the specified table attributes are projected into the index. The list of projected attributes is in <code>NonKeyAttributes</code>. </li> <li>  <code>ALL</code> - All of the table attributes are projected into the index. </li> </ul> </li> <li>  <code>NonKeyAttributes</code> - A list of one or more non-key attribute names that are projected into the secondary index. The total count of attributes provided in <code>NonKeyAttributes</code>, summed across all of the secondary indexes, must not exceed 100. If you project the same attribute into two different indexes, this counts as two distinct attributes when determining the total. </li> </ul> </li> </ul><br/>KeySchema:  Specifies the attributes that make up the primary key for a table or an index. The attributes in <code>KeySchema</code> must also be defined in the <code>AttributeDefinitions</code> array. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/DataModel.html">Data Model</a> in the <i>Amazon DynamoDB Developer Guide</i>. Each <code>KeySchemaElement</code> in the array is composed of: <ul> <li>  <code>AttributeName</code> - The name of this key attribute. </li> <li>  <code>KeyType</code> - The role that the key attribute will assume: <ul> <li>  <code>HASH</code> - partition key </li> <li>  <code>RANGE</code> - sort key </li> </ul> </li> </ul> <note> The partition key of an item is also known as its <i>hash attribute</i>. The term "hash attribute" derives from the DynamoDB usage of an internal hash function to evenly distribute data items across partitions, based on their partition key values. The sort key of an item is also known as its <i>range attribute</i>. The term "range attribute" derives from the way DynamoDB stores items with the same partition key physically close together, in sorted order by the sort key value. </note> For a simple primary key (partition key), you must provide exactly one element with a <code>KeyType</code> of <code>HASH</code>. For a composite primary key (partition key and sort key), you must provide exactly two elements, in this order: The first element must have a <code>KeyType</code> of <code>HASH</code>, and the second element must have a <code>KeyType</code> of <code>RANGE</code>. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/WorkingWithTables.html#WorkingWithTables.primary.key">Working with Tables</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>TableName:  The name of the table to create.<br/>AttributeDefinitions:  An array of attributes that describe the key schema for the table and indexes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WriteCapacityUnits</i> The maximum number of writes consumed per second before DynamoDB returns a <code>ThrottlingException</code>. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/WorkingWithTables.html#ProvisionedThroughput">Specifying Read and Write Requirements</a> in the <i>Amazon DynamoDB Developer Guide</i>. If read/write capacity mode is <code>PAY_PER_REQUEST</code> the value is set to 0.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadCapacityUnits</i> The maximum number of strongly consistent reads consumed per second before DynamoDB returns a <code>ThrottlingException</code>. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/WorkingWithTables.html#ProvisionedThroughput">Specifying Read and Write Requirements</a> in the <i>Amazon DynamoDB Developer Guide</i>. If read/write capacity mode is <code>PAY_PER_REQUEST</code> the value is set to 0.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamViewType</i>  When an item in the table is modified, <code>StreamViewType</code> determines what information is written to the stream for this table. Valid values for <code>StreamViewType</code> are: <ul> <li>  <code>KEYS_ONLY</code> - Only the key attributes of the modified item are written to the stream. </li> <li>  <code>NEW_IMAGE</code> - The entire item, as it appears after it was modified, is written to the stream. </li> <li>  <code>OLD_IMAGE</code> - The entire item, as it appeared before it was modified, is written to the stream. </li> <li>  <code>NEW_AND_OLD_IMAGES</code> - Both the new and the old item images of the item are written to the stream. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamEnabled</i> Indicates whether DynamoDB Streams is enabled (true) or disabled (false) on the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KMSMasterKeyId</i> The KMS Customer Master Key (CMK) which should be used for the KMS encryption. To specify a CMK, use its key ID, Amazon Resource Name (ARN), alias name, or alias ARN. Note that you should only provide this parameter if the key is different from the default DynamoDB Customer Master Key alias/aws/dynamodb.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SSEType</i> Server-side encryption type. The only supported value is: <ul> <li>  <code>KMS</code> - Server-side encryption which uses AWS Key Management Service. Key is stored in your account and is managed by AWS KMS (KMS charges apply). </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Indicates whether server-side encryption is done using an AWS managed CMK or an AWS owned CMK. If enabled (true), server-side encryption type is set to <code>KMS</code> and an AWS managed CMK is used (AWS KMS charges apply). If disabled (false) or not specified, server-side encryption is set to AWS owned CMK.<br/>
        :return: TableDescription:  Represents the properties of the table.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SSEDescription</i> The description of the server-side encryption status on the specified table.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SSEDescription</b>: The description of the server-side encryption status on the specified table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreSummary</i> Contains details for the restore.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RestoreSummary</b>: Contains details for the restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestStreamArn</i> The Amazon Resource Name (ARN) that uniquely identifies the latest stream for this table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestStreamLabel</i> A timestamp, in ISO 8601 format, for this stream. Note that <code>LatestStreamLabel</code> is not a unique identifier for the stream, because it is possible that a stream from another table might have the same timestamp. However, the combination of the following three elements is guaranteed to be unique: <ul> <li> the AWS customer ID. </li> <li> the table name. </li> <li> the <code>StreamLabel</code>. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamSpecification</i> The current DynamoDB Streams configuration for the table.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamSpecification</b>: The current DynamoDB Streams configuration for the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalSecondaryIndexes</i> The global secondary indexes, if any, on the table. Each index is scoped to a given partition key value. Each element is composed of: <ul> <li>  <code>Backfilling</code> - If true, then the index is currently in the backfilling phase. Backfilling occurs only when a new global secondary index is added to the table; it is the process by which DynamoDB populates the new index with data from the table. (This attribute does not appear for indexes that were created during a <code>CreateTable</code> operation.) </li> <li>  <code>IndexName</code> - The name of the global secondary index. </li> <li>  <code>IndexSizeBytes</code> - The total size of the global secondary index, in bytes. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value.  </li> <li>  <code>IndexStatus</code> - The current status of the global secondary index: <ul> <li>  <code>CREATING</code> - The index is being created. </li> <li>  <code>UPDATING</code> - The index is being updated. </li> <li>  <code>DELETING</code> - The index is being deleted. </li> <li>  <code>ACTIVE</code> - The index is ready for use. </li> </ul> </li> <li>  <code>ItemCount</code> - The number of items in the global secondary index. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value.  </li> <li>  <code>KeySchema</code> - Specifies the complete index key schema. The attribute names in the key schema must be between 1 and 255 characters (inclusive). The key schema must begin with the same partition key as the table. </li> <li>  <code>Projection</code> - Specifies attributes that are copied (projected) from the table into the index. These are in addition to the primary key attributes and index key attributes, which are automatically projected. Each attribute specification is composed of: <ul> <li>  <code>ProjectionType</code> - One of the following: <ul> <li>  <code>KEYS_ONLY</code> - Only the index and primary keys are projected into the index. </li> <li>  <code>INCLUDE</code> - Only the specified table attributes are projected into the index. The list of projected attributes are in <code>NonKeyAttributes</code>. </li> <li>  <code>ALL</code> - All of the table attributes are projected into the index. </li> </ul> </li> <li>  <code>NonKeyAttributes</code> - A list of one or more non-key attribute names that are projected into the secondary index. The total count of attributes provided in <code>NonKeyAttributes</code>, summed across all of the secondary indexes, must not exceed 20. If you project the same attribute into two different indexes, this counts as two distinct attributes when determining the total. </li> </ul> </li> <li>  <code>ProvisionedThroughput</code> - The provisioned throughput settings for the global secondary index, consisting of read and write capacity units, along with data about increases and decreases.  </li> </ul> If the table is in the <code>DELETING</code> state, no information about indexes will be returned.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LocalSecondaryIndexes</i> Represents one or more local secondary indexes on the table. Each index is scoped to a given partition key value. Tables with one or more local secondary indexes are subject to an item collection size limit, where the amount of data within a given item collection cannot exceed 10 GB. Each element is composed of: <ul> <li>  <code>IndexName</code> - The name of the local secondary index. </li> <li>  <code>KeySchema</code> - Specifies the complete index key schema. The attribute names in the key schema must be between 1 and 255 characters (inclusive). The key schema must begin with the same partition key as the table. </li> <li>  <code>Projection</code> - Specifies attributes that are copied (projected) from the table into the index. These are in addition to the primary key attributes and index key attributes, which are automatically projected. Each attribute specification is composed of: <ul> <li>  <code>ProjectionType</code> - One of the following: <ul> <li>  <code>KEYS_ONLY</code> - Only the index and primary keys are projected into the index. </li> <li>  <code>INCLUDE</code> - Only the specified table attributes are projected into the index. The list of projected attributes are in <code>NonKeyAttributes</code>. </li> <li>  <code>ALL</code> - All of the table attributes are projected into the index. </li> </ul> </li> <li>  <code>NonKeyAttributes</code> - A list of one or more non-key attribute names that are projected into the secondary index. The total count of attributes provided in <code>NonKeyAttributes</code>, summed across all of the secondary indexes, must not exceed 20. If you project the same attribute into two different indexes, this counts as two distinct attributes when determining the total. </li> </ul> </li> <li>  <code>IndexSizeBytes</code> - Represents the total size of the index, in bytes. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value. </li> <li>  <code>ItemCount</code> - Represents the number of items in the index. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value. </li> </ul> If the table is in the <code>DELETING</code> state, no information about indexes will be returned.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BillingModeSummary</i> Contains the details for the read/write capacity mode.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BillingModeSummary</b>: Contains the details for the read/write capacity mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableId</i> Unique identifier for the table for which the backup was created. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableArn</i> The Amazon Resource Name (ARN) that uniquely identifies the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ItemCount</i> The number of items in the specified table. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableSizeBytes</i> The total size of the specified table, in bytes. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisionedThroughput</i> The provisioned throughput settings for the table, consisting of read and write capacity units, along with data about increases and decreases.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ProvisionedThroughput</b>: The provisioned throughput settings for the table, consisting of read and write capacity units, along with data about increases and decreases.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDateTime</i> The date and time when the table was created, in <a href="http://www.epochconverter.com/">UNIX epoch time</a> format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableStatus</i> The current state of the table: <ul> <li>  <code>CREATING</code> - The table is being created. </li> <li>  <code>UPDATING</code> - The table is being updated. </li> <li>  <code>DELETING</code> - The table is being deleted. </li> <li>  <code>ACTIVE</code> - The table is ready for use. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeySchema</i> The primary key structure for the table. Each <code>KeySchemaElement</code> consists of: <ul> <li>  <code>AttributeName</code> - The name of the attribute. </li> <li>  <code>KeyType</code> - The role of the attribute: <ul> <li>  <code>HASH</code> - partition key </li> <li>  <code>RANGE</code> - sort key </li> </ul> <note> The partition key of an item is also known as its <i>hash attribute</i>. The term "hash attribute" derives from DynamoDB's usage of an internal hash function to evenly distribute data items across partitions, based on their partition key values. The sort key of an item is also known as its <i>range attribute</i>. The term "range attribute" derives from the way DynamoDB stores items with the same partition key physically close together, in sorted order by the sort key value. </note> </li> </ul> For more information about primary keys, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/DataModel.html#DataModelPrimaryKey">Primary Key</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableName</i> The name of the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttributeDefinitions</i> An array of <code>AttributeDefinition</code> objects. Each of these objects describes one attribute in the table and index key schema. Each <code>AttributeDefinition</code> object in this array is composed of: <ul> <li>  <code>AttributeName</code> - The name of the attribute. </li> <li>  <code>AttributeType</code> - The data type for the attribute. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WriteCapacityUnits</i> The maximum number of writes consumed per second before DynamoDB returns a <code>ThrottlingException</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadCapacityUnits</i> The maximum number of strongly consistent reads consumed per second before DynamoDB returns a <code>ThrottlingException</code>. Eventually consistent reads require less effort than strongly consistent reads, so a setting of 50 <code>ReadCapacityUnits</code> per second provides 100 eventually consistent <code>ReadCapacityUnits</code> per second.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfDecreasesToday</i> The number of provisioned throughput decreases for this table during this UTC calendar day. For current maximums on provisioned throughput decreases, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Limits.html">Limits</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LastDecreaseDateTime</i> The date and time of the last provisioned throughput decrease for this table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LastIncreaseDateTime</i> The date and time of the last provisioned throughput increase for this table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdateToPayPerRequestDateTime</i> Represents the time when <code>PAY_PER_REQUEST</code> was last set as the read/write capacity mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BillingMode</i> Controls how you are charged for read and write throughput and how you manage capacity. This setting can be changed later. <ul> <li>  <code>PROVISIONED</code> - Sets the read/write capacity mode to <code>PROVISIONED</code>. We recommend using <code>PROVISIONED</code> for predictable workloads. </li> <li>  <code>PAY_PER_REQUEST</code> - Sets the read/write capacity mode to <code>PAY_PER_REQUEST</code>. We recommend using <code>PAY_PER_REQUEST</code> for unpredictable workloads.  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamViewType</i>  When an item in the table is modified, <code>StreamViewType</code> determines what information is written to the stream for this table. Valid values for <code>StreamViewType</code> are: <ul> <li>  <code>KEYS_ONLY</code> - Only the key attributes of the modified item are written to the stream. </li> <li>  <code>NEW_IMAGE</code> - The entire item, as it appears after it was modified, is written to the stream. </li> <li>  <code>OLD_IMAGE</code> - The entire item, as it appeared before it was modified, is written to the stream. </li> <li>  <code>NEW_AND_OLD_IMAGES</code> - Both the new and the old item images of the item are written to the stream. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamEnabled</i> Indicates whether DynamoDB Streams is enabled (true) or disabled (false) on the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreInProgress</i> Indicates if a restore is in progress or not.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreDateTime</i> Point in time or source backup time.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceTableArn</i> ARN of the source table of the backup that is being restored.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceBackupArn</i> ARN of the backup from which the table was restored.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KMSMasterKeyArn</i> The KMS customer master key (CMK) ARN used for the KMS encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SSEType</i> Server-side encryption type. The only supported value is: <ul> <li>  <code>KMS</code> - Server-side encryption which uses AWS Key Management Service. Key is stored in your account and is managed by AWS KMS (KMS charges apply). </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Represents the current state of server-side encryption. The only supported values are: <ul> <li>  <code>ENABLED</code> - Server-side encryption is enabled. </li> <li>  <code>UPDATING</code> - Server-side encryption is being updated. </li> </ul><br/>        """
        pass

    def delete_backup(self, BackupArn: str) -> DeleteBackupOutput:
        """Deletes an existing backup of a table. You can call <code>DeleteBackup</code> at a maximum rate of 10 times per second.

        :param self:
        :param BackupArn:  The ARN associated with the backup.<br/>
        :return: BackupDescription:  Contains the description of the backup created for the table.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceTableFeatureDetails</i> Contains the details of the features enabled on the table when the backup was created. For example, LSIs, GSIs, streams, TTL.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SourceTableFeatureDetails</b>: Contains the details of the features enabled on the table when the backup was created. For example, LSIs, GSIs, streams, TTL.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceTableDetails</i> Contains the details of the table when the backup was created. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SourceTableDetails</b>: Contains the details of the table when the backup was created. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupDetails</i> Contains the details of the backup created for the table. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BackupDetails</b>: Contains the details of the backup created for the table. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupExpiryDateTime</i> Time at which the automatic on-demand backup created by DynamoDB will expire. This <code>SYSTEM</code> on-demand backup expires automatically 35 days after its creation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupCreationDateTime</i> Time at which the backup was created. This is the request time of the backup. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupType</i> BackupType: <ul> <li>  <code>USER</code> - You create and manage these using the on-demand backup feature. </li> <li>  <code>SYSTEM</code> - If you delete a table with point-in-time recovery enabled, a <code>SYSTEM</code> backup is automatically created and is retained for 35 days (at no additional cost). System backups allow you to restore the deleted table to the state it was in just before the point of deletion.  </li> <li>  <code>AWS_BACKUP</code> - On-demand backup created by you from AWS Backup service. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupStatus</i> Backup can be in one of the following states: CREATING, ACTIVE, DELETED. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupSizeBytes</i> Size of the backup in bytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupName</i> Name of the requested backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupArn</i> ARN associated with the backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TableCreationDateTime</i> Time when the source table was created. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KeySchema</i> Schema of the table. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TableSizeBytes</i> Size of the table in bytes. Please note this is an approximate value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TableArn</i> ARN of the table for which backup was created. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TableId</i> Unique identifier for the table for which the backup was created. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TableName</i> The name of the table for which the backup was created. <br/>Read IOPs and Write IOPS on the table when the backup was created.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ProvisionedThroughput</b>: Read IOPs and Write IOPS on the table when the backup was created.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalSecondaryIndexes</i> Represents the GSI properties for the table when the backup was created. It includes the IndexName, KeySchema, Projection and ProvisionedThroughput for the GSIs on the table at the time of backup. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LocalSecondaryIndexes</i> Represents the LSI properties for the table when the backup was created. It includes the IndexName, KeySchema and Projection for the LSIs on the table at the time of backup. <br/>Stream settings on the table when the backup was created.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamDescription</b>: Stream settings on the table when the backup was created.        """
        pass

    def delete_item(self, Key: Key, TableName: str, Expected: ExpectedAttributeMap=None, ConditionalOperator: str=None, ReturnValues: str=None, ReturnConsumedCapacity: str=None, ReturnItemCollectionMetrics: str=None, ConditionExpression: str=None, ExpressionAttributeNames: ExpressionAttributeNameMap=None, ExpressionAttributeValues: ExpressionAttributeValueMap=None) -> DeleteItemOutput:
        """Deletes a single item in a table by primary key. You can perform a conditional delete operation that deletes the item if it exists, or if it has an expected attribute value. In addition to deleting an item, you can also return the item's attribute values in the same operation, using the <code>ReturnValues</code> parameter. Unless you specify conditions, the <code>DeleteItem</code> is an idempotent operation; running it multiple times on the same item or attribute does <i>not</i> result in an error response. Conditional deletes are useful for deleting items only if specific conditions are met. If those conditions are met, DynamoDB performs the delete. Otherwise, the item is not deleted.

        :param self:
        :param ExpressionAttributeValues:  One or more values that can be substituted in an expression. Use the <b>:</b> (colon) character in an expression to dereference an attribute value. For example, suppose that you wanted to check whether the value of the <i>ProductStatus</i> attribute was one of the following:   <code>Available | Backordered | Discontinued</code>  You would first need to specify <code>ExpressionAttributeValues</code> as follows:  <code>{ ":avail":{"S":"Available"}, ":back":{"S":"Backordered"}, ":disc":{"S":"Discontinued"} }</code>  You could then use these values in an expression, such as this:  <code>ProductStatus IN (:avail, :back, :disc)</code>  For more information on expression attribute values, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Expressions.SpecifyingConditions.html">Condition Expressions</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>ExpressionAttributeNames:  One or more substitution tokens for attribute names in an expression. The following are some use cases for using <code>ExpressionAttributeNames</code>: <ul> <li> To access an attribute whose name conflicts with a DynamoDB reserved word. </li> <li> To create a placeholder for repeating occurrences of an attribute name in an expression. </li> <li> To prevent special characters in an attribute name from being misinterpreted in an expression. </li> </ul> Use the <b>#</b> character in an expression to dereference an attribute name. For example, consider the following attribute name: <ul> <li>  <code>Percentile</code>  </li> </ul> The name of this attribute conflicts with a reserved word, so it cannot be used directly in an expression. (For the complete list of reserved words, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/ReservedWords.html">Reserved Words</a> in the <i>Amazon DynamoDB Developer Guide</i>). To work around this, you could specify the following for <code>ExpressionAttributeNames</code>: <ul> <li>  <code>{"#P":"Percentile"}</code>  </li> </ul> You could then use this substitution in an expression, as in this example: <ul> <li>  <code>#P = :val</code>  </li> </ul> <note> Tokens that begin with the <b>:</b> character are <i>expression attribute values</i>, which are placeholders for the actual value at runtime. </note> For more information on expression attribute names, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Expressions.AccessingItemAttributes.html">Specifying Item Attributes</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>ConditionExpression:  A condition that must be satisfied in order for a conditional <code>DeleteItem</code> to succeed. An expression can contain any of the following: <ul> <li> Functions: <code>attribute_exists | attribute_not_exists | attribute_type | contains | begins_with | size</code>  These function names are case-sensitive. </li> <li> Comparison operators: <code>= | &lt;&gt; | &lt; | &gt; | &lt;= | &gt;= | BETWEEN | IN </code>  </li> <li>  Logical operators: <code>AND | OR | NOT</code>  </li> </ul> For more information about condition expressions, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Expressions.SpecifyingConditions.html">Condition Expressions</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>ReturnItemCollectionMetrics:  Determines whether item collection metrics are returned. If set to <code>SIZE</code>, the response includes statistics about item collections, if any, that were modified during the operation are returned in the response. If set to <code>NONE</code> (the default), no statistics are returned.<br/>ReturnConsumedCapacity:  Determines the level of detail about provisioned throughput consumption that is returned in the response: <ul> <li>  <code>INDEXES</code> - The response includes the aggregate <code>ConsumedCapacity</code> for the operation, together with <code>ConsumedCapacity</code> for each table and secondary index that was accessed. Note that some operations, such as <code>GetItem</code> and <code>BatchGetItem</code>, do not access any indexes at all. In these cases, specifying <code>INDEXES</code> will only return <code>ConsumedCapacity</code> information for table(s). </li> <li>  <code>TOTAL</code> - The response includes only the aggregate <code>ConsumedCapacity</code> for the operation. </li> <li>  <code>NONE</code> - No <code>ConsumedCapacity</code> details are included in the response. </li> </ul><br/>ReturnValues:  Use <code>ReturnValues</code> if you want to get the item attributes as they appeared before they were deleted. For <code>DeleteItem</code>, the valid values are: <ul> <li>  <code>NONE</code> - If <code>ReturnValues</code> is not specified, or if its value is <code>NONE</code>, then nothing is returned. (This setting is the default for <code>ReturnValues</code>.) </li> <li>  <code>ALL_OLD</code> - The content of the old item is returned. </li> </ul> <note> The <code>ReturnValues</code> parameter is used by several DynamoDB operations; however, <code>DeleteItem</code> does not recognize any values other than <code>NONE</code> or <code>ALL_OLD</code>. </note><br/>ConditionalOperator:  This is a legacy parameter. Use <code>ConditionExpression</code> instead. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/LegacyConditionalParameters.ConditionalOperator.html">ConditionalOperator</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>Expected:  This is a legacy parameter. Use <code>ConditionExpression</code> instead. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/LegacyConditionalParameters.Expected.html">Expected</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>Key:  A map of attribute names to <code>AttributeValue</code> objects, representing the primary key of the item to delete. For the primary key, you must provide all of the attributes. For example, with a simple primary key, you only need to provide a value for the partition key. For a composite primary key, you must provide values for both the partition key and the sort key.<br/>TableName:  The name of the table from which to delete the item.<br/>
        :return: ItemCollectionMetrics:  Information about item collections, if any, that were affected by the <code>DeleteItem</code> operation. <code>ItemCollectionMetrics</code> is only returned if the <code>ReturnItemCollectionMetrics</code> parameter was specified. If the table does not have any local secondary indexes, this information is not returned in the response. Each <code>ItemCollectionMetrics</code> element consists of: <ul> <li>  <code>ItemCollectionKey</code> - The partition key value of the item collection. This is the same as the partition key value of the item itself. </li> <li>  <code>SizeEstimateRangeGB</code> - An estimate of item collection size, in gigabytes. This value is a two-element array containing a lower bound and an upper bound for the estimate. The estimate includes the size of all the items in the table, plus the size of all attributes projected into all of the local secondary indexes on that table. Use this estimate to measure whether a local secondary index is approaching its size limit. The estimate is subject to change over time; therefore, do not rely on the precision or accuracy of the estimate. </li> </ul>:<br/>ConsumedCapacity:  The capacity units consumed by the <code>DeleteItem</code> operation. The data returned includes the total provisioned throughput consumed, along with statistics for the table and any indexes involved in the operation. <code>ConsumedCapacity</code> is only returned if the <code>ReturnConsumedCapacity</code> parameter was specified. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/ProvisionedThroughputIntro.html">Provisioned Mode</a> in the <i>Amazon DynamoDB Developer Guide</i>.:<br/>Attributes:  A map of attribute names to <code>AttributeValue</code> objects, representing the item as it appeared before the <code>DeleteItem</code> operation. This map appears in the response only if <code>ReturnValues</code> was specified as <code>ALL_OLD</code> in the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalSecondaryIndexes</i> The amount of throughput consumed on each global index affected by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LocalSecondaryIndexes</i> The amount of throughput consumed on each local index affected by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Table</i> The amount of throughput consumed on the table affected by the operation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Table</b>: The amount of throughput consumed on the table affected by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WriteCapacityUnits</i> The total number of write capacity units consumed by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadCapacityUnits</i> The total number of read capacity units consumed by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CapacityUnits</i> The total number of capacity units consumed by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableName</i> The name of the table that was affected by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CapacityUnits</i> The total number of capacity units consumed on a table or an index.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WriteCapacityUnits</i> The total number of write capacity units consumed on a table or an index.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadCapacityUnits</i> The total number of read capacity units consumed on a table or an index.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SizeEstimateRangeGB</i> An estimate of item collection size, in gigabytes. This value is a two-element array containing a lower bound and an upper bound for the estimate. The estimate includes the size of all the items in the table, plus the size of all attributes projected into all of the local secondary indexes on that table. Use this estimate to measure whether a local secondary index is approaching its size limit. The estimate is subject to change over time; therefore, do not rely on the precision or accuracy of the estimate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ItemCollectionKey</i> The partition key value of the item collection. This value is the same as the partition key value of the item.<br/>        """
        pass

    def delete_table(self, TableName: str) -> DeleteTableOutput:
        """The <code>DeleteTable</code> operation deletes a table and all of its items. After a <code>DeleteTable</code> request, the specified table is in the <code>DELETING</code> state until DynamoDB completes the deletion. If the table is in the <code>ACTIVE</code> state, you can delete it. If a table is in <code>CREATING</code> or <code>UPDATING</code> states, then DynamoDB returns a <code>ResourceInUseException</code>. If the specified table does not exist, DynamoDB returns a <code>ResourceNotFoundException</code>. If table is already in the <code>DELETING</code> state, no error is returned.  <note> DynamoDB might continue to accept data read and write operations, such as <code>GetItem</code> and <code>PutItem</code>, on a table in the <code>DELETING</code> state until the table deletion is complete. </note> When you delete a table, any indexes on that table are also deleted. If you have DynamoDB Streams enabled on the table, then the corresponding stream on that table goes into the <code>DISABLED</code> state, and the stream is automatically deleted after 24 hours. Use the <code>DescribeTable</code> action to check the status of the table. 

        :param self:
        :param TableName:  The name of the table to delete.<br/>
        :return: TableDescription:  Represents the properties of a table.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SSEDescription</i> The description of the server-side encryption status on the specified table.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SSEDescription</b>: The description of the server-side encryption status on the specified table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreSummary</i> Contains details for the restore.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RestoreSummary</b>: Contains details for the restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestStreamArn</i> The Amazon Resource Name (ARN) that uniquely identifies the latest stream for this table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestStreamLabel</i> A timestamp, in ISO 8601 format, for this stream. Note that <code>LatestStreamLabel</code> is not a unique identifier for the stream, because it is possible that a stream from another table might have the same timestamp. However, the combination of the following three elements is guaranteed to be unique: <ul> <li> the AWS customer ID. </li> <li> the table name. </li> <li> the <code>StreamLabel</code>. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamSpecification</i> The current DynamoDB Streams configuration for the table.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamSpecification</b>: The current DynamoDB Streams configuration for the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalSecondaryIndexes</i> The global secondary indexes, if any, on the table. Each index is scoped to a given partition key value. Each element is composed of: <ul> <li>  <code>Backfilling</code> - If true, then the index is currently in the backfilling phase. Backfilling occurs only when a new global secondary index is added to the table; it is the process by which DynamoDB populates the new index with data from the table. (This attribute does not appear for indexes that were created during a <code>CreateTable</code> operation.) </li> <li>  <code>IndexName</code> - The name of the global secondary index. </li> <li>  <code>IndexSizeBytes</code> - The total size of the global secondary index, in bytes. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value.  </li> <li>  <code>IndexStatus</code> - The current status of the global secondary index: <ul> <li>  <code>CREATING</code> - The index is being created. </li> <li>  <code>UPDATING</code> - The index is being updated. </li> <li>  <code>DELETING</code> - The index is being deleted. </li> <li>  <code>ACTIVE</code> - The index is ready for use. </li> </ul> </li> <li>  <code>ItemCount</code> - The number of items in the global secondary index. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value.  </li> <li>  <code>KeySchema</code> - Specifies the complete index key schema. The attribute names in the key schema must be between 1 and 255 characters (inclusive). The key schema must begin with the same partition key as the table. </li> <li>  <code>Projection</code> - Specifies attributes that are copied (projected) from the table into the index. These are in addition to the primary key attributes and index key attributes, which are automatically projected. Each attribute specification is composed of: <ul> <li>  <code>ProjectionType</code> - One of the following: <ul> <li>  <code>KEYS_ONLY</code> - Only the index and primary keys are projected into the index. </li> <li>  <code>INCLUDE</code> - Only the specified table attributes are projected into the index. The list of projected attributes are in <code>NonKeyAttributes</code>. </li> <li>  <code>ALL</code> - All of the table attributes are projected into the index. </li> </ul> </li> <li>  <code>NonKeyAttributes</code> - A list of one or more non-key attribute names that are projected into the secondary index. The total count of attributes provided in <code>NonKeyAttributes</code>, summed across all of the secondary indexes, must not exceed 20. If you project the same attribute into two different indexes, this counts as two distinct attributes when determining the total. </li> </ul> </li> <li>  <code>ProvisionedThroughput</code> - The provisioned throughput settings for the global secondary index, consisting of read and write capacity units, along with data about increases and decreases.  </li> </ul> If the table is in the <code>DELETING</code> state, no information about indexes will be returned.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LocalSecondaryIndexes</i> Represents one or more local secondary indexes on the table. Each index is scoped to a given partition key value. Tables with one or more local secondary indexes are subject to an item collection size limit, where the amount of data within a given item collection cannot exceed 10 GB. Each element is composed of: <ul> <li>  <code>IndexName</code> - The name of the local secondary index. </li> <li>  <code>KeySchema</code> - Specifies the complete index key schema. The attribute names in the key schema must be between 1 and 255 characters (inclusive). The key schema must begin with the same partition key as the table. </li> <li>  <code>Projection</code> - Specifies attributes that are copied (projected) from the table into the index. These are in addition to the primary key attributes and index key attributes, which are automatically projected. Each attribute specification is composed of: <ul> <li>  <code>ProjectionType</code> - One of the following: <ul> <li>  <code>KEYS_ONLY</code> - Only the index and primary keys are projected into the index. </li> <li>  <code>INCLUDE</code> - Only the specified table attributes are projected into the index. The list of projected attributes are in <code>NonKeyAttributes</code>. </li> <li>  <code>ALL</code> - All of the table attributes are projected into the index. </li> </ul> </li> <li>  <code>NonKeyAttributes</code> - A list of one or more non-key attribute names that are projected into the secondary index. The total count of attributes provided in <code>NonKeyAttributes</code>, summed across all of the secondary indexes, must not exceed 20. If you project the same attribute into two different indexes, this counts as two distinct attributes when determining the total. </li> </ul> </li> <li>  <code>IndexSizeBytes</code> - Represents the total size of the index, in bytes. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value. </li> <li>  <code>ItemCount</code> - Represents the number of items in the index. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value. </li> </ul> If the table is in the <code>DELETING</code> state, no information about indexes will be returned.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BillingModeSummary</i> Contains the details for the read/write capacity mode.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BillingModeSummary</b>: Contains the details for the read/write capacity mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableId</i> Unique identifier for the table for which the backup was created. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableArn</i> The Amazon Resource Name (ARN) that uniquely identifies the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ItemCount</i> The number of items in the specified table. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableSizeBytes</i> The total size of the specified table, in bytes. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisionedThroughput</i> The provisioned throughput settings for the table, consisting of read and write capacity units, along with data about increases and decreases.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ProvisionedThroughput</b>: The provisioned throughput settings for the table, consisting of read and write capacity units, along with data about increases and decreases.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDateTime</i> The date and time when the table was created, in <a href="http://www.epochconverter.com/">UNIX epoch time</a> format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableStatus</i> The current state of the table: <ul> <li>  <code>CREATING</code> - The table is being created. </li> <li>  <code>UPDATING</code> - The table is being updated. </li> <li>  <code>DELETING</code> - The table is being deleted. </li> <li>  <code>ACTIVE</code> - The table is ready for use. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeySchema</i> The primary key structure for the table. Each <code>KeySchemaElement</code> consists of: <ul> <li>  <code>AttributeName</code> - The name of the attribute. </li> <li>  <code>KeyType</code> - The role of the attribute: <ul> <li>  <code>HASH</code> - partition key </li> <li>  <code>RANGE</code> - sort key </li> </ul> <note> The partition key of an item is also known as its <i>hash attribute</i>. The term "hash attribute" derives from DynamoDB's usage of an internal hash function to evenly distribute data items across partitions, based on their partition key values. The sort key of an item is also known as its <i>range attribute</i>. The term "range attribute" derives from the way DynamoDB stores items with the same partition key physically close together, in sorted order by the sort key value. </note> </li> </ul> For more information about primary keys, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/DataModel.html#DataModelPrimaryKey">Primary Key</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableName</i> The name of the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttributeDefinitions</i> An array of <code>AttributeDefinition</code> objects. Each of these objects describes one attribute in the table and index key schema. Each <code>AttributeDefinition</code> object in this array is composed of: <ul> <li>  <code>AttributeName</code> - The name of the attribute. </li> <li>  <code>AttributeType</code> - The data type for the attribute. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WriteCapacityUnits</i> The maximum number of writes consumed per second before DynamoDB returns a <code>ThrottlingException</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadCapacityUnits</i> The maximum number of strongly consistent reads consumed per second before DynamoDB returns a <code>ThrottlingException</code>. Eventually consistent reads require less effort than strongly consistent reads, so a setting of 50 <code>ReadCapacityUnits</code> per second provides 100 eventually consistent <code>ReadCapacityUnits</code> per second.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfDecreasesToday</i> The number of provisioned throughput decreases for this table during this UTC calendar day. For current maximums on provisioned throughput decreases, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Limits.html">Limits</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LastDecreaseDateTime</i> The date and time of the last provisioned throughput decrease for this table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LastIncreaseDateTime</i> The date and time of the last provisioned throughput increase for this table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdateToPayPerRequestDateTime</i> Represents the time when <code>PAY_PER_REQUEST</code> was last set as the read/write capacity mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BillingMode</i> Controls how you are charged for read and write throughput and how you manage capacity. This setting can be changed later. <ul> <li>  <code>PROVISIONED</code> - Sets the read/write capacity mode to <code>PROVISIONED</code>. We recommend using <code>PROVISIONED</code> for predictable workloads. </li> <li>  <code>PAY_PER_REQUEST</code> - Sets the read/write capacity mode to <code>PAY_PER_REQUEST</code>. We recommend using <code>PAY_PER_REQUEST</code> for unpredictable workloads.  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamViewType</i>  When an item in the table is modified, <code>StreamViewType</code> determines what information is written to the stream for this table. Valid values for <code>StreamViewType</code> are: <ul> <li>  <code>KEYS_ONLY</code> - Only the key attributes of the modified item are written to the stream. </li> <li>  <code>NEW_IMAGE</code> - The entire item, as it appears after it was modified, is written to the stream. </li> <li>  <code>OLD_IMAGE</code> - The entire item, as it appeared before it was modified, is written to the stream. </li> <li>  <code>NEW_AND_OLD_IMAGES</code> - Both the new and the old item images of the item are written to the stream. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamEnabled</i> Indicates whether DynamoDB Streams is enabled (true) or disabled (false) on the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreInProgress</i> Indicates if a restore is in progress or not.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreDateTime</i> Point in time or source backup time.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceTableArn</i> ARN of the source table of the backup that is being restored.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceBackupArn</i> ARN of the backup from which the table was restored.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KMSMasterKeyArn</i> The KMS customer master key (CMK) ARN used for the KMS encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SSEType</i> Server-side encryption type. The only supported value is: <ul> <li>  <code>KMS</code> - Server-side encryption which uses AWS Key Management Service. Key is stored in your account and is managed by AWS KMS (KMS charges apply). </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Represents the current state of server-side encryption. The only supported values are: <ul> <li>  <code>ENABLED</code> - Server-side encryption is enabled. </li> <li>  <code>UPDATING</code> - Server-side encryption is being updated. </li> </ul><br/>        """
        pass

    def describe_backup(self, BackupArn: str) -> DescribeBackupOutput:
        """Describes an existing backup of a table. You can call <code>DescribeBackup</code> at a maximum rate of 10 times per second.

        :param self:
        :param BackupArn:  The Amazon Resource Name (ARN) associated with the backup.<br/>
        :return: BackupDescription:  Contains the description of the backup created for the table.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceTableFeatureDetails</i> Contains the details of the features enabled on the table when the backup was created. For example, LSIs, GSIs, streams, TTL.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SourceTableFeatureDetails</b>: Contains the details of the features enabled on the table when the backup was created. For example, LSIs, GSIs, streams, TTL.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceTableDetails</i> Contains the details of the table when the backup was created. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SourceTableDetails</b>: Contains the details of the table when the backup was created. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupDetails</i> Contains the details of the backup created for the table. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BackupDetails</b>: Contains the details of the backup created for the table. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupExpiryDateTime</i> Time at which the automatic on-demand backup created by DynamoDB will expire. This <code>SYSTEM</code> on-demand backup expires automatically 35 days after its creation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupCreationDateTime</i> Time at which the backup was created. This is the request time of the backup. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupType</i> BackupType: <ul> <li>  <code>USER</code> - You create and manage these using the on-demand backup feature. </li> <li>  <code>SYSTEM</code> - If you delete a table with point-in-time recovery enabled, a <code>SYSTEM</code> backup is automatically created and is retained for 35 days (at no additional cost). System backups allow you to restore the deleted table to the state it was in just before the point of deletion.  </li> <li>  <code>AWS_BACKUP</code> - On-demand backup created by you from AWS Backup service. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupStatus</i> Backup can be in one of the following states: CREATING, ACTIVE, DELETED. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupSizeBytes</i> Size of the backup in bytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupName</i> Name of the requested backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupArn</i> ARN associated with the backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TableCreationDateTime</i> Time when the source table was created. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KeySchema</i> Schema of the table. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TableSizeBytes</i> Size of the table in bytes. Please note this is an approximate value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TableArn</i> ARN of the table for which backup was created. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TableId</i> Unique identifier for the table for which the backup was created. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TableName</i> The name of the table for which the backup was created. <br/>Read IOPs and Write IOPS on the table when the backup was created.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ProvisionedThroughput</b>: Read IOPs and Write IOPS on the table when the backup was created.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalSecondaryIndexes</i> Represents the GSI properties for the table when the backup was created. It includes the IndexName, KeySchema, Projection and ProvisionedThroughput for the GSIs on the table at the time of backup. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LocalSecondaryIndexes</i> Represents the LSI properties for the table when the backup was created. It includes the IndexName, KeySchema and Projection for the LSIs on the table at the time of backup. <br/>Stream settings on the table when the backup was created.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamDescription</b>: Stream settings on the table when the backup was created.        """
        pass

    def describe_continuous_backups(self, TableName: str) -> DescribeContinuousBackupsOutput:
        """Checks the status of continuous backups and point in time recovery on the specified table. Continuous backups are <code>ENABLED</code> on all tables at table creation. If point in time recovery is enabled, <code>PointInTimeRecoveryStatus</code> will be set to ENABLED.  After continuous backups and point in time recovery are enabled, you can restore to any point in time within <code>EarliestRestorableDateTime</code> and <code>LatestRestorableDateTime</code>.   <code>LatestRestorableDateTime</code> is typically 5 minutes before the current time. You can restore your table to any point in time during the last 35 days.  You can call <code>DescribeContinuousBackups</code> at a maximum rate of 10 times per second.

        :param self:
        :param TableName:  Name of the table for which the customer wants to check the continuous backups and point in time recovery settings.<br/>
        :return: ContinuousBackupsDescription:  Represents the continuous backups and point in time recovery settings on the table.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PointInTimeRecoveryDescription</i> The description of the point in time recovery settings applied to the table.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PointInTimeRecoveryDescription</b>: The description of the point in time recovery settings applied to the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContinuousBackupsStatus</i>  <code>ContinuousBackupsStatus</code> can be one of the following states: ENABLED, DISABLED<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableDateTime</i>  <code>LatestRestorableDateTime</code> is typically 5 minutes before the current time. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestRestorableDateTime</i> Specifies the earliest point in time you can restore your table to. It You can restore your table to any point in time during the last 35 days. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PointInTimeRecoveryStatus</i> The current state of point in time recovery: <ul> <li>  <code>ENABLING</code> - Point in time recovery is being enabled. </li> <li>  <code>ENABLED</code> - Point in time recovery is enabled. </li> <li>  <code>DISABLED</code> - Point in time recovery is disabled. </li> </ul><br/>        """
        pass

    def describe_endpoints(self) -> DescribeEndpointsResponse:
        """Returns the regional endpoint information.

        :param self:
        :param 
        :return: Endpoints:  List of endpoints.<br/>        """
        pass

    def describe_global_table(self, GlobalTableName: str) -> DescribeGlobalTableOutput:
        """Returns information about the specified global table.

        :param self:
        :param GlobalTableName:  The name of the global table.<br/>
        :return: GlobalTableDescription:  Contains the details of the global table.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalTableName</i> The global table name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalTableStatus</i> The current state of the global table: <ul> <li>  <code>CREATING</code> - The global table is being created. </li> <li>  <code>UPDATING</code> - The global table is being updated. </li> <li>  <code>DELETING</code> - The global table is being deleted. </li> <li>  <code>ACTIVE</code> - The global table is ready for use. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDateTime</i> The creation time of the global table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalTableArn</i> The unique identifier of the global table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationGroup</i> The regions where the global table has replicas.<br/>        """
        pass

    def describe_global_table_settings(self, GlobalTableName: str) -> DescribeGlobalTableSettingsOutput:
        """Describes Region-specific settings for a global table.

        :param self:
        :param GlobalTableName:  The name of the global table to describe.<br/>
        :return: ReplicaSettings:  The Region-specific settings for the global table.<br/>GlobalTableName:  The name of the global table.<br/>        """
        pass

    def describe_limits(self) -> DescribeLimitsOutput:
        """Returns the current provisioned-capacity limits for your AWS account in a Region, both for the Region as a whole and for any one DynamoDB table that you create there. When you establish an AWS account, the account has initial limits on the maximum read capacity units and write capacity units that you can provision across all of your DynamoDB tables in a given Region. Also, there are per-table limits that apply when you create a table there. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Limits.html">Limits</a> page in the <i>Amazon DynamoDB Developer Guide</i>. Although you can increase these limits by filing a case at <a href="https://console.aws.amazon.com/support/home#/">AWS Support Center</a>, obtaining the increase is not instantaneous. The <code>DescribeLimits</code> action lets you write code to compare the capacity you are currently using to those limits imposed by your account so that you have enough time to apply for an increase before you hit a limit. For example, you could use one of the AWS SDKs to do the following: <ol> <li> Call <code>DescribeLimits</code> for a particular Region to obtain your current account limits on provisioned capacity there. </li> <li> Create a variable to hold the aggregate read capacity units provisioned for all your tables in that Region, and one to hold the aggregate write capacity units. Zero them both. </li> <li> Call <code>ListTables</code> to obtain a list of all your DynamoDB tables. </li> <li> For each table name listed by <code>ListTables</code>, do the following: <ul> <li> Call <code>DescribeTable</code> with the table name. </li> <li> Use the data returned by <code>DescribeTable</code> to add the read capacity units and write capacity units provisioned for the table itself to your variables. </li> <li> If the table has one or more global secondary indexes (GSIs), loop over these GSIs and add their provisioned capacity values to your variables as well. </li> </ul> </li> <li> Report the account limits for that Region returned by <code>DescribeLimits</code>, along with the total current provisioned capacity levels you have calculated. </li> </ol> This will let you see whether you are getting close to your account-level limits. The per-table limits apply only when you are creating a new table. They restrict the sum of the provisioned capacity of the new table itself and all its global secondary indexes. For existing tables and their GSIs, DynamoDB doesn't let you increase provisioned capacity extremely rapidly. But the only upper limit that applies is that the aggregate provisioned capacity over all your tables and GSIs cannot exceed either of the per-account limits. <note>  <code>DescribeLimits</code> should only be called periodically. You can expect throttling errors if you call it more than once in a minute. </note> The <code>DescribeLimits</code> Request element has no content.

        :param self:
        :param 
        :return: TableMaxWriteCapacityUnits:  The maximum write capacity units that your account allows you to provision for a new table that you are creating in this Region, including the write capacity units provisioned for its global secondary indexes (GSIs).<br/>TableMaxReadCapacityUnits:  The maximum read capacity units that your account allows you to provision for a new table that you are creating in this Region, including the read capacity units provisioned for its global secondary indexes (GSIs).<br/>AccountMaxWriteCapacityUnits:  The maximum total write capacity units that your account allows you to provision across all of your tables in this Region.<br/>AccountMaxReadCapacityUnits:  The maximum total read capacity units that your account allows you to provision across all of your tables in this Region.<br/>        """
        pass

    def describe_table(self, TableName: str) -> DescribeTableOutput:
        """Returns information about the table, including the current status of the table, when it was created, the primary key schema, and any indexes on the table. <note> If you issue a <code>DescribeTable</code> request immediately after a <code>CreateTable</code> request, DynamoDB might return a <code>ResourceNotFoundException</code>. This is because <code>DescribeTable</code> uses an eventually consistent query, and the metadata for your table might not be available at that moment. Wait for a few seconds, and then try the <code>DescribeTable</code> request again. </note>

        :param self:
        :param TableName:  The name of the table to describe.<br/>
        :return: Table:  The properties of the table.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SSEDescription</i> The description of the server-side encryption status on the specified table.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SSEDescription</b>: The description of the server-side encryption status on the specified table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreSummary</i> Contains details for the restore.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RestoreSummary</b>: Contains details for the restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestStreamArn</i> The Amazon Resource Name (ARN) that uniquely identifies the latest stream for this table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestStreamLabel</i> A timestamp, in ISO 8601 format, for this stream. Note that <code>LatestStreamLabel</code> is not a unique identifier for the stream, because it is possible that a stream from another table might have the same timestamp. However, the combination of the following three elements is guaranteed to be unique: <ul> <li> the AWS customer ID. </li> <li> the table name. </li> <li> the <code>StreamLabel</code>. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamSpecification</i> The current DynamoDB Streams configuration for the table.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamSpecification</b>: The current DynamoDB Streams configuration for the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalSecondaryIndexes</i> The global secondary indexes, if any, on the table. Each index is scoped to a given partition key value. Each element is composed of: <ul> <li>  <code>Backfilling</code> - If true, then the index is currently in the backfilling phase. Backfilling occurs only when a new global secondary index is added to the table; it is the process by which DynamoDB populates the new index with data from the table. (This attribute does not appear for indexes that were created during a <code>CreateTable</code> operation.) </li> <li>  <code>IndexName</code> - The name of the global secondary index. </li> <li>  <code>IndexSizeBytes</code> - The total size of the global secondary index, in bytes. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value.  </li> <li>  <code>IndexStatus</code> - The current status of the global secondary index: <ul> <li>  <code>CREATING</code> - The index is being created. </li> <li>  <code>UPDATING</code> - The index is being updated. </li> <li>  <code>DELETING</code> - The index is being deleted. </li> <li>  <code>ACTIVE</code> - The index is ready for use. </li> </ul> </li> <li>  <code>ItemCount</code> - The number of items in the global secondary index. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value.  </li> <li>  <code>KeySchema</code> - Specifies the complete index key schema. The attribute names in the key schema must be between 1 and 255 characters (inclusive). The key schema must begin with the same partition key as the table. </li> <li>  <code>Projection</code> - Specifies attributes that are copied (projected) from the table into the index. These are in addition to the primary key attributes and index key attributes, which are automatically projected. Each attribute specification is composed of: <ul> <li>  <code>ProjectionType</code> - One of the following: <ul> <li>  <code>KEYS_ONLY</code> - Only the index and primary keys are projected into the index. </li> <li>  <code>INCLUDE</code> - Only the specified table attributes are projected into the index. The list of projected attributes are in <code>NonKeyAttributes</code>. </li> <li>  <code>ALL</code> - All of the table attributes are projected into the index. </li> </ul> </li> <li>  <code>NonKeyAttributes</code> - A list of one or more non-key attribute names that are projected into the secondary index. The total count of attributes provided in <code>NonKeyAttributes</code>, summed across all of the secondary indexes, must not exceed 20. If you project the same attribute into two different indexes, this counts as two distinct attributes when determining the total. </li> </ul> </li> <li>  <code>ProvisionedThroughput</code> - The provisioned throughput settings for the global secondary index, consisting of read and write capacity units, along with data about increases and decreases.  </li> </ul> If the table is in the <code>DELETING</code> state, no information about indexes will be returned.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LocalSecondaryIndexes</i> Represents one or more local secondary indexes on the table. Each index is scoped to a given partition key value. Tables with one or more local secondary indexes are subject to an item collection size limit, where the amount of data within a given item collection cannot exceed 10 GB. Each element is composed of: <ul> <li>  <code>IndexName</code> - The name of the local secondary index. </li> <li>  <code>KeySchema</code> - Specifies the complete index key schema. The attribute names in the key schema must be between 1 and 255 characters (inclusive). The key schema must begin with the same partition key as the table. </li> <li>  <code>Projection</code> - Specifies attributes that are copied (projected) from the table into the index. These are in addition to the primary key attributes and index key attributes, which are automatically projected. Each attribute specification is composed of: <ul> <li>  <code>ProjectionType</code> - One of the following: <ul> <li>  <code>KEYS_ONLY</code> - Only the index and primary keys are projected into the index. </li> <li>  <code>INCLUDE</code> - Only the specified table attributes are projected into the index. The list of projected attributes are in <code>NonKeyAttributes</code>. </li> <li>  <code>ALL</code> - All of the table attributes are projected into the index. </li> </ul> </li> <li>  <code>NonKeyAttributes</code> - A list of one or more non-key attribute names that are projected into the secondary index. The total count of attributes provided in <code>NonKeyAttributes</code>, summed across all of the secondary indexes, must not exceed 20. If you project the same attribute into two different indexes, this counts as two distinct attributes when determining the total. </li> </ul> </li> <li>  <code>IndexSizeBytes</code> - Represents the total size of the index, in bytes. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value. </li> <li>  <code>ItemCount</code> - Represents the number of items in the index. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value. </li> </ul> If the table is in the <code>DELETING</code> state, no information about indexes will be returned.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BillingModeSummary</i> Contains the details for the read/write capacity mode.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BillingModeSummary</b>: Contains the details for the read/write capacity mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableId</i> Unique identifier for the table for which the backup was created. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableArn</i> The Amazon Resource Name (ARN) that uniquely identifies the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ItemCount</i> The number of items in the specified table. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableSizeBytes</i> The total size of the specified table, in bytes. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisionedThroughput</i> The provisioned throughput settings for the table, consisting of read and write capacity units, along with data about increases and decreases.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ProvisionedThroughput</b>: The provisioned throughput settings for the table, consisting of read and write capacity units, along with data about increases and decreases.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDateTime</i> The date and time when the table was created, in <a href="http://www.epochconverter.com/">UNIX epoch time</a> format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableStatus</i> The current state of the table: <ul> <li>  <code>CREATING</code> - The table is being created. </li> <li>  <code>UPDATING</code> - The table is being updated. </li> <li>  <code>DELETING</code> - The table is being deleted. </li> <li>  <code>ACTIVE</code> - The table is ready for use. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeySchema</i> The primary key structure for the table. Each <code>KeySchemaElement</code> consists of: <ul> <li>  <code>AttributeName</code> - The name of the attribute. </li> <li>  <code>KeyType</code> - The role of the attribute: <ul> <li>  <code>HASH</code> - partition key </li> <li>  <code>RANGE</code> - sort key </li> </ul> <note> The partition key of an item is also known as its <i>hash attribute</i>. The term "hash attribute" derives from DynamoDB's usage of an internal hash function to evenly distribute data items across partitions, based on their partition key values. The sort key of an item is also known as its <i>range attribute</i>. The term "range attribute" derives from the way DynamoDB stores items with the same partition key physically close together, in sorted order by the sort key value. </note> </li> </ul> For more information about primary keys, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/DataModel.html#DataModelPrimaryKey">Primary Key</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableName</i> The name of the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttributeDefinitions</i> An array of <code>AttributeDefinition</code> objects. Each of these objects describes one attribute in the table and index key schema. Each <code>AttributeDefinition</code> object in this array is composed of: <ul> <li>  <code>AttributeName</code> - The name of the attribute. </li> <li>  <code>AttributeType</code> - The data type for the attribute. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WriteCapacityUnits</i> The maximum number of writes consumed per second before DynamoDB returns a <code>ThrottlingException</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadCapacityUnits</i> The maximum number of strongly consistent reads consumed per second before DynamoDB returns a <code>ThrottlingException</code>. Eventually consistent reads require less effort than strongly consistent reads, so a setting of 50 <code>ReadCapacityUnits</code> per second provides 100 eventually consistent <code>ReadCapacityUnits</code> per second.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfDecreasesToday</i> The number of provisioned throughput decreases for this table during this UTC calendar day. For current maximums on provisioned throughput decreases, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Limits.html">Limits</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LastDecreaseDateTime</i> The date and time of the last provisioned throughput decrease for this table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LastIncreaseDateTime</i> The date and time of the last provisioned throughput increase for this table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdateToPayPerRequestDateTime</i> Represents the time when <code>PAY_PER_REQUEST</code> was last set as the read/write capacity mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BillingMode</i> Controls how you are charged for read and write throughput and how you manage capacity. This setting can be changed later. <ul> <li>  <code>PROVISIONED</code> - Sets the read/write capacity mode to <code>PROVISIONED</code>. We recommend using <code>PROVISIONED</code> for predictable workloads. </li> <li>  <code>PAY_PER_REQUEST</code> - Sets the read/write capacity mode to <code>PAY_PER_REQUEST</code>. We recommend using <code>PAY_PER_REQUEST</code> for unpredictable workloads.  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamViewType</i>  When an item in the table is modified, <code>StreamViewType</code> determines what information is written to the stream for this table. Valid values for <code>StreamViewType</code> are: <ul> <li>  <code>KEYS_ONLY</code> - Only the key attributes of the modified item are written to the stream. </li> <li>  <code>NEW_IMAGE</code> - The entire item, as it appears after it was modified, is written to the stream. </li> <li>  <code>OLD_IMAGE</code> - The entire item, as it appeared before it was modified, is written to the stream. </li> <li>  <code>NEW_AND_OLD_IMAGES</code> - Both the new and the old item images of the item are written to the stream. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamEnabled</i> Indicates whether DynamoDB Streams is enabled (true) or disabled (false) on the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreInProgress</i> Indicates if a restore is in progress or not.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreDateTime</i> Point in time or source backup time.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceTableArn</i> ARN of the source table of the backup that is being restored.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceBackupArn</i> ARN of the backup from which the table was restored.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KMSMasterKeyArn</i> The KMS customer master key (CMK) ARN used for the KMS encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SSEType</i> Server-side encryption type. The only supported value is: <ul> <li>  <code>KMS</code> - Server-side encryption which uses AWS Key Management Service. Key is stored in your account and is managed by AWS KMS (KMS charges apply). </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Represents the current state of server-side encryption. The only supported values are: <ul> <li>  <code>ENABLED</code> - Server-side encryption is enabled. </li> <li>  <code>UPDATING</code> - Server-side encryption is being updated. </li> </ul><br/>        """
        pass

    def describe_time_to_live(self, TableName: str) -> DescribeTimeToLiveOutput:
        """Gives a description of the Time to Live (TTL) status on the specified table. 

        :param self:
        :param TableName:  The name of the table to be described.<br/>
        :return: TimeToLiveDescription:  <p/>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttributeName</i>  The name of the TTL attribute for items in the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeToLiveStatus</i>  The TTL status for the table.<br/>        """
        pass

    def get_item(self, Key: Key, TableName: str, AttributesToGet: List[AttributeName]=None, ConsistentRead: ConsistentRead=None, ReturnConsumedCapacity: str=None, ProjectionExpression: str=None, ExpressionAttributeNames: ExpressionAttributeNameMap=None) -> GetItemOutput:
        """The <code>GetItem</code> operation returns a set of attributes for the item with the given primary key. If there is no matching item, <code>GetItem</code> does not return any data and there will be no <code>Item</code> element in the response.  <code>GetItem</code> provides an eventually consistent read by default. If your application requires a strongly consistent read, set <code>ConsistentRead</code> to <code>true</code>. Although a strongly consistent read might take more time than an eventually consistent read, it always returns the last updated value.

        :param self:
        :param ExpressionAttributeNames:  One or more substitution tokens for attribute names in an expression. The following are some use cases for using <code>ExpressionAttributeNames</code>: <ul> <li> To access an attribute whose name conflicts with a DynamoDB reserved word. </li> <li> To create a placeholder for repeating occurrences of an attribute name in an expression. </li> <li> To prevent special characters in an attribute name from being misinterpreted in an expression. </li> </ul> Use the <b>#</b> character in an expression to dereference an attribute name. For example, consider the following attribute name: <ul> <li>  <code>Percentile</code>  </li> </ul> The name of this attribute conflicts with a reserved word, so it cannot be used directly in an expression. (For the complete list of reserved words, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/ReservedWords.html">Reserved Words</a> in the <i>Amazon DynamoDB Developer Guide</i>). To work around this, you could specify the following for <code>ExpressionAttributeNames</code>: <ul> <li>  <code>{"#P":"Percentile"}</code>  </li> </ul> You could then use this substitution in an expression, as in this example: <ul> <li>  <code>#P = :val</code>  </li> </ul> <note> Tokens that begin with the <b>:</b> character are <i>expression attribute values</i>, which are placeholders for the actual value at runtime. </note> For more information on expression attribute names, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Expressions.AccessingItemAttributes.html">Specifying Item Attributes</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>ProjectionExpression:  A string that identifies one or more attributes to retrieve from the table. These attributes can include scalars, sets, or elements of a JSON document. The attributes in the expression must be separated by commas. If no attribute names are specified, then all attributes are returned. If any of the requested attributes are not found, they do not appear in the result. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Expressions.AccessingItemAttributes.html">Specifying Item Attributes</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>ReturnConsumedCapacity:  Determines the level of detail about provisioned throughput consumption that is returned in the response: <ul> <li>  <code>INDEXES</code> - The response includes the aggregate <code>ConsumedCapacity</code> for the operation, together with <code>ConsumedCapacity</code> for each table and secondary index that was accessed. Note that some operations, such as <code>GetItem</code> and <code>BatchGetItem</code>, do not access any indexes at all. In these cases, specifying <code>INDEXES</code> will only return <code>ConsumedCapacity</code> information for table(s). </li> <li>  <code>TOTAL</code> - The response includes only the aggregate <code>ConsumedCapacity</code> for the operation. </li> <li>  <code>NONE</code> - No <code>ConsumedCapacity</code> details are included in the response. </li> </ul><br/>ConsistentRead:  Determines the read consistency model: If set to <code>true</code>, then the operation uses strongly consistent reads; otherwise, the operation uses eventually consistent reads.<br/>AttributesToGet:  This is a legacy parameter. Use <code>ProjectionExpression</code> instead. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/LegacyConditionalParameters.AttributesToGet.html">AttributesToGet</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>Key:  A map of attribute names to <code>AttributeValue</code> objects, representing the primary key of the item to retrieve. For the primary key, you must provide all of the attributes. For example, with a simple primary key, you only need to provide a value for the partition key. For a composite primary key, you must provide values for both the partition key and the sort key.<br/>TableName:  The name of the table containing the requested item.<br/>
        :return: ConsumedCapacity:  The capacity units consumed by the <code>GetItem</code> operation. The data returned includes the total provisioned throughput consumed, along with statistics for the table and any indexes involved in the operation. <code>ConsumedCapacity</code> is only returned if the <code>ReturnConsumedCapacity</code> parameter was specified. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/ProvisionedThroughputIntro.html">Read/Write Capacity Mode</a> in the <i>Amazon DynamoDB Developer Guide</i>.:<br/>Item:  A map of attribute names to <code>AttributeValue</code> objects, as specified by <code>ProjectionExpression</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalSecondaryIndexes</i> The amount of throughput consumed on each global index affected by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LocalSecondaryIndexes</i> The amount of throughput consumed on each local index affected by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Table</i> The amount of throughput consumed on the table affected by the operation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Table</b>: The amount of throughput consumed on the table affected by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WriteCapacityUnits</i> The total number of write capacity units consumed by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadCapacityUnits</i> The total number of read capacity units consumed by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CapacityUnits</i> The total number of capacity units consumed by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableName</i> The name of the table that was affected by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CapacityUnits</i> The total number of capacity units consumed on a table or an index.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WriteCapacityUnits</i> The total number of write capacity units consumed on a table or an index.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadCapacityUnits</i> The total number of read capacity units consumed on a table or an index.<br/>        """
        pass

    def list_backups(self, TableName: str=None, Limit: int=None, TimeRangeLowerBound: TimeRangeLowerBound=None, TimeRangeUpperBound: TimeRangeUpperBound=None, ExclusiveStartBackupArn: str=None, BackupType: str=None) -> ListBackupsOutput:
        """List backups associated with an AWS account. To list backups for a given table, specify <code>TableName</code>. <code>ListBackups</code> returns a paginated list of results with at most 1 MB worth of items in a page. You can also specify a limit for the maximum number of entries to be returned in a page.  In the request, start time is inclusive, but end time is exclusive. Note that these limits are for the time at which the original backup was requested. You can call <code>ListBackups</code> a maximum of five times per second.

        :param self:
        :param BackupType:  The backups from the table specified by <code>BackupType</code> are listed. Where <code>BackupType</code> can be: <ul> <li>  <code>USER</code> - On-demand backup created by you. </li> <li>  <code>SYSTEM</code> - On-demand backup automatically created by DynamoDB. </li> <li>  <code>ALL</code> - All types of on-demand backups (USER and SYSTEM). </li> </ul><br/>ExclusiveStartBackupArn:   <code>LastEvaluatedBackupArn</code> is the Amazon Resource Name (ARN) of the backup last evaluated when the current page of results was returned, inclusive of the current page of results. This value may be specified as the <code>ExclusiveStartBackupArn</code> of a new <code>ListBackups</code> operation in order to fetch the next page of results. <br/>TimeRangeUpperBound:  Only backups created before this time are listed. <code>TimeRangeUpperBound</code> is exclusive. <br/>TimeRangeLowerBound:  Only backups created after this time are listed. <code>TimeRangeLowerBound</code> is inclusive.<br/>Limit:  Maximum number of backups to return at once.<br/>TableName:  The backups from the table specified by <code>TableName</code> are listed. <br/>
        :return: LastEvaluatedBackupArn:   The ARN of the backup last evaluated when the current page of results was returned, inclusive of the current page of results. This value may be specified as the <code>ExclusiveStartBackupArn</code> of a new <code>ListBackups</code> operation in order to fetch the next page of results.   If <code>LastEvaluatedBackupArn</code> is empty, then the last page of results has been processed and there are no more results to be retrieved.   If <code>LastEvaluatedBackupArn</code> is not empty, this may or may not indicate that there is more data to be returned. All results are guaranteed to have been returned if and only if no value for <code>LastEvaluatedBackupArn</code> is returned. <br/>BackupSummaries:  List of <code>BackupSummary</code> objects.<br/>        """
        pass

    def list_global_tables(self, ExclusiveStartGlobalTableName: str=None, Limit: int=None, RegionName: str=None) -> ListGlobalTablesOutput:
        """Lists all global tables that have a replica in the specified Region.

        :param self:
        :param RegionName:  Lists the global tables in a specific Region.<br/>Limit:  The maximum number of table names to return.<br/>ExclusiveStartGlobalTableName:  The first global table name that this operation will evaluate.<br/>
        :return: LastEvaluatedGlobalTableName:  Last evaluated global table name.<br/>GlobalTables:  List of global table names.<br/>        """
        pass

    def list_tables(self, ExclusiveStartTableName: str=None, Limit: int=None) -> ListTablesOutput:
        """Returns an array of table names associated with the current account and endpoint. The output from <code>ListTables</code> is paginated, with each page returning a maximum of 100 table names.

        :param self:
        :param Limit:  A maximum number of table names to return. If this parameter is not specified, the limit is 100.<br/>ExclusiveStartTableName:  The first table name that this operation will evaluate. Use the value that was returned for <code>LastEvaluatedTableName</code> in a previous operation, so that you can obtain the next page of results.<br/>
        :return: LastEvaluatedTableName:  The name of the last table in the current page of results. Use this value as the <code>ExclusiveStartTableName</code> in a new request to obtain the next page of results, until all the table names are returned. If you do not receive a <code>LastEvaluatedTableName</code> value in the response, this means that there are no more table names to be retrieved.<br/>TableNames:  The names of the tables associated with the current account at the current endpoint. The maximum size of this array is 100. If <code>LastEvaluatedTableName</code> also appears in the output, you can use this value as the <code>ExclusiveStartTableName</code> parameter in a subsequent <code>ListTables</code> request and obtain the next page of results.<br/>        """
        pass

    def list_tags_of_resource(self, ResourceArn: str, NextToken: str=None) -> ListTagsOfResourceOutput:
        """List all tags on an Amazon DynamoDB resource. You can call ListTagsOfResource up to 10 times per second, per account. For an overview on tagging DynamoDB resources, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Tagging.html">Tagging for DynamoDB</a> in the <i>Amazon DynamoDB Developer Guide</i>.

        :param self:
        :param NextToken:  An optional string that, if supplied, must be copied from the output of a previous call to ListTagOfResource. When provided in this manner, this API fetches the next page of results.<br/>ResourceArn:  The Amazon DynamoDB resource with tags to be listed. This value is an Amazon Resource Name (ARN).<br/>
        :return: NextToken:  If this value is returned, there are additional results to be displayed. To retrieve them, call ListTagsOfResource again, with NextToken set to this value.<br/>Tags:  The tags currently associated with the Amazon DynamoDB resource.<br/>        """
        pass

    def put_item(self, Item: PutItemInputAttributeMap, TableName: str, Expected: ExpectedAttributeMap=None, ReturnValues: str=None, ReturnConsumedCapacity: str=None, ReturnItemCollectionMetrics: str=None, ConditionalOperator: str=None, ConditionExpression: str=None, ExpressionAttributeNames: ExpressionAttributeNameMap=None, ExpressionAttributeValues: ExpressionAttributeValueMap=None) -> PutItemOutput:
        """Creates a new item, or replaces an old item with a new item. If an item that has the same primary key as the new item already exists in the specified table, the new item completely replaces the existing item. You can perform a conditional put operation (add a new item if one with the specified primary key doesn't exist), or replace an existing item if it has certain attribute values. You can return the item's attribute values in the same operation, using the <code>ReturnValues</code> parameter. <important> This topic provides general information about the <code>PutItem</code> API. For information on how to call the <code>PutItem</code> API using the AWS SDK in specific languages, see the following: <ul> <li>  <a href="http://docs.aws.amazon.com/goto/aws-cli/dynamodb-2012-08-10/PutItem"> PutItem in the AWS Command Line Interface</a>  </li> <li>  <a href="http://docs.aws.amazon.com/goto/DotNetSDKV3/dynamodb-2012-08-10/PutItem"> PutItem in the AWS SDK for .NET</a>  </li> <li>  <a href="http://docs.aws.amazon.com/goto/SdkForCpp/dynamodb-2012-08-10/PutItem"> PutItem in the AWS SDK for C++</a>  </li> <li>  <a href="http://docs.aws.amazon.com/goto/SdkForGoV1/dynamodb-2012-08-10/PutItem"> PutItem in the AWS SDK for Go</a>  </li> <li>  <a href="http://docs.aws.amazon.com/goto/SdkForJava/dynamodb-2012-08-10/PutItem"> PutItem in the AWS SDK for Java</a>  </li> <li>  <a href="http://docs.aws.amazon.com/goto/AWSJavaScriptSDK/dynamodb-2012-08-10/PutItem"> PutItem in the AWS SDK for JavaScript</a>  </li> <li>  <a href="http://docs.aws.amazon.com/goto/SdkForPHPV3/dynamodb-2012-08-10/PutItem"> PutItem in the AWS SDK for PHP V3</a>  </li> <li>  <a href="http://docs.aws.amazon.com/goto/boto3/dynamodb-2012-08-10/PutItem"> PutItem in the AWS SDK for Python</a>  </li> <li>  <a href="http://docs.aws.amazon.com/goto/SdkForRubyV2/dynamodb-2012-08-10/PutItem"> PutItem in the AWS SDK for Ruby V2</a>  </li> </ul> </important> When you add an item, the primary key attributes are the only required attributes. Attribute values cannot be null. String and Binary type attributes must have lengths greater than zero. Set type attributes cannot be empty. Requests with empty values will be rejected with a <code>ValidationException</code> exception. <note> To prevent a new item from replacing an existing item, use a conditional expression that contains the <code>attribute_not_exists</code> function with the name of the attribute being used as the partition key for the table. Since every record must contain that attribute, the <code>attribute_not_exists</code> function will only succeed if no matching item exists. </note> For more information about <code>PutItem</code>, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/WorkingWithItems.html">Working with Items</a> in the <i>Amazon DynamoDB Developer Guide</i>.

        :param self:
        :param ExpressionAttributeValues:  One or more values that can be substituted in an expression. Use the <b>:</b> (colon) character in an expression to dereference an attribute value. For example, suppose that you wanted to check whether the value of the <i>ProductStatus</i> attribute was one of the following:   <code>Available | Backordered | Discontinued</code>  You would first need to specify <code>ExpressionAttributeValues</code> as follows:  <code>{ ":avail":{"S":"Available"}, ":back":{"S":"Backordered"}, ":disc":{"S":"Discontinued"} }</code>  You could then use these values in an expression, such as this:  <code>ProductStatus IN (:avail, :back, :disc)</code>  For more information on expression attribute values, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Expressions.SpecifyingConditions.html">Condition Expressions</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>ExpressionAttributeNames:  One or more substitution tokens for attribute names in an expression. The following are some use cases for using <code>ExpressionAttributeNames</code>: <ul> <li> To access an attribute whose name conflicts with a DynamoDB reserved word. </li> <li> To create a placeholder for repeating occurrences of an attribute name in an expression. </li> <li> To prevent special characters in an attribute name from being misinterpreted in an expression. </li> </ul> Use the <b>#</b> character in an expression to dereference an attribute name. For example, consider the following attribute name: <ul> <li>  <code>Percentile</code>  </li> </ul> The name of this attribute conflicts with a reserved word, so it cannot be used directly in an expression. (For the complete list of reserved words, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/ReservedWords.html">Reserved Words</a> in the <i>Amazon DynamoDB Developer Guide</i>). To work around this, you could specify the following for <code>ExpressionAttributeNames</code>: <ul> <li>  <code>{"#P":"Percentile"}</code>  </li> </ul> You could then use this substitution in an expression, as in this example: <ul> <li>  <code>#P = :val</code>  </li> </ul> <note> Tokens that begin with the <b>:</b> character are <i>expression attribute values</i>, which are placeholders for the actual value at runtime. </note> For more information on expression attribute names, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Expressions.AccessingItemAttributes.html">Specifying Item Attributes</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>ConditionExpression:  A condition that must be satisfied in order for a conditional <code>PutItem</code> operation to succeed. An expression can contain any of the following: <ul> <li> Functions: <code>attribute_exists | attribute_not_exists | attribute_type | contains | begins_with | size</code>  These function names are case-sensitive. </li> <li> Comparison operators: <code>= | &lt;&gt; | &lt; | &gt; | &lt;= | &gt;= | BETWEEN | IN </code>  </li> <li>  Logical operators: <code>AND | OR | NOT</code>  </li> </ul> For more information on condition expressions, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Expressions.SpecifyingConditions.html">Condition Expressions</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>ConditionalOperator:  This is a legacy parameter. Use <code>ConditionExpression</code> instead. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/LegacyConditionalParameters.ConditionalOperator.html">ConditionalOperator</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>ReturnItemCollectionMetrics:  Determines whether item collection metrics are returned. If set to <code>SIZE</code>, the response includes statistics about item collections, if any, that were modified during the operation are returned in the response. If set to <code>NONE</code> (the default), no statistics are returned.<br/>ReturnConsumedCapacity:  Determines the level of detail about provisioned throughput consumption that is returned in the response: <ul> <li>  <code>INDEXES</code> - The response includes the aggregate <code>ConsumedCapacity</code> for the operation, together with <code>ConsumedCapacity</code> for each table and secondary index that was accessed. Note that some operations, such as <code>GetItem</code> and <code>BatchGetItem</code>, do not access any indexes at all. In these cases, specifying <code>INDEXES</code> will only return <code>ConsumedCapacity</code> information for table(s). </li> <li>  <code>TOTAL</code> - The response includes only the aggregate <code>ConsumedCapacity</code> for the operation. </li> <li>  <code>NONE</code> - No <code>ConsumedCapacity</code> details are included in the response. </li> </ul><br/>ReturnValues:  Use <code>ReturnValues</code> if you want to get the item attributes as they appeared before they were updated with the <code>PutItem</code> request. For <code>PutItem</code>, the valid values are: <ul> <li>  <code>NONE</code> - If <code>ReturnValues</code> is not specified, or if its value is <code>NONE</code>, then nothing is returned. (This setting is the default for <code>ReturnValues</code>.) </li> <li>  <code>ALL_OLD</code> - If <code>PutItem</code> overwrote an attribute name-value pair, then the content of the old item is returned. </li> </ul> <note> The <code>ReturnValues</code> parameter is used by several DynamoDB operations; however, <code>PutItem</code> does not recognize any values other than <code>NONE</code> or <code>ALL_OLD</code>. </note><br/>Expected:  This is a legacy parameter. Use <code>ConditionExpression</code> instead. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/LegacyConditionalParameters.Expected.html">Expected</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>Item:  A map of attribute name/value pairs, one for each attribute. Only the primary key attributes are required; you can optionally provide other attribute name-value pairs for the item. You must provide all of the attributes for the primary key. For example, with a simple primary key, you only need to provide a value for the partition key. For a composite primary key, you must provide both values for both the partition key and the sort key. If you specify any attributes that are part of an index key, then the data types for those attributes must match those of the schema in the table's attribute definition. For more information about primary keys, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/HowItWorks.CoreComponents.html#HowItWorks.CoreComponents.PrimaryKey">Primary Key</a> in the <i>Amazon DynamoDB Developer Guide</i>. Each element in the <code>Item</code> map is an <code>AttributeValue</code> object.<br/>TableName:  The name of the table to contain the item.<br/>
        :return: ItemCollectionMetrics:  Information about item collections, if any, that were affected by the <code>PutItem</code> operation. <code>ItemCollectionMetrics</code> is only returned if the <code>ReturnItemCollectionMetrics</code> parameter was specified. If the table does not have any local secondary indexes, this information is not returned in the response. Each <code>ItemCollectionMetrics</code> element consists of: <ul> <li>  <code>ItemCollectionKey</code> - The partition key value of the item collection. This is the same as the partition key value of the item itself. </li> <li>  <code>SizeEstimateRangeGB</code> - An estimate of item collection size, in gigabytes. This value is a two-element array containing a lower bound and an upper bound for the estimate. The estimate includes the size of all the items in the table, plus the size of all attributes projected into all of the local secondary indexes on that table. Use this estimate to measure whether a local secondary index is approaching its size limit. The estimate is subject to change over time; therefore, do not rely on the precision or accuracy of the estimate. </li> </ul>:<br/>ConsumedCapacity:  The capacity units consumed by the <code>PutItem</code> operation. The data returned includes the total provisioned throughput consumed, along with statistics for the table and any indexes involved in the operation. <code>ConsumedCapacity</code> is only returned if the <code>ReturnConsumedCapacity</code> parameter was specified. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/ProvisionedThroughputIntro.html">Read/Write Capacity Mode</a> in the <i>Amazon DynamoDB Developer Guide</i>.:<br/>Attributes:  The attribute values as they appeared before the <code>PutItem</code> operation, but only if <code>ReturnValues</code> is specified as <code>ALL_OLD</code> in the request. Each element consists of an attribute name and an attribute value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalSecondaryIndexes</i> The amount of throughput consumed on each global index affected by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LocalSecondaryIndexes</i> The amount of throughput consumed on each local index affected by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Table</i> The amount of throughput consumed on the table affected by the operation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Table</b>: The amount of throughput consumed on the table affected by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WriteCapacityUnits</i> The total number of write capacity units consumed by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadCapacityUnits</i> The total number of read capacity units consumed by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CapacityUnits</i> The total number of capacity units consumed by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableName</i> The name of the table that was affected by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CapacityUnits</i> The total number of capacity units consumed on a table or an index.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WriteCapacityUnits</i> The total number of write capacity units consumed on a table or an index.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadCapacityUnits</i> The total number of read capacity units consumed on a table or an index.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SizeEstimateRangeGB</i> An estimate of item collection size, in gigabytes. This value is a two-element array containing a lower bound and an upper bound for the estimate. The estimate includes the size of all the items in the table, plus the size of all attributes projected into all of the local secondary indexes on that table. Use this estimate to measure whether a local secondary index is approaching its size limit. The estimate is subject to change over time; therefore, do not rely on the precision or accuracy of the estimate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ItemCollectionKey</i> The partition key value of the item collection. This value is the same as the partition key value of the item.<br/>        """
        pass

    def query(self, TableName: str, IndexName: str=None, Select: str=None, AttributesToGet: List[AttributeName]=None, Limit: int=None, ConsistentRead: ConsistentRead=None, KeyConditions: KeyConditions=None, QueryFilter: FilterConditionMap=None, ConditionalOperator: str=None, ScanIndexForward: BooleanObject=None, ExclusiveStartKey: Key=None, ReturnConsumedCapacity: str=None, ProjectionExpression: str=None, FilterExpression: str=None, KeyConditionExpression: str=None, ExpressionAttributeNames: ExpressionAttributeNameMap=None, ExpressionAttributeValues: ExpressionAttributeValueMap=None) -> QueryOutput:
        """The <code>Query</code> operation finds items based on primary key values. You can query any table or secondary index that has a composite primary key (a partition key and a sort key).  Use the <code>KeyConditionExpression</code> parameter to provide a specific value for the partition key. The <code>Query</code> operation will return all of the items from the table or index with that partition key value. You can optionally narrow the scope of the <code>Query</code> operation by specifying a sort key value and a comparison operator in <code>KeyConditionExpression</code>. To further refine the <code>Query</code> results, you can optionally provide a <code>FilterExpression</code>. A <code>FilterExpression</code> determines which items within the results should be returned to you. All of the other results are discarded.   A <code>Query</code> operation always returns a result set. If no matching items are found, the result set will be empty. Queries that do not return results consume the minimum number of read capacity units for that type of read operation.  <note>  DynamoDB calculates the number of read capacity units consumed based on item size, not on the amount of data that is returned to an application. The number of capacity units consumed will be the same whether you request all of the attributes (the default behavior) or just some of them (using a projection expression). The number will also be the same whether or not you use a <code>FilterExpression</code>.  </note>  <code>Query</code> results are always sorted by the sort key value. If the data type of the sort key is Number, the results are returned in numeric order; otherwise, the results are returned in order of UTF-8 bytes. By default, the sort order is ascending. To reverse the order, set the <code>ScanIndexForward</code> parameter to false.   A single <code>Query</code> operation will read up to the maximum number of items set (if using the <code>Limit</code> parameter) or a maximum of 1 MB of data and then apply any filtering to the results using <code>FilterExpression</code>. If <code>LastEvaluatedKey</code> is present in the response, you will need to paginate the result set. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Query.html#Query.Pagination">Paginating the Results</a> in the <i>Amazon DynamoDB Developer Guide</i>.   <code>FilterExpression</code> is applied after a <code>Query</code> finishes, but before the results are returned. A <code>FilterExpression</code> cannot contain partition key or sort key attributes. You need to specify those attributes in the <code>KeyConditionExpression</code>.  <note>  A <code>Query</code> operation can return an empty result set and a <code>LastEvaluatedKey</code> if all the items read for the page of results are filtered out.  </note> You can query a table, a local secondary index, or a global secondary index. For a query on a table or on a local secondary index, you can set the <code>ConsistentRead</code> parameter to <code>true</code> and obtain a strongly consistent result. Global secondary indexes support eventually consistent reads only, so do not specify <code>ConsistentRead</code> when querying a global secondary index.

        :param self:
        :param ExpressionAttributeValues:  One or more values that can be substituted in an expression. Use the <b>:</b> (colon) character in an expression to dereference an attribute value. For example, suppose that you wanted to check whether the value of the <i>ProductStatus</i> attribute was one of the following:   <code>Available | Backordered | Discontinued</code>  You would first need to specify <code>ExpressionAttributeValues</code> as follows:  <code>{ ":avail":{"S":"Available"}, ":back":{"S":"Backordered"}, ":disc":{"S":"Discontinued"} }</code>  You could then use these values in an expression, such as this:  <code>ProductStatus IN (:avail, :back, :disc)</code>  For more information on expression attribute values, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Expressions.SpecifyingConditions.html">Specifying Conditions</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>ExpressionAttributeNames:  One or more substitution tokens for attribute names in an expression. The following are some use cases for using <code>ExpressionAttributeNames</code>: <ul> <li> To access an attribute whose name conflicts with a DynamoDB reserved word. </li> <li> To create a placeholder for repeating occurrences of an attribute name in an expression. </li> <li> To prevent special characters in an attribute name from being misinterpreted in an expression. </li> </ul> Use the <b>#</b> character in an expression to dereference an attribute name. For example, consider the following attribute name: <ul> <li>  <code>Percentile</code>  </li> </ul> The name of this attribute conflicts with a reserved word, so it cannot be used directly in an expression. (For the complete list of reserved words, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/ReservedWords.html">Reserved Words</a> in the <i>Amazon DynamoDB Developer Guide</i>). To work around this, you could specify the following for <code>ExpressionAttributeNames</code>: <ul> <li>  <code>{"#P":"Percentile"}</code>  </li> </ul> You could then use this substitution in an expression, as in this example: <ul> <li>  <code>#P = :val</code>  </li> </ul> <note> Tokens that begin with the <b>:</b> character are <i>expression attribute values</i>, which are placeholders for the actual value at runtime. </note> For more information on expression attribute names, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Expressions.AccessingItemAttributes.html">Specifying Item Attributes</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>KeyConditionExpression:  The condition that specifies the key values for items to be retrieved by the <code>Query</code> action. The condition must perform an equality test on a single partition key value. The condition can optionally perform one of several comparison tests on a single sort key value. This allows <code>Query</code> to retrieve one item with a given partition key value and sort key value, or several items that have the same partition key value but different sort key values. The partition key equality test is required, and must be specified in the following format:  <code>partitionKeyName</code> <i>=</i> <code>:partitionkeyval</code>  If you also want to provide a condition for the sort key, it must be combined using <code>AND</code> with the condition for the sort key. Following is an example, using the <b>=</b> comparison operator for the sort key:  <code>partitionKeyName</code> <code>=</code> <code>:partitionkeyval</code> <code>AND</code> <code>sortKeyName</code> <code>=</code> <code>:sortkeyval</code>  Valid comparisons for the sort key condition are as follows: <ul> <li>  <code>sortKeyName</code> <code>=</code> <code>:sortkeyval</code> - true if the sort key value is equal to <code>:sortkeyval</code>. </li> <li>  <code>sortKeyName</code> <code>&lt;</code> <code>:sortkeyval</code> - true if the sort key value is less than <code>:sortkeyval</code>. </li> <li>  <code>sortKeyName</code> <code>&lt;=</code> <code>:sortkeyval</code> - true if the sort key value is less than or equal to <code>:sortkeyval</code>. </li> <li>  <code>sortKeyName</code> <code>&gt;</code> <code>:sortkeyval</code> - true if the sort key value is greater than <code>:sortkeyval</code>. </li> <li>  <code>sortKeyName</code> <code>&gt;= </code> <code>:sortkeyval</code> - true if the sort key value is greater than or equal to <code>:sortkeyval</code>. </li> <li>  <code>sortKeyName</code> <code>BETWEEN</code> <code>:sortkeyval1</code> <code>AND</code> <code>:sortkeyval2</code> - true if the sort key value is greater than or equal to <code>:sortkeyval1</code>, and less than or equal to <code>:sortkeyval2</code>. </li> <li>  <code>begins_with (</code> <code>sortKeyName</code>, <code>:sortkeyval</code> <code>)</code> - true if the sort key value begins with a particular operand. (You cannot use this function with a sort key that is of type Number.) Note that the function name <code>begins_with</code> is case-sensitive. </li> </ul> Use the <code>ExpressionAttributeValues</code> parameter to replace tokens such as <code>:partitionval</code> and <code>:sortval</code> with actual values at runtime. You can optionally use the <code>ExpressionAttributeNames</code> parameter to replace the names of the partition key and sort key with placeholder tokens. This option might be necessary if an attribute name conflicts with a DynamoDB reserved word. For example, the following <code>KeyConditionExpression</code> parameter causes an error because <i>Size</i> is a reserved word: <ul> <li>  <code>Size = :myval</code>  </li> </ul> To work around this, define a placeholder (such a <code>#S</code>) to represent the attribute name <i>Size</i>. <code>KeyConditionExpression</code> then is as follows: <ul> <li>  <code>#S = :myval</code>  </li> </ul> For a list of reserved words, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/ReservedWords.html">Reserved Words</a> in the <i>Amazon DynamoDB Developer Guide</i>. For more information on <code>ExpressionAttributeNames</code> and <code>ExpressionAttributeValues</code>, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/ExpressionPlaceholders.html">Using Placeholders for Attribute Names and Values</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>FilterExpression:  A string that contains conditions that DynamoDB applies after the <code>Query</code> operation, but before the data is returned to you. Items that do not satisfy the <code>FilterExpression</code> criteria are not returned. A <code>FilterExpression</code> does not allow key attributes. You cannot define a filter expression based on a partition key or a sort key. <note> A <code>FilterExpression</code> is applied after the items have already been read; the process of filtering does not consume any additional read capacity units. </note> For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/QueryAndScan.html#FilteringResults">Filter Expressions</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>ProjectionExpression:  A string that identifies one or more attributes to retrieve from the table. These attributes can include scalars, sets, or elements of a JSON document. The attributes in the expression must be separated by commas. If no attribute names are specified, then all attributes will be returned. If any of the requested attributes are not found, they will not appear in the result. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Expressions.AccessingItemAttributes.html">Accessing Item Attributes</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>ReturnConsumedCapacity:  Determines the level of detail about provisioned throughput consumption that is returned in the response: <ul> <li>  <code>INDEXES</code> - The response includes the aggregate <code>ConsumedCapacity</code> for the operation, together with <code>ConsumedCapacity</code> for each table and secondary index that was accessed. Note that some operations, such as <code>GetItem</code> and <code>BatchGetItem</code>, do not access any indexes at all. In these cases, specifying <code>INDEXES</code> will only return <code>ConsumedCapacity</code> information for table(s). </li> <li>  <code>TOTAL</code> - The response includes only the aggregate <code>ConsumedCapacity</code> for the operation. </li> <li>  <code>NONE</code> - No <code>ConsumedCapacity</code> details are included in the response. </li> </ul><br/>ExclusiveStartKey:  The primary key of the first item that this operation will evaluate. Use the value that was returned for <code>LastEvaluatedKey</code> in the previous operation. The data type for <code>ExclusiveStartKey</code> must be String, Number, or Binary. No set data types are allowed.<br/>ScanIndexForward:  Specifies the order for index traversal: If <code>true</code> (default), the traversal is performed in ascending order; if <code>false</code>, the traversal is performed in descending order.  Items with the same partition key value are stored in sorted order by sort key. If the sort key data type is Number, the results are stored in numeric order. For type String, the results are stored in order of UTF-8 bytes. For type Binary, DynamoDB treats each byte of the binary data as unsigned. If <code>ScanIndexForward</code> is <code>true</code>, DynamoDB returns the results in the order in which they are stored (by sort key value). This is the default behavior. If <code>ScanIndexForward</code> is <code>false</code>, DynamoDB reads the results in reverse order by sort key value, and then returns the results to the client.<br/>ConditionalOperator:  This is a legacy parameter. Use <code>FilterExpression</code> instead. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/LegacyConditionalParameters.ConditionalOperator.html">ConditionalOperator</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>QueryFilter:  This is a legacy parameter. Use <code>FilterExpression</code> instead. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/LegacyConditionalParameters.QueryFilter.html">QueryFilter</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>KeyConditions:  This is a legacy parameter. Use <code>KeyConditionExpression</code> instead. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/LegacyConditionalParameters.KeyConditions.html">KeyConditions</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>ConsistentRead:  Determines the read consistency model: If set to <code>true</code>, then the operation uses strongly consistent reads; otherwise, the operation uses eventually consistent reads. Strongly consistent reads are not supported on global secondary indexes. If you query a global secondary index with <code>ConsistentRead</code> set to <code>true</code>, you will receive a <code>ValidationException</code>.<br/>Limit:  The maximum number of items to evaluate (not necessarily the number of matching items). If DynamoDB processes the number of items up to the limit while processing the results, it stops the operation and returns the matching values up to that point, and a key in <code>LastEvaluatedKey</code> to apply in a subsequent operation, so that you can pick up where you left off. Also, if the processed dataset size exceeds 1 MB before DynamoDB reaches this limit, it stops the operation and returns the matching values up to the limit, and a key in <code>LastEvaluatedKey</code> to apply in a subsequent operation to continue the operation. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/QueryAndScan.html">Query and Scan</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>AttributesToGet:  This is a legacy parameter. Use <code>ProjectionExpression</code> instead. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/LegacyConditionalParameters.AttributesToGet.html">AttributesToGet</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>Select:  The attributes to be returned in the result. You can retrieve all item attributes, specific item attributes, the count of matching items, or in the case of an index, some or all of the attributes projected into the index. <ul> <li>  <code>ALL_ATTRIBUTES</code> - Returns all of the item attributes from the specified table or index. If you query a local secondary index, then for each matching item in the index, DynamoDB fetches the entire item from the parent table. If the index is configured to project all item attributes, then all of the data can be obtained from the local secondary index, and no fetching is required. </li> <li>  <code>ALL_PROJECTED_ATTRIBUTES</code> - Allowed only when querying an index. Retrieves all attributes that have been projected into the index. If the index is configured to project all attributes, this return value is equivalent to specifying <code>ALL_ATTRIBUTES</code>. </li> <li>  <code>COUNT</code> - Returns the number of matching items, rather than the matching items themselves. </li> <li>  <code>SPECIFIC_ATTRIBUTES</code> - Returns only the attributes listed in <code>AttributesToGet</code>. This return value is equivalent to specifying <code>AttributesToGet</code> without specifying any value for <code>Select</code>. If you query or scan a local secondary index and request only attributes that are projected into that index, the operation will read only the index and not the table. If any of the requested attributes are not projected into the local secondary index, DynamoDB fetches each of these attributes from the parent table. This extra fetching incurs additional throughput cost and latency. If you query or scan a global secondary index, you can only request attributes that are projected into the index. Global secondary index queries cannot fetch attributes from the parent table. </li> </ul> If neither <code>Select</code> nor <code>AttributesToGet</code> are specified, DynamoDB defaults to <code>ALL_ATTRIBUTES</code> when accessing a table, and <code>ALL_PROJECTED_ATTRIBUTES</code> when accessing an index. You cannot use both <code>Select</code> and <code>AttributesToGet</code> together in a single request, unless the value for <code>Select</code> is <code>SPECIFIC_ATTRIBUTES</code>. (This usage is equivalent to specifying <code>AttributesToGet</code> without any value for <code>Select</code>.) <note> If you use the <code>ProjectionExpression</code> parameter, then the value for <code>Select</code> can only be <code>SPECIFIC_ATTRIBUTES</code>. Any other value for <code>Select</code> will return an error. </note><br/>IndexName:  The name of an index to query. This index can be any local secondary index or global secondary index on the table. Note that if you use the <code>IndexName</code> parameter, you must also provide <code>TableName.</code> <br/>TableName:  The name of the table containing the requested items.<br/>
        :return: ConsumedCapacity:  The capacity units consumed by the <code>Query</code> operation. The data returned includes the total provisioned throughput consumed, along with statistics for the table and any indexes involved in the operation. <code>ConsumedCapacity</code> is only returned if the <code>ReturnConsumedCapacity</code> parameter was specified. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/ProvisionedThroughputIntro.html">Provisioned Throughput</a> in the <i>Amazon DynamoDB Developer Guide</i>.:<br/>LastEvaluatedKey:  The primary key of the item where the operation stopped, inclusive of the previous result set. Use this value to start a new operation, excluding this value in the new request. If <code>LastEvaluatedKey</code> is empty, then the "last page" of results has been processed and there is no more data to be retrieved. If <code>LastEvaluatedKey</code> is not empty, it does not necessarily mean that there is more data in the result set. The only way to know when you have reached the end of the result set is when <code>LastEvaluatedKey</code> is empty.<br/>ScannedCount:  The number of items evaluated, before any <code>QueryFilter</code> is applied. A high <code>ScannedCount</code> value with few, or no, <code>Count</code> results indicates an inefficient <code>Query</code> operation. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/QueryAndScan.html#Count">Count and ScannedCount</a> in the <i>Amazon DynamoDB Developer Guide</i>. If you did not use a filter in the request, then <code>ScannedCount</code> is the same as <code>Count</code>.<br/>Count:  The number of items in the response. If you used a <code>QueryFilter</code> in the request, then <code>Count</code> is the number of items returned after the filter was applied, and <code>ScannedCount</code> is the number of matching items before the filter was applied. If you did not use a filter in the request, then <code>Count</code> and <code>ScannedCount</code> are the same.<br/>Items:  An array of item attributes that match the query criteria. Each element in this array consists of an attribute name and the value for that attribute.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalSecondaryIndexes</i> The amount of throughput consumed on each global index affected by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LocalSecondaryIndexes</i> The amount of throughput consumed on each local index affected by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Table</i> The amount of throughput consumed on the table affected by the operation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Table</b>: The amount of throughput consumed on the table affected by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WriteCapacityUnits</i> The total number of write capacity units consumed by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadCapacityUnits</i> The total number of read capacity units consumed by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CapacityUnits</i> The total number of capacity units consumed by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableName</i> The name of the table that was affected by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CapacityUnits</i> The total number of capacity units consumed on a table or an index.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WriteCapacityUnits</i> The total number of write capacity units consumed on a table or an index.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadCapacityUnits</i> The total number of read capacity units consumed on a table or an index.<br/>        """
        pass

    def restore_table_from_backup(self, BackupArn: str, TargetTableName: str) -> RestoreTableFromBackupOutput:
        """Creates a new table from an existing backup. Any number of users can execute up to 4 concurrent restores (any type of restore) in a given account.  You can call <code>RestoreTableFromBackup</code> at a maximum rate of 10 times per second. You must manually set up the following on the restored table: <ul> <li> Auto scaling policies </li> <li> IAM policies </li> <li> Amazon CloudWatch metrics and alarms </li> <li> Tags </li> <li> Stream settings </li> <li> Time to Live (TTL) settings </li> </ul>

        :param self:
        :param BackupArn:  The Amazon Resource Name (ARN) associated with the backup.<br/>TargetTableName:  The name of the new table to which the backup must be restored.<br/>
        :return: TableDescription:  The description of the table created from an existing backup.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SSEDescription</i> The description of the server-side encryption status on the specified table.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SSEDescription</b>: The description of the server-side encryption status on the specified table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreSummary</i> Contains details for the restore.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RestoreSummary</b>: Contains details for the restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestStreamArn</i> The Amazon Resource Name (ARN) that uniquely identifies the latest stream for this table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestStreamLabel</i> A timestamp, in ISO 8601 format, for this stream. Note that <code>LatestStreamLabel</code> is not a unique identifier for the stream, because it is possible that a stream from another table might have the same timestamp. However, the combination of the following three elements is guaranteed to be unique: <ul> <li> the AWS customer ID. </li> <li> the table name. </li> <li> the <code>StreamLabel</code>. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamSpecification</i> The current DynamoDB Streams configuration for the table.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamSpecification</b>: The current DynamoDB Streams configuration for the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalSecondaryIndexes</i> The global secondary indexes, if any, on the table. Each index is scoped to a given partition key value. Each element is composed of: <ul> <li>  <code>Backfilling</code> - If true, then the index is currently in the backfilling phase. Backfilling occurs only when a new global secondary index is added to the table; it is the process by which DynamoDB populates the new index with data from the table. (This attribute does not appear for indexes that were created during a <code>CreateTable</code> operation.) </li> <li>  <code>IndexName</code> - The name of the global secondary index. </li> <li>  <code>IndexSizeBytes</code> - The total size of the global secondary index, in bytes. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value.  </li> <li>  <code>IndexStatus</code> - The current status of the global secondary index: <ul> <li>  <code>CREATING</code> - The index is being created. </li> <li>  <code>UPDATING</code> - The index is being updated. </li> <li>  <code>DELETING</code> - The index is being deleted. </li> <li>  <code>ACTIVE</code> - The index is ready for use. </li> </ul> </li> <li>  <code>ItemCount</code> - The number of items in the global secondary index. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value.  </li> <li>  <code>KeySchema</code> - Specifies the complete index key schema. The attribute names in the key schema must be between 1 and 255 characters (inclusive). The key schema must begin with the same partition key as the table. </li> <li>  <code>Projection</code> - Specifies attributes that are copied (projected) from the table into the index. These are in addition to the primary key attributes and index key attributes, which are automatically projected. Each attribute specification is composed of: <ul> <li>  <code>ProjectionType</code> - One of the following: <ul> <li>  <code>KEYS_ONLY</code> - Only the index and primary keys are projected into the index. </li> <li>  <code>INCLUDE</code> - Only the specified table attributes are projected into the index. The list of projected attributes are in <code>NonKeyAttributes</code>. </li> <li>  <code>ALL</code> - All of the table attributes are projected into the index. </li> </ul> </li> <li>  <code>NonKeyAttributes</code> - A list of one or more non-key attribute names that are projected into the secondary index. The total count of attributes provided in <code>NonKeyAttributes</code>, summed across all of the secondary indexes, must not exceed 20. If you project the same attribute into two different indexes, this counts as two distinct attributes when determining the total. </li> </ul> </li> <li>  <code>ProvisionedThroughput</code> - The provisioned throughput settings for the global secondary index, consisting of read and write capacity units, along with data about increases and decreases.  </li> </ul> If the table is in the <code>DELETING</code> state, no information about indexes will be returned.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LocalSecondaryIndexes</i> Represents one or more local secondary indexes on the table. Each index is scoped to a given partition key value. Tables with one or more local secondary indexes are subject to an item collection size limit, where the amount of data within a given item collection cannot exceed 10 GB. Each element is composed of: <ul> <li>  <code>IndexName</code> - The name of the local secondary index. </li> <li>  <code>KeySchema</code> - Specifies the complete index key schema. The attribute names in the key schema must be between 1 and 255 characters (inclusive). The key schema must begin with the same partition key as the table. </li> <li>  <code>Projection</code> - Specifies attributes that are copied (projected) from the table into the index. These are in addition to the primary key attributes and index key attributes, which are automatically projected. Each attribute specification is composed of: <ul> <li>  <code>ProjectionType</code> - One of the following: <ul> <li>  <code>KEYS_ONLY</code> - Only the index and primary keys are projected into the index. </li> <li>  <code>INCLUDE</code> - Only the specified table attributes are projected into the index. The list of projected attributes are in <code>NonKeyAttributes</code>. </li> <li>  <code>ALL</code> - All of the table attributes are projected into the index. </li> </ul> </li> <li>  <code>NonKeyAttributes</code> - A list of one or more non-key attribute names that are projected into the secondary index. The total count of attributes provided in <code>NonKeyAttributes</code>, summed across all of the secondary indexes, must not exceed 20. If you project the same attribute into two different indexes, this counts as two distinct attributes when determining the total. </li> </ul> </li> <li>  <code>IndexSizeBytes</code> - Represents the total size of the index, in bytes. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value. </li> <li>  <code>ItemCount</code> - Represents the number of items in the index. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value. </li> </ul> If the table is in the <code>DELETING</code> state, no information about indexes will be returned.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BillingModeSummary</i> Contains the details for the read/write capacity mode.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BillingModeSummary</b>: Contains the details for the read/write capacity mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableId</i> Unique identifier for the table for which the backup was created. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableArn</i> The Amazon Resource Name (ARN) that uniquely identifies the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ItemCount</i> The number of items in the specified table. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableSizeBytes</i> The total size of the specified table, in bytes. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisionedThroughput</i> The provisioned throughput settings for the table, consisting of read and write capacity units, along with data about increases and decreases.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ProvisionedThroughput</b>: The provisioned throughput settings for the table, consisting of read and write capacity units, along with data about increases and decreases.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDateTime</i> The date and time when the table was created, in <a href="http://www.epochconverter.com/">UNIX epoch time</a> format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableStatus</i> The current state of the table: <ul> <li>  <code>CREATING</code> - The table is being created. </li> <li>  <code>UPDATING</code> - The table is being updated. </li> <li>  <code>DELETING</code> - The table is being deleted. </li> <li>  <code>ACTIVE</code> - The table is ready for use. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeySchema</i> The primary key structure for the table. Each <code>KeySchemaElement</code> consists of: <ul> <li>  <code>AttributeName</code> - The name of the attribute. </li> <li>  <code>KeyType</code> - The role of the attribute: <ul> <li>  <code>HASH</code> - partition key </li> <li>  <code>RANGE</code> - sort key </li> </ul> <note> The partition key of an item is also known as its <i>hash attribute</i>. The term "hash attribute" derives from DynamoDB's usage of an internal hash function to evenly distribute data items across partitions, based on their partition key values. The sort key of an item is also known as its <i>range attribute</i>. The term "range attribute" derives from the way DynamoDB stores items with the same partition key physically close together, in sorted order by the sort key value. </note> </li> </ul> For more information about primary keys, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/DataModel.html#DataModelPrimaryKey">Primary Key</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableName</i> The name of the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttributeDefinitions</i> An array of <code>AttributeDefinition</code> objects. Each of these objects describes one attribute in the table and index key schema. Each <code>AttributeDefinition</code> object in this array is composed of: <ul> <li>  <code>AttributeName</code> - The name of the attribute. </li> <li>  <code>AttributeType</code> - The data type for the attribute. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WriteCapacityUnits</i> The maximum number of writes consumed per second before DynamoDB returns a <code>ThrottlingException</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadCapacityUnits</i> The maximum number of strongly consistent reads consumed per second before DynamoDB returns a <code>ThrottlingException</code>. Eventually consistent reads require less effort than strongly consistent reads, so a setting of 50 <code>ReadCapacityUnits</code> per second provides 100 eventually consistent <code>ReadCapacityUnits</code> per second.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfDecreasesToday</i> The number of provisioned throughput decreases for this table during this UTC calendar day. For current maximums on provisioned throughput decreases, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Limits.html">Limits</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LastDecreaseDateTime</i> The date and time of the last provisioned throughput decrease for this table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LastIncreaseDateTime</i> The date and time of the last provisioned throughput increase for this table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdateToPayPerRequestDateTime</i> Represents the time when <code>PAY_PER_REQUEST</code> was last set as the read/write capacity mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BillingMode</i> Controls how you are charged for read and write throughput and how you manage capacity. This setting can be changed later. <ul> <li>  <code>PROVISIONED</code> - Sets the read/write capacity mode to <code>PROVISIONED</code>. We recommend using <code>PROVISIONED</code> for predictable workloads. </li> <li>  <code>PAY_PER_REQUEST</code> - Sets the read/write capacity mode to <code>PAY_PER_REQUEST</code>. We recommend using <code>PAY_PER_REQUEST</code> for unpredictable workloads.  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamViewType</i>  When an item in the table is modified, <code>StreamViewType</code> determines what information is written to the stream for this table. Valid values for <code>StreamViewType</code> are: <ul> <li>  <code>KEYS_ONLY</code> - Only the key attributes of the modified item are written to the stream. </li> <li>  <code>NEW_IMAGE</code> - The entire item, as it appears after it was modified, is written to the stream. </li> <li>  <code>OLD_IMAGE</code> - The entire item, as it appeared before it was modified, is written to the stream. </li> <li>  <code>NEW_AND_OLD_IMAGES</code> - Both the new and the old item images of the item are written to the stream. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamEnabled</i> Indicates whether DynamoDB Streams is enabled (true) or disabled (false) on the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreInProgress</i> Indicates if a restore is in progress or not.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreDateTime</i> Point in time or source backup time.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceTableArn</i> ARN of the source table of the backup that is being restored.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceBackupArn</i> ARN of the backup from which the table was restored.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KMSMasterKeyArn</i> The KMS customer master key (CMK) ARN used for the KMS encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SSEType</i> Server-side encryption type. The only supported value is: <ul> <li>  <code>KMS</code> - Server-side encryption which uses AWS Key Management Service. Key is stored in your account and is managed by AWS KMS (KMS charges apply). </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Represents the current state of server-side encryption. The only supported values are: <ul> <li>  <code>ENABLED</code> - Server-side encryption is enabled. </li> <li>  <code>UPDATING</code> - Server-side encryption is being updated. </li> </ul><br/>        """
        pass

    def restore_table_to_point_in_time(self, TargetTableName: str, SourceTableName: str, UseLatestRestorableTime: BooleanObject=None, RestoreDateTime: Date=None) -> RestoreTableToPointInTimeOutput:
        """Restores the specified table to the specified point in time within <code>EarliestRestorableDateTime</code> and <code>LatestRestorableDateTime</code>. You can restore your table to any point in time during the last 35 days. Any number of users can execute up to 4 concurrent restores (any type of restore) in a given account.   When you restore using point in time recovery, DynamoDB restores your table data to the state based on the selected date and time (day:hour:minute:second) to a new table.   Along with data, the following are also included on the new restored table using point in time recovery:  <ul> <li> Global secondary indexes (GSIs) </li> <li> Local secondary indexes (LSIs) </li> <li> Provisioned read and write capacity </li> <li> Encryption settings <important>  All these settings come from the current settings of the source table at the time of restore.  </important> </li> </ul> You must manually set up the following on the restored table: <ul> <li> Auto scaling policies </li> <li> IAM policies </li> <li> Amazon CloudWatch metrics and alarms </li> <li> Tags </li> <li> Stream settings </li> <li> Time to Live (TTL) settings </li> <li> Point in time recovery settings </li> </ul>

        :param self:
        :param RestoreDateTime:  Time in the past to restore the table to.<br/>UseLatestRestorableTime:  Restore the table to the latest possible time. <code>LatestRestorableDateTime</code> is typically 5 minutes before the current time. <br/>TargetTableName:  The name of the new table to which it must be restored to.<br/>SourceTableName:  Name of the source table that is being restored.<br/>
        :return: TableDescription:  Represents the properties of a table.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SSEDescription</i> The description of the server-side encryption status on the specified table.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SSEDescription</b>: The description of the server-side encryption status on the specified table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreSummary</i> Contains details for the restore.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RestoreSummary</b>: Contains details for the restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestStreamArn</i> The Amazon Resource Name (ARN) that uniquely identifies the latest stream for this table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestStreamLabel</i> A timestamp, in ISO 8601 format, for this stream. Note that <code>LatestStreamLabel</code> is not a unique identifier for the stream, because it is possible that a stream from another table might have the same timestamp. However, the combination of the following three elements is guaranteed to be unique: <ul> <li> the AWS customer ID. </li> <li> the table name. </li> <li> the <code>StreamLabel</code>. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamSpecification</i> The current DynamoDB Streams configuration for the table.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamSpecification</b>: The current DynamoDB Streams configuration for the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalSecondaryIndexes</i> The global secondary indexes, if any, on the table. Each index is scoped to a given partition key value. Each element is composed of: <ul> <li>  <code>Backfilling</code> - If true, then the index is currently in the backfilling phase. Backfilling occurs only when a new global secondary index is added to the table; it is the process by which DynamoDB populates the new index with data from the table. (This attribute does not appear for indexes that were created during a <code>CreateTable</code> operation.) </li> <li>  <code>IndexName</code> - The name of the global secondary index. </li> <li>  <code>IndexSizeBytes</code> - The total size of the global secondary index, in bytes. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value.  </li> <li>  <code>IndexStatus</code> - The current status of the global secondary index: <ul> <li>  <code>CREATING</code> - The index is being created. </li> <li>  <code>UPDATING</code> - The index is being updated. </li> <li>  <code>DELETING</code> - The index is being deleted. </li> <li>  <code>ACTIVE</code> - The index is ready for use. </li> </ul> </li> <li>  <code>ItemCount</code> - The number of items in the global secondary index. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value.  </li> <li>  <code>KeySchema</code> - Specifies the complete index key schema. The attribute names in the key schema must be between 1 and 255 characters (inclusive). The key schema must begin with the same partition key as the table. </li> <li>  <code>Projection</code> - Specifies attributes that are copied (projected) from the table into the index. These are in addition to the primary key attributes and index key attributes, which are automatically projected. Each attribute specification is composed of: <ul> <li>  <code>ProjectionType</code> - One of the following: <ul> <li>  <code>KEYS_ONLY</code> - Only the index and primary keys are projected into the index. </li> <li>  <code>INCLUDE</code> - Only the specified table attributes are projected into the index. The list of projected attributes are in <code>NonKeyAttributes</code>. </li> <li>  <code>ALL</code> - All of the table attributes are projected into the index. </li> </ul> </li> <li>  <code>NonKeyAttributes</code> - A list of one or more non-key attribute names that are projected into the secondary index. The total count of attributes provided in <code>NonKeyAttributes</code>, summed across all of the secondary indexes, must not exceed 20. If you project the same attribute into two different indexes, this counts as two distinct attributes when determining the total. </li> </ul> </li> <li>  <code>ProvisionedThroughput</code> - The provisioned throughput settings for the global secondary index, consisting of read and write capacity units, along with data about increases and decreases.  </li> </ul> If the table is in the <code>DELETING</code> state, no information about indexes will be returned.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LocalSecondaryIndexes</i> Represents one or more local secondary indexes on the table. Each index is scoped to a given partition key value. Tables with one or more local secondary indexes are subject to an item collection size limit, where the amount of data within a given item collection cannot exceed 10 GB. Each element is composed of: <ul> <li>  <code>IndexName</code> - The name of the local secondary index. </li> <li>  <code>KeySchema</code> - Specifies the complete index key schema. The attribute names in the key schema must be between 1 and 255 characters (inclusive). The key schema must begin with the same partition key as the table. </li> <li>  <code>Projection</code> - Specifies attributes that are copied (projected) from the table into the index. These are in addition to the primary key attributes and index key attributes, which are automatically projected. Each attribute specification is composed of: <ul> <li>  <code>ProjectionType</code> - One of the following: <ul> <li>  <code>KEYS_ONLY</code> - Only the index and primary keys are projected into the index. </li> <li>  <code>INCLUDE</code> - Only the specified table attributes are projected into the index. The list of projected attributes are in <code>NonKeyAttributes</code>. </li> <li>  <code>ALL</code> - All of the table attributes are projected into the index. </li> </ul> </li> <li>  <code>NonKeyAttributes</code> - A list of one or more non-key attribute names that are projected into the secondary index. The total count of attributes provided in <code>NonKeyAttributes</code>, summed across all of the secondary indexes, must not exceed 20. If you project the same attribute into two different indexes, this counts as two distinct attributes when determining the total. </li> </ul> </li> <li>  <code>IndexSizeBytes</code> - Represents the total size of the index, in bytes. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value. </li> <li>  <code>ItemCount</code> - Represents the number of items in the index. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value. </li> </ul> If the table is in the <code>DELETING</code> state, no information about indexes will be returned.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BillingModeSummary</i> Contains the details for the read/write capacity mode.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BillingModeSummary</b>: Contains the details for the read/write capacity mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableId</i> Unique identifier for the table for which the backup was created. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableArn</i> The Amazon Resource Name (ARN) that uniquely identifies the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ItemCount</i> The number of items in the specified table. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableSizeBytes</i> The total size of the specified table, in bytes. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisionedThroughput</i> The provisioned throughput settings for the table, consisting of read and write capacity units, along with data about increases and decreases.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ProvisionedThroughput</b>: The provisioned throughput settings for the table, consisting of read and write capacity units, along with data about increases and decreases.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDateTime</i> The date and time when the table was created, in <a href="http://www.epochconverter.com/">UNIX epoch time</a> format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableStatus</i> The current state of the table: <ul> <li>  <code>CREATING</code> - The table is being created. </li> <li>  <code>UPDATING</code> - The table is being updated. </li> <li>  <code>DELETING</code> - The table is being deleted. </li> <li>  <code>ACTIVE</code> - The table is ready for use. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeySchema</i> The primary key structure for the table. Each <code>KeySchemaElement</code> consists of: <ul> <li>  <code>AttributeName</code> - The name of the attribute. </li> <li>  <code>KeyType</code> - The role of the attribute: <ul> <li>  <code>HASH</code> - partition key </li> <li>  <code>RANGE</code> - sort key </li> </ul> <note> The partition key of an item is also known as its <i>hash attribute</i>. The term "hash attribute" derives from DynamoDB's usage of an internal hash function to evenly distribute data items across partitions, based on their partition key values. The sort key of an item is also known as its <i>range attribute</i>. The term "range attribute" derives from the way DynamoDB stores items with the same partition key physically close together, in sorted order by the sort key value. </note> </li> </ul> For more information about primary keys, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/DataModel.html#DataModelPrimaryKey">Primary Key</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableName</i> The name of the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttributeDefinitions</i> An array of <code>AttributeDefinition</code> objects. Each of these objects describes one attribute in the table and index key schema. Each <code>AttributeDefinition</code> object in this array is composed of: <ul> <li>  <code>AttributeName</code> - The name of the attribute. </li> <li>  <code>AttributeType</code> - The data type for the attribute. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WriteCapacityUnits</i> The maximum number of writes consumed per second before DynamoDB returns a <code>ThrottlingException</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadCapacityUnits</i> The maximum number of strongly consistent reads consumed per second before DynamoDB returns a <code>ThrottlingException</code>. Eventually consistent reads require less effort than strongly consistent reads, so a setting of 50 <code>ReadCapacityUnits</code> per second provides 100 eventually consistent <code>ReadCapacityUnits</code> per second.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfDecreasesToday</i> The number of provisioned throughput decreases for this table during this UTC calendar day. For current maximums on provisioned throughput decreases, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Limits.html">Limits</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LastDecreaseDateTime</i> The date and time of the last provisioned throughput decrease for this table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LastIncreaseDateTime</i> The date and time of the last provisioned throughput increase for this table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdateToPayPerRequestDateTime</i> Represents the time when <code>PAY_PER_REQUEST</code> was last set as the read/write capacity mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BillingMode</i> Controls how you are charged for read and write throughput and how you manage capacity. This setting can be changed later. <ul> <li>  <code>PROVISIONED</code> - Sets the read/write capacity mode to <code>PROVISIONED</code>. We recommend using <code>PROVISIONED</code> for predictable workloads. </li> <li>  <code>PAY_PER_REQUEST</code> - Sets the read/write capacity mode to <code>PAY_PER_REQUEST</code>. We recommend using <code>PAY_PER_REQUEST</code> for unpredictable workloads.  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamViewType</i>  When an item in the table is modified, <code>StreamViewType</code> determines what information is written to the stream for this table. Valid values for <code>StreamViewType</code> are: <ul> <li>  <code>KEYS_ONLY</code> - Only the key attributes of the modified item are written to the stream. </li> <li>  <code>NEW_IMAGE</code> - The entire item, as it appears after it was modified, is written to the stream. </li> <li>  <code>OLD_IMAGE</code> - The entire item, as it appeared before it was modified, is written to the stream. </li> <li>  <code>NEW_AND_OLD_IMAGES</code> - Both the new and the old item images of the item are written to the stream. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamEnabled</i> Indicates whether DynamoDB Streams is enabled (true) or disabled (false) on the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreInProgress</i> Indicates if a restore is in progress or not.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreDateTime</i> Point in time or source backup time.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceTableArn</i> ARN of the source table of the backup that is being restored.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceBackupArn</i> ARN of the backup from which the table was restored.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KMSMasterKeyArn</i> The KMS customer master key (CMK) ARN used for the KMS encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SSEType</i> Server-side encryption type. The only supported value is: <ul> <li>  <code>KMS</code> - Server-side encryption which uses AWS Key Management Service. Key is stored in your account and is managed by AWS KMS (KMS charges apply). </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Represents the current state of server-side encryption. The only supported values are: <ul> <li>  <code>ENABLED</code> - Server-side encryption is enabled. </li> <li>  <code>UPDATING</code> - Server-side encryption is being updated. </li> </ul><br/>        """
        pass

    def scan(self, TableName: str, IndexName: str=None, AttributesToGet: List[AttributeName]=None, Limit: int=None, Select: str=None, ScanFilter: FilterConditionMap=None, ConditionalOperator: str=None, ExclusiveStartKey: Key=None, ReturnConsumedCapacity: str=None, TotalSegments: int=None, Segment: int=None, ProjectionExpression: str=None, FilterExpression: str=None, ExpressionAttributeNames: ExpressionAttributeNameMap=None, ExpressionAttributeValues: ExpressionAttributeValueMap=None, ConsistentRead: ConsistentRead=None) -> ScanOutput:
        """The <code>Scan</code> operation returns one or more items and item attributes by accessing every item in a table or a secondary index. To have DynamoDB return fewer items, you can provide a <code>FilterExpression</code> operation. If the total number of scanned items exceeds the maximum dataset size limit of 1 MB, the scan stops and results are returned to the user as a <code>LastEvaluatedKey</code> value to continue the scan in a subsequent operation. The results also include the number of items exceeding the limit. A scan can result in no table data meeting the filter criteria.  A single <code>Scan</code> operation reads up to the maximum number of items set (if using the <code>Limit</code> parameter) or a maximum of 1 MB of data and then apply any filtering to the results using <code>FilterExpression</code>. If <code>LastEvaluatedKey</code> is present in the response, you need to paginate the result set. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Scan.html#Scan.Pagination">Paginating the Results</a> in the <i>Amazon DynamoDB Developer Guide</i>.   <code>Scan</code> operations proceed sequentially; however, for faster performance on a large table or secondary index, applications can request a parallel <code>Scan</code> operation by providing the <code>Segment</code> and <code>TotalSegments</code> parameters. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Scan.html#Scan.ParallelScan">Parallel Scan</a> in the <i>Amazon DynamoDB Developer Guide</i>.  <code>Scan</code> uses eventually consistent reads when accessing the data in a table; therefore, the result set might not include the changes to data in the table immediately before the operation began. If you need a consistent copy of the data, as of the time that the <code>Scan</code> begins, you can set the <code>ConsistentRead</code> parameter to <code>true</code>.

        :param self:
        :param ConsistentRead:  A Boolean value that determines the read consistency model during the scan: <ul> <li> If <code>ConsistentRead</code> is <code>false</code>, then the data returned from <code>Scan</code> might not contain the results from other recently completed write operations (<code>PutItem</code>, <code>UpdateItem</code>, or <code>DeleteItem</code>). </li> <li> If <code>ConsistentRead</code> is <code>true</code>, then all of the write operations that completed before the <code>Scan</code> began are guaranteed to be contained in the <code>Scan</code> response. </li> </ul> The default setting for <code>ConsistentRead</code> is <code>false</code>. The <code>ConsistentRead</code> parameter is not supported on global secondary indexes. If you scan a global secondary index with <code>ConsistentRead</code> set to true, you will receive a <code>ValidationException</code>.<br/>ExpressionAttributeValues:  One or more values that can be substituted in an expression. Use the <b>:</b> (colon) character in an expression to dereference an attribute value. For example, suppose that you wanted to check whether the value of the <code>ProductStatus</code> attribute was one of the following:   <code>Available | Backordered | Discontinued</code>  You would first need to specify <code>ExpressionAttributeValues</code> as follows:  <code>{ ":avail":{"S":"Available"}, ":back":{"S":"Backordered"}, ":disc":{"S":"Discontinued"} }</code>  You could then use these values in an expression, such as this:  <code>ProductStatus IN (:avail, :back, :disc)</code>  For more information on expression attribute values, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Expressions.SpecifyingConditions.html">Condition Expressions</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>ExpressionAttributeNames:  One or more substitution tokens for attribute names in an expression. The following are some use cases for using <code>ExpressionAttributeNames</code>: <ul> <li> To access an attribute whose name conflicts with a DynamoDB reserved word. </li> <li> To create a placeholder for repeating occurrences of an attribute name in an expression. </li> <li> To prevent special characters in an attribute name from being misinterpreted in an expression. </li> </ul> Use the <b>#</b> character in an expression to dereference an attribute name. For example, consider the following attribute name: <ul> <li>  <code>Percentile</code>  </li> </ul> The name of this attribute conflicts with a reserved word, so it cannot be used directly in an expression. (For the complete list of reserved words, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/ReservedWords.html">Reserved Words</a> in the <i>Amazon DynamoDB Developer Guide</i>). To work around this, you could specify the following for <code>ExpressionAttributeNames</code>: <ul> <li>  <code>{"#P":"Percentile"}</code>  </li> </ul> You could then use this substitution in an expression, as in this example: <ul> <li>  <code>#P = :val</code>  </li> </ul> <note> Tokens that begin with the <b>:</b> character are <i>expression attribute values</i>, which are placeholders for the actual value at runtime. </note> For more information on expression attribute names, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Expressions.AccessingItemAttributes.html">Specifying Item Attributes</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>FilterExpression:  A string that contains conditions that DynamoDB applies after the <code>Scan</code> operation, but before the data is returned to you. Items that do not satisfy the <code>FilterExpression</code> criteria are not returned. <note> A <code>FilterExpression</code> is applied after the items have already been read; the process of filtering does not consume any additional read capacity units. </note> For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/QueryAndScan.html#FilteringResults">Filter Expressions</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>ProjectionExpression:  A string that identifies one or more attributes to retrieve from the specified table or index. These attributes can include scalars, sets, or elements of a JSON document. The attributes in the expression must be separated by commas. If no attribute names are specified, then all attributes will be returned. If any of the requested attributes are not found, they will not appear in the result. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Expressions.AccessingItemAttributes.html">Specifying Item Attributes</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>Segment:  For a parallel <code>Scan</code> request, <code>Segment</code> identifies an individual segment to be scanned by an application worker. Segment IDs are zero-based, so the first segment is always 0. For example, if you want to use four application threads to scan a table or an index, then the first thread specifies a <code>Segment</code> value of 0, the second thread specifies 1, and so on. The value of <code>LastEvaluatedKey</code> returned from a parallel <code>Scan</code> request must be used as <code>ExclusiveStartKey</code> with the same segment ID in a subsequent <code>Scan</code> operation. The value for <code>Segment</code> must be greater than or equal to 0, and less than the value provided for <code>TotalSegments</code>. If you provide <code>Segment</code>, you must also provide <code>TotalSegments</code>.<br/>TotalSegments:  For a parallel <code>Scan</code> request, <code>TotalSegments</code> represents the total number of segments into which the <code>Scan</code> operation will be divided. The value of <code>TotalSegments</code> corresponds to the number of application workers that will perform the parallel scan. For example, if you want to use four application threads to scan a table or an index, specify a <code>TotalSegments</code> value of 4. The value for <code>TotalSegments</code> must be greater than or equal to 1, and less than or equal to 1000000. If you specify a <code>TotalSegments</code> value of 1, the <code>Scan</code> operation will be sequential rather than parallel. If you specify <code>TotalSegments</code>, you must also specify <code>Segment</code>.<br/>ReturnConsumedCapacity:  Determines the level of detail about provisioned throughput consumption that is returned in the response: <ul> <li>  <code>INDEXES</code> - The response includes the aggregate <code>ConsumedCapacity</code> for the operation, together with <code>ConsumedCapacity</code> for each table and secondary index that was accessed. Note that some operations, such as <code>GetItem</code> and <code>BatchGetItem</code>, do not access any indexes at all. In these cases, specifying <code>INDEXES</code> will only return <code>ConsumedCapacity</code> information for table(s). </li> <li>  <code>TOTAL</code> - The response includes only the aggregate <code>ConsumedCapacity</code> for the operation. </li> <li>  <code>NONE</code> - No <code>ConsumedCapacity</code> details are included in the response. </li> </ul><br/>ExclusiveStartKey:  The primary key of the first item that this operation will evaluate. Use the value that was returned for <code>LastEvaluatedKey</code> in the previous operation. The data type for <code>ExclusiveStartKey</code> must be String, Number or Binary. No set data types are allowed. In a parallel scan, a <code>Scan</code> request that includes <code>ExclusiveStartKey</code> must specify the same segment whose previous <code>Scan</code> returned the corresponding value of <code>LastEvaluatedKey</code>.<br/>ConditionalOperator:  This is a legacy parameter. Use <code>FilterExpression</code> instead. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/LegacyConditionalParameters.ConditionalOperator.html">ConditionalOperator</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>ScanFilter:  This is a legacy parameter. Use <code>FilterExpression</code> instead. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/LegacyConditionalParameters.ScanFilter.html">ScanFilter</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>Select:  The attributes to be returned in the result. You can retrieve all item attributes, specific item attributes, the count of matching items, or in the case of an index, some or all of the attributes projected into the index. <ul> <li>  <code>ALL_ATTRIBUTES</code> - Returns all of the item attributes from the specified table or index. If you query a local secondary index, then for each matching item in the index, DynamoDB fetches the entire item from the parent table. If the index is configured to project all item attributes, then all of the data can be obtained from the local secondary index, and no fetching is required. </li> <li>  <code>ALL_PROJECTED_ATTRIBUTES</code> - Allowed only when querying an index. Retrieves all attributes that have been projected into the index. If the index is configured to project all attributes, this return value is equivalent to specifying <code>ALL_ATTRIBUTES</code>. </li> <li>  <code>COUNT</code> - Returns the number of matching items, rather than the matching items themselves. </li> <li>  <code>SPECIFIC_ATTRIBUTES</code> - Returns only the attributes listed in <code>AttributesToGet</code>. This return value is equivalent to specifying <code>AttributesToGet</code> without specifying any value for <code>Select</code>. If you query or scan a local secondary index and request only attributes that are projected into that index, the operation reads only the index and not the table. If any of the requested attributes are not projected into the local secondary index, DynamoDB fetches each of these attributes from the parent table. This extra fetching incurs additional throughput cost and latency. If you query or scan a global secondary index, you can only request attributes that are projected into the index. Global secondary index queries cannot fetch attributes from the parent table. </li> </ul> If neither <code>Select</code> nor <code>AttributesToGet</code> are specified, DynamoDB defaults to <code>ALL_ATTRIBUTES</code> when accessing a table, and <code>ALL_PROJECTED_ATTRIBUTES</code> when accessing an index. You cannot use both <code>Select</code> and <code>AttributesToGet</code> together in a single request, unless the value for <code>Select</code> is <code>SPECIFIC_ATTRIBUTES</code>. (This usage is equivalent to specifying <code>AttributesToGet</code> without any value for <code>Select</code>.) <note> If you use the <code>ProjectionExpression</code> parameter, then the value for <code>Select</code> can only be <code>SPECIFIC_ATTRIBUTES</code>. Any other value for <code>Select</code> will return an error. </note><br/>Limit:  The maximum number of items to evaluate (not necessarily the number of matching items). If DynamoDB processes the number of items up to the limit while processing the results, it stops the operation and returns the matching values up to that point, and a key in <code>LastEvaluatedKey</code> to apply in a subsequent operation, so that you can pick up where you left off. Also, if the processed dataset size exceeds 1 MB before DynamoDB reaches this limit, it stops the operation and returns the matching values up to the limit, and a key in <code>LastEvaluatedKey</code> to apply in a subsequent operation to continue the operation. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/QueryAndScan.html">Working with Queries</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>AttributesToGet:  This is a legacy parameter. Use <code>ProjectionExpression</code> instead. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/LegacyConditionalParameters.AttributesToGet.html">AttributesToGet</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>IndexName:  The name of a secondary index to scan. This index can be any local secondary index or global secondary index. Note that if you use the <code>IndexName</code> parameter, you must also provide <code>TableName</code>.<br/>TableName:  The name of the table containing the requested items; or, if you provide <code>IndexName</code>, the name of the table to which that index belongs.<br/>
        :return: ConsumedCapacity:  The capacity units consumed by the <code>Scan</code> operation. The data returned includes the total provisioned throughput consumed, along with statistics for the table and any indexes involved in the operation. <code>ConsumedCapacity</code> is only returned if the <code>ReturnConsumedCapacity</code> parameter was specified. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/ProvisionedThroughputIntro.html">Provisioned Throughput</a> in the <i>Amazon DynamoDB Developer Guide</i>.:<br/>LastEvaluatedKey:  The primary key of the item where the operation stopped, inclusive of the previous result set. Use this value to start a new operation, excluding this value in the new request. If <code>LastEvaluatedKey</code> is empty, then the "last page" of results has been processed and there is no more data to be retrieved. If <code>LastEvaluatedKey</code> is not empty, it does not necessarily mean that there is more data in the result set. The only way to know when you have reached the end of the result set is when <code>LastEvaluatedKey</code> is empty.<br/>ScannedCount:  The number of items evaluated, before any <code>ScanFilter</code> is applied. A high <code>ScannedCount</code> value with few, or no, <code>Count</code> results indicates an inefficient <code>Scan</code> operation. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/QueryAndScan.html#Count">Count and ScannedCount</a> in the <i>Amazon DynamoDB Developer Guide</i>. If you did not use a filter in the request, then <code>ScannedCount</code> is the same as <code>Count</code>.<br/>Count:  The number of items in the response. If you set <code>ScanFilter</code> in the request, then <code>Count</code> is the number of items returned after the filter was applied, and <code>ScannedCount</code> is the number of matching items before the filter was applied. If you did not use a filter in the request, then <code>Count</code> is the same as <code>ScannedCount</code>.<br/>Items:  An array of item attributes that match the scan criteria. Each element in this array consists of an attribute name and the value for that attribute.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalSecondaryIndexes</i> The amount of throughput consumed on each global index affected by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LocalSecondaryIndexes</i> The amount of throughput consumed on each local index affected by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Table</i> The amount of throughput consumed on the table affected by the operation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Table</b>: The amount of throughput consumed on the table affected by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WriteCapacityUnits</i> The total number of write capacity units consumed by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadCapacityUnits</i> The total number of read capacity units consumed by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CapacityUnits</i> The total number of capacity units consumed by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableName</i> The name of the table that was affected by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CapacityUnits</i> The total number of capacity units consumed on a table or an index.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WriteCapacityUnits</i> The total number of write capacity units consumed on a table or an index.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadCapacityUnits</i> The total number of read capacity units consumed on a table or an index.<br/>        """
        pass

    def tag_resource(self, Tags: List[Tag], ResourceArn: str):
        """Associate a set of tags with an Amazon DynamoDB resource. You can then activate these user-defined tags so that they appear on the Billing and Cost Management console for cost allocation tracking. You can call TagResource up to five times per second, per account.  For an overview on tagging DynamoDB resources, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Tagging.html">Tagging for DynamoDB</a> in the <i>Amazon DynamoDB Developer Guide</i>.

        :param self:
        :param Tags:  The tags to be assigned to the Amazon DynamoDB resource.<br/>ResourceArn:  Identifies the Amazon DynamoDB resource to which tags should be added. This value is an Amazon Resource Name (ARN).<br/>
        :return:         """
        pass

    def transact_get_items(self, TransactItems: List[TransactGetItem], ReturnConsumedCapacity: str=None) -> TransactGetItemsOutput:
        """ <code>TransactGetItems</code> is a synchronous operation that atomically retrieves multiple items from one or more tables (but not from indexes) in a single account and Region. A <code>TransactGetItems</code> call can contain up to 25 <code>TransactGetItem</code> objects, each of which contains a <code>Get</code> structure that specifies an item to retrieve from a table in the account and Region. A call to <code>TransactGetItems</code> cannot retrieve items from tables in more than one AWS account or Region. The aggregate size of the items in the transaction cannot exceed 4 MB. <note> All AWS Regions and AWS GovCloud (US) support up to 25 items per transaction with up to 4 MB of data, except the following AWS Regions:  <ul> <li> China (Beijing) </li> <li> China (Ningxia) </li> </ul> The China (Beijing) and China (Ningxia) Regions support up to 10 items per transaction with up to 4 MB of data.  </note> DynamoDB rejects the entire <code>TransactGetItems</code> request if any of the following is true: <ul> <li> A conflicting operation is in the process of updating an item to be read. </li> <li> There is insufficient provisioned capacity for the transaction to be completed. </li> <li> There is a user error, such as an invalid data format. </li> <li> The aggregate size of the items in the transaction cannot exceed 4 MB. </li> </ul>

        :param self:
        :param ReturnConsumedCapacity:  A value of <code>TOTAL</code> causes consumed capacity information to be returned, and a value of <code>NONE</code> prevents that information from being returned. No other value is valid.<br/>TransactItems:  An ordered array of up to 25 <code>TransactGetItem</code> objects, each of which contains a <code>Get</code> structure.<br/>
        :return: Responses:  An ordered array of up to 25 <code>ItemResponse</code> objects, each of which corresponds to the <code>TransactGetItem</code> object in the same position in the <i>TransactItems</i> array. Each <code>ItemResponse</code> object contains a Map of the name-value pairs that are the projected attributes of the requested item. If a requested item could not be retrieved, the corresponding <code>ItemResponse</code> object is Null, or if the requested item has no projected attributes, the corresponding <code>ItemResponse</code> object is an empty Map. <br/>ConsumedCapacity:  If the <i>ReturnConsumedCapacity</i> value was <code>TOTAL</code>, this is an array of <code>ConsumedCapacity</code> objects, one for each table addressed by <code>TransactGetItem</code> objects in the <i>TransactItems</i> parameter. These <code>ConsumedCapacity</code> objects report the read-capacity units consumed by the <code>TransactGetItems</code> call in that table.<br/>        """
        pass

    def transact_write_items(self, TransactItems: List[TransactWriteItem], ReturnConsumedCapacity: str=None, ReturnItemCollectionMetrics: str=None, ClientRequestToken: str=None) -> TransactWriteItemsOutput:
        """ <code>TransactWriteItems</code> is a synchronous write operation that groups up to 25 action requests. These actions can target items in different tables, but not in different AWS accounts or Regions, and no two actions can target the same item. For example, you cannot both <code>ConditionCheck</code> and <code>Update</code> the same item. The aggregate size of the items in the transaction cannot exceed 4 MB. <note> All AWS Regions and AWS GovCloud (US) support up to 25 items per transaction with up to 4 MB of data, except the following AWS Regions:  <ul> <li> China (Beijing) </li> <li> China (Ningxia) </li> </ul> The China (Beijing) and China (Ningxia) Regions support up to 10 items per transaction with up to 4 MB of data.  </note> The actions are completed atomically so that either all of them succeed, or all of them fail. They are defined by the following objects: <ul> <li>  <code>Put</code>  &#x97;   Initiates a <code>PutItem</code> operation to write a new item. This structure specifies the primary key of the item to be written, the name of the table to write it in, an optional condition expression that must be satisfied for the write to succeed, a list of the item's attributes, and a field indicating whether to retrieve the item's attributes if the condition is not met. </li> <li>  <code>Update</code>  &#x97;   Initiates an <code>UpdateItem</code> operation to update an existing item. This structure specifies the primary key of the item to be updated, the name of the table where it resides, an optional condition expression that must be satisfied for the update to succeed, an expression that defines one or more attributes to be updated, and a field indicating whether to retrieve the item's attributes if the condition is not met. </li> <li>  <code>Delete</code>  &#x97;   Initiates a <code>DeleteItem</code> operation to delete an existing item. This structure specifies the primary key of the item to be deleted, the name of the table where it resides, an optional condition expression that must be satisfied for the deletion to succeed, and a field indicating whether to retrieve the item's attributes if the condition is not met. </li> <li>  <code>ConditionCheck</code>  &#x97;   Applies a condition to an item that is not being modified by the transaction. This structure specifies the primary key of the item to be checked, the name of the table where it resides, a condition expression that must be satisfied for the transaction to succeed, and a field indicating whether to retrieve the item's attributes if the condition is not met. </li> </ul> DynamoDB rejects the entire <code>TransactWriteItems</code> request if any of the following is true: <ul> <li> A condition in one of the condition expressions is not met. </li> <li> An ongoing operation is in the process of updating the same item. </li> <li> There is insufficient provisioned capacity for the transaction to be completed. </li> <li> An item size becomes too large (bigger than 400 KB), a local secondary index (LSI) becomes too large, or a similar validation error occurs because of changes made by the transaction. </li> <li> The aggregate size of the items in the transaction exceeds 4 MB. </li> <li> There is a user error, such as an invalid data format. </li> </ul>

        :param self:
        :param ClientRequestToken:  Providing a <code>ClientRequestToken</code> makes the call to <code>TransactWriteItems</code> idempotent, meaning that multiple identical calls have the same effect as one single call. Although multiple identical calls using the same client request token produce the same result on the server (no side effects), the responses to the calls might not be the same. If the <code>ReturnConsumedCapacity&gt;</code> parameter is set, then the initial <code>TransactWriteItems</code> call returns the amount of write capacity units consumed in making the changes. Subsequent <code>TransactWriteItems</code> calls with the same client token return the number of read capacity units consumed in reading the item. A client request token is valid for 10 minutes after the first request that uses it is completed. After 10 minutes, any request with the same client token is treated as a new request. Do not resubmit the same request with the same client token for more than 10 minutes, or the result might not be idempotent. If you submit a request with the same client token but a change in other parameters within the 10-minute idempotency window, DynamoDB returns an <code>IdempotentParameterMismatch</code> exception.<br/>ReturnItemCollectionMetrics:  Determines whether item collection metrics are returned. If set to <code>SIZE</code>, the response includes statistics about item collections (if any), that were modified during the operation and are returned in the response. If set to <code>NONE</code> (the default), no statistics are returned. <br/>ReturnConsumedCapacity:  Determines the level of detail about provisioned throughput consumption that is returned in the response: <ul> <li>  <code>INDEXES</code> - The response includes the aggregate <code>ConsumedCapacity</code> for the operation, together with <code>ConsumedCapacity</code> for each table and secondary index that was accessed. Note that some operations, such as <code>GetItem</code> and <code>BatchGetItem</code>, do not access any indexes at all. In these cases, specifying <code>INDEXES</code> will only return <code>ConsumedCapacity</code> information for table(s). </li> <li>  <code>TOTAL</code> - The response includes only the aggregate <code>ConsumedCapacity</code> for the operation. </li> <li>  <code>NONE</code> - No <code>ConsumedCapacity</code> details are included in the response. </li> </ul><br/>TransactItems:  An ordered array of up to 25 <code>TransactWriteItem</code> objects, each of which contains a <code>ConditionCheck</code>, <code>Put</code>, <code>Update</code>, or <code>Delete</code> object. These can operate on items in different tables, but the tables must reside in the same AWS account and Region, and no two of them can operate on the same item. <br/>
        :return: ItemCollectionMetrics:  A list of tables that were processed by <code>TransactWriteItems</code> and, for each table, information about any item collections that were affected by individual <code>UpdateItem</code>, <code>PutItem</code>, or <code>DeleteItem</code> operations. <br/>ConsumedCapacity:  The capacity units consumed by the entire <code>TransactWriteItems</code> operation. The values of the list are ordered according to the ordering of the <code>TransactItems</code> request parameter. <br/>        """
        pass

    def untag_resource(self, TagKeys: List[TagKeyString], ResourceArn: str):
        """Removes the association of tags from an Amazon DynamoDB resource. You can call <code>UntagResource</code> up to five times per second, per account.  For an overview on tagging DynamoDB resources, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Tagging.html">Tagging for DynamoDB</a> in the <i>Amazon DynamoDB Developer Guide</i>.

        :param self:
        :param TagKeys:  A list of tag keys. Existing tags of the resource whose keys are members of this list will be removed from the DynamoDB resource.<br/>ResourceArn:  The DynamoDB resource that the tags will be removed from. This value is an Amazon Resource Name (ARN).<br/>
        :return:         """
        pass

    def update_continuous_backups(self, PointInTimeRecoverySpecification: PointInTimeRecoverySpecification, TableName: str) -> UpdateContinuousBackupsOutput:
        """ <code>UpdateContinuousBackups</code> enables or disables point in time recovery for the specified table. A successful <code>UpdateContinuousBackups</code> call returns the current <code>ContinuousBackupsDescription</code>. Continuous backups are <code>ENABLED</code> on all tables at table creation. If point in time recovery is enabled, <code>PointInTimeRecoveryStatus</code> will be set to ENABLED.  Once continuous backups and point in time recovery are enabled, you can restore to any point in time within <code>EarliestRestorableDateTime</code> and <code>LatestRestorableDateTime</code>.   <code>LatestRestorableDateTime</code> is typically 5 minutes before the current time. You can restore your table to any point in time during the last 35 days. 

        :param self:
        :param PointInTimeRecoverySpecification:  Represents the settings used to enable point in time recovery.:<br/>TableName:  The name of the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PointInTimeRecoveryEnabled</i> Indicates whether point in time recovery is enabled (true) or disabled (false) on the table.<br/>
        :return: ContinuousBackupsDescription:  Represents the continuous backups and point in time recovery settings on the table.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PointInTimeRecoveryDescription</i> The description of the point in time recovery settings applied to the table.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PointInTimeRecoveryDescription</b>: The description of the point in time recovery settings applied to the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContinuousBackupsStatus</i>  <code>ContinuousBackupsStatus</code> can be one of the following states: ENABLED, DISABLED<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableDateTime</i>  <code>LatestRestorableDateTime</code> is typically 5 minutes before the current time. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestRestorableDateTime</i> Specifies the earliest point in time you can restore your table to. It You can restore your table to any point in time during the last 35 days. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PointInTimeRecoveryStatus</i> The current state of point in time recovery: <ul> <li>  <code>ENABLING</code> - Point in time recovery is being enabled. </li> <li>  <code>ENABLED</code> - Point in time recovery is enabled. </li> <li>  <code>DISABLED</code> - Point in time recovery is disabled. </li> </ul><br/>        """
        pass

    def update_global_table(self, ReplicaUpdates: List[ReplicaUpdate], GlobalTableName: str) -> UpdateGlobalTableOutput:
        """Adds or removes replicas in the specified global table. The global table must already exist to be able to use this operation. Any replica to be added must be empty, have the same name as the global table, have the same key schema, have DynamoDB Streams enabled, and have the same provisioned and maximum write capacity units. <note> Although you can use <code>UpdateGlobalTable</code> to add replicas and remove replicas in a single request, for simplicity we recommend that you issue separate requests for adding or removing replicas. </note>  If global secondary indexes are specified, then the following conditions must also be met:  <ul> <li>  The global secondary indexes must have the same name.  </li> <li>  The global secondary indexes must have the same hash key and sort key (if present).  </li> <li>  The global secondary indexes must have the same provisioned and maximum write capacity units.  </li> </ul>

        :param self:
        :param ReplicaUpdates:  A list of Regions that should be added or removed from the global table.<br/>GlobalTableName:  The global table name.<br/>
        :return: GlobalTableDescription:  Contains the details of the global table.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalTableName</i> The global table name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalTableStatus</i> The current state of the global table: <ul> <li>  <code>CREATING</code> - The global table is being created. </li> <li>  <code>UPDATING</code> - The global table is being updated. </li> <li>  <code>DELETING</code> - The global table is being deleted. </li> <li>  <code>ACTIVE</code> - The global table is ready for use. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDateTime</i> The creation time of the global table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalTableArn</i> The unique identifier of the global table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationGroup</i> The regions where the global table has replicas.<br/>        """
        pass

    def update_global_table_settings(self, GlobalTableName: str, GlobalTableBillingMode: str=None, GlobalTableProvisionedWriteCapacityUnits: PositiveLongObject=None, GlobalTableProvisionedWriteCapacityAutoScalingSettingsUpdate: AutoScalingSettingsUpdate=None, GlobalTableGlobalSecondaryIndexSettingsUpdate: List[GlobalTableGlobalSecondaryIndexSettingsUpdate]=None, ReplicaSettingsUpdate: List[ReplicaSettingsUpdate]=None) -> UpdateGlobalTableSettingsOutput:
        """Updates settings for a global table.

        :param self:
        :param ReplicaSettingsUpdate:  Represents the settings for a global table in a Region that will be modified.<br/>GlobalTableGlobalSecondaryIndexSettingsUpdate:  Represents the settings of a global secondary index for a global table that will be modified.<br/>GlobalTableProvisionedWriteCapacityAutoScalingSettingsUpdate:  Auto scaling settings for managing provisioned write capacity for the global table.:<br/>GlobalTableProvisionedWriteCapacityUnits:  The maximum number of writes consumed per second before DynamoDB returns a <code>ThrottlingException.</code> <br/>GlobalTableBillingMode:  The billing mode of the global table. If <code>GlobalTableBillingMode</code> is not specified, the global table defaults to <code>PROVISIONED</code> capacity billing mode.<br/>GlobalTableName:  The name of the global table<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ScalingPolicyUpdate</i> The scaling policy to apply for scaling target global table or global secondary index capacity units.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ScalingPolicyUpdate</b>: The scaling policy to apply for scaling target global table or global secondary index capacity units.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoScalingRoleArn</i> Role ARN used for configuring autoscaling policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoScalingDisabled</i> Disabled autoscaling for this global table or global secondary index.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumUnits</i> The maximum capacity units that a global table or global secondary index should be scaled up to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MinimumUnits</i> The minimum capacity units that a global table or global secondary index should be scaled down to.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyName</i> The name of the scaling policy.<br/>Represents a target tracking scaling policy configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TargetTrackingScalingPolicyConfiguration</b>: Represents a target tracking scaling policy configuration.
        :return: ReplicaSettings:  The Region-specific settings for the global table.<br/>GlobalTableName:  The name of the global table.<br/>        """
        pass

    def update_item(self, Key: Key, TableName: str, AttributeUpdates: AttributeUpdates=None, Expected: ExpectedAttributeMap=None, ConditionalOperator: str=None, ReturnValues: str=None, ReturnConsumedCapacity: str=None, ReturnItemCollectionMetrics: str=None, UpdateExpression: str=None, ConditionExpression: str=None, ExpressionAttributeNames: ExpressionAttributeNameMap=None, ExpressionAttributeValues: ExpressionAttributeValueMap=None) -> UpdateItemOutput:
        """Edits an existing item's attributes, or adds a new item to the table if it does not already exist. You can put, delete, or add attribute values. You can also perform a conditional update on an existing item (insert a new attribute name-value pair if it doesn't exist, or replace an existing name-value pair if it has certain expected attribute values). You can also return the item's attribute values in the same <code>UpdateItem</code> operation using the <code>ReturnValues</code> parameter.

        :param self:
        :param ExpressionAttributeValues:  One or more values that can be substituted in an expression. Use the <b>:</b> (colon) character in an expression to dereference an attribute value. For example, suppose that you wanted to check whether the value of the <code>ProductStatus</code> attribute was one of the following:   <code>Available | Backordered | Discontinued</code>  You would first need to specify <code>ExpressionAttributeValues</code> as follows:  <code>{ ":avail":{"S":"Available"}, ":back":{"S":"Backordered"}, ":disc":{"S":"Discontinued"} }</code>  You could then use these values in an expression, such as this:  <code>ProductStatus IN (:avail, :back, :disc)</code>  For more information on expression attribute values, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Expressions.SpecifyingConditions.html">Condition Expressions</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>ExpressionAttributeNames:  One or more substitution tokens for attribute names in an expression. The following are some use cases for using <code>ExpressionAttributeNames</code>: <ul> <li> To access an attribute whose name conflicts with a DynamoDB reserved word. </li> <li> To create a placeholder for repeating occurrences of an attribute name in an expression. </li> <li> To prevent special characters in an attribute name from being misinterpreted in an expression. </li> </ul> Use the <b>#</b> character in an expression to dereference an attribute name. For example, consider the following attribute name: <ul> <li>  <code>Percentile</code>  </li> </ul> The name of this attribute conflicts with a reserved word, so it cannot be used directly in an expression. (For the complete list of reserved words, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/ReservedWords.html">Reserved Words</a> in the <i>Amazon DynamoDB Developer Guide</i>.) To work around this, you could specify the following for <code>ExpressionAttributeNames</code>: <ul> <li>  <code>{"#P":"Percentile"}</code>  </li> </ul> You could then use this substitution in an expression, as in this example: <ul> <li>  <code>#P = :val</code>  </li> </ul> <note> Tokens that begin with the <b>:</b> character are <i>expression attribute values</i>, which are placeholders for the actual value at runtime. </note> For more information about expression attribute names, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Expressions.AccessingItemAttributes.html">Specifying Item Attributes</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>ConditionExpression:  A condition that must be satisfied in order for a conditional update to succeed. An expression can contain any of the following: <ul> <li> Functions: <code>attribute_exists | attribute_not_exists | attribute_type | contains | begins_with | size</code>  These function names are case-sensitive. </li> <li> Comparison operators: <code>= | &lt;&gt; | &lt; | &gt; | &lt;= | &gt;= | BETWEEN | IN </code>  </li> <li>  Logical operators: <code>AND | OR | NOT</code>  </li> </ul> For more information about condition expressions, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Expressions.SpecifyingConditions.html">Specifying Conditions</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>UpdateExpression:  An expression that defines one or more attributes to be updated, the action to be performed on them, and new values for them. The following action values are available for <code>UpdateExpression</code>. <ul> <li>  <code>SET</code> - Adds one or more attributes and values to an item. If any of these attributes already exist, they are replaced by the new values. You can also use <code>SET</code> to add or subtract from an attribute that is of type Number. For example: <code>SET myNum = myNum + :val</code>   <code>SET</code> supports the following functions: <ul> <li>  <code>if_not_exists (path, operand)</code> - if the item does not contain an attribute at the specified path, then <code>if_not_exists</code> evaluates to operand; otherwise, it evaluates to path. You can use this function to avoid overwriting an attribute that may already be present in the item. </li> <li>  <code>list_append (operand, operand)</code> - evaluates to a list with a new element added to it. You can append the new element to the start or the end of the list by reversing the order of the operands. </li> </ul> These function names are case-sensitive. </li> <li>  <code>REMOVE</code> - Removes one or more attributes from an item. </li> <li>  <code>ADD</code> - Adds the specified value to the item, if the attribute does not already exist. If the attribute does exist, then the behavior of <code>ADD</code> depends on the data type of the attribute: <ul> <li> If the existing attribute is a number, and if <code>Value</code> is also a number, then <code>Value</code> is mathematically added to the existing attribute. If <code>Value</code> is a negative number, then it is subtracted from the existing attribute. <note> If you use <code>ADD</code> to increment or decrement a number value for an item that doesn't exist before the update, DynamoDB uses <code>0</code> as the initial value. Similarly, if you use <code>ADD</code> for an existing item to increment or decrement an attribute value that doesn't exist before the update, DynamoDB uses <code>0</code> as the initial value. For example, suppose that the item you want to update doesn't have an attribute named <code>itemcount</code>, but you decide to <code>ADD</code> the number <code>3</code> to this attribute anyway. DynamoDB will create the <code>itemcount</code> attribute, set its initial value to <code>0</code>, and finally add <code>3</code> to it. The result will be a new <code>itemcount</code> attribute in the item, with a value of <code>3</code>. </note> </li> <li> If the existing data type is a set and if <code>Value</code> is also a set, then <code>Value</code> is added to the existing set. For example, if the attribute value is the set <code>[1,2]</code>, and the <code>ADD</code> action specified <code>[3]</code>, then the final attribute value is <code>[1,2,3]</code>. An error occurs if an <code>ADD</code> action is specified for a set attribute and the attribute type specified does not match the existing set type.  Both sets must have the same primitive data type. For example, if the existing data type is a set of strings, the <code>Value</code> must also be a set of strings. </li> </ul> <important> The <code>ADD</code> action only supports Number and set data types. In addition, <code>ADD</code> can only be used on top-level attributes, not nested attributes. </important> </li> <li>  <code>DELETE</code> - Deletes an element from a set. If a set of values is specified, then those values are subtracted from the old set. For example, if the attribute value was the set <code>[a,b,c]</code> and the <code>DELETE</code> action specifies <code>[a,c]</code>, then the final attribute value is <code>[b]</code>. Specifying an empty set is an error. <important> The <code>DELETE</code> action only supports set data types. In addition, <code>DELETE</code> can only be used on top-level attributes, not nested attributes. </important> </li> </ul> You can have many actions in a single expression, such as the following: <code>SET a=:value1, b=:value2 DELETE :value3, :value4, :value5</code>  For more information on update expressions, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Expressions.Modifying.html">Modifying Items and Attributes</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>ReturnItemCollectionMetrics:  Determines whether item collection metrics are returned. If set to <code>SIZE</code>, the response includes statistics about item collections, if any, that were modified during the operation are returned in the response. If set to <code>NONE</code> (the default), no statistics are returned.<br/>ReturnConsumedCapacity:  Determines the level of detail about provisioned throughput consumption that is returned in the response: <ul> <li>  <code>INDEXES</code> - The response includes the aggregate <code>ConsumedCapacity</code> for the operation, together with <code>ConsumedCapacity</code> for each table and secondary index that was accessed. Note that some operations, such as <code>GetItem</code> and <code>BatchGetItem</code>, do not access any indexes at all. In these cases, specifying <code>INDEXES</code> will only return <code>ConsumedCapacity</code> information for table(s). </li> <li>  <code>TOTAL</code> - The response includes only the aggregate <code>ConsumedCapacity</code> for the operation. </li> <li>  <code>NONE</code> - No <code>ConsumedCapacity</code> details are included in the response. </li> </ul><br/>ReturnValues:  Use <code>ReturnValues</code> if you want to get the item attributes as they appear before or after they are updated. For <code>UpdateItem</code>, the valid values are: <ul> <li>  <code>NONE</code> - If <code>ReturnValues</code> is not specified, or if its value is <code>NONE</code>, then nothing is returned. (This setting is the default for <code>ReturnValues</code>.) </li> <li>  <code>ALL_OLD</code> - Returns all of the attributes of the item, as they appeared before the UpdateItem operation. </li> <li>  <code>UPDATED_OLD</code> - Returns only the updated attributes, as they appeared before the UpdateItem operation. </li> <li>  <code>ALL_NEW</code> - Returns all of the attributes of the item, as they appear after the UpdateItem operation. </li> <li>  <code>UPDATED_NEW</code> - Returns only the updated attributes, as they appear after the UpdateItem operation. </li> </ul> There is no additional cost associated with requesting a return value aside from the small network and processing overhead of receiving a larger response. No read capacity units are consumed. The values returned are strongly consistent.<br/>ConditionalOperator:  This is a legacy parameter. Use <code>ConditionExpression</code> instead. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/LegacyConditionalParameters.ConditionalOperator.html">ConditionalOperator</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>Expected:  This is a legacy parameter. Use <code>ConditionExpression</code> instead. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/LegacyConditionalParameters.Expected.html">Expected</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>AttributeUpdates:  This is a legacy parameter. Use <code>UpdateExpression</code> instead. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/LegacyConditionalParameters.AttributeUpdates.html">AttributeUpdates</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>Key:  The primary key of the item to be updated. Each element consists of an attribute name and a value for that attribute. For the primary key, you must provide all of the attributes. For example, with a simple primary key, you only need to provide a value for the partition key. For a composite primary key, you must provide values for both the partition key and the sort key.<br/>TableName:  The name of the table containing the item to update.<br/>
        :return: ItemCollectionMetrics:  Information about item collections, if any, that were affected by the <code>UpdateItem</code> operation. <code>ItemCollectionMetrics</code> is only returned if the <code>ReturnItemCollectionMetrics</code> parameter was specified. If the table does not have any local secondary indexes, this information is not returned in the response. Each <code>ItemCollectionMetrics</code> element consists of: <ul> <li>  <code>ItemCollectionKey</code> - The partition key value of the item collection. This is the same as the partition key value of the item itself. </li> <li>  <code>SizeEstimateRangeGB</code> - An estimate of item collection size, in gigabytes. This value is a two-element array containing a lower bound and an upper bound for the estimate. The estimate includes the size of all the items in the table, plus the size of all attributes projected into all of the local secondary indexes on that table. Use this estimate to measure whether a local secondary index is approaching its size limit. The estimate is subject to change over time; therefore, do not rely on the precision or accuracy of the estimate. </li> </ul>:<br/>ConsumedCapacity:  The capacity units consumed by the <code>UpdateItem</code> operation. The data returned includes the total provisioned throughput consumed, along with statistics for the table and any indexes involved in the operation. <code>ConsumedCapacity</code> is only returned if the <code>ReturnConsumedCapacity</code> parameter was specified. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/ProvisionedThroughputIntro.html">Provisioned Throughput</a> in the <i>Amazon DynamoDB Developer Guide</i>.:<br/>Attributes:  A map of attribute values as they appear before or after the <code>UpdateItem</code> operation, as determined by the <code>ReturnValues</code> parameter. The <code>Attributes</code> map is only present if <code>ReturnValues</code> was specified as something other than <code>NONE</code> in the request. Each element represents one attribute.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalSecondaryIndexes</i> The amount of throughput consumed on each global index affected by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LocalSecondaryIndexes</i> The amount of throughput consumed on each local index affected by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Table</i> The amount of throughput consumed on the table affected by the operation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Table</b>: The amount of throughput consumed on the table affected by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WriteCapacityUnits</i> The total number of write capacity units consumed by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadCapacityUnits</i> The total number of read capacity units consumed by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CapacityUnits</i> The total number of capacity units consumed by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableName</i> The name of the table that was affected by the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CapacityUnits</i> The total number of capacity units consumed on a table or an index.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WriteCapacityUnits</i> The total number of write capacity units consumed on a table or an index.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadCapacityUnits</i> The total number of read capacity units consumed on a table or an index.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SizeEstimateRangeGB</i> An estimate of item collection size, in gigabytes. This value is a two-element array containing a lower bound and an upper bound for the estimate. The estimate includes the size of all the items in the table, plus the size of all attributes projected into all of the local secondary indexes on that table. Use this estimate to measure whether a local secondary index is approaching its size limit. The estimate is subject to change over time; therefore, do not rely on the precision or accuracy of the estimate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ItemCollectionKey</i> The partition key value of the item collection. This value is the same as the partition key value of the item.<br/>        """
        pass

    def update_table(self, TableName: str, AttributeDefinitions: List[AttributeDefinition]=None, BillingMode: str=None, ProvisionedThroughput: ProvisionedThroughput=None, GlobalSecondaryIndexUpdates: List[GlobalSecondaryIndexUpdate]=None, StreamSpecification: StreamSpecification=None, SSESpecification: SSESpecification=None) -> UpdateTableOutput:
        """Modifies the provisioned throughput settings, global secondary indexes, or DynamoDB Streams settings for a given table. You can only perform one of the following operations at once: <ul> <li> Modify the provisioned throughput settings of the table. </li> <li> Enable or disable DynamoDB Streams on the table. </li> <li> Remove a global secondary index from the table. </li> <li> Create a new global secondary index on the table. After the index begins backfilling, you can use <code>UpdateTable</code> to perform other operations. </li> </ul>  <code>UpdateTable</code> is an asynchronous operation; while it is executing, the table status changes from <code>ACTIVE</code> to <code>UPDATING</code>. While it is <code>UPDATING</code>, you cannot issue another <code>UpdateTable</code> request. When the table returns to the <code>ACTIVE</code> state, the <code>UpdateTable</code> operation is complete.

        :param self:
        :param SSESpecification:  The new server-side encryption settings for the specified table.:<br/>StreamSpecification:  Represents the DynamoDB Streams configuration for the table. <note> You receive a <code>ResourceInUseException</code> if you try to enable a stream on a table that already has a stream, or if you try to disable a stream on a table that doesn't have a stream. </note>:<br/>GlobalSecondaryIndexUpdates:  An array of one or more global secondary indexes for the table. For each index in the array, you can request one action: <ul> <li>  <code>Create</code> - add a new global secondary index to the table. </li> <li>  <code>Update</code> - modify the provisioned throughput settings of an existing global secondary index. </li> <li>  <code>Delete</code> - remove a global secondary index from the table. </li> </ul> For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/GSI.OnlineOps.html">Managing Global Secondary Indexes</a> in the <i>Amazon DynamoDB Developer Guide</i>. <br/>ProvisionedThroughput:  The new provisioned throughput settings for the specified table or index.:<br/>BillingMode:  Controls how you are charged for read and write throughput and how you manage capacity. When switching from pay-per-request to provisioned capacity, initial provisioned capacity values must be set. The initial provisioned capacity values are estimated based on the consumed read and write capacity of your table and global secondary indexes over the past 30 minutes. <ul> <li>  <code>PROVISIONED</code> - Sets the billing mode to <code>PROVISIONED</code>. We recommend using <code>PROVISIONED</code> for predictable workloads. </li> <li>  <code>PAY_PER_REQUEST</code> - Sets the billing mode to <code>PAY_PER_REQUEST</code>. We recommend using <code>PAY_PER_REQUEST</code> for unpredictable workloads.  </li> </ul><br/>TableName:  The name of the table to be updated.<br/>AttributeDefinitions:  An array of attributes that describe the key schema for the table and indexes. If you are adding a new global secondary index to the table, <code>AttributeDefinitions</code> must include the key element(s) of the new index.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WriteCapacityUnits</i> The maximum number of writes consumed per second before DynamoDB returns a <code>ThrottlingException</code>. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/WorkingWithTables.html#ProvisionedThroughput">Specifying Read and Write Requirements</a> in the <i>Amazon DynamoDB Developer Guide</i>. If read/write capacity mode is <code>PAY_PER_REQUEST</code> the value is set to 0.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadCapacityUnits</i> The maximum number of strongly consistent reads consumed per second before DynamoDB returns a <code>ThrottlingException</code>. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/WorkingWithTables.html#ProvisionedThroughput">Specifying Read and Write Requirements</a> in the <i>Amazon DynamoDB Developer Guide</i>. If read/write capacity mode is <code>PAY_PER_REQUEST</code> the value is set to 0.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamViewType</i>  When an item in the table is modified, <code>StreamViewType</code> determines what information is written to the stream for this table. Valid values for <code>StreamViewType</code> are: <ul> <li>  <code>KEYS_ONLY</code> - Only the key attributes of the modified item are written to the stream. </li> <li>  <code>NEW_IMAGE</code> - The entire item, as it appears after it was modified, is written to the stream. </li> <li>  <code>OLD_IMAGE</code> - The entire item, as it appeared before it was modified, is written to the stream. </li> <li>  <code>NEW_AND_OLD_IMAGES</code> - Both the new and the old item images of the item are written to the stream. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamEnabled</i> Indicates whether DynamoDB Streams is enabled (true) or disabled (false) on the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KMSMasterKeyId</i> The KMS Customer Master Key (CMK) which should be used for the KMS encryption. To specify a CMK, use its key ID, Amazon Resource Name (ARN), alias name, or alias ARN. Note that you should only provide this parameter if the key is different from the default DynamoDB Customer Master Key alias/aws/dynamodb.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SSEType</i> Server-side encryption type. The only supported value is: <ul> <li>  <code>KMS</code> - Server-side encryption which uses AWS Key Management Service. Key is stored in your account and is managed by AWS KMS (KMS charges apply). </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Indicates whether server-side encryption is done using an AWS managed CMK or an AWS owned CMK. If enabled (true), server-side encryption type is set to <code>KMS</code> and an AWS managed CMK is used (AWS KMS charges apply). If disabled (false) or not specified, server-side encryption is set to AWS owned CMK.<br/>
        :return: TableDescription:  Represents the properties of the table.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SSEDescription</i> The description of the server-side encryption status on the specified table.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SSEDescription</b>: The description of the server-side encryption status on the specified table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreSummary</i> Contains details for the restore.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RestoreSummary</b>: Contains details for the restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestStreamArn</i> The Amazon Resource Name (ARN) that uniquely identifies the latest stream for this table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestStreamLabel</i> A timestamp, in ISO 8601 format, for this stream. Note that <code>LatestStreamLabel</code> is not a unique identifier for the stream, because it is possible that a stream from another table might have the same timestamp. However, the combination of the following three elements is guaranteed to be unique: <ul> <li> the AWS customer ID. </li> <li> the table name. </li> <li> the <code>StreamLabel</code>. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamSpecification</i> The current DynamoDB Streams configuration for the table.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamSpecification</b>: The current DynamoDB Streams configuration for the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalSecondaryIndexes</i> The global secondary indexes, if any, on the table. Each index is scoped to a given partition key value. Each element is composed of: <ul> <li>  <code>Backfilling</code> - If true, then the index is currently in the backfilling phase. Backfilling occurs only when a new global secondary index is added to the table; it is the process by which DynamoDB populates the new index with data from the table. (This attribute does not appear for indexes that were created during a <code>CreateTable</code> operation.) </li> <li>  <code>IndexName</code> - The name of the global secondary index. </li> <li>  <code>IndexSizeBytes</code> - The total size of the global secondary index, in bytes. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value.  </li> <li>  <code>IndexStatus</code> - The current status of the global secondary index: <ul> <li>  <code>CREATING</code> - The index is being created. </li> <li>  <code>UPDATING</code> - The index is being updated. </li> <li>  <code>DELETING</code> - The index is being deleted. </li> <li>  <code>ACTIVE</code> - The index is ready for use. </li> </ul> </li> <li>  <code>ItemCount</code> - The number of items in the global secondary index. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value.  </li> <li>  <code>KeySchema</code> - Specifies the complete index key schema. The attribute names in the key schema must be between 1 and 255 characters (inclusive). The key schema must begin with the same partition key as the table. </li> <li>  <code>Projection</code> - Specifies attributes that are copied (projected) from the table into the index. These are in addition to the primary key attributes and index key attributes, which are automatically projected. Each attribute specification is composed of: <ul> <li>  <code>ProjectionType</code> - One of the following: <ul> <li>  <code>KEYS_ONLY</code> - Only the index and primary keys are projected into the index. </li> <li>  <code>INCLUDE</code> - Only the specified table attributes are projected into the index. The list of projected attributes are in <code>NonKeyAttributes</code>. </li> <li>  <code>ALL</code> - All of the table attributes are projected into the index. </li> </ul> </li> <li>  <code>NonKeyAttributes</code> - A list of one or more non-key attribute names that are projected into the secondary index. The total count of attributes provided in <code>NonKeyAttributes</code>, summed across all of the secondary indexes, must not exceed 20. If you project the same attribute into two different indexes, this counts as two distinct attributes when determining the total. </li> </ul> </li> <li>  <code>ProvisionedThroughput</code> - The provisioned throughput settings for the global secondary index, consisting of read and write capacity units, along with data about increases and decreases.  </li> </ul> If the table is in the <code>DELETING</code> state, no information about indexes will be returned.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LocalSecondaryIndexes</i> Represents one or more local secondary indexes on the table. Each index is scoped to a given partition key value. Tables with one or more local secondary indexes are subject to an item collection size limit, where the amount of data within a given item collection cannot exceed 10 GB. Each element is composed of: <ul> <li>  <code>IndexName</code> - The name of the local secondary index. </li> <li>  <code>KeySchema</code> - Specifies the complete index key schema. The attribute names in the key schema must be between 1 and 255 characters (inclusive). The key schema must begin with the same partition key as the table. </li> <li>  <code>Projection</code> - Specifies attributes that are copied (projected) from the table into the index. These are in addition to the primary key attributes and index key attributes, which are automatically projected. Each attribute specification is composed of: <ul> <li>  <code>ProjectionType</code> - One of the following: <ul> <li>  <code>KEYS_ONLY</code> - Only the index and primary keys are projected into the index. </li> <li>  <code>INCLUDE</code> - Only the specified table attributes are projected into the index. The list of projected attributes are in <code>NonKeyAttributes</code>. </li> <li>  <code>ALL</code> - All of the table attributes are projected into the index. </li> </ul> </li> <li>  <code>NonKeyAttributes</code> - A list of one or more non-key attribute names that are projected into the secondary index. The total count of attributes provided in <code>NonKeyAttributes</code>, summed across all of the secondary indexes, must not exceed 20. If you project the same attribute into two different indexes, this counts as two distinct attributes when determining the total. </li> </ul> </li> <li>  <code>IndexSizeBytes</code> - Represents the total size of the index, in bytes. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value. </li> <li>  <code>ItemCount</code> - Represents the number of items in the index. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value. </li> </ul> If the table is in the <code>DELETING</code> state, no information about indexes will be returned.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BillingModeSummary</i> Contains the details for the read/write capacity mode.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BillingModeSummary</b>: Contains the details for the read/write capacity mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableId</i> Unique identifier for the table for which the backup was created. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableArn</i> The Amazon Resource Name (ARN) that uniquely identifies the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ItemCount</i> The number of items in the specified table. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableSizeBytes</i> The total size of the specified table, in bytes. DynamoDB updates this value approximately every six hours. Recent changes might not be reflected in this value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisionedThroughput</i> The provisioned throughput settings for the table, consisting of read and write capacity units, along with data about increases and decreases.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ProvisionedThroughput</b>: The provisioned throughput settings for the table, consisting of read and write capacity units, along with data about increases and decreases.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDateTime</i> The date and time when the table was created, in <a href="http://www.epochconverter.com/">UNIX epoch time</a> format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableStatus</i> The current state of the table: <ul> <li>  <code>CREATING</code> - The table is being created. </li> <li>  <code>UPDATING</code> - The table is being updated. </li> <li>  <code>DELETING</code> - The table is being deleted. </li> <li>  <code>ACTIVE</code> - The table is ready for use. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeySchema</i> The primary key structure for the table. Each <code>KeySchemaElement</code> consists of: <ul> <li>  <code>AttributeName</code> - The name of the attribute. </li> <li>  <code>KeyType</code> - The role of the attribute: <ul> <li>  <code>HASH</code> - partition key </li> <li>  <code>RANGE</code> - sort key </li> </ul> <note> The partition key of an item is also known as its <i>hash attribute</i>. The term "hash attribute" derives from DynamoDB's usage of an internal hash function to evenly distribute data items across partitions, based on their partition key values. The sort key of an item is also known as its <i>range attribute</i>. The term "range attribute" derives from the way DynamoDB stores items with the same partition key physically close together, in sorted order by the sort key value. </note> </li> </ul> For more information about primary keys, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/DataModel.html#DataModelPrimaryKey">Primary Key</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableName</i> The name of the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttributeDefinitions</i> An array of <code>AttributeDefinition</code> objects. Each of these objects describes one attribute in the table and index key schema. Each <code>AttributeDefinition</code> object in this array is composed of: <ul> <li>  <code>AttributeName</code> - The name of the attribute. </li> <li>  <code>AttributeType</code> - The data type for the attribute. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WriteCapacityUnits</i> The maximum number of writes consumed per second before DynamoDB returns a <code>ThrottlingException</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadCapacityUnits</i> The maximum number of strongly consistent reads consumed per second before DynamoDB returns a <code>ThrottlingException</code>. Eventually consistent reads require less effort than strongly consistent reads, so a setting of 50 <code>ReadCapacityUnits</code> per second provides 100 eventually consistent <code>ReadCapacityUnits</code> per second.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfDecreasesToday</i> The number of provisioned throughput decreases for this table during this UTC calendar day. For current maximums on provisioned throughput decreases, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Limits.html">Limits</a> in the <i>Amazon DynamoDB Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LastDecreaseDateTime</i> The date and time of the last provisioned throughput decrease for this table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LastIncreaseDateTime</i> The date and time of the last provisioned throughput increase for this table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdateToPayPerRequestDateTime</i> Represents the time when <code>PAY_PER_REQUEST</code> was last set as the read/write capacity mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BillingMode</i> Controls how you are charged for read and write throughput and how you manage capacity. This setting can be changed later. <ul> <li>  <code>PROVISIONED</code> - Sets the read/write capacity mode to <code>PROVISIONED</code>. We recommend using <code>PROVISIONED</code> for predictable workloads. </li> <li>  <code>PAY_PER_REQUEST</code> - Sets the read/write capacity mode to <code>PAY_PER_REQUEST</code>. We recommend using <code>PAY_PER_REQUEST</code> for unpredictable workloads.  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamViewType</i>  When an item in the table is modified, <code>StreamViewType</code> determines what information is written to the stream for this table. Valid values for <code>StreamViewType</code> are: <ul> <li>  <code>KEYS_ONLY</code> - Only the key attributes of the modified item are written to the stream. </li> <li>  <code>NEW_IMAGE</code> - The entire item, as it appears after it was modified, is written to the stream. </li> <li>  <code>OLD_IMAGE</code> - The entire item, as it appeared before it was modified, is written to the stream. </li> <li>  <code>NEW_AND_OLD_IMAGES</code> - Both the new and the old item images of the item are written to the stream. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamEnabled</i> Indicates whether DynamoDB Streams is enabled (true) or disabled (false) on the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreInProgress</i> Indicates if a restore is in progress or not.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreDateTime</i> Point in time or source backup time.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceTableArn</i> ARN of the source table of the backup that is being restored.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceBackupArn</i> ARN of the backup from which the table was restored.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KMSMasterKeyArn</i> The KMS customer master key (CMK) ARN used for the KMS encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SSEType</i> Server-side encryption type. The only supported value is: <ul> <li>  <code>KMS</code> - Server-side encryption which uses AWS Key Management Service. Key is stored in your account and is managed by AWS KMS (KMS charges apply). </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Represents the current state of server-side encryption. The only supported values are: <ul> <li>  <code>ENABLED</code> - Server-side encryption is enabled. </li> <li>  <code>UPDATING</code> - Server-side encryption is being updated. </li> </ul><br/>        """
        pass

    def update_time_to_live(self, TimeToLiveSpecification: TimeToLiveSpecification, TableName: str) -> UpdateTimeToLiveOutput:
        """The <code>UpdateTimeToLive</code> method enables or disables Time to Live (TTL) for the specified table. A successful <code>UpdateTimeToLive</code> call returns the current <code>TimeToLiveSpecification</code>. It can take up to one hour for the change to fully process. Any additional <code>UpdateTimeToLive</code> calls for the same table during this one hour duration result in a <code>ValidationException</code>.  TTL compares the current time in epoch time format to the time stored in the TTL attribute of an item. If the epoch time value stored in the attribute is less than the current time, the item is marked as expired and subsequently deleted. <note>  The epoch time format is the number of seconds elapsed since 12:00:00 AM January 1, 1970 UTC.  </note> DynamoDB deletes expired items on a best-effort basis to ensure availability of throughput for other data operations.  <important> DynamoDB typically deletes expired items within two days of expiration. The exact duration within which an item gets deleted after expiration is specific to the nature of the workload. Items that have expired and not been deleted will still show up in reads, queries, and scans. </important> As items are deleted, they are removed from any local secondary index and global secondary index immediately in the same eventually consistent way as a standard delete operation. For more information, see <a href="https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/TTL.html">Time To Live</a> in the Amazon DynamoDB Developer Guide. 

        :param self:
        :param TimeToLiveSpecification:  Represents the settings used to enable or disable Time to Live for the specified table.:<br/>TableName:  The name of the table to be configured.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttributeName</i> The name of the TTL attribute used to store the expiration time for items in the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Indicates whether TTL is to be enabled (true) or disabled (false) on the table.<br/>
        :return: TimeToLiveSpecification:  Represents the output of an <code>UpdateTimeToLive</code> operation.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttributeName</i> The name of the TTL attribute used to store the expiration time for items in the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Indicates whether TTL is to be enabled (true) or disabled (false) on the table.<br/>        """
        pass

    class DynamodbResource:
        
        class Table:
            """Creates a Table resource.::

  table = dynamodb.Table('name')

:type name: string
:param name: The Table's name identifier. This **must** be set.

:rtype: :py:class:`DynamoDB.Table`
:returns: A Table resource"""

            def __init__(self, name):
                pass
            
            class KeySchemaElement(Mapping):
                pass

            class StreamSpecification(Mapping):
                pass

            class Long(object):
                pass

            class LocalSecondaryIndexDescription(Mapping):
                pass

            class BillingModeSummary(Mapping):
                pass

            class ProvisionedThroughputDescription(Mapping):
                pass

            class Date(object):
                pass

            class AttributeDefinition(Mapping):
                pass

            class RestoreSummary(Mapping):
                pass

            class SSEDescription(Mapping):
                pass

            class GlobalSecondaryIndexDescription(Mapping):
                pass
        
            attribute_definitions: List[AttributeDefinition]=None
            table_name: str=None
            key_schema: List[KeySchemaElement]=None
            table_status: str=None
            creation_date_time: Date=None
            provisioned_throughput: ProvisionedThroughputDescription=None
            table_size_bytes: Long=None
            item_count: Long=None
            table_arn: str=None
            table_id: str=None
            billing_mode_summary: BillingModeSummary=None
            local_secondary_indexes: List[LocalSecondaryIndexDescription]=None
            global_secondary_indexes: List[GlobalSecondaryIndexDescription]=None
            stream_specification: StreamSpecification=None
            latest_stream_label: str=None
            latest_stream_arn: str=None
            restore_summary: RestoreSummary=None
            sse_description: SSEDescription=None
            
            def wait_until_exists(self):
                pass

            def wait_until_not_exists(self):
                pass

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def delete_item(self, **kwargs):
                # type: (object) -> dict
                return {}

            def get_item(self, **kwargs):
                # type: (object) -> dict
                return {}

            def put_item(self, **kwargs):
                # type: (object) -> dict
                return {}

            def query(self, **kwargs):
                # type: (object) -> dict
                return {}

            def scan(self, **kwargs):
                # type: (object) -> dict
                return {}

            def update(self, **kwargs):
                # type: (object) -> Table
                return Table()

            def update_item(self, **kwargs):
                # type: (object) -> dict
                return {}


        def batch_get_item(self, **kwargs):
            # type: (object) -> dict
            pass

        def batch_write_item(self, **kwargs):
            # type: (object) -> dict
            pass

        def create_table(self, **kwargs):
            # type: (object) -> Table
            pass

        def get_available_subresources(self) -> List[str]:
            pass

        class TableResourceCollection(List[Table], ResourceCollection):
            pass

        tables: TableResourceCollection = None




class DynamoDBStreams(BaseClient):
    """<fullname>Amazon DynamoDB</fullname> Amazon DynamoDB Streams provides API actions for accessing streams and processing stream records. To learn more about application development with Streams, see <a href="http://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Streams.html">Capturing Table Activity with DynamoDB Streams</a> in the Amazon DynamoDB Developer Guide."""




    class DescribeStreamInput(Mapping):
        pass
    
    class DescribeStreamOutput(Mapping):
        pass
    
    class GetRecordsInput(Mapping):
        pass
    
    class GetRecordsOutput(Mapping):
        pass
    
    class GetShardIteratorInput(Mapping):
        pass
    
    class GetShardIteratorOutput(Mapping):
        pass
    
    class ListStreamsInput(Mapping):
        pass
    
    class ListStreamsOutput(Mapping):
        pass
    
    def describe_stream(self, StreamArn: str, Limit: int=None, ExclusiveStartShardId: str=None) -> DescribeStreamOutput:
        """Returns information about a stream, including the current status of the stream, its Amazon Resource Name (ARN), the composition of its shards, and its corresponding DynamoDB table. <note> You can call <code>DescribeStream</code> at a maximum rate of 10 times per second. </note> Each shard in the stream has a <code>SequenceNumberRange</code> associated with it. If the <code>SequenceNumberRange</code> has a <code>StartingSequenceNumber</code> but no <code>EndingSequenceNumber</code>, then the shard is still open (able to receive more stream records). If both <code>StartingSequenceNumber</code> and <code>EndingSequenceNumber</code> are present, then that shard is closed and can no longer receive more data.

        :param self:
        :param ExclusiveStartShardId:  The shard ID of the first item that this operation will evaluate. Use the value that was returned for <code>LastEvaluatedShardId</code> in the previous operation. <br/>Limit:  The maximum number of shard objects to return. The upper limit is 100.<br/>StreamArn:  The Amazon Resource Name (ARN) for the stream.<br/>
        :return: StreamDescription:  A complete description of the stream, including its creation date and time, the DynamoDB table associated with the stream, the shard IDs within the stream, and the beginning and ending sequence numbers of stream records within the shards.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastEvaluatedShardId</i> The shard ID of the item where the operation stopped, inclusive of the previous result set. Use this value to start a new operation, excluding this value in the new request. If <code>LastEvaluatedShardId</code> is empty, then the "last page" of results has been processed and there is currently no more data to be retrieved. If <code>LastEvaluatedShardId</code> is not empty, it does not necessarily mean that there is more data in the result set. The only way to know when you have reached the end of the result set is when <code>LastEvaluatedShardId</code> is empty.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Shards</i> The shards that comprise the stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeySchema</i> The key attribute(s) of the stream's DynamoDB table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableName</i> The DynamoDB table with which the stream is associated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationRequestDateTime</i> The date and time when the request to create this stream was issued.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamViewType</i> Indicates the format of the records within this stream: <ul> <li>  <code>KEYS_ONLY</code> - only the key attributes of items that were modified in the DynamoDB table. </li> <li>  <code>NEW_IMAGE</code> - entire items from the table, as they appeared after they were modified. </li> <li>  <code>OLD_IMAGE</code> - entire items from the table, as they appeared before they were modified. </li> <li>  <code>NEW_AND_OLD_IMAGES</code> - both the new and the old images of the items from the table. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamStatus</i> Indicates the current status of the stream: <ul> <li>  <code>ENABLING</code> - Streams is currently being enabled on the DynamoDB table. </li> <li>  <code>ENABLED</code> - the stream is enabled. </li> <li>  <code>DISABLING</code> - Streams is currently being disabled on the DynamoDB table. </li> <li>  <code>DISABLED</code> - the stream is disabled. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamLabel</i> A timestamp, in ISO 8601 format, for this stream. Note that <code>LatestStreamLabel</code> is not a unique identifier for the stream, because it is possible that a stream from another table might have the same timestamp. However, the combination of the following three elements is guaranteed to be unique: <ul> <li> the AWS customer ID. </li> <li> the table name </li> <li> the <code>StreamLabel</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamArn</i> The Amazon Resource Name (ARN) for the stream.<br/>        """
        pass

    def get_records(self, ShardIterator: str, Limit: int=None) -> GetRecordsOutput:
        """Retrieves the stream records from a given shard. Specify a shard iterator using the <code>ShardIterator</code> parameter. The shard iterator specifies the position in the shard from which you want to start reading stream records sequentially. If there are no stream records available in the portion of the shard that the iterator points to, <code>GetRecords</code> returns an empty list. Note that it might take multiple calls to get to a portion of the shard that contains stream records. <note>  <code>GetRecords</code> can retrieve a maximum of 1 MB of data or 1000 stream records, whichever comes first. </note>

        :param self:
        :param Limit:  The maximum number of records to return from the shard. The upper limit is 1000.<br/>ShardIterator:  A shard iterator that was retrieved from a previous GetShardIterator operation. This iterator can be used to access the stream records in this shard.<br/>
        :return: NextShardIterator:  The next position in the shard from which to start sequentially reading stream records. If set to <code>null</code>, the shard has been closed and the requested iterator will not return any more data.<br/>Records:  The stream records from the shard, which were retrieved using the shard iterator.<br/>        """
        pass

    def get_shard_iterator(self, ShardIteratorType: str, ShardId: str, StreamArn: str, SequenceNumber: str=None) -> GetShardIteratorOutput:
        """Returns a shard iterator. A shard iterator provides information about how to retrieve the stream records from within a shard. Use the shard iterator in a subsequent <code>GetRecords</code> request to read the stream records from the shard. <note> A shard iterator expires 15 minutes after it is returned to the requester. </note>

        :param self:
        :param SequenceNumber:  The sequence number of a stream record in the shard from which to start reading.<br/>ShardIteratorType:  Determines how the shard iterator is used to start reading stream records from the shard: <ul> <li>  <code>AT_SEQUENCE_NUMBER</code> - Start reading exactly from the position denoted by a specific sequence number. </li> <li>  <code>AFTER_SEQUENCE_NUMBER</code> - Start reading right after the position denoted by a specific sequence number. </li> <li>  <code>TRIM_HORIZON</code> - Start reading at the last (untrimmed) stream record, which is the oldest record in the shard. In DynamoDB Streams, there is a 24 hour limit on data retention. Stream records whose age exceeds this limit are subject to removal (trimming) from the stream. </li> <li>  <code>LATEST</code> - Start reading just after the most recent stream record in the shard, so that you always read the most recent data in the shard. </li> </ul><br/>ShardId:  The identifier of the shard. The iterator will be returned for this shard ID.<br/>StreamArn:  The Amazon Resource Name (ARN) for the stream.<br/>
        :return: ShardIterator:  The position in the shard from which to start reading stream records sequentially. A shard iterator specifies this position using the sequence number of a stream record in a shard.<br/>        """
        pass

    def list_streams(self, TableName: str=None, Limit: int=None, ExclusiveStartStreamArn: str=None) -> ListStreamsOutput:
        """Returns an array of stream ARNs associated with the current account and endpoint. If the <code>TableName</code> parameter is present, then <code>ListStreams</code> will return only the streams ARNs for that table. <note> You can call <code>ListStreams</code> at a maximum rate of 5 times per second. </note>

        :param self:
        :param ExclusiveStartStreamArn:  The ARN (Amazon Resource Name) of the first item that this operation will evaluate. Use the value that was returned for <code>LastEvaluatedStreamArn</code> in the previous operation. <br/>Limit:  The maximum number of streams to return. The upper limit is 100.<br/>TableName:  If this parameter is provided, then only the streams associated with this table name are returned.<br/>
        :return: LastEvaluatedStreamArn:  The stream ARN of the item where the operation stopped, inclusive of the previous result set. Use this value to start a new operation, excluding this value in the new request. If <code>LastEvaluatedStreamArn</code> is empty, then the "last page" of results has been processed and there is no more data to be retrieved. If <code>LastEvaluatedStreamArn</code> is not empty, it does not necessarily mean that there is more data in the result set. The only way to know when you have reached the end of the result set is when <code>LastEvaluatedStreamArn</code> is empty.<br/>Streams:  A list of stream descriptors associated with the current account and endpoint.<br/>        """
        pass



class EC2(BaseClient):
    """<fullname>Amazon Elastic Compute Cloud</fullname> Amazon Elastic Compute Cloud (Amazon EC2) provides secure and resizable computing capacity in the AWS cloud. Using Amazon EC2 eliminates the need to invest in hardware up front, so you can develop and deploy applications faster. To learn more, see the following resources: <ul> <li> Amazon EC2: <a href="http://aws.amazon.com/ec2">Amazon EC2 product page</a>, <a href="http://aws.amazon.com/documentation/ec2">Amazon EC2 documentation</a>  </li> <li> Amazon EBS: <a href="http://aws.amazon.com/ebs">Amazon EBS product page</a>, <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/AmazonEBS.html">Amazon EBS documentation</a>  </li> <li> Amazon VPC: <a href="http://aws.amazon.com/vpc">Amazon VPC product page</a>, <a href="http://aws.amazon.com/documentation/vpc">Amazon VPC documentation</a>  </li> <li> AWS VPN: <a href="http://aws.amazon.com/vpn">AWS VPN product page</a>, <a href="http://aws.amazon.com/documentation/vpn">AWS VPN documentation</a>  </li> </ul>"""

    class BundleTaskCompleteWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_bundle_tasks` for valid parameters"""
            pass
    class ConversionTaskCancelledWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_conversion_tasks` for valid parameters"""
            pass
    class ConversionTaskCompletedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_conversion_tasks` for valid parameters"""
            pass
    class ConversionTaskDeletedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_conversion_tasks` for valid parameters"""
            pass
    class CustomerGatewayAvailableWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_customer_gateways` for valid parameters"""
            pass
    class ExportTaskCancelledWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_export_tasks` for valid parameters"""
            pass
    class ExportTaskCompletedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_export_tasks` for valid parameters"""
            pass
    class ImageAvailableWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_images` for valid parameters"""
            pass
    class ImageExistsWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_images` for valid parameters"""
            pass
    class InstanceExistsWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_instances` for valid parameters"""
            pass
    class InstanceRunningWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_instances` for valid parameters"""
            pass
    class InstanceStatusOkWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_instance_status` for valid parameters"""
            pass
    class InstanceStoppedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_instances` for valid parameters"""
            pass
    class InstanceTerminatedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_instances` for valid parameters"""
            pass
    class KeyPairExistsWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_key_pairs` for valid parameters"""
            pass
    class NatGatewayAvailableWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_nat_gateways` for valid parameters"""
            pass
    class NetworkInterfaceAvailableWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_network_interfaces` for valid parameters"""
            pass
    class PasswordDataAvailableWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `get_password_data` for valid parameters"""
            pass
    class SnapshotCompletedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_snapshots` for valid parameters"""
            pass
    class SpotInstanceRequestFulfilledWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_spot_instance_requests` for valid parameters"""
            pass
    class SubnetAvailableWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_subnets` for valid parameters"""
            pass
    class SystemStatusOkWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_instance_status` for valid parameters"""
            pass
    class VolumeAvailableWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_volumes` for valid parameters"""
            pass
    class VolumeDeletedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_volumes` for valid parameters"""
            pass
    class VolumeInUseWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_volumes` for valid parameters"""
            pass
    class VpcAvailableWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_vpcs` for valid parameters"""
            pass
    class VpcExistsWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_vpcs` for valid parameters"""
            pass
    class VpcPeeringConnectionDeletedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_vpc_peering_connections` for valid parameters"""
            pass
    class VpcPeeringConnectionExistsWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_vpc_peering_connections` for valid parameters"""
            pass
    class VpnConnectionAvailableWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_vpn_connections` for valid parameters"""
            pass
    class VpnConnectionDeletedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_vpn_connections` for valid parameters"""
            pass


    class DescribeRouteTablesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_route_tables` for valid parameters"""
            pass
    class DescribeIamInstanceProfileAssociationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_iam_instance_profile_associations` for valid parameters"""
            pass
    class DescribeInstanceStatusPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_instance_status` for valid parameters"""
            pass
    class DescribeInstancesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_instances` for valid parameters"""
            pass
    class DescribeReservedInstancesOfferingsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_reserved_instances_offerings` for valid parameters"""
            pass
    class DescribeReservedInstancesModificationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_reserved_instances_modifications` for valid parameters"""
            pass
    class DescribeSecurityGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_security_groups` for valid parameters"""
            pass
    class DescribeSnapshotsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_snapshots` for valid parameters"""
            pass
    class DescribeSpotFleetInstancesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_spot_fleet_instances` for valid parameters"""
            pass
    class DescribeSpotFleetRequestsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_spot_fleet_requests` for valid parameters"""
            pass
    class DescribeSpotPriceHistoryPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_spot_price_history` for valid parameters"""
            pass
    class DescribeTagsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_tags` for valid parameters"""
            pass
    class DescribeVolumeStatusPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_volume_status` for valid parameters"""
            pass
    class DescribeVolumesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_volumes` for valid parameters"""
            pass
    class DescribeNatGatewaysPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_nat_gateways` for valid parameters"""
            pass
    class DescribeNetworkInterfacesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_network_interfaces` for valid parameters"""
            pass
    class DescribeVpcEndpointsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_vpc_endpoints` for valid parameters"""
            pass
    class DescribeVpcEndpointServicesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_vpc_endpoint_services` for valid parameters"""
            pass
    class DescribeVpcEndpointConnectionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_vpc_endpoint_connections` for valid parameters"""
            pass
    class DescribeByoipCidrsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_byoip_cidrs` for valid parameters"""
            pass
    class DescribeCapacityReservationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_capacity_reservations` for valid parameters"""
            pass
    class DescribeClassicLinkInstancesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_classic_link_instances` for valid parameters"""
            pass
    class DescribeClientVpnAuthorizationRulesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_client_vpn_authorization_rules` for valid parameters"""
            pass
    class DescribeClientVpnConnectionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_client_vpn_connections` for valid parameters"""
            pass
    class DescribeClientVpnEndpointsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_client_vpn_endpoints` for valid parameters"""
            pass
    class DescribeClientVpnRoutesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_client_vpn_routes` for valid parameters"""
            pass
    class DescribeClientVpnTargetNetworksPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_client_vpn_target_networks` for valid parameters"""
            pass
    class DescribeEgressOnlyInternetGatewaysPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_egress_only_internet_gateways` for valid parameters"""
            pass
    class DescribeFleetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_fleets` for valid parameters"""
            pass
    class DescribeFlowLogsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_flow_logs` for valid parameters"""
            pass
    class DescribeFpgaImagesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_fpga_images` for valid parameters"""
            pass
    class DescribeHostReservationOfferingsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_host_reservation_offerings` for valid parameters"""
            pass
    class DescribeHostReservationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_host_reservations` for valid parameters"""
            pass
    class DescribeHostsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_hosts` for valid parameters"""
            pass
    class DescribeImportImageTasksPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_import_image_tasks` for valid parameters"""
            pass
    class DescribeImportSnapshotTasksPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_import_snapshot_tasks` for valid parameters"""
            pass
    class DescribeInstanceCreditSpecificationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_instance_credit_specifications` for valid parameters"""
            pass
    class DescribeLaunchTemplateVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_launch_template_versions` for valid parameters"""
            pass
    class DescribeLaunchTemplatesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_launch_templates` for valid parameters"""
            pass
    class DescribeMovingAddressesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_moving_addresses` for valid parameters"""
            pass
    class DescribeNetworkInterfacePermissionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_network_interface_permissions` for valid parameters"""
            pass
    class DescribePrefixListsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_prefix_lists` for valid parameters"""
            pass
    class DescribePrincipalIdFormatPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_principal_id_format` for valid parameters"""
            pass
    class DescribePublicIpv4PoolsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_public_ipv_4_pools` for valid parameters"""
            pass
    class DescribeScheduledInstanceAvailabilityPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_scheduled_instance_availability` for valid parameters"""
            pass
    class DescribeScheduledInstancesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_scheduled_instances` for valid parameters"""
            pass
    class DescribeStaleSecurityGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_stale_security_groups` for valid parameters"""
            pass
    class DescribeTransitGatewayAttachmentsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_transit_gateway_attachments` for valid parameters"""
            pass
    class DescribeTransitGatewayRouteTablesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_transit_gateway_route_tables` for valid parameters"""
            pass
    class DescribeTransitGatewayVpcAttachmentsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_transit_gateway_vpc_attachments` for valid parameters"""
            pass
    class DescribeTransitGatewaysPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_transit_gateways` for valid parameters"""
            pass
    class DescribeVolumesModificationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_volumes_modifications` for valid parameters"""
            pass
    class DescribeVpcClassicLinkDnsSupportPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_vpc_classic_link_dns_support` for valid parameters"""
            pass
    class DescribeVpcEndpointConnectionNotificationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_vpc_endpoint_connection_notifications` for valid parameters"""
            pass
    class DescribeVpcEndpointServiceConfigurationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_vpc_endpoint_service_configurations` for valid parameters"""
            pass
    class DescribeVpcEndpointServicePermissionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_vpc_endpoint_service_permissions` for valid parameters"""
            pass
    class DescribeVpcPeeringConnectionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_vpc_peering_connections` for valid parameters"""
            pass
    class GetTransitGatewayAttachmentPropagationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_transit_gateway_attachment_propagations` for valid parameters"""
            pass
    class GetTransitGatewayRouteTableAssociationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_transit_gateway_route_table_associations` for valid parameters"""
            pass
    class GetTransitGatewayRouteTablePropagationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_transit_gateway_route_table_propagations` for valid parameters"""
            pass
    class DescribeInternetGatewaysPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_internet_gateways` for valid parameters"""
            pass
    class DescribeNetworkAclsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_network_acls` for valid parameters"""
            pass
    class DescribeVpcsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_vpcs` for valid parameters"""
            pass
    class DescribeSpotInstanceRequestsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_spot_instance_requests` for valid parameters"""
            pass
    class DescribeDhcpOptionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_dhcp_options` for valid parameters"""
            pass
    class DescribeSubnetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_subnets` for valid parameters"""
            pass
    class DescribeTrafficMirrorFiltersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_traffic_mirror_filters` for valid parameters"""
            pass
    class DescribeTrafficMirrorSessionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_traffic_mirror_sessions` for valid parameters"""
            pass
    class DescribeTrafficMirrorTargetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_traffic_mirror_targets` for valid parameters"""
            pass


    class AcceptReservedInstancesExchangeQuoteRequest(Mapping):
        pass
    
    class AcceptReservedInstancesExchangeQuoteResult(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class String(object):
        pass
    
    class TargetConfigurationRequest(Mapping):
        pass
    
    class AcceptTransitGatewayVpcAttachmentRequest(Mapping):
        pass
    
    class AcceptTransitGatewayVpcAttachmentResult(Mapping):
        pass
    
    class AcceptVpcEndpointConnectionsRequest(Mapping):
        pass
    
    class AcceptVpcEndpointConnectionsResult(Mapping):
        pass
    
    class AcceptVpcPeeringConnectionRequest(Mapping):
        pass
    
    class AcceptVpcPeeringConnectionResult(Mapping):
        pass
    
    class AdvertiseByoipCidrRequest(Mapping):
        pass
    
    class AdvertiseByoipCidrResult(Mapping):
        pass
    
    class AllocateAddressRequest(Mapping):
        pass
    
    class AllocateAddressResult(Mapping):
        pass
    
    class AllocateHostsRequest(Mapping):
        pass
    
    class AllocateHostsResult(Mapping):
        pass
    
    class TagSpecification(Mapping):
        pass
    
    class ApplySecurityGroupsToClientVpnTargetNetworkRequest(Mapping):
        pass
    
    class ApplySecurityGroupsToClientVpnTargetNetworkResult(Mapping):
        pass
    
    class AssignIpv6AddressesRequest(Mapping):
        pass
    
    class AssignIpv6AddressesResult(Mapping):
        pass
    
    class AssignPrivateIpAddressesRequest(Mapping):
        pass
    
    class AssignPrivateIpAddressesResult(Mapping):
        pass
    
    class AssociateAddressRequest(Mapping):
        pass
    
    class AssociateAddressResult(Mapping):
        pass
    
    class AssociateClientVpnTargetNetworkRequest(Mapping):
        pass
    
    class AssociateClientVpnTargetNetworkResult(Mapping):
        pass
    
    class AssociateDhcpOptionsRequest(Mapping):
        pass
    
    class AssociateIamInstanceProfileRequest(Mapping):
        pass
    
    class AssociateIamInstanceProfileResult(Mapping):
        pass
    
    class IamInstanceProfileSpecification(Mapping):
        pass
    
    class AssociateRouteTableRequest(Mapping):
        pass
    
    class AssociateRouteTableResult(Mapping):
        pass
    
    class AssociateSubnetCidrBlockRequest(Mapping):
        pass
    
    class AssociateSubnetCidrBlockResult(Mapping):
        pass
    
    class AssociateTransitGatewayRouteTableRequest(Mapping):
        pass
    
    class AssociateTransitGatewayRouteTableResult(Mapping):
        pass
    
    class AssociateVpcCidrBlockRequest(Mapping):
        pass
    
    class AssociateVpcCidrBlockResult(Mapping):
        pass
    
    class AttachClassicLinkVpcRequest(Mapping):
        pass
    
    class AttachClassicLinkVpcResult(Mapping):
        pass
    
    class AttachInternetGatewayRequest(Mapping):
        pass
    
    class AttachNetworkInterfaceRequest(Mapping):
        pass
    
    class AttachNetworkInterfaceResult(Mapping):
        pass
    
    class AttachVolumeRequest(Mapping):
        pass
    
    class VolumeAttachment(Mapping):
        pass
    
    class AttachVpnGatewayRequest(Mapping):
        pass
    
    class AttachVpnGatewayResult(Mapping):
        pass
    
    class AuthorizeClientVpnIngressRequest(Mapping):
        pass
    
    class AuthorizeClientVpnIngressResult(Mapping):
        pass
    
    class AuthorizeSecurityGroupEgressRequest(Mapping):
        pass
    
    class IpPermission(Mapping):
        pass
    
    class AuthorizeSecurityGroupIngressRequest(Mapping):
        pass
    
    class BundleInstanceRequest(Mapping):
        pass
    
    class BundleInstanceResult(Mapping):
        pass
    
    class Storage(Mapping):
        pass
    
    class CancelBundleTaskRequest(Mapping):
        pass
    
    class CancelBundleTaskResult(Mapping):
        pass
    
    class CancelCapacityReservationRequest(Mapping):
        pass
    
    class CancelCapacityReservationResult(Mapping):
        pass
    
    class CancelConversionRequest(Mapping):
        pass
    
    class CancelExportTaskRequest(Mapping):
        pass
    
    class CancelImportTaskRequest(Mapping):
        pass
    
    class CancelImportTaskResult(Mapping):
        pass
    
    class CancelReservedInstancesListingRequest(Mapping):
        pass
    
    class CancelReservedInstancesListingResult(Mapping):
        pass
    
    class CancelSpotFleetRequestsRequest(Mapping):
        pass
    
    class CancelSpotFleetRequestsResponse(Mapping):
        pass
    
    class CancelSpotInstanceRequestsRequest(Mapping):
        pass
    
    class CancelSpotInstanceRequestsResult(Mapping):
        pass
    
    class ConfirmProductInstanceRequest(Mapping):
        pass
    
    class ConfirmProductInstanceResult(Mapping):
        pass
    
    class CopyFpgaImageRequest(Mapping):
        pass
    
    class CopyFpgaImageResult(Mapping):
        pass
    
    class CopyImageRequest(Mapping):
        pass
    
    class CopyImageResult(Mapping):
        pass
    
    class CopySnapshotRequest(Mapping):
        pass
    
    class CopySnapshotResult(Mapping):
        pass
    
    class CreateCapacityReservationRequest(Mapping):
        pass
    
    class CreateCapacityReservationResult(Mapping):
        pass
    
    class DateTime(object):
        pass
    
    class CreateClientVpnEndpointRequest(Mapping):
        pass
    
    class CreateClientVpnEndpointResult(Mapping):
        pass
    
    class ClientVpnAuthenticationRequest(Mapping):
        pass
    
    class ConnectionLogOptions(Mapping):
        pass
    
    class CreateClientVpnRouteRequest(Mapping):
        pass
    
    class CreateClientVpnRouteResult(Mapping):
        pass
    
    class CreateCustomerGatewayRequest(Mapping):
        pass
    
    class CreateCustomerGatewayResult(Mapping):
        pass
    
    class CreateDefaultSubnetRequest(Mapping):
        pass
    
    class CreateDefaultSubnetResult(Mapping):
        pass
    
    class CreateDefaultVpcRequest(Mapping):
        pass
    
    class CreateDefaultVpcResult(Mapping):
        pass
    
    class CreateDhcpOptionsRequest(Mapping):
        pass
    
    class CreateDhcpOptionsResult(Mapping):
        pass
    
    class NewDhcpConfiguration(Mapping):
        pass
    
    class CreateEgressOnlyInternetGatewayRequest(Mapping):
        pass
    
    class CreateEgressOnlyInternetGatewayResult(Mapping):
        pass
    
    class CreateFleetRequest(Mapping):
        pass
    
    class CreateFleetResult(Mapping):
        pass
    
    class SpotOptionsRequest(Mapping):
        pass
    
    class OnDemandOptionsRequest(Mapping):
        pass
    
    class FleetLaunchTemplateConfigRequest(Mapping):
        pass
    
    class TargetCapacitySpecificationRequest(Mapping):
        pass
    
    class CreateFlowLogsRequest(Mapping):
        pass
    
    class CreateFlowLogsResult(Mapping):
        pass
    
    class CreateFpgaImageRequest(Mapping):
        pass
    
    class CreateFpgaImageResult(Mapping):
        pass
    
    class StorageLocation(Mapping):
        pass
    
    class CreateImageRequest(Mapping):
        pass
    
    class CreateImageResult(Mapping):
        pass
    
    class BlockDeviceMapping(Mapping):
        pass
    
    class CreateInstanceExportTaskRequest(Mapping):
        pass
    
    class CreateInstanceExportTaskResult(Mapping):
        pass
    
    class ExportToS3TaskSpecification(Mapping):
        pass
    
    class CreateInternetGatewayRequest(Mapping):
        pass
    
    class CreateInternetGatewayResult(Mapping):
        pass
    
    class CreateKeyPairRequest(Mapping):
        pass
    
    class KeyPair(Mapping):
        pass
    
    class CreateLaunchTemplateRequest(Mapping):
        pass
    
    class CreateLaunchTemplateResult(Mapping):
        pass
    
    class RequestLaunchTemplateData(Mapping):
        pass
    
    class CreateLaunchTemplateVersionRequest(Mapping):
        pass
    
    class CreateLaunchTemplateVersionResult(Mapping):
        pass
    
    class CreateNatGatewayRequest(Mapping):
        pass
    
    class CreateNatGatewayResult(Mapping):
        pass
    
    class CreateNetworkAclRequest(Mapping):
        pass
    
    class CreateNetworkAclResult(Mapping):
        pass
    
    class CreateNetworkAclEntryRequest(Mapping):
        pass
    
    class IcmpTypeCode(Mapping):
        pass
    
    class PortRange(Mapping):
        pass
    
    class CreateNetworkInterfaceRequest(Mapping):
        pass
    
    class CreateNetworkInterfaceResult(Mapping):
        pass
    
    class InstanceIpv6Address(Mapping):
        pass
    
    class PrivateIpAddressSpecification(Mapping):
        pass
    
    class CreateNetworkInterfacePermissionRequest(Mapping):
        pass
    
    class CreateNetworkInterfacePermissionResult(Mapping):
        pass
    
    class CreatePlacementGroupRequest(Mapping):
        pass
    
    class CreateReservedInstancesListingRequest(Mapping):
        pass
    
    class CreateReservedInstancesListingResult(Mapping):
        pass
    
    class PriceScheduleSpecification(Mapping):
        pass
    
    class CreateRouteRequest(Mapping):
        pass
    
    class CreateRouteResult(Mapping):
        pass
    
    class CreateRouteTableRequest(Mapping):
        pass
    
    class CreateRouteTableResult(Mapping):
        pass
    
    class CreateSecurityGroupRequest(Mapping):
        pass
    
    class CreateSecurityGroupResult(Mapping):
        pass
    
    class CreateSnapshotRequest(Mapping):
        pass
    
    class Snapshot(Mapping):
        pass
    
    class CreateSnapshotsRequest(Mapping):
        pass
    
    class CreateSnapshotsResult(Mapping):
        pass
    
    class InstanceSpecification(Mapping):
        pass
    
    class CreateSpotDatafeedSubscriptionRequest(Mapping):
        pass
    
    class CreateSpotDatafeedSubscriptionResult(Mapping):
        pass
    
    class CreateSubnetRequest(Mapping):
        pass
    
    class CreateSubnetResult(Mapping):
        pass
    
    class CreateTagsRequest(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreateTrafficMirrorFilterRequest(Mapping):
        pass
    
    class CreateTrafficMirrorFilterResult(Mapping):
        pass
    
    class CreateTrafficMirrorFilterRuleRequest(Mapping):
        pass
    
    class CreateTrafficMirrorFilterRuleResult(Mapping):
        pass
    
    class TrafficMirrorPortRangeRequest(Mapping):
        pass
    
    class CreateTrafficMirrorSessionRequest(Mapping):
        pass
    
    class CreateTrafficMirrorSessionResult(Mapping):
        pass
    
    class CreateTrafficMirrorTargetRequest(Mapping):
        pass
    
    class CreateTrafficMirrorTargetResult(Mapping):
        pass
    
    class CreateTransitGatewayRequest(Mapping):
        pass
    
    class CreateTransitGatewayResult(Mapping):
        pass
    
    class TransitGatewayRequestOptions(Mapping):
        pass
    
    class CreateTransitGatewayRouteRequest(Mapping):
        pass
    
    class CreateTransitGatewayRouteResult(Mapping):
        pass
    
    class CreateTransitGatewayRouteTableRequest(Mapping):
        pass
    
    class CreateTransitGatewayRouteTableResult(Mapping):
        pass
    
    class CreateTransitGatewayVpcAttachmentRequest(Mapping):
        pass
    
    class CreateTransitGatewayVpcAttachmentResult(Mapping):
        pass
    
    class CreateTransitGatewayVpcAttachmentRequestOptions(Mapping):
        pass
    
    class CreateVolumeRequest(Mapping):
        pass
    
    class Volume(Mapping):
        pass
    
    class CreateVpcRequest(Mapping):
        pass
    
    class CreateVpcResult(Mapping):
        pass
    
    class CreateVpcEndpointRequest(Mapping):
        pass
    
    class CreateVpcEndpointResult(Mapping):
        pass
    
    class CreateVpcEndpointConnectionNotificationRequest(Mapping):
        pass
    
    class CreateVpcEndpointConnectionNotificationResult(Mapping):
        pass
    
    class CreateVpcEndpointServiceConfigurationRequest(Mapping):
        pass
    
    class CreateVpcEndpointServiceConfigurationResult(Mapping):
        pass
    
    class CreateVpcPeeringConnectionRequest(Mapping):
        pass
    
    class CreateVpcPeeringConnectionResult(Mapping):
        pass
    
    class CreateVpnConnectionRequest(Mapping):
        pass
    
    class CreateVpnConnectionResult(Mapping):
        pass
    
    class VpnConnectionOptionsSpecification(Mapping):
        pass
    
    class CreateVpnConnectionRouteRequest(Mapping):
        pass
    
    class CreateVpnGatewayRequest(Mapping):
        pass
    
    class CreateVpnGatewayResult(Mapping):
        pass
    
    class Long(object):
        pass
    
    class DeleteClientVpnEndpointRequest(Mapping):
        pass
    
    class DeleteClientVpnEndpointResult(Mapping):
        pass
    
    class DeleteClientVpnRouteRequest(Mapping):
        pass
    
    class DeleteClientVpnRouteResult(Mapping):
        pass
    
    class DeleteCustomerGatewayRequest(Mapping):
        pass
    
    class DeleteDhcpOptionsRequest(Mapping):
        pass
    
    class DeleteEgressOnlyInternetGatewayRequest(Mapping):
        pass
    
    class DeleteEgressOnlyInternetGatewayResult(Mapping):
        pass
    
    class DeleteFleetsRequest(Mapping):
        pass
    
    class DeleteFleetsResult(Mapping):
        pass
    
    class FleetIdentifier(object):
        pass
    
    class DeleteFlowLogsRequest(Mapping):
        pass
    
    class DeleteFlowLogsResult(Mapping):
        pass
    
    class DeleteFpgaImageRequest(Mapping):
        pass
    
    class DeleteFpgaImageResult(Mapping):
        pass
    
    class DeleteInternetGatewayRequest(Mapping):
        pass
    
    class DeleteKeyPairRequest(Mapping):
        pass
    
    class DeleteLaunchTemplateRequest(Mapping):
        pass
    
    class DeleteLaunchTemplateResult(Mapping):
        pass
    
    class DeleteLaunchTemplateVersionsRequest(Mapping):
        pass
    
    class DeleteLaunchTemplateVersionsResult(Mapping):
        pass
    
    class DeleteNatGatewayRequest(Mapping):
        pass
    
    class DeleteNatGatewayResult(Mapping):
        pass
    
    class DeleteNetworkAclRequest(Mapping):
        pass
    
    class DeleteNetworkAclEntryRequest(Mapping):
        pass
    
    class DeleteNetworkInterfaceRequest(Mapping):
        pass
    
    class DeleteNetworkInterfacePermissionRequest(Mapping):
        pass
    
    class DeleteNetworkInterfacePermissionResult(Mapping):
        pass
    
    class DeletePlacementGroupRequest(Mapping):
        pass
    
    class DeleteRouteRequest(Mapping):
        pass
    
    class DeleteRouteTableRequest(Mapping):
        pass
    
    class DeleteSecurityGroupRequest(Mapping):
        pass
    
    class DeleteSnapshotRequest(Mapping):
        pass
    
    class DeleteSpotDatafeedSubscriptionRequest(Mapping):
        pass
    
    class DeleteSubnetRequest(Mapping):
        pass
    
    class DeleteTagsRequest(Mapping):
        pass
    
    class DeleteTrafficMirrorFilterRequest(Mapping):
        pass
    
    class DeleteTrafficMirrorFilterResult(Mapping):
        pass
    
    class DeleteTrafficMirrorFilterRuleRequest(Mapping):
        pass
    
    class DeleteTrafficMirrorFilterRuleResult(Mapping):
        pass
    
    class DeleteTrafficMirrorSessionRequest(Mapping):
        pass
    
    class DeleteTrafficMirrorSessionResult(Mapping):
        pass
    
    class DeleteTrafficMirrorTargetRequest(Mapping):
        pass
    
    class DeleteTrafficMirrorTargetResult(Mapping):
        pass
    
    class DeleteTransitGatewayRequest(Mapping):
        pass
    
    class DeleteTransitGatewayResult(Mapping):
        pass
    
    class DeleteTransitGatewayRouteRequest(Mapping):
        pass
    
    class DeleteTransitGatewayRouteResult(Mapping):
        pass
    
    class DeleteTransitGatewayRouteTableRequest(Mapping):
        pass
    
    class DeleteTransitGatewayRouteTableResult(Mapping):
        pass
    
    class DeleteTransitGatewayVpcAttachmentRequest(Mapping):
        pass
    
    class DeleteTransitGatewayVpcAttachmentResult(Mapping):
        pass
    
    class DeleteVolumeRequest(Mapping):
        pass
    
    class DeleteVpcRequest(Mapping):
        pass
    
    class DeleteVpcEndpointConnectionNotificationsRequest(Mapping):
        pass
    
    class DeleteVpcEndpointConnectionNotificationsResult(Mapping):
        pass
    
    class DeleteVpcEndpointServiceConfigurationsRequest(Mapping):
        pass
    
    class DeleteVpcEndpointServiceConfigurationsResult(Mapping):
        pass
    
    class DeleteVpcEndpointsRequest(Mapping):
        pass
    
    class DeleteVpcEndpointsResult(Mapping):
        pass
    
    class DeleteVpcPeeringConnectionRequest(Mapping):
        pass
    
    class DeleteVpcPeeringConnectionResult(Mapping):
        pass
    
    class DeleteVpnConnectionRequest(Mapping):
        pass
    
    class DeleteVpnConnectionRouteRequest(Mapping):
        pass
    
    class DeleteVpnGatewayRequest(Mapping):
        pass
    
    class DeprovisionByoipCidrRequest(Mapping):
        pass
    
    class DeprovisionByoipCidrResult(Mapping):
        pass
    
    class DeregisterImageRequest(Mapping):
        pass
    
    class DescribeAccountAttributesRequest(Mapping):
        pass
    
    class DescribeAccountAttributesResult(Mapping):
        pass
    
    class AccountAttributeName(object):
        pass
    
    class DescribeAddressesRequest(Mapping):
        pass
    
    class DescribeAddressesResult(Mapping):
        pass
    
    class Filter(Mapping):
        pass
    
    class DescribeAggregateIdFormatRequest(Mapping):
        pass
    
    class DescribeAggregateIdFormatResult(Mapping):
        pass
    
    class DescribeAvailabilityZonesRequest(Mapping):
        pass
    
    class DescribeAvailabilityZonesResult(Mapping):
        pass
    
    class DescribeBundleTasksRequest(Mapping):
        pass
    
    class DescribeBundleTasksResult(Mapping):
        pass
    
    class DescribeByoipCidrsRequest(Mapping):
        pass
    
    class DescribeByoipCidrsResult(Mapping):
        pass
    
    class DescribeCapacityReservationsRequest(Mapping):
        pass
    
    class DescribeCapacityReservationsResult(Mapping):
        pass
    
    class DescribeClassicLinkInstancesRequest(Mapping):
        pass
    
    class DescribeClassicLinkInstancesResult(Mapping):
        pass
    
    class DescribeClientVpnAuthorizationRulesRequest(Mapping):
        pass
    
    class DescribeClientVpnAuthorizationRulesResult(Mapping):
        pass
    
    class DescribeClientVpnConnectionsRequest(Mapping):
        pass
    
    class DescribeClientVpnConnectionsResult(Mapping):
        pass
    
    class DescribeClientVpnEndpointsRequest(Mapping):
        pass
    
    class DescribeClientVpnEndpointsResult(Mapping):
        pass
    
    class DescribeClientVpnRoutesRequest(Mapping):
        pass
    
    class DescribeClientVpnRoutesResult(Mapping):
        pass
    
    class DescribeClientVpnTargetNetworksRequest(Mapping):
        pass
    
    class DescribeClientVpnTargetNetworksResult(Mapping):
        pass
    
    class DescribeConversionTasksRequest(Mapping):
        pass
    
    class DescribeConversionTasksResult(Mapping):
        pass
    
    class DescribeCustomerGatewaysRequest(Mapping):
        pass
    
    class DescribeCustomerGatewaysResult(Mapping):
        pass
    
    class DescribeDhcpOptionsRequest(Mapping):
        pass
    
    class DescribeDhcpOptionsResult(Mapping):
        pass
    
    class DescribeEgressOnlyInternetGatewaysRequest(Mapping):
        pass
    
    class DescribeEgressOnlyInternetGatewaysResult(Mapping):
        pass
    
    class EgressOnlyInternetGatewayId(object):
        pass
    
    class DescribeElasticGpusRequest(Mapping):
        pass
    
    class DescribeElasticGpusResult(Mapping):
        pass
    
    class DescribeExportTasksRequest(Mapping):
        pass
    
    class DescribeExportTasksResult(Mapping):
        pass
    
    class DescribeFleetHistoryRequest(Mapping):
        pass
    
    class DescribeFleetHistoryResult(Mapping):
        pass
    
    class DescribeFleetInstancesRequest(Mapping):
        pass
    
    class DescribeFleetInstancesResult(Mapping):
        pass
    
    class DescribeFleetsRequest(Mapping):
        pass
    
    class DescribeFleetsResult(Mapping):
        pass
    
    class DescribeFlowLogsRequest(Mapping):
        pass
    
    class DescribeFlowLogsResult(Mapping):
        pass
    
    class DescribeFpgaImageAttributeRequest(Mapping):
        pass
    
    class DescribeFpgaImageAttributeResult(Mapping):
        pass
    
    class DescribeFpgaImagesRequest(Mapping):
        pass
    
    class DescribeFpgaImagesResult(Mapping):
        pass
    
    class DescribeHostReservationOfferingsRequest(Mapping):
        pass
    
    class DescribeHostReservationOfferingsResult(Mapping):
        pass
    
    class DescribeHostReservationsRequest(Mapping):
        pass
    
    class DescribeHostReservationsResult(Mapping):
        pass
    
    class DescribeHostsRequest(Mapping):
        pass
    
    class DescribeHostsResult(Mapping):
        pass
    
    class DescribeIamInstanceProfileAssociationsRequest(Mapping):
        pass
    
    class DescribeIamInstanceProfileAssociationsResult(Mapping):
        pass
    
    class DescribeIdFormatRequest(Mapping):
        pass
    
    class DescribeIdFormatResult(Mapping):
        pass
    
    class DescribeIdentityIdFormatRequest(Mapping):
        pass
    
    class DescribeIdentityIdFormatResult(Mapping):
        pass
    
    class DescribeImageAttributeRequest(Mapping):
        pass
    
    class ImageAttribute(Mapping):
        pass
    
    class DescribeImagesRequest(Mapping):
        pass
    
    class DescribeImagesResult(Mapping):
        pass
    
    class DescribeImportImageTasksRequest(Mapping):
        pass
    
    class DescribeImportImageTasksResult(Mapping):
        pass
    
    class DescribeImportSnapshotTasksRequest(Mapping):
        pass
    
    class DescribeImportSnapshotTasksResult(Mapping):
        pass
    
    class DescribeInstanceAttributeRequest(Mapping):
        pass
    
    class InstanceAttribute(Mapping):
        pass
    
    class DescribeInstanceCreditSpecificationsRequest(Mapping):
        pass
    
    class DescribeInstanceCreditSpecificationsResult(Mapping):
        pass
    
    class DescribeInstanceStatusRequest(Mapping):
        pass
    
    class DescribeInstanceStatusResult(Mapping):
        pass
    
    class DescribeInstancesRequest(Mapping):
        pass
    
    class DescribeInstancesResult(Mapping):
        pass
    
    class DescribeInternetGatewaysRequest(Mapping):
        pass
    
    class DescribeInternetGatewaysResult(Mapping):
        pass
    
    class DescribeKeyPairsRequest(Mapping):
        pass
    
    class DescribeKeyPairsResult(Mapping):
        pass
    
    class DescribeLaunchTemplateVersionsRequest(Mapping):
        pass
    
    class DescribeLaunchTemplateVersionsResult(Mapping):
        pass
    
    class DescribeLaunchTemplatesRequest(Mapping):
        pass
    
    class DescribeLaunchTemplatesResult(Mapping):
        pass
    
    class LaunchTemplateName(object):
        pass
    
    class DescribeMovingAddressesRequest(Mapping):
        pass
    
    class DescribeMovingAddressesResult(Mapping):
        pass
    
    class DescribeNatGatewaysRequest(Mapping):
        pass
    
    class DescribeNatGatewaysResult(Mapping):
        pass
    
    class DescribeNetworkAclsRequest(Mapping):
        pass
    
    class DescribeNetworkAclsResult(Mapping):
        pass
    
    class DescribeNetworkInterfaceAttributeRequest(Mapping):
        pass
    
    class DescribeNetworkInterfaceAttributeResult(Mapping):
        pass
    
    class DescribeNetworkInterfacePermissionsRequest(Mapping):
        pass
    
    class DescribeNetworkInterfacePermissionsResult(Mapping):
        pass
    
    class DescribeNetworkInterfacesRequest(Mapping):
        pass
    
    class DescribeNetworkInterfacesResult(Mapping):
        pass
    
    class DescribePlacementGroupsRequest(Mapping):
        pass
    
    class DescribePlacementGroupsResult(Mapping):
        pass
    
    class DescribePrefixListsRequest(Mapping):
        pass
    
    class DescribePrefixListsResult(Mapping):
        pass
    
    class DescribePrincipalIdFormatRequest(Mapping):
        pass
    
    class DescribePrincipalIdFormatResult(Mapping):
        pass
    
    class DescribePublicIpv4PoolsRequest(Mapping):
        pass
    
    class DescribePublicIpv4PoolsResult(Mapping):
        pass
    
    class DescribeRegionsRequest(Mapping):
        pass
    
    class DescribeRegionsResult(Mapping):
        pass
    
    class DescribeReservedInstancesRequest(Mapping):
        pass
    
    class DescribeReservedInstancesResult(Mapping):
        pass
    
    class DescribeReservedInstancesListingsRequest(Mapping):
        pass
    
    class DescribeReservedInstancesListingsResult(Mapping):
        pass
    
    class DescribeReservedInstancesModificationsRequest(Mapping):
        pass
    
    class DescribeReservedInstancesModificationsResult(Mapping):
        pass
    
    class DescribeReservedInstancesOfferingsRequest(Mapping):
        pass
    
    class DescribeReservedInstancesOfferingsResult(Mapping):
        pass
    
    class DescribeRouteTablesRequest(Mapping):
        pass
    
    class DescribeRouteTablesResult(Mapping):
        pass
    
    class DescribeScheduledInstanceAvailabilityRequest(Mapping):
        pass
    
    class DescribeScheduledInstanceAvailabilityResult(Mapping):
        pass
    
    class SlotDateTimeRangeRequest(Mapping):
        pass
    
    class ScheduledInstanceRecurrenceRequest(Mapping):
        pass
    
    class DescribeScheduledInstancesRequest(Mapping):
        pass
    
    class DescribeScheduledInstancesResult(Mapping):
        pass
    
    class SlotStartTimeRangeRequest(Mapping):
        pass
    
    class DescribeSecurityGroupReferencesRequest(Mapping):
        pass
    
    class DescribeSecurityGroupReferencesResult(Mapping):
        pass
    
    class DescribeSecurityGroupsRequest(Mapping):
        pass
    
    class DescribeSecurityGroupsResult(Mapping):
        pass
    
    class DescribeSnapshotAttributeRequest(Mapping):
        pass
    
    class DescribeSnapshotAttributeResult(Mapping):
        pass
    
    class DescribeSnapshotsRequest(Mapping):
        pass
    
    class DescribeSnapshotsResult(Mapping):
        pass
    
    class DescribeSpotDatafeedSubscriptionRequest(Mapping):
        pass
    
    class DescribeSpotDatafeedSubscriptionResult(Mapping):
        pass
    
    class DescribeSpotFleetInstancesRequest(Mapping):
        pass
    
    class DescribeSpotFleetInstancesResponse(Mapping):
        pass
    
    class DescribeSpotFleetRequestHistoryRequest(Mapping):
        pass
    
    class DescribeSpotFleetRequestHistoryResponse(Mapping):
        pass
    
    class DescribeSpotFleetRequestsRequest(Mapping):
        pass
    
    class DescribeSpotFleetRequestsResponse(Mapping):
        pass
    
    class DescribeSpotInstanceRequestsRequest(Mapping):
        pass
    
    class DescribeSpotInstanceRequestsResult(Mapping):
        pass
    
    class DescribeSpotPriceHistoryRequest(Mapping):
        pass
    
    class DescribeSpotPriceHistoryResult(Mapping):
        pass
    
    class InstanceType(object):
        pass
    
    class DescribeStaleSecurityGroupsRequest(Mapping):
        pass
    
    class DescribeStaleSecurityGroupsResult(Mapping):
        pass
    
    class DescribeSubnetsRequest(Mapping):
        pass
    
    class DescribeSubnetsResult(Mapping):
        pass
    
    class DescribeTagsRequest(Mapping):
        pass
    
    class DescribeTagsResult(Mapping):
        pass
    
    class DescribeTrafficMirrorFiltersRequest(Mapping):
        pass
    
    class DescribeTrafficMirrorFiltersResult(Mapping):
        pass
    
    class DescribeTrafficMirrorSessionsRequest(Mapping):
        pass
    
    class DescribeTrafficMirrorSessionsResult(Mapping):
        pass
    
    class DescribeTrafficMirrorTargetsRequest(Mapping):
        pass
    
    class DescribeTrafficMirrorTargetsResult(Mapping):
        pass
    
    class DescribeTransitGatewayAttachmentsRequest(Mapping):
        pass
    
    class DescribeTransitGatewayAttachmentsResult(Mapping):
        pass
    
    class DescribeTransitGatewayRouteTablesRequest(Mapping):
        pass
    
    class DescribeTransitGatewayRouteTablesResult(Mapping):
        pass
    
    class DescribeTransitGatewayVpcAttachmentsRequest(Mapping):
        pass
    
    class DescribeTransitGatewayVpcAttachmentsResult(Mapping):
        pass
    
    class DescribeTransitGatewaysRequest(Mapping):
        pass
    
    class DescribeTransitGatewaysResult(Mapping):
        pass
    
    class DescribeVolumeAttributeRequest(Mapping):
        pass
    
    class DescribeVolumeAttributeResult(Mapping):
        pass
    
    class DescribeVolumeStatusRequest(Mapping):
        pass
    
    class DescribeVolumeStatusResult(Mapping):
        pass
    
    class DescribeVolumesRequest(Mapping):
        pass
    
    class DescribeVolumesResult(Mapping):
        pass
    
    class DescribeVolumesModificationsRequest(Mapping):
        pass
    
    class DescribeVolumesModificationsResult(Mapping):
        pass
    
    class DescribeVpcAttributeRequest(Mapping):
        pass
    
    class DescribeVpcAttributeResult(Mapping):
        pass
    
    class DescribeVpcClassicLinkRequest(Mapping):
        pass
    
    class DescribeVpcClassicLinkResult(Mapping):
        pass
    
    class DescribeVpcClassicLinkDnsSupportRequest(Mapping):
        pass
    
    class DescribeVpcClassicLinkDnsSupportResult(Mapping):
        pass
    
    class DescribeVpcEndpointConnectionNotificationsRequest(Mapping):
        pass
    
    class DescribeVpcEndpointConnectionNotificationsResult(Mapping):
        pass
    
    class DescribeVpcEndpointConnectionsRequest(Mapping):
        pass
    
    class DescribeVpcEndpointConnectionsResult(Mapping):
        pass
    
    class DescribeVpcEndpointServiceConfigurationsRequest(Mapping):
        pass
    
    class DescribeVpcEndpointServiceConfigurationsResult(Mapping):
        pass
    
    class DescribeVpcEndpointServicePermissionsRequest(Mapping):
        pass
    
    class DescribeVpcEndpointServicePermissionsResult(Mapping):
        pass
    
    class DescribeVpcEndpointServicesRequest(Mapping):
        pass
    
    class DescribeVpcEndpointServicesResult(Mapping):
        pass
    
    class DescribeVpcEndpointsRequest(Mapping):
        pass
    
    class DescribeVpcEndpointsResult(Mapping):
        pass
    
    class DescribeVpcPeeringConnectionsRequest(Mapping):
        pass
    
    class DescribeVpcPeeringConnectionsResult(Mapping):
        pass
    
    class DescribeVpcsRequest(Mapping):
        pass
    
    class DescribeVpcsResult(Mapping):
        pass
    
    class DescribeVpnConnectionsRequest(Mapping):
        pass
    
    class DescribeVpnConnectionsResult(Mapping):
        pass
    
    class DescribeVpnGatewaysRequest(Mapping):
        pass
    
    class DescribeVpnGatewaysResult(Mapping):
        pass
    
    class DetachClassicLinkVpcRequest(Mapping):
        pass
    
    class DetachClassicLinkVpcResult(Mapping):
        pass
    
    class DetachInternetGatewayRequest(Mapping):
        pass
    
    class DetachNetworkInterfaceRequest(Mapping):
        pass
    
    class DetachVolumeRequest(Mapping):
        pass
    
    class DetachVpnGatewayRequest(Mapping):
        pass
    
    class DisableEbsEncryptionByDefaultRequest(Mapping):
        pass
    
    class DisableEbsEncryptionByDefaultResult(Mapping):
        pass
    
    class DisableTransitGatewayRouteTablePropagationRequest(Mapping):
        pass
    
    class DisableTransitGatewayRouteTablePropagationResult(Mapping):
        pass
    
    class DisableVgwRoutePropagationRequest(Mapping):
        pass
    
    class DisableVpcClassicLinkRequest(Mapping):
        pass
    
    class DisableVpcClassicLinkResult(Mapping):
        pass
    
    class DisableVpcClassicLinkDnsSupportRequest(Mapping):
        pass
    
    class DisableVpcClassicLinkDnsSupportResult(Mapping):
        pass
    
    class DisassociateAddressRequest(Mapping):
        pass
    
    class DisassociateClientVpnTargetNetworkRequest(Mapping):
        pass
    
    class DisassociateClientVpnTargetNetworkResult(Mapping):
        pass
    
    class DisassociateIamInstanceProfileRequest(Mapping):
        pass
    
    class DisassociateIamInstanceProfileResult(Mapping):
        pass
    
    class DisassociateRouteTableRequest(Mapping):
        pass
    
    class DisassociateSubnetCidrBlockRequest(Mapping):
        pass
    
    class DisassociateSubnetCidrBlockResult(Mapping):
        pass
    
    class DisassociateTransitGatewayRouteTableRequest(Mapping):
        pass
    
    class DisassociateTransitGatewayRouteTableResult(Mapping):
        pass
    
    class DisassociateVpcCidrBlockRequest(Mapping):
        pass
    
    class DisassociateVpcCidrBlockResult(Mapping):
        pass
    
    class EnableEbsEncryptionByDefaultRequest(Mapping):
        pass
    
    class EnableEbsEncryptionByDefaultResult(Mapping):
        pass
    
    class EnableTransitGatewayRouteTablePropagationRequest(Mapping):
        pass
    
    class EnableTransitGatewayRouteTablePropagationResult(Mapping):
        pass
    
    class EnableVgwRoutePropagationRequest(Mapping):
        pass
    
    class EnableVolumeIORequest(Mapping):
        pass
    
    class EnableVpcClassicLinkRequest(Mapping):
        pass
    
    class EnableVpcClassicLinkResult(Mapping):
        pass
    
    class EnableVpcClassicLinkDnsSupportRequest(Mapping):
        pass
    
    class EnableVpcClassicLinkDnsSupportResult(Mapping):
        pass
    
    class ExportClientVpnClientCertificateRevocationListRequest(Mapping):
        pass
    
    class ExportClientVpnClientCertificateRevocationListResult(Mapping):
        pass
    
    class ExportClientVpnClientConfigurationRequest(Mapping):
        pass
    
    class ExportClientVpnClientConfigurationResult(Mapping):
        pass
    
    class ExportTransitGatewayRoutesRequest(Mapping):
        pass
    
    class ExportTransitGatewayRoutesResult(Mapping):
        pass
    
    class GetConsoleOutputRequest(Mapping):
        pass
    
    class GetConsoleOutputResult(Mapping):
        pass
    
    class GetConsoleScreenshotRequest(Mapping):
        pass
    
    class GetConsoleScreenshotResult(Mapping):
        pass
    
    class GetEbsDefaultKmsKeyIdRequest(Mapping):
        pass
    
    class GetEbsDefaultKmsKeyIdResult(Mapping):
        pass
    
    class GetEbsEncryptionByDefaultRequest(Mapping):
        pass
    
    class GetEbsEncryptionByDefaultResult(Mapping):
        pass
    
    class GetHostReservationPurchasePreviewRequest(Mapping):
        pass
    
    class GetHostReservationPurchasePreviewResult(Mapping):
        pass
    
    class GetLaunchTemplateDataRequest(Mapping):
        pass
    
    class GetLaunchTemplateDataResult(Mapping):
        pass
    
    class GetPasswordDataRequest(Mapping):
        pass
    
    class GetPasswordDataResult(Mapping):
        pass
    
    class GetReservedInstancesExchangeQuoteRequest(Mapping):
        pass
    
    class GetReservedInstancesExchangeQuoteResult(Mapping):
        pass
    
    class GetTransitGatewayAttachmentPropagationsRequest(Mapping):
        pass
    
    class GetTransitGatewayAttachmentPropagationsResult(Mapping):
        pass
    
    class GetTransitGatewayRouteTableAssociationsRequest(Mapping):
        pass
    
    class GetTransitGatewayRouteTableAssociationsResult(Mapping):
        pass
    
    class GetTransitGatewayRouteTablePropagationsRequest(Mapping):
        pass
    
    class GetTransitGatewayRouteTablePropagationsResult(Mapping):
        pass
    
    class ImportClientVpnClientCertificateRevocationListRequest(Mapping):
        pass
    
    class ImportClientVpnClientCertificateRevocationListResult(Mapping):
        pass
    
    class ImportImageRequest(Mapping):
        pass
    
    class ImportImageResult(Mapping):
        pass
    
    class ClientData(Mapping):
        pass
    
    class ImageDiskContainer(Mapping):
        pass
    
    class ImportInstanceRequest(Mapping):
        pass
    
    class ImportInstanceResult(Mapping):
        pass
    
    class DiskImage(Mapping):
        pass
    
    class ImportInstanceLaunchSpecification(Mapping):
        pass
    
    class ImportKeyPairRequest(Mapping):
        pass
    
    class ImportKeyPairResult(Mapping):
        pass
    
    class Blob(object):
        pass
    
    class ImportSnapshotRequest(Mapping):
        pass
    
    class ImportSnapshotResult(Mapping):
        pass
    
    class SnapshotDiskContainer(Mapping):
        pass
    
    class ImportVolumeRequest(Mapping):
        pass
    
    class ImportVolumeResult(Mapping):
        pass
    
    class DiskImageDetail(Mapping):
        pass
    
    class VolumeDetail(Mapping):
        pass
    
    class ModifyCapacityReservationRequest(Mapping):
        pass
    
    class ModifyCapacityReservationResult(Mapping):
        pass
    
    class ModifyClientVpnEndpointRequest(Mapping):
        pass
    
    class ModifyClientVpnEndpointResult(Mapping):
        pass
    
    class DnsServersOptionsModifyStructure(Mapping):
        pass
    
    class ModifyEbsDefaultKmsKeyIdRequest(Mapping):
        pass
    
    class ModifyEbsDefaultKmsKeyIdResult(Mapping):
        pass
    
    class ModifyFleetRequest(Mapping):
        pass
    
    class ModifyFleetResult(Mapping):
        pass
    
    class ModifyFpgaImageAttributeRequest(Mapping):
        pass
    
    class ModifyFpgaImageAttributeResult(Mapping):
        pass
    
    class LoadPermissionModifications(Mapping):
        pass
    
    class ModifyHostsRequest(Mapping):
        pass
    
    class ModifyHostsResult(Mapping):
        pass
    
    class ModifyIdFormatRequest(Mapping):
        pass
    
    class ModifyIdentityIdFormatRequest(Mapping):
        pass
    
    class ModifyImageAttributeRequest(Mapping):
        pass
    
    class AttributeValue(Mapping):
        pass
    
    class LaunchPermissionModifications(Mapping):
        pass
    
    class ModifyInstanceAttributeRequest(Mapping):
        pass
    
    class AttributeBooleanValue(Mapping):
        pass
    
    class InstanceBlockDeviceMappingSpecification(Mapping):
        pass
    
    class BlobAttributeValue(Mapping):
        pass
    
    class ModifyInstanceCapacityReservationAttributesRequest(Mapping):
        pass
    
    class ModifyInstanceCapacityReservationAttributesResult(Mapping):
        pass
    
    class CapacityReservationSpecification(Mapping):
        pass
    
    class ModifyInstanceCreditSpecificationRequest(Mapping):
        pass
    
    class ModifyInstanceCreditSpecificationResult(Mapping):
        pass
    
    class InstanceCreditSpecificationRequest(Mapping):
        pass
    
    class ModifyInstanceEventStartTimeRequest(Mapping):
        pass
    
    class ModifyInstanceEventStartTimeResult(Mapping):
        pass
    
    class ModifyInstancePlacementRequest(Mapping):
        pass
    
    class ModifyInstancePlacementResult(Mapping):
        pass
    
    class ModifyLaunchTemplateRequest(Mapping):
        pass
    
    class ModifyLaunchTemplateResult(Mapping):
        pass
    
    class ModifyNetworkInterfaceAttributeRequest(Mapping):
        pass
    
    class NetworkInterfaceAttachmentChanges(Mapping):
        pass
    
    class ModifyReservedInstancesRequest(Mapping):
        pass
    
    class ModifyReservedInstancesResult(Mapping):
        pass
    
    class ReservedInstancesConfiguration(Mapping):
        pass
    
    class ModifySnapshotAttributeRequest(Mapping):
        pass
    
    class CreateVolumePermissionModifications(Mapping):
        pass
    
    class ModifySpotFleetRequestRequest(Mapping):
        pass
    
    class ModifySpotFleetRequestResponse(Mapping):
        pass
    
    class ModifySubnetAttributeRequest(Mapping):
        pass
    
    class ModifyTrafficMirrorFilterNetworkServicesRequest(Mapping):
        pass
    
    class ModifyTrafficMirrorFilterNetworkServicesResult(Mapping):
        pass
    
    class TrafficMirrorNetworkService(object):
        pass
    
    class ModifyTrafficMirrorFilterRuleRequest(Mapping):
        pass
    
    class ModifyTrafficMirrorFilterRuleResult(Mapping):
        pass
    
    class TrafficMirrorFilterRuleField(object):
        pass
    
    class ModifyTrafficMirrorSessionRequest(Mapping):
        pass
    
    class ModifyTrafficMirrorSessionResult(Mapping):
        pass
    
    class TrafficMirrorSessionField(object):
        pass
    
    class ModifyTransitGatewayVpcAttachmentRequest(Mapping):
        pass
    
    class ModifyTransitGatewayVpcAttachmentResult(Mapping):
        pass
    
    class ModifyTransitGatewayVpcAttachmentRequestOptions(Mapping):
        pass
    
    class ModifyVolumeRequest(Mapping):
        pass
    
    class ModifyVolumeResult(Mapping):
        pass
    
    class ModifyVolumeAttributeRequest(Mapping):
        pass
    
    class ModifyVpcAttributeRequest(Mapping):
        pass
    
    class ModifyVpcEndpointRequest(Mapping):
        pass
    
    class ModifyVpcEndpointResult(Mapping):
        pass
    
    class ModifyVpcEndpointConnectionNotificationRequest(Mapping):
        pass
    
    class ModifyVpcEndpointConnectionNotificationResult(Mapping):
        pass
    
    class ModifyVpcEndpointServiceConfigurationRequest(Mapping):
        pass
    
    class ModifyVpcEndpointServiceConfigurationResult(Mapping):
        pass
    
    class ModifyVpcEndpointServicePermissionsRequest(Mapping):
        pass
    
    class ModifyVpcEndpointServicePermissionsResult(Mapping):
        pass
    
    class ModifyVpcPeeringConnectionOptionsRequest(Mapping):
        pass
    
    class ModifyVpcPeeringConnectionOptionsResult(Mapping):
        pass
    
    class PeeringConnectionOptionsRequest(Mapping):
        pass
    
    class ModifyVpcTenancyRequest(Mapping):
        pass
    
    class ModifyVpcTenancyResult(Mapping):
        pass
    
    class ModifyVpnConnectionRequest(Mapping):
        pass
    
    class ModifyVpnConnectionResult(Mapping):
        pass
    
    class MonitorInstancesRequest(Mapping):
        pass
    
    class MonitorInstancesResult(Mapping):
        pass
    
    class MoveAddressToVpcRequest(Mapping):
        pass
    
    class MoveAddressToVpcResult(Mapping):
        pass
    
    class ProvisionByoipCidrRequest(Mapping):
        pass
    
    class ProvisionByoipCidrResult(Mapping):
        pass
    
    class CidrAuthorizationContext(Mapping):
        pass
    
    class PurchaseHostReservationRequest(Mapping):
        pass
    
    class PurchaseHostReservationResult(Mapping):
        pass
    
    class PurchaseReservedInstancesOfferingRequest(Mapping):
        pass
    
    class PurchaseReservedInstancesOfferingResult(Mapping):
        pass
    
    class ReservedInstanceLimitPrice(Mapping):
        pass
    
    class PurchaseScheduledInstancesRequest(Mapping):
        pass
    
    class PurchaseScheduledInstancesResult(Mapping):
        pass
    
    class PurchaseRequest(Mapping):
        pass
    
    class RebootInstancesRequest(Mapping):
        pass
    
    class RegisterImageRequest(Mapping):
        pass
    
    class RegisterImageResult(Mapping):
        pass
    
    class RejectTransitGatewayVpcAttachmentRequest(Mapping):
        pass
    
    class RejectTransitGatewayVpcAttachmentResult(Mapping):
        pass
    
    class RejectVpcEndpointConnectionsRequest(Mapping):
        pass
    
    class RejectVpcEndpointConnectionsResult(Mapping):
        pass
    
    class RejectVpcPeeringConnectionRequest(Mapping):
        pass
    
    class RejectVpcPeeringConnectionResult(Mapping):
        pass
    
    class ReleaseAddressRequest(Mapping):
        pass
    
    class ReleaseHostsRequest(Mapping):
        pass
    
    class ReleaseHostsResult(Mapping):
        pass
    
    class ReplaceIamInstanceProfileAssociationRequest(Mapping):
        pass
    
    class ReplaceIamInstanceProfileAssociationResult(Mapping):
        pass
    
    class ReplaceNetworkAclAssociationRequest(Mapping):
        pass
    
    class ReplaceNetworkAclAssociationResult(Mapping):
        pass
    
    class ReplaceNetworkAclEntryRequest(Mapping):
        pass
    
    class ReplaceRouteRequest(Mapping):
        pass
    
    class ReplaceRouteTableAssociationRequest(Mapping):
        pass
    
    class ReplaceRouteTableAssociationResult(Mapping):
        pass
    
    class ReplaceTransitGatewayRouteRequest(Mapping):
        pass
    
    class ReplaceTransitGatewayRouteResult(Mapping):
        pass
    
    class ReportInstanceStatusRequest(Mapping):
        pass
    
    class ReportInstanceReasonCodes(object):
        pass
    
    class RequestSpotFleetRequest(Mapping):
        pass
    
    class RequestSpotFleetResponse(Mapping):
        pass
    
    class SpotFleetRequestConfigData(Mapping):
        pass
    
    class RequestSpotInstancesRequest(Mapping):
        pass
    
    class RequestSpotInstancesResult(Mapping):
        pass
    
    class RequestSpotLaunchSpecification(Mapping):
        pass
    
    class ResetEbsDefaultKmsKeyIdRequest(Mapping):
        pass
    
    class ResetEbsDefaultKmsKeyIdResult(Mapping):
        pass
    
    class ResetFpgaImageAttributeRequest(Mapping):
        pass
    
    class ResetFpgaImageAttributeResult(Mapping):
        pass
    
    class ResetImageAttributeRequest(Mapping):
        pass
    
    class ResetInstanceAttributeRequest(Mapping):
        pass
    
    class ResetNetworkInterfaceAttributeRequest(Mapping):
        pass
    
    class ResetSnapshotAttributeRequest(Mapping):
        pass
    
    class RestoreAddressToClassicRequest(Mapping):
        pass
    
    class RestoreAddressToClassicResult(Mapping):
        pass
    
    class RevokeClientVpnIngressRequest(Mapping):
        pass
    
    class RevokeClientVpnIngressResult(Mapping):
        pass
    
    class RevokeSecurityGroupEgressRequest(Mapping):
        pass
    
    class RevokeSecurityGroupIngressRequest(Mapping):
        pass
    
    class RunInstancesRequest(Mapping):
        pass
    
    class Reservation(Mapping):
        pass
    
    class RunInstancesMonitoringEnabled(Mapping):
        pass
    
    class Placement(Mapping):
        pass
    
    class InstanceNetworkInterfaceSpecification(Mapping):
        pass
    
    class ElasticGpuSpecification(Mapping):
        pass
    
    class ElasticInferenceAccelerator(Mapping):
        pass
    
    class LaunchTemplateSpecification(Mapping):
        pass
    
    class InstanceMarketOptionsRequest(Mapping):
        pass
    
    class CreditSpecificationRequest(Mapping):
        pass
    
    class CpuOptionsRequest(Mapping):
        pass
    
    class HibernationOptionsRequest(Mapping):
        pass
    
    class LicenseConfigurationRequest(Mapping):
        pass
    
    class RunScheduledInstancesRequest(Mapping):
        pass
    
    class RunScheduledInstancesResult(Mapping):
        pass
    
    class ScheduledInstancesLaunchSpecification(Mapping):
        pass
    
    class SearchTransitGatewayRoutesRequest(Mapping):
        pass
    
    class SearchTransitGatewayRoutesResult(Mapping):
        pass
    
    class StartInstancesRequest(Mapping):
        pass
    
    class StartInstancesResult(Mapping):
        pass
    
    class StopInstancesRequest(Mapping):
        pass
    
    class StopInstancesResult(Mapping):
        pass
    
    class TerminateClientVpnConnectionsRequest(Mapping):
        pass
    
    class TerminateClientVpnConnectionsResult(Mapping):
        pass
    
    class TerminateInstancesRequest(Mapping):
        pass
    
    class TerminateInstancesResult(Mapping):
        pass
    
    class UnassignIpv6AddressesRequest(Mapping):
        pass
    
    class UnassignIpv6AddressesResult(Mapping):
        pass
    
    class UnassignPrivateIpAddressesRequest(Mapping):
        pass
    
    class UnmonitorInstancesRequest(Mapping):
        pass
    
    class UnmonitorInstancesResult(Mapping):
        pass
    
    class UpdateSecurityGroupRuleDescriptionsEgressRequest(Mapping):
        pass
    
    class UpdateSecurityGroupRuleDescriptionsEgressResult(Mapping):
        pass
    
    class UpdateSecurityGroupRuleDescriptionsIngressRequest(Mapping):
        pass
    
    class UpdateSecurityGroupRuleDescriptionsIngressResult(Mapping):
        pass
    
    class WithdrawByoipCidrRequest(Mapping):
        pass
    
    class WithdrawByoipCidrResult(Mapping):
        pass
    
    def accept_reserved_instances_exchange_quote(self, ReservedInstanceIds: List[String], DryRun: Boolean=None, TargetConfigurations: List[TargetConfigurationRequest]=None) -> AcceptReservedInstancesExchangeQuoteResult:
        """Accepts the Convertible Reserved Instance exchange quote described in the <a>GetReservedInstancesExchangeQuote</a> call.

        :param self:
        :param TargetConfigurations:  The configuration of the target Convertible Reserved Instance to exchange for your current Convertible Reserved Instances.<br/>ReservedInstanceIds:  The IDs of the Convertible Reserved Instances to exchange for another Convertible Reserved Instance of the same or higher value.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: ExchangeId:  The ID of the successful exchange.<br/>        """
        pass

    def accept_transit_gateway_vpc_attachment(self, TransitGatewayAttachmentId: str, DryRun: Boolean=None) -> AcceptTransitGatewayVpcAttachmentResult:
        """Accepts a request to attach a VPC to a transit gateway. The VPC attachment must be in the <code>pendingAcceptance</code> state. Use <a>DescribeTransitGatewayVpcAttachments</a> to view your pending VPC attachment requests. Use <a>RejectTransitGatewayVpcAttachment</a> to reject a VPC attachment request.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>TransitGatewayAttachmentId:  The ID of the attachment.<br/>
        :return: TransitGatewayVpcAttachment:  The VPC attachment.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The tags for the VPC attachment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> The VPC attachment options.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>: The VPC attachment options.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The creation time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> The IDs of the subnets.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the VPC attachment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcOwnerId</i> The ID of the AWS account that owns the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayId</i> The ID of the transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayAttachmentId</i> The ID of the attachment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Ipv6Support</i> Indicates whether IPv6 support is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DnsSupport</i> Indicates whether DNS support is enabled.<br/>        """
        pass

    def accept_vpc_endpoint_connections(self, VpcEndpointIds: List[String], ServiceId: str, DryRun: Boolean=None) -> AcceptVpcEndpointConnectionsResult:
        """Accepts one or more interface VPC endpoint connection requests to your VPC endpoint service.

        :param self:
        :param VpcEndpointIds:  The IDs of one or more interface VPC endpoints.<br/>ServiceId:  The ID of the endpoint service.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: Unsuccessful:  Information about the interface endpoints that were not accepted, if applicable.<br/>        """
        pass

    def accept_vpc_peering_connection(self, DryRun: Boolean=None, VpcPeeringConnectionId: str=None) -> AcceptVpcPeeringConnectionResult:
        """Accept a VPC peering connection request. To accept a request, the VPC peering connection must be in the <code>pending-acceptance</code> state, and you must be the owner of the peer VPC. Use <a>DescribeVpcPeeringConnections</a> to view your outstanding VPC peering connection requests. For an inter-Region VPC peering connection request, you must accept the VPC peering connection in the Region of the accepter VPC.

        :param self:
        :param VpcPeeringConnectionId:  The ID of the VPC peering connection. You must specify this parameter in the request.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: VpcPeeringConnection:  Information about the VPC peering connection.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcPeeringConnectionId</i> The ID of the VPC peering connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> Any tags assigned to the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the VPC peering connection.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>: The status of the VPC peering connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequesterVpcInfo</i> Information about the requester VPC. CIDR block information is only returned when describing an active VPC peering connection.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RequesterVpcInfo</b>: Information about the requester VPC. CIDR block information is only returned when describing an active VPC peering connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpirationTime</i> The time that an unaccepted VPC peering connection will expire.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccepterVpcInfo</i> Information about the accepter VPC. CIDR block information is only returned when describing an active VPC peering connection.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AccepterVpcInfo</b>: Information about the accepter VPC. CIDR block information is only returned when describing an active VPC peering connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The AWS account ID of the VPC owner.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CidrBlockSet</i> Information about the IPv4 CIDR blocks for the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Ipv6CidrBlockSet</i> The IPv6 CIDR block for the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CidrBlock</i> The IPv4 CIDR block for the VPC.<br/>Information about the VPC peering connection options for the accepter or requester VPC.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PeeringOptions</b>: Information about the VPC peering connection options for the accepter or requester VPC.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The AWS account ID of the VPC owner.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CidrBlockSet</i> Information about the IPv4 CIDR blocks for the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Ipv6CidrBlockSet</i> The IPv6 CIDR block for the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CidrBlock</i> The IPv4 CIDR block for the VPC.<br/>Information about the VPC peering connection options for the accepter or requester VPC.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PeeringOptions</b>: Information about the VPC peering connection options for the accepter or requester VPC.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> A message that provides more information about the status, if applicable.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Code</i> The status of the VPC peering connection.<br/>        """
        pass

    def advertise_byoip_cidr(self, Cidr: str, DryRun: Boolean=None) -> AdvertiseByoipCidrResult:
        """Advertises an IPv4 address range that is provisioned for use with your AWS resources through bring your own IP addresses (BYOIP). You can perform this operation at most once every 10 seconds, even if you specify different address ranges each time. We recommend that you stop advertising the BYOIP CIDR from other locations when you advertise it from AWS. To minimize down time, you can configure your AWS resources to use an address from a BYOIP CIDR before it is advertised, and then simultaneously stop advertising it from the current location and start advertising it through AWS. It can take a few minutes before traffic to the specified addresses starts routing to AWS because of BGP propagation delays. To stop advertising the BYOIP CIDR, use <a>WithdrawByoipCidr</a>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Cidr:  The IPv4 address range, in CIDR notation. This must be the exact range that you provisioned. You can't advertise only a portion of the provisioned range.<br/>
        :return: ByoipCidr:  Information about the address range.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the address pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> Upon success, contains the ID of the address pool. Otherwise, contains an error message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the address range.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Cidr</i> The public IPv4 address range, in CIDR notation.<br/>        """
        pass

    def allocate_address(self, Domain: str=None, Address: str=None, PublicIpv4Pool: str=None, DryRun: Boolean=None) -> AllocateAddressResult:
        """Allocates an Elastic IP address to your AWS account. After you allocate the Elastic IP address you can associate it with an instance or network interface. After you release an Elastic IP address, it is released to the IP address pool and can be allocated to a different AWS account. You can allocate an Elastic IP address from an address pool owned by AWS or from an address pool created from a public IPv4 address range that you have brought to AWS for use with your AWS resources using bring your own IP addresses (BYOIP). For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-byoip.html">Bring Your Own IP Addresses (BYOIP)</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. [EC2-VPC] If you release an Elastic IP address, you might be able to recover it. You cannot recover an Elastic IP address that you released after it is allocated to another AWS account. You cannot recover an Elastic IP address for EC2-Classic. To attempt to recover an Elastic IP address that you released, specify it in this operation. An Elastic IP address is for use either in the EC2-Classic platform or in a VPC. By default, you can allocate 5 Elastic IP addresses for EC2-Classic per Region and 5 Elastic IP addresses for EC2-VPC per Region. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/elastic-ip-addresses-eip.html">Elastic IP Addresses</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>PublicIpv4Pool:  The ID of an address pool that you own. Use this parameter to let Amazon EC2 select an address from the address pool. To specify a specific address from the address pool, use the <code>Address</code> parameter instead.<br/>Address:  [EC2-VPC] The Elastic IP address to recover or an IPv4 address from an address pool.<br/>Domain:  Set to <code>vpc</code> to allocate the address for use with instances in a VPC. Default: The address is for use with instances in EC2-Classic.<br/>
        :return: Domain:  Indicates whether this Elastic IP address is for use with instances in EC2-Classic (<code>standard</code>) or instances in a VPC (<code>vpc</code>).<br/>PublicIpv4Pool:  The ID of an address pool.<br/>AllocationId:  [EC2-VPC] The ID that AWS assigns to represent the allocation of the Elastic IP address for use with instances in a VPC.<br/>PublicIp:  The Elastic IP address.<br/>        """
        pass

    def allocate_hosts(self, Quantity: int, InstanceType: str, AvailabilityZone: str, AutoPlacement: str=None, ClientToken: str=None, TagSpecifications: List[TagSpecification]=None, HostRecovery: str=None) -> AllocateHostsResult:
        """Allocates a Dedicated Host to your account. At a minimum, specify the instance size type, Availability Zone, and quantity of hosts to allocate.

        :param self:
        :param HostRecovery:  Indicates whether to enable or disable host recovery for the Dedicated Host. Host recovery is disabled by default. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/dedicated-hosts-recovery.html"> Host Recovery</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. Default: <code>off</code> <br/>TagSpecifications:  The tags to apply to the Dedicated Host during creation.<br/>Quantity:  The number of Dedicated Hosts to allocate to your account with these parameters.<br/>InstanceType:  Specifies the instance type for which to configure your Dedicated Hosts. When you specify the instance type, that is the only instance type that you can launch onto that host.<br/>ClientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.<br/>AvailabilityZone:  The Availability Zone in which to allocate the Dedicated Host.<br/>AutoPlacement:  Indicates whether the host accepts any untargeted instance launches that match its instance type configuration, or if it only accepts Host tenancy instance launches that specify its unique host ID. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/how-dedicated-hosts-work.html#dedicated-hosts-understanding"> Understanding Instance Placement and Host Affinity</a> in the <i>Amazon EC2 User Guide for Linux Instances</i>. Default: <code>on</code> <br/>
        :return: HostIds:  The ID of the allocated Dedicated Host. This is used to launch an instance onto a specific host.<br/>        """
        pass

    def apply_security_groups_to_client_vpn_target_network(self, SecurityGroupIds: List[String], VpcId: str, ClientVpnEndpointId: str, DryRun: Boolean=None) -> ApplySecurityGroupsToClientVpnTargetNetworkResult:
        """Applies a security group to the association between the target network and the Client VPN endpoint. This action replaces the existing security groups with the specified security groups.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>SecurityGroupIds:  The IDs of the security groups to apply to the associated target network. Up to 5 security groups can be applied to an associated target network.<br/>VpcId:  The ID of the VPC in which the associated target network is located.<br/>ClientVpnEndpointId:  The ID of the Client VPN endpoint.<br/>
        :return: SecurityGroupIds:  The IDs of the applied security groups.<br/>        """
        pass

    def assign_ipv6_addresses(self, NetworkInterfaceId: str, Ipv6AddressCount: int=None, Ipv6Addresses: List[String]=None) -> AssignIpv6AddressesResult:
        """Assigns one or more IPv6 addresses to the specified network interface. You can specify one or more specific IPv6 addresses, or you can specify the number of IPv6 addresses to be automatically assigned from within the subnet's IPv6 CIDR block range. You can assign as many IPv6 addresses to a network interface as you can assign private IPv4 addresses, and the limit varies per instance type. For information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/using-eni.html#AvailableIpPerENI">IP Addresses Per Network Interface Per Instance Type</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param NetworkInterfaceId:  The ID of the network interface.<br/>Ipv6Addresses:  One or more specific IPv6 addresses to be assigned to the network interface. You can't use this option if you're specifying a number of IPv6 addresses.<br/>Ipv6AddressCount:  The number of IPv6 addresses to assign to the network interface. Amazon EC2 automatically selects the IPv6 addresses from the subnet range. You can't use this option if specifying specific IPv6 addresses.<br/>
        :return: NetworkInterfaceId:  The ID of the network interface.<br/>AssignedIpv6Addresses:  The IPv6 addresses assigned to the network interface.<br/>        """
        pass

    def assign_private_ip_addresses(self, NetworkInterfaceId: str, AllowReassignment: Boolean=None, PrivateIpAddresses: List[String]=None, SecondaryPrivateIpAddressCount: int=None) -> AssignPrivateIpAddressesResult:
        """Assigns one or more secondary private IP addresses to the specified network interface. You can specify one or more specific secondary IP addresses, or you can specify the number of secondary IP addresses to be automatically assigned within the subnet's CIDR block range. The number of secondary IP addresses that you can assign to an instance varies by instance type. For information about instance types, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html">Instance Types</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. For more information about Elastic IP addresses, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/elastic-ip-addresses-eip.html">Elastic IP Addresses</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. When you move a secondary private IP address to another network interface, any Elastic IP address that is associated with the IP address is also moved. Remapping an IP address is an asynchronous operation. When you move an IP address from one network interface to another, check <code>network/interfaces/macs/mac/local-ipv4s</code> in the instance metadata to confirm that the remapping is complete.

        :param self:
        :param SecondaryPrivateIpAddressCount:  The number of secondary IP addresses to assign to the network interface. You can't specify this parameter when also specifying private IP addresses.<br/>PrivateIpAddresses:  One or more IP addresses to be assigned as a secondary private IP address to the network interface. You can't specify this parameter when also specifying a number of secondary IP addresses. If you don't specify an IP address, Amazon EC2 automatically selects an IP address within the subnet range.<br/>NetworkInterfaceId:  The ID of the network interface.<br/>AllowReassignment:  Indicates whether to allow an IP address that is already assigned to another network interface or instance to be reassigned to the specified network interface.<br/>
        :return: AssignedPrivateIpAddresses:  The private IP addresses assigned to the network interface.<br/>NetworkInterfaceId:  The ID of the network interface.<br/>        """
        pass

    def associate_address(self, AllocationId: str=None, InstanceId: str=None, PublicIp: str=None, AllowReassociation: Boolean=None, DryRun: Boolean=None, NetworkInterfaceId: str=None, PrivateIpAddress: str=None) -> AssociateAddressResult:
        """Associates an Elastic IP address with an instance or a network interface. Before you can use an Elastic IP address, you must allocate it to your account. An Elastic IP address is for use in either the EC2-Classic platform or in a VPC. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/elastic-ip-addresses-eip.html">Elastic IP Addresses</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. [EC2-Classic, VPC in an EC2-VPC-only account] If the Elastic IP address is already associated with a different instance, it is disassociated from that instance and associated with the specified instance. If you associate an Elastic IP address with an instance that has an existing Elastic IP address, the existing address is disassociated from the instance, but remains allocated to your account. [VPC in an EC2-Classic account] If you don't specify a private IP address, the Elastic IP address is associated with the primary IP address. If the Elastic IP address is already associated with a different instance or a network interface, you get an error unless you allow reassociation. You cannot associate an Elastic IP address with an instance or network interface that has an existing Elastic IP address. <important> This is an idempotent operation. If you perform the operation more than once, Amazon EC2 doesn't return an error, and you may be charged for each time the Elastic IP address is remapped to the same instance. For more information, see the <i>Elastic IP Addresses</i> section of <a href="http://aws.amazon.com/ec2/pricing/">Amazon EC2 Pricing</a>. </important>

        :param self:
        :param PrivateIpAddress:  [EC2-VPC] The primary or secondary private IP address to associate with the Elastic IP address. If no private IP address is specified, the Elastic IP address is associated with the primary private IP address.<br/>NetworkInterfaceId:  [EC2-VPC] The ID of the network interface. If the instance has more than one network interface, you must specify a network interface ID. For EC2-VPC, you can specify either the instance ID or the network interface ID, but not both. <br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>AllowReassociation:  [EC2-VPC] For a VPC in an EC2-Classic account, specify true to allow an Elastic IP address that is already associated with an instance or network interface to be reassociated with the specified instance or network interface. Otherwise, the operation fails. In a VPC in an EC2-VPC-only account, reassociation is automatic, therefore you can specify false to ensure the operation fails if the Elastic IP address is already associated with another resource.<br/>PublicIp:  The Elastic IP address to associate with the instance. This is required for EC2-Classic.<br/>InstanceId:  The ID of the instance. This is required for EC2-Classic. For EC2-VPC, you can specify either the instance ID or the network interface ID, but not both. The operation fails if you specify an instance ID unless exactly one network interface is attached.<br/>AllocationId:  [EC2-VPC] The allocation ID. This is required for EC2-VPC.<br/>
        :return: AssociationId:  [EC2-VPC] The ID that represents the association of the Elastic IP address with an instance.<br/>        """
        pass

    def associate_client_vpn_target_network(self, SubnetId: str, ClientVpnEndpointId: str, ClientToken: str=None, DryRun: Boolean=None) -> AssociateClientVpnTargetNetworkResult:
        """Associates a target network with a Client VPN endpoint. A target network is a subnet in a VPC. You can associate multiple subnets from the same VPC with a Client VPN endpoint. You can associate only one subnet in each Availability Zone. We recommend that you associate at least two subnets to provide Availability Zone redundancy.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>ClientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.<br/>SubnetId:  The ID of the subnet to associate with the Client VPN endpoint.<br/>ClientVpnEndpointId:  The ID of the Client VPN endpoint.<br/>
        :return: Status:  The current state of the target network association.:<br/>AssociationId:  The unique ID of the target network association.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> A message about the status of the target network association, if applicable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Code</i> The state of the target network association.<br/>        """
        pass

    def associate_dhcp_options(self, VpcId: str, DhcpOptionsId: str, DryRun: Boolean=None):
        """Associates a set of DHCP options (that you've previously created) with the specified VPC, or associates no DHCP options with the VPC. After you associate the options with the VPC, any existing instances and all new instances that you launch in that VPC use the options. You don't need to restart or relaunch the instances. They automatically pick up the changes within a few hours, depending on how frequently the instance renews its DHCP lease. You can explicitly renew the lease using the operating system on the instance. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_DHCP_Options.html">DHCP Options Sets</a> in the <i>Amazon Virtual Private Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>VpcId:  The ID of the VPC.<br/>DhcpOptionsId:  The ID of the DHCP options set, or <code>default</code> to associate no DHCP options with the VPC.<br/>
        :return:         """
        pass

    def associate_iam_instance_profile(self, InstanceId: str, IamInstanceProfile: IamInstanceProfileSpecification) -> AssociateIamInstanceProfileResult:
        """Associates an IAM instance profile with a running or stopped instance. You cannot associate more than one IAM instance profile with an instance.

        :param self:
        :param InstanceId:  The ID of the instance.<br/>IamInstanceProfile:  The IAM instance profile.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the instance profile.<br/>
        :return: IamInstanceProfileAssociation:  Information about the IAM instance profile association.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timestamp</i> The time the IAM instance profile was associated with the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the association.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamInstanceProfile</i> The IAM instance profile.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>IamInstanceProfile</b>: The IAM instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceId</i> The ID of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociationId</i> The ID of the association.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the instance profile.<br/>        """
        pass

    def associate_route_table(self, SubnetId: str, RouteTableId: str, DryRun: Boolean=None) -> AssociateRouteTableResult:
        """Associates a subnet with a route table. The subnet and route table must be in the same VPC. This association causes traffic originating from the subnet to be routed according to the routes in the route table. The action returns an association ID, which you need in order to disassociate the route table from the subnet later. A route table can be associated with multiple subnets. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Route_Tables.html">Route Tables</a> in the <i>Amazon Virtual Private Cloud User Guide</i>.

        :param self:
        :param SubnetId:  The ID of the subnet.<br/>RouteTableId:  The ID of the route table.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: AssociationId:  The route table association ID. This ID is required for disassociating the route table.<br/>        """
        pass

    def associate_subnet_cidr_block(self, SubnetId: str, Ipv6CidrBlock: str) -> AssociateSubnetCidrBlockResult:
        """Associates a CIDR block with your subnet. You can only associate a single IPv6 CIDR block with your subnet. An IPv6 CIDR block must have a prefix length of /64.

        :param self:
        :param SubnetId:  The ID of your subnet.<br/>Ipv6CidrBlock:  The IPv6 CIDR block for your subnet. The subnet must have a /64 prefix length.<br/>
        :return: SubnetId:  The ID of the subnet.<br/>Ipv6CidrBlockAssociation:  Information about the IPv6 CIDR block association.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Ipv6CidrBlockState</i> Information about the state of the CIDR block.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Ipv6CidrBlockState</b>: Information about the state of the CIDR block.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Ipv6CidrBlock</i> The IPv6 CIDR block.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociationId</i> The association ID for the CIDR block.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> A message about the status of the CIDR block, if applicable.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of a CIDR block.<br/>        """
        pass

    def associate_transit_gateway_route_table(self, TransitGatewayAttachmentId: str, TransitGatewayRouteTableId: str, DryRun: Boolean=None) -> AssociateTransitGatewayRouteTableResult:
        """Associates the specified attachment with the specified transit gateway route table. You can associate only one route table with an attachment.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>TransitGatewayAttachmentId:  The ID of the attachment.<br/>TransitGatewayRouteTableId:  The ID of the transit gateway route table.<br/>
        :return: Association:  The ID of the association.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the association.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceType</i> The resource type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceId</i> The ID of the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayAttachmentId</i> The ID of the attachment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayRouteTableId</i> The ID of the transit gateway route table.<br/>        """
        pass

    def associate_vpc_cidr_block(self, VpcId: str, AmazonProvidedIpv6CidrBlock: Boolean=None, CidrBlock: str=None) -> AssociateVpcCidrBlockResult:
        """Associates a CIDR block with your VPC. You can associate a secondary IPv4 CIDR block, or you can associate an Amazon-provided IPv6 CIDR block. The IPv6 CIDR block size is fixed at /56. For more information about associating CIDR blocks with your VPC and applicable restrictions, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Subnets.html#VPC_Sizing">VPC and Subnet Sizing</a> in the <i>Amazon Virtual Private Cloud User Guide</i>.

        :param self:
        :param VpcId:  The ID of the VPC.<br/>CidrBlock:  An IPv4 CIDR block to associate with the VPC.<br/>AmazonProvidedIpv6CidrBlock:  Requests an Amazon-provided IPv6 CIDR block with a /56 prefix length for the VPC. You cannot specify the range of IPv6 addresses, or the size of the CIDR block.<br/>
        :return: VpcId:  The ID of the VPC.<br/>CidrBlockAssociation:  Information about the IPv4 CIDR block association.:<br/>Ipv6CidrBlockAssociation:  Information about the IPv6 CIDR block association.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Ipv6CidrBlockState</i> Information about the state of the CIDR block.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Ipv6CidrBlockState</b>: Information about the state of the CIDR block.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Ipv6CidrBlock</i> The IPv6 CIDR block.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociationId</i> The association ID for the IPv6 CIDR block.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> A message about the status of the CIDR block, if applicable.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the CIDR block.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CidrBlockState</i> Information about the state of the CIDR block.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CidrBlockState</b>: Information about the state of the CIDR block.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CidrBlock</i> The IPv4 CIDR block.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociationId</i> The association ID for the IPv4 CIDR block.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> A message about the status of the CIDR block, if applicable.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the CIDR block.<br/>        """
        pass

    def attach_classic_link_vpc(self, VpcId: str, InstanceId: str, Groups: List[String], DryRun: Boolean=None) -> AttachClassicLinkVpcResult:
        """Links an EC2-Classic instance to a ClassicLink-enabled VPC through one or more of the VPC's security groups. You cannot link an EC2-Classic instance to more than one VPC at a time. You can only link an instance that's in the <code>running</code> state. An instance is automatically unlinked from a VPC when it's stopped - you can link it to the VPC again when you restart it. After you've linked an instance, you cannot change the VPC security groups that are associated with it. To change the security groups, you must first unlink the instance, and then link it again. Linking your instance to a VPC is sometimes referred to as <i>attaching</i> your instance.

        :param self:
        :param VpcId:  The ID of a ClassicLink-enabled VPC.<br/>InstanceId:  The ID of an EC2-Classic instance to link to the ClassicLink-enabled VPC.<br/>Groups:  The ID of one or more of the VPC's security groups. You cannot specify security groups from a different VPC.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: Return:  Returns <code>true</code> if the request succeeds; otherwise, it returns an error.<br/>        """
        pass

    def attach_internet_gateway(self, VpcId: str, InternetGatewayId: str, DryRun: Boolean=None):
        """Attaches an internet gateway to a VPC, enabling connectivity between the internet and the VPC. For more information about your VPC and internet gateway, see the <a href="https://docs.aws.amazon.com/vpc/latest/userguide/">Amazon Virtual Private Cloud User Guide</a>.

        :param self:
        :param VpcId:  The ID of the VPC.<br/>InternetGatewayId:  The ID of the internet gateway.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return:         """
        pass

    def attach_network_interface(self, NetworkInterfaceId: str, InstanceId: str, DeviceIndex: int, DryRun: Boolean=None) -> AttachNetworkInterfaceResult:
        """Attaches a network interface to an instance.

        :param self:
        :param NetworkInterfaceId:  The ID of the network interface.<br/>InstanceId:  The ID of the instance.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>DeviceIndex:  The index of the device for the network interface attachment.<br/>
        :return: AttachmentId:  The ID of the network interface attachment.<br/>        """
        pass

    def attach_volume(self, VolumeId: str, InstanceId: str, Device: str, DryRun: Boolean=None) -> VolumeAttachment:
        """Attaches an EBS volume to a running or stopped instance and exposes it to the instance with the specified device name. Encrypted EBS volumes must be attached to instances that support Amazon EBS encryption. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html">Amazon EBS Encryption</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. After you attach an EBS volume, you must make it available. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ebs-using-volumes.html">Making an EBS Volume Available For Use</a>. If a volume has an AWS Marketplace product code: <ul> <li> The volume can be attached only to a stopped instance. </li> <li> AWS Marketplace product codes are copied from the volume to the instance. </li> <li> You must be subscribed to the product. </li> <li> The instance type and operating system of the instance must support the product. For example, you can't detach a volume from a Windows instance and attach it to a Linux instance. </li> </ul> For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ebs-attaching-volume.html">Attaching Amazon EBS Volumes</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>VolumeId:  The ID of the EBS volume. The volume and instance must be within the same Availability Zone.<br/>InstanceId:  The ID of the instance.<br/>Device:  The device name (for example, <code>/dev/sdh</code> or <code>xvdh</code>).<br/>
        :return: DeleteOnTermination:  Indicates whether the EBS volume is deleted on instance termination.<br/>VolumeId:  The ID of the volume.<br/>State:  The attachment state of the volume.<br/>InstanceId:  The ID of the instance.<br/>Device:  The device name.<br/>AttachTime:  The time stamp when the attachment initiated.<br/>        """
        pass

    def attach_vpn_gateway(self, VpnGatewayId: str, VpcId: str, DryRun: Boolean=None) -> AttachVpnGatewayResult:
        """Attaches a virtual private gateway to a VPC. You can attach one virtual private gateway to one VPC at a time. For more information, see <a href="https://docs.aws.amazon.com/vpn/latest/s2svpn/VPC_VPN.html">AWS Site-to-Site VPN</a> in the <i>AWS Site-to-Site VPN User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>VpnGatewayId:  The ID of the virtual private gateway.<br/>VpcId:  The ID of the VPC.<br/>
        :return: VpcAttachment:  Information about the attachment.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The current state of the attachment.<br/>        """
        pass

    def authorize_client_vpn_ingress(self, TargetNetworkCidr: str, ClientVpnEndpointId: str, AccessGroupId: str=None, AuthorizeAllGroups: Boolean=None, Description: str=None, ClientToken: str=None, DryRun: Boolean=None) -> AuthorizeClientVpnIngressResult:
        """Adds an ingress authorization rule to a Client VPN endpoint. Ingress authorization rules act as firewall rules that grant access to networks. You must configure ingress authorization rules to enable clients to access resources in AWS or on-premises networks.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>ClientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.<br/>Description:  A brief description of the authorization rule.<br/>AuthorizeAllGroups:  Indicates whether to grant access to all clients. Use <code>true</code> to grant all clients who successfully establish a VPN connection access to the network.<br/>AccessGroupId:  The ID of the Active Directory group to grant access.<br/>TargetNetworkCidr:  The IPv4 address range, in CIDR notation, of the network for which access is being authorized.<br/>ClientVpnEndpointId:  The ID of the Client VPN endpoint.<br/>
        :return: Status:  The current state of the authorization rule.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> A message about the status of the authorization rule, if applicable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Code</i> The state of the authorization rule.<br/>        """
        pass

    def authorize_security_group_egress(self, GroupId: str, DryRun: Boolean=None, IpPermissions: List[IpPermission]=None, CidrIp: str=None, FromPort: int=None, IpProtocol: str=None, ToPort: int=None, SourceSecurityGroupName: str=None, SourceSecurityGroupOwnerId: str=None):
        """[VPC only] Adds the specified egress rules to a security group for use with a VPC. An outbound rule permits instances to send traffic to the specified IPv4 or IPv6 CIDR address ranges, or to the instances associated with the specified destination security groups. You specify a protocol for each rule (for example, TCP). For the TCP and UDP protocols, you must also specify the destination port or port range. For the ICMP protocol, you must also specify the ICMP type and code. You can use -1 for the type or code to mean all types or all codes. Rule changes are propagated to affected instances as quickly as possible. However, a small delay might occur. For more information about VPC security group limits, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/amazon-vpc-limits.html">Amazon VPC Limits</a>.

        :param self:
        :param SourceSecurityGroupOwnerId:  Not supported. Use a set of IP permissions to specify a destination security group.<br/>SourceSecurityGroupName:  Not supported. Use a set of IP permissions to specify a destination security group.<br/>ToPort:  Not supported. Use a set of IP permissions to specify the port.<br/>IpProtocol:  Not supported. Use a set of IP permissions to specify the protocol name or number.<br/>FromPort:  Not supported. Use a set of IP permissions to specify the port.<br/>CidrIp:  Not supported. Use a set of IP permissions to specify the CIDR.<br/>IpPermissions:  The sets of IP permissions. You can't specify a destination security group and a CIDR IP address range in the same set of permissions.<br/>GroupId:  The ID of the security group.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return:         """
        pass

    def authorize_security_group_ingress(self, CidrIp: str=None, FromPort: int=None, GroupId: str=None, GroupName: str=None, IpPermissions: List[IpPermission]=None, IpProtocol: str=None, SourceSecurityGroupName: str=None, SourceSecurityGroupOwnerId: str=None, ToPort: int=None, DryRun: Boolean=None):
        """Adds the specified ingress rules to a security group. An inbound rule permits instances to receive traffic from the specified IPv4 or IPv6 CIDR address ranges, or from the instances associated with the specified destination security groups. You specify a protocol for each rule (for example, TCP). For TCP and UDP, you must also specify the destination port or port range. For ICMP/ICMPv6, you must also specify the ICMP/ICMPv6 type and code. You can use -1 to mean all types or all codes. Rule changes are propagated to instances within the security group as quickly as possible. However, a small delay might occur. For more information about VPC security group limits, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/amazon-vpc-limits.html">Amazon VPC Limits</a>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>ToPort:  The end of port range for the TCP and UDP protocols, or an ICMP code number. For the ICMP code number, use <code>-1</code> to specify all codes. If you specify all ICMP types, you must specify all codes. Alternatively, use a set of IP permissions to specify multiple rules and a description for the rule.<br/>SourceSecurityGroupOwnerId:  [nondefault VPC] The AWS account ID for the source security group, if the source security group is in a different account. You can't specify this parameter in combination with the following parameters: the CIDR IP address range, the IP protocol, the start of the port range, and the end of the port range. Creates rules that grant full ICMP, UDP, and TCP access. To create a rule with a specific IP protocol and port range, use a set of IP permissions instead.<br/>SourceSecurityGroupName:  [EC2-Classic, default VPC] The name of the source security group. You can't specify this parameter in combination with the following parameters: the CIDR IP address range, the start of the port range, the IP protocol, and the end of the port range. Creates rules that grant full ICMP, UDP, and TCP access. To create a rule with a specific IP protocol and port range, use a set of IP permissions instead. For EC2-VPC, the source security group must be in the same VPC.<br/>IpProtocol:  The IP protocol name (<code>tcp</code>, <code>udp</code>, <code>icmp</code>) or number (see <a href="http://www.iana.org/assignments/protocol-numbers/protocol-numbers.xhtml">Protocol Numbers</a>). To specify <code>icmpv6</code>, use a set of IP permissions. [VPC only] Use <code>-1</code> to specify all protocols. If you specify <code>-1</code> or a protocol other than <code>tcp</code>, <code>udp</code>, or <code>icmp</code>, traffic on all ports is allowed, regardless of any ports you specify. Alternatively, use a set of IP permissions to specify multiple rules and a description for the rule.<br/>IpPermissions:  The sets of IP permissions.<br/>GroupName:  [EC2-Classic, default VPC] The name of the security group. You must specify either the security group ID or the security group name in the request.<br/>GroupId:  The ID of the security group. You must specify either the security group ID or the security group name in the request. For security groups in a nondefault VPC, you must specify the security group ID.<br/>FromPort:  The start of port range for the TCP and UDP protocols, or an ICMP type number. For the ICMP type number, use <code>-1</code> to specify all types. If you specify all ICMP types, you must specify all codes. Alternatively, use a set of IP permissions to specify multiple rules and a description for the rule.<br/>CidrIp:  The IPv4 address range, in CIDR format. You can't specify this parameter when specifying a source security group. To specify an IPv6 address range, use a set of IP permissions. Alternatively, use a set of IP permissions to specify multiple rules and a description for the rule.<br/>
        :return:         """
        pass

    def bundle_instance(self, Storage: Storage, InstanceId: str, DryRun: Boolean=None) -> BundleInstanceResult:
        """Bundles an Amazon instance store-backed Windows instance. During bundling, only the root device volume (C:\) is bundled. Data on other instance store volumes is not preserved. <note> This action is not applicable for Linux/Unix instances or Windows instances that are backed by Amazon EBS. </note>

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Storage:  The bucket in which to store the AMI. You can specify a bucket that you already own or a new bucket that Amazon EC2 creates on your behalf. If you specify a bucket that belongs to someone else, Amazon EC2 returns an error.:<br/>InstanceId:  The ID of the instance to bundle. Type: String Default: None Required: Yes<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3</i> An Amazon S3 storage location.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3</b>: An Amazon S3 storage location.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UploadPolicySignature</i> The signature of the JSON document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UploadPolicy</i> An Amazon S3 upload policy that gives Amazon EC2 permission to upload items into Amazon S3 on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> The beginning of the file name of the AMI.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> The bucket in which to store the AMI. You can specify a bucket that you already own or a new bucket that Amazon EC2 creates on your behalf. If you specify a bucket that belongs to someone else, Amazon EC2 returns an error.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AWSAccessKeyId</i> The access key ID of the owner of the bucket. Before you specify a value for your access key ID, review and follow the guidance in <a href="https://docs.aws.amazon.com/general/latest/gr/aws-access-keys-best-practices.html">Best Practices for Managing AWS Access Keys</a>.<br/>
        :return: BundleTask:  Information about the bundle task.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateTime</i> The time of the most recent update for the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Storage</i> The Amazon S3 storage locations.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Storage</b>: The Amazon S3 storage locations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTime</i> The time this task started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Progress</i> The level of task completion, as a percent (for example, 20%).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceId</i> The ID of the instance associated with this bundle task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BundleTaskError</i> If the task fails, a description of the error.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BundleTaskError</b>: If the task fails, a description of the error.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BundleId</i> The ID of the bundle task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> The error message.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Code</i> The error code.<br/>An Amazon S3 storage location.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3</b>: An Amazon S3 storage location.        """
        pass

    def cancel_bundle_task(self, BundleId: str, DryRun: Boolean=None) -> CancelBundleTaskResult:
        """Cancels a bundling operation for an instance store-backed Windows instance.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>BundleId:  The ID of the bundle task.<br/>
        :return: BundleTask:  Information about the bundle task.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateTime</i> The time of the most recent update for the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Storage</i> The Amazon S3 storage locations.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Storage</b>: The Amazon S3 storage locations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTime</i> The time this task started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Progress</i> The level of task completion, as a percent (for example, 20%).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceId</i> The ID of the instance associated with this bundle task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BundleTaskError</i> If the task fails, a description of the error.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BundleTaskError</b>: If the task fails, a description of the error.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BundleId</i> The ID of the bundle task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> The error message.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Code</i> The error code.<br/>An Amazon S3 storage location.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3</b>: An Amazon S3 storage location.        """
        pass

    def cancel_capacity_reservation(self, CapacityReservationId: str, DryRun: Boolean=None) -> CancelCapacityReservationResult:
        """Cancels the specified Capacity Reservation, releases the reserved capacity, and changes the Capacity Reservation's state to <code>cancelled</code>. Instances running in the reserved capacity continue running until you stop them. Stopped instances that target the Capacity Reservation can no longer launch. Modify these instances to either target a different Capacity Reservation, launch On-Demand Instance capacity, or run in any open Capacity Reservation that has matching attributes and sufficient capacity.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>CapacityReservationId:  The ID of the Capacity Reservation to be cancelled.<br/>
        :return: Return:  Returns <code>true</code> if the request succeeds; otherwise, it returns an error.<br/>        """
        pass

    def cancel_conversion_task(self, ConversionTaskId: str, DryRun: Boolean=None, ReasonMessage: str=None):
        """Cancels an active conversion task. The task can be the import of an instance or volume. The action removes all artifacts of the conversion, including a partially uploaded volume or instance. If the conversion is complete or is in the process of transferring the final disk image, the command fails and returns an exception. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/CommandLineReference/ec2-cli-vmimport-export.html">Importing a Virtual Machine Using the Amazon EC2 CLI</a>.

        :param self:
        :param ReasonMessage:  The reason for canceling the conversion task.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>ConversionTaskId:  The ID of the conversion task.<br/>
        :return:         """
        pass

    def cancel_export_task(self, ExportTaskId: str):
        """Cancels an active export task. The request removes all artifacts of the export, including any partially-created Amazon S3 objects. If the export task is complete or is in the process of transferring the final disk image, the command fails and returns an error.

        :param self:
        :param ExportTaskId:  The ID of the export task. This is the ID returned by <code>CreateInstanceExportTask</code>.<br/>
        :return:         """
        pass

    def cancel_import_task(self, CancelReason: str=None, DryRun: Boolean=None, ImportTaskId: str=None) -> CancelImportTaskResult:
        """Cancels an in-process import virtual machine or import snapshot task.

        :param self:
        :param ImportTaskId:  The ID of the import image or import snapshot task to be canceled.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>CancelReason:  The reason for canceling the task.<br/>
        :return: State:  The current state of the task being canceled.<br/>PreviousState:  The current state of the task being canceled.<br/>ImportTaskId:  The ID of the task being canceled.<br/>        """
        pass

    def cancel_reserved_instances_listing(self, ReservedInstancesListingId: str) -> CancelReservedInstancesListingResult:
        """Cancels the specified Reserved Instance listing in the Reserved Instance Marketplace. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ri-market-general.html">Reserved Instance Marketplace</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param ReservedInstancesListingId:  The ID of the Reserved Instance listing.<br/>
        :return: ReservedInstancesListings:  The Reserved Instance listing.<br/>        """
        pass

    def cancel_spot_fleet_requests(self, TerminateInstances: Boolean, SpotFleetRequestIds: List[String], DryRun: Boolean=None) -> CancelSpotFleetRequestsResponse:
        """Cancels the specified Spot Fleet requests. After you cancel a Spot Fleet request, the Spot Fleet launches no new Spot Instances. You must specify whether the Spot Fleet should also terminate its Spot Instances. If you terminate the instances, the Spot Fleet request enters the <code>cancelled_terminating</code> state. Otherwise, the Spot Fleet request enters the <code>cancelled_running</code> state and the instances continue to run until they are interrupted or you terminate them manually.

        :param self:
        :param TerminateInstances:  Indicates whether to terminate instances for a Spot Fleet request if it is canceled successfully.<br/>SpotFleetRequestIds:  The IDs of the Spot Fleet requests.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: UnsuccessfulFleetRequests:  Information about the Spot Fleet requests that are not successfully canceled.<br/>SuccessfulFleetRequests:  Information about the Spot Fleet requests that are successfully canceled.<br/>        """
        pass

    def cancel_spot_instance_requests(self, SpotInstanceRequestIds: List[String], DryRun: Boolean=None) -> CancelSpotInstanceRequestsResult:
        """Cancels one or more Spot Instance requests. <important> Canceling a Spot Instance request does not terminate running Spot Instances associated with the request. </important>

        :param self:
        :param SpotInstanceRequestIds:  One or more Spot Instance request IDs.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: CancelledSpotInstanceRequests:  One or more Spot Instance requests.<br/>        """
        pass

    def confirm_product_instance(self, ProductCode: str, InstanceId: str, DryRun: Boolean=None) -> ConfirmProductInstanceResult:
        """Determines whether a product code is associated with an instance. This action can only be used by the owner of the product code. It is useful when a product code owner must verify whether another user's instance is eligible for support.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>ProductCode:  The product code. This must be a product code that you own.<br/>InstanceId:  The ID of the instance.<br/>
        :return: Return:  The return value of the request. Returns <code>true</code> if the specified product code is owned by the requester and associated with the specified instance.<br/>OwnerId:  The AWS account ID of the instance owner. This is only present if the product code is attached to the instance.<br/>        """
        pass

    def copy_fpga_image(self, SourceRegion: str, SourceFpgaImageId: str, DryRun: Boolean=None, Description: str=None, Name: str=None, ClientToken: str=None) -> CopyFpgaImageResult:
        """Copies the specified Amazon FPGA Image (AFI) to the current Region.

        :param self:
        :param ClientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Run_Instance_Idempotency.html">Ensuring Idempotency</a>.<br/>SourceRegion:  The Region that contains the source AFI.<br/>Name:  The name for the new AFI. The default is the name of the source AFI.<br/>Description:  The description for the new AFI.<br/>SourceFpgaImageId:  The ID of the source AFI.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: FpgaImageId:  The ID of the new AFI.<br/>        """
        pass

    def copy_image(self, SourceRegion: str, SourceImageId: str, Name: str, ClientToken: str=None, Description: str=None, Encrypted: Boolean=None, KmsKeyId: str=None, DryRun: Boolean=None) -> CopyImageResult:
        """Initiates the copy of an AMI from the specified source Region to the current Region. You specify the destination Region by using its endpoint when making the request. Copies of encrypted backing snapshots for the AMI are encrypted. Copies of unencrypted backing snapshots remain unencrypted, unless you set <code>Encrypted</code> during the copy operation. You cannot create an unencrypted copy of an encrypted backing snapshot. For more information about the prerequisites and limits when copying an AMI, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/CopyingAMIs.html">Copying an AMI</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>SourceRegion:  The name of the Region that contains the AMI to copy.<br/>SourceImageId:  The ID of the AMI to copy.<br/>Name:  The name of the new AMI in the destination Region.<br/>KmsKeyId:  An identifier for the AWS Key Management Service (AWS KMS) customer master key (CMK) to use when creating the encrypted volume. This parameter is only required if you want to use a non-default CMK; if this parameter is not specified, the default CMK for EBS is used. If a <code>KmsKeyId</code> is specified, the <code>Encrypted</code> flag must also be set.  To specify a CMK, use its key ID, Amazon Resource Name (ARN), alias name, or alias ARN. When using an alias name, prefix it with "alias/". For example: <ul> <li> Key ID: <code>1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Key ARN: <code>arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Alias name: <code>alias/ExampleAlias</code>  </li> <li> Alias ARN: <code>arn:aws:kms:us-east-2:111122223333:alias/ExampleAlias</code>  </li> </ul> AWS parses <code>KmsKeyId</code> asynchronously, meaning that the action you call may appear to complete even though you provided an invalid identifier. This action will eventually report failure.  The specified CMK must exist in the Region that the snapshot is being copied to. <br/>Encrypted:  Specifies whether the destination snapshots of the copied image should be encrypted. You can encrypt a copy of an unencrypted snapshot, but you cannot create an unencrypted copy of an encrypted snapshot. The default CMK for EBS is used unless you specify a non-default AWS Key Management Service (AWS KMS) CMK using <code>KmsKeyId</code>. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html">Amazon EBS Encryption</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.<br/>Description:  A description for the new AMI in the destination Region.<br/>ClientToken:  Unique, case-sensitive identifier you provide to ensure idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Run_Instance_Idempotency.html">How to Ensure Idempotency</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.<br/>
        :return: ImageId:  The ID of the new AMI.<br/>        """
        pass

    def copy_snapshot(self, SourceSnapshotId: str, SourceRegion: str, Description: str=None, DestinationRegion: str=None, Encrypted: Boolean=None, KmsKeyId: str=None, PresignedUrl: str=None, DryRun: Boolean=None) -> CopySnapshotResult:
        """Copies a point-in-time snapshot of an EBS volume and stores it in Amazon S3. You can copy the snapshot within the same Region or from one Region to another. You can use the snapshot to create EBS volumes or Amazon Machine Images (AMIs). Copies of encrypted EBS snapshots remain encrypted. Copies of unencrypted snapshots remain unencrypted, unless you enable encryption for the snapshot copy operation. By default, encrypted snapshot copies use the default AWS Key Management Service (AWS KMS) customer master key (CMK); however, you can specify a different CMK. To copy an encrypted snapshot that has been shared from another account, you must have permissions for the CMK used to encrypt the snapshot. Snapshots created by copying another snapshot have an arbitrary volume ID that should not be used for any purpose. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ebs-copy-snapshot.html">Copying an Amazon EBS Snapshot</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>SourceSnapshotId:  The ID of the EBS snapshot to copy.<br/>SourceRegion:  The ID of the Region that contains the snapshot to be copied.<br/>PresignedUrl:  When you copy an encrypted source snapshot using the Amazon EC2 Query API, you must supply a pre-signed URL. This parameter is optional for unencrypted snapshots. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html">Query Requests</a>. The <code>PresignedUrl</code> should use the snapshot source endpoint, the <code>CopySnapshot</code> action, and include the <code>SourceRegion</code>, <code>SourceSnapshotId</code>, and <code>DestinationRegion</code> parameters. The <code>PresignedUrl</code> must be signed using AWS Signature Version 4. Because EBS snapshots are stored in Amazon S3, the signing algorithm for this parameter uses the same logic that is described in <a href="https://docs.aws.amazon.com/AmazonS3/latest/API/sigv4-query-string-auth.html">Authenticating Requests by Using Query Parameters (AWS Signature Version 4)</a> in the <i>Amazon Simple Storage Service API Reference</i>. An invalid or improperly signed <code>PresignedUrl</code> will cause the copy operation to fail asynchronously, and the snapshot will move to an <code>error</code> state.<br/>KmsKeyId:  The identifier of the AWS Key Management Service (AWS KMS) customer master key (CMK) to use for Amazon EBS encryption. If this parameter is not specified, your AWS managed CMK for EBS is used. If <code>KmsKeyId</code> is specified, the encrypted state must be <code>true</code>. You can specify the CMK using any of the following: <ul> <li> Key ID. For example, key/1234abcd-12ab-34cd-56ef-1234567890ab. </li> <li> Key alias. For example, alias/ExampleAlias. </li> <li> Key ARN. For example, arn:aws:kms:<i>us-east-1</i>:<i>012345678910</i>:key/<i>abcd1234-a123-456a-a12b-a123b4cd56ef</i>. </li> <li> Alias ARN. For example, arn:aws:kms:<i>us-east-1</i>:<i>012345678910</i>:alias/<i>ExampleAlias</i>. </li> </ul> AWS authenticates the CMK asynchronously. Therefore, if you specify an ID, alias, or ARN that is not valid, the action can appear to complete, but eventually fails.<br/>Encrypted:  To encrypt a copy of an unencrypted snapshot if encryption by default is not enabled, enable encryption using this parameter. Otherwise, omit this parameter. Encrypted snapshots are encrypted, even if you omit this parameter and encryption by default is not enabled. You cannot set this parameter to false. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html">Amazon EBS Encryption</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.<br/>DestinationRegion:  The destination Region to use in the <code>PresignedUrl</code> parameter of a snapshot copy operation. This parameter is only valid for specifying the destination Region in a <code>PresignedUrl</code> parameter, where it is required. The snapshot copy is sent to the regional endpoint that you sent the HTTP request to (for example, <code>ec2.us-east-1.amazonaws.com</code>). With the AWS CLI, this is specified using the <code>--region</code> parameter or the default Region in your AWS configuration file.<br/>Description:  A description for the EBS snapshot.<br/>
        :return: SnapshotId:  The ID of the new snapshot.<br/>        """
        pass

    def create_capacity_reservation(self, InstanceCount: int, AvailabilityZone: str, InstancePlatform: str, InstanceType: str, ClientToken: str=None, Tenancy: str=None, EbsOptimized: Boolean=None, EphemeralStorage: Boolean=None, EndDate: DateTime=None, EndDateType: str=None, InstanceMatchCriteria: str=None, TagSpecifications: List[TagSpecification]=None, DryRun: Boolean=None) -> CreateCapacityReservationResult:
        """Creates a new Capacity Reservation with the specified attributes. Capacity Reservations enable you to reserve capacity for your Amazon EC2 instances in a specific Availability Zone for any duration. This gives you the flexibility to selectively add capacity reservations and still get the Regional RI discounts for that usage. By creating Capacity Reservations, you ensure that you always have access to Amazon EC2 capacity when you need it, for as long as you need it. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-capacity-reservations.html">Capacity Reservations</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. Your request to create a Capacity Reservation could fail if Amazon EC2 does not have sufficient capacity to fulfill the request. If your request fails due to Amazon EC2 capacity constraints, either try again at a later time, try in a different Availability Zone, or request a smaller capacity reservation. If your application is flexible across instance types and sizes, try to create a Capacity Reservation with different instance attributes. Your request could also fail if the requested quantity exceeds your On-Demand Instance limit for the selected instance type. If your request fails due to limit constraints, increase your On-Demand Instance limit for the required instance type and try again. For more information about increasing your instance limits, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-resource-limits.html">Amazon EC2 Service Limits</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>TagSpecifications:  The tags to apply to the Capacity Reservation during launch.<br/>InstanceMatchCriteria:  Indicates the type of instance launches that the Capacity Reservation accepts. The options include: <ul> <li>  <code>open</code> - The Capacity Reservation automatically matches all instances that have matching attributes (instance type, platform, and Availability Zone). Instances that have matching attributes run in the Capacity Reservation automatically without specifying any additional parameters. </li> <li>  <code>targeted</code> - The Capacity Reservation only accepts instances that have matching attributes (instance type, platform, and Availability Zone), and explicitly target the Capacity Reservation. This ensures that only permitted instances can use the reserved capacity.  </li> </ul> Default: <code>open</code> <br/>EndDateType:  Indicates the way in which the Capacity Reservation ends. A Capacity Reservation can have one of the following end types: <ul> <li>  <code>unlimited</code> - The Capacity Reservation remains active until you explicitly cancel it. Do not provide an <code>EndDate</code> if the <code>EndDateType</code> is <code>unlimited</code>. </li> <li>  <code>limited</code> - The Capacity Reservation expires automatically at a specified date and time. You must provide an <code>EndDate</code> value if the <code>EndDateType</code> value is <code>limited</code>. </li> </ul><br/>EndDate:  The date and time at which the Capacity Reservation expires. When a Capacity Reservation expires, the reserved capacity is released and you can no longer launch instances into it. The Capacity Reservation's state changes to <code>expired</code> when it reaches its end date and time. You must provide an <code>EndDate</code> value if <code>EndDateType</code> is <code>limited</code>. Omit <code>EndDate</code> if <code>EndDateType</code> is <code>unlimited</code>. If the <code>EndDateType</code> is <code>limited</code>, the Capacity Reservation is cancelled within an hour from the specified time. For example, if you specify 5/31/2019, 13:30:55, the Capacity Reservation is guaranteed to end between 13:30:55 and 14:30:55 on 5/31/2019.<br/>EphemeralStorage:  Indicates whether the Capacity Reservation supports instances with temporary, block-level storage.<br/>EbsOptimized:  Indicates whether the Capacity Reservation supports EBS-optimized instances. This optimization provides dedicated throughput to Amazon EBS and an optimized configuration stack to provide optimal I/O performance. This optimization isn't available with all instance types. Additional usage charges apply when using an EBS- optimized instance.<br/>InstanceCount:  The number of instances for which to reserve capacity.<br/>Tenancy:  Indicates the tenancy of the Capacity Reservation. A Capacity Reservation can have one of the following tenancy settings: <ul> <li>  <code>default</code> - The Capacity Reservation is created on hardware that is shared with other AWS accounts. </li> <li>  <code>dedicated</code> - The Capacity Reservation is created on single-tenant hardware that is dedicated to a single AWS account. </li> </ul><br/>AvailabilityZone:  The Availability Zone in which to create the Capacity Reservation.<br/>InstancePlatform:  The type of operating system for which to reserve capacity.<br/>InstanceType:  The instance type for which to reserve capacity. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html">Instance Types</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.<br/>ClientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>. Constraint: Maximum 64 ASCII characters.<br/>
        :return: CapacityReservation:  Information about the Capacity Reservation.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> Any tags assigned to the Capacity Reservation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateDate</i> The date and time at which the Capacity Reservation was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceMatchCriteria</i> Indicates the type of instance launches that the Capacity Reservation accepts. The options include: <ul> <li>  <code>open</code> - The Capacity Reservation accepts all instances that have matching attributes (instance type, platform, and Availability Zone). Instances that have matching attributes launch into the Capacity Reservation automatically without specifying any additional parameters. </li> <li>  <code>targeted</code> - The Capacity Reservation only accepts instances that have matching attributes (instance type, platform, and Availability Zone), and explicitly target the Capacity Reservation. This ensures that only permitted instances can use the reserved capacity.  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndDateType</i> Indicates the way in which the Capacity Reservation ends. A Capacity Reservation can have one of the following end types: <ul> <li>  <code>unlimited</code> - The Capacity Reservation remains active until you explicitly cancel it. </li> <li>  <code>limited</code> - The Capacity Reservation expires automatically at a specified date and time. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndDate</i> The date and time at which the Capacity Reservation expires. When a Capacity Reservation expires, the reserved capacity is released and you can no longer launch instances into it. The Capacity Reservation's state changes to <code>expired</code> when it reaches its end date and time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The current state of the Capacity Reservation. A Capacity Reservation can be in one of the following states: <ul> <li>  <code>active</code> - The Capacity Reservation is active and the capacity is available for your use. </li> <li>  <code>cancelled</code> - The Capacity Reservation expired automatically at the date and time specified in your request. The reserved capacity is no longer available for your use. </li> <li>  <code>expired</code> - The Capacity Reservation was manually cancelled. The reserved capacity is no longer available for your use. </li> <li>  <code>pending</code> - The Capacity Reservation request was successful but the capacity provisioning is still pending. </li> <li>  <code>failed</code> - The Capacity Reservation request has failed. A request might fail due to invalid request parameters, capacity constraints, or instance limit constraints. Failed requests are retained for 60 minutes. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EphemeralStorage</i> Indicates whether the Capacity Reservation supports instances with temporary, block-level storage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EbsOptimized</i> Indicates whether the Capacity Reservation supports EBS-optimized instances. This optimization provides dedicated throughput to Amazon EBS and an optimized configuration stack to provide optimal I/O performance. This optimization isn't available with all instance types. Additional usage charges apply when using an EBS- optimized instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailableInstanceCount</i> The remaining capacity. Indicates the number of instances that can be launched in the Capacity Reservation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalInstanceCount</i> The number of instances for which the Capacity Reservation reserves capacity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tenancy</i> Indicates the tenancy of the Capacity Reservation. A Capacity Reservation can have one of the following tenancy settings: <ul> <li>  <code>default</code> - The Capacity Reservation is created on hardware that is shared with other AWS accounts. </li> <li>  <code>dedicated</code> - The Capacity Reservation is created on single-tenant hardware that is dedicated to a single AWS account. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone in which the capacity is reserved.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstancePlatform</i> The type of operating system for which the Capacity Reservation reserves capacity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i> The type of instance for which the Capacity Reservation reserves capacity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CapacityReservationId</i> The ID of the Capacity Reservation.<br/>        """
        pass

    def create_client_vpn_endpoint(self, ConnectionLogOptions: ConnectionLogOptions, AuthenticationOptions: List[ClientVpnAuthenticationRequest], ServerCertificateArn: str, ClientCidrBlock: str, DnsServers: List[String]=None, TransportProtocol: str=None, Description: str=None, DryRun: Boolean=None, ClientToken: str=None, TagSpecifications: List[TagSpecification]=None) -> CreateClientVpnEndpointResult:
        """Creates a Client VPN endpoint. A Client VPN endpoint is the resource you create and configure to enable and manage client VPN sessions. It is the destination endpoint at which all client VPN sessions are terminated.

        :param self:
        :param TagSpecifications:  The tags to apply to the Client VPN endpoint during creation.<br/>ClientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Description:  A brief description of the Client VPN endpoint.<br/>TransportProtocol:  The transport protocol to be used by the VPN session. Default value: <code>udp</code> <br/>DnsServers:  Information about the DNS servers to be used for DNS resolution. A Client VPN endpoint can have up to two DNS servers. If no DNS server is specified, the DNS address of the VPC that is to be associated with Client VPN endpoint is used as the DNS server.<br/>ConnectionLogOptions:  Information about the client connection logging options. If you enable client connection logging, data about client connections is sent to a Cloudwatch Logs log stream. The following information is logged: <ul> <li> Client connection requests </li> <li> Client connection results (successful and unsuccessful) </li> <li> Reasons for unsuccessful client connection requests </li> <li> Client connection termination time </li> </ul>:<br/>AuthenticationOptions:  Information about the authentication method to be used to authenticate clients.<br/>ServerCertificateArn:  The ARN of the server certificate. For more information, see the <a href="https://docs.aws.amazon.com/acm/latest/userguide/">AWS Certificate Manager User Guide</a>.<br/>ClientCidrBlock:  The IPv4 address range, in CIDR notation, from which to assign client IP addresses. The address range cannot overlap with the local CIDR of the VPC in which the associated subnet is located, or the routes that you add manually. The address range cannot be changed after the Client VPN endpoint has been created. The CIDR block should be /22 or greater.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudwatchLogStream</i> The name of the CloudWatch Logs log stream to which the connection data is published.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudwatchLogGroup</i> The name of the CloudWatch Logs log group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Indicates whether connection logging is enabled.<br/>
        :return: DnsName:  The DNS name to be used by clients when establishing their VPN session.<br/>Status:  The current state of the Client VPN endpoint.:<br/>ClientVpnEndpointId:  The ID of the Client VPN endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> A message about the status of the Client VPN endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Code</i> The state of the Client VPN endpoint. Possible states include: <ul> <li>  <code>pending-associate</code> - The Client VPN endpoint has been created but no target networks have been associated. The Client VPN endpoint cannot accept connections. </li> <li>  <code>available</code> - The Client VPN endpoint has been created and a target network has been associated. The Client VPN endpoint can accept connections. </li> <li>  <code>deleting</code> - The Client VPN endpoint is being deleted. The Client VPN endpoint cannot accept connections. </li> <li>  <code>deleted</code> - The Client VPN endpoint has been deleted. The Client VPN endpoint cannot accept connections. </li> </ul><br/>        """
        pass

    def create_client_vpn_route(self, TargetVpcSubnetId: str, DestinationCidrBlock: str, ClientVpnEndpointId: str, Description: str=None, ClientToken: str=None, DryRun: Boolean=None) -> CreateClientVpnRouteResult:
        """Adds a route to a network to a Client VPN endpoint. Each Client VPN endpoint has a route table that describes the available destination network routes. Each route in the route table specifies the path for traﬃc to speciﬁc resources or networks.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>ClientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.<br/>Description:  A brief description of the route.<br/>TargetVpcSubnetId:  The ID of the subnet through which you want to route traffic. The specified subnet must be an existing target network of the Client VPN endpoint.<br/>DestinationCidrBlock:  The IPv4 address range, in CIDR notation, of the route destination. For example: <ul> <li> To add a route for Internet access, enter <code>0.0.0.0/0</code>  </li> <li> To add a route for a peered VPC, enter the peered VPC's IPv4 CIDR range </li> <li> To add a route for an on-premises network, enter the AWS Site-to-Site VPN connection's IPv4 CIDR range </li> </ul> Route address ranges cannot overlap with the CIDR range specified for client allocation.<br/>ClientVpnEndpointId:  The ID of the Client VPN endpoint to which to add the route.<br/>
        :return: Status:  The current state of the route.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> A message about the status of the Client VPN endpoint route, if applicable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Code</i> The state of the Client VPN endpoint route.<br/>        """
        pass

    def create_customer_gateway(self, Type: str, PublicIp: str, BgpAsn: int, DryRun: Boolean=None) -> CreateCustomerGatewayResult:
        """Provides information to AWS about your VPN customer gateway device. The customer gateway is the appliance at your end of the VPN connection. (The device on the AWS side of the VPN connection is the virtual private gateway.) You must provide the Internet-routable IP address of the customer gateway's external interface. The IP address must be static and may be behind a device performing network address translation (NAT). For devices that use Border Gateway Protocol (BGP), you can also provide the device's BGP Autonomous System Number (ASN). You can use an existing ASN assigned to your network. If you don't have an ASN already, you can use a private ASN (in the 64512 - 65534 range). <note> Amazon EC2 supports all 2-byte ASN numbers in the range of 1 - 65534, with the exception of 7224, which is reserved in the <code>us-east-1</code> Region, and 9059, which is reserved in the <code>eu-west-1</code> Region. </note> For more information, see <a href="https://docs.aws.amazon.com/vpn/latest/s2svpn/VPC_VPN.html">AWS Site-to-Site VPN</a> in the <i>AWS Site-to-Site VPN User Guide</i>. <important> You cannot create more than one customer gateway with the same VPN type, IP address, and BGP ASN parameter values. If you run an identical request more than one time, the first request creates the customer gateway, and subsequent requests return information about the existing customer gateway. The subsequent requests do not create new customer gateway resources. </important>

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Type:  The type of VPN connection that this customer gateway supports (<code>ipsec.1</code>).<br/>PublicIp:  The Internet-routable IP address for the customer gateway's outside interface. The address must be static.<br/>BgpAsn:  For devices that support BGP, the customer gateway's BGP ASN. Default: 65000<br/>
        :return: CustomerGateway:  Information about the customer gateway.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> Any tags assigned to the customer gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of VPN connection the customer gateway supports (<code>ipsec.1</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The current state of the customer gateway (<code>pending | available | deleting | deleted</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IpAddress</i> The Internet-routable IP address of the customer gateway's outside interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomerGatewayId</i> The ID of the customer gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BgpAsn</i> The customer gateway's Border Gateway Protocol (BGP) Autonomous System Number (ASN).<br/>        """
        pass

    def create_default_subnet(self, AvailabilityZone: str, DryRun: Boolean=None) -> CreateDefaultSubnetResult:
        """Creates a default subnet with a size <code>/20</code> IPv4 CIDR block in the specified Availability Zone in your default VPC. You can have only one default subnet per Availability Zone. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/default-vpc.html#create-default-subnet">Creating a Default Subnet</a> in the <i>Amazon Virtual Private Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>AvailabilityZone:  The Availability Zone in which to create the default subnet.<br/>
        :return: Subnet:  Information about the subnet.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetArn</i> The Amazon Resource Name (ARN) of the subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> Any tags assigned to the subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Ipv6CidrBlockAssociationSet</i> Information about the IPv6 CIDR blocks associated with the subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssignIpv6AddressOnCreation</i> Indicates whether a network interface created in this subnet (including a network interface created by <a>RunInstances</a>) receives an IPv6 address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The ID of the AWS account that owns the subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the VPC the subnet is in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetId</i> The ID of the subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The current state of the subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MapPublicIpOnLaunch</i> Indicates whether instances launched in this subnet receive a public IPv4 address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultForAz</i> Indicates whether this is the default subnet for the Availability Zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CidrBlock</i> The IPv4 CIDR block assigned to the subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailableIpAddressCount</i> The number of unused private IPv4 addresses in the subnet. The IPv4 addresses for any stopped instances are considered unavailable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZoneId</i> The AZ ID of the subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone of the subnet.<br/>        """
        pass

    def create_default_vpc(self, DryRun: Boolean=None) -> CreateDefaultVpcResult:
        """Creates a default VPC with a size <code>/16</code> IPv4 CIDR block and a default subnet in each Availability Zone. For more information about the components of a default VPC, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/default-vpc.html">Default VPC and Default Subnets</a> in the <i>Amazon Virtual Private Cloud User Guide</i>. You cannot specify the components of the default VPC yourself. If you deleted your previous default VPC, you can create a default VPC. You cannot have more than one default VPC per Region. If your account supports EC2-Classic, you cannot use this action to create a default VPC in a Region that supports EC2-Classic. If you want a default VPC in a Region that supports EC2-Classic, see "I really want a default VPC for my existing EC2 account. Is that possible?" in the <a href="http://aws.amazon.com/vpc/faqs/#Default_VPCs">Default VPCs FAQ</a>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: Vpc:  Information about the VPC.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> Any tags assigned to the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsDefault</i> Indicates whether the VPC is the default VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CidrBlockAssociationSet</i> Information about the IPv4 CIDR blocks associated with the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Ipv6CidrBlockAssociationSet</i> Information about the IPv6 CIDR blocks associated with the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceTenancy</i> The allowed tenancy of instances launched into the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The ID of the AWS account that owns the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The current state of the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DhcpOptionsId</i> The ID of the set of DHCP options you've associated with the VPC (or <code>default</code> if the default options are associated with the VPC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CidrBlock</i> The primary IPv4 CIDR block for the VPC.<br/>        """
        pass

    def create_dhcp_options(self, DhcpConfigurations: List[NewDhcpConfiguration], DryRun: Boolean=None) -> CreateDhcpOptionsResult:
        """Creates a set of DHCP options for your VPC. After creating the set, you must associate it with the VPC, causing all existing and new instances that you launch in the VPC to use this set of DHCP options. The following are the individual DHCP options you can specify. For more information about the options, see <a href="http://www.ietf.org/rfc/rfc2132.txt">RFC 2132</a>. <ul> <li>  <code>domain-name-servers</code> - The IP addresses of up to four domain name servers, or AmazonProvidedDNS. The default DHCP option set specifies AmazonProvidedDNS. If specifying more than one domain name server, specify the IP addresses in a single parameter, separated by commas. ITo have your instance to receive a custom DNS hostname as specified in <code>domain-name</code>, you must set <code>domain-name-servers</code> to a custom DNS server. </li> <li>  <code>domain-name</code> - If you're using AmazonProvidedDNS in <code>us-east-1</code>, specify <code>ec2.internal</code>. If you're using AmazonProvidedDNS in another Region, specify <code>region.compute.internal</code> (for example, <code>ap-northeast-1.compute.internal</code>). Otherwise, specify a domain name (for example, <code>MyCompany.com</code>). This value is used to complete unqualified DNS hostnames. <b>Important</b>: Some Linux operating systems accept multiple domain names separated by spaces. However, Windows and other Linux operating systems treat the value as a single domain, which results in unexpected behavior. If your DHCP options set is associated with a VPC that has instances with multiple operating systems, specify only one domain name. </li> <li>  <code>ntp-servers</code> - The IP addresses of up to four Network Time Protocol (NTP) servers. </li> <li>  <code>netbios-name-servers</code> - The IP addresses of up to four NetBIOS name servers. </li> <li>  <code>netbios-node-type</code> - The NetBIOS node type (1, 2, 4, or 8). We recommend that you specify 2 (broadcast and multicast are not currently supported). For more information about these node types, see <a href="http://www.ietf.org/rfc/rfc2132.txt">RFC 2132</a>. </li> </ul> Your VPC automatically starts out with a set of DHCP options that includes only a DNS server that we provide (AmazonProvidedDNS). If you create a set of options, and if your VPC has an internet gateway, make sure to set the <code>domain-name-servers</code> option either to <code>AmazonProvidedDNS</code> or to a domain name server of your choice. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_DHCP_Options.html">DHCP Options Sets</a> in the <i>Amazon Virtual Private Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>DhcpConfigurations:  A DHCP configuration option.<br/>
        :return: DhcpOptions:  A set of DHCP options.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> Any tags assigned to the DHCP options set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The ID of the AWS account that owns the DHCP options set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DhcpOptionsId</i> The ID of the set of DHCP options.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DhcpConfigurations</i> One or more DHCP options in the set.<br/>        """
        pass

    def create_egress_only_internet_gateway(self, VpcId: str, ClientToken: str=None, DryRun: Boolean=None) -> CreateEgressOnlyInternetGatewayResult:
        """[IPv6 only] Creates an egress-only internet gateway for your VPC. An egress-only internet gateway is used to enable outbound communication over IPv6 from instances in your VPC to the internet, and prevents hosts outside of your VPC from initiating an IPv6 connection with your instance.

        :param self:
        :param VpcId:  The ID of the VPC for which to create the egress-only internet gateway.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>ClientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.<br/>
        :return: EgressOnlyInternetGateway:  Information about the egress-only internet gateway.:<br/>ClientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EgressOnlyInternetGatewayId</i> The ID of the egress-only internet gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Attachments</i> Information about the attachment of the egress-only internet gateway.<br/>        """
        pass

    def create_fleet(self, TargetCapacitySpecification: TargetCapacitySpecificationRequest, LaunchTemplateConfigs: List[FleetLaunchTemplateConfigRequest], DryRun: Boolean=None, ClientToken: str=None, SpotOptions: SpotOptionsRequest=None, OnDemandOptions: OnDemandOptionsRequest=None, ExcessCapacityTerminationPolicy: str=None, TerminateInstancesWithExpiration: Boolean=None, Type: str=None, ValidFrom: DateTime=None, ValidUntil: DateTime=None, ReplaceUnhealthyInstances: Boolean=None, TagSpecifications: List[TagSpecification]=None) -> CreateFleetResult:
        """Launches an EC2 Fleet. You can create a single EC2 Fleet that includes multiple launch specifications that vary by instance type, AMI, Availability Zone, or subnet. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-fleet.html">Launching an EC2 Fleet</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param TagSpecifications:  The key-value pair for tagging the EC2 Fleet request on creation. The value for <code>ResourceType</code> must be <code>fleet</code>, otherwise the fleet request fails. To tag instances at launch, specify the tags in the <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-launch-templates.html#create-launch-template">launch template</a>. For information about tagging after launch, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Using_Tags.html#tag-resources">Tagging Your Resources</a>. <br/>ReplaceUnhealthyInstances:  Indicates whether EC2 Fleet should replace unhealthy instances.<br/>ValidUntil:  The end date and time of the request, in UTC format (for example, <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z). At this point, no new EC2 Fleet requests are placed or able to fulfill the request. If no value is specified, the request remains until you cancel it.<br/>ValidFrom:  The start date and time of the request, in UTC format (for example, <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z). The default is to start fulfilling the request immediately.<br/>Type:  The type of the request. By default, the EC2 Fleet places an asynchronous request for your desired capacity, and maintains it by replenishing interrupted Spot Instances (<code>maintain</code>). A value of <code>instant</code> places a synchronous one-time request, and returns errors for any instances that could not be launched. A value of <code>request</code> places an asynchronous one-time request without maintaining capacity or submitting requests in alternative capacity pools if capacity is unavailable. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-fleet-configuration-strategies.html#ec2-fleet-request-type">EC2 Fleet Request Types</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.<br/>TerminateInstancesWithExpiration:  Indicates whether running instances should be terminated when the EC2 Fleet expires.<br/>TargetCapacitySpecification:  The number of units to request.:<br/>LaunchTemplateConfigs:  The configuration for the EC2 Fleet.<br/>ExcessCapacityTerminationPolicy:  Indicates whether running instances should be terminated if the total target capacity of the EC2 Fleet is decreased below the current size of the EC2 Fleet.<br/>OnDemandOptions:  Describes the configuration of On-Demand Instances in an EC2 Fleet.:<br/>SpotOptions:  Describes the configuration of Spot Instances in an EC2 Fleet.:<br/>ClientToken:  Unique, case-sensitive identifier you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">Ensuring Idempotency</a>.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxTotalPrice</i> The maximum amount per hour for Spot Instances that you're willing to pay.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MinTargetCapacity</i> The minimum target capacity for Spot Instances in the fleet. If the minimum target capacity is not reached, the fleet launches no instances.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SingleAvailabilityZone</i> Indicates that the fleet launches all Spot Instances into a single Availability Zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SingleInstanceType</i> Indicates that the fleet uses a single instance type to launch all Spot Instances in the fleet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstancePoolsToUseCount</i> The number of Spot pools across which to allocate your target Spot capacity. Valid only when Spot <b>AllocationStrategy</b> is set to <code>lowest-price</code>. EC2 Fleet selects the cheapest Spot pools and evenly allocates your target Spot capacity across the number of Spot pools that you specify.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceInterruptionBehavior</i> The behavior when a Spot Instance is interrupted. The default is <code>terminate</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocationStrategy</i> Indicates how to allocate the target capacity across the Spot pools specified by the Spot Fleet request. The default is <code>lowestPrice</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxTotalPrice</i> The maximum amount per hour for On-Demand Instances that you're willing to pay.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MinTargetCapacity</i> The minimum target capacity for On-Demand Instances in the fleet. If the minimum target capacity is not reached, the fleet launches no instances.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SingleAvailabilityZone</i> Indicates that the fleet launches all On-Demand Instances into a single Availability Zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SingleInstanceType</i> Indicates that the fleet uses a single instance type to launch all On-Demand Instances in the fleet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocationStrategy</i> The order of the launch template overrides to use in fulfilling On-Demand capacity. If you specify <code>lowest-price</code>, EC2 Fleet uses price to determine the order, launching the lowest price first. If you specify <code>prioritized</code>, EC2 Fleet uses the priority that you assigned to each launch template override, launching the highest priority first. If you do not specify a value, EC2 Fleet defaults to <code>lowest-price</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultTargetCapacityType</i> The default <code>TotalTargetCapacity</code>, which is either <code>Spot</code> or <code>On-Demand</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SpotTargetCapacity</i> The number of Spot units to request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OnDemandTargetCapacity</i> The number of On-Demand units to request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalTargetCapacity</i> The number of units to request, filled using <code>DefaultTargetCapacityType</code>.<br/>
        :return: Instances:  Information about the instances that were launched by the fleet. Valid only when <b>Type</b> is set to <code>instant</code>.<br/>Errors:  Information about the instances that could not be launched by the fleet. Valid only when <b>Type</b> is set to <code>instant</code>.<br/>FleetId:  The ID of the EC2 Fleet.<br/>        """
        pass

    def create_flow_logs(self, TrafficType: str, ResourceType: str, ResourceIds: List[String], DryRun: Boolean=None, ClientToken: str=None, DeliverLogsPermissionArn: str=None, LogGroupName: str=None, LogDestinationType: str=None, LogDestination: str=None) -> CreateFlowLogsResult:
        """Creates one or more flow logs to capture information about IP traffic for a specific network interface, subnet, or VPC.  Flow log data for a monitored network interface is recorded as flow log records, which are log events consisting of fields that describe the traffic flow. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/flow-logs.html#flow-log-records">Flow Log Records</a> in the <i>Amazon Virtual Private Cloud User Guide</i>. When publishing to CloudWatch Logs, flow log records are published to a log group, and each network interface has a unique log stream in the log group. When publishing to Amazon S3, flow log records for all of the monitored network interfaces are published to a single log file object that is stored in the specified bucket. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/flow-logs.html">VPC Flow Logs</a> in the <i>Amazon Virtual Private Cloud User Guide</i>.

        :param self:
        :param LogDestination:  Specifies the destination to which the flow log data is to be published. Flow log data can be published to a CloudWatch Logs log group or an Amazon S3 bucket. The value specified for this parameter depends on the value specified for <code>LogDestinationType</code>. If LogDestinationType is not specified or <code>cloud-watch-logs</code>, specify the Amazon Resource Name (ARN) of the CloudWatch Logs log group. If LogDestinationType is <code>s3</code>, specify the ARN of the Amazon S3 bucket. You can also specify a subfolder in the bucket. To specify a subfolder in the bucket, use the following ARN format: <code>bucket_ARN/subfolder_name/</code>. For example, to specify a subfolder named <code>my-logs</code> in a bucket named <code>my-bucket</code>, use the following ARN: <code>arn:aws:s3:::my-bucket/my-logs/</code>. You cannot use <code>AWSLogs</code> as a subfolder name. This is a reserved term.<br/>LogDestinationType:  Specifies the type of destination to which the flow log data is to be published. Flow log data can be published to CloudWatch Logs or Amazon S3. To publish flow log data to CloudWatch Logs, specify <code>cloud-watch-logs</code>. To publish flow log data to Amazon S3, specify <code>s3</code>. If you specify <code>LogDestinationType</code> as <code>s3</code>, do not specify <code>DeliverLogsPermissionArn</code> or <code>LogGroupName</code>. Default: <code>cloud-watch-logs</code> <br/>TrafficType:  The type of traffic to log. You can log traffic that the resource accepts or rejects, or all traffic.<br/>ResourceType:  The type of resource for which to create the flow log. For example, if you specified a VPC ID for the <code>ResourceId</code> property, specify <code>VPC</code> for this property.<br/>ResourceIds:  The ID of the subnet, network interface, or VPC for which you want to create a flow log. Constraints: Maximum of 1000 resources<br/>LogGroupName:  The name of a new or existing CloudWatch Logs log group where Amazon EC2 publishes your flow logs. If you specify <code>LogDestinationType</code> as <code>s3</code>, do not specify <code>DeliverLogsPermissionArn</code> or <code>LogGroupName</code>.<br/>DeliverLogsPermissionArn:  The ARN for the IAM role that permits Amazon EC2 to publish flow logs to a CloudWatch Logs log group in your account. If you specify <code>LogDestinationType</code> as <code>s3</code>, do not specify <code>DeliverLogsPermissionArn</code> or <code>LogGroupName</code>.<br/>ClientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: Unsuccessful:  Information about the flow logs that could not be created successfully.<br/>FlowLogIds:  The IDs of the flow logs.<br/>ClientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request.<br/>        """
        pass

    def create_fpga_image(self, InputStorageLocation: StorageLocation, DryRun: Boolean=None, LogsStorageLocation: StorageLocation=None, Description: str=None, Name: str=None, ClientToken: str=None) -> CreateFpgaImageResult:
        """Creates an Amazon FPGA Image (AFI) from the specified design checkpoint (DCP). The create operation is asynchronous. To verify that the AFI is ready for use, check the output logs. An AFI contains the FPGA bitstream that is ready to download to an FPGA. You can securely deploy an AFI on multiple FPGA-accelerated instances. For more information, see the <a href="https://github.com/aws/aws-fpga/">AWS FPGA Hardware Development Kit</a>.

        :param self:
        :param ClientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Run_Instance_Idempotency.html">Ensuring Idempotency</a>.<br/>Name:  A name for the AFI.<br/>Description:  A description for the AFI.<br/>LogsStorageLocation:  The location in Amazon S3 for the output logs.:<br/>InputStorageLocation:  The location of the encrypted design checkpoint in Amazon S3. The input must be a tarball.:<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> The key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> The name of the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> The key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> The name of the S3 bucket.<br/>
        :return: FpgaImageGlobalId:  The global FPGA image identifier (AGFI ID).<br/>FpgaImageId:  The FPGA image identifier (AFI ID).<br/>        """
        pass

    def create_image(self, Name: str, InstanceId: str, BlockDeviceMappings: List[BlockDeviceMapping]=None, Description: str=None, DryRun: Boolean=None, NoReboot: Boolean=None) -> CreateImageResult:
        """Creates an Amazon EBS-backed AMI from an Amazon EBS-backed instance that is either running or stopped. If you customized your instance with instance store volumes or EBS volumes in addition to the root device volume, the new AMI contains block device mapping information for those volumes. When you launch an instance from this new AMI, the instance automatically launches with those additional volumes. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/creating-an-ami-ebs.html">Creating Amazon EBS-Backed Linux AMIs</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param NoReboot:  By default, Amazon EC2 attempts to shut down and reboot the instance before creating the image. If the 'No Reboot' option is set, Amazon EC2 doesn't shut down the instance before creating the image. When this option is used, file system integrity on the created image can't be guaranteed.<br/>Name:  A name for the new image. Constraints: 3-128 alphanumeric characters, parentheses (()), square brackets ([]), spaces ( ), periods (.), slashes (/), dashes (-), single quotes ('), at-signs (@), or underscores(_)<br/>InstanceId:  The ID of the instance.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Description:  A description for the new image.<br/>BlockDeviceMappings:  The block device mappings. This parameter cannot be used to modify the encryption status of existing volumes or snapshots. To create an AMI with encrypted snapshots, use the <a>CopyImage</a> action.<br/>
        :return: ImageId:  The ID of the new AMI.<br/>        """
        pass

    def create_instance_export_task(self, InstanceId: str, Description: str=None, ExportToS3Task: ExportToS3TaskSpecification=None, TargetEnvironment: str=None) -> CreateInstanceExportTaskResult:
        """Exports a running or stopped instance to an S3 bucket. For information about the supported operating systems, image formats, and known limitations for the types of instances you can export, see <a href="https://docs.aws.amazon.com/vm-import/latest/userguide/vmexport.html">Exporting an Instance as a VM Using VM Import/Export</a> in the <i>VM Import/Export User Guide</i>.

        :param self:
        :param TargetEnvironment:  The target virtualization environment.<br/>InstanceId:  The ID of the instance.<br/>ExportToS3Task:  The format and location for an instance export task.:<br/>Description:  A description for the conversion task or the resource being exported. The maximum length is 255 bytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Prefix</i> The image is written to a single object in the S3 bucket at the S3 key s3prefix + exportTaskId + '.' + diskImageFormat.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Bucket</i> The S3 bucket for the destination image. The destination bucket must exist and grant WRITE and READ_ACP permissions to the AWS account <code>vm-import-export@amazon.com</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DiskImageFormat</i> The format for the exported image.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContainerFormat</i> The container format used to combine disk images with metadata (such as OVF). If absent, only the disk image is exported.<br/>
        :return: ExportTask:  Information about the instance export task.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> The status message related to the export task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the export task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceExportDetails</i> Information about the instance to export.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InstanceExportDetails</b>: Information about the instance to export.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExportToS3Task</i> Information about the export task.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ExportToS3Task</b>: Information about the export task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExportTaskId</i> The ID of the export task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of the resource being exported.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Key</i> The encryption key for your S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Bucket</i> The S3 bucket for the destination image. The destination bucket must exist and grant WRITE and READ_ACP permissions to the AWS account <code>vm-import-export@amazon.com</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DiskImageFormat</i> The format for the exported image.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ContainerFormat</i> The container format used to combine disk images with metadata (such as OVF). If absent, only the disk image is exported.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetEnvironment</i> The target virtualization environment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceId</i> The ID of the resource being exported.<br/>        """
        pass

    def create_internet_gateway(self, DryRun: Boolean=None) -> CreateInternetGatewayResult:
        """Creates an internet gateway for use with a VPC. After creating the internet gateway, you attach it to a VPC using <a>AttachInternetGateway</a>. For more information about your VPC and internet gateway, see the <a href="https://docs.aws.amazon.com/vpc/latest/userguide/">Amazon Virtual Private Cloud User Guide</a>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: InternetGateway:  Information about the internet gateway.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> Any tags assigned to the internet gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The ID of the AWS account that owns the internet gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InternetGatewayId</i> The ID of the internet gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Attachments</i> Any VPCs attached to the internet gateway.<br/>        """
        pass

    def create_key_pair(self, KeyName: str, DryRun: Boolean=None) -> KeyPair:
        """Creates a 2048-bit RSA key pair with the specified name. Amazon EC2 stores the public key and displays the private key for you to save to a file. The private key is returned as an unencrypted PEM encoded PKCS#1 private key. If a key with the specified name already exists, Amazon EC2 returns an error. You can have up to five thousand key pairs per Region. The key pair returned to you is available only in the Region in which you create it. If you prefer, you can create your own key pair using a third-party tool and upload it to any Region using <a>ImportKeyPair</a>. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-key-pairs.html">Key Pairs</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>KeyName:  A unique name for the key pair. Constraints: Up to 255 ASCII characters<br/>
        :return: KeyName:  The name of the key pair.<br/>KeyMaterial:  An unencrypted PEM encoded RSA private key.<br/>KeyFingerprint:  The SHA-1 digest of the DER encoded private key.<br/>        """
        pass

    def create_launch_template(self, LaunchTemplateData: RequestLaunchTemplateData, LaunchTemplateName: str, DryRun: Boolean=None, ClientToken: str=None, VersionDescription: str=None) -> CreateLaunchTemplateResult:
        """Creates a launch template. A launch template contains the parameters to launch an instance. When you launch an instance using <a>RunInstances</a>, you can specify a launch template instead of providing the launch parameters in the request.

        :param self:
        :param LaunchTemplateData:  The information for the launch template.:<br/>VersionDescription:  A description for the first version of the launch template.<br/>LaunchTemplateName:  A name for the launch template.<br/>ClientToken:  Unique, case-sensitive identifier you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">Ensuring Idempotency</a>. Constraint: Maximum 128 ASCII characters.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HibernationOptions</i> Indicates whether an instance is enabled for hibernation. This parameter is valid only if the instance meets the <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Hibernate.html#hibernating-prerequisites">hibernation prerequisites</a>. Hibernation is currently supported only for Amazon Linux. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Hibernate.html">Hibernate Your Instance</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>HibernationOptions</b>: Indicates whether an instance is enabled for hibernation. This parameter is valid only if the instance meets the <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Hibernate.html#hibernating-prerequisites">hibernation prerequisites</a>. Hibernation is currently supported only for Amazon Linux. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Hibernate.html">Hibernate Your Instance</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseSpecifications</i> The license configurations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CapacityReservationSpecification</i> The Capacity Reservation targeting option. If you do not specify this parameter, the instance's Capacity Reservation preference defaults to <code>open</code>, which enables it to run in any open Capacity Reservation that has matching attributes (instance type, platform, Availability Zone).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CapacityReservationSpecification</b>: The Capacity Reservation targeting option. If you do not specify this parameter, the instance's Capacity Reservation preference defaults to <code>open</code>, which enables it to run in any open Capacity Reservation that has matching attributes (instance type, platform, Availability Zone).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CpuOptions</i> The CPU options for the instance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-optimize-cpu.html">Optimizing CPU Options</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CpuOptions</b>: The CPU options for the instance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-optimize-cpu.html">Optimizing CPU Options</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreditSpecification</i> The credit option for CPU usage of the instance. Valid for T2 or T3 instances only.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CreditSpecification</b>: The credit option for CPU usage of the instance. Valid for T2 or T3 instances only.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceMarketOptions</i> The market (purchasing) option for the instances.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InstanceMarketOptions</b>: The market (purchasing) option for the instances.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroups</i> [EC2-Classic, default VPC] One or more security group names. For a nondefault VPC, you must use security group IDs instead. You cannot specify both a security group ID and security name in the same request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> One or more security group IDs. You can create a security group using <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateSecurityGroup.html">CreateSecurityGroup</a>. You cannot specify both a security group ID and security name in the same request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticInferenceAccelerators</i>  The elastic inference accelerator for the instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticGpuSpecifications</i> An elastic GPU to associate with the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TagSpecifications</i> The tags to apply to the resources during launch. You can only tag instances and volumes on launch. The specified tags are applied to all instances or volumes that are created during launch. To tag a resource after it has been created, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateTags.html">CreateTags</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserData</i> The Base64-encoded user data to make available to the instance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/user-data.html">Running Commands on Your Linux Instance at Launch</a> (Linux) and <a href="https://docs.aws.amazon.com/AWSEC2/latest/WindowsGuide/ec2-instance-metadata.html#instancedata-add-user-data">Adding User Data</a> (Windows).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceInitiatedShutdownBehavior</i> Indicates whether an instance stops or terminates when you initiate shutdown from the instance (using the operating system command for system shutdown). Default: <code>stop</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisableApiTermination</i> If you set this parameter to <code>true</code>, you can't terminate the instance using the Amazon EC2 console, CLI, or API; otherwise, you can. To change this attribute after launch, use <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_ModifyInstanceAttribute.html">ModifyInstanceAttribute</a>. Alternatively, if you set <code>InstanceInitiatedShutdownBehavior</code> to <code>terminate</code>, you can terminate the instance by running the shutdown command from the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RamDiskId</i> The ID of the RAM disk. <important> We recommend that you use PV-GRUB instead of kernels and RAM disks. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/UserProvidedkernels.html">User Provided Kernels</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Placement</i> The placement for the instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Placement</b>: The placement for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Monitoring</i> The monitoring for the instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Monitoring</b>: The monitoring for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyName</i> The name of the key pair. You can create a key pair using <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateKeyPair.html">CreateKeyPair</a> or <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_ImportKeyPair.html">ImportKeyPair</a>. <important> If you do not specify a key pair, you can't connect to the instance unless you choose an AMI that is configured to allow users another way to log in. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i> The instance type. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html">Instance Types</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImageId</i> The ID of the AMI.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkInterfaces</i> One or more network interfaces. If you specify a network interface, you must specify any security groups and subnets as part of the network interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BlockDeviceMappings</i> The block device mapping. <important> Supplying both a snapshot ID and an encryption value as arguments for block-device mapping results in an error. This is because only blank volumes can be encrypted on start, and these are not created from a snapshot. If a snapshot is the basis for the volume, it contains data by definition and its encryption status cannot be changed using this action. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamInstanceProfile</i> The IAM instance profile.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>IamInstanceProfile</b>: The IAM instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EbsOptimized</i> Indicates whether the instance is optimized for Amazon EBS I/O. This optimization provides dedicated throughput to Amazon EBS and an optimized configuration stack to provide optimal Amazon EBS I/O performance. This optimization isn't available with all instance types. Additional usage charges apply when using an EBS-optimized instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KernelId</i> The ID of the kernel. <important> We recommend that you use PV-GRUB instead of kernels and RAM disks. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/UserProvidedkernels.html">User Provided Kernels</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specify <code>true</code> to enable detailed monitoring. Otherwise, basic monitoring is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SpreadDomain</i> Reserved for future use.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Tenancy</i> The tenancy of the instance (if the instance is running in a VPC). An instance with a tenancy of dedicated runs on single-tenant hardware.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostId</i> The ID of the Dedicated Host for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupName</i> The name of the placement group for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Affinity</i> The affinity setting for an instance on a Dedicated Host.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MarketType</i> The market type.<br/>The options for Spot Instances.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpotOptions</b>: The options for Spot Instances.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CpuCredits</i> The credit option for CPU usage of a T2 or T3 instance. Valid values are <code>standard</code> and <code>unlimited</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ThreadsPerCore</i> The number of threads per CPU core. To disable Intel Hyper-Threading Technology for the instance, specify a value of 1. Otherwise, specify the default value of 2.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CoreCount</i> The number of CPU cores for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CapacityReservationPreference</i> Indicates the instance's Capacity Reservation preferences. Possible preferences include: <ul> <li>  <code>open</code> - The instance can run in any <code>open</code> Capacity Reservation that has matching attributes (instance type, platform, Availability Zone). </li> <li>  <code>none</code> - The instance avoids running in a Capacity Reservation even if one is available. The instance runs in On-Demand capacity. </li> </ul><br/>Information about the target Capacity Reservation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CapacityReservationTarget</b>: Information about the target Capacity Reservation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Configured</i> If you set this parameter to <code>true</code>, the instance is enabled for hibernation. Default: <code>false</code> <br/>
        :return: LaunchTemplate:  Information about the launch template.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The tags for the launch template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestVersionNumber</i> The version number of the latest version of the launch template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultVersionNumber</i> The version number of the default version of the launch template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedBy</i> The principal that created the launch template. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateTime</i> The time launch template was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LaunchTemplateName</i> The name of the launch template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LaunchTemplateId</i> The ID of the launch template.<br/>        """
        pass

    def create_launch_template_version(self, LaunchTemplateData: RequestLaunchTemplateData, DryRun: Boolean=None, ClientToken: str=None, LaunchTemplateId: str=None, LaunchTemplateName: str=None, SourceVersion: str=None, VersionDescription: str=None) -> CreateLaunchTemplateVersionResult:
        """Creates a new version for a launch template. You can specify an existing version of launch template from which to base the new version. Launch template versions are numbered in the order in which they are created. You cannot specify, change, or replace the numbering of launch template versions.

        :param self:
        :param LaunchTemplateData:  The information for the launch template.:<br/>VersionDescription:  A description for the version of the launch template.<br/>SourceVersion:  The version number of the launch template version on which to base the new version. The new version inherits the same launch parameters as the source version, except for parameters that you specify in LaunchTemplateData.<br/>LaunchTemplateName:  The name of the launch template. You must specify either the launch template ID or launch template name in the request.<br/>LaunchTemplateId:  The ID of the launch template. You must specify either the launch template ID or launch template name in the request.<br/>ClientToken:  Unique, case-sensitive identifier you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">Ensuring Idempotency</a>. Constraint: Maximum 128 ASCII characters.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HibernationOptions</i> Indicates whether an instance is enabled for hibernation. This parameter is valid only if the instance meets the <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Hibernate.html#hibernating-prerequisites">hibernation prerequisites</a>. Hibernation is currently supported only for Amazon Linux. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Hibernate.html">Hibernate Your Instance</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>HibernationOptions</b>: Indicates whether an instance is enabled for hibernation. This parameter is valid only if the instance meets the <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Hibernate.html#hibernating-prerequisites">hibernation prerequisites</a>. Hibernation is currently supported only for Amazon Linux. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Hibernate.html">Hibernate Your Instance</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseSpecifications</i> The license configurations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CapacityReservationSpecification</i> The Capacity Reservation targeting option. If you do not specify this parameter, the instance's Capacity Reservation preference defaults to <code>open</code>, which enables it to run in any open Capacity Reservation that has matching attributes (instance type, platform, Availability Zone).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CapacityReservationSpecification</b>: The Capacity Reservation targeting option. If you do not specify this parameter, the instance's Capacity Reservation preference defaults to <code>open</code>, which enables it to run in any open Capacity Reservation that has matching attributes (instance type, platform, Availability Zone).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CpuOptions</i> The CPU options for the instance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-optimize-cpu.html">Optimizing CPU Options</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CpuOptions</b>: The CPU options for the instance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-optimize-cpu.html">Optimizing CPU Options</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreditSpecification</i> The credit option for CPU usage of the instance. Valid for T2 or T3 instances only.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CreditSpecification</b>: The credit option for CPU usage of the instance. Valid for T2 or T3 instances only.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceMarketOptions</i> The market (purchasing) option for the instances.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InstanceMarketOptions</b>: The market (purchasing) option for the instances.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroups</i> [EC2-Classic, default VPC] One or more security group names. For a nondefault VPC, you must use security group IDs instead. You cannot specify both a security group ID and security name in the same request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> One or more security group IDs. You can create a security group using <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateSecurityGroup.html">CreateSecurityGroup</a>. You cannot specify both a security group ID and security name in the same request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticInferenceAccelerators</i>  The elastic inference accelerator for the instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticGpuSpecifications</i> An elastic GPU to associate with the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TagSpecifications</i> The tags to apply to the resources during launch. You can only tag instances and volumes on launch. The specified tags are applied to all instances or volumes that are created during launch. To tag a resource after it has been created, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateTags.html">CreateTags</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserData</i> The Base64-encoded user data to make available to the instance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/user-data.html">Running Commands on Your Linux Instance at Launch</a> (Linux) and <a href="https://docs.aws.amazon.com/AWSEC2/latest/WindowsGuide/ec2-instance-metadata.html#instancedata-add-user-data">Adding User Data</a> (Windows).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceInitiatedShutdownBehavior</i> Indicates whether an instance stops or terminates when you initiate shutdown from the instance (using the operating system command for system shutdown). Default: <code>stop</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisableApiTermination</i> If you set this parameter to <code>true</code>, you can't terminate the instance using the Amazon EC2 console, CLI, or API; otherwise, you can. To change this attribute after launch, use <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_ModifyInstanceAttribute.html">ModifyInstanceAttribute</a>. Alternatively, if you set <code>InstanceInitiatedShutdownBehavior</code> to <code>terminate</code>, you can terminate the instance by running the shutdown command from the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RamDiskId</i> The ID of the RAM disk. <important> We recommend that you use PV-GRUB instead of kernels and RAM disks. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/UserProvidedkernels.html">User Provided Kernels</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Placement</i> The placement for the instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Placement</b>: The placement for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Monitoring</i> The monitoring for the instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Monitoring</b>: The monitoring for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyName</i> The name of the key pair. You can create a key pair using <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateKeyPair.html">CreateKeyPair</a> or <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_ImportKeyPair.html">ImportKeyPair</a>. <important> If you do not specify a key pair, you can't connect to the instance unless you choose an AMI that is configured to allow users another way to log in. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i> The instance type. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html">Instance Types</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImageId</i> The ID of the AMI.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkInterfaces</i> One or more network interfaces. If you specify a network interface, you must specify any security groups and subnets as part of the network interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BlockDeviceMappings</i> The block device mapping. <important> Supplying both a snapshot ID and an encryption value as arguments for block-device mapping results in an error. This is because only blank volumes can be encrypted on start, and these are not created from a snapshot. If a snapshot is the basis for the volume, it contains data by definition and its encryption status cannot be changed using this action. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamInstanceProfile</i> The IAM instance profile.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>IamInstanceProfile</b>: The IAM instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EbsOptimized</i> Indicates whether the instance is optimized for Amazon EBS I/O. This optimization provides dedicated throughput to Amazon EBS and an optimized configuration stack to provide optimal Amazon EBS I/O performance. This optimization isn't available with all instance types. Additional usage charges apply when using an EBS-optimized instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KernelId</i> The ID of the kernel. <important> We recommend that you use PV-GRUB instead of kernels and RAM disks. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/UserProvidedkernels.html">User Provided Kernels</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specify <code>true</code> to enable detailed monitoring. Otherwise, basic monitoring is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SpreadDomain</i> Reserved for future use.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Tenancy</i> The tenancy of the instance (if the instance is running in a VPC). An instance with a tenancy of dedicated runs on single-tenant hardware.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostId</i> The ID of the Dedicated Host for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupName</i> The name of the placement group for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Affinity</i> The affinity setting for an instance on a Dedicated Host.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MarketType</i> The market type.<br/>The options for Spot Instances.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpotOptions</b>: The options for Spot Instances.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CpuCredits</i> The credit option for CPU usage of a T2 or T3 instance. Valid values are <code>standard</code> and <code>unlimited</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ThreadsPerCore</i> The number of threads per CPU core. To disable Intel Hyper-Threading Technology for the instance, specify a value of 1. Otherwise, specify the default value of 2.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CoreCount</i> The number of CPU cores for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CapacityReservationPreference</i> Indicates the instance's Capacity Reservation preferences. Possible preferences include: <ul> <li>  <code>open</code> - The instance can run in any <code>open</code> Capacity Reservation that has matching attributes (instance type, platform, Availability Zone). </li> <li>  <code>none</code> - The instance avoids running in a Capacity Reservation even if one is available. The instance runs in On-Demand capacity. </li> </ul><br/>Information about the target Capacity Reservation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CapacityReservationTarget</b>: Information about the target Capacity Reservation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Configured</i> If you set this parameter to <code>true</code>, the instance is enabled for hibernation. Default: <code>false</code> <br/>
        :return: LaunchTemplateVersion:  Information about the launch template version.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LaunchTemplateData</i> Information about the launch template.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LaunchTemplateData</b>: Information about the launch template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultVersion</i> Indicates whether the version is the default version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedBy</i> The principal that created the version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateTime</i> The time the version was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VersionDescription</i> The description for the version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VersionNumber</i> The version number.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LaunchTemplateName</i> The name of the launch template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LaunchTemplateId</i> The ID of the launch template.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EbsOptimized</i> Indicates whether the instance is optimized for Amazon EBS I/O. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KernelId</i> The ID of the kernel, if applicable.<br/>The IAM instance profile.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>IamInstanceProfile</b>: The IAM instance profile.        """
        pass

    def create_nat_gateway(self, SubnetId: str, AllocationId: str, ClientToken: str=None) -> CreateNatGatewayResult:
        """Creates a NAT gateway in the specified public subnet. This action creates a network interface in the specified subnet with a private IP address from the IP address range of the subnet. Internet-bound traffic from a private subnet can be routed to the NAT gateway, therefore enabling instances in the private subnet to connect to the internet. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/vpc-nat-gateway.html">NAT Gateways</a> in the <i>Amazon Virtual Private Cloud User Guide</i>.

        :param self:
        :param SubnetId:  The subnet in which to create the NAT gateway.<br/>ClientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>. Constraint: Maximum 64 ASCII characters.<br/>AllocationId:  The allocation ID of an Elastic IP address to associate with the NAT gateway. If the Elastic IP address is associated with another resource, you must first disassociate it.<br/>
        :return: NatGateway:  Information about the NAT gateway.:<br/>ClientToken:  Unique, case-sensitive identifier to ensure the idempotency of the request. Only returned if a client token was provided in the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The tags for the NAT gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the VPC in which the NAT gateway is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetId</i> The ID of the subnet in which the NAT gateway is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the NAT gateway. <ul> <li>  <code>pending</code>: The NAT gateway is being created and is not ready to process traffic. </li> <li>  <code>failed</code>: The NAT gateway could not be created. Check the <code>failureCode</code> and <code>failureMessage</code> fields for the reason. </li> <li>  <code>available</code>: The NAT gateway is able to process traffic. This status remains until you delete the NAT gateway, and does not indicate the health of the NAT gateway. </li> <li>  <code>deleting</code>: The NAT gateway is in the process of being terminated and may still be processing traffic. </li> <li>  <code>deleted</code>: The NAT gateway has been terminated and is no longer processing traffic. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisionedBandwidth</i> Reserved. If you need to sustain traffic greater than the <a href="https://docs.aws.amazon.com/vpc/latest/userguide/vpc-nat-gateway.html">documented limits</a>, contact us through the <a href="https://console.aws.amazon.com/support/home?">Support Center</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ProvisionedBandwidth</b>: Reserved. If you need to sustain traffic greater than the <a href="https://docs.aws.amazon.com/vpc/latest/userguide/vpc-nat-gateway.html">documented limits</a>, contact us through the <a href="https://console.aws.amazon.com/support/home?">Support Center</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NatGatewayId</i> The ID of the NAT gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NatGatewayAddresses</i> Information about the IP addresses and network interface associated with the NAT gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureMessage</i> If the NAT gateway could not be created, specifies the error message for the failure, that corresponds to the error code. <ul> <li> For InsufficientFreeAddressesInSubnet: "Subnet has insufficient free addresses to create this NAT gateway" </li> <li> For Gateway.NotAttached: "Network vpc-xxxxxxxx has no Internet gateway attached" </li> <li> For InvalidAllocationID.NotFound: "Elastic IP address eipalloc-xxxxxxxx could not be associated with this NAT gateway" </li> <li> For Resource.AlreadyAssociated: "Elastic IP address eipalloc-xxxxxxxx is already associated" </li> <li> For InternalError: "Network interface eni-xxxxxxxx, created and used internally by this NAT gateway is in an invalid state. Please try again." </li> <li> For InvalidSubnetID.NotFound: "The specified subnet subnet-xxxxxxxx does not exist or could not be found." </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureCode</i> If the NAT gateway could not be created, specifies the error code for the failure. (<code>InsufficientFreeAddressesInSubnet</code> | <code>Gateway.NotAttached</code> | <code>InvalidAllocationID.NotFound</code> | <code>Resource.AlreadyAssociated</code> | <code>InternalError</code> | <code>InvalidSubnetID.NotFound</code>)<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeleteTime</i> The date and time the NAT gateway was deleted, if applicable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateTime</i> The date and time the NAT gateway was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Reserved. If you need to sustain traffic greater than the <a href="https://docs.aws.amazon.com/vpc/latest/userguide/vpc-nat-gateway.html">documented limits</a>, contact us through the <a href="https://console.aws.amazon.com/support/home?">Support Center</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Requested</i> Reserved. If you need to sustain traffic greater than the <a href="https://docs.aws.amazon.com/vpc/latest/userguide/vpc-nat-gateway.html">documented limits</a>, contact us through the <a href="https://console.aws.amazon.com/support/home?">Support Center</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestTime</i> Reserved. If you need to sustain traffic greater than the <a href="https://docs.aws.amazon.com/vpc/latest/userguide/vpc-nat-gateway.html">documented limits</a>, contact us through the <a href="https://console.aws.amazon.com/support/home?">Support Center</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Provisioned</i> Reserved. If you need to sustain traffic greater than the <a href="https://docs.aws.amazon.com/vpc/latest/userguide/vpc-nat-gateway.html">documented limits</a>, contact us through the <a href="https://console.aws.amazon.com/support/home?">Support Center</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisionTime</i> Reserved. If you need to sustain traffic greater than the <a href="https://docs.aws.amazon.com/vpc/latest/userguide/vpc-nat-gateway.html">documented limits</a>, contact us through the <a href="https://console.aws.amazon.com/support/home?">Support Center</a>.<br/>        """
        pass

    def create_network_acl(self, VpcId: str, DryRun: Boolean=None) -> CreateNetworkAclResult:
        """Creates a network ACL in a VPC. Network ACLs provide an optional layer of security (in addition to security groups) for the instances in your VPC. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_ACLs.html">Network ACLs</a> in the <i>Amazon Virtual Private Cloud User Guide</i>.

        :param self:
        :param VpcId:  The ID of the VPC.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: NetworkAcl:  Information about the network ACL.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The ID of the AWS account that owns the network ACL.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the VPC for the network ACL.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> Any tags assigned to the network ACL.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkAclId</i> The ID of the network ACL.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsDefault</i> Indicates whether this is the default network ACL for the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Entries</i> One or more entries (rules) in the network ACL.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Associations</i> Any associations between the network ACL and one or more subnets<br/>        """
        pass

    def create_network_acl_entry(self, RuleNumber: int, RuleAction: str, Protocol: str, NetworkAclId: str, Egress: Boolean, CidrBlock: str=None, DryRun: Boolean=None, IcmpTypeCode: IcmpTypeCode=None, Ipv6CidrBlock: str=None, PortRange: PortRange=None):
        """Creates an entry (a rule) in a network ACL with the specified rule number. Each network ACL has a set of numbered ingress rules and a separate set of numbered egress rules. When determining whether a packet should be allowed in or out of a subnet associated with the ACL, we process the entries in the ACL according to the rule numbers, in ascending order. Each network ACL has a set of ingress rules and a separate set of egress rules. We recommend that you leave room between the rule numbers (for example, 100, 110, 120, ...), and not number them one right after the other (for example, 101, 102, 103, ...). This makes it easier to add a rule between existing ones without having to renumber the rules. After you add an entry, you can't modify it; you must either replace it, or create an entry and delete the old one. For more information about network ACLs, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_ACLs.html">Network ACLs</a> in the <i>Amazon Virtual Private Cloud User Guide</i>.

        :param self:
        :param RuleNumber:  The rule number for the entry (for example, 100). ACL entries are processed in ascending order by rule number. Constraints: Positive integer from 1 to 32766. The range 32767 to 65535 is reserved for internal use.<br/>RuleAction:  Indicates whether to allow or deny the traffic that matches the rule.<br/>Protocol:  The protocol number. A value of "-1" means all protocols. If you specify "-1" or a protocol number other than "6" (TCP), "17" (UDP), or "1" (ICMP), traffic on all ports is allowed, regardless of any ports or ICMP types or codes that you specify. If you specify protocol "58" (ICMPv6) and specify an IPv4 CIDR block, traffic for all ICMP types and codes allowed, regardless of any that you specify. If you specify protocol "58" (ICMPv6) and specify an IPv6 CIDR block, you must specify an ICMP type and code.<br/>PortRange:  TCP or UDP protocols: The range of ports the rule applies to. Required if specifying protocol 6 (TCP) or 17 (UDP).:<br/>NetworkAclId:  The ID of the network ACL.<br/>Ipv6CidrBlock:  The IPv6 network range to allow or deny, in CIDR notation (for example <code>2001:db8:1234:1a00::/64</code>).<br/>IcmpTypeCode:  ICMP protocol: The ICMP or ICMPv6 type and code. Required if specifying protocol 1 (ICMP) or protocol 58 (ICMPv6) with an IPv6 CIDR block.:<br/>Egress:  Indicates whether this is an egress rule (rule is applied to traffic leaving the subnet).<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>CidrBlock:  The IPv4 network range to allow or deny, in CIDR notation (for example <code>172.16.0.0/24</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The ICMP type. A value of -1 means all types.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Code</i> The ICMP code. A value of -1 means all codes for the specified ICMP type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>To</i> The last port in the range.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>From</i> The first port in the range.<br/>
        :return:         """
        pass

    def create_network_interface(self, SubnetId: str, Description: str=None, DryRun: Boolean=None, Groups: List[String]=None, Ipv6AddressCount: int=None, Ipv6Addresses: List[InstanceIpv6Address]=None, PrivateIpAddress: str=None, PrivateIpAddresses: List[PrivateIpAddressSpecification]=None, SecondaryPrivateIpAddressCount: int=None, InterfaceType: str=None) -> CreateNetworkInterfaceResult:
        """Creates a network interface in the specified subnet. For more information about network interfaces, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/using-eni.html">Elastic Network Interfaces</a> in the <i>Amazon Virtual Private Cloud User Guide</i>.

        :param self:
        :param SubnetId:  The ID of the subnet to associate with the network interface.<br/>InterfaceType:  Indicates the type of network interface. To create an Elastic Fabric Adapter (EFA), specify <code>efa</code>. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/efa.html"> Elastic Fabric Adapter</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.<br/>SecondaryPrivateIpAddressCount:  The number of secondary private IPv4 addresses to assign to a network interface. When you specify a number of secondary IPv4 addresses, Amazon EC2 selects these IP addresses within the subnet's IPv4 CIDR range. You can't specify this option and specify more than one private IP address using <code>privateIpAddresses</code>. The number of IP addresses you can assign to a network interface varies by instance type. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/using-eni.html#AvailableIpPerENI">IP Addresses Per ENI Per Instance Type</a> in the <i>Amazon Virtual Private Cloud User Guide</i>.<br/>PrivateIpAddresses:  One or more private IPv4 addresses.<br/>PrivateIpAddress:  The primary private IPv4 address of the network interface. If you don't specify an IPv4 address, Amazon EC2 selects one for you from the subnet's IPv4 CIDR range. If you specify an IP address, you cannot indicate any IP addresses specified in <code>privateIpAddresses</code> as primary (only one IP address can be designated as primary).<br/>Ipv6Addresses:  One or more specific IPv6 addresses from the IPv6 CIDR block range of your subnet. You can't use this option if you're specifying a number of IPv6 addresses.<br/>Ipv6AddressCount:  The number of IPv6 addresses to assign to a network interface. Amazon EC2 automatically selects the IPv6 addresses from the subnet range. You can't use this option if specifying specific IPv6 addresses. If your subnet has the <code>AssignIpv6AddressOnCreation</code> attribute set to <code>true</code>, you can specify <code>0</code> to override this setting.<br/>Groups:  The IDs of one or more security groups.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Description:  A description for the network interface.<br/>
        :return: NetworkInterface:  Information about the network interface.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TagSet</i> Any tags assigned to the network interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetId</i> The ID of the subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the network interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceDestCheck</i> Indicates whether traffic to or from the instance is validated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequesterManaged</i> Indicates whether the network interface is being managed by AWS.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequesterId</i> The ID of the entity that launched the instance on your behalf (for example, AWS Management Console or Auto Scaling).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrivateIpAddresses</i> The private IPv4 addresses associated with the network interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrivateIpAddress</i> The IPv4 address of the network interface within the subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrivateDnsName</i> The private DNS name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The AWS account ID of the owner of the network interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkInterfaceId</i> The ID of the network interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MacAddress</i> The MAC address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Ipv6Addresses</i> The IPv6 addresses associated with the network interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InterfaceType</i> The type of network interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Groups</i> Any security groups for the network interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Attachment</i> The network interface attachment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Attachment</b>: The network interface attachment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Association</i> The association information for an Elastic IP address (IPv4) associated with the network interface.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Association</b>: The association information for an Elastic IP address (IPv4) associated with the network interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PublicIp</i> The address of the Elastic IP address bound to the network interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PublicDnsName</i> The public DNS name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IpOwnerId</i> The ID of the Elastic IP address owner.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociationId</i> The association ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocationId</i> The allocation ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The attachment state.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceOwnerId</i> The AWS account ID of the owner of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceId</i> The ID of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceIndex</i> The device index of the network interface attachment on the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DeleteOnTermination</i> Indicates whether the network interface is deleted when the instance is terminated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AttachmentId</i> The ID of the network interface attachment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AttachTime</i> The timestamp indicating when the attachment initiated.<br/>        """
        pass

    def create_network_interface_permission(self, Permission: str, NetworkInterfaceId: str, AwsAccountId: str=None, AwsService: str=None, DryRun: Boolean=None) -> CreateNetworkInterfacePermissionResult:
        """Grants an AWS-authorized account permission to attach the specified network interface to an instance in their account. You can grant permission to a single AWS account only, and only one account at a time.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Permission:  The type of permission to grant.<br/>AwsService:  The AWS service. Currently not supported.<br/>AwsAccountId:  The AWS account ID.<br/>NetworkInterfaceId:  The ID of the network interface.<br/>
        :return: InterfacePermission:  Information about the permission for the network interface.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PermissionState</i> Information about the state of the permission.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PermissionState</b>: Information about the state of the permission.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Permission</i> The type of permission.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AwsService</i> The AWS service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AwsAccountId</i> The AWS account ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkInterfaceId</i> The ID of the network interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkInterfacePermissionId</i> The ID of the network interface permission.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> A status message, if applicable.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the permission.<br/>        """
        pass

    def create_placement_group(self, DryRun: Boolean=None, GroupName: str=None, Strategy: str=None, PartitionCount: int=None):
        """Creates a placement group in which to launch instances. The strategy of the placement group determines how the instances are organized within the group.  A <code>cluster</code> placement group is a logical grouping of instances within a single Availability Zone that benefit from low network latency, high network throughput. A <code>spread</code> placement group places instances on distinct hardware. A <code>partition</code> placement group places groups of instances in different partitions, where instances in one partition do not share the same hardware with instances in another partition. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/placement-groups.html">Placement Groups</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param PartitionCount:  The number of partitions. Valid only when <b>Strategy</b> is set to <code>partition</code>.<br/>Strategy:  The placement strategy.<br/>GroupName:  A name for the placement group. Must be unique within the scope of your account for the Region. Constraints: Up to 255 ASCII characters<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return:         """
        pass

    def create_reserved_instances_listing(self, ReservedInstancesId: str, PriceSchedules: List[PriceScheduleSpecification], InstanceCount: int, ClientToken: str) -> CreateReservedInstancesListingResult:
        """Creates a listing for Amazon EC2 Standard Reserved Instances to be sold in the Reserved Instance Marketplace. You can submit one Standard Reserved Instance listing at a time. To get a list of your Standard Reserved Instances, you can use the <a>DescribeReservedInstances</a> operation. <note> Only Standard Reserved Instances can be sold in the Reserved Instance Marketplace. Convertible Reserved Instances cannot be sold. </note> The Reserved Instance Marketplace matches sellers who want to resell Standard Reserved Instance capacity that they no longer need with buyers who want to purchase additional capacity. Reserved Instances bought and sold through the Reserved Instance Marketplace work like any other Reserved Instances. To sell your Standard Reserved Instances, you must first register as a seller in the Reserved Instance Marketplace. After completing the registration process, you can create a Reserved Instance Marketplace listing of some or all of your Standard Reserved Instances, and specify the upfront price to receive for them. Your Standard Reserved Instance listings then become available for purchase. To view the details of your Standard Reserved Instance listing, you can use the <a>DescribeReservedInstancesListings</a> operation. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ri-market-general.html">Reserved Instance Marketplace</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param ReservedInstancesId:  The ID of the active Standard Reserved Instance.<br/>PriceSchedules:  A list specifying the price of the Standard Reserved Instance for each month remaining in the Reserved Instance term.<br/>InstanceCount:  The number of instances that are a part of a Reserved Instance account to be listed in the Reserved Instance Marketplace. This number should be less than or equal to the instance count associated with the Reserved Instance ID specified in this call.<br/>ClientToken:  Unique, case-sensitive identifier you provide to ensure idempotency of your listings. This helps avoid duplicate listings. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">Ensuring Idempotency</a>.<br/>
        :return: ReservedInstancesListings:  Information about the Standard Reserved Instance listing.<br/>        """
        pass

    def create_route(self, RouteTableId: str, DestinationCidrBlock: str=None, DestinationIpv6CidrBlock: str=None, DryRun: Boolean=None, EgressOnlyInternetGatewayId: str=None, GatewayId: str=None, InstanceId: str=None, NatGatewayId: str=None, TransitGatewayId: str=None, NetworkInterfaceId: str=None, VpcPeeringConnectionId: str=None) -> CreateRouteResult:
        """Creates a route in a route table within a VPC. You must specify one of the following targets: internet gateway or virtual private gateway, NAT instance, NAT gateway, VPC peering connection, network interface, or egress-only internet gateway. When determining how to route traffic, we use the route with the most specific match. For example, traffic is destined for the IPv4 address <code>192.0.2.3</code>, and the route table includes the following two IPv4 routes: <ul> <li>  <code>192.0.2.0/24</code> (goes to some target A) </li> <li>  <code>192.0.2.0/28</code> (goes to some target B) </li> </ul> Both routes apply to the traffic destined for <code>192.0.2.3</code>. However, the second route in the list covers a smaller number of IP addresses and is therefore more specific, so we use that route to determine where to target the traffic. For more information about route tables, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Route_Tables.html">Route Tables</a> in the <i>Amazon Virtual Private Cloud User Guide</i>.

        :param self:
        :param VpcPeeringConnectionId:  The ID of a VPC peering connection.<br/>RouteTableId:  The ID of the route table for the route.<br/>NetworkInterfaceId:  The ID of a network interface.<br/>TransitGatewayId:  The ID of a transit gateway.<br/>NatGatewayId:  [IPv4 traffic only] The ID of a NAT gateway.<br/>InstanceId:  The ID of a NAT instance in your VPC. The operation fails if you specify an instance ID unless exactly one network interface is attached.<br/>GatewayId:  The ID of an internet gateway or virtual private gateway attached to your VPC.<br/>EgressOnlyInternetGatewayId:  [IPv6 traffic only] The ID of an egress-only internet gateway.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>DestinationIpv6CidrBlock:  The IPv6 CIDR block used for the destination match. Routing decisions are based on the most specific match.<br/>DestinationCidrBlock:  The IPv4 CIDR address block used for the destination match. Routing decisions are based on the most specific match.<br/>
        :return: Return:  Returns <code>true</code> if the request succeeds; otherwise, it returns an error.<br/>        """
        pass

    def create_route_table(self, VpcId: str, DryRun: Boolean=None) -> CreateRouteTableResult:
        """Creates a route table for the specified VPC. After you create a route table, you can add routes and associate the table with a subnet. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Route_Tables.html">Route Tables</a> in the <i>Amazon Virtual Private Cloud User Guide</i>.

        :param self:
        :param VpcId:  The ID of the VPC.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: RouteTable:  Information about the route table.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The ID of the AWS account that owns the route table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> Any tags assigned to the route table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Routes</i> The routes in the route table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RouteTableId</i> The ID of the route table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PropagatingVgws</i> Any virtual private gateway (VGW) propagating routes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Associations</i> The associations between the route table and one or more subnets.<br/>        """
        pass

    def create_security_group(self, GroupName: str, Description: str, VpcId: str=None, DryRun: Boolean=None) -> CreateSecurityGroupResult:
        """Creates a security group. A security group acts as a virtual firewall for your instance to control inbound and outbound traffic. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/using-network-security.html">Amazon EC2 Security Groups</a> in the <i>Amazon Elastic Compute Cloud User Guide</i> and <a href="https://docs.aws.amazon.com/AmazonVPC/latest/UserGuide/VPC_SecurityGroups.html">Security Groups for Your VPC</a> in the <i>Amazon Virtual Private Cloud User Guide</i>. When you create a security group, you specify a friendly name of your choice. You can have a security group for use in EC2-Classic with the same name as a security group for use in a VPC. However, you can't have two security groups for use in EC2-Classic with the same name or two security groups for use in a VPC with the same name. You have a default security group for use in EC2-Classic and a default security group for use in your VPC. If you don't specify a security group when you launch an instance, the instance is launched into the appropriate default security group. A default security group includes a default rule that grants instances unrestricted network access to each other. You can add or remove rules from your security groups using <a>AuthorizeSecurityGroupIngress</a>, <a>AuthorizeSecurityGroupEgress</a>, <a>RevokeSecurityGroupIngress</a>, and <a>RevokeSecurityGroupEgress</a>. For more information about VPC security group limits, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/amazon-vpc-limits.html">Amazon VPC Limits</a>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>VpcId:  [EC2-VPC] The ID of the VPC. Required for EC2-VPC.<br/>GroupName:  The name of the security group. Constraints: Up to 255 characters in length. Cannot start with <code>sg-</code>. Constraints for EC2-Classic: ASCII characters Constraints for EC2-VPC: a-z, A-Z, 0-9, spaces, and ._-:/()#,@[]+=&amp;;{}!$*<br/>Description:  A description for the security group. This is informational only. Constraints: Up to 255 characters in length Constraints for EC2-Classic: ASCII characters Constraints for EC2-VPC: a-z, A-Z, 0-9, spaces, and ._-:/()#,@[]+=&amp;;{}!$*<br/>
        :return: GroupId:  The ID of the security group.<br/>        """
        pass

    def create_snapshot(self, VolumeId: str, Description: str=None, TagSpecifications: List[TagSpecification]=None, DryRun: Boolean=None) -> Snapshot:
        """Creates a snapshot of an EBS volume and stores it in Amazon S3. You can use snapshots for backups, to make copies of EBS volumes, and to save data before shutting down an instance. When a snapshot is created, any AWS Marketplace product codes that are associated with the source volume are propagated to the snapshot. You can take a snapshot of an attached volume that is in use. However, snapshots only capture data that has been written to your EBS volume at the time the snapshot command is issued; this may exclude any data that has been cached by any applications or the operating system. If you can pause any file systems on the volume long enough to take a snapshot, your snapshot should be complete. However, if you cannot pause all file writes to the volume, you should unmount the volume from within the instance, issue the snapshot command, and then remount the volume to ensure a consistent and complete snapshot. You may remount and use your volume while the snapshot status is <code>pending</code>. To create a snapshot for EBS volumes that serve as root devices, you should stop the instance before taking the snapshot. Snapshots that are taken from encrypted volumes are automatically encrypted. Volumes that are created from encrypted snapshots are also automatically encrypted. Your encrypted volumes and any associated snapshots always remain protected. You can tag your snapshots during creation. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Using_Tags.html">Tagging Your Amazon EC2 Resources</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/AmazonEBS.html">Amazon Elastic Block Store</a> and <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html">Amazon EBS Encryption</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>TagSpecifications:  The tags to apply to the snapshot during creation.<br/>VolumeId:  The ID of the EBS volume.<br/>Description:  A description for the snapshot.<br/>
        :return: Tags:  Any tags assigned to the snapshot.<br/>OwnerAlias:   Value from an Amazon-maintained list (<code>amazon</code> | <code>self</code> | <code>all</code> | <code>aws-marketplace</code> | <code>microsoft</code>) of snapshot owners. Not to be confused with the user-configured AWS account alias, which is set from the IAM console. <br/>VolumeSize:  The size of the volume, in GiB.<br/>VolumeId:  The ID of the volume that was used to create the snapshot. Snapshots created by the <a>CopySnapshot</a> action have an arbitrary volume ID that should not be used for any purpose.<br/>StateMessage:  Encrypted Amazon EBS snapshots are copied asynchronously. If a snapshot copy operation fails (for example, if the proper AWS Key Management Service (AWS KMS) permissions are not obtained) this field displays error state details to help you diagnose why the error occurred. This parameter is only returned by <a>DescribeSnapshots</a>.<br/>State:  The snapshot state.<br/>StartTime:  The time stamp when the snapshot was initiated.<br/>SnapshotId:  The ID of the snapshot. Each snapshot receives a unique identifier when it is created.<br/>Progress:  The progress of the snapshot, as a percentage.<br/>OwnerId:  The AWS account ID of the EBS snapshot owner.<br/>KmsKeyId:  The Amazon Resource Name (ARN) of the AWS Key Management Service (AWS KMS) customer master key (CMK) that was used to protect the volume encryption key for the parent volume.<br/>Encrypted:  Indicates whether the snapshot is encrypted.<br/>Description:  The description for the snapshot.<br/>DataEncryptionKeyId:  The data encryption key identifier for the snapshot. This value is a unique identifier that corresponds to the data encryption key that was used to encrypt the original volume or snapshot copy. Because data encryption keys are inherited by volumes created from snapshots, and vice versa, if snapshots share the same data encryption key identifier, then they belong to the same volume/snapshot lineage. This parameter is only returned by <a>DescribeSnapshots</a>.<br/>        """
        pass

    def create_snapshots(self, InstanceSpecification: InstanceSpecification, Description: str=None, TagSpecifications: List[TagSpecification]=None, DryRun: Boolean=None, CopyTagsFromSource: str=None) -> CreateSnapshotsResult:
        """Creates crash-consistent snapshots of multiple EBS volumes and stores the data in S3. Volumes are chosen by specifying an instance. Any attached volumes will produce one snapshot each that is crash-consistent across the instance. Boot volumes can be excluded by changing the paramaters. 

        :param self:
        :param CopyTagsFromSource:  Copies the tags from the specified instance to all snapshots.<br/>DryRun:  Checks whether you have the required permissions for the action without actually making the request. Provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation.<br/>TagSpecifications:  Tags to apply to every snapshot specified by the instance.<br/>InstanceSpecification:  The instance to specify which volumes should be included in the snapshots.:<br/>Description:   A description propagated to every snapshot specified by the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExcludeBootVolume</i> Excludes the root volume from being snapshotted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceId</i> The instance to specify which volumes should be snapshotted.<br/>
        :return: Snapshots:  List of snapshots.<br/>        """
        pass

    def create_spot_datafeed_subscription(self, Bucket: str, DryRun: Boolean=None, Prefix: str=None) -> CreateSpotDatafeedSubscriptionResult:
        """Creates a data feed for Spot Instances, enabling you to view Spot Instance usage logs. You can create one data feed per AWS account. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/spot-data-feeds.html">Spot Instance Data Feed</a> in the <i>Amazon EC2 User Guide for Linux Instances</i>.

        :param self:
        :param Prefix:  A prefix for the data feed file names.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Bucket:  The Amazon S3 bucket in which to store the Spot Instance data feed.<br/>
        :return: SpotDatafeedSubscription:  The Spot Instance data feed subscription.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the Spot Instance data feed subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> The prefix that is prepended to data feed files.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The AWS account ID of the account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Fault</i> The fault codes for the Spot Instance request, if any.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Fault</b>: The fault codes for the Spot Instance request, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> The Amazon S3 bucket where the Spot Instance data feed is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> The message for the Spot Instance state change.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Code</i> The reason code for the Spot Instance state change.<br/>        """
        pass

    def create_subnet(self, VpcId: str, CidrBlock: str, AvailabilityZone: str=None, AvailabilityZoneId: str=None, Ipv6CidrBlock: str=None, DryRun: Boolean=None) -> CreateSubnetResult:
        """Creates a subnet in an existing VPC. When you create each subnet, you provide the VPC ID and IPv4 CIDR block for the subnet. After you create a subnet, you can't change its CIDR block. The size of the subnet's IPv4 CIDR block can be the same as a VPC's IPv4 CIDR block, or a subset of a VPC's IPv4 CIDR block. If you create more than one subnet in a VPC, the subnets' CIDR blocks must not overlap. The smallest IPv4 subnet (and VPC) you can create uses a /28 netmask (16 IPv4 addresses), and the largest uses a /16 netmask (65,536 IPv4 addresses). If you've associated an IPv6 CIDR block with your VPC, you can create a subnet with an IPv6 CIDR block that uses a /64 prefix length.  <important> AWS reserves both the first four and the last IPv4 address in each subnet's CIDR block. They're not available for use. </important> If you add more than one subnet to a VPC, they're set up in a star topology with a logical router in the middle. If you launch an instance in a VPC using an Amazon EBS-backed AMI, the IP address doesn't change if you stop and restart the instance (unlike a similar instance launched outside a VPC, which gets a new IP address when restarted). It's therefore possible to have a subnet with no running instances (they're all stopped), but no remaining IP addresses available. For more information about subnets, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Subnets.html">Your VPC and Subnets</a> in the <i>Amazon Virtual Private Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>VpcId:  The ID of the VPC.<br/>Ipv6CidrBlock:  The IPv6 network range for the subnet, in CIDR notation. The subnet size must use a /64 prefix length.<br/>CidrBlock:  The IPv4 network range for the subnet, in CIDR notation. For example, <code>10.0.0.0/24</code>.<br/>AvailabilityZoneId:  The AZ ID of the subnet.<br/>AvailabilityZone:  The Availability Zone for the subnet. Default: AWS selects one for you. If you create more than one subnet in your VPC, we may not necessarily select a different zone for each subnet.<br/>
        :return: Subnet:  Information about the subnet.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetArn</i> The Amazon Resource Name (ARN) of the subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> Any tags assigned to the subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Ipv6CidrBlockAssociationSet</i> Information about the IPv6 CIDR blocks associated with the subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssignIpv6AddressOnCreation</i> Indicates whether a network interface created in this subnet (including a network interface created by <a>RunInstances</a>) receives an IPv6 address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The ID of the AWS account that owns the subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the VPC the subnet is in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetId</i> The ID of the subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The current state of the subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MapPublicIpOnLaunch</i> Indicates whether instances launched in this subnet receive a public IPv4 address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultForAz</i> Indicates whether this is the default subnet for the Availability Zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CidrBlock</i> The IPv4 CIDR block assigned to the subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailableIpAddressCount</i> The number of unused private IPv4 addresses in the subnet. The IPv4 addresses for any stopped instances are considered unavailable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZoneId</i> The AZ ID of the subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone of the subnet.<br/>        """
        pass

    def create_tags(self, Tags: List[Tag], Resources: List[String], DryRun: Boolean=None):
        """Adds or overwrites the specified tags for the specified Amazon EC2 resource or resources. Each resource can have a maximum of 50 tags. Each tag consists of a key and optional value. Tag keys must be unique per resource. For more information about tags, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Using_Tags.html">Tagging Your Resources</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. For more information about creating IAM policies that control users' access to resources based on tags, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-supported-iam-actions-resources.html">Supported Resource-Level Permissions for Amazon EC2 API Actions</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param Tags:  The tags. The <code>value</code> parameter is required, but if you don't want the tag to have a value, specify the parameter with no value, and we set the value to an empty string.<br/>Resources:  The IDs of the resources, separated by spaces. Constraints: Up to 1000 resource IDs. We recommend breaking up this request into smaller batches.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return:         """
        pass

    def create_traffic_mirror_filter(self, Description: str=None, TagSpecifications: List[TagSpecification]=None, DryRun: Boolean=None, ClientToken: str=None) -> CreateTrafficMirrorFilterResult:
        """Creates a Traffic Mirror filter. A Traffic Mirror filter is a set of rules that defines the traffic to mirror. By default, no traffic is mirrored. To mirror traffic, use <a>CreateTrafficMirrorFilterRule</a> to add Traffic Mirror rules to the filter. The rules you add define what traffic gets mirrored. You can also use <a>ModifyTrafficMirrorFilterNetworkServices</a> to mirror supported network services.

        :param self:
        :param ClientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>TagSpecifications:  The tags to assign to a Traffic Mirror filter.<br/>Description:  The description of the Traffic Mirror filter.<br/>
        :return: ClientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.<br/>TrafficMirrorFilter:  Information about the Traffic Mirror filter.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The tags assigned to the Traffic Mirror filter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the Traffic Mirror filter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkServices</i> The network service traffic that is associated with the Traffic Mirror filter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EgressFilterRules</i> Information about the egress rules that are associated with the Traffic Mirror filter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IngressFilterRules</i> Information about the ingress rules that are associated with the Traffic Mirror filter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrafficMirrorFilterId</i> The ID of the Traffic Mirror filter.<br/>        """
        pass

    def create_traffic_mirror_filter_rule(self, SourceCidrBlock: str, DestinationCidrBlock: str, RuleAction: str, RuleNumber: int, TrafficDirection: str, TrafficMirrorFilterId: str, DestinationPortRange: TrafficMirrorPortRangeRequest=None, SourcePortRange: TrafficMirrorPortRangeRequest=None, Protocol: int=None, Description: str=None, DryRun: Boolean=None, ClientToken: str=None) -> CreateTrafficMirrorFilterRuleResult:
        """Creates a Traffic Mirror rule.  A Traffic Mirror rule defines the Traffic Mirror source traffic to mirror. You need the Traffic Mirror filter ID when you create the rule.

        :param self:
        :param ClientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Description:  The description of the Traffic Mirror rule.<br/>SourceCidrBlock:  The source CIDR block to assign to the Traffic Mirror rule.<br/>DestinationCidrBlock:  The destination CIDR block to assign to the Traffic Mirror rule.<br/>Protocol:  The protocol, for example UDP, to assign to the Traffic Mirror rule. For information about the protocol value, see <a href="https://www.iana.org/assignments/protocol-numbers/protocol-numbers.xhtml">Protocol Numbers</a> on the Internet Assigned Numbers Authority (IANA) website.<br/>SourcePortRange:  The source port range.:<br/>DestinationPortRange:  The destination port range.:<br/>RuleAction:  The action to take (<code>accept</code> | <code>reject</code>) on the filtered traffic.<br/>RuleNumber:  The number of the Traffic Mirror rule. This number must be unique for each Traffic Mirror rule in a given direction. The rules are processed in ascending order by rule number.<br/>TrafficDirection:  The type of traffic (<code>ingress</code> | <code>egress</code>).<br/>TrafficMirrorFilterId:  The ID of the filter that this rule is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ToPort</i> The last port in the Traffic Mirror port range. This applies to the TCP and UDP protocols.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FromPort</i> The first port in the Traffic Mirror port range. This applies to the TCP and UDP protocols.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ToPort</i> The last port in the Traffic Mirror port range. This applies to the TCP and UDP protocols.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FromPort</i> The first port in the Traffic Mirror port range. This applies to the TCP and UDP protocols.<br/>
        :return: ClientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.<br/>TrafficMirrorFilterRule:  The Traffic Mirror rule.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the Traffic Mirror rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceCidrBlock</i> The source CIDR block assigned to the Traffic Mirror rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationCidrBlock</i> The destination CIDR block assigned to the Traffic Mirror rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourcePortRange</i> The source port range assigned to the Traffic Mirror rule.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SourcePortRange</b>: The source port range assigned to the Traffic Mirror rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationPortRange</i> The destination port range assigned to the Traffic Mirror rule.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DestinationPortRange</b>: The destination port range assigned to the Traffic Mirror rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Protocol</i> The protocol assigned to the Traffic Mirror rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleAction</i> The action assigned to the Traffic Mirror rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleNumber</i> The rule number of the Traffic Mirror rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrafficDirection</i> The traffic direction assigned to the Traffic Mirror rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrafficMirrorFilterId</i> The ID of the Traffic Mirror filter that the rule is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrafficMirrorFilterRuleId</i> The ID of the Traffic Mirror rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ToPort</i> The end of the Traffic Mirror port range. This applies to the TCP and UDP protocols.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FromPort</i> The start of the Traffic Mirror port range. This applies to the TCP and UDP protocols.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ToPort</i> The end of the Traffic Mirror port range. This applies to the TCP and UDP protocols.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FromPort</i> The start of the Traffic Mirror port range. This applies to the TCP and UDP protocols.<br/>        """
        pass

    def create_traffic_mirror_session(self, SessionNumber: int, TrafficMirrorFilterId: str, TrafficMirrorTargetId: str, NetworkInterfaceId: str, PacketLength: int=None, VirtualNetworkId: int=None, Description: str=None, TagSpecifications: List[TagSpecification]=None, DryRun: Boolean=None, ClientToken: str=None) -> CreateTrafficMirrorSessionResult:
        """Creates a Traffic Mirror session. A Traffic Mirror session actively copies packets from a Traffic Mirror source to a Traffic Mirror target. Create a filter, and then assign it to the session to define a subset of the traffic to mirror, for example all TCP traffic. The Traffic Mirror source and the Traffic Mirror target (monitoring appliances) can be in the same VPC, or in a different VPC connected via VPC peering or a transit gateway.  By default, no traffic is mirrored. Use <a>CreateTrafficMirrorFilter</a> to create filter rules that specify the traffic to mirror.

        :param self:
        :param ClientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>TagSpecifications:  The tags to assign to a Traffic Mirror session.<br/>Description:  The description of the Traffic Mirror session.<br/>VirtualNetworkId:  The VXLAN ID for the Traffic Mirror session. For more information about the VXLAN protocol, see <a href="https://tools.ietf.org/html/rfc7348">RFC 7348</a>. If you do not specify a <code>VirtualNetworkId</code>, an account-wide unique id is chosen at random.<br/>SessionNumber:  The session number determines the order in which sessions are evaluated when an interface is used by multiple sessions. The first session with a matching filter is the one that mirrors the packets. Valid values are 1-32766.<br/>PacketLength:  The number of bytes in each packet to mirror. These are bytes after the VXLAN header. Do not specify this parameter when you want to mirror the entire packet. To mirror a subset of the packet, set this to the length (in bytes) that you want to mirror. For example, if you set this value to 1network0, then the first 100 bytes that meet the filter criteria are copied to the target. If you do not want to mirror the entire packet, use the <code>PacketLength</code> parameter to specify the number of bytes in each packet to mirror.<br/>TrafficMirrorFilterId:  The ID of the Traffic Mirror filter.<br/>TrafficMirrorTargetId:  The ID of the Traffic Mirror target.<br/>NetworkInterfaceId:  The ID of the source network interface.<br/>
        :return: ClientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.<br/>TrafficMirrorSession:  Information about the Traffic Mirror session.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The tags assigned to the Traffic Mirror session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the Traffic Mirror session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VirtualNetworkId</i> The virtual network ID associated with the Traffic Mirror session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SessionNumber</i> The session number determines the order in which sessions are evaluated when an interface is used by multiple sessions. The first session with a matching filter is the one that mirrors the packets. Valid values are 1-32766.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PacketLength</i> The number of bytes in each packet to mirror. These are the bytes after the VXLAN header. To mirror a subset, set this to the length (in bytes) to mirror. For example, if you set this value to 100, then the first 100 bytes that meet the filter criteria are copied to the target. Do not specify this parameter when you want to mirror the entire packet<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The ID of the account that owns the Traffic Mirror session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkInterfaceId</i> The ID of the Traffic Mirror session's network interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrafficMirrorFilterId</i> The ID of the Traffic Mirror filter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrafficMirrorTargetId</i> The ID of the Traffic Mirror target.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrafficMirrorSessionId</i> The ID for the Traffic Mirror session.<br/>        """
        pass

    def create_traffic_mirror_target(self, NetworkInterfaceId: str=None, NetworkLoadBalancerArn: str=None, Description: str=None, TagSpecifications: List[TagSpecification]=None, DryRun: Boolean=None, ClientToken: str=None) -> CreateTrafficMirrorTargetResult:
        """Creates a target for your Traffic Mirror session. A Traffic Mirror target is the destination for mirrored traffic. The Traffic Mirror source and the Traffic Mirror target (monitoring appliances) can be in the same VPC, or in different VPCs connected via VPC peering or a transit gateway. A Traffic Mirror target can be a network interface, or a Network Load Balancer. To use the target in a Traffic Mirror session, use <a>CreateTrafficMirrorSession</a>.

        :param self:
        :param ClientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>TagSpecifications:  The tags to assign to the Traffic Mirror target.<br/>Description:  The description of the Traffic Mirror target.<br/>NetworkLoadBalancerArn:  The Amazon Resource Name (ARN) of the Network Load Balancer that is associated with the target.<br/>NetworkInterfaceId:  The network interface ID that is associated with the target.<br/>
        :return: ClientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.<br/>TrafficMirrorTarget:  Information about the Traffic Mirror target.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The tags assigned to the Traffic Mirror target.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The ID of the account that owns the Traffic Mirror target.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> Information about the Traffic Mirror target.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of Traffic Mirror target.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkLoadBalancerArn</i> The Amazon Resource Name (ARN) of the Network Load Balancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkInterfaceId</i> The network interface ID that is attached to the target.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrafficMirrorTargetId</i> The ID of the Traffic Mirror target.<br/>        """
        pass

    def create_transit_gateway(self, Description: str=None, Options: TransitGatewayRequestOptions=None, TagSpecifications: List[TagSpecification]=None, DryRun: Boolean=None) -> CreateTransitGatewayResult:
        """Creates a transit gateway. You can use a transit gateway to interconnect your virtual private clouds (VPC) and on-premises networks. After the transit gateway enters the <code>available</code> state, you can attach your VPCs and VPN connections to the transit gateway. To attach your VPCs, use <a>CreateTransitGatewayVpcAttachment</a>. To attach a VPN connection, use <a>CreateCustomerGateway</a> to create a customer gateway and specify the ID of the customer gateway and the ID of the transit gateway in a call to <a>CreateVpnConnection</a>. When you create a transit gateway, we create a default transit gateway route table and use it as the default association route table and the default propagation route table. You can use <a>CreateTransitGatewayRouteTable</a> to create additional transit gateway route tables. If you disable automatic route propagation, we do not create a default transit gateway route table. You can use <a>EnableTransitGatewayRouteTablePropagation</a> to propagate routes from a resource attachment to a transit gateway route table. If you disable automatic associations, you can use <a>AssociateTransitGatewayRouteTable</a> to associate a resource attachment with a transit gateway route table.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>TagSpecifications:  The tags to apply to the transit gateway.<br/>Options:  The transit gateway options.:<br/>Description:  A description of the transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DnsSupport</i> Enable or disable DNS support.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpnEcmpSupport</i> Enable or disable Equal Cost Multipath Protocol support.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultRouteTablePropagation</i> Enable or disable automatic propagation of routes to the default propagation route table. The default is <code>enable</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultRouteTableAssociation</i> Enable or disable automatic association with the default association route table. The default is <code>enable</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoAcceptSharedAttachments</i> Enable or disable automatic acceptance of attachment requests. The default is <code>disable</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AmazonSideAsn</i> A private Autonomous System Number (ASN) for the Amazon side of a BGP session. The range is 64512 to 65534 for 16-bit ASNs and 4200000000 to 4294967294 for 32-bit ASNs.<br/>
        :return: TransitGateway:  Information about the transit gateway.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The tags for the transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> The transit gateway options.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>: The transit gateway options.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The creation time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The ID of the AWS account ID that owns the transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayArn</i> The Amazon Resource Name (ARN) of the transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayId</i> The ID of the transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DnsSupport</i> Indicates whether DNS support is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VpnEcmpSupport</i> Indicates whether Equal Cost Multipath Protocol support is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PropagationDefaultRouteTableId</i> The ID of the default propagation route table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultRouteTablePropagation</i> Indicates whether resource attachments automatically propagate routes to the default propagation route table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociationDefaultRouteTableId</i> The ID of the default association route table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultRouteTableAssociation</i> Indicates whether resource attachments are automatically associated with the default association route table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoAcceptSharedAttachments</i> Indicates whether attachment requests are automatically accepted.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AmazonSideAsn</i> A private Autonomous System Number (ASN) for the Amazon side of a BGP session. The range is 64512 to 65534 for 16-bit ASNs and 4200000000 to 4294967294 for 32-bit ASNs.<br/>        """
        pass

    def create_transit_gateway_route(self, TransitGatewayRouteTableId: str, DestinationCidrBlock: str, TransitGatewayAttachmentId: str=None, Blackhole: Boolean=None, DryRun: Boolean=None) -> CreateTransitGatewayRouteResult:
        """Creates a static route for the specified transit gateway route table.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Blackhole:  Indicates whether to drop traffic that matches this route.<br/>TransitGatewayAttachmentId:  The ID of the attachment.<br/>TransitGatewayRouteTableId:  The ID of the transit gateway route table.<br/>DestinationCidrBlock:  The CIDR range used for destination matches. Routing decisions are based on the most specific match.<br/>
        :return: Route:  Information about the route.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the route.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The route type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayAttachments</i> The attachments.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationCidrBlock</i> The CIDR block used for destination matches.<br/>        """
        pass

    def create_transit_gateway_route_table(self, TransitGatewayId: str, TagSpecifications: List[TagSpecification]=None, DryRun: Boolean=None) -> CreateTransitGatewayRouteTableResult:
        """Creates a route table for the specified transit gateway.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>TagSpecifications:  The tags to apply to the transit gateway route table.<br/>TransitGatewayId:  The ID of the transit gateway.<br/>
        :return: TransitGatewayRouteTable:  Information about the transit gateway route table.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> Any tags assigned to the route table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The creation time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultPropagationRouteTable</i> Indicates whether this is the default propagation route table for the transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultAssociationRouteTable</i> Indicates whether this is the default association route table for the transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the transit gateway route table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayId</i> The ID of the transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayRouteTableId</i> The ID of the transit gateway route table.<br/>        """
        pass

    def create_transit_gateway_vpc_attachment(self, SubnetIds: List[String], VpcId: str, TransitGatewayId: str, Options: CreateTransitGatewayVpcAttachmentRequestOptions=None, TagSpecifications: List[TagSpecification]=None, DryRun: Boolean=None) -> CreateTransitGatewayVpcAttachmentResult:
        """Attaches the specified VPC to the specified transit gateway. If you attach a VPC with a CIDR range that overlaps the CIDR range of a VPC that is already attached, the new VPC CIDR range is not propagated to the default propagation route table. To send VPC traffic to an attached transit gateway, add a route to the VPC route table using <a>CreateRoute</a>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>TagSpecifications:  The tags to apply to the VPC attachment.<br/>Options:  The VPC attachment options.:<br/>SubnetIds:  The IDs of one or more subnets. You can specify only one subnet per Availability Zone. You must specify at least one subnet, but we recommend that you specify two subnets for better availability. The transit gateway uses one IP address from each specified subnet.<br/>VpcId:  The ID of the VPC.<br/>TransitGatewayId:  The ID of the transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Ipv6Support</i> Enable or disable IPv6 support. The default is <code>enable</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DnsSupport</i> Enable or disable DNS support. The default is <code>enable</code>.<br/>
        :return: TransitGatewayVpcAttachment:  Information about the VPC attachment.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The tags for the VPC attachment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> The VPC attachment options.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>: The VPC attachment options.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The creation time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> The IDs of the subnets.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the VPC attachment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcOwnerId</i> The ID of the AWS account that owns the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayId</i> The ID of the transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayAttachmentId</i> The ID of the attachment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Ipv6Support</i> Indicates whether IPv6 support is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DnsSupport</i> Indicates whether DNS support is enabled.<br/>        """
        pass

    def create_volume(self, AvailabilityZone: str, Encrypted: Boolean=None, Iops: int=None, KmsKeyId: str=None, Size: int=None, SnapshotId: str=None, VolumeType: str=None, DryRun: Boolean=None, TagSpecifications: List[TagSpecification]=None) -> Volume:
        """Creates an EBS volume that can be attached to an instance in the same Availability Zone. The volume is created in the regional endpoint that you send the HTTP request to. For more information see <a href="https://docs.aws.amazon.com/general/latest/gr/rande.html">Regions and Endpoints</a>. You can create a new empty volume or restore a volume from an EBS snapshot. Any AWS Marketplace product codes from the snapshot are propagated to the volume. You can create encrypted volumes. Encrypted volumes must be attached to instances that support Amazon EBS encryption. Volumes that are created from encrypted snapshots are also automatically encrypted. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html">Amazon EBS Encryption</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. You can tag your volumes during creation. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Using_Tags.html">Tagging Your Amazon EC2 Resources</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ebs-creating-volume.html">Creating an Amazon EBS Volume</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param TagSpecifications:  The tags to apply to the volume during creation.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>VolumeType:  The volume type. This can be <code>gp2</code> for General Purpose SSD, <code>io1</code> for Provisioned IOPS SSD, <code>st1</code> for Throughput Optimized HDD, <code>sc1</code> for Cold HDD, or <code>standard</code> for Magnetic volumes. Defaults: If no volume type is specified, the default is <code>standard</code> in us-east-1, eu-west-1, eu-central-1, us-west-2, us-west-1, sa-east-1, ap-northeast-1, ap-northeast-2, ap-southeast-1, ap-southeast-2, ap-south-1, us-gov-west-1, and cn-north-1. In all other Regions, EBS defaults to <code>gp2</code>.<br/>SnapshotId:  The snapshot from which to create the volume. <note> At least one of Size or SnapshotId are required. </note><br/>Size:  The size of the volume, in GiBs. Constraints: 1-16,384 for <code>gp2</code>, 4-16,384 for <code>io1</code>, 500-16,384 for <code>st1</code>, 500-16,384 for <code>sc1</code>, and 1-1,024 for <code>standard</code>. If you specify a snapshot, the volume size must be equal to or larger than the snapshot size. Default: If you're creating the volume from a snapshot and don't specify a volume size, the default is the snapshot size. <note> At least one of Size or SnapshotId is required. </note><br/>KmsKeyId:  The identifier of the AWS Key Management Service (AWS KMS) customer master key (CMK) to use for Amazon EBS encryption. If this parameter is not specified, your AWS managed CMK for EBS is used. If <code>KmsKeyId</code> is specified, the encrypted state must be <code>true</code>. You can specify the CMK using any of the following: <ul> <li> Key ID. For example, key/1234abcd-12ab-34cd-56ef-1234567890ab. </li> <li> Key alias. For example, alias/ExampleAlias. </li> <li> Key ARN. For example, arn:aws:kms:<i>us-east-1</i>:<i>012345678910</i>:key/<i>abcd1234-a123-456a-a12b-a123b4cd56ef</i>. </li> <li> Alias ARN. For example, arn:aws:kms:<i>us-east-1</i>:<i>012345678910</i>:alias/<i>ExampleAlias</i>. </li> </ul> AWS authenticates the CMK asynchronously. Therefore, if you specify an ID, alias, or ARN that is not valid, the action can appear to complete, but eventually fails.<br/>Iops:  The number of I/O operations per second (IOPS) to provision for the volume, with a maximum ratio of 50 IOPS/GiB. Range is 100 to 64,000 IOPS for volumes in most Regions. Maximum IOPS of 64,000 is guaranteed only on <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html#ec2-nitro-instances">Nitro-based instances</a>. Other instance families guarantee performance up to 32,000 IOPS. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSVolumeTypes.html">Amazon EBS Volume Types</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. This parameter is valid only for Provisioned IOPS SSD (io1) volumes.<br/>Encrypted:  Specifies whether the volume should be encrypted. The effect of setting the encryption state to <code>true</code> depends on the volume origin (new or from a snapshot), starting encryption state, ownership, and whether encryption by default is enabled. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html#encryption-by-default">Encryption by Default</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. Encrypted Amazon EBS volumes must be attached to instances that support Amazon EBS encryption. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html#EBSEncryption_supported_instances">Supported Instance Types</a>.<br/>AvailabilityZone:  The Availability Zone in which to create the volume.<br/>
        :return: VolumeType:  The volume type. This can be <code>gp2</code> for General Purpose SSD, <code>io1</code> for Provisioned IOPS SSD, <code>st1</code> for Throughput Optimized HDD, <code>sc1</code> for Cold HDD, or <code>standard</code> for Magnetic volumes.<br/>Tags:  Any tags assigned to the volume.<br/>Iops:  The number of I/O operations per second (IOPS) that the volume supports. For Provisioned IOPS SSD volumes, this represents the number of IOPS that are provisioned for the volume. For General Purpose SSD volumes, this represents the baseline performance of the volume and the rate at which the volume accumulates I/O credits for bursting. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSVolumeTypes.html">Amazon EBS Volume Types</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. Constraints: Range is 100-16,000 IOPS for <code>gp2</code> volumes and 100 to 64,000IOPS for <code>io1</code> volumes, in most Regions. The maximum IOPS for <code>io1</code> of 64,000 is guaranteed only on <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html#ec2-nitro-instances">Nitro-based instances</a>. Other instance families guarantee performance up to 32,000 IOPS. Condition: This parameter is required for requests to create <code>io1</code> volumes; it is not used in requests to create <code>gp2</code>, <code>st1</code>, <code>sc1</code>, or <code>standard</code> volumes.<br/>VolumeId:  The ID of the volume.<br/>State:  The volume state.<br/>SnapshotId:  The snapshot from which the volume was created, if applicable.<br/>Size:  The size of the volume, in GiBs.<br/>KmsKeyId:  The Amazon Resource Name (ARN) of the AWS Key Management Service (AWS KMS) customer master key (CMK) that was used to protect the volume encryption key for the volume.<br/>Encrypted:  Indicates whether the volume is encrypted.<br/>CreateTime:  The time stamp when volume creation was initiated.<br/>AvailabilityZone:  The Availability Zone for the volume.<br/>Attachments:  Information about the volume attachments.<br/>        """
        pass

    def create_vpc(self, CidrBlock: str, AmazonProvidedIpv6CidrBlock: Boolean=None, DryRun: Boolean=None, InstanceTenancy: str=None) -> CreateVpcResult:
        """Creates a VPC with the specified IPv4 CIDR block. The smallest VPC you can create uses a /28 netmask (16 IPv4 addresses), and the largest uses a /16 netmask (65,536 IPv4 addresses). For more information about how large to make your VPC, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Subnets.html">Your VPC and Subnets</a> in the <i>Amazon Virtual Private Cloud User Guide</i>. You can optionally request an Amazon-provided IPv6 CIDR block for the VPC. The IPv6 CIDR block uses a /56 prefix length, and is allocated from Amazon's pool of IPv6 addresses. You cannot choose the IPv6 range for your VPC. By default, each instance you launch in the VPC has the default DHCP options, which include only a default DNS server that we provide (AmazonProvidedDNS). For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_DHCP_Options.html">DHCP Options Sets</a> in the <i>Amazon Virtual Private Cloud User Guide</i>. You can specify the instance tenancy value for the VPC when you create it. You can't change this value for the VPC after you create it. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/dedicated-instance.html">Dedicated Instances</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param InstanceTenancy:  The tenancy options for instances launched into the VPC. For <code>default</code>, instances are launched with shared tenancy by default. You can launch instances with any tenancy into a shared tenancy VPC. For <code>dedicated</code>, instances are launched as dedicated tenancy instances by default. You can only launch instances with a tenancy of <code>dedicated</code> or <code>host</code> into a dedicated tenancy VPC.   <b>Important:</b> The <code>host</code> value cannot be used with this parameter. Use the <code>default</code> or <code>dedicated</code> values only. Default: <code>default</code> <br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>AmazonProvidedIpv6CidrBlock:  Requests an Amazon-provided IPv6 CIDR block with a /56 prefix length for the VPC. You cannot specify the range of IP addresses, or the size of the CIDR block.<br/>CidrBlock:  The IPv4 network range for the VPC, in CIDR notation. For example, <code>10.0.0.0/16</code>.<br/>
        :return: Vpc:  Information about the VPC.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> Any tags assigned to the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsDefault</i> Indicates whether the VPC is the default VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CidrBlockAssociationSet</i> Information about the IPv4 CIDR blocks associated with the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Ipv6CidrBlockAssociationSet</i> Information about the IPv6 CIDR blocks associated with the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceTenancy</i> The allowed tenancy of instances launched into the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The ID of the AWS account that owns the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The current state of the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DhcpOptionsId</i> The ID of the set of DHCP options you've associated with the VPC (or <code>default</code> if the default options are associated with the VPC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CidrBlock</i> The primary IPv4 CIDR block for the VPC.<br/>        """
        pass

    def create_vpc_endpoint(self, ServiceName: str, VpcId: str, DryRun: Boolean=None, VpcEndpointType: str=None, PolicyDocument: str=None, RouteTableIds: List[String]=None, SubnetIds: List[String]=None, SecurityGroupIds: List[String]=None, ClientToken: str=None, PrivateDnsEnabled: Boolean=None) -> CreateVpcEndpointResult:
        """Creates a VPC endpoint for a specified service. An endpoint enables you to create a private connection between your VPC and the service. The service may be provided by AWS, an AWS Marketplace partner, or another AWS account. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/vpc-endpoints.html">VPC Endpoints</a> in the <i>Amazon Virtual Private Cloud User Guide</i>. A <code>gateway</code> endpoint serves as a target for a route in your route table for traffic destined for the AWS service. You can specify an endpoint policy to attach to the endpoint that will control access to the service from your VPC. You can also specify the VPC route tables that use the endpoint. An <code>interface</code> endpoint is a network interface in your subnet that serves as an endpoint for communicating with the specified service. You can specify the subnets in which to create an endpoint, and the security groups to associate with the endpoint network interface. Use <a>DescribeVpcEndpointServices</a> to get a list of supported services.

        :param self:
        :param PrivateDnsEnabled:  (Interface endpoint) Indicate whether to associate a private hosted zone with the specified VPC. The private hosted zone contains a record set for the default public DNS name for the service for the Region (for example, <code>kinesis.us-east-1.amazonaws.com</code>) which resolves to the private IP addresses of the endpoint network interfaces in the VPC. This enables you to make requests to the default public DNS name for the service instead of the public DNS names that are automatically generated by the VPC endpoint service. To use a private hosted zone, you must set the following VPC attributes to <code>true</code>: <code>enableDnsHostnames</code> and <code>enableDnsSupport</code>. Use <a>ModifyVpcAttribute</a> to set the VPC attributes. Default: <code>true</code> <br/>ClientToken:  Unique, case-sensitive identifier you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.<br/>SecurityGroupIds:  (Interface endpoint) The ID of one or more security groups to associate with the endpoint network interface.<br/>SubnetIds:  (Interface endpoint) The ID of one or more subnets in which to create an endpoint network interface.<br/>RouteTableIds:  (Gateway endpoint) One or more route table IDs.<br/>PolicyDocument:  A policy to attach to the endpoint that controls access to the service. The policy must be in valid JSON format. If this parameter is not specified, we attach a default policy that allows full access to the service.<br/>ServiceName:  The service name. To get a list of available services, use the <a>DescribeVpcEndpointServices</a> request, or get the name from the service provider.<br/>VpcId:  The ID of the VPC in which the endpoint will be used.<br/>VpcEndpointType:  The type of endpoint. Default: Gateway<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: ClientToken:  Unique, case-sensitive identifier you provide to ensure the idempotency of the request.<br/>VpcEndpoint:  Information about the endpoint.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The ID of the AWS account that owns the VPC endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> Any tags assigned to the VPC endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTimestamp</i> The date and time the VPC endpoint was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DnsEntries</i> (Interface endpoint) The DNS entries for the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkInterfaceIds</i> (Interface endpoint) One or more network interfaces for the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequesterManaged</i> Indicates whether the VPC endpoint is being managed by its service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrivateDnsEnabled</i> (Interface endpoint) Indicates whether the VPC is associated with a private hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Groups</i> (Interface endpoint) Information about the security groups associated with the network interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> (Interface endpoint) One or more subnets in which the endpoint is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RouteTableIds</i> (Gateway endpoint) One or more route tables associated with the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyDocument</i> The policy document associated with the endpoint, if applicable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the VPC endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceName</i> The name of the service to which the endpoint is associated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the VPC to which the endpoint is associated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcEndpointType</i> The type of endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcEndpointId</i> The ID of the VPC endpoint.<br/>        """
        pass

    def create_vpc_endpoint_connection_notification(self, ConnectionEvents: List[String], ConnectionNotificationArn: str, DryRun: Boolean=None, ServiceId: str=None, VpcEndpointId: str=None, ClientToken: str=None) -> CreateVpcEndpointConnectionNotificationResult:
        """Creates a connection notification for a specified VPC endpoint or VPC endpoint service. A connection notification notifies you of specific endpoint events. You must create an SNS topic to receive notifications. For more information, see <a href="https://docs.aws.amazon.com/sns/latest/dg/CreateTopic.html">Create a Topic</a> in the <i>Amazon Simple Notification Service Developer Guide</i>. You can create a connection notification for interface endpoints only.

        :param self:
        :param ClientToken:  Unique, case-sensitive identifier you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.<br/>ConnectionEvents:  One or more endpoint events for which to receive notifications. Valid values are <code>Accept</code>, <code>Connect</code>, <code>Delete</code>, and <code>Reject</code>.<br/>ConnectionNotificationArn:  The ARN of the SNS topic for the notifications.<br/>VpcEndpointId:  The ID of the endpoint. <br/>ServiceId:  The ID of the endpoint service.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: ClientToken:  Unique, case-sensitive identifier you provide to ensure the idempotency of the request.<br/>ConnectionNotification:  Information about the notification.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectionNotificationState</i> The state of the notification.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectionEvents</i> The events for the notification. Valid values are <code>Accept</code>, <code>Connect</code>, <code>Delete</code>, and <code>Reject</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectionNotificationArn</i> The ARN of the SNS topic for the notification.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectionNotificationType</i> The type of notification.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcEndpointId</i> The ID of the VPC endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceId</i> The ID of the endpoint service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectionNotificationId</i> The ID of the notification.<br/>        """
        pass

    def create_vpc_endpoint_service_configuration(self, NetworkLoadBalancerArns: List[String], DryRun: Boolean=None, AcceptanceRequired: Boolean=None, ClientToken: str=None) -> CreateVpcEndpointServiceConfigurationResult:
        """Creates a VPC endpoint service configuration to which service consumers (AWS accounts, IAM users, and IAM roles) can connect. Service consumers can create an interface VPC endpoint to connect to your service. To create an endpoint service configuration, you must first create a Network Load Balancer for your service. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/endpoint-service.html">VPC Endpoint Services</a> in the <i>Amazon Virtual Private Cloud User Guide</i>. 

        :param self:
        :param ClientToken:  Unique, case-sensitive identifier you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.<br/>NetworkLoadBalancerArns:  The Amazon Resource Names (ARNs) of one or more Network Load Balancers for your service.<br/>AcceptanceRequired:  Indicate whether requests from service consumers to create an endpoint to your service must be accepted. To accept a request, use <a>AcceptVpcEndpointConnections</a>.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: ClientToken:  Unique, case-sensitive identifier you provide to ensure the idempotency of the request.<br/>ServiceConfiguration:  Information about the service configuration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> Any tags assigned to the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrivateDnsName</i> The private DNS name for the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BaseEndpointDnsNames</i> The DNS names for the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkLoadBalancerArns</i> The Amazon Resource Names (ARNs) of the Network Load Balancers for the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManagesVpcEndpoints</i> Indicates whether the service manages it's VPC endpoints. Management of the service VPC endpoints using the VPC endpoint API is restricted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AcceptanceRequired</i> Indicates whether requests from other AWS accounts to create an endpoint to the service must first be accepted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> In the Availability Zones in which the service is available.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceState</i> The service state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceName</i> The name of the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceId</i> The ID of the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceType</i> The type of service.<br/>        """
        pass

    def create_vpc_peering_connection(self, DryRun: Boolean=None, PeerOwnerId: str=None, PeerVpcId: str=None, VpcId: str=None, PeerRegion: str=None) -> CreateVpcPeeringConnectionResult:
        """Requests a VPC peering connection between two VPCs: a requester VPC that you own and an accepter VPC with which to create the connection. The accepter VPC can belong to another AWS account and can be in a different Region to the requester VPC. The requester VPC and accepter VPC cannot have overlapping CIDR blocks. <note> Limitations and rules apply to a VPC peering connection. For more information, see the <a href="https://docs.aws.amazon.com/vpc/latest/peering/vpc-peering-basics.html#vpc-peering-limitations">limitations</a> section in the <i>VPC Peering Guide</i>. </note> The owner of the accepter VPC must accept the peering request to activate the peering connection. The VPC peering connection request expires after 7 days, after which it cannot be accepted or rejected. If you create a VPC peering connection request between VPCs with overlapping CIDR blocks, the VPC peering connection has a status of <code>failed</code>.

        :param self:
        :param PeerRegion:  The Region code for the accepter VPC, if the accepter VPC is located in a Region other than the Region in which you make the request. Default: The Region in which you make the request.<br/>VpcId:  The ID of the requester VPC. You must specify this parameter in the request.<br/>PeerVpcId:  The ID of the VPC with which you are creating the VPC peering connection. You must specify this parameter in the request.<br/>PeerOwnerId:  The AWS account ID of the owner of the accepter VPC. Default: Your AWS account ID<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: VpcPeeringConnection:  Information about the VPC peering connection.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcPeeringConnectionId</i> The ID of the VPC peering connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> Any tags assigned to the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the VPC peering connection.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>: The status of the VPC peering connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequesterVpcInfo</i> Information about the requester VPC. CIDR block information is only returned when describing an active VPC peering connection.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RequesterVpcInfo</b>: Information about the requester VPC. CIDR block information is only returned when describing an active VPC peering connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpirationTime</i> The time that an unaccepted VPC peering connection will expire.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccepterVpcInfo</i> Information about the accepter VPC. CIDR block information is only returned when describing an active VPC peering connection.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AccepterVpcInfo</b>: Information about the accepter VPC. CIDR block information is only returned when describing an active VPC peering connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The AWS account ID of the VPC owner.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CidrBlockSet</i> Information about the IPv4 CIDR blocks for the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Ipv6CidrBlockSet</i> The IPv6 CIDR block for the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CidrBlock</i> The IPv4 CIDR block for the VPC.<br/>Information about the VPC peering connection options for the accepter or requester VPC.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PeeringOptions</b>: Information about the VPC peering connection options for the accepter or requester VPC.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The AWS account ID of the VPC owner.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CidrBlockSet</i> Information about the IPv4 CIDR blocks for the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Ipv6CidrBlockSet</i> The IPv6 CIDR block for the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CidrBlock</i> The IPv4 CIDR block for the VPC.<br/>Information about the VPC peering connection options for the accepter or requester VPC.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PeeringOptions</b>: Information about the VPC peering connection options for the accepter or requester VPC.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> A message that provides more information about the status, if applicable.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Code</i> The status of the VPC peering connection.<br/>        """
        pass

    def create_vpn_connection(self, Type: str, CustomerGatewayId: str, VpnGatewayId: str=None, TransitGatewayId: str=None, DryRun: Boolean=None, Options: VpnConnectionOptionsSpecification=None) -> CreateVpnConnectionResult:
        """Creates a VPN connection between an existing virtual private gateway and a VPN customer gateway. The supported connection types is <code>ipsec.1</code>. The response includes information that you need to give to your network administrator to configure your customer gateway. <important> We strongly recommend that you use HTTPS when calling this operation because the response contains sensitive cryptographic information for configuring your customer gateway. </important> If you decide to shut down your VPN connection for any reason and later create a new VPN connection, you must reconfigure your customer gateway with the new information returned from this call. This is an idempotent operation. If you perform the operation more than once, Amazon EC2 doesn't return an error. For more information, see <a href="https://docs.aws.amazon.com/vpn/latest/s2svpn/VPC_VPN.html">AWS Site-to-Site VPN</a> in the <i>AWS Site-to-Site VPN User Guide</i>.

        :param self:
        :param Options:  The options for the VPN connection.:<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>TransitGatewayId:  The ID of the transit gateway. If you specify a transit gateway, you cannot specify a virtual private gateway.<br/>VpnGatewayId:  The ID of the virtual private gateway. If you specify a virtual private gateway, you cannot specify a transit gateway.<br/>Type:  The type of VPN connection (<code>ipsec.1</code>).<br/>CustomerGatewayId:  The ID of the customer gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TunnelOptions</i> The tunnel options for the VPN connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StaticRoutesOnly</i> Indicate whether the VPN connection uses static routes only. If you are creating a VPN connection for a device that does not support BGP, you must specify <code>true</code>. Use <a>CreateVpnConnectionRoute</a> to create a static route. Default: <code>false</code> <br/>
        :return: VpnConnection:  Information about the VPN connection.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VgwTelemetry</i> Information about the VPN tunnel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> Any tags assigned to the VPN connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Routes</i> The static routes associated with the VPN connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> The VPN connection options.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>: The VPN connection options.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayId</i> The ID of the transit gateway associated with the VPN connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpnGatewayId</i> The ID of the virtual private gateway at the AWS side of the VPN connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpnConnectionId</i> The ID of the VPN connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of VPN connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The current state of the VPN connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Category</i> The category of the VPN connection. A value of <code>VPN</code> indicates an AWS VPN connection. A value of <code>VPN-Classic</code> indicates an AWS Classic VPN connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomerGatewayId</i> The ID of the customer gateway at your end of the VPN connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomerGatewayConfiguration</i> The configuration information for the VPN connection's customer gateway (in the native XML format). This element is always present in the <a>CreateVpnConnection</a> response; however, it's present in the <a>DescribeVpnConnections</a> response only if the VPN connection is in the <code>pending</code> or <code>available</code> state.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StaticRoutesOnly</i> Indicates whether the VPN connection uses static routes only. Static routes must be used for devices that don't support BGP.<br/>        """
        pass

    def create_vpn_connection_route(self, VpnConnectionId: str, DestinationCidrBlock: str):
        """Creates a static route associated with a VPN connection between an existing virtual private gateway and a VPN customer gateway. The static route allows traffic to be routed from the virtual private gateway to the VPN customer gateway. For more information, see <a href="https://docs.aws.amazon.com/vpn/latest/s2svpn/VPC_VPN.html">AWS Site-to-Site VPN</a> in the <i>AWS Site-to-Site VPN User Guide</i>.

        :param self:
        :param VpnConnectionId:  The ID of the VPN connection.<br/>DestinationCidrBlock:  The CIDR block associated with the local subnet of the customer network.<br/>
        :return:         """
        pass

    def create_vpn_gateway(self, Type: str, AvailabilityZone: str=None, AmazonSideAsn: Long=None, DryRun: Boolean=None) -> CreateVpnGatewayResult:
        """Creates a virtual private gateway. A virtual private gateway is the endpoint on the VPC side of your VPN connection. You can create a virtual private gateway before creating the VPC itself. For more information, see <a href="https://docs.aws.amazon.com/vpn/latest/s2svpn/VPC_VPN.html">AWS Site-to-Site VPN</a> in the <i>AWS Site-to-Site VPN User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>AmazonSideAsn:  A private Autonomous System Number (ASN) for the Amazon side of a BGP session. If you're using a 16-bit ASN, it must be in the 64512 to 65534 range. If you're using a 32-bit ASN, it must be in the 4200000000 to 4294967294 range. Default: 64512<br/>Type:  The type of VPN connection this virtual private gateway supports.<br/>AvailabilityZone:  The Availability Zone for the virtual private gateway.<br/>
        :return: VpnGateway:  Information about the virtual private gateway.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> Any tags assigned to the virtual private gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AmazonSideAsn</i> The private Autonomous System Number (ASN) for the Amazon side of a BGP session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpnGatewayId</i> The ID of the virtual private gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcAttachments</i> Any VPCs attached to the virtual private gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of VPN connection the virtual private gateway supports.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The current state of the virtual private gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone where the virtual private gateway was created, if applicable. This field may be empty or not returned.<br/>        """
        pass

    def delete_client_vpn_endpoint(self, ClientVpnEndpointId: str, DryRun: Boolean=None) -> DeleteClientVpnEndpointResult:
        """Deletes the specified Client VPN endpoint. You must disassociate all target networks before you can delete a Client VPN endpoint.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>ClientVpnEndpointId:  The ID of the Client VPN to be deleted.<br/>
        :return: Status:  The current state of the Client VPN endpoint.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> A message about the status of the Client VPN endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Code</i> The state of the Client VPN endpoint. Possible states include: <ul> <li>  <code>pending-associate</code> - The Client VPN endpoint has been created but no target networks have been associated. The Client VPN endpoint cannot accept connections. </li> <li>  <code>available</code> - The Client VPN endpoint has been created and a target network has been associated. The Client VPN endpoint can accept connections. </li> <li>  <code>deleting</code> - The Client VPN endpoint is being deleted. The Client VPN endpoint cannot accept connections. </li> <li>  <code>deleted</code> - The Client VPN endpoint has been deleted. The Client VPN endpoint cannot accept connections. </li> </ul><br/>        """
        pass

    def delete_client_vpn_route(self, DestinationCidrBlock: str, ClientVpnEndpointId: str, TargetVpcSubnetId: str=None, DryRun: Boolean=None) -> DeleteClientVpnRouteResult:
        """Deletes a route from a Client VPN endpoint. You can only delete routes that you manually added using the <b>CreateClientVpnRoute</b> action. You cannot delete routes that were automatically added when associating a subnet. To remove routes that have been automatically added, disassociate the target subnet from the Client VPN endpoint.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>DestinationCidrBlock:  The IPv4 address range, in CIDR notation, of the route to be deleted.<br/>TargetVpcSubnetId:  The ID of the target subnet used by the route.<br/>ClientVpnEndpointId:  The ID of the Client VPN endpoint from which the route is to be deleted.<br/>
        :return: Status:  The current state of the route.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> A message about the status of the Client VPN endpoint route, if applicable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Code</i> The state of the Client VPN endpoint route.<br/>        """
        pass

    def delete_customer_gateway(self, CustomerGatewayId: str, DryRun: Boolean=None):
        """Deletes the specified customer gateway. You must delete the VPN connection before you can delete the customer gateway.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>CustomerGatewayId:  The ID of the customer gateway.<br/>
        :return:         """
        pass

    def delete_dhcp_options(self, DhcpOptionsId: str, DryRun: Boolean=None):
        """Deletes the specified set of DHCP options. You must disassociate the set of DHCP options before you can delete it. You can disassociate the set of DHCP options by associating either a new set of options or the default set of options with the VPC.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>DhcpOptionsId:  The ID of the DHCP options set.<br/>
        :return:         """
        pass

    def delete_egress_only_internet_gateway(self, EgressOnlyInternetGatewayId: str, DryRun: Boolean=None) -> DeleteEgressOnlyInternetGatewayResult:
        """Deletes an egress-only internet gateway.

        :param self:
        :param EgressOnlyInternetGatewayId:  The ID of the egress-only internet gateway.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: ReturnCode:  Returns <code>true</code> if the request succeeds; otherwise, it returns an error.<br/>        """
        pass

    def delete_fleets(self, TerminateInstances: Boolean, FleetIds: List[FleetIdentifier], DryRun: Boolean=None) -> DeleteFleetsResult:
        """Deletes the specified EC2 Fleet. After you delete an EC2 Fleet, it launches no new instances. You must specify whether an EC2 Fleet should also terminate its instances. If you terminate the instances, the EC2 Fleet enters the <code>deleted_terminating</code> state. Otherwise, the EC2 Fleet enters the <code>deleted_running</code> state, and the instances continue to run until they are interrupted or you terminate them manually. 

        :param self:
        :param TerminateInstances:  Indicates whether to terminate instances for an EC2 Fleet if it is deleted successfully.<br/>FleetIds:  The IDs of the EC2 Fleets.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: UnsuccessfulFleetDeletions:  Information about the EC2 Fleets that are not successfully deleted.<br/>SuccessfulFleetDeletions:  Information about the EC2 Fleets that are successfully deleted.<br/>        """
        pass

    def delete_flow_logs(self, FlowLogIds: List[String], DryRun: Boolean=None) -> DeleteFlowLogsResult:
        """Deletes one or more flow logs.

        :param self:
        :param FlowLogIds:  One or more flow log IDs. Constraint: Maximum of 1000 flow log IDs.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: Unsuccessful:  Information about the flow logs that could not be deleted successfully.<br/>        """
        pass

    def delete_fpga_image(self, FpgaImageId: str, DryRun: Boolean=None) -> DeleteFpgaImageResult:
        """Deletes the specified Amazon FPGA Image (AFI).

        :param self:
        :param FpgaImageId:  The ID of the AFI.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: Return:  Is <code>true</code> if the request succeeds, and an error otherwise.<br/>        """
        pass

    def delete_internet_gateway(self, InternetGatewayId: str, DryRun: Boolean=None):
        """Deletes the specified internet gateway. You must detach the internet gateway from the VPC before you can delete it.

        :param self:
        :param InternetGatewayId:  The ID of the internet gateway.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return:         """
        pass

    def delete_key_pair(self, KeyName: str, DryRun: Boolean=None):
        """Deletes the specified key pair, by removing the public key from Amazon EC2.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>KeyName:  The name of the key pair.<br/>
        :return:         """
        pass

    def delete_launch_template(self, DryRun: Boolean=None, LaunchTemplateId: str=None, LaunchTemplateName: str=None) -> DeleteLaunchTemplateResult:
        """Deletes a launch template. Deleting a launch template deletes all of its versions.

        :param self:
        :param LaunchTemplateName:  The name of the launch template. You must specify either the launch template ID or launch template name in the request.<br/>LaunchTemplateId:  The ID of the launch template. You must specify either the launch template ID or launch template name in the request.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: LaunchTemplate:  Information about the launch template.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The tags for the launch template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestVersionNumber</i> The version number of the latest version of the launch template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultVersionNumber</i> The version number of the default version of the launch template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedBy</i> The principal that created the launch template. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateTime</i> The time launch template was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LaunchTemplateName</i> The name of the launch template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LaunchTemplateId</i> The ID of the launch template.<br/>        """
        pass

    def delete_launch_template_versions(self, Versions: List[String], DryRun: Boolean=None, LaunchTemplateId: str=None, LaunchTemplateName: str=None) -> DeleteLaunchTemplateVersionsResult:
        """Deletes one or more versions of a launch template. You cannot delete the default version of a launch template; you must first assign a different version as the default. If the default version is the only version for the launch template, you must delete the entire launch template using <a>DeleteLaunchTemplate</a>.

        :param self:
        :param Versions:  The version numbers of one or more launch template versions to delete.<br/>LaunchTemplateName:  The name of the launch template. You must specify either the launch template ID or launch template name in the request.<br/>LaunchTemplateId:  The ID of the launch template. You must specify either the launch template ID or launch template name in the request.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: UnsuccessfullyDeletedLaunchTemplateVersions:  Information about the launch template versions that could not be deleted.<br/>SuccessfullyDeletedLaunchTemplateVersions:  Information about the launch template versions that were successfully deleted.<br/>        """
        pass

    def delete_nat_gateway(self, NatGatewayId: str) -> DeleteNatGatewayResult:
        """Deletes the specified NAT gateway. Deleting a NAT gateway disassociates its Elastic IP address, but does not release the address from your account. Deleting a NAT gateway does not delete any NAT gateway routes in your route tables.

        :param self:
        :param NatGatewayId:  The ID of the NAT gateway.<br/>
        :return: NatGatewayId:  The ID of the NAT gateway.<br/>        """
        pass

    def delete_network_acl(self, NetworkAclId: str, DryRun: Boolean=None):
        """Deletes the specified network ACL. You can't delete the ACL if it's associated with any subnets. You can't delete the default network ACL.

        :param self:
        :param NetworkAclId:  The ID of the network ACL.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return:         """
        pass

    def delete_network_acl_entry(self, RuleNumber: int, NetworkAclId: str, Egress: Boolean, DryRun: Boolean=None):
        """Deletes the specified ingress or egress entry (rule) from the specified network ACL.

        :param self:
        :param RuleNumber:  The rule number of the entry to delete.<br/>NetworkAclId:  The ID of the network ACL.<br/>Egress:  Indicates whether the rule is an egress rule.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return:         """
        pass

    def delete_network_interface(self, NetworkInterfaceId: str, DryRun: Boolean=None):
        """Deletes the specified network interface. You must detach the network interface before you can delete it.

        :param self:
        :param NetworkInterfaceId:  The ID of the network interface.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return:         """
        pass

    def delete_network_interface_permission(self, NetworkInterfacePermissionId: str, Force: Boolean=None, DryRun: Boolean=None) -> DeleteNetworkInterfacePermissionResult:
        """Deletes a permission for a network interface. By default, you cannot delete the permission if the account for which you're removing the permission has attached the network interface to an instance. However, you can force delete the permission, regardless of any attachment.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Force:  Specify <code>true</code> to remove the permission even if the network interface is attached to an instance.<br/>NetworkInterfacePermissionId:  The ID of the network interface permission.<br/>
        :return: Return:  Returns <code>true</code> if the request succeeds, otherwise returns an error.<br/>        """
        pass

    def delete_placement_group(self, GroupName: str, DryRun: Boolean=None):
        """Deletes the specified placement group. You must terminate all instances in the placement group before you can delete the placement group. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/placement-groups.html">Placement Groups</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param GroupName:  The name of the placement group.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return:         """
        pass

    def delete_route(self, RouteTableId: str, DestinationCidrBlock: str=None, DestinationIpv6CidrBlock: str=None, DryRun: Boolean=None):
        """Deletes the specified route from the specified route table.

        :param self:
        :param RouteTableId:  The ID of the route table.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>DestinationIpv6CidrBlock:  The IPv6 CIDR range for the route. The value you specify must match the CIDR for the route exactly.<br/>DestinationCidrBlock:  The IPv4 CIDR range for the route. The value you specify must match the CIDR for the route exactly.<br/>
        :return:         """
        pass

    def delete_route_table(self, RouteTableId: str, DryRun: Boolean=None):
        """Deletes the specified route table. You must disassociate the route table from any subnets before you can delete it. You can't delete the main route table.

        :param self:
        :param RouteTableId:  The ID of the route table.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return:         """
        pass

    def delete_security_group(self, GroupId: str=None, GroupName: str=None, DryRun: Boolean=None):
        """Deletes a security group. If you attempt to delete a security group that is associated with an instance, or is referenced by another security group, the operation fails with <code>InvalidGroup.InUse</code> in EC2-Classic or <code>DependencyViolation</code> in EC2-VPC.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>GroupName:  [EC2-Classic, default VPC] The name of the security group. You can specify either the security group name or the security group ID.<br/>GroupId:  The ID of the security group. Required for a nondefault VPC.<br/>
        :return:         """
        pass

    def delete_snapshot(self, SnapshotId: str, DryRun: Boolean=None):
        """Deletes the specified snapshot. When you make periodic snapshots of a volume, the snapshots are incremental, and only the blocks on the device that have changed since your last snapshot are saved in the new snapshot. When you delete a snapshot, only the data not needed for any other snapshot is removed. So regardless of which prior snapshots have been deleted, all active snapshots will have access to all the information needed to restore the volume. You cannot delete a snapshot of the root device of an EBS volume used by a registered AMI. You must first de-register the AMI before you can delete the snapshot. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ebs-deleting-snapshot.html">Deleting an Amazon EBS Snapshot</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>SnapshotId:  The ID of the EBS snapshot.<br/>
        :return:         """
        pass

    def delete_spot_datafeed_subscription(self, DryRun: Boolean=None):
        """Deletes the data feed for Spot Instances.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return:         """
        pass

    def delete_subnet(self, SubnetId: str, DryRun: Boolean=None):
        """Deletes the specified subnet. You must terminate all running instances in the subnet before you can delete the subnet.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>SubnetId:  The ID of the subnet.<br/>
        :return:         """
        pass

    def delete_tags(self, Resources: List[String], DryRun: Boolean=None, Tags: List[Tag]=None):
        """Deletes the specified set of tags from the specified set of resources. To list the current tags, use <a>DescribeTags</a>. For more information about tags, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Using_Tags.html">Tagging Your Resources</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param Tags:  The tags to delete. Specify a tag key and an optional tag value to delete specific tags. If you specify a tag key without a tag value, we delete any tag with this key regardless of its value. If you specify a tag key with an empty string as the tag value, we delete the tag only if its value is an empty string. If you omit this parameter, we delete all user-defined tags for the specified resources. We do not delete AWS-generated tags (tags that have the <code>aws:</code> prefix).<br/>Resources:  The IDs of the resources, separated by spaces. Constraints: Up to 1000 resource IDs. We recommend breaking up this request into smaller batches.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return:         """
        pass

    def delete_traffic_mirror_filter(self, TrafficMirrorFilterId: str, DryRun: Boolean=None) -> DeleteTrafficMirrorFilterResult:
        """Deletes the specified Traffic Mirror filter. You cannot delete a Traffic Mirror filter that is in use by a Traffic Mirror session.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>TrafficMirrorFilterId:  The ID of the Traffic Mirror filter.<br/>
        :return: TrafficMirrorFilterId:  The ID of the Traffic Mirror filter.<br/>        """
        pass

    def delete_traffic_mirror_filter_rule(self, TrafficMirrorFilterRuleId: str, DryRun: Boolean=None) -> DeleteTrafficMirrorFilterRuleResult:
        """Deletes the specified Traffic Mirror rule.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>TrafficMirrorFilterRuleId:  The ID of the Traffic Mirror rule.<br/>
        :return: TrafficMirrorFilterRuleId:  The ID of the deleted Traffic Mirror rule.<br/>        """
        pass

    def delete_traffic_mirror_session(self, TrafficMirrorSessionId: str, DryRun: Boolean=None) -> DeleteTrafficMirrorSessionResult:
        """Deletes the specified Traffic Mirror session.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>TrafficMirrorSessionId:  The ID of the Traffic Mirror session.<br/>
        :return: TrafficMirrorSessionId:  The ID of the deleted Traffic Mirror session.<br/>        """
        pass

    def delete_traffic_mirror_target(self, TrafficMirrorTargetId: str, DryRun: Boolean=None) -> DeleteTrafficMirrorTargetResult:
        """Deletes the specified Traffic Mirror target. You cannot delete a Traffic Mirror target that is in use by a Traffic Mirror session.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>TrafficMirrorTargetId:  The ID of the Traffic Mirror target.<br/>
        :return: TrafficMirrorTargetId:  The ID of the deleted Traffic Mirror target.<br/>        """
        pass

    def delete_transit_gateway(self, TransitGatewayId: str, DryRun: Boolean=None) -> DeleteTransitGatewayResult:
        """Deletes the specified transit gateway.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>TransitGatewayId:  The ID of the transit gateway.<br/>
        :return: TransitGateway:  Information about the deleted transit gateway.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The tags for the transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> The transit gateway options.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>: The transit gateway options.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The creation time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The ID of the AWS account ID that owns the transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayArn</i> The Amazon Resource Name (ARN) of the transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayId</i> The ID of the transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DnsSupport</i> Indicates whether DNS support is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VpnEcmpSupport</i> Indicates whether Equal Cost Multipath Protocol support is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PropagationDefaultRouteTableId</i> The ID of the default propagation route table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultRouteTablePropagation</i> Indicates whether resource attachments automatically propagate routes to the default propagation route table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociationDefaultRouteTableId</i> The ID of the default association route table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultRouteTableAssociation</i> Indicates whether resource attachments are automatically associated with the default association route table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoAcceptSharedAttachments</i> Indicates whether attachment requests are automatically accepted.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AmazonSideAsn</i> A private Autonomous System Number (ASN) for the Amazon side of a BGP session. The range is 64512 to 65534 for 16-bit ASNs and 4200000000 to 4294967294 for 32-bit ASNs.<br/>        """
        pass

    def delete_transit_gateway_route(self, DestinationCidrBlock: str, TransitGatewayRouteTableId: str, DryRun: Boolean=None) -> DeleteTransitGatewayRouteResult:
        """Deletes the specified route from the specified transit gateway route table.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>DestinationCidrBlock:  The CIDR range for the route. This must match the CIDR for the route exactly.<br/>TransitGatewayRouteTableId:  The ID of the transit gateway route table.<br/>
        :return: Route:  Information about the route.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the route.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The route type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayAttachments</i> The attachments.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationCidrBlock</i> The CIDR block used for destination matches.<br/>        """
        pass

    def delete_transit_gateway_route_table(self, TransitGatewayRouteTableId: str, DryRun: Boolean=None) -> DeleteTransitGatewayRouteTableResult:
        """Deletes the specified transit gateway route table. You must disassociate the route table from any transit gateway route tables before you can delete it.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>TransitGatewayRouteTableId:  The ID of the transit gateway route table.<br/>
        :return: TransitGatewayRouteTable:  Information about the deleted transit gateway route table.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> Any tags assigned to the route table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The creation time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultPropagationRouteTable</i> Indicates whether this is the default propagation route table for the transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultAssociationRouteTable</i> Indicates whether this is the default association route table for the transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the transit gateway route table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayId</i> The ID of the transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayRouteTableId</i> The ID of the transit gateway route table.<br/>        """
        pass

    def delete_transit_gateway_vpc_attachment(self, TransitGatewayAttachmentId: str, DryRun: Boolean=None) -> DeleteTransitGatewayVpcAttachmentResult:
        """Deletes the specified VPC attachment.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>TransitGatewayAttachmentId:  The ID of the attachment.<br/>
        :return: TransitGatewayVpcAttachment:  Information about the deleted VPC attachment.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The tags for the VPC attachment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> The VPC attachment options.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>: The VPC attachment options.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The creation time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> The IDs of the subnets.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the VPC attachment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcOwnerId</i> The ID of the AWS account that owns the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayId</i> The ID of the transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayAttachmentId</i> The ID of the attachment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Ipv6Support</i> Indicates whether IPv6 support is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DnsSupport</i> Indicates whether DNS support is enabled.<br/>        """
        pass

    def delete_volume(self, VolumeId: str, DryRun: Boolean=None):
        """Deletes the specified EBS volume. The volume must be in the <code>available</code> state (not attached to an instance). The volume can remain in the <code>deleting</code> state for several minutes. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ebs-deleting-volume.html">Deleting an Amazon EBS Volume</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>VolumeId:  The ID of the volume.<br/>
        :return:         """
        pass

    def delete_vpc(self, VpcId: str, DryRun: Boolean=None):
        """Deletes the specified VPC. You must detach or delete all gateways and resources that are associated with the VPC before you can delete it. For example, you must terminate all instances running in the VPC, delete all security groups associated with the VPC (except the default one), delete all route tables associated with the VPC (except the default one), and so on.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>VpcId:  The ID of the VPC.<br/>
        :return:         """
        pass

    def delete_vpc_endpoint_connection_notifications(self, ConnectionNotificationIds: List[String], DryRun: Boolean=None) -> DeleteVpcEndpointConnectionNotificationsResult:
        """Deletes one or more VPC endpoint connection notifications.

        :param self:
        :param ConnectionNotificationIds:  One or more notification IDs.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: Unsuccessful:  Information about the notifications that could not be deleted successfully.<br/>        """
        pass

    def delete_vpc_endpoint_service_configurations(self, ServiceIds: List[String], DryRun: Boolean=None) -> DeleteVpcEndpointServiceConfigurationsResult:
        """Deletes one or more VPC endpoint service configurations in your account. Before you delete the endpoint service configuration, you must reject any <code>Available</code> or <code>PendingAcceptance</code> interface endpoint connections that are attached to the service.

        :param self:
        :param ServiceIds:  The IDs of one or more services.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: Unsuccessful:  Information about the service configurations that were not deleted, if applicable.<br/>        """
        pass

    def delete_vpc_endpoints(self, VpcEndpointIds: List[String], DryRun: Boolean=None) -> DeleteVpcEndpointsResult:
        """Deletes one or more specified VPC endpoints. Deleting a gateway endpoint also deletes the endpoint routes in the route tables that were associated with the endpoint. Deleting an interface endpoint deletes the endpoint network interfaces.

        :param self:
        :param VpcEndpointIds:  One or more VPC endpoint IDs.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: Unsuccessful:  Information about the VPC endpoints that were not successfully deleted.<br/>        """
        pass

    def delete_vpc_peering_connection(self, VpcPeeringConnectionId: str, DryRun: Boolean=None) -> DeleteVpcPeeringConnectionResult:
        """Deletes a VPC peering connection. Either the owner of the requester VPC or the owner of the accepter VPC can delete the VPC peering connection if it's in the <code>active</code> state. The owner of the requester VPC can delete a VPC peering connection in the <code>pending-acceptance</code> state. You cannot delete a VPC peering connection that's in the <code>failed</code> state.

        :param self:
        :param VpcPeeringConnectionId:  The ID of the VPC peering connection.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: Return:  Returns <code>true</code> if the request succeeds; otherwise, it returns an error.<br/>        """
        pass

    def delete_vpn_connection(self, VpnConnectionId: str, DryRun: Boolean=None):
        """Deletes the specified VPN connection. If you're deleting the VPC and its associated components, we recommend that you detach the virtual private gateway from the VPC and delete the VPC before deleting the VPN connection. If you believe that the tunnel credentials for your VPN connection have been compromised, you can delete the VPN connection and create a new one that has new keys, without needing to delete the VPC or virtual private gateway. If you create a new VPN connection, you must reconfigure the customer gateway using the new configuration information returned with the new VPN connection ID.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>VpnConnectionId:  The ID of the VPN connection.<br/>
        :return:         """
        pass

    def delete_vpn_connection_route(self, VpnConnectionId: str, DestinationCidrBlock: str):
        """Deletes the specified static route associated with a VPN connection between an existing virtual private gateway and a VPN customer gateway. The static route allows traffic to be routed from the virtual private gateway to the VPN customer gateway.

        :param self:
        :param VpnConnectionId:  The ID of the VPN connection.<br/>DestinationCidrBlock:  The CIDR block associated with the local subnet of the customer network.<br/>
        :return:         """
        pass

    def delete_vpn_gateway(self, VpnGatewayId: str, DryRun: Boolean=None):
        """Deletes the specified virtual private gateway. We recommend that before you delete a virtual private gateway, you detach it from the VPC and delete the VPN connection. Note that you don't need to delete the virtual private gateway if you plan to delete and recreate the VPN connection between your VPC and your network.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>VpnGatewayId:  The ID of the virtual private gateway.<br/>
        :return:         """
        pass

    def deprovision_byoip_cidr(self, Cidr: str, DryRun: Boolean=None) -> DeprovisionByoipCidrResult:
        """Releases the specified address range that you provisioned for use with your AWS resources through bring your own IP addresses (BYOIP) and deletes the corresponding address pool. Before you can release an address range, you must stop advertising it using <a>WithdrawByoipCidr</a> and you must not have any IP addresses allocated from its address range.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Cidr:  The public IPv4 address range, in CIDR notation. The prefix must be the same prefix that you specified when you provisioned the address range.<br/>
        :return: ByoipCidr:  Information about the address range.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the address pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> Upon success, contains the ID of the address pool. Otherwise, contains an error message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the address range.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Cidr</i> The public IPv4 address range, in CIDR notation.<br/>        """
        pass

    def deregister_image(self, ImageId: str, DryRun: Boolean=None):
        """Deregisters the specified AMI. After you deregister an AMI, it can't be used to launch new instances; however, it doesn't affect any instances that you've already launched from the AMI. You'll continue to incur usage costs for those instances until you terminate them. When you deregister an Amazon EBS-backed AMI, it doesn't affect the snapshot that was created for the root volume of the instance during the AMI creation process. When you deregister an instance store-backed AMI, it doesn't affect the files that you uploaded to Amazon S3 when you created the AMI.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>ImageId:  The ID of the AMI.<br/>
        :return:         """
        pass

    def describe_account_attributes(self, AttributeNames: List[AccountAttributeName]=None, DryRun: Boolean=None) -> DescribeAccountAttributesResult:
        """Describes attributes of your AWS account. The following are the supported account attributes: <ul> <li>  <code>supported-platforms</code>: Indicates whether your account can launch instances into EC2-Classic and EC2-VPC, or only into EC2-VPC. </li> <li>  <code>default-vpc</code>: The ID of the default VPC for your account, or <code>none</code>. </li> <li>  <code>max-instances</code>: The maximum number of On-Demand Instances that you can run. </li> <li>  <code>vpc-max-security-groups-per-interface</code>: The maximum number of security groups that you can assign to a network interface. </li> <li>  <code>max-elastic-ips</code>: The maximum number of Elastic IP addresses that you can allocate for use with EC2-Classic.  </li> <li>  <code>vpc-max-elastic-ips</code>: The maximum number of Elastic IP addresses that you can allocate for use with EC2-VPC. </li> </ul>

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>AttributeNames:  The account attribute names.<br/>
        :return: AccountAttributes:  Information about the account attributes.<br/>        """
        pass

    def describe_addresses(self, Filters: List[Filter]=None, PublicIps: List[String]=None, AllocationIds: List[String]=None, DryRun: Boolean=None) -> DescribeAddressesResult:
        """Describes the specified Elastic IP addresses or all of your Elastic IP addresses. An Elastic IP address is for use in either the EC2-Classic platform or in a VPC. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/elastic-ip-addresses-eip.html">Elastic IP Addresses</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>AllocationIds:  [EC2-VPC] Information about the allocation IDs.<br/>PublicIps:  One or more Elastic IP addresses. Default: Describes all your Elastic IP addresses.<br/>Filters:  One or more filters. Filter names and values are case-sensitive. <ul> <li>  <code>allocation-id</code> - [EC2-VPC] The allocation ID for the address. </li> <li>  <code>association-id</code> - [EC2-VPC] The association ID for the address. </li> <li>  <code>domain</code> - Indicates whether the address is for use in EC2-Classic (<code>standard</code>) or in a VPC (<code>vpc</code>). </li> <li>  <code>instance-id</code> - The ID of the instance the address is associated with, if any. </li> <li>  <code>network-interface-id</code> - [EC2-VPC] The ID of the network interface that the address is associated with, if any. </li> <li>  <code>network-interface-owner-id</code> - The AWS account ID of the owner. </li> <li>  <code>private-ip-address</code> - [EC2-VPC] The private IP address associated with the Elastic IP address. </li> <li>  <code>public-ip</code> - The Elastic IP address. </li> <li>  <code>tag</code>:&lt;key&gt; - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value. </li> <li>  <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value. </li> </ul><br/>
        :return: Addresses:  Information about the Elastic IP addresses.<br/>        """
        pass

    def describe_aggregate_id_format(self, DryRun: Boolean=None) -> DescribeAggregateIdFormatResult:
        """Describes the longer ID format settings for all resource types in a specific Region. This request is useful for performing a quick audit to determine whether a specific Region is fully opted in for longer IDs (17-character IDs). This request only returns information about resource types that support longer IDs. The following resource types support longer IDs: <code>bundle</code> | <code>conversion-task</code> | <code>customer-gateway</code> | <code>dhcp-options</code> | <code>elastic-ip-allocation</code> | <code>elastic-ip-association</code> | <code>export-task</code> | <code>flow-log</code> | <code>image</code> | <code>import-task</code> | <code>instance</code> | <code>internet-gateway</code> | <code>network-acl</code> | <code>network-acl-association</code> | <code>network-interface</code> | <code>network-interface-attachment</code> | <code>prefix-list</code> | <code>reservation</code> | <code>route-table</code> | <code>route-table-association</code> | <code>security-group</code> | <code>snapshot</code> | <code>subnet</code> | <code>subnet-cidr-block-association</code> | <code>volume</code> | <code>vpc</code> | <code>vpc-cidr-block-association</code> | <code>vpc-endpoint</code> | <code>vpc-peering-connection</code> | <code>vpn-connection</code> | <code>vpn-gateway</code>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: Statuses:  Information about each resource's ID format.<br/>UseLongIdsAggregated:  Indicates whether all resource types in the Region are configured to use longer IDs. This value is only <code>true</code> if all users are configured to use longer IDs for all resources types in the Region.<br/>        """
        pass

    def describe_availability_zones(self, Filters: List[Filter]=None, ZoneNames: List[String]=None, ZoneIds: List[String]=None, DryRun: Boolean=None) -> DescribeAvailabilityZonesResult:
        """Describes the Availability Zones that are available to you. The results include zones only for the Region you're currently using. If there is an event impacting an Availability Zone, you can use this request to view the state and any provided message for that Availability Zone. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/using-regions-availability-zones.html">Regions and Availability Zones</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>ZoneIds:  The IDs of the Availability Zones.<br/>ZoneNames:  The names of the Availability Zones.<br/>Filters:  The filters. <ul> <li>  <code>message</code> - Information about the Availability Zone. </li> <li>  <code>region-name</code> - The name of the Region for the Availability Zone (for example, <code>us-east-1</code>). </li> <li>  <code>state</code> - The state of the Availability Zone (<code>available</code> | <code>information</code> | <code>impaired</code> | <code>unavailable</code>). </li> <li>  <code>zone-id</code> - The ID of the Availability Zone (for example, <code>use1-az1</code>). </li> <li>  <code>zone-name</code> - The name of the Availability Zone (for example, <code>us-east-1a</code>). </li> </ul><br/>
        :return: AvailabilityZones:  Information about the Availability Zones.<br/>        """
        pass

    def describe_bundle_tasks(self, BundleIds: List[String]=None, Filters: List[Filter]=None, DryRun: Boolean=None) -> DescribeBundleTasksResult:
        """Describes the specified bundle tasks or all of your bundle tasks. <note> Completed bundle tasks are listed for only a limited time. If your bundle task is no longer in the list, you can still register an AMI from it. Just use <code>RegisterImage</code> with the Amazon S3 bucket name and image manifest name you provided to the bundle task. </note>

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Filters:  The filters. <ul> <li>  <code>bundle-id</code> - The ID of the bundle task. </li> <li>  <code>error-code</code> - If the task failed, the error code returned. </li> <li>  <code>error-message</code> - If the task failed, the error message returned. </li> <li>  <code>instance-id</code> - The ID of the instance. </li> <li>  <code>progress</code> - The level of task completion, as a percentage (for example, 20%). </li> <li>  <code>s3-bucket</code> - The Amazon S3 bucket to store the AMI. </li> <li>  <code>s3-prefix</code> - The beginning of the AMI name. </li> <li>  <code>start-time</code> - The time the task started (for example, 2013-09-15T17:15:20.000Z). </li> <li>  <code>state</code> - The state of the task (<code>pending</code> | <code>waiting-for-shutdown</code> | <code>bundling</code> | <code>storing</code> | <code>cancelling</code> | <code>complete</code> | <code>failed</code>). </li> <li>  <code>update-time</code> - The time of the most recent update for the task. </li> </ul><br/>BundleIds:  The bundle task IDs. Default: Describes all your bundle tasks.<br/>
        :return: BundleTasks:  Information about the bundle tasks.<br/>        """
        pass

    def describe_byoip_cidrs(self, MaxResults: int, DryRun: Boolean=None, NextToken: str=None) -> DescribeByoipCidrsResult:
        """Describes the IP address ranges that were specified in calls to <a>ProvisionByoipCidr</a>. To describe the address pools that were created when you provisioned the address ranges, use <a>DescribePublicIpv4Pools</a>.

        :param self:
        :param NextToken:  The token for the next page of results.<br/>MaxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>ByoipCidrs:  Information about your address ranges.<br/>        """
        pass

    def describe_capacity_reservations(self, CapacityReservationIds: List[String]=None, NextToken: str=None, MaxResults: int=None, Filters: List[Filter]=None, DryRun: Boolean=None) -> DescribeCapacityReservationsResult:
        """Describes one or more of your Capacity Reservations. The results describe only the Capacity Reservations in the AWS Region that you're currently using.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Filters:  One or more filters.<br/>MaxResults:  The maximum number of results to return for the request in a single page. The remaining results can be seen by sending another request with the returned nextToken value.<br/>NextToken:  The token to retrieve the next page of results.<br/>CapacityReservationIds:  The ID of the Capacity Reservation.<br/>
        :return: CapacityReservations:  Information about the Capacity Reservations.<br/>NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>        """
        pass

    def describe_classic_link_instances(self, Filters: List[Filter]=None, DryRun: Boolean=None, InstanceIds: List[String]=None, MaxResults: int=None, NextToken: str=None) -> DescribeClassicLinkInstancesResult:
        """Describes one or more of your linked EC2-Classic instances. This request only returns information about EC2-Classic instances linked to a VPC through ClassicLink. You cannot use this request to return information about other instances.

        :param self:
        :param NextToken:  The token for the next page of results.<br/>MaxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value. Constraint: If the value is greater than 1000, we return only 1000 items.<br/>InstanceIds:  One or more instance IDs. Must be instances linked to a VPC through ClassicLink.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Filters:  One or more filters. <ul> <li>  <code>group-id</code> - The ID of a VPC security group that's associated with the instance. </li> <li>  <code>instance-id</code> - The ID of the instance. </li> <li>  <code>tag</code>:&lt;key&gt; - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value. </li> <li>  <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value. </li> <li>  <code>vpc-id</code> - The ID of the VPC to which the instance is linked.  <code>vpc-id</code> - The ID of the VPC that the instance is linked to. </li> </ul><br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>Instances:  Information about one or more linked EC2-Classic instances.<br/>        """
        pass

    def describe_client_vpn_authorization_rules(self, ClientVpnEndpointId: str, DryRun: Boolean=None, NextToken: str=None, Filters: List[Filter]=None, MaxResults: int=None) -> DescribeClientVpnAuthorizationRulesResult:
        """Describes the authorization rules for a specified Client VPN endpoint.

        :param self:
        :param MaxResults:  The maximum number of results to return for the request in a single page. The remaining results can be seen by sending another request with the nextToken value.<br/>Filters:  One or more filters. Filter names and values are case-sensitive.<br/>NextToken:  The token to retrieve the next page of results.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>ClientVpnEndpointId:  The ID of the Client VPN endpoint.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>AuthorizationRules:  Information about the authorization rules.<br/>        """
        pass

    def describe_client_vpn_connections(self, ClientVpnEndpointId: str, Filters: List[Filter]=None, NextToken: str=None, MaxResults: int=None, DryRun: Boolean=None) -> DescribeClientVpnConnectionsResult:
        """Describes active client connections and connections that have been terminated within the last 60 minutes for the specified Client VPN endpoint.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>MaxResults:  The maximum number of results to return for the request in a single page. The remaining results can be seen by sending another request with the nextToken value.<br/>NextToken:  The token to retrieve the next page of results.<br/>Filters:  One or more filters. Filter names and values are case-sensitive.<br/>ClientVpnEndpointId:  The ID of the Client VPN endpoint.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>Connections:  Information about the active and terminated client connections.<br/>        """
        pass

    def describe_client_vpn_endpoints(self, ClientVpnEndpointIds: List[String]=None, MaxResults: int=None, NextToken: str=None, Filters: List[Filter]=None, DryRun: Boolean=None) -> DescribeClientVpnEndpointsResult:
        """Describes one or more Client VPN endpoints in the account.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Filters:  One or more filters. Filter names and values are case-sensitive.<br/>NextToken:  The token to retrieve the next page of results.<br/>MaxResults:  The maximum number of results to return for the request in a single page. The remaining results can be seen by sending another request with the nextToken value.<br/>ClientVpnEndpointIds:  The ID of the Client VPN endpoint.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>ClientVpnEndpoints:  Information about the Client VPN endpoints.<br/>        """
        pass

    def describe_client_vpn_routes(self, ClientVpnEndpointId: str, Filters: List[Filter]=None, MaxResults: int=None, NextToken: str=None, DryRun: Boolean=None) -> DescribeClientVpnRoutesResult:
        """Describes the routes for the specified Client VPN endpoint.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>NextToken:  The token to retrieve the next page of results.<br/>MaxResults:  The maximum number of results to return for the request in a single page. The remaining results can be seen by sending another request with the nextToken value.<br/>Filters:  One or more filters. Filter names and values are case-sensitive.<br/>ClientVpnEndpointId:  The ID of the Client VPN endpoint.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>Routes:  Information about the Client VPN endpoint routes.<br/>        """
        pass

    def describe_client_vpn_target_networks(self, ClientVpnEndpointId: str, AssociationIds: List[String]=None, MaxResults: int=None, NextToken: str=None, Filters: List[Filter]=None, DryRun: Boolean=None) -> DescribeClientVpnTargetNetworksResult:
        """Describes the target networks associated with the specified Client VPN endpoint.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Filters:  One or more filters. Filter names and values are case-sensitive.<br/>NextToken:  The token to retrieve the next page of results.<br/>MaxResults:  The maximum number of results to return for the request in a single page. The remaining results can be seen by sending another request with the nextToken value.<br/>AssociationIds:  The IDs of the target network associations.<br/>ClientVpnEndpointId:  The ID of the Client VPN endpoint.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>ClientVpnTargetNetworks:  Information about the associated target networks.<br/>        """
        pass

    def describe_conversion_tasks(self, ConversionTaskIds: List[String]=None, DryRun: Boolean=None) -> DescribeConversionTasksResult:
        """Describes the specified conversion tasks or all your conversion tasks. For more information, see the <a href="https://docs.aws.amazon.com/vm-import/latest/userguide/">VM Import/Export User Guide</a>. For information about the import manifest referenced by this API action, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/manifest.html">VM Import Manifest</a>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>ConversionTaskIds:  The conversion task IDs.<br/>
        :return: ConversionTasks:  Information about the conversion tasks.<br/>        """
        pass

    def describe_customer_gateways(self, CustomerGatewayIds: List[String]=None, Filters: List[Filter]=None, DryRun: Boolean=None) -> DescribeCustomerGatewaysResult:
        """Describes one or more of your VPN customer gateways. For more information, see <a href="https://docs.aws.amazon.com/vpn/latest/s2svpn/VPC_VPN.html">AWS Site-to-Site VPN</a> in the <i>AWS Site-to-Site VPN User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Filters:  One or more filters. <ul> <li>  <code>bgp-asn</code> - The customer gateway's Border Gateway Protocol (BGP) Autonomous System Number (ASN). </li> <li>  <code>customer-gateway-id</code> - The ID of the customer gateway. </li> <li>  <code>ip-address</code> - The IP address of the customer gateway's Internet-routable external interface. </li> <li>  <code>state</code> - The state of the customer gateway (<code>pending</code> | <code>available</code> | <code>deleting</code> | <code>deleted</code>). </li> <li>  <code>type</code> - The type of customer gateway. Currently, the only supported type is <code>ipsec.1</code>. </li> <li>  <code>tag</code>:&lt;key&gt; - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value. </li> <li>  <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value. </li> </ul><br/>CustomerGatewayIds:  One or more customer gateway IDs. Default: Describes all your customer gateways.<br/>
        :return: CustomerGateways:  Information about one or more customer gateways.<br/>        """
        pass

    def describe_dhcp_options(self, DhcpOptionsIds: List[String]=None, Filters: List[Filter]=None, DryRun: Boolean=None, NextToken: str=None, MaxResults: int=None) -> DescribeDhcpOptionsResult:
        """Describes one or more of your DHCP options sets. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_DHCP_Options.html">DHCP Options Sets</a> in the <i>Amazon Virtual Private Cloud User Guide</i>.

        :param self:
        :param MaxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>NextToken:  The token for the next page of results.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Filters:  One or more filters. <ul> <li>  <code>dhcp-options-id</code> - The ID of a DHCP options set. </li> <li>  <code>key</code> - The key for one of the options (for example, <code>domain-name</code>). </li> <li>  <code>value</code> - The value for one of the options. </li> <li>  <code>owner-id</code> - The ID of the AWS account that owns the DHCP options set. </li> <li>  <code>tag</code>:&lt;key&gt; - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value. </li> <li>  <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value. </li> </ul><br/>DhcpOptionsIds:  The IDs of one or more DHCP options sets. Default: Describes all your DHCP options sets.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>DhcpOptions:  Information about one or more DHCP options sets.<br/>        """
        pass

    def describe_egress_only_internet_gateways(self, DryRun: Boolean=None, EgressOnlyInternetGatewayIds: List[EgressOnlyInternetGatewayId]=None, MaxResults: int=None, NextToken: str=None) -> DescribeEgressOnlyInternetGatewaysResult:
        """Describes one or more of your egress-only internet gateways.

        :param self:
        :param NextToken:  The token for the next page of results.<br/>MaxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>EgressOnlyInternetGatewayIds:  One or more egress-only internet gateway IDs.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>EgressOnlyInternetGateways:  Information about the egress-only internet gateways.<br/>        """
        pass

    def describe_elastic_gpus(self, ElasticGpuIds: List[String]=None, DryRun: Boolean=None, Filters: List[Filter]=None, MaxResults: int=None, NextToken: str=None) -> DescribeElasticGpusResult:
        """Describes the Elastic Graphics accelerator associated with your instances. For more information about Elastic Graphics, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/WindowsGuide/elastic-graphics.html">Amazon Elastic Graphics</a>.

        :param self:
        :param NextToken:  The token to request the next page of results.<br/>MaxResults:  The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value. This value can be between 5 and 1000.<br/>Filters:  The filters. <ul> <li>  <code>availability-zone</code> - The Availability Zone in which the Elastic Graphics accelerator resides. </li> <li>  <code>elastic-gpu-health</code> - The status of the Elastic Graphics accelerator (<code>OK</code> | <code>IMPAIRED</code>). </li> <li>  <code>elastic-gpu-state</code> - The state of the Elastic Graphics accelerator (<code>ATTACHED</code>). </li> <li>  <code>elastic-gpu-type</code> - The type of Elastic Graphics accelerator; for example, <code>eg1.medium</code>. </li> <li>  <code>instance-id</code> - The ID of the instance to which the Elastic Graphics accelerator is associated. </li> </ul><br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>ElasticGpuIds:  The Elastic Graphics accelerator IDs.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>MaxResults:  The total number of items to return. If the total number of items available is more than the value specified in max-items then a Next-Token will be provided in the output that you can use to resume pagination.<br/>ElasticGpuSet:  Information about the Elastic Graphics accelerators.<br/>        """
        pass

    def describe_export_tasks(self, ExportTaskIds: List[String]=None) -> DescribeExportTasksResult:
        """Describes the specified export tasks or all your export tasks.

        :param self:
        :param ExportTaskIds:  The export task IDs.<br/>
        :return: ExportTasks:  Information about the export tasks.<br/>        """
        pass

    def describe_fleet_history(self, StartTime: DateTime, FleetId: str, DryRun: Boolean=None, EventType: str=None, MaxResults: int=None, NextToken: str=None) -> DescribeFleetHistoryResult:
        """Describes the events for the specified EC2 Fleet during the specified time.

        :param self:
        :param StartTime:  The start date and time for the events, in UTC format (for example, <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).<br/>FleetId:  The ID of the EC2 Fleet.<br/>NextToken:  The token for the next set of results.<br/>MaxResults:  The maximum number of results to return in a single call. Specify a value between 1 and 1000. The default value is 1000. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value.<br/>EventType:  The type of events to describe. By default, all events are described.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: StartTime:  The start date and time for the events, in UTC format (for example, <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).<br/>FleetId:  The ID of the EC Fleet.<br/>NextToken:  The token for the next set of results.<br/>LastEvaluatedTime:  The last date and time for the events, in UTC format (for example, <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z). All records up to this time were retrieved. If <code>nextToken</code> indicates that there are more results, this value is not present.<br/>HistoryRecords:  Information about the events in the history of the EC2 Fleet.<br/>        """
        pass

    def describe_fleet_instances(self, FleetId: str, DryRun: Boolean=None, MaxResults: int=None, NextToken: str=None, Filters: List[Filter]=None) -> DescribeFleetInstancesResult:
        """Describes the running instances for the specified EC2 Fleet.

        :param self:
        :param Filters:  The filters. <ul> <li>  <code>instance-type</code> - The instance type. </li> </ul><br/>FleetId:  The ID of the EC2 Fleet.<br/>NextToken:  The token for the next set of results.<br/>MaxResults:  The maximum number of results to return in a single call. Specify a value between 1 and 1000. The default value is 1000. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: FleetId:  The ID of the EC2 Fleet.<br/>NextToken:  The token for the next set of results.<br/>ActiveInstances:  The running instances. This list is refreshed periodically and might be out of date.<br/>        """
        pass

    def describe_fleets(self, DryRun: Boolean=None, MaxResults: int=None, NextToken: str=None, FleetIds: List[FleetIdentifier]=None, Filters: List[Filter]=None) -> DescribeFleetsResult:
        """Describes the specified EC2 Fleets or all your EC2 Fleets.

        :param self:
        :param Filters:  The filters. <ul> <li>  <code>activity-status</code> - The progress of the EC2 Fleet ( <code>error</code> | <code>pending-fulfillment</code> | <code>pending-termination</code> | <code>fulfilled</code>). </li> <li>  <code>excess-capacity-termination-policy</code> - Indicates whether to terminate running instances if the target capacity is decreased below the current EC2 Fleet size (<code>true</code> | <code>false</code>). </li> <li>  <code>fleet-state</code> - The state of the EC2 Fleet (<code>submitted</code> | <code>active</code> | <code>deleted</code> | <code>failed</code> | <code>deleted-running</code> | <code>deleted-terminating</code> | <code>modifying</code>). </li> <li>  <code>replace-unhealthy-instances</code> - Indicates whether EC2 Fleet should replace unhealthy instances (<code>true</code> | <code>false</code>). </li> <li>  <code>type</code> - The type of request (<code>instant</code> | <code>request</code> | <code>maintain</code>). </li> </ul><br/>FleetIds:  The ID of the EC2 Fleets.<br/>NextToken:  The token for the next set of results.<br/>MaxResults:  The maximum number of results to return in a single call. Specify a value between 1 and 1000. The default value is 1000. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: Fleets:  Information about the EC2 Fleets.<br/>NextToken:  The token for the next set of results.<br/>        """
        pass

    def describe_flow_logs(self, DryRun: Boolean=None, Filter: List[Filter]=None, FlowLogIds: List[String]=None, MaxResults: int=None, NextToken: str=None) -> DescribeFlowLogsResult:
        """Describes one or more flow logs. To view the information in your flow logs (the log streams for the network interfaces), you must use the CloudWatch Logs console or the CloudWatch Logs API.

        :param self:
        :param NextToken:  The token for the next page of results.<br/>MaxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>FlowLogIds:  One or more flow log IDs. Constraint: Maximum of 1000 flow log IDs.<br/>Filter:  One or more filters. <ul> <li>  <code>deliver-log-status</code> - The status of the logs delivery (<code>SUCCESS</code> | <code>FAILED</code>). </li> <li>  <code>log-destination-type</code> - The type of destination to which the flow log publishes data. Possible destination types include <code>cloud-watch-logs</code> and <code>S3</code>. </li> <li>  <code>flow-log-id</code> - The ID of the flow log. </li> <li>  <code>log-group-name</code> - The name of the log group. </li> <li>  <code>resource-id</code> - The ID of the VPC, subnet, or network interface. </li> <li>  <code>traffic-type</code> - The type of traffic (<code>ACCEPT</code> | <code>REJECT</code> | <code>ALL</code>). </li> </ul><br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>FlowLogs:  Information about the flow logs.<br/>        """
        pass

    def describe_fpga_image_attribute(self, Attribute: str, FpgaImageId: str, DryRun: Boolean=None) -> DescribeFpgaImageAttributeResult:
        """Describes the specified attribute of the specified Amazon FPGA Image (AFI).

        :param self:
        :param Attribute:  The AFI attribute.<br/>FpgaImageId:  The ID of the AFI.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: FpgaImageAttribute:  Information about the attribute.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductCodes</i> The product codes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LoadPermissions</i> The load permissions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the AFI.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the AFI.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FpgaImageId</i> The ID of the AFI.<br/>        """
        pass

    def describe_fpga_images(self, DryRun: Boolean=None, FpgaImageIds: List[String]=None, Owners: List[String]=None, Filters: List[Filter]=None, NextToken: str=None, MaxResults: int=None) -> DescribeFpgaImagesResult:
        """Describes the Amazon FPGA Images (AFIs) available to you. These include public AFIs, private AFIs that you own, and AFIs owned by other AWS accounts for which you have load permissions.

        :param self:
        :param MaxResults:  The maximum number of results to return in a single call.<br/>NextToken:  The token to retrieve the next page of results.<br/>Filters:  The filters. <ul> <li>  <code>create-time</code> - The creation time of the AFI. </li> <li>  <code>fpga-image-id</code> - The FPGA image identifier (AFI ID). </li> <li>  <code>fpga-image-global-id</code> - The global FPGA image identifier (AGFI ID). </li> <li>  <code>name</code> - The name of the AFI. </li> <li>  <code>owner-id</code> - The AWS account ID of the AFI owner. </li> <li>  <code>product-code</code> - The product code. </li> <li>  <code>shell-version</code> - The version of the AWS Shell that was used to create the bitstream. </li> <li>  <code>state</code> - The state of the AFI (<code>pending</code> | <code>failed</code> | <code>available</code> | <code>unavailable</code>). </li> <li>  <code>tag</code>:&lt;key&gt; - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value. </li> <li>  <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value. </li> <li>  <code>update-time</code> - The time of the most recent update. </li> </ul><br/>Owners:  Filters the AFI by owner. Specify an AWS account ID, <code>self</code> (owner is the sender of the request), or an AWS owner alias (valid values are <code>amazon</code> | <code>aws-marketplace</code>).<br/>FpgaImageIds:  The AFI IDs.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>FpgaImages:  Information about the FPGA images.<br/>        """
        pass

    def describe_host_reservation_offerings(self, Filter: List[Filter]=None, MaxDuration: int=None, MaxResults: int=None, MinDuration: int=None, NextToken: str=None, OfferingId: str=None) -> DescribeHostReservationOfferingsResult:
        """Describes the Dedicated Host reservations that are available to purchase. The results describe all the Dedicated Host reservation offerings, including offerings that may not match the instance family and Region of your Dedicated Hosts. When purchasing an offering, ensure that the instance family and Region of the offering matches that of the Dedicated Hosts with which it is to be associated. For more information about supported instance types, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/dedicated-hosts-overview.html">Dedicated Hosts Overview</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. 

        :param self:
        :param OfferingId:  The ID of the reservation offering.<br/>NextToken:  The token to use to retrieve the next page of results.<br/>MinDuration:  This is the minimum duration of the reservation you'd like to purchase, specified in seconds. Reservations are available in one-year and three-year terms. The number of seconds specified must be the number of seconds in a year (365x24x60x60) times one of the supported durations (1 or 3). For example, specify 31536000 for one year.<br/>MaxResults:  The maximum number of results to return for the request in a single page. The remaining results can be seen by sending another request with the returned <code>nextToken</code> value. This value can be between 5 and 500. If <code>maxResults</code> is given a larger value than 500, you receive an error.<br/>MaxDuration:  This is the maximum duration of the reservation to purchase, specified in seconds. Reservations are available in one-year and three-year terms. The number of seconds specified must be the number of seconds in a year (365x24x60x60) times one of the supported durations (1 or 3). For example, specify 94608000 for three years.<br/>Filter:  The filters. <ul> <li>  <code>instance-family</code> - The instance family of the offering (for example, <code>m4</code>). </li> <li>  <code>payment-option</code> - The payment option (<code>NoUpfront</code> | <code>PartialUpfront</code> | <code>AllUpfront</code>). </li> </ul><br/>
        :return: OfferingSet:  Information about the offerings.<br/>NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>        """
        pass

    def describe_host_reservations(self, Filter: List[Filter]=None, HostReservationIdSet: List[String]=None, MaxResults: int=None, NextToken: str=None) -> DescribeHostReservationsResult:
        """Describes reservations that are associated with Dedicated Hosts in your account.

        :param self:
        :param NextToken:  The token to use to retrieve the next page of results.<br/>MaxResults:  The maximum number of results to return for the request in a single page. The remaining results can be seen by sending another request with the returned <code>nextToken</code> value. This value can be between 5 and 500. If <code>maxResults</code> is given a larger value than 500, you receive an error.<br/>HostReservationIdSet:  The host reservation IDs.<br/>Filter:  The filters. <ul> <li>  <code>instance-family</code> - The instance family (for example, <code>m4</code>). </li> <li>  <code>payment-option</code> - The payment option (<code>NoUpfront</code> | <code>PartialUpfront</code> | <code>AllUpfront</code>). </li> <li>  <code>state</code> - The state of the reservation (<code>payment-pending</code> | <code>payment-failed</code> | <code>active</code> | <code>retired</code>). </li> <li>  <code>tag</code>:&lt;key&gt; - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value. </li> <li>  <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value. </li> </ul><br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>HostReservationSet:  Details about the reservation's configuration.<br/>        """
        pass

    def describe_hosts(self, Filter: List[Filter]=None, HostIds: List[String]=None, MaxResults: int=None, NextToken: str=None) -> DescribeHostsResult:
        """Describes the specified Dedicated Hosts or all your Dedicated Hosts. The results describe only the Dedicated Hosts in the Region you're currently using. All listed instances consume capacity on your Dedicated Host. Dedicated Hosts that have recently been released are listed with the state <code>released</code>.

        :param self:
        :param NextToken:  The token to use to retrieve the next page of results.<br/>MaxResults:  The maximum number of results to return for the request in a single page. The remaining results can be seen by sending another request with the returned <code>nextToken</code> value. This value can be between 5 and 500. If <code>maxResults</code> is given a larger value than 500, you receive an error. You cannot specify this parameter and the host IDs parameter in the same request.<br/>HostIds:  The IDs of the Dedicated Hosts. The IDs are used for targeted instance launches.<br/>Filter:  The filters. <ul> <li>  <code>auto-placement</code> - Whether auto-placement is enabled or disabled (<code>on</code> | <code>off</code>). </li> <li>  <code>availability-zone</code> - The Availability Zone of the host. </li> <li>  <code>client-token</code> - The idempotency token that you provided when you allocated the host. </li> <li>  <code>host-reservation-id</code> - The ID of the reservation assigned to this host. </li> <li>  <code>instance-type</code> - The instance type size that the Dedicated Host is configured to support. </li> <li>  <code>state</code> - The allocation state of the Dedicated Host (<code>available</code> | <code>under-assessment</code> | <code>permanent-failure</code> | <code>released</code> | <code>released-permanent-failure</code>). </li> <li>  <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value. </li> </ul><br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>Hosts:  Information about the Dedicated Hosts.<br/>        """
        pass

    def describe_iam_instance_profile_associations(self, AssociationIds: List[String]=None, Filters: List[Filter]=None, MaxResults: int=None, NextToken: str=None) -> DescribeIamInstanceProfileAssociationsResult:
        """Describes your IAM instance profile associations.

        :param self:
        :param NextToken:  The token to request the next page of results.<br/>MaxResults:  The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value.<br/>Filters:  The filters. <ul> <li>  <code>instance-id</code> - The ID of the instance. </li> <li>  <code>state</code> - The state of the association (<code>associating</code> | <code>associated</code> | <code>disassociating</code> | <code>disassociated</code>). </li> </ul><br/>AssociationIds:  The IAM instance profile associations.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>IamInstanceProfileAssociations:  Information about the IAM instance profile associations.<br/>        """
        pass

    def describe_id_format(self, Resource: str=None) -> DescribeIdFormatResult:
        """Describes the ID format settings for your resources on a per-Region basis, for example, to view which resource types are enabled for longer IDs. This request only returns information about resource types whose ID formats can be modified; it does not return information about other resource types. The following resource types support longer IDs: <code>bundle</code> | <code>conversion-task</code> | <code>customer-gateway</code> | <code>dhcp-options</code> | <code>elastic-ip-allocation</code> | <code>elastic-ip-association</code> | <code>export-task</code> | <code>flow-log</code> | <code>image</code> | <code>import-task</code> | <code>instance</code> | <code>internet-gateway</code> | <code>network-acl</code> | <code>network-acl-association</code> | <code>network-interface</code> | <code>network-interface-attachment</code> | <code>prefix-list</code> | <code>reservation</code> | <code>route-table</code> | <code>route-table-association</code> | <code>security-group</code> | <code>snapshot</code> | <code>subnet</code> | <code>subnet-cidr-block-association</code> | <code>volume</code> | <code>vpc</code> | <code>vpc-cidr-block-association</code> | <code>vpc-endpoint</code> | <code>vpc-peering-connection</code> | <code>vpn-connection</code> | <code>vpn-gateway</code>.  These settings apply to the IAM user who makes the request; they do not apply to the entire AWS account. By default, an IAM user defaults to the same settings as the root user, unless they explicitly override the settings by running the <a>ModifyIdFormat</a> command. Resources created with longer IDs are visible to all IAM users, regardless of these settings and provided that they have permission to use the relevant <code>Describe</code> command for the resource type.

        :param self:
        :param Resource:  The type of resource: <code>bundle</code> | <code>conversion-task</code> | <code>customer-gateway</code> | <code>dhcp-options</code> | <code>elastic-ip-allocation</code> | <code>elastic-ip-association</code> | <code>export-task</code> | <code>flow-log</code> | <code>image</code> | <code>import-task</code> | <code>instance</code> | <code>internet-gateway</code> | <code>network-acl</code> | <code>network-acl-association</code> | <code>network-interface</code> | <code>network-interface-attachment</code> | <code>prefix-list</code> | <code>reservation</code> | <code>route-table</code> | <code>route-table-association</code> | <code>security-group</code> | <code>snapshot</code> | <code>subnet</code> | <code>subnet-cidr-block-association</code> | <code>volume</code> | <code>vpc</code> | <code>vpc-cidr-block-association</code> | <code>vpc-endpoint</code> | <code>vpc-peering-connection</code> | <code>vpn-connection</code> | <code>vpn-gateway</code> <br/>
        :return: Statuses:  Information about the ID format for the resource.<br/>        """
        pass

    def describe_identity_id_format(self, PrincipalArn: str, Resource: str=None) -> DescribeIdentityIdFormatResult:
        """Describes the ID format settings for resources for the specified IAM user, IAM role, or root user. For example, you can view the resource types that are enabled for longer IDs. This request only returns information about resource types whose ID formats can be modified; it does not return information about other resource types. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/resource-ids.html">Resource IDs</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.  The following resource types support longer IDs: <code>bundle</code> | <code>conversion-task</code> | <code>customer-gateway</code> | <code>dhcp-options</code> | <code>elastic-ip-allocation</code> | <code>elastic-ip-association</code> | <code>export-task</code> | <code>flow-log</code> | <code>image</code> | <code>import-task</code> | <code>instance</code> | <code>internet-gateway</code> | <code>network-acl</code> | <code>network-acl-association</code> | <code>network-interface</code> | <code>network-interface-attachment</code> | <code>prefix-list</code> | <code>reservation</code> | <code>route-table</code> | <code>route-table-association</code> | <code>security-group</code> | <code>snapshot</code> | <code>subnet</code> | <code>subnet-cidr-block-association</code> | <code>volume</code> | <code>vpc</code> | <code>vpc-cidr-block-association</code> | <code>vpc-endpoint</code> | <code>vpc-peering-connection</code> | <code>vpn-connection</code> | <code>vpn-gateway</code>.  These settings apply to the principal specified in the request. They do not apply to the principal that makes the request.

        :param self:
        :param Resource:  The type of resource: <code>bundle</code> | <code>conversion-task</code> | <code>customer-gateway</code> | <code>dhcp-options</code> | <code>elastic-ip-allocation</code> | <code>elastic-ip-association</code> | <code>export-task</code> | <code>flow-log</code> | <code>image</code> | <code>import-task</code> | <code>instance</code> | <code>internet-gateway</code> | <code>network-acl</code> | <code>network-acl-association</code> | <code>network-interface</code> | <code>network-interface-attachment</code> | <code>prefix-list</code> | <code>reservation</code> | <code>route-table</code> | <code>route-table-association</code> | <code>security-group</code> | <code>snapshot</code> | <code>subnet</code> | <code>subnet-cidr-block-association</code> | <code>volume</code> | <code>vpc</code> | <code>vpc-cidr-block-association</code> | <code>vpc-endpoint</code> | <code>vpc-peering-connection</code> | <code>vpn-connection</code> | <code>vpn-gateway</code> <br/>PrincipalArn:  The ARN of the principal, which can be an IAM role, IAM user, or the root user.<br/>
        :return: Statuses:  Information about the ID format for the resources.<br/>        """
        pass

    def describe_image_attribute(self, ImageId: str, Attribute: str, DryRun: Boolean=None) -> ImageAttribute:
        """Describes the specified attribute of the specified AMI. You can specify only one attribute at a time.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>ImageId:  The ID of the AMI.<br/>Attribute:  The AMI attribute.  <b>Note</b>: Depending on your account privileges, the <code>blockDeviceMapping</code> attribute may return a <code>Client.AuthFailure</code> error. If this happens, use <a>DescribeImages</a> to get information about the block device mapping for the AMI.<br/>
        :return: SriovNetSupport:  Indicates whether enhanced networking with the Intel 82599 Virtual Function interface is enabled.:<br/>RamdiskId:  The RAM disk ID.:<br/>KernelId:  The kernel ID.:<br/>Description:  A description for the AMI.:<br/>ProductCodes:  The product codes.<br/>LaunchPermissions:  The launch permissions.<br/>ImageId:  The ID of the AMI.<br/>BlockDeviceMappings:  The block device mapping entries.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The value is case-sensitive.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The value is case-sensitive.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The value is case-sensitive.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The value is case-sensitive.<br/>        """
        pass

    def describe_images(self, ExecutableUsers: List[String]=None, Filters: List[Filter]=None, ImageIds: List[String]=None, Owners: List[String]=None, DryRun: Boolean=None) -> DescribeImagesResult:
        """Describes the specified images (AMIs, AKIs, and ARIs) available to you or all of the images available to you. The images available to you include public images, private images that you own, and private images owned by other AWS accounts for which you have explicit launch permissions. Recently deregistered images appear in the returned results for a short interval and then return empty results. After all instances that reference a deregistered AMI are terminated, specifying the ID of the image results in an error indicating that the AMI ID cannot be found.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Owners:  Filters the images by the owner. Specify an AWS account ID, <code>self</code> (owner is the sender of the request), or an AWS owner alias (valid values are <code>amazon</code> | <code>aws-marketplace</code> | <code>microsoft</code>). Omitting this option returns all images for which you have launch permissions, regardless of ownership.<br/>ImageIds:  The image IDs. Default: Describes all images available to you.<br/>Filters:  The filters. <ul> <li>  <code>architecture</code> - The image architecture (<code>i386</code> | <code>x86_64</code> | <code>arm64</code>). </li> <li>  <code>block-device-mapping.delete-on-termination</code> - A Boolean value that indicates whether the Amazon EBS volume is deleted on instance termination. </li> <li>  <code>block-device-mapping.device-name</code> - The device name specified in the block device mapping (for example, <code>/dev/sdh</code> or <code>xvdh</code>). </li> <li>  <code>block-device-mapping.snapshot-id</code> - The ID of the snapshot used for the EBS volume. </li> <li>  <code>block-device-mapping.volume-size</code> - The volume size of the EBS volume, in GiB. </li> <li>  <code>block-device-mapping.volume-type</code> - The volume type of the EBS volume (<code>gp2</code> | <code>io1</code> | <code>st1 </code>| <code>sc1</code> | <code>standard</code>). </li> <li>  <code>block-device-mapping.encrypted</code> - A Boolean that indicates whether the EBS volume is encrypted. </li> <li>  <code>description</code> - The description of the image (provided during image creation). </li> <li>  <code>ena-support</code> - A Boolean that indicates whether enhanced networking with ENA is enabled. </li> <li>  <code>hypervisor</code> - The hypervisor type (<code>ovm</code> | <code>xen</code>). </li> <li>  <code>image-id</code> - The ID of the image. </li> <li>  <code>image-type</code> - The image type (<code>machine</code> | <code>kernel</code> | <code>ramdisk</code>). </li> <li>  <code>is-public</code> - A Boolean that indicates whether the image is public. </li> <li>  <code>kernel-id</code> - The kernel ID. </li> <li>  <code>manifest-location</code> - The location of the image manifest. </li> <li>  <code>name</code> - The name of the AMI (provided during image creation). </li> <li>  <code>owner-alias</code> - String value from an Amazon-maintained list (<code>amazon</code> | <code>aws-marketplace</code> | <code>microsoft</code>) of snapshot owners. Not to be confused with the user-configured AWS account alias, which is set from the IAM console. </li> <li>  <code>owner-id</code> - The AWS account ID of the image owner. </li> <li>  <code>platform</code> - The platform. To only list Windows-based AMIs, use <code>windows</code>. </li> <li>  <code>product-code</code> - The product code. </li> <li>  <code>product-code.type</code> - The type of the product code (<code>devpay</code> | <code>marketplace</code>). </li> <li>  <code>ramdisk-id</code> - The RAM disk ID. </li> <li>  <code>root-device-name</code> - The device name of the root device volume (for example, <code>/dev/sda1</code>). </li> <li>  <code>root-device-type</code> - The type of the root device volume (<code>ebs</code> | <code>instance-store</code>). </li> <li>  <code>state</code> - The state of the image (<code>available</code> | <code>pending</code> | <code>failed</code>). </li> <li>  <code>state-reason-code</code> - The reason code for the state change. </li> <li>  <code>state-reason-message</code> - The message for the state change. </li> <li>  <code>sriov-net-support</code> - A value of <code>simple</code> indicates that enhanced networking with the Intel 82599 VF interface is enabled. </li> <li>  <code>tag</code>:&lt;key&gt; - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value. </li> <li>  <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value. </li> <li>  <code>virtualization-type</code> - The virtualization type (<code>paravirtual</code> | <code>hvm</code>). </li> </ul><br/>ExecutableUsers:  Scopes the images by users with explicit launch permissions. Specify an AWS account ID, <code>self</code> (the sender of the request), or <code>all</code> (public AMIs).<br/>
        :return: Images:  Information about the images.<br/>        """
        pass

    def describe_import_image_tasks(self, DryRun: Boolean=None, Filters: List[Filter]=None, ImportTaskIds: List[String]=None, MaxResults: int=None, NextToken: str=None) -> DescribeImportImageTasksResult:
        """Displays details about an import virtual machine or import snapshot tasks that are already created.

        :param self:
        :param NextToken:  A token that indicates the next page of results.<br/>MaxResults:  The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value.<br/>ImportTaskIds:  A list of import image task IDs.<br/>Filters:  Filter tasks using the <code>task-state</code> filter and one of the following values: active, completed, deleting, deleted.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: NextToken:  The token to use to get the next page of results. This value is <code>null</code> when there are no more results to return.<br/>ImportImageTasks:  A list of zero or more import image tasks that are currently active or were completed or canceled in the previous 7 days.<br/>        """
        pass

    def describe_import_snapshot_tasks(self, DryRun: Boolean=None, Filters: List[Filter]=None, ImportTaskIds: List[String]=None, MaxResults: int=None, NextToken: str=None) -> DescribeImportSnapshotTasksResult:
        """Describes your import snapshot tasks.

        :param self:
        :param NextToken:  A token that indicates the next page of results.<br/>MaxResults:  The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value.<br/>ImportTaskIds:  A list of import snapshot task IDs.<br/>Filters:  The filters.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: NextToken:  The token to use to get the next page of results. This value is <code>null</code> when there are no more results to return.<br/>ImportSnapshotTasks:  A list of zero or more import snapshot tasks that are currently active or were completed or canceled in the previous 7 days.<br/>        """
        pass

    def describe_instance_attribute(self, InstanceId: str, Attribute: str, DryRun: Boolean=None) -> InstanceAttribute:
        """Describes the specified attribute of the specified instance. You can specify only one attribute at a time. Valid attribute values are: <code>instanceType</code> | <code>kernel</code> | <code>ramdisk</code> | <code>userData</code> | <code>disableApiTermination</code> | <code>instanceInitiatedShutdownBehavior</code> | <code>rootDeviceName</code> | <code>blockDeviceMapping</code> | <code>productCodes</code> | <code>sourceDestCheck</code> | <code>groupSet</code> | <code>ebsOptimized</code> | <code>sriovNetSupport</code> 

        :param self:
        :param InstanceId:  The ID of the instance.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Attribute:  The instance attribute. Note: The <code>enaSupport</code> attribute is not supported at this time.<br/>
        :return: UserData:  The user data.:<br/>SriovNetSupport:  Indicates whether enhanced networking with the Intel 82599 Virtual Function interface is enabled.:<br/>SourceDestCheck:  Indicates whether source/destination checking is enabled. A value of <code>true</code> means that checking is enabled, and <code>false</code> means that checking is disabled. This value must be <code>false</code> for a NAT instance to perform NAT.:<br/>RootDeviceName:  The device name of the root device volume (for example, <code>/dev/sda1</code>).:<br/>RamdiskId:  The RAM disk ID.:<br/>ProductCodes:  A list of product codes.<br/>KernelId:  The kernel ID.:<br/>InstanceType:  The instance type.:<br/>InstanceInitiatedShutdownBehavior:  Indicates whether an instance stops or terminates when you initiate shutdown from the instance (using the operating system command for system shutdown).:<br/>InstanceId:  The ID of the instance.<br/>EbsOptimized:  Indicates whether the instance is optimized for Amazon EBS I/O.:<br/>EnaSupport:  Indicates whether enhanced networking with ENA is enabled.:<br/>DisableApiTermination:  If the value is <code>true</code>, you can't terminate the instance through the Amazon EC2 console, CLI, or API; otherwise, you can.:<br/>BlockDeviceMappings:  The block device mapping of the instance.<br/>Groups:  The security groups associated with the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The valid values are <code>true</code> or <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The valid values are <code>true</code> or <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The valid values are <code>true</code> or <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The value is case-sensitive.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The value is case-sensitive.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The value is case-sensitive.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The value is case-sensitive.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The value is case-sensitive.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The valid values are <code>true</code> or <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The value is case-sensitive.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The value is case-sensitive.<br/>        """
        pass

    def describe_instance_credit_specifications(self, DryRun: Boolean=None, Filters: List[Filter]=None, InstanceIds: List[String]=None, MaxResults: int=None, NextToken: str=None) -> DescribeInstanceCreditSpecificationsResult:
        """Describes the credit option for CPU usage of the specified T2 or T3 instances. The credit options are <code>standard</code> and <code>unlimited</code>. If you do not specify an instance ID, Amazon EC2 returns T2 and T3 instances with the <code>unlimited</code> credit option, as well as instances that were previously configured as T2 or T3 with the <code>unlimited</code> credit option. For example, if you resize a T2 instance, while it is configured as <code>unlimited</code>, to an M4 instance, Amazon EC2 returns the M4 instance. If you specify one or more instance IDs, Amazon EC2 returns the credit option (<code>standard</code> or <code>unlimited</code>) of those instances. If you specify an instance ID that is not valid, such as an instance that is not a T2 or T3 instance, an error is returned. Recently terminated instances might appear in the returned results. This interval is usually less than one hour. If an Availability Zone is experiencing a service disruption and you specify instance IDs in the affected zone, or do not specify any instance IDs at all, the call fails. If you specify only instance IDs in an unaffected zone, the call works normally. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/burstable-performance-instances.html">Burstable Performance Instances</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param NextToken:  The token to retrieve the next page of results.<br/>MaxResults:  The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value. This value can be between 5 and 1000. You cannot specify this parameter and the instance IDs parameter in the same call.<br/>InstanceIds:  The instance IDs. Default: Describes all your instances. Constraints: Maximum 1000 explicitly specified instance IDs.<br/>Filters:  The filters. <ul> <li>  <code>instance-id</code> - The ID of the instance. </li> </ul><br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>InstanceCreditSpecifications:  Information about the credit option for CPU usage of an instance.<br/>        """
        pass

    def describe_instance_status(self, Filters: List[Filter]=None, InstanceIds: List[String]=None, MaxResults: int=None, NextToken: str=None, DryRun: Boolean=None, IncludeAllInstances: Boolean=None) -> DescribeInstanceStatusResult:
        """Describes the status of the specified instances or all of your instances. By default, only running instances are described, unless you specifically indicate to return the status of all instances. Instance status includes the following components: <ul> <li>  <b>Status checks</b> - Amazon EC2 performs status checks on running EC2 instances to identify hardware and software issues. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/monitoring-system-instance-status-check.html">Status Checks for Your Instances</a> and <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/TroubleshootingInstances.html">Troubleshooting Instances with Failed Status Checks</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. </li> <li>  <b>Scheduled events</b> - Amazon EC2 can schedule events (such as reboot, stop, or terminate) for your instances related to hardware issues, software updates, or system maintenance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/monitoring-instances-status-check_sched.html">Scheduled Events for Your Instances</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. </li> <li>  <b>Instance state</b> - You can manage your instances from the moment you launch them through their termination. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-instance-lifecycle.html">Instance Lifecycle</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. </li> </ul>

        :param self:
        :param IncludeAllInstances:  When <code>true</code>, includes the health status for all instances. When <code>false</code>, includes the health status for running instances only. Default: <code>false</code> <br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>NextToken:  The token to retrieve the next page of results.<br/>MaxResults:  The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value. This value can be between 5 and 1000. You cannot specify this parameter and the instance IDs parameter in the same call.<br/>InstanceIds:  The instance IDs. Default: Describes all your instances. Constraints: Maximum 100 explicitly specified instance IDs.<br/>Filters:  The filters. <ul> <li>  <code>availability-zone</code> - The Availability Zone of the instance. </li> <li>  <code>event.code</code> - The code for the scheduled event (<code>instance-reboot</code> | <code>system-reboot</code> | <code>system-maintenance</code> | <code>instance-retirement</code> | <code>instance-stop</code>). </li> <li>  <code>event.description</code> - A description of the event. </li> <li>  <code>event.instance-event-id</code> - The ID of the event whose date and time you are modifying. </li> <li>  <code>event.not-after</code> - The latest end time for the scheduled event (for example, <code>2014-09-15T17:15:20.000Z</code>). </li> <li>  <code>event.not-before</code> - The earliest start time for the scheduled event (for example, <code>2014-09-15T17:15:20.000Z</code>). </li> <li>  <code>event.not-before-deadline</code> - The deadline for starting the event (for example, <code>2014-09-15T17:15:20.000Z</code>). </li> <li>  <code>instance-state-code</code> - The code for the instance state, as a 16-bit unsigned integer. The high byte is used for internal purposes and should be ignored. The low byte is set based on the state represented. The valid values are 0 (pending), 16 (running), 32 (shutting-down), 48 (terminated), 64 (stopping), and 80 (stopped). </li> <li>  <code>instance-state-name</code> - The state of the instance (<code>pending</code> | <code>running</code> | <code>shutting-down</code> | <code>terminated</code> | <code>stopping</code> | <code>stopped</code>). </li> <li>  <code>instance-status.reachability</code> - Filters on instance status where the name is <code>reachability</code> (<code>passed</code> | <code>failed</code> | <code>initializing</code> | <code>insufficient-data</code>). </li> <li>  <code>instance-status.status</code> - The status of the instance (<code>ok</code> | <code>impaired</code> | <code>initializing</code> | <code>insufficient-data</code> | <code>not-applicable</code>). </li> <li>  <code>system-status.reachability</code> - Filters on system status where the name is <code>reachability</code> (<code>passed</code> | <code>failed</code> | <code>initializing</code> | <code>insufficient-data</code>). </li> <li>  <code>system-status.status</code> - The system status of the instance (<code>ok</code> | <code>impaired</code> | <code>initializing</code> | <code>insufficient-data</code> | <code>not-applicable</code>). </li> </ul><br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>InstanceStatuses:  Information about the status of the instances.<br/>        """
        pass

    def describe_instances(self, Filters: List[Filter]=None, InstanceIds: List[String]=None, DryRun: Boolean=None, MaxResults: int=None, NextToken: str=None) -> DescribeInstancesResult:
        """Describes the specified instances or all of AWS account's instances. If you specify one or more instance IDs, Amazon EC2 returns information for those instances. If you do not specify instance IDs, Amazon EC2 returns information for all relevant instances. If you specify an instance ID that is not valid, an error is returned. If you specify an instance that you do not own, it is not included in the returned results. Recently terminated instances might appear in the returned results. This interval is usually less than one hour. If you describe instances in the rare case where an Availability Zone is experiencing a service disruption and you specify instance IDs that are in the affected zone, or do not specify any instance IDs at all, the call fails. If you describe instances and specify only instance IDs that are in an unaffected zone, the call works normally.

        :param self:
        :param NextToken:  The token to request the next page of results.<br/>MaxResults:  The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value. This value can be between 5 and 1000. You cannot specify this parameter and the instance IDs parameter in the same call.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>InstanceIds:  The instance IDs. Default: Describes all your instances.<br/>Filters:  The filters. <ul> <li>  <code>affinity</code> - The affinity setting for an instance running on a Dedicated Host (<code>default</code> | <code>host</code>). </li> <li>  <code>architecture</code> - The instance architecture (<code>i386</code> | <code>x86_64</code> | <code>arm64</code>). </li> <li>  <code>availability-zone</code> - The Availability Zone of the instance. </li> <li>  <code>block-device-mapping.attach-time</code> - The attach time for an EBS volume mapped to the instance, for example, <code>2010-09-15T17:15:20.000Z</code>. </li> <li>  <code>block-device-mapping.delete-on-termination</code> - A Boolean that indicates whether the EBS volume is deleted on instance termination. </li> <li>  <code>block-device-mapping.device-name</code> - The device name specified in the block device mapping (for example, <code>/dev/sdh</code> or <code>xvdh</code>). </li> <li>  <code>block-device-mapping.status</code> - The status for the EBS volume (<code>attaching</code> | <code>attached</code> | <code>detaching</code> | <code>detached</code>). </li> <li>  <code>block-device-mapping.volume-id</code> - The volume ID of the EBS volume. </li> <li>  <code>client-token</code> - The idempotency token you provided when you launched the instance. </li> <li>  <code>dns-name</code> - The public DNS name of the instance. </li> <li>  <code>group-id</code> - The ID of the security group for the instance. EC2-Classic only. </li> <li>  <code>group-name</code> - The name of the security group for the instance. EC2-Classic only. </li> <li>  <code>hibernation-options.configured</code> - A Boolean that indicates whether the instance is enabled for hibernation. A value of <code>true</code> means that the instance is enabled for hibernation.  </li> <li>  <code>host-id</code> - The ID of the Dedicated Host on which the instance is running, if applicable. </li> <li>  <code>hypervisor</code> - The hypervisor type of the instance (<code>ovm</code> | <code>xen</code>). </li> <li>  <code>iam-instance-profile.arn</code> - The instance profile associated with the instance. Specified as an ARN. </li> <li>  <code>image-id</code> - The ID of the image used to launch the instance. </li> <li>  <code>instance-id</code> - The ID of the instance. </li> <li>  <code>instance-lifecycle</code> - Indicates whether this is a Spot Instance or a Scheduled Instance (<code>spot</code> | <code>scheduled</code>). </li> <li>  <code>instance-state-code</code> - The state of the instance, as a 16-bit unsigned integer. The high byte is used for internal purposes and should be ignored. The low byte is set based on the state represented. The valid values are: 0 (pending), 16 (running), 32 (shutting-down), 48 (terminated), 64 (stopping), and 80 (stopped). </li> <li>  <code>instance-state-name</code> - The state of the instance (<code>pending</code> | <code>running</code> | <code>shutting-down</code> | <code>terminated</code> | <code>stopping</code> | <code>stopped</code>). </li> <li>  <code>instance-type</code> - The type of instance (for example, <code>t2.micro</code>). </li> <li>  <code>instance.group-id</code> - The ID of the security group for the instance.  </li> <li>  <code>instance.group-name</code> - The name of the security group for the instance.  </li> <li>  <code>ip-address</code> - The public IPv4 address of the instance. </li> <li>  <code>kernel-id</code> - The kernel ID. </li> <li>  <code>key-name</code> - The name of the key pair used when the instance was launched. </li> <li>  <code>launch-index</code> - When launching multiple instances, this is the index for the instance in the launch group (for example, 0, 1, 2, and so on).  </li> <li>  <code>launch-time</code> - The time when the instance was launched. </li> <li>  <code>monitoring-state</code> - Indicates whether detailed monitoring is enabled (<code>disabled</code> | <code>enabled</code>). </li> <li>  <code>network-interface.addresses.private-ip-address</code> - The private IPv4 address associated with the network interface. </li> <li>  <code>network-interface.addresses.primary</code> - Specifies whether the IPv4 address of the network interface is the primary private IPv4 address. </li> <li>  <code>network-interface.addresses.association.public-ip</code> - The ID of the association of an Elastic IP address (IPv4) with a network interface. </li> <li>  <code>network-interface.addresses.association.ip-owner-id</code> - The owner ID of the private IPv4 address associated with the network interface. </li> <li>  <code>network-interface.association.public-ip</code> - The address of the Elastic IP address (IPv4) bound to the network interface. </li> <li>  <code>network-interface.association.ip-owner-id</code> - The owner of the Elastic IP address (IPv4) associated with the network interface. </li> <li>  <code>network-interface.association.allocation-id</code> - The allocation ID returned when you allocated the Elastic IP address (IPv4) for your network interface. </li> <li>  <code>network-interface.association.association-id</code> - The association ID returned when the network interface was associated with an IPv4 address. </li> <li>  <code>network-interface.attachment.attachment-id</code> - The ID of the interface attachment. </li> <li>  <code>network-interface.attachment.instance-id</code> - The ID of the instance to which the network interface is attached. </li> <li>  <code>network-interface.attachment.instance-owner-id</code> - The owner ID of the instance to which the network interface is attached. </li> <li>  <code>network-interface.attachment.device-index</code> - The device index to which the network interface is attached. </li> <li>  <code>network-interface.attachment.status</code> - The status of the attachment (<code>attaching</code> | <code>attached</code> | <code>detaching</code> | <code>detached</code>). </li> <li>  <code>network-interface.attachment.attach-time</code> - The time that the network interface was attached to an instance. </li> <li>  <code>network-interface.attachment.delete-on-termination</code> - Specifies whether the attachment is deleted when an instance is terminated. </li> <li>  <code>network-interface.availability-zone</code> - The Availability Zone for the network interface. </li> <li>  <code>network-interface.description</code> - The description of the network interface. </li> <li>  <code>network-interface.group-id</code> - The ID of a security group associated with the network interface. </li> <li>  <code>network-interface.group-name</code> - The name of a security group associated with the network interface. </li> <li>  <code>network-interface.ipv6-addresses.ipv6-address</code> - The IPv6 address associated with the network interface. </li> <li>  <code>network-interface.mac-address</code> - The MAC address of the network interface. </li> <li>  <code>network-interface.network-interface-id</code> - The ID of the network interface. </li> <li>  <code>network-interface.owner-id</code> - The ID of the owner of the network interface. </li> <li>  <code>network-interface.private-dns-name</code> - The private DNS name of the network interface. </li> <li>  <code>network-interface.requester-id</code> - The requester ID for the network interface. </li> <li>  <code>network-interface.requester-managed</code> - Indicates whether the network interface is being managed by AWS. </li> <li>  <code>network-interface.status</code> - The status of the network interface (<code>available</code>) | <code>in-use</code>). </li> <li>  <code>network-interface.source-dest-check</code> - Whether the network interface performs source/destination checking. A value of <code>true</code> means that checking is enabled, and <code>false</code> means that checking is disabled. The value must be <code>false</code> for the network interface to perform network address translation (NAT) in your VPC. </li> <li>  <code>network-interface.subnet-id</code> - The ID of the subnet for the network interface. </li> <li>  <code>network-interface.vpc-id</code> - The ID of the VPC for the network interface. </li> <li>  <code>owner-id</code> - The AWS account ID of the instance owner. </li> <li>  <code>placement-group-name</code> - The name of the placement group for the instance. </li> <li>  <code>placement-partition-number</code> - The partition in which the instance is located. </li> <li>  <code>platform</code> - The platform. To list only Windows instances, use <code>windows</code>. </li> <li>  <code>private-dns-name</code> - The private IPv4 DNS name of the instance. </li> <li>  <code>private-ip-address</code> - The private IPv4 address of the instance. </li> <li>  <code>product-code</code> - The product code associated with the AMI used to launch the instance. </li> <li>  <code>product-code.type</code> - The type of product code (<code>devpay</code> | <code>marketplace</code>). </li> <li>  <code>ramdisk-id</code> - The RAM disk ID. </li> <li>  <code>reason</code> - The reason for the current state of the instance (for example, shows "User Initiated [date]" when you stop or terminate the instance). Similar to the state-reason-code filter. </li> <li>  <code>requester-id</code> - The ID of the entity that launched the instance on your behalf (for example, AWS Management Console, Auto Scaling, and so on). </li> <li>  <code>reservation-id</code> - The ID of the instance's reservation. A reservation ID is created any time you launch an instance. A reservation ID has a one-to-one relationship with an instance launch request, but can be associated with more than one instance if you launch multiple instances using the same launch request. For example, if you launch one instance, you get one reservation ID. If you launch ten instances using the same launch request, you also get one reservation ID. </li> <li>  <code>root-device-name</code> - The device name of the root device volume (for example, <code>/dev/sda1</code>). </li> <li>  <code>root-device-type</code> - The type of the root device volume (<code>ebs</code> | <code>instance-store</code>). </li> <li>  <code>source-dest-check</code> - Indicates whether the instance performs source/destination checking. A value of <code>true</code> means that checking is enabled, and <code>false</code> means that checking is disabled. The value must be <code>false</code> for the instance to perform network address translation (NAT) in your VPC.  </li> <li>  <code>spot-instance-request-id</code> - The ID of the Spot Instance request. </li> <li>  <code>state-reason-code</code> - The reason code for the state change. </li> <li>  <code>state-reason-message</code> - A message that describes the state change. </li> <li>  <code>subnet-id</code> - The ID of the subnet for the instance. </li> <li>  <code>tag</code>:&lt;key&gt; - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value. </li> <li>  <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources that have a tag with a specific key, regardless of the tag value. </li> <li>  <code>tenancy</code> - The tenancy of an instance (<code>dedicated</code> | <code>default</code> | <code>host</code>). </li> <li>  <code>virtualization-type</code> - The virtualization type of the instance (<code>paravirtual</code> | <code>hvm</code>). </li> <li>  <code>vpc-id</code> - The ID of the VPC that the instance is running in. </li> </ul><br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>Reservations:  Information about the reservations.<br/>        """
        pass

    def describe_internet_gateways(self, Filters: List[Filter]=None, DryRun: Boolean=None, InternetGatewayIds: List[String]=None, NextToken: str=None, MaxResults: int=None) -> DescribeInternetGatewaysResult:
        """Describes one or more of your internet gateways.

        :param self:
        :param MaxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>NextToken:  The token for the next page of results.<br/>InternetGatewayIds:  One or more internet gateway IDs. Default: Describes all your internet gateways.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Filters:  One or more filters. <ul> <li>  <code>attachment.state</code> - The current state of the attachment between the gateway and the VPC (<code>available</code>). Present only if a VPC is attached. </li> <li>  <code>attachment.vpc-id</code> - The ID of an attached VPC. </li> <li>  <code>internet-gateway-id</code> - The ID of the Internet gateway. </li> <li>  <code>owner-id</code> - The ID of the AWS account that owns the internet gateway. </li> <li>  <code>tag</code>:&lt;key&gt; - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value. </li> <li>  <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value. </li> </ul><br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>InternetGateways:  Information about one or more internet gateways.<br/>        """
        pass

    def describe_key_pairs(self, Filters: List[Filter]=None, KeyNames: List[String]=None, DryRun: Boolean=None) -> DescribeKeyPairsResult:
        """Describes the specified key pairs or all of your key pairs. For more information about key pairs, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-key-pairs.html">Key Pairs</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>KeyNames:  The key pair names. Default: Describes all your key pairs.<br/>Filters:  The filters. <ul> <li>  <code>fingerprint</code> - The fingerprint of the key pair. </li> <li>  <code>key-name</code> - The name of the key pair. </li> </ul><br/>
        :return: KeyPairs:  Information about the key pairs.<br/>        """
        pass

    def describe_launch_template_versions(self, DryRun: Boolean=None, LaunchTemplateId: str=None, LaunchTemplateName: str=None, Versions: List[String]=None, MinVersion: str=None, MaxVersion: str=None, NextToken: str=None, MaxResults: int=None, Filters: List[Filter]=None) -> DescribeLaunchTemplateVersionsResult:
        """Describes one or more versions of a specified launch template. You can describe all versions, individual versions, or a range of versions.

        :param self:
        :param Filters:  One or more filters. <ul> <li>  <code>create-time</code> - The time the launch template version was created. </li> <li>  <code>ebs-optimized</code> - A boolean that indicates whether the instance is optimized for Amazon EBS I/O. </li> <li>  <code>iam-instance-profile</code> - The ARN of the IAM instance profile. </li> <li>  <code>image-id</code> - The ID of the AMI. </li> <li>  <code>instance-type</code> - The instance type. </li> <li>  <code>is-default-version</code> - A boolean that indicates whether the launch template version is the default version. </li> <li>  <code>kernel-id</code> - The kernel ID. </li> <li>  <code>ram-disk-id</code> - The RAM disk ID. </li> </ul><br/>MaxResults:  The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value. This value can be between 1 and 200.<br/>NextToken:  The token to request the next page of results.<br/>MaxVersion:  The version number up to which to describe launch template versions.<br/>MinVersion:  The version number after which to describe launch template versions.<br/>Versions:  One or more versions of the launch template.<br/>LaunchTemplateName:  The name of the launch template. You must specify either the launch template ID or launch template name in the request.<br/>LaunchTemplateId:  The ID of the launch template. You must specify either the launch template ID or launch template name in the request.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>LaunchTemplateVersions:  Information about the launch template versions.<br/>        """
        pass

    def describe_launch_templates(self, DryRun: Boolean=None, LaunchTemplateIds: List[String]=None, LaunchTemplateNames: List[LaunchTemplateName]=None, Filters: List[Filter]=None, NextToken: str=None, MaxResults: int=None) -> DescribeLaunchTemplatesResult:
        """Describes one or more launch templates.

        :param self:
        :param MaxResults:  The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value. This value can be between 1 and 200.<br/>NextToken:  The token to request the next page of results.<br/>Filters:  One or more filters. <ul> <li>  <code>create-time</code> - The time the launch template was created. </li> <li>  <code>launch-template-name</code> - The name of the launch template. </li> <li>  <code>tag</code>:&lt;key&gt; - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value. </li> <li>  <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value. </li> </ul><br/>LaunchTemplateNames:  One or more launch template names.<br/>LaunchTemplateIds:  One or more launch template IDs.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>LaunchTemplates:  Information about the launch templates.<br/>        """
        pass

    def describe_moving_addresses(self, Filters: List[Filter]=None, DryRun: Boolean=None, MaxResults: int=None, NextToken: str=None, PublicIps: List[String]=None) -> DescribeMovingAddressesResult:
        """Describes your Elastic IP addresses that are being moved to the EC2-VPC platform, or that are being restored to the EC2-Classic platform. This request does not return information about any other Elastic IP addresses in your account.

        :param self:
        :param PublicIps:  One or more Elastic IP addresses.<br/>NextToken:  The token for the next page of results.<br/>MaxResults:  The maximum number of results to return for the request in a single page. The remaining results of the initial request can be seen by sending another request with the returned <code>NextToken</code> value. This value can be between 5 and 1000; if <code>MaxResults</code> is given a value outside of this range, an error is returned. Default: If no value is provided, the default is 1000.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Filters:  One or more filters. <ul> <li>  <code>moving-status</code> - The status of the Elastic IP address (<code>MovingToVpc</code> | <code>RestoringToClassic</code>). </li> </ul><br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>MovingAddressStatuses:  The status for each Elastic IP address.<br/>        """
        pass

    def describe_nat_gateways(self, Filter: List[Filter]=None, MaxResults: int=None, NatGatewayIds: List[String]=None, NextToken: str=None) -> DescribeNatGatewaysResult:
        """Describes one or more of your NAT gateways.

        :param self:
        :param NextToken:  The token for the next page of results.<br/>NatGatewayIds:  One or more NAT gateway IDs.<br/>MaxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>Filter:  One or more filters. <ul> <li>  <code>nat-gateway-id</code> - The ID of the NAT gateway. </li> <li>  <code>state</code> - The state of the NAT gateway (<code>pending</code> | <code>failed</code> | <code>available</code> | <code>deleting</code> | <code>deleted</code>). </li> <li>  <code>subnet-id</code> - The ID of the subnet in which the NAT gateway resides. </li> <li>  <code>tag</code>:&lt;key&gt; - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value. </li> <li>  <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value. </li> <li>  <code>vpc-id</code> - The ID of the VPC in which the NAT gateway resides. </li> </ul><br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>NatGateways:  Information about the NAT gateways.<br/>        """
        pass

    def describe_network_acls(self, Filters: List[Filter]=None, DryRun: Boolean=None, NetworkAclIds: List[String]=None, NextToken: str=None, MaxResults: int=None) -> DescribeNetworkAclsResult:
        """Describes one or more of your network ACLs. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_ACLs.html">Network ACLs</a> in the <i>Amazon Virtual Private Cloud User Guide</i>.

        :param self:
        :param MaxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>NextToken:  The token for the next page of results.<br/>NetworkAclIds:  One or more network ACL IDs. Default: Describes all your network ACLs.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Filters:  One or more filters. <ul> <li>  <code>association.association-id</code> - The ID of an association ID for the ACL. </li> <li>  <code>association.network-acl-id</code> - The ID of the network ACL involved in the association. </li> <li>  <code>association.subnet-id</code> - The ID of the subnet involved in the association. </li> <li>  <code>default</code> - Indicates whether the ACL is the default network ACL for the VPC. </li> <li>  <code>entry.cidr</code> - The IPv4 CIDR range specified in the entry. </li> <li>  <code>entry.icmp.code</code> - The ICMP code specified in the entry, if any. </li> <li>  <code>entry.icmp.type</code> - The ICMP type specified in the entry, if any. </li> <li>  <code>entry.ipv6-cidr</code> - The IPv6 CIDR range specified in the entry. </li> <li>  <code>entry.port-range.from</code> - The start of the port range specified in the entry.  </li> <li>  <code>entry.port-range.to</code> - The end of the port range specified in the entry.  </li> <li>  <code>entry.protocol</code> - The protocol specified in the entry (<code>tcp</code> | <code>udp</code> | <code>icmp</code> or a protocol number). </li> <li>  <code>entry.rule-action</code> - Allows or denies the matching traffic (<code>allow</code> | <code>deny</code>). </li> <li>  <code>entry.rule-number</code> - The number of an entry (in other words, rule) in the set of ACL entries. </li> <li>  <code>network-acl-id</code> - The ID of the network ACL. </li> <li>  <code>owner-id</code> - The ID of the AWS account that owns the network ACL. </li> <li>  <code>tag</code>:&lt;key&gt; - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value. </li> <li>  <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value. </li> <li>  <code>vpc-id</code> - The ID of the VPC for the network ACL. </li> </ul><br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>NetworkAcls:  Information about one or more network ACLs.<br/>        """
        pass

    def describe_network_interface_attribute(self, NetworkInterfaceId: str, Attribute: str=None, DryRun: Boolean=None) -> DescribeNetworkInterfaceAttributeResult:
        """Describes a network interface attribute. You can specify only one attribute at a time.

        :param self:
        :param NetworkInterfaceId:  The ID of the network interface.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Attribute:  The attribute of the network interface. This parameter is required.<br/>
        :return: SourceDestCheck:  Indicates whether source/destination checking is enabled.:<br/>NetworkInterfaceId:  The ID of the network interface.<br/>Groups:  The security groups associated with the network interface.<br/>Description:  The description of the network interface.:<br/>Attachment:  The attachment (if any) of the network interface.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The attachment state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceOwnerId</i> The AWS account ID of the owner of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceId</i> The ID of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceIndex</i> The device index of the network interface attachment on the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeleteOnTermination</i> Indicates whether the network interface is deleted when the instance is terminated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttachmentId</i> The ID of the network interface attachment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttachTime</i> The timestamp indicating when the attachment initiated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The value is case-sensitive.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The valid values are <code>true</code> or <code>false</code>.<br/>        """
        pass

    def describe_network_interface_permissions(self, NetworkInterfacePermissionIds: List[String]=None, Filters: List[Filter]=None, NextToken: str=None, MaxResults: int=None) -> DescribeNetworkInterfacePermissionsResult:
        """Describes the permissions for your network interfaces. 

        :param self:
        :param MaxResults:  The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value. If this parameter is not specified, up to 50 results are returned by default.<br/>NextToken:  The token to request the next page of results.<br/>Filters:  One or more filters. <ul> <li>  <code>network-interface-permission.network-interface-permission-id</code> - The ID of the permission. </li> <li>  <code>network-interface-permission.network-interface-id</code> - The ID of the network interface. </li> <li>  <code>network-interface-permission.aws-account-id</code> - The AWS account ID. </li> <li>  <code>network-interface-permission.aws-service</code> - The AWS service. </li> <li>  <code>network-interface-permission.permission</code> - The type of permission (<code>INSTANCE-ATTACH</code> | <code>EIP-ASSOCIATE</code>). </li> </ul><br/>NetworkInterfacePermissionIds:  One or more network interface permission IDs.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results.<br/>NetworkInterfacePermissions:  The network interface permissions.<br/>        """
        pass

    def describe_network_interfaces(self, Filters: List[Filter]=None, DryRun: Boolean=None, NetworkInterfaceIds: List[String]=None, NextToken: str=None, MaxResults: int=None) -> DescribeNetworkInterfacesResult:
        """Describes one or more of your network interfaces.

        :param self:
        :param MaxResults:  The maximum number of items to return for this request. The request returns a token that you can specify in a subsequent call to get the next set of results.<br/>NextToken:  The token to retrieve the next page of results.<br/>NetworkInterfaceIds:  One or more network interface IDs. Default: Describes all your network interfaces.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Filters:  One or more filters. <ul> <li>  <code>addresses.private-ip-address</code> - The private IPv4 addresses associated with the network interface. </li> <li>  <code>addresses.primary</code> - Whether the private IPv4 address is the primary IP address associated with the network interface.  </li> <li>  <code>addresses.association.public-ip</code> - The association ID returned when the network interface was associated with the Elastic IP address (IPv4). </li> <li>  <code>addresses.association.owner-id</code> - The owner ID of the addresses associated with the network interface. </li> <li>  <code>association.association-id</code> - The association ID returned when the network interface was associated with an IPv4 address. </li> <li>  <code>association.allocation-id</code> - The allocation ID returned when you allocated the Elastic IP address (IPv4) for your network interface. </li> <li>  <code>association.ip-owner-id</code> - The owner of the Elastic IP address (IPv4) associated with the network interface. </li> <li>  <code>association.public-ip</code> - The address of the Elastic IP address (IPv4) bound to the network interface. </li> <li>  <code>association.public-dns-name</code> - The public DNS name for the network interface (IPv4). </li> <li>  <code>attachment.attachment-id</code> - The ID of the interface attachment. </li> <li>  <code>attachment.attach.time</code> - The time that the network interface was attached to an instance. </li> <li>  <code>attachment.delete-on-termination</code> - Indicates whether the attachment is deleted when an instance is terminated. </li> <li>  <code>attachment.device-index</code> - The device index to which the network interface is attached. </li> <li>  <code>attachment.instance-id</code> - The ID of the instance to which the network interface is attached. </li> <li>  <code>attachment.instance-owner-id</code> - The owner ID of the instance to which the network interface is attached. </li> <li>  <code>attachment.nat-gateway-id</code> - The ID of the NAT gateway to which the network interface is attached. </li> <li>  <code>attachment.status</code> - The status of the attachment (<code>attaching</code> | <code>attached</code> | <code>detaching</code> | <code>detached</code>). </li> <li>  <code>availability-zone</code> - The Availability Zone of the network interface. </li> <li>  <code>description</code> - The description of the network interface. </li> <li>  <code>group-id</code> - The ID of a security group associated with the network interface. </li> <li>  <code>group-name</code> - The name of a security group associated with the network interface. </li> <li>  <code>ipv6-addresses.ipv6-address</code> - An IPv6 address associated with the network interface. </li> <li>  <code>mac-address</code> - The MAC address of the network interface. </li> <li>  <code>network-interface-id</code> - The ID of the network interface. </li> <li>  <code>owner-id</code> - The AWS account ID of the network interface owner. </li> <li>  <code>private-ip-address</code> - The private IPv4 address or addresses of the network interface. </li> <li>  <code>private-dns-name</code> - The private DNS name of the network interface (IPv4). </li> <li>  <code>requester-id</code> - The ID of the entity that launched the instance on your behalf (for example, AWS Management Console, Auto Scaling, and so on). </li> <li>  <code>requester-managed</code> - Indicates whether the network interface is being managed by an AWS service (for example, AWS Management Console, Auto Scaling, and so on). </li> <li>  <code>source-dest-check</code> - Indicates whether the network interface performs source/destination checking. A value of <code>true</code> means checking is enabled, and <code>false</code> means checking is disabled. The value must be <code>false</code> for the network interface to perform network address translation (NAT) in your VPC.  </li> <li>  <code>status</code> - The status of the network interface. If the network interface is not attached to an instance, the status is <code>available</code>; if a network interface is attached to an instance the status is <code>in-use</code>. </li> <li>  <code>subnet-id</code> - The ID of the subnet for the network interface. </li> <li>  <code>tag</code>:&lt;key&gt; - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value. </li> <li>  <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value. </li> <li>  <code>vpc-id</code> - The ID of the VPC for the network interface. </li> </ul><br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>NetworkInterfaces:  Information about one or more network interfaces.<br/>        """
        pass

    def describe_placement_groups(self, Filters: List[Filter]=None, DryRun: Boolean=None, GroupNames: List[String]=None) -> DescribePlacementGroupsResult:
        """Describes the specified placement groups or all of your placement groups. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/placement-groups.html">Placement Groups</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param GroupNames:  The names of the placement groups. Default: Describes all your placement groups, or only those otherwise specified.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Filters:  The filters. <ul> <li>  <code>group-name</code> - The name of the placement group. </li> <li>  <code>state</code> - The state of the placement group (<code>pending</code> | <code>available</code> | <code>deleting</code> | <code>deleted</code>). </li> <li>  <code>strategy</code> - The strategy of the placement group (<code>cluster</code> | <code>spread</code> | <code>partition</code>). </li> </ul><br/>
        :return: PlacementGroups:  Information about the placement groups.<br/>        """
        pass

    def describe_prefix_lists(self, DryRun: Boolean=None, Filters: List[Filter]=None, MaxResults: int=None, NextToken: str=None, PrefixListIds: List[String]=None) -> DescribePrefixListsResult:
        """Describes available AWS services in a prefix list format, which includes the prefix list name and prefix list ID of the service and the IP address range for the service. A prefix list ID is required for creating an outbound security group rule that allows traffic from a VPC to access an AWS service through a gateway VPC endpoint. Currently, the services that support this action are Amazon S3 and Amazon DynamoDB.

        :param self:
        :param PrefixListIds:  One or more prefix list IDs.<br/>NextToken:  The token for the next page of results.<br/>MaxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>Filters:  One or more filters. <ul> <li>  <code>prefix-list-id</code>: The ID of a prefix list. </li> <li>  <code>prefix-list-name</code>: The name of a prefix list. </li> </ul><br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: PrefixLists:  All available prefix lists.<br/>NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>        """
        pass

    def describe_principal_id_format(self, DryRun: Boolean=None, Resources: List[String]=None, MaxResults: int=None, NextToken: str=None) -> DescribePrincipalIdFormatResult:
        """Describes the ID format settings for the root user and all IAM roles and IAM users that have explicitly specified a longer ID (17-character ID) preference.  By default, all IAM roles and IAM users default to the same ID settings as the root user, unless they explicitly override the settings. This request is useful for identifying those IAM users and IAM roles that have overridden the default ID settings. The following resource types support longer IDs: <code>bundle</code> | <code>conversion-task</code> | <code>customer-gateway</code> | <code>dhcp-options</code> | <code>elastic-ip-allocation</code> | <code>elastic-ip-association</code> | <code>export-task</code> | <code>flow-log</code> | <code>image</code> | <code>import-task</code> | <code>instance</code> | <code>internet-gateway</code> | <code>network-acl</code> | <code>network-acl-association</code> | <code>network-interface</code> | <code>network-interface-attachment</code> | <code>prefix-list</code> | <code>reservation</code> | <code>route-table</code> | <code>route-table-association</code> | <code>security-group</code> | <code>snapshot</code> | <code>subnet</code> | <code>subnet-cidr-block-association</code> | <code>volume</code> | <code>vpc</code> | <code>vpc-cidr-block-association</code> | <code>vpc-endpoint</code> | <code>vpc-peering-connection</code> | <code>vpn-connection</code> | <code>vpn-gateway</code>. 

        :param self:
        :param NextToken:  The token to request the next page of results.<br/>MaxResults:  The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned NextToken value. <br/>Resources:  The type of resource: <code>bundle</code> | <code>conversion-task</code> | <code>customer-gateway</code> | <code>dhcp-options</code> | <code>elastic-ip-allocation</code> | <code>elastic-ip-association</code> | <code>export-task</code> | <code>flow-log</code> | <code>image</code> | <code>import-task</code> | <code>instance</code> | <code>internet-gateway</code> | <code>network-acl</code> | <code>network-acl-association</code> | <code>network-interface</code> | <code>network-interface-attachment</code> | <code>prefix-list</code> | <code>reservation</code> | <code>route-table</code> | <code>route-table-association</code> | <code>security-group</code> | <code>snapshot</code> | <code>subnet</code> | <code>subnet-cidr-block-association</code> | <code>volume</code> | <code>vpc</code> | <code>vpc-cidr-block-association</code> | <code>vpc-endpoint</code> | <code>vpc-peering-connection</code> | <code>vpn-connection</code> | <code>vpn-gateway</code> <br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is null when there are no more results to return.<br/>Principals:  Information about the ID format settings for the ARN.<br/>        """
        pass

    def describe_public_ipv_4_pools(self, PoolIds: List[String]=None, NextToken: str=None, MaxResults: int=None) -> DescribePublicIpv4PoolsResult:
        """Describes the specified IPv4 address pools.

        :param self:
        :param MaxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>NextToken:  The token for the next page of results.<br/>PoolIds:  The IDs of the address pools.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>PublicIpv4Pools:  Information about the address pools.<br/>        """
        pass

    def describe_regions(self, Filters: List[Filter]=None, RegionNames: List[String]=None, DryRun: Boolean=None) -> DescribeRegionsResult:
        """Describes the Regions that are currently available to you. The API returns a list of all the Regions, including Regions that are disabled for your account. For information about enabling Regions for your account, see <a href="https://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/manage-account-payment.html#manage-account-payment-enable-disable-regions">Enabling and Disabling Regions</a> in the <i>AWS Billing and Cost Management User Guide</i>. For a list of the Regions supported by Amazon EC2, see <a href="https://docs.aws.amazon.com/general/latest/gr/rande.html#ec2_region"> Regions and Endpoints</a>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>RegionNames:  The names of the Regions.<br/>Filters:  The filters. <ul> <li>  <code>endpoint</code> - The endpoint of the Region (for example, <code>ec2.us-east-1.amazonaws.com</code>). </li> <li>  <code>region-name</code> - The name of the Region (for example, <code>us-east-1</code>). </li> </ul><br/>
        :return: Regions:  Information about the Regions.<br/>        """
        pass

    def describe_reserved_instances(self, Filters: List[Filter]=None, OfferingClass: str=None, ReservedInstancesIds: List[String]=None, DryRun: Boolean=None, OfferingType: str=None) -> DescribeReservedInstancesResult:
        """Describes one or more of the Reserved Instances that you purchased. For more information about Reserved Instances, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/concepts-on-demand-reserved-instances.html">Reserved Instances</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param OfferingType:  The Reserved Instance offering type. If you are using tools that predate the 2011-11-01 API version, you only have access to the <code>Medium Utilization</code> Reserved Instance offering type.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>ReservedInstancesIds:  One or more Reserved Instance IDs. Default: Describes all your Reserved Instances, or only those otherwise specified.<br/>OfferingClass:  Describes whether the Reserved Instance is Standard or Convertible.<br/>Filters:  One or more filters. <ul> <li>  <code>availability-zone</code> - The Availability Zone where the Reserved Instance can be used. </li> <li>  <code>duration</code> - The duration of the Reserved Instance (one year or three years), in seconds (<code>31536000</code> | <code>94608000</code>). </li> <li>  <code>end</code> - The time when the Reserved Instance expires (for example, 2015-08-07T11:54:42.000Z). </li> <li>  <code>fixed-price</code> - The purchase price of the Reserved Instance (for example, 9800.0). </li> <li>  <code>instance-type</code> - The instance type that is covered by the reservation. </li> <li>  <code>scope</code> - The scope of the Reserved Instance (<code>Region</code> or <code>Availability Zone</code>). </li> <li>  <code>product-description</code> - The Reserved Instance product platform description. Instances that include <code>(Amazon VPC)</code> in the product platform description will only be displayed to EC2-Classic account holders and are for use with Amazon VPC (<code>Linux/UNIX</code> | <code>Linux/UNIX (Amazon VPC)</code> | <code>SUSE Linux</code> | <code>SUSE Linux (Amazon VPC)</code> | <code>Red Hat Enterprise Linux</code> | <code>Red Hat Enterprise Linux (Amazon VPC)</code> | <code>Windows</code> | <code>Windows (Amazon VPC)</code> | <code>Windows with SQL Server Standard</code> | <code>Windows with SQL Server Standard (Amazon VPC)</code> | <code>Windows with SQL Server Web</code> | <code>Windows with SQL Server Web (Amazon VPC)</code> | <code>Windows with SQL Server Enterprise</code> | <code>Windows with SQL Server Enterprise (Amazon VPC)</code>). </li> <li>  <code>reserved-instances-id</code> - The ID of the Reserved Instance. </li> <li>  <code>start</code> - The time at which the Reserved Instance purchase request was placed (for example, 2014-08-07T11:54:42.000Z). </li> <li>  <code>state</code> - The state of the Reserved Instance (<code>payment-pending</code> | <code>active</code> | <code>payment-failed</code> | <code>retired</code>). </li> <li>  <code>tag</code>:&lt;key&gt; - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value. </li> <li>  <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value. </li> <li>  <code>usage-price</code> - The usage price of the Reserved Instance, per hour (for example, 0.84). </li> </ul><br/>
        :return: ReservedInstances:  A list of Reserved Instances.<br/>        """
        pass

    def describe_reserved_instances_listings(self, Filters: List[Filter]=None, ReservedInstancesId: str=None, ReservedInstancesListingId: str=None) -> DescribeReservedInstancesListingsResult:
        """Describes your account's Reserved Instance listings in the Reserved Instance Marketplace. The Reserved Instance Marketplace matches sellers who want to resell Reserved Instance capacity that they no longer need with buyers who want to purchase additional capacity. Reserved Instances bought and sold through the Reserved Instance Marketplace work like any other Reserved Instances. As a seller, you choose to list some or all of your Reserved Instances, and you specify the upfront price to receive for them. Your Reserved Instances are then listed in the Reserved Instance Marketplace and are available for purchase. As a buyer, you specify the configuration of the Reserved Instance to purchase, and the Marketplace matches what you're searching for with what's available. The Marketplace first sells the lowest priced Reserved Instances to you, and continues to sell available Reserved Instance listings to you until your demand is met. You are charged based on the total price of all of the listings that you purchase. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ri-market-general.html">Reserved Instance Marketplace</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param ReservedInstancesListingId:  One or more Reserved Instance listing IDs.<br/>ReservedInstancesId:  One or more Reserved Instance IDs.<br/>Filters:  One or more filters. <ul> <li>  <code>reserved-instances-id</code> - The ID of the Reserved Instances. </li> <li>  <code>reserved-instances-listing-id</code> - The ID of the Reserved Instances listing. </li> <li>  <code>status</code> - The status of the Reserved Instance listing (<code>pending</code> | <code>active</code> | <code>cancelled</code> | <code>closed</code>). </li> <li>  <code>status-message</code> - The reason for the status. </li> </ul><br/>
        :return: ReservedInstancesListings:  Information about the Reserved Instance listing.<br/>        """
        pass

    def describe_reserved_instances_modifications(self, Filters: List[Filter]=None, ReservedInstancesModificationIds: List[String]=None, NextToken: str=None) -> DescribeReservedInstancesModificationsResult:
        """Describes the modifications made to your Reserved Instances. If no parameter is specified, information about all your Reserved Instances modification requests is returned. If a modification ID is specified, only information about the specific modification is returned. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ri-modifying.html">Modifying Reserved Instances</a> in the Amazon Elastic Compute Cloud User Guide.

        :param self:
        :param NextToken:  The token to retrieve the next page of results.<br/>ReservedInstancesModificationIds:  IDs for the submitted modification request.<br/>Filters:  One or more filters. <ul> <li>  <code>client-token</code> - The idempotency token for the modification request. </li> <li>  <code>create-date</code> - The time when the modification request was created. </li> <li>  <code>effective-date</code> - The time when the modification becomes effective. </li> <li>  <code>modification-result.reserved-instances-id</code> - The ID for the Reserved Instances created as part of the modification request. This ID is only available when the status of the modification is <code>fulfilled</code>. </li> <li>  <code>modification-result.target-configuration.availability-zone</code> - The Availability Zone for the new Reserved Instances. </li> <li>  <code>modification-result.target-configuration.instance-count </code> - The number of new Reserved Instances. </li> <li>  <code>modification-result.target-configuration.instance-type</code> - The instance type of the new Reserved Instances. </li> <li>  <code>modification-result.target-configuration.platform</code> - The network platform of the new Reserved Instances (<code>EC2-Classic</code> | <code>EC2-VPC</code>). </li> <li>  <code>reserved-instances-id</code> - The ID of the Reserved Instances modified. </li> <li>  <code>reserved-instances-modification-id</code> - The ID of the modification request. </li> <li>  <code>status</code> - The status of the Reserved Instances modification request (<code>processing</code> | <code>fulfilled</code> | <code>failed</code>). </li> <li>  <code>status-message</code> - The reason for the status. </li> <li>  <code>update-date</code> - The time when the modification request was last updated. </li> </ul><br/>
        :return: ReservedInstancesModifications:  The Reserved Instance modification information.<br/>NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>        """
        pass

    def describe_reserved_instances_offerings(self, AvailabilityZone: str=None, Filters: List[Filter]=None, IncludeMarketplace: Boolean=None, InstanceType: str=None, MaxDuration: Long=None, MaxInstanceCount: int=None, MinDuration: Long=None, OfferingClass: str=None, ProductDescription: str=None, ReservedInstancesOfferingIds: List[String]=None, DryRun: Boolean=None, InstanceTenancy: str=None, MaxResults: int=None, NextToken: str=None, OfferingType: str=None) -> DescribeReservedInstancesOfferingsResult:
        """Describes Reserved Instance offerings that are available for purchase. With Reserved Instances, you purchase the right to launch instances for a period of time. During that time period, you do not receive insufficient capacity errors, and you pay a lower usage rate than the rate charged for On-Demand instances for the actual time used. If you have listed your own Reserved Instances for sale in the Reserved Instance Marketplace, they will be excluded from these results. This is to ensure that you do not purchase your own Reserved Instances. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ri-market-general.html">Reserved Instance Marketplace</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param OfferingType:  The Reserved Instance offering type. If you are using tools that predate the 2011-11-01 API version, you only have access to the <code>Medium Utilization</code> Reserved Instance offering type. <br/>NextToken:  The token to retrieve the next page of results.<br/>MaxResults:  The maximum number of results to return for the request in a single page. The remaining results of the initial request can be seen by sending another request with the returned <code>NextToken</code> value. The maximum is 100. Default: 100<br/>InstanceTenancy:  The tenancy of the instances covered by the reservation. A Reserved Instance with a tenancy of <code>dedicated</code> is applied to instances that run in a VPC on single-tenant hardware (i.e., Dedicated Instances).  <b>Important:</b> The <code>host</code> value cannot be used with this parameter. Use the <code>default</code> or <code>dedicated</code> values only. Default: <code>default</code> <br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>ReservedInstancesOfferingIds:  One or more Reserved Instances offering IDs.<br/>ProductDescription:  The Reserved Instance product platform description. Instances that include <code>(Amazon VPC)</code> in the description are for use with Amazon VPC.<br/>OfferingClass:  The offering class of the Reserved Instance. Can be <code>standard</code> or <code>convertible</code>.<br/>MinDuration:  The minimum duration (in seconds) to filter when searching for offerings. Default: 2592000 (1 month)<br/>MaxInstanceCount:  The maximum number of instances to filter when searching for offerings. Default: 20<br/>MaxDuration:  The maximum duration (in seconds) to filter when searching for offerings. Default: 94608000 (3 years)<br/>InstanceType:  The instance type that the reservation will cover (for example, <code>m1.small</code>). For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html">Instance Types</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.<br/>IncludeMarketplace:  Include Reserved Instance Marketplace offerings in the response.<br/>Filters:  One or more filters. <ul> <li>  <code>availability-zone</code> - The Availability Zone where the Reserved Instance can be used. </li> <li>  <code>duration</code> - The duration of the Reserved Instance (for example, one year or three years), in seconds (<code>31536000</code> | <code>94608000</code>). </li> <li>  <code>fixed-price</code> - The purchase price of the Reserved Instance (for example, 9800.0). </li> <li>  <code>instance-type</code> - The instance type that is covered by the reservation. </li> <li>  <code>marketplace</code> - Set to <code>true</code> to show only Reserved Instance Marketplace offerings. When this filter is not used, which is the default behavior, all offerings from both AWS and the Reserved Instance Marketplace are listed. </li> <li>  <code>product-description</code> - The Reserved Instance product platform description. Instances that include <code>(Amazon VPC)</code> in the product platform description will only be displayed to EC2-Classic account holders and are for use with Amazon VPC. (<code>Linux/UNIX</code> | <code>Linux/UNIX (Amazon VPC)</code> | <code>SUSE Linux</code> | <code>SUSE Linux (Amazon VPC)</code> | <code>Red Hat Enterprise Linux</code> | <code>Red Hat Enterprise Linux (Amazon VPC)</code> | <code>Windows</code> | <code>Windows (Amazon VPC)</code> | <code>Windows with SQL Server Standard</code> | <code>Windows with SQL Server Standard (Amazon VPC)</code> | <code>Windows with SQL Server Web</code> | <code> Windows with SQL Server Web (Amazon VPC)</code> | <code>Windows with SQL Server Enterprise</code> | <code>Windows with SQL Server Enterprise (Amazon VPC)</code>)  </li> <li>  <code>reserved-instances-offering-id</code> - The Reserved Instances offering ID. </li> <li>  <code>scope</code> - The scope of the Reserved Instance (<code>Availability Zone</code> or <code>Region</code>). </li> <li>  <code>usage-price</code> - The usage price of the Reserved Instance, per hour (for example, 0.84). </li> </ul><br/>AvailabilityZone:  The Availability Zone in which the Reserved Instance can be used.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>ReservedInstancesOfferings:  A list of Reserved Instances offerings.<br/>        """
        pass

    def describe_route_tables(self, Filters: List[Filter]=None, DryRun: Boolean=None, RouteTableIds: List[String]=None, NextToken: str=None, MaxResults: int=None) -> DescribeRouteTablesResult:
        """Describes one or more of your route tables. Each subnet in your VPC must be associated with a route table. If a subnet is not explicitly associated with any route table, it is implicitly associated with the main route table. This command does not return the subnet ID for implicit associations. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Route_Tables.html">Route Tables</a> in the <i>Amazon Virtual Private Cloud User Guide</i>.

        :param self:
        :param MaxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>NextToken:  The token for the next page of results.<br/>RouteTableIds:  One or more route table IDs. Default: Describes all your route tables.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Filters:  One or more filters. <ul> <li>  <code>association.route-table-association-id</code> - The ID of an association ID for the route table. </li> <li>  <code>association.route-table-id</code> - The ID of the route table involved in the association. </li> <li>  <code>association.subnet-id</code> - The ID of the subnet involved in the association. </li> <li>  <code>association.main</code> - Indicates whether the route table is the main route table for the VPC (<code>true</code> | <code>false</code>). Route tables that do not have an association ID are not returned in the response. </li> <li>  <code>owner-id</code> - The ID of the AWS account that owns the route table. </li> <li>  <code>route-table-id</code> - The ID of the route table. </li> <li>  <code>route.destination-cidr-block</code> - The IPv4 CIDR range specified in a route in the table. </li> <li>  <code>route.destination-ipv6-cidr-block</code> - The IPv6 CIDR range specified in a route in the route table. </li> <li>  <code>route.destination-prefix-list-id</code> - The ID (prefix) of the AWS service specified in a route in the table. </li> <li>  <code>route.egress-only-internet-gateway-id</code> - The ID of an egress-only Internet gateway specified in a route in the route table. </li> <li>  <code>route.gateway-id</code> - The ID of a gateway specified in a route in the table. </li> <li>  <code>route.instance-id</code> - The ID of an instance specified in a route in the table. </li> <li>  <code>route.nat-gateway-id</code> - The ID of a NAT gateway. </li> <li>  <code>route.transit-gateway-id</code> - The ID of a transit gateway. </li> <li>  <code>route.origin</code> - Describes how the route was created. <code>CreateRouteTable</code> indicates that the route was automatically created when the route table was created; <code>CreateRoute</code> indicates that the route was manually added to the route table; <code>EnableVgwRoutePropagation</code> indicates that the route was propagated by route propagation. </li> <li>  <code>route.state</code> - The state of a route in the route table (<code>active</code> | <code>blackhole</code>). The blackhole state indicates that the route's target isn't available (for example, the specified gateway isn't attached to the VPC, the specified NAT instance has been terminated, and so on). </li> <li>  <code>route.vpc-peering-connection-id</code> - The ID of a VPC peering connection specified in a route in the table. </li> <li>  <code>tag</code>:&lt;key&gt; - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value. </li> <li>  <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value. </li> <li>  <code>transit-gateway-id</code> - The ID of a transit gateway. </li> <li>  <code>vpc-id</code> - The ID of the VPC for the route table. </li> </ul><br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>RouteTables:  Information about one or more route tables.<br/>        """
        pass

    def describe_scheduled_instance_availability(self, Recurrence: ScheduledInstanceRecurrenceRequest, FirstSlotStartTimeRange: SlotDateTimeRangeRequest, DryRun: Boolean=None, Filters: List[Filter]=None, MaxResults: int=None, MaxSlotDurationInHours: int=None, MinSlotDurationInHours: int=None, NextToken: str=None) -> DescribeScheduledInstanceAvailabilityResult:
        """Finds available schedules that meet the specified criteria. You can search for an available schedule no more than 3 months in advance. You must meet the minimum required duration of 1,200 hours per year. For example, the minimum daily schedule is 4 hours, the minimum weekly schedule is 24 hours, and the minimum monthly schedule is 100 hours. After you find a schedule that meets your needs, call <a>PurchaseScheduledInstances</a> to purchase Scheduled Instances with that schedule.

        :param self:
        :param Recurrence:  The schedule recurrence.:<br/>NextToken:  The token for the next set of results.<br/>MinSlotDurationInHours:  The minimum available duration, in hours. The minimum required duration is 1,200 hours per year. For example, the minimum daily schedule is 4 hours, the minimum weekly schedule is 24 hours, and the minimum monthly schedule is 100 hours.<br/>MaxSlotDurationInHours:  The maximum available duration, in hours. This value must be greater than <code>MinSlotDurationInHours</code> and less than 1,720.<br/>MaxResults:  The maximum number of results to return in a single call. This value can be between 5 and 300. The default value is 300. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value.<br/>FirstSlotStartTimeRange:  The time period for the first schedule to start.:<br/>Filters:  The filters. <ul> <li>  <code>availability-zone</code> - The Availability Zone (for example, <code>us-west-2a</code>). </li> <li>  <code>instance-type</code> - The instance type (for example, <code>c4.large</code>). </li> <li>  <code>network-platform</code> - The network platform (<code>EC2-Classic</code> or <code>EC2-VPC</code>). </li> <li>  <code>platform</code> - The platform (<code>Linux/UNIX</code> or <code>Windows</code>). </li> </ul><br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestTime</i> The latest date and time, in UTC, for the Scheduled Instance to start. This value must be later than or equal to the earliest date and at most three months in the future.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestTime</i> The earliest date and time, in UTC, for the Scheduled Instance to start.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OccurrenceUnit</i> The unit for <code>OccurrenceDays</code> (<code>DayOfWeek</code> or <code>DayOfMonth</code>). This value is required for a monthly schedule. You can't specify <code>DayOfWeek</code> with a weekly schedule. You can't specify this value with a daily schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OccurrenceRelativeToEnd</i> Indicates whether the occurrence is relative to the end of the specified week or month. You can't specify this value with a daily schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OccurrenceDays</i> The days. For a monthly schedule, this is one or more days of the month (1-31). For a weekly schedule, this is one or more days of the week (1-7, where 1 is Sunday). You can't specify this value with a daily schedule. If the occurrence is relative to the end of the month, you can specify only a single day.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Interval</i> The interval quantity. The interval unit depends on the value of <code>Frequency</code>. For example, every 2 weeks or every 2 months.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Frequency</i> The frequency (<code>Daily</code>, <code>Weekly</code>, or <code>Monthly</code>).<br/>
        :return: ScheduledInstanceAvailabilitySet:  Information about the available Scheduled Instances.<br/>NextToken:  The token required to retrieve the next set of results. This value is <code>null</code> when there are no more results to return.<br/>        """
        pass

    def describe_scheduled_instances(self, DryRun: Boolean=None, Filters: List[Filter]=None, MaxResults: int=None, NextToken: str=None, ScheduledInstanceIds: List[String]=None, SlotStartTimeRange: SlotStartTimeRangeRequest=None) -> DescribeScheduledInstancesResult:
        """Describes the specified Scheduled Instances or all your Scheduled Instances.

        :param self:
        :param SlotStartTimeRange:  The time period for the first schedule to start.:<br/>ScheduledInstanceIds:  The Scheduled Instance IDs.<br/>NextToken:  The token for the next set of results.<br/>MaxResults:  The maximum number of results to return in a single call. This value can be between 5 and 300. The default value is 100. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value.<br/>Filters:  The filters. <ul> <li>  <code>availability-zone</code> - The Availability Zone (for example, <code>us-west-2a</code>). </li> <li>  <code>instance-type</code> - The instance type (for example, <code>c4.large</code>). </li> <li>  <code>network-platform</code> - The network platform (<code>EC2-Classic</code> or <code>EC2-VPC</code>). </li> <li>  <code>platform</code> - The platform (<code>Linux/UNIX</code> or <code>Windows</code>). </li> </ul><br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestTime</i> The latest date and time, in UTC, for the Scheduled Instance to start.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestTime</i> The earliest date and time, in UTC, for the Scheduled Instance to start.<br/>
        :return: ScheduledInstanceSet:  Information about the Scheduled Instances.<br/>NextToken:  The token required to retrieve the next set of results. This value is <code>null</code> when there are no more results to return.<br/>        """
        pass

    def describe_security_group_references(self, GroupId: List[String], DryRun: Boolean=None) -> DescribeSecurityGroupReferencesResult:
        """[VPC only] Describes the VPCs on the other side of a VPC peering connection that are referencing the security groups you've specified in this request.

        :param self:
        :param GroupId:  The IDs of the security groups in your account.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: SecurityGroupReferenceSet:  Information about the VPCs with the referencing security groups.<br/>        """
        pass

    def describe_security_groups(self, Filters: List[Filter]=None, GroupIds: List[String]=None, GroupNames: List[String]=None, DryRun: Boolean=None, NextToken: str=None, MaxResults: int=None) -> DescribeSecurityGroupsResult:
        """Describes the specified security groups or all of your security groups. A security group is for use with instances either in the EC2-Classic platform or in a specific VPC. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/using-network-security.html">Amazon EC2 Security Groups</a> in the <i>Amazon Elastic Compute Cloud User Guide</i> and <a href="https://docs.aws.amazon.com/AmazonVPC/latest/UserGuide/VPC_SecurityGroups.html">Security Groups for Your VPC</a> in the <i>Amazon Virtual Private Cloud User Guide</i>.

        :param self:
        :param MaxResults:  The maximum number of results to return in a single call. To retrieve the remaining results, make another request with the returned <code>NextToken</code> value. This value can be between 5 and 1000. If this parameter is not specified, then all results are returned.<br/>NextToken:  The token to request the next page of results.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>GroupNames:  [EC2-Classic and default VPC only] The names of the security groups. You can specify either the security group name or the security group ID. For security groups in a nondefault VPC, use the <code>group-name</code> filter to describe security groups by name. Default: Describes all your security groups.<br/>GroupIds:  The IDs of the security groups. Required for security groups in a nondefault VPC. Default: Describes all your security groups.<br/>Filters:  The filters. If using multiple filters for rules, the results include security groups for which any combination of rules - not necessarily a single rule - match all filters. <ul> <li>  <code>description</code> - The description of the security group. </li> <li>  <code>egress.ip-permission.cidr</code> - An IPv4 CIDR block for an outbound security group rule. </li> <li>  <code>egress.ip-permission.from-port</code> - For an outbound rule, the start of port range for the TCP and UDP protocols, or an ICMP type number. </li> <li>  <code>egress.ip-permission.group-id</code> - The ID of a security group that has been referenced in an outbound security group rule. </li> <li>  <code>egress.ip-permission.group-name</code> - The name of a security group that has been referenced in an outbound security group rule. </li> <li>  <code>egress.ip-permission.ipv6-cidr</code> - An IPv6 CIDR block for an outbound security group rule. </li> <li>  <code>egress.ip-permission.prefix-list-id</code> - The ID (prefix) of the AWS service to which a security group rule allows outbound access. </li> <li>  <code>egress.ip-permission.protocol</code> - The IP protocol for an outbound security group rule (<code>tcp</code> | <code>udp</code> | <code>icmp</code> or a protocol number). </li> <li>  <code>egress.ip-permission.to-port</code> - For an outbound rule, the end of port range for the TCP and UDP protocols, or an ICMP code. </li> <li>  <code>egress.ip-permission.user-id</code> - The ID of an AWS account that has been referenced in an outbound security group rule. </li> <li>  <code>group-id</code> - The ID of the security group.  </li> <li>  <code>group-name</code> - The name of the security group. </li> <li>  <code>ip-permission.cidr</code> - An IPv4 CIDR block for an inbound security group rule. </li> <li>  <code>ip-permission.from-port</code> - For an inbound rule, the start of port range for the TCP and UDP protocols, or an ICMP type number. </li> <li>  <code>ip-permission.group-id</code> - The ID of a security group that has been referenced in an inbound security group rule. </li> <li>  <code>ip-permission.group-name</code> - The name of a security group that has been referenced in an inbound security group rule. </li> <li>  <code>ip-permission.ipv6-cidr</code> - An IPv6 CIDR block for an inbound security group rule. </li> <li>  <code>ip-permission.prefix-list-id</code> - The ID (prefix) of the AWS service from which a security group rule allows inbound access. </li> <li>  <code>ip-permission.protocol</code> - The IP protocol for an inbound security group rule (<code>tcp</code> | <code>udp</code> | <code>icmp</code> or a protocol number). </li> <li>  <code>ip-permission.to-port</code> - For an inbound rule, the end of port range for the TCP and UDP protocols, or an ICMP code. </li> <li>  <code>ip-permission.user-id</code> - The ID of an AWS account that has been referenced in an inbound security group rule. </li> <li>  <code>owner-id</code> - The AWS account ID of the owner of the security group. </li> <li>  <code>tag</code>:&lt;key&gt; - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value. </li> <li>  <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value. </li> <li>  <code>vpc-id</code> - The ID of the VPC specified when the security group was created. </li> </ul><br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>SecurityGroups:  Information about the security groups.<br/>        """
        pass

    def describe_snapshot_attribute(self, SnapshotId: str, Attribute: str, DryRun: Boolean=None) -> DescribeSnapshotAttributeResult:
        """Describes the specified attribute of the specified snapshot. You can specify only one attribute at a time. For more information about EBS snapshots, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSSnapshots.html">Amazon EBS Snapshots</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>SnapshotId:  The ID of the EBS snapshot.<br/>Attribute:  The snapshot attribute you would like to view.<br/>
        :return: SnapshotId:  The ID of the EBS snapshot.<br/>ProductCodes:  The product codes.<br/>CreateVolumePermissions:  The users and groups that have the permissions for creating volumes from the snapshot.<br/>        """
        pass

    def describe_snapshots(self, Filters: List[Filter]=None, MaxResults: int=None, NextToken: str=None, OwnerIds: List[String]=None, RestorableByUserIds: List[String]=None, SnapshotIds: List[String]=None, DryRun: Boolean=None) -> DescribeSnapshotsResult:
        """Describes the specified EBS snapshots available to you or all of the EBS snapshots available to you. The snapshots available to you include public snapshots, private snapshots that you own, and private snapshots owned by other AWS accounts for which you have explicit create volume permissions. The create volume permissions fall into the following categories: <ul> <li>  <i>public</i>: The owner of the snapshot granted create volume permissions for the snapshot to the <code>all</code> group. All AWS accounts have create volume permissions for these snapshots. </li> <li>  <i>explicit</i>: The owner of the snapshot granted create volume permissions to a specific AWS account. </li> <li>  <i>implicit</i>: An AWS account has implicit create volume permissions for all snapshots it owns. </li> </ul> The list of snapshots returned can be modified by specifying snapshot IDs, snapshot owners, or AWS accounts with create volume permissions. If no options are specified, Amazon EC2 returns all snapshots for which you have create volume permissions. If you specify one or more snapshot IDs, only snapshots that have the specified IDs are returned. If you specify an invalid snapshot ID, an error is returned. If you specify a snapshot ID for which you do not have access, it is not included in the returned results. If you specify one or more snapshot owners using the <code>OwnerIds</code> option, only snapshots from the specified owners and for which you have access are returned. The results can include the AWS account IDs of the specified owners, <code>amazon</code> for snapshots owned by Amazon, or <code>self</code> for snapshots that you own. If you specify a list of restorable users, only snapshots with create snapshot permissions for those users are returned. You can specify AWS account IDs (if you own the snapshots), <code>self</code> for snapshots for which you own or have explicit permissions, or <code>all</code> for public snapshots. If you are describing a long list of snapshots, you can paginate the output to make the list more manageable. The <code>MaxResults</code> parameter sets the maximum number of results returned in a single page. If the list of results exceeds your <code>MaxResults</code> value, then that number of results is returned along with a <code>NextToken</code> value that can be passed to a subsequent <code>DescribeSnapshots</code> request to retrieve the remaining results. For more information about EBS snapshots, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSSnapshots.html">Amazon EBS Snapshots</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>SnapshotIds:  The snapshot IDs. Default: Describes the snapshots for which you have create volume permissions.<br/>RestorableByUserIds:  The IDs of the AWS accounts that can create volumes from the snapshot.<br/>OwnerIds:  Describes the snapshots owned by these owners.<br/>NextToken:  The <code>NextToken</code> value returned from a previous paginated <code>DescribeSnapshots</code> request where <code>MaxResults</code> was used and the results exceeded the value of that parameter. Pagination continues from the end of the previous results that returned the <code>NextToken</code> value. This value is <code>null</code> when there are no more results to return.<br/>MaxResults:  The maximum number of snapshot results returned by <code>DescribeSnapshots</code> in paginated output. When this parameter is used, <code>DescribeSnapshots</code> only returns <code>MaxResults</code> results in a single page along with a <code>NextToken</code> response element. The remaining results of the initial request can be seen by sending another <code>DescribeSnapshots</code> request with the returned <code>NextToken</code> value. This value can be between 5 and 1000; if <code>MaxResults</code> is given a value larger than 1000, only 1000 results are returned. If this parameter is not used, then <code>DescribeSnapshots</code> returns all results. You cannot specify this parameter and the snapshot IDs parameter in the same request.<br/>Filters:  The filters. <ul> <li>  <code>description</code> - A description of the snapshot. </li> <li>  <code>encrypted</code> - Indicates whether the snapshot is encrypted (<code>true</code> | <code>false</code>) </li> <li>  <code>owner-alias</code> - Value from an Amazon-maintained list (<code>amazon</code> | <code>self</code> | <code>all</code> | <code>aws-marketplace</code> | <code>microsoft</code>) of snapshot owners. Not to be confused with the user-configured AWS account alias, which is set from the IAM console. </li> <li>  <code>owner-id</code> - The ID of the AWS account that owns the snapshot. </li> <li>  <code>progress</code> - The progress of the snapshot, as a percentage (for example, 80%). </li> <li>  <code>snapshot-id</code> - The snapshot ID. </li> <li>  <code>start-time</code> - The time stamp when the snapshot was initiated. </li> <li>  <code>status</code> - The status of the snapshot (<code>pending</code> | <code>completed</code> | <code>error</code>). </li> <li>  <code>tag</code>:&lt;key&gt; - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value. </li> <li>  <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value. </li> <li>  <code>volume-id</code> - The ID of the volume the snapshot is for. </li> <li>  <code>volume-size</code> - The size of the volume, in GiB. </li> </ul><br/>
        :return: NextToken:  The <code>NextToken</code> value to include in a future <code>DescribeSnapshots</code> request. When the results of a <code>DescribeSnapshots</code> request exceed <code>MaxResults</code>, this value can be used to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>Snapshots:  Information about the snapshots.<br/>        """
        pass

    def describe_spot_datafeed_subscription(self, DryRun: Boolean=None) -> DescribeSpotDatafeedSubscriptionResult:
        """Describes the data feed for Spot Instances. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/spot-data-feeds.html">Spot Instance Data Feed</a> in the <i>Amazon EC2 User Guide for Linux Instances</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: SpotDatafeedSubscription:  The Spot Instance data feed subscription.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the Spot Instance data feed subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> The prefix that is prepended to data feed files.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The AWS account ID of the account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Fault</i> The fault codes for the Spot Instance request, if any.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Fault</b>: The fault codes for the Spot Instance request, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> The Amazon S3 bucket where the Spot Instance data feed is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> The message for the Spot Instance state change.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Code</i> The reason code for the Spot Instance state change.<br/>        """
        pass

    def describe_spot_fleet_instances(self, SpotFleetRequestId: str, DryRun: Boolean=None, MaxResults: int=None, NextToken: str=None) -> DescribeSpotFleetInstancesResponse:
        """Describes the running instances for the specified Spot Fleet.

        :param self:
        :param SpotFleetRequestId:  The ID of the Spot Fleet request.<br/>NextToken:  The token for the next set of results.<br/>MaxResults:  The maximum number of results to return in a single call. Specify a value between 1 and 1000. The default value is 1000. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: SpotFleetRequestId:  The ID of the Spot Fleet request.<br/>NextToken:  The token required to retrieve the next set of results. This value is <code>null</code> when there are no more results to return.<br/>ActiveInstances:  The running instances. This list is refreshed periodically and might be out of date.<br/>        """
        pass

    def describe_spot_fleet_request_history(self, StartTime: DateTime, SpotFleetRequestId: str, DryRun: Boolean=None, EventType: str=None, MaxResults: int=None, NextToken: str=None) -> DescribeSpotFleetRequestHistoryResponse:
        """Describes the events for the specified Spot Fleet request during the specified time. Spot Fleet events are delayed by up to 30 seconds before they can be described. This ensures that you can query by the last evaluated time and not miss a recorded event. Spot Fleet events are available for 48 hours.

        :param self:
        :param StartTime:  The starting date and time for the events, in UTC format (for example, <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).<br/>SpotFleetRequestId:  The ID of the Spot Fleet request.<br/>NextToken:  The token for the next set of results.<br/>MaxResults:  The maximum number of results to return in a single call. Specify a value between 1 and 1000. The default value is 1000. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value.<br/>EventType:  The type of events to describe. By default, all events are described.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: StartTime:  The starting date and time for the events, in UTC format (for example, <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).<br/>SpotFleetRequestId:  The ID of the Spot Fleet request.<br/>NextToken:  The token required to retrieve the next set of results. This value is <code>null</code> when there are no more results to return.<br/>LastEvaluatedTime:  The last date and time for the events, in UTC format (for example, <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z). All records up to this time were retrieved. If <code>nextToken</code> indicates that there are more results, this value is not present.<br/>HistoryRecords:  Information about the events in the history of the Spot Fleet request.<br/>        """
        pass

    def describe_spot_fleet_requests(self, DryRun: Boolean=None, MaxResults: int=None, NextToken: str=None, SpotFleetRequestIds: List[String]=None) -> DescribeSpotFleetRequestsResponse:
        """Describes your Spot Fleet requests. Spot Fleet requests are deleted 48 hours after they are canceled and their instances are terminated.

        :param self:
        :param SpotFleetRequestIds:  The IDs of the Spot Fleet requests.<br/>NextToken:  The token for the next set of results.<br/>MaxResults:  The maximum number of results to return in a single call. Specify a value between 1 and 1000. The default value is 1000. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: SpotFleetRequestConfigs:  Information about the configuration of your Spot Fleet.<br/>NextToken:  The token required to retrieve the next set of results. This value is <code>null</code> when there are no more results to return.<br/>        """
        pass

    def describe_spot_instance_requests(self, Filters: List[Filter]=None, DryRun: Boolean=None, SpotInstanceRequestIds: List[String]=None, NextToken: str=None, MaxResults: int=None) -> DescribeSpotInstanceRequestsResult:
        """Describes the specified Spot Instance requests. You can use <code>DescribeSpotInstanceRequests</code> to find a running Spot Instance by examining the response. If the status of the Spot Instance is <code>fulfilled</code>, the instance ID appears in the response and contains the identifier of the instance. Alternatively, you can use <a>DescribeInstances</a> with a filter to look for instances where the instance lifecycle is <code>spot</code>. We recommend that you set <code>MaxResults</code> to a value between 5 and 1000 to limit the number of results returned. This paginates the output, which makes the list more manageable and returns the results faster. If the list of results exceeds your <code>MaxResults</code> value, then that number of results is returned along with a <code>NextToken</code> value that can be passed to a subsequent <code>DescribeSpotInstanceRequests</code> request to retrieve the remaining results. Spot Instance requests are deleted four hours after they are canceled and their instances are terminated.

        :param self:
        :param MaxResults:  The maximum number of results to return in a single call. Specify a value between 5 and 1000. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value.<br/>NextToken:  The token to request the next set of results. This value is <code>null</code> when there are no more results to return.<br/>SpotInstanceRequestIds:  One or more Spot Instance request IDs.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Filters:  One or more filters. <ul> <li>  <code>availability-zone-group</code> - The Availability Zone group. </li> <li>  <code>create-time</code> - The time stamp when the Spot Instance request was created. </li> <li>  <code>fault-code</code> - The fault code related to the request. </li> <li>  <code>fault-message</code> - The fault message related to the request. </li> <li>  <code>instance-id</code> - The ID of the instance that fulfilled the request. </li> <li>  <code>launch-group</code> - The Spot Instance launch group. </li> <li>  <code>launch.block-device-mapping.delete-on-termination</code> - Indicates whether the EBS volume is deleted on instance termination. </li> <li>  <code>launch.block-device-mapping.device-name</code> - The device name for the volume in the block device mapping (for example, <code>/dev/sdh</code> or <code>xvdh</code>). </li> <li>  <code>launch.block-device-mapping.snapshot-id</code> - The ID of the snapshot for the EBS volume. </li> <li>  <code>launch.block-device-mapping.volume-size</code> - The size of the EBS volume, in GiB. </li> <li>  <code>launch.block-device-mapping.volume-type</code> - The type of EBS volume: <code>gp2</code> for General Purpose SSD, <code>io1</code> for Provisioned IOPS SSD, <code>st1</code> for Throughput Optimized HDD, <code>sc1</code>for Cold HDD, or <code>standard</code> for Magnetic. </li> <li>  <code>launch.group-id</code> - The ID of the security group for the instance. </li> <li>  <code>launch.group-name</code> - The name of the security group for the instance. </li> <li>  <code>launch.image-id</code> - The ID of the AMI. </li> <li>  <code>launch.instance-type</code> - The type of instance (for example, <code>m3.medium</code>). </li> <li>  <code>launch.kernel-id</code> - The kernel ID. </li> <li>  <code>launch.key-name</code> - The name of the key pair the instance launched with. </li> <li>  <code>launch.monitoring-enabled</code> - Whether detailed monitoring is enabled for the Spot Instance. </li> <li>  <code>launch.ramdisk-id</code> - The RAM disk ID. </li> <li>  <code>launched-availability-zone</code> - The Availability Zone in which the request is launched. </li> <li>  <code>network-interface.addresses.primary</code> - Indicates whether the IP address is the primary private IP address. </li> <li>  <code>network-interface.delete-on-termination</code> - Indicates whether the network interface is deleted when the instance is terminated. </li> <li>  <code>network-interface.description</code> - A description of the network interface. </li> <li>  <code>network-interface.device-index</code> - The index of the device for the network interface attachment on the instance. </li> <li>  <code>network-interface.group-id</code> - The ID of the security group associated with the network interface. </li> <li>  <code>network-interface.network-interface-id</code> - The ID of the network interface. </li> <li>  <code>network-interface.private-ip-address</code> - The primary private IP address of the network interface. </li> <li>  <code>network-interface.subnet-id</code> - The ID of the subnet for the instance. </li> <li>  <code>product-description</code> - The product description associated with the instance (<code>Linux/UNIX</code> | <code>Windows</code>). </li> <li>  <code>spot-instance-request-id</code> - The Spot Instance request ID. </li> <li>  <code>spot-price</code> - The maximum hourly price for any Spot Instance launched to fulfill the request. </li> <li>  <code>state</code> - The state of the Spot Instance request (<code>open</code> | <code>active</code> | <code>closed</code> | <code>cancelled</code> | <code>failed</code>). Spot request status information can help you track your Amazon EC2 Spot Instance requests. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/spot-bid-status.html">Spot Request Status</a> in the <i>Amazon EC2 User Guide for Linux Instances</i>. </li> <li>  <code>status-code</code> - The short code describing the most recent evaluation of your Spot Instance request. </li> <li>  <code>status-message</code> - The message explaining the status of the Spot Instance request. </li> <li>  <code>tag</code>:&lt;key&gt; - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value. </li> <li>  <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value. </li> <li>  <code>type</code> - The type of Spot Instance request (<code>one-time</code> | <code>persistent</code>). </li> <li>  <code>valid-from</code> - The start date of the request. </li> <li>  <code>valid-until</code> - The end date of the request. </li> </ul><br/>
        :return: NextToken:  The token to use to retrieve the next set of results. This value is <code>null</code> when there are no more results to return.<br/>SpotInstanceRequests:  One or more Spot Instance requests.<br/>        """
        pass

    def describe_spot_price_history(self, Filters: List[Filter]=None, AvailabilityZone: str=None, DryRun: Boolean=None, EndTime: DateTime=None, InstanceTypes: List[InstanceType]=None, MaxResults: int=None, NextToken: str=None, ProductDescriptions: List[String]=None, StartTime: DateTime=None) -> DescribeSpotPriceHistoryResult:
        """Describes the Spot price history. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/using-spot-instances-history.html">Spot Instance Pricing History</a> in the <i>Amazon EC2 User Guide for Linux Instances</i>. When you specify a start and end time, this operation returns the prices of the instance types within the time range that you specified and the time when the price changed. The price is valid within the time period that you specified; the response merely indicates the last time that the price changed.

        :param self:
        :param StartTime:  The date and time, up to the past 90 days, from which to start retrieving the price history data, in UTC format (for example, <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).<br/>ProductDescriptions:  Filters the results by the specified basic product descriptions.<br/>NextToken:  The token for the next set of results.<br/>MaxResults:  The maximum number of results to return in a single call. Specify a value between 1 and 1000. The default value is 1000. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value.<br/>InstanceTypes:  Filters the results by the specified instance types.<br/>EndTime:  The date and time, up to the current date, from which to stop retrieving the price history data, in UTC format (for example, <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>AvailabilityZone:  Filters the results by the specified Availability Zone.<br/>Filters:  One or more filters. <ul> <li>  <code>availability-zone</code> - The Availability Zone for which prices should be returned. </li> <li>  <code>instance-type</code> - The type of instance (for example, <code>m3.medium</code>). </li> <li>  <code>product-description</code> - The product description for the Spot price (<code>Linux/UNIX</code> | <code>SUSE Linux</code> | <code>Windows</code> | <code>Linux/UNIX (Amazon VPC)</code> | <code>SUSE Linux (Amazon VPC)</code> | <code>Windows (Amazon VPC)</code>). </li> <li>  <code>spot-price</code> - The Spot price. The value must match exactly (or use wildcards; greater than or less than comparison is not supported). </li> <li>  <code>timestamp</code> - The time stamp of the Spot price history, in UTC format (for example, <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z). You can use wildcards (* and ?). Greater than or less than comparison is not supported. </li> </ul><br/>
        :return: SpotPriceHistory:  The historical Spot prices.<br/>NextToken:  The token required to retrieve the next set of results. This value is null or an empty string when there are no more results to return.<br/>        """
        pass

    def describe_stale_security_groups(self, VpcId: str, DryRun: Boolean=None, MaxResults: int=None, NextToken: str=None) -> DescribeStaleSecurityGroupsResult:
        """[VPC only] Describes the stale security group rules for security groups in a specified VPC. Rules are stale when they reference a deleted security group in a peer VPC, or a security group in a peer VPC for which the VPC peering connection has been deleted.

        :param self:
        :param VpcId:  The ID of the VPC.<br/>NextToken:  The token for the next set of items to return. (You received this token from a prior call.)<br/>MaxResults:  The maximum number of items to return for this request. The request returns a token that you can specify in a subsequent call to get the next set of results.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: StaleSecurityGroupSet:  Information about the stale security groups.<br/>NextToken:  The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.<br/>        """
        pass

    def describe_subnets(self, Filters: List[Filter]=None, SubnetIds: List[String]=None, DryRun: Boolean=None, NextToken: str=None, MaxResults: int=None) -> DescribeSubnetsResult:
        """Describes one or more of your subnets. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Subnets.html">Your VPC and Subnets</a> in the <i>Amazon Virtual Private Cloud User Guide</i>.

        :param self:
        :param MaxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>NextToken:  The token for the next page of results.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>SubnetIds:  One or more subnet IDs. Default: Describes all your subnets.<br/>Filters:  One or more filters. <ul> <li>  <code>availability-zone</code> - The Availability Zone for the subnet. You can also use <code>availabilityZone</code> as the filter name. </li> <li>  <code>availability-zone-id</code> - The ID of the Availability Zone for the subnet. You can also use <code>availabilityZoneId</code> as the filter name. </li> <li>  <code>available-ip-address-count</code> - The number of IPv4 addresses in the subnet that are available. </li> <li>  <code>cidr-block</code> - The IPv4 CIDR block of the subnet. The CIDR block you specify must exactly match the subnet's CIDR block for information to be returned for the subnet. You can also use <code>cidr</code> or <code>cidrBlock</code> as the filter names. </li> <li>  <code>default-for-az</code> - Indicates whether this is the default subnet for the Availability Zone. You can also use <code>defaultForAz</code> as the filter name. </li> <li>  <code>ipv6-cidr-block-association.ipv6-cidr-block</code> - An IPv6 CIDR block associated with the subnet. </li> <li>  <code>ipv6-cidr-block-association.association-id</code> - An association ID for an IPv6 CIDR block associated with the subnet. </li> <li>  <code>ipv6-cidr-block-association.state</code> - The state of an IPv6 CIDR block associated with the subnet. </li> <li>  <code>owner-id</code> - The ID of the AWS account that owns the subnet. </li> <li>  <code>state</code> - The state of the subnet (<code>pending</code> | <code>available</code>). </li> <li>  <code>subnet-arn</code> - The Amazon Resource Name (ARN) of the subnet. </li> <li>  <code>subnet-id</code> - The ID of the subnet. </li> <li>  <code>tag</code>:&lt;key&gt; - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value. </li> <li>  <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value. </li> <li>  <code>vpc-id</code> - The ID of the VPC for the subnet. </li> </ul><br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>Subnets:  Information about one or more subnets.<br/>        """
        pass

    def describe_tags(self, DryRun: Boolean=None, Filters: List[Filter]=None, MaxResults: int=None, NextToken: str=None) -> DescribeTagsResult:
        """Describes the specified tags for your EC2 resources. For more information about tags, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Using_Tags.html">Tagging Your Resources</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param NextToken:  The token to retrieve the next page of results.<br/>MaxResults:  The maximum number of results to return in a single call. This value can be between 5 and 1000. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value.<br/>Filters:  The filters. <ul> <li>  <code>key</code> - The tag key. </li> <li>  <code>resource-id</code> - The ID of the resource. </li> <li>  <code>resource-type</code> - The resource type (<code>customer-gateway</code> | <code>dedicated-host</code> | <code>dhcp-options</code> | <code>elastic-ip</code> | <code>fleet</code> | <code>fpga-image</code> | <code>image</code> | <code>instance</code> | <code>host-reservation</code> | <code>internet-gateway</code> | <code>launch-template</code> | <code>natgateway</code> | <code>network-acl</code> | <code>network-interface</code> | <code>reserved-instances</code> | <code>route-table</code> | <code>security-group</code> | <code>snapshot</code> | <code>spot-instances-request</code> | <code>subnet</code> | <code>volume</code> | <code>vpc</code> | <code>vpc-peering-connection</code> | <code>vpn-connection</code> | <code>vpn-gateway</code>). </li> <li>  <code>tag</code>:&lt;key&gt; - The key/value combination of the tag. For example, specify "tag:Owner" for the filter name and "TeamA" for the filter value to find resources with the tag "Owner=TeamA". </li> <li>  <code>value</code> - The tag value. </li> </ul><br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: Tags:  The tags.<br/>NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>        """
        pass

    def describe_traffic_mirror_filters(self, TrafficMirrorFilterIds: List[String]=None, DryRun: Boolean=None, Filters: List[Filter]=None, MaxResults: int=None, NextToken: str=None) -> DescribeTrafficMirrorFiltersResult:
        """Describes one or more Traffic Mirror filters.

        :param self:
        :param NextToken:  The token for the next page of results.<br/>MaxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>Filters:  One or more filters. The possible values are: <ul> <li>  <code>description</code>: The Traffic Mirror filter description. </li> <li>  <code>traffic-mirror-filter-id</code>: The ID of the Traffic Mirror filter. </li> </ul><br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>TrafficMirrorFilterIds:  The ID of the Traffic Mirror filter.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. The value is <code>null</code> when there are no more results to return.<br/>TrafficMirrorFilters:  Information about one or more Traffic Mirror filters.<br/>        """
        pass

    def describe_traffic_mirror_sessions(self, TrafficMirrorSessionIds: List[String]=None, DryRun: Boolean=None, Filters: List[Filter]=None, MaxResults: int=None, NextToken: str=None) -> DescribeTrafficMirrorSessionsResult:
        """Describes one or more Traffic Mirror sessions. By default, all Traffic Mirror sessions are described. Alternatively, you can filter the results.

        :param self:
        :param NextToken:  The token for the next page of results.<br/>MaxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>Filters:  One or more filters. The possible values are: <ul> <li>  <code>description</code>: The Traffic Mirror session description. </li> <li>  <code>network-interface-id</code>: The ID of the Traffic Mirror session network interface. </li> <li>  <code>owner-id</code>: The ID of the account that owns the Traffic Mirror session. </li> <li>  <code>packet-length</code>: The assigned number of packets to mirror.  </li> <li>  <code>session-number</code>: The assigned session number.  </li> <li>  <code>traffic-mirror-filter-id</code>: The ID of the Traffic Mirror filter. </li> <li>  <code>traffic-mirror-session-id</code>: The ID of the Traffic Mirror session. </li> <li>  <code>traffic-mirror-target-id</code>: The ID of the Traffic Mirror target. </li> <li>  <code>virtual-network-id</code>: The virtual network ID of the Traffic Mirror session. </li> </ul><br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>TrafficMirrorSessionIds:  The ID of the Traffic Mirror session.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. The value is <code>null</code> when there are no more results to return.<br/>TrafficMirrorSessions:  Describes one or more Traffic Mirror sessions. By default, all Traffic Mirror sessions are described. Alternatively, you can filter the results.<br/>        """
        pass

    def describe_traffic_mirror_targets(self, TrafficMirrorTargetIds: List[String]=None, DryRun: Boolean=None, Filters: List[Filter]=None, MaxResults: int=None, NextToken: str=None) -> DescribeTrafficMirrorTargetsResult:
        """Information about one or more Traffic Mirror targets.

        :param self:
        :param NextToken:  The token for the next page of results.<br/>MaxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>Filters:  One or more filters. The possible values are: <ul> <li>  <code>description</code>: The Traffic Mirror target description. </li> <li>  <code>network-interface-id</code>: The ID of the Traffic Mirror session network interface. </li> <li>  <code>network-load-balancer-arn</code>: The Amazon Resource Name (ARN) of the Network Load Balancer that is associated with the session. </li> <li>  <code>owner-id</code>: The ID of the account that owns the Traffic Mirror session. </li> <li>  <code>traffic-mirror-target-id</code>: The ID of the Traffic Mirror target. </li> </ul><br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>TrafficMirrorTargetIds:  The ID of the Traffic Mirror targets.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. The value is <code>null</code> when there are no more results to return.<br/>TrafficMirrorTargets:  Information about one or more Traffic Mirror targets.<br/>        """
        pass

    def describe_transit_gateway_attachments(self, TransitGatewayAttachmentIds: List[String]=None, Filters: List[Filter]=None, MaxResults: int=None, NextToken: str=None, DryRun: Boolean=None) -> DescribeTransitGatewayAttachmentsResult:
        """Describes one or more attachments between resources and transit gateways. By default, all attachments are described. Alternatively, you can filter the results by attachment ID, attachment state, resource ID, or resource owner.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>NextToken:  The token for the next page of results.<br/>MaxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>Filters:  One or more filters. The possible values are: <ul> <li>  <code>association.state</code> - The state of the association (<code>associating</code> | <code>associated</code> | <code>disassociating</code>). </li> <li>  <code>association.transit-gateway-route-table-id</code> - The ID of the route table for the transit gateway. </li> <li>  <code>resource-id</code> - The ID of the resource. </li> <li>  <code>resource-owner-id</code> - The ID of the AWS account that owns the resource. </li> <li>  <code>resource-type</code> - The resource type (<code>vpc</code> | <code>vpn</code>). </li> <li>  <code>state</code> - The state of the attachment (<code>available</code> | <code>deleted</code> | <code>deleting</code> | <code>failed</code> | <code>modifying</code> | <code>pendingAcceptance</code> | <code>pending</code> | <code>rollingBack</code> | <code>rejected</code> | <code>rejecting</code>). </li> <li>  <code>transit-gateway-attachment-id</code> - The ID of the attachment. </li> <li>  <code>transit-gateway-id</code> - The ID of the transit gateway. </li> <li>  <code>transit-gateway-owner-id</code> - The ID of the AWS account that owns the transit gateway. </li> </ul><br/>TransitGatewayAttachmentIds:  The IDs of the attachments.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>TransitGatewayAttachments:  Information about the attachments.<br/>        """
        pass

    def describe_transit_gateway_route_tables(self, TransitGatewayRouteTableIds: List[String]=None, Filters: List[Filter]=None, MaxResults: int=None, NextToken: str=None, DryRun: Boolean=None) -> DescribeTransitGatewayRouteTablesResult:
        """Describes one or more transit gateway route tables. By default, all transit gateway route tables are described. Alternatively, you can filter the results.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>NextToken:  The token for the next page of results.<br/>MaxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>Filters:  One or more filters. The possible values are: <ul> <li>  <code>default-association-route-table</code> - Indicates whether this is the default association route table for the transit gateway (<code>true</code> | <code>false</code>). </li> <li>  <code>default-propagation-route-table</code> - Indicates whether this is the default propagation route table for the transit gateway (<code>true</code> | <code>false</code>). </li> <li>  <code>state</code> - The state of the attachment (<code>available</code> | <code>deleted</code> | <code>deleting</code> | <code>failed</code> | <code>modifying</code> | <code>pendingAcceptance</code> | <code>pending</code> | <code>rollingBack</code> | <code>rejected</code> | <code>rejecting</code>). </li> <li>  <code>transit-gateway-id</code> - The ID of the transit gateway. </li> <li>  <code>transit-gateway-route-table-id</code> - The ID of the transit gateway route table. </li> </ul><br/>TransitGatewayRouteTableIds:  The IDs of the transit gateway route tables.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>TransitGatewayRouteTables:  Information about the transit gateway route tables.<br/>        """
        pass

    def describe_transit_gateway_vpc_attachments(self, TransitGatewayAttachmentIds: List[String]=None, Filters: List[Filter]=None, MaxResults: int=None, NextToken: str=None, DryRun: Boolean=None) -> DescribeTransitGatewayVpcAttachmentsResult:
        """Describes one or more VPC attachments. By default, all VPC attachments are described. Alternatively, you can filter the results.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>NextToken:  The token for the next page of results.<br/>MaxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>Filters:  One or more filters. The possible values are: <ul> <li>  <code>state</code> - The state of the attachment (<code>available</code> | <code>deleted</code> | <code>deleting</code> | <code>failed</code> | <code>modifying</code> | <code>pendingAcceptance</code> | <code>pending</code> | <code>rollingBack</code> | <code>rejected</code> | <code>rejecting</code>). </li> <li>  <code>transit-gateway-attachment-id</code> - The ID of the attachment. </li> <li>  <code>transit-gateway-id</code> - The ID of the transit gateway. </li> <li>  <code>vpc-id</code> - The ID of the VPC. </li> </ul><br/>TransitGatewayAttachmentIds:  The IDs of the attachments.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>TransitGatewayVpcAttachments:  Information about the VPC attachments.<br/>        """
        pass

    def describe_transit_gateways(self, TransitGatewayIds: List[String]=None, Filters: List[Filter]=None, MaxResults: int=None, NextToken: str=None, DryRun: Boolean=None) -> DescribeTransitGatewaysResult:
        """Describes one or more transit gateways. By default, all transit gateways are described. Alternatively, you can filter the results.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>NextToken:  The token for the next page of results.<br/>MaxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>Filters:  One or more filters. The possible values are: <ul> <li>  <code>options.propagation-default-route-table-id</code> - The ID of the default propagation route table. </li> <li>  <code>options.amazon-side-asn</code> - The private ASN for the Amazon side of a BGP session. </li> <li>  <code>options.association-default-route-table-id</code> - The ID of the default association route table. </li> <li>  <code>options.auto-accept-shared-attachments</code> - Indicates whether there is automatic acceptance of attachment requests (<code>enable</code> | <code>disable</code>). </li> <li>  <code>options.default-route-table-association</code> - Indicates whether resource attachments are automatically associated with the default association route table (<code>enable</code> | <code>disable</code>). </li> <li>  <code>options.default-route-table-propagation</code> - Indicates whether resource attachments automatically propagate routes to the default propagation route table (<code>enable</code> | <code>disable</code>). </li> <li>  <code>options.dns-support</code> - Indicates whether DNS support is enabled (<code>enable</code> | <code>disable</code>). </li> <li>  <code>options.vpn-ecmp-support</code> - Indicates whether Equal Cost Multipath Protocol support is enabled (<code>enable</code> | <code>disable</code>). </li> <li>  <code>owner-id</code> - The ID of the AWS account that owns the transit gateway. </li> <li>  <code>state</code> - The state of the attachment (<code>available</code> | <code>deleted</code> | <code>deleting</code> | <code>failed</code> | <code>modifying</code> | <code>pendingAcceptance</code> | <code>pending</code> | <code>rollingBack</code> | <code>rejected</code> | <code>rejecting</code>). </li> <li>  <code>transit-gateway-id</code> - The ID of the transit gateway. </li> </ul><br/>TransitGatewayIds:  The IDs of the transit gateways.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>TransitGateways:  Information about the transit gateways.<br/>        """
        pass

    def describe_volume_attribute(self, VolumeId: str, Attribute: str, DryRun: Boolean=None) -> DescribeVolumeAttributeResult:
        """Describes the specified attribute of the specified volume. You can specify only one attribute at a time. For more information about EBS volumes, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSVolumes.html">Amazon EBS Volumes</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>VolumeId:  The ID of the volume.<br/>Attribute:  The attribute of the volume. This parameter is required.<br/>
        :return: VolumeId:  The ID of the volume.<br/>ProductCodes:  A list of product codes.<br/>AutoEnableIO:  The state of <code>autoEnableIO</code> attribute.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The valid values are <code>true</code> or <code>false</code>.<br/>        """
        pass

    def describe_volume_status(self, Filters: List[Filter]=None, MaxResults: int=None, NextToken: str=None, VolumeIds: List[String]=None, DryRun: Boolean=None) -> DescribeVolumeStatusResult:
        """Describes the status of the specified volumes. Volume status provides the result of the checks performed on your volumes to determine events that can impair the performance of your volumes. The performance of a volume can be affected if an issue occurs on the volume's underlying host. If the volume's underlying host experiences a power outage or system issue, after the system is restored, there could be data inconsistencies on the volume. Volume events notify you if this occurs. Volume actions notify you if any action needs to be taken in response to the event. The <code>DescribeVolumeStatus</code> operation provides the following information about the specified volumes:  <i>Status</i>: Reflects the current status of the volume. The possible values are <code>ok</code>, <code>impaired</code> , <code>warning</code>, or <code>insufficient-data</code>. If all checks pass, the overall status of the volume is <code>ok</code>. If the check fails, the overall status is <code>impaired</code>. If the status is <code>insufficient-data</code>, then the checks may still be taking place on your volume at the time. We recommend that you retry the request. For more information about volume status, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/monitoring-volume-status.html">Monitoring the Status of Your Volumes</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.  <i>Events</i>: Reflect the cause of a volume status and may require you to take action. For example, if your volume returns an <code>impaired</code> status, then the volume event might be <code>potential-data-inconsistency</code>. This means that your volume has been affected by an issue with the underlying host, has all I/O operations disabled, and may have inconsistent data.  <i>Actions</i>: Reflect the actions you may have to take in response to an event. For example, if the status of the volume is <code>impaired</code> and the volume event shows <code>potential-data-inconsistency</code>, then the action shows <code>enable-volume-io</code>. This means that you may want to enable the I/O operations for the volume by calling the <a>EnableVolumeIO</a> action and then check the volume for data consistency. Volume status is based on the volume status checks, and does not reflect the volume state. Therefore, volume status does not indicate volumes in the <code>error</code> state (for example, when a volume is incapable of accepting I/O.)

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>VolumeIds:  The IDs of the volumes. Default: Describes all your volumes.<br/>NextToken:  The <code>NextToken</code> value to include in a future <code>DescribeVolumeStatus</code> request. When the results of the request exceed <code>MaxResults</code>, this value can be used to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>MaxResults:  The maximum number of volume results returned by <code>DescribeVolumeStatus</code> in paginated output. When this parameter is used, the request only returns <code>MaxResults</code> results in a single page along with a <code>NextToken</code> response element. The remaining results of the initial request can be seen by sending another request with the returned <code>NextToken</code> value. This value can be between 5 and 1000; if <code>MaxResults</code> is given a value larger than 1000, only 1000 results are returned. If this parameter is not used, then <code>DescribeVolumeStatus</code> returns all results. You cannot specify this parameter and the volume IDs parameter in the same request.<br/>Filters:  The filters. <ul> <li>  <code>action.code</code> - The action code for the event (for example, <code>enable-volume-io</code>). </li> <li>  <code>action.description</code> - A description of the action. </li> <li>  <code>action.event-id</code> - The event ID associated with the action. </li> <li>  <code>availability-zone</code> - The Availability Zone of the instance. </li> <li>  <code>event.description</code> - A description of the event. </li> <li>  <code>event.event-id</code> - The event ID. </li> <li>  <code>event.event-type</code> - The event type (for <code>io-enabled</code>: <code>passed</code> | <code>failed</code>; for <code>io-performance</code>: <code>io-performance:degraded</code> | <code>io-performance:severely-degraded</code> | <code>io-performance:stalled</code>). </li> <li>  <code>event.not-after</code> - The latest end time for the event. </li> <li>  <code>event.not-before</code> - The earliest start time for the event. </li> <li>  <code>volume-status.details-name</code> - The cause for <code>volume-status.status</code> (<code>io-enabled</code> | <code>io-performance</code>). </li> <li>  <code>volume-status.details-status</code> - The status of <code>volume-status.details-name</code> (for <code>io-enabled</code>: <code>passed</code> | <code>failed</code>; for <code>io-performance</code>: <code>normal</code> | <code>degraded</code> | <code>severely-degraded</code> | <code>stalled</code>). </li> <li>  <code>volume-status.status</code> - The status of the volume (<code>ok</code> | <code>impaired</code> | <code>warning</code> | <code>insufficient-data</code>). </li> </ul><br/>
        :return: VolumeStatuses:  Information about the status of the volumes.<br/>NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>        """
        pass

    def describe_volumes(self, Filters: List[Filter]=None, VolumeIds: List[String]=None, DryRun: Boolean=None, MaxResults: int=None, NextToken: str=None) -> DescribeVolumesResult:
        """Describes the specified EBS volumes or all of your EBS volumes. If you are describing a long list of volumes, you can paginate the output to make the list more manageable. The <code>MaxResults</code> parameter sets the maximum number of results returned in a single page. If the list of results exceeds your <code>MaxResults</code> value, then that number of results is returned along with a <code>NextToken</code> value that can be passed to a subsequent <code>DescribeVolumes</code> request to retrieve the remaining results. For more information about EBS volumes, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSVolumes.html">Amazon EBS Volumes</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param NextToken:  The <code>NextToken</code> value returned from a previous paginated <code>DescribeVolumes</code> request where <code>MaxResults</code> was used and the results exceeded the value of that parameter. Pagination continues from the end of the previous results that returned the <code>NextToken</code> value. This value is <code>null</code> when there are no more results to return.<br/>MaxResults:  The maximum number of volume results returned by <code>DescribeVolumes</code> in paginated output. When this parameter is used, <code>DescribeVolumes</code> only returns <code>MaxResults</code> results in a single page along with a <code>NextToken</code> response element. The remaining results of the initial request can be seen by sending another <code>DescribeVolumes</code> request with the returned <code>NextToken</code> value. This value can be between 5 and 500; if <code>MaxResults</code> is given a value larger than 500, only 500 results are returned. If this parameter is not used, then <code>DescribeVolumes</code> returns all results. You cannot specify this parameter and the volume IDs parameter in the same request.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>VolumeIds:  The volume IDs.<br/>Filters:  The filters. <ul> <li>  <code>attachment.attach-time</code> - The time stamp when the attachment initiated. </li> <li>  <code>attachment.delete-on-termination</code> - Whether the volume is deleted on instance termination. </li> <li>  <code>attachment.device</code> - The device name specified in the block device mapping (for example, <code>/dev/sda1</code>). </li> <li>  <code>attachment.instance-id</code> - The ID of the instance the volume is attached to. </li> <li>  <code>attachment.status</code> - The attachment state (<code>attaching</code> | <code>attached</code> | <code>detaching</code>). </li> <li>  <code>availability-zone</code> - The Availability Zone in which the volume was created. </li> <li>  <code>create-time</code> - The time stamp when the volume was created. </li> <li>  <code>encrypted</code> - Indicates whether the volume is encrypted (<code>true</code> | <code>false</code>) </li> <li>  <code>size</code> - The size of the volume, in GiB. </li> <li>  <code>snapshot-id</code> - The snapshot from which the volume was created. </li> <li>  <code>status</code> - The status of the volume (<code>creating</code> | <code>available</code> | <code>in-use</code> | <code>deleting</code> | <code>deleted</code> | <code>error</code>). </li> <li>  <code>tag</code>:&lt;key&gt; - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value. </li> <li>  <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value. </li> <li>  <code>volume-id</code> - The volume ID. </li> <li>  <code>volume-type</code> - The Amazon EBS volume type. This can be <code>gp2</code> for General Purpose SSD, <code>io1</code> for Provisioned IOPS SSD, <code>st1</code> for Throughput Optimized HDD, <code>sc1</code> for Cold HDD, or <code>standard</code> for Magnetic volumes. </li> </ul><br/>
        :return: NextToken:  The <code>NextToken</code> value to include in a future <code>DescribeVolumes</code> request. When the results of a <code>DescribeVolumes</code> request exceed <code>MaxResults</code>, this value can be used to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>Volumes:  Information about the volumes.<br/>        """
        pass

    def describe_volumes_modifications(self, DryRun: Boolean=None, VolumeIds: List[String]=None, Filters: List[Filter]=None, NextToken: str=None, MaxResults: int=None) -> DescribeVolumesModificationsResult:
        """Reports the current modification status of EBS volumes. Current-generation EBS volumes support modification of attributes including type, size, and (for <code>io1</code> volumes) IOPS provisioning while either attached to or detached from an instance. Following an action from the API or the console to modify a volume, the status of the modification may be <code>modifying</code>, <code>optimizing</code>, <code>completed</code>, or <code>failed</code>. If a volume has never been modified, then certain elements of the returned <code>VolumeModification</code> objects are null.   You can also use CloudWatch Events to check the status of a modification to an EBS volume. For information about CloudWatch Events, see the <a href="https://docs.aws.amazon.com/AmazonCloudWatch/latest/events/">Amazon CloudWatch Events User Guide</a>. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ebs-expand-volume.html#monitoring_mods">Monitoring Volume Modifications"</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param MaxResults:  The maximum number of results (up to a limit of 500) to be returned in a paginated request.<br/>NextToken:  The <code>nextToken</code> value returned by a previous paginated request.<br/>Filters:  The filters. Supported filters: <code>volume-id</code>, <code>modification-state</code>, <code>target-size</code>, <code>target-iops</code>, <code>target-volume-type</code>, <code>original-size</code>, <code>original-iops</code>, <code>original-volume-type</code>, <code>start-time</code>. <br/>VolumeIds:  The IDs of the volumes for which in-progress modifications will be described.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: NextToken:  Token for pagination, null if there are no more results <br/>VolumesModifications:  Information about the volume modifications.<br/>        """
        pass

    def describe_vpc_attribute(self, VpcId: str, Attribute: str, DryRun: Boolean=None) -> DescribeVpcAttributeResult:
        """Describes the specified attribute of the specified VPC. You can specify only one attribute at a time.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>VpcId:  The ID of the VPC.<br/>Attribute:  The VPC attribute.<br/>
        :return: EnableDnsSupport:  Indicates whether DNS resolution is enabled for the VPC. If this attribute is <code>true</code>, the Amazon DNS server resolves DNS hostnames for your instances to their corresponding IP addresses; otherwise, it does not.:<br/>EnableDnsHostnames:  Indicates whether the instances launched in the VPC get DNS hostnames. If this attribute is <code>true</code>, instances in the VPC get DNS hostnames; otherwise, they do not.:<br/>VpcId:  The ID of the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The valid values are <code>true</code> or <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The valid values are <code>true</code> or <code>false</code>.<br/>        """
        pass

    def describe_vpc_classic_link(self, Filters: List[Filter]=None, DryRun: Boolean=None, VpcIds: List[String]=None) -> DescribeVpcClassicLinkResult:
        """Describes the ClassicLink status of one or more VPCs.

        :param self:
        :param VpcIds:  One or more VPCs for which you want to describe the ClassicLink status.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Filters:  One or more filters. <ul> <li>  <code>is-classic-link-enabled</code> - Whether the VPC is enabled for ClassicLink (<code>true</code> | <code>false</code>). </li> <li>  <code>tag</code>:&lt;key&gt; - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value. </li> <li>  <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value. </li> </ul><br/>
        :return: Vpcs:  The ClassicLink status of one or more VPCs.<br/>        """
        pass

    def describe_vpc_classic_link_dns_support(self, MaxResults: int=None, NextToken: str=None, VpcIds: List[String]=None) -> DescribeVpcClassicLinkDnsSupportResult:
        """Describes the ClassicLink DNS support status of one or more VPCs. If enabled, the DNS hostname of a linked EC2-Classic instance resolves to its private IP address when addressed from an instance in the VPC to which it's linked. Similarly, the DNS hostname of an instance in a VPC resolves to its private IP address when addressed from a linked EC2-Classic instance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/vpc-classiclink.html">ClassicLink</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param VpcIds:  One or more VPC IDs.<br/>NextToken:  The token for the next page of results.<br/>MaxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>
        :return: Vpcs:  Information about the ClassicLink DNS support status of the VPCs.<br/>NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>        """
        pass

    def describe_vpc_endpoint_connection_notifications(self, DryRun: Boolean=None, ConnectionNotificationId: str=None, Filters: List[Filter]=None, MaxResults: int=None, NextToken: str=None) -> DescribeVpcEndpointConnectionNotificationsResult:
        """Describes the connection notifications for VPC endpoints and VPC endpoint services.

        :param self:
        :param NextToken:  The token to request the next page of results.<br/>MaxResults:  The maximum number of results to return in a single call. To retrieve the remaining results, make another request with the returned <code>NextToken</code> value.<br/>Filters:  One or more filters. <ul> <li>  <code>connection-notification-arn</code> - The ARN of SNS topic for the notification. </li> <li>  <code>connection-notification-id</code> - The ID of the notification. </li> <li>  <code>connection-notification-state</code> - The state of the notification (<code>Enabled</code> | <code>Disabled</code>). </li> <li>  <code>connection-notification-type</code> - The type of notification (<code>Topic</code>). </li> <li>  <code>service-id</code> - The ID of the endpoint service. </li> <li>  <code>vpc-endpoint-id</code> - The ID of the VPC endpoint. </li> </ul><br/>ConnectionNotificationId:  The ID of the notification.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>ConnectionNotificationSet:  One or more notifications.<br/>        """
        pass

    def describe_vpc_endpoint_connections(self, DryRun: Boolean=None, Filters: List[Filter]=None, MaxResults: int=None, NextToken: str=None) -> DescribeVpcEndpointConnectionsResult:
        """Describes the VPC endpoint connections to your VPC endpoint services, including any endpoints that are pending your acceptance.

        :param self:
        :param NextToken:  The token to retrieve the next page of results.<br/>MaxResults:  The maximum number of results to return for the request in a single page. The remaining results of the initial request can be seen by sending another request with the returned <code>NextToken</code> value. This value can be between 5 and 1000; if <code>MaxResults</code> is given a value larger than 1000, only 1000 results are returned.<br/>Filters:  One or more filters. <ul> <li>  <code>service-id</code> - The ID of the service. </li> <li>  <code>vpc-endpoint-owner</code> - The AWS account number of the owner of the endpoint. </li> <li>  <code>vpc-endpoint-state</code> - The state of the endpoint (<code>pendingAcceptance</code> | <code>pending</code> | <code>available</code> | <code>deleting</code> | <code>deleted</code> | <code>rejected</code> | <code>failed</code>). </li> <li>  <code>vpc-endpoint-id</code> - The ID of the endpoint. </li> </ul><br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>VpcEndpointConnections:  Information about one or more VPC endpoint connections.<br/>        """
        pass

    def describe_vpc_endpoint_service_configurations(self, DryRun: Boolean=None, ServiceIds: List[String]=None, Filters: List[Filter]=None, MaxResults: int=None, NextToken: str=None) -> DescribeVpcEndpointServiceConfigurationsResult:
        """Describes the VPC endpoint service configurations in your account (your services).

        :param self:
        :param NextToken:  The token to retrieve the next page of results.<br/>MaxResults:  The maximum number of results to return for the request in a single page. The remaining results of the initial request can be seen by sending another request with the returned <code>NextToken</code> value. This value can be between 5 and 1000; if <code>MaxResults</code> is given a value larger than 1000, only 1000 results are returned.<br/>Filters:  One or more filters. <ul> <li>  <code>service-name</code> - The name of the service. </li> <li>  <code>service-id</code> - The ID of the service. </li> <li>  <code>service-state</code> - The state of the service (<code>Pending</code> | <code>Available</code> | <code>Deleting</code> | <code>Deleted</code> | <code>Failed</code>).  </li> <li>  <code>tag</code>:&lt;key&gt; - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value. </li> <li>  <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value. </li> </ul><br/>ServiceIds:  The IDs of one or more services.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>ServiceConfigurations:  Information about one or more services.<br/>        """
        pass

    def describe_vpc_endpoint_service_permissions(self, ServiceId: str, DryRun: Boolean=None, Filters: List[Filter]=None, MaxResults: int=None, NextToken: str=None) -> DescribeVpcEndpointServicePermissionsResult:
        """Describes the principals (service consumers) that are permitted to discover your VPC endpoint service.

        :param self:
        :param NextToken:  The token to retrieve the next page of results.<br/>MaxResults:  The maximum number of results to return for the request in a single page. The remaining results of the initial request can be seen by sending another request with the returned <code>NextToken</code> value. This value can be between 5 and 1000; if <code>MaxResults</code> is given a value larger than 1000, only 1000 results are returned.<br/>Filters:  One or more filters. <ul> <li>  <code>principal</code> - The ARN of the principal. </li> <li>  <code>principal-type</code> - The principal type (<code>All</code> | <code>Service</code> | <code>OrganizationUnit</code> | <code>Account</code> | <code>User</code> | <code>Role</code>). </li> </ul><br/>ServiceId:  The ID of the service.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>AllowedPrincipals:  Information about one or more allowed principals.<br/>        """
        pass

    def describe_vpc_endpoint_services(self, DryRun: Boolean=None, ServiceNames: List[String]=None, Filters: List[Filter]=None, MaxResults: int=None, NextToken: str=None) -> DescribeVpcEndpointServicesResult:
        """Describes available services to which you can create a VPC endpoint.

        :param self:
        :param NextToken:  The token for the next set of items to return. (You received this token from a prior call.)<br/>MaxResults:  The maximum number of items to return for this request. The request returns a token that you can specify in a subsequent call to get the next set of results. Constraint: If the value is greater than 1000, we return only 1000 items.<br/>Filters:  One or more filters. <ul> <li>  <code>service-name</code>: The name of the service. </li> <li>  <code>tag</code>:&lt;key&gt; - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value. </li> <li>  <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value. </li> </ul><br/>ServiceNames:  One or more service names.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: NextToken:  The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.<br/>ServiceDetails:  Information about the service.<br/>ServiceNames:  A list of supported services.<br/>        """
        pass

    def describe_vpc_endpoints(self, DryRun: Boolean=None, VpcEndpointIds: List[String]=None, Filters: List[Filter]=None, MaxResults: int=None, NextToken: str=None) -> DescribeVpcEndpointsResult:
        """Describes one or more of your VPC endpoints.

        :param self:
        :param NextToken:  The token for the next set of items to return. (You received this token from a prior call.)<br/>MaxResults:  The maximum number of items to return for this request. The request returns a token that you can specify in a subsequent call to get the next set of results. Constraint: If the value is greater than 1000, we return only 1000 items.<br/>Filters:  One or more filters. <ul> <li>  <code>service-name</code>: The name of the service. </li> <li>  <code>vpc-id</code>: The ID of the VPC in which the endpoint resides. </li> <li>  <code>vpc-endpoint-id</code>: The ID of the endpoint. </li> <li>  <code>vpc-endpoint-state</code>: The state of the endpoint. (<code>pending</code> | <code>available</code> | <code>deleting</code> | <code>deleted</code>) </li> <li>  <code>tag</code>:&lt;key&gt; - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value. </li> <li>  <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value. </li> </ul><br/>VpcEndpointIds:  One or more endpoint IDs.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: NextToken:  The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.<br/>VpcEndpoints:  Information about the endpoints.<br/>        """
        pass

    def describe_vpc_peering_connections(self, Filters: List[Filter]=None, DryRun: Boolean=None, VpcPeeringConnectionIds: List[String]=None, NextToken: str=None, MaxResults: int=None) -> DescribeVpcPeeringConnectionsResult:
        """Describes one or more of your VPC peering connections.

        :param self:
        :param MaxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>NextToken:  The token for the next page of results.<br/>VpcPeeringConnectionIds:  One or more VPC peering connection IDs. Default: Describes all your VPC peering connections.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Filters:  One or more filters. <ul> <li>  <code>accepter-vpc-info.cidr-block</code> - The IPv4 CIDR block of the accepter VPC. </li> <li>  <code>accepter-vpc-info.owner-id</code> - The AWS account ID of the owner of the accepter VPC. </li> <li>  <code>accepter-vpc-info.vpc-id</code> - The ID of the accepter VPC. </li> <li>  <code>expiration-time</code> - The expiration date and time for the VPC peering connection. </li> <li>  <code>requester-vpc-info.cidr-block</code> - The IPv4 CIDR block of the requester's VPC. </li> <li>  <code>requester-vpc-info.owner-id</code> - The AWS account ID of the owner of the requester VPC. </li> <li>  <code>requester-vpc-info.vpc-id</code> - The ID of the requester VPC. </li> <li>  <code>status-code</code> - The status of the VPC peering connection (<code>pending-acceptance</code> | <code>failed</code> | <code>expired</code> | <code>provisioning</code> | <code>active</code> | <code>deleting</code> | <code>deleted</code> | <code>rejected</code>). </li> <li>  <code>status-message</code> - A message that provides more information about the status of the VPC peering connection, if applicable. </li> <li>  <code>tag</code>:&lt;key&gt; - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value. </li> <li>  <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value. </li> <li>  <code>vpc-peering-connection-id</code> - The ID of the VPC peering connection. </li> </ul><br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>VpcPeeringConnections:  Information about the VPC peering connections.<br/>        """
        pass

    def describe_vpcs(self, Filters: List[Filter]=None, VpcIds: List[String]=None, DryRun: Boolean=None, NextToken: str=None, MaxResults: int=None) -> DescribeVpcsResult:
        """Describes one or more of your VPCs.

        :param self:
        :param MaxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>NextToken:  The token for the next page of results.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>VpcIds:  One or more VPC IDs. Default: Describes all your VPCs.<br/>Filters:  One or more filters. <ul> <li>  <code>cidr</code> - The primary IPv4 CIDR block of the VPC. The CIDR block you specify must exactly match the VPC's CIDR block for information to be returned for the VPC. Must contain the slash followed by one or two digits (for example, <code>/28</code>). </li> <li>  <code>cidr-block-association.cidr-block</code> - An IPv4 CIDR block associated with the VPC. </li> <li>  <code>cidr-block-association.association-id</code> - The association ID for an IPv4 CIDR block associated with the VPC. </li> <li>  <code>cidr-block-association.state</code> - The state of an IPv4 CIDR block associated with the VPC. </li> <li>  <code>dhcp-options-id</code> - The ID of a set of DHCP options. </li> <li>  <code>ipv6-cidr-block-association.ipv6-cidr-block</code> - An IPv6 CIDR block associated with the VPC. </li> <li>  <code>ipv6-cidr-block-association.association-id</code> - The association ID for an IPv6 CIDR block associated with the VPC. </li> <li>  <code>ipv6-cidr-block-association.state</code> - The state of an IPv6 CIDR block associated with the VPC. </li> <li>  <code>isDefault</code> - Indicates whether the VPC is the default VPC. </li> <li>  <code>owner-id</code> - The ID of the AWS account that owns the VPC. </li> <li>  <code>state</code> - The state of the VPC (<code>pending</code> | <code>available</code>). </li> <li>  <code>tag</code>:&lt;key&gt; - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value. </li> <li>  <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value. </li> <li>  <code>vpc-id</code> - The ID of the VPC. </li> </ul><br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>Vpcs:  Information about one or more VPCs.<br/>        """
        pass

    def describe_vpn_connections(self, Filters: List[Filter]=None, VpnConnectionIds: List[String]=None, DryRun: Boolean=None) -> DescribeVpnConnectionsResult:
        """Describes one or more of your VPN connections. For more information, see <a href="https://docs.aws.amazon.com/vpn/latest/s2svpn/VPC_VPN.html">AWS Site-to-Site VPN</a> in the <i>AWS Site-to-Site VPN User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>VpnConnectionIds:  One or more VPN connection IDs. Default: Describes your VPN connections.<br/>Filters:  One or more filters. <ul> <li>  <code>customer-gateway-configuration</code> - The configuration information for the customer gateway. </li> <li>  <code>customer-gateway-id</code> - The ID of a customer gateway associated with the VPN connection. </li> <li>  <code>state</code> - The state of the VPN connection (<code>pending</code> | <code>available</code> | <code>deleting</code> | <code>deleted</code>). </li> <li>  <code>option.static-routes-only</code> - Indicates whether the connection has static routes only. Used for devices that do not support Border Gateway Protocol (BGP). </li> <li>  <code>route.destination-cidr-block</code> - The destination CIDR block. This corresponds to the subnet used in a customer data center. </li> <li>  <code>bgp-asn</code> - The BGP Autonomous System Number (ASN) associated with a BGP device. </li> <li>  <code>tag</code>:&lt;key&gt; - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value. </li> <li>  <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value. </li> <li>  <code>type</code> - The type of VPN connection. Currently the only supported type is <code>ipsec.1</code>. </li> <li>  <code>vpn-connection-id</code> - The ID of the VPN connection. </li> <li>  <code>vpn-gateway-id</code> - The ID of a virtual private gateway associated with the VPN connection. </li> </ul><br/>
        :return: VpnConnections:  Information about one or more VPN connections.<br/>        """
        pass

    def describe_vpn_gateways(self, Filters: List[Filter]=None, VpnGatewayIds: List[String]=None, DryRun: Boolean=None) -> DescribeVpnGatewaysResult:
        """Describes one or more of your virtual private gateways. For more information, see <a href="https://docs.aws.amazon.com/vpn/latest/s2svpn/VPC_VPN.html">AWS Site-to-Site VPN</a> in the <i>AWS Site-to-Site VPN User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>VpnGatewayIds:  One or more virtual private gateway IDs. Default: Describes all your virtual private gateways.<br/>Filters:  One or more filters. <ul> <li>  <code>amazon-side-asn</code> - The Autonomous System Number (ASN) for the Amazon side of the gateway. </li> <li>  <code>attachment.state</code> - The current state of the attachment between the gateway and the VPC (<code>attaching</code> | <code>attached</code> | <code>detaching</code> | <code>detached</code>). </li> <li>  <code>attachment.vpc-id</code> - The ID of an attached VPC. </li> <li>  <code>availability-zone</code> - The Availability Zone for the virtual private gateway (if applicable). </li> <li>  <code>state</code> - The state of the virtual private gateway (<code>pending</code> | <code>available</code> | <code>deleting</code> | <code>deleted</code>). </li> <li>  <code>tag</code>:&lt;key&gt; - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value. </li> <li>  <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value. </li> <li>  <code>type</code> - The type of virtual private gateway. Currently the only supported type is <code>ipsec.1</code>. </li> <li>  <code>vpn-gateway-id</code> - The ID of the virtual private gateway. </li> </ul><br/>
        :return: VpnGateways:  Information about one or more virtual private gateways.<br/>        """
        pass

    def detach_classic_link_vpc(self, VpcId: str, InstanceId: str, DryRun: Boolean=None) -> DetachClassicLinkVpcResult:
        """Unlinks (detaches) a linked EC2-Classic instance from a VPC. After the instance has been unlinked, the VPC security groups are no longer associated with it. An instance is automatically unlinked from a VPC when it's stopped.

        :param self:
        :param VpcId:  The ID of the VPC to which the instance is linked.<br/>InstanceId:  The ID of the instance to unlink from the VPC.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: Return:  Returns <code>true</code> if the request succeeds; otherwise, it returns an error.<br/>        """
        pass

    def detach_internet_gateway(self, VpcId: str, InternetGatewayId: str, DryRun: Boolean=None):
        """Detaches an internet gateway from a VPC, disabling connectivity between the internet and the VPC. The VPC must not contain any running instances with Elastic IP addresses or public IPv4 addresses.

        :param self:
        :param VpcId:  The ID of the VPC.<br/>InternetGatewayId:  The ID of the internet gateway.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return:         """
        pass

    def detach_network_interface(self, AttachmentId: str, DryRun: Boolean=None, Force: Boolean=None):
        """Detaches a network interface from an instance.

        :param self:
        :param Force:  Specifies whether to force a detachment.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>AttachmentId:  The ID of the attachment.<br/>
        :return:         """
        pass

    def detach_volume(self, VolumeId: str, Device: str=None, Force: Boolean=None, InstanceId: str=None, DryRun: Boolean=None) -> VolumeAttachment:
        """Detaches an EBS volume from an instance. Make sure to unmount any file systems on the device within your operating system before detaching the volume. Failure to do so can result in the volume becoming stuck in the <code>busy</code> state while detaching. If this happens, detachment can be delayed indefinitely until you unmount the volume, force detachment, reboot the instance, or all three. If an EBS volume is the root device of an instance, it can't be detached while the instance is running. To detach the root volume, stop the instance first. When a volume with an AWS Marketplace product code is detached from an instance, the product code is no longer associated with the instance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ebs-detaching-volume.html">Detaching an Amazon EBS Volume</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>VolumeId:  The ID of the volume.<br/>InstanceId:  The ID of the instance.<br/>Force:  Forces detachment if the previous detachment attempt did not occur cleanly (for example, logging into an instance, unmounting the volume, and detaching normally). This option can lead to data loss or a corrupted file system. Use this option only as a last resort to detach a volume from a failed instance. The instance won't have an opportunity to flush file system caches or file system metadata. If you use this option, you must perform file system check and repair procedures.<br/>Device:  The device name.<br/>
        :return: DeleteOnTermination:  Indicates whether the EBS volume is deleted on instance termination.<br/>VolumeId:  The ID of the volume.<br/>State:  The attachment state of the volume.<br/>InstanceId:  The ID of the instance.<br/>Device:  The device name.<br/>AttachTime:  The time stamp when the attachment initiated.<br/>        """
        pass

    def detach_vpn_gateway(self, VpnGatewayId: str, VpcId: str, DryRun: Boolean=None):
        """Detaches a virtual private gateway from a VPC. You do this if you're planning to turn off the VPC and not use it anymore. You can confirm a virtual private gateway has been completely detached from a VPC by describing the virtual private gateway (any attachments to the virtual private gateway are also described). You must wait for the attachment's state to switch to <code>detached</code> before you can delete the VPC or attach a different VPC to the virtual private gateway.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>VpnGatewayId:  The ID of the virtual private gateway.<br/>VpcId:  The ID of the VPC.<br/>
        :return:         """
        pass

    def disable_ebs_encryption_by_default(self, DryRun: Boolean=None) -> DisableEbsEncryptionByDefaultResult:
        """Disables EBS encryption by default for your account in the current Region. After you disable encryption by default, you can still create encrypted volumes by enabling encryption when you create each volume. Disabling encryption by default does not change the encryption status of your existing volumes. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html">Amazon EBS Encryption</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>. <br/>
        :return: EbsEncryptionByDefault:  The updated status of encryption by default.<br/>        """
        pass

    def disable_transit_gateway_route_table_propagation(self, TransitGatewayAttachmentId: str, TransitGatewayRouteTableId: str, DryRun: Boolean=None) -> DisableTransitGatewayRouteTablePropagationResult:
        """Disables the specified resource attachment from propagating routes to the specified propagation route table.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>TransitGatewayAttachmentId:  The ID of the attachment.<br/>TransitGatewayRouteTableId:  The ID of the propagation route table.<br/>
        :return: Propagation:  Information about route propagation.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayRouteTableId</i> The ID of the transit gateway route table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceType</i> The resource type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceId</i> The ID of the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayAttachmentId</i> The ID of the attachment.<br/>        """
        pass

    def disable_vgw_route_propagation(self, RouteTableId: str, GatewayId: str):
        """Disables a virtual private gateway (VGW) from propagating routes to a specified route table of a VPC.

        :param self:
        :param RouteTableId:  The ID of the route table.<br/>GatewayId:  The ID of the virtual private gateway.<br/>
        :return:         """
        pass

    def disable_vpc_classic_link(self, VpcId: str, DryRun: Boolean=None) -> DisableVpcClassicLinkResult:
        """Disables ClassicLink for a VPC. You cannot disable ClassicLink for a VPC that has EC2-Classic instances linked to it.

        :param self:
        :param VpcId:  The ID of the VPC.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: Return:  Returns <code>true</code> if the request succeeds; otherwise, it returns an error.<br/>        """
        pass

    def disable_vpc_classic_link_dns_support(self, VpcId: str=None) -> DisableVpcClassicLinkDnsSupportResult:
        """Disables ClassicLink DNS support for a VPC. If disabled, DNS hostnames resolve to public IP addresses when addressed between a linked EC2-Classic instance and instances in the VPC to which it's linked. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/vpc-classiclink.html">ClassicLink</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param VpcId:  The ID of the VPC.<br/>
        :return: Return:  Returns <code>true</code> if the request succeeds; otherwise, it returns an error.<br/>        """
        pass

    def disassociate_address(self, AssociationId: str=None, PublicIp: str=None, DryRun: Boolean=None):
        """Disassociates an Elastic IP address from the instance or network interface it's associated with. An Elastic IP address is for use in either the EC2-Classic platform or in a VPC. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/elastic-ip-addresses-eip.html">Elastic IP Addresses</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. This is an idempotent operation. If you perform the operation more than once, Amazon EC2 doesn't return an error.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>PublicIp:  [EC2-Classic] The Elastic IP address. Required for EC2-Classic.<br/>AssociationId:  [EC2-VPC] The association ID. Required for EC2-VPC.<br/>
        :return:         """
        pass

    def disassociate_client_vpn_target_network(self, AssociationId: str, ClientVpnEndpointId: str, DryRun: Boolean=None) -> DisassociateClientVpnTargetNetworkResult:
        """Disassociates a target network from the specified Client VPN endpoint. When you disassociate the last target network from a Client VPN, the following happens: <ul> <li> The route that was automatically added for the VPC is deleted </li> <li> All active client connections are terminated </li> <li> New client connections are disallowed </li> <li> The Client VPN endpoint's status changes to <code>pending-associate</code>  </li> </ul>

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>AssociationId:  The ID of the target network association.<br/>ClientVpnEndpointId:  The ID of the Client VPN endpoint from which to disassociate the target network.<br/>
        :return: Status:  The current state of the target network association.:<br/>AssociationId:  The ID of the target network association.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> A message about the status of the target network association, if applicable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Code</i> The state of the target network association.<br/>        """
        pass

    def disassociate_iam_instance_profile(self, AssociationId: str) -> DisassociateIamInstanceProfileResult:
        """Disassociates an IAM instance profile from a running or stopped instance. Use <a>DescribeIamInstanceProfileAssociations</a> to get the association ID.

        :param self:
        :param AssociationId:  The ID of the IAM instance profile association.<br/>
        :return: IamInstanceProfileAssociation:  Information about the IAM instance profile association.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timestamp</i> The time the IAM instance profile was associated with the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the association.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamInstanceProfile</i> The IAM instance profile.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>IamInstanceProfile</b>: The IAM instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceId</i> The ID of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociationId</i> The ID of the association.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the instance profile.<br/>        """
        pass

    def disassociate_route_table(self, AssociationId: str, DryRun: Boolean=None):
        """Disassociates a subnet from a route table. After you perform this action, the subnet no longer uses the routes in the route table. Instead, it uses the routes in the VPC's main route table. For more information about route tables, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Route_Tables.html">Route Tables</a> in the <i>Amazon Virtual Private Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>AssociationId:  The association ID representing the current association between the route table and subnet.<br/>
        :return:         """
        pass

    def disassociate_subnet_cidr_block(self, AssociationId: str) -> DisassociateSubnetCidrBlockResult:
        """Disassociates a CIDR block from a subnet. Currently, you can disassociate an IPv6 CIDR block only. You must detach or delete all gateways and resources that are associated with the CIDR block before you can disassociate it. 

        :param self:
        :param AssociationId:  The association ID for the CIDR block.<br/>
        :return: SubnetId:  The ID of the subnet.<br/>Ipv6CidrBlockAssociation:  Information about the IPv6 CIDR block association.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Ipv6CidrBlockState</i> Information about the state of the CIDR block.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Ipv6CidrBlockState</b>: Information about the state of the CIDR block.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Ipv6CidrBlock</i> The IPv6 CIDR block.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociationId</i> The association ID for the CIDR block.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> A message about the status of the CIDR block, if applicable.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of a CIDR block.<br/>        """
        pass

    def disassociate_transit_gateway_route_table(self, TransitGatewayAttachmentId: str, TransitGatewayRouteTableId: str, DryRun: Boolean=None) -> DisassociateTransitGatewayRouteTableResult:
        """Disassociates a resource attachment from a transit gateway route table.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>TransitGatewayAttachmentId:  The ID of the attachment.<br/>TransitGatewayRouteTableId:  The ID of the transit gateway route table.<br/>
        :return: Association:  Information about the association.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the association.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceType</i> The resource type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceId</i> The ID of the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayAttachmentId</i> The ID of the attachment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayRouteTableId</i> The ID of the transit gateway route table.<br/>        """
        pass

    def disassociate_vpc_cidr_block(self, AssociationId: str) -> DisassociateVpcCidrBlockResult:
        """Disassociates a CIDR block from a VPC. To disassociate the CIDR block, you must specify its association ID. You can get the association ID by using <a>DescribeVpcs</a>. You must detach or delete all gateways and resources that are associated with the CIDR block before you can disassociate it.  You cannot disassociate the CIDR block with which you originally created the VPC (the primary CIDR block).

        :param self:
        :param AssociationId:  The association ID for the CIDR block.<br/>
        :return: VpcId:  The ID of the VPC.<br/>CidrBlockAssociation:  Information about the IPv4 CIDR block association.:<br/>Ipv6CidrBlockAssociation:  Information about the IPv6 CIDR block association.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Ipv6CidrBlockState</i> Information about the state of the CIDR block.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Ipv6CidrBlockState</b>: Information about the state of the CIDR block.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Ipv6CidrBlock</i> The IPv6 CIDR block.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociationId</i> The association ID for the IPv6 CIDR block.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> A message about the status of the CIDR block, if applicable.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the CIDR block.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CidrBlockState</i> Information about the state of the CIDR block.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CidrBlockState</b>: Information about the state of the CIDR block.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CidrBlock</i> The IPv4 CIDR block.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociationId</i> The association ID for the IPv4 CIDR block.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> A message about the status of the CIDR block, if applicable.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the CIDR block.<br/>        """
        pass

    def enable_ebs_encryption_by_default(self, DryRun: Boolean=None) -> EnableEbsEncryptionByDefaultResult:
        """Enables EBS encryption by default for your account in the current Region. After you enable encryption by default, the EBS volumes that you create are are always encrypted, either using the default CMK or the CMK that you specified when you created each volume. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html">Amazon EBS Encryption</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. You can specify the default CMK for encryption by default using <a>ModifyEbsDefaultKmsKeyId</a> or <a>ResetEbsDefaultKmsKeyId</a>. Enabling encryption by default has no effect on the encryption status of your existing volumes. After you enable encryption by default, you can no longer launch instances using instance types that do not support encryption. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html#EBSEncryption_supported_instances">Supported Instance Types</a>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>. <br/>
        :return: EbsEncryptionByDefault:  The updated status of encryption by default.<br/>        """
        pass

    def enable_transit_gateway_route_table_propagation(self, TransitGatewayAttachmentId: str, TransitGatewayRouteTableId: str, DryRun: Boolean=None) -> EnableTransitGatewayRouteTablePropagationResult:
        """Enables the specified attachment to propagate routes to the specified propagation route table.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>TransitGatewayAttachmentId:  The ID of the attachment.<br/>TransitGatewayRouteTableId:  The ID of the propagation route table.<br/>
        :return: Propagation:  Information about route propagation.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayRouteTableId</i> The ID of the transit gateway route table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceType</i> The resource type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceId</i> The ID of the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayAttachmentId</i> The ID of the attachment.<br/>        """
        pass

    def enable_vgw_route_propagation(self, RouteTableId: str, GatewayId: str):
        """Enables a virtual private gateway (VGW) to propagate routes to the specified route table of a VPC.

        :param self:
        :param RouteTableId:  The ID of the route table. The routing table must be associated with the same VPC that the virtual private gateway is attached to. <br/>GatewayId:  The ID of the virtual private gateway that is attached to a VPC. The virtual private gateway must be attached to the same VPC that the routing tables are associated with. <br/>
        :return:         """
        pass

    def enable_volume_io(self, VolumeId: str, DryRun: Boolean=None):
        """Enables I/O operations for a volume that had I/O operations disabled because the data on the volume was potentially inconsistent.

        :param self:
        :param VolumeId:  The ID of the volume.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return:         """
        pass

    def enable_vpc_classic_link(self, VpcId: str, DryRun: Boolean=None) -> EnableVpcClassicLinkResult:
        """Enables a VPC for ClassicLink. You can then link EC2-Classic instances to your ClassicLink-enabled VPC to allow communication over private IP addresses. You cannot enable your VPC for ClassicLink if any of your VPC route tables have existing routes for address ranges within the <code>10.0.0.0/8</code> IP address range, excluding local routes for VPCs in the <code>10.0.0.0/16</code> and <code>10.1.0.0/16</code> IP address ranges. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/vpc-classiclink.html">ClassicLink</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param VpcId:  The ID of the VPC.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: Return:  Returns <code>true</code> if the request succeeds; otherwise, it returns an error.<br/>        """
        pass

    def enable_vpc_classic_link_dns_support(self, VpcId: str=None) -> EnableVpcClassicLinkDnsSupportResult:
        """Enables a VPC to support DNS hostname resolution for ClassicLink. If enabled, the DNS hostname of a linked EC2-Classic instance resolves to its private IP address when addressed from an instance in the VPC to which it's linked. Similarly, the DNS hostname of an instance in a VPC resolves to its private IP address when addressed from a linked EC2-Classic instance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/vpc-classiclink.html">ClassicLink</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param VpcId:  The ID of the VPC.<br/>
        :return: Return:  Returns <code>true</code> if the request succeeds; otherwise, it returns an error.<br/>        """
        pass

    def export_client_vpn_client_certificate_revocation_list(self, ClientVpnEndpointId: str, DryRun: Boolean=None) -> ExportClientVpnClientCertificateRevocationListResult:
        """Downloads the client certificate revocation list for the specified Client VPN endpoint.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>ClientVpnEndpointId:  The ID of the Client VPN endpoint.<br/>
        :return: Status:  The current state of the client certificate revocation list.:<br/>CertificateRevocationList:  Information about the client certificate revocation list.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> A message about the status of the client certificate revocation list, if applicable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Code</i> The state of the client certificate revocation list.<br/>        """
        pass

    def export_client_vpn_client_configuration(self, ClientVpnEndpointId: str, DryRun: Boolean=None) -> ExportClientVpnClientConfigurationResult:
        """Downloads the contents of the Client VPN endpoint configuration file for the specified Client VPN endpoint. The Client VPN endpoint configuration file includes the Client VPN endpoint and certificate information clients need to establish a connection with the Client VPN endpoint.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>ClientVpnEndpointId:  The ID of the Client VPN endpoint.<br/>
        :return: ClientConfiguration:  The contents of the Client VPN endpoint configuration file.<br/>        """
        pass

    def export_transit_gateway_routes(self, S3Bucket: str, TransitGatewayRouteTableId: str, Filters: List[Filter]=None, DryRun: Boolean=None) -> ExportTransitGatewayRoutesResult:
        """Exports routes from the specified transit gateway route table to the specified S3 bucket. By default, all routes are exported. Alternatively, you can filter by CIDR range.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>S3Bucket:  The name of the S3 bucket.<br/>Filters:  One or more filters. The possible values are: <ul> <li>  <code>attachment.transit-gateway-attachment-id</code> - The id of the transit gateway attachment. </li> <li>  <code>attachment.resource-id</code> - The resource id of the transit gateway attachment. </li> <li>  <code>route-search.exact-match</code> - The exact match of the specified filter. </li> <li>  <code>route-search.longest-prefix-match</code> - The longest prefix that matches the route. </li> <li>  <code>route-search.subnet-of-match</code> - The routes with a subnet that match the specified CIDR filter. </li> <li>  <code>route-search.supernet-of-match</code> - The routes with a CIDR that encompass the CIDR filter. For example, if you have 10.0.1.0/29 and 10.0.1.0/31 routes in your route table and you specify supernet-of-match as 10.0.1.0/30, then the result returns 10.0.1.0/29. </li> <li>  <code>state</code> - The state of the attachment (<code>available</code> | <code>deleted</code> | <code>deleting</code> | <code>failed</code> | <code>modifying</code> | <code>pendingAcceptance</code> | <code>pending</code> | <code>rollingBack</code> | <code>rejected</code> | <code>rejecting</code>). </li> <li>  <code>transit-gateway-route-destination-cidr-block</code> - The CIDR range. </li> <li>  <code>type</code> - The type of route (<code>active</code> | <code>blackhole</code>). </li> </ul><br/>TransitGatewayRouteTableId:  The ID of the route table.<br/>
        :return: S3Location:  The URL of the exported file in Amazon S3. For example, s3://<i>bucket_name</i>/VPCTransitGateway/TransitGatewayRouteTables/<i>file_name</i>.<br/>        """
        pass

    def get_console_output(self, InstanceId: str, DryRun: Boolean=None, Latest: Boolean=None) -> GetConsoleOutputResult:
        """Gets the console output for the specified instance. For Linux instances, the instance console output displays the exact console output that would normally be displayed on a physical monitor attached to a computer. For Windows instances, the instance console output includes the last three system event log errors. By default, the console output returns buffered information that was posted shortly after an instance transition state (start, stop, reboot, or terminate). This information is available for at least one hour after the most recent post. Only the most recent 64 KB of console output is available. You can optionally retrieve the latest serial console output at any time during the instance lifecycle. This option is supported on instance types that use the Nitro hypervisor. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-console.html#instance-console-console-output">Instance Console Output</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param Latest:  When enabled, retrieves the latest console output for the instance. Default: disabled (<code>false</code>)<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>InstanceId:  The ID of the instance.<br/>
        :return: Timestamp:  The time at which the output was last updated.<br/>Output:  The console output, base64-encoded. If you are using a command line tool, the tool decodes the output for you.<br/>InstanceId:  The ID of the instance.<br/>        """
        pass

    def get_console_screenshot(self, InstanceId: str, DryRun: Boolean=None, WakeUp: Boolean=None) -> GetConsoleScreenshotResult:
        """Retrieve a JPG-format screenshot of a running instance to help with troubleshooting. The returned content is Base64-encoded.

        :param self:
        :param WakeUp:  When set to <code>true</code>, acts as keystroke input and wakes up an instance that's in standby or "sleep" mode.<br/>InstanceId:  The ID of the instance.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: InstanceId:  The ID of the instance.<br/>ImageData:  The data that comprises the image.<br/>        """
        pass

    def get_ebs_default_kms_key_id(self, DryRun: Boolean=None) -> GetEbsDefaultKmsKeyIdResult:
        """Describes the default customer master key (CMK) for EBS encryption by default for your account in this Region. You can change the default CMK for encryption by default using <a>ModifyEbsDefaultKmsKeyId</a> or <a>ResetEbsDefaultKmsKeyId</a>. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html">Amazon EBS Encryption</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: KmsKeyId:  The Amazon Resource Name (ARN) of the default CMK for encryption by default.<br/>        """
        pass

    def get_ebs_encryption_by_default(self, DryRun: Boolean=None) -> GetEbsEncryptionByDefaultResult:
        """Describes whether EBS encryption by default is enabled for your account in the current Region. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html">Amazon EBS Encryption</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: EbsEncryptionByDefault:  Indicates whether encryption by default is enabled.<br/>        """
        pass

    def get_host_reservation_purchase_preview(self, OfferingId: str, HostIdSet: List[String]) -> GetHostReservationPurchasePreviewResult:
        """Preview a reservation purchase with configurations that match those of your Dedicated Host. You must have active Dedicated Hosts in your account before you purchase a reservation. This is a preview of the <a>PurchaseHostReservation</a> action and does not result in the offering being purchased.

        :param self:
        :param OfferingId:  The offering ID of the reservation.<br/>HostIdSet:  The IDs of the Dedicated Hosts with which the reservation is associated.<br/>
        :return: TotalUpfrontPrice:  The potential total upfront price. This is billed immediately.<br/>TotalHourlyPrice:  The potential total hourly price of the reservation per hour.<br/>Purchase:  The purchase information of the Dedicated Host reservation and the Dedicated Hosts associated with it.<br/>CurrencyCode:  The currency in which the <code>totalUpfrontPrice</code> and <code>totalHourlyPrice</code> amounts are specified. At this time, the only supported currency is <code>USD</code>.<br/>        """
        pass

    def get_launch_template_data(self, InstanceId: str, DryRun: Boolean=None) -> GetLaunchTemplateDataResult:
        """Retrieves the configuration data of the specified instance. You can use this data to create a launch template.

        :param self:
        :param InstanceId:  The ID of the instance.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: LaunchTemplateData:  The instance data.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HibernationOptions</i> Indicates whether an instance is configured for hibernation. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Hibernate.html">Hibernate Your Instance</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>HibernationOptions</b>: Indicates whether an instance is configured for hibernation. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Hibernate.html">Hibernate Your Instance</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseSpecifications</i> The license configurations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CapacityReservationSpecification</i> Information about the Capacity Reservation targeting option.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CapacityReservationSpecification</b>: Information about the Capacity Reservation targeting option.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CpuOptions</i> The CPU options for the instance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-optimize-cpu.html">Optimizing CPU Options</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CpuOptions</b>: The CPU options for the instance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-optimize-cpu.html">Optimizing CPU Options</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreditSpecification</i> The credit option for CPU usage of the instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CreditSpecification</b>: The credit option for CPU usage of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceMarketOptions</i> The market (purchasing) option for the instances.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InstanceMarketOptions</b>: The market (purchasing) option for the instances.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroups</i> The security group names.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The security group IDs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticInferenceAccelerators</i>  The elastic inference accelerator for the instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticGpuSpecifications</i> The elastic GPU specification.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TagSpecifications</i> The tags.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserData</i> The user data for the instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceInitiatedShutdownBehavior</i> Indicates whether an instance stops or terminates when you initiate shutdown from the instance (using the operating system command for system shutdown).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisableApiTermination</i> If set to <code>true</code>, indicates that the instance cannot be terminated using the Amazon EC2 console, command line tool, or API.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RamDiskId</i> The ID of the RAM disk, if applicable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Placement</i> The placement of the instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Placement</b>: The placement of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Monitoring</i> The monitoring for the instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Monitoring</b>: The monitoring for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyName</i> The name of the key pair.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i> The instance type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImageId</i> The ID of the AMI that was used to launch the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkInterfaces</i> The network interfaces.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BlockDeviceMappings</i> The block device mappings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamInstanceProfile</i> The IAM instance profile.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>IamInstanceProfile</b>: The IAM instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EbsOptimized</i> Indicates whether the instance is optimized for Amazon EBS I/O. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KernelId</i> The ID of the kernel, if applicable.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Indicates whether detailed monitoring is enabled. Otherwise, basic monitoring is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SpreadDomain</i> Reserved for future use.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Tenancy</i> The tenancy of the instance (if the instance is running in a VPC). An instance with a tenancy of <code>dedicated</code> runs on single-tenant hardware. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostId</i> The ID of the Dedicated Host for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupName</i> The name of the placement group for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Affinity</i> The affinity setting for the instance on the Dedicated Host.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MarketType</i> The market type.<br/>The options for Spot Instances.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpotOptions</b>: The options for Spot Instances.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CpuCredits</i> The credit option for CPU usage of a T2 or T3 instance. Valid values are <code>standard</code> and <code>unlimited</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ThreadsPerCore</i> The number of threads per CPU core.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CoreCount</i> The number of CPU cores for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CapacityReservationPreference</i> Indicates the instance's Capacity Reservation preferences. Possible preferences include: <ul> <li>  <code>open</code> - The instance can run in any <code>open</code> Capacity Reservation that has matching attributes (instance type, platform, Availability Zone). </li> <li>  <code>none</code> - The instance avoids running in a Capacity Reservation even if one is available. The instance runs in On-Demand capacity. </li> </ul><br/>Information about the target Capacity Reservation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CapacityReservationTarget</b>: Information about the target Capacity Reservation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Configured</i> If this parameter is set to <code>true</code>, the instance is enabled for hibernation; otherwise, it is not enabled for hibernation.<br/>        """
        pass

    def get_password_data(self, InstanceId: str, DryRun: Boolean=None) -> GetPasswordDataResult:
        """Retrieves the encrypted administrator password for a running Windows instance. The Windows password is generated at boot by the <code>EC2Config</code> service or <code>EC2Launch</code> scripts (Windows Server 2016 and later). This usually only happens the first time an instance is launched. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/WindowsGuide/UsingConfig_WinAMI.html">EC2Config</a> and <a href="https://docs.aws.amazon.com/AWSEC2/latest/WindowsGuide/ec2launch.html">EC2Launch</a> in the Amazon Elastic Compute Cloud User Guide. For the <code>EC2Config</code> service, the password is not generated for rebundled AMIs unless <code>Ec2SetPassword</code> is enabled before bundling. The password is encrypted using the key pair that you specified when you launched the instance. You must provide the corresponding key pair file. When you launch an instance, password generation and encryption may take a few minutes. If you try to retrieve the password before it's available, the output returns an empty string. We recommend that you wait up to 15 minutes after launching an instance before trying to retrieve the generated password.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>InstanceId:  The ID of the Windows instance.<br/>
        :return: Timestamp:  The time the data was last updated.<br/>PasswordData:  The password of the instance. Returns an empty string if the password is not available.<br/>InstanceId:  The ID of the Windows instance.<br/>        """
        pass

    def get_reserved_instances_exchange_quote(self, ReservedInstanceIds: List[String], DryRun: Boolean=None, TargetConfigurations: List[TargetConfigurationRequest]=None) -> GetReservedInstancesExchangeQuoteResult:
        """Returns a quote and exchange information for exchanging one or more specified Convertible Reserved Instances for a new Convertible Reserved Instance. If the exchange cannot be performed, the reason is returned in the response. Use <a>AcceptReservedInstancesExchangeQuote</a> to perform the exchange.

        :param self:
        :param TargetConfigurations:  The configuration of the target Convertible Reserved Instance to exchange for your current Convertible Reserved Instances.<br/>ReservedInstanceIds:  The IDs of the Convertible Reserved Instances to exchange.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: ValidationFailureReason:  Describes the reason why the exchange cannot be completed.<br/>TargetConfigurationValueSet:  The values of the target Convertible Reserved Instances.<br/>TargetConfigurationValueRollup:  The cost associated with the Reserved Instance.:<br/>ReservedInstanceValueSet:  The configuration of your Convertible Reserved Instances.<br/>ReservedInstanceValueRollup:  The cost associated with the Reserved Instance.:<br/>PaymentDue:  The total true upfront charge for the exchange.<br/>OutputReservedInstancesWillExpireAt:  The new end date of the reservation term.<br/>IsValidExchange:  If <code>true</code>, the exchange is valid. If <code>false</code>, the exchange cannot be completed.<br/>CurrencyCode:  The currency of the transaction.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RemainingUpfrontValue</i> The remaining upfront cost of the reservation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RemainingTotalValue</i> The balance of the total value (the sum of remainingUpfrontValue + hourlyPrice * number of hours remaining).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HourlyPrice</i> The hourly rate of the reservation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RemainingUpfrontValue</i> The remaining upfront cost of the reservation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RemainingTotalValue</i> The balance of the total value (the sum of remainingUpfrontValue + hourlyPrice * number of hours remaining).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HourlyPrice</i> The hourly rate of the reservation.<br/>        """
        pass

    def get_transit_gateway_attachment_propagations(self, TransitGatewayAttachmentId: str, Filters: List[Filter]=None, MaxResults: int=None, NextToken: str=None, DryRun: Boolean=None) -> GetTransitGatewayAttachmentPropagationsResult:
        """Lists the route tables to which the specified resource attachment propagates routes.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>NextToken:  The token for the next page of results.<br/>MaxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>Filters:  One or more filters. The possible values are: <ul> <li>  <code>transit-gateway-route-table-id</code> - The ID of the transit gateway route table. </li> </ul><br/>TransitGatewayAttachmentId:  The ID of the attachment.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>TransitGatewayAttachmentPropagations:  Information about the propagation route tables.<br/>        """
        pass

    def get_transit_gateway_route_table_associations(self, TransitGatewayRouteTableId: str, Filters: List[Filter]=None, MaxResults: int=None, NextToken: str=None, DryRun: Boolean=None) -> GetTransitGatewayRouteTableAssociationsResult:
        """Gets information about the associations for the specified transit gateway route table.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>NextToken:  The token for the next page of results.<br/>MaxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>Filters:  One or more filters. The possible values are: <ul> <li>  <code>resource-id</code> - The ID of the resource. </li> <li>  <code>resource-type</code> - The resource type (<code>vpc</code> | <code>vpn</code>). </li> <li>  <code>transit-gateway-attachment-id</code> - The ID of the attachment. </li> </ul><br/>TransitGatewayRouteTableId:  The ID of the transit gateway route table.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>Associations:  Information about the associations.<br/>        """
        pass

    def get_transit_gateway_route_table_propagations(self, TransitGatewayRouteTableId: str, Filters: List[Filter]=None, MaxResults: int=None, NextToken: str=None, DryRun: Boolean=None) -> GetTransitGatewayRouteTablePropagationsResult:
        """Gets information about the route table propagations for the specified transit gateway route table.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>NextToken:  The token for the next page of results.<br/>MaxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>Filters:  One or more filters. The possible values are: <ul> <li>  <code>resource-id</code> - The ID of the resource. </li> <li>  <code>resource-type</code> - The resource type (<code>vpc</code> | <code>vpn</code>). </li> <li>  <code>transit-gateway-attachment-id</code> - The ID of the attachment. </li> </ul><br/>TransitGatewayRouteTableId:  The ID of the transit gateway route table.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>TransitGatewayRouteTablePropagations:  Information about the route table propagations.<br/>        """
        pass

    def import_client_vpn_client_certificate_revocation_list(self, CertificateRevocationList: str, ClientVpnEndpointId: str, DryRun: Boolean=None) -> ImportClientVpnClientCertificateRevocationListResult:
        """Uploads a client certificate revocation list to the specified Client VPN endpoint. Uploading a client certificate revocation list overwrites the existing client certificate revocation list. Uploading a client certificate revocation list resets existing client connections.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>CertificateRevocationList:  The client certificate revocation list file. For more information, see <a href="https://docs.aws.amazon.com/vpn/latest/clientvpn-admin/cvpn-working-certificates.html#cvpn-working-certificates-generate">Generate a Client Certificate Revocation List</a> in the <i>AWS Client VPN Administrator Guide</i>.<br/>ClientVpnEndpointId:  The ID of the Client VPN endpoint to which the client certificate revocation list applies.<br/>
        :return: Return:  Returns <code>true</code> if the request succeeds; otherwise, it returns an error.<br/>        """
        pass

    def import_image(self, Architecture: str=None, ClientData: ClientData=None, ClientToken: str=None, Description: str=None, DiskContainers: List[ImageDiskContainer]=None, DryRun: Boolean=None, Encrypted: Boolean=None, Hypervisor: str=None, KmsKeyId: str=None, LicenseType: str=None, Platform: str=None, RoleName: str=None) -> ImportImageResult:
        """Import single or multi-volume disk images or EBS snapshots into an Amazon Machine Image (AMI). For more information, see <a href="https://docs.aws.amazon.com/vm-import/latest/userguide/vmimport-image-import.html">Importing a VM as an Image Using VM Import/Export</a> in the <i>VM Import/Export User Guide</i>.

        :param self:
        :param RoleName:  The name of the role to use when not using the default role, 'vmimport'.<br/>Platform:  The operating system of the virtual machine. Valid values: <code>Windows</code> | <code>Linux</code> <br/>LicenseType:  The license type to be used for the Amazon Machine Image (AMI) after importing. By default, we detect the source-system operating system (OS) and apply the appropriate license. Specify <code>AWS</code> to replace the source-system license with an AWS license, if appropriate. Specify <code>BYOL</code> to retain the source-system license, if appropriate. To use <code>BYOL</code>, you must have existing licenses with rights to use these licenses in a third party cloud, such as AWS. For more information, see <a href="https://docs.aws.amazon.com/vm-import/latest/userguide/vmimport-image-import.html#prerequisites-image">Prerequisites</a> in the VM Import/Export User Guide.<br/>KmsKeyId:  An identifier for the AWS Key Management Service (AWS KMS) customer master key (CMK) to use when creating the encrypted AMI. This parameter is only required if you want to use a non-default CMK; if this parameter is not specified, the default CMK for EBS is used. If a <code>KmsKeyId</code> is specified, the <code>Encrypted</code> flag must also be set.  The CMK identifier may be provided in any of the following formats:  <ul> <li> Key ID </li> <li> Key alias. The alias ARN contains the <code>arn:aws:kms</code> namespace, followed by the Region of the CMK, the AWS account ID of the CMK owner, the <code>alias</code> namespace, and then the CMK alias. For example, arn:aws:kms:<i>us-east-1</i>:<i>012345678910</i>:alias/<i>ExampleAlias</i>. </li> <li> ARN using key ID. The ID ARN contains the <code>arn:aws:kms</code> namespace, followed by the Region of the CMK, the AWS account ID of the CMK owner, the <code>key</code> namespace, and then the CMK ID. For example, arn:aws:kms:<i>us-east-1</i>:<i>012345678910</i>:key/<i>abcd1234-a123-456a-a12b-a123b4cd56ef</i>. </li> <li> ARN using key alias. The alias ARN contains the <code>arn:aws:kms</code> namespace, followed by the Region of the CMK, the AWS account ID of the CMK owner, the <code>alias</code> namespace, and then the CMK alias. For example, arn:aws:kms:<i>us-east-1</i>:<i>012345678910</i>:alias/<i>ExampleAlias</i>.  </li> </ul> AWS parses <code>KmsKeyId</code> asynchronously, meaning that the action you call may appear to complete even though you provided an invalid identifier. This action will eventually report failure.  The specified CMK must exist in the Region that the AMI is being copied to.<br/>Hypervisor:  The target hypervisor platform. Valid values: <code>xen</code> <br/>Encrypted:  Specifies whether the destination AMI of the imported image should be encrypted. The default CMK for EBS is used unless you specify a non-default AWS Key Management Service (AWS KMS) CMK using <code>KmsKeyId</code>. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html">Amazon EBS Encryption</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>DiskContainers:  Information about the disk containers.<br/>Description:  A description string for the import image task.<br/>ClientToken:  The token to enable idempotency for VM import requests.<br/>ClientData:  The client-specific data.:<br/>Architecture:  The architecture of the virtual machine. Valid values: <code>i386</code> | <code>x86_64</code> | <code>arm64</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UploadStart</i> The time that the disk upload starts.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UploadSize</i> The size of the uploaded disk image, in GiB.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UploadEnd</i> The time that the disk upload ends.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> A user-defined comment about the disk upload.<br/>
        :return: StatusMessage:  A detailed status message of the import task.<br/>Status:  A brief status of the task.<br/>SnapshotDetails:  Information about the snapshots.<br/>Progress:  The progress of the task.<br/>Platform:  The operating system of the virtual machine.<br/>LicenseType:  The license type of the virtual machine.<br/>KmsKeyId:  The identifier for the AWS Key Management Service (AWS KMS) customer master key (CMK) that was used to create the encrypted AMI.<br/>ImportTaskId:  The task ID of the import image task.<br/>ImageId:  The ID of the Amazon Machine Image (AMI) created by the import task.<br/>Hypervisor:  The target hypervisor of the import task.<br/>Encrypted:  Indicates whether the AMI is encypted.<br/>Description:  A description of the import task.<br/>Architecture:  The architecture of the virtual machine.<br/>        """
        pass

    def import_instance(self, Platform: str, Description: str=None, DiskImages: List[DiskImage]=None, DryRun: Boolean=None, LaunchSpecification: ImportInstanceLaunchSpecification=None) -> ImportInstanceResult:
        """Creates an import instance task using metadata from the specified disk image. <code>ImportInstance</code> only supports single-volume VMs. To import multi-volume VMs, use <a>ImportImage</a>. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/CommandLineReference/ec2-cli-vmimport-export.html">Importing a Virtual Machine Using the Amazon EC2 CLI</a>. For information about the import manifest referenced by this API action, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/manifest.html">VM Import Manifest</a>.

        :param self:
        :param Platform:  The instance operating system.<br/>LaunchSpecification:  The launch specification.:<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>DiskImages:  The disk image.<br/>Description:  A description for the instance being imported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserData</i> The Base64-encoded user data to make available to the instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>UserData</b>: The Base64-encoded user data to make available to the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetId</i> [EC2-VPC] The ID of the subnet in which to launch the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrivateIpAddress</i> [EC2-VPC] An available IP address from the IP address range of the subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Placement</i> The placement information for the instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Placement</b>: The placement information for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Monitoring</i> Indicates whether monitoring is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i> The instance type. For more information about the instance types that you can import, see <a href="https://docs.aws.amazon.com/vm-import/latest/userguide/vmie_prereqs.html#vmimport-instance-types">Instance Types</a> in the VM Import/Export User Guide.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceInitiatedShutdownBehavior</i> Indicates whether an instance stops or terminates when you initiate shutdown from the instance (using the operating system command for system shutdown).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupNames</i> The security group names.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupIds</i> The security group IDs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Architecture</i> The architecture of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdditionalInfo</i> Reserved.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SpreadDomain</i> Reserved for future use.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Tenancy</i> The tenancy of the instance (if the instance is running in a VPC). An instance with a tenancy of <code>dedicated</code> runs on single-tenant hardware. The <code>host</code> tenancy is not supported for the <a>ImportInstance</a> command.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostId</i> The ID of the Dedicated Host on which the instance resides. This parameter is not supported for the <a>ImportInstance</a> command.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PartitionNumber</i> The number of the partition the instance is in. Valid only if the placement group strategy is set to <code>partition</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupName</i> The name of the placement group the instance is in.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Affinity</i> The affinity setting for the instance on the Dedicated Host. This parameter is not supported for the <a>ImportInstance</a> command.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone of the instance. If not specified, an Availability Zone will be automatically chosen for you based on the load balancing criteria for the Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Data</i> The user data. If you are using an AWS SDK or command line tool, Base64-encoding is performed for you, and you can load the text from a file. Otherwise, you must provide Base64-encoded text.<br/>
        :return: ConversionTask:  Information about the conversion task.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> Any tags assigned to the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> The status message related to the conversion task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the conversion task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImportVolume</i> If the task is for importing a volume, this contains information about the import volume task.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ImportVolume</b>: If the task is for importing a volume, this contains information about the import volume task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImportInstance</i> If the task is for importing an instance, this contains information about the import instance task.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ImportInstance</b>: If the task is for importing an instance, this contains information about the import instance task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpirationTime</i> The time when the task expires. If the upload isn't complete before the expiration time, we automatically cancel the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConversionTaskId</i> The ID of the conversion task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Volumes</i> The volumes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The instance operating system.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceId</i> The ID of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description you provided when starting the import volume task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BytesConverted</i> The number of bytes converted so far.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone where the resulting volume will reside.<br/>The image.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Image</b>: The image.        """
        pass

    def import_key_pair(self, PublicKeyMaterial: Blob, KeyName: str, DryRun: Boolean=None) -> ImportKeyPairResult:
        """Imports the public key from an RSA key pair that you created with a third-party tool. Compare this with <a>CreateKeyPair</a>, in which AWS creates the key pair and gives the keys to you (AWS keeps a copy of the public key). With ImportKeyPair, you create the key pair and give AWS just the public key. The private key is never transferred between you and AWS. For more information about key pairs, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-key-pairs.html">Key Pairs</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param PublicKeyMaterial:  The public key. For API calls, the text must be base64-encoded. For command line tools, base64 encoding is performed for you.<br/>KeyName:  A unique name for the key pair.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: KeyName:  The key pair name you provided.<br/>KeyFingerprint:  The MD5 public key fingerprint as specified in section 4 of RFC 4716.<br/>        """
        pass

    def import_snapshot(self, ClientData: ClientData=None, ClientToken: str=None, Description: str=None, DiskContainer: SnapshotDiskContainer=None, DryRun: Boolean=None, Encrypted: Boolean=None, KmsKeyId: str=None, RoleName: str=None) -> ImportSnapshotResult:
        """Imports a disk into an EBS snapshot.

        :param self:
        :param RoleName:  The name of the role to use when not using the default role, 'vmimport'.<br/>KmsKeyId:  An identifier for the AWS Key Management Service (AWS KMS) customer master key (CMK) to use when creating the encrypted snapshot. This parameter is only required if you want to use a non-default CMK; if this parameter is not specified, the default CMK for EBS is used. If a <code>KmsKeyId</code> is specified, the <code>Encrypted</code> flag must also be set.  The CMK identifier may be provided in any of the following formats:  <ul> <li> Key ID </li> <li> Key alias. The alias ARN contains the <code>arn:aws:kms</code> namespace, followed by the Region of the CMK, the AWS account ID of the CMK owner, the <code>alias</code> namespace, and then the CMK alias. For example, arn:aws:kms:<i>us-east-1</i>:<i>012345678910</i>:alias/<i>ExampleAlias</i>. </li> <li> ARN using key ID. The ID ARN contains the <code>arn:aws:kms</code> namespace, followed by the Region of the CMK, the AWS account ID of the CMK owner, the <code>key</code> namespace, and then the CMK ID. For example, arn:aws:kms:<i>us-east-1</i>:<i>012345678910</i>:key/<i>abcd1234-a123-456a-a12b-a123b4cd56ef</i>. </li> <li> ARN using key alias. The alias ARN contains the <code>arn:aws:kms</code> namespace, followed by the Region of the CMK, the AWS account ID of the CMK owner, the <code>alias</code> namespace, and then the CMK alias. For example, arn:aws:kms:<i>us-east-1</i>:<i>012345678910</i>:alias/<i>ExampleAlias</i>.  </li> </ul> AWS parses <code>KmsKeyId</code> asynchronously, meaning that the action you call may appear to complete even though you provided an invalid identifier. This action will eventually report failure.  The specified CMK must exist in the Region that the snapshot is being copied to.<br/>Encrypted:  Specifies whether the destination snapshot of the imported image should be encrypted. The default CMK for EBS is used unless you specify a non-default AWS Key Management Service (AWS KMS) CMK using <code>KmsKeyId</code>. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html">Amazon EBS Encryption</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>DiskContainer:  Information about the disk container.:<br/>Description:  The description string for the import snapshot task.<br/>ClientToken:  Token to enable idempotency for VM import requests.<br/>ClientData:  The client-specific data.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UploadStart</i> The time that the disk upload starts.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UploadSize</i> The size of the uploaded disk image, in GiB.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UploadEnd</i> The time that the disk upload ends.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> A user-defined comment about the disk upload.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserBucket</i> The S3 bucket for the disk image.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>UserBucket</b>: The S3 bucket for the disk image.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Url</i> The URL to the Amazon S3-based disk image being imported. It can either be a https URL (https://..) or an Amazon S3 URL (s3://..).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> The format of the disk image being imported. Valid values: <code>VHD</code> | <code>VMDK</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the disk image being imported.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Key</i> The file name of the disk image.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Bucket</i> The name of the S3 bucket where the disk image is located.<br/>
        :return: SnapshotTaskDetail:  Information about the import snapshot task.:<br/>ImportTaskId:  The ID of the import snapshot task.<br/>Description:  A description of the import snapshot task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserBucket</i> The S3 bucket for the disk image.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>UserBucket</b>: The S3 bucket for the disk image.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Url</i> The URL of the disk image from which the snapshot is created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> A detailed status message for the import snapshot task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> A brief status for the import snapshot task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotId</i> The snapshot ID of the disk being imported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Progress</i> The percentage of completion for the import snapshot task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The identifier for the AWS Key Management Service (AWS KMS) customer master key (CMK) that was used to create the encrypted snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> The format of the disk image from which the snapshot is created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encrypted</i> Indicates whether the snapshot is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DiskImageSize</i> The size of the disk in the snapshot, in GiB.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Key</i> The file name of the disk image.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Bucket</i> The S3 bucket from which the disk image was created.<br/>        """
        pass

    def import_volume(self, Volume: VolumeDetail, Image: DiskImageDetail, AvailabilityZone: str, Description: str=None, DryRun: Boolean=None) -> ImportVolumeResult:
        """Creates an import volume task using metadata from the specified disk image.For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/CommandLineReference/importing-your-volumes-into-amazon-ebs.html">Importing Disks to Amazon EBS</a>. For information about the import manifest referenced by this API action, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/manifest.html">VM Import Manifest</a>.

        :param self:
        :param Volume:  The volume size.:<br/>Image:  The disk image.:<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Description:  A description of the volume.<br/>AvailabilityZone:  The Availability Zone for the resulting EBS volume.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImportManifestUrl</i> A presigned URL for the import manifest stored in Amazon S3 and presented here as an Amazon S3 presigned URL. For information about creating a presigned URL for an Amazon S3 object, read the "Query String Request Authentication Alternative" section of the <a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/RESTAuthentication.html">Authenticating REST Requests</a> topic in the <i>Amazon Simple Storage Service Developer Guide</i>. For information about the import manifest referenced by this API action, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/manifest.html">VM Import Manifest</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> The disk image format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Bytes</i> The size of the disk image, in GiB.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Size</i> The size of the volume, in GiB.<br/>
        :return: ConversionTask:  Information about the conversion task.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> Any tags assigned to the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> The status message related to the conversion task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the conversion task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImportVolume</i> If the task is for importing a volume, this contains information about the import volume task.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ImportVolume</b>: If the task is for importing a volume, this contains information about the import volume task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImportInstance</i> If the task is for importing an instance, this contains information about the import instance task.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ImportInstance</b>: If the task is for importing an instance, this contains information about the import instance task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpirationTime</i> The time when the task expires. If the upload isn't complete before the expiration time, we automatically cancel the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConversionTaskId</i> The ID of the conversion task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Volumes</i> The volumes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The instance operating system.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceId</i> The ID of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description you provided when starting the import volume task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BytesConverted</i> The number of bytes converted so far.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone where the resulting volume will reside.<br/>The image.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Image</b>: The image.        """
        pass

    def modify_capacity_reservation(self, CapacityReservationId: str, InstanceCount: int=None, EndDate: DateTime=None, EndDateType: str=None, DryRun: Boolean=None) -> ModifyCapacityReservationResult:
        """Modifies a Capacity Reservation's capacity and the conditions under which it is to be released. You cannot change a Capacity Reservation's instance type, EBS optimization, instance store settings, platform, Availability Zone, or instance eligibility. If you need to modify any of these attributes, we recommend that you cancel the Capacity Reservation, and then create a new one with the required attributes.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>EndDateType:  Indicates the way in which the Capacity Reservation ends. A Capacity Reservation can have one of the following end types: <ul> <li>  <code>unlimited</code> - The Capacity Reservation remains active until you explicitly cancel it. Do not provide an <code>EndDate</code> value if <code>EndDateType</code> is <code>unlimited</code>. </li> <li>  <code>limited</code> - The Capacity Reservation expires automatically at a specified date and time. You must provide an <code>EndDate</code> value if <code>EndDateType</code> is <code>limited</code>. </li> </ul><br/>EndDate:  The date and time at which the Capacity Reservation expires. When a Capacity Reservation expires, the reserved capacity is released and you can no longer launch instances into it. The Capacity Reservation's state changes to <code>expired</code> when it reaches its end date and time. The Capacity Reservation is cancelled within an hour from the specified time. For example, if you specify 5/31/2019, 13:30:55, the Capacity Reservation is guaranteed to end between 13:30:55 and 14:30:55 on 5/31/2019. You must provide an <code>EndDate</code> value if <code>EndDateType</code> is <code>limited</code>. Omit <code>EndDate</code> if <code>EndDateType</code> is <code>unlimited</code>.<br/>InstanceCount:  The number of instances for which to reserve capacity.<br/>CapacityReservationId:  The ID of the Capacity Reservation.<br/>
        :return: Return:  Information about the Capacity Reservation.<br/>        """
        pass

    def modify_client_vpn_endpoint(self, ClientVpnEndpointId: str, ServerCertificateArn: str=None, ConnectionLogOptions: ConnectionLogOptions=None, DnsServers: DnsServersOptionsModifyStructure=None, Description: str=None, DryRun: Boolean=None) -> ModifyClientVpnEndpointResult:
        """Modifies the specified Client VPN endpoint. You can only modify an endpoint's server certificate information, client connection logging information, DNS server, and description. Modifying the DNS server resets existing client connections.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Description:  A brief description of the Client VPN endpoint.<br/>DnsServers:  Information about the DNS servers to be used by Client VPN connections. A Client VPN endpoint can have up to two DNS servers.:<br/>ConnectionLogOptions:  Information about the client connection logging options. If you enable client connection logging, data about client connections is sent to a Cloudwatch Logs log stream. The following information is logged: <ul> <li> Client connection requests </li> <li> Client connection results (successful and unsuccessful) </li> <li> Reasons for unsuccessful client connection requests </li> <li> Client connection termination time </li> </ul>:<br/>ServerCertificateArn:  The ARN of the server certificate to be used. The server certificate must be provisioned in AWS Certificate Manager (ACM).<br/>ClientVpnEndpointId:  The ID of the Client VPN endpoint to modify.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudwatchLogStream</i> The name of the CloudWatch Logs log stream to which the connection data is published.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudwatchLogGroup</i> The name of the CloudWatch Logs log group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Indicates whether connection logging is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Indicates whether DNS servers should be used. Specify <code>False</code> to delete the existing DNS servers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomDnsServers</i> The IPv4 address range, in CIDR notation, of the DNS servers to be used. You can specify up to two DNS servers. Ensure that the DNS servers can be reached by the clients. The specified values overwrite the existing values.<br/>
        :return: Return:  Returns <code>true</code> if the request succeeds; otherwise, it returns an error.<br/>        """
        pass

    def modify_ebs_default_kms_key_id(self, KmsKeyId: str, DryRun: Boolean=None) -> ModifyEbsDefaultKmsKeyIdResult:
        """Changes the default customer master key (CMK) for EBS encryption by default for your account in this Region. AWS creates a unique AWS managed CMK in each Region for use with encryption by default. If you change the default CMK to a customer managed CMK, it is used instead of the AWS managed CMK. To reset the default CMK to the AWS managed CMK for EBS, use <a>ResetEbsDefaultKmsKeyId</a>. If you delete or disable the customer managed CMK that you specified for use with encryption by default, your instances will fail to launch. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html">Amazon EBS Encryption</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>KmsKeyId:  The identifier of the AWS Key Management Service (AWS KMS) customer master key (CMK) to use for Amazon EBS encryption. If this parameter is not specified, your AWS managed CMK for EBS is used. If <code>KmsKeyId</code> is specified, the encrypted state must be <code>true</code>. You can specify the CMK using any of the following: <ul> <li> Key ID. For example, key/1234abcd-12ab-34cd-56ef-1234567890ab. </li> <li> Key alias. For example, alias/ExampleAlias. </li> <li> Key ARN. For example, arn:aws:kms:<i>us-east-1</i>:<i>012345678910</i>:key/<i>abcd1234-a123-456a-a12b-a123b4cd56ef</i>. </li> <li> Alias ARN. For example, arn:aws:kms:<i>us-east-1</i>:<i>012345678910</i>:alias/<i>ExampleAlias</i>. </li> </ul> AWS authenticates the CMK asynchronously. Therefore, if you specify an ID, alias, or ARN that is not valid, the action can appear to complete, but eventually fails.<br/>
        :return: KmsKeyId:  The Amazon Resource Name (ARN) of the default CMK for encryption by default.<br/>        """
        pass

    def modify_fleet(self, TargetCapacitySpecification: TargetCapacitySpecificationRequest, FleetId: str, DryRun: Boolean=None, ExcessCapacityTerminationPolicy: str=None) -> ModifyFleetResult:
        """Modifies the specified EC2 Fleet. While the EC2 Fleet is being modified, it is in the <code>modifying</code> state.

        :param self:
        :param TargetCapacitySpecification:  The size of the EC2 Fleet.:<br/>FleetId:  The ID of the EC2 Fleet.<br/>ExcessCapacityTerminationPolicy:  Indicates whether running instances should be terminated if the total target capacity of the EC2 Fleet is decreased below the current size of the EC2 Fleet.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultTargetCapacityType</i> The default <code>TotalTargetCapacity</code>, which is either <code>Spot</code> or <code>On-Demand</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SpotTargetCapacity</i> The number of Spot units to request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OnDemandTargetCapacity</i> The number of On-Demand units to request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalTargetCapacity</i> The number of units to request, filled using <code>DefaultTargetCapacityType</code>.<br/>
        :return: Return:  Is <code>true</code> if the request succeeds, and an error otherwise.<br/>        """
        pass

    def modify_fpga_image_attribute(self, FpgaImageId: str, DryRun: Boolean=None, Attribute: str=None, OperationType: str=None, UserIds: List[String]=None, UserGroups: List[String]=None, ProductCodes: List[String]=None, LoadPermission: LoadPermissionModifications=None, Description: str=None, Name: str=None) -> ModifyFpgaImageAttributeResult:
        """Modifies the specified attribute of the specified Amazon FPGA Image (AFI).

        :param self:
        :param Name:  A name for the AFI.<br/>Description:  A description for the AFI.<br/>LoadPermission:  The load permission for the AFI.:<br/>ProductCodes:  The product codes. After you add a product code to an AFI, it can't be removed. This parameter is valid only when modifying the <code>productCodes</code> attribute.<br/>UserGroups:  The user groups. This parameter is valid only when modifying the <code>loadPermission</code> attribute.<br/>UserIds:  The AWS account IDs. This parameter is valid only when modifying the <code>loadPermission</code> attribute.<br/>OperationType:  The operation type.<br/>Attribute:  The name of the attribute.<br/>FpgaImageId:  The ID of the AFI.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Remove</i> The load permissions to remove.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Add</i> The load permissions to add.<br/>
        :return: FpgaImageAttribute:  Information about the attribute.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductCodes</i> The product codes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LoadPermissions</i> The load permissions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the AFI.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the AFI.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FpgaImageId</i> The ID of the AFI.<br/>        """
        pass

    def modify_hosts(self, HostIds: List[String], AutoPlacement: str=None, HostRecovery: str=None) -> ModifyHostsResult:
        """Modify the auto-placement setting of a Dedicated Host. When auto-placement is enabled, any instances that you launch with a tenancy of <code>host</code> but without a specific host ID are placed onto any available Dedicated Host in your account that has auto-placement enabled. When auto-placement is disabled, you need to provide a host ID to have the instance launch onto a specific host. If no host ID is provided, the instance is launched onto a suitable host with auto-placement enabled.

        :param self:
        :param HostRecovery:  Indicates whether to enable or disable host recovery for the Dedicated Host. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/dedicated-hosts-recovery.html"> Host Recovery</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.<br/>HostIds:  The IDs of the Dedicated Hosts to modify.<br/>AutoPlacement:  Specify whether to enable or disable auto-placement.<br/>
        :return: Unsuccessful:  The IDs of the Dedicated Hosts that could not be modified. Check whether the setting you requested can be used.<br/>Successful:  The IDs of the Dedicated Hosts that were successfully modified.<br/>        """
        pass

    def modify_id_format(self, UseLongIds: Boolean, Resource: str):
        """Modifies the ID format for the specified resource on a per-Region basis. You can specify that resources should receive longer IDs (17-character IDs) when they are created. This request can only be used to modify longer ID settings for resource types that are within the opt-in period. Resources currently in their opt-in period include: <code>bundle</code> | <code>conversion-task</code> | <code>customer-gateway</code> | <code>dhcp-options</code> | <code>elastic-ip-allocation</code> | <code>elastic-ip-association</code> | <code>export-task</code> | <code>flow-log</code> | <code>image</code> | <code>import-task</code> | <code>internet-gateway</code> | <code>network-acl</code> | <code>network-acl-association</code> | <code>network-interface</code> | <code>network-interface-attachment</code> | <code>prefix-list</code> | <code>route-table</code> | <code>route-table-association</code> | <code>security-group</code> | <code>subnet</code> | <code>subnet-cidr-block-association</code> | <code>vpc</code> | <code>vpc-cidr-block-association</code> | <code>vpc-endpoint</code> | <code>vpc-peering-connection</code> | <code>vpn-connection</code> | <code>vpn-gateway</code>. This setting applies to the IAM user who makes the request; it does not apply to the entire AWS account. By default, an IAM user defaults to the same settings as the root user. If you're using this action as the root user, then these settings apply to the entire account, unless an IAM user explicitly overrides these settings for themselves. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/resource-ids.html">Resource IDs</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. Resources created with longer IDs are visible to all IAM roles and users, regardless of these settings and provided that they have permission to use the relevant <code>Describe</code> command for the resource type.

        :param self:
        :param UseLongIds:  Indicate whether the resource should use longer IDs (17-character IDs).<br/>Resource:  The type of resource: <code>bundle</code> | <code>conversion-task</code> | <code>customer-gateway</code> | <code>dhcp-options</code> | <code>elastic-ip-allocation</code> | <code>elastic-ip-association</code> | <code>export-task</code> | <code>flow-log</code> | <code>image</code> | <code>import-task</code> | <code>internet-gateway</code> | <code>network-acl</code> | <code>network-acl-association</code> | <code>network-interface</code> | <code>network-interface-attachment</code> | <code>prefix-list</code> | <code>route-table</code> | <code>route-table-association</code> | <code>security-group</code> | <code>subnet</code> | <code>subnet-cidr-block-association</code> | <code>vpc</code> | <code>vpc-cidr-block-association</code> | <code>vpc-endpoint</code> | <code>vpc-peering-connection</code> | <code>vpn-connection</code> | <code>vpn-gateway</code>. Alternatively, use the <code>all-current</code> option to include all resource types that are currently within their opt-in period for longer IDs.<br/>
        :return:         """
        pass

    def modify_identity_id_format(self, UseLongIds: Boolean, Resource: str, PrincipalArn: str):
        """Modifies the ID format of a resource for a specified IAM user, IAM role, or the root user for an account; or all IAM users, IAM roles, and the root user for an account. You can specify that resources should receive longer IDs (17-character IDs) when they are created.  This request can only be used to modify longer ID settings for resource types that are within the opt-in period. Resources currently in their opt-in period include: <code>bundle</code> | <code>conversion-task</code> | <code>customer-gateway</code> | <code>dhcp-options</code> | <code>elastic-ip-allocation</code> | <code>elastic-ip-association</code> | <code>export-task</code> | <code>flow-log</code> | <code>image</code> | <code>import-task</code> | <code>internet-gateway</code> | <code>network-acl</code> | <code>network-acl-association</code> | <code>network-interface</code> | <code>network-interface-attachment</code> | <code>prefix-list</code> | <code>route-table</code> | <code>route-table-association</code> | <code>security-group</code> | <code>subnet</code> | <code>subnet-cidr-block-association</code> | <code>vpc</code> | <code>vpc-cidr-block-association</code> | <code>vpc-endpoint</code> | <code>vpc-peering-connection</code> | <code>vpn-connection</code> | <code>vpn-gateway</code>.  For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/resource-ids.html">Resource IDs</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.  This setting applies to the principal specified in the request; it does not apply to the principal that makes the request.  Resources created with longer IDs are visible to all IAM roles and users, regardless of these settings and provided that they have permission to use the relevant <code>Describe</code> command for the resource type.

        :param self:
        :param UseLongIds:  Indicates whether the resource should use longer IDs (17-character IDs)<br/>Resource:  The type of resource: <code>bundle</code> | <code>conversion-task</code> | <code>customer-gateway</code> | <code>dhcp-options</code> | <code>elastic-ip-allocation</code> | <code>elastic-ip-association</code> | <code>export-task</code> | <code>flow-log</code> | <code>image</code> | <code>import-task</code> | <code>internet-gateway</code> | <code>network-acl</code> | <code>network-acl-association</code> | <code>network-interface</code> | <code>network-interface-attachment</code> | <code>prefix-list</code> | <code>route-table</code> | <code>route-table-association</code> | <code>security-group</code> | <code>subnet</code> | <code>subnet-cidr-block-association</code> | <code>vpc</code> | <code>vpc-cidr-block-association</code> | <code>vpc-endpoint</code> | <code>vpc-peering-connection</code> | <code>vpn-connection</code> | <code>vpn-gateway</code>. Alternatively, use the <code>all-current</code> option to include all resource types that are currently within their opt-in period for longer IDs.<br/>PrincipalArn:  The ARN of the principal, which can be an IAM user, IAM role, or the root user. Specify <code>all</code> to modify the ID format for all IAM users, IAM roles, and the root user of the account.<br/>
        :return:         """
        pass

    def modify_image_attribute(self, ImageId: str, Attribute: str=None, Description: AttributeValue=None, LaunchPermission: LaunchPermissionModifications=None, OperationType: str=None, ProductCodes: List[String]=None, UserGroups: List[String]=None, UserIds: List[String]=None, Value: str=None, DryRun: Boolean=None):
        """Modifies the specified attribute of the specified AMI. You can specify only one attribute at a time. You can use the <code>Attribute</code> parameter to specify the attribute or one of the following parameters: <code>Description</code>, <code>LaunchPermission</code>, or <code>ProductCode</code>. AWS Marketplace product codes cannot be modified. Images with an AWS Marketplace product code cannot be made public. To enable the SriovNetSupport enhanced networking attribute of an image, enable SriovNetSupport on an instance and create an AMI from the instance.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Value:  The value of the attribute being modified. This parameter can be used only when the <code>Attribute</code> parameter is <code>description</code> or <code>productCodes</code>.<br/>UserIds:  The AWS account IDs. This parameter can be used only when the <code>Attribute</code> parameter is <code>launchPermission</code>.<br/>UserGroups:  The user groups. This parameter can be used only when the <code>Attribute</code> parameter is <code>launchPermission</code>.<br/>ProductCodes:  The DevPay product codes. After you add a product code to an AMI, it can't be removed.<br/>OperationType:  The operation type. This parameter can be used only when the <code>Attribute</code> parameter is <code>launchPermission</code>.<br/>LaunchPermission:  A new launch permission for the AMI.:<br/>ImageId:  The ID of the AMI.<br/>Description:  A new description for the AMI.:<br/>Attribute:  The name of the attribute to modify. The valid values are <code>description</code>, <code>launchPermission</code>, and <code>productCodes</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The value is case-sensitive.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Remove</i> The AWS account ID to remove from the list of launch permissions for the AMI.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Add</i> The AWS account ID to add to the list of launch permissions for the AMI.<br/>
        :return:         """
        pass

    def modify_instance_attribute(self, InstanceId: str, SourceDestCheck: AttributeBooleanValue=None, Attribute: str=None, BlockDeviceMappings: List[InstanceBlockDeviceMappingSpecification]=None, DisableApiTermination: AttributeBooleanValue=None, DryRun: Boolean=None, EbsOptimized: AttributeBooleanValue=None, EnaSupport: AttributeBooleanValue=None, Groups: List[String]=None, InstanceInitiatedShutdownBehavior: AttributeValue=None, InstanceType: AttributeValue=None, Kernel: AttributeValue=None, Ramdisk: AttributeValue=None, SriovNetSupport: AttributeValue=None, UserData: BlobAttributeValue=None, Value: str=None):
        """Modifies the specified attribute of the specified instance. You can specify only one attribute at a time.  <b>Note: </b>Using this action to change the security groups associated with an elastic network interface (ENI) attached to an instance in a VPC can result in an error if the instance has more than one ENI. To change the security groups associated with an ENI attached to an instance that has multiple ENIs, we recommend that you use the <a>ModifyNetworkInterfaceAttribute</a> action. To modify some attributes, the instance must be stopped. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Using_ChangingAttributesWhileInstanceStopped.html">Modifying Attributes of a Stopped Instance</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param Value:  A new value for the attribute. Use only with the <code>kernel</code>, <code>ramdisk</code>, <code>userData</code>, <code>disableApiTermination</code>, or <code>instanceInitiatedShutdownBehavior</code> attribute.<br/>UserData:  Changes the instance's user data to the specified value. If you are using an AWS SDK or command line tool, base64-encoding is performed for you, and you can load the text from a file. Otherwise, you must provide base64-encoded text.:<br/>SriovNetSupport:  Set to <code>simple</code> to enable enhanced networking with the Intel 82599 Virtual Function interface for the instance. There is no way to disable enhanced networking with the Intel 82599 Virtual Function interface at this time. This option is supported only for HVM instances. Specifying this option with a PV instance can make it unreachable.:<br/>Ramdisk:  Changes the instance's RAM disk to the specified value. We recommend that you use PV-GRUB instead of kernels and RAM disks. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/UserProvidedKernels.html">PV-GRUB</a>.:<br/>Kernel:  Changes the instance's kernel to the specified value. We recommend that you use PV-GRUB instead of kernels and RAM disks. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/UserProvidedKernels.html">PV-GRUB</a>.:<br/>InstanceType:  Changes the instance type to the specified value. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html">Instance Types</a>. If the instance type is not valid, the error returned is <code>InvalidInstanceAttributeValue</code>.:<br/>InstanceInitiatedShutdownBehavior:  Specifies whether an instance stops or terminates when you initiate shutdown from the instance (using the operating system command for system shutdown).:<br/>InstanceId:  The ID of the instance.<br/>Groups:  [EC2-VPC] Changes the security groups of the instance. You must specify at least one security group, even if it's just the default security group for the VPC. You must specify the security group ID, not the security group name.<br/>EnaSupport:  Set to <code>true</code> to enable enhanced networking with ENA for the instance. This option is supported only for HVM instances. Specifying this option with a PV instance can make it unreachable.:<br/>EbsOptimized:  Specifies whether the instance is optimized for Amazon EBS I/O. This optimization provides dedicated throughput to Amazon EBS and an optimized configuration stack to provide optimal EBS I/O performance. This optimization isn't available with all instance types. Additional usage charges apply when using an EBS Optimized instance.:<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>DisableApiTermination:  If the value is <code>true</code>, you can't terminate the instance using the Amazon EC2 console, CLI, or API; otherwise, you can. You cannot use this parameter for Spot Instances.:<br/>BlockDeviceMappings:  Modifies the <code>DeleteOnTermination</code> attribute for volumes that are currently attached. The volume must be owned by the caller. If no value is specified for <code>DeleteOnTermination</code>, the default is <code>true</code> and the volume is deleted when the instance is terminated. To add instance store volumes to an Amazon EBS-backed instance, you must add them when you launch the instance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/block-device-mapping-concepts.html#Using_OverridingAMIBDM">Updating the Block Device Mapping when Launching an Instance</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.<br/>Attribute:  The name of the attribute.<br/>SourceDestCheck:  Specifies whether source/destination checking is enabled. A value of <code>true</code> means that checking is enabled, and <code>false</code> means that checking is disabled. This value must be <code>false</code> for a NAT instance to perform NAT.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The valid values are <code>true</code> or <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The valid values are <code>true</code> or <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The valid values are <code>true</code> or <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The valid values are <code>true</code> or <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The value is case-sensitive.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The value is case-sensitive.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The value is case-sensitive.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The value is case-sensitive.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The value is case-sensitive.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> <br/>
        :return:         """
        pass

    def modify_instance_capacity_reservation_attributes(self, CapacityReservationSpecification: CapacityReservationSpecification, InstanceId: str, DryRun: Boolean=None) -> ModifyInstanceCapacityReservationAttributesResult:
        """Modifies the Capacity Reservation settings for a stopped instance. Use this action to configure an instance to target a specific Capacity Reservation, run in any <code>open</code> Capacity Reservation with matching attributes, or run On-Demand Instance capacity.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>CapacityReservationSpecification:  Information about the Capacity Reservation targeting option.:<br/>InstanceId:  The ID of the instance to be modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CapacityReservationTarget</i> Information about the target Capacity Reservation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CapacityReservationTarget</b>: Information about the target Capacity Reservation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CapacityReservationPreference</i> Indicates the instance's Capacity Reservation preferences. Possible preferences include: <ul> <li>  <code>open</code> - The instance can run in any <code>open</code> Capacity Reservation that has matching attributes (instance type, platform, Availability Zone). </li> <li>  <code>none</code> - The instance avoids running in a Capacity Reservation even if one is available. The instance runs as an On-Demand Instance. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CapacityReservationId</i> The ID of the Capacity Reservation.<br/>
        :return: Return:  Returns <code>true</code> if the request succeeds; otherwise, it returns an error.<br/>        """
        pass

    def modify_instance_credit_specification(self, InstanceCreditSpecifications: List[InstanceCreditSpecificationRequest], DryRun: Boolean=None, ClientToken: str=None) -> ModifyInstanceCreditSpecificationResult:
        """Modifies the credit option for CPU usage on a running or stopped T2 or T3 instance. The credit options are <code>standard</code> and <code>unlimited</code>. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/burstable-performance-instances.html">Burstable Performance Instances</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param InstanceCreditSpecifications:  Information about the credit option for CPU usage.<br/>ClientToken:  A unique, case-sensitive token that you provide to ensure idempotency of your modification request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">Ensuring Idempotency</a>.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: UnsuccessfulInstanceCreditSpecifications:  Information about the instances whose credit option for CPU usage was not modified.<br/>SuccessfulInstanceCreditSpecifications:  Information about the instances whose credit option for CPU usage was successfully modified.<br/>        """
        pass

    def modify_instance_event_start_time(self, NotBefore: DateTime, InstanceEventId: str, InstanceId: str, DryRun: Boolean=None) -> ModifyInstanceEventStartTimeResult:
        """Modifies the start time for a scheduled Amazon EC2 instance event.

        :param self:
        :param NotBefore:  The new date and time when the event will take place.<br/>InstanceEventId:  The ID of the event whose date and time you are modifying.<br/>InstanceId:  The ID of the instance with the scheduled event.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: Event:  Describes a scheduled event for an instance.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotBeforeDeadline</i> The deadline for starting the event.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotBefore</i> The earliest scheduled start time for the event.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotAfter</i> The latest scheduled end time for the event.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of the event. After a scheduled event is completed, it can still be described for up to a week. If the event has been completed, this description starts with the following text: [Completed].<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Code</i> The event code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceEventId</i> The ID of the event.<br/>        """
        pass

    def modify_instance_placement(self, InstanceId: str, Affinity: str=None, GroupName: str=None, HostId: str=None, Tenancy: str=None, PartitionNumber: int=None) -> ModifyInstancePlacementResult:
        """Modifies the placement attributes for a specified instance. You can do the following: <ul> <li> Modify the affinity between an instance and a <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/dedicated-hosts-overview.html">Dedicated Host</a>. When affinity is set to <code>host</code> and the instance is not associated with a specific Dedicated Host, the next time the instance is launched, it is automatically associated with the host on which it lands. If the instance is restarted or rebooted, this relationship persists. </li> <li> Change the Dedicated Host with which an instance is associated. </li> <li> Change the instance tenancy of an instance from <code>host</code> to <code>dedicated</code>, or from <code>dedicated</code> to <code>host</code>. </li> <li> Move an instance to or from a <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/placement-groups.html">placement group</a>. </li> </ul> At least one attribute for affinity, host ID, tenancy, or placement group name must be specified in the request. Affinity and tenancy can be modified in the same request. To modify the host ID, tenancy, placement group, or partition for an instance, the instance must be in the <code>stopped</code> state.

        :param self:
        :param PartitionNumber:  Reserved for future use.<br/>Tenancy:  The tenancy for the instance.<br/>InstanceId:  The ID of the instance that you are modifying.<br/>HostId:  The ID of the Dedicated Host with which to associate the instance.<br/>GroupName:  The name of the placement group in which to place the instance. For spread placement groups, the instance must have a tenancy of <code>default</code>. For cluster and partition placement groups, the instance must have a tenancy of <code>default</code> or <code>dedicated</code>. To remove an instance from a placement group, specify an empty string ("").<br/>Affinity:  The affinity setting for the instance.<br/>
        :return: Return:  Is <code>true</code> if the request succeeds, and an error otherwise.<br/>        """
        pass

    def modify_launch_template(self, DryRun: Boolean=None, ClientToken: str=None, LaunchTemplateId: str=None, LaunchTemplateName: str=None, DefaultVersion: str=None) -> ModifyLaunchTemplateResult:
        """Modifies a launch template. You can specify which version of the launch template to set as the default version. When launching an instance, the default version applies when a launch template version is not specified.

        :param self:
        :param DefaultVersion:  The version number of the launch template to set as the default version.<br/>LaunchTemplateName:  The name of the launch template. You must specify either the launch template ID or launch template name in the request.<br/>LaunchTemplateId:  The ID of the launch template. You must specify either the launch template ID or launch template name in the request.<br/>ClientToken:  Unique, case-sensitive identifier you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">Ensuring Idempotency</a>. Constraint: Maximum 128 ASCII characters.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: LaunchTemplate:  Information about the launch template.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The tags for the launch template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestVersionNumber</i> The version number of the latest version of the launch template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultVersionNumber</i> The version number of the default version of the launch template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedBy</i> The principal that created the launch template. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateTime</i> The time launch template was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LaunchTemplateName</i> The name of the launch template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LaunchTemplateId</i> The ID of the launch template.<br/>        """
        pass

    def modify_network_interface_attribute(self, NetworkInterfaceId: str, Attachment: NetworkInterfaceAttachmentChanges=None, Description: AttributeValue=None, DryRun: Boolean=None, Groups: List[String]=None, SourceDestCheck: AttributeBooleanValue=None):
        """Modifies the specified network interface attribute. You can specify only one attribute at a time. You can use this action to attach and detach security groups from an existing EC2 instance.

        :param self:
        :param SourceDestCheck:  Indicates whether source/destination checking is enabled. A value of <code>true</code> means checking is enabled, and <code>false</code> means checking is disabled. This value must be <code>false</code> for a NAT instance to perform NAT. For more information, see <a href="https://docs.aws.amazon.com/AmazonVPC/latest/UserGuide/VPC_NAT_Instance.html">NAT Instances</a> in the <i>Amazon Virtual Private Cloud User Guide</i>.:<br/>NetworkInterfaceId:  The ID of the network interface.<br/>Groups:  Changes the security groups for the network interface. The new set of groups you specify replaces the current set. You must specify at least one group, even if it's just the default security group in the VPC. You must specify the ID of the security group, not the name.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Description:  A description for the network interface.:<br/>Attachment:  Information about the interface attachment. If modifying the 'delete on termination' attribute, you must specify the ID of the interface attachment.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeleteOnTermination</i> Indicates whether the network interface is deleted when the instance is terminated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttachmentId</i> The ID of the network interface attachment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The value is case-sensitive.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The valid values are <code>true</code> or <code>false</code>.<br/>
        :return:         """
        pass

    def modify_reserved_instances(self, TargetConfigurations: List[ReservedInstancesConfiguration], ReservedInstancesIds: List[String], ClientToken: str=None) -> ModifyReservedInstancesResult:
        """Modifies the Availability Zone, instance count, instance type, or network platform (EC2-Classic or EC2-VPC) of your Reserved Instances. The Reserved Instances to be modified must be identical, except for Availability Zone, network platform, and instance type. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ri-modifying.html">Modifying Reserved Instances</a> in the Amazon Elastic Compute Cloud User Guide.

        :param self:
        :param TargetConfigurations:  The configuration settings for the Reserved Instances to modify.<br/>ClientToken:  A unique, case-sensitive token you provide to ensure idempotency of your modification request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">Ensuring Idempotency</a>.<br/>ReservedInstancesIds:  The IDs of the Reserved Instances to modify.<br/>
        :return: ReservedInstancesModificationId:  The ID for the modification.<br/>        """
        pass

    def modify_snapshot_attribute(self, SnapshotId: str, Attribute: str=None, CreateVolumePermission: CreateVolumePermissionModifications=None, GroupNames: List[String]=None, OperationType: str=None, UserIds: List[String]=None, DryRun: Boolean=None):
        """Adds or removes permission settings for the specified snapshot. You may add or remove specified AWS account IDs from a snapshot's list of create volume permissions, but you cannot do both in a single operation. If you need to both add and remove account IDs for a snapshot, you must use multiple operations. Encrypted snapshots and snapshots with AWS Marketplace product codes cannot be made public. Snapshots encrypted with your default CMK cannot be shared with other accounts. For more information about modifying snapshot permissions, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ebs-modifying-snapshot-permissions.html">Sharing Snapshots</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>UserIds:  The account ID to modify for the snapshot.<br/>SnapshotId:  The ID of the snapshot.<br/>OperationType:  The type of operation to perform to the attribute.<br/>GroupNames:  The group to modify for the snapshot.<br/>CreateVolumePermission:  A JSON representation of the snapshot attribute modification.:<br/>Attribute:  The snapshot attribute to modify. Only volume creation permissions can be modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Remove</i> Removes the specified AWS account ID or group from the list.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Add</i> Adds the specified AWS account ID or group to the list.<br/>
        :return:         """
        pass

    def modify_spot_fleet_request(self, SpotFleetRequestId: str, ExcessCapacityTerminationPolicy: str=None, TargetCapacity: int=None, OnDemandTargetCapacity: int=None) -> ModifySpotFleetRequestResponse:
        """Modifies the specified Spot Fleet request. You can only modify a Spot Fleet request of type <code>maintain</code>. While the Spot Fleet request is being modified, it is in the <code>modifying</code> state. To scale up your Spot Fleet, increase its target capacity. The Spot Fleet launches the additional Spot Instances according to the allocation strategy for the Spot Fleet request. If the allocation strategy is <code>lowestPrice</code>, the Spot Fleet launches instances using the Spot pool with the lowest price. If the allocation strategy is <code>diversified</code>, the Spot Fleet distributes the instances across the Spot pools. To scale down your Spot Fleet, decrease its target capacity. First, the Spot Fleet cancels any open requests that exceed the new target capacity. You can request that the Spot Fleet terminate Spot Instances until the size of the fleet no longer exceeds the new target capacity. If the allocation strategy is <code>lowestPrice</code>, the Spot Fleet terminates the instances with the highest price per unit. If the allocation strategy is <code>diversified</code>, the Spot Fleet terminates instances across the Spot pools. Alternatively, you can request that the Spot Fleet keep the fleet at its current size, but not replace any Spot Instances that are interrupted or that you terminate manually. If you are finished with your Spot Fleet for now, but will use it again later, you can set the target capacity to 0.

        :param self:
        :param OnDemandTargetCapacity:  The number of On-Demand Instances in the fleet.<br/>TargetCapacity:  The size of the fleet.<br/>SpotFleetRequestId:  The ID of the Spot Fleet request.<br/>ExcessCapacityTerminationPolicy:  Indicates whether running Spot Instances should be terminated if the target capacity of the Spot Fleet request is decreased below the current size of the Spot Fleet.<br/>
        :return: Return:  Is <code>true</code> if the request succeeds, and an error otherwise.<br/>        """
        pass

    def modify_subnet_attribute(self, SubnetId: str, AssignIpv6AddressOnCreation: AttributeBooleanValue=None, MapPublicIpOnLaunch: AttributeBooleanValue=None):
        """Modifies a subnet attribute. You can only modify one attribute at a time.

        :param self:
        :param SubnetId:  The ID of the subnet.<br/>MapPublicIpOnLaunch:  Specify <code>true</code> to indicate that ENIs attached to instances created in the specified subnet should be assigned a public IPv4 address.:<br/>AssignIpv6AddressOnCreation:  Specify <code>true</code> to indicate that network interfaces created in the specified subnet should be assigned an IPv6 address. This includes a network interface that's created when launching an instance into the subnet (the instance therefore receives an IPv6 address).  If you enable the IPv6 addressing feature for your subnet, your network interface or instance only receives an IPv6 address if it's created using version <code>2016-11-15</code> or later of the Amazon EC2 API.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The valid values are <code>true</code> or <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The valid values are <code>true</code> or <code>false</code>.<br/>
        :return:         """
        pass

    def modify_traffic_mirror_filter_network_services(self, TrafficMirrorFilterId: str, AddNetworkServices: List[TrafficMirrorNetworkService]=None, RemoveNetworkServices: List[TrafficMirrorNetworkService]=None, DryRun: Boolean=None) -> ModifyTrafficMirrorFilterNetworkServicesResult:
        """Allows or restricts mirroring network services.  By default, Amazon DNS network services are not eligible for Traffic Mirror. Use <code>AddNetworkServices</code> to add network services to a Traffic Mirror filter. When a network service is added to the Traffic Mirror filter, all traffic related to that network service will be mirrored. When you no longer want to mirror network services, use <code>RemoveNetworkServices</code> to remove the network services from the Traffic Mirror filter.  FFor information about filter rule properties, see <a href="https://docs.aws.amazon.com/vpc/latest/mirroring/traffic-mirroring-considerations.html#traffic-mirroring-network-services">Network Services</a> in the <i>Traffic Mirroring User Guide </i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>RemoveNetworkServices:  The network service, for example Amazon DNS, that you no longer want to mirror.<br/>AddNetworkServices:  The network service, for example Amazon DNS, that you want to mirror.<br/>TrafficMirrorFilterId:  The ID of the Traffic Mirror filter.<br/>
        :return: TrafficMirrorFilter:  The Traffic Mirror filter that the network service is associated with.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The tags assigned to the Traffic Mirror filter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the Traffic Mirror filter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkServices</i> The network service traffic that is associated with the Traffic Mirror filter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EgressFilterRules</i> Information about the egress rules that are associated with the Traffic Mirror filter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IngressFilterRules</i> Information about the ingress rules that are associated with the Traffic Mirror filter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrafficMirrorFilterId</i> The ID of the Traffic Mirror filter.<br/>        """
        pass

    def modify_traffic_mirror_filter_rule(self, TrafficMirrorFilterRuleId: str, TrafficDirection: str=None, RuleNumber: int=None, RuleAction: str=None, DestinationPortRange: TrafficMirrorPortRangeRequest=None, SourcePortRange: TrafficMirrorPortRangeRequest=None, Protocol: int=None, DestinationCidrBlock: str=None, SourceCidrBlock: str=None, Description: str=None, RemoveFields: List[TrafficMirrorFilterRuleField]=None, DryRun: Boolean=None) -> ModifyTrafficMirrorFilterRuleResult:
        """Modifies the specified Traffic Mirror rule.  <code>DestinationCidrBlock</code> and <code>SourceCidrBlock</code> must both be an IPv4 range or an IPv6 range.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>RemoveFields:  The properties that you want to remove from the Traffic Mirror filter rule. When you remove a property from a Traffic Mirror filter rule, the property is set to the default.<br/>Description:  The description to assign to the Traffic Mirror rule.<br/>SourceCidrBlock:  The source CIDR block to assign to the Traffic Mirror rule.<br/>DestinationCidrBlock:  The destination CIDR block to assign to the Traffic Mirror rule.<br/>Protocol:  The protocol, for example TCP, to assign to the Traffic Mirror rule.<br/>SourcePortRange:  The port range to assign to the Traffic Mirror rule.:<br/>DestinationPortRange:  The destination ports that are associated with the Traffic Mirror rule.:<br/>RuleAction:  The action to assign to the rule.<br/>RuleNumber:  The number of the Traffic Mirror rule. This number must be unique for each Traffic Mirror rule in a given direction. The rules are processed in ascending order by rule number.<br/>TrafficDirection:  The type of traffic (<code>ingress</code> | <code>egress</code>) to assign to the rule.<br/>TrafficMirrorFilterRuleId:  The ID of the Traffic Mirror rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ToPort</i> The last port in the Traffic Mirror port range. This applies to the TCP and UDP protocols.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FromPort</i> The first port in the Traffic Mirror port range. This applies to the TCP and UDP protocols.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ToPort</i> The last port in the Traffic Mirror port range. This applies to the TCP and UDP protocols.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FromPort</i> The first port in the Traffic Mirror port range. This applies to the TCP and UDP protocols.<br/>
        :return: TrafficMirrorFilterRule:  Modifies a Traffic Mirror rule.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the Traffic Mirror rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceCidrBlock</i> The source CIDR block assigned to the Traffic Mirror rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationCidrBlock</i> The destination CIDR block assigned to the Traffic Mirror rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourcePortRange</i> The source port range assigned to the Traffic Mirror rule.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SourcePortRange</b>: The source port range assigned to the Traffic Mirror rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationPortRange</i> The destination port range assigned to the Traffic Mirror rule.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DestinationPortRange</b>: The destination port range assigned to the Traffic Mirror rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Protocol</i> The protocol assigned to the Traffic Mirror rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleAction</i> The action assigned to the Traffic Mirror rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleNumber</i> The rule number of the Traffic Mirror rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrafficDirection</i> The traffic direction assigned to the Traffic Mirror rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrafficMirrorFilterId</i> The ID of the Traffic Mirror filter that the rule is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrafficMirrorFilterRuleId</i> The ID of the Traffic Mirror rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ToPort</i> The end of the Traffic Mirror port range. This applies to the TCP and UDP protocols.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FromPort</i> The start of the Traffic Mirror port range. This applies to the TCP and UDP protocols.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ToPort</i> The end of the Traffic Mirror port range. This applies to the TCP and UDP protocols.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FromPort</i> The start of the Traffic Mirror port range. This applies to the TCP and UDP protocols.<br/>        """
        pass

    def modify_traffic_mirror_session(self, TrafficMirrorSessionId: str, TrafficMirrorTargetId: str=None, TrafficMirrorFilterId: str=None, PacketLength: int=None, SessionNumber: int=None, VirtualNetworkId: int=None, Description: str=None, RemoveFields: List[TrafficMirrorSessionField]=None, DryRun: Boolean=None) -> ModifyTrafficMirrorSessionResult:
        """Modifies a Traffic Mirror session.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>RemoveFields:  The properties that you want to remove from the Traffic Mirror session. When you remove a property from a Traffic Mirror session, the property is set to the default.<br/>Description:  The description to assign to the Traffic Mirror session.<br/>VirtualNetworkId:  The virtual network ID of the Traffic Mirror session.<br/>SessionNumber:  The session number determines the order in which sessions are evaluated when an interface is used by multiple sessions. The first session with a matching filter is the one that mirrors the packets. Valid values are 1-32766.<br/>PacketLength:  The number of bytes in each packet to mirror. These are bytes after the VXLAN header. To mirror a subset, set this to the length (in bytes) to mirror. For example, if you set this value to 100, then the first 100 bytes that meet the filter criteria are copied to the target. Do not specify this parameter when you want to mirror the entire packet.<br/>TrafficMirrorFilterId:  The ID of the Traffic Mirror filter.<br/>TrafficMirrorTargetId:  The Traffic Mirror target. The target must be in the same VPC as the source, or have a VPC peering connection with the source.<br/>TrafficMirrorSessionId:  The ID of the Traffic Mirror session.<br/>
        :return: TrafficMirrorSession:  Information about the Traffic Mirror session.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The tags assigned to the Traffic Mirror session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the Traffic Mirror session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VirtualNetworkId</i> The virtual network ID associated with the Traffic Mirror session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SessionNumber</i> The session number determines the order in which sessions are evaluated when an interface is used by multiple sessions. The first session with a matching filter is the one that mirrors the packets. Valid values are 1-32766.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PacketLength</i> The number of bytes in each packet to mirror. These are the bytes after the VXLAN header. To mirror a subset, set this to the length (in bytes) to mirror. For example, if you set this value to 100, then the first 100 bytes that meet the filter criteria are copied to the target. Do not specify this parameter when you want to mirror the entire packet<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The ID of the account that owns the Traffic Mirror session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkInterfaceId</i> The ID of the Traffic Mirror session's network interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrafficMirrorFilterId</i> The ID of the Traffic Mirror filter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrafficMirrorTargetId</i> The ID of the Traffic Mirror target.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrafficMirrorSessionId</i> The ID for the Traffic Mirror session.<br/>        """
        pass

    def modify_transit_gateway_vpc_attachment(self, TransitGatewayAttachmentId: str, AddSubnetIds: List[String]=None, RemoveSubnetIds: List[String]=None, Options: ModifyTransitGatewayVpcAttachmentRequestOptions=None, DryRun: Boolean=None) -> ModifyTransitGatewayVpcAttachmentResult:
        """Modifies the specified VPC attachment.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Options:  The new VPC attachment options.:<br/>RemoveSubnetIds:  The IDs of one or more subnets to remove.<br/>AddSubnetIds:  The IDs of one or more subnets to add. You can specify at most one subnet per Availability Zone.<br/>TransitGatewayAttachmentId:  The ID of the attachment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Ipv6Support</i> Enable or disable IPv6 support. The default is <code>enable</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DnsSupport</i> Enable or disable DNS support. The default is <code>enable</code>.<br/>
        :return: TransitGatewayVpcAttachment:  Information about the modified attachment.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The tags for the VPC attachment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> The VPC attachment options.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>: The VPC attachment options.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The creation time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> The IDs of the subnets.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the VPC attachment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcOwnerId</i> The ID of the AWS account that owns the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayId</i> The ID of the transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayAttachmentId</i> The ID of the attachment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Ipv6Support</i> Indicates whether IPv6 support is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DnsSupport</i> Indicates whether DNS support is enabled.<br/>        """
        pass

    def modify_volume(self, VolumeId: str, DryRun: Boolean=None, Size: int=None, VolumeType: str=None, Iops: int=None) -> ModifyVolumeResult:
        """You can modify several parameters of an existing EBS volume, including volume size, volume type, and IOPS capacity. If your EBS volume is attached to a current-generation EC2 instance type, you may be able to apply these changes without stopping the instance or detaching the volume from it. For more information about modifying an EBS volume running Linux, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ebs-expand-volume.html">Modifying the Size, IOPS, or Type of an EBS Volume on Linux</a>. For more information about modifying an EBS volume running Windows, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/WindowsGuide/ebs-expand-volume.html">Modifying the Size, IOPS, or Type of an EBS Volume on Windows</a>.   When you complete a resize operation on your volume, you need to extend the volume's file-system size to take advantage of the new storage capacity. For information about extending a Linux file system, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ebs-expand-volume.html#recognize-expanded-volume-linux">Extending a Linux File System</a>. For information about extending a Windows file system, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/WindowsGuide/ebs-expand-volume.html#recognize-expanded-volume-windows">Extending a Windows File System</a>.   You can use CloudWatch Events to check the status of a modification to an EBS volume. For information about CloudWatch Events, see the <a href="https://docs.aws.amazon.com/AmazonCloudWatch/latest/events/">Amazon CloudWatch Events User Guide</a>. You can also track the status of a modification using <a>DescribeVolumesModifications</a>. For information about tracking status changes using either method, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ebs-expand-volume.html#monitoring_mods">Monitoring Volume Modifications</a>.  With previous-generation instance types, resizing an EBS volume may require detaching and reattaching the volume or stopping and restarting the instance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ebs-expand-volume.html">Modifying the Size, IOPS, or Type of an EBS Volume on Linux</a> and <a href="https://docs.aws.amazon.com/AWSEC2/latest/WindowsGuide/ebs-expand-volume.html">Modifying the Size, IOPS, or Type of an EBS Volume on Windows</a>. If you reach the maximum volume modification rate per volume limit, you will need to wait at least six hours before applying further modifications to the affected EBS volume.

        :param self:
        :param Iops:  The target IOPS rate of the volume. This is only valid for Provisioned IOPS SSD (<code>io1</code>) volumes. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSVolumeTypes.html#EBSVolumeTypes_piops">Provisioned IOPS SSD (io1) Volumes</a>. Default: If no IOPS value is specified, the existing value is retained.<br/>VolumeType:  The target EBS volume type of the volume. Default: If no type is specified, the existing type is retained.<br/>Size:  The target size of the volume, in GiB. The target volume size must be greater than or equal to than the existing size of the volume. For information about available EBS volume sizes, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSVolumeTypes.html">Amazon EBS Volume Types</a>. Default: If no size is specified, the existing size is retained.<br/>VolumeId:  The ID of the volume.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: VolumeModification:  Information about the volume modification.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> The modification completion or failure time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTime</i> The modification start time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Progress</i> The modification progress, from 0 to 100 percent complete.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OriginalVolumeType</i> The original EBS volume type of the volume.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OriginalIops</i> The original IOPS rate of the volume.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OriginalSize</i> The original size of the volume.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetVolumeType</i> The target EBS volume type of the volume.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetIops</i> The target IOPS rate of the volume.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetSize</i> The target size of the volume, in GiB.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> A status message about the modification progress or failure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModificationState</i> The current modification state. The modification state is null for unmodified volumes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeId</i> The ID of the volume.<br/>        """
        pass

    def modify_volume_attribute(self, VolumeId: str, AutoEnableIO: AttributeBooleanValue=None, DryRun: Boolean=None):
        """Modifies a volume attribute. By default, all I/O operations for the volume are suspended when the data on the volume is determined to be potentially inconsistent, to prevent undetectable, latent data corruption. The I/O access to the volume can be resumed by first enabling I/O access and then checking the data consistency on your volume. You can change the default behavior to resume I/O operations. We recommend that you change this only for boot volumes or for volumes that are stateless or disposable.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>VolumeId:  The ID of the volume.<br/>AutoEnableIO:  Indicates whether the volume should be auto-enabled for I/O operations.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The valid values are <code>true</code> or <code>false</code>.<br/>
        :return:         """
        pass

    def modify_vpc_attribute(self, VpcId: str, EnableDnsHostnames: AttributeBooleanValue=None, EnableDnsSupport: AttributeBooleanValue=None):
        """Modifies the specified attribute of the specified VPC.

        :param self:
        :param VpcId:  The ID of the VPC.<br/>EnableDnsSupport:  Indicates whether the DNS resolution is supported for the VPC. If enabled, queries to the Amazon provided DNS server at the 169.254.169.253 IP address, or the reserved IP address at the base of the VPC network range "plus two" succeed. If disabled, the Amazon provided DNS service in the VPC that resolves public DNS hostnames to IP addresses is not enabled. You cannot modify the DNS resolution and DNS hostnames attributes in the same request. Use separate requests for each attribute.:<br/>EnableDnsHostnames:  Indicates whether the instances launched in the VPC get DNS hostnames. If enabled, instances in the VPC get DNS hostnames; otherwise, they do not. You cannot modify the DNS resolution and DNS hostnames attributes in the same request. Use separate requests for each attribute. You can only enable DNS hostnames if you've enabled DNS support.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The valid values are <code>true</code> or <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The attribute value. The valid values are <code>true</code> or <code>false</code>.<br/>
        :return:         """
        pass

    def modify_vpc_endpoint(self, VpcEndpointId: str, DryRun: Boolean=None, ResetPolicy: Boolean=None, PolicyDocument: str=None, AddRouteTableIds: List[String]=None, RemoveRouteTableIds: List[String]=None, AddSubnetIds: List[String]=None, RemoveSubnetIds: List[String]=None, AddSecurityGroupIds: List[String]=None, RemoveSecurityGroupIds: List[String]=None, PrivateDnsEnabled: Boolean=None) -> ModifyVpcEndpointResult:
        """Modifies attributes of a specified VPC endpoint. The attributes that you can modify depend on the type of VPC endpoint (interface or gateway). For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/vpc-endpoints.html">VPC Endpoints</a> in the <i>Amazon Virtual Private Cloud User Guide</i>.

        :param self:
        :param PrivateDnsEnabled:  (Interface endpoint) Indicate whether a private hosted zone is associated with the VPC.<br/>RemoveSecurityGroupIds:  (Interface endpoint) One or more security group IDs to disassociate from the network interface.<br/>AddSecurityGroupIds:  (Interface endpoint) One or more security group IDs to associate with the network interface.<br/>RemoveSubnetIds:  (Interface endpoint) One or more subnets IDs in which to remove the endpoint.<br/>AddSubnetIds:  (Interface endpoint) One or more subnet IDs in which to serve the endpoint.<br/>RemoveRouteTableIds:  (Gateway endpoint) One or more route table IDs to disassociate from the endpoint.<br/>AddRouteTableIds:  (Gateway endpoint) One or more route tables IDs to associate with the endpoint.<br/>PolicyDocument:  A policy to attach to the endpoint that controls access to the service. The policy must be in valid JSON format. If this parameter is not specified, we attach a default policy that allows full access to the service.<br/>ResetPolicy:  (Gateway endpoint) Specify <code>true</code> to reset the policy document to the default policy. The default policy allows full access to the service.<br/>VpcEndpointId:  The ID of the endpoint.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: Return:  Returns <code>true</code> if the request succeeds; otherwise, it returns an error.<br/>        """
        pass

    def modify_vpc_endpoint_connection_notification(self, ConnectionNotificationId: str, DryRun: Boolean=None, ConnectionNotificationArn: str=None, ConnectionEvents: List[String]=None) -> ModifyVpcEndpointConnectionNotificationResult:
        """Modifies a connection notification for VPC endpoint or VPC endpoint service. You can change the SNS topic for the notification, or the events for which to be notified. 

        :param self:
        :param ConnectionEvents:  One or more events for the endpoint. Valid values are <code>Accept</code>, <code>Connect</code>, <code>Delete</code>, and <code>Reject</code>.<br/>ConnectionNotificationArn:  The ARN for the SNS topic for the notification.<br/>ConnectionNotificationId:  The ID of the notification.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: ReturnValue:  Returns <code>true</code> if the request succeeds; otherwise, it returns an error.<br/>        """
        pass

    def modify_vpc_endpoint_service_configuration(self, ServiceId: str, DryRun: Boolean=None, AcceptanceRequired: Boolean=None, AddNetworkLoadBalancerArns: List[String]=None, RemoveNetworkLoadBalancerArns: List[String]=None) -> ModifyVpcEndpointServiceConfigurationResult:
        """Modifies the attributes of your VPC endpoint service configuration. You can change the Network Load Balancers for your service, and you can specify whether acceptance is required for requests to connect to your endpoint service through an interface VPC endpoint.

        :param self:
        :param RemoveNetworkLoadBalancerArns:  The Amazon Resource Names (ARNs) of Network Load Balancers to remove from your service configuration.<br/>AddNetworkLoadBalancerArns:  The Amazon Resource Names (ARNs) of Network Load Balancers to add to your service configuration.<br/>AcceptanceRequired:  Indicate whether requests to create an endpoint to your service must be accepted.<br/>ServiceId:  The ID of the service.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: Return:  Returns <code>true</code> if the request succeeds; otherwise, it returns an error.<br/>        """
        pass

    def modify_vpc_endpoint_service_permissions(self, ServiceId: str, DryRun: Boolean=None, AddAllowedPrincipals: List[String]=None, RemoveAllowedPrincipals: List[String]=None) -> ModifyVpcEndpointServicePermissionsResult:
        """Modifies the permissions for your <a href="https://docs.aws.amazon.com/vpc/latest/userguide/endpoint-service.html">VPC endpoint service</a>. You can add or remove permissions for service consumers (IAM users, IAM roles, and AWS accounts) to connect to your endpoint service. If you grant permissions to all principals, the service is public. Any users who know the name of a public service can send a request to attach an endpoint. If the service does not require manual approval, attachments are automatically approved.

        :param self:
        :param RemoveAllowedPrincipals:  The Amazon Resource Names (ARN) of one or more principals. Permissions are revoked for principals in this list.<br/>AddAllowedPrincipals:  The Amazon Resource Names (ARN) of one or more principals. Permissions are granted to the principals in this list. To grant permissions to all principals, specify an asterisk (*).<br/>ServiceId:  The ID of the service.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: ReturnValue:  Returns <code>true</code> if the request succeeds; otherwise, it returns an error.<br/>        """
        pass

    def modify_vpc_peering_connection_options(self, VpcPeeringConnectionId: str, AccepterPeeringConnectionOptions: PeeringConnectionOptionsRequest=None, DryRun: Boolean=None, RequesterPeeringConnectionOptions: PeeringConnectionOptionsRequest=None) -> ModifyVpcPeeringConnectionOptionsResult:
        """Modifies the VPC peering connection options on one side of a VPC peering connection. You can do the following: <ul> <li> Enable/disable communication over the peering connection between an EC2-Classic instance that's linked to your VPC (using ClassicLink) and instances in the peer VPC. </li> <li> Enable/disable communication over the peering connection between instances in your VPC and an EC2-Classic instance that's linked to the peer VPC. </li> <li> Enable/disable the ability to resolve public DNS hostnames to private IP addresses when queried from instances in the peer VPC. </li> </ul> If the peered VPCs are in the same AWS account, you can enable DNS resolution for queries from the local VPC. This ensures that queries from the local VPC resolve to private IP addresses in the peer VPC. This option is not available if the peered VPCs are in different AWS accounts or different Regions. For peered VPCs in different AWS accounts, each AWS account owner must initiate a separate request to modify the peering connection options. For inter-region peering connections, you must use the Region for the requester VPC to modify the requester VPC peering options and the Region for the accepter VPC to modify the accepter VPC peering options. To verify which VPCs are the accepter and the requester for a VPC peering connection, use the <a>DescribeVpcPeeringConnections</a> command.

        :param self:
        :param VpcPeeringConnectionId:  The ID of the VPC peering connection.<br/>RequesterPeeringConnectionOptions:  The VPC peering connection options for the requester VPC.:<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>AccepterPeeringConnectionOptions:  The VPC peering connection options for the accepter VPC.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowEgressFromLocalVpcToRemoteClassicLink</i> If true, enables outbound communication from instances in a local VPC to an EC2-Classic instance that's linked to a peer VPC using ClassicLink.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowEgressFromLocalClassicLinkToRemoteVpc</i> If true, enables outbound communication from an EC2-Classic instance that's linked to a local VPC using ClassicLink to instances in a peer VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowDnsResolutionFromRemoteVpc</i> If true, enables a local VPC to resolve public DNS hostnames to private IP addresses when queried from instances in the peer VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowEgressFromLocalVpcToRemoteClassicLink</i> If true, enables outbound communication from instances in a local VPC to an EC2-Classic instance that's linked to a peer VPC using ClassicLink.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowEgressFromLocalClassicLinkToRemoteVpc</i> If true, enables outbound communication from an EC2-Classic instance that's linked to a local VPC using ClassicLink to instances in a peer VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowDnsResolutionFromRemoteVpc</i> If true, enables a local VPC to resolve public DNS hostnames to private IP addresses when queried from instances in the peer VPC.<br/>
        :return: RequesterPeeringConnectionOptions:  Information about the VPC peering connection options for the requester VPC.:<br/>AccepterPeeringConnectionOptions:  Information about the VPC peering connection options for the accepter VPC.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowEgressFromLocalVpcToRemoteClassicLink</i> If true, enables outbound communication from instances in a local VPC to an EC2-Classic instance that's linked to a peer VPC using ClassicLink.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowEgressFromLocalClassicLinkToRemoteVpc</i> If true, enables outbound communication from an EC2-Classic instance that's linked to a local VPC using ClassicLink to instances in a peer VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowDnsResolutionFromRemoteVpc</i> If true, the public DNS hostnames of instances in the specified VPC resolve to private IP addresses when queried from instances in the peer VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowEgressFromLocalVpcToRemoteClassicLink</i> If true, enables outbound communication from instances in a local VPC to an EC2-Classic instance that's linked to a peer VPC using ClassicLink.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowEgressFromLocalClassicLinkToRemoteVpc</i> If true, enables outbound communication from an EC2-Classic instance that's linked to a local VPC using ClassicLink to instances in a peer VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowDnsResolutionFromRemoteVpc</i> If true, the public DNS hostnames of instances in the specified VPC resolve to private IP addresses when queried from instances in the peer VPC.<br/>        """
        pass

    def modify_vpc_tenancy(self, InstanceTenancy: str, VpcId: str, DryRun: Boolean=None) -> ModifyVpcTenancyResult:
        """Modifies the instance tenancy attribute of the specified VPC. You can change the instance tenancy attribute of a VPC to <code>default</code> only. You cannot change the instance tenancy attribute to <code>dedicated</code>. After you modify the tenancy of the VPC, any new instances that you launch into the VPC have a tenancy of <code>default</code>, unless you specify otherwise during launch. The tenancy of any existing instances in the VPC is not affected. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/dedicated-instance.html">Dedicated Instances</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>InstanceTenancy:  The instance tenancy attribute for the VPC. <br/>VpcId:  The ID of the VPC.<br/>
        :return: ReturnValue:  Returns <code>true</code> if the request succeeds; otherwise, returns an error.<br/>        """
        pass

    def modify_vpn_connection(self, VpnConnectionId: str, TransitGatewayId: str=None, VpnGatewayId: str=None, DryRun: Boolean=None) -> ModifyVpnConnectionResult:
        """Modifies the target gateway of a AWS Site-to-Site VPN connection. The following migration options are available: <ul> <li> An existing virtual private gateway to a new virtual private gateway </li> <li> An existing virtual private gateway to a transit gateway </li> <li> An existing transit gateway to a new transit gateway </li> <li> An existing transit gateway to a virtual private gateway </li> </ul> Before you perform the migration to the new gateway, you must configure the new gateway. Use <a>CreateVpnGateway</a> to create a virtual private gateway, or <a>CreateTransitGateway</a> to create a transit gateway. This step is required when you migrate from a virtual private gateway with static routes to a transit gateway.  You must delete the static routes before you migrate to the new gateway. Keep a copy of the static route before you delete it. You will need to add back these routes to the transit gateway after the VPN connection migration is complete. After you migrate to the new gateway, you might need to modify your VPC route table. Use <a>CreateRoute</a> and <a>DeleteRoute</a> to make the changes described in <a href="https://docs.aws.amazon.com/vpn/latest/s2svpn/modify-vpn-target.html#step-update-routing">VPN Gateway Target Modification Required VPC Route Table Updates</a> in the <i>AWS Site-to-Site VPN User Guide</i>.  When the new gateway is a transit gateway, modify the transit gateway route table to allow traffic between the VPC and the AWS Site-to-Site VPN connection. Use <a>CreateTransitGatewayRoute</a> to add the routes.  If you deleted VPN static routes, you must add the static routes to the transit gateway route table. After you perform this operation, the AWS VPN endpoint's IP addresses on the AWS side and the tunnel options remain intact. Your s2slong; connection will be temporarily unavailable for approximately 10 minutes while we provision the new endpoints 

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>VpnGatewayId:  The ID of the virtual private gateway at the AWS side of the VPN connection.<br/>TransitGatewayId:  The ID of the transit gateway.<br/>VpnConnectionId:  The ID of the VPN connection.<br/>
        :return: VpnConnection:  Describes a VPN connection.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VgwTelemetry</i> Information about the VPN tunnel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> Any tags assigned to the VPN connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Routes</i> The static routes associated with the VPN connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> The VPN connection options.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>: The VPN connection options.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayId</i> The ID of the transit gateway associated with the VPN connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpnGatewayId</i> The ID of the virtual private gateway at the AWS side of the VPN connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpnConnectionId</i> The ID of the VPN connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of VPN connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The current state of the VPN connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Category</i> The category of the VPN connection. A value of <code>VPN</code> indicates an AWS VPN connection. A value of <code>VPN-Classic</code> indicates an AWS Classic VPN connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomerGatewayId</i> The ID of the customer gateway at your end of the VPN connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomerGatewayConfiguration</i> The configuration information for the VPN connection's customer gateway (in the native XML format). This element is always present in the <a>CreateVpnConnection</a> response; however, it's present in the <a>DescribeVpnConnections</a> response only if the VPN connection is in the <code>pending</code> or <code>available</code> state.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StaticRoutesOnly</i> Indicates whether the VPN connection uses static routes only. Static routes must be used for devices that don't support BGP.<br/>        """
        pass

    def monitor_instances(self, InstanceIds: List[String], DryRun: Boolean=None) -> MonitorInstancesResult:
        """Enables detailed monitoring for a running instance. Otherwise, basic monitoring is enabled. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/using-cloudwatch.html">Monitoring Your Instances and Volumes</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. To disable detailed monitoring, see .

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>InstanceIds:  The IDs of the instances.<br/>
        :return: InstanceMonitorings:  The monitoring information.<br/>        """
        pass

    def move_address_to_vpc(self, PublicIp: str, DryRun: Boolean=None) -> MoveAddressToVpcResult:
        """Moves an Elastic IP address from the EC2-Classic platform to the EC2-VPC platform. The Elastic IP address must be allocated to your account for more than 24 hours, and it must not be associated with an instance. After the Elastic IP address is moved, it is no longer available for use in the EC2-Classic platform, unless you move it back using the <a>RestoreAddressToClassic</a> request. You cannot move an Elastic IP address that was originally allocated for use in the EC2-VPC platform to the EC2-Classic platform. 

        :param self:
        :param PublicIp:  The Elastic IP address.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: Status:  The status of the move of the IP address.<br/>AllocationId:  The allocation ID for the Elastic IP address.<br/>        """
        pass

    def provision_byoip_cidr(self, Cidr: str, CidrAuthorizationContext: CidrAuthorizationContext=None, Description: str=None, DryRun: Boolean=None) -> ProvisionByoipCidrResult:
        """Provisions an address range for use with your AWS resources through bring your own IP addresses (BYOIP) and creates a corresponding address pool. After the address range is provisioned, it is ready to be advertised using <a>AdvertiseByoipCidr</a>. AWS verifies that you own the address range and are authorized to advertise it. You must ensure that the address range is registered to you and that you created an RPKI ROA to authorize Amazon ASNs 16509 and 14618 to advertise the address range. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-byoip.html">Bring Your Own IP Addresses (BYOIP)</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. Provisioning an address range is an asynchronous operation, so the call returns immediately, but the address range is not ready to use until its status changes from <code>pending-provision</code> to <code>provisioned</code>. To monitor the status of an address range, use <a>DescribeByoipCidrs</a>. To allocate an Elastic IP address from your address pool, use <a>AllocateAddress</a> with either the specific address from the address pool or the ID of the address pool.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Description:  A description for the address range and the address pool.<br/>CidrAuthorizationContext:  A signed document that proves that you are authorized to bring the specified IP address range to Amazon using BYOIP.:<br/>Cidr:  The public IPv4 address range, in CIDR notation. The most specific prefix that you can specify is /24. The address range cannot overlap with another address range that you've brought to this or another Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Signature</i> The signed authorization message for the prefix and account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> The plain-text authorization message for the prefix and account.<br/>
        :return: ByoipCidr:  Information about the address pool.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the address pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> Upon success, contains the ID of the address pool. Otherwise, contains an error message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the address range.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Cidr</i> The public IPv4 address range, in CIDR notation.<br/>        """
        pass

    def purchase_host_reservation(self, OfferingId: str, HostIdSet: List[String], ClientToken: str=None, CurrencyCode: str=None, LimitPrice: str=None) -> PurchaseHostReservationResult:
        """Purchase a reservation with configurations that match those of your Dedicated Host. You must have active Dedicated Hosts in your account before you purchase a reservation. This action results in the specified reservation being purchased and charged to your account.

        :param self:
        :param OfferingId:  The ID of the offering.<br/>LimitPrice:  The specified limit is checked against the total upfront cost of the reservation (calculated as the offering's upfront cost multiplied by the host count). If the total upfront cost is greater than the specified price limit, the request fails. This is used to ensure that the purchase does not exceed the expected upfront cost of the purchase. At this time, the only supported currency is <code>USD</code>. For example, to indicate a limit price of USD 100, specify 100.00.<br/>HostIdSet:  The IDs of the Dedicated Hosts with which the reservation will be associated.<br/>CurrencyCode:  The currency in which the <code>totalUpfrontPrice</code>, <code>LimitPrice</code>, and <code>totalHourlyPrice</code> amounts are specified. At this time, the only supported currency is <code>USD</code>.<br/>ClientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.<br/>
        :return: TotalUpfrontPrice:  The total amount charged to your account when you purchase the reservation.<br/>TotalHourlyPrice:  The total hourly price of the reservation calculated per hour.<br/>Purchase:  Describes the details of the purchase.<br/>CurrencyCode:  The currency in which the <code>totalUpfrontPrice</code> and <code>totalHourlyPrice</code> amounts are specified. At this time, the only supported currency is <code>USD</code>.<br/>ClientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.<br/>        """
        pass

    def purchase_reserved_instances_offering(self, ReservedInstancesOfferingId: str, InstanceCount: int, DryRun: Boolean=None, LimitPrice: ReservedInstanceLimitPrice=None) -> PurchaseReservedInstancesOfferingResult:
        """Purchases a Reserved Instance for use with your account. With Reserved Instances, you pay a lower hourly rate compared to On-Demand instance pricing. Use <a>DescribeReservedInstancesOfferings</a> to get a list of Reserved Instance offerings that match your specifications. After you've purchased a Reserved Instance, you can check for your new Reserved Instance with <a>DescribeReservedInstances</a>. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/concepts-on-demand-reserved-instances.html">Reserved Instances</a> and <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ri-market-general.html">Reserved Instance Marketplace</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param LimitPrice:  Specified for Reserved Instance Marketplace offerings to limit the total order and ensure that the Reserved Instances are not purchased at unexpected prices.:<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>ReservedInstancesOfferingId:  The ID of the Reserved Instance offering to purchase.<br/>InstanceCount:  The number of Reserved Instances to purchase.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrencyCode</i> The currency in which the <code>limitPrice</code> amount is specified. At this time, the only supported currency is <code>USD</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Amount</i> Used for Reserved Instance Marketplace offerings. Specifies the limit price on the total order (instanceCount * price).<br/>
        :return: ReservedInstancesId:  The IDs of the purchased Reserved Instances.<br/>        """
        pass

    def purchase_scheduled_instances(self, PurchaseRequests: List[PurchaseRequest], ClientToken: str=None, DryRun: Boolean=None) -> PurchaseScheduledInstancesResult:
        """Purchases the Scheduled Instances with the specified schedule. Scheduled Instances enable you to purchase Amazon EC2 compute capacity by the hour for a one-year term. Before you can purchase a Scheduled Instance, you must call <a>DescribeScheduledInstanceAvailability</a> to check for available schedules and obtain a purchase token. After you purchase a Scheduled Instance, you must call <a>RunScheduledInstances</a> during each scheduled time period. After you purchase a Scheduled Instance, you can't cancel, modify, or resell your purchase.

        :param self:
        :param PurchaseRequests:  The purchase requests.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>ClientToken:  Unique, case-sensitive identifier that ensures the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">Ensuring Idempotency</a>.<br/>
        :return: ScheduledInstanceSet:  Information about the Scheduled Instances.<br/>        """
        pass

    def reboot_instances(self, InstanceIds: List[String], DryRun: Boolean=None):
        """Requests a reboot of the specified instances. This operation is asynchronous; it only queues a request to reboot the specified instances. The operation succeeds if the instances are valid and belong to you. Requests to reboot terminated instances are ignored. If an instance does not cleanly shut down within four minutes, Amazon EC2 performs a hard reboot. For more information about troubleshooting, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-console.html">Getting Console Output and Rebooting Instances</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>InstanceIds:  The instance IDs.<br/>
        :return:         """
        pass

    def register_image(self, Name: str, ImageLocation: str=None, Architecture: str=None, BlockDeviceMappings: List[BlockDeviceMapping]=None, Description: str=None, DryRun: Boolean=None, EnaSupport: Boolean=None, KernelId: str=None, BillingProducts: List[String]=None, RamdiskId: str=None, RootDeviceName: str=None, SriovNetSupport: str=None, VirtualizationType: str=None) -> RegisterImageResult:
        """Registers an AMI. When you're creating an AMI, this is the final step you must complete before you can launch an instance from the AMI. For more information about creating AMIs, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/creating-an-ami.html">Creating Your Own AMIs</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. <note> For Amazon EBS-backed instances, <a>CreateImage</a> creates and registers the AMI in a single request, so you don't have to register the AMI yourself. </note> You can also use <code>RegisterImage</code> to create an Amazon EBS-backed Linux AMI from a snapshot of a root device volume. You specify the snapshot using the block device mapping. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-launch-snapshot.html">Launching a Linux Instance from a Backup</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. You can't register an image where a secondary (non-root) snapshot has AWS Marketplace product codes. Some Linux distributions, such as Red Hat Enterprise Linux (RHEL) and SUSE Linux Enterprise Server (SLES), use the EC2 billing product code associated with an AMI to verify the subscription status for package updates. Creating an AMI from an EBS snapshot does not maintain this billing code, and instances launched from such an AMI are not able to connect to package update infrastructure. If you purchase a Reserved Instance offering for one of these Linux distributions and launch instances using an AMI that does not contain the required billing code, your Reserved Instance is not applied to these instances. To create an AMI for operating systems that require a billing code, see <a>CreateImage</a>. If needed, you can deregister an AMI at any time. Any modifications you make to an AMI backed by an instance store volume invalidates its registration. If you make changes to an image, deregister the previous image and register the new image.

        :param self:
        :param VirtualizationType:  The type of virtualization (<code>hvm</code> | <code>paravirtual</code>). Default: <code>paravirtual</code> <br/>SriovNetSupport:  Set to <code>simple</code> to enable enhanced networking with the Intel 82599 Virtual Function interface for the AMI and any instances that you launch from the AMI. There is no way to disable <code>sriovNetSupport</code> at this time. This option is supported only for HVM AMIs. Specifying this option with a PV AMI can make instances launched from the AMI unreachable.<br/>RootDeviceName:  The device name of the root device volume (for example, <code>/dev/sda1</code>).<br/>RamdiskId:  The ID of the RAM disk.<br/>BillingProducts:  The billing product codes. Your account must be authorized to specify billing product codes. Otherwise, you can use the AWS Marketplace to bill for the use of an AMI.<br/>Name:  A name for your AMI. Constraints: 3-128 alphanumeric characters, parentheses (()), square brackets ([]), spaces ( ), periods (.), slashes (/), dashes (-), single quotes ('), at-signs (@), or underscores(_)<br/>KernelId:  The ID of the kernel.<br/>EnaSupport:  Set to <code>true</code> to enable enhanced networking with ENA for the AMI and any instances that you launch from the AMI. This option is supported only for HVM AMIs. Specifying this option with a PV AMI can make instances launched from the AMI unreachable.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Description:  A description for your AMI.<br/>BlockDeviceMappings:  The block device mapping entries.<br/>Architecture:  The architecture of the AMI. Default: For Amazon EBS-backed AMIs, <code>i386</code>. For instance store-backed AMIs, the architecture specified in the manifest file.<br/>ImageLocation:  The full path to your AMI manifest in Amazon S3 storage. The specified bucket must have the <code>aws-exec-read</code> canned access control list (ACL) to ensure that it can be accessed by Amazon EC2. For more information, see <a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/acl-overview.html#canned-acl">Canned ACLs</a> in the <i>Amazon S3 Service Developer Guide</i>.<br/>
        :return: ImageId:  The ID of the newly registered AMI.<br/>        """
        pass

    def reject_transit_gateway_vpc_attachment(self, TransitGatewayAttachmentId: str, DryRun: Boolean=None) -> RejectTransitGatewayVpcAttachmentResult:
        """Rejects a request to attach a VPC to a transit gateway. The VPC attachment must be in the <code>pendingAcceptance</code> state. Use <a>DescribeTransitGatewayVpcAttachments</a> to view your pending VPC attachment requests. Use <a>AcceptTransitGatewayVpcAttachment</a> to accept a VPC attachment request.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>TransitGatewayAttachmentId:  The ID of the attachment.<br/>
        :return: TransitGatewayVpcAttachment:  Information about the attachment.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The tags for the VPC attachment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> The VPC attachment options.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>: The VPC attachment options.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The creation time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> The IDs of the subnets.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the VPC attachment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcOwnerId</i> The ID of the AWS account that owns the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayId</i> The ID of the transit gateway.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayAttachmentId</i> The ID of the attachment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Ipv6Support</i> Indicates whether IPv6 support is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DnsSupport</i> Indicates whether DNS support is enabled.<br/>        """
        pass

    def reject_vpc_endpoint_connections(self, VpcEndpointIds: List[String], ServiceId: str, DryRun: Boolean=None) -> RejectVpcEndpointConnectionsResult:
        """Rejects one or more VPC endpoint connection requests to your VPC endpoint service.

        :param self:
        :param VpcEndpointIds:  The IDs of one or more VPC endpoints.<br/>ServiceId:  The ID of the service.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: Unsuccessful:  Information about the endpoints that were not rejected, if applicable.<br/>        """
        pass

    def reject_vpc_peering_connection(self, VpcPeeringConnectionId: str, DryRun: Boolean=None) -> RejectVpcPeeringConnectionResult:
        """Rejects a VPC peering connection request. The VPC peering connection must be in the <code>pending-acceptance</code> state. Use the <a>DescribeVpcPeeringConnections</a> request to view your outstanding VPC peering connection requests. To delete an active VPC peering connection, or to delete a VPC peering connection request that you initiated, use <a>DeleteVpcPeeringConnection</a>.

        :param self:
        :param VpcPeeringConnectionId:  The ID of the VPC peering connection.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: Return:  Returns <code>true</code> if the request succeeds; otherwise, it returns an error.<br/>        """
        pass

    def release_address(self, AllocationId: str=None, PublicIp: str=None, DryRun: Boolean=None):
        """Releases the specified Elastic IP address. [EC2-Classic, default VPC] Releasing an Elastic IP address automatically disassociates it from any instance that it's associated with. To disassociate an Elastic IP address without releasing it, use <a>DisassociateAddress</a>. [Nondefault VPC] You must use <a>DisassociateAddress</a> to disassociate the Elastic IP address before you can release it. Otherwise, Amazon EC2 returns an error (<code>InvalidIPAddress.InUse</code>). After releasing an Elastic IP address, it is released to the IP address pool. Be sure to update your DNS records and any servers or devices that communicate with the address. If you attempt to release an Elastic IP address that you already released, you'll get an <code>AuthFailure</code> error if the address is already allocated to another AWS account. [EC2-VPC] After you release an Elastic IP address for use in a VPC, you might be able to recover it. For more information, see <a>AllocateAddress</a>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>PublicIp:  [EC2-Classic] The Elastic IP address. Required for EC2-Classic.<br/>AllocationId:  [EC2-VPC] The allocation ID. Required for EC2-VPC.<br/>
        :return:         """
        pass

    def release_hosts(self, HostIds: List[String]) -> ReleaseHostsResult:
        """When you no longer want to use an On-Demand Dedicated Host it can be released. On-Demand billing is stopped and the host goes into <code>released</code> state. The host ID of Dedicated Hosts that have been released can no longer be specified in another request, for example, to modify the host. You must stop or terminate all instances on a host before it can be released. When Dedicated Hosts are released, it may take some time for them to stop counting toward your limit and you may receive capacity errors when trying to allocate new Dedicated Hosts. Wait a few minutes and then try again. Released hosts still appear in a <a>DescribeHosts</a> response.

        :param self:
        :param HostIds:  The IDs of the Dedicated Hosts to release.<br/>
        :return: Unsuccessful:  The IDs of the Dedicated Hosts that could not be released, including an error message.<br/>Successful:  The IDs of the Dedicated Hosts that were successfully released.<br/>        """
        pass

    def replace_iam_instance_profile_association(self, AssociationId: str, IamInstanceProfile: IamInstanceProfileSpecification) -> ReplaceIamInstanceProfileAssociationResult:
        """Replaces an IAM instance profile for the specified running instance. You can use this action to change the IAM instance profile that's associated with an instance without having to disassociate the existing IAM instance profile first. Use <a>DescribeIamInstanceProfileAssociations</a> to get the association ID.

        :param self:
        :param AssociationId:  The ID of the existing IAM instance profile association.<br/>IamInstanceProfile:  The IAM instance profile.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the instance profile.<br/>
        :return: IamInstanceProfileAssociation:  Information about the IAM instance profile association.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timestamp</i> The time the IAM instance profile was associated with the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the association.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamInstanceProfile</i> The IAM instance profile.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>IamInstanceProfile</b>: The IAM instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceId</i> The ID of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociationId</i> The ID of the association.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the instance profile.<br/>        """
        pass

    def replace_network_acl_association(self, NetworkAclId: str, AssociationId: str, DryRun: Boolean=None) -> ReplaceNetworkAclAssociationResult:
        """Changes which network ACL a subnet is associated with. By default when you create a subnet, it's automatically associated with the default network ACL. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_ACLs.html">Network ACLs</a> in the <i>Amazon Virtual Private Cloud User Guide</i>. This is an idempotent operation.

        :param self:
        :param NetworkAclId:  The ID of the new network ACL to associate with the subnet.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>AssociationId:  The ID of the current association between the original network ACL and the subnet.<br/>
        :return: NewAssociationId:  The ID of the new association.<br/>        """
        pass

    def replace_network_acl_entry(self, RuleNumber: int, RuleAction: str, Protocol: str, NetworkAclId: str, Egress: Boolean, CidrBlock: str=None, DryRun: Boolean=None, IcmpTypeCode: IcmpTypeCode=None, Ipv6CidrBlock: str=None, PortRange: PortRange=None):
        """Replaces an entry (rule) in a network ACL. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_ACLs.html">Network ACLs</a> in the <i>Amazon Virtual Private Cloud User Guide</i>.

        :param self:
        :param RuleNumber:  The rule number of the entry to replace.<br/>RuleAction:  Indicates whether to allow or deny the traffic that matches the rule.<br/>Protocol:  The protocol number. A value of "-1" means all protocols. If you specify "-1" or a protocol number other than "6" (TCP), "17" (UDP), or "1" (ICMP), traffic on all ports is allowed, regardless of any ports or ICMP types or codes that you specify. If you specify protocol "58" (ICMPv6) and specify an IPv4 CIDR block, traffic for all ICMP types and codes allowed, regardless of any that you specify. If you specify protocol "58" (ICMPv6) and specify an IPv6 CIDR block, you must specify an ICMP type and code.<br/>PortRange:  TCP or UDP protocols: The range of ports the rule applies to. Required if specifying protocol 6 (TCP) or 17 (UDP).:<br/>NetworkAclId:  The ID of the ACL.<br/>Ipv6CidrBlock:  The IPv6 network range to allow or deny, in CIDR notation (for example <code>2001:bd8:1234:1a00::/64</code>).<br/>IcmpTypeCode:  ICMP protocol: The ICMP or ICMPv6 type and code. Required if specifying protocol 1 (ICMP) or protocol 58 (ICMPv6) with an IPv6 CIDR block.:<br/>Egress:  Indicates whether to replace the egress rule. Default: If no value is specified, we replace the ingress rule.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>CidrBlock:  The IPv4 network range to allow or deny, in CIDR notation (for example <code>172.16.0.0/24</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The ICMP type. A value of -1 means all types.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Code</i> The ICMP code. A value of -1 means all codes for the specified ICMP type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>To</i> The last port in the range.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>From</i> The first port in the range.<br/>
        :return:         """
        pass

    def replace_route(self, RouteTableId: str, DestinationCidrBlock: str=None, DestinationIpv6CidrBlock: str=None, DryRun: Boolean=None, EgressOnlyInternetGatewayId: str=None, GatewayId: str=None, InstanceId: str=None, NatGatewayId: str=None, TransitGatewayId: str=None, NetworkInterfaceId: str=None, VpcPeeringConnectionId: str=None):
        """Replaces an existing route within a route table in a VPC. You must provide only one of the following: internet gateway or virtual private gateway, NAT instance, NAT gateway, VPC peering connection, network interface, or egress-only internet gateway. For more information, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Route_Tables.html">Route Tables</a> in the <i>Amazon Virtual Private Cloud User Guide</i>.

        :param self:
        :param VpcPeeringConnectionId:  The ID of a VPC peering connection.<br/>RouteTableId:  The ID of the route table.<br/>NetworkInterfaceId:  The ID of a network interface.<br/>TransitGatewayId:  The ID of a transit gateway.<br/>NatGatewayId:  [IPv4 traffic only] The ID of a NAT gateway.<br/>InstanceId:  The ID of a NAT instance in your VPC.<br/>GatewayId:  The ID of an internet gateway or virtual private gateway.<br/>EgressOnlyInternetGatewayId:  [IPv6 traffic only] The ID of an egress-only internet gateway.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>DestinationIpv6CidrBlock:  The IPv6 CIDR address block used for the destination match. The value that you provide must match the CIDR of an existing route in the table.<br/>DestinationCidrBlock:  The IPv4 CIDR address block used for the destination match. The value that you provide must match the CIDR of an existing route in the table.<br/>
        :return:         """
        pass

    def replace_route_table_association(self, RouteTableId: str, AssociationId: str, DryRun: Boolean=None) -> ReplaceRouteTableAssociationResult:
        """Changes the route table associated with a given subnet in a VPC. After the operation completes, the subnet uses the routes in the new route table it's associated with. For more information about route tables, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Route_Tables.html">Route Tables</a> in the <i>Amazon Virtual Private Cloud User Guide</i>. You can also use ReplaceRouteTableAssociation to change which table is the main route table in the VPC. You just specify the main route table's association ID and the route table to be the new main route table.

        :param self:
        :param RouteTableId:  The ID of the new route table to associate with the subnet.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>AssociationId:  The association ID.<br/>
        :return: NewAssociationId:  The ID of the new association.<br/>        """
        pass

    def replace_transit_gateway_route(self, TransitGatewayRouteTableId: str, DestinationCidrBlock: str, TransitGatewayAttachmentId: str=None, Blackhole: Boolean=None, DryRun: Boolean=None) -> ReplaceTransitGatewayRouteResult:
        """Replaces the specified route in the specified transit gateway route table.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Blackhole:  Indicates whether traffic matching this route is to be dropped.<br/>TransitGatewayAttachmentId:  The ID of the attachment.<br/>TransitGatewayRouteTableId:  The ID of the route table.<br/>DestinationCidrBlock:  The CIDR range used for the destination match. Routing decisions are based on the most specific match.<br/>
        :return: Route:  Information about the modified route.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the route.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The route type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitGatewayAttachments</i> The attachments.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationCidrBlock</i> The CIDR block used for destination matches.<br/>        """
        pass

    def report_instance_status(self, Status: str, ReasonCodes: List[ReportInstanceReasonCodes], Instances: List[String], Description: str=None, DryRun: Boolean=None, EndTime: DateTime=None, StartTime: DateTime=None):
        """Submits feedback about the status of an instance. The instance must be in the <code>running</code> state. If your experience with the instance differs from the instance status returned by <a>DescribeInstanceStatus</a>, use <a>ReportInstanceStatus</a> to report your experience with the instance. Amazon EC2 collects this information to improve the accuracy of status checks. Use of this action does not change the value returned by <a>DescribeInstanceStatus</a>.

        :param self:
        :param Status:  The status of all instances listed.<br/>StartTime:  The time at which the reported instance health state began.<br/>ReasonCodes:  The reason codes that describe the health state of your instance. <ul> <li>  <code>instance-stuck-in-state</code>: My instance is stuck in a state. </li> <li>  <code>unresponsive</code>: My instance is unresponsive. </li> <li>  <code>not-accepting-credentials</code>: My instance is not accepting my credentials. </li> <li>  <code>password-not-available</code>: A password is not available for my instance. </li> <li>  <code>performance-network</code>: My instance is experiencing performance problems that I believe are network related. </li> <li>  <code>performance-instance-store</code>: My instance is experiencing performance problems that I believe are related to the instance stores. </li> <li>  <code>performance-ebs-volume</code>: My instance is experiencing performance problems that I believe are related to an EBS volume. </li> <li>  <code>performance-other</code>: My instance is experiencing performance problems. </li> <li>  <code>other</code>: [explain using the description parameter] </li> </ul><br/>Instances:  The instances.<br/>EndTime:  The time at which the reported instance health state ended.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Description:  Descriptive text about the health state of your instance.<br/>
        :return:         """
        pass

    def request_spot_fleet(self, SpotFleetRequestConfig: SpotFleetRequestConfigData, DryRun: Boolean=None) -> RequestSpotFleetResponse:
        """Creates a Spot Fleet request. The Spot Fleet request specifies the total target capacity and the On-Demand target capacity. Amazon EC2 calculates the difference between the total capacity and On-Demand capacity, and launches the difference as Spot capacity. You can submit a single request that includes multiple launch specifications that vary by instance type, AMI, Availability Zone, or subnet. By default, the Spot Fleet requests Spot Instances in the Spot pool where the price per unit is the lowest. Each launch specification can include its own instance weighting that reflects the value of the instance type to your application workload. Alternatively, you can specify that the Spot Fleet distribute the target capacity across the Spot pools included in its launch specifications. By ensuring that the Spot Instances in your Spot Fleet are in different Spot pools, you can improve the availability of your fleet. You can specify tags for the Spot Instances. You cannot tag other resource types in a Spot Fleet request because only the <code>instance</code> resource type is supported. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/spot-fleet-requests.html">Spot Fleet Requests</a> in the <i>Amazon EC2 User Guide for Linux Instances</i>.

        :param self:
        :param SpotFleetRequestConfig:  The configuration for the Spot Fleet request.:<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstancePoolsToUseCount</i> The number of Spot pools across which to allocate your target Spot capacity. Valid only when Spot <b>AllocationStrategy</b> is set to <code>lowest-price</code>. Spot Fleet selects the cheapest Spot pools and evenly allocates your target Spot capacity across the number of Spot pools that you specify.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LoadBalancersConfig</i> One or more Classic Load Balancers and target groups to attach to the Spot Fleet request. Spot Fleet registers the running Spot Instances with the specified Classic Load Balancers and target groups. With Network Load Balancers, Spot Fleet cannot register instances that have the following instance types: C1, CC1, CC2, CG1, CG2, CR1, CS1, G1, G2, HI1, HS1, M1, M2, M3, and T1.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LoadBalancersConfig</b>: One or more Classic Load Balancers and target groups to attach to the Spot Fleet request. Spot Fleet registers the running Spot Instances with the specified Classic Load Balancers and target groups. With Network Load Balancers, Spot Fleet cannot register instances that have the following instance types: C1, CC1, CC2, CG1, CG2, CR1, CS1, G1, G2, HI1, HS1, M1, M2, M3, and T1.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceInterruptionBehavior</i> The behavior when a Spot Instance is interrupted. The default is <code>terminate</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplaceUnhealthyInstances</i> Indicates whether Spot Fleet should replace unhealthy instances.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ValidUntil</i> The end date and time of the request, in UTC format (<i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z). After the end date and time, no new Spot Instance requests are placed or able to fulfill the request. If no value is specified, the Spot Fleet request remains until you cancel it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ValidFrom</i> The start date and time of the request, in UTC format (<i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z). By default, Amazon EC2 starts fulfilling the request immediately.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of request. Indicates whether the Spot Fleet only requests the target capacity or also attempts to maintain it. When this value is <code>request</code>, the Spot Fleet only places the required requests. It does not attempt to replenish Spot Instances if capacity is diminished, nor does it submit requests in alternative Spot pools if capacity is not available. When this value is <code>maintain</code>, the Spot Fleet maintains the target capacity. The Spot Fleet places the required requests to meet capacity and automatically replenishes any interrupted instances. Default: <code>maintain</code>. <code>instant</code> is listed but is not used by Spot Fleet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TerminateInstancesWithExpiration</i> Indicates whether running Spot Instances are terminated when the Spot Fleet request expires.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SpotMaxTotalPrice</i> The maximum amount per hour for Spot Instances that you're willing to pay. You can use the <code>spotdMaxTotalPrice</code> parameter, the <code>onDemandMaxTotalPrice</code> parameter, or both parameters to ensure that your fleet cost does not exceed your budget. If you set a maximum price per hour for the On-Demand Instances and Spot Instances in your request, Spot Fleet will launch instances until it reaches the maximum amount you're willing to pay. When the maximum amount you're willing to pay is reached, the fleet stops launching instances even if it hasn’t met the target capacity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OnDemandMaxTotalPrice</i> The maximum amount per hour for On-Demand Instances that you're willing to pay. You can use the <code>onDemandMaxTotalPrice</code> parameter, the <code>spotMaxTotalPrice</code> parameter, or both parameters to ensure that your fleet cost does not exceed your budget. If you set a maximum price per hour for the On-Demand Instances and Spot Instances in your request, Spot Fleet will launch instances until it reaches the maximum amount you're willing to pay. When the maximum amount you're willing to pay is reached, the fleet stops launching instances even if it hasn’t met the target capacity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OnDemandTargetCapacity</i> The number of On-Demand units to request. You can choose to set the target capacity in terms of instances or a performance characteristic that is important to your application workload, such as vCPUs, memory, or I/O. If the request type is <code>maintain</code>, you can specify a target capacity of 0 and add capacity later.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetCapacity</i> The number of units to request for the Spot Fleet. You can choose to set the target capacity in terms of instances or a performance characteristic that is important to your application workload, such as vCPUs, memory, or I/O. If the request type is <code>maintain</code>, you can specify a target capacity of 0 and add capacity later.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SpotPrice</i> The maximum price per unit hour that you are willing to pay for a Spot Instance. The default is the On-Demand price.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LaunchTemplateConfigs</i> The launch template and overrides. If you specify <code>LaunchTemplateConfigs</code>, you can't specify <code>LaunchSpecifications</code>. If you include On-Demand capacity in your request, you must use <code>LaunchTemplateConfigs</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LaunchSpecifications</i> The launch specifications for the Spot Fleet request. If you specify <code>LaunchSpecifications</code>, you can't specify <code>LaunchTemplateConfigs</code>. If you include On-Demand capacity in your request, you must use <code>LaunchTemplateConfigs</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamFleetRole</i> The Amazon Resource Name (ARN) of an AWS Identity and Access Management (IAM) role that grants the Spot Fleet the permission to request, launch, terminate, and tag instances on your behalf. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/spot-fleet-requests.html#spot-fleet-prerequisites">Spot Fleet Prerequisites</a> in the <i>Amazon EC2 User Guide for Linux Instances</i>. Spot Fleet can terminate Spot Instances on your behalf when you cancel its Spot Fleet request using <a>CancelSpotFleetRequests</a> or when the Spot Fleet request expires, if you set <code>TerminateInstancesWithExpiration</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OnDemandFulfilledCapacity</i> The number of On-Demand units fulfilled by this request compared to the set target On-Demand capacity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FulfilledCapacity</i> The number of units fulfilled by this request compared to the set target capacity. You cannot set this value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExcessCapacityTerminationPolicy</i> Indicates whether running Spot Instances should be terminated if you decrease the target capacity of the Spot Fleet request below the current size of the Spot Fleet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientToken</i> A unique, case-sensitive identifier that you provide to ensure the idempotency of your listings. This helps to avoid duplicate listings. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">Ensuring Idempotency</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OnDemandAllocationStrategy</i> The order of the launch template overrides to use in fulfilling On-Demand capacity. If you specify <code>lowestPrice</code>, Spot Fleet uses price to determine the order, launching the lowest price first. If you specify <code>prioritized</code>, Spot Fleet uses the priority that you assign to each Spot Fleet launch template override, launching the highest priority first. If you do not specify a value, Spot Fleet defaults to <code>lowestPrice</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocationStrategy</i> Indicates how to allocate the target capacity across the Spot pools specified by the Spot Fleet request. The default is <code>lowestPrice</code>.<br/>The Classic Load Balancers.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ClassicLoadBalancersConfig</b>: The Classic Load Balancers.
        :return: SpotFleetRequestId:  The ID of the Spot Fleet request.<br/>        """
        pass

    def request_spot_instances(self, AvailabilityZoneGroup: str=None, BlockDurationMinutes: int=None, ClientToken: str=None, DryRun: Boolean=None, InstanceCount: int=None, LaunchGroup: str=None, LaunchSpecification: RequestSpotLaunchSpecification=None, SpotPrice: str=None, Type: str=None, ValidFrom: DateTime=None, ValidUntil: DateTime=None, InstanceInterruptionBehavior: str=None) -> RequestSpotInstancesResult:
        """Creates a Spot Instance request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/spot-requests.html">Spot Instance Requests</a> in the <i>Amazon EC2 User Guide for Linux Instances</i>.

        :param self:
        :param InstanceInterruptionBehavior:  The behavior when a Spot Instance is interrupted. The default is <code>terminate</code>.<br/>ValidUntil:  The end date of the request. If this is a one-time request, the request remains active until all instances launch, the request is canceled, or this date is reached. If the request is persistent, it remains active until it is canceled or this date is reached. The default end date is 7 days from the current date.<br/>ValidFrom:  The start date of the request. If this is a one-time request, the request becomes active at this date and time and remains active until all instances launch, the request expires, or the request is canceled. If the request is persistent, the request becomes active at this date and time and remains active until it expires or is canceled.<br/>Type:  The Spot Instance request type. Default: <code>one-time</code> <br/>SpotPrice:  The maximum price per hour that you are willing to pay for a Spot Instance. The default is the On-Demand price.<br/>LaunchSpecification:  The launch specification.:<br/>LaunchGroup:  The instance launch group. Launch groups are Spot Instances that launch together and terminate together. Default: Instances are launched and terminated individually<br/>InstanceCount:  The maximum number of Spot Instances to launch. Default: 1<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>ClientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Run_Instance_Idempotency.html">How to Ensure Idempotency</a> in the <i>Amazon EC2 User Guide for Linux Instances</i>.<br/>BlockDurationMinutes:  The required duration for the Spot Instances (also known as Spot blocks), in minutes. This value must be a multiple of 60 (60, 120, 180, 240, 300, or 360). The duration period starts as soon as your Spot Instance receives its instance ID. At the end of the duration period, Amazon EC2 marks the Spot Instance for termination and provides a Spot Instance termination notice, which gives the instance a two-minute warning before it terminates. You can't specify an Availability Zone group or a launch group if you specify a duration.<br/>AvailabilityZoneGroup:  The user-specified name for a logical grouping of requests. When you specify an Availability Zone group in a Spot Instance request, all Spot Instances in the request are launched in the same Availability Zone. Instance proximity is maintained with this parameter, but the choice of Availability Zone is not. The group applies only to requests for Spot Instances of the same instance type. Any additional Spot Instance requests that are specified with the same Availability Zone group name are launched in that same Availability Zone, as long as at least one instance from the group is still active. If there is no active instance running in the Availability Zone group that you specify for a new Spot Instance request (all instances are terminated, the request is expired, or the maximum price you specified falls below current Spot price), then Amazon EC2 launches the instance in any Availability Zone where the constraint can be met. Consequently, the subsequent set of Spot Instances could be placed in a different zone from the original request, even if you specified the same Availability Zone group. Default: Instances are launched in any available Availability Zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserData</i> The Base64-encoded user data for the instance. User data is limited to 16 KB.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetId</i> The ID of the subnet in which to launch the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RamdiskId</i> The ID of the RAM disk.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Placement</i> The placement information for the instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Placement</b>: The placement information for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkInterfaces</i> One or more network interfaces. If you specify a network interface, you must specify subnet IDs and security group IDs using the network interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Monitoring</i> Indicates whether basic or detailed monitoring is enabled for the instance. Default: Disabled&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Monitoring</b>: Indicates whether basic or detailed monitoring is enabled for the instance. Default: Disabled<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyName</i> The name of the key pair.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KernelId</i> The ID of the kernel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i> The instance type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImageId</i> The ID of the AMI.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamInstanceProfile</i> The IAM instance profile.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>IamInstanceProfile</b>: The IAM instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EbsOptimized</i> Indicates whether the instance is optimized for EBS I/O. This optimization provides dedicated throughput to Amazon EBS and an optimized configuration stack to provide optimal EBS I/O performance. This optimization isn't available with all instance types. Additional usage charges apply when using an EBS Optimized instance. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BlockDeviceMappings</i> One or more block device mapping entries. You can't specify both a snapshot ID and an encryption value. This is because only blank volumes can be encrypted on creation. If a snapshot is the basis for a volume, it is not blank and its encryption status is used for the volume encryption status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressingType</i> Deprecated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroups</i> One or more security groups. When requesting instances in a VPC, you must specify the IDs of the security groups. When requesting instances in EC2-Classic, you can specify the names or the IDs of the security groups.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> One or more security group IDs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Indicates whether detailed monitoring is enabled. Otherwise, basic monitoring is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Tenancy</i> The tenancy of the instance (if the instance is running in a VPC). An instance with a tenancy of <code>dedicated</code> runs on single-tenant hardware. The <code>host</code> tenancy is not supported for Spot Instances.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupName</i> The name of the placement group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone. [Spot Fleet only] To specify multiple Availability Zones, separate them using commas; for example, "us-west-2a, us-west-2b".<br/>
        :return: SpotInstanceRequests:  One or more Spot Instance requests.<br/>        """
        pass

    def reset_ebs_default_kms_key_id(self, DryRun: Boolean=None) -> ResetEbsDefaultKmsKeyIdResult:
        """Resets the default customer master key (CMK) for EBS encryption for your account in this Region to the AWS managed CMK for EBS. After resetting the default CMK to the AWS managed CMK, you can continue to encrypt by a customer managed CMK by specifying it when you create the volume. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html">Amazon EBS Encryption</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: KmsKeyId:  The Amazon Resource Name (ARN) of the default CMK for EBS encryption by default.<br/>        """
        pass

    def reset_fpga_image_attribute(self, FpgaImageId: str, DryRun: Boolean=None, Attribute: str=None) -> ResetFpgaImageAttributeResult:
        """Resets the specified attribute of the specified Amazon FPGA Image (AFI) to its default value. You can only reset the load permission attribute.

        :param self:
        :param Attribute:  The attribute.<br/>FpgaImageId:  The ID of the AFI.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: Return:  Is <code>true</code> if the request succeeds, and an error otherwise.<br/>        """
        pass

    def reset_image_attribute(self, ImageId: str, Attribute: str, DryRun: Boolean=None):
        """Resets an attribute of an AMI to its default value. <note> The productCodes attribute can't be reset. </note>

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>ImageId:  The ID of the AMI.<br/>Attribute:  The attribute to reset (currently you can only reset the launch permission attribute).<br/>
        :return:         """
        pass

    def reset_instance_attribute(self, InstanceId: str, Attribute: str, DryRun: Boolean=None):
        """Resets an attribute of an instance to its default value. To reset the <code>kernel</code> or <code>ramdisk</code>, the instance must be in a stopped state. To reset the <code>sourceDestCheck</code>, the instance can be either running or stopped. The <code>sourceDestCheck</code> attribute controls whether source/destination checking is enabled. The default value is <code>true</code>, which means checking is enabled. This value must be <code>false</code> for a NAT instance to perform NAT. For more information, see <a href="https://docs.aws.amazon.com/AmazonVPC/latest/UserGuide/VPC_NAT_Instance.html">NAT Instances</a> in the <i>Amazon Virtual Private Cloud User Guide</i>.

        :param self:
        :param InstanceId:  The ID of the instance.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Attribute:  The attribute to reset. <important> You can only reset the following attributes: <code>kernel</code> | <code>ramdisk</code> | <code>sourceDestCheck</code>. To change an instance attribute, use <a>ModifyInstanceAttribute</a>. </important><br/>
        :return:         """
        pass

    def reset_network_interface_attribute(self, NetworkInterfaceId: str, DryRun: Boolean=None, SourceDestCheck: str=None):
        """Resets a network interface attribute. You can specify only one attribute at a time.

        :param self:
        :param SourceDestCheck:  The source/destination checking attribute. Resets the value to <code>true</code>.<br/>NetworkInterfaceId:  The ID of the network interface.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return:         """
        pass

    def reset_snapshot_attribute(self, SnapshotId: str, Attribute: str, DryRun: Boolean=None):
        """Resets permission settings for the specified snapshot. For more information about modifying snapshot permissions, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ebs-modifying-snapshot-permissions.html">Sharing Snapshots</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>SnapshotId:  The ID of the snapshot.<br/>Attribute:  The attribute to reset. Currently, only the attribute for permission to create volumes can be reset.<br/>
        :return:         """
        pass

    def restore_address_to_classic(self, PublicIp: str, DryRun: Boolean=None) -> RestoreAddressToClassicResult:
        """Restores an Elastic IP address that was previously moved to the EC2-VPC platform back to the EC2-Classic platform. You cannot move an Elastic IP address that was originally allocated for use in EC2-VPC. The Elastic IP address must not be associated with an instance or network interface.

        :param self:
        :param PublicIp:  The Elastic IP address.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: Status:  The move status for the IP address.<br/>PublicIp:  The Elastic IP address.<br/>        """
        pass

    def revoke_client_vpn_ingress(self, TargetNetworkCidr: str, ClientVpnEndpointId: str, AccessGroupId: str=None, RevokeAllGroups: Boolean=None, DryRun: Boolean=None) -> RevokeClientVpnIngressResult:
        """Removes an ingress authorization rule from a Client VPN endpoint. 

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>RevokeAllGroups:  Indicates whether access should be revoked for all clients.<br/>AccessGroupId:  The ID of the Active Directory group for which to revoke access. <br/>TargetNetworkCidr:  The IPv4 address range, in CIDR notation, of the network for which access is being removed.<br/>ClientVpnEndpointId:  The ID of the Client VPN endpoint with which the authorization rule is associated.<br/>
        :return: Status:  The current state of the authorization rule.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> A message about the status of the authorization rule, if applicable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Code</i> The state of the authorization rule.<br/>        """
        pass

    def revoke_security_group_egress(self, GroupId: str, DryRun: Boolean=None, IpPermissions: List[IpPermission]=None, CidrIp: str=None, FromPort: int=None, IpProtocol: str=None, ToPort: int=None, SourceSecurityGroupName: str=None, SourceSecurityGroupOwnerId: str=None):
        """[VPC only] Removes the specified egress rules from a security group for EC2-VPC. This action doesn't apply to security groups for use in EC2-Classic. To remove a rule, the values that you specify (for example, ports) must match the existing rule's values exactly. Each rule consists of the protocol and the IPv4 or IPv6 CIDR range or source security group. For the TCP and UDP protocols, you must also specify the destination port or range of ports. For the ICMP protocol, you must also specify the ICMP type and code. If the security group rule has a description, you do not have to specify the description to revoke the rule. Rule changes are propagated to instances within the security group as quickly as possible. However, a small delay might occur.

        :param self:
        :param SourceSecurityGroupOwnerId:  Not supported. Use a set of IP permissions to specify a destination security group.<br/>SourceSecurityGroupName:  Not supported. Use a set of IP permissions to specify a destination security group.<br/>ToPort:  Not supported. Use a set of IP permissions to specify the port.<br/>IpProtocol:  Not supported. Use a set of IP permissions to specify the protocol name or number.<br/>FromPort:  Not supported. Use a set of IP permissions to specify the port.<br/>CidrIp:  Not supported. Use a set of IP permissions to specify the CIDR.<br/>IpPermissions:  The sets of IP permissions. You can't specify a destination security group and a CIDR IP address range in the same set of permissions.<br/>GroupId:  The ID of the security group.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return:         """
        pass

    def revoke_security_group_ingress(self, CidrIp: str=None, FromPort: int=None, GroupId: str=None, GroupName: str=None, IpPermissions: List[IpPermission]=None, IpProtocol: str=None, SourceSecurityGroupName: str=None, SourceSecurityGroupOwnerId: str=None, ToPort: int=None, DryRun: Boolean=None):
        """Removes the specified ingress rules from a security group. To remove a rule, the values that you specify (for example, ports) must match the existing rule's values exactly. <note> [EC2-Classic only] If the values you specify do not match the existing rule's values, no error is returned. Use <a>DescribeSecurityGroups</a> to verify that the rule has been removed. </note> Each rule consists of the protocol and the CIDR range or source security group. For the TCP and UDP protocols, you must also specify the destination port or range of ports. For the ICMP protocol, you must also specify the ICMP type and code. If the security group rule has a description, you do not have to specify the description to revoke the rule. Rule changes are propagated to instances within the security group as quickly as possible. However, a small delay might occur.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>ToPort:  The end of port range for the TCP and UDP protocols, or an ICMP code number. For the ICMP code number, use <code>-1</code> to specify all ICMP codes for the ICMP type.<br/>SourceSecurityGroupOwnerId:  [EC2-Classic] The AWS account ID of the source security group, if the source security group is in a different account. You can't specify this parameter in combination with the following parameters: the CIDR IP address range, the IP protocol, the start of the port range, and the end of the port range. To revoke a specific rule for an IP protocol and port range, use a set of IP permissions instead.<br/>SourceSecurityGroupName:  [EC2-Classic, default VPC] The name of the source security group. You can't specify this parameter in combination with the following parameters: the CIDR IP address range, the start of the port range, the IP protocol, and the end of the port range. For EC2-VPC, the source security group must be in the same VPC. To revoke a specific rule for an IP protocol and port range, use a set of IP permissions instead.<br/>IpProtocol:  The IP protocol name (<code>tcp</code>, <code>udp</code>, <code>icmp</code>) or number (see <a href="http://www.iana.org/assignments/protocol-numbers/protocol-numbers.xhtml">Protocol Numbers</a>). Use <code>-1</code> to specify all.<br/>IpPermissions:  The sets of IP permissions. You can't specify a source security group and a CIDR IP address range in the same set of permissions.<br/>GroupName:  [EC2-Classic, default VPC] The name of the security group. You must specify either the security group ID or the security group name in the request.<br/>GroupId:  The ID of the security group. You must specify either the security group ID or the security group name in the request. For security groups in a nondefault VPC, you must specify the security group ID.<br/>FromPort:  The start of port range for the TCP and UDP protocols, or an ICMP type number. For the ICMP type number, use <code>-1</code> to specify all ICMP types.<br/>CidrIp:  The CIDR IP address range. You can't specify this parameter when specifying a source security group.<br/>
        :return:         """
        pass

    def run_instances(self, MinCount: int, MaxCount: int, BlockDeviceMappings: List[BlockDeviceMapping]=None, ImageId: str=None, InstanceType: str=None, Ipv6AddressCount: int=None, Ipv6Addresses: List[InstanceIpv6Address]=None, KernelId: str=None, KeyName: str=None, Monitoring: RunInstancesMonitoringEnabled=None, Placement: Placement=None, RamdiskId: str=None, SecurityGroupIds: List[String]=None, SecurityGroups: List[String]=None, SubnetId: str=None, UserData: str=None, AdditionalInfo: str=None, ClientToken: str=None, DisableApiTermination: Boolean=None, DryRun: Boolean=None, EbsOptimized: Boolean=None, IamInstanceProfile: IamInstanceProfileSpecification=None, InstanceInitiatedShutdownBehavior: str=None, NetworkInterfaces: List[InstanceNetworkInterfaceSpecification]=None, PrivateIpAddress: str=None, ElasticGpuSpecification: List[ElasticGpuSpecification]=None, ElasticInferenceAccelerators: List[ElasticInferenceAccelerator]=None, TagSpecifications: List[TagSpecification]=None, LaunchTemplate: LaunchTemplateSpecification=None, InstanceMarketOptions: InstanceMarketOptionsRequest=None, CreditSpecification: CreditSpecificationRequest=None, CpuOptions: CpuOptionsRequest=None, CapacityReservationSpecification: CapacityReservationSpecification=None, HibernationOptions: HibernationOptionsRequest=None, LicenseSpecifications: List[LicenseConfigurationRequest]=None) -> Reservation:
        """Launches the specified number of instances using an AMI for which you have permissions.  You can specify a number of options, or leave the default options. The following rules apply: <ul> <li> [EC2-VPC] If you don't specify a subnet ID, we choose a default subnet from your default VPC for you. If you don't have a default VPC, you must specify a subnet ID in the request. </li> <li> [EC2-Classic] If don't specify an Availability Zone, we choose one for you. </li> <li> Some instance types must be launched into a VPC. If you do not have a default VPC, or if you do not specify a subnet ID, the request fails. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/using-vpc.html#vpc-only-instance-types">Instance Types Available Only in a VPC</a>. </li> <li> [EC2-VPC] All instances have a network interface with a primary private IPv4 address. If you don't specify this address, we choose one from the IPv4 range of your subnet. </li> <li> Not all instance types support IPv6 addresses. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html">Instance Types</a>. </li> <li> If you don't specify a security group ID, we use the default security group. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/using-network-security.html">Security Groups</a>. </li> <li> If any of the AMIs have a product code attached for which the user has not subscribed, the request fails. </li> </ul> You can create a <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-launch-templates.html">launch template</a>, which is a resource that contains the parameters to launch an instance. When you launch an instance using <a>RunInstances</a>, you can specify the launch template instead of specifying the launch parameters. To ensure faster instance launches, break up large requests into smaller batches. For example, create five separate launch requests for 100 instances each instead of one launch request for 500 instances. An instance is ready for you to use when it's in the <code>running</code> state. You can check the state of your instance using <a>DescribeInstances</a>. You can tag instances and EBS volumes during launch, after launch, or both. For more information, see <a>CreateTags</a> and <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Using_Tags.html">Tagging Your Amazon EC2 Resources</a>. Linux instances have access to the public key of the key pair at boot. You can use this key to provide secure access to the instance. Amazon EC2 public images use this feature to provide secure access without passwords. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-key-pairs.html">Key Pairs</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. For troubleshooting, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Using_InstanceStraightToTerminated.html">What To Do If An Instance Immediately Terminates</a>, and <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/TroubleshootingInstancesConnecting.html">Troubleshooting Connecting to Your Instance</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param LicenseSpecifications:  The license configurations.<br/>HibernationOptions:  Indicates whether an instance is enabled for hibernation. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Hibernate.html">Hibernate Your Instance</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.:<br/>CapacityReservationSpecification:  Information about the Capacity Reservation targeting option. If you do not specify this parameter, the instance's Capacity Reservation preference defaults to <code>open</code>, which enables it to run in any open Capacity Reservation that has matching attributes (instance type, platform, Availability Zone).:<br/>CpuOptions:  The CPU options for the instance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-optimize-cpu.html">Optimizing CPU Options</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.:<br/>CreditSpecification:  The credit option for CPU usage of the T2 or T3 instance. Valid values are <code>standard</code> and <code>unlimited</code>. To change this attribute after launch, use <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_ModifyInstanceCreditSpecification.html"> ModifyInstanceCreditSpecification</a>. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/burstable-performance-instances.html">Burstable Performance Instances</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. Default: <code>standard</code> (T2 instances) or <code>unlimited</code> (T3 instances):<br/>InstanceMarketOptions:  The market (purchasing) option for the instances. For <a>RunInstances</a>, persistent Spot Instance requests are only supported when <b>InstanceInterruptionBehavior</b> is set to either <code>hibernate</code> or <code>stop</code>.:<br/>LaunchTemplate:  The launch template to use to launch the instances. Any parameters that you specify in <a>RunInstances</a> override the same parameters in the launch template. You can specify either the name or ID of a launch template, but not both.:<br/>TagSpecifications:  The tags to apply to the resources during launch. You can only tag instances and volumes on launch. The specified tags are applied to all instances or volumes that are created during launch. To tag a resource after it has been created, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateTags.html">CreateTags</a>.<br/>ElasticInferenceAccelerators:  An elastic inference accelerator to associate with the instance. Elastic inference accelerators are a resource you can attach to your Amazon EC2 instances to accelerate your Deep Learning (DL) inference workloads.<br/>ElasticGpuSpecification:  An elastic GPU to associate with the instance. An Elastic GPU is a GPU resource that you can attach to your Windows instance to accelerate the graphics performance of your applications. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/WindowsGuide/elastic-graphics.html"> Amazon EC2 Elastic GPUs</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.<br/>PrivateIpAddress:  [EC2-VPC] The primary IPv4 address. You must specify a value from the IPv4 address range of the subnet. Only one private IP address can be designated as primary. You can't specify this option if you've specified the option to designate a private IP address as the primary IP address in a network interface specification. You cannot specify this option if you're launching more than one instance in the request. You cannot specify this option and the network interfaces option in the same request.<br/>NetworkInterfaces:  The network interfaces to associate with the instance. If you specify a network interface, you must specify any security groups and subnets as part of the network interface.<br/>InstanceInitiatedShutdownBehavior:  Indicates whether an instance stops or terminates when you initiate shutdown from the instance (using the operating system command for system shutdown). Default: <code>stop</code> <br/>IamInstanceProfile:  The IAM instance profile.:<br/>EbsOptimized:  Indicates whether the instance is optimized for Amazon EBS I/O. This optimization provides dedicated throughput to Amazon EBS and an optimized configuration stack to provide optimal Amazon EBS I/O performance. This optimization isn't available with all instance types. Additional usage charges apply when using an EBS-optimized instance. Default: <code>false</code> <br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>DisableApiTermination:  If you set this parameter to <code>true</code>, you can't terminate the instance using the Amazon EC2 console, CLI, or API; otherwise, you can. To change this attribute after launch, use <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_ModifyInstanceAttribute.html">ModifyInstanceAttribute</a>. Alternatively, if you set <code>InstanceInitiatedShutdownBehavior</code> to <code>terminate</code>, you can terminate the instance by running the shutdown command from the instance. Default: <code>false</code> <br/>ClientToken:  Unique, case-sensitive identifier you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">Ensuring Idempotency</a>. Constraints: Maximum 64 ASCII characters<br/>AdditionalInfo:  Reserved.<br/>UserData:  The user data to make available to the instance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/user-data.html">Running Commands on Your Linux Instance at Launch</a> (Linux) and <a href="https://docs.aws.amazon.com/AWSEC2/latest/WindowsGuide/ec2-instance-metadata.html#instancedata-add-user-data">Adding User Data</a> (Windows). If you are using a command line tool, base64-encoding is performed for you, and you can load the text from a file. Otherwise, you must provide base64-encoded text. User data is limited to 16 KB.<br/>SubnetId:  [EC2-VPC] The ID of the subnet to launch the instance into. If you specify a network interface, you must specify any subnets as part of the network interface.<br/>SecurityGroups:  [EC2-Classic, default VPC] The names of the security groups. For a nondefault VPC, you must use security group IDs instead. If you specify a network interface, you must specify any security groups as part of the network interface. Default: Amazon EC2 uses the default security group.<br/>SecurityGroupIds:  The IDs of the security groups. You can create a security group using <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateSecurityGroup.html">CreateSecurityGroup</a>. If you specify a network interface, you must specify any security groups as part of the network interface.<br/>RamdiskId:  The ID of the RAM disk to select. Some kernels require additional drivers at launch. Check the kernel requirements for information about whether you need to specify a RAM disk. To find kernel requirements, go to the AWS Resource Center and search for the kernel ID. <important> We recommend that you use PV-GRUB instead of kernels and RAM disks. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/UserProvidedkernels.html"> PV-GRUB</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. </important><br/>Placement:  The placement for the instance.:<br/>Monitoring:  Specifies whether detailed monitoring is enabled for the instance.:<br/>MinCount:  The minimum number of instances to launch. If you specify a minimum that is more instances than Amazon EC2 can launch in the target Availability Zone, Amazon EC2 launches no instances. Constraints: Between 1 and the maximum number you're allowed for the specified instance type. For more information about the default limits, and how to request an increase, see <a href="http://aws.amazon.com/ec2/faqs/#How_many_instances_can_I_run_in_Amazon_EC2">How many instances can I run in Amazon EC2</a> in the Amazon EC2 General FAQ.<br/>MaxCount:  The maximum number of instances to launch. If you specify more instances than Amazon EC2 can launch in the target Availability Zone, Amazon EC2 launches the largest possible number of instances above <code>MinCount</code>. Constraints: Between 1 and the maximum number you're allowed for the specified instance type. For more information about the default limits, and how to request an increase, see <a href="http://aws.amazon.com/ec2/faqs/#How_many_instances_can_I_run_in_Amazon_EC2">How many instances can I run in Amazon EC2</a> in the Amazon EC2 FAQ.<br/>KeyName:  The name of the key pair. You can create a key pair using <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateKeyPair.html">CreateKeyPair</a> or <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_ImportKeyPair.html">ImportKeyPair</a>. <important> If you do not specify a key pair, you can't connect to the instance unless you choose an AMI that is configured to allow users another way to log in. </important><br/>KernelId:  The ID of the kernel. <important> We recommend that you use PV-GRUB instead of kernels and RAM disks. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/UserProvidedkernels.html"> PV-GRUB</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. </important><br/>Ipv6Addresses:  [EC2-VPC] The IPv6 addresses from the range of the subnet to associate with the primary network interface. You cannot specify this option and the option to assign a number of IPv6 addresses in the same request. You cannot specify this option if you've specified a minimum number of instances to launch. You cannot specify this option and the network interfaces option in the same request.<br/>Ipv6AddressCount:  [EC2-VPC] The number of IPv6 addresses to associate with the primary network interface. Amazon EC2 chooses the IPv6 addresses from the range of your subnet. You cannot specify this option and the option to assign specific IPv6 addresses in the same request. You can specify this option if you've specified a minimum number of instances to launch. You cannot specify this option and the network interfaces option in the same request.<br/>InstanceType:  The instance type. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html">Instance Types</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. Default: <code>m1.small</code> <br/>ImageId:  The ID of the AMI. An AMI ID is required to launch an instance and must be specified here or in a launch template.<br/>BlockDeviceMappings:  The block device mapping entries.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Indicates whether detailed monitoring is enabled. Otherwise, basic monitoring is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SpreadDomain</i> Reserved for future use.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tenancy</i> The tenancy of the instance (if the instance is running in a VPC). An instance with a tenancy of <code>dedicated</code> runs on single-tenant hardware. The <code>host</code> tenancy is not supported for the <a>ImportInstance</a> command.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostId</i> The ID of the Dedicated Host on which the instance resides. This parameter is not supported for the <a>ImportInstance</a> command.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PartitionNumber</i> The number of the partition the instance is in. Valid only if the placement group strategy is set to <code>partition</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupName</i> The name of the placement group the instance is in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Affinity</i> The affinity setting for the instance on the Dedicated Host. This parameter is not supported for the <a>ImportInstance</a> command.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone of the instance. If not specified, an Availability Zone will be automatically chosen for you based on the load balancing criteria for the Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version number of the launch template. Default: The default version for the launch template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LaunchTemplateName</i> The name of the launch template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LaunchTemplateId</i> The ID of the launch template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SpotOptions</i> The options for Spot Instances.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpotOptions</b>: The options for Spot Instances.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MarketType</i> The market type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceInterruptionBehavior</i> The behavior when a Spot Instance is interrupted. The default is <code>terminate</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ValidUntil</i> The end date of the request. For a one-time request, the request remains active until all instances launch, the request is canceled, or this date is reached. If the request is persistent, it remains active until it is canceled or this date and time is reached. The default end date is 7 days from the current date.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BlockDurationMinutes</i> The required duration for the Spot Instances (also known as Spot blocks), in minutes. This value must be a multiple of 60 (60, 120, 180, 240, 300, or 360).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SpotInstanceType</i> The Spot Instance request type. For <a>RunInstances</a>, persistent Spot Instance requests are only supported when <b>InstanceInterruptionBehavior</b> is set to either <code>hibernate</code> or <code>stop</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxPrice</i> The maximum hourly price you're willing to pay for the Spot Instances. The default is the On-Demand price.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CpuCredits</i> The credit option for CPU usage of a T2 or T3 instance. Valid values are <code>standard</code> and <code>unlimited</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThreadsPerCore</i> The number of threads per CPU core. To disable Intel Hyper-Threading Technology for the instance, specify a value of <code>1</code>. Otherwise, specify the default value of <code>2</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CoreCount</i> The number of CPU cores for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CapacityReservationTarget</i> Information about the target Capacity Reservation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CapacityReservationTarget</b>: Information about the target Capacity Reservation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CapacityReservationPreference</i> Indicates the instance's Capacity Reservation preferences. Possible preferences include: <ul> <li>  <code>open</code> - The instance can run in any <code>open</code> Capacity Reservation that has matching attributes (instance type, platform, Availability Zone). </li> <li>  <code>none</code> - The instance avoids running in a Capacity Reservation even if one is available. The instance runs as an On-Demand Instance. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CapacityReservationId</i> The ID of the Capacity Reservation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Configured</i> If you set this parameter to <code>true</code>, your instance is enabled for hibernation. Default: <code>false</code> <br/>
        :return: ReservationId:  The ID of the reservation.<br/>RequesterId:  The ID of the requester that launched the instances on your behalf (for example, AWS Management Console or Auto Scaling).<br/>OwnerId:  The ID of the AWS account that owns the reservation.<br/>Instances:  The instances.<br/>Groups:  [EC2-Classic only] The security groups.<br/>        """
        pass

    def run_scheduled_instances(self, ScheduledInstanceId: str, LaunchSpecification: ScheduledInstancesLaunchSpecification, ClientToken: str=None, DryRun: Boolean=None, InstanceCount: int=None) -> RunScheduledInstancesResult:
        """Launches the specified Scheduled Instances. Before you can launch a Scheduled Instance, you must purchase it and obtain an identifier using <a>PurchaseScheduledInstances</a>. You must launch a Scheduled Instance during its scheduled time period. You can't stop or reboot a Scheduled Instance, but you can terminate it as needed. If you terminate a Scheduled Instance before the current scheduled time period ends, you can launch it again after a few minutes. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-scheduled-instances.html">Scheduled Instances</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param ScheduledInstanceId:  The Scheduled Instance ID.<br/>LaunchSpecification:  The launch specification. You must match the instance type, Availability Zone, network, and platform of the schedule that you purchased.:<br/>InstanceCount:  The number of instances. Default: 1<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>ClientToken:  Unique, case-sensitive identifier that ensures the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">Ensuring Idempotency</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserData</i> The base64-encoded MIME user data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetId</i> The ID of the subnet in which to launch the instances.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The IDs of the security groups.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RamdiskId</i> The ID of the RAM disk.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Placement</i> The placement information.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Placement</b>: The placement information.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkInterfaces</i> The network interfaces.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Monitoring</i> Enable or disable monitoring for the instances.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Monitoring</b>: Enable or disable monitoring for the instances.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyName</i> The name of the key pair.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KernelId</i> The ID of the kernel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i> The instance type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImageId</i> The ID of the Amazon Machine Image (AMI).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamInstanceProfile</i> The IAM instance profile.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>IamInstanceProfile</b>: The IAM instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EbsOptimized</i> Indicates whether the instances are optimized for EBS I/O. This optimization provides dedicated throughput to Amazon EBS and an optimized configuration stack to provide optimal EBS I/O performance. This optimization isn't available with all instance types. Additional usage charges apply when using an EBS-optimized instance. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BlockDeviceMappings</i> The block device mapping entries.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Indicates whether monitoring is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupName</i> The name of the placement group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone.<br/>
        :return: InstanceIdSet:  The IDs of the newly launched instances.<br/>        """
        pass

    def search_transit_gateway_routes(self, Filters: List[Filter], TransitGatewayRouteTableId: str, MaxResults: int=None, DryRun: Boolean=None) -> SearchTransitGatewayRoutesResult:
        """Searches for routes in the specified transit gateway route table.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>MaxResults:  The maximum number of routes to return.<br/>Filters:  One or more filters. The possible values are: <ul> <li>  <code>attachment.transit-gateway-attachment-id</code>- The id of the transit gateway attachment. </li> <li>  <code>attachment.resource-id</code> - The resource id of the transit gateway attachment. </li> <li>  <code>attachment.resource-type</code> - The attachment resource type (<code>vpc</code> | <code>vpn</code>). </li> <li>  <code>route-search.exact-match</code> - The exact match of the specified filter. </li> <li>  <code>route-search.longest-prefix-match</code> - The longest prefix that matches the route. </li> <li>  <code>route-search.subnet-of-match</code> - The routes with a subnet that match the specified CIDR filter. </li> <li>  <code>route-search.supernet-of-match</code> - The routes with a CIDR that encompass the CIDR filter. For example, if you have 10.0.1.0/29 and 10.0.1.0/31 routes in your route table and you specify supernet-of-match as 10.0.1.0/30, then the result returns 10.0.1.0/29. </li> <li>  <code>state</code> - The state of the route (<code>active</code> | <code>blackhole</code>). </li> <li>  <code>type</code> - The type of roue (<code>propagated</code> | <code>static</code>). </li> </ul><br/>TransitGatewayRouteTableId:  The ID of the transit gateway route table.<br/>
        :return: AdditionalRoutesAvailable:  Indicates whether there are additional routes available.<br/>Routes:  Information about the routes.<br/>        """
        pass

    def start_instances(self, InstanceIds: List[String], AdditionalInfo: str=None, DryRun: Boolean=None) -> StartInstancesResult:
        """Starts an Amazon EBS-backed instance that you've previously stopped. Instances that use Amazon EBS volumes as their root devices can be quickly stopped and started. When an instance is stopped, the compute resources are released and you are not billed for instance usage. However, your root partition Amazon EBS volume remains and continues to persist your data, and you are charged for Amazon EBS volume usage. You can restart your instance at any time. Every time you start your Windows instance, Amazon EC2 charges you for a full instance hour. If you stop and restart your Windows instance, a new instance hour begins and Amazon EC2 charges you for another full instance hour even if you are still within the same 60-minute period when it was stopped. Every time you start your Linux instance, Amazon EC2 charges a one-minute minimum for instance usage, and thereafter charges per second for instance usage. Before stopping an instance, make sure it is in a state from which it can be restarted. Stopping an instance does not preserve data stored in RAM. Performing this operation on an instance that uses an instance store as its root device returns an error. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Stop_Start.html">Stopping Instances</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>AdditionalInfo:  Reserved.<br/>InstanceIds:  The IDs of the instances.<br/>
        :return: StartingInstances:  Information about the started instances.<br/>        """
        pass

    def stop_instances(self, InstanceIds: List[String], Hibernate: Boolean=None, DryRun: Boolean=None, Force: Boolean=None) -> StopInstancesResult:
        """Stops an Amazon EBS-backed instance. You can use the Stop action to hibernate an instance if the instance is <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Hibernate.html#enabling-hibernation">enabled for hibernation</a> and it meets the <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Hibernate.html#hibernating-prerequisites">hibernation prerequisites</a>. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Hibernate.html">Hibernate Your Instance</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. We don't charge usage for a stopped instance, or data transfer fees; however, your root partition Amazon EBS volume remains and continues to persist your data, and you are charged for Amazon EBS volume usage. Every time you start your Windows instance, Amazon EC2 charges you for a full instance hour. If you stop and restart your Windows instance, a new instance hour begins and Amazon EC2 charges you for another full instance hour even if you are still within the same 60-minute period when it was stopped. Every time you start your Linux instance, Amazon EC2 charges a one-minute minimum for instance usage, and thereafter charges per second for instance usage. You can't start, stop, or hibernate Spot Instances, and you can't stop or hibernate instance store-backed instances. For information about using hibernation for Spot Instances, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/spot-interruptions.html#hibernate-spot-instances">Hibernating Interrupted Spot Instances</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. When you stop or hibernate an instance, we shut it down. You can restart your instance at any time. Before stopping or hibernating an instance, make sure it is in a state from which it can be restarted. Stopping an instance does not preserve data stored in RAM, but hibernating an instance does preserve data stored in RAM. If an instance cannot hibernate successfully, a normal shutdown occurs. Stopping and hibernating an instance is different to rebooting or terminating it. For example, when you stop or hibernate an instance, the root device and any other devices attached to the instance persist. When you terminate an instance, the root device and any other devices attached during the instance launch are automatically deleted. For more information about the differences between rebooting, stopping, hibernating, and terminating instances, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-instance-lifecycle.html">Instance Lifecycle</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. When you stop an instance, we attempt to shut it down forcibly after a short while. If your instance appears stuck in the stopping state after a period of time, there may be an issue with the underlying host computer. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/TroubleshootingInstancesStopping.html">Troubleshooting Stopping Your Instance</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param Force:  Forces the instances to stop. The instances do not have an opportunity to flush file system caches or file system metadata. If you use this option, you must perform file system check and repair procedures. This option is not recommended for Windows instances. Default: <code>false</code> <br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Hibernate:  Hibernates the instance if the instance was enabled for hibernation at launch. If the instance cannot hibernate successfully, a normal shutdown occurs. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Hibernate.html">Hibernate Your Instance</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.  Default: <code>false</code> <br/>InstanceIds:  The IDs of the instances.<br/>
        :return: StoppingInstances:  Information about the stopped instances.<br/>        """
        pass

    def terminate_client_vpn_connections(self, ClientVpnEndpointId: str, ConnectionId: str=None, Username: str=None, DryRun: Boolean=None) -> TerminateClientVpnConnectionsResult:
        """Terminates active Client VPN endpoint connections. This action can be used to terminate a specific client connection, or up to five connections established by a specific user.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Username:  The name of the user who initiated the connection. Use this option to terminate all active connections for the specified user. This option can only be used if the user has established up to five connections.<br/>ConnectionId:  The ID of the client connection to be terminated.<br/>ClientVpnEndpointId:  The ID of the Client VPN endpoint to which the client is connected.<br/>
        :return: ConnectionStatuses:  The current state of the client connections.<br/>Username:  The user who established the terminated client connections.<br/>ClientVpnEndpointId:  The ID of the Client VPN endpoint.<br/>        """
        pass

    def terminate_instances(self, InstanceIds: List[String], DryRun: Boolean=None) -> TerminateInstancesResult:
        """Shuts down the specified instances. This operation is idempotent; if you terminate an instance more than once, each call succeeds.  If you specify multiple instances and the request fails (for example, because of a single incorrect instance ID), none of the instances are terminated. Terminated instances remain visible after termination (for approximately one hour). By default, Amazon EC2 deletes all EBS volumes that were attached when the instance launched. Volumes attached after instance launch continue running. You can stop, start, and terminate EBS-backed instances. You can only terminate instance store-backed instances. What happens to an instance differs if you stop it or terminate it. For example, when you stop an instance, the root device and any other devices attached to the instance persist. When you terminate an instance, any attached EBS volumes with the <code>DeleteOnTermination</code> block device mapping parameter set to <code>true</code> are automatically deleted. For more information about the differences between stopping and terminating instances, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-instance-lifecycle.html">Instance Lifecycle</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. For more information about troubleshooting, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/TroubleshootingInstancesShuttingDown.html">Troubleshooting Terminating Your Instance</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>InstanceIds:  The IDs of the instances. Constraints: Up to 1000 instance IDs. We recommend breaking up this request into smaller batches.<br/>
        :return: TerminatingInstances:  Information about the terminated instances.<br/>        """
        pass

    def unassign_ipv6_addresses(self, NetworkInterfaceId: str, Ipv6Addresses: List[String]) -> UnassignIpv6AddressesResult:
        """Unassigns one or more IPv6 addresses from a network interface.

        :param self:
        :param NetworkInterfaceId:  The ID of the network interface.<br/>Ipv6Addresses:  The IPv6 addresses to unassign from the network interface.<br/>
        :return: UnassignedIpv6Addresses:  The IPv6 addresses that have been unassigned from the network interface.<br/>NetworkInterfaceId:  The ID of the network interface.<br/>        """
        pass

    def unassign_private_ip_addresses(self, PrivateIpAddresses: List[String], NetworkInterfaceId: str):
        """Unassigns one or more secondary private IP addresses from a network interface.

        :param self:
        :param PrivateIpAddresses:  The secondary private IP addresses to unassign from the network interface. You can specify this option multiple times to unassign more than one IP address.<br/>NetworkInterfaceId:  The ID of the network interface.<br/>
        :return:         """
        pass

    def unmonitor_instances(self, InstanceIds: List[String], DryRun: Boolean=None) -> UnmonitorInstancesResult:
        """Disables detailed monitoring for a running instance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/using-cloudwatch.html">Monitoring Your Instances and Volumes</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>InstanceIds:  The IDs of the instances.<br/>
        :return: InstanceMonitorings:  The monitoring information.<br/>        """
        pass

    def update_security_group_rule_descriptions_egress(self, IpPermissions: List[IpPermission], DryRun: Boolean=None, GroupId: str=None, GroupName: str=None) -> UpdateSecurityGroupRuleDescriptionsEgressResult:
        """[VPC only] Updates the description of an egress (outbound) security group rule. You can replace an existing description, or add a description to a rule that did not have one previously. You specify the description as part of the IP permissions structure. You can remove a description for a security group rule by omitting the description parameter in the request.

        :param self:
        :param IpPermissions:  The IP permissions for the security group rule.<br/>GroupName:  [Default VPC] The name of the security group. You must specify either the security group ID or the security group name in the request.<br/>GroupId:  The ID of the security group. You must specify either the security group ID or the security group name in the request. For security groups in a nondefault VPC, you must specify the security group ID.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: Return:  Returns <code>true</code> if the request succeeds; otherwise, returns an error.<br/>        """
        pass

    def update_security_group_rule_descriptions_ingress(self, IpPermissions: List[IpPermission], DryRun: Boolean=None, GroupId: str=None, GroupName: str=None) -> UpdateSecurityGroupRuleDescriptionsIngressResult:
        """Updates the description of an ingress (inbound) security group rule. You can replace an existing description, or add a description to a rule that did not have one previously. You specify the description as part of the IP permissions structure. You can remove a description for a security group rule by omitting the description parameter in the request.

        :param self:
        :param IpPermissions:  The IP permissions for the security group rule. <br/>GroupName:  [EC2-Classic, default VPC] The name of the security group. You must specify either the security group ID or the security group name in the request.<br/>GroupId:  The ID of the security group. You must specify either the security group ID or the security group name in the request. For security groups in a nondefault VPC, you must specify the security group ID.<br/>DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>
        :return: Return:  Returns <code>true</code> if the request succeeds; otherwise, returns an error.<br/>        """
        pass

    def withdraw_byoip_cidr(self, Cidr: str, DryRun: Boolean=None) -> WithdrawByoipCidrResult:
        """Stops advertising an IPv4 address range that is provisioned as an address pool. You can perform this operation at most once every 10 seconds, even if you specify different address ranges each time. It can take a few minutes before traffic to the specified addresses stops routing to AWS because of BGP propagation delays.

        :param self:
        :param DryRun:  Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.<br/>Cidr:  The public IPv4 address range, in CIDR notation.<br/>
        :return: ByoipCidr:  Information about the address pool.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the address pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> Upon success, contains the ID of the address pool. Otherwise, contains an error message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the address range.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Cidr</i> The public IPv4 address range, in CIDR notation.<br/>        """
        pass

    class Ec2Resource:
        
        class ClassicAddress:
            """Creates a ClassicAddress resource.::

  classic_address = ec2.ClassicAddress('public_ip')

:type public_ip: string
:param public_ip: The ClassicAddress's public_ip identifier. This **must** be set.

:rtype: :py:class:`EC2.ClassicAddress`
:returns: A ClassicAddress resource"""

            def __init__(self, public_ip):
                pass
            
            class Tag(Mapping):
                pass
        
            instance_id: str=None
            public_ip: str=None
            allocation_id: str=None
            association_id: str=None
            domain: str=None
            network_interface_id: str=None
            network_interface_owner_id: str=None
            private_ip_address: str=None
            tags: List[Tag]=None
            public_ipv4_pool: str=None
            
            def associate(self, **kwargs):
                # type: (object) -> dict
                return {}

            def disassociate(self, **kwargs):
                # type: (object) -> dict
                return {}

            def release(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class DhcpOptions:
            """Creates a DhcpOptions resource.::

  dhcp_options = ec2.DhcpOptions('id')

:type id: string
:param id: The DhcpOptions's id identifier. This **must** be set.

:rtype: :py:class:`EC2.DhcpOptions`
:returns: A DhcpOptions resource"""

            def __init__(self, id):
                pass
            
            class DhcpConfiguration(Mapping):
                pass

            class Tag(Mapping):
                pass
        
            dhcp_configurations: List[DhcpConfiguration]=None
            dhcp_options_id: str=None
            owner_id: str=None
            tags: List[Tag]=None
            
            def associate_with_vpc(self, **kwargs):
                # type: (object) -> dict
                return {}

            def create_tags(self, **kwargs):
                # type: (object) -> Tag
                return Tag()

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class Image:
            """Creates a Image resource.::

  image = ec2.Image('id')

:type id: string
:param id: The Image's id identifier. This **must** be set.

:rtype: :py:class:`EC2.Image`
:returns: A Image resource"""

            def __init__(self, id):
                pass
            
            class ProductCode(Mapping):
                pass

            class BlockDeviceMapping(Mapping):
                pass

            class StateReason(Mapping):
                pass

            class Boolean(object):
                pass

            class Tag(Mapping):
                pass
        
            architecture: str=None
            creation_date: str=None
            image_id: str=None
            image_location: str=None
            image_type: str=None
            public: Boolean=None
            kernel_id: str=None
            owner_id: str=None
            platform: str=None
            product_codes: List[ProductCode]=None
            ramdisk_id: str=None
            state: str=None
            block_device_mappings: List[BlockDeviceMapping]=None
            description: str=None
            ena_support: Boolean=None
            hypervisor: str=None
            image_owner_alias: str=None
            name: str=None
            root_device_name: str=None
            root_device_type: str=None
            sriov_net_support: str=None
            state_reason: StateReason=None
            tags: List[Tag]=None
            virtualization_type: str=None
            
            def wait_until_exists(self):
                pass

            def create_tags(self, **kwargs):
                # type: (object) -> Tag
                return Tag()

            def deregister(self, **kwargs):
                # type: (object) -> dict
                return {}

            def describe_attribute(self, **kwargs):
                # type: (object) -> dict
                return {}

            def modify_attribute(self, **kwargs):
                # type: (object) -> dict
                return {}

            def reset_attribute(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class Instance:
            """Creates a Instance resource.::

  instance = ec2.Instance('id')

:type id: string
:param id: The Instance's id identifier. This **must** be set.

:rtype: :py:class:`EC2.Instance`
:returns: A Instance resource"""

            def __init__(self, id):
                pass
            
            class ProductCode(Mapping):
                pass

            class InstanceBlockDeviceMapping(Mapping):
                pass

            class CpuOptions(Mapping):
                pass

            class LicenseConfiguration(Mapping):
                pass

            class ElasticGpuAssociation(Mapping):
                pass

            class Tag(Mapping):
                pass

            class InstanceState(Mapping):
                pass

            class DateTime(object):
                pass

            class GroupIdentifier(Mapping):
                pass

            class IamInstanceProfile(Mapping):
                pass

            class Placement(Mapping):
                pass

            class Monitoring(Mapping):
                pass

            class InstanceNetworkInterface(Mapping):
                pass

            class StateReason(Mapping):
                pass

            class HibernationOptions(Mapping):
                pass

            class ElasticInferenceAcceleratorAssociation(Mapping):
                pass

            class Boolean(object):
                pass

            class CapacityReservationSpecificationResponse(Mapping):
                pass
        
            ami_launch_index: int=None
            image_id: str=None
            instance_id: str=None
            instance_type: str=None
            kernel_id: str=None
            key_name: str=None
            launch_time: DateTime=None
            monitoring: Monitoring=None
            placement: Placement=None
            platform: str=None
            private_dns_name: str=None
            private_ip_address: str=None
            product_codes: List[ProductCode]=None
            public_dns_name: str=None
            public_ip_address: str=None
            ramdisk_id: str=None
            state: InstanceState=None
            state_transition_reason: str=None
            subnet_id: str=None
            vpc_id: str=None
            architecture: str=None
            block_device_mappings: List[InstanceBlockDeviceMapping]=None
            client_token: str=None
            ebs_optimized: Boolean=None
            ena_support: Boolean=None
            hypervisor: str=None
            iam_instance_profile: IamInstanceProfile=None
            instance_lifecycle: str=None
            elastic_gpu_associations: List[ElasticGpuAssociation]=None
            elastic_inference_accelerator_associations: List[ElasticInferenceAcceleratorAssociation]=None
            network_interfaces: List[InstanceNetworkInterface]=None
            root_device_name: str=None
            root_device_type: str=None
            security_groups: List[GroupIdentifier]=None
            source_dest_check: Boolean=None
            spot_instance_request_id: str=None
            sriov_net_support: str=None
            state_reason: StateReason=None
            tags: List[Tag]=None
            virtualization_type: str=None
            cpu_options: CpuOptions=None
            capacity_reservation_id: str=None
            capacity_reservation_specification: CapacityReservationSpecificationResponse=None
            hibernation_options: HibernationOptions=None
            licenses: List[LicenseConfiguration]=None
            
            def wait_until_exists(self):
                pass

            def wait_until_running(self):
                pass

            def wait_until_stopped(self):
                pass

            def wait_until_terminated(self):
                pass

            def attach_classic_link_vpc(self, **kwargs):
                # type: (object) -> dict
                return {}

            def attach_volume(self, **kwargs):
                # type: (object) -> dict
                return {}

            def console_output(self, **kwargs):
                # type: (object) -> dict
                return {}

            def create_image(self, **kwargs):
                # type: (object) -> Image
                return Image()

            def create_tags(self, **kwargs):
                # type: (object) -> Tag
                return Tag()

            def describe_attribute(self, **kwargs):
                # type: (object) -> dict
                return {}

            def detach_classic_link_vpc(self, **kwargs):
                # type: (object) -> dict
                return {}

            def detach_volume(self, **kwargs):
                # type: (object) -> dict
                return {}

            def modify_attribute(self, **kwargs):
                # type: (object) -> dict
                return {}

            def monitor(self, **kwargs):
                # type: (object) -> dict
                return {}

            def password_data(self, **kwargs):
                # type: (object) -> dict
                return {}

            def reboot(self, **kwargs):
                # type: (object) -> dict
                return {}

            def report_status(self, **kwargs):
                # type: (object) -> dict
                return {}

            def reset_attribute(self, **kwargs):
                # type: (object) -> dict
                return {}

            def reset_kernel(self, **kwargs):
                # type: (object) -> dict
                return {}

            def reset_ramdisk(self, **kwargs):
                # type: (object) -> dict
                return {}

            def reset_source_dest_check(self, **kwargs):
                # type: (object) -> dict
                return {}

            def start(self, **kwargs):
                # type: (object) -> dict
                return {}

            def stop(self, **kwargs):
                # type: (object) -> dict
                return {}

            def terminate(self, **kwargs):
                # type: (object) -> dict
                return {}

            def unmonitor(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class InternetGateway:
            """Creates a InternetGateway resource.::

  internet_gateway = ec2.InternetGateway('id')

:type id: string
:param id: The InternetGateway's id identifier. This **must** be set.

:rtype: :py:class:`EC2.InternetGateway`
:returns: A InternetGateway resource"""

            def __init__(self, id):
                pass
            
            class Tag(Mapping):
                pass

            class InternetGatewayAttachment(Mapping):
                pass
        
            attachments: List[InternetGatewayAttachment]=None
            internet_gateway_id: str=None
            owner_id: str=None
            tags: List[Tag]=None
            
            def attach_to_vpc(self, **kwargs):
                # type: (object) -> dict
                return {}

            def create_tags(self, **kwargs):
                # type: (object) -> Tag
                return Tag()

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def detach_from_vpc(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class KeyPair:
            """Creates a KeyPairInfo resource.::

  key_pair_info = ec2.KeyPair('name')

:type name: string
:param name: The KeyPair's name identifier. This **must** be set.

:rtype: :py:class:`EC2.KeyPairInfo`
:returns: A KeyPairInfo resource"""

            def __init__(self, name):
                pass
            
        
            key_fingerprint: str=None
            key_name: str=None
            
            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class NetworkAcl:
            """Creates a NetworkAcl resource.::

  network_acl = ec2.NetworkAcl('id')

:type id: string
:param id: The NetworkAcl's id identifier. This **must** be set.

:rtype: :py:class:`EC2.NetworkAcl`
:returns: A NetworkAcl resource"""

            def __init__(self, id):
                pass
            
            class Tag(Mapping):
                pass

            class Boolean(object):
                pass

            class NetworkAclAssociation(Mapping):
                pass

            class NetworkAclEntry(Mapping):
                pass
        
            associations: List[NetworkAclAssociation]=None
            entries: List[NetworkAclEntry]=None
            is_default: Boolean=None
            network_acl_id: str=None
            tags: List[Tag]=None
            vpc_id: str=None
            owner_id: str=None
            
            def create_entry(self, **kwargs):
                # type: (object) -> dict
                return {}

            def create_tags(self, **kwargs):
                # type: (object) -> Tag
                return Tag()

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def delete_entry(self, **kwargs):
                # type: (object) -> dict
                return {}

            def replace_association(self, **kwargs):
                # type: (object) -> dict
                return {}

            def replace_entry(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class NetworkInterface:
            """Creates a NetworkInterface resource.::

  network_interface = ec2.NetworkInterface('id')

:type id: string
:param id: The NetworkInterface's id identifier. This **must** be set.

:rtype: :py:class:`EC2.NetworkInterface`
:returns: A NetworkInterface resource"""

            def __init__(self, id):
                pass
            
            class Tag(Mapping):
                pass

            class NetworkInterfaceAttachment(Mapping):
                pass

            class GroupIdentifier(Mapping):
                pass

            class NetworkInterfaceAssociation(Mapping):
                pass

            class NetworkInterfacePrivateIpAddress(Mapping):
                pass

            class Boolean(object):
                pass

            class NetworkInterfaceIpv6Address(Mapping):
                pass
        
            association: NetworkInterfaceAssociation=None
            attachment: NetworkInterfaceAttachment=None
            availability_zone: str=None
            description: str=None
            groups: List[GroupIdentifier]=None
            interface_type: str=None
            ipv6_addresses: List[NetworkInterfaceIpv6Address]=None
            mac_address: str=None
            network_interface_id: str=None
            owner_id: str=None
            private_dns_name: str=None
            private_ip_address: str=None
            private_ip_addresses: List[NetworkInterfacePrivateIpAddress]=None
            requester_id: str=None
            requester_managed: Boolean=None
            source_dest_check: Boolean=None
            status: str=None
            subnet_id: str=None
            tag_set: List[Tag]=None
            vpc_id: str=None
            
            def assign_private_ip_addresses(self, **kwargs):
                # type: (object) -> dict
                return {}

            def attach(self, **kwargs):
                # type: (object) -> dict
                return {}

            def create_tags(self, **kwargs):
                # type: (object) -> Tag
                return Tag()

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def describe_attribute(self, **kwargs):
                # type: (object) -> dict
                return {}

            def detach(self, **kwargs):
                # type: (object) -> dict
                return {}

            def modify_attribute(self, **kwargs):
                # type: (object) -> dict
                return {}

            def reset_attribute(self, **kwargs):
                # type: (object) -> dict
                return {}

            def unassign_private_ip_addresses(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class NetworkInterfaceAssociation:
            """Creates a NetworkInterfaceAssociation resource.::

  network_interface_association = ec2.NetworkInterfaceAssociation('id')

:type id: string
:param id: The NetworkInterfaceAssociation's id identifier. This **must** be set.

:rtype: :py:class:`EC2.NetworkInterfaceAssociation`
:returns: A NetworkInterfaceAssociation resource"""

            def __init__(self, id):
                pass
            
        
            ip_owner_id: str=None
            public_dns_name: str=None
            public_ip: str=None
            
            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class PlacementGroup:
            """Creates a PlacementGroup resource.::

  placement_group = ec2.PlacementGroup('name')

:type name: string
:param name: The PlacementGroup's name identifier. This **must** be set.

:rtype: :py:class:`EC2.PlacementGroup`
:returns: A PlacementGroup resource"""

            def __init__(self, name):
                pass
            
        
            group_name: str=None
            state: str=None
            strategy: str=None
            partition_count: int=None
            
            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class Route:
            """Creates a Route resource.::

  route = ec2.Route('route_table_id','destination_cidr_block')

:type route_table_id: string
:param route_table_id: The Route's route_table_id identifier. This **must** be set.
:type destination_cidr_block: string
:param destination_cidr_block: The Route's destination_cidr_block identifier. This **must** be set.

:rtype: :py:class:`EC2.Route`
:returns: A Route resource"""

            def __init__(self, route_table_id, destination_cidr_block):
                pass
            
        
            destination_cidr_block: str=None
            destination_ipv6_cidr_block: str=None
            destination_prefix_list_id: str=None
            egress_only_internet_gateway_id: str=None
            gateway_id: str=None
            instance_id: str=None
            instance_owner_id: str=None
            nat_gateway_id: str=None
            transit_gateway_id: str=None
            network_interface_id: str=None
            origin: str=None
            state: str=None
            vpc_peering_connection_id: str=None
            
            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def replace(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class RouteTable:
            """Creates a RouteTable resource.::

  route_table = ec2.RouteTable('id')

:type id: string
:param id: The RouteTable's id identifier. This **must** be set.

:rtype: :py:class:`EC2.RouteTable`
:returns: A RouteTable resource"""

            def __init__(self, id):
                pass
            
            class Tag(Mapping):
                pass

            class Route(Mapping):
                pass

            class PropagatingVgw(Mapping):
                pass

            class RouteTableAssociation(Mapping):
                pass
        
            associations: List[RouteTableAssociation]=None
            propagating_vgws: List[PropagatingVgw]=None
            route_table_id: str=None
            routes: List[Route]=None
            tags: List[Tag]=None
            vpc_id: str=None
            owner_id: str=None
            
            def associate_with_subnet(self, **kwargs):
                # type: (object) -> RouteTableAssociation
                return RouteTableAssociation()

            def create_route(self, **kwargs):
                # type: (object) -> Route
                return Route()

            def create_tags(self, **kwargs):
                # type: (object) -> Tag
                return Tag()

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class RouteTableAssociation:
            """Creates a RouteTableAssociation resource.::

  route_table_association = ec2.RouteTableAssociation('id')

:type id: string
:param id: The RouteTableAssociation's id identifier. This **must** be set.

:rtype: :py:class:`EC2.RouteTableAssociation`
:returns: A RouteTableAssociation resource"""

            def __init__(self, id):
                pass
            
            class Boolean(object):
                pass
        
            main: Boolean=None
            route_table_association_id: str=None
            route_table_id: str=None
            subnet_id: str=None
            
            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def replace_subnet(self, **kwargs):
                # type: (object) -> RouteTableAssociation
                return RouteTableAssociation()


        
        class SecurityGroup:
            """Creates a SecurityGroup resource.::

  security_group = ec2.SecurityGroup('id')

:type id: string
:param id: The SecurityGroup's id identifier. This **must** be set.

:rtype: :py:class:`EC2.SecurityGroup`
:returns: A SecurityGroup resource"""

            def __init__(self, id):
                pass
            
            class Tag(Mapping):
                pass

            class IpPermission(Mapping):
                pass
        
            description: str=None
            group_name: str=None
            ip_permissions: List[IpPermission]=None
            owner_id: str=None
            group_id: str=None
            ip_permissions_egress: List[IpPermission]=None
            tags: List[Tag]=None
            vpc_id: str=None
            
            def authorize_egress(self, **kwargs):
                # type: (object) -> dict
                return {}

            def authorize_ingress(self, **kwargs):
                # type: (object) -> dict
                return {}

            def create_tags(self, **kwargs):
                # type: (object) -> Tag
                return Tag()

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def revoke_egress(self, **kwargs):
                # type: (object) -> dict
                return {}

            def revoke_ingress(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class Snapshot:
            """Creates a Snapshot resource.::

  snapshot = ec2.Snapshot('id')

:type id: string
:param id: The Snapshot's id identifier. This **must** be set.

:rtype: :py:class:`EC2.Snapshot`
:returns: A Snapshot resource"""

            def __init__(self, id):
                pass
            
            class Tag(Mapping):
                pass

            class Boolean(object):
                pass

            class DateTime(object):
                pass
        
            data_encryption_key_id: str=None
            description: str=None
            encrypted: Boolean=None
            kms_key_id: str=None
            owner_id: str=None
            progress: str=None
            snapshot_id: str=None
            start_time: DateTime=None
            state: str=None
            state_message: str=None
            volume_id: str=None
            volume_size: int=None
            owner_alias: str=None
            tags: List[Tag]=None
            
            def wait_until_completed(self):
                pass

            def copy(self, **kwargs):
                # type: (object) -> dict
                return {}

            def create_tags(self, **kwargs):
                # type: (object) -> Tag
                return Tag()

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def describe_attribute(self, **kwargs):
                # type: (object) -> dict
                return {}

            def modify_attribute(self, **kwargs):
                # type: (object) -> dict
                return {}

            def reset_attribute(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class Subnet:
            """Creates a Subnet resource.::

  subnet = ec2.Subnet('id')

:type id: string
:param id: The Subnet's id identifier. This **must** be set.

:rtype: :py:class:`EC2.Subnet`
:returns: A Subnet resource"""

            def __init__(self, id):
                pass
            
            class Tag(Mapping):
                pass

            class Boolean(object):
                pass

            class SubnetIpv6CidrBlockAssociation(Mapping):
                pass
        
            availability_zone: str=None
            availability_zone_id: str=None
            available_ip_address_count: int=None
            cidr_block: str=None
            default_for_az: Boolean=None
            map_public_ip_on_launch: Boolean=None
            state: str=None
            subnet_id: str=None
            vpc_id: str=None
            owner_id: str=None
            assign_ipv6_address_on_creation: Boolean=None
            ipv6_cidr_block_association_set: List[SubnetIpv6CidrBlockAssociation]=None
            tags: List[Tag]=None
            subnet_arn: str=None
            
            def create_instances(self, **kwargs):
                # type: (object) -> Instance
                return Instance()

            def create_network_interface(self, **kwargs):
                # type: (object) -> NetworkInterface
                return NetworkInterface()

            def create_tags(self, **kwargs):
                # type: (object) -> Tag
                return Tag()

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class Tag:
            """Creates a Tag resource.::

  tag = ec2.Tag('resource_id','key','value')

:type resource_id: string
:param resource_id: The Tag's resource_id identifier. This **must** be set.
:type key: string
:param key: The Tag's key identifier. This **must** be set.
:type value: string
:param value: The Tag's value identifier. This **must** be set.

:rtype: :py:class:`EC2.Tag`
:returns: A Tag resource"""

            def __init__(self, resource_id, key, value):
                pass
            
        
            key: str=None
            resource_id: str=None
            resource_type: str=None
            value: str=None
            
            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class Volume:
            """Creates a Volume resource.::

  volume = ec2.Volume('id')

:type id: string
:param id: The Volume's id identifier. This **must** be set.

:rtype: :py:class:`EC2.Volume`
:returns: A Volume resource"""

            def __init__(self, id):
                pass
            
            class Tag(Mapping):
                pass

            class Boolean(object):
                pass

            class DateTime(object):
                pass

            class VolumeAttachment(Mapping):
                pass
        
            attachments: List[VolumeAttachment]=None
            availability_zone: str=None
            create_time: DateTime=None
            encrypted: Boolean=None
            kms_key_id: str=None
            size: int=None
            snapshot_id: str=None
            state: str=None
            volume_id: str=None
            iops: int=None
            tags: List[Tag]=None
            volume_type: str=None
            
            def attach_to_instance(self, **kwargs):
                # type: (object) -> dict
                return {}

            def create_snapshot(self, **kwargs):
                # type: (object) -> Snapshot
                return Snapshot()

            def create_tags(self, **kwargs):
                # type: (object) -> Tag
                return Tag()

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def describe_attribute(self, **kwargs):
                # type: (object) -> dict
                return {}

            def describe_status(self, **kwargs):
                # type: (object) -> dict
                return {}

            def detach_from_instance(self, **kwargs):
                # type: (object) -> dict
                return {}

            def enable_io(self, **kwargs):
                # type: (object) -> dict
                return {}

            def modify_attribute(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class Vpc:
            """Creates a Vpc resource.::

  vpc = ec2.Vpc('id')

:type id: string
:param id: The Vpc's id identifier. This **must** be set.

:rtype: :py:class:`EC2.Vpc`
:returns: A Vpc resource"""

            def __init__(self, id):
                pass
            
            class Tag(Mapping):
                pass

            class Boolean(object):
                pass

            class VpcCidrBlockAssociation(Mapping):
                pass

            class VpcIpv6CidrBlockAssociation(Mapping):
                pass
        
            cidr_block: str=None
            dhcp_options_id: str=None
            state: str=None
            vpc_id: str=None
            owner_id: str=None
            instance_tenancy: str=None
            ipv6_cidr_block_association_set: List[VpcIpv6CidrBlockAssociation]=None
            cidr_block_association_set: List[VpcCidrBlockAssociation]=None
            is_default: Boolean=None
            tags: List[Tag]=None
            
            def wait_until_available(self):
                pass

            def wait_until_exists(self):
                pass

            def associate_dhcp_options(self, **kwargs):
                # type: (object) -> dict
                return {}

            def attach_classic_link_instance(self, **kwargs):
                # type: (object) -> dict
                return {}

            def attach_internet_gateway(self, **kwargs):
                # type: (object) -> dict
                return {}

            def create_network_acl(self, **kwargs):
                # type: (object) -> NetworkAcl
                return NetworkAcl()

            def create_route_table(self, **kwargs):
                # type: (object) -> RouteTable
                return RouteTable()

            def create_security_group(self, **kwargs):
                # type: (object) -> SecurityGroup
                return SecurityGroup()

            def create_subnet(self, **kwargs):
                # type: (object) -> Subnet
                return Subnet()

            def create_tags(self, **kwargs):
                # type: (object) -> Tag
                return Tag()

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def describe_attribute(self, **kwargs):
                # type: (object) -> dict
                return {}

            def detach_classic_link_instance(self, **kwargs):
                # type: (object) -> dict
                return {}

            def detach_internet_gateway(self, **kwargs):
                # type: (object) -> dict
                return {}

            def disable_classic_link(self, **kwargs):
                # type: (object) -> dict
                return {}

            def enable_classic_link(self, **kwargs):
                # type: (object) -> dict
                return {}

            def modify_attribute(self, **kwargs):
                # type: (object) -> dict
                return {}

            def request_vpc_peering_connection(self, **kwargs):
                # type: (object) -> VpcPeeringConnection
                return VpcPeeringConnection()


        
        class VpcPeeringConnection:
            """Creates a VpcPeeringConnection resource.::

  vpc_peering_connection = ec2.VpcPeeringConnection('id')

:type id: string
:param id: The VpcPeeringConnection's id identifier. This **must** be set.

:rtype: :py:class:`EC2.VpcPeeringConnection`
:returns: A VpcPeeringConnection resource"""

            def __init__(self, id):
                pass
            
            class VpcPeeringConnectionStateReason(Mapping):
                pass

            class Tag(Mapping):
                pass

            class DateTime(object):
                pass

            class VpcPeeringConnectionVpcInfo(Mapping):
                pass
        
            accepter_vpc_info: VpcPeeringConnectionVpcInfo=None
            expiration_time: DateTime=None
            requester_vpc_info: VpcPeeringConnectionVpcInfo=None
            status: VpcPeeringConnectionStateReason=None
            tags: List[Tag]=None
            vpc_peering_connection_id: str=None
            
            def wait_until_exists(self):
                pass

            def accept(self, **kwargs):
                # type: (object) -> dict
                return {}

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def reject(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class VpcAddress:
            """Creates a VpcAddress resource.::

  vpc_address = ec2.VpcAddress('allocation_id')

:type allocation_id: string
:param allocation_id: The VpcAddress's allocation_id identifier. This **must** be set.

:rtype: :py:class:`EC2.VpcAddress`
:returns: A VpcAddress resource"""

            def __init__(self, allocation_id):
                pass
            
            class Tag(Mapping):
                pass
        
            instance_id: str=None
            public_ip: str=None
            allocation_id: str=None
            association_id: str=None
            domain: str=None
            network_interface_id: str=None
            network_interface_owner_id: str=None
            private_ip_address: str=None
            tags: List[Tag]=None
            public_ipv4_pool: str=None
            
            def associate(self, **kwargs):
                # type: (object) -> dict
                return {}

            def release(self, **kwargs):
                # type: (object) -> dict
                return {}


        def create_dhcp_options(self, **kwargs):
            # type: (object) -> DhcpOptions
            pass

        def create_instances(self, **kwargs):
            # type: (object) -> Instance
            pass

        def create_internet_gateway(self, **kwargs):
            # type: (object) -> InternetGateway
            pass

        def create_key_pair(self, **kwargs):
            # type: (object) -> KeyPair
            pass

        def create_network_acl(self, **kwargs):
            # type: (object) -> NetworkAcl
            pass

        def create_network_interface(self, **kwargs):
            # type: (object) -> NetworkInterface
            pass

        def create_placement_group(self, **kwargs):
            # type: (object) -> PlacementGroup
            pass

        def create_route_table(self, **kwargs):
            # type: (object) -> RouteTable
            pass

        def create_security_group(self, **kwargs):
            # type: (object) -> SecurityGroup
            pass

        def create_snapshot(self, **kwargs):
            # type: (object) -> Snapshot
            pass

        def create_subnet(self, **kwargs):
            # type: (object) -> Subnet
            pass

        def create_tags(self, **kwargs):
            # type: (object) -> dict
            pass

        def create_volume(self, **kwargs):
            # type: (object) -> Volume
            pass

        def create_vpc(self, **kwargs):
            # type: (object) -> Vpc
            pass

        def create_vpc_peering_connection(self, **kwargs):
            # type: (object) -> VpcPeeringConnection
            pass

        def disassociate_route_table(self, **kwargs):
            # type: (object) -> dict
            pass

        def import_key_pair(self, **kwargs):
            # type: (object) -> KeyPairInfo
            pass

        def register_image(self, **kwargs):
            # type: (object) -> Image
            pass

        def get_available_subresources(self) -> List[str]:
            pass

        class ClassicAddressResourceCollection(List[ClassicAddress], ResourceCollection):
            pass

        classic_addresses: ClassicAddressResourceCollection = None
        class DhcpOptionsResourceCollection(List[DhcpOptions], ResourceCollection):
            pass

        dhcp_options_sets: DhcpOptionsResourceCollection = None
        class ImageResourceCollection(List[Image], ResourceCollection):
            pass

        images: ImageResourceCollection = None
        class InstanceResourceCollection(List[Instance], ResourceCollection):
            pass

        instances: InstanceResourceCollection = None
        class InternetGatewayResourceCollection(List[InternetGateway], ResourceCollection):
            pass

        internet_gateways: InternetGatewayResourceCollection = None
        class KeyPairInfoResourceCollection(List[KeyPair], ResourceCollection):
            pass

        key_pairs: KeyPairInfoResourceCollection = None
        class NetworkAclResourceCollection(List[NetworkAcl], ResourceCollection):
            pass

        network_acls: NetworkAclResourceCollection = None
        class NetworkInterfaceResourceCollection(List[NetworkInterface], ResourceCollection):
            pass

        network_interfaces: NetworkInterfaceResourceCollection = None
        class PlacementGroupResourceCollection(List[PlacementGroup], ResourceCollection):
            pass

        placement_groups: PlacementGroupResourceCollection = None
        class RouteTableResourceCollection(List[RouteTable], ResourceCollection):
            pass

        route_tables: RouteTableResourceCollection = None
        class SecurityGroupResourceCollection(List[SecurityGroup], ResourceCollection):
            pass

        security_groups: SecurityGroupResourceCollection = None
        class SnapshotResourceCollection(List[Snapshot], ResourceCollection):
            pass

        snapshots: SnapshotResourceCollection = None
        class SubnetResourceCollection(List[Subnet], ResourceCollection):
            pass

        subnets: SubnetResourceCollection = None
        class VolumeResourceCollection(List[Volume], ResourceCollection):
            pass

        volumes: VolumeResourceCollection = None
        class VpcAddressResourceCollection(List[VpcAddress], ResourceCollection):
            pass

        vpc_addresses: VpcAddressResourceCollection = None
        class VpcPeeringConnectionResourceCollection(List[VpcPeeringConnection], ResourceCollection):
            pass

        vpc_peering_connections: VpcPeeringConnectionResourceCollection = None
        class VpcResourceCollection(List[Vpc], ResourceCollection):
            pass

        vpcs: VpcResourceCollection = None




class EC2InstanceConnect(BaseClient):
    """AWS EC2 Connect Service is a service that enables system administrators to publish temporary SSH keys to their EC2 instances in order to establish connections to their instances without leaving a permanent authentication option."""




    class SendSSHPublicKeyRequest(Mapping):
        pass
    
    class SendSSHPublicKeyResponse(Mapping):
        pass
    
    def send_ssh_public_key(self, AvailabilityZone: str, SSHPublicKey: str, InstanceOSUser: str, InstanceId: str) -> SendSSHPublicKeyResponse:
        """Pushes an SSH public key to a particular OS user on a given EC2 instance for 60 seconds.

        :param self:
        :param AvailabilityZone:  The availability zone the EC2 instance was launched in.<br/>SSHPublicKey:  The public key to be published to the instance. To use it after publication you must have the matching private key.<br/>InstanceOSUser:  The OS user on the EC2 instance whom the key may be used to authenticate as.<br/>InstanceId:  The EC2 instance you wish to publish the SSH key to.<br/>
        :return: Success:  Indicates request success.<br/>RequestId:  The request ID as logged by EC2 Connect. Please provide this when contacting AWS Support.<br/>        """
        pass



class ECR(BaseClient):
    """Amazon Elastic Container Registry (Amazon ECR) is a managed Docker registry service. Customers can use the familiar Docker CLI to push, pull, and manage images. Amazon ECR provides a secure, scalable, and reliable registry. Amazon ECR supports private Docker repositories with resource-based permissions using IAM so that specific users or Amazon EC2 instances can access repositories and images. Developers can use the Docker CLI to author and manage images."""


    class ListImagesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_images` for valid parameters"""
            pass
    class DescribeImagesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_images` for valid parameters"""
            pass
    class DescribeRepositoriesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_repositories` for valid parameters"""
            pass


    class BatchCheckLayerAvailabilityRequest(Mapping):
        pass
    
    class BatchCheckLayerAvailabilityResponse(Mapping):
        pass
    
    class BatchedOperationLayerDigest(object):
        pass
    
    class BatchDeleteImageRequest(Mapping):
        pass
    
    class BatchDeleteImageResponse(Mapping):
        pass
    
    class ImageIdentifier(Mapping):
        pass
    
    class BatchGetImageRequest(Mapping):
        pass
    
    class BatchGetImageResponse(Mapping):
        pass
    
    class MediaType(object):
        pass
    
    class CompleteLayerUploadRequest(Mapping):
        pass
    
    class CompleteLayerUploadResponse(Mapping):
        pass
    
    class LayerDigest(object):
        pass
    
    class CreateRepositoryRequest(Mapping):
        pass
    
    class CreateRepositoryResponse(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class DeleteLifecyclePolicyRequest(Mapping):
        pass
    
    class DeleteLifecyclePolicyResponse(Mapping):
        pass
    
    class DeleteRepositoryRequest(Mapping):
        pass
    
    class DeleteRepositoryResponse(Mapping):
        pass
    
    class ForceFlag(object):
        pass
    
    class DeleteRepositoryPolicyRequest(Mapping):
        pass
    
    class DeleteRepositoryPolicyResponse(Mapping):
        pass
    
    class DescribeImagesRequest(Mapping):
        pass
    
    class DescribeImagesResponse(Mapping):
        pass
    
    class DescribeImagesFilter(Mapping):
        pass
    
    class DescribeRepositoriesRequest(Mapping):
        pass
    
    class DescribeRepositoriesResponse(Mapping):
        pass
    
    class RepositoryName(object):
        pass
    
    class GetAuthorizationTokenRequest(Mapping):
        pass
    
    class GetAuthorizationTokenResponse(Mapping):
        pass
    
    class RegistryId(object):
        pass
    
    class GetDownloadUrlForLayerRequest(Mapping):
        pass
    
    class GetDownloadUrlForLayerResponse(Mapping):
        pass
    
    class GetLifecyclePolicyRequest(Mapping):
        pass
    
    class GetLifecyclePolicyResponse(Mapping):
        pass
    
    class GetLifecyclePolicyPreviewRequest(Mapping):
        pass
    
    class GetLifecyclePolicyPreviewResponse(Mapping):
        pass
    
    class LifecyclePolicyPreviewFilter(Mapping):
        pass
    
    class GetRepositoryPolicyRequest(Mapping):
        pass
    
    class GetRepositoryPolicyResponse(Mapping):
        pass
    
    class InitiateLayerUploadRequest(Mapping):
        pass
    
    class InitiateLayerUploadResponse(Mapping):
        pass
    
    class ListImagesRequest(Mapping):
        pass
    
    class ListImagesResponse(Mapping):
        pass
    
    class ListImagesFilter(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class PutImageRequest(Mapping):
        pass
    
    class PutImageResponse(Mapping):
        pass
    
    class PutLifecyclePolicyRequest(Mapping):
        pass
    
    class PutLifecyclePolicyResponse(Mapping):
        pass
    
    class SetRepositoryPolicyRequest(Mapping):
        pass
    
    class SetRepositoryPolicyResponse(Mapping):
        pass
    
    class StartLifecyclePolicyPreviewRequest(Mapping):
        pass
    
    class StartLifecyclePolicyPreviewResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UploadLayerPartRequest(Mapping):
        pass
    
    class UploadLayerPartResponse(Mapping):
        pass
    
    class PartSize(object):
        pass
    
    class LayerPartBlob(object):
        pass
    
    def batch_check_layer_availability(self, layerDigests: List[BatchedOperationLayerDigest], repositoryName: str, registryId: str=None) -> BatchCheckLayerAvailabilityResponse:
        """Check the availability of multiple image layers in a specified registry and repository. <note> This operation is used by the Amazon ECR proxy, and it is not intended for general use by customers for pulling and pushing images. In most cases, you should use the <code>docker</code> CLI to pull, tag, and push images. </note>

        :param self:
        :param layerDigests:  The digests of the image layers to check.<br/>repositoryName:  The name of the repository that is associated with the image layers to check.<br/>registryId:  The AWS account ID associated with the registry that contains the image layers to check. If you do not specify a registry, the default registry is assumed.<br/>
        :return: failures:  Any failures associated with the call.<br/>layers:  A list of image layer objects corresponding to the image layer references in the request.<br/>        """
        pass

    def batch_delete_image(self, imageIds: List[ImageIdentifier], repositoryName: str, registryId: str=None) -> BatchDeleteImageResponse:
        """Deletes a list of specified images within a specified repository. Images are specified with either <code>imageTag</code> or <code>imageDigest</code>. You can remove a tag from an image by specifying the image's tag in your request. When you remove the last tag from an image, the image is deleted from your repository. You can completely delete an image (and all of its tags) by specifying the image's digest in your request.

        :param self:
        :param imageIds:  A list of image ID references that correspond to images to delete. The format of the <code>imageIds</code> reference is <code>imageTag=tag</code> or <code>imageDigest=digest</code>.<br/>repositoryName:  The repository that contains the image to delete.<br/>registryId:  The AWS account ID associated with the registry that contains the image to delete. If you do not specify a registry, the default registry is assumed.<br/>
        :return: failures:  Any failures associated with the call.<br/>imageIds:  The image IDs of the deleted images.<br/>        """
        pass

    def batch_get_image(self, imageIds: List[ImageIdentifier], repositoryName: str, registryId: str=None, acceptedMediaTypes: List[MediaType]=None) -> BatchGetImageResponse:
        """Gets detailed information for specified images within a specified repository. Images are specified with either <code>imageTag</code> or <code>imageDigest</code>.

        :param self:
        :param acceptedMediaTypes:  The accepted media types for the request. Valid values: <code>application/vnd.docker.distribution.manifest.v1+json</code> | <code>application/vnd.docker.distribution.manifest.v2+json</code> | <code>application/vnd.oci.image.manifest.v1+json</code> <br/>imageIds:  A list of image ID references that correspond to images to describe. The format of the <code>imageIds</code> reference is <code>imageTag=tag</code> or <code>imageDigest=digest</code>.<br/>repositoryName:  The repository that contains the images to describe.<br/>registryId:  The AWS account ID associated with the registry that contains the images to describe. If you do not specify a registry, the default registry is assumed.<br/>
        :return: failures:  Any failures associated with the call.<br/>images:  A list of image objects corresponding to the image references in the request.<br/>        """
        pass

    def complete_layer_upload(self, layerDigests: List[LayerDigest], uploadId: str, repositoryName: str, registryId: str=None) -> CompleteLayerUploadResponse:
        """Informs Amazon ECR that the image layer upload has completed for a specified registry, repository name, and upload ID. You can optionally provide a <code>sha256</code> digest of the image layer for data validation purposes. <note> This operation is used by the Amazon ECR proxy, and it is not intended for general use by customers for pulling and pushing images. In most cases, you should use the <code>docker</code> CLI to pull, tag, and push images. </note>

        :param self:
        :param layerDigests:  The <code>sha256</code> digest of the image layer.<br/>uploadId:  The upload ID from a previous <a>InitiateLayerUpload</a> operation to associate with the image layer.<br/>repositoryName:  The name of the repository to associate with the image layer.<br/>registryId:  The AWS account ID associated with the registry to which to upload layers. If you do not specify a registry, the default registry is assumed.<br/>
        :return: layerDigest:  The <code>sha256</code> digest of the image layer.<br/>uploadId:  The upload ID associated with the layer.<br/>repositoryName:  The repository name associated with the request.<br/>registryId:  The registry ID associated with the request.<br/>        """
        pass

    def create_repository(self, repositoryName: str, tags: List[Tag]=None) -> CreateRepositoryResponse:
        """Creates an image repository.

        :param self:
        :param tags:  <p/><br/>repositoryName:  The name to use for the repository. The repository name may be specified on its own (such as <code>nginx-web-app</code>) or it can be prepended with a namespace to group the repository into a category (such as <code>project-a/nginx-web-app</code>).<br/>
        :return: repository:  The repository that was created.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The date and time, in JavaScript date format, when the repository was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>repositoryUri</i> The URI for the repository. You can use this URI for Docker <code>push</code> or <code>pull</code> operations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>repositoryName</i> The name of the repository.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>registryId</i> The AWS account ID associated with the registry that contains the repository.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>repositoryArn</i> The Amazon Resource Name (ARN) that identifies the repository. The ARN contains the <code>arn:aws:ecr</code> namespace, followed by the region of the repository, AWS account ID of the repository owner, repository namespace, and repository name. For example, <code>arn:aws:ecr:region:012345678910:repository/test</code>.<br/>        """
        pass

    def delete_lifecycle_policy(self, repositoryName: str, registryId: str=None) -> DeleteLifecyclePolicyResponse:
        """Deletes the specified lifecycle policy.

        :param self:
        :param repositoryName:  The name of the repository.<br/>registryId:  The AWS account ID associated with the registry that contains the repository. If you do not specify a registry, the default registry is assumed.<br/>
        :return: lastEvaluatedAt:  The time stamp of the last time that the lifecycle policy was run.<br/>lifecyclePolicyText:  The JSON lifecycle policy text.<br/>repositoryName:  The repository name associated with the request.<br/>registryId:  The registry ID associated with the request.<br/>        """
        pass

    def delete_repository(self, repositoryName: str, registryId: str=None, force: ForceFlag=None) -> DeleteRepositoryResponse:
        """Deletes an existing image repository. If a repository contains images, you must use the <code>force</code> option to delete it.

        :param self:
        :param force:   If a repository contains images, forces the deletion.<br/>repositoryName:  The name of the repository to delete.<br/>registryId:  The AWS account ID associated with the registry that contains the repository to delete. If you do not specify a registry, the default registry is assumed.<br/>
        :return: repository:  The repository that was deleted.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The date and time, in JavaScript date format, when the repository was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>repositoryUri</i> The URI for the repository. You can use this URI for Docker <code>push</code> or <code>pull</code> operations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>repositoryName</i> The name of the repository.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>registryId</i> The AWS account ID associated with the registry that contains the repository.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>repositoryArn</i> The Amazon Resource Name (ARN) that identifies the repository. The ARN contains the <code>arn:aws:ecr</code> namespace, followed by the region of the repository, AWS account ID of the repository owner, repository namespace, and repository name. For example, <code>arn:aws:ecr:region:012345678910:repository/test</code>.<br/>        """
        pass

    def delete_repository_policy(self, repositoryName: str, registryId: str=None) -> DeleteRepositoryPolicyResponse:
        """Deletes the repository policy from a specified repository.

        :param self:
        :param repositoryName:  The name of the repository that is associated with the repository policy to delete.<br/>registryId:  The AWS account ID associated with the registry that contains the repository policy to delete. If you do not specify a registry, the default registry is assumed.<br/>
        :return: policyText:  The JSON repository policy that was deleted from the repository.<br/>repositoryName:  The repository name associated with the request.<br/>registryId:  The registry ID associated with the request.<br/>        """
        pass

    def describe_images(self, repositoryName: str, registryId: str=None, imageIds: List[ImageIdentifier]=None, nextToken: str=None, maxResults: int=None, filter: DescribeImagesFilter=None) -> DescribeImagesResponse:
        """Returns metadata about the images in a repository, including image size, image tags, and creation date. <note> Beginning with Docker version 1.9, the Docker client compresses image layers before pushing them to a V2 Docker registry. The output of the <code>docker images</code> command shows the uncompressed image size, so it may return a larger image size than the image sizes returned by <a>DescribeImages</a>. </note>

        :param self:
        :param filter:  The filter key and value with which to filter your <code>DescribeImages</code> results.:<br/>maxResults:  The maximum number of repository results returned by <code>DescribeImages</code> in paginated output. When this parameter is used, <code>DescribeImages</code> only returns <code>maxResults</code> results in a single page along with a <code>nextToken</code> response element. The remaining results of the initial request can be seen by sending another <code>DescribeImages</code> request with the returned <code>nextToken</code> value. This value can be between 1 and 1000. If this parameter is not used, then <code>DescribeImages</code> returns up to 100 results and a <code>nextToken</code> value, if applicable. This option cannot be used when you specify images with <code>imageIds</code>.<br/>nextToken:  The <code>nextToken</code> value returned from a previous paginated <code>DescribeImages</code> request where <code>maxResults</code> was used and the results exceeded the value of that parameter. Pagination continues from the end of the previous results that returned the <code>nextToken</code> value. This value is <code>null</code> when there are no more results to return. This option cannot be used when you specify images with <code>imageIds</code>.<br/>imageIds:  The list of image IDs for the requested repository.<br/>repositoryName:  A list of repositories to describe.<br/>registryId:  The AWS account ID associated with the registry that contains the repository in which to describe images. If you do not specify a registry, the default registry is assumed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tagStatus</i> The tag status with which to filter your <a>DescribeImages</a> results. You can filter results based on whether they are <code>TAGGED</code> or <code>UNTAGGED</code>.<br/>
        :return: nextToken:  The <code>nextToken</code> value to include in a future <code>DescribeImages</code> request. When the results of a <code>DescribeImages</code> request exceed <code>maxResults</code>, this value can be used to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>imageDetails:  A list of <a>ImageDetail</a> objects that contain data about the image.<br/>        """
        pass

    def describe_repositories(self, registryId: str=None, repositoryNames: List[RepositoryName]=None, nextToken: str=None, maxResults: int=None) -> DescribeRepositoriesResponse:
        """Describes image repositories in a registry.

        :param self:
        :param maxResults:  The maximum number of repository results returned by <code>DescribeRepositories</code> in paginated output. When this parameter is used, <code>DescribeRepositories</code> only returns <code>maxResults</code> results in a single page along with a <code>nextToken</code> response element. The remaining results of the initial request can be seen by sending another <code>DescribeRepositories</code> request with the returned <code>nextToken</code> value. This value can be between 1 and 1000. If this parameter is not used, then <code>DescribeRepositories</code> returns up to 100 results and a <code>nextToken</code> value, if applicable. This option cannot be used when you specify repositories with <code>repositoryNames</code>.<br/>nextToken:  The <code>nextToken</code> value returned from a previous paginated <code>DescribeRepositories</code> request where <code>maxResults</code> was used and the results exceeded the value of that parameter. Pagination continues from the end of the previous results that returned the <code>nextToken</code> value. This value is <code>null</code> when there are no more results to return. This option cannot be used when you specify repositories with <code>repositoryNames</code>. <note> This token should be treated as an opaque identifier that is only used to retrieve the next items in a list and not for other programmatic purposes. </note><br/>repositoryNames:  A list of repositories to describe. If this parameter is omitted, then all repositories in a registry are described.<br/>registryId:  The AWS account ID associated with the registry that contains the repositories to be described. If you do not specify a registry, the default registry is assumed.<br/>
        :return: nextToken:  The <code>nextToken</code> value to include in a future <code>DescribeRepositories</code> request. When the results of a <code>DescribeRepositories</code> request exceed <code>maxResults</code>, this value can be used to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>repositories:  A list of repository objects corresponding to valid repositories.<br/>        """
        pass

    def get_authorization_token(self, registryIds: List[RegistryId]=None) -> GetAuthorizationTokenResponse:
        """Retrieves a token that is valid for a specified registry for 12 hours. This command allows you to use the <code>docker</code> CLI to push and pull images with Amazon ECR. If you do not specify a registry, the default registry is assumed. The <code>authorizationToken</code> returned for each registry specified is a base64 encoded string that can be decoded and used in a <code>docker login</code> command to authenticate to a registry. The AWS CLI offers an <code>aws ecr get-login</code> command that simplifies the login process.

        :param self:
        :param registryIds:  A list of AWS account IDs that are associated with the registries for which to get authorization tokens. If you do not specify a registry, the default registry is assumed.<br/>
        :return: authorizationData:  A list of authorization token data objects that correspond to the <code>registryIds</code> values in the request.<br/>        """
        pass

    def get_download_url_for_layer(self, layerDigest: str, repositoryName: str, registryId: str=None) -> GetDownloadUrlForLayerResponse:
        """Retrieves the pre-signed Amazon S3 download URL corresponding to an image layer. You can only get URLs for image layers that are referenced in an image. <note> This operation is used by the Amazon ECR proxy, and it is not intended for general use by customers for pulling and pushing images. In most cases, you should use the <code>docker</code> CLI to pull, tag, and push images. </note>

        :param self:
        :param layerDigest:  The digest of the image layer to download.<br/>repositoryName:  The name of the repository that is associated with the image layer to download.<br/>registryId:  The AWS account ID associated with the registry that contains the image layer to download. If you do not specify a registry, the default registry is assumed.<br/>
        :return: layerDigest:  The digest of the image layer to download.<br/>downloadUrl:  The pre-signed Amazon S3 download URL for the requested layer.<br/>        """
        pass

    def get_lifecycle_policy(self, repositoryName: str, registryId: str=None) -> GetLifecyclePolicyResponse:
        """Retrieves the specified lifecycle policy.

        :param self:
        :param repositoryName:  The name of the repository.<br/>registryId:  The AWS account ID associated with the registry that contains the repository. If you do not specify a registry, the default registry is assumed.<br/>
        :return: lastEvaluatedAt:  The time stamp of the last time that the lifecycle policy was run.<br/>lifecyclePolicyText:  The JSON lifecycle policy text.<br/>repositoryName:  The repository name associated with the request.<br/>registryId:  The registry ID associated with the request.<br/>        """
        pass

    def get_lifecycle_policy_preview(self, repositoryName: str, registryId: str=None, imageIds: List[ImageIdentifier]=None, nextToken: str=None, maxResults: int=None, filter: LifecyclePolicyPreviewFilter=None) -> GetLifecyclePolicyPreviewResponse:
        """Retrieves the results of the specified lifecycle policy preview request.

        :param self:
        :param filter:  An optional parameter that filters results based on image tag status and all tags, if tagged.:<br/>maxResults:  The maximum number of repository results returned by <code>GetLifecyclePolicyPreviewRequest</code> in&#x2028; paginated output. When this parameter is used, <code>GetLifecyclePolicyPreviewRequest</code> only returns&#x2028; <code>maxResults</code> results in a single page along with a <code>nextToken</code>&#x2028; response element. The remaining results of the initial request can be seen by sending&#x2028; another <code>GetLifecyclePolicyPreviewRequest</code> request with the returned <code>nextToken</code>&#x2028; value. This value can be between 1 and 1000. If this&#x2028; parameter is not used, then <code>GetLifecyclePolicyPreviewRequest</code> returns up to&#x2028; 100 results and a <code>nextToken</code> value, if&#x2028; applicable. This option cannot be used when you specify images with <code>imageIds</code>.<br/>nextToken:  The <code>nextToken</code> value returned from a previous paginated&#x2028; <code>GetLifecyclePolicyPreviewRequest</code> request where <code>maxResults</code> was used and the&#x2028; results exceeded the value of that parameter. Pagination continues from the end of the&#x2028; previous results that returned the <code>nextToken</code> value. This value is&#x2028; <code>null</code> when there are no more results to return. This option cannot be used when you specify images with <code>imageIds</code>.<br/>imageIds:  The list of imageIDs to be included.<br/>repositoryName:  The name of the repository.<br/>registryId:  The AWS account ID associated with the registry that contains the repository. If you do not specify a registry, the default registry is assumed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tagStatus</i> The tag status of the image.<br/>
        :return: summary:  The list of images that is returned as a result of the action.:<br/>previewResults:  The results of the lifecycle policy preview request.<br/>nextToken:  The <code>nextToken</code> value to include in a future <code>GetLifecyclePolicyPreview</code> request. When the results of a <code>GetLifecyclePolicyPreview</code> request exceed <code>maxResults</code>, this value can be used to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>status:  The status of the lifecycle policy preview request.<br/>lifecyclePolicyText:  The JSON lifecycle policy text.<br/>repositoryName:  The repository name associated with the request.<br/>registryId:  The registry ID associated with the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>expiringImageTotalCount</i> The number of expiring images.<br/>        """
        pass

    def get_repository_policy(self, repositoryName: str, registryId: str=None) -> GetRepositoryPolicyResponse:
        """Retrieves the repository policy for a specified repository.

        :param self:
        :param repositoryName:  The name of the repository with the policy to retrieve.<br/>registryId:  The AWS account ID associated with the registry that contains the repository. If you do not specify a registry, the default registry is assumed.<br/>
        :return: policyText:  The JSON repository policy text associated with the repository.<br/>repositoryName:  The repository name associated with the request.<br/>registryId:  The registry ID associated with the request.<br/>        """
        pass

    def initiate_layer_upload(self, repositoryName: str, registryId: str=None) -> InitiateLayerUploadResponse:
        """Notify Amazon ECR that you intend to upload an image layer. <note> This operation is used by the Amazon ECR proxy, and it is not intended for general use by customers for pulling and pushing images. In most cases, you should use the <code>docker</code> CLI to pull, tag, and push images. </note>

        :param self:
        :param repositoryName:  The name of the repository to which you intend to upload layers.<br/>registryId:  The AWS account ID associated with the registry to which you intend to upload layers. If you do not specify a registry, the default registry is assumed.<br/>
        :return: partSize:  The size, in bytes, that Amazon ECR expects future layer part uploads to be.<br/>uploadId:  The upload ID for the layer upload. This parameter is passed to further <a>UploadLayerPart</a> and <a>CompleteLayerUpload</a> operations.<br/>        """
        pass

    def list_images(self, repositoryName: str, registryId: str=None, nextToken: str=None, maxResults: int=None, filter: ListImagesFilter=None) -> ListImagesResponse:
        """Lists all the image IDs for a given repository. You can filter images based on whether or not they are tagged by setting the <code>tagStatus</code> parameter to <code>TAGGED</code> or <code>UNTAGGED</code>. For example, you can filter your results to return only <code>UNTAGGED</code> images and then pipe that result to a <a>BatchDeleteImage</a> operation to delete them. Or, you can filter your results to return only <code>TAGGED</code> images to list all of the tags in your repository.

        :param self:
        :param filter:  The filter key and value with which to filter your <code>ListImages</code> results.:<br/>maxResults:  The maximum number of image results returned by <code>ListImages</code> in paginated output. When this parameter is used, <code>ListImages</code> only returns <code>maxResults</code> results in a single page along with a <code>nextToken</code> response element. The remaining results of the initial request can be seen by sending another <code>ListImages</code> request with the returned <code>nextToken</code> value. This value can be between 1 and 1000. If this parameter is not used, then <code>ListImages</code> returns up to 100 results and a <code>nextToken</code> value, if applicable.<br/>nextToken:  The <code>nextToken</code> value returned from a previous paginated <code>ListImages</code> request where <code>maxResults</code> was used and the results exceeded the value of that parameter. Pagination continues from the end of the previous results that returned the <code>nextToken</code> value. This value is <code>null</code> when there are no more results to return. <note> This token should be treated as an opaque identifier that is only used to retrieve the next items in a list and not for other programmatic purposes. </note><br/>repositoryName:  The repository with image IDs to be listed.<br/>registryId:  The AWS account ID associated with the registry that contains the repository in which to list images. If you do not specify a registry, the default registry is assumed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tagStatus</i> The tag status with which to filter your <a>ListImages</a> results. You can filter results based on whether they are <code>TAGGED</code> or <code>UNTAGGED</code>.<br/>
        :return: nextToken:  The <code>nextToken</code> value to include in a future <code>ListImages</code> request. When the results of a <code>ListImages</code> request exceed <code>maxResults</code>, this value can be used to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>imageIds:  The list of image IDs for the requested repository.<br/>        """
        pass

    def list_tags_for_resource(self, resourceArn: str) -> ListTagsForResourceResponse:
        """List the tags for an Amazon ECR resource.

        :param self:
        :param resourceArn:  The Amazon Resource Name (ARN) that identifies the resource for which to list the tags. Currently, the only supported resource is an Amazon ECR repository.<br/>
        :return: tags:  The tags for the resource.<br/>        """
        pass

    def put_image(self, imageManifest: str, repositoryName: str, registryId: str=None, imageTag: str=None) -> PutImageResponse:
        """Creates or updates the image manifest and tags associated with an image. <note> This operation is used by the Amazon ECR proxy, and it is not intended for general use by customers for pulling and pushing images. In most cases, you should use the <code>docker</code> CLI to pull, tag, and push images. </note>

        :param self:
        :param imageTag:  The tag to associate with the image. This parameter is required for images that use the Docker Image Manifest V2 Schema 2 or OCI formats.<br/>imageManifest:  The image manifest corresponding to the image to be uploaded.<br/>repositoryName:  The name of the repository in which to put the image.<br/>registryId:  The AWS account ID associated with the registry that contains the repository in which to put the image. If you do not specify a registry, the default registry is assumed.<br/>
        :return: image:  Details of the image uploaded.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>imageManifest</i> The image manifest associated with the image.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>imageId</i> An object containing the image tag and image digest associated with an image.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>imageId</b>: An object containing the image tag and image digest associated with an image.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>repositoryName</i> The name of the repository associated with the image.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>registryId</i> The AWS account ID associated with the registry containing the image.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>imageTag</i> The tag used for the image.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>imageDigest</i> The <code>sha256</code> digest of the image manifest.<br/>        """
        pass

    def put_lifecycle_policy(self, lifecyclePolicyText: str, repositoryName: str, registryId: str=None) -> PutLifecyclePolicyResponse:
        """Creates or updates a lifecycle policy. For information about lifecycle policy syntax, see <a href="http://docs.aws.amazon.com/AmazonECR/latest/userguide/LifecyclePolicies.html">Lifecycle Policy Template</a>.

        :param self:
        :param lifecyclePolicyText:  The JSON repository policy text to apply to the repository.<br/>repositoryName:  The name of the repository to receive the policy.<br/>registryId:  The AWS account ID associated with the registry that contains the repository. If you do&#x2028; not specify a registry, the default registry is assumed.<br/>
        :return: lifecyclePolicyText:  The JSON repository policy text.<br/>repositoryName:  The repository name associated with the request.<br/>registryId:  The registry ID associated with the request.<br/>        """
        pass

    def set_repository_policy(self, policyText: str, repositoryName: str, registryId: str=None, force: ForceFlag=None) -> SetRepositoryPolicyResponse:
        """Applies a repository policy on a specified repository to control access permissions.

        :param self:
        :param force:  If the policy you are attempting to set on a repository policy would prevent you from setting another policy in the future, you must force the <a>SetRepositoryPolicy</a> operation. This is intended to prevent accidental repository lock outs.<br/>policyText:  The JSON repository policy text to apply to the repository.<br/>repositoryName:  The name of the repository to receive the policy.<br/>registryId:  The AWS account ID associated with the registry that contains the repository. If you do not specify a registry, the default registry is assumed.<br/>
        :return: policyText:  The JSON repository policy text applied to the repository.<br/>repositoryName:  The repository name associated with the request.<br/>registryId:  The registry ID associated with the request.<br/>        """
        pass

    def start_lifecycle_policy_preview(self, repositoryName: str, registryId: str=None, lifecyclePolicyText: str=None) -> StartLifecyclePolicyPreviewResponse:
        """Starts a preview of the specified lifecycle policy. This allows you to see the results before creating the lifecycle policy.

        :param self:
        :param lifecyclePolicyText:  The policy to be evaluated against. If you do not specify a policy, the current policy for the repository is used.<br/>repositoryName:  The name of the repository to be evaluated.<br/>registryId:  The AWS account ID associated with the registry that contains the repository. If you do not specify a registry, the default registry is assumed.<br/>
        :return: status:  The status of the lifecycle policy preview request.<br/>lifecyclePolicyText:  The JSON repository policy text.<br/>repositoryName:  The repository name associated with the request.<br/>registryId:  The registry ID associated with the request.<br/>        """
        pass

    def tag_resource(self, tags: List[Tag], resourceArn: str) -> TagResourceResponse:
        """Adds specified tags to a resource with the specified ARN. Existing tags on a resource are not changed if they are not specified in the request parameters.

        :param self:
        :param tags:  The tags to add to the resource. A tag is an array of key-value pairs. Tag keys can have a maximum character length of 128 characters, and tag values can have a maximum length of 256 characters.<br/>resourceArn:  The Amazon Resource Name (ARN) of the the resource to which to add tags. Currently, the only supported resource is an Amazon ECR repository.<br/>
        :return:         """
        pass

    def untag_resource(self, tagKeys: List[TagKey], resourceArn: str) -> UntagResourceResponse:
        """Deletes specified tags from a resource.

        :param self:
        :param tagKeys:  The keys of the tags to be removed.<br/>resourceArn:  The Amazon Resource Name (ARN) of the resource from which to remove tags. Currently, the only supported resource is an Amazon ECR repository.<br/>
        :return:         """
        pass

    def upload_layer_part(self, layerPartBlob: LayerPartBlob, partLastByte: PartSize, partFirstByte: PartSize, uploadId: str, repositoryName: str, registryId: str=None) -> UploadLayerPartResponse:
        """Uploads an image layer part to Amazon ECR. <note> This operation is used by the Amazon ECR proxy, and it is not intended for general use by customers for pulling and pushing images. In most cases, you should use the <code>docker</code> CLI to pull, tag, and push images. </note>

        :param self:
        :param layerPartBlob:  The base64-encoded layer part payload.<br/>partLastByte:  The integer value of the last byte of the layer part.<br/>partFirstByte:  The integer value of the first byte of the layer part.<br/>uploadId:  The upload ID from a previous <a>InitiateLayerUpload</a> operation to associate with the layer part upload.<br/>repositoryName:  The name of the repository to which you are uploading layer parts.<br/>registryId:  The AWS account ID associated with the registry to which you are uploading layer parts. If you do not specify a registry, the default registry is assumed.<br/>
        :return: lastByteReceived:  The integer value of the last byte received in the request.<br/>uploadId:  The upload ID associated with the request.<br/>repositoryName:  The repository name associated with the request.<br/>registryId:  The registry ID associated with the request.<br/>        """
        pass



class ECS(BaseClient):
    """<fullname>Amazon Elastic Container Service</fullname> Amazon Elastic Container Service (Amazon ECS) is a highly scalable, fast, container management service that makes it easy to run, stop, and manage Docker containers on a cluster. You can host your cluster on a serverless infrastructure that is managed by Amazon ECS by launching your services or tasks using the Fargate launch type. For more control, you can host your tasks on a cluster of Amazon Elastic Compute Cloud (Amazon EC2) instances that you manage by using the EC2 launch type. For more information about launch types, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/launch_types.html">Amazon ECS Launch Types</a>. Amazon ECS lets you launch and stop container-based applications with simple API calls, allows you to get the state of your cluster from a centralized service, and gives you access to many familiar Amazon EC2 features. You can use Amazon ECS to schedule the placement of containers across your cluster based on your resource needs, isolation policies, and availability requirements. Amazon ECS eliminates the need for you to operate your own cluster management and configuration management systems or worry about scaling your management infrastructure."""

    class ServicesInactiveWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_services` for valid parameters"""
            pass
    class ServicesStableWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_services` for valid parameters"""
            pass
    class TasksRunningWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_tasks` for valid parameters"""
            pass
    class TasksStoppedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_tasks` for valid parameters"""
            pass


    class ListClustersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_clusters` for valid parameters"""
            pass
    class ListContainerInstancesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_container_instances` for valid parameters"""
            pass
    class ListTaskDefinitionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_task_definitions` for valid parameters"""
            pass
    class ListTaskDefinitionFamiliesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_task_definition_families` for valid parameters"""
            pass
    class ListTasksPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_tasks` for valid parameters"""
            pass
    class ListServicesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_services` for valid parameters"""
            pass
    class ListAccountSettingsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_account_settings` for valid parameters"""
            pass
    class ListAttributesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_attributes` for valid parameters"""
            pass


    class CreateClusterRequest(Mapping):
        pass
    
    class CreateClusterResponse(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class ClusterSetting(Mapping):
        pass
    
    class CreateServiceRequest(Mapping):
        pass
    
    class CreateServiceResponse(Mapping):
        pass
    
    class LoadBalancer(Mapping):
        pass
    
    class ServiceRegistry(Mapping):
        pass
    
    class DeploymentConfiguration(Mapping):
        pass
    
    class PlacementConstraint(Mapping):
        pass
    
    class PlacementStrategy(Mapping):
        pass
    
    class NetworkConfiguration(Mapping):
        pass
    
    class DeploymentController(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class CreateTaskSetRequest(Mapping):
        pass
    
    class CreateTaskSetResponse(Mapping):
        pass
    
    class Scale(Mapping):
        pass
    
    class DeleteAccountSettingRequest(Mapping):
        pass
    
    class DeleteAccountSettingResponse(Mapping):
        pass
    
    class DeleteAttributesRequest(Mapping):
        pass
    
    class DeleteAttributesResponse(Mapping):
        pass
    
    class Attribute(Mapping):
        pass
    
    class DeleteClusterRequest(Mapping):
        pass
    
    class DeleteClusterResponse(Mapping):
        pass
    
    class DeleteServiceRequest(Mapping):
        pass
    
    class DeleteServiceResponse(Mapping):
        pass
    
    class BoxedBoolean(object):
        pass
    
    class DeleteTaskSetRequest(Mapping):
        pass
    
    class DeleteTaskSetResponse(Mapping):
        pass
    
    class DeregisterContainerInstanceRequest(Mapping):
        pass
    
    class DeregisterContainerInstanceResponse(Mapping):
        pass
    
    class DeregisterTaskDefinitionRequest(Mapping):
        pass
    
    class DeregisterTaskDefinitionResponse(Mapping):
        pass
    
    class DescribeClustersRequest(Mapping):
        pass
    
    class DescribeClustersResponse(Mapping):
        pass
    
    class String(object):
        pass
    
    class ClusterField(object):
        pass
    
    class DescribeContainerInstancesRequest(Mapping):
        pass
    
    class DescribeContainerInstancesResponse(Mapping):
        pass
    
    class ContainerInstanceField(object):
        pass
    
    class DescribeServicesRequest(Mapping):
        pass
    
    class DescribeServicesResponse(Mapping):
        pass
    
    class ServiceField(object):
        pass
    
    class DescribeTaskDefinitionRequest(Mapping):
        pass
    
    class DescribeTaskDefinitionResponse(Mapping):
        pass
    
    class TaskDefinitionField(object):
        pass
    
    class DescribeTaskSetsRequest(Mapping):
        pass
    
    class DescribeTaskSetsResponse(Mapping):
        pass
    
    class DescribeTasksRequest(Mapping):
        pass
    
    class DescribeTasksResponse(Mapping):
        pass
    
    class TaskField(object):
        pass
    
    class DiscoverPollEndpointRequest(Mapping):
        pass
    
    class DiscoverPollEndpointResponse(Mapping):
        pass
    
    class ListAccountSettingsRequest(Mapping):
        pass
    
    class ListAccountSettingsResponse(Mapping):
        pass
    
    class ListAttributesRequest(Mapping):
        pass
    
    class ListAttributesResponse(Mapping):
        pass
    
    class ListClustersRequest(Mapping):
        pass
    
    class ListClustersResponse(Mapping):
        pass
    
    class ListContainerInstancesRequest(Mapping):
        pass
    
    class ListContainerInstancesResponse(Mapping):
        pass
    
    class ListServicesRequest(Mapping):
        pass
    
    class ListServicesResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class ListTaskDefinitionFamiliesRequest(Mapping):
        pass
    
    class ListTaskDefinitionFamiliesResponse(Mapping):
        pass
    
    class ListTaskDefinitionsRequest(Mapping):
        pass
    
    class ListTaskDefinitionsResponse(Mapping):
        pass
    
    class ListTasksRequest(Mapping):
        pass
    
    class ListTasksResponse(Mapping):
        pass
    
    class PutAccountSettingRequest(Mapping):
        pass
    
    class PutAccountSettingResponse(Mapping):
        pass
    
    class PutAccountSettingDefaultRequest(Mapping):
        pass
    
    class PutAccountSettingDefaultResponse(Mapping):
        pass
    
    class PutAttributesRequest(Mapping):
        pass
    
    class PutAttributesResponse(Mapping):
        pass
    
    class RegisterContainerInstanceRequest(Mapping):
        pass
    
    class RegisterContainerInstanceResponse(Mapping):
        pass
    
    class Resource(Mapping):
        pass
    
    class VersionInfo(Mapping):
        pass
    
    class PlatformDevice(Mapping):
        pass
    
    class RegisterTaskDefinitionRequest(Mapping):
        pass
    
    class RegisterTaskDefinitionResponse(Mapping):
        pass
    
    class ContainerDefinition(Mapping):
        pass
    
    class Volume(Mapping):
        pass
    
    class TaskDefinitionPlacementConstraint(Mapping):
        pass
    
    class Compatibility(object):
        pass
    
    class ProxyConfiguration(Mapping):
        pass
    
    class RunTaskRequest(Mapping):
        pass
    
    class RunTaskResponse(Mapping):
        pass
    
    class TaskOverride(Mapping):
        pass
    
    class StartTaskRequest(Mapping):
        pass
    
    class StartTaskResponse(Mapping):
        pass
    
    class StopTaskRequest(Mapping):
        pass
    
    class StopTaskResponse(Mapping):
        pass
    
    class SubmitAttachmentStateChangesRequest(Mapping):
        pass
    
    class SubmitAttachmentStateChangesResponse(Mapping):
        pass
    
    class AttachmentStateChange(Mapping):
        pass
    
    class SubmitContainerStateChangeRequest(Mapping):
        pass
    
    class SubmitContainerStateChangeResponse(Mapping):
        pass
    
    class NetworkBinding(Mapping):
        pass
    
    class SubmitTaskStateChangeRequest(Mapping):
        pass
    
    class SubmitTaskStateChangeResponse(Mapping):
        pass
    
    class ContainerStateChange(Mapping):
        pass
    
    class Timestamp(object):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateContainerAgentRequest(Mapping):
        pass
    
    class UpdateContainerAgentResponse(Mapping):
        pass
    
    class UpdateContainerInstancesStateRequest(Mapping):
        pass
    
    class UpdateContainerInstancesStateResponse(Mapping):
        pass
    
    class UpdateServiceRequest(Mapping):
        pass
    
    class UpdateServiceResponse(Mapping):
        pass
    
    class UpdateServicePrimaryTaskSetRequest(Mapping):
        pass
    
    class UpdateServicePrimaryTaskSetResponse(Mapping):
        pass
    
    class UpdateTaskSetRequest(Mapping):
        pass
    
    class UpdateTaskSetResponse(Mapping):
        pass
    
    def create_cluster(self, clusterName: str=None, tags: List[Tag]=None, settings: List[ClusterSetting]=None) -> CreateClusterResponse:
        """Creates a new Amazon ECS cluster. By default, your account receives a <code>default</code> cluster when you launch your first container instance. However, you can create your own cluster with a unique name with the <code>CreateCluster</code> action. <note> When you call the <a>CreateCluster</a> API operation, Amazon ECS attempts to create the service-linked role for your account so that required resources in other AWS services can be managed on your behalf. However, if the IAM user that makes the call does not have permissions to create the service-linked role, it is not created. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/using-service-linked-roles.html">Using Service-Linked Roles for Amazon ECS</a> in the <i>Amazon Elastic Container Service Developer Guide</i>. </note>

        :param self:
        :param settings:  The setting to use when creating a cluster. This parameter is used to enable CloudWatch Container Insights for a cluster. If this value is specified, it will override the <code>containerInsights</code> value set with <a>PutAccountSetting</a> or <a>PutAccountSettingDefault</a>.<br/>tags:  The metadata that you apply to the cluster to help you categorize and organize them. Each tag consists of a key and an optional value, both of which you define. Tag keys can have a maximum character length of 128 characters, and tag values can have a maximum length of 256 characters.<br/>clusterName:  The name of your cluster. If you do not specify a name for your cluster, you create a cluster named <code>default</code>. Up to 255 letters (uppercase and lowercase), numbers, and hyphens are allowed. <br/>
        :return: cluster:  The full description of your new cluster.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>settings</i> The settings for the cluster. This parameter indicates whether CloudWatch Container Insights is enabled or disabled for a cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> The metadata that you apply to the cluster to help you categorize and organize them. Each tag consists of a key and an optional value, both of which you define. Tag keys can have a maximum character length of 128 characters, and tag values can have a maximum length of 256 characters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statistics</i> Additional information about your clusters that are separated by launch type, including: <ul> <li> runningEC2TasksCount </li> <li> RunningFargateTasksCount </li> <li> pendingEC2TasksCount </li> <li> pendingFargateTasksCount </li> <li> activeEC2ServiceCount </li> <li> activeFargateServiceCount </li> <li> drainingEC2ServiceCount </li> <li> drainingFargateServiceCount </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>activeServicesCount</i> The number of services that are running on the cluster in an <code>ACTIVE</code> state. You can view these services with <a>ListServices</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pendingTasksCount</i> The number of tasks in the cluster that are in the <code>PENDING</code> state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>runningTasksCount</i> The number of tasks in the cluster that are in the <code>RUNNING</code> state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>registeredContainerInstancesCount</i> The number of container instances registered into the cluster. This includes container instances in both <code>ACTIVE</code> and <code>DRAINING</code> status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the cluster. The valid values are <code>ACTIVE</code> or <code>INACTIVE</code>. <code>ACTIVE</code> indicates that you can register container instances with the cluster and the associated instances can accept tasks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clusterName</i> A user-generated string that you use to identify your cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clusterArn</i> The Amazon Resource Name (ARN) that identifies the cluster. The ARN contains the <code>arn:aws:ecs</code> namespace, followed by the Region of the cluster, the AWS account ID of the cluster owner, the <code>cluster</code> namespace, and then the cluster name. For example, <code>arn:aws:ecs:region:012345678910:cluster/test</code>.<br/>        """
        pass

    def create_service(self, serviceName: str, cluster: str=None, taskDefinition: str=None, loadBalancers: List[LoadBalancer]=None, serviceRegistries: List[ServiceRegistry]=None, desiredCount: int=None, clientToken: str=None, launchType: str=None, platformVersion: str=None, role: str=None, deploymentConfiguration: DeploymentConfiguration=None, placementConstraints: List[PlacementConstraint]=None, placementStrategy: List[PlacementStrategy]=None, networkConfiguration: NetworkConfiguration=None, healthCheckGracePeriodSeconds: int=None, schedulingStrategy: str=None, deploymentController: DeploymentController=None, tags: List[Tag]=None, enableECSManagedTags: Boolean=None, propagateTags: str=None) -> CreateServiceResponse:
        """Runs and maintains a desired number of tasks from a specified task definition. If the number of tasks running in a service drops below the <code>desiredCount</code>, Amazon ECS spawns another copy of the task in the specified cluster. To update an existing service, see <a>UpdateService</a>. In addition to maintaining the desired count of tasks in your service, you can optionally run your service behind a load balancer. The load balancer distributes traffic across the tasks that are associated with the service. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/service-load-balancing.html">Service Load Balancing</a> in the <i>Amazon Elastic Container Service Developer Guide</i>. Tasks for services that <i>do not</i> use a load balancer are considered healthy if they're in the <code>RUNNING</code> state. Tasks for services that <i>do</i> use a load balancer are considered healthy if they're in the <code>RUNNING</code> state and the container instance that they're hosted on is reported as healthy by the load balancer. There are two service scheduler strategies available: <ul> <li>  <code>REPLICA</code> - The replica scheduling strategy places and maintains the desired number of tasks across your cluster. By default, the service scheduler spreads tasks across Availability Zones. You can use task placement strategies and constraints to customize task placement decisions. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/ecs_services.html">Service Scheduler Concepts</a> in the <i>Amazon Elastic Container Service Developer Guide</i>. </li> <li>  <code>DAEMON</code> - The daemon scheduling strategy deploys exactly one task on each active container instance that meets all of the task placement constraints that you specify in your cluster. When using this strategy, you don't need to specify a desired number of tasks, a task placement strategy, or use Service Auto Scaling policies. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/ecs_services.html">Service Scheduler Concepts</a> in the <i>Amazon Elastic Container Service Developer Guide</i>. </li> </ul> You can optionally specify a deployment configuration for your service. The deployment is triggered by changing properties, such as the task definition or the desired count of a service, with an <a>UpdateService</a> operation. The default value for a replica service for <code>minimumHealthyPercent</code> is 100%. The default value for a daemon service for <code>minimumHealthyPercent</code> is 0%. If a service is using the <code>ECS</code> deployment controller, the minimum healthy percent represents a lower limit on the number of tasks in a service that must remain in the <code>RUNNING</code> state during a deployment, as a percentage of the desired number of tasks (rounded up to the nearest integer), and while any container instances are in the <code>DRAINING</code> state if the service contains tasks using the EC2 launch type. This parameter enables you to deploy without using additional cluster capacity. For example, if your service has a desired number of four tasks and a minimum healthy percent of 50%, the scheduler might stop two existing tasks to free up cluster capacity before starting two new tasks. Tasks for services that <i>do not</i> use a load balancer are considered healthy if they're in the <code>RUNNING</code> state. Tasks for services that <i>do</i> use a load balancer are considered healthy if they're in the <code>RUNNING</code> state and they're reported as healthy by the load balancer. The default value for minimum healthy percent is 100%. If a service is using the <code>ECS</code> deployment controller, the <b>maximum percent</b> parameter represents an upper limit on the number of tasks in a service that are allowed in the <code>RUNNING</code> or <code>PENDING</code> state during a deployment, as a percentage of the desired number of tasks (rounded down to the nearest integer), and while any container instances are in the <code>DRAINING</code> state if the service contains tasks using the EC2 launch type. This parameter enables you to define the deployment batch size. For example, if your service has a desired number of four tasks and a maximum percent value of 200%, the scheduler may start four new tasks before stopping the four older tasks (provided that the cluster resources required to do this are available). The default value for maximum percent is 200%. If a service is using either the <code>CODE_DEPLOY</code> or <code>EXTERNAL</code> deployment controller types and tasks that use the EC2 launch type, the <b>minimum healthy percent</b> and <b>maximum percent</b> values are used only to define the lower and upper limit on the number of the tasks in the service that remain in the <code>RUNNING</code> state while the container instances are in the <code>DRAINING</code> state. If the tasks in the service use the Fargate launch type, the minimum healthy percent and maximum percent values aren't used, although they're currently visible when describing your service. When creating a service that uses the <code>EXTERNAL</code> deployment controller, you can specify only parameters that aren't controlled at the task set level. The only required parameter is the service name. You control your services using the <a>CreateTaskSet</a> operation. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/deployment-types.html">Amazon ECS Deployment Types</a> in the <i>Amazon Elastic Container Service Developer Guide</i>. When the service scheduler launches new tasks, it determines task placement in your cluster using the following logic: <ul> <li> Determine which of the container instances in your cluster can support your service's task definition (for example, they have the required CPU, memory, ports, and container instance attributes). </li> <li> By default, the service scheduler attempts to balance tasks across Availability Zones in this manner (although you can choose a different placement strategy) with the <code>placementStrategy</code> parameter): <ul> <li> Sort the valid container instances, giving priority to instances that have the fewest number of running tasks for this service in their respective Availability Zone. For example, if zone A has one running service task and zones B and C each have zero, valid container instances in either zone B or C are considered optimal for placement. </li> <li> Place the new service task on a valid container instance in an optimal Availability Zone (based on the previous steps), favoring container instances with the fewest number of running tasks for this service. </li> </ul> </li> </ul>

        :param self:
        :param propagateTags:  Specifies whether to propagate the tags from the task definition or the service to the tasks in the service. If no value is specified, the tags are not propagated. Tags can only be propagated to the tasks within the service during service creation. To add tags to a task after service creation, use the <a>TagResource</a> API action.<br/>enableECSManagedTags:  Specifies whether to enable Amazon ECS managed tags for the tasks within the service. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/ecs-using-tags.html">Tagging Your Amazon ECS Resources</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>tags:  The metadata that you apply to the service to help you categorize and organize them. Each tag consists of a key and an optional value, both of which you define. When a service is deleted, the tags are deleted as well. Tag keys can have a maximum character length of 128 characters, and tag values can have a maximum length of 256 characters.<br/>deploymentController:  The deployment controller to use for the service.:<br/>schedulingStrategy:  The scheduling strategy to use for the service. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/ecs_services.html">Services</a>. There are two service scheduler strategies available: <ul> <li>  <code>REPLICA</code>-The replica scheduling strategy places and maintains the desired number of tasks across your cluster. By default, the service scheduler spreads tasks across Availability Zones. You can use task placement strategies and constraints to customize task placement decisions. This scheduler strategy is required if the service is using the <code>CODE_DEPLOY</code> or <code>EXTERNAL</code> deployment controller types. </li> <li>  <code>DAEMON</code>-The daemon scheduling strategy deploys exactly one task on each active container instance that meets all of the task placement constraints that you specify in your cluster. When you're using this strategy, you don't need to specify a desired number of tasks, a task placement strategy, or use Service Auto Scaling policies. <note> Tasks using the Fargate launch type or the <code>CODE_DEPLOY</code> or <code>EXTERNAL</code> deployment controller types don't support the <code>DAEMON</code> scheduling strategy. </note> </li> </ul><br/>healthCheckGracePeriodSeconds:  The period of time, in seconds, that the Amazon ECS service scheduler should ignore unhealthy Elastic Load Balancing target health checks after a task has first started. This is only valid if your service is configured to use a load balancer. If your service's tasks take a while to start and respond to Elastic Load Balancing health checks, you can specify a health check grace period of up to 2,147,483,647 seconds. During that time, the ECS service scheduler ignores health check status. This grace period can prevent the ECS service scheduler from marking tasks as unhealthy and stopping them before they have time to come up.<br/>networkConfiguration:  The network configuration for the service. This parameter is required for task definitions that use the <code>awsvpc</code> network mode to receive their own elastic network interface, and it is not supported for other network modes. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task-networking.html">Task Networking</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.:<br/>placementStrategy:  The placement strategy objects to use for tasks in your service. You can specify a maximum of five strategy rules per service.<br/>placementConstraints:  An array of placement constraint objects to use for tasks in your service. You can specify a maximum of 10 constraints per task (this limit includes constraints in the task definition and those specified at runtime). <br/>deploymentConfiguration:  Optional deployment parameters that control how many tasks run during the deployment and the ordering of stopping and starting tasks.:<br/>role:  The name or full Amazon Resource Name (ARN) of the IAM role that allows Amazon ECS to make calls to your load balancer on your behalf. This parameter is only permitted if you are using a load balancer with your service and your task definition does not use the <code>awsvpc</code> network mode. If you specify the <code>role</code> parameter, you must also specify a load balancer object with the <code>loadBalancers</code> parameter. <important> If your account has already created the Amazon ECS service-linked role, that role is used by default for your service unless you specify a role here. The service-linked role is required if your task definition uses the <code>awsvpc</code> network mode, in which case you should not specify a role here. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/using-service-linked-roles.html">Using Service-Linked Roles for Amazon ECS</a> in the <i>Amazon Elastic Container Service Developer Guide</i>. </important> If your specified role has a path other than <code>/</code>, then you must either specify the full role ARN (this is recommended) or prefix the role name with the path. For example, if a role with the name <code>bar</code> has a path of <code>/foo/</code> then you would specify <code>/foo/bar</code> as the role name. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_identifiers.html#identifiers-friendly-names">Friendly Names and Paths</a> in the <i>IAM User Guide</i>.<br/>platformVersion:  The platform version that your tasks in the service are running on. A platform version is specified only for tasks using the Fargate launch type. If one isn't specified, the <code>LATEST</code> platform version is used by default. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/platform_versions.html">AWS Fargate Platform Versions</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>launchType:  The launch type on which to run your service. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/launch_types.html">Amazon ECS Launch Types</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>clientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. Up to 32 ASCII characters are allowed.<br/>desiredCount:  The number of instantiations of the specified task definition to place and keep running on your cluster.<br/>serviceRegistries:  The details of the service discovery registries to assign to this service. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/service-discovery.html">Service Discovery</a>. <note> Service discovery is supported for Fargate tasks if you are using platform version v1.1.0 or later. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/platform_versions.html">AWS Fargate Platform Versions</a>. </note><br/>loadBalancers:  A load balancer object representing the load balancer to use with your service. If the service is using the <code>ECS</code> deployment controller, you are limited to one load balancer or target group. If the service is using the <code>CODE_DEPLOY</code> deployment controller, the service is required to use either an Application Load Balancer or Network Load Balancer. When creating an AWS CodeDeploy deployment group, you specify two target groups (referred to as a <code>targetGroupPair</code>). During a deployment, AWS CodeDeploy determines which task set in your service has the status <code>PRIMARY</code> and associates one target group with it, and then associates the other target group with the replacement task set. The load balancer can also have up to two listeners: a required listener for production traffic and an optional listener that allows you perform validation tests with Lambda functions before routing production traffic to it. After you create a service using the <code>ECS</code> deployment controller, the load balancer name or target group ARN, container name, and container port specified in the service definition are immutable. If you are using the <code>CODE_DEPLOY</code> deployment controller, these values can be changed when updating the service. For Classic Load Balancers, this object must contain the load balancer name, the container name (as it appears in a container definition), and the container port to access from the load balancer. When a task from this service is placed on a container instance, the container instance is registered with the load balancer specified here. For Application Load Balancers and Network Load Balancers, this object must contain the load balancer target group ARN, the container name (as it appears in a container definition), and the container port to access from the load balancer. When a task from this service is placed on a container instance, the container instance and port combination is registered as a target in the target group specified here. Services with tasks that use the <code>awsvpc</code> network mode (for example, those with the Fargate launch type) only support Application Load Balancers and Network Load Balancers. Classic Load Balancers are not supported. Also, when you create any target groups for these services, you must choose <code>ip</code> as the target type, not <code>instance</code>, because tasks that use the <code>awsvpc</code> network mode are associated with an elastic network interface, not an Amazon EC2 instance.<br/>taskDefinition:  The <code>family</code> and <code>revision</code> (<code>family:revision</code>) or full ARN of the task definition to run in your service. If a <code>revision</code> is not specified, the latest <code>ACTIVE</code> revision is used. A task definition must be specified if the service is using the <code>ECS</code> deployment controller.<br/>serviceName:  The name of your service. Up to 255 letters (uppercase and lowercase), numbers, and hyphens are allowed. Service names must be unique within a cluster, but you can have similarly named services in multiple clusters within a Region or across multiple Regions.<br/>cluster:  The short name or full Amazon Resource Name (ARN) of the cluster on which to run your service. If you do not specify a cluster, the default cluster is assumed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>minimumHealthyPercent</i> If a service is using the rolling update (<code>ECS</code>) deployment type, the <b>minimum healthy percent</b> represents a lower limit on the number of tasks in a service that must remain in the <code>RUNNING</code> state during a deployment, as a percentage of the desired number of tasks (rounded up to the nearest integer), and while any container instances are in the <code>DRAINING</code> state if the service contains tasks using the EC2 launch type. This parameter enables you to deploy without using additional cluster capacity. For example, if your service has a desired number of four tasks and a minimum healthy percent of 50%, the scheduler may stop two existing tasks to free up cluster capacity before starting two new tasks. Tasks for services that <i>do not</i> use a load balancer are considered healthy if they are in the <code>RUNNING</code> state; tasks for services that <i>do</i> use a load balancer are considered healthy if they are in the <code>RUNNING</code> state and they are reported as healthy by the load balancer. The default value for minimum healthy percent is 100%. If a service is using the blue/green (<code>CODE_DEPLOY</code>) or <code>EXTERNAL</code> deployment types and tasks that use the EC2 launch type, the <b>minimum healthy percent</b> value is set to the default value and is used to define the lower limit on the number of the tasks in the service that remain in the <code>RUNNING</code> state while the container instances are in the <code>DRAINING</code> state. If the tasks in the service use the Fargate launch type, the minimum healthy percent value is not used, although it is returned when describing your service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>maximumPercent</i> If a service is using the rolling update (<code>ECS</code>) deployment type, the <b>maximum percent</b> parameter represents an upper limit on the number of tasks in a service that are allowed in the <code>RUNNING</code> or <code>PENDING</code> state during a deployment, as a percentage of the desired number of tasks (rounded down to the nearest integer), and while any container instances are in the <code>DRAINING</code> state if the service contains tasks using the EC2 launch type. This parameter enables you to define the deployment batch size. For example, if your service has a desired number of four tasks and a maximum percent value of 200%, the scheduler may start four new tasks before stopping the four older tasks (provided that the cluster resources required to do this are available). The default value for maximum percent is 200%. If a service is using the blue/green (<code>CODE_DEPLOY</code>) or <code>EXTERNAL</code> deployment types and tasks that use the EC2 launch type, the <b>maximum percent</b> value is set to the default value and is used to define the upper limit on the number of the tasks in the service that remain in the <code>RUNNING</code> state while the container instances are in the <code>DRAINING</code> state. If the tasks in the service use the Fargate launch type, the maximum percent value is not used, although it is returned when describing your service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>awsvpcConfiguration</i> The VPC subnets and security groups associated with a task. <note> All specified subnets and security groups must be from the same VPC. </note>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>awsvpcConfiguration</b>: The VPC subnets and security groups associated with a task. <note> All specified subnets and security groups must be from the same VPC. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>assignPublicIp</i> Whether the task's elastic network interface receives a public IP address. The default value is <code>DISABLED</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>securityGroups</i> The security groups associated with the task or service. If you do not specify a security group, the default security group for the VPC is used. There is a limit of 5 security groups that can be specified per <code>AwsVpcConfiguration</code>. <note> All specified security groups must be from the same VPC. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>subnets</i> The subnets associated with the task or service. There is a limit of 16 subnets that can be specified per <code>AwsVpcConfiguration</code>. <note> All specified subnets must be from the same VPC. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The deployment controller type to use. There are three deployment controller types available: <dl> <dt>ECS</dt> <dd> The rolling update (<code>ECS</code>) deployment type involves replacing the current running version of the container with the latest version. The number of containers Amazon ECS adds or removes from the service during a rolling update is controlled by adjusting the minimum and maximum number of healthy tasks allowed during a service deployment, as specified in the <a>DeploymentConfiguration</a>. </dd> <dt>CODE_DEPLOY</dt> <dd> The blue/green (<code>CODE_DEPLOY</code>) deployment type uses the blue/green deployment model powered by AWS CodeDeploy, which allows you to verify a new deployment of a service before sending production traffic to it. </dd> <dt>EXTERNAL</dt> <dd> The external (<code>EXTERNAL</code>) deployment type enables you to use any third-party deployment controller for full control over the deployment process for an Amazon ECS service. </dd> </dl><br/>
        :return: service:  The full description of your service following the create call. If a service is using the <code>ECS</code> deployment controller, the <code>deploymentController</code> and <code>taskSets</code> parameters will not be returned. If the service is using the <code>CODE_DEPLOY</code> deployment controller, the <code>deploymentController</code>, <code>taskSets</code> and <code>deployments</code> parameters will be returned, however the <code>deployments</code> parameter will be an empty list.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>propagateTags</i> Specifies whether to propagate the tags from the task definition or the service to the task. If no value is specified, the tags are not propagated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableECSManagedTags</i> Specifies whether to enable Amazon ECS managed tags for the tasks in the service. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/ecs-using-tags.html">Tagging Your Amazon ECS Resources</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdBy</i> The principal that created the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> The metadata that you apply to the service to help you categorize and organize them. Each tag consists of a key and an optional value, both of which you define. Tag keys can have a maximum character length of 128 characters, and tag values can have a maximum length of 256 characters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentController</i> The deployment controller type the service is using.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>deploymentController</b>: The deployment controller type the service is using.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>schedulingStrategy</i> The scheduling strategy to use for the service. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/ecs_services.html">Services</a>. There are two service scheduler strategies available: <ul> <li>  <code>REPLICA</code>-The replica scheduling strategy places and maintains the desired number of tasks across your cluster. By default, the service scheduler spreads tasks across Availability Zones. You can use task placement strategies and constraints to customize task placement decisions. </li> <li>  <code>DAEMON</code>-The daemon scheduling strategy deploys exactly one task on each container instance in your cluster. When you are using this strategy, do not specify a desired number of tasks or any task placement strategies. <note> Fargate tasks do not support the <code>DAEMON</code> scheduling strategy. </note> </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>healthCheckGracePeriodSeconds</i> The period of time, in seconds, that the Amazon ECS service scheduler ignores unhealthy Elastic Load Balancing target health checks after a task has first started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>networkConfiguration</i> The VPC subnet and security group configuration for tasks that receive their own elastic network interface by using the <code>awsvpc</code> networking mode.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>networkConfiguration</b>: The VPC subnet and security group configuration for tasks that receive their own elastic network interface by using the <code>awsvpc</code> networking mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>placementStrategy</i> The placement strategy that determines how tasks for the service are placed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>placementConstraints</i> The placement constraints for the tasks in the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix timestamp for when the service was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>events</i> The event stream for your service. A maximum of 100 of the latest events are displayed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The ARN of the IAM role associated with the service that allows the Amazon ECS container agent to register container instances with an Elastic Load Balancing load balancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deployments</i> The current state of deployments for the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>taskSets</i> Information about a set of Amazon ECS tasks in either an AWS CodeDeploy or an <code>EXTERNAL</code> deployment. An Amazon ECS task set includes details such as the desired number of tasks, how many tasks are running, and whether the task set serves production traffic.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentConfiguration</i> Optional deployment parameters that control how many tasks run during the deployment and the ordering of stopping and starting tasks.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>deploymentConfiguration</b>: Optional deployment parameters that control how many tasks run during the deployment and the ordering of stopping and starting tasks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>taskDefinition</i> The task definition to use for tasks in the service. This value is specified when the service is created with <a>CreateService</a>, and it can be modified with <a>UpdateService</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>platformVersion</i> The platform version on which to run your service. A platform version is only specified for tasks using the Fargate launch type. If one is not specified, the <code>LATEST</code> platform version is used by default. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/platform_versions.html">AWS Fargate Platform Versions</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>launchType</i> The launch type on which your service is running. If no value is specified, it will default to <code>EC2</code>. Valid values include <code>EC2</code> and <code>FARGATE</code>. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/launch_types.html">Amazon ECS Launch Types</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pendingCount</i> The number of tasks in the cluster that are in the <code>PENDING</code> state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>runningCount</i> The number of tasks in the cluster that are in the <code>RUNNING</code> state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>desiredCount</i> The desired number of instantiations of the task definition to keep running on the service. This value is specified when the service is created with <a>CreateService</a>, and it can be modified with <a>UpdateService</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the service. The valid values are <code>ACTIVE</code>, <code>DRAINING</code>, or <code>INACTIVE</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceRegistries</i> The details of the service discovery registries to assign to this service. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/service-discovery.html">Service Discovery</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>loadBalancers</i> A list of Elastic Load Balancing load balancer objects, containing the load balancer name, the container name (as it appears in a container definition), and the container port to access from the load balancer. Services with tasks that use the <code>awsvpc</code> network mode (for example, those with the Fargate launch type) only support Application Load Balancers and Network Load Balancers. Classic Load Balancers are not supported. Also, when you create any target groups for these services, you must choose <code>ip</code> as the target type, not <code>instance</code>. Tasks that use the <code>awsvpc</code> network mode are associated with an elastic network interface, not an Amazon EC2 instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clusterArn</i> The Amazon Resource Name (ARN) of the cluster that hosts the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceName</i> The name of your service. Up to 255 letters (uppercase and lowercase), numbers, and hyphens are allowed. Service names must be unique within a cluster, but you can have similarly named services in multiple clusters within a Region or across multiple Regions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceArn</i> The ARN that identifies the service. The ARN contains the <code>arn:aws:ecs</code> namespace, followed by the Region of the service, the AWS account ID of the service owner, the <code>service</code> namespace, and then the service name. For example, <code>arn:aws:ecs:region:012345678910:service/my-service</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>minimumHealthyPercent</i> If a service is using the rolling update (<code>ECS</code>) deployment type, the <b>minimum healthy percent</b> represents a lower limit on the number of tasks in a service that must remain in the <code>RUNNING</code> state during a deployment, as a percentage of the desired number of tasks (rounded up to the nearest integer), and while any container instances are in the <code>DRAINING</code> state if the service contains tasks using the EC2 launch type. This parameter enables you to deploy without using additional cluster capacity. For example, if your service has a desired number of four tasks and a minimum healthy percent of 50%, the scheduler may stop two existing tasks to free up cluster capacity before starting two new tasks. Tasks for services that <i>do not</i> use a load balancer are considered healthy if they are in the <code>RUNNING</code> state; tasks for services that <i>do</i> use a load balancer are considered healthy if they are in the <code>RUNNING</code> state and they are reported as healthy by the load balancer. The default value for minimum healthy percent is 100%. If a service is using the blue/green (<code>CODE_DEPLOY</code>) or <code>EXTERNAL</code> deployment types and tasks that use the EC2 launch type, the <b>minimum healthy percent</b> value is set to the default value and is used to define the lower limit on the number of the tasks in the service that remain in the <code>RUNNING</code> state while the container instances are in the <code>DRAINING</code> state. If the tasks in the service use the Fargate launch type, the minimum healthy percent value is not used, although it is returned when describing your service.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>maximumPercent</i> If a service is using the rolling update (<code>ECS</code>) deployment type, the <b>maximum percent</b> parameter represents an upper limit on the number of tasks in a service that are allowed in the <code>RUNNING</code> or <code>PENDING</code> state during a deployment, as a percentage of the desired number of tasks (rounded down to the nearest integer), and while any container instances are in the <code>DRAINING</code> state if the service contains tasks using the EC2 launch type. This parameter enables you to define the deployment batch size. For example, if your service has a desired number of four tasks and a maximum percent value of 200%, the scheduler may start four new tasks before stopping the four older tasks (provided that the cluster resources required to do this are available). The default value for maximum percent is 200%. If a service is using the blue/green (<code>CODE_DEPLOY</code>) or <code>EXTERNAL</code> deployment types and tasks that use the EC2 launch type, the <b>maximum percent</b> value is set to the default value and is used to define the upper limit on the number of the tasks in the service that remain in the <code>RUNNING</code> state while the container instances are in the <code>DRAINING</code> state. If the tasks in the service use the Fargate launch type, the maximum percent value is not used, although it is returned when describing your service.<br/>The VPC subnets and security groups associated with a task. <note> All specified subnets and security groups must be from the same VPC. </note>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>awsvpcConfiguration</b>: The VPC subnets and security groups associated with a task. <note> All specified subnets and security groups must be from the same VPC. </note>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The deployment controller type to use. There are three deployment controller types available: <dl> <dt>ECS</dt> <dd> The rolling update (<code>ECS</code>) deployment type involves replacing the current running version of the container with the latest version. The number of containers Amazon ECS adds or removes from the service during a rolling update is controlled by adjusting the minimum and maximum number of healthy tasks allowed during a service deployment, as specified in the <a>DeploymentConfiguration</a>. </dd> <dt>CODE_DEPLOY</dt> <dd> The blue/green (<code>CODE_DEPLOY</code>) deployment type uses the blue/green deployment model powered by AWS CodeDeploy, which allows you to verify a new deployment of a service before sending production traffic to it. </dd> <dt>EXTERNAL</dt> <dd> The external (<code>EXTERNAL</code>) deployment type enables you to use any third-party deployment controller for full control over the deployment process for an Amazon ECS service. </dd> </dl><br/>        """
        pass

    def create_task_set(self, taskDefinition: str, cluster: str, service: str, externalId: str=None, networkConfiguration: NetworkConfiguration=None, loadBalancers: List[LoadBalancer]=None, serviceRegistries: List[ServiceRegistry]=None, launchType: str=None, platformVersion: str=None, scale: Scale=None, clientToken: str=None) -> CreateTaskSetResponse:
        """Create a task set in the specified cluster and service. This is used when a service uses the <code>EXTERNAL</code> deployment controller type. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/deployment-types.html">Amazon ECS Deployment Types</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.

        :param self:
        :param clientToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. Up to 32 ASCII characters are allowed.<br/>scale:  A floating-point percentage of the desired number of tasks to place and keep running in the task set.:<br/>platformVersion:  The platform version that the tasks in the task set should use. A platform version is specified only for tasks using the Fargate launch type. If one isn't specified, the <code>LATEST</code> platform version is used by default.<br/>launchType:  The launch type that new tasks in the task set will use. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/launch_types.html">Amazon ECS Launch Types</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>serviceRegistries:  The details of the service discovery registries to assign to this task set. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/service-discovery.html">Service Discovery</a>.<br/>loadBalancers:  A load balancer object representing the load balancer to use with the task set. The supported load balancer types are either an Application Load Balancer or a Network Load Balancer.<br/>networkConfiguration:  An object representing the network configuration for a task or service.:<br/>taskDefinition:  The task definition for the tasks in the task set to use.<br/>externalId:  An optional non-unique tag that identifies this task set in external systems. If the task set is associated with a service discovery registry, the tasks in this task set will have the <code>ECS_TASK_SET_EXTERNAL_ID</code> AWS Cloud Map attribute set to the provided value.<br/>cluster:  The short name or full Amazon Resource Name (ARN) of the cluster that hosts the service to create the task set in.<br/>service:  The short name or full Amazon Resource Name (ARN) of the service to create the task set in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>awsvpcConfiguration</i> The VPC subnets and security groups associated with a task. <note> All specified subnets and security groups must be from the same VPC. </note>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>awsvpcConfiguration</b>: The VPC subnets and security groups associated with a task. <note> All specified subnets and security groups must be from the same VPC. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>assignPublicIp</i> Whether the task's elastic network interface receives a public IP address. The default value is <code>DISABLED</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>securityGroups</i> The security groups associated with the task or service. If you do not specify a security group, the default security group for the VPC is used. There is a limit of 5 security groups that can be specified per <code>AwsVpcConfiguration</code>. <note> All specified security groups must be from the same VPC. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>subnets</i> The subnets associated with the task or service. There is a limit of 16 subnets that can be specified per <code>AwsVpcConfiguration</code>. <note> All specified subnets must be from the same VPC. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>unit</i> The unit of measure for the scale value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>value</i> The value, specified as a percent total of a service's <code>desiredCount</code>, to scale the task set. Accepted values are numbers between 0 and 100.<br/>
        :return: taskSet:  Information about a set of Amazon ECS tasks in either an AWS CodeDeploy or an <code>EXTERNAL</code> deployment. An Amazon ECS task set includes details such as the desired number of tasks, how many tasks are running, and whether the task set serves production traffic.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stabilityStatusAt</i> The Unix timestamp for when the task set stability status was retrieved.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stabilityStatus</i> The stability status, which indicates whether the task set has reached a steady state. If the following conditions are met, the task set will be in <code>STEADY_STATE</code>: <ul> <li> The task <code>runningCount</code> is equal to the <code>computedDesiredCount</code>. </li> <li> The <code>pendingCount</code> is <code>0</code>. </li> <li> There are no tasks running on container instances in the <code>DRAINING</code> status. </li> <li> All tasks are reporting a healthy status from the load balancers, service discovery, and container health checks. <note> If a <code>healthCheckGracePeriodSeconds</code> value was set when the service was created, you may see a <code>STEADY_STATE</code> reached since unhealthy Elastic Load Balancing target health checks will be ignored until it expires. </note> </li> </ul> If any of those conditions are not met, the stability status returns <code>STABILIZING</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>scale</i> A floating-point percentage of the desired number of tasks to place and keep running in the task set.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>scale</b>: A floating-point percentage of the desired number of tasks to place and keep running in the task set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceRegistries</i> The details of the service discovery registries to assign to this task set. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/service-discovery.html">Service Discovery</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>loadBalancers</i> Details on a load balancer that is used with a task set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>networkConfiguration</i> The network configuration for the task set.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>networkConfiguration</b>: The network configuration for the task set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>platformVersion</i> The platform version on which the tasks in the task set are running. A platform version is only specified for tasks using the Fargate launch type. If one is not specified, the <code>LATEST</code> platform version is used by default. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/platform_versions.html">AWS Fargate Platform Versions</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>launchType</i> The launch type the tasks in the task set are using. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/launch_types.html">Amazon ECS Launch Types</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>updatedAt</i> The Unix timestamp for when the task set was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix timestamp for when the task set was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>runningCount</i> The number of tasks in the task set that are in the <code>RUNNING</code> status during a deployment. A task in the <code>RUNNING</code> state is running and ready for use.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pendingCount</i> The number of tasks in the task set that are in the <code>PENDING</code> status during a deployment. A task in the <code>PENDING</code> state is preparing to enter the <code>RUNNING</code> state. A task set enters the <code>PENDING</code> status when it launches for the first time or when it is restarted after being in the <code>STOPPED</code> state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>computedDesiredCount</i> The computed desired count for the task set. This is calculated by multiplying the service's <code>desiredCount</code> by the task set's <code>scale</code> percentage. The result is always rounded up. For example, if the computed desired count is 1.2, it rounds up to 2 tasks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>taskDefinition</i> The task definition the task set is using.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the task set. The following describes each state: <dl> <dt>PRIMARY</dt> <dd> The task set is serving production traffic. </dd> <dt>ACTIVE</dt> <dd> The task set is not serving production traffic. </dd> <dt>DRAINING</dt> <dd> The tasks in the task set are being stopped and their corresponding targets are being deregistered from their target group. </dd> </dl><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>externalId</i> The external ID associated with the task set. If a task set is created by an AWS CodeDeploy deployment, the <code>externalId</code> parameter contains the AWS CodeDeploy deployment ID. If a task set is created for an external deployment and is associated with a service discovery registry, the <code>externalId</code> parameter contains the <code>ECS_TASK_SET_EXTERNAL_ID</code> AWS Cloud Map attribute.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>startedBy</i> The tag specified when a task set is started. If the task set is created by an AWS CodeDeploy deployment, the <code>startedBy</code> parameter is <code>CODE_DEPLOY</code>. For a task set created for an external deployment, the startedBy field isn't used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clusterArn</i> The Amazon Resource Name (ARN) of the cluster that the service that hosts the task set exists in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceArn</i> The Amazon Resource Name (ARN) of the service the task set exists in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>taskSetArn</i> The Amazon Resource Name (ARN) of the task set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the task set.<br/>The VPC subnets and security groups associated with a task. <note> All specified subnets and security groups must be from the same VPC. </note>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>awsvpcConfiguration</b>: The VPC subnets and security groups associated with a task. <note> All specified subnets and security groups must be from the same VPC. </note>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>unit</i> The unit of measure for the scale value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>value</i> The value, specified as a percent total of a service's <code>desiredCount</code>, to scale the task set. Accepted values are numbers between 0 and 100.<br/>        """
        pass

    def delete_account_setting(self, name: str, principalArn: str=None) -> DeleteAccountSettingResponse:
        """Disables an account setting for a specified IAM user, IAM role, or the root user for an account.

        :param self:
        :param principalArn:  The ARN of the principal, which can be an IAM user, IAM role, or the root user. If you specify the root user, it disables the account setting for all IAM users, IAM roles, and the root user of the account unless an IAM user or role explicitly overrides these settings. If this field is omitted, the setting is changed only for the authenticated user.<br/>name:  The resource name for which to disable the account setting. If <code>serviceLongArnFormat</code> is specified, the ARN for your Amazon ECS services is affected. If <code>taskLongArnFormat</code> is specified, the ARN and resource ID for your Amazon ECS tasks is affected. If <code>containerInstanceLongArnFormat</code> is specified, the ARN and resource ID for your Amazon ECS container instances is affected. If <code>awsvpcTrunking</code> is specified, the ENI limit for your Amazon ECS container instances is affected.<br/>
        :return: setting:  The account setting for the specified principal ARN.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>principalArn</i> The ARN of the principal, which can be an IAM user, IAM role, or the root user. If this field is omitted, the authenticated user is assumed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>value</i> Whether the account setting is enabled or disabled for the specified resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The Amazon ECS resource name.<br/>        """
        pass

    def delete_attributes(self, attributes: List[Attribute], cluster: str=None) -> DeleteAttributesResponse:
        """Deletes one or more custom attributes from an Amazon ECS resource.

        :param self:
        :param attributes:  The attributes to delete from your resource. You can specify up to 10 attributes per request. For custom attributes, specify the attribute name and target ID, but do not specify the value. If you specify the target ID using the short form, you must also specify the target type.<br/>cluster:  The short name or full Amazon Resource Name (ARN) of the cluster that contains the resource to delete attributes. If you do not specify a cluster, the default cluster is assumed.<br/>
        :return: attributes:  A list of attribute objects that were successfully deleted from your resource.<br/>        """
        pass

    def delete_cluster(self, cluster: str) -> DeleteClusterResponse:
        """Deletes the specified cluster. You must deregister all container instances from this cluster before you may delete it. You can list the container instances in a cluster with <a>ListContainerInstances</a> and deregister them with <a>DeregisterContainerInstance</a>.

        :param self:
        :param cluster:  The short name or full Amazon Resource Name (ARN) of the cluster to delete.<br/>
        :return: cluster:  The full description of the deleted cluster.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>settings</i> The settings for the cluster. This parameter indicates whether CloudWatch Container Insights is enabled or disabled for a cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> The metadata that you apply to the cluster to help you categorize and organize them. Each tag consists of a key and an optional value, both of which you define. Tag keys can have a maximum character length of 128 characters, and tag values can have a maximum length of 256 characters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statistics</i> Additional information about your clusters that are separated by launch type, including: <ul> <li> runningEC2TasksCount </li> <li> RunningFargateTasksCount </li> <li> pendingEC2TasksCount </li> <li> pendingFargateTasksCount </li> <li> activeEC2ServiceCount </li> <li> activeFargateServiceCount </li> <li> drainingEC2ServiceCount </li> <li> drainingFargateServiceCount </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>activeServicesCount</i> The number of services that are running on the cluster in an <code>ACTIVE</code> state. You can view these services with <a>ListServices</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pendingTasksCount</i> The number of tasks in the cluster that are in the <code>PENDING</code> state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>runningTasksCount</i> The number of tasks in the cluster that are in the <code>RUNNING</code> state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>registeredContainerInstancesCount</i> The number of container instances registered into the cluster. This includes container instances in both <code>ACTIVE</code> and <code>DRAINING</code> status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the cluster. The valid values are <code>ACTIVE</code> or <code>INACTIVE</code>. <code>ACTIVE</code> indicates that you can register container instances with the cluster and the associated instances can accept tasks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clusterName</i> A user-generated string that you use to identify your cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clusterArn</i> The Amazon Resource Name (ARN) that identifies the cluster. The ARN contains the <code>arn:aws:ecs</code> namespace, followed by the Region of the cluster, the AWS account ID of the cluster owner, the <code>cluster</code> namespace, and then the cluster name. For example, <code>arn:aws:ecs:region:012345678910:cluster/test</code>.<br/>        """
        pass

    def delete_service(self, service: str, cluster: str=None, force: BoxedBoolean=None) -> DeleteServiceResponse:
        """Deletes a specified service within a cluster. You can delete a service if you have no running tasks in it and the desired task count is zero. If the service is actively maintaining tasks, you cannot delete it, and you must update the service to a desired task count of zero. For more information, see <a>UpdateService</a>. <note> When you delete a service, if there are still running tasks that require cleanup, the service status moves from <code>ACTIVE</code> to <code>DRAINING</code>, and the service is no longer visible in the console or in the <a>ListServices</a> API operation. After the tasks have stopped, then the service status moves from <code>DRAINING</code> to <code>INACTIVE</code>. Services in the <code>DRAINING</code> or <code>INACTIVE</code> status can still be viewed with the <a>DescribeServices</a> API operation. However, in the future, <code>INACTIVE</code> services may be cleaned up and purged from Amazon ECS record keeping, and <a>DescribeServices</a> calls on those services return a <code>ServiceNotFoundException</code> error. </note> <important> If you attempt to create a new service with the same name as an existing service in either <code>ACTIVE</code> or <code>DRAINING</code> status, you receive an error. </important>

        :param self:
        :param force:  If <code>true</code>, allows you to delete a service even if it has not been scaled down to zero tasks. It is only necessary to use this if the service is using the <code>REPLICA</code> scheduling strategy.<br/>service:  The name of the service to delete.<br/>cluster:  The short name or full Amazon Resource Name (ARN) of the cluster that hosts the service to delete. If you do not specify a cluster, the default cluster is assumed.<br/>
        :return: service:  The full description of the deleted service.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>propagateTags</i> Specifies whether to propagate the tags from the task definition or the service to the task. If no value is specified, the tags are not propagated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableECSManagedTags</i> Specifies whether to enable Amazon ECS managed tags for the tasks in the service. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/ecs-using-tags.html">Tagging Your Amazon ECS Resources</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdBy</i> The principal that created the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> The metadata that you apply to the service to help you categorize and organize them. Each tag consists of a key and an optional value, both of which you define. Tag keys can have a maximum character length of 128 characters, and tag values can have a maximum length of 256 characters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentController</i> The deployment controller type the service is using.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>deploymentController</b>: The deployment controller type the service is using.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>schedulingStrategy</i> The scheduling strategy to use for the service. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/ecs_services.html">Services</a>. There are two service scheduler strategies available: <ul> <li>  <code>REPLICA</code>-The replica scheduling strategy places and maintains the desired number of tasks across your cluster. By default, the service scheduler spreads tasks across Availability Zones. You can use task placement strategies and constraints to customize task placement decisions. </li> <li>  <code>DAEMON</code>-The daemon scheduling strategy deploys exactly one task on each container instance in your cluster. When you are using this strategy, do not specify a desired number of tasks or any task placement strategies. <note> Fargate tasks do not support the <code>DAEMON</code> scheduling strategy. </note> </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>healthCheckGracePeriodSeconds</i> The period of time, in seconds, that the Amazon ECS service scheduler ignores unhealthy Elastic Load Balancing target health checks after a task has first started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>networkConfiguration</i> The VPC subnet and security group configuration for tasks that receive their own elastic network interface by using the <code>awsvpc</code> networking mode.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>networkConfiguration</b>: The VPC subnet and security group configuration for tasks that receive their own elastic network interface by using the <code>awsvpc</code> networking mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>placementStrategy</i> The placement strategy that determines how tasks for the service are placed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>placementConstraints</i> The placement constraints for the tasks in the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix timestamp for when the service was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>events</i> The event stream for your service. A maximum of 100 of the latest events are displayed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The ARN of the IAM role associated with the service that allows the Amazon ECS container agent to register container instances with an Elastic Load Balancing load balancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deployments</i> The current state of deployments for the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>taskSets</i> Information about a set of Amazon ECS tasks in either an AWS CodeDeploy or an <code>EXTERNAL</code> deployment. An Amazon ECS task set includes details such as the desired number of tasks, how many tasks are running, and whether the task set serves production traffic.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentConfiguration</i> Optional deployment parameters that control how many tasks run during the deployment and the ordering of stopping and starting tasks.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>deploymentConfiguration</b>: Optional deployment parameters that control how many tasks run during the deployment and the ordering of stopping and starting tasks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>taskDefinition</i> The task definition to use for tasks in the service. This value is specified when the service is created with <a>CreateService</a>, and it can be modified with <a>UpdateService</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>platformVersion</i> The platform version on which to run your service. A platform version is only specified for tasks using the Fargate launch type. If one is not specified, the <code>LATEST</code> platform version is used by default. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/platform_versions.html">AWS Fargate Platform Versions</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>launchType</i> The launch type on which your service is running. If no value is specified, it will default to <code>EC2</code>. Valid values include <code>EC2</code> and <code>FARGATE</code>. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/launch_types.html">Amazon ECS Launch Types</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pendingCount</i> The number of tasks in the cluster that are in the <code>PENDING</code> state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>runningCount</i> The number of tasks in the cluster that are in the <code>RUNNING</code> state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>desiredCount</i> The desired number of instantiations of the task definition to keep running on the service. This value is specified when the service is created with <a>CreateService</a>, and it can be modified with <a>UpdateService</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the service. The valid values are <code>ACTIVE</code>, <code>DRAINING</code>, or <code>INACTIVE</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceRegistries</i> The details of the service discovery registries to assign to this service. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/service-discovery.html">Service Discovery</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>loadBalancers</i> A list of Elastic Load Balancing load balancer objects, containing the load balancer name, the container name (as it appears in a container definition), and the container port to access from the load balancer. Services with tasks that use the <code>awsvpc</code> network mode (for example, those with the Fargate launch type) only support Application Load Balancers and Network Load Balancers. Classic Load Balancers are not supported. Also, when you create any target groups for these services, you must choose <code>ip</code> as the target type, not <code>instance</code>. Tasks that use the <code>awsvpc</code> network mode are associated with an elastic network interface, not an Amazon EC2 instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clusterArn</i> The Amazon Resource Name (ARN) of the cluster that hosts the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceName</i> The name of your service. Up to 255 letters (uppercase and lowercase), numbers, and hyphens are allowed. Service names must be unique within a cluster, but you can have similarly named services in multiple clusters within a Region or across multiple Regions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceArn</i> The ARN that identifies the service. The ARN contains the <code>arn:aws:ecs</code> namespace, followed by the Region of the service, the AWS account ID of the service owner, the <code>service</code> namespace, and then the service name. For example, <code>arn:aws:ecs:region:012345678910:service/my-service</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>minimumHealthyPercent</i> If a service is using the rolling update (<code>ECS</code>) deployment type, the <b>minimum healthy percent</b> represents a lower limit on the number of tasks in a service that must remain in the <code>RUNNING</code> state during a deployment, as a percentage of the desired number of tasks (rounded up to the nearest integer), and while any container instances are in the <code>DRAINING</code> state if the service contains tasks using the EC2 launch type. This parameter enables you to deploy without using additional cluster capacity. For example, if your service has a desired number of four tasks and a minimum healthy percent of 50%, the scheduler may stop two existing tasks to free up cluster capacity before starting two new tasks. Tasks for services that <i>do not</i> use a load balancer are considered healthy if they are in the <code>RUNNING</code> state; tasks for services that <i>do</i> use a load balancer are considered healthy if they are in the <code>RUNNING</code> state and they are reported as healthy by the load balancer. The default value for minimum healthy percent is 100%. If a service is using the blue/green (<code>CODE_DEPLOY</code>) or <code>EXTERNAL</code> deployment types and tasks that use the EC2 launch type, the <b>minimum healthy percent</b> value is set to the default value and is used to define the lower limit on the number of the tasks in the service that remain in the <code>RUNNING</code> state while the container instances are in the <code>DRAINING</code> state. If the tasks in the service use the Fargate launch type, the minimum healthy percent value is not used, although it is returned when describing your service.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>maximumPercent</i> If a service is using the rolling update (<code>ECS</code>) deployment type, the <b>maximum percent</b> parameter represents an upper limit on the number of tasks in a service that are allowed in the <code>RUNNING</code> or <code>PENDING</code> state during a deployment, as a percentage of the desired number of tasks (rounded down to the nearest integer), and while any container instances are in the <code>DRAINING</code> state if the service contains tasks using the EC2 launch type. This parameter enables you to define the deployment batch size. For example, if your service has a desired number of four tasks and a maximum percent value of 200%, the scheduler may start four new tasks before stopping the four older tasks (provided that the cluster resources required to do this are available). The default value for maximum percent is 200%. If a service is using the blue/green (<code>CODE_DEPLOY</code>) or <code>EXTERNAL</code> deployment types and tasks that use the EC2 launch type, the <b>maximum percent</b> value is set to the default value and is used to define the upper limit on the number of the tasks in the service that remain in the <code>RUNNING</code> state while the container instances are in the <code>DRAINING</code> state. If the tasks in the service use the Fargate launch type, the maximum percent value is not used, although it is returned when describing your service.<br/>The VPC subnets and security groups associated with a task. <note> All specified subnets and security groups must be from the same VPC. </note>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>awsvpcConfiguration</b>: The VPC subnets and security groups associated with a task. <note> All specified subnets and security groups must be from the same VPC. </note>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The deployment controller type to use. There are three deployment controller types available: <dl> <dt>ECS</dt> <dd> The rolling update (<code>ECS</code>) deployment type involves replacing the current running version of the container with the latest version. The number of containers Amazon ECS adds or removes from the service during a rolling update is controlled by adjusting the minimum and maximum number of healthy tasks allowed during a service deployment, as specified in the <a>DeploymentConfiguration</a>. </dd> <dt>CODE_DEPLOY</dt> <dd> The blue/green (<code>CODE_DEPLOY</code>) deployment type uses the blue/green deployment model powered by AWS CodeDeploy, which allows you to verify a new deployment of a service before sending production traffic to it. </dd> <dt>EXTERNAL</dt> <dd> The external (<code>EXTERNAL</code>) deployment type enables you to use any third-party deployment controller for full control over the deployment process for an Amazon ECS service. </dd> </dl><br/>        """
        pass

    def delete_task_set(self, taskSet: str, service: str, cluster: str, force: BoxedBoolean=None) -> DeleteTaskSetResponse:
        """Deletes a specified task set within a service. This is used when a service uses the <code>EXTERNAL</code> deployment controller type. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/deployment-types.html">Amazon ECS Deployment Types</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.

        :param self:
        :param force:  If <code>true</code>, this allows you to delete a task set even if it hasn't been scaled down to zero.<br/>taskSet:  The task set ID or full Amazon Resource Name (ARN) of the task set to delete.<br/>service:  The short name or full Amazon Resource Name (ARN) of the service that hosts the task set to delete.<br/>cluster:  The short name or full Amazon Resource Name (ARN) of the cluster that hosts the service that the task set exists in to delete.<br/>
        :return: taskSet:  Information about a set of Amazon ECS tasks in either an AWS CodeDeploy or an <code>EXTERNAL</code> deployment. An Amazon ECS task set includes details such as the desired number of tasks, how many tasks are running, and whether the task set serves production traffic.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stabilityStatusAt</i> The Unix timestamp for when the task set stability status was retrieved.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stabilityStatus</i> The stability status, which indicates whether the task set has reached a steady state. If the following conditions are met, the task set will be in <code>STEADY_STATE</code>: <ul> <li> The task <code>runningCount</code> is equal to the <code>computedDesiredCount</code>. </li> <li> The <code>pendingCount</code> is <code>0</code>. </li> <li> There are no tasks running on container instances in the <code>DRAINING</code> status. </li> <li> All tasks are reporting a healthy status from the load balancers, service discovery, and container health checks. <note> If a <code>healthCheckGracePeriodSeconds</code> value was set when the service was created, you may see a <code>STEADY_STATE</code> reached since unhealthy Elastic Load Balancing target health checks will be ignored until it expires. </note> </li> </ul> If any of those conditions are not met, the stability status returns <code>STABILIZING</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>scale</i> A floating-point percentage of the desired number of tasks to place and keep running in the task set.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>scale</b>: A floating-point percentage of the desired number of tasks to place and keep running in the task set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceRegistries</i> The details of the service discovery registries to assign to this task set. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/service-discovery.html">Service Discovery</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>loadBalancers</i> Details on a load balancer that is used with a task set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>networkConfiguration</i> The network configuration for the task set.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>networkConfiguration</b>: The network configuration for the task set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>platformVersion</i> The platform version on which the tasks in the task set are running. A platform version is only specified for tasks using the Fargate launch type. If one is not specified, the <code>LATEST</code> platform version is used by default. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/platform_versions.html">AWS Fargate Platform Versions</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>launchType</i> The launch type the tasks in the task set are using. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/launch_types.html">Amazon ECS Launch Types</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>updatedAt</i> The Unix timestamp for when the task set was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix timestamp for when the task set was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>runningCount</i> The number of tasks in the task set that are in the <code>RUNNING</code> status during a deployment. A task in the <code>RUNNING</code> state is running and ready for use.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pendingCount</i> The number of tasks in the task set that are in the <code>PENDING</code> status during a deployment. A task in the <code>PENDING</code> state is preparing to enter the <code>RUNNING</code> state. A task set enters the <code>PENDING</code> status when it launches for the first time or when it is restarted after being in the <code>STOPPED</code> state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>computedDesiredCount</i> The computed desired count for the task set. This is calculated by multiplying the service's <code>desiredCount</code> by the task set's <code>scale</code> percentage. The result is always rounded up. For example, if the computed desired count is 1.2, it rounds up to 2 tasks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>taskDefinition</i> The task definition the task set is using.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the task set. The following describes each state: <dl> <dt>PRIMARY</dt> <dd> The task set is serving production traffic. </dd> <dt>ACTIVE</dt> <dd> The task set is not serving production traffic. </dd> <dt>DRAINING</dt> <dd> The tasks in the task set are being stopped and their corresponding targets are being deregistered from their target group. </dd> </dl><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>externalId</i> The external ID associated with the task set. If a task set is created by an AWS CodeDeploy deployment, the <code>externalId</code> parameter contains the AWS CodeDeploy deployment ID. If a task set is created for an external deployment and is associated with a service discovery registry, the <code>externalId</code> parameter contains the <code>ECS_TASK_SET_EXTERNAL_ID</code> AWS Cloud Map attribute.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>startedBy</i> The tag specified when a task set is started. If the task set is created by an AWS CodeDeploy deployment, the <code>startedBy</code> parameter is <code>CODE_DEPLOY</code>. For a task set created for an external deployment, the startedBy field isn't used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clusterArn</i> The Amazon Resource Name (ARN) of the cluster that the service that hosts the task set exists in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceArn</i> The Amazon Resource Name (ARN) of the service the task set exists in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>taskSetArn</i> The Amazon Resource Name (ARN) of the task set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the task set.<br/>The VPC subnets and security groups associated with a task. <note> All specified subnets and security groups must be from the same VPC. </note>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>awsvpcConfiguration</b>: The VPC subnets and security groups associated with a task. <note> All specified subnets and security groups must be from the same VPC. </note>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>unit</i> The unit of measure for the scale value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>value</i> The value, specified as a percent total of a service's <code>desiredCount</code>, to scale the task set. Accepted values are numbers between 0 and 100.<br/>        """
        pass

    def deregister_container_instance(self, containerInstance: str, cluster: str=None, force: BoxedBoolean=None) -> DeregisterContainerInstanceResponse:
        """Deregisters an Amazon ECS container instance from the specified cluster. This instance is no longer available to run tasks. If you intend to use the container instance for some other purpose after deregistration, you should stop all of the tasks running on the container instance before deregistration. That prevents any orphaned tasks from consuming resources. Deregistering a container instance removes the instance from a cluster, but it does not terminate the EC2 instance. If you are finished using the instance, be sure to terminate it in the Amazon EC2 console to stop billing. <note> If you terminate a running container instance, Amazon ECS automatically deregisters the instance from your cluster (stopped container instances or instances with disconnected agents are not automatically deregistered when terminated). </note>

        :param self:
        :param force:  Forces the deregistration of the container instance. If you have tasks running on the container instance when you deregister it with the <code>force</code> option, these tasks remain running until you terminate the instance or the tasks stop through some other means, but they are orphaned (no longer monitored or accounted for by Amazon ECS). If an orphaned task on your container instance is part of an Amazon ECS service, then the service scheduler starts another copy of that task, on a different container instance if possible.  Any containers in orphaned service tasks that are registered with a Classic Load Balancer or an Application Load Balancer target group are deregistered. They begin connection draining according to the settings on the load balancer or target group.<br/>containerInstance:  The container instance ID or full ARN of the container instance to deregister. The ARN contains the <code>arn:aws:ecs</code> namespace, followed by the Region of the container instance, the AWS account ID of the container instance owner, the <code>container-instance</code> namespace, and then the container instance ID. For example, <code>arn:aws:ecs:region:aws_account_id:container-instance/container_instance_ID</code>.<br/>cluster:  The short name or full Amazon Resource Name (ARN) of the cluster that hosts the container instance to deregister. If you do not specify a cluster, the default cluster is assumed.<br/>
        :return: containerInstance:  The container instance that was deregistered.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> The metadata that you apply to the container instance to help you categorize and organize them. Each tag consists of a key and an optional value, both of which you define. Tag keys can have a maximum character length of 128 characters, and tag values can have a maximum length of 256 characters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>attachments</i> The resources attached to a container instance, such as elastic network interfaces.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>registeredAt</i> The Unix timestamp for when the container instance was registered.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>attributes</i> The attributes set for the container instance, either by the Amazon ECS container agent at instance registration or manually with the <a>PutAttributes</a> operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>agentUpdateStatus</i> The status of the most recent agent update. If an update has never been requested, this value is <code>NULL</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pendingTasksCount</i> The number of tasks on the container instance that are in the <code>PENDING</code> status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>runningTasksCount</i> The number of tasks on the container instance that are in the <code>RUNNING</code> status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>agentConnected</i> This parameter returns <code>true</code> if the agent is connected to Amazon ECS. Registered instances with an agent that may be unhealthy or stopped return <code>false</code>. Only instances connected to an agent can accept placement requests.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusReason</i> The reason that the container instance reached its current status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the container instance. The valid values are <code>REGISTERING</code>, <code>REGISTRATION_FAILED</code>, <code>ACTIVE</code>, <code>INACTIVE</code>, <code>DEREGISTERING</code>, or <code>DRAINING</code>. If your account has opted in to the <code>awsvpcTrunking</code> account setting, then any newly registered container instance will transition to a <code>REGISTERING</code> status while the trunk elastic network interface is provisioned for the instance. If the registration fails, the instance will transition to a <code>REGISTRATION_FAILED</code> status. You can describe the container instance and see the reason for failure in the <code>statusReason</code> parameter. Once the container instance is terminated, the instance transitions to a <code>DEREGISTERING</code> status while the trunk elastic network interface is deprovisioned. The instance then transitions to an <code>INACTIVE</code> status. The <code>ACTIVE</code> status indicates that the container instance can accept tasks. The <code>DRAINING</code> indicates that new tasks are not placed on the container instance and any service tasks running on the container instance are removed if possible. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/container-instance-draining.html">Container Instance Draining</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>registeredResources</i> For CPU and memory resource types, this parameter describes the amount of each resource that was available on the container instance when the container agent registered it with Amazon ECS. This value represents the total amount of CPU and memory that can be allocated on this container instance to tasks. For port resource types, this parameter describes the ports that were reserved by the Amazon ECS container agent when it registered the container instance with Amazon ECS.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>remainingResources</i> For CPU and memory resource types, this parameter describes the remaining CPU and memory that has not already been allocated to tasks and is therefore available for new tasks. For port resource types, this parameter describes the ports that were reserved by the Amazon ECS container agent (at instance registration time) and any task containers that have reserved port mappings on the host (with the <code>host</code> or <code>bridge</code> network mode). Any port that is not specified here is available for new tasks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>versionInfo</i> The version information for the Amazon ECS container agent and Docker daemon running on the container instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>versionInfo</b>: The version information for the Amazon ECS container agent and Docker daemon running on the container instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version counter for the container instance. Every time a container instance experiences a change that triggers a CloudWatch event, the version counter is incremented. If you are replicating your Amazon ECS container instance state with CloudWatch Events, you can compare the version of a container instance reported by the Amazon ECS APIs with the version reported in CloudWatch Events for the container instance (inside the <code>detail</code> object) to verify that the version in your event stream is current.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ec2InstanceId</i> The EC2 instance ID of the container instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>containerInstanceArn</i> The Amazon Resource Name (ARN) of the container instance. The ARN contains the <code>arn:aws:ecs</code> namespace, followed by the Region of the container instance, the AWS account ID of the container instance owner, the <code>container-instance</code> namespace, and then the container instance ID. For example, <code>arn:aws:ecs:region:aws_account_id:container-instance/container_instance_ID</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>dockerVersion</i> The Docker version running on the container instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>agentHash</i> The Git commit hash for the Amazon ECS container agent build on the <a href="https://github.com/aws/amazon-ecs-agent/commits/master">amazon-ecs-agent </a> GitHub repository.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>agentVersion</i> The version number of the Amazon ECS container agent.<br/>        """
        pass

    def deregister_task_definition(self, taskDefinition: str) -> DeregisterTaskDefinitionResponse:
        """Deregisters the specified task definition by family and revision. Upon deregistration, the task definition is marked as <code>INACTIVE</code>. Existing tasks and services that reference an <code>INACTIVE</code> task definition continue to run without disruption. Existing services that reference an <code>INACTIVE</code> task definition can still scale up or down by modifying the service's desired count. You cannot use an <code>INACTIVE</code> task definition to run new tasks or create new services, and you cannot update an existing service to reference an <code>INACTIVE</code> task definition. However, there may be up to a 10-minute window following deregistration where these restrictions have not yet taken effect. <note> At this time, <code>INACTIVE</code> task definitions remain discoverable in your account indefinitely. However, this behavior is subject to change in the future, so you should not rely on <code>INACTIVE</code> task definitions persisting beyond the lifecycle of any associated tasks and services. </note>

        :param self:
        :param taskDefinition:  The <code>family</code> and <code>revision</code> (<code>family:revision</code>) or full Amazon Resource Name (ARN) of the task definition to deregister. You must specify a <code>revision</code>.<br/>
        :return: taskDefinition:  The full description of the deregistered task.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>proxyConfiguration</i> The configuration details for the App Mesh proxy. Your Amazon ECS container instances require at least version 1.26.0 of the container agent and at least version 1.26.0-1 of the <code>ecs-init</code> package to enable a proxy configuration. If your container instances are launched from the Amazon ECS-optimized AMI version <code>20190301</code> or later, then they contain the required versions of the container agent and <code>ecs-init</code>. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/ecs-optimized_AMI.html">Amazon ECS-optimized Linux AMI</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>proxyConfiguration</b>: The configuration details for the App Mesh proxy. Your Amazon ECS container instances require at least version 1.26.0 of the container agent and at least version 1.26.0-1 of the <code>ecs-init</code> package to enable a proxy configuration. If your container instances are launched from the Amazon ECS-optimized AMI version <code>20190301</code> or later, then they contain the required versions of the container agent and <code>ecs-init</code>. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/ecs-optimized_AMI.html">Amazon ECS-optimized Linux AMI</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ipcMode</i> The IPC resource namespace to use for the containers in the task. The valid values are <code>host</code>, <code>task</code>, or <code>none</code>. If <code>host</code> is specified, then all containers within the tasks that specified the <code>host</code> IPC mode on the same container instance share the same IPC resources with the host Amazon EC2 instance. If <code>task</code> is specified, all containers within the specified task share the same IPC resources. If <code>none</code> is specified, then IPC resources within the containers of a task are private and not shared with other containers in a task or on the container instance. If no value is specified, then the IPC resource namespace sharing depends on the Docker daemon setting on the container instance. For more information, see <a href="https://docs.docker.com/engine/reference/run/#ipc-settings---ipc">IPC settings</a> in the <i>Docker run reference</i>. If the <code>host</code> IPC mode is used, be aware that there is a heightened risk of undesired IPC namespace expose. For more information, see <a href="https://docs.docker.com/engine/security/security/">Docker security</a>. If you are setting namespaced kernel parameters using <code>systemControls</code> for the containers in the task, the following will apply to your IPC resource namespace. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task_definition_parameters.html">System Controls</a> in the <i>Amazon Elastic Container Service Developer Guide</i>. <ul> <li> For tasks that use the <code>host</code> IPC mode, IPC namespace related <code>systemControls</code> are not supported. </li> <li> For tasks that use the <code>task</code> IPC mode, IPC namespace related <code>systemControls</code> will apply to all containers within a task. </li> </ul> <note> This parameter is not supported for Windows containers or tasks using the Fargate launch type. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pidMode</i> The process namespace to use for the containers in the task. The valid values are <code>host</code> or <code>task</code>. If <code>host</code> is specified, then all containers within the tasks that specified the <code>host</code> PID mode on the same container instance share the same IPC resources with the host Amazon EC2 instance. If <code>task</code> is specified, all containers within the specified task share the same process namespace. If no value is specified, the default is a private namespace. For more information, see <a href="https://docs.docker.com/engine/reference/run/#pid-settings---pid">PID settings</a> in the <i>Docker run reference</i>. If the <code>host</code> PID mode is used, be aware that there is a heightened risk of undesired process namespace expose. For more information, see <a href="https://docs.docker.com/engine/security/security/">Docker security</a>. <note> This parameter is not supported for Windows containers or tasks using the Fargate launch type. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>memory</i> The amount (in MiB) of memory used by the task. If using the EC2 launch type, this field is optional and any value can be used. If using the Fargate launch type, this field is required and you must use one of the following values, which determines your range of valid values for the <code>cpu</code> parameter: <ul> <li> 512 (0.5 GB), 1024 (1 GB), 2048 (2 GB) - Available <code>cpu</code> values: 256 (.25 vCPU) </li> <li> 1024 (1 GB), 2048 (2 GB), 3072 (3 GB), 4096 (4 GB) - Available <code>cpu</code> values: 512 (.5 vCPU) </li> <li> 2048 (2 GB), 3072 (3 GB), 4096 (4 GB), 5120 (5 GB), 6144 (6 GB), 7168 (7 GB), 8192 (8 GB) - Available <code>cpu</code> values: 1024 (1 vCPU) </li> <li> Between 4096 (4 GB) and 16384 (16 GB) in increments of 1024 (1 GB) - Available <code>cpu</code> values: 2048 (2 vCPU) </li> <li> Between 8192 (8 GB) and 30720 (30 GB) in increments of 1024 (1 GB) - Available <code>cpu</code> values: 4096 (4 vCPU) </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cpu</i> The number of <code>cpu</code> units used by the task. If you are using the EC2 launch type, this field is optional and any value can be used. If you are using the Fargate launch type, this field is required and you must use one of the following values, which determines your range of valid values for the <code>memory</code> parameter: <ul> <li> 256 (.25 vCPU) - Available <code>memory</code> values: 512 (0.5 GB), 1024 (1 GB), 2048 (2 GB) </li> <li> 512 (.5 vCPU) - Available <code>memory</code> values: 1024 (1 GB), 2048 (2 GB), 3072 (3 GB), 4096 (4 GB) </li> <li> 1024 (1 vCPU) - Available <code>memory</code> values: 2048 (2 GB), 3072 (3 GB), 4096 (4 GB), 5120 (5 GB), 6144 (6 GB), 7168 (7 GB), 8192 (8 GB) </li> <li> 2048 (2 vCPU) - Available <code>memory</code> values: Between 4096 (4 GB) and 16384 (16 GB) in increments of 1024 (1 GB) </li> <li> 4096 (4 vCPU) - Available <code>memory</code> values: Between 8192 (8 GB) and 30720 (30 GB) in increments of 1024 (1 GB) </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>requiresCompatibilities</i> The launch type the task requires. If no value is specified, it will default to <code>EC2</code>. Valid values include <code>EC2</code> and <code>FARGATE</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>compatibilities</i> The launch type to use with your task. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/launch_types.html">Amazon ECS Launch Types</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>placementConstraints</i> An array of placement constraint objects to use for tasks. This field is not valid if you are using the Fargate launch type for your task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>requiresAttributes</i> The container instance attributes required by your task. This field is not valid if you are using the Fargate launch type for your task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the task definition.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>volumes</i> The list of volume definitions for the task. If your tasks are using the Fargate launch type, the <code>host</code> and <code>sourcePath</code> parameters are not supported. For more information about volume definition parameters and defaults, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task_definitions.html">Amazon ECS Task Definitions</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>revision</i> The revision of the task in a particular family. The revision is a version number of a task definition in a family. When you register a task definition for the first time, the revision is <code>1</code>. Each time that you register a new revision of a task definition in the same family, the revision value always increases by one, even if you have deregistered previous revisions in this family.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>networkMode</i> The Docker networking mode to use for the containers in the task. The valid values are <code>none</code>, <code>bridge</code>, <code>awsvpc</code>, and <code>host</code>. The default Docker network mode is <code>bridge</code>. If you are using the Fargate launch type, the <code>awsvpc</code> network mode is required. If you are using the EC2 launch type, any network mode can be used. If the network mode is set to <code>none</code>, you cannot specify port mappings in your container definitions, and the tasks containers do not have external connectivity. The <code>host</code> and <code>awsvpc</code> network modes offer the highest networking performance for containers because they use the EC2 network stack instead of the virtualized network stack provided by the <code>bridge</code> mode. With the <code>host</code> and <code>awsvpc</code> network modes, exposed container ports are mapped directly to the corresponding host port (for the <code>host</code> network mode) or the attached elastic network interface port (for the <code>awsvpc</code> network mode), so you cannot take advantage of dynamic host port mappings.  If the network mode is <code>awsvpc</code>, the task is allocated an elastic network interface, and you must specify a <a>NetworkConfiguration</a> value when you create a service or run a task with the task definition. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task-networking.html">Task Networking</a> in the <i>Amazon Elastic Container Service Developer Guide</i>. <note> Currently, only Amazon ECS-optimized AMIs, other Amazon Linux variants with the <code>ecs-init</code> package, or AWS Fargate infrastructure support the <code>awsvpc</code> network mode.  </note> If the network mode is <code>host</code>, you cannot run multiple instantiations of the same task on a single container instance when port mappings are used. Docker for Windows uses different network modes than Docker for Linux. When you register a task definition with Windows containers, you must not specify a network mode. If you use the console to register a task definition with Windows containers, you must choose the <code>&lt;default&gt;</code> network mode object.  For more information, see <a href="https://docs.docker.com/engine/reference/run/#network-settings">Network settings</a> in the <i>Docker run reference</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>executionRoleArn</i> The Amazon Resource Name (ARN) of the task execution role that containers in this task can assume. All containers in this task are granted the permissions that are specified in this role.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>taskRoleArn</i> The Amazon Resource Name (ARN) of an AWS Identity and Access Management (IAM) role that grants containers in the task permission to call AWS APIs on your behalf. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task_IAM_role.html">Amazon ECS Task Role</a> in the <i>Amazon Elastic Container Service Developer Guide</i>. IAM roles for tasks on Windows require that the <code>-EnableTaskIAMRole</code> option is set when you launch the Amazon ECS-optimized Windows AMI. Your containers must also run some configuration code in order to take advantage of the feature. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/windows_task_IAM_roles.html">Windows IAM Roles for Tasks</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>family</i> The name of a family that this task definition is registered to. A family groups multiple versions of a task definition. Amazon ECS gives the first task definition that you registered to a family a revision number of 1. Amazon ECS gives sequential revision numbers to each task definition that you add.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>containerDefinitions</i> A list of container definitions in JSON format that describe the different containers that make up your task. For more information about container definition parameters and defaults, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task_defintions.html">Amazon ECS Task Definitions</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>taskDefinitionArn</i> The full Amazon Resource Name (ARN) of the task definition.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>properties</i> The set of network configuration parameters to provide the Container Network Interface (CNI) plugin, specified as key-value pairs. <ul> <li>  <code>IgnoredUID</code> - (Required) The user ID (UID) of the proxy container as defined by the <code>user</code> parameter in a container definition. This is used to ensure the proxy ignores its own traffic. If <code>IgnoredGID</code> is specified, this field can be empty. </li> <li>  <code>IgnoredGID</code> - (Required) The group ID (GID) of the proxy container as defined by the <code>user</code> parameter in a container definition. This is used to ensure the proxy ignores its own traffic. If <code>IgnoredUID</code> is specified, this field can be empty. </li> <li>  <code>AppPorts</code> - (Required) The list of ports that the application uses. Network traffic to these ports is forwarded to the <code>ProxyIngressPort</code> and <code>ProxyEgressPort</code>. </li> <li>  <code>ProxyIngressPort</code> - (Required) Specifies the port that incoming traffic to the <code>AppPorts</code> is directed to. </li> <li>  <code>ProxyEgressPort</code> - (Required) Specifies the port that outgoing traffic from the <code>AppPorts</code> is directed to. </li> <li>  <code>EgressIgnoredPorts</code> - (Required) The egress traffic going to the specified ports is ignored and not redirected to the <code>ProxyEgressPort</code>. It can be an empty list. </li> <li>  <code>EgressIgnoredIPs</code> - (Required) The egress traffic going to the specified IP addresses is ignored and not redirected to the <code>ProxyEgressPort</code>. It can be an empty list. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>containerName</i> The name of the container that will serve as the App Mesh proxy.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The proxy type. The only supported value is <code>APPMESH</code>.<br/>        """
        pass

    def describe_clusters(self, clusters: List[String]=None, include: List[ClusterField]=None) -> DescribeClustersResponse:
        """Describes one or more of your clusters.

        :param self:
        :param include:  Additional information about your clusters to be separated by launch type, including: <ul> <li> runningEC2TasksCount </li> <li> runningFargateTasksCount </li> <li> pendingEC2TasksCount </li> <li> pendingFargateTasksCount </li> <li> activeEC2ServiceCount </li> <li> activeFargateServiceCount </li> <li> drainingEC2ServiceCount </li> <li> drainingFargateServiceCount </li> </ul><br/>clusters:  A list of up to 100 cluster names or full cluster Amazon Resource Name (ARN) entries. If you do not specify a cluster, the default cluster is assumed.<br/>
        :return: failures:  Any failures associated with the call.<br/>clusters:  The list of clusters.<br/>        """
        pass

    def describe_container_instances(self, containerInstances: List[String], cluster: str=None, include: List[ContainerInstanceField]=None) -> DescribeContainerInstancesResponse:
        """Describes Amazon Elastic Container Service container instances. Returns metadata about registered and remaining resources on each container instance requested.

        :param self:
        :param include:  Specifies whether you want to see the resource tags for the container instance. If <code>TAGS</code> is specified, the tags are included in the response. If this field is omitted, tags are not included in the response.<br/>containerInstances:  A list of up to 100 container instance IDs or full Amazon Resource Name (ARN) entries.<br/>cluster:  The short name or full Amazon Resource Name (ARN) of the cluster that hosts the container instances to describe. If you do not specify a cluster, the default cluster is assumed. This parameter is required if the container instance or container instances you are describing were launched in any cluster other than the default cluster.<br/>
        :return: failures:  Any failures associated with the call.<br/>containerInstances:  The list of container instances.<br/>        """
        pass

    def describe_services(self, services: List[String], cluster: str=None, include: List[ServiceField]=None) -> DescribeServicesResponse:
        """Describes the specified services running in your cluster.

        :param self:
        :param include:  Specifies whether you want to see the resource tags for the service. If <code>TAGS</code> is specified, the tags are included in the response. If this field is omitted, tags are not included in the response.<br/>services:  A list of services to describe. You may specify up to 10 services to describe in a single operation.<br/>cluster:  The short name or full Amazon Resource Name (ARN)the cluster that hosts the service to describe. If you do not specify a cluster, the default cluster is assumed. This parameter is required if the service or services you are describing were launched in any cluster other than the default cluster.<br/>
        :return: failures:  Any failures associated with the call.<br/>services:  The list of services described.<br/>        """
        pass

    def describe_task_definition(self, taskDefinition: str, include: List[TaskDefinitionField]=None) -> DescribeTaskDefinitionResponse:
        """Describes a task definition. You can specify a <code>family</code> and <code>revision</code> to find information about a specific task definition, or you can simply specify the family to find the latest <code>ACTIVE</code> revision in that family. <note> You can only describe <code>INACTIVE</code> task definitions while an active task or service references them. </note>

        :param self:
        :param include:  Specifies whether to see the resource tags for the task definition. If <code>TAGS</code> is specified, the tags are included in the response. If this field is omitted, tags are not included in the response.<br/>taskDefinition:  The <code>family</code> for the latest <code>ACTIVE</code> revision, <code>family</code> and <code>revision</code> (<code>family:revision</code>) for a specific revision in the family, or full Amazon Resource Name (ARN) of the task definition to describe.<br/>
        :return: tags:  The metadata that is applied to the task definition to help you categorize and organize them. Each tag consists of a key and an optional value, both of which you define. Tag keys can have a maximum character length of 128 characters, and tag values can have a maximum length of 256 characters.<br/>taskDefinition:  The full task definition description.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>proxyConfiguration</i> The configuration details for the App Mesh proxy. Your Amazon ECS container instances require at least version 1.26.0 of the container agent and at least version 1.26.0-1 of the <code>ecs-init</code> package to enable a proxy configuration. If your container instances are launched from the Amazon ECS-optimized AMI version <code>20190301</code> or later, then they contain the required versions of the container agent and <code>ecs-init</code>. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/ecs-optimized_AMI.html">Amazon ECS-optimized Linux AMI</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>proxyConfiguration</b>: The configuration details for the App Mesh proxy. Your Amazon ECS container instances require at least version 1.26.0 of the container agent and at least version 1.26.0-1 of the <code>ecs-init</code> package to enable a proxy configuration. If your container instances are launched from the Amazon ECS-optimized AMI version <code>20190301</code> or later, then they contain the required versions of the container agent and <code>ecs-init</code>. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/ecs-optimized_AMI.html">Amazon ECS-optimized Linux AMI</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ipcMode</i> The IPC resource namespace to use for the containers in the task. The valid values are <code>host</code>, <code>task</code>, or <code>none</code>. If <code>host</code> is specified, then all containers within the tasks that specified the <code>host</code> IPC mode on the same container instance share the same IPC resources with the host Amazon EC2 instance. If <code>task</code> is specified, all containers within the specified task share the same IPC resources. If <code>none</code> is specified, then IPC resources within the containers of a task are private and not shared with other containers in a task or on the container instance. If no value is specified, then the IPC resource namespace sharing depends on the Docker daemon setting on the container instance. For more information, see <a href="https://docs.docker.com/engine/reference/run/#ipc-settings---ipc">IPC settings</a> in the <i>Docker run reference</i>. If the <code>host</code> IPC mode is used, be aware that there is a heightened risk of undesired IPC namespace expose. For more information, see <a href="https://docs.docker.com/engine/security/security/">Docker security</a>. If you are setting namespaced kernel parameters using <code>systemControls</code> for the containers in the task, the following will apply to your IPC resource namespace. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task_definition_parameters.html">System Controls</a> in the <i>Amazon Elastic Container Service Developer Guide</i>. <ul> <li> For tasks that use the <code>host</code> IPC mode, IPC namespace related <code>systemControls</code> are not supported. </li> <li> For tasks that use the <code>task</code> IPC mode, IPC namespace related <code>systemControls</code> will apply to all containers within a task. </li> </ul> <note> This parameter is not supported for Windows containers or tasks using the Fargate launch type. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pidMode</i> The process namespace to use for the containers in the task. The valid values are <code>host</code> or <code>task</code>. If <code>host</code> is specified, then all containers within the tasks that specified the <code>host</code> PID mode on the same container instance share the same IPC resources with the host Amazon EC2 instance. If <code>task</code> is specified, all containers within the specified task share the same process namespace. If no value is specified, the default is a private namespace. For more information, see <a href="https://docs.docker.com/engine/reference/run/#pid-settings---pid">PID settings</a> in the <i>Docker run reference</i>. If the <code>host</code> PID mode is used, be aware that there is a heightened risk of undesired process namespace expose. For more information, see <a href="https://docs.docker.com/engine/security/security/">Docker security</a>. <note> This parameter is not supported for Windows containers or tasks using the Fargate launch type. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>memory</i> The amount (in MiB) of memory used by the task. If using the EC2 launch type, this field is optional and any value can be used. If using the Fargate launch type, this field is required and you must use one of the following values, which determines your range of valid values for the <code>cpu</code> parameter: <ul> <li> 512 (0.5 GB), 1024 (1 GB), 2048 (2 GB) - Available <code>cpu</code> values: 256 (.25 vCPU) </li> <li> 1024 (1 GB), 2048 (2 GB), 3072 (3 GB), 4096 (4 GB) - Available <code>cpu</code> values: 512 (.5 vCPU) </li> <li> 2048 (2 GB), 3072 (3 GB), 4096 (4 GB), 5120 (5 GB), 6144 (6 GB), 7168 (7 GB), 8192 (8 GB) - Available <code>cpu</code> values: 1024 (1 vCPU) </li> <li> Between 4096 (4 GB) and 16384 (16 GB) in increments of 1024 (1 GB) - Available <code>cpu</code> values: 2048 (2 vCPU) </li> <li> Between 8192 (8 GB) and 30720 (30 GB) in increments of 1024 (1 GB) - Available <code>cpu</code> values: 4096 (4 vCPU) </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cpu</i> The number of <code>cpu</code> units used by the task. If you are using the EC2 launch type, this field is optional and any value can be used. If you are using the Fargate launch type, this field is required and you must use one of the following values, which determines your range of valid values for the <code>memory</code> parameter: <ul> <li> 256 (.25 vCPU) - Available <code>memory</code> values: 512 (0.5 GB), 1024 (1 GB), 2048 (2 GB) </li> <li> 512 (.5 vCPU) - Available <code>memory</code> values: 1024 (1 GB), 2048 (2 GB), 3072 (3 GB), 4096 (4 GB) </li> <li> 1024 (1 vCPU) - Available <code>memory</code> values: 2048 (2 GB), 3072 (3 GB), 4096 (4 GB), 5120 (5 GB), 6144 (6 GB), 7168 (7 GB), 8192 (8 GB) </li> <li> 2048 (2 vCPU) - Available <code>memory</code> values: Between 4096 (4 GB) and 16384 (16 GB) in increments of 1024 (1 GB) </li> <li> 4096 (4 vCPU) - Available <code>memory</code> values: Between 8192 (8 GB) and 30720 (30 GB) in increments of 1024 (1 GB) </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>requiresCompatibilities</i> The launch type the task requires. If no value is specified, it will default to <code>EC2</code>. Valid values include <code>EC2</code> and <code>FARGATE</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>compatibilities</i> The launch type to use with your task. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/launch_types.html">Amazon ECS Launch Types</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>placementConstraints</i> An array of placement constraint objects to use for tasks. This field is not valid if you are using the Fargate launch type for your task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>requiresAttributes</i> The container instance attributes required by your task. This field is not valid if you are using the Fargate launch type for your task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the task definition.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>volumes</i> The list of volume definitions for the task. If your tasks are using the Fargate launch type, the <code>host</code> and <code>sourcePath</code> parameters are not supported. For more information about volume definition parameters and defaults, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task_definitions.html">Amazon ECS Task Definitions</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>revision</i> The revision of the task in a particular family. The revision is a version number of a task definition in a family. When you register a task definition for the first time, the revision is <code>1</code>. Each time that you register a new revision of a task definition in the same family, the revision value always increases by one, even if you have deregistered previous revisions in this family.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>networkMode</i> The Docker networking mode to use for the containers in the task. The valid values are <code>none</code>, <code>bridge</code>, <code>awsvpc</code>, and <code>host</code>. The default Docker network mode is <code>bridge</code>. If you are using the Fargate launch type, the <code>awsvpc</code> network mode is required. If you are using the EC2 launch type, any network mode can be used. If the network mode is set to <code>none</code>, you cannot specify port mappings in your container definitions, and the tasks containers do not have external connectivity. The <code>host</code> and <code>awsvpc</code> network modes offer the highest networking performance for containers because they use the EC2 network stack instead of the virtualized network stack provided by the <code>bridge</code> mode. With the <code>host</code> and <code>awsvpc</code> network modes, exposed container ports are mapped directly to the corresponding host port (for the <code>host</code> network mode) or the attached elastic network interface port (for the <code>awsvpc</code> network mode), so you cannot take advantage of dynamic host port mappings.  If the network mode is <code>awsvpc</code>, the task is allocated an elastic network interface, and you must specify a <a>NetworkConfiguration</a> value when you create a service or run a task with the task definition. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task-networking.html">Task Networking</a> in the <i>Amazon Elastic Container Service Developer Guide</i>. <note> Currently, only Amazon ECS-optimized AMIs, other Amazon Linux variants with the <code>ecs-init</code> package, or AWS Fargate infrastructure support the <code>awsvpc</code> network mode.  </note> If the network mode is <code>host</code>, you cannot run multiple instantiations of the same task on a single container instance when port mappings are used. Docker for Windows uses different network modes than Docker for Linux. When you register a task definition with Windows containers, you must not specify a network mode. If you use the console to register a task definition with Windows containers, you must choose the <code>&lt;default&gt;</code> network mode object.  For more information, see <a href="https://docs.docker.com/engine/reference/run/#network-settings">Network settings</a> in the <i>Docker run reference</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>executionRoleArn</i> The Amazon Resource Name (ARN) of the task execution role that containers in this task can assume. All containers in this task are granted the permissions that are specified in this role.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>taskRoleArn</i> The Amazon Resource Name (ARN) of an AWS Identity and Access Management (IAM) role that grants containers in the task permission to call AWS APIs on your behalf. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task_IAM_role.html">Amazon ECS Task Role</a> in the <i>Amazon Elastic Container Service Developer Guide</i>. IAM roles for tasks on Windows require that the <code>-EnableTaskIAMRole</code> option is set when you launch the Amazon ECS-optimized Windows AMI. Your containers must also run some configuration code in order to take advantage of the feature. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/windows_task_IAM_roles.html">Windows IAM Roles for Tasks</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>family</i> The name of a family that this task definition is registered to. A family groups multiple versions of a task definition. Amazon ECS gives the first task definition that you registered to a family a revision number of 1. Amazon ECS gives sequential revision numbers to each task definition that you add.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>containerDefinitions</i> A list of container definitions in JSON format that describe the different containers that make up your task. For more information about container definition parameters and defaults, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task_defintions.html">Amazon ECS Task Definitions</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>taskDefinitionArn</i> The full Amazon Resource Name (ARN) of the task definition.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>properties</i> The set of network configuration parameters to provide the Container Network Interface (CNI) plugin, specified as key-value pairs. <ul> <li>  <code>IgnoredUID</code> - (Required) The user ID (UID) of the proxy container as defined by the <code>user</code> parameter in a container definition. This is used to ensure the proxy ignores its own traffic. If <code>IgnoredGID</code> is specified, this field can be empty. </li> <li>  <code>IgnoredGID</code> - (Required) The group ID (GID) of the proxy container as defined by the <code>user</code> parameter in a container definition. This is used to ensure the proxy ignores its own traffic. If <code>IgnoredUID</code> is specified, this field can be empty. </li> <li>  <code>AppPorts</code> - (Required) The list of ports that the application uses. Network traffic to these ports is forwarded to the <code>ProxyIngressPort</code> and <code>ProxyEgressPort</code>. </li> <li>  <code>ProxyIngressPort</code> - (Required) Specifies the port that incoming traffic to the <code>AppPorts</code> is directed to. </li> <li>  <code>ProxyEgressPort</code> - (Required) Specifies the port that outgoing traffic from the <code>AppPorts</code> is directed to. </li> <li>  <code>EgressIgnoredPorts</code> - (Required) The egress traffic going to the specified ports is ignored and not redirected to the <code>ProxyEgressPort</code>. It can be an empty list. </li> <li>  <code>EgressIgnoredIPs</code> - (Required) The egress traffic going to the specified IP addresses is ignored and not redirected to the <code>ProxyEgressPort</code>. It can be an empty list. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>containerName</i> The name of the container that will serve as the App Mesh proxy.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The proxy type. The only supported value is <code>APPMESH</code>.<br/>        """
        pass

    def describe_task_sets(self, service: str, cluster: str, taskSets: List[String]=None) -> DescribeTaskSetsResponse:
        """Describes the task sets in the specified cluster and service. This is used when a service uses the <code>EXTERNAL</code> deployment controller type. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/deployment-types.html">Amazon ECS Deployment Types</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.

        :param self:
        :param taskSets:  The ID or full Amazon Resource Name (ARN) of task sets to describe.<br/>service:  The short name or full Amazon Resource Name (ARN) of the service that the task sets exist in.<br/>cluster:  The short name or full Amazon Resource Name (ARN) of the cluster that hosts the service that the task sets exist in.<br/>
        :return: failures:  Any failures associated with the call.<br/>taskSets:  The list of task sets described.<br/>        """
        pass

    def describe_tasks(self, tasks: List[String], cluster: str=None, include: List[TaskField]=None) -> DescribeTasksResponse:
        """Describes a specified task or tasks.

        :param self:
        :param include:  Specifies whether you want to see the resource tags for the task. If <code>TAGS</code> is specified, the tags are included in the response. If this field is omitted, tags are not included in the response.<br/>tasks:  A list of up to 100 task IDs or full ARN entries.<br/>cluster:  The short name or full Amazon Resource Name (ARN) of the cluster that hosts the task or tasks to describe. If you do not specify a cluster, the default cluster is assumed. This parameter is required if the task or tasks you are describing were launched in any cluster other than the default cluster.<br/>
        :return: failures:  Any failures associated with the call.<br/>tasks:  The list of tasks.<br/>        """
        pass

    def discover_poll_endpoint(self, containerInstance: str=None, cluster: str=None) -> DiscoverPollEndpointResponse:
        """<note> This action is only used by the Amazon ECS agent, and it is not intended for use outside of the agent. </note> Returns an endpoint for the Amazon ECS agent to poll for updates.

        :param self:
        :param cluster:  The short name or full Amazon Resource Name (ARN) of the cluster to which the container instance belongs.<br/>containerInstance:  The container instance ID or full ARN of the container instance. The ARN contains the <code>arn:aws:ecs</code> namespace, followed by the Region of the container instance, the AWS account ID of the container instance owner, the <code>container-instance</code> namespace, and then the container instance ID. For example, <code>arn:aws:ecs:region:aws_account_id:container-instance/container_instance_ID</code>.<br/>
        :return: telemetryEndpoint:  The telemetry endpoint for the Amazon ECS agent.<br/>endpoint:  The endpoint for the Amazon ECS agent to poll.<br/>        """
        pass

    def list_account_settings(self, name: str=None, value: str=None, principalArn: str=None, effectiveSettings: Boolean=None, nextToken: str=None, maxResults: int=None) -> ListAccountSettingsResponse:
        """Lists the account settings for a specified principal.

        :param self:
        :param maxResults:  The maximum number of account setting results returned by <code>ListAccountSettings</code> in paginated output. When this parameter is used, <code>ListAccountSettings</code> only returns <code>maxResults</code> results in a single page along with a <code>nextToken</code> response element. The remaining results of the initial request can be seen by sending another <code>ListAccountSettings</code> request with the returned <code>nextToken</code> value. This value can be between 1 and 10. If this parameter is not used, then <code>ListAccountSettings</code> returns up to 10 results and a <code>nextToken</code> value if applicable.<br/>nextToken:  The <code>nextToken</code> value returned from a previous paginated <code>ListAccountSettings</code> request where <code>maxResults</code> was used and the results exceeded the value of that parameter. Pagination continues from the end of the previous results that returned the <code>nextToken</code> value. <note> This token should be treated as an opaque identifier that is only used to retrieve the next items in a list and not for other programmatic purposes. </note><br/>effectiveSettings:  Specifies whether to return the effective settings. If <code>true</code>, the account settings for the root user or the default setting for the <code>principalArn</code> are returned. If <code>false</code>, the account settings for the <code>principalArn</code> are returned if they are set. Otherwise, no account settings are returned.<br/>principalArn:  The ARN of the principal, which can be an IAM user, IAM role, or the root user. If this field is omitted, the account settings are listed only for the authenticated user.<br/>value:  The value of the account settings with which to filter results. You must also specify an account setting name to use this parameter.<br/>name:  The resource name you want to list the account settings for.<br/>
        :return: nextToken:  The <code>nextToken</code> value to include in a future <code>ListAccountSettings</code> request. When the results of a <code>ListAccountSettings</code> request exceed <code>maxResults</code>, this value can be used to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>settings:  The account settings for the resource.<br/>        """
        pass

    def list_attributes(self, targetType: str, cluster: str=None, attributeName: str=None, attributeValue: str=None, nextToken: str=None, maxResults: int=None) -> ListAttributesResponse:
        """Lists the attributes for Amazon ECS resources within a specified target type and cluster. When you specify a target type and cluster, <code>ListAttributes</code> returns a list of attribute objects, one for each attribute on each resource. You can filter the list of results to a single attribute name to only return results that have that name. You can also filter the results by attribute name and value, for example, to see which container instances in a cluster are running a Linux AMI (<code>ecs.os-type=linux</code>). 

        :param self:
        :param maxResults:  The maximum number of cluster results returned by <code>ListAttributes</code> in paginated output. When this parameter is used, <code>ListAttributes</code> only returns <code>maxResults</code> results in a single page along with a <code>nextToken</code> response element. The remaining results of the initial request can be seen by sending another <code>ListAttributes</code> request with the returned <code>nextToken</code> value. This value can be between 1 and 100. If this parameter is not used, then <code>ListAttributes</code> returns up to 100 results and a <code>nextToken</code> value if applicable.<br/>nextToken:  The <code>nextToken</code> value returned from a previous paginated <code>ListAttributes</code> request where <code>maxResults</code> was used and the results exceeded the value of that parameter. Pagination continues from the end of the previous results that returned the <code>nextToken</code> value. <note> This token should be treated as an opaque identifier that is only used to retrieve the next items in a list and not for other programmatic purposes. </note><br/>attributeValue:  The value of the attribute with which to filter results. You must also specify an attribute name to use this parameter.<br/>attributeName:  The name of the attribute with which to filter the results. <br/>targetType:  The type of the target with which to list attributes.<br/>cluster:  The short name or full Amazon Resource Name (ARN) of the cluster to list attributes. If you do not specify a cluster, the default cluster is assumed.<br/>
        :return: nextToken:  The <code>nextToken</code> value to include in a future <code>ListAttributes</code> request. When the results of a <code>ListAttributes</code> request exceed <code>maxResults</code>, this value can be used to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>attributes:  A list of attribute objects that meet the criteria of the request.<br/>        """
        pass

    def list_clusters(self, nextToken: str=None, maxResults: int=None) -> ListClustersResponse:
        """Returns a list of existing clusters.

        :param self:
        :param maxResults:  The maximum number of cluster results returned by <code>ListClusters</code> in paginated output. When this parameter is used, <code>ListClusters</code> only returns <code>maxResults</code> results in a single page along with a <code>nextToken</code> response element. The remaining results of the initial request can be seen by sending another <code>ListClusters</code> request with the returned <code>nextToken</code> value. This value can be between 1 and 100. If this parameter is not used, then <code>ListClusters</code> returns up to 100 results and a <code>nextToken</code> value if applicable.<br/>nextToken:  The <code>nextToken</code> value returned from a previous paginated <code>ListClusters</code> request where <code>maxResults</code> was used and the results exceeded the value of that parameter. Pagination continues from the end of the previous results that returned the <code>nextToken</code> value. <note> This token should be treated as an opaque identifier that is only used to retrieve the next items in a list and not for other programmatic purposes. </note><br/>
        :return: nextToken:  The <code>nextToken</code> value to include in a future <code>ListClusters</code> request. When the results of a <code>ListClusters</code> request exceed <code>maxResults</code>, this value can be used to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>clusterArns:  The list of full Amazon Resource Name (ARN) entries for each cluster associated with your account.<br/>        """
        pass

    def list_container_instances(self, cluster: str=None, filter: str=None, nextToken: str=None, maxResults: int=None, status: str=None) -> ListContainerInstancesResponse:
        """Returns a list of container instances in a specified cluster. You can filter the results of a <code>ListContainerInstances</code> operation with cluster query language statements inside the <code>filter</code> parameter. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/cluster-query-language.html">Cluster Query Language</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.

        :param self:
        :param status:  Filters the container instances by status. For example, if you specify the <code>DRAINING</code> status, the results include only container instances that have been set to <code>DRAINING</code> using <a>UpdateContainerInstancesState</a>. If you do not specify this parameter, the default is to include container instances set to all states other than <code>INACTIVE</code>.<br/>maxResults:  The maximum number of container instance results returned by <code>ListContainerInstances</code> in paginated output. When this parameter is used, <code>ListContainerInstances</code> only returns <code>maxResults</code> results in a single page along with a <code>nextToken</code> response element. The remaining results of the initial request can be seen by sending another <code>ListContainerInstances</code> request with the returned <code>nextToken</code> value. This value can be between 1 and 100. If this parameter is not used, then <code>ListContainerInstances</code> returns up to 100 results and a <code>nextToken</code> value if applicable.<br/>nextToken:  The <code>nextToken</code> value returned from a previous paginated <code>ListContainerInstances</code> request where <code>maxResults</code> was used and the results exceeded the value of that parameter. Pagination continues from the end of the previous results that returned the <code>nextToken</code> value. <note> This token should be treated as an opaque identifier that is only used to retrieve the next items in a list and not for other programmatic purposes. </note><br/>filter:  You can filter the results of a <code>ListContainerInstances</code> operation with cluster query language statements. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/cluster-query-language.html">Cluster Query Language</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>cluster:  The short name or full Amazon Resource Name (ARN) of the cluster that hosts the container instances to list. If you do not specify a cluster, the default cluster is assumed.<br/>
        :return: nextToken:  The <code>nextToken</code> value to include in a future <code>ListContainerInstances</code> request. When the results of a <code>ListContainerInstances</code> request exceed <code>maxResults</code>, this value can be used to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>containerInstanceArns:  The list of container instances with full ARN entries for each container instance associated with the specified cluster.<br/>        """
        pass

    def list_services(self, cluster: str=None, nextToken: str=None, maxResults: int=None, launchType: str=None, schedulingStrategy: str=None) -> ListServicesResponse:
        """Lists the services that are running in a specified cluster.

        :param self:
        :param schedulingStrategy:  The scheduling strategy for services to list.<br/>launchType:  The launch type for the services to list.<br/>maxResults:  The maximum number of service results returned by <code>ListServices</code> in paginated output. When this parameter is used, <code>ListServices</code> only returns <code>maxResults</code> results in a single page along with a <code>nextToken</code> response element. The remaining results of the initial request can be seen by sending another <code>ListServices</code> request with the returned <code>nextToken</code> value. This value can be between 1 and 100. If this parameter is not used, then <code>ListServices</code> returns up to 10 results and a <code>nextToken</code> value if applicable.<br/>nextToken:  The <code>nextToken</code> value returned from a previous paginated <code>ListServices</code> request where <code>maxResults</code> was used and the results exceeded the value of that parameter. Pagination continues from the end of the previous results that returned the <code>nextToken</code> value. <note> This token should be treated as an opaque identifier that is only used to retrieve the next items in a list and not for other programmatic purposes. </note><br/>cluster:  The short name or full Amazon Resource Name (ARN) of the cluster that hosts the services to list. If you do not specify a cluster, the default cluster is assumed.<br/>
        :return: nextToken:  The <code>nextToken</code> value to include in a future <code>ListServices</code> request. When the results of a <code>ListServices</code> request exceed <code>maxResults</code>, this value can be used to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>serviceArns:  The list of full ARN entries for each service associated with the specified cluster.<br/>        """
        pass

    def list_tags_for_resource(self, resourceArn: str) -> ListTagsForResourceResponse:
        """List the tags for an Amazon ECS resource.

        :param self:
        :param resourceArn:  The Amazon Resource Name (ARN) that identifies the resource for which to list the tags. Currently, the supported resources are Amazon ECS tasks, services, task definitions, clusters, and container instances.<br/>
        :return: tags:  The tags for the resource.<br/>        """
        pass

    def list_task_definition_families(self, familyPrefix: str=None, status: str=None, nextToken: str=None, maxResults: int=None) -> ListTaskDefinitionFamiliesResponse:
        """Returns a list of task definition families that are registered to your account (which may include task definition families that no longer have any <code>ACTIVE</code> task definition revisions). You can filter out task definition families that do not contain any <code>ACTIVE</code> task definition revisions by setting the <code>status</code> parameter to <code>ACTIVE</code>. You can also filter the results with the <code>familyPrefix</code> parameter.

        :param self:
        :param maxResults:  The maximum number of task definition family results returned by <code>ListTaskDefinitionFamilies</code> in paginated output. When this parameter is used, <code>ListTaskDefinitions</code> only returns <code>maxResults</code> results in a single page along with a <code>nextToken</code> response element. The remaining results of the initial request can be seen by sending another <code>ListTaskDefinitionFamilies</code> request with the returned <code>nextToken</code> value. This value can be between 1 and 100. If this parameter is not used, then <code>ListTaskDefinitionFamilies</code> returns up to 100 results and a <code>nextToken</code> value if applicable.<br/>nextToken:  The <code>nextToken</code> value returned from a previous paginated <code>ListTaskDefinitionFamilies</code> request where <code>maxResults</code> was used and the results exceeded the value of that parameter. Pagination continues from the end of the previous results that returned the <code>nextToken</code> value. <note> This token should be treated as an opaque identifier that is only used to retrieve the next items in a list and not for other programmatic purposes. </note><br/>status:  The task definition family status with which to filter the <code>ListTaskDefinitionFamilies</code> results. By default, both <code>ACTIVE</code> and <code>INACTIVE</code> task definition families are listed. If this parameter is set to <code>ACTIVE</code>, only task definition families that have an <code>ACTIVE</code> task definition revision are returned. If this parameter is set to <code>INACTIVE</code>, only task definition families that do not have any <code>ACTIVE</code> task definition revisions are returned. If you paginate the resulting output, be sure to keep the <code>status</code> value constant in each subsequent request.<br/>familyPrefix:  The <code>familyPrefix</code> is a string that is used to filter the results of <code>ListTaskDefinitionFamilies</code>. If you specify a <code>familyPrefix</code>, only task definition family names that begin with the <code>familyPrefix</code> string are returned.<br/>
        :return: nextToken:  The <code>nextToken</code> value to include in a future <code>ListTaskDefinitionFamilies</code> request. When the results of a <code>ListTaskDefinitionFamilies</code> request exceed <code>maxResults</code>, this value can be used to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>families:  The list of task definition family names that match the <code>ListTaskDefinitionFamilies</code> request.<br/>        """
        pass

    def list_task_definitions(self, familyPrefix: str=None, status: str=None, sort: str=None, nextToken: str=None, maxResults: int=None) -> ListTaskDefinitionsResponse:
        """Returns a list of task definitions that are registered to your account. You can filter the results by family name with the <code>familyPrefix</code> parameter or by status with the <code>status</code> parameter.

        :param self:
        :param maxResults:  The maximum number of task definition results returned by <code>ListTaskDefinitions</code> in paginated output. When this parameter is used, <code>ListTaskDefinitions</code> only returns <code>maxResults</code> results in a single page along with a <code>nextToken</code> response element. The remaining results of the initial request can be seen by sending another <code>ListTaskDefinitions</code> request with the returned <code>nextToken</code> value. This value can be between 1 and 100. If this parameter is not used, then <code>ListTaskDefinitions</code> returns up to 100 results and a <code>nextToken</code> value if applicable.<br/>nextToken:  The <code>nextToken</code> value returned from a previous paginated <code>ListTaskDefinitions</code> request where <code>maxResults</code> was used and the results exceeded the value of that parameter. Pagination continues from the end of the previous results that returned the <code>nextToken</code> value. <note> This token should be treated as an opaque identifier that is only used to retrieve the next items in a list and not for other programmatic purposes. </note><br/>sort:  The order in which to sort the results. Valid values are <code>ASC</code> and <code>DESC</code>. By default (<code>ASC</code>), task definitions are listed lexicographically by family name and in ascending numerical order by revision so that the newest task definitions in a family are listed last. Setting this parameter to <code>DESC</code> reverses the sort order on family name and revision so that the newest task definitions in a family are listed first.<br/>status:  The task definition status with which to filter the <code>ListTaskDefinitions</code> results. By default, only <code>ACTIVE</code> task definitions are listed. By setting this parameter to <code>INACTIVE</code>, you can view task definitions that are <code>INACTIVE</code> as long as an active task or service still references them. If you paginate the resulting output, be sure to keep the <code>status</code> value constant in each subsequent request.<br/>familyPrefix:  The full family name with which to filter the <code>ListTaskDefinitions</code> results. Specifying a <code>familyPrefix</code> limits the listed task definitions to task definition revisions that belong to that family.<br/>
        :return: nextToken:  The <code>nextToken</code> value to include in a future <code>ListTaskDefinitions</code> request. When the results of a <code>ListTaskDefinitions</code> request exceed <code>maxResults</code>, this value can be used to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>taskDefinitionArns:  The list of task definition Amazon Resource Name (ARN) entries for the <code>ListTaskDefinitions</code> request.<br/>        """
        pass

    def list_tasks(self, cluster: str=None, containerInstance: str=None, family: str=None, nextToken: str=None, maxResults: int=None, startedBy: str=None, serviceName: str=None, desiredStatus: str=None, launchType: str=None) -> ListTasksResponse:
        """Returns a list of tasks for a specified cluster. You can filter the results by family name, by a particular container instance, or by the desired status of the task with the <code>family</code>, <code>containerInstance</code>, and <code>desiredStatus</code> parameters. Recently stopped tasks might appear in the returned results. Currently, stopped tasks appear in the returned results for at least one hour. 

        :param self:
        :param launchType:  The launch type for services to list.<br/>desiredStatus:  The task desired status with which to filter the <code>ListTasks</code> results. Specifying a <code>desiredStatus</code> of <code>STOPPED</code> limits the results to tasks that Amazon ECS has set the desired status to <code>STOPPED</code>. This can be useful for debugging tasks that are not starting properly or have died or finished. The default status filter is <code>RUNNING</code>, which shows tasks that Amazon ECS has set the desired status to <code>RUNNING</code>. <note> Although you can filter results based on a desired status of <code>PENDING</code>, this does not return any results. Amazon ECS never sets the desired status of a task to that value (only a task's <code>lastStatus</code> may have a value of <code>PENDING</code>). </note><br/>serviceName:  The name of the service with which to filter the <code>ListTasks</code> results. Specifying a <code>serviceName</code> limits the results to tasks that belong to that service.<br/>startedBy:  The <code>startedBy</code> value with which to filter the task results. Specifying a <code>startedBy</code> value limits the results to tasks that were started with that value.<br/>maxResults:  The maximum number of task results returned by <code>ListTasks</code> in paginated output. When this parameter is used, <code>ListTasks</code> only returns <code>maxResults</code> results in a single page along with a <code>nextToken</code> response element. The remaining results of the initial request can be seen by sending another <code>ListTasks</code> request with the returned <code>nextToken</code> value. This value can be between 1 and 100. If this parameter is not used, then <code>ListTasks</code> returns up to 100 results and a <code>nextToken</code> value if applicable.<br/>nextToken:  The <code>nextToken</code> value returned from a previous paginated <code>ListTasks</code> request where <code>maxResults</code> was used and the results exceeded the value of that parameter. Pagination continues from the end of the previous results that returned the <code>nextToken</code> value. <note> This token should be treated as an opaque identifier that is only used to retrieve the next items in a list and not for other programmatic purposes. </note><br/>family:  The name of the family with which to filter the <code>ListTasks</code> results. Specifying a <code>family</code> limits the results to tasks that belong to that family.<br/>containerInstance:  The container instance ID or full ARN of the container instance with which to filter the <code>ListTasks</code> results. Specifying a <code>containerInstance</code> limits the results to tasks that belong to that container instance.<br/>cluster:  The short name or full Amazon Resource Name (ARN) of the cluster that hosts the tasks to list. If you do not specify a cluster, the default cluster is assumed.<br/>
        :return: nextToken:  The <code>nextToken</code> value to include in a future <code>ListTasks</code> request. When the results of a <code>ListTasks</code> request exceed <code>maxResults</code>, this value can be used to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>taskArns:  The list of task ARN entries for the <code>ListTasks</code> request.<br/>        """
        pass

    def put_account_setting(self, value: str, name: str, principalArn: str=None) -> PutAccountSettingResponse:
        """Modifies an account setting. If you change the account setting for the root user, the default settings for all of the IAM users and roles for which no individual account setting has been specified are reset. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/ecs-account-settings.html">Account Settings</a> in the <i>Amazon Elastic Container Service Developer Guide</i>. When <code>serviceLongArnFormat</code>, <code>taskLongArnFormat</code>, or <code>containerInstanceLongArnFormat</code> are specified, the Amazon Resource Name (ARN) and resource ID format of the resource type for a specified IAM user, IAM role, or the root user for an account is affected. The opt-in and opt-out account setting must be set for each Amazon ECS resource separately. The ARN and resource ID format of a resource will be defined by the opt-in status of the IAM user or role that created the resource. You must enable this setting to use Amazon ECS features such as resource tagging. When <code>awsvpcTrunking</code> is specified, the elastic network interface (ENI) limit for any new container instances that support the feature is changed. If <code>awsvpcTrunking</code> is enabled, any new container instances that support the feature are launched have the increased ENI limits available to them. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/container-instance-eni.html">Elastic Network Interface Trunking</a> in the <i>Amazon Elastic Container Service Developer Guide</i>. When <code>containerInsights</code> is specified, the default setting indicating whether CloudWatch Container Insights is enabled for your clusters is changed. If <code>containerInsights</code> is enabled, any new clusters that are created will have Container Insights enabled unless you disable it during cluster creation. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/cloudwatch-container-insights.html">CloudWatch Container Insights</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.

        :param self:
        :param principalArn:  The ARN of the principal, which can be an IAM user, IAM role, or the root user. If you specify the root user, it modifies the account setting for all IAM users, IAM roles, and the root user of the account unless an IAM user or role explicitly overrides these settings. If this field is omitted, the setting is changed only for the authenticated user.<br/>value:  The account setting value for the specified principal ARN. Accepted values are <code>enabled</code> and <code>disabled</code>.<br/>name:  The Amazon ECS resource name for which to modify the account setting. If <code>serviceLongArnFormat</code> is specified, the ARN for your Amazon ECS services is affected. If <code>taskLongArnFormat</code> is specified, the ARN and resource ID for your Amazon ECS tasks is affected. If <code>containerInstanceLongArnFormat</code> is specified, the ARN and resource ID for your Amazon ECS container instances is affected. If <code>awsvpcTrunking</code> is specified, the elastic network interface (ENI) limit for your Amazon ECS container instances is affected. If <code>containerInsights</code> is specified, the default setting for CloudWatch Container Insights for your clusters is affected.<br/>
        :return: setting:  The current account setting for a resource.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>principalArn</i> The ARN of the principal, which can be an IAM user, IAM role, or the root user. If this field is omitted, the authenticated user is assumed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>value</i> Whether the account setting is enabled or disabled for the specified resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The Amazon ECS resource name.<br/>        """
        pass

    def put_account_setting_default(self, value: str, name: str) -> PutAccountSettingDefaultResponse:
        """Modifies an account setting for all IAM users on an account for whom no individual account setting has been specified.

        :param self:
        :param value:  The account setting value for the specified principal ARN. Accepted values are <code>enabled</code> and <code>disabled</code>.<br/>name:  The resource name for which to modify the account setting. If <code>serviceLongArnFormat</code> is specified, the ARN for your Amazon ECS services is affected. If <code>taskLongArnFormat</code> is specified, the ARN and resource ID for your Amazon ECS tasks is affected. If <code>containerInstanceLongArnFormat</code> is specified, the ARN and resource ID for your Amazon ECS container instances is affected. If <code>awsvpcTrunking</code> is specified, the ENI limit for your Amazon ECS container instances is affected. If <code>containerInsights</code> is specified, the default setting for CloudWatch Container Insights for your clusters is affected.<br/>
        :return: setting:  The current account setting for a resource.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>principalArn</i> The ARN of the principal, which can be an IAM user, IAM role, or the root user. If this field is omitted, the authenticated user is assumed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>value</i> Whether the account setting is enabled or disabled for the specified resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The Amazon ECS resource name.<br/>        """
        pass

    def put_attributes(self, attributes: List[Attribute], cluster: str=None) -> PutAttributesResponse:
        """Create or update an attribute on an Amazon ECS resource. If the attribute does not exist, it is created. If the attribute exists, its value is replaced with the specified value. To delete an attribute, use <a>DeleteAttributes</a>. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task-placement-constraints.html#attributes">Attributes</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.

        :param self:
        :param attributes:  The attributes to apply to your resource. You can specify up to 10 custom attributes per resource. You can specify up to 10 attributes in a single call.<br/>cluster:  The short name or full Amazon Resource Name (ARN) of the cluster that contains the resource to apply attributes. If you do not specify a cluster, the default cluster is assumed.<br/>
        :return: attributes:  The attributes applied to your resource.<br/>        """
        pass

    def register_container_instance(self, cluster: str=None, instanceIdentityDocument: str=None, instanceIdentityDocumentSignature: str=None, totalResources: List[Resource]=None, versionInfo: VersionInfo=None, containerInstanceArn: str=None, attributes: List[Attribute]=None, platformDevices: List[PlatformDevice]=None, tags: List[Tag]=None) -> RegisterContainerInstanceResponse:
        """<note> This action is only used by the Amazon ECS agent, and it is not intended for use outside of the agent. </note> Registers an EC2 instance into the specified cluster. This instance becomes available to place containers on.

        :param self:
        :param tags:  The metadata that you apply to the container instance to help you categorize and organize them. Each tag consists of a key and an optional value, both of which you define. Tag keys can have a maximum character length of 128 characters, and tag values can have a maximum length of 256 characters.<br/>platformDevices:  The devices that are available on the container instance. The only supported device type is a GPU.<br/>attributes:  The container instance attributes that this container instance supports.<br/>containerInstanceArn:  The ARN of the container instance (if it was previously registered).<br/>versionInfo:  The version information for the Amazon ECS container agent and Docker daemon running on the container instance.:<br/>totalResources:  The resources available on the instance.<br/>instanceIdentityDocumentSignature:  The instance identity document signature for the EC2 instance to register. This signature can be found by running the following command from the instance: <code>curl http://169.254.169.254/latest/dynamic/instance-identity/signature/</code> <br/>instanceIdentityDocument:  The instance identity document for the EC2 instance to register. This document can be found by running the following command from the instance: <code>curl http://169.254.169.254/latest/dynamic/instance-identity/document/</code> <br/>cluster:  The short name or full Amazon Resource Name (ARN) of the cluster with which to register your container instance. If you do not specify a cluster, the default cluster is assumed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>dockerVersion</i> The Docker version running on the container instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>agentHash</i> The Git commit hash for the Amazon ECS container agent build on the <a href="https://github.com/aws/amazon-ecs-agent/commits/master">amazon-ecs-agent </a> GitHub repository.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>agentVersion</i> The version number of the Amazon ECS container agent.<br/>
        :return: containerInstance:  The container instance that was registered.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> The metadata that you apply to the container instance to help you categorize and organize them. Each tag consists of a key and an optional value, both of which you define. Tag keys can have a maximum character length of 128 characters, and tag values can have a maximum length of 256 characters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>attachments</i> The resources attached to a container instance, such as elastic network interfaces.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>registeredAt</i> The Unix timestamp for when the container instance was registered.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>attributes</i> The attributes set for the container instance, either by the Amazon ECS container agent at instance registration or manually with the <a>PutAttributes</a> operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>agentUpdateStatus</i> The status of the most recent agent update. If an update has never been requested, this value is <code>NULL</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pendingTasksCount</i> The number of tasks on the container instance that are in the <code>PENDING</code> status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>runningTasksCount</i> The number of tasks on the container instance that are in the <code>RUNNING</code> status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>agentConnected</i> This parameter returns <code>true</code> if the agent is connected to Amazon ECS. Registered instances with an agent that may be unhealthy or stopped return <code>false</code>. Only instances connected to an agent can accept placement requests.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusReason</i> The reason that the container instance reached its current status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the container instance. The valid values are <code>REGISTERING</code>, <code>REGISTRATION_FAILED</code>, <code>ACTIVE</code>, <code>INACTIVE</code>, <code>DEREGISTERING</code>, or <code>DRAINING</code>. If your account has opted in to the <code>awsvpcTrunking</code> account setting, then any newly registered container instance will transition to a <code>REGISTERING</code> status while the trunk elastic network interface is provisioned for the instance. If the registration fails, the instance will transition to a <code>REGISTRATION_FAILED</code> status. You can describe the container instance and see the reason for failure in the <code>statusReason</code> parameter. Once the container instance is terminated, the instance transitions to a <code>DEREGISTERING</code> status while the trunk elastic network interface is deprovisioned. The instance then transitions to an <code>INACTIVE</code> status. The <code>ACTIVE</code> status indicates that the container instance can accept tasks. The <code>DRAINING</code> indicates that new tasks are not placed on the container instance and any service tasks running on the container instance are removed if possible. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/container-instance-draining.html">Container Instance Draining</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>registeredResources</i> For CPU and memory resource types, this parameter describes the amount of each resource that was available on the container instance when the container agent registered it with Amazon ECS. This value represents the total amount of CPU and memory that can be allocated on this container instance to tasks. For port resource types, this parameter describes the ports that were reserved by the Amazon ECS container agent when it registered the container instance with Amazon ECS.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>remainingResources</i> For CPU and memory resource types, this parameter describes the remaining CPU and memory that has not already been allocated to tasks and is therefore available for new tasks. For port resource types, this parameter describes the ports that were reserved by the Amazon ECS container agent (at instance registration time) and any task containers that have reserved port mappings on the host (with the <code>host</code> or <code>bridge</code> network mode). Any port that is not specified here is available for new tasks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>versionInfo</i> The version information for the Amazon ECS container agent and Docker daemon running on the container instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>versionInfo</b>: The version information for the Amazon ECS container agent and Docker daemon running on the container instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version counter for the container instance. Every time a container instance experiences a change that triggers a CloudWatch event, the version counter is incremented. If you are replicating your Amazon ECS container instance state with CloudWatch Events, you can compare the version of a container instance reported by the Amazon ECS APIs with the version reported in CloudWatch Events for the container instance (inside the <code>detail</code> object) to verify that the version in your event stream is current.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ec2InstanceId</i> The EC2 instance ID of the container instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>containerInstanceArn</i> The Amazon Resource Name (ARN) of the container instance. The ARN contains the <code>arn:aws:ecs</code> namespace, followed by the Region of the container instance, the AWS account ID of the container instance owner, the <code>container-instance</code> namespace, and then the container instance ID. For example, <code>arn:aws:ecs:region:aws_account_id:container-instance/container_instance_ID</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>dockerVersion</i> The Docker version running on the container instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>agentHash</i> The Git commit hash for the Amazon ECS container agent build on the <a href="https://github.com/aws/amazon-ecs-agent/commits/master">amazon-ecs-agent </a> GitHub repository.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>agentVersion</i> The version number of the Amazon ECS container agent.<br/>        """
        pass

    def register_task_definition(self, containerDefinitions: List[ContainerDefinition], family: str, taskRoleArn: str=None, executionRoleArn: str=None, networkMode: str=None, volumes: List[Volume]=None, placementConstraints: List[TaskDefinitionPlacementConstraint]=None, requiresCompatibilities: List[Compatibility]=None, cpu: str=None, memory: str=None, tags: List[Tag]=None, pidMode: str=None, ipcMode: str=None, proxyConfiguration: ProxyConfiguration=None) -> RegisterTaskDefinitionResponse:
        """Registers a new task definition from the supplied <code>family</code> and <code>containerDefinitions</code>. Optionally, you can add data volumes to your containers with the <code>volumes</code> parameter. For more information about task definition parameters and defaults, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task_defintions.html">Amazon ECS Task Definitions</a> in the <i>Amazon Elastic Container Service Developer Guide</i>. You can specify an IAM role for your task with the <code>taskRoleArn</code> parameter. When you specify an IAM role for a task, its containers can then use the latest versions of the AWS CLI or SDKs to make API requests to the AWS services that are specified in the IAM policy associated with the role. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task-iam-roles.html">IAM Roles for Tasks</a> in the <i>Amazon Elastic Container Service Developer Guide</i>. You can specify a Docker networking mode for the containers in your task definition with the <code>networkMode</code> parameter. The available network modes correspond to those described in <a href="https://docs.docker.com/engine/reference/run/#/network-settings">Network settings</a> in the Docker run reference. If you specify the <code>awsvpc</code> network mode, the task is allocated an elastic network interface, and you must specify a <a>NetworkConfiguration</a> when you create a service or run a task with the task definition. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task-networking.html">Task Networking</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.

        :param self:
        :param proxyConfiguration:  The configuration details for the App Mesh proxy. For tasks using the EC2 launch type, the container instances require at least version 1.26.0 of the container agent and at least version 1.26.0-1 of the <code>ecs-init</code> package to enable a proxy configuration. If your container instances are launched from the Amazon ECS-optimized AMI version <code>20190301</code> or later, then they contain the required versions of the container agent and <code>ecs-init</code>. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/ecs-optimized_AMI.html">Amazon ECS-optimized Linux AMI</a> in the <i>Amazon Elastic Container Service Developer Guide</i>. For tasks using the Fargate launch type, the task or service requires platform version 1.3.0 or later.:<br/>ipcMode:  The IPC resource namespace to use for the containers in the task. The valid values are <code>host</code>, <code>task</code>, or <code>none</code>. If <code>host</code> is specified, then all containers within the tasks that specified the <code>host</code> IPC mode on the same container instance share the same IPC resources with the host Amazon EC2 instance. If <code>task</code> is specified, all containers within the specified task share the same IPC resources. If <code>none</code> is specified, then IPC resources within the containers of a task are private and not shared with other containers in a task or on the container instance. If no value is specified, then the IPC resource namespace sharing depends on the Docker daemon setting on the container instance. For more information, see <a href="https://docs.docker.com/engine/reference/run/#ipc-settings---ipc">IPC settings</a> in the <i>Docker run reference</i>. If the <code>host</code> IPC mode is used, be aware that there is a heightened risk of undesired IPC namespace expose. For more information, see <a href="https://docs.docker.com/engine/security/security/">Docker security</a>. If you are setting namespaced kernel parameters using <code>systemControls</code> for the containers in the task, the following will apply to your IPC resource namespace. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task_definition_parameters.html">System Controls</a> in the <i>Amazon Elastic Container Service Developer Guide</i>. <ul> <li> For tasks that use the <code>host</code> IPC mode, IPC namespace related <code>systemControls</code> are not supported. </li> <li> For tasks that use the <code>task</code> IPC mode, IPC namespace related <code>systemControls</code> will apply to all containers within a task. </li> </ul> <note> This parameter is not supported for Windows containers or tasks using the Fargate launch type. </note><br/>pidMode:  The process namespace to use for the containers in the task. The valid values are <code>host</code> or <code>task</code>. If <code>host</code> is specified, then all containers within the tasks that specified the <code>host</code> PID mode on the same container instance share the same IPC resources with the host Amazon EC2 instance. If <code>task</code> is specified, all containers within the specified task share the same process namespace. If no value is specified, the default is a private namespace. For more information, see <a href="https://docs.docker.com/engine/reference/run/#pid-settings---pid">PID settings</a> in the <i>Docker run reference</i>. If the <code>host</code> PID mode is used, be aware that there is a heightened risk of undesired process namespace expose. For more information, see <a href="https://docs.docker.com/engine/security/security/">Docker security</a>. <note> This parameter is not supported for Windows containers or tasks using the Fargate launch type. </note><br/>tags:  The metadata that you apply to the task definition to help you categorize and organize them. Each tag consists of a key and an optional value, both of which you define. Tag keys can have a maximum character length of 128 characters, and tag values can have a maximum length of 256 characters.<br/>memory:  The amount of memory (in MiB) used by the task. It can be expressed as an integer using MiB, for example <code>1024</code>, or as a string using GB, for example <code>1GB</code> or <code>1 GB</code>, in a task definition. String values are converted to an integer indicating the MiB when the task definition is registered. <note> Task-level CPU and memory parameters are ignored for Windows containers. We recommend specifying container-level resources for Windows containers. </note> If using the EC2 launch type, this field is optional. If using the Fargate launch type, this field is required and you must use one of the following values, which determines your range of supported values for the <code>cpu</code> parameter: <ul> <li> 512 (0.5 GB), 1024 (1 GB), 2048 (2 GB) - Available <code>cpu</code> values: 256 (.25 vCPU) </li> <li> 1024 (1 GB), 2048 (2 GB), 3072 (3 GB), 4096 (4 GB) - Available <code>cpu</code> values: 512 (.5 vCPU) </li> <li> 2048 (2 GB), 3072 (3 GB), 4096 (4 GB), 5120 (5 GB), 6144 (6 GB), 7168 (7 GB), 8192 (8 GB) - Available <code>cpu</code> values: 1024 (1 vCPU) </li> <li> Between 4096 (4 GB) and 16384 (16 GB) in increments of 1024 (1 GB) - Available <code>cpu</code> values: 2048 (2 vCPU) </li> <li> Between 8192 (8 GB) and 30720 (30 GB) in increments of 1024 (1 GB) - Available <code>cpu</code> values: 4096 (4 vCPU) </li> </ul><br/>cpu:  The number of CPU units used by the task. It can be expressed as an integer using CPU units, for example <code>1024</code>, or as a string using vCPUs, for example <code>1 vCPU</code> or <code>1 vcpu</code>, in a task definition. String values are converted to an integer indicating the CPU units when the task definition is registered. <note> Task-level CPU and memory parameters are ignored for Windows containers. We recommend specifying container-level resources for Windows containers. </note> If you are using the EC2 launch type, this field is optional. Supported values are between <code>128</code> CPU units (<code>0.125</code> vCPUs) and <code>10240</code> CPU units (<code>10</code> vCPUs). If you are using the Fargate launch type, this field is required and you must use one of the following values, which determines your range of supported values for the <code>memory</code> parameter: <ul> <li> 256 (.25 vCPU) - Available <code>memory</code> values: 512 (0.5 GB), 1024 (1 GB), 2048 (2 GB) </li> <li> 512 (.5 vCPU) - Available <code>memory</code> values: 1024 (1 GB), 2048 (2 GB), 3072 (3 GB), 4096 (4 GB) </li> <li> 1024 (1 vCPU) - Available <code>memory</code> values: 2048 (2 GB), 3072 (3 GB), 4096 (4 GB), 5120 (5 GB), 6144 (6 GB), 7168 (7 GB), 8192 (8 GB) </li> <li> 2048 (2 vCPU) - Available <code>memory</code> values: Between 4096 (4 GB) and 16384 (16 GB) in increments of 1024 (1 GB) </li> <li> 4096 (4 vCPU) - Available <code>memory</code> values: Between 8192 (8 GB) and 30720 (30 GB) in increments of 1024 (1 GB) </li> </ul><br/>requiresCompatibilities:  The launch type required by the task. If no value is specified, it defaults to <code>EC2</code>.<br/>placementConstraints:  An array of placement constraint objects to use for the task. You can specify a maximum of 10 constraints per task (this limit includes constraints in the task definition and those specified at runtime).<br/>volumes:  A list of volume definitions in JSON format that containers in your task may use.<br/>containerDefinitions:  A list of container definitions in JSON format that describe the different containers that make up your task.<br/>networkMode:  The Docker networking mode to use for the containers in the task. The valid values are <code>none</code>, <code>bridge</code>, <code>awsvpc</code>, and <code>host</code>. The default Docker network mode is <code>bridge</code>. If you are using the Fargate launch type, the <code>awsvpc</code> network mode is required. If you are using the EC2 launch type, any network mode can be used. If the network mode is set to <code>none</code>, you cannot specify port mappings in your container definitions, and the tasks containers do not have external connectivity. The <code>host</code> and <code>awsvpc</code> network modes offer the highest networking performance for containers because they use the EC2 network stack instead of the virtualized network stack provided by the <code>bridge</code> mode. With the <code>host</code> and <code>awsvpc</code> network modes, exposed container ports are mapped directly to the corresponding host port (for the <code>host</code> network mode) or the attached elastic network interface port (for the <code>awsvpc</code> network mode), so you cannot take advantage of dynamic host port mappings.  If the network mode is <code>awsvpc</code>, the task is allocated an elastic network interface, and you must specify a <a>NetworkConfiguration</a> value when you create a service or run a task with the task definition. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task-networking.html">Task Networking</a> in the <i>Amazon Elastic Container Service Developer Guide</i>. <note> Currently, only Amazon ECS-optimized AMIs, other Amazon Linux variants with the <code>ecs-init</code> package, or AWS Fargate infrastructure support the <code>awsvpc</code> network mode.  </note> If the network mode is <code>host</code>, you cannot run multiple instantiations of the same task on a single container instance when port mappings are used. Docker for Windows uses different network modes than Docker for Linux. When you register a task definition with Windows containers, you must not specify a network mode. If you use the console to register a task definition with Windows containers, you must choose the <code>&lt;default&gt;</code> network mode object.  For more information, see <a href="https://docs.docker.com/engine/reference/run/#network-settings">Network settings</a> in the <i>Docker run reference</i>.<br/>executionRoleArn:  The Amazon Resource Name (ARN) of the task execution role that the Amazon ECS container agent and the Docker daemon can assume.<br/>taskRoleArn:  The short name or full Amazon Resource Name (ARN) of the IAM role that containers in this task can assume. All containers in this task are granted the permissions that are specified in this role. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task-iam-roles.html">IAM Roles for Tasks</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>family:  You must specify a <code>family</code> for a task definition, which allows you to track multiple versions of the same task definition. The <code>family</code> is used as a name for your task definition. Up to 255 letters (uppercase and lowercase), numbers, and hyphens are allowed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>properties</i> The set of network configuration parameters to provide the Container Network Interface (CNI) plugin, specified as key-value pairs. <ul> <li>  <code>IgnoredUID</code> - (Required) The user ID (UID) of the proxy container as defined by the <code>user</code> parameter in a container definition. This is used to ensure the proxy ignores its own traffic. If <code>IgnoredGID</code> is specified, this field can be empty. </li> <li>  <code>IgnoredGID</code> - (Required) The group ID (GID) of the proxy container as defined by the <code>user</code> parameter in a container definition. This is used to ensure the proxy ignores its own traffic. If <code>IgnoredUID</code> is specified, this field can be empty. </li> <li>  <code>AppPorts</code> - (Required) The list of ports that the application uses. Network traffic to these ports is forwarded to the <code>ProxyIngressPort</code> and <code>ProxyEgressPort</code>. </li> <li>  <code>ProxyIngressPort</code> - (Required) Specifies the port that incoming traffic to the <code>AppPorts</code> is directed to. </li> <li>  <code>ProxyEgressPort</code> - (Required) Specifies the port that outgoing traffic from the <code>AppPorts</code> is directed to. </li> <li>  <code>EgressIgnoredPorts</code> - (Required) The egress traffic going to the specified ports is ignored and not redirected to the <code>ProxyEgressPort</code>. It can be an empty list. </li> <li>  <code>EgressIgnoredIPs</code> - (Required) The egress traffic going to the specified IP addresses is ignored and not redirected to the <code>ProxyEgressPort</code>. It can be an empty list. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>containerName</i> The name of the container that will serve as the App Mesh proxy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The proxy type. The only supported value is <code>APPMESH</code>.<br/>
        :return: tags:  The list of tags associated with the task definition.<br/>taskDefinition:  The full description of the registered task definition.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>proxyConfiguration</i> The configuration details for the App Mesh proxy. Your Amazon ECS container instances require at least version 1.26.0 of the container agent and at least version 1.26.0-1 of the <code>ecs-init</code> package to enable a proxy configuration. If your container instances are launched from the Amazon ECS-optimized AMI version <code>20190301</code> or later, then they contain the required versions of the container agent and <code>ecs-init</code>. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/ecs-optimized_AMI.html">Amazon ECS-optimized Linux AMI</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>proxyConfiguration</b>: The configuration details for the App Mesh proxy. Your Amazon ECS container instances require at least version 1.26.0 of the container agent and at least version 1.26.0-1 of the <code>ecs-init</code> package to enable a proxy configuration. If your container instances are launched from the Amazon ECS-optimized AMI version <code>20190301</code> or later, then they contain the required versions of the container agent and <code>ecs-init</code>. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/ecs-optimized_AMI.html">Amazon ECS-optimized Linux AMI</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ipcMode</i> The IPC resource namespace to use for the containers in the task. The valid values are <code>host</code>, <code>task</code>, or <code>none</code>. If <code>host</code> is specified, then all containers within the tasks that specified the <code>host</code> IPC mode on the same container instance share the same IPC resources with the host Amazon EC2 instance. If <code>task</code> is specified, all containers within the specified task share the same IPC resources. If <code>none</code> is specified, then IPC resources within the containers of a task are private and not shared with other containers in a task or on the container instance. If no value is specified, then the IPC resource namespace sharing depends on the Docker daemon setting on the container instance. For more information, see <a href="https://docs.docker.com/engine/reference/run/#ipc-settings---ipc">IPC settings</a> in the <i>Docker run reference</i>. If the <code>host</code> IPC mode is used, be aware that there is a heightened risk of undesired IPC namespace expose. For more information, see <a href="https://docs.docker.com/engine/security/security/">Docker security</a>. If you are setting namespaced kernel parameters using <code>systemControls</code> for the containers in the task, the following will apply to your IPC resource namespace. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task_definition_parameters.html">System Controls</a> in the <i>Amazon Elastic Container Service Developer Guide</i>. <ul> <li> For tasks that use the <code>host</code> IPC mode, IPC namespace related <code>systemControls</code> are not supported. </li> <li> For tasks that use the <code>task</code> IPC mode, IPC namespace related <code>systemControls</code> will apply to all containers within a task. </li> </ul> <note> This parameter is not supported for Windows containers or tasks using the Fargate launch type. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pidMode</i> The process namespace to use for the containers in the task. The valid values are <code>host</code> or <code>task</code>. If <code>host</code> is specified, then all containers within the tasks that specified the <code>host</code> PID mode on the same container instance share the same IPC resources with the host Amazon EC2 instance. If <code>task</code> is specified, all containers within the specified task share the same process namespace. If no value is specified, the default is a private namespace. For more information, see <a href="https://docs.docker.com/engine/reference/run/#pid-settings---pid">PID settings</a> in the <i>Docker run reference</i>. If the <code>host</code> PID mode is used, be aware that there is a heightened risk of undesired process namespace expose. For more information, see <a href="https://docs.docker.com/engine/security/security/">Docker security</a>. <note> This parameter is not supported for Windows containers or tasks using the Fargate launch type. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>memory</i> The amount (in MiB) of memory used by the task. If using the EC2 launch type, this field is optional and any value can be used. If using the Fargate launch type, this field is required and you must use one of the following values, which determines your range of valid values for the <code>cpu</code> parameter: <ul> <li> 512 (0.5 GB), 1024 (1 GB), 2048 (2 GB) - Available <code>cpu</code> values: 256 (.25 vCPU) </li> <li> 1024 (1 GB), 2048 (2 GB), 3072 (3 GB), 4096 (4 GB) - Available <code>cpu</code> values: 512 (.5 vCPU) </li> <li> 2048 (2 GB), 3072 (3 GB), 4096 (4 GB), 5120 (5 GB), 6144 (6 GB), 7168 (7 GB), 8192 (8 GB) - Available <code>cpu</code> values: 1024 (1 vCPU) </li> <li> Between 4096 (4 GB) and 16384 (16 GB) in increments of 1024 (1 GB) - Available <code>cpu</code> values: 2048 (2 vCPU) </li> <li> Between 8192 (8 GB) and 30720 (30 GB) in increments of 1024 (1 GB) - Available <code>cpu</code> values: 4096 (4 vCPU) </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cpu</i> The number of <code>cpu</code> units used by the task. If you are using the EC2 launch type, this field is optional and any value can be used. If you are using the Fargate launch type, this field is required and you must use one of the following values, which determines your range of valid values for the <code>memory</code> parameter: <ul> <li> 256 (.25 vCPU) - Available <code>memory</code> values: 512 (0.5 GB), 1024 (1 GB), 2048 (2 GB) </li> <li> 512 (.5 vCPU) - Available <code>memory</code> values: 1024 (1 GB), 2048 (2 GB), 3072 (3 GB), 4096 (4 GB) </li> <li> 1024 (1 vCPU) - Available <code>memory</code> values: 2048 (2 GB), 3072 (3 GB), 4096 (4 GB), 5120 (5 GB), 6144 (6 GB), 7168 (7 GB), 8192 (8 GB) </li> <li> 2048 (2 vCPU) - Available <code>memory</code> values: Between 4096 (4 GB) and 16384 (16 GB) in increments of 1024 (1 GB) </li> <li> 4096 (4 vCPU) - Available <code>memory</code> values: Between 8192 (8 GB) and 30720 (30 GB) in increments of 1024 (1 GB) </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>requiresCompatibilities</i> The launch type the task requires. If no value is specified, it will default to <code>EC2</code>. Valid values include <code>EC2</code> and <code>FARGATE</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>compatibilities</i> The launch type to use with your task. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/launch_types.html">Amazon ECS Launch Types</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>placementConstraints</i> An array of placement constraint objects to use for tasks. This field is not valid if you are using the Fargate launch type for your task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>requiresAttributes</i> The container instance attributes required by your task. This field is not valid if you are using the Fargate launch type for your task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the task definition.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>volumes</i> The list of volume definitions for the task. If your tasks are using the Fargate launch type, the <code>host</code> and <code>sourcePath</code> parameters are not supported. For more information about volume definition parameters and defaults, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task_definitions.html">Amazon ECS Task Definitions</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>revision</i> The revision of the task in a particular family. The revision is a version number of a task definition in a family. When you register a task definition for the first time, the revision is <code>1</code>. Each time that you register a new revision of a task definition in the same family, the revision value always increases by one, even if you have deregistered previous revisions in this family.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>networkMode</i> The Docker networking mode to use for the containers in the task. The valid values are <code>none</code>, <code>bridge</code>, <code>awsvpc</code>, and <code>host</code>. The default Docker network mode is <code>bridge</code>. If you are using the Fargate launch type, the <code>awsvpc</code> network mode is required. If you are using the EC2 launch type, any network mode can be used. If the network mode is set to <code>none</code>, you cannot specify port mappings in your container definitions, and the tasks containers do not have external connectivity. The <code>host</code> and <code>awsvpc</code> network modes offer the highest networking performance for containers because they use the EC2 network stack instead of the virtualized network stack provided by the <code>bridge</code> mode. With the <code>host</code> and <code>awsvpc</code> network modes, exposed container ports are mapped directly to the corresponding host port (for the <code>host</code> network mode) or the attached elastic network interface port (for the <code>awsvpc</code> network mode), so you cannot take advantage of dynamic host port mappings.  If the network mode is <code>awsvpc</code>, the task is allocated an elastic network interface, and you must specify a <a>NetworkConfiguration</a> value when you create a service or run a task with the task definition. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task-networking.html">Task Networking</a> in the <i>Amazon Elastic Container Service Developer Guide</i>. <note> Currently, only Amazon ECS-optimized AMIs, other Amazon Linux variants with the <code>ecs-init</code> package, or AWS Fargate infrastructure support the <code>awsvpc</code> network mode.  </note> If the network mode is <code>host</code>, you cannot run multiple instantiations of the same task on a single container instance when port mappings are used. Docker for Windows uses different network modes than Docker for Linux. When you register a task definition with Windows containers, you must not specify a network mode. If you use the console to register a task definition with Windows containers, you must choose the <code>&lt;default&gt;</code> network mode object.  For more information, see <a href="https://docs.docker.com/engine/reference/run/#network-settings">Network settings</a> in the <i>Docker run reference</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>executionRoleArn</i> The Amazon Resource Name (ARN) of the task execution role that containers in this task can assume. All containers in this task are granted the permissions that are specified in this role.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>taskRoleArn</i> The Amazon Resource Name (ARN) of an AWS Identity and Access Management (IAM) role that grants containers in the task permission to call AWS APIs on your behalf. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task_IAM_role.html">Amazon ECS Task Role</a> in the <i>Amazon Elastic Container Service Developer Guide</i>. IAM roles for tasks on Windows require that the <code>-EnableTaskIAMRole</code> option is set when you launch the Amazon ECS-optimized Windows AMI. Your containers must also run some configuration code in order to take advantage of the feature. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/windows_task_IAM_roles.html">Windows IAM Roles for Tasks</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>family</i> The name of a family that this task definition is registered to. A family groups multiple versions of a task definition. Amazon ECS gives the first task definition that you registered to a family a revision number of 1. Amazon ECS gives sequential revision numbers to each task definition that you add.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>containerDefinitions</i> A list of container definitions in JSON format that describe the different containers that make up your task. For more information about container definition parameters and defaults, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task_defintions.html">Amazon ECS Task Definitions</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>taskDefinitionArn</i> The full Amazon Resource Name (ARN) of the task definition.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>properties</i> The set of network configuration parameters to provide the Container Network Interface (CNI) plugin, specified as key-value pairs. <ul> <li>  <code>IgnoredUID</code> - (Required) The user ID (UID) of the proxy container as defined by the <code>user</code> parameter in a container definition. This is used to ensure the proxy ignores its own traffic. If <code>IgnoredGID</code> is specified, this field can be empty. </li> <li>  <code>IgnoredGID</code> - (Required) The group ID (GID) of the proxy container as defined by the <code>user</code> parameter in a container definition. This is used to ensure the proxy ignores its own traffic. If <code>IgnoredUID</code> is specified, this field can be empty. </li> <li>  <code>AppPorts</code> - (Required) The list of ports that the application uses. Network traffic to these ports is forwarded to the <code>ProxyIngressPort</code> and <code>ProxyEgressPort</code>. </li> <li>  <code>ProxyIngressPort</code> - (Required) Specifies the port that incoming traffic to the <code>AppPorts</code> is directed to. </li> <li>  <code>ProxyEgressPort</code> - (Required) Specifies the port that outgoing traffic from the <code>AppPorts</code> is directed to. </li> <li>  <code>EgressIgnoredPorts</code> - (Required) The egress traffic going to the specified ports is ignored and not redirected to the <code>ProxyEgressPort</code>. It can be an empty list. </li> <li>  <code>EgressIgnoredIPs</code> - (Required) The egress traffic going to the specified IP addresses is ignored and not redirected to the <code>ProxyEgressPort</code>. It can be an empty list. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>containerName</i> The name of the container that will serve as the App Mesh proxy.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The proxy type. The only supported value is <code>APPMESH</code>.<br/>        """
        pass

    def run_task(self, taskDefinition: str, cluster: str=None, overrides: TaskOverride=None, count: int=None, startedBy: str=None, group: str=None, placementConstraints: List[PlacementConstraint]=None, placementStrategy: List[PlacementStrategy]=None, launchType: str=None, platformVersion: str=None, networkConfiguration: NetworkConfiguration=None, tags: List[Tag]=None, enableECSManagedTags: Boolean=None, propagateTags: str=None) -> RunTaskResponse:
        """Starts a new task using the specified task definition. You can allow Amazon ECS to place tasks for you, or you can customize how Amazon ECS places tasks using placement constraints and placement strategies. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/scheduling_tasks.html">Scheduling Tasks</a> in the <i>Amazon Elastic Container Service Developer Guide</i>. Alternatively, you can use <a>StartTask</a> to use your own scheduler or place tasks manually on specific container instances. The Amazon ECS API follows an eventual consistency model, due to the distributed nature of the system supporting the API. This means that the result of an API command you run that affects your Amazon ECS resources might not be immediately visible to all subsequent commands you run. Keep this in mind when you carry out an API command that immediately follows a previous API command. To manage eventual consistency, you can do the following: <ul> <li> Confirm the state of the resource before you run a command to modify it. Run the DescribeTasks command using an exponential backoff algorithm to ensure that you allow enough time for the previous command to propagate through the system. To do this, run the DescribeTasks command repeatedly, starting with a couple of seconds of wait time and increasing gradually up to five minutes of wait time. </li> <li> Add wait time between subsequent commands, even if the DescribeTasks command returns an accurate response. Apply an exponential backoff algorithm starting with a couple of seconds of wait time, and increase gradually up to about five minutes of wait time. </li> </ul>

        :param self:
        :param propagateTags:  Specifies whether to propagate the tags from the task definition to the task. If no value is specified, the tags are not propagated. Tags can only be propagated to the task during task creation. To add tags to a task after task creation, use the <a>TagResource</a> API action. <note> An error will be received if you specify the <code>SERVICE</code> option when running a task. </note><br/>enableECSManagedTags:  Specifies whether to enable Amazon ECS managed tags for the task. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/ecs-using-tags.html">Tagging Your Amazon ECS Resources</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>tags:  The metadata that you apply to the task to help you categorize and organize them. Each tag consists of a key and an optional value, both of which you define. Tag keys can have a maximum character length of 128 characters, and tag values can have a maximum length of 256 characters.<br/>networkConfiguration:  The network configuration for the task. This parameter is required for task definitions that use the <code>awsvpc</code> network mode to receive their own elastic network interface, and it is not supported for other network modes. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task-networking.html">Task Networking</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.:<br/>platformVersion:  The platform version the task should run. A platform version is only specified for tasks using the Fargate launch type. If one is not specified, the <code>LATEST</code> platform version is used by default. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/platform_versions.html">AWS Fargate Platform Versions</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>launchType:  The launch type on which to run your task. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/launch_types.html">Amazon ECS Launch Types</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>placementStrategy:  The placement strategy objects to use for the task. You can specify a maximum of five strategy rules per task.<br/>placementConstraints:  An array of placement constraint objects to use for the task. You can specify up to 10 constraints per task (including constraints in the task definition and those specified at runtime).<br/>group:  The name of the task group to associate with the task. The default value is the family name of the task definition (for example, family:my-family-name).<br/>startedBy:  An optional tag specified when a task is started. For example, if you automatically trigger a task to run a batch process job, you could apply a unique identifier for that job to your task with the <code>startedBy</code> parameter. You can then identify which tasks belong to that job by filtering the results of a <a>ListTasks</a> call with the <code>startedBy</code> value. Up to 36 letters (uppercase and lowercase), numbers, hyphens, and underscores are allowed. If a task is started by an Amazon ECS service, then the <code>startedBy</code> parameter contains the deployment ID of the service that starts it.<br/>count:  The number of instantiations of the specified task to place on your cluster. You can specify up to 10 tasks per call.<br/>overrides:  A list of container overrides in JSON format that specify the name of a container in the specified task definition and the overrides it should receive. You can override the default command for a container (that is specified in the task definition or Docker image) with a <code>command</code> override. You can also override existing environment variables (that are specified in the task definition or Docker image) on a container or add new environment variables to it with an <code>environment</code> override. <note> A total of 8192 characters are allowed for overrides. This limit includes the JSON formatting characters of the override structure. </note>:<br/>taskDefinition:  The <code>family</code> and <code>revision</code> (<code>family:revision</code>) or full ARN of the task definition to run. If a <code>revision</code> is not specified, the latest <code>ACTIVE</code> revision is used.<br/>cluster:  The short name or full Amazon Resource Name (ARN) of the cluster on which to run your task. If you do not specify a cluster, the default cluster is assumed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>executionRoleArn</i> The Amazon Resource Name (ARN) of the task execution role that the Amazon ECS container agent and the Docker daemon can assume.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>taskRoleArn</i> The Amazon Resource Name (ARN) of the IAM role that containers in this task can assume. All containers in this task are granted the permissions that are specified in this role.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>containerOverrides</i> One or more container overrides sent to a task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>awsvpcConfiguration</i> The VPC subnets and security groups associated with a task. <note> All specified subnets and security groups must be from the same VPC. </note>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>awsvpcConfiguration</b>: The VPC subnets and security groups associated with a task. <note> All specified subnets and security groups must be from the same VPC. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>assignPublicIp</i> Whether the task's elastic network interface receives a public IP address. The default value is <code>DISABLED</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>securityGroups</i> The security groups associated with the task or service. If you do not specify a security group, the default security group for the VPC is used. There is a limit of 5 security groups that can be specified per <code>AwsVpcConfiguration</code>. <note> All specified security groups must be from the same VPC. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>subnets</i> The subnets associated with the task or service. There is a limit of 16 subnets that can be specified per <code>AwsVpcConfiguration</code>. <note> All specified subnets must be from the same VPC. </note><br/>
        :return: failures:  Any failures associated with the call.<br/>tasks:  A full description of the tasks that were run. The tasks that were successfully placed on your cluster are described here.<br/>        """
        pass

    def start_task(self, containerInstances: List[String], taskDefinition: str, cluster: str=None, overrides: TaskOverride=None, startedBy: str=None, group: str=None, networkConfiguration: NetworkConfiguration=None, tags: List[Tag]=None, enableECSManagedTags: Boolean=None, propagateTags: str=None) -> StartTaskResponse:
        """Starts a new task from the specified task definition on the specified container instance or instances. Alternatively, you can use <a>RunTask</a> to place tasks for you. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/scheduling_tasks.html">Scheduling Tasks</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.

        :param self:
        :param propagateTags:  Specifies whether to propagate the tags from the task definition or the service to the task. If no value is specified, the tags are not propagated.<br/>enableECSManagedTags:  Specifies whether to enable Amazon ECS managed tags for the task. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/ecs-using-tags.html">Tagging Your Amazon ECS Resources</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>tags:  The metadata that you apply to the task to help you categorize and organize them. Each tag consists of a key and an optional value, both of which you define. Tag keys can have a maximum character length of 128 characters, and tag values can have a maximum length of 256 characters.<br/>networkConfiguration:  The VPC subnet and security group configuration for tasks that receive their own elastic network interface by using the <code>awsvpc</code> networking mode.:<br/>group:  The name of the task group to associate with the task. The default value is the family name of the task definition (for example, family:my-family-name).<br/>startedBy:  An optional tag specified when a task is started. For example, if you automatically trigger a task to run a batch process job, you could apply a unique identifier for that job to your task with the <code>startedBy</code> parameter. You can then identify which tasks belong to that job by filtering the results of a <a>ListTasks</a> call with the <code>startedBy</code> value. Up to 36 letters (uppercase and lowercase), numbers, hyphens, and underscores are allowed. If a task is started by an Amazon ECS service, then the <code>startedBy</code> parameter contains the deployment ID of the service that starts it.<br/>containerInstances:  The container instance IDs or full ARN entries for the container instances on which you would like to place your task. You can specify up to 10 container instances.<br/>overrides:  A list of container overrides in JSON format that specify the name of a container in the specified task definition and the overrides it should receive. You can override the default command for a container (that is specified in the task definition or Docker image) with a <code>command</code> override. You can also override existing environment variables (that are specified in the task definition or Docker image) on a container or add new environment variables to it with an <code>environment</code> override. <note> A total of 8192 characters are allowed for overrides. This limit includes the JSON formatting characters of the override structure. </note>:<br/>taskDefinition:  The <code>family</code> and <code>revision</code> (<code>family:revision</code>) or full ARN of the task definition to start. If a <code>revision</code> is not specified, the latest <code>ACTIVE</code> revision is used.<br/>cluster:  The short name or full Amazon Resource Name (ARN) of the cluster on which to start your task. If you do not specify a cluster, the default cluster is assumed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>executionRoleArn</i> The Amazon Resource Name (ARN) of the task execution role that the Amazon ECS container agent and the Docker daemon can assume.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>taskRoleArn</i> The Amazon Resource Name (ARN) of the IAM role that containers in this task can assume. All containers in this task are granted the permissions that are specified in this role.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>containerOverrides</i> One or more container overrides sent to a task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>awsvpcConfiguration</i> The VPC subnets and security groups associated with a task. <note> All specified subnets and security groups must be from the same VPC. </note>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>awsvpcConfiguration</b>: The VPC subnets and security groups associated with a task. <note> All specified subnets and security groups must be from the same VPC. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>assignPublicIp</i> Whether the task's elastic network interface receives a public IP address. The default value is <code>DISABLED</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>securityGroups</i> The security groups associated with the task or service. If you do not specify a security group, the default security group for the VPC is used. There is a limit of 5 security groups that can be specified per <code>AwsVpcConfiguration</code>. <note> All specified security groups must be from the same VPC. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>subnets</i> The subnets associated with the task or service. There is a limit of 16 subnets that can be specified per <code>AwsVpcConfiguration</code>. <note> All specified subnets must be from the same VPC. </note><br/>
        :return: failures:  Any failures associated with the call.<br/>tasks:  A full description of the tasks that were started. Each task that was successfully placed on your container instances is described.<br/>        """
        pass

    def stop_task(self, task: str, cluster: str=None, reason: str=None) -> StopTaskResponse:
        """Stops a running task. Any tags associated with the task will be deleted. When <a>StopTask</a> is called on a task, the equivalent of <code>docker stop</code> is issued to the containers running in the task. This results in a <code>SIGTERM</code> value and a default 30-second timeout, after which the <code>SIGKILL</code> value is sent and the containers are forcibly stopped. If the container handles the <code>SIGTERM</code> value gracefully and exits within 30 seconds from receiving it, no <code>SIGKILL</code> value is sent. <note> The default 30-second timeout can be configured on the Amazon ECS container agent with the <code>ECS_CONTAINER_STOP_TIMEOUT</code> variable. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/ecs-agent-config.html">Amazon ECS Container Agent Configuration</a> in the <i>Amazon Elastic Container Service Developer Guide</i>. </note>

        :param self:
        :param reason:  An optional message specified when a task is stopped. For example, if you are using a custom scheduler, you can use this parameter to specify the reason for stopping the task here, and the message appears in subsequent <a>DescribeTasks</a> API operations on this task. Up to 255 characters are allowed in this message.<br/>task:  The task ID or full Amazon Resource Name (ARN) of the task to stop.<br/>cluster:  The short name or full Amazon Resource Name (ARN) of the cluster that hosts the task to stop. If you do not specify a cluster, the default cluster is assumed.<br/>
        :return: task:  The task that was stopped.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> The metadata that you apply to the task to help you categorize and organize them. Each tag consists of a key and an optional value, both of which you define. Tag keys can have a maximum character length of 128 characters, and tag values can have a maximum length of 256 characters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>healthStatus</i> The health status for the task, which is determined by the health of the essential containers in the task. If all essential containers in the task are reporting as <code>HEALTHY</code>, then the task status also reports as <code>HEALTHY</code>. If any essential containers in the task are reporting as <code>UNHEALTHY</code> or <code>UNKNOWN</code>, then the task status also reports as <code>UNHEALTHY</code> or <code>UNKNOWN</code>, accordingly. <note> The Amazon ECS container agent does not monitor or report on Docker health checks that are embedded in a container image (such as those specified in a parent image or from the image's Dockerfile) and not specified in the container definition. Health check parameters that are specified in a container definition override any Docker health checks that exist in the container image. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>attachments</i> The Elastic Network Adapter associated with the task if the task uses the <code>awsvpc</code> network mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>platformVersion</i> The platform version on which your task is running. A platform version is only specified for tasks using the Fargate launch type. If one is not specified, the <code>LATEST</code> platform version is used by default. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/platform_versions.html">AWS Fargate Platform Versions</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>launchType</i> The launch type on which your task is running. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/launch_types.html">Amazon ECS Launch Types</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>group</i> The name of the task group associated with the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stoppedAt</i> The Unix timestamp for when the task was stopped (the task transitioned from the <code>RUNNING</code> state to the <code>STOPPED</code> state).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stoppingAt</i> The Unix timestamp for when the task stops (transitions from the <code>RUNNING</code> state to <code>STOPPED</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>startedAt</i> The Unix timestamp for when the task started (the task transitioned from the <code>PENDING</code> state to the <code>RUNNING</code> state).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix timestamp for when the task was created (the task entered the <code>PENDING</code> state).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>executionStoppedAt</i> The Unix timestamp for when the task execution stopped.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pullStoppedAt</i> The Unix timestamp for when the container image pull completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pullStartedAt</i> The Unix timestamp for when the container image pull began.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>connectivityAt</i> The Unix timestamp for when the task last went into <code>CONNECTED</code> status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>connectivity</i> The connectivity status of a task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stopCode</i> The stop code indicating why a task was stopped. The <code>stoppedReason</code> may contain additional details.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stoppedReason</i> The reason that the task was stopped.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version counter for the task. Every time a task experiences a change that triggers a CloudWatch event, the version counter is incremented. If you are replicating your Amazon ECS task state with CloudWatch Events, you can compare the version of a task reported by the Amazon ECS API actionss with the version reported in CloudWatch Events for the task (inside the <code>detail</code> object) to verify that the version in your event stream is current.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>startedBy</i> The tag specified when a task is started. If the task is started by an Amazon ECS service, then the <code>startedBy</code> parameter contains the deployment ID of the service that starts it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>containers</i> The containers associated with the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>memory</i> The amount of memory (in MiB) used by the task as expressed in a task definition. It can be expressed as an integer using MiB, for example <code>1024</code>. It can also be expressed as a string using GB, for example <code>1GB</code> or <code>1 GB</code>. String values are converted to an integer indicating the MiB when the task definition is registered. If you are using the EC2 launch type, this field is optional. If you are using the Fargate launch type, this field is required and you must use one of the following values, which determines your range of supported values for the <code>cpu</code> parameter: <ul> <li> 512 (0.5 GB), 1024 (1 GB), 2048 (2 GB) - Available <code>cpu</code> values: 256 (.25 vCPU) </li> <li> 1024 (1 GB), 2048 (2 GB), 3072 (3 GB), 4096 (4 GB) - Available <code>cpu</code> values: 512 (.5 vCPU) </li> <li> 2048 (2 GB), 3072 (3 GB), 4096 (4 GB), 5120 (5 GB), 6144 (6 GB), 7168 (7 GB), 8192 (8 GB) - Available <code>cpu</code> values: 1024 (1 vCPU) </li> <li> Between 4096 (4 GB) and 16384 (16 GB) in increments of 1024 (1 GB) - Available <code>cpu</code> values: 2048 (2 vCPU) </li> <li> Between 8192 (8 GB) and 30720 (30 GB) in increments of 1024 (1 GB) - Available <code>cpu</code> values: 4096 (4 vCPU) </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cpu</i> The number of CPU units used by the task as expressed in a task definition. It can be expressed as an integer using CPU units, for example <code>1024</code>. It can also be expressed as a string using vCPUs, for example <code>1 vCPU</code> or <code>1 vcpu</code>. String values are converted to an integer indicating the CPU units when the task definition is registered. If you are using the EC2 launch type, this field is optional. Supported values are between <code>128</code> CPU units (<code>0.125</code> vCPUs) and <code>10240</code> CPU units (<code>10</code> vCPUs). If you are using the Fargate launch type, this field is required and you must use one of the following values, which determines your range of supported values for the <code>memory</code> parameter: <ul> <li> 256 (.25 vCPU) - Available <code>memory</code> values: 512 (0.5 GB), 1024 (1 GB), 2048 (2 GB) </li> <li> 512 (.5 vCPU) - Available <code>memory</code> values: 1024 (1 GB), 2048 (2 GB), 3072 (3 GB), 4096 (4 GB) </li> <li> 1024 (1 vCPU) - Available <code>memory</code> values: 2048 (2 GB), 3072 (3 GB), 4096 (4 GB), 5120 (5 GB), 6144 (6 GB), 7168 (7 GB), 8192 (8 GB) </li> <li> 2048 (2 vCPU) - Available <code>memory</code> values: Between 4096 (4 GB) and 16384 (16 GB) in increments of 1024 (1 GB) </li> <li> 4096 (4 vCPU) - Available <code>memory</code> values: Between 8192 (8 GB) and 30720 (30 GB) in increments of 1024 (1 GB) </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>desiredStatus</i> The desired status of the task. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task-lifecycle.html">Task Lifecycle</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastStatus</i> The last known status of the task. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task-lifecycle.html">Task Lifecycle</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>overrides</i> One or more container overrides.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>overrides</b>: One or more container overrides.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>containerInstanceArn</i> The ARN of the container instances that host the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>taskDefinitionArn</i> The ARN of the task definition that creates the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clusterArn</i> The ARN of the cluster that hosts the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>taskArn</i> The Amazon Resource Name (ARN) of the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>executionRoleArn</i> The Amazon Resource Name (ARN) of the task execution role that the Amazon ECS container agent and the Docker daemon can assume.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>taskRoleArn</i> The Amazon Resource Name (ARN) of the IAM role that containers in this task can assume. All containers in this task are granted the permissions that are specified in this role.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>containerOverrides</i> One or more container overrides sent to a task.<br/>        """
        pass

    def submit_attachment_state_changes(self, attachments: List[AttachmentStateChange], cluster: str=None) -> SubmitAttachmentStateChangesResponse:
        """<note> This action is only used by the Amazon ECS agent, and it is not intended for use outside of the agent. </note> Sent to acknowledge that an attachment changed states.

        :param self:
        :param attachments:  Any attachments associated with the state change request.<br/>cluster:  The short name or full ARN of the cluster that hosts the container instance the attachment belongs to.<br/>
        :return: acknowledgment:  Acknowledgement of the state change.<br/>        """
        pass

    def submit_container_state_change(self, cluster: str=None, task: str=None, containerName: str=None, status: str=None, exitCode: int=None, reason: str=None, networkBindings: List[NetworkBinding]=None) -> SubmitContainerStateChangeResponse:
        """<note> This action is only used by the Amazon ECS agent, and it is not intended for use outside of the agent. </note> Sent to acknowledge that a container changed states.

        :param self:
        :param networkBindings:  The network bindings of the container.<br/>reason:  The reason for the state change request.<br/>exitCode:  The exit code returned for the state change request.<br/>status:  The status of the state change request.<br/>containerName:  The name of the container.<br/>task:  The task ID or full Amazon Resource Name (ARN) of the task that hosts the container.<br/>cluster:  The short name or full ARN of the cluster that hosts the container.<br/>
        :return: acknowledgment:  Acknowledgement of the state change.<br/>        """
        pass

    def submit_task_state_change(self, cluster: str=None, task: str=None, status: str=None, reason: str=None, containers: List[ContainerStateChange]=None, attachments: List[AttachmentStateChange]=None, pullStartedAt: Timestamp=None, pullStoppedAt: Timestamp=None, executionStoppedAt: Timestamp=None) -> SubmitTaskStateChangeResponse:
        """<note> This action is only used by the Amazon ECS agent, and it is not intended for use outside of the agent. </note> Sent to acknowledge that a task changed states.

        :param self:
        :param executionStoppedAt:  The Unix timestamp for when the task execution stopped.<br/>pullStoppedAt:  The Unix timestamp for when the container image pull completed.<br/>pullStartedAt:  The Unix timestamp for when the container image pull began.<br/>attachments:  Any attachments associated with the state change request.<br/>containers:  Any containers associated with the state change request.<br/>reason:  The reason for the state change request.<br/>status:  The status of the state change request.<br/>task:  The task ID or full ARN of the task in the state change request.<br/>cluster:  The short name or full Amazon Resource Name (ARN) of the cluster that hosts the task.<br/>
        :return: acknowledgment:  Acknowledgement of the state change.<br/>        """
        pass

    def tag_resource(self, tags: List[Tag], resourceArn: str) -> TagResourceResponse:
        """Associates the specified tags to a resource with the specified <code>resourceArn</code>. If existing tags on a resource are not specified in the request parameters, they are not changed. When a resource is deleted, the tags associated with that resource are deleted as well.

        :param self:
        :param tags:  The tags to add to the resource. A tag is an array of key-value pairs. Tag keys can have a maximum character length of 128 characters, and tag values can have a maximum length of 256 characters.<br/>resourceArn:  The Amazon Resource Name (ARN) of the resource to which to add tags. Currently, the supported resources are Amazon ECS tasks, services, task definitions, clusters, and container instances.<br/>
        :return:         """
        pass

    def untag_resource(self, tagKeys: List[TagKey], resourceArn: str) -> UntagResourceResponse:
        """Deletes specified tags from a resource.

        :param self:
        :param tagKeys:  The keys of the tags to be removed.<br/>resourceArn:  The Amazon Resource Name (ARN) of the resource from which to delete tags. Currently, the supported resources are Amazon ECS tasks, services, task definitions, clusters, and container instances.<br/>
        :return:         """
        pass

    def update_container_agent(self, containerInstance: str, cluster: str=None) -> UpdateContainerAgentResponse:
        """Updates the Amazon ECS container agent on a specified container instance. Updating the Amazon ECS container agent does not interrupt running tasks or services on the container instance. The process for updating the agent differs depending on whether your container instance was launched with the Amazon ECS-optimized AMI or another operating system.  <code>UpdateContainerAgent</code> requires the Amazon ECS-optimized AMI or Amazon Linux with the <code>ecs-init</code> service installed and running. For help updating the Amazon ECS container agent on other operating systems, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/ecs-agent-update.html#manually_update_agent">Manually Updating the Amazon ECS Container Agent</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.

        :param self:
        :param containerInstance:  The container instance ID or full ARN entries for the container instance on which you would like to update the Amazon ECS container agent.<br/>cluster:  The short name or full Amazon Resource Name (ARN) of the cluster that your container instance is running on. If you do not specify a cluster, the default cluster is assumed.<br/>
        :return: containerInstance:  The container instance for which the container agent was updated.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> The metadata that you apply to the container instance to help you categorize and organize them. Each tag consists of a key and an optional value, both of which you define. Tag keys can have a maximum character length of 128 characters, and tag values can have a maximum length of 256 characters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>attachments</i> The resources attached to a container instance, such as elastic network interfaces.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>registeredAt</i> The Unix timestamp for when the container instance was registered.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>attributes</i> The attributes set for the container instance, either by the Amazon ECS container agent at instance registration or manually with the <a>PutAttributes</a> operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>agentUpdateStatus</i> The status of the most recent agent update. If an update has never been requested, this value is <code>NULL</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pendingTasksCount</i> The number of tasks on the container instance that are in the <code>PENDING</code> status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>runningTasksCount</i> The number of tasks on the container instance that are in the <code>RUNNING</code> status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>agentConnected</i> This parameter returns <code>true</code> if the agent is connected to Amazon ECS. Registered instances with an agent that may be unhealthy or stopped return <code>false</code>. Only instances connected to an agent can accept placement requests.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusReason</i> The reason that the container instance reached its current status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the container instance. The valid values are <code>REGISTERING</code>, <code>REGISTRATION_FAILED</code>, <code>ACTIVE</code>, <code>INACTIVE</code>, <code>DEREGISTERING</code>, or <code>DRAINING</code>. If your account has opted in to the <code>awsvpcTrunking</code> account setting, then any newly registered container instance will transition to a <code>REGISTERING</code> status while the trunk elastic network interface is provisioned for the instance. If the registration fails, the instance will transition to a <code>REGISTRATION_FAILED</code> status. You can describe the container instance and see the reason for failure in the <code>statusReason</code> parameter. Once the container instance is terminated, the instance transitions to a <code>DEREGISTERING</code> status while the trunk elastic network interface is deprovisioned. The instance then transitions to an <code>INACTIVE</code> status. The <code>ACTIVE</code> status indicates that the container instance can accept tasks. The <code>DRAINING</code> indicates that new tasks are not placed on the container instance and any service tasks running on the container instance are removed if possible. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/container-instance-draining.html">Container Instance Draining</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>registeredResources</i> For CPU and memory resource types, this parameter describes the amount of each resource that was available on the container instance when the container agent registered it with Amazon ECS. This value represents the total amount of CPU and memory that can be allocated on this container instance to tasks. For port resource types, this parameter describes the ports that were reserved by the Amazon ECS container agent when it registered the container instance with Amazon ECS.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>remainingResources</i> For CPU and memory resource types, this parameter describes the remaining CPU and memory that has not already been allocated to tasks and is therefore available for new tasks. For port resource types, this parameter describes the ports that were reserved by the Amazon ECS container agent (at instance registration time) and any task containers that have reserved port mappings on the host (with the <code>host</code> or <code>bridge</code> network mode). Any port that is not specified here is available for new tasks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>versionInfo</i> The version information for the Amazon ECS container agent and Docker daemon running on the container instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>versionInfo</b>: The version information for the Amazon ECS container agent and Docker daemon running on the container instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version counter for the container instance. Every time a container instance experiences a change that triggers a CloudWatch event, the version counter is incremented. If you are replicating your Amazon ECS container instance state with CloudWatch Events, you can compare the version of a container instance reported by the Amazon ECS APIs with the version reported in CloudWatch Events for the container instance (inside the <code>detail</code> object) to verify that the version in your event stream is current.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ec2InstanceId</i> The EC2 instance ID of the container instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>containerInstanceArn</i> The Amazon Resource Name (ARN) of the container instance. The ARN contains the <code>arn:aws:ecs</code> namespace, followed by the Region of the container instance, the AWS account ID of the container instance owner, the <code>container-instance</code> namespace, and then the container instance ID. For example, <code>arn:aws:ecs:region:aws_account_id:container-instance/container_instance_ID</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>dockerVersion</i> The Docker version running on the container instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>agentHash</i> The Git commit hash for the Amazon ECS container agent build on the <a href="https://github.com/aws/amazon-ecs-agent/commits/master">amazon-ecs-agent </a> GitHub repository.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>agentVersion</i> The version number of the Amazon ECS container agent.<br/>        """
        pass

    def update_container_instances_state(self, status: str, containerInstances: List[String], cluster: str=None) -> UpdateContainerInstancesStateResponse:
        """Modifies the status of an Amazon ECS container instance. Once a container instance has reached an <code>ACTIVE</code> state, you can change the status of a container instance to <code>DRAINING</code> to manually remove an instance from a cluster, for example to perform system updates, update the Docker daemon, or scale down the cluster size. <important> A container instance cannot be changed to <code>DRAINING</code> until it has reached an <code>ACTIVE</code> status. If the instance is in any other status, an error will be received. </important> When you set a container instance to <code>DRAINING</code>, Amazon ECS prevents new tasks from being scheduled for placement on the container instance and replacement service tasks are started on other container instances in the cluster if the resources are available. Service tasks on the container instance that are in the <code>PENDING</code> state are stopped immediately. Service tasks on the container instance that are in the <code>RUNNING</code> state are stopped and replaced according to the service's deployment configuration parameters, <code>minimumHealthyPercent</code> and <code>maximumPercent</code>. You can change the deployment configuration of your service using <a>UpdateService</a>. <ul> <li> If <code>minimumHealthyPercent</code> is below 100%, the scheduler can ignore <code>desiredCount</code> temporarily during task replacement. For example, <code>desiredCount</code> is four tasks, a minimum of 50% allows the scheduler to stop two existing tasks before starting two new tasks. If the minimum is 100%, the service scheduler can't remove existing tasks until the replacement tasks are considered healthy. Tasks for services that do not use a load balancer are considered healthy if they are in the <code>RUNNING</code> state. Tasks for services that use a load balancer are considered healthy if they are in the <code>RUNNING</code> state and the container instance they are hosted on is reported as healthy by the load balancer. </li> <li> The <code>maximumPercent</code> parameter represents an upper limit on the number of running tasks during task replacement, which enables you to define the replacement batch size. For example, if <code>desiredCount</code> is four tasks, a maximum of 200% starts four new tasks before stopping the four tasks to be drained, provided that the cluster resources required to do this are available. If the maximum is 100%, then replacement tasks can't start until the draining tasks have stopped. </li> </ul> Any <code>PENDING</code> or <code>RUNNING</code> tasks that do not belong to a service are not affected. You must wait for them to finish or stop them manually. A container instance has completed draining when it has no more <code>RUNNING</code> tasks. You can verify this using <a>ListTasks</a>. When a container instance has been drained, you can set a container instance to <code>ACTIVE</code> status and once it has reached that status the Amazon ECS scheduler can begin scheduling tasks on the instance again.

        :param self:
        :param status:  The container instance state with which to update the container instance. The only valid values for this action are <code>ACTIVE</code> and <code>DRAINING</code>. A container instance can only be updated to <code>DRAINING</code> status once it has reached an <code>ACTIVE</code> state. If a container instance is in <code>REGISTERING</code>, <code>DEREGISTERING</code>, or <code>REGISTRATION_FAILED</code> state you can describe the container instance but will be unable to update the container instance state.<br/>containerInstances:  A list of container instance IDs or full ARN entries.<br/>cluster:  The short name or full Amazon Resource Name (ARN) of the cluster that hosts the container instance to update. If you do not specify a cluster, the default cluster is assumed.<br/>
        :return: failures:  Any failures associated with the call.<br/>containerInstances:  The list of container instances.<br/>        """
        pass

    def update_service(self, service: str, cluster: str=None, desiredCount: int=None, taskDefinition: str=None, deploymentConfiguration: DeploymentConfiguration=None, networkConfiguration: NetworkConfiguration=None, platformVersion: str=None, forceNewDeployment: Boolean=None, healthCheckGracePeriodSeconds: int=None) -> UpdateServiceResponse:
        """Modifies the parameters of a service. For services using the rolling update (<code>ECS</code>) deployment controller, the desired count, deployment configuration, network configuration, or task definition used can be updated. For services using the blue/green (<code>CODE_DEPLOY</code>) deployment controller, only the desired count, deployment configuration, and health check grace period can be updated using this API. If the network configuration, platform version, or task definition need to be updated, a new AWS CodeDeploy deployment should be created. For more information, see <a href="https://docs.aws.amazon.com/codedeploy/latest/APIReference/API_CreateDeployment.html">CreateDeployment</a> in the <i>AWS CodeDeploy API Reference</i>. For services using an external deployment controller, you can update only the desired count and health check grace period using this API. If the launch type, load balancer, network configuration, platform version, or task definition need to be updated, you should create a new task set. For more information, see <a>CreateTaskSet</a>. You can add to or subtract from the number of instantiations of a task definition in a service by specifying the cluster that the service is running in and a new <code>desiredCount</code> parameter. If you have updated the Docker image of your application, you can create a new task definition with that image and deploy it to your service. The service scheduler uses the minimum healthy percent and maximum percent parameters (in the service's deployment configuration) to determine the deployment strategy. <note> If your updated Docker image uses the same tag as what is in the existing task definition for your service (for example, <code>my_image:latest</code>), you do not need to create a new revision of your task definition. You can update the service using the <code>forceNewDeployment</code> option. The new tasks launched by the deployment pull the current image/tag combination from your repository when they start. </note> You can also update the deployment configuration of a service. When a deployment is triggered by updating the task definition of a service, the service scheduler uses the deployment configuration parameters, <code>minimumHealthyPercent</code> and <code>maximumPercent</code>, to determine the deployment strategy. <ul> <li> If <code>minimumHealthyPercent</code> is below 100%, the scheduler can ignore <code>desiredCount</code> temporarily during a deployment. For example, if <code>desiredCount</code> is four tasks, a minimum of 50% allows the scheduler to stop two existing tasks before starting two new tasks. Tasks for services that do not use a load balancer are considered healthy if they are in the <code>RUNNING</code> state. Tasks for services that use a load balancer are considered healthy if they are in the <code>RUNNING</code> state and the container instance they are hosted on is reported as healthy by the load balancer. </li> <li> The <code>maximumPercent</code> parameter represents an upper limit on the number of running tasks during a deployment, which enables you to define the deployment batch size. For example, if <code>desiredCount</code> is four tasks, a maximum of 200% starts four new tasks before stopping the four older tasks (provided that the cluster resources required to do this are available). </li> </ul> When <a>UpdateService</a> stops a task during a deployment, the equivalent of <code>docker stop</code> is issued to the containers running in the task. This results in a <code>SIGTERM</code> and a 30-second timeout, after which <code>SIGKILL</code> is sent and the containers are forcibly stopped. If the container handles the <code>SIGTERM</code> gracefully and exits within 30 seconds from receiving it, no <code>SIGKILL</code> is sent. When the service scheduler launches new tasks, it determines task placement in your cluster with the following logic: <ul> <li> Determine which of the container instances in your cluster can support your service's task definition (for example, they have the required CPU, memory, ports, and container instance attributes). </li> <li> By default, the service scheduler attempts to balance tasks across Availability Zones in this manner (although you can choose a different placement strategy): <ul> <li> Sort the valid container instances by the fewest number of running tasks for this service in the same Availability Zone as the instance. For example, if zone A has one running service task and zones B and C each have zero, valid container instances in either zone B or C are considered optimal for placement. </li> <li> Place the new service task on a valid container instance in an optimal Availability Zone (based on the previous steps), favoring container instances with the fewest number of running tasks for this service. </li> </ul> </li> </ul> When the service scheduler stops running tasks, it attempts to maintain balance across the Availability Zones in your cluster using the following logic:  <ul> <li> Sort the container instances by the largest number of running tasks for this service in the same Availability Zone as the instance. For example, if zone A has one running service task and zones B and C each have two, container instances in either zone B or C are considered optimal for termination. </li> <li> Stop the task on a container instance in an optimal Availability Zone (based on the previous steps), favoring container instances with the largest number of running tasks for this service. </li> </ul>

        :param self:
        :param healthCheckGracePeriodSeconds:  The period of time, in seconds, that the Amazon ECS service scheduler should ignore unhealthy Elastic Load Balancing target health checks after a task has first started. This is only valid if your service is configured to use a load balancer. If your service's tasks take a while to start and respond to Elastic Load Balancing health checks, you can specify a health check grace period of up to 1,800 seconds. During that time, the ECS service scheduler ignores the Elastic Load Balancing health check status. This grace period can prevent the ECS service scheduler from marking tasks as unhealthy and stopping them before they have time to come up.<br/>forceNewDeployment:  Whether to force a new deployment of the service. Deployments are not forced by default. You can use this option to trigger a new deployment with no service definition changes. For example, you can update a service's tasks to use a newer Docker image with the same image/tag combination (<code>my_image:latest</code>) or to roll Fargate tasks onto a newer platform version.<br/>platformVersion:  The platform version on which your tasks in the service are running. A platform version is only specified for tasks using the Fargate launch type. If one is not specified, the <code>LATEST</code> platform version is used by default. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/platform_versions.html">AWS Fargate Platform Versions</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>networkConfiguration:  The network configuration for the service. This parameter is required for task definitions that use the <code>awsvpc</code> network mode to receive their own elastic network interface, and it is not supported for other network modes. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task-networking.html">Task Networking</a> in the <i>Amazon Elastic Container Service Developer Guide</i>. <note> Updating a service to add a subnet to a list of existing subnets does not trigger a service deployment. For example, if your network configuration change is to keep the existing subnets and simply add another subnet to the network configuration, this does not trigger a new service deployment. </note>:<br/>deploymentConfiguration:  Optional deployment parameters that control how many tasks run during the deployment and the ordering of stopping and starting tasks.:<br/>taskDefinition:  The <code>family</code> and <code>revision</code> (<code>family:revision</code>) or full ARN of the task definition to run in your service. If a <code>revision</code> is not specified, the latest <code>ACTIVE</code> revision is used. If you modify the task definition with <code>UpdateService</code>, Amazon ECS spawns a task with the new version of the task definition and then stops an old task after the new version is running.<br/>desiredCount:  The number of instantiations of the task to place and keep running in your service.<br/>service:  The name of the service to update.<br/>cluster:  The short name or full Amazon Resource Name (ARN) of the cluster that your service is running on. If you do not specify a cluster, the default cluster is assumed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>minimumHealthyPercent</i> If a service is using the rolling update (<code>ECS</code>) deployment type, the <b>minimum healthy percent</b> represents a lower limit on the number of tasks in a service that must remain in the <code>RUNNING</code> state during a deployment, as a percentage of the desired number of tasks (rounded up to the nearest integer), and while any container instances are in the <code>DRAINING</code> state if the service contains tasks using the EC2 launch type. This parameter enables you to deploy without using additional cluster capacity. For example, if your service has a desired number of four tasks and a minimum healthy percent of 50%, the scheduler may stop two existing tasks to free up cluster capacity before starting two new tasks. Tasks for services that <i>do not</i> use a load balancer are considered healthy if they are in the <code>RUNNING</code> state; tasks for services that <i>do</i> use a load balancer are considered healthy if they are in the <code>RUNNING</code> state and they are reported as healthy by the load balancer. The default value for minimum healthy percent is 100%. If a service is using the blue/green (<code>CODE_DEPLOY</code>) or <code>EXTERNAL</code> deployment types and tasks that use the EC2 launch type, the <b>minimum healthy percent</b> value is set to the default value and is used to define the lower limit on the number of the tasks in the service that remain in the <code>RUNNING</code> state while the container instances are in the <code>DRAINING</code> state. If the tasks in the service use the Fargate launch type, the minimum healthy percent value is not used, although it is returned when describing your service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>maximumPercent</i> If a service is using the rolling update (<code>ECS</code>) deployment type, the <b>maximum percent</b> parameter represents an upper limit on the number of tasks in a service that are allowed in the <code>RUNNING</code> or <code>PENDING</code> state during a deployment, as a percentage of the desired number of tasks (rounded down to the nearest integer), and while any container instances are in the <code>DRAINING</code> state if the service contains tasks using the EC2 launch type. This parameter enables you to define the deployment batch size. For example, if your service has a desired number of four tasks and a maximum percent value of 200%, the scheduler may start four new tasks before stopping the four older tasks (provided that the cluster resources required to do this are available). The default value for maximum percent is 200%. If a service is using the blue/green (<code>CODE_DEPLOY</code>) or <code>EXTERNAL</code> deployment types and tasks that use the EC2 launch type, the <b>maximum percent</b> value is set to the default value and is used to define the upper limit on the number of the tasks in the service that remain in the <code>RUNNING</code> state while the container instances are in the <code>DRAINING</code> state. If the tasks in the service use the Fargate launch type, the maximum percent value is not used, although it is returned when describing your service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>awsvpcConfiguration</i> The VPC subnets and security groups associated with a task. <note> All specified subnets and security groups must be from the same VPC. </note>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>awsvpcConfiguration</b>: The VPC subnets and security groups associated with a task. <note> All specified subnets and security groups must be from the same VPC. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>assignPublicIp</i> Whether the task's elastic network interface receives a public IP address. The default value is <code>DISABLED</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>securityGroups</i> The security groups associated with the task or service. If you do not specify a security group, the default security group for the VPC is used. There is a limit of 5 security groups that can be specified per <code>AwsVpcConfiguration</code>. <note> All specified security groups must be from the same VPC. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>subnets</i> The subnets associated with the task or service. There is a limit of 16 subnets that can be specified per <code>AwsVpcConfiguration</code>. <note> All specified subnets must be from the same VPC. </note><br/>
        :return: service:  The full description of your service following the update call.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>propagateTags</i> Specifies whether to propagate the tags from the task definition or the service to the task. If no value is specified, the tags are not propagated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enableECSManagedTags</i> Specifies whether to enable Amazon ECS managed tags for the tasks in the service. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/ecs-using-tags.html">Tagging Your Amazon ECS Resources</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdBy</i> The principal that created the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> The metadata that you apply to the service to help you categorize and organize them. Each tag consists of a key and an optional value, both of which you define. Tag keys can have a maximum character length of 128 characters, and tag values can have a maximum length of 256 characters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentController</i> The deployment controller type the service is using.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>deploymentController</b>: The deployment controller type the service is using.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>schedulingStrategy</i> The scheduling strategy to use for the service. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/ecs_services.html">Services</a>. There are two service scheduler strategies available: <ul> <li>  <code>REPLICA</code>-The replica scheduling strategy places and maintains the desired number of tasks across your cluster. By default, the service scheduler spreads tasks across Availability Zones. You can use task placement strategies and constraints to customize task placement decisions. </li> <li>  <code>DAEMON</code>-The daemon scheduling strategy deploys exactly one task on each container instance in your cluster. When you are using this strategy, do not specify a desired number of tasks or any task placement strategies. <note> Fargate tasks do not support the <code>DAEMON</code> scheduling strategy. </note> </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>healthCheckGracePeriodSeconds</i> The period of time, in seconds, that the Amazon ECS service scheduler ignores unhealthy Elastic Load Balancing target health checks after a task has first started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>networkConfiguration</i> The VPC subnet and security group configuration for tasks that receive their own elastic network interface by using the <code>awsvpc</code> networking mode.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>networkConfiguration</b>: The VPC subnet and security group configuration for tasks that receive their own elastic network interface by using the <code>awsvpc</code> networking mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>placementStrategy</i> The placement strategy that determines how tasks for the service are placed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>placementConstraints</i> The placement constraints for the tasks in the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix timestamp for when the service was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>events</i> The event stream for your service. A maximum of 100 of the latest events are displayed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The ARN of the IAM role associated with the service that allows the Amazon ECS container agent to register container instances with an Elastic Load Balancing load balancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deployments</i> The current state of deployments for the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>taskSets</i> Information about a set of Amazon ECS tasks in either an AWS CodeDeploy or an <code>EXTERNAL</code> deployment. An Amazon ECS task set includes details such as the desired number of tasks, how many tasks are running, and whether the task set serves production traffic.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deploymentConfiguration</i> Optional deployment parameters that control how many tasks run during the deployment and the ordering of stopping and starting tasks.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>deploymentConfiguration</b>: Optional deployment parameters that control how many tasks run during the deployment and the ordering of stopping and starting tasks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>taskDefinition</i> The task definition to use for tasks in the service. This value is specified when the service is created with <a>CreateService</a>, and it can be modified with <a>UpdateService</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>platformVersion</i> The platform version on which to run your service. A platform version is only specified for tasks using the Fargate launch type. If one is not specified, the <code>LATEST</code> platform version is used by default. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/platform_versions.html">AWS Fargate Platform Versions</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>launchType</i> The launch type on which your service is running. If no value is specified, it will default to <code>EC2</code>. Valid values include <code>EC2</code> and <code>FARGATE</code>. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/launch_types.html">Amazon ECS Launch Types</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pendingCount</i> The number of tasks in the cluster that are in the <code>PENDING</code> state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>runningCount</i> The number of tasks in the cluster that are in the <code>RUNNING</code> state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>desiredCount</i> The desired number of instantiations of the task definition to keep running on the service. This value is specified when the service is created with <a>CreateService</a>, and it can be modified with <a>UpdateService</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the service. The valid values are <code>ACTIVE</code>, <code>DRAINING</code>, or <code>INACTIVE</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceRegistries</i> The details of the service discovery registries to assign to this service. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/service-discovery.html">Service Discovery</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>loadBalancers</i> A list of Elastic Load Balancing load balancer objects, containing the load balancer name, the container name (as it appears in a container definition), and the container port to access from the load balancer. Services with tasks that use the <code>awsvpc</code> network mode (for example, those with the Fargate launch type) only support Application Load Balancers and Network Load Balancers. Classic Load Balancers are not supported. Also, when you create any target groups for these services, you must choose <code>ip</code> as the target type, not <code>instance</code>. Tasks that use the <code>awsvpc</code> network mode are associated with an elastic network interface, not an Amazon EC2 instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clusterArn</i> The Amazon Resource Name (ARN) of the cluster that hosts the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceName</i> The name of your service. Up to 255 letters (uppercase and lowercase), numbers, and hyphens are allowed. Service names must be unique within a cluster, but you can have similarly named services in multiple clusters within a Region or across multiple Regions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceArn</i> The ARN that identifies the service. The ARN contains the <code>arn:aws:ecs</code> namespace, followed by the Region of the service, the AWS account ID of the service owner, the <code>service</code> namespace, and then the service name. For example, <code>arn:aws:ecs:region:012345678910:service/my-service</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>minimumHealthyPercent</i> If a service is using the rolling update (<code>ECS</code>) deployment type, the <b>minimum healthy percent</b> represents a lower limit on the number of tasks in a service that must remain in the <code>RUNNING</code> state during a deployment, as a percentage of the desired number of tasks (rounded up to the nearest integer), and while any container instances are in the <code>DRAINING</code> state if the service contains tasks using the EC2 launch type. This parameter enables you to deploy without using additional cluster capacity. For example, if your service has a desired number of four tasks and a minimum healthy percent of 50%, the scheduler may stop two existing tasks to free up cluster capacity before starting two new tasks. Tasks for services that <i>do not</i> use a load balancer are considered healthy if they are in the <code>RUNNING</code> state; tasks for services that <i>do</i> use a load balancer are considered healthy if they are in the <code>RUNNING</code> state and they are reported as healthy by the load balancer. The default value for minimum healthy percent is 100%. If a service is using the blue/green (<code>CODE_DEPLOY</code>) or <code>EXTERNAL</code> deployment types and tasks that use the EC2 launch type, the <b>minimum healthy percent</b> value is set to the default value and is used to define the lower limit on the number of the tasks in the service that remain in the <code>RUNNING</code> state while the container instances are in the <code>DRAINING</code> state. If the tasks in the service use the Fargate launch type, the minimum healthy percent value is not used, although it is returned when describing your service.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>maximumPercent</i> If a service is using the rolling update (<code>ECS</code>) deployment type, the <b>maximum percent</b> parameter represents an upper limit on the number of tasks in a service that are allowed in the <code>RUNNING</code> or <code>PENDING</code> state during a deployment, as a percentage of the desired number of tasks (rounded down to the nearest integer), and while any container instances are in the <code>DRAINING</code> state if the service contains tasks using the EC2 launch type. This parameter enables you to define the deployment batch size. For example, if your service has a desired number of four tasks and a maximum percent value of 200%, the scheduler may start four new tasks before stopping the four older tasks (provided that the cluster resources required to do this are available). The default value for maximum percent is 200%. If a service is using the blue/green (<code>CODE_DEPLOY</code>) or <code>EXTERNAL</code> deployment types and tasks that use the EC2 launch type, the <b>maximum percent</b> value is set to the default value and is used to define the upper limit on the number of the tasks in the service that remain in the <code>RUNNING</code> state while the container instances are in the <code>DRAINING</code> state. If the tasks in the service use the Fargate launch type, the maximum percent value is not used, although it is returned when describing your service.<br/>The VPC subnets and security groups associated with a task. <note> All specified subnets and security groups must be from the same VPC. </note>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>awsvpcConfiguration</b>: The VPC subnets and security groups associated with a task. <note> All specified subnets and security groups must be from the same VPC. </note>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The deployment controller type to use. There are three deployment controller types available: <dl> <dt>ECS</dt> <dd> The rolling update (<code>ECS</code>) deployment type involves replacing the current running version of the container with the latest version. The number of containers Amazon ECS adds or removes from the service during a rolling update is controlled by adjusting the minimum and maximum number of healthy tasks allowed during a service deployment, as specified in the <a>DeploymentConfiguration</a>. </dd> <dt>CODE_DEPLOY</dt> <dd> The blue/green (<code>CODE_DEPLOY</code>) deployment type uses the blue/green deployment model powered by AWS CodeDeploy, which allows you to verify a new deployment of a service before sending production traffic to it. </dd> <dt>EXTERNAL</dt> <dd> The external (<code>EXTERNAL</code>) deployment type enables you to use any third-party deployment controller for full control over the deployment process for an Amazon ECS service. </dd> </dl><br/>        """
        pass

    def update_service_primary_task_set(self, primaryTaskSet: str, service: str, cluster: str) -> UpdateServicePrimaryTaskSetResponse:
        """Modifies which task set in a service is the primary task set. Any parameters that are updated on the primary task set in a service will transition to the service. This is used when a service uses the <code>EXTERNAL</code> deployment controller type. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/deployment-types.html">Amazon ECS Deployment Types</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.

        :param self:
        :param primaryTaskSet:  The short name or full Amazon Resource Name (ARN) of the task set to set as the primary task set in the deployment.<br/>service:  The short name or full Amazon Resource Name (ARN) of the service that the task set exists in.<br/>cluster:  The short name or full Amazon Resource Name (ARN) of the cluster that hosts the service that the task set exists in.<br/>
        :return: taskSet:  Information about a set of Amazon ECS tasks in either an AWS CodeDeploy or an <code>EXTERNAL</code> deployment. An Amazon ECS task set includes details such as the desired number of tasks, how many tasks are running, and whether the task set serves production traffic.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stabilityStatusAt</i> The Unix timestamp for when the task set stability status was retrieved.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stabilityStatus</i> The stability status, which indicates whether the task set has reached a steady state. If the following conditions are met, the task set will be in <code>STEADY_STATE</code>: <ul> <li> The task <code>runningCount</code> is equal to the <code>computedDesiredCount</code>. </li> <li> The <code>pendingCount</code> is <code>0</code>. </li> <li> There are no tasks running on container instances in the <code>DRAINING</code> status. </li> <li> All tasks are reporting a healthy status from the load balancers, service discovery, and container health checks. <note> If a <code>healthCheckGracePeriodSeconds</code> value was set when the service was created, you may see a <code>STEADY_STATE</code> reached since unhealthy Elastic Load Balancing target health checks will be ignored until it expires. </note> </li> </ul> If any of those conditions are not met, the stability status returns <code>STABILIZING</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>scale</i> A floating-point percentage of the desired number of tasks to place and keep running in the task set.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>scale</b>: A floating-point percentage of the desired number of tasks to place and keep running in the task set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceRegistries</i> The details of the service discovery registries to assign to this task set. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/service-discovery.html">Service Discovery</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>loadBalancers</i> Details on a load balancer that is used with a task set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>networkConfiguration</i> The network configuration for the task set.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>networkConfiguration</b>: The network configuration for the task set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>platformVersion</i> The platform version on which the tasks in the task set are running. A platform version is only specified for tasks using the Fargate launch type. If one is not specified, the <code>LATEST</code> platform version is used by default. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/platform_versions.html">AWS Fargate Platform Versions</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>launchType</i> The launch type the tasks in the task set are using. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/launch_types.html">Amazon ECS Launch Types</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>updatedAt</i> The Unix timestamp for when the task set was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix timestamp for when the task set was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>runningCount</i> The number of tasks in the task set that are in the <code>RUNNING</code> status during a deployment. A task in the <code>RUNNING</code> state is running and ready for use.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pendingCount</i> The number of tasks in the task set that are in the <code>PENDING</code> status during a deployment. A task in the <code>PENDING</code> state is preparing to enter the <code>RUNNING</code> state. A task set enters the <code>PENDING</code> status when it launches for the first time or when it is restarted after being in the <code>STOPPED</code> state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>computedDesiredCount</i> The computed desired count for the task set. This is calculated by multiplying the service's <code>desiredCount</code> by the task set's <code>scale</code> percentage. The result is always rounded up. For example, if the computed desired count is 1.2, it rounds up to 2 tasks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>taskDefinition</i> The task definition the task set is using.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the task set. The following describes each state: <dl> <dt>PRIMARY</dt> <dd> The task set is serving production traffic. </dd> <dt>ACTIVE</dt> <dd> The task set is not serving production traffic. </dd> <dt>DRAINING</dt> <dd> The tasks in the task set are being stopped and their corresponding targets are being deregistered from their target group. </dd> </dl><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>externalId</i> The external ID associated with the task set. If a task set is created by an AWS CodeDeploy deployment, the <code>externalId</code> parameter contains the AWS CodeDeploy deployment ID. If a task set is created for an external deployment and is associated with a service discovery registry, the <code>externalId</code> parameter contains the <code>ECS_TASK_SET_EXTERNAL_ID</code> AWS Cloud Map attribute.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>startedBy</i> The tag specified when a task set is started. If the task set is created by an AWS CodeDeploy deployment, the <code>startedBy</code> parameter is <code>CODE_DEPLOY</code>. For a task set created for an external deployment, the startedBy field isn't used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clusterArn</i> The Amazon Resource Name (ARN) of the cluster that the service that hosts the task set exists in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceArn</i> The Amazon Resource Name (ARN) of the service the task set exists in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>taskSetArn</i> The Amazon Resource Name (ARN) of the task set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the task set.<br/>The VPC subnets and security groups associated with a task. <note> All specified subnets and security groups must be from the same VPC. </note>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>awsvpcConfiguration</b>: The VPC subnets and security groups associated with a task. <note> All specified subnets and security groups must be from the same VPC. </note>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>unit</i> The unit of measure for the scale value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>value</i> The value, specified as a percent total of a service's <code>desiredCount</code>, to scale the task set. Accepted values are numbers between 0 and 100.<br/>        """
        pass

    def update_task_set(self, scale: Scale, taskSet: str, service: str, cluster: str) -> UpdateTaskSetResponse:
        """Modifies a task set. This is used when a service uses the <code>EXTERNAL</code> deployment controller type. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/deployment-types.html">Amazon ECS Deployment Types</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.

        :param self:
        :param scale:  A floating-point percentage of the desired number of tasks to place and keep running in the task set.:<br/>taskSet:  The short name or full Amazon Resource Name (ARN) of the task set to update.<br/>service:  The short name or full Amazon Resource Name (ARN) of the service that the task set exists in.<br/>cluster:  The short name or full Amazon Resource Name (ARN) of the cluster that hosts the service that the task set exists in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>unit</i> The unit of measure for the scale value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>value</i> The value, specified as a percent total of a service's <code>desiredCount</code>, to scale the task set. Accepted values are numbers between 0 and 100.<br/>
        :return: taskSet:  Information about a set of Amazon ECS tasks in either an AWS CodeDeploy or an <code>EXTERNAL</code> deployment. An Amazon ECS task set includes details such as the desired number of tasks, how many tasks are running, and whether the task set serves production traffic.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stabilityStatusAt</i> The Unix timestamp for when the task set stability status was retrieved.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stabilityStatus</i> The stability status, which indicates whether the task set has reached a steady state. If the following conditions are met, the task set will be in <code>STEADY_STATE</code>: <ul> <li> The task <code>runningCount</code> is equal to the <code>computedDesiredCount</code>. </li> <li> The <code>pendingCount</code> is <code>0</code>. </li> <li> There are no tasks running on container instances in the <code>DRAINING</code> status. </li> <li> All tasks are reporting a healthy status from the load balancers, service discovery, and container health checks. <note> If a <code>healthCheckGracePeriodSeconds</code> value was set when the service was created, you may see a <code>STEADY_STATE</code> reached since unhealthy Elastic Load Balancing target health checks will be ignored until it expires. </note> </li> </ul> If any of those conditions are not met, the stability status returns <code>STABILIZING</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>scale</i> A floating-point percentage of the desired number of tasks to place and keep running in the task set.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>scale</b>: A floating-point percentage of the desired number of tasks to place and keep running in the task set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceRegistries</i> The details of the service discovery registries to assign to this task set. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/service-discovery.html">Service Discovery</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>loadBalancers</i> Details on a load balancer that is used with a task set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>networkConfiguration</i> The network configuration for the task set.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>networkConfiguration</b>: The network configuration for the task set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>platformVersion</i> The platform version on which the tasks in the task set are running. A platform version is only specified for tasks using the Fargate launch type. If one is not specified, the <code>LATEST</code> platform version is used by default. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/platform_versions.html">AWS Fargate Platform Versions</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>launchType</i> The launch type the tasks in the task set are using. For more information, see <a href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/launch_types.html">Amazon ECS Launch Types</a> in the <i>Amazon Elastic Container Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>updatedAt</i> The Unix timestamp for when the task set was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix timestamp for when the task set was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>runningCount</i> The number of tasks in the task set that are in the <code>RUNNING</code> status during a deployment. A task in the <code>RUNNING</code> state is running and ready for use.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pendingCount</i> The number of tasks in the task set that are in the <code>PENDING</code> status during a deployment. A task in the <code>PENDING</code> state is preparing to enter the <code>RUNNING</code> state. A task set enters the <code>PENDING</code> status when it launches for the first time or when it is restarted after being in the <code>STOPPED</code> state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>computedDesiredCount</i> The computed desired count for the task set. This is calculated by multiplying the service's <code>desiredCount</code> by the task set's <code>scale</code> percentage. The result is always rounded up. For example, if the computed desired count is 1.2, it rounds up to 2 tasks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>taskDefinition</i> The task definition the task set is using.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the task set. The following describes each state: <dl> <dt>PRIMARY</dt> <dd> The task set is serving production traffic. </dd> <dt>ACTIVE</dt> <dd> The task set is not serving production traffic. </dd> <dt>DRAINING</dt> <dd> The tasks in the task set are being stopped and their corresponding targets are being deregistered from their target group. </dd> </dl><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>externalId</i> The external ID associated with the task set. If a task set is created by an AWS CodeDeploy deployment, the <code>externalId</code> parameter contains the AWS CodeDeploy deployment ID. If a task set is created for an external deployment and is associated with a service discovery registry, the <code>externalId</code> parameter contains the <code>ECS_TASK_SET_EXTERNAL_ID</code> AWS Cloud Map attribute.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>startedBy</i> The tag specified when a task set is started. If the task set is created by an AWS CodeDeploy deployment, the <code>startedBy</code> parameter is <code>CODE_DEPLOY</code>. For a task set created for an external deployment, the startedBy field isn't used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clusterArn</i> The Amazon Resource Name (ARN) of the cluster that the service that hosts the task set exists in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serviceArn</i> The Amazon Resource Name (ARN) of the service the task set exists in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>taskSetArn</i> The Amazon Resource Name (ARN) of the task set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the task set.<br/>The VPC subnets and security groups associated with a task. <note> All specified subnets and security groups must be from the same VPC. </note>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>awsvpcConfiguration</b>: The VPC subnets and security groups associated with a task. <note> All specified subnets and security groups must be from the same VPC. </note>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>unit</i> The unit of measure for the scale value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>value</i> The value, specified as a percent total of a service's <code>desiredCount</code>, to scale the task set. Accepted values are numbers between 0 and 100.<br/>        """
        pass



class EFS(BaseClient):
    """<fullname>Amazon Elastic File System</fullname> Amazon Elastic File System (Amazon EFS) provides simple, scalable file storage for use with Amazon EC2 instances in the AWS Cloud. With Amazon EFS, storage capacity is elastic, growing and shrinking automatically as you add and remove files, so your applications have the storage they need, when they need it. For more information, see the <a href="https://docs.aws.amazon.com/efs/latest/ug/api-reference.html">User Guide</a>."""


    class DescribeFileSystemsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_file_systems` for valid parameters"""
            pass
    class DescribeMountTargetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_mount_targets` for valid parameters"""
            pass
    class DescribeTagsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_tags` for valid parameters"""
            pass


    class CreateFileSystemRequest(Mapping):
        pass
    
    class FileSystemDescription(Mapping):
        pass
    
    class Encrypted(object):
        pass
    
    class ProvisionedThroughputInMibps(object):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreateMountTargetRequest(Mapping):
        pass
    
    class MountTargetDescription(Mapping):
        pass
    
    class SecurityGroup(object):
        pass
    
    class CreateTagsRequest(Mapping):
        pass
    
    class DeleteFileSystemRequest(Mapping):
        pass
    
    class DeleteMountTargetRequest(Mapping):
        pass
    
    class DeleteTagsRequest(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class DescribeFileSystemsRequest(Mapping):
        pass
    
    class DescribeFileSystemsResponse(Mapping):
        pass
    
    class DescribeLifecycleConfigurationRequest(Mapping):
        pass
    
    class LifecycleConfigurationDescription(Mapping):
        pass
    
    class DescribeMountTargetSecurityGroupsRequest(Mapping):
        pass
    
    class DescribeMountTargetSecurityGroupsResponse(Mapping):
        pass
    
    class DescribeMountTargetsRequest(Mapping):
        pass
    
    class DescribeMountTargetsResponse(Mapping):
        pass
    
    class DescribeTagsRequest(Mapping):
        pass
    
    class DescribeTagsResponse(Mapping):
        pass
    
    class ModifyMountTargetSecurityGroupsRequest(Mapping):
        pass
    
    class PutLifecycleConfigurationRequest(Mapping):
        pass
    
    class LifecyclePolicy(Mapping):
        pass
    
    class UpdateFileSystemRequest(Mapping):
        pass
    
    def create_file_system(self, CreationToken: str, PerformanceMode: str=None, Encrypted: Encrypted=None, KmsKeyId: str=None, ThroughputMode: str=None, ProvisionedThroughputInMibps: ProvisionedThroughputInMibps=None, Tags: List[Tag]=None) -> FileSystemDescription:
        """Creates a new, empty file system. The operation requires a creation token in the request that Amazon EFS uses to ensure idempotent creation (calling the operation with same creation token has no effect). If a file system does not currently exist that is owned by the caller's AWS account with the specified creation token, this operation does the following: <ul> <li> Creates a new, empty file system. The file system will have an Amazon EFS assigned ID, and an initial lifecycle state <code>creating</code>. </li> <li> Returns with the description of the created file system. </li> </ul> Otherwise, this operation returns a <code>FileSystemAlreadyExists</code> error with the ID of the existing file system. <note> For basic use cases, you can use a randomly generated UUID for the creation token. </note>  The idempotent operation allows you to retry a <code>CreateFileSystem</code> call without risk of creating an extra file system. This can happen when an initial call fails in a way that leaves it uncertain whether or not a file system was actually created. An example might be that a transport level timeout occurred or your connection was reset. As long as you use the same creation token, if the initial call had succeeded in creating a file system, the client can learn of its existence from the <code>FileSystemAlreadyExists</code> error. <note> The <code>CreateFileSystem</code> call returns while the file system's lifecycle state is still <code>creating</code>. You can check the file system creation status by calling the <a>DescribeFileSystems</a> operation, which among other things returns the file system state. </note> This operation also takes an optional <code>PerformanceMode</code> parameter that you choose for your file system. We recommend <code>generalPurpose</code> performance mode for most file systems. File systems using the <code>maxIO</code> performance mode can scale to higher levels of aggregate throughput and operations per second with a tradeoff of slightly higher latencies for most file operations. The performance mode can't be changed after the file system has been created. For more information, see <a href="https://docs.aws.amazon.com/efs/latest/ug/performance.html#performancemodes.html">Amazon EFS: Performance Modes</a>. After the file system is fully created, Amazon EFS sets its lifecycle state to <code>available</code>, at which point you can create one or more mount targets for the file system in your VPC. For more information, see <a>CreateMountTarget</a>. You mount your Amazon EFS file system on an EC2 instances in your VPC by using the mount target. For more information, see <a href="https://docs.aws.amazon.com/efs/latest/ug/how-it-works.html">Amazon EFS: How it Works</a>.   This operation requires permissions for the <code>elasticfilesystem:CreateFileSystem</code> action. 

        :param self:
        :param Tags:  A value that specifies to create one or more tags associated with the file system. Each tag is a user-defined key-value pair. Name your file system on creation by including a <code>"Key":"Name","Value":"{value}"</code> key-value pair.<br/>ProvisionedThroughputInMibps:  The throughput, measured in MiB/s, that you want to provision for a file system that you're creating. Valid values are 1-1024. Required if <code>ThroughputMode</code> is set to <code>provisioned</code>. The upper limit for throughput is 1024 MiB/s. You can get this limit increased by contacting AWS Support. For more information, see <a href="https://docs.aws.amazon.com/efs/latest/ug/limits.html#soft-limits">Amazon EFS Limits That You Can Increase</a> in the <i>Amazon EFS User Guide.</i> <br/>ThroughputMode:  The throughput mode for the file system to be created. There are two throughput modes to choose from for your file system: <code>bursting</code> and <code>provisioned</code>. If you set <code>ThroughputMode</code> to <code>provisioned</code>, you must also set a value for <code>ProvisionedThroughPutInMibps</code>. You can decrease your file system's throughput in Provisioned Throughput mode or change between the throughput modes as long as it’s been more than 24 hours since the last decrease or throughput mode change. For more, see <a href="https://docs.aws.amazon.com/efs/latest/ug/performance.html#provisioned-throughput">Specifying Throughput with Provisioned Mode</a> in the <i>Amazon EFS User Guide.</i> <br/>KmsKeyId:  The ID of the AWS KMS CMK to be used to protect the encrypted file system. This parameter is only required if you want to use a nondefault CMK. If this parameter is not specified, the default CMK for Amazon EFS is used. This ID can be in one of the following formats: <ul> <li> Key ID - A unique identifier of the key, for example <code>1234abcd-12ab-34cd-56ef-1234567890ab</code>. </li> <li> ARN - An Amazon Resource Name (ARN) for the key, for example <code>arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>. </li> <li> Key alias - A previously created display name for a key, for example <code>alias/projectKey1</code>. </li> <li> Key alias ARN - An ARN for a key alias, for example <code>arn:aws:kms:us-west-2:444455556666:alias/projectKey1</code>. </li> </ul> If <code>KmsKeyId</code> is specified, the <a>CreateFileSystemRequest$Encrypted</a> parameter must be set to true.<br/>Encrypted:  A Boolean value that, if true, creates an encrypted file system. When creating an encrypted file system, you have the option of specifying <a>CreateFileSystemRequest$KmsKeyId</a> for an existing AWS Key Management Service (AWS KMS) customer master key (CMK). If you don't specify a CMK, then the default CMK for Amazon EFS, <code>/aws/elasticfilesystem</code>, is used to protect the encrypted file system. <br/>PerformanceMode:  The performance mode of the file system. We recommend <code>generalPurpose</code> performance mode for most file systems. File systems using the <code>maxIO</code> performance mode can scale to higher levels of aggregate throughput and operations per second with a tradeoff of slightly higher latencies for most file operations. The performance mode can't be changed after the file system has been created.<br/>CreationToken:  A string of up to 64 ASCII characters. Amazon EFS uses this to ensure idempotent creation.<br/>
        :return: Tags:  The tags associated with the file system, presented as an array of <code>Tag</code> objects.<br/>ProvisionedThroughputInMibps:  The throughput, measured in MiB/s, that you want to provision for a file system. Valid values are 1-1024. Required if <code>ThroughputMode</code> is set to <code>provisioned</code>. The limit on throughput is 1024 MiB/s. You can get these limits increased by contacting AWS Support. For more information, see <a href="https://docs.aws.amazon.com/efs/latest/ug/limits.html#soft-limits">Amazon EFS Limits That You Can Increase</a> in the <i>Amazon EFS User Guide.</i> <br/>ThroughputMode:  The throughput mode for a file system. There are two throughput modes to choose from for your file system: <code>bursting</code> and <code>provisioned</code>. If you set <code>ThroughputMode</code> to <code>provisioned</code>, you must also set a value for <code>ProvisionedThroughPutInMibps</code>. You can decrease your file system's throughput in Provisioned Throughput mode or change between the throughput modes as long as it’s been more than 24 hours since the last decrease or throughput mode change. <br/>KmsKeyId:  The ID of an AWS Key Management Service (AWS KMS) customer master key (CMK) that was used to protect the encrypted file system.<br/>Encrypted:  A Boolean value that, if true, indicates that the file system is encrypted.<br/>PerformanceMode:  The performance mode of the file system.<br/>SizeInBytes:  The latest known metered size (in bytes) of data stored in the file system, in its <code>Value</code> field, and the time at which that size was determined in its <code>Timestamp</code> field. The <code>Timestamp</code> value is the integer number of seconds since 1970-01-01T00:00:00Z. The <code>SizeInBytes</code> value doesn't represent the size of a consistent snapshot of the file system, but it is eventually consistent when there are no writes to the file system. That is, <code>SizeInBytes</code> represents actual size only if the file system is not modified for a period longer than a couple of hours. Otherwise, the value is not the exact size that the file system was at any point in time. :<br/>NumberOfMountTargets:  The current number of mount targets that the file system has. For more information, see <a>CreateMountTarget</a>.<br/>Name:  You can add tags to a file system, including a <code>Name</code> tag. For more information, see <a>CreateFileSystem</a>. If the file system has a <code>Name</code> tag, Amazon EFS returns the value in this field. <br/>LifeCycleState:  The lifecycle phase of the file system.<br/>CreationTime:  The time that the file system was created, in seconds (since 1970-01-01T00:00:00Z).<br/>FileSystemId:  The ID of the file system, assigned by Amazon EFS.<br/>CreationToken:  The opaque string specified in the request.<br/>OwnerId:  The AWS account that created the file system. If the file system was created by an IAM user, the parent account to which the user belongs is the owner.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ValueInStandard</i> The latest known metered size (in bytes) of data stored in the Standard storage class.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ValueInIA</i> The latest known metered size (in bytes) of data stored in the Infrequent Access storage class.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timestamp</i> The time at which the size of data, returned in the <code>Value</code> field, was determined. The value is the integer number of seconds since 1970-01-01T00:00:00Z.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The latest known metered size (in bytes) of data stored in the file system.<br/>        """
        pass

    def create_mount_target(self, SubnetId: str, FileSystemId: str, IpAddress: str=None, SecurityGroups: List[SecurityGroup]=None) -> MountTargetDescription:
        """Creates a mount target for a file system. You can then mount the file system on EC2 instances by using the mount target. You can create one mount target in each Availability Zone in your VPC. All EC2 instances in a VPC within a given Availability Zone share a single mount target for a given file system. If you have multiple subnets in an Availability Zone, you create a mount target in one of the subnets. EC2 instances do not need to be in the same subnet as the mount target in order to access their file system. For more information, see <a href="https://docs.aws.amazon.com/efs/latest/ug/how-it-works.html">Amazon EFS: How it Works</a>.  In the request, you also specify a file system ID for which you are creating the mount target and the file system's lifecycle state must be <code>available</code>. For more information, see <a>DescribeFileSystems</a>. In the request, you also provide a subnet ID, which determines the following: <ul> <li> VPC in which Amazon EFS creates the mount target </li> <li> Availability Zone in which Amazon EFS creates the mount target </li> <li> IP address range from which Amazon EFS selects the IP address of the mount target (if you don't specify an IP address in the request) </li> </ul> After creating the mount target, Amazon EFS returns a response that includes, a <code>MountTargetId</code> and an <code>IpAddress</code>. You use this IP address when mounting the file system in an EC2 instance. You can also use the mount target's DNS name when mounting the file system. The EC2 instance on which you mount the file system by using the mount target can resolve the mount target's DNS name to its IP address. For more information, see <a href="https://docs.aws.amazon.com/efs/latest/ug/how-it-works.html#how-it-works-implementation">How it Works: Implementation Overview</a>.  Note that you can create mount targets for a file system in only one VPC, and there can be only one mount target per Availability Zone. That is, if the file system already has one or more mount targets created for it, the subnet specified in the request to add another mount target must meet the following requirements: <ul> <li> Must belong to the same VPC as the subnets of the existing mount targets </li> <li> Must not be in the same Availability Zone as any of the subnets of the existing mount targets </li> </ul> If the request satisfies the requirements, Amazon EFS does the following: <ul> <li> Creates a new mount target in the specified subnet. </li> <li> Also creates a new network interface in the subnet as follows: <ul> <li> If the request provides an <code>IpAddress</code>, Amazon EFS assigns that IP address to the network interface. Otherwise, Amazon EFS assigns a free address in the subnet (in the same way that the Amazon EC2 <code>CreateNetworkInterface</code> call does when a request does not specify a primary private IP address). </li> <li> If the request provides <code>SecurityGroups</code>, this network interface is associated with those security groups. Otherwise, it belongs to the default security group for the subnet's VPC. </li> <li> Assigns the description <code>Mount target <i>fsmt-id</i> for file system <i>fs-id</i> </code> where <code> <i>fsmt-id</i> </code> is the mount target ID, and <code> <i>fs-id</i> </code> is the <code>FileSystemId</code>. </li> <li> Sets the <code>requesterManaged</code> property of the network interface to <code>true</code>, and the <code>requesterId</code> value to <code>EFS</code>. </li> </ul> Each Amazon EFS mount target has one corresponding requester-managed EC2 network interface. After the network interface is created, Amazon EFS sets the <code>NetworkInterfaceId</code> field in the mount target's description to the network interface ID, and the <code>IpAddress</code> field to its address. If network interface creation fails, the entire <code>CreateMountTarget</code> operation fails. </li> </ul> <note> The <code>CreateMountTarget</code> call returns only after creating the network interface, but while the mount target state is still <code>creating</code>, you can check the mount target creation status by calling the <a>DescribeMountTargets</a> operation, which among other things returns the mount target state. </note> We recommend that you create a mount target in each of the Availability Zones. There are cost considerations for using a file system in an Availability Zone through a mount target created in another Availability Zone. For more information, see <a href="http://aws.amazon.com/efs/">Amazon EFS</a>. In addition, by always using a mount target local to the instance's Availability Zone, you eliminate a partial failure scenario. If the Availability Zone in which your mount target is created goes down, then you can't access your file system through that mount target.  This operation requires permissions for the following action on the file system: <ul> <li>  <code>elasticfilesystem:CreateMountTarget</code>  </li> </ul> This operation also requires permissions for the following Amazon EC2 actions: <ul> <li>  <code>ec2:DescribeSubnets</code>  </li> <li>  <code>ec2:DescribeNetworkInterfaces</code>  </li> <li>  <code>ec2:CreateNetworkInterface</code>  </li> </ul>

        :param self:
        :param SecurityGroups:  Up to five VPC security group IDs, of the form <code>sg-xxxxxxxx</code>. These must be for the same VPC as subnet specified.<br/>IpAddress:  Valid IPv4 address within the address range of the specified subnet.<br/>SubnetId:  The ID of the subnet to add the mount target in.<br/>FileSystemId:  The ID of the file system for which to create the mount target.<br/>
        :return: NetworkInterfaceId:  The ID of the network interface that Amazon EFS created when it created the mount target.<br/>IpAddress:  Address at which the file system can be mounted by using the mount target.<br/>LifeCycleState:  Lifecycle state of the mount target.<br/>SubnetId:  The ID of the mount target's subnet.<br/>FileSystemId:  The ID of the file system for which the mount target is intended.<br/>MountTargetId:  System-assigned mount target ID.<br/>OwnerId:  AWS account ID that owns the resource.<br/>        """
        pass

    def create_tags(self, Tags: List[Tag], FileSystemId: str):
        """Creates or overwrites tags associated with a file system. Each tag is a key-value pair. If a tag key specified in the request already exists on the file system, this operation overwrites its value with the value provided in the request. If you add the <code>Name</code> tag to your file system, Amazon EFS returns it in the response to the <a>DescribeFileSystems</a> operation.  This operation requires permission for the <code>elasticfilesystem:CreateTags</code> action.

        :param self:
        :param Tags:  An array of <code>Tag</code> objects to add. Each <code>Tag</code> object is a key-value pair. <br/>FileSystemId:  The ID of the file system whose tags you want to modify (String). This operation modifies the tags only, not the file system.<br/>
        :return:         """
        pass

    def delete_file_system(self, FileSystemId: str):
        """Deletes a file system, permanently severing access to its contents. Upon return, the file system no longer exists and you can't access any contents of the deleted file system.  You can't delete a file system that is in use. That is, if the file system has any mount targets, you must first delete them. For more information, see <a>DescribeMountTargets</a> and <a>DeleteMountTarget</a>.  <note> The <code>DeleteFileSystem</code> call returns while the file system state is still <code>deleting</code>. You can check the file system deletion status by calling the <a>DescribeFileSystems</a> operation, which returns a list of file systems in your account. If you pass file system ID or creation token for the deleted file system, the <a>DescribeFileSystems</a> returns a <code>404 FileSystemNotFound</code> error. </note> This operation requires permissions for the <code>elasticfilesystem:DeleteFileSystem</code> action.

        :param self:
        :param FileSystemId:  The ID of the file system you want to delete.<br/>
        :return:         """
        pass

    def delete_mount_target(self, MountTargetId: str):
        """Deletes the specified mount target. This operation forcibly breaks any mounts of the file system by using the mount target that is being deleted, which might disrupt instances or applications using those mounts. To avoid applications getting cut off abruptly, you might consider unmounting any mounts of the mount target, if feasible. The operation also deletes the associated network interface. Uncommitted writes might be lost, but breaking a mount target using this operation does not corrupt the file system itself. The file system you created remains. You can mount an EC2 instance in your VPC by using another mount target. This operation requires permissions for the following action on the file system: <ul> <li>  <code>elasticfilesystem:DeleteMountTarget</code>  </li> </ul> <note> The <code>DeleteMountTarget</code> call returns while the mount target state is still <code>deleting</code>. You can check the mount target deletion by calling the <a>DescribeMountTargets</a> operation, which returns a list of mount target descriptions for the given file system.  </note> The operation also requires permissions for the following Amazon EC2 action on the mount target's network interface: <ul> <li>  <code>ec2:DeleteNetworkInterface</code>  </li> </ul>

        :param self:
        :param MountTargetId:  The ID of the mount target to delete (String).<br/>
        :return:         """
        pass

    def delete_tags(self, TagKeys: List[TagKey], FileSystemId: str):
        """Deletes the specified tags from a file system. If the <code>DeleteTags</code> request includes a tag key that doesn't exist, Amazon EFS ignores it and doesn't cause an error. For more information about tags and related restrictions, see <a href="https://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/cost-alloc-tags.html">Tag Restrictions</a> in the <i>AWS Billing and Cost Management User Guide</i>. This operation requires permissions for the <code>elasticfilesystem:DeleteTags</code> action.

        :param self:
        :param TagKeys:  A list of tag keys to delete.<br/>FileSystemId:  The ID of the file system whose tags you want to delete (String).<br/>
        :return:         """
        pass

    def describe_file_systems(self, MaxItems: int=None, Marker: str=None, CreationToken: str=None, FileSystemId: str=None) -> DescribeFileSystemsResponse:
        """Returns the description of a specific Amazon EFS file system if either the file system <code>CreationToken</code> or the <code>FileSystemId</code> is provided. Otherwise, it returns descriptions of all file systems owned by the caller's AWS account in the AWS Region of the endpoint that you're calling. When retrieving all file system descriptions, you can optionally specify the <code>MaxItems</code> parameter to limit the number of descriptions in a response. Currently, this number is automatically set to 10. If more file system descriptions remain, Amazon EFS returns a <code>NextMarker</code>, an opaque token, in the response. In this case, you should send a subsequent request with the <code>Marker</code> request parameter set to the value of <code>NextMarker</code>.  To retrieve a list of your file system descriptions, this operation is used in an iterative process, where <code>DescribeFileSystems</code> is called first without the <code>Marker</code> and then the operation continues to call it with the <code>Marker</code> parameter set to the value of the <code>NextMarker</code> from the previous response until the response has no <code>NextMarker</code>.   The order of file systems returned in the response of one <code>DescribeFileSystems</code> call and the order of file systems returned across the responses of a multi-call iteration is unspecified.   This operation requires permissions for the <code>elasticfilesystem:DescribeFileSystems</code> action. 

        :param self:
        :param FileSystemId:  (Optional) ID of the file system whose description you want to retrieve (String).<br/>CreationToken:  (Optional) Restricts the list to the file system with this creation token (String). You specify a creation token when you create an Amazon EFS file system.<br/>Marker:  (Optional) Opaque pagination token returned from a previous <code>DescribeFileSystems</code> operation (String). If present, specifies to continue the list from where the returning call had left off. <br/>MaxItems:  (Optional) Specifies the maximum number of file systems to return in the response (integer). Currently, this number is automatically set to 10, and other values are ignored. The response is paginated at 10 per page if you have more than 10 file systems. <br/>
        :return: NextMarker:  Present if there are more file systems than returned in the response (String). You can use the <code>NextMarker</code> in the subsequent request to fetch the descriptions.<br/>FileSystems:  An array of file system descriptions.<br/>Marker:  Present if provided by caller in the request (String).<br/>        """
        pass

    def describe_lifecycle_configuration(self, FileSystemId: str) -> LifecycleConfigurationDescription:
        """Returns the current <code>LifecycleConfiguration</code> object for the specified Amazon EFS file system. EFS lifecycle management uses the <code>LifecycleConfiguration</code> object to identify which files to move to the EFS Infrequent Access (IA) storage class. For a file system without a <code>LifecycleConfiguration</code> object, the call returns an empty array in the response. This operation requires permissions for the <code>elasticfilesystem:DescribeLifecycleConfiguration</code> operation.

        :param self:
        :param FileSystemId:  The ID of the file system whose <code>LifecycleConfiguration</code> object you want to retrieve (String).<br/>
        :return: LifecyclePolicies:  An array of lifecycle management policies. Currently, EFS supports a maximum of one policy per file system.<br/>        """
        pass

    def describe_mount_target_security_groups(self, MountTargetId: str) -> DescribeMountTargetSecurityGroupsResponse:
        """Returns the security groups currently in effect for a mount target. This operation requires that the network interface of the mount target has been created and the lifecycle state of the mount target is not <code>deleted</code>. This operation requires permissions for the following actions: <ul> <li>  <code>elasticfilesystem:DescribeMountTargetSecurityGroups</code> action on the mount target's file system.  </li> <li>  <code>ec2:DescribeNetworkInterfaceAttribute</code> action on the mount target's network interface.  </li> </ul>

        :param self:
        :param MountTargetId:  The ID of the mount target whose security groups you want to retrieve.<br/>
        :return: SecurityGroups:  An array of security groups.<br/>        """
        pass

    def describe_mount_targets(self, MaxItems: int=None, Marker: str=None, FileSystemId: str=None, MountTargetId: str=None) -> DescribeMountTargetsResponse:
        """Returns the descriptions of all the current mount targets, or a specific mount target, for a file system. When requesting all of the current mount targets, the order of mount targets returned in the response is unspecified. This operation requires permissions for the <code>elasticfilesystem:DescribeMountTargets</code> action, on either the file system ID that you specify in <code>FileSystemId</code>, or on the file system of the mount target that you specify in <code>MountTargetId</code>.

        :param self:
        :param MountTargetId:  (Optional) ID of the mount target that you want to have described (String). It must be included in your request if <code>FileSystemId</code> is not included.<br/>FileSystemId:  (Optional) ID of the file system whose mount targets you want to list (String). It must be included in your request if <code>MountTargetId</code> is not included.<br/>Marker:  (Optional) Opaque pagination token returned from a previous <code>DescribeMountTargets</code> operation (String). If present, it specifies to continue the list from where the previous returning call left off.<br/>MaxItems:  (Optional) Maximum number of mount targets to return in the response. Currently, this number is automatically set to 10, and other values are ignored. The response is paginated at 10 per page if you have more than 10 mount targets.<br/>
        :return: NextMarker:  If a value is present, there are more mount targets to return. In a subsequent request, you can provide <code>Marker</code> in your request with this value to retrieve the next set of mount targets.<br/>MountTargets:  Returns the file system's mount targets as an array of <code>MountTargetDescription</code> objects.<br/>Marker:  If the request included the <code>Marker</code>, the response returns that value in this field.<br/>        """
        pass

    def describe_tags(self, FileSystemId: str, MaxItems: int=None, Marker: str=None) -> DescribeTagsResponse:
        """Returns the tags associated with a file system. The order of tags returned in the response of one <code>DescribeTags</code> call and the order of tags returned across the responses of a multiple-call iteration (when using pagination) is unspecified.   This operation requires permissions for the <code>elasticfilesystem:DescribeTags</code> action. 

        :param self:
        :param FileSystemId:  The ID of the file system whose tag set you want to retrieve.<br/>Marker:  (Optional) An opaque pagination token returned from a previous <code>DescribeTags</code> operation (String). If present, it specifies to continue the list from where the previous call left off.<br/>MaxItems:  (Optional) The maximum number of file system tags to return in the response. Currently, this number is automatically set to 10, and other values are ignored. The response is paginated at 10 per page if you have more than 10 tags.<br/>
        :return: NextMarker:  If a value is present, there are more tags to return. In a subsequent request, you can provide the value of <code>NextMarker</code> as the value of the <code>Marker</code> parameter in your next request to retrieve the next set of tags.<br/>Tags:  Returns tags associated with the file system as an array of <code>Tag</code> objects. <br/>Marker:  If the request included a <code>Marker</code>, the response returns that value in this field.<br/>        """
        pass

    def modify_mount_target_security_groups(self, MountTargetId: str, SecurityGroups: List[SecurityGroup]=None):
        """Modifies the set of security groups in effect for a mount target. When you create a mount target, Amazon EFS also creates a new network interface. For more information, see <a>CreateMountTarget</a>. This operation replaces the security groups in effect for the network interface associated with a mount target, with the <code>SecurityGroups</code> provided in the request. This operation requires that the network interface of the mount target has been created and the lifecycle state of the mount target is not <code>deleted</code>.  The operation requires permissions for the following actions: <ul> <li>  <code>elasticfilesystem:ModifyMountTargetSecurityGroups</code> action on the mount target's file system.  </li> <li>  <code>ec2:ModifyNetworkInterfaceAttribute</code> action on the mount target's network interface.  </li> </ul>

        :param self:
        :param SecurityGroups:  An array of up to five VPC security group IDs.<br/>MountTargetId:  The ID of the mount target whose security groups you want to modify.<br/>
        :return:         """
        pass

    def put_lifecycle_configuration(self, LifecyclePolicies: List[LifecyclePolicy], FileSystemId: str) -> LifecycleConfigurationDescription:
        """Enables lifecycle management by creating a new <code>LifecycleConfiguration</code> object. A <code>LifecycleConfiguration</code> object defines when files in an Amazon EFS file system are automatically transitioned to the lower-cost EFS Infrequent Access (IA) storage class. A <code>LifecycleConfiguration</code> applies to all files in a file system. Each Amazon EFS file system supports one lifecycle configuration, which applies to all files in the file system. If a <code>LifecycleConfiguration</code> object already exists for the specified file system, a <code>PutLifecycleConfiguration</code> call modifies the existing configuration. A <code>PutLifecycleConfiguration</code> call with an empty <code>LifecyclePolicies</code> array in the request body deletes any existing <code>LifecycleConfiguration</code> and disables lifecycle management. In the request, specify the following:  <ul> <li> The ID for the file system for which you are enabling, disabling, or modifying lifecycle management. </li> <li> A <code>LifecyclePolicies</code> array of <code>LifecyclePolicy</code> objects that define when files are moved to the IA storage class. The array can contain only one <code>LifecyclePolicy</code> item. </li> </ul> This operation requires permissions for the <code>elasticfilesystem:PutLifecycleConfiguration</code> operation. To apply a <code>LifecycleConfiguration</code> object to an encrypted file system, you need the same AWS Key Management Service (AWS KMS) permissions as when you created the encrypted file system. 

        :param self:
        :param LifecyclePolicies:  An array of <code>LifecyclePolicy</code> objects that define the file system's <code>LifecycleConfiguration</code> object. A <code>LifecycleConfiguration</code> object tells lifecycle management when to transition files from the Standard storage class to the Infrequent Access storage class.<br/>FileSystemId:  The ID of the file system for which you are creating the <code>LifecycleConfiguration</code> object (String).<br/>
        :return: LifecyclePolicies:  An array of lifecycle management policies. Currently, EFS supports a maximum of one policy per file system.<br/>        """
        pass

    def update_file_system(self, FileSystemId: str, ThroughputMode: str=None, ProvisionedThroughputInMibps: ProvisionedThroughputInMibps=None) -> FileSystemDescription:
        """Updates the throughput mode or the amount of provisioned throughput of an existing file system.

        :param self:
        :param ProvisionedThroughputInMibps:  (Optional) The amount of throughput, in MiB/s, that you want to provision for your file system. Valid values are 1-1024. Required if <code>ThroughputMode</code> is changed to <code>provisioned</code> on update. If you're not updating the amount of provisioned throughput for your file system, you don't need to provide this value in your request. <br/>ThroughputMode:  (Optional) The throughput mode that you want your file system to use. If you're not updating your throughput mode, you don't need to provide this value in your request. If you are changing the <code>ThroughputMode</code> to <code>provisioned</code>, you must also set a value for <code>ProvisionedThroughputInMibps</code>.<br/>FileSystemId:  The ID of the file system that you want to update.<br/>
        :return: Tags:  The tags associated with the file system, presented as an array of <code>Tag</code> objects.<br/>ProvisionedThroughputInMibps:  The throughput, measured in MiB/s, that you want to provision for a file system. Valid values are 1-1024. Required if <code>ThroughputMode</code> is set to <code>provisioned</code>. The limit on throughput is 1024 MiB/s. You can get these limits increased by contacting AWS Support. For more information, see <a href="https://docs.aws.amazon.com/efs/latest/ug/limits.html#soft-limits">Amazon EFS Limits That You Can Increase</a> in the <i>Amazon EFS User Guide.</i> <br/>ThroughputMode:  The throughput mode for a file system. There are two throughput modes to choose from for your file system: <code>bursting</code> and <code>provisioned</code>. If you set <code>ThroughputMode</code> to <code>provisioned</code>, you must also set a value for <code>ProvisionedThroughPutInMibps</code>. You can decrease your file system's throughput in Provisioned Throughput mode or change between the throughput modes as long as it’s been more than 24 hours since the last decrease or throughput mode change. <br/>KmsKeyId:  The ID of an AWS Key Management Service (AWS KMS) customer master key (CMK) that was used to protect the encrypted file system.<br/>Encrypted:  A Boolean value that, if true, indicates that the file system is encrypted.<br/>PerformanceMode:  The performance mode of the file system.<br/>SizeInBytes:  The latest known metered size (in bytes) of data stored in the file system, in its <code>Value</code> field, and the time at which that size was determined in its <code>Timestamp</code> field. The <code>Timestamp</code> value is the integer number of seconds since 1970-01-01T00:00:00Z. The <code>SizeInBytes</code> value doesn't represent the size of a consistent snapshot of the file system, but it is eventually consistent when there are no writes to the file system. That is, <code>SizeInBytes</code> represents actual size only if the file system is not modified for a period longer than a couple of hours. Otherwise, the value is not the exact size that the file system was at any point in time. :<br/>NumberOfMountTargets:  The current number of mount targets that the file system has. For more information, see <a>CreateMountTarget</a>.<br/>Name:  You can add tags to a file system, including a <code>Name</code> tag. For more information, see <a>CreateFileSystem</a>. If the file system has a <code>Name</code> tag, Amazon EFS returns the value in this field. <br/>LifeCycleState:  The lifecycle phase of the file system.<br/>CreationTime:  The time that the file system was created, in seconds (since 1970-01-01T00:00:00Z).<br/>FileSystemId:  The ID of the file system, assigned by Amazon EFS.<br/>CreationToken:  The opaque string specified in the request.<br/>OwnerId:  The AWS account that created the file system. If the file system was created by an IAM user, the parent account to which the user belongs is the owner.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ValueInStandard</i> The latest known metered size (in bytes) of data stored in the Standard storage class.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ValueInIA</i> The latest known metered size (in bytes) of data stored in the Infrequent Access storage class.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timestamp</i> The time at which the size of data, returned in the <code>Value</code> field, was determined. The value is the integer number of seconds since 1970-01-01T00:00:00Z.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The latest known metered size (in bytes) of data stored in the file system.<br/>        """
        pass



class EKS(BaseClient):
    """Amazon Elastic Kubernetes Service (Amazon EKS) is a managed service that makes it easy for you to run Kubernetes on AWS without needing to stand up or maintain your own Kubernetes control plane. Kubernetes is an open-source system for automating the deployment, scaling, and management of containerized applications.  Amazon EKS runs up-to-date versions of the open-source Kubernetes software, so you can use all the existing plugins and tooling from the Kubernetes community. Applications running on Amazon EKS are fully compatible with applications running on any standard Kubernetes environment, whether running in on-premises data centers or public clouds. This means that you can easily migrate any standard Kubernetes application to Amazon EKS without any code modification required."""

    class ClusterActiveWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_cluster` for valid parameters"""
            pass
    class ClusterDeletedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_cluster` for valid parameters"""
            pass


    class ListClustersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_clusters` for valid parameters"""
            pass
    class ListUpdatesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_updates` for valid parameters"""
            pass


    class CreateClusterRequest(Mapping):
        pass
    
    class CreateClusterResponse(Mapping):
        pass
    
    class VpcConfigRequest(Mapping):
        pass
    
    class Logging(Mapping):
        pass
    
    class DeleteClusterRequest(Mapping):
        pass
    
    class DeleteClusterResponse(Mapping):
        pass
    
    class DescribeClusterRequest(Mapping):
        pass
    
    class DescribeClusterResponse(Mapping):
        pass
    
    class DescribeUpdateRequest(Mapping):
        pass
    
    class DescribeUpdateResponse(Mapping):
        pass
    
    class ListClustersRequest(Mapping):
        pass
    
    class ListClustersResponse(Mapping):
        pass
    
    class ListUpdatesRequest(Mapping):
        pass
    
    class ListUpdatesResponse(Mapping):
        pass
    
    class UpdateClusterConfigRequest(Mapping):
        pass
    
    class UpdateClusterConfigResponse(Mapping):
        pass
    
    class UpdateClusterVersionRequest(Mapping):
        pass
    
    class UpdateClusterVersionResponse(Mapping):
        pass
    
    def create_cluster(self, resourcesVpcConfig: VpcConfigRequest, roleArn: str, name: str, version: str=None, logging: Logging=None, clientRequestToken: str=None) -> CreateClusterResponse:
        """Creates an Amazon EKS control plane.  The Amazon EKS control plane consists of control plane instances that run the Kubernetes software, such as <code>etcd</code> and the API server. The control plane runs in an account managed by AWS, and the Kubernetes API is exposed via the Amazon EKS API server endpoint. Each Amazon EKS cluster control plane is single-tenant and unique and runs on its own set of Amazon EC2 instances. The cluster control plane is provisioned across multiple Availability Zones and fronted by an Elastic Load Balancing Network Load Balancer. Amazon EKS also provisions elastic network interfaces in your VPC subnets to provide connectivity from the control plane instances to the worker nodes (for example, to support <code>kubectl exec</code>, <code>logs</code>, and <code>proxy</code> data flows). Amazon EKS worker nodes run in your AWS account and connect to your cluster's control plane via the Kubernetes API server endpoint and a certificate file that is created for your cluster. You can use the <code>endpointPublicAccess</code> and <code>endpointPrivateAccess</code> parameters to enable or disable public and private access to your cluster's Kubernetes API server endpoint. By default, public access is enabled, and private access is disabled. For more information, see <a href="https://docs.aws.amazon.com/eks/latest/userguide/cluster-endpoint.html">Amazon EKS Cluster Endpoint Access Control</a> in the <i> <i>Amazon EKS User Guide</i> </i>.  You can use the <code>logging</code> parameter to enable or disable exporting the Kubernetes control plane logs for your cluster to CloudWatch Logs. By default, cluster control plane logs aren't exported to CloudWatch Logs. For more information, see <a href="https://docs.aws.amazon.com/eks/latest/userguide/control-plane-logs.html">Amazon EKS Cluster Control Plane Logs</a> in the <i> <i>Amazon EKS User Guide</i> </i>. <note> CloudWatch Logs ingestion, archive storage, and data scanning rates apply to exported control plane logs. For more information, see <a href="http://aws.amazon.com/cloudwatch/pricing/">Amazon CloudWatch Pricing</a>. </note> Cluster creation typically takes between 10 and 15 minutes. After you create an Amazon EKS cluster, you must configure your Kubernetes tooling to communicate with the API server and launch worker nodes into your cluster. For more information, see <a href="https://docs.aws.amazon.com/eks/latest/userguide/managing-auth.html">Managing Cluster Authentication</a> and <a href="https://docs.aws.amazon.com/eks/latest/userguide/launch-workers.html">Launching Amazon EKS Worker Nodes</a> in the <i>Amazon EKS User Guide</i>.

        :param self:
        :param clientRequestToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request.<br/>logging:  Enable or disable exporting the Kubernetes control plane logs for your cluster to CloudWatch Logs. By default, cluster control plane logs aren't exported to CloudWatch Logs. For more information, see <a href="https://docs.aws.amazon.com/eks/latest/userguide/control-plane-logs.html">Amazon EKS Cluster Control Plane Logs</a> in the <i> <i>Amazon EKS User Guide</i> </i>. <note> CloudWatch Logs ingestion, archive storage, and data scanning rates apply to exported control plane logs. For more information, see <a href="http://aws.amazon.com/cloudwatch/pricing/">Amazon CloudWatch Pricing</a>. </note>:<br/>resourcesVpcConfig:  The VPC configuration used by the cluster control plane. Amazon EKS VPC resources have specific requirements to work properly with Kubernetes. For more information, see <a href="https://docs.aws.amazon.com/eks/latest/userguide/network_reqs.html">Cluster VPC Considerations</a> and <a href="https://docs.aws.amazon.com/eks/latest/userguide/sec-group-reqs.html">Cluster Security Group Considerations</a> in the <i>Amazon EKS User Guide</i>. You must specify at least two subnets. You can specify up to five security groups, but we recommend that you use a dedicated security group for your cluster control plane.:<br/>roleArn:  The Amazon Resource Name (ARN) of the IAM role that provides permissions for Amazon EKS to make calls to other AWS API operations on your behalf. For more information, see <a href="https://docs.aws.amazon.com/eks/latest/userguide/service_IAM_role.html">Amazon EKS Service IAM Role</a> in the <i> <i>Amazon EKS User Guide</i> </i>.<br/>version:  The desired Kubernetes version for your cluster. If you don't specify a value here, the latest version available in Amazon EKS is used.<br/>name:  The unique name to give to your cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>endpointPrivateAccess</i> Set this value to <code>true</code> to enable private access for your cluster's Kubernetes API server endpoint. If you enable private access, Kubernetes API requests from within your cluster's VPC use the private VPC endpoint. The default value for this parameter is <code>false</code>, which disables private access for your Kubernetes API server. For more information, see <a href="https://docs.aws.amazon.com/eks/latest/userguide/cluster-endpoint.html">Amazon EKS Cluster Endpoint Access Control</a> in the <i> <i>Amazon EKS User Guide</i> </i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>endpointPublicAccess</i> Set this value to <code>false</code> to disable public access for your cluster's Kubernetes API server endpoint. If you disable public access, your cluster's Kubernetes API server can receive only requests from within the cluster VPC. The default value for this parameter is <code>true</code>, which enables public access for your Kubernetes API server. For more information, see <a href="https://docs.aws.amazon.com/eks/latest/userguide/cluster-endpoint.html">Amazon EKS Cluster Endpoint Access Control</a> in the <i> <i>Amazon EKS User Guide</i> </i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>securityGroupIds</i> Specify one or more security groups for the cross-account elastic network interfaces that Amazon EKS creates to use to allow communication between your worker nodes and the Kubernetes control plane. If you don't specify a security group, the default security group for your VPC is used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>subnetIds</i> Specify subnets for your Amazon EKS worker nodes. Amazon EKS creates cross-account elastic network interfaces in these subnets to allow communication between your worker nodes and the Kubernetes control plane.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clusterLogging</i> The cluster control plane logging configuration for your cluster.<br/>
        :return: cluster:  The full description of your new cluster.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>platformVersion</i> The platform version of your Amazon EKS cluster. For more information, see <a href="https://docs.aws.amazon.com/eks/latest/userguide/platform-versions.html">Platform Versions</a> in the <i> <i>Amazon EKS User Guide</i> </i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clientRequestToken</i> Unique, case-sensitive identifier that you provide to ensure the idempotency of the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>certificateAuthority</i> The <code>certificate-authority-data</code> for your cluster.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>certificateAuthority</b>: The <code>certificate-authority-data</code> for your cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>logging</i> The logging configuration for your cluster.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>logging</b>: The logging configuration for your cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourcesVpcConfig</i> The VPC configuration used by the cluster control plane. Amazon EKS VPC resources have specific requirements to work properly with Kubernetes. For more information, see <a href="https://docs.aws.amazon.com/eks/latest/userguide/network_reqs.html">Cluster VPC Considerations</a> and <a href="https://docs.aws.amazon.com/eks/latest/userguide/sec-group-reqs.html">Cluster Security Group Considerations</a> in the <i>Amazon EKS User Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>resourcesVpcConfig</b>: The VPC configuration used by the cluster control plane. Amazon EKS VPC resources have specific requirements to work properly with Kubernetes. For more information, see <a href="https://docs.aws.amazon.com/eks/latest/userguide/network_reqs.html">Cluster VPC Considerations</a> and <a href="https://docs.aws.amazon.com/eks/latest/userguide/sec-group-reqs.html">Cluster Security Group Considerations</a> in the <i>Amazon EKS User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The Amazon Resource Name (ARN) of the IAM role that provides permissions for the Kubernetes control plane to make calls to AWS API operations on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>endpoint</i> The endpoint for your Kubernetes API server.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The Kubernetes server version for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix epoch timestamp in seconds for when the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>endpointPrivateAccess</i> This parameter indicates whether the Amazon EKS private API server endpoint is enabled. If the Amazon EKS private API server endpoint is enabled, Kubernetes API requests that originate from within your cluster's VPC use the private VPC endpoint instead of traversing the internet.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>endpointPublicAccess</i> This parameter indicates whether the Amazon EKS public API server endpoint is enabled. If the Amazon EKS public API server endpoint is disabled, your cluster's Kubernetes API server can receive only requests that originate from within the cluster VPC. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>vpcId</i> The VPC associated with your cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>securityGroupIds</i> The security groups associated with the cross-account elastic network interfaces that are used to allow communication between your worker nodes and the Kubernetes control plane.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>subnetIds</i> The subnets associated with your cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>clusterLogging</i> The cluster control plane logging configuration for your cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>data</i> The Base64-encoded certificate data required to communicate with your cluster. Add this to the <code>certificate-authority-data</code> section of the <code>kubeconfig</code> file for your cluster.<br/>        """
        pass

    def delete_cluster(self, name: str) -> DeleteClusterResponse:
        """Deletes the Amazon EKS cluster control plane.  <note> If you have active services in your cluster that are associated with a load balancer, you must delete those services before deleting the cluster so that the load balancers are deleted properly. Otherwise, you can have orphaned resources in your VPC that prevent you from being able to delete the VPC. For more information, see <a href="https://docs.aws.amazon.com/eks/latest/userguide/delete-cluster.html">Deleting a Cluster</a> in the <i>Amazon EKS User Guide</i>. </note>

        :param self:
        :param name:  The name of the cluster to delete.<br/>
        :return: cluster:  The full description of the cluster to delete.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>platformVersion</i> The platform version of your Amazon EKS cluster. For more information, see <a href="https://docs.aws.amazon.com/eks/latest/userguide/platform-versions.html">Platform Versions</a> in the <i> <i>Amazon EKS User Guide</i> </i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clientRequestToken</i> Unique, case-sensitive identifier that you provide to ensure the idempotency of the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>certificateAuthority</i> The <code>certificate-authority-data</code> for your cluster.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>certificateAuthority</b>: The <code>certificate-authority-data</code> for your cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>logging</i> The logging configuration for your cluster.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>logging</b>: The logging configuration for your cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourcesVpcConfig</i> The VPC configuration used by the cluster control plane. Amazon EKS VPC resources have specific requirements to work properly with Kubernetes. For more information, see <a href="https://docs.aws.amazon.com/eks/latest/userguide/network_reqs.html">Cluster VPC Considerations</a> and <a href="https://docs.aws.amazon.com/eks/latest/userguide/sec-group-reqs.html">Cluster Security Group Considerations</a> in the <i>Amazon EKS User Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>resourcesVpcConfig</b>: The VPC configuration used by the cluster control plane. Amazon EKS VPC resources have specific requirements to work properly with Kubernetes. For more information, see <a href="https://docs.aws.amazon.com/eks/latest/userguide/network_reqs.html">Cluster VPC Considerations</a> and <a href="https://docs.aws.amazon.com/eks/latest/userguide/sec-group-reqs.html">Cluster Security Group Considerations</a> in the <i>Amazon EKS User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The Amazon Resource Name (ARN) of the IAM role that provides permissions for the Kubernetes control plane to make calls to AWS API operations on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>endpoint</i> The endpoint for your Kubernetes API server.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The Kubernetes server version for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix epoch timestamp in seconds for when the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>endpointPrivateAccess</i> This parameter indicates whether the Amazon EKS private API server endpoint is enabled. If the Amazon EKS private API server endpoint is enabled, Kubernetes API requests that originate from within your cluster's VPC use the private VPC endpoint instead of traversing the internet.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>endpointPublicAccess</i> This parameter indicates whether the Amazon EKS public API server endpoint is enabled. If the Amazon EKS public API server endpoint is disabled, your cluster's Kubernetes API server can receive only requests that originate from within the cluster VPC. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>vpcId</i> The VPC associated with your cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>securityGroupIds</i> The security groups associated with the cross-account elastic network interfaces that are used to allow communication between your worker nodes and the Kubernetes control plane.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>subnetIds</i> The subnets associated with your cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>clusterLogging</i> The cluster control plane logging configuration for your cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>data</i> The Base64-encoded certificate data required to communicate with your cluster. Add this to the <code>certificate-authority-data</code> section of the <code>kubeconfig</code> file for your cluster.<br/>        """
        pass

    def describe_cluster(self, name: str) -> DescribeClusterResponse:
        """Returns descriptive information about an Amazon EKS cluster. The API server endpoint and certificate authority data returned by this operation are required for <code>kubelet</code> and <code>kubectl</code> to communicate with your Kubernetes API server. For more information, see <a href="https://docs.aws.amazon.com/eks/latest/userguide/create-kubeconfig.html">Create a kubeconfig for Amazon EKS</a>. <note> The API server endpoint and certificate authority data aren't available until the cluster reaches the <code>ACTIVE</code> state. </note>

        :param self:
        :param name:  The name of the cluster to describe.<br/>
        :return: cluster:  The full description of your specified cluster.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>platformVersion</i> The platform version of your Amazon EKS cluster. For more information, see <a href="https://docs.aws.amazon.com/eks/latest/userguide/platform-versions.html">Platform Versions</a> in the <i> <i>Amazon EKS User Guide</i> </i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clientRequestToken</i> Unique, case-sensitive identifier that you provide to ensure the idempotency of the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>certificateAuthority</i> The <code>certificate-authority-data</code> for your cluster.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>certificateAuthority</b>: The <code>certificate-authority-data</code> for your cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>logging</i> The logging configuration for your cluster.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>logging</b>: The logging configuration for your cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourcesVpcConfig</i> The VPC configuration used by the cluster control plane. Amazon EKS VPC resources have specific requirements to work properly with Kubernetes. For more information, see <a href="https://docs.aws.amazon.com/eks/latest/userguide/network_reqs.html">Cluster VPC Considerations</a> and <a href="https://docs.aws.amazon.com/eks/latest/userguide/sec-group-reqs.html">Cluster Security Group Considerations</a> in the <i>Amazon EKS User Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>resourcesVpcConfig</b>: The VPC configuration used by the cluster control plane. Amazon EKS VPC resources have specific requirements to work properly with Kubernetes. For more information, see <a href="https://docs.aws.amazon.com/eks/latest/userguide/network_reqs.html">Cluster VPC Considerations</a> and <a href="https://docs.aws.amazon.com/eks/latest/userguide/sec-group-reqs.html">Cluster Security Group Considerations</a> in the <i>Amazon EKS User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The Amazon Resource Name (ARN) of the IAM role that provides permissions for the Kubernetes control plane to make calls to AWS API operations on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>endpoint</i> The endpoint for your Kubernetes API server.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The Kubernetes server version for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix epoch timestamp in seconds for when the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>endpointPrivateAccess</i> This parameter indicates whether the Amazon EKS private API server endpoint is enabled. If the Amazon EKS private API server endpoint is enabled, Kubernetes API requests that originate from within your cluster's VPC use the private VPC endpoint instead of traversing the internet.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>endpointPublicAccess</i> This parameter indicates whether the Amazon EKS public API server endpoint is enabled. If the Amazon EKS public API server endpoint is disabled, your cluster's Kubernetes API server can receive only requests that originate from within the cluster VPC. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>vpcId</i> The VPC associated with your cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>securityGroupIds</i> The security groups associated with the cross-account elastic network interfaces that are used to allow communication between your worker nodes and the Kubernetes control plane.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>subnetIds</i> The subnets associated with your cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>clusterLogging</i> The cluster control plane logging configuration for your cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>data</i> The Base64-encoded certificate data required to communicate with your cluster. Add this to the <code>certificate-authority-data</code> section of the <code>kubeconfig</code> file for your cluster.<br/>        """
        pass

    def describe_update(self, updateId: str, name: str) -> DescribeUpdateResponse:
        """Returns descriptive information about an update against your Amazon EKS cluster. When the status of the update is <code>Succeeded</code>, the update is complete. If an update fails, the status is <code>Failed</code>, and an error detail explains the reason for the failure.

        :param self:
        :param updateId:  The ID of the update to describe.<br/>name:  The name of the Amazon EKS cluster to update.<br/>
        :return: update:  The full description of the specified update.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errors</i> Any errors associated with a <code>Failed</code> update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix epoch timestamp in seconds for when the update was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>params</i> A key-value map that contains the parameters associated with the update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of the update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> A UUID that is used to track the update.<br/>        """
        pass

    def list_clusters(self, maxResults: int=None, nextToken: str=None) -> ListClustersResponse:
        """Lists the Amazon EKS clusters in your AWS account in the specified Region.

        :param self:
        :param nextToken:  The <code>nextToken</code> value returned from a previous paginated <code>ListClusters</code> request where <code>maxResults</code> was used and the results exceeded the value of that parameter. Pagination continues from the end of the previous results that returned the <code>nextToken</code> value. <note> This token should be treated as an opaque identifier that is used only to retrieve the next items in a list and not for other programmatic purposes. </note><br/>maxResults:  The maximum number of cluster results returned by <code>ListClusters</code> in paginated output. When you use this parameter, <code>ListClusters</code> returns only <code>maxResults</code> results in a single page along with a <code>nextToken</code> response element. You can see the remaining results of the initial request by sending another <code>ListClusters</code> request with the returned <code>nextToken</code> value. This value can be between 1 and 100. If you don't use this parameter, <code>ListClusters</code> returns up to 100 results and a <code>nextToken</code> value if applicable.<br/>
        :return: nextToken:  The <code>nextToken</code> value to include in a future <code>ListClusters</code> request. When the results of a <code>ListClusters</code> request exceed <code>maxResults</code>, you can use this value to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>clusters:  A list of all of the clusters for your account in the specified Region.<br/>        """
        pass

    def list_updates(self, name: str, nextToken: str=None, maxResults: int=None) -> ListUpdatesResponse:
        """Lists the updates associated with an Amazon EKS cluster in your AWS account, in the specified Region.

        :param self:
        :param maxResults:  The maximum number of update results returned by <code>ListUpdates</code> in paginated output. When you use this parameter, <code>ListUpdates</code> returns only <code>maxResults</code> results in a single page along with a <code>nextToken</code> response element. You can see the remaining results of the initial request by sending another <code>ListUpdates</code> request with the returned <code>nextToken</code> value. This value can be between 1 and 100. If you don't use this parameter, <code>ListUpdates</code> returns up to 100 results and a <code>nextToken</code> value if applicable.<br/>nextToken:  The <code>nextToken</code> value returned from a previous paginated <code>ListUpdates</code> request where <code>maxResults</code> was used and the results exceeded the value of that parameter. Pagination continues from the end of the previous results that returned the <code>nextToken</code> value.<br/>name:  The name of the Amazon EKS cluster to list updates for.<br/>
        :return: nextToken:  The <code>nextToken</code> value to include in a future <code>ListUpdates</code> request. When the results of a <code>ListUpdates</code> request exceed <code>maxResults</code>, you can use this value to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>updateIds:  A list of all the updates for the specified cluster and Region.<br/>        """
        pass

    def update_cluster_config(self, name: str, resourcesVpcConfig: VpcConfigRequest=None, logging: Logging=None, clientRequestToken: str=None) -> UpdateClusterConfigResponse:
        """Updates an Amazon EKS cluster configuration. Your cluster continues to function during the update. The response output includes an update ID that you can use to track the status of your cluster update with the <a>DescribeUpdate</a> API operation. You can use this API operation to enable or disable exporting the Kubernetes control plane logs for your cluster to CloudWatch Logs. By default, cluster control plane logs aren't exported to CloudWatch Logs. For more information, see <a href="https://docs.aws.amazon.com/eks/latest/userguide/control-plane-logs.html">Amazon EKS Cluster Control Plane Logs</a> in the <i> <i>Amazon EKS User Guide</i> </i>. <note> CloudWatch Logs ingestion, archive storage, and data scanning rates apply to exported control plane logs. For more information, see <a href="http://aws.amazon.com/cloudwatch/pricing/">Amazon CloudWatch Pricing</a>. </note> You can also use this API operation to enable or disable public and private access to your cluster's Kubernetes API server endpoint. By default, public access is enabled, and private access is disabled. For more information, see <a href="https://docs.aws.amazon.com/eks/latest/userguide/cluster-endpoint.html">Amazon EKS Cluster Endpoint Access Control</a> in the <i> <i>Amazon EKS User Guide</i> </i>.  <important> At this time, you can not update the subnets or security group IDs for an existing cluster. </important> Cluster updates are asynchronous, and they should finish within a few minutes. During an update, the cluster status moves to <code>UPDATING</code> (this status transition is eventually consistent). When the update is complete (either <code>Failed</code> or <code>Successful</code>), the cluster status moves to <code>Active</code>.

        :param self:
        :param clientRequestToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request.<br/>logging:  Enable or disable exporting the Kubernetes control plane logs for your cluster to CloudWatch Logs. By default, cluster control plane logs aren't exported to CloudWatch Logs. For more information, see <a href="https://docs.aws.amazon.com/eks/latest/userguide/control-plane-logs.html">Amazon EKS Cluster Control Plane Logs</a> in the <i> <i>Amazon EKS User Guide</i> </i>. <note> CloudWatch Logs ingestion, archive storage, and data scanning rates apply to exported control plane logs. For more information, see <a href="http://aws.amazon.com/cloudwatch/pricing/">Amazon CloudWatch Pricing</a>. </note>:<br/>resourcesVpcConfig:  An object representing the VPC configuration to use for an Amazon EKS cluster.:<br/>name:  The name of the Amazon EKS cluster to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>endpointPrivateAccess</i> Set this value to <code>true</code> to enable private access for your cluster's Kubernetes API server endpoint. If you enable private access, Kubernetes API requests from within your cluster's VPC use the private VPC endpoint. The default value for this parameter is <code>false</code>, which disables private access for your Kubernetes API server. For more information, see <a href="https://docs.aws.amazon.com/eks/latest/userguide/cluster-endpoint.html">Amazon EKS Cluster Endpoint Access Control</a> in the <i> <i>Amazon EKS User Guide</i> </i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>endpointPublicAccess</i> Set this value to <code>false</code> to disable public access for your cluster's Kubernetes API server endpoint. If you disable public access, your cluster's Kubernetes API server can receive only requests from within the cluster VPC. The default value for this parameter is <code>true</code>, which enables public access for your Kubernetes API server. For more information, see <a href="https://docs.aws.amazon.com/eks/latest/userguide/cluster-endpoint.html">Amazon EKS Cluster Endpoint Access Control</a> in the <i> <i>Amazon EKS User Guide</i> </i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>securityGroupIds</i> Specify one or more security groups for the cross-account elastic network interfaces that Amazon EKS creates to use to allow communication between your worker nodes and the Kubernetes control plane. If you don't specify a security group, the default security group for your VPC is used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>subnetIds</i> Specify subnets for your Amazon EKS worker nodes. Amazon EKS creates cross-account elastic network interfaces in these subnets to allow communication between your worker nodes and the Kubernetes control plane.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clusterLogging</i> The cluster control plane logging configuration for your cluster.<br/>
        :return: update:  An object representing an asynchronous update.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errors</i> Any errors associated with a <code>Failed</code> update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix epoch timestamp in seconds for when the update was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>params</i> A key-value map that contains the parameters associated with the update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of the update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> A UUID that is used to track the update.<br/>        """
        pass

    def update_cluster_version(self, version: str, name: str, clientRequestToken: str=None) -> UpdateClusterVersionResponse:
        """Updates an Amazon EKS cluster to the specified Kubernetes version. Your cluster continues to function during the update. The response output includes an update ID that you can use to track the status of your cluster update with the <a>DescribeUpdate</a> API operation. Cluster updates are asynchronous, and they should finish within a few minutes. During an update, the cluster status moves to <code>UPDATING</code> (this status transition is eventually consistent). When the update is complete (either <code>Failed</code> or <code>Successful</code>), the cluster status moves to <code>Active</code>.

        :param self:
        :param clientRequestToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request.<br/>version:  The desired Kubernetes version following a successful update.<br/>name:  The name of the Amazon EKS cluster to update.<br/>
        :return: update:  The full description of the specified update:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errors</i> Any errors associated with a <code>Failed</code> update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The Unix epoch timestamp in seconds for when the update was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>params</i> A key-value map that contains the parameters associated with the update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of the update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> A UUID that is used to track the update.<br/>        """
        pass



class ElastiCache(BaseClient):
    """<fullname>Amazon ElastiCache</fullname> Amazon ElastiCache is a web service that makes it easier to set up, operate, and scale a distributed cache in the cloud. With ElastiCache, customers get all of the benefits of a high-performance, in-memory cache with less of the administrative burden involved in launching and managing a distributed cache. The service makes setup, scaling, and cluster failure handling much simpler than in a self-managed cache deployment. In addition, through integration with Amazon CloudWatch, customers get enhanced visibility into the key performance statistics associated with their cache and can receive alarms if a part of their cache runs hot."""

    class CacheClusterAvailableWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_cache_clusters` for valid parameters"""
            pass
    class CacheClusterDeletedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_cache_clusters` for valid parameters"""
            pass
    class ReplicationGroupAvailableWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_replication_groups` for valid parameters"""
            pass
    class ReplicationGroupDeletedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_replication_groups` for valid parameters"""
            pass


    class DescribeCacheClustersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_cache_clusters` for valid parameters"""
            pass
    class DescribeCacheEngineVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_cache_engine_versions` for valid parameters"""
            pass
    class DescribeCacheParameterGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_cache_parameter_groups` for valid parameters"""
            pass
    class DescribeCacheParametersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_cache_parameters` for valid parameters"""
            pass
    class DescribeCacheSecurityGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_cache_security_groups` for valid parameters"""
            pass
    class DescribeCacheSubnetGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_cache_subnet_groups` for valid parameters"""
            pass
    class DescribeEngineDefaultParametersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_engine_default_parameters` for valid parameters"""
            pass
    class DescribeEventsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_events` for valid parameters"""
            pass
    class DescribeReservedCacheNodesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_reserved_cache_nodes` for valid parameters"""
            pass
    class DescribeReservedCacheNodesOfferingsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_reserved_cache_nodes_offerings` for valid parameters"""
            pass
    class DescribeReplicationGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_replication_groups` for valid parameters"""
            pass
    class DescribeSnapshotsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_snapshots` for valid parameters"""
            pass
    class DescribeServiceUpdatesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_service_updates` for valid parameters"""
            pass
    class DescribeUpdateActionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_update_actions` for valid parameters"""
            pass


    class AddTagsToResourceMessage(Mapping):
        pass
    
    class TagListMessage(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class AuthorizeCacheSecurityGroupIngressMessage(Mapping):
        pass
    
    class AuthorizeCacheSecurityGroupIngressResult(Mapping):
        pass
    
    class BatchApplyUpdateActionMessage(Mapping):
        pass
    
    class UpdateActionResultsMessage(Mapping):
        pass
    
    class String(object):
        pass
    
    class BatchStopUpdateActionMessage(Mapping):
        pass
    
    class CopySnapshotMessage(Mapping):
        pass
    
    class CopySnapshotResult(Mapping):
        pass
    
    class CreateCacheClusterMessage(Mapping):
        pass
    
    class CreateCacheClusterResult(Mapping):
        pass
    
    class BooleanOptional(object):
        pass
    
    class CreateCacheParameterGroupMessage(Mapping):
        pass
    
    class CreateCacheParameterGroupResult(Mapping):
        pass
    
    class CreateCacheSecurityGroupMessage(Mapping):
        pass
    
    class CreateCacheSecurityGroupResult(Mapping):
        pass
    
    class CreateCacheSubnetGroupMessage(Mapping):
        pass
    
    class CreateCacheSubnetGroupResult(Mapping):
        pass
    
    class CreateReplicationGroupMessage(Mapping):
        pass
    
    class CreateReplicationGroupResult(Mapping):
        pass
    
    class NodeGroupConfiguration(Mapping):
        pass
    
    class CreateSnapshotMessage(Mapping):
        pass
    
    class CreateSnapshotResult(Mapping):
        pass
    
    class DecreaseReplicaCountMessage(Mapping):
        pass
    
    class DecreaseReplicaCountResult(Mapping):
        pass
    
    class ConfigureShard(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class DeleteCacheClusterMessage(Mapping):
        pass
    
    class DeleteCacheClusterResult(Mapping):
        pass
    
    class DeleteCacheParameterGroupMessage(Mapping):
        pass
    
    class DeleteCacheSecurityGroupMessage(Mapping):
        pass
    
    class DeleteCacheSubnetGroupMessage(Mapping):
        pass
    
    class DeleteReplicationGroupMessage(Mapping):
        pass
    
    class DeleteReplicationGroupResult(Mapping):
        pass
    
    class DeleteSnapshotMessage(Mapping):
        pass
    
    class DeleteSnapshotResult(Mapping):
        pass
    
    class DescribeCacheClustersMessage(Mapping):
        pass
    
    class CacheClusterMessage(Mapping):
        pass
    
    class DescribeCacheEngineVersionsMessage(Mapping):
        pass
    
    class CacheEngineVersionMessage(Mapping):
        pass
    
    class DescribeCacheParameterGroupsMessage(Mapping):
        pass
    
    class CacheParameterGroupsMessage(Mapping):
        pass
    
    class DescribeCacheParametersMessage(Mapping):
        pass
    
    class CacheParameterGroupDetails(Mapping):
        pass
    
    class DescribeCacheSecurityGroupsMessage(Mapping):
        pass
    
    class CacheSecurityGroupMessage(Mapping):
        pass
    
    class DescribeCacheSubnetGroupsMessage(Mapping):
        pass
    
    class CacheSubnetGroupMessage(Mapping):
        pass
    
    class DescribeEngineDefaultParametersMessage(Mapping):
        pass
    
    class DescribeEngineDefaultParametersResult(Mapping):
        pass
    
    class DescribeEventsMessage(Mapping):
        pass
    
    class EventsMessage(Mapping):
        pass
    
    class TStamp(object):
        pass
    
    class DescribeReplicationGroupsMessage(Mapping):
        pass
    
    class ReplicationGroupMessage(Mapping):
        pass
    
    class DescribeReservedCacheNodesMessage(Mapping):
        pass
    
    class ReservedCacheNodeMessage(Mapping):
        pass
    
    class DescribeReservedCacheNodesOfferingsMessage(Mapping):
        pass
    
    class ReservedCacheNodesOfferingMessage(Mapping):
        pass
    
    class DescribeServiceUpdatesMessage(Mapping):
        pass
    
    class ServiceUpdatesMessage(Mapping):
        pass
    
    class ServiceUpdateStatus(object):
        pass
    
    class DescribeSnapshotsMessage(Mapping):
        pass
    
    class DescribeSnapshotsListMessage(Mapping):
        pass
    
    class DescribeUpdateActionsMessage(Mapping):
        pass
    
    class UpdateActionsMessage(Mapping):
        pass
    
    class TimeRangeFilter(Mapping):
        pass
    
    class UpdateActionStatus(object):
        pass
    
    class IncreaseReplicaCountMessage(Mapping):
        pass
    
    class IncreaseReplicaCountResult(Mapping):
        pass
    
    class ListAllowedNodeTypeModificationsMessage(Mapping):
        pass
    
    class AllowedNodeTypeModificationsMessage(Mapping):
        pass
    
    class ListTagsForResourceMessage(Mapping):
        pass
    
    class ModifyCacheClusterMessage(Mapping):
        pass
    
    class ModifyCacheClusterResult(Mapping):
        pass
    
    class ModifyCacheParameterGroupMessage(Mapping):
        pass
    
    class CacheParameterGroupNameMessage(Mapping):
        pass
    
    class ParameterNameValue(Mapping):
        pass
    
    class ModifyCacheSubnetGroupMessage(Mapping):
        pass
    
    class ModifyCacheSubnetGroupResult(Mapping):
        pass
    
    class ModifyReplicationGroupMessage(Mapping):
        pass
    
    class ModifyReplicationGroupResult(Mapping):
        pass
    
    class ModifyReplicationGroupShardConfigurationMessage(Mapping):
        pass
    
    class ModifyReplicationGroupShardConfigurationResult(Mapping):
        pass
    
    class ReshardingConfiguration(Mapping):
        pass
    
    class AllowedNodeGroupId(object):
        pass
    
    class PurchaseReservedCacheNodesOfferingMessage(Mapping):
        pass
    
    class PurchaseReservedCacheNodesOfferingResult(Mapping):
        pass
    
    class RebootCacheClusterMessage(Mapping):
        pass
    
    class RebootCacheClusterResult(Mapping):
        pass
    
    class RemoveTagsFromResourceMessage(Mapping):
        pass
    
    class ResetCacheParameterGroupMessage(Mapping):
        pass
    
    class RevokeCacheSecurityGroupIngressMessage(Mapping):
        pass
    
    class RevokeCacheSecurityGroupIngressResult(Mapping):
        pass
    
    class TestFailoverMessage(Mapping):
        pass
    
    class TestFailoverResult(Mapping):
        pass
    
    def add_tags_to_resource(self, Tags: List[Tag], ResourceName: str) -> TagListMessage:
        """Adds up to 50 cost allocation tags to the named resource. A cost allocation tag is a key-value pair where the key and value are case-sensitive. You can use cost allocation tags to categorize and track your AWS costs.  When you apply tags to your ElastiCache resources, AWS generates a cost allocation report as a comma-separated value (CSV) file with your usage and costs aggregated by your tags. You can apply tags that represent business categories (such as cost centers, application names, or owners) to organize your costs across multiple services. For more information, see <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/red-ug/Tagging.html">Using Cost Allocation Tags in Amazon ElastiCache</a> in the <i>ElastiCache User Guide</i>.

        :param self:
        :param Tags:  A list of cost allocation tags to be added to this resource. A tag is a key-value pair. A tag key must be accompanied by a tag value.<br/>ResourceName:  The Amazon Resource Name (ARN) of the resource to which the tags are to be added, for example <code>arn:aws:elasticache:us-west-2:0123456789:cluster:myCluster</code> or <code>arn:aws:elasticache:us-west-2:0123456789:snapshot:mySnapshot</code>. ElastiCache resources are <i>cluster</i> and <i>snapshot</i>. For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>
        :return: TagList:  A list of cost allocation tags as key-value pairs.<br/>        """
        pass

    def authorize_cache_security_group_ingress(self, EC2SecurityGroupOwnerId: str, EC2SecurityGroupName: str, CacheSecurityGroupName: str) -> AuthorizeCacheSecurityGroupIngressResult:
        """Allows network ingress to a cache security group. Applications using ElastiCache must be running on Amazon EC2, and Amazon EC2 security groups are used as the authorization mechanism. <note> You cannot authorize ingress from an Amazon EC2 security group in one region to an ElastiCache cluster in another region. </note>

        :param self:
        :param EC2SecurityGroupOwnerId:  The AWS account number of the Amazon EC2 security group owner. Note that this is not the same thing as an AWS access key ID - you must provide a valid AWS account number for this parameter.<br/>EC2SecurityGroupName:  The Amazon EC2 security group to be authorized for ingress to the cache security group.<br/>CacheSecurityGroupName:  The cache security group that allows network ingress.<br/>
        :return: CacheSecurityGroup:  Represents the output of one of the following operations: <ul> <li>  <code>AuthorizeCacheSecurityGroupIngress</code>  </li> <li>  <code>CreateCacheSecurityGroup</code>  </li> <li>  <code>RevokeCacheSecurityGroupIngress</code>  </li> </ul>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EC2SecurityGroups</i> A list of Amazon EC2 security groups that are associated with this cache security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the cache security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheSecurityGroupName</i> The name of the cache security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The AWS account ID of the cache security group owner.<br/>        """
        pass

    def batch_apply_update_action(self, ServiceUpdateName: str, ReplicationGroupIds: List[String]) -> UpdateActionResultsMessage:
        """Apply the service update. For more information on service updates and applying them, see <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/red-ug/applying-updates.html">Applying Service Updates</a>.

        :param self:
        :param ServiceUpdateName:  The unique ID of the service update<br/>ReplicationGroupIds:  The replication group IDs<br/>
        :return: UnprocessedUpdateActions:  Update actions that haven't been processed successfully<br/>ProcessedUpdateActions:  Update actions that have been processed successfully<br/>        """
        pass

    def batch_stop_update_action(self, ServiceUpdateName: str, ReplicationGroupIds: List[String]) -> UpdateActionResultsMessage:
        """Stop the service update. For more information on service updates and stopping them, see <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/red-ug/stopping-self-service-updates.html">Stopping Service Updates</a>.

        :param self:
        :param ServiceUpdateName:  The unique ID of the service update<br/>ReplicationGroupIds:  The replication group IDs<br/>
        :return: UnprocessedUpdateActions:  Update actions that haven't been processed successfully<br/>ProcessedUpdateActions:  Update actions that have been processed successfully<br/>        """
        pass

    def copy_snapshot(self, TargetSnapshotName: str, SourceSnapshotName: str, TargetBucket: str=None) -> CopySnapshotResult:
        """Makes a copy of an existing snapshot. <note> This operation is valid for Redis only. </note> <important> Users or groups that have permissions to use the <code>CopySnapshot</code> operation can create their own Amazon S3 buckets and copy snapshots to it. To control access to your snapshots, use an IAM policy to control who has the ability to use the <code>CopySnapshot</code> operation. For more information about using IAM to control the use of ElastiCache operations, see <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/red-ug/backups-exporting.html">Exporting Snapshots</a> and <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/red-ug/IAM.html">Authentication &amp; Access Control</a>. </important> You could receive the following error messages. <p class="title"> <b>Error Messages</b>  <ul> <li>  <b>Error Message:</b> The S3 bucket %s is outside of the region.  <b>Solution:</b> Create an Amazon S3 bucket in the same region as your snapshot. For more information, see <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/red-ug/backups-exporting.html#backups-exporting-create-s3-bucket">Step 1: Create an Amazon S3 Bucket</a> in the ElastiCache User Guide. </li> <li>  <b>Error Message:</b> The S3 bucket %s does not exist.  <b>Solution:</b> Create an Amazon S3 bucket in the same region as your snapshot. For more information, see <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/red-ug/backups-exporting.html#backups-exporting-create-s3-bucket">Step 1: Create an Amazon S3 Bucket</a> in the ElastiCache User Guide. </li> <li>  <b>Error Message:</b> The S3 bucket %s is not owned by the authenticated user.  <b>Solution:</b> Create an Amazon S3 bucket in the same region as your snapshot. For more information, see <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/red-ug/backups-exporting.html#backups-exporting-create-s3-bucket">Step 1: Create an Amazon S3 Bucket</a> in the ElastiCache User Guide. </li> <li>  <b>Error Message:</b> The authenticated user does not have sufficient permissions to perform the desired activity.  <b>Solution:</b> Contact your system administrator to get the needed permissions. </li> <li>  <b>Error Message:</b> The S3 bucket %s already contains an object with key %s.  <b>Solution:</b> Give the <code>TargetSnapshotName</code> a new and unique value. If exporting a snapshot, you could alternatively create a new Amazon S3 bucket and use this same value for <code>TargetSnapshotName</code>. </li> <li>  <b>Error Message: </b> ElastiCache has not been granted READ permissions %s on the S3 Bucket.  <b>Solution:</b> Add List and Read permissions on the bucket. For more information, see <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/red-ug/backups-exporting.html#backups-exporting-grant-access">Step 2: Grant ElastiCache Access to Your Amazon S3 Bucket</a> in the ElastiCache User Guide. </li> <li>  <b>Error Message: </b> ElastiCache has not been granted WRITE permissions %s on the S3 Bucket.  <b>Solution:</b> Add Upload/Delete permissions on the bucket. For more information, see <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/red-ug/backups-exporting.html#backups-exporting-grant-access">Step 2: Grant ElastiCache Access to Your Amazon S3 Bucket</a> in the ElastiCache User Guide. </li> <li>  <b>Error Message: </b> ElastiCache has not been granted READ_ACP permissions %s on the S3 Bucket.  <b>Solution:</b> Add View Permissions on the bucket. For more information, see <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/red-ug/backups-exporting.html#backups-exporting-grant-access">Step 2: Grant ElastiCache Access to Your Amazon S3 Bucket</a> in the ElastiCache User Guide. </li> </ul>

        :param self:
        :param TargetBucket:  The Amazon S3 bucket to which the snapshot is exported. This parameter is used only when exporting a snapshot for external access. When using this parameter to export a snapshot, be sure Amazon ElastiCache has the needed permissions to this S3 bucket. For more information, see <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/red-ug/backups-exporting.html#backups-exporting-grant-access">Step 2: Grant ElastiCache Access to Your Amazon S3 Bucket</a> in the <i>Amazon ElastiCache User Guide</i>. For more information, see <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/red-ug/Snapshots.Exporting.html">Exporting a Snapshot</a> in the <i>Amazon ElastiCache User Guide</i>.<br/>TargetSnapshotName:  A name for the snapshot copy. ElastiCache does not permit overwriting a snapshot, therefore this name must be unique within its context - ElastiCache or an Amazon S3 bucket if exporting.<br/>SourceSnapshotName:  The name of an existing snapshot from which to make a copy.<br/>
        :return: Snapshot:  Represents a copy of an entire Redis cluster as of the time when the snapshot was taken.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeSnapshots</i> A list of the cache nodes in the source cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomaticFailover</i> Indicates the status of Multi-AZ with automatic failover for the source Redis replication group. Amazon ElastiCache for Redis does not support Multi-AZ with automatic failover on: <ul> <li> Redis versions earlier than 2.8.6. </li> <li> Redis (cluster mode disabled): T1 node types. </li> <li> Redis (cluster mode enabled): T1 node types. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumNodeGroups</i> The number of node groups (shards) in this snapshot. When restoring from a snapshot, the number of node groups (shards) in the snapshot and in the restored replication group must be the same.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotWindow</i> The daily time range during which ElastiCache takes daily snapshots of the source cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotRetentionLimit</i> For an automatic snapshot, the number of days for which ElastiCache retains the snapshot before deleting it. For manual snapshots, this field reflects the <code>SnapshotRetentionLimit</code> for the source cluster when the snapshot was created. This field is otherwise ignored: Manual snapshots do not expire, and can only be deleted using the <code>DeleteSnapshot</code> operation.   <b>Important</b> If the value of SnapshotRetentionLimit is set to zero (0), backups are turned off.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> This parameter is currently disabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The Amazon Virtual Private Cloud identifier (VPC ID) of the cache subnet group for the source cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheSubnetGroupName</i> The name of the cache subnet group associated with the source cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheParameterGroupName</i> The cache parameter group that is associated with the source cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port number used by each cache nodes in the source cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicArn</i> The Amazon Resource Name (ARN) for the topic used by the source cluster for publishing notifications.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which maintenance on the cluster is performed. It is specified as a range in the format ddd:hh24:mi-ddd:hh24:mi (24H Clock UTC). The minimum maintenance window is a 60 minute period. Valid values for <code>ddd</code> are: <ul> <li>  <code>sun</code>  </li> <li>  <code>mon</code>  </li> <li>  <code>tue</code>  </li> <li>  <code>wed</code>  </li> <li>  <code>thu</code>  </li> <li>  <code>fri</code>  </li> <li>  <code>sat</code>  </li> </ul> Example: <code>sun:23:00-mon:01:30</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheClusterCreateTime</i> The date and time when the source cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredAvailabilityZone</i> The name of the Availability Zone in which the source cluster is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumCacheNodes</i> The number of cache nodes in the source cluster. For clusters running Redis, this value must be 1. For clusters running Memcached, this value must be between 1 and 20.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> The version of the cache engine version that is used by the source cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> The name of the cache engine (<code>memcached</code> or <code>redis</code>) used by the source cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodeType</i> The name of the compute and memory capacity node type for the source cluster. The following node types are supported by ElastiCache. Generally speaking, the current generation types provide more memory and computational power at lower cost when compared to their equivalent previous generation counterparts. <ul> <li> General purpose: <ul> <li> Current generation:   <b>M5 node types:</b> <code>cache.m5.large</code>, <code>cache.m5.xlarge</code>, <code>cache.m5.2xlarge</code>, <code>cache.m5.4xlarge</code>, <code>cache.m5.12xlarge</code>, <code>cache.m5.24xlarge</code>   <b>M4 node types:</b> <code>cache.m4.large</code>, <code>cache.m4.xlarge</code>, <code>cache.m4.2xlarge</code>, <code>cache.m4.4xlarge</code>, <code>cache.m4.10xlarge</code>   <b>T2 node types:</b> <code>cache.t2.micro</code>, <code>cache.t2.small</code>, <code>cache.t2.medium</code>  </li> <li> Previous generation: (not recommended)  <b>T1 node types:</b> <code>cache.t1.micro</code>   <b>M1 node types:</b> <code>cache.m1.small</code>, <code>cache.m1.medium</code>, <code>cache.m1.large</code>, <code>cache.m1.xlarge</code>   <b>M3 node types:</b> <code>cache.m3.medium</code>, <code>cache.m3.large</code>, <code>cache.m3.xlarge</code>, <code>cache.m3.2xlarge</code>  </li> </ul> </li> <li> Compute optimized: <ul> <li> Previous generation: (not recommended)  <b>C1 node types:</b> <code>cache.c1.xlarge</code>  </li> </ul> </li> <li> Memory optimized: <ul> <li> Current generation:   <b>R5 node types:</b> <code>cache.r5.large</code>, <code>cache.r5.xlarge</code>, <code>cache.r5.2xlarge</code>, <code>cache.r5.4xlarge</code>, <code>cache.r5.12xlarge</code>, <code>cache.r5.24xlarge</code>   <b>R4 node types:</b> <code>cache.r4.large</code>, <code>cache.r4.xlarge</code>, <code>cache.r4.2xlarge</code>, <code>cache.r4.4xlarge</code>, <code>cache.r4.8xlarge</code>, <code>cache.r4.16xlarge</code>  </li> <li> Previous generation: (not recommended)  <b>M2 node types:</b> <code>cache.m2.xlarge</code>, <code>cache.m2.2xlarge</code>, <code>cache.m2.4xlarge</code>   <b>R3 node types:</b> <code>cache.r3.large</code>, <code>cache.r3.xlarge</code>, <code>cache.r3.2xlarge</code>, <code>cache.r3.4xlarge</code>, <code>cache.r3.8xlarge</code>  </li> </ul> </li> </ul>  <b>Additional node type info</b>  <ul> <li> All current generation instance types are created in Amazon VPC by default. </li> <li> Redis append-only files (AOF) are not supported for T1 or T2 instances. </li> <li> Redis Multi-AZ with automatic failover is not supported on T1 instances. </li> <li> Redis configuration variables <code>appendonly</code> and <code>appendfsync</code> are not supported on Redis version 2.8.22 and later. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotSource</i> Indicates whether the snapshot is from an automatic backup (<code>automated</code>) or was created manually (<code>manual</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotStatus</i> The status of the snapshot. Valid values: <code>creating</code> | <code>available</code> | <code>restoring</code> | <code>copying</code> | <code>deleting</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheClusterId</i> The user-supplied identifier of the source cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationGroupDescription</i> A description of the source replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationGroupId</i> The unique identifier of the source replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotName</i> The name of a snapshot. For an automatic snapshot, the name is system-generated. For a manual snapshot, this is the user-provided name.<br/>        """
        pass

    def create_cache_cluster(self, CacheClusterId: str, ReplicationGroupId: str=None, AZMode: str=None, PreferredAvailabilityZone: str=None, PreferredAvailabilityZones: List[String]=None, NumCacheNodes: int=None, CacheNodeType: str=None, Engine: str=None, EngineVersion: str=None, CacheParameterGroupName: str=None, CacheSubnetGroupName: str=None, CacheSecurityGroupNames: List[String]=None, SecurityGroupIds: List[String]=None, Tags: List[Tag]=None, SnapshotArns: List[String]=None, SnapshotName: str=None, PreferredMaintenanceWindow: str=None, Port: int=None, NotificationTopicArn: str=None, AutoMinorVersionUpgrade: BooleanOptional=None, SnapshotRetentionLimit: int=None, SnapshotWindow: str=None, AuthToken: str=None) -> CreateCacheClusterResult:
        """Creates a cluster. All nodes in the cluster run the same protocol-compliant cache engine software, either Memcached or Redis. This operation is not supported for Redis (cluster mode enabled) clusters.

        :param self:
        :param AuthToken:   <b>Reserved parameter.</b> The password used to access a password protected server. Password constraints: <ul> <li> Must be only printable ASCII characters. </li> <li> Must be at least 16 characters and no more than 128 characters in length. </li> <li> Cannot contain any of the following characters: '/', '"', or '@'.  </li> </ul> For more information, see <a href="http://redis.io/commands/AUTH">AUTH password</a> at http://redis.io/commands/AUTH.<br/>SnapshotWindow:  The daily time range (in UTC) during which ElastiCache begins taking a daily snapshot of your node group (shard). Example: <code>05:00-09:00</code>  If you do not specify this parameter, ElastiCache automatically chooses an appropriate time range. <note> This parameter is only valid if the <code>Engine</code> parameter is <code>redis</code>. </note><br/>SnapshotRetentionLimit:  The number of days for which ElastiCache retains automatic snapshots before deleting them. For example, if you set <code>SnapshotRetentionLimit</code> to 5, a snapshot taken today is retained for 5 days before being deleted. <note> This parameter is only valid if the <code>Engine</code> parameter is <code>redis</code>. </note> Default: 0 (i.e., automatic backups are disabled for this cache cluster).<br/>AutoMinorVersionUpgrade:  This parameter is currently disabled.<br/>NotificationTopicArn:  The Amazon Resource Name (ARN) of the Amazon Simple Notification Service (SNS) topic to which notifications are sent. <note> The Amazon SNS topic owner must be the same as the cluster owner. </note><br/>Port:  The port number on which each of the cache nodes accepts connections.<br/>PreferredMaintenanceWindow:  Specifies the weekly time range during which maintenance on the cluster is performed. It is specified as a range in the format ddd:hh24:mi-ddd:hh24:mi (24H Clock UTC). The minimum maintenance window is a 60 minute period. Valid values for <code>ddd</code> are: Specifies the weekly time range during which maintenance on the cluster is performed. It is specified as a range in the format ddd:hh24:mi-ddd:hh24:mi (24H Clock UTC). The minimum maintenance window is a 60 minute period. Valid values for <code>ddd</code> are: <ul> <li>  <code>sun</code>  </li> <li>  <code>mon</code>  </li> <li>  <code>tue</code>  </li> <li>  <code>wed</code>  </li> <li>  <code>thu</code>  </li> <li>  <code>fri</code>  </li> <li>  <code>sat</code>  </li> </ul> Example: <code>sun:23:00-mon:01:30</code> <br/>SnapshotName:  The name of a Redis snapshot from which to restore data into the new node group (shard). The snapshot status changes to <code>restoring</code> while the new node group (shard) is being created. <note> This parameter is only valid if the <code>Engine</code> parameter is <code>redis</code>. </note><br/>SnapshotArns:  A single-element string list containing an Amazon Resource Name (ARN) that uniquely identifies a Redis RDB snapshot file stored in Amazon S3. The snapshot file is used to populate the node group (shard). The Amazon S3 object name in the ARN cannot contain any commas. <note> This parameter is only valid if the <code>Engine</code> parameter is <code>redis</code>. </note> Example of an Amazon S3 ARN: <code>arn:aws:s3:::my_bucket/snapshot1.rdb</code> <br/>Tags:  A list of cost allocation tags to be added to this resource.<br/>SecurityGroupIds:  One or more VPC security groups associated with the cluster. Use this parameter only when you are creating a cluster in an Amazon Virtual Private Cloud (Amazon VPC).<br/>CacheSecurityGroupNames:  A list of security group names to associate with this cluster. Use this parameter only when you are creating a cluster outside of an Amazon Virtual Private Cloud (Amazon VPC).<br/>CacheSubnetGroupName:  The name of the subnet group to be used for the cluster. Use this parameter only when you are creating a cluster in an Amazon Virtual Private Cloud (Amazon VPC). <important> If you're going to launch your cluster in an Amazon VPC, you need to create a subnet group before you start creating a cluster. For more information, see <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/red-ug/SubnetGroups.html">Subnets and Subnet Groups</a>. </important><br/>CacheParameterGroupName:  The name of the parameter group to associate with this cluster. If this argument is omitted, the default parameter group for the specified engine is used. You cannot use any parameter group which has <code>cluster-enabled='yes'</code> when creating a cluster.<br/>EngineVersion:  The version number of the cache engine to be used for this cluster. To view the supported cache engine versions, use the DescribeCacheEngineVersions operation.  <b>Important:</b> You can upgrade to a newer engine version (see <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/red-ug/SelectEngine.html#VersionManagement">Selecting a Cache Engine and Version</a>), but you cannot downgrade to an earlier engine version. If you want to use an earlier engine version, you must delete the existing cluster or replication group and create it anew with the earlier engine version. <br/>Engine:  The name of the cache engine to be used for this cluster. Valid values for this parameter are: <code>memcached</code> | <code>redis</code> <br/>CacheNodeType:  The compute and memory capacity of the nodes in the node group (shard). The following node types are supported by ElastiCache. Generally speaking, the current generation types provide more memory and computational power at lower cost when compared to their equivalent previous generation counterparts. <ul> <li> General purpose: <ul> <li> Current generation:   <b>M5 node types:</b> <code>cache.m5.large</code>, <code>cache.m5.xlarge</code>, <code>cache.m5.2xlarge</code>, <code>cache.m5.4xlarge</code>, <code>cache.m5.12xlarge</code>, <code>cache.m5.24xlarge</code>   <b>M4 node types:</b> <code>cache.m4.large</code>, <code>cache.m4.xlarge</code>, <code>cache.m4.2xlarge</code>, <code>cache.m4.4xlarge</code>, <code>cache.m4.10xlarge</code>   <b>T2 node types:</b> <code>cache.t2.micro</code>, <code>cache.t2.small</code>, <code>cache.t2.medium</code>  </li> <li> Previous generation: (not recommended)  <b>T1 node types:</b> <code>cache.t1.micro</code>   <b>M1 node types:</b> <code>cache.m1.small</code>, <code>cache.m1.medium</code>, <code>cache.m1.large</code>, <code>cache.m1.xlarge</code>   <b>M3 node types:</b> <code>cache.m3.medium</code>, <code>cache.m3.large</code>, <code>cache.m3.xlarge</code>, <code>cache.m3.2xlarge</code>  </li> </ul> </li> <li> Compute optimized: <ul> <li> Previous generation: (not recommended)  <b>C1 node types:</b> <code>cache.c1.xlarge</code>  </li> </ul> </li> <li> Memory optimized: <ul> <li> Current generation:   <b>R5 node types:</b> <code>cache.r5.large</code>, <code>cache.r5.xlarge</code>, <code>cache.r5.2xlarge</code>, <code>cache.r5.4xlarge</code>, <code>cache.r5.12xlarge</code>, <code>cache.r5.24xlarge</code>   <b>R4 node types:</b> <code>cache.r4.large</code>, <code>cache.r4.xlarge</code>, <code>cache.r4.2xlarge</code>, <code>cache.r4.4xlarge</code>, <code>cache.r4.8xlarge</code>, <code>cache.r4.16xlarge</code>  </li> <li> Previous generation: (not recommended)  <b>M2 node types:</b> <code>cache.m2.xlarge</code>, <code>cache.m2.2xlarge</code>, <code>cache.m2.4xlarge</code>   <b>R3 node types:</b> <code>cache.r3.large</code>, <code>cache.r3.xlarge</code>, <code>cache.r3.2xlarge</code>, <code>cache.r3.4xlarge</code>, <code>cache.r3.8xlarge</code>  </li> </ul> </li> </ul>  <b>Additional node type info</b>  <ul> <li> All current generation instance types are created in Amazon VPC by default. </li> <li> Redis append-only files (AOF) are not supported for T1 or T2 instances. </li> <li> Redis Multi-AZ with automatic failover is not supported on T1 instances. </li> <li> Redis configuration variables <code>appendonly</code> and <code>appendfsync</code> are not supported on Redis version 2.8.22 and later. </li> </ul><br/>NumCacheNodes:  The initial number of cache nodes that the cluster has. For clusters running Redis, this value must be 1. For clusters running Memcached, this value must be between 1 and 20. If you need more than 20 nodes for your Memcached cluster, please fill out the ElastiCache Limit Increase Request form at <a href="http://aws.amazon.com/contact-us/elasticache-node-limit-request/">http://aws.amazon.com/contact-us/elasticache-node-limit-request/</a>.<br/>PreferredAvailabilityZones:  A list of the Availability Zones in which cache nodes are created. The order of the zones in the list is not important. This option is only supported on Memcached. <note> If you are creating your cluster in an Amazon VPC (recommended) you can only locate nodes in Availability Zones that are associated with the subnets in the selected subnet group. The number of Availability Zones listed must equal the value of <code>NumCacheNodes</code>. </note> If you want all the nodes in the same Availability Zone, use <code>PreferredAvailabilityZone</code> instead, or repeat the Availability Zone multiple times in the list. Default: System chosen Availability Zones.<br/>PreferredAvailabilityZone:  The EC2 Availability Zone in which the cluster is created. All nodes belonging to this Memcached cluster are placed in the preferred Availability Zone. If you want to create your nodes across multiple Availability Zones, use <code>PreferredAvailabilityZones</code>. Default: System chosen Availability Zone.<br/>AZMode:  Specifies whether the nodes in this Memcached cluster are created in a single Availability Zone or created across multiple Availability Zones in the cluster's region. This parameter is only supported for Memcached clusters. If the <code>AZMode</code> and <code>PreferredAvailabilityZones</code> are not specified, ElastiCache assumes <code>single-az</code> mode.<br/>ReplicationGroupId:  The ID of the replication group to which this cluster should belong. If this parameter is specified, the cluster is added to the specified replication group as a read replica; otherwise, the cluster is a standalone primary that is not part of any replication group. If the specified replication group is Multi-AZ enabled and the Availability Zone is not specified, the cluster is created in Availability Zones that provide the best spread of read replicas across Availability Zones. <note> This parameter is only valid if the <code>Engine</code> parameter is <code>redis</code>. </note><br/>CacheClusterId:  The node group (shard) identifier. This parameter is stored as a lowercase string.  <b>Constraints:</b>  <ul> <li> A name must contain from 1 to 20 alphanumeric characters or hyphens. </li> <li> The first character must be a letter. </li> <li> A name cannot end with a hyphen or contain two consecutive hyphens. </li> </ul><br/>
        :return: CacheCluster:  Contains all of the attributes of a specific cluster.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AtRestEncryptionEnabled</i> A flag that enables encryption at-rest when set to <code>true</code>. You cannot modify the value of <code>AtRestEncryptionEnabled</code> after the cluster is created. To enable at-rest encryption on a cluster you must set <code>AtRestEncryptionEnabled</code> to <code>true</code> when you create a cluster.  <b>Required:</b> Only available when creating a replication group in an Amazon VPC using redis version <code>3.2.6</code>, <code>4.x</code> or later. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitEncryptionEnabled</i> A flag that enables in-transit encryption when set to <code>true</code>. You cannot modify the value of <code>TransitEncryptionEnabled</code> after the cluster is created. To enable in-transit encryption on a cluster you must set <code>TransitEncryptionEnabled</code> to <code>true</code> when you create a cluster.  <b>Required:</b> Only available when creating a replication group in an Amazon VPC using redis version <code>3.2.6</code>, <code>4.x</code> or later. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthTokenEnabled</i> A flag that enables using an <code>AuthToken</code> (password) when issuing Redis commands. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotWindow</i> The daily time range (in UTC) during which ElastiCache begins taking a daily snapshot of your cluster. Example: <code>05:00-09:00</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotRetentionLimit</i> The number of days for which ElastiCache retains automatic cluster snapshots before deleting them. For example, if you set <code>SnapshotRetentionLimit</code> to 5, a snapshot that was taken today is retained for 5 days before being deleted. <important>  If the value of SnapshotRetentionLimit is set to zero (0), backups are turned off. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationGroupId</i> The replication group to which this cluster belongs. If this field is empty, the cluster is not associated with any replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroups</i> A list of VPC Security Groups associated with the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> This parameter is currently disabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodes</i> A list of cache nodes that are members of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheSubnetGroupName</i> The name of the cache subnet group associated with the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheParameterGroup</i> Status of the cache parameter group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CacheParameterGroup</b>: Status of the cache parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheSecurityGroups</i> A list of cache security group elements, composed of name and status sub-elements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationConfiguration</i> Describes a notification topic and its status. Notification topics are used for publishing ElastiCache events to subscribers using Amazon Simple Notification Service (SNS). &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NotificationConfiguration</b>: Describes a notification topic and its status. Notification topics are used for publishing ElastiCache events to subscribers using Amazon Simple Notification Service (SNS). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> A group of settings that are applied to the cluster in the future, or that are currently being applied.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: A group of settings that are applied to the cluster in the future, or that are currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which maintenance on the cluster is performed. It is specified as a range in the format ddd:hh24:mi-ddd:hh24:mi (24H Clock UTC). The minimum maintenance window is a 60 minute period. Valid values for <code>ddd</code> are: <ul> <li>  <code>sun</code>  </li> <li>  <code>mon</code>  </li> <li>  <code>tue</code>  </li> <li>  <code>wed</code>  </li> <li>  <code>thu</code>  </li> <li>  <code>fri</code>  </li> <li>  <code>sat</code>  </li> </ul> Example: <code>sun:23:00-mon:01:30</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheClusterCreateTime</i> The date and time when the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredAvailabilityZone</i> The name of the Availability Zone in which the cluster is located or "Multiple" if the cache nodes are located in different Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumCacheNodes</i> The number of cache nodes in the cluster. For clusters running Redis, this value must be 1. For clusters running Memcached, this value must be between 1 and 20.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheClusterStatus</i> The current state of this cluster, one of the following values: <code>available</code>, <code>creating</code>, <code>deleted</code>, <code>deleting</code>, <code>incompatible-network</code>, <code>modifying</code>, <code>rebooting cluster nodes</code>, <code>restore-failed</code>, or <code>snapshotting</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> The version of the cache engine that is used in this cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> The name of the cache engine (<code>memcached</code> or <code>redis</code>) to be used for this cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodeType</i> The name of the compute and memory capacity node type for the cluster. The following node types are supported by ElastiCache. Generally speaking, the current generation types provide more memory and computational power at lower cost when compared to their equivalent previous generation counterparts. <ul> <li> General purpose: <ul> <li> Current generation:   <b>M5 node types:</b> <code>cache.m5.large</code>, <code>cache.m5.xlarge</code>, <code>cache.m5.2xlarge</code>, <code>cache.m5.4xlarge</code>, <code>cache.m5.12xlarge</code>, <code>cache.m5.24xlarge</code>   <b>M4 node types:</b> <code>cache.m4.large</code>, <code>cache.m4.xlarge</code>, <code>cache.m4.2xlarge</code>, <code>cache.m4.4xlarge</code>, <code>cache.m4.10xlarge</code>   <b>T2 node types:</b> <code>cache.t2.micro</code>, <code>cache.t2.small</code>, <code>cache.t2.medium</code>  </li> <li> Previous generation: (not recommended)  <b>T1 node types:</b> <code>cache.t1.micro</code>   <b>M1 node types:</b> <code>cache.m1.small</code>, <code>cache.m1.medium</code>, <code>cache.m1.large</code>, <code>cache.m1.xlarge</code>   <b>M3 node types:</b> <code>cache.m3.medium</code>, <code>cache.m3.large</code>, <code>cache.m3.xlarge</code>, <code>cache.m3.2xlarge</code>  </li> </ul> </li> <li> Compute optimized: <ul> <li> Previous generation: (not recommended)  <b>C1 node types:</b> <code>cache.c1.xlarge</code>  </li> </ul> </li> <li> Memory optimized: <ul> <li> Current generation:   <b>R5 node types:</b> <code>cache.r5.large</code>, <code>cache.r5.xlarge</code>, <code>cache.r5.2xlarge</code>, <code>cache.r5.4xlarge</code>, <code>cache.r5.12xlarge</code>, <code>cache.r5.24xlarge</code>   <b>R4 node types:</b> <code>cache.r4.large</code>, <code>cache.r4.xlarge</code>, <code>cache.r4.2xlarge</code>, <code>cache.r4.4xlarge</code>, <code>cache.r4.8xlarge</code>, <code>cache.r4.16xlarge</code>  </li> <li> Previous generation: (not recommended)  <b>M2 node types:</b> <code>cache.m2.xlarge</code>, <code>cache.m2.2xlarge</code>, <code>cache.m2.4xlarge</code>   <b>R3 node types:</b> <code>cache.r3.large</code>, <code>cache.r3.xlarge</code>, <code>cache.r3.2xlarge</code>, <code>cache.r3.4xlarge</code>, <code>cache.r3.8xlarge</code>  </li> </ul> </li> </ul>  <b>Additional node type info</b>  <ul> <li> All current generation instance types are created in Amazon VPC by default. </li> <li> Redis append-only files (AOF) are not supported for T1 or T2 instances. </li> <li> Redis Multi-AZ with automatic failover is not supported on T1 instances. </li> <li> Redis configuration variables <code>appendonly</code> and <code>appendfsync</code> are not supported on Redis version 2.8.22 and later. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientDownloadLandingPage</i> The URL of the web page where you can download the latest ElastiCache client library.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigurationEndpoint</i> Represents a Memcached cluster endpoint which, if Automatic Discovery is enabled on the cluster, can be used by an application to connect to any node in the cluster. The configuration endpoint will always have <code>.cfg</code> in it. Example: <code>mem-3.9dvc4r<u>.cfg</u>.usw2.cache.amazonaws.com:11211</code> &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ConfigurationEndpoint</b>: Represents a Memcached cluster endpoint which, if Automatic Discovery is enabled on the cluster, can be used by an application to connect to any node in the cluster. The configuration endpoint will always have <code>.cfg</code> in it. Example: <code>mem-3.9dvc4r<u>.cfg</u>.usw2.cache.amazonaws.com:11211</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheClusterId</i> The user-supplied identifier of the cluster. This identifier is a unique key that identifies a cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port number that the cache engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS hostname of the cache node.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodeType</i> The cache node type that this cluster or replication group is scaled to.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> The new cache engine version that the cluster runs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodeIdsToRemove</i> A list of cache node IDs that are being removed (or will be removed) from the cluster. A node ID is a 4-digit numeric identifier (0001, 0002, etc.).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumCacheNodes</i> The new number of cache nodes for the cluster. For clusters running Redis, this value must be 1. For clusters running Memcached, this value must be between 1 and 20.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicStatus</i> The current state of the topic.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicArn</i> The Amazon Resource Name (ARN) that identifies the topic.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodeIdsToReboot</i> A list of the cache node IDs which need to be rebooted for parameter changes to be applied. A node ID is a numeric identifier (0001, 0002, etc.).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterApplyStatus</i> The status of parameter updates.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheParameterGroupName</i> The name of the cache parameter group.<br/>        """
        pass

    def create_cache_parameter_group(self, Description: str, CacheParameterGroupFamily: str, CacheParameterGroupName: str) -> CreateCacheParameterGroupResult:
        """Creates a new Amazon ElastiCache cache parameter group. An ElastiCache cache parameter group is a collection of parameters and their values that are applied to all of the nodes in any cluster or replication group using the CacheParameterGroup. A newly created CacheParameterGroup is an exact duplicate of the default parameter group for the CacheParameterGroupFamily. To customize the newly created CacheParameterGroup you can change the values of specific parameters. For more information, see: <ul> <li>  <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/APIReference/API_ModifyCacheParameterGroup.html">ModifyCacheParameterGroup</a> in the ElastiCache API Reference. </li> <li>  <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/red-ug/ParameterGroups.html">Parameters and Parameter Groups</a> in the ElastiCache User Guide. </li> </ul>

        :param self:
        :param Description:  A user-specified description for the cache parameter group.<br/>CacheParameterGroupFamily:  The name of the cache parameter group family that the cache parameter group can be used with. Valid values are: <code>memcached1.4</code> | <code>memcached1.5</code> | <code>redis2.6</code> | <code>redis2.8</code> | <code>redis3.2</code> | <code>redis4.0</code> | <code>redis5.0</code> | <br/>CacheParameterGroupName:  A user-specified name for the cache parameter group.<br/>
        :return: CacheParameterGroup:  Represents the output of a <code>CreateCacheParameterGroup</code> operation.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description for this cache parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheParameterGroupFamily</i> The name of the cache parameter group family that this cache parameter group is compatible with. Valid values are: <code>memcached1.4</code> | <code>memcached1.5</code> | <code>redis2.6</code> | <code>redis2.8</code> | <code>redis3.2</code> | <code>redis4.0</code> | <code>redis5.0</code> | <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheParameterGroupName</i> The name of the cache parameter group.<br/>        """
        pass

    def create_cache_security_group(self, Description: str, CacheSecurityGroupName: str) -> CreateCacheSecurityGroupResult:
        """Creates a new cache security group. Use a cache security group to control access to one or more clusters. Cache security groups are only used when you are creating a cluster outside of an Amazon Virtual Private Cloud (Amazon VPC). If you are creating a cluster inside of a VPC, use a cache subnet group instead. For more information, see <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/APIReference/API_CreateCacheSubnetGroup.html">CreateCacheSubnetGroup</a>.

        :param self:
        :param Description:  A description for the cache security group.<br/>CacheSecurityGroupName:  A name for the cache security group. This value is stored as a lowercase string. Constraints: Must contain no more than 255 alphanumeric characters. Cannot be the word "Default". Example: <code>mysecuritygroup</code> <br/>
        :return: CacheSecurityGroup:  Represents the output of one of the following operations: <ul> <li>  <code>AuthorizeCacheSecurityGroupIngress</code>  </li> <li>  <code>CreateCacheSecurityGroup</code>  </li> <li>  <code>RevokeCacheSecurityGroupIngress</code>  </li> </ul>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EC2SecurityGroups</i> A list of Amazon EC2 security groups that are associated with this cache security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the cache security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheSecurityGroupName</i> The name of the cache security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The AWS account ID of the cache security group owner.<br/>        """
        pass

    def create_cache_subnet_group(self, SubnetIds: List[String], CacheSubnetGroupDescription: str, CacheSubnetGroupName: str) -> CreateCacheSubnetGroupResult:
        """Creates a new cache subnet group. Use this parameter only when you are creating a cluster in an Amazon Virtual Private Cloud (Amazon VPC).

        :param self:
        :param SubnetIds:  A list of VPC subnet IDs for the cache subnet group.<br/>CacheSubnetGroupDescription:  A description for the cache subnet group.<br/>CacheSubnetGroupName:  A name for the cache subnet group. This value is stored as a lowercase string. Constraints: Must contain no more than 255 alphanumeric characters or hyphens. Example: <code>mysubnetgroup</code> <br/>
        :return: CacheSubnetGroup:  Represents the output of one of the following operations: <ul> <li>  <code>CreateCacheSubnetGroup</code>  </li> <li>  <code>ModifyCacheSubnetGroup</code>  </li> </ul>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> A list of subnets associated with the cache subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The Amazon Virtual Private Cloud identifier (VPC ID) of the cache subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheSubnetGroupDescription</i> The description of the cache subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheSubnetGroupName</i> The name of the cache subnet group.<br/>        """
        pass

    def create_replication_group(self, ReplicationGroupDescription: str, ReplicationGroupId: str, PrimaryClusterId: str=None, AutomaticFailoverEnabled: BooleanOptional=None, NumCacheClusters: int=None, PreferredCacheClusterAZs: List[String]=None, NumNodeGroups: int=None, ReplicasPerNodeGroup: int=None, NodeGroupConfiguration: List[NodeGroupConfiguration]=None, CacheNodeType: str=None, Engine: str=None, EngineVersion: str=None, CacheParameterGroupName: str=None, CacheSubnetGroupName: str=None, CacheSecurityGroupNames: List[String]=None, SecurityGroupIds: List[String]=None, Tags: List[Tag]=None, SnapshotArns: List[String]=None, SnapshotName: str=None, PreferredMaintenanceWindow: str=None, Port: int=None, NotificationTopicArn: str=None, AutoMinorVersionUpgrade: BooleanOptional=None, SnapshotRetentionLimit: int=None, SnapshotWindow: str=None, AuthToken: str=None, TransitEncryptionEnabled: BooleanOptional=None, AtRestEncryptionEnabled: BooleanOptional=None) -> CreateReplicationGroupResult:
        """Creates a Redis (cluster mode disabled) or a Redis (cluster mode enabled) replication group. A Redis (cluster mode disabled) replication group is a collection of clusters, where one of the clusters is a read/write primary and the others are read-only replicas. Writes to the primary are asynchronously propagated to the replicas. A Redis (cluster mode enabled) replication group is a collection of 1 to 90 node groups (shards). Each node group (shard) has one read/write primary node and up to 5 read-only replica nodes. Writes to the primary are asynchronously propagated to the replicas. Redis (cluster mode enabled) replication groups partition the data across node groups (shards). When a Redis (cluster mode disabled) replication group has been successfully created, you can add one or more read replicas to it, up to a total of 5 read replicas. You cannot alter a Redis (cluster mode enabled) replication group after it has been created. However, if you need to increase or decrease the number of node groups (console: shards), you can avail yourself of ElastiCache for Redis' enhanced backup and restore. For more information, see <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/red-ug/backups-restoring.html">Restoring From a Backup with Cluster Resizing</a> in the <i>ElastiCache User Guide</i>. <note> This operation is valid for Redis only. </note>

        :param self:
        :param AtRestEncryptionEnabled:  A flag that enables encryption at rest when set to <code>true</code>. You cannot modify the value of <code>AtRestEncryptionEnabled</code> after the replication group is created. To enable encryption at rest on a replication group you must set <code>AtRestEncryptionEnabled</code> to <code>true</code> when you create the replication group.   <b>Required:</b> Only available when creating a replication group in an Amazon VPC using redis version <code>3.2.6</code>, <code>4.x</code> or later. Default: <code>false</code> <br/>TransitEncryptionEnabled:  A flag that enables in-transit encryption when set to <code>true</code>. You cannot modify the value of <code>TransitEncryptionEnabled</code> after the cluster is created. To enable in-transit encryption on a cluster you must set <code>TransitEncryptionEnabled</code> to <code>true</code> when you create a cluster. This parameter is valid only if the <code>Engine</code> parameter is <code>redis</code>, the <code>EngineVersion</code> parameter is <code>3.2.6</code> or <code>4.x</code>, and the cluster is being created in an Amazon VPC. If you enable in-transit encryption, you must also specify a value for <code>CacheSubnetGroup</code>.  <b>Required:</b> Only available when creating a replication group in an Amazon VPC using redis version <code>3.2.6</code>, <code>4.x</code> or later. Default: <code>false</code>  <important> For HIPAA compliance, you must specify <code>TransitEncryptionEnabled</code> as <code>true</code>, an <code>AuthToken</code>, and a <code>CacheSubnetGroup</code>. </important><br/>AuthToken:   <b>Reserved parameter.</b> The password used to access a password protected server.  <code>AuthToken</code> can be specified only on replication groups where <code>TransitEncryptionEnabled</code> is <code>true</code>. <important> For HIPAA compliance, you must specify <code>TransitEncryptionEnabled</code> as <code>true</code>, an <code>AuthToken</code>, and a <code>CacheSubnetGroup</code>. </important> Password constraints: <ul> <li> Must be only printable ASCII characters. </li> <li> Must be at least 16 characters and no more than 128 characters in length. </li> <li> Cannot contain any of the following characters: '/', '"', or '@'.  </li> </ul> For more information, see <a href="http://redis.io/commands/AUTH">AUTH password</a> at http://redis.io/commands/AUTH.<br/>SnapshotWindow:  The daily time range (in UTC) during which ElastiCache begins taking a daily snapshot of your node group (shard). Example: <code>05:00-09:00</code>  If you do not specify this parameter, ElastiCache automatically chooses an appropriate time range.<br/>SnapshotRetentionLimit:  The number of days for which ElastiCache retains automatic snapshots before deleting them. For example, if you set <code>SnapshotRetentionLimit</code> to 5, a snapshot that was taken today is retained for 5 days before being deleted. Default: 0 (i.e., automatic backups are disabled for this cluster).<br/>AutoMinorVersionUpgrade:  This parameter is currently disabled.<br/>NotificationTopicArn:  The Amazon Resource Name (ARN) of the Amazon Simple Notification Service (SNS) topic to which notifications are sent. <note> The Amazon SNS topic owner must be the same as the cluster owner. </note><br/>Port:  The port number on which each member of the replication group accepts connections.<br/>PreferredMaintenanceWindow:  Specifies the weekly time range during which maintenance on the cluster is performed. It is specified as a range in the format ddd:hh24:mi-ddd:hh24:mi (24H Clock UTC). The minimum maintenance window is a 60 minute period. Valid values for <code>ddd</code> are: Specifies the weekly time range during which maintenance on the cluster is performed. It is specified as a range in the format ddd:hh24:mi-ddd:hh24:mi (24H Clock UTC). The minimum maintenance window is a 60 minute period. Valid values for <code>ddd</code> are: <ul> <li>  <code>sun</code>  </li> <li>  <code>mon</code>  </li> <li>  <code>tue</code>  </li> <li>  <code>wed</code>  </li> <li>  <code>thu</code>  </li> <li>  <code>fri</code>  </li> <li>  <code>sat</code>  </li> </ul> Example: <code>sun:23:00-mon:01:30</code> <br/>SnapshotName:  The name of a snapshot from which to restore data into the new replication group. The snapshot status changes to <code>restoring</code> while the new replication group is being created.<br/>SnapshotArns:  A list of Amazon Resource Names (ARN) that uniquely identify the Redis RDB snapshot files stored in Amazon S3. The snapshot files are used to populate the new replication group. The Amazon S3 object name in the ARN cannot contain any commas. The new replication group will have the number of node groups (console: shards) specified by the parameter <i>NumNodeGroups</i> or the number of node groups configured by <i>NodeGroupConfiguration</i> regardless of the number of ARNs specified here. Example of an Amazon S3 ARN: <code>arn:aws:s3:::my_bucket/snapshot1.rdb</code> <br/>Tags:  A list of cost allocation tags to be added to this resource. Tags are comma-separated key,value pairs (e.g. Key=<code>myKey</code>, Value=<code>myKeyValue</code>. You can include multiple tags as shown following: Key=<code>myKey</code>, Value=<code>myKeyValue</code> Key=<code>mySecondKey</code>, Value=<code>mySecondKeyValue</code>.<br/>SecurityGroupIds:  One or more Amazon VPC security groups associated with this replication group. Use this parameter only when you are creating a replication group in an Amazon Virtual Private Cloud (Amazon VPC).<br/>CacheSecurityGroupNames:  A list of cache security group names to associate with this replication group.<br/>CacheSubnetGroupName:  The name of the cache subnet group to be used for the replication group. <important> If you're going to launch your cluster in an Amazon VPC, you need to create a subnet group before you start creating a cluster. For more information, see <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/red-ug/SubnetGroups.html">Subnets and Subnet Groups</a>. </important><br/>CacheParameterGroupName:  The name of the parameter group to associate with this replication group. If this argument is omitted, the default cache parameter group for the specified engine is used. <note> If you are restoring to an engine version that is different than the original, you must specify the default version of that version. For example, <code>CacheParameterGroupName=default.redis4.0</code>. </note> If you are running Redis version 3.2.4 or later, only one node group (shard), and want to use a default parameter group, we recommend that you specify the parameter group by name.  <ul> <li> To create a Redis (cluster mode disabled) replication group, use <code>CacheParameterGroupName=default.redis3.2</code>. </li> <li> To create a Redis (cluster mode enabled) replication group, use <code>CacheParameterGroupName=default.redis3.2.cluster.on</code>. </li> </ul><br/>EngineVersion:  The version number of the cache engine to be used for the clusters in this replication group. To view the supported cache engine versions, use the <code>DescribeCacheEngineVersions</code> operation.  <b>Important:</b> You can upgrade to a newer engine version (see <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/red-ug/SelectEngine.html#VersionManagement">Selecting a Cache Engine and Version</a>) in the <i>ElastiCache User Guide</i>, but you cannot downgrade to an earlier engine version. If you want to use an earlier engine version, you must delete the existing cluster or replication group and create it anew with the earlier engine version. <br/>Engine:  The name of the cache engine to be used for the clusters in this replication group.<br/>CacheNodeType:  The compute and memory capacity of the nodes in the node group (shard). The following node types are supported by ElastiCache. Generally speaking, the current generation types provide more memory and computational power at lower cost when compared to their equivalent previous generation counterparts. <ul> <li> General purpose: <ul> <li> Current generation:   <b>M5 node types:</b> <code>cache.m5.large</code>, <code>cache.m5.xlarge</code>, <code>cache.m5.2xlarge</code>, <code>cache.m5.4xlarge</code>, <code>cache.m5.12xlarge</code>, <code>cache.m5.24xlarge</code>   <b>M4 node types:</b> <code>cache.m4.large</code>, <code>cache.m4.xlarge</code>, <code>cache.m4.2xlarge</code>, <code>cache.m4.4xlarge</code>, <code>cache.m4.10xlarge</code>   <b>T2 node types:</b> <code>cache.t2.micro</code>, <code>cache.t2.small</code>, <code>cache.t2.medium</code>  </li> <li> Previous generation: (not recommended)  <b>T1 node types:</b> <code>cache.t1.micro</code>   <b>M1 node types:</b> <code>cache.m1.small</code>, <code>cache.m1.medium</code>, <code>cache.m1.large</code>, <code>cache.m1.xlarge</code>   <b>M3 node types:</b> <code>cache.m3.medium</code>, <code>cache.m3.large</code>, <code>cache.m3.xlarge</code>, <code>cache.m3.2xlarge</code>  </li> </ul> </li> <li> Compute optimized: <ul> <li> Previous generation: (not recommended)  <b>C1 node types:</b> <code>cache.c1.xlarge</code>  </li> </ul> </li> <li> Memory optimized: <ul> <li> Current generation:   <b>R5 node types:</b> <code>cache.r5.large</code>, <code>cache.r5.xlarge</code>, <code>cache.r5.2xlarge</code>, <code>cache.r5.4xlarge</code>, <code>cache.r5.12xlarge</code>, <code>cache.r5.24xlarge</code>   <b>R4 node types:</b> <code>cache.r4.large</code>, <code>cache.r4.xlarge</code>, <code>cache.r4.2xlarge</code>, <code>cache.r4.4xlarge</code>, <code>cache.r4.8xlarge</code>, <code>cache.r4.16xlarge</code>  </li> <li> Previous generation: (not recommended)  <b>M2 node types:</b> <code>cache.m2.xlarge</code>, <code>cache.m2.2xlarge</code>, <code>cache.m2.4xlarge</code>   <b>R3 node types:</b> <code>cache.r3.large</code>, <code>cache.r3.xlarge</code>, <code>cache.r3.2xlarge</code>, <code>cache.r3.4xlarge</code>, <code>cache.r3.8xlarge</code>  </li> </ul> </li> </ul>  <b>Additional node type info</b>  <ul> <li> All current generation instance types are created in Amazon VPC by default. </li> <li> Redis append-only files (AOF) are not supported for T1 or T2 instances. </li> <li> Redis Multi-AZ with automatic failover is not supported on T1 instances. </li> <li> Redis configuration variables <code>appendonly</code> and <code>appendfsync</code> are not supported on Redis version 2.8.22 and later. </li> </ul><br/>NodeGroupConfiguration:  A list of node group (shard) configuration options. Each node group (shard) configuration has the following members: <code>PrimaryAvailabilityZone</code>, <code>ReplicaAvailabilityZones</code>, <code>ReplicaCount</code>, and <code>Slots</code>. If you're creating a Redis (cluster mode disabled) or a Redis (cluster mode enabled) replication group, you can use this parameter to individually configure each node group (shard), or you can omit this parameter. However, when seeding a Redis (cluster mode enabled) cluster from a S3 rdb file, you must configure each node group (shard) using this parameter because you must specify the slots for each node group.<br/>ReplicasPerNodeGroup:  An optional parameter that specifies the number of replica nodes in each node group (shard). Valid values are 0 to 5.<br/>NumNodeGroups:  An optional parameter that specifies the number of node groups (shards) for this Redis (cluster mode enabled) replication group. For Redis (cluster mode disabled) either omit this parameter or set it to 1. Default: 1<br/>PreferredCacheClusterAZs:  A list of EC2 Availability Zones in which the replication group's clusters are created. The order of the Availability Zones in the list is the order in which clusters are allocated. The primary cluster is created in the first AZ in the list. This parameter is not used if there is more than one node group (shard). You should use <code>NodeGroupConfiguration</code> instead. <note> If you are creating your replication group in an Amazon VPC (recommended), you can only locate clusters in Availability Zones associated with the subnets in the selected subnet group. The number of Availability Zones listed must equal the value of <code>NumCacheClusters</code>. </note> Default: system chosen Availability Zones.<br/>NumCacheClusters:  The number of clusters this replication group initially has. This parameter is not used if there is more than one node group (shard). You should use <code>ReplicasPerNodeGroup</code> instead. If <code>AutomaticFailoverEnabled</code> is <code>true</code>, the value of this parameter must be at least 2. If <code>AutomaticFailoverEnabled</code> is <code>false</code> you can omit this parameter (it will default to 1), or you can explicitly set it to a value between 2 and 6. The maximum permitted value for <code>NumCacheClusters</code> is 6 (1 primary plus 5 replicas).<br/>AutomaticFailoverEnabled:  Specifies whether a read-only replica is automatically promoted to read/write primary if the existing primary fails. If <code>true</code>, Multi-AZ is enabled for this replication group. If <code>false</code>, Multi-AZ is disabled for this replication group.  <code>AutomaticFailoverEnabled</code> must be enabled for Redis (cluster mode enabled) replication groups. Default: false Amazon ElastiCache for Redis does not support Multi-AZ with automatic failover on: <ul> <li> Redis versions earlier than 2.8.6. </li> <li> Redis (cluster mode disabled): T1 node types. </li> <li> Redis (cluster mode enabled): T1 node types. </li> </ul><br/>PrimaryClusterId:  The identifier of the cluster that serves as the primary for this replication group. This cluster must already exist and have a status of <code>available</code>. This parameter is not required if <code>NumCacheClusters</code>, <code>NumNodeGroups</code>, or <code>ReplicasPerNodeGroup</code> is specified.<br/>ReplicationGroupDescription:  A user-created description for the replication group.<br/>ReplicationGroupId:  The replication group identifier. This parameter is stored as a lowercase string. Constraints: <ul> <li> A name must contain from 1 to 20 alphanumeric characters or hyphens. </li> <li> The first character must be a letter. </li> <li> A name cannot end with a hyphen or contain two consecutive hyphens. </li> </ul><br/>
        :return: ReplicationGroup:  Contains all of the attributes of a specific Redis replication group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AtRestEncryptionEnabled</i> A flag that enables encryption at-rest when set to <code>true</code>. You cannot modify the value of <code>AtRestEncryptionEnabled</code> after the cluster is created. To enable encryption at-rest on a cluster you must set <code>AtRestEncryptionEnabled</code> to <code>true</code> when you create a cluster.  <b>Required:</b> Only available when creating a replication group in an Amazon VPC using redis version <code>3.2.6</code>, <code>4.x</code> or later. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitEncryptionEnabled</i> A flag that enables in-transit encryption when set to <code>true</code>. You cannot modify the value of <code>TransitEncryptionEnabled</code> after the cluster is created. To enable in-transit encryption on a cluster you must set <code>TransitEncryptionEnabled</code> to <code>true</code> when you create a cluster.  <b>Required:</b> Only available when creating a replication group in an Amazon VPC using redis version <code>3.2.6</code>, <code>4.x</code> or later. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthTokenEnabled</i> A flag that enables using an <code>AuthToken</code> (password) when issuing Redis commands. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodeType</i> The name of the compute and memory capacity node type for each node in the replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterEnabled</i> A flag indicating whether or not this replication group is cluster enabled; i.e., whether its data can be partitioned across multiple shards (API/CLI: node groups). Valid values: <code>true</code> | <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotWindow</i> The daily time range (in UTC) during which ElastiCache begins taking a daily snapshot of your node group (shard). Example: <code>05:00-09:00</code>  If you do not specify this parameter, ElastiCache automatically chooses an appropriate time range. <note> This parameter is only valid if the <code>Engine</code> parameter is <code>redis</code>. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotRetentionLimit</i> The number of days for which ElastiCache retains automatic cluster snapshots before deleting them. For example, if you set <code>SnapshotRetentionLimit</code> to 5, a snapshot that was taken today is retained for 5 days before being deleted. <important>  If the value of <code>SnapshotRetentionLimit</code> is set to zero (0), backups are turned off. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigurationEndpoint</i> The configuration endpoint for this replication group. Use the configuration endpoint to connect to this replication group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ConfigurationEndpoint</b>: The configuration endpoint for this replication group. Use the configuration endpoint to connect to this replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomaticFailover</i> Indicates the status of Multi-AZ with automatic failover for this Redis replication group. Amazon ElastiCache for Redis does not support Multi-AZ with automatic failover on: <ul> <li> Redis versions earlier than 2.8.6. </li> <li> Redis (cluster mode disabled): T1 node types. </li> <li> Redis (cluster mode enabled): T1 node types. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshottingClusterId</i> The cluster ID that is used as the daily snapshot source for the replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeGroups</i> A list of node groups in this replication group. For Redis (cluster mode disabled) replication groups, this is a single-element list. For Redis (cluster mode enabled) replication groups, the list contains an entry for each node group (shard).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MemberClusters</i> The names of all the cache clusters that are part of this replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> A group of settings to be applied to the replication group, either immediately or during the next maintenance window.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: A group of settings to be applied to the replication group, either immediately or during the next maintenance window.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current state of this replication group - <code>creating</code>, <code>available</code>, <code>modifying</code>, <code>deleting</code>, <code>create-failed</code>, <code>snapshotting</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The user supplied description of the replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationGroupId</i> The identifier for the replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomaticFailoverStatus</i> Indicates the status of Multi-AZ with automatic failover for this Redis replication group. Amazon ElastiCache for Redis does not support Multi-AZ with automatic failover on: <ul> <li> Redis versions earlier than 2.8.6. </li> <li> Redis (cluster mode disabled): T1 node types. </li> <li> Redis (cluster mode enabled): T1 node types. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PrimaryClusterId</i> The primary cluster ID that is applied immediately (if <code>--apply-immediately</code> was specified), or during the next maintenance window.<br/>The status of an online resharding operation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Resharding</b>: The status of an online resharding operation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port number that the cache engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS hostname of the cache node.<br/>        """
        pass

    def create_snapshot(self, SnapshotName: str, ReplicationGroupId: str=None, CacheClusterId: str=None) -> CreateSnapshotResult:
        """Creates a copy of an entire cluster or replication group at a specific moment in time. <note> This operation is valid for Redis only. </note>

        :param self:
        :param SnapshotName:  A name for the snapshot being created.<br/>CacheClusterId:  The identifier of an existing cluster. The snapshot is created from this cluster.<br/>ReplicationGroupId:  The identifier of an existing replication group. The snapshot is created from this replication group.<br/>
        :return: Snapshot:  Represents a copy of an entire Redis cluster as of the time when the snapshot was taken.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeSnapshots</i> A list of the cache nodes in the source cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomaticFailover</i> Indicates the status of Multi-AZ with automatic failover for the source Redis replication group. Amazon ElastiCache for Redis does not support Multi-AZ with automatic failover on: <ul> <li> Redis versions earlier than 2.8.6. </li> <li> Redis (cluster mode disabled): T1 node types. </li> <li> Redis (cluster mode enabled): T1 node types. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumNodeGroups</i> The number of node groups (shards) in this snapshot. When restoring from a snapshot, the number of node groups (shards) in the snapshot and in the restored replication group must be the same.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotWindow</i> The daily time range during which ElastiCache takes daily snapshots of the source cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotRetentionLimit</i> For an automatic snapshot, the number of days for which ElastiCache retains the snapshot before deleting it. For manual snapshots, this field reflects the <code>SnapshotRetentionLimit</code> for the source cluster when the snapshot was created. This field is otherwise ignored: Manual snapshots do not expire, and can only be deleted using the <code>DeleteSnapshot</code> operation.   <b>Important</b> If the value of SnapshotRetentionLimit is set to zero (0), backups are turned off.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> This parameter is currently disabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The Amazon Virtual Private Cloud identifier (VPC ID) of the cache subnet group for the source cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheSubnetGroupName</i> The name of the cache subnet group associated with the source cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheParameterGroupName</i> The cache parameter group that is associated with the source cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port number used by each cache nodes in the source cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicArn</i> The Amazon Resource Name (ARN) for the topic used by the source cluster for publishing notifications.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which maintenance on the cluster is performed. It is specified as a range in the format ddd:hh24:mi-ddd:hh24:mi (24H Clock UTC). The minimum maintenance window is a 60 minute period. Valid values for <code>ddd</code> are: <ul> <li>  <code>sun</code>  </li> <li>  <code>mon</code>  </li> <li>  <code>tue</code>  </li> <li>  <code>wed</code>  </li> <li>  <code>thu</code>  </li> <li>  <code>fri</code>  </li> <li>  <code>sat</code>  </li> </ul> Example: <code>sun:23:00-mon:01:30</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheClusterCreateTime</i> The date and time when the source cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredAvailabilityZone</i> The name of the Availability Zone in which the source cluster is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumCacheNodes</i> The number of cache nodes in the source cluster. For clusters running Redis, this value must be 1. For clusters running Memcached, this value must be between 1 and 20.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> The version of the cache engine version that is used by the source cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> The name of the cache engine (<code>memcached</code> or <code>redis</code>) used by the source cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodeType</i> The name of the compute and memory capacity node type for the source cluster. The following node types are supported by ElastiCache. Generally speaking, the current generation types provide more memory and computational power at lower cost when compared to their equivalent previous generation counterparts. <ul> <li> General purpose: <ul> <li> Current generation:   <b>M5 node types:</b> <code>cache.m5.large</code>, <code>cache.m5.xlarge</code>, <code>cache.m5.2xlarge</code>, <code>cache.m5.4xlarge</code>, <code>cache.m5.12xlarge</code>, <code>cache.m5.24xlarge</code>   <b>M4 node types:</b> <code>cache.m4.large</code>, <code>cache.m4.xlarge</code>, <code>cache.m4.2xlarge</code>, <code>cache.m4.4xlarge</code>, <code>cache.m4.10xlarge</code>   <b>T2 node types:</b> <code>cache.t2.micro</code>, <code>cache.t2.small</code>, <code>cache.t2.medium</code>  </li> <li> Previous generation: (not recommended)  <b>T1 node types:</b> <code>cache.t1.micro</code>   <b>M1 node types:</b> <code>cache.m1.small</code>, <code>cache.m1.medium</code>, <code>cache.m1.large</code>, <code>cache.m1.xlarge</code>   <b>M3 node types:</b> <code>cache.m3.medium</code>, <code>cache.m3.large</code>, <code>cache.m3.xlarge</code>, <code>cache.m3.2xlarge</code>  </li> </ul> </li> <li> Compute optimized: <ul> <li> Previous generation: (not recommended)  <b>C1 node types:</b> <code>cache.c1.xlarge</code>  </li> </ul> </li> <li> Memory optimized: <ul> <li> Current generation:   <b>R5 node types:</b> <code>cache.r5.large</code>, <code>cache.r5.xlarge</code>, <code>cache.r5.2xlarge</code>, <code>cache.r5.4xlarge</code>, <code>cache.r5.12xlarge</code>, <code>cache.r5.24xlarge</code>   <b>R4 node types:</b> <code>cache.r4.large</code>, <code>cache.r4.xlarge</code>, <code>cache.r4.2xlarge</code>, <code>cache.r4.4xlarge</code>, <code>cache.r4.8xlarge</code>, <code>cache.r4.16xlarge</code>  </li> <li> Previous generation: (not recommended)  <b>M2 node types:</b> <code>cache.m2.xlarge</code>, <code>cache.m2.2xlarge</code>, <code>cache.m2.4xlarge</code>   <b>R3 node types:</b> <code>cache.r3.large</code>, <code>cache.r3.xlarge</code>, <code>cache.r3.2xlarge</code>, <code>cache.r3.4xlarge</code>, <code>cache.r3.8xlarge</code>  </li> </ul> </li> </ul>  <b>Additional node type info</b>  <ul> <li> All current generation instance types are created in Amazon VPC by default. </li> <li> Redis append-only files (AOF) are not supported for T1 or T2 instances. </li> <li> Redis Multi-AZ with automatic failover is not supported on T1 instances. </li> <li> Redis configuration variables <code>appendonly</code> and <code>appendfsync</code> are not supported on Redis version 2.8.22 and later. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotSource</i> Indicates whether the snapshot is from an automatic backup (<code>automated</code>) or was created manually (<code>manual</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotStatus</i> The status of the snapshot. Valid values: <code>creating</code> | <code>available</code> | <code>restoring</code> | <code>copying</code> | <code>deleting</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheClusterId</i> The user-supplied identifier of the source cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationGroupDescription</i> A description of the source replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationGroupId</i> The unique identifier of the source replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotName</i> The name of a snapshot. For an automatic snapshot, the name is system-generated. For a manual snapshot, this is the user-provided name.<br/>        """
        pass

    def decrease_replica_count(self, ApplyImmediately: Boolean, ReplicationGroupId: str, NewReplicaCount: int=None, ReplicaConfiguration: List[ConfigureShard]=None, ReplicasToRemove: List[String]=None) -> DecreaseReplicaCountResult:
        """Dynamically decreases the number of replics in a Redis (cluster mode disabled) replication group or the number of replica nodes in one or more node groups (shards) of a Redis (cluster mode enabled) replication group. This operation is performed with no cluster down time.

        :param self:
        :param ApplyImmediately:  If <code>True</code>, the number of replica nodes is decreased immediately. <code>ApplyImmediately=False</code> is not currently supported.<br/>ReplicasToRemove:  A list of the node ids to remove from the replication group or node group (shard).<br/>ReplicaConfiguration:  A list of <code>ConfigureShard</code> objects that can be used to configure each shard in a Redis (cluster mode enabled) replication group. The <code>ConfigureShard</code> has three members: <code>NewReplicaCount</code>, <code>NodeGroupId</code>, and <code>PreferredAvailabilityZones</code>.<br/>NewReplicaCount:  The number of read replica nodes you want at the completion of this operation. For Redis (cluster mode disabled) replication groups, this is the number of replica nodes in the replication group. For Redis (cluster mode enabled) replication groups, this is the number of replica nodes in each of the replication group's node groups. The minimum number of replicas in a shard or replication group is: <ul> <li> Redis (cluster mode disabled) <ul> <li> If Multi-AZ with Automatic Failover is enabled: 1 </li> <li> If Multi-AZ with Automatic Failover is not enabled: 0 </li> </ul> </li> <li> Redis (cluster mode enabled): 0 (though you will not be able to failover to a replica if your primary node fails) </li> </ul><br/>ReplicationGroupId:  The id of the replication group from which you want to remove replica nodes.<br/>
        :return: ReplicationGroup:  Contains all of the attributes of a specific Redis replication group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AtRestEncryptionEnabled</i> A flag that enables encryption at-rest when set to <code>true</code>. You cannot modify the value of <code>AtRestEncryptionEnabled</code> after the cluster is created. To enable encryption at-rest on a cluster you must set <code>AtRestEncryptionEnabled</code> to <code>true</code> when you create a cluster.  <b>Required:</b> Only available when creating a replication group in an Amazon VPC using redis version <code>3.2.6</code>, <code>4.x</code> or later. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitEncryptionEnabled</i> A flag that enables in-transit encryption when set to <code>true</code>. You cannot modify the value of <code>TransitEncryptionEnabled</code> after the cluster is created. To enable in-transit encryption on a cluster you must set <code>TransitEncryptionEnabled</code> to <code>true</code> when you create a cluster.  <b>Required:</b> Only available when creating a replication group in an Amazon VPC using redis version <code>3.2.6</code>, <code>4.x</code> or later. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthTokenEnabled</i> A flag that enables using an <code>AuthToken</code> (password) when issuing Redis commands. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodeType</i> The name of the compute and memory capacity node type for each node in the replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterEnabled</i> A flag indicating whether or not this replication group is cluster enabled; i.e., whether its data can be partitioned across multiple shards (API/CLI: node groups). Valid values: <code>true</code> | <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotWindow</i> The daily time range (in UTC) during which ElastiCache begins taking a daily snapshot of your node group (shard). Example: <code>05:00-09:00</code>  If you do not specify this parameter, ElastiCache automatically chooses an appropriate time range. <note> This parameter is only valid if the <code>Engine</code> parameter is <code>redis</code>. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotRetentionLimit</i> The number of days for which ElastiCache retains automatic cluster snapshots before deleting them. For example, if you set <code>SnapshotRetentionLimit</code> to 5, a snapshot that was taken today is retained for 5 days before being deleted. <important>  If the value of <code>SnapshotRetentionLimit</code> is set to zero (0), backups are turned off. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigurationEndpoint</i> The configuration endpoint for this replication group. Use the configuration endpoint to connect to this replication group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ConfigurationEndpoint</b>: The configuration endpoint for this replication group. Use the configuration endpoint to connect to this replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomaticFailover</i> Indicates the status of Multi-AZ with automatic failover for this Redis replication group. Amazon ElastiCache for Redis does not support Multi-AZ with automatic failover on: <ul> <li> Redis versions earlier than 2.8.6. </li> <li> Redis (cluster mode disabled): T1 node types. </li> <li> Redis (cluster mode enabled): T1 node types. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshottingClusterId</i> The cluster ID that is used as the daily snapshot source for the replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeGroups</i> A list of node groups in this replication group. For Redis (cluster mode disabled) replication groups, this is a single-element list. For Redis (cluster mode enabled) replication groups, the list contains an entry for each node group (shard).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MemberClusters</i> The names of all the cache clusters that are part of this replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> A group of settings to be applied to the replication group, either immediately or during the next maintenance window.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: A group of settings to be applied to the replication group, either immediately or during the next maintenance window.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current state of this replication group - <code>creating</code>, <code>available</code>, <code>modifying</code>, <code>deleting</code>, <code>create-failed</code>, <code>snapshotting</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The user supplied description of the replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationGroupId</i> The identifier for the replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomaticFailoverStatus</i> Indicates the status of Multi-AZ with automatic failover for this Redis replication group. Amazon ElastiCache for Redis does not support Multi-AZ with automatic failover on: <ul> <li> Redis versions earlier than 2.8.6. </li> <li> Redis (cluster mode disabled): T1 node types. </li> <li> Redis (cluster mode enabled): T1 node types. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PrimaryClusterId</i> The primary cluster ID that is applied immediately (if <code>--apply-immediately</code> was specified), or during the next maintenance window.<br/>The status of an online resharding operation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Resharding</b>: The status of an online resharding operation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port number that the cache engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS hostname of the cache node.<br/>        """
        pass

    def delete_cache_cluster(self, CacheClusterId: str, FinalSnapshotIdentifier: str=None) -> DeleteCacheClusterResult:
        """Deletes a previously provisioned cluster. <code>DeleteCacheCluster</code> deletes all associated cache nodes, node endpoints and the cluster itself. When you receive a successful response from this operation, Amazon ElastiCache immediately begins deleting the cluster; you cannot cancel or revert this operation. This operation is not valid for: <ul> <li> Redis (cluster mode enabled) clusters </li> <li> A cluster that is the last read replica of a replication group </li> <li> A node group (shard) that has Multi-AZ mode enabled </li> <li> A cluster from a Redis (cluster mode enabled) replication group </li> <li> A cluster that is not in the <code>available</code> state </li> </ul>

        :param self:
        :param FinalSnapshotIdentifier:  The user-supplied name of a final cluster snapshot. This is the unique name that identifies the snapshot. ElastiCache creates the snapshot, and then deletes the cluster immediately afterward.<br/>CacheClusterId:  The cluster identifier for the cluster to be deleted. This parameter is not case sensitive.<br/>
        :return: CacheCluster:  Contains all of the attributes of a specific cluster.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AtRestEncryptionEnabled</i> A flag that enables encryption at-rest when set to <code>true</code>. You cannot modify the value of <code>AtRestEncryptionEnabled</code> after the cluster is created. To enable at-rest encryption on a cluster you must set <code>AtRestEncryptionEnabled</code> to <code>true</code> when you create a cluster.  <b>Required:</b> Only available when creating a replication group in an Amazon VPC using redis version <code>3.2.6</code>, <code>4.x</code> or later. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitEncryptionEnabled</i> A flag that enables in-transit encryption when set to <code>true</code>. You cannot modify the value of <code>TransitEncryptionEnabled</code> after the cluster is created. To enable in-transit encryption on a cluster you must set <code>TransitEncryptionEnabled</code> to <code>true</code> when you create a cluster.  <b>Required:</b> Only available when creating a replication group in an Amazon VPC using redis version <code>3.2.6</code>, <code>4.x</code> or later. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthTokenEnabled</i> A flag that enables using an <code>AuthToken</code> (password) when issuing Redis commands. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotWindow</i> The daily time range (in UTC) during which ElastiCache begins taking a daily snapshot of your cluster. Example: <code>05:00-09:00</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotRetentionLimit</i> The number of days for which ElastiCache retains automatic cluster snapshots before deleting them. For example, if you set <code>SnapshotRetentionLimit</code> to 5, a snapshot that was taken today is retained for 5 days before being deleted. <important>  If the value of SnapshotRetentionLimit is set to zero (0), backups are turned off. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationGroupId</i> The replication group to which this cluster belongs. If this field is empty, the cluster is not associated with any replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroups</i> A list of VPC Security Groups associated with the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> This parameter is currently disabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodes</i> A list of cache nodes that are members of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheSubnetGroupName</i> The name of the cache subnet group associated with the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheParameterGroup</i> Status of the cache parameter group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CacheParameterGroup</b>: Status of the cache parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheSecurityGroups</i> A list of cache security group elements, composed of name and status sub-elements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationConfiguration</i> Describes a notification topic and its status. Notification topics are used for publishing ElastiCache events to subscribers using Amazon Simple Notification Service (SNS). &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NotificationConfiguration</b>: Describes a notification topic and its status. Notification topics are used for publishing ElastiCache events to subscribers using Amazon Simple Notification Service (SNS). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> A group of settings that are applied to the cluster in the future, or that are currently being applied.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: A group of settings that are applied to the cluster in the future, or that are currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which maintenance on the cluster is performed. It is specified as a range in the format ddd:hh24:mi-ddd:hh24:mi (24H Clock UTC). The minimum maintenance window is a 60 minute period. Valid values for <code>ddd</code> are: <ul> <li>  <code>sun</code>  </li> <li>  <code>mon</code>  </li> <li>  <code>tue</code>  </li> <li>  <code>wed</code>  </li> <li>  <code>thu</code>  </li> <li>  <code>fri</code>  </li> <li>  <code>sat</code>  </li> </ul> Example: <code>sun:23:00-mon:01:30</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheClusterCreateTime</i> The date and time when the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredAvailabilityZone</i> The name of the Availability Zone in which the cluster is located or "Multiple" if the cache nodes are located in different Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumCacheNodes</i> The number of cache nodes in the cluster. For clusters running Redis, this value must be 1. For clusters running Memcached, this value must be between 1 and 20.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheClusterStatus</i> The current state of this cluster, one of the following values: <code>available</code>, <code>creating</code>, <code>deleted</code>, <code>deleting</code>, <code>incompatible-network</code>, <code>modifying</code>, <code>rebooting cluster nodes</code>, <code>restore-failed</code>, or <code>snapshotting</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> The version of the cache engine that is used in this cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> The name of the cache engine (<code>memcached</code> or <code>redis</code>) to be used for this cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodeType</i> The name of the compute and memory capacity node type for the cluster. The following node types are supported by ElastiCache. Generally speaking, the current generation types provide more memory and computational power at lower cost when compared to their equivalent previous generation counterparts. <ul> <li> General purpose: <ul> <li> Current generation:   <b>M5 node types:</b> <code>cache.m5.large</code>, <code>cache.m5.xlarge</code>, <code>cache.m5.2xlarge</code>, <code>cache.m5.4xlarge</code>, <code>cache.m5.12xlarge</code>, <code>cache.m5.24xlarge</code>   <b>M4 node types:</b> <code>cache.m4.large</code>, <code>cache.m4.xlarge</code>, <code>cache.m4.2xlarge</code>, <code>cache.m4.4xlarge</code>, <code>cache.m4.10xlarge</code>   <b>T2 node types:</b> <code>cache.t2.micro</code>, <code>cache.t2.small</code>, <code>cache.t2.medium</code>  </li> <li> Previous generation: (not recommended)  <b>T1 node types:</b> <code>cache.t1.micro</code>   <b>M1 node types:</b> <code>cache.m1.small</code>, <code>cache.m1.medium</code>, <code>cache.m1.large</code>, <code>cache.m1.xlarge</code>   <b>M3 node types:</b> <code>cache.m3.medium</code>, <code>cache.m3.large</code>, <code>cache.m3.xlarge</code>, <code>cache.m3.2xlarge</code>  </li> </ul> </li> <li> Compute optimized: <ul> <li> Previous generation: (not recommended)  <b>C1 node types:</b> <code>cache.c1.xlarge</code>  </li> </ul> </li> <li> Memory optimized: <ul> <li> Current generation:   <b>R5 node types:</b> <code>cache.r5.large</code>, <code>cache.r5.xlarge</code>, <code>cache.r5.2xlarge</code>, <code>cache.r5.4xlarge</code>, <code>cache.r5.12xlarge</code>, <code>cache.r5.24xlarge</code>   <b>R4 node types:</b> <code>cache.r4.large</code>, <code>cache.r4.xlarge</code>, <code>cache.r4.2xlarge</code>, <code>cache.r4.4xlarge</code>, <code>cache.r4.8xlarge</code>, <code>cache.r4.16xlarge</code>  </li> <li> Previous generation: (not recommended)  <b>M2 node types:</b> <code>cache.m2.xlarge</code>, <code>cache.m2.2xlarge</code>, <code>cache.m2.4xlarge</code>   <b>R3 node types:</b> <code>cache.r3.large</code>, <code>cache.r3.xlarge</code>, <code>cache.r3.2xlarge</code>, <code>cache.r3.4xlarge</code>, <code>cache.r3.8xlarge</code>  </li> </ul> </li> </ul>  <b>Additional node type info</b>  <ul> <li> All current generation instance types are created in Amazon VPC by default. </li> <li> Redis append-only files (AOF) are not supported for T1 or T2 instances. </li> <li> Redis Multi-AZ with automatic failover is not supported on T1 instances. </li> <li> Redis configuration variables <code>appendonly</code> and <code>appendfsync</code> are not supported on Redis version 2.8.22 and later. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientDownloadLandingPage</i> The URL of the web page where you can download the latest ElastiCache client library.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigurationEndpoint</i> Represents a Memcached cluster endpoint which, if Automatic Discovery is enabled on the cluster, can be used by an application to connect to any node in the cluster. The configuration endpoint will always have <code>.cfg</code> in it. Example: <code>mem-3.9dvc4r<u>.cfg</u>.usw2.cache.amazonaws.com:11211</code> &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ConfigurationEndpoint</b>: Represents a Memcached cluster endpoint which, if Automatic Discovery is enabled on the cluster, can be used by an application to connect to any node in the cluster. The configuration endpoint will always have <code>.cfg</code> in it. Example: <code>mem-3.9dvc4r<u>.cfg</u>.usw2.cache.amazonaws.com:11211</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheClusterId</i> The user-supplied identifier of the cluster. This identifier is a unique key that identifies a cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port number that the cache engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS hostname of the cache node.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodeType</i> The cache node type that this cluster or replication group is scaled to.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> The new cache engine version that the cluster runs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodeIdsToRemove</i> A list of cache node IDs that are being removed (or will be removed) from the cluster. A node ID is a 4-digit numeric identifier (0001, 0002, etc.).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumCacheNodes</i> The new number of cache nodes for the cluster. For clusters running Redis, this value must be 1. For clusters running Memcached, this value must be between 1 and 20.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicStatus</i> The current state of the topic.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicArn</i> The Amazon Resource Name (ARN) that identifies the topic.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodeIdsToReboot</i> A list of the cache node IDs which need to be rebooted for parameter changes to be applied. A node ID is a numeric identifier (0001, 0002, etc.).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterApplyStatus</i> The status of parameter updates.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheParameterGroupName</i> The name of the cache parameter group.<br/>        """
        pass

    def delete_cache_parameter_group(self, CacheParameterGroupName: str):
        """Deletes the specified cache parameter group. You cannot delete a cache parameter group if it is associated with any cache clusters.

        :param self:
        :param CacheParameterGroupName:  The name of the cache parameter group to delete. <note> The specified cache security group must not be associated with any clusters. </note><br/>
        :return:         """
        pass

    def delete_cache_security_group(self, CacheSecurityGroupName: str):
        """Deletes a cache security group. <note> You cannot delete a cache security group if it is associated with any clusters. </note>

        :param self:
        :param CacheSecurityGroupName:  The name of the cache security group to delete. <note> You cannot delete the default security group. </note><br/>
        :return:         """
        pass

    def delete_cache_subnet_group(self, CacheSubnetGroupName: str):
        """Deletes a cache subnet group. <note> You cannot delete a cache subnet group if it is associated with any clusters. </note>

        :param self:
        :param CacheSubnetGroupName:  The name of the cache subnet group to delete. Constraints: Must contain no more than 255 alphanumeric characters or hyphens.<br/>
        :return:         """
        pass

    def delete_replication_group(self, ReplicationGroupId: str, RetainPrimaryCluster: BooleanOptional=None, FinalSnapshotIdentifier: str=None) -> DeleteReplicationGroupResult:
        """Deletes an existing replication group. By default, this operation deletes the entire replication group, including the primary/primaries and all of the read replicas. If the replication group has only one primary, you can optionally delete only the read replicas, while retaining the primary by setting <code>RetainPrimaryCluster=true</code>. When you receive a successful response from this operation, Amazon ElastiCache immediately begins deleting the selected resources; you cannot cancel or revert this operation. <note> This operation is valid for Redis only. </note>

        :param self:
        :param FinalSnapshotIdentifier:  The name of a final node group (shard) snapshot. ElastiCache creates the snapshot from the primary node in the cluster, rather than one of the replicas; this is to ensure that it captures the freshest data. After the final snapshot is taken, the replication group is immediately deleted.<br/>RetainPrimaryCluster:  If set to <code>true</code>, all of the read replicas are deleted, but the primary node is retained.<br/>ReplicationGroupId:  The identifier for the cluster to be deleted. This parameter is not case sensitive.<br/>
        :return: ReplicationGroup:  Contains all of the attributes of a specific Redis replication group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AtRestEncryptionEnabled</i> A flag that enables encryption at-rest when set to <code>true</code>. You cannot modify the value of <code>AtRestEncryptionEnabled</code> after the cluster is created. To enable encryption at-rest on a cluster you must set <code>AtRestEncryptionEnabled</code> to <code>true</code> when you create a cluster.  <b>Required:</b> Only available when creating a replication group in an Amazon VPC using redis version <code>3.2.6</code>, <code>4.x</code> or later. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitEncryptionEnabled</i> A flag that enables in-transit encryption when set to <code>true</code>. You cannot modify the value of <code>TransitEncryptionEnabled</code> after the cluster is created. To enable in-transit encryption on a cluster you must set <code>TransitEncryptionEnabled</code> to <code>true</code> when you create a cluster.  <b>Required:</b> Only available when creating a replication group in an Amazon VPC using redis version <code>3.2.6</code>, <code>4.x</code> or later. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthTokenEnabled</i> A flag that enables using an <code>AuthToken</code> (password) when issuing Redis commands. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodeType</i> The name of the compute and memory capacity node type for each node in the replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterEnabled</i> A flag indicating whether or not this replication group is cluster enabled; i.e., whether its data can be partitioned across multiple shards (API/CLI: node groups). Valid values: <code>true</code> | <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotWindow</i> The daily time range (in UTC) during which ElastiCache begins taking a daily snapshot of your node group (shard). Example: <code>05:00-09:00</code>  If you do not specify this parameter, ElastiCache automatically chooses an appropriate time range. <note> This parameter is only valid if the <code>Engine</code> parameter is <code>redis</code>. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotRetentionLimit</i> The number of days for which ElastiCache retains automatic cluster snapshots before deleting them. For example, if you set <code>SnapshotRetentionLimit</code> to 5, a snapshot that was taken today is retained for 5 days before being deleted. <important>  If the value of <code>SnapshotRetentionLimit</code> is set to zero (0), backups are turned off. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigurationEndpoint</i> The configuration endpoint for this replication group. Use the configuration endpoint to connect to this replication group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ConfigurationEndpoint</b>: The configuration endpoint for this replication group. Use the configuration endpoint to connect to this replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomaticFailover</i> Indicates the status of Multi-AZ with automatic failover for this Redis replication group. Amazon ElastiCache for Redis does not support Multi-AZ with automatic failover on: <ul> <li> Redis versions earlier than 2.8.6. </li> <li> Redis (cluster mode disabled): T1 node types. </li> <li> Redis (cluster mode enabled): T1 node types. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshottingClusterId</i> The cluster ID that is used as the daily snapshot source for the replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeGroups</i> A list of node groups in this replication group. For Redis (cluster mode disabled) replication groups, this is a single-element list. For Redis (cluster mode enabled) replication groups, the list contains an entry for each node group (shard).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MemberClusters</i> The names of all the cache clusters that are part of this replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> A group of settings to be applied to the replication group, either immediately or during the next maintenance window.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: A group of settings to be applied to the replication group, either immediately or during the next maintenance window.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current state of this replication group - <code>creating</code>, <code>available</code>, <code>modifying</code>, <code>deleting</code>, <code>create-failed</code>, <code>snapshotting</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The user supplied description of the replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationGroupId</i> The identifier for the replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomaticFailoverStatus</i> Indicates the status of Multi-AZ with automatic failover for this Redis replication group. Amazon ElastiCache for Redis does not support Multi-AZ with automatic failover on: <ul> <li> Redis versions earlier than 2.8.6. </li> <li> Redis (cluster mode disabled): T1 node types. </li> <li> Redis (cluster mode enabled): T1 node types. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PrimaryClusterId</i> The primary cluster ID that is applied immediately (if <code>--apply-immediately</code> was specified), or during the next maintenance window.<br/>The status of an online resharding operation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Resharding</b>: The status of an online resharding operation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port number that the cache engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS hostname of the cache node.<br/>        """
        pass

    def delete_snapshot(self, SnapshotName: str) -> DeleteSnapshotResult:
        """Deletes an existing snapshot. When you receive a successful response from this operation, ElastiCache immediately begins deleting the snapshot; you cannot cancel or revert this operation. <note> This operation is valid for Redis only. </note>

        :param self:
        :param SnapshotName:  The name of the snapshot to be deleted.<br/>
        :return: Snapshot:  Represents a copy of an entire Redis cluster as of the time when the snapshot was taken.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeSnapshots</i> A list of the cache nodes in the source cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomaticFailover</i> Indicates the status of Multi-AZ with automatic failover for the source Redis replication group. Amazon ElastiCache for Redis does not support Multi-AZ with automatic failover on: <ul> <li> Redis versions earlier than 2.8.6. </li> <li> Redis (cluster mode disabled): T1 node types. </li> <li> Redis (cluster mode enabled): T1 node types. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumNodeGroups</i> The number of node groups (shards) in this snapshot. When restoring from a snapshot, the number of node groups (shards) in the snapshot and in the restored replication group must be the same.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotWindow</i> The daily time range during which ElastiCache takes daily snapshots of the source cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotRetentionLimit</i> For an automatic snapshot, the number of days for which ElastiCache retains the snapshot before deleting it. For manual snapshots, this field reflects the <code>SnapshotRetentionLimit</code> for the source cluster when the snapshot was created. This field is otherwise ignored: Manual snapshots do not expire, and can only be deleted using the <code>DeleteSnapshot</code> operation.   <b>Important</b> If the value of SnapshotRetentionLimit is set to zero (0), backups are turned off.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> This parameter is currently disabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The Amazon Virtual Private Cloud identifier (VPC ID) of the cache subnet group for the source cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheSubnetGroupName</i> The name of the cache subnet group associated with the source cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheParameterGroupName</i> The cache parameter group that is associated with the source cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port number used by each cache nodes in the source cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicArn</i> The Amazon Resource Name (ARN) for the topic used by the source cluster for publishing notifications.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which maintenance on the cluster is performed. It is specified as a range in the format ddd:hh24:mi-ddd:hh24:mi (24H Clock UTC). The minimum maintenance window is a 60 minute period. Valid values for <code>ddd</code> are: <ul> <li>  <code>sun</code>  </li> <li>  <code>mon</code>  </li> <li>  <code>tue</code>  </li> <li>  <code>wed</code>  </li> <li>  <code>thu</code>  </li> <li>  <code>fri</code>  </li> <li>  <code>sat</code>  </li> </ul> Example: <code>sun:23:00-mon:01:30</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheClusterCreateTime</i> The date and time when the source cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredAvailabilityZone</i> The name of the Availability Zone in which the source cluster is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumCacheNodes</i> The number of cache nodes in the source cluster. For clusters running Redis, this value must be 1. For clusters running Memcached, this value must be between 1 and 20.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> The version of the cache engine version that is used by the source cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> The name of the cache engine (<code>memcached</code> or <code>redis</code>) used by the source cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodeType</i> The name of the compute and memory capacity node type for the source cluster. The following node types are supported by ElastiCache. Generally speaking, the current generation types provide more memory and computational power at lower cost when compared to their equivalent previous generation counterparts. <ul> <li> General purpose: <ul> <li> Current generation:   <b>M5 node types:</b> <code>cache.m5.large</code>, <code>cache.m5.xlarge</code>, <code>cache.m5.2xlarge</code>, <code>cache.m5.4xlarge</code>, <code>cache.m5.12xlarge</code>, <code>cache.m5.24xlarge</code>   <b>M4 node types:</b> <code>cache.m4.large</code>, <code>cache.m4.xlarge</code>, <code>cache.m4.2xlarge</code>, <code>cache.m4.4xlarge</code>, <code>cache.m4.10xlarge</code>   <b>T2 node types:</b> <code>cache.t2.micro</code>, <code>cache.t2.small</code>, <code>cache.t2.medium</code>  </li> <li> Previous generation: (not recommended)  <b>T1 node types:</b> <code>cache.t1.micro</code>   <b>M1 node types:</b> <code>cache.m1.small</code>, <code>cache.m1.medium</code>, <code>cache.m1.large</code>, <code>cache.m1.xlarge</code>   <b>M3 node types:</b> <code>cache.m3.medium</code>, <code>cache.m3.large</code>, <code>cache.m3.xlarge</code>, <code>cache.m3.2xlarge</code>  </li> </ul> </li> <li> Compute optimized: <ul> <li> Previous generation: (not recommended)  <b>C1 node types:</b> <code>cache.c1.xlarge</code>  </li> </ul> </li> <li> Memory optimized: <ul> <li> Current generation:   <b>R5 node types:</b> <code>cache.r5.large</code>, <code>cache.r5.xlarge</code>, <code>cache.r5.2xlarge</code>, <code>cache.r5.4xlarge</code>, <code>cache.r5.12xlarge</code>, <code>cache.r5.24xlarge</code>   <b>R4 node types:</b> <code>cache.r4.large</code>, <code>cache.r4.xlarge</code>, <code>cache.r4.2xlarge</code>, <code>cache.r4.4xlarge</code>, <code>cache.r4.8xlarge</code>, <code>cache.r4.16xlarge</code>  </li> <li> Previous generation: (not recommended)  <b>M2 node types:</b> <code>cache.m2.xlarge</code>, <code>cache.m2.2xlarge</code>, <code>cache.m2.4xlarge</code>   <b>R3 node types:</b> <code>cache.r3.large</code>, <code>cache.r3.xlarge</code>, <code>cache.r3.2xlarge</code>, <code>cache.r3.4xlarge</code>, <code>cache.r3.8xlarge</code>  </li> </ul> </li> </ul>  <b>Additional node type info</b>  <ul> <li> All current generation instance types are created in Amazon VPC by default. </li> <li> Redis append-only files (AOF) are not supported for T1 or T2 instances. </li> <li> Redis Multi-AZ with automatic failover is not supported on T1 instances. </li> <li> Redis configuration variables <code>appendonly</code> and <code>appendfsync</code> are not supported on Redis version 2.8.22 and later. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotSource</i> Indicates whether the snapshot is from an automatic backup (<code>automated</code>) or was created manually (<code>manual</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotStatus</i> The status of the snapshot. Valid values: <code>creating</code> | <code>available</code> | <code>restoring</code> | <code>copying</code> | <code>deleting</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheClusterId</i> The user-supplied identifier of the source cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationGroupDescription</i> A description of the source replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationGroupId</i> The unique identifier of the source replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotName</i> The name of a snapshot. For an automatic snapshot, the name is system-generated. For a manual snapshot, this is the user-provided name.<br/>        """
        pass

    def describe_cache_clusters(self, CacheClusterId: str=None, MaxRecords: int=None, Marker: str=None, ShowCacheNodeInfo: BooleanOptional=None, ShowCacheClustersNotInReplicationGroups: BooleanOptional=None) -> CacheClusterMessage:
        """Returns information about all provisioned clusters if no cluster identifier is specified, or about a specific cache cluster if a cluster identifier is supplied. By default, abbreviated information about the clusters is returned. You can use the optional <i>ShowCacheNodeInfo</i> flag to retrieve detailed information about the cache nodes associated with the clusters. These details include the DNS address and port for the cache node endpoint. If the cluster is in the <i>creating</i> state, only cluster-level information is displayed until all of the nodes are successfully provisioned. If the cluster is in the <i>deleting</i> state, only cluster-level information is displayed. If cache nodes are currently being added to the cluster, node endpoint information and creation time for the additional nodes are not displayed until they are completely provisioned. When the cluster state is <i>available</i>, the cluster is ready for use. If cache nodes are currently being removed from the cluster, no endpoint information for the removed nodes is displayed.

        :param self:
        :param ShowCacheClustersNotInReplicationGroups:  An optional flag that can be included in the <code>DescribeCacheCluster</code> request to show only nodes (API/CLI: clusters) that are not members of a replication group. In practice, this mean Memcached and single node Redis clusters.<br/>ShowCacheNodeInfo:  An optional flag that can be included in the <code>DescribeCacheCluster</code> request to retrieve information about the individual cache nodes.<br/>Marker:  An optional marker returned from a prior request. Use this marker for pagination of results from this operation. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:  The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a marker is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: minimum 20; maximum 100.<br/>CacheClusterId:  The user-supplied cluster identifier. If this parameter is specified, only information about that specific cluster is returned. This parameter isn't case sensitive.<br/>
        :return: CacheClusters:  A list of clusters. Each item in the list contains detailed information about one cluster.<br/>Marker:  Provides an identifier to allow retrieval of paginated results.<br/>        """
        pass

    def describe_cache_engine_versions(self, Engine: str=None, EngineVersion: str=None, CacheParameterGroupFamily: str=None, MaxRecords: int=None, Marker: str=None, DefaultOnly: Boolean=None) -> CacheEngineVersionMessage:
        """Returns a list of the available cache engines and their versions.

        :param self:
        :param DefaultOnly:  If <code>true</code>, specifies that only the default version of the specified engine or engine and major version combination is to be returned.<br/>Marker:  An optional marker returned from a prior request. Use this marker for pagination of results from this operation. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:  The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a marker is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: minimum 20; maximum 100.<br/>CacheParameterGroupFamily:  The name of a specific cache parameter group family to return details for. Valid values are: <code>memcached1.4</code> | <code>memcached1.5</code> | <code>redis2.6</code> | <code>redis2.8</code> | <code>redis3.2</code> | <code>redis4.0</code> | <code>redis5.0</code> |  Constraints: <ul> <li> Must be 1 to 255 alphanumeric characters </li> <li> First character must be a letter </li> <li> Cannot end with a hyphen or contain two consecutive hyphens </li> </ul><br/>EngineVersion:  The cache engine version to return. Example: <code>1.4.14</code> <br/>Engine:  The cache engine to return. Valid values: <code>memcached</code> | <code>redis</code> <br/>
        :return: CacheEngineVersions:  A list of cache engine version details. Each element in the list contains detailed information about one cache engine version.<br/>Marker:  Provides an identifier to allow retrieval of paginated results.<br/>        """
        pass

    def describe_cache_parameter_groups(self, CacheParameterGroupName: str=None, MaxRecords: int=None, Marker: str=None) -> CacheParameterGroupsMessage:
        """Returns a list of cache parameter group descriptions. If a cache parameter group name is specified, the list contains only the descriptions for that group.

        :param self:
        :param Marker:  An optional marker returned from a prior request. Use this marker for pagination of results from this operation. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:  The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a marker is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: minimum 20; maximum 100.<br/>CacheParameterGroupName:  The name of a specific cache parameter group to return details for.<br/>
        :return: CacheParameterGroups:  A list of cache parameter groups. Each element in the list contains detailed information about one cache parameter group.<br/>Marker:  Provides an identifier to allow retrieval of paginated results.<br/>        """
        pass

    def describe_cache_parameters(self, CacheParameterGroupName: str, Source: str=None, MaxRecords: int=None, Marker: str=None) -> CacheParameterGroupDetails:
        """Returns the detailed parameter list for a particular cache parameter group.

        :param self:
        :param Marker:  An optional marker returned from a prior request. Use this marker for pagination of results from this operation. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:  The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a marker is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: minimum 20; maximum 100.<br/>Source:  The parameter types to return. Valid values: <code>user</code> | <code>system</code> | <code>engine-default</code> <br/>CacheParameterGroupName:  The name of a specific cache parameter group to return details for.<br/>
        :return: CacheNodeTypeSpecificParameters:  A list of parameters specific to a particular cache node type. Each element in the list contains detailed information about one parameter.<br/>Parameters:  A list of <a>Parameter</a> instances.<br/>Marker:  Provides an identifier to allow retrieval of paginated results.<br/>        """
        pass

    def describe_cache_security_groups(self, CacheSecurityGroupName: str=None, MaxRecords: int=None, Marker: str=None) -> CacheSecurityGroupMessage:
        """Returns a list of cache security group descriptions. If a cache security group name is specified, the list contains only the description of that group. This applicable only when you have ElastiCache in Classic setup 

        :param self:
        :param Marker:  An optional marker returned from a prior request. Use this marker for pagination of results from this operation. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:  The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a marker is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: minimum 20; maximum 100.<br/>CacheSecurityGroupName:  The name of the cache security group to return details for.<br/>
        :return: CacheSecurityGroups:  A list of cache security groups. Each element in the list contains detailed information about one group.<br/>Marker:  Provides an identifier to allow retrieval of paginated results.<br/>        """
        pass

    def describe_cache_subnet_groups(self, CacheSubnetGroupName: str=None, MaxRecords: int=None, Marker: str=None) -> CacheSubnetGroupMessage:
        """Returns a list of cache subnet group descriptions. If a subnet group name is specified, the list contains only the description of that group. This is applicable only when you have ElastiCache in VPC setup. All ElastiCache clusters now launch in VPC by default. 

        :param self:
        :param Marker:  An optional marker returned from a prior request. Use this marker for pagination of results from this operation. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:  The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a marker is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: minimum 20; maximum 100.<br/>CacheSubnetGroupName:  The name of the cache subnet group to return details for.<br/>
        :return: CacheSubnetGroups:  A list of cache subnet groups. Each element in the list contains detailed information about one group.<br/>Marker:  Provides an identifier to allow retrieval of paginated results.<br/>        """
        pass

    def describe_engine_default_parameters(self, CacheParameterGroupFamily: str, MaxRecords: int=None, Marker: str=None) -> DescribeEngineDefaultParametersResult:
        """Returns the default engine and system parameter information for the specified cache engine.

        :param self:
        :param Marker:  An optional marker returned from a prior request. Use this marker for pagination of results from this operation. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:  The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a marker is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: minimum 20; maximum 100.<br/>CacheParameterGroupFamily:  The name of the cache parameter group family. Valid values are: <code>memcached1.4</code> | <code>memcached1.5</code> | <code>redis2.6</code> | <code>redis2.8</code> | <code>redis3.2</code> | <code>redis4.0</code> | <code>redis5.0</code> | <br/>
        :return: EngineDefaults:  Represents the output of a <code>DescribeEngineDefaultParameters</code> operation.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodeTypeSpecificParameters</i> A list of parameters specific to a particular cache node type. Each element in the list contains detailed information about one parameter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> Contains a list of engine default parameters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Marker</i> Provides an identifier to allow retrieval of paginated results.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheParameterGroupFamily</i> Specifies the name of the cache parameter group family to which the engine default parameters apply. Valid values are: <code>memcached1.4</code> | <code>memcached1.5</code> | <code>redis2.6</code> | <code>redis2.8</code> | <code>redis3.2</code> | <code>redis4.0</code> | <code>redis5.0</code> | <br/>        """
        pass

    def describe_events(self, SourceIdentifier: str=None, SourceType: str=None, StartTime: TStamp=None, EndTime: TStamp=None, Duration: int=None, MaxRecords: int=None, Marker: str=None) -> EventsMessage:
        """Returns events related to clusters, cache security groups, and cache parameter groups. You can obtain events specific to a particular cluster, cache security group, or cache parameter group by providing the name as a parameter. By default, only the events occurring within the last hour are returned; however, you can retrieve up to 14 days' worth of events if necessary.

        :param self:
        :param Marker:  An optional marker returned from a prior request. Use this marker for pagination of results from this operation. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:  The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a marker is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: minimum 20; maximum 100.<br/>Duration:  The number of minutes worth of events to retrieve.<br/>EndTime:  The end of the time interval for which to retrieve events, specified in ISO 8601 format.  <b>Example:</b> 2017-03-30T07:03:49.555Z<br/>StartTime:  The beginning of the time interval to retrieve events for, specified in ISO 8601 format.  <b>Example:</b> 2017-03-30T07:03:49.555Z<br/>SourceType:  The event source to retrieve events for. If no value is specified, all events are returned.<br/>SourceIdentifier:  The identifier of the event source for which events are returned. If not specified, all sources are included in the response.<br/>
        :return: Events:  A list of events. Each element in the list contains detailed information about one event.<br/>Marker:  Provides an identifier to allow retrieval of paginated results.<br/>        """
        pass

    def describe_replication_groups(self, ReplicationGroupId: str=None, MaxRecords: int=None, Marker: str=None) -> ReplicationGroupMessage:
        """Returns information about a particular replication group. If no identifier is specified, <code>DescribeReplicationGroups</code> returns information about all replication groups. <note> This operation is valid for Redis only. </note>

        :param self:
        :param Marker:  An optional marker returned from a prior request. Use this marker for pagination of results from this operation. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:  The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a marker is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: minimum 20; maximum 100.<br/>ReplicationGroupId:  The identifier for the replication group to be described. This parameter is not case sensitive. If you do not specify this parameter, information about all replication groups is returned.<br/>
        :return: ReplicationGroups:  A list of replication groups. Each item in the list contains detailed information about one replication group.<br/>Marker:  Provides an identifier to allow retrieval of paginated results.<br/>        """
        pass

    def describe_reserved_cache_nodes(self, ReservedCacheNodeId: str=None, ReservedCacheNodesOfferingId: str=None, CacheNodeType: str=None, Duration: str=None, ProductDescription: str=None, OfferingType: str=None, MaxRecords: int=None, Marker: str=None) -> ReservedCacheNodeMessage:
        """Returns information about reserved cache nodes for this account, or about a specified reserved cache node.

        :param self:
        :param Marker:  An optional marker returned from a prior request. Use this marker for pagination of results from this operation. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:  The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a marker is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: minimum 20; maximum 100.<br/>OfferingType:  The offering type filter value. Use this parameter to show only the available offerings matching the specified offering type. Valid values: <code>"Light Utilization"|"Medium Utilization"|"Heavy Utilization"</code> <br/>ProductDescription:  The product description filter value. Use this parameter to show only those reservations matching the specified product description.<br/>Duration:  The duration filter value, specified in years or seconds. Use this parameter to show only reservations for this duration. Valid Values: <code>1 | 3 | 31536000 | 94608000</code> <br/>CacheNodeType:  The cache node type filter value. Use this parameter to show only those reservations matching the specified cache node type. The following node types are supported by ElastiCache. Generally speaking, the current generation types provide more memory and computational power at lower cost when compared to their equivalent previous generation counterparts. <ul> <li> General purpose: <ul> <li> Current generation:   <b>M5 node types:</b> <code>cache.m5.large</code>, <code>cache.m5.xlarge</code>, <code>cache.m5.2xlarge</code>, <code>cache.m5.4xlarge</code>, <code>cache.m5.12xlarge</code>, <code>cache.m5.24xlarge</code>   <b>M4 node types:</b> <code>cache.m4.large</code>, <code>cache.m4.xlarge</code>, <code>cache.m4.2xlarge</code>, <code>cache.m4.4xlarge</code>, <code>cache.m4.10xlarge</code>   <b>T2 node types:</b> <code>cache.t2.micro</code>, <code>cache.t2.small</code>, <code>cache.t2.medium</code>  </li> <li> Previous generation: (not recommended)  <b>T1 node types:</b> <code>cache.t1.micro</code>   <b>M1 node types:</b> <code>cache.m1.small</code>, <code>cache.m1.medium</code>, <code>cache.m1.large</code>, <code>cache.m1.xlarge</code>   <b>M3 node types:</b> <code>cache.m3.medium</code>, <code>cache.m3.large</code>, <code>cache.m3.xlarge</code>, <code>cache.m3.2xlarge</code>  </li> </ul> </li> <li> Compute optimized: <ul> <li> Previous generation: (not recommended)  <b>C1 node types:</b> <code>cache.c1.xlarge</code>  </li> </ul> </li> <li> Memory optimized: <ul> <li> Current generation:   <b>R5 node types:</b> <code>cache.r5.large</code>, <code>cache.r5.xlarge</code>, <code>cache.r5.2xlarge</code>, <code>cache.r5.4xlarge</code>, <code>cache.r5.12xlarge</code>, <code>cache.r5.24xlarge</code>   <b>R4 node types:</b> <code>cache.r4.large</code>, <code>cache.r4.xlarge</code>, <code>cache.r4.2xlarge</code>, <code>cache.r4.4xlarge</code>, <code>cache.r4.8xlarge</code>, <code>cache.r4.16xlarge</code>  </li> <li> Previous generation: (not recommended)  <b>M2 node types:</b> <code>cache.m2.xlarge</code>, <code>cache.m2.2xlarge</code>, <code>cache.m2.4xlarge</code>   <b>R3 node types:</b> <code>cache.r3.large</code>, <code>cache.r3.xlarge</code>, <code>cache.r3.2xlarge</code>, <code>cache.r3.4xlarge</code>, <code>cache.r3.8xlarge</code>  </li> </ul> </li> </ul>  <b>Additional node type info</b>  <ul> <li> All current generation instance types are created in Amazon VPC by default. </li> <li> Redis append-only files (AOF) are not supported for T1 or T2 instances. </li> <li> Redis Multi-AZ with automatic failover is not supported on T1 instances. </li> <li> Redis configuration variables <code>appendonly</code> and <code>appendfsync</code> are not supported on Redis version 2.8.22 and later. </li> </ul><br/>ReservedCacheNodesOfferingId:  The offering identifier filter value. Use this parameter to show only purchased reservations matching the specified offering identifier.<br/>ReservedCacheNodeId:  The reserved cache node identifier filter value. Use this parameter to show only the reservation that matches the specified reservation ID.<br/>
        :return: ReservedCacheNodes:  A list of reserved cache nodes. Each element in the list contains detailed information about one node.<br/>Marker:  Provides an identifier to allow retrieval of paginated results.<br/>        """
        pass

    def describe_reserved_cache_nodes_offerings(self, ReservedCacheNodesOfferingId: str=None, CacheNodeType: str=None, Duration: str=None, ProductDescription: str=None, OfferingType: str=None, MaxRecords: int=None, Marker: str=None) -> ReservedCacheNodesOfferingMessage:
        """Lists available reserved cache node offerings.

        :param self:
        :param Marker:  An optional marker returned from a prior request. Use this marker for pagination of results from this operation. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:  The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a marker is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: minimum 20; maximum 100.<br/>OfferingType:  The offering type filter value. Use this parameter to show only the available offerings matching the specified offering type. Valid Values: <code>"Light Utilization"|"Medium Utilization"|"Heavy Utilization"</code> <br/>ProductDescription:  The product description filter value. Use this parameter to show only the available offerings matching the specified product description.<br/>Duration:  Duration filter value, specified in years or seconds. Use this parameter to show only reservations for a given duration. Valid Values: <code>1 | 3 | 31536000 | 94608000</code> <br/>CacheNodeType:  The cache node type filter value. Use this parameter to show only the available offerings matching the specified cache node type. The following node types are supported by ElastiCache. Generally speaking, the current generation types provide more memory and computational power at lower cost when compared to their equivalent previous generation counterparts. <ul> <li> General purpose: <ul> <li> Current generation:   <b>M5 node types:</b> <code>cache.m5.large</code>, <code>cache.m5.xlarge</code>, <code>cache.m5.2xlarge</code>, <code>cache.m5.4xlarge</code>, <code>cache.m5.12xlarge</code>, <code>cache.m5.24xlarge</code>   <b>M4 node types:</b> <code>cache.m4.large</code>, <code>cache.m4.xlarge</code>, <code>cache.m4.2xlarge</code>, <code>cache.m4.4xlarge</code>, <code>cache.m4.10xlarge</code>   <b>T2 node types:</b> <code>cache.t2.micro</code>, <code>cache.t2.small</code>, <code>cache.t2.medium</code>  </li> <li> Previous generation: (not recommended)  <b>T1 node types:</b> <code>cache.t1.micro</code>   <b>M1 node types:</b> <code>cache.m1.small</code>, <code>cache.m1.medium</code>, <code>cache.m1.large</code>, <code>cache.m1.xlarge</code>   <b>M3 node types:</b> <code>cache.m3.medium</code>, <code>cache.m3.large</code>, <code>cache.m3.xlarge</code>, <code>cache.m3.2xlarge</code>  </li> </ul> </li> <li> Compute optimized: <ul> <li> Previous generation: (not recommended)  <b>C1 node types:</b> <code>cache.c1.xlarge</code>  </li> </ul> </li> <li> Memory optimized: <ul> <li> Current generation:   <b>R5 node types:</b> <code>cache.r5.large</code>, <code>cache.r5.xlarge</code>, <code>cache.r5.2xlarge</code>, <code>cache.r5.4xlarge</code>, <code>cache.r5.12xlarge</code>, <code>cache.r5.24xlarge</code>   <b>R4 node types:</b> <code>cache.r4.large</code>, <code>cache.r4.xlarge</code>, <code>cache.r4.2xlarge</code>, <code>cache.r4.4xlarge</code>, <code>cache.r4.8xlarge</code>, <code>cache.r4.16xlarge</code>  </li> <li> Previous generation: (not recommended)  <b>M2 node types:</b> <code>cache.m2.xlarge</code>, <code>cache.m2.2xlarge</code>, <code>cache.m2.4xlarge</code>   <b>R3 node types:</b> <code>cache.r3.large</code>, <code>cache.r3.xlarge</code>, <code>cache.r3.2xlarge</code>, <code>cache.r3.4xlarge</code>, <code>cache.r3.8xlarge</code>  </li> </ul> </li> </ul>  <b>Additional node type info</b>  <ul> <li> All current generation instance types are created in Amazon VPC by default. </li> <li> Redis append-only files (AOF) are not supported for T1 or T2 instances. </li> <li> Redis Multi-AZ with automatic failover is not supported on T1 instances. </li> <li> Redis configuration variables <code>appendonly</code> and <code>appendfsync</code> are not supported on Redis version 2.8.22 and later. </li> </ul><br/>ReservedCacheNodesOfferingId:  The offering identifier filter value. Use this parameter to show only the available offering that matches the specified reservation identifier. Example: <code>438012d3-4052-4cc7-b2e3-8d3372e0e706</code> <br/>
        :return: ReservedCacheNodesOfferings:  A list of reserved cache node offerings. Each element in the list contains detailed information about one offering.<br/>Marker:  Provides an identifier to allow retrieval of paginated results.<br/>        """
        pass

    def describe_service_updates(self, ServiceUpdateName: str=None, ServiceUpdateStatus: List[ServiceUpdateStatus]=None, MaxRecords: int=None, Marker: str=None) -> ServiceUpdatesMessage:
        """Returns details of the service updates

        :param self:
        :param Marker:  An optional marker returned from a prior request. Use this marker for pagination of results from this operation. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:  The maximum number of records to include in the response<br/>ServiceUpdateStatus:  The status of the service update<br/>ServiceUpdateName:  The unique ID of the service update<br/>
        :return: ServiceUpdates:  A list of service updates<br/>Marker:  An optional marker returned from a prior request. Use this marker for pagination of results from this operation. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>        """
        pass

    def describe_snapshots(self, ReplicationGroupId: str=None, CacheClusterId: str=None, SnapshotName: str=None, SnapshotSource: str=None, Marker: str=None, MaxRecords: int=None, ShowNodeGroupConfig: BooleanOptional=None) -> DescribeSnapshotsListMessage:
        """Returns information about cluster or replication group snapshots. By default, <code>DescribeSnapshots</code> lists all of your snapshots; it can optionally describe a single snapshot, or just the snapshots associated with a particular cache cluster. <note> This operation is valid for Redis only. </note>

        :param self:
        :param ShowNodeGroupConfig:  A Boolean value which if true, the node group (shard) configuration is included in the snapshot description.<br/>MaxRecords:  The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a marker is included in the response so that the remaining results can be retrieved. Default: 50 Constraints: minimum 20; maximum 50.<br/>Marker:  An optional marker returned from a prior request. Use this marker for pagination of results from this operation. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>SnapshotSource:  If set to <code>system</code>, the output shows snapshots that were automatically created by ElastiCache. If set to <code>user</code> the output shows snapshots that were manually created. If omitted, the output shows both automatically and manually created snapshots.<br/>SnapshotName:  A user-supplied name of the snapshot. If this parameter is specified, only this snapshot are described.<br/>CacheClusterId:  A user-supplied cluster identifier. If this parameter is specified, only snapshots associated with that specific cluster are described.<br/>ReplicationGroupId:  A user-supplied replication group identifier. If this parameter is specified, only snapshots associated with that specific replication group are described.<br/>
        :return: Snapshots:  A list of snapshots. Each item in the list contains detailed information about one snapshot.<br/>Marker:  An optional marker returned from a prior request. Use this marker for pagination of results from this operation. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>        """
        pass

    def describe_update_actions(self, ServiceUpdateName: str=None, ReplicationGroupIds: List[String]=None, ServiceUpdateStatus: List[ServiceUpdateStatus]=None, ServiceUpdateTimeRange: TimeRangeFilter=None, UpdateActionStatus: List[UpdateActionStatus]=None, ShowNodeLevelUpdateStatus: BooleanOptional=None, MaxRecords: int=None, Marker: str=None) -> UpdateActionsMessage:
        """Returns details of the update actions 

        :param self:
        :param Marker:  An optional marker returned from a prior request. Use this marker for pagination of results from this operation. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:  The maximum number of records to include in the response<br/>ShowNodeLevelUpdateStatus:  Dictates whether to include node level update status in the response <br/>UpdateActionStatus:  The status of the update action.<br/>ServiceUpdateTimeRange:  The range of time specified to search for service updates that are in available status:<br/>ServiceUpdateStatus:  The status of the service update<br/>ReplicationGroupIds:  The replication group IDs<br/>ServiceUpdateName:  The unique ID of the service update<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> The end time of the time range filter<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTime</i> The start time of the time range filter<br/>
        :return: UpdateActions:  Returns a list of update actions<br/>Marker:  An optional marker returned from a prior request. Use this marker for pagination of results from this operation. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>        """
        pass

    def increase_replica_count(self, ApplyImmediately: Boolean, ReplicationGroupId: str, NewReplicaCount: int=None, ReplicaConfiguration: List[ConfigureShard]=None) -> IncreaseReplicaCountResult:
        """Dynamically increases the number of replics in a Redis (cluster mode disabled) replication group or the number of replica nodes in one or more node groups (shards) of a Redis (cluster mode enabled) replication group. This operation is performed with no cluster down time.

        :param self:
        :param ApplyImmediately:  If <code>True</code>, the number of replica nodes is increased immediately. <code>ApplyImmediately=False</code> is not currently supported.<br/>ReplicaConfiguration:  A list of <code>ConfigureShard</code> objects that can be used to configure each shard in a Redis (cluster mode enabled) replication group. The <code>ConfigureShard</code> has three members: <code>NewReplicaCount</code>, <code>NodeGroupId</code>, and <code>PreferredAvailabilityZones</code>.<br/>NewReplicaCount:  The number of read replica nodes you want at the completion of this operation. For Redis (cluster mode disabled) replication groups, this is the number of replica nodes in the replication group. For Redis (cluster mode enabled) replication groups, this is the number of replica nodes in each of the replication group's node groups.<br/>ReplicationGroupId:  The id of the replication group to which you want to add replica nodes.<br/>
        :return: ReplicationGroup:  Contains all of the attributes of a specific Redis replication group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AtRestEncryptionEnabled</i> A flag that enables encryption at-rest when set to <code>true</code>. You cannot modify the value of <code>AtRestEncryptionEnabled</code> after the cluster is created. To enable encryption at-rest on a cluster you must set <code>AtRestEncryptionEnabled</code> to <code>true</code> when you create a cluster.  <b>Required:</b> Only available when creating a replication group in an Amazon VPC using redis version <code>3.2.6</code>, <code>4.x</code> or later. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitEncryptionEnabled</i> A flag that enables in-transit encryption when set to <code>true</code>. You cannot modify the value of <code>TransitEncryptionEnabled</code> after the cluster is created. To enable in-transit encryption on a cluster you must set <code>TransitEncryptionEnabled</code> to <code>true</code> when you create a cluster.  <b>Required:</b> Only available when creating a replication group in an Amazon VPC using redis version <code>3.2.6</code>, <code>4.x</code> or later. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthTokenEnabled</i> A flag that enables using an <code>AuthToken</code> (password) when issuing Redis commands. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodeType</i> The name of the compute and memory capacity node type for each node in the replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterEnabled</i> A flag indicating whether or not this replication group is cluster enabled; i.e., whether its data can be partitioned across multiple shards (API/CLI: node groups). Valid values: <code>true</code> | <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotWindow</i> The daily time range (in UTC) during which ElastiCache begins taking a daily snapshot of your node group (shard). Example: <code>05:00-09:00</code>  If you do not specify this parameter, ElastiCache automatically chooses an appropriate time range. <note> This parameter is only valid if the <code>Engine</code> parameter is <code>redis</code>. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotRetentionLimit</i> The number of days for which ElastiCache retains automatic cluster snapshots before deleting them. For example, if you set <code>SnapshotRetentionLimit</code> to 5, a snapshot that was taken today is retained for 5 days before being deleted. <important>  If the value of <code>SnapshotRetentionLimit</code> is set to zero (0), backups are turned off. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigurationEndpoint</i> The configuration endpoint for this replication group. Use the configuration endpoint to connect to this replication group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ConfigurationEndpoint</b>: The configuration endpoint for this replication group. Use the configuration endpoint to connect to this replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomaticFailover</i> Indicates the status of Multi-AZ with automatic failover for this Redis replication group. Amazon ElastiCache for Redis does not support Multi-AZ with automatic failover on: <ul> <li> Redis versions earlier than 2.8.6. </li> <li> Redis (cluster mode disabled): T1 node types. </li> <li> Redis (cluster mode enabled): T1 node types. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshottingClusterId</i> The cluster ID that is used as the daily snapshot source for the replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeGroups</i> A list of node groups in this replication group. For Redis (cluster mode disabled) replication groups, this is a single-element list. For Redis (cluster mode enabled) replication groups, the list contains an entry for each node group (shard).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MemberClusters</i> The names of all the cache clusters that are part of this replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> A group of settings to be applied to the replication group, either immediately or during the next maintenance window.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: A group of settings to be applied to the replication group, either immediately or during the next maintenance window.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current state of this replication group - <code>creating</code>, <code>available</code>, <code>modifying</code>, <code>deleting</code>, <code>create-failed</code>, <code>snapshotting</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The user supplied description of the replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationGroupId</i> The identifier for the replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomaticFailoverStatus</i> Indicates the status of Multi-AZ with automatic failover for this Redis replication group. Amazon ElastiCache for Redis does not support Multi-AZ with automatic failover on: <ul> <li> Redis versions earlier than 2.8.6. </li> <li> Redis (cluster mode disabled): T1 node types. </li> <li> Redis (cluster mode enabled): T1 node types. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PrimaryClusterId</i> The primary cluster ID that is applied immediately (if <code>--apply-immediately</code> was specified), or during the next maintenance window.<br/>The status of an online resharding operation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Resharding</b>: The status of an online resharding operation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port number that the cache engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS hostname of the cache node.<br/>        """
        pass

    def list_allowed_node_type_modifications(self, CacheClusterId: str=None, ReplicationGroupId: str=None) -> AllowedNodeTypeModificationsMessage:
        """Lists all available node types that you can scale your Redis cluster's or replication group's current node type up to. When you use the <code>ModifyCacheCluster</code> or <code>ModifyReplicationGroup</code> operations to scale up your cluster or replication group, the value of the <code>CacheNodeType</code> parameter must be one of the node types returned by this operation.

        :param self:
        :param ReplicationGroupId:  The name of the replication group want to scale up to a larger node type. ElastiCache uses the replication group id to identify the current node type being used by this replication group, and from that to create a list of node types you can scale up to. <important> You must provide a value for either the <code>CacheClusterId</code> or the <code>ReplicationGroupId</code>. </important><br/>CacheClusterId:  The name of the cluster you want to scale up to a larger node instanced type. ElastiCache uses the cluster id to identify the current node type of this cluster and from that to create a list of node types you can scale up to. <important> You must provide a value for either the <code>CacheClusterId</code> or the <code>ReplicationGroupId</code>. </important><br/>
        :return: ScaleUpModifications:  A string list, each element of which specifies a cache node type which you can use to scale your cluster or replication group. When scaling up a Redis cluster or replication group using <code>ModifyCacheCluster</code> or <code>ModifyReplicationGroup</code>, use a value from this list for the <code>CacheNodeType</code> parameter.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceName: str) -> TagListMessage:
        """Lists all cost allocation tags currently on the named resource. A <code>cost allocation tag</code> is a key-value pair where the key is case-sensitive and the value is optional. You can use cost allocation tags to categorize and track your AWS costs. If the cluster is not in the <i>available</i> state, <code>ListTagsForResource</code> returns an error. You can have a maximum of 50 cost allocation tags on an ElastiCache resource. For more information, see <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/red-ug/Tagging.html">Monitoring Costs with Tags</a>.

        :param self:
        :param ResourceName:  The Amazon Resource Name (ARN) of the resource for which you want the list of tags, for example <code>arn:aws:elasticache:us-west-2:0123456789:cluster:myCluster</code> or <code>arn:aws:elasticache:us-west-2:0123456789:snapshot:mySnapshot</code>. For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>
        :return: TagList:  A list of cost allocation tags as key-value pairs.<br/>        """
        pass

    def modify_cache_cluster(self, CacheClusterId: str, NumCacheNodes: int=None, CacheNodeIdsToRemove: List[String]=None, AZMode: str=None, NewAvailabilityZones: List[String]=None, CacheSecurityGroupNames: List[String]=None, SecurityGroupIds: List[String]=None, PreferredMaintenanceWindow: str=None, NotificationTopicArn: str=None, CacheParameterGroupName: str=None, NotificationTopicStatus: str=None, ApplyImmediately: Boolean=None, EngineVersion: str=None, AutoMinorVersionUpgrade: BooleanOptional=None, SnapshotRetentionLimit: int=None, SnapshotWindow: str=None, CacheNodeType: str=None) -> ModifyCacheClusterResult:
        """Modifies the settings for a cluster. You can use this operation to change one or more cluster configuration parameters by specifying the parameters and the new values.

        :param self:
        :param CacheNodeType:  A valid cache node type that you want to scale this cluster up to.<br/>SnapshotWindow:  The daily time range (in UTC) during which ElastiCache begins taking a daily snapshot of your cluster. <br/>SnapshotRetentionLimit:  The number of days for which ElastiCache retains automatic cluster snapshots before deleting them. For example, if you set <code>SnapshotRetentionLimit</code> to 5, a snapshot that was taken today is retained for 5 days before being deleted. <note> If the value of <code>SnapshotRetentionLimit</code> is set to zero (0), backups are turned off. </note><br/>AutoMinorVersionUpgrade:  This parameter is currently disabled.<br/>EngineVersion:  The upgraded version of the cache engine to be run on the cache nodes.  <b>Important:</b> You can upgrade to a newer engine version (see <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/red-ug/SelectEngine.html#VersionManagement">Selecting a Cache Engine and Version</a>), but you cannot downgrade to an earlier engine version. If you want to use an earlier engine version, you must delete the existing cluster and create it anew with the earlier engine version. <br/>ApplyImmediately:  If <code>true</code>, this parameter causes the modifications in this request and any pending modifications to be applied, asynchronously and as soon as possible, regardless of the <code>PreferredMaintenanceWindow</code> setting for the cluster. If <code>false</code>, changes to the cluster are applied on the next maintenance reboot, or the next failure reboot, whichever occurs first. <important> If you perform a <code>ModifyCacheCluster</code> before a pending modification is applied, the pending modification is replaced by the newer modification. </important> Valid values: <code>true</code> | <code>false</code>  Default: <code>false</code> <br/>NotificationTopicStatus:  The status of the Amazon SNS notification topic. Notifications are sent only if the status is <code>active</code>. Valid values: <code>active</code> | <code>inactive</code> <br/>CacheParameterGroupName:  The name of the cache parameter group to apply to this cluster. This change is asynchronously applied as soon as possible for parameters when the <code>ApplyImmediately</code> parameter is specified as <code>true</code> for this request.<br/>NotificationTopicArn:  The Amazon Resource Name (ARN) of the Amazon SNS topic to which notifications are sent. <note> The Amazon SNS topic owner must be same as the cluster owner. </note><br/>PreferredMaintenanceWindow:  Specifies the weekly time range during which maintenance on the cluster is performed. It is specified as a range in the format ddd:hh24:mi-ddd:hh24:mi (24H Clock UTC). The minimum maintenance window is a 60 minute period. Valid values for <code>ddd</code> are: <ul> <li>  <code>sun</code>  </li> <li>  <code>mon</code>  </li> <li>  <code>tue</code>  </li> <li>  <code>wed</code>  </li> <li>  <code>thu</code>  </li> <li>  <code>fri</code>  </li> <li>  <code>sat</code>  </li> </ul> Example: <code>sun:23:00-mon:01:30</code> <br/>SecurityGroupIds:  Specifies the VPC Security Groups associated with the cluster. This parameter can be used only with clusters that are created in an Amazon Virtual Private Cloud (Amazon VPC).<br/>CacheSecurityGroupNames:  A list of cache security group names to authorize on this cluster. This change is asynchronously applied as soon as possible. You can use this parameter only with clusters that are created outside of an Amazon Virtual Private Cloud (Amazon VPC). Constraints: Must contain no more than 255 alphanumeric characters. Must not be "Default".<br/>NewAvailabilityZones:  The list of Availability Zones where the new Memcached cache nodes are created. This parameter is only valid when <code>NumCacheNodes</code> in the request is greater than the sum of the number of active cache nodes and the number of cache nodes pending creation (which may be zero). The number of Availability Zones supplied in this list must match the cache nodes being added in this request. This option is only supported on Memcached clusters. Scenarios: <ul> <li>  <b>Scenario 1:</b> You have 3 active nodes and wish to add 2 nodes. Specify <code>NumCacheNodes=5</code> (3 + 2) and optionally specify two Availability Zones for the two new nodes. </li> <li>  <b>Scenario 2:</b> You have 3 active nodes and 2 nodes pending creation (from the scenario 1 call) and want to add 1 more node. Specify <code>NumCacheNodes=6</code> ((3 + 2) + 1) and optionally specify an Availability Zone for the new node. </li> <li>  <b>Scenario 3:</b> You want to cancel all pending operations. Specify <code>NumCacheNodes=3</code> to cancel all pending operations. </li> </ul> The Availability Zone placement of nodes pending creation cannot be modified. If you wish to cancel any nodes pending creation, add 0 nodes by setting <code>NumCacheNodes</code> to the number of current nodes. If <code>cross-az</code> is specified, existing Memcached nodes remain in their current Availability Zone. Only newly created nodes can be located in different Availability Zones. For guidance on how to move existing Memcached nodes to different Availability Zones, see the <b>Availability Zone Considerations</b> section of <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/mem-ug/CacheNodes.SupportedTypes.html">Cache Node Considerations for Memcached</a>.  <b>Impact of new add/remove requests upon pending requests</b>  <ul> <li> Scenario-1 <ul> <li> Pending Action: Delete </li> <li> New Request: Delete </li> <li> Result: The new delete, pending or immediate, replaces the pending delete. </li> </ul> </li> <li> Scenario-2 <ul> <li> Pending Action: Delete </li> <li> New Request: Create </li> <li> Result: The new create, pending or immediate, replaces the pending delete. </li> </ul> </li> <li> Scenario-3 <ul> <li> Pending Action: Create </li> <li> New Request: Delete </li> <li> Result: The new delete, pending or immediate, replaces the pending create. </li> </ul> </li> <li> Scenario-4 <ul> <li> Pending Action: Create </li> <li> New Request: Create </li> <li> Result: The new create is added to the pending create. <important>  <b>Important:</b> If the new create request is <b>Apply Immediately - Yes</b>, all creates are performed immediately. If the new create request is <b>Apply Immediately - No</b>, all creates are pending. </important> </li> </ul> </li> </ul><br/>AZMode:  Specifies whether the new nodes in this Memcached cluster are all created in a single Availability Zone or created across multiple Availability Zones. Valid values: <code>single-az</code> | <code>cross-az</code>. This option is only supported for Memcached clusters. <note> You cannot specify <code>single-az</code> if the Memcached cluster already has cache nodes in different Availability Zones. If <code>cross-az</code> is specified, existing Memcached nodes remain in their current Availability Zone. Only newly created nodes are located in different Availability Zones. For instructions on how to move existing Memcached nodes to different Availability Zones, see the <b>Availability Zone Considerations</b> section of <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/mem-ug/CacheNodes.SupportedTypes.html">Cache Node Considerations for Memcached</a>. </note><br/>CacheNodeIdsToRemove:  A list of cache node IDs to be removed. A node ID is a numeric identifier (0001, 0002, etc.). This parameter is only valid when <code>NumCacheNodes</code> is less than the existing number of cache nodes. The number of cache node IDs supplied in this parameter must match the difference between the existing number of cache nodes in the cluster or pending cache nodes, whichever is greater, and the value of <code>NumCacheNodes</code> in the request. For example: If you have 3 active cache nodes, 7 pending cache nodes, and the number of cache nodes in this <code>ModifyCacheCluster</code> call is 5, you must list 2 (7 - 5) cache node IDs to remove.<br/>NumCacheNodes:  The number of cache nodes that the cluster should have. If the value for <code>NumCacheNodes</code> is greater than the sum of the number of current cache nodes and the number of cache nodes pending creation (which may be zero), more nodes are added. If the value is less than the number of existing cache nodes, nodes are removed. If the value is equal to the number of current cache nodes, any pending add or remove requests are canceled. If you are removing cache nodes, you must use the <code>CacheNodeIdsToRemove</code> parameter to provide the IDs of the specific cache nodes to remove. For clusters running Redis, this value must be 1. For clusters running Memcached, this value must be between 1 and 20. <note> Adding or removing Memcached cache nodes can be applied immediately or as a pending operation (see <code>ApplyImmediately</code>). A pending operation to modify the number of cache nodes in a cluster during its maintenance window, whether by adding or removing nodes in accordance with the scale out architecture, is not queued. The customer's latest request to add or remove nodes to the cluster overrides any previous pending operations to modify the number of cache nodes in the cluster. For example, a request to remove 2 nodes would override a previous pending operation to remove 3 nodes. Similarly, a request to add 2 nodes would override a previous pending operation to remove 3 nodes and vice versa. As Memcached cache nodes may now be provisioned in different Availability Zones with flexible cache node placement, a request to add nodes does not automatically override a previous pending operation to add nodes. The customer can modify the previous pending operation to add more nodes or explicitly cancel the pending request and retry the new request. To cancel pending operations to modify the number of cache nodes in a cluster, use the <code>ModifyCacheCluster</code> request and set <code>NumCacheNodes</code> equal to the number of cache nodes currently in the cluster. </note><br/>CacheClusterId:  The cluster identifier. This value is stored as a lowercase string.<br/>
        :return: CacheCluster:  Contains all of the attributes of a specific cluster.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AtRestEncryptionEnabled</i> A flag that enables encryption at-rest when set to <code>true</code>. You cannot modify the value of <code>AtRestEncryptionEnabled</code> after the cluster is created. To enable at-rest encryption on a cluster you must set <code>AtRestEncryptionEnabled</code> to <code>true</code> when you create a cluster.  <b>Required:</b> Only available when creating a replication group in an Amazon VPC using redis version <code>3.2.6</code>, <code>4.x</code> or later. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitEncryptionEnabled</i> A flag that enables in-transit encryption when set to <code>true</code>. You cannot modify the value of <code>TransitEncryptionEnabled</code> after the cluster is created. To enable in-transit encryption on a cluster you must set <code>TransitEncryptionEnabled</code> to <code>true</code> when you create a cluster.  <b>Required:</b> Only available when creating a replication group in an Amazon VPC using redis version <code>3.2.6</code>, <code>4.x</code> or later. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthTokenEnabled</i> A flag that enables using an <code>AuthToken</code> (password) when issuing Redis commands. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotWindow</i> The daily time range (in UTC) during which ElastiCache begins taking a daily snapshot of your cluster. Example: <code>05:00-09:00</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotRetentionLimit</i> The number of days for which ElastiCache retains automatic cluster snapshots before deleting them. For example, if you set <code>SnapshotRetentionLimit</code> to 5, a snapshot that was taken today is retained for 5 days before being deleted. <important>  If the value of SnapshotRetentionLimit is set to zero (0), backups are turned off. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationGroupId</i> The replication group to which this cluster belongs. If this field is empty, the cluster is not associated with any replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroups</i> A list of VPC Security Groups associated with the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> This parameter is currently disabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodes</i> A list of cache nodes that are members of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheSubnetGroupName</i> The name of the cache subnet group associated with the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheParameterGroup</i> Status of the cache parameter group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CacheParameterGroup</b>: Status of the cache parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheSecurityGroups</i> A list of cache security group elements, composed of name and status sub-elements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationConfiguration</i> Describes a notification topic and its status. Notification topics are used for publishing ElastiCache events to subscribers using Amazon Simple Notification Service (SNS). &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NotificationConfiguration</b>: Describes a notification topic and its status. Notification topics are used for publishing ElastiCache events to subscribers using Amazon Simple Notification Service (SNS). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> A group of settings that are applied to the cluster in the future, or that are currently being applied.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: A group of settings that are applied to the cluster in the future, or that are currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which maintenance on the cluster is performed. It is specified as a range in the format ddd:hh24:mi-ddd:hh24:mi (24H Clock UTC). The minimum maintenance window is a 60 minute period. Valid values for <code>ddd</code> are: <ul> <li>  <code>sun</code>  </li> <li>  <code>mon</code>  </li> <li>  <code>tue</code>  </li> <li>  <code>wed</code>  </li> <li>  <code>thu</code>  </li> <li>  <code>fri</code>  </li> <li>  <code>sat</code>  </li> </ul> Example: <code>sun:23:00-mon:01:30</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheClusterCreateTime</i> The date and time when the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredAvailabilityZone</i> The name of the Availability Zone in which the cluster is located or "Multiple" if the cache nodes are located in different Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumCacheNodes</i> The number of cache nodes in the cluster. For clusters running Redis, this value must be 1. For clusters running Memcached, this value must be between 1 and 20.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheClusterStatus</i> The current state of this cluster, one of the following values: <code>available</code>, <code>creating</code>, <code>deleted</code>, <code>deleting</code>, <code>incompatible-network</code>, <code>modifying</code>, <code>rebooting cluster nodes</code>, <code>restore-failed</code>, or <code>snapshotting</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> The version of the cache engine that is used in this cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> The name of the cache engine (<code>memcached</code> or <code>redis</code>) to be used for this cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodeType</i> The name of the compute and memory capacity node type for the cluster. The following node types are supported by ElastiCache. Generally speaking, the current generation types provide more memory and computational power at lower cost when compared to their equivalent previous generation counterparts. <ul> <li> General purpose: <ul> <li> Current generation:   <b>M5 node types:</b> <code>cache.m5.large</code>, <code>cache.m5.xlarge</code>, <code>cache.m5.2xlarge</code>, <code>cache.m5.4xlarge</code>, <code>cache.m5.12xlarge</code>, <code>cache.m5.24xlarge</code>   <b>M4 node types:</b> <code>cache.m4.large</code>, <code>cache.m4.xlarge</code>, <code>cache.m4.2xlarge</code>, <code>cache.m4.4xlarge</code>, <code>cache.m4.10xlarge</code>   <b>T2 node types:</b> <code>cache.t2.micro</code>, <code>cache.t2.small</code>, <code>cache.t2.medium</code>  </li> <li> Previous generation: (not recommended)  <b>T1 node types:</b> <code>cache.t1.micro</code>   <b>M1 node types:</b> <code>cache.m1.small</code>, <code>cache.m1.medium</code>, <code>cache.m1.large</code>, <code>cache.m1.xlarge</code>   <b>M3 node types:</b> <code>cache.m3.medium</code>, <code>cache.m3.large</code>, <code>cache.m3.xlarge</code>, <code>cache.m3.2xlarge</code>  </li> </ul> </li> <li> Compute optimized: <ul> <li> Previous generation: (not recommended)  <b>C1 node types:</b> <code>cache.c1.xlarge</code>  </li> </ul> </li> <li> Memory optimized: <ul> <li> Current generation:   <b>R5 node types:</b> <code>cache.r5.large</code>, <code>cache.r5.xlarge</code>, <code>cache.r5.2xlarge</code>, <code>cache.r5.4xlarge</code>, <code>cache.r5.12xlarge</code>, <code>cache.r5.24xlarge</code>   <b>R4 node types:</b> <code>cache.r4.large</code>, <code>cache.r4.xlarge</code>, <code>cache.r4.2xlarge</code>, <code>cache.r4.4xlarge</code>, <code>cache.r4.8xlarge</code>, <code>cache.r4.16xlarge</code>  </li> <li> Previous generation: (not recommended)  <b>M2 node types:</b> <code>cache.m2.xlarge</code>, <code>cache.m2.2xlarge</code>, <code>cache.m2.4xlarge</code>   <b>R3 node types:</b> <code>cache.r3.large</code>, <code>cache.r3.xlarge</code>, <code>cache.r3.2xlarge</code>, <code>cache.r3.4xlarge</code>, <code>cache.r3.8xlarge</code>  </li> </ul> </li> </ul>  <b>Additional node type info</b>  <ul> <li> All current generation instance types are created in Amazon VPC by default. </li> <li> Redis append-only files (AOF) are not supported for T1 or T2 instances. </li> <li> Redis Multi-AZ with automatic failover is not supported on T1 instances. </li> <li> Redis configuration variables <code>appendonly</code> and <code>appendfsync</code> are not supported on Redis version 2.8.22 and later. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientDownloadLandingPage</i> The URL of the web page where you can download the latest ElastiCache client library.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigurationEndpoint</i> Represents a Memcached cluster endpoint which, if Automatic Discovery is enabled on the cluster, can be used by an application to connect to any node in the cluster. The configuration endpoint will always have <code>.cfg</code> in it. Example: <code>mem-3.9dvc4r<u>.cfg</u>.usw2.cache.amazonaws.com:11211</code> &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ConfigurationEndpoint</b>: Represents a Memcached cluster endpoint which, if Automatic Discovery is enabled on the cluster, can be used by an application to connect to any node in the cluster. The configuration endpoint will always have <code>.cfg</code> in it. Example: <code>mem-3.9dvc4r<u>.cfg</u>.usw2.cache.amazonaws.com:11211</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheClusterId</i> The user-supplied identifier of the cluster. This identifier is a unique key that identifies a cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port number that the cache engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS hostname of the cache node.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodeType</i> The cache node type that this cluster or replication group is scaled to.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> The new cache engine version that the cluster runs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodeIdsToRemove</i> A list of cache node IDs that are being removed (or will be removed) from the cluster. A node ID is a 4-digit numeric identifier (0001, 0002, etc.).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumCacheNodes</i> The new number of cache nodes for the cluster. For clusters running Redis, this value must be 1. For clusters running Memcached, this value must be between 1 and 20.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicStatus</i> The current state of the topic.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicArn</i> The Amazon Resource Name (ARN) that identifies the topic.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodeIdsToReboot</i> A list of the cache node IDs which need to be rebooted for parameter changes to be applied. A node ID is a numeric identifier (0001, 0002, etc.).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterApplyStatus</i> The status of parameter updates.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheParameterGroupName</i> The name of the cache parameter group.<br/>        """
        pass

    def modify_cache_parameter_group(self, ParameterNameValues: List[ParameterNameValue], CacheParameterGroupName: str) -> CacheParameterGroupNameMessage:
        """Modifies the parameters of a cache parameter group. You can modify up to 20 parameters in a single request by submitting a list parameter name and value pairs.

        :param self:
        :param ParameterNameValues:  An array of parameter names and values for the parameter update. You must supply at least one parameter name and value; subsequent arguments are optional. A maximum of 20 parameters may be modified per request.<br/>CacheParameterGroupName:  The name of the cache parameter group to modify.<br/>
        :return: CacheParameterGroupName:  The name of the cache parameter group.<br/>        """
        pass

    def modify_cache_subnet_group(self, CacheSubnetGroupName: str, CacheSubnetGroupDescription: str=None, SubnetIds: List[String]=None) -> ModifyCacheSubnetGroupResult:
        """Modifies an existing cache subnet group.

        :param self:
        :param SubnetIds:  The EC2 subnet IDs for the cache subnet group.<br/>CacheSubnetGroupDescription:  A description of the cache subnet group.<br/>CacheSubnetGroupName:  The name for the cache subnet group. This value is stored as a lowercase string. Constraints: Must contain no more than 255 alphanumeric characters or hyphens. Example: <code>mysubnetgroup</code> <br/>
        :return: CacheSubnetGroup:  Represents the output of one of the following operations: <ul> <li>  <code>CreateCacheSubnetGroup</code>  </li> <li>  <code>ModifyCacheSubnetGroup</code>  </li> </ul>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> A list of subnets associated with the cache subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The Amazon Virtual Private Cloud identifier (VPC ID) of the cache subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheSubnetGroupDescription</i> The description of the cache subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheSubnetGroupName</i> The name of the cache subnet group.<br/>        """
        pass

    def modify_replication_group(self, ReplicationGroupId: str, ReplicationGroupDescription: str=None, PrimaryClusterId: str=None, SnapshottingClusterId: str=None, AutomaticFailoverEnabled: BooleanOptional=None, CacheSecurityGroupNames: List[String]=None, SecurityGroupIds: List[String]=None, PreferredMaintenanceWindow: str=None, NotificationTopicArn: str=None, CacheParameterGroupName: str=None, NotificationTopicStatus: str=None, ApplyImmediately: Boolean=None, EngineVersion: str=None, AutoMinorVersionUpgrade: BooleanOptional=None, SnapshotRetentionLimit: int=None, SnapshotWindow: str=None, CacheNodeType: str=None, NodeGroupId: str=None) -> ModifyReplicationGroupResult:
        """Modifies the settings for a replication group. For Redis (cluster mode enabled) clusters, this operation cannot be used to change a cluster's node type or engine version. For more information, see: <ul> <li>  <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/red-ug/scaling-redis-cluster-mode-enabled.html">Scaling for Amazon ElastiCache for Redis (cluster mode enabled)</a> in the ElastiCache User Guide </li> <li>  <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/APIReference/API_ModifyReplicationGroupShardConfiguration.html">ModifyReplicationGroupShardConfiguration</a> in the ElastiCache API Reference </li> </ul> <note> This operation is valid for Redis only. </note>

        :param self:
        :param NodeGroupId:  Deprecated. This parameter is not used.<br/>CacheNodeType:  A valid cache node type that you want to scale this replication group to.<br/>SnapshotWindow:  The daily time range (in UTC) during which ElastiCache begins taking a daily snapshot of the node group (shard) specified by <code>SnapshottingClusterId</code>. Example: <code>05:00-09:00</code>  If you do not specify this parameter, ElastiCache automatically chooses an appropriate time range.<br/>SnapshotRetentionLimit:  The number of days for which ElastiCache retains automatic node group (shard) snapshots before deleting them. For example, if you set <code>SnapshotRetentionLimit</code> to 5, a snapshot that was taken today is retained for 5 days before being deleted.  <b>Important</b> If the value of SnapshotRetentionLimit is set to zero (0), backups are turned off.<br/>AutoMinorVersionUpgrade:  This parameter is currently disabled.<br/>EngineVersion:  The upgraded version of the cache engine to be run on the clusters in the replication group.  <b>Important:</b> You can upgrade to a newer engine version (see <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/red-ug/SelectEngine.html#VersionManagement">Selecting a Cache Engine and Version</a>), but you cannot downgrade to an earlier engine version. If you want to use an earlier engine version, you must delete the existing replication group and create it anew with the earlier engine version. <br/>ApplyImmediately:  If <code>true</code>, this parameter causes the modifications in this request and any pending modifications to be applied, asynchronously and as soon as possible, regardless of the <code>PreferredMaintenanceWindow</code> setting for the replication group. If <code>false</code>, changes to the nodes in the replication group are applied on the next maintenance reboot, or the next failure reboot, whichever occurs first. Valid values: <code>true</code> | <code>false</code>  Default: <code>false</code> <br/>NotificationTopicStatus:  The status of the Amazon SNS notification topic for the replication group. Notifications are sent only if the status is <code>active</code>. Valid values: <code>active</code> | <code>inactive</code> <br/>CacheParameterGroupName:  The name of the cache parameter group to apply to all of the clusters in this replication group. This change is asynchronously applied as soon as possible for parameters when the <code>ApplyImmediately</code> parameter is specified as <code>true</code> for this request.<br/>NotificationTopicArn:  The Amazon Resource Name (ARN) of the Amazon SNS topic to which notifications are sent. <note> The Amazon SNS topic owner must be same as the replication group owner.  </note><br/>PreferredMaintenanceWindow:  Specifies the weekly time range during which maintenance on the cluster is performed. It is specified as a range in the format ddd:hh24:mi-ddd:hh24:mi (24H Clock UTC). The minimum maintenance window is a 60 minute period. Valid values for <code>ddd</code> are: <ul> <li>  <code>sun</code>  </li> <li>  <code>mon</code>  </li> <li>  <code>tue</code>  </li> <li>  <code>wed</code>  </li> <li>  <code>thu</code>  </li> <li>  <code>fri</code>  </li> <li>  <code>sat</code>  </li> </ul> Example: <code>sun:23:00-mon:01:30</code> <br/>SecurityGroupIds:  Specifies the VPC Security Groups associated with the clusters in the replication group. This parameter can be used only with replication group containing clusters running in an Amazon Virtual Private Cloud (Amazon VPC).<br/>CacheSecurityGroupNames:  A list of cache security group names to authorize for the clusters in this replication group. This change is asynchronously applied as soon as possible. This parameter can be used only with replication group containing clusters running outside of an Amazon Virtual Private Cloud (Amazon VPC). Constraints: Must contain no more than 255 alphanumeric characters. Must not be <code>Default</code>.<br/>AutomaticFailoverEnabled:  Determines whether a read replica is automatically promoted to read/write primary if the existing primary encounters a failure. Valid values: <code>true</code> | <code>false</code>  Amazon ElastiCache for Redis does not support Multi-AZ with automatic failover on: <ul> <li> Redis versions earlier than 2.8.6. </li> <li> Redis (cluster mode disabled): T1 node types. </li> <li> Redis (cluster mode enabled): T1 node types. </li> </ul><br/>SnapshottingClusterId:  The cluster ID that is used as the daily snapshot source for the replication group. This parameter cannot be set for Redis (cluster mode enabled) replication groups.<br/>PrimaryClusterId:  For replication groups with a single primary, if this parameter is specified, ElastiCache promotes the specified cluster in the specified replication group to the primary role. The nodes of all other clusters in the replication group are read replicas.<br/>ReplicationGroupDescription:  A description for the replication group. Maximum length is 255 characters.<br/>ReplicationGroupId:  The identifier of the replication group to modify.<br/>
        :return: ReplicationGroup:  Contains all of the attributes of a specific Redis replication group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AtRestEncryptionEnabled</i> A flag that enables encryption at-rest when set to <code>true</code>. You cannot modify the value of <code>AtRestEncryptionEnabled</code> after the cluster is created. To enable encryption at-rest on a cluster you must set <code>AtRestEncryptionEnabled</code> to <code>true</code> when you create a cluster.  <b>Required:</b> Only available when creating a replication group in an Amazon VPC using redis version <code>3.2.6</code>, <code>4.x</code> or later. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitEncryptionEnabled</i> A flag that enables in-transit encryption when set to <code>true</code>. You cannot modify the value of <code>TransitEncryptionEnabled</code> after the cluster is created. To enable in-transit encryption on a cluster you must set <code>TransitEncryptionEnabled</code> to <code>true</code> when you create a cluster.  <b>Required:</b> Only available when creating a replication group in an Amazon VPC using redis version <code>3.2.6</code>, <code>4.x</code> or later. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthTokenEnabled</i> A flag that enables using an <code>AuthToken</code> (password) when issuing Redis commands. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodeType</i> The name of the compute and memory capacity node type for each node in the replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterEnabled</i> A flag indicating whether or not this replication group is cluster enabled; i.e., whether its data can be partitioned across multiple shards (API/CLI: node groups). Valid values: <code>true</code> | <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotWindow</i> The daily time range (in UTC) during which ElastiCache begins taking a daily snapshot of your node group (shard). Example: <code>05:00-09:00</code>  If you do not specify this parameter, ElastiCache automatically chooses an appropriate time range. <note> This parameter is only valid if the <code>Engine</code> parameter is <code>redis</code>. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotRetentionLimit</i> The number of days for which ElastiCache retains automatic cluster snapshots before deleting them. For example, if you set <code>SnapshotRetentionLimit</code> to 5, a snapshot that was taken today is retained for 5 days before being deleted. <important>  If the value of <code>SnapshotRetentionLimit</code> is set to zero (0), backups are turned off. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigurationEndpoint</i> The configuration endpoint for this replication group. Use the configuration endpoint to connect to this replication group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ConfigurationEndpoint</b>: The configuration endpoint for this replication group. Use the configuration endpoint to connect to this replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomaticFailover</i> Indicates the status of Multi-AZ with automatic failover for this Redis replication group. Amazon ElastiCache for Redis does not support Multi-AZ with automatic failover on: <ul> <li> Redis versions earlier than 2.8.6. </li> <li> Redis (cluster mode disabled): T1 node types. </li> <li> Redis (cluster mode enabled): T1 node types. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshottingClusterId</i> The cluster ID that is used as the daily snapshot source for the replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeGroups</i> A list of node groups in this replication group. For Redis (cluster mode disabled) replication groups, this is a single-element list. For Redis (cluster mode enabled) replication groups, the list contains an entry for each node group (shard).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MemberClusters</i> The names of all the cache clusters that are part of this replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> A group of settings to be applied to the replication group, either immediately or during the next maintenance window.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: A group of settings to be applied to the replication group, either immediately or during the next maintenance window.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current state of this replication group - <code>creating</code>, <code>available</code>, <code>modifying</code>, <code>deleting</code>, <code>create-failed</code>, <code>snapshotting</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The user supplied description of the replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationGroupId</i> The identifier for the replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomaticFailoverStatus</i> Indicates the status of Multi-AZ with automatic failover for this Redis replication group. Amazon ElastiCache for Redis does not support Multi-AZ with automatic failover on: <ul> <li> Redis versions earlier than 2.8.6. </li> <li> Redis (cluster mode disabled): T1 node types. </li> <li> Redis (cluster mode enabled): T1 node types. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PrimaryClusterId</i> The primary cluster ID that is applied immediately (if <code>--apply-immediately</code> was specified), or during the next maintenance window.<br/>The status of an online resharding operation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Resharding</b>: The status of an online resharding operation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port number that the cache engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS hostname of the cache node.<br/>        """
        pass

    def modify_replication_group_shard_configuration(self, ApplyImmediately: Boolean, NodeGroupCount: int, ReplicationGroupId: str, ReshardingConfiguration: List[ReshardingConfiguration]=None, NodeGroupsToRemove: List[AllowedNodeGroupId]=None, NodeGroupsToRetain: List[AllowedNodeGroupId]=None) -> ModifyReplicationGroupShardConfigurationResult:
        """Modifies a replication group's shards (node groups) by allowing you to add shards, remove shards, or rebalance the keyspaces among exisiting shards.

        :param self:
        :param NodeGroupsToRetain:  If the value of <code>NodeGroupCount</code> is less than the current number of node groups (shards), then either <code>NodeGroupsToRemove</code> or <code>NodeGroupsToRetain</code> is required. <code>NodeGroupsToRetain</code> is a list of <code>NodeGroupId</code>s to retain in the cluster. ElastiCache for Redis will attempt to remove all node groups except those listed by <code>NodeGroupsToRetain</code> from the cluster.<br/>NodeGroupsToRemove:  If the value of <code>NodeGroupCount</code> is less than the current number of node groups (shards), then either <code>NodeGroupsToRemove</code> or <code>NodeGroupsToRetain</code> is required. <code>NodeGroupsToRemove</code> is a list of <code>NodeGroupId</code>s to remove from the cluster. ElastiCache for Redis will attempt to remove all node groups listed by <code>NodeGroupsToRemove</code> from the cluster.<br/>ReshardingConfiguration:  Specifies the preferred availability zones for each node group in the cluster. If the value of <code>NodeGroupCount</code> is greater than the current number of node groups (shards), you can use this parameter to specify the preferred availability zones of the cluster's shards. If you omit this parameter ElastiCache selects availability zones for you. You can specify this parameter only if the value of <code>NodeGroupCount</code> is greater than the current number of node groups (shards).<br/>ApplyImmediately:  Indicates that the shard reconfiguration process begins immediately. At present, the only permitted value for this parameter is <code>true</code>. Value: true<br/>NodeGroupCount:  The number of node groups (shards) that results from the modification of the shard configuration.<br/>ReplicationGroupId:  The name of the Redis (cluster mode enabled) cluster (replication group) on which the shards are to be configured.<br/>
        :return: ReplicationGroup:  Contains all of the attributes of a specific Redis replication group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AtRestEncryptionEnabled</i> A flag that enables encryption at-rest when set to <code>true</code>. You cannot modify the value of <code>AtRestEncryptionEnabled</code> after the cluster is created. To enable encryption at-rest on a cluster you must set <code>AtRestEncryptionEnabled</code> to <code>true</code> when you create a cluster.  <b>Required:</b> Only available when creating a replication group in an Amazon VPC using redis version <code>3.2.6</code>, <code>4.x</code> or later. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitEncryptionEnabled</i> A flag that enables in-transit encryption when set to <code>true</code>. You cannot modify the value of <code>TransitEncryptionEnabled</code> after the cluster is created. To enable in-transit encryption on a cluster you must set <code>TransitEncryptionEnabled</code> to <code>true</code> when you create a cluster.  <b>Required:</b> Only available when creating a replication group in an Amazon VPC using redis version <code>3.2.6</code>, <code>4.x</code> or later. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthTokenEnabled</i> A flag that enables using an <code>AuthToken</code> (password) when issuing Redis commands. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodeType</i> The name of the compute and memory capacity node type for each node in the replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterEnabled</i> A flag indicating whether or not this replication group is cluster enabled; i.e., whether its data can be partitioned across multiple shards (API/CLI: node groups). Valid values: <code>true</code> | <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotWindow</i> The daily time range (in UTC) during which ElastiCache begins taking a daily snapshot of your node group (shard). Example: <code>05:00-09:00</code>  If you do not specify this parameter, ElastiCache automatically chooses an appropriate time range. <note> This parameter is only valid if the <code>Engine</code> parameter is <code>redis</code>. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotRetentionLimit</i> The number of days for which ElastiCache retains automatic cluster snapshots before deleting them. For example, if you set <code>SnapshotRetentionLimit</code> to 5, a snapshot that was taken today is retained for 5 days before being deleted. <important>  If the value of <code>SnapshotRetentionLimit</code> is set to zero (0), backups are turned off. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigurationEndpoint</i> The configuration endpoint for this replication group. Use the configuration endpoint to connect to this replication group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ConfigurationEndpoint</b>: The configuration endpoint for this replication group. Use the configuration endpoint to connect to this replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomaticFailover</i> Indicates the status of Multi-AZ with automatic failover for this Redis replication group. Amazon ElastiCache for Redis does not support Multi-AZ with automatic failover on: <ul> <li> Redis versions earlier than 2.8.6. </li> <li> Redis (cluster mode disabled): T1 node types. </li> <li> Redis (cluster mode enabled): T1 node types. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshottingClusterId</i> The cluster ID that is used as the daily snapshot source for the replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeGroups</i> A list of node groups in this replication group. For Redis (cluster mode disabled) replication groups, this is a single-element list. For Redis (cluster mode enabled) replication groups, the list contains an entry for each node group (shard).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MemberClusters</i> The names of all the cache clusters that are part of this replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> A group of settings to be applied to the replication group, either immediately or during the next maintenance window.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: A group of settings to be applied to the replication group, either immediately or during the next maintenance window.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current state of this replication group - <code>creating</code>, <code>available</code>, <code>modifying</code>, <code>deleting</code>, <code>create-failed</code>, <code>snapshotting</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The user supplied description of the replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationGroupId</i> The identifier for the replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomaticFailoverStatus</i> Indicates the status of Multi-AZ with automatic failover for this Redis replication group. Amazon ElastiCache for Redis does not support Multi-AZ with automatic failover on: <ul> <li> Redis versions earlier than 2.8.6. </li> <li> Redis (cluster mode disabled): T1 node types. </li> <li> Redis (cluster mode enabled): T1 node types. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PrimaryClusterId</i> The primary cluster ID that is applied immediately (if <code>--apply-immediately</code> was specified), or during the next maintenance window.<br/>The status of an online resharding operation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Resharding</b>: The status of an online resharding operation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port number that the cache engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS hostname of the cache node.<br/>        """
        pass

    def purchase_reserved_cache_nodes_offering(self, ReservedCacheNodesOfferingId: str, ReservedCacheNodeId: str=None, CacheNodeCount: int=None) -> PurchaseReservedCacheNodesOfferingResult:
        """Allows you to purchase a reserved cache node offering.

        :param self:
        :param CacheNodeCount:  The number of cache node instances to reserve. Default: <code>1</code> <br/>ReservedCacheNodeId:  A customer-specified identifier to track this reservation. <note> The Reserved Cache Node ID is an unique customer-specified identifier to track this reservation. If this parameter is not specified, ElastiCache automatically generates an identifier for the reservation. </note> Example: myreservationID<br/>ReservedCacheNodesOfferingId:  The ID of the reserved cache node offering to purchase. Example: <code>438012d3-4052-4cc7-b2e3-8d3372e0e706</code> <br/>
        :return: ReservedCacheNode:  Represents the output of a <code>PurchaseReservedCacheNodesOffering</code> operation.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservationARN</i> The Amazon Resource Name (ARN) of the reserved cache node. Example: <code>arn:aws:elasticache:us-east-1:123456789012:reserved-instance:ri-2017-03-27-08-33-25-582</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecurringCharges</i> The recurring price charged to run this reserved cache node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the reserved cache node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OfferingType</i> The offering type of this reserved cache node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductDescription</i> The description of the reserved cache node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodeCount</i> The number of cache nodes that have been reserved.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UsagePrice</i> The hourly price charged for this reserved cache node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FixedPrice</i> The fixed price charged for this reserved cache node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Duration</i> The duration of the reservation in seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTime</i> The time the reservation started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodeType</i> The cache node type for the reserved cache nodes. The following node types are supported by ElastiCache. Generally speaking, the current generation types provide more memory and computational power at lower cost when compared to their equivalent previous generation counterparts. <ul> <li> General purpose: <ul> <li> Current generation:   <b>M5 node types:</b> <code>cache.m5.large</code>, <code>cache.m5.xlarge</code>, <code>cache.m5.2xlarge</code>, <code>cache.m5.4xlarge</code>, <code>cache.m5.12xlarge</code>, <code>cache.m5.24xlarge</code>   <b>M4 node types:</b> <code>cache.m4.large</code>, <code>cache.m4.xlarge</code>, <code>cache.m4.2xlarge</code>, <code>cache.m4.4xlarge</code>, <code>cache.m4.10xlarge</code>   <b>T2 node types:</b> <code>cache.t2.micro</code>, <code>cache.t2.small</code>, <code>cache.t2.medium</code>  </li> <li> Previous generation: (not recommended)  <b>T1 node types:</b> <code>cache.t1.micro</code>   <b>M1 node types:</b> <code>cache.m1.small</code>, <code>cache.m1.medium</code>, <code>cache.m1.large</code>, <code>cache.m1.xlarge</code>   <b>M3 node types:</b> <code>cache.m3.medium</code>, <code>cache.m3.large</code>, <code>cache.m3.xlarge</code>, <code>cache.m3.2xlarge</code>  </li> </ul> </li> <li> Compute optimized: <ul> <li> Previous generation: (not recommended)  <b>C1 node types:</b> <code>cache.c1.xlarge</code>  </li> </ul> </li> <li> Memory optimized: <ul> <li> Current generation:   <b>R5 node types:</b> <code>cache.r5.large</code>, <code>cache.r5.xlarge</code>, <code>cache.r5.2xlarge</code>, <code>cache.r5.4xlarge</code>, <code>cache.r5.12xlarge</code>, <code>cache.r5.24xlarge</code>   <b>R4 node types:</b> <code>cache.r4.large</code>, <code>cache.r4.xlarge</code>, <code>cache.r4.2xlarge</code>, <code>cache.r4.4xlarge</code>, <code>cache.r4.8xlarge</code>, <code>cache.r4.16xlarge</code>  </li> <li> Previous generation: (not recommended)  <b>M2 node types:</b> <code>cache.m2.xlarge</code>, <code>cache.m2.2xlarge</code>, <code>cache.m2.4xlarge</code>   <b>R3 node types:</b> <code>cache.r3.large</code>, <code>cache.r3.xlarge</code>, <code>cache.r3.2xlarge</code>, <code>cache.r3.4xlarge</code>, <code>cache.r3.8xlarge</code>  </li> </ul> </li> </ul>  <b>Additional node type info</b>  <ul> <li> All current generation instance types are created in Amazon VPC by default. </li> <li> Redis append-only files (AOF) are not supported for T1 or T2 instances. </li> <li> Redis Multi-AZ with automatic failover is not supported on T1 instances. </li> <li> Redis configuration variables <code>appendonly</code> and <code>appendfsync</code> are not supported on Redis version 2.8.22 and later. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservedCacheNodesOfferingId</i> The offering identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservedCacheNodeId</i> The unique identifier for the reservation.<br/>        """
        pass

    def reboot_cache_cluster(self, CacheNodeIdsToReboot: List[String], CacheClusterId: str) -> RebootCacheClusterResult:
        """Reboots some, or all, of the cache nodes within a provisioned cluster. This operation applies any modified cache parameter groups to the cluster. The reboot operation takes place as soon as possible, and results in a momentary outage to the cluster. During the reboot, the cluster status is set to REBOOTING. The reboot causes the contents of the cache (for each cache node being rebooted) to be lost. When the reboot is complete, a cluster event is created. Rebooting a cluster is currently supported on Memcached and Redis (cluster mode disabled) clusters. Rebooting is not supported on Redis (cluster mode enabled) clusters. If you make changes to parameters that require a Redis (cluster mode enabled) cluster reboot for the changes to be applied, see <a href="http://docs.aws.amazon.com/AmazonElastiCache/latest/red-ug/Clusters.Rebooting.html">Rebooting a Cluster</a> for an alternate process.

        :param self:
        :param CacheNodeIdsToReboot:  A list of cache node IDs to reboot. A node ID is a numeric identifier (0001, 0002, etc.). To reboot an entire cluster, specify all of the cache node IDs.<br/>CacheClusterId:  The cluster identifier. This parameter is stored as a lowercase string.<br/>
        :return: CacheCluster:  Contains all of the attributes of a specific cluster.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AtRestEncryptionEnabled</i> A flag that enables encryption at-rest when set to <code>true</code>. You cannot modify the value of <code>AtRestEncryptionEnabled</code> after the cluster is created. To enable at-rest encryption on a cluster you must set <code>AtRestEncryptionEnabled</code> to <code>true</code> when you create a cluster.  <b>Required:</b> Only available when creating a replication group in an Amazon VPC using redis version <code>3.2.6</code>, <code>4.x</code> or later. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitEncryptionEnabled</i> A flag that enables in-transit encryption when set to <code>true</code>. You cannot modify the value of <code>TransitEncryptionEnabled</code> after the cluster is created. To enable in-transit encryption on a cluster you must set <code>TransitEncryptionEnabled</code> to <code>true</code> when you create a cluster.  <b>Required:</b> Only available when creating a replication group in an Amazon VPC using redis version <code>3.2.6</code>, <code>4.x</code> or later. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthTokenEnabled</i> A flag that enables using an <code>AuthToken</code> (password) when issuing Redis commands. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotWindow</i> The daily time range (in UTC) during which ElastiCache begins taking a daily snapshot of your cluster. Example: <code>05:00-09:00</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotRetentionLimit</i> The number of days for which ElastiCache retains automatic cluster snapshots before deleting them. For example, if you set <code>SnapshotRetentionLimit</code> to 5, a snapshot that was taken today is retained for 5 days before being deleted. <important>  If the value of SnapshotRetentionLimit is set to zero (0), backups are turned off. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationGroupId</i> The replication group to which this cluster belongs. If this field is empty, the cluster is not associated with any replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroups</i> A list of VPC Security Groups associated with the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> This parameter is currently disabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodes</i> A list of cache nodes that are members of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheSubnetGroupName</i> The name of the cache subnet group associated with the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheParameterGroup</i> Status of the cache parameter group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CacheParameterGroup</b>: Status of the cache parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheSecurityGroups</i> A list of cache security group elements, composed of name and status sub-elements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationConfiguration</i> Describes a notification topic and its status. Notification topics are used for publishing ElastiCache events to subscribers using Amazon Simple Notification Service (SNS). &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NotificationConfiguration</b>: Describes a notification topic and its status. Notification topics are used for publishing ElastiCache events to subscribers using Amazon Simple Notification Service (SNS). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> A group of settings that are applied to the cluster in the future, or that are currently being applied.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: A group of settings that are applied to the cluster in the future, or that are currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which maintenance on the cluster is performed. It is specified as a range in the format ddd:hh24:mi-ddd:hh24:mi (24H Clock UTC). The minimum maintenance window is a 60 minute period. Valid values for <code>ddd</code> are: <ul> <li>  <code>sun</code>  </li> <li>  <code>mon</code>  </li> <li>  <code>tue</code>  </li> <li>  <code>wed</code>  </li> <li>  <code>thu</code>  </li> <li>  <code>fri</code>  </li> <li>  <code>sat</code>  </li> </ul> Example: <code>sun:23:00-mon:01:30</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheClusterCreateTime</i> The date and time when the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredAvailabilityZone</i> The name of the Availability Zone in which the cluster is located or "Multiple" if the cache nodes are located in different Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumCacheNodes</i> The number of cache nodes in the cluster. For clusters running Redis, this value must be 1. For clusters running Memcached, this value must be between 1 and 20.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheClusterStatus</i> The current state of this cluster, one of the following values: <code>available</code>, <code>creating</code>, <code>deleted</code>, <code>deleting</code>, <code>incompatible-network</code>, <code>modifying</code>, <code>rebooting cluster nodes</code>, <code>restore-failed</code>, or <code>snapshotting</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> The version of the cache engine that is used in this cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> The name of the cache engine (<code>memcached</code> or <code>redis</code>) to be used for this cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodeType</i> The name of the compute and memory capacity node type for the cluster. The following node types are supported by ElastiCache. Generally speaking, the current generation types provide more memory and computational power at lower cost when compared to their equivalent previous generation counterparts. <ul> <li> General purpose: <ul> <li> Current generation:   <b>M5 node types:</b> <code>cache.m5.large</code>, <code>cache.m5.xlarge</code>, <code>cache.m5.2xlarge</code>, <code>cache.m5.4xlarge</code>, <code>cache.m5.12xlarge</code>, <code>cache.m5.24xlarge</code>   <b>M4 node types:</b> <code>cache.m4.large</code>, <code>cache.m4.xlarge</code>, <code>cache.m4.2xlarge</code>, <code>cache.m4.4xlarge</code>, <code>cache.m4.10xlarge</code>   <b>T2 node types:</b> <code>cache.t2.micro</code>, <code>cache.t2.small</code>, <code>cache.t2.medium</code>  </li> <li> Previous generation: (not recommended)  <b>T1 node types:</b> <code>cache.t1.micro</code>   <b>M1 node types:</b> <code>cache.m1.small</code>, <code>cache.m1.medium</code>, <code>cache.m1.large</code>, <code>cache.m1.xlarge</code>   <b>M3 node types:</b> <code>cache.m3.medium</code>, <code>cache.m3.large</code>, <code>cache.m3.xlarge</code>, <code>cache.m3.2xlarge</code>  </li> </ul> </li> <li> Compute optimized: <ul> <li> Previous generation: (not recommended)  <b>C1 node types:</b> <code>cache.c1.xlarge</code>  </li> </ul> </li> <li> Memory optimized: <ul> <li> Current generation:   <b>R5 node types:</b> <code>cache.r5.large</code>, <code>cache.r5.xlarge</code>, <code>cache.r5.2xlarge</code>, <code>cache.r5.4xlarge</code>, <code>cache.r5.12xlarge</code>, <code>cache.r5.24xlarge</code>   <b>R4 node types:</b> <code>cache.r4.large</code>, <code>cache.r4.xlarge</code>, <code>cache.r4.2xlarge</code>, <code>cache.r4.4xlarge</code>, <code>cache.r4.8xlarge</code>, <code>cache.r4.16xlarge</code>  </li> <li> Previous generation: (not recommended)  <b>M2 node types:</b> <code>cache.m2.xlarge</code>, <code>cache.m2.2xlarge</code>, <code>cache.m2.4xlarge</code>   <b>R3 node types:</b> <code>cache.r3.large</code>, <code>cache.r3.xlarge</code>, <code>cache.r3.2xlarge</code>, <code>cache.r3.4xlarge</code>, <code>cache.r3.8xlarge</code>  </li> </ul> </li> </ul>  <b>Additional node type info</b>  <ul> <li> All current generation instance types are created in Amazon VPC by default. </li> <li> Redis append-only files (AOF) are not supported for T1 or T2 instances. </li> <li> Redis Multi-AZ with automatic failover is not supported on T1 instances. </li> <li> Redis configuration variables <code>appendonly</code> and <code>appendfsync</code> are not supported on Redis version 2.8.22 and later. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientDownloadLandingPage</i> The URL of the web page where you can download the latest ElastiCache client library.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigurationEndpoint</i> Represents a Memcached cluster endpoint which, if Automatic Discovery is enabled on the cluster, can be used by an application to connect to any node in the cluster. The configuration endpoint will always have <code>.cfg</code> in it. Example: <code>mem-3.9dvc4r<u>.cfg</u>.usw2.cache.amazonaws.com:11211</code> &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ConfigurationEndpoint</b>: Represents a Memcached cluster endpoint which, if Automatic Discovery is enabled on the cluster, can be used by an application to connect to any node in the cluster. The configuration endpoint will always have <code>.cfg</code> in it. Example: <code>mem-3.9dvc4r<u>.cfg</u>.usw2.cache.amazonaws.com:11211</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheClusterId</i> The user-supplied identifier of the cluster. This identifier is a unique key that identifies a cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port number that the cache engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS hostname of the cache node.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodeType</i> The cache node type that this cluster or replication group is scaled to.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> The new cache engine version that the cluster runs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodeIdsToRemove</i> A list of cache node IDs that are being removed (or will be removed) from the cluster. A node ID is a 4-digit numeric identifier (0001, 0002, etc.).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumCacheNodes</i> The new number of cache nodes for the cluster. For clusters running Redis, this value must be 1. For clusters running Memcached, this value must be between 1 and 20.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicStatus</i> The current state of the topic.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicArn</i> The Amazon Resource Name (ARN) that identifies the topic.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodeIdsToReboot</i> A list of the cache node IDs which need to be rebooted for parameter changes to be applied. A node ID is a numeric identifier (0001, 0002, etc.).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterApplyStatus</i> The status of parameter updates.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheParameterGroupName</i> The name of the cache parameter group.<br/>        """
        pass

    def remove_tags_from_resource(self, TagKeys: List[String], ResourceName: str) -> TagListMessage:
        """Removes the tags identified by the <code>TagKeys</code> list from the named resource.

        :param self:
        :param TagKeys:  A list of <code>TagKeys</code> identifying the tags you want removed from the named resource.<br/>ResourceName:  The Amazon Resource Name (ARN) of the resource from which you want the tags removed, for example <code>arn:aws:elasticache:us-west-2:0123456789:cluster:myCluster</code> or <code>arn:aws:elasticache:us-west-2:0123456789:snapshot:mySnapshot</code>. For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>
        :return: TagList:  A list of cost allocation tags as key-value pairs.<br/>        """
        pass

    def reset_cache_parameter_group(self, CacheParameterGroupName: str, ResetAllParameters: Boolean=None, ParameterNameValues: List[ParameterNameValue]=None) -> CacheParameterGroupNameMessage:
        """Modifies the parameters of a cache parameter group to the engine or system default value. You can reset specific parameters by submitting a list of parameter names. To reset the entire cache parameter group, specify the <code>ResetAllParameters</code> and <code>CacheParameterGroupName</code> parameters.

        :param self:
        :param ParameterNameValues:  An array of parameter names to reset to their default values. If <code>ResetAllParameters</code> is <code>true</code>, do not use <code>ParameterNameValues</code>. If <code>ResetAllParameters</code> is <code>false</code>, you must specify the name of at least one parameter to reset.<br/>ResetAllParameters:  If <code>true</code>, all parameters in the cache parameter group are reset to their default values. If <code>false</code>, only the parameters listed by <code>ParameterNameValues</code> are reset to their default values. Valid values: <code>true</code> | <code>false</code> <br/>CacheParameterGroupName:  The name of the cache parameter group to reset.<br/>
        :return: CacheParameterGroupName:  The name of the cache parameter group.<br/>        """
        pass

    def revoke_cache_security_group_ingress(self, EC2SecurityGroupOwnerId: str, EC2SecurityGroupName: str, CacheSecurityGroupName: str) -> RevokeCacheSecurityGroupIngressResult:
        """Revokes ingress from a cache security group. Use this operation to disallow access from an Amazon EC2 security group that had been previously authorized.

        :param self:
        :param EC2SecurityGroupOwnerId:  The AWS account number of the Amazon EC2 security group owner. Note that this is not the same thing as an AWS access key ID - you must provide a valid AWS account number for this parameter.<br/>EC2SecurityGroupName:  The name of the Amazon EC2 security group to revoke access from.<br/>CacheSecurityGroupName:  The name of the cache security group to revoke ingress from.<br/>
        :return: CacheSecurityGroup:  Represents the output of one of the following operations: <ul> <li>  <code>AuthorizeCacheSecurityGroupIngress</code>  </li> <li>  <code>CreateCacheSecurityGroup</code>  </li> <li>  <code>RevokeCacheSecurityGroupIngress</code>  </li> </ul>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EC2SecurityGroups</i> A list of Amazon EC2 security groups that are associated with this cache security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the cache security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheSecurityGroupName</i> The name of the cache security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The AWS account ID of the cache security group owner.<br/>        """
        pass

    def test_failover(self, NodeGroupId: str, ReplicationGroupId: str) -> TestFailoverResult:
        """Represents the input of a <code>TestFailover</code> operation which test automatic failover on a specified node group (called shard in the console) in a replication group (called cluster in the console). <p class="title"> <b>Note the following</b>  <ul> <li> A customer can use this operation to test automatic failover on up to 5 shards (called node groups in the ElastiCache API and AWS CLI) in any rolling 24-hour period. </li> <li> If calling this operation on shards in different clusters (called replication groups in the API and CLI), the calls can be made concurrently.   </li> <li> If calling this operation multiple times on different shards in the same Redis (cluster mode enabled) replication group, the first node replacement must complete before a subsequent call can be made. </li> <li> To determine whether the node replacement is complete you can check Events using the Amazon ElastiCache console, the AWS CLI, or the ElastiCache API. Look for the following automatic failover related events, listed here in order of occurrance: <ol> <li> Replication group message: <code>Test Failover API called for node group &lt;node-group-id&gt;</code>  </li> <li> Cache cluster message: <code>Failover from master node &lt;primary-node-id&gt; to replica node &lt;node-id&gt; completed</code>  </li> <li> Replication group message: <code>Failover from master node &lt;primary-node-id&gt; to replica node &lt;node-id&gt; completed</code>  </li> <li> Cache cluster message: <code>Recovering cache nodes &lt;node-id&gt;</code>  </li> <li> Cache cluster message: <code>Finished recovery for cache nodes &lt;node-id&gt;</code>  </li> </ol> For more information see: <ul> <li>  <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/red-ug/ECEvents.Viewing.html">Viewing ElastiCache Events</a> in the <i>ElastiCache User Guide</i>  </li> <li>  <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/APIReference/API_DescribeEvents.html">DescribeEvents</a> in the ElastiCache API Reference </li> </ul> </li> </ul> Also see, <a href="https://docs.aws.amazon.com/AmazonElastiCache/latest/red-ug/AutoFailover.html#auto-failover-test">Testing Multi-AZ with Automatic Failover</a> in the <i>ElastiCache User Guide</i>.

        :param self:
        :param NodeGroupId:  The name of the node group (called shard in the console) in this replication group on which automatic failover is to be tested. You may test automatic failover on up to 5 node groups in any rolling 24-hour period.<br/>ReplicationGroupId:  The name of the replication group (console: cluster) whose automatic failover is being tested by this operation.<br/>
        :return: ReplicationGroup:  Contains all of the attributes of a specific Redis replication group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AtRestEncryptionEnabled</i> A flag that enables encryption at-rest when set to <code>true</code>. You cannot modify the value of <code>AtRestEncryptionEnabled</code> after the cluster is created. To enable encryption at-rest on a cluster you must set <code>AtRestEncryptionEnabled</code> to <code>true</code> when you create a cluster.  <b>Required:</b> Only available when creating a replication group in an Amazon VPC using redis version <code>3.2.6</code>, <code>4.x</code> or later. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransitEncryptionEnabled</i> A flag that enables in-transit encryption when set to <code>true</code>. You cannot modify the value of <code>TransitEncryptionEnabled</code> after the cluster is created. To enable in-transit encryption on a cluster you must set <code>TransitEncryptionEnabled</code> to <code>true</code> when you create a cluster.  <b>Required:</b> Only available when creating a replication group in an Amazon VPC using redis version <code>3.2.6</code>, <code>4.x</code> or later. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AuthTokenEnabled</i> A flag that enables using an <code>AuthToken</code> (password) when issuing Redis commands. Default: <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CacheNodeType</i> The name of the compute and memory capacity node type for each node in the replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterEnabled</i> A flag indicating whether or not this replication group is cluster enabled; i.e., whether its data can be partitioned across multiple shards (API/CLI: node groups). Valid values: <code>true</code> | <code>false</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotWindow</i> The daily time range (in UTC) during which ElastiCache begins taking a daily snapshot of your node group (shard). Example: <code>05:00-09:00</code>  If you do not specify this parameter, ElastiCache automatically chooses an appropriate time range. <note> This parameter is only valid if the <code>Engine</code> parameter is <code>redis</code>. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotRetentionLimit</i> The number of days for which ElastiCache retains automatic cluster snapshots before deleting them. For example, if you set <code>SnapshotRetentionLimit</code> to 5, a snapshot that was taken today is retained for 5 days before being deleted. <important>  If the value of <code>SnapshotRetentionLimit</code> is set to zero (0), backups are turned off. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigurationEndpoint</i> The configuration endpoint for this replication group. Use the configuration endpoint to connect to this replication group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ConfigurationEndpoint</b>: The configuration endpoint for this replication group. Use the configuration endpoint to connect to this replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomaticFailover</i> Indicates the status of Multi-AZ with automatic failover for this Redis replication group. Amazon ElastiCache for Redis does not support Multi-AZ with automatic failover on: <ul> <li> Redis versions earlier than 2.8.6. </li> <li> Redis (cluster mode disabled): T1 node types. </li> <li> Redis (cluster mode enabled): T1 node types. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshottingClusterId</i> The cluster ID that is used as the daily snapshot source for the replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeGroups</i> A list of node groups in this replication group. For Redis (cluster mode disabled) replication groups, this is a single-element list. For Redis (cluster mode enabled) replication groups, the list contains an entry for each node group (shard).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MemberClusters</i> The names of all the cache clusters that are part of this replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> A group of settings to be applied to the replication group, either immediately or during the next maintenance window.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: A group of settings to be applied to the replication group, either immediately or during the next maintenance window.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current state of this replication group - <code>creating</code>, <code>available</code>, <code>modifying</code>, <code>deleting</code>, <code>create-failed</code>, <code>snapshotting</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The user supplied description of the replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationGroupId</i> The identifier for the replication group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomaticFailoverStatus</i> Indicates the status of Multi-AZ with automatic failover for this Redis replication group. Amazon ElastiCache for Redis does not support Multi-AZ with automatic failover on: <ul> <li> Redis versions earlier than 2.8.6. </li> <li> Redis (cluster mode disabled): T1 node types. </li> <li> Redis (cluster mode enabled): T1 node types. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PrimaryClusterId</i> The primary cluster ID that is applied immediately (if <code>--apply-immediately</code> was specified), or during the next maintenance window.<br/>The status of an online resharding operation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Resharding</b>: The status of an online resharding operation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port number that the cache engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS hostname of the cache node.<br/>        """
        pass



class ElasticBeanstalk(BaseClient):
    """<fullname>AWS Elastic Beanstalk</fullname> AWS Elastic Beanstalk makes it easy for you to create, deploy, and manage scalable, fault-tolerant applications running on the Amazon Web Services cloud. For more information about this product, go to the <a href="http://aws.amazon.com/elasticbeanstalk/">AWS Elastic Beanstalk</a> details page. The location of the latest AWS Elastic Beanstalk WSDL is <a href="http://elasticbeanstalk.s3.amazonaws.com/doc/2010-12-01/AWSElasticBeanstalk.wsdl">http://elasticbeanstalk.s3.amazonaws.com/doc/2010-12-01/AWSElasticBeanstalk.wsdl</a>. To install the Software Development Kits (SDKs), Integrated Development Environment (IDE) Toolkits, and command line tools that enable you to access the API, go to <a href="http://aws.amazon.com/tools/">Tools for Amazon Web Services</a>.  <b>Endpoints</b>  For a list of region-specific endpoints that AWS Elastic Beanstalk supports, go to <a href="https://docs.aws.amazon.com/general/latest/gr/rande.html#elasticbeanstalk_region">Regions and Endpoints</a> in the <i>Amazon Web Services Glossary</i>."""


    class DescribeEventsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_events` for valid parameters"""
            pass
    class DescribeApplicationVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_application_versions` for valid parameters"""
            pass
    class DescribeEnvironmentManagedActionHistoryPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_environment_managed_action_history` for valid parameters"""
            pass
    class DescribeEnvironmentsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_environments` for valid parameters"""
            pass
    class ListPlatformVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_platform_versions` for valid parameters"""
            pass


    class AbortEnvironmentUpdateMessage(Mapping):
        pass
    
    class ApplyEnvironmentManagedActionRequest(Mapping):
        pass
    
    class ApplyEnvironmentManagedActionResult(Mapping):
        pass
    
    class CheckDNSAvailabilityMessage(Mapping):
        pass
    
    class CheckDNSAvailabilityResultMessage(Mapping):
        pass
    
    class ComposeEnvironmentsMessage(Mapping):
        pass
    
    class EnvironmentDescriptionsMessage(Mapping):
        pass
    
    class VersionLabel(object):
        pass
    
    class CreateApplicationMessage(Mapping):
        pass
    
    class ApplicationDescriptionMessage(Mapping):
        pass
    
    class ApplicationResourceLifecycleConfig(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreateApplicationVersionMessage(Mapping):
        pass
    
    class ApplicationVersionDescriptionMessage(Mapping):
        pass
    
    class SourceBuildInformation(Mapping):
        pass
    
    class S3Location(Mapping):
        pass
    
    class BuildConfiguration(Mapping):
        pass
    
    class AutoCreateApplication(object):
        pass
    
    class ApplicationVersionProccess(object):
        pass
    
    class CreateConfigurationTemplateMessage(Mapping):
        pass
    
    class ConfigurationSettingsDescription(Mapping):
        pass
    
    class SourceConfiguration(Mapping):
        pass
    
    class ConfigurationOptionSetting(Mapping):
        pass
    
    class CreateEnvironmentMessage(Mapping):
        pass
    
    class EnvironmentDescription(Mapping):
        pass
    
    class EnvironmentTier(Mapping):
        pass
    
    class OptionSpecification(Mapping):
        pass
    
    class CreatePlatformVersionRequest(Mapping):
        pass
    
    class CreatePlatformVersionResult(Mapping):
        pass
    
    class CreateStorageLocationResultMessage(Mapping):
        pass
    
    class DeleteApplicationMessage(Mapping):
        pass
    
    class TerminateEnvForce(object):
        pass
    
    class DeleteApplicationVersionMessage(Mapping):
        pass
    
    class DeleteSourceBundle(object):
        pass
    
    class DeleteConfigurationTemplateMessage(Mapping):
        pass
    
    class DeleteEnvironmentConfigurationMessage(Mapping):
        pass
    
    class DeletePlatformVersionRequest(Mapping):
        pass
    
    class DeletePlatformVersionResult(Mapping):
        pass
    
    class DescribeAccountAttributesResult(Mapping):
        pass
    
    class DescribeApplicationVersionsMessage(Mapping):
        pass
    
    class ApplicationVersionDescriptionsMessage(Mapping):
        pass
    
    class DescribeApplicationsMessage(Mapping):
        pass
    
    class ApplicationDescriptionsMessage(Mapping):
        pass
    
    class ApplicationName(object):
        pass
    
    class DescribeConfigurationOptionsMessage(Mapping):
        pass
    
    class ConfigurationOptionsDescription(Mapping):
        pass
    
    class DescribeConfigurationSettingsMessage(Mapping):
        pass
    
    class ConfigurationSettingsDescriptions(Mapping):
        pass
    
    class DescribeEnvironmentHealthRequest(Mapping):
        pass
    
    class DescribeEnvironmentHealthResult(Mapping):
        pass
    
    class EnvironmentHealthAttribute(object):
        pass
    
    class DescribeEnvironmentManagedActionHistoryRequest(Mapping):
        pass
    
    class DescribeEnvironmentManagedActionHistoryResult(Mapping):
        pass
    
    class DescribeEnvironmentManagedActionsRequest(Mapping):
        pass
    
    class DescribeEnvironmentManagedActionsResult(Mapping):
        pass
    
    class DescribeEnvironmentResourcesMessage(Mapping):
        pass
    
    class EnvironmentResourceDescriptionsMessage(Mapping):
        pass
    
    class DescribeEnvironmentsMessage(Mapping):
        pass
    
    class EnvironmentId(object):
        pass
    
    class EnvironmentName(object):
        pass
    
    class IncludeDeleted(object):
        pass
    
    class IncludeDeletedBackTo(object):
        pass
    
    class DescribeEventsMessage(Mapping):
        pass
    
    class EventDescriptionsMessage(Mapping):
        pass
    
    class TimeFilterStart(object):
        pass
    
    class TimeFilterEnd(object):
        pass
    
    class DescribeInstancesHealthRequest(Mapping):
        pass
    
    class DescribeInstancesHealthResult(Mapping):
        pass
    
    class InstancesHealthAttribute(object):
        pass
    
    class DescribePlatformVersionRequest(Mapping):
        pass
    
    class DescribePlatformVersionResult(Mapping):
        pass
    
    class ListAvailableSolutionStacksResultMessage(Mapping):
        pass
    
    class ListPlatformVersionsRequest(Mapping):
        pass
    
    class ListPlatformVersionsResult(Mapping):
        pass
    
    class PlatformFilter(Mapping):
        pass
    
    class ListTagsForResourceMessage(Mapping):
        pass
    
    class ResourceTagsDescriptionMessage(Mapping):
        pass
    
    class RebuildEnvironmentMessage(Mapping):
        pass
    
    class RequestEnvironmentInfoMessage(Mapping):
        pass
    
    class RestartAppServerMessage(Mapping):
        pass
    
    class RetrieveEnvironmentInfoMessage(Mapping):
        pass
    
    class RetrieveEnvironmentInfoResultMessage(Mapping):
        pass
    
    class SwapEnvironmentCNAMEsMessage(Mapping):
        pass
    
    class TerminateEnvironmentMessage(Mapping):
        pass
    
    class TerminateEnvironmentResources(object):
        pass
    
    class ForceTerminate(object):
        pass
    
    class UpdateApplicationMessage(Mapping):
        pass
    
    class UpdateApplicationResourceLifecycleMessage(Mapping):
        pass
    
    class ApplicationResourceLifecycleDescriptionMessage(Mapping):
        pass
    
    class UpdateApplicationVersionMessage(Mapping):
        pass
    
    class UpdateConfigurationTemplateMessage(Mapping):
        pass
    
    class UpdateEnvironmentMessage(Mapping):
        pass
    
    class UpdateTagsForResourceMessage(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class ValidateConfigurationSettingsMessage(Mapping):
        pass
    
    class ConfigurationSettingsValidationMessages(Mapping):
        pass
    
    def abort_environment_update(self, EnvironmentId: str=None, EnvironmentName: str=None):
        """Cancels in-progress environment configuration update or application version deployment.

        :param self:
        :param EnvironmentName:  This specifies the name of the environment with the in-progress update that you want to cancel.<br/>EnvironmentId:  This specifies the ID of the environment with the in-progress update that you want to cancel.<br/>
        :return:         """
        pass

    def apply_environment_managed_action(self, ActionId: str, EnvironmentName: str=None, EnvironmentId: str=None) -> ApplyEnvironmentManagedActionResult:
        """Applies a scheduled managed action immediately. A managed action can be applied only if its status is <code>Scheduled</code>. Get the status and action ID of a managed action with <a>DescribeEnvironmentManagedActions</a>.

        :param self:
        :param ActionId:  The action ID of the scheduled managed action to execute.<br/>EnvironmentId:  The environment ID of the target environment.<br/>EnvironmentName:  The name of the target environment.<br/>
        :return: Status:  The status of the managed action.<br/>ActionType:  The type of managed action.<br/>ActionDescription:  A description of the managed action.<br/>ActionId:  The action ID of the managed action.<br/>        """
        pass

    def check_dns_availability(self, CNAMEPrefix: str) -> CheckDNSAvailabilityResultMessage:
        """Checks if the specified CNAME is available.

        :param self:
        :param CNAMEPrefix:  The prefix used when this CNAME is reserved.<br/>
        :return: FullyQualifiedCNAME:  The fully qualified CNAME to reserve when <a>CreateEnvironment</a> is called with the provided prefix.<br/>Available:  Indicates if the specified CNAME is available: <ul> <li>  <code>true</code> : The CNAME is available. </li> <li>  <code>false</code> : The CNAME is not available. </li> </ul><br/>        """
        pass

    def compose_environments(self, ApplicationName: str=None, GroupName: str=None, VersionLabels: List[VersionLabel]=None) -> EnvironmentDescriptionsMessage:
        """Create or update a group of environments that each run a separate component of a single application. Takes a list of version labels that specify application source bundles for each of the environments to create or update. The name of each environment and other required information must be included in the source bundles in an environment manifest named <code>env.yaml</code>. See <a href="https://docs.aws.amazon.com/elasticbeanstalk/latest/dg/environment-mgmt-compose.html">Compose Environments</a> for details.

        :param self:
        :param VersionLabels:  A list of version labels, specifying one or more application source bundles that belong to the target application. Each source bundle must include an environment manifest that specifies the name of the environment and the name of the solution stack to use, and optionally can specify environment links to create.<br/>GroupName:  The name of the group to which the target environments belong. Specify a group name only if the environment name defined in each target environment's manifest ends with a + (plus) character. See <a href="https://docs.aws.amazon.com/elasticbeanstalk/latest/dg/environment-cfg-manifest.html">Environment Manifest (env.yaml)</a> for details.<br/>ApplicationName:  The name of the application to which the specified source bundles belong.<br/>
        :return: NextToken:  In a paginated request, the token that you can pass in a subsequent request to get the next response page.<br/>Environments:   Returns an <a>EnvironmentDescription</a> list. <br/>        """
        pass

    def create_application(self, ApplicationName: str, Description: str=None, ResourceLifecycleConfig: ApplicationResourceLifecycleConfig=None, Tags: List[Tag]=None) -> ApplicationDescriptionMessage:
        """ Creates an application that has one configuration template named <code>default</code> and no application versions. 

        :param self:
        :param Tags:  Specifies the tags applied to the application. Elastic Beanstalk applies these tags only to the application. Environments that you create in the application don't inherit the tags.<br/>ResourceLifecycleConfig:  Specify an application resource lifecycle configuration to prevent your application from accumulating too many versions.:<br/>Description:  Describes the application.<br/>ApplicationName:  The name of the application. Constraint: This name must be unique within your account. If the specified name already exists, the action returns an <code>InvalidParameterValue</code> error.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VersionLifecycleConfig</i> The application version lifecycle configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VersionLifecycleConfig</b>: The application version lifecycle configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceRole</i> The ARN of an IAM service role that Elastic Beanstalk has permission to assume. The <code>ServiceRole</code> property is required the first time that you provide a <code>VersionLifecycleConfig</code> for the application in one of the supporting calls (<code>CreateApplication</code> or <code>UpdateApplicationResourceLifecycle</code>). After you provide it once, in either one of the calls, Elastic Beanstalk persists the Service Role with the application, and you don't need to specify it again in subsequent <code>UpdateApplicationResourceLifecycle</code> calls. You can, however, specify it in subsequent calls to change the Service Role to another value.<br/>Specify a max count rule to restrict the number of application versions that are retained for an application.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>MaxCountRule</b>: Specify a max count rule to restrict the number of application versions that are retained for an application.
        :return: Application:   The <a>ApplicationDescription</a> of the application. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceLifecycleConfig</i> The lifecycle settings for the application.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResourceLifecycleConfig</b>: The lifecycle settings for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigurationTemplates</i> The names of the configuration templates associated with this application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Versions</i> The names of the versions for this application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DateUpdated</i> The date when the application was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DateCreated</i> The date when the application was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> User-defined description of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationName</i> The name of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationArn</i> The Amazon Resource Name (ARN) of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceRole</i> The ARN of an IAM service role that Elastic Beanstalk has permission to assume. The <code>ServiceRole</code> property is required the first time that you provide a <code>VersionLifecycleConfig</code> for the application in one of the supporting calls (<code>CreateApplication</code> or <code>UpdateApplicationResourceLifecycle</code>). After you provide it once, in either one of the calls, Elastic Beanstalk persists the Service Role with the application, and you don't need to specify it again in subsequent <code>UpdateApplicationResourceLifecycle</code> calls. You can, however, specify it in subsequent calls to change the Service Role to another value.<br/>The application version lifecycle configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VersionLifecycleConfig</b>: The application version lifecycle configuration.        """
        pass

    def create_application_version(self, VersionLabel: str, ApplicationName: str, Description: str=None, SourceBuildInformation: SourceBuildInformation=None, SourceBundle: S3Location=None, BuildConfiguration: BuildConfiguration=None, AutoCreateApplication: AutoCreateApplication=None, Process: ApplicationVersionProccess=None, Tags: List[Tag]=None) -> ApplicationVersionDescriptionMessage:
        """Creates an application version for the specified application. You can create an application version from a source bundle in Amazon S3, a commit in AWS CodeCommit, or the output of an AWS CodeBuild build as follows: Specify a commit in an AWS CodeCommit repository with <code>SourceBuildInformation</code>. Specify a build in an AWS CodeBuild with <code>SourceBuildInformation</code> and <code>BuildConfiguration</code>. Specify a source bundle in S3 with <code>SourceBundle</code>  Omit both <code>SourceBuildInformation</code> and <code>SourceBundle</code> to use the default sample application. <note> Once you create an application version with a specified Amazon S3 bucket and key location, you cannot change that Amazon S3 location. If you change the Amazon S3 location, you receive an exception when you attempt to launch an environment from the application version. </note>

        :param self:
        :param Tags:  Specifies the tags applied to the application version. Elastic Beanstalk applies these tags only to the application version. Environments that use the application version don't inherit the tags.<br/>Process:  Pre-processes and validates the environment manifest (<code>env.yaml</code>) and configuration files (<code>*.config</code> files in the <code>.ebextensions</code> folder) in the source bundle. Validating configuration files can identify issues prior to deploying the application version to an environment. You must turn processing on for application versions that you create using AWS CodeBuild or AWS CodeCommit. For application versions built from a source bundle in Amazon S3, processing is optional. <note> The <code>Process</code> option validates Elastic Beanstalk configuration files. It doesn't validate your application's configuration files, like proxy server or Docker configuration. </note><br/>AutoCreateApplication:  Set to <code>true</code> to create an application with the specified name if it doesn't already exist.<br/>BuildConfiguration:  Settings for an AWS CodeBuild build.:<br/>SourceBundle:  The Amazon S3 bucket and key that identify the location of the source bundle for this version. <note> The Amazon S3 bucket must be in the same region as the environment. </note> Specify a source bundle in S3 or a commit in an AWS CodeCommit repository (with <code>SourceBuildInformation</code>), but not both. If neither <code>SourceBundle</code> nor <code>SourceBuildInformation</code> are provided, Elastic Beanstalk uses a sample application.:<br/>SourceBuildInformation:  Specify a commit in an AWS CodeCommit Git repository to use as the source code for the application version.:<br/>Description:  Describes this version.<br/>VersionLabel:  A label identifying this version. Constraint: Must be unique per application. If an application version already exists with this label for the specified application, AWS Elastic Beanstalk returns an <code>InvalidParameterValue</code> error. <br/>ApplicationName:   The name of the application. If no application is found with this name, and <code>AutoCreateApplication</code> is <code>false</code>, returns an <code>InvalidParameterValue</code> error. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceLocation</i> The location of the source code, as a formatted string, depending on the value of <code>SourceRepository</code>  <ul> <li> For <code>CodeCommit</code>, the format is the repository name and commit ID, separated by a forward slash. For example, <code>my-git-repo/265cfa0cf6af46153527f55d6503ec030551f57a</code>. </li> <li> For <code>S3</code>, the format is the S3 bucket name and object key, separated by a forward slash. For example, <code>my-s3-bucket/Folders/my-source-file</code>. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceRepository</i> Location where the repository is stored. <ul> <li>  <code>CodeCommit</code>  </li> <li>  <code>S3</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceType</i> The type of repository. <ul> <li>  <code>Git</code>  </li> <li>  <code>Zip</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Key</i> The Amazon S3 key where the data is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Bucket</i> The Amazon S3 bucket where the data is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeoutInMinutes</i> How long in minutes, from 5 to 480 (8 hours), for AWS CodeBuild to wait until timing out any related build that does not get marked as completed. The default is 60 minutes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Image</i> The ID of the Docker image to use for this build project.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ComputeType</i> Information about the compute resources the build project will use. <ul> <li>  <code>BUILD_GENERAL1_SMALL: Use up to 3 GB memory and 2 vCPUs for builds</code>  </li> <li>  <code>BUILD_GENERAL1_MEDIUM: Use up to 7 GB memory and 4 vCPUs for builds</code>  </li> <li>  <code>BUILD_GENERAL1_LARGE: Use up to 15 GB memory and 8 vCPUs for builds</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CodeBuildServiceRole</i> The Amazon Resource Name (ARN) of the AWS Identity and Access Management (IAM) role that enables AWS CodeBuild to interact with dependent AWS services on behalf of the AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ArtifactName</i> The name of the artifact of the CodeBuild build. If provided, Elastic Beanstalk stores the build artifact in the S3 location <i>S3-bucket</i>/resources/<i>application-name</i>/codebuild/codebuild-<i>version-label</i>-<i>artifact-name</i>.zip. If not provided, Elastic Beanstalk stores the build artifact in the S3 location <i>S3-bucket</i>/resources/<i>application-name</i>/codebuild/codebuild-<i>version-label</i>.zip. <br/>
        :return: ApplicationVersion:   The <a>ApplicationVersionDescription</a> of the application version. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The processing status of the application version. Reflects the state of the application version during its creation. Many of the values are only applicable if you specified <code>True</code> for the <code>Process</code> parameter of the <code>CreateApplicationVersion</code> action. The following list describes the possible values. <ul> <li>  <code>Unprocessed</code> – Application version wasn't pre-processed or validated. Elastic Beanstalk will validate configuration files during deployment of the application version to an environment. </li> <li>  <code>Processing</code> – Elastic Beanstalk is currently processing the application version. </li> <li>  <code>Building</code> – Application version is currently undergoing an AWS CodeBuild build. </li> <li>  <code>Processed</code> – Elastic Beanstalk was successfully pre-processed and validated. </li> <li>  <code>Failed</code> – Either the AWS CodeBuild build failed or configuration files didn't pass validation. This application version isn't usable. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DateUpdated</i> The last modified date of the application version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DateCreated</i> The creation date of the application version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceBundle</i> The storage location of the application version's source bundle in Amazon S3.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SourceBundle</b>: The storage location of the application version's source bundle in Amazon S3.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BuildArn</i> Reference to the artifact from the AWS CodeBuild build.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceBuildInformation</i> If the version's source code was retrieved from AWS CodeCommit, the location of the source code for the application version.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SourceBuildInformation</b>: If the version's source code was retrieved from AWS CodeCommit, the location of the source code for the application version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VersionLabel</i> A unique identifier for the application version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the application version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationName</i> The name of the application to which the application version belongs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationVersionArn</i> The Amazon Resource Name (ARN) of the application version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceLocation</i> The location of the source code, as a formatted string, depending on the value of <code>SourceRepository</code>  <ul> <li> For <code>CodeCommit</code>, the format is the repository name and commit ID, separated by a forward slash. For example, <code>my-git-repo/265cfa0cf6af46153527f55d6503ec030551f57a</code>. </li> <li> For <code>S3</code>, the format is the S3 bucket name and object key, separated by a forward slash. For example, <code>my-s3-bucket/Folders/my-source-file</code>. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceRepository</i> Location where the repository is stored. <ul> <li>  <code>CodeCommit</code>  </li> <li>  <code>S3</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceType</i> The type of repository. <ul> <li>  <code>Git</code>  </li> <li>  <code>Zip</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Key</i> The Amazon S3 key where the data is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Bucket</i> The Amazon S3 bucket where the data is located.<br/>        """
        pass

    def create_configuration_template(self, TemplateName: str, ApplicationName: str, SolutionStackName: str=None, PlatformArn: str=None, SourceConfiguration: SourceConfiguration=None, EnvironmentId: str=None, Description: str=None, OptionSettings: List[ConfigurationOptionSetting]=None, Tags: List[Tag]=None) -> ConfigurationSettingsDescription:
        """Creates a configuration template. Templates are associated with a specific application and are used to deploy different versions of the application with the same configuration settings. Templates aren't associated with any environment. The <code>EnvironmentName</code> response element is always <code>null</code>. Related Topics <ul> <li>  <a>DescribeConfigurationOptions</a>  </li> <li>  <a>DescribeConfigurationSettings</a>  </li> <li>  <a>ListAvailableSolutionStacks</a>  </li> </ul>

        :param self:
        :param Tags:  Specifies the tags applied to the configuration template.<br/>OptionSettings:  If specified, AWS Elastic Beanstalk sets the specified configuration option to the requested value. The new value overrides the value obtained from the solution stack or the source configuration template.<br/>Description:  Describes this configuration.<br/>EnvironmentId:  The ID of the environment used with this configuration template.<br/>SourceConfiguration:  If specified, AWS Elastic Beanstalk uses the configuration values from the specified configuration template to create a new configuration.  Values specified in the <code>OptionSettings</code> parameter of this call overrides any values obtained from the <code>SourceConfiguration</code>.   If no configuration template is found, returns an <code>InvalidParameterValue</code> error.   Constraint: If both the solution stack name parameter and the source configuration parameters are specified, the solution stack of the source configuration template must match the specified solution stack name or else AWS Elastic Beanstalk returns an <code>InvalidParameterCombination</code> error. :<br/>PlatformArn:  The ARN of the custom platform.<br/>SolutionStackName:  The name of the solution stack used by this configuration. The solution stack specifies the operating system, architecture, and application server for a configuration template. It determines the set of configuration options as well as the possible and default values.  Use <a>ListAvailableSolutionStacks</a> to obtain a list of available solution stacks.   A solution stack name or a source configuration parameter must be specified, otherwise AWS Elastic Beanstalk returns an <code>InvalidParameterValue</code> error.  If a solution stack name is not specified and the source configuration parameter is specified, AWS Elastic Beanstalk uses the same solution stack as the source configuration template.<br/>TemplateName:  The name of the configuration template. Constraint: This name must be unique per application. Default: If a configuration template already exists with this name, AWS Elastic Beanstalk returns an <code>InvalidParameterValue</code> error. <br/>ApplicationName:  The name of the application to associate with this configuration template. If no application is found with this name, AWS Elastic Beanstalk returns an <code>InvalidParameterValue</code> error. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TemplateName</i> The name of the configuration template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationName</i> The name of the application associated with the configuration.<br/>
        :return: OptionSettings:  A list of the configuration options and their values in this configuration set.<br/>DateUpdated:  The date (in UTC time) when this configuration set was last modified.<br/>DateCreated:  The date (in UTC time) when this configuration set was created.<br/>DeploymentStatus:   If this configuration set is associated with an environment, the <code>DeploymentStatus</code> parameter indicates the deployment status of this configuration set:  <ul> <li>  <code>null</code>: This configuration is not associated with a running environment. </li> <li>  <code>pending</code>: This is a draft configuration that is not deployed to the associated environment but is in the process of deploying. </li> <li>  <code>deployed</code>: This is the configuration that is currently deployed to the associated running environment. </li> <li>  <code>failed</code>: This is a draft configuration that failed to successfully deploy. </li> </ul><br/>EnvironmentName:   If not <code>null</code>, the name of the environment for this configuration set. <br/>Description:  Describes this configuration set.<br/>TemplateName:   If not <code>null</code>, the name of the configuration template for this configuration set. <br/>ApplicationName:  The name of the application associated with this configuration set.<br/>PlatformArn:  The ARN of the platform.<br/>SolutionStackName:  The name of the solution stack this configuration set uses.<br/>        """
        pass

    def create_environment(self, ApplicationName: str, EnvironmentName: str=None, GroupName: str=None, Description: str=None, CNAMEPrefix: str=None, Tier: EnvironmentTier=None, Tags: List[Tag]=None, VersionLabel: str=None, TemplateName: str=None, SolutionStackName: str=None, PlatformArn: str=None, OptionSettings: List[ConfigurationOptionSetting]=None, OptionsToRemove: List[OptionSpecification]=None) -> EnvironmentDescription:
        """Launches an environment for the specified application using the specified configuration.

        :param self:
        :param OptionsToRemove:  A list of custom user-defined configuration options to remove from the configuration set for this new environment.<br/>OptionSettings:  If specified, AWS Elastic Beanstalk sets the specified configuration options to the requested value in the configuration set for the new environment. These override the values obtained from the solution stack or the configuration template.<br/>PlatformArn:  The ARN of the platform.<br/>SolutionStackName:  This is an alternative to specifying a template name. If specified, AWS Elastic Beanstalk sets the configuration values to the default values associated with the specified solution stack. For a list of current solution stacks, see <a href="https://docs.aws.amazon.com/elasticbeanstalk/latest/dg/concepts.platforms.html">Elastic Beanstalk Supported Platforms</a>.<br/>TemplateName:   The name of the configuration template to use in deployment. If no configuration template is found with this name, AWS Elastic Beanstalk returns an <code>InvalidParameterValue</code> error. <br/>VersionLabel:  The name of the application version to deploy.  If the specified application has no associated application versions, AWS Elastic Beanstalk <code>UpdateEnvironment</code> returns an <code>InvalidParameterValue</code> error.  Default: If not specified, AWS Elastic Beanstalk attempts to launch the sample application in the container.<br/>Tags:  Specifies the tags applied to resources in the environment.<br/>Tier:  This specifies the tier to use for creating this environment.:<br/>CNAMEPrefix:  If specified, the environment attempts to use this value as the prefix for the CNAME. If not specified, the CNAME is generated automatically by appending a random alphanumeric string to the environment name.<br/>Description:  Describes this environment.<br/>GroupName:  The name of the group to which the target environment belongs. Specify a group name only if the environment's name is specified in an environment manifest and not with the environment name parameter. See <a href="https://docs.aws.amazon.com/elasticbeanstalk/latest/dg/environment-cfg-manifest.html">Environment Manifest (env.yaml)</a> for details.<br/>EnvironmentName:  A unique name for the deployment environment. Used in the application URL. Constraint: Must be from 4 to 40 characters in length. The name can contain only letters, numbers, and hyphens. It cannot start or end with a hyphen. This name must be unique within a region in your account. If the specified name already exists in the region, AWS Elastic Beanstalk returns an <code>InvalidParameterValue</code> error.  Default: If the CNAME parameter is not specified, the environment name becomes part of the CNAME, and therefore part of the visible URL for your application.<br/>ApplicationName:  The name of the application that contains the version to be deployed.  If no application is found with this name, <code>CreateEnvironment</code> returns an <code>InvalidParameterValue</code> error. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version of this environment tier. When you don't set a value to it, Elastic Beanstalk uses the latest compatible worker tier version. <note> This member is deprecated. Any specific version that you set may become out of date. We recommend leaving it unspecified. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of this environment tier. Valid values: <ul> <li> For <i>Web server tier</i> – <code>Standard</code>  </li> <li> For <i>Worker tier</i> – <code>SQS/HTTP</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of this environment tier. Valid values: <ul> <li> For <i>Web server tier</i> – <code>WebServer</code>  </li> <li> For <i>Worker tier</i> – <code>Worker</code>  </li> </ul><br/>
        :return: EnvironmentArn:  The environment's Amazon Resource Name (ARN), which can be used in other API requests that require an ARN.<br/>EnvironmentLinks:  A list of links to other environments in the same group.<br/>Tier:  Describes the current tier of this environment.:<br/>Resources:  The description of the AWS resources used by this environment.:<br/>HealthStatus:  Returns the health status of the application running in your environment. For more information, see <a href="https://docs.aws.amazon.com/elasticbeanstalk/latest/dg/health-enhanced-status.html">Health Colors and Statuses</a>.<br/>Health:  Describes the health status of the environment. AWS Elastic Beanstalk indicates the failure levels for a running environment: <ul> <li>  <code>Red</code>: Indicates the environment is not responsive. Occurs when three or more consecutive failures occur for an environment. </li> <li>  <code>Yellow</code>: Indicates that something is wrong. Occurs when two consecutive failures occur for an environment. </li> <li>  <code>Green</code>: Indicates the environment is healthy and fully functional. </li> <li>  <code>Grey</code>: Default health for a new environment. The environment is not fully launched and health checks have not started or health checks are suspended during an <code>UpdateEnvironment</code> or <code>RestartEnvironment</code> request. </li> </ul>  Default: <code>Grey</code> <br/>AbortableOperationInProgress:  Indicates if there is an in-progress environment configuration update or application version deployment that you can cancel.  <code>true:</code> There is an update in progress.   <code>false:</code> There are no updates currently in progress. <br/>Status:  The current operational status of the environment: <ul> <li>  <code>Launching</code>: Environment is in the process of initial deployment. </li> <li>  <code>Updating</code>: Environment is in the process of updating its configuration settings or application version. </li> <li>  <code>Ready</code>: Environment is available to have an action performed on it, such as update or terminate. </li> <li>  <code>Terminating</code>: Environment is in the shut-down process. </li> <li>  <code>Terminated</code>: Environment is not running. </li> </ul><br/>DateUpdated:  The last modified date for this environment.<br/>DateCreated:  The creation date for this environment.<br/>CNAME:  The URL to the CNAME for this environment.<br/>EndpointURL:  For load-balanced, autoscaling environments, the URL to the LoadBalancer. For single-instance environments, the IP address of the instance.<br/>Description:  Describes this environment.<br/>TemplateName:  The name of the configuration template used to originally launch this environment.<br/>PlatformArn:  The ARN of the platform.<br/>SolutionStackName:   The name of the <code>SolutionStack</code> deployed with this environment. <br/>VersionLabel:  The application version deployed in this environment.<br/>ApplicationName:  The name of the application associated with this environment.<br/>EnvironmentId:  The ID of this environment.<br/>EnvironmentName:  The name of this environment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LoadBalancer</i> Describes the LoadBalancer.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LoadBalancer</b>: Describes the LoadBalancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Listeners</i> A list of Listeners used by the LoadBalancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Domain</i> The domain name of the LoadBalancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LoadBalancerName</i> The name of the LoadBalancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version of this environment tier. When you don't set a value to it, Elastic Beanstalk uses the latest compatible worker tier version. <note> This member is deprecated. Any specific version that you set may become out of date. We recommend leaving it unspecified. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of this environment tier. Valid values: <ul> <li> For <i>Web server tier</i> – <code>Standard</code>  </li> <li> For <i>Worker tier</i> – <code>SQS/HTTP</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of this environment tier. Valid values: <ul> <li> For <i>Web server tier</i> – <code>WebServer</code>  </li> <li> For <i>Worker tier</i> – <code>Worker</code>  </li> </ul><br/>        """
        pass

    def create_platform_version(self, PlatformDefinitionBundle: S3Location, PlatformVersion: str, PlatformName: str, EnvironmentName: str=None, OptionSettings: List[ConfigurationOptionSetting]=None, Tags: List[Tag]=None) -> CreatePlatformVersionResult:
        """Create a new version of your custom platform.

        :param self:
        :param Tags:  Specifies the tags applied to the new platform version. Elastic Beanstalk applies these tags only to the platform version. Environments that you create using the platform version don't inherit the tags.<br/>OptionSettings:  The configuration option settings to apply to the builder environment.<br/>EnvironmentName:  The name of the builder environment.<br/>PlatformDefinitionBundle:  The location of the platform definition archive in Amazon S3.:<br/>PlatformVersion:  The number, such as 1.0.2, for the new platform version.<br/>PlatformName:  The name of your custom platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Key</i> The Amazon S3 key where the data is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Bucket</i> The Amazon S3 bucket where the data is located.<br/>
        :return: Builder:  The builder used to create the custom platform.:<br/>PlatformSummary:  Detailed information about the new version of the custom platform.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedAddonList</i> The additions associated with the platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedTierList</i> The tiers in which the platform runs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OperatingSystemVersion</i> The version of the operating system used by the platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OperatingSystemName</i> The operating system used by the platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlatformCategory</i> The category of platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlatformStatus</i> The status of the platform. You can create an environment from the platform once it is ready.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlatformOwner</i> The AWS account ID of the person who created the platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlatformArn</i> The ARN of the platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ARN</i> The ARN of the builder.<br/>        """
        pass

    def create_storage_location(self) -> CreateStorageLocationResultMessage:
        """Creates a bucket in Amazon S3 to store application versions, logs, and other files used by Elastic Beanstalk environments. The Elastic Beanstalk console and EB CLI call this API the first time you create an environment in a region. If the storage location already exists, <code>CreateStorageLocation</code> still returns the bucket name but does not create a new bucket.

        :param self:
        :param 
        :return: S3Bucket:  The name of the Amazon S3 bucket created.<br/>        """
        pass

    def delete_application(self, ApplicationName: str, TerminateEnvByForce: TerminateEnvForce=None):
        """Deletes the specified application along with all associated versions and configurations. The application versions will not be deleted from your Amazon S3 bucket. <note> You cannot delete an application that has a running environment. </note>

        :param self:
        :param TerminateEnvByForce:  When set to true, running environments will be terminated before deleting the application.<br/>ApplicationName:  The name of the application to delete.<br/>
        :return:         """
        pass

    def delete_application_version(self, VersionLabel: str, ApplicationName: str, DeleteSourceBundle: DeleteSourceBundle=None):
        """Deletes the specified version from the specified application. <note> You cannot delete an application version that is associated with a running environment. </note>

        :param self:
        :param DeleteSourceBundle:  Set to <code>true</code> to delete the source bundle from your storage bucket. Otherwise, the application version is deleted only from Elastic Beanstalk and the source bundle remains in Amazon S3.<br/>VersionLabel:  The label of the version to delete.<br/>ApplicationName:  The name of the application to which the version belongs.<br/>
        :return:         """
        pass

    def delete_configuration_template(self, TemplateName: str, ApplicationName: str):
        """Deletes the specified configuration template. <note> When you launch an environment using a configuration template, the environment gets a copy of the template. You can delete or modify the environment's copy of the template without affecting the running environment. </note>

        :param self:
        :param TemplateName:  The name of the configuration template to delete.<br/>ApplicationName:  The name of the application to delete the configuration template from.<br/>
        :return:         """
        pass

    def delete_environment_configuration(self, EnvironmentName: str, ApplicationName: str):
        """Deletes the draft configuration associated with the running environment. Updating a running environment with any configuration changes creates a draft configuration set. You can get the draft configuration using <a>DescribeConfigurationSettings</a> while the update is in progress or if the update fails. The <code>DeploymentStatus</code> for the draft configuration indicates whether the deployment is in process or has failed. The draft configuration remains in existence until it is deleted with this action.

        :param self:
        :param EnvironmentName:  The name of the environment to delete the draft configuration from.<br/>ApplicationName:  The name of the application the environment is associated with.<br/>
        :return:         """
        pass

    def delete_platform_version(self, PlatformArn: str=None) -> DeletePlatformVersionResult:
        """Deletes the specified version of a custom platform.

        :param self:
        :param PlatformArn:  The ARN of the version of the custom platform.<br/>
        :return: PlatformSummary:  Detailed information about the version of the custom platform.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedAddonList</i> The additions associated with the platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedTierList</i> The tiers in which the platform runs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OperatingSystemVersion</i> The version of the operating system used by the platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OperatingSystemName</i> The operating system used by the platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlatformCategory</i> The category of platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlatformStatus</i> The status of the platform. You can create an environment from the platform once it is ready.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlatformOwner</i> The AWS account ID of the person who created the platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlatformArn</i> The ARN of the platform.<br/>        """
        pass

    def describe_account_attributes(self) -> DescribeAccountAttributesResult:
        """Returns attributes related to AWS Elastic Beanstalk that are associated with the calling AWS account. The result currently has one set of attributes—resource quotas.

        :param self:
        :param 
        :return: ResourceQuotas:  The Elastic Beanstalk resource quotas associated with the calling AWS account.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomPlatformQuota</i> The quota for custom platforms in the AWS account.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CustomPlatformQuota</b>: The quota for custom platforms in the AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigurationTemplateQuota</i> The quota for configuration templates in the AWS account.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ConfigurationTemplateQuota</b>: The quota for configuration templates in the AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnvironmentQuota</i> The quota for environments in the AWS account.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EnvironmentQuota</b>: The quota for environments in the AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationVersionQuota</i> The quota for application versions in the AWS account.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ApplicationVersionQuota</b>: The quota for application versions in the AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationQuota</i> The quota for applications in the AWS account.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ApplicationQuota</b>: The quota for applications in the AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Maximum</i> The maximum number of instances of this Elastic Beanstalk resource type that an AWS account can use.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Maximum</i> The maximum number of instances of this Elastic Beanstalk resource type that an AWS account can use.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Maximum</i> The maximum number of instances of this Elastic Beanstalk resource type that an AWS account can use.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Maximum</i> The maximum number of instances of this Elastic Beanstalk resource type that an AWS account can use.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Maximum</i> The maximum number of instances of this Elastic Beanstalk resource type that an AWS account can use.<br/>        """
        pass

    def describe_application_versions(self, ApplicationName: str=None, VersionLabels: List[VersionLabel]=None, MaxRecords: int=None, NextToken: str=None) -> ApplicationVersionDescriptionsMessage:
        """Retrieve a list of application versions.

        :param self:
        :param NextToken:  For a paginated request. Specify a token from a previous response page to retrieve the next response page. All other parameter values must be identical to the ones specified in the initial request. If no <code>NextToken</code> is specified, the first page is retrieved.<br/>MaxRecords:  For a paginated request. Specify a maximum number of application versions to include in each response. If no <code>MaxRecords</code> is specified, all available application versions are retrieved in a single response.<br/>VersionLabels:  Specify a version label to show a specific application version.<br/>ApplicationName:  Specify an application name to show only application versions for that application.<br/>
        :return: NextToken:  In a paginated request, the token that you can pass in a subsequent request to get the next response page.<br/>ApplicationVersions:  List of <code>ApplicationVersionDescription</code> objects sorted in order of creation.<br/>        """
        pass

    def describe_applications(self, ApplicationNames: List[ApplicationName]=None) -> ApplicationDescriptionsMessage:
        """Returns the descriptions of existing applications.

        :param self:
        :param ApplicationNames:  If specified, AWS Elastic Beanstalk restricts the returned descriptions to only include those with the specified names.<br/>
        :return: Applications:  This parameter contains a list of <a>ApplicationDescription</a>.<br/>        """
        pass

    def describe_configuration_options(self, ApplicationName: str=None, TemplateName: str=None, EnvironmentName: str=None, SolutionStackName: str=None, PlatformArn: str=None, Options: List[OptionSpecification]=None) -> ConfigurationOptionsDescription:
        """Describes the configuration options that are used in a particular configuration template or environment, or that a specified solution stack defines. The description includes the values the options, their default values, and an indication of the required action on a running environment if an option value is changed.

        :param self:
        :param Options:  If specified, restricts the descriptions to only the specified options.<br/>PlatformArn:  The ARN of the custom platform.<br/>SolutionStackName:  The name of the solution stack whose configuration options you want to describe.<br/>EnvironmentName:  The name of the environment whose configuration options you want to describe.<br/>TemplateName:  The name of the configuration template whose configuration options you want to describe.<br/>ApplicationName:  The name of the application associated with the configuration template or environment. Only needed if you want to describe the configuration options associated with either the configuration template or environment.<br/>
        :return: Options:   A list of <a>ConfigurationOptionDescription</a>. <br/>PlatformArn:  The ARN of the platform.<br/>SolutionStackName:  The name of the solution stack these configuration options belong to.<br/>        """
        pass

    def describe_configuration_settings(self, ApplicationName: str, TemplateName: str=None, EnvironmentName: str=None) -> ConfigurationSettingsDescriptions:
        """Returns a description of the settings for the specified configuration set, that is, either a configuration template or the configuration set associated with a running environment. When describing the settings for the configuration set associated with a running environment, it is possible to receive two sets of setting descriptions. One is the deployed configuration set, and the other is a draft configuration of an environment that is either in the process of deployment or that failed to deploy. Related Topics <ul> <li>  <a>DeleteEnvironmentConfiguration</a>  </li> </ul>

        :param self:
        :param EnvironmentName:  The name of the environment to describe.  Condition: You must specify either this or a TemplateName, but not both. If you specify both, AWS Elastic Beanstalk returns an <code>InvalidParameterCombination</code> error. If you do not specify either, AWS Elastic Beanstalk returns <code>MissingRequiredParameter</code> error. <br/>TemplateName:  The name of the configuration template to describe.  Conditional: You must specify either this parameter or an EnvironmentName, but not both. If you specify both, AWS Elastic Beanstalk returns an <code>InvalidParameterCombination</code> error. If you do not specify either, AWS Elastic Beanstalk returns a <code>MissingRequiredParameter</code> error. <br/>ApplicationName:  The application for the environment or configuration template.<br/>
        :return: ConfigurationSettings:   A list of <a>ConfigurationSettingsDescription</a>. <br/>        """
        pass

    def describe_environment_health(self, EnvironmentName: str=None, EnvironmentId: str=None, AttributeNames: List[EnvironmentHealthAttribute]=None) -> DescribeEnvironmentHealthResult:
        """Returns information about the overall health of the specified environment. The <b>DescribeEnvironmentHealth</b> operation is only available with AWS Elastic Beanstalk Enhanced Health.

        :param self:
        :param AttributeNames:  Specify the response elements to return. To retrieve all attributes, set to <code>All</code>. If no attribute names are specified, returns the name of the environment.<br/>EnvironmentId:  Specify the environment by ID. You must specify either this or an EnvironmentName, or both.<br/>EnvironmentName:  Specify the environment by name. You must specify either this or an EnvironmentName, or both.<br/>
        :return: RefreshedAt:  The date and time that the health information was retrieved.<br/>InstancesHealth:  Summary health information for the instances in the environment.:<br/>ApplicationMetrics:  Application request metrics for the environment.:<br/>Causes:  Descriptions of the data that contributed to the environment's current health status.<br/>Color:  The <a href="https://docs.aws.amazon.com/elasticbeanstalk/latest/dg/health-enhanced-status.html">health color</a> of the environment.<br/>Status:  The environment's operational status. <code>Ready</code>, <code>Launching</code>, <code>Updating</code>, <code>Terminating</code>, or <code>Terminated</code>.<br/>HealthStatus:  The <a href="https://docs.aws.amazon.com/elasticbeanstalk/latest/dg/health-enhanced-status.html">health status</a> of the environment. For example, <code>Ok</code>.<br/>EnvironmentName:  The environment's name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Latency</i> Represents the average latency for the slowest X percent of requests over the last 10 seconds. Latencies are in seconds with one millisecond resolution.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Latency</b>: Represents the average latency for the slowest X percent of requests over the last 10 seconds. Latencies are in seconds with one millisecond resolution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusCodes</i> Represents the percentage of requests over the last 10 seconds that resulted in each type of status code response.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StatusCodes</b>: Represents the percentage of requests over the last 10 seconds that resulted in each type of status code response.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestCount</i> Average number of requests handled by the web server per second over the last 10 seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Duration</i> The amount of time that the metrics cover (usually 10 seconds). For example, you might have 5 requests (<code>request_count</code>) within the most recent time slice of 10 seconds (<code>duration</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status5xx</i> The percentage of requests over the last 10 seconds that resulted in a 5xx (500, 501, etc.) status code.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status4xx</i> The percentage of requests over the last 10 seconds that resulted in a 4xx (400, 401, etc.) status code.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status3xx</i> The percentage of requests over the last 10 seconds that resulted in a 3xx (300, 301, etc.) status code.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status2xx</i> The percentage of requests over the last 10 seconds that resulted in a 2xx (200, 201, etc.) status code.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>P10</i> The average latency for the slowest 90 percent of requests over the last 10 seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>P50</i> The average latency for the slowest 50 percent of requests over the last 10 seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>P75</i> The average latency for the slowest 25 percent of requests over the last 10 seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>P85</i> The average latency for the slowest 15 percent of requests over the last 10 seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>P90</i> The average latency for the slowest 10 percent of requests over the last 10 seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>P95</i> The average latency for the slowest 5 percent of requests over the last 10 seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>P99</i> The average latency for the slowest 1 percent of requests over the last 10 seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>P999</i> The average latency for the slowest 0.1 percent of requests over the last 10 seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Severe</i>  <b>Red.</b> The health agent is reporting a very high number of request failures or other issues for an instance or environment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Degraded</i>  <b>Red.</b> The health agent is reporting a high number of request failures or other issues for an instance or environment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Warning</i>  <b>Yellow.</b> The health agent is reporting a moderate number of request failures or other issues for an instance or environment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Info</i>  <b>Green.</b> An operation is in progress on an instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Ok</i>  <b>Green.</b> An instance is passing health checks and the health agent is not reporting any problems.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Pending</i>  <b>Grey.</b> An operation is in progress on an instance within the command timeout.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Unknown</i>  <b>Grey.</b> AWS Elastic Beanstalk and the health agent are reporting an insufficient amount of data on an instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NoData</i>  <b>Grey.</b> AWS Elastic Beanstalk and the health agent are reporting no data on an instance.<br/>        """
        pass

    def describe_environment_managed_action_history(self, EnvironmentId: str=None, EnvironmentName: str=None, NextToken: str=None, MaxItems: int=None) -> DescribeEnvironmentManagedActionHistoryResult:
        """Lists an environment's completed and failed managed actions.

        :param self:
        :param MaxItems:  The maximum number of items to return for a single request.<br/>NextToken:  The pagination token returned by a previous request.<br/>EnvironmentName:  The name of the target environment.<br/>EnvironmentId:  The environment ID of the target environment.<br/>
        :return: NextToken:  A pagination token that you pass to <a>DescribeEnvironmentManagedActionHistory</a> to get the next page of results.<br/>ManagedActionHistoryItems:  A list of completed and failed managed actions.<br/>        """
        pass

    def describe_environment_managed_actions(self, EnvironmentName: str=None, EnvironmentId: str=None, Status: str=None) -> DescribeEnvironmentManagedActionsResult:
        """Lists an environment's upcoming and in-progress managed actions.

        :param self:
        :param Status:  To show only actions with a particular status, specify a status.<br/>EnvironmentId:  The environment ID of the target environment.<br/>EnvironmentName:  The name of the target environment.<br/>
        :return: ManagedActions:  A list of upcoming and in-progress managed actions.<br/>        """
        pass

    def describe_environment_resources(self, EnvironmentId: str=None, EnvironmentName: str=None) -> EnvironmentResourceDescriptionsMessage:
        """Returns AWS resources for this environment.

        :param self:
        :param EnvironmentName:  The name of the environment to retrieve AWS resource usage data.  Condition: You must specify either this or an EnvironmentId, or both. If you do not specify either, AWS Elastic Beanstalk returns <code>MissingRequiredParameter</code> error. <br/>EnvironmentId:  The ID of the environment to retrieve AWS resource usage data.  Condition: You must specify either this or an EnvironmentName, or both. If you do not specify either, AWS Elastic Beanstalk returns <code>MissingRequiredParameter</code> error. <br/>
        :return: EnvironmentResources:   A list of <a>EnvironmentResourceDescription</a>. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Queues</i> The queues used by this environment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Triggers</i> The <code>AutoScaling</code> triggers in use by this environment. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LoadBalancers</i> The LoadBalancers in use by this environment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LaunchTemplates</i> The Amazon EC2 launch templates in use by this environment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LaunchConfigurations</i> The Auto Scaling launch configurations in use by this environment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Instances</i> The Amazon EC2 instances used by this environment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoScalingGroups</i>  The <code>AutoScalingGroups</code> used by this environment. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnvironmentName</i> The name of the environment.<br/>        """
        pass

    def describe_environments(self, ApplicationName: str=None, VersionLabel: str=None, EnvironmentIds: List[EnvironmentId]=None, EnvironmentNames: List[EnvironmentName]=None, IncludeDeleted: IncludeDeleted=None, IncludedDeletedBackTo: IncludeDeletedBackTo=None, MaxRecords: int=None, NextToken: str=None) -> EnvironmentDescriptionsMessage:
        """Returns descriptions for existing environments.

        :param self:
        :param NextToken:  For a paginated request. Specify a token from a previous response page to retrieve the next response page. All other parameter values must be identical to the ones specified in the initial request. If no <code>NextToken</code> is specified, the first page is retrieved.<br/>MaxRecords:  For a paginated request. Specify a maximum number of environments to include in each response. If no <code>MaxRecords</code> is specified, all available environments are retrieved in a single response.<br/>IncludedDeletedBackTo:   If specified when <code>IncludeDeleted</code> is set to <code>true</code>, then environments deleted after this date are displayed. <br/>IncludeDeleted:  Indicates whether to include deleted environments:  <code>true</code>: Environments that have been deleted after <code>IncludedDeletedBackTo</code> are displayed.  <code>false</code>: Do not include deleted environments.<br/>EnvironmentNames:  If specified, AWS Elastic Beanstalk restricts the returned descriptions to include only those that have the specified names.<br/>EnvironmentIds:  If specified, AWS Elastic Beanstalk restricts the returned descriptions to include only those that have the specified IDs.<br/>VersionLabel:  If specified, AWS Elastic Beanstalk restricts the returned descriptions to include only those that are associated with this application version.<br/>ApplicationName:  If specified, AWS Elastic Beanstalk restricts the returned descriptions to include only those that are associated with this application.<br/>
        :return: NextToken:  In a paginated request, the token that you can pass in a subsequent request to get the next response page.<br/>Environments:   Returns an <a>EnvironmentDescription</a> list. <br/>        """
        pass

    def describe_events(self, ApplicationName: str=None, VersionLabel: str=None, TemplateName: str=None, EnvironmentId: str=None, EnvironmentName: str=None, PlatformArn: str=None, RequestId: str=None, Severity: str=None, StartTime: TimeFilterStart=None, EndTime: TimeFilterEnd=None, MaxRecords: int=None, NextToken: str=None) -> EventDescriptionsMessage:
        """Returns list of event descriptions matching criteria up to the last 6 weeks. <note> This action returns the most recent 1,000 events from the specified <code>NextToken</code>. </note>

        :param self:
        :param NextToken:  Pagination token. If specified, the events return the next batch of results.<br/>MaxRecords:  Specifies the maximum number of events that can be returned, beginning with the most recent event.<br/>EndTime:   If specified, AWS Elastic Beanstalk restricts the returned descriptions to those that occur up to, but not including, the <code>EndTime</code>. <br/>StartTime:  If specified, AWS Elastic Beanstalk restricts the returned descriptions to those that occur on or after this time.<br/>Severity:  If specified, limits the events returned from this call to include only those with the specified severity or higher.<br/>RequestId:  If specified, AWS Elastic Beanstalk restricts the described events to include only those associated with this request ID.<br/>PlatformArn:  The ARN of the version of the custom platform.<br/>EnvironmentName:  If specified, AWS Elastic Beanstalk restricts the returned descriptions to those associated with this environment.<br/>EnvironmentId:  If specified, AWS Elastic Beanstalk restricts the returned descriptions to those associated with this environment.<br/>TemplateName:  If specified, AWS Elastic Beanstalk restricts the returned descriptions to those that are associated with this environment configuration.<br/>VersionLabel:  If specified, AWS Elastic Beanstalk restricts the returned descriptions to those associated with this application version.<br/>ApplicationName:  If specified, AWS Elastic Beanstalk restricts the returned descriptions to include only those associated with this application.<br/>
        :return: NextToken:   If returned, this indicates that there are more results to obtain. Use this token in the next <a>DescribeEvents</a> call to get the next batch of events. <br/>Events:   A list of <a>EventDescription</a>. <br/>        """
        pass

    def describe_instances_health(self, EnvironmentName: str=None, EnvironmentId: str=None, AttributeNames: List[InstancesHealthAttribute]=None, NextToken: str=None) -> DescribeInstancesHealthResult:
        """Retrieves detailed information about the health of instances in your AWS Elastic Beanstalk. This operation requires <a href="https://docs.aws.amazon.com/elasticbeanstalk/latest/dg/health-enhanced.html">enhanced health reporting</a>.

        :param self:
        :param NextToken:  Specify the pagination token returned by a previous call.<br/>AttributeNames:  Specifies the response elements you wish to receive. To retrieve all attributes, set to <code>All</code>. If no attribute names are specified, returns a list of instances.<br/>EnvironmentId:  Specify the AWS Elastic Beanstalk environment by ID.<br/>EnvironmentName:  Specify the AWS Elastic Beanstalk environment by name.<br/>
        :return: NextToken:  Pagination token for the next page of results, if available.<br/>RefreshedAt:  The date and time that the health information was retrieved.<br/>InstanceHealthList:  Detailed health information about each instance. The output differs slightly between Linux and Windows environments. There is a difference in the members that are supported under the <code>&lt;CPUUtilization&gt;</code> type.<br/>        """
        pass

    def describe_platform_version(self, PlatformArn: str=None) -> DescribePlatformVersionResult:
        """Describes the version of the platform.

        :param self:
        :param PlatformArn:  The ARN of the version of the platform.<br/>
        :return: PlatformDescription:  Detailed information about the version of the platform.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedAddonList</i> The additions supported by the platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedTierList</i> The tiers supported by the platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomAmiList</i> The custom AMIs supported by the platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Frameworks</i> The frameworks supported by the platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProgrammingLanguages</i> The programming languages supported by the platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OperatingSystemVersion</i> The version of the operating system used by the platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OperatingSystemName</i> The operating system used by the platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Maintainer</i> Information about the maintainer of the platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlatformCategory</i> The category of the platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DateUpdated</i> The date when the platform was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DateCreated</i> The date when the platform was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlatformStatus</i> The status of the platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SolutionStackName</i> The name of the solution stack used by the platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlatformVersion</i> The version of the platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlatformName</i> The name of the platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlatformOwner</i> The AWS account ID of the person who created the platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlatformArn</i> The ARN of the platform.<br/>        """
        pass

    def list_available_solution_stacks(self) -> ListAvailableSolutionStacksResultMessage:
        """Returns a list of the available solution stack names, with the public version first and then in reverse chronological order.

        :param self:
        :param 
        :return: SolutionStackDetails:   A list of available solution stacks and their <a>SolutionStackDescription</a>. <br/>SolutionStacks:  A list of available solution stacks.<br/>        """
        pass

    def list_platform_versions(self, Filters: List[PlatformFilter]=None, MaxRecords: int=None, NextToken: str=None) -> ListPlatformVersionsResult:
        """Lists the available platforms.

        :param self:
        :param NextToken:  The starting index into the remaining list of platforms. Use the <code>NextToken</code> value from a previous <code>ListPlatformVersion</code> call.<br/>MaxRecords:  The maximum number of platform values returned in one call.<br/>Filters:  List only the platforms where the platform member value relates to one of the supplied values.<br/>
        :return: NextToken:  The starting index into the remaining list of platforms. if this value is not <code>null</code>, you can use it in a subsequent <code>ListPlatformVersion</code> call. <br/>PlatformSummaryList:  Detailed information about the platforms.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceArn: str) -> ResourceTagsDescriptionMessage:
        """Returns the tags applied to an AWS Elastic Beanstalk resource. The response contains a list of tag key-value pairs. Currently, Elastic Beanstalk only supports tagging of Elastic Beanstalk environments. For details about environment tagging, see <a href="https://docs.aws.amazon.com/elasticbeanstalk/latest/dg/using-features.tagging.html">Tagging Resources in Your Elastic Beanstalk Environment</a>.

        :param self:
        :param ResourceArn:  The Amazon Resource Name (ARN) of the resouce for which a tag list is requested. Must be the ARN of an Elastic Beanstalk environment.<br/>
        :return: ResourceTags:  A list of tag key-value pairs.<br/>ResourceArn:  The Amazon Resource Name (ARN) of the resouce for which a tag list was requested.<br/>        """
        pass

    def rebuild_environment(self, EnvironmentId: str=None, EnvironmentName: str=None):
        """Deletes and recreates all of the AWS resources (for example: the Auto Scaling group, load balancer, etc.) for a specified environment and forces a restart.

        :param self:
        :param EnvironmentName:  The name of the environment to rebuild.  Condition: You must specify either this or an EnvironmentId, or both. If you do not specify either, AWS Elastic Beanstalk returns <code>MissingRequiredParameter</code> error. <br/>EnvironmentId:  The ID of the environment to rebuild.  Condition: You must specify either this or an EnvironmentName, or both. If you do not specify either, AWS Elastic Beanstalk returns <code>MissingRequiredParameter</code> error. <br/>
        :return:         """
        pass

    def request_environment_info(self, InfoType: str, EnvironmentId: str=None, EnvironmentName: str=None):
        """Initiates a request to compile the specified type of information of the deployed environment.  Setting the <code>InfoType</code> to <code>tail</code> compiles the last lines from the application server log files of every Amazon EC2 instance in your environment.   Setting the <code>InfoType</code> to <code>bundle</code> compresses the application server log files for every Amazon EC2 instance into a <code>.zip</code> file. Legacy and .NET containers do not support bundle logs.   Use <a>RetrieveEnvironmentInfo</a> to obtain the set of logs.  Related Topics <ul> <li>  <a>RetrieveEnvironmentInfo</a>  </li> </ul>

        :param self:
        :param InfoType:  The type of information to request.<br/>EnvironmentName:  The name of the environment of the requested data. If no such environment is found, <code>RequestEnvironmentInfo</code> returns an <code>InvalidParameterValue</code> error.  Condition: You must specify either this or an EnvironmentId, or both. If you do not specify either, AWS Elastic Beanstalk returns <code>MissingRequiredParameter</code> error. <br/>EnvironmentId:  The ID of the environment of the requested data. If no such environment is found, <code>RequestEnvironmentInfo</code> returns an <code>InvalidParameterValue</code> error.  Condition: You must specify either this or an EnvironmentName, or both. If you do not specify either, AWS Elastic Beanstalk returns <code>MissingRequiredParameter</code> error. <br/>
        :return:         """
        pass

    def restart_app_server(self, EnvironmentId: str=None, EnvironmentName: str=None):
        """Causes the environment to restart the application container server running on each Amazon EC2 instance.

        :param self:
        :param EnvironmentName:  The name of the environment to restart the server for.  Condition: You must specify either this or an EnvironmentId, or both. If you do not specify either, AWS Elastic Beanstalk returns <code>MissingRequiredParameter</code> error. <br/>EnvironmentId:  The ID of the environment to restart the server for.  Condition: You must specify either this or an EnvironmentName, or both. If you do not specify either, AWS Elastic Beanstalk returns <code>MissingRequiredParameter</code> error. <br/>
        :return:         """
        pass

    def retrieve_environment_info(self, InfoType: str, EnvironmentId: str=None, EnvironmentName: str=None) -> RetrieveEnvironmentInfoResultMessage:
        """Retrieves the compiled information from a <a>RequestEnvironmentInfo</a> request. Related Topics <ul> <li>  <a>RequestEnvironmentInfo</a>  </li> </ul>

        :param self:
        :param InfoType:  The type of information to retrieve.<br/>EnvironmentName:  The name of the data's environment.  If no such environment is found, returns an <code>InvalidParameterValue</code> error.   Condition: You must specify either this or an EnvironmentId, or both. If you do not specify either, AWS Elastic Beanstalk returns <code>MissingRequiredParameter</code> error. <br/>EnvironmentId:  The ID of the data's environment. If no such environment is found, returns an <code>InvalidParameterValue</code> error. Condition: You must specify either this or an EnvironmentName, or both. If you do not specify either, AWS Elastic Beanstalk returns <code>MissingRequiredParameter</code> error.<br/>
        :return: EnvironmentInfo:   The <a>EnvironmentInfoDescription</a> of the environment. <br/>        """
        pass

    def swap_environment_cnames(self, SourceEnvironmentId: str=None, SourceEnvironmentName: str=None, DestinationEnvironmentId: str=None, DestinationEnvironmentName: str=None):
        """Swaps the CNAMEs of two environments.

        :param self:
        :param DestinationEnvironmentName:  The name of the destination environment.  Condition: You must specify at least the <code>DestinationEnvironmentID</code> or the <code>DestinationEnvironmentName</code>. You may also specify both. You must specify the <code>SourceEnvironmentName</code> with the <code>DestinationEnvironmentName</code>. <br/>DestinationEnvironmentId:  The ID of the destination environment.  Condition: You must specify at least the <code>DestinationEnvironmentID</code> or the <code>DestinationEnvironmentName</code>. You may also specify both. You must specify the <code>SourceEnvironmentId</code> with the <code>DestinationEnvironmentId</code>. <br/>SourceEnvironmentName:  The name of the source environment.  Condition: You must specify at least the <code>SourceEnvironmentID</code> or the <code>SourceEnvironmentName</code>. You may also specify both. If you specify the <code>SourceEnvironmentName</code>, you must specify the <code>DestinationEnvironmentName</code>. <br/>SourceEnvironmentId:  The ID of the source environment.  Condition: You must specify at least the <code>SourceEnvironmentID</code> or the <code>SourceEnvironmentName</code>. You may also specify both. If you specify the <code>SourceEnvironmentId</code>, you must specify the <code>DestinationEnvironmentId</code>. <br/>
        :return:         """
        pass

    def terminate_environment(self, EnvironmentId: str=None, EnvironmentName: str=None, TerminateResources: TerminateEnvironmentResources=None, ForceTerminate: ForceTerminate=None) -> EnvironmentDescription:
        """Terminates the specified environment.

        :param self:
        :param ForceTerminate:  Terminates the target environment even if another environment in the same group is dependent on it.<br/>TerminateResources:  Indicates whether the associated AWS resources should shut down when the environment is terminated: <ul> <li>  <code>true</code>: The specified environment as well as the associated AWS resources, such as Auto Scaling group and LoadBalancer, are terminated. </li> <li>  <code>false</code>: AWS Elastic Beanstalk resource management is removed from the environment, but the AWS resources continue to operate. </li> </ul>  For more information, see the <a href="https://docs.aws.amazon.com/elasticbeanstalk/latest/ug/"> AWS Elastic Beanstalk User Guide. </a>   Default: <code>true</code>   Valid Values: <code>true</code> | <code>false</code> <br/>EnvironmentName:  The name of the environment to terminate.  Condition: You must specify either this or an EnvironmentId, or both. If you do not specify either, AWS Elastic Beanstalk returns <code>MissingRequiredParameter</code> error. <br/>EnvironmentId:  The ID of the environment to terminate.  Condition: You must specify either this or an EnvironmentName, or both. If you do not specify either, AWS Elastic Beanstalk returns <code>MissingRequiredParameter</code> error. <br/>
        :return: EnvironmentArn:  The environment's Amazon Resource Name (ARN), which can be used in other API requests that require an ARN.<br/>EnvironmentLinks:  A list of links to other environments in the same group.<br/>Tier:  Describes the current tier of this environment.:<br/>Resources:  The description of the AWS resources used by this environment.:<br/>HealthStatus:  Returns the health status of the application running in your environment. For more information, see <a href="https://docs.aws.amazon.com/elasticbeanstalk/latest/dg/health-enhanced-status.html">Health Colors and Statuses</a>.<br/>Health:  Describes the health status of the environment. AWS Elastic Beanstalk indicates the failure levels for a running environment: <ul> <li>  <code>Red</code>: Indicates the environment is not responsive. Occurs when three or more consecutive failures occur for an environment. </li> <li>  <code>Yellow</code>: Indicates that something is wrong. Occurs when two consecutive failures occur for an environment. </li> <li>  <code>Green</code>: Indicates the environment is healthy and fully functional. </li> <li>  <code>Grey</code>: Default health for a new environment. The environment is not fully launched and health checks have not started or health checks are suspended during an <code>UpdateEnvironment</code> or <code>RestartEnvironment</code> request. </li> </ul>  Default: <code>Grey</code> <br/>AbortableOperationInProgress:  Indicates if there is an in-progress environment configuration update or application version deployment that you can cancel.  <code>true:</code> There is an update in progress.   <code>false:</code> There are no updates currently in progress. <br/>Status:  The current operational status of the environment: <ul> <li>  <code>Launching</code>: Environment is in the process of initial deployment. </li> <li>  <code>Updating</code>: Environment is in the process of updating its configuration settings or application version. </li> <li>  <code>Ready</code>: Environment is available to have an action performed on it, such as update or terminate. </li> <li>  <code>Terminating</code>: Environment is in the shut-down process. </li> <li>  <code>Terminated</code>: Environment is not running. </li> </ul><br/>DateUpdated:  The last modified date for this environment.<br/>DateCreated:  The creation date for this environment.<br/>CNAME:  The URL to the CNAME for this environment.<br/>EndpointURL:  For load-balanced, autoscaling environments, the URL to the LoadBalancer. For single-instance environments, the IP address of the instance.<br/>Description:  Describes this environment.<br/>TemplateName:  The name of the configuration template used to originally launch this environment.<br/>PlatformArn:  The ARN of the platform.<br/>SolutionStackName:   The name of the <code>SolutionStack</code> deployed with this environment. <br/>VersionLabel:  The application version deployed in this environment.<br/>ApplicationName:  The name of the application associated with this environment.<br/>EnvironmentId:  The ID of this environment.<br/>EnvironmentName:  The name of this environment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LoadBalancer</i> Describes the LoadBalancer.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LoadBalancer</b>: Describes the LoadBalancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Listeners</i> A list of Listeners used by the LoadBalancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Domain</i> The domain name of the LoadBalancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LoadBalancerName</i> The name of the LoadBalancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version of this environment tier. When you don't set a value to it, Elastic Beanstalk uses the latest compatible worker tier version. <note> This member is deprecated. Any specific version that you set may become out of date. We recommend leaving it unspecified. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of this environment tier. Valid values: <ul> <li> For <i>Web server tier</i> – <code>Standard</code>  </li> <li> For <i>Worker tier</i> – <code>SQS/HTTP</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of this environment tier. Valid values: <ul> <li> For <i>Web server tier</i> – <code>WebServer</code>  </li> <li> For <i>Worker tier</i> – <code>Worker</code>  </li> </ul><br/>        """
        pass

    def update_application(self, ApplicationName: str, Description: str=None) -> ApplicationDescriptionMessage:
        """Updates the specified application to have the specified properties. <note> If a property (for example, <code>description</code>) is not provided, the value remains unchanged. To clear these properties, specify an empty string. </note>

        :param self:
        :param Description:  A new description for the application. Default: If not specified, AWS Elastic Beanstalk does not update the description.<br/>ApplicationName:  The name of the application to update. If no such application is found, <code>UpdateApplication</code> returns an <code>InvalidParameterValue</code> error. <br/>
        :return: Application:   The <a>ApplicationDescription</a> of the application. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceLifecycleConfig</i> The lifecycle settings for the application.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResourceLifecycleConfig</b>: The lifecycle settings for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigurationTemplates</i> The names of the configuration templates associated with this application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Versions</i> The names of the versions for this application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DateUpdated</i> The date when the application was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DateCreated</i> The date when the application was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> User-defined description of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationName</i> The name of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationArn</i> The Amazon Resource Name (ARN) of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceRole</i> The ARN of an IAM service role that Elastic Beanstalk has permission to assume. The <code>ServiceRole</code> property is required the first time that you provide a <code>VersionLifecycleConfig</code> for the application in one of the supporting calls (<code>CreateApplication</code> or <code>UpdateApplicationResourceLifecycle</code>). After you provide it once, in either one of the calls, Elastic Beanstalk persists the Service Role with the application, and you don't need to specify it again in subsequent <code>UpdateApplicationResourceLifecycle</code> calls. You can, however, specify it in subsequent calls to change the Service Role to another value.<br/>The application version lifecycle configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VersionLifecycleConfig</b>: The application version lifecycle configuration.        """
        pass

    def update_application_resource_lifecycle(self, ResourceLifecycleConfig: ApplicationResourceLifecycleConfig, ApplicationName: str) -> ApplicationResourceLifecycleDescriptionMessage:
        """Modifies lifecycle settings for an application.

        :param self:
        :param ResourceLifecycleConfig:  The lifecycle configuration.:<br/>ApplicationName:  The name of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VersionLifecycleConfig</i> The application version lifecycle configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VersionLifecycleConfig</b>: The application version lifecycle configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceRole</i> The ARN of an IAM service role that Elastic Beanstalk has permission to assume. The <code>ServiceRole</code> property is required the first time that you provide a <code>VersionLifecycleConfig</code> for the application in one of the supporting calls (<code>CreateApplication</code> or <code>UpdateApplicationResourceLifecycle</code>). After you provide it once, in either one of the calls, Elastic Beanstalk persists the Service Role with the application, and you don't need to specify it again in subsequent <code>UpdateApplicationResourceLifecycle</code> calls. You can, however, specify it in subsequent calls to change the Service Role to another value.<br/>Specify a max count rule to restrict the number of application versions that are retained for an application.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>MaxCountRule</b>: Specify a max count rule to restrict the number of application versions that are retained for an application.
        :return: ResourceLifecycleConfig:  The lifecycle configuration.:<br/>ApplicationName:  The name of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VersionLifecycleConfig</i> The application version lifecycle configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VersionLifecycleConfig</b>: The application version lifecycle configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceRole</i> The ARN of an IAM service role that Elastic Beanstalk has permission to assume. The <code>ServiceRole</code> property is required the first time that you provide a <code>VersionLifecycleConfig</code> for the application in one of the supporting calls (<code>CreateApplication</code> or <code>UpdateApplicationResourceLifecycle</code>). After you provide it once, in either one of the calls, Elastic Beanstalk persists the Service Role with the application, and you don't need to specify it again in subsequent <code>UpdateApplicationResourceLifecycle</code> calls. You can, however, specify it in subsequent calls to change the Service Role to another value.<br/>Specify a max count rule to restrict the number of application versions that are retained for an application.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>MaxCountRule</b>: Specify a max count rule to restrict the number of application versions that are retained for an application.        """
        pass

    def update_application_version(self, VersionLabel: str, ApplicationName: str, Description: str=None) -> ApplicationVersionDescriptionMessage:
        """Updates the specified application version to have the specified properties. <note> If a property (for example, <code>description</code>) is not provided, the value remains unchanged. To clear properties, specify an empty string. </note>

        :param self:
        :param Description:  A new description for this version.<br/>VersionLabel:  The name of the version to update. If no application version is found with this label, <code>UpdateApplication</code> returns an <code>InvalidParameterValue</code> error. <br/>ApplicationName:  The name of the application associated with this version.  If no application is found with this name, <code>UpdateApplication</code> returns an <code>InvalidParameterValue</code> error.<br/>
        :return: ApplicationVersion:   The <a>ApplicationVersionDescription</a> of the application version. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The processing status of the application version. Reflects the state of the application version during its creation. Many of the values are only applicable if you specified <code>True</code> for the <code>Process</code> parameter of the <code>CreateApplicationVersion</code> action. The following list describes the possible values. <ul> <li>  <code>Unprocessed</code> – Application version wasn't pre-processed or validated. Elastic Beanstalk will validate configuration files during deployment of the application version to an environment. </li> <li>  <code>Processing</code> – Elastic Beanstalk is currently processing the application version. </li> <li>  <code>Building</code> – Application version is currently undergoing an AWS CodeBuild build. </li> <li>  <code>Processed</code> – Elastic Beanstalk was successfully pre-processed and validated. </li> <li>  <code>Failed</code> – Either the AWS CodeBuild build failed or configuration files didn't pass validation. This application version isn't usable. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DateUpdated</i> The last modified date of the application version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DateCreated</i> The creation date of the application version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceBundle</i> The storage location of the application version's source bundle in Amazon S3.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SourceBundle</b>: The storage location of the application version's source bundle in Amazon S3.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BuildArn</i> Reference to the artifact from the AWS CodeBuild build.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceBuildInformation</i> If the version's source code was retrieved from AWS CodeCommit, the location of the source code for the application version.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SourceBuildInformation</b>: If the version's source code was retrieved from AWS CodeCommit, the location of the source code for the application version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VersionLabel</i> A unique identifier for the application version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the application version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationName</i> The name of the application to which the application version belongs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationVersionArn</i> The Amazon Resource Name (ARN) of the application version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceLocation</i> The location of the source code, as a formatted string, depending on the value of <code>SourceRepository</code>  <ul> <li> For <code>CodeCommit</code>, the format is the repository name and commit ID, separated by a forward slash. For example, <code>my-git-repo/265cfa0cf6af46153527f55d6503ec030551f57a</code>. </li> <li> For <code>S3</code>, the format is the S3 bucket name and object key, separated by a forward slash. For example, <code>my-s3-bucket/Folders/my-source-file</code>. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceRepository</i> Location where the repository is stored. <ul> <li>  <code>CodeCommit</code>  </li> <li>  <code>S3</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceType</i> The type of repository. <ul> <li>  <code>Git</code>  </li> <li>  <code>Zip</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Key</i> The Amazon S3 key where the data is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Bucket</i> The Amazon S3 bucket where the data is located.<br/>        """
        pass

    def update_configuration_template(self, TemplateName: str, ApplicationName: str, Description: str=None, OptionSettings: List[ConfigurationOptionSetting]=None, OptionsToRemove: List[OptionSpecification]=None) -> ConfigurationSettingsDescription:
        """Updates the specified configuration template to have the specified properties or configuration option values. <note> If a property (for example, <code>ApplicationName</code>) is not provided, its value remains unchanged. To clear such properties, specify an empty string. </note> Related Topics <ul> <li>  <a>DescribeConfigurationOptions</a>  </li> </ul>

        :param self:
        :param OptionsToRemove:  A list of configuration options to remove from the configuration set.  Constraint: You can remove only <code>UserDefined</code> configuration options. <br/>OptionSettings:  A list of configuration option settings to update with the new specified option value.<br/>Description:  A new description for the configuration.<br/>TemplateName:  The name of the configuration template to update.  If no configuration template is found with this name, <code>UpdateConfigurationTemplate</code> returns an <code>InvalidParameterValue</code> error. <br/>ApplicationName:  The name of the application associated with the configuration template to update.  If no application is found with this name, <code>UpdateConfigurationTemplate</code> returns an <code>InvalidParameterValue</code> error. <br/>
        :return: OptionSettings:  A list of the configuration options and their values in this configuration set.<br/>DateUpdated:  The date (in UTC time) when this configuration set was last modified.<br/>DateCreated:  The date (in UTC time) when this configuration set was created.<br/>DeploymentStatus:   If this configuration set is associated with an environment, the <code>DeploymentStatus</code> parameter indicates the deployment status of this configuration set:  <ul> <li>  <code>null</code>: This configuration is not associated with a running environment. </li> <li>  <code>pending</code>: This is a draft configuration that is not deployed to the associated environment but is in the process of deploying. </li> <li>  <code>deployed</code>: This is the configuration that is currently deployed to the associated running environment. </li> <li>  <code>failed</code>: This is a draft configuration that failed to successfully deploy. </li> </ul><br/>EnvironmentName:   If not <code>null</code>, the name of the environment for this configuration set. <br/>Description:  Describes this configuration set.<br/>TemplateName:   If not <code>null</code>, the name of the configuration template for this configuration set. <br/>ApplicationName:  The name of the application associated with this configuration set.<br/>PlatformArn:  The ARN of the platform.<br/>SolutionStackName:  The name of the solution stack this configuration set uses.<br/>        """
        pass

    def update_environment(self, ApplicationName: str=None, EnvironmentId: str=None, EnvironmentName: str=None, GroupName: str=None, Description: str=None, Tier: EnvironmentTier=None, VersionLabel: str=None, TemplateName: str=None, SolutionStackName: str=None, PlatformArn: str=None, OptionSettings: List[ConfigurationOptionSetting]=None, OptionsToRemove: List[OptionSpecification]=None) -> EnvironmentDescription:
        """Updates the environment description, deploys a new application version, updates the configuration settings to an entirely new configuration template, or updates select configuration option values in the running environment.  Attempting to update both the release and configuration is not allowed and AWS Elastic Beanstalk returns an <code>InvalidParameterCombination</code> error.   When updating the configuration settings to a new template or individual settings, a draft configuration is created and <a>DescribeConfigurationSettings</a> for this environment returns two setting descriptions with different <code>DeploymentStatus</code> values. 

        :param self:
        :param OptionsToRemove:  A list of custom user-defined configuration options to remove from the configuration set for this environment.<br/>OptionSettings:  If specified, AWS Elastic Beanstalk updates the configuration set associated with the running environment and sets the specified configuration options to the requested value.<br/>PlatformArn:  The ARN of the platform, if used.<br/>SolutionStackName:  This specifies the platform version that the environment will run after the environment is updated.<br/>TemplateName:  If this parameter is specified, AWS Elastic Beanstalk deploys this configuration template to the environment. If no such configuration template is found, AWS Elastic Beanstalk returns an <code>InvalidParameterValue</code> error. <br/>VersionLabel:  If this parameter is specified, AWS Elastic Beanstalk deploys the named application version to the environment. If no such application version is found, returns an <code>InvalidParameterValue</code> error. <br/>Tier:  This specifies the tier to use to update the environment. Condition: At this time, if you change the tier version, name, or type, AWS Elastic Beanstalk returns <code>InvalidParameterValue</code> error. :<br/>Description:  If this parameter is specified, AWS Elastic Beanstalk updates the description of this environment.<br/>GroupName:  The name of the group to which the target environment belongs. Specify a group name only if the environment's name is specified in an environment manifest and not with the environment name or environment ID parameters. See <a href="https://docs.aws.amazon.com/elasticbeanstalk/latest/dg/environment-cfg-manifest.html">Environment Manifest (env.yaml)</a> for details.<br/>EnvironmentName:  The name of the environment to update. If no environment with this name exists, AWS Elastic Beanstalk returns an <code>InvalidParameterValue</code> error.  Condition: You must specify either this or an EnvironmentId, or both. If you do not specify either, AWS Elastic Beanstalk returns <code>MissingRequiredParameter</code> error. <br/>EnvironmentId:  The ID of the environment to update. If no environment with this ID exists, AWS Elastic Beanstalk returns an <code>InvalidParameterValue</code> error. Condition: You must specify either this or an EnvironmentName, or both. If you do not specify either, AWS Elastic Beanstalk returns <code>MissingRequiredParameter</code> error. <br/>ApplicationName:  The name of the application with which the environment is associated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version of this environment tier. When you don't set a value to it, Elastic Beanstalk uses the latest compatible worker tier version. <note> This member is deprecated. Any specific version that you set may become out of date. We recommend leaving it unspecified. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of this environment tier. Valid values: <ul> <li> For <i>Web server tier</i> – <code>Standard</code>  </li> <li> For <i>Worker tier</i> – <code>SQS/HTTP</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of this environment tier. Valid values: <ul> <li> For <i>Web server tier</i> – <code>WebServer</code>  </li> <li> For <i>Worker tier</i> – <code>Worker</code>  </li> </ul><br/>
        :return: EnvironmentArn:  The environment's Amazon Resource Name (ARN), which can be used in other API requests that require an ARN.<br/>EnvironmentLinks:  A list of links to other environments in the same group.<br/>Tier:  Describes the current tier of this environment.:<br/>Resources:  The description of the AWS resources used by this environment.:<br/>HealthStatus:  Returns the health status of the application running in your environment. For more information, see <a href="https://docs.aws.amazon.com/elasticbeanstalk/latest/dg/health-enhanced-status.html">Health Colors and Statuses</a>.<br/>Health:  Describes the health status of the environment. AWS Elastic Beanstalk indicates the failure levels for a running environment: <ul> <li>  <code>Red</code>: Indicates the environment is not responsive. Occurs when three or more consecutive failures occur for an environment. </li> <li>  <code>Yellow</code>: Indicates that something is wrong. Occurs when two consecutive failures occur for an environment. </li> <li>  <code>Green</code>: Indicates the environment is healthy and fully functional. </li> <li>  <code>Grey</code>: Default health for a new environment. The environment is not fully launched and health checks have not started or health checks are suspended during an <code>UpdateEnvironment</code> or <code>RestartEnvironment</code> request. </li> </ul>  Default: <code>Grey</code> <br/>AbortableOperationInProgress:  Indicates if there is an in-progress environment configuration update or application version deployment that you can cancel.  <code>true:</code> There is an update in progress.   <code>false:</code> There are no updates currently in progress. <br/>Status:  The current operational status of the environment: <ul> <li>  <code>Launching</code>: Environment is in the process of initial deployment. </li> <li>  <code>Updating</code>: Environment is in the process of updating its configuration settings or application version. </li> <li>  <code>Ready</code>: Environment is available to have an action performed on it, such as update or terminate. </li> <li>  <code>Terminating</code>: Environment is in the shut-down process. </li> <li>  <code>Terminated</code>: Environment is not running. </li> </ul><br/>DateUpdated:  The last modified date for this environment.<br/>DateCreated:  The creation date for this environment.<br/>CNAME:  The URL to the CNAME for this environment.<br/>EndpointURL:  For load-balanced, autoscaling environments, the URL to the LoadBalancer. For single-instance environments, the IP address of the instance.<br/>Description:  Describes this environment.<br/>TemplateName:  The name of the configuration template used to originally launch this environment.<br/>PlatformArn:  The ARN of the platform.<br/>SolutionStackName:   The name of the <code>SolutionStack</code> deployed with this environment. <br/>VersionLabel:  The application version deployed in this environment.<br/>ApplicationName:  The name of the application associated with this environment.<br/>EnvironmentId:  The ID of this environment.<br/>EnvironmentName:  The name of this environment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LoadBalancer</i> Describes the LoadBalancer.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LoadBalancer</b>: Describes the LoadBalancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Listeners</i> A list of Listeners used by the LoadBalancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Domain</i> The domain name of the LoadBalancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LoadBalancerName</i> The name of the LoadBalancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version of this environment tier. When you don't set a value to it, Elastic Beanstalk uses the latest compatible worker tier version. <note> This member is deprecated. Any specific version that you set may become out of date. We recommend leaving it unspecified. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of this environment tier. Valid values: <ul> <li> For <i>Web server tier</i> – <code>Standard</code>  </li> <li> For <i>Worker tier</i> – <code>SQS/HTTP</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of this environment tier. Valid values: <ul> <li> For <i>Web server tier</i> – <code>WebServer</code>  </li> <li> For <i>Worker tier</i> – <code>Worker</code>  </li> </ul><br/>        """
        pass

    def update_tags_for_resource(self, ResourceArn: str, TagsToAdd: List[Tag]=None, TagsToRemove: List[TagKey]=None):
        """Update the list of tags applied to an AWS Elastic Beanstalk resource. Two lists can be passed: <code>TagsToAdd</code> for tags to add or update, and <code>TagsToRemove</code>. Currently, Elastic Beanstalk only supports tagging of Elastic Beanstalk environments. For details about environment tagging, see <a href="https://docs.aws.amazon.com/elasticbeanstalk/latest/dg/using-features.tagging.html">Tagging Resources in Your Elastic Beanstalk Environment</a>. If you create a custom IAM user policy to control permission to this operation, specify one of the following two virtual actions (or both) instead of the API operation name: <dl> <dt>elasticbeanstalk:AddTags</dt> <dd> Controls permission to call <code>UpdateTagsForResource</code> and pass a list of tags to add in the <code>TagsToAdd</code> parameter. </dd> <dt>elasticbeanstalk:RemoveTags</dt> <dd> Controls permission to call <code>UpdateTagsForResource</code> and pass a list of tag keys to remove in the <code>TagsToRemove</code> parameter. </dd> </dl> For details about creating a custom user policy, see <a href="https://docs.aws.amazon.com/elasticbeanstalk/latest/dg/AWSHowTo.iam.managed-policies.html#AWSHowTo.iam.policies">Creating a Custom User Policy</a>.

        :param self:
        :param TagsToRemove:  A list of tag keys to remove. If a tag key doesn't exist, it is silently ignored.<br/>TagsToAdd:  A list of tags to add or update. If a key of an existing tag is added, the tag's value is updated.<br/>ResourceArn:  The Amazon Resource Name (ARN) of the resouce to be updated. Must be the ARN of an Elastic Beanstalk environment.<br/>
        :return:         """
        pass

    def validate_configuration_settings(self, OptionSettings: List[ConfigurationOptionSetting], ApplicationName: str, TemplateName: str=None, EnvironmentName: str=None) -> ConfigurationSettingsValidationMessages:
        """Takes a set of configuration settings and either a configuration template or environment, and determines whether those values are valid. This action returns a list of messages indicating any errors or warnings associated with the selection of option values.

        :param self:
        :param OptionSettings:  A list of the options and desired values to evaluate.<br/>EnvironmentName:  The name of the environment to validate the settings against. Condition: You cannot specify both this and a configuration template name.<br/>TemplateName:  The name of the configuration template to validate the settings against. Condition: You cannot specify both this and an environment name.<br/>ApplicationName:  The name of the application that the configuration template or environment belongs to.<br/>
        :return: Messages:   A list of <a>ValidationMessage</a>. <br/>        """
        pass



class ElasticTranscoder(BaseClient):
    """<fullname>AWS Elastic Transcoder Service</fullname> The AWS Elastic Transcoder Service."""

    class JobCompleteWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `read_job` for valid parameters"""
            pass


    class ListJobsByPipelinePaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_jobs_by_pipeline` for valid parameters"""
            pass
    class ListJobsByStatusPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_jobs_by_status` for valid parameters"""
            pass
    class ListPipelinesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_pipelines` for valid parameters"""
            pass
    class ListPresetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_presets` for valid parameters"""
            pass


    class CancelJobRequest(Mapping):
        pass
    
    class CancelJobResponse(Mapping):
        pass
    
    class CreateJobRequest(Mapping):
        pass
    
    class CreateJobResponse(Mapping):
        pass
    
    class JobInput(Mapping):
        pass
    
    class CreateJobOutput(Mapping):
        pass
    
    class CreateJobPlaylist(Mapping):
        pass
    
    class UserMetadata(object):
        pass
    
    class CreatePipelineRequest(Mapping):
        pass
    
    class CreatePipelineResponse(Mapping):
        pass
    
    class Notifications(Mapping):
        pass
    
    class PipelineOutputConfig(Mapping):
        pass
    
    class CreatePresetRequest(Mapping):
        pass
    
    class CreatePresetResponse(Mapping):
        pass
    
    class VideoParameters(Mapping):
        pass
    
    class AudioParameters(Mapping):
        pass
    
    class Thumbnails(Mapping):
        pass
    
    class DeletePipelineRequest(Mapping):
        pass
    
    class DeletePipelineResponse(Mapping):
        pass
    
    class DeletePresetRequest(Mapping):
        pass
    
    class DeletePresetResponse(Mapping):
        pass
    
    class ListJobsByPipelineRequest(Mapping):
        pass
    
    class ListJobsByPipelineResponse(Mapping):
        pass
    
    class ListJobsByStatusRequest(Mapping):
        pass
    
    class ListJobsByStatusResponse(Mapping):
        pass
    
    class ListPipelinesRequest(Mapping):
        pass
    
    class ListPipelinesResponse(Mapping):
        pass
    
    class ListPresetsRequest(Mapping):
        pass
    
    class ListPresetsResponse(Mapping):
        pass
    
    class ReadJobRequest(Mapping):
        pass
    
    class ReadJobResponse(Mapping):
        pass
    
    class ReadPipelineRequest(Mapping):
        pass
    
    class ReadPipelineResponse(Mapping):
        pass
    
    class ReadPresetRequest(Mapping):
        pass
    
    class ReadPresetResponse(Mapping):
        pass
    
    class TestRoleRequest(Mapping):
        pass
    
    class TestRoleResponse(Mapping):
        pass
    
    class SnsTopic(object):
        pass
    
    class UpdatePipelineRequest(Mapping):
        pass
    
    class UpdatePipelineResponse(Mapping):
        pass
    
    class UpdatePipelineNotificationsRequest(Mapping):
        pass
    
    class UpdatePipelineNotificationsResponse(Mapping):
        pass
    
    class UpdatePipelineStatusRequest(Mapping):
        pass
    
    class UpdatePipelineStatusResponse(Mapping):
        pass
    
    def cancel_job(self, Id: str) -> CancelJobResponse:
        """The CancelJob operation cancels an unfinished job. <note> You can only cancel a job that has a status of <code>Submitted</code>. To prevent a pipeline from starting to process a job while you're getting the job identifier, use <a>UpdatePipelineStatus</a> to temporarily pause the pipeline. </note>

        :param self:
        :param Id:  The identifier of the job that you want to cancel. To get a list of the jobs (including their <code>jobId</code>) that have a status of <code>Submitted</code>, use the <a>ListJobsByStatus</a> API action.<br/>
        :return:         """
        pass

    def create_job(self, PipelineId: str, Input: JobInput=None, Inputs: List[JobInput]=None, Output: CreateJobOutput=None, Outputs: List[CreateJobOutput]=None, OutputKeyPrefix: str=None, Playlists: List[CreateJobPlaylist]=None, UserMetadata: UserMetadata=None) -> CreateJobResponse:
        """When you create a job, Elastic Transcoder returns JSON data that includes the values that you specified plus information about the job that is created. If you have specified more than one output for your jobs (for example, one output for the Kindle Fire and another output for the Apple iPhone 4s), you currently must use the Elastic Transcoder API to list the jobs (as opposed to the AWS Console).

        :param self:
        :param UserMetadata:  User-defined metadata that you want to associate with an Elastic Transcoder job. You specify metadata in <code>key/value</code> pairs, and you can add up to 10 <code>key/value</code> pairs per job. Elastic Transcoder does not guarantee that <code>key/value</code> pairs are returned in the same order in which you specify them.<br/>Playlists:  If you specify a preset in <code>PresetId</code> for which the value of <code>Container</code> is fmp4 (Fragmented MP4) or ts (MPEG-TS), Playlists contains information about the master playlists that you want Elastic Transcoder to create. The maximum number of master playlists in a job is 30.<br/>OutputKeyPrefix:  The value, if any, that you want Elastic Transcoder to prepend to the names of all files that this job creates, including output files, thumbnails, and playlists.<br/>Outputs:   A section of the request body that provides information about the transcoded (target) files. We recommend that you use the <code>Outputs</code> syntax instead of the <code>Output</code> syntax. <br/>Output:   A section of the request body that provides information about the transcoded (target) file. We strongly recommend that you use the <code>Outputs</code> syntax instead of the <code>Output</code> syntax. :<br/>Inputs:  A section of the request body that provides information about the files that are being transcoded.<br/>Input:  A section of the request body that provides information about the file that is being transcoded.:<br/>PipelineId:  The <code>Id</code> of the pipeline that you want Elastic Transcoder to use for transcoding. The pipeline determines several settings, including the Amazon S3 bucket from which Elastic Transcoder gets the files to transcode and the bucket into which Elastic Transcoder puts the transcoded files.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DetectedProperties</i> The detected properties of the input file.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DetectedProperties</b>: The detected properties of the input file.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputCaptions</i> You can configure Elastic Transcoder to transcode captions, or subtitles, from one format to another. All captions must be in UTF-8. Elastic Transcoder supports two types of captions: <ul> <li>  <b>Embedded:</b> Embedded captions are included in the same file as the audio and video. Elastic Transcoder supports only one embedded caption per language, to a maximum of 300 embedded captions per file. Valid input values include: <code>CEA-608 (EIA-608</code>, first non-empty channel only), <code>CEA-708 (EIA-708</code>, first non-empty channel only), and <code>mov-text</code>  Valid outputs include: <code>mov-text</code>  Elastic Transcoder supports a maximum of one embedded format per output. </li> <li>  <b>Sidecar:</b> Sidecar captions are kept in a separate metadata file from the audio and video data. Sidecar captions require a player that is capable of understanding the relationship between the video file and the sidecar file. Elastic Transcoder supports only one sidecar caption per language, to a maximum of 20 sidecar captions per file. Valid input values include: <code>dfxp</code> (first div element only), <code>ebu-tt</code>, <code>scc</code>, <code>smpt</code>, <code>srt</code>, <code>ttml</code> (first div element only), and <code>webvtt</code>  Valid outputs include: <code>dfxp</code> (first div element only), <code>scc</code>, <code>srt</code>, and <code>webvtt</code>. </li> </ul> If you want ttml or smpte-tt compatible captions, specify dfxp as your output format. Elastic Transcoder does not support OCR (Optical Character Recognition), does not accept pictures as a valid input for captions, and is not available for audio-only transcoding. Elastic Transcoder does not preserve text formatting (for example, italics) during the transcoding process. To remove captions or leave the captions empty, set <code>Captions</code> to null. To pass through existing captions unchanged, set the <code>MergePolicy</code> to <code>MergeRetain</code>, and pass in a null <code>CaptionSources</code> array. For more information on embedded files, see the Subtitles Wikipedia page. For more information on sidecar files, see the Extensible Metadata Platform and Sidecar file Wikipedia pages.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputCaptions</b>: You can configure Elastic Transcoder to transcode captions, or subtitles, from one format to another. All captions must be in UTF-8. Elastic Transcoder supports two types of captions: <ul> <li>  <b>Embedded:</b> Embedded captions are included in the same file as the audio and video. Elastic Transcoder supports only one embedded caption per language, to a maximum of 300 embedded captions per file. Valid input values include: <code>CEA-608 (EIA-608</code>, first non-empty channel only), <code>CEA-708 (EIA-708</code>, first non-empty channel only), and <code>mov-text</code>  Valid outputs include: <code>mov-text</code>  Elastic Transcoder supports a maximum of one embedded format per output. </li> <li>  <b>Sidecar:</b> Sidecar captions are kept in a separate metadata file from the audio and video data. Sidecar captions require a player that is capable of understanding the relationship between the video file and the sidecar file. Elastic Transcoder supports only one sidecar caption per language, to a maximum of 20 sidecar captions per file. Valid input values include: <code>dfxp</code> (first div element only), <code>ebu-tt</code>, <code>scc</code>, <code>smpt</code>, <code>srt</code>, <code>ttml</code> (first div element only), and <code>webvtt</code>  Valid outputs include: <code>dfxp</code> (first div element only), <code>scc</code>, <code>srt</code>, and <code>webvtt</code>. </li> </ul> If you want ttml or smpte-tt compatible captions, specify dfxp as your output format. Elastic Transcoder does not support OCR (Optical Character Recognition), does not accept pictures as a valid input for captions, and is not available for audio-only transcoding. Elastic Transcoder does not preserve text formatting (for example, italics) during the transcoding process. To remove captions or leave the captions empty, set <code>Captions</code> to null. To pass through existing captions unchanged, set the <code>MergePolicy</code> to <code>MergeRetain</code>, and pass in a null <code>CaptionSources</code> array. For more information on embedded files, see the Subtitles Wikipedia page. For more information on sidecar files, see the Extensible Metadata Platform and Sidecar file Wikipedia pages.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeSpan</i> Settings for clipping an input. Each input can have different clip settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TimeSpan</b>: Settings for clipping an input. Each input can have different clip settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> The encryption settings, if any, that are used for decrypting your input files. If your input file is encrypted, you must specify the mode that Elastic Transcoder uses to decrypt your file.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: The encryption settings, if any, that are used for decrypting your input files. If your input file is encrypted, you must specify the mode that Elastic Transcoder uses to decrypt your file.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Container</i> The container type for the input file. If you want Elastic Transcoder to automatically detect the container type of the input file, specify <code>auto</code>. If you want to specify the container type for the input file, enter one of the following values:   <code>3gp</code>, <code>aac</code>, <code>asf</code>, <code>avi</code>, <code>divx</code>, <code>flv</code>, <code>m4a</code>, <code>mkv</code>, <code>mov</code>, <code>mp3</code>, <code>mp4</code>, <code>mpeg</code>, <code>mpeg-ps</code>, <code>mpeg-ts</code>, <code>mxf</code>, <code>ogg</code>, <code>vob</code>, <code>wav</code>, <code>webm</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Interlaced</i> Whether the input file is interlaced. If you want Elastic Transcoder to automatically detect whether the input file is interlaced, specify <code>auto</code>. If you want to specify whether the input file is interlaced, enter one of the following values:  <code>true</code>, <code>false</code>  If you specify a value other than <code>auto</code>, Elastic Transcoder disables automatic detection of interlacing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AspectRatio</i>  The aspect ratio of the input file. If you want Elastic Transcoder to automatically detect the aspect ratio of the input file, specify <code>auto</code>. If you want to specify the aspect ratio for the output file, enter one of the following values:   <code>1:1</code>, <code>4:3</code>, <code>3:2</code>, <code>16:9</code>   If you specify a value other than <code>auto</code>, Elastic Transcoder disables automatic detection of the aspect ratio. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Resolution</i> This value must be <code>auto</code>, which causes Elastic Transcoder to automatically detect the resolution of the input file.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameRate</i> The frame rate of the input file. If you want Elastic Transcoder to automatically detect the frame rate of the input file, specify <code>auto</code>. If you want to specify the frame rate for the input file, enter one of the following values:   <code>10</code>, <code>15</code>, <code>23.97</code>, <code>24</code>, <code>25</code>, <code>29.97</code>, <code>30</code>, <code>60</code>  If you specify a value other than <code>auto</code>, Elastic Transcoder disables automatic detection of the frame rate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i>  The name of the file to transcode. Elsewhere in the body of the JSON block is the the ID of the pipeline to use for processing the job. The <code>InputBucket</code> object in that pipeline tells Elastic Transcoder which Amazon S3 bucket to get the file from.  If the file name includes a prefix, such as <code>cooking/lasagna.mpg</code>, include the prefix in the key. If the file isn't in the specified bucket, Elastic Transcoder returns an error.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InitializationVector</i> The series of random bits created by a random bit generator, unique for every encryption operation, that you used to encrypt your input files or that you want Elastic Transcoder to use to encrypt your output files. The initialization vector must be base64-encoded, and it must be exactly 16 bytes long before being base64-encoded.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyMd5</i> The MD5 digest of the key that you used to encrypt your input file, or that you want Elastic Transcoder to use to encrypt your output file. Elastic Transcoder uses the key digest as a checksum to make sure your key was not corrupted in transit. The key MD5 must be base64-encoded, and it must be exactly 16 bytes long before being base64-encoded.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> The data encryption key that you want Elastic Transcoder to use to encrypt your output file, or that was used to encrypt your input file. The key must be base64-encoded and it must be one of the following bit lengths before being base64-encoded:  <code>128</code>, <code>192</code>, or <code>256</code>.  The key must also be encrypted by using the Amazon Key Management Service.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> The specific server-side encryption mode that you want Elastic Transcoder to use when decrypting your input files or encrypting your output files. Elastic Transcoder supports the following options: <ul> <li>  <b>s3:</b> Amazon S3 creates and manages the keys used for encrypting your files. </li> <li>  <b>s3-aws-kms:</b> Amazon S3 calls the Amazon Key Management Service, which creates and manages the keys that are used for encrypting your files. If you specify <code>s3-aws-kms</code> and you don't want to use the default key, you must add the AWS-KMS key that you want to use to your pipeline. </li> <li>  <b>aes-cbc-pkcs7:</b> A padded cipher-block mode of operation originally used for HLS files. </li> <li>  <b>aes-ctr:</b> AES Counter Mode. </li> <li>  <b>aes-gcm:</b> AES Galois Counter Mode, a mode of operation that is an authenticated encryption format, meaning that a file, key, or initialization vector that has been tampered with fails the decryption process. </li> </ul> For all three AES options, you must provide the following settings, which must be base64-encoded: <ul> <li>  <b>Key</b>  </li> <li>  <b>Key MD5</b>  </li> <li>  <b>Initialization Vector</b>  </li> </ul> <important> For the AES modes, your private encryption keys and your unencrypted data are never stored by AWS; therefore, it is important that you safely manage your encryption keys. If you lose them, you won't be able to unencrypt your data. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Duration</i> The duration of the clip. The format can be either HH:mm:ss.SSS (maximum value: 23:59:59.999; SSS is thousandths of a second) or sssss.SSS (maximum value: 86399.999). If you don't specify a value, Elastic Transcoder creates an output file from StartTime to the end of the file. If you specify a value longer than the duration of the input file, Elastic Transcoder transcodes the file and returns a warning message.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTime</i> The place in the input file where you want a clip to start. The format can be either HH:mm:ss.SSS (maximum value: 23:59:59.999; SSS is thousandths of a second) or sssss.SSS (maximum value: 86399.999). If you don't specify a value, Elastic Transcoder starts at the beginning of the input file.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CaptionSources</i> Source files for the input sidecar captions used during the transcoding process. To omit all sidecar captions, leave <code>CaptionSources</code> blank.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MergePolicy</i> A policy that determines how Elastic Transcoder handles the existence of multiple captions. <ul> <li>  <b>MergeOverride:</b> Elastic Transcoder transcodes both embedded and sidecar captions into outputs. If captions for a language are embedded in the input file and also appear in a sidecar file, Elastic Transcoder uses the sidecar captions and ignores the embedded captions for that language. </li> <li>  <b>MergeRetain:</b> Elastic Transcoder transcodes both embedded and sidecar captions into outputs. If captions for a language are embedded in the input file and also appear in a sidecar file, Elastic Transcoder uses the embedded captions and ignores the sidecar captions for that language. If <code>CaptionSources</code> is empty, Elastic Transcoder omits all sidecar captions from the output files. </li> <li>  <b>Override:</b> Elastic Transcoder transcodes only the sidecar captions that you specify in <code>CaptionSources</code>. </li> </ul>  <code>MergePolicy</code> cannot be null.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DurationMillis</i> The detected duration of the input file, in milliseconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FileSize</i> The detected file size of the input file, in bytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameRate</i> The detected frame rate of the input file, in frames per second.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Height</i> The detected height of the input file, in pixels.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Width</i> The detected width of the input file, in pixels.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> You can specify encryption settings for any output files that you want to use for a transcoding job. This includes the output file and any watermarks, thumbnails, album art, or captions that you want to use. You must specify encryption settings for each file individually.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: You can specify encryption settings for any output files that you want to use for a transcoding job. This includes the output file and any watermarks, thumbnails, album art, or captions that you want to use. You must specify encryption settings for each file individually.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Captions</i> You can configure Elastic Transcoder to transcode captions, or subtitles, from one format to another. All captions must be in UTF-8. Elastic Transcoder supports two types of captions: <ul> <li>  <b>Embedded:</b> Embedded captions are included in the same file as the audio and video. Elastic Transcoder supports only one embedded caption per language, to a maximum of 300 embedded captions per file. Valid input values include: <code>CEA-608 (EIA-608</code>, first non-empty channel only), <code>CEA-708 (EIA-708</code>, first non-empty channel only), and <code>mov-text</code>  Valid outputs include: <code>mov-text</code>  Elastic Transcoder supports a maximum of one embedded format per output. </li> <li>  <b>Sidecar:</b> Sidecar captions are kept in a separate metadata file from the audio and video data. Sidecar captions require a player that is capable of understanding the relationship between the video file and the sidecar file. Elastic Transcoder supports only one sidecar caption per language, to a maximum of 20 sidecar captions per file. Valid input values include: <code>dfxp</code> (first div element only), <code>ebu-tt</code>, <code>scc</code>, <code>smpt</code>, <code>srt</code>, <code>ttml</code> (first div element only), and <code>webvtt</code>  Valid outputs include: <code>dfxp</code> (first div element only), <code>scc</code>, <code>srt</code>, and <code>webvtt</code>. </li> </ul> If you want ttml or smpte-tt compatible captions, specify dfxp as your output format. Elastic Transcoder does not support OCR (Optical Character Recognition), does not accept pictures as a valid input for captions, and is not available for audio-only transcoding. Elastic Transcoder does not preserve text formatting (for example, italics) during the transcoding process. To remove captions or leave the captions empty, set <code>Captions</code> to null. To pass through existing captions unchanged, set the <code>MergePolicy</code> to <code>MergeRetain</code>, and pass in a null <code>CaptionSources</code> array. For more information on embedded files, see the Subtitles Wikipedia page. For more information on sidecar files, see the Extensible Metadata Platform and Sidecar file Wikipedia pages.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Captions</b>: You can configure Elastic Transcoder to transcode captions, or subtitles, from one format to another. All captions must be in UTF-8. Elastic Transcoder supports two types of captions: <ul> <li>  <b>Embedded:</b> Embedded captions are included in the same file as the audio and video. Elastic Transcoder supports only one embedded caption per language, to a maximum of 300 embedded captions per file. Valid input values include: <code>CEA-608 (EIA-608</code>, first non-empty channel only), <code>CEA-708 (EIA-708</code>, first non-empty channel only), and <code>mov-text</code>  Valid outputs include: <code>mov-text</code>  Elastic Transcoder supports a maximum of one embedded format per output. </li> <li>  <b>Sidecar:</b> Sidecar captions are kept in a separate metadata file from the audio and video data. Sidecar captions require a player that is capable of understanding the relationship between the video file and the sidecar file. Elastic Transcoder supports only one sidecar caption per language, to a maximum of 20 sidecar captions per file. Valid input values include: <code>dfxp</code> (first div element only), <code>ebu-tt</code>, <code>scc</code>, <code>smpt</code>, <code>srt</code>, <code>ttml</code> (first div element only), and <code>webvtt</code>  Valid outputs include: <code>dfxp</code> (first div element only), <code>scc</code>, <code>srt</code>, and <code>webvtt</code>. </li> </ul> If you want ttml or smpte-tt compatible captions, specify dfxp as your output format. Elastic Transcoder does not support OCR (Optical Character Recognition), does not accept pictures as a valid input for captions, and is not available for audio-only transcoding. Elastic Transcoder does not preserve text formatting (for example, italics) during the transcoding process. To remove captions or leave the captions empty, set <code>Captions</code> to null. To pass through existing captions unchanged, set the <code>MergePolicy</code> to <code>MergeRetain</code>, and pass in a null <code>CaptionSources</code> array. For more information on embedded files, see the Subtitles Wikipedia page. For more information on sidecar files, see the Extensible Metadata Platform and Sidecar file Wikipedia pages.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Composition</i> You can create an output file that contains an excerpt from the input file. This excerpt, called a clip, can come from the beginning, middle, or end of the file. The Composition object contains settings for the clips that make up an output file. For the current release, you can only specify settings for a single clip per output file. The Composition object cannot be null.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AlbumArt</i> Information about the album art that you want Elastic Transcoder to add to the file during transcoding. You can specify up to twenty album artworks for each output. Settings for each artwork must be defined in the job for the current output.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AlbumArt</b>: Information about the album art that you want Elastic Transcoder to add to the file during transcoding. You can specify up to twenty album artworks for each output. Settings for each artwork must be defined in the job for the current output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Watermarks</i> Information about the watermarks that you want Elastic Transcoder to add to the video during transcoding. You can specify up to four watermarks for each output. Settings for each watermark must be defined in the preset for the current output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDuration</i> <important> (Outputs in Fragmented MP4 or MPEG-TS format only. </important> If you specify a preset in <code>PresetId</code> for which the value of <code>Container</code> is <code>fmp4</code> (Fragmented MP4) or <code>ts</code> (MPEG-TS), <code>SegmentDuration</code> is the target maximum duration of each segment in seconds. For <code>HLSv3</code> format playlists, each media segment is stored in a separate <code>.ts</code> file. For <code>HLSv4</code> and <code>Smooth</code> playlists, all media segments for an output are stored in a single file. Each segment is approximately the length of the <code>SegmentDuration</code>, though individual segments might be shorter or longer. The range of valid values is 1 to 60 seconds. If the duration of the video is not evenly divisible by <code>SegmentDuration</code>, the duration of the last segment is the remainder of total length/SegmentDuration. Elastic Transcoder creates an output-specific playlist for each output <code>HLS</code> output that you specify in OutputKeys. To add an output to the master playlist for this job, include it in the <code>OutputKeys</code> of the associated playlist.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PresetId</i>  The <code>Id</code> of the preset to use for this job. The preset determines the audio, video, and thumbnail settings that Elastic Transcoder uses for transcoding. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Rotate</i>  The number of degrees clockwise by which you want Elastic Transcoder to rotate the output relative to the input. Enter one of the following values: <code>auto</code>, <code>0</code>, <code>90</code>, <code>180</code>, <code>270</code>. The value <code>auto</code> generally works only if the file that you're transcoding contains rotation metadata. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThumbnailEncryption</i> The encryption settings, if any, that you want Elastic Transcoder to apply to your thumbnail.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ThumbnailEncryption</b>: The encryption settings, if any, that you want Elastic Transcoder to apply to your thumbnail.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThumbnailPattern</i> Whether you want Elastic Transcoder to create thumbnails for your videos and, if so, how you want Elastic Transcoder to name the files. If you don't want Elastic Transcoder to create thumbnails, specify "". If you do want Elastic Transcoder to create thumbnails, specify the information that you want to include in the file name for each thumbnail. You can specify the following values in any sequence: <ul> <li>  <b> <code>{count}</code> (Required)</b>: If you want to create thumbnails, you must include <code>{count}</code> in the <code>ThumbnailPattern</code> object. Wherever you specify <code>{count}</code>, Elastic Transcoder adds a five-digit sequence number (beginning with <b>00001</b>) to thumbnail file names. The number indicates where a given thumbnail appears in the sequence of thumbnails for a transcoded file.  <important> If you specify a literal value and/or <code>{resolution}</code> but you omit <code>{count}</code>, Elastic Transcoder returns a validation error and does not create the job. </important> </li> <li>  <b>Literal values (Optional)</b>: You can specify literal values anywhere in the <code>ThumbnailPattern</code> object. For example, you can include them as a file name prefix or as a delimiter between <code>{resolution}</code> and <code>{count}</code>.  </li> <li>  <b> <code>{resolution}</code> (Optional)</b>: If you want Elastic Transcoder to include the resolution in the file name, include <code>{resolution}</code> in the <code>ThumbnailPattern</code> object.  </li> </ul> When creating thumbnails, Elastic Transcoder automatically saves the files in the format (.jpg or .png) that appears in the preset that you specified in the <code>PresetID</code> value of <code>CreateJobOutput</code>. Elastic Transcoder also appends the applicable file name extension.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i>  The name to assign to the transcoded file. Elastic Transcoder saves the file in the Amazon S3 bucket specified by the <code>OutputBucket</code> object in the pipeline that is specified by the pipeline ID. If a file with the specified name already exists in the output bucket, the job fails. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InitializationVector</i> The series of random bits created by a random bit generator, unique for every encryption operation, that you used to encrypt your input files or that you want Elastic Transcoder to use to encrypt your output files. The initialization vector must be base64-encoded, and it must be exactly 16 bytes long before being base64-encoded.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyMd5</i> The MD5 digest of the key that you used to encrypt your input file, or that you want Elastic Transcoder to use to encrypt your output file. Elastic Transcoder uses the key digest as a checksum to make sure your key was not corrupted in transit. The key MD5 must be base64-encoded, and it must be exactly 16 bytes long before being base64-encoded.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> The data encryption key that you want Elastic Transcoder to use to encrypt your output file, or that was used to encrypt your input file. The key must be base64-encoded and it must be one of the following bit lengths before being base64-encoded:  <code>128</code>, <code>192</code>, or <code>256</code>.  The key must also be encrypted by using the Amazon Key Management Service.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> The specific server-side encryption mode that you want Elastic Transcoder to use when decrypting your input files or encrypting your output files. Elastic Transcoder supports the following options: <ul> <li>  <b>s3:</b> Amazon S3 creates and manages the keys used for encrypting your files. </li> <li>  <b>s3-aws-kms:</b> Amazon S3 calls the Amazon Key Management Service, which creates and manages the keys that are used for encrypting your files. If you specify <code>s3-aws-kms</code> and you don't want to use the default key, you must add the AWS-KMS key that you want to use to your pipeline. </li> <li>  <b>aes-cbc-pkcs7:</b> A padded cipher-block mode of operation originally used for HLS files. </li> <li>  <b>aes-ctr:</b> AES Counter Mode. </li> <li>  <b>aes-gcm:</b> AES Galois Counter Mode, a mode of operation that is an authenticated encryption format, meaning that a file, key, or initialization vector that has been tampered with fails the decryption process. </li> </ul> For all three AES options, you must provide the following settings, which must be base64-encoded: <ul> <li>  <b>Key</b>  </li> <li>  <b>Key MD5</b>  </li> <li>  <b>Initialization Vector</b>  </li> </ul> <important> For the AES modes, your private encryption keys and your unencrypted data are never stored by AWS; therefore, it is important that you safely manage your encryption keys. If you lose them, you won't be able to unencrypt your data. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Artwork</i> The file to be used as album art. There can be multiple artworks associated with an audio file, to a maximum of 20. Valid formats are <code>.jpg</code> and <code>.png</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MergePolicy</i> A policy that determines how Elastic Transcoder handles the existence of multiple album artwork files. <ul> <li>  <code>Replace:</code> The specified album art replaces any existing album art. </li> <li>  <code>Prepend:</code> The specified album art is placed in front of any existing album art. </li> <li>  <code>Append:</code> The specified album art is placed after any existing album art. </li> <li>  <code>Fallback:</code> If the original input file contains artwork, Elastic Transcoder uses that artwork for the output. If the original input does not contain artwork, Elastic Transcoder uses the specified album art file. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CaptionFormats</i> The array of file formats for the output captions. If you leave this value blank, Elastic Transcoder returns an error.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CaptionSources</i> Source files for the input sidecar captions used during the transcoding process. To omit all sidecar captions, leave <code>CaptionSources</code> blank.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MergePolicy</i> A policy that determines how Elastic Transcoder handles the existence of multiple captions. <ul> <li>  <b>MergeOverride:</b> Elastic Transcoder transcodes both embedded and sidecar captions into outputs. If captions for a language are embedded in the input file and also appear in a sidecar file, Elastic Transcoder uses the sidecar captions and ignores the embedded captions for that language. </li> <li>  <b>MergeRetain:</b> Elastic Transcoder transcodes both embedded and sidecar captions into outputs. If captions for a language are embedded in the input file and also appear in a sidecar file, Elastic Transcoder uses the embedded captions and ignores the sidecar captions for that language. If <code>CaptionSources</code> is empty, Elastic Transcoder omits all sidecar captions from the output files. </li> <li>  <b>Override:</b> Elastic Transcoder transcodes only the sidecar captions that you specify in <code>CaptionSources</code>. </li> </ul>  <code>MergePolicy</code> cannot be null.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InitializationVector</i> The series of random bits created by a random bit generator, unique for every encryption operation, that you used to encrypt your input files or that you want Elastic Transcoder to use to encrypt your output files. The initialization vector must be base64-encoded, and it must be exactly 16 bytes long before being base64-encoded.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyMd5</i> The MD5 digest of the key that you used to encrypt your input file, or that you want Elastic Transcoder to use to encrypt your output file. Elastic Transcoder uses the key digest as a checksum to make sure your key was not corrupted in transit. The key MD5 must be base64-encoded, and it must be exactly 16 bytes long before being base64-encoded.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> The data encryption key that you want Elastic Transcoder to use to encrypt your output file, or that was used to encrypt your input file. The key must be base64-encoded and it must be one of the following bit lengths before being base64-encoded:  <code>128</code>, <code>192</code>, or <code>256</code>.  The key must also be encrypted by using the Amazon Key Management Service.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> The specific server-side encryption mode that you want Elastic Transcoder to use when decrypting your input files or encrypting your output files. Elastic Transcoder supports the following options: <ul> <li>  <b>s3:</b> Amazon S3 creates and manages the keys used for encrypting your files. </li> <li>  <b>s3-aws-kms:</b> Amazon S3 calls the Amazon Key Management Service, which creates and manages the keys that are used for encrypting your files. If you specify <code>s3-aws-kms</code> and you don't want to use the default key, you must add the AWS-KMS key that you want to use to your pipeline. </li> <li>  <b>aes-cbc-pkcs7:</b> A padded cipher-block mode of operation originally used for HLS files. </li> <li>  <b>aes-ctr:</b> AES Counter Mode. </li> <li>  <b>aes-gcm:</b> AES Galois Counter Mode, a mode of operation that is an authenticated encryption format, meaning that a file, key, or initialization vector that has been tampered with fails the decryption process. </li> </ul> For all three AES options, you must provide the following settings, which must be base64-encoded: <ul> <li>  <b>Key</b>  </li> <li>  <b>Key MD5</b>  </li> <li>  <b>Initialization Vector</b>  </li> </ul> <important> For the AES modes, your private encryption keys and your unencrypted data are never stored by AWS; therefore, it is important that you safely manage your encryption keys. If you lose them, you won't be able to unencrypt your data. </important><br/>
        :return: Job:  A section of the response body that provides information about the job that is created.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timing</i> Details about the timing of a job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Timing</b>: Details about the timing of a job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserMetadata</i> User-defined metadata that you want to associate with an Elastic Transcoder job. You specify metadata in <code>key/value</code> pairs, and you can add up to 10 <code>key/value</code> pairs per job. Elastic Transcoder does not guarantee that <code>key/value</code> pairs are returned in the same order in which you specify them. Metadata <code>keys</code> and <code>values</code> must use characters from the following list: <ul> <li>  <code>0-9</code>  </li> <li>  <code>A-Z</code> and <code>a-z</code>  </li> <li>  <code>Space</code>  </li> <li> The following symbols: <code>_.:/=+-%@</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i>  The status of the job: <code>Submitted</code>, <code>Progressing</code>, <code>Complete</code>, <code>Canceled</code>, or <code>Error</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Playlists</i> <important> Outputs in Fragmented MP4 or MPEG-TS format only. </important> If you specify a preset in <code>PresetId</code> for which the value of <code>Container</code> is fmp4 (Fragmented MP4) or ts (MPEG-TS), <code>Playlists</code> contains information about the master playlists that you want Elastic Transcoder to create. The maximum number of master playlists in a job is 30.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputKeyPrefix</i> The value, if any, that you want Elastic Transcoder to prepend to the names of all files that this job creates, including output files, thumbnails, and playlists. We recommend that you add a / or some other delimiter to the end of the <code>OutputKeyPrefix</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Outputs</i> Information about the output files. We recommend that you use the <code>Outputs</code> syntax for all jobs, even when you want Elastic Transcoder to transcode a file into only one format. Do not use both the <code>Outputs</code> and <code>Output</code> syntaxes in the same request. You can create a maximum of 30 outputs per job.  If you specify more than one output for a job, Elastic Transcoder creates the files for each output in the order in which you specify them in the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Output</i> If you specified one output for a job, information about that output. If you specified multiple outputs for a job, the Output object lists information about the first output. This duplicates the information that is listed for the first output in the Outputs object. <important> Outputs recommended instead. </important> A section of the request or response body that provides information about the transcoded (target) file. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Output</b>: If you specified one output for a job, information about that output. If you specified multiple outputs for a job, the Output object lists information about the first output. This duplicates the information that is listed for the first output in the Outputs object. <important> Outputs recommended instead. </important> A section of the request or response body that provides information about the transcoded (target) file. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Inputs</i> Information about the files that you're transcoding. If you specified multiple files for this job, Elastic Transcoder stitches the files together to make one output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Input</i> A section of the request or response body that provides information about the file that is being transcoded.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Input</b>: A section of the request or response body that provides information about the file that is being transcoded.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PipelineId</i>  The <code>Id</code> of the pipeline that you want Elastic Transcoder to use for transcoding. The pipeline determines several settings, including the Amazon S3 bucket from which Elastic Transcoder gets the files to transcode and the bucket into which Elastic Transcoder puts the transcoded files. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) for the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier that Elastic Transcoder assigned to the job. You use this value to get settings for the job or to delete the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Container</i> The container type for the input file. If you want Elastic Transcoder to automatically detect the container type of the input file, specify <code>auto</code>. If you want to specify the container type for the input file, enter one of the following values:   <code>3gp</code>, <code>aac</code>, <code>asf</code>, <code>avi</code>, <code>divx</code>, <code>flv</code>, <code>m4a</code>, <code>mkv</code>, <code>mov</code>, <code>mp3</code>, <code>mp4</code>, <code>mpeg</code>, <code>mpeg-ps</code>, <code>mpeg-ts</code>, <code>mxf</code>, <code>ogg</code>, <code>vob</code>, <code>wav</code>, <code>webm</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Interlaced</i> Whether the input file is interlaced. If you want Elastic Transcoder to automatically detect whether the input file is interlaced, specify <code>auto</code>. If you want to specify whether the input file is interlaced, enter one of the following values:  <code>true</code>, <code>false</code>  If you specify a value other than <code>auto</code>, Elastic Transcoder disables automatic detection of interlacing.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AspectRatio</i>  The aspect ratio of the input file. If you want Elastic Transcoder to automatically detect the aspect ratio of the input file, specify <code>auto</code>. If you want to specify the aspect ratio for the output file, enter one of the following values:   <code>1:1</code>, <code>4:3</code>, <code>3:2</code>, <code>16:9</code>   If you specify a value other than <code>auto</code>, Elastic Transcoder disables automatic detection of the aspect ratio. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Resolution</i> This value must be <code>auto</code>, which causes Elastic Transcoder to automatically detect the resolution of the input file.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameRate</i> The frame rate of the input file. If you want Elastic Transcoder to automatically detect the frame rate of the input file, specify <code>auto</code>. If you want to specify the frame rate for the input file, enter one of the following values:   <code>10</code>, <code>15</code>, <code>23.97</code>, <code>24</code>, <code>25</code>, <code>29.97</code>, <code>30</code>, <code>60</code>  If you specify a value other than <code>auto</code>, Elastic Transcoder disables automatic detection of the frame rate.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i>  The name of the file to transcode. Elsewhere in the body of the JSON block is the the ID of the pipeline to use for processing the job. The <code>InputBucket</code> object in that pipeline tells Elastic Transcoder which Amazon S3 bucket to get the file from.  If the file name includes a prefix, such as <code>cooking/lasagna.mpg</code>, include the prefix in the key. If the file isn't in the specified bucket, Elastic Transcoder returns an error.<br/>The encryption settings, if any, that are used for decrypting your input files. If your input file is encrypted, you must specify the mode that Elastic Transcoder uses to decrypt your file.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: The encryption settings, if any, that are used for decrypting your input files. If your input file is encrypted, you must specify the mode that Elastic Transcoder uses to decrypt your file.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ThumbnailPattern</i> Whether you want Elastic Transcoder to create thumbnails for your videos and, if so, how you want Elastic Transcoder to name the files. If you don't want Elastic Transcoder to create thumbnails, specify "". If you do want Elastic Transcoder to create thumbnails, specify the information that you want to include in the file name for each thumbnail. You can specify the following values in any sequence: <ul> <li>  <b> <code>{count}</code> (Required)</b>: If you want to create thumbnails, you must include <code>{count}</code> in the <code>ThumbnailPattern</code> object. Wherever you specify <code>{count}</code>, Elastic Transcoder adds a five-digit sequence number (beginning with <b>00001</b>) to thumbnail file names. The number indicates where a given thumbnail appears in the sequence of thumbnails for a transcoded file.  <important> If you specify a literal value and/or <code>{resolution}</code> but you omit <code>{count}</code>, Elastic Transcoder returns a validation error and does not create the job. </important> </li> <li>  <b>Literal values (Optional)</b>: You can specify literal values anywhere in the <code>ThumbnailPattern</code> object. For example, you can include them as a file name prefix or as a delimiter between <code>{resolution}</code> and <code>{count}</code>.  </li> <li>  <b> <code>{resolution}</code> (Optional)</b>: If you want Elastic Transcoder to include the resolution in the file name, include <code>{resolution}</code> in the <code>ThumbnailPattern</code> object.  </li> </ul> When creating thumbnails, Elastic Transcoder automatically saves the files in the format (.jpg or .png) that appears in the preset that you specified in the <code>PresetID</code> value of <code>CreateJobOutput</code>. Elastic Transcoder also appends the applicable file name extension.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i>  The name to assign to the transcoded file. Elastic Transcoder saves the file in the Amazon S3 bucket specified by the <code>OutputBucket</code> object in the pipeline that is specified by the pipeline ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> A sequential counter, starting with 1, that identifies an output among the outputs from the current job. In the Output syntax, this value is always 1.<br/>The encryption settings, if any, that you want Elastic Transcoder to apply to your thumbnail.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ThumbnailEncryption</b>: The encryption settings, if any, that you want Elastic Transcoder to apply to your thumbnail.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FinishTimeMillis</i> The time the job finished transcoding, in epoch milliseconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTimeMillis</i> The time the job began transcoding, in epoch milliseconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmitTimeMillis</i> The time the job was submitted to Elastic Transcoder, in epoch milliseconds.<br/>        """
        pass

    def create_pipeline(self, Role: str, InputBucket: str, Name: str, OutputBucket: str=None, AwsKmsKeyArn: str=None, Notifications: Notifications=None, ContentConfig: PipelineOutputConfig=None, ThumbnailConfig: PipelineOutputConfig=None) -> CreatePipelineResponse:
        """The CreatePipeline operation creates a pipeline with settings that you specify.

        :param self:
        :param ThumbnailConfig:  The <code>ThumbnailConfig</code> object specifies several values, including the Amazon S3 bucket in which you want Elastic Transcoder to save thumbnail files, which users you want to have access to the files, the type of access you want users to have, and the storage class that you want to assign to the files. If you specify values for <code>ContentConfig</code>, you must also specify values for <code>ThumbnailConfig</code> even if you don't want to create thumbnails. If you specify values for <code>ContentConfig</code> and <code>ThumbnailConfig</code>, omit the <code>OutputBucket</code> object. <ul> <li>  <b>Bucket</b>: The Amazon S3 bucket in which you want Elastic Transcoder to save thumbnail files. </li> <li>  <b>Permissions</b> (Optional): The <code>Permissions</code> object specifies which users and/or predefined Amazon S3 groups you want to have access to thumbnail files, and the type of access you want them to have. You can grant permissions to a maximum of 30 users and/or predefined Amazon S3 groups. </li> <li>  <b>GranteeType</b>: Specify the type of value that appears in the Grantee object:  <ul> <li>  <b>Canonical</b>: The value in the <code>Grantee</code> object is either the canonical user ID for an AWS account or an origin access identity for an Amazon CloudFront distribution. <important> A canonical user ID is not the same as an AWS account number. </important> </li> <li>  <b>Email</b>: The value in the <code>Grantee</code> object is the registered email address of an AWS account.  </li> <li>  <b>Group</b>: The value in the <code>Grantee</code> object is one of the following predefined Amazon S3 groups: <code>AllUsers</code>, <code>AuthenticatedUsers</code>, or <code>LogDelivery</code>. </li> </ul> </li> <li>  <b>Grantee</b>: The AWS user or group that you want to have access to thumbnail files. To identify the user or group, you can specify the canonical user ID for an AWS account, an origin access identity for a CloudFront distribution, the registered email address of an AWS account, or a predefined Amazon S3 group.  </li> <li>  <b>Access</b>: The permission that you want to give to the AWS user that you specified in <code>Grantee</code>. Permissions are granted on the thumbnail files that Elastic Transcoder adds to the bucket. Valid values include:  <ul> <li>  <code>READ</code>: The grantee can read the thumbnails and metadata for objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>READ_ACP</code>: The grantee can read the object ACL for thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>WRITE_ACP</code>: The grantee can write the ACL for the thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>FULL_CONTROL</code>: The grantee has <code>READ</code>, <code>READ_ACP</code>, and <code>WRITE_ACP</code> permissions for the thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> </ul> </li> <li>  <b>StorageClass</b>: The Amazon S3 storage class, <code>Standard</code> or <code>ReducedRedundancy</code>, that you want Elastic Transcoder to assign to the thumbnails that it stores in your Amazon S3 bucket. </li> </ul>:<br/>ContentConfig:  The optional <code>ContentConfig</code> object specifies information about the Amazon S3 bucket in which you want Elastic Transcoder to save transcoded files and playlists: which bucket to use, which users you want to have access to the files, the type of access you want users to have, and the storage class that you want to assign to the files. If you specify values for <code>ContentConfig</code>, you must also specify values for <code>ThumbnailConfig</code>. If you specify values for <code>ContentConfig</code> and <code>ThumbnailConfig</code>, omit the <code>OutputBucket</code> object. <ul> <li>  <b>Bucket</b>: The Amazon S3 bucket in which you want Elastic Transcoder to save transcoded files and playlists. </li> <li>  <b>Permissions</b> (Optional): The Permissions object specifies which users you want to have access to transcoded files and the type of access you want them to have. You can grant permissions to a maximum of 30 users and/or predefined Amazon S3 groups. </li> <li>  <b>Grantee Type</b>: Specify the type of value that appears in the <code>Grantee</code> object:  <ul> <li>  <b>Canonical</b>: The value in the <code>Grantee</code> object is either the canonical user ID for an AWS account or an origin access identity for an Amazon CloudFront distribution. For more information about canonical user IDs, see Access Control List (ACL) Overview in the Amazon Simple Storage Service Developer Guide. For more information about using CloudFront origin access identities to require that users use CloudFront URLs instead of Amazon S3 URLs, see Using an Origin Access Identity to Restrict Access to Your Amazon S3 Content. <important> A canonical user ID is not the same as an AWS account number. </important> </li> <li>  <b>Email</b>: The value in the <code>Grantee</code> object is the registered email address of an AWS account. </li> <li>  <b>Group</b>: The value in the <code>Grantee</code> object is one of the following predefined Amazon S3 groups: <code>AllUsers</code>, <code>AuthenticatedUsers</code>, or <code>LogDelivery</code>. </li> </ul> </li> <li>  <b>Grantee</b>: The AWS user or group that you want to have access to transcoded files and playlists. To identify the user or group, you can specify the canonical user ID for an AWS account, an origin access identity for a CloudFront distribution, the registered email address of an AWS account, or a predefined Amazon S3 group  </li> <li>  <b>Access</b>: The permission that you want to give to the AWS user that you specified in <code>Grantee</code>. Permissions are granted on the files that Elastic Transcoder adds to the bucket, including playlists and video files. Valid values include:  <ul> <li>  <code>READ</code>: The grantee can read the objects and metadata for objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>READ_ACP</code>: The grantee can read the object ACL for objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>WRITE_ACP</code>: The grantee can write the ACL for the objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>FULL_CONTROL</code>: The grantee has <code>READ</code>, <code>READ_ACP</code>, and <code>WRITE_ACP</code> permissions for the objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> </ul> </li> <li>  <b>StorageClass</b>: The Amazon S3 storage class, <code>Standard</code> or <code>ReducedRedundancy</code>, that you want Elastic Transcoder to assign to the video files and playlists that it stores in your Amazon S3 bucket. </li> </ul>:<br/>Notifications:  The Amazon Simple Notification Service (Amazon SNS) topic that you want to notify to report job status. <important> To receive notifications, you must also subscribe to the new topic in the Amazon SNS console. </important> <ul> <li>  <b>Progressing</b>: The topic ARN for the Amazon Simple Notification Service (Amazon SNS) topic that you want to notify when Elastic Transcoder has started to process a job in this pipeline. This is the ARN that Amazon SNS returned when you created the topic. For more information, see Create a Topic in the Amazon Simple Notification Service Developer Guide. </li> <li>  <b>Complete</b>: The topic ARN for the Amazon SNS topic that you want to notify when Elastic Transcoder has finished processing a job in this pipeline. This is the ARN that Amazon SNS returned when you created the topic. </li> <li>  <b>Warning</b>: The topic ARN for the Amazon SNS topic that you want to notify when Elastic Transcoder encounters a warning condition while processing a job in this pipeline. This is the ARN that Amazon SNS returned when you created the topic. </li> <li>  <b>Error</b>: The topic ARN for the Amazon SNS topic that you want to notify when Elastic Transcoder encounters an error condition while processing a job in this pipeline. This is the ARN that Amazon SNS returned when you created the topic. </li> </ul>:<br/>AwsKmsKeyArn:  The AWS Key Management Service (AWS KMS) key that you want to use with this pipeline. If you use either <code>s3</code> or <code>s3-aws-kms</code> as your <code>Encryption:Mode</code>, you don't need to provide a key with your job because a default key, known as an AWS-KMS key, is created for you automatically. You need to provide an AWS-KMS key only if you want to use a non-default AWS-KMS key, or if you are using an <code>Encryption:Mode</code> of <code>aes-cbc-pkcs7</code>, <code>aes-ctr</code>, or <code>aes-gcm</code>.<br/>Role:  The IAM Amazon Resource Name (ARN) for the role that you want Elastic Transcoder to use to create the pipeline.<br/>OutputBucket:  The Amazon S3 bucket in which you want Elastic Transcoder to save the transcoded files. (Use this, or use ContentConfig:Bucket plus ThumbnailConfig:Bucket.) Specify this value when all of the following are true: <ul> <li> You want to save transcoded files, thumbnails (if any), and playlists (if any) together in one bucket. </li> <li> You do not want to specify the users or groups who have access to the transcoded files, thumbnails, and playlists. </li> <li> You do not want to specify the permissions that Elastic Transcoder grants to the files.  <important> When Elastic Transcoder saves files in <code>OutputBucket</code>, it grants full control over the files only to the AWS account that owns the role that is specified by <code>Role</code>. </important> </li> <li> You want to associate the transcoded files and thumbnails with the Amazon S3 Standard storage class. </li> </ul> If you want to save transcoded files and playlists in one bucket and thumbnails in another bucket, specify which users can access the transcoded files or the permissions the users have, or change the Amazon S3 storage class, omit <code>OutputBucket</code> and specify values for <code>ContentConfig</code> and <code>ThumbnailConfig</code> instead.<br/>InputBucket:  The Amazon S3 bucket in which you saved the media files that you want to transcode.<br/>Name:  The name of the pipeline. We recommend that the name be unique within the AWS account, but uniqueness is not enforced. Constraints: Maximum 40 characters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Error</i> The Amazon SNS topic that you want to notify when Elastic Transcoder encounters an error condition.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Warning</i> The Amazon SNS topic that you want to notify when Elastic Transcoder encounters a warning condition.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Completed</i> The Amazon SNS topic that you want to notify when Elastic Transcoder has finished processing the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Progressing</i> The Amazon Simple Notification Service (Amazon SNS) topic that you want to notify when Elastic Transcoder has started to process the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Permissions</i> Optional. The <code>Permissions</code> object specifies which users and/or predefined Amazon S3 groups you want to have access to transcoded files and playlists, and the type of access you want them to have. You can grant permissions to a maximum of 30 users and/or predefined Amazon S3 groups. If you include <code>Permissions</code>, Elastic Transcoder grants only the permissions that you specify. It does not grant full permissions to the owner of the role specified by <code>Role</code>. If you want that user to have full control, you must explicitly grant full control to the user.  If you omit <code>Permissions</code>, Elastic Transcoder grants full control over the transcoded files and playlists to the owner of the role specified by <code>Role</code>, and grants no other permissions to any other user or group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageClass</i>  The Amazon S3 storage class, <code>Standard</code> or <code>ReducedRedundancy</code>, that you want Elastic Transcoder to assign to the video files and playlists that it stores in your Amazon S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i>  The Amazon S3 bucket in which you want Elastic Transcoder to save the transcoded files. Specify this value when all of the following are true: <ul> <li> You want to save transcoded files, thumbnails (if any), and playlists (if any) together in one bucket. </li> <li> You do not want to specify the users or groups who have access to the transcoded files, thumbnails, and playlists. </li> <li> You do not want to specify the permissions that Elastic Transcoder grants to the files. </li> <li> You want to associate the transcoded files and thumbnails with the Amazon S3 Standard storage class. </li> </ul> If you want to save transcoded files and playlists in one bucket and thumbnails in another bucket, specify which users can access the transcoded files or the permissions the users have, or change the Amazon S3 storage class, omit OutputBucket and specify values for <code>ContentConfig</code> and <code>ThumbnailConfig</code> instead. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Permissions</i> Optional. The <code>Permissions</code> object specifies which users and/or predefined Amazon S3 groups you want to have access to transcoded files and playlists, and the type of access you want them to have. You can grant permissions to a maximum of 30 users and/or predefined Amazon S3 groups. If you include <code>Permissions</code>, Elastic Transcoder grants only the permissions that you specify. It does not grant full permissions to the owner of the role specified by <code>Role</code>. If you want that user to have full control, you must explicitly grant full control to the user.  If you omit <code>Permissions</code>, Elastic Transcoder grants full control over the transcoded files and playlists to the owner of the role specified by <code>Role</code>, and grants no other permissions to any other user or group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageClass</i>  The Amazon S3 storage class, <code>Standard</code> or <code>ReducedRedundancy</code>, that you want Elastic Transcoder to assign to the video files and playlists that it stores in your Amazon S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i>  The Amazon S3 bucket in which you want Elastic Transcoder to save the transcoded files. Specify this value when all of the following are true: <ul> <li> You want to save transcoded files, thumbnails (if any), and playlists (if any) together in one bucket. </li> <li> You do not want to specify the users or groups who have access to the transcoded files, thumbnails, and playlists. </li> <li> You do not want to specify the permissions that Elastic Transcoder grants to the files. </li> <li> You want to associate the transcoded files and thumbnails with the Amazon S3 Standard storage class. </li> </ul> If you want to save transcoded files and playlists in one bucket and thumbnails in another bucket, specify which users can access the transcoded files or the permissions the users have, or change the Amazon S3 storage class, omit OutputBucket and specify values for <code>ContentConfig</code> and <code>ThumbnailConfig</code> instead. <br/>
        :return: Warnings:  Elastic Transcoder returns a warning if the resources used by your pipeline are not in the same region as the pipeline. Using resources in the same region, such as your Amazon S3 buckets, Amazon SNS notification topics, and AWS KMS key, reduces processing time and prevents cross-regional charges.<br/>Pipeline:  A section of the response body that provides information about the pipeline that is created.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThumbnailConfig</i> Information about the Amazon S3 bucket in which you want Elastic Transcoder to save thumbnail files. Either you specify both <code>ContentConfig</code> and <code>ThumbnailConfig</code>, or you specify <code>OutputBucket</code>. <ul> <li>  <code>Bucket</code>: The Amazon S3 bucket in which you want Elastic Transcoder to save thumbnail files.  </li> <li>  <code>Permissions</code>: A list of the users and/or predefined Amazon S3 groups you want to have access to thumbnail files, and the type of access that you want them to have.  <ul> <li> GranteeType: The type of value that appears in the Grantee object: <ul> <li>  <code>Canonical</code>: Either the canonical user ID for an AWS account or an origin access identity for an Amazon CloudFront distribution. <important> A canonical user ID is not the same as an AWS account number. </important> </li> <li>  <code>Email</code>: The registered email address of an AWS account. </li> <li>  <code>Group</code>: One of the following predefined Amazon S3 groups: <code>AllUsers</code>, <code>AuthenticatedUsers</code>, or <code>LogDelivery</code>. </li> </ul> </li> <li>  <code>Grantee</code>: The AWS user or group that you want to have access to thumbnail files. </li> <li> Access: The permission that you want to give to the AWS user that is listed in Grantee. Valid values include:  <ul> <li>  <code>READ</code>: The grantee can read the thumbnails and metadata for thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>READ_ACP</code>: The grantee can read the object ACL for thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>WRITE_ACP</code>: The grantee can write the ACL for the thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>FULL_CONTROL</code>: The grantee has READ, READ_ACP, and WRITE_ACP permissions for the thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> </ul> </li> </ul> </li> <li>  <code>StorageClass</code>: The Amazon S3 storage class, <code>Standard</code> or <code>ReducedRedundancy</code>, that you want Elastic Transcoder to assign to the thumbnails that it stores in your Amazon S3 bucket. </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ThumbnailConfig</b>: Information about the Amazon S3 bucket in which you want Elastic Transcoder to save thumbnail files. Either you specify both <code>ContentConfig</code> and <code>ThumbnailConfig</code>, or you specify <code>OutputBucket</code>. <ul> <li>  <code>Bucket</code>: The Amazon S3 bucket in which you want Elastic Transcoder to save thumbnail files.  </li> <li>  <code>Permissions</code>: A list of the users and/or predefined Amazon S3 groups you want to have access to thumbnail files, and the type of access that you want them to have.  <ul> <li> GranteeType: The type of value that appears in the Grantee object: <ul> <li>  <code>Canonical</code>: Either the canonical user ID for an AWS account or an origin access identity for an Amazon CloudFront distribution. <important> A canonical user ID is not the same as an AWS account number. </important> </li> <li>  <code>Email</code>: The registered email address of an AWS account. </li> <li>  <code>Group</code>: One of the following predefined Amazon S3 groups: <code>AllUsers</code>, <code>AuthenticatedUsers</code>, or <code>LogDelivery</code>. </li> </ul> </li> <li>  <code>Grantee</code>: The AWS user or group that you want to have access to thumbnail files. </li> <li> Access: The permission that you want to give to the AWS user that is listed in Grantee. Valid values include:  <ul> <li>  <code>READ</code>: The grantee can read the thumbnails and metadata for thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>READ_ACP</code>: The grantee can read the object ACL for thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>WRITE_ACP</code>: The grantee can write the ACL for the thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>FULL_CONTROL</code>: The grantee has READ, READ_ACP, and WRITE_ACP permissions for the thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> </ul> </li> </ul> </li> <li>  <code>StorageClass</code>: The Amazon S3 storage class, <code>Standard</code> or <code>ReducedRedundancy</code>, that you want Elastic Transcoder to assign to the thumbnails that it stores in your Amazon S3 bucket. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContentConfig</i> Information about the Amazon S3 bucket in which you want Elastic Transcoder to save transcoded files and playlists. Either you specify both <code>ContentConfig</code> and <code>ThumbnailConfig</code>, or you specify <code>OutputBucket</code>. <ul> <li>  <b>Bucket</b>: The Amazon S3 bucket in which you want Elastic Transcoder to save transcoded files and playlists. </li> <li>  <b>Permissions</b>: A list of the users and/or predefined Amazon S3 groups you want to have access to transcoded files and playlists, and the type of access that you want them to have.  <ul> <li> GranteeType: The type of value that appears in the <code>Grantee</code> object:  <ul> <li>  <code>Canonical</code>: Either the canonical user ID for an AWS account or an origin access identity for an Amazon CloudFront distribution. </li> <li>  <code>Email</code>: The registered email address of an AWS account. </li> <li>  <code>Group</code>: One of the following predefined Amazon S3 groups: <code>AllUsers</code>, <code>AuthenticatedUsers</code>, or <code>LogDelivery</code>. </li> </ul> </li> <li>  <code>Grantee</code>: The AWS user or group that you want to have access to transcoded files and playlists. </li> <li>  <code>Access</code>: The permission that you want to give to the AWS user that is listed in <code>Grantee</code>. Valid values include: <ul> <li>  <code>READ</code>: The grantee can read the objects and metadata for objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>READ_ACP</code>: The grantee can read the object ACL for objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>WRITE_ACP</code>: The grantee can write the ACL for the objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>FULL_CONTROL</code>: The grantee has <code>READ</code>, <code>READ_ACP</code>, and <code>WRITE_ACP</code> permissions for the objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> </ul> </li> </ul> </li> <li>  <b>StorageClass</b>: The Amazon S3 storage class, Standard or ReducedRedundancy, that you want Elastic Transcoder to assign to the video files and playlists that it stores in your Amazon S3 bucket.  </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ContentConfig</b>: Information about the Amazon S3 bucket in which you want Elastic Transcoder to save transcoded files and playlists. Either you specify both <code>ContentConfig</code> and <code>ThumbnailConfig</code>, or you specify <code>OutputBucket</code>. <ul> <li>  <b>Bucket</b>: The Amazon S3 bucket in which you want Elastic Transcoder to save transcoded files and playlists. </li> <li>  <b>Permissions</b>: A list of the users and/or predefined Amazon S3 groups you want to have access to transcoded files and playlists, and the type of access that you want them to have.  <ul> <li> GranteeType: The type of value that appears in the <code>Grantee</code> object:  <ul> <li>  <code>Canonical</code>: Either the canonical user ID for an AWS account or an origin access identity for an Amazon CloudFront distribution. </li> <li>  <code>Email</code>: The registered email address of an AWS account. </li> <li>  <code>Group</code>: One of the following predefined Amazon S3 groups: <code>AllUsers</code>, <code>AuthenticatedUsers</code>, or <code>LogDelivery</code>. </li> </ul> </li> <li>  <code>Grantee</code>: The AWS user or group that you want to have access to transcoded files and playlists. </li> <li>  <code>Access</code>: The permission that you want to give to the AWS user that is listed in <code>Grantee</code>. Valid values include: <ul> <li>  <code>READ</code>: The grantee can read the objects and metadata for objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>READ_ACP</code>: The grantee can read the object ACL for objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>WRITE_ACP</code>: The grantee can write the ACL for the objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>FULL_CONTROL</code>: The grantee has <code>READ</code>, <code>READ_ACP</code>, and <code>WRITE_ACP</code> permissions for the objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> </ul> </li> </ul> </li> <li>  <b>StorageClass</b>: The Amazon S3 storage class, Standard or ReducedRedundancy, that you want Elastic Transcoder to assign to the video files and playlists that it stores in your Amazon S3 bucket.  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Notifications</i> The Amazon Simple Notification Service (Amazon SNS) topic that you want to notify to report job status. <important> To receive notifications, you must also subscribe to the new topic in the Amazon SNS console. </important> <ul> <li>  <b>Progressing</b> (optional): The Amazon Simple Notification Service (Amazon SNS) topic that you want to notify when Elastic Transcoder has started to process the job. </li> <li>  <b>Complete</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder has finished processing the job. </li> <li>  <b>Warning</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder encounters a warning condition. </li> <li>  <b>Error</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder encounters an error condition. </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Notifications</b>: The Amazon Simple Notification Service (Amazon SNS) topic that you want to notify to report job status. <important> To receive notifications, you must also subscribe to the new topic in the Amazon SNS console. </important> <ul> <li>  <b>Progressing</b> (optional): The Amazon Simple Notification Service (Amazon SNS) topic that you want to notify when Elastic Transcoder has started to process the job. </li> <li>  <b>Complete</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder has finished processing the job. </li> <li>  <b>Warning</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder encounters a warning condition. </li> <li>  <b>Error</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder encounters an error condition. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AwsKmsKeyArn</i> The AWS Key Management Service (AWS KMS) key that you want to use with this pipeline. If you use either <code>s3</code> or <code>s3-aws-kms</code> as your <code>Encryption:Mode</code>, you don't need to provide a key with your job because a default key, known as an AWS-KMS key, is created for you automatically. You need to provide an AWS-KMS key only if you want to use a non-default AWS-KMS key, or if you are using an <code>Encryption:Mode</code> of <code>aes-cbc-pkcs7</code>, <code>aes-ctr</code>, or <code>aes-gcm</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Role</i> The IAM Amazon Resource Name (ARN) for the role that Elastic Transcoder uses to transcode jobs for this pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputBucket</i> The Amazon S3 bucket in which you want Elastic Transcoder to save transcoded files, thumbnails, and playlists. Either you specify this value, or you specify both <code>ContentConfig</code> and <code>ThumbnailConfig</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputBucket</i> The Amazon S3 bucket from which Elastic Transcoder gets media files for transcoding and the graphics files, if any, that you want to use for watermarks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current status of the pipeline: <ul> <li>  <code>Active</code>: The pipeline is processing jobs. </li> <li>  <code>Paused</code>: The pipeline is not currently processing jobs. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the pipeline. We recommend that the name be unique within the AWS account, but uniqueness is not enforced. Constraints: Maximum 40 characters<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) for the pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier for the pipeline. You use this value to identify the pipeline in which you want to perform a variety of operations, such as creating a job or a preset.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Error</i> The Amazon SNS topic that you want to notify when Elastic Transcoder encounters an error condition.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Warning</i> The Amazon SNS topic that you want to notify when Elastic Transcoder encounters a warning condition.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Completed</i> The Amazon SNS topic that you want to notify when Elastic Transcoder has finished processing the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Progressing</i> The Amazon Simple Notification Service (Amazon SNS) topic that you want to notify when Elastic Transcoder has started to process the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Permissions</i> Optional. The <code>Permissions</code> object specifies which users and/or predefined Amazon S3 groups you want to have access to transcoded files and playlists, and the type of access you want them to have. You can grant permissions to a maximum of 30 users and/or predefined Amazon S3 groups. If you include <code>Permissions</code>, Elastic Transcoder grants only the permissions that you specify. It does not grant full permissions to the owner of the role specified by <code>Role</code>. If you want that user to have full control, you must explicitly grant full control to the user.  If you omit <code>Permissions</code>, Elastic Transcoder grants full control over the transcoded files and playlists to the owner of the role specified by <code>Role</code>, and grants no other permissions to any other user or group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageClass</i>  The Amazon S3 storage class, <code>Standard</code> or <code>ReducedRedundancy</code>, that you want Elastic Transcoder to assign to the video files and playlists that it stores in your Amazon S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i>  The Amazon S3 bucket in which you want Elastic Transcoder to save the transcoded files. Specify this value when all of the following are true: <ul> <li> You want to save transcoded files, thumbnails (if any), and playlists (if any) together in one bucket. </li> <li> You do not want to specify the users or groups who have access to the transcoded files, thumbnails, and playlists. </li> <li> You do not want to specify the permissions that Elastic Transcoder grants to the files. </li> <li> You want to associate the transcoded files and thumbnails with the Amazon S3 Standard storage class. </li> </ul> If you want to save transcoded files and playlists in one bucket and thumbnails in another bucket, specify which users can access the transcoded files or the permissions the users have, or change the Amazon S3 storage class, omit OutputBucket and specify values for <code>ContentConfig</code> and <code>ThumbnailConfig</code> instead. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Permissions</i> Optional. The <code>Permissions</code> object specifies which users and/or predefined Amazon S3 groups you want to have access to transcoded files and playlists, and the type of access you want them to have. You can grant permissions to a maximum of 30 users and/or predefined Amazon S3 groups. If you include <code>Permissions</code>, Elastic Transcoder grants only the permissions that you specify. It does not grant full permissions to the owner of the role specified by <code>Role</code>. If you want that user to have full control, you must explicitly grant full control to the user.  If you omit <code>Permissions</code>, Elastic Transcoder grants full control over the transcoded files and playlists to the owner of the role specified by <code>Role</code>, and grants no other permissions to any other user or group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageClass</i>  The Amazon S3 storage class, <code>Standard</code> or <code>ReducedRedundancy</code>, that you want Elastic Transcoder to assign to the video files and playlists that it stores in your Amazon S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i>  The Amazon S3 bucket in which you want Elastic Transcoder to save the transcoded files. Specify this value when all of the following are true: <ul> <li> You want to save transcoded files, thumbnails (if any), and playlists (if any) together in one bucket. </li> <li> You do not want to specify the users or groups who have access to the transcoded files, thumbnails, and playlists. </li> <li> You do not want to specify the permissions that Elastic Transcoder grants to the files. </li> <li> You want to associate the transcoded files and thumbnails with the Amazon S3 Standard storage class. </li> </ul> If you want to save transcoded files and playlists in one bucket and thumbnails in another bucket, specify which users can access the transcoded files or the permissions the users have, or change the Amazon S3 storage class, omit OutputBucket and specify values for <code>ContentConfig</code> and <code>ThumbnailConfig</code> instead. <br/>        """
        pass

    def create_preset(self, Container: str, Name: str, Description: str=None, Video: VideoParameters=None, Audio: AudioParameters=None, Thumbnails: Thumbnails=None) -> CreatePresetResponse:
        """The CreatePreset operation creates a preset with settings that you specify. <important> Elastic Transcoder checks the CreatePreset settings to ensure that they meet Elastic Transcoder requirements and to determine whether they comply with H.264 standards. If your settings are not valid for Elastic Transcoder, Elastic Transcoder returns an HTTP 400 response (<code>ValidationException</code>) and does not create the preset. If the settings are valid for Elastic Transcoder but aren't strictly compliant with the H.264 standard, Elastic Transcoder creates the preset and returns a warning message in the response. This helps you determine whether your settings comply with the H.264 standard while giving you greater flexibility with respect to the video that Elastic Transcoder produces. </important> Elastic Transcoder uses the H.264 video-compression format. For more information, see the International Telecommunication Union publication <i>Recommendation ITU-T H.264: Advanced video coding for generic audiovisual services</i>.

        :param self:
        :param Thumbnails:  A section of the request body that specifies the thumbnail parameters, if any.:<br/>Audio:  A section of the request body that specifies the audio parameters.:<br/>Video:  A section of the request body that specifies the video parameters.:<br/>Container:  The container type for the output file. Valid values include <code>flac</code>, <code>flv</code>, <code>fmp4</code>, <code>gif</code>, <code>mp3</code>, <code>mp4</code>, <code>mpg</code>, <code>mxf</code>, <code>oga</code>, <code>ogg</code>, <code>ts</code>, and <code>webm</code>.<br/>Description:  A description of the preset.<br/>Name:  The name of the preset. We recommend that the name be unique within the AWS account, but uniqueness is not enforced.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Watermarks</i> Settings for the size, location, and opacity of graphics that you want Elastic Transcoder to overlay over videos that are transcoded using this preset. You can specify settings for up to four watermarks. Watermarks appear in the specified size and location, and with the specified opacity for the duration of the transcoded video. Watermarks can be in .png or .jpg format. If you want to display a watermark that is not rectangular, use the .png format, which supports transparency. When you create a job that uses this preset, you specify the .png or .jpg graphics that you want Elastic Transcoder to include in the transcoded videos. You can specify fewer graphics in the job than you specify watermark settings in the preset, which allows you to use the same preset for up to four watermarks that have different dimensions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PaddingPolicy</i> When you set <code>PaddingPolicy</code> to <code>Pad</code>, Elastic Transcoder may add black bars to the top and bottom and/or left and right sides of the output video to make the total size of the output video match the values that you specified for <code>MaxWidth</code> and <code>MaxHeight</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SizingPolicy</i> Specify one of the following values to control scaling of the output video: <ul> <li>  <code>Fit</code>: Elastic Transcoder scales the output video so it matches the value that you specified in either <code>MaxWidth</code> or <code>MaxHeight</code> without exceeding the other value. </li> <li>  <code>Fill</code>: Elastic Transcoder scales the output video so it matches the value that you specified in either <code>MaxWidth</code> or <code>MaxHeight</code> and matches or exceeds the other value. Elastic Transcoder centers the output video and then crops it in the dimension (if any) that exceeds the maximum value. </li> <li>  <code>Stretch</code>: Elastic Transcoder stretches the output video to match the values that you specified for <code>MaxWidth</code> and <code>MaxHeight</code>. If the relative proportions of the input video and the output video are different, the output video will be distorted. </li> <li>  <code>Keep</code>: Elastic Transcoder does not scale the output video. If either dimension of the input video exceeds the values that you specified for <code>MaxWidth</code> and <code>MaxHeight</code>, Elastic Transcoder crops the output video. </li> <li>  <code>ShrinkToFit</code>: Elastic Transcoder scales the output video down so that its dimensions match the values that you specified for at least one of <code>MaxWidth</code> and <code>MaxHeight</code> without exceeding either value. If you specify this option, Elastic Transcoder does not scale the video up. </li> <li>  <code>ShrinkToFill</code>: Elastic Transcoder scales the output video down so that its dimensions match the values that you specified for at least one of <code>MaxWidth</code> and <code>MaxHeight</code> without dropping below either value. If you specify this option, Elastic Transcoder does not scale the video up. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayAspectRatio</i> The value that Elastic Transcoder adds to the metadata in the output file.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxHeight</i> The maximum height of the output video in pixels. If you specify <code>auto</code>, Elastic Transcoder uses 1080 (Full HD) as the default value. If you specify a numeric value, enter an even integer between 96 and 3072.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxWidth</i>  The maximum width of the output video in pixels. If you specify <code>auto</code>, Elastic Transcoder uses 1920 (Full HD) as the default value. If you specify a numeric value, enter an even integer between 128 and 4096. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AspectRatio</i> <important> To better control resolution and aspect ratio of output videos, we recommend that you use the values <code>MaxWidth</code>, <code>MaxHeight</code>, <code>SizingPolicy</code>, <code>PaddingPolicy</code>, and <code>DisplayAspectRatio</code> instead of <code>Resolution</code> and <code>AspectRatio</code>. The two groups of settings are mutually exclusive. Do not use them together. </important> The display aspect ratio of the video in the output file. Valid values include:  <code>auto</code>, <code>1:1</code>, <code>4:3</code>, <code>3:2</code>, <code>16:9</code>  If you specify <code>auto</code>, Elastic Transcoder tries to preserve the aspect ratio of the input file. If you specify an aspect ratio for the output file that differs from aspect ratio of the input file, Elastic Transcoder adds pillarboxing (black bars on the sides) or letterboxing (black bars on the top and bottom) to maintain the aspect ratio of the active region of the video.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Resolution</i> <important> To better control resolution and aspect ratio of output videos, we recommend that you use the values <code>MaxWidth</code>, <code>MaxHeight</code>, <code>SizingPolicy</code>, <code>PaddingPolicy</code>, and <code>DisplayAspectRatio</code> instead of <code>Resolution</code> and <code>AspectRatio</code>. The two groups of settings are mutually exclusive. Do not use them together. </important> The width and height of the video in the output file, in pixels. Valid values are <code>auto</code> and <i>width</i> x <i>height</i>: <ul> <li>  <code>auto</code>: Elastic Transcoder attempts to preserve the width and height of the input file, subject to the following rules. </li> <li>  <code> <i>width</i> x <i>height</i> </code>: The width and height of the output video in pixels. </li> </ul> Note the following about specifying the width and height: <ul> <li> The width must be an even integer between 128 and 4096, inclusive. </li> <li> The height must be an even integer between 96 and 3072, inclusive. </li> <li> If you specify a resolution that is less than the resolution of the input file, Elastic Transcoder rescales the output file to the lower resolution. </li> <li> If you specify a resolution that is greater than the resolution of the input file, Elastic Transcoder rescales the output to the higher resolution. </li> <li> We recommend that you specify a resolution for which the product of width and height is less than or equal to the applicable value in the following list (<i>List - Max width x height value</i>): <ul> <li> 1 - 25344 </li> <li> 1b - 25344 </li> <li> 1.1 - 101376 </li> <li> 1.2 - 101376 </li> <li> 1.3 - 101376 </li> <li> 2 - 101376 </li> <li> 2.1 - 202752 </li> <li> 2.2 - 404720 </li> <li> 3 - 404720 </li> <li> 3.1 - 921600 </li> <li> 3.2 - 1310720 </li> <li> 4 - 2097152 </li> <li> 4.1 - 2097152 </li> </ul> </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxFrameRate</i> If you specify <code>auto</code> for <code>FrameRate</code>, Elastic Transcoder uses the frame rate of the input video for the frame rate of the output video. Specify the maximum frame rate that you want Elastic Transcoder to use when the frame rate of the input video is greater than the desired maximum frame rate of the output video. Valid values include: <code>10</code>, <code>15</code>, <code>23.97</code>, <code>24</code>, <code>25</code>, <code>29.97</code>, <code>30</code>, <code>60</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameRate</i> The frames per second for the video stream in the output file. Valid values include:  <code>auto</code>, <code>10</code>, <code>15</code>, <code>23.97</code>, <code>24</code>, <code>25</code>, <code>29.97</code>, <code>30</code>, <code>60</code>  If you specify <code>auto</code>, Elastic Transcoder uses the detected frame rate of the input source. If you specify a frame rate, we recommend that you perform the following calculation:  <code>Frame rate = maximum recommended decoding speed in luma samples/second / (width in pixels * height in pixels)</code>  where: <ul> <li>  <i>width in pixels</i> and <i>height in pixels</i> represent the Resolution of the output video. </li> <li>  <i>maximum recommended decoding speed in Luma samples/second</i> is less than or equal to the maximum value listed in the following table, based on the value that you specified for Level. </li> </ul> The maximum recommended decoding speed in Luma samples/second for each level is described in the following list (<i>Level - Decoding speed</i>): <ul> <li> 1 - 380160 </li> <li> 1b - 380160 </li> <li> 1.1 - 76800 </li> <li> 1.2 - 1536000 </li> <li> 1.3 - 3041280 </li> <li> 2 - 3041280 </li> <li> 2.1 - 5068800 </li> <li> 2.2 - 5184000 </li> <li> 3 - 10368000 </li> <li> 3.1 - 27648000 </li> <li> 3.2 - 55296000 </li> <li> 4 - 62914560 </li> <li> 4.1 - 62914560 </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BitRate</i> The bit rate of the video stream in the output file, in kilobits/second. Valid values depend on the values of <code>Level</code> and <code>Profile</code>. If you specify <code>auto</code>, Elastic Transcoder uses the detected bit rate of the input source. If you specify a value other than <code>auto</code>, we recommend that you specify a value less than or equal to the maximum H.264-compliant value listed for your level and profile:  <i>Level - Maximum video bit rate in kilobits/second (baseline and main Profile) : maximum video bit rate in kilobits/second (high Profile)</i>  <ul> <li> 1 - 64 : 80 </li> <li> 1b - 128 : 160 </li> <li> 1.1 - 192 : 240 </li> <li> 1.2 - 384 : 480 </li> <li> 1.3 - 768 : 960 </li> <li> 2 - 2000 : 2500 </li> <li> 3 - 10000 : 12500 </li> <li> 3.1 - 14000 : 17500 </li> <li> 3.2 - 20000 : 25000 </li> <li> 4 - 20000 : 25000 </li> <li> 4.1 - 50000 : 62500 </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FixedGOP</i> Applicable only when the value of Video:Codec is one of <code>H.264</code>, <code>MPEG2</code>, or <code>VP8</code>. Whether to use a fixed value for <code>FixedGOP</code>. Valid values are <code>true</code> and <code>false</code>: <ul> <li>  <code>true</code>: Elastic Transcoder uses the value of <code>KeyframesMaxDist</code> for the distance between key frames (the number of frames in a group of pictures, or GOP). </li> <li>  <code>false</code>: The distance between key frames can vary. </li> </ul> <important>  <code>FixedGOP</code> must be set to <code>true</code> for <code>fmp4</code> containers. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyframesMaxDist</i> Applicable only when the value of Video:Codec is one of <code>H.264</code>, <code>MPEG2</code>, or <code>VP8</code>. The maximum number of frames between key frames. Key frames are fully encoded frames; the frames between key frames are encoded based, in part, on the content of the key frames. The value is an integer formatted as a string; valid values are between 1 (every frame is a key frame) and 100000, inclusive. A higher value results in higher compression but may also discernibly decrease video quality. For <code>Smooth</code> outputs, the <code>FrameRate</code> must have a constant ratio to the <code>KeyframesMaxDist</code>. This allows <code>Smooth</code> playlists to switch between different quality levels while the file is being played. For example, an input file can have a <code>FrameRate</code> of 30 with a <code>KeyframesMaxDist</code> of 90. The output file then needs to have a ratio of 1:3. Valid outputs would have <code>FrameRate</code> of 30, 25, and 10, and <code>KeyframesMaxDist</code> of 90, 75, and 30, respectively. Alternately, this can be achieved by setting <code>FrameRate</code> to auto and having the same values for <code>MaxFrameRate</code> and <code>KeyframesMaxDist</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CodecOptions</i>  <b>Profile (H.264/VP8/VP9 Only)</b>  The H.264 profile that you want to use for the output file. Elastic Transcoder supports the following profiles: <ul> <li>  <code>baseline</code>: The profile most commonly used for videoconferencing and for mobile applications. </li> <li>  <code>main</code>: The profile used for standard-definition digital TV broadcasts. </li> <li>  <code>high</code>: The profile used for high-definition digital TV broadcasts and for Blu-ray discs. </li> </ul>  <b>Level (H.264 Only)</b>  The H.264 level that you want to use for the output file. Elastic Transcoder supports the following levels:  <code>1</code>, <code>1b</code>, <code>1.1</code>, <code>1.2</code>, <code>1.3</code>, <code>2</code>, <code>2.1</code>, <code>2.2</code>, <code>3</code>, <code>3.1</code>, <code>3.2</code>, <code>4</code>, <code>4.1</code>   <b>MaxReferenceFrames (H.264 Only)</b>  Applicable only when the value of Video:Codec is H.264. The maximum number of previously decoded frames to use as a reference for decoding future frames. Valid values are integers 0 through 16, but we recommend that you not use a value greater than the following:  <code>Min(Floor(Maximum decoded picture buffer in macroblocks * 256 / (Width in pixels * Height in pixels)), 16)</code>  where <i>Width in pixels</i> and <i>Height in pixels</i> represent either MaxWidth and MaxHeight, or Resolution. <i>Maximum decoded picture buffer in macroblocks</i> depends on the value of the <code>Level</code> object. See the list below. (A macroblock is a block of pixels measuring 16x16.)  <ul> <li> 1 - 396 </li> <li> 1b - 396 </li> <li> 1.1 - 900 </li> <li> 1.2 - 2376 </li> <li> 1.3 - 2376 </li> <li> 2 - 2376 </li> <li> 2.1 - 4752 </li> <li> 2.2 - 8100 </li> <li> 3 - 8100 </li> <li> 3.1 - 18000 </li> <li> 3.2 - 20480 </li> <li> 4 - 32768 </li> <li> 4.1 - 32768 </li> </ul>  <b>MaxBitRate (Optional, H.264/MPEG2/VP8/VP9 only)</b>  The maximum number of bits per second in a video buffer; the size of the buffer is specified by <code>BufferSize</code>. Specify a value between 16 and 62,500. You can reduce the bandwidth required to stream a video by reducing the maximum bit rate, but this also reduces the quality of the video.  <b>BufferSize (Optional, H.264/MPEG2/VP8/VP9 only)</b>  The maximum number of bits in any x seconds of the output video. This window is commonly 10 seconds, the standard segment duration when you're using FMP4 or MPEG-TS for the container type of the output video. Specify an integer greater than 0. If you specify <code>MaxBitRate</code> and omit <code>BufferSize</code>, Elastic Transcoder sets <code>BufferSize</code> to 10 times the value of <code>MaxBitRate</code>.  <b>InterlacedMode (Optional, H.264/MPEG2 Only)</b>  The interlace mode for the output video. Interlaced video is used to double the perceived frame rate for a video by interlacing two fields (one field on every other line, the other field on the other lines) so that the human eye registers multiple pictures per frame. Interlacing reduces the bandwidth required for transmitting a video, but can result in blurred images and flickering. Valid values include <code>Progressive</code> (no interlacing, top to bottom), <code>TopFirst</code> (top field first), <code>BottomFirst</code> (bottom field first), and <code>Auto</code>. If <code>InterlaceMode</code> is not specified, Elastic Transcoder uses <code>Progressive</code> for the output. If <code>Auto</code> is specified, Elastic Transcoder interlaces the output.  <b>ColorSpaceConversionMode (Optional, H.264/MPEG2 Only)</b>  The color space conversion Elastic Transcoder applies to the output video. Color spaces are the algorithms used by the computer to store information about how to render color. <code>Bt.601</code> is the standard for standard definition video, while <code>Bt.709</code> is the standard for high definition video. Valid values include <code>None</code>, <code>Bt709toBt601</code>, <code>Bt601toBt709</code>, and <code>Auto</code>. If you chose <code>Auto</code> for <code>ColorSpaceConversionMode</code> and your output is interlaced, your frame rate is one of <code>23.97</code>, <code>24</code>, <code>25</code>, <code>29.97</code>, <code>50</code>, or <code>60</code>, your <code>SegmentDuration</code> is null, and you are using one of the resolution changes from the list below, Elastic Transcoder applies the following color space conversions: <ul> <li>  <i>Standard to HD, 720x480 to 1920x1080</i> - Elastic Transcoder applies <code>Bt601ToBt709</code>  </li> <li>  <i>Standard to HD, 720x576 to 1920x1080</i> - Elastic Transcoder applies <code>Bt601ToBt709</code>  </li> <li>  <i>HD to Standard, 1920x1080 to 720x480</i> - Elastic Transcoder applies <code>Bt709ToBt601</code>  </li> <li>  <i>HD to Standard, 1920x1080 to 720x576</i> - Elastic Transcoder applies <code>Bt709ToBt601</code>  </li> </ul> <note> Elastic Transcoder may change the behavior of the <code>ColorspaceConversionMode</code> <code>Auto</code> mode in the future. All outputs in a playlist must use the same <code>ColorSpaceConversionMode</code>. </note> If you do not specify a <code>ColorSpaceConversionMode</code>, Elastic Transcoder does not change the color space of a file. If you are unsure what <code>ColorSpaceConversionMode</code> was applied to your output file, you can check the <code>AppliedColorSpaceConversion</code> parameter included in your job response. If your job does not have an <code>AppliedColorSpaceConversion</code> in its response, no <code>ColorSpaceConversionMode</code> was applied.  <b>ChromaSubsampling</b>  The sampling pattern for the chroma (color) channels of the output video. Valid values include <code>yuv420p</code> and <code>yuv422p</code>.  <code>yuv420p</code> samples the chroma information of every other horizontal and every other vertical line, <code>yuv422p</code> samples the color information of every horizontal line and every other vertical line.  <b>LoopCount (Gif Only)</b>  The number of times you want the output gif to loop. Valid values include <code>Infinite</code> and integers between <code>0</code> and <code>100</code>, inclusive.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Codec</i> The video codec for the output file. Valid values include <code>gif</code>, <code>H.264</code>, <code>mpeg2</code>, <code>vp8</code>, and <code>vp9</code>. You can only specify <code>vp8</code> and <code>vp9</code> when the container type is <code>webm</code>, <code>gif</code> when the container type is <code>gif</code>, and <code>mpeg2</code> when the container type is <code>mpg</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CodecOptions</i> If you specified <code>AAC</code> for <code>Audio:Codec</code>, this is the <code>AAC</code> compression profile to use. Valid values include:  <code>auto</code>, <code>AAC-LC</code>, <code>HE-AAC</code>, <code>HE-AACv2</code>  If you specify <code>auto</code>, Elastic Transcoder chooses a profile based on the bit rate of the output file.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CodecOptions</b>: If you specified <code>AAC</code> for <code>Audio:Codec</code>, this is the <code>AAC</code> compression profile to use. Valid values include:  <code>auto</code>, <code>AAC-LC</code>, <code>HE-AAC</code>, <code>HE-AACv2</code>  If you specify <code>auto</code>, Elastic Transcoder chooses a profile based on the bit rate of the output file.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AudioPackingMode</i> The method of organizing audio channels and tracks. Use <code>Audio:Channels</code> to specify the number of channels in your output, and <code>Audio:AudioPackingMode</code> to specify the number of tracks and their relation to the channels. If you do not specify an <code>Audio:AudioPackingMode</code>, Elastic Transcoder uses <code>SingleTrack</code>. The following values are valid:  <code>SingleTrack</code>, <code>OneChannelPerTrack</code>, and <code>OneChannelPerTrackWithMosTo8Tracks</code>  When you specify <code>SingleTrack</code>, Elastic Transcoder creates a single track for your output. The track can have up to eight channels. Use <code>SingleTrack</code> for all non-<code>mxf</code> containers. The outputs of <code>SingleTrack</code> for a specific channel value and inputs are as follows: <ul> <li>  <code>0</code> <b> channels with any input:</b> Audio omitted from the output </li> <li>  <code>1, 2, or auto </code> <b>channels with no audio input:</b> Audio omitted from the output </li> <li>  <code>1 </code> <b>channel with any input with audio:</b> One track with one channel, downmixed if necessary </li> <li>  <code>2 </code> <b>channels with one track with one channel:</b> One track with two identical channels </li> <li>  <code>2 or auto </code> <b>channels with two tracks with one channel each:</b> One track with two channels </li> <li>  <code>2 or auto </code> <b>channels with one track with two channels:</b> One track with two channels </li> <li>  <code>2 </code> <b>channels with one track with multiple channels:</b> One track with two channels </li> <li>  <code>auto </code> <b>channels with one track with one channel:</b> One track with one channel </li> <li>  <code>auto </code> <b>channels with one track with multiple channels:</b> One track with multiple channels </li> </ul> When you specify <code>OneChannelPerTrack</code>, Elastic Transcoder creates a new track for every channel in your output. Your output can have up to eight single-channel tracks. The outputs of <code>OneChannelPerTrack</code> for a specific channel value and inputs are as follows: <ul> <li>  <code>0 </code> <b>channels with any input:</b> Audio omitted from the output </li> <li>  <code>1, 2, or auto </code> <b>channels with no audio input:</b> Audio omitted from the output </li> <li>  <code>1 </code> <b>channel with any input with audio:</b> One track with one channel, downmixed if necessary </li> <li>  <code>2 </code> <b>channels with one track with one channel:</b> Two tracks with one identical channel each </li> <li>  <code>2 or auto </code> <b>channels with two tracks with one channel each:</b> Two tracks with one channel each </li> <li>  <code>2 or auto </code> <b>channels with one track with two channels:</b> Two tracks with one channel each </li> <li>  <code>2 </code> <b>channels with one track with multiple channels:</b> Two tracks with one channel each </li> <li>  <code>auto </code> <b>channels with one track with one channel:</b> One track with one channel </li> <li>  <code>auto </code> <b>channels with one track with multiple channels:</b> Up to eight tracks with one channel each </li> </ul> When you specify <code>OneChannelPerTrackWithMosTo8Tracks</code>, Elastic Transcoder creates eight single-channel tracks for your output. All tracks that do not contain audio data from an input channel are MOS, or Mit Out Sound, tracks. The outputs of <code>OneChannelPerTrackWithMosTo8Tracks</code> for a specific channel value and inputs are as follows: <ul> <li>  <code>0 </code> <b>channels with any input:</b> Audio omitted from the output </li> <li>  <code>1, 2, or auto </code> <b>channels with no audio input:</b> Audio omitted from the output </li> <li>  <code>1 </code> <b>channel with any input with audio:</b> One track with one channel, downmixed if necessary, plus six MOS tracks </li> <li>  <code>2 </code> <b>channels with one track with one channel:</b> Two tracks with one identical channel each, plus six MOS tracks </li> <li>  <code>2 or auto </code> <b>channels with two tracks with one channel each:</b> Two tracks with one channel each, plus six MOS tracks </li> <li>  <code>2 or auto </code> <b>channels with one track with two channels:</b> Two tracks with one channel each, plus six MOS tracks </li> <li>  <code>2 </code> <b>channels with one track with multiple channels:</b> Two tracks with one channel each, plus six MOS tracks </li> <li>  <code>auto </code> <b>channels with one track with one channel:</b> One track with one channel, plus seven MOS tracks </li> <li>  <code>auto </code> <b>channels with one track with multiple channels:</b> Up to eight tracks with one channel each, plus MOS tracks until there are eight tracks in all </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Channels</i> The number of audio channels in the output file. The following values are valid:  <code>auto</code>, <code>0</code>, <code>1</code>, <code>2</code>  One channel carries the information played by a single speaker. For example, a stereo track with two channels sends one channel to the left speaker, and the other channel to the right speaker. The output channels are organized into tracks. If you want Elastic Transcoder to automatically detect the number of audio channels in the input file and use that value for the output file, select <code>auto</code>. The output of a specific channel value and inputs are as follows: <ul> <li>  <code>auto</code> <b> channel specified, with any input:</b> Pass through up to eight input channels. </li> <li>  <code>0</code> <b> channels specified, with any input:</b> Audio omitted from the output. </li> <li>  <code>1</code> <b> channel specified, with at least one input channel:</b> Mono sound. </li> <li>  <code>2</code> <b> channels specified, with any input:</b> Two identical mono channels or stereo. For more information about tracks, see <code>Audio:AudioPackingMode.</code>  </li> </ul>  For more information about how Elastic Transcoder organizes channels and tracks, see <code>Audio:AudioPackingMode</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BitRate</i> The bit rate of the audio stream in the output file, in kilobits/second. Enter an integer between 64 and 320, inclusive.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SampleRate</i> The sample rate of the audio stream in the output file, in Hertz. Valid values include:  <code>auto</code>, <code>22050</code>, <code>32000</code>, <code>44100</code>, <code>48000</code>, <code>96000</code>  If you specify <code>auto</code>, Elastic Transcoder automatically detects the sample rate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Codec</i> The audio codec for the output file. Valid values include <code>aac</code>, <code>flac</code>, <code>mp2</code>, <code>mp3</code>, <code>pcm</code>, and <code>vorbis</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Signed</i> You can only choose whether an audio sample is signed when you specify <code>pcm</code> for the value of Audio:Codec. Whether audio samples are represented with negative and positive numbers (signed) or only positive numbers (unsigned). The supported value is <code>Signed</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BitOrder</i> You can only choose an audio bit order when you specify <code>pcm</code> for the value of Audio:Codec. The order the bits of a PCM sample are stored in. The supported value is <code>LittleEndian</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BitDepth</i> You can only choose an audio bit depth when you specify <code>flac</code> or <code>pcm</code> for the value of Audio:Codec. The bit depth of a sample is how many bits of information are included in the audio samples. The higher the bit depth, the better the audio, but the larger the file. Valid values are <code>16</code> and <code>24</code>. The most common bit depth is <code>24</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Profile</i> You can only choose an audio profile when you specify AAC for the value of Audio:Codec. Specify the AAC profile for the output file. Elastic Transcoder supports the following profiles: <ul> <li>  <code>auto</code>: If you specify <code>auto</code>, Elastic Transcoder selects the profile based on the bit rate selected for the output file. </li> <li>  <code>AAC-LC</code>: The most common AAC profile. Use for bit rates larger than 64 kbps. </li> <li>  <code>HE-AAC</code>: Not supported on some older players and devices. Use for bit rates between 40 and 80 kbps. </li> <li>  <code>HE-AACv2</code>: Not supported on some players and devices. Use for bit rates less than 48 kbps. </li> </ul> All outputs in a <code>Smooth</code> playlist must have the same value for <code>Profile</code>. <note> If you created any presets before AAC profiles were added, Elastic Transcoder automatically updated your presets to use AAC-LC. You can change the value as required. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PaddingPolicy</i> When you set <code>PaddingPolicy</code> to <code>Pad</code>, Elastic Transcoder may add black bars to the top and bottom and/or left and right sides of thumbnails to make the total size of the thumbnails match the values that you specified for thumbnail <code>MaxWidth</code> and <code>MaxHeight</code> settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SizingPolicy</i> Specify one of the following values to control scaling of thumbnails: <ul> <li>  <code>Fit</code>: Elastic Transcoder scales thumbnails so they match the value that you specified in thumbnail MaxWidth or MaxHeight settings without exceeding the other value.  </li> <li>  <code>Fill</code>: Elastic Transcoder scales thumbnails so they match the value that you specified in thumbnail <code>MaxWidth</code> or <code>MaxHeight</code> settings and matches or exceeds the other value. Elastic Transcoder centers the image in thumbnails and then crops in the dimension (if any) that exceeds the maximum value. </li> <li>  <code>Stretch</code>: Elastic Transcoder stretches thumbnails to match the values that you specified for thumbnail <code>MaxWidth</code> and <code>MaxHeight</code> settings. If the relative proportions of the input video and thumbnails are different, the thumbnails will be distorted. </li> <li>  <code>Keep</code>: Elastic Transcoder does not scale thumbnails. If either dimension of the input video exceeds the values that you specified for thumbnail <code>MaxWidth</code> and <code>MaxHeight</code> settings, Elastic Transcoder crops the thumbnails. </li> <li>  <code>ShrinkToFit</code>: Elastic Transcoder scales thumbnails down so that their dimensions match the values that you specified for at least one of thumbnail <code>MaxWidth</code> and <code>MaxHeight</code> without exceeding either value. If you specify this option, Elastic Transcoder does not scale thumbnails up. </li> <li>  <code>ShrinkToFill</code>: Elastic Transcoder scales thumbnails down so that their dimensions match the values that you specified for at least one of <code>MaxWidth</code> and <code>MaxHeight</code> without dropping below either value. If you specify this option, Elastic Transcoder does not scale thumbnails up. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxHeight</i> The maximum height of thumbnails in pixels. If you specify auto, Elastic Transcoder uses 1080 (Full HD) as the default value. If you specify a numeric value, enter an even integer between 32 and 3072.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxWidth</i> The maximum width of thumbnails in pixels. If you specify auto, Elastic Transcoder uses 1920 (Full HD) as the default value. If you specify a numeric value, enter an even integer between 32 and 4096.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AspectRatio</i> <important> To better control resolution and aspect ratio of thumbnails, we recommend that you use the values <code>MaxWidth</code>, <code>MaxHeight</code>, <code>SizingPolicy</code>, and <code>PaddingPolicy</code> instead of <code>Resolution</code> and <code>AspectRatio</code>. The two groups of settings are mutually exclusive. Do not use them together. </important> The aspect ratio of thumbnails. Valid values include:  <code>auto</code>, <code>1:1</code>, <code>4:3</code>, <code>3:2</code>, <code>16:9</code>  If you specify <code>auto</code>, Elastic Transcoder tries to preserve the aspect ratio of the video in the output file.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Resolution</i> <important> To better control resolution and aspect ratio of thumbnails, we recommend that you use the values <code>MaxWidth</code>, <code>MaxHeight</code>, <code>SizingPolicy</code>, and <code>PaddingPolicy</code> instead of <code>Resolution</code> and <code>AspectRatio</code>. The two groups of settings are mutually exclusive. Do not use them together. </important> The width and height of thumbnail files in pixels. Specify a value in the format <code> <i>width</i> </code> x <code> <i>height</i> </code> where both values are even integers. The values cannot exceed the width and height that you specified in the <code>Video:Resolution</code> object.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Interval</i> The approximate number of seconds between thumbnails. Specify an integer value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> The format of thumbnails, if any. Valid values are <code>jpg</code> and <code>png</code>.  You specify whether you want Elastic Transcoder to create thumbnails when you create a job.<br/>
        :return: Warning:  If the preset settings don't comply with the standards for the video codec but Elastic Transcoder created the preset, this message explains the reason the preset settings don't meet the standard. Elastic Transcoder created the preset because the settings might produce acceptable output.<br/>Preset:  A section of the response body that provides information about the preset that is created.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> Whether the preset is a default preset provided by Elastic Transcoder (<code>System</code>) or a preset that you have defined (<code>Custom</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Thumbnails</i> A section of the response body that provides information about the thumbnail preset values, if any.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Thumbnails</b>: A section of the response body that provides information about the thumbnail preset values, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Video</i> A section of the response body that provides information about the video preset values.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Video</b>: A section of the response body that provides information about the video preset values.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Audio</i> A section of the response body that provides information about the audio preset values.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Audio</b>: A section of the response body that provides information about the audio preset values.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Container</i> The container type for the output file. Valid values include <code>flac</code>, <code>flv</code>, <code>fmp4</code>, <code>gif</code>, <code>mp3</code>, <code>mp4</code>, <code>mpg</code>, <code>mxf</code>, <code>oga</code>, <code>ogg</code>, <code>ts</code>, and <code>webm</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of the preset.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the preset.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) for the preset.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> Identifier for the new preset. You use this value to get settings for the preset or to delete it.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AudioPackingMode</i> The method of organizing audio channels and tracks. Use <code>Audio:Channels</code> to specify the number of channels in your output, and <code>Audio:AudioPackingMode</code> to specify the number of tracks and their relation to the channels. If you do not specify an <code>Audio:AudioPackingMode</code>, Elastic Transcoder uses <code>SingleTrack</code>. The following values are valid:  <code>SingleTrack</code>, <code>OneChannelPerTrack</code>, and <code>OneChannelPerTrackWithMosTo8Tracks</code>  When you specify <code>SingleTrack</code>, Elastic Transcoder creates a single track for your output. The track can have up to eight channels. Use <code>SingleTrack</code> for all non-<code>mxf</code> containers. The outputs of <code>SingleTrack</code> for a specific channel value and inputs are as follows: <ul> <li>  <code>0</code> <b> channels with any input:</b> Audio omitted from the output </li> <li>  <code>1, 2, or auto </code> <b>channels with no audio input:</b> Audio omitted from the output </li> <li>  <code>1 </code> <b>channel with any input with audio:</b> One track with one channel, downmixed if necessary </li> <li>  <code>2 </code> <b>channels with one track with one channel:</b> One track with two identical channels </li> <li>  <code>2 or auto </code> <b>channels with two tracks with one channel each:</b> One track with two channels </li> <li>  <code>2 or auto </code> <b>channels with one track with two channels:</b> One track with two channels </li> <li>  <code>2 </code> <b>channels with one track with multiple channels:</b> One track with two channels </li> <li>  <code>auto </code> <b>channels with one track with one channel:</b> One track with one channel </li> <li>  <code>auto </code> <b>channels with one track with multiple channels:</b> One track with multiple channels </li> </ul> When you specify <code>OneChannelPerTrack</code>, Elastic Transcoder creates a new track for every channel in your output. Your output can have up to eight single-channel tracks. The outputs of <code>OneChannelPerTrack</code> for a specific channel value and inputs are as follows: <ul> <li>  <code>0 </code> <b>channels with any input:</b> Audio omitted from the output </li> <li>  <code>1, 2, or auto </code> <b>channels with no audio input:</b> Audio omitted from the output </li> <li>  <code>1 </code> <b>channel with any input with audio:</b> One track with one channel, downmixed if necessary </li> <li>  <code>2 </code> <b>channels with one track with one channel:</b> Two tracks with one identical channel each </li> <li>  <code>2 or auto </code> <b>channels with two tracks with one channel each:</b> Two tracks with one channel each </li> <li>  <code>2 or auto </code> <b>channels with one track with two channels:</b> Two tracks with one channel each </li> <li>  <code>2 </code> <b>channels with one track with multiple channels:</b> Two tracks with one channel each </li> <li>  <code>auto </code> <b>channels with one track with one channel:</b> One track with one channel </li> <li>  <code>auto </code> <b>channels with one track with multiple channels:</b> Up to eight tracks with one channel each </li> </ul> When you specify <code>OneChannelPerTrackWithMosTo8Tracks</code>, Elastic Transcoder creates eight single-channel tracks for your output. All tracks that do not contain audio data from an input channel are MOS, or Mit Out Sound, tracks. The outputs of <code>OneChannelPerTrackWithMosTo8Tracks</code> for a specific channel value and inputs are as follows: <ul> <li>  <code>0 </code> <b>channels with any input:</b> Audio omitted from the output </li> <li>  <code>1, 2, or auto </code> <b>channels with no audio input:</b> Audio omitted from the output </li> <li>  <code>1 </code> <b>channel with any input with audio:</b> One track with one channel, downmixed if necessary, plus six MOS tracks </li> <li>  <code>2 </code> <b>channels with one track with one channel:</b> Two tracks with one identical channel each, plus six MOS tracks </li> <li>  <code>2 or auto </code> <b>channels with two tracks with one channel each:</b> Two tracks with one channel each, plus six MOS tracks </li> <li>  <code>2 or auto </code> <b>channels with one track with two channels:</b> Two tracks with one channel each, plus six MOS tracks </li> <li>  <code>2 </code> <b>channels with one track with multiple channels:</b> Two tracks with one channel each, plus six MOS tracks </li> <li>  <code>auto </code> <b>channels with one track with one channel:</b> One track with one channel, plus seven MOS tracks </li> <li>  <code>auto </code> <b>channels with one track with multiple channels:</b> Up to eight tracks with one channel each, plus MOS tracks until there are eight tracks in all </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Channels</i> The number of audio channels in the output file. The following values are valid:  <code>auto</code>, <code>0</code>, <code>1</code>, <code>2</code>  One channel carries the information played by a single speaker. For example, a stereo track with two channels sends one channel to the left speaker, and the other channel to the right speaker. The output channels are organized into tracks. If you want Elastic Transcoder to automatically detect the number of audio channels in the input file and use that value for the output file, select <code>auto</code>. The output of a specific channel value and inputs are as follows: <ul> <li>  <code>auto</code> <b> channel specified, with any input:</b> Pass through up to eight input channels. </li> <li>  <code>0</code> <b> channels specified, with any input:</b> Audio omitted from the output. </li> <li>  <code>1</code> <b> channel specified, with at least one input channel:</b> Mono sound. </li> <li>  <code>2</code> <b> channels specified, with any input:</b> Two identical mono channels or stereo. For more information about tracks, see <code>Audio:AudioPackingMode.</code>  </li> </ul>  For more information about how Elastic Transcoder organizes channels and tracks, see <code>Audio:AudioPackingMode</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BitRate</i> The bit rate of the audio stream in the output file, in kilobits/second. Enter an integer between 64 and 320, inclusive.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SampleRate</i> The sample rate of the audio stream in the output file, in Hertz. Valid values include:  <code>auto</code>, <code>22050</code>, <code>32000</code>, <code>44100</code>, <code>48000</code>, <code>96000</code>  If you specify <code>auto</code>, Elastic Transcoder automatically detects the sample rate.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Codec</i> The audio codec for the output file. Valid values include <code>aac</code>, <code>flac</code>, <code>mp2</code>, <code>mp3</code>, <code>pcm</code>, and <code>vorbis</code>.<br/>If you specified <code>AAC</code> for <code>Audio:Codec</code>, this is the <code>AAC</code> compression profile to use. Valid values include:  <code>auto</code>, <code>AAC-LC</code>, <code>HE-AAC</code>, <code>HE-AACv2</code>  If you specify <code>auto</code>, Elastic Transcoder chooses a profile based on the bit rate of the output file.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CodecOptions</b>: If you specified <code>AAC</code> for <code>Audio:Codec</code>, this is the <code>AAC</code> compression profile to use. Valid values include:  <code>auto</code>, <code>AAC-LC</code>, <code>HE-AAC</code>, <code>HE-AACv2</code>  If you specify <code>auto</code>, Elastic Transcoder chooses a profile based on the bit rate of the output file.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Watermarks</i> Settings for the size, location, and opacity of graphics that you want Elastic Transcoder to overlay over videos that are transcoded using this preset. You can specify settings for up to four watermarks. Watermarks appear in the specified size and location, and with the specified opacity for the duration of the transcoded video. Watermarks can be in .png or .jpg format. If you want to display a watermark that is not rectangular, use the .png format, which supports transparency. When you create a job that uses this preset, you specify the .png or .jpg graphics that you want Elastic Transcoder to include in the transcoded videos. You can specify fewer graphics in the job than you specify watermark settings in the preset, which allows you to use the same preset for up to four watermarks that have different dimensions.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PaddingPolicy</i> When you set <code>PaddingPolicy</code> to <code>Pad</code>, Elastic Transcoder may add black bars to the top and bottom and/or left and right sides of the output video to make the total size of the output video match the values that you specified for <code>MaxWidth</code> and <code>MaxHeight</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SizingPolicy</i> Specify one of the following values to control scaling of the output video: <ul> <li>  <code>Fit</code>: Elastic Transcoder scales the output video so it matches the value that you specified in either <code>MaxWidth</code> or <code>MaxHeight</code> without exceeding the other value. </li> <li>  <code>Fill</code>: Elastic Transcoder scales the output video so it matches the value that you specified in either <code>MaxWidth</code> or <code>MaxHeight</code> and matches or exceeds the other value. Elastic Transcoder centers the output video and then crops it in the dimension (if any) that exceeds the maximum value. </li> <li>  <code>Stretch</code>: Elastic Transcoder stretches the output video to match the values that you specified for <code>MaxWidth</code> and <code>MaxHeight</code>. If the relative proportions of the input video and the output video are different, the output video will be distorted. </li> <li>  <code>Keep</code>: Elastic Transcoder does not scale the output video. If either dimension of the input video exceeds the values that you specified for <code>MaxWidth</code> and <code>MaxHeight</code>, Elastic Transcoder crops the output video. </li> <li>  <code>ShrinkToFit</code>: Elastic Transcoder scales the output video down so that its dimensions match the values that you specified for at least one of <code>MaxWidth</code> and <code>MaxHeight</code> without exceeding either value. If you specify this option, Elastic Transcoder does not scale the video up. </li> <li>  <code>ShrinkToFill</code>: Elastic Transcoder scales the output video down so that its dimensions match the values that you specified for at least one of <code>MaxWidth</code> and <code>MaxHeight</code> without dropping below either value. If you specify this option, Elastic Transcoder does not scale the video up. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayAspectRatio</i> The value that Elastic Transcoder adds to the metadata in the output file.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxHeight</i> The maximum height of the output video in pixels. If you specify <code>auto</code>, Elastic Transcoder uses 1080 (Full HD) as the default value. If you specify a numeric value, enter an even integer between 96 and 3072.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxWidth</i>  The maximum width of the output video in pixels. If you specify <code>auto</code>, Elastic Transcoder uses 1920 (Full HD) as the default value. If you specify a numeric value, enter an even integer between 128 and 4096. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AspectRatio</i> <important> To better control resolution and aspect ratio of output videos, we recommend that you use the values <code>MaxWidth</code>, <code>MaxHeight</code>, <code>SizingPolicy</code>, <code>PaddingPolicy</code>, and <code>DisplayAspectRatio</code> instead of <code>Resolution</code> and <code>AspectRatio</code>. The two groups of settings are mutually exclusive. Do not use them together. </important> The display aspect ratio of the video in the output file. Valid values include:  <code>auto</code>, <code>1:1</code>, <code>4:3</code>, <code>3:2</code>, <code>16:9</code>  If you specify <code>auto</code>, Elastic Transcoder tries to preserve the aspect ratio of the input file. If you specify an aspect ratio for the output file that differs from aspect ratio of the input file, Elastic Transcoder adds pillarboxing (black bars on the sides) or letterboxing (black bars on the top and bottom) to maintain the aspect ratio of the active region of the video.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Resolution</i> <important> To better control resolution and aspect ratio of output videos, we recommend that you use the values <code>MaxWidth</code>, <code>MaxHeight</code>, <code>SizingPolicy</code>, <code>PaddingPolicy</code>, and <code>DisplayAspectRatio</code> instead of <code>Resolution</code> and <code>AspectRatio</code>. The two groups of settings are mutually exclusive. Do not use them together. </important> The width and height of the video in the output file, in pixels. Valid values are <code>auto</code> and <i>width</i> x <i>height</i>: <ul> <li>  <code>auto</code>: Elastic Transcoder attempts to preserve the width and height of the input file, subject to the following rules. </li> <li>  <code> <i>width</i> x <i>height</i> </code>: The width and height of the output video in pixels. </li> </ul> Note the following about specifying the width and height: <ul> <li> The width must be an even integer between 128 and 4096, inclusive. </li> <li> The height must be an even integer between 96 and 3072, inclusive. </li> <li> If you specify a resolution that is less than the resolution of the input file, Elastic Transcoder rescales the output file to the lower resolution. </li> <li> If you specify a resolution that is greater than the resolution of the input file, Elastic Transcoder rescales the output to the higher resolution. </li> <li> We recommend that you specify a resolution for which the product of width and height is less than or equal to the applicable value in the following list (<i>List - Max width x height value</i>): <ul> <li> 1 - 25344 </li> <li> 1b - 25344 </li> <li> 1.1 - 101376 </li> <li> 1.2 - 101376 </li> <li> 1.3 - 101376 </li> <li> 2 - 101376 </li> <li> 2.1 - 202752 </li> <li> 2.2 - 404720 </li> <li> 3 - 404720 </li> <li> 3.1 - 921600 </li> <li> 3.2 - 1310720 </li> <li> 4 - 2097152 </li> <li> 4.1 - 2097152 </li> </ul> </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxFrameRate</i> If you specify <code>auto</code> for <code>FrameRate</code>, Elastic Transcoder uses the frame rate of the input video for the frame rate of the output video. Specify the maximum frame rate that you want Elastic Transcoder to use when the frame rate of the input video is greater than the desired maximum frame rate of the output video. Valid values include: <code>10</code>, <code>15</code>, <code>23.97</code>, <code>24</code>, <code>25</code>, <code>29.97</code>, <code>30</code>, <code>60</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameRate</i> The frames per second for the video stream in the output file. Valid values include:  <code>auto</code>, <code>10</code>, <code>15</code>, <code>23.97</code>, <code>24</code>, <code>25</code>, <code>29.97</code>, <code>30</code>, <code>60</code>  If you specify <code>auto</code>, Elastic Transcoder uses the detected frame rate of the input source. If you specify a frame rate, we recommend that you perform the following calculation:  <code>Frame rate = maximum recommended decoding speed in luma samples/second / (width in pixels * height in pixels)</code>  where: <ul> <li>  <i>width in pixels</i> and <i>height in pixels</i> represent the Resolution of the output video. </li> <li>  <i>maximum recommended decoding speed in Luma samples/second</i> is less than or equal to the maximum value listed in the following table, based on the value that you specified for Level. </li> </ul> The maximum recommended decoding speed in Luma samples/second for each level is described in the following list (<i>Level - Decoding speed</i>): <ul> <li> 1 - 380160 </li> <li> 1b - 380160 </li> <li> 1.1 - 76800 </li> <li> 1.2 - 1536000 </li> <li> 1.3 - 3041280 </li> <li> 2 - 3041280 </li> <li> 2.1 - 5068800 </li> <li> 2.2 - 5184000 </li> <li> 3 - 10368000 </li> <li> 3.1 - 27648000 </li> <li> 3.2 - 55296000 </li> <li> 4 - 62914560 </li> <li> 4.1 - 62914560 </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BitRate</i> The bit rate of the video stream in the output file, in kilobits/second. Valid values depend on the values of <code>Level</code> and <code>Profile</code>. If you specify <code>auto</code>, Elastic Transcoder uses the detected bit rate of the input source. If you specify a value other than <code>auto</code>, we recommend that you specify a value less than or equal to the maximum H.264-compliant value listed for your level and profile:  <i>Level - Maximum video bit rate in kilobits/second (baseline and main Profile) : maximum video bit rate in kilobits/second (high Profile)</i>  <ul> <li> 1 - 64 : 80 </li> <li> 1b - 128 : 160 </li> <li> 1.1 - 192 : 240 </li> <li> 1.2 - 384 : 480 </li> <li> 1.3 - 768 : 960 </li> <li> 2 - 2000 : 2500 </li> <li> 3 - 10000 : 12500 </li> <li> 3.1 - 14000 : 17500 </li> <li> 3.2 - 20000 : 25000 </li> <li> 4 - 20000 : 25000 </li> <li> 4.1 - 50000 : 62500 </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FixedGOP</i> Applicable only when the value of Video:Codec is one of <code>H.264</code>, <code>MPEG2</code>, or <code>VP8</code>. Whether to use a fixed value for <code>FixedGOP</code>. Valid values are <code>true</code> and <code>false</code>: <ul> <li>  <code>true</code>: Elastic Transcoder uses the value of <code>KeyframesMaxDist</code> for the distance between key frames (the number of frames in a group of pictures, or GOP). </li> <li>  <code>false</code>: The distance between key frames can vary. </li> </ul> <important>  <code>FixedGOP</code> must be set to <code>true</code> for <code>fmp4</code> containers. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyframesMaxDist</i> Applicable only when the value of Video:Codec is one of <code>H.264</code>, <code>MPEG2</code>, or <code>VP8</code>. The maximum number of frames between key frames. Key frames are fully encoded frames; the frames between key frames are encoded based, in part, on the content of the key frames. The value is an integer formatted as a string; valid values are between 1 (every frame is a key frame) and 100000, inclusive. A higher value results in higher compression but may also discernibly decrease video quality. For <code>Smooth</code> outputs, the <code>FrameRate</code> must have a constant ratio to the <code>KeyframesMaxDist</code>. This allows <code>Smooth</code> playlists to switch between different quality levels while the file is being played. For example, an input file can have a <code>FrameRate</code> of 30 with a <code>KeyframesMaxDist</code> of 90. The output file then needs to have a ratio of 1:3. Valid outputs would have <code>FrameRate</code> of 30, 25, and 10, and <code>KeyframesMaxDist</code> of 90, 75, and 30, respectively. Alternately, this can be achieved by setting <code>FrameRate</code> to auto and having the same values for <code>MaxFrameRate</code> and <code>KeyframesMaxDist</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CodecOptions</i>  <b>Profile (H.264/VP8/VP9 Only)</b>  The H.264 profile that you want to use for the output file. Elastic Transcoder supports the following profiles: <ul> <li>  <code>baseline</code>: The profile most commonly used for videoconferencing and for mobile applications. </li> <li>  <code>main</code>: The profile used for standard-definition digital TV broadcasts. </li> <li>  <code>high</code>: The profile used for high-definition digital TV broadcasts and for Blu-ray discs. </li> </ul>  <b>Level (H.264 Only)</b>  The H.264 level that you want to use for the output file. Elastic Transcoder supports the following levels:  <code>1</code>, <code>1b</code>, <code>1.1</code>, <code>1.2</code>, <code>1.3</code>, <code>2</code>, <code>2.1</code>, <code>2.2</code>, <code>3</code>, <code>3.1</code>, <code>3.2</code>, <code>4</code>, <code>4.1</code>   <b>MaxReferenceFrames (H.264 Only)</b>  Applicable only when the value of Video:Codec is H.264. The maximum number of previously decoded frames to use as a reference for decoding future frames. Valid values are integers 0 through 16, but we recommend that you not use a value greater than the following:  <code>Min(Floor(Maximum decoded picture buffer in macroblocks * 256 / (Width in pixels * Height in pixels)), 16)</code>  where <i>Width in pixels</i> and <i>Height in pixels</i> represent either MaxWidth and MaxHeight, or Resolution. <i>Maximum decoded picture buffer in macroblocks</i> depends on the value of the <code>Level</code> object. See the list below. (A macroblock is a block of pixels measuring 16x16.)  <ul> <li> 1 - 396 </li> <li> 1b - 396 </li> <li> 1.1 - 900 </li> <li> 1.2 - 2376 </li> <li> 1.3 - 2376 </li> <li> 2 - 2376 </li> <li> 2.1 - 4752 </li> <li> 2.2 - 8100 </li> <li> 3 - 8100 </li> <li> 3.1 - 18000 </li> <li> 3.2 - 20480 </li> <li> 4 - 32768 </li> <li> 4.1 - 32768 </li> </ul>  <b>MaxBitRate (Optional, H.264/MPEG2/VP8/VP9 only)</b>  The maximum number of bits per second in a video buffer; the size of the buffer is specified by <code>BufferSize</code>. Specify a value between 16 and 62,500. You can reduce the bandwidth required to stream a video by reducing the maximum bit rate, but this also reduces the quality of the video.  <b>BufferSize (Optional, H.264/MPEG2/VP8/VP9 only)</b>  The maximum number of bits in any x seconds of the output video. This window is commonly 10 seconds, the standard segment duration when you're using FMP4 or MPEG-TS for the container type of the output video. Specify an integer greater than 0. If you specify <code>MaxBitRate</code> and omit <code>BufferSize</code>, Elastic Transcoder sets <code>BufferSize</code> to 10 times the value of <code>MaxBitRate</code>.  <b>InterlacedMode (Optional, H.264/MPEG2 Only)</b>  The interlace mode for the output video. Interlaced video is used to double the perceived frame rate for a video by interlacing two fields (one field on every other line, the other field on the other lines) so that the human eye registers multiple pictures per frame. Interlacing reduces the bandwidth required for transmitting a video, but can result in blurred images and flickering. Valid values include <code>Progressive</code> (no interlacing, top to bottom), <code>TopFirst</code> (top field first), <code>BottomFirst</code> (bottom field first), and <code>Auto</code>. If <code>InterlaceMode</code> is not specified, Elastic Transcoder uses <code>Progressive</code> for the output. If <code>Auto</code> is specified, Elastic Transcoder interlaces the output.  <b>ColorSpaceConversionMode (Optional, H.264/MPEG2 Only)</b>  The color space conversion Elastic Transcoder applies to the output video. Color spaces are the algorithms used by the computer to store information about how to render color. <code>Bt.601</code> is the standard for standard definition video, while <code>Bt.709</code> is the standard for high definition video. Valid values include <code>None</code>, <code>Bt709toBt601</code>, <code>Bt601toBt709</code>, and <code>Auto</code>. If you chose <code>Auto</code> for <code>ColorSpaceConversionMode</code> and your output is interlaced, your frame rate is one of <code>23.97</code>, <code>24</code>, <code>25</code>, <code>29.97</code>, <code>50</code>, or <code>60</code>, your <code>SegmentDuration</code> is null, and you are using one of the resolution changes from the list below, Elastic Transcoder applies the following color space conversions: <ul> <li>  <i>Standard to HD, 720x480 to 1920x1080</i> - Elastic Transcoder applies <code>Bt601ToBt709</code>  </li> <li>  <i>Standard to HD, 720x576 to 1920x1080</i> - Elastic Transcoder applies <code>Bt601ToBt709</code>  </li> <li>  <i>HD to Standard, 1920x1080 to 720x480</i> - Elastic Transcoder applies <code>Bt709ToBt601</code>  </li> <li>  <i>HD to Standard, 1920x1080 to 720x576</i> - Elastic Transcoder applies <code>Bt709ToBt601</code>  </li> </ul> <note> Elastic Transcoder may change the behavior of the <code>ColorspaceConversionMode</code> <code>Auto</code> mode in the future. All outputs in a playlist must use the same <code>ColorSpaceConversionMode</code>. </note> If you do not specify a <code>ColorSpaceConversionMode</code>, Elastic Transcoder does not change the color space of a file. If you are unsure what <code>ColorSpaceConversionMode</code> was applied to your output file, you can check the <code>AppliedColorSpaceConversion</code> parameter included in your job response. If your job does not have an <code>AppliedColorSpaceConversion</code> in its response, no <code>ColorSpaceConversionMode</code> was applied.  <b>ChromaSubsampling</b>  The sampling pattern for the chroma (color) channels of the output video. Valid values include <code>yuv420p</code> and <code>yuv422p</code>.  <code>yuv420p</code> samples the chroma information of every other horizontal and every other vertical line, <code>yuv422p</code> samples the color information of every horizontal line and every other vertical line.  <b>LoopCount (Gif Only)</b>  The number of times you want the output gif to loop. Valid values include <code>Infinite</code> and integers between <code>0</code> and <code>100</code>, inclusive.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Codec</i> The video codec for the output file. Valid values include <code>gif</code>, <code>H.264</code>, <code>mpeg2</code>, <code>vp8</code>, and <code>vp9</code>. You can only specify <code>vp8</code> and <code>vp9</code> when the container type is <code>webm</code>, <code>gif</code> when the container type is <code>gif</code>, and <code>mpeg2</code> when the container type is <code>mpg</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PaddingPolicy</i> When you set <code>PaddingPolicy</code> to <code>Pad</code>, Elastic Transcoder may add black bars to the top and bottom and/or left and right sides of thumbnails to make the total size of the thumbnails match the values that you specified for thumbnail <code>MaxWidth</code> and <code>MaxHeight</code> settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SizingPolicy</i> Specify one of the following values to control scaling of thumbnails: <ul> <li>  <code>Fit</code>: Elastic Transcoder scales thumbnails so they match the value that you specified in thumbnail MaxWidth or MaxHeight settings without exceeding the other value.  </li> <li>  <code>Fill</code>: Elastic Transcoder scales thumbnails so they match the value that you specified in thumbnail <code>MaxWidth</code> or <code>MaxHeight</code> settings and matches or exceeds the other value. Elastic Transcoder centers the image in thumbnails and then crops in the dimension (if any) that exceeds the maximum value. </li> <li>  <code>Stretch</code>: Elastic Transcoder stretches thumbnails to match the values that you specified for thumbnail <code>MaxWidth</code> and <code>MaxHeight</code> settings. If the relative proportions of the input video and thumbnails are different, the thumbnails will be distorted. </li> <li>  <code>Keep</code>: Elastic Transcoder does not scale thumbnails. If either dimension of the input video exceeds the values that you specified for thumbnail <code>MaxWidth</code> and <code>MaxHeight</code> settings, Elastic Transcoder crops the thumbnails. </li> <li>  <code>ShrinkToFit</code>: Elastic Transcoder scales thumbnails down so that their dimensions match the values that you specified for at least one of thumbnail <code>MaxWidth</code> and <code>MaxHeight</code> without exceeding either value. If you specify this option, Elastic Transcoder does not scale thumbnails up. </li> <li>  <code>ShrinkToFill</code>: Elastic Transcoder scales thumbnails down so that their dimensions match the values that you specified for at least one of <code>MaxWidth</code> and <code>MaxHeight</code> without dropping below either value. If you specify this option, Elastic Transcoder does not scale thumbnails up. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxHeight</i> The maximum height of thumbnails in pixels. If you specify auto, Elastic Transcoder uses 1080 (Full HD) as the default value. If you specify a numeric value, enter an even integer between 32 and 3072.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxWidth</i> The maximum width of thumbnails in pixels. If you specify auto, Elastic Transcoder uses 1920 (Full HD) as the default value. If you specify a numeric value, enter an even integer between 32 and 4096.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AspectRatio</i> <important> To better control resolution and aspect ratio of thumbnails, we recommend that you use the values <code>MaxWidth</code>, <code>MaxHeight</code>, <code>SizingPolicy</code>, and <code>PaddingPolicy</code> instead of <code>Resolution</code> and <code>AspectRatio</code>. The two groups of settings are mutually exclusive. Do not use them together. </important> The aspect ratio of thumbnails. Valid values include:  <code>auto</code>, <code>1:1</code>, <code>4:3</code>, <code>3:2</code>, <code>16:9</code>  If you specify <code>auto</code>, Elastic Transcoder tries to preserve the aspect ratio of the video in the output file.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Resolution</i> <important> To better control resolution and aspect ratio of thumbnails, we recommend that you use the values <code>MaxWidth</code>, <code>MaxHeight</code>, <code>SizingPolicy</code>, and <code>PaddingPolicy</code> instead of <code>Resolution</code> and <code>AspectRatio</code>. The two groups of settings are mutually exclusive. Do not use them together. </important> The width and height of thumbnail files in pixels. Specify a value in the format <code> <i>width</i> </code> x <code> <i>height</i> </code> where both values are even integers. The values cannot exceed the width and height that you specified in the <code>Video:Resolution</code> object.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Interval</i> The approximate number of seconds between thumbnails. Specify an integer value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> The format of thumbnails, if any. Valid values are <code>jpg</code> and <code>png</code>.  You specify whether you want Elastic Transcoder to create thumbnails when you create a job.<br/>        """
        pass

    def delete_pipeline(self, Id: str) -> DeletePipelineResponse:
        """The DeletePipeline operation removes a pipeline.  You can only delete a pipeline that has never been used or that is not currently in use (doesn't contain any active jobs). If the pipeline is currently in use, <code>DeletePipeline</code> returns an error. 

        :param self:
        :param Id:  The identifier of the pipeline that you want to delete.<br/>
        :return:         """
        pass

    def delete_preset(self, Id: str) -> DeletePresetResponse:
        """The DeletePreset operation removes a preset that you've added in an AWS region. <note> You can't delete the default presets that are included with Elastic Transcoder. </note>

        :param self:
        :param Id:  The identifier of the preset for which you want to get detailed information.<br/>
        :return:         """
        pass

    def list_jobs_by_pipeline(self, PipelineId: str, Ascending: str=None, PageToken: str=None) -> ListJobsByPipelineResponse:
        """The ListJobsByPipeline operation gets a list of the jobs currently in a pipeline. Elastic Transcoder returns all of the jobs currently in the specified pipeline. The response body contains one element for each job that satisfies the search criteria.

        :param self:
        :param PageToken:   When Elastic Transcoder returns more than one page of results, use <code>pageToken</code> in subsequent <code>GET</code> requests to get each successive page of results. <br/>Ascending:   To list jobs in chronological order by the date and time that they were submitted, enter <code>true</code>. To list jobs in reverse chronological order, enter <code>false</code>. <br/>PipelineId:  The ID of the pipeline for which you want to get job information.<br/>
        :return: NextPageToken:   A value that you use to access the second and subsequent pages of results, if any. When the jobs in the specified pipeline fit on one page or when you've reached the last page of results, the value of <code>NextPageToken</code> is <code>null</code>. <br/>Jobs:  An array of <code>Job</code> objects that are in the specified pipeline.<br/>        """
        pass

    def list_jobs_by_status(self, Status: str, Ascending: str=None, PageToken: str=None) -> ListJobsByStatusResponse:
        """The ListJobsByStatus operation gets a list of jobs that have a specified status. The response body contains one element for each job that satisfies the search criteria.

        :param self:
        :param PageToken:   When Elastic Transcoder returns more than one page of results, use <code>pageToken</code> in subsequent <code>GET</code> requests to get each successive page of results. <br/>Ascending:   To list jobs in chronological order by the date and time that they were submitted, enter <code>true</code>. To list jobs in reverse chronological order, enter <code>false</code>. <br/>Status:  To get information about all of the jobs associated with the current AWS account that have a given status, specify the following status: <code>Submitted</code>, <code>Progressing</code>, <code>Complete</code>, <code>Canceled</code>, or <code>Error</code>.<br/>
        :return: NextPageToken:   A value that you use to access the second and subsequent pages of results, if any. When the jobs in the specified pipeline fit on one page or when you've reached the last page of results, the value of <code>NextPageToken</code> is <code>null</code>. <br/>Jobs:  An array of <code>Job</code> objects that have the specified status.<br/>        """
        pass

    def list_pipelines(self, Ascending: str=None, PageToken: str=None) -> ListPipelinesResponse:
        """The ListPipelines operation gets a list of the pipelines associated with the current AWS account.

        :param self:
        :param PageToken:  When Elastic Transcoder returns more than one page of results, use <code>pageToken</code> in subsequent <code>GET</code> requests to get each successive page of results. <br/>Ascending:  To list pipelines in chronological order by the date and time that they were created, enter <code>true</code>. To list pipelines in reverse chronological order, enter <code>false</code>.<br/>
        :return: NextPageToken:  A value that you use to access the second and subsequent pages of results, if any. When the pipelines fit on one page or when you've reached the last page of results, the value of <code>NextPageToken</code> is <code>null</code>.<br/>Pipelines:  An array of <code>Pipeline</code> objects.<br/>        """
        pass

    def list_presets(self, Ascending: str=None, PageToken: str=None) -> ListPresetsResponse:
        """The ListPresets operation gets a list of the default presets included with Elastic Transcoder and the presets that you've added in an AWS region.

        :param self:
        :param PageToken:  When Elastic Transcoder returns more than one page of results, use <code>pageToken</code> in subsequent <code>GET</code> requests to get each successive page of results. <br/>Ascending:  To list presets in chronological order by the date and time that they were created, enter <code>true</code>. To list presets in reverse chronological order, enter <code>false</code>.<br/>
        :return: NextPageToken:  A value that you use to access the second and subsequent pages of results, if any. When the presets fit on one page or when you've reached the last page of results, the value of <code>NextPageToken</code> is <code>null</code>.<br/>Presets:  An array of <code>Preset</code> objects.<br/>        """
        pass

    def read_job(self, Id: str) -> ReadJobResponse:
        """The ReadJob operation returns detailed information about a job.

        :param self:
        :param Id:  The identifier of the job for which you want to get detailed information.<br/>
        :return: Job:  A section of the response body that provides information about the job.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timing</i> Details about the timing of a job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Timing</b>: Details about the timing of a job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserMetadata</i> User-defined metadata that you want to associate with an Elastic Transcoder job. You specify metadata in <code>key/value</code> pairs, and you can add up to 10 <code>key/value</code> pairs per job. Elastic Transcoder does not guarantee that <code>key/value</code> pairs are returned in the same order in which you specify them. Metadata <code>keys</code> and <code>values</code> must use characters from the following list: <ul> <li>  <code>0-9</code>  </li> <li>  <code>A-Z</code> and <code>a-z</code>  </li> <li>  <code>Space</code>  </li> <li> The following symbols: <code>_.:/=+-%@</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i>  The status of the job: <code>Submitted</code>, <code>Progressing</code>, <code>Complete</code>, <code>Canceled</code>, or <code>Error</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Playlists</i> <important> Outputs in Fragmented MP4 or MPEG-TS format only. </important> If you specify a preset in <code>PresetId</code> for which the value of <code>Container</code> is fmp4 (Fragmented MP4) or ts (MPEG-TS), <code>Playlists</code> contains information about the master playlists that you want Elastic Transcoder to create. The maximum number of master playlists in a job is 30.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputKeyPrefix</i> The value, if any, that you want Elastic Transcoder to prepend to the names of all files that this job creates, including output files, thumbnails, and playlists. We recommend that you add a / or some other delimiter to the end of the <code>OutputKeyPrefix</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Outputs</i> Information about the output files. We recommend that you use the <code>Outputs</code> syntax for all jobs, even when you want Elastic Transcoder to transcode a file into only one format. Do not use both the <code>Outputs</code> and <code>Output</code> syntaxes in the same request. You can create a maximum of 30 outputs per job.  If you specify more than one output for a job, Elastic Transcoder creates the files for each output in the order in which you specify them in the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Output</i> If you specified one output for a job, information about that output. If you specified multiple outputs for a job, the Output object lists information about the first output. This duplicates the information that is listed for the first output in the Outputs object. <important> Outputs recommended instead. </important> A section of the request or response body that provides information about the transcoded (target) file. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Output</b>: If you specified one output for a job, information about that output. If you specified multiple outputs for a job, the Output object lists information about the first output. This duplicates the information that is listed for the first output in the Outputs object. <important> Outputs recommended instead. </important> A section of the request or response body that provides information about the transcoded (target) file. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Inputs</i> Information about the files that you're transcoding. If you specified multiple files for this job, Elastic Transcoder stitches the files together to make one output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Input</i> A section of the request or response body that provides information about the file that is being transcoded.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Input</b>: A section of the request or response body that provides information about the file that is being transcoded.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PipelineId</i>  The <code>Id</code> of the pipeline that you want Elastic Transcoder to use for transcoding. The pipeline determines several settings, including the Amazon S3 bucket from which Elastic Transcoder gets the files to transcode and the bucket into which Elastic Transcoder puts the transcoded files. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) for the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier that Elastic Transcoder assigned to the job. You use this value to get settings for the job or to delete the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Container</i> The container type for the input file. If you want Elastic Transcoder to automatically detect the container type of the input file, specify <code>auto</code>. If you want to specify the container type for the input file, enter one of the following values:   <code>3gp</code>, <code>aac</code>, <code>asf</code>, <code>avi</code>, <code>divx</code>, <code>flv</code>, <code>m4a</code>, <code>mkv</code>, <code>mov</code>, <code>mp3</code>, <code>mp4</code>, <code>mpeg</code>, <code>mpeg-ps</code>, <code>mpeg-ts</code>, <code>mxf</code>, <code>ogg</code>, <code>vob</code>, <code>wav</code>, <code>webm</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Interlaced</i> Whether the input file is interlaced. If you want Elastic Transcoder to automatically detect whether the input file is interlaced, specify <code>auto</code>. If you want to specify whether the input file is interlaced, enter one of the following values:  <code>true</code>, <code>false</code>  If you specify a value other than <code>auto</code>, Elastic Transcoder disables automatic detection of interlacing.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AspectRatio</i>  The aspect ratio of the input file. If you want Elastic Transcoder to automatically detect the aspect ratio of the input file, specify <code>auto</code>. If you want to specify the aspect ratio for the output file, enter one of the following values:   <code>1:1</code>, <code>4:3</code>, <code>3:2</code>, <code>16:9</code>   If you specify a value other than <code>auto</code>, Elastic Transcoder disables automatic detection of the aspect ratio. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Resolution</i> This value must be <code>auto</code>, which causes Elastic Transcoder to automatically detect the resolution of the input file.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameRate</i> The frame rate of the input file. If you want Elastic Transcoder to automatically detect the frame rate of the input file, specify <code>auto</code>. If you want to specify the frame rate for the input file, enter one of the following values:   <code>10</code>, <code>15</code>, <code>23.97</code>, <code>24</code>, <code>25</code>, <code>29.97</code>, <code>30</code>, <code>60</code>  If you specify a value other than <code>auto</code>, Elastic Transcoder disables automatic detection of the frame rate.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i>  The name of the file to transcode. Elsewhere in the body of the JSON block is the the ID of the pipeline to use for processing the job. The <code>InputBucket</code> object in that pipeline tells Elastic Transcoder which Amazon S3 bucket to get the file from.  If the file name includes a prefix, such as <code>cooking/lasagna.mpg</code>, include the prefix in the key. If the file isn't in the specified bucket, Elastic Transcoder returns an error.<br/>The encryption settings, if any, that are used for decrypting your input files. If your input file is encrypted, you must specify the mode that Elastic Transcoder uses to decrypt your file.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: The encryption settings, if any, that are used for decrypting your input files. If your input file is encrypted, you must specify the mode that Elastic Transcoder uses to decrypt your file.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ThumbnailPattern</i> Whether you want Elastic Transcoder to create thumbnails for your videos and, if so, how you want Elastic Transcoder to name the files. If you don't want Elastic Transcoder to create thumbnails, specify "". If you do want Elastic Transcoder to create thumbnails, specify the information that you want to include in the file name for each thumbnail. You can specify the following values in any sequence: <ul> <li>  <b> <code>{count}</code> (Required)</b>: If you want to create thumbnails, you must include <code>{count}</code> in the <code>ThumbnailPattern</code> object. Wherever you specify <code>{count}</code>, Elastic Transcoder adds a five-digit sequence number (beginning with <b>00001</b>) to thumbnail file names. The number indicates where a given thumbnail appears in the sequence of thumbnails for a transcoded file.  <important> If you specify a literal value and/or <code>{resolution}</code> but you omit <code>{count}</code>, Elastic Transcoder returns a validation error and does not create the job. </important> </li> <li>  <b>Literal values (Optional)</b>: You can specify literal values anywhere in the <code>ThumbnailPattern</code> object. For example, you can include them as a file name prefix or as a delimiter between <code>{resolution}</code> and <code>{count}</code>.  </li> <li>  <b> <code>{resolution}</code> (Optional)</b>: If you want Elastic Transcoder to include the resolution in the file name, include <code>{resolution}</code> in the <code>ThumbnailPattern</code> object.  </li> </ul> When creating thumbnails, Elastic Transcoder automatically saves the files in the format (.jpg or .png) that appears in the preset that you specified in the <code>PresetID</code> value of <code>CreateJobOutput</code>. Elastic Transcoder also appends the applicable file name extension.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i>  The name to assign to the transcoded file. Elastic Transcoder saves the file in the Amazon S3 bucket specified by the <code>OutputBucket</code> object in the pipeline that is specified by the pipeline ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> A sequential counter, starting with 1, that identifies an output among the outputs from the current job. In the Output syntax, this value is always 1.<br/>The encryption settings, if any, that you want Elastic Transcoder to apply to your thumbnail.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ThumbnailEncryption</b>: The encryption settings, if any, that you want Elastic Transcoder to apply to your thumbnail.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FinishTimeMillis</i> The time the job finished transcoding, in epoch milliseconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTimeMillis</i> The time the job began transcoding, in epoch milliseconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmitTimeMillis</i> The time the job was submitted to Elastic Transcoder, in epoch milliseconds.<br/>        """
        pass

    def read_pipeline(self, Id: str) -> ReadPipelineResponse:
        """The ReadPipeline operation gets detailed information about a pipeline.

        :param self:
        :param Id:  The identifier of the pipeline to read.<br/>
        :return: Warnings:  Elastic Transcoder returns a warning if the resources used by your pipeline are not in the same region as the pipeline. Using resources in the same region, such as your Amazon S3 buckets, Amazon SNS notification topics, and AWS KMS key, reduces processing time and prevents cross-regional charges.<br/>Pipeline:  A section of the response body that provides information about the pipeline.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThumbnailConfig</i> Information about the Amazon S3 bucket in which you want Elastic Transcoder to save thumbnail files. Either you specify both <code>ContentConfig</code> and <code>ThumbnailConfig</code>, or you specify <code>OutputBucket</code>. <ul> <li>  <code>Bucket</code>: The Amazon S3 bucket in which you want Elastic Transcoder to save thumbnail files.  </li> <li>  <code>Permissions</code>: A list of the users and/or predefined Amazon S3 groups you want to have access to thumbnail files, and the type of access that you want them to have.  <ul> <li> GranteeType: The type of value that appears in the Grantee object: <ul> <li>  <code>Canonical</code>: Either the canonical user ID for an AWS account or an origin access identity for an Amazon CloudFront distribution. <important> A canonical user ID is not the same as an AWS account number. </important> </li> <li>  <code>Email</code>: The registered email address of an AWS account. </li> <li>  <code>Group</code>: One of the following predefined Amazon S3 groups: <code>AllUsers</code>, <code>AuthenticatedUsers</code>, or <code>LogDelivery</code>. </li> </ul> </li> <li>  <code>Grantee</code>: The AWS user or group that you want to have access to thumbnail files. </li> <li> Access: The permission that you want to give to the AWS user that is listed in Grantee. Valid values include:  <ul> <li>  <code>READ</code>: The grantee can read the thumbnails and metadata for thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>READ_ACP</code>: The grantee can read the object ACL for thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>WRITE_ACP</code>: The grantee can write the ACL for the thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>FULL_CONTROL</code>: The grantee has READ, READ_ACP, and WRITE_ACP permissions for the thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> </ul> </li> </ul> </li> <li>  <code>StorageClass</code>: The Amazon S3 storage class, <code>Standard</code> or <code>ReducedRedundancy</code>, that you want Elastic Transcoder to assign to the thumbnails that it stores in your Amazon S3 bucket. </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ThumbnailConfig</b>: Information about the Amazon S3 bucket in which you want Elastic Transcoder to save thumbnail files. Either you specify both <code>ContentConfig</code> and <code>ThumbnailConfig</code>, or you specify <code>OutputBucket</code>. <ul> <li>  <code>Bucket</code>: The Amazon S3 bucket in which you want Elastic Transcoder to save thumbnail files.  </li> <li>  <code>Permissions</code>: A list of the users and/or predefined Amazon S3 groups you want to have access to thumbnail files, and the type of access that you want them to have.  <ul> <li> GranteeType: The type of value that appears in the Grantee object: <ul> <li>  <code>Canonical</code>: Either the canonical user ID for an AWS account or an origin access identity for an Amazon CloudFront distribution. <important> A canonical user ID is not the same as an AWS account number. </important> </li> <li>  <code>Email</code>: The registered email address of an AWS account. </li> <li>  <code>Group</code>: One of the following predefined Amazon S3 groups: <code>AllUsers</code>, <code>AuthenticatedUsers</code>, or <code>LogDelivery</code>. </li> </ul> </li> <li>  <code>Grantee</code>: The AWS user or group that you want to have access to thumbnail files. </li> <li> Access: The permission that you want to give to the AWS user that is listed in Grantee. Valid values include:  <ul> <li>  <code>READ</code>: The grantee can read the thumbnails and metadata for thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>READ_ACP</code>: The grantee can read the object ACL for thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>WRITE_ACP</code>: The grantee can write the ACL for the thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>FULL_CONTROL</code>: The grantee has READ, READ_ACP, and WRITE_ACP permissions for the thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> </ul> </li> </ul> </li> <li>  <code>StorageClass</code>: The Amazon S3 storage class, <code>Standard</code> or <code>ReducedRedundancy</code>, that you want Elastic Transcoder to assign to the thumbnails that it stores in your Amazon S3 bucket. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContentConfig</i> Information about the Amazon S3 bucket in which you want Elastic Transcoder to save transcoded files and playlists. Either you specify both <code>ContentConfig</code> and <code>ThumbnailConfig</code>, or you specify <code>OutputBucket</code>. <ul> <li>  <b>Bucket</b>: The Amazon S3 bucket in which you want Elastic Transcoder to save transcoded files and playlists. </li> <li>  <b>Permissions</b>: A list of the users and/or predefined Amazon S3 groups you want to have access to transcoded files and playlists, and the type of access that you want them to have.  <ul> <li> GranteeType: The type of value that appears in the <code>Grantee</code> object:  <ul> <li>  <code>Canonical</code>: Either the canonical user ID for an AWS account or an origin access identity for an Amazon CloudFront distribution. </li> <li>  <code>Email</code>: The registered email address of an AWS account. </li> <li>  <code>Group</code>: One of the following predefined Amazon S3 groups: <code>AllUsers</code>, <code>AuthenticatedUsers</code>, or <code>LogDelivery</code>. </li> </ul> </li> <li>  <code>Grantee</code>: The AWS user or group that you want to have access to transcoded files and playlists. </li> <li>  <code>Access</code>: The permission that you want to give to the AWS user that is listed in <code>Grantee</code>. Valid values include: <ul> <li>  <code>READ</code>: The grantee can read the objects and metadata for objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>READ_ACP</code>: The grantee can read the object ACL for objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>WRITE_ACP</code>: The grantee can write the ACL for the objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>FULL_CONTROL</code>: The grantee has <code>READ</code>, <code>READ_ACP</code>, and <code>WRITE_ACP</code> permissions for the objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> </ul> </li> </ul> </li> <li>  <b>StorageClass</b>: The Amazon S3 storage class, Standard or ReducedRedundancy, that you want Elastic Transcoder to assign to the video files and playlists that it stores in your Amazon S3 bucket.  </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ContentConfig</b>: Information about the Amazon S3 bucket in which you want Elastic Transcoder to save transcoded files and playlists. Either you specify both <code>ContentConfig</code> and <code>ThumbnailConfig</code>, or you specify <code>OutputBucket</code>. <ul> <li>  <b>Bucket</b>: The Amazon S3 bucket in which you want Elastic Transcoder to save transcoded files and playlists. </li> <li>  <b>Permissions</b>: A list of the users and/or predefined Amazon S3 groups you want to have access to transcoded files and playlists, and the type of access that you want them to have.  <ul> <li> GranteeType: The type of value that appears in the <code>Grantee</code> object:  <ul> <li>  <code>Canonical</code>: Either the canonical user ID for an AWS account or an origin access identity for an Amazon CloudFront distribution. </li> <li>  <code>Email</code>: The registered email address of an AWS account. </li> <li>  <code>Group</code>: One of the following predefined Amazon S3 groups: <code>AllUsers</code>, <code>AuthenticatedUsers</code>, or <code>LogDelivery</code>. </li> </ul> </li> <li>  <code>Grantee</code>: The AWS user or group that you want to have access to transcoded files and playlists. </li> <li>  <code>Access</code>: The permission that you want to give to the AWS user that is listed in <code>Grantee</code>. Valid values include: <ul> <li>  <code>READ</code>: The grantee can read the objects and metadata for objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>READ_ACP</code>: The grantee can read the object ACL for objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>WRITE_ACP</code>: The grantee can write the ACL for the objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>FULL_CONTROL</code>: The grantee has <code>READ</code>, <code>READ_ACP</code>, and <code>WRITE_ACP</code> permissions for the objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> </ul> </li> </ul> </li> <li>  <b>StorageClass</b>: The Amazon S3 storage class, Standard or ReducedRedundancy, that you want Elastic Transcoder to assign to the video files and playlists that it stores in your Amazon S3 bucket.  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Notifications</i> The Amazon Simple Notification Service (Amazon SNS) topic that you want to notify to report job status. <important> To receive notifications, you must also subscribe to the new topic in the Amazon SNS console. </important> <ul> <li>  <b>Progressing</b> (optional): The Amazon Simple Notification Service (Amazon SNS) topic that you want to notify when Elastic Transcoder has started to process the job. </li> <li>  <b>Complete</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder has finished processing the job. </li> <li>  <b>Warning</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder encounters a warning condition. </li> <li>  <b>Error</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder encounters an error condition. </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Notifications</b>: The Amazon Simple Notification Service (Amazon SNS) topic that you want to notify to report job status. <important> To receive notifications, you must also subscribe to the new topic in the Amazon SNS console. </important> <ul> <li>  <b>Progressing</b> (optional): The Amazon Simple Notification Service (Amazon SNS) topic that you want to notify when Elastic Transcoder has started to process the job. </li> <li>  <b>Complete</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder has finished processing the job. </li> <li>  <b>Warning</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder encounters a warning condition. </li> <li>  <b>Error</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder encounters an error condition. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AwsKmsKeyArn</i> The AWS Key Management Service (AWS KMS) key that you want to use with this pipeline. If you use either <code>s3</code> or <code>s3-aws-kms</code> as your <code>Encryption:Mode</code>, you don't need to provide a key with your job because a default key, known as an AWS-KMS key, is created for you automatically. You need to provide an AWS-KMS key only if you want to use a non-default AWS-KMS key, or if you are using an <code>Encryption:Mode</code> of <code>aes-cbc-pkcs7</code>, <code>aes-ctr</code>, or <code>aes-gcm</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Role</i> The IAM Amazon Resource Name (ARN) for the role that Elastic Transcoder uses to transcode jobs for this pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputBucket</i> The Amazon S3 bucket in which you want Elastic Transcoder to save transcoded files, thumbnails, and playlists. Either you specify this value, or you specify both <code>ContentConfig</code> and <code>ThumbnailConfig</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputBucket</i> The Amazon S3 bucket from which Elastic Transcoder gets media files for transcoding and the graphics files, if any, that you want to use for watermarks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current status of the pipeline: <ul> <li>  <code>Active</code>: The pipeline is processing jobs. </li> <li>  <code>Paused</code>: The pipeline is not currently processing jobs. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the pipeline. We recommend that the name be unique within the AWS account, but uniqueness is not enforced. Constraints: Maximum 40 characters<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) for the pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier for the pipeline. You use this value to identify the pipeline in which you want to perform a variety of operations, such as creating a job or a preset.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Error</i> The Amazon SNS topic that you want to notify when Elastic Transcoder encounters an error condition.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Warning</i> The Amazon SNS topic that you want to notify when Elastic Transcoder encounters a warning condition.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Completed</i> The Amazon SNS topic that you want to notify when Elastic Transcoder has finished processing the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Progressing</i> The Amazon Simple Notification Service (Amazon SNS) topic that you want to notify when Elastic Transcoder has started to process the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Permissions</i> Optional. The <code>Permissions</code> object specifies which users and/or predefined Amazon S3 groups you want to have access to transcoded files and playlists, and the type of access you want them to have. You can grant permissions to a maximum of 30 users and/or predefined Amazon S3 groups. If you include <code>Permissions</code>, Elastic Transcoder grants only the permissions that you specify. It does not grant full permissions to the owner of the role specified by <code>Role</code>. If you want that user to have full control, you must explicitly grant full control to the user.  If you omit <code>Permissions</code>, Elastic Transcoder grants full control over the transcoded files and playlists to the owner of the role specified by <code>Role</code>, and grants no other permissions to any other user or group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageClass</i>  The Amazon S3 storage class, <code>Standard</code> or <code>ReducedRedundancy</code>, that you want Elastic Transcoder to assign to the video files and playlists that it stores in your Amazon S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i>  The Amazon S3 bucket in which you want Elastic Transcoder to save the transcoded files. Specify this value when all of the following are true: <ul> <li> You want to save transcoded files, thumbnails (if any), and playlists (if any) together in one bucket. </li> <li> You do not want to specify the users or groups who have access to the transcoded files, thumbnails, and playlists. </li> <li> You do not want to specify the permissions that Elastic Transcoder grants to the files. </li> <li> You want to associate the transcoded files and thumbnails with the Amazon S3 Standard storage class. </li> </ul> If you want to save transcoded files and playlists in one bucket and thumbnails in another bucket, specify which users can access the transcoded files or the permissions the users have, or change the Amazon S3 storage class, omit OutputBucket and specify values for <code>ContentConfig</code> and <code>ThumbnailConfig</code> instead. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Permissions</i> Optional. The <code>Permissions</code> object specifies which users and/or predefined Amazon S3 groups you want to have access to transcoded files and playlists, and the type of access you want them to have. You can grant permissions to a maximum of 30 users and/or predefined Amazon S3 groups. If you include <code>Permissions</code>, Elastic Transcoder grants only the permissions that you specify. It does not grant full permissions to the owner of the role specified by <code>Role</code>. If you want that user to have full control, you must explicitly grant full control to the user.  If you omit <code>Permissions</code>, Elastic Transcoder grants full control over the transcoded files and playlists to the owner of the role specified by <code>Role</code>, and grants no other permissions to any other user or group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageClass</i>  The Amazon S3 storage class, <code>Standard</code> or <code>ReducedRedundancy</code>, that you want Elastic Transcoder to assign to the video files and playlists that it stores in your Amazon S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i>  The Amazon S3 bucket in which you want Elastic Transcoder to save the transcoded files. Specify this value when all of the following are true: <ul> <li> You want to save transcoded files, thumbnails (if any), and playlists (if any) together in one bucket. </li> <li> You do not want to specify the users or groups who have access to the transcoded files, thumbnails, and playlists. </li> <li> You do not want to specify the permissions that Elastic Transcoder grants to the files. </li> <li> You want to associate the transcoded files and thumbnails with the Amazon S3 Standard storage class. </li> </ul> If you want to save transcoded files and playlists in one bucket and thumbnails in another bucket, specify which users can access the transcoded files or the permissions the users have, or change the Amazon S3 storage class, omit OutputBucket and specify values for <code>ContentConfig</code> and <code>ThumbnailConfig</code> instead. <br/>        """
        pass

    def read_preset(self, Id: str) -> ReadPresetResponse:
        """The ReadPreset operation gets detailed information about a preset.

        :param self:
        :param Id:  The identifier of the preset for which you want to get detailed information.<br/>
        :return: Preset:  A section of the response body that provides information about the preset.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> Whether the preset is a default preset provided by Elastic Transcoder (<code>System</code>) or a preset that you have defined (<code>Custom</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Thumbnails</i> A section of the response body that provides information about the thumbnail preset values, if any.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Thumbnails</b>: A section of the response body that provides information about the thumbnail preset values, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Video</i> A section of the response body that provides information about the video preset values.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Video</b>: A section of the response body that provides information about the video preset values.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Audio</i> A section of the response body that provides information about the audio preset values.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Audio</b>: A section of the response body that provides information about the audio preset values.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Container</i> The container type for the output file. Valid values include <code>flac</code>, <code>flv</code>, <code>fmp4</code>, <code>gif</code>, <code>mp3</code>, <code>mp4</code>, <code>mpg</code>, <code>mxf</code>, <code>oga</code>, <code>ogg</code>, <code>ts</code>, and <code>webm</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of the preset.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the preset.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) for the preset.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> Identifier for the new preset. You use this value to get settings for the preset or to delete it.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AudioPackingMode</i> The method of organizing audio channels and tracks. Use <code>Audio:Channels</code> to specify the number of channels in your output, and <code>Audio:AudioPackingMode</code> to specify the number of tracks and their relation to the channels. If you do not specify an <code>Audio:AudioPackingMode</code>, Elastic Transcoder uses <code>SingleTrack</code>. The following values are valid:  <code>SingleTrack</code>, <code>OneChannelPerTrack</code>, and <code>OneChannelPerTrackWithMosTo8Tracks</code>  When you specify <code>SingleTrack</code>, Elastic Transcoder creates a single track for your output. The track can have up to eight channels. Use <code>SingleTrack</code> for all non-<code>mxf</code> containers. The outputs of <code>SingleTrack</code> for a specific channel value and inputs are as follows: <ul> <li>  <code>0</code> <b> channels with any input:</b> Audio omitted from the output </li> <li>  <code>1, 2, or auto </code> <b>channels with no audio input:</b> Audio omitted from the output </li> <li>  <code>1 </code> <b>channel with any input with audio:</b> One track with one channel, downmixed if necessary </li> <li>  <code>2 </code> <b>channels with one track with one channel:</b> One track with two identical channels </li> <li>  <code>2 or auto </code> <b>channels with two tracks with one channel each:</b> One track with two channels </li> <li>  <code>2 or auto </code> <b>channels with one track with two channels:</b> One track with two channels </li> <li>  <code>2 </code> <b>channels with one track with multiple channels:</b> One track with two channels </li> <li>  <code>auto </code> <b>channels with one track with one channel:</b> One track with one channel </li> <li>  <code>auto </code> <b>channels with one track with multiple channels:</b> One track with multiple channels </li> </ul> When you specify <code>OneChannelPerTrack</code>, Elastic Transcoder creates a new track for every channel in your output. Your output can have up to eight single-channel tracks. The outputs of <code>OneChannelPerTrack</code> for a specific channel value and inputs are as follows: <ul> <li>  <code>0 </code> <b>channels with any input:</b> Audio omitted from the output </li> <li>  <code>1, 2, or auto </code> <b>channels with no audio input:</b> Audio omitted from the output </li> <li>  <code>1 </code> <b>channel with any input with audio:</b> One track with one channel, downmixed if necessary </li> <li>  <code>2 </code> <b>channels with one track with one channel:</b> Two tracks with one identical channel each </li> <li>  <code>2 or auto </code> <b>channels with two tracks with one channel each:</b> Two tracks with one channel each </li> <li>  <code>2 or auto </code> <b>channels with one track with two channels:</b> Two tracks with one channel each </li> <li>  <code>2 </code> <b>channels with one track with multiple channels:</b> Two tracks with one channel each </li> <li>  <code>auto </code> <b>channels with one track with one channel:</b> One track with one channel </li> <li>  <code>auto </code> <b>channels with one track with multiple channels:</b> Up to eight tracks with one channel each </li> </ul> When you specify <code>OneChannelPerTrackWithMosTo8Tracks</code>, Elastic Transcoder creates eight single-channel tracks for your output. All tracks that do not contain audio data from an input channel are MOS, or Mit Out Sound, tracks. The outputs of <code>OneChannelPerTrackWithMosTo8Tracks</code> for a specific channel value and inputs are as follows: <ul> <li>  <code>0 </code> <b>channels with any input:</b> Audio omitted from the output </li> <li>  <code>1, 2, or auto </code> <b>channels with no audio input:</b> Audio omitted from the output </li> <li>  <code>1 </code> <b>channel with any input with audio:</b> One track with one channel, downmixed if necessary, plus six MOS tracks </li> <li>  <code>2 </code> <b>channels with one track with one channel:</b> Two tracks with one identical channel each, plus six MOS tracks </li> <li>  <code>2 or auto </code> <b>channels with two tracks with one channel each:</b> Two tracks with one channel each, plus six MOS tracks </li> <li>  <code>2 or auto </code> <b>channels with one track with two channels:</b> Two tracks with one channel each, plus six MOS tracks </li> <li>  <code>2 </code> <b>channels with one track with multiple channels:</b> Two tracks with one channel each, plus six MOS tracks </li> <li>  <code>auto </code> <b>channels with one track with one channel:</b> One track with one channel, plus seven MOS tracks </li> <li>  <code>auto </code> <b>channels with one track with multiple channels:</b> Up to eight tracks with one channel each, plus MOS tracks until there are eight tracks in all </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Channels</i> The number of audio channels in the output file. The following values are valid:  <code>auto</code>, <code>0</code>, <code>1</code>, <code>2</code>  One channel carries the information played by a single speaker. For example, a stereo track with two channels sends one channel to the left speaker, and the other channel to the right speaker. The output channels are organized into tracks. If you want Elastic Transcoder to automatically detect the number of audio channels in the input file and use that value for the output file, select <code>auto</code>. The output of a specific channel value and inputs are as follows: <ul> <li>  <code>auto</code> <b> channel specified, with any input:</b> Pass through up to eight input channels. </li> <li>  <code>0</code> <b> channels specified, with any input:</b> Audio omitted from the output. </li> <li>  <code>1</code> <b> channel specified, with at least one input channel:</b> Mono sound. </li> <li>  <code>2</code> <b> channels specified, with any input:</b> Two identical mono channels or stereo. For more information about tracks, see <code>Audio:AudioPackingMode.</code>  </li> </ul>  For more information about how Elastic Transcoder organizes channels and tracks, see <code>Audio:AudioPackingMode</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BitRate</i> The bit rate of the audio stream in the output file, in kilobits/second. Enter an integer between 64 and 320, inclusive.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SampleRate</i> The sample rate of the audio stream in the output file, in Hertz. Valid values include:  <code>auto</code>, <code>22050</code>, <code>32000</code>, <code>44100</code>, <code>48000</code>, <code>96000</code>  If you specify <code>auto</code>, Elastic Transcoder automatically detects the sample rate.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Codec</i> The audio codec for the output file. Valid values include <code>aac</code>, <code>flac</code>, <code>mp2</code>, <code>mp3</code>, <code>pcm</code>, and <code>vorbis</code>.<br/>If you specified <code>AAC</code> for <code>Audio:Codec</code>, this is the <code>AAC</code> compression profile to use. Valid values include:  <code>auto</code>, <code>AAC-LC</code>, <code>HE-AAC</code>, <code>HE-AACv2</code>  If you specify <code>auto</code>, Elastic Transcoder chooses a profile based on the bit rate of the output file.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CodecOptions</b>: If you specified <code>AAC</code> for <code>Audio:Codec</code>, this is the <code>AAC</code> compression profile to use. Valid values include:  <code>auto</code>, <code>AAC-LC</code>, <code>HE-AAC</code>, <code>HE-AACv2</code>  If you specify <code>auto</code>, Elastic Transcoder chooses a profile based on the bit rate of the output file.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Watermarks</i> Settings for the size, location, and opacity of graphics that you want Elastic Transcoder to overlay over videos that are transcoded using this preset. You can specify settings for up to four watermarks. Watermarks appear in the specified size and location, and with the specified opacity for the duration of the transcoded video. Watermarks can be in .png or .jpg format. If you want to display a watermark that is not rectangular, use the .png format, which supports transparency. When you create a job that uses this preset, you specify the .png or .jpg graphics that you want Elastic Transcoder to include in the transcoded videos. You can specify fewer graphics in the job than you specify watermark settings in the preset, which allows you to use the same preset for up to four watermarks that have different dimensions.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PaddingPolicy</i> When you set <code>PaddingPolicy</code> to <code>Pad</code>, Elastic Transcoder may add black bars to the top and bottom and/or left and right sides of the output video to make the total size of the output video match the values that you specified for <code>MaxWidth</code> and <code>MaxHeight</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SizingPolicy</i> Specify one of the following values to control scaling of the output video: <ul> <li>  <code>Fit</code>: Elastic Transcoder scales the output video so it matches the value that you specified in either <code>MaxWidth</code> or <code>MaxHeight</code> without exceeding the other value. </li> <li>  <code>Fill</code>: Elastic Transcoder scales the output video so it matches the value that you specified in either <code>MaxWidth</code> or <code>MaxHeight</code> and matches or exceeds the other value. Elastic Transcoder centers the output video and then crops it in the dimension (if any) that exceeds the maximum value. </li> <li>  <code>Stretch</code>: Elastic Transcoder stretches the output video to match the values that you specified for <code>MaxWidth</code> and <code>MaxHeight</code>. If the relative proportions of the input video and the output video are different, the output video will be distorted. </li> <li>  <code>Keep</code>: Elastic Transcoder does not scale the output video. If either dimension of the input video exceeds the values that you specified for <code>MaxWidth</code> and <code>MaxHeight</code>, Elastic Transcoder crops the output video. </li> <li>  <code>ShrinkToFit</code>: Elastic Transcoder scales the output video down so that its dimensions match the values that you specified for at least one of <code>MaxWidth</code> and <code>MaxHeight</code> without exceeding either value. If you specify this option, Elastic Transcoder does not scale the video up. </li> <li>  <code>ShrinkToFill</code>: Elastic Transcoder scales the output video down so that its dimensions match the values that you specified for at least one of <code>MaxWidth</code> and <code>MaxHeight</code> without dropping below either value. If you specify this option, Elastic Transcoder does not scale the video up. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayAspectRatio</i> The value that Elastic Transcoder adds to the metadata in the output file.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxHeight</i> The maximum height of the output video in pixels. If you specify <code>auto</code>, Elastic Transcoder uses 1080 (Full HD) as the default value. If you specify a numeric value, enter an even integer between 96 and 3072.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxWidth</i>  The maximum width of the output video in pixels. If you specify <code>auto</code>, Elastic Transcoder uses 1920 (Full HD) as the default value. If you specify a numeric value, enter an even integer between 128 and 4096. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AspectRatio</i> <important> To better control resolution and aspect ratio of output videos, we recommend that you use the values <code>MaxWidth</code>, <code>MaxHeight</code>, <code>SizingPolicy</code>, <code>PaddingPolicy</code>, and <code>DisplayAspectRatio</code> instead of <code>Resolution</code> and <code>AspectRatio</code>. The two groups of settings are mutually exclusive. Do not use them together. </important> The display aspect ratio of the video in the output file. Valid values include:  <code>auto</code>, <code>1:1</code>, <code>4:3</code>, <code>3:2</code>, <code>16:9</code>  If you specify <code>auto</code>, Elastic Transcoder tries to preserve the aspect ratio of the input file. If you specify an aspect ratio for the output file that differs from aspect ratio of the input file, Elastic Transcoder adds pillarboxing (black bars on the sides) or letterboxing (black bars on the top and bottom) to maintain the aspect ratio of the active region of the video.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Resolution</i> <important> To better control resolution and aspect ratio of output videos, we recommend that you use the values <code>MaxWidth</code>, <code>MaxHeight</code>, <code>SizingPolicy</code>, <code>PaddingPolicy</code>, and <code>DisplayAspectRatio</code> instead of <code>Resolution</code> and <code>AspectRatio</code>. The two groups of settings are mutually exclusive. Do not use them together. </important> The width and height of the video in the output file, in pixels. Valid values are <code>auto</code> and <i>width</i> x <i>height</i>: <ul> <li>  <code>auto</code>: Elastic Transcoder attempts to preserve the width and height of the input file, subject to the following rules. </li> <li>  <code> <i>width</i> x <i>height</i> </code>: The width and height of the output video in pixels. </li> </ul> Note the following about specifying the width and height: <ul> <li> The width must be an even integer between 128 and 4096, inclusive. </li> <li> The height must be an even integer between 96 and 3072, inclusive. </li> <li> If you specify a resolution that is less than the resolution of the input file, Elastic Transcoder rescales the output file to the lower resolution. </li> <li> If you specify a resolution that is greater than the resolution of the input file, Elastic Transcoder rescales the output to the higher resolution. </li> <li> We recommend that you specify a resolution for which the product of width and height is less than or equal to the applicable value in the following list (<i>List - Max width x height value</i>): <ul> <li> 1 - 25344 </li> <li> 1b - 25344 </li> <li> 1.1 - 101376 </li> <li> 1.2 - 101376 </li> <li> 1.3 - 101376 </li> <li> 2 - 101376 </li> <li> 2.1 - 202752 </li> <li> 2.2 - 404720 </li> <li> 3 - 404720 </li> <li> 3.1 - 921600 </li> <li> 3.2 - 1310720 </li> <li> 4 - 2097152 </li> <li> 4.1 - 2097152 </li> </ul> </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxFrameRate</i> If you specify <code>auto</code> for <code>FrameRate</code>, Elastic Transcoder uses the frame rate of the input video for the frame rate of the output video. Specify the maximum frame rate that you want Elastic Transcoder to use when the frame rate of the input video is greater than the desired maximum frame rate of the output video. Valid values include: <code>10</code>, <code>15</code>, <code>23.97</code>, <code>24</code>, <code>25</code>, <code>29.97</code>, <code>30</code>, <code>60</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameRate</i> The frames per second for the video stream in the output file. Valid values include:  <code>auto</code>, <code>10</code>, <code>15</code>, <code>23.97</code>, <code>24</code>, <code>25</code>, <code>29.97</code>, <code>30</code>, <code>60</code>  If you specify <code>auto</code>, Elastic Transcoder uses the detected frame rate of the input source. If you specify a frame rate, we recommend that you perform the following calculation:  <code>Frame rate = maximum recommended decoding speed in luma samples/second / (width in pixels * height in pixels)</code>  where: <ul> <li>  <i>width in pixels</i> and <i>height in pixels</i> represent the Resolution of the output video. </li> <li>  <i>maximum recommended decoding speed in Luma samples/second</i> is less than or equal to the maximum value listed in the following table, based on the value that you specified for Level. </li> </ul> The maximum recommended decoding speed in Luma samples/second for each level is described in the following list (<i>Level - Decoding speed</i>): <ul> <li> 1 - 380160 </li> <li> 1b - 380160 </li> <li> 1.1 - 76800 </li> <li> 1.2 - 1536000 </li> <li> 1.3 - 3041280 </li> <li> 2 - 3041280 </li> <li> 2.1 - 5068800 </li> <li> 2.2 - 5184000 </li> <li> 3 - 10368000 </li> <li> 3.1 - 27648000 </li> <li> 3.2 - 55296000 </li> <li> 4 - 62914560 </li> <li> 4.1 - 62914560 </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BitRate</i> The bit rate of the video stream in the output file, in kilobits/second. Valid values depend on the values of <code>Level</code> and <code>Profile</code>. If you specify <code>auto</code>, Elastic Transcoder uses the detected bit rate of the input source. If you specify a value other than <code>auto</code>, we recommend that you specify a value less than or equal to the maximum H.264-compliant value listed for your level and profile:  <i>Level - Maximum video bit rate in kilobits/second (baseline and main Profile) : maximum video bit rate in kilobits/second (high Profile)</i>  <ul> <li> 1 - 64 : 80 </li> <li> 1b - 128 : 160 </li> <li> 1.1 - 192 : 240 </li> <li> 1.2 - 384 : 480 </li> <li> 1.3 - 768 : 960 </li> <li> 2 - 2000 : 2500 </li> <li> 3 - 10000 : 12500 </li> <li> 3.1 - 14000 : 17500 </li> <li> 3.2 - 20000 : 25000 </li> <li> 4 - 20000 : 25000 </li> <li> 4.1 - 50000 : 62500 </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FixedGOP</i> Applicable only when the value of Video:Codec is one of <code>H.264</code>, <code>MPEG2</code>, or <code>VP8</code>. Whether to use a fixed value for <code>FixedGOP</code>. Valid values are <code>true</code> and <code>false</code>: <ul> <li>  <code>true</code>: Elastic Transcoder uses the value of <code>KeyframesMaxDist</code> for the distance between key frames (the number of frames in a group of pictures, or GOP). </li> <li>  <code>false</code>: The distance between key frames can vary. </li> </ul> <important>  <code>FixedGOP</code> must be set to <code>true</code> for <code>fmp4</code> containers. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyframesMaxDist</i> Applicable only when the value of Video:Codec is one of <code>H.264</code>, <code>MPEG2</code>, or <code>VP8</code>. The maximum number of frames between key frames. Key frames are fully encoded frames; the frames between key frames are encoded based, in part, on the content of the key frames. The value is an integer formatted as a string; valid values are between 1 (every frame is a key frame) and 100000, inclusive. A higher value results in higher compression but may also discernibly decrease video quality. For <code>Smooth</code> outputs, the <code>FrameRate</code> must have a constant ratio to the <code>KeyframesMaxDist</code>. This allows <code>Smooth</code> playlists to switch between different quality levels while the file is being played. For example, an input file can have a <code>FrameRate</code> of 30 with a <code>KeyframesMaxDist</code> of 90. The output file then needs to have a ratio of 1:3. Valid outputs would have <code>FrameRate</code> of 30, 25, and 10, and <code>KeyframesMaxDist</code> of 90, 75, and 30, respectively. Alternately, this can be achieved by setting <code>FrameRate</code> to auto and having the same values for <code>MaxFrameRate</code> and <code>KeyframesMaxDist</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CodecOptions</i>  <b>Profile (H.264/VP8/VP9 Only)</b>  The H.264 profile that you want to use for the output file. Elastic Transcoder supports the following profiles: <ul> <li>  <code>baseline</code>: The profile most commonly used for videoconferencing and for mobile applications. </li> <li>  <code>main</code>: The profile used for standard-definition digital TV broadcasts. </li> <li>  <code>high</code>: The profile used for high-definition digital TV broadcasts and for Blu-ray discs. </li> </ul>  <b>Level (H.264 Only)</b>  The H.264 level that you want to use for the output file. Elastic Transcoder supports the following levels:  <code>1</code>, <code>1b</code>, <code>1.1</code>, <code>1.2</code>, <code>1.3</code>, <code>2</code>, <code>2.1</code>, <code>2.2</code>, <code>3</code>, <code>3.1</code>, <code>3.2</code>, <code>4</code>, <code>4.1</code>   <b>MaxReferenceFrames (H.264 Only)</b>  Applicable only when the value of Video:Codec is H.264. The maximum number of previously decoded frames to use as a reference for decoding future frames. Valid values are integers 0 through 16, but we recommend that you not use a value greater than the following:  <code>Min(Floor(Maximum decoded picture buffer in macroblocks * 256 / (Width in pixels * Height in pixels)), 16)</code>  where <i>Width in pixels</i> and <i>Height in pixels</i> represent either MaxWidth and MaxHeight, or Resolution. <i>Maximum decoded picture buffer in macroblocks</i> depends on the value of the <code>Level</code> object. See the list below. (A macroblock is a block of pixels measuring 16x16.)  <ul> <li> 1 - 396 </li> <li> 1b - 396 </li> <li> 1.1 - 900 </li> <li> 1.2 - 2376 </li> <li> 1.3 - 2376 </li> <li> 2 - 2376 </li> <li> 2.1 - 4752 </li> <li> 2.2 - 8100 </li> <li> 3 - 8100 </li> <li> 3.1 - 18000 </li> <li> 3.2 - 20480 </li> <li> 4 - 32768 </li> <li> 4.1 - 32768 </li> </ul>  <b>MaxBitRate (Optional, H.264/MPEG2/VP8/VP9 only)</b>  The maximum number of bits per second in a video buffer; the size of the buffer is specified by <code>BufferSize</code>. Specify a value between 16 and 62,500. You can reduce the bandwidth required to stream a video by reducing the maximum bit rate, but this also reduces the quality of the video.  <b>BufferSize (Optional, H.264/MPEG2/VP8/VP9 only)</b>  The maximum number of bits in any x seconds of the output video. This window is commonly 10 seconds, the standard segment duration when you're using FMP4 or MPEG-TS for the container type of the output video. Specify an integer greater than 0. If you specify <code>MaxBitRate</code> and omit <code>BufferSize</code>, Elastic Transcoder sets <code>BufferSize</code> to 10 times the value of <code>MaxBitRate</code>.  <b>InterlacedMode (Optional, H.264/MPEG2 Only)</b>  The interlace mode for the output video. Interlaced video is used to double the perceived frame rate for a video by interlacing two fields (one field on every other line, the other field on the other lines) so that the human eye registers multiple pictures per frame. Interlacing reduces the bandwidth required for transmitting a video, but can result in blurred images and flickering. Valid values include <code>Progressive</code> (no interlacing, top to bottom), <code>TopFirst</code> (top field first), <code>BottomFirst</code> (bottom field first), and <code>Auto</code>. If <code>InterlaceMode</code> is not specified, Elastic Transcoder uses <code>Progressive</code> for the output. If <code>Auto</code> is specified, Elastic Transcoder interlaces the output.  <b>ColorSpaceConversionMode (Optional, H.264/MPEG2 Only)</b>  The color space conversion Elastic Transcoder applies to the output video. Color spaces are the algorithms used by the computer to store information about how to render color. <code>Bt.601</code> is the standard for standard definition video, while <code>Bt.709</code> is the standard for high definition video. Valid values include <code>None</code>, <code>Bt709toBt601</code>, <code>Bt601toBt709</code>, and <code>Auto</code>. If you chose <code>Auto</code> for <code>ColorSpaceConversionMode</code> and your output is interlaced, your frame rate is one of <code>23.97</code>, <code>24</code>, <code>25</code>, <code>29.97</code>, <code>50</code>, or <code>60</code>, your <code>SegmentDuration</code> is null, and you are using one of the resolution changes from the list below, Elastic Transcoder applies the following color space conversions: <ul> <li>  <i>Standard to HD, 720x480 to 1920x1080</i> - Elastic Transcoder applies <code>Bt601ToBt709</code>  </li> <li>  <i>Standard to HD, 720x576 to 1920x1080</i> - Elastic Transcoder applies <code>Bt601ToBt709</code>  </li> <li>  <i>HD to Standard, 1920x1080 to 720x480</i> - Elastic Transcoder applies <code>Bt709ToBt601</code>  </li> <li>  <i>HD to Standard, 1920x1080 to 720x576</i> - Elastic Transcoder applies <code>Bt709ToBt601</code>  </li> </ul> <note> Elastic Transcoder may change the behavior of the <code>ColorspaceConversionMode</code> <code>Auto</code> mode in the future. All outputs in a playlist must use the same <code>ColorSpaceConversionMode</code>. </note> If you do not specify a <code>ColorSpaceConversionMode</code>, Elastic Transcoder does not change the color space of a file. If you are unsure what <code>ColorSpaceConversionMode</code> was applied to your output file, you can check the <code>AppliedColorSpaceConversion</code> parameter included in your job response. If your job does not have an <code>AppliedColorSpaceConversion</code> in its response, no <code>ColorSpaceConversionMode</code> was applied.  <b>ChromaSubsampling</b>  The sampling pattern for the chroma (color) channels of the output video. Valid values include <code>yuv420p</code> and <code>yuv422p</code>.  <code>yuv420p</code> samples the chroma information of every other horizontal and every other vertical line, <code>yuv422p</code> samples the color information of every horizontal line and every other vertical line.  <b>LoopCount (Gif Only)</b>  The number of times you want the output gif to loop. Valid values include <code>Infinite</code> and integers between <code>0</code> and <code>100</code>, inclusive.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Codec</i> The video codec for the output file. Valid values include <code>gif</code>, <code>H.264</code>, <code>mpeg2</code>, <code>vp8</code>, and <code>vp9</code>. You can only specify <code>vp8</code> and <code>vp9</code> when the container type is <code>webm</code>, <code>gif</code> when the container type is <code>gif</code>, and <code>mpeg2</code> when the container type is <code>mpg</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PaddingPolicy</i> When you set <code>PaddingPolicy</code> to <code>Pad</code>, Elastic Transcoder may add black bars to the top and bottom and/or left and right sides of thumbnails to make the total size of the thumbnails match the values that you specified for thumbnail <code>MaxWidth</code> and <code>MaxHeight</code> settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SizingPolicy</i> Specify one of the following values to control scaling of thumbnails: <ul> <li>  <code>Fit</code>: Elastic Transcoder scales thumbnails so they match the value that you specified in thumbnail MaxWidth or MaxHeight settings without exceeding the other value.  </li> <li>  <code>Fill</code>: Elastic Transcoder scales thumbnails so they match the value that you specified in thumbnail <code>MaxWidth</code> or <code>MaxHeight</code> settings and matches or exceeds the other value. Elastic Transcoder centers the image in thumbnails and then crops in the dimension (if any) that exceeds the maximum value. </li> <li>  <code>Stretch</code>: Elastic Transcoder stretches thumbnails to match the values that you specified for thumbnail <code>MaxWidth</code> and <code>MaxHeight</code> settings. If the relative proportions of the input video and thumbnails are different, the thumbnails will be distorted. </li> <li>  <code>Keep</code>: Elastic Transcoder does not scale thumbnails. If either dimension of the input video exceeds the values that you specified for thumbnail <code>MaxWidth</code> and <code>MaxHeight</code> settings, Elastic Transcoder crops the thumbnails. </li> <li>  <code>ShrinkToFit</code>: Elastic Transcoder scales thumbnails down so that their dimensions match the values that you specified for at least one of thumbnail <code>MaxWidth</code> and <code>MaxHeight</code> without exceeding either value. If you specify this option, Elastic Transcoder does not scale thumbnails up. </li> <li>  <code>ShrinkToFill</code>: Elastic Transcoder scales thumbnails down so that their dimensions match the values that you specified for at least one of <code>MaxWidth</code> and <code>MaxHeight</code> without dropping below either value. If you specify this option, Elastic Transcoder does not scale thumbnails up. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxHeight</i> The maximum height of thumbnails in pixels. If you specify auto, Elastic Transcoder uses 1080 (Full HD) as the default value. If you specify a numeric value, enter an even integer between 32 and 3072.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxWidth</i> The maximum width of thumbnails in pixels. If you specify auto, Elastic Transcoder uses 1920 (Full HD) as the default value. If you specify a numeric value, enter an even integer between 32 and 4096.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AspectRatio</i> <important> To better control resolution and aspect ratio of thumbnails, we recommend that you use the values <code>MaxWidth</code>, <code>MaxHeight</code>, <code>SizingPolicy</code>, and <code>PaddingPolicy</code> instead of <code>Resolution</code> and <code>AspectRatio</code>. The two groups of settings are mutually exclusive. Do not use them together. </important> The aspect ratio of thumbnails. Valid values include:  <code>auto</code>, <code>1:1</code>, <code>4:3</code>, <code>3:2</code>, <code>16:9</code>  If you specify <code>auto</code>, Elastic Transcoder tries to preserve the aspect ratio of the video in the output file.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Resolution</i> <important> To better control resolution and aspect ratio of thumbnails, we recommend that you use the values <code>MaxWidth</code>, <code>MaxHeight</code>, <code>SizingPolicy</code>, and <code>PaddingPolicy</code> instead of <code>Resolution</code> and <code>AspectRatio</code>. The two groups of settings are mutually exclusive. Do not use them together. </important> The width and height of thumbnail files in pixels. Specify a value in the format <code> <i>width</i> </code> x <code> <i>height</i> </code> where both values are even integers. The values cannot exceed the width and height that you specified in the <code>Video:Resolution</code> object.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Interval</i> The approximate number of seconds between thumbnails. Specify an integer value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> The format of thumbnails, if any. Valid values are <code>jpg</code> and <code>png</code>.  You specify whether you want Elastic Transcoder to create thumbnails when you create a job.<br/>        """
        pass

    def test_role(self, Topics: List[SnsTopic], OutputBucket: str, InputBucket: str, Role: str) -> TestRoleResponse:
        """The TestRole operation tests the IAM role used to create the pipeline. The <code>TestRole</code> action lets you determine whether the IAM role you are using has sufficient permissions to let Elastic Transcoder perform tasks associated with the transcoding process. The action attempts to assume the specified IAM role, checks read access to the input and output buckets, and tries to send a test notification to Amazon SNS topics that you specify.

        :param self:
        :param Topics:  The ARNs of one or more Amazon Simple Notification Service (Amazon SNS) topics that you want the action to send a test notification to.<br/>OutputBucket:  The Amazon S3 bucket that Elastic Transcoder writes transcoded media files to. The action attempts to read from this bucket.<br/>InputBucket:  The Amazon S3 bucket that contains media files to be transcoded. The action attempts to read from this bucket.<br/>Role:  The IAM Amazon Resource Name (ARN) for the role that you want Elastic Transcoder to test.<br/>
        :return: Messages:  If the <code>Success</code> element contains <code>false</code>, this value is an array of one or more error messages that were generated during the test process.<br/>Success:  If the operation is successful, this value is <code>true</code>; otherwise, the value is <code>false</code>.<br/>        """
        pass

    def update_pipeline(self, Id: str, Name: str=None, InputBucket: str=None, Role: str=None, AwsKmsKeyArn: str=None, Notifications: Notifications=None, ContentConfig: PipelineOutputConfig=None, ThumbnailConfig: PipelineOutputConfig=None) -> UpdatePipelineResponse:
        """ Use the <code>UpdatePipeline</code> operation to update settings for a pipeline. <important> When you change pipeline settings, your changes take effect immediately. Jobs that you have already submitted and that Elastic Transcoder has not started to process are affected in addition to jobs that you submit after you change settings.  </important>

        :param self:
        :param ThumbnailConfig:  The <code>ThumbnailConfig</code> object specifies several values, including the Amazon S3 bucket in which you want Elastic Transcoder to save thumbnail files, which users you want to have access to the files, the type of access you want users to have, and the storage class that you want to assign to the files. If you specify values for <code>ContentConfig</code>, you must also specify values for <code>ThumbnailConfig</code> even if you don't want to create thumbnails. If you specify values for <code>ContentConfig</code> and <code>ThumbnailConfig</code>, omit the <code>OutputBucket</code> object. <ul> <li>  <b>Bucket</b>: The Amazon S3 bucket in which you want Elastic Transcoder to save thumbnail files. </li> <li>  <b>Permissions</b> (Optional): The <code>Permissions</code> object specifies which users and/or predefined Amazon S3 groups you want to have access to thumbnail files, and the type of access you want them to have. You can grant permissions to a maximum of 30 users and/or predefined Amazon S3 groups. </li> <li>  <b>GranteeType</b>: Specify the type of value that appears in the Grantee object: <ul> <li>  <b>Canonical</b>: The value in the <code>Grantee</code> object is either the canonical user ID for an AWS account or an origin access identity for an Amazon CloudFront distribution. <important> A canonical user ID is not the same as an AWS account number. </important> </li> <li>  <b>Email</b>: The value in the <code>Grantee</code> object is the registered email address of an AWS account. </li> <li>  <b>Group</b>: The value in the <code>Grantee</code> object is one of the following predefined Amazon S3 groups: <code>AllUsers</code>, <code>AuthenticatedUsers</code>, or <code>LogDelivery</code>. </li> </ul> </li> <li>  <b>Grantee</b>: The AWS user or group that you want to have access to thumbnail files. To identify the user or group, you can specify the canonical user ID for an AWS account, an origin access identity for a CloudFront distribution, the registered email address of an AWS account, or a predefined Amazon S3 group.  </li> <li>  <b>Access</b>: The permission that you want to give to the AWS user that you specified in <code>Grantee</code>. Permissions are granted on the thumbnail files that Elastic Transcoder adds to the bucket. Valid values include:  <ul> <li>  <code>READ</code>: The grantee can read the thumbnails and metadata for objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>READ_ACP</code>: The grantee can read the object ACL for thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>WRITE_ACP</code>: The grantee can write the ACL for the thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>FULL_CONTROL</code>: The grantee has <code>READ</code>, <code>READ_ACP</code>, and <code>WRITE_ACP</code> permissions for the thumbnails that Elastic Transcoder adds to the Amazon S3 bucket.  </li> </ul> </li> <li>  <b>StorageClass</b>: The Amazon S3 storage class, <code>Standard</code> or <code>ReducedRedundancy</code>, that you want Elastic Transcoder to assign to the thumbnails that it stores in your Amazon S3 bucket. </li> </ul>:<br/>ContentConfig:  The optional <code>ContentConfig</code> object specifies information about the Amazon S3 bucket in which you want Elastic Transcoder to save transcoded files and playlists: which bucket to use, which users you want to have access to the files, the type of access you want users to have, and the storage class that you want to assign to the files. If you specify values for <code>ContentConfig</code>, you must also specify values for <code>ThumbnailConfig</code>. If you specify values for <code>ContentConfig</code> and <code>ThumbnailConfig</code>, omit the <code>OutputBucket</code> object. <ul> <li>  <b>Bucket</b>: The Amazon S3 bucket in which you want Elastic Transcoder to save transcoded files and playlists. </li> <li>  <b>Permissions</b> (Optional): The Permissions object specifies which users you want to have access to transcoded files and the type of access you want them to have. You can grant permissions to a maximum of 30 users and/or predefined Amazon S3 groups. </li> <li>  <b>Grantee Type</b>: Specify the type of value that appears in the <code>Grantee</code> object: <ul> <li>  <b>Canonical</b>: The value in the <code>Grantee</code> object is either the canonical user ID for an AWS account or an origin access identity for an Amazon CloudFront distribution. For more information about canonical user IDs, see Access Control List (ACL) Overview in the Amazon Simple Storage Service Developer Guide. For more information about using CloudFront origin access identities to require that users use CloudFront URLs instead of Amazon S3 URLs, see Using an Origin Access Identity to Restrict Access to Your Amazon S3 Content. <important> A canonical user ID is not the same as an AWS account number. </important> </li> <li>  <b>Email</b>: The value in the <code>Grantee</code> object is the registered email address of an AWS account. </li> <li>  <b>Group</b>: The value in the <code>Grantee</code> object is one of the following predefined Amazon S3 groups: <code>AllUsers</code>, <code>AuthenticatedUsers</code>, or <code>LogDelivery</code>. </li> </ul> </li> <li>  <b>Grantee</b>: The AWS user or group that you want to have access to transcoded files and playlists. To identify the user or group, you can specify the canonical user ID for an AWS account, an origin access identity for a CloudFront distribution, the registered email address of an AWS account, or a predefined Amazon S3 group  </li> <li>  <b>Access</b>: The permission that you want to give to the AWS user that you specified in <code>Grantee</code>. Permissions are granted on the files that Elastic Transcoder adds to the bucket, including playlists and video files. Valid values include:  <ul> <li>  <code>READ</code>: The grantee can read the objects and metadata for objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>READ_ACP</code>: The grantee can read the object ACL for objects that Elastic Transcoder adds to the Amazon S3 bucket.  </li> <li>  <code>WRITE_ACP</code>: The grantee can write the ACL for the objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>FULL_CONTROL</code>: The grantee has <code>READ</code>, <code>READ_ACP</code>, and <code>WRITE_ACP</code> permissions for the objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> </ul> </li> <li>  <b>StorageClass</b>: The Amazon S3 storage class, <code>Standard</code> or <code>ReducedRedundancy</code>, that you want Elastic Transcoder to assign to the video files and playlists that it stores in your Amazon S3 bucket. </li> </ul>:<br/>Notifications:  The topic ARN for the Amazon Simple Notification Service (Amazon SNS) topic that you want to notify to report job status. <important> To receive notifications, you must also subscribe to the new topic in the Amazon SNS console. </important> <ul> <li>  <b>Progressing</b>: The topic ARN for the Amazon Simple Notification Service (Amazon SNS) topic that you want to notify when Elastic Transcoder has started to process jobs that are added to this pipeline. This is the ARN that Amazon SNS returned when you created the topic. </li> <li>  <b>Complete</b>: The topic ARN for the Amazon SNS topic that you want to notify when Elastic Transcoder has finished processing a job. This is the ARN that Amazon SNS returned when you created the topic. </li> <li>  <b>Warning</b>: The topic ARN for the Amazon SNS topic that you want to notify when Elastic Transcoder encounters a warning condition. This is the ARN that Amazon SNS returned when you created the topic. </li> <li>  <b>Error</b>: The topic ARN for the Amazon SNS topic that you want to notify when Elastic Transcoder encounters an error condition. This is the ARN that Amazon SNS returned when you created the topic. </li> </ul>:<br/>AwsKmsKeyArn:  The AWS Key Management Service (AWS KMS) key that you want to use with this pipeline. If you use either <code>s3</code> or <code>s3-aws-kms</code> as your <code>Encryption:Mode</code>, you don't need to provide a key with your job because a default key, known as an AWS-KMS key, is created for you automatically. You need to provide an AWS-KMS key only if you want to use a non-default AWS-KMS key, or if you are using an <code>Encryption:Mode</code> of <code>aes-cbc-pkcs7</code>, <code>aes-ctr</code>, or <code>aes-gcm</code>.<br/>Role:  The IAM Amazon Resource Name (ARN) for the role that you want Elastic Transcoder to use to transcode jobs for this pipeline.<br/>InputBucket:  The Amazon S3 bucket in which you saved the media files that you want to transcode and the graphics that you want to use as watermarks.<br/>Name:  The name of the pipeline. We recommend that the name be unique within the AWS account, but uniqueness is not enforced. Constraints: Maximum 40 characters<br/>Id:  The ID of the pipeline that you want to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Error</i> The Amazon SNS topic that you want to notify when Elastic Transcoder encounters an error condition.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Warning</i> The Amazon SNS topic that you want to notify when Elastic Transcoder encounters a warning condition.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Completed</i> The Amazon SNS topic that you want to notify when Elastic Transcoder has finished processing the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Progressing</i> The Amazon Simple Notification Service (Amazon SNS) topic that you want to notify when Elastic Transcoder has started to process the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Permissions</i> Optional. The <code>Permissions</code> object specifies which users and/or predefined Amazon S3 groups you want to have access to transcoded files and playlists, and the type of access you want them to have. You can grant permissions to a maximum of 30 users and/or predefined Amazon S3 groups. If you include <code>Permissions</code>, Elastic Transcoder grants only the permissions that you specify. It does not grant full permissions to the owner of the role specified by <code>Role</code>. If you want that user to have full control, you must explicitly grant full control to the user.  If you omit <code>Permissions</code>, Elastic Transcoder grants full control over the transcoded files and playlists to the owner of the role specified by <code>Role</code>, and grants no other permissions to any other user or group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageClass</i>  The Amazon S3 storage class, <code>Standard</code> or <code>ReducedRedundancy</code>, that you want Elastic Transcoder to assign to the video files and playlists that it stores in your Amazon S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i>  The Amazon S3 bucket in which you want Elastic Transcoder to save the transcoded files. Specify this value when all of the following are true: <ul> <li> You want to save transcoded files, thumbnails (if any), and playlists (if any) together in one bucket. </li> <li> You do not want to specify the users or groups who have access to the transcoded files, thumbnails, and playlists. </li> <li> You do not want to specify the permissions that Elastic Transcoder grants to the files. </li> <li> You want to associate the transcoded files and thumbnails with the Amazon S3 Standard storage class. </li> </ul> If you want to save transcoded files and playlists in one bucket and thumbnails in another bucket, specify which users can access the transcoded files or the permissions the users have, or change the Amazon S3 storage class, omit OutputBucket and specify values for <code>ContentConfig</code> and <code>ThumbnailConfig</code> instead. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Permissions</i> Optional. The <code>Permissions</code> object specifies which users and/or predefined Amazon S3 groups you want to have access to transcoded files and playlists, and the type of access you want them to have. You can grant permissions to a maximum of 30 users and/or predefined Amazon S3 groups. If you include <code>Permissions</code>, Elastic Transcoder grants only the permissions that you specify. It does not grant full permissions to the owner of the role specified by <code>Role</code>. If you want that user to have full control, you must explicitly grant full control to the user.  If you omit <code>Permissions</code>, Elastic Transcoder grants full control over the transcoded files and playlists to the owner of the role specified by <code>Role</code>, and grants no other permissions to any other user or group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageClass</i>  The Amazon S3 storage class, <code>Standard</code> or <code>ReducedRedundancy</code>, that you want Elastic Transcoder to assign to the video files and playlists that it stores in your Amazon S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i>  The Amazon S3 bucket in which you want Elastic Transcoder to save the transcoded files. Specify this value when all of the following are true: <ul> <li> You want to save transcoded files, thumbnails (if any), and playlists (if any) together in one bucket. </li> <li> You do not want to specify the users or groups who have access to the transcoded files, thumbnails, and playlists. </li> <li> You do not want to specify the permissions that Elastic Transcoder grants to the files. </li> <li> You want to associate the transcoded files and thumbnails with the Amazon S3 Standard storage class. </li> </ul> If you want to save transcoded files and playlists in one bucket and thumbnails in another bucket, specify which users can access the transcoded files or the permissions the users have, or change the Amazon S3 storage class, omit OutputBucket and specify values for <code>ContentConfig</code> and <code>ThumbnailConfig</code> instead. <br/>
        :return: Warnings:  Elastic Transcoder returns a warning if the resources used by your pipeline are not in the same region as the pipeline. Using resources in the same region, such as your Amazon S3 buckets, Amazon SNS notification topics, and AWS KMS key, reduces processing time and prevents cross-regional charges.<br/>Pipeline:  The pipeline updated by this <code>UpdatePipelineResponse</code> call.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThumbnailConfig</i> Information about the Amazon S3 bucket in which you want Elastic Transcoder to save thumbnail files. Either you specify both <code>ContentConfig</code> and <code>ThumbnailConfig</code>, or you specify <code>OutputBucket</code>. <ul> <li>  <code>Bucket</code>: The Amazon S3 bucket in which you want Elastic Transcoder to save thumbnail files.  </li> <li>  <code>Permissions</code>: A list of the users and/or predefined Amazon S3 groups you want to have access to thumbnail files, and the type of access that you want them to have.  <ul> <li> GranteeType: The type of value that appears in the Grantee object: <ul> <li>  <code>Canonical</code>: Either the canonical user ID for an AWS account or an origin access identity for an Amazon CloudFront distribution. <important> A canonical user ID is not the same as an AWS account number. </important> </li> <li>  <code>Email</code>: The registered email address of an AWS account. </li> <li>  <code>Group</code>: One of the following predefined Amazon S3 groups: <code>AllUsers</code>, <code>AuthenticatedUsers</code>, or <code>LogDelivery</code>. </li> </ul> </li> <li>  <code>Grantee</code>: The AWS user or group that you want to have access to thumbnail files. </li> <li> Access: The permission that you want to give to the AWS user that is listed in Grantee. Valid values include:  <ul> <li>  <code>READ</code>: The grantee can read the thumbnails and metadata for thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>READ_ACP</code>: The grantee can read the object ACL for thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>WRITE_ACP</code>: The grantee can write the ACL for the thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>FULL_CONTROL</code>: The grantee has READ, READ_ACP, and WRITE_ACP permissions for the thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> </ul> </li> </ul> </li> <li>  <code>StorageClass</code>: The Amazon S3 storage class, <code>Standard</code> or <code>ReducedRedundancy</code>, that you want Elastic Transcoder to assign to the thumbnails that it stores in your Amazon S3 bucket. </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ThumbnailConfig</b>: Information about the Amazon S3 bucket in which you want Elastic Transcoder to save thumbnail files. Either you specify both <code>ContentConfig</code> and <code>ThumbnailConfig</code>, or you specify <code>OutputBucket</code>. <ul> <li>  <code>Bucket</code>: The Amazon S3 bucket in which you want Elastic Transcoder to save thumbnail files.  </li> <li>  <code>Permissions</code>: A list of the users and/or predefined Amazon S3 groups you want to have access to thumbnail files, and the type of access that you want them to have.  <ul> <li> GranteeType: The type of value that appears in the Grantee object: <ul> <li>  <code>Canonical</code>: Either the canonical user ID for an AWS account or an origin access identity for an Amazon CloudFront distribution. <important> A canonical user ID is not the same as an AWS account number. </important> </li> <li>  <code>Email</code>: The registered email address of an AWS account. </li> <li>  <code>Group</code>: One of the following predefined Amazon S3 groups: <code>AllUsers</code>, <code>AuthenticatedUsers</code>, or <code>LogDelivery</code>. </li> </ul> </li> <li>  <code>Grantee</code>: The AWS user or group that you want to have access to thumbnail files. </li> <li> Access: The permission that you want to give to the AWS user that is listed in Grantee. Valid values include:  <ul> <li>  <code>READ</code>: The grantee can read the thumbnails and metadata for thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>READ_ACP</code>: The grantee can read the object ACL for thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>WRITE_ACP</code>: The grantee can write the ACL for the thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>FULL_CONTROL</code>: The grantee has READ, READ_ACP, and WRITE_ACP permissions for the thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> </ul> </li> </ul> </li> <li>  <code>StorageClass</code>: The Amazon S3 storage class, <code>Standard</code> or <code>ReducedRedundancy</code>, that you want Elastic Transcoder to assign to the thumbnails that it stores in your Amazon S3 bucket. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContentConfig</i> Information about the Amazon S3 bucket in which you want Elastic Transcoder to save transcoded files and playlists. Either you specify both <code>ContentConfig</code> and <code>ThumbnailConfig</code>, or you specify <code>OutputBucket</code>. <ul> <li>  <b>Bucket</b>: The Amazon S3 bucket in which you want Elastic Transcoder to save transcoded files and playlists. </li> <li>  <b>Permissions</b>: A list of the users and/or predefined Amazon S3 groups you want to have access to transcoded files and playlists, and the type of access that you want them to have.  <ul> <li> GranteeType: The type of value that appears in the <code>Grantee</code> object:  <ul> <li>  <code>Canonical</code>: Either the canonical user ID for an AWS account or an origin access identity for an Amazon CloudFront distribution. </li> <li>  <code>Email</code>: The registered email address of an AWS account. </li> <li>  <code>Group</code>: One of the following predefined Amazon S3 groups: <code>AllUsers</code>, <code>AuthenticatedUsers</code>, or <code>LogDelivery</code>. </li> </ul> </li> <li>  <code>Grantee</code>: The AWS user or group that you want to have access to transcoded files and playlists. </li> <li>  <code>Access</code>: The permission that you want to give to the AWS user that is listed in <code>Grantee</code>. Valid values include: <ul> <li>  <code>READ</code>: The grantee can read the objects and metadata for objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>READ_ACP</code>: The grantee can read the object ACL for objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>WRITE_ACP</code>: The grantee can write the ACL for the objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>FULL_CONTROL</code>: The grantee has <code>READ</code>, <code>READ_ACP</code>, and <code>WRITE_ACP</code> permissions for the objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> </ul> </li> </ul> </li> <li>  <b>StorageClass</b>: The Amazon S3 storage class, Standard or ReducedRedundancy, that you want Elastic Transcoder to assign to the video files and playlists that it stores in your Amazon S3 bucket.  </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ContentConfig</b>: Information about the Amazon S3 bucket in which you want Elastic Transcoder to save transcoded files and playlists. Either you specify both <code>ContentConfig</code> and <code>ThumbnailConfig</code>, or you specify <code>OutputBucket</code>. <ul> <li>  <b>Bucket</b>: The Amazon S3 bucket in which you want Elastic Transcoder to save transcoded files and playlists. </li> <li>  <b>Permissions</b>: A list of the users and/or predefined Amazon S3 groups you want to have access to transcoded files and playlists, and the type of access that you want them to have.  <ul> <li> GranteeType: The type of value that appears in the <code>Grantee</code> object:  <ul> <li>  <code>Canonical</code>: Either the canonical user ID for an AWS account or an origin access identity for an Amazon CloudFront distribution. </li> <li>  <code>Email</code>: The registered email address of an AWS account. </li> <li>  <code>Group</code>: One of the following predefined Amazon S3 groups: <code>AllUsers</code>, <code>AuthenticatedUsers</code>, or <code>LogDelivery</code>. </li> </ul> </li> <li>  <code>Grantee</code>: The AWS user or group that you want to have access to transcoded files and playlists. </li> <li>  <code>Access</code>: The permission that you want to give to the AWS user that is listed in <code>Grantee</code>. Valid values include: <ul> <li>  <code>READ</code>: The grantee can read the objects and metadata for objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>READ_ACP</code>: The grantee can read the object ACL for objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>WRITE_ACP</code>: The grantee can write the ACL for the objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>FULL_CONTROL</code>: The grantee has <code>READ</code>, <code>READ_ACP</code>, and <code>WRITE_ACP</code> permissions for the objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> </ul> </li> </ul> </li> <li>  <b>StorageClass</b>: The Amazon S3 storage class, Standard or ReducedRedundancy, that you want Elastic Transcoder to assign to the video files and playlists that it stores in your Amazon S3 bucket.  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Notifications</i> The Amazon Simple Notification Service (Amazon SNS) topic that you want to notify to report job status. <important> To receive notifications, you must also subscribe to the new topic in the Amazon SNS console. </important> <ul> <li>  <b>Progressing</b> (optional): The Amazon Simple Notification Service (Amazon SNS) topic that you want to notify when Elastic Transcoder has started to process the job. </li> <li>  <b>Complete</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder has finished processing the job. </li> <li>  <b>Warning</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder encounters a warning condition. </li> <li>  <b>Error</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder encounters an error condition. </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Notifications</b>: The Amazon Simple Notification Service (Amazon SNS) topic that you want to notify to report job status. <important> To receive notifications, you must also subscribe to the new topic in the Amazon SNS console. </important> <ul> <li>  <b>Progressing</b> (optional): The Amazon Simple Notification Service (Amazon SNS) topic that you want to notify when Elastic Transcoder has started to process the job. </li> <li>  <b>Complete</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder has finished processing the job. </li> <li>  <b>Warning</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder encounters a warning condition. </li> <li>  <b>Error</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder encounters an error condition. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AwsKmsKeyArn</i> The AWS Key Management Service (AWS KMS) key that you want to use with this pipeline. If you use either <code>s3</code> or <code>s3-aws-kms</code> as your <code>Encryption:Mode</code>, you don't need to provide a key with your job because a default key, known as an AWS-KMS key, is created for you automatically. You need to provide an AWS-KMS key only if you want to use a non-default AWS-KMS key, or if you are using an <code>Encryption:Mode</code> of <code>aes-cbc-pkcs7</code>, <code>aes-ctr</code>, or <code>aes-gcm</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Role</i> The IAM Amazon Resource Name (ARN) for the role that Elastic Transcoder uses to transcode jobs for this pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputBucket</i> The Amazon S3 bucket in which you want Elastic Transcoder to save transcoded files, thumbnails, and playlists. Either you specify this value, or you specify both <code>ContentConfig</code> and <code>ThumbnailConfig</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputBucket</i> The Amazon S3 bucket from which Elastic Transcoder gets media files for transcoding and the graphics files, if any, that you want to use for watermarks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current status of the pipeline: <ul> <li>  <code>Active</code>: The pipeline is processing jobs. </li> <li>  <code>Paused</code>: The pipeline is not currently processing jobs. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the pipeline. We recommend that the name be unique within the AWS account, but uniqueness is not enforced. Constraints: Maximum 40 characters<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) for the pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier for the pipeline. You use this value to identify the pipeline in which you want to perform a variety of operations, such as creating a job or a preset.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Error</i> The Amazon SNS topic that you want to notify when Elastic Transcoder encounters an error condition.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Warning</i> The Amazon SNS topic that you want to notify when Elastic Transcoder encounters a warning condition.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Completed</i> The Amazon SNS topic that you want to notify when Elastic Transcoder has finished processing the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Progressing</i> The Amazon Simple Notification Service (Amazon SNS) topic that you want to notify when Elastic Transcoder has started to process the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Permissions</i> Optional. The <code>Permissions</code> object specifies which users and/or predefined Amazon S3 groups you want to have access to transcoded files and playlists, and the type of access you want them to have. You can grant permissions to a maximum of 30 users and/or predefined Amazon S3 groups. If you include <code>Permissions</code>, Elastic Transcoder grants only the permissions that you specify. It does not grant full permissions to the owner of the role specified by <code>Role</code>. If you want that user to have full control, you must explicitly grant full control to the user.  If you omit <code>Permissions</code>, Elastic Transcoder grants full control over the transcoded files and playlists to the owner of the role specified by <code>Role</code>, and grants no other permissions to any other user or group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageClass</i>  The Amazon S3 storage class, <code>Standard</code> or <code>ReducedRedundancy</code>, that you want Elastic Transcoder to assign to the video files and playlists that it stores in your Amazon S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i>  The Amazon S3 bucket in which you want Elastic Transcoder to save the transcoded files. Specify this value when all of the following are true: <ul> <li> You want to save transcoded files, thumbnails (if any), and playlists (if any) together in one bucket. </li> <li> You do not want to specify the users or groups who have access to the transcoded files, thumbnails, and playlists. </li> <li> You do not want to specify the permissions that Elastic Transcoder grants to the files. </li> <li> You want to associate the transcoded files and thumbnails with the Amazon S3 Standard storage class. </li> </ul> If you want to save transcoded files and playlists in one bucket and thumbnails in another bucket, specify which users can access the transcoded files or the permissions the users have, or change the Amazon S3 storage class, omit OutputBucket and specify values for <code>ContentConfig</code> and <code>ThumbnailConfig</code> instead. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Permissions</i> Optional. The <code>Permissions</code> object specifies which users and/or predefined Amazon S3 groups you want to have access to transcoded files and playlists, and the type of access you want them to have. You can grant permissions to a maximum of 30 users and/or predefined Amazon S3 groups. If you include <code>Permissions</code>, Elastic Transcoder grants only the permissions that you specify. It does not grant full permissions to the owner of the role specified by <code>Role</code>. If you want that user to have full control, you must explicitly grant full control to the user.  If you omit <code>Permissions</code>, Elastic Transcoder grants full control over the transcoded files and playlists to the owner of the role specified by <code>Role</code>, and grants no other permissions to any other user or group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageClass</i>  The Amazon S3 storage class, <code>Standard</code> or <code>ReducedRedundancy</code>, that you want Elastic Transcoder to assign to the video files and playlists that it stores in your Amazon S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i>  The Amazon S3 bucket in which you want Elastic Transcoder to save the transcoded files. Specify this value when all of the following are true: <ul> <li> You want to save transcoded files, thumbnails (if any), and playlists (if any) together in one bucket. </li> <li> You do not want to specify the users or groups who have access to the transcoded files, thumbnails, and playlists. </li> <li> You do not want to specify the permissions that Elastic Transcoder grants to the files. </li> <li> You want to associate the transcoded files and thumbnails with the Amazon S3 Standard storage class. </li> </ul> If you want to save transcoded files and playlists in one bucket and thumbnails in another bucket, specify which users can access the transcoded files or the permissions the users have, or change the Amazon S3 storage class, omit OutputBucket and specify values for <code>ContentConfig</code> and <code>ThumbnailConfig</code> instead. <br/>        """
        pass

    def update_pipeline_notifications(self, Notifications: Notifications, Id: str) -> UpdatePipelineNotificationsResponse:
        """With the UpdatePipelineNotifications operation, you can update Amazon Simple Notification Service (Amazon SNS) notifications for a pipeline. When you update notifications for a pipeline, Elastic Transcoder returns the values that you specified in the request.

        :param self:
        :param Notifications:  The topic ARN for the Amazon Simple Notification Service (Amazon SNS) topic that you want to notify to report job status. <important> To receive notifications, you must also subscribe to the new topic in the Amazon SNS console. </important> <ul> <li>  <b>Progressing</b>: The topic ARN for the Amazon Simple Notification Service (Amazon SNS) topic that you want to notify when Elastic Transcoder has started to process jobs that are added to this pipeline. This is the ARN that Amazon SNS returned when you created the topic. </li> <li>  <b>Complete</b>: The topic ARN for the Amazon SNS topic that you want to notify when Elastic Transcoder has finished processing a job. This is the ARN that Amazon SNS returned when you created the topic. </li> <li>  <b>Warning</b>: The topic ARN for the Amazon SNS topic that you want to notify when Elastic Transcoder encounters a warning condition. This is the ARN that Amazon SNS returned when you created the topic. </li> <li>  <b>Error</b>: The topic ARN for the Amazon SNS topic that you want to notify when Elastic Transcoder encounters an error condition. This is the ARN that Amazon SNS returned when you created the topic. </li> </ul>:<br/>Id:  The identifier of the pipeline for which you want to change notification settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Error</i> The Amazon SNS topic that you want to notify when Elastic Transcoder encounters an error condition.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Warning</i> The Amazon SNS topic that you want to notify when Elastic Transcoder encounters a warning condition.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Completed</i> The Amazon SNS topic that you want to notify when Elastic Transcoder has finished processing the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Progressing</i> The Amazon Simple Notification Service (Amazon SNS) topic that you want to notify when Elastic Transcoder has started to process the job.<br/>
        :return: Pipeline:  A section of the response body that provides information about the pipeline associated with this notification.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThumbnailConfig</i> Information about the Amazon S3 bucket in which you want Elastic Transcoder to save thumbnail files. Either you specify both <code>ContentConfig</code> and <code>ThumbnailConfig</code>, or you specify <code>OutputBucket</code>. <ul> <li>  <code>Bucket</code>: The Amazon S3 bucket in which you want Elastic Transcoder to save thumbnail files.  </li> <li>  <code>Permissions</code>: A list of the users and/or predefined Amazon S3 groups you want to have access to thumbnail files, and the type of access that you want them to have.  <ul> <li> GranteeType: The type of value that appears in the Grantee object: <ul> <li>  <code>Canonical</code>: Either the canonical user ID for an AWS account or an origin access identity for an Amazon CloudFront distribution. <important> A canonical user ID is not the same as an AWS account number. </important> </li> <li>  <code>Email</code>: The registered email address of an AWS account. </li> <li>  <code>Group</code>: One of the following predefined Amazon S3 groups: <code>AllUsers</code>, <code>AuthenticatedUsers</code>, or <code>LogDelivery</code>. </li> </ul> </li> <li>  <code>Grantee</code>: The AWS user or group that you want to have access to thumbnail files. </li> <li> Access: The permission that you want to give to the AWS user that is listed in Grantee. Valid values include:  <ul> <li>  <code>READ</code>: The grantee can read the thumbnails and metadata for thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>READ_ACP</code>: The grantee can read the object ACL for thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>WRITE_ACP</code>: The grantee can write the ACL for the thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>FULL_CONTROL</code>: The grantee has READ, READ_ACP, and WRITE_ACP permissions for the thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> </ul> </li> </ul> </li> <li>  <code>StorageClass</code>: The Amazon S3 storage class, <code>Standard</code> or <code>ReducedRedundancy</code>, that you want Elastic Transcoder to assign to the thumbnails that it stores in your Amazon S3 bucket. </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ThumbnailConfig</b>: Information about the Amazon S3 bucket in which you want Elastic Transcoder to save thumbnail files. Either you specify both <code>ContentConfig</code> and <code>ThumbnailConfig</code>, or you specify <code>OutputBucket</code>. <ul> <li>  <code>Bucket</code>: The Amazon S3 bucket in which you want Elastic Transcoder to save thumbnail files.  </li> <li>  <code>Permissions</code>: A list of the users and/or predefined Amazon S3 groups you want to have access to thumbnail files, and the type of access that you want them to have.  <ul> <li> GranteeType: The type of value that appears in the Grantee object: <ul> <li>  <code>Canonical</code>: Either the canonical user ID for an AWS account or an origin access identity for an Amazon CloudFront distribution. <important> A canonical user ID is not the same as an AWS account number. </important> </li> <li>  <code>Email</code>: The registered email address of an AWS account. </li> <li>  <code>Group</code>: One of the following predefined Amazon S3 groups: <code>AllUsers</code>, <code>AuthenticatedUsers</code>, or <code>LogDelivery</code>. </li> </ul> </li> <li>  <code>Grantee</code>: The AWS user or group that you want to have access to thumbnail files. </li> <li> Access: The permission that you want to give to the AWS user that is listed in Grantee. Valid values include:  <ul> <li>  <code>READ</code>: The grantee can read the thumbnails and metadata for thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>READ_ACP</code>: The grantee can read the object ACL for thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>WRITE_ACP</code>: The grantee can write the ACL for the thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>FULL_CONTROL</code>: The grantee has READ, READ_ACP, and WRITE_ACP permissions for the thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> </ul> </li> </ul> </li> <li>  <code>StorageClass</code>: The Amazon S3 storage class, <code>Standard</code> or <code>ReducedRedundancy</code>, that you want Elastic Transcoder to assign to the thumbnails that it stores in your Amazon S3 bucket. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContentConfig</i> Information about the Amazon S3 bucket in which you want Elastic Transcoder to save transcoded files and playlists. Either you specify both <code>ContentConfig</code> and <code>ThumbnailConfig</code>, or you specify <code>OutputBucket</code>. <ul> <li>  <b>Bucket</b>: The Amazon S3 bucket in which you want Elastic Transcoder to save transcoded files and playlists. </li> <li>  <b>Permissions</b>: A list of the users and/or predefined Amazon S3 groups you want to have access to transcoded files and playlists, and the type of access that you want them to have.  <ul> <li> GranteeType: The type of value that appears in the <code>Grantee</code> object:  <ul> <li>  <code>Canonical</code>: Either the canonical user ID for an AWS account or an origin access identity for an Amazon CloudFront distribution. </li> <li>  <code>Email</code>: The registered email address of an AWS account. </li> <li>  <code>Group</code>: One of the following predefined Amazon S3 groups: <code>AllUsers</code>, <code>AuthenticatedUsers</code>, or <code>LogDelivery</code>. </li> </ul> </li> <li>  <code>Grantee</code>: The AWS user or group that you want to have access to transcoded files and playlists. </li> <li>  <code>Access</code>: The permission that you want to give to the AWS user that is listed in <code>Grantee</code>. Valid values include: <ul> <li>  <code>READ</code>: The grantee can read the objects and metadata for objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>READ_ACP</code>: The grantee can read the object ACL for objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>WRITE_ACP</code>: The grantee can write the ACL for the objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>FULL_CONTROL</code>: The grantee has <code>READ</code>, <code>READ_ACP</code>, and <code>WRITE_ACP</code> permissions for the objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> </ul> </li> </ul> </li> <li>  <b>StorageClass</b>: The Amazon S3 storage class, Standard or ReducedRedundancy, that you want Elastic Transcoder to assign to the video files and playlists that it stores in your Amazon S3 bucket.  </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ContentConfig</b>: Information about the Amazon S3 bucket in which you want Elastic Transcoder to save transcoded files and playlists. Either you specify both <code>ContentConfig</code> and <code>ThumbnailConfig</code>, or you specify <code>OutputBucket</code>. <ul> <li>  <b>Bucket</b>: The Amazon S3 bucket in which you want Elastic Transcoder to save transcoded files and playlists. </li> <li>  <b>Permissions</b>: A list of the users and/or predefined Amazon S3 groups you want to have access to transcoded files and playlists, and the type of access that you want them to have.  <ul> <li> GranteeType: The type of value that appears in the <code>Grantee</code> object:  <ul> <li>  <code>Canonical</code>: Either the canonical user ID for an AWS account or an origin access identity for an Amazon CloudFront distribution. </li> <li>  <code>Email</code>: The registered email address of an AWS account. </li> <li>  <code>Group</code>: One of the following predefined Amazon S3 groups: <code>AllUsers</code>, <code>AuthenticatedUsers</code>, or <code>LogDelivery</code>. </li> </ul> </li> <li>  <code>Grantee</code>: The AWS user or group that you want to have access to transcoded files and playlists. </li> <li>  <code>Access</code>: The permission that you want to give to the AWS user that is listed in <code>Grantee</code>. Valid values include: <ul> <li>  <code>READ</code>: The grantee can read the objects and metadata for objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>READ_ACP</code>: The grantee can read the object ACL for objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>WRITE_ACP</code>: The grantee can write the ACL for the objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>FULL_CONTROL</code>: The grantee has <code>READ</code>, <code>READ_ACP</code>, and <code>WRITE_ACP</code> permissions for the objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> </ul> </li> </ul> </li> <li>  <b>StorageClass</b>: The Amazon S3 storage class, Standard or ReducedRedundancy, that you want Elastic Transcoder to assign to the video files and playlists that it stores in your Amazon S3 bucket.  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Notifications</i> The Amazon Simple Notification Service (Amazon SNS) topic that you want to notify to report job status. <important> To receive notifications, you must also subscribe to the new topic in the Amazon SNS console. </important> <ul> <li>  <b>Progressing</b> (optional): The Amazon Simple Notification Service (Amazon SNS) topic that you want to notify when Elastic Transcoder has started to process the job. </li> <li>  <b>Complete</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder has finished processing the job. </li> <li>  <b>Warning</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder encounters a warning condition. </li> <li>  <b>Error</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder encounters an error condition. </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Notifications</b>: The Amazon Simple Notification Service (Amazon SNS) topic that you want to notify to report job status. <important> To receive notifications, you must also subscribe to the new topic in the Amazon SNS console. </important> <ul> <li>  <b>Progressing</b> (optional): The Amazon Simple Notification Service (Amazon SNS) topic that you want to notify when Elastic Transcoder has started to process the job. </li> <li>  <b>Complete</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder has finished processing the job. </li> <li>  <b>Warning</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder encounters a warning condition. </li> <li>  <b>Error</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder encounters an error condition. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AwsKmsKeyArn</i> The AWS Key Management Service (AWS KMS) key that you want to use with this pipeline. If you use either <code>s3</code> or <code>s3-aws-kms</code> as your <code>Encryption:Mode</code>, you don't need to provide a key with your job because a default key, known as an AWS-KMS key, is created for you automatically. You need to provide an AWS-KMS key only if you want to use a non-default AWS-KMS key, or if you are using an <code>Encryption:Mode</code> of <code>aes-cbc-pkcs7</code>, <code>aes-ctr</code>, or <code>aes-gcm</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Role</i> The IAM Amazon Resource Name (ARN) for the role that Elastic Transcoder uses to transcode jobs for this pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputBucket</i> The Amazon S3 bucket in which you want Elastic Transcoder to save transcoded files, thumbnails, and playlists. Either you specify this value, or you specify both <code>ContentConfig</code> and <code>ThumbnailConfig</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputBucket</i> The Amazon S3 bucket from which Elastic Transcoder gets media files for transcoding and the graphics files, if any, that you want to use for watermarks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current status of the pipeline: <ul> <li>  <code>Active</code>: The pipeline is processing jobs. </li> <li>  <code>Paused</code>: The pipeline is not currently processing jobs. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the pipeline. We recommend that the name be unique within the AWS account, but uniqueness is not enforced. Constraints: Maximum 40 characters<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) for the pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier for the pipeline. You use this value to identify the pipeline in which you want to perform a variety of operations, such as creating a job or a preset.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Error</i> The Amazon SNS topic that you want to notify when Elastic Transcoder encounters an error condition.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Warning</i> The Amazon SNS topic that you want to notify when Elastic Transcoder encounters a warning condition.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Completed</i> The Amazon SNS topic that you want to notify when Elastic Transcoder has finished processing the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Progressing</i> The Amazon Simple Notification Service (Amazon SNS) topic that you want to notify when Elastic Transcoder has started to process the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Permissions</i> Optional. The <code>Permissions</code> object specifies which users and/or predefined Amazon S3 groups you want to have access to transcoded files and playlists, and the type of access you want them to have. You can grant permissions to a maximum of 30 users and/or predefined Amazon S3 groups. If you include <code>Permissions</code>, Elastic Transcoder grants only the permissions that you specify. It does not grant full permissions to the owner of the role specified by <code>Role</code>. If you want that user to have full control, you must explicitly grant full control to the user.  If you omit <code>Permissions</code>, Elastic Transcoder grants full control over the transcoded files and playlists to the owner of the role specified by <code>Role</code>, and grants no other permissions to any other user or group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageClass</i>  The Amazon S3 storage class, <code>Standard</code> or <code>ReducedRedundancy</code>, that you want Elastic Transcoder to assign to the video files and playlists that it stores in your Amazon S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i>  The Amazon S3 bucket in which you want Elastic Transcoder to save the transcoded files. Specify this value when all of the following are true: <ul> <li> You want to save transcoded files, thumbnails (if any), and playlists (if any) together in one bucket. </li> <li> You do not want to specify the users or groups who have access to the transcoded files, thumbnails, and playlists. </li> <li> You do not want to specify the permissions that Elastic Transcoder grants to the files. </li> <li> You want to associate the transcoded files and thumbnails with the Amazon S3 Standard storage class. </li> </ul> If you want to save transcoded files and playlists in one bucket and thumbnails in another bucket, specify which users can access the transcoded files or the permissions the users have, or change the Amazon S3 storage class, omit OutputBucket and specify values for <code>ContentConfig</code> and <code>ThumbnailConfig</code> instead. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Permissions</i> Optional. The <code>Permissions</code> object specifies which users and/or predefined Amazon S3 groups you want to have access to transcoded files and playlists, and the type of access you want them to have. You can grant permissions to a maximum of 30 users and/or predefined Amazon S3 groups. If you include <code>Permissions</code>, Elastic Transcoder grants only the permissions that you specify. It does not grant full permissions to the owner of the role specified by <code>Role</code>. If you want that user to have full control, you must explicitly grant full control to the user.  If you omit <code>Permissions</code>, Elastic Transcoder grants full control over the transcoded files and playlists to the owner of the role specified by <code>Role</code>, and grants no other permissions to any other user or group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageClass</i>  The Amazon S3 storage class, <code>Standard</code> or <code>ReducedRedundancy</code>, that you want Elastic Transcoder to assign to the video files and playlists that it stores in your Amazon S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i>  The Amazon S3 bucket in which you want Elastic Transcoder to save the transcoded files. Specify this value when all of the following are true: <ul> <li> You want to save transcoded files, thumbnails (if any), and playlists (if any) together in one bucket. </li> <li> You do not want to specify the users or groups who have access to the transcoded files, thumbnails, and playlists. </li> <li> You do not want to specify the permissions that Elastic Transcoder grants to the files. </li> <li> You want to associate the transcoded files and thumbnails with the Amazon S3 Standard storage class. </li> </ul> If you want to save transcoded files and playlists in one bucket and thumbnails in another bucket, specify which users can access the transcoded files or the permissions the users have, or change the Amazon S3 storage class, omit OutputBucket and specify values for <code>ContentConfig</code> and <code>ThumbnailConfig</code> instead. <br/>        """
        pass

    def update_pipeline_status(self, Status: str, Id: str) -> UpdatePipelineStatusResponse:
        """The UpdatePipelineStatus operation pauses or reactivates a pipeline, so that the pipeline stops or restarts the processing of jobs. Changing the pipeline status is useful if you want to cancel one or more jobs. You can't cancel jobs after Elastic Transcoder has started processing them; if you pause the pipeline to which you submitted the jobs, you have more time to get the job IDs for the jobs that you want to cancel, and to send a <a>CancelJob</a> request. 

        :param self:
        :param Status:  The desired status of the pipeline: <ul> <li>  <code>Active</code>: The pipeline is processing jobs. </li> <li>  <code>Paused</code>: The pipeline is not currently processing jobs. </li> </ul><br/>Id:  The identifier of the pipeline to update.<br/>
        :return: Pipeline:  A section of the response body that provides information about the pipeline.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThumbnailConfig</i> Information about the Amazon S3 bucket in which you want Elastic Transcoder to save thumbnail files. Either you specify both <code>ContentConfig</code> and <code>ThumbnailConfig</code>, or you specify <code>OutputBucket</code>. <ul> <li>  <code>Bucket</code>: The Amazon S3 bucket in which you want Elastic Transcoder to save thumbnail files.  </li> <li>  <code>Permissions</code>: A list of the users and/or predefined Amazon S3 groups you want to have access to thumbnail files, and the type of access that you want them to have.  <ul> <li> GranteeType: The type of value that appears in the Grantee object: <ul> <li>  <code>Canonical</code>: Either the canonical user ID for an AWS account or an origin access identity for an Amazon CloudFront distribution. <important> A canonical user ID is not the same as an AWS account number. </important> </li> <li>  <code>Email</code>: The registered email address of an AWS account. </li> <li>  <code>Group</code>: One of the following predefined Amazon S3 groups: <code>AllUsers</code>, <code>AuthenticatedUsers</code>, or <code>LogDelivery</code>. </li> </ul> </li> <li>  <code>Grantee</code>: The AWS user or group that you want to have access to thumbnail files. </li> <li> Access: The permission that you want to give to the AWS user that is listed in Grantee. Valid values include:  <ul> <li>  <code>READ</code>: The grantee can read the thumbnails and metadata for thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>READ_ACP</code>: The grantee can read the object ACL for thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>WRITE_ACP</code>: The grantee can write the ACL for the thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>FULL_CONTROL</code>: The grantee has READ, READ_ACP, and WRITE_ACP permissions for the thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> </ul> </li> </ul> </li> <li>  <code>StorageClass</code>: The Amazon S3 storage class, <code>Standard</code> or <code>ReducedRedundancy</code>, that you want Elastic Transcoder to assign to the thumbnails that it stores in your Amazon S3 bucket. </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ThumbnailConfig</b>: Information about the Amazon S3 bucket in which you want Elastic Transcoder to save thumbnail files. Either you specify both <code>ContentConfig</code> and <code>ThumbnailConfig</code>, or you specify <code>OutputBucket</code>. <ul> <li>  <code>Bucket</code>: The Amazon S3 bucket in which you want Elastic Transcoder to save thumbnail files.  </li> <li>  <code>Permissions</code>: A list of the users and/or predefined Amazon S3 groups you want to have access to thumbnail files, and the type of access that you want them to have.  <ul> <li> GranteeType: The type of value that appears in the Grantee object: <ul> <li>  <code>Canonical</code>: Either the canonical user ID for an AWS account or an origin access identity for an Amazon CloudFront distribution. <important> A canonical user ID is not the same as an AWS account number. </important> </li> <li>  <code>Email</code>: The registered email address of an AWS account. </li> <li>  <code>Group</code>: One of the following predefined Amazon S3 groups: <code>AllUsers</code>, <code>AuthenticatedUsers</code>, or <code>LogDelivery</code>. </li> </ul> </li> <li>  <code>Grantee</code>: The AWS user or group that you want to have access to thumbnail files. </li> <li> Access: The permission that you want to give to the AWS user that is listed in Grantee. Valid values include:  <ul> <li>  <code>READ</code>: The grantee can read the thumbnails and metadata for thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>READ_ACP</code>: The grantee can read the object ACL for thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>WRITE_ACP</code>: The grantee can write the ACL for the thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>FULL_CONTROL</code>: The grantee has READ, READ_ACP, and WRITE_ACP permissions for the thumbnails that Elastic Transcoder adds to the Amazon S3 bucket. </li> </ul> </li> </ul> </li> <li>  <code>StorageClass</code>: The Amazon S3 storage class, <code>Standard</code> or <code>ReducedRedundancy</code>, that you want Elastic Transcoder to assign to the thumbnails that it stores in your Amazon S3 bucket. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContentConfig</i> Information about the Amazon S3 bucket in which you want Elastic Transcoder to save transcoded files and playlists. Either you specify both <code>ContentConfig</code> and <code>ThumbnailConfig</code>, or you specify <code>OutputBucket</code>. <ul> <li>  <b>Bucket</b>: The Amazon S3 bucket in which you want Elastic Transcoder to save transcoded files and playlists. </li> <li>  <b>Permissions</b>: A list of the users and/or predefined Amazon S3 groups you want to have access to transcoded files and playlists, and the type of access that you want them to have.  <ul> <li> GranteeType: The type of value that appears in the <code>Grantee</code> object:  <ul> <li>  <code>Canonical</code>: Either the canonical user ID for an AWS account or an origin access identity for an Amazon CloudFront distribution. </li> <li>  <code>Email</code>: The registered email address of an AWS account. </li> <li>  <code>Group</code>: One of the following predefined Amazon S3 groups: <code>AllUsers</code>, <code>AuthenticatedUsers</code>, or <code>LogDelivery</code>. </li> </ul> </li> <li>  <code>Grantee</code>: The AWS user or group that you want to have access to transcoded files and playlists. </li> <li>  <code>Access</code>: The permission that you want to give to the AWS user that is listed in <code>Grantee</code>. Valid values include: <ul> <li>  <code>READ</code>: The grantee can read the objects and metadata for objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>READ_ACP</code>: The grantee can read the object ACL for objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>WRITE_ACP</code>: The grantee can write the ACL for the objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>FULL_CONTROL</code>: The grantee has <code>READ</code>, <code>READ_ACP</code>, and <code>WRITE_ACP</code> permissions for the objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> </ul> </li> </ul> </li> <li>  <b>StorageClass</b>: The Amazon S3 storage class, Standard or ReducedRedundancy, that you want Elastic Transcoder to assign to the video files and playlists that it stores in your Amazon S3 bucket.  </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ContentConfig</b>: Information about the Amazon S3 bucket in which you want Elastic Transcoder to save transcoded files and playlists. Either you specify both <code>ContentConfig</code> and <code>ThumbnailConfig</code>, or you specify <code>OutputBucket</code>. <ul> <li>  <b>Bucket</b>: The Amazon S3 bucket in which you want Elastic Transcoder to save transcoded files and playlists. </li> <li>  <b>Permissions</b>: A list of the users and/or predefined Amazon S3 groups you want to have access to transcoded files and playlists, and the type of access that you want them to have.  <ul> <li> GranteeType: The type of value that appears in the <code>Grantee</code> object:  <ul> <li>  <code>Canonical</code>: Either the canonical user ID for an AWS account or an origin access identity for an Amazon CloudFront distribution. </li> <li>  <code>Email</code>: The registered email address of an AWS account. </li> <li>  <code>Group</code>: One of the following predefined Amazon S3 groups: <code>AllUsers</code>, <code>AuthenticatedUsers</code>, or <code>LogDelivery</code>. </li> </ul> </li> <li>  <code>Grantee</code>: The AWS user or group that you want to have access to transcoded files and playlists. </li> <li>  <code>Access</code>: The permission that you want to give to the AWS user that is listed in <code>Grantee</code>. Valid values include: <ul> <li>  <code>READ</code>: The grantee can read the objects and metadata for objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>READ_ACP</code>: The grantee can read the object ACL for objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>WRITE_ACP</code>: The grantee can write the ACL for the objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> <li>  <code>FULL_CONTROL</code>: The grantee has <code>READ</code>, <code>READ_ACP</code>, and <code>WRITE_ACP</code> permissions for the objects that Elastic Transcoder adds to the Amazon S3 bucket. </li> </ul> </li> </ul> </li> <li>  <b>StorageClass</b>: The Amazon S3 storage class, Standard or ReducedRedundancy, that you want Elastic Transcoder to assign to the video files and playlists that it stores in your Amazon S3 bucket.  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Notifications</i> The Amazon Simple Notification Service (Amazon SNS) topic that you want to notify to report job status. <important> To receive notifications, you must also subscribe to the new topic in the Amazon SNS console. </important> <ul> <li>  <b>Progressing</b> (optional): The Amazon Simple Notification Service (Amazon SNS) topic that you want to notify when Elastic Transcoder has started to process the job. </li> <li>  <b>Complete</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder has finished processing the job. </li> <li>  <b>Warning</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder encounters a warning condition. </li> <li>  <b>Error</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder encounters an error condition. </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Notifications</b>: The Amazon Simple Notification Service (Amazon SNS) topic that you want to notify to report job status. <important> To receive notifications, you must also subscribe to the new topic in the Amazon SNS console. </important> <ul> <li>  <b>Progressing</b> (optional): The Amazon Simple Notification Service (Amazon SNS) topic that you want to notify when Elastic Transcoder has started to process the job. </li> <li>  <b>Complete</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder has finished processing the job. </li> <li>  <b>Warning</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder encounters a warning condition. </li> <li>  <b>Error</b> (optional): The Amazon SNS topic that you want to notify when Elastic Transcoder encounters an error condition. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AwsKmsKeyArn</i> The AWS Key Management Service (AWS KMS) key that you want to use with this pipeline. If you use either <code>s3</code> or <code>s3-aws-kms</code> as your <code>Encryption:Mode</code>, you don't need to provide a key with your job because a default key, known as an AWS-KMS key, is created for you automatically. You need to provide an AWS-KMS key only if you want to use a non-default AWS-KMS key, or if you are using an <code>Encryption:Mode</code> of <code>aes-cbc-pkcs7</code>, <code>aes-ctr</code>, or <code>aes-gcm</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Role</i> The IAM Amazon Resource Name (ARN) for the role that Elastic Transcoder uses to transcode jobs for this pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputBucket</i> The Amazon S3 bucket in which you want Elastic Transcoder to save transcoded files, thumbnails, and playlists. Either you specify this value, or you specify both <code>ContentConfig</code> and <code>ThumbnailConfig</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputBucket</i> The Amazon S3 bucket from which Elastic Transcoder gets media files for transcoding and the graphics files, if any, that you want to use for watermarks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current status of the pipeline: <ul> <li>  <code>Active</code>: The pipeline is processing jobs. </li> <li>  <code>Paused</code>: The pipeline is not currently processing jobs. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the pipeline. We recommend that the name be unique within the AWS account, but uniqueness is not enforced. Constraints: Maximum 40 characters<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) for the pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier for the pipeline. You use this value to identify the pipeline in which you want to perform a variety of operations, such as creating a job or a preset.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Error</i> The Amazon SNS topic that you want to notify when Elastic Transcoder encounters an error condition.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Warning</i> The Amazon SNS topic that you want to notify when Elastic Transcoder encounters a warning condition.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Completed</i> The Amazon SNS topic that you want to notify when Elastic Transcoder has finished processing the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Progressing</i> The Amazon Simple Notification Service (Amazon SNS) topic that you want to notify when Elastic Transcoder has started to process the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Permissions</i> Optional. The <code>Permissions</code> object specifies which users and/or predefined Amazon S3 groups you want to have access to transcoded files and playlists, and the type of access you want them to have. You can grant permissions to a maximum of 30 users and/or predefined Amazon S3 groups. If you include <code>Permissions</code>, Elastic Transcoder grants only the permissions that you specify. It does not grant full permissions to the owner of the role specified by <code>Role</code>. If you want that user to have full control, you must explicitly grant full control to the user.  If you omit <code>Permissions</code>, Elastic Transcoder grants full control over the transcoded files and playlists to the owner of the role specified by <code>Role</code>, and grants no other permissions to any other user or group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageClass</i>  The Amazon S3 storage class, <code>Standard</code> or <code>ReducedRedundancy</code>, that you want Elastic Transcoder to assign to the video files and playlists that it stores in your Amazon S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i>  The Amazon S3 bucket in which you want Elastic Transcoder to save the transcoded files. Specify this value when all of the following are true: <ul> <li> You want to save transcoded files, thumbnails (if any), and playlists (if any) together in one bucket. </li> <li> You do not want to specify the users or groups who have access to the transcoded files, thumbnails, and playlists. </li> <li> You do not want to specify the permissions that Elastic Transcoder grants to the files. </li> <li> You want to associate the transcoded files and thumbnails with the Amazon S3 Standard storage class. </li> </ul> If you want to save transcoded files and playlists in one bucket and thumbnails in another bucket, specify which users can access the transcoded files or the permissions the users have, or change the Amazon S3 storage class, omit OutputBucket and specify values for <code>ContentConfig</code> and <code>ThumbnailConfig</code> instead. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Permissions</i> Optional. The <code>Permissions</code> object specifies which users and/or predefined Amazon S3 groups you want to have access to transcoded files and playlists, and the type of access you want them to have. You can grant permissions to a maximum of 30 users and/or predefined Amazon S3 groups. If you include <code>Permissions</code>, Elastic Transcoder grants only the permissions that you specify. It does not grant full permissions to the owner of the role specified by <code>Role</code>. If you want that user to have full control, you must explicitly grant full control to the user.  If you omit <code>Permissions</code>, Elastic Transcoder grants full control over the transcoded files and playlists to the owner of the role specified by <code>Role</code>, and grants no other permissions to any other user or group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageClass</i>  The Amazon S3 storage class, <code>Standard</code> or <code>ReducedRedundancy</code>, that you want Elastic Transcoder to assign to the video files and playlists that it stores in your Amazon S3 bucket. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i>  The Amazon S3 bucket in which you want Elastic Transcoder to save the transcoded files. Specify this value when all of the following are true: <ul> <li> You want to save transcoded files, thumbnails (if any), and playlists (if any) together in one bucket. </li> <li> You do not want to specify the users or groups who have access to the transcoded files, thumbnails, and playlists. </li> <li> You do not want to specify the permissions that Elastic Transcoder grants to the files. </li> <li> You want to associate the transcoded files and thumbnails with the Amazon S3 Standard storage class. </li> </ul> If you want to save transcoded files and playlists in one bucket and thumbnails in another bucket, specify which users can access the transcoded files or the permissions the users have, or change the Amazon S3 storage class, omit OutputBucket and specify values for <code>ContentConfig</code> and <code>ThumbnailConfig</code> instead. <br/>        """
        pass



class ElasticLoadBalancing(BaseClient):
    """<fullname>Elastic Load Balancing</fullname> A load balancer can distribute incoming traffic across your EC2 instances. This enables you to increase the availability of your application. The load balancer also monitors the health of its registered instances and ensures that it routes traffic only to healthy instances. You configure your load balancer to accept incoming traffic by specifying one or more listeners, which are configured with a protocol and port number for connections from clients to the load balancer and a protocol and port number for connections from the load balancer to the instances. Elastic Load Balancing supports three types of load balancers: Application Load Balancers, Network Load Balancers, and Classic Load Balancers. You can select a load balancer based on your application needs. For more information, see the <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/userguide/">Elastic Load Balancing User Guide</a>. This reference covers the 2012-06-01 API, which supports Classic Load Balancers. The 2015-12-01 API supports Application Load Balancers and Network Load Balancers. To get started, create a load balancer with one or more listeners using <a>CreateLoadBalancer</a>. Register your instances with the load balancer using <a>RegisterInstancesWithLoadBalancer</a>. All Elastic Load Balancing operations are <i>idempotent</i>, which means that they complete at most one time. If you repeat an operation, it succeeds with a 200 OK response code."""

    class AnyInstanceInServiceWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_instance_health` for valid parameters"""
            pass
    class InstanceDeregisteredWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_instance_health` for valid parameters"""
            pass
    class InstanceInServiceWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_instance_health` for valid parameters"""
            pass


    class DescribeLoadBalancersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_load_balancers` for valid parameters"""
            pass
    class DescribeAccountLimitsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_account_limits` for valid parameters"""
            pass


    class AddTagsInput(Mapping):
        pass
    
    class AddTagsOutput(Mapping):
        pass
    
    class AccessPointName(object):
        pass
    
    class Tag(Mapping):
        pass
    
    class ApplySecurityGroupsToLoadBalancerInput(Mapping):
        pass
    
    class ApplySecurityGroupsToLoadBalancerOutput(Mapping):
        pass
    
    class SecurityGroupId(object):
        pass
    
    class AttachLoadBalancerToSubnetsInput(Mapping):
        pass
    
    class AttachLoadBalancerToSubnetsOutput(Mapping):
        pass
    
    class SubnetId(object):
        pass
    
    class ConfigureHealthCheckInput(Mapping):
        pass
    
    class ConfigureHealthCheckOutput(Mapping):
        pass
    
    class HealthCheck(Mapping):
        pass
    
    class CreateAppCookieStickinessPolicyInput(Mapping):
        pass
    
    class CreateAppCookieStickinessPolicyOutput(Mapping):
        pass
    
    class CreateLBCookieStickinessPolicyInput(Mapping):
        pass
    
    class CreateLBCookieStickinessPolicyOutput(Mapping):
        pass
    
    class CookieExpirationPeriod(object):
        pass
    
    class CreateAccessPointInput(Mapping):
        pass
    
    class CreateAccessPointOutput(Mapping):
        pass
    
    class Listener(Mapping):
        pass
    
    class AvailabilityZone(object):
        pass
    
    class CreateLoadBalancerListenerInput(Mapping):
        pass
    
    class CreateLoadBalancerListenerOutput(Mapping):
        pass
    
    class CreateLoadBalancerPolicyInput(Mapping):
        pass
    
    class CreateLoadBalancerPolicyOutput(Mapping):
        pass
    
    class PolicyAttribute(Mapping):
        pass
    
    class DeleteAccessPointInput(Mapping):
        pass
    
    class DeleteAccessPointOutput(Mapping):
        pass
    
    class DeleteLoadBalancerListenerInput(Mapping):
        pass
    
    class DeleteLoadBalancerListenerOutput(Mapping):
        pass
    
    class AccessPointPort(object):
        pass
    
    class DeleteLoadBalancerPolicyInput(Mapping):
        pass
    
    class DeleteLoadBalancerPolicyOutput(Mapping):
        pass
    
    class DeregisterEndPointsInput(Mapping):
        pass
    
    class DeregisterEndPointsOutput(Mapping):
        pass
    
    class Instance(Mapping):
        pass
    
    class DescribeAccountLimitsInput(Mapping):
        pass
    
    class DescribeAccountLimitsOutput(Mapping):
        pass
    
    class DescribeEndPointStateInput(Mapping):
        pass
    
    class DescribeEndPointStateOutput(Mapping):
        pass
    
    class DescribeLoadBalancerAttributesInput(Mapping):
        pass
    
    class DescribeLoadBalancerAttributesOutput(Mapping):
        pass
    
    class DescribeLoadBalancerPoliciesInput(Mapping):
        pass
    
    class DescribeLoadBalancerPoliciesOutput(Mapping):
        pass
    
    class PolicyName(object):
        pass
    
    class DescribeLoadBalancerPolicyTypesInput(Mapping):
        pass
    
    class DescribeLoadBalancerPolicyTypesOutput(Mapping):
        pass
    
    class PolicyTypeName(object):
        pass
    
    class DescribeAccessPointsInput(Mapping):
        pass
    
    class DescribeAccessPointsOutput(Mapping):
        pass
    
    class DescribeTagsInput(Mapping):
        pass
    
    class DescribeTagsOutput(Mapping):
        pass
    
    class DetachLoadBalancerFromSubnetsInput(Mapping):
        pass
    
    class DetachLoadBalancerFromSubnetsOutput(Mapping):
        pass
    
    class RemoveAvailabilityZonesInput(Mapping):
        pass
    
    class RemoveAvailabilityZonesOutput(Mapping):
        pass
    
    class AddAvailabilityZonesInput(Mapping):
        pass
    
    class AddAvailabilityZonesOutput(Mapping):
        pass
    
    class ModifyLoadBalancerAttributesInput(Mapping):
        pass
    
    class ModifyLoadBalancerAttributesOutput(Mapping):
        pass
    
    class LoadBalancerAttributes(Mapping):
        pass
    
    class RegisterEndPointsInput(Mapping):
        pass
    
    class RegisterEndPointsOutput(Mapping):
        pass
    
    class RemoveTagsInput(Mapping):
        pass
    
    class RemoveTagsOutput(Mapping):
        pass
    
    class TagKeyOnly(Mapping):
        pass
    
    class SetLoadBalancerListenerSSLCertificateInput(Mapping):
        pass
    
    class SetLoadBalancerListenerSSLCertificateOutput(Mapping):
        pass
    
    class SetLoadBalancerPoliciesForBackendServerInput(Mapping):
        pass
    
    class SetLoadBalancerPoliciesForBackendServerOutput(Mapping):
        pass
    
    class SetLoadBalancerPoliciesOfListenerInput(Mapping):
        pass
    
    class SetLoadBalancerPoliciesOfListenerOutput(Mapping):
        pass
    
    def add_tags(self, Tags: List[Tag], LoadBalancerNames: List[AccessPointName]) -> AddTagsOutput:
        """Adds the specified tags to the specified load balancer. Each load balancer can have a maximum of 10 tags. Each tag consists of a key and an optional value. If a tag with the same key is already associated with the load balancer, <code>AddTags</code> updates its value. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/add-remove-tags.html">Tag Your Classic Load Balancer</a> in the <i>Classic Load Balancers Guide</i>.

        :param self:
        :param Tags:  The tags.<br/>LoadBalancerNames:  The name of the load balancer. You can specify one load balancer only.<br/>
        :return:         """
        pass

    def apply_security_groups_to_load_balancer(self, SecurityGroups: List[SecurityGroupId], LoadBalancerName: str) -> ApplySecurityGroupsToLoadBalancerOutput:
        """Associates one or more security groups with your load balancer in a virtual private cloud (VPC). The specified security groups override the previously associated security groups. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-security-groups.html#elb-vpc-security-groups">Security Groups for Load Balancers in a VPC</a> in the <i>Classic Load Balancers Guide</i>.

        :param self:
        :param SecurityGroups:  The IDs of the security groups to associate with the load balancer. Note that you cannot specify the name of the security group.<br/>LoadBalancerName:  The name of the load balancer.<br/>
        :return: SecurityGroups:  The IDs of the security groups associated with the load balancer.<br/>        """
        pass

    def attach_load_balancer_to_subnets(self, Subnets: List[SubnetId], LoadBalancerName: str) -> AttachLoadBalancerToSubnetsOutput:
        """Adds one or more subnets to the set of configured subnets for the specified load balancer. The load balancer evenly distributes requests across all registered subnets. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-manage-subnets.html">Add or Remove Subnets for Your Load Balancer in a VPC</a> in the <i>Classic Load Balancers Guide</i>.

        :param self:
        :param Subnets:  The IDs of the subnets to add. You can add only one subnet per Availability Zone.<br/>LoadBalancerName:  The name of the load balancer.<br/>
        :return: Subnets:  The IDs of the subnets attached to the load balancer.<br/>        """
        pass

    def configure_health_check(self, HealthCheck: HealthCheck, LoadBalancerName: str) -> ConfigureHealthCheckOutput:
        """Specifies the health check settings to use when evaluating the health state of your EC2 instances. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-healthchecks.html">Configure Health Checks for Your Load Balancer</a> in the <i>Classic Load Balancers Guide</i>.

        :param self:
        :param HealthCheck:  The configuration information.:<br/>LoadBalancerName:  The name of the load balancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HealthyThreshold</i> The number of consecutive health checks successes required before moving the instance to the <code>Healthy</code> state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UnhealthyThreshold</i> The number of consecutive health check failures required before moving the instance to the <code>Unhealthy</code> state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timeout</i> The amount of time, in seconds, during which no response means a failed health check. This value must be less than the <code>Interval</code> value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Interval</i> The approximate interval, in seconds, between health checks of an individual instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Target</i> The instance being checked. The protocol is either TCP, HTTP, HTTPS, or SSL. The range of valid ports is one (1) through 65535. TCP is the default, specified as a TCP: port pair, for example "TCP:5000". In this case, a health check simply attempts to open a TCP connection to the instance on the specified port. Failure to connect within the configured timeout is considered unhealthy. SSL is also specified as SSL: port pair, for example, SSL:5000. For HTTP/HTTPS, you must include a ping path in the string. HTTP is specified as a HTTP:port;/;PathToPing; grouping, for example "HTTP:80/weather/us/wa/seattle". In this case, a HTTP GET request is issued to the instance on the given port and path. Any answer other than "200 OK" within the timeout period is considered unhealthy. The total length of the HTTP ping target must be 1024 16-bit Unicode characters or less.<br/>
        :return: HealthCheck:  The updated health check.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HealthyThreshold</i> The number of consecutive health checks successes required before moving the instance to the <code>Healthy</code> state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UnhealthyThreshold</i> The number of consecutive health check failures required before moving the instance to the <code>Unhealthy</code> state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timeout</i> The amount of time, in seconds, during which no response means a failed health check. This value must be less than the <code>Interval</code> value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Interval</i> The approximate interval, in seconds, between health checks of an individual instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Target</i> The instance being checked. The protocol is either TCP, HTTP, HTTPS, or SSL. The range of valid ports is one (1) through 65535. TCP is the default, specified as a TCP: port pair, for example "TCP:5000". In this case, a health check simply attempts to open a TCP connection to the instance on the specified port. Failure to connect within the configured timeout is considered unhealthy. SSL is also specified as SSL: port pair, for example, SSL:5000. For HTTP/HTTPS, you must include a ping path in the string. HTTP is specified as a HTTP:port;/;PathToPing; grouping, for example "HTTP:80/weather/us/wa/seattle". In this case, a HTTP GET request is issued to the instance on the given port and path. Any answer other than "200 OK" within the timeout period is considered unhealthy. The total length of the HTTP ping target must be 1024 16-bit Unicode characters or less.<br/>        """
        pass

    def create_app_cookie_stickiness_policy(self, CookieName: str, PolicyName: str, LoadBalancerName: str) -> CreateAppCookieStickinessPolicyOutput:
        """Generates a stickiness policy with sticky session lifetimes that follow that of an application-generated cookie. This policy can be associated only with HTTP/HTTPS listeners. This policy is similar to the policy created by <a>CreateLBCookieStickinessPolicy</a>, except that the lifetime of the special Elastic Load Balancing cookie, <code>AWSELB</code>, follows the lifetime of the application-generated cookie specified in the policy configuration. The load balancer only inserts a new stickiness cookie when the application response includes a new application cookie. If the application cookie is explicitly removed or expires, the session stops being sticky until a new application cookie is issued. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-sticky-sessions.html#enable-sticky-sessions-application">Application-Controlled Session Stickiness</a> in the <i>Classic Load Balancers Guide</i>.

        :param self:
        :param CookieName:  The name of the application cookie used for stickiness.<br/>PolicyName:  The name of the policy being created. Policy names must consist of alphanumeric characters and dashes (-). This name must be unique within the set of policies for this load balancer.<br/>LoadBalancerName:  The name of the load balancer.<br/>
        :return:         """
        pass

    def create_lb_cookie_stickiness_policy(self, PolicyName: str, LoadBalancerName: str, CookieExpirationPeriod: CookieExpirationPeriod=None) -> CreateLBCookieStickinessPolicyOutput:
        """Generates a stickiness policy with sticky session lifetimes controlled by the lifetime of the browser (user-agent) or a specified expiration period. This policy can be associated only with HTTP/HTTPS listeners. When a load balancer implements this policy, the load balancer uses a special cookie to track the instance for each request. When the load balancer receives a request, it first checks to see if this cookie is present in the request. If so, the load balancer sends the request to the application server specified in the cookie. If not, the load balancer sends the request to a server that is chosen based on the existing load-balancing algorithm. A cookie is inserted into the response for binding subsequent requests from the same user to that server. The validity of the cookie is based on the cookie expiration time, which is specified in the policy configuration. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-sticky-sessions.html#enable-sticky-sessions-duration">Duration-Based Session Stickiness</a> in the <i>Classic Load Balancers Guide</i>.

        :param self:
        :param CookieExpirationPeriod:  The time period, in seconds, after which the cookie should be considered stale. If you do not specify this parameter, the default value is 0, which indicates that the sticky session should last for the duration of the browser session.<br/>PolicyName:  The name of the policy being created. Policy names must consist of alphanumeric characters and dashes (-). This name must be unique within the set of policies for this load balancer.<br/>LoadBalancerName:  The name of the load balancer.<br/>
        :return:         """
        pass

    def create_load_balancer(self, Listeners: List[Listener], LoadBalancerName: str, AvailabilityZones: List[AvailabilityZone]=None, Subnets: List[SubnetId]=None, SecurityGroups: List[SecurityGroupId]=None, Scheme: str=None, Tags: List[Tag]=None) -> CreateAccessPointOutput:
        """Creates a Classic Load Balancer. You can add listeners, security groups, subnets, and tags when you create your load balancer, or you can add them later using <a>CreateLoadBalancerListeners</a>, <a>ApplySecurityGroupsToLoadBalancer</a>, <a>AttachLoadBalancerToSubnets</a>, and <a>AddTags</a>. To describe your current load balancers, see <a>DescribeLoadBalancers</a>. When you are finished with a load balancer, you can delete it using <a>DeleteLoadBalancer</a>. You can create up to 20 load balancers per region per account. You can request an increase for the number of load balancers for your account. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-limits.html">Limits for Your Classic Load Balancer</a> in the <i>Classic Load Balancers Guide</i>.

        :param self:
        :param Tags:  A list of tags to assign to the load balancer. For more information about tagging your load balancer, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/add-remove-tags.html">Tag Your Classic Load Balancer</a> in the <i>Classic Load Balancers Guide</i>.<br/>Scheme:  The type of a load balancer. Valid only for load balancers in a VPC. By default, Elastic Load Balancing creates an Internet-facing load balancer with a DNS name that resolves to public IP addresses. For more information about Internet-facing and Internal load balancers, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/userguide/how-elastic-load-balancing-works.html#load-balancer-scheme">Load Balancer Scheme</a> in the <i>Elastic Load Balancing User Guide</i>. Specify <code>internal</code> to create a load balancer with a DNS name that resolves to private IP addresses.<br/>SecurityGroups:  The IDs of the security groups to assign to the load balancer.<br/>Subnets:  The IDs of the subnets in your VPC to attach to the load balancer. Specify one subnet per Availability Zone specified in <code>AvailabilityZones</code>.<br/>AvailabilityZones:  One or more Availability Zones from the same region as the load balancer. You must specify at least one Availability Zone. You can add more Availability Zones after you create the load balancer using <a>EnableAvailabilityZonesForLoadBalancer</a>.<br/>Listeners:  The listeners. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-listener-config.html">Listeners for Your Classic Load Balancer</a> in the <i>Classic Load Balancers Guide</i>.<br/>LoadBalancerName:  The name of the load balancer. This name must be unique within your set of load balancers for the region, must have a maximum of 32 characters, must contain only alphanumeric characters or hyphens, and cannot begin or end with a hyphen.<br/>
        :return: DNSName:  The DNS name of the load balancer.<br/>        """
        pass

    def create_load_balancer_listeners(self, Listeners: List[Listener], LoadBalancerName: str) -> CreateLoadBalancerListenerOutput:
        """Creates one or more listeners for the specified load balancer. If a listener with the specified port does not already exist, it is created; otherwise, the properties of the new listener must match the properties of the existing listener. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-listener-config.html">Listeners for Your Classic Load Balancer</a> in the <i>Classic Load Balancers Guide</i>.

        :param self:
        :param Listeners:  The listeners.<br/>LoadBalancerName:  The name of the load balancer.<br/>
        :return:         """
        pass

    def create_load_balancer_policy(self, PolicyTypeName: str, PolicyName: str, LoadBalancerName: str, PolicyAttributes: List[PolicyAttribute]=None) -> CreateLoadBalancerPolicyOutput:
        """Creates a policy with the specified attributes for the specified load balancer. Policies are settings that are saved for your load balancer and that can be applied to the listener or the application server, depending on the policy type.

        :param self:
        :param PolicyAttributes:  The policy attributes.<br/>PolicyTypeName:  The name of the base policy type. To get the list of policy types, use <a>DescribeLoadBalancerPolicyTypes</a>.<br/>PolicyName:  The name of the load balancer policy to be created. This name must be unique within the set of policies for this load balancer.<br/>LoadBalancerName:  The name of the load balancer.<br/>
        :return:         """
        pass

    def delete_load_balancer(self, LoadBalancerName: str) -> DeleteAccessPointOutput:
        """Deletes the specified load balancer. If you are attempting to recreate a load balancer, you must reconfigure all settings. The DNS name associated with a deleted load balancer are no longer usable. The name and associated DNS record of the deleted load balancer no longer exist and traffic sent to any of its IP addresses is no longer delivered to your instances. If the load balancer does not exist or has already been deleted, the call to <code>DeleteLoadBalancer</code> still succeeds.

        :param self:
        :param LoadBalancerName:  The name of the load balancer.<br/>
        :return:         """
        pass

    def delete_load_balancer_listeners(self, LoadBalancerPorts: List[AccessPointPort], LoadBalancerName: str) -> DeleteLoadBalancerListenerOutput:
        """Deletes the specified listeners from the specified load balancer.

        :param self:
        :param LoadBalancerPorts:  The client port numbers of the listeners.<br/>LoadBalancerName:  The name of the load balancer.<br/>
        :return:         """
        pass

    def delete_load_balancer_policy(self, PolicyName: str, LoadBalancerName: str) -> DeleteLoadBalancerPolicyOutput:
        """Deletes the specified policy from the specified load balancer. This policy must not be enabled for any listeners.

        :param self:
        :param PolicyName:  The name of the policy.<br/>LoadBalancerName:  The name of the load balancer.<br/>
        :return:         """
        pass

    def deregister_instances_from_load_balancer(self, Instances: List[Instance], LoadBalancerName: str) -> DeregisterEndPointsOutput:
        """Deregisters the specified instances from the specified load balancer. After the instance is deregistered, it no longer receives traffic from the load balancer. You can use <a>DescribeLoadBalancers</a> to verify that the instance is deregistered from the load balancer. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-deregister-register-instances.html">Register or De-Register EC2 Instances</a> in the <i>Classic Load Balancers Guide</i>.

        :param self:
        :param Instances:  The IDs of the instances.<br/>LoadBalancerName:  The name of the load balancer.<br/>
        :return: Instances:  The remaining instances registered with the load balancer.<br/>        """
        pass

    def describe_account_limits(self, Marker: str=None, PageSize: int=None) -> DescribeAccountLimitsOutput:
        """Describes the current Elastic Load Balancing resource limits for your AWS account. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-limits.html">Limits for Your Classic Load Balancer</a> in the <i>Classic Load Balancers Guide</i>.

        :param self:
        :param PageSize:  The maximum number of results to return with this call.<br/>Marker:  The marker for the next set of results. (You received this marker from a previous call.)<br/>
        :return: NextMarker:  The marker to use when requesting the next set of results. If there are no additional results, the string is empty.<br/>Limits:  Information about the limits.<br/>        """
        pass

    def describe_instance_health(self, LoadBalancerName: str, Instances: List[Instance]=None) -> DescribeEndPointStateOutput:
        """Describes the state of the specified instances with respect to the specified load balancer. If no instances are specified, the call describes the state of all instances that are currently registered with the load balancer. If instances are specified, their state is returned even if they are no longer registered with the load balancer. The state of terminated instances is not returned.

        :param self:
        :param Instances:  The IDs of the instances.<br/>LoadBalancerName:  The name of the load balancer.<br/>
        :return: InstanceStates:  Information about the health of the instances.<br/>        """
        pass

    def describe_load_balancer_attributes(self, LoadBalancerName: str) -> DescribeLoadBalancerAttributesOutput:
        """Describes the attributes for the specified load balancer.

        :param self:
        :param LoadBalancerName:  The name of the load balancer.<br/>
        :return: LoadBalancerAttributes:  Information about the load balancer attributes.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdditionalAttributes</i> This parameter is reserved.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectionSettings</i> If enabled, the load balancer allows the connections to remain idle (no data is sent over the connection) for the specified duration. By default, Elastic Load Balancing maintains a 60-second idle connection timeout for both front-end and back-end connections of your load balancer. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/config-idle-timeout.html">Configure Idle Connection Timeout</a> in the <i>Classic Load Balancers Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ConnectionSettings</b>: If enabled, the load balancer allows the connections to remain idle (no data is sent over the connection) for the specified duration. By default, Elastic Load Balancing maintains a 60-second idle connection timeout for both front-end and back-end connections of your load balancer. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/config-idle-timeout.html">Configure Idle Connection Timeout</a> in the <i>Classic Load Balancers Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectionDraining</i> If enabled, the load balancer allows existing requests to complete before the load balancer shifts traffic away from a deregistered or unhealthy instance. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/config-conn-drain.html">Configure Connection Draining</a> in the <i>Classic Load Balancers Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ConnectionDraining</b>: If enabled, the load balancer allows existing requests to complete before the load balancer shifts traffic away from a deregistered or unhealthy instance. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/config-conn-drain.html">Configure Connection Draining</a> in the <i>Classic Load Balancers Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccessLog</i> If enabled, the load balancer captures detailed information of all requests and delivers the information to the Amazon S3 bucket that you specify. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/enable-access-logs.html">Enable Access Logs</a> in the <i>Classic Load Balancers Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AccessLog</b>: If enabled, the load balancer captures detailed information of all requests and delivers the information to the Amazon S3 bucket that you specify. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/enable-access-logs.html">Enable Access Logs</a> in the <i>Classic Load Balancers Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CrossZoneLoadBalancing</i> If enabled, the load balancer routes the request traffic evenly across all instances regardless of the Availability Zones. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/enable-disable-crosszone-lb.html">Configure Cross-Zone Load Balancing</a> in the <i>Classic Load Balancers Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CrossZoneLoadBalancing</b>: If enabled, the load balancer routes the request traffic evenly across all instances regardless of the Availability Zones. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/enable-disable-crosszone-lb.html">Configure Cross-Zone Load Balancing</a> in the <i>Classic Load Balancers Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether cross-zone load balancing is enabled for the load balancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3BucketPrefix</i> The logical hierarchy you created for your Amazon S3 bucket, for example <code>my-bucket-prefix/prod</code>. If the prefix is not provided, the log is placed at the root level of the bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EmitInterval</i> The interval for publishing the access logs. You can specify an interval of either 5 minutes or 60 minutes. Default: 60 minutes<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3BucketName</i> The name of the Amazon S3 bucket where the access logs are stored.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether access logs are enabled for the load balancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Timeout</i> The maximum time, in seconds, to keep the existing connections open before deregistering the instances.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether connection draining is enabled for the load balancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IdleTimeout</i> The time, in seconds, that the connection is allowed to be idle (no data has been sent over the connection) before it is closed by the load balancer.<br/>        """
        pass

    def describe_load_balancer_policies(self, LoadBalancerName: str=None, PolicyNames: List[PolicyName]=None) -> DescribeLoadBalancerPoliciesOutput:
        """Describes the specified policies. If you specify a load balancer name, the action returns the descriptions of all policies created for the load balancer. If you specify a policy name associated with your load balancer, the action returns the description of that policy. If you don't specify a load balancer name, the action returns descriptions of the specified sample policies, or descriptions of all sample policies. The names of the sample policies have the <code>ELBSample-</code> prefix.

        :param self:
        :param PolicyNames:  The names of the policies.<br/>LoadBalancerName:  The name of the load balancer.<br/>
        :return: PolicyDescriptions:  Information about the policies.<br/>        """
        pass

    def describe_load_balancer_policy_types(self, PolicyTypeNames: List[PolicyTypeName]=None) -> DescribeLoadBalancerPolicyTypesOutput:
        """Describes the specified load balancer policy types or all load balancer policy types. The description of each type indicates how it can be used. For example, some policies can be used only with layer 7 listeners, some policies can be used only with layer 4 listeners, and some policies can be used only with your EC2 instances. You can use <a>CreateLoadBalancerPolicy</a> to create a policy configuration for any of these policy types. Then, depending on the policy type, use either <a>SetLoadBalancerPoliciesOfListener</a> or <a>SetLoadBalancerPoliciesForBackendServer</a> to set the policy.

        :param self:
        :param PolicyTypeNames:  The names of the policy types. If no names are specified, describes all policy types defined by Elastic Load Balancing.<br/>
        :return: PolicyTypeDescriptions:  Information about the policy types.<br/>        """
        pass

    def describe_load_balancers(self, LoadBalancerNames: List[AccessPointName]=None, Marker: str=None, PageSize: int=None) -> DescribeAccessPointsOutput:
        """Describes the specified the load balancers. If no load balancers are specified, the call describes all of your load balancers.

        :param self:
        :param PageSize:  The maximum number of results to return with this call (a number from 1 to 400). The default is 400.<br/>Marker:  The marker for the next set of results. (You received this marker from a previous call.)<br/>LoadBalancerNames:  The names of the load balancers.<br/>
        :return: NextMarker:  The marker to use when requesting the next set of results. If there are no additional results, the string is empty.<br/>LoadBalancerDescriptions:  Information about the load balancers.<br/>        """
        pass

    def describe_tags(self, LoadBalancerNames: List[AccessPointName]) -> DescribeTagsOutput:
        """Describes the tags associated with the specified load balancers.

        :param self:
        :param LoadBalancerNames:  The names of the load balancers.<br/>
        :return: TagDescriptions:  Information about the tags.<br/>        """
        pass

    def detach_load_balancer_from_subnets(self, Subnets: List[SubnetId], LoadBalancerName: str) -> DetachLoadBalancerFromSubnetsOutput:
        """Removes the specified subnets from the set of configured subnets for the load balancer. After a subnet is removed, all EC2 instances registered with the load balancer in the removed subnet go into the <code>OutOfService</code> state. Then, the load balancer balances the traffic among the remaining routable subnets.

        :param self:
        :param Subnets:  The IDs of the subnets.<br/>LoadBalancerName:  The name of the load balancer.<br/>
        :return: Subnets:  The IDs of the remaining subnets for the load balancer.<br/>        """
        pass

    def disable_availability_zones_for_load_balancer(self, AvailabilityZones: List[AvailabilityZone], LoadBalancerName: str) -> RemoveAvailabilityZonesOutput:
        """Removes the specified Availability Zones from the set of Availability Zones for the specified load balancer in EC2-Classic or a default VPC. For load balancers in a non-default VPC, use <a>DetachLoadBalancerFromSubnets</a>. There must be at least one Availability Zone registered with a load balancer at all times. After an Availability Zone is removed, all instances registered with the load balancer that are in the removed Availability Zone go into the <code>OutOfService</code> state. Then, the load balancer attempts to equally balance the traffic among its remaining Availability Zones. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/enable-disable-az.html">Add or Remove Availability Zones</a> in the <i>Classic Load Balancers Guide</i>.

        :param self:
        :param AvailabilityZones:  The Availability Zones.<br/>LoadBalancerName:  The name of the load balancer.<br/>
        :return: AvailabilityZones:  The remaining Availability Zones for the load balancer.<br/>        """
        pass

    def enable_availability_zones_for_load_balancer(self, AvailabilityZones: List[AvailabilityZone], LoadBalancerName: str) -> AddAvailabilityZonesOutput:
        """Adds the specified Availability Zones to the set of Availability Zones for the specified load balancer in EC2-Classic or a default VPC. For load balancers in a non-default VPC, use <a>AttachLoadBalancerToSubnets</a>. The load balancer evenly distributes requests across all its registered Availability Zones that contain instances. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/enable-disable-az.html">Add or Remove Availability Zones</a> in the <i>Classic Load Balancers Guide</i>.

        :param self:
        :param AvailabilityZones:  The Availability Zones. These must be in the same region as the load balancer.<br/>LoadBalancerName:  The name of the load balancer.<br/>
        :return: AvailabilityZones:  The updated list of Availability Zones for the load balancer.<br/>        """
        pass

    def modify_load_balancer_attributes(self, LoadBalancerAttributes: LoadBalancerAttributes, LoadBalancerName: str) -> ModifyLoadBalancerAttributesOutput:
        """Modifies the attributes of the specified load balancer. You can modify the load balancer attributes, such as <code>AccessLogs</code>, <code>ConnectionDraining</code>, and <code>CrossZoneLoadBalancing</code> by either enabling or disabling them. Or, you can modify the load balancer attribute <code>ConnectionSettings</code> by specifying an idle connection timeout value for your load balancer. For more information, see the following in the <i>Classic Load Balancers Guide</i>: <ul> <li>  <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/enable-disable-crosszone-lb.html">Cross-Zone Load Balancing</a>  </li> <li>  <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/config-conn-drain.html">Connection Draining</a>  </li> <li>  <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/access-log-collection.html">Access Logs</a>  </li> <li>  <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/config-idle-timeout.html">Idle Connection Timeout</a>  </li> </ul>

        :param self:
        :param LoadBalancerAttributes:  The attributes for the load balancer.:<br/>LoadBalancerName:  The name of the load balancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdditionalAttributes</i> This parameter is reserved.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectionSettings</i> If enabled, the load balancer allows the connections to remain idle (no data is sent over the connection) for the specified duration. By default, Elastic Load Balancing maintains a 60-second idle connection timeout for both front-end and back-end connections of your load balancer. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/config-idle-timeout.html">Configure Idle Connection Timeout</a> in the <i>Classic Load Balancers Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ConnectionSettings</b>: If enabled, the load balancer allows the connections to remain idle (no data is sent over the connection) for the specified duration. By default, Elastic Load Balancing maintains a 60-second idle connection timeout for both front-end and back-end connections of your load balancer. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/config-idle-timeout.html">Configure Idle Connection Timeout</a> in the <i>Classic Load Balancers Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectionDraining</i> If enabled, the load balancer allows existing requests to complete before the load balancer shifts traffic away from a deregistered or unhealthy instance. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/config-conn-drain.html">Configure Connection Draining</a> in the <i>Classic Load Balancers Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ConnectionDraining</b>: If enabled, the load balancer allows existing requests to complete before the load balancer shifts traffic away from a deregistered or unhealthy instance. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/config-conn-drain.html">Configure Connection Draining</a> in the <i>Classic Load Balancers Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccessLog</i> If enabled, the load balancer captures detailed information of all requests and delivers the information to the Amazon S3 bucket that you specify. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/enable-access-logs.html">Enable Access Logs</a> in the <i>Classic Load Balancers Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AccessLog</b>: If enabled, the load balancer captures detailed information of all requests and delivers the information to the Amazon S3 bucket that you specify. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/enable-access-logs.html">Enable Access Logs</a> in the <i>Classic Load Balancers Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CrossZoneLoadBalancing</i> If enabled, the load balancer routes the request traffic evenly across all instances regardless of the Availability Zones. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/enable-disable-crosszone-lb.html">Configure Cross-Zone Load Balancing</a> in the <i>Classic Load Balancers Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CrossZoneLoadBalancing</b>: If enabled, the load balancer routes the request traffic evenly across all instances regardless of the Availability Zones. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/enable-disable-crosszone-lb.html">Configure Cross-Zone Load Balancing</a> in the <i>Classic Load Balancers Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether cross-zone load balancing is enabled for the load balancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3BucketPrefix</i> The logical hierarchy you created for your Amazon S3 bucket, for example <code>my-bucket-prefix/prod</code>. If the prefix is not provided, the log is placed at the root level of the bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EmitInterval</i> The interval for publishing the access logs. You can specify an interval of either 5 minutes or 60 minutes. Default: 60 minutes<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3BucketName</i> The name of the Amazon S3 bucket where the access logs are stored.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether access logs are enabled for the load balancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Timeout</i> The maximum time, in seconds, to keep the existing connections open before deregistering the instances.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether connection draining is enabled for the load balancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IdleTimeout</i> The time, in seconds, that the connection is allowed to be idle (no data has been sent over the connection) before it is closed by the load balancer.<br/>
        :return: LoadBalancerAttributes:  Information about the load balancer attributes.:<br/>LoadBalancerName:  The name of the load balancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdditionalAttributes</i> This parameter is reserved.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectionSettings</i> If enabled, the load balancer allows the connections to remain idle (no data is sent over the connection) for the specified duration. By default, Elastic Load Balancing maintains a 60-second idle connection timeout for both front-end and back-end connections of your load balancer. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/config-idle-timeout.html">Configure Idle Connection Timeout</a> in the <i>Classic Load Balancers Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ConnectionSettings</b>: If enabled, the load balancer allows the connections to remain idle (no data is sent over the connection) for the specified duration. By default, Elastic Load Balancing maintains a 60-second idle connection timeout for both front-end and back-end connections of your load balancer. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/config-idle-timeout.html">Configure Idle Connection Timeout</a> in the <i>Classic Load Balancers Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectionDraining</i> If enabled, the load balancer allows existing requests to complete before the load balancer shifts traffic away from a deregistered or unhealthy instance. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/config-conn-drain.html">Configure Connection Draining</a> in the <i>Classic Load Balancers Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ConnectionDraining</b>: If enabled, the load balancer allows existing requests to complete before the load balancer shifts traffic away from a deregistered or unhealthy instance. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/config-conn-drain.html">Configure Connection Draining</a> in the <i>Classic Load Balancers Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccessLog</i> If enabled, the load balancer captures detailed information of all requests and delivers the information to the Amazon S3 bucket that you specify. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/enable-access-logs.html">Enable Access Logs</a> in the <i>Classic Load Balancers Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AccessLog</b>: If enabled, the load balancer captures detailed information of all requests and delivers the information to the Amazon S3 bucket that you specify. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/enable-access-logs.html">Enable Access Logs</a> in the <i>Classic Load Balancers Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CrossZoneLoadBalancing</i> If enabled, the load balancer routes the request traffic evenly across all instances regardless of the Availability Zones. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/enable-disable-crosszone-lb.html">Configure Cross-Zone Load Balancing</a> in the <i>Classic Load Balancers Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CrossZoneLoadBalancing</b>: If enabled, the load balancer routes the request traffic evenly across all instances regardless of the Availability Zones. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/enable-disable-crosszone-lb.html">Configure Cross-Zone Load Balancing</a> in the <i>Classic Load Balancers Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether cross-zone load balancing is enabled for the load balancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3BucketPrefix</i> The logical hierarchy you created for your Amazon S3 bucket, for example <code>my-bucket-prefix/prod</code>. If the prefix is not provided, the log is placed at the root level of the bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EmitInterval</i> The interval for publishing the access logs. You can specify an interval of either 5 minutes or 60 minutes. Default: 60 minutes<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3BucketName</i> The name of the Amazon S3 bucket where the access logs are stored.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether access logs are enabled for the load balancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Timeout</i> The maximum time, in seconds, to keep the existing connections open before deregistering the instances.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether connection draining is enabled for the load balancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IdleTimeout</i> The time, in seconds, that the connection is allowed to be idle (no data has been sent over the connection) before it is closed by the load balancer.<br/>        """
        pass

    def register_instances_with_load_balancer(self, Instances: List[Instance], LoadBalancerName: str) -> RegisterEndPointsOutput:
        """Adds the specified instances to the specified load balancer. The instance must be a running instance in the same network as the load balancer (EC2-Classic or the same VPC). If you have EC2-Classic instances and a load balancer in a VPC with ClassicLink enabled, you can link the EC2-Classic instances to that VPC and then register the linked EC2-Classic instances with the load balancer in the VPC. Note that <code>RegisterInstanceWithLoadBalancer</code> completes when the request has been registered. Instance registration takes a little time to complete. To check the state of the registered instances, use <a>DescribeLoadBalancers</a> or <a>DescribeInstanceHealth</a>. After the instance is registered, it starts receiving traffic and requests from the load balancer. Any instance that is not in one of the Availability Zones registered for the load balancer is moved to the <code>OutOfService</code> state. If an Availability Zone is added to the load balancer later, any instances registered with the load balancer move to the <code>InService</code> state. To deregister instances from a load balancer, use <a>DeregisterInstancesFromLoadBalancer</a>. For more information, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-deregister-register-instances.html">Register or De-Register EC2 Instances</a> in the <i>Classic Load Balancers Guide</i>.

        :param self:
        :param Instances:  The IDs of the instances.<br/>LoadBalancerName:  The name of the load balancer.<br/>
        :return: Instances:  The updated list of instances for the load balancer.<br/>        """
        pass

    def remove_tags(self, Tags: List[TagKeyOnly], LoadBalancerNames: List[AccessPointName]) -> RemoveTagsOutput:
        """Removes one or more tags from the specified load balancer.

        :param self:
        :param Tags:  The list of tag keys to remove.<br/>LoadBalancerNames:  The name of the load balancer. You can specify a maximum of one load balancer name.<br/>
        :return:         """
        pass

    def set_load_balancer_listener_ssl_certificate(self, SSLCertificateId: str, LoadBalancerPort: int, LoadBalancerName: str) -> SetLoadBalancerListenerSSLCertificateOutput:
        """Sets the certificate that terminates the specified listener's SSL connections. The specified certificate replaces any prior certificate that was used on the same load balancer and port. For more information about updating your SSL certificate, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-update-ssl-cert.html">Replace the SSL Certificate for Your Load Balancer</a> in the <i>Classic Load Balancers Guide</i>.

        :param self:
        :param SSLCertificateId:  The Amazon Resource Name (ARN) of the SSL certificate.<br/>LoadBalancerPort:  The port that uses the specified SSL certificate.<br/>LoadBalancerName:  The name of the load balancer.<br/>
        :return:         """
        pass

    def set_load_balancer_policies_for_backend_server(self, PolicyNames: List[PolicyName], InstancePort: int, LoadBalancerName: str) -> SetLoadBalancerPoliciesForBackendServerOutput:
        """Replaces the set of policies associated with the specified port on which the EC2 instance is listening with a new set of policies. At this time, only the back-end server authentication policy type can be applied to the instance ports; this policy type is composed of multiple public key policies. Each time you use <code>SetLoadBalancerPoliciesForBackendServer</code> to enable the policies, use the <code>PolicyNames</code> parameter to list the policies that you want to enable. You can use <a>DescribeLoadBalancers</a> or <a>DescribeLoadBalancerPolicies</a> to verify that the policy is associated with the EC2 instance. For more information about enabling back-end instance authentication, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-create-https-ssl-load-balancer.html#configure_backendauth_clt">Configure Back-end Instance Authentication</a> in the <i>Classic Load Balancers Guide</i>. For more information about Proxy Protocol, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/enable-proxy-protocol.html">Configure Proxy Protocol Support</a> in the <i>Classic Load Balancers Guide</i>.

        :param self:
        :param PolicyNames:  The names of the policies. If the list is empty, then all current polices are removed from the EC2 instance.<br/>InstancePort:  The port number associated with the EC2 instance.<br/>LoadBalancerName:  The name of the load balancer.<br/>
        :return:         """
        pass

    def set_load_balancer_policies_of_listener(self, PolicyNames: List[PolicyName], LoadBalancerPort: int, LoadBalancerName: str) -> SetLoadBalancerPoliciesOfListenerOutput:
        """Replaces the current set of policies for the specified load balancer port with the specified set of policies. To enable back-end server authentication, use <a>SetLoadBalancerPoliciesForBackendServer</a>. For more information about setting policies, see <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/ssl-config-update.html">Update the SSL Negotiation Configuration</a>, <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-sticky-sessions.html#enable-sticky-sessions-duration">Duration-Based Session Stickiness</a>, and <a href="http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-sticky-sessions.html#enable-sticky-sessions-application">Application-Controlled Session Stickiness</a> in the <i>Classic Load Balancers Guide</i>.

        :param self:
        :param PolicyNames:  The names of the policies. This list must include all policies to be enabled. If you omit a policy that is currently enabled, it is disabled. If the list is empty, all current policies are disabled.<br/>LoadBalancerPort:  The external port of the load balancer.<br/>LoadBalancerName:  The name of the load balancer.<br/>
        :return:         """
        pass



class ElasticLoadBalancingv2(BaseClient):
    """<fullname>Elastic Load Balancing</fullname> A load balancer distributes incoming traffic across targets, such as your EC2 instances. This enables you to increase the availability of your application. The load balancer also monitors the health of its registered targets and ensures that it routes traffic only to healthy targets. You configure your load balancer to accept incoming traffic by specifying one or more listeners, which are configured with a protocol and port number for connections from clients to the load balancer. You configure a target group with a protocol and port number for connections from the load balancer to the targets, and with health check settings to be used when checking the health status of the targets. Elastic Load Balancing supports the following types of load balancers: Application Load Balancers, Network Load Balancers, and Classic Load Balancers. An Application Load Balancer makes routing and load balancing decisions at the application layer (HTTP/HTTPS). A Network Load Balancer makes routing and load balancing decisions at the transport layer (TCP/TLS). Both Application Load Balancers and Network Load Balancers can route requests to one or more ports on each EC2 instance or container instance in your virtual private cloud (VPC). A Classic Load Balancer makes routing and load balancing decisions either at the transport layer (TCP/SSL) or the application layer (HTTP/HTTPS), and supports either EC2-Classic or a VPC. For more information, see the <a href="https://docs.aws.amazon.com/elasticloadbalancing/latest/userguide/">Elastic Load Balancing User Guide</a>. This reference covers the 2015-12-01 API, which supports Application Load Balancers and Network Load Balancers. The 2012-06-01 API supports Classic Load Balancers. To get started, complete the following tasks: <ol> <li> Create a load balancer using <a>CreateLoadBalancer</a>. </li> <li> Create a target group using <a>CreateTargetGroup</a>. </li> <li> Register targets for the target group using <a>RegisterTargets</a>. </li> <li> Create one or more listeners for your load balancer using <a>CreateListener</a>. </li> </ol> To delete a load balancer and its related resources, complete the following tasks: <ol> <li> Delete the load balancer using <a>DeleteLoadBalancer</a>. </li> <li> Delete the target group using <a>DeleteTargetGroup</a>. </li> </ol> All Elastic Load Balancing operations are idempotent, which means that they complete at most one time. If you repeat an operation, it succeeds."""

    class LoadBalancerAvailableWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_load_balancers` for valid parameters"""
            pass
    class LoadBalancerExistsWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_load_balancers` for valid parameters"""
            pass
    class LoadBalancersDeletedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_load_balancers` for valid parameters"""
            pass
    class TargetDeregisteredWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_target_health` for valid parameters"""
            pass
    class TargetInServiceWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_target_health` for valid parameters"""
            pass


    class DescribeLoadBalancersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_load_balancers` for valid parameters"""
            pass
    class DescribeTargetGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_target_groups` for valid parameters"""
            pass
    class DescribeListenersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_listeners` for valid parameters"""
            pass
    class DescribeAccountLimitsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_account_limits` for valid parameters"""
            pass
    class DescribeListenerCertificatesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_listener_certificates` for valid parameters"""
            pass
    class DescribeRulesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_rules` for valid parameters"""
            pass
    class DescribeSSLPoliciesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_ssl_policies` for valid parameters"""
            pass


    class AddListenerCertificatesInput(Mapping):
        pass
    
    class AddListenerCertificatesOutput(Mapping):
        pass
    
    class Certificate(Mapping):
        pass
    
    class AddTagsInput(Mapping):
        pass
    
    class AddTagsOutput(Mapping):
        pass
    
    class ResourceArn(object):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreateListenerInput(Mapping):
        pass
    
    class CreateListenerOutput(Mapping):
        pass
    
    class Action(Mapping):
        pass
    
    class CreateLoadBalancerInput(Mapping):
        pass
    
    class CreateLoadBalancerOutput(Mapping):
        pass
    
    class SubnetId(object):
        pass
    
    class SubnetMapping(Mapping):
        pass
    
    class SecurityGroupId(object):
        pass
    
    class CreateRuleInput(Mapping):
        pass
    
    class CreateRuleOutput(Mapping):
        pass
    
    class RuleCondition(Mapping):
        pass
    
    class CreateTargetGroupInput(Mapping):
        pass
    
    class CreateTargetGroupOutput(Mapping):
        pass
    
    class HealthCheckEnabled(object):
        pass
    
    class Matcher(Mapping):
        pass
    
    class DeleteListenerInput(Mapping):
        pass
    
    class DeleteListenerOutput(Mapping):
        pass
    
    class DeleteLoadBalancerInput(Mapping):
        pass
    
    class DeleteLoadBalancerOutput(Mapping):
        pass
    
    class DeleteRuleInput(Mapping):
        pass
    
    class DeleteRuleOutput(Mapping):
        pass
    
    class DeleteTargetGroupInput(Mapping):
        pass
    
    class DeleteTargetGroupOutput(Mapping):
        pass
    
    class DeregisterTargetsInput(Mapping):
        pass
    
    class DeregisterTargetsOutput(Mapping):
        pass
    
    class TargetDescription(Mapping):
        pass
    
    class DescribeAccountLimitsInput(Mapping):
        pass
    
    class DescribeAccountLimitsOutput(Mapping):
        pass
    
    class DescribeListenerCertificatesInput(Mapping):
        pass
    
    class DescribeListenerCertificatesOutput(Mapping):
        pass
    
    class DescribeListenersInput(Mapping):
        pass
    
    class DescribeListenersOutput(Mapping):
        pass
    
    class ListenerArn(object):
        pass
    
    class DescribeLoadBalancerAttributesInput(Mapping):
        pass
    
    class DescribeLoadBalancerAttributesOutput(Mapping):
        pass
    
    class DescribeLoadBalancersInput(Mapping):
        pass
    
    class DescribeLoadBalancersOutput(Mapping):
        pass
    
    class LoadBalancerArn(object):
        pass
    
    class LoadBalancerName(object):
        pass
    
    class DescribeRulesInput(Mapping):
        pass
    
    class DescribeRulesOutput(Mapping):
        pass
    
    class RuleArn(object):
        pass
    
    class DescribeSSLPoliciesInput(Mapping):
        pass
    
    class DescribeSSLPoliciesOutput(Mapping):
        pass
    
    class SslPolicyName(object):
        pass
    
    class DescribeTagsInput(Mapping):
        pass
    
    class DescribeTagsOutput(Mapping):
        pass
    
    class DescribeTargetGroupAttributesInput(Mapping):
        pass
    
    class DescribeTargetGroupAttributesOutput(Mapping):
        pass
    
    class DescribeTargetGroupsInput(Mapping):
        pass
    
    class DescribeTargetGroupsOutput(Mapping):
        pass
    
    class TargetGroupArn(object):
        pass
    
    class TargetGroupName(object):
        pass
    
    class DescribeTargetHealthInput(Mapping):
        pass
    
    class DescribeTargetHealthOutput(Mapping):
        pass
    
    class ModifyListenerInput(Mapping):
        pass
    
    class ModifyListenerOutput(Mapping):
        pass
    
    class ModifyLoadBalancerAttributesInput(Mapping):
        pass
    
    class ModifyLoadBalancerAttributesOutput(Mapping):
        pass
    
    class LoadBalancerAttribute(Mapping):
        pass
    
    class ModifyRuleInput(Mapping):
        pass
    
    class ModifyRuleOutput(Mapping):
        pass
    
    class ModifyTargetGroupInput(Mapping):
        pass
    
    class ModifyTargetGroupOutput(Mapping):
        pass
    
    class ModifyTargetGroupAttributesInput(Mapping):
        pass
    
    class ModifyTargetGroupAttributesOutput(Mapping):
        pass
    
    class TargetGroupAttribute(Mapping):
        pass
    
    class RegisterTargetsInput(Mapping):
        pass
    
    class RegisterTargetsOutput(Mapping):
        pass
    
    class RemoveListenerCertificatesInput(Mapping):
        pass
    
    class RemoveListenerCertificatesOutput(Mapping):
        pass
    
    class RemoveTagsInput(Mapping):
        pass
    
    class RemoveTagsOutput(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class SetIpAddressTypeInput(Mapping):
        pass
    
    class SetIpAddressTypeOutput(Mapping):
        pass
    
    class SetRulePrioritiesInput(Mapping):
        pass
    
    class SetRulePrioritiesOutput(Mapping):
        pass
    
    class RulePriorityPair(Mapping):
        pass
    
    class SetSecurityGroupsInput(Mapping):
        pass
    
    class SetSecurityGroupsOutput(Mapping):
        pass
    
    class SetSubnetsInput(Mapping):
        pass
    
    class SetSubnetsOutput(Mapping):
        pass
    
    def add_listener_certificates(self, Certificates: List[Certificate], ListenerArn: str) -> AddListenerCertificatesOutput:
        """Adds the specified SSL server certificate to the certificate list for the specified HTTPS listener. If the certificate in already in the certificate list, the call is successful but the certificate is not added again. To get the certificate list for a listener, use <a>DescribeListenerCertificates</a>. To remove certificates from the certificate list for a listener, use <a>RemoveListenerCertificates</a>. To replace the default certificate for a listener, use <a>ModifyListener</a>. For more information, see <a href="https://docs.aws.amazon.com/elasticloadbalancing/latest/application/create-https-listener.html#https-listener-certificates">SSL Certificates</a> in the <i>Application Load Balancers Guide</i>.

        :param self:
        :param Certificates:  The certificate to add. You can specify one certificate per call. Set <code>CertificateArn</code> to the certificate ARN but do not set <code>IsDefault</code>.<br/>ListenerArn:  The Amazon Resource Name (ARN) of the listener.<br/>
        :return: Certificates:  Information about the certificates in the certificate list.<br/>        """
        pass

    def add_tags(self, Tags: List[Tag], ResourceArns: List[ResourceArn]) -> AddTagsOutput:
        """Adds the specified tags to the specified Elastic Load Balancing resource. You can tag your Application Load Balancers, Network Load Balancers, and your target groups. Each tag consists of a key and an optional value. If a resource already has a tag with the same key, <code>AddTags</code> updates its value. To list the current tags for your resources, use <a>DescribeTags</a>. To remove tags from your resources, use <a>RemoveTags</a>.

        :param self:
        :param Tags:  The tags. Each resource can have a maximum of 10 tags.<br/>ResourceArns:  The Amazon Resource Name (ARN) of the resource.<br/>
        :return:         """
        pass

    def create_listener(self, DefaultActions: List[Action], Port: int, Protocol: str, LoadBalancerArn: str, SslPolicy: str=None, Certificates: List[Certificate]=None) -> CreateListenerOutput:
        """Creates a listener for the specified Application Load Balancer or Network Load Balancer. To update a listener, use <a>ModifyListener</a>. When you are finished with a listener, you can delete it using <a>DeleteListener</a>. If you are finished with both the listener and the load balancer, you can delete them both using <a>DeleteLoadBalancer</a>. This operation is idempotent, which means that it completes at most one time. If you attempt to create multiple listeners with the same settings, each call succeeds. For more information, see <a href="https://docs.aws.amazon.com/elasticloadbalancing/latest/application/load-balancer-listeners.html">Listeners for Your Application Load Balancers</a> in the <i>Application Load Balancers Guide</i> and <a href="https://docs.aws.amazon.com/elasticloadbalancing/latest/network/load-balancer-listeners.html">Listeners for Your Network Load Balancers</a> in the <i>Network Load Balancers Guide</i>.

        :param self:
        :param DefaultActions:  The actions for the default rule. The rule must include one forward action or one or more fixed-response actions. If the action type is <code>forward</code>, you specify a target group. The protocol of the target group must be HTTP or HTTPS for an Application Load Balancer. The protocol of the target group must be TCP, TLS, UDP, or TCP_UDP for a Network Load Balancer. [HTTPS listeners] If the action type is <code>authenticate-oidc</code>, you authenticate users through an identity provider that is OpenID Connect (OIDC) compliant. [HTTPS listeners] If the action type is <code>authenticate-cognito</code>, you authenticate users through the user pools supported by Amazon Cognito. [Application Load Balancer] If the action type is <code>redirect</code>, you redirect specified client requests from one URL to another. [Application Load Balancer] If the action type is <code>fixed-response</code>, you drop specified client requests and return a custom HTTP response.<br/>Certificates:  [HTTPS and TLS listeners] The default certificate for the listener. You must provide exactly one certificate. Set <code>CertificateArn</code> to the certificate ARN but do not set <code>IsDefault</code>. To create a certificate list for the listener, use <a>AddListenerCertificates</a>.<br/>SslPolicy:  [HTTPS and TLS listeners] The security policy that defines which ciphers and protocols are supported. The default is the current predefined security policy.<br/>Port:  The port on which the load balancer is listening.<br/>Protocol:  The protocol for connections from clients to the load balancer. For Application Load Balancers, the supported protocols are HTTP and HTTPS. For Network Load Balancers, the supported protocols are TCP, TLS, UDP, and TCP_UDP.<br/>LoadBalancerArn:  The Amazon Resource Name (ARN) of the load balancer.<br/>
        :return: Listeners:  Information about the listener.<br/>        """
        pass

    def create_load_balancer(self, Name: str, Subnets: List[SubnetId]=None, SubnetMappings: List[SubnetMapping]=None, SecurityGroups: List[SecurityGroupId]=None, Scheme: str=None, Tags: List[Tag]=None, Type: str=None, IpAddressType: str=None) -> CreateLoadBalancerOutput:
        """Creates an Application Load Balancer or a Network Load Balancer. When you create a load balancer, you can specify security groups, public subnets, IP address type, and tags. Otherwise, you could do so later using <a>SetSecurityGroups</a>, <a>SetSubnets</a>, <a>SetIpAddressType</a>, and <a>AddTags</a>. To create listeners for your load balancer, use <a>CreateListener</a>. To describe your current load balancers, see <a>DescribeLoadBalancers</a>. When you are finished with a load balancer, you can delete it using <a>DeleteLoadBalancer</a>. For limit information, see <a href="https://docs.aws.amazon.com/elasticloadbalancing/latest/application/load-balancer-limits.html">Limits for Your Application Load Balancer</a> in the <i>Application Load Balancers Guide</i> and <a href="https://docs.aws.amazon.com/elasticloadbalancing/latest/network/load-balancer-limits.html">Limits for Your Network Load Balancer</a> in the <i>Network Load Balancers Guide</i>. This operation is idempotent, which means that it completes at most one time. If you attempt to create multiple load balancers with the same settings, each call succeeds. For more information, see <a href="https://docs.aws.amazon.com/elasticloadbalancing/latest/application/application-load-balancers.html">Application Load Balancers</a> in the <i>Application Load Balancers Guide</i> and <a href="https://docs.aws.amazon.com/elasticloadbalancing/latest/network/network-load-balancers.html">Network Load Balancers</a> in the <i>Network Load Balancers Guide</i>.

        :param self:
        :param IpAddressType:  [Application Load Balancers] The type of IP addresses used by the subnets for your load balancer. The possible values are <code>ipv4</code> (for IPv4 addresses) and <code>dualstack</code> (for IPv4 and IPv6 addresses). Internal load balancers must use <code>ipv4</code>.<br/>Type:  The type of load balancer. The default is <code>application</code>.<br/>Tags:  One or more tags to assign to the load balancer.<br/>Scheme:  The nodes of an Internet-facing load balancer have public IP addresses. The DNS name of an Internet-facing load balancer is publicly resolvable to the public IP addresses of the nodes. Therefore, Internet-facing load balancers can route requests from clients over the internet. The nodes of an internal load balancer have only private IP addresses. The DNS name of an internal load balancer is publicly resolvable to the private IP addresses of the nodes. Therefore, internal load balancers can only route requests from clients with access to the VPC for the load balancer. The default is an Internet-facing load balancer.<br/>SecurityGroups:  [Application Load Balancers] The IDs of the security groups for the load balancer.<br/>SubnetMappings:  The IDs of the public subnets. You can specify only one subnet per Availability Zone. You must specify either subnets or subnet mappings. [Application Load Balancers] You must specify subnets from at least two Availability Zones. You cannot specify Elastic IP addresses for your subnets. [Network Load Balancers] You can specify subnets from one or more Availability Zones. You can specify one Elastic IP address per subnet.<br/>Subnets:  The IDs of the public subnets. You can specify only one subnet per Availability Zone. You must specify either subnets or subnet mappings. [Application Load Balancers] You must specify subnets from at least two Availability Zones. [Network Load Balancers] You can specify subnets from one or more Availability Zones.<br/>Name:  The name of the load balancer. This name must be unique per region per account, can have a maximum of 32 characters, must contain only alphanumeric characters or hyphens, must not begin or end with a hyphen, and must not begin with "internal-".<br/>
        :return: LoadBalancers:  Information about the load balancer.<br/>        """
        pass

    def create_rule(self, Actions: List[Action], Priority: int, Conditions: List[RuleCondition], ListenerArn: str) -> CreateRuleOutput:
        """Creates a rule for the specified listener. The listener must be associated with an Application Load Balancer. Rules are evaluated in priority order, from the lowest value to the highest value. When the conditions for a rule are met, its actions are performed. If the conditions for no rules are met, the actions for the default rule are performed. For more information, see <a href="https://docs.aws.amazon.com/elasticloadbalancing/latest/application/load-balancer-listeners.html#listener-rules">Listener Rules</a> in the <i>Application Load Balancers Guide</i>. To view your current rules, use <a>DescribeRules</a>. To update a rule, use <a>ModifyRule</a>. To set the priorities of your rules, use <a>SetRulePriorities</a>. To delete a rule, use <a>DeleteRule</a>.

        :param self:
        :param Actions:  The actions. Each rule must include exactly one of the following types of actions: <code>forward</code>, <code>fixed-response</code>, or <code>redirect</code>. If the action type is <code>forward</code>, you specify a target group. The protocol of the target group must be HTTP or HTTPS for an Application Load Balancer. The protocol of the target group must be TCP, TLS, UDP, or TCP_UDP for a Network Load Balancer. [HTTPS listeners] If the action type is <code>authenticate-oidc</code>, you authenticate users through an identity provider that is OpenID Connect (OIDC) compliant. [HTTPS listeners] If the action type is <code>authenticate-cognito</code>, you authenticate users through the user pools supported by Amazon Cognito. [Application Load Balancer] If the action type is <code>redirect</code>, you redirect specified client requests from one URL to another. [Application Load Balancer] If the action type is <code>fixed-response</code>, you drop specified client requests and return a custom HTTP response.<br/>Priority:  The rule priority. A listener can't have multiple rules with the same priority.<br/>Conditions:  The conditions. Each rule can include zero or one of the following conditions: <code>http-request-method</code>, <code>host-header</code>, <code>path-pattern</code>, and <code>source-ip</code>, and zero or more of the following conditions: <code>http-header</code> and <code>query-string</code>.<br/>ListenerArn:  The Amazon Resource Name (ARN) of the listener.<br/>
        :return: Rules:  Information about the rule.<br/>        """
        pass

    def create_target_group(self, Name: str, Protocol: str=None, Port: int=None, VpcId: str=None, HealthCheckProtocol: str=None, HealthCheckPort: str=None, HealthCheckEnabled: HealthCheckEnabled=None, HealthCheckPath: str=None, HealthCheckIntervalSeconds: int=None, HealthCheckTimeoutSeconds: int=None, HealthyThresholdCount: int=None, UnhealthyThresholdCount: int=None, Matcher: Matcher=None, TargetType: str=None) -> CreateTargetGroupOutput:
        """Creates a target group. To register targets with the target group, use <a>RegisterTargets</a>. To update the health check settings for the target group, use <a>ModifyTargetGroup</a>. To monitor the health of targets in the target group, use <a>DescribeTargetHealth</a>. To route traffic to the targets in a target group, specify the target group in an action using <a>CreateListener</a> or <a>CreateRule</a>. To delete a target group, use <a>DeleteTargetGroup</a>. This operation is idempotent, which means that it completes at most one time. If you attempt to create multiple target groups with the same settings, each call succeeds. For more information, see <a href="https://docs.aws.amazon.com/elasticloadbalancing/latest/application/load-balancer-target-groups.html">Target Groups for Your Application Load Balancers</a> in the <i>Application Load Balancers Guide</i> or <a href="https://docs.aws.amazon.com/elasticloadbalancing/latest/network/load-balancer-target-groups.html">Target Groups for Your Network Load Balancers</a> in the <i>Network Load Balancers Guide</i>.

        :param self:
        :param TargetType:  The type of target that you must specify when registering targets with this target group. You can't specify targets for a target group using more than one target type. <ul> <li>  <code>instance</code> - Targets are specified by instance ID. This is the default value. If the target group protocol is UDP or TCP_UDP, the target type must be <code>instance</code>. </li> <li>  <code>ip</code> - Targets are specified by IP address. You can specify IP addresses from the subnets of the virtual private cloud (VPC) for the target group, the RFC 1918 range (10.0.0.0/8, 172.16.0.0/12, and 192.168.0.0/16), and the RFC 6598 range (100.64.0.0/10). You can't specify publicly routable IP addresses. </li> <li>  <code>lambda</code> - The target groups contains a single Lambda function. </li> </ul><br/>Matcher:  [HTTP/HTTPS health checks] The HTTP codes to use when checking for a successful response from a target.:<br/>UnhealthyThresholdCount:  The number of consecutive health check failures required before considering a target unhealthy. For target groups with a protocol of HTTP or HTTPS, the default is 2. For target groups with a protocol of TCP or TLS, this value must be the same as the healthy threshold count. If the target type is <code>lambda</code>, the default is 2.<br/>HealthyThresholdCount:  The number of consecutive health checks successes required before considering an unhealthy target healthy. For target groups with a protocol of HTTP or HTTPS, the default is 5. For target groups with a protocol of TCP or TLS, the default is 3. If the target type is <code>lambda</code>, the default is 5.<br/>HealthCheckTimeoutSeconds:  The amount of time, in seconds, during which no response from a target means a failed health check. For target groups with a protocol of HTTP or HTTPS, the default is 5 seconds. For target groups with a protocol of TCP or TLS, this value must be 6 seconds for HTTP health checks and 10 seconds for TCP and HTTPS health checks. If the target type is <code>lambda</code>, the default is 30 seconds.<br/>HealthCheckIntervalSeconds:  The approximate amount of time, in seconds, between health checks of an individual target. For HTTP and HTTPS health checks, the range is 5–300 seconds. For TCP health checks, the supported values are 10 and 30 seconds. If the target type is <code>instance</code> or <code>ip</code>, the default is 30 seconds. If the target type is <code>lambda</code>, the default is 35 seconds.<br/>HealthCheckPath:  [HTTP/HTTPS health checks] The ping path that is the destination on the targets for health checks. The default is /.<br/>HealthCheckEnabled:  Indicates whether health checks are enabled. If the target type is <code>lambda</code>, health checks are disabled by default but can be enabled. If the target type is <code>instance</code> or <code>ip</code>, health checks are always enabled and cannot be disabled.<br/>HealthCheckPort:  The port the load balancer uses when performing health checks on targets. The default is <code>traffic-port</code>, which is the port on which each target receives traffic from the load balancer.<br/>HealthCheckProtocol:  The protocol the load balancer uses when performing health checks on targets. For Application Load Balancers, the default is HTTP. For Network Load Balancers, the default is TCP. The TCP protocol is supported for health checks only if the protocol of the target group is TCP, TLS, UDP, or TCP_UDP. The TLS, UDP, and TCP_UDP protocols are not supported for health checks.<br/>VpcId:  The identifier of the virtual private cloud (VPC). If the target is a Lambda function, this parameter does not apply.<br/>Port:  The port on which the targets receive traffic. This port is used unless you specify a port override when registering the target. If the target is a Lambda function, this parameter does not apply.<br/>Protocol:  The protocol to use for routing traffic to the targets. For Application Load Balancers, the supported protocols are HTTP and HTTPS. For Network Load Balancers, the supported protocols are TCP, TLS, UDP, or TCP_UDP. A TCP_UDP listener must be associated with a TCP_UDP target group. If the target is a Lambda function, this parameter does not apply.<br/>Name:  The name of the target group. This name must be unique per region per account, can have a maximum of 32 characters, must contain only alphanumeric characters or hyphens, and must not begin or end with a hyphen.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HttpCode</i> The HTTP codes. For Application Load Balancers, you can specify values between 200 and 499, and the default value is 200. You can specify multiple values (for example, "200,202") or a range of values (for example, "200-299"). For Network Load Balancers, this is 200–399.<br/>
        :return: TargetGroups:  Information about the target group.<br/>        """
        pass

    def delete_listener(self, ListenerArn: str) -> DeleteListenerOutput:
        """Deletes the specified listener. Alternatively, your listener is deleted when you delete the load balancer to which it is attached, using <a>DeleteLoadBalancer</a>.

        :param self:
        :param ListenerArn:  The Amazon Resource Name (ARN) of the listener.<br/>
        :return:         """
        pass

    def delete_load_balancer(self, LoadBalancerArn: str) -> DeleteLoadBalancerOutput:
        """Deletes the specified Application Load Balancer or Network Load Balancer and its attached listeners. You can't delete a load balancer if deletion protection is enabled. If the load balancer does not exist or has already been deleted, the call succeeds. Deleting a load balancer does not affect its registered targets. For example, your EC2 instances continue to run and are still registered to their target groups. If you no longer need these EC2 instances, you can stop or terminate them.

        :param self:
        :param LoadBalancerArn:  The Amazon Resource Name (ARN) of the load balancer.<br/>
        :return:         """
        pass

    def delete_rule(self, RuleArn: str) -> DeleteRuleOutput:
        """Deletes the specified rule.

        :param self:
        :param RuleArn:  The Amazon Resource Name (ARN) of the rule.<br/>
        :return:         """
        pass

    def delete_target_group(self, TargetGroupArn: str) -> DeleteTargetGroupOutput:
        """Deletes the specified target group. You can delete a target group if it is not referenced by any actions. Deleting a target group also deletes any associated health checks.

        :param self:
        :param TargetGroupArn:  The Amazon Resource Name (ARN) of the target group.<br/>
        :return:         """
        pass

    def deregister_targets(self, Targets: List[TargetDescription], TargetGroupArn: str) -> DeregisterTargetsOutput:
        """Deregisters the specified targets from the specified target group. After the targets are deregistered, they no longer receive traffic from the load balancer.

        :param self:
        :param Targets:  The targets. If you specified a port override when you registered a target, you must specify both the target ID and the port when you deregister it.<br/>TargetGroupArn:  The Amazon Resource Name (ARN) of the target group.<br/>
        :return:         """
        pass

    def describe_account_limits(self, Marker: str=None, PageSize: int=None) -> DescribeAccountLimitsOutput:
        """Describes the current Elastic Load Balancing resource limits for your AWS account. For more information, see <a href="https://docs.aws.amazon.com/elasticloadbalancing/latest/application/load-balancer-limits.html">Limits for Your Application Load Balancers</a> in the <i>Application Load Balancer Guide</i> or <a href="https://docs.aws.amazon.com/elasticloadbalancing/latest/network/load-balancer-limits.html">Limits for Your Network Load Balancers</a> in the <i>Network Load Balancers Guide</i>.

        :param self:
        :param PageSize:  The maximum number of results to return with this call.<br/>Marker:  The marker for the next set of results. (You received this marker from a previous call.)<br/>
        :return: NextMarker:  If there are additional results, this is the marker for the next set of results. Otherwise, this is null.<br/>Limits:  Information about the limits.<br/>        """
        pass

    def describe_listener_certificates(self, ListenerArn: str, Marker: str=None, PageSize: int=None) -> DescribeListenerCertificatesOutput:
        """Describes the default certificate and the certificate list for the specified HTTPS listener. If the default certificate is also in the certificate list, it appears twice in the results (once with <code>IsDefault</code> set to true and once with <code>IsDefault</code> set to false). For more information, see <a href="https://docs.aws.amazon.com/elasticloadbalancing/latest/application/create-https-listener.html#https-listener-certificates">SSL Certificates</a> in the <i>Application Load Balancers Guide</i>.

        :param self:
        :param PageSize:  The maximum number of results to return with this call.<br/>Marker:  The marker for the next set of results. (You received this marker from a previous call.)<br/>ListenerArn:  The Amazon Resource Names (ARN) of the listener.<br/>
        :return: NextMarker:  If there are additional results, this is the marker for the next set of results. Otherwise, this is null.<br/>Certificates:  Information about the certificates.<br/>        """
        pass

    def describe_listeners(self, LoadBalancerArn: str=None, ListenerArns: List[ListenerArn]=None, Marker: str=None, PageSize: int=None) -> DescribeListenersOutput:
        """Describes the specified listeners or the listeners for the specified Application Load Balancer or Network Load Balancer. You must specify either a load balancer or one or more listeners. For an HTTPS or TLS listener, the output includes the default certificate for the listener. To describe the certificate list for the listener, use <a>DescribeListenerCertificates</a>.

        :param self:
        :param PageSize:  The maximum number of results to return with this call.<br/>Marker:  The marker for the next set of results. (You received this marker from a previous call.)<br/>ListenerArns:  The Amazon Resource Names (ARN) of the listeners.<br/>LoadBalancerArn:  The Amazon Resource Name (ARN) of the load balancer.<br/>
        :return: NextMarker:  If there are additional results, this is the marker for the next set of results. Otherwise, this is null.<br/>Listeners:  Information about the listeners.<br/>        """
        pass

    def describe_load_balancer_attributes(self, LoadBalancerArn: str) -> DescribeLoadBalancerAttributesOutput:
        """Describes the attributes for the specified Application Load Balancer or Network Load Balancer. For more information, see <a href="https://docs.aws.amazon.com/elasticloadbalancing/latest/application/application-load-balancers.html#load-balancer-attributes">Load Balancer Attributes</a> in the <i>Application Load Balancers Guide</i> or <a href="https://docs.aws.amazon.com/elasticloadbalancing/latest/network/network-load-balancers.html#load-balancer-attributes">Load Balancer Attributes</a> in the <i>Network Load Balancers Guide</i>.

        :param self:
        :param LoadBalancerArn:  The Amazon Resource Name (ARN) of the load balancer.<br/>
        :return: Attributes:  Information about the load balancer attributes.<br/>        """
        pass

    def describe_load_balancers(self, LoadBalancerArns: List[LoadBalancerArn]=None, Names: List[LoadBalancerName]=None, Marker: str=None, PageSize: int=None) -> DescribeLoadBalancersOutput:
        """Describes the specified load balancers or all of your load balancers. To describe the listeners for a load balancer, use <a>DescribeListeners</a>. To describe the attributes for a load balancer, use <a>DescribeLoadBalancerAttributes</a>.

        :param self:
        :param PageSize:  The maximum number of results to return with this call.<br/>Marker:  The marker for the next set of results. (You received this marker from a previous call.)<br/>Names:  The names of the load balancers.<br/>LoadBalancerArns:  The Amazon Resource Names (ARN) of the load balancers. You can specify up to 20 load balancers in a single call.<br/>
        :return: NextMarker:  If there are additional results, this is the marker for the next set of results. Otherwise, this is null.<br/>LoadBalancers:  Information about the load balancers.<br/>        """
        pass

    def describe_rules(self, ListenerArn: str=None, RuleArns: List[RuleArn]=None, Marker: str=None, PageSize: int=None) -> DescribeRulesOutput:
        """Describes the specified rules or the rules for the specified listener. You must specify either a listener or one or more rules.

        :param self:
        :param PageSize:  The maximum number of results to return with this call.<br/>Marker:  The marker for the next set of results. (You received this marker from a previous call.)<br/>RuleArns:  The Amazon Resource Names (ARN) of the rules.<br/>ListenerArn:  The Amazon Resource Name (ARN) of the listener.<br/>
        :return: NextMarker:  If there are additional results, this is the marker for the next set of results. Otherwise, this is null.<br/>Rules:  Information about the rules.<br/>        """
        pass

    def describe_ssl_policies(self, Names: List[SslPolicyName]=None, Marker: str=None, PageSize: int=None) -> DescribeSSLPoliciesOutput:
        """Describes the specified policies or all policies used for SSL negotiation. For more information, see <a href="https://docs.aws.amazon.com/elasticloadbalancing/latest/application/create-https-listener.html#describe-ssl-policies">Security Policies</a> in the <i>Application Load Balancers Guide</i>.

        :param self:
        :param PageSize:  The maximum number of results to return with this call.<br/>Marker:  The marker for the next set of results. (You received this marker from a previous call.)<br/>Names:  The names of the policies.<br/>
        :return: NextMarker:  If there are additional results, this is the marker for the next set of results. Otherwise, this is null.<br/>SslPolicies:  Information about the policies.<br/>        """
        pass

    def describe_tags(self, ResourceArns: List[ResourceArn]) -> DescribeTagsOutput:
        """Describes the tags for the specified resources. You can describe the tags for one or more Application Load Balancers, Network Load Balancers, and target groups.

        :param self:
        :param ResourceArns:  The Amazon Resource Names (ARN) of the resources.<br/>
        :return: TagDescriptions:  Information about the tags.<br/>        """
        pass

    def describe_target_group_attributes(self, TargetGroupArn: str) -> DescribeTargetGroupAttributesOutput:
        """Describes the attributes for the specified target group. For more information, see <a href="https://docs.aws.amazon.com/elasticloadbalancing/latest/application/load-balancer-target-groups.html#target-group-attributes">Target Group Attributes</a> in the <i>Application Load Balancers Guide</i> or <a href="https://docs.aws.amazon.com/elasticloadbalancing/latest/network/load-balancer-target-groups.html#target-group-attributes">Target Group Attributes</a> in the <i>Network Load Balancers Guide</i>.

        :param self:
        :param TargetGroupArn:  The Amazon Resource Name (ARN) of the target group.<br/>
        :return: Attributes:  Information about the target group attributes<br/>        """
        pass

    def describe_target_groups(self, LoadBalancerArn: str=None, TargetGroupArns: List[TargetGroupArn]=None, Names: List[TargetGroupName]=None, Marker: str=None, PageSize: int=None) -> DescribeTargetGroupsOutput:
        """Describes the specified target groups or all of your target groups. By default, all target groups are described. Alternatively, you can specify one of the following to filter the results: the ARN of the load balancer, the names of one or more target groups, or the ARNs of one or more target groups. To describe the targets for a target group, use <a>DescribeTargetHealth</a>. To describe the attributes of a target group, use <a>DescribeTargetGroupAttributes</a>.

        :param self:
        :param PageSize:  The maximum number of results to return with this call.<br/>Marker:  The marker for the next set of results. (You received this marker from a previous call.)<br/>Names:  The names of the target groups.<br/>TargetGroupArns:  The Amazon Resource Names (ARN) of the target groups.<br/>LoadBalancerArn:  The Amazon Resource Name (ARN) of the load balancer.<br/>
        :return: NextMarker:  If there are additional results, this is the marker for the next set of results. Otherwise, this is null.<br/>TargetGroups:  Information about the target groups.<br/>        """
        pass

    def describe_target_health(self, TargetGroupArn: str, Targets: List[TargetDescription]=None) -> DescribeTargetHealthOutput:
        """Describes the health of the specified targets or all of your targets.

        :param self:
        :param Targets:  The targets.<br/>TargetGroupArn:  The Amazon Resource Name (ARN) of the target group.<br/>
        :return: TargetHealthDescriptions:  Information about the health of the targets.<br/>        """
        pass

    def modify_listener(self, ListenerArn: str, Port: int=None, Protocol: str=None, SslPolicy: str=None, Certificates: List[Certificate]=None, DefaultActions: List[Action]=None) -> ModifyListenerOutput:
        """Modifies the specified properties of the specified listener. Any properties that you do not specify retain their current values. However, changing the protocol from HTTPS to HTTP, or from TLS to TCP, removes the security policy and default certificate properties. If you change the protocol from HTTP to HTTPS, or from TCP to TLS, you must add the security policy and default certificate properties.

        :param self:
        :param DefaultActions:  The actions for the default rule. The rule must include one forward action or one or more fixed-response actions. If the action type is <code>forward</code>, you specify a target group. The protocol of the target group must be HTTP or HTTPS for an Application Load Balancer. The protocol of the target group must be TCP, TLS, UDP, or TCP_UDP for a Network Load Balancer. [HTTPS listeners] If the action type is <code>authenticate-oidc</code>, you authenticate users through an identity provider that is OpenID Connect (OIDC) compliant. [HTTPS listeners] If the action type is <code>authenticate-cognito</code>, you authenticate users through the user pools supported by Amazon Cognito. [Application Load Balancer] If the action type is <code>redirect</code>, you redirect specified client requests from one URL to another. [Application Load Balancer] If the action type is <code>fixed-response</code>, you drop specified client requests and return a custom HTTP response.<br/>Certificates:  [HTTPS and TLS listeners] The default certificate for the listener. You must provide exactly one certificate. Set <code>CertificateArn</code> to the certificate ARN but do not set <code>IsDefault</code>. To create a certificate list, use <a>AddListenerCertificates</a>.<br/>SslPolicy:  [HTTPS and TLS listeners] The security policy that defines which protocols and ciphers are supported. For more information, see <a href="https://docs.aws.amazon.com/elasticloadbalancing/latest/application/create-https-listener.html#describe-ssl-policies">Security Policies</a> in the <i>Application Load Balancers Guide</i>.<br/>Protocol:  The protocol for connections from clients to the load balancer. Application Load Balancers support the HTTP and HTTPS protocols. Network Load Balancers support the TCP, TLS, UDP, and TCP_UDP protocols.<br/>Port:  The port for connections from clients to the load balancer.<br/>ListenerArn:  The Amazon Resource Name (ARN) of the listener.<br/>
        :return: Listeners:  Information about the modified listener.<br/>        """
        pass

    def modify_load_balancer_attributes(self, Attributes: List[LoadBalancerAttribute], LoadBalancerArn: str) -> ModifyLoadBalancerAttributesOutput:
        """Modifies the specified attributes of the specified Application Load Balancer or Network Load Balancer. If any of the specified attributes can't be modified as requested, the call fails. Any existing attributes that you do not modify retain their current values.

        :param self:
        :param Attributes:  The load balancer attributes.<br/>LoadBalancerArn:  The Amazon Resource Name (ARN) of the load balancer.<br/>
        :return: Attributes:  Information about the load balancer attributes.<br/>        """
        pass

    def modify_rule(self, RuleArn: str, Conditions: List[RuleCondition]=None, Actions: List[Action]=None) -> ModifyRuleOutput:
        """Modifies the specified rule. Any existing properties that you do not modify retain their current values. To modify the actions for the default rule, use <a>ModifyListener</a>.

        :param self:
        :param Actions:  The actions. Each rule must include exactly one of the following types of actions: <code>forward</code>, <code>fixed-response</code>, or <code>redirect</code>. If the action type is <code>forward</code>, you specify a target group. The protocol of the target group must be HTTP or HTTPS for an Application Load Balancer. The protocol of the target group must be TCP, TLS, UDP, or TCP_UDP for a Network Load Balancer. [HTTPS listeners] If the action type is <code>authenticate-oidc</code>, you authenticate users through an identity provider that is OpenID Connect (OIDC) compliant. [HTTPS listeners] If the action type is <code>authenticate-cognito</code>, you authenticate users through the user pools supported by Amazon Cognito. [Application Load Balancer] If the action type is <code>redirect</code>, you redirect specified client requests from one URL to another. [Application Load Balancer] If the action type is <code>fixed-response</code>, you drop specified client requests and return a custom HTTP response.<br/>Conditions:  The conditions. Each rule can include zero or one of the following conditions: <code>http-request-method</code>, <code>host-header</code>, <code>path-pattern</code>, and <code>source-ip</code>, and zero or more of the following conditions: <code>http-header</code> and <code>query-string</code>.<br/>RuleArn:  The Amazon Resource Name (ARN) of the rule.<br/>
        :return: Rules:  Information about the modified rule.<br/>        """
        pass

    def modify_target_group(self, TargetGroupArn: str, HealthCheckProtocol: str=None, HealthCheckPort: str=None, HealthCheckPath: str=None, HealthCheckEnabled: HealthCheckEnabled=None, HealthCheckIntervalSeconds: int=None, HealthCheckTimeoutSeconds: int=None, HealthyThresholdCount: int=None, UnhealthyThresholdCount: int=None, Matcher: Matcher=None) -> ModifyTargetGroupOutput:
        """Modifies the health checks used when evaluating the health state of the targets in the specified target group. To monitor the health of the targets, use <a>DescribeTargetHealth</a>.

        :param self:
        :param Matcher:  [HTTP/HTTPS health checks] The HTTP codes to use when checking for a successful response from a target. If the protocol of the target group is TCP, you can't modify this setting.:<br/>UnhealthyThresholdCount:  The number of consecutive health check failures required before considering the target unhealthy. For Network Load Balancers, this value must be the same as the healthy threshold count.<br/>HealthyThresholdCount:  The number of consecutive health checks successes required before considering an unhealthy target healthy.<br/>HealthCheckTimeoutSeconds:  [HTTP/HTTPS health checks] The amount of time, in seconds, during which no response means a failed health check. If the protocol of the target group is TCP, you can't modify this setting.<br/>HealthCheckIntervalSeconds:  The approximate amount of time, in seconds, between health checks of an individual target. For Application Load Balancers, the range is 5 to 300 seconds. For Network Load Balancers, the supported values are 10 or 30 seconds. If the protocol of the target group is TCP, you can't modify this setting.<br/>HealthCheckEnabled:  Indicates whether health checks are enabled.<br/>HealthCheckPath:  [HTTP/HTTPS health checks] The ping path that is the destination for the health check request.<br/>HealthCheckPort:  The port the load balancer uses when performing health checks on targets.<br/>HealthCheckProtocol:  The protocol the load balancer uses when performing health checks on targets. The TCP protocol is supported for health checks only if the protocol of the target group is TCP, TLS, UDP, or TCP_UDP. The TLS, UDP, and TCP_UDP protocols are not supported for health checks. If the protocol of the target group is TCP, you can't modify this setting.<br/>TargetGroupArn:  The Amazon Resource Name (ARN) of the target group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HttpCode</i> The HTTP codes. For Application Load Balancers, you can specify values between 200 and 499, and the default value is 200. You can specify multiple values (for example, "200,202") or a range of values (for example, "200-299"). For Network Load Balancers, this is 200–399.<br/>
        :return: TargetGroups:  Information about the modified target group.<br/>        """
        pass

    def modify_target_group_attributes(self, Attributes: List[TargetGroupAttribute], TargetGroupArn: str) -> ModifyTargetGroupAttributesOutput:
        """Modifies the specified attributes of the specified target group.

        :param self:
        :param Attributes:  The attributes.<br/>TargetGroupArn:  The Amazon Resource Name (ARN) of the target group.<br/>
        :return: Attributes:  Information about the attributes.<br/>        """
        pass

    def register_targets(self, Targets: List[TargetDescription], TargetGroupArn: str) -> RegisterTargetsOutput:
        """Registers the specified targets with the specified target group. If the target is an EC2 instance, it must be in the <code>running</code> state when you register it. By default, the load balancer routes requests to registered targets using the protocol and port for the target group. Alternatively, you can override the port for a target when you register it. You can register each EC2 instance or IP address with the same target group multiple times using different ports. With a Network Load Balancer, you cannot register instances by instance ID if they have the following instance types: C1, CC1, CC2, CG1, CG2, CR1, CS1, G1, G2, HI1, HS1, M1, M2, M3, and T1. You can register instances of these types by IP address. To remove a target from a target group, use <a>DeregisterTargets</a>.

        :param self:
        :param Targets:  The targets. To register a target by instance ID, specify the instance ID. To register a target by IP address, specify the IP address. To register a Lambda function, specify the ARN of the Lambda function.<br/>TargetGroupArn:  The Amazon Resource Name (ARN) of the target group.<br/>
        :return:         """
        pass

    def remove_listener_certificates(self, Certificates: List[Certificate], ListenerArn: str) -> RemoveListenerCertificatesOutput:
        """Removes the specified certificate from the certificate list for the specified HTTPS listener. You can't remove the default certificate for a listener. To replace the default certificate, call <a>ModifyListener</a>. To list the certificates for your listener, use <a>DescribeListenerCertificates</a>.

        :param self:
        :param Certificates:  The certificate to remove. You can specify one certificate per call. Set <code>CertificateArn</code> to the certificate ARN but do not set <code>IsDefault</code>.<br/>ListenerArn:  The Amazon Resource Name (ARN) of the listener.<br/>
        :return:         """
        pass

    def remove_tags(self, TagKeys: List[TagKey], ResourceArns: List[ResourceArn]) -> RemoveTagsOutput:
        """Removes the specified tags from the specified Elastic Load Balancing resource. To list the current tags for your resources, use <a>DescribeTags</a>.

        :param self:
        :param TagKeys:  The tag keys for the tags to remove.<br/>ResourceArns:  The Amazon Resource Name (ARN) of the resource.<br/>
        :return:         """
        pass

    def set_ip_address_type(self, IpAddressType: str, LoadBalancerArn: str) -> SetIpAddressTypeOutput:
        """Sets the type of IP addresses used by the subnets of the specified Application Load Balancer or Network Load Balancer.

        :param self:
        :param IpAddressType:  The IP address type. The possible values are <code>ipv4</code> (for IPv4 addresses) and <code>dualstack</code> (for IPv4 and IPv6 addresses). Internal load balancers must use <code>ipv4</code>. Network Load Balancers must use <code>ipv4</code>.<br/>LoadBalancerArn:  The Amazon Resource Name (ARN) of the load balancer.<br/>
        :return: IpAddressType:  The IP address type.<br/>        """
        pass

    def set_rule_priorities(self, RulePriorities: List[RulePriorityPair]) -> SetRulePrioritiesOutput:
        """Sets the priorities of the specified rules. You can reorder the rules as long as there are no priority conflicts in the new order. Any existing rules that you do not specify retain their current priority.

        :param self:
        :param RulePriorities:  The rule priorities.<br/>
        :return: Rules:  Information about the rules.<br/>        """
        pass

    def set_security_groups(self, SecurityGroups: List[SecurityGroupId], LoadBalancerArn: str) -> SetSecurityGroupsOutput:
        """Associates the specified security groups with the specified Application Load Balancer. The specified security groups override the previously associated security groups. You can't specify a security group for a Network Load Balancer.

        :param self:
        :param SecurityGroups:  The IDs of the security groups.<br/>LoadBalancerArn:  The Amazon Resource Name (ARN) of the load balancer.<br/>
        :return: SecurityGroupIds:  The IDs of the security groups associated with the load balancer.<br/>        """
        pass

    def set_subnets(self, LoadBalancerArn: str, Subnets: List[SubnetId]=None, SubnetMappings: List[SubnetMapping]=None) -> SetSubnetsOutput:
        """Enables the Availability Zone for the specified public subnets for the specified Application Load Balancer. The specified subnets replace the previously enabled subnets. You can't change the subnets for a Network Load Balancer.

        :param self:
        :param SubnetMappings:  The IDs of the public subnets. You must specify subnets from at least two Availability Zones. You can specify only one subnet per Availability Zone. You must specify either subnets or subnet mappings. You cannot specify Elastic IP addresses for your subnets.<br/>Subnets:  The IDs of the public subnets. You must specify subnets from at least two Availability Zones. You can specify only one subnet per Availability Zone. You must specify either subnets or subnet mappings.<br/>LoadBalancerArn:  The Amazon Resource Name (ARN) of the load balancer.<br/>
        :return: AvailabilityZones:  Information about the subnet and Availability Zone.<br/>        """
        pass



class EMR(BaseClient):
    """Amazon EMR is a web service that makes it easy to process large amounts of data efficiently. Amazon EMR uses Hadoop processing combined with several AWS products to do tasks such as web indexing, data mining, log file analysis, machine learning, scientific simulation, and data warehousing."""

    class ClusterRunningWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_cluster` for valid parameters"""
            pass
    class ClusterTerminatedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_cluster` for valid parameters"""
            pass
    class StepCompleteWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_step` for valid parameters"""
            pass


    class ListBootstrapActionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_bootstrap_actions` for valid parameters"""
            pass
    class ListClustersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_clusters` for valid parameters"""
            pass
    class ListInstanceGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_instance_groups` for valid parameters"""
            pass
    class ListInstancesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_instances` for valid parameters"""
            pass
    class ListStepsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_steps` for valid parameters"""
            pass
    class ListInstanceFleetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_instance_fleets` for valid parameters"""
            pass
    class ListSecurityConfigurationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_security_configurations` for valid parameters"""
            pass


    class AddInstanceFleetInput(Mapping):
        pass
    
    class AddInstanceFleetOutput(Mapping):
        pass
    
    class InstanceFleetConfig(Mapping):
        pass
    
    class AddInstanceGroupsInput(Mapping):
        pass
    
    class AddInstanceGroupsOutput(Mapping):
        pass
    
    class InstanceGroupConfig(Mapping):
        pass
    
    class AddJobFlowStepsInput(Mapping):
        pass
    
    class AddJobFlowStepsOutput(Mapping):
        pass
    
    class StepConfig(Mapping):
        pass
    
    class AddTagsInput(Mapping):
        pass
    
    class AddTagsOutput(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CancelStepsInput(Mapping):
        pass
    
    class CancelStepsOutput(Mapping):
        pass
    
    class XmlStringMaxLen256(object):
        pass
    
    class CreateSecurityConfigurationInput(Mapping):
        pass
    
    class CreateSecurityConfigurationOutput(Mapping):
        pass
    
    class DeleteSecurityConfigurationInput(Mapping):
        pass
    
    class DeleteSecurityConfigurationOutput(Mapping):
        pass
    
    class DescribeClusterInput(Mapping):
        pass
    
    class DescribeClusterOutput(Mapping):
        pass
    
    class DescribeJobFlowsInput(Mapping):
        pass
    
    class DescribeJobFlowsOutput(Mapping):
        pass
    
    class Date(object):
        pass
    
    class XmlString(object):
        pass
    
    class JobFlowExecutionState(object):
        pass
    
    class DescribeSecurityConfigurationInput(Mapping):
        pass
    
    class DescribeSecurityConfigurationOutput(Mapping):
        pass
    
    class DescribeStepInput(Mapping):
        pass
    
    class DescribeStepOutput(Mapping):
        pass
    
    class ListBootstrapActionsInput(Mapping):
        pass
    
    class ListBootstrapActionsOutput(Mapping):
        pass
    
    class ListClustersInput(Mapping):
        pass
    
    class ListClustersOutput(Mapping):
        pass
    
    class ClusterState(object):
        pass
    
    class ListInstanceFleetsInput(Mapping):
        pass
    
    class ListInstanceFleetsOutput(Mapping):
        pass
    
    class ListInstanceGroupsInput(Mapping):
        pass
    
    class ListInstanceGroupsOutput(Mapping):
        pass
    
    class ListInstancesInput(Mapping):
        pass
    
    class ListInstancesOutput(Mapping):
        pass
    
    class InstanceGroupType(object):
        pass
    
    class InstanceState(object):
        pass
    
    class ListSecurityConfigurationsInput(Mapping):
        pass
    
    class ListSecurityConfigurationsOutput(Mapping):
        pass
    
    class ListStepsInput(Mapping):
        pass
    
    class ListStepsOutput(Mapping):
        pass
    
    class StepState(object):
        pass
    
    class ModifyInstanceFleetInput(Mapping):
        pass
    
    class InstanceFleetModifyConfig(Mapping):
        pass
    
    class ModifyInstanceGroupsInput(Mapping):
        pass
    
    class InstanceGroupModifyConfig(Mapping):
        pass
    
    class PutAutoScalingPolicyInput(Mapping):
        pass
    
    class PutAutoScalingPolicyOutput(Mapping):
        pass
    
    class AutoScalingPolicy(Mapping):
        pass
    
    class RemoveAutoScalingPolicyInput(Mapping):
        pass
    
    class RemoveAutoScalingPolicyOutput(Mapping):
        pass
    
    class RemoveTagsInput(Mapping):
        pass
    
    class RemoveTagsOutput(Mapping):
        pass
    
    class String(object):
        pass
    
    class RunJobFlowInput(Mapping):
        pass
    
    class RunJobFlowOutput(Mapping):
        pass
    
    class JobFlowInstancesConfig(Mapping):
        pass
    
    class BootstrapActionConfig(Mapping):
        pass
    
    class SupportedProductConfig(Mapping):
        pass
    
    class Application(Mapping):
        pass
    
    class Configuration(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class KerberosAttributes(Mapping):
        pass
    
    class SetTerminationProtectionInput(Mapping):
        pass
    
    class SetVisibleToAllUsersInput(Mapping):
        pass
    
    class TerminateJobFlowsInput(Mapping):
        pass
    
    def add_instance_fleet(self, InstanceFleet: InstanceFleetConfig, ClusterId: str) -> AddInstanceFleetOutput:
        """Adds an instance fleet to a running cluster. <note> The instance fleet configuration is available only in Amazon EMR versions 4.8.0 and later, excluding 5.0.x. </note>

        :param self:
        :param InstanceFleet:  Specifies the configuration of the instance fleet.:<br/>ClusterId:  The unique identifier of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LaunchSpecifications</i> The launch specification for the instance fleet.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LaunchSpecifications</b>: The launch specification for the instance fleet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceTypeConfigs</i> The instance type configurations that define the EC2 instances in the instance fleet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetSpotCapacity</i> The target capacity of Spot units for the instance fleet, which determines how many Spot instances to provision. When the instance fleet launches, Amazon EMR tries to provision Spot instances as specified by <a>InstanceTypeConfig</a>. Each instance configuration has a specified <code>WeightedCapacity</code>. When a Spot instance is provisioned, the <code>WeightedCapacity</code> units count toward the target capacity. Amazon EMR provisions instances until the target capacity is totally fulfilled, even if this results in an overage. For example, if there are 2 units remaining to fulfill capacity, and Amazon EMR can only provision an instance with a <code>WeightedCapacity</code> of 5 units, the instance is provisioned, and the target capacity is exceeded by 3 units. <note> If not specified or set to 0, only On-Demand instances are provisioned for the instance fleet. At least one of <code>TargetSpotCapacity</code> and <code>TargetOnDemandCapacity</code> should be greater than 0. For a master instance fleet, only one of <code>TargetSpotCapacity</code> and <code>TargetOnDemandCapacity</code> can be specified, and its value must be 1. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetOnDemandCapacity</i> The target capacity of On-Demand units for the instance fleet, which determines how many On-Demand instances to provision. When the instance fleet launches, Amazon EMR tries to provision On-Demand instances as specified by <a>InstanceTypeConfig</a>. Each instance configuration has a specified <code>WeightedCapacity</code>. When an On-Demand instance is provisioned, the <code>WeightedCapacity</code> units count toward the target capacity. Amazon EMR provisions instances until the target capacity is totally fulfilled, even if this results in an overage. For example, if there are 2 units remaining to fulfill capacity, and Amazon EMR can only provision an instance with a <code>WeightedCapacity</code> of 5 units, the instance is provisioned, and the target capacity is exceeded by 3 units. <note> If not specified or set to 0, only Spot instances are provisioned for the instance fleet using <code>TargetSpotCapacity</code>. At least one of <code>TargetSpotCapacity</code> and <code>TargetOnDemandCapacity</code> should be greater than 0. For a master instance fleet, only one of <code>TargetSpotCapacity</code> and <code>TargetOnDemandCapacity</code> can be specified, and its value must be 1. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceFleetType</i> The node type that the instance fleet hosts. Valid values are MASTER,CORE,and TASK.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The friendly name of the instance fleet.<br/>The launch specification for Spot instances in the fleet, which determines the defined duration and provisioning timeout behavior.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpotSpecification</b>: The launch specification for Spot instances in the fleet, which determines the defined duration and provisioning timeout behavior.
        :return: InstanceFleetId:  The unique identifier of the instance fleet.<br/>ClusterId:  The unique identifier of the cluster.<br/>        """
        pass

    def add_instance_groups(self, JobFlowId: str, InstanceGroups: List[InstanceGroupConfig]) -> AddInstanceGroupsOutput:
        """Adds one or more instance groups to a running cluster.

        :param self:
        :param JobFlowId:  Job flow in which to add the instance groups.<br/>InstanceGroups:  Instance groups to add.<br/>
        :return: InstanceGroupIds:  Instance group IDs of the newly created instance groups.<br/>JobFlowId:  The job flow ID in which the instance groups are added.<br/>        """
        pass

    def add_job_flow_steps(self, Steps: List[StepConfig], JobFlowId: str) -> AddJobFlowStepsOutput:
        """AddJobFlowSteps adds new steps to a running cluster. A maximum of 256 steps are allowed in each job flow. If your cluster is long-running (such as a Hive data warehouse) or complex, you may require more than 256 steps to process your data. You can bypass the 256-step limitation in various ways, including using SSH to connect to the master node and submitting queries directly to the software running on the master node, such as Hive and Hadoop. For more information on how to do this, see <a href="https://docs.aws.amazon.com/emr/latest/ManagementGuide/AddMoreThan256Steps.html">Add More than 256 Steps to a Cluster</a> in the <i>Amazon EMR Management Guide</i>. A step specifies the location of a JAR file stored either on the master node of the cluster or in Amazon S3. Each step is performed by the main function of the main class of the JAR file. The main class can be specified either in the manifest of the JAR or by using the MainFunction parameter of the step. Amazon EMR executes each step in the order listed. For a step to be considered complete, the main function must exit with a zero exit code and all Hadoop jobs started while the step was running must have completed and run successfully. You can only add steps to a cluster that is in one of the following states: STARTING, BOOTSTRAPPING, RUNNING, or WAITING.

        :param self:
        :param Steps:   A list of <a>StepConfig</a> to be executed by the job flow. <br/>JobFlowId:  A string that uniquely identifies the job flow. This identifier is returned by <a>RunJobFlow</a> and can also be obtained from <a>ListClusters</a>. <br/>
        :return: StepIds:  The identifiers of the list of steps added to the job flow.<br/>        """
        pass

    def add_tags(self, Tags: List[Tag], ResourceId: str) -> AddTagsOutput:
        """Adds tags to an Amazon EMR resource. Tags make it easier to associate clusters in various ways, such as grouping clusters to track your Amazon EMR resource allocation costs. For more information, see <a href="https://docs.aws.amazon.com/emr/latest/ManagementGuide/emr-plan-tags.html">Tag Clusters</a>. 

        :param self:
        :param Tags:  A list of tags to associate with a cluster and propagate to EC2 instances. Tags are user-defined key/value pairs that consist of a required key string with a maximum of 128 characters, and an optional value string with a maximum of 256 characters.<br/>ResourceId:  The Amazon EMR resource identifier to which tags will be added. This value must be a cluster identifier.<br/>
        :return:         """
        pass

    def cancel_steps(self, ClusterId: str=None, StepIds: List[XmlStringMaxLen256]=None) -> CancelStepsOutput:
        """Cancels a pending step or steps in a running cluster. Available only in Amazon EMR versions 4.8.0 and later, excluding version 5.0.0. A maximum of 256 steps are allowed in each CancelSteps request. CancelSteps is idempotent but asynchronous; it does not guarantee a step will be canceled, even if the request is successfully submitted. You can only cancel steps that are in a <code>PENDING</code> state.

        :param self:
        :param StepIds:  The list of <code>StepIDs</code> to cancel. Use <a>ListSteps</a> to get steps and their states for the specified cluster.<br/>ClusterId:  The <code>ClusterID</code> for which specified steps will be canceled. Use <a>RunJobFlow</a> and <a>ListClusters</a> to get ClusterIDs. <br/>
        :return: CancelStepsInfoList:  A list of <a>CancelStepsInfo</a>, which shows the status of specified cancel requests for each <code>StepID</code> specified.<br/>        """
        pass

    def create_security_configuration(self, SecurityConfiguration: str, Name: str) -> CreateSecurityConfigurationOutput:
        """Creates a security configuration, which is stored in the service and can be specified when a cluster is created.

        :param self:
        :param SecurityConfiguration:  The security configuration details in JSON format. For JSON parameters and examples, see <a href="https://docs.aws.amazon.com/emr/latest/ManagementGuide/emr-security-configurations.html">Use Security Configurations to Set Up Cluster Security</a> in the <i>Amazon EMR Management Guide</i>.<br/>Name:  The name of the security configuration.<br/>
        :return: CreationDateTime:  The date and time the security configuration was created.<br/>Name:  The name of the security configuration.<br/>        """
        pass

    def delete_security_configuration(self, Name: str) -> DeleteSecurityConfigurationOutput:
        """Deletes a security configuration.

        :param self:
        :param Name:  The name of the security configuration.<br/>
        :return:         """
        pass

    def describe_cluster(self, ClusterId: str) -> DescribeClusterOutput:
        """Provides cluster-level details including status, hardware and software configuration, VPC settings, and so on. 

        :param self:
        :param ClusterId:  The identifier of the cluster to describe.<br/>
        :return: Cluster:  This output contains the details for the requested cluster.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KerberosAttributes</i> Attributes for Kerberos configuration when Kerberos authentication is enabled using a security configuration. For more information see <a href="https://docs.aws.amazon.com/emr/latest/ManagementGuide/emr-kerberos.html">Use Kerberos Authentication</a> in the <i>EMR Management Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KerberosAttributes</b>: Attributes for Kerberos configuration when Kerberos authentication is enabled using a security configuration. For more information see <a href="https://docs.aws.amazon.com/emr/latest/ManagementGuide/emr-kerberos.html">Use Kerberos Authentication</a> in the <i>EMR Management Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RepoUpgradeOnBoot</i> Applies only when <code>CustomAmiID</code> is used. Specifies the type of updates that are applied from the Amazon Linux AMI package repositories when an instance boots using the AMI.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EbsRootVolumeSize</i> The size, in GiB, of the EBS root device volume of the Linux AMI that is used for each EC2 instance. Available in Amazon EMR version 4.x and later.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomAmiId</i> Available only in Amazon EMR version 5.7.0 and later. The ID of a custom Amazon EBS-backed Linux AMI if the cluster uses a custom AMI.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ScaleDownBehavior</i> The way that individual Amazon EC2 instances terminate when an automatic scale-in activity occurs or an instance group is resized. <code>TERMINATE_AT_INSTANCE_HOUR</code> indicates that Amazon EMR terminates nodes at the instance-hour boundary, regardless of when the request to terminate the instance was submitted. This option is only available with Amazon EMR 5.1.0 and later and is the default for clusters created using that version. <code>TERMINATE_AT_TASK_COMPLETION</code> indicates that Amazon EMR blacklists and drains tasks from nodes before terminating the Amazon EC2 instances, regardless of the instance-hour boundary. With either behavior, Amazon EMR removes the least active nodes first and blocks instance termination if it could lead to HDFS corruption. <code>TERMINATE_AT_TASK_COMPLETION</code> is available only in Amazon EMR version 4.1.0 and later, and is the default for versions of Amazon EMR earlier than 5.1.0.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoScalingRole</i> An IAM role for automatic scaling policies. The default role is <code>EMR_AutoScaling_DefaultRole</code>. The IAM role provides permissions that the automatic scaling feature requires to launch and terminate EC2 instances in an instance group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityConfiguration</i> The name of the security configuration applied to the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Configurations</i> Applies only to Amazon EMR releases 4.x and later. The list of Configurations supplied to the EMR cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterPublicDnsName</i> The DNS name of the master node. If the cluster is on a private subnet, this is the private DNS name. On a public subnet, this is the public DNS name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NormalizedInstanceHours</i> An approximation of the cost of the cluster, represented in m1.small/hours. This value is incremented one time for every hour an m1.small instance runs. Larger instances are weighted more, so an EC2 instance that is roughly four times more expensive would result in the normalized instance hours being incremented by four. This result is only an approximation and does not reflect the actual billing rate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceRole</i> The IAM role that will be assumed by the Amazon EMR service to access AWS resources on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> A list of tags associated with a cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Applications</i> The applications installed on this cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VisibleToAllUsers</i> Indicates whether the cluster is visible to all IAM users of the AWS account associated with the cluster. If this value is set to <code>true</code>, all IAM users of that AWS account can view and manage the cluster if they have the proper policy permissions set. If this value is <code>false</code>, only the IAM user that created the cluster can view and manage it. This value can be changed using the <a>SetVisibleToAllUsers</a> action.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TerminationProtected</i> Indicates whether Amazon EMR will lock the cluster to prevent the EC2 instances from being terminated by an API call or user intervention, or in the event of a cluster error.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoTerminate</i> Specifies whether the cluster should terminate after completing all steps.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReleaseLabel</i> The Amazon EMR release label, which determines the version of open-source application packages installed on the cluster. Release labels are in the form <code>emr-x.x.x</code>, where x.x.x is an Amazon EMR release version, for example, <code>emr-5.14.0</code>. For more information about Amazon EMR release versions and included application versions and features, see <a href="https://docs.aws.amazon.com/emr/latest/ReleaseGuide/">https://docs.aws.amazon.com/emr/latest/ReleaseGuide/</a>. The release label applies only to Amazon EMR releases versions 4.x and later. Earlier versions use <code>AmiVersion</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RunningAmiVersion</i> The AMI version running on this cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestedAmiVersion</i> The AMI version requested for this cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogUri</i> The path to the Amazon S3 location where logs for this cluster are stored.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCollectionType</i> <note> The instance fleet configuration is available only in Amazon EMR versions 4.8.0 and later, excluding 5.0.x versions. </note> The instance group configuration of the cluster. A value of <code>INSTANCE_GROUP</code> indicates a uniform instance group configuration. A value of <code>INSTANCE_FLEET</code> indicates an instance fleets configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Ec2InstanceAttributes</i> Provides information about the EC2 instances in a cluster grouped by category. For example, key name, subnet ID, IAM instance profile, and so on.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Ec2InstanceAttributes</b>: Provides information about the EC2 instances in a cluster grouped by category. For example, key name, subnet ID, IAM instance profile, and so on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current status details about the cluster.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>: The current status details about the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The current state of the cluster.<br/>The reason for the cluster status change.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StateChangeReason</b>: The reason for the cluster status change.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AdditionalSlaveSecurityGroups</i> A list of additional Amazon EC2 security group IDs for the core and task nodes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AdditionalMasterSecurityGroups</i> A list of additional Amazon EC2 security group IDs for the master node.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessSecurityGroup</i> The identifier of the Amazon EC2 security group for the Amazon EMR service to access clusters in VPC private subnets.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EmrManagedSlaveSecurityGroup</i> The identifier of the Amazon EC2 security group for the core and task nodes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EmrManagedMasterSecurityGroup</i> The identifier of the Amazon EC2 security group for the master node.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IamInstanceProfile</i> The IAM role that was specified when the cluster was launched. The EC2 instances of the cluster assume this role.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestedEc2AvailabilityZones</i> Applies to clusters configured with the instance fleets option. Specifies one or more Availability Zones in which to launch EC2 cluster instances when the EC2-Classic network configuration is supported. Amazon EMR chooses the Availability Zone with the best fit from among the list of <code>RequestedEc2AvailabilityZones</code>, and then launches all cluster instances within that Availability Zone. If you do not specify this value, Amazon EMR chooses the Availability Zone for you. <code>RequestedEc2SubnetIDs</code> and <code>RequestedEc2AvailabilityZones</code> cannot be specified together.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Ec2AvailabilityZone</i> The Availability Zone in which the cluster will run. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestedEc2SubnetIds</i> Applies to clusters configured with the instance fleets option. Specifies the unique identifier of one or more Amazon EC2 subnets in which to launch EC2 cluster instances. Subnets must exist within the same VPC. Amazon EMR chooses the EC2 subnet with the best fit from among the list of <code>RequestedEc2SubnetIds</code>, and then launches all cluster instances within that Subnet. If this value is not specified, and the account and region support EC2-Classic networks, the cluster launches instances in the EC2-Classic network and uses <code>RequestedEc2AvailabilityZones</code> instead of this setting. If EC2-Classic is not supported, and no Subnet is specified, Amazon EMR chooses the subnet for you. <code>RequestedEc2SubnetIDs</code> and <code>RequestedEc2AvailabilityZones</code> cannot be specified together.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Ec2SubnetId</i> To launch the cluster in Amazon VPC, set this parameter to the identifier of the Amazon VPC subnet where you want the cluster to launch. If you do not specify this value, the cluster is launched in the normal AWS cloud, outside of a VPC. Amazon VPC currently does not support cluster compute quadruple extra large (cc1.4xlarge) instances. Thus, you cannot specify the cc1.4xlarge instance type for nodes of a cluster launched in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Ec2KeyName</i> The name of the Amazon EC2 key pair to use when connecting with SSH into the master node as a user named "hadoop".<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ADDomainJoinPassword</i> The Active Directory password for <code>ADDomainJoinUser</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ADDomainJoinUser</i> Required only when establishing a cross-realm trust with an Active Directory domain. A user with sufficient privileges to join resources to the domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CrossRealmTrustPrincipalPassword</i> Required only when establishing a cross-realm trust with a KDC in a different realm. The cross-realm principal password, which must be identical across realms.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KdcAdminPassword</i> The password used within the cluster for the kadmin service on the cluster-dedicated KDC, which maintains Kerberos principals, password policies, and keytabs for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Realm</i> The name of the Kerberos realm to which all nodes in a cluster belong. For example, <code>EC2.INTERNAL</code>. <br/>        """
        pass

    def describe_job_flows(self, CreatedAfter: Date=None, CreatedBefore: Date=None, JobFlowIds: List[XmlString]=None, JobFlowStates: List[JobFlowExecutionState]=None) -> DescribeJobFlowsOutput:
        """This API is deprecated and will eventually be removed. We recommend you use <a>ListClusters</a>, <a>DescribeCluster</a>, <a>ListSteps</a>, <a>ListInstanceGroups</a> and <a>ListBootstrapActions</a> instead. DescribeJobFlows returns a list of job flows that match all of the supplied parameters. The parameters can include a list of job flow IDs, job flow states, and restrictions on job flow creation date and time. Regardless of supplied parameters, only job flows created within the last two months are returned. If no parameters are supplied, then job flows matching either of the following criteria are returned: <ul> <li> Job flows created and completed in the last two weeks </li> <li>  Job flows created within the last two months that are in one of the following states: <code>RUNNING</code>, <code>WAITING</code>, <code>SHUTTING_DOWN</code>, <code>STARTING</code>  </li> </ul> Amazon EMR can return a maximum of 512 job flow descriptions.

        :param self:
        :param JobFlowStates:  Return only job flows whose state is contained in this list.<br/>JobFlowIds:  Return only job flows whose job flow ID is contained in this list.<br/>CreatedBefore:  Return only job flows created before this date and time.<br/>CreatedAfter:  Return only job flows created after this date and time.<br/>
        :return: JobFlows:  A list of job flows matching the parameters supplied.<br/>        """
        pass

    def describe_security_configuration(self, Name: str) -> DescribeSecurityConfigurationOutput:
        """Provides the details of a security configuration by returning the configuration JSON.

        :param self:
        :param Name:  The name of the security configuration.<br/>
        :return: CreationDateTime:  The date and time the security configuration was created<br/>SecurityConfiguration:  The security configuration details in JSON format.<br/>Name:  The name of the security configuration.<br/>        """
        pass

    def describe_step(self, StepId: str, ClusterId: str) -> DescribeStepOutput:
        """Provides more detail about the cluster step.

        :param self:
        :param StepId:  The identifier of the step to describe.<br/>ClusterId:  The identifier of the cluster with steps to describe.<br/>
        :return: Step:  The step details for the requested step identifier.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current execution status details of the cluster step.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>: The current execution status details of the cluster step.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActionOnFailure</i> The action to take when the cluster step fails. Possible values are TERMINATE_CLUSTER, CANCEL_AND_WAIT, and CONTINUE. TERMINATE_JOB_FLOW is provided for backward compatibility. We recommend using TERMINATE_CLUSTER instead.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Config</i> The Hadoop job configuration of the cluster step.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Config</b>: The Hadoop job configuration of the cluster step.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the cluster step.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier of the cluster step.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Args</i> The list of command line arguments to pass to the JAR file's main function for execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MainClass</i> The name of the main class in the specified Java file. If not specified, the JAR file should specify a main class in its manifest file.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Properties</i> The list of Java properties that are set when the step runs. You can use these properties to pass key value pairs to your main function.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Jar</i> The path to the JAR file that runs during the step.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The execution state of the cluster step.<br/>The reason for the step execution status change.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StateChangeReason</b>: The reason for the step execution status change.        """
        pass

    def list_bootstrap_actions(self, ClusterId: str, Marker: str=None) -> ListBootstrapActionsOutput:
        """Provides information about the bootstrap actions associated with a cluster.

        :param self:
        :param Marker:  The pagination token that indicates the next set of results to retrieve.<br/>ClusterId:  The cluster identifier for the bootstrap actions to list.<br/>
        :return: Marker:  The pagination token that indicates the next set of results to retrieve.<br/>BootstrapActions:  The bootstrap actions associated with the cluster.<br/>        """
        pass

    def list_clusters(self, CreatedAfter: Date=None, CreatedBefore: Date=None, ClusterStates: List[ClusterState]=None, Marker: str=None) -> ListClustersOutput:
        """Provides the status of all clusters visible to this AWS account. Allows you to filter the list of clusters based on certain criteria; for example, filtering by cluster creation date and time or by status. This call returns a maximum of 50 clusters per call, but returns a marker to track the paging of the cluster list across multiple ListClusters calls.

        :param self:
        :param Marker:  The pagination token that indicates the next set of results to retrieve.<br/>ClusterStates:  The cluster state filters to apply when listing clusters.<br/>CreatedBefore:  The creation date and time end value filter for listing clusters.<br/>CreatedAfter:  The creation date and time beginning value filter for listing clusters.<br/>
        :return: Marker:  The pagination token that indicates the next set of results to retrieve.<br/>Clusters:  The list of clusters for the account based on the given filters.<br/>        """
        pass

    def list_instance_fleets(self, ClusterId: str, Marker: str=None) -> ListInstanceFleetsOutput:
        """Lists all available details about the instance fleets in a cluster. <note> The instance fleet configuration is available only in Amazon EMR versions 4.8.0 and later, excluding 5.0.x versions. </note>

        :param self:
        :param Marker:  The pagination token that indicates the next set of results to retrieve.<br/>ClusterId:  The unique identifier of the cluster.<br/>
        :return: Marker:  The pagination token that indicates the next set of results to retrieve.<br/>InstanceFleets:  The list of instance fleets for the cluster and given filters.<br/>        """
        pass

    def list_instance_groups(self, ClusterId: str, Marker: str=None) -> ListInstanceGroupsOutput:
        """Provides all available details about the instance groups in a cluster.

        :param self:
        :param Marker:  The pagination token that indicates the next set of results to retrieve.<br/>ClusterId:  The identifier of the cluster for which to list the instance groups.<br/>
        :return: Marker:  The pagination token that indicates the next set of results to retrieve.<br/>InstanceGroups:  The list of instance groups for the cluster and given filters.<br/>        """
        pass

    def list_instances(self, ClusterId: str, InstanceGroupId: str=None, InstanceGroupTypes: List[InstanceGroupType]=None, InstanceFleetId: str=None, InstanceFleetType: str=None, InstanceStates: List[InstanceState]=None, Marker: str=None) -> ListInstancesOutput:
        """Provides information for all active EC2 instances and EC2 instances terminated in the last 30 days, up to a maximum of 2,000. EC2 instances in any of the following states are considered active: AWAITING_FULFILLMENT, PROVISIONING, BOOTSTRAPPING, RUNNING.

        :param self:
        :param Marker:  The pagination token that indicates the next set of results to retrieve.<br/>InstanceStates:  A list of instance states that will filter the instances returned with this request.<br/>InstanceFleetType:  The node type of the instance fleet. For example MASTER, CORE, or TASK.<br/>InstanceFleetId:  The unique identifier of the instance fleet.<br/>InstanceGroupTypes:  The type of instance group for which to list the instances.<br/>InstanceGroupId:  The identifier of the instance group for which to list the instances.<br/>ClusterId:  The identifier of the cluster for which to list the instances.<br/>
        :return: Marker:  The pagination token that indicates the next set of results to retrieve.<br/>Instances:  The list of instances for the cluster and given filters.<br/>        """
        pass

    def list_security_configurations(self, Marker: str=None) -> ListSecurityConfigurationsOutput:
        """Lists all the security configurations visible to this account, providing their creation dates and times, and their names. This call returns a maximum of 50 clusters per call, but returns a marker to track the paging of the cluster list across multiple ListSecurityConfigurations calls.

        :param self:
        :param Marker:  The pagination token that indicates the set of results to retrieve.<br/>
        :return: Marker:  A pagination token that indicates the next set of results to retrieve. Include the marker in the next ListSecurityConfiguration call to retrieve the next page of results, if required.<br/>SecurityConfigurations:  The creation date and time, and name, of each security configuration.<br/>        """
        pass

    def list_steps(self, ClusterId: str, StepStates: List[StepState]=None, StepIds: List[XmlString]=None, Marker: str=None) -> ListStepsOutput:
        """Provides a list of steps for the cluster in reverse order unless you specify stepIds with the request.

        :param self:
        :param Marker:  The pagination token that indicates the next set of results to retrieve.<br/>StepIds:  The filter to limit the step list based on the identifier of the steps.<br/>StepStates:  The filter to limit the step list based on certain states.<br/>ClusterId:  The identifier of the cluster for which to list the steps.<br/>
        :return: Marker:  The pagination token that indicates the next set of results to retrieve.<br/>Steps:  The filtered list of steps for the cluster.<br/>        """
        pass

    def modify_instance_fleet(self, InstanceFleet: InstanceFleetModifyConfig, ClusterId: str):
        """Modifies the target On-Demand and target Spot capacities for the instance fleet with the specified InstanceFleetID within the cluster specified using ClusterID. The call either succeeds or fails atomically. <note> The instance fleet configuration is available only in Amazon EMR versions 4.8.0 and later, excluding 5.0.x versions. </note>

        :param self:
        :param InstanceFleet:  The unique identifier of the instance fleet.:<br/>ClusterId:  The unique identifier of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetSpotCapacity</i> The target capacity of Spot units for the instance fleet. For more information, see <a>InstanceFleetConfig$TargetSpotCapacity</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetOnDemandCapacity</i> The target capacity of On-Demand units for the instance fleet. For more information see <a>InstanceFleetConfig$TargetOnDemandCapacity</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceFleetId</i> A unique identifier for the instance fleet.<br/>
        :return:         """
        pass

    def modify_instance_groups(self, ClusterId: str=None, InstanceGroups: List[InstanceGroupModifyConfig]=None):
        """ModifyInstanceGroups modifies the number of nodes and configuration settings of an instance group. The input parameters include the new target instance count for the group and the instance group ID. The call will either succeed or fail atomically.

        :param self:
        :param InstanceGroups:  Instance groups to change.<br/>ClusterId:  The ID of the cluster to which the instance group belongs.<br/>
        :return:         """
        pass

    def put_auto_scaling_policy(self, AutoScalingPolicy: AutoScalingPolicy, InstanceGroupId: str, ClusterId: str) -> PutAutoScalingPolicyOutput:
        """Creates or updates an automatic scaling policy for a core instance group or task instance group in an Amazon EMR cluster. The automatic scaling policy defines how an instance group dynamically adds and terminates EC2 instances in response to the value of a CloudWatch metric.

        :param self:
        :param AutoScalingPolicy:  Specifies the definition of the automatic scaling policy.:<br/>InstanceGroupId:  Specifies the ID of the instance group to which the automatic scaling policy is applied.<br/>ClusterId:  Specifies the ID of a cluster. The instance group to which the automatic scaling policy is applied is within this cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Rules</i> The scale-in and scale-out rules that comprise the automatic scaling policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Constraints</i> The upper and lower EC2 instance limits for an automatic scaling policy. Automatic scaling activity will not cause an instance group to grow above or below these limits.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Constraints</b>: The upper and lower EC2 instance limits for an automatic scaling policy. Automatic scaling activity will not cause an instance group to grow above or below these limits.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxCapacity</i> The upper boundary of EC2 instances in an instance group beyond which scaling activities are not allowed to grow. Scale-out activities will not add instances beyond this boundary.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinCapacity</i> The lower boundary of EC2 instances in an instance group below which scaling activities are not allowed to shrink. Scale-in activities will not terminate instances below this boundary.<br/>
        :return: AutoScalingPolicy:  The automatic scaling policy definition.:<br/>InstanceGroupId:  Specifies the ID of the instance group to which the scaling policy is applied.<br/>ClusterId:  Specifies the ID of a cluster. The instance group to which the automatic scaling policy is applied is within this cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Rules</i> The scale-in and scale-out rules that comprise the automatic scaling policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Constraints</i> The upper and lower EC2 instance limits for an automatic scaling policy. Automatic scaling activity will not cause an instance group to grow above or below these limits.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Constraints</b>: The upper and lower EC2 instance limits for an automatic scaling policy. Automatic scaling activity will not cause an instance group to grow above or below these limits.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of an automatic scaling policy. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>: The status of an automatic scaling policy. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> Indicates the status of the automatic scaling policy.<br/>The reason for a change in status.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StateChangeReason</b>: The reason for a change in status.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxCapacity</i> The upper boundary of EC2 instances in an instance group beyond which scaling activities are not allowed to grow. Scale-out activities will not add instances beyond this boundary.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinCapacity</i> The lower boundary of EC2 instances in an instance group below which scaling activities are not allowed to shrink. Scale-in activities will not terminate instances below this boundary.<br/>        """
        pass

    def remove_auto_scaling_policy(self, InstanceGroupId: str, ClusterId: str) -> RemoveAutoScalingPolicyOutput:
        """Removes an automatic scaling policy from a specified instance group within an EMR cluster.

        :param self:
        :param InstanceGroupId:  Specifies the ID of the instance group to which the scaling policy is applied.<br/>ClusterId:  Specifies the ID of a cluster. The instance group to which the automatic scaling policy is applied is within this cluster.<br/>
        :return:         """
        pass

    def remove_tags(self, TagKeys: List[String], ResourceId: str) -> RemoveTagsOutput:
        """Removes tags from an Amazon EMR resource. Tags make it easier to associate clusters in various ways, such as grouping clusters to track your Amazon EMR resource allocation costs. For more information, see <a href="https://docs.aws.amazon.com/emr/latest/ManagementGuide/emr-plan-tags.html">Tag Clusters</a>.  The following example removes the stack tag with value Prod from a cluster:

        :param self:
        :param TagKeys:  A list of tag keys to remove from a resource.<br/>ResourceId:  The Amazon EMR resource identifier from which tags will be removed. This value must be a cluster identifier.<br/>
        :return:         """
        pass

    def run_job_flow(self, Instances: JobFlowInstancesConfig, Name: str, LogUri: str=None, AdditionalInfo: str=None, AmiVersion: str=None, ReleaseLabel: str=None, Steps: List[StepConfig]=None, BootstrapActions: List[BootstrapActionConfig]=None, SupportedProducts: List[XmlStringMaxLen256]=None, NewSupportedProducts: List[SupportedProductConfig]=None, Applications: List[Application]=None, Configurations: List[Configuration]=None, VisibleToAllUsers: Boolean=None, JobFlowRole: str=None, ServiceRole: str=None, Tags: List[Tag]=None, SecurityConfiguration: str=None, AutoScalingRole: str=None, ScaleDownBehavior: str=None, CustomAmiId: str=None, EbsRootVolumeSize: int=None, RepoUpgradeOnBoot: str=None, KerberosAttributes: KerberosAttributes=None) -> RunJobFlowOutput:
        """RunJobFlow creates and starts running a new cluster (job flow). The cluster runs the steps specified. After the steps complete, the cluster stops and the HDFS partition is lost. To prevent loss of data, configure the last step of the job flow to store results in Amazon S3. If the <a>JobFlowInstancesConfig</a> <code>KeepJobFlowAliveWhenNoSteps</code> parameter is set to <code>TRUE</code>, the cluster transitions to the WAITING state rather than shutting down after the steps have completed.  For additional protection, you can set the <a>JobFlowInstancesConfig</a> <code>TerminationProtected</code> parameter to <code>TRUE</code> to lock the cluster and prevent it from being terminated by API call, user intervention, or in the event of a job flow error. A maximum of 256 steps are allowed in each job flow. If your cluster is long-running (such as a Hive data warehouse) or complex, you may require more than 256 steps to process your data. You can bypass the 256-step limitation in various ways, including using the SSH shell to connect to the master node and submitting queries directly to the software running on the master node, such as Hive and Hadoop. For more information on how to do this, see <a href="https://docs.aws.amazon.com/emr/latest/ManagementGuide/AddMoreThan256Steps.html">Add More than 256 Steps to a Cluster</a> in the <i>Amazon EMR Management Guide</i>. For long running clusters, we recommend that you periodically store your results. <note> The instance fleets configuration is available only in Amazon EMR versions 4.8.0 and later, excluding 5.0.x versions. The RunJobFlow request can contain InstanceFleets parameters or InstanceGroups parameters, but not both. </note>

        :param self:
        :param KerberosAttributes:  Attributes for Kerberos configuration when Kerberos authentication is enabled using a security configuration. For more information see <a href="https://docs.aws.amazon.com/emr/latest/ManagementGuide/emr-kerberos.html">Use Kerberos Authentication</a> in the <i>EMR Management Guide</i>.:<br/>RepoUpgradeOnBoot:  Applies only when <code>CustomAmiID</code> is used. Specifies which updates from the Amazon Linux AMI package repositories to apply automatically when the instance boots using the AMI. If omitted, the default is <code>SECURITY</code>, which indicates that only security updates are applied. If <code>NONE</code> is specified, no updates are applied, and all updates must be applied manually.<br/>EbsRootVolumeSize:  The size, in GiB, of the EBS root device volume of the Linux AMI that is used for each EC2 instance. Available in Amazon EMR version 4.x and later.<br/>CustomAmiId:  Available only in Amazon EMR version 5.7.0 and later. The ID of a custom Amazon EBS-backed Linux AMI. If specified, Amazon EMR uses this AMI when it launches cluster EC2 instances. For more information about custom AMIs in Amazon EMR, see <a href="https://docs.aws.amazon.com/emr/latest/ManagementGuide/emr-custom-ami.html">Using a Custom AMI</a> in the <i>Amazon EMR Management Guide</i>. If omitted, the cluster uses the base Linux AMI for the <code>ReleaseLabel</code> specified. For Amazon EMR versions 2.x and 3.x, use <code>AmiVersion</code> instead. For information about creating a custom AMI, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/creating-an-ami-ebs.html">Creating an Amazon EBS-Backed Linux AMI</a> in the <i>Amazon Elastic Compute Cloud User Guide for Linux Instances</i>. For information about finding an AMI ID, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/finding-an-ami.html">Finding a Linux AMI</a>. <br/>ScaleDownBehavior:  Specifies the way that individual Amazon EC2 instances terminate when an automatic scale-in activity occurs or an instance group is resized. <code>TERMINATE_AT_INSTANCE_HOUR</code> indicates that Amazon EMR terminates nodes at the instance-hour boundary, regardless of when the request to terminate the instance was submitted. This option is only available with Amazon EMR 5.1.0 and later and is the default for clusters created using that version. <code>TERMINATE_AT_TASK_COMPLETION</code> indicates that Amazon EMR blacklists and drains tasks from nodes before terminating the Amazon EC2 instances, regardless of the instance-hour boundary. With either behavior, Amazon EMR removes the least active nodes first and blocks instance termination if it could lead to HDFS corruption. <code>TERMINATE_AT_TASK_COMPLETION</code> available only in Amazon EMR version 4.1.0 and later, and is the default for versions of Amazon EMR earlier than 5.1.0.<br/>AutoScalingRole:  An IAM role for automatic scaling policies. The default role is <code>EMR_AutoScaling_DefaultRole</code>. The IAM role provides permissions that the automatic scaling feature requires to launch and terminate EC2 instances in an instance group.<br/>SecurityConfiguration:  The name of a security configuration to apply to the cluster.<br/>Tags:  A list of tags to associate with a cluster and propagate to Amazon EC2 instances.<br/>ServiceRole:  The IAM role that will be assumed by the Amazon EMR service to access AWS resources on your behalf.<br/>JobFlowRole:  Also called instance profile and EC2 role. An IAM role for an EMR cluster. The EC2 instances of the cluster assume this role. The default role is <code>EMR_EC2_DefaultRole</code>. In order to use the default role, you must have already created it using the CLI or console.<br/>VisibleToAllUsers:  Whether the cluster is visible to all IAM users of the AWS account associated with the cluster. If this value is set to <code>true</code>, all IAM users of that AWS account can view and (if they have the proper policy permissions set) manage the cluster. If it is set to <code>false</code>, only the IAM user that created the cluster can view and manage it.<br/>Configurations:  For Amazon EMR releases 4.0 and later. The list of configurations supplied for the EMR cluster you are creating.<br/>Applications:  Applies to Amazon EMR releases 4.0 and later. A case-insensitive list of applications for Amazon EMR to install and configure when launching the cluster. For a list of applications available for each Amazon EMR release version, see the <a href="https://docs.aws.amazon.com/emr/latest/ReleaseGuide/">Amazon EMR Release Guide</a>.<br/>NewSupportedProducts:  <note> For Amazon EMR releases 3.x and 2.x. For Amazon EMR releases 4.x and later, use Applications. </note> A list of strings that indicates third-party software to use with the job flow that accepts a user argument list. EMR accepts and forwards the argument list to the corresponding installation script as bootstrap action arguments. For more information, see "Launch a Job Flow on the MapR Distribution for Hadoop" in the <a href="https://docs.aws.amazon.com/emr/latest/DeveloperGuide/emr-dg.pdf">Amazon EMR Developer Guide</a>. Supported values are: <ul> <li> "mapr-m3" - launch the cluster using MapR M3 Edition. </li> <li> "mapr-m5" - launch the cluster using MapR M5 Edition. </li> <li> "mapr" with the user arguments specifying "--edition,m3" or "--edition,m5" - launch the job flow using MapR M3 or M5 Edition respectively. </li> <li> "mapr-m7" - launch the cluster using MapR M7 Edition. </li> <li> "hunk" - launch the cluster with the Hunk Big Data Analtics Platform. </li> <li> "hue"- launch the cluster with Hue installed. </li> <li> "spark" - launch the cluster with Apache Spark installed. </li> <li> "ganglia" - launch the cluster with the Ganglia Monitoring System installed. </li> </ul><br/>SupportedProducts:  <note> For Amazon EMR releases 3.x and 2.x. For Amazon EMR releases 4.x and later, use Applications. </note> A list of strings that indicates third-party software to use. For more information, see the <a href="https://docs.aws.amazon.com/emr/latest/DeveloperGuide/emr-dg.pdf">Amazon EMR Developer Guide</a>. Currently supported values are: <ul> <li> "mapr-m3" - launch the job flow using MapR M3 Edition. </li> <li> "mapr-m5" - launch the job flow using MapR M5 Edition. </li> </ul><br/>BootstrapActions:  A list of bootstrap actions to run before Hadoop starts on the cluster nodes.<br/>Steps:  A list of steps to run.<br/>Instances:  A specification of the number and type of Amazon EC2 instances.:<br/>ReleaseLabel:  The Amazon EMR release label, which determines the version of open-source application packages installed on the cluster. Release labels are in the form <code>emr-x.x.x</code>, where x.x.x is an Amazon EMR release version, for example, <code>emr-5.14.0</code>. For more information about Amazon EMR release versions and included application versions and features, see <a href="https://docs.aws.amazon.com/emr/latest/ReleaseGuide/">https://docs.aws.amazon.com/emr/latest/ReleaseGuide/</a>. The release label applies only to Amazon EMR releases versions 4.x and later. Earlier versions use <code>AmiVersion</code>.<br/>AmiVersion:  Applies only to Amazon EMR AMI versions 3.x and 2.x. For Amazon EMR releases 4.0 and later, <code>ReleaseLabel</code> is used. To specify a custom AMI, use <code>CustomAmiID</code>.<br/>AdditionalInfo:  A JSON string for selecting additional features.<br/>LogUri:  The location in Amazon S3 to write the log files of the job flow. If a value is not provided, logs are not created.<br/>Name:  The name of the job flow.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdditionalSlaveSecurityGroups</i> A list of additional Amazon EC2 security group IDs for the core and task nodes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdditionalMasterSecurityGroups</i> A list of additional Amazon EC2 security group IDs for the master node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceAccessSecurityGroup</i> The identifier of the Amazon EC2 security group for the Amazon EMR service to access clusters in VPC private subnets.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EmrManagedSlaveSecurityGroup</i> The identifier of the Amazon EC2 security group for the core and task nodes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EmrManagedMasterSecurityGroup</i> The identifier of the Amazon EC2 security group for the master node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Ec2SubnetIds</i> Applies to clusters that use the instance fleet configuration. When multiple EC2 subnet IDs are specified, Amazon EMR evaluates them and launches instances in the optimal subnet. <note> The instance fleet configuration is available only in Amazon EMR versions 4.8.0 and later, excluding 5.0.x versions. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Ec2SubnetId</i> Applies to clusters that use the uniform instance group configuration. To launch the cluster in Amazon Virtual Private Cloud (Amazon VPC), set this parameter to the identifier of the Amazon VPC subnet where you want the cluster to launch. If you do not specify this value, the cluster launches in the normal Amazon Web Services cloud, outside of an Amazon VPC, if the account launching the cluster supports EC2 Classic networks in the region where the cluster launches. Amazon VPC currently does not support cluster compute quadruple extra large (cc1.4xlarge) instances. Thus you cannot specify the cc1.4xlarge instance type for clusters launched in an Amazon VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HadoopVersion</i> Applies only to Amazon EMR release versions earlier than 4.0. The Hadoop version for the cluster. Valid inputs are "0.18" (deprecated), "0.20" (deprecated), "0.20.205" (deprecated), "1.0.3", "2.2.0", or "2.4.0". If you do not set this value, the default of 0.18 is used, unless the <code>AmiVersion</code> parameter is set in the RunJobFlow call, in which case the default version of Hadoop for that AMI version is used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TerminationProtected</i> Specifies whether to lock the cluster to prevent the Amazon EC2 instances from being terminated by API call, user intervention, or in the event of a job-flow error.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeepJobFlowAliveWhenNoSteps</i> Specifies whether the cluster should remain available after completing all steps.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Placement</i> The Availability Zone in which the cluster runs.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Placement</b>: The Availability Zone in which the cluster runs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Ec2KeyName</i> The name of the EC2 key pair that can be used to ssh to the master node as the user called "hadoop."<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceFleets</i> <note> The instance fleet configuration is available only in Amazon EMR versions 4.8.0 and later, excluding 5.0.x versions. </note> Describes the EC2 instances and instance configurations for clusters that use the instance fleet configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceGroups</i> Configuration for the instance groups in a cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCount</i> The number of EC2 instances in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SlaveInstanceType</i> The EC2 instance type of the core and task nodes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterInstanceType</i> The EC2 instance type of the master node.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> When multiple Availability Zones are specified, Amazon EMR evaluates them and launches instances in the optimal Availability Zone. <code>AvailabilityZones</code> is used for instance fleets, while <code>AvailabilityZone</code> (singular) is used for uniform instance groups. <note> The instance fleet configuration is available only in Amazon EMR versions 4.8.0 and later, excluding 5.0.x versions. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Amazon EC2 Availability Zone for the cluster. <code>AvailabilityZone</code> is used for uniform instance groups, while <code>AvailabilityZones</code> (plural) is used for instance fleets.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ADDomainJoinPassword</i> The Active Directory password for <code>ADDomainJoinUser</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ADDomainJoinUser</i> Required only when establishing a cross-realm trust with an Active Directory domain. A user with sufficient privileges to join resources to the domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CrossRealmTrustPrincipalPassword</i> Required only when establishing a cross-realm trust with a KDC in a different realm. The cross-realm principal password, which must be identical across realms.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KdcAdminPassword</i> The password used within the cluster for the kadmin service on the cluster-dedicated KDC, which maintains Kerberos principals, password policies, and keytabs for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Realm</i> The name of the Kerberos realm to which all nodes in a cluster belong. For example, <code>EC2.INTERNAL</code>. <br/>
        :return: JobFlowId:  An unique identifier for the job flow.<br/>        """
        pass

    def set_termination_protection(self, TerminationProtected: Boolean, JobFlowIds: List[XmlString]):
        """SetTerminationProtection locks a cluster (job flow) so the EC2 instances in the cluster cannot be terminated by user intervention, an API call, or in the event of a job-flow error. The cluster still terminates upon successful completion of the job flow. Calling <code>SetTerminationProtection</code> on a cluster is similar to calling the Amazon EC2 <code>DisableAPITermination</code> API on all EC2 instances in a cluster.  <code>SetTerminationProtection</code> is used to prevent accidental termination of a cluster and to ensure that in the event of an error, the instances persist so that you can recover any data stored in their ephemeral instance storage.  To terminate a cluster that has been locked by setting <code>SetTerminationProtection</code> to <code>true</code>, you must first unlock the job flow by a subsequent call to <code>SetTerminationProtection</code> in which you set the value to <code>false</code>.   For more information, see<a href="https://docs.aws.amazon.com/emr/latest/ManagementGuide/UsingEMR_TerminationProtection.html">Managing Cluster Termination</a> in the <i>Amazon EMR Management Guide</i>. 

        :param self:
        :param TerminationProtected:  A Boolean that indicates whether to protect the cluster and prevent the Amazon EC2 instances in the cluster from shutting down due to API calls, user intervention, or job-flow error.<br/>JobFlowIds:   A list of strings that uniquely identify the clusters to protect. This identifier is returned by <a>RunJobFlow</a> and can also be obtained from <a>DescribeJobFlows</a> . <br/>
        :return:         """
        pass

    def set_visible_to_all_users(self, VisibleToAllUsers: Boolean, JobFlowIds: List[XmlString]):
        """Sets whether all AWS Identity and Access Management (IAM) users under your account can access the specified clusters (job flows). This action works on running clusters. You can also set the visibility of a cluster when you launch it using the <code>VisibleToAllUsers</code> parameter of <a>RunJobFlow</a>. The SetVisibleToAllUsers action can be called only by an IAM user who created the cluster or the AWS account that owns the cluster.

        :param self:
        :param VisibleToAllUsers:  Whether the specified clusters are visible to all IAM users of the AWS account associated with the cluster. If this value is set to True, all IAM users of that AWS account can view and, if they have the proper IAM policy permissions set, manage the clusters. If it is set to False, only the IAM user that created a cluster can view and manage it.<br/>JobFlowIds:  Identifiers of the job flows to receive the new visibility setting.<br/>
        :return:         """
        pass

    def terminate_job_flows(self, JobFlowIds: List[XmlString]):
        """TerminateJobFlows shuts a list of clusters (job flows) down. When a job flow is shut down, any step not yet completed is canceled and the EC2 instances on which the cluster is running are stopped. Any log files not already saved are uploaded to Amazon S3 if a LogUri was specified when the cluster was created. The maximum number of clusters allowed is 10. The call to <code>TerminateJobFlows</code> is asynchronous. Depending on the configuration of the cluster, it may take up to 1-5 minutes for the cluster to completely terminate and release allocated resources, such as Amazon EC2 instances.

        :param self:
        :param JobFlowIds:  A list of job flows to be shutdown.<br/>
        :return:         """
        pass



class ElasticsearchService(BaseClient):
    """<fullname>Amazon Elasticsearch Configuration Service</fullname> Use the Amazon Elasticsearch configuration API to create, configure, and manage Elasticsearch domains. The endpoint for configuration service requests is region-specific: es.<i>region</i>.amazonaws.com. For example, es.us-east-1.amazonaws.com. For a current list of supported regions and endpoints, see <a href="http://docs.aws.amazon.com/general/latest/gr/rande.html#elasticsearch-service-regions" target="_blank">Regions and Endpoints</a>."""


    class ListElasticsearchInstanceTypesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_elasticsearch_instance_types` for valid parameters"""
            pass
    class ListElasticsearchVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_elasticsearch_versions` for valid parameters"""
            pass
    class DescribeReservedElasticsearchInstanceOfferingsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_reserved_elasticsearch_instance_offerings` for valid parameters"""
            pass
    class DescribeReservedElasticsearchInstancesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_reserved_elasticsearch_instances` for valid parameters"""
            pass
    class GetUpgradeHistoryPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_upgrade_history` for valid parameters"""
            pass


    class AddTagsRequest(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CancelElasticsearchServiceSoftwareUpdateRequest(Mapping):
        pass
    
    class CancelElasticsearchServiceSoftwareUpdateResponse(Mapping):
        pass
    
    class CreateElasticsearchDomainRequest(Mapping):
        pass
    
    class CreateElasticsearchDomainResponse(Mapping):
        pass
    
    class ElasticsearchClusterConfig(Mapping):
        pass
    
    class EBSOptions(Mapping):
        pass
    
    class SnapshotOptions(Mapping):
        pass
    
    class VPCOptions(Mapping):
        pass
    
    class CognitoOptions(Mapping):
        pass
    
    class EncryptionAtRestOptions(Mapping):
        pass
    
    class NodeToNodeEncryptionOptions(Mapping):
        pass
    
    class AdvancedOptions(object):
        pass
    
    class LogPublishingOptions(object):
        pass
    
    class DeleteElasticsearchDomainRequest(Mapping):
        pass
    
    class DeleteElasticsearchDomainResponse(Mapping):
        pass
    
    class DescribeElasticsearchDomainRequest(Mapping):
        pass
    
    class DescribeElasticsearchDomainResponse(Mapping):
        pass
    
    class DescribeElasticsearchDomainConfigRequest(Mapping):
        pass
    
    class DescribeElasticsearchDomainConfigResponse(Mapping):
        pass
    
    class DescribeElasticsearchDomainsRequest(Mapping):
        pass
    
    class DescribeElasticsearchDomainsResponse(Mapping):
        pass
    
    class DomainName(object):
        pass
    
    class DescribeElasticsearchInstanceTypeLimitsRequest(Mapping):
        pass
    
    class DescribeElasticsearchInstanceTypeLimitsResponse(Mapping):
        pass
    
    class DescribeReservedElasticsearchInstanceOfferingsRequest(Mapping):
        pass
    
    class DescribeReservedElasticsearchInstanceOfferingsResponse(Mapping):
        pass
    
    class DescribeReservedElasticsearchInstancesRequest(Mapping):
        pass
    
    class DescribeReservedElasticsearchInstancesResponse(Mapping):
        pass
    
    class GetCompatibleElasticsearchVersionsRequest(Mapping):
        pass
    
    class GetCompatibleElasticsearchVersionsResponse(Mapping):
        pass
    
    class GetUpgradeHistoryRequest(Mapping):
        pass
    
    class GetUpgradeHistoryResponse(Mapping):
        pass
    
    class GetUpgradeStatusRequest(Mapping):
        pass
    
    class GetUpgradeStatusResponse(Mapping):
        pass
    
    class ListDomainNamesResponse(Mapping):
        pass
    
    class ListElasticsearchInstanceTypesRequest(Mapping):
        pass
    
    class ListElasticsearchInstanceTypesResponse(Mapping):
        pass
    
    class ListElasticsearchVersionsRequest(Mapping):
        pass
    
    class ListElasticsearchVersionsResponse(Mapping):
        pass
    
    class ListTagsRequest(Mapping):
        pass
    
    class ListTagsResponse(Mapping):
        pass
    
    class PurchaseReservedElasticsearchInstanceOfferingRequest(Mapping):
        pass
    
    class PurchaseReservedElasticsearchInstanceOfferingResponse(Mapping):
        pass
    
    class RemoveTagsRequest(Mapping):
        pass
    
    class String(object):
        pass
    
    class StartElasticsearchServiceSoftwareUpdateRequest(Mapping):
        pass
    
    class StartElasticsearchServiceSoftwareUpdateResponse(Mapping):
        pass
    
    class UpdateElasticsearchDomainConfigRequest(Mapping):
        pass
    
    class UpdateElasticsearchDomainConfigResponse(Mapping):
        pass
    
    class UpgradeElasticsearchDomainRequest(Mapping):
        pass
    
    class UpgradeElasticsearchDomainResponse(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    def add_tags(self, TagList: List[Tag], ARN: str):
        """Attaches tags to an existing Elasticsearch domain. Tags are a set of case-sensitive key value pairs. An Elasticsearch domain may have up to 10 tags. See <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-managedomains.html#es-managedomains-awsresorcetagging" target="_blank"> Tagging Amazon Elasticsearch Service Domains for more information.</a>

        :param self:
        :param TagList:   List of <code>Tag</code> that need to be added for the Elasticsearch domain. <br/>ARN:   Specify the <code>ARN</code> for which you want to add the tags.<br/>
        :return:         """
        pass

    def cancel_elasticsearch_service_software_update(self, DomainName: str) -> CancelElasticsearchServiceSoftwareUpdateResponse:
        """Cancels a scheduled service software update for an Amazon ES domain. You can only perform this operation before the <code>AutomatedUpdateDate</code> and when the <code>UpdateStatus</code> is in the <code>PENDING_UPDATE</code> state.

        :param self:
        :param DomainName:  The name of the domain that you want to stop the latest service software update on.<br/>
        :return: ServiceSoftwareOptions:  The current status of the Elasticsearch service software update.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedUpdateDate</i> Timestamp, in Epoch time, until which you can manually request a service software update. After this date, we automatically update your service software.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the <code>UpdateStatus</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateStatus</i> The status of your service software update. This field can take the following values: <code>ELIGIBLE</code>, <code>PENDING_UPDATE</code>, <code>IN_PROGRESS</code>, <code>COMPLETED</code>, and <code>NOT_ELIGIBLE</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Cancellable</i> <code>True</code> if you are able to cancel your service software version update. <code>False</code> if you are not able to cancel your service software version. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateAvailable</i> <code>True</code> if you are able to update you service software version. <code>False</code> if you are not able to update your service software version. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NewVersion</i> The new service software version if one is available.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentVersion</i> The current service software version that is present on the domain.<br/>        """
        pass

    def create_elasticsearch_domain(self, DomainName: str, ElasticsearchVersion: str=None, ElasticsearchClusterConfig: ElasticsearchClusterConfig=None, EBSOptions: EBSOptions=None, AccessPolicies: str=None, SnapshotOptions: SnapshotOptions=None, VPCOptions: VPCOptions=None, CognitoOptions: CognitoOptions=None, EncryptionAtRestOptions: EncryptionAtRestOptions=None, NodeToNodeEncryptionOptions: NodeToNodeEncryptionOptions=None, AdvancedOptions: AdvancedOptions=None, LogPublishingOptions: LogPublishingOptions=None) -> CreateElasticsearchDomainResponse:
        """Creates a new Elasticsearch domain. For more information, see <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-createupdatedomains.html#es-createdomains" target="_blank">Creating Elasticsearch Domains</a> in the <i>Amazon Elasticsearch Service Developer Guide</i>.

        :param self:
        :param LogPublishingOptions:  Map of <code>LogType</code> and <code>LogPublishingOption</code>, each containing options to publish a given type of Elasticsearch log.<br/>AdvancedOptions:   Option to allow references to indices in an HTTP request body. Must be <code>false</code> when configuring access to individual sub-resources. By default, the value is <code>true</code>. See <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-createupdatedomains.html#es-createdomain-configure-advanced-options" target="_blank">Configuration Advanced Options</a> for more information.<br/>NodeToNodeEncryptionOptions:  Specifies the NodeToNodeEncryptionOptions.:<br/>EncryptionAtRestOptions:  Specifies the Encryption At Rest Options.:<br/>CognitoOptions:  Options to specify the Cognito user and identity pools for Kibana authentication. For more information, see <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-cognito-auth.html" target="_blank">Amazon Cognito Authentication for Kibana</a>.:<br/>VPCOptions:  Options to specify the subnets and security groups for VPC endpoint. For more information, see <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-vpc.html#es-creating-vpc" target="_blank">Creating a VPC</a> in <i>VPC Endpoints for Amazon Elasticsearch Service Domains</i>:<br/>SnapshotOptions:  Option to set time, in UTC format, of the daily automated snapshot. Default value is 0 hours. :<br/>AccessPolicies:   IAM access policy as a JSON-formatted string.<br/>EBSOptions:  Options to enable, disable and specify the type and size of EBS storage volumes. :<br/>ElasticsearchClusterConfig:  Configuration options for an Elasticsearch domain. Specifies the instance type and number of instances in the domain cluster. :<br/>ElasticsearchVersion:  String of format X.Y to specify version for the Elasticsearch domain eg. "1.5" or "2.3". For more information, see <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-createupdatedomains.html#es-createdomains" target="_blank">Creating Elasticsearch Domains</a> in the <i>Amazon Elasticsearch Service Developer Guide</i>.<br/>DomainName:  The name of the Elasticsearch domain that you are creating. Domain names are unique across the domains owned by an account within an AWS region. Domain names must start with a letter or number and can contain the following characters: a-z (lowercase), 0-9, and - (hyphen).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DedicatedMasterCount</i> Total number of dedicated master nodes, active and on standby, for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DedicatedMasterType</i> The instance type for a dedicated master node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ZoneAwarenessConfig</i> Specifies the zone awareness configuration for a domain when zone awareness is enabled.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ZoneAwarenessConfig</b>: Specifies the zone awareness configuration for a domain when zone awareness is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ZoneAwarenessEnabled</i> A boolean value to indicate whether zone awareness is enabled. See <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-managedomains.html#es-managedomains-zoneawareness" target="_blank">About Zone Awareness</a> for more information.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DedicatedMasterEnabled</i> A boolean value to indicate whether a dedicated master node is enabled. See <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-managedomains.html#es-managedomains-dedicatedmasternodes" target="_blank">About Dedicated Master Nodes</a> for more information.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCount</i> The number of instances in the specified domain cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i> The instance type for an Elasticsearch cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZoneCount</i> An integer value to indicate the number of availability zones for a domain when zone awareness is enabled. This should be equal to number of subnets if VPC endpoints is enabled<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the IOPD for a Provisioned IOPS EBS volume (SSD).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeSize</i>  Integer to specify the size of an EBS volume.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeType</i>  Specifies the volume type for EBS-based storage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EBSEnabled</i> Specifies whether EBS-based storage is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotStartHour</i> Specifies the time, in UTC format, when the service takes a daily automated snapshot of the specified Elasticsearch domain. Default value is <code>0</code> hours.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> Specifies the security groups for VPC endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> Specifies the subnets for VPC endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> Specifies the role ARN that provides Elasticsearch permissions for accessing Cognito resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IdentityPoolId</i> Specifies the Cognito identity pool ID for Kibana authentication.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolId</i> Specifies the Cognito user pool ID for Kibana authentication.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies the option to enable Cognito for Kibana authentication.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i>  Specifies the KMS Key ID for Encryption At Rest options.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies the option to enable Encryption At Rest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specify true to enable node-to-node encryption.<br/>
        :return: DomainStatus:  The status of the newly created Elasticsearch domain. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceSoftwareOptions</i> The current status of the Elasticsearch domain's service software.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ServiceSoftwareOptions</b>: The current status of the Elasticsearch domain's service software.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogPublishingOptions</i> Log publishing options for the given domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdvancedOptions</i> Specifies the status of the <code>AdvancedOptions</code><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeToNodeEncryptionOptions</i> Specifies the status of the <code>NodeToNodeEncryptionOptions</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NodeToNodeEncryptionOptions</b>: Specifies the status of the <code>NodeToNodeEncryptionOptions</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionAtRestOptions</i>  Specifies the status of the <code>EncryptionAtRestOptions</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncryptionAtRestOptions</b>:  Specifies the status of the <code>EncryptionAtRestOptions</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CognitoOptions</i> The <code>CognitoOptions</code> for the specified domain. For more information, see <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-cognito-auth.html" target="_blank">Amazon Cognito Authentication for Kibana</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CognitoOptions</b>: The <code>CognitoOptions</code> for the specified domain. For more information, see <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-cognito-auth.html" target="_blank">Amazon Cognito Authentication for Kibana</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VPCOptions</i> The <code>VPCOptions</code> for the specified domain. For more information, see <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-vpc.html" target="_blank">VPC Endpoints for Amazon Elasticsearch Service Domains</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VPCOptions</b>: The <code>VPCOptions</code> for the specified domain. For more information, see <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-vpc.html" target="_blank">VPC Endpoints for Amazon Elasticsearch Service Domains</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotOptions</i> Specifies the status of the <code>SnapshotOptions</code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SnapshotOptions</b>: Specifies the status of the <code>SnapshotOptions</code><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccessPolicies</i>  IAM access policy as a JSON-formatted string.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EBSOptions</i> The <code>EBSOptions</code> for the specified domain. See <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-createupdatedomains.html#es-createdomain-configure-ebs" target="_blank">Configuring EBS-based Storage</a> for more information.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EBSOptions</b>: The <code>EBSOptions</code> for the specified domain. See <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-createupdatedomains.html#es-createdomain-configure-ebs" target="_blank">Configuring EBS-based Storage</a> for more information.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticsearchClusterConfig</i> The type and number of instances in the domain cluster.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ElasticsearchClusterConfig</b>: The type and number of instances in the domain cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticsearchVersion</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpgradeProcessing</i> The status of an Elasticsearch domain version upgrade. <code>True</code> if Amazon Elasticsearch Service is undergoing a version upgrade. <code>False</code> if the configuration is active.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Processing</i> The status of the Elasticsearch domain configuration. <code>True</code> if Amazon Elasticsearch Service is processing configuration changes. <code>False</code> if the configuration is active.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoints</i> Map containing the Elasticsearch domain endpoints used to submit index and search requests. Example <code>key, value</code>: <code>'vpc','vpc-endpoint-h2dsd34efgyghrtguk5gt6j2foh4.us-east-1.es.amazonaws.com'</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> The Elasticsearch domain endpoint that you use to submit index and search requests.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Deleted</i> The domain deletion status. <code>True</code> if a delete request has been received for the domain but resource cleanup is still in progress. <code>False</code> if the domain has not been deleted. Once domain deletion is complete, the status of the domain is no longer returned.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Created</i> The domain creation status. <code>True</code> if the creation of an Elasticsearch domain is complete. <code>False</code> if domain creation is still in progress.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ARN</i> The Amazon resource name (ARN) of an Elasticsearch domain. See <a href="http://docs.aws.amazon.com/IAM/latest/UserGuide/index.html?Using_Identifiers.html" target="_blank">Identifiers for IAM Entities</a> in <i>Using AWS Identity and Access Management</i> for more information.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainName</i> The name of an Elasticsearch domain. Domain names are unique across the domains owned by an account within an AWS region. Domain names start with a letter or number and can contain the following characters: a-z (lowercase), 0-9, and - (hyphen).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainId</i> The unique identifier for the specified Elasticsearch domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ZoneAwarenessEnabled</i> A boolean value to indicate whether zone awareness is enabled. See <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-managedomains.html#es-managedomains-zoneawareness" target="_blank">About Zone Awareness</a> for more information.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DedicatedMasterEnabled</i> A boolean value to indicate whether a dedicated master node is enabled. See <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-managedomains.html#es-managedomains-dedicatedmasternodes" target="_blank">About Dedicated Master Nodes</a> for more information.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCount</i> The number of instances in the specified domain cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i> The instance type for an Elasticsearch cluster.<br/>Specifies the zone awareness configuration for a domain when zone awareness is enabled.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ZoneAwarenessConfig</b>: Specifies the zone awareness configuration for a domain when zone awareness is enabled.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the IOPD for a Provisioned IOPS EBS volume (SSD).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeSize</i>  Integer to specify the size of an EBS volume.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeType</i>  Specifies the volume type for EBS-based storage.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EBSEnabled</i> Specifies whether EBS-based storage is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotStartHour</i> Specifies the time, in UTC format, when the service takes a daily automated snapshot of the specified Elasticsearch domain. Default value is <code>0</code> hours.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> Specifies the security groups for VPC endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> The availability zones for the Elasticsearch domain. Exists only if the domain was created with VPCOptions.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> Specifies the subnets for VPC endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VPCId</i> The VPC Id for the Elasticsearch domain. Exists only if the domain was created with VPCOptions.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> Specifies the role ARN that provides Elasticsearch permissions for accessing Cognito resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IdentityPoolId</i> Specifies the Cognito identity pool ID for Kibana authentication.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolId</i> Specifies the Cognito user pool ID for Kibana authentication.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies the option to enable Cognito for Kibana authentication.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i>  Specifies the KMS Key ID for Encryption At Rest options.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies the option to enable Encryption At Rest.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specify true to enable node-to-node encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedUpdateDate</i> Timestamp, in Epoch time, until which you can manually request a service software update. After this date, we automatically update your service software.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the <code>UpdateStatus</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateStatus</i> The status of your service software update. This field can take the following values: <code>ELIGIBLE</code>, <code>PENDING_UPDATE</code>, <code>IN_PROGRESS</code>, <code>COMPLETED</code>, and <code>NOT_ELIGIBLE</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Cancellable</i> <code>True</code> if you are able to cancel your service software version update. <code>False</code> if you are not able to cancel your service software version. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateAvailable</i> <code>True</code> if you are able to update you service software version. <code>False</code> if you are not able to update your service software version. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NewVersion</i> The new service software version if one is available.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentVersion</i> The current service software version that is present on the domain.<br/>        """
        pass

    def delete_elasticsearch_domain(self, DomainName: str) -> DeleteElasticsearchDomainResponse:
        """Permanently deletes the specified Elasticsearch domain and all of its data. Once a domain is deleted, it cannot be recovered.

        :param self:
        :param DomainName:  The name of the Elasticsearch domain that you want to permanently delete.<br/>
        :return: DomainStatus:  The status of the Elasticsearch domain being deleted.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceSoftwareOptions</i> The current status of the Elasticsearch domain's service software.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ServiceSoftwareOptions</b>: The current status of the Elasticsearch domain's service software.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogPublishingOptions</i> Log publishing options for the given domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdvancedOptions</i> Specifies the status of the <code>AdvancedOptions</code><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeToNodeEncryptionOptions</i> Specifies the status of the <code>NodeToNodeEncryptionOptions</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NodeToNodeEncryptionOptions</b>: Specifies the status of the <code>NodeToNodeEncryptionOptions</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionAtRestOptions</i>  Specifies the status of the <code>EncryptionAtRestOptions</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncryptionAtRestOptions</b>:  Specifies the status of the <code>EncryptionAtRestOptions</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CognitoOptions</i> The <code>CognitoOptions</code> for the specified domain. For more information, see <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-cognito-auth.html" target="_blank">Amazon Cognito Authentication for Kibana</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CognitoOptions</b>: The <code>CognitoOptions</code> for the specified domain. For more information, see <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-cognito-auth.html" target="_blank">Amazon Cognito Authentication for Kibana</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VPCOptions</i> The <code>VPCOptions</code> for the specified domain. For more information, see <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-vpc.html" target="_blank">VPC Endpoints for Amazon Elasticsearch Service Domains</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VPCOptions</b>: The <code>VPCOptions</code> for the specified domain. For more information, see <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-vpc.html" target="_blank">VPC Endpoints for Amazon Elasticsearch Service Domains</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotOptions</i> Specifies the status of the <code>SnapshotOptions</code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SnapshotOptions</b>: Specifies the status of the <code>SnapshotOptions</code><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccessPolicies</i>  IAM access policy as a JSON-formatted string.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EBSOptions</i> The <code>EBSOptions</code> for the specified domain. See <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-createupdatedomains.html#es-createdomain-configure-ebs" target="_blank">Configuring EBS-based Storage</a> for more information.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EBSOptions</b>: The <code>EBSOptions</code> for the specified domain. See <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-createupdatedomains.html#es-createdomain-configure-ebs" target="_blank">Configuring EBS-based Storage</a> for more information.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticsearchClusterConfig</i> The type and number of instances in the domain cluster.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ElasticsearchClusterConfig</b>: The type and number of instances in the domain cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticsearchVersion</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpgradeProcessing</i> The status of an Elasticsearch domain version upgrade. <code>True</code> if Amazon Elasticsearch Service is undergoing a version upgrade. <code>False</code> if the configuration is active.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Processing</i> The status of the Elasticsearch domain configuration. <code>True</code> if Amazon Elasticsearch Service is processing configuration changes. <code>False</code> if the configuration is active.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoints</i> Map containing the Elasticsearch domain endpoints used to submit index and search requests. Example <code>key, value</code>: <code>'vpc','vpc-endpoint-h2dsd34efgyghrtguk5gt6j2foh4.us-east-1.es.amazonaws.com'</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> The Elasticsearch domain endpoint that you use to submit index and search requests.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Deleted</i> The domain deletion status. <code>True</code> if a delete request has been received for the domain but resource cleanup is still in progress. <code>False</code> if the domain has not been deleted. Once domain deletion is complete, the status of the domain is no longer returned.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Created</i> The domain creation status. <code>True</code> if the creation of an Elasticsearch domain is complete. <code>False</code> if domain creation is still in progress.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ARN</i> The Amazon resource name (ARN) of an Elasticsearch domain. See <a href="http://docs.aws.amazon.com/IAM/latest/UserGuide/index.html?Using_Identifiers.html" target="_blank">Identifiers for IAM Entities</a> in <i>Using AWS Identity and Access Management</i> for more information.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainName</i> The name of an Elasticsearch domain. Domain names are unique across the domains owned by an account within an AWS region. Domain names start with a letter or number and can contain the following characters: a-z (lowercase), 0-9, and - (hyphen).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainId</i> The unique identifier for the specified Elasticsearch domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ZoneAwarenessEnabled</i> A boolean value to indicate whether zone awareness is enabled. See <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-managedomains.html#es-managedomains-zoneawareness" target="_blank">About Zone Awareness</a> for more information.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DedicatedMasterEnabled</i> A boolean value to indicate whether a dedicated master node is enabled. See <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-managedomains.html#es-managedomains-dedicatedmasternodes" target="_blank">About Dedicated Master Nodes</a> for more information.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCount</i> The number of instances in the specified domain cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i> The instance type for an Elasticsearch cluster.<br/>Specifies the zone awareness configuration for a domain when zone awareness is enabled.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ZoneAwarenessConfig</b>: Specifies the zone awareness configuration for a domain when zone awareness is enabled.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the IOPD for a Provisioned IOPS EBS volume (SSD).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeSize</i>  Integer to specify the size of an EBS volume.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeType</i>  Specifies the volume type for EBS-based storage.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EBSEnabled</i> Specifies whether EBS-based storage is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotStartHour</i> Specifies the time, in UTC format, when the service takes a daily automated snapshot of the specified Elasticsearch domain. Default value is <code>0</code> hours.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> Specifies the security groups for VPC endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> The availability zones for the Elasticsearch domain. Exists only if the domain was created with VPCOptions.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> Specifies the subnets for VPC endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VPCId</i> The VPC Id for the Elasticsearch domain. Exists only if the domain was created with VPCOptions.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> Specifies the role ARN that provides Elasticsearch permissions for accessing Cognito resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IdentityPoolId</i> Specifies the Cognito identity pool ID for Kibana authentication.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolId</i> Specifies the Cognito user pool ID for Kibana authentication.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies the option to enable Cognito for Kibana authentication.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i>  Specifies the KMS Key ID for Encryption At Rest options.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies the option to enable Encryption At Rest.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specify true to enable node-to-node encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedUpdateDate</i> Timestamp, in Epoch time, until which you can manually request a service software update. After this date, we automatically update your service software.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the <code>UpdateStatus</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateStatus</i> The status of your service software update. This field can take the following values: <code>ELIGIBLE</code>, <code>PENDING_UPDATE</code>, <code>IN_PROGRESS</code>, <code>COMPLETED</code>, and <code>NOT_ELIGIBLE</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Cancellable</i> <code>True</code> if you are able to cancel your service software version update. <code>False</code> if you are not able to cancel your service software version. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateAvailable</i> <code>True</code> if you are able to update you service software version. <code>False</code> if you are not able to update your service software version. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NewVersion</i> The new service software version if one is available.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentVersion</i> The current service software version that is present on the domain.<br/>        """
        pass

    def delete_elasticsearch_service_role(self):
        """Deletes the service-linked role that Elasticsearch Service uses to manage and maintain VPC domains. Role deletion will fail if any existing VPC domains use the role. You must delete any such Elasticsearch domains before deleting the role. See <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-vpc.html#es-enabling-slr" target="_blank">Deleting Elasticsearch Service Role</a> in <i>VPC Endpoints for Amazon Elasticsearch Service Domains</i>.

        :param self:
        :param 
        :return:         """
        pass

    def describe_elasticsearch_domain(self, DomainName: str) -> DescribeElasticsearchDomainResponse:
        """Returns domain configuration information about the specified Elasticsearch domain, including the domain ID, domain endpoint, and domain ARN.

        :param self:
        :param DomainName:  The name of the Elasticsearch domain for which you want information.<br/>
        :return: DomainStatus:  The current status of the Elasticsearch domain.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceSoftwareOptions</i> The current status of the Elasticsearch domain's service software.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ServiceSoftwareOptions</b>: The current status of the Elasticsearch domain's service software.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogPublishingOptions</i> Log publishing options for the given domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdvancedOptions</i> Specifies the status of the <code>AdvancedOptions</code><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeToNodeEncryptionOptions</i> Specifies the status of the <code>NodeToNodeEncryptionOptions</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NodeToNodeEncryptionOptions</b>: Specifies the status of the <code>NodeToNodeEncryptionOptions</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionAtRestOptions</i>  Specifies the status of the <code>EncryptionAtRestOptions</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncryptionAtRestOptions</b>:  Specifies the status of the <code>EncryptionAtRestOptions</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CognitoOptions</i> The <code>CognitoOptions</code> for the specified domain. For more information, see <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-cognito-auth.html" target="_blank">Amazon Cognito Authentication for Kibana</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CognitoOptions</b>: The <code>CognitoOptions</code> for the specified domain. For more information, see <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-cognito-auth.html" target="_blank">Amazon Cognito Authentication for Kibana</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VPCOptions</i> The <code>VPCOptions</code> for the specified domain. For more information, see <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-vpc.html" target="_blank">VPC Endpoints for Amazon Elasticsearch Service Domains</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VPCOptions</b>: The <code>VPCOptions</code> for the specified domain. For more information, see <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-vpc.html" target="_blank">VPC Endpoints for Amazon Elasticsearch Service Domains</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotOptions</i> Specifies the status of the <code>SnapshotOptions</code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SnapshotOptions</b>: Specifies the status of the <code>SnapshotOptions</code><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccessPolicies</i>  IAM access policy as a JSON-formatted string.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EBSOptions</i> The <code>EBSOptions</code> for the specified domain. See <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-createupdatedomains.html#es-createdomain-configure-ebs" target="_blank">Configuring EBS-based Storage</a> for more information.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EBSOptions</b>: The <code>EBSOptions</code> for the specified domain. See <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-createupdatedomains.html#es-createdomain-configure-ebs" target="_blank">Configuring EBS-based Storage</a> for more information.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticsearchClusterConfig</i> The type and number of instances in the domain cluster.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ElasticsearchClusterConfig</b>: The type and number of instances in the domain cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticsearchVersion</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpgradeProcessing</i> The status of an Elasticsearch domain version upgrade. <code>True</code> if Amazon Elasticsearch Service is undergoing a version upgrade. <code>False</code> if the configuration is active.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Processing</i> The status of the Elasticsearch domain configuration. <code>True</code> if Amazon Elasticsearch Service is processing configuration changes. <code>False</code> if the configuration is active.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoints</i> Map containing the Elasticsearch domain endpoints used to submit index and search requests. Example <code>key, value</code>: <code>'vpc','vpc-endpoint-h2dsd34efgyghrtguk5gt6j2foh4.us-east-1.es.amazonaws.com'</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> The Elasticsearch domain endpoint that you use to submit index and search requests.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Deleted</i> The domain deletion status. <code>True</code> if a delete request has been received for the domain but resource cleanup is still in progress. <code>False</code> if the domain has not been deleted. Once domain deletion is complete, the status of the domain is no longer returned.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Created</i> The domain creation status. <code>True</code> if the creation of an Elasticsearch domain is complete. <code>False</code> if domain creation is still in progress.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ARN</i> The Amazon resource name (ARN) of an Elasticsearch domain. See <a href="http://docs.aws.amazon.com/IAM/latest/UserGuide/index.html?Using_Identifiers.html" target="_blank">Identifiers for IAM Entities</a> in <i>Using AWS Identity and Access Management</i> for more information.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainName</i> The name of an Elasticsearch domain. Domain names are unique across the domains owned by an account within an AWS region. Domain names start with a letter or number and can contain the following characters: a-z (lowercase), 0-9, and - (hyphen).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainId</i> The unique identifier for the specified Elasticsearch domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ZoneAwarenessEnabled</i> A boolean value to indicate whether zone awareness is enabled. See <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-managedomains.html#es-managedomains-zoneawareness" target="_blank">About Zone Awareness</a> for more information.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DedicatedMasterEnabled</i> A boolean value to indicate whether a dedicated master node is enabled. See <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-managedomains.html#es-managedomains-dedicatedmasternodes" target="_blank">About Dedicated Master Nodes</a> for more information.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCount</i> The number of instances in the specified domain cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i> The instance type for an Elasticsearch cluster.<br/>Specifies the zone awareness configuration for a domain when zone awareness is enabled.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ZoneAwarenessConfig</b>: Specifies the zone awareness configuration for a domain when zone awareness is enabled.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the IOPD for a Provisioned IOPS EBS volume (SSD).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeSize</i>  Integer to specify the size of an EBS volume.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeType</i>  Specifies the volume type for EBS-based storage.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EBSEnabled</i> Specifies whether EBS-based storage is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotStartHour</i> Specifies the time, in UTC format, when the service takes a daily automated snapshot of the specified Elasticsearch domain. Default value is <code>0</code> hours.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> Specifies the security groups for VPC endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> The availability zones for the Elasticsearch domain. Exists only if the domain was created with VPCOptions.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> Specifies the subnets for VPC endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VPCId</i> The VPC Id for the Elasticsearch domain. Exists only if the domain was created with VPCOptions.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> Specifies the role ARN that provides Elasticsearch permissions for accessing Cognito resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IdentityPoolId</i> Specifies the Cognito identity pool ID for Kibana authentication.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolId</i> Specifies the Cognito user pool ID for Kibana authentication.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies the option to enable Cognito for Kibana authentication.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i>  Specifies the KMS Key ID for Encryption At Rest options.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies the option to enable Encryption At Rest.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specify true to enable node-to-node encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedUpdateDate</i> Timestamp, in Epoch time, until which you can manually request a service software update. After this date, we automatically update your service software.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the <code>UpdateStatus</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateStatus</i> The status of your service software update. This field can take the following values: <code>ELIGIBLE</code>, <code>PENDING_UPDATE</code>, <code>IN_PROGRESS</code>, <code>COMPLETED</code>, and <code>NOT_ELIGIBLE</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Cancellable</i> <code>True</code> if you are able to cancel your service software version update. <code>False</code> if you are not able to cancel your service software version. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateAvailable</i> <code>True</code> if you are able to update you service software version. <code>False</code> if you are not able to update your service software version. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NewVersion</i> The new service software version if one is available.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentVersion</i> The current service software version that is present on the domain.<br/>        """
        pass

    def describe_elasticsearch_domain_config(self, DomainName: str) -> DescribeElasticsearchDomainConfigResponse:
        """Provides cluster configuration information about the specified Elasticsearch domain, such as the state, creation date, update version, and update date for cluster options.

        :param self:
        :param DomainName:  The Elasticsearch domain that you want to get information about.<br/>
        :return: DomainConfig:  The configuration information of the domain requested in the <code>DescribeElasticsearchDomainConfig</code> request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogPublishingOptions</i> Log publishing options for the given domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LogPublishingOptions</b>: Log publishing options for the given domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdvancedOptions</i> Specifies the <code>AdvancedOptions</code> for the domain. See <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-createupdatedomains.html#es-createdomain-configure-advanced-options" target="_blank">Configuring Advanced Options</a> for more information.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AdvancedOptions</b>: Specifies the <code>AdvancedOptions</code> for the domain. See <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-createupdatedomains.html#es-createdomain-configure-advanced-options" target="_blank">Configuring Advanced Options</a> for more information.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeToNodeEncryptionOptions</i> Specifies the <code>NodeToNodeEncryptionOptions</code> for the Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NodeToNodeEncryptionOptions</b>: Specifies the <code>NodeToNodeEncryptionOptions</code> for the Elasticsearch domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionAtRestOptions</i> Specifies the <code>EncryptionAtRestOptions</code> for the Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncryptionAtRestOptions</b>: Specifies the <code>EncryptionAtRestOptions</code> for the Elasticsearch domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CognitoOptions</i> The <code>CognitoOptions</code> for the specified domain. For more information, see <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-cognito-auth.html" target="_blank">Amazon Cognito Authentication for Kibana</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CognitoOptions</b>: The <code>CognitoOptions</code> for the specified domain. For more information, see <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-cognito-auth.html" target="_blank">Amazon Cognito Authentication for Kibana</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VPCOptions</i> The <code>VPCOptions</code> for the specified domain. For more information, see <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-vpc.html" target="_blank">VPC Endpoints for Amazon Elasticsearch Service Domains</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VPCOptions</b>: The <code>VPCOptions</code> for the specified domain. For more information, see <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-vpc.html" target="_blank">VPC Endpoints for Amazon Elasticsearch Service Domains</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotOptions</i> Specifies the <code>SnapshotOptions</code> for the Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SnapshotOptions</b>: Specifies the <code>SnapshotOptions</code> for the Elasticsearch domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccessPolicies</i> IAM access policy as a JSON-formatted string.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AccessPolicies</b>: IAM access policy as a JSON-formatted string.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EBSOptions</i> Specifies the <code>EBSOptions</code> for the Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EBSOptions</b>: Specifies the <code>EBSOptions</code> for the Elasticsearch domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticsearchClusterConfig</i> Specifies the <code>ElasticsearchClusterConfig</code> for the Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ElasticsearchClusterConfig</b>: Specifies the <code>ElasticsearchClusterConfig</code> for the Elasticsearch domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticsearchVersion</i> String of format X.Y to specify version for the Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ElasticsearchVersion</b>: String of format X.Y to specify version for the Elasticsearch domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i>  Specifies the Elasticsearch version for the specified Elasticsearch domain.<br/> Specifies the status of the Elasticsearch version options for the specified Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>:  Specifies the status of the Elasticsearch version options for the specified Elasticsearch domain. Specifies the cluster configuration for the specified Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>:  Specifies the cluster configuration for the specified Elasticsearch domain. Specifies the EBS options for the specified Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>:  Specifies the EBS options for the specified Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> The access policy configured for the Elasticsearch domain. Access policies may be resource-based, IP-based, or IAM-based. See <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-createupdatedomains.html#es-createdomain-configure-access-policies" target="_blank"> Configuring Access Policies</a>for more information.<br/>The status of the access policy for the Elasticsearch domain. See <code>OptionStatus</code> for the status information that's included. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>: The status of the access policy for the Elasticsearch domain. See <code>OptionStatus</code> for the status information that's included. Specifies the daily snapshot options specified for the Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>: Specifies the daily snapshot options specified for the Elasticsearch domain. Specifies the VPC options for the specified Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>:  Specifies the VPC options for the specified Elasticsearch domain.Specifies the Cognito options for the specified Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>: Specifies the Cognito options for the specified Elasticsearch domain. Specifies the Encryption At Rest options for the specified Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>:  Specifies the Encryption At Rest options for the specified Elasticsearch domain.Specifies the node-to-node encryption options for the specified Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>: Specifies the node-to-node encryption options for the specified Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i>  Specifies the status of advanced options for the specified Elasticsearch domain.<br/> Specifies the status of <code>OptionStatus</code> for advanced options for the specified Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>:  Specifies the status of <code>OptionStatus</code> for advanced options for the specified Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> The log publishing options configured for the Elasticsearch domain.<br/>The status of the log publishing options for the Elasticsearch domain. See <code>OptionStatus</code> for the status information that's included. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>: The status of the log publishing options for the Elasticsearch domain. See <code>OptionStatus</code> for the status information that's included.         """
        pass

    def describe_elasticsearch_domains(self, DomainNames: List[DomainName]) -> DescribeElasticsearchDomainsResponse:
        """Returns domain configuration information about the specified Elasticsearch domains, including the domain ID, domain endpoint, and domain ARN.

        :param self:
        :param DomainNames:  The Elasticsearch domains for which you want information.<br/>
        :return: DomainStatusList:  The status of the domains requested in the <code>DescribeElasticsearchDomains</code> request.<br/>        """
        pass

    def describe_elasticsearch_instance_type_limits(self, ElasticsearchVersion: str, InstanceType: str, DomainName: str=None) -> DescribeElasticsearchInstanceTypeLimitsResponse:
        """ Describe Elasticsearch Limits for a given InstanceType and ElasticsearchVersion. When modifying existing Domain, specify the <code> <a>DomainName</a> </code> to know what Limits are supported for modifying. 

        :param self:
        :param ElasticsearchVersion:   Version of Elasticsearch for which <code> <a>Limits</a> </code> are needed. <br/>InstanceType:   The instance type for an Elasticsearch cluster for which Elasticsearch <code> <a>Limits</a> </code> are needed. <br/>DomainName:   DomainName represents the name of the Domain that we are trying to modify. This should be present only if we are querying for Elasticsearch <code> <a>Limits</a> </code> for existing domain. <br/>
        :return: LimitsByRole:   Map of Role of the Instance and Limits that are applicable. Role performed by given Instance in Elasticsearch can be one of the following: <ul> <li>Data: If the given InstanceType is used as Data node</li> <li>Master: If the given InstanceType is used as Master node</li> </ul> <br/>        """
        pass

    def describe_reserved_elasticsearch_instance_offerings(self, ReservedElasticsearchInstanceOfferingId: str=None, MaxResults: int=None, NextToken: str=None) -> DescribeReservedElasticsearchInstanceOfferingsResponse:
        """Lists available reserved Elasticsearch instance offerings.

        :param self:
        :param NextToken:  NextToken should be sent in case if earlier API call produced result containing NextToken. It is used for pagination.<br/>MaxResults:  Set this value to limit the number of results returned. If not specified, defaults to 100.<br/>ReservedElasticsearchInstanceOfferingId:  The offering identifier filter value. Use this parameter to show only the available offering that matches the specified reservation identifier.<br/>
        :return: ReservedElasticsearchInstanceOfferings:  List of reserved Elasticsearch instance offerings<br/>NextToken:  Provides an identifier to allow retrieval of paginated results.<br/>        """
        pass

    def describe_reserved_elasticsearch_instances(self, ReservedElasticsearchInstanceId: str=None, MaxResults: int=None, NextToken: str=None) -> DescribeReservedElasticsearchInstancesResponse:
        """Returns information about reserved Elasticsearch instances for this account.

        :param self:
        :param NextToken:  NextToken should be sent in case if earlier API call produced result containing NextToken. It is used for pagination.<br/>MaxResults:  Set this value to limit the number of results returned. If not specified, defaults to 100.<br/>ReservedElasticsearchInstanceId:  The reserved instance identifier filter value. Use this parameter to show only the reservation that matches the specified reserved Elasticsearch instance ID.<br/>
        :return: ReservedElasticsearchInstances:  List of reserved Elasticsearch instances.<br/>NextToken:  Provides an identifier to allow retrieval of paginated results.<br/>        """
        pass

    def get_compatible_elasticsearch_versions(self, DomainName: str=None) -> GetCompatibleElasticsearchVersionsResponse:
        """ Returns a list of upgrade compatible Elastisearch versions. You can optionally pass a <code> <a>DomainName</a> </code> to get all upgrade compatible Elasticsearch versions for that specific domain. 

        :param self:
        :param DomainName:  The name of an Elasticsearch domain. Domain names are unique across the domains owned by an account within an AWS region. Domain names start with a letter or number and can contain the following characters: a-z (lowercase), 0-9, and - (hyphen).<br/>
        :return: CompatibleElasticsearchVersions:   A map of compatible Elasticsearch versions returned as part of the <code> <a>GetCompatibleElasticsearchVersions</a> </code> operation. <br/>        """
        pass

    def get_upgrade_history(self, DomainName: str, MaxResults: int=None, NextToken: str=None) -> GetUpgradeHistoryResponse:
        """Retrieves the complete history of the last 10 upgrades that were performed on the domain.

        :param self:
        :param NextToken:   Paginated APIs accepts NextToken input to returns next page results and provides a NextToken output in the response which can be used by the client to retrieve more results. <br/>MaxResults:   Set this value to limit the number of results returned. <br/>DomainName:  The name of an Elasticsearch domain. Domain names are unique across the domains owned by an account within an AWS region. Domain names start with a letter or number and can contain the following characters: a-z (lowercase), 0-9, and - (hyphen).<br/>
        :return: NextToken:  Pagination token that needs to be supplied to the next call to get the next page of results<br/>UpgradeHistories:   A list of <code> <a>UpgradeHistory</a> </code> objects corresponding to each Upgrade or Upgrade Eligibility Check performed on a domain returned as part of <code> <a>GetUpgradeHistoryResponse</a> </code> object. <br/>        """
        pass

    def get_upgrade_status(self, DomainName: str) -> GetUpgradeStatusResponse:
        """Retrieves the latest status of the last upgrade or upgrade eligibility check that was performed on the domain.

        :param self:
        :param DomainName:  The name of an Elasticsearch domain. Domain names are unique across the domains owned by an account within an AWS region. Domain names start with a letter or number and can contain the following characters: a-z (lowercase), 0-9, and - (hyphen).<br/>
        :return: UpgradeName:  A string that describes the update briefly<br/>StepStatus:   One of 4 statuses that a step can go through returned as part of the <code> <a>GetUpgradeStatusResponse</a> </code> object. The status can take one of the following values: <ul> <li>In Progress</li> <li>Succeeded</li> <li>Succeeded with Issues</li> <li>Failed</li> </ul> <br/>UpgradeStep:   Represents one of 3 steps that an Upgrade or Upgrade Eligibility Check does through: <ul> <li>PreUpgradeCheck</li> <li>Snapshot</li> <li>Upgrade</li> </ul> <br/>        """
        pass

    def list_domain_names(self) -> ListDomainNamesResponse:
        """Returns the name of all Elasticsearch domains owned by the current user's account. 

        :param self:
        :param 
        :return: DomainNames:  List of Elasticsearch domain names.<br/>        """
        pass

    def list_elasticsearch_instance_types(self, ElasticsearchVersion: str, DomainName: str=None, MaxResults: int=None, NextToken: str=None) -> ListElasticsearchInstanceTypesResponse:
        """List all Elasticsearch instance types that are supported for given ElasticsearchVersion

        :param self:
        :param NextToken:  NextToken should be sent in case if earlier API call produced result containing NextToken. It is used for pagination. <br/>MaxResults:   Set this value to limit the number of results returned. Value provided must be greater than 30 else it wont be honored. <br/>DomainName:  DomainName represents the name of the Domain that we are trying to modify. This should be present only if we are querying for list of available Elasticsearch instance types when modifying existing domain. <br/>ElasticsearchVersion:  Version of Elasticsearch for which list of supported elasticsearch instance types are needed. <br/>
        :return: NextToken:  In case if there are more results available NextToken would be present, make further request to the same API with received NextToken to paginate remaining results. <br/>ElasticsearchInstanceTypes:   List of instance types supported by Amazon Elasticsearch service for given <code> <a>ElasticsearchVersion</a> </code> <br/>        """
        pass

    def list_elasticsearch_versions(self, MaxResults: int=None, NextToken: str=None) -> ListElasticsearchVersionsResponse:
        """List all supported Elasticsearch versions

        :param self:
        :param NextToken:   Paginated APIs accepts NextToken input to returns next page results and provides a NextToken output in the response which can be used by the client to retrieve more results. <br/>MaxResults:   Set this value to limit the number of results returned. Value provided must be greater than 10 else it wont be honored. <br/>
        :return: NextToken:   Paginated APIs accepts NextToken input to returns next page results and provides a NextToken output in the response which can be used by the client to retrieve more results. <br/>ElasticsearchVersions:  List of supported elastic search versions. <br/>        """
        pass

    def list_tags(self, ARN: str) -> ListTagsResponse:
        """Returns all tags for the given Elasticsearch domain.

        :param self:
        :param ARN:   Specify the <code>ARN</code> for the Elasticsearch domain to which the tags are attached that you want to view.<br/>
        :return: TagList:   List of <code>Tag</code> for the requested Elasticsearch domain.<br/>        """
        pass

    def purchase_reserved_elasticsearch_instance_offering(self, ReservationName: str, ReservedElasticsearchInstanceOfferingId: str, InstanceCount: int=None) -> PurchaseReservedElasticsearchInstanceOfferingResponse:
        """Allows you to purchase reserved Elasticsearch instances.

        :param self:
        :param InstanceCount:  The number of Elasticsearch instances to reserve.<br/>ReservationName:  A customer-specified identifier to track this reservation.<br/>ReservedElasticsearchInstanceOfferingId:  The ID of the reserved Elasticsearch instance offering to purchase.<br/>
        :return: ReservationName:  The customer-specified identifier used to track this reservation.<br/>ReservedElasticsearchInstanceId:  Details of the reserved Elasticsearch instance which was purchased.<br/>        """
        pass

    def remove_tags(self, TagKeys: List[String], ARN: str):
        """Removes the specified set of tags from the specified Elasticsearch domain.

        :param self:
        :param TagKeys:  Specifies the <code>TagKey</code> list which you want to remove from the Elasticsearch domain.<br/>ARN:  Specifies the <code>ARN</code> for the Elasticsearch domain from which you want to delete the specified tags.<br/>
        :return:         """
        pass

    def start_elasticsearch_service_software_update(self, DomainName: str) -> StartElasticsearchServiceSoftwareUpdateResponse:
        """Schedules a service software update for an Amazon ES domain.

        :param self:
        :param DomainName:  The name of the domain that you want to update to the latest service software.<br/>
        :return: ServiceSoftwareOptions:  The current status of the Elasticsearch service software update.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedUpdateDate</i> Timestamp, in Epoch time, until which you can manually request a service software update. After this date, we automatically update your service software.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the <code>UpdateStatus</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateStatus</i> The status of your service software update. This field can take the following values: <code>ELIGIBLE</code>, <code>PENDING_UPDATE</code>, <code>IN_PROGRESS</code>, <code>COMPLETED</code>, and <code>NOT_ELIGIBLE</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Cancellable</i> <code>True</code> if you are able to cancel your service software version update. <code>False</code> if you are not able to cancel your service software version. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateAvailable</i> <code>True</code> if you are able to update you service software version. <code>False</code> if you are not able to update your service software version. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NewVersion</i> The new service software version if one is available.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentVersion</i> The current service software version that is present on the domain.<br/>        """
        pass

    def update_elasticsearch_domain_config(self, DomainName: str, ElasticsearchClusterConfig: ElasticsearchClusterConfig=None, EBSOptions: EBSOptions=None, SnapshotOptions: SnapshotOptions=None, VPCOptions: VPCOptions=None, CognitoOptions: CognitoOptions=None, AdvancedOptions: AdvancedOptions=None, AccessPolicies: str=None, LogPublishingOptions: LogPublishingOptions=None) -> UpdateElasticsearchDomainConfigResponse:
        """Modifies the cluster configuration of the specified Elasticsearch domain, setting as setting the instance type and the number of instances. 

        :param self:
        :param LogPublishingOptions:  Map of <code>LogType</code> and <code>LogPublishingOption</code>, each containing options to publish a given type of Elasticsearch log.<br/>AccessPolicies:  IAM access policy as a JSON-formatted string.<br/>AdvancedOptions:  Modifies the advanced option to allow references to indices in an HTTP request body. Must be <code>false</code> when configuring access to individual sub-resources. By default, the value is <code>true</code>. See <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-createupdatedomains.html#es-createdomain-configure-advanced-options" target="_blank">Configuration Advanced Options</a> for more information.<br/>CognitoOptions:  Options to specify the Cognito user and identity pools for Kibana authentication. For more information, see <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-cognito-auth.html" target="_blank">Amazon Cognito Authentication for Kibana</a>.:<br/>VPCOptions:  Options to specify the subnets and security groups for VPC endpoint. For more information, see <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-vpc.html#es-creating-vpc" target="_blank">Creating a VPC</a> in <i>VPC Endpoints for Amazon Elasticsearch Service Domains</i>:<br/>SnapshotOptions:  Option to set the time, in UTC format, for the daily automated snapshot. Default value is <code>0</code> hours. :<br/>EBSOptions:  Specify the type and size of the EBS volume that you want to use. :<br/>ElasticsearchClusterConfig:  The type and number of instances to instantiate for the domain cluster.:<br/>DomainName:  The name of the Elasticsearch domain that you are updating. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DedicatedMasterCount</i> Total number of dedicated master nodes, active and on standby, for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DedicatedMasterType</i> The instance type for a dedicated master node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ZoneAwarenessConfig</i> Specifies the zone awareness configuration for a domain when zone awareness is enabled.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ZoneAwarenessConfig</b>: Specifies the zone awareness configuration for a domain when zone awareness is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ZoneAwarenessEnabled</i> A boolean value to indicate whether zone awareness is enabled. See <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-managedomains.html#es-managedomains-zoneawareness" target="_blank">About Zone Awareness</a> for more information.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DedicatedMasterEnabled</i> A boolean value to indicate whether a dedicated master node is enabled. See <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-managedomains.html#es-managedomains-dedicatedmasternodes" target="_blank">About Dedicated Master Nodes</a> for more information.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCount</i> The number of instances in the specified domain cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i> The instance type for an Elasticsearch cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZoneCount</i> An integer value to indicate the number of availability zones for a domain when zone awareness is enabled. This should be equal to number of subnets if VPC endpoints is enabled<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the IOPD for a Provisioned IOPS EBS volume (SSD).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeSize</i>  Integer to specify the size of an EBS volume.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeType</i>  Specifies the volume type for EBS-based storage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EBSEnabled</i> Specifies whether EBS-based storage is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotStartHour</i> Specifies the time, in UTC format, when the service takes a daily automated snapshot of the specified Elasticsearch domain. Default value is <code>0</code> hours.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> Specifies the security groups for VPC endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> Specifies the subnets for VPC endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> Specifies the role ARN that provides Elasticsearch permissions for accessing Cognito resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IdentityPoolId</i> Specifies the Cognito identity pool ID for Kibana authentication.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserPoolId</i> Specifies the Cognito user pool ID for Kibana authentication.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies the option to enable Cognito for Kibana authentication.<br/>
        :return: DomainConfig:  The status of the updated Elasticsearch domain. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogPublishingOptions</i> Log publishing options for the given domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LogPublishingOptions</b>: Log publishing options for the given domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdvancedOptions</i> Specifies the <code>AdvancedOptions</code> for the domain. See <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-createupdatedomains.html#es-createdomain-configure-advanced-options" target="_blank">Configuring Advanced Options</a> for more information.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AdvancedOptions</b>: Specifies the <code>AdvancedOptions</code> for the domain. See <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-createupdatedomains.html#es-createdomain-configure-advanced-options" target="_blank">Configuring Advanced Options</a> for more information.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeToNodeEncryptionOptions</i> Specifies the <code>NodeToNodeEncryptionOptions</code> for the Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NodeToNodeEncryptionOptions</b>: Specifies the <code>NodeToNodeEncryptionOptions</code> for the Elasticsearch domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionAtRestOptions</i> Specifies the <code>EncryptionAtRestOptions</code> for the Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncryptionAtRestOptions</b>: Specifies the <code>EncryptionAtRestOptions</code> for the Elasticsearch domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CognitoOptions</i> The <code>CognitoOptions</code> for the specified domain. For more information, see <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-cognito-auth.html" target="_blank">Amazon Cognito Authentication for Kibana</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CognitoOptions</b>: The <code>CognitoOptions</code> for the specified domain. For more information, see <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-cognito-auth.html" target="_blank">Amazon Cognito Authentication for Kibana</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VPCOptions</i> The <code>VPCOptions</code> for the specified domain. For more information, see <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-vpc.html" target="_blank">VPC Endpoints for Amazon Elasticsearch Service Domains</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VPCOptions</b>: The <code>VPCOptions</code> for the specified domain. For more information, see <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-vpc.html" target="_blank">VPC Endpoints for Amazon Elasticsearch Service Domains</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotOptions</i> Specifies the <code>SnapshotOptions</code> for the Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SnapshotOptions</b>: Specifies the <code>SnapshotOptions</code> for the Elasticsearch domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccessPolicies</i> IAM access policy as a JSON-formatted string.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AccessPolicies</b>: IAM access policy as a JSON-formatted string.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EBSOptions</i> Specifies the <code>EBSOptions</code> for the Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EBSOptions</b>: Specifies the <code>EBSOptions</code> for the Elasticsearch domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticsearchClusterConfig</i> Specifies the <code>ElasticsearchClusterConfig</code> for the Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ElasticsearchClusterConfig</b>: Specifies the <code>ElasticsearchClusterConfig</code> for the Elasticsearch domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticsearchVersion</i> String of format X.Y to specify version for the Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ElasticsearchVersion</b>: String of format X.Y to specify version for the Elasticsearch domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i>  Specifies the Elasticsearch version for the specified Elasticsearch domain.<br/> Specifies the status of the Elasticsearch version options for the specified Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>:  Specifies the status of the Elasticsearch version options for the specified Elasticsearch domain. Specifies the cluster configuration for the specified Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>:  Specifies the cluster configuration for the specified Elasticsearch domain. Specifies the EBS options for the specified Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>:  Specifies the EBS options for the specified Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> The access policy configured for the Elasticsearch domain. Access policies may be resource-based, IP-based, or IAM-based. See <a href="http://docs.aws.amazon.com/elasticsearch-service/latest/developerguide/es-createupdatedomains.html#es-createdomain-configure-access-policies" target="_blank"> Configuring Access Policies</a>for more information.<br/>The status of the access policy for the Elasticsearch domain. See <code>OptionStatus</code> for the status information that's included. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>: The status of the access policy for the Elasticsearch domain. See <code>OptionStatus</code> for the status information that's included. Specifies the daily snapshot options specified for the Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>: Specifies the daily snapshot options specified for the Elasticsearch domain. Specifies the VPC options for the specified Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>:  Specifies the VPC options for the specified Elasticsearch domain.Specifies the Cognito options for the specified Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>: Specifies the Cognito options for the specified Elasticsearch domain. Specifies the Encryption At Rest options for the specified Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>:  Specifies the Encryption At Rest options for the specified Elasticsearch domain.Specifies the node-to-node encryption options for the specified Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Options</b>: Specifies the node-to-node encryption options for the specified Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i>  Specifies the status of advanced options for the specified Elasticsearch domain.<br/> Specifies the status of <code>OptionStatus</code> for advanced options for the specified Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>:  Specifies the status of <code>OptionStatus</code> for advanced options for the specified Elasticsearch domain.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> The log publishing options configured for the Elasticsearch domain.<br/>The status of the log publishing options for the Elasticsearch domain. See <code>OptionStatus</code> for the status information that's included. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>: The status of the log publishing options for the Elasticsearch domain. See <code>OptionStatus</code> for the status information that's included.         """
        pass

    def upgrade_elasticsearch_domain(self, TargetVersion: str, DomainName: str, PerformCheckOnly: Boolean=None) -> UpgradeElasticsearchDomainResponse:
        """Allows you to either upgrade your domain or perform an Upgrade eligibility check to a compatible Elasticsearch version.

        :param self:
        :param PerformCheckOnly:   This flag, when set to True, indicates that an Upgrade Eligibility Check needs to be performed. This will not actually perform the Upgrade. <br/>TargetVersion:  The version of Elasticsearch that you intend to upgrade the domain to.<br/>DomainName:  The name of an Elasticsearch domain. Domain names are unique across the domains owned by an account within an AWS region. Domain names start with a letter or number and can contain the following characters: a-z (lowercase), 0-9, and - (hyphen).<br/>
        :return: PerformCheckOnly:   This flag, when set to True, indicates that an Upgrade Eligibility Check needs to be performed. This will not actually perform the Upgrade. <br/>TargetVersion:  The version of Elasticsearch that you intend to upgrade the domain to.<br/>DomainName:  The name of an Elasticsearch domain. Domain names are unique across the domains owned by an account within an AWS region. Domain names start with a letter or number and can contain the following characters: a-z (lowercase), 0-9, and - (hyphen).<br/>        """
        pass



class EventBridge(BaseClient):
    """Amazon EventBridge helps you to respond to state changes in your AWS resources. When your resources change state, they automatically send events into an event stream. You can create rules that match selected events in the stream and route them to targets to take action. You can also use rules to take action on a predetermined schedule. For example, you can configure rules to: <ul> <li> Automatically invoke an AWS Lambda function to update DNS entries when an event notifies you that Amazon EC2 instance enters the running state </li> <li> Direct specific API records from AWS CloudTrail to an Amazon Kinesis data stream for detailed analysis of potential security or availability risks </li> <li> Periodically invoke a built-in target to create a snapshot of an Amazon EBS volume </li> </ul> For more information about the features of Amazon EventBridge, see the <a href="https://docs.aws.amazon.com/eventbridge/latest/userguide/">Amazon EventBridge User Guide</a>."""


    class ListRuleNamesByTargetPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_rule_names_by_target` for valid parameters"""
            pass
    class ListRulesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_rules` for valid parameters"""
            pass
    class ListTargetsByRulePaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_targets_by_rule` for valid parameters"""
            pass


    class ActivateEventSourceRequest(Mapping):
        pass
    
    class CreateEventBusRequest(Mapping):
        pass
    
    class CreateEventBusResponse(Mapping):
        pass
    
    class CreatePartnerEventSourceRequest(Mapping):
        pass
    
    class CreatePartnerEventSourceResponse(Mapping):
        pass
    
    class DeactivateEventSourceRequest(Mapping):
        pass
    
    class DeleteEventBusRequest(Mapping):
        pass
    
    class DeletePartnerEventSourceRequest(Mapping):
        pass
    
    class DeleteRuleRequest(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class DescribeEventBusRequest(Mapping):
        pass
    
    class DescribeEventBusResponse(Mapping):
        pass
    
    class DescribeEventSourceRequest(Mapping):
        pass
    
    class DescribeEventSourceResponse(Mapping):
        pass
    
    class DescribePartnerEventSourceRequest(Mapping):
        pass
    
    class DescribePartnerEventSourceResponse(Mapping):
        pass
    
    class DescribeRuleRequest(Mapping):
        pass
    
    class DescribeRuleResponse(Mapping):
        pass
    
    class DisableRuleRequest(Mapping):
        pass
    
    class EnableRuleRequest(Mapping):
        pass
    
    class ListEventBusesRequest(Mapping):
        pass
    
    class ListEventBusesResponse(Mapping):
        pass
    
    class ListEventSourcesRequest(Mapping):
        pass
    
    class ListEventSourcesResponse(Mapping):
        pass
    
    class ListPartnerEventSourceAccountsRequest(Mapping):
        pass
    
    class ListPartnerEventSourceAccountsResponse(Mapping):
        pass
    
    class ListPartnerEventSourcesRequest(Mapping):
        pass
    
    class ListPartnerEventSourcesResponse(Mapping):
        pass
    
    class ListRuleNamesByTargetRequest(Mapping):
        pass
    
    class ListRuleNamesByTargetResponse(Mapping):
        pass
    
    class ListRulesRequest(Mapping):
        pass
    
    class ListRulesResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class ListTargetsByRuleRequest(Mapping):
        pass
    
    class ListTargetsByRuleResponse(Mapping):
        pass
    
    class PutEventsRequest(Mapping):
        pass
    
    class PutEventsResponse(Mapping):
        pass
    
    class PutEventsRequestEntry(Mapping):
        pass
    
    class PutPartnerEventsRequest(Mapping):
        pass
    
    class PutPartnerEventsResponse(Mapping):
        pass
    
    class PutPartnerEventsRequestEntry(Mapping):
        pass
    
    class PutPermissionRequest(Mapping):
        pass
    
    class Condition(Mapping):
        pass
    
    class PutRuleRequest(Mapping):
        pass
    
    class PutRuleResponse(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class PutTargetsRequest(Mapping):
        pass
    
    class PutTargetsResponse(Mapping):
        pass
    
    class Target(Mapping):
        pass
    
    class RemovePermissionRequest(Mapping):
        pass
    
    class RemoveTargetsRequest(Mapping):
        pass
    
    class RemoveTargetsResponse(Mapping):
        pass
    
    class TargetId(object):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class TestEventPatternRequest(Mapping):
        pass
    
    class TestEventPatternResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    def activate_event_source(self, Name: str):
        """Activates a partner event source that has been deactivated. Once activated, your matching event bus will start receiving events from the event source. <note> This operation is performed by AWS customers, not by SaaS partners. </note>

        :param self:
        :param Name:  The name of the partner event source to activate.<br/>
        :return:         """
        pass

    def create_event_bus(self, Name: str, EventSourceName: str=None) -> CreateEventBusResponse:
        """Creates a new event bus within your account. This can be a custom event bus which you can use to receive events from your own custom applications and services, or it can be a partner event bus which can be matched to a partner event source. <note> This operation is used by AWS customers, not by SaaS partners. </note>

        :param self:
        :param EventSourceName:  If you're creating a partner event bus, this specifies the partner event source that the new event bus will be matched with.<br/>Name:  The name of the new event bus.  The names of custom event buses can't contain the <code>/</code> character. You can't use the name <code>default</code> for a custom event bus because this name is already used for your account's default event bus. If this is a partner event bus, the name must exactly match the name of the partner event source that this event bus is matched to. This name will include the <code>/</code> character.<br/>
        :return: EventBusArn:  The ARN of the new event bus.<br/>        """
        pass

    def create_partner_event_source(self, Account: str, Name: str) -> CreatePartnerEventSourceResponse:
        """Called by an SaaS partner to create a partner event source. <note> This operation is not used by AWS customers. </note> Each partner event source can be used by one AWS account to create a matching partner event bus in that AWS account. A SaaS partner must create one partner event source for each AWS account that wants to receive those event types.  A partner event source creates events based on resources in the SaaS partner's service or application. An AWS account that creates a partner event bus that matches the partner event source can use that event bus to receive events from the partner, and then process them using AWS Events rules and targets. Partner event source names follow this format:  <code>aws.partner/<i>partner_name</i>/<i>event_namespace</i>/<i>event_name</i> </code>  <ul> <li>  <i>partner_name</i> is determined during partner registration and identifies the partner to AWS customers. </li> <li> For <i>event_namespace</i>, we recommend that partners use a string that identifies the AWS customer within the partner's system. This should not be the customer's AWS account ID. </li> <li>  <i>event_name</i> is determined by the partner, and should uniquely identify an event-generating resource within the partner system. This should help AWS customers decide whether to create an event bus to receive these events. </li> </ul>

        :param self:
        :param Account:  The AWS account ID of the customer who is permitted to create a matching partner event bus for this partner event source.<br/>Name:  The name of the partner event source. This name must be unique and must be in the format <code> <i>partner_name</i>/<i>event_namespace</i>/<i>event_name</i> </code>. The AWS account that wants to use this partner event source must create a partner event bus with a name that matches the name of the partner event source.<br/>
        :return: EventSourceArn:  The ARN of the partner event source.<br/>        """
        pass

    def deactivate_event_source(self, Name: str):
        """An AWS customer uses this operation to temporarily stop receiving events from the specified partner event source. The matching event bus isn't deleted.  When you deactivate a partner event source, the source goes into <code>PENDING</code> state. If it remains in <code>PENDING</code> state for more than two weeks, it's deleted. To activate a deactivated partner event source, use <a>ActivateEventSource</a>.

        :param self:
        :param Name:  The name of the partner event source to deactivate.<br/>
        :return:         """
        pass

    def delete_event_bus(self, Name: str):
        """Deletes the specified custom event bus or partner event bus. All rules associated with this event bus are also deleted. You can't delete your account's default event bus. <note> This operation is performed by AWS customers, not by SaaS partners. </note>

        :param self:
        :param Name:  The name of the event bus to delete.<br/>
        :return:         """
        pass

    def delete_partner_event_source(self, Account: str, Name: str):
        """This operation is used by SaaS partners to delete a partner event source. AWS customers don't use this operation. When you delete an event source, the status of the corresponding partner event bus in the AWS customer account becomes <code>DELETED</code>.

        :param self:
        :param Account:  The AWS account ID of the AWS customer that the event source was created for.<br/>Name:  The name of the event source to delete.<br/>
        :return:         """
        pass

    def delete_rule(self, Name: str, EventBusName: str=None, Force: Boolean=None):
        """Deletes the specified rule. Before you can delete the rule, you must remove all targets, using <a>RemoveTargets</a>. When you delete a rule, incoming events might continue to match to the deleted rule. Allow a short period of time for changes to take effect. Managed rules are rules created and managed by another AWS service on your behalf. These rules are created by those other AWS services to support functionality in those services. You can delete these rules using the <code>Force</code> option, but you should do so only if you're sure that the other service isn't still using that rule.

        :param self:
        :param Force:  If this is a managed rule, created by an AWS service on your behalf, you must specify <code>Force</code> as <code>True</code> to delete the rule. This parameter is ignored for rules that are not managed rules. You can check whether a rule is a managed rule by using <code>DescribeRule</code> or <code>ListRules</code> and checking the <code>ManagedBy</code> field of the response.<br/>EventBusName:  The event bus associated with the rule. If you omit this, the default event bus is used.<br/>Name:  The name of the rule.<br/>
        :return:         """
        pass

    def describe_event_bus(self, Name: str=None) -> DescribeEventBusResponse:
        """Displays details about an event bus in your account. This can include the external AWS accounts that are permitted to write events to your default event bus, and the associated policy. For custom event buses and partner event buses, it displays the name, ARN, policy, state, and creation time.  To enable your account to receive events from other accounts on its default event bus, use <a>PutPermission</a>. For more information about partner event buses, see <a>CreateEventBus</a>.

        :param self:
        :param Name:  The name of the event bus to show details for. If you omit this, the default event bus is displayed.<br/>
        :return: Policy:  The policy that enables the external account to send events to your account.<br/>Arn:  The Amazon Resource Name (ARN) of the account permitted to write events to the current account.<br/>Name:  The name of the event bus. Currently, this is always <code>default</code>.<br/>        """
        pass

    def describe_event_source(self, Name: str) -> DescribeEventSourceResponse:
        """This operation lists details about a partner event source that is shared with your account. <note> This operation is run by AWS customers, not by SaaS partners. </note>

        :param self:
        :param Name:  The name of the partner event source to display the details of.<br/>
        :return: State:  The state of the event source. If it's <code>ACTIVE</code>, you have already created a matching event bus for this event source, and that event bus is active. If it's <code>PENDING</code>, either you haven't yet created a matching event bus, or that event bus is deactivated. If it's <code>DELETED</code>, you have created a matching event bus, but the event source has since been deleted.<br/>Name:  The name of the partner event source.<br/>ExpirationTime:  The date and time that the event source will expire if you don't create a matching event bus.<br/>CreationTime:  The date and time that the event source was created.<br/>CreatedBy:  The name of the SaaS partner that created the event source.<br/>Arn:  The ARN of the partner event source.<br/>        """
        pass

    def describe_partner_event_source(self, Name: str) -> DescribePartnerEventSourceResponse:
        """An SaaS partner can use this operation to list details about a partner event source that they have created. <note> AWS customers do not use this operation. Instead, AWS customers can use <a>DescribeEventSource</a> to see details about a partner event source that is shared with them. </note>

        :param self:
        :param Name:  The name of the event source to display.<br/>
        :return: Name:  The name of the event source.<br/>Arn:  The ARN of the event source.<br/>        """
        pass

    def describe_rule(self, Name: str, EventBusName: str=None) -> DescribeRuleResponse:
        """Describes the specified rule.  <code>DescribeRule</code> doesn't list the targets of a rule. To see the targets associated with a rule, use <a>ListTargetsByRule</a>.

        :param self:
        :param EventBusName:  The event bus associated with the rule. If you omit this, the default event bus is used.<br/>Name:  The name of the rule.<br/>
        :return: EventBusName:  The event bus associated with the rule.<br/>ManagedBy:  If this is a managed rule, created by an AWS service on your behalf, this field displays the principal name of the AWS service that created the rule.<br/>RoleArn:  The Amazon Resource Name (ARN) of the IAM role associated with the rule.<br/>Description:  The description of the rule.<br/>State:  Specifies whether the rule is enabled or disabled.<br/>ScheduleExpression:  The scheduling expression: for example, <code>"cron(0 20 * * ? *)"</code> or <code>"rate(5 minutes)"</code>.<br/>EventPattern:  The event pattern. For more information, see <a href="https://docs.aws.amazon.com/eventbridge/latest/userguide/eventbridge-and-event-patterns.html">Event Patterns</a> in the <i>Amazon EventBridge User Guide</i>.<br/>Arn:  The Amazon Resource Name (ARN) of the rule.<br/>Name:  The name of the rule.<br/>        """
        pass

    def disable_rule(self, Name: str, EventBusName: str=None):
        """Disables the specified rule. A disabled rule won't match any events and won't self-trigger if it has a schedule expression. When you disable a rule, incoming events might continue to match to the disabled rule. Allow a short period of time for changes to take effect.

        :param self:
        :param EventBusName:  The event bus associated with the rule. If you omit this, the default event bus is used.<br/>Name:  The name of the rule.<br/>
        :return:         """
        pass

    def enable_rule(self, Name: str, EventBusName: str=None):
        """Enables the specified rule. If the rule doesn't exist, the operation fails. When you enable a rule, incoming events might not immediately start matching to a newly enabled rule. Allow a short period of time for changes to take effect.

        :param self:
        :param EventBusName:  The event bus associated with the rule. If you omit this, the default event bus is used.<br/>Name:  The name of the rule.<br/>
        :return:         """
        pass

    def list_event_buses(self, NamePrefix: str=None, NextToken: str=None, Limit: int=None) -> ListEventBusesResponse:
        """Lists all the event buses in your account, including the default event bus, custom event buses, and partner event buses. <note> This operation is run by AWS customers, not by SaaS partners. </note>

        :param self:
        :param Limit:  Specifying this limits the number of results returned by this operation. The operation also returns a <code>NextToken</code> that you can use in a subsequent operation to retrieve the next set of results.<br/>NextToken:  The token returned by a previous call to retrieve the next set of results.<br/>NamePrefix:  Specifying this limits the results to only those event buses with names that start with the specified prefix.<br/>
        :return: NextToken:  A token you can use in a subsequent operation to retrieve the next set of results.<br/>EventBuses:  This list of event buses.<br/>        """
        pass

    def list_event_sources(self, NamePrefix: str=None, NextToken: str=None, Limit: int=None) -> ListEventSourcesResponse:
        """You can use this to see all the partner event sources that have been shared with your AWS account. For more information about partner event sources, see <a>CreateEventBus</a>. <note> This operation is run by AWS customers, not by SaaS partners. </note>

        :param self:
        :param Limit:  Specifying this limits the number of results returned by this operation. The operation also returns a <code>NextToken</code> that you can use in a subsequent operation to retrieve the next set of results.<br/>NextToken:  The token returned by a previous call to retrieve the next set of results.<br/>NamePrefix:  Specifying this limits the results to only those partner event sources with names that start with the specified prefix.<br/>
        :return: NextToken:  A token you can use in a subsequent operation to retrieve the next set of results.<br/>EventSources:  The list of event sources.<br/>        """
        pass

    def list_partner_event_source_accounts(self, EventSourceName: str, NextToken: str=None, Limit: int=None) -> ListPartnerEventSourceAccountsResponse:
        """An SaaS partner can use this operation to display the AWS account ID that a particular partner event source name is associated with. <note> This operation is used by SaaS partners, not by AWS customers. </note>

        :param self:
        :param Limit:  Specifying this limits the number of results returned by this operation. The operation also returns a <code>NextToken</code> that you can use in a subsequent operation to retrieve the next set of results.<br/>NextToken:  The token returned by a previous call to this operation. Specifying this retrieves the next set of results.<br/>EventSourceName:  The name of the partner event source to display account information about.<br/>
        :return: NextToken:  A token you can use in a subsequent operation to retrieve the next set of results.<br/>PartnerEventSourceAccounts:  The list of partner event sources returned by the operation.<br/>        """
        pass

    def list_partner_event_sources(self, NamePrefix: str, NextToken: str=None, Limit: int=None) -> ListPartnerEventSourcesResponse:
        """An SaaS partner can use this operation to list all the partner event source names that they have created. <note> This operation is not used by AWS customers. </note>

        :param self:
        :param Limit:  pecifying this limits the number of results returned by this operation. The operation also returns a <code>NextToken</code> that you can use in a subsequent operation to retrieve the next set of results.<br/>NextToken:  The token returned by a previous call to this operation. Specifying this retrieves the next set of results.<br/>NamePrefix:  If you specify this, the results are limited to only those partner event sources that start with the string you specify.<br/>
        :return: NextToken:  A token you can use in a subsequent operation to retrieve the next set of results.<br/>PartnerEventSources:  The list of partner event sources returned by the operation.<br/>        """
        pass

    def list_rule_names_by_target(self, TargetArn: str, EventBusName: str=None, NextToken: str=None, Limit: int=None) -> ListRuleNamesByTargetResponse:
        """Lists the rules for the specified target. You can see which rules can invoke a specific target in your account.

        :param self:
        :param Limit:  The maximum number of results to return.<br/>NextToken:  The token returned by a previous call to retrieve the next set of results.<br/>EventBusName:  Limits the results to show only the rules associated with the specified event bus.<br/>TargetArn:  The Amazon Resource Name (ARN) of the target resource.<br/>
        :return: NextToken:  Indicates whether there are additional results to retrieve. If there are no more results, the value is null.<br/>RuleNames:  The names of the rules that can invoke the given target.<br/>        """
        pass

    def list_rules(self, NamePrefix: str=None, EventBusName: str=None, NextToken: str=None, Limit: int=None) -> ListRulesResponse:
        """Lists your EventBridge rules. You can either list all the rules or provide a prefix to match to the rule names.  <code>ListRules</code> doesn't list the targets of a rule. To see the targets associated with a rule, use <a>ListTargetsByRule</a>.

        :param self:
        :param Limit:  The maximum number of results to return.<br/>NextToken:  The token returned by a previous call to retrieve the next set of results.<br/>EventBusName:  Limits the results to show only the rules associated with the specified event bus.<br/>NamePrefix:  The prefix matching the rule name.<br/>
        :return: NextToken:  Indicates whether there are additional results to retrieve. If there are no more results, the value is null.<br/>Rules:  The rules that match the specified criteria.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceARN: str) -> ListTagsForResourceResponse:
        """Displays the tags associated with an EventBridge resource. In EventBridge, rules can be tagged.

        :param self:
        :param ResourceARN:  The ARN of the rule for which you want to view tags.<br/>
        :return: Tags:  The list of tag keys and values associated with the rule that you specified.<br/>        """
        pass

    def list_targets_by_rule(self, Rule: str, EventBusName: str=None, NextToken: str=None, Limit: int=None) -> ListTargetsByRuleResponse:
        """Lists the targets assigned to the specified rule.

        :param self:
        :param Limit:  The maximum number of results to return.<br/>NextToken:  The token returned by a previous call to retrieve the next set of results.<br/>EventBusName:  The event bus associated with the rule. If you omit this, the default event bus is used.<br/>Rule:  The name of the rule.<br/>
        :return: NextToken:  Indicates whether there are additional results to retrieve. If there are no more results, the value is null.<br/>Targets:  The targets assigned to the rule.<br/>        """
        pass

    def put_events(self, Entries: List[PutEventsRequestEntry]) -> PutEventsResponse:
        """Sends custom events to EventBridge so that they can be matched to rules. These events can be from your custom applications and services.

        :param self:
        :param Entries:  The entry that defines an event in your system. You can specify several parameters for the entry such as the source and type of the event, resources associated with the event, and so on.<br/>
        :return: Entries:  The successfully and unsuccessfully ingested events results. If the ingestion was successful, the entry has the event ID in it. Otherwise, you can use the error code and error message to identify the problem with the entry.<br/>FailedEntryCount:  The number of failed entries.<br/>        """
        pass

    def put_partner_events(self, Entries: List[PutPartnerEventsRequestEntry]) -> PutPartnerEventsResponse:
        """This is used by SaaS partners to write events to a customer's partner event bus. <note> AWS customers do not use this operation. Instead, AWS customers can use <a>PutEvents</a> to write custom events from their own applications to an event bus. </note>

        :param self:
        :param Entries:  The list of events to write to the event bus.<br/>
        :return: Entries:  The list of events from this operation that were successfully written to the partner event bus.<br/>FailedEntryCount:  The number of events from this operation that couldn't be written to the partner event bus.<br/>        """
        pass

    def put_permission(self, StatementId: str, Principal: str, Action: str, EventBusName: str=None, Condition: Condition=None):
        """Running <code>PutPermission</code> permits the specified AWS account or AWS organization to put events to the specified <i>event bus</i>. Rules in your account are triggered by these events arriving to an event bus in your account.  For another account to send events to your account, that external account must have a rule with your account's event bus as a target. To enable multiple AWS accounts to put events to an event bus, run <code>PutPermission</code> once for each of these accounts. Or, if all the accounts are members of the same AWS organization, you can run <code>PutPermission</code> once specifying <code>Principal</code> as "*" and specifying the AWS organization ID in <code>Condition</code>, to grant permissions to all accounts in that organization. If you grant permissions using an organization, then accounts in that organization must specify a <code>RoleArn</code> with proper permissions when they use <code>PutTarget</code> to add your account's event bus as a target. For more information, see <a href="https://docs.aws.amazon.com/eventbridge/latest/userguide/eventbridge-cross-account-event-delivery.html">Sending and Receiving Events Between AWS Accounts</a> in the <i>Amazon EventBridge User Guide</i>. The permission policy on an event bus can't exceed 10 KB in size.

        :param self:
        :param Condition:  This parameter enables you to limit the permission to accounts that fulfill a certain condition, such as being a member of a certain AWS organization. For more information about AWS Organizations, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_introduction.html">What Is AWS Organizations?</a> in the <i>AWS Organizations User Guide</i>. If you specify <code>Condition</code> with an AWS organization ID and specify "*" as the value for <code>Principal</code>, you grant permission to all the accounts in the named organization. The <code>Condition</code> is a JSON string that must contain <code>Type</code>, <code>Key</code>, and <code>Value</code> fields.:<br/>StatementId:  An identifier string for the external account that you're granting permissions to. If you later want to revoke the permission for this external account, specify this <code>StatementId</code> when you run <a>RemovePermission</a>.<br/>Principal:  The 12-digit AWS account ID that you are permitting to put events to your default event bus. Specify "*" to permit any account to put events to your default event bus. If you specify "*" without specifying <code>Condition</code>, avoid creating rules that might match undesirable events. To create more secure rules, make sure that the event pattern for each rule contains an <code>account</code> field with a specific account ID to receive events from. Rules with an account field don't match any events sent from other accounts.<br/>Action:  The action that you're enabling the other account to perform. Currently, this must be <code>events:PutEvents</code>.<br/>EventBusName:  The event bus associated with the rule. If you omit this, the default event bus is used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The value for the key. Currently, this must be the ID of the organization.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> The key for the condition. Currently, the only supported key is <code>aws:PrincipalOrgID</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of condition. Currently, the only supported value is <code>StringEquals</code>.<br/>
        :return:         """
        pass

    def put_rule(self, Name: str, ScheduleExpression: str=None, EventPattern: str=None, State: str=None, Description: str=None, RoleArn: str=None, Tags: List[Tag]=None, EventBusName: str=None) -> PutRuleResponse:
        """Creates or updates the specified rule. Rules are enabled by default or based on value of the state. You can disable a rule using <a>DisableRule</a>. A single rule watches for events from a single event bus. Events generated by AWS services go to your account's default event bus. Events generated by SaaS partner services or applications go to the matching partner event bus. If you have custom applications or services, you can specify whether their events go to your default event bus or a custom event bus that you have created. For more information, see <a>CreateEventBus</a>. If you're updating an existing rule, the rule is replaced with what you specify in this <code>PutRule</code> command. If you omit arguments in <code>PutRule</code>, the old values for those arguments aren't kept. Instead, they're replaced with null values. When you create or update a rule, incoming events might not immediately start matching to new or updated rules. Allow a short period of time for changes to take effect. A rule must contain at least an <code>EventPattern</code> or <code>ScheduleExpression</code>. Rules with <code>EventPatterns</code> are triggered when a matching event is observed. Rules with <code>ScheduleExpressions</code> self-trigger based on the given schedule. A rule can have both an <code>EventPattern</code> and a <code>ScheduleExpression</code>, in which case the rule triggers on matching events as well as on a schedule. When you initially create a rule, you can optionally assign one or more tags to the rule. Tags can help you organize and categorize your resources. You can also use them to scope user permissions, by granting a user permission to access or change only rules with certain tag values. To use the <code>PutRule</code> operation and assign tags, you must have both the <code>events:PutRule</code> and <code>events:TagResource</code> permissions. If you are updating an existing rule, any tags you specify in the <code>PutRule</code> operation are ignored. To update the tags of an existing rule, use <a>TagResource</a> and <a>UntagResource</a>. Most services in AWS treat <code>:</code> or <code>/</code> as the same character in Amazon Resource Names (ARNs). However, EventBridge uses an exact match in event patterns and rules. Be sure to use the correct ARN characters when creating event patterns so that they match the ARN syntax in the event that you want to match. In EventBridge, you could create rules that lead to infinite loops, where a rule is fired repeatedly. For example, a rule might detect that ACLs have changed on an S3 bucket, and trigger software to change them to the desired state. If you don't write the rule carefully, the subsequent change to the ACLs fires the rule again, creating an infinite loop. To prevent this, write the rules so that the triggered actions don't refire the same rule. For example, your rule could fire only if ACLs are found to be in a bad state, instead of after any change.  An infinite loop can quickly cause higher than expected charges. We recommend that you use budgeting, which alerts you when charges exceed your specified limit. For more information, see <a href="https://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/budgets-managing-costs.html">Managing Your Costs with Budgets</a>.

        :param self:
        :param EventBusName:  The event bus to associate with this rule. If you omit this, the default event bus is used.<br/>Tags:  The list of key-value pairs to associate with the rule.<br/>RoleArn:  The Amazon Resource Name (ARN) of the IAM role associated with the rule.<br/>Description:  A description of the rule.<br/>State:  Indicates whether the rule is enabled or disabled.<br/>EventPattern:  The event pattern. For more information, see <a href="https://docs.aws.amazon.com/eventbridge/latest/userguide/eventbridge-and-event-patterns.html">Event Patterns</a> in the <i>Amazon EventBridge User Guide</i>.<br/>ScheduleExpression:  The scheduling expression: for example, <code>"cron(0 20 * * ? *)"</code> or <code>"rate(5 minutes)"</code>.<br/>Name:  The name of the rule that you're creating or updating.<br/>
        :return: RuleArn:  The Amazon Resource Name (ARN) of the rule.<br/>        """
        pass

    def put_targets(self, Targets: List[Target], Rule: str, EventBusName: str=None) -> PutTargetsResponse:
        """Adds the specified targets to the specified rule, or updates the targets if they're already associated with the rule. Targets are the resources that are invoked when a rule is triggered. You can configure the following as targets in EventBridge: <ul> <li> EC2 instances </li> <li> SSM Run Command </li> <li> SSM Automation </li> <li> AWS Lambda functions </li> <li> Data streams in Amazon Kinesis Data Streams </li> <li> Data delivery streams in Amazon Kinesis Data Firehose </li> <li> Amazon ECS tasks </li> <li> AWS Step Functions state machines </li> <li> AWS Batch jobs </li> <li> AWS CodeBuild projects </li> <li> Pipelines in AWS CodePipeline </li> <li> Amazon Inspector assessment templates </li> <li> Amazon SNS topics </li> <li> Amazon SQS queues, including FIFO queues </li> <li> The default event bus of another AWS account </li> </ul> Creating rules with built-in targets is supported only on the AWS Management Console. The built-in targets are <code>EC2 CreateSnapshot API call</code>, <code>EC2 RebootInstances API call</code>, <code>EC2 StopInstances API call</code>, and <code>EC2 TerminateInstances API call</code>.  For some target types, <code>PutTargets</code> provides target-specific parameters. If the target is a Kinesis data stream, you can optionally specify which shard the event goes to by using the <code>KinesisParameters</code> argument. To invoke a command on multiple EC2 instances with one rule, you can use the <code>RunCommandParameters</code> field. To be able to make API calls against the resources that you own, Amazon EventBridge needs the appropriate permissions. For AWS Lambda and Amazon SNS resources, EventBridge relies on resource-based policies. For EC2 instances, Kinesis data streams, and AWS Step Functions state machines, EventBridge relies on IAM roles that you specify in the <code>RoleARN</code> argument in <code>PutTargets</code>. For more information, see <a href="https://docs.aws.amazon.com/eventbridge/latest/userguide/auth-and-access-control-eventbridge.html">Authentication and Access Control</a> in the <i>Amazon EventBridge User Guide</i>. If another AWS account is in the same Region and has granted you permission (using <code>PutPermission</code>), you can send events to that account. Set that account's event bus as a target of the rules in your account. To send the matched events to the other account, specify that account's event bus as the <code>Arn</code> value when you run <code>PutTargets</code>. If your account sends events to another account, your account is charged for each sent event. Each event sent to another account is charged as a custom event. The account receiving the event isn't charged. For more information, see <a href="https://aws.amazon.com/eventbridge/pricing/">Amazon EventBridge Pricing</a>. If you're setting an event bus in another account as the target and that account granted permission to your account through an organization instead of directly by the account ID, you must specify a <code>RoleArn</code> with proper permissions in the <code>Target</code> structure. For more information, see <a href="https://docs.aws.amazon.com/eventbridge/latest/userguide/eventbridge-cross-account-event-delivery.html">Sending and Receiving Events Between AWS Accounts</a> in the <i>Amazon EventBridge User Guide</i>. For more information about enabling cross-account events, see <a>PutPermission</a>.  <code>Input</code>, <code>InputPath</code>, and <code>InputTransformer</code> are mutually exclusive and optional parameters of a target. When a rule is triggered due to a matched event: <ul> <li> If none of the following arguments are specified for a target, the entire event is passed to the target in JSON format (unless the target is Amazon EC2 Run Command or Amazon ECS task, in which case nothing from the event is passed to the target). </li> <li> If <code>Input</code> is specified in the form of valid JSON, then the matched event is overridden with this constant. </li> <li> If <code>InputPath</code> is specified in the form of JSONPath (for example, <code>$.detail</code>), only the part of the event specified in the path is passed to the target (for example, only the detail part of the event is passed). </li> <li> If <code>InputTransformer</code> is specified, one or more specified JSONPaths are extracted from the event and used as values in a template that you specify as the input to the target. </li> </ul> When you specify <code>InputPath</code> or <code>InputTransformer</code>, you must use JSON dot notation, not bracket notation. When you add targets to a rule and the associated rule triggers soon after, new or updated targets might not be immediately invoked. Allow a short period of time for changes to take effect. This action can partially fail if too many requests are made at the same time. If that happens, <code>FailedEntryCount</code> is nonzero in the response, and each entry in <code>FailedEntries</code> provides the ID of the failed target and the error code.

        :param self:
        :param Targets:  The targets to update or add to the rule.<br/>EventBusName:  The name of the event bus associated with the rule. If you omit this, the default event bus is used.<br/>Rule:  The name of the rule.<br/>
        :return: FailedEntries:  The failed target entries.<br/>FailedEntryCount:  The number of failed entries.<br/>        """
        pass

    def remove_permission(self, StatementId: str, EventBusName: str=None):
        """Revokes the permission of another AWS account to be able to put events to the specified event bus. Specify the account to revoke by the <code>StatementId</code> value that you associated with the account when you granted it permission with <code>PutPermission</code>. You can find the <code>StatementId</code> by using <a>DescribeEventBus</a>.

        :param self:
        :param EventBusName:  The name of the event bus to revoke permissions for. If you omit this, the default event bus is used.<br/>StatementId:  The statement ID corresponding to the account that is no longer allowed to put events to the default event bus.<br/>
        :return:         """
        pass

    def remove_targets(self, Ids: List[TargetId], Rule: str, EventBusName: str=None, Force: Boolean=None) -> RemoveTargetsResponse:
        """Removes the specified targets from the specified rule. When the rule is triggered, those targets are no longer be invoked. When you remove a target, when the associated rule triggers, removed targets might continue to be invoked. Allow a short period of time for changes to take effect. This action can partially fail if too many requests are made at the same time. If that happens, <code>FailedEntryCount</code> is non-zero in the response and each entry in <code>FailedEntries</code> provides the ID of the failed target and the error code.

        :param self:
        :param Force:  If this is a managed rule created by an AWS service on your behalf, you must specify <code>Force</code> as <code>True</code> to remove targets. This parameter is ignored for rules that aren't managed rules. You can check whether a rule is a managed rule by using <code>DescribeRule</code> or <code>ListRules</code> and checking the <code>ManagedBy</code> field of the response.<br/>Ids:  The IDs of the targets to remove from the rule.<br/>EventBusName:  The name of the event bus associated with the rule.<br/>Rule:  The name of the rule.<br/>
        :return: FailedEntries:  The failed target entries.<br/>FailedEntryCount:  The number of failed entries.<br/>        """
        pass

    def tag_resource(self, Tags: List[Tag], ResourceARN: str) -> TagResourceResponse:
        """Assigns one or more tags (key-value pairs) to the specified EventBridge resource. Tags can help you organize and categorize your resources. You can also use them to scope user permissions by granting a user permission to access or change only resources with certain tag values. In EventBridge, rules can be tagged. Tags don't have any semantic meaning to AWS and are interpreted strictly as strings of characters. You can use the <code>TagResource</code> action with a rule that already has tags. If you specify a new tag key for the rule, this tag is appended to the list of tags associated with the rule. If you specify a tag key that is already associated with the rule, the new tag value that you specify replaces the previous value for that tag. You can associate as many as 50 tags with a resource.

        :param self:
        :param Tags:  The list of key-value pairs to associate with the rule.<br/>ResourceARN:  The ARN of the rule that you're adding tags to.<br/>
        :return:         """
        pass

    def test_event_pattern(self, Event: str, EventPattern: str) -> TestEventPatternResponse:
        """Tests whether the specified event pattern matches the provided event. Most services in AWS treat <code>:</code> or <code>/</code> as the same character in Amazon Resource Names (ARNs). However, EventBridge uses an exact match in event patterns and rules. Be sure to use the correct ARN characters when creating event patterns so that they match the ARN syntax in the event that you want to match.

        :param self:
        :param Event:  The event, in JSON format, to test against the event pattern.<br/>EventPattern:  The event pattern. For more information, see <a href="https://docs.aws.amazon.com/eventbridge/latest/userguide/eventbridge-and-event-patterns.html">Event Patterns</a> in the <i>Amazon EventBridge User Guide</i>.<br/>
        :return: Result:  Indicates whether the event matches the event pattern.<br/>        """
        pass

    def untag_resource(self, TagKeys: List[TagKey], ResourceARN: str) -> UntagResourceResponse:
        """Removes one or more tags from the specified EventBridge resource. In EventBridge, rules can be tagged.

        :param self:
        :param TagKeys:  The list of tag keys to remove from the resource.<br/>ResourceARN:  The ARN of the rule that you're removing tags from.<br/>
        :return:         """
        pass



class Firehose(BaseClient):
    """<fullname>Amazon Kinesis Data Firehose API Reference</fullname> Amazon Kinesis Data Firehose is a fully managed service that delivers real-time streaming data to destinations such as Amazon Simple Storage Service (Amazon S3), Amazon Elasticsearch Service (Amazon ES), Amazon Redshift, and Splunk."""




    class CreateDeliveryStreamInput(Mapping):
        pass
    
    class CreateDeliveryStreamOutput(Mapping):
        pass
    
    class KinesisStreamSourceConfiguration(Mapping):
        pass
    
    class S3DestinationConfiguration(Mapping):
        pass
    
    class ExtendedS3DestinationConfiguration(Mapping):
        pass
    
    class RedshiftDestinationConfiguration(Mapping):
        pass
    
    class ElasticsearchDestinationConfiguration(Mapping):
        pass
    
    class SplunkDestinationConfiguration(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class DeleteDeliveryStreamInput(Mapping):
        pass
    
    class DeleteDeliveryStreamOutput(Mapping):
        pass
    
    class DescribeDeliveryStreamInput(Mapping):
        pass
    
    class DescribeDeliveryStreamOutput(Mapping):
        pass
    
    class ListDeliveryStreamsInput(Mapping):
        pass
    
    class ListDeliveryStreamsOutput(Mapping):
        pass
    
    class ListTagsForDeliveryStreamInput(Mapping):
        pass
    
    class ListTagsForDeliveryStreamOutput(Mapping):
        pass
    
    class PutRecordInput(Mapping):
        pass
    
    class PutRecordOutput(Mapping):
        pass
    
    class Record(Mapping):
        pass
    
    class PutRecordBatchInput(Mapping):
        pass
    
    class PutRecordBatchOutput(Mapping):
        pass
    
    class StartDeliveryStreamEncryptionInput(Mapping):
        pass
    
    class StartDeliveryStreamEncryptionOutput(Mapping):
        pass
    
    class StopDeliveryStreamEncryptionInput(Mapping):
        pass
    
    class StopDeliveryStreamEncryptionOutput(Mapping):
        pass
    
    class TagDeliveryStreamInput(Mapping):
        pass
    
    class TagDeliveryStreamOutput(Mapping):
        pass
    
    class UntagDeliveryStreamInput(Mapping):
        pass
    
    class UntagDeliveryStreamOutput(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateDestinationInput(Mapping):
        pass
    
    class UpdateDestinationOutput(Mapping):
        pass
    
    class S3DestinationUpdate(Mapping):
        pass
    
    class ExtendedS3DestinationUpdate(Mapping):
        pass
    
    class RedshiftDestinationUpdate(Mapping):
        pass
    
    class ElasticsearchDestinationUpdate(Mapping):
        pass
    
    class SplunkDestinationUpdate(Mapping):
        pass
    
    def create_delivery_stream(self, DeliveryStreamName: str, DeliveryStreamType: str=None, KinesisStreamSourceConfiguration: KinesisStreamSourceConfiguration=None, S3DestinationConfiguration: S3DestinationConfiguration=None, ExtendedS3DestinationConfiguration: ExtendedS3DestinationConfiguration=None, RedshiftDestinationConfiguration: RedshiftDestinationConfiguration=None, ElasticsearchDestinationConfiguration: ElasticsearchDestinationConfiguration=None, SplunkDestinationConfiguration: SplunkDestinationConfiguration=None, Tags: List[Tag]=None) -> CreateDeliveryStreamOutput:
        """Creates a Kinesis Data Firehose delivery stream. By default, you can create up to 50 delivery streams per AWS Region. This is an asynchronous operation that immediately returns. The initial status of the delivery stream is <code>CREATING</code>. After the delivery stream is created, its status is <code>ACTIVE</code> and it now accepts data. Attempts to send data to a delivery stream that is not in the <code>ACTIVE</code> state cause an exception. To check the state of a delivery stream, use <a>DescribeDeliveryStream</a>. A Kinesis Data Firehose delivery stream can be configured to receive records directly from providers using <a>PutRecord</a> or <a>PutRecordBatch</a>, or it can be configured to use an existing Kinesis stream as its source. To specify a Kinesis data stream as input, set the <code>DeliveryStreamType</code> parameter to <code>KinesisStreamAsSource</code>, and provide the Kinesis stream Amazon Resource Name (ARN) and role ARN in the <code>KinesisStreamSourceConfiguration</code> parameter. A delivery stream is configured with a single destination: Amazon S3, Amazon ES, Amazon Redshift, or Splunk. You must specify only one of the following destination configuration parameters: <code>ExtendedS3DestinationConfiguration</code>, <code>S3DestinationConfiguration</code>, <code>ElasticsearchDestinationConfiguration</code>, <code>RedshiftDestinationConfiguration</code>, or <code>SplunkDestinationConfiguration</code>. When you specify <code>S3DestinationConfiguration</code>, you can also provide the following optional values: BufferingHints, <code>EncryptionConfiguration</code>, and <code>CompressionFormat</code>. By default, if no <code>BufferingHints</code> value is provided, Kinesis Data Firehose buffers data up to 5 MB or for 5 minutes, whichever condition is satisfied first. <code>BufferingHints</code> is a hint, so there are some cases where the service cannot adhere to these conditions strictly. For example, record boundaries might be such that the size is a little over or under the configured buffering size. By default, no encryption is performed. We strongly recommend that you enable encryption to ensure secure data storage in Amazon S3. A few notes about Amazon Redshift as a destination: <ul> <li> An Amazon Redshift destination requires an S3 bucket as intermediate location. Kinesis Data Firehose first delivers data to Amazon S3 and then uses <code>COPY</code> syntax to load data into an Amazon Redshift table. This is specified in the <code>RedshiftDestinationConfiguration.S3Configuration</code> parameter. </li> <li> The compression formats <code>SNAPPY</code> or <code>ZIP</code> cannot be specified in <code>RedshiftDestinationConfiguration.S3Configuration</code> because the Amazon Redshift <code>COPY</code> operation that reads from the S3 bucket doesn't support these compression formats. </li> <li> We strongly recommend that you use the user name and password you provide exclusively with Kinesis Data Firehose, and that the permissions for the account are restricted for Amazon Redshift <code>INSERT</code> permissions. </li> </ul> Kinesis Data Firehose assumes the IAM role that is configured as part of the destination. The role should allow the Kinesis Data Firehose principal to assume the role, and the role should have permissions that allow the service to deliver the data. For more information, see <a href="http://docs.aws.amazon.com/firehose/latest/dev/controlling-access.html#using-iam-s3">Grant Kinesis Data Firehose Access to an Amazon S3 Destination</a> in the <i>Amazon Kinesis Data Firehose Developer Guide</i>.

        :param self:
        :param Tags:  A set of tags to assign to the delivery stream. A tag is a key-value pair that you can define and assign to AWS resources. Tags are metadata. For example, you can add friendly names and descriptions or other types of information that can help you distinguish the delivery stream. For more information about tags, see <a href="https://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/cost-alloc-tags.html">Using Cost Allocation Tags</a> in the AWS Billing and Cost Management User Guide. You can specify up to 50 tags when creating a delivery stream.<br/>SplunkDestinationConfiguration:  The destination in Splunk. You can specify only one destination.:<br/>ElasticsearchDestinationConfiguration:  The destination in Amazon ES. You can specify only one destination.:<br/>RedshiftDestinationConfiguration:  The destination in Amazon Redshift. You can specify only one destination.:<br/>ExtendedS3DestinationConfiguration:  The destination in Amazon S3. You can specify only one destination.:<br/>S3DestinationConfiguration:  [Deprecated] The destination in Amazon S3. You can specify only one destination.:<br/>KinesisStreamSourceConfiguration:  When a Kinesis data stream is used as the source for the delivery stream, a <a>KinesisStreamSourceConfiguration</a> containing the Kinesis data stream Amazon Resource Name (ARN) and the role ARN for the source stream.:<br/>DeliveryStreamType:  The delivery stream type. This parameter can be one of the following values: <ul> <li>  <code>DirectPut</code>: Provider applications access the delivery stream directly. </li> <li>  <code>KinesisStreamAsSource</code>: The delivery stream uses a Kinesis data stream as a source. </li> </ul><br/>DeliveryStreamName:  The name of the delivery stream. This name must be unique per AWS account in the same AWS Region. If the delivery streams are in different accounts or different Regions, you can have multiple delivery streams with the same name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> The ARN of the role that provides access to the source Kinesis data stream. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#arn-syntax-iam">AWS Identity and Access Management (IAM) ARN Format</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KinesisStreamARN</i> The ARN of the source Kinesis data stream. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#arn-syntax-kinesis-streams">Amazon Kinesis Data Streams ARN Format</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchLoggingOptions</i> The CloudWatch logging options for your delivery stream.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CloudWatchLoggingOptions</b>: The CloudWatch logging options for your delivery stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionConfiguration</i> The encryption configuration. If no value is specified, the default is no encryption.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncryptionConfiguration</b>: The encryption configuration. If no value is specified, the default is no encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompressionFormat</i> The compression format. If no value is specified, the default is <code>UNCOMPRESSED</code>. The compression formats <code>SNAPPY</code> or <code>ZIP</code> cannot be specified for Amazon Redshift destinations because they are not supported by the Amazon Redshift <code>COPY</code> operation that reads from the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BufferingHints</i> The buffering option. If no value is specified, <code>BufferingHints</code> object default values are used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BufferingHints</b>: The buffering option. If no value is specified, <code>BufferingHints</code> object default values are used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorOutputPrefix</i> A prefix that Kinesis Data Firehose evaluates and adds to failed records before writing them to S3. This prefix appears immediately following the bucket name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> The "YYYY/MM/DD/HH" time format prefix is automatically used for delivered Amazon S3 files. You can specify an extra prefix to be added in front of the time format prefix. If the prefix ends with a slash, it appears as a folder in the S3 bucket. For more information, see <a href="http://docs.aws.amazon.com/firehose/latest/dev/basic-deliver.html#s3-object-name">Amazon S3 Object Name Format</a> in the <i>Amazon Kinesis Data Firehose Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketARN</i> The ARN of the S3 bucket. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> The Amazon Resource Name (ARN) of the AWS credentials. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IntervalInSeconds</i> Buffer incoming data for the specified period of time, in seconds, before delivering it to the destination. The default value is 300.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SizeInMBs</i> Buffer incoming data to the specified size, in MBs, before delivering it to the destination. The default value is 5. We recommend setting this parameter to a value greater than the amount of data you typically ingest into the delivery stream in 10 seconds. For example, if you typically ingest data at 1 MB/sec, the value should be 10 MB or higher.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NoEncryptionConfig</i> Specifically override existing encryption information to ensure that no encryption is used.<br/>The encryption key.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KMSEncryptionConfig</b>: The encryption key.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LogStreamName</i> The CloudWatch log stream name for logging. This value is required if CloudWatch logging is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LogGroupName</i> The CloudWatch group name for logging. This value is required if CloudWatch logging is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Enables or disables CloudWatch logging.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataFormatConversionConfiguration</i> The serializer, deserializer, and schema for converting data from the JSON format to the Parquet or ORC format before writing it to Amazon S3.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DataFormatConversionConfiguration</b>: The serializer, deserializer, and schema for converting data from the JSON format to the Parquet or ORC format before writing it to Amazon S3.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3BackupConfiguration</i> The configuration for backup in Amazon S3.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3BackupConfiguration</b>: The configuration for backup in Amazon S3.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3BackupMode</i> The Amazon S3 backup mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessingConfiguration</i> The data processing configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ProcessingConfiguration</b>: The data processing configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchLoggingOptions</i> The Amazon CloudWatch logging options for your delivery stream.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CloudWatchLoggingOptions</b>: The Amazon CloudWatch logging options for your delivery stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionConfiguration</i> The encryption configuration. If no value is specified, the default is no encryption.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncryptionConfiguration</b>: The encryption configuration. If no value is specified, the default is no encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompressionFormat</i> The compression format. If no value is specified, the default is UNCOMPRESSED.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BufferingHints</i> The buffering option.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BufferingHints</b>: The buffering option.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorOutputPrefix</i> A prefix that Kinesis Data Firehose evaluates and adds to failed records before writing them to S3. This prefix appears immediately following the bucket name. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> The "YYYY/MM/DD/HH" time format prefix is automatically used for delivered Amazon S3 files. You can specify an extra prefix to be added in front of the time format prefix. If the prefix ends with a slash, it appears as a folder in the S3 bucket. For more information, see <a href="http://docs.aws.amazon.com/firehose/latest/dev/basic-deliver.html#s3-object-name">Amazon S3 Object Name Format</a> in the <i>Amazon Kinesis Data Firehose Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketARN</i> The ARN of the S3 bucket. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> The Amazon Resource Name (ARN) of the AWS credentials. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IntervalInSeconds</i> Buffer incoming data for the specified period of time, in seconds, before delivering it to the destination. The default value is 300.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SizeInMBs</i> Buffer incoming data to the specified size, in MBs, before delivering it to the destination. The default value is 5. We recommend setting this parameter to a value greater than the amount of data you typically ingest into the delivery stream in 10 seconds. For example, if you typically ingest data at 1 MB/sec, the value should be 10 MB or higher.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NoEncryptionConfig</i> Specifically override existing encryption information to ensure that no encryption is used.<br/>The encryption key.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KMSEncryptionConfig</b>: The encryption key.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LogStreamName</i> The CloudWatch log stream name for logging. This value is required if CloudWatch logging is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LogGroupName</i> The CloudWatch group name for logging. This value is required if CloudWatch logging is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Enables or disables CloudWatch logging.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Processors</i> The data processors.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Enables or disables data processing.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorOutputPrefix</i> A prefix that Kinesis Data Firehose evaluates and adds to failed records before writing them to S3. This prefix appears immediately following the bucket name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> The "YYYY/MM/DD/HH" time format prefix is automatically used for delivered Amazon S3 files. You can specify an extra prefix to be added in front of the time format prefix. If the prefix ends with a slash, it appears as a folder in the S3 bucket. For more information, see <a href="http://docs.aws.amazon.com/firehose/latest/dev/basic-deliver.html#s3-object-name">Amazon S3 Object Name Format</a> in the <i>Amazon Kinesis Data Firehose Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketARN</i> The ARN of the S3 bucket. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> The Amazon Resource Name (ARN) of the AWS credentials. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>The buffering option. If no value is specified, <code>BufferingHints</code> object default values are used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BufferingHints</b>: The buffering option. If no value is specified, <code>BufferingHints</code> object default values are used.Specifies the AWS Glue Data Catalog table that contains the column information.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SchemaConfiguration</b>: Specifies the AWS Glue Data Catalog table that contains the column information.&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchLoggingOptions</i> The CloudWatch logging options for your delivery stream.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CloudWatchLoggingOptions</b>: The CloudWatch logging options for your delivery stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3BackupConfiguration</i> The configuration for backup in Amazon S3.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3BackupConfiguration</b>: The configuration for backup in Amazon S3.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3BackupMode</i> The Amazon S3 backup mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessingConfiguration</i> The data processing configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ProcessingConfiguration</b>: The data processing configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Configuration</i> The configuration for the intermediate Amazon S3 location from which Amazon Redshift obtains data. Restrictions are described in the topic for <a>CreateDeliveryStream</a>. The compression formats <code>SNAPPY</code> or <code>ZIP</code> cannot be specified in <code>RedshiftDestinationConfiguration.S3Configuration</code> because the Amazon Redshift <code>COPY</code> operation that reads from the S3 bucket doesn't support these compression formats.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Configuration</b>: The configuration for the intermediate Amazon S3 location from which Amazon Redshift obtains data. Restrictions are described in the topic for <a>CreateDeliveryStream</a>. The compression formats <code>SNAPPY</code> or <code>ZIP</code> cannot be specified in <code>RedshiftDestinationConfiguration.S3Configuration</code> because the Amazon Redshift <code>COPY</code> operation that reads from the S3 bucket doesn't support these compression formats.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RetryOptions</i> The retry behavior in case Kinesis Data Firehose is unable to deliver documents to Amazon Redshift. Default value is 3600 (60 minutes).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RetryOptions</b>: The retry behavior in case Kinesis Data Firehose is unable to deliver documents to Amazon Redshift. Default value is 3600 (60 minutes).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Password</i> The user password.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> The name of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyCommand</i> The <code>COPY</code> command.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CopyCommand</b>: The <code>COPY</code> command.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterJDBCURL</i> The database connection string.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> The Amazon Resource Name (ARN) of the AWS credentials. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyOptions</i> Optional parameters to use with the Amazon Redshift <code>COPY</code> command. For more information, see the "Optional Parameters" section of <a href="http://docs.aws.amazon.com/redshift/latest/dg/r_COPY.html">Amazon Redshift COPY command</a>. Some possible examples that would apply to Kinesis Data Firehose are as follows:  <code>delimiter '\t' lzop;</code> - fields are delimited with "\t" (TAB character) and compressed using lzop.  <code>delimiter '|'</code> - fields are delimited with "|" (this is the default delimiter).  <code>delimiter '|' escape</code> - the delimiter should be escaped.  <code>fixedwidth 'venueid:3,venuename:25,venuecity:12,venuestate:2,venueseats:6'</code> - fields are fixed width in the source, with each width specified after every column in the table.  <code>JSON 's3://mybucket/jsonpaths.txt'</code> - data is in JSON format, and the path specified is the format of the data. For more examples, see <a href="http://docs.aws.amazon.com/redshift/latest/dg/r_COPY_command_examples.html">Amazon Redshift COPY command examples</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTableColumns</i> A comma-separated list of column names.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTableName</i> The name of the target table. The table must already exist in the database.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DurationInSeconds</i> The length of time during which Kinesis Data Firehose retries delivery after a failure, starting from the initial request and including the first attempt. The default value is 3600 seconds (60 minutes). Kinesis Data Firehose does not retry if the value of <code>DurationInSeconds</code> is 0 (zero) or if the first delivery attempt takes longer than the current value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorOutputPrefix</i> A prefix that Kinesis Data Firehose evaluates and adds to failed records before writing them to S3. This prefix appears immediately following the bucket name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> The "YYYY/MM/DD/HH" time format prefix is automatically used for delivered Amazon S3 files. You can specify an extra prefix to be added in front of the time format prefix. If the prefix ends with a slash, it appears as a folder in the S3 bucket. For more information, see <a href="http://docs.aws.amazon.com/firehose/latest/dev/basic-deliver.html#s3-object-name">Amazon S3 Object Name Format</a> in the <i>Amazon Kinesis Data Firehose Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketARN</i> The ARN of the S3 bucket. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> The Amazon Resource Name (ARN) of the AWS credentials. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>The buffering option. If no value is specified, <code>BufferingHints</code> object default values are used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BufferingHints</b>: The buffering option. If no value is specified, <code>BufferingHints</code> object default values are used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Processors</i> The data processors.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Enables or disables data processing.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorOutputPrefix</i> A prefix that Kinesis Data Firehose evaluates and adds to failed records before writing them to S3. This prefix appears immediately following the bucket name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> The "YYYY/MM/DD/HH" time format prefix is automatically used for delivered Amazon S3 files. You can specify an extra prefix to be added in front of the time format prefix. If the prefix ends with a slash, it appears as a folder in the S3 bucket. For more information, see <a href="http://docs.aws.amazon.com/firehose/latest/dev/basic-deliver.html#s3-object-name">Amazon S3 Object Name Format</a> in the <i>Amazon Kinesis Data Firehose Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketARN</i> The ARN of the S3 bucket. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> The Amazon Resource Name (ARN) of the AWS credentials. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>The buffering option. If no value is specified, <code>BufferingHints</code> object default values are used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BufferingHints</b>: The buffering option. If no value is specified, <code>BufferingHints</code> object default values are used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LogStreamName</i> The CloudWatch log stream name for logging. This value is required if CloudWatch logging is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LogGroupName</i> The CloudWatch group name for logging. This value is required if CloudWatch logging is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Enables or disables CloudWatch logging.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchLoggingOptions</i> The Amazon CloudWatch logging options for your delivery stream.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CloudWatchLoggingOptions</b>: The Amazon CloudWatch logging options for your delivery stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessingConfiguration</i> The data processing configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ProcessingConfiguration</b>: The data processing configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Configuration</i> The configuration for the backup Amazon S3 location.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Configuration</b>: The configuration for the backup Amazon S3 location.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3BackupMode</i> Defines how documents should be delivered to Amazon S3. When it is set to <code>FailedDocumentsOnly</code>, Kinesis Data Firehose writes any documents that could not be indexed to the configured Amazon S3 destination, with <code>elasticsearch-failed/</code> appended to the key prefix. When set to <code>AllDocuments</code>, Kinesis Data Firehose delivers all incoming records to Amazon S3, and also writes failed documents with <code>elasticsearch-failed/</code> appended to the prefix. For more information, see <a href="http://docs.aws.amazon.com/firehose/latest/dev/basic-deliver.html#es-s3-backup">Amazon S3 Backup for the Amazon ES Destination</a>. Default value is <code>FailedDocumentsOnly</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RetryOptions</i> The retry behavior in case Kinesis Data Firehose is unable to deliver documents to Amazon ES. The default value is 300 (5 minutes).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RetryOptions</b>: The retry behavior in case Kinesis Data Firehose is unable to deliver documents to Amazon ES. The default value is 300 (5 minutes).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BufferingHints</i> The buffering options. If no value is specified, the default values for <code>ElasticsearchBufferingHints</code> are used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BufferingHints</b>: The buffering options. If no value is specified, the default values for <code>ElasticsearchBufferingHints</code> are used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IndexRotationPeriod</i> The Elasticsearch index rotation period. Index rotation appends a timestamp to the <code>IndexName</code> to facilitate the expiration of old data. For more information, see <a href="http://docs.aws.amazon.com/firehose/latest/dev/basic-deliver.html#es-index-rotation">Index Rotation for the Amazon ES Destination</a>. The default value is <code>OneDay</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TypeName</i> The Elasticsearch type name. For Elasticsearch 6.x, there can be only one type per index. If you try to specify a new type for an existing index that already has another type, Kinesis Data Firehose returns an error during run time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IndexName</i> The Elasticsearch index name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainARN</i> The ARN of the Amazon ES domain. The IAM role must have permissions for <code>DescribeElasticsearchDomain</code>, <code>DescribeElasticsearchDomains</code>, and <code>DescribeElasticsearchDomainConfig</code> after assuming the role specified in <b>RoleARN</b>. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> The Amazon Resource Name (ARN) of the IAM role to be assumed by Kinesis Data Firehose for calling the Amazon ES Configuration API and for indexing documents. For more information, see <a href="http://docs.aws.amazon.com/firehose/latest/dev/controlling-access.html#using-iam-s3">Grant Kinesis Data Firehose Access to an Amazon S3 Destination</a> and <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SizeInMBs</i> Buffer incoming data to the specified size, in MBs, before delivering it to the destination. The default value is 5. We recommend setting this parameter to a value greater than the amount of data you typically ingest into the delivery stream in 10 seconds. For example, if you typically ingest data at 1 MB/sec, the value should be 10 MB or higher.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IntervalInSeconds</i> Buffer incoming data for the specified period of time, in seconds, before delivering it to the destination. The default value is 300 (5 minutes).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DurationInSeconds</i> After an initial failure to deliver to Amazon ES, the total amount of time during which Kinesis Data Firehose retries delivery (including the first attempt). After this time has elapsed, the failed documents are written to Amazon S3. Default value is 300 seconds (5 minutes). A value of 0 (zero) results in no retries.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorOutputPrefix</i> A prefix that Kinesis Data Firehose evaluates and adds to failed records before writing them to S3. This prefix appears immediately following the bucket name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> The "YYYY/MM/DD/HH" time format prefix is automatically used for delivered Amazon S3 files. You can specify an extra prefix to be added in front of the time format prefix. If the prefix ends with a slash, it appears as a folder in the S3 bucket. For more information, see <a href="http://docs.aws.amazon.com/firehose/latest/dev/basic-deliver.html#s3-object-name">Amazon S3 Object Name Format</a> in the <i>Amazon Kinesis Data Firehose Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketARN</i> The ARN of the S3 bucket. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> The Amazon Resource Name (ARN) of the AWS credentials. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>The buffering option. If no value is specified, <code>BufferingHints</code> object default values are used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BufferingHints</b>: The buffering option. If no value is specified, <code>BufferingHints</code> object default values are used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Processors</i> The data processors.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Enables or disables data processing.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LogStreamName</i> The CloudWatch log stream name for logging. This value is required if CloudWatch logging is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LogGroupName</i> The CloudWatch group name for logging. This value is required if CloudWatch logging is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Enables or disables CloudWatch logging.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchLoggingOptions</i> The Amazon CloudWatch logging options for your delivery stream.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CloudWatchLoggingOptions</b>: The Amazon CloudWatch logging options for your delivery stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessingConfiguration</i> The data processing configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ProcessingConfiguration</b>: The data processing configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Configuration</i> The configuration for the backup Amazon S3 location.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Configuration</b>: The configuration for the backup Amazon S3 location.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3BackupMode</i> Defines how documents should be delivered to Amazon S3. When set to <code>FailedDocumentsOnly</code>, Kinesis Data Firehose writes any data that could not be indexed to the configured Amazon S3 destination. When set to <code>AllDocuments</code>, Kinesis Data Firehose delivers all incoming records to Amazon S3, and also writes failed documents to Amazon S3. Default value is <code>FailedDocumentsOnly</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RetryOptions</i> The retry behavior in case Kinesis Data Firehose is unable to deliver data to Splunk, or if it doesn't receive an acknowledgment of receipt from Splunk.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RetryOptions</b>: The retry behavior in case Kinesis Data Firehose is unable to deliver data to Splunk, or if it doesn't receive an acknowledgment of receipt from Splunk.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HECAcknowledgmentTimeoutInSeconds</i> The amount of time that Kinesis Data Firehose waits to receive an acknowledgment from Splunk after it sends it data. At the end of the timeout period, Kinesis Data Firehose either tries to send the data again or considers it an error, based on your retry settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HECToken</i> This is a GUID that you obtain from your Splunk cluster when you create a new HEC endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HECEndpointType</i> This type can be either "Raw" or "Event."<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HECEndpoint</i> The HTTP Event Collector (HEC) endpoint to which Kinesis Data Firehose sends your data.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DurationInSeconds</i> The total amount of time that Kinesis Data Firehose spends on retries. This duration starts after the initial attempt to send data to Splunk fails. It doesn't include the periods during which Kinesis Data Firehose waits for acknowledgment from Splunk after each attempt.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorOutputPrefix</i> A prefix that Kinesis Data Firehose evaluates and adds to failed records before writing them to S3. This prefix appears immediately following the bucket name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> The "YYYY/MM/DD/HH" time format prefix is automatically used for delivered Amazon S3 files. You can specify an extra prefix to be added in front of the time format prefix. If the prefix ends with a slash, it appears as a folder in the S3 bucket. For more information, see <a href="http://docs.aws.amazon.com/firehose/latest/dev/basic-deliver.html#s3-object-name">Amazon S3 Object Name Format</a> in the <i>Amazon Kinesis Data Firehose Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketARN</i> The ARN of the S3 bucket. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> The Amazon Resource Name (ARN) of the AWS credentials. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>The buffering option. If no value is specified, <code>BufferingHints</code> object default values are used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BufferingHints</b>: The buffering option. If no value is specified, <code>BufferingHints</code> object default values are used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Processors</i> The data processors.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Enables or disables data processing.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LogStreamName</i> The CloudWatch log stream name for logging. This value is required if CloudWatch logging is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LogGroupName</i> The CloudWatch group name for logging. This value is required if CloudWatch logging is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Enables or disables CloudWatch logging.<br/>
        :return: DeliveryStreamARN:  The ARN of the delivery stream.<br/>        """
        pass

    def delete_delivery_stream(self, DeliveryStreamName: str) -> DeleteDeliveryStreamOutput:
        """Deletes a delivery stream and its data. You can delete a delivery stream only if it is in <code>ACTIVE</code> or <code>DELETING</code> state, and not in the <code>CREATING</code> state. While the deletion request is in process, the delivery stream is in the <code>DELETING</code> state. To check the state of a delivery stream, use <a>DescribeDeliveryStream</a>. While the delivery stream is <code>DELETING</code> state, the service might continue to accept the records, but it doesn't make any guarantees with respect to delivering the data. Therefore, as a best practice, you should first stop any applications that are sending records before deleting a delivery stream.

        :param self:
        :param DeliveryStreamName:  The name of the delivery stream.<br/>
        :return:         """
        pass

    def describe_delivery_stream(self, DeliveryStreamName: str, Limit: int=None, ExclusiveStartDestinationId: str=None) -> DescribeDeliveryStreamOutput:
        """Describes the specified delivery stream and gets the status. For example, after your delivery stream is created, call <code>DescribeDeliveryStream</code> to see whether the delivery stream is <code>ACTIVE</code> and therefore ready for data to be sent to it.

        :param self:
        :param ExclusiveStartDestinationId:  The ID of the destination to start returning the destination information. Kinesis Data Firehose supports one destination per delivery stream.<br/>Limit:  The limit on the number of destinations to return. You can have one destination per delivery stream.<br/>DeliveryStreamName:  The name of the delivery stream.<br/>
        :return: DeliveryStreamDescription:  Information about the delivery stream.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasMoreDestinations</i> Indicates whether there are more destinations available to list.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Destinations</i> The destinations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Source</i> If the <code>DeliveryStreamType</code> parameter is <code>KinesisStreamAsSource</code>, a <a>SourceDescription</a> object describing the source Kinesis data stream.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Source</b>: If the <code>DeliveryStreamType</code> parameter is <code>KinesisStreamAsSource</code>, a <a>SourceDescription</a> object describing the source Kinesis data stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdateTimestamp</i> The date and time that the delivery stream was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateTimestamp</i> The date and time that the delivery stream was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VersionId</i> Each time the destination is updated for a delivery stream, the version ID is changed, and the current version ID is required when updating the destination. This is so that the service knows it is applying the changes to the correct version of the delivery stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeliveryStreamType</i> The delivery stream type. This can be one of the following values: <ul> <li>  <code>DirectPut</code>: Provider applications access the delivery stream directly. </li> <li>  <code>KinesisStreamAsSource</code>: The delivery stream uses a Kinesis data stream as a source. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeliveryStreamEncryptionConfiguration</i> Indicates the server-side encryption (SSE) status for the delivery stream.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DeliveryStreamEncryptionConfiguration</b>: Indicates the server-side encryption (SSE) status for the delivery stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeliveryStreamStatus</i> The status of the delivery stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeliveryStreamARN</i> The Amazon Resource Name (ARN) of the delivery stream. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeliveryStreamName</i> The name of the delivery stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> For a full description of the different values of this status, see <a>StartDeliveryStreamEncryption</a> and <a>StopDeliveryStreamEncryption</a>.<br/>The <a>KinesisStreamSourceDescription</a> value for the source Kinesis data stream.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KinesisStreamSourceDescription</b>: The <a>KinesisStreamSourceDescription</a> value for the source Kinesis data stream.        """
        pass

    def list_delivery_streams(self, Limit: int=None, DeliveryStreamType: str=None, ExclusiveStartDeliveryStreamName: str=None) -> ListDeliveryStreamsOutput:
        """Lists your delivery streams in alphabetical order of their names. The number of delivery streams might be too large to return using a single call to <code>ListDeliveryStreams</code>. You can limit the number of delivery streams returned, using the <code>Limit</code> parameter. To determine whether there are more delivery streams to list, check the value of <code>HasMoreDeliveryStreams</code> in the output. If there are more delivery streams to list, you can request them by calling this operation again and setting the <code>ExclusiveStartDeliveryStreamName</code> parameter to the name of the last delivery stream returned in the last call.

        :param self:
        :param ExclusiveStartDeliveryStreamName:  The list of delivery streams returned by this call to <code>ListDeliveryStreams</code> will start with the delivery stream whose name comes alphabetically immediately after the name you specify in <code>ExclusiveStartDeliveryStreamName</code>.<br/>DeliveryStreamType:  The delivery stream type. This can be one of the following values: <ul> <li>  <code>DirectPut</code>: Provider applications access the delivery stream directly. </li> <li>  <code>KinesisStreamAsSource</code>: The delivery stream uses a Kinesis data stream as a source. </li> </ul> This parameter is optional. If this parameter is omitted, delivery streams of all types are returned.<br/>Limit:  The maximum number of delivery streams to list. The default value is 10.<br/>
        :return: HasMoreDeliveryStreams:  Indicates whether there are more delivery streams available to list.<br/>DeliveryStreamNames:  The names of the delivery streams.<br/>        """
        pass

    def list_tags_for_delivery_stream(self, DeliveryStreamName: str, ExclusiveStartTagKey: str=None, Limit: int=None) -> ListTagsForDeliveryStreamOutput:
        """Lists the tags for the specified delivery stream. This operation has a limit of five transactions per second per account. 

        :param self:
        :param Limit:  The number of tags to return. If this number is less than the total number of tags associated with the delivery stream, <code>HasMoreTags</code> is set to <code>true</code> in the response. To list additional tags, set <code>ExclusiveStartTagKey</code> to the last key in the response. <br/>ExclusiveStartTagKey:  The key to use as the starting point for the list of tags. If you set this parameter, <code>ListTagsForDeliveryStream</code> gets all tags that occur after <code>ExclusiveStartTagKey</code>.<br/>DeliveryStreamName:  The name of the delivery stream whose tags you want to list.<br/>
        :return: HasMoreTags:  If this is <code>true</code> in the response, more tags are available. To list the remaining tags, set <code>ExclusiveStartTagKey</code> to the key of the last tag returned and call <code>ListTagsForDeliveryStream</code> again.<br/>Tags:  A list of tags associated with <code>DeliveryStreamName</code>, starting with the first tag after <code>ExclusiveStartTagKey</code> and up to the specified <code>Limit</code>.<br/>        """
        pass

    def put_record(self, Record: Record, DeliveryStreamName: str) -> PutRecordOutput:
        """Writes a single data record into an Amazon Kinesis Data Firehose delivery stream. To write multiple data records into a delivery stream, use <a>PutRecordBatch</a>. Applications using these operations are referred to as producers. By default, each delivery stream can take in up to 2,000 transactions per second, 5,000 records per second, or 5 MB per second. If you use <a>PutRecord</a> and <a>PutRecordBatch</a>, the limits are an aggregate across these two operations for each delivery stream. For more information about limits and how to request an increase, see <a href="http://docs.aws.amazon.com/firehose/latest/dev/limits.html">Amazon Kinesis Data Firehose Limits</a>.  You must specify the name of the delivery stream and the data record when using <a>PutRecord</a>. The data record consists of a data blob that can be up to 1,000 KB in size, and any kind of data. For example, it can be a segment from a log file, geographic location data, website clickstream data, and so on. Kinesis Data Firehose buffers records before delivering them to the destination. To disambiguate the data blobs at the destination, a common solution is to use delimiters in the data, such as a newline (<code>\n</code>) or some other character unique within the data. This allows the consumer application to parse individual data items when reading the data from the destination. The <code>PutRecord</code> operation returns a <code>RecordId</code>, which is a unique string assigned to each record. Producer applications can use this ID for purposes such as auditability and investigation. If the <code>PutRecord</code> operation throws a <code>ServiceUnavailableException</code>, back off and retry. If the exception persists, it is possible that the throughput limits have been exceeded for the delivery stream.  Data records sent to Kinesis Data Firehose are stored for 24 hours from the time they are added to a delivery stream as it tries to send the records to the destination. If the destination is unreachable for more than 24 hours, the data is no longer available. <important> Don't concatenate two or more base64 strings to form the data fields of your records. Instead, concatenate the raw data, then perform base64 encoding. </important>

        :param self:
        :param Record:  The record.:<br/>DeliveryStreamName:  The name of the delivery stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Data</i> The data blob, which is base64-encoded when the blob is serialized. The maximum size of the data blob, before base64-encoding, is 1,000 KiB.<br/>
        :return: Encrypted:  Indicates whether server-side encryption (SSE) was enabled during this operation.<br/>RecordId:  The ID of the record.<br/>        """
        pass

    def put_record_batch(self, Records: List[Record], DeliveryStreamName: str) -> PutRecordBatchOutput:
        """Writes multiple data records into a delivery stream in a single call, which can achieve higher throughput per producer than when writing single records. To write single data records into a delivery stream, use <a>PutRecord</a>. Applications using these operations are referred to as producers. By default, each delivery stream can take in up to 2,000 transactions per second, 5,000 records per second, or 5 MB per second. If you use <a>PutRecord</a> and <a>PutRecordBatch</a>, the limits are an aggregate across these two operations for each delivery stream. For more information about limits, see <a href="http://docs.aws.amazon.com/firehose/latest/dev/limits.html">Amazon Kinesis Data Firehose Limits</a>. Each <a>PutRecordBatch</a> request supports up to 500 records. Each record in the request can be as large as 1,000 KB (before 64-bit encoding), up to a limit of 4 MB for the entire request. These limits cannot be changed. You must specify the name of the delivery stream and the data record when using <a>PutRecord</a>. The data record consists of a data blob that can be up to 1,000 KB in size, and any kind of data. For example, it could be a segment from a log file, geographic location data, website clickstream data, and so on. Kinesis Data Firehose buffers records before delivering them to the destination. To disambiguate the data blobs at the destination, a common solution is to use delimiters in the data, such as a newline (<code>\n</code>) or some other character unique within the data. This allows the consumer application to parse individual data items when reading the data from the destination. The <a>PutRecordBatch</a> response includes a count of failed records, <code>FailedPutCount</code>, and an array of responses, <code>RequestResponses</code>. Even if the <a>PutRecordBatch</a> call succeeds, the value of <code>FailedPutCount</code> may be greater than 0, indicating that there are records for which the operation didn't succeed. Each entry in the <code>RequestResponses</code> array provides additional information about the processed record. It directly correlates with a record in the request array using the same ordering, from the top to the bottom. The response array always includes the same number of records as the request array. <code>RequestResponses</code> includes both successfully and unsuccessfully processed records. Kinesis Data Firehose tries to process all records in each <a>PutRecordBatch</a> request. A single record failure does not stop the processing of subsequent records.  A successfully processed record includes a <code>RecordId</code> value, which is unique for the record. An unsuccessfully processed record includes <code>ErrorCode</code> and <code>ErrorMessage</code> values. <code>ErrorCode</code> reflects the type of error, and is one of the following values: <code>ServiceUnavailableException</code> or <code>InternalFailure</code>. <code>ErrorMessage</code> provides more detailed information about the error. If there is an internal server error or a timeout, the write might have completed or it might have failed. If <code>FailedPutCount</code> is greater than 0, retry the request, resending only those records that might have failed processing. This minimizes the possible duplicate records and also reduces the total bytes sent (and corresponding charges). We recommend that you handle any duplicates at the destination. If <a>PutRecordBatch</a> throws <code>ServiceUnavailableException</code>, back off and retry. If the exception persists, it is possible that the throughput limits have been exceeded for the delivery stream. Data records sent to Kinesis Data Firehose are stored for 24 hours from the time they are added to a delivery stream as it attempts to send the records to the destination. If the destination is unreachable for more than 24 hours, the data is no longer available. <important> Don't concatenate two or more base64 strings to form the data fields of your records. Instead, concatenate the raw data, then perform base64 encoding. </important>

        :param self:
        :param Records:  One or more records.<br/>DeliveryStreamName:  The name of the delivery stream.<br/>
        :return: RequestResponses:  The results array. For each record, the index of the response element is the same as the index used in the request array.<br/>Encrypted:  Indicates whether server-side encryption (SSE) was enabled during this operation.<br/>FailedPutCount:  The number of records that might have failed processing. This number might be greater than 0 even if the <a>PutRecordBatch</a> call succeeds. Check <code>FailedPutCount</code> to determine whether there are records that you need to resend.<br/>        """
        pass

    def start_delivery_stream_encryption(self, DeliveryStreamName: str) -> StartDeliveryStreamEncryptionOutput:
        """Enables server-side encryption (SSE) for the delivery stream.  This operation is asynchronous. It returns immediately. When you invoke it, Kinesis Data Firehose first sets the status of the stream to <code>ENABLING</code>, and then to <code>ENABLED</code>. You can continue to read and write data to your stream while its status is <code>ENABLING</code>, but the data is not encrypted. It can take up to 5 seconds after the encryption status changes to <code>ENABLED</code> before all records written to the delivery stream are encrypted. To find out whether a record or a batch of records was encrypted, check the response elements <a>PutRecordOutput$Encrypted</a> and <a>PutRecordBatchOutput$Encrypted</a>, respectively. To check the encryption state of a delivery stream, use <a>DescribeDeliveryStream</a>. You can only enable SSE for a delivery stream that uses <code>DirectPut</code> as its source.  The <code>StartDeliveryStreamEncryption</code> and <code>StopDeliveryStreamEncryption</code> operations have a combined limit of 25 calls per delivery stream per 24 hours. For example, you reach the limit if you call <code>StartDeliveryStreamEncryption</code> 13 times and <code>StopDeliveryStreamEncryption</code> 12 times for the same delivery stream in a 24-hour period.

        :param self:
        :param DeliveryStreamName:  The name of the delivery stream for which you want to enable server-side encryption (SSE).<br/>
        :return:         """
        pass

    def stop_delivery_stream_encryption(self, DeliveryStreamName: str) -> StopDeliveryStreamEncryptionOutput:
        """Disables server-side encryption (SSE) for the delivery stream.  This operation is asynchronous. It returns immediately. When you invoke it, Kinesis Data Firehose first sets the status of the stream to <code>DISABLING</code>, and then to <code>DISABLED</code>. You can continue to read and write data to your stream while its status is <code>DISABLING</code>. It can take up to 5 seconds after the encryption status changes to <code>DISABLED</code> before all records written to the delivery stream are no longer subject to encryption. To find out whether a record or a batch of records was encrypted, check the response elements <a>PutRecordOutput$Encrypted</a> and <a>PutRecordBatchOutput$Encrypted</a>, respectively. To check the encryption state of a delivery stream, use <a>DescribeDeliveryStream</a>.  The <code>StartDeliveryStreamEncryption</code> and <code>StopDeliveryStreamEncryption</code> operations have a combined limit of 25 calls per delivery stream per 24 hours. For example, you reach the limit if you call <code>StartDeliveryStreamEncryption</code> 13 times and <code>StopDeliveryStreamEncryption</code> 12 times for the same delivery stream in a 24-hour period.

        :param self:
        :param DeliveryStreamName:  The name of the delivery stream for which you want to disable server-side encryption (SSE).<br/>
        :return:         """
        pass

    def tag_delivery_stream(self, Tags: List[Tag], DeliveryStreamName: str) -> TagDeliveryStreamOutput:
        """Adds or updates tags for the specified delivery stream. A tag is a key-value pair that you can define and assign to AWS resources. If you specify a tag that already exists, the tag value is replaced with the value that you specify in the request. Tags are metadata. For example, you can add friendly names and descriptions or other types of information that can help you distinguish the delivery stream. For more information about tags, see <a href="https://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/cost-alloc-tags.html">Using Cost Allocation Tags</a> in the <i>AWS Billing and Cost Management User Guide</i>.  Each delivery stream can have up to 50 tags.  This operation has a limit of five transactions per second per account. 

        :param self:
        :param Tags:  A set of key-value pairs to use to create the tags.<br/>DeliveryStreamName:  The name of the delivery stream to which you want to add the tags.<br/>
        :return:         """
        pass

    def untag_delivery_stream(self, TagKeys: List[TagKey], DeliveryStreamName: str) -> UntagDeliveryStreamOutput:
        """Removes tags from the specified delivery stream. Removed tags are deleted, and you can't recover them after this operation successfully completes. If you specify a tag that doesn't exist, the operation ignores it. This operation has a limit of five transactions per second per account. 

        :param self:
        :param TagKeys:  A list of tag keys. Each corresponding tag is removed from the delivery stream.<br/>DeliveryStreamName:  The name of the delivery stream.<br/>
        :return:         """
        pass

    def update_destination(self, DestinationId: str, CurrentDeliveryStreamVersionId: str, DeliveryStreamName: str, S3DestinationUpdate: S3DestinationUpdate=None, ExtendedS3DestinationUpdate: ExtendedS3DestinationUpdate=None, RedshiftDestinationUpdate: RedshiftDestinationUpdate=None, ElasticsearchDestinationUpdate: ElasticsearchDestinationUpdate=None, SplunkDestinationUpdate: SplunkDestinationUpdate=None) -> UpdateDestinationOutput:
        """Updates the specified destination of the specified delivery stream. Use this operation to change the destination type (for example, to replace the Amazon S3 destination with Amazon Redshift) or change the parameters associated with a destination (for example, to change the bucket name of the Amazon S3 destination). The update might not occur immediately. The target delivery stream remains active while the configurations are updated, so data writes to the delivery stream can continue during this process. The updated configurations are usually effective within a few minutes. Switching between Amazon ES and other services is not supported. For an Amazon ES destination, you can only update to another Amazon ES destination. If the destination type is the same, Kinesis Data Firehose merges the configuration parameters specified with the destination configuration that already exists on the delivery stream. If any of the parameters are not specified in the call, the existing values are retained. For example, in the Amazon S3 destination, if <a>EncryptionConfiguration</a> is not specified, then the existing <code>EncryptionConfiguration</code> is maintained on the destination. If the destination type is not the same, for example, changing the destination from Amazon S3 to Amazon Redshift, Kinesis Data Firehose does not merge any parameters. In this case, all parameters must be specified. Kinesis Data Firehose uses <code>CurrentDeliveryStreamVersionId</code> to avoid race conditions and conflicting merges. This is a required field, and the service updates the configuration only if the existing configuration has a version ID that matches. After the update is applied successfully, the version ID is updated, and can be retrieved using <a>DescribeDeliveryStream</a>. Use the new version ID to set <code>CurrentDeliveryStreamVersionId</code> in the next call.

        :param self:
        :param SplunkDestinationUpdate:  Describes an update for a destination in Splunk.:<br/>ElasticsearchDestinationUpdate:  Describes an update for a destination in Amazon ES.:<br/>RedshiftDestinationUpdate:  Describes an update for a destination in Amazon Redshift.:<br/>ExtendedS3DestinationUpdate:  Describes an update for a destination in Amazon S3.:<br/>S3DestinationUpdate:  [Deprecated] Describes an update for a destination in Amazon S3.:<br/>DestinationId:  The ID of the destination.<br/>CurrentDeliveryStreamVersionId:  Obtain this value from the <code>VersionId</code> result of <a>DeliveryStreamDescription</a>. This value is required, and helps the service perform conditional operations. For example, if there is an interleaving update and this value is null, then the update destination fails. After the update is successful, the <code>VersionId</code> value is updated. The service then performs a merge of the old configuration with the new configuration.<br/>DeliveryStreamName:  The name of the delivery stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchLoggingOptions</i> The CloudWatch logging options for your delivery stream.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CloudWatchLoggingOptions</b>: The CloudWatch logging options for your delivery stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionConfiguration</i> The encryption configuration. If no value is specified, the default is no encryption.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncryptionConfiguration</b>: The encryption configuration. If no value is specified, the default is no encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompressionFormat</i> The compression format. If no value is specified, the default is <code>UNCOMPRESSED</code>. The compression formats <code>SNAPPY</code> or <code>ZIP</code> cannot be specified for Amazon Redshift destinations because they are not supported by the Amazon Redshift <code>COPY</code> operation that reads from the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BufferingHints</i> The buffering option. If no value is specified, <code>BufferingHints</code> object default values are used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BufferingHints</b>: The buffering option. If no value is specified, <code>BufferingHints</code> object default values are used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorOutputPrefix</i> A prefix that Kinesis Data Firehose evaluates and adds to failed records before writing them to S3. This prefix appears immediately following the bucket name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> The "YYYY/MM/DD/HH" time format prefix is automatically used for delivered Amazon S3 files. You can specify an extra prefix to be added in front of the time format prefix. If the prefix ends with a slash, it appears as a folder in the S3 bucket. For more information, see <a href="http://docs.aws.amazon.com/firehose/latest/dev/basic-deliver.html#s3-object-name">Amazon S3 Object Name Format</a> in the <i>Amazon Kinesis Data Firehose Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketARN</i> The ARN of the S3 bucket. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> The Amazon Resource Name (ARN) of the AWS credentials. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IntervalInSeconds</i> Buffer incoming data for the specified period of time, in seconds, before delivering it to the destination. The default value is 300.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SizeInMBs</i> Buffer incoming data to the specified size, in MBs, before delivering it to the destination. The default value is 5. We recommend setting this parameter to a value greater than the amount of data you typically ingest into the delivery stream in 10 seconds. For example, if you typically ingest data at 1 MB/sec, the value should be 10 MB or higher.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NoEncryptionConfig</i> Specifically override existing encryption information to ensure that no encryption is used.<br/>The encryption key.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KMSEncryptionConfig</b>: The encryption key.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LogStreamName</i> The CloudWatch log stream name for logging. This value is required if CloudWatch logging is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LogGroupName</i> The CloudWatch group name for logging. This value is required if CloudWatch logging is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Enables or disables CloudWatch logging.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataFormatConversionConfiguration</i> The serializer, deserializer, and schema for converting data from the JSON format to the Parquet or ORC format before writing it to Amazon S3.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DataFormatConversionConfiguration</b>: The serializer, deserializer, and schema for converting data from the JSON format to the Parquet or ORC format before writing it to Amazon S3.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3BackupUpdate</i> The Amazon S3 destination for backup.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3BackupUpdate</b>: The Amazon S3 destination for backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3BackupMode</i> Enables or disables Amazon S3 backup mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessingConfiguration</i> The data processing configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ProcessingConfiguration</b>: The data processing configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchLoggingOptions</i> The Amazon CloudWatch logging options for your delivery stream.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CloudWatchLoggingOptions</b>: The Amazon CloudWatch logging options for your delivery stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionConfiguration</i> The encryption configuration. If no value is specified, the default is no encryption.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncryptionConfiguration</b>: The encryption configuration. If no value is specified, the default is no encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompressionFormat</i> The compression format. If no value is specified, the default is <code>UNCOMPRESSED</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BufferingHints</i> The buffering option.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BufferingHints</b>: The buffering option.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorOutputPrefix</i> A prefix that Kinesis Data Firehose evaluates and adds to failed records before writing them to S3. This prefix appears immediately following the bucket name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> The "YYYY/MM/DD/HH" time format prefix is automatically used for delivered Amazon S3 files. You can specify an extra prefix to be added in front of the time format prefix. If the prefix ends with a slash, it appears as a folder in the S3 bucket. For more information, see <a href="http://docs.aws.amazon.com/firehose/latest/dev/basic-deliver.html#s3-object-name">Amazon S3 Object Name Format</a> in the <i>Amazon Kinesis Data Firehose Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketARN</i> The ARN of the S3 bucket. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> The Amazon Resource Name (ARN) of the AWS credentials. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IntervalInSeconds</i> Buffer incoming data for the specified period of time, in seconds, before delivering it to the destination. The default value is 300.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SizeInMBs</i> Buffer incoming data to the specified size, in MBs, before delivering it to the destination. The default value is 5. We recommend setting this parameter to a value greater than the amount of data you typically ingest into the delivery stream in 10 seconds. For example, if you typically ingest data at 1 MB/sec, the value should be 10 MB or higher.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NoEncryptionConfig</i> Specifically override existing encryption information to ensure that no encryption is used.<br/>The encryption key.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KMSEncryptionConfig</b>: The encryption key.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LogStreamName</i> The CloudWatch log stream name for logging. This value is required if CloudWatch logging is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LogGroupName</i> The CloudWatch group name for logging. This value is required if CloudWatch logging is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Enables or disables CloudWatch logging.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Processors</i> The data processors.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Enables or disables data processing.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorOutputPrefix</i> A prefix that Kinesis Data Firehose evaluates and adds to failed records before writing them to S3. This prefix appears immediately following the bucket name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> The "YYYY/MM/DD/HH" time format prefix is automatically used for delivered Amazon S3 files. You can specify an extra prefix to be added in front of the time format prefix. If the prefix ends with a slash, it appears as a folder in the S3 bucket. For more information, see <a href="http://docs.aws.amazon.com/firehose/latest/dev/basic-deliver.html#s3-object-name">Amazon S3 Object Name Format</a> in the <i>Amazon Kinesis Data Firehose Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketARN</i> The ARN of the S3 bucket. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> The Amazon Resource Name (ARN) of the AWS credentials. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>The buffering option. If no value is specified, <code>BufferingHints</code> object default values are used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BufferingHints</b>: The buffering option. If no value is specified, <code>BufferingHints</code> object default values are used.Specifies the AWS Glue Data Catalog table that contains the column information.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SchemaConfiguration</b>: Specifies the AWS Glue Data Catalog table that contains the column information.&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchLoggingOptions</i> The Amazon CloudWatch logging options for your delivery stream.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CloudWatchLoggingOptions</b>: The Amazon CloudWatch logging options for your delivery stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3BackupUpdate</i> The Amazon S3 destination for backup.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3BackupUpdate</b>: The Amazon S3 destination for backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3BackupMode</i> The Amazon S3 backup mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessingConfiguration</i> The data processing configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ProcessingConfiguration</b>: The data processing configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Update</i> The Amazon S3 destination. The compression formats <code>SNAPPY</code> or <code>ZIP</code> cannot be specified in <code>RedshiftDestinationUpdate.S3Update</code> because the Amazon Redshift <code>COPY</code> operation that reads from the S3 bucket doesn't support these compression formats.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Update</b>: The Amazon S3 destination. The compression formats <code>SNAPPY</code> or <code>ZIP</code> cannot be specified in <code>RedshiftDestinationUpdate.S3Update</code> because the Amazon Redshift <code>COPY</code> operation that reads from the S3 bucket doesn't support these compression formats.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RetryOptions</i> The retry behavior in case Kinesis Data Firehose is unable to deliver documents to Amazon Redshift. Default value is 3600 (60 minutes).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RetryOptions</b>: The retry behavior in case Kinesis Data Firehose is unable to deliver documents to Amazon Redshift. Default value is 3600 (60 minutes).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Password</i> The user password.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> The name of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyCommand</i> The <code>COPY</code> command.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CopyCommand</b>: The <code>COPY</code> command.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterJDBCURL</i> The database connection string.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> The Amazon Resource Name (ARN) of the AWS credentials. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyOptions</i> Optional parameters to use with the Amazon Redshift <code>COPY</code> command. For more information, see the "Optional Parameters" section of <a href="http://docs.aws.amazon.com/redshift/latest/dg/r_COPY.html">Amazon Redshift COPY command</a>. Some possible examples that would apply to Kinesis Data Firehose are as follows:  <code>delimiter '\t' lzop;</code> - fields are delimited with "\t" (TAB character) and compressed using lzop.  <code>delimiter '|'</code> - fields are delimited with "|" (this is the default delimiter).  <code>delimiter '|' escape</code> - the delimiter should be escaped.  <code>fixedwidth 'venueid:3,venuename:25,venuecity:12,venuestate:2,venueseats:6'</code> - fields are fixed width in the source, with each width specified after every column in the table.  <code>JSON 's3://mybucket/jsonpaths.txt'</code> - data is in JSON format, and the path specified is the format of the data. For more examples, see <a href="http://docs.aws.amazon.com/redshift/latest/dg/r_COPY_command_examples.html">Amazon Redshift COPY command examples</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTableColumns</i> A comma-separated list of column names.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTableName</i> The name of the target table. The table must already exist in the database.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DurationInSeconds</i> The length of time during which Kinesis Data Firehose retries delivery after a failure, starting from the initial request and including the first attempt. The default value is 3600 seconds (60 minutes). Kinesis Data Firehose does not retry if the value of <code>DurationInSeconds</code> is 0 (zero) or if the first delivery attempt takes longer than the current value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorOutputPrefix</i> A prefix that Kinesis Data Firehose evaluates and adds to failed records before writing them to S3. This prefix appears immediately following the bucket name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> The "YYYY/MM/DD/HH" time format prefix is automatically used for delivered Amazon S3 files. You can specify an extra prefix to be added in front of the time format prefix. If the prefix ends with a slash, it appears as a folder in the S3 bucket. For more information, see <a href="http://docs.aws.amazon.com/firehose/latest/dev/basic-deliver.html#s3-object-name">Amazon S3 Object Name Format</a> in the <i>Amazon Kinesis Data Firehose Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketARN</i> The ARN of the S3 bucket. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> The Amazon Resource Name (ARN) of the AWS credentials. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>The buffering option. If no value is specified, <code>BufferingHints</code> object default values are used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BufferingHints</b>: The buffering option. If no value is specified, <code>BufferingHints</code> object default values are used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Processors</i> The data processors.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Enables or disables data processing.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorOutputPrefix</i> A prefix that Kinesis Data Firehose evaluates and adds to failed records before writing them to S3. This prefix appears immediately following the bucket name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> The "YYYY/MM/DD/HH" time format prefix is automatically used for delivered Amazon S3 files. You can specify an extra prefix to be added in front of the time format prefix. If the prefix ends with a slash, it appears as a folder in the S3 bucket. For more information, see <a href="http://docs.aws.amazon.com/firehose/latest/dev/basic-deliver.html#s3-object-name">Amazon S3 Object Name Format</a> in the <i>Amazon Kinesis Data Firehose Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketARN</i> The ARN of the S3 bucket. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> The Amazon Resource Name (ARN) of the AWS credentials. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>The buffering option. If no value is specified, <code>BufferingHints</code> object default values are used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BufferingHints</b>: The buffering option. If no value is specified, <code>BufferingHints</code> object default values are used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LogStreamName</i> The CloudWatch log stream name for logging. This value is required if CloudWatch logging is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LogGroupName</i> The CloudWatch group name for logging. This value is required if CloudWatch logging is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Enables or disables CloudWatch logging.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchLoggingOptions</i> The CloudWatch logging options for your delivery stream.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CloudWatchLoggingOptions</b>: The CloudWatch logging options for your delivery stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessingConfiguration</i> The data processing configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ProcessingConfiguration</b>: The data processing configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Update</i> The Amazon S3 destination.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Update</b>: The Amazon S3 destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RetryOptions</i> The retry behavior in case Kinesis Data Firehose is unable to deliver documents to Amazon ES. The default value is 300 (5 minutes).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RetryOptions</b>: The retry behavior in case Kinesis Data Firehose is unable to deliver documents to Amazon ES. The default value is 300 (5 minutes).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BufferingHints</i> The buffering options. If no value is specified, <code>ElasticsearchBufferingHints</code> object default values are used. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BufferingHints</b>: The buffering options. If no value is specified, <code>ElasticsearchBufferingHints</code> object default values are used. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IndexRotationPeriod</i> The Elasticsearch index rotation period. Index rotation appends a timestamp to <code>IndexName</code> to facilitate the expiration of old data. For more information, see <a href="http://docs.aws.amazon.com/firehose/latest/dev/basic-deliver.html#es-index-rotation">Index Rotation for the Amazon ES Destination</a>. Default value is <code>OneDay</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TypeName</i> The Elasticsearch type name. For Elasticsearch 6.x, there can be only one type per index. If you try to specify a new type for an existing index that already has another type, Kinesis Data Firehose returns an error during runtime.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IndexName</i> The Elasticsearch index name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainARN</i> The ARN of the Amazon ES domain. The IAM role must have permissions for <code>DescribeElasticsearchDomain</code>, <code>DescribeElasticsearchDomains</code>, and <code>DescribeElasticsearchDomainConfig</code> after assuming the IAM role specified in <code>RoleARN</code>. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> The Amazon Resource Name (ARN) of the IAM role to be assumed by Kinesis Data Firehose for calling the Amazon ES Configuration API and for indexing documents. For more information, see <a href="http://docs.aws.amazon.com/firehose/latest/dev/controlling-access.html#using-iam-s3">Grant Kinesis Data Firehose Access to an Amazon S3 Destination</a> and <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SizeInMBs</i> Buffer incoming data to the specified size, in MBs, before delivering it to the destination. The default value is 5. We recommend setting this parameter to a value greater than the amount of data you typically ingest into the delivery stream in 10 seconds. For example, if you typically ingest data at 1 MB/sec, the value should be 10 MB or higher.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IntervalInSeconds</i> Buffer incoming data for the specified period of time, in seconds, before delivering it to the destination. The default value is 300 (5 minutes).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DurationInSeconds</i> After an initial failure to deliver to Amazon ES, the total amount of time during which Kinesis Data Firehose retries delivery (including the first attempt). After this time has elapsed, the failed documents are written to Amazon S3. Default value is 300 seconds (5 minutes). A value of 0 (zero) results in no retries.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorOutputPrefix</i> A prefix that Kinesis Data Firehose evaluates and adds to failed records before writing them to S3. This prefix appears immediately following the bucket name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> The "YYYY/MM/DD/HH" time format prefix is automatically used for delivered Amazon S3 files. You can specify an extra prefix to be added in front of the time format prefix. If the prefix ends with a slash, it appears as a folder in the S3 bucket. For more information, see <a href="http://docs.aws.amazon.com/firehose/latest/dev/basic-deliver.html#s3-object-name">Amazon S3 Object Name Format</a> in the <i>Amazon Kinesis Data Firehose Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketARN</i> The ARN of the S3 bucket. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> The Amazon Resource Name (ARN) of the AWS credentials. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>The buffering option. If no value is specified, <code>BufferingHints</code> object default values are used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BufferingHints</b>: The buffering option. If no value is specified, <code>BufferingHints</code> object default values are used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Processors</i> The data processors.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Enables or disables data processing.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LogStreamName</i> The CloudWatch log stream name for logging. This value is required if CloudWatch logging is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LogGroupName</i> The CloudWatch group name for logging. This value is required if CloudWatch logging is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Enables or disables CloudWatch logging.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchLoggingOptions</i> The Amazon CloudWatch logging options for your delivery stream.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CloudWatchLoggingOptions</b>: The Amazon CloudWatch logging options for your delivery stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessingConfiguration</i> The data processing configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ProcessingConfiguration</b>: The data processing configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Update</i> Your update to the configuration of the backup Amazon S3 location.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Update</b>: Your update to the configuration of the backup Amazon S3 location.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3BackupMode</i> Defines how documents should be delivered to Amazon S3. When set to <code>FailedDocumentsOnly</code>, Kinesis Data Firehose writes any data that could not be indexed to the configured Amazon S3 destination. When set to <code>AllDocuments</code>, Kinesis Data Firehose delivers all incoming records to Amazon S3, and also writes failed documents to Amazon S3. Default value is <code>FailedDocumentsOnly</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RetryOptions</i> The retry behavior in case Kinesis Data Firehose is unable to deliver data to Splunk or if it doesn't receive an acknowledgment of receipt from Splunk.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RetryOptions</b>: The retry behavior in case Kinesis Data Firehose is unable to deliver data to Splunk or if it doesn't receive an acknowledgment of receipt from Splunk.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HECAcknowledgmentTimeoutInSeconds</i> The amount of time that Kinesis Data Firehose waits to receive an acknowledgment from Splunk after it sends data. At the end of the timeout period, Kinesis Data Firehose either tries to send the data again or considers it an error, based on your retry settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HECToken</i> A GUID that you obtain from your Splunk cluster when you create a new HEC endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HECEndpointType</i> This type can be either "Raw" or "Event."<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HECEndpoint</i> The HTTP Event Collector (HEC) endpoint to which Kinesis Data Firehose sends your data.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DurationInSeconds</i> The total amount of time that Kinesis Data Firehose spends on retries. This duration starts after the initial attempt to send data to Splunk fails. It doesn't include the periods during which Kinesis Data Firehose waits for acknowledgment from Splunk after each attempt.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorOutputPrefix</i> A prefix that Kinesis Data Firehose evaluates and adds to failed records before writing them to S3. This prefix appears immediately following the bucket name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> The "YYYY/MM/DD/HH" time format prefix is automatically used for delivered Amazon S3 files. You can specify an extra prefix to be added in front of the time format prefix. If the prefix ends with a slash, it appears as a folder in the S3 bucket. For more information, see <a href="http://docs.aws.amazon.com/firehose/latest/dev/basic-deliver.html#s3-object-name">Amazon S3 Object Name Format</a> in the <i>Amazon Kinesis Data Firehose Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketARN</i> The ARN of the S3 bucket. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> The Amazon Resource Name (ARN) of the AWS credentials. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>The buffering option. If no value is specified, <code>BufferingHints</code> object default values are used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BufferingHints</b>: The buffering option. If no value is specified, <code>BufferingHints</code> object default values are used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Processors</i> The data processors.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Enables or disables data processing.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LogStreamName</i> The CloudWatch log stream name for logging. This value is required if CloudWatch logging is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LogGroupName</i> The CloudWatch group name for logging. This value is required if CloudWatch logging is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Enables or disables CloudWatch logging.<br/>
        :return:         """
        pass



class FMS(BaseClient):
    """<fullname>AWS Firewall Manager</fullname> This is the <i>AWS Firewall Manager API Reference</i>. This guide is for developers who need detailed information about the AWS Firewall Manager API actions, data types, and errors. For detailed information about AWS Firewall Manager features, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/fms-chapter.html">AWS Firewall Manager Developer Guide</a>."""


    class ListComplianceStatusPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_compliance_status` for valid parameters"""
            pass
    class ListMemberAccountsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_member_accounts` for valid parameters"""
            pass
    class ListPoliciesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_policies` for valid parameters"""
            pass


    class AssociateAdminAccountRequest(Mapping):
        pass
    
    class DeleteNotificationChannelRequest(Mapping):
        pass
    
    class DeletePolicyRequest(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class DisassociateAdminAccountRequest(Mapping):
        pass
    
    class GetAdminAccountRequest(Mapping):
        pass
    
    class GetAdminAccountResponse(Mapping):
        pass
    
    class GetComplianceDetailRequest(Mapping):
        pass
    
    class GetComplianceDetailResponse(Mapping):
        pass
    
    class GetNotificationChannelRequest(Mapping):
        pass
    
    class GetNotificationChannelResponse(Mapping):
        pass
    
    class GetPolicyRequest(Mapping):
        pass
    
    class GetPolicyResponse(Mapping):
        pass
    
    class GetProtectionStatusRequest(Mapping):
        pass
    
    class GetProtectionStatusResponse(Mapping):
        pass
    
    class TimeStamp(object):
        pass
    
    class ListComplianceStatusRequest(Mapping):
        pass
    
    class ListComplianceStatusResponse(Mapping):
        pass
    
    class ListMemberAccountsRequest(Mapping):
        pass
    
    class ListMemberAccountsResponse(Mapping):
        pass
    
    class ListPoliciesRequest(Mapping):
        pass
    
    class ListPoliciesResponse(Mapping):
        pass
    
    class PutNotificationChannelRequest(Mapping):
        pass
    
    class PutPolicyRequest(Mapping):
        pass
    
    class PutPolicyResponse(Mapping):
        pass
    
    class Policy(Mapping):
        pass
    
    def associate_admin_account(self, AdminAccount: str):
        """Sets the AWS Firewall Manager administrator account. AWS Firewall Manager must be associated with the master account your AWS organization or associated with a member account that has the appropriate permissions. If the account ID that you submit is not an AWS Organizations master account, AWS Firewall Manager will set the appropriate permissions for the given member account. The account that you associate with AWS Firewall Manager is called the AWS Firewall Manager administrator account. 

        :param self:
        :param AdminAccount:  The AWS account ID to associate with AWS Firewall Manager as the AWS Firewall Manager administrator account. This can be an AWS Organizations master account or a member account. For more information about AWS Organizations and master accounts, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_manage_accounts.html">Managing the AWS Accounts in Your Organization</a>.<br/>
        :return:         """
        pass

    def delete_notification_channel(self):
        """Deletes an AWS Firewall Manager association with the IAM role and the Amazon Simple Notification Service (SNS) topic that is used to record AWS Firewall Manager SNS logs.

        :param self:
        :param 
        :return:         """
        pass

    def delete_policy(self, PolicyId: str, DeleteAllPolicyResources: Boolean=None):
        """Permanently deletes an AWS Firewall Manager policy. 

        :param self:
        :param DeleteAllPolicyResources:  If <code>True</code>, the request will also perform a clean-up process that will: <ul> <li> Delete rule groups created by AWS Firewall Manager </li> <li> Remove web ACLs from in-scope resources </li> <li> Delete web ACLs that contain no rules or rule groups </li> </ul> After the cleanup, in-scope resources will no longer be protected by web ACLs in this policy. Protection of out-of-scope resources will remain unchanged. Scope is determined by tags and accounts associated with the policy. When creating the policy, if you specified that only resources in specific accounts or with specific tags be protected by the policy, those resources are in-scope. All others are out of scope. If you did not specify tags or accounts, all resources are in-scope. <br/>PolicyId:  The ID of the policy that you want to delete. <code>PolicyId</code> is returned by <code>PutPolicy</code> and by <code>ListPolicies</code>.<br/>
        :return:         """
        pass

    def disassociate_admin_account(self):
        """Disassociates the account that has been set as the AWS Firewall Manager administrator account. To set a different account as the administrator account, you must submit an <code>AssociateAdminAccount</code> request .

        :param self:
        :param 
        :return:         """
        pass

    def get_admin_account(self) -> GetAdminAccountResponse:
        """Returns the AWS Organizations master account that is associated with AWS Firewall Manager as the AWS Firewall Manager administrator.

        :param self:
        :param 
        :return: RoleStatus:  The status of the AWS account that you set as the AWS Firewall Manager administrator.<br/>AdminAccount:  The AWS account that is set as the AWS Firewall Manager administrator.<br/>        """
        pass

    def get_compliance_detail(self, MemberAccount: str, PolicyId: str) -> GetComplianceDetailResponse:
        """Returns detailed compliance information about the specified member account. Details include resources that are in and out of compliance with the specified policy. Resources are considered non-compliant if the specified policy has not been applied to them.

        :param self:
        :param MemberAccount:  The AWS account that owns the resources that you want to get the details for.<br/>PolicyId:  The ID of the policy that you want to get the details for. <code>PolicyId</code> is returned by <code>PutPolicy</code> and by <code>ListPolicies</code>.<br/>
        :return: PolicyComplianceDetail:  Information about the resources and the policy that you specified in the <code>GetComplianceDetail</code> request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IssueInfoMap</i> Details about problems with dependent services, such as AWS WAF or AWS Config, that are causing a resource to be non-compliant. The details include the name of the dependent service and the error message received that indicates the problem with the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpiredAt</i> A time stamp that indicates when the returned information should be considered out-of-date.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EvaluationLimitExceeded</i> Indicates if over 100 resources are non-compliant with the AWS Firewall Manager policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Violators</i> An array of resources that are not protected by the policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MemberAccount</i> The AWS account ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyId</i> The ID of the AWS Firewall Manager policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyOwner</i> The AWS account that created the AWS Firewall Manager policy.<br/>        """
        pass

    def get_notification_channel(self) -> GetNotificationChannelResponse:
        """Returns information about the Amazon Simple Notification Service (SNS) topic that is used to record AWS Firewall Manager SNS logs.

        :param self:
        :param 
        :return: SnsRoleName:  The IAM role that is used by AWS Firewall Manager to record activity to SNS.<br/>SnsTopicArn:  The SNS topic that records AWS Firewall Manager activity. <br/>        """
        pass

    def get_policy(self, PolicyId: str) -> GetPolicyResponse:
        """Returns information about the specified AWS Firewall Manager policy.

        :param self:
        :param PolicyId:  The ID of the AWS Firewall Manager policy that you want the details for.<br/>
        :return: PolicyArn:  The Amazon Resource Name (ARN) of the specified policy.<br/>Policy:  Information about the specified AWS Firewall Manager policy.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExcludeMap</i> Specifies the AWS account IDs to exclude from the policy. The <code>IncludeMap</code> values are evaluated first, with all the appropriate account IDs added to the policy. Then the accounts listed in <code>ExcludeMap</code> are removed, resulting in the final list of accounts to add to the policy. The key to the map is <code>ACCOUNT</code>. For example, a valid <code>ExcludeMap</code> would be <code>{“ACCOUNT” : [“accountID1”, “accountID2”]}</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeMap</i> Specifies the AWS account IDs to include in the policy. If <code>IncludeMap</code> is null, all accounts in the organization in AWS Organizations are included in the policy. If <code>IncludeMap</code> is not null, only values listed in <code>IncludeMap</code> are included in the policy. The key to the map is <code>ACCOUNT</code>. For example, a valid <code>IncludeMap</code> would be <code>{“ACCOUNT” : [“accountID1”, “accountID2”]}</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RemediationEnabled</i> Indicates if the policy should be automatically applied to new resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExcludeResourceTags</i> If set to <code>True</code>, resources with the tags that are specified in the <code>ResourceTag</code> array are not protected by the policy. If set to <code>False</code>, and the <code>ResourceTag</code> array is not null, only resources with the specified tags are associated with the policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceTags</i> An array of <code>ResourceTag</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceTypeList</i> An array of <code>ResourceType</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceType</i> The type of resource to protect with the policy. This is in the format shown in <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-template-resource-type-ref.html">AWS Resource Types Reference</a>. For example: <code>AWS::ElasticLoadBalancingV2::LoadBalancer</code> or <code>AWS::CloudFront::Distribution</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityServicePolicyData</i> Details about the security service that is being used to protect the resources.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SecurityServicePolicyData</b>: Details about the security service that is being used to protect the resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyUpdateToken</i> A unique identifier for each update to the policy. When issuing a <code>PutPolicy</code> request, the <code>PolicyUpdateToken</code> in the request must match the <code>PolicyUpdateToken</code> of the current policy version. To get the <code>PolicyUpdateToken</code> of the current policy version, use a <code>GetPolicy</code> request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyName</i> The friendly name of the AWS Firewall Manager policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyId</i> The ID of the AWS Firewall Manager policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ManagedServiceData</i> Details about the service. This contains <code>WAF</code> data in JSON format, as shown in the following example:  <code>ManagedServiceData": "{\"type\": \"WAF\", \"ruleGroups\": [{\"id\": \"12345678-1bcd-9012-efga-0987654321ab\", \"overrideAction\" : {\"type\": \"COUNT\"}}], \"defaultAction\": {\"type\": \"BLOCK\"}}</code>  If this is a Shield Advanced policy, this string will be empty.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The service that the policy is using to protect the resources. This specifies the type of policy that is created, either a WAF policy or Shield Advanced policy.<br/>        """
        pass

    def get_protection_status(self, PolicyId: str, MemberAccountId: str=None, StartTime: TimeStamp=None, EndTime: TimeStamp=None, NextToken: str=None, MaxResults: int=None) -> GetProtectionStatusResponse:
        """If you created a Shield Advanced policy, returns policy-level attack summary information in the event of a potential DDoS attack.

        :param self:
        :param MaxResults:  Specifies the number of objects that you want AWS Firewall Manager to return for this request. If you have more objects than the number that you specify for <code>MaxResults</code>, the response includes a <code>NextToken</code> value that you can use to get another batch of objects.<br/>NextToken:  If you specify a value for <code>MaxResults</code> and you have more objects than the number that you specify for <code>MaxResults</code>, AWS Firewall Manager returns a <code>NextToken</code> value in the response that allows you to list another group of objects. For the second and subsequent <code>GetProtectionStatus</code> requests, specify the value of <code>NextToken</code> from the previous response to get information about another batch of objects.<br/>EndTime:  The end of the time period to query for the attacks. This is a <code>timestamp</code> type. The sample request above indicates a number type because the default used by AWS Firewall Manager is Unix time in seconds. However, any valid <code>timestamp</code> format is allowed.<br/>StartTime:  The start of the time period to query for the attacks. This is a <code>timestamp</code> type. The sample request above indicates a number type because the default used by AWS Firewall Manager is Unix time in seconds. However, any valid <code>timestamp</code> format is allowed.<br/>MemberAccountId:  The AWS account that is in scope of the policy that you want to get the details for.<br/>PolicyId:  The ID of the policy for which you want to get the attack information.<br/>
        :return: NextToken:  If you have more objects than the number that you specified for <code>MaxResults</code> in the request, the response includes a <code>NextToken</code> value. To list more objects, submit another <code>GetProtectionStatus</code> request, and specify the <code>NextToken</code> value from the response in the <code>NextToken</code> value in the next request. AWS SDKs provide auto-pagination that identify <code>NextToken</code> in a response and make subsequent request calls automatically on your behalf. However, this feature is not supported by <code>GetProtectionStatus</code>. You must submit subsequent requests with <code>NextToken</code> using your own processes. <br/>Data:  Details about the attack, including the following: <ul> <li> Attack type </li> <li> Account ID </li> <li> ARN of the resource attacked </li> <li> Start time of the attack </li> <li> End time of the attack (ongoing attacks will not have an end time) </li> </ul> The details are in JSON format. An example is shown in the Examples section below.<br/>ServiceType:  The service type that is protected by the policy. Currently, this is always <code>SHIELD_ADVANCED</code>.<br/>AdminAccountId:  The ID of the AWS Firewall administrator account for this policy.<br/>        """
        pass

    def list_compliance_status(self, PolicyId: str, NextToken: str=None, MaxResults: int=None) -> ListComplianceStatusResponse:
        """Returns an array of <code>PolicyComplianceStatus</code> objects in the response. Use <code>PolicyComplianceStatus</code> to get a summary of which member accounts are protected by the specified policy. 

        :param self:
        :param MaxResults:  Specifies the number of <code>PolicyComplianceStatus</code> objects that you want AWS Firewall Manager to return for this request. If you have more <code>PolicyComplianceStatus</code> objects than the number that you specify for <code>MaxResults</code>, the response includes a <code>NextToken</code> value that you can use to get another batch of <code>PolicyComplianceStatus</code> objects.<br/>NextToken:  If you specify a value for <code>MaxResults</code> and you have more <code>PolicyComplianceStatus</code> objects than the number that you specify for <code>MaxResults</code>, AWS Firewall Manager returns a <code>NextToken</code> value in the response that allows you to list another group of <code>PolicyComplianceStatus</code> objects. For the second and subsequent <code>ListComplianceStatus</code> requests, specify the value of <code>NextToken</code> from the previous response to get information about another batch of <code>PolicyComplianceStatus</code> objects.<br/>PolicyId:  The ID of the AWS Firewall Manager policy that you want the details for.<br/>
        :return: NextToken:  If you have more <code>PolicyComplianceStatus</code> objects than the number that you specified for <code>MaxResults</code> in the request, the response includes a <code>NextToken</code> value. To list more <code>PolicyComplianceStatus</code> objects, submit another <code>ListComplianceStatus</code> request, and specify the <code>NextToken</code> value from the response in the <code>NextToken</code> value in the next request.<br/>PolicyComplianceStatusList:  An array of <code>PolicyComplianceStatus</code> objects.<br/>        """
        pass

    def list_member_accounts(self, NextToken: str=None, MaxResults: int=None) -> ListMemberAccountsResponse:
        """Returns a <code>MemberAccounts</code> object that lists the member accounts in the administrator's AWS organization. The <code>ListMemberAccounts</code> must be submitted by the account that is set as the AWS Firewall Manager administrator.

        :param self:
        :param MaxResults:  Specifies the number of member account IDs that you want AWS Firewall Manager to return for this request. If you have more IDs than the number that you specify for <code>MaxResults</code>, the response includes a <code>NextToken</code> value that you can use to get another batch of member account IDs.<br/>NextToken:  If you specify a value for <code>MaxResults</code> and you have more account IDs than the number that you specify for <code>MaxResults</code>, AWS Firewall Manager returns a <code>NextToken</code> value in the response that allows you to list another group of IDs. For the second and subsequent <code>ListMemberAccountsRequest</code> requests, specify the value of <code>NextToken</code> from the previous response to get information about another batch of member account IDs.<br/>
        :return: NextToken:  If you have more member account IDs than the number that you specified for <code>MaxResults</code> in the request, the response includes a <code>NextToken</code> value. To list more IDs, submit another <code>ListMemberAccounts</code> request, and specify the <code>NextToken</code> value from the response in the <code>NextToken</code> value in the next request.<br/>MemberAccounts:  An array of account IDs.<br/>        """
        pass

    def list_policies(self, NextToken: str=None, MaxResults: int=None) -> ListPoliciesResponse:
        """Returns an array of <code>PolicySummary</code> objects in the response.

        :param self:
        :param MaxResults:  Specifies the number of <code>PolicySummary</code> objects that you want AWS Firewall Manager to return for this request. If you have more <code>PolicySummary</code> objects than the number that you specify for <code>MaxResults</code>, the response includes a <code>NextToken</code> value that you can use to get another batch of <code>PolicySummary</code> objects.<br/>NextToken:  If you specify a value for <code>MaxResults</code> and you have more <code>PolicySummary</code> objects than the number that you specify for <code>MaxResults</code>, AWS Firewall Manager returns a <code>NextToken</code> value in the response that allows you to list another group of <code>PolicySummary</code> objects. For the second and subsequent <code>ListPolicies</code> requests, specify the value of <code>NextToken</code> from the previous response to get information about another batch of <code>PolicySummary</code> objects.<br/>
        :return: NextToken:  If you have more <code>PolicySummary</code> objects than the number that you specified for <code>MaxResults</code> in the request, the response includes a <code>NextToken</code> value. To list more <code>PolicySummary</code> objects, submit another <code>ListPolicies</code> request, and specify the <code>NextToken</code> value from the response in the <code>NextToken</code> value in the next request.<br/>PolicyList:  An array of <code>PolicySummary</code> objects.<br/>        """
        pass

    def put_notification_channel(self, SnsRoleName: str, SnsTopicArn: str):
        """Designates the IAM role and Amazon Simple Notification Service (SNS) topic that AWS Firewall Manager uses to record SNS logs.

        :param self:
        :param SnsRoleName:  The Amazon Resource Name (ARN) of the IAM role that allows Amazon SNS to record AWS Firewall Manager activity. <br/>SnsTopicArn:  The Amazon Resource Name (ARN) of the SNS topic that collects notifications from AWS Firewall Manager.<br/>
        :return:         """
        pass

    def put_policy(self, Policy: Policy) -> PutPolicyResponse:
        """Creates an AWS Firewall Manager policy. Firewall Manager provides two types of policies: A Shield Advanced policy, which applies Shield Advanced protection to specified accounts and resources, or a WAF policy, which contains a rule group and defines which resources are to be protected by that rule group. A policy is specific to either WAF or Shield Advanced. If you want to enforce both WAF rules and Shield Advanced protection across accounts, you can create multiple policies. You can create one or more policies for WAF rules, and one or more policies for Shield Advanced. You must be subscribed to Shield Advanced to create a Shield Advanced policy. For more information on subscribing to Shield Advanced, see <a href="https://docs.aws.amazon.com/waf/latest/DDOSAPIReference/API_CreateSubscription.html">CreateSubscription</a>.

        :param self:
        :param Policy:  The details of the AWS Firewall Manager policy to be created.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExcludeMap</i> Specifies the AWS account IDs to exclude from the policy. The <code>IncludeMap</code> values are evaluated first, with all the appropriate account IDs added to the policy. Then the accounts listed in <code>ExcludeMap</code> are removed, resulting in the final list of accounts to add to the policy. The key to the map is <code>ACCOUNT</code>. For example, a valid <code>ExcludeMap</code> would be <code>{“ACCOUNT” : [“accountID1”, “accountID2”]}</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeMap</i> Specifies the AWS account IDs to include in the policy. If <code>IncludeMap</code> is null, all accounts in the organization in AWS Organizations are included in the policy. If <code>IncludeMap</code> is not null, only values listed in <code>IncludeMap</code> are included in the policy. The key to the map is <code>ACCOUNT</code>. For example, a valid <code>IncludeMap</code> would be <code>{“ACCOUNT” : [“accountID1”, “accountID2”]}</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RemediationEnabled</i> Indicates if the policy should be automatically applied to new resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExcludeResourceTags</i> If set to <code>True</code>, resources with the tags that are specified in the <code>ResourceTag</code> array are not protected by the policy. If set to <code>False</code>, and the <code>ResourceTag</code> array is not null, only resources with the specified tags are associated with the policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceTags</i> An array of <code>ResourceTag</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceTypeList</i> An array of <code>ResourceType</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceType</i> The type of resource to protect with the policy. This is in the format shown in <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-template-resource-type-ref.html">AWS Resource Types Reference</a>. For example: <code>AWS::ElasticLoadBalancingV2::LoadBalancer</code> or <code>AWS::CloudFront::Distribution</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityServicePolicyData</i> Details about the security service that is being used to protect the resources.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SecurityServicePolicyData</b>: Details about the security service that is being used to protect the resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyUpdateToken</i> A unique identifier for each update to the policy. When issuing a <code>PutPolicy</code> request, the <code>PolicyUpdateToken</code> in the request must match the <code>PolicyUpdateToken</code> of the current policy version. To get the <code>PolicyUpdateToken</code> of the current policy version, use a <code>GetPolicy</code> request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyName</i> The friendly name of the AWS Firewall Manager policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyId</i> The ID of the AWS Firewall Manager policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ManagedServiceData</i> Details about the service. This contains <code>WAF</code> data in JSON format, as shown in the following example:  <code>ManagedServiceData": "{\"type\": \"WAF\", \"ruleGroups\": [{\"id\": \"12345678-1bcd-9012-efga-0987654321ab\", \"overrideAction\" : {\"type\": \"COUNT\"}}], \"defaultAction\": {\"type\": \"BLOCK\"}}</code>  If this is a Shield Advanced policy, this string will be empty.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The service that the policy is using to protect the resources. This specifies the type of policy that is created, either a WAF policy or Shield Advanced policy.<br/>
        :return: PolicyArn:  The Amazon Resource Name (ARN) of the policy that was created.<br/>Policy:  The details of the AWS Firewall Manager policy that was created.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExcludeMap</i> Specifies the AWS account IDs to exclude from the policy. The <code>IncludeMap</code> values are evaluated first, with all the appropriate account IDs added to the policy. Then the accounts listed in <code>ExcludeMap</code> are removed, resulting in the final list of accounts to add to the policy. The key to the map is <code>ACCOUNT</code>. For example, a valid <code>ExcludeMap</code> would be <code>{“ACCOUNT” : [“accountID1”, “accountID2”]}</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeMap</i> Specifies the AWS account IDs to include in the policy. If <code>IncludeMap</code> is null, all accounts in the organization in AWS Organizations are included in the policy. If <code>IncludeMap</code> is not null, only values listed in <code>IncludeMap</code> are included in the policy. The key to the map is <code>ACCOUNT</code>. For example, a valid <code>IncludeMap</code> would be <code>{“ACCOUNT” : [“accountID1”, “accountID2”]}</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RemediationEnabled</i> Indicates if the policy should be automatically applied to new resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExcludeResourceTags</i> If set to <code>True</code>, resources with the tags that are specified in the <code>ResourceTag</code> array are not protected by the policy. If set to <code>False</code>, and the <code>ResourceTag</code> array is not null, only resources with the specified tags are associated with the policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceTags</i> An array of <code>ResourceTag</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceTypeList</i> An array of <code>ResourceType</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceType</i> The type of resource to protect with the policy. This is in the format shown in <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-template-resource-type-ref.html">AWS Resource Types Reference</a>. For example: <code>AWS::ElasticLoadBalancingV2::LoadBalancer</code> or <code>AWS::CloudFront::Distribution</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityServicePolicyData</i> Details about the security service that is being used to protect the resources.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SecurityServicePolicyData</b>: Details about the security service that is being used to protect the resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyUpdateToken</i> A unique identifier for each update to the policy. When issuing a <code>PutPolicy</code> request, the <code>PolicyUpdateToken</code> in the request must match the <code>PolicyUpdateToken</code> of the current policy version. To get the <code>PolicyUpdateToken</code> of the current policy version, use a <code>GetPolicy</code> request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyName</i> The friendly name of the AWS Firewall Manager policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyId</i> The ID of the AWS Firewall Manager policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ManagedServiceData</i> Details about the service. This contains <code>WAF</code> data in JSON format, as shown in the following example:  <code>ManagedServiceData": "{\"type\": \"WAF\", \"ruleGroups\": [{\"id\": \"12345678-1bcd-9012-efga-0987654321ab\", \"overrideAction\" : {\"type\": \"COUNT\"}}], \"defaultAction\": {\"type\": \"BLOCK\"}}</code>  If this is a Shield Advanced policy, this string will be empty.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The service that the policy is using to protect the resources. This specifies the type of policy that is created, either a WAF policy or Shield Advanced policy.<br/>        """
        pass



class FSx(BaseClient):
    """Amazon FSx is a fully managed service that makes it easy for storage and application administrators to launch and use shared file storage."""


    class DescribeBackupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_backups` for valid parameters"""
            pass
    class DescribeFileSystemsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_file_systems` for valid parameters"""
            pass
    class ListTagsForResourcePaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_tags_for_resource` for valid parameters"""
            pass


    class CreateBackupRequest(Mapping):
        pass
    
    class CreateBackupResponse(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreateFileSystemRequest(Mapping):
        pass
    
    class CreateFileSystemResponse(Mapping):
        pass
    
    class SubnetId(object):
        pass
    
    class SecurityGroupId(object):
        pass
    
    class CreateFileSystemWindowsConfiguration(Mapping):
        pass
    
    class CreateFileSystemLustreConfiguration(Mapping):
        pass
    
    class CreateFileSystemFromBackupRequest(Mapping):
        pass
    
    class CreateFileSystemFromBackupResponse(Mapping):
        pass
    
    class DeleteBackupRequest(Mapping):
        pass
    
    class DeleteBackupResponse(Mapping):
        pass
    
    class DeleteFileSystemRequest(Mapping):
        pass
    
    class DeleteFileSystemResponse(Mapping):
        pass
    
    class DeleteFileSystemWindowsConfiguration(Mapping):
        pass
    
    class DescribeBackupsRequest(Mapping):
        pass
    
    class DescribeBackupsResponse(Mapping):
        pass
    
    class BackupId(object):
        pass
    
    class Filter(Mapping):
        pass
    
    class DescribeFileSystemsRequest(Mapping):
        pass
    
    class DescribeFileSystemsResponse(Mapping):
        pass
    
    class FileSystemId(object):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateFileSystemRequest(Mapping):
        pass
    
    class UpdateFileSystemResponse(Mapping):
        pass
    
    class UpdateFileSystemWindowsConfiguration(Mapping):
        pass
    
    class UpdateFileSystemLustreConfiguration(Mapping):
        pass
    
    def create_backup(self, FileSystemId: str, ClientRequestToken: str=None, Tags: List[Tag]=None) -> CreateBackupResponse:
        """Creates a backup of an existing Amazon FSx for Windows File Server file system. Creating regular backups for your file system is a best practice that complements the replication that Amazon FSx for Windows File Server performs for your file system. It also enables you to restore from user modification of data. If a backup with the specified client request token exists, and the parameters match, this operation returns the description of the existing backup. If a backup specified client request token exists, and the parameters don't match, this operation returns <code>IncompatibleParameterError</code>. If a backup with the specified client request token doesn't exist, <code>CreateBackup</code> does the following:  <ul> <li> Creates a new Amazon FSx backup with an assigned ID, and an initial lifecycle state of <code>CREATING</code>. </li> <li> Returns the description of the backup. </li> </ul> By using the idempotent operation, you can retry a <code>CreateBackup</code> operation without the risk of creating an extra backup. This approach can be useful when an initial call fails in a way that makes it unclear whether a backup was created. If you use the same client request token and the initial call created a backup, the operation returns a successful result because all the parameters are the same. The <code>CreateFileSystem</code> operation returns while the backup's lifecycle state is still <code>CREATING</code>. You can check the file system creation status by calling the <a>DescribeBackups</a> operation, which returns the backup state along with other information. <note> <p/> </note>

        :param self:
        :param Tags:  The tags to apply to the backup at backup creation. The key value of the <code>Name</code> tag appears in the console as the backup name.<br/>ClientRequestToken:  (Optional) A string of up to 64 ASCII characters that Amazon FSx uses to ensure idempotent creation. This string is automatically filled on your behalf when you use the AWS Command Line Interface (AWS CLI) or an AWS SDK.<br/>FileSystemId:  The ID of the file system to back up.<br/>
        :return: Backup:  A description of the backup.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DirectoryInformation</i> The configuration of the self-managed Microsoft Active Directory (AD) to which the Windows File Server instance is joined.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DirectoryInformation</b>: The configuration of the self-managed Microsoft Active Directory (AD) to which the Windows File Server instance is joined.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FileSystem</i> Metadata of the file system associated with the backup. This metadata is persisted even if the file system is deleted.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>FileSystem</b>: Metadata of the file system associated with the backup. This metadata is persisted even if the file system is deleted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> Tags associated with a particular file system.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceARN</i> The Amazon Resource Name (ARN) for the backup resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The ID of the AWS Key Management Service (AWS KMS) key used to encrypt this backup's data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The time when a particular backup was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProgressPercent</i> The current percent of progress of an asynchronous task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of the backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureDetails</i> Details explaining any failures that occur when creating a backup.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>FailureDetails</b>: Details explaining any failures that occur when creating a backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Lifecycle</i> The lifecycle status of the backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupId</i> The ID of the backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> A message describing the backup creation failure.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Lifecycle</i> The lifecycle status of the file system: <ul> <li>  <code>AVAILABLE</code> indicates that the file system is reachable and available for use. </li> <li>  <code>CREATING</code> indicates that Amazon FSx is in the process of creating the new file system. </li> <li>  <code>DELETING</code> indicates that Amazon FSx is in the process of deleting the file system. </li> <li>  <code>FAILED</code> indicates that Amazon FSx was not able to create the file system. </li> <li>  <code>MISCONFIGURED</code> indicates that the file system is in a failed but recoverable state. </li> <li>  <code>UPDATING</code> indicates that the file system is undergoing a customer initiated update. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FileSystemType</i> The type of Amazon FSx file system, either <code>LUSTRE</code> or <code>WINDOWS</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FileSystemId</i> The system-generated, unique 17-digit ID of the file system.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The time that the file system was created, in seconds (since 1970-01-01T00:00:00Z), also known as Unix time.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The AWS account that created the file system. If the file system was created by an AWS Identity and Access Management (IAM) user, the AWS account to which the IAM user belongs is the owner.<br/>A structure providing details of any failures that occur when creating the file system has failed.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>FailureDetails</b>: A structure providing details of any failures that occur when creating the file system has failed.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ActiveDirectoryId</i> The ID of the AWS Managed Microsoft Active Directory instance to which the file system is joined.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainName</i> The fully qualified domain name of the self-managed AD directory.<br/>        """
        pass

    def create_file_system(self, SubnetIds: List[SubnetId], StorageCapacity: int, FileSystemType: str, ClientRequestToken: str=None, SecurityGroupIds: List[SecurityGroupId]=None, Tags: List[Tag]=None, KmsKeyId: str=None, WindowsConfiguration: CreateFileSystemWindowsConfiguration=None, LustreConfiguration: CreateFileSystemLustreConfiguration=None) -> CreateFileSystemResponse:
        """Creates a new, empty Amazon FSx file system. If a file system with the specified client request token exists and the parameters match, <code>CreateFileSystem</code> returns the description of the existing file system. If a file system specified client request token exists and the parameters don't match, this call returns <code>IncompatibleParameterError</code>. If a file system with the specified client request token doesn't exist, <code>CreateFileSystem</code> does the following:  <ul> <li> Creates a new, empty Amazon FSx file system with an assigned ID, and an initial lifecycle state of <code>CREATING</code>. </li> <li> Returns the description of the file system. </li> </ul> This operation requires a client request token in the request that Amazon FSx uses to ensure idempotent creation. This means that calling the operation multiple times with the same client request token has no effect. By using the idempotent operation, you can retry a <code>CreateFileSystem</code> operation without the risk of creating an extra file system. This approach can be useful when an initial call fails in a way that makes it unclear whether a file system was created. Examples are if a transport level timeout occurred, or your connection was reset. If you use the same client request token and the initial call created a file system, the client receives success as long as the parameters are the same. <note> The <code>CreateFileSystem</code> call returns while the file system's lifecycle state is still <code>CREATING</code>. You can check the file-system creation status by calling the <a>DescribeFileSystems</a> operation, which returns the file system state along with other information. </note>

        :param self:
        :param LustreConfiguration:  The Lustre configuration for the file system being created. This value is required if <code>FileSystemType</code> is set to <code>LUSTRE</code>.:<br/>WindowsConfiguration:  The Microsoft Windows configuration for the file system being created. This value is required if <code>FileSystemType</code> is set to <code>WINDOWS</code>.:<br/>KmsKeyId:  The ID of your AWS Key Management Service (AWS KMS) key. This ID is used to encrypt the data in your file system at rest. For more information, see <a href="https://docs.aws.amazon.com/kms/latest/APIReference/API_Encrypt.html">Encrypt</a> in the <i>AWS Key Management Service API Reference</i>.<br/>Tags:  The tags to apply to the file system being created. The key value of the <code>Name</code> tag appears in the console as the file system name.<br/>SecurityGroupIds:  A list of IDs specifying the security groups to apply to all network interfaces created for file system access. This list isn't returned in later requests to describe the file system.<br/>SubnetIds:  The IDs of the subnets that the file system will be accessible from. File systems support only one subnet. The file server is also launched in that subnet's Availability Zone.<br/>StorageCapacity:  The storage capacity of the file system being created. For Windows file systems, the storage capacity has a minimum of 300 GiB, and a maximum of 65,536 GiB. For Lustre file systems, the storage capacity has a minimum of 3,600 GiB. Storage capacity is provisioned in increments of 3,600 GiB.<br/>FileSystemType:  The type of Amazon FSx file system to create.<br/>ClientRequestToken:  (Optional) A string of up to 64 ASCII characters that Amazon FSx uses to ensure idempotent creation. This string is automatically filled on your behalf when you use the AWS Command Line Interface (AWS CLI) or an AWS SDK.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyTagsToBackups</i> A boolean flag indicating whether tags for the file system should be copied to backups. This value defaults to false. If it's set to true, all tags for the file system are copied to all automatic and user-initiated backups where the user doesn't specify tags. If this value is true, and you specify one or more tags, only the specified tags are copied to backups.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomaticBackupRetentionDays</i> The number of days to retain automatic backups. The default is to retain backups for 7 days. Setting this value to 0 disables the creation of automatic backups. The maximum retention period for backups is 35 days.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DailyAutomaticBackupStartTime</i> The preferred time to take daily automatic backups, formatted HH:MM in the UTC time zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WeeklyMaintenanceStartTime</i> The preferred start time to perform weekly maintenance, formatted d:HH:MM in the UTC time zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThroughputCapacity</i> The throughput of an Amazon FSx file system, measured in megabytes per second, in 2 to the <i>n</i>th increments, between 2^3 (8) and 2^11 (2048).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SelfManagedActiveDirectoryConfiguration</i> The configuration that Amazon FSx uses to join the Windows File Server instance to your self-managed (including on-premises) Microsoft Active Directory (AD) directory.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SelfManagedActiveDirectoryConfiguration</b>: The configuration that Amazon FSx uses to join the Windows File Server instance to your self-managed (including on-premises) Microsoft Active Directory (AD) directory.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActiveDirectoryId</i> The ID for an existing AWS Managed Microsoft Active Directory (AD) instance that the file system should join when it's created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DnsIps</i> A list of up to two IP addresses of DNS servers or domain controllers in the self-managed AD directory. The IP addresses need to be either in the same VPC CIDR range as the one in which your Amazon FSx file system is being created, or in the private IP version 4 (Iv4) address ranges, as specified in <a href="http://www.faqs.org/rfcs/rfc1918.html">RFC 1918</a>: <ul> <li> 10.0.0.0 - 10.255.255.255 (10/8 prefix) </li> <li> 172.16.0.0 - 172.31.255.255 (172.16/12 prefix) </li> <li> 192.168.0.0 - 192.168.255.255 (192.168/16 prefix) </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Password</i> The password for the service account on your self-managed AD domain that Amazon FSx will use to join to your AD domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UserName</i> The user name for the service account on your self-managed AD domain that Amazon FSx will use to join to your AD domain. This account must have the permission to join computers to the domain in the organizational unit provided in <code>OrganizationalUnitDistinguishedName</code>, or in the default location of your AD domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FileSystemAdministratorsGroup</i> (Optional) The name of the domain group whose members are granted administrative privileges for the file system. Administrative privileges include taking ownership of files and folders, and setting audit controls (audit ACLs) on files and folders. The group that you specify must already exist in your domain. If you don't provide one, your AD domain's Domain Admins group is used.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationalUnitDistinguishedName</i> (Optional) The fully qualified distinguished name of the organizational unit within your self-managed AD directory that the Windows File Server instance will join. Amazon FSx only accepts OU as the direct parent of the file system. An example is <code>OU=FSx,DC=yourdomain,DC=corp,DC=com</code>. To learn more, see <a href="https://tools.ietf.org/html/rfc2253">RFC 2253</a>. If none is provided, the FSx file system is created in the default location of your self-managed AD directory.  <important> Only Organizational Unit (OU) objects can be the direct parent of the file system that you're creating. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainName</i> The fully qualified domain name of the self-managed AD directory, such as <code>corp.example.com</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImportedFileChunkSize</i> (Optional) For files imported from a data repository, this value determines the stripe count and maximum amount of data per file (in MiB) stored on a single physical disk. The maximum number of disks that a single file can be striped across is limited by the total number of disks that make up the file system. The chunk size default is 1,024 MiB (1 GiB) and can go as high as 512,000 MiB (500 GiB). Amazon S3 objects have a maximum size of 5 TB.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExportPath</i> (Optional) The path in Amazon S3 where the root of your Amazon FSx file system is exported. The path must use the same Amazon S3 bucket as specified in ImportPath. You can provide an optional prefix to which new and changed data is to be exported from your Amazon FSx for Lustre file system. If an <code>ExportPath</code> value is not provided, Amazon FSx sets a default export path, <code>s3://import-bucket/FSxLustre[creation-timestamp]</code>. The timestamp is in UTC format, for example <code>s3://import-bucket/FSxLustre20181105T222312Z</code>. The Amazon S3 export bucket must be the same as the import bucket specified by <code>ImportPath</code>. If you only specify a bucket name, such as <code>s3://import-bucket</code>, you get a 1:1 mapping of file system objects to S3 bucket objects. This mapping means that the input data in S3 is overwritten on export. If you provide a custom prefix in the export path, such as <code>s3://import-bucket/[custom-optional-prefix]</code>, Amazon FSx exports the contents of your file system to that export prefix in the Amazon S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImportPath</i> (Optional) The path to the Amazon S3 bucket (including the optional prefix) that you're using as the data repository for your Amazon FSx for Lustre file system. The root of your FSx for Lustre file system will be mapped to the root of the Amazon S3 bucket you select. An example is <code>s3://import-bucket/optional-prefix</code>. If you specify a prefix after the Amazon S3 bucket name, only object keys with that prefix are loaded into the file system.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WeeklyMaintenanceStartTime</i> The preferred time to perform weekly maintenance, in the UTC time zone.<br/>
        :return: FileSystem:  The configuration of the file system that was created.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LustreConfiguration</i> The configuration for the Amazon FSx for Lustre file system.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LustreConfiguration</b>: The configuration for the Amazon FSx for Lustre file system.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WindowsConfiguration</i> The configuration for this Microsoft Windows file system.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>WindowsConfiguration</b>: The configuration for this Microsoft Windows file system.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The tags to associate with the file system. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Using_Tags.html">Tagging Your Amazon EC2 Resources</a> in the <i>Amazon EC2 User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceARN</i> The Amazon Resource Name (ARN) for the file system resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The ID of the AWS Key Management Service (AWS KMS) key used to encrypt the file system's data for an Amazon FSx for Windows File Server file system.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DNSName</i> The DNS name for the file system.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkInterfaceIds</i> The IDs of the elastic network interface from which a specific file system is accessible. The elastic network interface is automatically created in the same VPC that the Amazon FSx file system was created in. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/using-eni.html">Elastic Network Interfaces</a> in the <i>Amazon EC2 User Guide.</i>  For an Amazon FSx for Windows File Server file system, you can have one network interface ID. For an Amazon FSx for Lustre file system, you can have more than one.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> The ID of the subnet to contain the endpoint for the file system. One and only one is supported. The file system is launched in the Availability Zone associated with this subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the primary VPC for the file system.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageCapacity</i> The storage capacity of the file system in gigabytes (GB).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureDetails</i> A structure providing details of any failures that occur when creating the file system has failed.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>FailureDetails</b>: A structure providing details of any failures that occur when creating the file system has failed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Lifecycle</i> The lifecycle status of the file system: <ul> <li>  <code>AVAILABLE</code> indicates that the file system is reachable and available for use. </li> <li>  <code>CREATING</code> indicates that Amazon FSx is in the process of creating the new file system. </li> <li>  <code>DELETING</code> indicates that Amazon FSx is in the process of deleting the file system. </li> <li>  <code>FAILED</code> indicates that Amazon FSx was not able to create the file system. </li> <li>  <code>MISCONFIGURED</code> indicates that the file system is in a failed but recoverable state. </li> <li>  <code>UPDATING</code> indicates that the file system is undergoing a customer initiated update. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FileSystemType</i> The type of Amazon FSx file system, either <code>LUSTRE</code> or <code>WINDOWS</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FileSystemId</i> The system-generated, unique 17-digit ID of the file system.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The time that the file system was created, in seconds (since 1970-01-01T00:00:00Z), also known as Unix time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The AWS account that created the file system. If the file system was created by an AWS Identity and Access Management (IAM) user, the AWS account to which the IAM user belongs is the owner.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> A message describing any failures that occurred during file system creation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ActiveDirectoryId</i> The ID for an existing Microsoft Active Directory instance that the file system should join when it's created.<br/>The configuration of the self-managed Microsoft Active Directory (AD) directory to which the Windows File Server instance is joined.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SelfManagedActiveDirectoryConfiguration</b>: The configuration of the self-managed Microsoft Active Directory (AD) directory to which the Windows File Server instance is joined.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WeeklyMaintenanceStartTime</i> The UTC time that you want to begin your weekly maintenance window.<br/>The data repository configuration object for Lustre file systems returned in the response of the <code>CreateFileSystem</code> operation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DataRepositoryConfiguration</b>: The data repository configuration object for Lustre file systems returned in the response of the <code>CreateFileSystem</code> operation.        """
        pass

    def create_file_system_from_backup(self, SubnetIds: List[SubnetId], BackupId: str, ClientRequestToken: str=None, SecurityGroupIds: List[SecurityGroupId]=None, Tags: List[Tag]=None, WindowsConfiguration: CreateFileSystemWindowsConfiguration=None) -> CreateFileSystemFromBackupResponse:
        """Creates a new Amazon FSx file system from an existing Amazon FSx for Windows File Server backup. If a file system with the specified client request token exists and the parameters match, this operation returns the description of the file system. If a client request token specified by the file system exists and the parameters don't match, this call returns <code>IncompatibleParameterError</code>. If a file system with the specified client request token doesn't exist, this operation does the following: <ul> <li> Creates a new Amazon FSx file system from backup with an assigned ID, and an initial lifecycle state of <code>CREATING</code>. </li> <li> Returns the description of the file system. </li> </ul> Parameters like Active Directory, default share name, automatic backup, and backup settings default to the parameters of the file system that was backed up, unless overridden. You can explicitly supply other settings. By using the idempotent operation, you can retry a <code>CreateFileSystemFromBackup</code> call without the risk of creating an extra file system. This approach can be useful when an initial call fails in a way that makes it unclear whether a file system was created. Examples are if a transport level timeout occurred, or your connection was reset. If you use the same client request token and the initial call created a file system, the client receives success as long as the parameters are the same. <note> The <code>CreateFileSystemFromBackup</code> call returns while the file system's lifecycle state is still <code>CREATING</code>. You can check the file-system creation status by calling the <a>DescribeFileSystems</a> operation, which returns the file system state along with other information. </note>

        :param self:
        :param WindowsConfiguration:  The configuration for this Microsoft Windows file system.:<br/>Tags:  The tags to be applied to the file system at file system creation. The key value of the <code>Name</code> tag appears in the console as the file system name.<br/>SecurityGroupIds:  A list of IDs for the security groups that apply to the specified network interfaces created for file system access. These security groups apply to all network interfaces. This value isn't returned in later describe requests.<br/>SubnetIds:  A list of IDs for the subnets that the file system will be accessible from. Currently, you can specify only one subnet. The file server is also launched in that subnet's Availability Zone.<br/>ClientRequestToken:  (Optional) A string of up to 64 ASCII characters that Amazon FSx uses to ensure idempotent creation. This string is automatically filled on your behalf when you use the AWS Command Line Interface (AWS CLI) or an AWS SDK.<br/>BackupId:  The ID of the backup. Specifies the backup to use if you're creating a file system from an existing backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyTagsToBackups</i> A boolean flag indicating whether tags for the file system should be copied to backups. This value defaults to false. If it's set to true, all tags for the file system are copied to all automatic and user-initiated backups where the user doesn't specify tags. If this value is true, and you specify one or more tags, only the specified tags are copied to backups.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomaticBackupRetentionDays</i> The number of days to retain automatic backups. The default is to retain backups for 7 days. Setting this value to 0 disables the creation of automatic backups. The maximum retention period for backups is 35 days.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DailyAutomaticBackupStartTime</i> The preferred time to take daily automatic backups, formatted HH:MM in the UTC time zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WeeklyMaintenanceStartTime</i> The preferred start time to perform weekly maintenance, formatted d:HH:MM in the UTC time zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThroughputCapacity</i> The throughput of an Amazon FSx file system, measured in megabytes per second, in 2 to the <i>n</i>th increments, between 2^3 (8) and 2^11 (2048).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SelfManagedActiveDirectoryConfiguration</i> The configuration that Amazon FSx uses to join the Windows File Server instance to your self-managed (including on-premises) Microsoft Active Directory (AD) directory.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SelfManagedActiveDirectoryConfiguration</b>: The configuration that Amazon FSx uses to join the Windows File Server instance to your self-managed (including on-premises) Microsoft Active Directory (AD) directory.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActiveDirectoryId</i> The ID for an existing AWS Managed Microsoft Active Directory (AD) instance that the file system should join when it's created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DnsIps</i> A list of up to two IP addresses of DNS servers or domain controllers in the self-managed AD directory. The IP addresses need to be either in the same VPC CIDR range as the one in which your Amazon FSx file system is being created, or in the private IP version 4 (Iv4) address ranges, as specified in <a href="http://www.faqs.org/rfcs/rfc1918.html">RFC 1918</a>: <ul> <li> 10.0.0.0 - 10.255.255.255 (10/8 prefix) </li> <li> 172.16.0.0 - 172.31.255.255 (172.16/12 prefix) </li> <li> 192.168.0.0 - 192.168.255.255 (192.168/16 prefix) </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Password</i> The password for the service account on your self-managed AD domain that Amazon FSx will use to join to your AD domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UserName</i> The user name for the service account on your self-managed AD domain that Amazon FSx will use to join to your AD domain. This account must have the permission to join computers to the domain in the organizational unit provided in <code>OrganizationalUnitDistinguishedName</code>, or in the default location of your AD domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FileSystemAdministratorsGroup</i> (Optional) The name of the domain group whose members are granted administrative privileges for the file system. Administrative privileges include taking ownership of files and folders, and setting audit controls (audit ACLs) on files and folders. The group that you specify must already exist in your domain. If you don't provide one, your AD domain's Domain Admins group is used.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationalUnitDistinguishedName</i> (Optional) The fully qualified distinguished name of the organizational unit within your self-managed AD directory that the Windows File Server instance will join. Amazon FSx only accepts OU as the direct parent of the file system. An example is <code>OU=FSx,DC=yourdomain,DC=corp,DC=com</code>. To learn more, see <a href="https://tools.ietf.org/html/rfc2253">RFC 2253</a>. If none is provided, the FSx file system is created in the default location of your self-managed AD directory.  <important> Only Organizational Unit (OU) objects can be the direct parent of the file system that you're creating. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainName</i> The fully qualified domain name of the self-managed AD directory, such as <code>corp.example.com</code>.<br/>
        :return: FileSystem:  A description of the file system.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LustreConfiguration</i> The configuration for the Amazon FSx for Lustre file system.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LustreConfiguration</b>: The configuration for the Amazon FSx for Lustre file system.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WindowsConfiguration</i> The configuration for this Microsoft Windows file system.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>WindowsConfiguration</b>: The configuration for this Microsoft Windows file system.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The tags to associate with the file system. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Using_Tags.html">Tagging Your Amazon EC2 Resources</a> in the <i>Amazon EC2 User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceARN</i> The Amazon Resource Name (ARN) for the file system resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The ID of the AWS Key Management Service (AWS KMS) key used to encrypt the file system's data for an Amazon FSx for Windows File Server file system.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DNSName</i> The DNS name for the file system.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkInterfaceIds</i> The IDs of the elastic network interface from which a specific file system is accessible. The elastic network interface is automatically created in the same VPC that the Amazon FSx file system was created in. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/using-eni.html">Elastic Network Interfaces</a> in the <i>Amazon EC2 User Guide.</i>  For an Amazon FSx for Windows File Server file system, you can have one network interface ID. For an Amazon FSx for Lustre file system, you can have more than one.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> The ID of the subnet to contain the endpoint for the file system. One and only one is supported. The file system is launched in the Availability Zone associated with this subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the primary VPC for the file system.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageCapacity</i> The storage capacity of the file system in gigabytes (GB).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureDetails</i> A structure providing details of any failures that occur when creating the file system has failed.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>FailureDetails</b>: A structure providing details of any failures that occur when creating the file system has failed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Lifecycle</i> The lifecycle status of the file system: <ul> <li>  <code>AVAILABLE</code> indicates that the file system is reachable and available for use. </li> <li>  <code>CREATING</code> indicates that Amazon FSx is in the process of creating the new file system. </li> <li>  <code>DELETING</code> indicates that Amazon FSx is in the process of deleting the file system. </li> <li>  <code>FAILED</code> indicates that Amazon FSx was not able to create the file system. </li> <li>  <code>MISCONFIGURED</code> indicates that the file system is in a failed but recoverable state. </li> <li>  <code>UPDATING</code> indicates that the file system is undergoing a customer initiated update. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FileSystemType</i> The type of Amazon FSx file system, either <code>LUSTRE</code> or <code>WINDOWS</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FileSystemId</i> The system-generated, unique 17-digit ID of the file system.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The time that the file system was created, in seconds (since 1970-01-01T00:00:00Z), also known as Unix time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The AWS account that created the file system. If the file system was created by an AWS Identity and Access Management (IAM) user, the AWS account to which the IAM user belongs is the owner.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> A message describing any failures that occurred during file system creation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ActiveDirectoryId</i> The ID for an existing Microsoft Active Directory instance that the file system should join when it's created.<br/>The configuration of the self-managed Microsoft Active Directory (AD) directory to which the Windows File Server instance is joined.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SelfManagedActiveDirectoryConfiguration</b>: The configuration of the self-managed Microsoft Active Directory (AD) directory to which the Windows File Server instance is joined.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WeeklyMaintenanceStartTime</i> The UTC time that you want to begin your weekly maintenance window.<br/>The data repository configuration object for Lustre file systems returned in the response of the <code>CreateFileSystem</code> operation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DataRepositoryConfiguration</b>: The data repository configuration object for Lustre file systems returned in the response of the <code>CreateFileSystem</code> operation.        """
        pass

    def delete_backup(self, BackupId: str, ClientRequestToken: str=None) -> DeleteBackupResponse:
        """Deletes an Amazon FSx for Windows File Server backup, deleting its contents. After deletion, the backup no longer exists, and its data is gone. The <code>DeleteBackup</code> call returns instantly. The backup will not show up in later <code>DescribeBackups</code> calls. <important> The data in a deleted backup is also deleted and can't be recovered by any means. </important>

        :param self:
        :param ClientRequestToken:  (Optional) A string of up to 64 ASCII characters that Amazon FSx uses to ensure idempotent deletion. This is automatically filled on your behalf when using the AWS CLI or SDK.<br/>BackupId:  The ID of the backup you want to delete.<br/>
        :return: Lifecycle:  The lifecycle of the backup. Should be <code>DELETED</code>.<br/>BackupId:  The ID of the backup deleted.<br/>        """
        pass

    def delete_file_system(self, FileSystemId: str, ClientRequestToken: str=None, WindowsConfiguration: DeleteFileSystemWindowsConfiguration=None) -> DeleteFileSystemResponse:
        """Deletes a file system, deleting its contents. After deletion, the file system no longer exists, and its data is gone. Any existing automatic backups will also be deleted. By default, when you delete an Amazon FSx for Windows File Server file system, a final backup is created upon deletion. This final backup is not subject to the file system's retention policy, and must be manually deleted. The <code>DeleteFileSystem</code> action returns while the file system has the <code>DELETING</code> status. You can check the file system deletion status by calling the <a>DescribeFileSystems</a> action, which returns a list of file systems in your account. If you pass the file system ID for a deleted file system, the <a>DescribeFileSystems</a> returns a <code>FileSystemNotFound</code> error. <important> The data in a deleted file system is also deleted and can't be recovered by any means. </important>

        :param self:
        :param WindowsConfiguration:  The configuration object for the Microsoft Windows file system used in the <code>DeleteFileSystem</code> operation.:<br/>ClientRequestToken:  (Optional) A string of up to 64 ASCII characters that Amazon FSx uses to ensure idempotent deletion. This is automatically filled on your behalf when using the AWS CLI or SDK.<br/>FileSystemId:  The ID of the file system you want to delete.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FinalBackupTags</i> A set of tags for your final backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SkipFinalBackup</i> By default, Amazon FSx for Windows takes a final backup on your behalf when the <code>DeleteFileSystem</code> operation is invoked. Doing this helps protect you from data loss, and we highly recommend taking the final backup. If you want to skip this backup, use this flag to do so.<br/>
        :return: WindowsResponse:  The response object for the Microsoft Windows file system used in the <code>DeleteFileSystem</code> operation.:<br/>Lifecycle:  The file system lifecycle for the deletion request. Should be <code>DELETING</code>.<br/>FileSystemId:  The ID of the file system being deleted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FinalBackupTags</i> The set of tags applied to the final backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FinalBackupId</i> The ID of the final backup for this file system.<br/>        """
        pass

    def describe_backups(self, BackupIds: List[BackupId]=None, Filters: List[Filter]=None, MaxResults: int=None, NextToken: str=None) -> DescribeBackupsResponse:
        """Returns the description of specific Amazon FSx for Windows File Server backups, if a <code>BackupIds</code> value is provided for that backup. Otherwise, it returns all backups owned by your AWS account in the AWS Region of the endpoint that you're calling. When retrieving all backups, you can optionally specify the <code>MaxResults</code> parameter to limit the number of backups in a response. If more backups remain, Amazon FSx returns a <code>NextToken</code> value in the response. In this case, send a later request with the <code>NextToken</code> request parameter set to the value of <code>NextToken</code> from the last response. This action is used in an iterative process to retrieve a list of your backups. <code>DescribeBackups</code> is called first without a <code>NextToken</code>value. Then the action continues to be called with the <code>NextToken</code> parameter set to the value of the last <code>NextToken</code> value until a response has no <code>NextToken</code>. When using this action, keep the following in mind: <ul> <li> The implementation might return fewer than <code>MaxResults</code> file system descriptions while still including a <code>NextToken</code> value. </li> <li> The order of backups returned in the response of one <code>DescribeBackups</code> call and the order of backups returned across the responses of a multi-call iteration is unspecified. </li> </ul>

        :param self:
        :param NextToken:  (Optional) Opaque pagination token returned from a previous <code>DescribeBackups</code> operation (String). If a token present, the action continues the list from where the returning call left off.<br/>MaxResults:  (Optional) Maximum number of backups to return in the response (integer). This parameter value must be greater than 0. The number of items that Amazon FSx returns is the minimum of the <code>MaxResults</code> parameter specified in the request and the service's internal maximum number of items per page.<br/>Filters:  (Optional) Filters structure. Supported names are file-system-id and backup-type.<br/>BackupIds:  (Optional) IDs of the backups you want to retrieve (String). This overrides any filters. If any IDs are not found, BackupNotFound will be thrown.<br/>
        :return: NextToken:  This is present if there are more backups than returned in the response (String). You can use the <code>NextToken</code> value in the later request to fetch the backups. <br/>Backups:  Any array of backups.<br/>        """
        pass

    def describe_file_systems(self, FileSystemIds: List[FileSystemId]=None, MaxResults: int=None, NextToken: str=None) -> DescribeFileSystemsResponse:
        """Returns the description of specific Amazon FSx file systems, if a <code>FileSystemIds</code> value is provided for that file system. Otherwise, it returns descriptions of all file systems owned by your AWS account in the AWS Region of the endpoint that you're calling. When retrieving all file system descriptions, you can optionally specify the <code>MaxResults</code> parameter to limit the number of descriptions in a response. If more file system descriptions remain, Amazon FSx returns a <code>NextToken</code> value in the response. In this case, send a later request with the <code>NextToken</code> request parameter set to the value of <code>NextToken</code> from the last response. This action is used in an iterative process to retrieve a list of your file system descriptions. <code>DescribeFileSystems</code> is called first without a <code>NextToken</code>value. Then the action continues to be called with the <code>NextToken</code> parameter set to the value of the last <code>NextToken</code> value until a response has no <code>NextToken</code>. When using this action, keep the following in mind: <ul> <li> The implementation might return fewer than <code>MaxResults</code> file system descriptions while still including a <code>NextToken</code> value. </li> <li> The order of file systems returned in the response of one <code>DescribeFileSystems</code> call and the order of file systems returned across the responses of a multicall iteration is unspecified. </li> </ul>

        :param self:
        :param NextToken:  (Optional) Opaque pagination token returned from a previous <code>DescribeFileSystems</code> operation (String). If a token present, the action continues the list from where the returning call left off.<br/>MaxResults:  (Optional) Maximum number of file systems to return in the response (integer). This parameter value must be greater than 0. The number of items that Amazon FSx returns is the minimum of the <code>MaxResults</code> parameter specified in the request and the service's internal maximum number of items per page.<br/>FileSystemIds:  (Optional) IDs of the file systems whose descriptions you want to retrieve (String).<br/>
        :return: NextToken:  Present if there are more file systems than returned in the response (String). You can use the <code>NextToken</code> value in the later request to fetch the descriptions. <br/>FileSystems:  An array of file system descriptions.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceARN: str, MaxResults: int=None, NextToken: str=None) -> ListTagsForResourceResponse:
        """Lists tags for an Amazon FSx file systems and backups in the case of Amazon FSx for Windows File Server. When retrieving all tags, you can optionally specify the <code>MaxResults</code> parameter to limit the number of tags in a response. If more tags remain, Amazon FSx returns a <code>NextToken</code> value in the response. In this case, send a later request with the <code>NextToken</code> request parameter set to the value of <code>NextToken</code> from the last response. This action is used in an iterative process to retrieve a list of your tags. <code>ListTagsForResource</code> is called first without a <code>NextToken</code>value. Then the action continues to be called with the <code>NextToken</code> parameter set to the value of the last <code>NextToken</code> value until a response has no <code>NextToken</code>. When using this action, keep the following in mind: <ul> <li> The implementation might return fewer than <code>MaxResults</code> file system descriptions while still including a <code>NextToken</code> value. </li> <li> The order of tags returned in the response of one <code>ListTagsForResource</code> call and the order of tags returned across the responses of a multi-call iteration is unspecified. </li> </ul>

        :param self:
        :param NextToken:  (Optional) Opaque pagination token returned from a previous <code>ListTagsForResource</code> operation (String). If a token present, the action continues the list from where the returning call left off.<br/>MaxResults:  (Optional) Maximum number of tags to return in the response (integer). This parameter value must be greater than 0. The number of items that Amazon FSx returns is the minimum of the <code>MaxResults</code> parameter specified in the request and the service's internal maximum number of items per page.<br/>ResourceARN:  The ARN of the Amazon FSx resource that will have its tags listed.<br/>
        :return: NextToken:  This is present if there are more tags than returned in the response (String). You can use the <code>NextToken</code> value in the later request to fetch the tags. <br/>Tags:  A list of tags on the resource.<br/>        """
        pass

    def tag_resource(self, Tags: List[Tag], ResourceARN: str) -> TagResourceResponse:
        """Tags an Amazon FSx resource.

        :param self:
        :param Tags:  A list of tags for the resource. If a tag with a given key already exists, the value is replaced by the one specified in this parameter.<br/>ResourceARN:  The Amazon Resource Name (ARN) of the Amazon FSx resource that you want to tag.<br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[TagKey], ResourceARN: str) -> UntagResourceResponse:
        """This action removes a tag from an Amazon FSx resource.

        :param self:
        :param TagKeys:  A list of keys of tags on the resource to untag. In case the tag key doesn't exist, the call will still succeed to be idempotent.<br/>ResourceARN:  The ARN of the Amazon FSx resource to untag.<br/>
        :return:         """
        pass

    def update_file_system(self, FileSystemId: str, ClientRequestToken: str=None, WindowsConfiguration: UpdateFileSystemWindowsConfiguration=None, LustreConfiguration: UpdateFileSystemLustreConfiguration=None) -> UpdateFileSystemResponse:
        """Updates a file system configuration.

        :param self:
        :param LustreConfiguration:  The configuration object for Amazon FSx for Lustre file systems used in the <code>UpdateFileSystem</code> operation.:<br/>WindowsConfiguration:  The configuration update for this Microsoft Windows file system. The only supported options are for backup and maintenance and for self-managed Active Directory configuration.:<br/>ClientRequestToken:  (Optional) A string of up to 64 ASCII characters that Amazon FSx uses to ensure idempotent updates. This string is automatically filled on your behalf when you use the AWS Command Line Interface (AWS CLI) or an AWS SDK.<br/>FileSystemId:  The globally unique ID of the file system, assigned by Amazon FSx.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SelfManagedActiveDirectoryConfiguration</i> The configuration Amazon FSx uses to join the Windows File Server instance to the self-managed Microsoft AD directory.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SelfManagedActiveDirectoryConfiguration</b>: The configuration Amazon FSx uses to join the Windows File Server instance to the self-managed Microsoft AD directory.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomaticBackupRetentionDays</i> The number of days to retain automatic backups. Setting this to 0 disables automatic backups. You can retain automatic backups for a maximum of 35 days.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DailyAutomaticBackupStartTime</i> The preferred time to take daily automatic backups, in the UTC time zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WeeklyMaintenanceStartTime</i> The preferred time to perform weekly maintenance, in the UTC time zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DnsIps</i> A list of up to two IP addresses of DNS servers or domain controllers in the self-managed AD directory.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Password</i> The password for the service account on your self-managed AD domain that Amazon FSx will use to join to your AD domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UserName</i> The user name for the service account on your self-managed AD domain that Amazon FSx will use to join to your AD domain. This account must have the permission to join computers to the domain in the organizational unit provided in <code>OrganizationalUnitDistinguishedName</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WeeklyMaintenanceStartTime</i> The preferred time to perform weekly maintenance, in the UTC time zone.<br/>
        :return: FileSystem:  A description of the file system that was updated.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LustreConfiguration</i> The configuration for the Amazon FSx for Lustre file system.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LustreConfiguration</b>: The configuration for the Amazon FSx for Lustre file system.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WindowsConfiguration</i> The configuration for this Microsoft Windows file system.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>WindowsConfiguration</b>: The configuration for this Microsoft Windows file system.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The tags to associate with the file system. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Using_Tags.html">Tagging Your Amazon EC2 Resources</a> in the <i>Amazon EC2 User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceARN</i> The Amazon Resource Name (ARN) for the file system resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The ID of the AWS Key Management Service (AWS KMS) key used to encrypt the file system's data for an Amazon FSx for Windows File Server file system.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DNSName</i> The DNS name for the file system.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkInterfaceIds</i> The IDs of the elastic network interface from which a specific file system is accessible. The elastic network interface is automatically created in the same VPC that the Amazon FSx file system was created in. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/using-eni.html">Elastic Network Interfaces</a> in the <i>Amazon EC2 User Guide.</i>  For an Amazon FSx for Windows File Server file system, you can have one network interface ID. For an Amazon FSx for Lustre file system, you can have more than one.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> The ID of the subnet to contain the endpoint for the file system. One and only one is supported. The file system is launched in the Availability Zone associated with this subnet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the primary VPC for the file system.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageCapacity</i> The storage capacity of the file system in gigabytes (GB).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureDetails</i> A structure providing details of any failures that occur when creating the file system has failed.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>FailureDetails</b>: A structure providing details of any failures that occur when creating the file system has failed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Lifecycle</i> The lifecycle status of the file system: <ul> <li>  <code>AVAILABLE</code> indicates that the file system is reachable and available for use. </li> <li>  <code>CREATING</code> indicates that Amazon FSx is in the process of creating the new file system. </li> <li>  <code>DELETING</code> indicates that Amazon FSx is in the process of deleting the file system. </li> <li>  <code>FAILED</code> indicates that Amazon FSx was not able to create the file system. </li> <li>  <code>MISCONFIGURED</code> indicates that the file system is in a failed but recoverable state. </li> <li>  <code>UPDATING</code> indicates that the file system is undergoing a customer initiated update. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FileSystemType</i> The type of Amazon FSx file system, either <code>LUSTRE</code> or <code>WINDOWS</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FileSystemId</i> The system-generated, unique 17-digit ID of the file system.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The time that the file system was created, in seconds (since 1970-01-01T00:00:00Z), also known as Unix time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The AWS account that created the file system. If the file system was created by an AWS Identity and Access Management (IAM) user, the AWS account to which the IAM user belongs is the owner.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> A message describing any failures that occurred during file system creation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ActiveDirectoryId</i> The ID for an existing Microsoft Active Directory instance that the file system should join when it's created.<br/>The configuration of the self-managed Microsoft Active Directory (AD) directory to which the Windows File Server instance is joined.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SelfManagedActiveDirectoryConfiguration</b>: The configuration of the self-managed Microsoft Active Directory (AD) directory to which the Windows File Server instance is joined.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WeeklyMaintenanceStartTime</i> The UTC time that you want to begin your weekly maintenance window.<br/>The data repository configuration object for Lustre file systems returned in the response of the <code>CreateFileSystem</code> operation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DataRepositoryConfiguration</b>: The data repository configuration object for Lustre file systems returned in the response of the <code>CreateFileSystem</code> operation.        """
        pass



class GameLift(BaseClient):
    """<fullname>Amazon GameLift Service</fullname>  Amazon GameLift is a managed service for developers who need a scalable, dedicated server solution for their multiplayer games. Use Amazon GameLift for these tasks: (1) set up computing resources and deploy your game servers, (2) run game sessions and get players into games, (3) automatically scale your resources to meet player demand and manage costs, and (4) track in-depth metrics on game server performance and player usage. When setting up hosting resources, you can deploy your custom game server or use the Amazon GameLift Realtime Servers. Realtime Servers gives you the ability to quickly stand up lightweight, efficient game servers with the core Amazon GameLift infrastructure already built in.  <b>Get Amazon GameLift Tools and Resources</b>  This reference guide describes the low-level service API for Amazon GameLift and provides links to language-specific SDK reference topics. See also <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-components.html"> Amazon GameLift Tools and Resources</a>.  <b>API Summary</b>  The Amazon GameLift service API includes two key sets of actions: <ul> <li> Manage game sessions and player access -- Integrate this functionality into game client services in order to create new game sessions, retrieve information on existing game sessions; reserve a player slot in a game session, request matchmaking, etc. </li> <li> Configure and manage game server resources -- Manage your Amazon GameLift hosting resources, including builds, scripts, fleets, queues, and aliases. Set up matchmakers, configure auto-scaling, retrieve game logs, and get hosting and game metrics. </li> </ul>  <b> <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/reference-awssdk.html"> Task-based list of API actions</a> </b> """


    class DescribeFleetAttributesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_fleet_attributes` for valid parameters"""
            pass
    class DescribeFleetCapacityPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_fleet_capacity` for valid parameters"""
            pass
    class DescribeFleetEventsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_fleet_events` for valid parameters"""
            pass
    class DescribeFleetUtilizationPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_fleet_utilization` for valid parameters"""
            pass
    class DescribeGameSessionDetailsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_game_session_details` for valid parameters"""
            pass
    class DescribeGameSessionQueuesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_game_session_queues` for valid parameters"""
            pass
    class DescribeGameSessionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_game_sessions` for valid parameters"""
            pass
    class DescribeInstancesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_instances` for valid parameters"""
            pass
    class DescribeMatchmakingConfigurationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_matchmaking_configurations` for valid parameters"""
            pass
    class DescribeMatchmakingRuleSetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_matchmaking_rule_sets` for valid parameters"""
            pass
    class DescribePlayerSessionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_player_sessions` for valid parameters"""
            pass
    class DescribeScalingPoliciesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_scaling_policies` for valid parameters"""
            pass
    class ListAliasesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_aliases` for valid parameters"""
            pass
    class ListBuildsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_builds` for valid parameters"""
            pass
    class ListFleetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_fleets` for valid parameters"""
            pass
    class SearchGameSessionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `search_game_sessions` for valid parameters"""
            pass


    class AcceptMatchInput(Mapping):
        pass
    
    class AcceptMatchOutput(Mapping):
        pass
    
    class NonZeroAndMaxString(object):
        pass
    
    class CreateAliasInput(Mapping):
        pass
    
    class CreateAliasOutput(Mapping):
        pass
    
    class RoutingStrategy(Mapping):
        pass
    
    class CreateBuildInput(Mapping):
        pass
    
    class CreateBuildOutput(Mapping):
        pass
    
    class S3Location(Mapping):
        pass
    
    class CreateFleetInput(Mapping):
        pass
    
    class CreateFleetOutput(Mapping):
        pass
    
    class IpPermission(Mapping):
        pass
    
    class RuntimeConfiguration(Mapping):
        pass
    
    class ResourceCreationLimitPolicy(Mapping):
        pass
    
    class MetricGroup(object):
        pass
    
    class CreateGameSessionInput(Mapping):
        pass
    
    class CreateGameSessionOutput(Mapping):
        pass
    
    class GameProperty(Mapping):
        pass
    
    class CreateGameSessionQueueInput(Mapping):
        pass
    
    class CreateGameSessionQueueOutput(Mapping):
        pass
    
    class PlayerLatencyPolicy(Mapping):
        pass
    
    class GameSessionQueueDestination(Mapping):
        pass
    
    class CreateMatchmakingConfigurationInput(Mapping):
        pass
    
    class CreateMatchmakingConfigurationOutput(Mapping):
        pass
    
    class ArnStringModel(object):
        pass
    
    class BooleanModel(object):
        pass
    
    class CreateMatchmakingRuleSetInput(Mapping):
        pass
    
    class CreateMatchmakingRuleSetOutput(Mapping):
        pass
    
    class CreatePlayerSessionInput(Mapping):
        pass
    
    class CreatePlayerSessionOutput(Mapping):
        pass
    
    class CreatePlayerSessionsInput(Mapping):
        pass
    
    class CreatePlayerSessionsOutput(Mapping):
        pass
    
    class PlayerDataMap(object):
        pass
    
    class CreateScriptInput(Mapping):
        pass
    
    class CreateScriptOutput(Mapping):
        pass
    
    class ZipBlob(object):
        pass
    
    class CreateVpcPeeringAuthorizationInput(Mapping):
        pass
    
    class CreateVpcPeeringAuthorizationOutput(Mapping):
        pass
    
    class CreateVpcPeeringConnectionInput(Mapping):
        pass
    
    class CreateVpcPeeringConnectionOutput(Mapping):
        pass
    
    class DeleteAliasInput(Mapping):
        pass
    
    class DeleteBuildInput(Mapping):
        pass
    
    class DeleteFleetInput(Mapping):
        pass
    
    class DeleteGameSessionQueueInput(Mapping):
        pass
    
    class DeleteGameSessionQueueOutput(Mapping):
        pass
    
    class DeleteMatchmakingConfigurationInput(Mapping):
        pass
    
    class DeleteMatchmakingConfigurationOutput(Mapping):
        pass
    
    class DeleteMatchmakingRuleSetInput(Mapping):
        pass
    
    class DeleteMatchmakingRuleSetOutput(Mapping):
        pass
    
    class DeleteScalingPolicyInput(Mapping):
        pass
    
    class DeleteScriptInput(Mapping):
        pass
    
    class DeleteVpcPeeringAuthorizationInput(Mapping):
        pass
    
    class DeleteVpcPeeringAuthorizationOutput(Mapping):
        pass
    
    class DeleteVpcPeeringConnectionInput(Mapping):
        pass
    
    class DeleteVpcPeeringConnectionOutput(Mapping):
        pass
    
    class DescribeAliasInput(Mapping):
        pass
    
    class DescribeAliasOutput(Mapping):
        pass
    
    class DescribeBuildInput(Mapping):
        pass
    
    class DescribeBuildOutput(Mapping):
        pass
    
    class DescribeEC2InstanceLimitsInput(Mapping):
        pass
    
    class DescribeEC2InstanceLimitsOutput(Mapping):
        pass
    
    class DescribeFleetAttributesInput(Mapping):
        pass
    
    class DescribeFleetAttributesOutput(Mapping):
        pass
    
    class FleetId(object):
        pass
    
    class DescribeFleetCapacityInput(Mapping):
        pass
    
    class DescribeFleetCapacityOutput(Mapping):
        pass
    
    class DescribeFleetEventsInput(Mapping):
        pass
    
    class DescribeFleetEventsOutput(Mapping):
        pass
    
    class Timestamp(object):
        pass
    
    class DescribeFleetPortSettingsInput(Mapping):
        pass
    
    class DescribeFleetPortSettingsOutput(Mapping):
        pass
    
    class DescribeFleetUtilizationInput(Mapping):
        pass
    
    class DescribeFleetUtilizationOutput(Mapping):
        pass
    
    class DescribeGameSessionDetailsInput(Mapping):
        pass
    
    class DescribeGameSessionDetailsOutput(Mapping):
        pass
    
    class DescribeGameSessionPlacementInput(Mapping):
        pass
    
    class DescribeGameSessionPlacementOutput(Mapping):
        pass
    
    class DescribeGameSessionQueuesInput(Mapping):
        pass
    
    class DescribeGameSessionQueuesOutput(Mapping):
        pass
    
    class GameSessionQueueName(object):
        pass
    
    class DescribeGameSessionsInput(Mapping):
        pass
    
    class DescribeGameSessionsOutput(Mapping):
        pass
    
    class DescribeInstancesInput(Mapping):
        pass
    
    class DescribeInstancesOutput(Mapping):
        pass
    
    class DescribeMatchmakingInput(Mapping):
        pass
    
    class DescribeMatchmakingOutput(Mapping):
        pass
    
    class MatchmakingIdStringModel(object):
        pass
    
    class DescribeMatchmakingConfigurationsInput(Mapping):
        pass
    
    class DescribeMatchmakingConfigurationsOutput(Mapping):
        pass
    
    class DescribeMatchmakingRuleSetsInput(Mapping):
        pass
    
    class DescribeMatchmakingRuleSetsOutput(Mapping):
        pass
    
    class DescribePlayerSessionsInput(Mapping):
        pass
    
    class DescribePlayerSessionsOutput(Mapping):
        pass
    
    class DescribeRuntimeConfigurationInput(Mapping):
        pass
    
    class DescribeRuntimeConfigurationOutput(Mapping):
        pass
    
    class DescribeScalingPoliciesInput(Mapping):
        pass
    
    class DescribeScalingPoliciesOutput(Mapping):
        pass
    
    class DescribeScriptInput(Mapping):
        pass
    
    class DescribeScriptOutput(Mapping):
        pass
    
    class DescribeVpcPeeringAuthorizationsInput(Mapping):
        pass
    
    class DescribeVpcPeeringAuthorizationsOutput(Mapping):
        pass
    
    class DescribeVpcPeeringConnectionsInput(Mapping):
        pass
    
    class DescribeVpcPeeringConnectionsOutput(Mapping):
        pass
    
    class GetGameSessionLogUrlInput(Mapping):
        pass
    
    class GetGameSessionLogUrlOutput(Mapping):
        pass
    
    class GetInstanceAccessInput(Mapping):
        pass
    
    class GetInstanceAccessOutput(Mapping):
        pass
    
    class ListAliasesInput(Mapping):
        pass
    
    class ListAliasesOutput(Mapping):
        pass
    
    class ListBuildsInput(Mapping):
        pass
    
    class ListBuildsOutput(Mapping):
        pass
    
    class ListFleetsInput(Mapping):
        pass
    
    class ListFleetsOutput(Mapping):
        pass
    
    class ListScriptsInput(Mapping):
        pass
    
    class ListScriptsOutput(Mapping):
        pass
    
    class PutScalingPolicyInput(Mapping):
        pass
    
    class PutScalingPolicyOutput(Mapping):
        pass
    
    class Double(object):
        pass
    
    class TargetConfiguration(Mapping):
        pass
    
    class RequestUploadCredentialsInput(Mapping):
        pass
    
    class RequestUploadCredentialsOutput(Mapping):
        pass
    
    class ResolveAliasInput(Mapping):
        pass
    
    class ResolveAliasOutput(Mapping):
        pass
    
    class SearchGameSessionsInput(Mapping):
        pass
    
    class SearchGameSessionsOutput(Mapping):
        pass
    
    class StartFleetActionsInput(Mapping):
        pass
    
    class StartFleetActionsOutput(Mapping):
        pass
    
    class FleetAction(object):
        pass
    
    class StartGameSessionPlacementInput(Mapping):
        pass
    
    class StartGameSessionPlacementOutput(Mapping):
        pass
    
    class PlayerLatency(Mapping):
        pass
    
    class DesiredPlayerSession(Mapping):
        pass
    
    class StartMatchBackfillInput(Mapping):
        pass
    
    class StartMatchBackfillOutput(Mapping):
        pass
    
    class Player(Mapping):
        pass
    
    class StartMatchmakingInput(Mapping):
        pass
    
    class StartMatchmakingOutput(Mapping):
        pass
    
    class StopFleetActionsInput(Mapping):
        pass
    
    class StopFleetActionsOutput(Mapping):
        pass
    
    class StopGameSessionPlacementInput(Mapping):
        pass
    
    class StopGameSessionPlacementOutput(Mapping):
        pass
    
    class StopMatchmakingInput(Mapping):
        pass
    
    class StopMatchmakingOutput(Mapping):
        pass
    
    class UpdateAliasInput(Mapping):
        pass
    
    class UpdateAliasOutput(Mapping):
        pass
    
    class UpdateBuildInput(Mapping):
        pass
    
    class UpdateBuildOutput(Mapping):
        pass
    
    class UpdateFleetAttributesInput(Mapping):
        pass
    
    class UpdateFleetAttributesOutput(Mapping):
        pass
    
    class UpdateFleetCapacityInput(Mapping):
        pass
    
    class UpdateFleetCapacityOutput(Mapping):
        pass
    
    class UpdateFleetPortSettingsInput(Mapping):
        pass
    
    class UpdateFleetPortSettingsOutput(Mapping):
        pass
    
    class UpdateGameSessionInput(Mapping):
        pass
    
    class UpdateGameSessionOutput(Mapping):
        pass
    
    class UpdateGameSessionQueueInput(Mapping):
        pass
    
    class UpdateGameSessionQueueOutput(Mapping):
        pass
    
    class UpdateMatchmakingConfigurationInput(Mapping):
        pass
    
    class UpdateMatchmakingConfigurationOutput(Mapping):
        pass
    
    class UpdateRuntimeConfigurationInput(Mapping):
        pass
    
    class UpdateRuntimeConfigurationOutput(Mapping):
        pass
    
    class UpdateScriptInput(Mapping):
        pass
    
    class UpdateScriptOutput(Mapping):
        pass
    
    class ValidateMatchmakingRuleSetInput(Mapping):
        pass
    
    class ValidateMatchmakingRuleSetOutput(Mapping):
        pass
    
    def accept_match(self, AcceptanceType: str, PlayerIds: List[NonZeroAndMaxString], TicketId: str) -> AcceptMatchOutput:
        """Registers a player's acceptance or rejection of a proposed FlexMatch match. A matchmaking configuration may require player acceptance; if so, then matches built with that configuration cannot be completed unless all players accept the proposed match within a specified time limit.  When FlexMatch builds a match, all the matchmaking tickets involved in the proposed match are placed into status <code>REQUIRES_ACCEPTANCE</code>. This is a trigger for your game to get acceptance from all players in the ticket. Acceptances are only valid for tickets when they are in this status; all other acceptances result in an error. To register acceptance, specify the ticket ID, a response, and one or more players. Once all players have registered acceptance, the matchmaking tickets advance to status <code>PLACING</code>, where a new game session is created for the match.  If any player rejects the match, or if acceptances are not received before a specified timeout, the proposed match is dropped. The matchmaking tickets are then handled in one of two ways: For tickets where one or more players rejected the match, the ticket status is returned to <code>SEARCHING</code> to find a new match. For tickets where one or more players failed to respond, the ticket status is set to <code>CANCELLED</code>, and processing is terminated. A new matchmaking request for these players can be submitted as needed.   <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/match-client.html"> Add FlexMatch to a Game Client</a>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/match-events.html"> FlexMatch Events Reference</a>   <b>Related operations</b>  <ul> <li>  <a>StartMatchmaking</a>  </li> <li>  <a>DescribeMatchmaking</a>  </li> <li>  <a>StopMatchmaking</a>  </li> <li>  <a>AcceptMatch</a>  </li> <li>  <a>StartMatchBackfill</a>  </li> </ul>

        :param self:
        :param AcceptanceType:  Player response to the proposed match.<br/>PlayerIds:  Unique identifier for a player delivering the response. This parameter can include one or multiple player IDs.<br/>TicketId:  Unique identifier for a matchmaking ticket. The ticket must be in status <code>REQUIRES_ACCEPTANCE</code>; otherwise this request will fail.<br/>
        :return:         """
        pass

    def create_alias(self, RoutingStrategy: RoutingStrategy, Name: str, Description: str=None) -> CreateAliasOutput:
        """Creates an alias for a fleet. In most situations, you can use an alias ID in place of a fleet ID. By using a fleet alias instead of a specific fleet ID, you can switch gameplay and players to a new fleet without changing your game client or other game components. For example, for games in production, using an alias allows you to seamlessly redirect your player base to a new game server update.  Amazon GameLift supports two types of routing strategies for aliases: simple and terminal. A simple alias points to an active fleet. A terminal alias is used to display messaging or link to a URL instead of routing players to an active fleet. For example, you might use a terminal alias when a game version is no longer supported and you want to direct players to an upgrade site.  To create a fleet alias, specify an alias name, routing strategy, and optional description. Each simple alias can point to only one fleet, but a fleet can have multiple aliases. If successful, a new alias record is returned, including an alias ID, which you can reference when creating a game session. You can reassign an alias to another fleet by calling <code>UpdateAlias</code>. <ul> <li>  <a>CreateAlias</a>  </li> <li>  <a>ListAliases</a>  </li> <li>  <a>DescribeAlias</a>  </li> <li>  <a>UpdateAlias</a>  </li> <li>  <a>DeleteAlias</a>  </li> <li>  <a>ResolveAlias</a>  </li> </ul>

        :param self:
        :param RoutingStrategy:  Object that specifies the fleet and routing type to use for the alias.:<br/>Description:  Human-readable description of an alias.<br/>Name:  Descriptive label that is associated with an alias. Alias names do not need to be unique.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> Message text to be used with a terminal routing strategy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FleetId</i> Unique identifier for a fleet that the alias points to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> Type of routing strategy. Possible routing types include the following: <ul> <li>  <b>SIMPLE</b> -- The alias resolves to one specific fleet. Use this type when routing to active fleets. </li> <li>  <b>TERMINAL</b> -- The alias does not resolve to a fleet but instead can be used to display a message to the user. A terminal alias throws a TerminalRoutingStrategyException with the <a>RoutingStrategy</a> message embedded. </li> </ul><br/>
        :return: Alias:  Object that describes the newly created alias record.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdatedTime</i> Time stamp indicating when this data object was last modified. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> Time stamp indicating when this data object was created. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoutingStrategy</i> Alias configuration for the alias, including routing type and settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RoutingStrategy</b>: Alias configuration for the alias, including routing type and settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> Human-readable description of an alias.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AliasArn</i> Unique identifier for an alias; alias ARNs are unique across all regions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Descriptive label that is associated with an alias. Alias names do not need to be unique.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AliasId</i> Unique identifier for an alias; alias IDs are unique within a region.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> Message text to be used with a terminal routing strategy.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FleetId</i> Unique identifier for a fleet that the alias points to.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> Type of routing strategy. Possible routing types include the following: <ul> <li>  <b>SIMPLE</b> -- The alias resolves to one specific fleet. Use this type when routing to active fleets. </li> <li>  <b>TERMINAL</b> -- The alias does not resolve to a fleet but instead can be used to display a message to the user. A terminal alias throws a TerminalRoutingStrategyException with the <a>RoutingStrategy</a> message embedded. </li> </ul><br/>        """
        pass

    def create_build(self, Name: str=None, Version: str=None, StorageLocation: S3Location=None, OperatingSystem: str=None) -> CreateBuildOutput:
        """Creates a new Amazon GameLift build record for your game server binary files and points to the location of your game server build files in an Amazon Simple Storage Service (Amazon S3) location.  Game server binaries must be combined into a <code>.zip</code> file for use with Amazon GameLift.  <important> To create new builds quickly and easily, use the AWS CLI command <b> <a href="https://docs.aws.amazon.com/cli/latest/reference/gamelift/upload-build.html">upload-build</a> </b>. This helper command uploads your build and creates a new build record in one step, and automatically handles the necessary permissions.  </important> The <code>CreateBuild</code> operation should be used only when you need to manually upload your build files, as in the following scenarios: <ul> <li> Store a build file in an Amazon S3 bucket under your own AWS account. To use this option, you must first give Amazon GameLift access to that Amazon S3 bucket. To create a new build record using files in your Amazon S3 bucket, call <code>CreateBuild</code> and specify a build name, operating system, and the storage location of your game build. </li> <li> Upload a build file directly to Amazon GameLift's Amazon S3 account. To use this option, you first call <code>CreateBuild</code> with a build name and operating system. This action creates a new build record and returns an Amazon S3 storage location (bucket and key only) and temporary access credentials. Use the credentials to manually upload your build file to the storage location (see the Amazon S3 topic <a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/UploadingObjects.html">Uploading Objects</a>). You can upload files to a location only once.  </li> </ul> If successful, this operation creates a new build record with a unique build ID and places it in <code>INITIALIZED</code> status. You can use <a>DescribeBuild</a> to check the status of your build. A build must be in <code>READY</code> status before it can be used to create fleets.  <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-build-intro.html">Uploading Your Game</a>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-build-cli-uploading.html#gamelift-build-cli-uploading-create-build"> Create a Build with Files in Amazon S3</a>   <b>Related operations</b>  <ul> <li>  <a>CreateBuild</a>  </li> <li>  <a>ListBuilds</a>  </li> <li>  <a>DescribeBuild</a>  </li> <li>  <a>UpdateBuild</a>  </li> <li>  <a>DeleteBuild</a>  </li> </ul>

        :param self:
        :param OperatingSystem:  Operating system that the game server binaries are built to run on. This value determines the type of fleet resources that you can use for this build. If your game build contains multiple executables, they all must run on the same operating system. If an operating system is not specified when creating a build, Amazon GameLift uses the default value (WINDOWS_2012). This value cannot be changed later.<br/>StorageLocation:  Information indicating where your game build files are stored. Use this parameter only when creating a build with files stored in an Amazon S3 bucket that you own. The storage location must specify an Amazon S3 bucket name and key, as well as a the ARN for a role that you set up to allow Amazon GameLift to access your Amazon S3 bucket. The S3 bucket must be in the same region that you want to create a new build in.:<br/>Version:  Version that is associated with a build or script. Version strings do not need to be unique. You can use <a>UpdateBuild</a> to change this value later. <br/>Name:  Descriptive label that is associated with a build. Build names do not need to be unique. You can use <a>UpdateBuild</a> to change this value later. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ObjectVersion</i> Version of the file, if object versioning is turned on for the bucket. Amazon GameLift uses this information when retrieving files from an S3 bucket that you own. Use this parameter to specify a specific version of the file; if not set, the latest version of the file is retrieved. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> Amazon Resource Name (<a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/s3-arn-format.html">ARN</a>) for an IAM role that allows Amazon GameLift to access the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> Name of the zip file containing the build files or script files. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> Amazon S3 bucket identifier. This is the name of the S3 bucket.<br/>
        :return: StorageLocation:  Amazon S3 location for your game build file, including bucket name and key.:<br/>UploadCredentials:  This element is returned only when the operation is called without a storage location. It contains credentials to use when you are uploading a build file to an Amazon S3 bucket that is owned by Amazon GameLift. Credentials have a limited life span. To refresh these credentials, call <a>RequestUploadCredentials</a>. :<br/>Build:  The newly created build record, including a unique build ID and status. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> Time stamp indicating when this data object was created. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OperatingSystem</i> Operating system that the game server binaries are built to run on. This value determines the type of fleet resources that you can use for this build.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SizeOnDisk</i> File size of the uploaded game build, expressed in bytes. When the build status is <code>INITIALIZED</code>, this value is 0.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Current status of the build. Possible build statuses include the following: <ul> <li>  <b>INITIALIZED</b> -- A new build has been defined, but no files have been uploaded. You cannot create fleets for builds that are in this status. When a build is successfully created, the build status is set to this value.  </li> <li>  <b>READY</b> -- The game build has been successfully uploaded. You can now create new fleets for this build. </li> <li>  <b>FAILED</b> -- The game build upload failed. You cannot create new fleets for this build.  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> Version that is associated with a build or script. Version strings do not need to be unique. This value can be set using <a>CreateBuild</a> or <a>UpdateBuild</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Descriptive label that is associated with a build. Build names do not need to be unique. It can be set using <a>CreateBuild</a> or <a>UpdateBuild</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BuildId</i> Unique identifier for a build.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SessionToken</i> Token used to associate a specific build ID with the files uploaded using these credentials.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecretAccessKey</i> Temporary secret key allowing access to the Amazon GameLift S3 account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccessKeyId</i> Temporary key allowing access to the Amazon GameLift S3 account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ObjectVersion</i> Version of the file, if object versioning is turned on for the bucket. Amazon GameLift uses this information when retrieving files from an S3 bucket that you own. Use this parameter to specify a specific version of the file; if not set, the latest version of the file is retrieved. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> Amazon Resource Name (<a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/s3-arn-format.html">ARN</a>) for an IAM role that allows Amazon GameLift to access the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> Name of the zip file containing the build files or script files. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> Amazon S3 bucket identifier. This is the name of the S3 bucket.<br/>        """
        pass

    def create_fleet(self, EC2InstanceType: str, Name: str, Description: str=None, BuildId: str=None, ScriptId: str=None, ServerLaunchPath: str=None, ServerLaunchParameters: str=None, LogPaths: List[NonZeroAndMaxString]=None, EC2InboundPermissions: List[IpPermission]=None, NewGameSessionProtectionPolicy: str=None, RuntimeConfiguration: RuntimeConfiguration=None, ResourceCreationLimitPolicy: ResourceCreationLimitPolicy=None, MetricGroups: List[MetricGroup]=None, PeerVpcAwsAccountId: str=None, PeerVpcId: str=None, FleetType: str=None, InstanceRoleArn: str=None) -> CreateFleetOutput:
        """Creates a new fleet to run your game servers. whether they are custom game builds or Realtime Servers with game-specific script. A fleet is a set of Amazon Elastic Compute Cloud (Amazon EC2) instances, each of which can host multiple game sessions. When creating a fleet, you choose the hardware specifications, set some configuration options, and specify the game server to deploy on the new fleet.  To create a new fleet, you must provide the following: (1) a fleet name, (2) an EC2 instance type and fleet type (spot or on-demand), (3) the build ID for your game build or script ID if using Realtime Servers, and (4) a run-time configuration, which determines how game servers will run on each instance in the fleet.  <note> When creating a Realtime Servers fleet, we recommend using a minimal version of the Realtime script (see this <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/realtime-script.html#realtime-script-examples"> working code example </a>). This will make it much easier to troubleshoot any fleet creation issues. Once the fleet is active, you can update your Realtime script as needed. </note> If the <code>CreateFleet</code> call is successful, Amazon GameLift performs the following tasks. You can track the process of a fleet by checking the fleet status or by monitoring fleet creation events: <ul> <li> Creates a fleet record. Status: <code>NEW</code>. </li> <li> Begins writing events to the fleet event log, which can be accessed in the Amazon GameLift console. Sets the fleet's target capacity to 1 (desired instances), which triggers Amazon GameLift to start one new EC2 instance. </li> <li> Downloads the game build or Realtime script to the new instance and installs it. Statuses: <code>DOWNLOADING</code>, <code>VALIDATING</code>, <code>BUILDING</code>.  </li> <li> Starts launching server processes on the instance. If the fleet is configured to run multiple server processes per instance, Amazon GameLift staggers each launch by a few seconds. Status: <code>ACTIVATING</code>. </li> <li> Sets the fleet's status to <code>ACTIVE</code> as soon as one server process is ready to host a game session. </li> </ul>  <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/fleets-intro.html"> Working with Fleets</a>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/fleets-creating-debug.html"> Debug Fleet Creation Issues</a>   <b>Related operations</b>  <ul> <li>  <a>CreateFleet</a>  </li> <li>  <a>ListFleets</a>  </li> <li>  <a>DeleteFleet</a>  </li> <li> Describe fleets: <ul> <li>  <a>DescribeFleetAttributes</a>  </li> <li>  <a>DescribeFleetCapacity</a>  </li> <li>  <a>DescribeFleetPortSettings</a>  </li> <li>  <a>DescribeFleetUtilization</a>  </li> <li>  <a>DescribeRuntimeConfiguration</a>  </li> <li>  <a>DescribeEC2InstanceLimits</a>  </li> <li>  <a>DescribeFleetEvents</a>  </li> </ul> </li> <li> Update fleets: <ul> <li>  <a>UpdateFleetAttributes</a>  </li> <li>  <a>UpdateFleetCapacity</a>  </li> <li>  <a>UpdateFleetPortSettings</a>  </li> <li>  <a>UpdateRuntimeConfiguration</a>  </li> </ul> </li> <li> Manage fleet actions: <ul> <li>  <a>StartFleetActions</a>  </li> <li>  <a>StopFleetActions</a>  </li> </ul> </li> </ul>

        :param self:
        :param InstanceRoleArn:  Unique identifier for an AWS IAM role that manages access to your AWS services. With an instance role ARN set, any application that runs on an instance in this fleet can assume the role, including install scripts, server processes, daemons (background processes). Create a role or look up a role's ARN using the <a href="https://console.aws.amazon.com/iam/">IAM dashboard</a> in the AWS Management Console. Learn more about using on-box credentials for your game servers at <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-sdk-server-resources.html"> Access external resources from a game server</a>.<br/>FleetType:  Indicates whether to use on-demand instances or spot instances for this fleet. If empty, the default is ON_DEMAND. Both categories of instances use identical hardware and configurations based on the instance type selected for this fleet. Learn more about <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-ec2-instances.html#gamelift-ec2-instances-spot"> On-Demand versus Spot Instances</a>. <br/>PeerVpcId:  Unique identifier for a VPC with resources to be accessed by your Amazon GameLift fleet. The VPC must be in the same region where your fleet is deployed. Look up a VPC ID using the <a href="https://console.aws.amazon.com/vpc/">VPC Dashboard</a> in the AWS Management Console. Learn more about VPC peering in <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/vpc-peering.html">VPC Peering with Amazon GameLift Fleets</a>. <br/>PeerVpcAwsAccountId:  Unique identifier for the AWS account with the VPC that you want to peer your Amazon GameLift fleet with. You can find your Account ID in the AWS Management Console under account settings. <br/>MetricGroups:  Name of an Amazon CloudWatch metric group to add this fleet to. A metric group aggregates the metrics for all fleets in the group. Specify an existing metric group name, or provide a new name to create a new metric group. A fleet can only be included in one metric group at a time. <br/>ResourceCreationLimitPolicy:  Policy that limits the number of game sessions an individual player can create over a span of time for this fleet.:<br/>RuntimeConfiguration:  Instructions for launching server processes on each instance in the fleet. Server processes run either a custom game build executable or a Realtime Servers script. The run-time configuration lists the types of server processes to run on an instance and includes the following configuration settings: the server executable or launch script file, launch parameters, and the number of processes to run concurrently on each instance. A CreateFleet request must include a run-time configuration with at least one server process configuration.:<br/>NewGameSessionProtectionPolicy:  Game session protection policy to apply to all instances in this fleet. If this parameter is not set, instances in this fleet default to no protection. You can change a fleet's protection policy using <a>UpdateFleetAttributes</a>, but this change will only affect sessions created after the policy change. You can also set protection for individual instances using <a>UpdateGameSession</a>. <ul> <li>  <b>NoProtection</b> -- The game session can be terminated during a scale-down event. </li> <li>  <b>FullProtection</b> -- If the game session is in an <code>ACTIVE</code> status, it cannot be terminated during a scale-down event. </li> </ul><br/>EC2InboundPermissions:  Range of IP addresses and port settings that permit inbound traffic to access game sessions that running on the fleet. For fleets using a custom game build, this parameter is required before game sessions running on the fleet can accept connections. For Realtime Servers fleets, Amazon GameLift automatically sets TCP and UDP ranges for use by the Realtime servers. You can specify multiple permission settings or add more by updating the fleet.<br/>EC2InstanceType:  Name of an EC2 instance type that is supported in Amazon GameLift. A fleet instance type determines the computing resources of each instance in the fleet, including CPU, memory, storage, and networking capacity. Amazon GameLift supports the following EC2 instance types. See <a href="http://aws.amazon.com/ec2/instance-types/">Amazon EC2 Instance Types</a> for detailed descriptions.<br/>LogPaths:  This parameter is no longer used. Instead, to specify where Amazon GameLift should store log files once a server process shuts down, use the Amazon GameLift server API <code>ProcessReady()</code> and specify one or more directory paths in <code>logParameters</code>. See more information in the <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-sdk-server-api-ref.html#gamelift-sdk-server-api-ref-dataypes-process">Server API Reference</a>. <br/>ServerLaunchParameters:  This parameter is no longer used. Instead, specify server launch parameters in the <code>RuntimeConfiguration</code> parameter. (Requests that specify a server launch path and launch parameters instead of a run-time configuration will continue to work.)<br/>ServerLaunchPath:  This parameter is no longer used. Instead, specify a server launch path using the <code>RuntimeConfiguration</code> parameter. (Requests that specify a server launch path and launch parameters instead of a run-time configuration will continue to work.)<br/>ScriptId:  Unique identifier for a Realtime script to be deployed on the new fleet. The Realtime script must have been successfully uploaded to Amazon GameLift. This fleet setting cannot be changed once the fleet is created.<br/>BuildId:  Unique identifier for a build to be deployed on the new fleet. The custom game server build must have been successfully uploaded to Amazon GameLift and be in a <code>READY</code> status. This fleet setting cannot be changed once the fleet is created.<br/>Description:  Human-readable description of a fleet.<br/>Name:  Descriptive label that is associated with a fleet. Fleet names do not need to be unique.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionActivationTimeoutSeconds</i> Maximum amount of time (in seconds) that a game session can remain in status <code>ACTIVATING</code>. If the game session is not active before the timeout, activation is terminated and the game session status is changed to <code>TERMINATED</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxConcurrentGameSessionActivations</i> Maximum number of game sessions with status <code>ACTIVATING</code> to allow on an instance simultaneously. This setting limits the amount of instance resources that can be used for new game activations at any one time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerProcesses</i> Collection of server process configurations that describe which server processes to run on each instance in a fleet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyPeriodInMinutes</i> Time span used in evaluating the resource creation limit policy. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NewGameSessionsPerCreator</i> Maximum number of game sessions that an individual can create during the policy period. <br/>
        :return: FleetAttributes:  Properties for the newly created fleet.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceRoleArn</i> Unique identifier for an AWS IAM role that manages access to your AWS services. With an instance role ARN set, any application that runs on an instance in this fleet can assume the role, including install scripts, server processes, daemons (background processes). Create a role or look up a role's ARN using the <a href="https://console.aws.amazon.com/iam/">IAM dashboard</a> in the AWS Management Console. Learn more about using on-box credentials for your game servers at <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-sdk-server-resources.html"> Access external resources from a game server</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StoppedActions</i> List of fleet actions that have been suspended using <a>StopFleetActions</a>. This includes auto-scaling.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricGroups</i> Names of metric groups that this fleet is included in. In Amazon CloudWatch, you can view metrics for an individual fleet or aggregated metrics for fleets that are in a fleet metric group. A fleet can be included in only one metric group at a time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceCreationLimitPolicy</i> Fleet policy to limit the number of game sessions an individual player can create over a span of time.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResourceCreationLimitPolicy</b>: Fleet policy to limit the number of game sessions an individual player can create over a span of time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OperatingSystem</i> Operating system of the fleet's computing resources. A fleet's operating system depends on the OS specified for the build that is deployed on this fleet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NewGameSessionProtectionPolicy</i> Type of game session protection to set for all new instances started in the fleet. <ul> <li>  <b>NoProtection</b> -- The game session can be terminated during a scale-down event. </li> <li>  <b>FullProtection</b> -- If the game session is in an <code>ACTIVE</code> status, it cannot be terminated during a scale-down event. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogPaths</i> Location of default log files. When a server process is shut down, Amazon GameLift captures and stores any log files in this location. These logs are in addition to game session logs; see more on game session logs in the <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-sdk-server-api.html#gamelift-sdk-server-api-server-code">Amazon GameLift Developer Guide</a>. If no default log path for a fleet is specified, Amazon GameLift automatically uploads logs that are stored on each instance at <code>C:\game\logs</code> (for Windows) or <code>/local/game/logs</code> (for Linux). Use the Amazon GameLift console to access stored logs. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerLaunchParameters</i> Game server launch parameters specified for fleets created before 2016-08-04 (or AWS SDK v. 0.12.16). Server launch parameters for fleets created after this date are specified in the fleet's <a>RuntimeConfiguration</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerLaunchPath</i> Path to a game server executable in the fleet's build, specified for fleets created before 2016-08-04 (or AWS SDK v. 0.12.16). Server launch paths for fleets created after this date are specified in the fleet's <a>RuntimeConfiguration</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ScriptId</i> Unique identifier for a Realtime script.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BuildId</i> Unique identifier for a build.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Current status of the fleet. Possible fleet statuses include the following: <ul> <li>  <b>NEW</b> -- A new fleet has been defined and desired instances is set to 1.  </li> <li>  <b>DOWNLOADING/VALIDATING/BUILDING/ACTIVATING</b> -- Amazon GameLift is setting up the new fleet, creating new instances with the game build or Realtime script and starting server processes. </li> <li>  <b>ACTIVE</b> -- Hosts can now accept game sessions. </li> <li>  <b>ERROR</b> -- An error occurred when downloading, validating, building, or activating the fleet. </li> <li>  <b>DELETING</b> -- Hosts are responding to a delete fleet request. </li> <li>  <b>TERMINATED</b> -- The fleet no longer exists. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TerminationTime</i> Time stamp indicating when this data object was terminated. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> Time stamp indicating when this data object was created. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Descriptive label that is associated with a fleet. Fleet names do not need to be unique.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> Human-readable description of the fleet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i> EC2 instance type indicating the computing resources of each instance in the fleet, including CPU, memory, storage, and networking capacity. See <a href="http://aws.amazon.com/ec2/instance-types/">Amazon EC2 Instance Types</a> for detailed descriptions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FleetType</i> Indicates whether the fleet uses on-demand or spot instances. A spot instance in use may be interrupted with a two-minute notification.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FleetArn</i> Identifier for a fleet that is unique across all regions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FleetId</i> Unique identifier for a fleet.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyPeriodInMinutes</i> Time span used in evaluating the resource creation limit policy. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NewGameSessionsPerCreator</i> Maximum number of game sessions that an individual can create during the policy period. <br/>        """
        pass

    def create_game_session(self, MaximumPlayerSessionCount: int, FleetId: str=None, AliasId: str=None, Name: str=None, GameProperties: List[GameProperty]=None, CreatorId: str=None, GameSessionId: str=None, IdempotencyToken: str=None, GameSessionData: str=None) -> CreateGameSessionOutput:
        """Creates a multiplayer game session for players. This action creates a game session record and assigns an available server process in the specified fleet to host the game session. A fleet must have an <code>ACTIVE</code> status before a game session can be created in it. To create a game session, specify either fleet ID or alias ID and indicate a maximum number of players to allow in the game session. You can also provide a name and game-specific properties for this game session. If successful, a <a>GameSession</a> object is returned containing the game session properties and other settings you specified.  <b>Idempotency tokens.</b> You can add a token that uniquely identifies game session requests. This is useful for ensuring that game session requests are idempotent. Multiple requests with the same idempotency token are processed only once; subsequent requests return the original result. All response values are the same with the exception of game session status, which may change.  <b>Resource creation limits.</b> If you are creating a game session on a fleet with a resource creation limit policy in force, then you must specify a creator ID. Without this ID, Amazon GameLift has no way to evaluate the policy for this new game session request.  <b>Player acceptance policy.</b> By default, newly created game sessions are open to new players. You can restrict new player access by using <a>UpdateGameSession</a> to change the game session's player session creation policy.  <b>Game session logs.</b> Logs are retained for all active game sessions for 14 days. To access the logs, call <a>GetGameSessionLogUrl</a> to download the log files.  <i>Available in Amazon GameLift Local.</i>  <ul> <li>  <a>CreateGameSession</a>  </li> <li>  <a>DescribeGameSessions</a>  </li> <li>  <a>DescribeGameSessionDetails</a>  </li> <li>  <a>SearchGameSessions</a>  </li> <li>  <a>UpdateGameSession</a>  </li> <li>  <a>GetGameSessionLogUrl</a>  </li> <li> Game session placements <ul> <li>  <a>StartGameSessionPlacement</a>  </li> <li>  <a>DescribeGameSessionPlacement</a>  </li> <li>  <a>StopGameSessionPlacement</a>  </li> </ul> </li> </ul>

        :param self:
        :param GameSessionData:  Set of custom game session properties, formatted as a single string value. This data is passed to a game server process in the <a>GameSession</a> object with a request to start a new game session (see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-sdk-server-api.html#gamelift-sdk-server-startsession">Start a Game Session</a>).<br/>IdempotencyToken:  Custom string that uniquely identifies a request for a new game session. Maximum token length is 48 characters. If provided, this string is included in the new game session's ID. (A game session ARN has the following format: <code>arn:aws:gamelift:&lt;region&gt;::gamesession/&lt;fleet ID&gt;/&lt;custom ID string or idempotency token&gt;</code>.) Idempotency tokens remain in use for 30 days after a game session has ended; game session objects are retained for this time period and then deleted.<br/>GameSessionId:   <i>This parameter is no longer preferred. Please use <code>IdempotencyToken</code> instead.</i> Custom string that uniquely identifies a request for a new game session. Maximum token length is 48 characters. If provided, this string is included in the new game session's ID. (A game session ARN has the following format: <code>arn:aws:gamelift:&lt;region&gt;::gamesession/&lt;fleet ID&gt;/&lt;custom ID string or idempotency token&gt;</code>.) <br/>CreatorId:  Unique identifier for a player or entity creating the game session. This ID is used to enforce a resource protection policy (if one exists) that limits the number of concurrent active game sessions one player can have.<br/>GameProperties:  Set of custom properties for a game session, formatted as key:value pairs. These properties are passed to a game server process in the <a>GameSession</a> object with a request to start a new game session (see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-sdk-server-api.html#gamelift-sdk-server-startsession">Start a Game Session</a>).<br/>Name:  Descriptive label that is associated with a game session. Session names do not need to be unique.<br/>MaximumPlayerSessionCount:  Maximum number of players that can be connected simultaneously to the game session.<br/>AliasId:  Unique identifier for an alias associated with the fleet to create a game session in. Each request must reference either a fleet ID or alias ID, but not both.<br/>FleetId:  Unique identifier for a fleet to create a game session in. Each request must reference either a fleet ID or alias ID, but not both.<br/>
        :return: GameSession:  Object that describes the newly created game session record.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MatchmakerData</i> Information about the matchmaking process that was used to create the game session. It is in JSON syntax, formatted as a string. In addition the matchmaking configuration used, it contains data on all players assigned to the match, including player attributes and team assignments. For more details on matchmaker data, see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/match-server.html#match-server-data">Match Data</a>. Matchmaker data is useful when requesting match backfills, and is updated whenever new players are added during a successful backfill (see <a>StartMatchBackfill</a>). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionData</i> Set of custom game session properties, formatted as a single string value. This data is passed to a game server process in the <a>GameSession</a> object with a request to start a new game session (see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-sdk-server-api.html#gamelift-sdk-server-startsession">Start a Game Session</a>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatorId</i> Unique identifier for a player. This ID is used to enforce a resource protection policy (if one exists), that limits the number of game sessions a player can create.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlayerSessionCreationPolicy</i> Indicates whether or not the game session is accepting new players.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Port number for the game session. To connect to a Amazon GameLift game server, an app needs both the IP address and port number.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IpAddress</i> IP address of the game session. To connect to a Amazon GameLift game server, an app needs both the IP address and port number.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameProperties</i> Set of custom properties for a game session, formatted as key:value pairs. These properties are passed to a game server process in the <a>GameSession</a> object with a request to start a new game session (see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-sdk-server-api.html#gamelift-sdk-server-startsession">Start a Game Session</a>). You can search for active game sessions based on this custom data with <a>SearchGameSessions</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusReason</i> Provides additional information about game session status. <code>INTERRUPTED</code> indicates that the game session was hosted on a spot instance that was reclaimed, causing the active game session to be terminated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Current status of the game session. A game session must have an <code>ACTIVE</code> status to have player sessions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumPlayerSessionCount</i> Maximum number of players that can be connected simultaneously to the game session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentPlayerSessionCount</i> Number of players currently in the game session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TerminationTime</i> Time stamp indicating when this data object was terminated. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> Time stamp indicating when this data object was created. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FleetId</i> Unique identifier for a fleet that the game session is running on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Descriptive label that is associated with a game session. Session names do not need to be unique.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionId</i> Unique identifier for the game session. A game session ARN has the following format: <code>arn:aws:gamelift:&lt;region&gt;::gamesession/&lt;fleet ID&gt;/&lt;custom ID string or idempotency token&gt;</code>.<br/>        """
        pass

    def create_game_session_queue(self, Name: str, TimeoutInSeconds: int=None, PlayerLatencyPolicies: List[PlayerLatencyPolicy]=None, Destinations: List[GameSessionQueueDestination]=None) -> CreateGameSessionQueueOutput:
        """Establishes a new queue for processing requests to place new game sessions. A queue identifies where new game sessions can be hosted -- by specifying a list of destinations (fleets or aliases) -- and how long requests can wait in the queue before timing out. You can set up a queue to try to place game sessions on fleets in multiple regions. To add placement requests to a queue, call <a>StartGameSessionPlacement</a> and reference the queue name.  <b>Destination order.</b> When processing a request for a game session, Amazon GameLift tries each destination in order until it finds one with available resources to host the new game session. A queue's default order is determined by how destinations are listed. The default order is overridden when a game session placement request provides player latency information. Player latency information enables Amazon GameLift to prioritize destinations where players report the lowest average latency, as a result placing the new game session where the majority of players will have the best possible gameplay experience.  <b>Player latency policies.</b> For placement requests containing player latency information, use player latency policies to protect individual players from very high latencies. With a latency cap, even when a destination can deliver a low latency for most players, the game is not placed where any individual player is reporting latency higher than a policy's maximum. A queue can have multiple latency policies, which are enforced consecutively starting with the policy with the lowest latency cap. Use multiple policies to gradually relax latency controls; for example, you might set a policy with a low latency cap for the first 60 seconds, a second policy with a higher cap for the next 60 seconds, etc.  To create a new queue, provide a name, timeout value, a list of destinations and, if desired, a set of latency policies. If successful, a new queue object is returned. <ul> <li>  <a>CreateGameSessionQueue</a>  </li> <li>  <a>DescribeGameSessionQueues</a>  </li> <li>  <a>UpdateGameSessionQueue</a>  </li> <li>  <a>DeleteGameSessionQueue</a>  </li> </ul>

        :param self:
        :param Destinations:  List of fleets that can be used to fulfill game session placement requests in the queue. Fleets are identified by either a fleet ARN or a fleet alias ARN. Destinations are listed in default preference order.<br/>PlayerLatencyPolicies:  Collection of latency policies to apply when processing game sessions placement requests with player latency information. Multiple policies are evaluated in order of the maximum latency value, starting with the lowest latency values. With just one policy, it is enforced at the start of the game session placement for the duration period. With multiple policies, each policy is enforced consecutively for its duration period. For example, a queue might enforce a 60-second policy followed by a 120-second policy, and then no policy for the remainder of the placement. A player latency policy must set a value for MaximumIndividualPlayerLatencyMilliseconds; if none is set, this API requests will fail.<br/>TimeoutInSeconds:  Maximum time, in seconds, that a new game session placement request remains in the queue. When a request exceeds this time, the game session placement changes to a <code>TIMED_OUT</code> status.<br/>Name:  Descriptive label that is associated with game session queue. Queue names must be unique within each region.<br/>
        :return: GameSessionQueue:  Object that describes the newly created game session queue.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Destinations</i> List of fleets that can be used to fulfill game session placement requests in the queue. Fleets are identified by either a fleet ARN or a fleet alias ARN. Destinations are listed in default preference order.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlayerLatencyPolicies</i> Collection of latency policies to apply when processing game sessions placement requests with player latency information. Multiple policies are evaluated in order of the maximum latency value, starting with the lowest latency values. With just one policy, it is enforced at the start of the game session placement for the duration period. With multiple policies, each policy is enforced consecutively for its duration period. For example, a queue might enforce a 60-second policy followed by a 120-second policy, and then no policy for the remainder of the placement. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeoutInSeconds</i> Maximum time, in seconds, that a new game session placement request remains in the queue. When a request exceeds this time, the game session placement changes to a <code>TIMED_OUT</code> status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionQueueArn</i> Amazon Resource Name (<a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/s3-arn-format.html">ARN</a>) that is assigned to a game session queue and uniquely identifies it. Format is <code>arn:aws:gamelift:&lt;region&gt;:&lt;aws account&gt;:gamesessionqueue/&lt;queue name&gt;</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Descriptive label that is associated with game session queue. Queue names must be unique within each region.<br/>        """
        pass

    def create_matchmaking_configuration(self, RuleSetName: str, AcceptanceRequired: BooleanModel, RequestTimeoutSeconds: int, GameSessionQueueArns: List[ArnStringModel], Name: str, Description: str=None, AcceptanceTimeoutSeconds: int=None, NotificationTarget: str=None, AdditionalPlayerCount: int=None, CustomEventData: str=None, GameProperties: List[GameProperty]=None, GameSessionData: str=None, BackfillMode: str=None) -> CreateMatchmakingConfigurationOutput:
        """Defines a new matchmaking configuration for use with FlexMatch. A matchmaking configuration sets out guidelines for matching players and getting the matches into games. You can set up multiple matchmaking configurations to handle the scenarios needed for your game. Each matchmaking ticket (<a>StartMatchmaking</a> or <a>StartMatchBackfill</a>) specifies a configuration for the match and provides player attributes to support the configuration being used.  To create a matchmaking configuration, at a minimum you must specify the following: configuration name; a rule set that governs how to evaluate players and find acceptable matches; a game session queue to use when placing a new game session for the match; and the maximum time allowed for a matchmaking attempt. There are two ways to track the progress of matchmaking tickets: (1) polling ticket status with <a>DescribeMatchmaking</a>; or (2) receiving notifications with Amazon Simple Notification Service (SNS). To use notifications, you first need to set up an SNS topic to receive the notifications, and provide the topic ARN in the matchmaking configuration. Since notifications promise only "best effort" delivery, we recommend calling <code>DescribeMatchmaking</code> if no notifications are received within 30 seconds.  <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/match-configuration.html"> Design a FlexMatch Matchmaker</a>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/match-notification.html"> Setting up Notifications for Matchmaking</a>   <b>Related operations</b>  <ul> <li>  <a>CreateMatchmakingConfiguration</a>  </li> <li>  <a>DescribeMatchmakingConfigurations</a>  </li> <li>  <a>UpdateMatchmakingConfiguration</a>  </li> <li>  <a>DeleteMatchmakingConfiguration</a>  </li> <li>  <a>CreateMatchmakingRuleSet</a>  </li> <li>  <a>DescribeMatchmakingRuleSets</a>  </li> <li>  <a>ValidateMatchmakingRuleSet</a>  </li> <li>  <a>DeleteMatchmakingRuleSet</a>  </li> </ul>

        :param self:
        :param BackfillMode:  Method used to backfill game sessions created with this matchmaking configuration. Specify MANUAL when your game manages backfill requests manually or does not use the match backfill feature. Specify AUTOMATIC to have GameLift create a <a>StartMatchBackfill</a> request whenever a game session has one or more open slots. Learn more about manual and automatic backfill in <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/match-backfill.html"> Backfill Existing Games with FlexMatch</a>. <br/>GameSessionData:  Set of custom game session properties, formatted as a single string value. This data is passed to a game server process in the <a>GameSession</a> object with a request to start a new game session (see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-sdk-server-api.html#gamelift-sdk-server-startsession">Start a Game Session</a>). This information is added to the new <a>GameSession</a> object that is created for a successful match.<br/>GameProperties:  Set of custom properties for a game session, formatted as key:value pairs. These properties are passed to a game server process in the <a>GameSession</a> object with a request to start a new game session (see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-sdk-server-api.html#gamelift-sdk-server-startsession">Start a Game Session</a>). This information is added to the new <a>GameSession</a> object that is created for a successful match. <br/>CustomEventData:  Information to be added to all events related to this matchmaking configuration. <br/>AdditionalPlayerCount:  Number of player slots in a match to keep open for future players. For example, if the configuration's rule set specifies a match for a single 12-person team, and the additional player count is set to 2, only 10 players are selected for the match.<br/>NotificationTarget:  SNS topic ARN that is set up to receive matchmaking notifications.<br/>RuleSetName:  Unique identifier for a matchmaking rule set to use with this configuration. A matchmaking configuration can only use rule sets that are defined in the same region.<br/>AcceptanceRequired:  Flag that determines whether a match that was created with this configuration must be accepted by the matched players. To require acceptance, set to TRUE.<br/>AcceptanceTimeoutSeconds:  Length of time (in seconds) to wait for players to accept a proposed match. If any player rejects the match or fails to accept before the timeout, the ticket continues to look for an acceptable match.<br/>RequestTimeoutSeconds:  Maximum duration, in seconds, that a matchmaking ticket can remain in process before timing out. Requests that fail due to timing out can be resubmitted as needed.<br/>GameSessionQueueArns:  Amazon Resource Name (<a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/s3-arn-format.html">ARN</a>) that is assigned to a game session queue and uniquely identifies it. Format is <code>arn:aws:gamelift:&lt;region&gt;:&lt;aws account&gt;:gamesessionqueue/&lt;queue name&gt;</code>. These queues are used when placing game sessions for matches that are created with this matchmaking configuration. Queues can be located in any region.<br/>Description:  Meaningful description of the matchmaking configuration. <br/>Name:  Unique identifier for a matchmaking configuration. This name is used to identify the configuration associated with a matchmaking request or ticket.<br/>
        :return: Configuration:  Object that describes the newly created matchmaking configuration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackfillMode</i> Method used to backfill game sessions created with this matchmaking configuration. MANUAL indicates that the game makes backfill requests or does not use the match backfill feature. AUTOMATIC indicates that GameLift creates <a>StartMatchBackfill</a> requests whenever a game session has one or more open slots. Learn more about manual and automatic backfill in <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/match-backfill.html">Backfill Existing Games with FlexMatch</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionData</i> Set of custom game session properties, formatted as a single string value. This data is passed to a game server process in the <a>GameSession</a> object with a request to start a new game session (see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-sdk-server-api.html#gamelift-sdk-server-startsession">Start a Game Session</a>). This information is added to the new <a>GameSession</a> object that is created for a successful match. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameProperties</i> Set of custom properties for a game session, formatted as key:value pairs. These properties are passed to a game server process in the <a>GameSession</a> object with a request to start a new game session (see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-sdk-server-api.html#gamelift-sdk-server-startsession">Start a Game Session</a>). This information is added to the new <a>GameSession</a> object that is created for a successful match. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> Time stamp indicating when this data object was created. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomEventData</i> Information to attach to all events related to the matchmaking configuration. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdditionalPlayerCount</i> Number of player slots in a match to keep open for future players. For example, if the configuration's rule set specifies a match for a single 12-person team, and the additional player count is set to 2, only 10 players are selected for the match.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationTarget</i> SNS topic ARN that is set up to receive matchmaking notifications.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleSetName</i> Unique identifier for a matchmaking rule set to use with this configuration. A matchmaking configuration can only use rule sets that are defined in the same region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AcceptanceRequired</i> Flag that determines whether a match that was created with this configuration must be accepted by the matched players. To require acceptance, set to TRUE.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AcceptanceTimeoutSeconds</i> Length of time (in seconds) to wait for players to accept a proposed match. If any player rejects the match or fails to accept before the timeout, the ticket continues to look for an acceptable match.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestTimeoutSeconds</i> Maximum duration, in seconds, that a matchmaking ticket can remain in process before timing out. Requests that fail due to timing out can be resubmitted as needed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionQueueArns</i> Amazon Resource Name (<a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/s3-arn-format.html">ARN</a>) that is assigned to a game session queue and uniquely identifies it. Format is <code>arn:aws:gamelift:&lt;region&gt;:&lt;aws account&gt;:gamesessionqueue/&lt;queue name&gt;</code>. These queues are used when placing game sessions for matches that are created with this matchmaking configuration. Queues can be located in any region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> Descriptive label that is associated with matchmaking configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Unique identifier for a matchmaking configuration. This name is used to identify the configuration associated with a matchmaking request or ticket.<br/>        """
        pass

    def create_matchmaking_rule_set(self, RuleSetBody: str, Name: str) -> CreateMatchmakingRuleSetOutput:
        """Creates a new rule set for FlexMatch matchmaking. A rule set describes the type of match to create, such as the number and size of teams, and sets the parameters for acceptable player matches, such as minimum skill level or character type. A rule set is used by a <a>MatchmakingConfiguration</a>.  To create a matchmaking rule set, provide unique rule set name and the rule set body in JSON format. Rule sets must be defined in the same region as the matchmaking configuration they are used with. Since matchmaking rule sets cannot be edited, it is a good idea to check the rule set syntax using <a>ValidateMatchmakingRuleSet</a> before creating a new rule set.  <b>Learn more</b>  <ul> <li>  <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/match-rulesets.html">Build a Rule Set</a>  </li> <li>  <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/match-configuration.html">Design a Matchmaker</a>  </li> <li>  <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/match-intro.html">Matchmaking with FlexMatch</a>  </li> </ul>  <b>Related operations</b>  <ul> <li>  <a>CreateMatchmakingConfiguration</a>  </li> <li>  <a>DescribeMatchmakingConfigurations</a>  </li> <li>  <a>UpdateMatchmakingConfiguration</a>  </li> <li>  <a>DeleteMatchmakingConfiguration</a>  </li> <li>  <a>CreateMatchmakingRuleSet</a>  </li> <li>  <a>DescribeMatchmakingRuleSets</a>  </li> <li>  <a>ValidateMatchmakingRuleSet</a>  </li> <li>  <a>DeleteMatchmakingRuleSet</a>  </li> </ul>

        :param self:
        :param RuleSetBody:  Collection of matchmaking rules, formatted as a JSON string. Comments are not allowed in JSON, but most elements support a description field.<br/>Name:  Unique identifier for a matchmaking rule set. A matchmaking configuration identifies the rule set it uses by this name value. (Note: The rule set name is different from the optional "name" field in the rule set body.) <br/>
        :return: RuleSet:  Object that describes the newly created matchmaking rule set.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> Time stamp indicating when this data object was created. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleSetBody</i> Collection of matchmaking rules, formatted as a JSON string. Comments are not allowed in JSON, but most elements support a description field.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleSetName</i> Unique identifier for a matchmaking rule set<br/>        """
        pass

    def create_player_session(self, PlayerId: str, GameSessionId: str, PlayerData: str=None) -> CreatePlayerSessionOutput:
        """Reserves an open player slot in an active game session. Before a player can be added, a game session must have an <code>ACTIVE</code> status, have a creation policy of <code>ALLOW_ALL</code>, and have an open player slot. To add a group of players to a game session, use <a>CreatePlayerSessions</a>. When the player connects to the game server and references a player session ID, the game server contacts the Amazon GameLift service to validate the player reservation and accept the player. To create a player session, specify a game session ID, player ID, and optionally a string of player data. If successful, a slot is reserved in the game session for the player and a new <a>PlayerSession</a> object is returned. Player sessions cannot be updated.   <i>Available in Amazon GameLift Local.</i>  <ul> <li>  <a>CreatePlayerSession</a>  </li> <li>  <a>CreatePlayerSessions</a>  </li> <li>  <a>DescribePlayerSessions</a>  </li> <li> Game session placements <ul> <li>  <a>StartGameSessionPlacement</a>  </li> <li>  <a>DescribeGameSessionPlacement</a>  </li> <li>  <a>StopGameSessionPlacement</a>  </li> </ul> </li> </ul>

        :param self:
        :param PlayerData:  Developer-defined information related to a player. Amazon GameLift does not use this data, so it can be formatted as needed for use in the game.<br/>PlayerId:  Unique identifier for a player. Player IDs are developer-defined.<br/>GameSessionId:  Unique identifier for the game session to add a player to.<br/>
        :return: PlayerSession:  Object that describes the newly created player session record.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlayerData</i> Developer-defined information related to a player. Amazon GameLift does not use this data, so it can be formatted as needed for use in the game. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Port number for the game session. To connect to a Amazon GameLift server process, an app needs both the IP address and port number.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IpAddress</i> IP address of the game session. To connect to a Amazon GameLift game server, an app needs both the IP address and port number.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Current status of the player session. Possible player session statuses include the following: <ul> <li>  <b>RESERVED</b> -- The player session request has been received, but the player has not yet connected to the server process and/or been validated.  </li> <li>  <b>ACTIVE</b> -- The player has been validated by the server process and is currently connected. </li> <li>  <b>COMPLETED</b> -- The player connection has been dropped. </li> <li>  <b>TIMEDOUT</b> -- A player session request was received, but the player did not connect and/or was not validated within the timeout limit (60 seconds). </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TerminationTime</i> Time stamp indicating when this data object was terminated. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> Time stamp indicating when this data object was created. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FleetId</i> Unique identifier for a fleet that the player's game session is running on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionId</i> Unique identifier for the game session that the player session is connected to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlayerId</i> Unique identifier for a player that is associated with this player session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlayerSessionId</i> Unique identifier for a player session.<br/>        """
        pass

    def create_player_sessions(self, PlayerIds: List[NonZeroAndMaxString], GameSessionId: str, PlayerDataMap: PlayerDataMap=None) -> CreatePlayerSessionsOutput:
        """Reserves open slots in a game session for a group of players. Before players can be added, a game session must have an <code>ACTIVE</code> status, have a creation policy of <code>ALLOW_ALL</code>, and have an open player slot. To add a single player to a game session, use <a>CreatePlayerSession</a>. When a player connects to the game server and references a player session ID, the game server contacts the Amazon GameLift service to validate the player reservation and accept the player. To create player sessions, specify a game session ID, a list of player IDs, and optionally a set of player data strings. If successful, a slot is reserved in the game session for each player and a set of new <a>PlayerSession</a> objects is returned. Player sessions cannot be updated.  <i>Available in Amazon GameLift Local.</i>  <ul> <li>  <a>CreatePlayerSession</a>  </li> <li>  <a>CreatePlayerSessions</a>  </li> <li>  <a>DescribePlayerSessions</a>  </li> <li> Game session placements <ul> <li>  <a>StartGameSessionPlacement</a>  </li> <li>  <a>DescribeGameSessionPlacement</a>  </li> <li>  <a>StopGameSessionPlacement</a>  </li> </ul> </li> </ul>

        :param self:
        :param PlayerDataMap:  Map of string pairs, each specifying a player ID and a set of developer-defined information related to the player. Amazon GameLift does not use this data, so it can be formatted as needed for use in the game. Player data strings for player IDs not included in the <code>PlayerIds</code> parameter are ignored. <br/>PlayerIds:  List of unique identifiers for the players to be added.<br/>GameSessionId:  Unique identifier for the game session to add players to.<br/>
        :return: PlayerSessions:  Collection of player session objects created for the added players.<br/>        """
        pass

    def create_script(self, Name: str=None, Version: str=None, StorageLocation: S3Location=None, ZipFile: ZipBlob=None) -> CreateScriptOutput:
        """Creates a new script record for your Realtime Servers script. Realtime scripts are JavaScript that provide configuration settings and optional custom game logic for your game. The script is deployed when you create a Realtime Servers fleet to host your game sessions. Script logic is executed during an active game session.  To create a new script record, specify a script name and provide the script file(s). The script files and all dependencies must be zipped into a single file. You can pull the zip file from either of these locations:  <ul> <li> A locally available directory. Use the <i>ZipFile</i> parameter for this option. </li> <li> An Amazon Simple Storage Service (Amazon S3) bucket under your AWS account. Use the <i>StorageLocation</i> parameter for this option. You'll need to have an Identity Access Management (IAM) role that allows the Amazon GameLift service to access your S3 bucket.  </li> </ul> If the call is successful, a new script record is created with a unique script ID. If the script file is provided as a local file, the file is uploaded to an Amazon GameLift-owned S3 bucket and the script record's storage location reflects this location. If the script file is provided as an S3 bucket, Amazon GameLift accesses the file at this storage location as needed for deployment.  <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/realtime-intro.html">Amazon GameLift Realtime Servers</a>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/setting-up-role.html">Set Up a Role for Amazon GameLift Access</a>   <b>Related operations</b>  <ul> <li>  <a>CreateScript</a>  </li> <li>  <a>ListScripts</a>  </li> <li>  <a>DescribeScript</a>  </li> <li>  <a>UpdateScript</a>  </li> <li>  <a>DeleteScript</a>  </li> </ul>

        :param self:
        :param ZipFile:  Data object containing your Realtime scripts and dependencies as a zip file. The zip file can have one or multiple files. Maximum size of a zip file is 5 MB. When using the AWS CLI tool to create a script, this parameter is set to the zip file name. It must be prepended with the string "fileb://" to indicate that the file data is a binary object. For example: <code>--zip-file fileb://myRealtimeScript.zip</code>.<br/>StorageLocation:  Location of the Amazon S3 bucket where a zipped file containing your Realtime scripts is stored. The storage location must specify the Amazon S3 bucket name, the zip file name (the "key"), and a role ARN that allows Amazon GameLift to access the Amazon S3 storage location. The S3 bucket must be in the same region where you want to create a new script. By default, Amazon GameLift uploads the latest version of the zip file; if you have S3 object versioning turned on, you can use the <code>ObjectVersion</code> parameter to specify an earlier version. :<br/>Version:  Version that is associated with a build or script. Version strings do not need to be unique. You can use <a>UpdateScript</a> to change this value later. <br/>Name:  Descriptive label that is associated with a script. Script names do not need to be unique. You can use <a>UpdateScript</a> to change this value later. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ObjectVersion</i> Version of the file, if object versioning is turned on for the bucket. Amazon GameLift uses this information when retrieving files from an S3 bucket that you own. Use this parameter to specify a specific version of the file; if not set, the latest version of the file is retrieved. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> Amazon Resource Name (<a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/s3-arn-format.html">ARN</a>) for an IAM role that allows Amazon GameLift to access the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> Name of the zip file containing the build files or script files. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> Amazon S3 bucket identifier. This is the name of the S3 bucket.<br/>
        :return: Script:  The newly created script record with a unique script ID. The new script's storage location reflects an Amazon S3 location: (1) If the script was uploaded from an S3 bucket under your account, the storage location reflects the information that was provided in the <i>CreateScript</i> request; (2) If the script file was uploaded from a local zip file, the storage location reflects an S3 location controls by the Amazon GameLift service.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageLocation</i> Location in Amazon Simple Storage Service (Amazon S3) where build or script files are stored for access by Amazon GameLift. This location is specified in <a>CreateBuild</a>, <a>CreateScript</a>, and <a>UpdateScript</a> requests. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StorageLocation</b>: Location in Amazon Simple Storage Service (Amazon S3) where build or script files are stored for access by Amazon GameLift. This location is specified in <a>CreateBuild</a>, <a>CreateScript</a>, and <a>UpdateScript</a> requests. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> Time stamp indicating when this data object was created. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SizeOnDisk</i> File size of the uploaded Realtime script, expressed in bytes. When files are uploaded from an S3 location, this value remains at "0".<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> Version that is associated with a build or script. Version strings do not need to be unique.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Descriptive label that is associated with a script. Script names do not need to be unique.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ScriptId</i> Unique identifier for a Realtime script<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ObjectVersion</i> Version of the file, if object versioning is turned on for the bucket. Amazon GameLift uses this information when retrieving files from an S3 bucket that you own. Use this parameter to specify a specific version of the file; if not set, the latest version of the file is retrieved. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> Amazon Resource Name (<a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/s3-arn-format.html">ARN</a>) for an IAM role that allows Amazon GameLift to access the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> Name of the zip file containing the build files or script files. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> Amazon S3 bucket identifier. This is the name of the S3 bucket.<br/>        """
        pass

    def create_vpc_peering_authorization(self, PeerVpcId: str, GameLiftAwsAccountId: str) -> CreateVpcPeeringAuthorizationOutput:
        """Requests authorization to create or delete a peer connection between the VPC for your Amazon GameLift fleet and a virtual private cloud (VPC) in your AWS account. VPC peering enables the game servers on your fleet to communicate directly with other AWS resources. Once you've received authorization, call <a>CreateVpcPeeringConnection</a> to establish the peering connection. For more information, see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/vpc-peering.html">VPC Peering with Amazon GameLift Fleets</a>. You can peer with VPCs that are owned by any AWS account you have access to, including the account that you use to manage your Amazon GameLift fleets. You cannot peer with VPCs that are in different regions. To request authorization to create a connection, call this operation from the AWS account with the VPC that you want to peer to your Amazon GameLift fleet. For example, to enable your game servers to retrieve data from a DynamoDB table, use the account that manages that DynamoDB resource. Identify the following values: (1) The ID of the VPC that you want to peer with, and (2) the ID of the AWS account that you use to manage Amazon GameLift. If successful, VPC peering is authorized for the specified VPC.  To request authorization to delete a connection, call this operation from the AWS account with the VPC that is peered with your Amazon GameLift fleet. Identify the following values: (1) VPC ID that you want to delete the peering connection for, and (2) ID of the AWS account that you use to manage Amazon GameLift.  The authorization remains valid for 24 hours unless it is canceled by a call to <a>DeleteVpcPeeringAuthorization</a>. You must create or delete the peering connection while the authorization is valid.  <ul> <li>  <a>CreateVpcPeeringAuthorization</a>  </li> <li>  <a>DescribeVpcPeeringAuthorizations</a>  </li> <li>  <a>DeleteVpcPeeringAuthorization</a>  </li> <li>  <a>CreateVpcPeeringConnection</a>  </li> <li>  <a>DescribeVpcPeeringConnections</a>  </li> <li>  <a>DeleteVpcPeeringConnection</a>  </li> </ul>

        :param self:
        :param PeerVpcId:  Unique identifier for a VPC with resources to be accessed by your Amazon GameLift fleet. The VPC must be in the same region where your fleet is deployed. Look up a VPC ID using the <a href="https://console.aws.amazon.com/vpc/">VPC Dashboard</a> in the AWS Management Console. Learn more about VPC peering in <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/vpc-peering.html">VPC Peering with Amazon GameLift Fleets</a>.<br/>GameLiftAwsAccountId:  Unique identifier for the AWS account that you use to manage your Amazon GameLift fleet. You can find your Account ID in the AWS Management Console under account settings.<br/>
        :return: VpcPeeringAuthorization:  Details on the requested VPC peering authorization, including expiration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpirationTime</i> Time stamp indicating when this authorization expires (24 hours after issuance). Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> Time stamp indicating when this authorization was issued. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PeerVpcId</i> Unique identifier for a VPC with resources to be accessed by your Amazon GameLift fleet. The VPC must be in the same region where your fleet is deployed. Look up a VPC ID using the <a href="https://console.aws.amazon.com/vpc/">VPC Dashboard</a> in the AWS Management Console. Learn more about VPC peering in <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/vpc-peering.html">VPC Peering with Amazon GameLift Fleets</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PeerVpcAwsAccountId</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameLiftAwsAccountId</i> Unique identifier for the AWS account that you use to manage your Amazon GameLift fleet. You can find your Account ID in the AWS Management Console under account settings.<br/>        """
        pass

    def create_vpc_peering_connection(self, PeerVpcId: str, PeerVpcAwsAccountId: str, FleetId: str) -> CreateVpcPeeringConnectionOutput:
        """Establishes a VPC peering connection between a virtual private cloud (VPC) in an AWS account with the VPC for your Amazon GameLift fleet. VPC peering enables the game servers on your fleet to communicate directly with other AWS resources. You can peer with VPCs in any AWS account that you have access to, including the account that you use to manage your Amazon GameLift fleets. You cannot peer with VPCs that are in different regions. For more information, see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/vpc-peering.html">VPC Peering with Amazon GameLift Fleets</a>. Before calling this operation to establish the peering connection, you first need to call <a>CreateVpcPeeringAuthorization</a> and identify the VPC you want to peer with. Once the authorization for the specified VPC is issued, you have 24 hours to establish the connection. These two operations handle all tasks necessary to peer the two VPCs, including acceptance, updating routing tables, etc.  To establish the connection, call this operation from the AWS account that is used to manage the Amazon GameLift fleets. Identify the following values: (1) The ID of the fleet you want to be enable a VPC peering connection for; (2) The AWS account with the VPC that you want to peer with; and (3) The ID of the VPC you want to peer with. This operation is asynchronous. If successful, a <a>VpcPeeringConnection</a> request is created. You can use continuous polling to track the request's status using <a>DescribeVpcPeeringConnections</a>, or by monitoring fleet events for success or failure using <a>DescribeFleetEvents</a>.  <ul> <li>  <a>CreateVpcPeeringAuthorization</a>  </li> <li>  <a>DescribeVpcPeeringAuthorizations</a>  </li> <li>  <a>DeleteVpcPeeringAuthorization</a>  </li> <li>  <a>CreateVpcPeeringConnection</a>  </li> <li>  <a>DescribeVpcPeeringConnections</a>  </li> <li>  <a>DeleteVpcPeeringConnection</a>  </li> </ul>

        :param self:
        :param PeerVpcId:  Unique identifier for a VPC with resources to be accessed by your Amazon GameLift fleet. The VPC must be in the same region where your fleet is deployed. Look up a VPC ID using the <a href="https://console.aws.amazon.com/vpc/">VPC Dashboard</a> in the AWS Management Console. Learn more about VPC peering in <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/vpc-peering.html">VPC Peering with Amazon GameLift Fleets</a>.<br/>PeerVpcAwsAccountId:  Unique identifier for the AWS account with the VPC that you want to peer your Amazon GameLift fleet with. You can find your Account ID in the AWS Management Console under account settings.<br/>FleetId:  Unique identifier for a fleet. This tells Amazon GameLift which GameLift VPC to peer with. <br/>
        :return:         """
        pass

    def delete_alias(self, AliasId: str):
        """Deletes an alias. This action removes all record of the alias. Game clients attempting to access a server process using the deleted alias receive an error. To delete an alias, specify the alias ID to be deleted. <ul> <li>  <a>CreateAlias</a>  </li> <li>  <a>ListAliases</a>  </li> <li>  <a>DescribeAlias</a>  </li> <li>  <a>UpdateAlias</a>  </li> <li>  <a>DeleteAlias</a>  </li> <li>  <a>ResolveAlias</a>  </li> </ul>

        :param self:
        :param AliasId:  Unique identifier for a fleet alias. Specify the alias you want to delete.<br/>
        :return:         """
        pass

    def delete_build(self, BuildId: str):
        """Deletes a build. This action permanently deletes the build record and any uploaded build files. To delete a build, specify its ID. Deleting a build does not affect the status of any active fleets using the build, but you can no longer create new fleets with the deleted build.  <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/build-intro.html"> Working with Builds</a>   <b>Related operations</b>  <ul> <li>  <a>CreateBuild</a>  </li> <li>  <a>ListBuilds</a>  </li> <li>  <a>DescribeBuild</a>  </li> <li>  <a>UpdateBuild</a>  </li> <li>  <a>DeleteBuild</a>  </li> </ul>

        :param self:
        :param BuildId:  Unique identifier for a build to delete.<br/>
        :return:         """
        pass

    def delete_fleet(self, FleetId: str):
        """Deletes everything related to a fleet. Before deleting a fleet, you must set the fleet's desired capacity to zero. See <a>UpdateFleetCapacity</a>. If the fleet being deleted has a VPC peering connection, you first need to get a valid authorization (good for 24 hours) by calling <a>CreateVpcPeeringAuthorization</a>. You do not need to explicitly delete the VPC peering connection--this is done as part of the delete fleet process. This action removes the fleet's resources and the fleet record. Once a fleet is deleted, you can no longer use that fleet.  <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/fleets-intro.html"> Working with Fleets</a>.  <b>Related operations</b>  <ul> <li>  <a>CreateFleet</a>  </li> <li>  <a>ListFleets</a>  </li> <li>  <a>DeleteFleet</a>  </li> <li> Describe fleets: <ul> <li>  <a>DescribeFleetAttributes</a>  </li> <li>  <a>DescribeFleetCapacity</a>  </li> <li>  <a>DescribeFleetPortSettings</a>  </li> <li>  <a>DescribeFleetUtilization</a>  </li> <li>  <a>DescribeRuntimeConfiguration</a>  </li> <li>  <a>DescribeEC2InstanceLimits</a>  </li> <li>  <a>DescribeFleetEvents</a>  </li> </ul> </li> <li> Update fleets: <ul> <li>  <a>UpdateFleetAttributes</a>  </li> <li>  <a>UpdateFleetCapacity</a>  </li> <li>  <a>UpdateFleetPortSettings</a>  </li> <li>  <a>UpdateRuntimeConfiguration</a>  </li> </ul> </li> <li> Manage fleet actions: <ul> <li>  <a>StartFleetActions</a>  </li> <li>  <a>StopFleetActions</a>  </li> </ul> </li> </ul>

        :param self:
        :param FleetId:  Unique identifier for a fleet to be deleted.<br/>
        :return:         """
        pass

    def delete_game_session_queue(self, Name: str) -> DeleteGameSessionQueueOutput:
        """Deletes a game session queue. This action means that any <a>StartGameSessionPlacement</a> requests that reference this queue will fail. To delete a queue, specify the queue name. <ul> <li>  <a>CreateGameSessionQueue</a>  </li> <li>  <a>DescribeGameSessionQueues</a>  </li> <li>  <a>UpdateGameSessionQueue</a>  </li> <li>  <a>DeleteGameSessionQueue</a>  </li> </ul>

        :param self:
        :param Name:  Descriptive label that is associated with game session queue. Queue names must be unique within each region.<br/>
        :return:         """
        pass

    def delete_matchmaking_configuration(self, Name: str) -> DeleteMatchmakingConfigurationOutput:
        """Permanently removes a FlexMatch matchmaking configuration. To delete, specify the configuration name. A matchmaking configuration cannot be deleted if it is being used in any active matchmaking tickets.  <b>Related operations</b>  <ul> <li>  <a>CreateMatchmakingConfiguration</a>  </li> <li>  <a>DescribeMatchmakingConfigurations</a>  </li> <li>  <a>UpdateMatchmakingConfiguration</a>  </li> <li>  <a>DeleteMatchmakingConfiguration</a>  </li> <li>  <a>CreateMatchmakingRuleSet</a>  </li> <li>  <a>DescribeMatchmakingRuleSets</a>  </li> <li>  <a>ValidateMatchmakingRuleSet</a>  </li> <li>  <a>DeleteMatchmakingRuleSet</a>  </li> </ul>

        :param self:
        :param Name:  Unique identifier for a matchmaking configuration<br/>
        :return:         """
        pass

    def delete_matchmaking_rule_set(self, Name: str) -> DeleteMatchmakingRuleSetOutput:
        """Deletes an existing matchmaking rule set. To delete the rule set, provide the rule set name. Rule sets cannot be deleted if they are currently being used by a matchmaking configuration.   <b>Learn more</b>  <ul> <li>  <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/match-rulesets.html">Build a Rule Set</a>  </li> </ul>  <b>Related operations</b>  <ul> <li>  <a>CreateMatchmakingConfiguration</a>  </li> <li>  <a>DescribeMatchmakingConfigurations</a>  </li> <li>  <a>UpdateMatchmakingConfiguration</a>  </li> <li>  <a>DeleteMatchmakingConfiguration</a>  </li> <li>  <a>CreateMatchmakingRuleSet</a>  </li> <li>  <a>DescribeMatchmakingRuleSets</a>  </li> <li>  <a>ValidateMatchmakingRuleSet</a>  </li> <li>  <a>DeleteMatchmakingRuleSet</a>  </li> </ul>

        :param self:
        :param Name:  Unique identifier for a matchmaking rule set to be deleted. (Note: The rule set name is different from the optional "name" field in the rule set body.) <br/>
        :return:         """
        pass

    def delete_scaling_policy(self, FleetId: str, Name: str):
        """Deletes a fleet scaling policy. This action means that the policy is no longer in force and removes all record of it. To delete a scaling policy, specify both the scaling policy name and the fleet ID it is associated with. To temporarily suspend scaling policies, call <a>StopFleetActions</a>. This operation suspends all policies for the fleet. <ul> <li>  <a>DescribeFleetCapacity</a>  </li> <li>  <a>UpdateFleetCapacity</a>  </li> <li>  <a>DescribeEC2InstanceLimits</a>  </li> <li> Manage scaling policies: <ul> <li>  <a>PutScalingPolicy</a> (auto-scaling) </li> <li>  <a>DescribeScalingPolicies</a> (auto-scaling) </li> <li>  <a>DeleteScalingPolicy</a> (auto-scaling) </li> </ul> </li> <li> Manage fleet actions: <ul> <li>  <a>StartFleetActions</a>  </li> <li>  <a>StopFleetActions</a>  </li> </ul> </li> </ul>

        :param self:
        :param FleetId:  Unique identifier for a fleet to be deleted.<br/>Name:  Descriptive label that is associated with a scaling policy. Policy names do not need to be unique.<br/>
        :return:         """
        pass

    def delete_script(self, ScriptId: str):
        """Deletes a Realtime script. This action permanently deletes the script record. If script files were uploaded, they are also deleted (files stored in an S3 bucket are not deleted).  To delete a script, specify the script ID. Before deleting a script, be sure to terminate all fleets that are deployed with the script being deleted. Fleet instances periodically check for script updates, and if the script record no longer exists, the instance will go into an error state and be unable to host game sessions.  <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/realtime-intro.html">Amazon GameLift Realtime Servers</a>   <b>Related operations</b>  <ul> <li>  <a>CreateScript</a>  </li> <li>  <a>ListScripts</a>  </li> <li>  <a>DescribeScript</a>  </li> <li>  <a>UpdateScript</a>  </li> <li>  <a>DeleteScript</a>  </li> </ul>

        :param self:
        :param ScriptId:  Unique identifier for a Realtime script to delete.<br/>
        :return:         """
        pass

    def delete_vpc_peering_authorization(self, PeerVpcId: str, GameLiftAwsAccountId: str) -> DeleteVpcPeeringAuthorizationOutput:
        """Cancels a pending VPC peering authorization for the specified VPC. If you need to delete an existing VPC peering connection, call <a>DeleteVpcPeeringConnection</a>.  <ul> <li>  <a>CreateVpcPeeringAuthorization</a>  </li> <li>  <a>DescribeVpcPeeringAuthorizations</a>  </li> <li>  <a>DeleteVpcPeeringAuthorization</a>  </li> <li>  <a>CreateVpcPeeringConnection</a>  </li> <li>  <a>DescribeVpcPeeringConnections</a>  </li> <li>  <a>DeleteVpcPeeringConnection</a>  </li> </ul>

        :param self:
        :param PeerVpcId:  Unique identifier for a VPC with resources to be accessed by your Amazon GameLift fleet. The VPC must be in the same region where your fleet is deployed. Look up a VPC ID using the <a href="https://console.aws.amazon.com/vpc/">VPC Dashboard</a> in the AWS Management Console. Learn more about VPC peering in <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/vpc-peering.html">VPC Peering with Amazon GameLift Fleets</a>.<br/>GameLiftAwsAccountId:  Unique identifier for the AWS account that you use to manage your Amazon GameLift fleet. You can find your Account ID in the AWS Management Console under account settings.<br/>
        :return:         """
        pass

    def delete_vpc_peering_connection(self, VpcPeeringConnectionId: str, FleetId: str) -> DeleteVpcPeeringConnectionOutput:
        """Removes a VPC peering connection. To delete the connection, you must have a valid authorization for the VPC peering connection that you want to delete. You can check for an authorization by calling <a>DescribeVpcPeeringAuthorizations</a> or request a new one using <a>CreateVpcPeeringAuthorization</a>.  Once a valid authorization exists, call this operation from the AWS account that is used to manage the Amazon GameLift fleets. Identify the connection to delete by the connection ID and fleet ID. If successful, the connection is removed.  <ul> <li>  <a>CreateVpcPeeringAuthorization</a>  </li> <li>  <a>DescribeVpcPeeringAuthorizations</a>  </li> <li>  <a>DeleteVpcPeeringAuthorization</a>  </li> <li>  <a>CreateVpcPeeringConnection</a>  </li> <li>  <a>DescribeVpcPeeringConnections</a>  </li> <li>  <a>DeleteVpcPeeringConnection</a>  </li> </ul>

        :param self:
        :param VpcPeeringConnectionId:  Unique identifier for a VPC peering connection. This value is included in the <a>VpcPeeringConnection</a> object, which can be retrieved by calling <a>DescribeVpcPeeringConnections</a>.<br/>FleetId:  Unique identifier for a fleet. This value must match the fleet ID referenced in the VPC peering connection record.<br/>
        :return:         """
        pass

    def describe_alias(self, AliasId: str) -> DescribeAliasOutput:
        """Retrieves properties for an alias. This operation returns all alias metadata and settings. To get an alias's target fleet ID only, use <code>ResolveAlias</code>.  To get alias properties, specify the alias ID. If successful, the requested alias record is returned. <ul> <li>  <a>CreateAlias</a>  </li> <li>  <a>ListAliases</a>  </li> <li>  <a>DescribeAlias</a>  </li> <li>  <a>UpdateAlias</a>  </li> <li>  <a>DeleteAlias</a>  </li> <li>  <a>ResolveAlias</a>  </li> </ul>

        :param self:
        :param AliasId:  Unique identifier for a fleet alias. Specify the alias you want to retrieve.<br/>
        :return: Alias:  Object that contains the requested alias.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdatedTime</i> Time stamp indicating when this data object was last modified. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> Time stamp indicating when this data object was created. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoutingStrategy</i> Alias configuration for the alias, including routing type and settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RoutingStrategy</b>: Alias configuration for the alias, including routing type and settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> Human-readable description of an alias.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AliasArn</i> Unique identifier for an alias; alias ARNs are unique across all regions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Descriptive label that is associated with an alias. Alias names do not need to be unique.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AliasId</i> Unique identifier for an alias; alias IDs are unique within a region.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> Message text to be used with a terminal routing strategy.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FleetId</i> Unique identifier for a fleet that the alias points to.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> Type of routing strategy. Possible routing types include the following: <ul> <li>  <b>SIMPLE</b> -- The alias resolves to one specific fleet. Use this type when routing to active fleets. </li> <li>  <b>TERMINAL</b> -- The alias does not resolve to a fleet but instead can be used to display a message to the user. A terminal alias throws a TerminalRoutingStrategyException with the <a>RoutingStrategy</a> message embedded. </li> </ul><br/>        """
        pass

    def describe_build(self, BuildId: str) -> DescribeBuildOutput:
        """Retrieves properties for a build. To request a build record, specify a build ID. If successful, an object containing the build properties is returned.  <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/build-intro.html"> Working with Builds</a>   <b>Related operations</b>  <ul> <li>  <a>CreateBuild</a>  </li> <li>  <a>ListBuilds</a>  </li> <li>  <a>DescribeBuild</a>  </li> <li>  <a>UpdateBuild</a>  </li> <li>  <a>DeleteBuild</a>  </li> </ul>

        :param self:
        :param BuildId:  Unique identifier for a build to retrieve properties for.<br/>
        :return: Build:  Set of properties describing the requested build.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> Time stamp indicating when this data object was created. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OperatingSystem</i> Operating system that the game server binaries are built to run on. This value determines the type of fleet resources that you can use for this build.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SizeOnDisk</i> File size of the uploaded game build, expressed in bytes. When the build status is <code>INITIALIZED</code>, this value is 0.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Current status of the build. Possible build statuses include the following: <ul> <li>  <b>INITIALIZED</b> -- A new build has been defined, but no files have been uploaded. You cannot create fleets for builds that are in this status. When a build is successfully created, the build status is set to this value.  </li> <li>  <b>READY</b> -- The game build has been successfully uploaded. You can now create new fleets for this build. </li> <li>  <b>FAILED</b> -- The game build upload failed. You cannot create new fleets for this build.  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> Version that is associated with a build or script. Version strings do not need to be unique. This value can be set using <a>CreateBuild</a> or <a>UpdateBuild</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Descriptive label that is associated with a build. Build names do not need to be unique. It can be set using <a>CreateBuild</a> or <a>UpdateBuild</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BuildId</i> Unique identifier for a build.<br/>        """
        pass

    def describe_ec2_instance_limits(self, EC2InstanceType: str=None) -> DescribeEC2InstanceLimitsOutput:
        """Retrieves the following information for the specified EC2 instance type: <ul> <li> maximum number of instances allowed per AWS account (service limit) </li> <li> current usage level for the AWS account </li> </ul> Service limits vary depending on region. Available regions for Amazon GameLift can be found in the AWS Management Console for Amazon GameLift (see the drop-down list in the upper right corner).  <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/fleets-intro.html"> Working with Fleets</a>.  <b>Related operations</b>  <ul> <li>  <a>CreateFleet</a>  </li> <li>  <a>ListFleets</a>  </li> <li>  <a>DeleteFleet</a>  </li> <li> Describe fleets: <ul> <li>  <a>DescribeFleetAttributes</a>  </li> <li>  <a>DescribeFleetCapacity</a>  </li> <li>  <a>DescribeFleetPortSettings</a>  </li> <li>  <a>DescribeFleetUtilization</a>  </li> <li>  <a>DescribeRuntimeConfiguration</a>  </li> <li>  <a>DescribeEC2InstanceLimits</a>  </li> <li>  <a>DescribeFleetEvents</a>  </li> </ul> </li> <li> Update fleets: <ul> <li>  <a>UpdateFleetAttributes</a>  </li> <li>  <a>UpdateFleetCapacity</a>  </li> <li>  <a>UpdateFleetPortSettings</a>  </li> <li>  <a>UpdateRuntimeConfiguration</a>  </li> </ul> </li> <li> Manage fleet actions: <ul> <li>  <a>StartFleetActions</a>  </li> <li>  <a>StopFleetActions</a>  </li> </ul> </li> </ul>

        :param self:
        :param EC2InstanceType:  Name of an EC2 instance type that is supported in Amazon GameLift. A fleet instance type determines the computing resources of each instance in the fleet, including CPU, memory, storage, and networking capacity. Amazon GameLift supports the following EC2 instance types. See <a href="http://aws.amazon.com/ec2/instance-types/">Amazon EC2 Instance Types</a> for detailed descriptions. Leave this parameter blank to retrieve limits for all types.<br/>
        :return: EC2InstanceLimits:  Object that contains the maximum number of instances for the specified instance type.<br/>        """
        pass

    def describe_fleet_attributes(self, FleetIds: List[FleetId]=None, Limit: int=None, NextToken: str=None) -> DescribeFleetAttributesOutput:
        """Retrieves fleet properties, including metadata, status, and configuration, for one or more fleets. You can request attributes for all fleets, or specify a list of one or more fleet IDs. When requesting multiple fleets, use the pagination parameters to retrieve results as a set of sequential pages. If successful, a <a>FleetAttributes</a> object is returned for each requested fleet ID. When specifying a list of fleet IDs, attribute objects are returned only for fleets that currently exist.  <note> Some API actions may limit the number of fleet IDs allowed in one request. If a request exceeds this limit, the request fails and the error message includes the maximum allowed. </note>  <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/fleets-intro.html"> Working with Fleets</a>.  <b>Related operations</b>  <ul> <li>  <a>CreateFleet</a>  </li> <li>  <a>ListFleets</a>  </li> <li>  <a>DeleteFleet</a>  </li> <li> Describe fleets: <ul> <li>  <a>DescribeFleetAttributes</a>  </li> <li>  <a>DescribeFleetCapacity</a>  </li> <li>  <a>DescribeFleetPortSettings</a>  </li> <li>  <a>DescribeFleetUtilization</a>  </li> <li>  <a>DescribeRuntimeConfiguration</a>  </li> <li>  <a>DescribeEC2InstanceLimits</a>  </li> <li>  <a>DescribeFleetEvents</a>  </li> </ul> </li> <li> Update fleets: <ul> <li>  <a>UpdateFleetAttributes</a>  </li> <li>  <a>UpdateFleetCapacity</a>  </li> <li>  <a>UpdateFleetPortSettings</a>  </li> <li>  <a>UpdateRuntimeConfiguration</a>  </li> </ul> </li> <li> Manage fleet actions: <ul> <li>  <a>StartFleetActions</a>  </li> <li>  <a>StopFleetActions</a>  </li> </ul> </li> </ul>

        :param self:
        :param NextToken:  Token that indicates the start of the next sequential page of results. Use the token that is returned with a previous call to this action. To start at the beginning of the result set, do not specify a value. This parameter is ignored when the request specifies one or a list of fleet IDs.<br/>Limit:  Maximum number of results to return. Use this parameter with <code>NextToken</code> to get results as a set of sequential pages. This parameter is ignored when the request specifies one or a list of fleet IDs.<br/>FleetIds:  Unique identifier for a fleet(s) to retrieve attributes for. To request attributes for all fleets, leave this parameter empty.<br/>
        :return: NextToken:  Token that indicates where to resume retrieving results on the next call to this action. If no token is returned, these results represent the end of the list.<br/>FleetAttributes:  Collection of objects containing attribute metadata for each requested fleet ID.<br/>        """
        pass

    def describe_fleet_capacity(self, FleetIds: List[FleetId]=None, Limit: int=None, NextToken: str=None) -> DescribeFleetCapacityOutput:
        """Retrieves the current status of fleet capacity for one or more fleets. This information includes the number of instances that have been requested for the fleet and the number currently active. You can request capacity for all fleets, or specify a list of one or more fleet IDs. When requesting multiple fleets, use the pagination parameters to retrieve results as a set of sequential pages. If successful, a <a>FleetCapacity</a> object is returned for each requested fleet ID. When specifying a list of fleet IDs, attribute objects are returned only for fleets that currently exist.  <note> Some API actions may limit the number of fleet IDs allowed in one request. If a request exceeds this limit, the request fails and the error message includes the maximum allowed. </note>  <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/fleets-intro.html"> Working with Fleets</a>.  <b>Related operations</b>  <ul> <li>  <a>CreateFleet</a>  </li> <li>  <a>ListFleets</a>  </li> <li>  <a>DeleteFleet</a>  </li> <li> Describe fleets: <ul> <li>  <a>DescribeFleetAttributes</a>  </li> <li>  <a>DescribeFleetCapacity</a>  </li> <li>  <a>DescribeFleetPortSettings</a>  </li> <li>  <a>DescribeFleetUtilization</a>  </li> <li>  <a>DescribeRuntimeConfiguration</a>  </li> <li>  <a>DescribeEC2InstanceLimits</a>  </li> <li>  <a>DescribeFleetEvents</a>  </li> </ul> </li> <li> Update fleets: <ul> <li>  <a>UpdateFleetAttributes</a>  </li> <li>  <a>UpdateFleetCapacity</a>  </li> <li>  <a>UpdateFleetPortSettings</a>  </li> <li>  <a>UpdateRuntimeConfiguration</a>  </li> </ul> </li> <li> Manage fleet actions: <ul> <li>  <a>StartFleetActions</a>  </li> <li>  <a>StopFleetActions</a>  </li> </ul> </li> </ul>

        :param self:
        :param NextToken:  Token that indicates the start of the next sequential page of results. Use the token that is returned with a previous call to this action. To start at the beginning of the result set, do not specify a value. This parameter is ignored when the request specifies one or a list of fleet IDs.<br/>Limit:  Maximum number of results to return. Use this parameter with <code>NextToken</code> to get results as a set of sequential pages. This parameter is ignored when the request specifies one or a list of fleet IDs.<br/>FleetIds:  Unique identifier for a fleet(s) to retrieve capacity information for. To request capacity information for all fleets, leave this parameter empty.<br/>
        :return: NextToken:  Token that indicates where to resume retrieving results on the next call to this action. If no token is returned, these results represent the end of the list.<br/>FleetCapacity:  Collection of objects containing capacity information for each requested fleet ID. Leave this parameter empty to retrieve capacity information for all fleets.<br/>        """
        pass

    def describe_fleet_events(self, FleetId: str, StartTime: Timestamp=None, EndTime: Timestamp=None, Limit: int=None, NextToken: str=None) -> DescribeFleetEventsOutput:
        """Retrieves entries from the specified fleet's event log. You can specify a time range to limit the result set. Use the pagination parameters to retrieve results as a set of sequential pages. If successful, a collection of event log entries matching the request are returned.  <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/fleets-intro.html"> Working with Fleets</a>.  <b>Related operations</b>  <ul> <li>  <a>CreateFleet</a>  </li> <li>  <a>ListFleets</a>  </li> <li>  <a>DeleteFleet</a>  </li> <li> Describe fleets: <ul> <li>  <a>DescribeFleetAttributes</a>  </li> <li>  <a>DescribeFleetCapacity</a>  </li> <li>  <a>DescribeFleetPortSettings</a>  </li> <li>  <a>DescribeFleetUtilization</a>  </li> <li>  <a>DescribeRuntimeConfiguration</a>  </li> <li>  <a>DescribeEC2InstanceLimits</a>  </li> <li>  <a>DescribeFleetEvents</a>  </li> </ul> </li> <li> Update fleets: <ul> <li>  <a>UpdateFleetAttributes</a>  </li> <li>  <a>UpdateFleetCapacity</a>  </li> <li>  <a>UpdateFleetPortSettings</a>  </li> <li>  <a>UpdateRuntimeConfiguration</a>  </li> </ul> </li> <li> Manage fleet actions: <ul> <li>  <a>StartFleetActions</a>  </li> <li>  <a>StopFleetActions</a>  </li> </ul> </li> </ul>

        :param self:
        :param NextToken:  Token that indicates the start of the next sequential page of results. Use the token that is returned with a previous call to this action. To start at the beginning of the result set, do not specify a value.<br/>Limit:  Maximum number of results to return. Use this parameter with <code>NextToken</code> to get results as a set of sequential pages.<br/>EndTime:  Most recent date to retrieve event logs for. If no end time is specified, this call returns entries from the specified start time up to the present. Format is a number expressed in Unix time as milliseconds (ex: "1469498468.057").<br/>StartTime:  Earliest date to retrieve event logs for. If no start time is specified, this call returns entries starting from when the fleet was created to the specified end time. Format is a number expressed in Unix time as milliseconds (ex: "1469498468.057").<br/>FleetId:  Unique identifier for a fleet to get event logs for.<br/>
        :return: NextToken:  Token that indicates where to resume retrieving results on the next call to this action. If no token is returned, these results represent the end of the list.<br/>Events:  Collection of objects containing event log entries for the specified fleet.<br/>        """
        pass

    def describe_fleet_port_settings(self, FleetId: str) -> DescribeFleetPortSettingsOutput:
        """Retrieves the inbound connection permissions for a fleet. Connection permissions include a range of IP addresses and port settings that incoming traffic can use to access server processes in the fleet. To get a fleet's inbound connection permissions, specify a fleet ID. If successful, a collection of <a>IpPermission</a> objects is returned for the requested fleet ID. If the requested fleet has been deleted, the result set is empty.  <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/fleets-intro.html"> Working with Fleets</a>.  <b>Related operations</b>  <ul> <li>  <a>CreateFleet</a>  </li> <li>  <a>ListFleets</a>  </li> <li>  <a>DeleteFleet</a>  </li> <li> Describe fleets: <ul> <li>  <a>DescribeFleetAttributes</a>  </li> <li>  <a>DescribeFleetCapacity</a>  </li> <li>  <a>DescribeFleetPortSettings</a>  </li> <li>  <a>DescribeFleetUtilization</a>  </li> <li>  <a>DescribeRuntimeConfiguration</a>  </li> <li>  <a>DescribeEC2InstanceLimits</a>  </li> <li>  <a>DescribeFleetEvents</a>  </li> </ul> </li> <li> Update fleets: <ul> <li>  <a>UpdateFleetAttributes</a>  </li> <li>  <a>UpdateFleetCapacity</a>  </li> <li>  <a>UpdateFleetPortSettings</a>  </li> <li>  <a>UpdateRuntimeConfiguration</a>  </li> </ul> </li> <li> Manage fleet actions: <ul> <li>  <a>StartFleetActions</a>  </li> <li>  <a>StopFleetActions</a>  </li> </ul> </li> </ul>

        :param self:
        :param FleetId:  Unique identifier for a fleet to retrieve port settings for.<br/>
        :return: InboundPermissions:  Object that contains port settings for the requested fleet ID.<br/>        """
        pass

    def describe_fleet_utilization(self, FleetIds: List[FleetId]=None, Limit: int=None, NextToken: str=None) -> DescribeFleetUtilizationOutput:
        """Retrieves utilization statistics for one or more fleets. You can request utilization data for all fleets, or specify a list of one or more fleet IDs. When requesting multiple fleets, use the pagination parameters to retrieve results as a set of sequential pages. If successful, a <a>FleetUtilization</a> object is returned for each requested fleet ID. When specifying a list of fleet IDs, utilization objects are returned only for fleets that currently exist.  <note> Some API actions may limit the number of fleet IDs allowed in one request. If a request exceeds this limit, the request fails and the error message includes the maximum allowed. </note>  <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/fleets-intro.html"> Working with Fleets</a>.  <b>Related operations</b>  <ul> <li>  <a>CreateFleet</a>  </li> <li>  <a>ListFleets</a>  </li> <li>  <a>DeleteFleet</a>  </li> <li> Describe fleets: <ul> <li>  <a>DescribeFleetAttributes</a>  </li> <li>  <a>DescribeFleetCapacity</a>  </li> <li>  <a>DescribeFleetPortSettings</a>  </li> <li>  <a>DescribeFleetUtilization</a>  </li> <li>  <a>DescribeRuntimeConfiguration</a>  </li> <li>  <a>DescribeEC2InstanceLimits</a>  </li> <li>  <a>DescribeFleetEvents</a>  </li> </ul> </li> <li> Update fleets: <ul> <li>  <a>UpdateFleetAttributes</a>  </li> <li>  <a>UpdateFleetCapacity</a>  </li> <li>  <a>UpdateFleetPortSettings</a>  </li> <li>  <a>UpdateRuntimeConfiguration</a>  </li> </ul> </li> <li> Manage fleet actions: <ul> <li>  <a>StartFleetActions</a>  </li> <li>  <a>StopFleetActions</a>  </li> </ul> </li> </ul>

        :param self:
        :param NextToken:  Token that indicates the start of the next sequential page of results. Use the token that is returned with a previous call to this action. To start at the beginning of the result set, do not specify a value. This parameter is ignored when the request specifies one or a list of fleet IDs.<br/>Limit:  Maximum number of results to return. Use this parameter with <code>NextToken</code> to get results as a set of sequential pages. This parameter is ignored when the request specifies one or a list of fleet IDs.<br/>FleetIds:  Unique identifier for a fleet(s) to retrieve utilization data for. To request utilization data for all fleets, leave this parameter empty.<br/>
        :return: NextToken:  Token that indicates where to resume retrieving results on the next call to this action. If no token is returned, these results represent the end of the list.<br/>FleetUtilization:  Collection of objects containing utilization information for each requested fleet ID.<br/>        """
        pass

    def describe_game_session_details(self, FleetId: str=None, GameSessionId: str=None, AliasId: str=None, StatusFilter: str=None, Limit: int=None, NextToken: str=None) -> DescribeGameSessionDetailsOutput:
        """Retrieves properties, including the protection policy in force, for one or more game sessions. This action can be used in several ways: (1) provide a <code>GameSessionId</code> or <code>GameSessionArn</code> to request details for a specific game session; (2) provide either a <code>FleetId</code> or an <code>AliasId</code> to request properties for all game sessions running on a fleet.  To get game session record(s), specify just one of the following: game session ID, fleet ID, or alias ID. You can filter this request by game session status. Use the pagination parameters to retrieve results as a set of sequential pages. If successful, a <a>GameSessionDetail</a> object is returned for each session matching the request. <ul> <li>  <a>CreateGameSession</a>  </li> <li>  <a>DescribeGameSessions</a>  </li> <li>  <a>DescribeGameSessionDetails</a>  </li> <li>  <a>SearchGameSessions</a>  </li> <li>  <a>UpdateGameSession</a>  </li> <li>  <a>GetGameSessionLogUrl</a>  </li> <li> Game session placements <ul> <li>  <a>StartGameSessionPlacement</a>  </li> <li>  <a>DescribeGameSessionPlacement</a>  </li> <li>  <a>StopGameSessionPlacement</a>  </li> </ul> </li> </ul>

        :param self:
        :param NextToken:  Token that indicates the start of the next sequential page of results. Use the token that is returned with a previous call to this action. To start at the beginning of the result set, do not specify a value.<br/>Limit:  Maximum number of results to return. Use this parameter with <code>NextToken</code> to get results as a set of sequential pages.<br/>StatusFilter:  Game session status to filter results on. Possible game session statuses include <code>ACTIVE</code>, <code>TERMINATED</code>, <code>ACTIVATING</code> and <code>TERMINATING</code> (the last two are transitory). <br/>AliasId:  Unique identifier for an alias associated with the fleet to retrieve all game sessions for.<br/>GameSessionId:  Unique identifier for the game session to retrieve.<br/>FleetId:  Unique identifier for a fleet to retrieve all game sessions active on the fleet.<br/>
        :return: NextToken:  Token that indicates where to resume retrieving results on the next call to this action. If no token is returned, these results represent the end of the list.<br/>GameSessionDetails:  Collection of objects containing game session properties and the protection policy currently in force for each session matching the request.<br/>        """
        pass

    def describe_game_session_placement(self, PlacementId: str) -> DescribeGameSessionPlacementOutput:
        """Retrieves properties and current status of a game session placement request. To get game session placement details, specify the placement ID. If successful, a <a>GameSessionPlacement</a> object is returned. <ul> <li>  <a>CreateGameSession</a>  </li> <li>  <a>DescribeGameSessions</a>  </li> <li>  <a>DescribeGameSessionDetails</a>  </li> <li>  <a>SearchGameSessions</a>  </li> <li>  <a>UpdateGameSession</a>  </li> <li>  <a>GetGameSessionLogUrl</a>  </li> <li> Game session placements <ul> <li>  <a>StartGameSessionPlacement</a>  </li> <li>  <a>DescribeGameSessionPlacement</a>  </li> <li>  <a>StopGameSessionPlacement</a>  </li> </ul> </li> </ul>

        :param self:
        :param PlacementId:  Unique identifier for a game session placement to retrieve.<br/>
        :return: GameSessionPlacement:  Object that describes the requested game session placement.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MatchmakerData</i> Information on the matchmaking process for this game. Data is in JSON syntax, formatted as a string. It identifies the matchmaking configuration used to create the match, and contains data on all players assigned to the match, including player attributes and team assignments. For more details on matchmaker data, see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/match-server.html#match-server-data">Match Data</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionData</i> Set of custom game session properties, formatted as a single string value. This data is passed to a game server process in the <a>GameSession</a> object with a request to start a new game session (see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-sdk-server-api.html#gamelift-sdk-server-startsession">Start a Game Session</a>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlacedPlayerSessions</i> Collection of information on player sessions created in response to the game session placement request. These player sessions are created only once a new game session is successfully placed (placement status is <code>FULFILLED</code>). This information includes the player ID (as provided in the placement request) and the corresponding player session ID. Retrieve full player sessions by calling <a>DescribePlayerSessions</a> with the player session ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Port number for the game session. To connect to a Amazon GameLift game server, an app needs both the IP address and port number. This value is set once the new game session is placed (placement status is <code>FULFILLED</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IpAddress</i> IP address of the game session. To connect to a Amazon GameLift game server, an app needs both the IP address and port number. This value is set once the new game session is placed (placement status is <code>FULFILLED</code>). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> Time stamp indicating when this request was completed, canceled, or timed out.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTime</i> Time stamp indicating when this request was placed in the queue. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlayerLatencies</i> Set of values, expressed in milliseconds, indicating the amount of latency that a player experiences when connected to AWS regions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionRegion</i> Name of the region where the game session created by this placement request is running. This value is set once the new game session is placed (placement status is <code>FULFILLED</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionArn</i> Identifier for the game session created by this placement request. This value is set once the new game session is placed (placement status is <code>FULFILLED</code>). This identifier is unique across all regions. You can use this value as a <code>GameSessionId</code> value as needed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionId</i> Unique identifier for the game session. This value is set once the new game session is placed (placement status is <code>FULFILLED</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionName</i> Descriptive label that is associated with a game session. Session names do not need to be unique.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumPlayerSessionCount</i> Maximum number of players that can be connected simultaneously to the game session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameProperties</i> Set of custom properties for a game session, formatted as key:value pairs. These properties are passed to a game server process in the <a>GameSession</a> object with a request to start a new game session (see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-sdk-server-api.html#gamelift-sdk-server-startsession">Start a Game Session</a>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Current status of the game session placement request. <ul> <li>  <b>PENDING</b> -- The placement request is currently in the queue waiting to be processed. </li> <li>  <b>FULFILLED</b> -- A new game session and player sessions (if requested) have been successfully created. Values for <i>GameSessionArn</i> and <i>GameSessionRegion</i> are available.  </li> <li>  <b>CANCELLED</b> -- The placement request was canceled with a call to <a>StopGameSessionPlacement</a>. </li> <li>  <b>TIMED_OUT</b> -- A new game session was not successfully created before the time limit expired. You can resubmit the placement request as needed. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionQueueName</i> Descriptive label that is associated with game session queue. Queue names must be unique within each region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlacementId</i> Unique identifier for a game session placement.<br/>        """
        pass

    def describe_game_session_queues(self, Names: List[GameSessionQueueName]=None, Limit: int=None, NextToken: str=None) -> DescribeGameSessionQueuesOutput:
        """Retrieves the properties for one or more game session queues. When requesting multiple queues, use the pagination parameters to retrieve results as a set of sequential pages. If successful, a <a>GameSessionQueue</a> object is returned for each requested queue. When specifying a list of queues, objects are returned only for queues that currently exist in the region. <ul> <li>  <a>CreateGameSessionQueue</a>  </li> <li>  <a>DescribeGameSessionQueues</a>  </li> <li>  <a>UpdateGameSessionQueue</a>  </li> <li>  <a>DeleteGameSessionQueue</a>  </li> </ul>

        :param self:
        :param NextToken:  Token that indicates the start of the next sequential page of results. Use the token that is returned with a previous call to this action. To start at the beginning of the result set, do not specify a value.<br/>Limit:  Maximum number of results to return. Use this parameter with <code>NextToken</code> to get results as a set of sequential pages.<br/>Names:  List of queue names to retrieve information for. To request settings for all queues, leave this parameter empty.<br/>
        :return: NextToken:  Token that indicates where to resume retrieving results on the next call to this action. If no token is returned, these results represent the end of the list.<br/>GameSessionQueues:  Collection of objects that describes the requested game session queues.<br/>        """
        pass

    def describe_game_sessions(self, FleetId: str=None, GameSessionId: str=None, AliasId: str=None, StatusFilter: str=None, Limit: int=None, NextToken: str=None) -> DescribeGameSessionsOutput:
        """Retrieves a set of one or more game sessions. Request a specific game session or request all game sessions on a fleet. Alternatively, use <a>SearchGameSessions</a> to request a set of active game sessions that are filtered by certain criteria. To retrieve protection policy settings for game sessions, use <a>DescribeGameSessionDetails</a>. To get game sessions, specify one of the following: game session ID, fleet ID, or alias ID. You can filter this request by game session status. Use the pagination parameters to retrieve results as a set of sequential pages. If successful, a <a>GameSession</a> object is returned for each game session matching the request.  <i>Available in Amazon GameLift Local.</i>  <ul> <li>  <a>CreateGameSession</a>  </li> <li>  <a>DescribeGameSessions</a>  </li> <li>  <a>DescribeGameSessionDetails</a>  </li> <li>  <a>SearchGameSessions</a>  </li> <li>  <a>UpdateGameSession</a>  </li> <li>  <a>GetGameSessionLogUrl</a>  </li> <li> Game session placements <ul> <li>  <a>StartGameSessionPlacement</a>  </li> <li>  <a>DescribeGameSessionPlacement</a>  </li> <li>  <a>StopGameSessionPlacement</a>  </li> </ul> </li> </ul>

        :param self:
        :param NextToken:  Token that indicates the start of the next sequential page of results. Use the token that is returned with a previous call to this action. To start at the beginning of the result set, do not specify a value.<br/>Limit:  Maximum number of results to return. Use this parameter with <code>NextToken</code> to get results as a set of sequential pages.<br/>StatusFilter:  Game session status to filter results on. Possible game session statuses include <code>ACTIVE</code>, <code>TERMINATED</code>, <code>ACTIVATING</code>, and <code>TERMINATING</code> (the last two are transitory). <br/>AliasId:  Unique identifier for an alias associated with the fleet to retrieve all game sessions for. <br/>GameSessionId:  Unique identifier for the game session to retrieve. You can use either a <code>GameSessionId</code> or <code>GameSessionArn</code> value. <br/>FleetId:  Unique identifier for a fleet to retrieve all game sessions for.<br/>
        :return: NextToken:  Token that indicates where to resume retrieving results on the next call to this action. If no token is returned, these results represent the end of the list.<br/>GameSessions:  Collection of objects containing game session properties for each session matching the request.<br/>        """
        pass

    def describe_instances(self, FleetId: str, InstanceId: str=None, Limit: int=None, NextToken: str=None) -> DescribeInstancesOutput:
        """Retrieves information about a fleet's instances, including instance IDs. Use this action to get details on all instances in the fleet or get details on one specific instance. To get a specific instance, specify fleet ID and instance ID. To get all instances in a fleet, specify a fleet ID only. Use the pagination parameters to retrieve results as a set of sequential pages. If successful, an <a>Instance</a> object is returned for each result.

        :param self:
        :param NextToken:  Token that indicates the start of the next sequential page of results. Use the token that is returned with a previous call to this action. To start at the beginning of the result set, do not specify a value.<br/>Limit:  Maximum number of results to return. Use this parameter with <code>NextToken</code> to get results as a set of sequential pages.<br/>InstanceId:  Unique identifier for an instance to retrieve. Specify an instance ID or leave blank to retrieve all instances in the fleet.<br/>FleetId:  Unique identifier for a fleet to retrieve instance information for.<br/>
        :return: NextToken:  Token that indicates where to resume retrieving results on the next call to this action. If no token is returned, these results represent the end of the list.<br/>Instances:  Collection of objects containing properties for each instance returned.<br/>        """
        pass

    def describe_matchmaking(self, TicketIds: List[MatchmakingIdStringModel]) -> DescribeMatchmakingOutput:
        """Retrieves one or more matchmaking tickets. Use this operation to retrieve ticket information, including status and--once a successful match is made--acquire connection information for the resulting new game session.  You can use this operation to track the progress of matchmaking requests (through polling) as an alternative to using event notifications. See more details on tracking matchmaking requests through polling or notifications in <a>StartMatchmaking</a>.  To request matchmaking tickets, provide a list of up to 10 ticket IDs. If the request is successful, a ticket object is returned for each requested ID that currently exists.  <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/match-client.html"> Add FlexMatch to a Game Client</a>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguidematch-notification.html"> Set Up FlexMatch Event Notification</a>   <b>Related operations</b>  <ul> <li>  <a>StartMatchmaking</a>  </li> <li>  <a>DescribeMatchmaking</a>  </li> <li>  <a>StopMatchmaking</a>  </li> <li>  <a>AcceptMatch</a>  </li> <li>  <a>StartMatchBackfill</a>  </li> </ul>

        :param self:
        :param TicketIds:  Unique identifier for a matchmaking ticket. You can include up to 10 ID values. <br/>
        :return: TicketList:  Collection of existing matchmaking ticket objects matching the request.<br/>        """
        pass

    def describe_matchmaking_configurations(self, Names: List[MatchmakingIdStringModel]=None, RuleSetName: str=None, Limit: int=None, NextToken: str=None) -> DescribeMatchmakingConfigurationsOutput:
        """Retrieves the details of FlexMatch matchmaking configurations. With this operation, you have the following options: (1) retrieve all existing configurations, (2) provide the names of one or more configurations to retrieve, or (3) retrieve all configurations that use a specified rule set name. When requesting multiple items, use the pagination parameters to retrieve results as a set of sequential pages. If successful, a configuration is returned for each requested name. When specifying a list of names, only configurations that currently exist are returned.   <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/matchmaker-build.html"> Setting Up FlexMatch Matchmakers</a>   <b>Related operations</b>  <ul> <li>  <a>CreateMatchmakingConfiguration</a>  </li> <li>  <a>DescribeMatchmakingConfigurations</a>  </li> <li>  <a>UpdateMatchmakingConfiguration</a>  </li> <li>  <a>DeleteMatchmakingConfiguration</a>  </li> <li>  <a>CreateMatchmakingRuleSet</a>  </li> <li>  <a>DescribeMatchmakingRuleSets</a>  </li> <li>  <a>ValidateMatchmakingRuleSet</a>  </li> <li>  <a>DeleteMatchmakingRuleSet</a>  </li> </ul>

        :param self:
        :param NextToken:  Token that indicates the start of the next sequential page of results. Use the token that is returned with a previous call to this action. To start at the beginning of the result set, do not specify a value.<br/>Limit:  Maximum number of results to return. Use this parameter with <code>NextToken</code> to get results as a set of sequential pages. This parameter is limited to 10.<br/>RuleSetName:  Unique identifier for a matchmaking rule set. Use this parameter to retrieve all matchmaking configurations that use this rule set.<br/>Names:  Unique identifier for a matchmaking configuration(s) to retrieve. To request all existing configurations, leave this parameter empty.<br/>
        :return: NextToken:  Token that indicates where to resume retrieving results on the next call to this action. If no token is returned, these results represent the end of the list.<br/>Configurations:  Collection of requested matchmaking configuration objects.<br/>        """
        pass

    def describe_matchmaking_rule_sets(self, Names: List[MatchmakingIdStringModel]=None, Limit: int=None, NextToken: str=None) -> DescribeMatchmakingRuleSetsOutput:
        """Retrieves the details for FlexMatch matchmaking rule sets. You can request all existing rule sets for the region, or provide a list of one or more rule set names. When requesting multiple items, use the pagination parameters to retrieve results as a set of sequential pages. If successful, a rule set is returned for each requested name.   <b>Learn more</b>  <ul> <li>  <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/match-rulesets.html">Build a Rule Set</a>  </li> </ul>  <b>Related operations</b>  <ul> <li>  <a>CreateMatchmakingConfiguration</a>  </li> <li>  <a>DescribeMatchmakingConfigurations</a>  </li> <li>  <a>UpdateMatchmakingConfiguration</a>  </li> <li>  <a>DeleteMatchmakingConfiguration</a>  </li> <li>  <a>CreateMatchmakingRuleSet</a>  </li> <li>  <a>DescribeMatchmakingRuleSets</a>  </li> <li>  <a>ValidateMatchmakingRuleSet</a>  </li> <li>  <a>DeleteMatchmakingRuleSet</a>  </li> </ul>

        :param self:
        :param NextToken:  Token that indicates the start of the next sequential page of results. Use the token that is returned with a previous call to this action. To start at the beginning of the result set, do not specify a value.<br/>Limit:  Maximum number of results to return. Use this parameter with <code>NextToken</code> to get results as a set of sequential pages.<br/>Names:  List of one or more matchmaking rule set names to retrieve details for. (Note: The rule set name is different from the optional "name" field in the rule set body.) <br/>
        :return: NextToken:  Token that indicates where to resume retrieving results on the next call to this action. If no token is returned, these results represent the end of the list.<br/>RuleSets:  Collection of requested matchmaking rule set objects. <br/>        """
        pass

    def describe_player_sessions(self, GameSessionId: str=None, PlayerId: str=None, PlayerSessionId: str=None, PlayerSessionStatusFilter: str=None, Limit: int=None, NextToken: str=None) -> DescribePlayerSessionsOutput:
        """Retrieves properties for one or more player sessions. This action can be used in several ways: (1) provide a <code>PlayerSessionId</code> to request properties for a specific player session; (2) provide a <code>GameSessionId</code> to request properties for all player sessions in the specified game session; (3) provide a <code>PlayerId</code> to request properties for all player sessions of a specified player.  To get game session record(s), specify only one of the following: a player session ID, a game session ID, or a player ID. You can filter this request by player session status. Use the pagination parameters to retrieve results as a set of sequential pages. If successful, a <a>PlayerSession</a> object is returned for each session matching the request.  <i>Available in Amazon GameLift Local.</i>  <ul> <li>  <a>CreatePlayerSession</a>  </li> <li>  <a>CreatePlayerSessions</a>  </li> <li>  <a>DescribePlayerSessions</a>  </li> <li> Game session placements <ul> <li>  <a>StartGameSessionPlacement</a>  </li> <li>  <a>DescribeGameSessionPlacement</a>  </li> <li>  <a>StopGameSessionPlacement</a>  </li> </ul> </li> </ul>

        :param self:
        :param NextToken:  Token that indicates the start of the next sequential page of results. Use the token that is returned with a previous call to this action. To start at the beginning of the result set, do not specify a value. If a player session ID is specified, this parameter is ignored.<br/>Limit:  Maximum number of results to return. Use this parameter with <code>NextToken</code> to get results as a set of sequential pages. If a player session ID is specified, this parameter is ignored.<br/>PlayerSessionStatusFilter:  Player session status to filter results on. Possible player session statuses include the following: <ul> <li>  <b>RESERVED</b> -- The player session request has been received, but the player has not yet connected to the server process and/or been validated.  </li> <li>  <b>ACTIVE</b> -- The player has been validated by the server process and is currently connected. </li> <li>  <b>COMPLETED</b> -- The player connection has been dropped. </li> <li>  <b>TIMEDOUT</b> -- A player session request was received, but the player did not connect and/or was not validated within the timeout limit (60 seconds). </li> </ul><br/>PlayerSessionId:  Unique identifier for a player session to retrieve.<br/>PlayerId:  Unique identifier for a player to retrieve player sessions for.<br/>GameSessionId:  Unique identifier for the game session to retrieve player sessions for.<br/>
        :return: NextToken:  Token that indicates where to resume retrieving results on the next call to this action. If no token is returned, these results represent the end of the list.<br/>PlayerSessions:  Collection of objects containing properties for each player session that matches the request.<br/>        """
        pass

    def describe_runtime_configuration(self, FleetId: str) -> DescribeRuntimeConfigurationOutput:
        """Retrieves the current run-time configuration for the specified fleet. The run-time configuration tells Amazon GameLift how to launch server processes on instances in the fleet.  <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/fleets-intro.html"> Working with Fleets</a>.  <b>Related operations</b>  <ul> <li>  <a>CreateFleet</a>  </li> <li>  <a>ListFleets</a>  </li> <li>  <a>DeleteFleet</a>  </li> <li> Describe fleets: <ul> <li>  <a>DescribeFleetAttributes</a>  </li> <li>  <a>DescribeFleetCapacity</a>  </li> <li>  <a>DescribeFleetPortSettings</a>  </li> <li>  <a>DescribeFleetUtilization</a>  </li> <li>  <a>DescribeRuntimeConfiguration</a>  </li> <li>  <a>DescribeEC2InstanceLimits</a>  </li> <li>  <a>DescribeFleetEvents</a>  </li> </ul> </li> <li> Update fleets: <ul> <li>  <a>UpdateFleetAttributes</a>  </li> <li>  <a>UpdateFleetCapacity</a>  </li> <li>  <a>UpdateFleetPortSettings</a>  </li> <li>  <a>UpdateRuntimeConfiguration</a>  </li> </ul> </li> <li> Manage fleet actions: <ul> <li>  <a>StartFleetActions</a>  </li> <li>  <a>StopFleetActions</a>  </li> </ul> </li> </ul>

        :param self:
        :param FleetId:  Unique identifier for a fleet to get the run-time configuration for.<br/>
        :return: RuntimeConfiguration:  Instructions describing how server processes should be launched and maintained on each instance in the fleet.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionActivationTimeoutSeconds</i> Maximum amount of time (in seconds) that a game session can remain in status <code>ACTIVATING</code>. If the game session is not active before the timeout, activation is terminated and the game session status is changed to <code>TERMINATED</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxConcurrentGameSessionActivations</i> Maximum number of game sessions with status <code>ACTIVATING</code> to allow on an instance simultaneously. This setting limits the amount of instance resources that can be used for new game activations at any one time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerProcesses</i> Collection of server process configurations that describe which server processes to run on each instance in a fleet.<br/>        """
        pass

    def describe_scaling_policies(self, FleetId: str, StatusFilter: str=None, Limit: int=None, NextToken: str=None) -> DescribeScalingPoliciesOutput:
        """Retrieves all scaling policies applied to a fleet. To get a fleet's scaling policies, specify the fleet ID. You can filter this request by policy status, such as to retrieve only active scaling policies. Use the pagination parameters to retrieve results as a set of sequential pages. If successful, set of <a>ScalingPolicy</a> objects is returned for the fleet. A fleet may have all of its scaling policies suspended (<a>StopFleetActions</a>). This action does not affect the status of the scaling policies, which remains ACTIVE. To see whether a fleet's scaling policies are in force or suspended, call <a>DescribeFleetAttributes</a> and check the stopped actions. <ul> <li>  <a>DescribeFleetCapacity</a>  </li> <li>  <a>UpdateFleetCapacity</a>  </li> <li>  <a>DescribeEC2InstanceLimits</a>  </li> <li> Manage scaling policies: <ul> <li>  <a>PutScalingPolicy</a> (auto-scaling) </li> <li>  <a>DescribeScalingPolicies</a> (auto-scaling) </li> <li>  <a>DeleteScalingPolicy</a> (auto-scaling) </li> </ul> </li> <li> Manage fleet actions: <ul> <li>  <a>StartFleetActions</a>  </li> <li>  <a>StopFleetActions</a>  </li> </ul> </li> </ul>

        :param self:
        :param NextToken:  Token that indicates the start of the next sequential page of results. Use the token that is returned with a previous call to this action. To start at the beginning of the result set, do not specify a value.<br/>Limit:  Maximum number of results to return. Use this parameter with <code>NextToken</code> to get results as a set of sequential pages.<br/>StatusFilter:  Scaling policy status to filter results on. A scaling policy is only in force when in an <code>ACTIVE</code> status. <ul> <li>  <b>ACTIVE</b> -- The scaling policy is currently in force. </li> <li>  <b>UPDATEREQUESTED</b> -- A request to update the scaling policy has been received. </li> <li>  <b>UPDATING</b> -- A change is being made to the scaling policy. </li> <li>  <b>DELETEREQUESTED</b> -- A request to delete the scaling policy has been received. </li> <li>  <b>DELETING</b> -- The scaling policy is being deleted. </li> <li>  <b>DELETED</b> -- The scaling policy has been deleted. </li> <li>  <b>ERROR</b> -- An error occurred in creating the policy. It should be removed and recreated. </li> </ul><br/>FleetId:  Unique identifier for a fleet to retrieve scaling policies for.<br/>
        :return: NextToken:  Token that indicates where to resume retrieving results on the next call to this action. If no token is returned, these results represent the end of the list.<br/>ScalingPolicies:  Collection of objects containing the scaling policies matching the request.<br/>        """
        pass

    def describe_script(self, ScriptId: str) -> DescribeScriptOutput:
        """Retrieves properties for a Realtime script.  To request a script record, specify the script ID. If successful, an object containing the script properties is returned.  <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/realtime-intro.html">Amazon GameLift Realtime Servers</a>   <b>Related operations</b>  <ul> <li>  <a>CreateScript</a>  </li> <li>  <a>ListScripts</a>  </li> <li>  <a>DescribeScript</a>  </li> <li>  <a>UpdateScript</a>  </li> <li>  <a>DeleteScript</a>  </li> </ul>

        :param self:
        :param ScriptId:  Unique identifier for a Realtime script to retrieve properties for.<br/>
        :return: Script:  Set of properties describing the requested script.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageLocation</i> Location in Amazon Simple Storage Service (Amazon S3) where build or script files are stored for access by Amazon GameLift. This location is specified in <a>CreateBuild</a>, <a>CreateScript</a>, and <a>UpdateScript</a> requests. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StorageLocation</b>: Location in Amazon Simple Storage Service (Amazon S3) where build or script files are stored for access by Amazon GameLift. This location is specified in <a>CreateBuild</a>, <a>CreateScript</a>, and <a>UpdateScript</a> requests. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> Time stamp indicating when this data object was created. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SizeOnDisk</i> File size of the uploaded Realtime script, expressed in bytes. When files are uploaded from an S3 location, this value remains at "0".<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> Version that is associated with a build or script. Version strings do not need to be unique.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Descriptive label that is associated with a script. Script names do not need to be unique.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ScriptId</i> Unique identifier for a Realtime script<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ObjectVersion</i> Version of the file, if object versioning is turned on for the bucket. Amazon GameLift uses this information when retrieving files from an S3 bucket that you own. Use this parameter to specify a specific version of the file; if not set, the latest version of the file is retrieved. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> Amazon Resource Name (<a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/s3-arn-format.html">ARN</a>) for an IAM role that allows Amazon GameLift to access the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> Name of the zip file containing the build files or script files. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> Amazon S3 bucket identifier. This is the name of the S3 bucket.<br/>        """
        pass

    def describe_vpc_peering_authorizations(self) -> DescribeVpcPeeringAuthorizationsOutput:
        """Retrieves valid VPC peering authorizations that are pending for the AWS account. This operation returns all VPC peering authorizations and requests for peering. This includes those initiated and received by this account.  <ul> <li>  <a>CreateVpcPeeringAuthorization</a>  </li> <li>  <a>DescribeVpcPeeringAuthorizations</a>  </li> <li>  <a>DeleteVpcPeeringAuthorization</a>  </li> <li>  <a>CreateVpcPeeringConnection</a>  </li> <li>  <a>DescribeVpcPeeringConnections</a>  </li> <li>  <a>DeleteVpcPeeringConnection</a>  </li> </ul>

        :param self:
        :param 
        :return: VpcPeeringAuthorizations:  Collection of objects that describe all valid VPC peering operations for the current AWS account.<br/>        """
        pass

    def describe_vpc_peering_connections(self, FleetId: str=None) -> DescribeVpcPeeringConnectionsOutput:
        """Retrieves information on VPC peering connections. Use this operation to get peering information for all fleets or for one specific fleet ID.  To retrieve connection information, call this operation from the AWS account that is used to manage the Amazon GameLift fleets. Specify a fleet ID or leave the parameter empty to retrieve all connection records. If successful, the retrieved information includes both active and pending connections. Active connections identify the IpV4 CIDR block that the VPC uses to connect.  <ul> <li>  <a>CreateVpcPeeringAuthorization</a>  </li> <li>  <a>DescribeVpcPeeringAuthorizations</a>  </li> <li>  <a>DeleteVpcPeeringAuthorization</a>  </li> <li>  <a>CreateVpcPeeringConnection</a>  </li> <li>  <a>DescribeVpcPeeringConnections</a>  </li> <li>  <a>DeleteVpcPeeringConnection</a>  </li> </ul>

        :param self:
        :param FleetId:  Unique identifier for a fleet.<br/>
        :return: VpcPeeringConnections:  Collection of VPC peering connection records that match the request.<br/>        """
        pass

    def get_game_session_log_url(self, GameSessionId: str) -> GetGameSessionLogUrlOutput:
        """Retrieves the location of stored game session logs for a specified game session. When a game session is terminated, Amazon GameLift automatically stores the logs in Amazon S3 and retains them for 14 days. Use this URL to download the logs. <note> See the <a href="https://docs.aws.amazon.com/general/latest/gr/aws_service_limits.html#limits_gamelift">AWS Service Limits</a> page for maximum log file sizes. Log files that exceed this limit are not saved. </note> <ul> <li>  <a>CreateGameSession</a>  </li> <li>  <a>DescribeGameSessions</a>  </li> <li>  <a>DescribeGameSessionDetails</a>  </li> <li>  <a>SearchGameSessions</a>  </li> <li>  <a>UpdateGameSession</a>  </li> <li>  <a>GetGameSessionLogUrl</a>  </li> <li> Game session placements <ul> <li>  <a>StartGameSessionPlacement</a>  </li> <li>  <a>DescribeGameSessionPlacement</a>  </li> <li>  <a>StopGameSessionPlacement</a>  </li> </ul> </li> </ul>

        :param self:
        :param GameSessionId:  Unique identifier for the game session to get logs for.<br/>
        :return: PreSignedUrl:  Location of the requested game session logs, available for download. This URL is valid for 15 minutes, after which S3 will reject any download request using this URL. You can request a new URL any time within the 14-day period that the logs are retained.<br/>        """
        pass

    def get_instance_access(self, InstanceId: str, FleetId: str) -> GetInstanceAccessOutput:
        """Requests remote access to a fleet instance. Remote access is useful for debugging, gathering benchmarking data, or watching activity in real time.  Access requires credentials that match the operating system of the instance. For a Windows instance, Amazon GameLift returns a user name and password as strings for use with a Windows Remote Desktop client. For a Linux instance, Amazon GameLift returns a user name and RSA private key, also as strings, for use with an SSH client. The private key must be saved in the proper format to a <code>.pem</code> file before using. If you're making this request using the AWS CLI, saving the secret can be handled as part of the GetInstanceAccess request. (See the example later in this topic). For more information on remote access, see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/fleets-remote-access.html">Remotely Accessing an Instance</a>. To request access to a specific instance, specify the IDs of both the instance and the fleet it belongs to. You can retrieve a fleet's instance IDs by calling <a>DescribeInstances</a>. If successful, an <a>InstanceAccess</a> object is returned containing the instance's IP address and a set of credentials.

        :param self:
        :param InstanceId:  Unique identifier for an instance you want to get access to. You can access an instance in any status.<br/>FleetId:  Unique identifier for a fleet that contains the instance you want access to. The fleet can be in any of the following statuses: <code>ACTIVATING</code>, <code>ACTIVE</code>, or <code>ERROR</code>. Fleets with an <code>ERROR</code> status may be accessible for a short time before they are deleted.<br/>
        :return: InstanceAccess:  Object that contains connection information for a fleet instance, including IP address and access credentials.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Credentials</i> Credentials required to access the instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Credentials</b>: Credentials required to access the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OperatingSystem</i> Operating system that is running on the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IpAddress</i> IP address assigned to the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceId</i> Unique identifier for an instance being accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FleetId</i> Unique identifier for a fleet containing the instance being accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Secret</i> Secret string. For Windows instances, the secret is a password for use with Windows Remote Desktop. For Linux instances, it is a private key (which must be saved as a <code>.pem</code> file) for use with SSH.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UserName</i> User login string.<br/>        """
        pass

    def list_aliases(self, RoutingStrategyType: str=None, Name: str=None, Limit: int=None, NextToken: str=None) -> ListAliasesOutput:
        """Retrieves all aliases for this AWS account. You can filter the result set by alias name and/or routing strategy type. Use the pagination parameters to retrieve results in sequential pages. <note> Returned aliases are not listed in any particular order. </note> <ul> <li>  <a>CreateAlias</a>  </li> <li>  <a>ListAliases</a>  </li> <li>  <a>DescribeAlias</a>  </li> <li>  <a>UpdateAlias</a>  </li> <li>  <a>DeleteAlias</a>  </li> <li>  <a>ResolveAlias</a>  </li> </ul>

        :param self:
        :param NextToken:  Token that indicates the start of the next sequential page of results. Use the token that is returned with a previous call to this action. To start at the beginning of the result set, do not specify a value.<br/>Limit:  Maximum number of results to return. Use this parameter with <code>NextToken</code> to get results as a set of sequential pages.<br/>Name:  Descriptive label that is associated with an alias. Alias names do not need to be unique.<br/>RoutingStrategyType:  Type of routing to filter results on. Use this parameter to retrieve only aliases of a certain type. To retrieve all aliases, leave this parameter empty. Possible routing types include the following: <ul> <li>  <b>SIMPLE</b> -- The alias resolves to one specific fleet. Use this type when routing to active fleets. </li> <li>  <b>TERMINAL</b> -- The alias does not resolve to a fleet but instead can be used to display a message to the user. A terminal alias throws a TerminalRoutingStrategyException with the <a>RoutingStrategy</a> message embedded. </li> </ul><br/>
        :return: NextToken:  Token that indicates where to resume retrieving results on the next call to this action. If no token is returned, these results represent the end of the list.<br/>Aliases:  Collection of alias records that match the list request.<br/>        """
        pass

    def list_builds(self, Status: str=None, Limit: int=None, NextToken: str=None) -> ListBuildsOutput:
        """Retrieves build records for all builds associated with the AWS account in use. You can limit results to builds that are in a specific status by using the <code>Status</code> parameter. Use the pagination parameters to retrieve results in a set of sequential pages.  <note> Build records are not listed in any particular order. </note>  <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/build-intro.html"> Working with Builds</a>   <b>Related operations</b>  <ul> <li>  <a>CreateBuild</a>  </li> <li>  <a>ListBuilds</a>  </li> <li>  <a>DescribeBuild</a>  </li> <li>  <a>UpdateBuild</a>  </li> <li>  <a>DeleteBuild</a>  </li> </ul>

        :param self:
        :param NextToken:  Token that indicates the start of the next sequential page of results. Use the token that is returned with a previous call to this action. To start at the beginning of the result set, do not specify a value.<br/>Limit:  Maximum number of results to return. Use this parameter with <code>NextToken</code> to get results as a set of sequential pages.<br/>Status:  Build status to filter results by. To retrieve all builds, leave this parameter empty. Possible build statuses include the following: <ul> <li>  <b>INITIALIZED</b> -- A new build has been defined, but no files have been uploaded. You cannot create fleets for builds that are in this status. When a build is successfully created, the build status is set to this value.  </li> <li>  <b>READY</b> -- The game build has been successfully uploaded. You can now create new fleets for this build. </li> <li>  <b>FAILED</b> -- The game build upload failed. You cannot create new fleets for this build.  </li> </ul><br/>
        :return: NextToken:  Token that indicates where to resume retrieving results on the next call to this action. If no token is returned, these results represent the end of the list.<br/>Builds:  Collection of build records that match the request.<br/>        """
        pass

    def list_fleets(self, BuildId: str=None, ScriptId: str=None, Limit: int=None, NextToken: str=None) -> ListFleetsOutput:
        """Retrieves a collection of fleet records for this AWS account. You can filter the result set to find only those fleets that are deployed with a specific build or script. Use the pagination parameters to retrieve results in sequential pages. <note> Fleet records are not listed in a particular order. </note>  <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/fleets-intro.html"> Set Up Fleets</a>.  <b>Related operations</b>  <ul> <li>  <a>CreateFleet</a>  </li> <li>  <a>ListFleets</a>  </li> <li>  <a>DeleteFleet</a>  </li> <li> Describe fleets: <ul> <li>  <a>DescribeFleetAttributes</a>  </li> <li>  <a>DescribeFleetCapacity</a>  </li> <li>  <a>DescribeFleetPortSettings</a>  </li> <li>  <a>DescribeFleetUtilization</a>  </li> <li>  <a>DescribeRuntimeConfiguration</a>  </li> <li>  <a>DescribeEC2InstanceLimits</a>  </li> <li>  <a>DescribeFleetEvents</a>  </li> </ul> </li> <li> Update fleets: <ul> <li>  <a>UpdateFleetAttributes</a>  </li> <li>  <a>UpdateFleetCapacity</a>  </li> <li>  <a>UpdateFleetPortSettings</a>  </li> <li>  <a>UpdateRuntimeConfiguration</a>  </li> </ul> </li> <li> Manage fleet actions: <ul> <li>  <a>StartFleetActions</a>  </li> <li>  <a>StopFleetActions</a>  </li> </ul> </li> </ul>

        :param self:
        :param NextToken:  Token that indicates the start of the next sequential page of results. Use the token that is returned with a previous call to this action. To start at the beginning of the result set, do not specify a value.<br/>Limit:  Maximum number of results to return. Use this parameter with <code>NextToken</code> to get results as a set of sequential pages.<br/>ScriptId:  Unique identifier for a Realtime script to return fleets for. Use this parameter to return only fleets using the specified script. To retrieve all fleets, leave this parameter empty.<br/>BuildId:  Unique identifier for a build to return fleets for. Use this parameter to return only fleets using the specified build. To retrieve all fleets, leave this parameter empty.<br/>
        :return: NextToken:  Token that indicates where to resume retrieving results on the next call to this action. If no token is returned, these results represent the end of the list.<br/>FleetIds:  Set of fleet IDs matching the list request. You can retrieve additional information about all returned fleets by passing this result set to a call to <a>DescribeFleetAttributes</a>, <a>DescribeFleetCapacity</a>, or <a>DescribeFleetUtilization</a>.<br/>        """
        pass

    def list_scripts(self, Limit: int=None, NextToken: str=None) -> ListScriptsOutput:
        """Retrieves script records for all Realtime scripts that are associated with the AWS account in use.   <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/realtime-intro.html">Amazon GameLift Realtime Servers</a>   <b>Related operations</b>  <ul> <li>  <a>CreateScript</a>  </li> <li>  <a>ListScripts</a>  </li> <li>  <a>DescribeScript</a>  </li> <li>  <a>UpdateScript</a>  </li> <li>  <a>DeleteScript</a>  </li> </ul>

        :param self:
        :param NextToken:  Token that indicates the start of the next sequential page of results. Use the token that is returned with a previous call to this action. To start at the beginning of the result set, do not specify a value.<br/>Limit:  Maximum number of results to return. Use this parameter with <code>NextToken</code> to get results as a set of sequential pages.<br/>
        :return: NextToken:  Token that indicates where to resume retrieving results on the next call to this action. If no token is returned, these results represent the end of the list.<br/>Scripts:  Set of properties describing the requested script.<br/>        """
        pass

    def put_scaling_policy(self, MetricName: str, FleetId: str, Name: str, ScalingAdjustment: int=None, ScalingAdjustmentType: str=None, Threshold: Double=None, ComparisonOperator: str=None, EvaluationPeriods: int=None, PolicyType: str=None, TargetConfiguration: TargetConfiguration=None) -> PutScalingPolicyOutput:
        """Creates or updates a scaling policy for a fleet. Scaling policies are used to automatically scale a fleet's hosting capacity to meet player demand. An active scaling policy instructs Amazon GameLift to track a fleet metric and automatically change the fleet's capacity when a certain threshold is reached. There are two types of scaling policies: target-based and rule-based. Use a target-based policy to quickly and efficiently manage fleet scaling; this option is the most commonly used. Use rule-based policies when you need to exert fine-grained control over auto-scaling.  Fleets can have multiple scaling policies of each type in force at the same time; you can have one target-based policy, one or multiple rule-based scaling policies, or both. We recommend caution, however, because multiple auto-scaling policies can have unintended consequences. You can temporarily suspend all scaling policies for a fleet by calling <a>StopFleetActions</a> with the fleet action AUTO_SCALING. To resume scaling policies, call <a>StartFleetActions</a> with the same fleet action. To stop just one scaling policy--or to permanently remove it, you must delete the policy with <a>DeleteScalingPolicy</a>. Learn more about how to work with auto-scaling in <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/fleets-autoscaling.html">Set Up Fleet Automatic Scaling</a>.  <b>Target-based policy</b>  A target-based policy tracks a single metric: PercentAvailableGameSessions. This metric tells us how much of a fleet's hosting capacity is ready to host game sessions but is not currently in use. This is the fleet's buffer; it measures the additional player demand that the fleet could handle at current capacity. With a target-based policy, you set your ideal buffer size and leave it to Amazon GameLift to take whatever action is needed to maintain that target.  For example, you might choose to maintain a 10% buffer for a fleet that has the capacity to host 100 simultaneous game sessions. This policy tells Amazon GameLift to take action whenever the fleet's available capacity falls below or rises above 10 game sessions. Amazon GameLift will start new instances or stop unused instances in order to return to the 10% buffer.  To create or update a target-based policy, specify a fleet ID and name, and set the policy type to "TargetBased". Specify the metric to track (PercentAvailableGameSessions) and reference a <a>TargetConfiguration</a> object with your desired buffer value. Exclude all other parameters. On a successful request, the policy name is returned. The scaling policy is automatically in force as soon as it's successfully created. If the fleet's auto-scaling actions are temporarily suspended, the new policy will be in force once the fleet actions are restarted.  <b>Rule-based policy</b>  A rule-based policy tracks specified fleet metric, sets a threshold value, and specifies the type of action to initiate when triggered. With a rule-based policy, you can select from several available fleet metrics. Each policy specifies whether to scale up or scale down (and by how much), so you need one policy for each type of action.  For example, a policy may make the following statement: "If the percentage of idle instances is greater than 20% for more than 15 minutes, then reduce the fleet capacity by 10%." A policy's rule statement has the following structure: If <code>[MetricName]</code> is <code>[ComparisonOperator]</code> <code>[Threshold]</code> for <code>[EvaluationPeriods]</code> minutes, then <code>[ScalingAdjustmentType]</code> to/by <code>[ScalingAdjustment]</code>. To implement the example, the rule statement would look like this: If <code>[PercentIdleInstances]</code> is <code>[GreaterThanThreshold]</code> <code>[20]</code> for <code>[15]</code> minutes, then <code>[PercentChangeInCapacity]</code> to/by <code>[10]</code>. To create or update a scaling policy, specify a unique combination of name and fleet ID, and set the policy type to "RuleBased". Specify the parameter values for a policy rule statement. On a successful request, the policy name is returned. Scaling policies are automatically in force as soon as they're successfully created. If the fleet's auto-scaling actions are temporarily suspended, the new policy will be in force once the fleet actions are restarted. <ul> <li>  <a>DescribeFleetCapacity</a>  </li> <li>  <a>UpdateFleetCapacity</a>  </li> <li>  <a>DescribeEC2InstanceLimits</a>  </li> <li> Manage scaling policies: <ul> <li>  <a>PutScalingPolicy</a> (auto-scaling) </li> <li>  <a>DescribeScalingPolicies</a> (auto-scaling) </li> <li>  <a>DeleteScalingPolicy</a> (auto-scaling) </li> </ul> </li> <li> Manage fleet actions: <ul> <li>  <a>StartFleetActions</a>  </li> <li>  <a>StopFleetActions</a>  </li> </ul> </li> </ul>

        :param self:
        :param TargetConfiguration:  Object that contains settings for a target-based scaling policy.:<br/>PolicyType:  Type of scaling policy to create. For a target-based policy, set the parameter <i>MetricName</i> to 'PercentAvailableGameSessions' and specify a <i>TargetConfiguration</i>. For a rule-based policy set the following parameters: <i>MetricName</i>, <i>ComparisonOperator</i>, <i>Threshold</i>, <i>EvaluationPeriods</i>, <i>ScalingAdjustmentType</i>, and <i>ScalingAdjustment</i>.<br/>MetricName:  Name of the Amazon GameLift-defined metric that is used to trigger a scaling adjustment. For detailed descriptions of fleet metrics, see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/monitoring-cloudwatch.html">Monitor Amazon GameLift with Amazon CloudWatch</a>.  <ul> <li>  <b>ActivatingGameSessions</b> -- Game sessions in the process of being created. </li> <li>  <b>ActiveGameSessions</b> -- Game sessions that are currently running. </li> <li>  <b>ActiveInstances</b> -- Fleet instances that are currently running at least one game session. </li> <li>  <b>AvailableGameSessions</b> -- Additional game sessions that fleet could host simultaneously, given current capacity. </li> <li>  <b>AvailablePlayerSessions</b> -- Empty player slots in currently active game sessions. This includes game sessions that are not currently accepting players. Reserved player slots are not included. </li> <li>  <b>CurrentPlayerSessions</b> -- Player slots in active game sessions that are being used by a player or are reserved for a player.  </li> <li>  <b>IdleInstances</b> -- Active instances that are currently hosting zero game sessions.  </li> <li>  <b>PercentAvailableGameSessions</b> -- Unused percentage of the total number of game sessions that a fleet could host simultaneously, given current capacity. Use this metric for a target-based scaling policy. </li> <li>  <b>PercentIdleInstances</b> -- Percentage of the total number of active instances that are hosting zero game sessions. </li> <li>  <b>QueueDepth</b> -- Pending game session placement requests, in any queue, where the current fleet is the top-priority destination. </li> <li>  <b>WaitTime</b> -- Current wait time for pending game session placement requests, in any queue, where the current fleet is the top-priority destination.  </li> </ul><br/>EvaluationPeriods:  Length of time (in minutes) the metric must be at or beyond the threshold before a scaling event is triggered.<br/>ComparisonOperator:  Comparison operator to use when measuring the metric against the threshold value.<br/>Threshold:  Metric value used to trigger a scaling event.<br/>ScalingAdjustmentType:  Type of adjustment to make to a fleet's instance count (see <a>FleetCapacity</a>): <ul> <li>  <b>ChangeInCapacity</b> -- add (or subtract) the scaling adjustment value from the current instance count. Positive values scale up while negative values scale down. </li> <li>  <b>ExactCapacity</b> -- set the instance count to the scaling adjustment value. </li> <li>  <b>PercentChangeInCapacity</b> -- increase or reduce the current instance count by the scaling adjustment, read as a percentage. Positive values scale up while negative values scale down; for example, a value of "-10" scales the fleet down by 10%. </li> </ul><br/>ScalingAdjustment:  Amount of adjustment to make, based on the scaling adjustment type.<br/>FleetId:  Unique identifier for a fleet to apply this policy to. The fleet cannot be in any of the following statuses: ERROR or DELETING.<br/>Name:  Descriptive label that is associated with a scaling policy. Policy names do not need to be unique. A fleet can have only one scaling policy with the same name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetValue</i> Desired value to use with a target-based scaling policy. The value must be relevant for whatever metric the scaling policy is using. For example, in a policy using the metric PercentAvailableGameSessions, the target value should be the preferred size of the fleet's buffer (the percent of capacity that should be idle and ready for new game sessions).<br/>
        :return: Name:  Descriptive label that is associated with a scaling policy. Policy names do not need to be unique.<br/>        """
        pass

    def request_upload_credentials(self, BuildId: str) -> RequestUploadCredentialsOutput:
        """Retrieves a fresh set of credentials for use when uploading a new set of game build files to Amazon GameLift's Amazon S3. This is done as part of the build creation process; see <a>CreateBuild</a>. To request new credentials, specify the build ID as returned with an initial <code>CreateBuild</code> request. If successful, a new set of credentials are returned, along with the S3 storage location associated with the build ID.  <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-build-intro.html">Uploading Your Game</a>   <b>Related operations</b>  <ul> <li>  <a>CreateBuild</a>  </li> <li>  <a>ListBuilds</a>  </li> <li>  <a>DescribeBuild</a>  </li> <li>  <a>UpdateBuild</a>  </li> <li>  <a>DeleteBuild</a>  </li> </ul>

        :param self:
        :param BuildId:  Unique identifier for a build to get credentials for.<br/>
        :return: StorageLocation:  Amazon S3 path and key, identifying where the game build files are stored.:<br/>UploadCredentials:  AWS credentials required when uploading a game build to the storage location. These credentials have a limited lifespan and are valid only for the build they were issued for.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SessionToken</i> Token used to associate a specific build ID with the files uploaded using these credentials.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecretAccessKey</i> Temporary secret key allowing access to the Amazon GameLift S3 account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccessKeyId</i> Temporary key allowing access to the Amazon GameLift S3 account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ObjectVersion</i> Version of the file, if object versioning is turned on for the bucket. Amazon GameLift uses this information when retrieving files from an S3 bucket that you own. Use this parameter to specify a specific version of the file; if not set, the latest version of the file is retrieved. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> Amazon Resource Name (<a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/s3-arn-format.html">ARN</a>) for an IAM role that allows Amazon GameLift to access the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> Name of the zip file containing the build files or script files. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> Amazon S3 bucket identifier. This is the name of the S3 bucket.<br/>        """
        pass

    def resolve_alias(self, AliasId: str) -> ResolveAliasOutput:
        """Retrieves the fleet ID that a specified alias is currently pointing to. <ul> <li>  <a>CreateAlias</a>  </li> <li>  <a>ListAliases</a>  </li> <li>  <a>DescribeAlias</a>  </li> <li>  <a>UpdateAlias</a>  </li> <li>  <a>DeleteAlias</a>  </li> <li>  <a>ResolveAlias</a>  </li> </ul>

        :param self:
        :param AliasId:  Unique identifier for the alias you want to resolve.<br/>
        :return: FleetId:  Fleet identifier that is associated with the requested alias.<br/>        """
        pass

    def search_game_sessions(self, FleetId: str=None, AliasId: str=None, FilterExpression: str=None, SortExpression: str=None, Limit: int=None, NextToken: str=None) -> SearchGameSessionsOutput:
        """Retrieves all active game sessions that match a set of search criteria and sorts them in a specified order. You can search or sort by the following game session attributes: <ul> <li>  <b>gameSessionId</b> -- Unique identifier for the game session. You can use either a <code>GameSessionId</code> or <code>GameSessionArn</code> value.  </li> <li>  <b>gameSessionName</b> -- Name assigned to a game session. This value is set when requesting a new game session with <a>CreateGameSession</a> or updating with <a>UpdateGameSession</a>. Game session names do not need to be unique to a game session. </li> <li>  <b>gameSessionProperties</b> -- Custom data defined in a game session's <code>GameProperty</code> parameter. <code>GameProperty</code> values are stored as key:value pairs; the filter expression must indicate the key and a string to search the data values for. For example, to search for game sessions with custom data containing the key:value pair "gameMode:brawl", specify the following: <code>gameSessionProperties.gameMode = "brawl"</code>. All custom data values are searched as strings. </li> <li>  <b>maximumSessions</b> -- Maximum number of player sessions allowed for a game session. This value is set when requesting a new game session with <a>CreateGameSession</a> or updating with <a>UpdateGameSession</a>. </li> <li>  <b>creationTimeMillis</b> -- Value indicating when a game session was created. It is expressed in Unix time as milliseconds. </li> <li>  <b>playerSessionCount</b> -- Number of players currently connected to a game session. This value changes rapidly as players join the session or drop out. </li> <li>  <b>hasAvailablePlayerSessions</b> -- Boolean value indicating whether a game session has reached its maximum number of players. It is highly recommended that all search requests include this filter attribute to optimize search performance and return only sessions that players can join.  </li> </ul> <note> Returned values for <code>playerSessionCount</code> and <code>hasAvailablePlayerSessions</code> change quickly as players join sessions and others drop out. Results should be considered a snapshot in time. Be sure to refresh search results often, and handle sessions that fill up before a player can join.  </note> To search or sort, specify either a fleet ID or an alias ID, and provide a search filter expression, a sort expression, or both. If successful, a collection of <a>GameSession</a> objects matching the request is returned. Use the pagination parameters to retrieve results as a set of sequential pages.  You can search for game sessions one fleet at a time only. To find game sessions across multiple fleets, you must search each fleet separately and combine the results. This search feature finds only game sessions that are in <code>ACTIVE</code> status. To locate games in statuses other than active, use <a>DescribeGameSessionDetails</a>. <ul> <li>  <a>CreateGameSession</a>  </li> <li>  <a>DescribeGameSessions</a>  </li> <li>  <a>DescribeGameSessionDetails</a>  </li> <li>  <a>SearchGameSessions</a>  </li> <li>  <a>UpdateGameSession</a>  </li> <li>  <a>GetGameSessionLogUrl</a>  </li> <li> Game session placements <ul> <li>  <a>StartGameSessionPlacement</a>  </li> <li>  <a>DescribeGameSessionPlacement</a>  </li> <li>  <a>StopGameSessionPlacement</a>  </li> </ul> </li> </ul>

        :param self:
        :param NextToken:  Token that indicates the start of the next sequential page of results. Use the token that is returned with a previous call to this action. To start at the beginning of the result set, do not specify a value.<br/>Limit:  Maximum number of results to return. Use this parameter with <code>NextToken</code> to get results as a set of sequential pages. The maximum number of results returned is 20, even if this value is not set or is set higher than 20. <br/>SortExpression:  Instructions on how to sort the search results. If no sort expression is included, the request returns results in random order. A sort expression consists of the following elements: <ul> <li>  <b>Operand</b> -- Name of a game session attribute. Valid values are <code>gameSessionName</code>, <code>gameSessionId</code>, <code>gameSessionProperties</code>, <code>maximumSessions</code>, <code>creationTimeMillis</code>, <code>playerSessionCount</code>, <code>hasAvailablePlayerSessions</code>. </li> <li>  <b>Order</b> -- Valid sort orders are <code>ASC</code> (ascending) and <code>DESC</code> (descending). </li> </ul> For example, this sort expression returns the oldest active sessions first: <code>"SortExpression": "creationTimeMillis ASC"</code>. Results with a null value for the sort operand are returned at the end of the list.<br/>FilterExpression:  String containing the search criteria for the session search. If no filter expression is included, the request returns results for all game sessions in the fleet that are in <code>ACTIVE</code> status. A filter expression can contain one or multiple conditions. Each condition consists of the following: <ul> <li>  <b>Operand</b> -- Name of a game session attribute. Valid values are <code>gameSessionName</code>, <code>gameSessionId</code>, <code>gameSessionProperties</code>, <code>maximumSessions</code>, <code>creationTimeMillis</code>, <code>playerSessionCount</code>, <code>hasAvailablePlayerSessions</code>. </li> <li>  <b>Comparator</b> -- Valid comparators are: <code>=</code>, <code>&lt;&gt;</code>, <code>&lt;</code>, <code>&gt;</code>, <code>&lt;=</code>, <code>&gt;=</code>.  </li> <li>  <b>Value</b> -- Value to be searched for. Values may be numbers, boolean values (true/false) or strings depending on the operand. String values are case sensitive and must be enclosed in single quotes. Special characters must be escaped. Boolean and string values can only be used with the comparators <code>=</code> and <code>&lt;&gt;</code>. For example, the following filter expression searches on <code>gameSessionName</code>: "<code>FilterExpression": "gameSessionName = 'Matt\\'s Awesome Game 1'"</code>.  </li> </ul> To chain multiple conditions in a single expression, use the logical keywords <code>AND</code>, <code>OR</code>, and <code>NOT</code> and parentheses as needed. For example: <code>x AND y AND NOT z</code>, <code>NOT (x OR y)</code>. Session search evaluates conditions from left to right using the following precedence rules: <ol> <li>  <code>=</code>, <code>&lt;&gt;</code>, <code>&lt;</code>, <code>&gt;</code>, <code>&lt;=</code>, <code>&gt;=</code>  </li> <li> Parentheses </li> <li> NOT </li> <li> AND </li> <li> OR </li> </ol> For example, this filter expression retrieves game sessions hosting at least ten players that have an open player slot: <code>"maximumSessions&gt;=10 AND hasAvailablePlayerSessions=true"</code>. <br/>AliasId:  Unique identifier for an alias associated with the fleet to search for active game sessions. Each request must reference either a fleet ID or alias ID, but not both.<br/>FleetId:  Unique identifier for a fleet to search for active game sessions. Each request must reference either a fleet ID or alias ID, but not both.<br/>
        :return: NextToken:  Token that indicates where to resume retrieving results on the next call to this action. If no token is returned, these results represent the end of the list.<br/>GameSessions:  Collection of objects containing game session properties for each session matching the request.<br/>        """
        pass

    def start_fleet_actions(self, Actions: List[FleetAction], FleetId: str) -> StartFleetActionsOutput:
        """Resumes activity on a fleet that was suspended with <a>StopFleetActions</a>. Currently, this operation is used to restart a fleet's auto-scaling activity.  To start fleet actions, specify the fleet ID and the type of actions to restart. When auto-scaling fleet actions are restarted, Amazon GameLift once again initiates scaling events as triggered by the fleet's scaling policies. If actions on the fleet were never stopped, this operation will have no effect. You can view a fleet's stopped actions using <a>DescribeFleetAttributes</a>.  <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/fleets-intro.html"> Working with Fleets</a>.  <b>Related operations</b>  <ul> <li>  <a>CreateFleet</a>  </li> <li>  <a>ListFleets</a>  </li> <li>  <a>DeleteFleet</a>  </li> <li> Describe fleets: <ul> <li>  <a>DescribeFleetAttributes</a>  </li> <li>  <a>DescribeFleetCapacity</a>  </li> <li>  <a>DescribeFleetPortSettings</a>  </li> <li>  <a>DescribeFleetUtilization</a>  </li> <li>  <a>DescribeRuntimeConfiguration</a>  </li> <li>  <a>DescribeEC2InstanceLimits</a>  </li> <li>  <a>DescribeFleetEvents</a>  </li> </ul> </li> <li> Update fleets: <ul> <li>  <a>UpdateFleetAttributes</a>  </li> <li>  <a>UpdateFleetCapacity</a>  </li> <li>  <a>UpdateFleetPortSettings</a>  </li> <li>  <a>UpdateRuntimeConfiguration</a>  </li> </ul> </li> <li> Manage fleet actions: <ul> <li>  <a>StartFleetActions</a>  </li> <li>  <a>StopFleetActions</a>  </li> </ul> </li> </ul>

        :param self:
        :param Actions:  List of actions to restart on the fleet.<br/>FleetId:  Unique identifier for a fleet<br/>
        :return:         """
        pass

    def start_game_session_placement(self, MaximumPlayerSessionCount: int, GameSessionQueueName: str, PlacementId: str, GameProperties: List[GameProperty]=None, GameSessionName: str=None, PlayerLatencies: List[PlayerLatency]=None, DesiredPlayerSessions: List[DesiredPlayerSession]=None, GameSessionData: str=None) -> StartGameSessionPlacementOutput:
        """Places a request for a new game session in a queue (see <a>CreateGameSessionQueue</a>). When processing a placement request, Amazon GameLift searches for available resources on the queue's destinations, scanning each until it finds resources or the placement request times out. A game session placement request can also request player sessions. When a new game session is successfully created, Amazon GameLift creates a player session for each player included in the request. When placing a game session, by default Amazon GameLift tries each fleet in the order they are listed in the queue configuration. Ideally, a queue's destinations are listed in preference order. Alternatively, when requesting a game session with players, you can also provide latency data for each player in relevant regions. Latency data indicates the performance lag a player experiences when connected to a fleet in the region. Amazon GameLift uses latency data to reorder the list of destinations to place the game session in a region with minimal lag. If latency data is provided for multiple players, Amazon GameLift calculates each region's average lag for all players and reorders to get the best game play across all players.  To place a new game session request, specify the following: <ul> <li> The queue name and a set of game session properties and settings </li> <li> A unique ID (such as a UUID) for the placement. You use this ID to track the status of the placement request </li> <li> (Optional) A set of player data and a unique player ID for each player that you are joining to the new game session (player data is optional, but if you include it, you must also provide a unique ID for each player) </li> <li> Latency data for all players (if you want to optimize game play for the players) </li> </ul> If successful, a new game session placement is created. To track the status of a placement request, call <a>DescribeGameSessionPlacement</a> and check the request's status. If the status is <code>FULFILLED</code>, a new game session has been created and a game session ARN and region are referenced. If the placement request times out, you can resubmit the request or retry it with a different queue.  <ul> <li>  <a>CreateGameSession</a>  </li> <li>  <a>DescribeGameSessions</a>  </li> <li>  <a>DescribeGameSessionDetails</a>  </li> <li>  <a>SearchGameSessions</a>  </li> <li>  <a>UpdateGameSession</a>  </li> <li>  <a>GetGameSessionLogUrl</a>  </li> <li> Game session placements <ul> <li>  <a>StartGameSessionPlacement</a>  </li> <li>  <a>DescribeGameSessionPlacement</a>  </li> <li>  <a>StopGameSessionPlacement</a>  </li> </ul> </li> </ul>

        :param self:
        :param GameSessionData:  Set of custom game session properties, formatted as a single string value. This data is passed to a game server process in the <a>GameSession</a> object with a request to start a new game session (see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-sdk-server-api.html#gamelift-sdk-server-startsession">Start a Game Session</a>).<br/>DesiredPlayerSessions:  Set of information on each player to create a player session for.<br/>PlayerLatencies:  Set of values, expressed in milliseconds, indicating the amount of latency that a player experiences when connected to AWS regions. This information is used to try to place the new game session where it can offer the best possible gameplay experience for the players. <br/>GameSessionName:  Descriptive label that is associated with a game session. Session names do not need to be unique.<br/>MaximumPlayerSessionCount:  Maximum number of players that can be connected simultaneously to the game session.<br/>GameProperties:  Set of custom properties for a game session, formatted as key:value pairs. These properties are passed to a game server process in the <a>GameSession</a> object with a request to start a new game session (see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-sdk-server-api.html#gamelift-sdk-server-startsession">Start a Game Session</a>).<br/>GameSessionQueueName:  Name of the queue to use to place the new game session.<br/>PlacementId:  Unique identifier to assign to the new game session placement. This value is developer-defined. The value must be unique across all regions and cannot be reused unless you are resubmitting a canceled or timed-out placement request.<br/>
        :return: GameSessionPlacement:  Object that describes the newly created game session placement. This object includes all the information provided in the request, as well as start/end time stamps and placement status. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MatchmakerData</i> Information on the matchmaking process for this game. Data is in JSON syntax, formatted as a string. It identifies the matchmaking configuration used to create the match, and contains data on all players assigned to the match, including player attributes and team assignments. For more details on matchmaker data, see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/match-server.html#match-server-data">Match Data</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionData</i> Set of custom game session properties, formatted as a single string value. This data is passed to a game server process in the <a>GameSession</a> object with a request to start a new game session (see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-sdk-server-api.html#gamelift-sdk-server-startsession">Start a Game Session</a>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlacedPlayerSessions</i> Collection of information on player sessions created in response to the game session placement request. These player sessions are created only once a new game session is successfully placed (placement status is <code>FULFILLED</code>). This information includes the player ID (as provided in the placement request) and the corresponding player session ID. Retrieve full player sessions by calling <a>DescribePlayerSessions</a> with the player session ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Port number for the game session. To connect to a Amazon GameLift game server, an app needs both the IP address and port number. This value is set once the new game session is placed (placement status is <code>FULFILLED</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IpAddress</i> IP address of the game session. To connect to a Amazon GameLift game server, an app needs both the IP address and port number. This value is set once the new game session is placed (placement status is <code>FULFILLED</code>). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> Time stamp indicating when this request was completed, canceled, or timed out.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTime</i> Time stamp indicating when this request was placed in the queue. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlayerLatencies</i> Set of values, expressed in milliseconds, indicating the amount of latency that a player experiences when connected to AWS regions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionRegion</i> Name of the region where the game session created by this placement request is running. This value is set once the new game session is placed (placement status is <code>FULFILLED</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionArn</i> Identifier for the game session created by this placement request. This value is set once the new game session is placed (placement status is <code>FULFILLED</code>). This identifier is unique across all regions. You can use this value as a <code>GameSessionId</code> value as needed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionId</i> Unique identifier for the game session. This value is set once the new game session is placed (placement status is <code>FULFILLED</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionName</i> Descriptive label that is associated with a game session. Session names do not need to be unique.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumPlayerSessionCount</i> Maximum number of players that can be connected simultaneously to the game session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameProperties</i> Set of custom properties for a game session, formatted as key:value pairs. These properties are passed to a game server process in the <a>GameSession</a> object with a request to start a new game session (see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-sdk-server-api.html#gamelift-sdk-server-startsession">Start a Game Session</a>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Current status of the game session placement request. <ul> <li>  <b>PENDING</b> -- The placement request is currently in the queue waiting to be processed. </li> <li>  <b>FULFILLED</b> -- A new game session and player sessions (if requested) have been successfully created. Values for <i>GameSessionArn</i> and <i>GameSessionRegion</i> are available.  </li> <li>  <b>CANCELLED</b> -- The placement request was canceled with a call to <a>StopGameSessionPlacement</a>. </li> <li>  <b>TIMED_OUT</b> -- A new game session was not successfully created before the time limit expired. You can resubmit the placement request as needed. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionQueueName</i> Descriptive label that is associated with game session queue. Queue names must be unique within each region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlacementId</i> Unique identifier for a game session placement.<br/>        """
        pass

    def start_match_backfill(self, Players: List[Player], GameSessionArn: str, ConfigurationName: str, TicketId: str=None) -> StartMatchBackfillOutput:
        """Finds new players to fill open slots in an existing game session. This operation can be used to add players to matched games that start with fewer than the maximum number of players or to replace players when they drop out. By backfilling with the same matchmaker used to create the original match, you ensure that new players meet the match criteria and maintain a consistent experience throughout the game session. You can backfill a match anytime after a game session has been created.  To request a match backfill, specify a unique ticket ID, the existing game session's ARN, a matchmaking configuration, and a set of data that describes all current players in the game session. If successful, a match backfill ticket is created and returned with status set to QUEUED. The ticket is placed in the matchmaker's ticket pool and processed. Track the status of the ticket to respond as needed.  The process of finding backfill matches is essentially identical to the initial matchmaking process. The matchmaker searches the pool and groups tickets together to form potential matches, allowing only one backfill ticket per potential match. Once the a match is formed, the matchmaker creates player sessions for the new players. All tickets in the match are updated with the game session's connection information, and the <a>GameSession</a> object is updated to include matchmaker data on the new players. For more detail on how match backfill requests are processed, see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-match.html"> How Amazon GameLift FlexMatch Works</a>.   <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/match-backfill.html"> Backfill Existing Games with FlexMatch</a>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-match.html"> How GameLift FlexMatch Works</a>   <b>Related operations</b>  <ul> <li>  <a>StartMatchmaking</a>  </li> <li>  <a>DescribeMatchmaking</a>  </li> <li>  <a>StopMatchmaking</a>  </li> <li>  <a>AcceptMatch</a>  </li> <li>  <a>StartMatchBackfill</a>  </li> </ul>

        :param self:
        :param Players:  Match information on all players that are currently assigned to the game session. This information is used by the matchmaker to find new players and add them to the existing game. <ul> <li> PlayerID, PlayerAttributes, Team -\\- This information is maintained in the <a>GameSession</a> object, <code>MatchmakerData</code> property, for all players who are currently assigned to the game session. The matchmaker data is in JSON syntax, formatted as a string. For more details, see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/match-server.html#match-server-data"> Match Data</a>.  </li> <li> LatencyInMs -\\- If the matchmaker uses player latency, include a latency value, in milliseconds, for the region that the game session is currently in. Do not include latency values for any other region. </li> </ul><br/>GameSessionArn:  Amazon Resource Name (<a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/s3-arn-format.html">ARN</a>) that is assigned to a game session and uniquely identifies it. <br/>ConfigurationName:  Name of the matchmaker to use for this request. The name of the matchmaker that was used with the original game session is listed in the <a>GameSession</a> object, <code>MatchmakerData</code> property. This property contains a matchmaking configuration ARN value, which includes the matchmaker name. (In the ARN value "arn:aws:gamelift:us-west-2:111122223333:matchmakingconfiguration/MM-4v4", the matchmaking configuration name is "MM-4v4".) Use only the name for this parameter.<br/>TicketId:  Unique identifier for a matchmaking ticket. If no ticket ID is specified here, Amazon GameLift will generate one in the form of a UUID. Use this identifier to track the match backfill ticket status and retrieve match results.<br/>
        :return: MatchmakingTicket:  Ticket representing the backfill matchmaking request. This object includes the information in the request, ticket status, and match results as generated during the matchmaking process.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedWaitTime</i> Average amount of time (in seconds) that players are currently waiting for a match. If there is not enough recent data, this property may be empty.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionConnectionInfo</i> Identifier and connection information of the game session created for the match. This information is added to the ticket only after the matchmaking request has been successfully completed.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>GameSessionConnectionInfo</b>: Identifier and connection information of the game session created for the match. This information is added to the ticket only after the matchmaking request has been successfully completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Players</i> A set of <code>Player</code> objects, each representing a player to find matches for. Players are identified by a unique player ID and may include latency data for use during matchmaking. If the ticket is in status <code>COMPLETED</code>, the <code>Player</code> objects include the team the players were assigned to in the resulting match.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> Time stamp indicating when this matchmaking request stopped being processed due to success, failure, or cancellation. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTime</i> Time stamp indicating when this matchmaking request was received. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> Additional information about the current status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusReason</i> Code to explain the current status. For example, a status reason may indicate when a ticket has returned to <code>SEARCHING</code> status after a proposed match fails to receive player acceptances.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Current status of the matchmaking request. <ul> <li>  <b>QUEUED</b> -- The matchmaking request has been received and is currently waiting to be processed. </li> <li>  <b>SEARCHING</b> -- The matchmaking request is currently being processed.  </li> <li>  <b>REQUIRES_ACCEPTANCE</b> -- A match has been proposed and the players must accept the match (see <a>AcceptMatch</a>). This status is used only with requests that use a matchmaking configuration with a player acceptance requirement. </li> <li>  <b>PLACING</b> -- The FlexMatch engine has matched players and is in the process of placing a new game session for the match. </li> <li>  <b>COMPLETED</b> -- Players have been matched and a game session is ready to host the players. A ticket in this state contains the necessary connection information for players. </li> <li>  <b>FAILED</b> -- The matchmaking request was not completed. </li> <li>  <b>CANCELLED</b> -- The matchmaking request was canceled. This may be the result of a call to <a>StopMatchmaking</a> or a proposed match that one or more players failed to accept. </li> <li>  <b>TIMED_OUT</b> -- The matchmaking request was not successful within the duration specified in the matchmaking configuration.  </li> </ul> <note> Matchmaking requests that fail to successfully complete (statuses FAILED, CANCELLED, TIMED_OUT) can be resubmitted as new requests with new ticket IDs. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigurationName</i> Name of the <a>MatchmakingConfiguration</a> that is used with this ticket. Matchmaking configurations determine how players are grouped into a match and how a new game session is created for the match.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TicketId</i> Unique identifier for a matchmaking ticket.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MatchedPlayerSessions</i> Collection of player session IDs, one for each player ID that was included in the original matchmaking request. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Port number for the game session. To connect to a Amazon GameLift game server, an app needs both the IP address and port number.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IpAddress</i> IP address of the game session. To connect to a Amazon GameLift game server, an app needs both the IP address and port number.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionArn</i> Amazon Resource Name (<a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/s3-arn-format.html">ARN</a>) that is assigned to a game session and uniquely identifies it.<br/>        """
        pass

    def start_matchmaking(self, Players: List[Player], ConfigurationName: str, TicketId: str=None) -> StartMatchmakingOutput:
        """Uses FlexMatch to create a game match for a group of players based on custom matchmaking rules, and starts a new game for the matched players. Each matchmaking request specifies the type of match to build (team configuration, rules for an acceptable match, etc.). The request also specifies the players to find a match for and where to host the new game session for optimal performance. A matchmaking request might start with a single player or a group of players who want to play together. FlexMatch finds additional players as needed to fill the match. Match type, rules, and the queue used to place a new game session are defined in a <code>MatchmakingConfiguration</code>.  To start matchmaking, provide a unique ticket ID, specify a matchmaking configuration, and include the players to be matched. You must also include a set of player attributes relevant for the matchmaking configuration. If successful, a matchmaking ticket is returned with status set to <code>QUEUED</code>. Track the status of the ticket to respond as needed and acquire game session connection information for successfully completed matches.  <b>Tracking ticket status</b> -- A couple of options are available for tracking the status of matchmaking requests:  <ul> <li> Polling -- Call <code>DescribeMatchmaking</code>. This operation returns the full ticket object, including current status and (for completed tickets) game session connection info. We recommend polling no more than once every 10 seconds. </li> <li> Notifications -- Get event notifications for changes in ticket status using Amazon Simple Notification Service (SNS). Notifications are easy to set up (see <a>CreateMatchmakingConfiguration</a>) and typically deliver match status changes faster and more efficiently than polling. We recommend that you use polling to back up to notifications (since delivery is not guaranteed) and call <code>DescribeMatchmaking</code> only when notifications are not received within 30 seconds. </li> </ul>  <b>Processing a matchmaking request</b> -- FlexMatch handles a matchmaking request as follows:  <ol> <li> Your client code submits a <code>StartMatchmaking</code> request for one or more players and tracks the status of the request ticket.  </li> <li> FlexMatch uses this ticket and others in process to build an acceptable match. When a potential match is identified, all tickets in the proposed match are advanced to the next status.  </li> <li> If the match requires player acceptance (set in the matchmaking configuration), the tickets move into status <code>REQUIRES_ACCEPTANCE</code>. This status triggers your client code to solicit acceptance from all players in every ticket involved in the match, and then call <a>AcceptMatch</a> for each player. If any player rejects or fails to accept the match before a specified timeout, the proposed match is dropped (see <code>AcceptMatch</code> for more details). </li> <li> Once a match is proposed and accepted, the matchmaking tickets move into status <code>PLACING</code>. FlexMatch locates resources for a new game session using the game session queue (set in the matchmaking configuration) and creates the game session based on the match data.  </li> <li> When the match is successfully placed, the matchmaking tickets move into <code>COMPLETED</code> status. Connection information (including game session endpoint and player session) is added to the matchmaking tickets. Matched players can use the connection information to join the game.  </li> </ol>  <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/match-client.html"> Add FlexMatch to a Game Client</a>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/match-notification.html"> Set Up FlexMatch Event Notification</a>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/match-tasks.html"> FlexMatch Integration Roadmap</a>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-match.html"> How GameLift FlexMatch Works</a>   <b>Related operations</b>  <ul> <li>  <a>StartMatchmaking</a>  </li> <li>  <a>DescribeMatchmaking</a>  </li> <li>  <a>StopMatchmaking</a>  </li> <li>  <a>AcceptMatch</a>  </li> <li>  <a>StartMatchBackfill</a>  </li> </ul>

        :param self:
        :param Players:  Information on each player to be matched. This information must include a player ID, and may contain player attributes and latency data to be used in the matchmaking process. After a successful match, <code>Player</code> objects contain the name of the team the player is assigned to.<br/>ConfigurationName:  Name of the matchmaking configuration to use for this request. Matchmaking configurations must exist in the same region as this request.<br/>TicketId:  Unique identifier for a matchmaking ticket. If no ticket ID is specified here, Amazon GameLift will generate one in the form of a UUID. Use this identifier to track the matchmaking ticket status and retrieve match results.<br/>
        :return: MatchmakingTicket:  Ticket representing the matchmaking request. This object include the information included in the request, ticket status, and match results as generated during the matchmaking process.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedWaitTime</i> Average amount of time (in seconds) that players are currently waiting for a match. If there is not enough recent data, this property may be empty.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionConnectionInfo</i> Identifier and connection information of the game session created for the match. This information is added to the ticket only after the matchmaking request has been successfully completed.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>GameSessionConnectionInfo</b>: Identifier and connection information of the game session created for the match. This information is added to the ticket only after the matchmaking request has been successfully completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Players</i> A set of <code>Player</code> objects, each representing a player to find matches for. Players are identified by a unique player ID and may include latency data for use during matchmaking. If the ticket is in status <code>COMPLETED</code>, the <code>Player</code> objects include the team the players were assigned to in the resulting match.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> Time stamp indicating when this matchmaking request stopped being processed due to success, failure, or cancellation. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTime</i> Time stamp indicating when this matchmaking request was received. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> Additional information about the current status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusReason</i> Code to explain the current status. For example, a status reason may indicate when a ticket has returned to <code>SEARCHING</code> status after a proposed match fails to receive player acceptances.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Current status of the matchmaking request. <ul> <li>  <b>QUEUED</b> -- The matchmaking request has been received and is currently waiting to be processed. </li> <li>  <b>SEARCHING</b> -- The matchmaking request is currently being processed.  </li> <li>  <b>REQUIRES_ACCEPTANCE</b> -- A match has been proposed and the players must accept the match (see <a>AcceptMatch</a>). This status is used only with requests that use a matchmaking configuration with a player acceptance requirement. </li> <li>  <b>PLACING</b> -- The FlexMatch engine has matched players and is in the process of placing a new game session for the match. </li> <li>  <b>COMPLETED</b> -- Players have been matched and a game session is ready to host the players. A ticket in this state contains the necessary connection information for players. </li> <li>  <b>FAILED</b> -- The matchmaking request was not completed. </li> <li>  <b>CANCELLED</b> -- The matchmaking request was canceled. This may be the result of a call to <a>StopMatchmaking</a> or a proposed match that one or more players failed to accept. </li> <li>  <b>TIMED_OUT</b> -- The matchmaking request was not successful within the duration specified in the matchmaking configuration.  </li> </ul> <note> Matchmaking requests that fail to successfully complete (statuses FAILED, CANCELLED, TIMED_OUT) can be resubmitted as new requests with new ticket IDs. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigurationName</i> Name of the <a>MatchmakingConfiguration</a> that is used with this ticket. Matchmaking configurations determine how players are grouped into a match and how a new game session is created for the match.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TicketId</i> Unique identifier for a matchmaking ticket.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MatchedPlayerSessions</i> Collection of player session IDs, one for each player ID that was included in the original matchmaking request. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Port number for the game session. To connect to a Amazon GameLift game server, an app needs both the IP address and port number.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IpAddress</i> IP address of the game session. To connect to a Amazon GameLift game server, an app needs both the IP address and port number.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionArn</i> Amazon Resource Name (<a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/s3-arn-format.html">ARN</a>) that is assigned to a game session and uniquely identifies it.<br/>        """
        pass

    def stop_fleet_actions(self, Actions: List[FleetAction], FleetId: str) -> StopFleetActionsOutput:
        """Suspends activity on a fleet. Currently, this operation is used to stop a fleet's auto-scaling activity. It is used to temporarily stop scaling events triggered by the fleet's scaling policies. The policies can be retained and auto-scaling activity can be restarted using <a>StartFleetActions</a>. You can view a fleet's stopped actions using <a>DescribeFleetAttributes</a>. To stop fleet actions, specify the fleet ID and the type of actions to suspend. When auto-scaling fleet actions are stopped, Amazon GameLift no longer initiates scaling events except to maintain the fleet's desired instances setting (<a>FleetCapacity</a>. Changes to the fleet's capacity must be done manually using <a>UpdateFleetCapacity</a>.   <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/fleets-intro.html"> Working with Fleets</a>.  <b>Related operations</b>  <ul> <li>  <a>CreateFleet</a>  </li> <li>  <a>ListFleets</a>  </li> <li>  <a>DeleteFleet</a>  </li> <li> Describe fleets: <ul> <li>  <a>DescribeFleetAttributes</a>  </li> <li>  <a>DescribeFleetCapacity</a>  </li> <li>  <a>DescribeFleetPortSettings</a>  </li> <li>  <a>DescribeFleetUtilization</a>  </li> <li>  <a>DescribeRuntimeConfiguration</a>  </li> <li>  <a>DescribeEC2InstanceLimits</a>  </li> <li>  <a>DescribeFleetEvents</a>  </li> </ul> </li> <li> Update fleets: <ul> <li>  <a>UpdateFleetAttributes</a>  </li> <li>  <a>UpdateFleetCapacity</a>  </li> <li>  <a>UpdateFleetPortSettings</a>  </li> <li>  <a>UpdateRuntimeConfiguration</a>  </li> </ul> </li> <li> Manage fleet actions: <ul> <li>  <a>StartFleetActions</a>  </li> <li>  <a>StopFleetActions</a>  </li> </ul> </li> </ul>

        :param self:
        :param Actions:  List of actions to suspend on the fleet. <br/>FleetId:  Unique identifier for a fleet<br/>
        :return:         """
        pass

    def stop_game_session_placement(self, PlacementId: str) -> StopGameSessionPlacementOutput:
        """Cancels a game session placement that is in <code>PENDING</code> status. To stop a placement, provide the placement ID values. If successful, the placement is moved to <code>CANCELLED</code> status. <ul> <li>  <a>CreateGameSession</a>  </li> <li>  <a>DescribeGameSessions</a>  </li> <li>  <a>DescribeGameSessionDetails</a>  </li> <li>  <a>SearchGameSessions</a>  </li> <li>  <a>UpdateGameSession</a>  </li> <li>  <a>GetGameSessionLogUrl</a>  </li> <li> Game session placements <ul> <li>  <a>StartGameSessionPlacement</a>  </li> <li>  <a>DescribeGameSessionPlacement</a>  </li> <li>  <a>StopGameSessionPlacement</a>  </li> </ul> </li> </ul>

        :param self:
        :param PlacementId:  Unique identifier for a game session placement to cancel.<br/>
        :return: GameSessionPlacement:  Object that describes the canceled game session placement, with <code>CANCELLED</code> status and an end time stamp. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MatchmakerData</i> Information on the matchmaking process for this game. Data is in JSON syntax, formatted as a string. It identifies the matchmaking configuration used to create the match, and contains data on all players assigned to the match, including player attributes and team assignments. For more details on matchmaker data, see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/match-server.html#match-server-data">Match Data</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionData</i> Set of custom game session properties, formatted as a single string value. This data is passed to a game server process in the <a>GameSession</a> object with a request to start a new game session (see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-sdk-server-api.html#gamelift-sdk-server-startsession">Start a Game Session</a>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlacedPlayerSessions</i> Collection of information on player sessions created in response to the game session placement request. These player sessions are created only once a new game session is successfully placed (placement status is <code>FULFILLED</code>). This information includes the player ID (as provided in the placement request) and the corresponding player session ID. Retrieve full player sessions by calling <a>DescribePlayerSessions</a> with the player session ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Port number for the game session. To connect to a Amazon GameLift game server, an app needs both the IP address and port number. This value is set once the new game session is placed (placement status is <code>FULFILLED</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IpAddress</i> IP address of the game session. To connect to a Amazon GameLift game server, an app needs both the IP address and port number. This value is set once the new game session is placed (placement status is <code>FULFILLED</code>). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> Time stamp indicating when this request was completed, canceled, or timed out.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTime</i> Time stamp indicating when this request was placed in the queue. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlayerLatencies</i> Set of values, expressed in milliseconds, indicating the amount of latency that a player experiences when connected to AWS regions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionRegion</i> Name of the region where the game session created by this placement request is running. This value is set once the new game session is placed (placement status is <code>FULFILLED</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionArn</i> Identifier for the game session created by this placement request. This value is set once the new game session is placed (placement status is <code>FULFILLED</code>). This identifier is unique across all regions. You can use this value as a <code>GameSessionId</code> value as needed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionId</i> Unique identifier for the game session. This value is set once the new game session is placed (placement status is <code>FULFILLED</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionName</i> Descriptive label that is associated with a game session. Session names do not need to be unique.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumPlayerSessionCount</i> Maximum number of players that can be connected simultaneously to the game session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameProperties</i> Set of custom properties for a game session, formatted as key:value pairs. These properties are passed to a game server process in the <a>GameSession</a> object with a request to start a new game session (see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-sdk-server-api.html#gamelift-sdk-server-startsession">Start a Game Session</a>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Current status of the game session placement request. <ul> <li>  <b>PENDING</b> -- The placement request is currently in the queue waiting to be processed. </li> <li>  <b>FULFILLED</b> -- A new game session and player sessions (if requested) have been successfully created. Values for <i>GameSessionArn</i> and <i>GameSessionRegion</i> are available.  </li> <li>  <b>CANCELLED</b> -- The placement request was canceled with a call to <a>StopGameSessionPlacement</a>. </li> <li>  <b>TIMED_OUT</b> -- A new game session was not successfully created before the time limit expired. You can resubmit the placement request as needed. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionQueueName</i> Descriptive label that is associated with game session queue. Queue names must be unique within each region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlacementId</i> Unique identifier for a game session placement.<br/>        """
        pass

    def stop_matchmaking(self, TicketId: str) -> StopMatchmakingOutput:
        """Cancels a matchmaking ticket or match backfill ticket that is currently being processed. To stop the matchmaking operation, specify the ticket ID. If successful, work on the ticket is stopped, and the ticket status is changed to <code>CANCELLED</code>. This call is also used to turn off automatic backfill for an individual game session. This is for game sessions that are created with a matchmaking configuration that has automatic backfill enabled. The ticket ID is included in the <code>MatchmakerData</code> of an updated game session object, which is provided to the game server. <note> If the action is successful, the service sends back an empty JSON struct with the HTTP 200 response (not an empty HTTP body). </note>  <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/match-client.html"> Add FlexMatch to a Game Client</a>   <b>Related operations</b>  <ul> <li>  <a>StartMatchmaking</a>  </li> <li>  <a>DescribeMatchmaking</a>  </li> <li>  <a>StopMatchmaking</a>  </li> <li>  <a>AcceptMatch</a>  </li> <li>  <a>StartMatchBackfill</a>  </li> </ul>

        :param self:
        :param TicketId:  Unique identifier for a matchmaking ticket.<br/>
        :return:         """
        pass

    def update_alias(self, AliasId: str, Name: str=None, Description: str=None, RoutingStrategy: RoutingStrategy=None) -> UpdateAliasOutput:
        """Updates properties for an alias. To update properties, specify the alias ID to be updated and provide the information to be changed. To reassign an alias to another fleet, provide an updated routing strategy. If successful, the updated alias record is returned. <ul> <li>  <a>CreateAlias</a>  </li> <li>  <a>ListAliases</a>  </li> <li>  <a>DescribeAlias</a>  </li> <li>  <a>UpdateAlias</a>  </li> <li>  <a>DeleteAlias</a>  </li> <li>  <a>ResolveAlias</a>  </li> </ul>

        :param self:
        :param RoutingStrategy:  Object that specifies the fleet and routing type to use for the alias.:<br/>Description:  Human-readable description of an alias.<br/>Name:  Descriptive label that is associated with an alias. Alias names do not need to be unique.<br/>AliasId:  Unique identifier for a fleet alias. Specify the alias you want to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> Message text to be used with a terminal routing strategy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FleetId</i> Unique identifier for a fleet that the alias points to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> Type of routing strategy. Possible routing types include the following: <ul> <li>  <b>SIMPLE</b> -- The alias resolves to one specific fleet. Use this type when routing to active fleets. </li> <li>  <b>TERMINAL</b> -- The alias does not resolve to a fleet but instead can be used to display a message to the user. A terminal alias throws a TerminalRoutingStrategyException with the <a>RoutingStrategy</a> message embedded. </li> </ul><br/>
        :return: Alias:  Object that contains the updated alias configuration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdatedTime</i> Time stamp indicating when this data object was last modified. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> Time stamp indicating when this data object was created. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoutingStrategy</i> Alias configuration for the alias, including routing type and settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RoutingStrategy</b>: Alias configuration for the alias, including routing type and settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> Human-readable description of an alias.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AliasArn</i> Unique identifier for an alias; alias ARNs are unique across all regions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Descriptive label that is associated with an alias. Alias names do not need to be unique.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AliasId</i> Unique identifier for an alias; alias IDs are unique within a region.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> Message text to be used with a terminal routing strategy.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FleetId</i> Unique identifier for a fleet that the alias points to.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> Type of routing strategy. Possible routing types include the following: <ul> <li>  <b>SIMPLE</b> -- The alias resolves to one specific fleet. Use this type when routing to active fleets. </li> <li>  <b>TERMINAL</b> -- The alias does not resolve to a fleet but instead can be used to display a message to the user. A terminal alias throws a TerminalRoutingStrategyException with the <a>RoutingStrategy</a> message embedded. </li> </ul><br/>        """
        pass

    def update_build(self, BuildId: str, Name: str=None, Version: str=None) -> UpdateBuildOutput:
        """Updates metadata in a build record, including the build name and version. To update the metadata, specify the build ID to update and provide the new values. If successful, a build object containing the updated metadata is returned.  <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/build-intro.html"> Working with Builds</a>   <b>Related operations</b>  <ul> <li>  <a>CreateBuild</a>  </li> <li>  <a>ListBuilds</a>  </li> <li>  <a>DescribeBuild</a>  </li> <li>  <a>UpdateBuild</a>  </li> <li>  <a>DeleteBuild</a>  </li> </ul>

        :param self:
        :param Version:  Version that is associated with a build or script. Version strings do not need to be unique.<br/>Name:  Descriptive label that is associated with a build. Build names do not need to be unique. <br/>BuildId:  Unique identifier for a build to update.<br/>
        :return: Build:  Object that contains the updated build record.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> Time stamp indicating when this data object was created. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OperatingSystem</i> Operating system that the game server binaries are built to run on. This value determines the type of fleet resources that you can use for this build.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SizeOnDisk</i> File size of the uploaded game build, expressed in bytes. When the build status is <code>INITIALIZED</code>, this value is 0.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Current status of the build. Possible build statuses include the following: <ul> <li>  <b>INITIALIZED</b> -- A new build has been defined, but no files have been uploaded. You cannot create fleets for builds that are in this status. When a build is successfully created, the build status is set to this value.  </li> <li>  <b>READY</b> -- The game build has been successfully uploaded. You can now create new fleets for this build. </li> <li>  <b>FAILED</b> -- The game build upload failed. You cannot create new fleets for this build.  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> Version that is associated with a build or script. Version strings do not need to be unique. This value can be set using <a>CreateBuild</a> or <a>UpdateBuild</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Descriptive label that is associated with a build. Build names do not need to be unique. It can be set using <a>CreateBuild</a> or <a>UpdateBuild</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BuildId</i> Unique identifier for a build.<br/>        """
        pass

    def update_fleet_attributes(self, FleetId: str, Name: str=None, Description: str=None, NewGameSessionProtectionPolicy: str=None, ResourceCreationLimitPolicy: ResourceCreationLimitPolicy=None, MetricGroups: List[MetricGroup]=None) -> UpdateFleetAttributesOutput:
        """Updates fleet properties, including name and description, for a fleet. To update metadata, specify the fleet ID and the property values that you want to change. If successful, the fleet ID for the updated fleet is returned.  <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/fleets-intro.html"> Working with Fleets</a>.  <b>Related operations</b>  <ul> <li>  <a>CreateFleet</a>  </li> <li>  <a>ListFleets</a>  </li> <li>  <a>DeleteFleet</a>  </li> <li> Describe fleets: <ul> <li>  <a>DescribeFleetAttributes</a>  </li> <li>  <a>DescribeFleetCapacity</a>  </li> <li>  <a>DescribeFleetPortSettings</a>  </li> <li>  <a>DescribeFleetUtilization</a>  </li> <li>  <a>DescribeRuntimeConfiguration</a>  </li> <li>  <a>DescribeEC2InstanceLimits</a>  </li> <li>  <a>DescribeFleetEvents</a>  </li> </ul> </li> <li> Update fleets: <ul> <li>  <a>UpdateFleetAttributes</a>  </li> <li>  <a>UpdateFleetCapacity</a>  </li> <li>  <a>UpdateFleetPortSettings</a>  </li> <li>  <a>UpdateRuntimeConfiguration</a>  </li> </ul> </li> <li> Manage fleet actions: <ul> <li>  <a>StartFleetActions</a>  </li> <li>  <a>StopFleetActions</a>  </li> </ul> </li> </ul>

        :param self:
        :param MetricGroups:  Names of metric groups to include this fleet in. Amazon CloudWatch uses a fleet metric group is to aggregate metrics from multiple fleets. Use an existing metric group name to add this fleet to the group. Or use a new name to create a new metric group. A fleet can only be included in one metric group at a time.<br/>ResourceCreationLimitPolicy:  Policy that limits the number of game sessions an individual player can create over a span of time. :<br/>NewGameSessionProtectionPolicy:  Game session protection policy to apply to all new instances created in this fleet. Instances that already exist are not affected. You can set protection for individual instances using <a>UpdateGameSession</a>. <ul> <li>  <b>NoProtection</b> -- The game session can be terminated during a scale-down event. </li> <li>  <b>FullProtection</b> -- If the game session is in an <code>ACTIVE</code> status, it cannot be terminated during a scale-down event. </li> </ul><br/>Description:  Human-readable description of a fleet.<br/>Name:  Descriptive label that is associated with a fleet. Fleet names do not need to be unique.<br/>FleetId:  Unique identifier for a fleet to update attribute metadata for.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyPeriodInMinutes</i> Time span used in evaluating the resource creation limit policy. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NewGameSessionsPerCreator</i> Maximum number of game sessions that an individual can create during the policy period. <br/>
        :return: FleetId:  Unique identifier for a fleet that was updated.<br/>        """
        pass

    def update_fleet_capacity(self, FleetId: str, DesiredInstances: int=None, MinSize: int=None, MaxSize: int=None) -> UpdateFleetCapacityOutput:
        """Updates capacity settings for a fleet. Use this action to specify the number of EC2 instances (hosts) that you want this fleet to contain. Before calling this action, you may want to call <a>DescribeEC2InstanceLimits</a> to get the maximum capacity based on the fleet's EC2 instance type. Specify minimum and maximum number of instances. Amazon GameLift will not change fleet capacity to values fall outside of this range. This is particularly important when using auto-scaling (see <a>PutScalingPolicy</a>) to allow capacity to adjust based on player demand while imposing limits on automatic adjustments. To update fleet capacity, specify the fleet ID and the number of instances you want the fleet to host. If successful, Amazon GameLift starts or terminates instances so that the fleet's active instance count matches the desired instance count. You can view a fleet's current capacity information by calling <a>DescribeFleetCapacity</a>. If the desired instance count is higher than the instance type's limit, the "Limit Exceeded" exception occurs.  <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/fleets-intro.html"> Working with Fleets</a>.  <b>Related operations</b>  <ul> <li>  <a>CreateFleet</a>  </li> <li>  <a>ListFleets</a>  </li> <li>  <a>DeleteFleet</a>  </li> <li> Describe fleets: <ul> <li>  <a>DescribeFleetAttributes</a>  </li> <li>  <a>DescribeFleetCapacity</a>  </li> <li>  <a>DescribeFleetPortSettings</a>  </li> <li>  <a>DescribeFleetUtilization</a>  </li> <li>  <a>DescribeRuntimeConfiguration</a>  </li> <li>  <a>DescribeEC2InstanceLimits</a>  </li> <li>  <a>DescribeFleetEvents</a>  </li> </ul> </li> <li> Update fleets: <ul> <li>  <a>UpdateFleetAttributes</a>  </li> <li>  <a>UpdateFleetCapacity</a>  </li> <li>  <a>UpdateFleetPortSettings</a>  </li> <li>  <a>UpdateRuntimeConfiguration</a>  </li> </ul> </li> <li> Manage fleet actions: <ul> <li>  <a>StartFleetActions</a>  </li> <li>  <a>StopFleetActions</a>  </li> </ul> </li> </ul>

        :param self:
        :param MaxSize:  Maximum value allowed for the fleet's instance count. Default if not set is 1.<br/>MinSize:  Minimum value allowed for the fleet's instance count. Default if not set is 0.<br/>DesiredInstances:  Number of EC2 instances you want this fleet to host.<br/>FleetId:  Unique identifier for a fleet to update capacity for.<br/>
        :return: FleetId:  Unique identifier for a fleet that was updated.<br/>        """
        pass

    def update_fleet_port_settings(self, FleetId: str, InboundPermissionAuthorizations: List[IpPermission]=None, InboundPermissionRevocations: List[IpPermission]=None) -> UpdateFleetPortSettingsOutput:
        """Updates port settings for a fleet. To update settings, specify the fleet ID to be updated and list the permissions you want to update. List the permissions you want to add in <code>InboundPermissionAuthorizations</code>, and permissions you want to remove in <code>InboundPermissionRevocations</code>. Permissions to be removed must match existing fleet permissions. If successful, the fleet ID for the updated fleet is returned.  <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/fleets-intro.html"> Working with Fleets</a>.  <b>Related operations</b>  <ul> <li>  <a>CreateFleet</a>  </li> <li>  <a>ListFleets</a>  </li> <li>  <a>DeleteFleet</a>  </li> <li> Describe fleets: <ul> <li>  <a>DescribeFleetAttributes</a>  </li> <li>  <a>DescribeFleetCapacity</a>  </li> <li>  <a>DescribeFleetPortSettings</a>  </li> <li>  <a>DescribeFleetUtilization</a>  </li> <li>  <a>DescribeRuntimeConfiguration</a>  </li> <li>  <a>DescribeEC2InstanceLimits</a>  </li> <li>  <a>DescribeFleetEvents</a>  </li> </ul> </li> <li> Update fleets: <ul> <li>  <a>UpdateFleetAttributes</a>  </li> <li>  <a>UpdateFleetCapacity</a>  </li> <li>  <a>UpdateFleetPortSettings</a>  </li> <li>  <a>UpdateRuntimeConfiguration</a>  </li> </ul> </li> <li> Manage fleet actions: <ul> <li>  <a>StartFleetActions</a>  </li> <li>  <a>StopFleetActions</a>  </li> </ul> </li> </ul>

        :param self:
        :param InboundPermissionRevocations:  Collection of port settings to be removed from the fleet record.<br/>InboundPermissionAuthorizations:  Collection of port settings to be added to the fleet record.<br/>FleetId:  Unique identifier for a fleet to update port settings for.<br/>
        :return: FleetId:  Unique identifier for a fleet that was updated.<br/>        """
        pass

    def update_game_session(self, GameSessionId: str, MaximumPlayerSessionCount: int=None, Name: str=None, PlayerSessionCreationPolicy: str=None, ProtectionPolicy: str=None) -> UpdateGameSessionOutput:
        """Updates game session properties. This includes the session name, maximum player count, protection policy, which controls whether or not an active game session can be terminated during a scale-down event, and the player session creation policy, which controls whether or not new players can join the session. To update a game session, specify the game session ID and the values you want to change. If successful, an updated <a>GameSession</a> object is returned.  <ul> <li>  <a>CreateGameSession</a>  </li> <li>  <a>DescribeGameSessions</a>  </li> <li>  <a>DescribeGameSessionDetails</a>  </li> <li>  <a>SearchGameSessions</a>  </li> <li>  <a>UpdateGameSession</a>  </li> <li>  <a>GetGameSessionLogUrl</a>  </li> <li> Game session placements <ul> <li>  <a>StartGameSessionPlacement</a>  </li> <li>  <a>DescribeGameSessionPlacement</a>  </li> <li>  <a>StopGameSessionPlacement</a>  </li> </ul> </li> </ul>

        :param self:
        :param ProtectionPolicy:  Game session protection policy to apply to this game session only. <ul> <li>  <b>NoProtection</b> -- The game session can be terminated during a scale-down event. </li> <li>  <b>FullProtection</b> -- If the game session is in an <code>ACTIVE</code> status, it cannot be terminated during a scale-down event. </li> </ul><br/>PlayerSessionCreationPolicy:  Policy determining whether or not the game session accepts new players.<br/>Name:  Descriptive label that is associated with a game session. Session names do not need to be unique.<br/>MaximumPlayerSessionCount:  Maximum number of players that can be connected simultaneously to the game session.<br/>GameSessionId:  Unique identifier for the game session to update.<br/>
        :return: GameSession:  Object that contains the updated game session metadata.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MatchmakerData</i> Information about the matchmaking process that was used to create the game session. It is in JSON syntax, formatted as a string. In addition the matchmaking configuration used, it contains data on all players assigned to the match, including player attributes and team assignments. For more details on matchmaker data, see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/match-server.html#match-server-data">Match Data</a>. Matchmaker data is useful when requesting match backfills, and is updated whenever new players are added during a successful backfill (see <a>StartMatchBackfill</a>). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionData</i> Set of custom game session properties, formatted as a single string value. This data is passed to a game server process in the <a>GameSession</a> object with a request to start a new game session (see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-sdk-server-api.html#gamelift-sdk-server-startsession">Start a Game Session</a>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatorId</i> Unique identifier for a player. This ID is used to enforce a resource protection policy (if one exists), that limits the number of game sessions a player can create.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlayerSessionCreationPolicy</i> Indicates whether or not the game session is accepting new players.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Port number for the game session. To connect to a Amazon GameLift game server, an app needs both the IP address and port number.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IpAddress</i> IP address of the game session. To connect to a Amazon GameLift game server, an app needs both the IP address and port number.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameProperties</i> Set of custom properties for a game session, formatted as key:value pairs. These properties are passed to a game server process in the <a>GameSession</a> object with a request to start a new game session (see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-sdk-server-api.html#gamelift-sdk-server-startsession">Start a Game Session</a>). You can search for active game sessions based on this custom data with <a>SearchGameSessions</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusReason</i> Provides additional information about game session status. <code>INTERRUPTED</code> indicates that the game session was hosted on a spot instance that was reclaimed, causing the active game session to be terminated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Current status of the game session. A game session must have an <code>ACTIVE</code> status to have player sessions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumPlayerSessionCount</i> Maximum number of players that can be connected simultaneously to the game session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentPlayerSessionCount</i> Number of players currently in the game session.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TerminationTime</i> Time stamp indicating when this data object was terminated. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> Time stamp indicating when this data object was created. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FleetId</i> Unique identifier for a fleet that the game session is running on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Descriptive label that is associated with a game session. Session names do not need to be unique.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionId</i> Unique identifier for the game session. A game session ARN has the following format: <code>arn:aws:gamelift:&lt;region&gt;::gamesession/&lt;fleet ID&gt;/&lt;custom ID string or idempotency token&gt;</code>.<br/>        """
        pass

    def update_game_session_queue(self, Name: str, TimeoutInSeconds: int=None, PlayerLatencyPolicies: List[PlayerLatencyPolicy]=None, Destinations: List[GameSessionQueueDestination]=None) -> UpdateGameSessionQueueOutput:
        """Updates settings for a game session queue, which determines how new game session requests in the queue are processed. To update settings, specify the queue name to be updated and provide the new settings. When updating destinations, provide a complete list of destinations.  <ul> <li>  <a>CreateGameSessionQueue</a>  </li> <li>  <a>DescribeGameSessionQueues</a>  </li> <li>  <a>UpdateGameSessionQueue</a>  </li> <li>  <a>DeleteGameSessionQueue</a>  </li> </ul>

        :param self:
        :param Destinations:  List of fleets that can be used to fulfill game session placement requests in the queue. Fleets are identified by either a fleet ARN or a fleet alias ARN. Destinations are listed in default preference order. When updating this list, provide a complete list of destinations.<br/>PlayerLatencyPolicies:  Collection of latency policies to apply when processing game sessions placement requests with player latency information. Multiple policies are evaluated in order of the maximum latency value, starting with the lowest latency values. With just one policy, it is enforced at the start of the game session placement for the duration period. With multiple policies, each policy is enforced consecutively for its duration period. For example, a queue might enforce a 60-second policy followed by a 120-second policy, and then no policy for the remainder of the placement. When updating policies, provide a complete collection of policies.<br/>TimeoutInSeconds:  Maximum time, in seconds, that a new game session placement request remains in the queue. When a request exceeds this time, the game session placement changes to a <code>TIMED_OUT</code> status.<br/>Name:  Descriptive label that is associated with game session queue. Queue names must be unique within each region.<br/>
        :return: GameSessionQueue:  Object that describes the newly updated game session queue.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Destinations</i> List of fleets that can be used to fulfill game session placement requests in the queue. Fleets are identified by either a fleet ARN or a fleet alias ARN. Destinations are listed in default preference order.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlayerLatencyPolicies</i> Collection of latency policies to apply when processing game sessions placement requests with player latency information. Multiple policies are evaluated in order of the maximum latency value, starting with the lowest latency values. With just one policy, it is enforced at the start of the game session placement for the duration period. With multiple policies, each policy is enforced consecutively for its duration period. For example, a queue might enforce a 60-second policy followed by a 120-second policy, and then no policy for the remainder of the placement. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeoutInSeconds</i> Maximum time, in seconds, that a new game session placement request remains in the queue. When a request exceeds this time, the game session placement changes to a <code>TIMED_OUT</code> status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionQueueArn</i> Amazon Resource Name (<a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/s3-arn-format.html">ARN</a>) that is assigned to a game session queue and uniquely identifies it. Format is <code>arn:aws:gamelift:&lt;region&gt;:&lt;aws account&gt;:gamesessionqueue/&lt;queue name&gt;</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Descriptive label that is associated with game session queue. Queue names must be unique within each region.<br/>        """
        pass

    def update_matchmaking_configuration(self, Name: str, Description: str=None, GameSessionQueueArns: List[ArnStringModel]=None, RequestTimeoutSeconds: int=None, AcceptanceTimeoutSeconds: int=None, AcceptanceRequired: BooleanModel=None, RuleSetName: str=None, NotificationTarget: str=None, AdditionalPlayerCount: int=None, CustomEventData: str=None, GameProperties: List[GameProperty]=None, GameSessionData: str=None, BackfillMode: str=None) -> UpdateMatchmakingConfigurationOutput:
        """Updates settings for a FlexMatch matchmaking configuration. These changes affect all matches and game sessions that are created after the update. To update settings, specify the configuration name to be updated and provide the new settings.   <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/match-configuration.html"> Design a FlexMatch Matchmaker</a>   <b>Related operations</b>  <ul> <li>  <a>CreateMatchmakingConfiguration</a>  </li> <li>  <a>DescribeMatchmakingConfigurations</a>  </li> <li>  <a>UpdateMatchmakingConfiguration</a>  </li> <li>  <a>DeleteMatchmakingConfiguration</a>  </li> <li>  <a>CreateMatchmakingRuleSet</a>  </li> <li>  <a>DescribeMatchmakingRuleSets</a>  </li> <li>  <a>ValidateMatchmakingRuleSet</a>  </li> <li>  <a>DeleteMatchmakingRuleSet</a>  </li> </ul>

        :param self:
        :param BackfillMode:  Method used to backfill game sessions created with this matchmaking configuration. Specify MANUAL when your game manages backfill requests manually or does not use the match backfill feature. Specify AUTOMATIC to have GameLift create a <a>StartMatchBackfill</a> request whenever a game session has one or more open slots. Learn more about manual and automatic backfill in <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/match-backfill.html">Backfill Existing Games with FlexMatch</a>.<br/>GameSessionData:  Set of custom game session properties, formatted as a single string value. This data is passed to a game server process in the <a>GameSession</a> object with a request to start a new game session (see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-sdk-server-api.html#gamelift-sdk-server-startsession">Start a Game Session</a>). This information is added to the new <a>GameSession</a> object that is created for a successful match. <br/>GameProperties:  Set of custom properties for a game session, formatted as key:value pairs. These properties are passed to a game server process in the <a>GameSession</a> object with a request to start a new game session (see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-sdk-server-api.html#gamelift-sdk-server-startsession">Start a Game Session</a>). This information is added to the new <a>GameSession</a> object that is created for a successful match. <br/>CustomEventData:  Information to add to all events related to the matchmaking configuration. <br/>AdditionalPlayerCount:  Number of player slots in a match to keep open for future players. For example, if the configuration's rule set specifies a match for a single 12-person team, and the additional player count is set to 2, only 10 players are selected for the match.<br/>NotificationTarget:  SNS topic ARN that is set up to receive matchmaking notifications. See <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/match-notification.html"> Setting up Notifications for Matchmaking</a> for more information.<br/>RuleSetName:  Unique identifier for a matchmaking rule set to use with this configuration. A matchmaking configuration can only use rule sets that are defined in the same region.<br/>AcceptanceRequired:  Flag that determines whether a match that was created with this configuration must be accepted by the matched players. To require acceptance, set to TRUE.<br/>AcceptanceTimeoutSeconds:  Length of time (in seconds) to wait for players to accept a proposed match. If any player rejects the match or fails to accept before the timeout, the ticket continues to look for an acceptable match.<br/>RequestTimeoutSeconds:  Maximum duration, in seconds, that a matchmaking ticket can remain in process before timing out. Requests that fail due to timing out can be resubmitted as needed.<br/>GameSessionQueueArns:  Amazon Resource Name (<a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/s3-arn-format.html">ARN</a>) that is assigned to a game session queue and uniquely identifies it. Format is <code>arn:aws:gamelift:&lt;region&gt;:&lt;aws account&gt;:gamesessionqueue/&lt;queue name&gt;</code>. These queues are used when placing game sessions for matches that are created with this matchmaking configuration. Queues can be located in any region.<br/>Description:  Descriptive label that is associated with matchmaking configuration.<br/>Name:  Unique identifier for a matchmaking configuration to update.<br/>
        :return: Configuration:  Object that describes the updated matchmaking configuration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackfillMode</i> Method used to backfill game sessions created with this matchmaking configuration. MANUAL indicates that the game makes backfill requests or does not use the match backfill feature. AUTOMATIC indicates that GameLift creates <a>StartMatchBackfill</a> requests whenever a game session has one or more open slots. Learn more about manual and automatic backfill in <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/match-backfill.html">Backfill Existing Games with FlexMatch</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionData</i> Set of custom game session properties, formatted as a single string value. This data is passed to a game server process in the <a>GameSession</a> object with a request to start a new game session (see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-sdk-server-api.html#gamelift-sdk-server-startsession">Start a Game Session</a>). This information is added to the new <a>GameSession</a> object that is created for a successful match. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameProperties</i> Set of custom properties for a game session, formatted as key:value pairs. These properties are passed to a game server process in the <a>GameSession</a> object with a request to start a new game session (see <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/gamelift-sdk-server-api.html#gamelift-sdk-server-startsession">Start a Game Session</a>). This information is added to the new <a>GameSession</a> object that is created for a successful match. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> Time stamp indicating when this data object was created. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomEventData</i> Information to attach to all events related to the matchmaking configuration. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdditionalPlayerCount</i> Number of player slots in a match to keep open for future players. For example, if the configuration's rule set specifies a match for a single 12-person team, and the additional player count is set to 2, only 10 players are selected for the match.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationTarget</i> SNS topic ARN that is set up to receive matchmaking notifications.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleSetName</i> Unique identifier for a matchmaking rule set to use with this configuration. A matchmaking configuration can only use rule sets that are defined in the same region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AcceptanceRequired</i> Flag that determines whether a match that was created with this configuration must be accepted by the matched players. To require acceptance, set to TRUE.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AcceptanceTimeoutSeconds</i> Length of time (in seconds) to wait for players to accept a proposed match. If any player rejects the match or fails to accept before the timeout, the ticket continues to look for an acceptable match.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestTimeoutSeconds</i> Maximum duration, in seconds, that a matchmaking ticket can remain in process before timing out. Requests that fail due to timing out can be resubmitted as needed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionQueueArns</i> Amazon Resource Name (<a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/s3-arn-format.html">ARN</a>) that is assigned to a game session queue and uniquely identifies it. Format is <code>arn:aws:gamelift:&lt;region&gt;:&lt;aws account&gt;:gamesessionqueue/&lt;queue name&gt;</code>. These queues are used when placing game sessions for matches that are created with this matchmaking configuration. Queues can be located in any region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> Descriptive label that is associated with matchmaking configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Unique identifier for a matchmaking configuration. This name is used to identify the configuration associated with a matchmaking request or ticket.<br/>        """
        pass

    def update_runtime_configuration(self, RuntimeConfiguration: RuntimeConfiguration, FleetId: str) -> UpdateRuntimeConfigurationOutput:
        """Updates the current run-time configuration for the specified fleet, which tells Amazon GameLift how to launch server processes on instances in the fleet. You can update a fleet's run-time configuration at any time after the fleet is created; it does not need to be in an <code>ACTIVE</code> status. To update run-time configuration, specify the fleet ID and provide a <code>RuntimeConfiguration</code> object with an updated set of server process configurations. Each instance in a Amazon GameLift fleet checks regularly for an updated run-time configuration and changes how it launches server processes to comply with the latest version. Existing server processes are not affected by the update; run-time configuration changes are applied gradually as existing processes shut down and new processes are launched during Amazon GameLift's normal process recycling activity.  <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/fleets-intro.html"> Working with Fleets</a>.  <b>Related operations</b>  <ul> <li>  <a>CreateFleet</a>  </li> <li>  <a>ListFleets</a>  </li> <li>  <a>DeleteFleet</a>  </li> <li> Describe fleets: <ul> <li>  <a>DescribeFleetAttributes</a>  </li> <li>  <a>DescribeFleetCapacity</a>  </li> <li>  <a>DescribeFleetPortSettings</a>  </li> <li>  <a>DescribeFleetUtilization</a>  </li> <li>  <a>DescribeRuntimeConfiguration</a>  </li> <li>  <a>DescribeEC2InstanceLimits</a>  </li> <li>  <a>DescribeFleetEvents</a>  </li> </ul> </li> <li> Update fleets: <ul> <li>  <a>UpdateFleetAttributes</a>  </li> <li>  <a>UpdateFleetCapacity</a>  </li> <li>  <a>UpdateFleetPortSettings</a>  </li> <li>  <a>UpdateRuntimeConfiguration</a>  </li> </ul> </li> <li> Manage fleet actions: <ul> <li>  <a>StartFleetActions</a>  </li> <li>  <a>StopFleetActions</a>  </li> </ul> </li> </ul>

        :param self:
        :param RuntimeConfiguration:  Instructions for launching server processes on each instance in the fleet. Server processes run either a custom game build executable or a Realtime Servers script. The run-time configuration lists the types of server processes to run on an instance and includes the following configuration settings: the server executable or launch script file, launch parameters, and the number of processes to run concurrently on each instance. A CreateFleet request must include a run-time configuration with at least one server process configuration.:<br/>FleetId:  Unique identifier for a fleet to update run-time configuration for.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionActivationTimeoutSeconds</i> Maximum amount of time (in seconds) that a game session can remain in status <code>ACTIVATING</code>. If the game session is not active before the timeout, activation is terminated and the game session status is changed to <code>TERMINATED</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxConcurrentGameSessionActivations</i> Maximum number of game sessions with status <code>ACTIVATING</code> to allow on an instance simultaneously. This setting limits the amount of instance resources that can be used for new game activations at any one time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerProcesses</i> Collection of server process configurations that describe which server processes to run on each instance in a fleet.<br/>
        :return: RuntimeConfiguration:  The run-time configuration currently in force. If the update was successful, this object matches the one in the request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GameSessionActivationTimeoutSeconds</i> Maximum amount of time (in seconds) that a game session can remain in status <code>ACTIVATING</code>. If the game session is not active before the timeout, activation is terminated and the game session status is changed to <code>TERMINATED</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxConcurrentGameSessionActivations</i> Maximum number of game sessions with status <code>ACTIVATING</code> to allow on an instance simultaneously. This setting limits the amount of instance resources that can be used for new game activations at any one time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerProcesses</i> Collection of server process configurations that describe which server processes to run on each instance in a fleet.<br/>        """
        pass

    def update_script(self, ScriptId: str, Name: str=None, Version: str=None, StorageLocation: S3Location=None, ZipFile: ZipBlob=None) -> UpdateScriptOutput:
        """Updates Realtime script metadata and content. To update script metadata, specify the script ID and provide updated name and/or version values.  To update script content, provide an updated zip file by pointing to either a local file or an Amazon S3 bucket location. You can use either method regardless of how the original script was uploaded. Use the <i>Version</i> parameter to track updates to the script. If the call is successful, the updated metadata is stored in the script record and a revised script is uploaded to the Amazon GameLift service. Once the script is updated and acquired by a fleet instance, the new version is used for all new game sessions.   <b>Learn more</b>   <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/realtime-intro.html">Amazon GameLift Realtime Servers</a>   <b>Related operations</b>  <ul> <li>  <a>CreateScript</a>  </li> <li>  <a>ListScripts</a>  </li> <li>  <a>DescribeScript</a>  </li> <li>  <a>UpdateScript</a>  </li> <li>  <a>DeleteScript</a>  </li> </ul>

        :param self:
        :param ZipFile:  Data object containing your Realtime scripts and dependencies as a zip file. The zip file can have one or multiple files. Maximum size of a zip file is 5 MB. When using the AWS CLI tool to create a script, this parameter is set to the zip file name. It must be prepended with the string "fileb://" to indicate that the file data is a binary object. For example: <code>--zip-file fileb://myRealtimeScript.zip</code>.<br/>StorageLocation:  Location of the Amazon S3 bucket where a zipped file containing your Realtime scripts is stored. The storage location must specify the Amazon S3 bucket name, the zip file name (the "key"), and a role ARN that allows Amazon GameLift to access the Amazon S3 storage location. The S3 bucket must be in the same region where you want to create a new script. By default, Amazon GameLift uploads the latest version of the zip file; if you have S3 object versioning turned on, you can use the <code>ObjectVersion</code> parameter to specify an earlier version. :<br/>Version:  Version that is associated with a build or script. Version strings do not need to be unique.<br/>Name:  Descriptive label that is associated with a script. Script names do not need to be unique.<br/>ScriptId:  Unique identifier for a Realtime script to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ObjectVersion</i> Version of the file, if object versioning is turned on for the bucket. Amazon GameLift uses this information when retrieving files from an S3 bucket that you own. Use this parameter to specify a specific version of the file; if not set, the latest version of the file is retrieved. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> Amazon Resource Name (<a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/s3-arn-format.html">ARN</a>) for an IAM role that allows Amazon GameLift to access the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> Name of the zip file containing the build files or script files. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> Amazon S3 bucket identifier. This is the name of the S3 bucket.<br/>
        :return: Script:  The newly created script record with a unique script ID. The new script's storage location reflects an Amazon S3 location: (1) If the script was uploaded from an S3 bucket under your account, the storage location reflects the information that was provided in the <i>CreateScript</i> request; (2) If the script file was uploaded from a local zip file, the storage location reflects an S3 location controls by the Amazon GameLift service.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageLocation</i> Location in Amazon Simple Storage Service (Amazon S3) where build or script files are stored for access by Amazon GameLift. This location is specified in <a>CreateBuild</a>, <a>CreateScript</a>, and <a>UpdateScript</a> requests. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StorageLocation</b>: Location in Amazon Simple Storage Service (Amazon S3) where build or script files are stored for access by Amazon GameLift. This location is specified in <a>CreateBuild</a>, <a>CreateScript</a>, and <a>UpdateScript</a> requests. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> Time stamp indicating when this data object was created. Format is a number expressed in Unix time as milliseconds (for example "1469498468.057").<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SizeOnDisk</i> File size of the uploaded Realtime script, expressed in bytes. When files are uploaded from an S3 location, this value remains at "0".<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> Version that is associated with a build or script. Version strings do not need to be unique.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Descriptive label that is associated with a script. Script names do not need to be unique.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ScriptId</i> Unique identifier for a Realtime script<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ObjectVersion</i> Version of the file, if object versioning is turned on for the bucket. Amazon GameLift uses this information when retrieving files from an S3 bucket that you own. Use this parameter to specify a specific version of the file; if not set, the latest version of the file is retrieved. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> Amazon Resource Name (<a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/s3-arn-format.html">ARN</a>) for an IAM role that allows Amazon GameLift to access the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> Name of the zip file containing the build files or script files. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> Amazon S3 bucket identifier. This is the name of the S3 bucket.<br/>        """
        pass

    def validate_matchmaking_rule_set(self, RuleSetBody: str) -> ValidateMatchmakingRuleSetOutput:
        """Validates the syntax of a matchmaking rule or rule set. This operation checks that the rule set is using syntactically correct JSON and that it conforms to allowed property expressions. To validate syntax, provide a rule set JSON string.  <b>Learn more</b>  <ul> <li>  <a href="https://docs.aws.amazon.com/gamelift/latest/developerguide/match-rulesets.html">Build a Rule Set</a>  </li> </ul>  <b>Related operations</b>  <ul> <li>  <a>CreateMatchmakingConfiguration</a>  </li> <li>  <a>DescribeMatchmakingConfigurations</a>  </li> <li>  <a>UpdateMatchmakingConfiguration</a>  </li> <li>  <a>DeleteMatchmakingConfiguration</a>  </li> <li>  <a>CreateMatchmakingRuleSet</a>  </li> <li>  <a>DescribeMatchmakingRuleSets</a>  </li> <li>  <a>ValidateMatchmakingRuleSet</a>  </li> <li>  <a>DeleteMatchmakingRuleSet</a>  </li> </ul>

        :param self:
        :param RuleSetBody:  Collection of matchmaking rules to validate, formatted as a JSON string.<br/>
        :return: Valid:  Response indicating whether the rule set is valid.<br/>        """
        pass



class Glacier(BaseClient):
    """ Amazon S3 Glacier (Glacier) is a storage solution for "cold data." Glacier is an extremely low-cost storage service that provides secure, durable, and easy-to-use storage for data backup and archival. With Glacier, customers can store their data cost effectively for months, years, or decades. Glacier also enables customers to offload the administrative burdens of operating and scaling storage to AWS, so they don't have to worry about capacity planning, hardware provisioning, data replication, hardware failure and recovery, or time-consuming hardware migrations. Glacier is a great storage choice when low storage cost is paramount and your data is rarely retrieved. If your application requires fast or frequent access to your data, consider using Amazon S3. For more information, see <a href="http://aws.amazon.com/s3/">Amazon Simple Storage Service (Amazon S3)</a>. You can store any kind of data in any format. There is no maximum limit on the total amount of data you can store in Glacier. If you are a first-time user of Glacier, we recommend that you begin by reading the following sections in the <i>Amazon S3 Glacier Developer Guide</i>: <ul> <li>  <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/introduction.html">What is Amazon S3 Glacier</a> - This section of the Developer Guide describes the underlying data model, the operations it supports, and the AWS SDKs that you can use to interact with the service. </li> <li>  <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/amazon-glacier-getting-started.html">Getting Started with Amazon S3 Glacier</a> - The Getting Started section walks you through the process of creating a vault, uploading archives, creating jobs to download archives, retrieving the job output, and deleting archives. </li> </ul>"""

    class VaultExistsWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_vault` for valid parameters"""
            pass
    class VaultNotExistsWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_vault` for valid parameters"""
            pass


    class ListJobsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_jobs` for valid parameters"""
            pass
    class ListMultipartUploadsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_multipart_uploads` for valid parameters"""
            pass
    class ListPartsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_parts` for valid parameters"""
            pass
    class ListVaultsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_vaults` for valid parameters"""
            pass


    class AbortMultipartUploadInput(Mapping):
        pass
    
    class AbortVaultLockInput(Mapping):
        pass
    
    class AddTagsToVaultInput(Mapping):
        pass
    
    class TagMap(object):
        pass
    
    class CompleteMultipartUploadInput(Mapping):
        pass
    
    class ArchiveCreationOutput(Mapping):
        pass
    
    class CompleteVaultLockInput(Mapping):
        pass
    
    class CreateVaultInput(Mapping):
        pass
    
    class CreateVaultOutput(Mapping):
        pass
    
    class DeleteArchiveInput(Mapping):
        pass
    
    class DeleteVaultInput(Mapping):
        pass
    
    class DeleteVaultAccessPolicyInput(Mapping):
        pass
    
    class DeleteVaultNotificationsInput(Mapping):
        pass
    
    class DescribeJobInput(Mapping):
        pass
    
    class GlacierJobDescription(Mapping):
        pass
    
    class DescribeVaultInput(Mapping):
        pass
    
    class DescribeVaultOutput(Mapping):
        pass
    
    class GetDataRetrievalPolicyInput(Mapping):
        pass
    
    class GetDataRetrievalPolicyOutput(Mapping):
        pass
    
    class GetJobOutputInput(Mapping):
        pass
    
    class GetJobOutputOutput(Mapping):
        pass
    
    class GetVaultAccessPolicyInput(Mapping):
        pass
    
    class GetVaultAccessPolicyOutput(Mapping):
        pass
    
    class GetVaultLockInput(Mapping):
        pass
    
    class GetVaultLockOutput(Mapping):
        pass
    
    class GetVaultNotificationsInput(Mapping):
        pass
    
    class GetVaultNotificationsOutput(Mapping):
        pass
    
    class InitiateJobInput(Mapping):
        pass
    
    class InitiateJobOutput(Mapping):
        pass
    
    class JobParameters(Mapping):
        pass
    
    class InitiateMultipartUploadInput(Mapping):
        pass
    
    class InitiateMultipartUploadOutput(Mapping):
        pass
    
    class InitiateVaultLockInput(Mapping):
        pass
    
    class InitiateVaultLockOutput(Mapping):
        pass
    
    class VaultLockPolicy(Mapping):
        pass
    
    class ListJobsInput(Mapping):
        pass
    
    class ListJobsOutput(Mapping):
        pass
    
    class ListMultipartUploadsInput(Mapping):
        pass
    
    class ListMultipartUploadsOutput(Mapping):
        pass
    
    class ListPartsInput(Mapping):
        pass
    
    class ListPartsOutput(Mapping):
        pass
    
    class ListProvisionedCapacityInput(Mapping):
        pass
    
    class ListProvisionedCapacityOutput(Mapping):
        pass
    
    class ListTagsForVaultInput(Mapping):
        pass
    
    class ListTagsForVaultOutput(Mapping):
        pass
    
    class ListVaultsInput(Mapping):
        pass
    
    class ListVaultsOutput(Mapping):
        pass
    
    class PurchaseProvisionedCapacityInput(Mapping):
        pass
    
    class PurchaseProvisionedCapacityOutput(Mapping):
        pass
    
    class RemoveTagsFromVaultInput(Mapping):
        pass
    
    class string(object):
        pass
    
    class SetDataRetrievalPolicyInput(Mapping):
        pass
    
    class DataRetrievalPolicy(Mapping):
        pass
    
    class SetVaultAccessPolicyInput(Mapping):
        pass
    
    class VaultAccessPolicy(Mapping):
        pass
    
    class SetVaultNotificationsInput(Mapping):
        pass
    
    class VaultNotificationConfig(Mapping):
        pass
    
    class UploadArchiveInput(Mapping):
        pass
    
    class Stream(object):
        pass
    
    class UploadMultipartPartInput(Mapping):
        pass
    
    class UploadMultipartPartOutput(Mapping):
        pass
    
    def abort_multipart_upload(self, uploadId: str, vaultName: str, accountId: str):
        """This operation aborts a multipart upload identified by the upload ID. After the Abort Multipart Upload request succeeds, you cannot upload any more parts to the multipart upload or complete the multipart upload. Aborting a completed upload fails. However, aborting an already-aborted upload will succeed, for a short time. For more information about uploading a part and completing a multipart upload, see <a>UploadMultipartPart</a> and <a>CompleteMultipartUpload</a>. This operation is idempotent. An AWS account has full permission to perform all operations (actions). However, AWS Identity and Access Management (IAM) users don't have any permissions by default. You must grant them explicit permission to perform specific actions. For more information, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/using-iam-with-amazon-glacier.html">Access Control Using AWS Identity and Access Management (IAM)</a>.  For conceptual information and underlying REST API, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/working-with-archives.html">Working with Archives in Amazon S3 Glacier</a> and <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/api-multipart-abort-upload.html">Abort Multipart Upload</a> in the <i>Amazon Glacier Developer Guide</i>. 

        :param self:
        :param uploadId:  The upload ID of the multipart upload to delete.<br/>vaultName:  The name of the vault.<br/>accountId:  The <code>AccountId</code> value is the AWS account ID of the account that owns the vault. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon S3 Glacier uses the AWS account ID associated with the credentials used to sign the request. If you use an account ID, do not include any hyphens ('-') in the ID.<br/>
        :return:         """
        pass

    def abort_vault_lock(self, vaultName: str, accountId: str):
        """This operation aborts the vault locking process if the vault lock is not in the <code>Locked</code> state. If the vault lock is in the <code>Locked</code> state when this operation is requested, the operation returns an <code>AccessDeniedException</code> error. Aborting the vault locking process removes the vault lock policy from the specified vault.  A vault lock is put into the <code>InProgress</code> state by calling <a>InitiateVaultLock</a>. A vault lock is put into the <code>Locked</code> state by calling <a>CompleteVaultLock</a>. You can get the state of a vault lock by calling <a>GetVaultLock</a>. For more information about the vault locking process, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/vault-lock.html">Amazon Glacier Vault Lock</a>. For more information about vault lock policies, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/vault-lock-policy.html">Amazon Glacier Access Control with Vault Lock Policies</a>.  This operation is idempotent. You can successfully invoke this operation multiple times, if the vault lock is in the <code>InProgress</code> state or if there is no policy associated with the vault.

        :param self:
        :param vaultName:  The name of the vault.<br/>accountId:  The <code>AccountId</code> value is the AWS account ID. This value must match the AWS account ID associated with the credentials used to sign the request. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon Glacier uses the AWS account ID associated with the credentials used to sign the request. If you specify your account ID, do not include any hyphens ('-') in the ID.<br/>
        :return:         """
        pass

    def add_tags_to_vault(self, vaultName: str, accountId: str, Tags: TagMap=None):
        """This operation adds the specified tags to a vault. Each tag is composed of a key and a value. Each vault can have up to 10 tags. If your request would cause the tag limit for the vault to be exceeded, the operation throws the <code>LimitExceededException</code> error. If a tag already exists on the vault under a specified key, the existing key value will be overwritten. For more information about tags, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/tagging.html">Tagging Amazon S3 Glacier Resources</a>. 

        :param self:
        :param Tags:  The tags to add to the vault. Each tag is composed of a key and a value. The value can be an empty string.<br/>vaultName:  The name of the vault.<br/>accountId:  The <code>AccountId</code> value is the AWS account ID of the account that owns the vault. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon S3 Glacier uses the AWS account ID associated with the credentials used to sign the request. If you use an account ID, do not include any hyphens ('-') in the ID.<br/>
        :return:         """
        pass

    def complete_multipart_upload(self, uploadId: str, vaultName: str, accountId: str, archiveSize: str=None, checksum: str=None) -> ArchiveCreationOutput:
        """You call this operation to inform Amazon S3 Glacier (Glacier) that all the archive parts have been uploaded and that Glacier can now assemble the archive from the uploaded parts. After assembling and saving the archive to the vault, Glacier returns the URI path of the newly created archive resource. Using the URI path, you can then access the archive. After you upload an archive, you should save the archive ID returned to retrieve the archive at a later point. You can also get the vault inventory to obtain a list of archive IDs in a vault. For more information, see <a>InitiateJob</a>. In the request, you must include the computed SHA256 tree hash of the entire archive you have uploaded. For information about computing a SHA256 tree hash, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/checksum-calculations.html">Computing Checksums</a>. On the server side, Glacier also constructs the SHA256 tree hash of the assembled archive. If the values match, Glacier saves the archive to the vault; otherwise, it returns an error, and the operation fails. The <a>ListParts</a> operation returns a list of parts uploaded for a specific multipart upload. It includes checksum information for each uploaded part that can be used to debug a bad checksum issue. Additionally, Glacier also checks for any missing content ranges when assembling the archive, if missing content ranges are found, Glacier returns an error and the operation fails. Complete Multipart Upload is an idempotent operation. After your first successful complete multipart upload, if you call the operation again within a short period, the operation will succeed and return the same archive ID. This is useful in the event you experience a network issue that causes an aborted connection or receive a 500 server error, in which case you can repeat your Complete Multipart Upload request and get the same archive ID without creating duplicate archives. Note, however, that after the multipart upload completes, you cannot call the List Parts operation and the multipart upload will not appear in List Multipart Uploads response, even if idempotent complete is possible. An AWS account has full permission to perform all operations (actions). However, AWS Identity and Access Management (IAM) users don't have any permissions by default. You must grant them explicit permission to perform specific actions. For more information, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/using-iam-with-amazon-glacier.html">Access Control Using AWS Identity and Access Management (IAM)</a>.  For conceptual information and underlying REST API, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/uploading-archive-mpu.html">Uploading Large Archives in Parts (Multipart Upload)</a> and <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/api-multipart-complete-upload.html">Complete Multipart Upload</a> in the <i>Amazon Glacier Developer Guide</i>. 

        :param self:
        :param checksum:  The SHA256 tree hash of the entire archive. It is the tree hash of SHA256 tree hash of the individual parts. If the value you specify in the request does not match the SHA256 tree hash of the final assembled archive as computed by Amazon S3 Glacier (Glacier), Glacier returns an error and the request fails.<br/>archiveSize:  The total size, in bytes, of the entire archive. This value should be the sum of all the sizes of the individual parts that you uploaded.<br/>uploadId:  The upload ID of the multipart upload.<br/>vaultName:  The name of the vault.<br/>accountId:  The <code>AccountId</code> value is the AWS account ID of the account that owns the vault. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon S3 Glacier uses the AWS account ID associated with the credentials used to sign the request. If you use an account ID, do not include any hyphens ('-') in the ID.<br/>
        :return: archiveId:  The ID of the archive. This value is also included as part of the location.<br/>checksum:  The checksum of the archive computed by Amazon S3 Glacier.<br/>location:  The relative URI path of the newly added archive resource.<br/>        """
        pass

    def complete_vault_lock(self, lockId: str, vaultName: str, accountId: str):
        """This operation completes the vault locking process by transitioning the vault lock from the <code>InProgress</code> state to the <code>Locked</code> state, which causes the vault lock policy to become unchangeable. A vault lock is put into the <code>InProgress</code> state by calling <a>InitiateVaultLock</a>. You can obtain the state of the vault lock by calling <a>GetVaultLock</a>. For more information about the vault locking process, <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/vault-lock.html">Amazon Glacier Vault Lock</a>.  This operation is idempotent. This request is always successful if the vault lock is in the <code>Locked</code> state and the provided lock ID matches the lock ID originally used to lock the vault. If an invalid lock ID is passed in the request when the vault lock is in the <code>Locked</code> state, the operation returns an <code>AccessDeniedException</code> error. If an invalid lock ID is passed in the request when the vault lock is in the <code>InProgress</code> state, the operation throws an <code>InvalidParameter</code> error.

        :param self:
        :param lockId:  The <code>lockId</code> value is the lock ID obtained from a <a>InitiateVaultLock</a> request.<br/>vaultName:  The name of the vault.<br/>accountId:  The <code>AccountId</code> value is the AWS account ID. This value must match the AWS account ID associated with the credentials used to sign the request. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon Glacier uses the AWS account ID associated with the credentials used to sign the request. If you specify your account ID, do not include any hyphens ('-') in the ID.<br/>
        :return:         """
        pass

    def create_vault(self, vaultName: str, accountId: str) -> CreateVaultOutput:
        """This operation creates a new vault with the specified name. The name of the vault must be unique within a region for an AWS account. You can create up to 1,000 vaults per account. If you need to create more vaults, contact Amazon S3 Glacier. You must use the following guidelines when naming a vault. <ul> <li> Names can be between 1 and 255 characters long. </li> <li> Allowed characters are a-z, A-Z, 0-9, '_' (underscore), '-' (hyphen), and '.' (period). </li> </ul> This operation is idempotent. An AWS account has full permission to perform all operations (actions). However, AWS Identity and Access Management (IAM) users don't have any permissions by default. You must grant them explicit permission to perform specific actions. For more information, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/using-iam-with-amazon-glacier.html">Access Control Using AWS Identity and Access Management (IAM)</a>.  For conceptual information and underlying REST API, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/creating-vaults.html">Creating a Vault in Amazon Glacier</a> and <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/api-vault-put.html">Create Vault </a> in the <i>Amazon Glacier Developer Guide</i>. 

        :param self:
        :param vaultName:  The name of the vault.<br/>accountId:  The <code>AccountId</code> value is the AWS account ID. This value must match the AWS account ID associated with the credentials used to sign the request. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon S3 Glacier uses the AWS account ID associated with the credentials used to sign the request. If you specify your account ID, do not include any hyphens ('-') in the ID.<br/>
        :return: location:  The URI of the vault that was created.<br/>        """
        pass

    def delete_archive(self, archiveId: str, vaultName: str, accountId: str):
        """This operation deletes an archive from a vault. Subsequent requests to initiate a retrieval of this archive will fail. Archive retrievals that are in progress for this archive ID may or may not succeed according to the following scenarios: <ul> <li> If the archive retrieval job is actively preparing the data for download when Amazon S3 Glacier receives the delete archive request, the archival retrieval operation might fail. </li> <li> If the archive retrieval job has successfully prepared the archive for download when Amazon S3 Glacier receives the delete archive request, you will be able to download the output. </li> </ul> This operation is idempotent. Attempting to delete an already-deleted archive does not result in an error. An AWS account has full permission to perform all operations (actions). However, AWS Identity and Access Management (IAM) users don't have any permissions by default. You must grant them explicit permission to perform specific actions. For more information, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/using-iam-with-amazon-glacier.html">Access Control Using AWS Identity and Access Management (IAM)</a>.  For conceptual information and underlying REST API, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/deleting-an-archive.html">Deleting an Archive in Amazon Glacier</a> and <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/api-archive-delete.html">Delete Archive</a> in the <i>Amazon Glacier Developer Guide</i>. 

        :param self:
        :param archiveId:  The ID of the archive to delete.<br/>vaultName:  The name of the vault.<br/>accountId:  The <code>AccountId</code> value is the AWS account ID of the account that owns the vault. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon S3 Glacier uses the AWS account ID associated with the credentials used to sign the request. If you use an account ID, do not include any hyphens ('-') in the ID.<br/>
        :return:         """
        pass

    def delete_vault(self, vaultName: str, accountId: str):
        """This operation deletes a vault. Amazon S3 Glacier will delete a vault only if there are no archives in the vault as of the last inventory and there have been no writes to the vault since the last inventory. If either of these conditions is not satisfied, the vault deletion fails (that is, the vault is not removed) and Amazon S3 Glacier returns an error. You can use <a>DescribeVault</a> to return the number of archives in a vault, and you can use <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/api-initiate-job-post.html">Initiate a Job (POST jobs)</a> to initiate a new inventory retrieval for a vault. The inventory contains the archive IDs you use to delete archives using <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/api-archive-delete.html">Delete Archive (DELETE archive)</a>. This operation is idempotent. An AWS account has full permission to perform all operations (actions). However, AWS Identity and Access Management (IAM) users don't have any permissions by default. You must grant them explicit permission to perform specific actions. For more information, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/using-iam-with-amazon-glacier.html">Access Control Using AWS Identity and Access Management (IAM)</a>.  For conceptual information and underlying REST API, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/deleting-vaults.html">Deleting a Vault in Amazon Glacier</a> and <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/api-vault-delete.html">Delete Vault </a> in the <i>Amazon S3 Glacier Developer Guide</i>. 

        :param self:
        :param vaultName:  The name of the vault.<br/>accountId:  The <code>AccountId</code> value is the AWS account ID of the account that owns the vault. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon S3 Glacier uses the AWS account ID associated with the credentials used to sign the request. If you use an account ID, do not include any hyphens ('-') in the ID.<br/>
        :return:         """
        pass

    def delete_vault_access_policy(self, vaultName: str, accountId: str):
        """This operation deletes the access policy associated with the specified vault. The operation is eventually consistent; that is, it might take some time for Amazon S3 Glacier to completely remove the access policy, and you might still see the effect of the policy for a short time after you send the delete request. This operation is idempotent. You can invoke delete multiple times, even if there is no policy associated with the vault. For more information about vault access policies, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/vault-access-policy.html">Amazon Glacier Access Control with Vault Access Policies</a>. 

        :param self:
        :param vaultName:  The name of the vault.<br/>accountId:  The <code>AccountId</code> value is the AWS account ID of the account that owns the vault. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon S3 Glacier uses the AWS account ID associated with the credentials used to sign the request. If you use an account ID, do not include any hyphens ('-') in the ID. <br/>
        :return:         """
        pass

    def delete_vault_notifications(self, vaultName: str, accountId: str):
        """This operation deletes the notification configuration set for a vault. The operation is eventually consistent; that is, it might take some time for Amazon S3 Glacier to completely disable the notifications and you might still receive some notifications for a short time after you send the delete request. An AWS account has full permission to perform all operations (actions). However, AWS Identity and Access Management (IAM) users don't have any permissions by default. You must grant them explicit permission to perform specific actions. For more information, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/using-iam-with-amazon-glacier.html">Access Control Using AWS Identity and Access Management (IAM)</a>.  For conceptual information and underlying REST API, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/configuring-notifications.html">Configuring Vault Notifications in Amazon S3 Glacier</a> and <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/api-vault-notifications-delete.html">Delete Vault Notification Configuration </a> in the Amazon S3 Glacier Developer Guide. 

        :param self:
        :param vaultName:  The name of the vault.<br/>accountId:  The <code>AccountId</code> value is the AWS account ID of the account that owns the vault. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon S3 Glacier uses the AWS account ID associated with the credentials used to sign the request. If you use an account ID, do not include any hyphens ('-') in the ID. <br/>
        :return:         """
        pass

    def describe_job(self, jobId: str, vaultName: str, accountId: str) -> GlacierJobDescription:
        """This operation returns information about a job you previously initiated, including the job initiation date, the user who initiated the job, the job status code/message and the Amazon SNS topic to notify after Amazon S3 Glacier (Glacier) completes the job. For more information about initiating a job, see <a>InitiateJob</a>.  <note> This operation enables you to check the status of your job. However, it is strongly recommended that you set up an Amazon SNS topic and specify it in your initiate job request so that Glacier can notify the topic after it completes the job. </note> A job ID will not expire for at least 24 hours after Glacier completes the job. An AWS account has full permission to perform all operations (actions). However, AWS Identity and Access Management (IAM) users don't have any permissions by default. You must grant them explicit permission to perform specific actions. For more information, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/using-iam-with-amazon-glacier.html">Access Control Using AWS Identity and Access Management (IAM)</a>.  For more information about using this operation, see the documentation for the underlying REST API <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/api-describe-job-get.html">Describe Job</a> in the <i>Amazon Glacier Developer Guide</i>. 

        :param self:
        :param jobId:  The ID of the job to describe.<br/>vaultName:  The name of the vault.<br/>accountId:  The <code>AccountId</code> value is the AWS account ID of the account that owns the vault. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon S3 Glacier uses the AWS account ID associated with the credentials used to sign the request. If you use an account ID, do not include any hyphens ('-') in the ID. <br/>
        :return: OutputLocation:  Contains the location where the data from the select job is stored.:<br/>SelectParameters:  Contains the parameters used for a select.:<br/>JobOutputPath:  Contains the job output location.<br/>InventoryRetrievalParameters:  Parameters used for range inventory retrieval.:<br/>Tier:  The tier to use for a select or an archive retrieval. Valid values are <code>Expedited</code>, <code>Standard</code>, or <code>Bulk</code>. <code>Standard</code> is the default.<br/>RetrievalByteRange:  The retrieved byte range for archive retrieval jobs in the form <i>StartByteValue</i>-<i>EndByteValue</i>. If no range was specified in the archive retrieval, then the whole archive is retrieved. In this case, <i>StartByteValue</i> equals 0 and <i>EndByteValue</i> equals the size of the archive minus 1. For inventory retrieval or select jobs, this field is null. <br/>ArchiveSHA256TreeHash:  The SHA256 tree hash of the entire archive for an archive retrieval. For inventory retrieval or select jobs, this field is null.<br/>SHA256TreeHash:  For an archive retrieval job, this value is the checksum of the archive. Otherwise, this value is null. The SHA256 tree hash value for the requested range of an archive. If the <b>InitiateJob</b> request for an archive specified a tree-hash aligned range, then this field returns a value. If the whole archive is retrieved, this value is the same as the ArchiveSHA256TreeHash value. This field is null for the following: <ul> <li> Archive retrieval jobs that specify a range that is not tree-hash aligned </li> </ul> <ul> <li> Archival jobs that specify a range that is equal to the whole archive, when the job status is <code>InProgress</code>  </li> </ul> <ul> <li> Inventory jobs </li> <li> Select jobs </li> </ul><br/>CompletionDate:  The UTC time that the job request completed. While the job is in progress, the value is null.<br/>SNSTopic:  An Amazon SNS topic that receives notification.<br/>InventorySizeInBytes:  For an inventory retrieval job, this value is the size in bytes of the inventory requested for download. For an archive retrieval or select job, this value is null.<br/>ArchiveSizeInBytes:  For an archive retrieval job, this value is the size in bytes of the archive being requested for download. For an inventory retrieval or select job, this value is null.<br/>StatusMessage:  A friendly message that describes the job status.<br/>StatusCode:  The status code can be <code>InProgress</code>, <code>Succeeded</code>, or <code>Failed</code>, and indicates the status of the job.<br/>Completed:  The job status. When a job is completed, you get the job's output using Get Job Output (GET output).<br/>CreationDate:  The UTC date when the job was created. This value is a string representation of ISO 8601 date format, for example <code>"2012-03-20T17:03:43.221Z"</code>.<br/>VaultARN:  The Amazon Resource Name (ARN) of the vault from which an archive retrieval was requested.<br/>ArchiveId:  The archive ID requested for a select job or archive retrieval. Otherwise, this field is null.<br/>Action:  The job type. This value is either <code>ArchiveRetrieval</code>, <code>InventoryRetrieval</code>, or <code>Select</code>. <br/>JobDescription:  The job description provided when initiating the job.<br/>JobId:  An opaque string that identifies an Amazon S3 Glacier job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Marker</i> An opaque string that represents where to continue pagination of the vault inventory retrieval results. You use the marker in a new <b>InitiateJob</b> request to obtain additional inventory items. If there are no more inventory items, this value is <code>null</code>. For more information, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/api-initiate-job-post.html#api-initiate-job-post-vault-inventory-list-filtering"> Range Inventory Retrieval</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Limit</i> The maximum number of inventory items returned per vault inventory retrieval request. This limit is set when initiating the job with the a <b>InitiateJob</b> request. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndDate</i> The end of the date range in UTC for vault inventory retrieval that includes archives created before this date. This value should be a string in the ISO 8601 date format, for example <code>2013-03-20T17:03:43Z</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartDate</i> The start of the date range in Universal Coordinated Time (UTC) for vault inventory retrieval that includes archives created on or after this date. This value should be a string in the ISO 8601 date format, for example <code>2013-03-20T17:03:43Z</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> The output format for the vault inventory list, which is set by the <b>InitiateJob</b> request when initiating a job to retrieve a vault inventory. Valid values are <code>CSV</code> and <code>JSON</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputSerialization</i> Describes how the results of the select job are serialized.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>OutputSerialization</b>: Describes how the results of the select job are serialized.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Expression</i> The expression that is used to select the object.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpressionType</i> The type of the provided expression, for example <code>SQL</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputSerialization</i> Describes the serialization format of the object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputSerialization</b>: Describes the serialization format of the object.<br/>Describes the serialization of a CSV-encoded object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>csv</b>: Describes the serialization of a CSV-encoded object.Describes the serialization of CSV-encoded query results.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>csv</b>: Describes the serialization of CSV-encoded query results.&nbsp;&nbsp;&nbsp;&nbsp; <i>S3</i> Describes an S3 location that will receive the results of the job request.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3</b>: Describes an S3 location that will receive the results of the job request.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> The prefix that is prepended to the results for this request.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketName</i> The name of the Amazon S3 bucket where the job results are stored.<br/>Contains information about the encryption used to store the job results in Amazon S3.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: Contains information about the encryption used to store the job results in Amazon S3.        """
        pass

    def describe_vault(self, vaultName: str, accountId: str) -> DescribeVaultOutput:
        """This operation returns information about a vault, including the vault's Amazon Resource Name (ARN), the date the vault was created, the number of archives it contains, and the total size of all the archives in the vault. The number of archives and their total size are as of the last inventory generation. This means that if you add or remove an archive from a vault, and then immediately use Describe Vault, the change in contents will not be immediately reflected. If you want to retrieve the latest inventory of the vault, use <a>InitiateJob</a>. Amazon S3 Glacier generates vault inventories approximately daily. For more information, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/vault-inventory.html">Downloading a Vault Inventory in Amazon S3 Glacier</a>.  An AWS account has full permission to perform all operations (actions). However, AWS Identity and Access Management (IAM) users don't have any permissions by default. You must grant them explicit permission to perform specific actions. For more information, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/using-iam-with-amazon-glacier.html">Access Control Using AWS Identity and Access Management (IAM)</a>. For conceptual information and underlying REST API, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/retrieving-vault-info.html">Retrieving Vault Metadata in Amazon S3 Glacier</a> and <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/api-vault-get.html">Describe Vault </a> in the <i>Amazon Glacier Developer Guide</i>. 

        :param self:
        :param vaultName:  The name of the vault.<br/>accountId:  The <code>AccountId</code> value is the AWS account ID of the account that owns the vault. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon S3 Glacier uses the AWS account ID associated with the credentials used to sign the request. If you use an account ID, do not include any hyphens ('-') in the ID. <br/>
        :return: SizeInBytes:  Total size, in bytes, of the archives in the vault as of the last inventory date. This field will return null if an inventory has not yet run on the vault, for example if you just created the vault.<br/>NumberOfArchives:  The number of archives in the vault as of the last inventory date. This field will return <code>null</code> if an inventory has not yet run on the vault, for example if you just created the vault.<br/>LastInventoryDate:  The Universal Coordinated Time (UTC) date when Amazon S3 Glacier completed the last vault inventory. This value should be a string in the ISO 8601 date format, for example <code>2012-03-20T17:03:43.221Z</code>.<br/>CreationDate:  The Universal Coordinated Time (UTC) date when the vault was created. This value should be a string in the ISO 8601 date format, for example <code>2012-03-20T17:03:43.221Z</code>.<br/>VaultName:  The name of the vault.<br/>VaultARN:  The Amazon Resource Name (ARN) of the vault.<br/>        """
        pass

    def get_data_retrieval_policy(self, accountId: str) -> GetDataRetrievalPolicyOutput:
        """This operation returns the current data retrieval policy for the account and region specified in the GET request. For more information about data retrieval policies, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/data-retrieval-policy.html">Amazon Glacier Data Retrieval Policies</a>.

        :param self:
        :param accountId:  The <code>AccountId</code> value is the AWS account ID. This value must match the AWS account ID associated with the credentials used to sign the request. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon Glacier uses the AWS account ID associated with the credentials used to sign the request. If you specify your account ID, do not include any hyphens ('-') in the ID. <br/>
        :return: Policy:  Contains the returned data retrieval policy in JSON format.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Rules</i> The policy rule. Although this is a list type, currently there must be only one rule, which contains a Strategy field and optionally a BytesPerHour field.<br/>        """
        pass

    def get_job_output(self, jobId: str, vaultName: str, accountId: str, range: str=None) -> GetJobOutputOutput:
        """This operation downloads the output of the job you initiated using <a>InitiateJob</a>. Depending on the job type you specified when you initiated the job, the output will be either the content of an archive or a vault inventory. You can download all the job output or download a portion of the output by specifying a byte range. In the case of an archive retrieval job, depending on the byte range you specify, Amazon S3 Glacier (Glacier) returns the checksum for the portion of the data. You can compute the checksum on the client and verify that the values match to ensure the portion you downloaded is the correct data. A job ID will not expire for at least 24 hours after Glacier completes the job. That a byte range. For both archive and inventory retrieval jobs, you should verify the downloaded size against the size returned in the headers from the <b>Get Job Output</b> response. For archive retrieval jobs, you should also verify that the size is what you expected. If you download a portion of the output, the expected size is based on the range of bytes you specified. For example, if you specify a range of <code>bytes=0-1048575</code>, you should verify your download size is 1,048,576 bytes. If you download an entire archive, the expected size is the size of the archive when you uploaded it to Amazon S3 Glacier The expected size is also returned in the headers from the <b>Get Job Output</b> response. In the case of an archive retrieval job, depending on the byte range you specify, Glacier returns the checksum for the portion of the data. To ensure the portion you downloaded is the correct data, compute the checksum on the client, verify that the values match, and verify that the size is what you expected. A job ID does not expire for at least 24 hours after Glacier completes the job. That is, you can download the job output within the 24 hours period after Amazon Glacier completes the job. An AWS account has full permission to perform all operations (actions). However, AWS Identity and Access Management (IAM) users don't have any permissions by default. You must grant them explicit permission to perform specific actions. For more information, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/using-iam-with-amazon-glacier.html">Access Control Using AWS Identity and Access Management (IAM)</a>. For conceptual information and the underlying REST API, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/vault-inventory.html">Downloading a Vault Inventory</a>, <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/downloading-an-archive.html">Downloading an Archive</a>, and <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/api-job-output-get.html">Get Job Output </a> 

        :param self:
        :param range:  The range of bytes to retrieve from the output. For example, if you want to download the first 1,048,576 bytes, specify the range as <code>bytes=0-1048575</code>. By default, this operation downloads the entire output. If the job output is large, then you can use a range to retrieve a portion of the output. This allows you to download the entire output in smaller chunks of bytes. For example, suppose you have 1 GB of job output you want to download and you decide to download 128 MB chunks of data at a time, which is a total of eight Get Job Output requests. You use the following process to download the job output: <ol> <li> Download a 128 MB chunk of output by specifying the appropriate byte range. Verify that all 128 MB of data was received. </li> <li> Along with the data, the response includes a SHA256 tree hash of the payload. You compute the checksum of the payload on the client and compare it with the checksum you received in the response to ensure you received all the expected data. </li> <li> Repeat steps 1 and 2 for all the eight 128 MB chunks of output data, each time specifying the appropriate byte range. </li> <li> After downloading all the parts of the job output, you have a list of eight checksum values. Compute the tree hash of these values to find the checksum of the entire output. Using the <a>DescribeJob</a> API, obtain job information of the job that provided you the output. The response includes the checksum of the entire archive stored in Amazon S3 Glacier. You compare this value with the checksum you computed to ensure you have downloaded the entire archive content with no errors. <p/> </li> </ol><br/>jobId:  The job ID whose data is downloaded.<br/>vaultName:  The name of the vault.<br/>accountId:  The <code>AccountId</code> value is the AWS account ID of the account that owns the vault. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon S3 Glacier uses the AWS account ID associated with the credentials used to sign the request. If you use an account ID, do not include any hyphens ('-') in the ID.<br/>
        :return: archiveDescription:  The description of an archive.<br/>contentType:  The Content-Type depends on whether the job output is an archive or a vault inventory. For archive data, the Content-Type is application/octet-stream. For vault inventory, if you requested CSV format when you initiated the job, the Content-Type is text/csv. Otherwise, by default, vault inventory is returned as JSON, and the Content-Type is application/json.<br/>acceptRanges:  Indicates the range units accepted. For more information, see <a href="http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html">RFC2616</a>. <br/>contentRange:  The range of bytes returned by Amazon S3 Glacier. If only partial output is downloaded, the response provides the range of bytes Amazon S3 Glacier returned. For example, bytes 0-1048575/8388608 returns the first 1 MB from 8 MB.<br/>status:  The HTTP response code for a job output request. The value depends on whether a range was specified in the request.<br/>checksum:  The checksum of the data in the response. This header is returned only when retrieving the output for an archive retrieval job. Furthermore, this header appears only under the following conditions: <ul> <li> You get the entire range of the archive. </li> <li> You request a range to return of the archive that starts and ends on a multiple of 1 MB. For example, if you have an 3.1 MB archive and you specify a range to return that starts at 1 MB and ends at 2 MB, then the x-amz-sha256-tree-hash is returned as a response header. </li> <li> You request a range of the archive to return that starts on a multiple of 1 MB and goes to the end of the archive. For example, if you have a 3.1 MB archive and you specify a range that starts at 2 MB and ends at 3.1 MB (the end of the archive), then the x-amz-sha256-tree-hash is returned as a response header. </li> </ul><br/>body:  The job data, either archive data or inventory data.<br/>        """
        pass

    def get_vault_access_policy(self, vaultName: str, accountId: str) -> GetVaultAccessPolicyOutput:
        """This operation retrieves the <code>access-policy</code> subresource set on the vault; for more information on setting this subresource, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/api-SetVaultAccessPolicy.html">Set Vault Access Policy (PUT access-policy)</a>. If there is no access policy set on the vault, the operation returns a <code>404 Not found</code> error. For more information about vault access policies, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/vault-access-policy.html">Amazon Glacier Access Control with Vault Access Policies</a>.

        :param self:
        :param vaultName:  The name of the vault.<br/>accountId:  The <code>AccountId</code> value is the AWS account ID of the account that owns the vault. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon S3 Glacier uses the AWS account ID associated with the credentials used to sign the request. If you use an account ID, do not include any hyphens ('-') in the ID.<br/>
        :return: policy:  Contains the returned vault access policy as a JSON string.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Policy</i> The vault access policy.<br/>        """
        pass

    def get_vault_lock(self, vaultName: str, accountId: str) -> GetVaultLockOutput:
        """This operation retrieves the following attributes from the <code>lock-policy</code> subresource set on the specified vault:  <ul> <li> The vault lock policy set on the vault. </li> <li> The state of the vault lock, which is either <code>InProgess</code> or <code>Locked</code>. </li> <li> When the lock ID expires. The lock ID is used to complete the vault locking process. </li> <li> When the vault lock was initiated and put into the <code>InProgress</code> state. </li> </ul> A vault lock is put into the <code>InProgress</code> state by calling <a>InitiateVaultLock</a>. A vault lock is put into the <code>Locked</code> state by calling <a>CompleteVaultLock</a>. You can abort the vault locking process by calling <a>AbortVaultLock</a>. For more information about the vault locking process, <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/vault-lock.html">Amazon Glacier Vault Lock</a>.  If there is no vault lock policy set on the vault, the operation returns a <code>404 Not found</code> error. For more information about vault lock policies, <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/vault-lock-policy.html">Amazon Glacier Access Control with Vault Lock Policies</a>. 

        :param self:
        :param vaultName:  The name of the vault.<br/>accountId:  The <code>AccountId</code> value is the AWS account ID of the account that owns the vault. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon S3 Glacier uses the AWS account ID associated with the credentials used to sign the request. If you use an account ID, do not include any hyphens ('-') in the ID.<br/>
        :return: CreationDate:  The UTC date and time at which the vault lock was put into the <code>InProgress</code> state.<br/>ExpirationDate:  The UTC date and time at which the lock ID expires. This value can be <code>null</code> if the vault lock is in a <code>Locked</code> state.<br/>State:  The state of the vault lock. <code>InProgress</code> or <code>Locked</code>.<br/>Policy:  The vault lock policy as a JSON string, which uses "\" as an escape character.<br/>        """
        pass

    def get_vault_notifications(self, vaultName: str, accountId: str) -> GetVaultNotificationsOutput:
        """This operation retrieves the <code>notification-configuration</code> subresource of the specified vault. For information about setting a notification configuration on a vault, see <a>SetVaultNotifications</a>. If a notification configuration for a vault is not set, the operation returns a <code>404 Not Found</code> error. For more information about vault notifications, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/configuring-notifications.html">Configuring Vault Notifications in Amazon S3 Glacier</a>.  An AWS account has full permission to perform all operations (actions). However, AWS Identity and Access Management (IAM) users don't have any permissions by default. You must grant them explicit permission to perform specific actions. For more information, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/using-iam-with-amazon-glacier.html">Access Control Using AWS Identity and Access Management (IAM)</a>. For conceptual information and underlying REST API, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/configuring-notifications.html">Configuring Vault Notifications in Amazon S3 Glacier</a> and <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/api-vault-notifications-get.html">Get Vault Notification Configuration </a> in the <i>Amazon Glacier Developer Guide</i>. 

        :param self:
        :param vaultName:  The name of the vault.<br/>accountId:  The <code>AccountId</code> value is the AWS account ID of the account that owns the vault. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon S3 Glacier uses the AWS account ID associated with the credentials used to sign the request. If you use an account ID, do not include any hyphens ('-') in the ID.<br/>
        :return: vaultNotificationConfig:  Returns the notification configuration set on the vault.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Events</i> A list of one or more events for which Amazon S3 Glacier will send a notification to the specified Amazon SNS topic.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SNSTopic</i> The Amazon Simple Notification Service (Amazon SNS) topic Amazon Resource Name (ARN).<br/>        """
        pass

    def initiate_job(self, vaultName: str, accountId: str, jobParameters: JobParameters=None) -> InitiateJobOutput:
        """This operation initiates a job of the specified type, which can be a select, an archival retrieval, or a vault retrieval. For more information about using this operation, see the documentation for the underlying REST API <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/api-initiate-job-post.html">Initiate a Job</a>. 

        :param self:
        :param jobParameters:  Provides options for specifying job information.:<br/>vaultName:  The name of the vault.<br/>accountId:  The <code>AccountId</code> value is the AWS account ID of the account that owns the vault. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon S3 Glacier uses the AWS account ID associated with the credentials used to sign the request. If you use an account ID, do not include any hyphens ('-') in the ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputLocation</i> Contains information about the location where the select job results are stored.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>OutputLocation</b>: Contains information about the location where the select job results are stored.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SelectParameters</i> Contains the parameters that define a job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SelectParameters</b>: Contains the parameters that define a job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InventoryRetrievalParameters</i> Input parameters used for range inventory retrieval.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InventoryRetrievalParameters</b>: Input parameters used for range inventory retrieval.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tier</i> The tier to use for a select or an archive retrieval job. Valid values are <code>Expedited</code>, <code>Standard</code>, or <code>Bulk</code>. <code>Standard</code> is the default.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RetrievalByteRange</i> The byte range to retrieve for an archive retrieval. in the form "<i>StartByteValue</i>-<i>EndByteValue</i>" If not specified, the whole archive is retrieved. If specified, the byte range must be megabyte (1024*1024) aligned which means that <i>StartByteValue</i> must be divisible by 1 MB and <i>EndByteValue</i> plus 1 must be divisible by 1 MB or be the end of the archive specified as the archive byte size value minus 1. If RetrievalByteRange is not megabyte aligned, this operation returns a 400 response.  An error occurs if you specify this field for an inventory retrieval job request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SNSTopic</i> The Amazon SNS topic ARN to which Amazon S3 Glacier sends a notification when the job is completed and the output is ready for you to download. The specified topic publishes the notification to its subscribers. The SNS topic must exist.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The optional description for the job. The description must be less than or equal to 1,024 bytes. The allowable characters are 7-bit ASCII without control codes-specifically, ASCII values 32-126 decimal or 0x20-0x7E hexadecimal.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ArchiveId</i> The ID of the archive that you want to retrieve. This field is required only if <code>Type</code> is set to <code>select</code> or <code>archive-retrieval</code>code&gt;. An error occurs if you specify this request parameter for an inventory retrieval job request. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The job type. You can initiate a job to perform a select query on an archive, retrieve an archive, or get an inventory of a vault. Valid values are "select", "archive-retrieval" and "inventory-retrieval".<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> When initiating a job to retrieve a vault inventory, you can optionally add this parameter to your request to specify the output format. If you are initiating an inventory job and do not specify a Format field, JSON is the default format. Valid values are "CSV" and "JSON".<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Marker</i> An opaque string that represents where to continue pagination of the vault inventory retrieval results. You use the marker in a new <b>InitiateJob</b> request to obtain additional inventory items. If there are no more inventory items, this value is <code>null</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Limit</i> Specifies the maximum number of inventory items returned per vault inventory retrieval request. Valid values are greater than or equal to 1.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EndDate</i> The end of the date range in UTC for vault inventory retrieval that includes archives created before this date. This value should be a string in the ISO 8601 date format, for example <code>2013-03-20T17:03:43Z</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StartDate</i> The start of the date range in UTC for vault inventory retrieval that includes archives created on or after this date. This value should be a string in the ISO 8601 date format, for example <code>2013-03-20T17:03:43Z</code>.<br/>Describes the serialization format of the object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputSerialization</b>: Describes the serialization format of the object.Describes an S3 location that will receive the results of the job request.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3</b>: Describes an S3 location that will receive the results of the job request.
        :return: jobOutputPath:  The path to the location of where the select results are stored.<br/>jobId:  The ID of the job.<br/>location:  The relative URI path of the job.<br/>        """
        pass

    def initiate_multipart_upload(self, vaultName: str, accountId: str, archiveDescription: str=None, partSize: str=None) -> InitiateMultipartUploadOutput:
        """This operation initiates a multipart upload. Amazon S3 Glacier creates a multipart upload resource and returns its ID in the response. The multipart upload ID is used in subsequent requests to upload parts of an archive (see <a>UploadMultipartPart</a>). When you initiate a multipart upload, you specify the part size in number of bytes. The part size must be a megabyte (1024 KB) multiplied by a power of 2-for example, 1048576 (1 MB), 2097152 (2 MB), 4194304 (4 MB), 8388608 (8 MB), and so on. The minimum allowable part size is 1 MB, and the maximum is 4 GB. Every part you upload to this resource (see <a>UploadMultipartPart</a>), except the last one, must have the same size. The last one can be the same size or smaller. For example, suppose you want to upload a 16.2 MB file. If you initiate the multipart upload with a part size of 4 MB, you will upload four parts of 4 MB each and one part of 0.2 MB.  <note> You don't need to know the size of the archive when you start a multipart upload because Amazon S3 Glacier does not require you to specify the overall archive size. </note> After you complete the multipart upload, Amazon S3 Glacier (Glacier) removes the multipart upload resource referenced by the ID. Glacier also removes the multipart upload resource if you cancel the multipart upload or it may be removed if there is no activity for a period of 24 hours. An AWS account has full permission to perform all operations (actions). However, AWS Identity and Access Management (IAM) users don't have any permissions by default. You must grant them explicit permission to perform specific actions. For more information, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/using-iam-with-amazon-glacier.html">Access Control Using AWS Identity and Access Management (IAM)</a>. For conceptual information and underlying REST API, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/uploading-archive-mpu.html">Uploading Large Archives in Parts (Multipart Upload)</a> and <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/api-multipart-initiate-upload.html">Initiate Multipart Upload</a> in the <i>Amazon Glacier Developer Guide</i>.

        :param self:
        :param partSize:  The size of each part except the last, in bytes. The last part can be smaller than this part size.<br/>archiveDescription:  The archive description that you are uploading in parts. The part size must be a megabyte (1024 KB) multiplied by a power of 2, for example 1048576 (1 MB), 2097152 (2 MB), 4194304 (4 MB), 8388608 (8 MB), and so on. The minimum allowable part size is 1 MB, and the maximum is 4 GB (4096 MB).<br/>vaultName:  The name of the vault.<br/>accountId:  The <code>AccountId</code> value is the AWS account ID of the account that owns the vault. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon S3 Glacier uses the AWS account ID associated with the credentials used to sign the request. If you use an account ID, do not include any hyphens ('-') in the ID. <br/>
        :return: uploadId:  The ID of the multipart upload. This value is also included as part of the location.<br/>location:  The relative URI path of the multipart upload ID Amazon S3 Glacier created.<br/>        """
        pass

    def initiate_vault_lock(self, vaultName: str, accountId: str, policy: VaultLockPolicy=None) -> InitiateVaultLockOutput:
        """This operation initiates the vault locking process by doing the following: <ul> <li> Installing a vault lock policy on the specified vault. </li> <li> Setting the lock state of vault lock to <code>InProgress</code>. </li> <li> Returning a lock ID, which is used to complete the vault locking process. </li> </ul> You can set one vault lock policy for each vault and this policy can be up to 20 KB in size. For more information about vault lock policies, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/vault-lock-policy.html">Amazon Glacier Access Control with Vault Lock Policies</a>.  You must complete the vault locking process within 24 hours after the vault lock enters the <code>InProgress</code> state. After the 24 hour window ends, the lock ID expires, the vault automatically exits the <code>InProgress</code> state, and the vault lock policy is removed from the vault. You call <a>CompleteVaultLock</a> to complete the vault locking process by setting the state of the vault lock to <code>Locked</code>.  After a vault lock is in the <code>Locked</code> state, you cannot initiate a new vault lock for the vault. You can abort the vault locking process by calling <a>AbortVaultLock</a>. You can get the state of the vault lock by calling <a>GetVaultLock</a>. For more information about the vault locking process, <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/vault-lock.html">Amazon Glacier Vault Lock</a>. If this operation is called when the vault lock is in the <code>InProgress</code> state, the operation returns an <code>AccessDeniedException</code> error. When the vault lock is in the <code>InProgress</code> state you must call <a>AbortVaultLock</a> before you can initiate a new vault lock policy. 

        :param self:
        :param policy:  The vault lock policy as a JSON string, which uses "\" as an escape character.:<br/>vaultName:  The name of the vault.<br/>accountId:  The <code>AccountId</code> value is the AWS account ID. This value must match the AWS account ID associated with the credentials used to sign the request. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon Glacier uses the AWS account ID associated with the credentials used to sign the request. If you specify your account ID, do not include any hyphens ('-') in the ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Policy</i> The vault lock policy.<br/>
        :return: lockId:  The lock ID, which is used to complete the vault locking process.<br/>        """
        pass

    def list_jobs(self, vaultName: str, accountId: str, limit: str=None, marker: str=None, statuscode: str=None, completed: str=None) -> ListJobsOutput:
        """This operation lists jobs for a vault, including jobs that are in-progress and jobs that have recently finished. The List Job operation returns a list of these jobs sorted by job initiation time. <note> Amazon Glacier retains recently completed jobs for a period before deleting them; however, it eventually removes completed jobs. The output of completed jobs can be retrieved. Retaining completed jobs for a period of time after they have completed enables you to get a job output in the event you miss the job completion notification or your first attempt to download it fails. For example, suppose you start an archive retrieval job to download an archive. After the job completes, you start to download the archive but encounter a network error. In this scenario, you can retry and download the archive while the job exists. </note> The List Jobs operation supports pagination. You should always check the response <code>Marker</code> field. If there are no more jobs to list, the <code>Marker</code> field is set to <code>null</code>. If there are more jobs to list, the <code>Marker</code> field is set to a non-null value, which you can use to continue the pagination of the list. To return a list of jobs that begins at a specific job, set the marker request parameter to the <code>Marker</code> value for that job that you obtained from a previous List Jobs request. You can set a maximum limit for the number of jobs returned in the response by specifying the <code>limit</code> parameter in the request. The default limit is 50. The number of jobs returned might be fewer than the limit, but the number of returned jobs never exceeds the limit. Additionally, you can filter the jobs list returned by specifying the optional <code>statuscode</code> parameter or <code>completed</code> parameter, or both. Using the <code>statuscode</code> parameter, you can specify to return only jobs that match either the <code>InProgress</code>, <code>Succeeded</code>, or <code>Failed</code> status. Using the <code>completed</code> parameter, you can specify to return only jobs that were completed (<code>true</code>) or jobs that were not completed (<code>false</code>). For more information about using this operation, see the documentation for the underlying REST API <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/api-jobs-get.html">List Jobs</a>. 

        :param self:
        :param completed:  The state of the jobs to return. You can specify <code>true</code> or <code>false</code>.<br/>statuscode:  The type of job status to return. You can specify the following values: <code>InProgress</code>, <code>Succeeded</code>, or <code>Failed</code>.<br/>marker:  An opaque string used for pagination. This value specifies the job at which the listing of jobs should begin. Get the marker value from a previous List Jobs response. You only need to include the marker if you are continuing the pagination of results started in a previous List Jobs request.<br/>limit:  The maximum number of jobs to be returned. The default limit is 50. The number of jobs returned might be fewer than the specified limit, but the number of returned jobs never exceeds the limit.<br/>vaultName:  The name of the vault.<br/>accountId:  The <code>AccountId</code> value is the AWS account ID of the account that owns the vault. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon S3 Glacier uses the AWS account ID associated with the credentials used to sign the request. If you use an account ID, do not include any hyphens ('-') in the ID. <br/>
        :return: Marker:   An opaque string used for pagination that specifies the job at which the listing of jobs should begin. You get the <code>marker</code> value from a previous List Jobs response. You only need to include the marker if you are continuing the pagination of the results started in a previous List Jobs request. <br/>JobList:  A list of job objects. Each job object contains metadata describing the job.<br/>        """
        pass

    def list_multipart_uploads(self, vaultName: str, accountId: str, marker: str=None, limit: str=None) -> ListMultipartUploadsOutput:
        """This operation lists in-progress multipart uploads for the specified vault. An in-progress multipart upload is a multipart upload that has been initiated by an <a>InitiateMultipartUpload</a> request, but has not yet been completed or aborted. The list returned in the List Multipart Upload response has no guaranteed order.  The List Multipart Uploads operation supports pagination. By default, this operation returns up to 50 multipart uploads in the response. You should always check the response for a <code>marker</code> at which to continue the list; if there are no more items the <code>marker</code> is <code>null</code>. To return a list of multipart uploads that begins at a specific upload, set the <code>marker</code> request parameter to the value you obtained from a previous List Multipart Upload request. You can also limit the number of uploads returned in the response by specifying the <code>limit</code> parameter in the request. Note the difference between this operation and listing parts (<a>ListParts</a>). The List Multipart Uploads operation lists all multipart uploads for a vault and does not require a multipart upload ID. The List Parts operation requires a multipart upload ID since parts are associated with a single upload. An AWS account has full permission to perform all operations (actions). However, AWS Identity and Access Management (IAM) users don't have any permissions by default. You must grant them explicit permission to perform specific actions. For more information, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/using-iam-with-amazon-glacier.html">Access Control Using AWS Identity and Access Management (IAM)</a>. For conceptual information and the underlying REST API, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/working-with-archives.html">Working with Archives in Amazon S3 Glacier</a> and <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/api-multipart-list-uploads.html">List Multipart Uploads </a> in the <i>Amazon Glacier Developer Guide</i>.

        :param self:
        :param limit:  Specifies the maximum number of uploads returned in the response body. If this value is not specified, the List Uploads operation returns up to 50 uploads.<br/>marker:  An opaque string used for pagination. This value specifies the upload at which the listing of uploads should begin. Get the marker value from a previous List Uploads response. You need only include the marker if you are continuing the pagination of results started in a previous List Uploads request.<br/>vaultName:  The name of the vault.<br/>accountId:  The <code>AccountId</code> value is the AWS account ID of the account that owns the vault. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon S3 Glacier uses the AWS account ID associated with the credentials used to sign the request. If you use an account ID, do not include any hyphens ('-') in the ID. <br/>
        :return: Marker:  An opaque string that represents where to continue pagination of the results. You use the marker in a new List Multipart Uploads request to obtain more uploads in the list. If there are no more uploads, this value is <code>null</code>.<br/>UploadsList:  A list of in-progress multipart uploads.<br/>        """
        pass

    def list_parts(self, uploadId: str, vaultName: str, accountId: str, marker: str=None, limit: str=None) -> ListPartsOutput:
        """This operation lists the parts of an archive that have been uploaded in a specific multipart upload. You can make this request at any time during an in-progress multipart upload before you complete the upload (see <a>CompleteMultipartUpload</a>. List Parts returns an error for completed uploads. The list returned in the List Parts response is sorted by part range.  The List Parts operation supports pagination. By default, this operation returns up to 50 uploaded parts in the response. You should always check the response for a <code>marker</code> at which to continue the list; if there are no more items the <code>marker</code> is <code>null</code>. To return a list of parts that begins at a specific part, set the <code>marker</code> request parameter to the value you obtained from a previous List Parts request. You can also limit the number of parts returned in the response by specifying the <code>limit</code> parameter in the request.  An AWS account has full permission to perform all operations (actions). However, AWS Identity and Access Management (IAM) users don't have any permissions by default. You must grant them explicit permission to perform specific actions. For more information, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/using-iam-with-amazon-glacier.html">Access Control Using AWS Identity and Access Management (IAM)</a>. For conceptual information and the underlying REST API, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/working-with-archives.html">Working with Archives in Amazon S3 Glacier</a> and <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/api-multipart-list-parts.html">List Parts</a> in the <i>Amazon Glacier Developer Guide</i>.

        :param self:
        :param limit:  The maximum number of parts to be returned. The default limit is 50. The number of parts returned might be fewer than the specified limit, but the number of returned parts never exceeds the limit.<br/>marker:  An opaque string used for pagination. This value specifies the part at which the listing of parts should begin. Get the marker value from the response of a previous List Parts response. You need only include the marker if you are continuing the pagination of results started in a previous List Parts request.<br/>uploadId:  The upload ID of the multipart upload.<br/>vaultName:  The name of the vault.<br/>accountId:  The <code>AccountId</code> value is the AWS account ID of the account that owns the vault. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon S3 Glacier uses the AWS account ID associated with the credentials used to sign the request. If you use an account ID, do not include any hyphens ('-') in the ID. <br/>
        :return: Marker:  An opaque string that represents where to continue pagination of the results. You use the marker in a new List Parts request to obtain more jobs in the list. If there are no more parts, this value is <code>null</code>.<br/>Parts:  A list of the part sizes of the multipart upload. Each object in the array contains a <code>RangeBytes</code> and <code>sha256-tree-hash</code> name/value pair.<br/>CreationDate:  The UTC time at which the multipart upload was initiated.<br/>PartSizeInBytes:  The part size in bytes. This is the same value that you specified in the Initiate Multipart Upload request.<br/>ArchiveDescription:  The description of the archive that was specified in the Initiate Multipart Upload request.<br/>VaultARN:  The Amazon Resource Name (ARN) of the vault to which the multipart upload was initiated.<br/>MultipartUploadId:  The ID of the upload to which the parts are associated.<br/>        """
        pass

    def list_provisioned_capacity(self, accountId: str) -> ListProvisionedCapacityOutput:
        """This operation lists the provisioned capacity units for the specified AWS account.

        :param self:
        :param accountId:  The AWS account ID of the account that owns the vault. You can either specify an AWS account ID or optionally a single '-' (hyphen), in which case Amazon S3 Glacier uses the AWS account ID associated with the credentials used to sign the request. If you use an account ID, don't include any hyphens ('-') in the ID. <br/>
        :return: ProvisionedCapacityList:  The response body contains the following JSON fields.<br/>        """
        pass

    def list_tags_for_vault(self, vaultName: str, accountId: str) -> ListTagsForVaultOutput:
        """This operation lists all the tags attached to a vault. The operation returns an empty map if there are no tags. For more information about tags, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/tagging.html">Tagging Amazon S3 Glacier Resources</a>.

        :param self:
        :param vaultName:  The name of the vault.<br/>accountId:  The <code>AccountId</code> value is the AWS account ID of the account that owns the vault. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon S3 Glacier uses the AWS account ID associated with the credentials used to sign the request. If you use an account ID, do not include any hyphens ('-') in the ID.<br/>
        :return: Tags:  The tags attached to the vault. Each tag is composed of a key and a value.<br/>        """
        pass

    def list_vaults(self, accountId: str, marker: str=None, limit: str=None) -> ListVaultsOutput:
        """This operation lists all vaults owned by the calling user's account. The list returned in the response is ASCII-sorted by vault name. By default, this operation returns up to 10 items. If there are more vaults to list, the response <code>marker</code> field contains the vault Amazon Resource Name (ARN) at which to continue the list with a new List Vaults request; otherwise, the <code>marker</code> field is <code>null</code>. To return a list of vaults that begins at a specific vault, set the <code>marker</code> request parameter to the vault ARN you obtained from a previous List Vaults request. You can also limit the number of vaults returned in the response by specifying the <code>limit</code> parameter in the request.  An AWS account has full permission to perform all operations (actions). However, AWS Identity and Access Management (IAM) users don't have any permissions by default. You must grant them explicit permission to perform specific actions. For more information, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/using-iam-with-amazon-glacier.html">Access Control Using AWS Identity and Access Management (IAM)</a>. For conceptual information and underlying REST API, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/retrieving-vault-info.html">Retrieving Vault Metadata in Amazon S3 Glacier</a> and <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/api-vaults-get.html">List Vaults </a> in the <i>Amazon Glacier Developer Guide</i>. 

        :param self:
        :param limit:  The maximum number of vaults to be returned. The default limit is 10. The number of vaults returned might be fewer than the specified limit, but the number of returned vaults never exceeds the limit.<br/>marker:  A string used for pagination. The marker specifies the vault ARN after which the listing of vaults should begin.<br/>accountId:  The <code>AccountId</code> value is the AWS account ID. This value must match the AWS account ID associated with the credentials used to sign the request. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon Glacier uses the AWS account ID associated with the credentials used to sign the request. If you specify your account ID, do not include any hyphens ('-') in the ID.<br/>
        :return: Marker:  The vault ARN at which to continue pagination of the results. You use the marker in another List Vaults request to obtain more vaults in the list.<br/>VaultList:  List of vaults.<br/>        """
        pass

    def purchase_provisioned_capacity(self, accountId: str) -> PurchaseProvisionedCapacityOutput:
        """This operation purchases a provisioned capacity unit for an AWS account. 

        :param self:
        :param accountId:  The AWS account ID of the account that owns the vault. You can either specify an AWS account ID or optionally a single '-' (hyphen), in which case Amazon S3 Glacier uses the AWS account ID associated with the credentials used to sign the request. If you use an account ID, don't include any hyphens ('-') in the ID. <br/>
        :return: capacityId:  The ID that identifies the provisioned capacity unit.<br/>        """
        pass

    def remove_tags_from_vault(self, vaultName: str, accountId: str, TagKeys: List[string]=None):
        """This operation removes one or more tags from the set of tags attached to a vault. For more information about tags, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/tagging.html">Tagging Amazon S3 Glacier Resources</a>. This operation is idempotent. The operation will be successful, even if there are no tags attached to the vault. 

        :param self:
        :param TagKeys:  A list of tag keys. Each corresponding tag is removed from the vault.<br/>vaultName:  The name of the vault.<br/>accountId:  The <code>AccountId</code> value is the AWS account ID of the account that owns the vault. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon S3 Glacier uses the AWS account ID associated with the credentials used to sign the request. If you use an account ID, do not include any hyphens ('-') in the ID.<br/>
        :return:         """
        pass

    def set_data_retrieval_policy(self, accountId: str, Policy: DataRetrievalPolicy=None):
        """This operation sets and then enacts a data retrieval policy in the region specified in the PUT request. You can set one policy per region for an AWS account. The policy is enacted within a few minutes of a successful PUT operation. The set policy operation does not affect retrieval jobs that were in progress before the policy was enacted. For more information about data retrieval policies, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/data-retrieval-policy.html">Amazon Glacier Data Retrieval Policies</a>. 

        :param self:
        :param Policy:  The data retrieval policy in JSON format.:<br/>accountId:  The <code>AccountId</code> value is the AWS account ID. This value must match the AWS account ID associated with the credentials used to sign the request. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon Glacier uses the AWS account ID associated with the credentials used to sign the request. If you specify your account ID, do not include any hyphens ('-') in the ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Rules</i> The policy rule. Although this is a list type, currently there must be only one rule, which contains a Strategy field and optionally a BytesPerHour field.<br/>
        :return:         """
        pass

    def set_vault_access_policy(self, vaultName: str, accountId: str, policy: VaultAccessPolicy=None):
        """This operation configures an access policy for a vault and will overwrite an existing policy. To configure a vault access policy, send a PUT request to the <code>access-policy</code> subresource of the vault. An access policy is specific to a vault and is also called a vault subresource. You can set one access policy per vault and the policy can be up to 20 KB in size. For more information about vault access policies, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/vault-access-policy.html">Amazon Glacier Access Control with Vault Access Policies</a>. 

        :param self:
        :param policy:  The vault access policy as a JSON string.:<br/>vaultName:  The name of the vault.<br/>accountId:  The <code>AccountId</code> value is the AWS account ID of the account that owns the vault. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon S3 Glacier uses the AWS account ID associated with the credentials used to sign the request. If you use an account ID, do not include any hyphens ('-') in the ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Policy</i> The vault access policy.<br/>
        :return:         """
        pass

    def set_vault_notifications(self, vaultName: str, accountId: str, vaultNotificationConfig: VaultNotificationConfig=None):
        """This operation configures notifications that will be sent when specific events happen to a vault. By default, you don't get any notifications. To configure vault notifications, send a PUT request to the <code>notification-configuration</code> subresource of the vault. The request should include a JSON document that provides an Amazon SNS topic and specific events for which you want Amazon S3 Glacier to send notifications to the topic. Amazon SNS topics must grant permission to the vault to be allowed to publish notifications to the topic. You can configure a vault to publish a notification for the following vault events: <ul> <li>  <b>ArchiveRetrievalCompleted</b> This event occurs when a job that was initiated for an archive retrieval is completed (<a>InitiateJob</a>). The status of the completed job can be "Succeeded" or "Failed". The notification sent to the SNS topic is the same output as returned from <a>DescribeJob</a>.  </li> <li>  <b>InventoryRetrievalCompleted</b> This event occurs when a job that was initiated for an inventory retrieval is completed (<a>InitiateJob</a>). The status of the completed job can be "Succeeded" or "Failed". The notification sent to the SNS topic is the same output as returned from <a>DescribeJob</a>.  </li> </ul> An AWS account has full permission to perform all operations (actions). However, AWS Identity and Access Management (IAM) users don't have any permissions by default. You must grant them explicit permission to perform specific actions. For more information, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/using-iam-with-amazon-glacier.html">Access Control Using AWS Identity and Access Management (IAM)</a>. For conceptual information and underlying REST API, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/configuring-notifications.html">Configuring Vault Notifications in Amazon S3 Glacier</a> and <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/api-vault-notifications-put.html">Set Vault Notification Configuration </a> in the <i>Amazon Glacier Developer Guide</i>. 

        :param self:
        :param vaultNotificationConfig:  Provides options for specifying notification configuration.:<br/>vaultName:  The name of the vault.<br/>accountId:  The <code>AccountId</code> value is the AWS account ID of the account that owns the vault. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon S3 Glacier uses the AWS account ID associated with the credentials used to sign the request. If you use an account ID, do not include any hyphens ('-') in the ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Events</i> A list of one or more events for which Amazon S3 Glacier will send a notification to the specified Amazon SNS topic.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SNSTopic</i> The Amazon Simple Notification Service (Amazon SNS) topic Amazon Resource Name (ARN).<br/>
        :return:         """
        pass

    def upload_archive(self, accountId: str, vaultName: str, archiveDescription: str=None, checksum: str=None, body: Stream=None) -> ArchiveCreationOutput:
        """This operation adds an archive to a vault. This is a synchronous operation, and for a successful upload, your data is durably persisted. Amazon S3 Glacier returns the archive ID in the <code>x-amz-archive-id</code> header of the response.  You must use the archive ID to access your data in Amazon S3 Glacier. After you upload an archive, you should save the archive ID returned so that you can retrieve or delete the archive later. Besides saving the archive ID, you can also index it and give it a friendly name to allow for better searching. You can also use the optional archive description field to specify how the archive is referred to in an external index of archives, such as you might create in Amazon DynamoDB. You can also get the vault inventory to obtain a list of archive IDs in a vault. For more information, see <a>InitiateJob</a>.  You must provide a SHA256 tree hash of the data you are uploading. For information about computing a SHA256 tree hash, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/checksum-calculations.html">Computing Checksums</a>.  You can optionally specify an archive description of up to 1,024 printable ASCII characters. You can get the archive description when you either retrieve the archive or get the vault inventory. For more information, see <a>InitiateJob</a>. Amazon Glacier does not interpret the description in any way. An archive description does not need to be unique. You cannot use the description to retrieve or sort the archive list.  Archives are immutable. After you upload an archive, you cannot edit the archive or its description. An AWS account has full permission to perform all operations (actions). However, AWS Identity and Access Management (IAM) users don't have any permissions by default. You must grant them explicit permission to perform specific actions. For more information, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/using-iam-with-amazon-glacier.html">Access Control Using AWS Identity and Access Management (IAM)</a>.  For conceptual information and underlying REST API, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/uploading-an-archive.html">Uploading an Archive in Amazon Glacier</a> and <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/api-archive-post.html">Upload Archive</a> in the <i>Amazon Glacier Developer Guide</i>. 

        :param self:
        :param body:  The data to upload.<br/>checksum:  The SHA256 tree hash of the data being uploaded.<br/>archiveDescription:  The optional description of the archive you are uploading.<br/>accountId:  The <code>AccountId</code> value is the AWS account ID of the account that owns the vault. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon S3 Glacier uses the AWS account ID associated with the credentials used to sign the request. If you use an account ID, do not include any hyphens ('-') in the ID. <br/>vaultName:  The name of the vault.<br/>
        :return: archiveId:  The ID of the archive. This value is also included as part of the location.<br/>checksum:  The checksum of the archive computed by Amazon S3 Glacier.<br/>location:  The relative URI path of the newly added archive resource.<br/>        """
        pass

    def upload_multipart_part(self, uploadId: str, vaultName: str, accountId: str, checksum: str=None, range: str=None, body: Stream=None) -> UploadMultipartPartOutput:
        """This operation uploads a part of an archive. You can upload archive parts in any order. You can also upload them in parallel. You can upload up to 10,000 parts for a multipart upload. Amazon Glacier rejects your upload part request if any of the following conditions is true: <ul> <li>  <b>SHA256 tree hash does not match</b>To ensure that part data is not corrupted in transmission, you compute a SHA256 tree hash of the part and include it in your request. Upon receiving the part data, Amazon S3 Glacier also computes a SHA256 tree hash. If these hash values don't match, the operation fails. For information about computing a SHA256 tree hash, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/checksum-calculations.html">Computing Checksums</a>. </li> <li>  <b>Part size does not match</b>The size of each part except the last must match the size specified in the corresponding <a>InitiateMultipartUpload</a> request. The size of the last part must be the same size as, or smaller than, the specified size. <note> If you upload a part whose size is smaller than the part size you specified in your initiate multipart upload request and that part is not the last part, then the upload part request will succeed. However, the subsequent Complete Multipart Upload request will fail. </note> </li> <li>  <b>Range does not align</b>The byte range value in the request does not align with the part size specified in the corresponding initiate request. For example, if you specify a part size of 4194304 bytes (4 MB), then 0 to 4194303 bytes (4 MB - 1) and 4194304 (4 MB) to 8388607 (8 MB - 1) are valid part ranges. However, if you set a range value of 2 MB to 6 MB, the range does not align with the part size and the upload will fail.  </li> </ul> This operation is idempotent. If you upload the same part multiple times, the data included in the most recent request overwrites the previously uploaded data. An AWS account has full permission to perform all operations (actions). However, AWS Identity and Access Management (IAM) users don't have any permissions by default. You must grant them explicit permission to perform specific actions. For more information, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/using-iam-with-amazon-glacier.html">Access Control Using AWS Identity and Access Management (IAM)</a>.  For conceptual information and underlying REST API, see <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/uploading-archive-mpu.html">Uploading Large Archives in Parts (Multipart Upload)</a> and <a href="https://docs.aws.amazon.com/amazonglacier/latest/dev/api-upload-part.html">Upload Part </a> in the <i>Amazon Glacier Developer Guide</i>.

        :param self:
        :param body:  The data to upload.<br/>range:  Identifies the range of bytes in the assembled archive that will be uploaded in this part. Amazon S3 Glacier uses this information to assemble the archive in the proper sequence. The format of this header follows RFC 2616. An example header is Content-Range:bytes 0-4194303/*.<br/>checksum:  The SHA256 tree hash of the data being uploaded.<br/>uploadId:  The upload ID of the multipart upload.<br/>vaultName:  The name of the vault.<br/>accountId:  The <code>AccountId</code> value is the AWS account ID of the account that owns the vault. You can either specify an AWS account ID or optionally a single '<code>-</code>' (hyphen), in which case Amazon S3 Glacier uses the AWS account ID associated with the credentials used to sign the request. If you use an account ID, do not include any hyphens ('-') in the ID. <br/>
        :return: checksum:  The SHA256 tree hash that Amazon S3 Glacier computed for the uploaded part.<br/>        """
        pass

    class GlacierResource:
        
        class Account:
            """Creates a Account resource.::

  account = glacier.Account('id')

:type id: string
:param id: The Account's id identifier. This **must** be set.

:rtype: :py:class:`Glacier.Account`
:returns: A Account resource"""

            def __init__(self, id):
                pass
            
        
            
            def create_vault(self, **kwargs):
                # type: (object) -> Vault
                return Vault()


        
        class Archive:
            """Creates a Archive resource.::

  archive = glacier.Archive('account_id','vault_name','id')

:type account_id: string
:param account_id: The Archive's account_id identifier. This **must** be set.
:type vault_name: string
:param vault_name: The Archive's vault_name identifier. This **must** be set.
:type id: string
:param id: The Archive's id identifier. This **must** be set.

:rtype: :py:class:`Glacier.Archive`
:returns: A Archive resource"""

            def __init__(self, account_id, vault_name, id):
                pass
            
        
            
            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def initiate_archive_retrieval(self, **kwargs):
                # type: (object) -> Job
                return Job()


        
        class Job:
            """Creates a Job resource.::

  job = glacier.Job('account_id','vault_name','id')

:type account_id: string
:param account_id: The Job's account_id identifier. This **must** be set.
:type vault_name: string
:param vault_name: The Job's vault_name identifier. This **must** be set.
:type id: string
:param id: The Job's id identifier. This **must** be set.

:rtype: :py:class:`Glacier.Job`
:returns: A Job resource"""

            def __init__(self, account_id, vault_name, id):
                pass
            
            class SelectParameters(Mapping):
                pass

            class OutputLocation(Mapping):
                pass

            class InventoryRetrievalJobDescription(Mapping):
                pass

            class Size(object):
                pass

            class boolean(object):
                pass
        
            job_id: str=None
            job_description: str=None
            action: str=None
            archive_id: str=None
            vault_arn: str=None
            creation_date: str=None
            completed: boolean=None
            status_code: str=None
            status_message: str=None
            archive_size_in_bytes: Size=None
            inventory_size_in_bytes: Size=None
            sns_topic: str=None
            completion_date: str=None
            sha256_tree_hash: str=None
            archive_sha256_tree_hash: str=None
            retrieval_byte_range: str=None
            tier: str=None
            inventory_retrieval_parameters: InventoryRetrievalJobDescription=None
            job_output_path: str=None
            select_parameters: SelectParameters=None
            output_location: OutputLocation=None
            
            def get_output(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class MultipartUpload:
            """Creates a MultipartUpload resource.::

  multipart_upload = glacier.MultipartUpload('account_id','vault_name','id')

:type account_id: string
:param account_id: The MultipartUpload's account_id identifier. This **must** be set.
:type vault_name: string
:param vault_name: The MultipartUpload's vault_name identifier. This **must** be set.
:type id: string
:param id: The MultipartUpload's id identifier. This **must** be set.

:rtype: :py:class:`Glacier.MultipartUpload`
:returns: A MultipartUpload resource"""

            def __init__(self, account_id, vault_name, id):
                pass
            
            class long(object):
                pass
        
            multipart_upload_id: str=None
            vault_arn: str=None
            archive_description: str=None
            part_size_in_bytes: long=None
            creation_date: str=None
            
            def abort(self, **kwargs):
                # type: (object) -> dict
                return {}

            def complete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def parts(self, **kwargs):
                # type: (object) -> dict
                return {}

            def upload_part(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class Notification:
            """Creates a Notification resource.::

  notification = glacier.Notification('account_id','vault_name')

:type account_id: string
:param account_id: The Notification's account_id identifier. This **must** be set.
:type vault_name: string
:param vault_name: The Notification's vault_name identifier. This **must** be set.

:rtype: :py:class:`Glacier.Notification`
:returns: A Notification resource"""

            def __init__(self, account_id, vault_name):
                pass
            
            class string(object):
                pass
        
            sns_topic: str=None
            events: List[string]=None
            
            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def set(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class Vault:
            """Creates a Vault resource.::

  vault = glacier.Vault('account_id','name')

:type account_id: string
:param account_id: The Vault's account_id identifier. This **must** be set.
:type name: string
:param name: The Vault's name identifier. This **must** be set.

:rtype: :py:class:`Glacier.Vault`
:returns: A Vault resource"""

            def __init__(self, account_id, name):
                pass
            
            class long(object):
                pass
        
            vault_arn: str=None
            vault_name: str=None
            creation_date: str=None
            last_inventory_date: str=None
            number_of_archives: long=None
            size_in_bytes: long=None
            
            def create(self, **kwargs):
                # type: (object) -> dict
                return {}

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def initiate_inventory_retrieval(self, **kwargs):
                # type: (object) -> Job
                return Job()

            def initiate_multipart_upload(self, **kwargs):
                # type: (object) -> MultipartUpload
                return MultipartUpload()

            def upload_archive(self, **kwargs):
                # type: (object) -> Archive
                return Archive()


        def create_vault(self, **kwargs):
            # type: (object) -> Vault
            pass

        def get_available_subresources(self) -> List[str]:
            pass

        class VaultResourceCollection(List[Vault], ResourceCollection):
            pass

        vaults: VaultResourceCollection = None




class GlobalAccelerator(BaseClient):
    """<fullname>AWS Global Accelerator</fullname> This is the <i>AWS Global Accelerator API Reference</i>. This guide is for developers who need detailed information about AWS Global Accelerator API actions, data types, and errors. For more information about Global Accelerator features, see the <a href="https://docs.aws.amazon.com/global-accelerator/latest/dg/Welcome.html">AWS Global Accelerator Developer Guide</a>.  AWS Global Accelerator is a network layer service in which you create accelerators to improve availability and performance for internet applications used by a global audience.  Global Accelerator provides you with static IP addresses that you associate with your accelerator. These IP addresses are anycast from the AWS edge network and distribute incoming application traffic across multiple endpoint resources in multiple AWS Regions, which increases the availability of your applications. Endpoints can be Elastic IP addresses, Network Load Balancers, and Application Load Balancers that are located in one AWS Region or multiple Regions. Global Accelerator uses the AWS global network to route traffic to the optimal regional endpoint based on health, client location, and policies that you configure. The service reacts instantly to changes in health or configuration to ensure that internet traffic from clients is directed to only healthy endpoints. Global Accelerator includes components that work together to help you improve performance and availability for your applications: <dl> <dt>Static IP address</dt> <dd> AWS Global Accelerator provides you with a set of static IP addresses which are anycast from the AWS edge network and serve as the single fixed entry points for your clients. If you already have Elastic Load Balancing or Elastic IP address resources set up for your applications, you can easily add those to Global Accelerator to allow the resources to be accessed by a Global Accelerator static IP address. </dd> <dt>Accelerator</dt> <dd> An accelerator directs traffic to optimal endpoints over the AWS global network to improve availability and performance for your internet applications that have a global audience. Each accelerator includes one or more listeners. </dd> <dt>Network zone</dt> <dd> A network zone services the static IP addresses for your accelerator from a unique IP subnet. Similar to an AWS Availability Zone, a network zone is an isolated unit with its own set of physical infrastructure. When you configure an accelerator, Global Accelerator allocates two IPv4 addresses for it. If one IP address from a network zone becomes unavailable due to IP address blocking by certain client networks, or network disruptions, then client applications can retry on the healthy static IP address from the other isolated network zone. </dd> <dt>Listener</dt> <dd> A listener processes inbound connections from clients to Global Accelerator, based on the protocol and port that you configure. Each listener has one or more endpoint groups associated with it, and traffic is forwarded to endpoints in one of the groups. You associate endpoint groups with listeners by specifying the Regions that you want to distribute traffic to. Traffic is distributed to optimal endpoints within the endpoint groups associated with a listener. </dd> <dt>Endpoint group</dt> <dd> Each endpoint group is associated with a specific AWS Region. Endpoint groups include one or more endpoints in the Region. You can increase or reduce the percentage of traffic that would be otherwise directed to an endpoint group by adjusting a setting called a <i>traffic dial</i>. The traffic dial lets you easily do performance testing or blue/green deployment testing for new releases across different AWS Regions, for example.  </dd> <dt>Endpoint</dt> <dd> An endpoint is an Elastic IP address, Network Load Balancer, or Application Load Balancer. Traffic is routed to endpoints based on several factors, including the geo-proximity to the user, the health of the endpoint, and the configuration options that you choose, such as endpoint weights. For each endpoint, you can configure weights, which are numbers that you can use to specify the proportion of traffic to route to each one. This can be useful, for example, to do performance testing within a Region. </dd> </dl>"""


    class ListAcceleratorsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_accelerators` for valid parameters"""
            pass
    class ListEndpointGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_endpoint_groups` for valid parameters"""
            pass
    class ListListenersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_listeners` for valid parameters"""
            pass


    class CreateAcceleratorRequest(Mapping):
        pass
    
    class CreateAcceleratorResponse(Mapping):
        pass
    
    class GenericBoolean(object):
        pass
    
    class CreateEndpointGroupRequest(Mapping):
        pass
    
    class CreateEndpointGroupResponse(Mapping):
        pass
    
    class EndpointConfiguration(Mapping):
        pass
    
    class TrafficDialPercentage(object):
        pass
    
    class CreateListenerRequest(Mapping):
        pass
    
    class CreateListenerResponse(Mapping):
        pass
    
    class PortRange(Mapping):
        pass
    
    class DeleteAcceleratorRequest(Mapping):
        pass
    
    class DeleteEndpointGroupRequest(Mapping):
        pass
    
    class DeleteListenerRequest(Mapping):
        pass
    
    class DescribeAcceleratorRequest(Mapping):
        pass
    
    class DescribeAcceleratorResponse(Mapping):
        pass
    
    class DescribeAcceleratorAttributesRequest(Mapping):
        pass
    
    class DescribeAcceleratorAttributesResponse(Mapping):
        pass
    
    class DescribeEndpointGroupRequest(Mapping):
        pass
    
    class DescribeEndpointGroupResponse(Mapping):
        pass
    
    class DescribeListenerRequest(Mapping):
        pass
    
    class DescribeListenerResponse(Mapping):
        pass
    
    class ListAcceleratorsRequest(Mapping):
        pass
    
    class ListAcceleratorsResponse(Mapping):
        pass
    
    class ListEndpointGroupsRequest(Mapping):
        pass
    
    class ListEndpointGroupsResponse(Mapping):
        pass
    
    class ListListenersRequest(Mapping):
        pass
    
    class ListListenersResponse(Mapping):
        pass
    
    class UpdateAcceleratorRequest(Mapping):
        pass
    
    class UpdateAcceleratorResponse(Mapping):
        pass
    
    class UpdateAcceleratorAttributesRequest(Mapping):
        pass
    
    class UpdateAcceleratorAttributesResponse(Mapping):
        pass
    
    class UpdateEndpointGroupRequest(Mapping):
        pass
    
    class UpdateEndpointGroupResponse(Mapping):
        pass
    
    class UpdateListenerRequest(Mapping):
        pass
    
    class UpdateListenerResponse(Mapping):
        pass
    
    def create_accelerator(self, IdempotencyToken: str, Name: str, IpAddressType: str=None, Enabled: GenericBoolean=None) -> CreateAcceleratorResponse:
        """Create an accelerator. An accelerator includes one or more listeners that process inbound connections and direct traffic to one or more endpoint groups, each of which includes endpoints, such as Network Load Balancers. To see an AWS CLI example of creating an accelerator, scroll down to <b>Example</b>.

        :param self:
        :param IdempotencyToken:  A unique, case-sensitive identifier that you provide to ensure the idempotency—that is, the uniqueness—of an accelerator.<br/>Enabled:  Indicates whether an accelerator is enabled. The value is true or false. The default value is true.  If the value is set to true, an accelerator cannot be deleted. If set to false, the accelerator can be deleted.<br/>IpAddressType:  The value for the address type must be IPv4. <br/>Name:  The name of an accelerator. The name can have a maximum of 32 characters, must contain only alphanumeric characters or hyphens (-), and must not begin or end with a hyphen.<br/>
        :return: Accelerator:  The accelerator that is created by specifying a listener and the supported IP address types.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedTime</i> The date and time that the accelerator was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The date and time that the accelerator was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Describes the deployment status of the accelerator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IpSets</i> IP address set associated with the accelerator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Indicates whether theaccelerator is enabled. The value is true or false. The default value is true.  If the value is set to true, the accelerator cannot be deleted. If set to false, accelerator can be deleted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IpAddressType</i> The value for the address type must be IPv4. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the accelerator. The name can have a maximum of 32 characters, must contain only alphanumeric characters or hyphens (-), and must not begin or end with a hyphen.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AcceleratorArn</i> The Amazon Resource Name (ARN) of the accelerator.<br/>        """
        pass

    def create_endpoint_group(self, IdempotencyToken: str, EndpointGroupRegion: str, ListenerArn: str, EndpointConfigurations: List[EndpointConfiguration]=None, TrafficDialPercentage: TrafficDialPercentage=None, HealthCheckPort: int=None, HealthCheckProtocol: str=None, HealthCheckPath: str=None, HealthCheckIntervalSeconds: int=None, ThresholdCount: int=None) -> CreateEndpointGroupResponse:
        """Create an endpoint group for the specified listener. An endpoint group is a collection of endpoints in one AWS Region. To see an AWS CLI example of creating an endpoint group, scroll down to <b>Example</b>.

        :param self:
        :param IdempotencyToken:  A unique, case-sensitive identifier that you provide to ensure the idempotency—that is, the uniqueness—of the request.<br/>ThresholdCount:  The number of consecutive health checks required to set the state of a healthy endpoint to unhealthy, or to set an unhealthy endpoint to healthy. The default value is 3.<br/>HealthCheckIntervalSeconds:  The time—10 seconds or 30 seconds—between each health check for an endpoint. The default value is 30.<br/>HealthCheckPath:  If the protocol is HTTP/S, then this specifies the path that is the destination for health check targets. The default value is slash (/).<br/>HealthCheckProtocol:  The protocol that AWS Global Accelerator uses to check the health of endpoints that are part of this endpoint group. The default value is TCP.<br/>HealthCheckPort:  The port that AWS Global Accelerator uses to check the health of endpoints that are part of this endpoint group. The default port is the listener port that this endpoint group is associated with. If listener port is a list of ports, Global Accelerator uses the first port in the list.<br/>TrafficDialPercentage:  The percentage of traffic to send to an AWS Region. Additional traffic is distributed to other endpoint groups for this listener.  Use this action to increase (dial up) or decrease (dial down) traffic to a specific Region. The percentage is applied to the traffic that would otherwise have been routed to the Region based on optimal routing. The default value is 100.<br/>EndpointConfigurations:  The list of endpoint objects.<br/>EndpointGroupRegion:  The name of the AWS Region where the endpoint group is located. A listener can have only one endpoint group in a specific Region.<br/>ListenerArn:  The Amazon Resource Name (ARN) of the listener.<br/>
        :return: EndpointGroup:  The information about the endpoint group that was created.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThresholdCount</i> The number of consecutive health checks required to set the state of a healthy endpoint to unhealthy, or to set an unhealthy endpoint to healthy. The default value is 3.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HealthCheckIntervalSeconds</i> The time—10 seconds or 30 seconds—between health checks for each endpoint. The default value is 30.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HealthCheckPath</i> If the protocol is HTTP/S, then this value provides the ping path that Global Accelerator uses for the destination on the endpoints for health checks. The default is slash (/).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HealthCheckProtocol</i> The protocol that Global Accelerator uses to perform health checks on endpoints that are part of this endpoint group. The default value is TCP.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HealthCheckPort</i> The port that Global Accelerator uses to perform health checks on endpoints that are part of this endpoint group.  The default port is the port for the listener that this endpoint group is associated with. If the listener port is a list, Global Accelerator uses the first specified port in the list of ports.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrafficDialPercentage</i> The percentage of traffic to send to an AWS Region. Additional traffic is distributed to other endpoint groups for this listener.  Use this action to increase (dial up) or decrease (dial down) traffic to a specific Region. The percentage is applied to the traffic that would otherwise have been routed to the Region based on optimal routing. The default value is 100.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointDescriptions</i> The list of endpoint objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointGroupRegion</i> The AWS Region that this endpoint group belongs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointGroupArn</i> The Amazon Resource Name (ARN) of the endpoint group.<br/>        """
        pass

    def create_listener(self, IdempotencyToken: str, Protocol: str, PortRanges: List[PortRange], AcceleratorArn: str, ClientAffinity: str=None) -> CreateListenerResponse:
        """Create a listener to process inbound connections from clients to an accelerator. Connections arrive to assigned static IP addresses on a port, port range, or list of port ranges that you specify. To see an AWS CLI example of creating a listener, scroll down to <b>Example</b>.

        :param self:
        :param IdempotencyToken:  A unique, case-sensitive identifier that you provide to ensure the idempotency—that is, the uniqueness—of the request.<br/>ClientAffinity:  Client affinity lets you direct all requests from a user to the same endpoint, if you have stateful applications, regardless of the port and protocol of the client request. Clienty affinity gives you control over whether to always route each client to the same specific endpoint. AWS Global Accelerator uses a consistent-flow hashing algorithm to choose the optimal endpoint for a connection. If client affinity is <code>NONE</code>, Global Accelerator uses the "five-tuple" (5-tuple) properties—source IP address, source port, destination IP address, destination port, and protocol—to select the hash value, and then chooses the best endpoint. However, with this setting, if someone uses different ports to connect to Global Accelerator, their connections might not be always routed to the same endpoint because the hash value changes.  If you want a given client to always be routed to the same endpoint, set client affinity to <code>SOURCE_IP</code> instead. When you use the <code>SOURCE_IP</code> setting, Global Accelerator uses the "two-tuple" (2-tuple) properties— source (client) IP address and destination IP address—to select the hash value. The default value is <code>NONE</code>.<br/>Protocol:  The protocol for connections from clients to your accelerator.<br/>PortRanges:  The list of port ranges to support for connections from clients to your accelerator.<br/>AcceleratorArn:  The Amazon Resource Name (ARN) of your accelerator.<br/>
        :return: Listener:  The listener that you've created.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientAffinity</i> Client affinity lets you direct all requests from a user to the same endpoint, if you have stateful applications, regardless of the port and protocol of the client request. Clienty affinity gives you control over whether to always route each client to the same specific endpoint. AWS Global Accelerator uses a consistent-flow hashing algorithm to choose the optimal endpoint for a connection. If client affinity is <code>NONE</code>, Global Accelerator uses the "five-tuple" (5-tuple) properties—source IP address, source port, destination IP address, destination port, and protocol—to select the hash value, and then chooses the best endpoint. However, with this setting, if someone uses different ports to connect to Global Accelerator, their connections might not be always routed to the same endpoint because the hash value changes.  If you want a given client to always be routed to the same endpoint, set client affinity to <code>SOURCE_IP</code> instead. When you use the <code>SOURCE_IP</code> setting, Global Accelerator uses the "two-tuple" (2-tuple) properties— source (client) IP address and destination IP address—to select the hash value. The default value is <code>NONE</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Protocol</i> The protocol for the connections from clients to the accelerator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PortRanges</i> The list of port ranges for the connections from clients to the accelerator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ListenerArn</i> The Amazon Resource Name (ARN) of the listener.<br/>        """
        pass

    def delete_accelerator(self, AcceleratorArn: str):
        """Delete an accelerator. Note: before you can delete an accelerator, you must disable it and remove all dependent resources (listeners and endpoint groups).

        :param self:
        :param AcceleratorArn:  The Amazon Resource Name (ARN) of an accelerator.<br/>
        :return:         """
        pass

    def delete_endpoint_group(self, EndpointGroupArn: str):
        """Delete an endpoint group from a listener.

        :param self:
        :param EndpointGroupArn:  The Amazon Resource Name (ARN) of the endpoint group to delete.<br/>
        :return:         """
        pass

    def delete_listener(self, ListenerArn: str):
        """Delete a listener from an accelerator.

        :param self:
        :param ListenerArn:  The Amazon Resource Name (ARN) of the listener.<br/>
        :return:         """
        pass

    def describe_accelerator(self, AcceleratorArn: str) -> DescribeAcceleratorResponse:
        """Describe an accelerator. To see an AWS CLI example of describing an accelerator, scroll down to <b>Example</b>.

        :param self:
        :param AcceleratorArn:  The Amazon Resource Name (ARN) of the accelerator to describe.<br/>
        :return: Accelerator:  The description of the accelerator.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedTime</i> The date and time that the accelerator was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The date and time that the accelerator was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Describes the deployment status of the accelerator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IpSets</i> IP address set associated with the accelerator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Indicates whether theaccelerator is enabled. The value is true or false. The default value is true.  If the value is set to true, the accelerator cannot be deleted. If set to false, accelerator can be deleted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IpAddressType</i> The value for the address type must be IPv4. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the accelerator. The name can have a maximum of 32 characters, must contain only alphanumeric characters or hyphens (-), and must not begin or end with a hyphen.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AcceleratorArn</i> The Amazon Resource Name (ARN) of the accelerator.<br/>        """
        pass

    def describe_accelerator_attributes(self, AcceleratorArn: str=None) -> DescribeAcceleratorAttributesResponse:
        """Describe the attributes of an accelerator.

        :param self:
        :param AcceleratorArn:  The Amazon Resource Name (ARN) of the accelerator with the attributes that you want to describe. Value is required.<br/>
        :return: AcceleratorAttributes:  The attributes of the accelerator.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FlowLogsS3Prefix</i> The prefix for the location in the Amazon S3 bucket for the flow logs. Attribute is required if <code>FlowLogsEnabled</code> is <code>true</code>. If you don’t specify a prefix, the flow logs are stored in the root of the bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FlowLogsS3Bucket</i> The name of the Amazon S3 bucket for the flow logs. Attribute is required if <code>FlowLogsEnabled</code> is <code>true</code>. The bucket must exist and have a bucket policy that grants AWS Global Accelerator permission to write to the bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FlowLogsEnabled</i> Indicates whether flow logs are enabled. The default value is false. If the value is true, <code>FlowLogsS3Bucket</code> and <code>FlowLogsS3Prefix</code> must be specified. For more information, see <a href="https://docs.aws.amazon.com/global-accelerator/latest/dg/monitoring-global-accelerator.flow-logs.html">Flow Logs</a> in the <i>AWS Global Accelerator Developer Guide</i>.<br/>        """
        pass

    def describe_endpoint_group(self, EndpointGroupArn: str) -> DescribeEndpointGroupResponse:
        """Describe an endpoint group.

        :param self:
        :param EndpointGroupArn:  The Amazon Resource Name (ARN) of the endpoint group to describe.<br/>
        :return: EndpointGroup:  The description of an endpoint group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThresholdCount</i> The number of consecutive health checks required to set the state of a healthy endpoint to unhealthy, or to set an unhealthy endpoint to healthy. The default value is 3.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HealthCheckIntervalSeconds</i> The time—10 seconds or 30 seconds—between health checks for each endpoint. The default value is 30.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HealthCheckPath</i> If the protocol is HTTP/S, then this value provides the ping path that Global Accelerator uses for the destination on the endpoints for health checks. The default is slash (/).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HealthCheckProtocol</i> The protocol that Global Accelerator uses to perform health checks on endpoints that are part of this endpoint group. The default value is TCP.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HealthCheckPort</i> The port that Global Accelerator uses to perform health checks on endpoints that are part of this endpoint group.  The default port is the port for the listener that this endpoint group is associated with. If the listener port is a list, Global Accelerator uses the first specified port in the list of ports.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrafficDialPercentage</i> The percentage of traffic to send to an AWS Region. Additional traffic is distributed to other endpoint groups for this listener.  Use this action to increase (dial up) or decrease (dial down) traffic to a specific Region. The percentage is applied to the traffic that would otherwise have been routed to the Region based on optimal routing. The default value is 100.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointDescriptions</i> The list of endpoint objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointGroupRegion</i> The AWS Region that this endpoint group belongs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointGroupArn</i> The Amazon Resource Name (ARN) of the endpoint group.<br/>        """
        pass

    def describe_listener(self, ListenerArn: str) -> DescribeListenerResponse:
        """Describe a listener.

        :param self:
        :param ListenerArn:  The Amazon Resource Name (ARN) of the listener to describe.<br/>
        :return: Listener:  The description of a listener.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientAffinity</i> Client affinity lets you direct all requests from a user to the same endpoint, if you have stateful applications, regardless of the port and protocol of the client request. Clienty affinity gives you control over whether to always route each client to the same specific endpoint. AWS Global Accelerator uses a consistent-flow hashing algorithm to choose the optimal endpoint for a connection. If client affinity is <code>NONE</code>, Global Accelerator uses the "five-tuple" (5-tuple) properties—source IP address, source port, destination IP address, destination port, and protocol—to select the hash value, and then chooses the best endpoint. However, with this setting, if someone uses different ports to connect to Global Accelerator, their connections might not be always routed to the same endpoint because the hash value changes.  If you want a given client to always be routed to the same endpoint, set client affinity to <code>SOURCE_IP</code> instead. When you use the <code>SOURCE_IP</code> setting, Global Accelerator uses the "two-tuple" (2-tuple) properties— source (client) IP address and destination IP address—to select the hash value. The default value is <code>NONE</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Protocol</i> The protocol for the connections from clients to the accelerator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PortRanges</i> The list of port ranges for the connections from clients to the accelerator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ListenerArn</i> The Amazon Resource Name (ARN) of the listener.<br/>        """
        pass

    def list_accelerators(self, MaxResults: int=None, NextToken: str=None) -> ListAcceleratorsResponse:
        """List the accelerators for an AWS account.

        :param self:
        :param NextToken:  The token for the next set of results. You receive this token from a previous call.<br/>MaxResults:  The number of Global Accelerator objects that you want to return with this call. The default value is 10.<br/>
        :return: NextToken:  The token for the next set of results. You receive this token from a previous call.<br/>Accelerators:  The list of accelerators for a customer account.<br/>        """
        pass

    def list_endpoint_groups(self, ListenerArn: str, MaxResults: int=None, NextToken: str=None) -> ListEndpointGroupsResponse:
        """List the endpoint groups that are associated with a listener.

        :param self:
        :param NextToken:  The token for the next set of results. You receive this token from a previous call.<br/>MaxResults:  The number of endpoint group objects that you want to return with this call. The default value is 10.<br/>ListenerArn:  The Amazon Resource Name (ARN) of the listener.<br/>
        :return: NextToken:  The token for the next set of results. You receive this token from a previous call.<br/>EndpointGroups:  The list of the endpoint groups associated with a listener.<br/>        """
        pass

    def list_listeners(self, AcceleratorArn: str, MaxResults: int=None, NextToken: str=None) -> ListListenersResponse:
        """List the listeners for an accelerator.

        :param self:
        :param NextToken:  The token for the next set of results. You receive this token from a previous call.<br/>MaxResults:  The number of listener objects that you want to return with this call. The default value is 10.<br/>AcceleratorArn:  The Amazon Resource Name (ARN) of the accelerator for which you want to list listener objects.<br/>
        :return: NextToken:  The token for the next set of results. You receive this token from a previous call.<br/>Listeners:  The list of listeners for an accelerator.<br/>        """
        pass

    def update_accelerator(self, AcceleratorArn: str, Name: str=None, IpAddressType: str=None, Enabled: GenericBoolean=None) -> UpdateAcceleratorResponse:
        """Update an accelerator.

        :param self:
        :param Enabled:  Indicates whether an accelerator is enabled. The value is true or false. The default value is true.  If the value is set to true, the accelerator cannot be deleted. If set to false, the accelerator can be deleted.<br/>IpAddressType:  The value for the address type must be IPv4. <br/>Name:  The name of the accelerator. The name can have a maximum of 32 characters, must contain only alphanumeric characters or hyphens (-), and must not begin or end with a hyphen.<br/>AcceleratorArn:  The Amazon Resource Name (ARN) of the accelerator to update.<br/>
        :return: Accelerator:  Information about the updated accelerator.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedTime</i> The date and time that the accelerator was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The date and time that the accelerator was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Describes the deployment status of the accelerator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IpSets</i> IP address set associated with the accelerator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Indicates whether theaccelerator is enabled. The value is true or false. The default value is true.  If the value is set to true, the accelerator cannot be deleted. If set to false, accelerator can be deleted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IpAddressType</i> The value for the address type must be IPv4. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the accelerator. The name can have a maximum of 32 characters, must contain only alphanumeric characters or hyphens (-), and must not begin or end with a hyphen.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AcceleratorArn</i> The Amazon Resource Name (ARN) of the accelerator.<br/>        """
        pass

    def update_accelerator_attributes(self, AcceleratorArn: str=None, FlowLogsEnabled: GenericBoolean=None, FlowLogsS3Bucket: str=None, FlowLogsS3Prefix: str=None) -> UpdateAcceleratorAttributesResponse:
        """Update the attributes for an accelerator. To see an AWS CLI example of updating an accelerator to enable flow logs, scroll down to <b>Example</b>.

        :param self:
        :param FlowLogsS3Prefix:  Update the prefix for the location in the Amazon S3 bucket for the flow logs. Attribute is required if <code>FlowLogsEnabled</code> is <code>true</code>. If you don’t specify a prefix, the flow logs are stored in the root of the bucket.<br/>FlowLogsS3Bucket:  The name of the Amazon S3 bucket for the flow logs. Attribute is required if <code>FlowLogsEnabled</code> is <code>true</code>. The bucket must exist and have a bucket policy that grants AWS Global Accelerator permission to write to the bucket.<br/>FlowLogsEnabled:  Update whether flow logs are enabled. The default value is false. If the value is true, <code>FlowLogsS3Bucket</code> and <code>FlowLogsS3Prefix</code> must be specified. For more information, see <a href="https://docs.aws.amazon.com/global-accelerator/latest/dg/monitoring-global-accelerator.flow-logs.html">Flow Logs</a> in the <i>AWS Global Accelerator Developer Guide</i>.<br/>AcceleratorArn:  The Amazon Resource Name (ARN) of the accelerator that you want to update. Attribute is required.<br/>
        :return: AcceleratorAttributes:  Updated attributes for the accelerator.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FlowLogsS3Prefix</i> The prefix for the location in the Amazon S3 bucket for the flow logs. Attribute is required if <code>FlowLogsEnabled</code> is <code>true</code>. If you don’t specify a prefix, the flow logs are stored in the root of the bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FlowLogsS3Bucket</i> The name of the Amazon S3 bucket for the flow logs. Attribute is required if <code>FlowLogsEnabled</code> is <code>true</code>. The bucket must exist and have a bucket policy that grants AWS Global Accelerator permission to write to the bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FlowLogsEnabled</i> Indicates whether flow logs are enabled. The default value is false. If the value is true, <code>FlowLogsS3Bucket</code> and <code>FlowLogsS3Prefix</code> must be specified. For more information, see <a href="https://docs.aws.amazon.com/global-accelerator/latest/dg/monitoring-global-accelerator.flow-logs.html">Flow Logs</a> in the <i>AWS Global Accelerator Developer Guide</i>.<br/>        """
        pass

    def update_endpoint_group(self, EndpointGroupArn: str, EndpointConfigurations: List[EndpointConfiguration]=None, TrafficDialPercentage: TrafficDialPercentage=None, HealthCheckPort: int=None, HealthCheckProtocol: str=None, HealthCheckPath: str=None, HealthCheckIntervalSeconds: int=None, ThresholdCount: int=None) -> UpdateEndpointGroupResponse:
        """Update an endpoint group. To see an AWS CLI example of updating an endpoint group, scroll down to <b>Example</b>.

        :param self:
        :param ThresholdCount:  The number of consecutive health checks required to set the state of a healthy endpoint to unhealthy, or to set an unhealthy endpoint to healthy. The default value is 3.<br/>HealthCheckIntervalSeconds:  The time—10 seconds or 30 seconds—between each health check for an endpoint. The default value is 30.<br/>HealthCheckPath:  If the protocol is HTTP/S, then this specifies the path that is the destination for health check targets. The default value is slash (/).<br/>HealthCheckProtocol:  The protocol that AWS Global Accelerator uses to check the health of endpoints that are part of this endpoint group. The default value is TCP.<br/>HealthCheckPort:  The port that AWS Global Accelerator uses to check the health of endpoints that are part of this endpoint group. The default port is the listener port that this endpoint group is associated with. If the listener port is a list of ports, Global Accelerator uses the first port in the list.<br/>TrafficDialPercentage:  The percentage of traffic to send to an AWS Region. Additional traffic is distributed to other endpoint groups for this listener.  Use this action to increase (dial up) or decrease (dial down) traffic to a specific Region. The percentage is applied to the traffic that would otherwise have been routed to the Region based on optimal routing. The default value is 100.<br/>EndpointConfigurations:  The list of endpoint objects.<br/>EndpointGroupArn:  The Amazon Resource Name (ARN) of the endpoint group.<br/>
        :return: EndpointGroup:  The information about the endpoint group that was updated.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThresholdCount</i> The number of consecutive health checks required to set the state of a healthy endpoint to unhealthy, or to set an unhealthy endpoint to healthy. The default value is 3.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HealthCheckIntervalSeconds</i> The time—10 seconds or 30 seconds—between health checks for each endpoint. The default value is 30.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HealthCheckPath</i> If the protocol is HTTP/S, then this value provides the ping path that Global Accelerator uses for the destination on the endpoints for health checks. The default is slash (/).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HealthCheckProtocol</i> The protocol that Global Accelerator uses to perform health checks on endpoints that are part of this endpoint group. The default value is TCP.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HealthCheckPort</i> The port that Global Accelerator uses to perform health checks on endpoints that are part of this endpoint group.  The default port is the port for the listener that this endpoint group is associated with. If the listener port is a list, Global Accelerator uses the first specified port in the list of ports.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrafficDialPercentage</i> The percentage of traffic to send to an AWS Region. Additional traffic is distributed to other endpoint groups for this listener.  Use this action to increase (dial up) or decrease (dial down) traffic to a specific Region. The percentage is applied to the traffic that would otherwise have been routed to the Region based on optimal routing. The default value is 100.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointDescriptions</i> The list of endpoint objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointGroupRegion</i> The AWS Region that this endpoint group belongs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointGroupArn</i> The Amazon Resource Name (ARN) of the endpoint group.<br/>        """
        pass

    def update_listener(self, ListenerArn: str, PortRanges: List[PortRange]=None, Protocol: str=None, ClientAffinity: str=None) -> UpdateListenerResponse:
        """Update a listener.

        :param self:
        :param ClientAffinity:  Client affinity lets you direct all requests from a user to the same endpoint, if you have stateful applications, regardless of the port and protocol of the client request. Clienty affinity gives you control over whether to always route each client to the same specific endpoint. AWS Global Accelerator uses a consistent-flow hashing algorithm to choose the optimal endpoint for a connection. If client affinity is <code>NONE</code>, Global Accelerator uses the "five-tuple" (5-tuple) properties—source IP address, source port, destination IP address, destination port, and protocol—to select the hash value, and then chooses the best endpoint. However, with this setting, if someone uses different ports to connect to Global Accelerator, their connections might not be always routed to the same endpoint because the hash value changes.  If you want a given client to always be routed to the same endpoint, set client affinity to <code>SOURCE_IP</code> instead. When you use the <code>SOURCE_IP</code> setting, Global Accelerator uses the "two-tuple" (2-tuple) properties— source (client) IP address and destination IP address—to select the hash value. The default value is <code>NONE</code>.<br/>Protocol:  The updated protocol for the connections from clients to the accelerator.<br/>PortRanges:  The updated list of port ranges for the connections from clients to the accelerator.<br/>ListenerArn:  The Amazon Resource Name (ARN) of the listener to update.<br/>
        :return: Listener:  Information for the updated listener.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientAffinity</i> Client affinity lets you direct all requests from a user to the same endpoint, if you have stateful applications, regardless of the port and protocol of the client request. Clienty affinity gives you control over whether to always route each client to the same specific endpoint. AWS Global Accelerator uses a consistent-flow hashing algorithm to choose the optimal endpoint for a connection. If client affinity is <code>NONE</code>, Global Accelerator uses the "five-tuple" (5-tuple) properties—source IP address, source port, destination IP address, destination port, and protocol—to select the hash value, and then chooses the best endpoint. However, with this setting, if someone uses different ports to connect to Global Accelerator, their connections might not be always routed to the same endpoint because the hash value changes.  If you want a given client to always be routed to the same endpoint, set client affinity to <code>SOURCE_IP</code> instead. When you use the <code>SOURCE_IP</code> setting, Global Accelerator uses the "two-tuple" (2-tuple) properties— source (client) IP address and destination IP address—to select the hash value. The default value is <code>NONE</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Protocol</i> The protocol for the connections from clients to the accelerator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PortRanges</i> The list of port ranges for the connections from clients to the accelerator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ListenerArn</i> The Amazon Resource Name (ARN) of the listener.<br/>        """
        pass



class Glue(BaseClient):
    """<fullname>AWS Glue</fullname> Defines the public endpoint for the AWS Glue service."""


    class GetJobsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_jobs` for valid parameters"""
            pass
    class GetPartitionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_partitions` for valid parameters"""
            pass
    class GetDatabasesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_databases` for valid parameters"""
            pass
    class GetClassifiersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_classifiers` for valid parameters"""
            pass
    class GetTableVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_table_versions` for valid parameters"""
            pass
    class GetCrawlersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_crawlers` for valid parameters"""
            pass
    class GetDevEndpointsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_dev_endpoints` for valid parameters"""
            pass
    class GetJobRunsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_job_runs` for valid parameters"""
            pass
    class GetTriggersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_triggers` for valid parameters"""
            pass
    class GetTablesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_tables` for valid parameters"""
            pass
    class GetUserDefinedFunctionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_user_defined_functions` for valid parameters"""
            pass
    class GetCrawlerMetricsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_crawler_metrics` for valid parameters"""
            pass
    class GetConnectionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_connections` for valid parameters"""
            pass
    class GetSecurityConfigurationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_security_configurations` for valid parameters"""
            pass


    class BatchCreatePartitionRequest(Mapping):
        pass
    
    class BatchCreatePartitionResponse(Mapping):
        pass
    
    class PartitionInput(Mapping):
        pass
    
    class BatchDeleteConnectionRequest(Mapping):
        pass
    
    class BatchDeleteConnectionResponse(Mapping):
        pass
    
    class NameString(object):
        pass
    
    class BatchDeletePartitionRequest(Mapping):
        pass
    
    class BatchDeletePartitionResponse(Mapping):
        pass
    
    class PartitionValueList(Mapping):
        pass
    
    class BatchDeleteTableRequest(Mapping):
        pass
    
    class BatchDeleteTableResponse(Mapping):
        pass
    
    class BatchDeleteTableVersionRequest(Mapping):
        pass
    
    class BatchDeleteTableVersionResponse(Mapping):
        pass
    
    class VersionString(object):
        pass
    
    class BatchGetCrawlersRequest(Mapping):
        pass
    
    class BatchGetCrawlersResponse(Mapping):
        pass
    
    class BatchGetDevEndpointsRequest(Mapping):
        pass
    
    class BatchGetDevEndpointsResponse(Mapping):
        pass
    
    class GenericString(object):
        pass
    
    class BatchGetJobsRequest(Mapping):
        pass
    
    class BatchGetJobsResponse(Mapping):
        pass
    
    class BatchGetPartitionRequest(Mapping):
        pass
    
    class BatchGetPartitionResponse(Mapping):
        pass
    
    class BatchGetTriggersRequest(Mapping):
        pass
    
    class BatchGetTriggersResponse(Mapping):
        pass
    
    class BatchGetWorkflowsRequest(Mapping):
        pass
    
    class BatchGetWorkflowsResponse(Mapping):
        pass
    
    class NullableBoolean(object):
        pass
    
    class BatchStopJobRunRequest(Mapping):
        pass
    
    class BatchStopJobRunResponse(Mapping):
        pass
    
    class IdString(object):
        pass
    
    class CreateClassifierRequest(Mapping):
        pass
    
    class CreateClassifierResponse(Mapping):
        pass
    
    class CreateGrokClassifierRequest(Mapping):
        pass
    
    class CreateXMLClassifierRequest(Mapping):
        pass
    
    class CreateJsonClassifierRequest(Mapping):
        pass
    
    class CreateCsvClassifierRequest(Mapping):
        pass
    
    class CreateConnectionRequest(Mapping):
        pass
    
    class CreateConnectionResponse(Mapping):
        pass
    
    class ConnectionInput(Mapping):
        pass
    
    class CreateCrawlerRequest(Mapping):
        pass
    
    class CreateCrawlerResponse(Mapping):
        pass
    
    class CrawlerTargets(Mapping):
        pass
    
    class SchemaChangePolicy(Mapping):
        pass
    
    class TagsMap(object):
        pass
    
    class CreateDatabaseRequest(Mapping):
        pass
    
    class CreateDatabaseResponse(Mapping):
        pass
    
    class DatabaseInput(Mapping):
        pass
    
    class CreateDevEndpointRequest(Mapping):
        pass
    
    class CreateDevEndpointResponse(Mapping):
        pass
    
    class MapValue(object):
        pass
    
    class CreateJobRequest(Mapping):
        pass
    
    class CreateJobResponse(Mapping):
        pass
    
    class ExecutionProperty(Mapping):
        pass
    
    class JobCommand(Mapping):
        pass
    
    class GenericMap(object):
        pass
    
    class ConnectionsList(Mapping):
        pass
    
    class NullableDouble(object):
        pass
    
    class NotificationProperty(Mapping):
        pass
    
    class CreatePartitionRequest(Mapping):
        pass
    
    class CreatePartitionResponse(Mapping):
        pass
    
    class CreateScriptRequest(Mapping):
        pass
    
    class CreateScriptResponse(Mapping):
        pass
    
    class CodeGenNode(Mapping):
        pass
    
    class CodeGenEdge(Mapping):
        pass
    
    class CreateSecurityConfigurationRequest(Mapping):
        pass
    
    class CreateSecurityConfigurationResponse(Mapping):
        pass
    
    class EncryptionConfiguration(Mapping):
        pass
    
    class CreateTableRequest(Mapping):
        pass
    
    class CreateTableResponse(Mapping):
        pass
    
    class TableInput(Mapping):
        pass
    
    class CreateTriggerRequest(Mapping):
        pass
    
    class CreateTriggerResponse(Mapping):
        pass
    
    class Predicate(Mapping):
        pass
    
    class Action(Mapping):
        pass
    
    class BooleanValue(object):
        pass
    
    class CreateUserDefinedFunctionRequest(Mapping):
        pass
    
    class CreateUserDefinedFunctionResponse(Mapping):
        pass
    
    class UserDefinedFunctionInput(Mapping):
        pass
    
    class CreateWorkflowRequest(Mapping):
        pass
    
    class CreateWorkflowResponse(Mapping):
        pass
    
    class WorkflowRunProperties(object):
        pass
    
    class DeleteClassifierRequest(Mapping):
        pass
    
    class DeleteClassifierResponse(Mapping):
        pass
    
    class DeleteConnectionRequest(Mapping):
        pass
    
    class DeleteConnectionResponse(Mapping):
        pass
    
    class DeleteCrawlerRequest(Mapping):
        pass
    
    class DeleteCrawlerResponse(Mapping):
        pass
    
    class DeleteDatabaseRequest(Mapping):
        pass
    
    class DeleteDatabaseResponse(Mapping):
        pass
    
    class DeleteDevEndpointRequest(Mapping):
        pass
    
    class DeleteDevEndpointResponse(Mapping):
        pass
    
    class DeleteJobRequest(Mapping):
        pass
    
    class DeleteJobResponse(Mapping):
        pass
    
    class DeletePartitionRequest(Mapping):
        pass
    
    class DeletePartitionResponse(Mapping):
        pass
    
    class ValueString(object):
        pass
    
    class DeleteResourcePolicyRequest(Mapping):
        pass
    
    class DeleteResourcePolicyResponse(Mapping):
        pass
    
    class DeleteSecurityConfigurationRequest(Mapping):
        pass
    
    class DeleteSecurityConfigurationResponse(Mapping):
        pass
    
    class DeleteTableRequest(Mapping):
        pass
    
    class DeleteTableResponse(Mapping):
        pass
    
    class DeleteTableVersionRequest(Mapping):
        pass
    
    class DeleteTableVersionResponse(Mapping):
        pass
    
    class DeleteTriggerRequest(Mapping):
        pass
    
    class DeleteTriggerResponse(Mapping):
        pass
    
    class DeleteUserDefinedFunctionRequest(Mapping):
        pass
    
    class DeleteUserDefinedFunctionResponse(Mapping):
        pass
    
    class DeleteWorkflowRequest(Mapping):
        pass
    
    class DeleteWorkflowResponse(Mapping):
        pass
    
    class GetCatalogImportStatusRequest(Mapping):
        pass
    
    class GetCatalogImportStatusResponse(Mapping):
        pass
    
    class GetClassifierRequest(Mapping):
        pass
    
    class GetClassifierResponse(Mapping):
        pass
    
    class GetClassifiersRequest(Mapping):
        pass
    
    class GetClassifiersResponse(Mapping):
        pass
    
    class GetConnectionRequest(Mapping):
        pass
    
    class GetConnectionResponse(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class GetConnectionsRequest(Mapping):
        pass
    
    class GetConnectionsResponse(Mapping):
        pass
    
    class GetConnectionsFilter(Mapping):
        pass
    
    class GetCrawlerRequest(Mapping):
        pass
    
    class GetCrawlerResponse(Mapping):
        pass
    
    class GetCrawlerMetricsRequest(Mapping):
        pass
    
    class GetCrawlerMetricsResponse(Mapping):
        pass
    
    class GetCrawlersRequest(Mapping):
        pass
    
    class GetCrawlersResponse(Mapping):
        pass
    
    class GetDataCatalogEncryptionSettingsRequest(Mapping):
        pass
    
    class GetDataCatalogEncryptionSettingsResponse(Mapping):
        pass
    
    class GetDatabaseRequest(Mapping):
        pass
    
    class GetDatabaseResponse(Mapping):
        pass
    
    class GetDatabasesRequest(Mapping):
        pass
    
    class GetDatabasesResponse(Mapping):
        pass
    
    class GetDataflowGraphRequest(Mapping):
        pass
    
    class GetDataflowGraphResponse(Mapping):
        pass
    
    class GetDevEndpointRequest(Mapping):
        pass
    
    class GetDevEndpointResponse(Mapping):
        pass
    
    class GetDevEndpointsRequest(Mapping):
        pass
    
    class GetDevEndpointsResponse(Mapping):
        pass
    
    class GetJobRequest(Mapping):
        pass
    
    class GetJobResponse(Mapping):
        pass
    
    class GetJobRunRequest(Mapping):
        pass
    
    class GetJobRunResponse(Mapping):
        pass
    
    class GetJobRunsRequest(Mapping):
        pass
    
    class GetJobRunsResponse(Mapping):
        pass
    
    class GetJobsRequest(Mapping):
        pass
    
    class GetJobsResponse(Mapping):
        pass
    
    class GetMappingRequest(Mapping):
        pass
    
    class GetMappingResponse(Mapping):
        pass
    
    class CatalogEntry(Mapping):
        pass
    
    class Location(Mapping):
        pass
    
    class GetPartitionRequest(Mapping):
        pass
    
    class GetPartitionResponse(Mapping):
        pass
    
    class GetPartitionsRequest(Mapping):
        pass
    
    class GetPartitionsResponse(Mapping):
        pass
    
    class Segment(Mapping):
        pass
    
    class GetPlanRequest(Mapping):
        pass
    
    class GetPlanResponse(Mapping):
        pass
    
    class MappingEntry(Mapping):
        pass
    
    class GetResourcePolicyRequest(Mapping):
        pass
    
    class GetResourcePolicyResponse(Mapping):
        pass
    
    class GetSecurityConfigurationRequest(Mapping):
        pass
    
    class GetSecurityConfigurationResponse(Mapping):
        pass
    
    class GetSecurityConfigurationsRequest(Mapping):
        pass
    
    class GetSecurityConfigurationsResponse(Mapping):
        pass
    
    class GetTableRequest(Mapping):
        pass
    
    class GetTableResponse(Mapping):
        pass
    
    class GetTableVersionRequest(Mapping):
        pass
    
    class GetTableVersionResponse(Mapping):
        pass
    
    class GetTableVersionsRequest(Mapping):
        pass
    
    class GetTableVersionsResponse(Mapping):
        pass
    
    class GetTablesRequest(Mapping):
        pass
    
    class GetTablesResponse(Mapping):
        pass
    
    class GetTagsRequest(Mapping):
        pass
    
    class GetTagsResponse(Mapping):
        pass
    
    class GetTriggerRequest(Mapping):
        pass
    
    class GetTriggerResponse(Mapping):
        pass
    
    class GetTriggersRequest(Mapping):
        pass
    
    class GetTriggersResponse(Mapping):
        pass
    
    class GetUserDefinedFunctionRequest(Mapping):
        pass
    
    class GetUserDefinedFunctionResponse(Mapping):
        pass
    
    class GetUserDefinedFunctionsRequest(Mapping):
        pass
    
    class GetUserDefinedFunctionsResponse(Mapping):
        pass
    
    class GetWorkflowRequest(Mapping):
        pass
    
    class GetWorkflowResponse(Mapping):
        pass
    
    class GetWorkflowRunRequest(Mapping):
        pass
    
    class GetWorkflowRunResponse(Mapping):
        pass
    
    class GetWorkflowRunPropertiesRequest(Mapping):
        pass
    
    class GetWorkflowRunPropertiesResponse(Mapping):
        pass
    
    class GetWorkflowRunsRequest(Mapping):
        pass
    
    class GetWorkflowRunsResponse(Mapping):
        pass
    
    class ImportCatalogToGlueRequest(Mapping):
        pass
    
    class ImportCatalogToGlueResponse(Mapping):
        pass
    
    class ListCrawlersRequest(Mapping):
        pass
    
    class ListCrawlersResponse(Mapping):
        pass
    
    class ListDevEndpointsRequest(Mapping):
        pass
    
    class ListDevEndpointsResponse(Mapping):
        pass
    
    class ListJobsRequest(Mapping):
        pass
    
    class ListJobsResponse(Mapping):
        pass
    
    class ListTriggersRequest(Mapping):
        pass
    
    class ListTriggersResponse(Mapping):
        pass
    
    class ListWorkflowsRequest(Mapping):
        pass
    
    class ListWorkflowsResponse(Mapping):
        pass
    
    class PutDataCatalogEncryptionSettingsRequest(Mapping):
        pass
    
    class PutDataCatalogEncryptionSettingsResponse(Mapping):
        pass
    
    class DataCatalogEncryptionSettings(Mapping):
        pass
    
    class PutResourcePolicyRequest(Mapping):
        pass
    
    class PutResourcePolicyResponse(Mapping):
        pass
    
    class PutWorkflowRunPropertiesRequest(Mapping):
        pass
    
    class PutWorkflowRunPropertiesResponse(Mapping):
        pass
    
    class ResetJobBookmarkRequest(Mapping):
        pass
    
    class ResetJobBookmarkResponse(Mapping):
        pass
    
    class StartCrawlerRequest(Mapping):
        pass
    
    class StartCrawlerResponse(Mapping):
        pass
    
    class StartCrawlerScheduleRequest(Mapping):
        pass
    
    class StartCrawlerScheduleResponse(Mapping):
        pass
    
    class StartJobRunRequest(Mapping):
        pass
    
    class StartJobRunResponse(Mapping):
        pass
    
    class StartTriggerRequest(Mapping):
        pass
    
    class StartTriggerResponse(Mapping):
        pass
    
    class StartWorkflowRunRequest(Mapping):
        pass
    
    class StartWorkflowRunResponse(Mapping):
        pass
    
    class StopCrawlerRequest(Mapping):
        pass
    
    class StopCrawlerResponse(Mapping):
        pass
    
    class StopCrawlerScheduleRequest(Mapping):
        pass
    
    class StopCrawlerScheduleResponse(Mapping):
        pass
    
    class StopTriggerRequest(Mapping):
        pass
    
    class StopTriggerResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateClassifierRequest(Mapping):
        pass
    
    class UpdateClassifierResponse(Mapping):
        pass
    
    class UpdateGrokClassifierRequest(Mapping):
        pass
    
    class UpdateXMLClassifierRequest(Mapping):
        pass
    
    class UpdateJsonClassifierRequest(Mapping):
        pass
    
    class UpdateCsvClassifierRequest(Mapping):
        pass
    
    class UpdateConnectionRequest(Mapping):
        pass
    
    class UpdateConnectionResponse(Mapping):
        pass
    
    class UpdateCrawlerRequest(Mapping):
        pass
    
    class UpdateCrawlerResponse(Mapping):
        pass
    
    class UpdateCrawlerScheduleRequest(Mapping):
        pass
    
    class UpdateCrawlerScheduleResponse(Mapping):
        pass
    
    class UpdateDatabaseRequest(Mapping):
        pass
    
    class UpdateDatabaseResponse(Mapping):
        pass
    
    class UpdateDevEndpointRequest(Mapping):
        pass
    
    class UpdateDevEndpointResponse(Mapping):
        pass
    
    class DevEndpointCustomLibraries(Mapping):
        pass
    
    class UpdateJobRequest(Mapping):
        pass
    
    class UpdateJobResponse(Mapping):
        pass
    
    class JobUpdate(Mapping):
        pass
    
    class UpdatePartitionRequest(Mapping):
        pass
    
    class UpdatePartitionResponse(Mapping):
        pass
    
    class UpdateTableRequest(Mapping):
        pass
    
    class UpdateTableResponse(Mapping):
        pass
    
    class BooleanNullable(object):
        pass
    
    class UpdateTriggerRequest(Mapping):
        pass
    
    class UpdateTriggerResponse(Mapping):
        pass
    
    class TriggerUpdate(Mapping):
        pass
    
    class UpdateUserDefinedFunctionRequest(Mapping):
        pass
    
    class UpdateUserDefinedFunctionResponse(Mapping):
        pass
    
    class UpdateWorkflowRequest(Mapping):
        pass
    
    class UpdateWorkflowResponse(Mapping):
        pass
    
    def batch_create_partition(self, PartitionInputList: List[PartitionInput], TableName: str, DatabaseName: str, CatalogId: str=None) -> BatchCreatePartitionResponse:
        """Creates one or more partitions in a batch operation.

        :param self:
        :param PartitionInputList:  A list of <code>PartitionInput</code> structures that define the partitions to be created.<br/>TableName:  The name of the metadata table in which the partition is to be created.<br/>DatabaseName:  The name of the metadata database in which the partition is to be created.<br/>CatalogId:  The ID of the catalog in which the partion is to be created. Currently, this should be the AWS account ID.<br/>
        :return: Errors:  Errors encountered when trying to create the requested partitions.<br/>        """
        pass

    def batch_delete_connection(self, ConnectionNameList: List[NameString], CatalogId: str=None) -> BatchDeleteConnectionResponse:
        """Deletes a list of connection definitions from the Data Catalog.

        :param self:
        :param ConnectionNameList:  A list of names of the connections to delete.<br/>CatalogId:  The ID of the Data Catalog in which the connections reside. If none is provided, the AWS account ID is used by default.<br/>
        :return: Errors:  A map of the names of connections that were not successfully deleted to error details.<br/>Succeeded:  A list of names of the connection definitions that were successfully deleted.<br/>        """
        pass

    def batch_delete_partition(self, PartitionsToDelete: List[PartitionValueList], TableName: str, DatabaseName: str, CatalogId: str=None) -> BatchDeletePartitionResponse:
        """Deletes one or more partitions in a batch operation.

        :param self:
        :param PartitionsToDelete:  A list of <code>PartitionInput</code> structures that define the partitions to be deleted.<br/>TableName:  The name of the table where the partitions to be deleted is located.<br/>DatabaseName:  The name of the catalog database in which the table in question resides.<br/>CatalogId:  The ID of the Data Catalog where the partition to be deleted resides. If none is supplied, the AWS account ID is used by default.<br/>
        :return: Errors:  Errors encountered when trying to delete the requested partitions.<br/>        """
        pass

    def batch_delete_table(self, TablesToDelete: List[NameString], DatabaseName: str, CatalogId: str=None) -> BatchDeleteTableResponse:
        """Deletes multiple tables at once. <note> After completing this operation, you will no longer have access to the table versions and partitions that belong to the deleted table. AWS Glue deletes these "orphaned" resources asynchronously in a timely manner, at the discretion of the service. To ensure immediate deletion of all related resources, before calling <code>BatchDeleteTable</code>, use <code>DeleteTableVersion</code> or <code>BatchDeleteTableVersion</code>, and <code>DeletePartition</code> or <code>BatchDeletePartition</code>, to delete any resources that belong to the table. </note>

        :param self:
        :param TablesToDelete:  A list of the table to delete.<br/>DatabaseName:  The name of the catalog database where the tables to delete reside. For Hive compatibility, this name is entirely lowercase.<br/>CatalogId:  The ID of the Data Catalog where the table resides. If none is supplied, the AWS account ID is used by default.<br/>
        :return: Errors:  A list of errors encountered in attempting to delete the specified tables.<br/>        """
        pass

    def batch_delete_table_version(self, VersionIds: List[VersionString], TableName: str, DatabaseName: str, CatalogId: str=None) -> BatchDeleteTableVersionResponse:
        """Deletes a specified batch of versions of a table.

        :param self:
        :param VersionIds:  A list of the IDs of versions to be deleted. A <code>VersionId</code> is a string representation of an integer. Each version is incremented by 1.<br/>TableName:  The name of the table. For Hive compatibility, this name is entirely lowercase.<br/>DatabaseName:  The database in the catalog in which the table resides. For Hive compatibility, this name is entirely lowercase.<br/>CatalogId:  The ID of the Data Catalog where the tables reside. If none is supplied, the AWS account ID is used by default.<br/>
        :return: Errors:  A list of errors encountered while trying to delete the specified table versions.<br/>        """
        pass

    def batch_get_crawlers(self, CrawlerNames: List[NameString]) -> BatchGetCrawlersResponse:
        """Returns a list of resource metadata for a given list of crawler names. After calling the <code>ListCrawlers</code> operation, you can call this operation to access the data to which you have been granted permissions. This operation supports all IAM permissions, including permission conditions that uses tags.

        :param self:
        :param CrawlerNames:  A list of crawler names, which might be the names returned from the <code>ListCrawlers</code> operation.<br/>
        :return: CrawlersNotFound:  A list of names of crawlers that were not found.<br/>Crawlers:  A list of crawler definitions.<br/>        """
        pass

    def batch_get_dev_endpoints(self, DevEndpointNames: List[GenericString]) -> BatchGetDevEndpointsResponse:
        """Returns a list of resource metadata for a given list of DevEndpoint names. After calling the <code>ListDevEndpoints</code> operation, you can call this operation to access the data to which you have been granted permissions. This operation supports all IAM permissions, including permission conditions that uses tags.

        :param self:
        :param DevEndpointNames:  The list of DevEndpoint names, which may be the names returned from the <code>ListDevEndpoint</code> operation.<br/>
        :return: DevEndpointsNotFound:  A list of DevEndpoints not found.<br/>DevEndpoints:  A list of DevEndpoint definitions.<br/>        """
        pass

    def batch_get_jobs(self, JobNames: List[NameString]) -> BatchGetJobsResponse:
        """Returns a list of resource metadata for a given list of job names. After calling the <code>ListJobs</code> operation, you can call this operation to access the data to which you have been granted permissions. This operation supports all IAM permissions, including permission conditions that uses tags. 

        :param self:
        :param JobNames:  A list of job names, which might be the names returned from the <code>ListJobs</code> operation.<br/>
        :return: JobsNotFound:  A list of names of jobs not found.<br/>Jobs:  A list of job definitions.<br/>        """
        pass

    def batch_get_partition(self, PartitionsToGet: List[PartitionValueList], TableName: str, DatabaseName: str, CatalogId: str=None) -> BatchGetPartitionResponse:
        """Retrieves partitions in a batch request.

        :param self:
        :param PartitionsToGet:  A list of partition values identifying the partitions to retrieve.<br/>TableName:  The name of the partitions' table.<br/>DatabaseName:  The name of the catalog database where the partitions reside.<br/>CatalogId:  The ID of the Data Catalog where the partitions in question reside. If none is supplied, the AWS account ID is used by default.<br/>
        :return: UnprocessedKeys:  A list of the partition values in the request for which partions were not returned.<br/>Partitions:  A list of the requested partitions.<br/>        """
        pass

    def batch_get_triggers(self, TriggerNames: List[NameString]) -> BatchGetTriggersResponse:
        """Returns a list of resource metadata for a given list of trigger names. After calling the <code>ListTriggers</code> operation, you can call this operation to access the data to which you have been granted permissions. This operation supports all IAM permissions, including permission conditions that uses tags.

        :param self:
        :param TriggerNames:  A list of trigger names, which may be the names returned from the <code>ListTriggers</code> operation.<br/>
        :return: TriggersNotFound:  A list of names of triggers not found.<br/>Triggers:  A list of trigger definitions.<br/>        """
        pass

    def batch_get_workflows(self, Names: List[NameString], IncludeGraph: NullableBoolean=None) -> BatchGetWorkflowsResponse:
        """Returns a list of resource metadata for a given list of workflow names. After calling the <code>ListWorkflows</code> operation, you can call this operation to access the data to which you have been granted permissions. This operation supports all IAM permissions, including permission conditions that uses tags.

        :param self:
        :param IncludeGraph:  Specifies whether to include a graph when returning the workflow resource metadata.<br/>Names:  A list of workflow names, which may be the names returned from the <code>ListWorkflows</code> operation.<br/>
        :return: MissingWorkflows:  A list of names of workflows not found.<br/>Workflows:  A list of workflow resource metadata.<br/>        """
        pass

    def batch_stop_job_run(self, JobRunIds: List[IdString], JobName: str) -> BatchStopJobRunResponse:
        """Stops one or more job runs for a specified job definition.

        :param self:
        :param JobRunIds:  A list of the <code>JobRunIds</code> that should be stopped for that job definition.<br/>JobName:  The name of the job definition for which to stop job runs.<br/>
        :return: Errors:  A list of the errors that were encountered in trying to stop <code>JobRuns</code>, including the <code>JobRunId</code> for which each error was encountered and details about the error.<br/>SuccessfulSubmissions:  A list of the JobRuns that were successfully submitted for stopping.<br/>        """
        pass

    def create_classifier(self, GrokClassifier: CreateGrokClassifierRequest=None, XMLClassifier: CreateXMLClassifierRequest=None, JsonClassifier: CreateJsonClassifierRequest=None, CsvClassifier: CreateCsvClassifierRequest=None) -> CreateClassifierResponse:
        """Creates a classifier in the user's account. This can be a <code>GrokClassifier</code>, an <code>XMLClassifier</code>, a <code>JsonClassifier</code>, or a <code>CsvClassifier</code>, depending on which field of the request is present.

        :param self:
        :param CsvClassifier:  A <code>CsvClassifier</code> object specifying the classifier to create.:<br/>JsonClassifier:  A <code>JsonClassifier</code> object specifying the classifier to create.:<br/>XMLClassifier:  An <code>XMLClassifier</code> object specifying the classifier to create.:<br/>GrokClassifier:  A <code>GrokClassifier</code> object specifying the classifier to create.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomPatterns</i> Optional custom grok patterns used by this classifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GrokPattern</i> The grok pattern used by this classifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the new classifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Classification</i> An identifier of the data format that the classifier matches, such as Twitter, JSON, Omniture logs, Amazon CloudWatch Logs, and so on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RowTag</i> The XML tag designating the element that contains each record in an XML document being parsed. This can't identify a self-closing element (closed by <code>/&gt;</code>). An empty row element that contains only attributes can be parsed as long as it ends with a closing tag (for example, <code>&lt;row item_a="A" item_b="B"&gt;&lt;/row&gt;</code> is okay, but <code>&lt;row item_a="A" item_b="B" /&gt;</code> is not).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the classifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Classification</i> An identifier of the data format that the classifier matches.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JsonPath</i> A <code>JsonPath</code> string defining the JSON data for the classifier to classify. AWS Glue supports a subset of <code>JsonPath</code>, as described in <a href="https://docs.aws.amazon.com/glue/latest/dg/custom-classifier.html#custom-classifier-json">Writing JsonPath Custom Classifiers</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the classifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowSingleColumn</i> Enables the processing of files that contain only one column.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisableValueTrimming</i> Specifies not to trim values before identifying the type of column values. The default value is true.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Header</i> A list of strings representing column names.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContainsHeader</i> Indicates whether the CSV file contains a header.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QuoteSymbol</i> A custom symbol to denote what combines content into a single column value. Must be different from the column delimiter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Delimiter</i> A custom symbol to denote what separates each column entry in the row.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the classifier.<br/>
        :return:         """
        pass

    def create_connection(self, ConnectionInput: ConnectionInput, CatalogId: str=None) -> CreateConnectionResponse:
        """Creates a connection definition in the Data Catalog.

        :param self:
        :param ConnectionInput:  A <code>ConnectionInput</code> object defining the connection to create.:<br/>CatalogId:  The ID of the Data Catalog in which to create the connection. If none is provided, the AWS account ID is used by default.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhysicalConnectionRequirements</i> A map of physical connection requirements, such as virtual private cloud (VPC) and <code>SecurityGroup</code>, that are needed to successfully make this connection.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PhysicalConnectionRequirements</b>: A map of physical connection requirements, such as virtual private cloud (VPC) and <code>SecurityGroup</code>, that are needed to successfully make this connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectionProperties</i> These key-value pairs define parameters for the connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MatchCriteria</i> A list of criteria that can be used in selecting this connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectionType</i> The type of the connection. Currently, only JDBC is supported; SFTP is not supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The connection's Availability Zone. This field is redundant because the specified subnet implies the Availability Zone to be used. Currently the field must be populated, but it will be deprecated in the future.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIdList</i> The security group ID list used by the connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetId</i> The subnet ID used by the connection.<br/>
        :return:         """
        pass

    def create_crawler(self, Targets: CrawlerTargets, Role: str, Name: str, DatabaseName: str=None, Description: str=None, Schedule: str=None, Classifiers: List[NameString]=None, TablePrefix: str=None, SchemaChangePolicy: SchemaChangePolicy=None, Configuration: str=None, CrawlerSecurityConfiguration: str=None, Tags: TagsMap=None) -> CreateCrawlerResponse:
        """Creates a new crawler with specified targets, role, configuration, and optional schedule. At least one crawl target must be specified, in the <code>s3Targets</code> field, the <code>jdbcTargets</code> field, or the <code>DynamoDBTargets</code> field.

        :param self:
        :param Tags:  The tags to use with this crawler request. You can use tags to limit access to the crawler. For more information, see <a href="http://docs.aws.amazon.com/glue/latest/dg/monitor-tags.html">AWS Tags in AWS Glue</a>.<br/>CrawlerSecurityConfiguration:  The name of the <code>SecurityConfiguration</code> structure to be used by this crawler.<br/>Configuration:  The crawler configuration information. This versioned JSON string allows users to specify aspects of a crawler's behavior. For more information, see <a href="http://docs.aws.amazon.com/glue/latest/dg/crawler-configuration.html">Configuring a Crawler</a>.<br/>SchemaChangePolicy:  The policy for the crawler's update and deletion behavior.:<br/>TablePrefix:  The table prefix used for catalog tables that are created.<br/>Classifiers:  A list of custom classifiers that the user has registered. By default, all built-in classifiers are included in a crawl, but these custom classifiers always override the default classifiers for a given classification.<br/>Schedule:  A <code>cron</code> expression used to specify the schedule. For more information, see <a href="http://docs.aws.amazon.com/glue/latest/dg/monitor-data-warehouse-schedule.html">Time-Based Schedules for Jobs and Crawlers</a>. For example, to run something every day at 12:15 UTC, specify <code>cron(15 12 * * ? *)</code>.<br/>Targets:  A list of collection of targets to crawl.:<br/>Description:  A description of the new crawler.<br/>DatabaseName:  The AWS Glue database where results are written, such as: <code>arn:aws:daylight:us-east-1::database/sometable/*</code>.<br/>Role:  The IAM role or Amazon Resource Name (ARN) of an IAM role used by the new crawler to access customer resources.<br/>Name:  Name of the new crawler.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CatalogTargets</i> Specifies AWS Glue Data Catalog targets.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DynamoDBTargets</i> Specifies Amazon DynamoDB targets.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JdbcTargets</i> Specifies JDBC targets.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Targets</i> Specifies Amazon Simple Storage Service (Amazon S3) targets.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeleteBehavior</i> The deletion behavior when the crawler finds a deleted object.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateBehavior</i> The update behavior when the crawler finds a changed schema.<br/>
        :return:         """
        pass

    def create_database(self, DatabaseInput: DatabaseInput, CatalogId: str=None) -> CreateDatabaseResponse:
        """Creates a new database in a Data Catalog.

        :param self:
        :param DatabaseInput:  A <code>DatabaseInput</code> object defining the metadata database to create in the catalog.:<br/>CatalogId:  The ID of the Data Catalog in which to create the database. If none is supplied, the AWS account ID is used by default.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> Thes key-value pairs define parameters and properties of the database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LocationUri</i> The location of the database (for example, an HDFS path).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> Description of the database<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Name of the database. For Hive compatibility, this is folded to lowercase when it is stored.<br/>
        :return:         """
        pass

    def create_dev_endpoint(self, RoleArn: str, EndpointName: str, SecurityGroupIds: List[GenericString]=None, SubnetId: str=None, PublicKey: str=None, PublicKeys: List[GenericString]=None, NumberOfNodes: int=None, ExtraPythonLibsS3Path: str=None, ExtraJarsS3Path: str=None, SecurityConfiguration: str=None, Tags: TagsMap=None, Arguments: MapValue=None) -> CreateDevEndpointResponse:
        """Creates a new DevEndpoint.

        :param self:
        :param Arguments:  A map of arguments used to configure the DevEndpoint.<br/>Tags:  The tags to use with this DevEndpoint. You may use tags to limit access to the DevEndpoint. For more information about tags in AWS Glue, see <a href="http://docs.aws.amazon.com/glue/latest/dg/monitor-tags.html">AWS Tags in AWS Glue</a> in the developer guide.<br/>SecurityConfiguration:  The name of the SecurityConfiguration structure to be used with this DevEndpoint.<br/>ExtraJarsS3Path:  Path to one or more Java Jars in an S3 bucket that should be loaded in your DevEndpoint.<br/>ExtraPythonLibsS3Path:  Path(s) to one or more Python libraries in an S3 bucket that should be loaded in your DevEndpoint. Multiple values must be complete paths separated by a comma. Please note that only pure Python libraries can currently be used on a DevEndpoint. Libraries that rely on C extensions, such as the <a href="http://pandas.pydata.org/">pandas</a> Python data analysis library, are not yet supported.<br/>NumberOfNodes:  The number of AWS Glue Data Processing Units (DPUs) to allocate to this DevEndpoint.<br/>PublicKeys:  A list of public keys to be used by the DevEndpoints for authentication. The use of this attribute is preferred over a single public key because the public keys allow you to have a different private key per client. <note> If you previously created an endpoint with a public key, you must remove that key to be able to set a list of public keys: call the <code>UpdateDevEndpoint</code> API with the public key content in the <code>deletePublicKeys</code> attribute, and the list of new keys in the <code>addPublicKeys</code> attribute. </note><br/>PublicKey:  The public key to be used by this DevEndpoint for authentication. This attribute is provided for backward compatibility, as the recommended attribute to use is public keys.<br/>SubnetId:  The subnet ID for the new DevEndpoint to use.<br/>SecurityGroupIds:  Security group IDs for the security groups to be used by the new DevEndpoint.<br/>RoleArn:  The IAM role for the DevEndpoint.<br/>EndpointName:  The name to be assigned to the new DevEndpoint.<br/>
        :return: Arguments:  The map of arguments used to configure this DevEndpoint.<br/>CreatedTimestamp:  The point in time at which this DevEndpoint was created.<br/>SecurityConfiguration:  The name of the SecurityConfiguration structure being used with this DevEndpoint.<br/>FailureReason:  The reason for a current failure in this DevEndpoint.<br/>ExtraJarsS3Path:  Path to one or more Java Jars in an S3 bucket that will be loaded in your DevEndpoint.<br/>ExtraPythonLibsS3Path:  Path(s) to one or more Python libraries in an S3 bucket that will be loaded in your DevEndpoint.<br/>VpcId:  The ID of the VPC used by this DevEndpoint.<br/>AvailabilityZone:  The AWS availability zone where this DevEndpoint is located.<br/>NumberOfNodes:  The number of AWS Glue Data Processing Units (DPUs) allocated to this DevEndpoint.<br/>ZeppelinRemoteSparkInterpreterPort:  The Apache Zeppelin port for the remote Apache Spark interpreter.<br/>YarnEndpointAddress:  The address of the YARN endpoint used by this DevEndpoint.<br/>RoleArn:  The AWS ARN of the role assigned to the new DevEndpoint.<br/>SubnetId:  The subnet ID assigned to the new DevEndpoint.<br/>SecurityGroupIds:  The security groups assigned to the new DevEndpoint.<br/>Status:  The current status of the new DevEndpoint.<br/>EndpointName:  The name assigned to the new DevEndpoint.<br/>        """
        pass

    def create_job(self, Command: JobCommand, Role: str, Name: str, Description: str=None, LogUri: str=None, ExecutionProperty: ExecutionProperty=None, DefaultArguments: GenericMap=None, Connections: ConnectionsList=None, MaxRetries: int=None, AllocatedCapacity: int=None, Timeout: int=None, MaxCapacity: NullableDouble=None, SecurityConfiguration: str=None, Tags: TagsMap=None, NotificationProperty: NotificationProperty=None, NumberOfWorkers: int=None, WorkerType: str=None) -> CreateJobResponse:
        """Creates a new job definition.

        :param self:
        :param WorkerType:  The type of predefined worker that is allocated when a job runs. Accepts a value of Standard, G.1X, or G.2X. <ul> <li> For the <code>Standard</code> worker type, each worker provides 4 vCPU, 16 GB of memory and a 50GB disk, and 2 executors per worker. </li> <li> For the <code>G.1X</code> worker type, each worker maps to 1 DPU (4 vCPU, 16 GB of memory, 64 GB disk), and provides 1 executor per worker. We recommend this worker type for memory-intensive jobs. </li> <li> For the <code>G.2X</code> worker type, each worker maps to 2 DPU (8 vCPU, 32 GB of memory, 128 GB disk), and provides 1 executor per worker. We recommend this worker type for memory-intensive jobs. </li> </ul><br/>NumberOfWorkers:  The number of workers of a defined <code>workerType</code> that are allocated when a job runs. The maximum number of workers you can define are 299 for <code>G.1X</code>, and 149 for <code>G.2X</code>. <br/>NotificationProperty:  Specifies configuration properties of a job notification.:<br/>Tags:  The tags to use with this job. You may use tags to limit access to the job. For more information about tags in AWS Glue, see <a href="https://docs.aws.amazon.com/glue/latest/dg/monitor-tags.html">AWS Tags in AWS Glue</a> in the developer guide.<br/>SecurityConfiguration:  The name of the <code>SecurityConfiguration</code> structure to be used with this job.<br/>MaxCapacity:  The number of AWS Glue data processing units (DPUs) that can be allocated when this job runs. A DPU is a relative measure of processing power that consists of 4 vCPUs of compute capacity and 16 GB of memory. For more information, see the <a href="https://aws.amazon.com/glue/pricing/">AWS Glue pricing page</a>. Do not set <code>Max Capacity</code> if using <code>WorkerType</code> and <code>NumberOfWorkers</code>. The value that can be allocated for <code>MaxCapacity</code> depends on whether you are running a Python shell job or an Apache Spark ETL job: <ul> <li> When you specify a Python shell job (<code>JobCommand.Name</code>="pythonshell"), you can allocate either 0.0625 or 1 DPU. The default is 0.0625 DPU. </li> <li> When you specify an Apache Spark ETL job (<code>JobCommand.Name</code>="glueetl"), you can allocate from 2 to 100 DPUs. The default is 10 DPUs. This job type cannot have a fractional DPU allocation. </li> </ul><br/>Timeout:  The job timeout in minutes. This is the maximum time that a job run can consume resources before it is terminated and enters <code>TIMEOUT</code> status. The default is 2,880 minutes (48 hours).<br/>AllocatedCapacity:  This parameter is deprecated. Use <code>MaxCapacity</code> instead. The number of AWS Glue data processing units (DPUs) to allocate to this Job. You can allocate from 2 to 100 DPUs; the default is 10. A DPU is a relative measure of processing power that consists of 4 vCPUs of compute capacity and 16 GB of memory. For more information, see the <a href="https://aws.amazon.com/glue/pricing/">AWS Glue pricing page</a>.<br/>MaxRetries:  The maximum number of times to retry this job if it fails.<br/>Connections:  The connections used for this job.:<br/>DefaultArguments:  The default arguments for this job. You can specify arguments here that your own job-execution script consumes, as well as arguments that AWS Glue itself consumes. For information about how to specify and consume your own Job arguments, see the <a href="https://docs.aws.amazon.com/glue/latest/dg/aws-glue-programming-python-calling.html">Calling AWS Glue APIs in Python</a> topic in the developer guide. For information about the key-value pairs that AWS Glue consumes to set up your job, see the <a href="https://docs.aws.amazon.com/glue/latest/dg/aws-glue-programming-etl-glue-arguments.html">Special Parameters Used by AWS Glue</a> topic in the developer guide.<br/>Command:  The <code>JobCommand</code> that executes this job.:<br/>ExecutionProperty:  An <code>ExecutionProperty</code> specifying the maximum number of concurrent runs allowed for this job.:<br/>Role:  The name or Amazon Resource Name (ARN) of the IAM role associated with this job.<br/>LogUri:  This field is reserved for future use.<br/>Description:  Description of the job being defined.<br/>Name:  The name you assign to this job definition. It must be unique in your account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxConcurrentRuns</i> The maximum number of concurrent runs allowed for the job. The default is 1. An error is returned when this threshold is reached. The maximum value you can specify is controlled by a service limit.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PythonVersion</i> The Python version being used to execute a Python shell job. Allowed values are 2 or 3.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ScriptLocation</i> Specifies the Amazon Simple Storage Service (Amazon S3) path to a script that executes a job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the job command. For an Apache Spark ETL job, this must be <code>glueetl</code>. For a Python shell job, it must be <code>pythonshell</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Connections</i> A list of connections used by the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotifyDelayAfter</i> After a job run starts, the number of minutes to wait before sending a job run delay notification.<br/>
        :return: Name:  The unique name that was provided for this job definition.<br/>        """
        pass

    def create_partition(self, PartitionInput: PartitionInput, TableName: str, DatabaseName: str, CatalogId: str=None) -> CreatePartitionResponse:
        """Creates a new partition.

        :param self:
        :param PartitionInput:  A <code>PartitionInput</code> structure defining the partition to be created.:<br/>TableName:  The name of the metadata table in which the partition is to be created.<br/>DatabaseName:  The name of the metadata database in which the partition is to be created.<br/>CatalogId:  The ID of the catalog in which the partion is to be created. Currently, this should be the AWS account ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastAnalyzedTime</i> The last time at which column statistics were computed for this partition.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> These key-value pairs define partition parameters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageDescriptor</i> Provides information about the physical location where the partition is stored.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StorageDescriptor</b>: Provides information about the physical location where the partition is stored.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastAccessTime</i> The last time at which the partition was accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Values</i> The values of the partition. Although this parameter is not required by the SDK, you must specify this parameter for a valid input.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfBuckets</i> Must be specified if the table contains any dimension columns.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Compressed</i> True if the data in the table is compressed, or False if not.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputFormat</i> The output format: <code>SequenceFileOutputFormat</code> (binary), or <code>IgnoreKeyTextOutputFormat</code>, or a custom format.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InputFormat</i> The input format: <code>SequenceFileInputFormat</code> (binary), or <code>TextInputFormat</code>, or a custom format.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Location</i> The physical location of the table. By default this takes the form of the warehouse location, followed by the database location in the warehouse, followed by the table name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Columns</i> A list of the <code>Columns</code> in the table.<br/>Serialization/deserialization (SerDe) information.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SerdeInfo</b>: Serialization/deserialization (SerDe) information.
        :return:         """
        pass

    def create_script(self, DagNodes: List[CodeGenNode]=None, DagEdges: List[CodeGenEdge]=None, Language: str=None) -> CreateScriptResponse:
        """Transforms a directed acyclic graph (DAG) into code.

        :param self:
        :param Language:  The programming language of the resulting code from the DAG.<br/>DagEdges:  A list of the edges in the DAG.<br/>DagNodes:  A list of the nodes in the DAG.<br/>
        :return: ScalaCode:  The Scala code generated from the DAG.<br/>PythonScript:  The Python script generated from the DAG.<br/>        """
        pass

    def create_security_configuration(self, EncryptionConfiguration: EncryptionConfiguration, Name: str) -> CreateSecurityConfigurationResponse:
        """Creates a new security configuration.

        :param self:
        :param EncryptionConfiguration:  The encryption configuration for the new security configuration.:<br/>Name:  The name for the new security configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobBookmarksEncryption</i> The encryption configuration for Job Bookmarks.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>JobBookmarksEncryption</b>: The encryption configuration for Job Bookmarks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchEncryption</i> The encryption configuration for CloudWatch.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CloudWatchEncryption</b>: The encryption configuration for CloudWatch.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Encryption</i> The encryption configuration for S3 data.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyArn</i> The AWS ARN of the KMS key to be used to encrypt the data.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchEncryptionMode</i> The encryption mode to use for CloudWatch data.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyArn</i> The AWS ARN of the KMS key to be used to encrypt the data.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>JobBookmarksEncryptionMode</i> The encryption mode to use for Job bookmarks data.<br/>
        :return: CreatedTimestamp:  The time at which the new security configuration was created.<br/>Name:  The name assigned to the new security configuration.<br/>        """
        pass

    def create_table(self, TableInput: TableInput, DatabaseName: str, CatalogId: str=None) -> CreateTableResponse:
        """Creates a new table definition in the Data Catalog.

        :param self:
        :param TableInput:  The <code>TableInput</code> object that defines the metadata table to create in the catalog.:<br/>DatabaseName:  The catalog database in which to create the new table. For Hive compatibility, this name is entirely lowercase.<br/>CatalogId:  The ID of the Data Catalog in which to create the <code>Table</code>. If none is supplied, the AWS account ID is used by default.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> These key-value pairs define properties associated with the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableType</i> The type of this table (<code>EXTERNAL_TABLE</code>, <code>VIRTUAL_VIEW</code>, etc.).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ViewExpandedText</i> If the table is a view, the expanded text of the view; otherwise <code>null</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ViewOriginalText</i> If the table is a view, the original text of the view; otherwise <code>null</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PartitionKeys</i> A list of columns by which the table is partitioned. Only primitive types are supported as partition keys. When creating a table used by Athena, and you do not specify any <code>partitionKeys</code>, you must at least set the value of <code>partitionKeys</code> to an empty list. For example:  <code>"PartitionKeys": []</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageDescriptor</i> A storage descriptor containing information about the physical storage of this table.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StorageDescriptor</b>: A storage descriptor containing information about the physical storage of this table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Retention</i> Retention time for this table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastAnalyzedTime</i> Last time column statistics were computed for this table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastAccessTime</i> Last time the table was accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Owner</i> Owner of the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> Description of the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Name of the table. For Hive compatibility, this is folded to lowercase when it is stored.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfBuckets</i> Must be specified if the table contains any dimension columns.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Compressed</i> True if the data in the table is compressed, or False if not.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputFormat</i> The output format: <code>SequenceFileOutputFormat</code> (binary), or <code>IgnoreKeyTextOutputFormat</code>, or a custom format.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InputFormat</i> The input format: <code>SequenceFileInputFormat</code> (binary), or <code>TextInputFormat</code>, or a custom format.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Location</i> The physical location of the table. By default this takes the form of the warehouse location, followed by the database location in the warehouse, followed by the table name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Columns</i> A list of the <code>Columns</code> in the table.<br/>Serialization/deserialization (SerDe) information.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SerdeInfo</b>: Serialization/deserialization (SerDe) information.
        :return:         """
        pass

    def create_trigger(self, Actions: List[Action], Type: str, Name: str, WorkflowName: str=None, Schedule: str=None, Predicate: Predicate=None, Description: str=None, StartOnCreation: BooleanValue=None, Tags: TagsMap=None) -> CreateTriggerResponse:
        """Creates a new trigger.

        :param self:
        :param Tags:  The tags to use with this trigger. You may use tags to limit access to the trigger. For more information about tags in AWS Glue, see <a href="https://docs.aws.amazon.com/glue/latest/dg/monitor-tags.html">AWS Tags in AWS Glue</a> in the developer guide. <br/>StartOnCreation:  Set to <code>true</code> to start <code>SCHEDULED</code> and <code>CONDITIONAL</code> triggers when created. True is not supported for <code>ON_DEMAND</code> triggers.<br/>Description:  A description of the new trigger.<br/>Actions:  The actions initiated by this trigger when it fires.<br/>Predicate:  A predicate to specify when the new trigger should fire. This field is required when the trigger type is <code>CONDITIONAL</code>.:<br/>Schedule:  A <code>cron</code> expression used to specify the schedule (see <a href="https://docs.aws.amazon.com/glue/latest/dg/monitor-data-warehouse-schedule.html">Time-Based Schedules for Jobs and Crawlers</a>. For example, to run something every day at 12:15 UTC, you would specify: <code>cron(15 12 * * ? *)</code>. This field is required when the trigger type is SCHEDULED.<br/>Type:  The type of the new trigger.<br/>WorkflowName:  The name of the workflow associated with the trigger.<br/>Name:  The name of the trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Conditions</i> A list of the conditions that determine when the trigger will fire.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Logical</i> An optional field if only one condition is listed. If multiple conditions are listed, then this field is required.<br/>
        :return: Name:  The name of the trigger.<br/>        """
        pass

    def create_user_defined_function(self, FunctionInput: UserDefinedFunctionInput, DatabaseName: str, CatalogId: str=None) -> CreateUserDefinedFunctionResponse:
        """Creates a new function definition in the Data Catalog.

        :param self:
        :param FunctionInput:  A <code>FunctionInput</code> object that defines the function to create in the Data Catalog.:<br/>DatabaseName:  The name of the catalog database in which to create the function.<br/>CatalogId:  The ID of the Data Catalog in which to create the function. If none is supplied, the AWS account ID is used by default.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceUris</i> The resource URIs for the function.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerType</i> The owner type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerName</i> The owner of the function.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClassName</i> The Java class that contains the function code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FunctionName</i> The name of the function.<br/>
        :return:         """
        pass

    def create_workflow(self, Name: str, Description: str=None, DefaultRunProperties: WorkflowRunProperties=None, Tags: TagsMap=None) -> CreateWorkflowResponse:
        """Creates a new workflow.

        :param self:
        :param Tags:  The tags to be used with this workflow.<br/>DefaultRunProperties:  A collection of properties to be used as part of each execution of the workflow.<br/>Description:  A description of the workflow.<br/>Name:  The name to be assigned to the workflow. It should be unique within your account.<br/>
        :return: Name:  The name of the workflow which was provided as part of the request.<br/>        """
        pass

    def delete_classifier(self, Name: str) -> DeleteClassifierResponse:
        """Removes a classifier from the Data Catalog.

        :param self:
        :param Name:  Name of the classifier to remove.<br/>
        :return:         """
        pass

    def delete_connection(self, ConnectionName: str, CatalogId: str=None) -> DeleteConnectionResponse:
        """Deletes a connection from the Data Catalog.

        :param self:
        :param ConnectionName:  The name of the connection to delete.<br/>CatalogId:  The ID of the Data Catalog in which the connection resides. If none is provided, the AWS account ID is used by default.<br/>
        :return:         """
        pass

    def delete_crawler(self, Name: str) -> DeleteCrawlerResponse:
        """Removes a specified crawler from the AWS Glue Data Catalog, unless the crawler state is <code>RUNNING</code>.

        :param self:
        :param Name:  The name of the crawler to remove.<br/>
        :return:         """
        pass

    def delete_database(self, Name: str, CatalogId: str=None) -> DeleteDatabaseResponse:
        """Removes a specified Database from a Data Catalog. <note> After completing this operation, you will no longer have access to the tables (and all table versions and partitions that might belong to the tables) and the user-defined functions in the deleted database. AWS Glue deletes these "orphaned" resources asynchronously in a timely manner, at the discretion of the service. To ensure immediate deletion of all related resources, before calling <code>DeleteDatabase</code>, use <code>DeleteTableVersion</code> or <code>BatchDeleteTableVersion</code>, <code>DeletePartition</code> or <code>BatchDeletePartition</code>, <code>DeleteUserDefinedFunction</code>, and <code>DeleteTable</code> or <code>BatchDeleteTable</code>, to delete any resources that belong to the database. </note>

        :param self:
        :param Name:  The name of the Database to delete. For Hive compatibility, this must be all lowercase.<br/>CatalogId:  The ID of the Data Catalog in which the database resides. If none is supplied, the AWS account ID is used by default.<br/>
        :return:         """
        pass

    def delete_dev_endpoint(self, EndpointName: str) -> DeleteDevEndpointResponse:
        """Deletes a specified DevEndpoint.

        :param self:
        :param EndpointName:  The name of the DevEndpoint.<br/>
        :return:         """
        pass

    def delete_job(self, JobName: str) -> DeleteJobResponse:
        """Deletes a specified job definition. If the job definition is not found, no exception is thrown.

        :param self:
        :param JobName:  The name of the job definition to delete.<br/>
        :return: JobName:  The name of the job definition that was deleted.<br/>        """
        pass

    def delete_partition(self, PartitionValues: List[ValueString], TableName: str, DatabaseName: str, CatalogId: str=None) -> DeletePartitionResponse:
        """Deletes a specified partition.

        :param self:
        :param PartitionValues:  The values that define the partition.<br/>TableName:  The name of the table where the partition to be deleted is located.<br/>DatabaseName:  The name of the catalog database in which the table in question resides.<br/>CatalogId:  The ID of the Data Catalog where the partition to be deleted resides. If none is supplied, the AWS account ID is used by default.<br/>
        :return:         """
        pass

    def delete_resource_policy(self, PolicyHashCondition: str=None) -> DeleteResourcePolicyResponse:
        """Deletes a specified policy.

        :param self:
        :param PolicyHashCondition:  The hash value returned when this policy was set.<br/>
        :return:         """
        pass

    def delete_security_configuration(self, Name: str) -> DeleteSecurityConfigurationResponse:
        """Deletes a specified security configuration.

        :param self:
        :param Name:  The name of the security configuration to delete.<br/>
        :return:         """
        pass

    def delete_table(self, Name: str, DatabaseName: str, CatalogId: str=None) -> DeleteTableResponse:
        """Removes a table definition from the Data Catalog. <note> After completing this operation, you will no longer have access to the table versions and partitions that belong to the deleted table. AWS Glue deletes these "orphaned" resources asynchronously in a timely manner, at the discretion of the service. To ensure immediate deletion of all related resources, before calling <code>DeleteTable</code>, use <code>DeleteTableVersion</code> or <code>BatchDeleteTableVersion</code>, and <code>DeletePartition</code> or <code>BatchDeletePartition</code>, to delete any resources that belong to the table. </note>

        :param self:
        :param Name:  The name of the table to be deleted. For Hive compatibility, this name is entirely lowercase.<br/>DatabaseName:  The name of the catalog database in which the table resides. For Hive compatibility, this name is entirely lowercase.<br/>CatalogId:  The ID of the Data Catalog where the table resides. If none is supplied, the AWS account ID is used by default.<br/>
        :return:         """
        pass

    def delete_table_version(self, VersionId: str, TableName: str, DatabaseName: str, CatalogId: str=None) -> DeleteTableVersionResponse:
        """Deletes a specified version of a table.

        :param self:
        :param VersionId:  The ID of the table version to be deleted. A <code>VersionID</code> is a string representation of an integer. Each version is incremented by 1.<br/>TableName:  The name of the table. For Hive compatibility, this name is entirely lowercase.<br/>DatabaseName:  The database in the catalog in which the table resides. For Hive compatibility, this name is entirely lowercase.<br/>CatalogId:  The ID of the Data Catalog where the tables reside. If none is supplied, the AWS account ID is used by default.<br/>
        :return:         """
        pass

    def delete_trigger(self, Name: str) -> DeleteTriggerResponse:
        """Deletes a specified trigger. If the trigger is not found, no exception is thrown.

        :param self:
        :param Name:  The name of the trigger to delete.<br/>
        :return: Name:  The name of the trigger that was deleted.<br/>        """
        pass

    def delete_user_defined_function(self, FunctionName: str, DatabaseName: str, CatalogId: str=None) -> DeleteUserDefinedFunctionResponse:
        """Deletes an existing function definition from the Data Catalog.

        :param self:
        :param FunctionName:  The name of the function definition to be deleted.<br/>DatabaseName:  The name of the catalog database where the function is located.<br/>CatalogId:  The ID of the Data Catalog where the function to be deleted is located. If none is supplied, the AWS account ID is used by default.<br/>
        :return:         """
        pass

    def delete_workflow(self, Name: str) -> DeleteWorkflowResponse:
        """Deletes a workflow.

        :param self:
        :param Name:  Name of the workflow to be deleted.<br/>
        :return: Name:  Name of the workflow specified in input.<br/>        """
        pass

    def get_catalog_import_status(self, CatalogId: str=None) -> GetCatalogImportStatusResponse:
        """Retrieves the status of a migration operation.

        :param self:
        :param CatalogId:  The ID of the catalog to migrate. Currently, this should be the AWS account ID.<br/>
        :return: ImportStatus:  The status of the specified catalog migration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImportedBy</i> The name of the person who initiated the migration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImportTime</i> The time that the migration was started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImportCompleted</i> True if the migration has completed, or False otherwise.<br/>        """
        pass

    def get_classifier(self, Name: str) -> GetClassifierResponse:
        """Retrieve a classifier by name.

        :param self:
        :param Name:  Name of the classifier to retrieve.<br/>
        :return: Classifier:  The requested classifier.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CsvClassifier</i> A classifier for comma-separated values (CSV).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CsvClassifier</b>: A classifier for comma-separated values (CSV).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JsonClassifier</i> A classifier for JSON content.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>JsonClassifier</b>: A classifier for JSON content.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>XMLClassifier</i> A classifier for XML content.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>XMLClassifier</b>: A classifier for XML content.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GrokClassifier</i> A classifier that uses <code>grok</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>GrokClassifier</b>: A classifier that uses <code>grok</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomPatterns</i> Optional custom grok patterns defined by this classifier. For more information, see custom patterns in <a href="http://docs.aws.amazon.com/glue/latest/dg/custom-classifier.html">Writing Custom Classifiers</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>GrokPattern</i> The grok pattern applied to a data store by this classifier. For more information, see built-in patterns in <a href="http://docs.aws.amazon.com/glue/latest/dg/custom-classifier.html">Writing Custom Classifiers</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version of this classifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdated</i> The time that this classifier was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The time that this classifier was registered.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Classification</i> An identifier of the data format that the classifier matches, such as Twitter, JSON, Omniture logs, and so on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the classifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RowTag</i> The XML tag designating the element that contains each record in an XML document being parsed. This can't identify a self-closing element (closed by <code>/&gt;</code>). An empty row element that contains only attributes can be parsed as long as it ends with a closing tag (for example, <code>&lt;row item_a="A" item_b="B"&gt;&lt;/row&gt;</code> is okay, but <code>&lt;row item_a="A" item_b="B" /&gt;</code> is not).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version of this classifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdated</i> The time that this classifier was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The time that this classifier was registered.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Classification</i> An identifier of the data format that the classifier matches.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the classifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>JsonPath</i> A <code>JsonPath</code> string defining the JSON data for the classifier to classify. AWS Glue supports a subset of <code>JsonPath</code>, as described in <a href="https://docs.aws.amazon.com/glue/latest/dg/custom-classifier.html#custom-classifier-json">Writing JsonPath Custom Classifiers</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version of this classifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdated</i> The time that this classifier was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The time that this classifier was registered.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the classifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowSingleColumn</i> Enables the processing of files that contain only one column.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DisableValueTrimming</i> Specifies not to trim values before identifying the type of column values. The default value is <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Header</i> A list of strings representing column names.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ContainsHeader</i> Indicates whether the CSV file contains a header.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>QuoteSymbol</i> A custom symbol to denote what combines content into a single column value. It must be different from the column delimiter.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Delimiter</i> A custom symbol to denote what separates each column entry in the row.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version of this classifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdated</i> The time that this classifier was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The time that this classifier was registered.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the classifier.<br/>        """
        pass

    def get_classifiers(self, MaxResults: int=None, NextToken: str=None) -> GetClassifiersResponse:
        """Lists all classifier objects in the Data Catalog.

        :param self:
        :param NextToken:  An optional continuation token.<br/>MaxResults:  The size of the list to return (optional).<br/>
        :return: NextToken:  A continuation token.<br/>Classifiers:  The requested list of classifier objects.<br/>        """
        pass

    def get_connection(self, Name: str, CatalogId: str=None, HidePassword: Boolean=None) -> GetConnectionResponse:
        """Retrieves a connection definition from the Data Catalog.

        :param self:
        :param HidePassword:  Allows you to retrieve the connection metadata without returning the password. For instance, the AWS Glue console uses this flag to retrieve the connection, and does not display the password. Set this parameter when the caller might not have permission to use the AWS KMS key to decrypt the password, but does have permission to access the rest of the connection properties.<br/>Name:  The name of the connection definition to retrieve.<br/>CatalogId:  The ID of the Data Catalog in which the connection resides. If none is provided, the AWS account ID is used by default.<br/>
        :return: Connection:  The requested connection definition.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdatedBy</i> The user, group, or role that last updated this connection definition.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdatedTime</i> The last time that this connection definition was updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The time that this connection definition was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhysicalConnectionRequirements</i> A map of physical connection requirements, such as virtual private cloud (VPC) and <code>SecurityGroup</code>, that are needed to make this connection successfully.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PhysicalConnectionRequirements</b>: A map of physical connection requirements, such as virtual private cloud (VPC) and <code>SecurityGroup</code>, that are needed to make this connection successfully.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectionProperties</i> These key-value pairs define parameters for the connection: <ul> <li>  <code>HOST</code> - The host URI: either the fully qualified domain name (FQDN) or the IPv4 address of the database host. </li> <li>  <code>PORT</code> - The port number, between 1024 and 65535, of the port on which the database host is listening for database connections. </li> <li>  <code>USER_NAME</code> - The name under which to log in to the database. The value string for <code>USER_NAME</code> is "<code>USERNAME</code>". </li> <li>  <code>PASSWORD</code> - A password, if one is used, for the user name. </li> <li>  <code>ENCRYPTED_PASSWORD</code> - When you enable connection password protection by setting <code>ConnectionPasswordEncryption</code> in the Data Catalog encryption settings, this field stores the encrypted password. </li> <li>  <code>JDBC_DRIVER_JAR_URI</code> - The Amazon S3 path of the JAR file that contains the JDBC driver to use. </li> <li>  <code>JDBC_DRIVER_CLASS_NAME</code> - The class name of the JDBC driver to use. </li> <li>  <code>JDBC_ENGINE</code> - The name of the JDBC engine to use. </li> <li>  <code>JDBC_ENGINE_VERSION</code> - The version of the JDBC engine to use. </li> <li>  <code>CONFIG_FILES</code> - (Reserved for future use). </li> <li>  <code>INSTANCE_ID</code> - The instance ID to use. </li> <li>  <code>JDBC_CONNECTION_URL</code> - The URL for the JDBC connection. </li> <li>  <code>JDBC_ENFORCE_SSL</code> - A Boolean string (true, false) specifying whether Secure Sockets Layer (SSL) with hostname matching will be enforced for the JDBC connection on the client. The default is false. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MatchCriteria</i> A list of criteria that can be used in selecting this connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectionType</i> The type of the connection. Currently, only JDBC is supported; SFTP is not supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the connection definition.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The connection's Availability Zone. This field is redundant because the specified subnet implies the Availability Zone to be used. Currently the field must be populated, but it will be deprecated in the future.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIdList</i> The security group ID list used by the connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetId</i> The subnet ID used by the connection.<br/>        """
        pass

    def get_connections(self, CatalogId: str=None, Filter: GetConnectionsFilter=None, HidePassword: Boolean=None, NextToken: str=None, MaxResults: int=None) -> GetConnectionsResponse:
        """Retrieves a list of connection definitions from the Data Catalog.

        :param self:
        :param MaxResults:  The maximum number of connections to return in one response.<br/>NextToken:  A continuation token, if this is a continuation call.<br/>HidePassword:  Allows you to retrieve the connection metadata without returning the password. For instance, the AWS Glue console uses this flag to retrieve the connection, and does not display the password. Set this parameter when the caller might not have permission to use the AWS KMS key to decrypt the password, but does have permission to access the rest of the connection properties.<br/>Filter:  A filter that controls which connections will be returned.:<br/>CatalogId:  The ID of the Data Catalog in which the connections reside. If none is provided, the AWS account ID is used by default.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectionType</i> The type of connections to return. Currently, only JDBC is supported; SFTP is not supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MatchCriteria</i> A criteria string that must match the criteria recorded in the connection definition for that connection definition to be returned.<br/>
        :return: NextToken:  A continuation token, if the list of connections returned does not include the last of the filtered connections.<br/>ConnectionList:  A list of requested connection definitions.<br/>        """
        pass

    def get_crawler(self, Name: str) -> GetCrawlerResponse:
        """Retrieves metadata for a specified crawler.

        :param self:
        :param Name:  The name of the crawler to retrieve metadata for.<br/>
        :return: Crawler:  The metadata for the specified crawler.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CrawlerSecurityConfiguration</i> The name of the <code>SecurityConfiguration</code> structure to be used by this crawler.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Configuration</i> Crawler configuration information. This versioned JSON string allows users to specify aspects of a crawler's behavior. For more information, see <a href="http://docs.aws.amazon.com/glue/latest/dg/crawler-configuration.html">Configuring a Crawler</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version of the crawler.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastCrawl</i> The status of the last crawl, and potentially error information if an error occurred.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LastCrawl</b>: The status of the last crawl, and potentially error information if an error occurred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdated</i> The time that the crawler was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The time that the crawler was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CrawlElapsedTime</i> If the crawler is running, contains the total time elapsed since the last crawl began.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Schedule</i> For scheduled crawlers, the schedule when the crawler runs.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Schedule</b>: For scheduled crawlers, the schedule when the crawler runs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TablePrefix</i> The prefix added to the names of tables that are created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> Indicates whether the crawler is running, or whether a run is pending.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SchemaChangePolicy</i> The policy that specifies update and delete behaviors for the crawler.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SchemaChangePolicy</b>: The policy that specifies update and delete behaviors for the crawler.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Classifiers</i> A list of UTF-8 strings that specify the custom classifiers that are associated with the crawler.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of the crawler.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> The name of the database in which the crawler's output is stored.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Targets</i> A collection of targets to crawl.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Targets</b>: A collection of targets to crawl.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Role</i> The Amazon Resource Name (ARN) of an IAM role that's used to access customer resources, such as Amazon Simple Storage Service (Amazon S3) data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the crawler.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CatalogTargets</i> Specifies AWS Glue Data Catalog targets.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DynamoDBTargets</i> Specifies Amazon DynamoDB targets.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>JdbcTargets</i> Specifies JDBC targets.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Targets</i> Specifies Amazon Simple Storage Service (Amazon S3) targets.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DeleteBehavior</i> The deletion behavior when the crawler finds a deleted object.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateBehavior</i> The update behavior when the crawler finds a changed schema.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ScheduleExpression</i> A <code>cron</code> expression used to specify the schedule. For more information, see <a href="http://docs.aws.amazon.com/glue/latest/dg/monitor-data-warehouse-schedule.html">Time-Based Schedules for Jobs and Crawlers</a>. For example, to run something every day at 12:15 UTC, specify <code>cron(15 12 * * ? *)</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTime</i> The time at which the crawl started.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MessagePrefix</i> The prefix for a message about this crawl.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LogStream</i> The log stream for the last crawl.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LogGroup</i> The log group for the last crawl.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorMessage</i> If an error occurred, the error information about the last crawl.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Status of the last crawl.<br/>        """
        pass

    def get_crawler_metrics(self, CrawlerNameList: List[NameString]=None, MaxResults: int=None, NextToken: str=None) -> GetCrawlerMetricsResponse:
        """Retrieves metrics about specified crawlers.

        :param self:
        :param NextToken:  A continuation token, if this is a continuation call.<br/>MaxResults:  The maximum size of a list to return.<br/>CrawlerNameList:  A list of the names of crawlers about which to retrieve metrics.<br/>
        :return: NextToken:  A continuation token, if the returned list does not contain the last metric available.<br/>CrawlerMetricsList:  A list of metrics for the specified crawler.<br/>        """
        pass

    def get_crawlers(self, MaxResults: int=None, NextToken: str=None) -> GetCrawlersResponse:
        """Retrieves metadata for all crawlers defined in the customer account.

        :param self:
        :param NextToken:  A continuation token, if this is a continuation request.<br/>MaxResults:  The number of crawlers to return on each call.<br/>
        :return: NextToken:  A continuation token, if the returned list has not reached the end of those defined in this customer account.<br/>Crawlers:  A list of crawler metadata.<br/>        """
        pass

    def get_data_catalog_encryption_settings(self, CatalogId: str=None) -> GetDataCatalogEncryptionSettingsResponse:
        """Retrieves the security configuration for a specified catalog.

        :param self:
        :param CatalogId:  The ID of the Data Catalog for which to retrieve the security configuration. If none is provided, the AWS account ID is used by default.<br/>
        :return: DataCatalogEncryptionSettings:  The requested security configuration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectionPasswordEncryption</i> When connection password protection is enabled, the Data Catalog uses a customer-provided key to encrypt the password as part of <code>CreateConnection</code> or <code>UpdateConnection</code> and store it in the <code>ENCRYPTED_PASSWORD</code> field in the connection properties. You can enable catalog encryption or only password encryption.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ConnectionPasswordEncryption</b>: When connection password protection is enabled, the Data Catalog uses a customer-provided key to encrypt the password as part of <code>CreateConnection</code> or <code>UpdateConnection</code> and store it in the <code>ENCRYPTED_PASSWORD</code> field in the connection properties. You can enable catalog encryption or only password encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionAtRest</i> Specifies the encryption-at-rest configuration for the Data Catalog.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncryptionAtRest</b>: Specifies the encryption-at-rest configuration for the Data Catalog.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SseAwsKmsKeyId</i> The ID of the AWS KMS key to use for encryption at rest.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CatalogEncryptionMode</i> The encryption-at-rest mode for encrypting Data Catalog data.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AwsKmsKeyId</i> An AWS KMS key that is used to encrypt the connection password.  If connection password protection is enabled, the caller of <code>CreateConnection</code> and <code>UpdateConnection</code> needs at least <code>kms:Encrypt</code> permission on the specified AWS KMS key, to encrypt passwords before storing them in the Data Catalog.  You can set the decrypt permission to enable or restrict access on the password key according to your security requirements.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReturnConnectionPasswordEncrypted</i> When the <code>ReturnConnectionPasswordEncrypted</code> flag is set to "true", passwords remain encrypted in the responses of <code>GetConnection</code> and <code>GetConnections</code>. This encryption takes effect independently from catalog encryption. <br/>        """
        pass

    def get_database(self, Name: str, CatalogId: str=None) -> GetDatabaseResponse:
        """Retrieves the definition of a specified database.

        :param self:
        :param Name:  The name of the database to retrieve. For Hive compatibility, this should be all lowercase.<br/>CatalogId:  The ID of the Data Catalog in which the database resides. If none is supplied, the AWS account ID is used by default.<br/>
        :return: Database:  The definition of the specified database in the catalog.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateTime</i> The time at which the metadata database was created in the catalog.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> These key-value pairs define parameters and properties of the database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LocationUri</i> The location of the database (for example, an HDFS path).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> Description of the database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Name of the database. For Hive compatibility, this is folded to lowercase when it is stored.<br/>        """
        pass

    def get_databases(self, CatalogId: str=None, NextToken: str=None, MaxResults: int=None) -> GetDatabasesResponse:
        """Retrieves all Databases defined in a given Data Catalog.

        :param self:
        :param MaxResults:  The maximum number of databases to return in one response.<br/>NextToken:  A continuation token, if this is a continuation call.<br/>CatalogId:  The ID of the Data Catalog from which to retrieve <code>Databases</code>. If none is supplied, the AWS account ID is used by default.<br/>
        :return: NextToken:  A continuation token for paginating the returned list of tokens, returned if the current segment of the list is not the last.<br/>DatabaseList:  A list of <code>Database</code> objects from the specified catalog.<br/>        """
        pass

    def get_dataflow_graph(self, PythonScript: str=None) -> GetDataflowGraphResponse:
        """Transforms a Python script into a directed acyclic graph (DAG). 

        :param self:
        :param PythonScript:  The Python script to transform.<br/>
        :return: DagEdges:  A list of the edges in the resulting DAG.<br/>DagNodes:  A list of the nodes in the resulting DAG.<br/>        """
        pass

    def get_dev_endpoint(self, EndpointName: str) -> GetDevEndpointResponse:
        """Retrieves information about a specified DevEndpoint. <note> When you create a development endpoint in a virtual private cloud (VPC), AWS Glue returns only a private IP address, and the public IP address field is not populated. When you create a non-VPC development endpoint, AWS Glue returns only a public IP address. </note>

        :param self:
        :param EndpointName:  Name of the DevEndpoint for which to retrieve information.<br/>
        :return: DevEndpoint:  A DevEndpoint definition.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arguments</i> A map of arguments used to configure the DevEndpoint. Note that currently, we only support "--enable-glue-datacatalog": "" as a valid argument.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityConfiguration</i> The name of the SecurityConfiguration structure to be used with this DevEndpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PublicKeys</i> A list of public keys to be used by the DevEndpoints for authentication. The use of this attribute is preferred over a single public key because the public keys allow you to have a different private key per client. <note> If you previously created an endpoint with a public key, you must remove that key to be able to set a list of public keys: call the <code>UpdateDevEndpoint</code> API with the public key content in the <code>deletePublicKeys</code> attribute, and the list of new keys in the <code>addPublicKeys</code> attribute. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PublicKey</i> The public key to be used by this DevEndpoint for authentication. This attribute is provided for backward compatibility, as the recommended attribute to use is public keys.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedTimestamp</i> The point in time at which this DevEndpoint was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The point in time at which this DevEndpoint was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdateStatus</i> The status of the last update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureReason</i> The reason for a current failure in this DevEndpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExtraJarsS3Path</i> Path to one or more Java Jars in an S3 bucket that should be loaded in your DevEndpoint. Please note that only pure Java/Scala libraries can currently be used on a DevEndpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExtraPythonLibsS3Path</i> Path(s) to one or more Python libraries in an S3 bucket that should be loaded in your DevEndpoint. Multiple values must be complete paths separated by a comma. Please note that only pure Python libraries can currently be used on a DevEndpoint. Libraries that rely on C extensions, such as the <a href="http://pandas.pydata.org/">pandas</a> Python data analysis library, are not yet supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the virtual private cloud (VPC) used by this DevEndpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The AWS availability zone where this DevEndpoint is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The number of AWS Glue Data Processing Units (DPUs) allocated to this DevEndpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current status of this DevEndpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PublicAddress</i> The public IP address used by this DevEndpoint. The PublicAddress field is present only when you create a non-VPC (virtual private cloud) DevEndpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ZeppelinRemoteSparkInterpreterPort</i> The Apache Zeppelin port for the remote Apache Spark interpreter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrivateAddress</i> A private IP address to access the DevEndpoint within a VPC, if the DevEndpoint is created within one. The PrivateAddress field is present only when you create the DevEndpoint within your virtual private cloud (VPC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>YarnEndpointAddress</i> The YARN endpoint address used by this DevEndpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetId</i> The subnet ID for this DevEndpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> A list of security group identifiers used in this DevEndpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The AWS ARN of the IAM role used in this DevEndpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointName</i> The name of the DevEndpoint.<br/>        """
        pass

    def get_dev_endpoints(self, MaxResults: int=None, NextToken: str=None) -> GetDevEndpointsResponse:
        """Retrieves all the DevEndpoints in this AWS account. <note> When you create a development endpoint in a virtual private cloud (VPC), AWS Glue returns only a private IP address and the public IP address field is not populated. When you create a non-VPC development endpoint, AWS Glue returns only a public IP address. </note>

        :param self:
        :param NextToken:  A continuation token, if this is a continuation call.<br/>MaxResults:  The maximum size of information to return.<br/>
        :return: NextToken:  A continuation token, if not all DevEndpoint definitions have yet been returned.<br/>DevEndpoints:  A list of DevEndpoint definitions.<br/>        """
        pass

    def get_job(self, JobName: str) -> GetJobResponse:
        """Retrieves an existing job definition.

        :param self:
        :param JobName:  The name of the job definition to retrieve.<br/>
        :return: Job:  The requested job definition.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationProperty</i> Specifies configuration properties of a job notification.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NotificationProperty</b>: Specifies configuration properties of a job notification.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityConfiguration</i> The name of the <code>SecurityConfiguration</code> structure to be used with this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfWorkers</i> The number of workers of a defined <code>workerType</code> that are allocated when a job runs. The maximum number of workers you can define are 299 for <code>G.1X</code>, and 149 for <code>G.2X</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WorkerType</i> The type of predefined worker that is allocated when a job runs. Accepts a value of Standard, G.1X, or G.2X. <ul> <li> For the <code>Standard</code> worker type, each worker provides 4 vCPU, 16 GB of memory and a 50GB disk, and 2 executors per worker. </li> <li> For the <code>G.1X</code> worker type, each worker maps to 1 DPU (4 vCPU, 16 GB of memory, 64 GB disk), and provides 1 executor per worker. We recommend this worker type for memory-intensive jobs. </li> <li> For the <code>G.2X</code> worker type, each worker maps to 2 DPU (8 vCPU, 32 GB of memory, 128 GB disk), and provides 1 executor per worker. We recommend this worker type for memory-intensive jobs. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxCapacity</i> The number of AWS Glue data processing units (DPUs) that can be allocated when this job runs. A DPU is a relative measure of processing power that consists of 4 vCPUs of compute capacity and 16 GB of memory. For more information, see the <a href="https://aws.amazon.com/glue/pricing/">AWS Glue pricing page</a>. Do not set <code>Max Capacity</code> if using <code>WorkerType</code> and <code>NumberOfWorkers</code>. The value that can be allocated for <code>MaxCapacity</code> depends on whether you are running a Python shell job or an Apache Spark ETL job: <ul> <li> When you specify a Python shell job (<code>JobCommand.Name</code>="pythonshell"), you can allocate either 0.0625 or 1 DPU. The default is 0.0625 DPU. </li> <li> When you specify an Apache Spark ETL job (<code>JobCommand.Name</code>="glueetl"), you can allocate from 2 to 100 DPUs. The default is 10 DPUs. This job type cannot have a fractional DPU allocation. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timeout</i> The job timeout in minutes. This is the maximum time that a job run can consume resources before it is terminated and enters <code>TIMEOUT</code> status. The default is 2,880 minutes (48 hours).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedCapacity</i> This field is deprecated. Use <code>MaxCapacity</code> instead. The number of AWS Glue data processing units (DPUs) allocated to runs of this job. You can allocate from 2 to 100 DPUs; the default is 10. A DPU is a relative measure of processing power that consists of 4 vCPUs of compute capacity and 16 GB of memory. For more information, see the <a href="https://aws.amazon.com/glue/pricing/">AWS Glue pricing page</a>. <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxRetries</i> The maximum number of times to retry this job after a JobRun fails.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Connections</i> The connections used for this job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Connections</b>: The connections used for this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultArguments</i> The default arguments for this job, specified as name-value pairs. You can specify arguments here that your own job-execution script consumes, as well as arguments that AWS Glue itself consumes. For information about how to specify and consume your own Job arguments, see the <a href="https://docs.aws.amazon.com/glue/latest/dg/aws-glue-programming-python-calling.html">Calling AWS Glue APIs in Python</a> topic in the developer guide. For information about the key-value pairs that AWS Glue consumes to set up your job, see the <a href="https://docs.aws.amazon.com/glue/latest/dg/aws-glue-programming-etl-glue-arguments.html">Special Parameters Used by AWS Glue</a> topic in the developer guide.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Command</i> The <code>JobCommand</code> that executes this job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Command</b>: The <code>JobCommand</code> that executes this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExecutionProperty</i> An <code>ExecutionProperty</code> specifying the maximum number of concurrent runs allowed for this job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ExecutionProperty</b>: An <code>ExecutionProperty</code> specifying the maximum number of concurrent runs allowed for this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedOn</i> The last point in time when this job definition was modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedOn</i> The time and date that this job definition was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Role</i> The name or Amazon Resource Name (ARN) of the IAM role associated with this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogUri</i> This field is reserved for future use.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name you assign to this job definition.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxConcurrentRuns</i> The maximum number of concurrent runs allowed for the job. The default is 1. An error is returned when this threshold is reached. The maximum value you can specify is controlled by a service limit.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PythonVersion</i> The Python version being used to execute a Python shell job. Allowed values are 2 or 3.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ScriptLocation</i> Specifies the Amazon Simple Storage Service (Amazon S3) path to a script that executes a job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the job command. For an Apache Spark ETL job, this must be <code>glueetl</code>. For a Python shell job, it must be <code>pythonshell</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Connections</i> A list of connections used by the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NotifyDelayAfter</i> After a job run starts, the number of minutes to wait before sending a job run delay notification.<br/>        """
        pass

    def get_job_run(self, RunId: str, JobName: str, PredecessorsIncluded: BooleanValue=None) -> GetJobRunResponse:
        """Retrieves the metadata for a given job run.

        :param self:
        :param PredecessorsIncluded:  True if a list of predecessor runs should be returned.<br/>RunId:  The ID of the job run.<br/>JobName:  Name of the job definition being run.<br/>
        :return: JobRun:  The requested job-run metadata.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationProperty</i> Specifies configuration properties of a job run notification.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NotificationProperty</b>: Specifies configuration properties of a job run notification.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogGroupName</i> The name of the log group for secure logging that can be server-side encrypted in Amazon CloudWatch using AWS KMS. This name can be <code>/aws-glue/jobs/</code>, in which case the default encryption is <code>NONE</code>. If you add a role name and <code>SecurityConfiguration</code> name (in other words, <code>/aws-glue/jobs-yourRoleName-yourSecurityConfigurationName/</code>), then that security configuration is used to encrypt the log group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityConfiguration</i> The name of the <code>SecurityConfiguration</code> structure to be used with this job run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfWorkers</i> The number of workers of a defined <code>workerType</code> that are allocated when a job runs. The maximum number of workers you can define are 299 for <code>G.1X</code>, and 149 for <code>G.2X</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WorkerType</i> The type of predefined worker that is allocated when a job runs. Accepts a value of Standard, G.1X, or G.2X. <ul> <li> For the <code>Standard</code> worker type, each worker provides 4 vCPU, 16 GB of memory and a 50GB disk, and 2 executors per worker. </li> <li> For the <code>G.1X</code> worker type, each worker provides 4 vCPU, 16 GB of memory and a 64GB disk, and 1 executor per worker. </li> <li> For the <code>G.2X</code> worker type, each worker provides 8 vCPU, 32 GB of memory and a 128GB disk, and 1 executor per worker. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxCapacity</i> The number of AWS Glue data processing units (DPUs) that can be allocated when this job runs. A DPU is a relative measure of processing power that consists of 4 vCPUs of compute capacity and 16 GB of memory. For more information, see the <a href="https://docs.aws.amazon.com/https:/aws.amazon.com/glue/pricing/">AWS Glue pricing page</a>. Do not set <code>Max Capacity</code> if using <code>WorkerType</code> and <code>NumberOfWorkers</code>. The value that can be allocated for <code>MaxCapacity</code> depends on whether you are running a Python shell job or an Apache Spark ETL job: <ul> <li> When you specify a Python shell job (<code>JobCommand.Name</code>="pythonshell"), you can allocate either 0.0625 or 1 DPU. The default is 0.0625 DPU. </li> <li> When you specify an Apache Spark ETL job (<code>JobCommand.Name</code>="glueetl"), you can allocate from 2 to 100 DPUs. The default is 10 DPUs. This job type cannot have a fractional DPU allocation. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timeout</i> The <code>JobRun</code> timeout in minutes. This is the maximum time that a job run can consume resources before it is terminated and enters <code>TIMEOUT</code> status. The default is 2,880 minutes (48 hours). This overrides the timeout value set in the parent job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExecutionTime</i> The amount of time (in seconds) that the job run consumed resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedCapacity</i> This field is deprecated. Use <code>MaxCapacity</code> instead. The number of AWS Glue data processing units (DPUs) allocated to this JobRun. From 2 to 100 DPUs can be allocated; the default is 10. A DPU is a relative measure of processing power that consists of 4 vCPUs of compute capacity and 16 GB of memory. For more information, see the <a href="https://aws.amazon.com/glue/pricing/">AWS Glue pricing page</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PredecessorRuns</i> A list of predecessors to this job run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorMessage</i> An error message associated with this job run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arguments</i> The job arguments associated with this run. For this job run, they replace the default arguments set in the job definition itself. You can specify arguments here that your own job-execution script consumes, as well as arguments that AWS Glue itself consumes. For information about how to specify and consume your own job arguments, see the <a href="https://docs.aws.amazon.com/glue/latest/dg/aws-glue-programming-python-calling.html">Calling AWS Glue APIs in Python</a> topic in the developer guide. For information about the key-value pairs that AWS Glue consumes to set up your job, see the <a href="https://docs.aws.amazon.com/glue/latest/dg/aws-glue-programming-etl-glue-arguments.html">Special Parameters Used by AWS Glue</a> topic in the developer guide.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobRunState</i> The current state of the job run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompletedOn</i> The date and time that this job run completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedOn</i> The last time that this job run was modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartedOn</i> The date and time at which this job run was started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobName</i> The name of the job definition being used in this run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TriggerName</i> The name of the trigger that started this job run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreviousRunId</i> The ID of the previous run of this job. For example, the <code>JobRunId</code> specified in the <code>StartJobRun</code> action.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Attempt</i> The number of the attempt to run this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of this job run.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NotifyDelayAfter</i> After a job run starts, the number of minutes to wait before sending a job run delay notification.<br/>        """
        pass

    def get_job_runs(self, JobName: str, NextToken: str=None, MaxResults: int=None) -> GetJobRunsResponse:
        """Retrieves metadata for all runs of a given job definition.

        :param self:
        :param MaxResults:  The maximum size of the response.<br/>NextToken:  A continuation token, if this is a continuation call.<br/>JobName:  The name of the job definition for which to retrieve all job runs.<br/>
        :return: NextToken:  A continuation token, if not all requested job runs have been returned.<br/>JobRuns:  A list of job-run metadata objects.<br/>        """
        pass

    def get_jobs(self, NextToken: str=None, MaxResults: int=None) -> GetJobsResponse:
        """Retrieves all current job definitions.

        :param self:
        :param MaxResults:  The maximum size of the response.<br/>NextToken:  A continuation token, if this is a continuation call.<br/>
        :return: NextToken:  A continuation token, if not all job definitions have yet been returned.<br/>Jobs:  A list of job definitions.<br/>        """
        pass

    def get_mapping(self, Source: CatalogEntry, Sinks: List[CatalogEntry]=None, Location: Location=None) -> GetMappingResponse:
        """Creates mappings.

        :param self:
        :param Location:  Parameters for the mapping.:<br/>Sinks:  A list of target tables.<br/>Source:  Specifies the source table.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableName</i> The name of the table in question.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> The database in which the table metadata resides.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DynamoDB</i> An Amazon DynamoDB table location.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3</i> An Amazon Simple Storage Service (Amazon S3) location.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Jdbc</i> A JDBC location.<br/>
        :return: Mapping:  A list of mappings to the specified targets.<br/>        """
        pass

    def get_partition(self, PartitionValues: List[ValueString], TableName: str, DatabaseName: str, CatalogId: str=None) -> GetPartitionResponse:
        """Retrieves information about a specified partition.

        :param self:
        :param PartitionValues:  The values that define the partition.<br/>TableName:  The name of the partition's table.<br/>DatabaseName:  The name of the catalog database where the partition resides.<br/>CatalogId:  The ID of the Data Catalog where the partition in question resides. If none is supplied, the AWS account ID is used by default.<br/>
        :return: Partition:  The requested information, in the form of a <code>Partition</code> object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastAnalyzedTime</i> The last time at which column statistics were computed for this partition.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> These key-value pairs define partition parameters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageDescriptor</i> Provides information about the physical location where the partition is stored.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StorageDescriptor</b>: Provides information about the physical location where the partition is stored.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastAccessTime</i> The last time at which the partition was accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The time at which the partition was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableName</i> The name of the table in question.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> The name of the catalog database where the table in question is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Values</i> The values of the partition.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfBuckets</i> Must be specified if the table contains any dimension columns.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Compressed</i> True if the data in the table is compressed, or False if not.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputFormat</i> The output format: <code>SequenceFileOutputFormat</code> (binary), or <code>IgnoreKeyTextOutputFormat</code>, or a custom format.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InputFormat</i> The input format: <code>SequenceFileInputFormat</code> (binary), or <code>TextInputFormat</code>, or a custom format.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Location</i> The physical location of the table. By default this takes the form of the warehouse location, followed by the database location in the warehouse, followed by the table name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Columns</i> A list of the <code>Columns</code> in the table.<br/>Serialization/deserialization (SerDe) information.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SerdeInfo</b>: Serialization/deserialization (SerDe) information.        """
        pass

    def get_partitions(self, TableName: str, DatabaseName: str, CatalogId: str=None, Expression: str=None, NextToken: str=None, Segment: Segment=None, MaxResults: int=None) -> GetPartitionsResponse:
        """Retrieves information about the partitions in a table.

        :param self:
        :param MaxResults:  The maximum number of partitions to return in a single response.<br/>Segment:  The segment of the table's partitions to scan in this request.:<br/>NextToken:  A continuation token, if this is not the first call to retrieve these partitions.<br/>Expression:  An expression filtering the partitions to be returned. The expression uses SQL syntax similar to the SQL <code>WHERE</code> filter clause. The SQL statement parser <a href="http://jsqlparser.sourceforge.net/home.php">JSQLParser</a> parses the expression.   <i>Operators</i>: The following are the operators that you can use in the <code>Expression</code> API call: <dl> <dt>=</dt> <dd> Checks if the values of the two operands are equal or not; if yes, then the condition becomes true. Example: Assume 'variable a' holds 10 and 'variable b' holds 20.  (a = b) is not true. </dd> <dt>&lt; &gt;</dt> <dd> Checks if the values of two operands are equal or not; if the values are not equal, then the condition becomes true. Example: (a &lt; &gt; b) is true. </dd> <dt>&gt;</dt> <dd> Checks if the value of the left operand is greater than the value of the right operand; if yes, then the condition becomes true. Example: (a &gt; b) is not true. </dd> <dt>&lt;</dt> <dd> Checks if the value of the left operand is less than the value of the right operand; if yes, then the condition becomes true. Example: (a &lt; b) is true. </dd> <dt>&gt;=</dt> <dd> Checks if the value of the left operand is greater than or equal to the value of the right operand; if yes, then the condition becomes true. Example: (a &gt;= b) is not true. </dd> <dt>&lt;=</dt> <dd> Checks if the value of the left operand is less than or equal to the value of the right operand; if yes, then the condition becomes true. Example: (a &lt;= b) is true. </dd> <dt>AND, OR, IN, BETWEEN, LIKE, NOT, IS NULL</dt> <dd> Logical operators. </dd> </dl>  <i>Supported Partition Key Types</i>: The following are the the supported partition keys. <ul> <li>  <code>string</code>  </li> <li>  <code>date</code>  </li> <li>  <code>timestamp</code>  </li> <li>  <code>int</code>  </li> <li>  <code>bigint</code>  </li> <li>  <code>long</code>  </li> <li>  <code>tinyint</code>  </li> <li>  <code>smallint</code>  </li> <li>  <code>decimal</code>  </li> </ul> If an invalid type is encountered, an exception is thrown.  The following list shows the valid operators on each type. When you define a crawler, the <code>partitionKey</code> type is created as a <code>STRING</code>, to be compatible with the catalog partitions.   <i>Sample API Call</i>: <br/>TableName:  The name of the partitions' table.<br/>DatabaseName:  The name of the catalog database where the partitions reside.<br/>CatalogId:  The ID of the Data Catalog where the partitions in question reside. If none is supplied, the AWS account ID is used by default.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalSegments</i> The total numer of segments.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentNumber</i> The zero-based index number of the this segment. For example, if the total number of segments is 4, SegmentNumber values will range from zero through three.<br/>
        :return: NextToken:  A continuation token, if the returned list of partitions does not does not include the last one.<br/>Partitions:  A list of requested partitions.<br/>        """
        pass

    def get_plan(self, Source: CatalogEntry, Mapping: List[MappingEntry], Sinks: List[CatalogEntry]=None, Location: Location=None, Language: str=None) -> GetPlanResponse:
        """Gets code to perform a specified mapping.

        :param self:
        :param Language:  The programming language of the code to perform the mapping.<br/>Location:  The parameters for the mapping.:<br/>Sinks:  The target tables.<br/>Source:  The source table.:<br/>Mapping:  The list of mappings from a source table to target tables.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableName</i> The name of the table in question.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> The database in which the table metadata resides.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DynamoDB</i> An Amazon DynamoDB table location.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3</i> An Amazon Simple Storage Service (Amazon S3) location.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Jdbc</i> A JDBC location.<br/>
        :return: ScalaCode:  The Scala code to perform the mapping.<br/>PythonScript:  A Python script to perform the mapping.<br/>        """
        pass

    def get_resource_policy(self) -> GetResourcePolicyResponse:
        """Retrieves a specified resource policy.

        :param self:
        :param 
        :return: UpdateTime:  The date and time at which the policy was last updated.<br/>CreateTime:  The date and time at which the policy was created.<br/>PolicyHash:  Contains the hash value associated with this policy.<br/>PolicyInJson:  Contains the requested policy document, in JSON format.<br/>        """
        pass

    def get_security_configuration(self, Name: str) -> GetSecurityConfigurationResponse:
        """Retrieves a specified security configuration.

        :param self:
        :param Name:  The name of the security configuration to retrieve.<br/>
        :return: SecurityConfiguration:  The requested security configuration:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionConfiguration</i> The encryption configuration associated with this security configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncryptionConfiguration</b>: The encryption configuration associated with this security configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimeStamp</i> The time at which this security configuration was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the security configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Encryption</i> The encryption configuration for S3 data.<br/>The encryption configuration for CloudWatch.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CloudWatchEncryption</b>: The encryption configuration for CloudWatch.        """
        pass

    def get_security_configurations(self, MaxResults: int=None, NextToken: str=None) -> GetSecurityConfigurationsResponse:
        """Retrieves a list of all security configurations.

        :param self:
        :param NextToken:  A continuation token, if this is a continuation call.<br/>MaxResults:  The maximum number of results to return.<br/>
        :return: NextToken:  A continuation token, if there are more security configurations to return.<br/>SecurityConfigurations:  A list of security configurations.<br/>        """
        pass

    def get_table(self, Name: str, DatabaseName: str, CatalogId: str=None) -> GetTableResponse:
        """Retrieves the <code>Table</code> definition in a Data Catalog for a specified table.

        :param self:
        :param Name:  The name of the table for which to retrieve the definition. For Hive compatibility, this name is entirely lowercase.<br/>DatabaseName:  The name of the database in the catalog in which the table resides. For Hive compatibility, this name is entirely lowercase.<br/>CatalogId:  The ID of the Data Catalog where the table resides. If none is supplied, the AWS account ID is used by default.<br/>
        :return: Table:  The <code>Table</code> object that defines the specified table.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedBy</i> Person or entity who created the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> These key-value pairs define properties associated with the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableType</i> The type of this table (<code>EXTERNAL_TABLE</code>, <code>VIRTUAL_VIEW</code>, etc.).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ViewExpandedText</i> If the table is a view, the expanded text of the view; otherwise <code>null</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ViewOriginalText</i> If the table is a view, the original text of the view; otherwise <code>null</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PartitionKeys</i> A list of columns by which the table is partitioned. Only primitive types are supported as partition keys. When creating a table used by Athena, and you do not specify any <code>partitionKeys</code>, you must at least set the value of <code>partitionKeys</code> to an empty list. For example:  <code>"PartitionKeys": []</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageDescriptor</i> A storage descriptor containing information about the physical storage of this table.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StorageDescriptor</b>: A storage descriptor containing information about the physical storage of this table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Retention</i> Retention time for this table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastAnalyzedTime</i> Last time column statistics were computed for this table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastAccessTime</i> Last time the table was accessed. This is usually taken from HDFS, and may not be reliable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateTime</i> Last time the table was updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateTime</i> Time when the table definition was created in the Data Catalog.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Owner</i> Owner of the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> Description of the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> Name of the metadata database where the table metadata resides. For Hive compatibility, this must be all lowercase.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Name of the table. For Hive compatibility, this must be entirely lowercase.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfBuckets</i> Must be specified if the table contains any dimension columns.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Compressed</i> True if the data in the table is compressed, or False if not.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputFormat</i> The output format: <code>SequenceFileOutputFormat</code> (binary), or <code>IgnoreKeyTextOutputFormat</code>, or a custom format.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InputFormat</i> The input format: <code>SequenceFileInputFormat</code> (binary), or <code>TextInputFormat</code>, or a custom format.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Location</i> The physical location of the table. By default this takes the form of the warehouse location, followed by the database location in the warehouse, followed by the table name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Columns</i> A list of the <code>Columns</code> in the table.<br/>Serialization/deserialization (SerDe) information.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SerdeInfo</b>: Serialization/deserialization (SerDe) information.        """
        pass

    def get_table_version(self, TableName: str, DatabaseName: str, CatalogId: str=None, VersionId: str=None) -> GetTableVersionResponse:
        """Retrieves a specified version of a table.

        :param self:
        :param VersionId:  The ID value of the table version to be retrieved. A <code>VersionID</code> is a string representation of an integer. Each version is incremented by 1. <br/>TableName:  The name of the table. For Hive compatibility, this name is entirely lowercase.<br/>DatabaseName:  The database in the catalog in which the table resides. For Hive compatibility, this name is entirely lowercase.<br/>CatalogId:  The ID of the Data Catalog where the tables reside. If none is supplied, the AWS account ID is used by default.<br/>
        :return: TableVersion:  The requested table version.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VersionId</i> The ID value that identifies this table version. A <code>VersionId</code> is a string representation of an integer. Each version is incremented by 1.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Table</i> The table in question&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Table</b>: The table in question<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Retention</i> Retention time for this table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LastAnalyzedTime</i> Last time column statistics were computed for this table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LastAccessTime</i> Last time the table was accessed. This is usually taken from HDFS, and may not be reliable.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateTime</i> Last time the table was updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateTime</i> Time when the table definition was created in the Data Catalog.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Owner</i> Owner of the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> Description of the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> Name of the metadata database where the table metadata resides. For Hive compatibility, this must be all lowercase.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Name of the table. For Hive compatibility, this must be entirely lowercase.<br/>A storage descriptor containing information about the physical storage of this table.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StorageDescriptor</b>: A storage descriptor containing information about the physical storage of this table.        """
        pass

    def get_table_versions(self, TableName: str, DatabaseName: str, CatalogId: str=None, NextToken: str=None, MaxResults: int=None) -> GetTableVersionsResponse:
        """Retrieves a list of strings that identify available versions of a specified table.

        :param self:
        :param MaxResults:  The maximum number of table versions to return in one response.<br/>NextToken:  A continuation token, if this is not the first call.<br/>TableName:  The name of the table. For Hive compatibility, this name is entirely lowercase.<br/>DatabaseName:  The database in the catalog in which the table resides. For Hive compatibility, this name is entirely lowercase.<br/>CatalogId:  The ID of the Data Catalog where the tables reside. If none is supplied, the AWS account ID is used by default.<br/>
        :return: NextToken:  A continuation token, if the list of available versions does not include the last one.<br/>TableVersions:  A list of strings identifying available versions of the specified table.<br/>        """
        pass

    def get_tables(self, DatabaseName: str, CatalogId: str=None, Expression: str=None, NextToken: str=None, MaxResults: int=None) -> GetTablesResponse:
        """Retrieves the definitions of some or all of the tables in a given <code>Database</code>.

        :param self:
        :param MaxResults:  The maximum number of tables to return in a single response.<br/>NextToken:  A continuation token, included if this is a continuation call.<br/>Expression:  A regular expression pattern. If present, only those tables whose names match the pattern are returned.<br/>DatabaseName:  The database in the catalog whose tables to list. For Hive compatibility, this name is entirely lowercase.<br/>CatalogId:  The ID of the Data Catalog where the tables reside. If none is supplied, the AWS account ID is used by default.<br/>
        :return: NextToken:  A continuation token, present if the current list segment is not the last.<br/>TableList:  A list of the requested <code>Table</code> objects.<br/>        """
        pass

    def get_tags(self, ResourceArn: str) -> GetTagsResponse:
        """Retrieves a list of tags associated with a resource.

        :param self:
        :param ResourceArn:  The Amazon Resource Name (ARN) of the resource for which to retrieve tags.<br/>
        :return: Tags:  The requested tags.<br/>        """
        pass

    def get_trigger(self, Name: str) -> GetTriggerResponse:
        """Retrieves the definition of a trigger.

        :param self:
        :param Name:  The name of the trigger to retrieve.<br/>
        :return: Trigger:  The requested trigger definition.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Predicate</i> The predicate of this trigger, which defines when it will fire.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Predicate</b>: The predicate of this trigger, which defines when it will fire.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Actions</i> The actions initiated by this trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Schedule</i> A <code>cron</code> expression used to specify the schedule (see <a href="https://docs.aws.amazon.com/glue/latest/dg/monitor-data-warehouse-schedule.html">Time-Based Schedules for Jobs and Crawlers</a>. For example, to run something every day at 12:15 UTC, you would specify: <code>cron(15 12 * * ? *)</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of this trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The current state of the trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of trigger that this is.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> Reserved for future use.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WorkflowName</i> The name of the workflow associated with the trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Conditions</i> A list of the conditions that determine when the trigger will fire.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Logical</i> An optional field if only one condition is listed. If multiple conditions are listed, then this field is required.<br/>        """
        pass

    def get_triggers(self, NextToken: str=None, DependentJobName: str=None, MaxResults: int=None) -> GetTriggersResponse:
        """Gets all the triggers associated with a job.

        :param self:
        :param MaxResults:  The maximum size of the response.<br/>DependentJobName:  The name of the job to retrieve triggers for. The trigger that can start this job is returned, and if there is no such trigger, all triggers are returned.<br/>NextToken:  A continuation token, if this is a continuation call.<br/>
        :return: NextToken:  A continuation token, if not all the requested triggers have yet been returned.<br/>Triggers:  A list of triggers for the specified job.<br/>        """
        pass

    def get_user_defined_function(self, FunctionName: str, DatabaseName: str, CatalogId: str=None) -> GetUserDefinedFunctionResponse:
        """Retrieves a specified function definition from the Data Catalog.

        :param self:
        :param FunctionName:  The name of the function.<br/>DatabaseName:  The name of the catalog database where the function is located.<br/>CatalogId:  The ID of the Data Catalog where the function to be retrieved is located. If none is supplied, the AWS account ID is used by default.<br/>
        :return: UserDefinedFunction:  The requested function definition.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceUris</i> The resource URIs for the function.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateTime</i> The time at which the function was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerType</i> The owner type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerName</i> The owner of the function.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClassName</i> The Java class that contains the function code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FunctionName</i> The name of the function.<br/>        """
        pass

    def get_user_defined_functions(self, Pattern: str, DatabaseName: str, CatalogId: str=None, NextToken: str=None, MaxResults: int=None) -> GetUserDefinedFunctionsResponse:
        """Retrieves a multiple function definitions from the Data Catalog.

        :param self:
        :param MaxResults:  The maximum number of functions to return in one response.<br/>NextToken:  A continuation token, if this is a continuation call.<br/>Pattern:  An optional function-name pattern string that filters the function definitions returned.<br/>DatabaseName:  The name of the catalog database where the functions are located.<br/>CatalogId:  The ID of the Data Catalog where the functions to be retrieved are located. If none is supplied, the AWS account ID is used by default.<br/>
        :return: NextToken:  A continuation token, if the list of functions returned does not include the last requested function.<br/>UserDefinedFunctions:  A list of requested function definitions.<br/>        """
        pass

    def get_workflow(self, Name: str, IncludeGraph: NullableBoolean=None) -> GetWorkflowResponse:
        """Retrieves resource metadata for a workflow.

        :param self:
        :param IncludeGraph:  Specifies whether to include a graph when returning the workflow resource metadata.<br/>Name:  The name of the workflow to retrieve.<br/>
        :return: Workflow:  The resource metadata for the workflow.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Graph</i> The graph representing all the AWS Glue components that belong to the workflow as nodes and directed connections between them as edges.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Graph</b>: The graph representing all the AWS Glue components that belong to the workflow as nodes and directed connections between them as edges.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastRun</i> The information about the last execution of the workflow.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LastRun</b>: The information about the last execution of the workflow.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedOn</i> The date and time when the workflow was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedOn</i> The date and time when the workflow was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultRunProperties</i> A collection of properties to be used as part of each execution of the workflow.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of the workflow.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the workflow representing the flow.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the workflow run.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CompletedOn</i> The date and time when the workflow run completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StartedOn</i> The date and time when the workflow run was started.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WorkflowRunProperties</i> The workflow run properties which were set during the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WorkflowRunId</i> The ID of this workflow run.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Name of the workflow which was executed.<br/>The statistics of the run.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Statistics</b>: The statistics of the run.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Edges</i> A list of all the directed connections between the nodes belonging to the workflow.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Nodes</i> A list of the the AWS Glue components belong to the workflow represented as nodes.<br/>        """
        pass

    def get_workflow_run(self, RunId: str, Name: str, IncludeGraph: NullableBoolean=None) -> GetWorkflowRunResponse:
        """Retrieves the metadata for a given workflow run. 

        :param self:
        :param IncludeGraph:  Specifies whether to include the workflow graph in response or not.<br/>RunId:  The ID of the workflow run.<br/>Name:  Name of the workflow being run.<br/>
        :return: Run:  The requested workflow run metadata.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Graph</i> The graph representing all the AWS Glue components that belong to the workflow as nodes and directed connections between them as edges.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Graph</b>: The graph representing all the AWS Glue components that belong to the workflow as nodes and directed connections between them as edges.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Statistics</i> The statistics of the run.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Statistics</b>: The statistics of the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the workflow run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompletedOn</i> The date and time when the workflow run completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartedOn</i> The date and time when the workflow run was started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WorkflowRunProperties</i> The workflow run properties which were set during the run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WorkflowRunId</i> The ID of this workflow run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Name of the workflow which was executed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RunningActions</i> Total number Actions in running state.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SucceededActions</i> Total number of Actions which have succeeded.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StoppedActions</i> Total number of Actions which have stopped.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FailedActions</i> Total number of Actions which have failed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeoutActions</i> Total number of Actions which timed out.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalActions</i> Total number of Actions in the workflow run.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Edges</i> A list of all the directed connections between the nodes belonging to the workflow.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Nodes</i> A list of the the AWS Glue components belong to the workflow represented as nodes.<br/>        """
        pass

    def get_workflow_run_properties(self, RunId: str, Name: str) -> GetWorkflowRunPropertiesResponse:
        """Retrieves the workflow run properties which were set during the run.

        :param self:
        :param RunId:  The ID of the workflow run whose run properties should be returned.<br/>Name:  Name of the workflow which was run.<br/>
        :return: RunProperties:  The workflow run properties which were set during the specified run.<br/>        """
        pass

    def get_workflow_runs(self, Name: str, IncludeGraph: NullableBoolean=None, NextToken: str=None, MaxResults: int=None) -> GetWorkflowRunsResponse:
        """Retrieves metadata for all runs of a given workflow.

        :param self:
        :param MaxResults:  The maximum number of workflow runs to be included in the response.<br/>NextToken:  The maximum size of the response.<br/>IncludeGraph:  Specifies whether to include the workflow graph in response or not.<br/>Name:  Name of the workflow whose metadata of runs should be returned.<br/>
        :return: NextToken:  A continuation token, if not all requested workflow runs have been returned.<br/>Runs:  A list of workflow run metadata objects.<br/>        """
        pass

    def import_catalog_to_glue(self, CatalogId: str=None) -> ImportCatalogToGlueResponse:
        """Imports an existing Athena Data Catalog to AWS Glue

        :param self:
        :param CatalogId:  The ID of the catalog to import. Currently, this should be the AWS account ID.<br/>
        :return:         """
        pass

    def list_crawlers(self, MaxResults: int=None, NextToken: str=None, Tags: TagsMap=None) -> ListCrawlersResponse:
        """Retrieves the names of all crawler resources in this AWS account, or the resources with the specified tag. This operation allows you to see which resources are available in your account, and their names. This operation takes the optional <code>Tags</code> field, which you can use as a filter on the response so that tagged resources can be retrieved as a group. If you choose to use tags filtering, only resources with the tag are retrieved.

        :param self:
        :param Tags:  Specifies to return only these tagged resources.<br/>NextToken:  A continuation token, if this is a continuation request.<br/>MaxResults:  The maximum size of a list to return.<br/>
        :return: NextToken:  A continuation token, if the returned list does not contain the last metric available.<br/>CrawlerNames:  The names of all crawlers in the account, or the crawlers with the specified tags.<br/>        """
        pass

    def list_dev_endpoints(self, NextToken: str=None, MaxResults: int=None, Tags: TagsMap=None) -> ListDevEndpointsResponse:
        """Retrieves the names of all <code>DevEndpoint</code> resources in this AWS account, or the resources with the specified tag. This operation allows you to see which resources are available in your account, and their names. This operation takes the optional <code>Tags</code> field, which you can use as a filter on the response so that tagged resources can be retrieved as a group. If you choose to use tags filtering, only resources with the tag are retrieved.

        :param self:
        :param Tags:  Specifies to return only these tagged resources.<br/>MaxResults:  The maximum size of a list to return.<br/>NextToken:  A continuation token, if this is a continuation request.<br/>
        :return: NextToken:  A continuation token, if the returned list does not contain the last metric available.<br/>DevEndpointNames:  The names of all the <code>DevEndpoint</code>s in the account, or the <code>DevEndpoint</code>s with the specified tags.<br/>        """
        pass

    def list_jobs(self, NextToken: str=None, MaxResults: int=None, Tags: TagsMap=None) -> ListJobsResponse:
        """Retrieves the names of all job resources in this AWS account, or the resources with the specified tag. This operation allows you to see which resources are available in your account, and their names. This operation takes the optional <code>Tags</code> field, which you can use as a filter on the response so that tagged resources can be retrieved as a group. If you choose to use tags filtering, only resources with the tag are retrieved.

        :param self:
        :param Tags:  Specifies to return only these tagged resources.<br/>MaxResults:  The maximum size of a list to return.<br/>NextToken:  A continuation token, if this is a continuation request.<br/>
        :return: NextToken:  A continuation token, if the returned list does not contain the last metric available.<br/>JobNames:  The names of all jobs in the account, or the jobs with the specified tags.<br/>        """
        pass

    def list_triggers(self, NextToken: str=None, DependentJobName: str=None, MaxResults: int=None, Tags: TagsMap=None) -> ListTriggersResponse:
        """Retrieves the names of all trigger resources in this AWS account, or the resources with the specified tag. This operation allows you to see which resources are available in your account, and their names. This operation takes the optional <code>Tags</code> field, which you can use as a filter on the response so that tagged resources can be retrieved as a group. If you choose to use tags filtering, only resources with the tag are retrieved.

        :param self:
        :param Tags:  Specifies to return only these tagged resources.<br/>MaxResults:  The maximum size of a list to return.<br/>DependentJobName:   The name of the job for which to retrieve triggers. The trigger that can start this job is returned. If there is no such trigger, all triggers are returned.<br/>NextToken:  A continuation token, if this is a continuation request.<br/>
        :return: NextToken:  A continuation token, if the returned list does not contain the last metric available.<br/>TriggerNames:  The names of all triggers in the account, or the triggers with the specified tags.<br/>        """
        pass

    def list_workflows(self, NextToken: str=None, MaxResults: int=None) -> ListWorkflowsResponse:
        """Lists names of workflows created in the account.

        :param self:
        :param MaxResults:  The maximum size of a list to return.<br/>NextToken:  A continuation token, if this is a continuation request.<br/>
        :return: NextToken:  A continuation token, if not all workflow names have been returned.<br/>Workflows:  List of names of workflows in the account.<br/>        """
        pass

    def put_data_catalog_encryption_settings(self, DataCatalogEncryptionSettings: DataCatalogEncryptionSettings, CatalogId: str=None) -> PutDataCatalogEncryptionSettingsResponse:
        """Sets the security configuration for a specified catalog. After the configuration has been set, the specified encryption is applied to every catalog write thereafter.

        :param self:
        :param DataCatalogEncryptionSettings:  The security configuration to set.:<br/>CatalogId:  The ID of the Data Catalog for which to set the security configuration. If none is provided, the AWS account ID is used by default.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectionPasswordEncryption</i> When connection password protection is enabled, the Data Catalog uses a customer-provided key to encrypt the password as part of <code>CreateConnection</code> or <code>UpdateConnection</code> and store it in the <code>ENCRYPTED_PASSWORD</code> field in the connection properties. You can enable catalog encryption or only password encryption.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ConnectionPasswordEncryption</b>: When connection password protection is enabled, the Data Catalog uses a customer-provided key to encrypt the password as part of <code>CreateConnection</code> or <code>UpdateConnection</code> and store it in the <code>ENCRYPTED_PASSWORD</code> field in the connection properties. You can enable catalog encryption or only password encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionAtRest</i> Specifies the encryption-at-rest configuration for the Data Catalog.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncryptionAtRest</b>: Specifies the encryption-at-rest configuration for the Data Catalog.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SseAwsKmsKeyId</i> The ID of the AWS KMS key to use for encryption at rest.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CatalogEncryptionMode</i> The encryption-at-rest mode for encrypting Data Catalog data.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AwsKmsKeyId</i> An AWS KMS key that is used to encrypt the connection password.  If connection password protection is enabled, the caller of <code>CreateConnection</code> and <code>UpdateConnection</code> needs at least <code>kms:Encrypt</code> permission on the specified AWS KMS key, to encrypt passwords before storing them in the Data Catalog.  You can set the decrypt permission to enable or restrict access on the password key according to your security requirements.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReturnConnectionPasswordEncrypted</i> When the <code>ReturnConnectionPasswordEncrypted</code> flag is set to "true", passwords remain encrypted in the responses of <code>GetConnection</code> and <code>GetConnections</code>. This encryption takes effect independently from catalog encryption. <br/>
        :return:         """
        pass

    def put_resource_policy(self, PolicyInJson: str, PolicyHashCondition: str=None, PolicyExistsCondition: str=None) -> PutResourcePolicyResponse:
        """Sets the Data Catalog resource policy for access control.

        :param self:
        :param PolicyExistsCondition:  A value of <code>MUST_EXIST</code> is used to update a policy. A value of <code>NOT_EXIST</code> is used to create a new policy. If a value of <code>NONE</code> or a null value is used, the call will not depend on the existence of a policy.<br/>PolicyHashCondition:  The hash value returned when the previous policy was set using <code>PutResourcePolicy</code>. Its purpose is to prevent concurrent modifications of a policy. Do not use this parameter if no previous policy has been set.<br/>PolicyInJson:  Contains the policy document to set, in JSON format.<br/>
        :return: PolicyHash:  A hash of the policy that has just been set. This must be included in a subsequent call that overwrites or updates this policy.<br/>        """
        pass

    def put_workflow_run_properties(self, RunProperties: WorkflowRunProperties, RunId: str, Name: str) -> PutWorkflowRunPropertiesResponse:
        """Puts the specified workflow run properties for the given workflow run. If a property already exists for the specified run, then it overrides the value otherwise adds the property to existing properties.

        :param self:
        :param RunProperties:  The properties to put for the specified run.<br/>RunId:  The ID of the workflow run for which the run properties should be updated.<br/>Name:  Name of the workflow which was run.<br/>
        :return:         """
        pass

    def reset_job_bookmark(self, JobName: str) -> ResetJobBookmarkResponse:
        """Resets a bookmark entry.

        :param self:
        :param JobName:  The name of the job in question.<br/>
        :return: JobBookmarkEntry:  The reset bookmark entry.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobBookmark</i> The bookmark itself.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Attempt</i> The attempt ID number.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Run</i> The run ID number.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> Version of the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobName</i> Name of the job in question.<br/>        """
        pass

    def start_crawler(self, Name: str) -> StartCrawlerResponse:
        """Starts a crawl using the specified crawler, regardless of what is scheduled. If the crawler is already running, returns a <a href="https://docs.aws.amazon.com/glue/latest/dg/aws-glue-api-exceptions.html#aws-glue-api-exceptions-CrawlerRunningException">CrawlerRunningException</a>.

        :param self:
        :param Name:  Name of the crawler to start.<br/>
        :return:         """
        pass

    def start_crawler_schedule(self, CrawlerName: str) -> StartCrawlerScheduleResponse:
        """Changes the schedule state of the specified crawler to <code>SCHEDULED</code>, unless the crawler is already running or the schedule state is already <code>SCHEDULED</code>.

        :param self:
        :param CrawlerName:  Name of the crawler to schedule.<br/>
        :return:         """
        pass

    def start_job_run(self, JobName: str, JobRunId: str=None, Arguments: GenericMap=None, AllocatedCapacity: int=None, Timeout: int=None, MaxCapacity: NullableDouble=None, SecurityConfiguration: str=None, NotificationProperty: NotificationProperty=None, WorkerType: str=None, NumberOfWorkers: int=None) -> StartJobRunResponse:
        """Starts a job run using a job definition.

        :param self:
        :param NumberOfWorkers:  The number of workers of a defined <code>workerType</code> that are allocated when a job runs. The maximum number of workers you can define are 299 for <code>G.1X</code>, and 149 for <code>G.2X</code>. <br/>WorkerType:  The type of predefined worker that is allocated when a job runs. Accepts a value of Standard, G.1X, or G.2X. <ul> <li> For the <code>Standard</code> worker type, each worker provides 4 vCPU, 16 GB of memory and a 50GB disk, and 2 executors per worker. </li> <li> For the <code>G.1X</code> worker type, each worker provides 4 vCPU, 16 GB of memory and a 64GB disk, and 1 executor per worker. </li> <li> For the <code>G.2X</code> worker type, each worker provides 8 vCPU, 32 GB of memory and a 128GB disk, and 1 executor per worker. </li> </ul><br/>NotificationProperty:  Specifies configuration properties of a job run notification.:<br/>SecurityConfiguration:  The name of the <code>SecurityConfiguration</code> structure to be used with this job run.<br/>MaxCapacity:  The number of AWS Glue data processing units (DPUs) that can be allocated when this job runs. A DPU is a relative measure of processing power that consists of 4 vCPUs of compute capacity and 16 GB of memory. For more information, see the <a href="https://docs.aws.amazon.com/https:/aws.amazon.com/glue/pricing/">AWS Glue pricing page</a>. Do not set <code>Max Capacity</code> if using <code>WorkerType</code> and <code>NumberOfWorkers</code>. The value that can be allocated for <code>MaxCapacity</code> depends on whether you are running a Python shell job, or an Apache Spark ETL job: <ul> <li> When you specify a Python shell job (<code>JobCommand.Name</code>="pythonshell"), you can allocate either 0.0625 or 1 DPU. The default is 0.0625 DPU. </li> <li> When you specify an Apache Spark ETL job (<code>JobCommand.Name</code>="glueetl"), you can allocate from 2 to 100 DPUs. The default is 10 DPUs. This job type cannot have a fractional DPU allocation. </li> </ul><br/>Timeout:  The <code>JobRun</code> timeout in minutes. This is the maximum time that a job run can consume resources before it is terminated and enters <code>TIMEOUT</code> status. The default is 2,880 minutes (48 hours). This overrides the timeout value set in the parent job.<br/>AllocatedCapacity:  This field is deprecated. Use <code>MaxCapacity</code> instead. The number of AWS Glue data processing units (DPUs) to allocate to this JobRun. From 2 to 100 DPUs can be allocated; the default is 10. A DPU is a relative measure of processing power that consists of 4 vCPUs of compute capacity and 16 GB of memory. For more information, see the <a href="https://docs.aws.amazon.com/https:/aws.amazon.com/glue/pricing/">AWS Glue pricing page</a>.<br/>Arguments:  The job arguments specifically for this run. For this job run, they replace the default arguments set in the job definition itself. You can specify arguments here that your own job-execution script consumes, as well as arguments that AWS Glue itself consumes. For information about how to specify and consume your own Job arguments, see the <a href="https://docs.aws.amazon.com/glue/latest/dg/aws-glue-programming-python-calling.html">Calling AWS Glue APIs in Python</a> topic in the developer guide. For information about the key-value pairs that AWS Glue consumes to set up your job, see the <a href="https://docs.aws.amazon.com/glue/latest/dg/aws-glue-programming-etl-glue-arguments.html">Special Parameters Used by AWS Glue</a> topic in the developer guide.<br/>JobRunId:  The ID of a previous <code>JobRun</code> to retry.<br/>JobName:  The name of the job definition to use.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotifyDelayAfter</i> After a job run starts, the number of minutes to wait before sending a job run delay notification.<br/>
        :return: JobRunId:  The ID assigned to this job run.<br/>        """
        pass

    def start_trigger(self, Name: str) -> StartTriggerResponse:
        """Starts an existing trigger. See <a href="https://docs.aws.amazon.com/glue/latest/dg/trigger-job.html">Triggering Jobs</a> for information about how different types of trigger are started.

        :param self:
        :param Name:  The name of the trigger to start.<br/>
        :return: Name:  The name of the trigger that was started.<br/>        """
        pass

    def start_workflow_run(self, Name: str) -> StartWorkflowRunResponse:
        """Starts a new run of the specified workflow.

        :param self:
        :param Name:  The name of the workflow to start.<br/>
        :return: RunId:  An Id for the new run.<br/>        """
        pass

    def stop_crawler(self, Name: str) -> StopCrawlerResponse:
        """If the specified crawler is running, stops the crawl.

        :param self:
        :param Name:  Name of the crawler to stop.<br/>
        :return:         """
        pass

    def stop_crawler_schedule(self, CrawlerName: str) -> StopCrawlerScheduleResponse:
        """Sets the schedule state of the specified crawler to <code>NOT_SCHEDULED</code>, but does not stop the crawler if it is already running.

        :param self:
        :param CrawlerName:  Name of the crawler whose schedule state to set.<br/>
        :return:         """
        pass

    def stop_trigger(self, Name: str) -> StopTriggerResponse:
        """Stops a specified trigger.

        :param self:
        :param Name:  The name of the trigger to stop.<br/>
        :return: Name:  The name of the trigger that was stopped.<br/>        """
        pass

    def tag_resource(self, TagsToAdd: TagsMap, ResourceArn: str) -> TagResourceResponse:
        """Adds tags to a resource. A tag is a label you can assign to an AWS resource. In AWS Glue, you can tag only certain resources. For information about what resources you can tag, see <a href="https://docs.aws.amazon.com/glue/latest/dg/monitor-tags.html">AWS Tags in AWS Glue</a>.

        :param self:
        :param TagsToAdd:  Tags to add to this resource.<br/>ResourceArn:  The ARN of the AWS Glue resource to which to add the tags. For more information about AWS Glue resource ARNs, see the <a href="https://docs.aws.amazon.com/glue/latest/dg/aws-glue-api-common.html#aws-glue-api-regex-aws-glue-arn-id">AWS Glue ARN string pattern</a>.<br/>
        :return:         """
        pass

    def untag_resource(self, TagsToRemove: List[TagKey], ResourceArn: str) -> UntagResourceResponse:
        """Removes tags from a resource.

        :param self:
        :param TagsToRemove:  Tags to remove from this resource.<br/>ResourceArn:  The Amazon Resource Name (ARN) of the resource from which to remove the tags.<br/>
        :return:         """
        pass

    def update_classifier(self, GrokClassifier: UpdateGrokClassifierRequest=None, XMLClassifier: UpdateXMLClassifierRequest=None, JsonClassifier: UpdateJsonClassifierRequest=None, CsvClassifier: UpdateCsvClassifierRequest=None) -> UpdateClassifierResponse:
        """Modifies an existing classifier (a <code>GrokClassifier</code>, an <code>XMLClassifier</code>, a <code>JsonClassifier</code>, or a <code>CsvClassifier</code>, depending on which field is present).

        :param self:
        :param CsvClassifier:  A <code>CsvClassifier</code> object with updated fields.:<br/>JsonClassifier:  A <code>JsonClassifier</code> object with updated fields.:<br/>XMLClassifier:  An <code>XMLClassifier</code> object with updated fields.:<br/>GrokClassifier:  A <code>GrokClassifier</code> object with updated fields.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomPatterns</i> Optional custom grok patterns used by this classifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GrokPattern</i> The grok pattern used by this classifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Classification</i> An identifier of the data format that the classifier matches, such as Twitter, JSON, Omniture logs, Amazon CloudWatch Logs, and so on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the <code>GrokClassifier</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RowTag</i> The XML tag designating the element that contains each record in an XML document being parsed. This cannot identify a self-closing element (closed by <code>/&gt;</code>). An empty row element that contains only attributes can be parsed as long as it ends with a closing tag (for example, <code>&lt;row item_a="A" item_b="B"&gt;&lt;/row&gt;</code> is okay, but <code>&lt;row item_a="A" item_b="B" /&gt;</code> is not).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Classification</i> An identifier of the data format that the classifier matches.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the classifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JsonPath</i> A <code>JsonPath</code> string defining the JSON data for the classifier to classify. AWS Glue supports a subset of <code>JsonPath</code>, as described in <a href="https://docs.aws.amazon.com/glue/latest/dg/custom-classifier.html#custom-classifier-json">Writing JsonPath Custom Classifiers</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the classifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowSingleColumn</i> Enables the processing of files that contain only one column.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisableValueTrimming</i> Specifies not to trim values before identifying the type of column values. The default value is true.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Header</i> A list of strings representing column names.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContainsHeader</i> Indicates whether the CSV file contains a header.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QuoteSymbol</i> A custom symbol to denote what combines content into a single column value. It must be different from the column delimiter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Delimiter</i> A custom symbol to denote what separates each column entry in the row.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the classifier.<br/>
        :return:         """
        pass

    def update_connection(self, ConnectionInput: ConnectionInput, Name: str, CatalogId: str=None) -> UpdateConnectionResponse:
        """Updates a connection definition in the Data Catalog.

        :param self:
        :param ConnectionInput:  A <code>ConnectionInput</code> object that redefines the connection in question.:<br/>Name:  The name of the connection definition to update.<br/>CatalogId:  The ID of the Data Catalog in which the connection resides. If none is provided, the AWS account ID is used by default.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhysicalConnectionRequirements</i> A map of physical connection requirements, such as virtual private cloud (VPC) and <code>SecurityGroup</code>, that are needed to successfully make this connection.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PhysicalConnectionRequirements</b>: A map of physical connection requirements, such as virtual private cloud (VPC) and <code>SecurityGroup</code>, that are needed to successfully make this connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectionProperties</i> These key-value pairs define parameters for the connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MatchCriteria</i> A list of criteria that can be used in selecting this connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectionType</i> The type of the connection. Currently, only JDBC is supported; SFTP is not supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The connection's Availability Zone. This field is redundant because the specified subnet implies the Availability Zone to be used. Currently the field must be populated, but it will be deprecated in the future.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIdList</i> The security group ID list used by the connection.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetId</i> The subnet ID used by the connection.<br/>
        :return:         """
        pass

    def update_crawler(self, Name: str, Role: str=None, DatabaseName: str=None, Description: str=None, Targets: CrawlerTargets=None, Schedule: str=None, Classifiers: List[NameString]=None, TablePrefix: str=None, SchemaChangePolicy: SchemaChangePolicy=None, Configuration: str=None, CrawlerSecurityConfiguration: str=None) -> UpdateCrawlerResponse:
        """Updates a crawler. If a crawler is running, you must stop it using <code>StopCrawler</code> before updating it.

        :param self:
        :param CrawlerSecurityConfiguration:  The name of the <code>SecurityConfiguration</code> structure to be used by this crawler.<br/>Configuration:  The crawler configuration information. This versioned JSON string allows users to specify aspects of a crawler's behavior. For more information, see <a href="http://docs.aws.amazon.com/glue/latest/dg/crawler-configuration.html">Configuring a Crawler</a>.<br/>SchemaChangePolicy:  The policy for the crawler's update and deletion behavior.:<br/>TablePrefix:  The table prefix used for catalog tables that are created.<br/>Classifiers:  A list of custom classifiers that the user has registered. By default, all built-in classifiers are included in a crawl, but these custom classifiers always override the default classifiers for a given classification.<br/>Schedule:  A <code>cron</code> expression used to specify the schedule. For more information, see <a href="http://docs.aws.amazon.com/glue/latest/dg/monitor-data-warehouse-schedule.html">Time-Based Schedules for Jobs and Crawlers</a>. For example, to run something every day at 12:15 UTC, specify <code>cron(15 12 * * ? *)</code>.<br/>Targets:  A list of targets to crawl.:<br/>Description:  A description of the new crawler.<br/>DatabaseName:  The AWS Glue database where results are stored, such as: <code>arn:aws:daylight:us-east-1::database/sometable/*</code>.<br/>Role:  The IAM role or Amazon Resource Name (ARN) of an IAM role that is used by the new crawler to access customer resources.<br/>Name:  Name of the new crawler.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CatalogTargets</i> Specifies AWS Glue Data Catalog targets.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DynamoDBTargets</i> Specifies Amazon DynamoDB targets.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JdbcTargets</i> Specifies JDBC targets.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Targets</i> Specifies Amazon Simple Storage Service (Amazon S3) targets.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeleteBehavior</i> The deletion behavior when the crawler finds a deleted object.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateBehavior</i> The update behavior when the crawler finds a changed schema.<br/>
        :return:         """
        pass

    def update_crawler_schedule(self, CrawlerName: str, Schedule: str=None) -> UpdateCrawlerScheduleResponse:
        """Updates the schedule of a crawler using a <code>cron</code> expression. 

        :param self:
        :param Schedule:  The updated <code>cron</code> expression used to specify the schedule. For more information, see <a href="http://docs.aws.amazon.com/glue/latest/dg/monitor-data-warehouse-schedule.html">Time-Based Schedules for Jobs and Crawlers</a>. For example, to run something every day at 12:15 UTC, specify <code>cron(15 12 * * ? *)</code>.<br/>CrawlerName:  The name of the crawler whose schedule to update.<br/>
        :return:         """
        pass

    def update_database(self, DatabaseInput: DatabaseInput, Name: str, CatalogId: str=None) -> UpdateDatabaseResponse:
        """Updates an existing database definition in a Data Catalog.

        :param self:
        :param DatabaseInput:  A <code>DatabaseInput</code> object specifying the new definition of the metadata database in the catalog.:<br/>Name:  The name of the database to update in the catalog. For Hive compatibility, this is folded to lowercase.<br/>CatalogId:  The ID of the Data Catalog in which the metadata database resides. If none is supplied, the AWS account ID is used by default.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> Thes key-value pairs define parameters and properties of the database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LocationUri</i> The location of the database (for example, an HDFS path).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> Description of the database<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Name of the database. For Hive compatibility, this is folded to lowercase when it is stored.<br/>
        :return:         """
        pass

    def update_dev_endpoint(self, EndpointName: str, PublicKey: str=None, AddPublicKeys: List[GenericString]=None, DeletePublicKeys: List[GenericString]=None, CustomLibraries: DevEndpointCustomLibraries=None, UpdateEtlLibraries: BooleanValue=None, DeleteArguments: List[GenericString]=None, AddArguments: MapValue=None) -> UpdateDevEndpointResponse:
        """Updates a specified DevEndpoint.

        :param self:
        :param AddArguments:  The map of arguments to add the map of arguments used to configure the DevEndpoint.<br/>DeleteArguments:  The list of argument keys to be deleted from the map of arguments used to configure the DevEndpoint.<br/>UpdateEtlLibraries:  True if the list of custom libraries to be loaded in the development endpoint needs to be updated, or False otherwise.<br/>CustomLibraries:  Custom Python or Java libraries to be loaded in the DevEndpoint.:<br/>DeletePublicKeys:  The list of public keys to be deleted from the DevEndpoint.<br/>AddPublicKeys:  The list of public keys for the DevEndpoint to use.<br/>PublicKey:  The public key for the DevEndpoint to use.<br/>EndpointName:  The name of the DevEndpoint to be updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExtraJarsS3Path</i> Path to one or more Java Jars in an S3 bucket that should be loaded in your DevEndpoint. Please note that only pure Java/Scala libraries can currently be used on a DevEndpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExtraPythonLibsS3Path</i> Path(s) to one or more Python libraries in an S3 bucket that should be loaded in your DevEndpoint. Multiple values must be complete paths separated by a comma. Please note that only pure Python libraries can currently be used on a DevEndpoint. Libraries that rely on C extensions, such as the <a href="http://pandas.pydata.org/">pandas</a> Python data analysis library, are not yet supported.<br/>
        :return:         """
        pass

    def update_job(self, JobUpdate: JobUpdate, JobName: str) -> UpdateJobResponse:
        """Updates an existing job definition.

        :param self:
        :param JobUpdate:  Specifies the values with which to update the job definition.:<br/>JobName:  The name of the job definition to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationProperty</i> Specifies the configuration properties of a job notification.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NotificationProperty</b>: Specifies the configuration properties of a job notification.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityConfiguration</i> The name of the <code>SecurityConfiguration</code> structure to be used with this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfWorkers</i> The number of workers of a defined <code>workerType</code> that are allocated when a job runs. The maximum number of workers you can define are 299 for <code>G.1X</code>, and 149 for <code>G.2X</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WorkerType</i> The type of predefined worker that is allocated when a job runs. Accepts a value of Standard, G.1X, or G.2X. <ul> <li> For the <code>Standard</code> worker type, each worker provides 4 vCPU, 16 GB of memory and a 50GB disk, and 2 executors per worker. </li> <li> For the <code>G.1X</code> worker type, each worker maps to 1 DPU (4 vCPU, 16 GB of memory, 64 GB disk), and provides 1 executor per worker. We recommend this worker type for memory-intensive jobs. </li> <li> For the <code>G.2X</code> worker type, each worker maps to 2 DPU (8 vCPU, 32 GB of memory, 128 GB disk), and provides 1 executor per worker. We recommend this worker type for memory-intensive jobs. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxCapacity</i> The number of AWS Glue data processing units (DPUs) that can be allocated when this job runs. A DPU is a relative measure of processing power that consists of 4 vCPUs of compute capacity and 16 GB of memory. For more information, see the <a href="https://aws.amazon.com/glue/pricing/">AWS Glue pricing page</a>. Do not set <code>Max Capacity</code> if using <code>WorkerType</code> and <code>NumberOfWorkers</code>. The value that can be allocated for <code>MaxCapacity</code> depends on whether you are running a Python shell job or an Apache Spark ETL job: <ul> <li> When you specify a Python shell job (<code>JobCommand.Name</code>="pythonshell"), you can allocate either 0.0625 or 1 DPU. The default is 0.0625 DPU. </li> <li> When you specify an Apache Spark ETL job (<code>JobCommand.Name</code>="glueetl"), you can allocate from 2 to 100 DPUs. The default is 10 DPUs. This job type cannot have a fractional DPU allocation. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timeout</i> The job timeout in minutes. This is the maximum time that a job run can consume resources before it is terminated and enters <code>TIMEOUT</code> status. The default is 2,880 minutes (48 hours).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedCapacity</i> This field is deprecated. Use <code>MaxCapacity</code> instead. The number of AWS Glue data processing units (DPUs) to allocate to this job. You can allocate from 2 to 100 DPUs; the default is 10. A DPU is a relative measure of processing power that consists of 4 vCPUs of compute capacity and 16 GB of memory. For more information, see the <a href="https://aws.amazon.com/glue/pricing/">AWS Glue pricing page</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxRetries</i> The maximum number of times to retry this job if it fails.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Connections</i> The connections used for this job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Connections</b>: The connections used for this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultArguments</i> The default arguments for this job. You can specify arguments here that your own job-execution script consumes, as well as arguments that AWS Glue itself consumes. For information about how to specify and consume your own Job arguments, see the <a href="https://docs.aws.amazon.com/glue/latest/dg/aws-glue-programming-python-calling.html">Calling AWS Glue APIs in Python</a> topic in the developer guide. For information about the key-value pairs that AWS Glue consumes to set up your job, see the <a href="https://docs.aws.amazon.com/glue/latest/dg/aws-glue-programming-etl-glue-arguments.html">Special Parameters Used by AWS Glue</a> topic in the developer guide.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Command</i> The <code>JobCommand</code> that executes this job (required).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Command</b>: The <code>JobCommand</code> that executes this job (required).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExecutionProperty</i> An <code>ExecutionProperty</code> specifying the maximum number of concurrent runs allowed for this job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ExecutionProperty</b>: An <code>ExecutionProperty</code> specifying the maximum number of concurrent runs allowed for this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Role</i> The name or Amazon Resource Name (ARN) of the IAM role associated with this job (required).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogUri</i> This field is reserved for future use.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> Description of the job being defined.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxConcurrentRuns</i> The maximum number of concurrent runs allowed for the job. The default is 1. An error is returned when this threshold is reached. The maximum value you can specify is controlled by a service limit.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PythonVersion</i> The Python version being used to execute a Python shell job. Allowed values are 2 or 3.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ScriptLocation</i> Specifies the Amazon Simple Storage Service (Amazon S3) path to a script that executes a job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the job command. For an Apache Spark ETL job, this must be <code>glueetl</code>. For a Python shell job, it must be <code>pythonshell</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Connections</i> A list of connections used by the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NotifyDelayAfter</i> After a job run starts, the number of minutes to wait before sending a job run delay notification.<br/>
        :return: JobName:  Returns the name of the updated job definition.<br/>        """
        pass

    def update_partition(self, PartitionInput: PartitionInput, PartitionValueList: List[ValueString], TableName: str, DatabaseName: str, CatalogId: str=None) -> UpdatePartitionResponse:
        """Updates a partition.

        :param self:
        :param PartitionInput:  The new partition object to which to update the partition.:<br/>PartitionValueList:  A list of the values defining the partition.<br/>TableName:  The name of the table where the partition to be updated is located.<br/>DatabaseName:  The name of the catalog database in which the table in question resides.<br/>CatalogId:  The ID of the Data Catalog where the partition to be updated resides. If none is supplied, the AWS account ID is used by default.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastAnalyzedTime</i> The last time at which column statistics were computed for this partition.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> These key-value pairs define partition parameters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageDescriptor</i> Provides information about the physical location where the partition is stored.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StorageDescriptor</b>: Provides information about the physical location where the partition is stored.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastAccessTime</i> The last time at which the partition was accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Values</i> The values of the partition. Although this parameter is not required by the SDK, you must specify this parameter for a valid input.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfBuckets</i> Must be specified if the table contains any dimension columns.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Compressed</i> True if the data in the table is compressed, or False if not.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputFormat</i> The output format: <code>SequenceFileOutputFormat</code> (binary), or <code>IgnoreKeyTextOutputFormat</code>, or a custom format.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InputFormat</i> The input format: <code>SequenceFileInputFormat</code> (binary), or <code>TextInputFormat</code>, or a custom format.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Location</i> The physical location of the table. By default this takes the form of the warehouse location, followed by the database location in the warehouse, followed by the table name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Columns</i> A list of the <code>Columns</code> in the table.<br/>Serialization/deserialization (SerDe) information.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SerdeInfo</b>: Serialization/deserialization (SerDe) information.
        :return:         """
        pass

    def update_table(self, TableInput: TableInput, DatabaseName: str, CatalogId: str=None, SkipArchive: BooleanNullable=None) -> UpdateTableResponse:
        """Updates a metadata table in the Data Catalog.

        :param self:
        :param SkipArchive:  By default, <code>UpdateTable</code> always creates an archived version of the table before updating it. If <code>skipArchive</code> is set to true, however, <code>UpdateTable</code> does not create the archived version.<br/>TableInput:  An updated <code>TableInput</code> object to define the metadata table in the catalog.:<br/>DatabaseName:  The name of the catalog database in which the table resides. For Hive compatibility, this name is entirely lowercase.<br/>CatalogId:  The ID of the Data Catalog where the table resides. If none is supplied, the AWS account ID is used by default.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> These key-value pairs define properties associated with the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableType</i> The type of this table (<code>EXTERNAL_TABLE</code>, <code>VIRTUAL_VIEW</code>, etc.).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ViewExpandedText</i> If the table is a view, the expanded text of the view; otherwise <code>null</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ViewOriginalText</i> If the table is a view, the original text of the view; otherwise <code>null</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PartitionKeys</i> A list of columns by which the table is partitioned. Only primitive types are supported as partition keys. When creating a table used by Athena, and you do not specify any <code>partitionKeys</code>, you must at least set the value of <code>partitionKeys</code> to an empty list. For example:  <code>"PartitionKeys": []</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageDescriptor</i> A storage descriptor containing information about the physical storage of this table.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StorageDescriptor</b>: A storage descriptor containing information about the physical storage of this table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Retention</i> Retention time for this table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastAnalyzedTime</i> Last time column statistics were computed for this table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastAccessTime</i> Last time the table was accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Owner</i> Owner of the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> Description of the table.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Name of the table. For Hive compatibility, this is folded to lowercase when it is stored.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfBuckets</i> Must be specified if the table contains any dimension columns.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Compressed</i> True if the data in the table is compressed, or False if not.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputFormat</i> The output format: <code>SequenceFileOutputFormat</code> (binary), or <code>IgnoreKeyTextOutputFormat</code>, or a custom format.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InputFormat</i> The input format: <code>SequenceFileInputFormat</code> (binary), or <code>TextInputFormat</code>, or a custom format.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Location</i> The physical location of the table. By default this takes the form of the warehouse location, followed by the database location in the warehouse, followed by the table name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Columns</i> A list of the <code>Columns</code> in the table.<br/>Serialization/deserialization (SerDe) information.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SerdeInfo</b>: Serialization/deserialization (SerDe) information.
        :return:         """
        pass

    def update_trigger(self, TriggerUpdate: TriggerUpdate, Name: str) -> UpdateTriggerResponse:
        """Updates a trigger definition.

        :param self:
        :param TriggerUpdate:  The new values with which to update the trigger.:<br/>Name:  The name of the trigger to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Predicate</i> The predicate of this trigger, which defines when it will fire.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Predicate</b>: The predicate of this trigger, which defines when it will fire.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Actions</i> The actions initiated by this trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Schedule</i> A <code>cron</code> expression used to specify the schedule (see <a href="https://docs.aws.amazon.com/glue/latest/dg/monitor-data-warehouse-schedule.html">Time-Based Schedules for Jobs and Crawlers</a>. For example, to run something every day at 12:15 UTC, you would specify: <code>cron(15 12 * * ? *)</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of this trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Reserved for future use.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Conditions</i> A list of the conditions that determine when the trigger will fire.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Logical</i> An optional field if only one condition is listed. If multiple conditions are listed, then this field is required.<br/>
        :return: Trigger:  The resulting trigger definition.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Predicate</i> The predicate of this trigger, which defines when it will fire.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Predicate</b>: The predicate of this trigger, which defines when it will fire.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Actions</i> The actions initiated by this trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Schedule</i> A <code>cron</code> expression used to specify the schedule (see <a href="https://docs.aws.amazon.com/glue/latest/dg/monitor-data-warehouse-schedule.html">Time-Based Schedules for Jobs and Crawlers</a>. For example, to run something every day at 12:15 UTC, you would specify: <code>cron(15 12 * * ? *)</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of this trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The current state of the trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of trigger that this is.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> Reserved for future use.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WorkflowName</i> The name of the workflow associated with the trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the trigger.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Conditions</i> A list of the conditions that determine when the trigger will fire.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Logical</i> An optional field if only one condition is listed. If multiple conditions are listed, then this field is required.<br/>        """
        pass

    def update_user_defined_function(self, FunctionInput: UserDefinedFunctionInput, FunctionName: str, DatabaseName: str, CatalogId: str=None) -> UpdateUserDefinedFunctionResponse:
        """Updates an existing function definition in the Data Catalog.

        :param self:
        :param FunctionInput:  A <code>FunctionInput</code> object that re-defines the function in the Data Catalog.:<br/>FunctionName:  The name of the function.<br/>DatabaseName:  The name of the catalog database where the function to be updated is located.<br/>CatalogId:  The ID of the Data Catalog where the function to be updated is located. If none is supplied, the AWS account ID is used by default.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceUris</i> The resource URIs for the function.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerType</i> The owner type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerName</i> The owner of the function.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClassName</i> The Java class that contains the function code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FunctionName</i> The name of the function.<br/>
        :return:         """
        pass

    def update_workflow(self, Name: str, Description: str=None, DefaultRunProperties: WorkflowRunProperties=None) -> UpdateWorkflowResponse:
        """Updates an existing workflow.

        :param self:
        :param DefaultRunProperties:  A collection of properties to be used as part of each execution of the workflow.<br/>Description:  The description of the workflow.<br/>Name:  Name of the workflow to be updated.<br/>
        :return: Name:  The name of the workflow which was specified in input.<br/>        """
        pass



class Greengrass(BaseClient):
    """AWS IoT Greengrass seamlessly extends AWS onto physical devices so they can act locally on the data they generate, while still using the cloud for management, analytics, and durable storage. AWS IoT Greengrass ensures your devices can respond quickly to local events and operate with intermittent connectivity. AWS IoT Greengrass minimizes the cost of transmitting data to the cloud by allowing you to author AWS Lambda functions that execute locally."""


    class ListBulkDeploymentDetailedReportsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_bulk_deployment_detailed_reports` for valid parameters"""
            pass
    class ListBulkDeploymentsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_bulk_deployments` for valid parameters"""
            pass
    class ListConnectorDefinitionVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_connector_definition_versions` for valid parameters"""
            pass
    class ListConnectorDefinitionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_connector_definitions` for valid parameters"""
            pass
    class ListCoreDefinitionVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_core_definition_versions` for valid parameters"""
            pass
    class ListCoreDefinitionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_core_definitions` for valid parameters"""
            pass
    class ListDeploymentsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_deployments` for valid parameters"""
            pass
    class ListDeviceDefinitionVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_device_definition_versions` for valid parameters"""
            pass
    class ListDeviceDefinitionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_device_definitions` for valid parameters"""
            pass
    class ListFunctionDefinitionVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_function_definition_versions` for valid parameters"""
            pass
    class ListFunctionDefinitionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_function_definitions` for valid parameters"""
            pass
    class ListGroupVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_group_versions` for valid parameters"""
            pass
    class ListGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_groups` for valid parameters"""
            pass
    class ListLoggerDefinitionVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_logger_definition_versions` for valid parameters"""
            pass
    class ListLoggerDefinitionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_logger_definitions` for valid parameters"""
            pass
    class ListResourceDefinitionVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_resource_definition_versions` for valid parameters"""
            pass
    class ListResourceDefinitionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_resource_definitions` for valid parameters"""
            pass
    class ListSubscriptionDefinitionVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_subscription_definition_versions` for valid parameters"""
            pass
    class ListSubscriptionDefinitionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_subscription_definitions` for valid parameters"""
            pass


    class AssociateRoleToGroupRequest(Mapping):
        pass
    
    class AssociateRoleToGroupResponse(Mapping):
        pass
    
    class AssociateServiceRoleToAccountRequest(Mapping):
        pass
    
    class AssociateServiceRoleToAccountResponse(Mapping):
        pass
    
    class CreateConnectorDefinitionRequest(Mapping):
        pass
    
    class CreateConnectorDefinitionResponse(Mapping):
        pass
    
    class ConnectorDefinitionVersion(Mapping):
        pass
    
    class Tags(object):
        pass
    
    class CreateConnectorDefinitionVersionRequest(Mapping):
        pass
    
    class CreateConnectorDefinitionVersionResponse(Mapping):
        pass
    
    class Connector(Mapping):
        pass
    
    class CreateCoreDefinitionRequest(Mapping):
        pass
    
    class CreateCoreDefinitionResponse(Mapping):
        pass
    
    class CoreDefinitionVersion(Mapping):
        pass
    
    class CreateCoreDefinitionVersionRequest(Mapping):
        pass
    
    class CreateCoreDefinitionVersionResponse(Mapping):
        pass
    
    class Core(Mapping):
        pass
    
    class CreateDeploymentRequest(Mapping):
        pass
    
    class CreateDeploymentResponse(Mapping):
        pass
    
    class CreateDeviceDefinitionRequest(Mapping):
        pass
    
    class CreateDeviceDefinitionResponse(Mapping):
        pass
    
    class DeviceDefinitionVersion(Mapping):
        pass
    
    class CreateDeviceDefinitionVersionRequest(Mapping):
        pass
    
    class CreateDeviceDefinitionVersionResponse(Mapping):
        pass
    
    class Device(Mapping):
        pass
    
    class CreateFunctionDefinitionRequest(Mapping):
        pass
    
    class CreateFunctionDefinitionResponse(Mapping):
        pass
    
    class FunctionDefinitionVersion(Mapping):
        pass
    
    class CreateFunctionDefinitionVersionRequest(Mapping):
        pass
    
    class CreateFunctionDefinitionVersionResponse(Mapping):
        pass
    
    class FunctionDefaultConfig(Mapping):
        pass
    
    class Function(Mapping):
        pass
    
    class CreateGroupRequest(Mapping):
        pass
    
    class CreateGroupResponse(Mapping):
        pass
    
    class GroupVersion(Mapping):
        pass
    
    class CreateGroupCertificateAuthorityRequest(Mapping):
        pass
    
    class CreateGroupCertificateAuthorityResponse(Mapping):
        pass
    
    class CreateGroupVersionRequest(Mapping):
        pass
    
    class CreateGroupVersionResponse(Mapping):
        pass
    
    class CreateLoggerDefinitionRequest(Mapping):
        pass
    
    class CreateLoggerDefinitionResponse(Mapping):
        pass
    
    class LoggerDefinitionVersion(Mapping):
        pass
    
    class CreateLoggerDefinitionVersionRequest(Mapping):
        pass
    
    class CreateLoggerDefinitionVersionResponse(Mapping):
        pass
    
    class Logger(Mapping):
        pass
    
    class CreateResourceDefinitionRequest(Mapping):
        pass
    
    class CreateResourceDefinitionResponse(Mapping):
        pass
    
    class ResourceDefinitionVersion(Mapping):
        pass
    
    class CreateResourceDefinitionVersionRequest(Mapping):
        pass
    
    class CreateResourceDefinitionVersionResponse(Mapping):
        pass
    
    class Resource(Mapping):
        pass
    
    class CreateSoftwareUpdateJobRequest(Mapping):
        pass
    
    class CreateSoftwareUpdateJobResponse(Mapping):
        pass
    
    class __string(object):
        pass
    
    class CreateSubscriptionDefinitionRequest(Mapping):
        pass
    
    class CreateSubscriptionDefinitionResponse(Mapping):
        pass
    
    class SubscriptionDefinitionVersion(Mapping):
        pass
    
    class CreateSubscriptionDefinitionVersionRequest(Mapping):
        pass
    
    class CreateSubscriptionDefinitionVersionResponse(Mapping):
        pass
    
    class Subscription(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class __mapOf__string(object):
        pass
    
    class DeleteConnectorDefinitionRequest(Mapping):
        pass
    
    class DeleteConnectorDefinitionResponse(Mapping):
        pass
    
    class DeleteCoreDefinitionRequest(Mapping):
        pass
    
    class DeleteCoreDefinitionResponse(Mapping):
        pass
    
    class DeleteDeviceDefinitionRequest(Mapping):
        pass
    
    class DeleteDeviceDefinitionResponse(Mapping):
        pass
    
    class DeleteFunctionDefinitionRequest(Mapping):
        pass
    
    class DeleteFunctionDefinitionResponse(Mapping):
        pass
    
    class DeleteGroupRequest(Mapping):
        pass
    
    class DeleteGroupResponse(Mapping):
        pass
    
    class DeleteLoggerDefinitionRequest(Mapping):
        pass
    
    class DeleteLoggerDefinitionResponse(Mapping):
        pass
    
    class DeleteResourceDefinitionRequest(Mapping):
        pass
    
    class DeleteResourceDefinitionResponse(Mapping):
        pass
    
    class DeleteSubscriptionDefinitionRequest(Mapping):
        pass
    
    class DeleteSubscriptionDefinitionResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class DisassociateRoleFromGroupRequest(Mapping):
        pass
    
    class DisassociateRoleFromGroupResponse(Mapping):
        pass
    
    class DisassociateServiceRoleFromAccountRequest(Mapping):
        pass
    
    class DisassociateServiceRoleFromAccountResponse(Mapping):
        pass
    
    class GetAssociatedRoleRequest(Mapping):
        pass
    
    class GetAssociatedRoleResponse(Mapping):
        pass
    
    class GetBulkDeploymentStatusRequest(Mapping):
        pass
    
    class GetBulkDeploymentStatusResponse(Mapping):
        pass
    
    class GetConnectivityInfoRequest(Mapping):
        pass
    
    class GetConnectivityInfoResponse(Mapping):
        pass
    
    class GetConnectorDefinitionRequest(Mapping):
        pass
    
    class GetConnectorDefinitionResponse(Mapping):
        pass
    
    class GetConnectorDefinitionVersionRequest(Mapping):
        pass
    
    class GetConnectorDefinitionVersionResponse(Mapping):
        pass
    
    class GetCoreDefinitionRequest(Mapping):
        pass
    
    class GetCoreDefinitionResponse(Mapping):
        pass
    
    class GetCoreDefinitionVersionRequest(Mapping):
        pass
    
    class GetCoreDefinitionVersionResponse(Mapping):
        pass
    
    class GetDeploymentStatusRequest(Mapping):
        pass
    
    class GetDeploymentStatusResponse(Mapping):
        pass
    
    class GetDeviceDefinitionRequest(Mapping):
        pass
    
    class GetDeviceDefinitionResponse(Mapping):
        pass
    
    class GetDeviceDefinitionVersionRequest(Mapping):
        pass
    
    class GetDeviceDefinitionVersionResponse(Mapping):
        pass
    
    class GetFunctionDefinitionRequest(Mapping):
        pass
    
    class GetFunctionDefinitionResponse(Mapping):
        pass
    
    class GetFunctionDefinitionVersionRequest(Mapping):
        pass
    
    class GetFunctionDefinitionVersionResponse(Mapping):
        pass
    
    class GetGroupRequest(Mapping):
        pass
    
    class GetGroupResponse(Mapping):
        pass
    
    class GetGroupCertificateAuthorityRequest(Mapping):
        pass
    
    class GetGroupCertificateAuthorityResponse(Mapping):
        pass
    
    class GetGroupCertificateConfigurationRequest(Mapping):
        pass
    
    class GetGroupCertificateConfigurationResponse(Mapping):
        pass
    
    class GetGroupVersionRequest(Mapping):
        pass
    
    class GetGroupVersionResponse(Mapping):
        pass
    
    class GetLoggerDefinitionRequest(Mapping):
        pass
    
    class GetLoggerDefinitionResponse(Mapping):
        pass
    
    class GetLoggerDefinitionVersionRequest(Mapping):
        pass
    
    class GetLoggerDefinitionVersionResponse(Mapping):
        pass
    
    class GetResourceDefinitionRequest(Mapping):
        pass
    
    class GetResourceDefinitionResponse(Mapping):
        pass
    
    class GetResourceDefinitionVersionRequest(Mapping):
        pass
    
    class GetResourceDefinitionVersionResponse(Mapping):
        pass
    
    class GetServiceRoleForAccountRequest(Mapping):
        pass
    
    class GetServiceRoleForAccountResponse(Mapping):
        pass
    
    class GetSubscriptionDefinitionRequest(Mapping):
        pass
    
    class GetSubscriptionDefinitionResponse(Mapping):
        pass
    
    class GetSubscriptionDefinitionVersionRequest(Mapping):
        pass
    
    class GetSubscriptionDefinitionVersionResponse(Mapping):
        pass
    
    class ListBulkDeploymentDetailedReportsRequest(Mapping):
        pass
    
    class ListBulkDeploymentDetailedReportsResponse(Mapping):
        pass
    
    class ListBulkDeploymentsRequest(Mapping):
        pass
    
    class ListBulkDeploymentsResponse(Mapping):
        pass
    
    class ListConnectorDefinitionVersionsRequest(Mapping):
        pass
    
    class ListConnectorDefinitionVersionsResponse(Mapping):
        pass
    
    class ListConnectorDefinitionsRequest(Mapping):
        pass
    
    class ListConnectorDefinitionsResponse(Mapping):
        pass
    
    class ListCoreDefinitionVersionsRequest(Mapping):
        pass
    
    class ListCoreDefinitionVersionsResponse(Mapping):
        pass
    
    class ListCoreDefinitionsRequest(Mapping):
        pass
    
    class ListCoreDefinitionsResponse(Mapping):
        pass
    
    class ListDeploymentsRequest(Mapping):
        pass
    
    class ListDeploymentsResponse(Mapping):
        pass
    
    class ListDeviceDefinitionVersionsRequest(Mapping):
        pass
    
    class ListDeviceDefinitionVersionsResponse(Mapping):
        pass
    
    class ListDeviceDefinitionsRequest(Mapping):
        pass
    
    class ListDeviceDefinitionsResponse(Mapping):
        pass
    
    class ListFunctionDefinitionVersionsRequest(Mapping):
        pass
    
    class ListFunctionDefinitionVersionsResponse(Mapping):
        pass
    
    class ListFunctionDefinitionsRequest(Mapping):
        pass
    
    class ListFunctionDefinitionsResponse(Mapping):
        pass
    
    class ListGroupCertificateAuthoritiesRequest(Mapping):
        pass
    
    class ListGroupCertificateAuthoritiesResponse(Mapping):
        pass
    
    class ListGroupVersionsRequest(Mapping):
        pass
    
    class ListGroupVersionsResponse(Mapping):
        pass
    
    class ListGroupsRequest(Mapping):
        pass
    
    class ListGroupsResponse(Mapping):
        pass
    
    class ListLoggerDefinitionVersionsRequest(Mapping):
        pass
    
    class ListLoggerDefinitionVersionsResponse(Mapping):
        pass
    
    class ListLoggerDefinitionsRequest(Mapping):
        pass
    
    class ListLoggerDefinitionsResponse(Mapping):
        pass
    
    class ListResourceDefinitionVersionsRequest(Mapping):
        pass
    
    class ListResourceDefinitionVersionsResponse(Mapping):
        pass
    
    class ListResourceDefinitionsRequest(Mapping):
        pass
    
    class ListResourceDefinitionsResponse(Mapping):
        pass
    
    class ListSubscriptionDefinitionVersionsRequest(Mapping):
        pass
    
    class ListSubscriptionDefinitionVersionsResponse(Mapping):
        pass
    
    class ListSubscriptionDefinitionsRequest(Mapping):
        pass
    
    class ListSubscriptionDefinitionsResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class ResetDeploymentsRequest(Mapping):
        pass
    
    class ResetDeploymentsResponse(Mapping):
        pass
    
    class __boolean(object):
        pass
    
    class StartBulkDeploymentRequest(Mapping):
        pass
    
    class StartBulkDeploymentResponse(Mapping):
        pass
    
    class StopBulkDeploymentRequest(Mapping):
        pass
    
    class StopBulkDeploymentResponse(Mapping):
        pass
    
    class UpdateConnectivityInfoRequest(Mapping):
        pass
    
    class UpdateConnectivityInfoResponse(Mapping):
        pass
    
    class ConnectivityInfo(Mapping):
        pass
    
    class UpdateConnectorDefinitionRequest(Mapping):
        pass
    
    class UpdateConnectorDefinitionResponse(Mapping):
        pass
    
    class UpdateCoreDefinitionRequest(Mapping):
        pass
    
    class UpdateCoreDefinitionResponse(Mapping):
        pass
    
    class UpdateDeviceDefinitionRequest(Mapping):
        pass
    
    class UpdateDeviceDefinitionResponse(Mapping):
        pass
    
    class UpdateFunctionDefinitionRequest(Mapping):
        pass
    
    class UpdateFunctionDefinitionResponse(Mapping):
        pass
    
    class UpdateGroupRequest(Mapping):
        pass
    
    class UpdateGroupResponse(Mapping):
        pass
    
    class UpdateGroupCertificateConfigurationRequest(Mapping):
        pass
    
    class UpdateGroupCertificateConfigurationResponse(Mapping):
        pass
    
    class UpdateLoggerDefinitionRequest(Mapping):
        pass
    
    class UpdateLoggerDefinitionResponse(Mapping):
        pass
    
    class UpdateResourceDefinitionRequest(Mapping):
        pass
    
    class UpdateResourceDefinitionResponse(Mapping):
        pass
    
    class UpdateSubscriptionDefinitionRequest(Mapping):
        pass
    
    class UpdateSubscriptionDefinitionResponse(Mapping):
        pass
    
    def associate_role_to_group(self, GroupId: str, RoleArn: str=None) -> AssociateRoleToGroupResponse:
        """Associates a role with a group. Your Greengrass core will use the role to access AWS cloud services. The role's permissions should allow Greengrass core Lambda functions to perform actions against the cloud.

        :param self:
        :param RoleArn:  The ARN of the role you wish to associate with this group.<br/>GroupId:  The ID of the Greengrass group.<br/>
        :return: AssociatedAt:  The time, in milliseconds since the epoch, when the role ARN was associated with the group.<br/>        """
        pass

    def associate_service_role_to_account(self, RoleArn: str=None) -> AssociateServiceRoleToAccountResponse:
        """Associates a role with your account. AWS IoT Greengrass will use the role to access your Lambda functions and AWS IoT resources. This is necessary for deployments to succeed. The role must have at least minimum permissions in the policy ''AWSGreengrassResourceAccessRolePolicy''.

        :param self:
        :param RoleArn:  The ARN of the service role you wish to associate with your account.<br/>
        :return: AssociatedAt:  The time when the service role was associated with the account.<br/>        """
        pass

    def create_connector_definition(self, AmznClientToken: str=None, InitialVersion: ConnectorDefinitionVersion=None, Name: str=None, tags: Tags=None) -> CreateConnectorDefinitionResponse:
        """Creates a connector definition. You may provide the initial version of the connector definition now or use ''CreateConnectorDefinitionVersion'' at a later time.

        :param self:
        :param tags:  Tag(s) to add to the new resource<br/>Name:  The name of the connector definition.<br/>InitialVersion:  Information about the initial version of the connector definition.:<br/>AmznClientToken:  A client token used to correlate requests and responses.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Connectors</i> A list of references to connectors in this version, with their corresponding configuration settings.<br/>
        :return: Name:  The name of the definition.<br/>LatestVersionArn:  The ARN of the latest version of the definition.<br/>LatestVersion:  The latest version of the definition.<br/>LastUpdatedTimestamp:  The time, in milliseconds since the epoch, when the definition was last updated.<br/>Id:  The ID of the definition.<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the definition was created.<br/>Arn:  The ARN of the definition.<br/>        """
        pass

    def create_connector_definition_version(self, ConnectorDefinitionId: str, AmznClientToken: str=None, Connectors: List[Connector]=None) -> CreateConnectorDefinitionVersionResponse:
        """Creates a version of a connector definition which has already been defined.

        :param self:
        :param Connectors:  A list of references to connectors in this version, with their corresponding configuration settings.<br/>ConnectorDefinitionId:  The ID of the connector definition.<br/>AmznClientToken:  A client token used to correlate requests and responses.<br/>
        :return: Version:  The unique ID of the version.<br/>Id:  The ID of the version.<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the version was created.<br/>Arn:  The ARN of the version.<br/>        """
        pass

    def create_core_definition(self, AmznClientToken: str=None, InitialVersion: CoreDefinitionVersion=None, Name: str=None, tags: Tags=None) -> CreateCoreDefinitionResponse:
        """Creates a core definition. You may provide the initial version of the core definition now or use ''CreateCoreDefinitionVersion'' at a later time. Greengrass groups must each contain exactly one Greengrass core.

        :param self:
        :param tags:  Tag(s) to add to the new resource<br/>Name:  The name of the core definition.<br/>InitialVersion:  Information about the initial version of the core definition.:<br/>AmznClientToken:  A client token used to correlate requests and responses.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Cores</i> A list of cores in the core definition version.<br/>
        :return: Name:  The name of the definition.<br/>LatestVersionArn:  The ARN of the latest version of the definition.<br/>LatestVersion:  The latest version of the definition.<br/>LastUpdatedTimestamp:  The time, in milliseconds since the epoch, when the definition was last updated.<br/>Id:  The ID of the definition.<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the definition was created.<br/>Arn:  The ARN of the definition.<br/>        """
        pass

    def create_core_definition_version(self, CoreDefinitionId: str, AmznClientToken: str=None, Cores: List[Core]=None) -> CreateCoreDefinitionVersionResponse:
        """Creates a version of a core definition that has already been defined. Greengrass groups must each contain exactly one Greengrass core.

        :param self:
        :param Cores:  A list of cores in the core definition version.<br/>CoreDefinitionId:  The ID of the core definition.<br/>AmznClientToken:  A client token used to correlate requests and responses.<br/>
        :return: Version:  The unique ID of the version.<br/>Id:  The ID of the version.<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the version was created.<br/>Arn:  The ARN of the version.<br/>        """
        pass

    def create_deployment(self, GroupId: str, AmznClientToken: str=None, DeploymentId: str=None, DeploymentType: str=None, GroupVersionId: str=None) -> CreateDeploymentResponse:
        """Creates a deployment. ''CreateDeployment'' requests are idempotent with respect to the ''X-Amzn-Client-Token'' token and the request parameters.

        :param self:
        :param GroupVersionId:  The ID of the group version to be deployed.<br/>GroupId:  The ID of the Greengrass group.<br/>DeploymentType:  The type of deployment. When used for ''CreateDeployment'', only ''NewDeployment'' and ''Redeployment'' are valid.<br/>DeploymentId:  The ID of the deployment if you wish to redeploy a previous deployment.<br/>AmznClientToken:  A client token used to correlate requests and responses.<br/>
        :return: DeploymentId:  The ID of the deployment.<br/>DeploymentArn:  The ARN of the deployment.<br/>        """
        pass

    def create_device_definition(self, AmznClientToken: str=None, InitialVersion: DeviceDefinitionVersion=None, Name: str=None, tags: Tags=None) -> CreateDeviceDefinitionResponse:
        """Creates a device definition. You may provide the initial version of the device definition now or use ''CreateDeviceDefinitionVersion'' at a later time.

        :param self:
        :param tags:  Tag(s) to add to the new resource<br/>Name:  The name of the device definition.<br/>InitialVersion:  Information about the initial version of the device definition.:<br/>AmznClientToken:  A client token used to correlate requests and responses.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Devices</i> A list of devices in the definition version.<br/>
        :return: Name:  The name of the definition.<br/>LatestVersionArn:  The ARN of the latest version of the definition.<br/>LatestVersion:  The latest version of the definition.<br/>LastUpdatedTimestamp:  The time, in milliseconds since the epoch, when the definition was last updated.<br/>Id:  The ID of the definition.<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the definition was created.<br/>Arn:  The ARN of the definition.<br/>        """
        pass

    def create_device_definition_version(self, DeviceDefinitionId: str, AmznClientToken: str=None, Devices: List[Device]=None) -> CreateDeviceDefinitionVersionResponse:
        """Creates a version of a device definition that has already been defined.

        :param self:
        :param Devices:  A list of devices in the definition version.<br/>DeviceDefinitionId:  The ID of the device definition.<br/>AmznClientToken:  A client token used to correlate requests and responses.<br/>
        :return: Version:  The unique ID of the version.<br/>Id:  The ID of the version.<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the version was created.<br/>Arn:  The ARN of the version.<br/>        """
        pass

    def create_function_definition(self, AmznClientToken: str=None, InitialVersion: FunctionDefinitionVersion=None, Name: str=None, tags: Tags=None) -> CreateFunctionDefinitionResponse:
        """Creates a Lambda function definition which contains a list of Lambda functions and their configurations to be used in a group. You can create an initial version of the definition by providing a list of Lambda functions and their configurations now, or use ''CreateFunctionDefinitionVersion'' later.

        :param self:
        :param tags:  Tag(s) to add to the new resource<br/>Name:  The name of the function definition.<br/>InitialVersion:  Information about the initial version of the function definition.:<br/>AmznClientToken:  A client token used to correlate requests and responses.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Functions</i> A list of Lambda functions in this function definition version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultConfig</i> The default configuration that applies to all Lambda functions in this function definition version. Individual Lambda functions can override these settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DefaultConfig</b>: The default configuration that applies to all Lambda functions in this function definition version. Individual Lambda functions can override these settings.<br/>Configuration information that specifies how a Lambda function runs. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Execution</b>: Configuration information that specifies how a Lambda function runs. 
        :return: Name:  The name of the definition.<br/>LatestVersionArn:  The ARN of the latest version of the definition.<br/>LatestVersion:  The latest version of the definition.<br/>LastUpdatedTimestamp:  The time, in milliseconds since the epoch, when the definition was last updated.<br/>Id:  The ID of the definition.<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the definition was created.<br/>Arn:  The ARN of the definition.<br/>        """
        pass

    def create_function_definition_version(self, FunctionDefinitionId: str, AmznClientToken: str=None, DefaultConfig: FunctionDefaultConfig=None, Functions: List[Function]=None) -> CreateFunctionDefinitionVersionResponse:
        """Creates a version of a Lambda function definition that has already been defined.

        :param self:
        :param Functions:  A list of Lambda functions in this function definition version.<br/>FunctionDefinitionId:  The ID of the Lambda function definition.<br/>DefaultConfig:  The default configuration that applies to all Lambda functions in this function definition version. Individual Lambda functions can override these settings.:<br/>AmznClientToken:  A client token used to correlate requests and responses.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Execution</i> Configuration information that specifies how a Lambda function runs. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Execution</b>: Configuration information that specifies how a Lambda function runs. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IsolationMode</i> Specifies whether the Lambda function runs in a Greengrass container (default) or without containerization. Unless your scenario requires that you run without containerization, we recommend that you run in a Greengrass container. Omit this value to run the Lambda function with the default containerization for the group.<br/>Specifies the user and group whose permissions are used when running the Lambda function. You can specify one or both values to override the default values. We recommend that you avoid running as root unless absolutely necessary to minimize the risk of unintended changes or malicious attacks. To run as root, you must set ''IsolationMode'' to ''NoContainer'' and update config.json in ''greengrass-root/config'' to set ''allowFunctionsToRunAsRoot'' to ''yes''.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RunAs</b>: Specifies the user and group whose permissions are used when running the Lambda function. You can specify one or both values to override the default values. We recommend that you avoid running as root unless absolutely necessary to minimize the risk of unintended changes or malicious attacks. To run as root, you must set ''IsolationMode'' to ''NoContainer'' and update config.json in ''greengrass-root/config'' to set ''allowFunctionsToRunAsRoot'' to ''yes''.
        :return: Version:  The unique ID of the version.<br/>Id:  The ID of the version.<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the version was created.<br/>Arn:  The ARN of the version.<br/>        """
        pass

    def create_group(self, AmznClientToken: str=None, InitialVersion: GroupVersion=None, Name: str=None, tags: Tags=None) -> CreateGroupResponse:
        """Creates a group. You may provide the initial version of the group or use ''CreateGroupVersion'' at a later time. Tip: You can use the ''gg_group_setup'' package (https://github.com/awslabs/aws-greengrass-group-setup) as a library or command-line application to create and deploy Greengrass groups.

        :param self:
        :param tags:  Tag(s) to add to the new resource<br/>Name:  The name of the group.<br/>InitialVersion:  Information about the initial version of the group.:<br/>AmznClientToken:  A client token used to correlate requests and responses.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubscriptionDefinitionVersionArn</i> The ARN of the subscription definition version for this group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceDefinitionVersionArn</i> The ARN of the resource definition version for this group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LoggerDefinitionVersionArn</i> The ARN of the logger definition version for this group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FunctionDefinitionVersionArn</i> The ARN of the function definition version for this group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceDefinitionVersionArn</i> The ARN of the device definition version for this group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CoreDefinitionVersionArn</i> The ARN of the core definition version for this group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectorDefinitionVersionArn</i> The ARN of the connector definition version for this group.<br/>
        :return: Name:  The name of the definition.<br/>LatestVersionArn:  The ARN of the latest version of the definition.<br/>LatestVersion:  The latest version of the definition.<br/>LastUpdatedTimestamp:  The time, in milliseconds since the epoch, when the definition was last updated.<br/>Id:  The ID of the definition.<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the definition was created.<br/>Arn:  The ARN of the definition.<br/>        """
        pass

    def create_group_certificate_authority(self, GroupId: str, AmznClientToken: str=None) -> CreateGroupCertificateAuthorityResponse:
        """Creates a CA for the group. If a CA already exists, it will rotate the existing CA.

        :param self:
        :param GroupId:  The ID of the Greengrass group.<br/>AmznClientToken:  A client token used to correlate requests and responses.<br/>
        :return: GroupCertificateAuthorityArn:  The ARN of the group certificate authority.<br/>        """
        pass

    def create_group_version(self, GroupId: str, AmznClientToken: str=None, ConnectorDefinitionVersionArn: str=None, CoreDefinitionVersionArn: str=None, DeviceDefinitionVersionArn: str=None, FunctionDefinitionVersionArn: str=None, LoggerDefinitionVersionArn: str=None, ResourceDefinitionVersionArn: str=None, SubscriptionDefinitionVersionArn: str=None) -> CreateGroupVersionResponse:
        """Creates a version of a group which has already been defined.

        :param self:
        :param SubscriptionDefinitionVersionArn:  The ARN of the subscription definition version for this group.<br/>ResourceDefinitionVersionArn:  The ARN of the resource definition version for this group.<br/>LoggerDefinitionVersionArn:  The ARN of the logger definition version for this group.<br/>GroupId:  The ID of the Greengrass group.<br/>FunctionDefinitionVersionArn:  The ARN of the function definition version for this group.<br/>DeviceDefinitionVersionArn:  The ARN of the device definition version for this group.<br/>CoreDefinitionVersionArn:  The ARN of the core definition version for this group.<br/>ConnectorDefinitionVersionArn:  The ARN of the connector definition version for this group.<br/>AmznClientToken:  A client token used to correlate requests and responses.<br/>
        :return: Version:  The unique ID of the version.<br/>Id:  The ID of the version.<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the version was created.<br/>Arn:  The ARN of the version.<br/>        """
        pass

    def create_logger_definition(self, AmznClientToken: str=None, InitialVersion: LoggerDefinitionVersion=None, Name: str=None, tags: Tags=None) -> CreateLoggerDefinitionResponse:
        """Creates a logger definition. You may provide the initial version of the logger definition now or use ''CreateLoggerDefinitionVersion'' at a later time.

        :param self:
        :param tags:  Tag(s) to add to the new resource<br/>Name:  The name of the logger definition.<br/>InitialVersion:  Information about the initial version of the logger definition.:<br/>AmznClientToken:  A client token used to correlate requests and responses.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Loggers</i> A list of loggers.<br/>
        :return: Name:  The name of the definition.<br/>LatestVersionArn:  The ARN of the latest version of the definition.<br/>LatestVersion:  The latest version of the definition.<br/>LastUpdatedTimestamp:  The time, in milliseconds since the epoch, when the definition was last updated.<br/>Id:  The ID of the definition.<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the definition was created.<br/>Arn:  The ARN of the definition.<br/>        """
        pass

    def create_logger_definition_version(self, LoggerDefinitionId: str, AmznClientToken: str=None, Loggers: List[Logger]=None) -> CreateLoggerDefinitionVersionResponse:
        """Creates a version of a logger definition that has already been defined.

        :param self:
        :param Loggers:  A list of loggers.<br/>LoggerDefinitionId:  The ID of the logger definition.<br/>AmznClientToken:  A client token used to correlate requests and responses.<br/>
        :return: Version:  The unique ID of the version.<br/>Id:  The ID of the version.<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the version was created.<br/>Arn:  The ARN of the version.<br/>        """
        pass

    def create_resource_definition(self, AmznClientToken: str=None, InitialVersion: ResourceDefinitionVersion=None, Name: str=None, tags: Tags=None) -> CreateResourceDefinitionResponse:
        """Creates a resource definition which contains a list of resources to be used in a group. You can create an initial version of the definition by providing a list of resources now, or use ''CreateResourceDefinitionVersion'' later.

        :param self:
        :param tags:  Tag(s) to add to the new resource<br/>Name:  The name of the resource definition.<br/>InitialVersion:  Information about the initial version of the resource definition.:<br/>AmznClientToken:  A client token used to correlate requests and responses.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Resources</i> A list of resources.<br/>
        :return: Name:  The name of the definition.<br/>LatestVersionArn:  The ARN of the latest version of the definition.<br/>LatestVersion:  The latest version of the definition.<br/>LastUpdatedTimestamp:  The time, in milliseconds since the epoch, when the definition was last updated.<br/>Id:  The ID of the definition.<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the definition was created.<br/>Arn:  The ARN of the definition.<br/>        """
        pass

    def create_resource_definition_version(self, ResourceDefinitionId: str, AmznClientToken: str=None, Resources: List[Resource]=None) -> CreateResourceDefinitionVersionResponse:
        """Creates a version of a resource definition that has already been defined.

        :param self:
        :param Resources:  A list of resources.<br/>ResourceDefinitionId:  The ID of the resource definition.<br/>AmznClientToken:  A client token used to correlate requests and responses.<br/>
        :return: Version:  The unique ID of the version.<br/>Id:  The ID of the version.<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the version was created.<br/>Arn:  The ARN of the version.<br/>        """
        pass

    def create_software_update_job(self, AmznClientToken: str=None, S3UrlSignerRole: str=None, SoftwareToUpdate: str=None, UpdateAgentLogLevel: str=None, UpdateTargets: List[__string]=None, UpdateTargetsArchitecture: str=None, UpdateTargetsOperatingSystem: str=None) -> CreateSoftwareUpdateJobResponse:
        """Creates a software update for a core or group of cores (specified as an IoT thing group.) Use this to update the OTA Agent as well as the Greengrass core software. It makes use of the IoT Jobs feature which provides additional commands to manage a Greengrass core software update job.

        :param self:
        :param UpdateTargetsOperatingSystem:  The operating system of the cores which are the targets of an update.<br/>UpdateTargetsArchitecture:  The architecture of the cores which are the targets of an update.<br/>UpdateTargets:  The ARNs of the targets (IoT things or IoT thing groups) that this update will be applied to.<br/>UpdateAgentLogLevel:  The minimum level of log statements that should be logged by the OTA Agent during an update.<br/>SoftwareToUpdate:  The piece of software on the Greengrass core that will be updated.<br/>S3UrlSignerRole:  The IAM Role that Greengrass will use to create pre-signed URLs pointing towards the update artifact.<br/>AmznClientToken:  A client token used to correlate requests and responses.<br/>
        :return: IotJobId:  The IoT Job Id corresponding to this update.<br/>IotJobArn:  The IoT Job ARN corresponding to this update.<br/>        """
        pass

    def create_subscription_definition(self, AmznClientToken: str=None, InitialVersion: SubscriptionDefinitionVersion=None, Name: str=None, tags: Tags=None) -> CreateSubscriptionDefinitionResponse:
        """Creates a subscription definition. You may provide the initial version of the subscription definition now or use ''CreateSubscriptionDefinitionVersion'' at a later time.

        :param self:
        :param tags:  Tag(s) to add to the new resource<br/>Name:  The name of the subscription definition.<br/>InitialVersion:  Information about the initial version of the subscription definition.:<br/>AmznClientToken:  A client token used to correlate requests and responses.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subscriptions</i> A list of subscriptions.<br/>
        :return: Name:  The name of the definition.<br/>LatestVersionArn:  The ARN of the latest version of the definition.<br/>LatestVersion:  The latest version of the definition.<br/>LastUpdatedTimestamp:  The time, in milliseconds since the epoch, when the definition was last updated.<br/>Id:  The ID of the definition.<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the definition was created.<br/>Arn:  The ARN of the definition.<br/>        """
        pass

    def create_subscription_definition_version(self, SubscriptionDefinitionId: str, AmznClientToken: str=None, Subscriptions: List[Subscription]=None) -> CreateSubscriptionDefinitionVersionResponse:
        """Creates a version of a subscription definition which has already been defined.

        :param self:
        :param Subscriptions:  A list of subscriptions.<br/>SubscriptionDefinitionId:  The ID of the subscription definition.<br/>AmznClientToken:  A client token used to correlate requests and responses.<br/>
        :return: Version:  The unique ID of the version.<br/>Id:  The ID of the version.<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the version was created.<br/>Arn:  The ARN of the version.<br/>        """
        pass

    def tag_resource(self, tags: __mapOf__string, ResourceArn: str):
        """Add tags to a resource.

        :param self:
        :param tags:  A map of the key-value pairs for the resource tag.<br/>ResourceArn:  The Amazon Resource Name (ARN) of the resource.<br/>
        :return:         """
        pass

    def delete_connector_definition(self, ConnectorDefinitionId: str) -> DeleteConnectorDefinitionResponse:
        """Deletes a connector definition.

        :param self:
        :param ConnectorDefinitionId:  The ID of the connector definition.<br/>
        :return:         """
        pass

    def delete_core_definition(self, CoreDefinitionId: str) -> DeleteCoreDefinitionResponse:
        """Deletes a core definition.

        :param self:
        :param CoreDefinitionId:  The ID of the core definition.<br/>
        :return:         """
        pass

    def delete_device_definition(self, DeviceDefinitionId: str) -> DeleteDeviceDefinitionResponse:
        """Deletes a device definition.

        :param self:
        :param DeviceDefinitionId:  The ID of the device definition.<br/>
        :return:         """
        pass

    def delete_function_definition(self, FunctionDefinitionId: str) -> DeleteFunctionDefinitionResponse:
        """Deletes a Lambda function definition.

        :param self:
        :param FunctionDefinitionId:  The ID of the Lambda function definition.<br/>
        :return:         """
        pass

    def delete_group(self, GroupId: str) -> DeleteGroupResponse:
        """Deletes a group.

        :param self:
        :param GroupId:  The ID of the Greengrass group.<br/>
        :return:         """
        pass

    def delete_logger_definition(self, LoggerDefinitionId: str) -> DeleteLoggerDefinitionResponse:
        """Deletes a logger definition.

        :param self:
        :param LoggerDefinitionId:  The ID of the logger definition.<br/>
        :return:         """
        pass

    def delete_resource_definition(self, ResourceDefinitionId: str) -> DeleteResourceDefinitionResponse:
        """Deletes a resource definition.

        :param self:
        :param ResourceDefinitionId:  The ID of the resource definition.<br/>
        :return:         """
        pass

    def delete_subscription_definition(self, SubscriptionDefinitionId: str) -> DeleteSubscriptionDefinitionResponse:
        """Deletes a subscription definition.

        :param self:
        :param SubscriptionDefinitionId:  The ID of the subscription definition.<br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[__string], ResourceArn: str):
        """Remove tags with specified keys from a resource.

        :param self:
        :param TagKeys:  A list of the keys to remove from the resource tags.<br/>ResourceArn:  The Amazon Resource Name (ARN) of the resource.<br/>
        :return:         """
        pass

    def disassociate_role_from_group(self, GroupId: str) -> DisassociateRoleFromGroupResponse:
        """Disassociates the role from a group.

        :param self:
        :param GroupId:  The ID of the Greengrass group.<br/>
        :return: DisassociatedAt:  The time, in milliseconds since the epoch, when the role was disassociated from the group.<br/>        """
        pass

    def disassociate_service_role_from_account(self) -> DisassociateServiceRoleFromAccountResponse:
        """Disassociates the service role from your account. Without a service role, deployments will not work.

        :param self:
        :param 
        :return: DisassociatedAt:  The time when the service role was disassociated from the account.<br/>        """
        pass

    def get_associated_role(self, GroupId: str) -> GetAssociatedRoleResponse:
        """Retrieves the role associated with a particular group.

        :param self:
        :param GroupId:  The ID of the Greengrass group.<br/>
        :return: RoleArn:  The ARN of the role that is associated with the group.<br/>AssociatedAt:  The time when the role was associated with the group.<br/>        """
        pass

    def get_bulk_deployment_status(self, BulkDeploymentId: str) -> GetBulkDeploymentStatusResponse:
        """Returns the status of a bulk deployment.

        :param self:
        :param BulkDeploymentId:  The ID of the bulk deployment.<br/>
        :return: tags:  The tags for the definition.<br/>ErrorMessage:  Error message<br/>ErrorDetails:  Error details<br/>CreatedAt:  The time, in ISO format, when the deployment was created.<br/>BulkDeploymentStatus:  The status of the bulk deployment.<br/>BulkDeploymentMetrics:  Relevant metrics on input records processed during bulk deployment.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RetryAttempts</i> The total number of deployment attempts that returned a retryable error. For example, a retry is triggered if the attempt to deploy a group returns a throttling error. ''StartBulkDeployment'' retries a group deployment up to five times.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordsProcessed</i> The total number of group records from the input file that have been processed so far, or attempted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InvalidInputRecords</i> The total number of records that returned a non-retryable error. For example, this can occur if a group record from the input file uses an invalid format or specifies a nonexistent group version, or if the execution role doesn't grant permission to deploy a group or group version.<br/>        """
        pass

    def get_connectivity_info(self, ThingName: str) -> GetConnectivityInfoResponse:
        """Retrieves the connectivity information for a core.

        :param self:
        :param ThingName:  The thing name.<br/>
        :return: Message:  A message about the connectivity info request.<br/>ConnectivityInfo:  Connectivity info list.<br/>        """
        pass

    def get_connector_definition(self, ConnectorDefinitionId: str) -> GetConnectorDefinitionResponse:
        """Retrieves information about a connector definition.

        :param self:
        :param ConnectorDefinitionId:  The ID of the connector definition.<br/>
        :return: tags:  The tags for the definition.<br/>Name:  The name of the definition.<br/>LatestVersionArn:  The ARN of the latest version of the definition.<br/>LatestVersion:  The latest version of the definition.<br/>LastUpdatedTimestamp:  The time, in milliseconds since the epoch, when the definition was last updated.<br/>Id:  The ID of the definition.<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the definition was created.<br/>Arn:  The ARN of the definition.<br/>        """
        pass

    def get_connector_definition_version(self, ConnectorDefinitionVersionId: str, ConnectorDefinitionId: str, NextToken: str=None) -> GetConnectorDefinitionVersionResponse:
        """Retrieves information about a connector definition version, including the connectors that the version contains. Connectors are prebuilt modules that interact with local infrastructure, device protocols, AWS, and other cloud services.

        :param self:
        :param NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>ConnectorDefinitionVersionId:  The ID of the connector definition version.<br/>ConnectorDefinitionId:  The ID of the connector definition.<br/>
        :return: Version:  The version of the connector definition version.<br/>NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>Id:  The ID of the connector definition version.<br/>Definition:  Information about the connector definition version.:<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the connector definition version was created.<br/>Arn:  The ARN of the connector definition version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Connectors</i> A list of references to connectors in this version, with their corresponding configuration settings.<br/>        """
        pass

    def get_core_definition(self, CoreDefinitionId: str) -> GetCoreDefinitionResponse:
        """Retrieves information about a core definition version.

        :param self:
        :param CoreDefinitionId:  The ID of the core definition.<br/>
        :return: tags:  The tags for the definition.<br/>Name:  The name of the definition.<br/>LatestVersionArn:  The ARN of the latest version of the definition.<br/>LatestVersion:  The latest version of the definition.<br/>LastUpdatedTimestamp:  The time, in milliseconds since the epoch, when the definition was last updated.<br/>Id:  The ID of the definition.<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the definition was created.<br/>Arn:  The ARN of the definition.<br/>        """
        pass

    def get_core_definition_version(self, CoreDefinitionVersionId: str, CoreDefinitionId: str) -> GetCoreDefinitionVersionResponse:
        """Retrieves information about a core definition version.

        :param self:
        :param CoreDefinitionVersionId:  The ID of the core definition version.<br/>CoreDefinitionId:  The ID of the core definition.<br/>
        :return: Version:  The version of the core definition version.<br/>NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>Id:  The ID of the core definition version.<br/>Definition:  Information about the core definition version.:<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the core definition version was created.<br/>Arn:  The ARN of the core definition version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Cores</i> A list of cores in the core definition version.<br/>        """
        pass

    def get_deployment_status(self, GroupId: str, DeploymentId: str) -> GetDeploymentStatusResponse:
        """Returns the status of a deployment.

        :param self:
        :param GroupId:  The ID of the Greengrass group.<br/>DeploymentId:  The ID of the deployment.<br/>
        :return: UpdatedAt:  The time, in milliseconds since the epoch, when the deployment status was updated.<br/>ErrorMessage:  Error message<br/>ErrorDetails:  Error details<br/>DeploymentType:  The type of the deployment.<br/>DeploymentStatus:  The status of the deployment: ''InProgress'', ''Building'', ''Success'', or ''Failure''.<br/>        """
        pass

    def get_device_definition(self, DeviceDefinitionId: str) -> GetDeviceDefinitionResponse:
        """Retrieves information about a device definition.

        :param self:
        :param DeviceDefinitionId:  The ID of the device definition.<br/>
        :return: tags:  The tags for the definition.<br/>Name:  The name of the definition.<br/>LatestVersionArn:  The ARN of the latest version of the definition.<br/>LatestVersion:  The latest version of the definition.<br/>LastUpdatedTimestamp:  The time, in milliseconds since the epoch, when the definition was last updated.<br/>Id:  The ID of the definition.<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the definition was created.<br/>Arn:  The ARN of the definition.<br/>        """
        pass

    def get_device_definition_version(self, DeviceDefinitionVersionId: str, DeviceDefinitionId: str, NextToken: str=None) -> GetDeviceDefinitionVersionResponse:
        """Retrieves information about a device definition version.

        :param self:
        :param NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>DeviceDefinitionVersionId:  The ID of the device definition version.<br/>DeviceDefinitionId:  The ID of the device definition.<br/>
        :return: Version:  The version of the device definition version.<br/>NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>Id:  The ID of the device definition version.<br/>Definition:  Information about the device definition version.:<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the device definition version was created.<br/>Arn:  The ARN of the device definition version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Devices</i> A list of devices in the definition version.<br/>        """
        pass

    def get_function_definition(self, FunctionDefinitionId: str) -> GetFunctionDefinitionResponse:
        """Retrieves information about a Lambda function definition, including its creation time and latest version.

        :param self:
        :param FunctionDefinitionId:  The ID of the Lambda function definition.<br/>
        :return: tags:  The tags for the definition.<br/>Name:  The name of the definition.<br/>LatestVersionArn:  The ARN of the latest version of the definition.<br/>LatestVersion:  The latest version of the definition.<br/>LastUpdatedTimestamp:  The time, in milliseconds since the epoch, when the definition was last updated.<br/>Id:  The ID of the definition.<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the definition was created.<br/>Arn:  The ARN of the definition.<br/>        """
        pass

    def get_function_definition_version(self, FunctionDefinitionVersionId: str, FunctionDefinitionId: str, NextToken: str=None) -> GetFunctionDefinitionVersionResponse:
        """Retrieves information about a Lambda function definition version, including which Lambda functions are included in the version and their configurations.

        :param self:
        :param NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>FunctionDefinitionVersionId:  The ID of the function definition version.<br/>FunctionDefinitionId:  The ID of the Lambda function definition.<br/>
        :return: Version:  The version of the function definition version.<br/>NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>Id:  The ID of the function definition version.<br/>Definition:  Information on the definition.:<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the function definition version was created.<br/>Arn:  The ARN of the function definition version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Functions</i> A list of Lambda functions in this function definition version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultConfig</i> The default configuration that applies to all Lambda functions in this function definition version. Individual Lambda functions can override these settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DefaultConfig</b>: The default configuration that applies to all Lambda functions in this function definition version. Individual Lambda functions can override these settings.<br/>Configuration information that specifies how a Lambda function runs. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Execution</b>: Configuration information that specifies how a Lambda function runs.         """
        pass

    def get_group(self, GroupId: str) -> GetGroupResponse:
        """Retrieves information about a group.

        :param self:
        :param GroupId:  The ID of the Greengrass group.<br/>
        :return: tags:  The tags for the definition.<br/>Name:  The name of the definition.<br/>LatestVersionArn:  The ARN of the latest version of the definition.<br/>LatestVersion:  The latest version of the definition.<br/>LastUpdatedTimestamp:  The time, in milliseconds since the epoch, when the definition was last updated.<br/>Id:  The ID of the definition.<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the definition was created.<br/>Arn:  The ARN of the definition.<br/>        """
        pass

    def get_group_certificate_authority(self, GroupId: str, CertificateAuthorityId: str) -> GetGroupCertificateAuthorityResponse:
        """Retreives the CA associated with a group. Returns the public key of the CA.

        :param self:
        :param GroupId:  The ID of the Greengrass group.<br/>CertificateAuthorityId:  The ID of the certificate authority.<br/>
        :return: PemEncodedCertificate:  The PEM encoded certificate for the group.<br/>GroupCertificateAuthorityId:  The ID of the certificate authority for the group.<br/>GroupCertificateAuthorityArn:  The ARN of the certificate authority for the group.<br/>        """
        pass

    def get_group_certificate_configuration(self, GroupId: str) -> GetGroupCertificateConfigurationResponse:
        """Retrieves the current configuration for the CA used by the group.

        :param self:
        :param GroupId:  The ID of the Greengrass group.<br/>
        :return: GroupId:  The ID of the group certificate configuration.<br/>CertificateExpiryInMilliseconds:  The amount of time remaining before the certificate expires, in milliseconds.<br/>CertificateAuthorityExpiryInMilliseconds:  The amount of time remaining before the certificate authority expires, in milliseconds.<br/>        """
        pass

    def get_group_version(self, GroupVersionId: str, GroupId: str) -> GetGroupVersionResponse:
        """Retrieves information about a group version.

        :param self:
        :param GroupVersionId:  The ID of the group version.<br/>GroupId:  The ID of the Greengrass group.<br/>
        :return: Version:  The unique ID for the version of the group.<br/>Id:  The ID of the group version.<br/>Definition:  Information about the group version definition.:<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the group version was created.<br/>Arn:  The ARN of the group version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubscriptionDefinitionVersionArn</i> The ARN of the subscription definition version for this group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceDefinitionVersionArn</i> The ARN of the resource definition version for this group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LoggerDefinitionVersionArn</i> The ARN of the logger definition version for this group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FunctionDefinitionVersionArn</i> The ARN of the function definition version for this group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceDefinitionVersionArn</i> The ARN of the device definition version for this group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CoreDefinitionVersionArn</i> The ARN of the core definition version for this group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectorDefinitionVersionArn</i> The ARN of the connector definition version for this group.<br/>        """
        pass

    def get_logger_definition(self, LoggerDefinitionId: str) -> GetLoggerDefinitionResponse:
        """Retrieves information about a logger definition.

        :param self:
        :param LoggerDefinitionId:  The ID of the logger definition.<br/>
        :return: tags:  The tags for the definition.<br/>Name:  The name of the definition.<br/>LatestVersionArn:  The ARN of the latest version of the definition.<br/>LatestVersion:  The latest version of the definition.<br/>LastUpdatedTimestamp:  The time, in milliseconds since the epoch, when the definition was last updated.<br/>Id:  The ID of the definition.<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the definition was created.<br/>Arn:  The ARN of the definition.<br/>        """
        pass

    def get_logger_definition_version(self, LoggerDefinitionVersionId: str, LoggerDefinitionId: str, NextToken: str=None) -> GetLoggerDefinitionVersionResponse:
        """Retrieves information about a logger definition version.

        :param self:
        :param NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>LoggerDefinitionVersionId:  The ID of the logger definition version.<br/>LoggerDefinitionId:  The ID of the logger definition.<br/>
        :return: Version:  The version of the logger definition version.<br/>Id:  The ID of the logger definition version.<br/>Definition:  Information about the logger definition version.:<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the logger definition version was created.<br/>Arn:  The ARN of the logger definition version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Loggers</i> A list of loggers.<br/>        """
        pass

    def get_resource_definition(self, ResourceDefinitionId: str) -> GetResourceDefinitionResponse:
        """Retrieves information about a resource definition, including its creation time and latest version.

        :param self:
        :param ResourceDefinitionId:  The ID of the resource definition.<br/>
        :return: tags:  The tags for the definition.<br/>Name:  The name of the definition.<br/>LatestVersionArn:  The ARN of the latest version of the definition.<br/>LatestVersion:  The latest version of the definition.<br/>LastUpdatedTimestamp:  The time, in milliseconds since the epoch, when the definition was last updated.<br/>Id:  The ID of the definition.<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the definition was created.<br/>Arn:  The ARN of the definition.<br/>        """
        pass

    def get_resource_definition_version(self, ResourceDefinitionVersionId: str, ResourceDefinitionId: str) -> GetResourceDefinitionVersionResponse:
        """Retrieves information about a resource definition version, including which resources are included in the version.

        :param self:
        :param ResourceDefinitionVersionId:  The ID of the resource definition version.<br/>ResourceDefinitionId:  The ID of the resource definition.<br/>
        :return: Version:  The version of the resource definition version.<br/>Id:  The ID of the resource definition version.<br/>Definition:  Information about the definition.:<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the resource definition version was created.<br/>Arn:  Arn of the resource definition version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Resources</i> A list of resources.<br/>        """
        pass

    def get_service_role_for_account(self) -> GetServiceRoleForAccountResponse:
        """Retrieves the service role that is attached to your account.

        :param self:
        :param 
        :return: RoleArn:  The ARN of the role which is associated with the account.<br/>AssociatedAt:  The time when the service role was associated with the account.<br/>        """
        pass

    def get_subscription_definition(self, SubscriptionDefinitionId: str) -> GetSubscriptionDefinitionResponse:
        """Retrieves information about a subscription definition.

        :param self:
        :param SubscriptionDefinitionId:  The ID of the subscription definition.<br/>
        :return: tags:  The tags for the definition.<br/>Name:  The name of the definition.<br/>LatestVersionArn:  The ARN of the latest version of the definition.<br/>LatestVersion:  The latest version of the definition.<br/>LastUpdatedTimestamp:  The time, in milliseconds since the epoch, when the definition was last updated.<br/>Id:  The ID of the definition.<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the definition was created.<br/>Arn:  The ARN of the definition.<br/>        """
        pass

    def get_subscription_definition_version(self, SubscriptionDefinitionVersionId: str, SubscriptionDefinitionId: str, NextToken: str=None) -> GetSubscriptionDefinitionVersionResponse:
        """Retrieves information about a subscription definition version.

        :param self:
        :param SubscriptionDefinitionVersionId:  The ID of the subscription definition version.<br/>SubscriptionDefinitionId:  The ID of the subscription definition.<br/>NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>
        :return: Version:  The version of the subscription definition version.<br/>NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>Id:  The ID of the subscription definition version.<br/>Definition:  Information about the subscription definition version.:<br/>CreationTimestamp:  The time, in milliseconds since the epoch, when the subscription definition version was created.<br/>Arn:  The ARN of the subscription definition version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subscriptions</i> A list of subscriptions.<br/>        """
        pass

    def list_bulk_deployment_detailed_reports(self, BulkDeploymentId: str, MaxResults: str=None, NextToken: str=None) -> ListBulkDeploymentDetailedReportsResponse:
        """Gets a paginated list of the deployments that have been started in a bulk deployment operation, and their current deployment status.

        :param self:
        :param NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>MaxResults:  The maximum number of results to be returned per request.<br/>BulkDeploymentId:  The ID of the bulk deployment.<br/>
        :return: NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>Deployments:  A list of the individual group deployments in the bulk deployment operation.<br/>        """
        pass

    def list_bulk_deployments(self, MaxResults: str=None, NextToken: str=None) -> ListBulkDeploymentsResponse:
        """Returns a list of bulk deployments.

        :param self:
        :param NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>MaxResults:  The maximum number of results to be returned per request.<br/>
        :return: NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>BulkDeployments:  A list of bulk deployments.<br/>        """
        pass

    def list_connector_definition_versions(self, ConnectorDefinitionId: str, MaxResults: str=None, NextToken: str=None) -> ListConnectorDefinitionVersionsResponse:
        """Lists the versions of a connector definition, which are containers for connectors. Connectors run on the Greengrass core and contain built-in integration with local infrastructure, device protocols, AWS, and other cloud services.

        :param self:
        :param NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>MaxResults:  The maximum number of results to be returned per request.<br/>ConnectorDefinitionId:  The ID of the connector definition.<br/>
        :return: Versions:  Information about a version.<br/>NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>        """
        pass

    def list_connector_definitions(self, MaxResults: str=None, NextToken: str=None) -> ListConnectorDefinitionsResponse:
        """Retrieves a list of connector definitions.

        :param self:
        :param NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>MaxResults:  The maximum number of results to be returned per request.<br/>
        :return: NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>Definitions:  Information about a definition.<br/>        """
        pass

    def list_core_definition_versions(self, CoreDefinitionId: str, MaxResults: str=None, NextToken: str=None) -> ListCoreDefinitionVersionsResponse:
        """Lists the versions of a core definition.

        :param self:
        :param NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>MaxResults:  The maximum number of results to be returned per request.<br/>CoreDefinitionId:  The ID of the core definition.<br/>
        :return: Versions:  Information about a version.<br/>NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>        """
        pass

    def list_core_definitions(self, MaxResults: str=None, NextToken: str=None) -> ListCoreDefinitionsResponse:
        """Retrieves a list of core definitions.

        :param self:
        :param NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>MaxResults:  The maximum number of results to be returned per request.<br/>
        :return: NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>Definitions:  Information about a definition.<br/>        """
        pass

    def list_deployments(self, GroupId: str, MaxResults: str=None, NextToken: str=None) -> ListDeploymentsResponse:
        """Returns a history of deployments for the group.

        :param self:
        :param NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>MaxResults:  The maximum number of results to be returned per request.<br/>GroupId:  The ID of the Greengrass group.<br/>
        :return: NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>Deployments:  A list of deployments for the requested groups.<br/>        """
        pass

    def list_device_definition_versions(self, DeviceDefinitionId: str, MaxResults: str=None, NextToken: str=None) -> ListDeviceDefinitionVersionsResponse:
        """Lists the versions of a device definition.

        :param self:
        :param NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>MaxResults:  The maximum number of results to be returned per request.<br/>DeviceDefinitionId:  The ID of the device definition.<br/>
        :return: Versions:  Information about a version.<br/>NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>        """
        pass

    def list_device_definitions(self, MaxResults: str=None, NextToken: str=None) -> ListDeviceDefinitionsResponse:
        """Retrieves a list of device definitions.

        :param self:
        :param NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>MaxResults:  The maximum number of results to be returned per request.<br/>
        :return: NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>Definitions:  Information about a definition.<br/>        """
        pass

    def list_function_definition_versions(self, FunctionDefinitionId: str, MaxResults: str=None, NextToken: str=None) -> ListFunctionDefinitionVersionsResponse:
        """Lists the versions of a Lambda function definition.

        :param self:
        :param NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>MaxResults:  The maximum number of results to be returned per request.<br/>FunctionDefinitionId:  The ID of the Lambda function definition.<br/>
        :return: Versions:  Information about a version.<br/>NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>        """
        pass

    def list_function_definitions(self, MaxResults: str=None, NextToken: str=None) -> ListFunctionDefinitionsResponse:
        """Retrieves a list of Lambda function definitions.

        :param self:
        :param NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>MaxResults:  The maximum number of results to be returned per request.<br/>
        :return: NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>Definitions:  Information about a definition.<br/>        """
        pass

    def list_group_certificate_authorities(self, GroupId: str) -> ListGroupCertificateAuthoritiesResponse:
        """Retrieves the current CAs for a group.

        :param self:
        :param GroupId:  The ID of the Greengrass group.<br/>
        :return: GroupCertificateAuthorities:  A list of certificate authorities associated with the group.<br/>        """
        pass

    def list_group_versions(self, GroupId: str, MaxResults: str=None, NextToken: str=None) -> ListGroupVersionsResponse:
        """Lists the versions of a group.

        :param self:
        :param NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>MaxResults:  The maximum number of results to be returned per request.<br/>GroupId:  The ID of the Greengrass group.<br/>
        :return: Versions:  Information about a version.<br/>NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>        """
        pass

    def list_groups(self, MaxResults: str=None, NextToken: str=None) -> ListGroupsResponse:
        """Retrieves a list of groups.

        :param self:
        :param NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>MaxResults:  The maximum number of results to be returned per request.<br/>
        :return: NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>Groups:  Information about a group.<br/>        """
        pass

    def list_logger_definition_versions(self, LoggerDefinitionId: str, MaxResults: str=None, NextToken: str=None) -> ListLoggerDefinitionVersionsResponse:
        """Lists the versions of a logger definition.

        :param self:
        :param NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>MaxResults:  The maximum number of results to be returned per request.<br/>LoggerDefinitionId:  The ID of the logger definition.<br/>
        :return: Versions:  Information about a version.<br/>NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>        """
        pass

    def list_logger_definitions(self, MaxResults: str=None, NextToken: str=None) -> ListLoggerDefinitionsResponse:
        """Retrieves a list of logger definitions.

        :param self:
        :param NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>MaxResults:  The maximum number of results to be returned per request.<br/>
        :return: NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>Definitions:  Information about a definition.<br/>        """
        pass

    def list_resource_definition_versions(self, ResourceDefinitionId: str, MaxResults: str=None, NextToken: str=None) -> ListResourceDefinitionVersionsResponse:
        """Lists the versions of a resource definition.

        :param self:
        :param ResourceDefinitionId:  The ID of the resource definition.<br/>NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>MaxResults:  The maximum number of results to be returned per request.<br/>
        :return: Versions:  Information about a version.<br/>NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>        """
        pass

    def list_resource_definitions(self, MaxResults: str=None, NextToken: str=None) -> ListResourceDefinitionsResponse:
        """Retrieves a list of resource definitions.

        :param self:
        :param NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>MaxResults:  The maximum number of results to be returned per request.<br/>
        :return: NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>Definitions:  Information about a definition.<br/>        """
        pass

    def list_subscription_definition_versions(self, SubscriptionDefinitionId: str, MaxResults: str=None, NextToken: str=None) -> ListSubscriptionDefinitionVersionsResponse:
        """Lists the versions of a subscription definition.

        :param self:
        :param SubscriptionDefinitionId:  The ID of the subscription definition.<br/>NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>MaxResults:  The maximum number of results to be returned per request.<br/>
        :return: Versions:  Information about a version.<br/>NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>        """
        pass

    def list_subscription_definitions(self, MaxResults: str=None, NextToken: str=None) -> ListSubscriptionDefinitionsResponse:
        """Retrieves a list of subscription definitions.

        :param self:
        :param NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>MaxResults:  The maximum number of results to be returned per request.<br/>
        :return: NextToken:  The token for the next set of results, or ''null'' if there are no additional results.<br/>Definitions:  Information about a definition.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceArn: str) -> ListTagsForResourceResponse:
        """Retrieves the tags for a resource.

        :param self:
        :param ResourceArn:  The Amazon Resource Name (ARN) of the resource.<br/>
        :return: tags:  A map of the key-value pairs for the resource tag.<br/>        """
        pass

    def reset_deployments(self, GroupId: str, AmznClientToken: str=None, Force: __boolean=None) -> ResetDeploymentsResponse:
        """Resets a group's deployments.

        :param self:
        :param GroupId:  The ID of the Greengrass group.<br/>Force:  If true, performs a best-effort only core reset.<br/>AmznClientToken:  A client token used to correlate requests and responses.<br/>
        :return: DeploymentId:  The ID of the deployment.<br/>DeploymentArn:  The ARN of the deployment.<br/>        """
        pass

    def start_bulk_deployment(self, AmznClientToken: str=None, ExecutionRoleArn: str=None, InputFileUri: str=None, tags: Tags=None) -> StartBulkDeploymentResponse:
        """Deploys multiple groups in one operation. This action starts the bulk deployment of a specified set of group versions. Each group version deployment will be triggered with an adaptive rate that has a fixed upper limit. We recommend that you include an ''X-Amzn-Client-Token'' token in every ''StartBulkDeployment'' request. These requests are idempotent with respect to the token and the request parameters.

        :param self:
        :param tags:  Tag(s) to add to the new resource<br/>InputFileUri:  The URI of the input file contained in the S3 bucket. The execution role must have ''getObject'' permissions on this bucket to access the input file. The input file is a JSON-serialized, line delimited file with UTF-8 encoding that provides a list of group and version IDs and the deployment type. This file must be less than 100 MB. Currently, AWS IoT Greengrass supports only ''NewDeployment'' deployment types.<br/>ExecutionRoleArn:  The ARN of the execution role to associate with the bulk deployment operation. This IAM role must allow the ''greengrass:CreateDeployment'' action for all group versions that are listed in the input file. This IAM role must have access to the S3 bucket containing the input file.<br/>AmznClientToken:  A client token used to correlate requests and responses.<br/>
        :return: BulkDeploymentId:  The ID of the bulk deployment.<br/>BulkDeploymentArn:  The ARN of the bulk deployment.<br/>        """
        pass

    def stop_bulk_deployment(self, BulkDeploymentId: str) -> StopBulkDeploymentResponse:
        """Stops the execution of a bulk deployment. This action returns a status of ''Stopping'' until the deployment is stopped. You cannot start a new bulk deployment while a previous deployment is in the ''Stopping'' state. This action doesn't rollback completed deployments or cancel pending deployments.

        :param self:
        :param BulkDeploymentId:  The ID of the bulk deployment.<br/>
        :return:         """
        pass

    def update_connectivity_info(self, ThingName: str, ConnectivityInfo: List[ConnectivityInfo]=None) -> UpdateConnectivityInfoResponse:
        """Updates the connectivity information for the core. Any devices that belong to the group which has this core will receive this information in order to find the location of the core and connect to it.

        :param self:
        :param ThingName:  The thing name.<br/>ConnectivityInfo:  A list of connectivity info.<br/>
        :return: Version:  The new version of the connectivity info.<br/>Message:  A message about the connectivity info update request.<br/>        """
        pass

    def update_connector_definition(self, ConnectorDefinitionId: str, Name: str=None) -> UpdateConnectorDefinitionResponse:
        """Updates a connector definition.

        :param self:
        :param Name:  The name of the definition.<br/>ConnectorDefinitionId:  The ID of the connector definition.<br/>
        :return:         """
        pass

    def update_core_definition(self, CoreDefinitionId: str, Name: str=None) -> UpdateCoreDefinitionResponse:
        """Updates a core definition.

        :param self:
        :param Name:  The name of the definition.<br/>CoreDefinitionId:  The ID of the core definition.<br/>
        :return:         """
        pass

    def update_device_definition(self, DeviceDefinitionId: str, Name: str=None) -> UpdateDeviceDefinitionResponse:
        """Updates a device definition.

        :param self:
        :param Name:  The name of the definition.<br/>DeviceDefinitionId:  The ID of the device definition.<br/>
        :return:         """
        pass

    def update_function_definition(self, FunctionDefinitionId: str, Name: str=None) -> UpdateFunctionDefinitionResponse:
        """Updates a Lambda function definition.

        :param self:
        :param Name:  The name of the definition.<br/>FunctionDefinitionId:  The ID of the Lambda function definition.<br/>
        :return:         """
        pass

    def update_group(self, GroupId: str, Name: str=None) -> UpdateGroupResponse:
        """Updates a group.

        :param self:
        :param Name:  The name of the definition.<br/>GroupId:  The ID of the Greengrass group.<br/>
        :return:         """
        pass

    def update_group_certificate_configuration(self, GroupId: str, CertificateExpiryInMilliseconds: str=None) -> UpdateGroupCertificateConfigurationResponse:
        """Updates the Certificate expiry time for a group.

        :param self:
        :param GroupId:  The ID of the Greengrass group.<br/>CertificateExpiryInMilliseconds:  The amount of time remaining before the certificate expires, in milliseconds.<br/>
        :return: GroupId:  The ID of the group certificate configuration.<br/>CertificateExpiryInMilliseconds:  The amount of time remaining before the certificate expires, in milliseconds.<br/>CertificateAuthorityExpiryInMilliseconds:  The amount of time remaining before the certificate authority expires, in milliseconds.<br/>        """
        pass

    def update_logger_definition(self, LoggerDefinitionId: str, Name: str=None) -> UpdateLoggerDefinitionResponse:
        """Updates a logger definition.

        :param self:
        :param Name:  The name of the definition.<br/>LoggerDefinitionId:  The ID of the logger definition.<br/>
        :return:         """
        pass

    def update_resource_definition(self, ResourceDefinitionId: str, Name: str=None) -> UpdateResourceDefinitionResponse:
        """Updates a resource definition.

        :param self:
        :param ResourceDefinitionId:  The ID of the resource definition.<br/>Name:  The name of the definition.<br/>
        :return:         """
        pass

    def update_subscription_definition(self, SubscriptionDefinitionId: str, Name: str=None) -> UpdateSubscriptionDefinitionResponse:
        """Updates a subscription definition.

        :param self:
        :param SubscriptionDefinitionId:  The ID of the subscription definition.<br/>Name:  The name of the definition.<br/>
        :return:         """
        pass



class GroundStation(BaseClient):
    """Welcome to the AWS Ground Station API Reference. AWS Ground Station is a fully managed service that
      enables you to control satellite communications, downlink and process satellite data, and
      scale your satellite operations efficiently and cost-effectively without having
      to build or manage your own ground station infrastructure."""


    class ListConfigsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_configs` for valid parameters"""
            pass
    class ListContactsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_contacts` for valid parameters"""
            pass
    class ListDataflowEndpointGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_dataflow_endpoint_groups` for valid parameters"""
            pass
    class ListMissionProfilesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_mission_profiles` for valid parameters"""
            pass
    class ListGroundStationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_ground_stations` for valid parameters"""
            pass
    class ListSatellitesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_satellites` for valid parameters"""
            pass


    class CancelContactRequest(Mapping):
        pass
    
    class ContactIdResponse(Mapping):
        pass
    
    class CreateConfigRequest(Mapping):
        pass
    
    class ConfigIdResponse(Mapping):
        pass
    
    class ConfigTypeData(Mapping):
        pass
    
    class TagsMap(object):
        pass
    
    class CreateDataflowEndpointGroupRequest(Mapping):
        pass
    
    class DataflowEndpointGroupIdResponse(Mapping):
        pass
    
    class EndpointDetails(Mapping):
        pass
    
    class CreateMissionProfileRequest(Mapping):
        pass
    
    class MissionProfileIdResponse(Mapping):
        pass
    
    class DataflowEdge(object):
        pass
    
    class DeleteConfigRequest(Mapping):
        pass
    
    class DeleteDataflowEndpointGroupRequest(Mapping):
        pass
    
    class DeleteMissionProfileRequest(Mapping):
        pass
    
    class DescribeContactRequest(Mapping):
        pass
    
    class DescribeContactResponse(Mapping):
        pass
    
    class GetConfigRequest(Mapping):
        pass
    
    class GetConfigResponse(Mapping):
        pass
    
    class GetDataflowEndpointGroupRequest(Mapping):
        pass
    
    class GetDataflowEndpointGroupResponse(Mapping):
        pass
    
    class GetMissionProfileRequest(Mapping):
        pass
    
    class GetMissionProfileResponse(Mapping):
        pass
    
    class ListConfigsRequest(Mapping):
        pass
    
    class ListConfigsResponse(Mapping):
        pass
    
    class ListContactsRequest(Mapping):
        pass
    
    class ListContactsResponse(Mapping):
        pass
    
    class Timestamp(object):
        pass
    
    class ContactStatus(object):
        pass
    
    class ListDataflowEndpointGroupsRequest(Mapping):
        pass
    
    class ListDataflowEndpointGroupsResponse(Mapping):
        pass
    
    class ListMissionProfilesRequest(Mapping):
        pass
    
    class ListMissionProfilesResponse(Mapping):
        pass
    
    class ReserveContactRequest(Mapping):
        pass
    
    class UpdateConfigRequest(Mapping):
        pass
    
    class UpdateMissionProfileRequest(Mapping):
        pass
    
    class GetMinuteUsageRequest(Mapping):
        pass
    
    class GetMinuteUsageResponse(Mapping):
        pass
    
    class GetSatelliteRequest(Mapping):
        pass
    
    class GetSatelliteResponse(Mapping):
        pass
    
    class ListGroundStationsRequest(Mapping):
        pass
    
    class ListGroundStationsResponse(Mapping):
        pass
    
    class ListSatellitesRequest(Mapping):
        pass
    
    class ListSatellitesResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class String(object):
        pass
    
    def cancel_contact(self, contactId: str) -> ContactIdResponse:
        """Cancels a contact with a specified contact ID.

        :param self:
        :param contactId:  UUID of a contact.<br/>
        :return: contactId:  UUID of a contact.<br/>        """
        pass

    def create_config(self, name: str, configData: ConfigTypeData, tags: TagsMap=None) -> ConfigIdResponse:
        """Creates a <code>Config</code> with the specified <code>configData</code> parameters.
         Only one type of <code>configData</code> can be specified.

        :param self:
        :param tags:  Tags assigned to a <code>Config</code>.<br/>name:  Name of a <code>Config</code>.<br/>configData:  Parameters of a <code>Config</code>.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>uplinkEchoConfig</i> Information about an uplink echo <code>Config</code>.
         Parameters from the <code>AntennaUplinkConfig</code>, corresponding to the specified <code>AntennaUplinkConfigArn</code>, are used when this <code>UplinkEchoConfig</code> is used in a contact.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>uplinkEchoConfig</b>: Information about an uplink echo <code>Config</code>.
         Parameters from the <code>AntennaUplinkConfig</code>, corresponding to the specified <code>AntennaUplinkConfigArn</code>, are used when this <code>UplinkEchoConfig</code> is used in a contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>trackingConfig</i> Object that determines whether tracking should be used during a contact executed with this <code>Config</code> in the mission profile. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>trackingConfig</b>: Object that determines whether tracking should be used during a contact executed with this <code>Config</code> in the mission profile. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>dataflowEndpointConfig</i> Information about the dataflow endpoint <code>Config</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>dataflowEndpointConfig</b>: Information about the dataflow endpoint <code>Config</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>antennaUplinkConfig</i> Information about how AWS Ground Station should conﬁgure an antenna for uplink during a contact.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>antennaUplinkConfig</b>: Information about how AWS Ground Station should conﬁgure an antenna for uplink during a contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>antennaDownlinkDemodDecodeConfig</i> Information about how AWS Ground Station should conﬁgure an antenna for downlink demod decode during a contact.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>antennaDownlinkDemodDecodeConfig</b>: Information about how AWS Ground Station should conﬁgure an antenna for downlink demod decode during a contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>antennaDownlinkConfig</i> Information about how AWS Ground Station should configure an antenna for downlink during a contact.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>antennaDownlinkConfig</b>: Information about how AWS Ground Station should configure an antenna for downlink during a contact.<br/>Object that describes a spectral <code>Config</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>spectrumConfig</b>: Object that describes a spectral <code>Config</code>.Information about the decode <code>Config</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>decodeConfig</b>: Information about the decode <code>Config</code>.Information about the uplink spectral <code>Config</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>spectrumConfig</b>: Information about the uplink spectral <code>Config</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>dataflowEndpointName</i> Name of a dataflow endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>autotrack</i> Current setting for autotrack.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>enabled</i> Whether or not an uplink <code>Config</code> is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>antennaUplinkConfigArn</i> ARN of an uplink <code>Config</code>.<br/>
        :return: configType:  Type of a <code>Config</code>.<br/>configId:  UUID of a <code>Config</code>.<br/>configArn:  ARN of a <code>Config</code>.<br/>        """
        pass

    def create_dataflow_endpoint_group(self, endpointDetails: List[EndpointDetails], tags: TagsMap=None) -> DataflowEndpointGroupIdResponse:
        """Creates a <code>DataflowEndpoint</code> group containing the specified list of <code>DataflowEndpoint</code> objects.
         The <code>name</code> field in each endpoint is used in your mission profile <code>DataflowEndpointConfig</code> 
         to specify which endpoints to use during a contact. 
         When a contact uses multiple <code>DataflowEndpointConfig</code> objects, each <code>Config</code> 
         must match a <code>DataflowEndpoint</code> in the same group.

        :param self:
        :param tags:  Tags of a dataflow endpoint group.<br/>endpointDetails:  Endpoint details of each endpoint in the dataflow endpoint group.<br/>
        :return: dataflowEndpointGroupId:  ID of a dataflow endpoint group.<br/>        """
        pass

    def create_mission_profile(self, trackingConfigArn: str, name: str, minimumViableContactDurationSeconds: int, dataflowEdges: List[DataflowEdge], contactPostPassDurationSeconds: int=None, contactPrePassDurationSeconds: int=None, tags: TagsMap=None) -> MissionProfileIdResponse:
        """Creates a mission profile.
         
            <code>dataflowEdges</code> is a list of lists of strings. Each lower level list of strings
         has two elements: a <i>from ARN</i> and a <i>to ARN</i>.

        :param self:
        :param trackingConfigArn:  ARN of a tracking <code>Config</code>.<br/>tags:  Tags assigned to a mission profile.<br/>name:  Name of a mission profile.<br/>minimumViableContactDurationSeconds:  Smallest amount of time in seconds that you’d like to see for an available contact. AWS Ground Station will not present you with contacts shorter than this duration.<br/>dataflowEdges:  A list of lists of ARNs. Each list of ARNs is an edge, with a from <code>Config</code> and a to 
         <code>Config</code>.<br/>contactPrePassDurationSeconds:  Amount of time prior to contact start you’d like to receive a CloudWatch event indicating an upcoming pass.<br/>contactPostPassDurationSeconds:  Amount of time after a contact ends that you’d like to receive a CloudWatch event indicating the pass has finished.<br/>
        :return: missionProfileId:  ID of a mission profile.<br/>        """
        pass

    def delete_config(self, configType: str, configId: str) -> ConfigIdResponse:
        """Deletes a <code>Config</code>.

        :param self:
        :param configType:  Type of a <code>Config</code>.<br/>configId:  UUID of a <code>Config</code>.<br/>
        :return: configType:  Type of a <code>Config</code>.<br/>configId:  UUID of a <code>Config</code>.<br/>configArn:  ARN of a <code>Config</code>.<br/>        """
        pass

    def delete_dataflow_endpoint_group(self, dataflowEndpointGroupId: str) -> DataflowEndpointGroupIdResponse:
        """Deletes a dataflow endpoint group.

        :param self:
        :param dataflowEndpointGroupId:  ID of a dataflow endpoint group.<br/>
        :return: dataflowEndpointGroupId:  ID of a dataflow endpoint group.<br/>        """
        pass

    def delete_mission_profile(self, missionProfileId: str) -> MissionProfileIdResponse:
        """Deletes a mission profile.

        :param self:
        :param missionProfileId:  UUID of a mission profile.<br/>
        :return: missionProfileId:  ID of a mission profile.<br/>        """
        pass

    def describe_contact(self, contactId: str) -> DescribeContactResponse:
        """Describes an existing contact.

        :param self:
        :param contactId:  UUID of a contact.<br/>
        :return: tags:  Tags assigned to a contact.<br/>startTime:  Start time of a contact.<br/>satelliteArn:  ARN of a satellite.<br/>prePassStartTime:  Amount of time prior to contact start you’d like to receive a CloudWatch event indicating an upcoming pass.<br/>postPassEndTime:  Amount of time after a contact ends that you’d like to receive a CloudWatch event indicating the pass has finished.<br/>missionProfileArn:  ARN of a mission profile.<br/>maximumElevation:  Maximum elevation angle of a contact.:<br/>groundStation:  Ground station for a contact.<br/>errorMessage:  Error message for a contact.<br/>endTime:  End time of a contact.<br/>contactStatus:  Status of a contact.<br/>contactId:  UUID of a contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>value</i> Elevation angle value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>unit</i> Elevation angle units.<br/>        """
        pass

    def get_config(self, configType: str, configId: str) -> GetConfigResponse:
        """Returns <code>Config</code> information.
         Only one <code>Config</code> response can be returned.

        :param self:
        :param configType:  Type of a <code>Config</code>.<br/>configId:  UUID of a <code>Config</code>.<br/>
        :return: tags:  Tags assigned to a <code>Config</code>.<br/>name:  Name of a <code>Config</code>.<br/>configType:  Type of a <code>Config</code>.<br/>configId:  UUID of a <code>Config</code>.<br/>configData:  Data elements in a <code>Config</code>.:<br/>configArn:  ARN of a <code>Config</code>
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>uplinkEchoConfig</i> Information about an uplink echo <code>Config</code>.
         Parameters from the <code>AntennaUplinkConfig</code>, corresponding to the specified <code>AntennaUplinkConfigArn</code>, are used when this <code>UplinkEchoConfig</code> is used in a contact.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>uplinkEchoConfig</b>: Information about an uplink echo <code>Config</code>.
         Parameters from the <code>AntennaUplinkConfig</code>, corresponding to the specified <code>AntennaUplinkConfigArn</code>, are used when this <code>UplinkEchoConfig</code> is used in a contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>trackingConfig</i> Object that determines whether tracking should be used during a contact executed with this <code>Config</code> in the mission profile. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>trackingConfig</b>: Object that determines whether tracking should be used during a contact executed with this <code>Config</code> in the mission profile. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>dataflowEndpointConfig</i> Information about the dataflow endpoint <code>Config</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>dataflowEndpointConfig</b>: Information about the dataflow endpoint <code>Config</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>antennaUplinkConfig</i> Information about how AWS Ground Station should conﬁgure an antenna for uplink during a contact.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>antennaUplinkConfig</b>: Information about how AWS Ground Station should conﬁgure an antenna for uplink during a contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>antennaDownlinkDemodDecodeConfig</i> Information about how AWS Ground Station should conﬁgure an antenna for downlink demod decode during a contact.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>antennaDownlinkDemodDecodeConfig</b>: Information about how AWS Ground Station should conﬁgure an antenna for downlink demod decode during a contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>antennaDownlinkConfig</i> Information about how AWS Ground Station should configure an antenna for downlink during a contact.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>antennaDownlinkConfig</b>: Information about how AWS Ground Station should configure an antenna for downlink during a contact.<br/>Object that describes a spectral <code>Config</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>spectrumConfig</b>: Object that describes a spectral <code>Config</code>.Information about the decode <code>Config</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>decodeConfig</b>: Information about the decode <code>Config</code>.Information about the uplink spectral <code>Config</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>spectrumConfig</b>: Information about the uplink spectral <code>Config</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>dataflowEndpointName</i> Name of a dataflow endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>autotrack</i> Current setting for autotrack.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>enabled</i> Whether or not an uplink <code>Config</code> is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>antennaUplinkConfigArn</i> ARN of an uplink <code>Config</code>.<br/>        """
        pass

    def get_dataflow_endpoint_group(self, dataflowEndpointGroupId: str) -> GetDataflowEndpointGroupResponse:
        """Returns the dataflow endpoint group.

        :param self:
        :param dataflowEndpointGroupId:  UUID of a dataflow endpoint group.<br/>
        :return: tags:  Tags assigned to a dataflow endpoint group.<br/>endpointsDetails:  Details of a dataflow endpoint.<br/>dataflowEndpointGroupId:  UUID of a dataflow endpoint group.<br/>dataflowEndpointGroupArn:  ARN of a dataflow endpoint group.<br/>        """
        pass

    def get_mission_profile(self, missionProfileId: str) -> GetMissionProfileResponse:
        """Returns a mission profile.

        :param self:
        :param missionProfileId:  UUID of a mission profile.<br/>
        :return: trackingConfigArn:  ARN of a tracking <code>Config</code>.<br/>tags:  Tags assigned to a mission profile.<br/>region:  Region of a mission profile.<br/>name:  Name of a mission profile.<br/>missionProfileId:  ID of a mission profile.<br/>missionProfileArn:  ARN of a mission profile.<br/>minimumViableContactDurationSeconds:  Smallest amount of time in seconds that you’d like to see for an available contact. AWS Ground Station will not present you with contacts shorter than this duration.<br/>dataflowEdges:  A list of lists of ARNs. Each list of ARNs is an edge, with a from <code>Config</code> and a to 
         <code>Config</code>.<br/>contactPrePassDurationSeconds:  Amount of time prior to contact start you’d like to receive a CloudWatch event indicating an upcoming pass.<br/>contactPostPassDurationSeconds:  Amount of time after a contact ends that you’d like to receive a CloudWatch event indicating the pass has finished.<br/>        """
        pass

    def list_configs(self, maxResults: int=None, nextToken: str=None) -> ListConfigsResponse:
        """Returns a list of <code>Config</code> objects.

        :param self:
        :param nextToken:  Next token returned in the request of a previous <code>ListConfigs</code> call. Used to get the next page of results.<br/>maxResults:  Maximum number of <code>Configs</code> returned.<br/>
        :return: nextToken:  Next token returned in the response of a previous <code>ListConfigs</code> call. Used to get the next page of results.<br/>configList:  List of <code>Config</code> items.<br/>        """
        pass

    def list_contacts(self, statusList: List[ContactStatus], startTime: Timestamp, endTime: Timestamp, groundStation: str=None, maxResults: int=None, missionProfileArn: str=None, nextToken: str=None, satelliteArn: str=None) -> ListContactsResponse:
        """Returns a list of contacts.
         If <code>statusList</code> contains AVAILABLE, the request must include
      <code>groundstation</code>, <code>missionprofileArn</code>, and <code>satelliteArn</code>.
      

        :param self:
        :param statusList:  Status of a contact reservation.<br/>startTime:  Start time of a contact.<br/>satelliteArn:  ARN of a satellite.<br/>nextToken:  Next token returned in the request of a previous <code>ListContacts</code> call. Used to get the next page of results.<br/>missionProfileArn:  ARN of a mission profile.<br/>maxResults:  Maximum number of contacts returned.<br/>groundStation:  Name of a ground station.<br/>endTime:  End time of a contact.<br/>
        :return: nextToken:  Next token returned in the response of a previous <code>ListContacts</code> call. Used to get the next page of results.<br/>contactList:  List of contacts.<br/>        """
        pass

    def list_dataflow_endpoint_groups(self, maxResults: int=None, nextToken: str=None) -> ListDataflowEndpointGroupsResponse:
        """Returns a list of <code>DataflowEndpoint</code> groups.

        :param self:
        :param nextToken:  Next token returned in the request of a previous <code>ListDataflowEndpointGroups</code> call. Used to get the next page of results.<br/>maxResults:  Maximum number of dataflow endpoint groups returned.<br/>
        :return: nextToken:  Next token returned in the response of a previous <code>ListDataflowEndpointGroups</code> call. Used to get the next page of results.<br/>dataflowEndpointGroupList:  A list of dataflow endpoint groups.<br/>        """
        pass

    def list_mission_profiles(self, maxResults: int=None, nextToken: str=None) -> ListMissionProfilesResponse:
        """Returns a list of mission profiles.

        :param self:
        :param nextToken:  Next token returned in the request of a previous <code>ListMissionProfiles</code> call. Used to get the next page of results.<br/>maxResults:  Maximum number of mission profiles returned.<br/>
        :return: nextToken:  Next token returned in the response of a previous <code>ListMissionProfiles</code> call. Used to get the next page of results.<br/>missionProfileList:  List of mission profiles<br/>        """
        pass

    def reserve_contact(self, startTime: Timestamp, satelliteArn: str, missionProfileArn: str, groundStation: str, endTime: Timestamp, tags: TagsMap=None) -> ContactIdResponse:
        """Reserves a contact using specified parameters.

        :param self:
        :param tags:  Tags assigned to a contact.<br/>startTime:  Start time of a contact.<br/>satelliteArn:  ARN of a satellite<br/>missionProfileArn:  ARN of a mission profile.<br/>groundStation:  Name of a ground station.<br/>endTime:  End time of a contact.<br/>
        :return: contactId:  UUID of a contact.<br/>        """
        pass

    def update_config(self, name: str, configType: str, configId: str, configData: ConfigTypeData) -> ConfigIdResponse:
        """Updates the <code>Config</code> used when scheduling contacts.
         Updating a <code>Config</code> will not update the execution parameters
         for existing future contacts scheduled with this <code>Config</code>.

        :param self:
        :param name:  Name of a <code>Config</code>.<br/>configType:  Type of a <code>Config</code>.<br/>configId:  UUID of a <code>Config</code>.<br/>configData:  Parameters for a <code>Config</code>.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>uplinkEchoConfig</i> Information about an uplink echo <code>Config</code>.
         Parameters from the <code>AntennaUplinkConfig</code>, corresponding to the specified <code>AntennaUplinkConfigArn</code>, are used when this <code>UplinkEchoConfig</code> is used in a contact.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>uplinkEchoConfig</b>: Information about an uplink echo <code>Config</code>.
         Parameters from the <code>AntennaUplinkConfig</code>, corresponding to the specified <code>AntennaUplinkConfigArn</code>, are used when this <code>UplinkEchoConfig</code> is used in a contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>trackingConfig</i> Object that determines whether tracking should be used during a contact executed with this <code>Config</code> in the mission profile. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>trackingConfig</b>: Object that determines whether tracking should be used during a contact executed with this <code>Config</code> in the mission profile. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>dataflowEndpointConfig</i> Information about the dataflow endpoint <code>Config</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>dataflowEndpointConfig</b>: Information about the dataflow endpoint <code>Config</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>antennaUplinkConfig</i> Information about how AWS Ground Station should conﬁgure an antenna for uplink during a contact.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>antennaUplinkConfig</b>: Information about how AWS Ground Station should conﬁgure an antenna for uplink during a contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>antennaDownlinkDemodDecodeConfig</i> Information about how AWS Ground Station should conﬁgure an antenna for downlink demod decode during a contact.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>antennaDownlinkDemodDecodeConfig</b>: Information about how AWS Ground Station should conﬁgure an antenna for downlink demod decode during a contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>antennaDownlinkConfig</i> Information about how AWS Ground Station should configure an antenna for downlink during a contact.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>antennaDownlinkConfig</b>: Information about how AWS Ground Station should configure an antenna for downlink during a contact.<br/>Object that describes a spectral <code>Config</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>spectrumConfig</b>: Object that describes a spectral <code>Config</code>.Information about the decode <code>Config</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>decodeConfig</b>: Information about the decode <code>Config</code>.Information about the uplink spectral <code>Config</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>spectrumConfig</b>: Information about the uplink spectral <code>Config</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>dataflowEndpointName</i> Name of a dataflow endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>autotrack</i> Current setting for autotrack.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>enabled</i> Whether or not an uplink <code>Config</code> is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>antennaUplinkConfigArn</i> ARN of an uplink <code>Config</code>.<br/>
        :return: configType:  Type of a <code>Config</code>.<br/>configId:  UUID of a <code>Config</code>.<br/>configArn:  ARN of a <code>Config</code>.<br/>        """
        pass

    def update_mission_profile(self, missionProfileId: str, contactPostPassDurationSeconds: int=None, contactPrePassDurationSeconds: int=None, dataflowEdges: List[DataflowEdge]=None, minimumViableContactDurationSeconds: int=None, name: str=None, trackingConfigArn: str=None) -> MissionProfileIdResponse:
        """Updates a mission profile.
         Updating a mission profile will not update the execution parameters
         for existing future contacts.

        :param self:
        :param trackingConfigArn:  ARN of a tracking <code>Config</code>.<br/>name:  Name of a mission profile.<br/>missionProfileId:  ID of a mission profile.<br/>minimumViableContactDurationSeconds:  Smallest amount of time in seconds that you’d like to see for an available contact. AWS Ground Station will not present you with contacts shorter than this duration.<br/>dataflowEdges:  A list of lists of ARNs. Each list of ARNs is an edge, with a from <code>Config</code> and a to 
         <code>Config</code>.<br/>contactPrePassDurationSeconds:  Amount of time after a contact ends that you’d like to receive a CloudWatch event indicating the pass has finished.<br/>contactPostPassDurationSeconds:  Amount of time after a contact ends that you’d like to receive a CloudWatch event indicating the pass has finished.<br/>
        :return: missionProfileId:  ID of a mission profile.<br/>        """
        pass

    def get_minute_usage(self, year: int, month: int) -> GetMinuteUsageResponse:
        """Returns the number of minutes used by account.

        :param self:
        :param year:  The year being requested, in the format of YYYY.<br/>month:  The month being requested, with a value of 1-12.<br/>
        :return: upcomingMinutesScheduled:  Upcoming minutes scheduled for an account, specific to the month being requested.<br/>totalScheduledMinutes:  Total scheduled minutes for an account, specific to the month being requested.<br/>totalReservedMinuteAllocation:  Total number of reserved minutes allocated, specific to the month being requested.<br/>isReservedMinutesCustomer:  Returns whether or not an account has signed up for the reserved minutes pricing plan, specific to the month being requested.<br/>estimatedMinutesRemaining:  Estimated number of minutes remaining for an account, specific to the month being requested.<br/>        """
        pass

    def get_satellite(self, satelliteId: str) -> GetSatelliteResponse:
        """Returns a satellite.

        :param self:
        :param satelliteId:  UUID of a satellite.<br/>
        :return: tags:  Tags assigned to a satellite.<br/>satelliteId:  UUID of a satellite.<br/>satelliteArn:  ARN of a satellite.<br/>noradSatelliteID:  NORAD satellite ID number.<br/>lastUpdated:  When a satellite was last updated.<br/>dateCreated:  When a satellite was created.<br/>        """
        pass

    def list_ground_stations(self, maxResults: int=None, nextToken: str=None) -> ListGroundStationsResponse:
        """Returns a list of ground stations. 

        :param self:
        :param nextToken:  Next token that can be supplied in the next call to get the next page of ground stations.<br/>maxResults:  Maximum number of ground stations returned.<br/>
        :return: nextToken:  Next token that can be supplied in the next call to get the next page of ground stations.<br/>groundStationList:  List of ground stations.<br/>        """
        pass

    def list_satellites(self, maxResults: int=None, nextToken: str=None) -> ListSatellitesResponse:
        """Returns a list of satellites.

        :param self:
        :param nextToken:  Next token that can be supplied in the next call to get the next page of satellites.<br/>maxResults:  Maximum number of satellites returned.<br/>
        :return: satellites:  List of satellites.<br/>nextToken:  Next token that can be supplied in the next call to get the next page of satellites.<br/>        """
        pass

    def list_tags_for_resource(self, resourceArn: str) -> ListTagsForResourceResponse:
        """Returns a list of tags or a specified resource.

        :param self:
        :param resourceArn:  ARN of a resource.<br/>
        :return: tags:  Tags assigned to a resource.<br/>        """
        pass

    def tag_resource(self, resourceArn: str, tags: TagsMap=None) -> TagResourceResponse:
        """Assigns a tag to a resource.

        :param self:
        :param tags:  Tags assigned to a resource.<br/>resourceArn:  ARN of a resource tag.<br/>
        :return:         """
        pass

    def untag_resource(self, tagKeys: List[String], resourceArn: str) -> UntagResourceResponse:
        """Deassigns a resource tag.

        :param self:
        :param tagKeys:  Keys of a resource tag.<br/>resourceArn:  ARN of a resource.<br/>
        :return:         """
        pass



class GuardDuty(BaseClient):
    """Amazon GuardDuty is a continuous security monitoring service that analyzes and processes the following data sources: VPC Flow Logs, AWS CloudTrail event logs, and DNS logs. It uses threat intelligence feeds, such as lists of malicious IPs and domains, and machine learning to identify unexpected and potentially unauthorized and malicious activity within your AWS environment. This can include issues like escalations of privileges, uses of exposed credentials, or communication with malicious IPs, URLs, or domains. For example, GuardDuty can detect compromised EC2 instances serving malware or mining bitcoin. It also monitors AWS account access behavior for signs of compromise, such as unauthorized infrastructure deployments, like instances deployed in a region that has never been used, or unusual API calls, like a password policy change to reduce password strength. GuardDuty informs you of the status of your AWS environment by producing security findings that you can view in the GuardDuty console or through Amazon CloudWatch events. For more information, see <a href="https://docs.aws.amazon.com/guardduty/latest/ug/what-is-guardduty.html"> Amazon GuardDuty User Guide</a>. """


    class ListDetectorsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_detectors` for valid parameters"""
            pass
    class ListFindingsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_findings` for valid parameters"""
            pass
    class ListIPSetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_ip_sets` for valid parameters"""
            pass
    class ListThreatIntelSetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_threat_intel_sets` for valid parameters"""
            pass
    class ListInvitationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_invitations` for valid parameters"""
            pass
    class ListMembersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_members` for valid parameters"""
            pass
    class ListFiltersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_filters` for valid parameters"""
            pass


    class AcceptInvitationRequest(Mapping):
        pass
    
    class AcceptInvitationResponse(Mapping):
        pass
    
    class ArchiveFindingsRequest(Mapping):
        pass
    
    class ArchiveFindingsResponse(Mapping):
        pass
    
    class FindingId(object):
        pass
    
    class CreateDetectorRequest(Mapping):
        pass
    
    class CreateDetectorResponse(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class TagMap(object):
        pass
    
    class CreateFilterRequest(Mapping):
        pass
    
    class CreateFilterResponse(Mapping):
        pass
    
    class FindingCriteria(Mapping):
        pass
    
    class CreateIPSetRequest(Mapping):
        pass
    
    class CreateIPSetResponse(Mapping):
        pass
    
    class CreateMembersRequest(Mapping):
        pass
    
    class CreateMembersResponse(Mapping):
        pass
    
    class AccountDetail(Mapping):
        pass
    
    class CreateSampleFindingsRequest(Mapping):
        pass
    
    class CreateSampleFindingsResponse(Mapping):
        pass
    
    class FindingType(object):
        pass
    
    class CreateThreatIntelSetRequest(Mapping):
        pass
    
    class CreateThreatIntelSetResponse(Mapping):
        pass
    
    class DeclineInvitationsRequest(Mapping):
        pass
    
    class DeclineInvitationsResponse(Mapping):
        pass
    
    class AccountId(object):
        pass
    
    class DeleteDetectorRequest(Mapping):
        pass
    
    class DeleteDetectorResponse(Mapping):
        pass
    
    class DeleteFilterRequest(Mapping):
        pass
    
    class DeleteFilterResponse(Mapping):
        pass
    
    class DeleteIPSetRequest(Mapping):
        pass
    
    class DeleteIPSetResponse(Mapping):
        pass
    
    class DeleteInvitationsRequest(Mapping):
        pass
    
    class DeleteInvitationsResponse(Mapping):
        pass
    
    class DeleteMembersRequest(Mapping):
        pass
    
    class DeleteMembersResponse(Mapping):
        pass
    
    class DeleteThreatIntelSetRequest(Mapping):
        pass
    
    class DeleteThreatIntelSetResponse(Mapping):
        pass
    
    class DisassociateFromMasterAccountRequest(Mapping):
        pass
    
    class DisassociateFromMasterAccountResponse(Mapping):
        pass
    
    class DisassociateMembersRequest(Mapping):
        pass
    
    class DisassociateMembersResponse(Mapping):
        pass
    
    class GetDetectorRequest(Mapping):
        pass
    
    class GetDetectorResponse(Mapping):
        pass
    
    class GetFilterRequest(Mapping):
        pass
    
    class GetFilterResponse(Mapping):
        pass
    
    class GetFindingsRequest(Mapping):
        pass
    
    class GetFindingsResponse(Mapping):
        pass
    
    class SortCriteria(Mapping):
        pass
    
    class GetFindingsStatisticsRequest(Mapping):
        pass
    
    class GetFindingsStatisticsResponse(Mapping):
        pass
    
    class FindingStatisticType(object):
        pass
    
    class GetIPSetRequest(Mapping):
        pass
    
    class GetIPSetResponse(Mapping):
        pass
    
    class GetInvitationsCountRequest(Mapping):
        pass
    
    class GetInvitationsCountResponse(Mapping):
        pass
    
    class GetMasterAccountRequest(Mapping):
        pass
    
    class GetMasterAccountResponse(Mapping):
        pass
    
    class GetMembersRequest(Mapping):
        pass
    
    class GetMembersResponse(Mapping):
        pass
    
    class GetThreatIntelSetRequest(Mapping):
        pass
    
    class GetThreatIntelSetResponse(Mapping):
        pass
    
    class InviteMembersRequest(Mapping):
        pass
    
    class InviteMembersResponse(Mapping):
        pass
    
    class ListDetectorsRequest(Mapping):
        pass
    
    class ListDetectorsResponse(Mapping):
        pass
    
    class ListFiltersRequest(Mapping):
        pass
    
    class ListFiltersResponse(Mapping):
        pass
    
    class ListFindingsRequest(Mapping):
        pass
    
    class ListFindingsResponse(Mapping):
        pass
    
    class ListIPSetsRequest(Mapping):
        pass
    
    class ListIPSetsResponse(Mapping):
        pass
    
    class ListInvitationsRequest(Mapping):
        pass
    
    class ListInvitationsResponse(Mapping):
        pass
    
    class ListMembersRequest(Mapping):
        pass
    
    class ListMembersResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class ListThreatIntelSetsRequest(Mapping):
        pass
    
    class ListThreatIntelSetsResponse(Mapping):
        pass
    
    class StartMonitoringMembersRequest(Mapping):
        pass
    
    class StartMonitoringMembersResponse(Mapping):
        pass
    
    class StopMonitoringMembersRequest(Mapping):
        pass
    
    class StopMonitoringMembersResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UnarchiveFindingsRequest(Mapping):
        pass
    
    class UnarchiveFindingsResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateDetectorRequest(Mapping):
        pass
    
    class UpdateDetectorResponse(Mapping):
        pass
    
    class UpdateFilterRequest(Mapping):
        pass
    
    class UpdateFilterResponse(Mapping):
        pass
    
    class UpdateFindingsFeedbackRequest(Mapping):
        pass
    
    class UpdateFindingsFeedbackResponse(Mapping):
        pass
    
    class UpdateIPSetRequest(Mapping):
        pass
    
    class UpdateIPSetResponse(Mapping):
        pass
    
    class UpdateThreatIntelSetRequest(Mapping):
        pass
    
    class UpdateThreatIntelSetResponse(Mapping):
        pass
    
    def accept_invitation(self, InvitationId: str, MasterId: str, DetectorId: str) -> AcceptInvitationResponse:
        """Accepts the invitation to be monitored by a master GuardDuty account.

        :param self:
        :param InvitationId:  This value is used to validate the master account to the member account.<br/>MasterId:  The account ID of the master GuardDuty account whose invitation you're accepting.<br/>DetectorId:  The unique ID of the detector of the GuardDuty member account.<br/>
        :return:         """
        pass

    def archive_findings(self, FindingIds: List[FindingId], DetectorId: str) -> ArchiveFindingsResponse:
        """Archives Amazon GuardDuty findings specified by the list of finding IDs.

        :param self:
        :param FindingIds:  IDs of the findings that you want to archive.<br/>DetectorId:  The ID of the detector that specifies the GuardDuty service whose findings you want to archive.<br/>
        :return:         """
        pass

    def create_detector(self, Enable: Boolean, ClientToken: str=None, FindingPublishingFrequency: str=None, Tags: TagMap=None) -> CreateDetectorResponse:
        """Creates a single Amazon GuardDuty detector. A detector is an object that represents the GuardDuty service. A detector must be created in order for GuardDuty to become operational.

        :param self:
        :param Tags:  The tags to be added to a new detector resource.<br/>FindingPublishingFrequency:  A enum value that specifies how frequently customer got Finding updates published.<br/>ClientToken:  The idempotency token for the create request.<br/>Enable:  A boolean value that specifies whether the detector is to be enabled.<br/>
        :return: DetectorId:  The unique ID of the created detector.<br/>        """
        pass

    def create_filter(self, FindingCriteria: FindingCriteria, Name: str, DetectorId: str, Description: str=None, Action: str=None, Rank: int=None, ClientToken: str=None, Tags: TagMap=None) -> CreateFilterResponse:
        """Creates a filter using the specified finding criteria.

        :param self:
        :param Tags:  The tags to be added to a new filter resource.<br/>ClientToken:  The idempotency token for the create request.<br/>FindingCriteria:  Represents the criteria to be used in the filter for querying findings.:<br/>Rank:  Specifies the position of the filter in the list of current filters. Also specifies the order in which this filter is applied to the findings.<br/>Action:  Specifies the action that is to be applied to the findings that match the filter.<br/>Description:  The description of the filter.<br/>Name:  The name of the filter.<br/>DetectorId:  The unique ID of the detector of the GuardDuty account for which you want to create a filter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Criterion</i> Represents a map of finding properties that match specified conditions and values when querying findings.<br/>
        :return: Name:  The name of the successfully created filter.<br/>        """
        pass

    def create_ip_set(self, Activate: Boolean, Location: str, Format: str, Name: str, DetectorId: str, ClientToken: str=None, Tags: TagMap=None) -> CreateIPSetResponse:
        """Creates a new IPSet - a list of trusted IP addresses that have been whitelisted for secure communication with AWS infrastructure and applications.

        :param self:
        :param Tags:  The tags to be added to a new IP set resource.<br/>ClientToken:  The idempotency token for the create request.<br/>Activate:  A boolean value that indicates whether GuardDuty is to start using the uploaded IPSet.<br/>Location:  The URI of the file that contains the IPSet. For example (https://s3.us-west-2.amazonaws.com/my-bucket/my-object-key)<br/>Format:  The format of the file that contains the IPSet.<br/>Name:  The user friendly name to identify the IPSet. This name is displayed in all findings that are triggered by activity that involves IP addresses included in this IPSet.<br/>DetectorId:  The unique ID of the detector of the GuardDuty account for which you want to create an IPSet.<br/>
        :return: IpSetId:  The ID of the IPSet resource.<br/>        """
        pass

    def create_members(self, AccountDetails: List[AccountDetail], DetectorId: str) -> CreateMembersResponse:
        """Creates member accounts of the current AWS account by specifying a list of AWS account IDs. The current AWS account can then invite these members to manage GuardDuty in their accounts.

        :param self:
        :param AccountDetails:  A list of account ID and email address pairs of the accounts that you want to associate with the master GuardDuty account.<br/>DetectorId:  The unique ID of the detector of the GuardDuty account with which you want to associate member accounts.<br/>
        :return: UnprocessedAccounts:  A list of objects containing the unprocessed account and a result string explaining why it was unprocessed.<br/>        """
        pass

    def create_sample_findings(self, DetectorId: str, FindingTypes: List[FindingType]=None) -> CreateSampleFindingsResponse:
        """Generates example findings of types specified by the list of finding types. If 'NULL' is specified for findingTypes, the API generates example findings of all supported finding types.

        :param self:
        :param FindingTypes:  Types of sample findings that you want to generate.<br/>DetectorId:  The ID of the detector to create sample findings for.<br/>
        :return:         """
        pass

    def create_threat_intel_set(self, Activate: Boolean, Location: str, Format: str, Name: str, DetectorId: str, ClientToken: str=None, Tags: TagMap=None) -> CreateThreatIntelSetResponse:
        """Create a new ThreatIntelSet. ThreatIntelSets consist of known malicious IP addresses. GuardDuty generates findings based on ThreatIntelSets.

        :param self:
        :param Tags:  The tags to be added to a new Threat List resource.<br/>ClientToken:  The idempotency token for the create request.<br/>Activate:  A boolean value that indicates whether GuardDuty is to start using the uploaded ThreatIntelSet.<br/>Location:  The URI of the file that contains the ThreatIntelSet. For example (https://s3.us-west-2.amazonaws.com/my-bucket/my-object-key).<br/>Format:  The format of the file that contains the ThreatIntelSet.<br/>Name:  A user-friendly ThreatIntelSet name that is displayed in all finding generated by activity that involves IP addresses included in this ThreatIntelSet.<br/>DetectorId:  The unique ID of the detector of the GuardDuty account for which you want to create a threatIntelSet.<br/>
        :return: ThreatIntelSetId:  The ID of the ThreatIntelSet resource.<br/>        """
        pass

    def decline_invitations(self, AccountIds: List[AccountId]) -> DeclineInvitationsResponse:
        """Declines invitations sent to the current member account by AWS account specified by their account IDs.

        :param self:
        :param AccountIds:  A list of account IDs of the AWS accounts that sent invitations to the current member account that you want to decline invitations from.<br/>
        :return: UnprocessedAccounts:  A list of objects containing the unprocessed account and a result string explaining why it was unprocessed.<br/>        """
        pass

    def delete_detector(self, DetectorId: str) -> DeleteDetectorResponse:
        """Deletes a Amazon GuardDuty detector specified by the detector ID.

        :param self:
        :param DetectorId:  The unique ID of the detector that you want to delete.<br/>
        :return:         """
        pass

    def delete_filter(self, FilterName: str, DetectorId: str) -> DeleteFilterResponse:
        """Deletes the filter specified by the filter name.

        :param self:
        :param FilterName:  The name of the filter you want to delete.<br/>DetectorId:  The unique ID of the detector the filter is associated with.<br/>
        :return:         """
        pass

    def delete_ip_set(self, IpSetId: str, DetectorId: str) -> DeleteIPSetResponse:
        """Deletes the IPSet specified by the IPSet ID.

        :param self:
        :param IpSetId:  The unique ID of the ipSet you want to delete.<br/>DetectorId:  The unique ID of the detector the ipSet is associated with.<br/>
        :return:         """
        pass

    def delete_invitations(self, AccountIds: List[AccountId]) -> DeleteInvitationsResponse:
        """Deletes invitations sent to the current member account by AWS accounts specified by their account IDs.

        :param self:
        :param AccountIds:  A list of account IDs of the AWS accounts that sent invitations to the current member account that you want to delete invitations from.<br/>
        :return: UnprocessedAccounts:  A list of objects containing the unprocessed account and a result string explaining why it was unprocessed.<br/>        """
        pass

    def delete_members(self, AccountIds: List[AccountId], DetectorId: str) -> DeleteMembersResponse:
        """Deletes GuardDuty member accounts (to the current GuardDuty master account) specified by the account IDs.

        :param self:
        :param AccountIds:  A list of account IDs of the GuardDuty member accounts that you want to delete.<br/>DetectorId:  The unique ID of the detector of the GuardDuty account whose members you want to delete.<br/>
        :return: UnprocessedAccounts:  A list of objects containing the unprocessed account and a result string explaining why it was unprocessed.<br/>        """
        pass

    def delete_threat_intel_set(self, ThreatIntelSetId: str, DetectorId: str) -> DeleteThreatIntelSetResponse:
        """Deletes ThreatIntelSet specified by the ThreatIntelSet ID.

        :param self:
        :param ThreatIntelSetId:  The unique ID of the threatIntelSet you want to delete.<br/>DetectorId:  The unique ID of the detector the threatIntelSet is associated with.<br/>
        :return:         """
        pass

    def disassociate_from_master_account(self, DetectorId: str) -> DisassociateFromMasterAccountResponse:
        """Disassociates the current GuardDuty member account from its master account.

        :param self:
        :param DetectorId:  The unique ID of the detector of the GuardDuty member account.<br/>
        :return:         """
        pass

    def disassociate_members(self, AccountIds: List[AccountId], DetectorId: str) -> DisassociateMembersResponse:
        """Disassociates GuardDuty member accounts (to the current GuardDuty master account) specified by the account IDs.

        :param self:
        :param AccountIds:  A list of account IDs of the GuardDuty member accounts that you want to disassociate from master.<br/>DetectorId:  The unique ID of the detector of the GuardDuty account whose members you want to disassociate from master.<br/>
        :return: UnprocessedAccounts:  A list of objects containing the unprocessed account and a result string explaining why it was unprocessed.<br/>        """
        pass

    def get_detector(self, DetectorId: str) -> GetDetectorResponse:
        """Retrieves an Amazon GuardDuty detector specified by the detectorId.

        :param self:
        :param DetectorId:  The unique ID of the detector that you want to get.<br/>
        :return: Tags:  The tags of the detector resource.<br/>UpdatedAt:  Detector last update timestamp.<br/>Status:  The detector status.<br/>ServiceRole:  The GuardDuty service role.<br/>FindingPublishingFrequency:  Finding publishing frequency.<br/>CreatedAt:  Detector creation timestamp.<br/>        """
        pass

    def get_filter(self, FilterName: str, DetectorId: str) -> GetFilterResponse:
        """Returns the details of the filter specified by the filter name.

        :param self:
        :param FilterName:  The name of the filter you want to get.<br/>DetectorId:  The unique ID of the detector the filter is associated with.<br/>
        :return: Tags:  The tags of the filter resource.<br/>FindingCriteria:  Represents the criteria to be used in the filter for querying findings.:<br/>Rank:  Specifies the position of the filter in the list of current filters. Also specifies the order in which this filter is applied to the findings.<br/>Action:  Specifies the action that is to be applied to the findings that match the filter.<br/>Description:  The description of the filter.<br/>Name:  The name of the filter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Criterion</i> Represents a map of finding properties that match specified conditions and values when querying findings.<br/>        """
        pass

    def get_findings(self, FindingIds: List[FindingId], DetectorId: str, SortCriteria: SortCriteria=None) -> GetFindingsResponse:
        """Describes Amazon GuardDuty findings specified by finding IDs.

        :param self:
        :param SortCriteria:  Represents the criteria used for sorting findings.:<br/>FindingIds:  IDs of the findings that you want to retrieve.<br/>DetectorId:  The ID of the detector that specifies the GuardDuty service whose findings you want to retrieve.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OrderBy</i> Order by which the sorted findings are to be displayed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttributeName</i> Represents the finding attribute (for example, accountId) by which to sort findings.<br/>
        :return: Findings:  A list of findings.<br/>        """
        pass

    def get_findings_statistics(self, FindingStatisticTypes: List[FindingStatisticType], DetectorId: str, FindingCriteria: FindingCriteria=None) -> GetFindingsStatisticsResponse:
        """Lists Amazon GuardDuty findings' statistics for the specified detector ID.

        :param self:
        :param FindingCriteria:  Represents the criteria used for querying findings.:<br/>FindingStatisticTypes:  Types of finding statistics to retrieve.<br/>DetectorId:  The ID of the detector that specifies the GuardDuty service whose findings' statistics you want to retrieve.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Criterion</i> Represents a map of finding properties that match specified conditions and values when querying findings.<br/>
        :return: FindingStatistics:  Finding statistics object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CountBySeverity</i> Represents a map of severity to count statistic for a set of findings<br/>        """
        pass

    def get_ip_set(self, IpSetId: str, DetectorId: str) -> GetIPSetResponse:
        """Retrieves the IPSet specified by the IPSet ID.

        :param self:
        :param IpSetId:  The unique ID of the ipSet you want to get.<br/>DetectorId:  The unique ID of the detector the ipSet is associated with.<br/>
        :return: Tags:  The tags of the IP set resource.<br/>Status:  The status of ipSet file uploaded.<br/>Location:  The URI of the file that contains the IPSet. For example (https://s3.us-west-2.amazonaws.com/my-bucket/my-object-key)<br/>Format:  The format of the file that contains the IPSet.<br/>Name:  The user friendly name to identify the IPSet. This name is displayed in all findings that are triggered by activity that involves IP addresses included in this IPSet.<br/>        """
        pass

    def get_invitations_count(self) -> GetInvitationsCountResponse:
        """Returns the count of all GuardDuty membership invitations that were sent to the current member account except the currently accepted invitation.

        :param self:
        :param 
        :return: InvitationsCount:  The number of received invitations.<br/>        """
        pass

    def get_master_account(self, DetectorId: str) -> GetMasterAccountResponse:
        """Provides the details for the GuardDuty master account to the current GuardDuty member account.

        :param self:
        :param DetectorId:  The unique ID of the detector of the GuardDuty member account.<br/>
        :return: Master:  Master account details.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InvitedAt</i> Timestamp at which the invitation was sent<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RelationshipStatus</i> The status of the relationship between the master and member accounts.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InvitationId</i> This value is used to validate the master account to the member account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountId</i> Master account ID<br/>        """
        pass

    def get_members(self, AccountIds: List[AccountId], DetectorId: str) -> GetMembersResponse:
        """Retrieves GuardDuty member accounts (to the current GuardDuty master account) specified by the account IDs.

        :param self:
        :param AccountIds:  A list of account IDs of the GuardDuty member accounts that you want to describe.<br/>DetectorId:  The unique ID of the detector of the GuardDuty account whose members you want to retrieve.<br/>
        :return: UnprocessedAccounts:  A list of objects containing the unprocessed account and a result string explaining why it was unprocessed.<br/>Members:  A list of members.<br/>        """
        pass

    def get_threat_intel_set(self, ThreatIntelSetId: str, DetectorId: str) -> GetThreatIntelSetResponse:
        """Retrieves the ThreatIntelSet that is specified by the ThreatIntelSet ID.

        :param self:
        :param ThreatIntelSetId:  The unique ID of the threatIntelSet you want to get.<br/>DetectorId:  The unique ID of the detector the threatIntelSet is associated with.<br/>
        :return: Tags:  The tags of the Threat List resource.<br/>Status:  The status of threatIntelSet file uploaded.<br/>Location:  The URI of the file that contains the ThreatIntelSet. For example (https://s3.us-west-2.amazonaws.com/my-bucket/my-object-key).<br/>Format:  The format of the threatIntelSet.<br/>Name:  A user-friendly ThreatIntelSet name that is displayed in all finding generated by activity that involves IP addresses included in this ThreatIntelSet.<br/>        """
        pass

    def invite_members(self, AccountIds: List[AccountId], DetectorId: str, DisableEmailNotification: Boolean=None, Message: str=None) -> InviteMembersResponse:
        """Invites other AWS accounts (created as members of the current AWS account by CreateMembers) to enable GuardDuty and allow the current AWS account to view and manage these accounts' GuardDuty findings on their behalf as the master account.

        :param self:
        :param Message:  The invitation message that you want to send to the accounts that you’re inviting to GuardDuty as members.<br/>DisableEmailNotification:  A boolean value that specifies whether you want to disable email notification to the accounts that you’re inviting to GuardDuty as members.<br/>AccountIds:  A list of account IDs of the accounts that you want to invite to GuardDuty as members.<br/>DetectorId:  The unique ID of the detector of the GuardDuty account with which you want to invite members.<br/>
        :return: UnprocessedAccounts:  A list of objects containing the unprocessed account and a result string explaining why it was unprocessed.<br/>        """
        pass

    def list_detectors(self, MaxResults: int=None, NextToken: str=None) -> ListDetectorsResponse:
        """Lists detectorIds of all the existing Amazon GuardDuty detector resources.

        :param self:
        :param NextToken:  You can use this parameter when paginating results. Set the value of this parameter to null on your first call to the list action. For subsequent calls to the action fill nextToken in the request with the value of NextToken from the previous response to continue listing data.<br/>MaxResults:  You can use this parameter to indicate the maximum number of items you want in the response. The default value is 50. The maximum value is 50.<br/>
        :return: NextToken:  Pagination parameter to be used on the next list operation to retrieve more items.<br/>DetectorIds:  A list of detector Ids.<br/>        """
        pass

    def list_filters(self, DetectorId: str, MaxResults: int=None, NextToken: str=None) -> ListFiltersResponse:
        """Returns a paginated list of the current filters.

        :param self:
        :param NextToken:  You can use this parameter when paginating results. Set the value of this parameter to null on your first call to the list action. For subsequent calls to the action fill nextToken in the request with the value of NextToken from the previous response to continue listing data.<br/>MaxResults:  You can use this parameter to indicate the maximum number of items you want in the response. The default value is 50. The maximum value is 50.<br/>DetectorId:  The unique ID of the detector the filter is associated with.<br/>
        :return: NextToken:  Pagination parameter to be used on the next list operation to retrieve more items.<br/>FilterNames:  A list of filter names<br/>        """
        pass

    def list_findings(self, DetectorId: str, FindingCriteria: FindingCriteria=None, SortCriteria: SortCriteria=None, MaxResults: int=None, NextToken: str=None) -> ListFindingsResponse:
        """Lists Amazon GuardDuty findings for the specified detector ID.

        :param self:
        :param NextToken:  You can use this parameter when paginating results. Set the value of this parameter to null on your first call to the list action. For subsequent calls to the action fill nextToken in the request with the value of NextToken from the previous response to continue listing data.<br/>MaxResults:  You can use this parameter to indicate the maximum number of items you want in the response. The default value is 50. The maximum value is 50.<br/>SortCriteria:  Represents the criteria used for sorting findings.:<br/>FindingCriteria:  Represents the criteria used for querying findings.:<br/>DetectorId:  The ID of the detector that specifies the GuardDuty service whose findings you want to list.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Criterion</i> Represents a map of finding properties that match specified conditions and values when querying findings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OrderBy</i> Order by which the sorted findings are to be displayed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttributeName</i> Represents the finding attribute (for example, accountId) by which to sort findings.<br/>
        :return: NextToken:  Pagination parameter to be used on the next list operation to retrieve more items.<br/>FindingIds:  The IDs of the findings you are listing.<br/>        """
        pass

    def list_ip_sets(self, DetectorId: str, MaxResults: int=None, NextToken: str=None) -> ListIPSetsResponse:
        """Lists the IPSets of the GuardDuty service specified by the detector ID.

        :param self:
        :param NextToken:  You can use this parameter when paginating results. Set the value of this parameter to null on your first call to the list action. For subsequent calls to the action fill nextToken in the request with the value of NextToken from the previous response to continue listing data.<br/>MaxResults:  You can use this parameter to indicate the maximum number of items you want in the response. The default value is 50. The maximum value is 50.<br/>DetectorId:  The unique ID of the detector the ipSet is associated with.<br/>
        :return: NextToken:  Pagination parameter to be used on the next list operation to retrieve more items.<br/>IpSetIds:  The IDs of the IPSet resources.<br/>        """
        pass

    def list_invitations(self, MaxResults: int=None, NextToken: str=None) -> ListInvitationsResponse:
        """Lists all GuardDuty membership invitations that were sent to the current AWS account.

        :param self:
        :param NextToken:  You can use this parameter when paginating results. Set the value of this parameter to null on your first call to the list action. For subsequent calls to the action fill nextToken in the request with the value of NextToken from the previous response to continue listing data.<br/>MaxResults:  You can use this parameter to indicate the maximum number of items you want in the response. The default value is 50. The maximum value is 50.<br/>
        :return: NextToken:  Pagination parameter to be used on the next list operation to retrieve more items.<br/>Invitations:  A list of invitation descriptions.<br/>        """
        pass

    def list_members(self, DetectorId: str, MaxResults: int=None, NextToken: str=None, OnlyAssociated: str=None) -> ListMembersResponse:
        """Lists details about all member accounts for the current GuardDuty master account.

        :param self:
        :param OnlyAssociated:  Specifies whether to only return associated members or to return all members (including members which haven't been invited yet or have been disassociated).<br/>NextToken:  You can use this parameter when paginating results. Set the value of this parameter to null on your first call to the list action. For subsequent calls to the action fill nextToken in the request with the value of NextToken from the previous response to continue listing data.<br/>MaxResults:  You can use this parameter to indicate the maximum number of items you want in the response. The default value is 50. The maximum value is 50.<br/>DetectorId:  The unique ID of the detector the member is associated with.<br/>
        :return: NextToken:  Pagination parameter to be used on the next list operation to retrieve more items.<br/>Members:  A list of members.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceArn: str) -> ListTagsForResourceResponse:
        """Lists tags for a resource. Tagging is currently supported for detectors, finding filters, IP sets, and Threat Intel sets, with a limit of 50 tags per resource. When invoked, this operation returns all assigned tags for a given resource..

        :param self:
        :param ResourceArn:  The Amazon Resource Name (ARN) for the given GuardDuty resource <br/>
        :return: Tags:  <br/>        """
        pass

    def list_threat_intel_sets(self, DetectorId: str, MaxResults: int=None, NextToken: str=None) -> ListThreatIntelSetsResponse:
        """Lists the ThreatIntelSets of the GuardDuty service specified by the detector ID.

        :param self:
        :param NextToken:  You can use this parameter when paginating results. Set the value of this parameter to null on your first call to the list action. For subsequent calls to the action fill nextToken in the request with the value of NextToken from the previous response to continue listing data.<br/>MaxResults:  You can use this parameter to indicate the maximum number of items you want in the response. The default value is 50. The maximum value is 50.<br/>DetectorId:  The unique ID of the detector the threatIntelSet is associated with.<br/>
        :return: NextToken:  Pagination parameter to be used on the next list operation to retrieve more items.<br/>ThreatIntelSetIds:  The IDs of the ThreatIntelSet resources.<br/>        """
        pass

    def start_monitoring_members(self, AccountIds: List[AccountId], DetectorId: str) -> StartMonitoringMembersResponse:
        """Re-enables GuardDuty to monitor findings of the member accounts specified by the account IDs. A master GuardDuty account can run this command after disabling GuardDuty from monitoring these members' findings by running StopMonitoringMembers.

        :param self:
        :param AccountIds:  A list of account IDs of the GuardDuty member accounts whose findings you want the master account to monitor.<br/>DetectorId:  The unique ID of the detector of the GuardDuty account whom you want to re-enable to monitor members' findings.<br/>
        :return: UnprocessedAccounts:  A list of objects containing the unprocessed account and a result string explaining why it was unprocessed.<br/>        """
        pass

    def stop_monitoring_members(self, AccountIds: List[AccountId], DetectorId: str) -> StopMonitoringMembersResponse:
        """Disables GuardDuty from monitoring findings of the member accounts specified by the account IDs. After running this command, a master GuardDuty account can run StartMonitoringMembers to re-enable GuardDuty to monitor these members’ findings.

        :param self:
        :param AccountIds:  A list of account IDs of the GuardDuty member accounts whose findings you want the master account to stop monitoring.<br/>DetectorId:  The unique ID of the detector of the GuardDuty account that you want to stop from monitor members' findings.<br/>
        :return: UnprocessedAccounts:  A list of objects containing the unprocessed account and a result string explaining why it was unprocessed.<br/>        """
        pass

    def tag_resource(self, Tags: TagMap, ResourceArn: str) -> TagResourceResponse:
        """Adds tags to a resource.

        :param self:
        :param Tags:  The tags to be added to a resource.<br/>ResourceArn:  The Amazon Resource Name (ARN) for the given GuardDuty resource <br/>
        :return:         """
        pass

    def unarchive_findings(self, FindingIds: List[FindingId], DetectorId: str) -> UnarchiveFindingsResponse:
        """Unarchives Amazon GuardDuty findings specified by the list of finding IDs.

        :param self:
        :param FindingIds:  IDs of the findings that you want to unarchive.<br/>DetectorId:  The ID of the detector that specifies the GuardDuty service whose findings you want to unarchive.<br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[TagKey], ResourceArn: str) -> UntagResourceResponse:
        """Removes tags from a resource.

        :param self:
        :param TagKeys:  The tag keys to remove from a resource.<br/>ResourceArn:  The Amazon Resource Name (ARN) for the given GuardDuty resource <br/>
        :return:         """
        pass

    def update_detector(self, DetectorId: str, Enable: Boolean=None, FindingPublishingFrequency: str=None) -> UpdateDetectorResponse:
        """Updates an Amazon GuardDuty detector specified by the detectorId.

        :param self:
        :param FindingPublishingFrequency:  A enum value that specifies how frequently customer got Finding updates published.<br/>Enable:  Updated boolean value for the detector that specifies whether the detector is enabled.<br/>DetectorId:  The unique ID of the detector that you want to update.<br/>
        :return:         """
        pass

    def update_filter(self, FilterName: str, DetectorId: str, Description: str=None, Action: str=None, Rank: int=None, FindingCriteria: FindingCriteria=None) -> UpdateFilterResponse:
        """Updates the filter specified by the filter name.

        :param self:
        :param FindingCriteria:  Represents the criteria to be used in the filter for querying findings.:<br/>Rank:  Specifies the position of the filter in the list of current filters. Also specifies the order in which this filter is applied to the findings.<br/>Action:  Specifies the action that is to be applied to the findings that match the filter.<br/>Description:  The description of the filter.<br/>FilterName:  The name of the filter.<br/>DetectorId:  The unique ID of the detector that specifies the GuardDuty service where you want to update a filter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Criterion</i> Represents a map of finding properties that match specified conditions and values when querying findings.<br/>
        :return: Name:  The name of the filter.<br/>        """
        pass

    def update_findings_feedback(self, Feedback: str, FindingIds: List[FindingId], DetectorId: str, Comments: str=None) -> UpdateFindingsFeedbackResponse:
        """Marks specified Amazon GuardDuty findings as useful or not useful.

        :param self:
        :param Comments:  Additional feedback about the GuardDuty findings.<br/>Feedback:  Valid values: USEFUL | NOT_USEFUL<br/>FindingIds:  IDs of the findings that you want to mark as useful or not useful.<br/>DetectorId:  The ID of the detector that specifies the GuardDuty service whose findings you want to mark as useful or not useful.<br/>
        :return:         """
        pass

    def update_ip_set(self, IpSetId: str, DetectorId: str, Name: str=None, Location: str=None, Activate: Boolean=None) -> UpdateIPSetResponse:
        """Updates the IPSet specified by the IPSet ID.

        :param self:
        :param Activate:  The updated boolean value that specifies whether the IPSet is active or not.<br/>Location:  The updated URI of the file that contains the IPSet. For example (https://s3.us-west-2.amazonaws.com/my-bucket/my-object-key).<br/>Name:  The unique ID that specifies the IPSet that you want to update.<br/>IpSetId:  The unique ID that specifies the IPSet that you want to update.<br/>DetectorId:  The detectorID that specifies the GuardDuty service whose IPSet you want to update.<br/>
        :return:         """
        pass

    def update_threat_intel_set(self, ThreatIntelSetId: str, DetectorId: str, Name: str=None, Location: str=None, Activate: Boolean=None) -> UpdateThreatIntelSetResponse:
        """Updates the ThreatIntelSet specified by ThreatIntelSet ID.

        :param self:
        :param Activate:  The updated boolean value that specifies whether the ThreateIntelSet is active or not.<br/>Location:  The updated URI of the file that contains the ThreateIntelSet. For example (https://s3.us-west-2.amazonaws.com/my-bucket/my-object-key)<br/>Name:  The unique ID that specifies the ThreatIntelSet that you want to update.<br/>ThreatIntelSetId:  The unique ID that specifies the ThreatIntelSet that you want to update.<br/>DetectorId:  The detectorID that specifies the GuardDuty service whose ThreatIntelSet you want to update.<br/>
        :return:         """
        pass



class Health(BaseClient):
    """<fullname>AWS Health</fullname> The AWS Health API provides programmatic access to the AWS Health information that is presented in the <a href="https://phd.aws.amazon.com/phd/home#/">AWS Personal Health Dashboard</a>. You can get information about events that affect your AWS resources: <ul> <li>  <a>DescribeEvents</a>: Summary information about events. </li> <li>  <a>DescribeEventDetails</a>: Detailed information about one or more events. </li> <li>  <a>DescribeAffectedEntities</a>: Information about AWS resources that are affected by one or more events. </li> </ul> In addition, these operations provide information about event types and summary counts of events or affected entities: <ul> <li>  <a>DescribeEventTypes</a>: Information about the kinds of events that AWS Health tracks. </li> <li>  <a>DescribeEventAggregates</a>: A count of the number of events that meet specified criteria. </li> <li>  <a>DescribeEntityAggregates</a>: A count of the number of affected entities that meet specified criteria. </li> </ul> The Health API requires a Business or Enterprise support plan from <a href="http://aws.amazon.com/premiumsupport/">AWS Support</a>. Calling the Health API from an account that does not have a Business or Enterprise support plan causes a <code>SubscriptionRequiredException</code>.  For authentication of requests, AWS Health uses the <a href="https://docs.aws.amazon.com/general/latest/gr/signature-version-4.html">Signature Version 4 Signing Process</a>. See the <a href="https://docs.aws.amazon.com/health/latest/ug/what-is-aws-health.html">AWS Health User Guide</a> for information about how to use the API.  <b>Service Endpoint</b>  The HTTP endpoint for the AWS Health API is: <ul> <li> https://health.us-east-1.amazonaws.com  </li> </ul>"""


    class DescribeAffectedEntitiesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_affected_entities` for valid parameters"""
            pass
    class DescribeEventAggregatesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_event_aggregates` for valid parameters"""
            pass
    class DescribeEventsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_events` for valid parameters"""
            pass
    class DescribeEventTypesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_event_types` for valid parameters"""
            pass


    class DescribeAffectedEntitiesRequest(Mapping):
        pass
    
    class DescribeAffectedEntitiesResponse(Mapping):
        pass
    
    class EntityFilter(Mapping):
        pass
    
    class DescribeEntityAggregatesRequest(Mapping):
        pass
    
    class DescribeEntityAggregatesResponse(Mapping):
        pass
    
    class eventArn(object):
        pass
    
    class DescribeEventAggregatesRequest(Mapping):
        pass
    
    class DescribeEventAggregatesResponse(Mapping):
        pass
    
    class EventFilter(Mapping):
        pass
    
    class DescribeEventDetailsRequest(Mapping):
        pass
    
    class DescribeEventDetailsResponse(Mapping):
        pass
    
    class DescribeEventTypesRequest(Mapping):
        pass
    
    class DescribeEventTypesResponse(Mapping):
        pass
    
    class EventTypeFilter(Mapping):
        pass
    
    class DescribeEventsRequest(Mapping):
        pass
    
    class DescribeEventsResponse(Mapping):
        pass
    
    def describe_affected_entities(self, filter: EntityFilter, locale: str=None, nextToken: str=None, maxResults: int=None) -> DescribeAffectedEntitiesResponse:
        """Returns a list of entities that have been affected by the specified events, based on the specified filter criteria. Entities can refer to individual customer resources, groups of customer resources, or any other construct, depending on the AWS service. Events that have impact beyond that of the affected entities, or where the extent of impact is unknown, include at least one entity indicating this. At least one event ARN is required. Results are sorted by the <code>lastUpdatedTime</code> of the entity, starting with the most recent.

        :param self:
        :param maxResults:  The maximum number of items to return in one batch, between 10 and 100, inclusive.<br/>nextToken:  If the results of a search are large, only a portion of the results are returned, and a <code>nextToken</code> pagination token is returned in the response. To retrieve the next batch of results, reissue the search request and include the returned token. When all results have been returned, the response does not contain a pagination token value.<br/>locale:  The locale (language) to return information in. English (en) is the default and the only supported value at this time.<br/>filter:  Values to narrow the results returned. At least one event ARN is required. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusCodes</i> A list of entity status codes (<code>IMPAIRED</code>, <code>UNIMPAIRED</code>, or <code>UNKNOWN</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> A map of entity tags attached to the affected entity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedTimes</i> A list of the most recent dates and times that the entity was updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>entityValues</i> A list of IDs for affected entities.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>entityArns</i> A list of entity ARNs (unique identifiers).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>eventArns</i> A list of event ARNs (unique identifiers). For example: <code>"arn:aws:health:us-east-1::event/EC2/EC2_INSTANCE_RETIREMENT_SCHEDULED/EC2_INSTANCE_RETIREMENT_SCHEDULED_ABC123-CDE456", "arn:aws:health:us-west-1::event/EBS/AWS_EBS_LOST_VOLUME/AWS_EBS_LOST_VOLUME_CHI789_JKL101"</code> <br/>
        :return: nextToken:  If the results of a search are large, only a portion of the results are returned, and a <code>nextToken</code> pagination token is returned in the response. To retrieve the next batch of results, reissue the search request and include the returned token. When all results have been returned, the response does not contain a pagination token value.<br/>entities:  The entities that match the filter criteria.<br/>        """
        pass

    def describe_entity_aggregates(self, eventArns: List[eventArn]=None) -> DescribeEntityAggregatesResponse:
        """Returns the number of entities that are affected by each of the specified events. If no events are specified, the counts of all affected entities are returned.

        :param self:
        :param eventArns:  A list of event ARNs (unique identifiers). For example: <code>"arn:aws:health:us-east-1::event/EC2/EC2_INSTANCE_RETIREMENT_SCHEDULED/EC2_INSTANCE_RETIREMENT_SCHEDULED_ABC123-CDE456", "arn:aws:health:us-west-1::event/EBS/AWS_EBS_LOST_VOLUME/AWS_EBS_LOST_VOLUME_CHI789_JKL101"</code> <br/>
        :return: entityAggregates:  The number of entities that are affected by each of the specified events.<br/>        """
        pass

    def describe_event_aggregates(self, aggregateField: str, filter: EventFilter=None, maxResults: int=None, nextToken: str=None) -> DescribeEventAggregatesResponse:
        """Returns the number of events of each event type (issue, scheduled change, and account notification). If no filter is specified, the counts of all events in each category are returned.

        :param self:
        :param nextToken:  If the results of a search are large, only a portion of the results are returned, and a <code>nextToken</code> pagination token is returned in the response. To retrieve the next batch of results, reissue the search request and include the returned token. When all results have been returned, the response does not contain a pagination token value.<br/>maxResults:  The maximum number of items to return in one batch, between 10 and 100, inclusive.<br/>aggregateField:  The only currently supported value is <code>eventTypeCategory</code>.<br/>filter:  Values to narrow the results returned.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>eventStatusCodes</i> A list of event status codes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> A map of entity tags attached to the affected entity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>eventTypeCategories</i> A list of event type category codes (<code>issue</code>, <code>scheduledChange</code>, or <code>accountNotification</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>entityValues</i> A list of entity identifiers, such as EC2 instance IDs (<code>i-34ab692e</code>) or EBS volumes (<code>vol-426ab23e</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>entityArns</i> A list of entity ARNs (unique identifiers).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedTimes</i> A list of dates and times that the event was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>endTimes</i> A list of dates and times that the event ended.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>startTimes</i> A list of dates and times that the event began.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>availabilityZones</i> A list of AWS availability zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>regions</i> A list of AWS regions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>services</i> The AWS services associated with the event. For example, <code>EC2</code>, <code>RDS</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>eventTypeCodes</i> A list of unique identifiers for event types. For example, <code>"AWS_EC2_SYSTEM_MAINTENANCE_EVENT","AWS_RDS_MAINTENANCE_SCHEDULED"</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>eventArns</i> A list of event ARNs (unique identifiers). For example: <code>"arn:aws:health:us-east-1::event/EC2/EC2_INSTANCE_RETIREMENT_SCHEDULED/EC2_INSTANCE_RETIREMENT_SCHEDULED_ABC123-CDE456", "arn:aws:health:us-west-1::event/EBS/AWS_EBS_LOST_VOLUME/AWS_EBS_LOST_VOLUME_CHI789_JKL101"</code> <br/>
        :return: nextToken:  If the results of a search are large, only a portion of the results are returned, and a <code>nextToken</code> pagination token is returned in the response. To retrieve the next batch of results, reissue the search request and include the returned token. When all results have been returned, the response does not contain a pagination token value.<br/>eventAggregates:  The number of events in each category that meet the optional filter criteria.<br/>        """
        pass

    def describe_event_details(self, eventArns: List[eventArn], locale: str=None) -> DescribeEventDetailsResponse:
        """Returns detailed information about one or more specified events. Information includes standard event data (region, service, etc., as returned by <a>DescribeEvents</a>), a detailed event description, and possible additional metadata that depends upon the nature of the event. Affected entities are not included; to retrieve those, use the <a>DescribeAffectedEntities</a> operation. If a specified event cannot be retrieved, an error message is returned for that event.

        :param self:
        :param locale:  The locale (language) to return information in. English (en) is the default and the only supported value at this time.<br/>eventArns:  A list of event ARNs (unique identifiers). For example: <code>"arn:aws:health:us-east-1::event/EC2/EC2_INSTANCE_RETIREMENT_SCHEDULED/EC2_INSTANCE_RETIREMENT_SCHEDULED_ABC123-CDE456", "arn:aws:health:us-west-1::event/EBS/AWS_EBS_LOST_VOLUME/AWS_EBS_LOST_VOLUME_CHI789_JKL101"</code> <br/>
        :return: failedSet:  Error messages for any events that could not be retrieved.<br/>successfulSet:  Information about the events that could be retrieved.<br/>        """
        pass

    def describe_event_types(self, filter: EventTypeFilter=None, locale: str=None, nextToken: str=None, maxResults: int=None) -> DescribeEventTypesResponse:
        """Returns the event types that meet the specified filter criteria. If no filter criteria are specified, all event types are returned, in no particular order.

        :param self:
        :param maxResults:  The maximum number of items to return in one batch, between 10 and 100, inclusive.<br/>nextToken:  If the results of a search are large, only a portion of the results are returned, and a <code>nextToken</code> pagination token is returned in the response. To retrieve the next batch of results, reissue the search request and include the returned token. When all results have been returned, the response does not contain a pagination token value.<br/>locale:  The locale (language) to return information in. English (en) is the default and the only supported value at this time.<br/>filter:  Values to narrow the results returned.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>eventTypeCategories</i> A list of event type category codes (<code>issue</code>, <code>scheduledChange</code>, or <code>accountNotification</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>services</i> The AWS services associated with the event. For example, <code>EC2</code>, <code>RDS</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>eventTypeCodes</i> A list of event type codes.<br/>
        :return: nextToken:  If the results of a search are large, only a portion of the results are returned, and a <code>nextToken</code> pagination token is returned in the response. To retrieve the next batch of results, reissue the search request and include the returned token. When all results have been returned, the response does not contain a pagination token value.<br/>eventTypes:  A list of event types that match the filter criteria. Event types have a category (<code>issue</code>, <code>accountNotification</code>, or <code>scheduledChange</code>), a service (for example, <code>EC2</code>, <code>RDS</code>, <code>DATAPIPELINE</code>, <code>BILLING</code>), and a code (in the format <code>AWS_<i>SERVICE</i>_<i>DESCRIPTION</i> </code>; for example, <code>AWS_EC2_SYSTEM_MAINTENANCE_EVENT</code>).<br/>        """
        pass

    def describe_events(self, filter: EventFilter=None, nextToken: str=None, maxResults: int=None, locale: str=None) -> DescribeEventsResponse:
        """Returns information about events that meet the specified filter criteria. Events are returned in a summary form and do not include the detailed description, any additional metadata that depends on the event type, or any affected resources. To retrieve that information, use the <a>DescribeEventDetails</a> and <a>DescribeAffectedEntities</a> operations. If no filter criteria are specified, all events are returned. Results are sorted by <code>lastModifiedTime</code>, starting with the most recent.

        :param self:
        :param locale:  The locale (language) to return information in. English (en) is the default and the only supported value at this time.<br/>maxResults:  The maximum number of items to return in one batch, between 10 and 100, inclusive.<br/>nextToken:  If the results of a search are large, only a portion of the results are returned, and a <code>nextToken</code> pagination token is returned in the response. To retrieve the next batch of results, reissue the search request and include the returned token. When all results have been returned, the response does not contain a pagination token value.<br/>filter:  Values to narrow the results returned.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>eventStatusCodes</i> A list of event status codes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> A map of entity tags attached to the affected entity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>eventTypeCategories</i> A list of event type category codes (<code>issue</code>, <code>scheduledChange</code>, or <code>accountNotification</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>entityValues</i> A list of entity identifiers, such as EC2 instance IDs (<code>i-34ab692e</code>) or EBS volumes (<code>vol-426ab23e</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>entityArns</i> A list of entity ARNs (unique identifiers).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedTimes</i> A list of dates and times that the event was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>endTimes</i> A list of dates and times that the event ended.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>startTimes</i> A list of dates and times that the event began.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>availabilityZones</i> A list of AWS availability zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>regions</i> A list of AWS regions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>services</i> The AWS services associated with the event. For example, <code>EC2</code>, <code>RDS</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>eventTypeCodes</i> A list of unique identifiers for event types. For example, <code>"AWS_EC2_SYSTEM_MAINTENANCE_EVENT","AWS_RDS_MAINTENANCE_SCHEDULED"</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>eventArns</i> A list of event ARNs (unique identifiers). For example: <code>"arn:aws:health:us-east-1::event/EC2/EC2_INSTANCE_RETIREMENT_SCHEDULED/EC2_INSTANCE_RETIREMENT_SCHEDULED_ABC123-CDE456", "arn:aws:health:us-west-1::event/EBS/AWS_EBS_LOST_VOLUME/AWS_EBS_LOST_VOLUME_CHI789_JKL101"</code> <br/>
        :return: nextToken:  If the results of a search are large, only a portion of the results are returned, and a <code>nextToken</code> pagination token is returned in the response. To retrieve the next batch of results, reissue the search request and include the returned token. When all results have been returned, the response does not contain a pagination token value.<br/>events:  The events that match the specified filter criteria.<br/>        """
        pass



class IAM(BaseClient):
    """<fullname>AWS Identity and Access Management</fullname> AWS Identity and Access Management (IAM) is a web service that you can use to manage users and user permissions under your AWS account. This guide provides descriptions of IAM actions that you can call programmatically. For general information about IAM, see <a href="http://aws.amazon.com/iam/">AWS Identity and Access Management (IAM)</a>. For the user guide for IAM, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/">Using IAM</a>.  <note> AWS provides SDKs that consist of libraries and sample code for various programming languages and platforms (Java, Ruby, .NET, iOS, Android, etc.). The SDKs provide a convenient way to create programmatic access to IAM and AWS. For example, the SDKs take care of tasks such as cryptographically signing requests (see below), managing errors, and retrying requests automatically. For information about the AWS SDKs, including how to download and install them, see the <a href="http://aws.amazon.com/tools/">Tools for Amazon Web Services</a> page.  </note> We recommend that you use the AWS SDKs to make programmatic API calls to IAM. However, you can also use the IAM Query API to make direct calls to the IAM web service. To learn more about the IAM Query API, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/IAM_UsingQueryAPI.html">Making Query Requests</a> in the <i>Using IAM</i> guide. IAM supports GET and POST requests for all actions. That is, the API does not require you to use GET for some actions and POST for others. However, GET requests are subject to the limitation size of a URL. Therefore, for operations that require larger sizes, use a POST request.   <b>Signing Requests</b>  Requests must be signed using an access key ID and a secret access key. We strongly recommend that you do not use your AWS account access key ID and secret access key for everyday work with IAM. You can use the access key ID and secret access key for an IAM user or you can use the AWS Security Token Service to generate temporary security credentials and use those to sign requests. To sign requests, we recommend that you use <a href="https://docs.aws.amazon.com/general/latest/gr/signature-version-4.html">Signature Version 4</a>. If you have an existing application that uses Signature Version 2, you do not have to update it to use Signature Version 4. However, some operations now require Signature Version 4. The documentation for operations that require version 4 indicate this requirement.   <b>Additional Resources</b>  For more information, see the following: <ul> <li>  <a href="https://docs.aws.amazon.com/general/latest/gr/aws-security-credentials.html">AWS Security Credentials</a>. This topic provides general information about the types of credentials used for accessing AWS.  </li> <li>  <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/IAMBestPractices.html">IAM Best Practices</a>. This topic presents a list of suggestions for using the IAM service to help secure your AWS resources.  </li> <li>  <a href="https://docs.aws.amazon.com/general/latest/gr/signing_aws_api_requests.html">Signing AWS API Requests</a>. This set of topics walk you through the process of signing a request using an access key ID and secret access key.  </li> </ul>"""

    class InstanceProfileExistsWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `get_instance_profile` for valid parameters"""
            pass
    class PolicyExistsWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `get_policy` for valid parameters"""
            pass
    class RoleExistsWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `get_role` for valid parameters"""
            pass
    class UserExistsWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `get_user` for valid parameters"""
            pass


    class GetAccountAuthorizationDetailsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_account_authorization_details` for valid parameters"""
            pass
    class GetGroupPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_group` for valid parameters"""
            pass
    class ListAccessKeysPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_access_keys` for valid parameters"""
            pass
    class ListAccountAliasesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_account_aliases` for valid parameters"""
            pass
    class ListAttachedGroupPoliciesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_attached_group_policies` for valid parameters"""
            pass
    class ListAttachedRolePoliciesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_attached_role_policies` for valid parameters"""
            pass
    class ListAttachedUserPoliciesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_attached_user_policies` for valid parameters"""
            pass
    class ListEntitiesForPolicyPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_entities_for_policy` for valid parameters"""
            pass
    class ListGroupPoliciesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_group_policies` for valid parameters"""
            pass
    class ListGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_groups` for valid parameters"""
            pass
    class ListGroupsForUserPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_groups_for_user` for valid parameters"""
            pass
    class ListInstanceProfilesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_instance_profiles` for valid parameters"""
            pass
    class ListInstanceProfilesForRolePaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_instance_profiles_for_role` for valid parameters"""
            pass
    class ListMFADevicesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_mfa_devices` for valid parameters"""
            pass
    class ListPoliciesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_policies` for valid parameters"""
            pass
    class ListPolicyVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_policy_versions` for valid parameters"""
            pass
    class ListRolePoliciesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_role_policies` for valid parameters"""
            pass
    class ListRolesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_roles` for valid parameters"""
            pass
    class ListServerCertificatesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_server_certificates` for valid parameters"""
            pass
    class ListSigningCertificatesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_signing_certificates` for valid parameters"""
            pass
    class ListSSHPublicKeysPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_ssh_public_keys` for valid parameters"""
            pass
    class ListUserPoliciesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_user_policies` for valid parameters"""
            pass
    class ListUsersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_users` for valid parameters"""
            pass
    class ListVirtualMFADevicesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_virtual_mfa_devices` for valid parameters"""
            pass
    class SimulateCustomPolicyPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `simulate_custom_policy` for valid parameters"""
            pass
    class SimulatePrincipalPolicyPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `simulate_principal_policy` for valid parameters"""
            pass


    class AddClientIDToOpenIDConnectProviderRequest(Mapping):
        pass
    
    class AddRoleToInstanceProfileRequest(Mapping):
        pass
    
    class AddUserToGroupRequest(Mapping):
        pass
    
    class AttachGroupPolicyRequest(Mapping):
        pass
    
    class AttachRolePolicyRequest(Mapping):
        pass
    
    class AttachUserPolicyRequest(Mapping):
        pass
    
    class ChangePasswordRequest(Mapping):
        pass
    
    class CreateAccessKeyRequest(Mapping):
        pass
    
    class CreateAccessKeyResponse(Mapping):
        pass
    
    class CreateAccountAliasRequest(Mapping):
        pass
    
    class CreateGroupRequest(Mapping):
        pass
    
    class CreateGroupResponse(Mapping):
        pass
    
    class CreateInstanceProfileRequest(Mapping):
        pass
    
    class CreateInstanceProfileResponse(Mapping):
        pass
    
    class CreateLoginProfileRequest(Mapping):
        pass
    
    class CreateLoginProfileResponse(Mapping):
        pass
    
    class booleanType(object):
        pass
    
    class CreateOpenIDConnectProviderRequest(Mapping):
        pass
    
    class CreateOpenIDConnectProviderResponse(Mapping):
        pass
    
    class clientIDType(object):
        pass
    
    class thumbprintType(object):
        pass
    
    class CreatePolicyRequest(Mapping):
        pass
    
    class CreatePolicyResponse(Mapping):
        pass
    
    class CreatePolicyVersionRequest(Mapping):
        pass
    
    class CreatePolicyVersionResponse(Mapping):
        pass
    
    class CreateRoleRequest(Mapping):
        pass
    
    class CreateRoleResponse(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreateSAMLProviderRequest(Mapping):
        pass
    
    class CreateSAMLProviderResponse(Mapping):
        pass
    
    class CreateServiceLinkedRoleRequest(Mapping):
        pass
    
    class CreateServiceLinkedRoleResponse(Mapping):
        pass
    
    class CreateServiceSpecificCredentialRequest(Mapping):
        pass
    
    class CreateServiceSpecificCredentialResponse(Mapping):
        pass
    
    class CreateUserRequest(Mapping):
        pass
    
    class CreateUserResponse(Mapping):
        pass
    
    class CreateVirtualMFADeviceRequest(Mapping):
        pass
    
    class CreateVirtualMFADeviceResponse(Mapping):
        pass
    
    class DeactivateMFADeviceRequest(Mapping):
        pass
    
    class DeleteAccessKeyRequest(Mapping):
        pass
    
    class DeleteAccountAliasRequest(Mapping):
        pass
    
    class DeleteGroupRequest(Mapping):
        pass
    
    class DeleteGroupPolicyRequest(Mapping):
        pass
    
    class DeleteInstanceProfileRequest(Mapping):
        pass
    
    class DeleteLoginProfileRequest(Mapping):
        pass
    
    class DeleteOpenIDConnectProviderRequest(Mapping):
        pass
    
    class DeletePolicyRequest(Mapping):
        pass
    
    class DeletePolicyVersionRequest(Mapping):
        pass
    
    class DeleteRoleRequest(Mapping):
        pass
    
    class DeleteRolePermissionsBoundaryRequest(Mapping):
        pass
    
    class DeleteRolePolicyRequest(Mapping):
        pass
    
    class DeleteSAMLProviderRequest(Mapping):
        pass
    
    class DeleteSSHPublicKeyRequest(Mapping):
        pass
    
    class DeleteServerCertificateRequest(Mapping):
        pass
    
    class DeleteServiceLinkedRoleRequest(Mapping):
        pass
    
    class DeleteServiceLinkedRoleResponse(Mapping):
        pass
    
    class DeleteServiceSpecificCredentialRequest(Mapping):
        pass
    
    class DeleteSigningCertificateRequest(Mapping):
        pass
    
    class DeleteUserRequest(Mapping):
        pass
    
    class DeleteUserPermissionsBoundaryRequest(Mapping):
        pass
    
    class DeleteUserPolicyRequest(Mapping):
        pass
    
    class DeleteVirtualMFADeviceRequest(Mapping):
        pass
    
    class DetachGroupPolicyRequest(Mapping):
        pass
    
    class DetachRolePolicyRequest(Mapping):
        pass
    
    class DetachUserPolicyRequest(Mapping):
        pass
    
    class EnableMFADeviceRequest(Mapping):
        pass
    
    class GenerateCredentialReportResponse(Mapping):
        pass
    
    class GenerateOrganizationsAccessReportRequest(Mapping):
        pass
    
    class GenerateOrganizationsAccessReportResponse(Mapping):
        pass
    
    class GenerateServiceLastAccessedDetailsRequest(Mapping):
        pass
    
    class GenerateServiceLastAccessedDetailsResponse(Mapping):
        pass
    
    class GetAccessKeyLastUsedRequest(Mapping):
        pass
    
    class GetAccessKeyLastUsedResponse(Mapping):
        pass
    
    class GetAccountAuthorizationDetailsRequest(Mapping):
        pass
    
    class GetAccountAuthorizationDetailsResponse(Mapping):
        pass
    
    class EntityType(object):
        pass
    
    class GetAccountPasswordPolicyResponse(Mapping):
        pass
    
    class GetAccountSummaryResponse(Mapping):
        pass
    
    class GetContextKeysForCustomPolicyRequest(Mapping):
        pass
    
    class GetContextKeysForPolicyResponse(Mapping):
        pass
    
    class policyDocumentType(object):
        pass
    
    class GetContextKeysForPrincipalPolicyRequest(Mapping):
        pass
    
    class GetCredentialReportResponse(Mapping):
        pass
    
    class GetGroupRequest(Mapping):
        pass
    
    class GetGroupResponse(Mapping):
        pass
    
    class GetGroupPolicyRequest(Mapping):
        pass
    
    class GetGroupPolicyResponse(Mapping):
        pass
    
    class GetInstanceProfileRequest(Mapping):
        pass
    
    class GetInstanceProfileResponse(Mapping):
        pass
    
    class GetLoginProfileRequest(Mapping):
        pass
    
    class GetLoginProfileResponse(Mapping):
        pass
    
    class GetOpenIDConnectProviderRequest(Mapping):
        pass
    
    class GetOpenIDConnectProviderResponse(Mapping):
        pass
    
    class GetOrganizationsAccessReportRequest(Mapping):
        pass
    
    class GetOrganizationsAccessReportResponse(Mapping):
        pass
    
    class GetPolicyRequest(Mapping):
        pass
    
    class GetPolicyResponse(Mapping):
        pass
    
    class GetPolicyVersionRequest(Mapping):
        pass
    
    class GetPolicyVersionResponse(Mapping):
        pass
    
    class GetRoleRequest(Mapping):
        pass
    
    class GetRoleResponse(Mapping):
        pass
    
    class GetRolePolicyRequest(Mapping):
        pass
    
    class GetRolePolicyResponse(Mapping):
        pass
    
    class GetSAMLProviderRequest(Mapping):
        pass
    
    class GetSAMLProviderResponse(Mapping):
        pass
    
    class GetSSHPublicKeyRequest(Mapping):
        pass
    
    class GetSSHPublicKeyResponse(Mapping):
        pass
    
    class GetServerCertificateRequest(Mapping):
        pass
    
    class GetServerCertificateResponse(Mapping):
        pass
    
    class GetServiceLastAccessedDetailsRequest(Mapping):
        pass
    
    class GetServiceLastAccessedDetailsResponse(Mapping):
        pass
    
    class GetServiceLastAccessedDetailsWithEntitiesRequest(Mapping):
        pass
    
    class GetServiceLastAccessedDetailsWithEntitiesResponse(Mapping):
        pass
    
    class GetServiceLinkedRoleDeletionStatusRequest(Mapping):
        pass
    
    class GetServiceLinkedRoleDeletionStatusResponse(Mapping):
        pass
    
    class GetUserRequest(Mapping):
        pass
    
    class GetUserResponse(Mapping):
        pass
    
    class GetUserPolicyRequest(Mapping):
        pass
    
    class GetUserPolicyResponse(Mapping):
        pass
    
    class ListAccessKeysRequest(Mapping):
        pass
    
    class ListAccessKeysResponse(Mapping):
        pass
    
    class ListAccountAliasesRequest(Mapping):
        pass
    
    class ListAccountAliasesResponse(Mapping):
        pass
    
    class ListAttachedGroupPoliciesRequest(Mapping):
        pass
    
    class ListAttachedGroupPoliciesResponse(Mapping):
        pass
    
    class ListAttachedRolePoliciesRequest(Mapping):
        pass
    
    class ListAttachedRolePoliciesResponse(Mapping):
        pass
    
    class ListAttachedUserPoliciesRequest(Mapping):
        pass
    
    class ListAttachedUserPoliciesResponse(Mapping):
        pass
    
    class ListEntitiesForPolicyRequest(Mapping):
        pass
    
    class ListEntitiesForPolicyResponse(Mapping):
        pass
    
    class ListGroupPoliciesRequest(Mapping):
        pass
    
    class ListGroupPoliciesResponse(Mapping):
        pass
    
    class ListGroupsRequest(Mapping):
        pass
    
    class ListGroupsResponse(Mapping):
        pass
    
    class ListGroupsForUserRequest(Mapping):
        pass
    
    class ListGroupsForUserResponse(Mapping):
        pass
    
    class ListInstanceProfilesRequest(Mapping):
        pass
    
    class ListInstanceProfilesResponse(Mapping):
        pass
    
    class ListInstanceProfilesForRoleRequest(Mapping):
        pass
    
    class ListInstanceProfilesForRoleResponse(Mapping):
        pass
    
    class ListMFADevicesRequest(Mapping):
        pass
    
    class ListMFADevicesResponse(Mapping):
        pass
    
    class ListOpenIDConnectProvidersRequest(Mapping):
        pass
    
    class ListOpenIDConnectProvidersResponse(Mapping):
        pass
    
    class ListPoliciesRequest(Mapping):
        pass
    
    class ListPoliciesResponse(Mapping):
        pass
    
    class ListPoliciesGrantingServiceAccessRequest(Mapping):
        pass
    
    class ListPoliciesGrantingServiceAccessResponse(Mapping):
        pass
    
    class serviceNamespaceType(object):
        pass
    
    class ListPolicyVersionsRequest(Mapping):
        pass
    
    class ListPolicyVersionsResponse(Mapping):
        pass
    
    class ListRolePoliciesRequest(Mapping):
        pass
    
    class ListRolePoliciesResponse(Mapping):
        pass
    
    class ListRoleTagsRequest(Mapping):
        pass
    
    class ListRoleTagsResponse(Mapping):
        pass
    
    class ListRolesRequest(Mapping):
        pass
    
    class ListRolesResponse(Mapping):
        pass
    
    class ListSAMLProvidersRequest(Mapping):
        pass
    
    class ListSAMLProvidersResponse(Mapping):
        pass
    
    class ListSSHPublicKeysRequest(Mapping):
        pass
    
    class ListSSHPublicKeysResponse(Mapping):
        pass
    
    class ListServerCertificatesRequest(Mapping):
        pass
    
    class ListServerCertificatesResponse(Mapping):
        pass
    
    class ListServiceSpecificCredentialsRequest(Mapping):
        pass
    
    class ListServiceSpecificCredentialsResponse(Mapping):
        pass
    
    class ListSigningCertificatesRequest(Mapping):
        pass
    
    class ListSigningCertificatesResponse(Mapping):
        pass
    
    class ListUserPoliciesRequest(Mapping):
        pass
    
    class ListUserPoliciesResponse(Mapping):
        pass
    
    class ListUserTagsRequest(Mapping):
        pass
    
    class ListUserTagsResponse(Mapping):
        pass
    
    class ListUsersRequest(Mapping):
        pass
    
    class ListUsersResponse(Mapping):
        pass
    
    class ListVirtualMFADevicesRequest(Mapping):
        pass
    
    class ListVirtualMFADevicesResponse(Mapping):
        pass
    
    class PutGroupPolicyRequest(Mapping):
        pass
    
    class PutRolePermissionsBoundaryRequest(Mapping):
        pass
    
    class PutRolePolicyRequest(Mapping):
        pass
    
    class PutUserPermissionsBoundaryRequest(Mapping):
        pass
    
    class PutUserPolicyRequest(Mapping):
        pass
    
    class RemoveClientIDFromOpenIDConnectProviderRequest(Mapping):
        pass
    
    class RemoveRoleFromInstanceProfileRequest(Mapping):
        pass
    
    class RemoveUserFromGroupRequest(Mapping):
        pass
    
    class ResetServiceSpecificCredentialRequest(Mapping):
        pass
    
    class ResetServiceSpecificCredentialResponse(Mapping):
        pass
    
    class ResyncMFADeviceRequest(Mapping):
        pass
    
    class SetDefaultPolicyVersionRequest(Mapping):
        pass
    
    class SetSecurityTokenServicePreferencesRequest(Mapping):
        pass
    
    class SimulateCustomPolicyRequest(Mapping):
        pass
    
    class SimulatePolicyResponse(Mapping):
        pass
    
    class ActionNameType(object):
        pass
    
    class ResourceNameType(object):
        pass
    
    class ContextEntry(Mapping):
        pass
    
    class SimulatePrincipalPolicyRequest(Mapping):
        pass
    
    class TagRoleRequest(Mapping):
        pass
    
    class TagUserRequest(Mapping):
        pass
    
    class UntagRoleRequest(Mapping):
        pass
    
    class tagKeyType(object):
        pass
    
    class UntagUserRequest(Mapping):
        pass
    
    class UpdateAccessKeyRequest(Mapping):
        pass
    
    class UpdateAccountPasswordPolicyRequest(Mapping):
        pass
    
    class booleanObjectType(object):
        pass
    
    class UpdateAssumeRolePolicyRequest(Mapping):
        pass
    
    class UpdateGroupRequest(Mapping):
        pass
    
    class UpdateLoginProfileRequest(Mapping):
        pass
    
    class UpdateOpenIDConnectProviderThumbprintRequest(Mapping):
        pass
    
    class UpdateRoleRequest(Mapping):
        pass
    
    class UpdateRoleResponse(Mapping):
        pass
    
    class UpdateRoleDescriptionRequest(Mapping):
        pass
    
    class UpdateRoleDescriptionResponse(Mapping):
        pass
    
    class UpdateSAMLProviderRequest(Mapping):
        pass
    
    class UpdateSAMLProviderResponse(Mapping):
        pass
    
    class UpdateSSHPublicKeyRequest(Mapping):
        pass
    
    class UpdateServerCertificateRequest(Mapping):
        pass
    
    class UpdateServiceSpecificCredentialRequest(Mapping):
        pass
    
    class UpdateSigningCertificateRequest(Mapping):
        pass
    
    class UpdateUserRequest(Mapping):
        pass
    
    class UploadSSHPublicKeyRequest(Mapping):
        pass
    
    class UploadSSHPublicKeyResponse(Mapping):
        pass
    
    class UploadServerCertificateRequest(Mapping):
        pass
    
    class UploadServerCertificateResponse(Mapping):
        pass
    
    class UploadSigningCertificateRequest(Mapping):
        pass
    
    class UploadSigningCertificateResponse(Mapping):
        pass
    
    def add_client_id_to_open_id_connect_provider(self, ClientID: str, OpenIDConnectProviderArn: str):
        """Adds a new client ID (also known as audience) to the list of client IDs already registered for the specified IAM OpenID Connect (OIDC) provider resource. This operation is idempotent; it does not fail or return an error if you add an existing client ID to the provider.

        :param self:
        :param ClientID:  The client ID (also known as audience) to add to the IAM OpenID Connect provider resource.<br/>OpenIDConnectProviderArn:  The Amazon Resource Name (ARN) of the IAM OpenID Connect (OIDC) provider resource to add the client ID to. You can get a list of OIDC provider ARNs by using the <a>ListOpenIDConnectProviders</a> operation.<br/>
        :return:         """
        pass

    def add_role_to_instance_profile(self, RoleName: str, InstanceProfileName: str):
        """Adds the specified IAM role to the specified instance profile. An instance profile can contain only one role, and this limit cannot be increased. You can remove the existing role and then add a different role to an instance profile. You must then wait for the change to appear across all of AWS because of <a href="https://en.wikipedia.org/wiki/Eventual_consistency">eventual consistency</a>. To force the change, you must <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DisassociateIamInstanceProfile.html">disassociate the instance profile</a> and then <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_AssociateIamInstanceProfile.html">associate the instance profile</a>, or you can stop your instance and then restart it. <note> The caller of this API must be granted the <code>PassRole</code> permission on the IAM role by a permissions policy. </note> For more information about roles, go to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/WorkingWithRoles.html">Working with Roles</a>. For more information about instance profiles, go to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/AboutInstanceProfiles.html">About Instance Profiles</a>.

        :param self:
        :param RoleName:  The name of the role to add. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>InstanceProfileName:  The name of the instance profile to update. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def add_user_to_group(self, UserName: str, GroupName: str):
        """Adds the specified user to the specified group.

        :param self:
        :param UserName:  The name of the user to add. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>GroupName:  The name of the group to update. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def attach_group_policy(self, PolicyArn: str, GroupName: str):
        """Attaches the specified managed policy to the specified IAM group. You use this API to attach a managed policy to a group. To embed an inline policy in a group, use <a>PutGroupPolicy</a>. For more information about policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-vs-inline.html">Managed Policies and Inline Policies</a> in the <i>IAM User Guide</i>.

        :param self:
        :param PolicyArn:  The Amazon Resource Name (ARN) of the IAM policy you want to attach. For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the <i>AWS General Reference</i>.<br/>GroupName:  The name (friendly name, not ARN) of the group to attach the policy to. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def attach_role_policy(self, PolicyArn: str, RoleName: str):
        """Attaches the specified managed policy to the specified IAM role. When you attach a managed policy to a role, the managed policy becomes part of the role's permission (access) policy. <note> You cannot use a managed policy as the role's trust policy. The role's trust policy is created at the same time as the role, using <a>CreateRole</a>. You can update a role's trust policy using <a>UpdateAssumeRolePolicy</a>. </note> Use this API to attach a <i>managed</i> policy to a role. To embed an inline policy in a role, use <a>PutRolePolicy</a>. For more information about policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-vs-inline.html">Managed Policies and Inline Policies</a> in the <i>IAM User Guide</i>.

        :param self:
        :param PolicyArn:  The Amazon Resource Name (ARN) of the IAM policy you want to attach. For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the <i>AWS General Reference</i>.<br/>RoleName:  The name (friendly name, not ARN) of the role to attach the policy to. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def attach_user_policy(self, PolicyArn: str, UserName: str):
        """Attaches the specified managed policy to the specified user. You use this API to attach a <i>managed</i> policy to a user. To embed an inline policy in a user, use <a>PutUserPolicy</a>. For more information about policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-vs-inline.html">Managed Policies and Inline Policies</a> in the <i>IAM User Guide</i>.

        :param self:
        :param PolicyArn:  The Amazon Resource Name (ARN) of the IAM policy you want to attach. For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the <i>AWS General Reference</i>.<br/>UserName:  The name (friendly name, not ARN) of the IAM user to attach the policy to. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def change_password(self, NewPassword: str, OldPassword: str):
        """Changes the password of the IAM user who is calling this operation. The AWS account root user password is not affected by this operation. To change the password for a different user, see <a>UpdateLoginProfile</a>. For more information about modifying passwords, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_ManagingLogins.html">Managing Passwords</a> in the <i>IAM User Guide</i>.

        :param self:
        :param NewPassword:  The new password. The new password must conform to the AWS account's password policy, if one exists. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> that is used to validate this parameter is a string of characters. That string can include almost any printable ASCII character from the space (\u0020) through the end of the ASCII character range (\u00FF). You can also include the tab (\u0009), line feed (\u000A), and carriage return (\u000D) characters. Any of these characters are valid in a password. However, many tools, such as the AWS Management Console, might restrict the ability to type certain characters because they have special meaning within that tool.<br/>OldPassword:  The IAM user's current password.<br/>
        :return:         """
        pass

    def create_access_key(self, UserName: str=None) -> CreateAccessKeyResponse:
        """ Creates a new AWS secret access key and corresponding AWS access key ID for the specified user. The default status for new keys is <code>Active</code>. If you do not specify a user name, IAM determines the user name implicitly based on the AWS access key ID signing the request. This operation works for access keys under the AWS account. Consequently, you can use this operation to manage AWS account root user credentials. This is true even if the AWS account has no associated users.  For information about limits on the number of keys you can create, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/LimitationsOnEntities.html">Limitations on IAM Entities</a> in the <i>IAM User Guide</i>. <important> To ensure the security of your AWS account, the secret access key is accessible only during key and user creation. You must save the key (for example, in a text file) if you want to be able to access it again. If a secret key is lost, you can delete the access keys for the associated user and then create new keys. </important>

        :param self:
        :param UserName:  The name of the IAM user that the new key will belong to. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: AccessKey:  A structure with details about the access key.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateDate</i> The date when the access key was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecretAccessKey</i> The secret key used to sign requests.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the access key. <code>Active</code> means that the key is valid for API calls, while <code>Inactive</code> means it is not. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccessKeyId</i> The ID for this access key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserName</i> The name of the IAM user that the access key is associated with.<br/>        """
        pass

    def create_account_alias(self, AccountAlias: str):
        """Creates an alias for your AWS account. For information about using an AWS account alias, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/AccountAlias.html">Using an Alias for Your AWS Account ID</a> in the <i>IAM User Guide</i>.

        :param self:
        :param AccountAlias:  The account alias to create. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of lowercase letters, digits, and dashes. You cannot start or finish with a dash, nor can you have two dashes in a row.<br/>
        :return:         """
        pass

    def create_group(self, GroupName: str, Path: str=None) -> CreateGroupResponse:
        """Creates a new group.  For information about the number of groups you can create, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/LimitationsOnEntities.html">Limitations on IAM Entities</a> in the <i>IAM User Guide</i>.

        :param self:
        :param GroupName:  The name of the group to create. Do not include the path in this value. IAM user, group, role, and policy names must be unique within the account. Names are not distinguished by case. For example, you cannot create resources named both "MyResource" and "myresource".<br/>Path:   The path to the group. For more information about paths, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>IAM User Guide</i>. This parameter is optional. If it is not included, it defaults to a slash (/). This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of either a forward slash (/) by itself or a string that must begin and end with forward slashes. In addition, it can contain any ASCII character from the ! (\u0021) through the DEL character (\u007F), including most punctuation characters, digits, and upper and lowercased letters.<br/>
        :return: Group:  A structure containing details about the new group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateDate</i> The date and time, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>, when the group was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i>  The Amazon Resource Name (ARN) specifying the group. For more information about ARNs and how to use them in policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupId</i>  The stable and unique string identifying the group. For more information about IDs, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupName</i> The friendly name that identifies the group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Path</i> The path to the group. For more information about paths, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide. <br/>        """
        pass

    def create_instance_profile(self, InstanceProfileName: str, Path: str=None) -> CreateInstanceProfileResponse:
        """ Creates a new instance profile. For information about instance profiles, go to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/AboutInstanceProfiles.html">About Instance Profiles</a>.  For information about the number of instance profiles you can create, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/LimitationsOnEntities.html">Limitations on IAM Entities</a> in the <i>IAM User Guide</i>.

        :param self:
        :param Path:   The path to the instance profile. For more information about paths, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>IAM User Guide</i>. This parameter is optional. If it is not included, it defaults to a slash (/). This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of either a forward slash (/) by itself or a string that must begin and end with forward slashes. In addition, it can contain any ASCII character from the ! (\u0021) through the DEL character (\u007F), including most punctuation characters, digits, and upper and lowercased letters.<br/>InstanceProfileName:  The name of the instance profile to create. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: InstanceProfile:  A structure containing details about the new instance profile.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Roles</i> The role associated with the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateDate</i> The date when the instance profile was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i>  The Amazon Resource Name (ARN) specifying the instance profile. For more information about ARNs and how to use them in policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceProfileId</i>  The stable and unique string identifying the instance profile. For more information about IDs, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceProfileName</i> The name identifying the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Path</i>  The path to the instance profile. For more information about paths, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide. <br/>        """
        pass

    def create_login_profile(self, Password: str, UserName: str, PasswordResetRequired: booleanType=None) -> CreateLoginProfileResponse:
        """ Creates a password for the specified user, giving the user the ability to access AWS services through the AWS Management Console. For more information about managing passwords, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_ManagingLogins.html">Managing Passwords</a> in the <i>IAM User Guide</i>.

        :param self:
        :param PasswordResetRequired:  Specifies whether the user is required to set a new password on next sign-in.<br/>Password:  The new password for the user. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> that is used to validate this parameter is a string of characters. That string can include almost any printable ASCII character from the space (\u0020) through the end of the ASCII character range (\u00FF). You can also include the tab (\u0009), line feed (\u000A), and carriage return (\u000D) characters. Any of these characters are valid in a password. However, many tools, such as the AWS Management Console, might restrict the ability to type certain characters because they have special meaning within that tool.<br/>UserName:  The name of the IAM user to create a password for. The user must already exist. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: LoginProfile:  A structure containing the user name and password create date.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PasswordResetRequired</i> Specifies whether the user is required to set a new password on next sign-in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateDate</i> The date when the password for the user was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserName</i> The name of the user, which can be used for signing in to the AWS Management Console.<br/>        """
        pass

    def create_open_id_connect_provider(self, ThumbprintList: List[thumbprintType], Url: str, ClientIDList: List[clientIDType]=None) -> CreateOpenIDConnectProviderResponse:
        """Creates an IAM entity to describe an identity provider (IdP) that supports <a href="http://openid.net/connect/">OpenID Connect (OIDC)</a>. The OIDC provider that you create with this operation can be used as a principal in a role's trust policy. Such a policy establishes a trust relationship between AWS and the OIDC provider. When you create the IAM OIDC provider, you specify the following: <ul> <li> The URL of the OIDC identity provider (IdP) to trust </li> <li> A list of client IDs (also known as audiences) that identify the application or applications that are allowed to authenticate using the OIDC provider </li> <li> A list of thumbprints of the server certificate(s) that the IdP uses </li> </ul> You get all of this information from the OIDC IdP that you want to use to access AWS. <note> The trust for the OIDC provider is derived from the IAM provider that this operation creates. Therefore, it is best to limit access to the <a>CreateOpenIDConnectProvider</a> operation to highly privileged users. </note>

        :param self:
        :param ThumbprintList:  A list of server certificate thumbprints for the OpenID Connect (OIDC) identity provider's server certificates. Typically this list includes only one entry. However, IAM lets you have up to five thumbprints for an OIDC provider. This lets you maintain multiple thumbprints if the identity provider is rotating certificates. The server certificate thumbprint is the hex-encoded SHA-1 hash value of the X.509 certificate used by the domain where the OpenID Connect provider makes its keys available. It is always a 40-character string. You must provide at least one thumbprint when creating an IAM OIDC provider. For example, assume that the OIDC provider is <code>server.example.com</code> and the provider stores its keys at https://keys.server.example.com/openid-connect. In that case, the thumbprint string would be the hex-encoded SHA-1 hash value of the certificate used by https://keys.server.example.com. For more information about obtaining the OIDC provider's thumbprint, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/identity-providers-oidc-obtain-thumbprint.html">Obtaining the Thumbprint for an OpenID Connect Provider</a> in the <i>IAM User Guide</i>.<br/>ClientIDList:  A list of client IDs (also known as audiences). When a mobile or web app registers with an OpenID Connect provider, they establish a value that identifies the application. (This is the value that's sent as the <code>client_id</code> parameter on OAuth requests.) You can register multiple client IDs with the same provider. For example, you might have multiple applications that use the same OIDC provider. You cannot register more than 100 client IDs with a single IAM OIDC provider. There is no defined format for a client ID. The <code>CreateOpenIDConnectProviderRequest</code> operation accepts client IDs up to 255 characters long.<br/>Url:  The URL of the identity provider. The URL must begin with <code>https://</code> and should correspond to the <code>iss</code> claim in the provider's OpenID Connect ID tokens. Per the OIDC standard, path components are allowed but query parameters are not. Typically the URL consists of only a hostname, like <code>https://server.example.org</code> or <code>https://example.com</code>. You cannot register the same provider multiple times in a single AWS account. If you try to submit a URL that has already been used for an OpenID Connect provider in the AWS account, you will get an error.<br/>
        :return: OpenIDConnectProviderArn:  The Amazon Resource Name (ARN) of the new IAM OpenID Connect provider that is created. For more information, see <a>OpenIDConnectProviderListEntry</a>. <br/>        """
        pass

    def create_policy(self, PolicyDocument: str, PolicyName: str, Path: str=None, Description: str=None) -> CreatePolicyResponse:
        """Creates a new managed policy for your AWS account. This operation creates a policy version with a version identifier of <code>v1</code> and sets v1 as the policy's default version. For more information about policy versions, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-versions.html">Versioning for Managed Policies</a> in the <i>IAM User Guide</i>. For more information about managed policies in general, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-vs-inline.html">Managed Policies and Inline Policies</a> in the <i>IAM User Guide</i>.

        :param self:
        :param Description:  A friendly description of the policy. Typically used to store information about the permissions defined in the policy. For example, "Grants access to production DynamoDB tables." The policy description is immutable. After a value is assigned, it cannot be changed.<br/>PolicyDocument:  The JSON policy document that you want to use as the content for the new policy. You must provide policies in JSON format in IAM. However, for AWS CloudFormation templates formatted in YAML, you can provide the policy in JSON or YAML format. AWS CloudFormation always converts a YAML policy to JSON format before submitting it to IAM. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> used to validate this parameter is a string of characters consisting of the following: <ul> <li> Any printable ASCII character ranging from the space character (\u0020) through the end of the ASCII character range </li> <li> The printable characters in the Basic Latin and Latin-1 Supplement character set (through \u00FF) </li> <li> The special characters tab (\u0009), line feed (\u000A), and carriage return (\u000D) </li> </ul><br/>Path:  The path for the policy. For more information about paths, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>IAM User Guide</i>. This parameter is optional. If it is not included, it defaults to a slash (/). This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of either a forward slash (/) by itself or a string that must begin and end with forward slashes. In addition, it can contain any ASCII character from the ! (\u0021) through the DEL character (\u007F), including most punctuation characters, digits, and upper and lowercased letters.<br/>PolicyName:  The friendly name of the policy. IAM user, group, role, and policy names must be unique within the account. Names are not distinguished by case. For example, you cannot create resources named both "MyResource" and "myresource".<br/>
        :return: Policy:  A structure containing details about the new policy.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateDate</i> The date and time, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>, when the policy was last updated. When a policy has only one version, this field contains the date and time when the policy was created. When a policy has more than one version, this field contains the date and time when the most recent policy version was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateDate</i> The date and time, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>, when the policy was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A friendly description of the policy. This element is included in the response to the <a>GetPolicy</a> operation. It is not included in the response to the <a>ListPolicies</a> operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsAttachable</i> Specifies whether the policy can be attached to an IAM user, group, or role.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PermissionsBoundaryUsageCount</i> The number of entities (users and roles) for which the policy is used to set the permissions boundary.  For more information about permissions boundaries, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies_boundaries.html">Permissions Boundaries for IAM Identities </a> in the <i>IAM User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttachmentCount</i> The number of entities (users, groups, and roles) that the policy is attached to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultVersionId</i> The identifier for the version of the policy that is set as the default version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Path</i> The path to the policy. For more information about paths, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN). ARNs are unique identifiers for AWS resources. For more information about ARNs, go to <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the <i>AWS General Reference</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyId</i> The stable and unique string identifying the policy. For more information about IDs, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyName</i> The friendly name (not ARN) identifying the policy.<br/>        """
        pass

    def create_policy_version(self, PolicyDocument: str, PolicyArn: str, SetAsDefault: booleanType=None) -> CreatePolicyVersionResponse:
        """Creates a new version of the specified managed policy. To update a managed policy, you create a new policy version. A managed policy can have up to five versions. If the policy has five versions, you must delete an existing version using <a>DeletePolicyVersion</a> before you create a new version. Optionally, you can set the new version as the policy's default version. The default version is the version that is in effect for the IAM users, groups, and roles to which the policy is attached. For more information about managed policy versions, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-versions.html">Versioning for Managed Policies</a> in the <i>IAM User Guide</i>.

        :param self:
        :param SetAsDefault:  Specifies whether to set this version as the policy's default version. When this parameter is <code>true</code>, the new policy version becomes the operative version. That is, it becomes the version that is in effect for the IAM users, groups, and roles that the policy is attached to. For more information about managed policy versions, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-versions.html">Versioning for Managed Policies</a> in the <i>IAM User Guide</i>.<br/>PolicyDocument:  The JSON policy document that you want to use as the content for this new version of the policy. You must provide policies in JSON format in IAM. However, for AWS CloudFormation templates formatted in YAML, you can provide the policy in JSON or YAML format. AWS CloudFormation always converts a YAML policy to JSON format before submitting it to IAM. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> used to validate this parameter is a string of characters consisting of the following: <ul> <li> Any printable ASCII character ranging from the space character (\u0020) through the end of the ASCII character range </li> <li> The printable characters in the Basic Latin and Latin-1 Supplement character set (through \u00FF) </li> <li> The special characters tab (\u0009), line feed (\u000A), and carriage return (\u000D) </li> </ul><br/>PolicyArn:  The Amazon Resource Name (ARN) of the IAM policy to which you want to add a new version. For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the <i>AWS General Reference</i>.<br/>
        :return: PolicyVersion:  A structure containing details about the new policy version.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateDate</i> The date and time, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>, when the policy version was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsDefaultVersion</i> Specifies whether the policy version is set as the policy's default version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VersionId</i> The identifier for the policy version. Policy version identifiers always begin with <code>v</code> (always lowercase). When a policy is created, the first policy version is <code>v1</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Document</i> The policy document. The policy document is returned in the response to the <a>GetPolicyVersion</a> and <a>GetAccountAuthorizationDetails</a> operations. It is not returned in the response to the <a>CreatePolicyVersion</a> or <a>ListPolicyVersions</a> operations.  The policy document returned in this structure is URL-encoded compliant with <a href="https://tools.ietf.org/html/rfc3986">RFC 3986</a>. You can use a URL decoding method to convert the policy back to plain JSON text. For example, if you use Java, you can use the <code>decode</code> method of the <code>java.net.URLDecoder</code> utility class in the Java SDK. Other languages and SDKs provide similar functionality.<br/>        """
        pass

    def create_role(self, AssumeRolePolicyDocument: str, RoleName: str, Path: str=None, Description: str=None, MaxSessionDuration: int=None, PermissionsBoundary: str=None, Tags: List[Tag]=None) -> CreateRoleResponse:
        """Creates a new role for your AWS account. For more information about roles, go to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/WorkingWithRoles.html">IAM Roles</a>. For information about limitations on role names and the number of roles you can create, go to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/LimitationsOnEntities.html">Limitations on IAM Entities</a> in the <i>IAM User Guide</i>.

        :param self:
        :param Tags:  A list of tags that you want to attach to the newly created role. Each tag consists of a key name and an associated value. For more information about tagging, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_tags.html">Tagging IAM Identities</a> in the <i>IAM User Guide</i>. <note> If any one of the tags is invalid or if you exceed the allowed number of tags per role, then the entire request fails and the role is not created. </note><br/>PermissionsBoundary:  The ARN of the policy that is used to set the permissions boundary for the role.<br/>MaxSessionDuration:  The maximum session duration (in seconds) that you want to set for the specified role. If you do not specify a value for this setting, the default maximum of one hour is applied. This setting can have a value from 1 hour to 12 hours. Anyone who assumes the role from the AWS CLI or API can use the <code>DurationSeconds</code> API parameter or the <code>duration-seconds</code> CLI parameter to request a longer session. The <code>MaxSessionDuration</code> setting determines the maximum duration that can be requested using the <code>DurationSeconds</code> parameter. If users don't specify a value for the <code>DurationSeconds</code> parameter, their security credentials are valid for one hour by default. This applies when you use the <code>AssumeRole*</code> API operations or the <code>assume-role*</code> CLI operations but does not apply when you use those operations to create a console URL. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_use.html">Using IAM Roles</a> in the <i>IAM User Guide</i>.<br/>Description:  A description of the role.<br/>AssumeRolePolicyDocument:  The trust relationship policy document that grants an entity permission to assume the role. In IAM, you must provide a JSON policy that has been converted to a string. However, for AWS CloudFormation templates formatted in YAML, you can provide the policy in JSON or YAML format. AWS CloudFormation always converts a YAML policy to JSON format before submitting it to IAM. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> used to validate this parameter is a string of characters consisting of the following: <ul> <li> Any printable ASCII character ranging from the space character (\u0020) through the end of the ASCII character range </li> <li> The printable characters in the Basic Latin and Latin-1 Supplement character set (through \u00FF) </li> <li> The special characters tab (\u0009), line feed (\u000A), and carriage return (\u000D) </li> </ul>  Upon success, the response includes the same trust policy as a URL-encoded JSON string.<br/>RoleName:  The name of the role to create. IAM user, group, role, and policy names must be unique within the account. Names are not distinguished by case. For example, you cannot create resources named both "MyResource" and "myresource".<br/>Path:   The path to the role. For more information about paths, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>IAM User Guide</i>. This parameter is optional. If it is not included, it defaults to a slash (/). This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of either a forward slash (/) by itself or a string that must begin and end with forward slashes. In addition, it can contain any ASCII character from the ! (\u0021) through the DEL character (\u007F), including most punctuation characters, digits, and upper and lowercased letters.<br/>
        :return: Role:  A structure containing details about the new role.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> A list of tags that are attached to the specified role. For more information about tagging, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_tags.html">Tagging IAM Identities</a> in the <i>IAM User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PermissionsBoundary</i> The ARN of the policy used to set the permissions boundary for the role. For more information about permissions boundaries, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies_boundaries.html">Permissions Boundaries for IAM Identities </a> in the <i>IAM User Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PermissionsBoundary</b>: The ARN of the policy used to set the permissions boundary for the role. For more information about permissions boundaries, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies_boundaries.html">Permissions Boundaries for IAM Identities </a> in the <i>IAM User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxSessionDuration</i> The maximum session duration (in seconds) for the specified role. Anyone who uses the AWS CLI, or API to assume the role can specify the duration using the optional <code>DurationSeconds</code> API parameter or <code>duration-seconds</code> CLI parameter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of the role that you provide.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssumeRolePolicyDocument</i> The policy that grants an entity permission to assume the role.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateDate</i> The date and time, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>, when the role was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i>  The Amazon Resource Name (ARN) specifying the role. For more information about ARNs and how to use them in policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>IAM User Guide</i> guide. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleId</i>  The stable and unique string identifying the role. For more information about IDs, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleName</i> The friendly name that identifies the role.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Path</i>  The path to the role. For more information about paths, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PermissionsBoundaryArn</i>  The ARN of the policy used to set the permissions boundary for the user or role.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PermissionsBoundaryType</i>  The permissions boundary usage type that indicates what type of IAM resource is used as the permissions boundary for an entity. This data type can only have a value of <code>Policy</code>.<br/>        """
        pass

    def create_saml_provider(self, Name: str, SAMLMetadataDocument: str) -> CreateSAMLProviderResponse:
        """Creates an IAM resource that describes an identity provider (IdP) that supports SAML 2.0. The SAML provider resource that you create with this operation can be used as a principal in an IAM role's trust policy. Such a policy can enable federated users who sign in using the SAML IdP to assume the role. You can create an IAM role that supports Web-based single sign-on (SSO) to the AWS Management Console or one that supports API access to AWS. When you create the SAML provider resource, you upload a SAML metadata document that you get from your IdP. That document includes the issuer's name, expiration information, and keys that can be used to validate the SAML authentication response (assertions) that the IdP sends. You must generate the metadata document using the identity management software that is used as your organization's IdP. <note>  This operation requires <a href="https://docs.aws.amazon.com/general/latest/gr/signature-version-4.html">Signature Version 4</a>. </note>  For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_providers_enable-console-saml.html">Enabling SAML 2.0 Federated Users to Access the AWS Management Console</a> and <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_providers_saml.html">About SAML 2.0-based Federation</a> in the <i>IAM User Guide</i>.

        :param self:
        :param Name:  The name of the provider to create. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>SAMLMetadataDocument:  An XML document generated by an identity provider (IdP) that supports SAML 2.0. The document includes the issuer's name, expiration information, and keys that can be used to validate the SAML authentication response (assertions) that are received from the IdP. You must generate the metadata document using the identity management software that is used as your organization's IdP. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_providers_saml.html">About SAML 2.0-based Federation</a> in the <i>IAM User Guide</i> <br/>
        :return: SAMLProviderArn:  The Amazon Resource Name (ARN) of the new SAML provider resource in IAM.<br/>        """
        pass

    def create_service_linked_role(self, AWSServiceName: str, Description: str=None, CustomSuffix: str=None) -> CreateServiceLinkedRoleResponse:
        """Creates an IAM role that is linked to a specific AWS service. The service controls the attached policies and when the role can be deleted. This helps ensure that the service is not broken by an unexpectedly changed or deleted role, which could put your AWS resources into an unknown state. Allowing the service to control the role helps improve service stability and proper cleanup when a service and its role are no longer needed. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/using-service-linked-roles.html">Using Service-Linked Roles</a> in the <i>IAM User Guide</i>.  To attach a policy to this service-linked role, you must make the request using the AWS service that depends on this role.

        :param self:
        :param CustomSuffix:  <p/> A string that you provide, which is combined with the service-provided prefix to form the complete role name. If you make multiple requests for the same service, then you must supply a different <code>CustomSuffix</code> for each request. Otherwise the request fails with a duplicate role name error. For example, you could add <code>-1</code> or <code>-debug</code> to the suffix. Some services do not support the <code>CustomSuffix</code> parameter. If you provide an optional suffix and the operation fails, try the operation again without the suffix.<br/>Description:  The description of the role.<br/>AWSServiceName:  The service principal for the AWS service to which this role is attached. You use a string similar to a URL but without the http:// in front. For example: <code>elasticbeanstalk.amazonaws.com</code>.  Service principals are unique and case-sensitive. To find the exact service principal for your service-linked role, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_aws-services-that-work-with-iam.html">AWS Services That Work with IAM</a> in the <i>IAM User Guide</i>. Look for the services that have <b>Yes </b>in the <b>Service-Linked Role</b> column. Choose the <b>Yes</b> link to view the service-linked role documentation for that service.<br/>
        :return: Role:  A <a>Role</a> object that contains details about the newly created role.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> A list of tags that are attached to the specified role. For more information about tagging, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_tags.html">Tagging IAM Identities</a> in the <i>IAM User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PermissionsBoundary</i> The ARN of the policy used to set the permissions boundary for the role. For more information about permissions boundaries, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies_boundaries.html">Permissions Boundaries for IAM Identities </a> in the <i>IAM User Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PermissionsBoundary</b>: The ARN of the policy used to set the permissions boundary for the role. For more information about permissions boundaries, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies_boundaries.html">Permissions Boundaries for IAM Identities </a> in the <i>IAM User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxSessionDuration</i> The maximum session duration (in seconds) for the specified role. Anyone who uses the AWS CLI, or API to assume the role can specify the duration using the optional <code>DurationSeconds</code> API parameter or <code>duration-seconds</code> CLI parameter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of the role that you provide.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssumeRolePolicyDocument</i> The policy that grants an entity permission to assume the role.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateDate</i> The date and time, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>, when the role was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i>  The Amazon Resource Name (ARN) specifying the role. For more information about ARNs and how to use them in policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>IAM User Guide</i> guide. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleId</i>  The stable and unique string identifying the role. For more information about IDs, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleName</i> The friendly name that identifies the role.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Path</i>  The path to the role. For more information about paths, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PermissionsBoundaryArn</i>  The ARN of the policy used to set the permissions boundary for the user or role.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PermissionsBoundaryType</i>  The permissions boundary usage type that indicates what type of IAM resource is used as the permissions boundary for an entity. This data type can only have a value of <code>Policy</code>.<br/>        """
        pass

    def create_service_specific_credential(self, ServiceName: str, UserName: str) -> CreateServiceSpecificCredentialResponse:
        """Generates a set of credentials consisting of a user name and password that can be used to access the service specified in the request. These credentials are generated by IAM, and can be used only for the specified service.  You can have a maximum of two sets of service-specific credentials for each supported service per user. The only supported service at this time is AWS CodeCommit. You can reset the password to a new service-generated value by calling <a>ResetServiceSpecificCredential</a>. For more information about service-specific credentials, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_ssh-keys.html">Using IAM with AWS CodeCommit: Git Credentials, SSH Keys, and AWS Access Keys</a> in the <i>IAM User Guide</i>.

        :param self:
        :param ServiceName:  The name of the AWS service that is to be associated with the credentials. The service you specify here is the only service that can be accessed using these credentials.<br/>UserName:  The name of the IAM user that is to be associated with the credentials. The new service-specific credentials have the same permissions as the associated user except that they can be used only to access the specified service. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: ServiceSpecificCredential:  A structure that contains information about the newly created service-specific credential. <important> This is the only time that the password for this credential set is available. It cannot be recovered later. Instead, you must reset the password with <a>ResetServiceSpecificCredential</a>. </important>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the service-specific credential. <code>Active</code> means that the key is valid for API calls, while <code>Inactive</code> means it is not.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserName</i> The name of the IAM user associated with the service-specific credential.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceSpecificCredentialId</i> The unique identifier for the service-specific credential.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServicePassword</i> The generated password for the service-specific credential.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceUserName</i> The generated user name for the service-specific credential. This value is generated by combining the IAM user's name combined with the ID number of the AWS account, as in <code>jane-at-123456789012</code>, for example. This value cannot be configured by the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceName</i> The name of the service associated with the service-specific credential.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateDate</i> The date and time, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>, when the service-specific credential were created.<br/>        """
        pass

    def create_user(self, UserName: str, Path: str=None, PermissionsBoundary: str=None, Tags: List[Tag]=None) -> CreateUserResponse:
        """Creates a new IAM user for your AWS account.  For information about limitations on the number of IAM users you can create, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/LimitationsOnEntities.html">Limitations on IAM Entities</a> in the <i>IAM User Guide</i>.

        :param self:
        :param Tags:  A list of tags that you want to attach to the newly created user. Each tag consists of a key name and an associated value. For more information about tagging, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_tags.html">Tagging IAM Identities</a> in the <i>IAM User Guide</i>. <note> If any one of the tags is invalid or if you exceed the allowed number of tags per user, then the entire request fails and the user is not created. </note><br/>PermissionsBoundary:  The ARN of the policy that is used to set the permissions boundary for the user.<br/>UserName:  The name of the user to create. IAM user, group, role, and policy names must be unique within the account. Names are not distinguished by case. For example, you cannot create resources named both "MyResource" and "myresource".<br/>Path:   The path for the user name. For more information about paths, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>IAM User Guide</i>. This parameter is optional. If it is not included, it defaults to a slash (/). This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of either a forward slash (/) by itself or a string that must begin and end with forward slashes. In addition, it can contain any ASCII character from the ! (\u0021) through the DEL character (\u007F), including most punctuation characters, digits, and upper and lowercased letters.<br/>
        :return: User:  A structure with details about the new IAM user.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> A list of tags that are associated with the specified user. For more information about tagging, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_tags.html">Tagging IAM Identities</a> in the <i>IAM User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PermissionsBoundary</i> The ARN of the policy used to set the permissions boundary for the user. For more information about permissions boundaries, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies_boundaries.html">Permissions Boundaries for IAM Identities </a> in the <i>IAM User Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PermissionsBoundary</b>: The ARN of the policy used to set the permissions boundary for the user. For more information about permissions boundaries, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies_boundaries.html">Permissions Boundaries for IAM Identities </a> in the <i>IAM User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PasswordLastUsed</i> The date and time, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>, when the user's password was last used to sign in to an AWS website. For a list of AWS websites that capture a user's last sign-in time, see the <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/credential-reports.html">Credential Reports</a> topic in the <i>Using IAM</i> guide. If a password is used more than once in a five-minute span, only the first use is returned in this field. If the field is null (no value), then it indicates that they never signed in with a password. This can be because: <ul> <li> The user never had a password. </li> <li> A password exists but has not been used since IAM started tracking this information on October 20, 2014. </li> </ul> A null value does not mean that the user <i>never</i> had a password. Also, if the user does not currently have a password, but had one in the past, then this field contains the date and time the most recent password was used. This value is returned only in the <a>GetUser</a> and <a>ListUsers</a> operations. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateDate</i> The date and time, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>, when the user was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) that identifies the user. For more information about ARNs and how to use ARNs in policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserId</i> The stable and unique string identifying the user. For more information about IDs, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserName</i> The friendly name identifying the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Path</i> The path to the user. For more information about paths, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PermissionsBoundaryArn</i>  The ARN of the policy used to set the permissions boundary for the user or role.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PermissionsBoundaryType</i>  The permissions boundary usage type that indicates what type of IAM resource is used as the permissions boundary for an entity. This data type can only have a value of <code>Policy</code>.<br/>        """
        pass

    def create_virtual_mfa_device(self, VirtualMFADeviceName: str, Path: str=None) -> CreateVirtualMFADeviceResponse:
        """Creates a new virtual MFA device for the AWS account. After creating the virtual MFA, use <a>EnableMFADevice</a> to attach the MFA device to an IAM user. For more information about creating and working with virtual MFA devices, go to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_VirtualMFA.html">Using a Virtual MFA Device</a> in the <i>IAM User Guide</i>. For information about limits on the number of MFA devices you can create, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/LimitationsOnEntities.html">Limitations on Entities</a> in the <i>IAM User Guide</i>. <important> The seed information contained in the QR code and the Base32 string should be treated like any other secret access information. In other words, protect the seed information as you would your AWS access keys or your passwords. After you provision your virtual device, you should ensure that the information is destroyed following secure procedures. </important>

        :param self:
        :param VirtualMFADeviceName:  The name of the virtual MFA device. Use with path to uniquely identify a virtual MFA device. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>Path:   The path for the virtual MFA device. For more information about paths, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>IAM User Guide</i>. This parameter is optional. If it is not included, it defaults to a slash (/). This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of either a forward slash (/) by itself or a string that must begin and end with forward slashes. In addition, it can contain any ASCII character from the ! (\u0021) through the DEL character (\u007F), including most punctuation characters, digits, and upper and lowercased letters.<br/>
        :return: VirtualMFADevice:  A structure containing details about the new virtual MFA device.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnableDate</i> The date and time on which the virtual MFA device was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>User</i> The IAM user associated with this virtual MFA device.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>User</b>: The IAM user associated with this virtual MFA device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QRCodePNG</i>  A QR code PNG image that encodes <code>otpauth://totp/$virtualMFADeviceName@$AccountName?secret=$Base32String</code> where <code>$virtualMFADeviceName</code> is one of the create call arguments. <code>AccountName</code> is the user name if set (otherwise, the account ID otherwise), and <code>Base32String</code> is the seed in base32 format. The <code>Base32String</code> value is base64-encoded. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Base32StringSeed</i>  The base32 seed defined as specified in <a href="https://tools.ietf.org/html/rfc3548.txt">RFC3548</a>. The <code>Base32StringSeed</code> is base64-encoded. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SerialNumber</i> The serial number associated with <code>VirtualMFADevice</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PasswordLastUsed</i> The date and time, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>, when the user's password was last used to sign in to an AWS website. For a list of AWS websites that capture a user's last sign-in time, see the <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/credential-reports.html">Credential Reports</a> topic in the <i>Using IAM</i> guide. If a password is used more than once in a five-minute span, only the first use is returned in this field. If the field is null (no value), then it indicates that they never signed in with a password. This can be because: <ul> <li> The user never had a password. </li> <li> A password exists but has not been used since IAM started tracking this information on October 20, 2014. </li> </ul> A null value does not mean that the user <i>never</i> had a password. Also, if the user does not currently have a password, but had one in the past, then this field contains the date and time the most recent password was used. This value is returned only in the <a>GetUser</a> and <a>ListUsers</a> operations. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateDate</i> The date and time, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>, when the user was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) that identifies the user. For more information about ARNs and how to use ARNs in policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UserId</i> The stable and unique string identifying the user. For more information about IDs, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UserName</i> The friendly name identifying the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Path</i> The path to the user. For more information about paths, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide.<br/>The ARN of the policy used to set the permissions boundary for the user. For more information about permissions boundaries, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies_boundaries.html">Permissions Boundaries for IAM Identities </a> in the <i>IAM User Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PermissionsBoundary</b>: The ARN of the policy used to set the permissions boundary for the user. For more information about permissions boundaries, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies_boundaries.html">Permissions Boundaries for IAM Identities </a> in the <i>IAM User Guide</i>.        """
        pass

    def deactivate_mfa_device(self, SerialNumber: str, UserName: str):
        """Deactivates the specified MFA device and removes it from association with the user name for which it was originally enabled. For more information about creating and working with virtual MFA devices, go to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_VirtualMFA.html">Enabling a Virtual Multi-factor Authentication (MFA) Device</a> in the <i>IAM User Guide</i>.

        :param self:
        :param SerialNumber:  The serial number that uniquely identifies the MFA device. For virtual MFA devices, the serial number is the device ARN. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: =,.@:/-<br/>UserName:  The name of the user whose MFA device you want to deactivate. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def delete_access_key(self, AccessKeyId: str, UserName: str=None):
        """Deletes the access key pair associated with the specified IAM user. If you do not specify a user name, IAM determines the user name implicitly based on the AWS access key ID signing the request. This operation works for access keys under the AWS account. Consequently, you can use this operation to manage AWS account root user credentials even if the AWS account has no associated users.

        :param self:
        :param AccessKeyId:  The access key ID for the access key ID and secret access key you want to delete. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters that can consist of any upper or lowercased letter or digit.<br/>UserName:  The name of the user whose access key pair you want to delete. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def delete_account_alias(self, AccountAlias: str):
        """ Deletes the specified AWS account alias. For information about using an AWS account alias, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/AccountAlias.html">Using an Alias for Your AWS Account ID</a> in the <i>IAM User Guide</i>.

        :param self:
        :param AccountAlias:  The name of the account alias to delete. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of lowercase letters, digits, and dashes. You cannot start or finish with a dash, nor can you have two dashes in a row.<br/>
        :return:         """
        pass

    def delete_account_password_policy(self):
        """Deletes the password policy for the AWS account. There are no parameters.

        :param self:
        :param 
        :return:         """
        pass

    def delete_group(self, GroupName: str):
        """Deletes the specified IAM group. The group must not contain any users or have any attached policies.

        :param self:
        :param GroupName:  The name of the IAM group to delete. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def delete_group_policy(self, PolicyName: str, GroupName: str):
        """Deletes the specified inline policy that is embedded in the specified IAM group. A group can also have managed policies attached to it. To detach a managed policy from a group, use <a>DetachGroupPolicy</a>. For more information about policies, refer to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-vs-inline.html">Managed Policies and Inline Policies</a> in the <i>IAM User Guide</i>.

        :param self:
        :param PolicyName:  The name identifying the policy document to delete. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>GroupName:  The name (friendly name, not ARN) identifying the group that the policy is embedded in. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def delete_instance_profile(self, InstanceProfileName: str):
        """Deletes the specified instance profile. The instance profile must not have an associated role. <important> Make sure that you do not have any Amazon EC2 instances running with the instance profile you are about to delete. Deleting a role or instance profile that is associated with a running instance will break any applications running on the instance. </important> For more information about instance profiles, go to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/AboutInstanceProfiles.html">About Instance Profiles</a>.

        :param self:
        :param InstanceProfileName:  The name of the instance profile to delete. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def delete_login_profile(self, UserName: str):
        """Deletes the password for the specified IAM user, which terminates the user's ability to access AWS services through the AWS Management Console. <important>  Deleting a user's password does not prevent a user from accessing AWS through the command line interface or the API. To prevent all user access, you must also either make any access keys inactive or delete them. For more information about making keys inactive or deleting them, see <a>UpdateAccessKey</a> and <a>DeleteAccessKey</a>.  </important>

        :param self:
        :param UserName:  The name of the user whose password you want to delete. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def delete_open_id_connect_provider(self, OpenIDConnectProviderArn: str):
        """Deletes an OpenID Connect identity provider (IdP) resource object in IAM. Deleting an IAM OIDC provider resource does not update any roles that reference the provider as a principal in their trust policies. Any attempt to assume a role that references a deleted provider fails. This operation is idempotent; it does not fail or return an error if you call the operation for a provider that does not exist.

        :param self:
        :param OpenIDConnectProviderArn:  The Amazon Resource Name (ARN) of the IAM OpenID Connect provider resource object to delete. You can get a list of OpenID Connect provider resource ARNs by using the <a>ListOpenIDConnectProviders</a> operation.<br/>
        :return:         """
        pass

    def delete_policy(self, PolicyArn: str):
        """Deletes the specified managed policy. Before you can delete a managed policy, you must first detach the policy from all users, groups, and roles that it is attached to. In addition, you must delete all the policy's versions. The following steps describe the process for deleting a managed policy: <ul> <li> Detach the policy from all users, groups, and roles that the policy is attached to, using the <a>DetachUserPolicy</a>, <a>DetachGroupPolicy</a>, or <a>DetachRolePolicy</a> API operations. To list all the users, groups, and roles that a policy is attached to, use <a>ListEntitiesForPolicy</a>. </li> <li> Delete all versions of the policy using <a>DeletePolicyVersion</a>. To list the policy's versions, use <a>ListPolicyVersions</a>. You cannot use <a>DeletePolicyVersion</a> to delete the version that is marked as the default version. You delete the policy's default version in the next step of the process. </li> <li> Delete the policy (this automatically deletes the policy's default version) using this API. </li> </ul> For information about managed policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-vs-inline.html">Managed Policies and Inline Policies</a> in the <i>IAM User Guide</i>.

        :param self:
        :param PolicyArn:  The Amazon Resource Name (ARN) of the IAM policy you want to delete. For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the <i>AWS General Reference</i>.<br/>
        :return:         """
        pass

    def delete_policy_version(self, VersionId: str, PolicyArn: str):
        """Deletes the specified version from the specified managed policy. You cannot delete the default version from a policy using this API. To delete the default version from a policy, use <a>DeletePolicy</a>. To find out which version of a policy is marked as the default version, use <a>ListPolicyVersions</a>. For information about versions for managed policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-versions.html">Versioning for Managed Policies</a> in the <i>IAM User Guide</i>.

        :param self:
        :param VersionId:  The policy version to delete. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters that consists of the lowercase letter 'v' followed by one or two digits, and optionally followed by a period '.' and a string of letters and digits. For more information about managed policy versions, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-versions.html">Versioning for Managed Policies</a> in the <i>IAM User Guide</i>.<br/>PolicyArn:  The Amazon Resource Name (ARN) of the IAM policy from which you want to delete a version. For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the <i>AWS General Reference</i>.<br/>
        :return:         """
        pass

    def delete_role(self, RoleName: str):
        """Deletes the specified role. The role must not have any policies attached. For more information about roles, go to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/WorkingWithRoles.html">Working with Roles</a>. <important> Make sure that you do not have any Amazon EC2 instances running with the role you are about to delete. Deleting a role or instance profile that is associated with a running instance will break any applications running on the instance. </important>

        :param self:
        :param RoleName:  The name of the role to delete. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def delete_role_permissions_boundary(self, RoleName: str):
        """Deletes the permissions boundary for the specified IAM role.  <important> Deleting the permissions boundary for a role might increase its permissions. For example, it might allow anyone who assumes the role to perform all the actions granted in its permissions policies.  </important>

        :param self:
        :param RoleName:  The name (friendly name, not ARN) of the IAM role from which you want to remove the permissions boundary.<br/>
        :return:         """
        pass

    def delete_role_policy(self, PolicyName: str, RoleName: str):
        """Deletes the specified inline policy that is embedded in the specified IAM role. A role can also have managed policies attached to it. To detach a managed policy from a role, use <a>DetachRolePolicy</a>. For more information about policies, refer to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-vs-inline.html">Managed Policies and Inline Policies</a> in the <i>IAM User Guide</i>.

        :param self:
        :param PolicyName:  The name of the inline policy to delete from the specified IAM role. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>RoleName:  The name (friendly name, not ARN) identifying the role that the policy is embedded in. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def delete_saml_provider(self, SAMLProviderArn: str):
        """Deletes a SAML provider resource in IAM. Deleting the provider resource from IAM does not update any roles that reference the SAML provider resource's ARN as a principal in their trust policies. Any attempt to assume a role that references a non-existent provider resource ARN fails. <note>  This operation requires <a href="https://docs.aws.amazon.com/general/latest/gr/signature-version-4.html">Signature Version 4</a>. </note>

        :param self:
        :param SAMLProviderArn:  The Amazon Resource Name (ARN) of the SAML provider to delete.<br/>
        :return:         """
        pass

    def delete_ssh_public_key(self, SSHPublicKeyId: str, UserName: str):
        """Deletes the specified SSH public key. The SSH public key deleted by this operation is used only for authenticating the associated IAM user to an AWS CodeCommit repository. For more information about using SSH keys to authenticate to an AWS CodeCommit repository, see <a href="https://docs.aws.amazon.com/codecommit/latest/userguide/setting-up-credentials-ssh.html">Set up AWS CodeCommit for SSH Connections</a> in the <i>AWS CodeCommit User Guide</i>.

        :param self:
        :param SSHPublicKeyId:  The unique identifier for the SSH public key. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters that can consist of any upper or lowercased letter or digit.<br/>UserName:  The name of the IAM user associated with the SSH public key. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def delete_server_certificate(self, ServerCertificateName: str):
        """Deletes the specified server certificate. For more information about working with server certificates, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_server-certs.html">Working with Server Certificates</a> in the <i>IAM User Guide</i>. This topic also includes a list of AWS services that can use the server certificates that you manage with IAM. <important>  If you are using a server certificate with Elastic Load Balancing, deleting the certificate could have implications for your application. If Elastic Load Balancing doesn't detect the deletion of bound certificates, it may continue to use the certificates. This could cause Elastic Load Balancing to stop accepting traffic. We recommend that you remove the reference to the certificate from Elastic Load Balancing before using this command to delete the certificate. For more information, go to <a href="https://docs.aws.amazon.com/ElasticLoadBalancing/latest/APIReference/API_DeleteLoadBalancerListeners.html">DeleteLoadBalancerListeners</a> in the <i>Elastic Load Balancing API Reference</i>. </important>

        :param self:
        :param ServerCertificateName:  The name of the server certificate you want to delete. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def delete_service_linked_role(self, RoleName: str) -> DeleteServiceLinkedRoleResponse:
        """Submits a service-linked role deletion request and returns a <code>DeletionTaskId</code>, which you can use to check the status of the deletion. Before you call this operation, confirm that the role has no active sessions and that any resources used by the role in the linked service are deleted. If you call this operation more than once for the same service-linked role and an earlier deletion task is not complete, then the <code>DeletionTaskId</code> of the earlier request is returned. If you submit a deletion request for a service-linked role whose linked service is still accessing a resource, then the deletion task fails. If it fails, the <a>GetServiceLinkedRoleDeletionStatus</a> API operation returns the reason for the failure, usually including the resources that must be deleted. To delete the service-linked role, you must first remove those resources from the linked service and then submit the deletion request again. Resources are specific to the service that is linked to the role. For more information about removing resources from a service, see the <a href="http://docs.aws.amazon.com/">AWS documentation</a> for your service. For more information about service-linked roles, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_terms-and-concepts.html#iam-term-service-linked-role">Roles Terms and Concepts: AWS Service-Linked Role</a> in the <i>IAM User Guide</i>.

        :param self:
        :param RoleName:  The name of the service-linked role to be deleted.<br/>
        :return: DeletionTaskId:  The deletion task identifier that you can use to check the status of the deletion. This identifier is returned in the format <code>task/aws-service-role/&lt;service-principal-name&gt;/&lt;role-name&gt;/&lt;task-uuid&gt;</code>.<br/>        """
        pass

    def delete_service_specific_credential(self, ServiceSpecificCredentialId: str, UserName: str=None):
        """Deletes the specified service-specific credential.

        :param self:
        :param ServiceSpecificCredentialId:  The unique identifier of the service-specific credential. You can get this value by calling <a>ListServiceSpecificCredentials</a>. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters that can consist of any upper or lowercased letter or digit.<br/>UserName:  The name of the IAM user associated with the service-specific credential. If this value is not specified, then the operation assumes the user whose credentials are used to call the operation. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def delete_signing_certificate(self, CertificateId: str, UserName: str=None):
        """Deletes a signing certificate associated with the specified IAM user. If you do not specify a user name, IAM determines the user name implicitly based on the AWS access key ID signing the request. This operation works for access keys under the AWS account. Consequently, you can use this operation to manage AWS account root user credentials even if the AWS account has no associated IAM users.

        :param self:
        :param CertificateId:  The ID of the signing certificate to delete. The format of this parameter, as described by its <a href="http://wikipedia.org/wiki/regex">regex</a> pattern, is a string of characters that can be upper- or lower-cased letters or digits.<br/>UserName:  The name of the user the signing certificate belongs to. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def delete_user(self, UserName: str):
        """Deletes the specified IAM user. Unlike the AWS Management Console, when you delete a user programmatically, you must delete the items attached to the user manually, or the deletion fails. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_users_manage.html#id_users_deleting_cli">Deleting an IAM User</a>. Before attempting to delete a user, remove the following items: <ul> <li> Password (<a>DeleteLoginProfile</a>) </li> <li> Access keys (<a>DeleteAccessKey</a>) </li> <li> Signing certificate (<a>DeleteSigningCertificate</a>) </li> <li> SSH public key (<a>DeleteSSHPublicKey</a>) </li> <li> Git credentials (<a>DeleteServiceSpecificCredential</a>) </li> <li> Multi-factor authentication (MFA) device (<a>DeactivateMFADevice</a>, <a>DeleteVirtualMFADevice</a>) </li> <li> Inline policies (<a>DeleteUserPolicy</a>) </li> <li> Attached managed policies (<a>DetachUserPolicy</a>) </li> <li> Group memberships (<a>RemoveUserFromGroup</a>) </li> </ul>

        :param self:
        :param UserName:  The name of the user to delete. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def delete_user_permissions_boundary(self, UserName: str):
        """Deletes the permissions boundary for the specified IAM user. <important> Deleting the permissions boundary for a user might increase its permissions by allowing the user to perform all the actions granted in its permissions policies.  </important>

        :param self:
        :param UserName:  The name (friendly name, not ARN) of the IAM user from which you want to remove the permissions boundary.<br/>
        :return:         """
        pass

    def delete_user_policy(self, PolicyName: str, UserName: str):
        """Deletes the specified inline policy that is embedded in the specified IAM user. A user can also have managed policies attached to it. To detach a managed policy from a user, use <a>DetachUserPolicy</a>. For more information about policies, refer to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-vs-inline.html">Managed Policies and Inline Policies</a> in the <i>IAM User Guide</i>.

        :param self:
        :param PolicyName:  The name identifying the policy document to delete. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>UserName:  The name (friendly name, not ARN) identifying the user that the policy is embedded in. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def delete_virtual_mfa_device(self, SerialNumber: str):
        """Deletes a virtual MFA device. <note>  You must deactivate a user's virtual MFA device before you can delete it. For information about deactivating MFA devices, see <a>DeactivateMFADevice</a>.  </note>

        :param self:
        :param SerialNumber:  The serial number that uniquely identifies the MFA device. For virtual MFA devices, the serial number is the same as the ARN. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: =,.@:/-<br/>
        :return:         """
        pass

    def detach_group_policy(self, PolicyArn: str, GroupName: str):
        """Removes the specified managed policy from the specified IAM group. A group can also have inline policies embedded with it. To delete an inline policy, use the <a>DeleteGroupPolicy</a> API. For information about policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-vs-inline.html">Managed Policies and Inline Policies</a> in the <i>IAM User Guide</i>.

        :param self:
        :param PolicyArn:  The Amazon Resource Name (ARN) of the IAM policy you want to detach. For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the <i>AWS General Reference</i>.<br/>GroupName:  The name (friendly name, not ARN) of the IAM group to detach the policy from. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def detach_role_policy(self, PolicyArn: str, RoleName: str):
        """Removes the specified managed policy from the specified role. A role can also have inline policies embedded with it. To delete an inline policy, use the <a>DeleteRolePolicy</a> API. For information about policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-vs-inline.html">Managed Policies and Inline Policies</a> in the <i>IAM User Guide</i>.

        :param self:
        :param PolicyArn:  The Amazon Resource Name (ARN) of the IAM policy you want to detach. For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the <i>AWS General Reference</i>.<br/>RoleName:  The name (friendly name, not ARN) of the IAM role to detach the policy from. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def detach_user_policy(self, PolicyArn: str, UserName: str):
        """Removes the specified managed policy from the specified user. A user can also have inline policies embedded with it. To delete an inline policy, use the <a>DeleteUserPolicy</a> API. For information about policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-vs-inline.html">Managed Policies and Inline Policies</a> in the <i>IAM User Guide</i>.

        :param self:
        :param PolicyArn:  The Amazon Resource Name (ARN) of the IAM policy you want to detach. For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the <i>AWS General Reference</i>.<br/>UserName:  The name (friendly name, not ARN) of the IAM user to detach the policy from. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def enable_mfa_device(self, AuthenticationCode2: str, AuthenticationCode1: str, SerialNumber: str, UserName: str):
        """Enables the specified MFA device and associates it with the specified IAM user. When enabled, the MFA device is required for every subsequent login by the IAM user associated with the device.

        :param self:
        :param AuthenticationCode2:  A subsequent authentication code emitted by the device. The format for this parameter is a string of six digits. <important> Submit your request immediately after generating the authentication codes. If you generate the codes and then wait too long to submit the request, the MFA device successfully associates with the user but the MFA device becomes out of sync. This happens because time-based one-time passwords (TOTP) expire after a short period of time. If this happens, you can <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_mfa_sync.html">resync the device</a>. </important><br/>AuthenticationCode1:  An authentication code emitted by the device.  The format for this parameter is a string of six digits. <important> Submit your request immediately after generating the authentication codes. If you generate the codes and then wait too long to submit the request, the MFA device successfully associates with the user but the MFA device becomes out of sync. This happens because time-based one-time passwords (TOTP) expire after a short period of time. If this happens, you can <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_mfa_sync.html">resync the device</a>. </important><br/>SerialNumber:  The serial number that uniquely identifies the MFA device. For virtual MFA devices, the serial number is the device ARN. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: =,.@:/-<br/>UserName:  The name of the IAM user for whom you want to enable the MFA device. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def generate_credential_report(self) -> GenerateCredentialReportResponse:
        """ Generates a credential report for the AWS account. For more information about the credential report, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/credential-reports.html">Getting Credential Reports</a> in the <i>IAM User Guide</i>.

        :param self:
        :param 
        :return: Description:  Information about the credential report.<br/>State:  Information about the state of the credential report.<br/>        """
        pass

    def generate_organizations_access_report(self, EntityPath: str, OrganizationsPolicyId: str=None) -> GenerateOrganizationsAccessReportResponse:
        """Generates a report for service last accessed data for AWS Organizations. You can generate a report for any entities (organization root, organizational unit, or account) or policies in your organization. To call this operation, you must be signed in using your AWS Organizations master account credentials. You can use your long-term IAM user or root user credentials, or temporary credentials from assuming an IAM role. SCPs must be enabled for your organization root. You must have the required IAM and AWS Organizations permissions. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies_access-advisor.html">Refining Permissions Using Service Last Accessed Data</a> in the <i>IAM User Guide</i>. You can generate a service last accessed data report for entities by specifying only the entity's path. This data includes a list of services that are allowed by any service control policies (SCPs) that apply to the entity. You can generate a service last accessed data report for a policy by specifying an entity's path and an optional AWS Organizations policy ID. This data includes a list of services that are allowed by the specified SCP. For each service in both report types, the data includes the most recent account activity that the policy allows to account principals in the entity or the entity's children. For important information about the data, reporting period, permissions required, troubleshooting, and supported Regions see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies_access-advisor.html">Reducing Permissions Using Service Last Accessed Data</a> in the <i>IAM User Guide</i>. <important> The data includes all attempts to access AWS, not just the successful ones. This includes all attempts that were made using the AWS Management Console, the AWS API through any of the SDKs, or any of the command line tools. An unexpected entry in the service last accessed data does not mean that an account has been compromised, because the request might have been denied. Refer to your CloudTrail logs as the authoritative source for information about all API calls and whether they were successful or denied access. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/cloudtrail-integration.html">Logging IAM Events with CloudTrail</a> in the <i>IAM User Guide</i>. </important> This operation returns a <code>JobId</code>. Use this parameter in the <code> <a>GetOrganizationsAccessReport</a> </code> operation to check the status of the report generation. To check the status of this request, use the <code>JobId</code> parameter in the <code> <a>GetOrganizationsAccessReport</a> </code> operation and test the <code>JobStatus</code> response parameter. When the job is complete, you can retrieve the report. To generate a service last accessed data report for entities, specify an entity path without specifying the optional AWS Organizations policy ID. The type of entity that you specify determines the data returned in the report. <ul> <li>  <b>Root</b> – When you specify the organizations root as the entity, the resulting report lists all of the services allowed by SCPs that are attached to your root. For each service, the report includes data for all accounts in your organization except the master account, because the master account is not limited by SCPs. </li> <li>  <b>OU</b> – When you specify an organizational unit (OU) as the entity, the resulting report lists all of the services allowed by SCPs that are attached to the OU and its parents. For each service, the report includes data for all accounts in the OU or its children. This data excludes the master account, because the master account is not limited by SCPs. </li> <li>  <b>Master account</b> – When you specify the master account, the resulting report lists all AWS services, because the master account is not limited by SCPs. For each service, the report includes data for only the master account. </li> <li>  <b>Account</b> – When you specify another account as the entity, the resulting report lists all of the services allowed by SCPs that are attached to the account and its parents. For each service, the report includes data for only the specified account. </li> </ul> To generate a service last accessed data report for policies, specify an entity path and the optional AWS Organizations policy ID. The type of entity that you specify determines the data returned for each service. <ul> <li>  <b>Root</b> – When you specify the root entity and a policy ID, the resulting report lists all of the services that are allowed by the specified SCP. For each service, the report includes data for all accounts in your organization to which the SCP applies. This data excludes the master account, because the master account is not limited by SCPs. If the SCP is not attached to any entities in the organization, then the report will return a list of services with no data. </li> <li>  <b>OU</b> – When you specify an OU entity and a policy ID, the resulting report lists all of the services that are allowed by the specified SCP. For each service, the report includes data for all accounts in the OU or its children to which the SCP applies. This means that other accounts outside the OU that are affected by the SCP might not be included in the data. This data excludes the master account, because the master account is not limited by SCPs. If the SCP is not attached to the OU or one of its children, the report will return a list of services with no data. </li> <li>  <b>Master account</b> – When you specify the master account, the resulting report lists all AWS services, because the master account is not limited by SCPs. If you specify a policy ID in the CLI or API, the policy is ignored. For each service, the report includes data for only the master account. </li> <li>  <b>Account</b> – When you specify another account entity and a policy ID, the resulting report lists all of the services that are allowed by the specified SCP. For each service, the report includes data for only the specified account. This means that other accounts in the organization that are affected by the SCP might not be included in the data. If the SCP is not attached to the account, the report will return a list of services with no data. </li> </ul> <note> Service last accessed data does not use other policy types when determining whether a principal could access a service. These other policy types include identity-based policies, resource-based policies, access control lists, IAM permissions boundaries, and STS assume role policies. It only applies SCP logic. For more about the evaluation of policy types, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_policies_evaluation-logic.html#policy-eval-basics">Evaluating Policies</a> in the <i>IAM User Guide</i>. </note> For more information about service last accessed data, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies_access-advisor.html">Reducing Policy Scope by Viewing User Activity</a> in the <i>IAM User Guide</i>.

        :param self:
        :param OrganizationsPolicyId:  The identifier of the AWS Organizations service control policy (SCP). This parameter is optional. This ID is used to generate information about when an account principal that is limited by the SCP attempted to access an AWS service.<br/>EntityPath:  The path of the AWS Organizations entity (root, OU, or account). You can build an entity path using the known structure of your organization. For example, assume that your account ID is <code>123456789012</code> and its parent OU ID is <code>ou-rge0-awsabcde</code>. The organization root ID is <code>r-f6g7h8i9j0example</code> and your organization ID is <code>o-a1b2c3d4e5</code>. Your entity path is <code>o-a1b2c3d4e5/r-f6g7h8i9j0example/ou-rge0-awsabcde/123456789012</code>.<br/>
        :return: JobId:  The job identifier that you can use in the <a>GetOrganizationsAccessReport</a> operation.<br/>        """
        pass

    def generate_service_last_accessed_details(self, Arn: str) -> GenerateServiceLastAccessedDetailsResponse:
        """Generates a report that includes details about when an IAM resource (user, group, role, or policy) was last used in an attempt to access AWS services. Recent activity usually appears within four hours. IAM reports activity for the last 365 days, or less if your Region began supporting this feature within the last year. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies_access-advisor.html#access-advisor_tracking-period">Regions Where Data Is Tracked</a>. <important> The service last accessed data includes all attempts to access an AWS API, not just the successful ones. This includes all attempts that were made using the AWS Management Console, the AWS API through any of the SDKs, or any of the command line tools. An unexpected entry in the service last accessed data does not mean that your account has been compromised, because the request might have been denied. Refer to your CloudTrail logs as the authoritative source for information about all API calls and whether they were successful or denied access. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/cloudtrail-integration.html">Logging IAM Events with CloudTrail</a> in the <i>IAM User Guide</i>. </important> The <code>GenerateServiceLastAccessedDetails</code> operation returns a <code>JobId</code>. Use this parameter in the following operations to retrieve the following details from your report:  <ul> <li>  <a>GetServiceLastAccessedDetails</a> – Use this operation for users, groups, roles, or policies to list every AWS service that the resource could access using permissions policies. For each service, the response includes information about the most recent access attempt.  </li> <li>  <a>GetServiceLastAccessedDetailsWithEntities</a> – Use this operation for groups and policies to list information about the associated entities (users or roles) that attempted to access a specific AWS service.  </li> </ul> To check the status of the <code>GenerateServiceLastAccessedDetails</code> request, use the <code>JobId</code> parameter in the same operations and test the <code>JobStatus</code> response parameter. For additional information about the permissions policies that allow an identity (user, group, or role) to access specific services, use the <a>ListPoliciesGrantingServiceAccess</a> operation. <note> Service last accessed data does not use other policy types when determining whether a resource could access a service. These other policy types include resource-based policies, access control lists, AWS Organizations policies, IAM permissions boundaries, and AWS STS assume role policies. It only applies permissions policy logic. For more about the evaluation of policy types, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_policies_evaluation-logic.html#policy-eval-basics">Evaluating Policies</a> in the <i>IAM User Guide</i>. </note> For more information about service last accessed data, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies_access-advisor.html">Reducing Policy Scope by Viewing User Activity</a> in the <i>IAM User Guide</i>.

        :param self:
        :param Arn:  The ARN of the IAM resource (user, group, role, or managed policy) used to generate information about when the resource was last used in an attempt to access an AWS service.<br/>
        :return: JobId:  The job ID that you can use in the <a>GetServiceLastAccessedDetails</a> or <a>GetServiceLastAccessedDetailsWithEntities</a> operations.<br/>        """
        pass

    def get_access_key_last_used(self, AccessKeyId: str) -> GetAccessKeyLastUsedResponse:
        """Retrieves information about when the specified access key was last used. The information includes the date and time of last use, along with the AWS service and Region that were specified in the last request made with that key.

        :param self:
        :param AccessKeyId:  The identifier of an access key. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters that can consist of any upper or lowercased letter or digit.<br/>
        :return: AccessKeyLastUsed:  Contains information about the last time the access key was used.:<br/>UserName:  The name of the AWS IAM user that owns this access key. <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Region</i> The AWS Region where this access key was most recently used. The value for this field is "N/A" in the following situations: <ul> <li> The user does not have an access key. </li> <li> An access key exists but has not been used since IAM began tracking this information. </li> <li> There is no sign-in data associated with the user. </li> </ul> For more information about AWS Regions, see <a href="https://docs.aws.amazon.com/general/latest/gr/rande.html">Regions and Endpoints</a> in the Amazon Web Services General Reference.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceName</i> The name of the AWS service with which this access key was most recently used. The value of this field is "N/A" in the following situations: <ul> <li> The user does not have an access key. </li> <li> An access key exists but has not been used since IAM started tracking this information. </li> <li> There is no sign-in data associated with the user. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUsedDate</i> The date and time, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>, when the access key was most recently used. This field is null in the following situations: <ul> <li> The user does not have an access key. </li> <li> An access key exists but has not been used since IAM began tracking this information. </li> <li> There is no sign-in data associated with the user. </li> </ul><br/>        """
        pass

    def get_account_authorization_details(self, Filter: List[EntityType]=None, MaxItems: int=None, Marker: str=None) -> GetAccountAuthorizationDetailsResponse:
        """Retrieves information about all IAM users, groups, roles, and policies in your AWS account, including their relationships to one another. Use this API to obtain a snapshot of the configuration of IAM permissions (users, groups, roles, and policies) in your account. <note> Policies returned by this API are URL-encoded compliant with <a href="https://tools.ietf.org/html/rfc3986">RFC 3986</a>. You can use a URL decoding method to convert the policy back to plain JSON text. For example, if you use Java, you can use the <code>decode</code> method of the <code>java.net.URLDecoder</code> utility class in the Java SDK. Other languages and SDKs provide similar functionality. </note> You can optionally filter the results using the <code>Filter</code> parameter. You can paginate the results using the <code>MaxItems</code> and <code>Marker</code> parameters.

        :param self:
        :param Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>MaxItems:  Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>Filter:  A list of entity types used to filter the results. Only the entities that match the types you specify are included in the output. Use the value <code>LocalManagedPolicy</code> to include customer managed policies. The format for this parameter is a comma-separated (if more than one) list of strings. Each string value in the list must be one of the valid values listed below.<br/>
        :return: Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when there are more results available. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>Policies:  A list containing information about managed policies.<br/>RoleDetailList:  A list containing information about IAM roles.<br/>GroupDetailList:  A list containing information about IAM groups.<br/>UserDetailList:  A list containing information about IAM users.<br/>        """
        pass

    def get_account_password_policy(self) -> GetAccountPasswordPolicyResponse:
        """Retrieves the password policy for the AWS account. For more information about using a password policy, go to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_ManagingPasswordPolicies.html">Managing an IAM Password Policy</a>.

        :param self:
        :param 
        :return: PasswordPolicy:  A structure that contains details about the account's password policy.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HardExpiry</i> Specifies whether IAM users are prevented from setting a new password after their password has expired.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PasswordReusePrevention</i> Specifies the number of previous passwords that IAM users are prevented from reusing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxPasswordAge</i> The number of days that an IAM user password is valid.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpirePasswords</i> Indicates whether passwords in the account expire. Returns true if <code>MaxPasswordAge</code> contains a value greater than 0. Returns false if MaxPasswordAge is 0 or not present.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowUsersToChangePassword</i> Specifies whether IAM users are allowed to change their own password.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequireLowercaseCharacters</i> Specifies whether to require lowercase characters for IAM user passwords.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequireUppercaseCharacters</i> Specifies whether to require uppercase characters for IAM user passwords.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequireNumbers</i> Specifies whether to require numbers for IAM user passwords.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequireSymbols</i> Specifies whether to require symbols for IAM user passwords.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MinimumPasswordLength</i> Minimum length to require for IAM user passwords.<br/>        """
        pass

    def get_account_summary(self) -> GetAccountSummaryResponse:
        """Retrieves information about IAM entity usage and IAM quotas in the AWS account.  For information about limitations on IAM entities, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/LimitationsOnEntities.html">Limitations on IAM Entities</a> in the <i>IAM User Guide</i>.

        :param self:
        :param 
        :return: SummaryMap:  A set of key–value pairs containing information about IAM entity usage and IAM quotas.<br/>        """
        pass

    def get_context_keys_for_custom_policy(self, PolicyInputList: List[policyDocumentType]) -> GetContextKeysForPolicyResponse:
        """Gets a list of all of the context keys referenced in the input policies. The policies are supplied as a list of one or more strings. To get the context keys from policies associated with an IAM user, group, or role, use <a>GetContextKeysForPrincipalPolicy</a>. Context keys are variables maintained by AWS and its services that provide details about the context of an API query request. Context keys can be evaluated by testing against a value specified in an IAM policy. Use <code>GetContextKeysForCustomPolicy</code> to understand what key names and values you must supply when you call <a>SimulateCustomPolicy</a>. Note that all parameters are shown in unencoded form here for clarity but must be URL encoded to be included as a part of a real HTML request.

        :param self:
        :param PolicyInputList:  A list of policies for which you want the list of context keys referenced in those policies. Each document is specified as a string containing the complete, valid JSON text of an IAM policy. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> used to validate this parameter is a string of characters consisting of the following: <ul> <li> Any printable ASCII character ranging from the space character (\u0020) through the end of the ASCII character range </li> <li> The printable characters in the Basic Latin and Latin-1 Supplement character set (through \u00FF) </li> <li> The special characters tab (\u0009), line feed (\u000A), and carriage return (\u000D) </li> </ul><br/>
        :return: ContextKeyNames:  The list of context keys that are referenced in the input policies.<br/>        """
        pass

    def get_context_keys_for_principal_policy(self, PolicySourceArn: str, PolicyInputList: List[policyDocumentType]=None) -> GetContextKeysForPolicyResponse:
        """Gets a list of all of the context keys referenced in all the IAM policies that are attached to the specified IAM entity. The entity can be an IAM user, group, or role. If you specify a user, then the request also includes all of the policies attached to groups that the user is a member of. You can optionally include a list of one or more additional policies, specified as strings. If you want to include <i>only</i> a list of policies by string, use <a>GetContextKeysForCustomPolicy</a> instead.  <b>Note:</b> This API discloses information about the permissions granted to other users. If you do not want users to see other user's permissions, then consider allowing them to use <a>GetContextKeysForCustomPolicy</a> instead. Context keys are variables maintained by AWS and its services that provide details about the context of an API query request. Context keys can be evaluated by testing against a value in an IAM policy. Use <a>GetContextKeysForPrincipalPolicy</a> to understand what key names and values you must supply when you call <a>SimulatePrincipalPolicy</a>.

        :param self:
        :param PolicyInputList:  An optional list of additional policies for which you want the list of context keys that are referenced. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> used to validate this parameter is a string of characters consisting of the following: <ul> <li> Any printable ASCII character ranging from the space character (\u0020) through the end of the ASCII character range </li> <li> The printable characters in the Basic Latin and Latin-1 Supplement character set (through \u00FF) </li> <li> The special characters tab (\u0009), line feed (\u000A), and carriage return (\u000D) </li> </ul><br/>PolicySourceArn:  The ARN of a user, group, or role whose policies contain the context keys that you want listed. If you specify a user, the list includes context keys that are found in all policies that are attached to the user. The list also includes all groups that the user is a member of. If you pick a group or a role, then it includes only those context keys that are found in policies attached to that entity. Note that all parameters are shown in unencoded form here for clarity, but must be URL encoded to be included as a part of a real HTML request. For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the <i>AWS General Reference</i>.<br/>
        :return: ContextKeyNames:  The list of context keys that are referenced in the input policies.<br/>        """
        pass

    def get_credential_report(self) -> GetCredentialReportResponse:
        """ Retrieves a credential report for the AWS account. For more information about the credential report, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/credential-reports.html">Getting Credential Reports</a> in the <i>IAM User Guide</i>.

        :param self:
        :param 
        :return: GeneratedTime:   The date and time when the credential report was created, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>.<br/>ReportFormat:  The format (MIME type) of the credential report.<br/>Content:  Contains the credential report. The report is Base64-encoded.<br/>        """
        pass

    def get_group(self, GroupName: str, Marker: str=None, MaxItems: int=None) -> GetGroupResponse:
        """ Returns a list of IAM users that are in the specified IAM group. You can paginate the results using the <code>MaxItems</code> and <code>Marker</code> parameters.

        :param self:
        :param MaxItems:  Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>GroupName:  The name of the group. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when there are more results available. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>Users:  A list of users in the group.<br/>Group:  A structure that contains details about the group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateDate</i> The date and time, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>, when the group was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i>  The Amazon Resource Name (ARN) specifying the group. For more information about ARNs and how to use them in policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupId</i>  The stable and unique string identifying the group. For more information about IDs, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupName</i> The friendly name that identifies the group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Path</i> The path to the group. For more information about paths, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide. <br/>        """
        pass

    def get_group_policy(self, PolicyName: str, GroupName: str) -> GetGroupPolicyResponse:
        """Retrieves the specified inline policy document that is embedded in the specified IAM group. <note> Policies returned by this API are URL-encoded compliant with <a href="https://tools.ietf.org/html/rfc3986">RFC 3986</a>. You can use a URL decoding method to convert the policy back to plain JSON text. For example, if you use Java, you can use the <code>decode</code> method of the <code>java.net.URLDecoder</code> utility class in the Java SDK. Other languages and SDKs provide similar functionality. </note> An IAM group can also have managed policies attached to it. To retrieve a managed policy document that is attached to a group, use <a>GetPolicy</a> to determine the policy's default version, then use <a>GetPolicyVersion</a> to retrieve the policy document. For more information about policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-vs-inline.html">Managed Policies and Inline Policies</a> in the <i>IAM User Guide</i>.

        :param self:
        :param PolicyName:  The name of the policy document to get. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>GroupName:  The name of the group the policy is associated with. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: PolicyDocument:  The policy document. IAM stores policies in JSON format. However, resources that were created using AWS CloudFormation templates can be formatted in YAML. AWS CloudFormation always converts a YAML policy to JSON format before submitting it to IAM.<br/>PolicyName:  The name of the policy.<br/>GroupName:  The group the policy is associated with.<br/>        """
        pass

    def get_instance_profile(self, InstanceProfileName: str) -> GetInstanceProfileResponse:
        """ Retrieves information about the specified instance profile, including the instance profile's path, GUID, ARN, and role. For more information about instance profiles, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/AboutInstanceProfiles.html">About Instance Profiles</a> in the <i>IAM User Guide</i>.

        :param self:
        :param InstanceProfileName:  The name of the instance profile to get information about. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: InstanceProfile:  A structure containing details about the instance profile.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Roles</i> The role associated with the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateDate</i> The date when the instance profile was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i>  The Amazon Resource Name (ARN) specifying the instance profile. For more information about ARNs and how to use them in policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceProfileId</i>  The stable and unique string identifying the instance profile. For more information about IDs, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceProfileName</i> The name identifying the instance profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Path</i>  The path to the instance profile. For more information about paths, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide. <br/>        """
        pass

    def get_login_profile(self, UserName: str) -> GetLoginProfileResponse:
        """Retrieves the user name and password-creation date for the specified IAM user. If the user has not been assigned a password, the operation returns a 404 (<code>NoSuchEntity</code>) error.

        :param self:
        :param UserName:  The name of the user whose login profile you want to retrieve. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: LoginProfile:  A structure containing the user name and password create date for the user.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PasswordResetRequired</i> Specifies whether the user is required to set a new password on next sign-in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateDate</i> The date when the password for the user was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserName</i> The name of the user, which can be used for signing in to the AWS Management Console.<br/>        """
        pass

    def get_open_id_connect_provider(self, OpenIDConnectProviderArn: str) -> GetOpenIDConnectProviderResponse:
        """Returns information about the specified OpenID Connect (OIDC) provider resource object in IAM.

        :param self:
        :param OpenIDConnectProviderArn:  The Amazon Resource Name (ARN) of the OIDC provider resource object in IAM to get information for. You can get a list of OIDC provider resource ARNs by using the <a>ListOpenIDConnectProviders</a> operation. For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the <i>AWS General Reference</i>.<br/>
        :return: CreateDate:  The date and time when the IAM OIDC provider resource object was created in the AWS account.<br/>ThumbprintList:  A list of certificate thumbprints that are associated with the specified IAM OIDC provider resource object. For more information, see <a>CreateOpenIDConnectProvider</a>. <br/>ClientIDList:  A list of client IDs (also known as audiences) that are associated with the specified IAM OIDC provider resource object. For more information, see <a>CreateOpenIDConnectProvider</a>.<br/>Url:  The URL that the IAM OIDC provider resource object is associated with. For more information, see <a>CreateOpenIDConnectProvider</a>.<br/>        """
        pass

    def get_organizations_access_report(self, JobId: str, MaxItems: int=None, Marker: str=None, SortKey: str=None) -> GetOrganizationsAccessReportResponse:
        """Retrieves the service last accessed data report for AWS Organizations that was previously generated using the <code> <a>GenerateOrganizationsAccessReport</a> </code> operation. This operation retrieves the status of your report job and the report contents. Depending on the parameters that you passed when you generated the report, the data returned could include different information. For details, see <a>GenerateOrganizationsAccessReport</a>. To call this operation, you must be signed in to the master account in your organization. SCPs must be enabled for your organization root. You must have permissions to perform this operation. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies_access-advisor.html">Refining Permissions Using Service Last Accessed Data</a> in the <i>IAM User Guide</i>. For each service that principals in an account (root users, IAM users, or IAM roles) could access using SCPs, the operation returns details about the most recent access attempt. If there was no attempt, the service is listed without details about the most recent attempt to access the service. If the operation fails, it returns the reason that it failed. By default, the list is sorted by service namespace.

        :param self:
        :param SortKey:  The key that is used to sort the results. If you choose the namespace key, the results are returned in alphabetical order. If you choose the time key, the results are sorted numerically by the date and time.<br/>Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>MaxItems:  Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>JobId:  The identifier of the request generated by the <a>GenerateOrganizationsAccessReport</a> operation.<br/>
        :return: ErrorDetails:  Contains information about the reason that the operation failed. This data type is used as a response element in the <a>GetOrganizationsAccessReport</a>, <a>GetServiceLastAccessedDetails</a>, and <a>GetServiceLastAccessedDetailsWithEntities</a> operations.:<br/>Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when there are more results available. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>AccessDetails:  An object that contains details about the most recent attempt to access the service.<br/>NumberOfServicesNotAccessed:  The number of services that account principals are allowed but did not attempt to access.<br/>NumberOfServicesAccessible:  The number of services that the applicable SCPs allow account principals to access.<br/>JobCompletionDate:  The date and time, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>, when the generated report job was completed or failed. This field is null if the job is still in progress, as indicated by a job status value of <code>IN_PROGRESS</code>.<br/>JobCreationDate:  The date and time, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>, when the report job was created.<br/>JobStatus:  The status of the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Code</i> The error code associated with the operation failure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> Detailed information about the reason that the operation failed.<br/>        """
        pass

    def get_policy(self, PolicyArn: str) -> GetPolicyResponse:
        """Retrieves information about the specified managed policy, including the policy's default version and the total number of IAM users, groups, and roles to which the policy is attached. To retrieve the list of the specific users, groups, and roles that the policy is attached to, use the <a>ListEntitiesForPolicy</a> API. This API returns metadata about the policy. To retrieve the actual policy document for a specific version of the policy, use <a>GetPolicyVersion</a>. This API retrieves information about managed policies. To retrieve information about an inline policy that is embedded with an IAM user, group, or role, use the <a>GetUserPolicy</a>, <a>GetGroupPolicy</a>, or <a>GetRolePolicy</a> API. For more information about policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-vs-inline.html">Managed Policies and Inline Policies</a> in the <i>IAM User Guide</i>.

        :param self:
        :param PolicyArn:  The Amazon Resource Name (ARN) of the managed policy that you want information about. For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the <i>AWS General Reference</i>.<br/>
        :return: Policy:  A structure containing details about the policy.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateDate</i> The date and time, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>, when the policy was last updated. When a policy has only one version, this field contains the date and time when the policy was created. When a policy has more than one version, this field contains the date and time when the most recent policy version was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateDate</i> The date and time, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>, when the policy was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A friendly description of the policy. This element is included in the response to the <a>GetPolicy</a> operation. It is not included in the response to the <a>ListPolicies</a> operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsAttachable</i> Specifies whether the policy can be attached to an IAM user, group, or role.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PermissionsBoundaryUsageCount</i> The number of entities (users and roles) for which the policy is used to set the permissions boundary.  For more information about permissions boundaries, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies_boundaries.html">Permissions Boundaries for IAM Identities </a> in the <i>IAM User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttachmentCount</i> The number of entities (users, groups, and roles) that the policy is attached to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultVersionId</i> The identifier for the version of the policy that is set as the default version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Path</i> The path to the policy. For more information about paths, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN). ARNs are unique identifiers for AWS resources. For more information about ARNs, go to <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the <i>AWS General Reference</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyId</i> The stable and unique string identifying the policy. For more information about IDs, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyName</i> The friendly name (not ARN) identifying the policy.<br/>        """
        pass

    def get_policy_version(self, VersionId: str, PolicyArn: str) -> GetPolicyVersionResponse:
        """Retrieves information about the specified version of the specified managed policy, including the policy document. <note> Policies returned by this API are URL-encoded compliant with <a href="https://tools.ietf.org/html/rfc3986">RFC 3986</a>. You can use a URL decoding method to convert the policy back to plain JSON text. For example, if you use Java, you can use the <code>decode</code> method of the <code>java.net.URLDecoder</code> utility class in the Java SDK. Other languages and SDKs provide similar functionality. </note> To list the available versions for a policy, use <a>ListPolicyVersions</a>. This API retrieves information about managed policies. To retrieve information about an inline policy that is embedded in a user, group, or role, use the <a>GetUserPolicy</a>, <a>GetGroupPolicy</a>, or <a>GetRolePolicy</a> API. For more information about the types of policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-vs-inline.html">Managed Policies and Inline Policies</a> in the <i>IAM User Guide</i>. For more information about managed policy versions, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-versions.html">Versioning for Managed Policies</a> in the <i>IAM User Guide</i>.

        :param self:
        :param VersionId:  Identifies the policy version to retrieve. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters that consists of the lowercase letter 'v' followed by one or two digits, and optionally followed by a period '.' and a string of letters and digits.<br/>PolicyArn:  The Amazon Resource Name (ARN) of the managed policy that you want information about. For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the <i>AWS General Reference</i>.<br/>
        :return: PolicyVersion:  A structure containing details about the policy version.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateDate</i> The date and time, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>, when the policy version was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsDefaultVersion</i> Specifies whether the policy version is set as the policy's default version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VersionId</i> The identifier for the policy version. Policy version identifiers always begin with <code>v</code> (always lowercase). When a policy is created, the first policy version is <code>v1</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Document</i> The policy document. The policy document is returned in the response to the <a>GetPolicyVersion</a> and <a>GetAccountAuthorizationDetails</a> operations. It is not returned in the response to the <a>CreatePolicyVersion</a> or <a>ListPolicyVersions</a> operations.  The policy document returned in this structure is URL-encoded compliant with <a href="https://tools.ietf.org/html/rfc3986">RFC 3986</a>. You can use a URL decoding method to convert the policy back to plain JSON text. For example, if you use Java, you can use the <code>decode</code> method of the <code>java.net.URLDecoder</code> utility class in the Java SDK. Other languages and SDKs provide similar functionality.<br/>        """
        pass

    def get_role(self, RoleName: str) -> GetRoleResponse:
        """Retrieves information about the specified role, including the role's path, GUID, ARN, and the role's trust policy that grants permission to assume the role. For more information about roles, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/WorkingWithRoles.html">Working with Roles</a>. <note> Policies returned by this API are URL-encoded compliant with <a href="https://tools.ietf.org/html/rfc3986">RFC 3986</a>. You can use a URL decoding method to convert the policy back to plain JSON text. For example, if you use Java, you can use the <code>decode</code> method of the <code>java.net.URLDecoder</code> utility class in the Java SDK. Other languages and SDKs provide similar functionality. </note>

        :param self:
        :param RoleName:  The name of the IAM role to get information about. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: Role:  A structure containing details about the IAM role.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> A list of tags that are attached to the specified role. For more information about tagging, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_tags.html">Tagging IAM Identities</a> in the <i>IAM User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PermissionsBoundary</i> The ARN of the policy used to set the permissions boundary for the role. For more information about permissions boundaries, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies_boundaries.html">Permissions Boundaries for IAM Identities </a> in the <i>IAM User Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PermissionsBoundary</b>: The ARN of the policy used to set the permissions boundary for the role. For more information about permissions boundaries, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies_boundaries.html">Permissions Boundaries for IAM Identities </a> in the <i>IAM User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxSessionDuration</i> The maximum session duration (in seconds) for the specified role. Anyone who uses the AWS CLI, or API to assume the role can specify the duration using the optional <code>DurationSeconds</code> API parameter or <code>duration-seconds</code> CLI parameter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of the role that you provide.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssumeRolePolicyDocument</i> The policy that grants an entity permission to assume the role.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateDate</i> The date and time, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>, when the role was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i>  The Amazon Resource Name (ARN) specifying the role. For more information about ARNs and how to use them in policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>IAM User Guide</i> guide. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleId</i>  The stable and unique string identifying the role. For more information about IDs, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleName</i> The friendly name that identifies the role.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Path</i>  The path to the role. For more information about paths, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PermissionsBoundaryArn</i>  The ARN of the policy used to set the permissions boundary for the user or role.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PermissionsBoundaryType</i>  The permissions boundary usage type that indicates what type of IAM resource is used as the permissions boundary for an entity. This data type can only have a value of <code>Policy</code>.<br/>        """
        pass

    def get_role_policy(self, PolicyName: str, RoleName: str) -> GetRolePolicyResponse:
        """Retrieves the specified inline policy document that is embedded with the specified IAM role. <note> Policies returned by this API are URL-encoded compliant with <a href="https://tools.ietf.org/html/rfc3986">RFC 3986</a>. You can use a URL decoding method to convert the policy back to plain JSON text. For example, if you use Java, you can use the <code>decode</code> method of the <code>java.net.URLDecoder</code> utility class in the Java SDK. Other languages and SDKs provide similar functionality. </note> An IAM role can also have managed policies attached to it. To retrieve a managed policy document that is attached to a role, use <a>GetPolicy</a> to determine the policy's default version, then use <a>GetPolicyVersion</a> to retrieve the policy document. For more information about policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-vs-inline.html">Managed Policies and Inline Policies</a> in the <i>IAM User Guide</i>. For more information about roles, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/roles-toplevel.html">Using Roles to Delegate Permissions and Federate Identities</a>.

        :param self:
        :param PolicyName:  The name of the policy document to get. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>RoleName:  The name of the role associated with the policy. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: PolicyDocument:  The policy document. IAM stores policies in JSON format. However, resources that were created using AWS CloudFormation templates can be formatted in YAML. AWS CloudFormation always converts a YAML policy to JSON format before submitting it to IAM.<br/>PolicyName:  The name of the policy.<br/>RoleName:  The role the policy is associated with.<br/>        """
        pass

    def get_saml_provider(self, SAMLProviderArn: str) -> GetSAMLProviderResponse:
        """Returns the SAML provider metadocument that was uploaded when the IAM SAML provider resource object was created or updated. <note> This operation requires <a href="https://docs.aws.amazon.com/general/latest/gr/signature-version-4.html">Signature Version 4</a>. </note>

        :param self:
        :param SAMLProviderArn:  The Amazon Resource Name (ARN) of the SAML provider resource object in IAM to get information about. For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the <i>AWS General Reference</i>.<br/>
        :return: ValidUntil:  The expiration date and time for the SAML provider.<br/>CreateDate:  The date and time when the SAML provider was created.<br/>SAMLMetadataDocument:  The XML metadata document that includes information about an identity provider.<br/>        """
        pass

    def get_ssh_public_key(self, Encoding: str, SSHPublicKeyId: str, UserName: str) -> GetSSHPublicKeyResponse:
        """Retrieves the specified SSH public key, including metadata about the key. The SSH public key retrieved by this operation is used only for authenticating the associated IAM user to an AWS CodeCommit repository. For more information about using SSH keys to authenticate to an AWS CodeCommit repository, see <a href="https://docs.aws.amazon.com/codecommit/latest/userguide/setting-up-credentials-ssh.html">Set up AWS CodeCommit for SSH Connections</a> in the <i>AWS CodeCommit User Guide</i>.

        :param self:
        :param Encoding:  Specifies the public key encoding format to use in the response. To retrieve the public key in ssh-rsa format, use <code>SSH</code>. To retrieve the public key in PEM format, use <code>PEM</code>.<br/>SSHPublicKeyId:  The unique identifier for the SSH public key. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters that can consist of any upper or lowercased letter or digit.<br/>UserName:  The name of the IAM user associated with the SSH public key. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: SSHPublicKey:  A structure containing details about the SSH public key.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UploadDate</i> The date and time, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>, when the SSH public key was uploaded.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the SSH public key. <code>Active</code> means that the key can be used for authentication with an AWS CodeCommit repository. <code>Inactive</code> means that the key cannot be used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SSHPublicKeyBody</i> The SSH public key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Fingerprint</i> The MD5 message digest of the SSH public key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SSHPublicKeyId</i> The unique identifier for the SSH public key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserName</i> The name of the IAM user associated with the SSH public key.<br/>        """
        pass

    def get_server_certificate(self, ServerCertificateName: str) -> GetServerCertificateResponse:
        """Retrieves information about the specified server certificate stored in IAM. For more information about working with server certificates, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_server-certs.html">Working with Server Certificates</a> in the <i>IAM User Guide</i>. This topic includes a list of AWS services that can use the server certificates that you manage with IAM.

        :param self:
        :param ServerCertificateName:  The name of the server certificate you want to retrieve information about. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: ServerCertificate:  A structure containing details about the server certificate.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateChain</i> The contents of the public key certificate chain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateBody</i> The contents of the public key certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerCertificateMetadata</i> The meta information of the server certificate, such as its name, path, ID, and ARN.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ServerCertificateMetadata</b>: The meta information of the server certificate, such as its name, path, ID, and ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Expiration</i> The date on which the certificate is set to expire.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UploadDate</i> The date when the server certificate was uploaded.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i>  The Amazon Resource Name (ARN) specifying the server certificate. For more information about ARNs and how to use them in policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerCertificateId</i>  The stable and unique string identifying the server certificate. For more information about IDs, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerCertificateName</i> The name that identifies the server certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Path</i>  The path to the server certificate. For more information about paths, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide. <br/>        """
        pass

    def get_service_last_accessed_details(self, JobId: str, MaxItems: int=None, Marker: str=None) -> GetServiceLastAccessedDetailsResponse:
        """Retrieves a service last accessed report that was created using the <code>GenerateServiceLastAccessedDetails</code> operation. You can use the <code>JobId</code> parameter in <code>GetServiceLastAccessedDetails</code> to retrieve the status of your report job. When the report is complete, you can retrieve the generated report. The report includes a list of AWS services that the resource (user, group, role, or managed policy) can access. <note> Service last accessed data does not use other policy types when determining whether a resource could access a service. These other policy types include resource-based policies, access control lists, AWS Organizations policies, IAM permissions boundaries, and AWS STS assume role policies. It only applies permissions policy logic. For more about the evaluation of policy types, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_policies_evaluation-logic.html#policy-eval-basics">Evaluating Policies</a> in the <i>IAM User Guide</i>. </note> For each service that the resource could access using permissions policies, the operation returns details about the most recent access attempt. If there was no attempt, the service is listed without details about the most recent attempt to access the service. If the operation fails, the <code>GetServiceLastAccessedDetails</code> operation returns the reason that it failed. The <code>GetServiceLastAccessedDetails</code> operation returns a list of services. This list includes the number of entities that have attempted to access the service and the date and time of the last attempt. It also returns the ARN of the following entity, depending on the resource ARN that you used to generate the report: <ul> <li>  <b>User</b> – Returns the user ARN that you used to generate the report </li> <li>  <b>Group</b> – Returns the ARN of the group member (user) that last attempted to access the service </li> <li>  <b>Role</b> – Returns the role ARN that you used to generate the report </li> <li>  <b>Policy</b> – Returns the ARN of the user or role that last used the policy to attempt to access the service </li> </ul> By default, the list is sorted by service namespace.

        :param self:
        :param Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>MaxItems:  Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>JobId:  The ID of the request generated by the <a>GenerateServiceLastAccessedDetails</a> operation.<br/>
        :return: Error:  An object that contains details about the reason the operation failed.:<br/>Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  <p/> A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when there are more results available. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>JobCompletionDate:  The date and time, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>, when the generated report job was completed or failed. This field is null if the job is still in progress, as indicated by a job status value of <code>IN_PROGRESS</code>.<br/>ServicesLastAccessed:   A <code>ServiceLastAccessed</code> object that contains details about the most recent attempt to access the service.<br/>JobCreationDate:  The date and time, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>, when the report job was created.<br/>JobStatus:  The status of the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Code</i> The error code associated with the operation failure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> Detailed information about the reason that the operation failed.<br/>        """
        pass

    def get_service_last_accessed_details_with_entities(self, ServiceNamespace: str, JobId: str, MaxItems: int=None, Marker: str=None) -> GetServiceLastAccessedDetailsWithEntitiesResponse:
        """After you generate a group or policy report using the <code>GenerateServiceLastAccessedDetails</code> operation, you can use the <code>JobId</code> parameter in <code>GetServiceLastAccessedDetailsWithEntities</code>. This operation retrieves the status of your report job and a list of entities that could have used group or policy permissions to access the specified service. <ul> <li>  <b>Group</b> – For a group report, this operation returns a list of users in the group that could have used the group’s policies in an attempt to access the service. </li> <li>  <b>Policy</b> – For a policy report, this operation returns a list of entities (users or roles) that could have used the policy in an attempt to access the service. </li> </ul> You can also use this operation for user or role reports to retrieve details about those entities. If the operation fails, the <code>GetServiceLastAccessedDetailsWithEntities</code> operation returns the reason that it failed. By default, the list of associated entities is sorted by date, with the most recent access listed first.

        :param self:
        :param Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>MaxItems:  Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>ServiceNamespace:  The service namespace for an AWS service. Provide the service namespace to learn when the IAM entity last attempted to access the specified service. To learn the service namespace for a service, go to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_policies_actions-resources-contextkeys.html">Actions, Resources, and Condition Keys for AWS Services</a> in the <i>IAM User Guide</i>. Choose the name of the service to view details for that service. In the first paragraph, find the service prefix. For example, <code>(service prefix: a4b)</code>. For more information about service namespaces, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#genref-aws-service-namespaces">AWS Service Namespaces</a> in the <i>AWS General Reference</i>.<br/>JobId:  The ID of the request generated by the <code>GenerateServiceLastAccessedDetails</code> operation.<br/>
        :return: Error:  An object that contains details about the reason the operation failed.:<br/>Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when there are more results available. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>EntityDetailsList:  An <code>EntityDetailsList</code> object that contains details about when an IAM entity (user or role) used group or policy permissions in an attempt to access the specified AWS service.<br/>JobCompletionDate:  The date and time, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>, when the generated report job was completed or failed. This field is null if the job is still in progress, as indicated by a job status value of <code>IN_PROGRESS</code>.<br/>JobCreationDate:  The date and time, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>, when the report job was created.<br/>JobStatus:  The status of the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Code</i> The error code associated with the operation failure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> Detailed information about the reason that the operation failed.<br/>        """
        pass

    def get_service_linked_role_deletion_status(self, DeletionTaskId: str) -> GetServiceLinkedRoleDeletionStatusResponse:
        """Retrieves the status of your service-linked role deletion. After you use the <a>DeleteServiceLinkedRole</a> API operation to submit a service-linked role for deletion, you can use the <code>DeletionTaskId</code> parameter in <code>GetServiceLinkedRoleDeletionStatus</code> to check the status of the deletion. If the deletion fails, this operation returns the reason that it failed, if that information is returned by the service.

        :param self:
        :param DeletionTaskId:  The deletion task identifier. This identifier is returned by the <a>DeleteServiceLinkedRole</a> operation in the format <code>task/aws-service-role/&lt;service-principal-name&gt;/&lt;role-name&gt;/&lt;task-uuid&gt;</code>.<br/>
        :return: Reason:  An object that contains details about the reason the deletion failed.:<br/>Status:  The status of the deletion.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleUsageList</i> A list of objects that contains details about the service-linked role deletion failure, if that information is returned by the service. If the service-linked role has active sessions or if any resources that were used by the role have not been deleted from the linked service, the role can't be deleted. This parameter includes a list of the resources that are associated with the role and the Region in which the resources are being used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Reason</i> A short description of the reason that the service-linked role deletion failed.<br/>        """
        pass

    def get_user(self, UserName: str=None) -> GetUserResponse:
        """Retrieves information about the specified IAM user, including the user's creation date, path, unique ID, and ARN. If you do not specify a user name, IAM determines the user name implicitly based on the AWS access key ID used to sign the request to this API.

        :param self:
        :param UserName:  The name of the user to get information about. This parameter is optional. If it is not included, it defaults to the user making the request. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: User:  A structure containing details about the IAM user. <important> Due to a service issue, password last used data does not include password use from May 3, 2018 22:50 PDT to May 23, 2018 14:08 PDT. This affects <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_finding-unused.html">last sign-in</a> dates shown in the IAM console and password last used dates in the <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_getting-report.html">IAM credential report</a>, and returned by this GetUser API. If users signed in during the affected time, the password last used date that is returned is the date the user last signed in before May 3, 2018. For users that signed in after May 23, 2018 14:08 PDT, the returned password last used date is accurate. You can use password last used information to identify unused credentials for deletion. For example, you might delete users who did not sign in to AWS in the last 90 days. In cases like this, we recommend that you adjust your evaluation window to include dates after May 23, 2018. Alternatively, if your users use access keys to access AWS programmatically you can refer to access key last used information because it is accurate for all dates.  </important>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> A list of tags that are associated with the specified user. For more information about tagging, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_tags.html">Tagging IAM Identities</a> in the <i>IAM User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PermissionsBoundary</i> The ARN of the policy used to set the permissions boundary for the user. For more information about permissions boundaries, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies_boundaries.html">Permissions Boundaries for IAM Identities </a> in the <i>IAM User Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PermissionsBoundary</b>: The ARN of the policy used to set the permissions boundary for the user. For more information about permissions boundaries, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies_boundaries.html">Permissions Boundaries for IAM Identities </a> in the <i>IAM User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PasswordLastUsed</i> The date and time, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>, when the user's password was last used to sign in to an AWS website. For a list of AWS websites that capture a user's last sign-in time, see the <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/credential-reports.html">Credential Reports</a> topic in the <i>Using IAM</i> guide. If a password is used more than once in a five-minute span, only the first use is returned in this field. If the field is null (no value), then it indicates that they never signed in with a password. This can be because: <ul> <li> The user never had a password. </li> <li> A password exists but has not been used since IAM started tracking this information on October 20, 2014. </li> </ul> A null value does not mean that the user <i>never</i> had a password. Also, if the user does not currently have a password, but had one in the past, then this field contains the date and time the most recent password was used. This value is returned only in the <a>GetUser</a> and <a>ListUsers</a> operations. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateDate</i> The date and time, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>, when the user was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) that identifies the user. For more information about ARNs and how to use ARNs in policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserId</i> The stable and unique string identifying the user. For more information about IDs, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserName</i> The friendly name identifying the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Path</i> The path to the user. For more information about paths, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PermissionsBoundaryArn</i>  The ARN of the policy used to set the permissions boundary for the user or role.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PermissionsBoundaryType</i>  The permissions boundary usage type that indicates what type of IAM resource is used as the permissions boundary for an entity. This data type can only have a value of <code>Policy</code>.<br/>        """
        pass

    def get_user_policy(self, PolicyName: str, UserName: str) -> GetUserPolicyResponse:
        """Retrieves the specified inline policy document that is embedded in the specified IAM user. <note> Policies returned by this API are URL-encoded compliant with <a href="https://tools.ietf.org/html/rfc3986">RFC 3986</a>. You can use a URL decoding method to convert the policy back to plain JSON text. For example, if you use Java, you can use the <code>decode</code> method of the <code>java.net.URLDecoder</code> utility class in the Java SDK. Other languages and SDKs provide similar functionality. </note> An IAM user can also have managed policies attached to it. To retrieve a managed policy document that is attached to a user, use <a>GetPolicy</a> to determine the policy's default version. Then use <a>GetPolicyVersion</a> to retrieve the policy document. For more information about policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-vs-inline.html">Managed Policies and Inline Policies</a> in the <i>IAM User Guide</i>.

        :param self:
        :param PolicyName:  The name of the policy document to get. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>UserName:  The name of the user who the policy is associated with. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: PolicyDocument:  The policy document. IAM stores policies in JSON format. However, resources that were created using AWS CloudFormation templates can be formatted in YAML. AWS CloudFormation always converts a YAML policy to JSON format before submitting it to IAM.<br/>PolicyName:  The name of the policy.<br/>UserName:  The user the policy is associated with.<br/>        """
        pass

    def list_access_keys(self, UserName: str=None, Marker: str=None, MaxItems: int=None) -> ListAccessKeysResponse:
        """Returns information about the access key IDs associated with the specified IAM user. If there is none, the operation returns an empty list. Although each user is limited to a small number of keys, you can still paginate the results using the <code>MaxItems</code> and <code>Marker</code> parameters. If the <code>UserName</code> field is not specified, the user name is determined implicitly based on the AWS access key ID used to sign the request. This operation works for access keys under the AWS account. Consequently, you can use this operation to manage AWS account root user credentials even if the AWS account has no associated users. <note> To ensure the security of your AWS account, the secret access key is accessible only during key and user creation. </note>

        :param self:
        :param MaxItems:  Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>UserName:  The name of the user. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when there are more results available. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>AccessKeyMetadata:  A list of objects containing metadata about the access keys.<br/>        """
        pass

    def list_account_aliases(self, Marker: str=None, MaxItems: int=None) -> ListAccountAliasesResponse:
        """Lists the account alias associated with the AWS account (Note: you can have only one). For information about using an AWS account alias, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/AccountAlias.html">Using an Alias for Your AWS Account ID</a> in the <i>IAM User Guide</i>.

        :param self:
        :param MaxItems:  Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>
        :return: Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when there are more results available. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>AccountAliases:  A list of aliases associated with the account. AWS supports only one alias per account.<br/>        """
        pass

    def list_attached_group_policies(self, GroupName: str, PathPrefix: str=None, Marker: str=None, MaxItems: int=None) -> ListAttachedGroupPoliciesResponse:
        """Lists all managed policies that are attached to the specified IAM group. An IAM group can also have inline policies embedded with it. To list the inline policies for a group, use the <a>ListGroupPolicies</a> API. For information about policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-vs-inline.html">Managed Policies and Inline Policies</a> in the <i>IAM User Guide</i>. You can paginate the results using the <code>MaxItems</code> and <code>Marker</code> parameters. You can use the <code>PathPrefix</code> parameter to limit the list of policies to only those matching the specified path prefix. If there are no policies attached to the specified group (or none that match the specified path prefix), the operation returns an empty list.

        :param self:
        :param MaxItems:  Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>PathPrefix:  The path prefix for filtering the results. This parameter is optional. If it is not included, it defaults to a slash (/), listing all policies. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of either a forward slash (/) by itself or a string that must begin and end with forward slashes. In addition, it can contain any ASCII character from the ! (\u0021) through the DEL character (\u007F), including most punctuation characters, digits, and upper and lowercased letters.<br/>GroupName:  The name (friendly name, not ARN) of the group to list attached policies for. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when there are more results available. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>AttachedPolicies:  A list of the attached policies.<br/>        """
        pass

    def list_attached_role_policies(self, RoleName: str, PathPrefix: str=None, Marker: str=None, MaxItems: int=None) -> ListAttachedRolePoliciesResponse:
        """Lists all managed policies that are attached to the specified IAM role. An IAM role can also have inline policies embedded with it. To list the inline policies for a role, use the <a>ListRolePolicies</a> API. For information about policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-vs-inline.html">Managed Policies and Inline Policies</a> in the <i>IAM User Guide</i>. You can paginate the results using the <code>MaxItems</code> and <code>Marker</code> parameters. You can use the <code>PathPrefix</code> parameter to limit the list of policies to only those matching the specified path prefix. If there are no policies attached to the specified role (or none that match the specified path prefix), the operation returns an empty list.

        :param self:
        :param MaxItems:  Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>PathPrefix:  The path prefix for filtering the results. This parameter is optional. If it is not included, it defaults to a slash (/), listing all policies. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of either a forward slash (/) by itself or a string that must begin and end with forward slashes. In addition, it can contain any ASCII character from the ! (\u0021) through the DEL character (\u007F), including most punctuation characters, digits, and upper and lowercased letters.<br/>RoleName:  The name (friendly name, not ARN) of the role to list attached policies for. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when there are more results available. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>AttachedPolicies:  A list of the attached policies.<br/>        """
        pass

    def list_attached_user_policies(self, UserName: str, PathPrefix: str=None, Marker: str=None, MaxItems: int=None) -> ListAttachedUserPoliciesResponse:
        """Lists all managed policies that are attached to the specified IAM user. An IAM user can also have inline policies embedded with it. To list the inline policies for a user, use the <a>ListUserPolicies</a> API. For information about policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-vs-inline.html">Managed Policies and Inline Policies</a> in the <i>IAM User Guide</i>. You can paginate the results using the <code>MaxItems</code> and <code>Marker</code> parameters. You can use the <code>PathPrefix</code> parameter to limit the list of policies to only those matching the specified path prefix. If there are no policies attached to the specified group (or none that match the specified path prefix), the operation returns an empty list.

        :param self:
        :param MaxItems:  Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>PathPrefix:  The path prefix for filtering the results. This parameter is optional. If it is not included, it defaults to a slash (/), listing all policies. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of either a forward slash (/) by itself or a string that must begin and end with forward slashes. In addition, it can contain any ASCII character from the ! (\u0021) through the DEL character (\u007F), including most punctuation characters, digits, and upper and lowercased letters.<br/>UserName:  The name (friendly name, not ARN) of the user to list attached policies for. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when there are more results available. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>AttachedPolicies:  A list of the attached policies.<br/>        """
        pass

    def list_entities_for_policy(self, PolicyArn: str, EntityFilter: str=None, PathPrefix: str=None, PolicyUsageFilter: str=None, Marker: str=None, MaxItems: int=None) -> ListEntitiesForPolicyResponse:
        """Lists all IAM users, groups, and roles that the specified managed policy is attached to. You can use the optional <code>EntityFilter</code> parameter to limit the results to a particular type of entity (users, groups, or roles). For example, to list only the roles that are attached to the specified policy, set <code>EntityFilter</code> to <code>Role</code>. You can paginate the results using the <code>MaxItems</code> and <code>Marker</code> parameters.

        :param self:
        :param MaxItems:  Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>PolicyUsageFilter:  The policy usage method to use for filtering the results. To list only permissions policies, set <code>PolicyUsageFilter</code> to <code>PermissionsPolicy</code>. To list only the policies used to set permissions boundaries, set the value to <code>PermissionsBoundary</code>. This parameter is optional. If it is not included, all policies are returned. <br/>PathPrefix:  The path prefix for filtering the results. This parameter is optional. If it is not included, it defaults to a slash (/), listing all entities. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of either a forward slash (/) by itself or a string that must begin and end with forward slashes. In addition, it can contain any ASCII character from the ! (\u0021) through the DEL character (\u007F), including most punctuation characters, digits, and upper and lowercased letters.<br/>EntityFilter:  The entity type to use for filtering the results. For example, when <code>EntityFilter</code> is <code>Role</code>, only the roles that are attached to the specified policy are returned. This parameter is optional. If it is not included, all attached entities (users, groups, and roles) are returned. The argument for this parameter must be one of the valid values listed below.<br/>PolicyArn:  The Amazon Resource Name (ARN) of the IAM policy for which you want the versions. For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the <i>AWS General Reference</i>.<br/>
        :return: Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when there are more results available. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>PolicyRoles:  A list of IAM roles that the policy is attached to.<br/>PolicyUsers:  A list of IAM users that the policy is attached to.<br/>PolicyGroups:  A list of IAM groups that the policy is attached to.<br/>        """
        pass

    def list_group_policies(self, GroupName: str, Marker: str=None, MaxItems: int=None) -> ListGroupPoliciesResponse:
        """Lists the names of the inline policies that are embedded in the specified IAM group. An IAM group can also have managed policies attached to it. To list the managed policies that are attached to a group, use <a>ListAttachedGroupPolicies</a>. For more information about policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-vs-inline.html">Managed Policies and Inline Policies</a> in the <i>IAM User Guide</i>. You can paginate the results using the <code>MaxItems</code> and <code>Marker</code> parameters. If there are no inline policies embedded with the specified group, the operation returns an empty list.

        :param self:
        :param MaxItems:  Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>GroupName:  The name of the group to list policies for. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when there are more results available. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>PolicyNames:  A list of policy names. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>        """
        pass

    def list_groups(self, PathPrefix: str=None, Marker: str=None, MaxItems: int=None) -> ListGroupsResponse:
        """Lists the IAM groups that have the specified path prefix.  You can paginate the results using the <code>MaxItems</code> and <code>Marker</code> parameters.

        :param self:
        :param MaxItems:  Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>PathPrefix:   The path prefix for filtering the results. For example, the prefix <code>/division_abc/subdivision_xyz/</code> gets all groups whose path starts with <code>/division_abc/subdivision_xyz/</code>. This parameter is optional. If it is not included, it defaults to a slash (/), listing all groups. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of either a forward slash (/) by itself or a string that must begin and end with forward slashes. In addition, it can contain any ASCII character from the ! (\u0021) through the DEL character (\u007F), including most punctuation characters, digits, and upper and lowercased letters.<br/>
        :return: Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when there are more results available. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>Groups:  A list of groups.<br/>        """
        pass

    def list_groups_for_user(self, UserName: str, Marker: str=None, MaxItems: int=None) -> ListGroupsForUserResponse:
        """Lists the IAM groups that the specified IAM user belongs to. You can paginate the results using the <code>MaxItems</code> and <code>Marker</code> parameters.

        :param self:
        :param MaxItems:  Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>UserName:  The name of the user to list groups for. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when there are more results available. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>Groups:  A list of groups.<br/>        """
        pass

    def list_instance_profiles(self, PathPrefix: str=None, Marker: str=None, MaxItems: int=None) -> ListInstanceProfilesResponse:
        """Lists the instance profiles that have the specified path prefix. If there are none, the operation returns an empty list. For more information about instance profiles, go to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/AboutInstanceProfiles.html">About Instance Profiles</a>. You can paginate the results using the <code>MaxItems</code> and <code>Marker</code> parameters.

        :param self:
        :param MaxItems:  Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>PathPrefix:   The path prefix for filtering the results. For example, the prefix <code>/application_abc/component_xyz/</code> gets all instance profiles whose path starts with <code>/application_abc/component_xyz/</code>. This parameter is optional. If it is not included, it defaults to a slash (/), listing all instance profiles. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of either a forward slash (/) by itself or a string that must begin and end with forward slashes. In addition, it can contain any ASCII character from the ! (\u0021) through the DEL character (\u007F), including most punctuation characters, digits, and upper and lowercased letters.<br/>
        :return: Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when there are more results available. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>InstanceProfiles:  A list of instance profiles.<br/>        """
        pass

    def list_instance_profiles_for_role(self, RoleName: str, Marker: str=None, MaxItems: int=None) -> ListInstanceProfilesForRoleResponse:
        """Lists the instance profiles that have the specified associated IAM role. If there are none, the operation returns an empty list. For more information about instance profiles, go to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/AboutInstanceProfiles.html">About Instance Profiles</a>. You can paginate the results using the <code>MaxItems</code> and <code>Marker</code> parameters.

        :param self:
        :param MaxItems:  Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>RoleName:  The name of the role to list instance profiles for. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when there are more results available. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>InstanceProfiles:  A list of instance profiles.<br/>        """
        pass

    def list_mfa_devices(self, UserName: str=None, Marker: str=None, MaxItems: int=None) -> ListMFADevicesResponse:
        """Lists the MFA devices for an IAM user. If the request includes a IAM user name, then this operation lists all the MFA devices associated with the specified user. If you do not specify a user name, IAM determines the user name implicitly based on the AWS access key ID signing the request for this API. You can paginate the results using the <code>MaxItems</code> and <code>Marker</code> parameters.

        :param self:
        :param MaxItems:  Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>UserName:  The name of the user whose MFA devices you want to list. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when there are more results available. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>MFADevices:  A list of MFA devices.<br/>        """
        pass

    def list_open_id_connect_providers(self) -> ListOpenIDConnectProvidersResponse:
        """Lists information about the IAM OpenID Connect (OIDC) provider resource objects defined in the AWS account.

        :param self:
        :param 
        :return: OpenIDConnectProviderList:  The list of IAM OIDC provider resource objects defined in the AWS account.<br/>        """
        pass

    def list_policies(self, Scope: str=None, OnlyAttached: booleanType=None, PathPrefix: str=None, PolicyUsageFilter: str=None, Marker: str=None, MaxItems: int=None) -> ListPoliciesResponse:
        """Lists all the managed policies that are available in your AWS account, including your own customer-defined managed policies and all AWS managed policies. You can filter the list of policies that is returned using the optional <code>OnlyAttached</code>, <code>Scope</code>, and <code>PathPrefix</code> parameters. For example, to list only the customer managed policies in your AWS account, set <code>Scope</code> to <code>Local</code>. To list only AWS managed policies, set <code>Scope</code> to <code>AWS</code>. You can paginate the results using the <code>MaxItems</code> and <code>Marker</code> parameters. For more information about managed policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-vs-inline.html">Managed Policies and Inline Policies</a> in the <i>IAM User Guide</i>.

        :param self:
        :param MaxItems:  Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>PolicyUsageFilter:  The policy usage method to use for filtering the results. To list only permissions policies, set <code>PolicyUsageFilter</code> to <code>PermissionsPolicy</code>. To list only the policies used to set permissions boundaries, set the value to <code>PermissionsBoundary</code>. This parameter is optional. If it is not included, all policies are returned. <br/>PathPrefix:  The path prefix for filtering the results. This parameter is optional. If it is not included, it defaults to a slash (/), listing all policies. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of either a forward slash (/) by itself or a string that must begin and end with forward slashes. In addition, it can contain any ASCII character from the ! (\u0021) through the DEL character (\u007F), including most punctuation characters, digits, and upper and lowercased letters.<br/>OnlyAttached:  A flag to filter the results to only the attached policies. When <code>OnlyAttached</code> is <code>true</code>, the returned list contains only the policies that are attached to an IAM user, group, or role. When <code>OnlyAttached</code> is <code>false</code>, or when the parameter is not included, all policies are returned.<br/>Scope:  The scope to use for filtering the results. To list only AWS managed policies, set <code>Scope</code> to <code>AWS</code>. To list only the customer managed policies in your AWS account, set <code>Scope</code> to <code>Local</code>. This parameter is optional. If it is not included, or if it is set to <code>All</code>, all policies are returned.<br/>
        :return: Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when there are more results available. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>Policies:  A list of policies.<br/>        """
        pass

    def list_policies_granting_service_access(self, ServiceNamespaces: List[serviceNamespaceType], Arn: str, Marker: str=None) -> ListPoliciesGrantingServiceAccessResponse:
        """Retrieves a list of policies that the IAM identity (user, group, or role) can use to access each specified service. <note> This operation does not use other policy types when determining whether a resource could access a service. These other policy types include resource-based policies, access control lists, AWS Organizations policies, IAM permissions boundaries, and AWS STS assume role policies. It only applies permissions policy logic. For more about the evaluation of policy types, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_policies_evaluation-logic.html#policy-eval-basics">Evaluating Policies</a> in the <i>IAM User Guide</i>. </note> The list of policies returned by the operation depends on the ARN of the identity that you provide. <ul> <li>  <b>User</b> – The list of policies includes the managed and inline policies that are attached to the user directly. The list also includes any additional managed and inline policies that are attached to the group to which the user belongs.  </li> <li>  <b>Group</b> – The list of policies includes only the managed and inline policies that are attached to the group directly. Policies that are attached to the group’s user are not included. </li> <li>  <b>Role</b> – The list of policies includes only the managed and inline policies that are attached to the role. </li> </ul> For each managed policy, this operation returns the ARN and policy name. For each inline policy, it returns the policy name and the entity to which it is attached. Inline policies do not have an ARN. For more information about these policy types, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies_managed-vs-inline.html">Managed Policies and Inline Policies</a> in the <i>IAM User Guide</i>. Policies that are attached to users and roles as permissions boundaries are not returned. To view which managed policy is currently used to set the permissions boundary for a user or role, use the <a>GetUser</a> or <a>GetRole</a> operations.

        :param self:
        :param ServiceNamespaces:  The service namespace for the AWS services whose policies you want to list. To learn the service namespace for a service, go to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_policies_actions-resources-contextkeys.html">Actions, Resources, and Condition Keys for AWS Services</a> in the <i>IAM User Guide</i>. Choose the name of the service to view details for that service. In the first paragraph, find the service prefix. For example, <code>(service prefix: a4b)</code>. For more information about service namespaces, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#genref-aws-service-namespaces">AWS Service Namespaces</a> in the <i>AWS General Reference</i>.<br/>Arn:  The ARN of the IAM identity (user, group, or role) whose policies you want to list.<br/>Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>
        :return: Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>PoliciesGrantingServiceAccess:  A <code>ListPoliciesGrantingServiceAccess</code> object that contains details about the permissions policies attached to the specified identity (user, group, or role).<br/>        """
        pass

    def list_policy_versions(self, PolicyArn: str, Marker: str=None, MaxItems: int=None) -> ListPolicyVersionsResponse:
        """Lists information about the versions of the specified managed policy, including the version that is currently set as the policy's default version. For more information about managed policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-vs-inline.html">Managed Policies and Inline Policies</a> in the <i>IAM User Guide</i>.

        :param self:
        :param MaxItems:  Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>PolicyArn:  The Amazon Resource Name (ARN) of the IAM policy for which you want the versions. For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the <i>AWS General Reference</i>.<br/>
        :return: Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when there are more results available. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>Versions:  A list of policy versions. For more information about managed policy versions, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-versions.html">Versioning for Managed Policies</a> in the <i>IAM User Guide</i>.<br/>        """
        pass

    def list_role_policies(self, RoleName: str, Marker: str=None, MaxItems: int=None) -> ListRolePoliciesResponse:
        """Lists the names of the inline policies that are embedded in the specified IAM role. An IAM role can also have managed policies attached to it. To list the managed policies that are attached to a role, use <a>ListAttachedRolePolicies</a>. For more information about policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-vs-inline.html">Managed Policies and Inline Policies</a> in the <i>IAM User Guide</i>. You can paginate the results using the <code>MaxItems</code> and <code>Marker</code> parameters. If there are no inline policies embedded with the specified role, the operation returns an empty list.

        :param self:
        :param MaxItems:  Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>RoleName:  The name of the role to list policies for. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when there are more results available. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>PolicyNames:  A list of policy names.<br/>        """
        pass

    def list_role_tags(self, RoleName: str, Marker: str=None, MaxItems: int=None) -> ListRoleTagsResponse:
        """Lists the tags that are attached to the specified role. The returned list of tags is sorted by tag key. For more information about tagging, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_tags.html">Tagging IAM Identities</a> in the <i>IAM User Guide</i>.

        :param self:
        :param MaxItems:  (Optional) Use this only when paginating results to indicate the maximum number of items that you want in the response. If additional items exist beyond the maximum that you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, it defaults to 100. Note that IAM might return fewer results, even when more results are available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>RoleName:  The name of the IAM role for which you want to see the list of tags. This parameter accepts (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters that consist of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can use the <code>Marker</code> request parameter to make a subsequent pagination request that retrieves more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when more results are available. Check <code>IsTruncated</code> after every call to ensure that you receive all of your results.<br/>Tags:  The list of tags currently that is attached to the role. Each tag consists of a key name and an associated value. If no tags are attached to the specified role, the response contains an empty list.<br/>        """
        pass

    def list_roles(self, PathPrefix: str=None, Marker: str=None, MaxItems: int=None) -> ListRolesResponse:
        """Lists the IAM roles that have the specified path prefix. If there are none, the operation returns an empty list. For more information about roles, go to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/WorkingWithRoles.html">Working with Roles</a>. You can paginate the results using the <code>MaxItems</code> and <code>Marker</code> parameters.

        :param self:
        :param MaxItems:  Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>PathPrefix:   The path prefix for filtering the results. For example, the prefix <code>/application_abc/component_xyz/</code> gets all roles whose path starts with <code>/application_abc/component_xyz/</code>. This parameter is optional. If it is not included, it defaults to a slash (/), listing all roles. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of either a forward slash (/) by itself or a string that must begin and end with forward slashes. In addition, it can contain any ASCII character from the ! (\u0021) through the DEL character (\u007F), including most punctuation characters, digits, and upper and lowercased letters.<br/>
        :return: Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when there are more results available. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>Roles:  A list of roles.<br/>        """
        pass

    def list_saml_providers(self) -> ListSAMLProvidersResponse:
        """Lists the SAML provider resource objects defined in IAM in the account. <note>  This operation requires <a href="https://docs.aws.amazon.com/general/latest/gr/signature-version-4.html">Signature Version 4</a>. </note>

        :param self:
        :param 
        :return: SAMLProviderList:  The list of SAML provider resource objects defined in IAM for this AWS account.<br/>        """
        pass

    def list_ssh_public_keys(self, UserName: str=None, Marker: str=None, MaxItems: int=None) -> ListSSHPublicKeysResponse:
        """Returns information about the SSH public keys associated with the specified IAM user. If none exists, the operation returns an empty list. The SSH public keys returned by this operation are used only for authenticating the IAM user to an AWS CodeCommit repository. For more information about using SSH keys to authenticate to an AWS CodeCommit repository, see <a href="https://docs.aws.amazon.com/codecommit/latest/userguide/setting-up-credentials-ssh.html">Set up AWS CodeCommit for SSH Connections</a> in the <i>AWS CodeCommit User Guide</i>. Although each user is limited to a small number of keys, you can still paginate the results using the <code>MaxItems</code> and <code>Marker</code> parameters.

        :param self:
        :param MaxItems:  Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>UserName:  The name of the IAM user to list SSH public keys for. If none is specified, the <code>UserName</code> field is determined implicitly based on the AWS access key used to sign the request. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when there are more results available. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>SSHPublicKeys:  A list of the SSH public keys assigned to IAM user.<br/>        """
        pass

    def list_server_certificates(self, PathPrefix: str=None, Marker: str=None, MaxItems: int=None) -> ListServerCertificatesResponse:
        """Lists the server certificates stored in IAM that have the specified path prefix. If none exist, the operation returns an empty list.  You can paginate the results using the <code>MaxItems</code> and <code>Marker</code> parameters. For more information about working with server certificates, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_server-certs.html">Working with Server Certificates</a> in the <i>IAM User Guide</i>. This topic also includes a list of AWS services that can use the server certificates that you manage with IAM.

        :param self:
        :param MaxItems:  Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>PathPrefix:   The path prefix for filtering the results. For example: <code>/company/servercerts</code> would get all server certificates for which the path starts with <code>/company/servercerts</code>. This parameter is optional. If it is not included, it defaults to a slash (/), listing all server certificates. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of either a forward slash (/) by itself or a string that must begin and end with forward slashes. In addition, it can contain any ASCII character from the ! (\u0021) through the DEL character (\u007F), including most punctuation characters, digits, and upper and lowercased letters.<br/>
        :return: Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when there are more results available. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>ServerCertificateMetadataList:  A list of server certificates.<br/>        """
        pass

    def list_service_specific_credentials(self, UserName: str=None, ServiceName: str=None) -> ListServiceSpecificCredentialsResponse:
        """Returns information about the service-specific credentials associated with the specified IAM user. If none exists, the operation returns an empty list. The service-specific credentials returned by this operation are used only for authenticating the IAM user to a specific service. For more information about using service-specific credentials to authenticate to an AWS service, see <a href="https://docs.aws.amazon.com/codecommit/latest/userguide/setting-up-gc.html">Set Up service-specific credentials</a> in the AWS CodeCommit User Guide.

        :param self:
        :param ServiceName:  Filters the returned results to only those for the specified AWS service. If not specified, then AWS returns service-specific credentials for all services.<br/>UserName:  The name of the user whose service-specific credentials you want information about. If this value is not specified, then the operation assumes the user whose credentials are used to call the operation. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: ServiceSpecificCredentials:  A list of structures that each contain details about a service-specific credential.<br/>        """
        pass

    def list_signing_certificates(self, UserName: str=None, Marker: str=None, MaxItems: int=None) -> ListSigningCertificatesResponse:
        """Returns information about the signing certificates associated with the specified IAM user. If none exists, the operation returns an empty list. Although each user is limited to a small number of signing certificates, you can still paginate the results using the <code>MaxItems</code> and <code>Marker</code> parameters. If the <code>UserName</code> field is not specified, the user name is determined implicitly based on the AWS access key ID used to sign the request for this API. This operation works for access keys under the AWS account. Consequently, you can use this operation to manage AWS account root user credentials even if the AWS account has no associated users.

        :param self:
        :param MaxItems:  Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>UserName:  The name of the IAM user whose signing certificates you want to examine. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when there are more results available. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>Certificates:  A list of the user's signing certificate information.<br/>        """
        pass

    def list_user_policies(self, UserName: str, Marker: str=None, MaxItems: int=None) -> ListUserPoliciesResponse:
        """Lists the names of the inline policies embedded in the specified IAM user. An IAM user can also have managed policies attached to it. To list the managed policies that are attached to a user, use <a>ListAttachedUserPolicies</a>. For more information about policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-vs-inline.html">Managed Policies and Inline Policies</a> in the <i>IAM User Guide</i>. You can paginate the results using the <code>MaxItems</code> and <code>Marker</code> parameters. If there are no inline policies embedded with the specified user, the operation returns an empty list.

        :param self:
        :param MaxItems:  Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>UserName:  The name of the user to list policies for. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when there are more results available. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>PolicyNames:  A list of policy names.<br/>        """
        pass

    def list_user_tags(self, UserName: str, Marker: str=None, MaxItems: int=None) -> ListUserTagsResponse:
        """Lists the tags that are attached to the specified user. The returned list of tags is sorted by tag key. For more information about tagging, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_tags.html">Tagging IAM Identities</a> in the <i>IAM User Guide</i>.

        :param self:
        :param MaxItems:  (Optional) Use this only when paginating results to indicate the maximum number of items that you want in the response. If additional items exist beyond the maximum that you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, it defaults to 100. Note that IAM might return fewer results, even when more results are available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>UserName:  The name of the IAM user whose tags you want to see. This parameter accepts (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters that consist of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: =,.@-<br/>
        :return: Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can use the <code>Marker</code> request parameter to make a subsequent pagination request that retrieves more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when more results are available. Check <code>IsTruncated</code> after every call to ensure that you receive all of your results.<br/>Tags:  The list of tags that are currently attached to the user. Each tag consists of a key name and an associated value. If no tags are attached to the specified user, the response contains an empty list.<br/>        """
        pass

    def list_users(self, PathPrefix: str=None, Marker: str=None, MaxItems: int=None) -> ListUsersResponse:
        """Lists the IAM users that have the specified path prefix. If no path prefix is specified, the operation returns all users in the AWS account. If there are none, the operation returns an empty list. You can paginate the results using the <code>MaxItems</code> and <code>Marker</code> parameters.

        :param self:
        :param MaxItems:  Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>PathPrefix:   The path prefix for filtering the results. For example: <code>/division_abc/subdivision_xyz/</code>, which would get all user names whose path starts with <code>/division_abc/subdivision_xyz/</code>. This parameter is optional. If it is not included, it defaults to a slash (/), listing all user names. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of either a forward slash (/) by itself or a string that must begin and end with forward slashes. In addition, it can contain any ASCII character from the ! (\u0021) through the DEL character (\u007F), including most punctuation characters, digits, and upper and lowercased letters.<br/>
        :return: Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when there are more results available. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>Users:  A list of users.<br/>        """
        pass

    def list_virtual_mfa_devices(self, AssignmentStatus: str=None, Marker: str=None, MaxItems: int=None) -> ListVirtualMFADevicesResponse:
        """Lists the virtual MFA devices defined in the AWS account by assignment status. If you do not specify an assignment status, the operation returns a list of all virtual MFA devices. Assignment status can be <code>Assigned</code>, <code>Unassigned</code>, or <code>Any</code>. You can paginate the results using the <code>MaxItems</code> and <code>Marker</code> parameters.

        :param self:
        :param MaxItems:  Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>AssignmentStatus:   The status (<code>Unassigned</code> or <code>Assigned</code>) of the devices to list. If you do not specify an <code>AssignmentStatus</code>, the operation defaults to <code>Any</code>, which lists both assigned and unassigned virtual MFA devices.,<br/>
        :return: Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when there are more results available. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>VirtualMFADevices:   The list of virtual MFA devices in the current account that match the <code>AssignmentStatus</code> value that was passed in the request.<br/>        """
        pass

    def put_group_policy(self, PolicyDocument: str, PolicyName: str, GroupName: str):
        """Adds or updates an inline policy document that is embedded in the specified IAM group. A user can also have managed policies attached to it. To attach a managed policy to a group, use <a>AttachGroupPolicy</a>. To create a new managed policy, use <a>CreatePolicy</a>. For information about policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-vs-inline.html">Managed Policies and Inline Policies</a> in the <i>IAM User Guide</i>. For information about limits on the number of inline policies that you can embed in a group, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/LimitationsOnEntities.html">Limitations on IAM Entities</a> in the <i>IAM User Guide</i>. <note> Because policy documents can be large, you should use POST rather than GET when calling <code>PutGroupPolicy</code>. For general information about using the Query API with IAM, go to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/IAM_UsingQueryAPI.html">Making Query Requests</a> in the <i>IAM User Guide</i>. </note>

        :param self:
        :param PolicyDocument:  The policy document. You must provide policies in JSON format in IAM. However, for AWS CloudFormation templates formatted in YAML, you can provide the policy in JSON or YAML format. AWS CloudFormation always converts a YAML policy to JSON format before submitting it to IAM. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> used to validate this parameter is a string of characters consisting of the following: <ul> <li> Any printable ASCII character ranging from the space character (\u0020) through the end of the ASCII character range </li> <li> The printable characters in the Basic Latin and Latin-1 Supplement character set (through \u00FF) </li> <li> The special characters tab (\u0009), line feed (\u000A), and carriage return (\u000D) </li> </ul><br/>PolicyName:  The name of the policy document. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>GroupName:  The name of the group to associate the policy with. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-.<br/>
        :return:         """
        pass

    def put_role_permissions_boundary(self, PermissionsBoundary: str, RoleName: str):
        """Adds or updates the policy that is specified as the IAM role's permissions boundary. You can use an AWS managed policy or a customer managed policy to set the boundary for a role. Use the boundary to control the maximum permissions that the role can have. Setting a permissions boundary is an advanced feature that can affect the permissions for the role. You cannot set the boundary for a service-linked role.  <important> Policies used as permissions boundaries do not provide permissions. You must also attach a permissions policy to the role. To learn how the effective permissions for a role are evaluated, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_policies_evaluation-logic.html">IAM JSON Policy Evaluation Logic</a> in the IAM User Guide.  </important>

        :param self:
        :param PermissionsBoundary:  The ARN of the policy that is used to set the permissions boundary for the role.<br/>RoleName:  The name (friendly name, not ARN) of the IAM role for which you want to set the permissions boundary.<br/>
        :return:         """
        pass

    def put_role_policy(self, PolicyDocument: str, PolicyName: str, RoleName: str):
        """Adds or updates an inline policy document that is embedded in the specified IAM role. When you embed an inline policy in a role, the inline policy is used as part of the role's access (permissions) policy. The role's trust policy is created at the same time as the role, using <a>CreateRole</a>. You can update a role's trust policy using <a>UpdateAssumeRolePolicy</a>. For more information about IAM roles, go to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/roles-toplevel.html">Using Roles to Delegate Permissions and Federate Identities</a>. A role can also have a managed policy attached to it. To attach a managed policy to a role, use <a>AttachRolePolicy</a>. To create a new managed policy, use <a>CreatePolicy</a>. For information about policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-vs-inline.html">Managed Policies and Inline Policies</a> in the <i>IAM User Guide</i>. For information about limits on the number of inline policies that you can embed with a role, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/LimitationsOnEntities.html">Limitations on IAM Entities</a> in the <i>IAM User Guide</i>. <note> Because policy documents can be large, you should use POST rather than GET when calling <code>PutRolePolicy</code>. For general information about using the Query API with IAM, go to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/IAM_UsingQueryAPI.html">Making Query Requests</a> in the <i>IAM User Guide</i>. </note>

        :param self:
        :param PolicyDocument:  The policy document. You must provide policies in JSON format in IAM. However, for AWS CloudFormation templates formatted in YAML, you can provide the policy in JSON or YAML format. AWS CloudFormation always converts a YAML policy to JSON format before submitting it to IAM. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> used to validate this parameter is a string of characters consisting of the following: <ul> <li> Any printable ASCII character ranging from the space character (\u0020) through the end of the ASCII character range </li> <li> The printable characters in the Basic Latin and Latin-1 Supplement character set (through \u00FF) </li> <li> The special characters tab (\u0009), line feed (\u000A), and carriage return (\u000D) </li> </ul><br/>PolicyName:  The name of the policy document. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>RoleName:  The name of the role to associate the policy with. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def put_user_permissions_boundary(self, PermissionsBoundary: str, UserName: str):
        """Adds or updates the policy that is specified as the IAM user's permissions boundary. You can use an AWS managed policy or a customer managed policy to set the boundary for a user. Use the boundary to control the maximum permissions that the user can have. Setting a permissions boundary is an advanced feature that can affect the permissions for the user. <important> Policies that are used as permissions boundaries do not provide permissions. You must also attach a permissions policy to the user. To learn how the effective permissions for a user are evaluated, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_policies_evaluation-logic.html">IAM JSON Policy Evaluation Logic</a> in the IAM User Guide.  </important>

        :param self:
        :param PermissionsBoundary:  The ARN of the policy that is used to set the permissions boundary for the user.<br/>UserName:  The name (friendly name, not ARN) of the IAM user for which you want to set the permissions boundary.<br/>
        :return:         """
        pass

    def put_user_policy(self, PolicyDocument: str, PolicyName: str, UserName: str):
        """Adds or updates an inline policy document that is embedded in the specified IAM user. An IAM user can also have a managed policy attached to it. To attach a managed policy to a user, use <a>AttachUserPolicy</a>. To create a new managed policy, use <a>CreatePolicy</a>. For information about policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-vs-inline.html">Managed Policies and Inline Policies</a> in the <i>IAM User Guide</i>. For information about limits on the number of inline policies that you can embed in a user, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/LimitationsOnEntities.html">Limitations on IAM Entities</a> in the <i>IAM User Guide</i>. <note> Because policy documents can be large, you should use POST rather than GET when calling <code>PutUserPolicy</code>. For general information about using the Query API with IAM, go to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/IAM_UsingQueryAPI.html">Making Query Requests</a> in the <i>IAM User Guide</i>. </note>

        :param self:
        :param PolicyDocument:  The policy document. You must provide policies in JSON format in IAM. However, for AWS CloudFormation templates formatted in YAML, you can provide the policy in JSON or YAML format. AWS CloudFormation always converts a YAML policy to JSON format before submitting it to IAM. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> used to validate this parameter is a string of characters consisting of the following: <ul> <li> Any printable ASCII character ranging from the space character (\u0020) through the end of the ASCII character range </li> <li> The printable characters in the Basic Latin and Latin-1 Supplement character set (through \u00FF) </li> <li> The special characters tab (\u0009), line feed (\u000A), and carriage return (\u000D) </li> </ul><br/>PolicyName:  The name of the policy document. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>UserName:  The name of the user to associate the policy with. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def remove_client_id_from_open_id_connect_provider(self, ClientID: str, OpenIDConnectProviderArn: str):
        """Removes the specified client ID (also known as audience) from the list of client IDs registered for the specified IAM OpenID Connect (OIDC) provider resource object. This operation is idempotent; it does not fail or return an error if you try to remove a client ID that does not exist.

        :param self:
        :param ClientID:  The client ID (also known as audience) to remove from the IAM OIDC provider resource. For more information about client IDs, see <a>CreateOpenIDConnectProvider</a>.<br/>OpenIDConnectProviderArn:  The Amazon Resource Name (ARN) of the IAM OIDC provider resource to remove the client ID from. You can get a list of OIDC provider ARNs by using the <a>ListOpenIDConnectProviders</a> operation. For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the <i>AWS General Reference</i>.<br/>
        :return:         """
        pass

    def remove_role_from_instance_profile(self, RoleName: str, InstanceProfileName: str):
        """Removes the specified IAM role from the specified EC2 instance profile. <important> Make sure that you do not have any Amazon EC2 instances running with the role you are about to remove from the instance profile. Removing a role from an instance profile that is associated with a running instance might break any applications running on the instance. </important>  For more information about IAM roles, go to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/WorkingWithRoles.html">Working with Roles</a>. For more information about instance profiles, go to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/AboutInstanceProfiles.html">About Instance Profiles</a>.

        :param self:
        :param RoleName:  The name of the role to remove. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>InstanceProfileName:  The name of the instance profile to update. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def remove_user_from_group(self, UserName: str, GroupName: str):
        """Removes the specified user from the specified group.

        :param self:
        :param UserName:  The name of the user to remove. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>GroupName:  The name of the group to update. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def reset_service_specific_credential(self, ServiceSpecificCredentialId: str, UserName: str=None) -> ResetServiceSpecificCredentialResponse:
        """Resets the password for a service-specific credential. The new password is AWS generated and cryptographically strong. It cannot be configured by the user. Resetting the password immediately invalidates the previous password associated with this user.

        :param self:
        :param ServiceSpecificCredentialId:  The unique identifier of the service-specific credential. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters that can consist of any upper or lowercased letter or digit.<br/>UserName:  The name of the IAM user associated with the service-specific credential. If this value is not specified, then the operation assumes the user whose credentials are used to call the operation. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: ServiceSpecificCredential:  A structure with details about the updated service-specific credential, including the new password. <important> This is the <b>only</b> time that you can access the password. You cannot recover the password later, but you can reset it again. </important>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the service-specific credential. <code>Active</code> means that the key is valid for API calls, while <code>Inactive</code> means it is not.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserName</i> The name of the IAM user associated with the service-specific credential.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceSpecificCredentialId</i> The unique identifier for the service-specific credential.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServicePassword</i> The generated password for the service-specific credential.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceUserName</i> The generated user name for the service-specific credential. This value is generated by combining the IAM user's name combined with the ID number of the AWS account, as in <code>jane-at-123456789012</code>, for example. This value cannot be configured by the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceName</i> The name of the service associated with the service-specific credential.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateDate</i> The date and time, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>, when the service-specific credential were created.<br/>        """
        pass

    def resync_mfa_device(self, AuthenticationCode2: str, AuthenticationCode1: str, SerialNumber: str, UserName: str):
        """Synchronizes the specified MFA device with its IAM resource object on the AWS servers. For more information about creating and working with virtual MFA devices, go to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_VirtualMFA.html">Using a Virtual MFA Device</a> in the <i>IAM User Guide</i>.

        :param self:
        :param AuthenticationCode2:  A subsequent authentication code emitted by the device. The format for this parameter is a sequence of six digits.<br/>AuthenticationCode1:  An authentication code emitted by the device. The format for this parameter is a sequence of six digits.<br/>SerialNumber:  Serial number that uniquely identifies the MFA device. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>UserName:  The name of the user whose MFA device you want to resynchronize. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def set_default_policy_version(self, VersionId: str, PolicyArn: str):
        """Sets the specified version of the specified policy as the policy's default (operative) version. This operation affects all users, groups, and roles that the policy is attached to. To list the users, groups, and roles that the policy is attached to, use the <a>ListEntitiesForPolicy</a> API. For information about managed policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-vs-inline.html">Managed Policies and Inline Policies</a> in the <i>IAM User Guide</i>.

        :param self:
        :param VersionId:  The version of the policy to set as the default (operative) version. For more information about managed policy versions, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies-managed-versions.html">Versioning for Managed Policies</a> in the <i>IAM User Guide</i>.<br/>PolicyArn:  The Amazon Resource Name (ARN) of the IAM policy whose default version you want to set. For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the <i>AWS General Reference</i>.<br/>
        :return:         """
        pass

    def set_security_token_service_preferences(self, GlobalEndpointTokenVersion: str):
        """Sets the specified version of the global endpoint token as the token version used for the AWS account. By default, AWS Security Token Service (STS) is available as a global service, and all STS requests go to a single endpoint at <code>https://sts.amazonaws.com</code>. AWS recommends using Regional STS endpoints to reduce latency, build in redundancy, and increase session token availability. For information about Regional endpoints for STS, see <a href="https://docs.aws.amazon.com/general/latest/gr/rande.html#sts_region">AWS Regions and Endpoints</a> in the <i>AWS General Reference</i>. If you make an STS call to the global endpoint, the resulting session tokens might be valid in some Regions but not others. It depends on the version that is set in this operation. Version 1 tokens are valid only in AWS Regions that are available by default. These tokens do not work in manually enabled Regions, such as Asia Pacific (Hong Kong). Version 2 tokens are valid in all Regions. However, version 2 tokens are longer and might affect systems where you temporarily store tokens. For information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_temp_enable-regions.html">Activating and Deactivating STS in an AWS Region</a> in the <i>IAM User Guide</i>. To view the current session token version, see the <code>GlobalEndpointTokenVersion</code> entry in the response of the <a>GetAccountSummary</a> operation.

        :param self:
        :param GlobalEndpointTokenVersion:  The version of the global endpoint token. Version 1 tokens are valid only in AWS Regions that are available by default. These tokens do not work in manually enabled Regions, such as Asia Pacific (Hong Kong). Version 2 tokens are valid in all Regions. However, version 2 tokens are longer and might affect systems where you temporarily store tokens. For information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_temp_enable-regions.html">Activating and Deactivating STS in an AWS Region</a> in the <i>IAM User Guide</i>.<br/>
        :return:         """
        pass

    def simulate_custom_policy(self, ActionNames: List[ActionNameType], PolicyInputList: List[policyDocumentType], ResourceArns: List[ResourceNameType]=None, ResourcePolicy: str=None, ResourceOwner: str=None, CallerArn: str=None, ContextEntries: List[ContextEntry]=None, ResourceHandlingOption: str=None, MaxItems: int=None, Marker: str=None) -> SimulatePolicyResponse:
        """Simulate how a set of IAM policies and optionally a resource-based policy works with a list of API operations and AWS resources to determine the policies' effective permissions. The policies are provided as strings. The simulation does not perform the API operations; it only checks the authorization to determine if the simulated policies allow or deny the operations. If you want to simulate existing policies attached to an IAM user, group, or role, use <a>SimulatePrincipalPolicy</a> instead. Context keys are variables maintained by AWS and its services that provide details about the context of an API query request. You can use the <code>Condition</code> element of an IAM policy to evaluate context keys. To get the list of context keys that the policies require for correct simulation, use <a>GetContextKeysForCustomPolicy</a>. If the output is long, you can use <code>MaxItems</code> and <code>Marker</code> parameters to paginate the results.

        :param self:
        :param Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>MaxItems:  Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>ResourceHandlingOption:  Specifies the type of simulation to run. Different API operations that support resource-based policies require different combinations of resources. By specifying the type of simulation to run, you enable the policy simulator to enforce the presence of the required resources to ensure reliable simulation results. If your simulation does not match one of the following scenarios, then you can omit this parameter. The following list shows each of the supported scenario values and the resources that you must define to run the simulation. Each of the EC2 scenarios requires that you specify instance, image, and security-group resources. If your scenario includes an EBS volume, then you must specify that volume as a resource. If the EC2 scenario includes VPC, then you must supply the network-interface resource. If it includes an IP subnet, then you must specify the subnet resource. For more information on the EC2 scenario options, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-supported-platforms.html">Supported Platforms</a> in the <i>Amazon EC2 User Guide</i>. <ul> <li>  <b>EC2-Classic-InstanceStore</b>  instance, image, security-group </li> <li>  <b>EC2-Classic-EBS</b>  instance, image, security-group, volume </li> <li>  <b>EC2-VPC-InstanceStore</b>  instance, image, security-group, network-interface </li> <li>  <b>EC2-VPC-InstanceStore-Subnet</b>  instance, image, security-group, network-interface, subnet </li> <li>  <b>EC2-VPC-EBS</b>  instance, image, security-group, network-interface, volume </li> <li>  <b>EC2-VPC-EBS-Subnet</b>  instance, image, security-group, network-interface, subnet, volume </li> </ul><br/>ContextEntries:  A list of context keys and corresponding values for the simulation to use. Whenever a context key is evaluated in one of the simulated IAM permissions policies, the corresponding value is supplied.<br/>CallerArn:  The ARN of the IAM user that you want to use as the simulated caller of the API operations. <code>CallerArn</code> is required if you include a <code>ResourcePolicy</code> so that the policy's <code>Principal</code> element has a value to use in evaluating the policy. You can specify only the ARN of an IAM user. You cannot specify the ARN of an assumed role, federated user, or a service principal.<br/>ResourceOwner:  An ARN representing the AWS account ID that specifies the owner of any simulated resource that does not identify its owner in the resource ARN. Examples of resource ARNs include an S3 bucket or object. If <code>ResourceOwner</code> is specified, it is also used as the account owner of any <code>ResourcePolicy</code> included in the simulation. If the <code>ResourceOwner</code> parameter is not specified, then the owner of the resources and the resource policy defaults to the account of the identity provided in <code>CallerArn</code>. This parameter is required only if you specify a resource-based policy and account that owns the resource is different from the account that owns the simulated calling user <code>CallerArn</code>. The ARN for an account uses the following syntax: <code>arn:aws:iam::<i>AWS-account-ID</i>:root</code>. For example, to represent the account with the 112233445566 ID, use the following ARN: <code>arn:aws:iam::112233445566-ID:root</code>. <br/>ResourcePolicy:  A resource-based policy to include in the simulation provided as a string. Each resource in the simulation is treated as if it had this policy attached. You can include only one resource-based policy in a simulation. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> used to validate this parameter is a string of characters consisting of the following: <ul> <li> Any printable ASCII character ranging from the space character (\u0020) through the end of the ASCII character range </li> <li> The printable characters in the Basic Latin and Latin-1 Supplement character set (through \u00FF) </li> <li> The special characters tab (\u0009), line feed (\u000A), and carriage return (\u000D) </li> </ul><br/>ResourceArns:  A list of ARNs of AWS resources to include in the simulation. If this parameter is not provided, then the value defaults to <code>*</code> (all resources). Each API in the <code>ActionNames</code> parameter is evaluated for each resource in this list. The simulation determines the access result (allowed or denied) of each combination and reports it in the response. The simulation does not automatically retrieve policies for the specified resources. If you want to include a resource policy in the simulation, then you must include the policy as a string in the <code>ResourcePolicy</code> parameter. If you include a <code>ResourcePolicy</code>, then it must be applicable to all of the resources included in the simulation or you receive an invalid input error. For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the <i>AWS General Reference</i>.<br/>ActionNames:  A list of names of API operations to evaluate in the simulation. Each operation is evaluated against each resource. Each operation must include the service identifier, such as <code>iam:CreateUser</code>. This operation does not support using wildcards (*) in an action name.<br/>PolicyInputList:  A list of policy documents to include in the simulation. Each document is specified as a string containing the complete, valid JSON text of an IAM policy. Do not include any resource-based policies in this parameter. Any resource-based policy must be submitted with the <code>ResourcePolicy</code> parameter. The policies cannot be "scope-down" policies, such as you could include in a call to <a href="https://docs.aws.amazon.com/IAM/latest/APIReference/API_GetFederationToken.html">GetFederationToken</a> or one of the <a href="https://docs.aws.amazon.com/IAM/latest/APIReference/API_AssumeRole.html">AssumeRole</a> API operations. In other words, do not use policies designed to restrict what a user can do while using the temporary credentials. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> used to validate this parameter is a string of characters consisting of the following: <ul> <li> Any printable ASCII character ranging from the space character (\u0020) through the end of the ASCII character range </li> <li> The printable characters in the Basic Latin and Latin-1 Supplement character set (through \u00FF) </li> <li> The special characters tab (\u0009), line feed (\u000A), and carriage return (\u000D) </li> </ul><br/>
        :return: Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when there are more results available. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>EvaluationResults:  The results of the simulation.<br/>        """
        pass

    def simulate_principal_policy(self, ActionNames: List[ActionNameType], PolicySourceArn: str, PolicyInputList: List[policyDocumentType]=None, ResourceArns: List[ResourceNameType]=None, ResourcePolicy: str=None, ResourceOwner: str=None, CallerArn: str=None, ContextEntries: List[ContextEntry]=None, ResourceHandlingOption: str=None, MaxItems: int=None, Marker: str=None) -> SimulatePolicyResponse:
        """Simulate how a set of IAM policies attached to an IAM entity works with a list of API operations and AWS resources to determine the policies' effective permissions. The entity can be an IAM user, group, or role. If you specify a user, then the simulation also includes all of the policies that are attached to groups that the user belongs to. You can optionally include a list of one or more additional policies specified as strings to include in the simulation. If you want to simulate only policies specified as strings, use <a>SimulateCustomPolicy</a> instead. You can also optionally include one resource-based policy to be evaluated with each of the resources included in the simulation. The simulation does not perform the API operations; it only checks the authorization to determine if the simulated policies allow or deny the operations.  <b>Note:</b> This API discloses information about the permissions granted to other users. If you do not want users to see other user's permissions, then consider allowing them to use <a>SimulateCustomPolicy</a> instead. Context keys are variables maintained by AWS and its services that provide details about the context of an API query request. You can use the <code>Condition</code> element of an IAM policy to evaluate context keys. To get the list of context keys that the policies require for correct simulation, use <a>GetContextKeysForPrincipalPolicy</a>. If the output is long, you can use the <code>MaxItems</code> and <code>Marker</code> parameters to paginate the results.

        :param self:
        :param Marker:  Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the <code>Marker</code> element in the response that you received to indicate where the next call should start.<br/>MaxItems:  Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the <code>IsTruncated</code> response element is <code>true</code>. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the <code>IsTruncated</code> response element returns <code>true</code>, and <code>Marker</code> contains a value to include in the subsequent call that tells the service where to continue from.<br/>ResourceHandlingOption:  Specifies the type of simulation to run. Different API operations that support resource-based policies require different combinations of resources. By specifying the type of simulation to run, you enable the policy simulator to enforce the presence of the required resources to ensure reliable simulation results. If your simulation does not match one of the following scenarios, then you can omit this parameter. The following list shows each of the supported scenario values and the resources that you must define to run the simulation. Each of the EC2 scenarios requires that you specify instance, image, and security group resources. If your scenario includes an EBS volume, then you must specify that volume as a resource. If the EC2 scenario includes VPC, then you must supply the network interface resource. If it includes an IP subnet, then you must specify the subnet resource. For more information on the EC2 scenario options, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-supported-platforms.html">Supported Platforms</a> in the <i>Amazon EC2 User Guide</i>. <ul> <li>  <b>EC2-Classic-InstanceStore</b>  instance, image, security group </li> <li>  <b>EC2-Classic-EBS</b>  instance, image, security group, volume </li> <li>  <b>EC2-VPC-InstanceStore</b>  instance, image, security group, network interface </li> <li>  <b>EC2-VPC-InstanceStore-Subnet</b>  instance, image, security group, network interface, subnet </li> <li>  <b>EC2-VPC-EBS</b>  instance, image, security group, network interface, volume </li> <li>  <b>EC2-VPC-EBS-Subnet</b>  instance, image, security group, network interface, subnet, volume </li> </ul><br/>ContextEntries:  A list of context keys and corresponding values for the simulation to use. Whenever a context key is evaluated in one of the simulated IAM permissions policies, the corresponding value is supplied.<br/>CallerArn:  The ARN of the IAM user that you want to specify as the simulated caller of the API operations. If you do not specify a <code>CallerArn</code>, it defaults to the ARN of the user that you specify in <code>PolicySourceArn</code>, if you specified a user. If you include both a <code>PolicySourceArn</code> (for example, <code>arn:aws:iam::123456789012:user/David</code>) and a <code>CallerArn</code> (for example, <code>arn:aws:iam::123456789012:user/Bob</code>), the result is that you simulate calling the API operations as Bob, as if Bob had David's policies. You can specify only the ARN of an IAM user. You cannot specify the ARN of an assumed role, federated user, or a service principal.  <code>CallerArn</code> is required if you include a <code>ResourcePolicy</code> and the <code>PolicySourceArn</code> is not the ARN for an IAM user. This is required so that the resource-based policy's <code>Principal</code> element has a value to use in evaluating the policy. For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the <i>AWS General Reference</i>.<br/>ResourceOwner:  An AWS account ID that specifies the owner of any simulated resource that does not identify its owner in the resource ARN. Examples of resource ARNs include an S3 bucket or object. If <code>ResourceOwner</code> is specified, it is also used as the account owner of any <code>ResourcePolicy</code> included in the simulation. If the <code>ResourceOwner</code> parameter is not specified, then the owner of the resources and the resource policy defaults to the account of the identity provided in <code>CallerArn</code>. This parameter is required only if you specify a resource-based policy and account that owns the resource is different from the account that owns the simulated calling user <code>CallerArn</code>.<br/>ResourcePolicy:  A resource-based policy to include in the simulation provided as a string. Each resource in the simulation is treated as if it had this policy attached. You can include only one resource-based policy in a simulation. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> used to validate this parameter is a string of characters consisting of the following: <ul> <li> Any printable ASCII character ranging from the space character (\u0020) through the end of the ASCII character range </li> <li> The printable characters in the Basic Latin and Latin-1 Supplement character set (through \u00FF) </li> <li> The special characters tab (\u0009), line feed (\u000A), and carriage return (\u000D) </li> </ul><br/>ResourceArns:  A list of ARNs of AWS resources to include in the simulation. If this parameter is not provided, then the value defaults to <code>*</code> (all resources). Each API in the <code>ActionNames</code> parameter is evaluated for each resource in this list. The simulation determines the access result (allowed or denied) of each combination and reports it in the response. The simulation does not automatically retrieve policies for the specified resources. If you want to include a resource policy in the simulation, then you must include the policy as a string in the <code>ResourcePolicy</code> parameter. For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the <i>AWS General Reference</i>.<br/>ActionNames:  A list of names of API operations to evaluate in the simulation. Each operation is evaluated for each resource. Each operation must include the service identifier, such as <code>iam:CreateUser</code>.<br/>PolicyInputList:  An optional list of additional policy documents to include in the simulation. Each document is specified as a string containing the complete, valid JSON text of an IAM policy. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> used to validate this parameter is a string of characters consisting of the following: <ul> <li> Any printable ASCII character ranging from the space character (\u0020) through the end of the ASCII character range </li> <li> The printable characters in the Basic Latin and Latin-1 Supplement character set (through \u00FF) </li> <li> The special characters tab (\u0009), line feed (\u000A), and carriage return (\u000D) </li> </ul><br/>PolicySourceArn:  The Amazon Resource Name (ARN) of a user, group, or role whose policies you want to include in the simulation. If you specify a user, group, or role, the simulation includes all policies that are associated with that entity. If you specify a user, the simulation also includes all policies that are attached to any groups the user belongs to. For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the <i>AWS General Reference</i>.<br/>
        :return: Marker:  When <code>IsTruncated</code> is <code>true</code>, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent pagination request.<br/>IsTruncated:  A flag that indicates whether there are more items to return. If your results were truncated, you can make a subsequent pagination request using the <code>Marker</code> request parameter to retrieve more items. Note that IAM might return fewer than the <code>MaxItems</code> number of results even when there are more results available. We recommend that you check <code>IsTruncated</code> after every call to ensure that you receive all your results.<br/>EvaluationResults:  The results of the simulation.<br/>        """
        pass

    def tag_role(self, Tags: List[Tag], RoleName: str):
        """Adds one or more tags to an IAM role. The role can be a regular role or a service-linked role. If a tag with the same key name already exists, then that tag is overwritten with the new value. A tag consists of a key name and an associated value. By assigning tags to your resources, you can do the following: <ul> <li>  <b>Administrative grouping and discovery</b> - Attach tags to resources to aid in organization and search. For example, you could search for all resources with the key name <i>Project</i> and the value <i>MyImportantProject</i>. Or search for all resources with the key name <i>Cost Center</i> and the value <i>41200</i>.  </li> <li>  <b>Access control</b> - Reference tags in IAM user-based and resource-based policies. You can use tags to restrict access to only an IAM user or role that has a specified tag attached. You can also restrict access to only those resources that have a certain tag attached. For examples of policies that show how to use tags to control access, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_tags.html">Control Access Using IAM Tags</a> in the <i>IAM User Guide</i>. </li> <li>  <b>Cost allocation</b> - Use tags to help track which individuals and teams are using which AWS resources. </li> </ul> <note> <ul> <li> Make sure that you have no invalid tags and that you do not exceed the allowed number of tags per role. In either case, the entire request fails and <i>no</i> tags are added to the role. </li> <li> AWS always interprets the tag <code>Value</code> as a single string. If you need to store an array, you can store comma-separated values in the string. However, you must interpret the value in your code. </li> </ul> </note> For more information about tagging, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_tags.html">Tagging IAM Identities</a> in the <i>IAM User Guide</i>.

        :param self:
        :param Tags:  The list of tags that you want to attach to the role. Each tag consists of a key name and an associated value. You can specify this with a JSON string.<br/>RoleName:  The name of the role that you want to add tags to. This parameter accepts (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters that consist of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def tag_user(self, Tags: List[Tag], UserName: str):
        """Adds one or more tags to an IAM user. If a tag with the same key name already exists, then that tag is overwritten with the new value. A tag consists of a key name and an associated value. By assigning tags to your resources, you can do the following: <ul> <li>  <b>Administrative grouping and discovery</b> - Attach tags to resources to aid in organization and search. For example, you could search for all resources with the key name <i>Project</i> and the value <i>MyImportantProject</i>. Or search for all resources with the key name <i>Cost Center</i> and the value <i>41200</i>.  </li> <li>  <b>Access control</b> - Reference tags in IAM user-based and resource-based policies. You can use tags to restrict access to only an IAM requesting user or to a role that has a specified tag attached. You can also restrict access to only those resources that have a certain tag attached. For examples of policies that show how to use tags to control access, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_tags.html">Control Access Using IAM Tags</a> in the <i>IAM User Guide</i>. </li> <li>  <b>Cost allocation</b> - Use tags to help track which individuals and teams are using which AWS resources. </li> </ul> <note> <ul> <li> Make sure that you have no invalid tags and that you do not exceed the allowed number of tags per role. In either case, the entire request fails and <i>no</i> tags are added to the role. </li> <li> AWS always interprets the tag <code>Value</code> as a single string. If you need to store an array, you can store comma-separated values in the string. However, you must interpret the value in your code. </li> </ul> </note> For more information about tagging, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_tags.html">Tagging IAM Identities</a> in the <i>IAM User Guide</i>.

        :param self:
        :param Tags:  The list of tags that you want to attach to the user. Each tag consists of a key name and an associated value.<br/>UserName:  The name of the user that you want to add tags to. This parameter accepts (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters that consist of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: =,.@-<br/>
        :return:         """
        pass

    def untag_role(self, TagKeys: List[tagKeyType], RoleName: str):
        """Removes the specified tags from the role. For more information about tagging, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_tags.html">Tagging IAM Identities</a> in the <i>IAM User Guide</i>.

        :param self:
        :param TagKeys:  A list of key names as a simple array of strings. The tags with matching keys are removed from the specified role.<br/>RoleName:  The name of the IAM role from which you want to remove tags. This parameter accepts (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters that consist of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def untag_user(self, TagKeys: List[tagKeyType], UserName: str):
        """Removes the specified tags from the user. For more information about tagging, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_tags.html">Tagging IAM Identities</a> in the <i>IAM User Guide</i>.

        :param self:
        :param TagKeys:  A list of key names as a simple array of strings. The tags with matching keys are removed from the specified user.<br/>UserName:  The name of the IAM user from which you want to remove tags. This parameter accepts (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters that consist of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: =,.@-<br/>
        :return:         """
        pass

    def update_access_key(self, Status: str, AccessKeyId: str, UserName: str=None):
        """Changes the status of the specified access key from Active to Inactive, or vice versa. This operation can be used to disable a user's key as part of a key rotation workflow. If the <code>UserName</code> is not specified, the user name is determined implicitly based on the AWS access key ID used to sign the request. This operation works for access keys under the AWS account. Consequently, you can use this operation to manage AWS account root user credentials even if the AWS account has no associated users. For information about rotating keys, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/ManagingCredentials.html">Managing Keys and Certificates</a> in the <i>IAM User Guide</i>.

        :param self:
        :param Status:   The status you want to assign to the secret access key. <code>Active</code> means that the key can be used for API calls to AWS, while <code>Inactive</code> means that the key cannot be used.<br/>AccessKeyId:  The access key ID of the secret access key you want to update. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters that can consist of any upper or lowercased letter or digit.<br/>UserName:  The name of the user whose key you want to update. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def update_account_password_policy(self, MinimumPasswordLength: int=None, RequireSymbols: booleanType=None, RequireNumbers: booleanType=None, RequireUppercaseCharacters: booleanType=None, RequireLowercaseCharacters: booleanType=None, AllowUsersToChangePassword: booleanType=None, MaxPasswordAge: int=None, PasswordReusePrevention: int=None, HardExpiry: booleanObjectType=None):
        """Updates the password policy settings for the AWS account. <note> <ul> <li> This operation does not support partial updates. No parameters are required, but if you do not specify a parameter, that parameter's value reverts to its default value. See the <b>Request Parameters</b> section for each parameter's default value. Also note that some parameters do not allow the default parameter to be explicitly set. Instead, to invoke the default value, do not include that parameter when you invoke the operation. </li> </ul> </note>  For more information about using a password policy, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_ManagingPasswordPolicies.html">Managing an IAM Password Policy</a> in the <i>IAM User Guide</i>.

        :param self:
        :param HardExpiry:  Prevents IAM users from setting a new password after their password has expired. The IAM user cannot be accessed until an administrator resets the password. If you do not specify a value for this parameter, then the operation uses the default value of <code>false</code>. The result is that IAM users can change their passwords after they expire and continue to sign in as the user.<br/>PasswordReusePrevention:  Specifies the number of previous passwords that IAM users are prevented from reusing. If you do not specify a value for this parameter, then the operation uses the default value of <code>0</code>. The result is that IAM users are not prevented from reusing previous passwords.<br/>MaxPasswordAge:  The number of days that an IAM user password is valid. If you do not specify a value for this parameter, then the operation uses the default value of <code>0</code>. The result is that IAM user passwords never expire.<br/>AllowUsersToChangePassword:   Allows all IAM users in your account to use the AWS Management Console to change their own passwords. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/HowToPwdIAMUser.html">Letting IAM Users Change Their Own Passwords</a> in the <i>IAM User Guide</i>. If you do not specify a value for this parameter, then the operation uses the default value of <code>false</code>. The result is that IAM users in the account do not automatically have permissions to change their own password.<br/>RequireLowercaseCharacters:  Specifies whether IAM user passwords must contain at least one lowercase character from the ISO basic Latin alphabet (a to z). If you do not specify a value for this parameter, then the operation uses the default value of <code>false</code>. The result is that passwords do not require at least one lowercase character.<br/>RequireUppercaseCharacters:  Specifies whether IAM user passwords must contain at least one uppercase character from the ISO basic Latin alphabet (A to Z). If you do not specify a value for this parameter, then the operation uses the default value of <code>false</code>. The result is that passwords do not require at least one uppercase character.<br/>RequireNumbers:  Specifies whether IAM user passwords must contain at least one numeric character (0 to 9). If you do not specify a value for this parameter, then the operation uses the default value of <code>false</code>. The result is that passwords do not require at least one numeric character.<br/>RequireSymbols:  Specifies whether IAM user passwords must contain at least one of the following non-alphanumeric characters: ! @ # $ % ^ &amp; * ( ) _ + - = [ ] { } | ' If you do not specify a value for this parameter, then the operation uses the default value of <code>false</code>. The result is that passwords do not require at least one symbol character.<br/>MinimumPasswordLength:  The minimum number of characters allowed in an IAM user password. If you do not specify a value for this parameter, then the operation uses the default value of <code>6</code>.<br/>
        :return:         """
        pass

    def update_assume_role_policy(self, PolicyDocument: str, RoleName: str):
        """Updates the policy that grants an IAM entity permission to assume a role. This is typically referred to as the "role trust policy". For more information about roles, go to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/roles-toplevel.html">Using Roles to Delegate Permissions and Federate Identities</a>.

        :param self:
        :param PolicyDocument:  The policy that grants an entity permission to assume the role. You must provide policies in JSON format in IAM. However, for AWS CloudFormation templates formatted in YAML, you can provide the policy in JSON or YAML format. AWS CloudFormation always converts a YAML policy to JSON format before submitting it to IAM. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> used to validate this parameter is a string of characters consisting of the following: <ul> <li> Any printable ASCII character ranging from the space character (\u0020) through the end of the ASCII character range </li> <li> The printable characters in the Basic Latin and Latin-1 Supplement character set (through \u00FF) </li> <li> The special characters tab (\u0009), line feed (\u000A), and carriage return (\u000D) </li> </ul><br/>RoleName:  The name of the role to update with the new policy. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def update_group(self, GroupName: str, NewPath: str=None, NewGroupName: str=None):
        """Updates the name and/or the path of the specified IAM group. <important>  You should understand the implications of changing a group's path or name. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_WorkingWithGroupsAndUsers.html">Renaming Users and Groups</a> in the <i>IAM User Guide</i>. </important> <note> The person making the request (the principal), must have permission to change the role group with the old name and the new name. For example, to change the group named <code>Managers</code> to <code>MGRs</code>, the principal must have a policy that allows them to update both groups. If the principal has permission to update the <code>Managers</code> group, but not the <code>MGRs</code> group, then the update fails. For more information about permissions, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access.html">Access Management</a>.  </note>

        :param self:
        :param NewGroupName:  New name for the IAM group. Only include this if changing the group's name. IAM user, group, role, and policy names must be unique within the account. Names are not distinguished by case. For example, you cannot create resources named both "MyResource" and "myresource".<br/>NewPath:  New path for the IAM group. Only include this if changing the group's path. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of either a forward slash (/) by itself or a string that must begin and end with forward slashes. In addition, it can contain any ASCII character from the ! (\u0021) through the DEL character (\u007F), including most punctuation characters, digits, and upper and lowercased letters.<br/>GroupName:  Name of the IAM group to update. If you're changing the name of the group, this is the original name. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def update_login_profile(self, UserName: str, Password: str=None, PasswordResetRequired: booleanObjectType=None):
        """Changes the password for the specified IAM user. IAM users can change their own passwords by calling <a>ChangePassword</a>. For more information about modifying passwords, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_ManagingLogins.html">Managing Passwords</a> in the <i>IAM User Guide</i>.

        :param self:
        :param PasswordResetRequired:  Allows this new password to be used only once by requiring the specified IAM user to set a new password on next sign-in.<br/>Password:  The new password for the specified IAM user. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> used to validate this parameter is a string of characters consisting of the following: <ul> <li> Any printable ASCII character ranging from the space character (\u0020) through the end of the ASCII character range </li> <li> The printable characters in the Basic Latin and Latin-1 Supplement character set (through \u00FF) </li> <li> The special characters tab (\u0009), line feed (\u000A), and carriage return (\u000D) </li> </ul> However, the format can be further restricted by the account administrator by setting a password policy on the AWS account. For more information, see <a>UpdateAccountPasswordPolicy</a>.<br/>UserName:  The name of the user whose password you want to update. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def update_open_id_connect_provider_thumbprint(self, ThumbprintList: List[thumbprintType], OpenIDConnectProviderArn: str):
        """Replaces the existing list of server certificate thumbprints associated with an OpenID Connect (OIDC) provider resource object with a new list of thumbprints. The list that you pass with this operation completely replaces the existing list of thumbprints. (The lists are not merged.) Typically, you need to update a thumbprint only when the identity provider's certificate changes, which occurs rarely. However, if the provider's certificate <i>does</i> change, any attempt to assume an IAM role that specifies the OIDC provider as a principal fails until the certificate thumbprint is updated. <note> Trust for the OIDC provider is derived from the provider's certificate and is validated by the thumbprint. Therefore, it is best to limit access to the <code>UpdateOpenIDConnectProviderThumbprint</code> operation to highly privileged users. </note>

        :param self:
        :param ThumbprintList:  A list of certificate thumbprints that are associated with the specified IAM OpenID Connect provider. For more information, see <a>CreateOpenIDConnectProvider</a>. <br/>OpenIDConnectProviderArn:  The Amazon Resource Name (ARN) of the IAM OIDC provider resource object for which you want to update the thumbprint. You can get a list of OIDC provider ARNs by using the <a>ListOpenIDConnectProviders</a> operation. For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the <i>AWS General Reference</i>.<br/>
        :return:         """
        pass

    def update_role(self, RoleName: str, Description: str=None, MaxSessionDuration: int=None) -> UpdateRoleResponse:
        """Updates the description or maximum session duration setting of a role.

        :param self:
        :param MaxSessionDuration:  The maximum session duration (in seconds) that you want to set for the specified role. If you do not specify a value for this setting, the default maximum of one hour is applied. This setting can have a value from 1 hour to 12 hours. Anyone who assumes the role from the AWS CLI or API can use the <code>DurationSeconds</code> API parameter or the <code>duration-seconds</code> CLI parameter to request a longer session. The <code>MaxSessionDuration</code> setting determines the maximum duration that can be requested using the <code>DurationSeconds</code> parameter. If users don't specify a value for the <code>DurationSeconds</code> parameter, their security credentials are valid for one hour by default. This applies when you use the <code>AssumeRole*</code> API operations or the <code>assume-role*</code> CLI operations but does not apply when you use those operations to create a console URL. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_use.html">Using IAM Roles</a> in the <i>IAM User Guide</i>.<br/>Description:  The new description that you want to apply to the specified role.<br/>RoleName:  The name of the role that you want to modify.<br/>
        :return:         """
        pass

    def update_role_description(self, Description: str, RoleName: str) -> UpdateRoleDescriptionResponse:
        """Use <a>UpdateRole</a> instead. Modifies only the description of a role. This operation performs the same function as the <code>Description</code> parameter in the <code>UpdateRole</code> operation.

        :param self:
        :param Description:  The new description that you want to apply to the specified role.<br/>RoleName:  The name of the role that you want to modify.<br/>
        :return: Role:  A structure that contains details about the modified role.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> A list of tags that are attached to the specified role. For more information about tagging, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_tags.html">Tagging IAM Identities</a> in the <i>IAM User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PermissionsBoundary</i> The ARN of the policy used to set the permissions boundary for the role. For more information about permissions boundaries, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies_boundaries.html">Permissions Boundaries for IAM Identities </a> in the <i>IAM User Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PermissionsBoundary</b>: The ARN of the policy used to set the permissions boundary for the role. For more information about permissions boundaries, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies_boundaries.html">Permissions Boundaries for IAM Identities </a> in the <i>IAM User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxSessionDuration</i> The maximum session duration (in seconds) for the specified role. Anyone who uses the AWS CLI, or API to assume the role can specify the duration using the optional <code>DurationSeconds</code> API parameter or <code>duration-seconds</code> CLI parameter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of the role that you provide.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssumeRolePolicyDocument</i> The policy that grants an entity permission to assume the role.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateDate</i> The date and time, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>, when the role was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i>  The Amazon Resource Name (ARN) specifying the role. For more information about ARNs and how to use them in policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>IAM User Guide</i> guide. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleId</i>  The stable and unique string identifying the role. For more information about IDs, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleName</i> The friendly name that identifies the role.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Path</i>  The path to the role. For more information about paths, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PermissionsBoundaryArn</i>  The ARN of the policy used to set the permissions boundary for the user or role.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PermissionsBoundaryType</i>  The permissions boundary usage type that indicates what type of IAM resource is used as the permissions boundary for an entity. This data type can only have a value of <code>Policy</code>.<br/>        """
        pass

    def update_saml_provider(self, SAMLProviderArn: str, SAMLMetadataDocument: str) -> UpdateSAMLProviderResponse:
        """Updates the metadata document for an existing SAML provider resource object. <note> This operation requires <a href="https://docs.aws.amazon.com/general/latest/gr/signature-version-4.html">Signature Version 4</a>. </note>

        :param self:
        :param SAMLProviderArn:  The Amazon Resource Name (ARN) of the SAML provider to update. For more information about ARNs, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the <i>AWS General Reference</i>.<br/>SAMLMetadataDocument:  An XML document generated by an identity provider (IdP) that supports SAML 2.0. The document includes the issuer's name, expiration information, and keys that can be used to validate the SAML authentication response (assertions) that are received from the IdP. You must generate the metadata document using the identity management software that is used as your organization's IdP.<br/>
        :return: SAMLProviderArn:  The Amazon Resource Name (ARN) of the SAML provider that was updated.<br/>        """
        pass

    def update_ssh_public_key(self, Status: str, SSHPublicKeyId: str, UserName: str):
        """Sets the status of an IAM user's SSH public key to active or inactive. SSH public keys that are inactive cannot be used for authentication. This operation can be used to disable a user's SSH public key as part of a key rotation work flow. The SSH public key affected by this operation is used only for authenticating the associated IAM user to an AWS CodeCommit repository. For more information about using SSH keys to authenticate to an AWS CodeCommit repository, see <a href="https://docs.aws.amazon.com/codecommit/latest/userguide/setting-up-credentials-ssh.html">Set up AWS CodeCommit for SSH Connections</a> in the <i>AWS CodeCommit User Guide</i>.

        :param self:
        :param Status:  The status to assign to the SSH public key. <code>Active</code> means that the key can be used for authentication with an AWS CodeCommit repository. <code>Inactive</code> means that the key cannot be used.<br/>SSHPublicKeyId:  The unique identifier for the SSH public key. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters that can consist of any upper or lowercased letter or digit.<br/>UserName:  The name of the IAM user associated with the SSH public key. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def update_server_certificate(self, ServerCertificateName: str, NewPath: str=None, NewServerCertificateName: str=None):
        """Updates the name and/or the path of the specified server certificate stored in IAM. For more information about working with server certificates, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_server-certs.html">Working with Server Certificates</a> in the <i>IAM User Guide</i>. This topic also includes a list of AWS services that can use the server certificates that you manage with IAM. <important> You should understand the implications of changing a server certificate's path or name. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_server-certs_manage.html#RenamingServerCerts">Renaming a Server Certificate</a> in the <i>IAM User Guide</i>. </important> <note> The person making the request (the principal), must have permission to change the server certificate with the old name and the new name. For example, to change the certificate named <code>ProductionCert</code> to <code>ProdCert</code>, the principal must have a policy that allows them to update both certificates. If the principal has permission to update the <code>ProductionCert</code> group, but not the <code>ProdCert</code> certificate, then the update fails. For more information about permissions, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access.html">Access Management</a> in the <i>IAM User Guide</i>. </note>

        :param self:
        :param NewServerCertificateName:  The new name for the server certificate. Include this only if you are updating the server certificate's name. The name of the certificate cannot contain any spaces. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>NewPath:  The new path for the server certificate. Include this only if you are updating the server certificate's path. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of either a forward slash (/) by itself or a string that must begin and end with forward slashes. In addition, it can contain any ASCII character from the ! (\u0021) through the DEL character (\u007F), including most punctuation characters, digits, and upper and lowercased letters.<br/>ServerCertificateName:  The name of the server certificate that you want to update. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def update_service_specific_credential(self, Status: str, ServiceSpecificCredentialId: str, UserName: str=None):
        """Sets the status of a service-specific credential to <code>Active</code> or <code>Inactive</code>. Service-specific credentials that are inactive cannot be used for authentication to the service. This operation can be used to disable a user's service-specific credential as part of a credential rotation work flow.

        :param self:
        :param Status:  The status to be assigned to the service-specific credential.<br/>ServiceSpecificCredentialId:  The unique identifier of the service-specific credential. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters that can consist of any upper or lowercased letter or digit.<br/>UserName:  The name of the IAM user associated with the service-specific credential. If you do not specify this value, then the operation assumes the user whose credentials are used to call the operation. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def update_signing_certificate(self, Status: str, CertificateId: str, UserName: str=None):
        """Changes the status of the specified user signing certificate from active to disabled, or vice versa. This operation can be used to disable an IAM user's signing certificate as part of a certificate rotation work flow. If the <code>UserName</code> field is not specified, the user name is determined implicitly based on the AWS access key ID used to sign the request. This operation works for access keys under the AWS account. Consequently, you can use this operation to manage AWS account root user credentials even if the AWS account has no associated users.

        :param self:
        :param Status:   The status you want to assign to the certificate. <code>Active</code> means that the certificate can be used for API calls to AWS <code>Inactive</code> means that the certificate cannot be used.<br/>CertificateId:  The ID of the signing certificate you want to update. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters that can consist of any upper or lowercased letter or digit.<br/>UserName:  The name of the IAM user the signing certificate belongs to. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def update_user(self, UserName: str, NewPath: str=None, NewUserName: str=None):
        """Updates the name and/or the path of the specified IAM user. <important>  You should understand the implications of changing an IAM user's path or name. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_users_manage.html#id_users_renaming">Renaming an IAM User</a> and <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_groups_manage_rename.html">Renaming an IAM Group</a> in the <i>IAM User Guide</i>. </important> <note>  To change a user name, the requester must have appropriate permissions on both the source object and the target object. For example, to change Bob to Robert, the entity making the request must have permission on Bob and Robert, or must have permission on all (*). For more information about permissions, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/PermissionsAndPolicies.html">Permissions and Policies</a>.  </note>

        :param self:
        :param NewUserName:  New name for the user. Include this parameter only if you're changing the user's name. IAM user, group, role, and policy names must be unique within the account. Names are not distinguished by case. For example, you cannot create resources named both "MyResource" and "myresource".<br/>NewPath:  New path for the IAM user. Include this parameter only if you're changing the user's path. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of either a forward slash (/) by itself or a string that must begin and end with forward slashes. In addition, it can contain any ASCII character from the ! (\u0021) through the DEL character (\u007F), including most punctuation characters, digits, and upper and lowercased letters.<br/>UserName:  Name of the user to update. If you're changing the name of the user, this is the original user name. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return:         """
        pass

    def upload_ssh_public_key(self, SSHPublicKeyBody: str, UserName: str) -> UploadSSHPublicKeyResponse:
        """Uploads an SSH public key and associates it with the specified IAM user. The SSH public key uploaded by this operation can be used only for authenticating the associated IAM user to an AWS CodeCommit repository. For more information about using SSH keys to authenticate to an AWS CodeCommit repository, see <a href="https://docs.aws.amazon.com/codecommit/latest/userguide/setting-up-credentials-ssh.html">Set up AWS CodeCommit for SSH Connections</a> in the <i>AWS CodeCommit User Guide</i>.

        :param self:
        :param SSHPublicKeyBody:  The SSH public key. The public key must be encoded in ssh-rsa format or PEM format. The minimum bit-length of the public key is 2048 bits. For example, you can generate a 2048-bit key, and the resulting PEM file is 1679 bytes long. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> used to validate this parameter is a string of characters consisting of the following: <ul> <li> Any printable ASCII character ranging from the space character (\u0020) through the end of the ASCII character range </li> <li> The printable characters in the Basic Latin and Latin-1 Supplement character set (through \u00FF) </li> <li> The special characters tab (\u0009), line feed (\u000A), and carriage return (\u000D) </li> </ul><br/>UserName:  The name of the IAM user to associate the SSH public key with. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: SSHPublicKey:  Contains information about the SSH public key.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UploadDate</i> The date and time, in <a href="http://www.iso.org/iso/iso8601">ISO 8601 date-time format</a>, when the SSH public key was uploaded.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the SSH public key. <code>Active</code> means that the key can be used for authentication with an AWS CodeCommit repository. <code>Inactive</code> means that the key cannot be used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SSHPublicKeyBody</i> The SSH public key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Fingerprint</i> The MD5 message digest of the SSH public key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SSHPublicKeyId</i> The unique identifier for the SSH public key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserName</i> The name of the IAM user associated with the SSH public key.<br/>        """
        pass

    def upload_server_certificate(self, PrivateKey: str, CertificateBody: str, ServerCertificateName: str, Path: str=None, CertificateChain: str=None) -> UploadServerCertificateResponse:
        """Uploads a server certificate entity for the AWS account. The server certificate entity includes a public key certificate, a private key, and an optional certificate chain, which should all be PEM-encoded. We recommend that you use <a href="https://docs.aws.amazon.com/acm/">AWS Certificate Manager</a> to provision, manage, and deploy your server certificates. With ACM you can request a certificate, deploy it to AWS resources, and let ACM handle certificate renewals for you. Certificates provided by ACM are free. For more information about using ACM, see the <a href="https://docs.aws.amazon.com/acm/latest/userguide/">AWS Certificate Manager User Guide</a>. For more information about working with server certificates, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_server-certs.html">Working with Server Certificates</a> in the <i>IAM User Guide</i>. This topic includes a list of AWS services that can use the server certificates that you manage with IAM. For information about the number of server certificates you can upload, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_iam-limits.html">Limitations on IAM Entities and Objects</a> in the <i>IAM User Guide</i>. <note> Because the body of the public key certificate, private key, and the certificate chain can be large, you should use POST rather than GET when calling <code>UploadServerCertificate</code>. For information about setting up signatures and authorization through the API, go to <a href="https://docs.aws.amazon.com/general/latest/gr/signing_aws_api_requests.html">Signing AWS API Requests</a> in the <i>AWS General Reference</i>. For general information about using the Query API with IAM, go to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/programming.html">Calling the API by Making HTTP Query Requests</a> in the <i>IAM User Guide</i>. </note>

        :param self:
        :param CertificateChain:  The contents of the certificate chain. This is typically a concatenation of the PEM-encoded public key certificates of the chain. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> used to validate this parameter is a string of characters consisting of the following: <ul> <li> Any printable ASCII character ranging from the space character (\u0020) through the end of the ASCII character range </li> <li> The printable characters in the Basic Latin and Latin-1 Supplement character set (through \u00FF) </li> <li> The special characters tab (\u0009), line feed (\u000A), and carriage return (\u000D) </li> </ul><br/>PrivateKey:  The contents of the private key in PEM-encoded format. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> used to validate this parameter is a string of characters consisting of the following: <ul> <li> Any printable ASCII character ranging from the space character (\u0020) through the end of the ASCII character range </li> <li> The printable characters in the Basic Latin and Latin-1 Supplement character set (through \u00FF) </li> <li> The special characters tab (\u0009), line feed (\u000A), and carriage return (\u000D) </li> </ul><br/>CertificateBody:  The contents of the public key certificate in PEM-encoded format. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> used to validate this parameter is a string of characters consisting of the following: <ul> <li> Any printable ASCII character ranging from the space character (\u0020) through the end of the ASCII character range </li> <li> The printable characters in the Basic Latin and Latin-1 Supplement character set (through \u00FF) </li> <li> The special characters tab (\u0009), line feed (\u000A), and carriage return (\u000D) </li> </ul><br/>ServerCertificateName:  The name for the server certificate. Do not include the path in this value. The name of the certificate cannot contain any spaces. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>Path:  The path for the server certificate. For more information about paths, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>IAM User Guide</i>. This parameter is optional. If it is not included, it defaults to a slash (/). This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of either a forward slash (/) by itself or a string that must begin and end with forward slashes. In addition, it can contain any ASCII character from the ! (\u0021) through the DEL character (\u007F), including most punctuation characters, digits, and upper and lowercased letters. <note>  If you are uploading a server certificate specifically for use with Amazon CloudFront distributions, you must specify a path using the <code>path</code> parameter. The path must begin with <code>/cloudfront</code> and must include a trailing slash (for example, <code>/cloudfront/test/</code>). </note><br/>
        :return: ServerCertificateMetadata:  The meta information of the uploaded server certificate without its certificate body, certificate chain, and private key.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Expiration</i> The date on which the certificate is set to expire.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UploadDate</i> The date when the server certificate was uploaded.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i>  The Amazon Resource Name (ARN) specifying the server certificate. For more information about ARNs and how to use them in policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerCertificateId</i>  The stable and unique string identifying the server certificate. For more information about IDs, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerCertificateName</i> The name that identifies the server certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Path</i>  The path to the server certificate. For more information about paths, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">IAM Identifiers</a> in the <i>Using IAM</i> guide. <br/>        """
        pass

    def upload_signing_certificate(self, CertificateBody: str, UserName: str=None) -> UploadSigningCertificateResponse:
        """Uploads an X.509 signing certificate and associates it with the specified IAM user. Some AWS services use X.509 signing certificates to validate requests that are signed with a corresponding private key. When you upload the certificate, its default status is <code>Active</code>. If the <code>UserName</code> is not specified, the IAM user name is determined implicitly based on the AWS access key ID used to sign the request. This operation works for access keys under the AWS account. Consequently, you can use this operation to manage AWS account root user credentials even if the AWS account has no associated users. <note> Because the body of an X.509 certificate can be large, you should use POST rather than GET when calling <code>UploadSigningCertificate</code>. For information about setting up signatures and authorization through the API, go to <a href="https://docs.aws.amazon.com/general/latest/gr/signing_aws_api_requests.html">Signing AWS API Requests</a> in the <i>AWS General Reference</i>. For general information about using the Query API with IAM, go to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/IAM_UsingQueryAPI.html">Making Query Requests</a> in the <i>IAM User Guide</i>. </note>

        :param self:
        :param CertificateBody:  The contents of the signing certificate. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> used to validate this parameter is a string of characters consisting of the following: <ul> <li> Any printable ASCII character ranging from the space character (\u0020) through the end of the ASCII character range </li> <li> The printable characters in the Basic Latin and Latin-1 Supplement character set (through \u00FF) </li> <li> The special characters tab (\u0009), line feed (\u000A), and carriage return (\u000D) </li> </ul><br/>UserName:  The name of the user the signing certificate is for. This parameter allows (through its <a href="http://wikipedia.org/wiki/regex">regex pattern</a>) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-<br/>
        :return: Certificate:  Information about the certificate.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UploadDate</i> The date when the signing certificate was uploaded.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the signing certificate. <code>Active</code> means that the key is valid for API calls, while <code>Inactive</code> means it is not.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateBody</i> The contents of the signing certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateId</i> The ID for the signing certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserName</i> The name of the user the signing certificate is associated with.<br/>        """
        pass

    class IamResource:
        
        class AccessKey:
            """Creates a AccessKey resource.::

  access_key = iam.AccessKey('user_name','id')

:type user_name: string
:param user_name: The AccessKey's user_name identifier. This **must** be set.
:type id: string
:param id: The AccessKey's id identifier. This **must** be set.

:rtype: :py:class:`IAM.AccessKey`
:returns: A AccessKey resource"""

            def __init__(self, user_name, id):
                pass
            
            class dateType(object):
                pass
        
            user_name: str=None
            access_key_id: str=None
            status: str=None
            create_date: dateType=None
            
            def activate(self, **kwargs):
                # type: (object) -> dict
                return {}

            def deactivate(self, **kwargs):
                # type: (object) -> dict
                return {}

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class AccessKeyPair:
            """Creates a AccessKeyPair resource.::

  access_key_pair = iam.AccessKeyPair('user_name','id','secret')

:type user_name: string
:param user_name: The AccessKeyPair's user_name identifier. This **must** be set.
:type id: string
:param id: The AccessKeyPair's id identifier. This **must** be set.
:type secret: string
:param secret: The AccessKeyPair's secret identifier. This **must** be set.

:rtype: :py:class:`IAM.AccessKeyPair`
:returns: A AccessKeyPair resource"""

            def __init__(self, user_name, id, secret):
                pass
            
            class dateType(object):
                pass
        
            user_name: str=None
            access_key_id: str=None
            status: str=None
            secret_access_key: str=None
            create_date: dateType=None
            
            def activate(self, **kwargs):
                # type: (object) -> dict
                return {}

            def deactivate(self, **kwargs):
                # type: (object) -> dict
                return {}

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class AccountPasswordPolicy:
            """Creates a AccountPasswordPolicy resource.::

  account_password_policy = iam.AccountPasswordPolicy()


:rtype: :py:class:`IAM.AccountPasswordPolicy`
:returns: A AccountPasswordPolicy resource"""

            def __init__(self):
                pass
            
            class booleanType(object):
                pass

            class booleanObjectType(object):
                pass
        
            minimum_password_length: int=None
            require_symbols: booleanType=None
            require_numbers: booleanType=None
            require_uppercase_characters: booleanType=None
            require_lowercase_characters: booleanType=None
            allow_users_to_change_password: booleanType=None
            expire_passwords: booleanType=None
            max_password_age: int=None
            password_reuse_prevention: int=None
            hard_expiry: booleanObjectType=None
            
            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def update(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class AccountSummary:
            """Creates a AccountSummary resource.::

  account_summary = iam.AccountSummary()


:rtype: :py:class:`IAM.AccountSummary`
:returns: A AccountSummary resource"""

            def __init__(self):
                pass
            
            class summaryMapType(object):
                pass
        
            summary_map: summaryMapType=None
            

        
        class AssumeRolePolicy:
            """Creates a AssumeRolePolicy resource.::

  assume_role_policy = iam.AssumeRolePolicy('role_name')

:type role_name: string
:param role_name: The AssumeRolePolicy's role_name identifier. This **must** be set.

:rtype: :py:class:`IAM.AssumeRolePolicy`
:returns: A AssumeRolePolicy resource"""

            def __init__(self, role_name):
                pass
            
        
            
            def update(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class CurrentUser:
            """Creates a CurrentUser resource.::

  current_user = iam.CurrentUser()


:rtype: :py:class:`IAM.CurrentUser`
:returns: A CurrentUser resource"""

            def __init__(self):
                pass
            
            class AttachedPermissionsBoundary(Mapping):
                pass

            class Tag(Mapping):
                pass

            class dateType(object):
                pass
        
            path: str=None
            user_name: str=None
            user_id: str=None
            arn: str=None
            create_date: dateType=None
            password_last_used: dateType=None
            permissions_boundary: AttachedPermissionsBoundary=None
            tags: List[Tag]=None
            

        
        class Group:
            """Creates a Group resource.::

  group = iam.Group('name')

:type name: string
:param name: The Group's name identifier. This **must** be set.

:rtype: :py:class:`IAM.Group`
:returns: A Group resource"""

            def __init__(self, name):
                pass
            
            class dateType(object):
                pass
        
            path: str=None
            group_name: str=None
            group_id: str=None
            arn: str=None
            create_date: dateType=None
            
            def add_user(self, **kwargs):
                # type: (object) -> dict
                return {}

            def attach_policy(self, **kwargs):
                # type: (object) -> dict
                return {}

            def create(self, **kwargs):
                # type: (object) -> Group
                return Group()

            def create_policy(self, **kwargs):
                # type: (object) -> GroupPolicy
                return GroupPolicy()

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def detach_policy(self, **kwargs):
                # type: (object) -> dict
                return {}

            def remove_user(self, **kwargs):
                # type: (object) -> dict
                return {}

            def update(self, **kwargs):
                # type: (object) -> Group
                return Group()


        
        class GroupPolicy:
            """Creates a GroupPolicy resource.::

  group_policy = iam.GroupPolicy('group_name','name')

:type group_name: string
:param group_name: The GroupPolicy's group_name identifier. This **must** be set.
:type name: string
:param name: The GroupPolicy's name identifier. This **must** be set.

:rtype: :py:class:`IAM.GroupPolicy`
:returns: A GroupPolicy resource"""

            def __init__(self, group_name, name):
                pass
            
        
            group_name: str=None
            policy_name: str=None
            policy_document: str=None
            
            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def put(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class InstanceProfile:
            """Creates a InstanceProfile resource.::

  instance_profile = iam.InstanceProfile('name')

:type name: string
:param name: The InstanceProfile's name identifier. This **must** be set.

:rtype: :py:class:`IAM.InstanceProfile`
:returns: A InstanceProfile resource"""

            def __init__(self, name):
                pass
            
            class dateType(object):
                pass

            class Role(Mapping):
                pass
        
            path: str=None
            instance_profile_name: str=None
            instance_profile_id: str=None
            arn: str=None
            create_date: dateType=None
            roles: List[Role]=None
            
            def add_role(self, **kwargs):
                # type: (object) -> dict
                return {}

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def remove_role(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class LoginProfile:
            """Creates a LoginProfile resource.::

  login_profile = iam.LoginProfile('user_name')

:type user_name: string
:param user_name: The LoginProfile's user_name identifier. This **must** be set.

:rtype: :py:class:`IAM.LoginProfile`
:returns: A LoginProfile resource"""

            def __init__(self, user_name):
                pass
            
            class booleanType(object):
                pass

            class dateType(object):
                pass
        
            user_name: str=None
            create_date: dateType=None
            password_reset_required: booleanType=None
            
            def create(self, **kwargs):
                # type: (object) -> LoginProfile
                return LoginProfile()

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def update(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class MfaDevice:
            """Creates a MfaDevice resource.::

  mfa_device = iam.MfaDevice('user_name','serial_number')

:type user_name: string
:param user_name: The MfaDevice's user_name identifier. This **must** be set.
:type serial_number: string
:param serial_number: The MfaDevice's serial_number identifier. This **must** be set.

:rtype: :py:class:`IAM.MfaDevice`
:returns: A MfaDevice resource"""

            def __init__(self, user_name, serial_number):
                pass
            
            class dateType(object):
                pass
        
            user_name: str=None
            serial_number: str=None
            enable_date: dateType=None
            
            def associate(self, **kwargs):
                # type: (object) -> dict
                return {}

            def disassociate(self, **kwargs):
                # type: (object) -> dict
                return {}

            def resync(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class Policy:
            """Creates a Policy resource.::

  policy = iam.Policy('policy_arn')

:type policy_arn: string
:param policy_arn: The Policy's policy_arn identifier. This **must** be set.

:rtype: :py:class:`IAM.Policy`
:returns: A Policy resource"""

            def __init__(self, policy_arn):
                pass
            
            class booleanType(object):
                pass

            class dateType(object):
                pass
        
            policy_name: str=None
            policy_id: str=None
            arn: str=None
            path: str=None
            default_version_id: str=None
            attachment_count: int=None
            permissions_boundary_usage_count: int=None
            is_attachable: booleanType=None
            description: str=None
            create_date: dateType=None
            update_date: dateType=None
            
            def attach_group(self, **kwargs):
                # type: (object) -> dict
                return {}

            def attach_role(self, **kwargs):
                # type: (object) -> dict
                return {}

            def attach_user(self, **kwargs):
                # type: (object) -> dict
                return {}

            def create_version(self, **kwargs):
                # type: (object) -> PolicyVersion
                return PolicyVersion()

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def detach_group(self, **kwargs):
                # type: (object) -> dict
                return {}

            def detach_role(self, **kwargs):
                # type: (object) -> dict
                return {}

            def detach_user(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class PolicyVersion:
            """Creates a PolicyVersion resource.::

  policy_version = iam.PolicyVersion('arn','version_id')

:type arn: string
:param arn: The PolicyVersion's arn identifier. This **must** be set.
:type version_id: string
:param version_id: The PolicyVersion's version_id identifier. This **must** be set.

:rtype: :py:class:`IAM.PolicyVersion`
:returns: A PolicyVersion resource"""

            def __init__(self, arn, version_id):
                pass
            
            class booleanType(object):
                pass

            class dateType(object):
                pass
        
            document: str=None
            version_id: str=None
            is_default_version: booleanType=None
            create_date: dateType=None
            
            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def set_as_default(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class Role:
            """Creates a Role resource.::

  role = iam.Role('name')

:type name: string
:param name: The Role's name identifier. This **must** be set.

:rtype: :py:class:`IAM.Role`
:returns: A Role resource"""

            def __init__(self, name):
                pass
            
            class AttachedPermissionsBoundary(Mapping):
                pass

            class Tag(Mapping):
                pass

            class dateType(object):
                pass
        
            path: str=None
            role_name: str=None
            role_id: str=None
            arn: str=None
            create_date: dateType=None
            assume_role_policy_document: str=None
            description: str=None
            max_session_duration: int=None
            permissions_boundary: AttachedPermissionsBoundary=None
            tags: List[Tag]=None
            
            def attach_policy(self, **kwargs):
                # type: (object) -> dict
                return {}

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def detach_policy(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class RolePolicy:
            """Creates a RolePolicy resource.::

  role_policy = iam.RolePolicy('role_name','name')

:type role_name: string
:param role_name: The RolePolicy's role_name identifier. This **must** be set.
:type name: string
:param name: The RolePolicy's name identifier. This **must** be set.

:rtype: :py:class:`IAM.RolePolicy`
:returns: A RolePolicy resource"""

            def __init__(self, role_name, name):
                pass
            
        
            role_name: str=None
            policy_name: str=None
            policy_document: str=None
            
            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def put(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class SamlProvider:
            """Creates a SamlProvider resource.::

  saml_provider = iam.SamlProvider('arn')

:type arn: string
:param arn: The SamlProvider's arn identifier. This **must** be set.

:rtype: :py:class:`IAM.SamlProvider`
:returns: A SamlProvider resource"""

            def __init__(self, arn):
                pass
            
            class dateType(object):
                pass
        
            saml_metadata_document: str=None
            create_date: dateType=None
            valid_until: dateType=None
            
            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def update(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class ServerCertificate:
            """Creates a ServerCertificate resource.::

  server_certificate = iam.ServerCertificate('name')

:type name: string
:param name: The ServerCertificate's name identifier. This **must** be set.

:rtype: :py:class:`IAM.ServerCertificate`
:returns: A ServerCertificate resource"""

            def __init__(self, name):
                pass
            
            class ServerCertificateMetadata(Mapping):
                pass
        
            server_certificate_metadata: ServerCertificateMetadata=None
            certificate_body: str=None
            certificate_chain: str=None
            
            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def update(self, **kwargs):
                # type: (object) -> ServerCertificate
                return ServerCertificate()


        
        class SigningCertificate:
            """Creates a SigningCertificate resource.::

  signing_certificate = iam.SigningCertificate('user_name','id')

:type user_name: string
:param user_name: The SigningCertificate's user_name identifier. This **must** be set.
:type id: string
:param id: The SigningCertificate's id identifier. This **must** be set.

:rtype: :py:class:`IAM.SigningCertificate`
:returns: A SigningCertificate resource"""

            def __init__(self, user_name, id):
                pass
            
            class dateType(object):
                pass
        
            user_name: str=None
            certificate_id: str=None
            certificate_body: str=None
            status: str=None
            upload_date: dateType=None
            
            def activate(self, **kwargs):
                # type: (object) -> dict
                return {}

            def deactivate(self, **kwargs):
                # type: (object) -> dict
                return {}

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class User:
            """Creates a User resource.::

  user = iam.User('name')

:type name: string
:param name: The User's name identifier. This **must** be set.

:rtype: :py:class:`IAM.User`
:returns: A User resource"""

            def __init__(self, name):
                pass
            
            class AttachedPermissionsBoundary(Mapping):
                pass

            class Tag(Mapping):
                pass

            class dateType(object):
                pass
        
            path: str=None
            user_name: str=None
            user_id: str=None
            arn: str=None
            create_date: dateType=None
            password_last_used: dateType=None
            permissions_boundary: AttachedPermissionsBoundary=None
            tags: List[Tag]=None
            
            def add_group(self, **kwargs):
                # type: (object) -> dict
                return {}

            def attach_policy(self, **kwargs):
                # type: (object) -> dict
                return {}

            def create(self, **kwargs):
                # type: (object) -> User
                return User()

            def create_access_key_pair(self, **kwargs):
                # type: (object) -> AccessKeyPair
                return AccessKeyPair()

            def create_login_profile(self, **kwargs):
                # type: (object) -> LoginProfile
                return LoginProfile()

            def create_policy(self, **kwargs):
                # type: (object) -> UserPolicy
                return UserPolicy()

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def detach_policy(self, **kwargs):
                # type: (object) -> dict
                return {}

            def enable_mfa(self, **kwargs):
                # type: (object) -> MfaDevice
                return MfaDevice()

            def remove_group(self, **kwargs):
                # type: (object) -> dict
                return {}

            def update(self, **kwargs):
                # type: (object) -> User
                return User()


        
        class UserPolicy:
            """Creates a UserPolicy resource.::

  user_policy = iam.UserPolicy('user_name','name')

:type user_name: string
:param user_name: The UserPolicy's user_name identifier. This **must** be set.
:type name: string
:param name: The UserPolicy's name identifier. This **must** be set.

:rtype: :py:class:`IAM.UserPolicy`
:returns: A UserPolicy resource"""

            def __init__(self, user_name, name):
                pass
            
        
            user_name: str=None
            policy_name: str=None
            policy_document: str=None
            
            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def put(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class VirtualMfaDevice:
            """Creates a VirtualMfaDevice resource.::

  virtual_mfa_device = iam.VirtualMfaDevice('serial_number')

:type serial_number: string
:param serial_number: The VirtualMfaDevice's serial_number identifier. This **must** be set.

:rtype: :py:class:`IAM.VirtualMfaDevice`
:returns: A VirtualMfaDevice resource"""

            def __init__(self, serial_number):
                pass
            
            class User(Mapping):
                pass

            class dateType(object):
                pass

            class BootstrapDatum(object):
                pass
        
            serial_number: str=None
            base32_string_seed: BootstrapDatum=None
            qr_code_png: BootstrapDatum=None
            user: User=None
            enable_date: dateType=None
            
            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}


        def change_password(self, **kwargs):
            # type: (object) -> dict
            pass

        def create_account_alias(self, **kwargs):
            # type: (object) -> dict
            pass

        def create_account_password_policy(self, **kwargs):
            # type: (object) -> AccountPasswordPolicy
            pass

        def create_group(self, **kwargs):
            # type: (object) -> Group
            pass

        def create_instance_profile(self, **kwargs):
            # type: (object) -> InstanceProfile
            pass

        def create_policy(self, **kwargs):
            # type: (object) -> Policy
            pass

        def create_role(self, **kwargs):
            # type: (object) -> Role
            pass

        def create_saml_provider(self, **kwargs):
            # type: (object) -> SamlProvider
            pass

        def create_server_certificate(self, **kwargs):
            # type: (object) -> ServerCertificate
            pass

        def create_signing_certificate(self, **kwargs):
            # type: (object) -> SigningCertificate
            pass

        def create_user(self, **kwargs):
            # type: (object) -> User
            pass

        def create_virtual_mfa_device(self, **kwargs):
            # type: (object) -> VirtualMfaDevice
            pass

        def get_available_subresources(self) -> List[str]:
            pass

        class GroupResourceCollection(List[Group], ResourceCollection):
            pass

        groups: GroupResourceCollection = None
        class InstanceProfileResourceCollection(List[InstanceProfile], ResourceCollection):
            pass

        instance_profiles: InstanceProfileResourceCollection = None
        class PolicyResourceCollection(List[Policy], ResourceCollection):
            pass

        policies: PolicyResourceCollection = None
        class RoleResourceCollection(List[Role], ResourceCollection):
            pass

        roles: RoleResourceCollection = None
        class SamlProviderResourceCollection(List[SamlProvider], ResourceCollection):
            pass

        saml_providers: SamlProviderResourceCollection = None
        class ServerCertificateResourceCollection(List[ServerCertificate], ResourceCollection):
            pass

        server_certificates: ServerCertificateResourceCollection = None
        class UserResourceCollection(List[User], ResourceCollection):
            pass

        users: UserResourceCollection = None
        class VirtualMfaDeviceResourceCollection(List[VirtualMfaDevice], ResourceCollection):
            pass

        virtual_mfa_devices: VirtualMfaDeviceResourceCollection = None




class ImportExport(BaseClient):
    """<fullname>AWS Import/Export Service</fullname> AWS Import/Export accelerates transferring large amounts of data between the AWS cloud and portable storage devices that you mail to us. AWS Import/Export transfers data directly onto and off of your storage devices using Amazon's high-speed internal network and bypassing the Internet. For large data sets, AWS Import/Export is often faster than Internet transfer and more cost effective than upgrading your connectivity."""


    class ListJobsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_jobs` for valid parameters"""
            pass


    class CancelJobInput(Mapping):
        pass
    
    class CancelJobOutput(Mapping):
        pass
    
    class CreateJobInput(Mapping):
        pass
    
    class CreateJobOutput(Mapping):
        pass
    
    class ValidateOnly(object):
        pass
    
    class GetShippingLabelInput(Mapping):
        pass
    
    class GetShippingLabelOutput(Mapping):
        pass
    
    class GenericString(object):
        pass
    
    class GetStatusInput(Mapping):
        pass
    
    class GetStatusOutput(Mapping):
        pass
    
    class ListJobsInput(Mapping):
        pass
    
    class ListJobsOutput(Mapping):
        pass
    
    class UpdateJobInput(Mapping):
        pass
    
    class UpdateJobOutput(Mapping):
        pass
    
    def cancel_job(self, JobId: str, APIVersion: str=None) -> CancelJobOutput:
        """This operation cancels a specified job. Only the job owner can cancel it. The operation fails if the job has already started or is complete.

        :param self:
        :param APIVersion:  Specifies the version of the client tool.<br/>JobId:  A unique identifier which refers to a particular job.<br/>
        :return: Success:  Specifies whether (true) or not (false) AWS Import/Export updated your job.<br/>        """
        pass

    def create_job(self, ValidateOnly: ValidateOnly, Manifest: str, JobType: str, ManifestAddendum: str=None, APIVersion: str=None) -> CreateJobOutput:
        """This operation initiates the process of scheduling an upload or download of your data. You include in the request a manifest that describes the data transfer specifics. The response to the request includes a job ID, which you can use in other operations, a signature that you use to identify your storage device, and the address where you should ship your storage device.

        :param self:
        :param APIVersion:  Specifies the version of the client tool.<br/>ValidateOnly:  Validate the manifest and parameter values in the request but do not actually create a job.<br/>ManifestAddendum:  For internal use only.<br/>Manifest:  The UTF-8 encoded text of the manifest file.<br/>JobType:  Specifies whether the job to initiate is an import or export job.<br/>
        :return: ArtifactList:  A collection of artifacts.<br/>WarningMessage:  An optional message notifying you of non-fatal issues with the job, such as use of an incompatible Amazon S3 bucket name.<br/>SignatureFileContents:  The actual text of the SIGNATURE file to be written to disk.<br/>Signature:  An encrypted code used to authenticate the request and response, for example, "DV+TpDfx1/TdSE9ktyK9k/bDTVI=". Only use this value is you want to create the signature file yourself. Generally you should use the SignatureFileContents value.<br/>JobType:  Specifies whether the job to initiate is an import or export job.<br/>JobId:  A unique identifier which refers to a particular job.<br/>        """
        pass

    def get_shipping_label(self, jobIds: List[GenericString], name: str=None, company: str=None, phoneNumber: str=None, country: str=None, stateOrProvince: str=None, city: str=None, postalCode: str=None, street1: str=None, street2: str=None, street3: str=None, APIVersion: str=None) -> GetShippingLabelOutput:
        """This operation generates a pre-paid UPS shipping label that you will use to ship your device to AWS for processing.

        :param self:
        :param APIVersion:  Specifies the version of the client tool.<br/>street3:  Specifies the optional third part of the street address for the return address, for example c/o Jane Doe.<br/>street2:  Specifies the optional second part of the street address for the return address, for example Suite 100.<br/>street1:  Specifies the first part of the street address for the return address, for example 1234 Main Street.<br/>postalCode:  Specifies the postal code for the return address.<br/>city:  Specifies the name of your city for the return address.<br/>stateOrProvince:  Specifies the name of your state or your province for the return address.<br/>country:  Specifies the name of your country for the return address.<br/>phoneNumber:  Specifies the phone number of the person responsible for shipping this package.<br/>company:  Specifies the name of the company that will ship this package.<br/>name:  Specifies the name of the person responsible for shipping this package.<br/>jobIds:  <br/>
        :return: Warning:  <br/>ShippingLabelURL:  <br/>        """
        pass

    def get_status(self, JobId: str, APIVersion: str=None) -> GetStatusOutput:
        """This operation returns information about a job, including where the job is in the processing pipeline, the status of the results, and the signature value associated with the job. You can only return information about jobs you own.

        :param self:
        :param APIVersion:  Specifies the version of the client tool.<br/>JobId:  A unique identifier which refers to a particular job.<br/>
        :return: ArtifactList:  A collection of artifacts.<br/>CreationDate:  Timestamp of the CreateJob request in ISO8601 date format. For example "2010-03-28T20:27:35Z".<br/>CurrentManifest:  The last manifest submitted, which will be used to process the job.<br/>SignatureFileContents:  An encrypted code used to authenticate the request and response, for example, "DV+TpDfx1/TdSE9ktyK9k/bDTVI=". Only use this value is you want to create the signature file yourself. Generally you should use the SignatureFileContents value.<br/>Signature:  An encrypted code used to authenticate the request and response, for example, "DV+TpDfx1/TdSE9ktyK9k/bDTVI=". Only use this value is you want to create the signature file yourself. Generally you should use the SignatureFileContents value.<br/>ErrorCount:  Number of errors. We return this value when the ProgressCode is Success or SuccessWithErrors.<br/>LogKey:  The key where the user logs were stored.<br/>LogBucket:  Amazon S3 bucket for user logs.<br/>TrackingNumber:  The shipping tracking number assigned by AWS Import/Export to the storage device when it's returned to you. We return this value when the LocationCode is "Returned".<br/>Carrier:  Name of the shipping company. This value is included when the LocationCode is "Returned".<br/>ProgressMessage:  A more human readable form of the job status.<br/>ProgressCode:  A token representing the state of the job, such as "Started".<br/>LocationMessage:  A more human readable form of the physical location of the storage device.<br/>LocationCode:  A token representing the location of the storage device, such as "AtAWS".<br/>JobType:  Specifies whether the job to initiate is an import or export job.<br/>JobId:  A unique identifier which refers to a particular job.<br/>        """
        pass

    def list_jobs(self, MaxJobs: int=None, Marker: str=None, APIVersion: str=None) -> ListJobsOutput:
        """This operation returns the jobs associated with the requester. AWS Import/Export lists the jobs in reverse chronological order based on the date of creation. For example if Job Test1 was created 2009Dec30 and Test2 was created 2010Feb05, the ListJobs operation would return Test2 followed by Test1.

        :param self:
        :param APIVersion:  Specifies the version of the client tool.<br/>Marker:  Specifies the JOBID to start after when listing the jobs created with your account. AWS Import/Export lists your jobs in reverse chronological order. See MaxJobs.<br/>MaxJobs:  Sets the maximum number of jobs returned in the response. If there are additional jobs that were not returned because MaxJobs was exceeded, the response contains &lt;IsTruncated&gt;true&lt;/IsTruncated&gt;. To return the additional jobs, see Marker.<br/>
        :return: IsTruncated:  Indicates whether the list of jobs was truncated. If true, then call ListJobs again using the last JobId element as the marker.<br/>Jobs:  A list container for Jobs returned by the ListJobs operation.<br/>        """
        pass

    def update_job(self, ValidateOnly: ValidateOnly, JobType: str, Manifest: str, JobId: str, APIVersion: str=None) -> UpdateJobOutput:
        """You use this operation to change the parameters specified in the original manifest file by supplying a new manifest file. The manifest file attached to this request replaces the original manifest file. You can only use the operation after a CreateJob request but before the data transfer starts and you can only use it on jobs you own.

        :param self:
        :param APIVersion:  Specifies the version of the client tool.<br/>ValidateOnly:  Validate the manifest and parameter values in the request but do not actually create a job.<br/>JobType:  Specifies whether the job to initiate is an import or export job.<br/>Manifest:  The UTF-8 encoded text of the manifest file.<br/>JobId:  A unique identifier which refers to a particular job.<br/>
        :return: ArtifactList:  A collection of artifacts.<br/>WarningMessage:  An optional message notifying you of non-fatal issues with the job, such as use of an incompatible Amazon S3 bucket name.<br/>Success:  Specifies whether (true) or not (false) AWS Import/Export updated your job.<br/>        """
        pass



class Inspector(BaseClient):
    """<fullname>Amazon Inspector</fullname> Amazon Inspector enables you to analyze the behavior of your AWS resources and to identify potential security issues. For more information, see <a href="https://docs.aws.amazon.com/inspector/latest/userguide/inspector_introduction.html"> Amazon Inspector User Guide</a>."""


    class ListFindingsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_findings` for valid parameters"""
            pass
    class ListAssessmentTemplatesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_assessment_templates` for valid parameters"""
            pass
    class PreviewAgentsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `preview_agents` for valid parameters"""
            pass
    class ListEventSubscriptionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_event_subscriptions` for valid parameters"""
            pass
    class ListRulesPackagesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_rules_packages` for valid parameters"""
            pass
    class ListAssessmentRunAgentsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_assessment_run_agents` for valid parameters"""
            pass
    class ListAssessmentRunsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_assessment_runs` for valid parameters"""
            pass
    class ListAssessmentTargetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_assessment_targets` for valid parameters"""
            pass
    class ListExclusionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_exclusions` for valid parameters"""
            pass


    class AddAttributesToFindingsRequest(Mapping):
        pass
    
    class AddAttributesToFindingsResponse(Mapping):
        pass
    
    class Arn(object):
        pass
    
    class Attribute(Mapping):
        pass
    
    class CreateAssessmentTargetRequest(Mapping):
        pass
    
    class CreateAssessmentTargetResponse(Mapping):
        pass
    
    class CreateAssessmentTemplateRequest(Mapping):
        pass
    
    class CreateAssessmentTemplateResponse(Mapping):
        pass
    
    class CreateExclusionsPreviewRequest(Mapping):
        pass
    
    class CreateExclusionsPreviewResponse(Mapping):
        pass
    
    class CreateResourceGroupRequest(Mapping):
        pass
    
    class CreateResourceGroupResponse(Mapping):
        pass
    
    class ResourceGroupTag(Mapping):
        pass
    
    class DeleteAssessmentRunRequest(Mapping):
        pass
    
    class DeleteAssessmentTargetRequest(Mapping):
        pass
    
    class DeleteAssessmentTemplateRequest(Mapping):
        pass
    
    class DescribeAssessmentRunsRequest(Mapping):
        pass
    
    class DescribeAssessmentRunsResponse(Mapping):
        pass
    
    class DescribeAssessmentTargetsRequest(Mapping):
        pass
    
    class DescribeAssessmentTargetsResponse(Mapping):
        pass
    
    class DescribeAssessmentTemplatesRequest(Mapping):
        pass
    
    class DescribeAssessmentTemplatesResponse(Mapping):
        pass
    
    class DescribeCrossAccountAccessRoleResponse(Mapping):
        pass
    
    class DescribeExclusionsRequest(Mapping):
        pass
    
    class DescribeExclusionsResponse(Mapping):
        pass
    
    class DescribeFindingsRequest(Mapping):
        pass
    
    class DescribeFindingsResponse(Mapping):
        pass
    
    class DescribeResourceGroupsRequest(Mapping):
        pass
    
    class DescribeResourceGroupsResponse(Mapping):
        pass
    
    class DescribeRulesPackagesRequest(Mapping):
        pass
    
    class DescribeRulesPackagesResponse(Mapping):
        pass
    
    class GetAssessmentReportRequest(Mapping):
        pass
    
    class GetAssessmentReportResponse(Mapping):
        pass
    
    class GetExclusionsPreviewRequest(Mapping):
        pass
    
    class GetExclusionsPreviewResponse(Mapping):
        pass
    
    class GetTelemetryMetadataRequest(Mapping):
        pass
    
    class GetTelemetryMetadataResponse(Mapping):
        pass
    
    class ListAssessmentRunAgentsRequest(Mapping):
        pass
    
    class ListAssessmentRunAgentsResponse(Mapping):
        pass
    
    class AgentFilter(Mapping):
        pass
    
    class ListAssessmentRunsRequest(Mapping):
        pass
    
    class ListAssessmentRunsResponse(Mapping):
        pass
    
    class AssessmentRunFilter(Mapping):
        pass
    
    class ListAssessmentTargetsRequest(Mapping):
        pass
    
    class ListAssessmentTargetsResponse(Mapping):
        pass
    
    class AssessmentTargetFilter(Mapping):
        pass
    
    class ListAssessmentTemplatesRequest(Mapping):
        pass
    
    class ListAssessmentTemplatesResponse(Mapping):
        pass
    
    class AssessmentTemplateFilter(Mapping):
        pass
    
    class ListEventSubscriptionsRequest(Mapping):
        pass
    
    class ListEventSubscriptionsResponse(Mapping):
        pass
    
    class ListExclusionsRequest(Mapping):
        pass
    
    class ListExclusionsResponse(Mapping):
        pass
    
    class ListFindingsRequest(Mapping):
        pass
    
    class ListFindingsResponse(Mapping):
        pass
    
    class FindingFilter(Mapping):
        pass
    
    class ListRulesPackagesRequest(Mapping):
        pass
    
    class ListRulesPackagesResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class PreviewAgentsRequest(Mapping):
        pass
    
    class PreviewAgentsResponse(Mapping):
        pass
    
    class RegisterCrossAccountAccessRoleRequest(Mapping):
        pass
    
    class RemoveAttributesFromFindingsRequest(Mapping):
        pass
    
    class RemoveAttributesFromFindingsResponse(Mapping):
        pass
    
    class AttributeKey(object):
        pass
    
    class SetTagsForResourceRequest(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class StartAssessmentRunRequest(Mapping):
        pass
    
    class StartAssessmentRunResponse(Mapping):
        pass
    
    class StopAssessmentRunRequest(Mapping):
        pass
    
    class SubscribeToEventRequest(Mapping):
        pass
    
    class UnsubscribeFromEventRequest(Mapping):
        pass
    
    class UpdateAssessmentTargetRequest(Mapping):
        pass
    
    def add_attributes_to_findings(self, attributes: List[Attribute], findingArns: List[Arn]) -> AddAttributesToFindingsResponse:
        """Assigns attributes (key and value pairs) to the findings that are specified by the ARNs of the findings.

        :param self:
        :param attributes:  The array of attributes that you want to assign to specified findings.<br/>findingArns:  The ARNs that specify the findings that you want to assign attributes to.<br/>
        :return: failedItems:  Attribute details that cannot be described. An error code is provided for each failed item.<br/>        """
        pass

    def create_assessment_target(self, assessmentTargetName: str, resourceGroupArn: str=None) -> CreateAssessmentTargetResponse:
        """Creates a new assessment target using the ARN of the resource group that is generated by <a>CreateResourceGroup</a>. If resourceGroupArn is not specified, all EC2 instances in the current AWS account and region are included in the assessment target. If the <a href="https://docs.aws.amazon.com/inspector/latest/userguide/inspector_slr.html">service-linked role</a> isn’t already registered, this action also creates and registers a service-linked role to grant Amazon Inspector access to AWS Services needed to perform security assessments. You can create up to 50 assessment targets per AWS account. You can run up to 500 concurrent agents per AWS account. For more information, see <a href="https://docs.aws.amazon.com/inspector/latest/userguide/inspector_applications.html"> Amazon Inspector Assessment Targets</a>.

        :param self:
        :param resourceGroupArn:  The ARN that specifies the resource group that is used to create the assessment target. If resourceGroupArn is not specified, all EC2 instances in the current AWS account and region are included in the assessment target.<br/>assessmentTargetName:  The user-defined name that identifies the assessment target that you want to create. The name must be unique within the AWS account.<br/>
        :return: assessmentTargetArn:  The ARN that specifies the assessment target that is created.<br/>        """
        pass

    def create_assessment_template(self, rulesPackageArns: List[Arn], durationInSeconds: int, assessmentTemplateName: str, assessmentTargetArn: str, userAttributesForFindings: List[Attribute]=None) -> CreateAssessmentTemplateResponse:
        """Creates an assessment template for the assessment target that is specified by the ARN of the assessment target. If the <a href="https://docs.aws.amazon.com/inspector/latest/userguide/inspector_slr.html">service-linked role</a> isn’t already registered, this action also creates and registers a service-linked role to grant Amazon Inspector access to AWS Services needed to perform security assessments.

        :param self:
        :param userAttributesForFindings:  The user-defined attributes that are assigned to every finding that is generated by the assessment run that uses this assessment template. An attribute is a key and value pair (an <a>Attribute</a> object). Within an assessment template, each key must be unique.<br/>rulesPackageArns:  The ARNs that specify the rules packages that you want to attach to the assessment template.<br/>durationInSeconds:  The duration of the assessment run in seconds.<br/>assessmentTemplateName:  The user-defined name that identifies the assessment template that you want to create. You can create several assessment templates for an assessment target. The names of the assessment templates that correspond to a particular assessment target must be unique.<br/>assessmentTargetArn:  The ARN that specifies the assessment target for which you want to create the assessment template.<br/>
        :return: assessmentTemplateArn:  The ARN that specifies the assessment template that is created.<br/>        """
        pass

    def create_exclusions_preview(self, assessmentTemplateArn: str) -> CreateExclusionsPreviewResponse:
        """Starts the generation of an exclusions preview for the specified assessment template. The exclusions preview lists the potential exclusions (ExclusionPreview) that Inspector can detect before it runs the assessment. 

        :param self:
        :param assessmentTemplateArn:  The ARN that specifies the assessment template for which you want to create an exclusions preview.<br/>
        :return: previewToken:  Specifies the unique identifier of the requested exclusions preview. You can use the unique identifier to retrieve the exclusions preview when running the GetExclusionsPreview API.<br/>        """
        pass

    def create_resource_group(self, resourceGroupTags: List[ResourceGroupTag]) -> CreateResourceGroupResponse:
        """Creates a resource group using the specified set of tags (key and value pairs) that are used to select the EC2 instances to be included in an Amazon Inspector assessment target. The created resource group is then used to create an Amazon Inspector assessment target. For more information, see <a>CreateAssessmentTarget</a>.

        :param self:
        :param resourceGroupTags:  A collection of keys and an array of possible values, '[{"key":"key1","values":["Value1","Value2"]},{"key":"Key2","values":["Value3"]}]'. For example,'[{"key":"Name","values":["TestEC2Instance"]}]'.<br/>
        :return: resourceGroupArn:  The ARN that specifies the resource group that is created.<br/>        """
        pass

    def delete_assessment_run(self, assessmentRunArn: str):
        """Deletes the assessment run that is specified by the ARN of the assessment run.

        :param self:
        :param assessmentRunArn:  The ARN that specifies the assessment run that you want to delete.<br/>
        :return:         """
        pass

    def delete_assessment_target(self, assessmentTargetArn: str):
        """Deletes the assessment target that is specified by the ARN of the assessment target.

        :param self:
        :param assessmentTargetArn:  The ARN that specifies the assessment target that you want to delete.<br/>
        :return:         """
        pass

    def delete_assessment_template(self, assessmentTemplateArn: str):
        """Deletes the assessment template that is specified by the ARN of the assessment template.

        :param self:
        :param assessmentTemplateArn:  The ARN that specifies the assessment template that you want to delete.<br/>
        :return:         """
        pass

    def describe_assessment_runs(self, assessmentRunArns: List[Arn]) -> DescribeAssessmentRunsResponse:
        """Describes the assessment runs that are specified by the ARNs of the assessment runs.

        :param self:
        :param assessmentRunArns:  The ARN that specifies the assessment run that you want to describe.<br/>
        :return: failedItems:  Assessment run details that cannot be described. An error code is provided for each failed item.<br/>assessmentRuns:  Information about the assessment run.<br/>        """
        pass

    def describe_assessment_targets(self, assessmentTargetArns: List[Arn]) -> DescribeAssessmentTargetsResponse:
        """Describes the assessment targets that are specified by the ARNs of the assessment targets.

        :param self:
        :param assessmentTargetArns:  The ARNs that specifies the assessment targets that you want to describe.<br/>
        :return: failedItems:  Assessment target details that cannot be described. An error code is provided for each failed item.<br/>assessmentTargets:  Information about the assessment targets.<br/>        """
        pass

    def describe_assessment_templates(self, assessmentTemplateArns: List[Arn]) -> DescribeAssessmentTemplatesResponse:
        """Describes the assessment templates that are specified by the ARNs of the assessment templates.

        :param self:
        :param assessmentTemplateArns:  <br/>
        :return: failedItems:  Assessment template details that cannot be described. An error code is provided for each failed item.<br/>assessmentTemplates:  Information about the assessment templates.<br/>        """
        pass

    def describe_cross_account_access_role(self) -> DescribeCrossAccountAccessRoleResponse:
        """Describes the IAM role that enables Amazon Inspector to access your AWS account.

        :param self:
        :param 
        :return: registeredAt:  The date when the cross-account access role was registered.<br/>valid:  A Boolean value that specifies whether the IAM role has the necessary policies attached to enable Amazon Inspector to access your AWS account.<br/>roleArn:  The ARN that specifies the IAM role that Amazon Inspector uses to access your AWS account.<br/>        """
        pass

    def describe_exclusions(self, exclusionArns: List[Arn], locale: str=None) -> DescribeExclusionsResponse:
        """Describes the exclusions that are specified by the exclusions' ARNs.

        :param self:
        :param locale:  The locale into which you want to translate the exclusion's title, description, and recommendation.<br/>exclusionArns:  The list of ARNs that specify the exclusions that you want to describe.<br/>
        :return: failedItems:  Exclusion details that cannot be described. An error code is provided for each failed item.<br/>exclusions:  Information about the exclusions.<br/>        """
        pass

    def describe_findings(self, findingArns: List[Arn], locale: str=None) -> DescribeFindingsResponse:
        """Describes the findings that are specified by the ARNs of the findings.

        :param self:
        :param locale:  The locale into which you want to translate a finding description, recommendation, and the short description that identifies the finding.<br/>findingArns:  The ARN that specifies the finding that you want to describe.<br/>
        :return: failedItems:  Finding details that cannot be described. An error code is provided for each failed item.<br/>findings:  Information about the finding.<br/>        """
        pass

    def describe_resource_groups(self, resourceGroupArns: List[Arn]) -> DescribeResourceGroupsResponse:
        """Describes the resource groups that are specified by the ARNs of the resource groups.

        :param self:
        :param resourceGroupArns:  The ARN that specifies the resource group that you want to describe.<br/>
        :return: failedItems:  Resource group details that cannot be described. An error code is provided for each failed item.<br/>resourceGroups:  Information about a resource group.<br/>        """
        pass

    def describe_rules_packages(self, rulesPackageArns: List[Arn], locale: str=None) -> DescribeRulesPackagesResponse:
        """Describes the rules packages that are specified by the ARNs of the rules packages.

        :param self:
        :param locale:  The locale that you want to translate a rules package description into.<br/>rulesPackageArns:  The ARN that specifies the rules package that you want to describe.<br/>
        :return: failedItems:  Rules package details that cannot be described. An error code is provided for each failed item.<br/>rulesPackages:  Information about the rules package.<br/>        """
        pass

    def get_assessment_report(self, reportType: str, reportFileFormat: str, assessmentRunArn: str) -> GetAssessmentReportResponse:
        """Produces an assessment report that includes detailed and comprehensive results of a specified assessment run. 

        :param self:
        :param reportType:  Specifies the type of the assessment report that you want to generate. There are two types of assessment reports: a finding report and a full report. For more information, see <a href="https://docs.aws.amazon.com/inspector/latest/userguide/inspector_reports.html">Assessment Reports</a>. <br/>reportFileFormat:  Specifies the file format (html or pdf) of the assessment report that you want to generate.<br/>assessmentRunArn:  The ARN that specifies the assessment run for which you want to generate a report.<br/>
        :return: url:  Specifies the URL where you can find the generated assessment report. This parameter is only returned if the report is successfully generated.<br/>status:  Specifies the status of the request to generate an assessment report. <br/>        """
        pass

    def get_exclusions_preview(self, previewToken: str, assessmentTemplateArn: str, nextToken: str=None, maxResults: int=None, locale: str=None) -> GetExclusionsPreviewResponse:
        """Retrieves the exclusions preview (a list of ExclusionPreview objects) specified by the preview token. You can obtain the preview token by running the CreateExclusionsPreview API.

        :param self:
        :param locale:  The locale into which you want to translate the exclusion's title, description, and recommendation.<br/>maxResults:  You can use this parameter to indicate the maximum number of items you want in the response. The default value is 100. The maximum value is 500.<br/>nextToken:  You can use this parameter when paginating results. Set the value of this parameter to null on your first call to the GetExclusionsPreviewRequest action. Subsequent calls to the action fill nextToken in the request with the value of nextToken from the previous response to continue listing data.<br/>previewToken:  The unique identifier associated of the exclusions preview.<br/>assessmentTemplateArn:  The ARN that specifies the assessment template for which the exclusions preview was requested.<br/>
        :return: nextToken:  When a response is generated, if there is more data to be listed, this parameters is present in the response and contains the value to use for the nextToken parameter in a subsequent pagination request. If there is no more data to be listed, this parameter is set to null.<br/>exclusionPreviews:  Information about the exclusions included in the preview.<br/>previewStatus:  Specifies the status of the request to generate an exclusions preview.<br/>        """
        pass

    def get_telemetry_metadata(self, assessmentRunArn: str) -> GetTelemetryMetadataResponse:
        """Information about the data that is collected for the specified assessment run.

        :param self:
        :param assessmentRunArn:  The ARN that specifies the assessment run that has the telemetry data that you want to obtain.<br/>
        :return: telemetryMetadata:  Telemetry details.<br/>        """
        pass

    def list_assessment_run_agents(self, assessmentRunArn: str, filter: AgentFilter=None, nextToken: str=None, maxResults: int=None) -> ListAssessmentRunAgentsResponse:
        """Lists the agents of the assessment runs that are specified by the ARNs of the assessment runs.

        :param self:
        :param maxResults:  You can use this parameter to indicate the maximum number of items that you want in the response. The default value is 10. The maximum value is 500.<br/>nextToken:  You can use this parameter when paginating results. Set the value of this parameter to null on your first call to the <b>ListAssessmentRunAgents</b> action. Subsequent calls to the action fill <b>nextToken</b> in the request with the value of <b>NextToken</b> from the previous response to continue listing data.<br/>filter:  You can use this parameter to specify a subset of data to be included in the action's response. For a record to match a filter, all specified filter attributes must match. When multiple values are specified for a filter attribute, any of the values can match.:<br/>assessmentRunArn:  The ARN that specifies the assessment run whose agents you want to list.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>agentHealthCodes</i> The detailed health state of the agent. Values can be set to <b>IDLE</b>, <b>RUNNING</b>, <b>SHUTDOWN</b>, <b>UNHEALTHY</b>, <b>THROTTLED</b>, and <b>UNKNOWN</b>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>agentHealths</i> The current health state of the agent. Values can be set to <b>HEALTHY</b> or <b>UNHEALTHY</b>.<br/>
        :return: nextToken:   When a response is generated, if there is more data to be listed, this parameter is present in the response and contains the value to use for the <b>nextToken</b> parameter in a subsequent pagination request. If there is no more data to be listed, this parameter is set to null.<br/>assessmentRunAgents:  A list of ARNs that specifies the agents returned by the action.<br/>        """
        pass

    def list_assessment_runs(self, assessmentTemplateArns: List[Arn]=None, filter: AssessmentRunFilter=None, nextToken: str=None, maxResults: int=None) -> ListAssessmentRunsResponse:
        """Lists the assessment runs that correspond to the assessment templates that are specified by the ARNs of the assessment templates.

        :param self:
        :param maxResults:  You can use this parameter to indicate the maximum number of items that you want in the response. The default value is 10. The maximum value is 500.<br/>nextToken:  You can use this parameter when paginating results. Set the value of this parameter to null on your first call to the <b>ListAssessmentRuns</b> action. Subsequent calls to the action fill <b>nextToken</b> in the request with the value of <b>NextToken</b> from the previous response to continue listing data.<br/>filter:  You can use this parameter to specify a subset of data to be included in the action's response. For a record to match a filter, all specified filter attributes must match. When multiple values are specified for a filter attribute, any of the values can match.:<br/>assessmentTemplateArns:  The ARNs that specify the assessment templates whose assessment runs you want to list.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>stateChangeTimeRange</i> For a record to match a filter, the value that is specified for this data type property must match the <b>stateChangedAt</b> property of the <a>AssessmentRun</a> data type.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>stateChangeTimeRange</b>: For a record to match a filter, the value that is specified for this data type property must match the <b>stateChangedAt</b> property of the <a>AssessmentRun</a> data type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>completionTimeRange</i> For a record to match a filter, the value that is specified for this data type property must inclusively match any value between the specified minimum and maximum values of the <b>completedAt</b> property of the <a>AssessmentRun</a> data type.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>completionTimeRange</b>: For a record to match a filter, the value that is specified for this data type property must inclusively match any value between the specified minimum and maximum values of the <b>completedAt</b> property of the <a>AssessmentRun</a> data type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>startTimeRange</i> For a record to match a filter, the value that is specified for this data type property must inclusively match any value between the specified minimum and maximum values of the <b>startTime</b> property of the <a>AssessmentRun</a> data type.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>startTimeRange</b>: For a record to match a filter, the value that is specified for this data type property must inclusively match any value between the specified minimum and maximum values of the <b>startTime</b> property of the <a>AssessmentRun</a> data type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>rulesPackageArns</i> For a record to match a filter, the value that is specified for this data type property must be contained in the list of values of the <b>rulesPackages</b> property of the <a>AssessmentRun</a> data type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>durationRange</i> For a record to match a filter, the value that is specified for this data type property must inclusively match any value between the specified minimum and maximum values of the <b>durationInSeconds</b> property of the <a>AssessmentRun</a> data type.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>durationRange</b>: For a record to match a filter, the value that is specified for this data type property must inclusively match any value between the specified minimum and maximum values of the <b>durationInSeconds</b> property of the <a>AssessmentRun</a> data type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>states</i> For a record to match a filter, one of the values specified for this data type property must be the exact match of the value of the <b>assessmentRunState</b> property of the <a>AssessmentRun</a> data type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>namePattern</i> For a record to match a filter, an explicit value or a string containing a wildcard that is specified for this data type property must match the value of the <b>assessmentRunName</b> property of the <a>AssessmentRun</a> data type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>maxSeconds</i> The maximum value of the duration range. Must be less than or equal to 604800 seconds (1 week).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>minSeconds</i> The minimum value of the duration range. Must be greater than zero.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>endDate</i> The maximum value of the timestamp range.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>beginDate</i> The minimum value of the timestamp range.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>endDate</i> The maximum value of the timestamp range.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>beginDate</i> The minimum value of the timestamp range.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>endDate</i> The maximum value of the timestamp range.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>beginDate</i> The minimum value of the timestamp range.<br/>
        :return: nextToken:   When a response is generated, if there is more data to be listed, this parameter is present in the response and contains the value to use for the <b>nextToken</b> parameter in a subsequent pagination request. If there is no more data to be listed, this parameter is set to null.<br/>assessmentRunArns:  A list of ARNs that specifies the assessment runs that are returned by the action.<br/>        """
        pass

    def list_assessment_targets(self, filter: AssessmentTargetFilter=None, nextToken: str=None, maxResults: int=None) -> ListAssessmentTargetsResponse:
        """Lists the ARNs of the assessment targets within this AWS account. For more information about assessment targets, see <a href="https://docs.aws.amazon.com/inspector/latest/userguide/inspector_applications.html">Amazon Inspector Assessment Targets</a>.

        :param self:
        :param maxResults:  You can use this parameter to indicate the maximum number of items you want in the response. The default value is 10. The maximum value is 500.<br/>nextToken:  You can use this parameter when paginating results. Set the value of this parameter to null on your first call to the <b>ListAssessmentTargets</b> action. Subsequent calls to the action fill <b>nextToken</b> in the request with the value of <b>NextToken</b> from the previous response to continue listing data.<br/>filter:  You can use this parameter to specify a subset of data to be included in the action's response. For a record to match a filter, all specified filter attributes must match. When multiple values are specified for a filter attribute, any of the values can match.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>assessmentTargetNamePattern</i> For a record to match a filter, an explicit value or a string that contains a wildcard that is specified for this data type property must match the value of the <b>assessmentTargetName</b> property of the <a>AssessmentTarget</a> data type.<br/>
        :return: nextToken:   When a response is generated, if there is more data to be listed, this parameter is present in the response and contains the value to use for the <b>nextToken</b> parameter in a subsequent pagination request. If there is no more data to be listed, this parameter is set to null.<br/>assessmentTargetArns:  A list of ARNs that specifies the assessment targets that are returned by the action.<br/>        """
        pass

    def list_assessment_templates(self, assessmentTargetArns: List[Arn]=None, filter: AssessmentTemplateFilter=None, nextToken: str=None, maxResults: int=None) -> ListAssessmentTemplatesResponse:
        """Lists the assessment templates that correspond to the assessment targets that are specified by the ARNs of the assessment targets.

        :param self:
        :param maxResults:  You can use this parameter to indicate the maximum number of items you want in the response. The default value is 10. The maximum value is 500.<br/>nextToken:  You can use this parameter when paginating results. Set the value of this parameter to null on your first call to the <b>ListAssessmentTemplates</b> action. Subsequent calls to the action fill <b>nextToken</b> in the request with the value of <b>NextToken</b> from the previous response to continue listing data.<br/>filter:  You can use this parameter to specify a subset of data to be included in the action's response. For a record to match a filter, all specified filter attributes must match. When multiple values are specified for a filter attribute, any of the values can match.:<br/>assessmentTargetArns:  A list of ARNs that specifies the assessment targets whose assessment templates you want to list.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>rulesPackageArns</i> For a record to match a filter, the values that are specified for this data type property must be contained in the list of values of the <b>rulesPackageArns</b> property of the <a>AssessmentTemplate</a> data type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>durationRange</i> For a record to match a filter, the value specified for this data type property must inclusively match any value between the specified minimum and maximum values of the <b>durationInSeconds</b> property of the <a>AssessmentTemplate</a> data type.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>durationRange</b>: For a record to match a filter, the value specified for this data type property must inclusively match any value between the specified minimum and maximum values of the <b>durationInSeconds</b> property of the <a>AssessmentTemplate</a> data type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>namePattern</i> For a record to match a filter, an explicit value or a string that contains a wildcard that is specified for this data type property must match the value of the <b>assessmentTemplateName</b> property of the <a>AssessmentTemplate</a> data type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>maxSeconds</i> The maximum value of the duration range. Must be less than or equal to 604800 seconds (1 week).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>minSeconds</i> The minimum value of the duration range. Must be greater than zero.<br/>
        :return: nextToken:   When a response is generated, if there is more data to be listed, this parameter is present in the response and contains the value to use for the <b>nextToken</b> parameter in a subsequent pagination request. If there is no more data to be listed, this parameter is set to null.<br/>assessmentTemplateArns:  A list of ARNs that specifies the assessment templates returned by the action.<br/>        """
        pass

    def list_event_subscriptions(self, resourceArn: str=None, nextToken: str=None, maxResults: int=None) -> ListEventSubscriptionsResponse:
        """Lists all the event subscriptions for the assessment template that is specified by the ARN of the assessment template. For more information, see <a>SubscribeToEvent</a> and <a>UnsubscribeFromEvent</a>.

        :param self:
        :param maxResults:  You can use this parameter to indicate the maximum number of items you want in the response. The default value is 10. The maximum value is 500.<br/>nextToken:  You can use this parameter when paginating results. Set the value of this parameter to null on your first call to the <b>ListEventSubscriptions</b> action. Subsequent calls to the action fill <b>nextToken</b> in the request with the value of <b>NextToken</b> from the previous response to continue listing data.<br/>resourceArn:  The ARN of the assessment template for which you want to list the existing event subscriptions.<br/>
        :return: nextToken:   When a response is generated, if there is more data to be listed, this parameter is present in the response and contains the value to use for the <b>nextToken</b> parameter in a subsequent pagination request. If there is no more data to be listed, this parameter is set to null.<br/>subscriptions:  Details of the returned event subscriptions.<br/>        """
        pass

    def list_exclusions(self, assessmentRunArn: str, nextToken: str=None, maxResults: int=None) -> ListExclusionsResponse:
        """List exclusions that are generated by the assessment run.

        :param self:
        :param maxResults:  You can use this parameter to indicate the maximum number of items you want in the response. The default value is 100. The maximum value is 500.<br/>nextToken:  You can use this parameter when paginating results. Set the value of this parameter to null on your first call to the ListExclusionsRequest action. Subsequent calls to the action fill nextToken in the request with the value of nextToken from the previous response to continue listing data.<br/>assessmentRunArn:  The ARN of the assessment run that generated the exclusions that you want to list.<br/>
        :return: nextToken:  When a response is generated, if there is more data to be listed, this parameters is present in the response and contains the value to use for the nextToken parameter in a subsequent pagination request. If there is no more data to be listed, this parameter is set to null.<br/>exclusionArns:  A list of exclusions' ARNs returned by the action.<br/>        """
        pass

    def list_findings(self, assessmentRunArns: List[Arn]=None, filter: FindingFilter=None, nextToken: str=None, maxResults: int=None) -> ListFindingsResponse:
        """Lists findings that are generated by the assessment runs that are specified by the ARNs of the assessment runs.

        :param self:
        :param maxResults:  You can use this parameter to indicate the maximum number of items you want in the response. The default value is 10. The maximum value is 500.<br/>nextToken:  You can use this parameter when paginating results. Set the value of this parameter to null on your first call to the <b>ListFindings</b> action. Subsequent calls to the action fill <b>nextToken</b> in the request with the value of <b>NextToken</b> from the previous response to continue listing data.<br/>filter:  You can use this parameter to specify a subset of data to be included in the action's response. For a record to match a filter, all specified filter attributes must match. When multiple values are specified for a filter attribute, any of the values can match.:<br/>assessmentRunArns:  The ARNs of the assessment runs that generate the findings that you want to list.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationTimeRange</i> The time range during which the finding is generated.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>creationTimeRange</b>: The time range during which the finding is generated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>userAttributes</i> For a record to match a filter, the value that is specified for this data type property must be contained in the list of values of the <b>userAttributes</b> property of the <a>Finding</a> data type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>attributes</i> For a record to match a filter, the list of values that are specified for this data type property must be contained in the list of values of the <b>attributes</b> property of the <a>Finding</a> data type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>rulesPackageArns</i> For a record to match a filter, one of the values that is specified for this data type property must be the exact match of the value of the <b>rulesPackageArn</b> property of the <a>Finding</a> data type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>severities</i> For a record to match a filter, one of the values that is specified for this data type property must be the exact match of the value of the <b>severity</b> property of the <a>Finding</a> data type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ruleNames</i> For a record to match a filter, one of the values that is specified for this data type property must be the exact match of the value of the <b>ruleName</b> property of the <a>Finding</a> data type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>autoScalingGroups</i> For a record to match a filter, one of the values that is specified for this data type property must be the exact match of the value of the <b>autoScalingGroup</b> property of the <a>Finding</a> data type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>agentIds</i> For a record to match a filter, one of the values that is specified for this data type property must be the exact match of the value of the <b>agentId</b> property of the <a>Finding</a> data type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>endDate</i> The maximum value of the timestamp range.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>beginDate</i> The minimum value of the timestamp range.<br/>
        :return: nextToken:   When a response is generated, if there is more data to be listed, this parameter is present in the response and contains the value to use for the <b>nextToken</b> parameter in a subsequent pagination request. If there is no more data to be listed, this parameter is set to null.<br/>findingArns:  A list of ARNs that specifies the findings returned by the action.<br/>        """
        pass

    def list_rules_packages(self, nextToken: str=None, maxResults: int=None) -> ListRulesPackagesResponse:
        """Lists all available Amazon Inspector rules packages.

        :param self:
        :param maxResults:  You can use this parameter to indicate the maximum number of items you want in the response. The default value is 10. The maximum value is 500.<br/>nextToken:  You can use this parameter when paginating results. Set the value of this parameter to null on your first call to the <b>ListRulesPackages</b> action. Subsequent calls to the action fill <b>nextToken</b> in the request with the value of <b>NextToken</b> from the previous response to continue listing data.<br/>
        :return: nextToken:   When a response is generated, if there is more data to be listed, this parameter is present in the response and contains the value to use for the <b>nextToken</b> parameter in a subsequent pagination request. If there is no more data to be listed, this parameter is set to null.<br/>rulesPackageArns:  The list of ARNs that specifies the rules packages returned by the action.<br/>        """
        pass

    def list_tags_for_resource(self, resourceArn: str) -> ListTagsForResourceResponse:
        """Lists all tags associated with an assessment template.

        :param self:
        :param resourceArn:  The ARN that specifies the assessment template whose tags you want to list.<br/>
        :return: tags:  A collection of key and value pairs.<br/>        """
        pass

    def preview_agents(self, previewAgentsArn: str, nextToken: str=None, maxResults: int=None) -> PreviewAgentsResponse:
        """Previews the agents installed on the EC2 instances that are part of the specified assessment target.

        :param self:
        :param maxResults:  You can use this parameter to indicate the maximum number of items you want in the response. The default value is 10. The maximum value is 500.<br/>nextToken:  You can use this parameter when paginating results. Set the value of this parameter to null on your first call to the <b>PreviewAgents</b> action. Subsequent calls to the action fill <b>nextToken</b> in the request with the value of <b>NextToken</b> from the previous response to continue listing data.<br/>previewAgentsArn:  The ARN of the assessment target whose agents you want to preview.<br/>
        :return: nextToken:   When a response is generated, if there is more data to be listed, this parameter is present in the response and contains the value to use for the <b>nextToken</b> parameter in a subsequent pagination request. If there is no more data to be listed, this parameter is set to null.<br/>agentPreviews:  The resulting list of agents.<br/>        """
        pass

    def register_cross_account_access_role(self, roleArn: str):
        """Registers the IAM role that grants Amazon Inspector access to AWS Services needed to perform security assessments.

        :param self:
        :param roleArn:  The ARN of the IAM role that grants Amazon Inspector access to AWS Services needed to perform security assessments. <br/>
        :return:         """
        pass

    def remove_attributes_from_findings(self, attributeKeys: List[AttributeKey], findingArns: List[Arn]) -> RemoveAttributesFromFindingsResponse:
        """Removes entire attributes (key and value pairs) from the findings that are specified by the ARNs of the findings where an attribute with the specified key exists.

        :param self:
        :param attributeKeys:  The array of attribute keys that you want to remove from specified findings.<br/>findingArns:  The ARNs that specify the findings that you want to remove attributes from.<br/>
        :return: failedItems:  Attributes details that cannot be described. An error code is provided for each failed item.<br/>        """
        pass

    def set_tags_for_resource(self, resourceArn: str, tags: List[Tag]=None):
        """Sets tags (key and value pairs) to the assessment template that is specified by the ARN of the assessment template.

        :param self:
        :param tags:  A collection of key and value pairs that you want to set to the assessment template.<br/>resourceArn:  The ARN of the assessment template that you want to set tags to.<br/>
        :return:         """
        pass

    def start_assessment_run(self, assessmentTemplateArn: str, assessmentRunName: str=None) -> StartAssessmentRunResponse:
        """Starts the assessment run specified by the ARN of the assessment template. For this API to function properly, you must not exceed the limit of running up to 500 concurrent agents per AWS account.

        :param self:
        :param assessmentRunName:  You can specify the name for the assessment run. The name must be unique for the assessment template whose ARN is used to start the assessment run.<br/>assessmentTemplateArn:  The ARN of the assessment template of the assessment run that you want to start.<br/>
        :return: assessmentRunArn:  The ARN of the assessment run that has been started.<br/>        """
        pass

    def stop_assessment_run(self, assessmentRunArn: str, stopAction: str=None):
        """Stops the assessment run that is specified by the ARN of the assessment run.

        :param self:
        :param stopAction:  An input option that can be set to either START_EVALUATION or SKIP_EVALUATION. START_EVALUATION (the default value), stops the AWS agent from collecting data and begins the results evaluation and the findings generation process. SKIP_EVALUATION cancels the assessment run immediately, after which no findings are generated.<br/>assessmentRunArn:  The ARN of the assessment run that you want to stop.<br/>
        :return:         """
        pass

    def subscribe_to_event(self, topicArn: str, event: str, resourceArn: str):
        """Enables the process of sending Amazon Simple Notification Service (SNS) notifications about a specified event to a specified SNS topic.

        :param self:
        :param topicArn:  The ARN of the SNS topic to which the SNS notifications are sent.<br/>event:  The event for which you want to receive SNS notifications.<br/>resourceArn:  The ARN of the assessment template that is used during the event for which you want to receive SNS notifications.<br/>
        :return:         """
        pass

    def unsubscribe_from_event(self, topicArn: str, event: str, resourceArn: str):
        """Disables the process of sending Amazon Simple Notification Service (SNS) notifications about a specified event to a specified SNS topic.

        :param self:
        :param topicArn:  The ARN of the SNS topic to which SNS notifications are sent.<br/>event:  The event for which you want to stop receiving SNS notifications.<br/>resourceArn:  The ARN of the assessment template that is used during the event for which you want to stop receiving SNS notifications.<br/>
        :return:         """
        pass

    def update_assessment_target(self, assessmentTargetName: str, assessmentTargetArn: str, resourceGroupArn: str=None):
        """Updates the assessment target that is specified by the ARN of the assessment target. If resourceGroupArn is not specified, all EC2 instances in the current AWS account and region are included in the assessment target.

        :param self:
        :param resourceGroupArn:  The ARN of the resource group that is used to specify the new resource group to associate with the assessment target.<br/>assessmentTargetName:  The name of the assessment target that you want to update.<br/>assessmentTargetArn:  The ARN of the assessment target that you want to update.<br/>
        :return:         """
        pass



class IoT(BaseClient):
    """<fullname>AWS IoT</fullname> AWS IoT provides secure, bi-directional communication between Internet-connected devices (such as sensors, actuators, embedded devices, or smart appliances) and the AWS cloud. You can discover your custom IoT-Data endpoint to communicate with, configure rules for data processing and integration with other services, organize resources associated with each device (Registry), configure logging, and create and manage policies and credentials to authenticate devices. For more information about how AWS IoT works, see the <a href="https://docs.aws.amazon.com/iot/latest/developerguide/aws-iot-how-it-works.html">Developer Guide</a>. For information about how to use the credentials provider for AWS IoT, see <a href="https://docs.aws.amazon.com/iot/latest/developerguide/authorizing-direct-aws.html">Authorizing Direct Calls to AWS Services</a>."""


    class ListCACertificatesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_ca_certificates` for valid parameters"""
            pass
    class ListCertificatesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_certificates` for valid parameters"""
            pass
    class ListCertificatesByCAPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_certificates_by_ca` for valid parameters"""
            pass
    class ListOutgoingCertificatesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_outgoing_certificates` for valid parameters"""
            pass
    class ListPoliciesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_policies` for valid parameters"""
            pass
    class ListPolicyPrincipalsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_policy_principals` for valid parameters"""
            pass
    class ListPrincipalPoliciesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_principal_policies` for valid parameters"""
            pass
    class ListPrincipalThingsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_principal_things` for valid parameters"""
            pass
    class ListThingTypesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_thing_types` for valid parameters"""
            pass
    class ListThingsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_things` for valid parameters"""
            pass
    class ListTopicRulesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_topic_rules` for valid parameters"""
            pass
    class ListActiveViolationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_active_violations` for valid parameters"""
            pass
    class ListAttachedPoliciesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_attached_policies` for valid parameters"""
            pass
    class ListAuditFindingsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_audit_findings` for valid parameters"""
            pass
    class ListAuditTasksPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_audit_tasks` for valid parameters"""
            pass
    class ListAuthorizersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_authorizers` for valid parameters"""
            pass
    class ListBillingGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_billing_groups` for valid parameters"""
            pass
    class ListIndicesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_indices` for valid parameters"""
            pass
    class ListJobExecutionsForJobPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_job_executions_for_job` for valid parameters"""
            pass
    class ListJobExecutionsForThingPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_job_executions_for_thing` for valid parameters"""
            pass
    class ListJobsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_jobs` for valid parameters"""
            pass
    class ListOTAUpdatesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_ota_updates` for valid parameters"""
            pass
    class ListRoleAliasesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_role_aliases` for valid parameters"""
            pass
    class ListScheduledAuditsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_scheduled_audits` for valid parameters"""
            pass
    class ListSecurityProfilesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_security_profiles` for valid parameters"""
            pass
    class ListSecurityProfilesForTargetPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_security_profiles_for_target` for valid parameters"""
            pass
    class ListStreamsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_streams` for valid parameters"""
            pass
    class ListTagsForResourcePaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_tags_for_resource` for valid parameters"""
            pass
    class ListTargetsForPolicyPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_targets_for_policy` for valid parameters"""
            pass
    class ListTargetsForSecurityProfilePaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_targets_for_security_profile` for valid parameters"""
            pass
    class ListThingGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_thing_groups` for valid parameters"""
            pass
    class ListThingGroupsForThingPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_thing_groups_for_thing` for valid parameters"""
            pass
    class ListThingRegistrationTasksPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_thing_registration_tasks` for valid parameters"""
            pass
    class ListThingsInBillingGroupPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_things_in_billing_group` for valid parameters"""
            pass
    class ListThingsInThingGroupPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_things_in_thing_group` for valid parameters"""
            pass
    class ListV2LoggingLevelsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_v2_logging_levels` for valid parameters"""
            pass
    class ListViolationEventsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_violation_events` for valid parameters"""
            pass


    class AcceptCertificateTransferRequest(Mapping):
        pass
    
    class SetAsActive(object):
        pass
    
    class AddThingToBillingGroupRequest(Mapping):
        pass
    
    class AddThingToBillingGroupResponse(Mapping):
        pass
    
    class AddThingToThingGroupRequest(Mapping):
        pass
    
    class AddThingToThingGroupResponse(Mapping):
        pass
    
    class OverrideDynamicGroups(object):
        pass
    
    class AssociateTargetsWithJobRequest(Mapping):
        pass
    
    class AssociateTargetsWithJobResponse(Mapping):
        pass
    
    class TargetArn(object):
        pass
    
    class AttachPolicyRequest(Mapping):
        pass
    
    class AttachPrincipalPolicyRequest(Mapping):
        pass
    
    class AttachSecurityProfileRequest(Mapping):
        pass
    
    class AttachSecurityProfileResponse(Mapping):
        pass
    
    class AttachThingPrincipalRequest(Mapping):
        pass
    
    class AttachThingPrincipalResponse(Mapping):
        pass
    
    class CancelAuditTaskRequest(Mapping):
        pass
    
    class CancelAuditTaskResponse(Mapping):
        pass
    
    class CancelCertificateTransferRequest(Mapping):
        pass
    
    class CancelJobRequest(Mapping):
        pass
    
    class CancelJobResponse(Mapping):
        pass
    
    class ForceFlag(object):
        pass
    
    class CancelJobExecutionRequest(Mapping):
        pass
    
    class ExpectedVersion(object):
        pass
    
    class DetailsMap(object):
        pass
    
    class ClearDefaultAuthorizerRequest(Mapping):
        pass
    
    class ClearDefaultAuthorizerResponse(Mapping):
        pass
    
    class CreateAuthorizerRequest(Mapping):
        pass
    
    class CreateAuthorizerResponse(Mapping):
        pass
    
    class PublicKeyMap(object):
        pass
    
    class CreateBillingGroupRequest(Mapping):
        pass
    
    class CreateBillingGroupResponse(Mapping):
        pass
    
    class BillingGroupProperties(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreateCertificateFromCsrRequest(Mapping):
        pass
    
    class CreateCertificateFromCsrResponse(Mapping):
        pass
    
    class CreateDynamicThingGroupRequest(Mapping):
        pass
    
    class CreateDynamicThingGroupResponse(Mapping):
        pass
    
    class ThingGroupProperties(Mapping):
        pass
    
    class CreateJobRequest(Mapping):
        pass
    
    class CreateJobResponse(Mapping):
        pass
    
    class PresignedUrlConfig(Mapping):
        pass
    
    class JobExecutionsRolloutConfig(Mapping):
        pass
    
    class AbortConfig(Mapping):
        pass
    
    class TimeoutConfig(Mapping):
        pass
    
    class CreateKeysAndCertificateRequest(Mapping):
        pass
    
    class CreateKeysAndCertificateResponse(Mapping):
        pass
    
    class CreateOTAUpdateRequest(Mapping):
        pass
    
    class CreateOTAUpdateResponse(Mapping):
        pass
    
    class Target(object):
        pass
    
    class AwsJobExecutionsRolloutConfig(Mapping):
        pass
    
    class OTAUpdateFile(Mapping):
        pass
    
    class AdditionalParameterMap(object):
        pass
    
    class CreatePolicyRequest(Mapping):
        pass
    
    class CreatePolicyResponse(Mapping):
        pass
    
    class CreatePolicyVersionRequest(Mapping):
        pass
    
    class CreatePolicyVersionResponse(Mapping):
        pass
    
    class SetAsDefault(object):
        pass
    
    class CreateRoleAliasRequest(Mapping):
        pass
    
    class CreateRoleAliasResponse(Mapping):
        pass
    
    class CreateScheduledAuditRequest(Mapping):
        pass
    
    class CreateScheduledAuditResponse(Mapping):
        pass
    
    class AuditCheckName(object):
        pass
    
    class CreateSecurityProfileRequest(Mapping):
        pass
    
    class CreateSecurityProfileResponse(Mapping):
        pass
    
    class Behavior(Mapping):
        pass
    
    class AlertTargets(object):
        pass
    
    class BehaviorMetric(object):
        pass
    
    class CreateStreamRequest(Mapping):
        pass
    
    class CreateStreamResponse(Mapping):
        pass
    
    class StreamFile(Mapping):
        pass
    
    class CreateThingRequest(Mapping):
        pass
    
    class CreateThingResponse(Mapping):
        pass
    
    class AttributePayload(Mapping):
        pass
    
    class CreateThingGroupRequest(Mapping):
        pass
    
    class CreateThingGroupResponse(Mapping):
        pass
    
    class CreateThingTypeRequest(Mapping):
        pass
    
    class CreateThingTypeResponse(Mapping):
        pass
    
    class ThingTypeProperties(Mapping):
        pass
    
    class CreateTopicRuleRequest(Mapping):
        pass
    
    class TopicRulePayload(Mapping):
        pass
    
    class DeleteAccountAuditConfigurationRequest(Mapping):
        pass
    
    class DeleteAccountAuditConfigurationResponse(Mapping):
        pass
    
    class DeleteScheduledAudits(object):
        pass
    
    class DeleteAuthorizerRequest(Mapping):
        pass
    
    class DeleteAuthorizerResponse(Mapping):
        pass
    
    class DeleteBillingGroupRequest(Mapping):
        pass
    
    class DeleteBillingGroupResponse(Mapping):
        pass
    
    class OptionalVersion(object):
        pass
    
    class DeleteCACertificateRequest(Mapping):
        pass
    
    class DeleteCACertificateResponse(Mapping):
        pass
    
    class DeleteCertificateRequest(Mapping):
        pass
    
    class ForceDelete(object):
        pass
    
    class DeleteDynamicThingGroupRequest(Mapping):
        pass
    
    class DeleteDynamicThingGroupResponse(Mapping):
        pass
    
    class DeleteJobRequest(Mapping):
        pass
    
    class DeleteJobExecutionRequest(Mapping):
        pass
    
    class ExecutionNumber(object):
        pass
    
    class DeleteOTAUpdateRequest(Mapping):
        pass
    
    class DeleteOTAUpdateResponse(Mapping):
        pass
    
    class DeleteStream(object):
        pass
    
    class ForceDeleteAWSJob(object):
        pass
    
    class DeletePolicyRequest(Mapping):
        pass
    
    class DeletePolicyVersionRequest(Mapping):
        pass
    
    class DeleteRegistrationCodeRequest(Mapping):
        pass
    
    class DeleteRegistrationCodeResponse(Mapping):
        pass
    
    class DeleteRoleAliasRequest(Mapping):
        pass
    
    class DeleteRoleAliasResponse(Mapping):
        pass
    
    class DeleteScheduledAuditRequest(Mapping):
        pass
    
    class DeleteScheduledAuditResponse(Mapping):
        pass
    
    class DeleteSecurityProfileRequest(Mapping):
        pass
    
    class DeleteSecurityProfileResponse(Mapping):
        pass
    
    class DeleteStreamRequest(Mapping):
        pass
    
    class DeleteStreamResponse(Mapping):
        pass
    
    class DeleteThingRequest(Mapping):
        pass
    
    class DeleteThingResponse(Mapping):
        pass
    
    class DeleteThingGroupRequest(Mapping):
        pass
    
    class DeleteThingGroupResponse(Mapping):
        pass
    
    class DeleteThingTypeRequest(Mapping):
        pass
    
    class DeleteThingTypeResponse(Mapping):
        pass
    
    class DeleteTopicRuleRequest(Mapping):
        pass
    
    class DeleteV2LoggingLevelRequest(Mapping):
        pass
    
    class DeprecateThingTypeRequest(Mapping):
        pass
    
    class DeprecateThingTypeResponse(Mapping):
        pass
    
    class UndoDeprecate(object):
        pass
    
    class DescribeAccountAuditConfigurationRequest(Mapping):
        pass
    
    class DescribeAccountAuditConfigurationResponse(Mapping):
        pass
    
    class DescribeAuditTaskRequest(Mapping):
        pass
    
    class DescribeAuditTaskResponse(Mapping):
        pass
    
    class DescribeAuthorizerRequest(Mapping):
        pass
    
    class DescribeAuthorizerResponse(Mapping):
        pass
    
    class DescribeBillingGroupRequest(Mapping):
        pass
    
    class DescribeBillingGroupResponse(Mapping):
        pass
    
    class DescribeCACertificateRequest(Mapping):
        pass
    
    class DescribeCACertificateResponse(Mapping):
        pass
    
    class DescribeCertificateRequest(Mapping):
        pass
    
    class DescribeCertificateResponse(Mapping):
        pass
    
    class DescribeDefaultAuthorizerRequest(Mapping):
        pass
    
    class DescribeDefaultAuthorizerResponse(Mapping):
        pass
    
    class DescribeEndpointRequest(Mapping):
        pass
    
    class DescribeEndpointResponse(Mapping):
        pass
    
    class DescribeEventConfigurationsRequest(Mapping):
        pass
    
    class DescribeEventConfigurationsResponse(Mapping):
        pass
    
    class DescribeIndexRequest(Mapping):
        pass
    
    class DescribeIndexResponse(Mapping):
        pass
    
    class DescribeJobRequest(Mapping):
        pass
    
    class DescribeJobResponse(Mapping):
        pass
    
    class DescribeJobExecutionRequest(Mapping):
        pass
    
    class DescribeJobExecutionResponse(Mapping):
        pass
    
    class DescribeRoleAliasRequest(Mapping):
        pass
    
    class DescribeRoleAliasResponse(Mapping):
        pass
    
    class DescribeScheduledAuditRequest(Mapping):
        pass
    
    class DescribeScheduledAuditResponse(Mapping):
        pass
    
    class DescribeSecurityProfileRequest(Mapping):
        pass
    
    class DescribeSecurityProfileResponse(Mapping):
        pass
    
    class DescribeStreamRequest(Mapping):
        pass
    
    class DescribeStreamResponse(Mapping):
        pass
    
    class DescribeThingRequest(Mapping):
        pass
    
    class DescribeThingResponse(Mapping):
        pass
    
    class DescribeThingGroupRequest(Mapping):
        pass
    
    class DescribeThingGroupResponse(Mapping):
        pass
    
    class DescribeThingRegistrationTaskRequest(Mapping):
        pass
    
    class DescribeThingRegistrationTaskResponse(Mapping):
        pass
    
    class DescribeThingTypeRequest(Mapping):
        pass
    
    class DescribeThingTypeResponse(Mapping):
        pass
    
    class DetachPolicyRequest(Mapping):
        pass
    
    class DetachPrincipalPolicyRequest(Mapping):
        pass
    
    class DetachSecurityProfileRequest(Mapping):
        pass
    
    class DetachSecurityProfileResponse(Mapping):
        pass
    
    class DetachThingPrincipalRequest(Mapping):
        pass
    
    class DetachThingPrincipalResponse(Mapping):
        pass
    
    class DisableTopicRuleRequest(Mapping):
        pass
    
    class EnableTopicRuleRequest(Mapping):
        pass
    
    class GetEffectivePoliciesRequest(Mapping):
        pass
    
    class GetEffectivePoliciesResponse(Mapping):
        pass
    
    class GetIndexingConfigurationRequest(Mapping):
        pass
    
    class GetIndexingConfigurationResponse(Mapping):
        pass
    
    class GetJobDocumentRequest(Mapping):
        pass
    
    class GetJobDocumentResponse(Mapping):
        pass
    
    class GetLoggingOptionsRequest(Mapping):
        pass
    
    class GetLoggingOptionsResponse(Mapping):
        pass
    
    class GetOTAUpdateRequest(Mapping):
        pass
    
    class GetOTAUpdateResponse(Mapping):
        pass
    
    class GetPolicyRequest(Mapping):
        pass
    
    class GetPolicyResponse(Mapping):
        pass
    
    class GetPolicyVersionRequest(Mapping):
        pass
    
    class GetPolicyVersionResponse(Mapping):
        pass
    
    class GetRegistrationCodeRequest(Mapping):
        pass
    
    class GetRegistrationCodeResponse(Mapping):
        pass
    
    class GetStatisticsRequest(Mapping):
        pass
    
    class GetStatisticsResponse(Mapping):
        pass
    
    class GetTopicRuleRequest(Mapping):
        pass
    
    class GetTopicRuleResponse(Mapping):
        pass
    
    class GetV2LoggingOptionsRequest(Mapping):
        pass
    
    class GetV2LoggingOptionsResponse(Mapping):
        pass
    
    class ListActiveViolationsRequest(Mapping):
        pass
    
    class ListActiveViolationsResponse(Mapping):
        pass
    
    class ListAttachedPoliciesRequest(Mapping):
        pass
    
    class ListAttachedPoliciesResponse(Mapping):
        pass
    
    class Recursive(object):
        pass
    
    class ListAuditFindingsRequest(Mapping):
        pass
    
    class ListAuditFindingsResponse(Mapping):
        pass
    
    class ResourceIdentifier(Mapping):
        pass
    
    class Timestamp(object):
        pass
    
    class ListAuditTasksRequest(Mapping):
        pass
    
    class ListAuditTasksResponse(Mapping):
        pass
    
    class ListAuthorizersRequest(Mapping):
        pass
    
    class ListAuthorizersResponse(Mapping):
        pass
    
    class AscendingOrder(object):
        pass
    
    class ListBillingGroupsRequest(Mapping):
        pass
    
    class ListBillingGroupsResponse(Mapping):
        pass
    
    class ListCACertificatesRequest(Mapping):
        pass
    
    class ListCACertificatesResponse(Mapping):
        pass
    
    class ListCertificatesRequest(Mapping):
        pass
    
    class ListCertificatesResponse(Mapping):
        pass
    
    class ListCertificatesByCARequest(Mapping):
        pass
    
    class ListCertificatesByCAResponse(Mapping):
        pass
    
    class ListIndicesRequest(Mapping):
        pass
    
    class ListIndicesResponse(Mapping):
        pass
    
    class ListJobExecutionsForJobRequest(Mapping):
        pass
    
    class ListJobExecutionsForJobResponse(Mapping):
        pass
    
    class ListJobExecutionsForThingRequest(Mapping):
        pass
    
    class ListJobExecutionsForThingResponse(Mapping):
        pass
    
    class ListJobsRequest(Mapping):
        pass
    
    class ListJobsResponse(Mapping):
        pass
    
    class ListOTAUpdatesRequest(Mapping):
        pass
    
    class ListOTAUpdatesResponse(Mapping):
        pass
    
    class ListOutgoingCertificatesRequest(Mapping):
        pass
    
    class ListOutgoingCertificatesResponse(Mapping):
        pass
    
    class ListPoliciesRequest(Mapping):
        pass
    
    class ListPoliciesResponse(Mapping):
        pass
    
    class ListPolicyPrincipalsRequest(Mapping):
        pass
    
    class ListPolicyPrincipalsResponse(Mapping):
        pass
    
    class ListPolicyVersionsRequest(Mapping):
        pass
    
    class ListPolicyVersionsResponse(Mapping):
        pass
    
    class ListPrincipalPoliciesRequest(Mapping):
        pass
    
    class ListPrincipalPoliciesResponse(Mapping):
        pass
    
    class ListPrincipalThingsRequest(Mapping):
        pass
    
    class ListPrincipalThingsResponse(Mapping):
        pass
    
    class ListRoleAliasesRequest(Mapping):
        pass
    
    class ListRoleAliasesResponse(Mapping):
        pass
    
    class ListScheduledAuditsRequest(Mapping):
        pass
    
    class ListScheduledAuditsResponse(Mapping):
        pass
    
    class ListSecurityProfilesRequest(Mapping):
        pass
    
    class ListSecurityProfilesResponse(Mapping):
        pass
    
    class ListSecurityProfilesForTargetRequest(Mapping):
        pass
    
    class ListSecurityProfilesForTargetResponse(Mapping):
        pass
    
    class ListStreamsRequest(Mapping):
        pass
    
    class ListStreamsResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class ListTargetsForPolicyRequest(Mapping):
        pass
    
    class ListTargetsForPolicyResponse(Mapping):
        pass
    
    class ListTargetsForSecurityProfileRequest(Mapping):
        pass
    
    class ListTargetsForSecurityProfileResponse(Mapping):
        pass
    
    class ListThingGroupsRequest(Mapping):
        pass
    
    class ListThingGroupsResponse(Mapping):
        pass
    
    class RecursiveWithoutDefault(object):
        pass
    
    class ListThingGroupsForThingRequest(Mapping):
        pass
    
    class ListThingGroupsForThingResponse(Mapping):
        pass
    
    class ListThingPrincipalsRequest(Mapping):
        pass
    
    class ListThingPrincipalsResponse(Mapping):
        pass
    
    class ListThingRegistrationTaskReportsRequest(Mapping):
        pass
    
    class ListThingRegistrationTaskReportsResponse(Mapping):
        pass
    
    class ListThingRegistrationTasksRequest(Mapping):
        pass
    
    class ListThingRegistrationTasksResponse(Mapping):
        pass
    
    class ListThingTypesRequest(Mapping):
        pass
    
    class ListThingTypesResponse(Mapping):
        pass
    
    class ListThingsRequest(Mapping):
        pass
    
    class ListThingsResponse(Mapping):
        pass
    
    class ListThingsInBillingGroupRequest(Mapping):
        pass
    
    class ListThingsInBillingGroupResponse(Mapping):
        pass
    
    class ListThingsInThingGroupRequest(Mapping):
        pass
    
    class ListThingsInThingGroupResponse(Mapping):
        pass
    
    class ListTopicRulesRequest(Mapping):
        pass
    
    class ListTopicRulesResponse(Mapping):
        pass
    
    class IsDisabled(object):
        pass
    
    class ListV2LoggingLevelsRequest(Mapping):
        pass
    
    class ListV2LoggingLevelsResponse(Mapping):
        pass
    
    class ListViolationEventsRequest(Mapping):
        pass
    
    class ListViolationEventsResponse(Mapping):
        pass
    
    class RegisterCACertificateRequest(Mapping):
        pass
    
    class RegisterCACertificateResponse(Mapping):
        pass
    
    class AllowAutoRegistration(object):
        pass
    
    class RegistrationConfig(Mapping):
        pass
    
    class RegisterCertificateRequest(Mapping):
        pass
    
    class RegisterCertificateResponse(Mapping):
        pass
    
    class SetAsActiveFlag(object):
        pass
    
    class RegisterThingRequest(Mapping):
        pass
    
    class RegisterThingResponse(Mapping):
        pass
    
    class Parameters(object):
        pass
    
    class RejectCertificateTransferRequest(Mapping):
        pass
    
    class RemoveThingFromBillingGroupRequest(Mapping):
        pass
    
    class RemoveThingFromBillingGroupResponse(Mapping):
        pass
    
    class RemoveThingFromThingGroupRequest(Mapping):
        pass
    
    class RemoveThingFromThingGroupResponse(Mapping):
        pass
    
    class ReplaceTopicRuleRequest(Mapping):
        pass
    
    class SearchIndexRequest(Mapping):
        pass
    
    class SearchIndexResponse(Mapping):
        pass
    
    class SetDefaultAuthorizerRequest(Mapping):
        pass
    
    class SetDefaultAuthorizerResponse(Mapping):
        pass
    
    class SetDefaultPolicyVersionRequest(Mapping):
        pass
    
    class SetLoggingOptionsRequest(Mapping):
        pass
    
    class LoggingOptionsPayload(Mapping):
        pass
    
    class SetV2LoggingLevelRequest(Mapping):
        pass
    
    class LogTarget(Mapping):
        pass
    
    class SetV2LoggingOptionsRequest(Mapping):
        pass
    
    class DisableAllLogs(object):
        pass
    
    class StartOnDemandAuditTaskRequest(Mapping):
        pass
    
    class StartOnDemandAuditTaskResponse(Mapping):
        pass
    
    class StartThingRegistrationTaskRequest(Mapping):
        pass
    
    class StartThingRegistrationTaskResponse(Mapping):
        pass
    
    class StopThingRegistrationTaskRequest(Mapping):
        pass
    
    class StopThingRegistrationTaskResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class TestAuthorizationRequest(Mapping):
        pass
    
    class TestAuthorizationResponse(Mapping):
        pass
    
    class AuthInfo(Mapping):
        pass
    
    class PolicyName(object):
        pass
    
    class TestInvokeAuthorizerRequest(Mapping):
        pass
    
    class TestInvokeAuthorizerResponse(Mapping):
        pass
    
    class TransferCertificateRequest(Mapping):
        pass
    
    class TransferCertificateResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateAccountAuditConfigurationRequest(Mapping):
        pass
    
    class UpdateAccountAuditConfigurationResponse(Mapping):
        pass
    
    class AuditNotificationTargetConfigurations(object):
        pass
    
    class AuditCheckConfigurations(object):
        pass
    
    class UpdateAuthorizerRequest(Mapping):
        pass
    
    class UpdateAuthorizerResponse(Mapping):
        pass
    
    class UpdateBillingGroupRequest(Mapping):
        pass
    
    class UpdateBillingGroupResponse(Mapping):
        pass
    
    class UpdateCACertificateRequest(Mapping):
        pass
    
    class RemoveAutoRegistration(object):
        pass
    
    class UpdateCertificateRequest(Mapping):
        pass
    
    class UpdateDynamicThingGroupRequest(Mapping):
        pass
    
    class UpdateDynamicThingGroupResponse(Mapping):
        pass
    
    class UpdateEventConfigurationsRequest(Mapping):
        pass
    
    class UpdateEventConfigurationsResponse(Mapping):
        pass
    
    class EventConfigurations(object):
        pass
    
    class UpdateIndexingConfigurationRequest(Mapping):
        pass
    
    class UpdateIndexingConfigurationResponse(Mapping):
        pass
    
    class ThingIndexingConfiguration(Mapping):
        pass
    
    class ThingGroupIndexingConfiguration(Mapping):
        pass
    
    class UpdateJobRequest(Mapping):
        pass
    
    class UpdateRoleAliasRequest(Mapping):
        pass
    
    class UpdateRoleAliasResponse(Mapping):
        pass
    
    class UpdateScheduledAuditRequest(Mapping):
        pass
    
    class UpdateScheduledAuditResponse(Mapping):
        pass
    
    class UpdateSecurityProfileRequest(Mapping):
        pass
    
    class UpdateSecurityProfileResponse(Mapping):
        pass
    
    class DeleteBehaviors(object):
        pass
    
    class DeleteAlertTargets(object):
        pass
    
    class DeleteAdditionalMetricsToRetain(object):
        pass
    
    class UpdateStreamRequest(Mapping):
        pass
    
    class UpdateStreamResponse(Mapping):
        pass
    
    class UpdateThingRequest(Mapping):
        pass
    
    class UpdateThingResponse(Mapping):
        pass
    
    class RemoveThingType(object):
        pass
    
    class UpdateThingGroupRequest(Mapping):
        pass
    
    class UpdateThingGroupResponse(Mapping):
        pass
    
    class UpdateThingGroupsForThingRequest(Mapping):
        pass
    
    class UpdateThingGroupsForThingResponse(Mapping):
        pass
    
    class ThingGroupName(object):
        pass
    
    class ValidateSecurityProfileBehaviorsRequest(Mapping):
        pass
    
    class ValidateSecurityProfileBehaviorsResponse(Mapping):
        pass
    
    def accept_certificate_transfer(self, certificateId: str, setAsActive: SetAsActive=None):
        """Accepts a pending certificate transfer. The default state of the certificate is INACTIVE. To check for pending certificate transfers, call <a>ListCertificates</a> to enumerate your certificates.

        :param self:
        :param setAsActive:  Specifies whether the certificate is active.<br/>certificateId:  The ID of the certificate. (The last part of the certificate ARN contains the certificate ID.)<br/>
        :return:         """
        pass

    def add_thing_to_billing_group(self, billingGroupName: str=None, billingGroupArn: str=None, thingName: str=None, thingArn: str=None) -> AddThingToBillingGroupResponse:
        """Adds a thing to a billing group.

        :param self:
        :param thingArn:  The ARN of the thing to be added to the billing group.<br/>thingName:  The name of the thing to be added to the billing group.<br/>billingGroupArn:  The ARN of the billing group.<br/>billingGroupName:  The name of the billing group.<br/>
        :return:         """
        pass

    def add_thing_to_thing_group(self, thingGroupName: str=None, thingGroupArn: str=None, thingName: str=None, thingArn: str=None, overrideDynamicGroups: OverrideDynamicGroups=None) -> AddThingToThingGroupResponse:
        """Adds a thing to a thing group.

        :param self:
        :param overrideDynamicGroups:  Override dynamic thing groups with static thing groups when 10-group limit is reached. If a thing belongs to 10 thing groups, and one or more of those groups are dynamic thing groups, adding a thing to a static group removes the thing from the last dynamic group.<br/>thingArn:  The ARN of the thing to add to a group.<br/>thingName:  The name of the thing to add to a group.<br/>thingGroupArn:  The ARN of the group to which you are adding a thing.<br/>thingGroupName:  The name of the group to which you are adding a thing.<br/>
        :return:         """
        pass

    def associate_targets_with_job(self, jobId: str, targets: List[TargetArn], comment: str=None) -> AssociateTargetsWithJobResponse:
        """Associates a group with a continuous job. The following criteria must be met:  <ul> <li> The job must have been created with the <code>targetSelection</code> field set to "CONTINUOUS". </li> <li> The job status must currently be "IN_PROGRESS". </li> <li> The total number of targets associated with a job must not exceed 100. </li> </ul>

        :param self:
        :param comment:  An optional comment string describing why the job was associated with the targets.<br/>jobId:  The unique identifier you assigned to this job when it was created.<br/>targets:  A list of thing group ARNs that define the targets of the job.<br/>
        :return: description:  A short text description of the job.<br/>jobId:  The unique identifier you assigned to this job when it was created.<br/>jobArn:  An ARN identifying the job.<br/>        """
        pass

    def attach_policy(self, target: str, policyName: str):
        """Attaches a policy to the specified target.

        :param self:
        :param target:  The <a href="https://docs.aws.amazon.com/iot/latest/developerguide/iot-security-identity.html">identity</a> to which the policy is attached.<br/>policyName:  The name of the policy to attach.<br/>
        :return:         """
        pass

    def attach_principal_policy(self, principal: str, policyName: str):
        """Attaches the specified policy to the specified principal (certificate or other credential).  <b>Note:</b> This API is deprecated. Please use <a>AttachPolicy</a> instead.

        :param self:
        :param principal:  The principal, which can be a certificate ARN (as returned from the CreateCertificate operation) or an Amazon Cognito ID.<br/>policyName:  The policy name.<br/>
        :return:         """
        pass

    def attach_security_profile(self, securityProfileTargetArn: str, securityProfileName: str) -> AttachSecurityProfileResponse:
        """Associates a Device Defender security profile with a thing group or with this account. Each thing group or account can have up to five security profiles associated with it.

        :param self:
        :param securityProfileTargetArn:  The ARN of the target (thing group) to which the security profile is attached.<br/>securityProfileName:  The security profile that is attached.<br/>
        :return:         """
        pass

    def attach_thing_principal(self, principal: str, thingName: str) -> AttachThingPrincipalResponse:
        """Attaches the specified principal to the specified thing. A principal can be X.509 certificates, IAM users, groups, and roles, Amazon Cognito identities or federated identities.

        :param self:
        :param principal:  The principal, such as a certificate or other credential.<br/>thingName:  The name of the thing.<br/>
        :return:         """
        pass

    def cancel_audit_task(self, taskId: str) -> CancelAuditTaskResponse:
        """Cancels an audit that is in progress. The audit can be either scheduled or on-demand. If the audit is not in progress, an "InvalidRequestException" occurs.

        :param self:
        :param taskId:  The ID of the audit you want to cancel. You can only cancel an audit that is "IN_PROGRESS".<br/>
        :return:         """
        pass

    def cancel_certificate_transfer(self, certificateId: str):
        """Cancels a pending transfer for the specified certificate.  <b>Note</b> Only the transfer source account can use this operation to cancel a transfer. (Transfer destinations can use <a>RejectCertificateTransfer</a> instead.) After transfer, AWS IoT returns the certificate to the source account in the INACTIVE state. After the destination account has accepted the transfer, the transfer cannot be cancelled. After a certificate transfer is cancelled, the status of the certificate changes from PENDING_TRANSFER to INACTIVE.

        :param self:
        :param certificateId:  The ID of the certificate. (The last part of the certificate ARN contains the certificate ID.)<br/>
        :return:         """
        pass

    def cancel_job(self, jobId: str, reasonCode: str=None, comment: str=None, force: ForceFlag=None) -> CancelJobResponse:
        """Cancels a job.

        :param self:
        :param force:  (Optional) If <code>true</code> job executions with status "IN_PROGRESS" and "QUEUED" are canceled, otherwise only job executions with status "QUEUED" are canceled. The default is <code>false</code>. Canceling a job which is "IN_PROGRESS", will cause a device which is executing the job to be unable to update the job execution status. Use caution and ensure that each device executing a job which is canceled is able to recover to a valid state.<br/>comment:  An optional comment string describing why the job was canceled.<br/>reasonCode:  (Optional)A reason code string that explains why the job was canceled.<br/>jobId:  The unique identifier you assigned to this job when it was created.<br/>
        :return: description:  A short text description of the job.<br/>jobId:  The unique identifier you assigned to this job when it was created.<br/>jobArn:  The job ARN.<br/>        """
        pass

    def cancel_job_execution(self, thingName: str, jobId: str, force: ForceFlag=None, expectedVersion: ExpectedVersion=None, statusDetails: DetailsMap=None):
        """Cancels the execution of a job for a given thing.

        :param self:
        :param statusDetails:  A collection of name/value pairs that describe the status of the job execution. If not specified, the statusDetails are unchanged. You can specify at most 10 name/value pairs.<br/>expectedVersion:  (Optional) The expected current version of the job execution. Each time you update the job execution, its version is incremented. If the version of the job execution stored in Jobs does not match, the update is rejected with a VersionMismatch error, and an ErrorResponse that contains the current job execution status data is returned. (This makes it unnecessary to perform a separate DescribeJobExecution request in order to obtain the job execution status data.)<br/>force:  (Optional) If <code>true</code> the job execution will be canceled if it has status IN_PROGRESS or QUEUED, otherwise the job execution will be canceled only if it has status QUEUED. If you attempt to cancel a job execution that is IN_PROGRESS, and you do not set <code>force</code> to <code>true</code>, then an <code>InvalidStateTransitionException</code> will be thrown. The default is <code>false</code>. Canceling a job execution which is "IN_PROGRESS", will cause the device to be unable to update the job execution status. Use caution and ensure that the device is able to recover to a valid state.<br/>thingName:  The name of the thing whose execution of the job will be canceled.<br/>jobId:  The ID of the job to be canceled.<br/>
        :return:         """
        pass

    def clear_default_authorizer(self) -> ClearDefaultAuthorizerResponse:
        """Clears the default authorizer.

        :param self:
        :param 
        :return:         """
        pass

    def create_authorizer(self, tokenSigningPublicKeys: PublicKeyMap, tokenKeyName: str, authorizerFunctionArn: str, authorizerName: str, status: str=None) -> CreateAuthorizerResponse:
        """Creates an authorizer.

        :param self:
        :param status:  The status of the create authorizer request.<br/>tokenSigningPublicKeys:  The public keys used to verify the digital signature returned by your custom authentication service.<br/>tokenKeyName:  The name of the token key used to extract the token from the HTTP headers.<br/>authorizerFunctionArn:  The ARN of the authorizer's Lambda function.<br/>authorizerName:  The authorizer name.<br/>
        :return: authorizerArn:  The authorizer ARN.<br/>authorizerName:  The authorizer's name.<br/>        """
        pass

    def create_billing_group(self, billingGroupName: str, billingGroupProperties: BillingGroupProperties=None, tags: List[Tag]=None) -> CreateBillingGroupResponse:
        """Creates a billing group.

        :param self:
        :param tags:  Metadata which can be used to manage the billing group.<br/>billingGroupProperties:  The properties of the billing group.:<br/>billingGroupName:  The name you wish to give to the billing group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>billingGroupDescription</i> The description of the billing group.<br/>
        :return: billingGroupId:  The ID of the billing group.<br/>billingGroupArn:  The ARN of the billing group.<br/>billingGroupName:  The name you gave to the billing group.<br/>        """
        pass

    def create_certificate_from_csr(self, certificateSigningRequest: str, setAsActive: SetAsActive=None) -> CreateCertificateFromCsrResponse:
        """Creates an X.509 certificate using the specified certificate signing request.  <b>Note:</b> The CSR must include a public key that is either an RSA key with a length of at least 2048 bits or an ECC key from NIST P-256 or NIST P-384 curves.   <b>Note:</b> Reusing the same certificate signing request (CSR) results in a distinct certificate. You can create multiple certificates in a batch by creating a directory, copying multiple .csr files into that directory, and then specifying that directory on the command line. The following commands show how to create a batch of certificates given a batch of CSRs. Assuming a set of CSRs are located inside of the directory my-csr-directory: On Linux and OS X, the command is: $ ls my-csr-directory/ | xargs -I {} aws iot create-certificate-from-csr --certificate-signing-request file://my-csr-directory/{} This command lists all of the CSRs in my-csr-directory and pipes each CSR file name to the aws iot create-certificate-from-csr AWS CLI command to create a certificate for the corresponding CSR. The aws iot create-certificate-from-csr part of the command can also be run in parallel to speed up the certificate creation process: $ ls my-csr-directory/ | xargs -P 10 -I {} aws iot create-certificate-from-csr --certificate-signing-request file://my-csr-directory/{} On Windows PowerShell, the command to create certificates for all CSRs in my-csr-directory is: &gt; ls -Name my-csr-directory | %{aws iot create-certificate-from-csr --certificate-signing-request file://my-csr-directory/$_} On a Windows command prompt, the command to create certificates for all CSRs in my-csr-directory is: &gt; forfiles /p my-csr-directory /c "cmd /c aws iot create-certificate-from-csr --certificate-signing-request file://@path"

        :param self:
        :param setAsActive:  Specifies whether the certificate is active.<br/>certificateSigningRequest:  The certificate signing request (CSR).<br/>
        :return: certificatePem:  The certificate data, in PEM format.<br/>certificateId:  The ID of the certificate. Certificate management operations only take a certificateId.<br/>certificateArn:  The Amazon Resource Name (ARN) of the certificate. You can use the ARN as a principal for policy operations.<br/>        """
        pass

    def create_dynamic_thing_group(self, queryString: str, thingGroupName: str, thingGroupProperties: ThingGroupProperties=None, indexName: str=None, queryVersion: str=None, tags: List[Tag]=None) -> CreateDynamicThingGroupResponse:
        """Creates a dynamic thing group.

        :param self:
        :param tags:  Metadata which can be used to manage the dynamic thing group.<br/>queryVersion:  The dynamic thing group query version. <note> Currently one query version is supported: "2017-09-30". If not specified, the query version defaults to this value. </note><br/>queryString:  The dynamic thing group search query string. See <a href="https://docs.aws.amazon.com/iot/latest/developerguide/query-syntax.html">Query Syntax</a> for information about query string syntax.<br/>indexName:  The dynamic thing group index name. <note> Currently one index is supported: "AWS_Things". </note><br/>thingGroupProperties:  The dynamic thing group properties.:<br/>thingGroupName:  The dynamic thing group name to create.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>attributePayload</i> The thing group attributes in JSON format.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>attributePayload</b>: The thing group attributes in JSON format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>thingGroupDescription</i> The thing group description.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>merge</i> Specifies whether the list of attributes provided in the <code>AttributePayload</code> is merged with the attributes stored in the registry, instead of overwriting them. To remove an attribute, call <code>UpdateThing</code> with an empty attribute value. <note> The <code>merge</code> attribute is only valid when calling <code>UpdateThing</code>. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>attributes</i> A JSON string containing up to three key-value pair in JSON format. For example:  <code>{\"attributes\":{\"string1\":\"string2\"}}</code> <br/>
        :return: queryVersion:  The dynamic thing group query version.<br/>queryString:  The dynamic thing group search query string.<br/>indexName:  The dynamic thing group index name.<br/>thingGroupId:  The dynamic thing group ID.<br/>thingGroupArn:  The dynamic thing group ARN.<br/>thingGroupName:  The dynamic thing group name.<br/>        """
        pass

    def create_job(self, targets: List[TargetArn], jobId: str, documentSource: str=None, document: str=None, description: str=None, presignedUrlConfig: PresignedUrlConfig=None, targetSelection: str=None, jobExecutionsRolloutConfig: JobExecutionsRolloutConfig=None, abortConfig: AbortConfig=None, timeoutConfig: TimeoutConfig=None, tags: List[Tag]=None) -> CreateJobResponse:
        """Creates a job.

        :param self:
        :param tags:  Metadata which can be used to manage the job.<br/>timeoutConfig:  Specifies the amount of time each device has to finish its execution of the job. The timer is started when the job execution status is set to <code>IN_PROGRESS</code>. If the job execution status is not set to another terminal state before the time expires, it will be automatically set to <code>TIMED_OUT</code>.:<br/>abortConfig:  Allows you to create criteria to abort a job.:<br/>jobExecutionsRolloutConfig:  Allows you to create a staged rollout of the job.:<br/>targetSelection:  Specifies whether the job will continue to run (CONTINUOUS), or will be complete after all those things specified as targets have completed the job (SNAPSHOT). If continuous, the job may also be run on a thing when a change is detected in a target. For example, a job will run on a thing when the thing is added to a target group, even after the job was completed by all things originally in the group.<br/>presignedUrlConfig:  Configuration information for pre-signed S3 URLs.:<br/>description:  A short text description of the job.<br/>document:  The job document. <note> If the job document resides in an S3 bucket, you must use a placeholder link when specifying the document. The placeholder link is of the following form:  <code>${aws:iot:s3-presigned-url:https://s3.amazonaws.com/<i>bucket</i>/<i>key</i>}</code>  where <i>bucket</i> is your bucket name and <i>key</i> is the object in the bucket to which you are linking. </note><br/>documentSource:  An S3 link to the job document.<br/>targets:  A list of things and thing groups to which the job should be sent.<br/>jobId:  A job identifier which must be unique for your AWS account. We recommend using a UUID. Alpha-numeric characters, "-" and "_" are valid for use here.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>expiresInSec</i> How long (in seconds) pre-signed URLs are valid. Valid values are 60 - 3600, the default value is 3600 seconds. Pre-signed URLs are generated when Jobs receives an MQTT request for the job document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The ARN of an IAM role that grants grants permission to download files from the S3 bucket where the job data/updates are stored. The role must also grant permission for IoT to download the files.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>exponentialRate</i> The rate of increase for a job rollout. This parameter allows you to define an exponential rate for a job rollout.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>exponentialRate</b>: The rate of increase for a job rollout. This parameter allows you to define an exponential rate for a job rollout.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>maximumPerMinute</i> The maximum number of things that will be notified of a pending job, per minute. This parameter allows you to create a staged rollout.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>incrementFactor</i> The exponential factor to increase the rate of rollout for a job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>baseRatePerMinute</i> The minimum number of things that will be notified of a pending job, per minute at the start of job rollout. This parameter allows you to define the initial rate of rollout.<br/>The criteria to initiate the increase in rate of rollout for a job. AWS IoT supports up to one digit after the decimal (for example, 1.5, but not 1.55).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>rateIncreaseCriteria</b>: The criteria to initiate the increase in rate of rollout for a job. AWS IoT supports up to one digit after the decimal (for example, 1.5, but not 1.55).&nbsp;&nbsp;&nbsp;&nbsp; <i>criteriaList</i> The list of abort criteria to define rules to abort the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>inProgressTimeoutInMinutes</i> Specifies the amount of time, in minutes, this device has to finish execution of this job. The timeout interval can be anywhere between 1 minute and 7 days (1 to 10080 minutes). The in progress timer can't be updated and will apply to all job executions for the job. Whenever a job execution remains in the IN_PROGRESS status for longer than this interval, the job execution will fail and switch to the terminal <code>TIMED_OUT</code> status.<br/>
        :return: description:  The job description.<br/>jobId:  The unique identifier you assigned to this job.<br/>jobArn:  The job ARN.<br/>        """
        pass

    def create_keys_and_certificate(self, setAsActive: SetAsActive=None) -> CreateKeysAndCertificateResponse:
        """Creates a 2048-bit RSA key pair and issues an X.509 certificate using the issued public key.  <b>Note</b> This is the only time AWS IoT issues the private key for this certificate, so it is important to keep it in a secure location.

        :param self:
        :param setAsActive:  Specifies whether the certificate is active.<br/>
        :return: keyPair:  The generated key pair.:<br/>certificatePem:  The certificate data, in PEM format.<br/>certificateId:  The ID of the certificate. AWS IoT issues a default subject name for the certificate (for example, AWS IoT Certificate).<br/>certificateArn:  The ARN of the certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrivateKey</i> The private key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PublicKey</i> The public key.<br/>        """
        pass

    def create_ota_update(self, roleArn: str, files: List[OTAUpdateFile], targets: List[Target], otaUpdateId: str, description: str=None, targetSelection: str=None, awsJobExecutionsRolloutConfig: AwsJobExecutionsRolloutConfig=None, additionalParameters: AdditionalParameterMap=None, tags: List[Tag]=None) -> CreateOTAUpdateResponse:
        """Creates an AWS IoT OTAUpdate on a target group of things or groups.

        :param self:
        :param tags:  Metadata which can be used to manage updates.<br/>additionalParameters:  A list of additional OTA update parameters which are name-value pairs.<br/>roleArn:  The IAM role that allows access to the AWS IoT Jobs service.<br/>files:  The files to be streamed by the OTA update.<br/>awsJobExecutionsRolloutConfig:  Configuration for the rollout of OTA updates.:<br/>targetSelection:  Specifies whether the update will continue to run (CONTINUOUS), or will be complete after all the things specified as targets have completed the update (SNAPSHOT). If continuous, the update may also be run on a thing when a change is detected in a target. For example, an update will run on a thing when the thing is added to a target group, even after the update was completed by all things originally in the group. Valid values: CONTINUOUS | SNAPSHOT.<br/>targets:  The targeted devices to receive OTA updates.<br/>description:  The description of the OTA update.<br/>otaUpdateId:  The ID of the OTA update to be created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>maximumPerMinute</i> The maximum number of OTA update job executions started per minute.<br/>
        :return: otaUpdateStatus:  The OTA update status.<br/>awsIotJobArn:  The AWS IoT job ARN associated with the OTA update.<br/>otaUpdateArn:  The OTA update ARN.<br/>awsIotJobId:  The AWS IoT job ID associated with the OTA update.<br/>otaUpdateId:  The OTA update ID.<br/>        """
        pass

    def create_policy(self, policyDocument: str, policyName: str) -> CreatePolicyResponse:
        """Creates an AWS IoT policy. The created policy is the default version for the policy. This operation creates a policy version with a version identifier of <b>1</b> and sets <b>1</b> as the policy's default version.

        :param self:
        :param policyDocument:  The JSON document that describes the policy. <b>policyDocument</b> must have a minimum length of 1, with a maximum length of 2048, excluding whitespace.<br/>policyName:  The policy name.<br/>
        :return: policyVersionId:  The policy version ID.<br/>policyDocument:  The JSON document that describes the policy.<br/>policyArn:  The policy ARN.<br/>policyName:  The policy name.<br/>        """
        pass

    def create_policy_version(self, policyDocument: str, policyName: str, setAsDefault: SetAsDefault=None) -> CreatePolicyVersionResponse:
        """Creates a new version of the specified AWS IoT policy. To update a policy, create a new policy version. A managed policy can have up to five versions. If the policy has five versions, you must use <a>DeletePolicyVersion</a> to delete an existing version before you create a new one. Optionally, you can set the new version as the policy's default version. The default version is the operative version (that is, the version that is in effect for the certificates to which the policy is attached).

        :param self:
        :param setAsDefault:  Specifies whether the policy version is set as the default. When this parameter is true, the new policy version becomes the operative version (that is, the version that is in effect for the certificates to which the policy is attached).<br/>policyDocument:  The JSON document that describes the policy. Minimum length of 1. Maximum length of 2048, excluding whitespace.<br/>policyName:  The policy name.<br/>
        :return: isDefaultVersion:  Specifies whether the policy version is the default.<br/>policyVersionId:  The policy version ID.<br/>policyDocument:  The JSON document that describes the policy.<br/>policyArn:  The policy ARN.<br/>        """
        pass

    def create_role_alias(self, roleArn: str, roleAlias: str, credentialDurationSeconds: int=None) -> CreateRoleAliasResponse:
        """Creates a role alias.

        :param self:
        :param credentialDurationSeconds:  How long (in seconds) the credentials will be valid.<br/>roleArn:  The role ARN.<br/>roleAlias:  The role alias that points to a role ARN. This allows you to change the role without having to update the device.<br/>
        :return: roleAliasArn:  The role alias ARN.<br/>roleAlias:  The role alias.<br/>        """
        pass

    def create_scheduled_audit(self, scheduledAuditName: str, targetCheckNames: List[AuditCheckName], frequency: str, dayOfMonth: str=None, dayOfWeek: str=None, tags: List[Tag]=None) -> CreateScheduledAuditResponse:
        """Creates a scheduled audit that is run at a specified time interval.

        :param self:
        :param scheduledAuditName:  The name you want to give to the scheduled audit. (Max. 128 chars)<br/>tags:  Metadata which can be used to manage the scheduled audit.<br/>targetCheckNames:  Which checks are performed during the scheduled audit. Checks must be enabled for your account. (Use <code>DescribeAccountAuditConfiguration</code> to see the list of all checks including those that are enabled or <code>UpdateAccountAuditConfiguration</code> to select which checks are enabled.)<br/>dayOfWeek:  The day of the week on which the scheduled audit takes place. Can be one of "SUN", "MON", "TUE", "WED", "THU", "FRI" or "SAT". This field is required if the "frequency" parameter is set to "WEEKLY" or "BIWEEKLY".<br/>dayOfMonth:  The day of the month on which the scheduled audit takes place. Can be "1" through "31" or "LAST". This field is required if the "frequency" parameter is set to "MONTHLY". If days 29-31 are specified, and the month does not have that many days, the audit takes place on the "LAST" day of the month.<br/>frequency:  How often the scheduled audit takes place. Can be one of "DAILY", "WEEKLY", "BIWEEKLY" or "MONTHLY". The actual start time of each audit is determined by the system.<br/>
        :return: scheduledAuditArn:  The ARN of the scheduled audit.<br/>        """
        pass

    def create_security_profile(self, securityProfileName: str, securityProfileDescription: str=None, behaviors: List[Behavior]=None, alertTargets: AlertTargets=None, additionalMetricsToRetain: List[BehaviorMetric]=None, tags: List[Tag]=None) -> CreateSecurityProfileResponse:
        """Creates a Device Defender security profile.

        :param self:
        :param tags:  Metadata which can be used to manage the security profile.<br/>additionalMetricsToRetain:  A list of metrics whose data is retained (stored). By default, data is retained for any metric used in the profile's <code>behaviors</code> but it is also retained for any metric specified here.<br/>alertTargets:  Specifies the destinations to which alerts are sent. (Alerts are always sent to the console.) Alerts are generated when a device (thing) violates a behavior.<br/>behaviors:  Specifies the behaviors that, when violated by a device (thing), cause an alert.<br/>securityProfileDescription:  A description of the security profile.<br/>securityProfileName:  The name you are giving to the security profile.<br/>
        :return: securityProfileArn:  The ARN of the security profile.<br/>securityProfileName:  The name you gave to the security profile.<br/>        """
        pass

    def create_stream(self, roleArn: str, files: List[StreamFile], streamId: str, description: str=None, tags: List[Tag]=None) -> CreateStreamResponse:
        """Creates a stream for delivering one or more large files in chunks over MQTT. A stream transports data bytes in chunks or blocks packaged as MQTT messages from a source like S3. You can have one or more files associated with a stream. The total size of a file associated with the stream cannot exceed more than 2 MB. The stream will be created with version 0. If a stream is created with the same streamID as a stream that existed and was deleted within last 90 days, we will resurrect that old stream by incrementing the version by 1.

        :param self:
        :param tags:  Metadata which can be used to manage streams.<br/>roleArn:  An IAM role that allows the IoT service principal assumes to access your S3 files.<br/>files:  The files to stream.<br/>description:  A description of the stream.<br/>streamId:  The stream ID.<br/>
        :return: streamVersion:  The version of the stream.<br/>description:  A description of the stream.<br/>streamArn:  The stream ARN.<br/>streamId:  The stream ID.<br/>        """
        pass

    def create_thing(self, thingName: str, thingTypeName: str=None, attributePayload: AttributePayload=None, billingGroupName: str=None) -> CreateThingResponse:
        """Creates a thing record in the registry. If this call is made multiple times using the same thing name and configuration, the call will succeed. If this call is made with the same thing name but different configuration a <code>ResourceAlreadyExistsException</code> is thrown. <note> This is a control plane operation. See <a href="https://docs.aws.amazon.com/iot/latest/developerguide/authorization.html">Authorization</a> for information about authorizing control plane actions. </note>

        :param self:
        :param billingGroupName:  The name of the billing group the thing will be added to.<br/>attributePayload:  The attribute payload, which consists of up to three name/value pairs in a JSON document. For example:  <code>{\"attributes\":{\"string1\":\"string2\"}}</code> :<br/>thingTypeName:  The name of the thing type associated with the new thing.<br/>thingName:  The name of the thing to create.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>merge</i> Specifies whether the list of attributes provided in the <code>AttributePayload</code> is merged with the attributes stored in the registry, instead of overwriting them. To remove an attribute, call <code>UpdateThing</code> with an empty attribute value. <note> The <code>merge</code> attribute is only valid when calling <code>UpdateThing</code>. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>attributes</i> A JSON string containing up to three key-value pair in JSON format. For example:  <code>{\"attributes\":{\"string1\":\"string2\"}}</code> <br/>
        :return: thingId:  The thing ID.<br/>thingArn:  The ARN of the new thing.<br/>thingName:  The name of the new thing.<br/>        """
        pass

    def create_thing_group(self, thingGroupName: str, parentGroupName: str=None, thingGroupProperties: ThingGroupProperties=None, tags: List[Tag]=None) -> CreateThingGroupResponse:
        """Create a thing group. <note> This is a control plane operation. See <a href="https://docs.aws.amazon.com/iot/latest/developerguide/authorization.html">Authorization</a> for information about authorizing control plane actions. </note>

        :param self:
        :param tags:  Metadata which can be used to manage the thing group.<br/>thingGroupProperties:  The thing group properties.:<br/>parentGroupName:  The name of the parent thing group.<br/>thingGroupName:  The thing group name to create.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>attributePayload</i> The thing group attributes in JSON format.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>attributePayload</b>: The thing group attributes in JSON format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>thingGroupDescription</i> The thing group description.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>merge</i> Specifies whether the list of attributes provided in the <code>AttributePayload</code> is merged with the attributes stored in the registry, instead of overwriting them. To remove an attribute, call <code>UpdateThing</code> with an empty attribute value. <note> The <code>merge</code> attribute is only valid when calling <code>UpdateThing</code>. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>attributes</i> A JSON string containing up to three key-value pair in JSON format. For example:  <code>{\"attributes\":{\"string1\":\"string2\"}}</code> <br/>
        :return: thingGroupId:  The thing group ID.<br/>thingGroupArn:  The thing group ARN.<br/>thingGroupName:  The thing group name.<br/>        """
        pass

    def create_thing_type(self, thingTypeName: str, thingTypeProperties: ThingTypeProperties=None, tags: List[Tag]=None) -> CreateThingTypeResponse:
        """Creates a new thing type.

        :param self:
        :param tags:  Metadata which can be used to manage the thing type.<br/>thingTypeProperties:  The ThingTypeProperties for the thing type to create. It contains information about the new thing type including a description, and a list of searchable thing attribute names.:<br/>thingTypeName:  The name of the thing type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>searchableAttributes</i> A list of searchable thing attribute names.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>thingTypeDescription</i> The description of the thing type.<br/>
        :return: thingTypeId:  The thing type ID.<br/>thingTypeArn:  The Amazon Resource Name (ARN) of the thing type.<br/>thingTypeName:  The name of the thing type.<br/>        """
        pass

    def create_topic_rule(self, topicRulePayload: TopicRulePayload, ruleName: str, tags: str=None):
        """Creates a rule. Creating rules is an administrator-level action. Any user who has permission to create rules will be able to access data processed by the rule.

        :param self:
        :param tags:  Metadata which can be used to manage the topic rule. <note> For URI Request parameters use format: ...key1=value1&amp;key2=value2... For the CLI command-line parameter use format: --tags "key1=value1&amp;key2=value2..." For the cli-input-json file use format: "tags": "key1=value1&amp;key2=value2..." </note><br/>topicRulePayload:  The rule payload.:<br/>ruleName:  The name of the rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorAction</i> The action to take when an error occurs.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>errorAction</b>: The action to take when an error occurs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>awsIotSqlVersion</i> The version of the SQL rules engine to use when evaluating the rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ruleDisabled</i> Specifies whether the rule is disabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>actions</i> The actions associated with the rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The description of the rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>sql</i> The SQL statement used to query the topic. For more information, see <a href="https://docs.aws.amazon.com/iot/latest/developerguide/iot-rules.html#aws-iot-sql-reference">AWS IoT SQL Reference</a> in the <i>AWS IoT Developer Guide</i>.<br/>Write to a DynamoDB table.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>dynamoDB</b>: Write to a DynamoDB table.
        :return:         """
        pass

    def delete_account_audit_configuration(self, deleteScheduledAudits: DeleteScheduledAudits=None) -> DeleteAccountAuditConfigurationResponse:
        """Restores the default settings for Device Defender audits for this account. Any configuration data you entered is deleted and all audit checks are reset to disabled. 

        :param self:
        :param deleteScheduledAudits:  If true, all scheduled audits are deleted.<br/>
        :return:         """
        pass

    def delete_authorizer(self, authorizerName: str) -> DeleteAuthorizerResponse:
        """Deletes an authorizer.

        :param self:
        :param authorizerName:  The name of the authorizer to delete.<br/>
        :return:         """
        pass

    def delete_billing_group(self, billingGroupName: str, expectedVersion: OptionalVersion=None) -> DeleteBillingGroupResponse:
        """Deletes the billing group.

        :param self:
        :param expectedVersion:  The expected version of the billing group. If the version of the billing group does not match the expected version specified in the request, the <code>DeleteBillingGroup</code> request is rejected with a <code>VersionConflictException</code>.<br/>billingGroupName:  The name of the billing group.<br/>
        :return:         """
        pass

    def delete_ca_certificate(self, certificateId: str) -> DeleteCACertificateResponse:
        """Deletes a registered CA certificate.

        :param self:
        :param certificateId:  The ID of the certificate to delete. (The last part of the certificate ARN contains the certificate ID.)<br/>
        :return:         """
        pass

    def delete_certificate(self, certificateId: str, forceDelete: ForceDelete=None):
        """Deletes the specified certificate. A certificate cannot be deleted if it has a policy attached to it or if its status is set to ACTIVE. To delete a certificate, first use the <a>DetachPrincipalPolicy</a> API to detach all policies. Next, use the <a>UpdateCertificate</a> API to set the certificate to the INACTIVE status.

        :param self:
        :param forceDelete:  Forces a certificate request to be deleted.<br/>certificateId:  The ID of the certificate. (The last part of the certificate ARN contains the certificate ID.)<br/>
        :return:         """
        pass

    def delete_dynamic_thing_group(self, thingGroupName: str, expectedVersion: OptionalVersion=None) -> DeleteDynamicThingGroupResponse:
        """Deletes a dynamic thing group.

        :param self:
        :param expectedVersion:  The expected version of the dynamic thing group to delete.<br/>thingGroupName:  The name of the dynamic thing group to delete.<br/>
        :return:         """
        pass

    def delete_job(self, jobId: str, force: ForceFlag=None):
        """Deletes a job and its related job executions. Deleting a job may take time, depending on the number of job executions created for the job and various other factors. While the job is being deleted, the status of the job will be shown as "DELETION_IN_PROGRESS". Attempting to delete or cancel a job whose status is already "DELETION_IN_PROGRESS" will result in an error. Only 10 jobs may have status "DELETION_IN_PROGRESS" at the same time, or a LimitExceededException will occur.

        :param self:
        :param force:  (Optional) When true, you can delete a job which is "IN_PROGRESS". Otherwise, you can only delete a job which is in a terminal state ("COMPLETED" or "CANCELED") or an exception will occur. The default is false. <note> Deleting a job which is "IN_PROGRESS", will cause a device which is executing the job to be unable to access job information or update the job execution status. Use caution and ensure that each device executing a job which is deleted is able to recover to a valid state. </note><br/>jobId:  The ID of the job to be deleted. After a job deletion is completed, you may reuse this jobId when you create a new job. However, this is not recommended, and you must ensure that your devices are not using the jobId to refer to the deleted job.<br/>
        :return:         """
        pass

    def delete_job_execution(self, executionNumber: ExecutionNumber, thingName: str, jobId: str, force: ForceFlag=None):
        """Deletes a job execution.

        :param self:
        :param force:  (Optional) When true, you can delete a job execution which is "IN_PROGRESS". Otherwise, you can only delete a job execution which is in a terminal state ("SUCCEEDED", "FAILED", "REJECTED", "REMOVED" or "CANCELED") or an exception will occur. The default is false. <note> Deleting a job execution which is "IN_PROGRESS", will cause the device to be unable to access job information or update the job execution status. Use caution and ensure that the device is able to recover to a valid state. </note><br/>executionNumber:  The ID of the job execution to be deleted. The <code>executionNumber</code> refers to the execution of a particular job on a particular device. Note that once a job execution is deleted, the <code>executionNumber</code> may be reused by IoT, so be sure you get and use the correct value here.<br/>thingName:  The name of the thing whose job execution will be deleted.<br/>jobId:  The ID of the job whose execution on a particular device will be deleted.<br/>
        :return:         """
        pass

    def delete_ota_update(self, otaUpdateId: str, deleteStream: DeleteStream=None, forceDeleteAWSJob: ForceDeleteAWSJob=None) -> DeleteOTAUpdateResponse:
        """Delete an OTA update.

        :param self:
        :param forceDeleteAWSJob:  Specifies if the AWS Job associated with the OTA update should be deleted with the OTA update is deleted.<br/>deleteStream:  Specifies if the stream associated with an OTA update should be deleted when the OTA update is deleted.<br/>otaUpdateId:  The OTA update ID to delete.<br/>
        :return:         """
        pass

    def delete_policy(self, policyName: str):
        """Deletes the specified policy. A policy cannot be deleted if it has non-default versions or it is attached to any certificate. To delete a policy, use the DeletePolicyVersion API to delete all non-default versions of the policy; use the DetachPrincipalPolicy API to detach the policy from any certificate; and then use the DeletePolicy API to delete the policy. When a policy is deleted using DeletePolicy, its default version is deleted with it.

        :param self:
        :param policyName:  The name of the policy to delete.<br/>
        :return:         """
        pass

    def delete_policy_version(self, policyVersionId: str, policyName: str):
        """Deletes the specified version of the specified policy. You cannot delete the default version of a policy using this API. To delete the default version of a policy, use <a>DeletePolicy</a>. To find out which version of a policy is marked as the default version, use ListPolicyVersions.

        :param self:
        :param policyVersionId:  The policy version ID.<br/>policyName:  The name of the policy.<br/>
        :return:         """
        pass

    def delete_registration_code(self) -> DeleteRegistrationCodeResponse:
        """Deletes a CA certificate registration code.

        :param self:
        :param 
        :return:         """
        pass

    def delete_role_alias(self, roleAlias: str) -> DeleteRoleAliasResponse:
        """Deletes a role alias

        :param self:
        :param roleAlias:  The role alias to delete.<br/>
        :return:         """
        pass

    def delete_scheduled_audit(self, scheduledAuditName: str) -> DeleteScheduledAuditResponse:
        """Deletes a scheduled audit.

        :param self:
        :param scheduledAuditName:  The name of the scheduled audit you want to delete.<br/>
        :return:         """
        pass

    def delete_security_profile(self, securityProfileName: str, expectedVersion: OptionalVersion=None) -> DeleteSecurityProfileResponse:
        """Deletes a Device Defender security profile.

        :param self:
        :param expectedVersion:  The expected version of the security profile. A new version is generated whenever the security profile is updated. If you specify a value that is different than the actual version, a <code>VersionConflictException</code> is thrown.<br/>securityProfileName:  The name of the security profile to be deleted.<br/>
        :return:         """
        pass

    def delete_stream(self, streamId: str) -> DeleteStreamResponse:
        """Deletes a stream.

        :param self:
        :param streamId:  The stream ID.<br/>
        :return:         """
        pass

    def delete_thing(self, thingName: str, expectedVersion: OptionalVersion=None) -> DeleteThingResponse:
        """Deletes the specified thing. Returns successfully with no error if the deletion is successful or you specify a thing that doesn't exist.

        :param self:
        :param expectedVersion:  The expected version of the thing record in the registry. If the version of the record in the registry does not match the expected version specified in the request, the <code>DeleteThing</code> request is rejected with a <code>VersionConflictException</code>.<br/>thingName:  The name of the thing to delete.<br/>
        :return:         """
        pass

    def delete_thing_group(self, thingGroupName: str, expectedVersion: OptionalVersion=None) -> DeleteThingGroupResponse:
        """Deletes a thing group.

        :param self:
        :param expectedVersion:  The expected version of the thing group to delete.<br/>thingGroupName:  The name of the thing group to delete.<br/>
        :return:         """
        pass

    def delete_thing_type(self, thingTypeName: str) -> DeleteThingTypeResponse:
        """Deletes the specified thing type. You cannot delete a thing type if it has things associated with it. To delete a thing type, first mark it as deprecated by calling <a>DeprecateThingType</a>, then remove any associated things by calling <a>UpdateThing</a> to change the thing type on any associated thing, and finally use <a>DeleteThingType</a> to delete the thing type.

        :param self:
        :param thingTypeName:  The name of the thing type.<br/>
        :return:         """
        pass

    def delete_topic_rule(self, ruleName: str):
        """Deletes the rule.

        :param self:
        :param ruleName:  The name of the rule.<br/>
        :return:         """
        pass

    def delete_v2_logging_level(self, targetName: str, targetType: str):
        """Deletes a logging level.

        :param self:
        :param targetName:  The name of the resource for which you are configuring logging.<br/>targetType:  The type of resource for which you are configuring logging. Must be <code>THING_Group</code>.<br/>
        :return:         """
        pass

    def deprecate_thing_type(self, thingTypeName: str, undoDeprecate: UndoDeprecate=None) -> DeprecateThingTypeResponse:
        """Deprecates a thing type. You can not associate new things with deprecated thing type.

        :param self:
        :param undoDeprecate:  Whether to undeprecate a deprecated thing type. If <b>true</b>, the thing type will not be deprecated anymore and you can associate it with things.<br/>thingTypeName:  The name of the thing type to deprecate.<br/>
        :return:         """
        pass

    def describe_account_audit_configuration(self) -> DescribeAccountAuditConfigurationResponse:
        """Gets information about the Device Defender audit settings for this account. Settings include how audit notifications are sent and which audit checks are enabled or disabled.

        :param self:
        :param 
        :return: auditCheckConfigurations:  Which audit checks are enabled and disabled for this account.<br/>auditNotificationTargetConfigurations:  Information about the targets to which audit notifications are sent for this account.<br/>roleArn:  The ARN of the role that grants permission to AWS IoT to access information about your devices, policies, certificates and other items as necessary when performing an audit. On the first call to <code>UpdateAccountAuditConfiguration</code> this parameter is required.<br/>        """
        pass

    def describe_audit_task(self, taskId: str) -> DescribeAuditTaskResponse:
        """Gets information about a Device Defender audit.

        :param self:
        :param taskId:  The ID of the audit whose information you want to get.<br/>
        :return: auditDetails:  Detailed information about each check performed during this audit.<br/>scheduledAuditName:  The name of the scheduled audit (only if the audit was a scheduled audit).<br/>taskStatistics:  Statistical information about the audit.:<br/>taskStartTime:  The time the audit started.<br/>taskType:  The type of audit: "ON_DEMAND_AUDIT_TASK" or "SCHEDULED_AUDIT_TASK".<br/>taskStatus:  The status of the audit: one of "IN_PROGRESS", "COMPLETED", "FAILED", or "CANCELED".<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>canceledChecks</i> The number of checks that did not run because the audit was canceled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>failedChecks</i> The number of checks <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>nonCompliantChecks</i> The number of checks that found non-compliant resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>compliantChecks</i> The number of checks that found compliant resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>waitingForDataCollectionChecks</i> The number of checks waiting for data collection.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>inProgressChecks</i> The number of checks in progress.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>totalChecks</i> The number of checks in this audit.<br/>        """
        pass

    def describe_authorizer(self, authorizerName: str) -> DescribeAuthorizerResponse:
        """Describes an authorizer.

        :param self:
        :param authorizerName:  The name of the authorizer to describe.<br/>
        :return: authorizerDescription:  The authorizer description.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastModifiedDate</i> The UNIX timestamp of when the authorizer was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDate</i> The UNIX timestamp of when the authorizer was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the authorizer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tokenSigningPublicKeys</i> The public keys used to validate the token signature returned by your custom authentication service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tokenKeyName</i> The key used to extract the token from the HTTP headers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authorizerFunctionArn</i> The authorizer's Lambda function ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authorizerArn</i> The authorizer ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authorizerName</i> The authorizer name.<br/>        """
        pass

    def describe_billing_group(self, billingGroupName: str) -> DescribeBillingGroupResponse:
        """Returns information about a billing group.

        :param self:
        :param billingGroupName:  The name of the billing group.<br/>
        :return: billingGroupMetadata:  Additional information about the billing group.:<br/>billingGroupProperties:  The properties of the billing group.:<br/>version:  The version of the billing group.<br/>billingGroupArn:  The ARN of the billing group.<br/>billingGroupId:  The ID of the billing group.<br/>billingGroupName:  The name of the billing group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>billingGroupDescription</i> The description of the billing group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDate</i> The date the billing group was created.<br/>        """
        pass

    def describe_ca_certificate(self, certificateId: str) -> DescribeCACertificateResponse:
        """Describes a registered CA certificate.

        :param self:
        :param certificateId:  The CA certificate identifier.<br/>
        :return: registrationConfig:  Information about the registration configuration.:<br/>certificateDescription:  The CA certificate description.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>validity</i> When the CA certificate is valid.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>validity</b>: When the CA certificate is valid.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>generationId</i> The generation ID of the CA certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customerVersion</i> The customer version of the CA certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastModifiedDate</i> The date the CA certificate was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>autoRegistrationStatus</i> Whether the CA certificate configured for auto registration of device certificates. Valid values are "ENABLE" and "DISABLE"<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDate</i> The date the CA certificate was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ownedBy</i> The owner of the CA certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>certificatePem</i> The CA certificate data, in PEM format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of a CA certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>certificateId</i> The CA certificate ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>certificateArn</i> The CA certificate ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>notAfter</i> The certificate is not valid after this date.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>notBefore</i> The certificate is not valid before this date.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The ARN of the role.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>templateBody</i> The template body.<br/>        """
        pass

    def describe_certificate(self, certificateId: str) -> DescribeCertificateResponse:
        """Gets information about the specified certificate.

        :param self:
        :param certificateId:  The ID of the certificate. (The last part of the certificate ARN contains the certificate ID.)<br/>
        :return: certificateDescription:  The description of the certificate.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>validity</i> When the certificate is valid.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>validity</b>: When the certificate is valid.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>generationId</i> The generation ID of the certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>transferData</i> The transfer data.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>transferData</b>: The transfer data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>customerVersion</i> The customer version of the certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastModifiedDate</i> The date and time the certificate was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDate</i> The date and time the certificate was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>previousOwnedBy</i> The ID of the AWS account of the previous owner of the certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ownedBy</i> The ID of the AWS account that owns the certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>certificatePem</i> The certificate data, in PEM format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>caCertificateId</i> The certificate ID of the CA certificate used to sign this certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>certificateId</i> The ID of the certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>certificateArn</i> The ARN of the certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>rejectDate</i> The date the transfer was rejected.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>acceptDate</i> The date the transfer was accepted.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>transferDate</i> The date the transfer took place.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>rejectReason</i> The reason why the transfer was rejected.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>transferMessage</i> The transfer message.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>notAfter</i> The certificate is not valid after this date.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>notBefore</i> The certificate is not valid before this date.<br/>        """
        pass

    def describe_default_authorizer(self) -> DescribeDefaultAuthorizerResponse:
        """Describes the default authorizer.

        :param self:
        :param 
        :return: authorizerDescription:  The default authorizer's description.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastModifiedDate</i> The UNIX timestamp of when the authorizer was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDate</i> The UNIX timestamp of when the authorizer was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the authorizer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tokenSigningPublicKeys</i> The public keys used to validate the token signature returned by your custom authentication service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tokenKeyName</i> The key used to extract the token from the HTTP headers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authorizerFunctionArn</i> The authorizer's Lambda function ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authorizerArn</i> The authorizer ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>authorizerName</i> The authorizer name.<br/>        """
        pass

    def describe_endpoint(self, endpointType: str=None) -> DescribeEndpointResponse:
        """Returns a unique endpoint specific to the AWS account making the call.

        :param self:
        :param endpointType:  The endpoint type. Valid endpoint types include: <ul> <li>  <code>iot:Data</code> - Returns a VeriSign signed data endpoint. </li> </ul> <ul> <li>  <code>iot:Data-ATS</code> - Returns an ATS signed data endpoint. </li> </ul> <ul> <li>  <code>iot:CredentialProvider</code> - Returns an AWS IoT credentials provider API endpoint. </li> </ul> <ul> <li>  <code>iot:Jobs</code> - Returns an AWS IoT device management Jobs API endpoint. </li> </ul><br/>
        :return: endpointAddress:  The endpoint. The format of the endpoint is as follows: <i>identifier</i>.iot.<i>region</i>.amazonaws.com.<br/>        """
        pass

    def describe_event_configurations(self) -> DescribeEventConfigurationsResponse:
        """Describes event configurations.

        :param self:
        :param 
        :return: lastModifiedDate:  The date the event configurations were last modified.<br/>creationDate:  The creation date of the event configuration.<br/>eventConfigurations:  The event configurations.<br/>        """
        pass

    def describe_index(self, indexName: str) -> DescribeIndexResponse:
        """Describes a search index.

        :param self:
        :param indexName:  The index name.<br/>
        :return: schema:  Contains a value that specifies the type of indexing performed. Valid values are: <ul> <li> REGISTRY – Your thing index contains only registry data. </li> <li> REGISTRY_AND_SHADOW - Your thing index contains registry data and shadow data. </li> <li> REGISTRY_AND_CONNECTIVITY_STATUS - Your thing index contains registry data and thing connectivity status data. </li> <li> REGISTRY_AND_SHADOW_AND_CONNECTIVITY_STATUS - Your thing index contains registry data, shadow data, and thing connectivity status data. </li> </ul><br/>indexStatus:  The index status.<br/>indexName:  The index name.<br/>        """
        pass

    def describe_job(self, jobId: str) -> DescribeJobResponse:
        """Describes a job.

        :param self:
        :param jobId:  The unique identifier you assigned to this job when it was created.<br/>
        :return: job:  Information about the job.:<br/>documentSource:  An S3 link to the job document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>timeoutConfig</i> Specifies the amount of time each device has to finish its execution of the job. A timer is started when the job execution status is set to <code>IN_PROGRESS</code>. If the job execution status is not set to another terminal state before the timer expires, it will be automatically set to <code>TIMED_OUT</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>timeoutConfig</b>: Specifies the amount of time each device has to finish its execution of the job. A timer is started when the job execution status is set to <code>IN_PROGRESS</code>. If the job execution status is not set to another terminal state before the timer expires, it will be automatically set to <code>TIMED_OUT</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jobProcessDetails</i> Details about the job process.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>jobProcessDetails</b>: Details about the job process.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>completedAt</i> The time, in seconds since the epoch, when the job was completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedAt</i> The time, in seconds since the epoch, when the job was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The time, in seconds since the epoch, when the job was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>abortConfig</i> Configuration for criteria to abort the job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>abortConfig</b>: Configuration for criteria to abort the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jobExecutionsRolloutConfig</i> Allows you to create a staged rollout of a job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>jobExecutionsRolloutConfig</b>: Allows you to create a staged rollout of a job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>presignedUrlConfig</i> Configuration for pre-signed S3 URLs.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>presignedUrlConfig</b>: Configuration for pre-signed S3 URLs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> A short text description of the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>targets</i> A list of IoT things and thing groups to which the job should be sent.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>comment</i> If the job was updated, describes the reason for the update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>reasonCode</i> If the job was updated, provides the reason code for the update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>forceCanceled</i> Will be <code>true</code> if the job was canceled with the optional <code>force</code> parameter set to <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the job, one of <code>IN_PROGRESS</code>, <code>CANCELED</code>, <code>DELETION_IN_PROGRESS</code> or <code>COMPLETED</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>targetSelection</i> Specifies whether the job will continue to run (CONTINUOUS), or will be complete after all those things specified as targets have completed the job (SNAPSHOT). If continuous, the job may also be run on a thing when a change is detected in a target. For example, a job will run on a device when the thing representing the device is added to a target group, even after the job was completed by all things originally in the group. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jobId</i> The unique identifier you assigned to this job when it was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jobArn</i> An ARN identifying the job with format "arn:aws:iot:region:account:job/jobId".<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>expiresInSec</i> How long (in seconds) pre-signed URLs are valid. Valid values are 60 - 3600, the default value is 3600 seconds. Pre-signed URLs are generated when Jobs receives an MQTT request for the job document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The ARN of an IAM role that grants grants permission to download files from the S3 bucket where the job data/updates are stored. The role must also grant permission for IoT to download the files.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>maximumPerMinute</i> The maximum number of things that will be notified of a pending job, per minute. This parameter allows you to create a staged rollout.<br/>The rate of increase for a job rollout. This parameter allows you to define an exponential rate for a job rollout.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>exponentialRate</b>: The rate of increase for a job rollout. This parameter allows you to define an exponential rate for a job rollout.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>criteriaList</i> The list of abort criteria to define rules to abort the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>numberOfTimedOutThings</i> The number of things whose job execution status is <code>TIMED_OUT</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>numberOfRemovedThings</i> The number of things that are no longer scheduled to execute the job because they have been deleted or have been removed from the group that was a target of the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>numberOfInProgressThings</i> The number of things currently executing the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>numberOfQueuedThings</i> The number of things that are awaiting execution of the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>numberOfRejectedThings</i> The number of things that rejected the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>numberOfFailedThings</i> The number of things that failed executing the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>numberOfSucceededThings</i> The number of things which successfully completed the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>numberOfCanceledThings</i> The number of things that cancelled the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>processingTargets</i> The target devices to which the job execution is being rolled out. This value will be null after the job execution has finished rolling out to all the target devices.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>inProgressTimeoutInMinutes</i> Specifies the amount of time, in minutes, this device has to finish execution of this job. The timeout interval can be anywhere between 1 minute and 7 days (1 to 10080 minutes). The in progress timer can't be updated and will apply to all job executions for the job. Whenever a job execution remains in the IN_PROGRESS status for longer than this interval, the job execution will fail and switch to the terminal <code>TIMED_OUT</code> status.<br/>        """
        pass

    def describe_job_execution(self, thingName: str, jobId: str, executionNumber: ExecutionNumber=None) -> DescribeJobExecutionResponse:
        """Describes a job execution.

        :param self:
        :param executionNumber:  A string (consisting of the digits "0" through "9" which is used to specify a particular job execution on a particular device.<br/>thingName:  The name of the thing on which the job execution is running.<br/>jobId:  The unique identifier you assigned to this job when it was created.<br/>
        :return: execution:  Information about the job execution.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>approximateSecondsBeforeTimedOut</i> The estimated number of seconds that remain before the job execution status will be changed to <code>TIMED_OUT</code>. The timeout interval can be anywhere between 1 minute and 7 days (1 to 10080 minutes). The actual job execution timeout can occur up to 60 seconds later than the estimated duration. This value will not be included if the job execution has reached a terminal status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>versionNumber</i> The version of the job execution. Job execution versions are incremented each time they are updated by a device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>executionNumber</i> A string (consisting of the digits "0" through "9") which identifies this particular job execution on this particular device. It can be used in commands which return or update job execution information. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedAt</i> The time, in seconds since the epoch, when the job execution was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>startedAt</i> The time, in seconds since the epoch, when the job execution started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>queuedAt</i> The time, in seconds since the epoch, when the job execution was queued.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>thingArn</i> The ARN of the thing on which the job execution is running.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusDetails</i> A collection of name/value pairs that describe the status of the job execution.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>statusDetails</b>: A collection of name/value pairs that describe the status of the job execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>forceCanceled</i> Will be <code>true</code> if the job execution was canceled with the optional <code>force</code> parameter set to <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the job execution (IN_PROGRESS, QUEUED, FAILED, SUCCEEDED, TIMED_OUT, CANCELED, or REJECTED).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jobId</i> The unique identifier you assigned to the job when it was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>detailsMap</i> The job execution status.<br/>        """
        pass

    def describe_role_alias(self, roleAlias: str) -> DescribeRoleAliasResponse:
        """Describes a role alias.

        :param self:
        :param roleAlias:  The role alias to describe.<br/>
        :return: roleAliasDescription:  The role alias description.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastModifiedDate</i> The UNIX timestamp of when the role alias was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDate</i> The UNIX timestamp of when the role alias was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>credentialDurationSeconds</i> The number of seconds for which the credential is valid.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>owner</i> The role alias owner.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The role ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleAliasArn</i> The ARN of the role alias.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleAlias</i> The role alias.<br/>        """
        pass

    def describe_scheduled_audit(self, scheduledAuditName: str) -> DescribeScheduledAuditResponse:
        """Gets information about a scheduled audit.

        :param self:
        :param scheduledAuditName:  The name of the scheduled audit whose information you want to get.<br/>
        :return: scheduledAuditArn:  The ARN of the scheduled audit.<br/>scheduledAuditName:  The name of the scheduled audit.<br/>targetCheckNames:  Which checks are performed during the scheduled audit. (Note that checks must be enabled for your account. (Use <code>DescribeAccountAuditConfiguration</code> to see the list of all checks including those that are enabled or <code>UpdateAccountAuditConfiguration</code> to select which checks are enabled.)<br/>dayOfWeek:  The day of the week on which the scheduled audit takes place. One of "SUN", "MON", "TUE", "WED", "THU", "FRI" or "SAT".<br/>dayOfMonth:  The day of the month on which the scheduled audit takes place. Will be "1" through "31" or "LAST". If days 29-31 are specified, and the month does not have that many days, the audit takes place on the "LAST" day of the month.<br/>frequency:  How often the scheduled audit takes place. One of "DAILY", "WEEKLY", "BIWEEKLY" or "MONTHLY". The actual start time of each audit is determined by the system.<br/>        """
        pass

    def describe_security_profile(self, securityProfileName: str) -> DescribeSecurityProfileResponse:
        """Gets information about a Device Defender security profile.

        :param self:
        :param securityProfileName:  The name of the security profile whose information you want to get.<br/>
        :return: lastModifiedDate:  The time the security profile was last modified.<br/>creationDate:  The time the security profile was created.<br/>version:  The version of the security profile. A new version is generated whenever the security profile is updated.<br/>additionalMetricsToRetain:  A list of metrics whose data is retained (stored). By default, data is retained for any metric used in the profile's <code>behaviors</code> but it is also retained for any metric specified here.<br/>alertTargets:  Where the alerts are sent. (Alerts are always sent to the console.)<br/>behaviors:  Specifies the behaviors that, when violated by a device (thing), cause an alert.<br/>securityProfileDescription:  A description of the security profile (associated with the security profile when it was created or updated).<br/>securityProfileArn:  The ARN of the security profile.<br/>securityProfileName:  The name of the security profile.<br/>        """
        pass

    def describe_stream(self, streamId: str) -> DescribeStreamResponse:
        """Gets information about a stream.

        :param self:
        :param streamId:  The stream ID.<br/>
        :return: streamInfo:  Information about the stream.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> An IAM role AWS IoT assumes to access your S3 files.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedAt</i> The date when the stream was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The date when the stream was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>files</i> The files to stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The description of the stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>streamVersion</i> The stream version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>streamArn</i> The stream ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>streamId</i> The stream ID.<br/>        """
        pass

    def describe_thing(self, thingName: str) -> DescribeThingResponse:
        """Gets information about the specified thing.

        :param self:
        :param thingName:  The name of the thing.<br/>
        :return: billingGroupName:  The name of the billing group the thing belongs to.<br/>version:  The current version of the thing record in the registry. <note> To avoid unintentional changes to the information in the registry, you can pass the version information in the <code>expectedVersion</code> parameter of the <code>UpdateThing</code> and <code>DeleteThing</code> calls. </note><br/>attributes:  The thing attributes.<br/>thingTypeName:  The thing type name.<br/>thingArn:  The ARN of the thing to describe.<br/>thingId:  The ID of the thing to describe.<br/>thingName:  The name of the thing.<br/>defaultClientId:  The default client ID.<br/>        """
        pass

    def describe_thing_group(self, thingGroupName: str) -> DescribeThingGroupResponse:
        """Describe a thing group.

        :param self:
        :param thingGroupName:  The name of the thing group.<br/>
        :return: status:  The dynamic thing group status.<br/>queryVersion:  The dynamic thing group query version.<br/>queryString:  The dynamic thing group search query string.<br/>indexName:  The dynamic thing group index name.<br/>thingGroupMetadata:  Thing group metadata.:<br/>thingGroupProperties:  The thing group properties.:<br/>version:  The version of the thing group.<br/>thingGroupArn:  The thing group ARN.<br/>thingGroupId:  The thing group ID.<br/>thingGroupName:  The name of the thing group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>attributePayload</i> The thing group attributes in JSON format.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>attributePayload</b>: The thing group attributes in JSON format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>thingGroupDescription</i> The thing group description.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>merge</i> Specifies whether the list of attributes provided in the <code>AttributePayload</code> is merged with the attributes stored in the registry, instead of overwriting them. To remove an attribute, call <code>UpdateThing</code> with an empty attribute value. <note> The <code>merge</code> attribute is only valid when calling <code>UpdateThing</code>. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>attributes</i> A JSON string containing up to three key-value pair in JSON format. For example:  <code>{\"attributes\":{\"string1\":\"string2\"}}</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDate</i> The UNIX timestamp of when the thing group was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>rootToParentThingGroups</i> The root parent thing group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>parentGroupName</i> The parent thing group name.<br/>        """
        pass

    def describe_thing_registration_task(self, taskId: str) -> DescribeThingRegistrationTaskResponse:
        """Describes a bulk thing provisioning task.

        :param self:
        :param taskId:  The task ID.<br/>
        :return: percentageProgress:  The progress of the bulk provisioning task expressed as a percentage.<br/>failureCount:  The number of things that failed to be provisioned.<br/>successCount:  The number of things successfully provisioned.<br/>message:  The message.<br/>status:  The status of the bulk thing provisioning task.<br/>roleArn:  The role ARN that grants access to the input file bucket.<br/>inputFileKey:  The input file key.<br/>inputFileBucket:  The S3 bucket that contains the input file.<br/>templateBody:  The task's template.<br/>lastModifiedDate:  The date when the task was last modified.<br/>creationDate:  The task creation date.<br/>taskId:  The task ID.<br/>        """
        pass

    def describe_thing_type(self, thingTypeName: str) -> DescribeThingTypeResponse:
        """Gets information about the specified thing type.

        :param self:
        :param thingTypeName:  The name of the thing type.<br/>
        :return: thingTypeMetadata:  The ThingTypeMetadata contains additional information about the thing type including: creation date and time, a value indicating whether the thing type is deprecated, and a date and time when it was deprecated.:<br/>thingTypeProperties:  The ThingTypeProperties contains information about the thing type including description, and a list of searchable thing attribute names.:<br/>thingTypeArn:  The thing type ARN.<br/>thingTypeId:  The thing type ID.<br/>thingTypeName:  The name of the thing type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>searchableAttributes</i> A list of searchable thing attribute names.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>thingTypeDescription</i> The description of the thing type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDate</i> The date and time when the thing type was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deprecationDate</i> The date and time when the thing type was deprecated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deprecated</i> Whether the thing type is deprecated. If <b>true</b>, no new things could be associated with this type.<br/>        """
        pass

    def detach_policy(self, target: str, policyName: str):
        """Detaches a policy from the specified target.

        :param self:
        :param target:  The target from which the policy will be detached.<br/>policyName:  The policy to detach.<br/>
        :return:         """
        pass

    def detach_principal_policy(self, principal: str, policyName: str):
        """Removes the specified policy from the specified certificate.  <b>Note:</b> This API is deprecated. Please use <a>DetachPolicy</a> instead.

        :param self:
        :param principal:  The principal. If the principal is a certificate, specify the certificate ARN. If the principal is an Amazon Cognito identity, specify the identity ID.<br/>policyName:  The name of the policy to detach.<br/>
        :return:         """
        pass

    def detach_security_profile(self, securityProfileTargetArn: str, securityProfileName: str) -> DetachSecurityProfileResponse:
        """Disassociates a Device Defender security profile from a thing group or from this account.

        :param self:
        :param securityProfileTargetArn:  The ARN of the thing group from which the security profile is detached.<br/>securityProfileName:  The security profile that is detached.<br/>
        :return:         """
        pass

    def detach_thing_principal(self, principal: str, thingName: str) -> DetachThingPrincipalResponse:
        """Detaches the specified principal from the specified thing. A principal can be X.509 certificates, IAM users, groups, and roles, Amazon Cognito identities or federated identities. <note> This call is asynchronous. It might take several seconds for the detachment to propagate. </note>

        :param self:
        :param principal:  If the principal is a certificate, this value must be ARN of the certificate. If the principal is an Amazon Cognito identity, this value must be the ID of the Amazon Cognito identity.<br/>thingName:  The name of the thing.<br/>
        :return:         """
        pass

    def disable_topic_rule(self, ruleName: str):
        """Disables the rule.

        :param self:
        :param ruleName:  The name of the rule to disable.<br/>
        :return:         """
        pass

    def enable_topic_rule(self, ruleName: str):
        """Enables the rule.

        :param self:
        :param ruleName:  The name of the topic rule to enable.<br/>
        :return:         """
        pass

    def get_effective_policies(self, principal: str=None, cognitoIdentityPoolId: str=None, thingName: str=None) -> GetEffectivePoliciesResponse:
        """Gets a list of the policies that have an effect on the authorization behavior of the specified device when it connects to the AWS IoT device gateway.

        :param self:
        :param thingName:  The thing name.<br/>cognitoIdentityPoolId:  The Cognito identity pool ID.<br/>principal:  The principal.<br/>
        :return: effectivePolicies:  The effective policies.<br/>        """
        pass

    def get_indexing_configuration(self) -> GetIndexingConfigurationResponse:
        """Gets the search configuration.

        :param self:
        :param 
        :return: thingGroupIndexingConfiguration:  The index configuration.:<br/>thingIndexingConfiguration:  Thing indexing configuration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>thingConnectivityIndexingMode</i> Thing connectivity indexing mode. Valid values are:  <ul> <li> STATUS – Your thing index contains connectivity status. To enable thing connectivity indexing, thingIndexMode must not be set to OFF. </li> <li> OFF - Thing connectivity status indexing is disabled. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>thingIndexingMode</i> Thing indexing mode. Valid values are: <ul> <li> REGISTRY – Your thing index contains registry data only. </li> <li> REGISTRY_AND_SHADOW - Your thing index contains registry and shadow data. </li> <li> OFF - Thing indexing is disabled. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>thingGroupIndexingMode</i> Thing group indexing mode.<br/>        """
        pass

    def get_job_document(self, jobId: str) -> GetJobDocumentResponse:
        """Gets a job document.

        :param self:
        :param jobId:  The unique identifier you assigned to this job when it was created.<br/>
        :return: document:  The job document content.<br/>        """
        pass

    def get_logging_options(self) -> GetLoggingOptionsResponse:
        """Gets the logging options. NOTE: use of this command is not recommended. Use <code>GetV2LoggingOptions</code> instead.

        :param self:
        :param 
        :return: logLevel:  The logging level.<br/>roleArn:  The ARN of the IAM role that grants access.<br/>        """
        pass

    def get_ota_update(self, otaUpdateId: str) -> GetOTAUpdateResponse:
        """Gets an OTA update.

        :param self:
        :param otaUpdateId:  The OTA update ID.<br/>
        :return: otaUpdateInfo:  The OTA update info.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>additionalParameters</i> A collection of name/value pairs<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorInfo</i> Error information associated with the OTA update.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>errorInfo</b>: Error information associated with the OTA update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>awsIotJobArn</i> The AWS IoT job ARN associated with the OTA update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>awsIotJobId</i> The AWS IoT job ID associated with the OTA update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>otaUpdateStatus</i> The status of the OTA update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>otaUpdateFiles</i> A list of files associated with the OTA update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>targetSelection</i> Specifies whether the OTA update will continue to run (CONTINUOUS), or will be complete after all those things specified as targets have completed the OTA update (SNAPSHOT). If continuous, the OTA update may also be run on a thing when a change is detected in a target. For example, an OTA update will run on a thing when the thing is added to a target group, even after the OTA update was completed by all things originally in the group. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>awsJobExecutionsRolloutConfig</i> Configuration for the rollout of OTA updates.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>awsJobExecutionsRolloutConfig</b>: Configuration for the rollout of OTA updates.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>targets</i> The targets of the OTA update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> A description of the OTA update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastModifiedDate</i> The date when the OTA update was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDate</i> The date when the OTA update was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>otaUpdateArn</i> The OTA update ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>otaUpdateId</i> The OTA update ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>maximumPerMinute</i> The maximum number of OTA update job executions started per minute.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>message</i> The error message.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>code</i> The error code.<br/>        """
        pass

    def get_policy(self, policyName: str) -> GetPolicyResponse:
        """Gets information about the specified policy with the policy document of the default version.

        :param self:
        :param policyName:  The name of the policy.<br/>
        :return: generationId:  The generation ID of the policy.<br/>lastModifiedDate:  The date the policy was last modified.<br/>creationDate:  The date the policy was created.<br/>defaultVersionId:  The default policy version ID.<br/>policyDocument:  The JSON document that describes the policy.<br/>policyArn:  The policy ARN.<br/>policyName:  The policy name.<br/>        """
        pass

    def get_policy_version(self, policyVersionId: str, policyName: str) -> GetPolicyVersionResponse:
        """Gets information about the specified policy version.

        :param self:
        :param policyVersionId:  The policy version ID.<br/>policyName:  The name of the policy.<br/>
        :return: generationId:  The generation ID of the policy version.<br/>lastModifiedDate:  The date the policy version was last modified.<br/>creationDate:  The date the policy version was created.<br/>isDefaultVersion:  Specifies whether the policy version is the default.<br/>policyVersionId:  The policy version ID.<br/>policyDocument:  The JSON document that describes the policy.<br/>policyName:  The policy name.<br/>policyArn:  The policy ARN.<br/>        """
        pass

    def get_registration_code(self) -> GetRegistrationCodeResponse:
        """Gets a registration code used to register a CA certificate with AWS IoT.

        :param self:
        :param 
        :return: registrationCode:  The CA certificate registration code.<br/>        """
        pass

    def get_statistics(self, queryString: str, indexName: str=None, aggregationField: str=None, queryVersion: str=None) -> GetStatisticsResponse:
        """Gets statistics about things that match the specified query.

        :param self:
        :param queryVersion:  The version of the query used to search.<br/>aggregationField:  The aggregation field name. Currently not supported.<br/>queryString:  The query used to search. You can specify "*" for the query string to get the count of all indexed things in your AWS account.<br/>indexName:  The name of the index to search. The default value is <code>AWS_Things</code>.<br/>
        :return: statistics:  The statistics returned by the Fleet Indexing service based on the query and aggregation field.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>count</i> The count of things that match the query.<br/>        """
        pass

    def get_topic_rule(self, ruleName: str) -> GetTopicRuleResponse:
        """Gets information about the rule.

        :param self:
        :param ruleName:  The name of the rule.<br/>
        :return: rule:  The rule.:<br/>ruleArn:  The rule ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorAction</i> The action to perform when an error occurs.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>errorAction</b>: The action to perform when an error occurs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>awsIotSqlVersion</i> The version of the SQL rules engine to use when evaluating the rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ruleDisabled</i> Specifies whether the rule is disabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>actions</i> The actions associated with the rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The date and time the rule was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The description of the rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>sql</i> The SQL statement used to query the topic. When using a SQL query with multiple lines, be sure to escape the newline characters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ruleName</i> The name of the rule.<br/>Write to a DynamoDB table.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>dynamoDB</b>: Write to a DynamoDB table.        """
        pass

    def get_v2_logging_options(self) -> GetV2LoggingOptionsResponse:
        """Gets the fine grained logging options.

        :param self:
        :param 
        :return: disableAllLogs:  Disables all logs.<br/>defaultLogLevel:  The default log level.<br/>roleArn:  The IAM role ARN AWS IoT uses to write to your CloudWatch logs.<br/>        """
        pass

    def list_active_violations(self, thingName: str=None, securityProfileName: str=None, nextToken: str=None, maxResults: int=None) -> ListActiveViolationsResponse:
        """Lists the active violations for a given Device Defender security profile.

        :param self:
        :param maxResults:  The maximum number of results to return at one time.<br/>nextToken:  The token for the next set of results.<br/>securityProfileName:  The name of the Device Defender security profile for which violations are listed.<br/>thingName:  The name of the thing whose active violations are listed.<br/>
        :return: nextToken:  A token that can be used to retrieve the next set of results, or <code>null</code> if there are no additional results.<br/>activeViolations:  The list of active violations.<br/>        """
        pass

    def list_attached_policies(self, target: str, recursive: Recursive=None, marker: str=None, pageSize: int=None) -> ListAttachedPoliciesResponse:
        """Lists the policies attached to the specified thing group.

        :param self:
        :param pageSize:  The maximum number of results to be returned per request.<br/>marker:  The token to retrieve the next set of results.<br/>recursive:  When true, recursively list attached policies.<br/>target:  The group for which the policies will be listed.<br/>
        :return: nextMarker:  The token to retrieve the next set of results, or ``null`` if there are no more results.<br/>policies:  The policies.<br/>        """
        pass

    def list_audit_findings(self, taskId: str=None, checkName: str=None, resourceIdentifier: ResourceIdentifier=None, maxResults: int=None, nextToken: str=None, startTime: Timestamp=None, endTime: Timestamp=None) -> ListAuditFindingsResponse:
        """Lists the findings (results) of a Device Defender audit or of the audits performed during a specified time period. (Findings are retained for 180 days.)

        :param self:
        :param endTime:  A filter to limit results to those found before the specified time. You must specify either the startTime and endTime or the taskId, but not both.<br/>startTime:  A filter to limit results to those found after the specified time. You must specify either the startTime and endTime or the taskId, but not both.<br/>nextToken:  The token for the next set of results.<br/>maxResults:  The maximum number of results to return at one time. The default is 25.<br/>resourceIdentifier:  Information identifying the non-compliant resource.:<br/>checkName:  A filter to limit results to the findings for the specified audit check.<br/>taskId:  A filter to limit results to the audit with the specified ID. You must specify either the taskId or the startTime and endTime, but not both.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>account</i> The account with which the resource is associated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>policyVersionIdentifier</i> The version of the policy associated with the resource.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>policyVersionIdentifier</b>: The version of the policy associated with the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>clientId</i> The client ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cognitoIdentityPoolId</i> The ID of the Cognito Identity Pool.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>caCertificateId</i> The ID of the CA certificate used to authorize the certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deviceCertificateId</i> The ID of the certificate attached to the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>policyVersionId</i> The ID of the version of the policy associated with the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>policyName</i> The name of the policy.<br/>
        :return: nextToken:  A token that can be used to retrieve the next set of results, or <code>null</code> if there are no additional results.<br/>findings:  The findings (results) of the audit.<br/>        """
        pass

    def list_audit_tasks(self, endTime: Timestamp, startTime: Timestamp, taskType: str=None, taskStatus: str=None, nextToken: str=None, maxResults: int=None) -> ListAuditTasksResponse:
        """Lists the Device Defender audits that have been performed during a given time period.

        :param self:
        :param maxResults:  The maximum number of results to return at one time. The default is 25.<br/>nextToken:  The token for the next set of results.<br/>taskStatus:  A filter to limit the output to audits with the specified completion status: can be one of "IN_PROGRESS", "COMPLETED", "FAILED" or "CANCELED".<br/>taskType:  A filter to limit the output to the specified type of audit: can be one of "ON_DEMAND_AUDIT_TASK" or "SCHEDULED__AUDIT_TASK".<br/>endTime:  The end of the time period.<br/>startTime:  The beginning of the time period. Note that audit information is retained for a limited time (180 days). Requesting a start time prior to what is retained results in an "InvalidRequestException".<br/>
        :return: nextToken:  A token that can be used to retrieve the next set of results, or <code>null</code> if there are no additional results.<br/>tasks:  The audits that were performed during the specified time period.<br/>        """
        pass

    def list_authorizers(self, pageSize: int=None, marker: str=None, ascendingOrder: AscendingOrder=None, status: str=None) -> ListAuthorizersResponse:
        """Lists the authorizers registered in your account.

        :param self:
        :param status:  The status of the list authorizers request.<br/>ascendingOrder:  Return the list of authorizers in ascending alphabetical order.<br/>marker:  A marker used to get the next set of results.<br/>pageSize:  The maximum number of results to return at one time.<br/>
        :return: nextMarker:  A marker used to get the next set of results.<br/>authorizers:  The authorizers.<br/>        """
        pass

    def list_billing_groups(self, nextToken: str=None, maxResults: int=None, namePrefixFilter: str=None) -> ListBillingGroupsResponse:
        """Lists the billing groups you have created.

        :param self:
        :param namePrefixFilter:  Limit the results to billing groups whose names have the given prefix.<br/>maxResults:  The maximum number of results to return per request.<br/>nextToken:  The token to retrieve the next set of results.<br/>
        :return: nextToken:  The token used to get the next set of results, or <b>null</b> if there are no additional results.<br/>billingGroups:  The list of billing groups.<br/>        """
        pass

    def list_ca_certificates(self, pageSize: int=None, marker: str=None, ascendingOrder: AscendingOrder=None) -> ListCACertificatesResponse:
        """Lists the CA certificates registered for your AWS account. The results are paginated with a default page size of 25. You can use the returned marker to retrieve additional results.

        :param self:
        :param ascendingOrder:  Determines the order of the results.<br/>marker:  The marker for the next set of results.<br/>pageSize:  The result page size.<br/>
        :return: nextMarker:  The current position within the list of CA certificates.<br/>certificates:  The CA certificates registered in your AWS account.<br/>        """
        pass

    def list_certificates(self, pageSize: int=None, marker: str=None, ascendingOrder: AscendingOrder=None) -> ListCertificatesResponse:
        """Lists the certificates registered in your AWS account. The results are paginated with a default page size of 25. You can use the returned marker to retrieve additional results.

        :param self:
        :param ascendingOrder:  Specifies the order for results. If True, the results are returned in ascending order, based on the creation date.<br/>marker:  The marker for the next set of results.<br/>pageSize:  The result page size.<br/>
        :return: nextMarker:  The marker for the next set of results, or null if there are no additional results.<br/>certificates:  The descriptions of the certificates.<br/>        """
        pass

    def list_certificates_by_ca(self, caCertificateId: str, pageSize: int=None, marker: str=None, ascendingOrder: AscendingOrder=None) -> ListCertificatesByCAResponse:
        """List the device certificates signed by the specified CA certificate.

        :param self:
        :param ascendingOrder:  Specifies the order for results. If True, the results are returned in ascending order, based on the creation date.<br/>marker:  The marker for the next set of results.<br/>pageSize:  The result page size.<br/>caCertificateId:  The ID of the CA certificate. This operation will list all registered device certificate that were signed by this CA certificate.<br/>
        :return: nextMarker:  The marker for the next set of results, or null if there are no additional results.<br/>certificates:  The device certificates signed by the specified CA certificate.<br/>        """
        pass

    def list_indices(self, nextToken: str=None, maxResults: int=None) -> ListIndicesResponse:
        """Lists the search indices.

        :param self:
        :param maxResults:  The maximum number of results to return at one time.<br/>nextToken:  The token used to get the next set of results, or null if there are no additional results.<br/>
        :return: nextToken:  The token used to get the next set of results, or null if there are no additional results.<br/>indexNames:  The index names.<br/>        """
        pass

    def list_job_executions_for_job(self, jobId: str, status: str=None, maxResults: int=None, nextToken: str=None) -> ListJobExecutionsForJobResponse:
        """Lists the job executions for a job.

        :param self:
        :param nextToken:  The token to retrieve the next set of results.<br/>maxResults:  The maximum number of results to be returned per request.<br/>status:  The status of the job.<br/>jobId:  The unique identifier you assigned to this job when it was created.<br/>
        :return: nextToken:  The token for the next set of results, or <b>null</b> if there are no additional results.<br/>executionSummaries:  A list of job execution summaries.<br/>        """
        pass

    def list_job_executions_for_thing(self, thingName: str, status: str=None, maxResults: int=None, nextToken: str=None) -> ListJobExecutionsForThingResponse:
        """Lists the job executions for the specified thing.

        :param self:
        :param nextToken:  The token to retrieve the next set of results.<br/>maxResults:  The maximum number of results to be returned per request.<br/>status:  An optional filter that lets you search for jobs that have the specified status.<br/>thingName:  The thing name.<br/>
        :return: nextToken:  The token for the next set of results, or <b>null</b> if there are no additional results.<br/>executionSummaries:  A list of job execution summaries.<br/>        """
        pass

    def list_jobs(self, status: str=None, targetSelection: str=None, maxResults: int=None, nextToken: str=None, thingGroupName: str=None, thingGroupId: str=None) -> ListJobsResponse:
        """Lists jobs.

        :param self:
        :param thingGroupId:  A filter that limits the returned jobs to those for the specified group.<br/>thingGroupName:  A filter that limits the returned jobs to those for the specified group.<br/>nextToken:  The token to retrieve the next set of results.<br/>maxResults:  The maximum number of results to return per request.<br/>targetSelection:  Specifies whether the job will continue to run (CONTINUOUS), or will be complete after all those things specified as targets have completed the job (SNAPSHOT). If continuous, the job may also be run on a thing when a change is detected in a target. For example, a job will run on a thing when the thing is added to a target group, even after the job was completed by all things originally in the group. <br/>status:  An optional filter that lets you search for jobs that have the specified status.<br/>
        :return: nextToken:  The token for the next set of results, or <b>null</b> if there are no additional results.<br/>jobs:  A list of jobs.<br/>        """
        pass

    def list_ota_updates(self, maxResults: int=None, nextToken: str=None, otaUpdateStatus: str=None) -> ListOTAUpdatesResponse:
        """Lists OTA updates.

        :param self:
        :param otaUpdateStatus:  The OTA update job status.<br/>nextToken:  A token used to retrieve the next set of results.<br/>maxResults:  The maximum number of results to return at one time.<br/>
        :return: nextToken:  A token to use to get the next set of results.<br/>otaUpdates:  A list of OTA update jobs.<br/>        """
        pass

    def list_outgoing_certificates(self, pageSize: int=None, marker: str=None, ascendingOrder: AscendingOrder=None) -> ListOutgoingCertificatesResponse:
        """Lists certificates that are being transferred but not yet accepted.

        :param self:
        :param ascendingOrder:  Specifies the order for results. If True, the results are returned in ascending order, based on the creation date.<br/>marker:  The marker for the next set of results.<br/>pageSize:  The result page size.<br/>
        :return: nextMarker:  The marker for the next set of results.<br/>outgoingCertificates:  The certificates that are being transferred but not yet accepted.<br/>        """
        pass

    def list_policies(self, marker: str=None, pageSize: int=None, ascendingOrder: AscendingOrder=None) -> ListPoliciesResponse:
        """Lists your policies.

        :param self:
        :param ascendingOrder:  Specifies the order for results. If true, the results are returned in ascending creation order.<br/>pageSize:  The result page size.<br/>marker:  The marker for the next set of results.<br/>
        :return: nextMarker:  The marker for the next set of results, or null if there are no additional results.<br/>policies:  The descriptions of the policies.<br/>        """
        pass

    def list_policy_principals(self, policyName: str, marker: str=None, pageSize: int=None, ascendingOrder: AscendingOrder=None) -> ListPolicyPrincipalsResponse:
        """Lists the principals associated with the specified policy.  <b>Note:</b> This API is deprecated. Please use <a>ListTargetsForPolicy</a> instead.

        :param self:
        :param ascendingOrder:  Specifies the order for results. If true, the results are returned in ascending creation order.<br/>pageSize:  The result page size.<br/>marker:  The marker for the next set of results.<br/>policyName:  The policy name.<br/>
        :return: nextMarker:  The marker for the next set of results, or null if there are no additional results.<br/>principals:  The descriptions of the principals.<br/>        """
        pass

    def list_policy_versions(self, policyName: str) -> ListPolicyVersionsResponse:
        """Lists the versions of the specified policy and identifies the default version.

        :param self:
        :param policyName:  The policy name.<br/>
        :return: policyVersions:  The policy versions.<br/>        """
        pass

    def list_principal_policies(self, principal: str, marker: str=None, pageSize: int=None, ascendingOrder: AscendingOrder=None) -> ListPrincipalPoliciesResponse:
        """Lists the policies attached to the specified principal. If you use an Cognito identity, the ID must be in <a href="https://docs.aws.amazon.com/cognitoidentity/latest/APIReference/API_GetCredentialsForIdentity.html#API_GetCredentialsForIdentity_RequestSyntax">AmazonCognito Identity format</a>.  <b>Note:</b> This API is deprecated. Please use <a>ListAttachedPolicies</a> instead.

        :param self:
        :param ascendingOrder:  Specifies the order for results. If true, results are returned in ascending creation order.<br/>pageSize:  The result page size.<br/>marker:  The marker for the next set of results.<br/>principal:  The principal.<br/>
        :return: nextMarker:  The marker for the next set of results, or null if there are no additional results.<br/>policies:  The policies.<br/>        """
        pass

    def list_principal_things(self, principal: str, nextToken: str=None, maxResults: int=None) -> ListPrincipalThingsResponse:
        """Lists the things associated with the specified principal. A principal can be X.509 certificates, IAM users, groups, and roles, Amazon Cognito identities or federated identities. 

        :param self:
        :param principal:  The principal.<br/>maxResults:  The maximum number of results to return in this operation.<br/>nextToken:  The token to retrieve the next set of results.<br/>
        :return: nextToken:  The token used to get the next set of results, or <b>null</b> if there are no additional results.<br/>things:  The things.<br/>        """
        pass

    def list_role_aliases(self, pageSize: int=None, marker: str=None, ascendingOrder: AscendingOrder=None) -> ListRoleAliasesResponse:
        """Lists the role aliases registered in your account.

        :param self:
        :param ascendingOrder:  Return the list of role aliases in ascending alphabetical order.<br/>marker:  A marker used to get the next set of results.<br/>pageSize:  The maximum number of results to return at one time.<br/>
        :return: nextMarker:  A marker used to get the next set of results.<br/>roleAliases:  The role aliases.<br/>        """
        pass

    def list_scheduled_audits(self, nextToken: str=None, maxResults: int=None) -> ListScheduledAuditsResponse:
        """Lists all of your scheduled audits.

        :param self:
        :param maxResults:  The maximum number of results to return at one time. The default is 25.<br/>nextToken:  The token for the next set of results.<br/>
        :return: nextToken:  A token that can be used to retrieve the next set of results, or <code>null</code> if there are no additional results.<br/>scheduledAudits:  The list of scheduled audits.<br/>        """
        pass

    def list_security_profiles(self, nextToken: str=None, maxResults: int=None) -> ListSecurityProfilesResponse:
        """Lists the Device Defender security profiles you have created. You can use filters to list only those security profiles associated with a thing group or only those associated with your account.

        :param self:
        :param maxResults:  The maximum number of results to return at one time.<br/>nextToken:  The token for the next set of results.<br/>
        :return: nextToken:  A token that can be used to retrieve the next set of results, or <code>null</code> if there are no additional results.<br/>securityProfileIdentifiers:  A list of security profile identifiers (names and ARNs).<br/>        """
        pass

    def list_security_profiles_for_target(self, securityProfileTargetArn: str, nextToken: str=None, maxResults: int=None, recursive: Recursive=None) -> ListSecurityProfilesForTargetResponse:
        """Lists the Device Defender security profiles attached to a target (thing group).

        :param self:
        :param securityProfileTargetArn:  The ARN of the target (thing group) whose attached security profiles you want to get.<br/>recursive:  If true, return child groups as well.<br/>maxResults:  The maximum number of results to return at one time.<br/>nextToken:  The token for the next set of results.<br/>
        :return: nextToken:  A token that can be used to retrieve the next set of results, or <code>null</code> if there are no additional results.<br/>securityProfileTargetMappings:  A list of security profiles and their associated targets.<br/>        """
        pass

    def list_streams(self, maxResults: int=None, nextToken: str=None, ascendingOrder: AscendingOrder=None) -> ListStreamsResponse:
        """Lists all of the streams in your AWS account.

        :param self:
        :param ascendingOrder:  Set to true to return the list of streams in ascending order.<br/>nextToken:  A token used to get the next set of results.<br/>maxResults:  The maximum number of results to return at a time.<br/>
        :return: nextToken:  A token used to get the next set of results.<br/>streams:  A list of streams.<br/>        """
        pass

    def list_tags_for_resource(self, resourceArn: str, nextToken: str=None) -> ListTagsForResourceResponse:
        """Lists the tags (metadata) you have assigned to the resource.

        :param self:
        :param nextToken:  The token to retrieve the next set of results.<br/>resourceArn:  The ARN of the resource.<br/>
        :return: nextToken:  The token used to get the next set of results, or <b>null</b> if there are no additional results.<br/>tags:  The list of tags assigned to the resource.<br/>        """
        pass

    def list_targets_for_policy(self, policyName: str, marker: str=None, pageSize: int=None) -> ListTargetsForPolicyResponse:
        """List targets for the specified policy.

        :param self:
        :param pageSize:  The maximum number of results to return at one time.<br/>marker:  A marker used to get the next set of results.<br/>policyName:  The policy name.<br/>
        :return: nextMarker:  A marker used to get the next set of results.<br/>targets:  The policy targets.<br/>        """
        pass

    def list_targets_for_security_profile(self, securityProfileName: str, nextToken: str=None, maxResults: int=None) -> ListTargetsForSecurityProfileResponse:
        """Lists the targets (thing groups) associated with a given Device Defender security profile.

        :param self:
        :param maxResults:  The maximum number of results to return at one time.<br/>nextToken:  The token for the next set of results.<br/>securityProfileName:  The security profile.<br/>
        :return: nextToken:  A token that can be used to retrieve the next set of results, or <code>null</code> if there are no additional results.<br/>securityProfileTargets:  The thing groups to which the security profile is attached.<br/>        """
        pass

    def list_thing_groups(self, nextToken: str=None, maxResults: int=None, parentGroup: str=None, namePrefixFilter: str=None, recursive: RecursiveWithoutDefault=None) -> ListThingGroupsResponse:
        """List the thing groups in your account.

        :param self:
        :param recursive:  If true, return child groups as well.<br/>namePrefixFilter:  A filter that limits the results to those with the specified name prefix.<br/>parentGroup:  A filter that limits the results to those with the specified parent group.<br/>maxResults:  The maximum number of results to return at one time.<br/>nextToken:  The token to retrieve the next set of results.<br/>
        :return: nextToken:  The token used to get the next set of results, or <b>null</b> if there are no additional results.<br/>thingGroups:  The thing groups.<br/>        """
        pass

    def list_thing_groups_for_thing(self, thingName: str, nextToken: str=None, maxResults: int=None) -> ListThingGroupsForThingResponse:
        """List the thing groups to which the specified thing belongs.

        :param self:
        :param maxResults:  The maximum number of results to return at one time.<br/>nextToken:  The token to retrieve the next set of results.<br/>thingName:  The thing name.<br/>
        :return: nextToken:  The token used to get the next set of results, or <b>null</b> if there are no additional results.<br/>thingGroups:  The thing groups.<br/>        """
        pass

    def list_thing_principals(self, thingName: str) -> ListThingPrincipalsResponse:
        """Lists the principals associated with the specified thing. A principal can be X.509 certificates, IAM users, groups, and roles, Amazon Cognito identities or federated identities.

        :param self:
        :param thingName:  The name of the thing.<br/>
        :return: principals:  The principals associated with the thing.<br/>        """
        pass

    def list_thing_registration_task_reports(self, reportType: str, taskId: str, nextToken: str=None, maxResults: int=None) -> ListThingRegistrationTaskReportsResponse:
        """Information about the thing registration tasks.

        :param self:
        :param maxResults:  The maximum number of results to return per request.<br/>nextToken:  The token to retrieve the next set of results.<br/>reportType:  The type of task report.<br/>taskId:  The id of the task.<br/>
        :return: nextToken:  The token used to get the next set of results, or <b>null</b> if there are no additional results.<br/>reportType:  The type of task report.<br/>resourceLinks:  Links to the task resources.<br/>        """
        pass

    def list_thing_registration_tasks(self, nextToken: str=None, maxResults: int=None, status: str=None) -> ListThingRegistrationTasksResponse:
        """List bulk thing provisioning tasks.

        :param self:
        :param status:  The status of the bulk thing provisioning task.<br/>maxResults:  The maximum number of results to return at one time.<br/>nextToken:  The token to retrieve the next set of results.<br/>
        :return: nextToken:  The token used to get the next set of results, or <b>null</b> if there are no additional results.<br/>taskIds:  A list of bulk thing provisioning task IDs.<br/>        """
        pass

    def list_thing_types(self, nextToken: str=None, maxResults: int=None, thingTypeName: str=None) -> ListThingTypesResponse:
        """Lists the existing thing types.

        :param self:
        :param thingTypeName:  The name of the thing type.<br/>maxResults:  The maximum number of results to return in this operation.<br/>nextToken:  The token to retrieve the next set of results.<br/>
        :return: nextToken:  The token for the next set of results, or <b>null</b> if there are no additional results.<br/>thingTypes:  The thing types.<br/>        """
        pass

    def list_things(self, nextToken: str=None, maxResults: int=None, attributeName: str=None, attributeValue: str=None, thingTypeName: str=None) -> ListThingsResponse:
        """Lists your things. Use the <b>attributeName</b> and <b>attributeValue</b> parameters to filter your things. For example, calling <code>ListThings</code> with attributeName=Color and attributeValue=Red retrieves all things in the registry that contain an attribute <b>Color</b> with the value <b>Red</b>. 

        :param self:
        :param thingTypeName:  The name of the thing type used to search for things.<br/>attributeValue:  The attribute value used to search for things.<br/>attributeName:  The attribute name used to search for things.<br/>maxResults:  The maximum number of results to return in this operation.<br/>nextToken:  The token to retrieve the next set of results.<br/>
        :return: nextToken:  The token used to get the next set of results, or <b>null</b> if there are no additional results.<br/>things:  The things.<br/>        """
        pass

    def list_things_in_billing_group(self, billingGroupName: str, nextToken: str=None, maxResults: int=None) -> ListThingsInBillingGroupResponse:
        """Lists the things you have added to the given billing group.

        :param self:
        :param maxResults:  The maximum number of results to return per request.<br/>nextToken:  The token to retrieve the next set of results.<br/>billingGroupName:  The name of the billing group.<br/>
        :return: nextToken:  The token used to get the next set of results, or <b>null</b> if there are no additional results.<br/>things:  A list of things in the billing group.<br/>        """
        pass

    def list_things_in_thing_group(self, thingGroupName: str, recursive: Recursive=None, nextToken: str=None, maxResults: int=None) -> ListThingsInThingGroupResponse:
        """Lists the things in the specified group.

        :param self:
        :param maxResults:  The maximum number of results to return at one time.<br/>nextToken:  The token to retrieve the next set of results.<br/>recursive:  When true, list things in this thing group and in all child groups as well.<br/>thingGroupName:  The thing group name.<br/>
        :return: nextToken:  The token used to get the next set of results, or <b>null</b> if there are no additional results.<br/>things:  The things in the specified thing group.<br/>        """
        pass

    def list_topic_rules(self, topic: str=None, maxResults: int=None, nextToken: str=None, ruleDisabled: IsDisabled=None) -> ListTopicRulesResponse:
        """Lists the rules for the specific topic.

        :param self:
        :param ruleDisabled:  Specifies whether the rule is disabled.<br/>nextToken:  A token used to retrieve the next value.<br/>maxResults:  The maximum number of results to return.<br/>topic:  The topic.<br/>
        :return: nextToken:  A token used to retrieve the next value.<br/>rules:  The rules.<br/>        """
        pass

    def list_v2_logging_levels(self, targetType: str=None, nextToken: str=None, maxResults: int=None) -> ListV2LoggingLevelsResponse:
        """Lists logging levels.

        :param self:
        :param maxResults:  The maximum number of results to return at one time.<br/>nextToken:  The token used to get the next set of results, or <b>null</b> if there are no additional results.<br/>targetType:  The type of resource for which you are configuring logging. Must be <code>THING_Group</code>.<br/>
        :return: nextToken:  The token used to get the next set of results, or <b>null</b> if there are no additional results.<br/>logTargetConfigurations:  The logging configuration for a target.<br/>        """
        pass

    def list_violation_events(self, endTime: Timestamp, startTime: Timestamp, thingName: str=None, securityProfileName: str=None, nextToken: str=None, maxResults: int=None) -> ListViolationEventsResponse:
        """Lists the Device Defender security profile violations discovered during the given time period. You can use filters to limit the results to those alerts issued for a particular security profile, behavior or thing (device).

        :param self:
        :param maxResults:  The maximum number of results to return at one time.<br/>nextToken:  The token for the next set of results.<br/>securityProfileName:  A filter to limit results to those alerts generated by the specified security profile.<br/>thingName:  A filter to limit results to those alerts caused by the specified thing.<br/>endTime:  The end time for the alerts to be listed.<br/>startTime:  The start time for the alerts to be listed.<br/>
        :return: nextToken:  A token that can be used to retrieve the next set of results, or <code>null</code> if there are no additional results.<br/>violationEvents:  The security profile violation alerts issued for this account during the given time frame, potentially filtered by security profile, behavior violated, or thing (device) violating.<br/>        """
        pass

    def register_ca_certificate(self, verificationCertificate: str, caCertificate: str, setAsActive: SetAsActive=None, allowAutoRegistration: AllowAutoRegistration=None, registrationConfig: RegistrationConfig=None) -> RegisterCACertificateResponse:
        """Registers a CA certificate with AWS IoT. This CA certificate can then be used to sign device certificates, which can be then registered with AWS IoT. You can register up to 10 CA certificates per AWS account that have the same subject field. This enables you to have up to 10 certificate authorities sign your device certificates. If you have more than one CA certificate registered, make sure you pass the CA certificate when you register your device certificates with the RegisterCertificate API.

        :param self:
        :param registrationConfig:  Information about the registration configuration.:<br/>allowAutoRegistration:  Allows this CA certificate to be used for auto registration of device certificates.<br/>setAsActive:  A boolean value that specifies if the CA certificate is set to active.<br/>verificationCertificate:  The private key verification certificate.<br/>caCertificate:  The CA certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The ARN of the role.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>templateBody</i> The template body.<br/>
        :return: certificateId:  The CA certificate identifier.<br/>certificateArn:  The CA certificate ARN.<br/>        """
        pass

    def register_certificate(self, certificatePem: str, caCertificatePem: str=None, setAsActive: SetAsActiveFlag=None, status: str=None) -> RegisterCertificateResponse:
        """Registers a device certificate with AWS IoT. If you have more than one CA certificate that has the same subject field, you must specify the CA certificate that was used to sign the device certificate being registered.

        :param self:
        :param status:  The status of the register certificate request.<br/>setAsActive:  A boolean value that specifies if the certificate is set to active.<br/>caCertificatePem:  The CA certificate used to sign the device certificate being registered.<br/>certificatePem:  The certificate data, in PEM format.<br/>
        :return: certificateId:  The certificate identifier.<br/>certificateArn:  The certificate ARN.<br/>        """
        pass

    def register_thing(self, templateBody: str, parameters: Parameters=None) -> RegisterThingResponse:
        """Provisions a thing.

        :param self:
        :param parameters:  The parameters for provisioning a thing. See <a href="https://docs.aws.amazon.com/iot/latest/developerguide/programmatic-provisioning.html">Programmatic Provisioning</a> for more information.<br/>templateBody:  The provisioning template. See <a href="https://docs.aws.amazon.com/iot/latest/developerguide/programmatic-provisioning.html">Programmatic Provisioning</a> for more information.<br/>
        :return: resourceArns:  ARNs for the generated resources.<br/>certificatePem:  .<br/>        """
        pass

    def reject_certificate_transfer(self, certificateId: str, rejectReason: str=None):
        """Rejects a pending certificate transfer. After AWS IoT rejects a certificate transfer, the certificate status changes from <b>PENDING_TRANSFER</b> to <b>INACTIVE</b>. To check for pending certificate transfers, call <a>ListCertificates</a> to enumerate your certificates. This operation can only be called by the transfer destination. After it is called, the certificate will be returned to the source's account in the INACTIVE state.

        :param self:
        :param rejectReason:  The reason the certificate transfer was rejected.<br/>certificateId:  The ID of the certificate. (The last part of the certificate ARN contains the certificate ID.)<br/>
        :return:         """
        pass

    def remove_thing_from_billing_group(self, billingGroupName: str=None, billingGroupArn: str=None, thingName: str=None, thingArn: str=None) -> RemoveThingFromBillingGroupResponse:
        """Removes the given thing from the billing group.

        :param self:
        :param thingArn:  The ARN of the thing to be removed from the billing group.<br/>thingName:  The name of the thing to be removed from the billing group.<br/>billingGroupArn:  The ARN of the billing group.<br/>billingGroupName:  The name of the billing group.<br/>
        :return:         """
        pass

    def remove_thing_from_thing_group(self, thingGroupName: str=None, thingGroupArn: str=None, thingName: str=None, thingArn: str=None) -> RemoveThingFromThingGroupResponse:
        """Remove the specified thing from the specified group.

        :param self:
        :param thingArn:  The ARN of the thing to remove from the group.<br/>thingName:  The name of the thing to remove from the group.<br/>thingGroupArn:  The group ARN.<br/>thingGroupName:  The group name.<br/>
        :return:         """
        pass

    def replace_topic_rule(self, topicRulePayload: TopicRulePayload, ruleName: str):
        """Replaces the rule. You must specify all parameters for the new rule. Creating rules is an administrator-level action. Any user who has permission to create rules will be able to access data processed by the rule.

        :param self:
        :param topicRulePayload:  The rule payload.:<br/>ruleName:  The name of the rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorAction</i> The action to take when an error occurs.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>errorAction</b>: The action to take when an error occurs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>awsIotSqlVersion</i> The version of the SQL rules engine to use when evaluating the rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ruleDisabled</i> Specifies whether the rule is disabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>actions</i> The actions associated with the rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The description of the rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>sql</i> The SQL statement used to query the topic. For more information, see <a href="https://docs.aws.amazon.com/iot/latest/developerguide/iot-rules.html#aws-iot-sql-reference">AWS IoT SQL Reference</a> in the <i>AWS IoT Developer Guide</i>.<br/>Write to a DynamoDB table.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>dynamoDB</b>: Write to a DynamoDB table.
        :return:         """
        pass

    def search_index(self, queryString: str, indexName: str=None, nextToken: str=None, maxResults: int=None, queryVersion: str=None) -> SearchIndexResponse:
        """The query search index.

        :param self:
        :param queryVersion:  The query version.<br/>maxResults:  The maximum number of results to return at one time.<br/>nextToken:  The token used to get the next set of results, or null if there are no additional results.<br/>queryString:  The search query string.<br/>indexName:  The search index name.<br/>
        :return: thingGroups:  The thing groups that match the search query.<br/>things:  The things that match the search query.<br/>nextToken:  The token used to get the next set of results, or null if there are no additional results.<br/>        """
        pass

    def set_default_authorizer(self, authorizerName: str) -> SetDefaultAuthorizerResponse:
        """Sets the default authorizer. This will be used if a websocket connection is made without specifying an authorizer.

        :param self:
        :param authorizerName:  The authorizer name.<br/>
        :return: authorizerArn:  The authorizer ARN.<br/>authorizerName:  The authorizer name.<br/>        """
        pass

    def set_default_policy_version(self, policyVersionId: str, policyName: str):
        """Sets the specified version of the specified policy as the policy's default (operative) version. This action affects all certificates to which the policy is attached. To list the principals the policy is attached to, use the ListPrincipalPolicy API.

        :param self:
        :param policyVersionId:  The policy version ID.<br/>policyName:  The policy name.<br/>
        :return:         """
        pass

    def set_logging_options(self, loggingOptionsPayload: LoggingOptionsPayload):
        """Sets the logging options. NOTE: use of this command is not recommended. Use <code>SetV2LoggingOptions</code> instead.

        :param self:
        :param loggingOptionsPayload:  The logging options payload.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>logLevel</i> The log level.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The ARN of the IAM role that grants access.<br/>
        :return:         """
        pass

    def set_v2_logging_level(self, logLevel: str, logTarget: LogTarget):
        """Sets the logging level.

        :param self:
        :param logLevel:  The log level.<br/>logTarget:  The log target.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>targetName</i> The target name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>targetType</i> The target type.<br/>
        :return:         """
        pass

    def set_v2_logging_options(self, roleArn: str=None, defaultLogLevel: str=None, disableAllLogs: DisableAllLogs=None):
        """Sets the logging options for the V2 logging service.

        :param self:
        :param disableAllLogs:  If true all logs are disabled. The default is false.<br/>defaultLogLevel:  The default logging level.<br/>roleArn:  The ARN of the role that allows IoT to write to Cloudwatch logs.<br/>
        :return:         """
        pass

    def start_on_demand_audit_task(self, targetCheckNames: List[AuditCheckName]) -> StartOnDemandAuditTaskResponse:
        """Starts an on-demand Device Defender audit.

        :param self:
        :param targetCheckNames:  Which checks are performed during the audit. The checks you specify must be enabled for your account or an exception occurs. Use <code>DescribeAccountAuditConfiguration</code> to see the list of all checks including those that are enabled or <code>UpdateAccountAuditConfiguration</code> to select which checks are enabled.<br/>
        :return: taskId:  The ID of the on-demand audit you started.<br/>        """
        pass

    def start_thing_registration_task(self, roleArn: str, inputFileKey: str, inputFileBucket: str, templateBody: str) -> StartThingRegistrationTaskResponse:
        """Creates a bulk thing provisioning task.

        :param self:
        :param roleArn:  The IAM role ARN that grants permission the input file.<br/>inputFileKey:  The name of input file within the S3 bucket. This file contains a newline delimited JSON file. Each line contains the parameter values to provision one device (thing).<br/>inputFileBucket:  The S3 bucket that contains the input file.<br/>templateBody:  The provisioning template.<br/>
        :return: taskId:  The bulk thing provisioning task ID.<br/>        """
        pass

    def stop_thing_registration_task(self, taskId: str) -> StopThingRegistrationTaskResponse:
        """Cancels a bulk thing provisioning task.

        :param self:
        :param taskId:  The bulk thing provisioning task ID.<br/>
        :return:         """
        pass

    def tag_resource(self, tags: List[Tag], resourceArn: str) -> TagResourceResponse:
        """Adds to or modifies the tags of the given resource. Tags are metadata which can be used to manage a resource.

        :param self:
        :param tags:  The new or modified tags for the resource.<br/>resourceArn:  The ARN of the resource.<br/>
        :return:         """
        pass

    def test_authorization(self, authInfos: List[AuthInfo], principal: str=None, cognitoIdentityPoolId: str=None, clientId: str=None, policyNamesToAdd: List[PolicyName]=None, policyNamesToSkip: List[PolicyName]=None) -> TestAuthorizationResponse:
        """Tests if a specified principal is authorized to perform an AWS IoT action on a specified resource. Use this to test and debug the authorization behavior of devices that connect to the AWS IoT device gateway.

        :param self:
        :param policyNamesToSkip:  When testing custom authorization, the policies specified here are treated as if they are not attached to the principal being authorized.<br/>policyNamesToAdd:  When testing custom authorization, the policies specified here are treated as if they are attached to the principal being authorized.<br/>clientId:  The MQTT client ID.<br/>authInfos:  A list of authorization info objects. Simulating authorization will create a response for each <code>authInfo</code> object in the list.<br/>cognitoIdentityPoolId:  The Cognito identity pool ID.<br/>principal:  The principal.<br/>
        :return: authResults:  The authentication results.<br/>        """
        pass

    def test_invoke_authorizer(self, tokenSignature: str, token: str, authorizerName: str) -> TestInvokeAuthorizerResponse:
        """Tests a custom authorization behavior by invoking a specified custom authorizer. Use this to test and debug the custom authorization behavior of devices that connect to the AWS IoT device gateway.

        :param self:
        :param tokenSignature:  The signature made with the token and your custom authentication service's private key.<br/>token:  The token returned by your custom authentication service.<br/>authorizerName:  The custom authorizer name.<br/>
        :return: disconnectAfterInSeconds:  The number of seconds after which the connection is terminated.<br/>refreshAfterInSeconds:  The number of seconds after which the temporary credentials are refreshed.<br/>policyDocuments:  IAM policy documents.<br/>principalId:  The principal ID.<br/>isAuthenticated:  True if the token is authenticated, otherwise false.<br/>        """
        pass

    def transfer_certificate(self, targetAwsAccount: str, certificateId: str, transferMessage: str=None) -> TransferCertificateResponse:
        """Transfers the specified certificate to the specified AWS account. You can cancel the transfer until it is acknowledged by the recipient. No notification is sent to the transfer destination's account. It is up to the caller to notify the transfer target. The certificate being transferred must not be in the ACTIVE state. You can use the UpdateCertificate API to deactivate it. The certificate must not have any policies attached to it. You can use the DetachPrincipalPolicy API to detach them.

        :param self:
        :param transferMessage:  The transfer message.<br/>targetAwsAccount:  The AWS account.<br/>certificateId:  The ID of the certificate. (The last part of the certificate ARN contains the certificate ID.)<br/>
        :return: transferredCertificateArn:  The ARN of the certificate.<br/>        """
        pass

    def untag_resource(self, tagKeys: List[TagKey], resourceArn: str) -> UntagResourceResponse:
        """Removes the given tags (metadata) from the resource.

        :param self:
        :param tagKeys:  A list of the keys of the tags to be removed from the resource.<br/>resourceArn:  The ARN of the resource.<br/>
        :return:         """
        pass

    def update_account_audit_configuration(self, roleArn: str=None, auditNotificationTargetConfigurations: AuditNotificationTargetConfigurations=None, auditCheckConfigurations: AuditCheckConfigurations=None) -> UpdateAccountAuditConfigurationResponse:
        """Configures or reconfigures the Device Defender audit settings for this account. Settings include how audit notifications are sent and which audit checks are enabled or disabled.

        :param self:
        :param auditCheckConfigurations:  Specifies which audit checks are enabled and disabled for this account. Use <code>DescribeAccountAuditConfiguration</code> to see the list of all checks including those that are currently enabled. Note that some data collection may begin immediately when certain checks are enabled. When a check is disabled, any data collected so far in relation to the check is deleted. You cannot disable a check if it is used by any scheduled audit. You must first delete the check from the scheduled audit or delete the scheduled audit itself. On the first call to <code>UpdateAccountAuditConfiguration</code> this parameter is required and must specify at least one enabled check.<br/>auditNotificationTargetConfigurations:  Information about the targets to which audit notifications are sent.<br/>roleArn:  The ARN of the role that grants permission to AWS IoT to access information about your devices, policies, certificates and other items as necessary when performing an audit.<br/>
        :return:         """
        pass

    def update_authorizer(self, authorizerName: str, authorizerFunctionArn: str=None, tokenKeyName: str=None, tokenSigningPublicKeys: PublicKeyMap=None, status: str=None) -> UpdateAuthorizerResponse:
        """Updates an authorizer.

        :param self:
        :param status:  The status of the update authorizer request.<br/>tokenSigningPublicKeys:  The public keys used to verify the token signature.<br/>tokenKeyName:  The key used to extract the token from the HTTP headers. <br/>authorizerFunctionArn:  The ARN of the authorizer's Lambda function.<br/>authorizerName:  The authorizer name.<br/>
        :return: authorizerArn:  The authorizer ARN.<br/>authorizerName:  The authorizer name.<br/>        """
        pass

    def update_billing_group(self, billingGroupProperties: BillingGroupProperties, billingGroupName: str, expectedVersion: OptionalVersion=None) -> UpdateBillingGroupResponse:
        """Updates information about the billing group.

        :param self:
        :param expectedVersion:  The expected version of the billing group. If the version of the billing group does not match the expected version specified in the request, the <code>UpdateBillingGroup</code> request is rejected with a <code>VersionConflictException</code>.<br/>billingGroupProperties:  The properties of the billing group.:<br/>billingGroupName:  The name of the billing group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>billingGroupDescription</i> The description of the billing group.<br/>
        :return: version:  The latest version of the billing group.<br/>        """
        pass

    def update_ca_certificate(self, certificateId: str, newStatus: str=None, newAutoRegistrationStatus: str=None, registrationConfig: RegistrationConfig=None, removeAutoRegistration: RemoveAutoRegistration=None):
        """Updates a registered CA certificate.

        :param self:
        :param removeAutoRegistration:  If true, remove auto registration.<br/>registrationConfig:  Information about the registration configuration.:<br/>newAutoRegistrationStatus:  The new value for the auto registration status. Valid values are: "ENABLE" or "DISABLE".<br/>newStatus:  The updated status of the CA certificate.  <b>Note:</b> The status value REGISTER_INACTIVE is deprecated and should not be used.<br/>certificateId:  The CA certificate identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The ARN of the role.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>templateBody</i> The template body.<br/>
        :return:         """
        pass

    def update_certificate(self, newStatus: str, certificateId: str):
        """Updates the status of the specified certificate. This operation is idempotent. Moving a certificate from the ACTIVE state (including REVOKED) will not disconnect currently connected devices, but these devices will be unable to reconnect. The ACTIVE state is required to authenticate devices connecting to AWS IoT using a certificate.

        :param self:
        :param newStatus:  The new status.  <b>Note:</b> Setting the status to PENDING_TRANSFER will result in an exception being thrown. PENDING_TRANSFER is a status used internally by AWS IoT. It is not intended for developer use.  <b>Note:</b> The status value REGISTER_INACTIVE is deprecated and should not be used.<br/>certificateId:  The ID of the certificate. (The last part of the certificate ARN contains the certificate ID.)<br/>
        :return:         """
        pass

    def update_dynamic_thing_group(self, thingGroupProperties: ThingGroupProperties, thingGroupName: str, expectedVersion: OptionalVersion=None, indexName: str=None, queryString: str=None, queryVersion: str=None) -> UpdateDynamicThingGroupResponse:
        """Updates a dynamic thing group.

        :param self:
        :param queryVersion:  The dynamic thing group query version to update. <note> Currently one query version is supported: "2017-09-30". If not specified, the query version defaults to this value. </note><br/>queryString:  The dynamic thing group search query string to update.<br/>indexName:  The dynamic thing group index to update. <note> Currently one index is supported: 'AWS_Things'. </note><br/>expectedVersion:  The expected version of the dynamic thing group to update.<br/>thingGroupProperties:  The dynamic thing group properties to update.:<br/>thingGroupName:  The name of the dynamic thing group to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>attributePayload</i> The thing group attributes in JSON format.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>attributePayload</b>: The thing group attributes in JSON format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>thingGroupDescription</i> The thing group description.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>merge</i> Specifies whether the list of attributes provided in the <code>AttributePayload</code> is merged with the attributes stored in the registry, instead of overwriting them. To remove an attribute, call <code>UpdateThing</code> with an empty attribute value. <note> The <code>merge</code> attribute is only valid when calling <code>UpdateThing</code>. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>attributes</i> A JSON string containing up to three key-value pair in JSON format. For example:  <code>{\"attributes\":{\"string1\":\"string2\"}}</code> <br/>
        :return: version:  The dynamic thing group version.<br/>        """
        pass

    def update_event_configurations(self, eventConfigurations: EventConfigurations=None) -> UpdateEventConfigurationsResponse:
        """Updates the event configurations.

        :param self:
        :param eventConfigurations:  The new event configuration values.<br/>
        :return:         """
        pass

    def update_indexing_configuration(self, thingIndexingConfiguration: ThingIndexingConfiguration=None, thingGroupIndexingConfiguration: ThingGroupIndexingConfiguration=None) -> UpdateIndexingConfigurationResponse:
        """Updates the search configuration.

        :param self:
        :param thingGroupIndexingConfiguration:  Thing group indexing configuration.:<br/>thingIndexingConfiguration:  Thing indexing configuration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>thingConnectivityIndexingMode</i> Thing connectivity indexing mode. Valid values are:  <ul> <li> STATUS – Your thing index contains connectivity status. To enable thing connectivity indexing, thingIndexMode must not be set to OFF. </li> <li> OFF - Thing connectivity status indexing is disabled. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>thingIndexingMode</i> Thing indexing mode. Valid values are: <ul> <li> REGISTRY – Your thing index contains registry data only. </li> <li> REGISTRY_AND_SHADOW - Your thing index contains registry and shadow data. </li> <li> OFF - Thing indexing is disabled. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>thingGroupIndexingMode</i> Thing group indexing mode.<br/>
        :return:         """
        pass

    def update_job(self, jobId: str, description: str=None, presignedUrlConfig: PresignedUrlConfig=None, jobExecutionsRolloutConfig: JobExecutionsRolloutConfig=None, abortConfig: AbortConfig=None, timeoutConfig: TimeoutConfig=None):
        """Updates supported fields of the specified job.

        :param self:
        :param timeoutConfig:  Specifies the amount of time each device has to finish its execution of the job. The timer is started when the job execution status is set to <code>IN_PROGRESS</code>. If the job execution status is not set to another terminal state before the time expires, it will be automatically set to <code>TIMED_OUT</code>. :<br/>abortConfig:  Allows you to create criteria to abort a job.:<br/>jobExecutionsRolloutConfig:  Allows you to create a staged rollout of the job.:<br/>presignedUrlConfig:  Configuration information for pre-signed S3 URLs.:<br/>description:  A short text description of the job.<br/>jobId:  The ID of the job to be updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>expiresInSec</i> How long (in seconds) pre-signed URLs are valid. Valid values are 60 - 3600, the default value is 3600 seconds. Pre-signed URLs are generated when Jobs receives an MQTT request for the job document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The ARN of an IAM role that grants grants permission to download files from the S3 bucket where the job data/updates are stored. The role must also grant permission for IoT to download the files.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>exponentialRate</i> The rate of increase for a job rollout. This parameter allows you to define an exponential rate for a job rollout.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>exponentialRate</b>: The rate of increase for a job rollout. This parameter allows you to define an exponential rate for a job rollout.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>maximumPerMinute</i> The maximum number of things that will be notified of a pending job, per minute. This parameter allows you to create a staged rollout.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>incrementFactor</i> The exponential factor to increase the rate of rollout for a job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>baseRatePerMinute</i> The minimum number of things that will be notified of a pending job, per minute at the start of job rollout. This parameter allows you to define the initial rate of rollout.<br/>The criteria to initiate the increase in rate of rollout for a job. AWS IoT supports up to one digit after the decimal (for example, 1.5, but not 1.55).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>rateIncreaseCriteria</b>: The criteria to initiate the increase in rate of rollout for a job. AWS IoT supports up to one digit after the decimal (for example, 1.5, but not 1.55).&nbsp;&nbsp;&nbsp;&nbsp; <i>criteriaList</i> The list of abort criteria to define rules to abort the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>inProgressTimeoutInMinutes</i> Specifies the amount of time, in minutes, this device has to finish execution of this job. The timeout interval can be anywhere between 1 minute and 7 days (1 to 10080 minutes). The in progress timer can't be updated and will apply to all job executions for the job. Whenever a job execution remains in the IN_PROGRESS status for longer than this interval, the job execution will fail and switch to the terminal <code>TIMED_OUT</code> status.<br/>
        :return:         """
        pass

    def update_role_alias(self, roleAlias: str, roleArn: str=None, credentialDurationSeconds: int=None) -> UpdateRoleAliasResponse:
        """Updates a role alias.

        :param self:
        :param credentialDurationSeconds:  The number of seconds the credential will be valid.<br/>roleArn:  The role ARN.<br/>roleAlias:  The role alias to update.<br/>
        :return: roleAliasArn:  The role alias ARN.<br/>roleAlias:  The role alias.<br/>        """
        pass

    def update_scheduled_audit(self, scheduledAuditName: str, frequency: str=None, dayOfMonth: str=None, dayOfWeek: str=None, targetCheckNames: List[AuditCheckName]=None) -> UpdateScheduledAuditResponse:
        """Updates a scheduled audit, including what checks are performed and how often the audit takes place.

        :param self:
        :param scheduledAuditName:  The name of the scheduled audit. (Max. 128 chars)<br/>targetCheckNames:  Which checks are performed during the scheduled audit. Checks must be enabled for your account. (Use <code>DescribeAccountAuditConfiguration</code> to see the list of all checks including those that are enabled or <code>UpdateAccountAuditConfiguration</code> to select which checks are enabled.)<br/>dayOfWeek:  The day of the week on which the scheduled audit takes place. Can be one of "SUN", "MON", "TUE", "WED", "THU", "FRI" or "SAT". This field is required if the "frequency" parameter is set to "WEEKLY" or "BIWEEKLY".<br/>dayOfMonth:  The day of the month on which the scheduled audit takes place. Can be "1" through "31" or "LAST". This field is required if the "frequency" parameter is set to "MONTHLY". If days 29-31 are specified, and the month does not have that many days, the audit takes place on the "LAST" day of the month.<br/>frequency:  How often the scheduled audit takes place. Can be one of "DAILY", "WEEKLY", "BIWEEKLY" or "MONTHLY". The actual start time of each audit is determined by the system.<br/>
        :return: scheduledAuditArn:  The ARN of the scheduled audit.<br/>        """
        pass

    def update_security_profile(self, securityProfileName: str, securityProfileDescription: str=None, behaviors: List[Behavior]=None, alertTargets: AlertTargets=None, additionalMetricsToRetain: List[BehaviorMetric]=None, deleteBehaviors: DeleteBehaviors=None, deleteAlertTargets: DeleteAlertTargets=None, deleteAdditionalMetricsToRetain: DeleteAdditionalMetricsToRetain=None, expectedVersion: OptionalVersion=None) -> UpdateSecurityProfileResponse:
        """Updates a Device Defender security profile.

        :param self:
        :param expectedVersion:  The expected version of the security profile. A new version is generated whenever the security profile is updated. If you specify a value that is different than the actual version, a <code>VersionConflictException</code> is thrown.<br/>deleteAdditionalMetricsToRetain:  If true, delete all <code>additionalMetricsToRetain</code> defined for this security profile. If any <code>additionalMetricsToRetain</code> are defined in the current invocation an exception occurs.<br/>deleteAlertTargets:  If true, delete all <code>alertTargets</code> defined for this security profile. If any <code>alertTargets</code> are defined in the current invocation an exception occurs.<br/>deleteBehaviors:  If true, delete all <code>behaviors</code> defined for this security profile. If any <code>behaviors</code> are defined in the current invocation an exception occurs.<br/>additionalMetricsToRetain:  A list of metrics whose data is retained (stored). By default, data is retained for any metric used in the profile's <code>behaviors</code> but it is also retained for any metric specified here.<br/>alertTargets:  Where the alerts are sent. (Alerts are always sent to the console.)<br/>behaviors:  Specifies the behaviors that, when violated by a device (thing), cause an alert.<br/>securityProfileDescription:  A description of the security profile.<br/>securityProfileName:  The name of the security profile you want to update.<br/>
        :return: lastModifiedDate:  The time the security profile was last modified.<br/>creationDate:  The time the security profile was created.<br/>version:  The updated version of the security profile.<br/>additionalMetricsToRetain:  A list of metrics whose data is retained (stored). By default, data is retained for any metric used in the security profile's <code>behaviors</code> but it is also retained for any metric specified here.<br/>alertTargets:  Where the alerts are sent. (Alerts are always sent to the console.)<br/>behaviors:  Specifies the behaviors that, when violated by a device (thing), cause an alert.<br/>securityProfileDescription:  The description of the security profile.<br/>securityProfileArn:  The ARN of the security profile that was updated.<br/>securityProfileName:  The name of the security profile that was updated.<br/>        """
        pass

    def update_stream(self, streamId: str, description: str=None, files: List[StreamFile]=None, roleArn: str=None) -> UpdateStreamResponse:
        """Updates an existing stream. The stream version will be incremented by one.

        :param self:
        :param roleArn:  An IAM role that allows the IoT service principal assumes to access your S3 files.<br/>files:  The files associated with the stream.<br/>description:  The description of the stream.<br/>streamId:  The stream ID.<br/>
        :return: streamVersion:  The stream version.<br/>description:  A description of the stream.<br/>streamArn:  The stream ARN.<br/>streamId:  The stream ID.<br/>        """
        pass

    def update_thing(self, thingName: str, thingTypeName: str=None, attributePayload: AttributePayload=None, expectedVersion: OptionalVersion=None, removeThingType: RemoveThingType=None) -> UpdateThingResponse:
        """Updates the data for a thing.

        :param self:
        :param removeThingType:  Remove a thing type association. If <b>true</b>, the association is removed.<br/>expectedVersion:  The expected version of the thing record in the registry. If the version of the record in the registry does not match the expected version specified in the request, the <code>UpdateThing</code> request is rejected with a <code>VersionConflictException</code>.<br/>attributePayload:  A list of thing attributes, a JSON string containing name-value pairs. For example:  <code>{\"attributes\":{\"name1\":\"value2\"}}</code>  This data is used to add new attributes or update existing attributes.:<br/>thingTypeName:  The name of the thing type.<br/>thingName:  The name of the thing to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>merge</i> Specifies whether the list of attributes provided in the <code>AttributePayload</code> is merged with the attributes stored in the registry, instead of overwriting them. To remove an attribute, call <code>UpdateThing</code> with an empty attribute value. <note> The <code>merge</code> attribute is only valid when calling <code>UpdateThing</code>. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>attributes</i> A JSON string containing up to three key-value pair in JSON format. For example:  <code>{\"attributes\":{\"string1\":\"string2\"}}</code> <br/>
        :return:         """
        pass

    def update_thing_group(self, thingGroupProperties: ThingGroupProperties, thingGroupName: str, expectedVersion: OptionalVersion=None) -> UpdateThingGroupResponse:
        """Update a thing group.

        :param self:
        :param expectedVersion:  The expected version of the thing group. If this does not match the version of the thing group being updated, the update will fail.<br/>thingGroupProperties:  The thing group properties.:<br/>thingGroupName:  The thing group to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>attributePayload</i> The thing group attributes in JSON format.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>attributePayload</b>: The thing group attributes in JSON format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>thingGroupDescription</i> The thing group description.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>merge</i> Specifies whether the list of attributes provided in the <code>AttributePayload</code> is merged with the attributes stored in the registry, instead of overwriting them. To remove an attribute, call <code>UpdateThing</code> with an empty attribute value. <note> The <code>merge</code> attribute is only valid when calling <code>UpdateThing</code>. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>attributes</i> A JSON string containing up to three key-value pair in JSON format. For example:  <code>{\"attributes\":{\"string1\":\"string2\"}}</code> <br/>
        :return: version:  The version of the updated thing group.<br/>        """
        pass

    def update_thing_groups_for_thing(self, thingName: str=None, thingGroupsToAdd: List[ThingGroupName]=None, thingGroupsToRemove: List[ThingGroupName]=None, overrideDynamicGroups: OverrideDynamicGroups=None) -> UpdateThingGroupsForThingResponse:
        """Updates the groups to which the thing belongs.

        :param self:
        :param overrideDynamicGroups:  Override dynamic thing groups with static thing groups when 10-group limit is reached. If a thing belongs to 10 thing groups, and one or more of those groups are dynamic thing groups, adding a thing to a static group removes the thing from the last dynamic group.<br/>thingGroupsToRemove:  The groups from which the thing will be removed.<br/>thingGroupsToAdd:  The groups to which the thing will be added.<br/>thingName:  The thing whose group memberships will be updated.<br/>
        :return:         """
        pass

    def validate_security_profile_behaviors(self, behaviors: List[Behavior]) -> ValidateSecurityProfileBehaviorsResponse:
        """Validates a Device Defender security profile behaviors specification.

        :param self:
        :param behaviors:  Specifies the behaviors that, when violated by a device (thing), cause an alert.<br/>
        :return: validationErrors:  The list of any errors found in the behaviors.<br/>valid:  True if the behaviors were valid.<br/>        """
        pass



class IoTDataPlane(BaseClient):
    """<fullname>AWS IoT</fullname> AWS IoT-Data enables secure, bi-directional communication between Internet-connected things (such as sensors, actuators, embedded devices, or smart appliances) and the AWS cloud. It implements a broker for applications and things to publish messages over HTTP (Publish) and retrieve, update, and delete thing shadows. A thing shadow is a persistent representation of your things and their state in the AWS cloud."""



    class DeleteThingShadowRequest(Mapping):
        pass
    
    class DeleteThingShadowResponse(Mapping):
        pass
    
    class GetThingShadowRequest(Mapping):
        pass
    
    class GetThingShadowResponse(Mapping):
        pass
    
    class PublishRequest(Mapping):
        pass
    
    class Payload(object):
        pass
    
    class UpdateThingShadowRequest(Mapping):
        pass
    
    class UpdateThingShadowResponse(Mapping):
        pass
    
    class JsonDocument(object):
        pass
    
    def delete_thing_shadow(self, thingName: str) -> DeleteThingShadowResponse:
        """Deletes the thing shadow for the specified thing. For more information, see <a href="http://docs.aws.amazon.com/iot/latest/developerguide/API_DeleteThingShadow.html">DeleteThingShadow</a> in the <i>AWS IoT Developer Guide</i>.

        :param self:
        :param thingName:  The name of the thing.<br/>
        :return: payload:  The state information, in JSON format.<br/>        """
        pass

    def get_thing_shadow(self, thingName: str) -> GetThingShadowResponse:
        """Gets the thing shadow for the specified thing. For more information, see <a href="http://docs.aws.amazon.com/iot/latest/developerguide/API_GetThingShadow.html">GetThingShadow</a> in the <i>AWS IoT Developer Guide</i>.

        :param self:
        :param thingName:  The name of the thing.<br/>
        :return: payload:  The state information, in JSON format.<br/>        """
        pass

    def publish(self, topic: str, qos: int=None, payload: Payload=None):
        """Publishes state information. For more information, see <a href="http://docs.aws.amazon.com/iot/latest/developerguide/protocols.html#http">HTTP Protocol</a> in the <i>AWS IoT Developer Guide</i>.

        :param self:
        :param payload:  The state information, in JSON format.<br/>qos:  The Quality of Service (QoS) level.<br/>topic:  The name of the MQTT topic.<br/>
        :return:         """
        pass

    def update_thing_shadow(self, payload: JsonDocument, thingName: str) -> UpdateThingShadowResponse:
        """Updates the thing shadow for the specified thing. For more information, see <a href="http://docs.aws.amazon.com/iot/latest/developerguide/API_UpdateThingShadow.html">UpdateThingShadow</a> in the <i>AWS IoT Developer Guide</i>.

        :param self:
        :param payload:  The state information, in JSON format.<br/>thingName:  The name of the thing.<br/>
        :return: payload:  The state information, in JSON format.<br/>        """
        pass



class IoTJobsDataPlane(BaseClient):
    """AWS IoT Jobs is a service that allows you to define a set of jobs — remote operations that are sent to and executed on one or more devices connected to AWS IoT. For example, you can define a job that instructs a set of devices to download and install application or firmware updates, reboot, rotate certificates, or perform remote troubleshooting operations.  To create a job, you make a job document which is a description of the remote operations to be performed, and you specify a list of targets that should perform the operations. The targets can be individual things, thing groups or both.  AWS IoT Jobs sends a message to inform the targets that a job is available. The target starts the execution of the job by downloading the job document, performing the operations it specifies, and reporting its progress to AWS IoT. The Jobs service provides commands to track the progress of a job on a specific target and for all the targets of the job"""




    class DescribeJobExecutionRequest(Mapping):
        pass
    
    class DescribeJobExecutionResponse(Mapping):
        pass
    
    class IncludeJobDocument(object):
        pass
    
    class ExecutionNumber(object):
        pass
    
    class GetPendingJobExecutionsRequest(Mapping):
        pass
    
    class GetPendingJobExecutionsResponse(Mapping):
        pass
    
    class StartNextPendingJobExecutionRequest(Mapping):
        pass
    
    class StartNextPendingJobExecutionResponse(Mapping):
        pass
    
    class DetailsMap(object):
        pass
    
    class StepTimeoutInMinutes(object):
        pass
    
    class UpdateJobExecutionRequest(Mapping):
        pass
    
    class UpdateJobExecutionResponse(Mapping):
        pass
    
    class ExpectedVersion(object):
        pass
    
    class IncludeExecutionState(object):
        pass
    
    def describe_job_execution(self, thingName: str, jobId: str, includeJobDocument: IncludeJobDocument=None, executionNumber: ExecutionNumber=None) -> DescribeJobExecutionResponse:
        """Gets details of a job execution.

        :param self:
        :param executionNumber:  Optional. A number that identifies a particular job execution on a particular device. If not specified, the latest job execution is returned.<br/>includeJobDocument:  Optional. When set to true, the response contains the job document. The default is false.<br/>thingName:  The thing name associated with the device the job execution is running on.<br/>jobId:  The unique identifier assigned to this job when it was created.<br/>
        :return: execution:  Contains data about a job execution.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jobDocument</i> The content of the job document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>executionNumber</i> A number that identifies a particular job execution on a particular device. It can be used later in commands that return or update job execution information.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>versionNumber</i> The version of the job execution. Job execution versions are incremented each time they are updated by a device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>approximateSecondsBeforeTimedOut</i> The estimated number of seconds that remain before the job execution status will be changed to <code>TIMED_OUT</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedAt</i> The time, in milliseconds since the epoch, when the job execution was last updated. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>startedAt</i> The time, in milliseconds since the epoch, when the job execution was started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>queuedAt</i> The time, in milliseconds since the epoch, when the job execution was enqueued.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusDetails</i> A collection of name/value pairs that describe the status of the job execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the job execution. Can be one of: "QUEUED", "IN_PROGRESS", "FAILED", "SUCCESS", "CANCELED", "REJECTED", or "REMOVED".<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>thingName</i> The name of the thing that is executing the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jobId</i> The unique identifier you assigned to this job when it was created.<br/>        """
        pass

    def get_pending_job_executions(self, thingName: str) -> GetPendingJobExecutionsResponse:
        """Gets the list of all jobs for a thing that are not in a terminal status.

        :param self:
        :param thingName:  The name of the thing that is executing the job.<br/>
        :return: queuedJobs:  A list of JobExecutionSummary objects with status QUEUED.<br/>inProgressJobs:  A list of JobExecutionSummary objects with status IN_PROGRESS.<br/>        """
        pass

    def start_next_pending_job_execution(self, thingName: str, statusDetails: DetailsMap=None, stepTimeoutInMinutes: StepTimeoutInMinutes=None) -> StartNextPendingJobExecutionResponse:
        """Gets and starts the next pending (status IN_PROGRESS or QUEUED) job execution for a thing.

        :param self:
        :param stepTimeoutInMinutes:  Specifies the amount of time this device has to finish execution of this job. If the job execution status is not set to a terminal state before this timer expires, or before the timer is reset (by calling <code>UpdateJobExecution</code>, setting the status to <code>IN_PROGRESS</code> and specifying a new timeout value in field <code>stepTimeoutInMinutes</code>) the job execution status will be automatically set to <code>TIMED_OUT</code>. Note that setting this timeout has no effect on that job execution timeout which may have been specified when the job was created (<code>CreateJob</code> using field <code>timeoutConfig</code>).<br/>statusDetails:  A collection of name/value pairs that describe the status of the job execution. If not specified, the statusDetails are unchanged.<br/>thingName:  The name of the thing associated with the device.<br/>
        :return: execution:  A JobExecution object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jobDocument</i> The content of the job document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>executionNumber</i> A number that identifies a particular job execution on a particular device. It can be used later in commands that return or update job execution information.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>versionNumber</i> The version of the job execution. Job execution versions are incremented each time they are updated by a device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>approximateSecondsBeforeTimedOut</i> The estimated number of seconds that remain before the job execution status will be changed to <code>TIMED_OUT</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedAt</i> The time, in milliseconds since the epoch, when the job execution was last updated. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>startedAt</i> The time, in milliseconds since the epoch, when the job execution was started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>queuedAt</i> The time, in milliseconds since the epoch, when the job execution was enqueued.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusDetails</i> A collection of name/value pairs that describe the status of the job execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the job execution. Can be one of: "QUEUED", "IN_PROGRESS", "FAILED", "SUCCESS", "CANCELED", "REJECTED", or "REMOVED".<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>thingName</i> The name of the thing that is executing the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jobId</i> The unique identifier you assigned to this job when it was created.<br/>        """
        pass

    def update_job_execution(self, status: str, thingName: str, jobId: str, statusDetails: DetailsMap=None, stepTimeoutInMinutes: StepTimeoutInMinutes=None, expectedVersion: ExpectedVersion=None, includeJobExecutionState: IncludeExecutionState=None, includeJobDocument: IncludeJobDocument=None, executionNumber: ExecutionNumber=None) -> UpdateJobExecutionResponse:
        """Updates the status of a job execution.

        :param self:
        :param executionNumber:  Optional. A number that identifies a particular job execution on a particular device.<br/>includeJobDocument:  Optional. When set to true, the response contains the job document. The default is false.<br/>includeJobExecutionState:  Optional. When included and set to true, the response contains the JobExecutionState data. The default is false.<br/>expectedVersion:  Optional. The expected current version of the job execution. Each time you update the job execution, its version is incremented. If the version of the job execution stored in Jobs does not match, the update is rejected with a VersionMismatch error, and an ErrorResponse that contains the current job execution status data is returned. (This makes it unnecessary to perform a separate DescribeJobExecution request in order to obtain the job execution status data.)<br/>stepTimeoutInMinutes:  Specifies the amount of time this device has to finish execution of this job. If the job execution status is not set to a terminal state before this timer expires, or before the timer is reset (by again calling <code>UpdateJobExecution</code>, setting the status to <code>IN_PROGRESS</code> and specifying a new timeout value in this field) the job execution status will be automatically set to <code>TIMED_OUT</code>. Note that setting or resetting this timeout has no effect on that job execution timeout which may have been specified when the job was created (<code>CreateJob</code> using field <code>timeoutConfig</code>).<br/>statusDetails:   Optional. A collection of name/value pairs that describe the status of the job execution. If not specified, the statusDetails are unchanged.<br/>status:  The new status for the job execution (IN_PROGRESS, FAILED, SUCCESS, or REJECTED). This must be specified on every update.<br/>thingName:  The name of the thing associated with the device.<br/>jobId:  The unique identifier assigned to this job when it was created.<br/>
        :return: jobDocument:  The contents of the Job Documents.<br/>executionState:  A JobExecutionState object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>versionNumber</i> The version of the job execution. Job execution versions are incremented each time they are updated by a device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusDetails</i> A collection of name/value pairs that describe the status of the job execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the job execution. Can be one of: "QUEUED", "IN_PROGRESS", "FAILED", "SUCCESS", "CANCELED", "REJECTED", or "REMOVED".<br/>        """
        pass



class IoT1ClickDevicesService(BaseClient):
    """Describes all of the AWS IoT 1-Click device-related API operations for the service.
 Also provides sample requests, responses, and errors for the supported web services
 protocols."""


    class ListDeviceEventsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_device_events` for valid parameters"""
            pass
    class ListDevicesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_devices` for valid parameters"""
            pass


    class ClaimDevicesByClaimCodeRequest(Mapping):
        pass
    
    class ClaimDevicesByClaimCodeResponse(Mapping):
        pass
    
    class DescribeDeviceRequest(Mapping):
        pass
    
    class DescribeDeviceResponse(Mapping):
        pass
    
    class FinalizeDeviceClaimRequest(Mapping):
        pass
    
    class FinalizeDeviceClaimResponse(Mapping):
        pass
    
    class __mapOf__string(object):
        pass
    
    class GetDeviceMethodsRequest(Mapping):
        pass
    
    class GetDeviceMethodsResponse(Mapping):
        pass
    
    class InitiateDeviceClaimRequest(Mapping):
        pass
    
    class InitiateDeviceClaimResponse(Mapping):
        pass
    
    class InvokeDeviceMethodRequest(Mapping):
        pass
    
    class InvokeDeviceMethodResponse(Mapping):
        pass
    
    class DeviceMethod(Mapping):
        pass
    
    class ListDeviceEventsRequest(Mapping):
        pass
    
    class ListDeviceEventsResponse(Mapping):
        pass
    
    class __timestampIso8601(object):
        pass
    
    class ListDevicesRequest(Mapping):
        pass
    
    class ListDevicesResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class UnclaimDeviceRequest(Mapping):
        pass
    
    class UnclaimDeviceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class __string(object):
        pass
    
    class UpdateDeviceStateRequest(Mapping):
        pass
    
    class UpdateDeviceStateResponse(Mapping):
        pass
    
    class __boolean(object):
        pass
    
    def claim_devices_by_claim_code(self, ClaimCode: str) -> ClaimDevicesByClaimCodeResponse:
        """Adds device(s) to your account (i.e., claim one or more devices) if and only if you
 received a claim code with the device(s).

        :param self:
        :param ClaimCode:  The claim code, starting with "C-", as provided by the device manufacturer.<br/>
        :return: Total:  The total number of devices associated with the claim code that has been processed in
 the claim request.<br/>ClaimCode:  The claim code provided by the device manufacturer.<br/>        """
        pass

    def describe_device(self, DeviceId: str) -> DescribeDeviceResponse:
        """Given a device ID, returns a DescribeDeviceResponse object describing the
 details of the device.

        :param self:
        :param DeviceId:  The unique identifier of the device.<br/>
        :return: DeviceDescription:  Device details.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The tags currently associated with the AWS IoT 1-Click device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of the device, such as "button".<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RemainingLife</i> A value between 0 and 1 inclusive, representing the fraction of life remaining for the
 device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> A Boolean value indicating whether or not the device is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceId</i> The unique identifier of the device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Attributes</i> An array of zero or more elements of DeviceAttribute objects providing
 user specified device attributes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN of the device.<br/>        """
        pass

    def finalize_device_claim(self, DeviceId: str, Tags: __mapOf__string=None) -> FinalizeDeviceClaimResponse:
        """Given a device ID, finalizes the claim request for the associated device.<note>
 Claiming a device consists of initiating a claim, then publishing a device event,
 and finalizing the claim. For a device of type button, a device event can
 be published by simply clicking the device.
 </note>

        :param self:
        :param Tags:  A collection of key/value pairs defining the resource tags. For example, {
 "tags": {"key1": "value1", "key2": "value2"} }. For more information, see <a href="https://aws.amazon.com/answers/account-management/aws-tagging-strategies/">AWS
 Tagging Strategies</a>.
 
 <br/>DeviceId:  The unique identifier of the device.<br/>
        :return: State:  The device's final claim state.<br/>        """
        pass

    def get_device_methods(self, DeviceId: str) -> GetDeviceMethodsResponse:
        """Given a device ID, returns the invokable methods associated with the device.

        :param self:
        :param DeviceId:  The unique identifier of the device.<br/>
        :return: DeviceMethods:  List of available device APIs.<br/>        """
        pass

    def initiate_device_claim(self, DeviceId: str) -> InitiateDeviceClaimResponse:
        """Given a device ID, initiates a claim request for the associated device.<note>
 Claiming a device consists of initiating a claim, then publishing a device event,
 and finalizing the claim. For a device of type button, a device event can
 be published by simply clicking the device.
 </note>

        :param self:
        :param DeviceId:  The unique identifier of the device.<br/>
        :return: State:  The device's final claim state.<br/>        """
        pass

    def invoke_device_method(self, DeviceId: str, DeviceMethod: DeviceMethod=None, DeviceMethodParameters: str=None) -> InvokeDeviceMethodResponse:
        """Given a device ID, issues a request to invoke a named device method (with possible
 parameters). See the "Example POST" code snippet below.

        :param self:
        :param DeviceMethodParameters:  A JSON encoded string containing the device method request parameters.<br/>DeviceMethod:  The device method to invoke.:<br/>DeviceId:  The unique identifier of the device.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MethodName</i> The name of the method applicable to the deviceType.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceType</i> The type of the device, such as "button".<br/>
        :return: DeviceMethodResponse:  A JSON encoded string containing the device method response.<br/>        """
        pass

    def list_device_events(self, ToTimeStamp: __timestampIso8601, FromTimeStamp: __timestampIso8601, DeviceId: str, MaxResults: int=None, NextToken: str=None) -> ListDeviceEventsResponse:
        """Using a device ID, returns a DeviceEventsResponse object containing an
 array of events for the device.

        :param self:
        :param ToTimeStamp:  The end date for the device event query, in ISO8061 format. For example,
 2018-03-28T15:45:12.880Z
 <br/>NextToken:  The token to retrieve the next set of results.<br/>MaxResults:  The maximum number of results to return per request. If not set, a default value of
 100 is used.<br/>FromTimeStamp:  The start date for the device event query, in ISO8061 format. For example,
 2018-03-28T15:45:12.880Z
 <br/>DeviceId:  The unique identifier of the device.<br/>
        :return: NextToken:  The token to retrieve the next set of results.<br/>Events:  An array of zero or more elements describing the event(s) associated with the
 device.<br/>        """
        pass

    def list_devices(self, DeviceType: str=None, MaxResults: int=None, NextToken: str=None) -> ListDevicesResponse:
        """Lists the 1-Click compatible devices associated with your AWS account.

        :param self:
        :param NextToken:  The token to retrieve the next set of results.<br/>MaxResults:  The maximum number of results to return per request. If not set, a default value of
 100 is used.<br/>DeviceType:  The type of the device, such as "button".<br/>
        :return: NextToken:  The token to retrieve the next set of results.<br/>Devices:  A list of devices.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceArn: str) -> ListTagsForResourceResponse:
        """Lists the tags associated with the specified resource ARN.

        :param self:
        :param ResourceArn:  The ARN of the resource.<br/>
        :return: Tags:  A collection of key/value pairs defining the resource tags. For example, {
 "tags": {"key1": "value1", "key2": "value2"} }. For more information, see <a href="https://aws.amazon.com/answers/account-management/aws-tagging-strategies/">AWS
 Tagging Strategies</a>.
 
 <br/>        """
        pass

    def tag_resource(self, Tags: __mapOf__string, ResourceArn: str):
        """Adds or updates the tags associated with the resource ARN. See <a href="https://docs.aws.amazon.com/iot-1-click/latest/developerguide/1click-appendix.html#1click-limits">AWS IoT 1-Click Service Limits</a> for the maximum number of tags allowed per
 resource.

        :param self:
        :param Tags:  A collection of key/value pairs defining the resource tags. For example, {
 "tags": {"key1": "value1", "key2": "value2"} }. For more information, see <a href="https://aws.amazon.com/answers/account-management/aws-tagging-strategies/">AWS
 Tagging Strategies</a>.
 
 <br/>ResourceArn:  The ARN of the resource.<br/>
        :return:         """
        pass

    def unclaim_device(self, DeviceId: str) -> UnclaimDeviceResponse:
        """Disassociates a device from your AWS account using its device ID.

        :param self:
        :param DeviceId:  The unique identifier of the device.<br/>
        :return: State:  The device's final claim state.<br/>        """
        pass

    def untag_resource(self, TagKeys: List[__string], ResourceArn: str):
        """Using tag keys, deletes the tags (key/value pairs) associated with the specified
 resource ARN.

        :param self:
        :param TagKeys:  A collections of tag keys. For example, {"key1","key2"}<br/>ResourceArn:  The ARN of the resource.<br/>
        :return:         """
        pass

    def update_device_state(self, DeviceId: str, Enabled: __boolean=None) -> UpdateDeviceStateResponse:
        """Using a Boolean value (true or false), this operation
 enables or disables the device given a device ID.

        :param self:
        :param Enabled:  If true, the device is enabled. If false, the device is
 disabled.<br/>DeviceId:  The unique identifier of the device.<br/>
        :return:         """
        pass



class IoT1ClickProjects(BaseClient):
    """The AWS IoT 1-Click Projects API Reference"""


    class ListPlacementsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_placements` for valid parameters"""
            pass
    class ListProjectsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_projects` for valid parameters"""
            pass


    class AssociateDeviceWithPlacementRequest(Mapping):
        pass
    
    class AssociateDeviceWithPlacementResponse(Mapping):
        pass
    
    class CreatePlacementRequest(Mapping):
        pass
    
    class CreatePlacementResponse(Mapping):
        pass
    
    class PlacementAttributeMap(object):
        pass
    
    class CreateProjectRequest(Mapping):
        pass
    
    class CreateProjectResponse(Mapping):
        pass
    
    class PlacementTemplate(Mapping):
        pass
    
    class TagMap(object):
        pass
    
    class DeletePlacementRequest(Mapping):
        pass
    
    class DeletePlacementResponse(Mapping):
        pass
    
    class DeleteProjectRequest(Mapping):
        pass
    
    class DeleteProjectResponse(Mapping):
        pass
    
    class DescribePlacementRequest(Mapping):
        pass
    
    class DescribePlacementResponse(Mapping):
        pass
    
    class DescribeProjectRequest(Mapping):
        pass
    
    class DescribeProjectResponse(Mapping):
        pass
    
    class DisassociateDeviceFromPlacementRequest(Mapping):
        pass
    
    class DisassociateDeviceFromPlacementResponse(Mapping):
        pass
    
    class GetDevicesInPlacementRequest(Mapping):
        pass
    
    class GetDevicesInPlacementResponse(Mapping):
        pass
    
    class ListPlacementsRequest(Mapping):
        pass
    
    class ListPlacementsResponse(Mapping):
        pass
    
    class ListProjectsRequest(Mapping):
        pass
    
    class ListProjectsResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdatePlacementRequest(Mapping):
        pass
    
    class UpdatePlacementResponse(Mapping):
        pass
    
    class UpdateProjectRequest(Mapping):
        pass
    
    class UpdateProjectResponse(Mapping):
        pass
    
    def associate_device_with_placement(self, deviceTemplateName: str, deviceId: str, placementName: str, projectName: str) -> AssociateDeviceWithPlacementResponse:
        """Associates a physical device with a placement.

        :param self:
        :param deviceTemplateName:  The device template name to associate with the device ID.<br/>deviceId:  The ID of the physical device to be associated with the given placement in the project. Note that a mandatory 4 character prefix is required for all <code>deviceId</code> values.<br/>placementName:  The name of the placement in which to associate the device.<br/>projectName:  The name of the project containing the placement in which to associate the device.<br/>
        :return:         """
        pass

    def create_placement(self, projectName: str, placementName: str, attributes: PlacementAttributeMap=None) -> CreatePlacementResponse:
        """Creates an empty placement.

        :param self:
        :param attributes:  Optional user-defined key/value pairs providing contextual data (such as location or function) for the placement.<br/>projectName:  The name of the project in which to create the placement.<br/>placementName:  The name of the placement to be created.<br/>
        :return:         """
        pass

    def create_project(self, projectName: str, description: str=None, placementTemplate: PlacementTemplate=None, tags: TagMap=None) -> CreateProjectResponse:
        """Creates an empty project with a placement template. A project contains zero or more placements that adhere to the placement template defined in the project.

        :param self:
        :param tags:  Optional tags (metadata key/value pairs) to be associated with the project. For example, <code>{ {"key1": "value1", "key2": "value2"} }</code>. For more information, see <a href="https://aws.amazon.com/answers/account-management/aws-tagging-strategies/">AWS Tagging Strategies</a>.<br/>placementTemplate:  The schema defining the placement to be created. A placement template defines placement default attributes and device templates. You cannot add or remove device templates after the project has been created. However, you can update <code>callbackOverrides</code> for the device templates using the <code>UpdateProject</code> API.:<br/>description:  An optional description for the project.<br/>projectName:  The name of the project to create.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deviceTemplates</i> An object specifying the <a>DeviceTemplate</a> for all placements using this (<a>PlacementTemplate</a>) template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultAttributes</i> The default attributes (key/value pairs) to be applied to all placements using this template.<br/>
        :return:         """
        pass

    def delete_placement(self, projectName: str, placementName: str) -> DeletePlacementResponse:
        """Deletes a placement. To delete a placement, it must not have any devices associated with it. <note> When you delete a placement, all associated data becomes irretrievable. </note>

        :param self:
        :param projectName:  The project containing the empty placement to delete.<br/>placementName:  The name of the empty placement to delete.<br/>
        :return:         """
        pass

    def delete_project(self, projectName: str) -> DeleteProjectResponse:
        """Deletes a project. To delete a project, it must not have any placements associated with it. <note> When you delete a project, all associated data becomes irretrievable. </note>

        :param self:
        :param projectName:  The name of the empty project to delete.<br/>
        :return:         """
        pass

    def describe_placement(self, projectName: str, placementName: str) -> DescribePlacementResponse:
        """Describes a placement in a project.

        :param self:
        :param projectName:  The project containing the placement to be described.<br/>placementName:  The name of the placement within a project.<br/>
        :return: placement:  An object describing the placement.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>updatedDate</i> The date when the placement was last updated, in UNIX epoch time format. If the placement was not updated, then <code>createdDate</code> and <code>updatedDate</code> are the same.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdDate</i> The date when the placement was initially created, in UNIX epoch time format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>attributes</i> The user-defined attributes associated with the placement.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>placementName</i> The name of the placement.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>projectName</i> The name of the project containing the placement.<br/>        """
        pass

    def describe_project(self, projectName: str) -> DescribeProjectResponse:
        """Returns an object describing a project.

        :param self:
        :param projectName:  The name of the project to be described.<br/>
        :return: project:  An object describing the project.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> The tags (metadata key/value pairs) associated with the project.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>placementTemplate</i> An object describing the project's placement specifications.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>placementTemplate</b>: An object describing the project's placement specifications.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>updatedDate</i> The date when the project was last updated, in UNIX epoch time format. If the project was not updated, then <code>createdDate</code> and <code>updatedDate</code> are the same.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdDate</i> The date when the project was originally created, in UNIX epoch time format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The description of the project.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>projectName</i> The name of the project for which to obtain information from.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The ARN of the project.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>deviceTemplates</i> An object specifying the <a>DeviceTemplate</a> for all placements using this (<a>PlacementTemplate</a>) template.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultAttributes</i> The default attributes (key/value pairs) to be applied to all placements using this template.<br/>        """
        pass

    def disassociate_device_from_placement(self, deviceTemplateName: str, placementName: str, projectName: str) -> DisassociateDeviceFromPlacementResponse:
        """Removes a physical device from a placement.

        :param self:
        :param deviceTemplateName:  The device ID that should be removed from the placement.<br/>placementName:  The name of the placement that the device should be removed from.<br/>projectName:  The name of the project that contains the placement.<br/>
        :return:         """
        pass

    def get_devices_in_placement(self, placementName: str, projectName: str) -> GetDevicesInPlacementResponse:
        """Returns an object enumerating the devices in a placement.

        :param self:
        :param placementName:  The name of the placement to get the devices from.<br/>projectName:  The name of the project containing the placement.<br/>
        :return: devices:  An object containing the devices (zero or more) within the placement.<br/>        """
        pass

    def list_placements(self, projectName: str, nextToken: str=None, maxResults: int=None) -> ListPlacementsResponse:
        """Lists the placement(s) of a project.

        :param self:
        :param maxResults:  The maximum number of results to return per request. If not set, a default value of 100 is used.<br/>nextToken:  The token to retrieve the next set of results.<br/>projectName:  The project containing the placements to be listed.<br/>
        :return: nextToken:  The token used to retrieve the next set of results - will be effectively empty if there are no further results.<br/>placements:  An object listing the requested placements.<br/>        """
        pass

    def list_projects(self, nextToken: str=None, maxResults: int=None) -> ListProjectsResponse:
        """Lists the AWS IoT 1-Click project(s) associated with your AWS account and region.

        :param self:
        :param maxResults:  The maximum number of results to return per request. If not set, a default value of 100 is used.<br/>nextToken:  The token to retrieve the next set of results.<br/>
        :return: nextToken:  The token used to retrieve the next set of results - will be effectively empty if there are no further results.<br/>projects:  An object containing the list of projects.<br/>        """
        pass

    def list_tags_for_resource(self, resourceArn: str) -> ListTagsForResourceResponse:
        """Lists the tags (metadata key/value pairs) which you have assigned to the resource.

        :param self:
        :param resourceArn:  The ARN of the resource whose tags you want to list.<br/>
        :return: tags:  The tags (metadata key/value pairs) which you have assigned to the resource.<br/>        """
        pass

    def tag_resource(self, tags: TagMap, resourceArn: str) -> TagResourceResponse:
        """Creates or modifies tags for a resource. Tags are key/value pairs (metadata) that can be used to manage a resource. For more information, see <a href="https://aws.amazon.com/answers/account-management/aws-tagging-strategies/">AWS Tagging Strategies</a>.

        :param self:
        :param tags:  The new or modifying tag(s) for the resource. See <a href="https://docs.aws.amazon.com/iot-1-click/latest/developerguide/1click-appendix.html#1click-limits">AWS IoT 1-Click Service Limits</a> for the maximum number of tags allowed per resource.<br/>resourceArn:  The ARN of the resouce for which tag(s) should be added or modified.<br/>
        :return:         """
        pass

    def untag_resource(self, tagKeys: List[TagKey], resourceArn: str) -> UntagResourceResponse:
        """Removes one or more tags (metadata key/value pairs) from a resource.

        :param self:
        :param tagKeys:  The keys of those tags which you want to remove.<br/>resourceArn:  The ARN of the resource whose tag you want to remove.<br/>
        :return:         """
        pass

    def update_placement(self, projectName: str, placementName: str, attributes: PlacementAttributeMap=None) -> UpdatePlacementResponse:
        """Updates a placement with the given attributes. To clear an attribute, pass an empty value (i.e., "").

        :param self:
        :param attributes:  The user-defined object of attributes used to update the placement. The maximum number of key/value pairs is 50.<br/>projectName:  The name of the project containing the placement to be updated.<br/>placementName:  The name of the placement to update.<br/>
        :return:         """
        pass

    def update_project(self, projectName: str, description: str=None, placementTemplate: PlacementTemplate=None) -> UpdateProjectResponse:
        """Updates a project associated with your AWS account and region. With the exception of device template names, you can pass just the values that need to be updated because the update request will change only the values that are provided. To clear a value, pass the empty string (i.e., <code>""</code>).

        :param self:
        :param placementTemplate:  An object defining the project update. Once a project has been created, you cannot add device template names to the project. However, for a given <code>placementTemplate</code>, you can update the associated <code>callbackOverrides</code> for the device definition using this API.:<br/>description:  An optional user-defined description for the project.<br/>projectName:  The name of the project to be updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deviceTemplates</i> An object specifying the <a>DeviceTemplate</a> for all placements using this (<a>PlacementTemplate</a>) template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultAttributes</i> The default attributes (key/value pairs) to be applied to all placements using this template.<br/>
        :return:         """
        pass



class IoTAnalytics(BaseClient):
    """AWS IoT Analytics allows you to collect large amounts of device data, process messages, and store them. You can then query the data and run sophisticated analytics on it. AWS IoT Analytics enables advanced data exploration through integration with Jupyter Notebooks and data visualization through integration with Amazon QuickSight. Traditional analytics and business intelligence tools are designed to process structured data. IoT data often comes from devices that record noisy processes (such as temperature, motion, or sound). As a result the data from these devices can have significant gaps, corrupted messages, and false readings that must be cleaned up before analysis can occur. Also, IoT data is often only meaningful in the context of other data from external sources.  AWS IoT Analytics automates the steps required to analyze data from IoT devices. AWS IoT Analytics filters, transforms, and enriches IoT data before storing it in a time-series data store for analysis. You can set up the service to collect only the data you need from your devices, apply mathematical transforms to process the data, and enrich the data with device-specific metadata such as device type and location before storing it. Then, you can analyze your data by running queries using the built-in SQL query engine, or perform more complex analytics and machine learning inference. AWS IoT Analytics includes pre-built models for common IoT use cases so you can answer questions like which devices are about to fail or which customers are at risk of abandoning their wearable devices."""


    class ListChannelsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_channels` for valid parameters"""
            pass
    class ListDatasetContentsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_dataset_contents` for valid parameters"""
            pass
    class ListDatasetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_datasets` for valid parameters"""
            pass
    class ListDatastoresPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_datastores` for valid parameters"""
            pass
    class ListPipelinesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_pipelines` for valid parameters"""
            pass


    class BatchPutMessageRequest(Mapping):
        pass
    
    class BatchPutMessageResponse(Mapping):
        pass
    
    class Message(Mapping):
        pass
    
    class CancelPipelineReprocessingRequest(Mapping):
        pass
    
    class CancelPipelineReprocessingResponse(Mapping):
        pass
    
    class CreateChannelRequest(Mapping):
        pass
    
    class CreateChannelResponse(Mapping):
        pass
    
    class ChannelStorage(Mapping):
        pass
    
    class RetentionPeriod(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreateDatasetRequest(Mapping):
        pass
    
    class CreateDatasetResponse(Mapping):
        pass
    
    class DatasetAction(Mapping):
        pass
    
    class DatasetTrigger(Mapping):
        pass
    
    class DatasetContentDeliveryRule(Mapping):
        pass
    
    class VersioningConfiguration(Mapping):
        pass
    
    class CreateDatasetContentRequest(Mapping):
        pass
    
    class CreateDatasetContentResponse(Mapping):
        pass
    
    class CreateDatastoreRequest(Mapping):
        pass
    
    class CreateDatastoreResponse(Mapping):
        pass
    
    class DatastoreStorage(Mapping):
        pass
    
    class CreatePipelineRequest(Mapping):
        pass
    
    class CreatePipelineResponse(Mapping):
        pass
    
    class PipelineActivity(Mapping):
        pass
    
    class DeleteChannelRequest(Mapping):
        pass
    
    class DeleteDatasetRequest(Mapping):
        pass
    
    class DeleteDatasetContentRequest(Mapping):
        pass
    
    class DeleteDatastoreRequest(Mapping):
        pass
    
    class DeletePipelineRequest(Mapping):
        pass
    
    class DescribeChannelRequest(Mapping):
        pass
    
    class DescribeChannelResponse(Mapping):
        pass
    
    class IncludeStatisticsFlag(object):
        pass
    
    class DescribeDatasetRequest(Mapping):
        pass
    
    class DescribeDatasetResponse(Mapping):
        pass
    
    class DescribeDatastoreRequest(Mapping):
        pass
    
    class DescribeDatastoreResponse(Mapping):
        pass
    
    class DescribeLoggingOptionsRequest(Mapping):
        pass
    
    class DescribeLoggingOptionsResponse(Mapping):
        pass
    
    class DescribePipelineRequest(Mapping):
        pass
    
    class DescribePipelineResponse(Mapping):
        pass
    
    class GetDatasetContentRequest(Mapping):
        pass
    
    class GetDatasetContentResponse(Mapping):
        pass
    
    class ListChannelsRequest(Mapping):
        pass
    
    class ListChannelsResponse(Mapping):
        pass
    
    class ListDatasetContentsRequest(Mapping):
        pass
    
    class ListDatasetContentsResponse(Mapping):
        pass
    
    class Timestamp(object):
        pass
    
    class ListDatasetsRequest(Mapping):
        pass
    
    class ListDatasetsResponse(Mapping):
        pass
    
    class ListDatastoresRequest(Mapping):
        pass
    
    class ListDatastoresResponse(Mapping):
        pass
    
    class ListPipelinesRequest(Mapping):
        pass
    
    class ListPipelinesResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class PutLoggingOptionsRequest(Mapping):
        pass
    
    class LoggingOptions(Mapping):
        pass
    
    class RunPipelineActivityRequest(Mapping):
        pass
    
    class RunPipelineActivityResponse(Mapping):
        pass
    
    class MessagePayload(object):
        pass
    
    class SampleChannelDataRequest(Mapping):
        pass
    
    class SampleChannelDataResponse(Mapping):
        pass
    
    class StartTime(object):
        pass
    
    class EndTime(object):
        pass
    
    class StartPipelineReprocessingRequest(Mapping):
        pass
    
    class StartPipelineReprocessingResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateChannelRequest(Mapping):
        pass
    
    class UpdateDatasetRequest(Mapping):
        pass
    
    class UpdateDatastoreRequest(Mapping):
        pass
    
    class UpdatePipelineRequest(Mapping):
        pass
    
    def batch_put_message(self, messages: List[Message], channelName: str) -> BatchPutMessageResponse:
        """Sends messages to a channel.

        :param self:
        :param messages:  The list of messages to be sent. Each message has format: '{ "messageId": "string", "payload": "string"}'. Note that the field names of message payloads (data) that you send to AWS IoT Analytics: <ul> <li> Must contain only alphanumeric characters and undescores (_); no other special characters are allowed. </li> <li> Must begin with an alphabetic character or single underscore (_). </li> <li> Cannot contain hyphens (-). </li> <li> In regular expression terms: "^[A-Za-z_]([A-Za-z0-9]*|[A-Za-z0-9][A-Za-z0-9_]*)$".  </li> <li> Cannot be greater than 255 characters. </li> <li> Are case-insensitive. (Fields named "foo" and "FOO" in the same payload are considered duplicates.) </li> </ul> For example, {"temp_01": 29} or {"_temp_01": 29} are valid, but {"temp-01": 29}, {"01_temp": 29} or {"__temp_01": 29} are invalid in message payloads. <br/>channelName:  The name of the channel where the messages are sent.<br/>
        :return: batchPutMessageErrorEntries:  A list of any errors encountered when sending the messages to the channel.<br/>        """
        pass

    def cancel_pipeline_reprocessing(self, reprocessingId: str, pipelineName: str) -> CancelPipelineReprocessingResponse:
        """Cancels the reprocessing of data through the pipeline.

        :param self:
        :param reprocessingId:  The ID of the reprocessing task (returned by "StartPipelineReprocessing").<br/>pipelineName:  The name of pipeline for which data reprocessing is canceled.<br/>
        :return:         """
        pass

    def create_channel(self, channelName: str, channelStorage: ChannelStorage=None, retentionPeriod: RetentionPeriod=None, tags: List[Tag]=None) -> CreateChannelResponse:
        """Creates a channel. A channel collects data from an MQTT topic and archives the raw, unprocessed messages before publishing the data to a pipeline.

        :param self:
        :param tags:  Metadata which can be used to manage the channel.<br/>retentionPeriod:  How long, in days, message data is kept for the channel.:<br/>channelStorage:  Where channel data is stored.:<br/>channelName:  The name of the channel.<br/>Use this to store channel data in an S3 bucket managed by the AWS IoT Analytics service.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>serviceManagedS3</b>: Use this to store channel data in an S3 bucket managed by the AWS IoT Analytics service.&nbsp;&nbsp;&nbsp;&nbsp; <i>numberOfDays</i> The number of days that message data is kept. The "unlimited" parameter must be false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>unlimited</i> If true, message data is kept indefinitely.<br/>
        :return: retentionPeriod:  How long, in days, message data is kept for the channel.:<br/>channelArn:  The ARN of the channel.<br/>channelName:  The name of the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>numberOfDays</i> The number of days that message data is kept. The "unlimited" parameter must be false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>unlimited</i> If true, message data is kept indefinitely.<br/>        """
        pass

    def create_dataset(self, actions: List[DatasetAction], datasetName: str, triggers: List[DatasetTrigger]=None, contentDeliveryRules: List[DatasetContentDeliveryRule]=None, retentionPeriod: RetentionPeriod=None, versioningConfiguration: VersioningConfiguration=None, tags: List[Tag]=None) -> CreateDatasetResponse:
        """Creates a data set. A data set stores data retrieved from a data store by applying a "queryAction" (a SQL query) or a "containerAction" (executing a containerized application). This operation creates the skeleton of a data set. The data set can be populated manually by calling "CreateDatasetContent" or automatically according to a "trigger" you specify.

        :param self:
        :param tags:  Metadata which can be used to manage the data set.<br/>versioningConfiguration:  [Optional] How many versions of data set contents are kept. If not specified or set to null, only the latest version plus the latest succeeded version (if they are different) are kept for the time period specified by the "retentionPeriod" parameter. (For more information, see https://docs.aws.amazon.com/iotanalytics/latest/userguide/getting-started.html#aws-iot-analytics-dataset-versions):<br/>retentionPeriod:  [Optional] How long, in days, versions of data set contents are kept for the data set. If not specified or set to null, versions of data set contents are retained for at most 90 days. The number of versions of data set contents retained is determined by the <code>versioningConfiguration</code> parameter. (For more information, see https://docs.aws.amazon.com/iotanalytics/latest/userguide/getting-started.html#aws-iot-analytics-dataset-versions):<br/>contentDeliveryRules:  When data set contents are created they are delivered to destinations specified here.<br/>triggers:  A list of triggers. A trigger causes data set contents to be populated at a specified time interval or when another data set's contents are created. The list of triggers can be empty or contain up to five <b>DataSetTrigger</b> objects.<br/>actions:  A list of actions that create the data set contents.<br/>datasetName:  The name of the data set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>numberOfDays</i> The number of days that message data is kept. The "unlimited" parameter must be false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>unlimited</i> If true, message data is kept indefinitely.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>maxVersions</i> How many versions of data set contents will be kept. The "unlimited" parameter must be false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>unlimited</i> If true, unlimited versions of data set contents will be kept.<br/>
        :return: retentionPeriod:  How long, in days, data set contents are kept for the data set.:<br/>datasetArn:  The ARN of the data set.<br/>datasetName:  The name of the data set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>numberOfDays</i> The number of days that message data is kept. The "unlimited" parameter must be false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>unlimited</i> If true, message data is kept indefinitely.<br/>        """
        pass

    def create_dataset_content(self, datasetName: str) -> CreateDatasetContentResponse:
        """Creates the content of a data set by applying a "queryAction" (a SQL query) or a "containerAction" (executing a containerized application).

        :param self:
        :param datasetName:  The name of the data set.<br/>
        :return: versionId:  The version ID of the data set contents which are being created.<br/>        """
        pass

    def create_datastore(self, datastoreName: str, datastoreStorage: DatastoreStorage=None, retentionPeriod: RetentionPeriod=None, tags: List[Tag]=None) -> CreateDatastoreResponse:
        """Creates a data store, which is a repository for messages.

        :param self:
        :param tags:  Metadata which can be used to manage the data store.<br/>retentionPeriod:  How long, in days, message data is kept for the data store.:<br/>datastoreStorage:  Where data store data is stored.:<br/>datastoreName:  The name of the data store.<br/>Use this to store data store data in an S3 bucket managed by the AWS IoT Analytics service.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>serviceManagedS3</b>: Use this to store data store data in an S3 bucket managed by the AWS IoT Analytics service.&nbsp;&nbsp;&nbsp;&nbsp; <i>numberOfDays</i> The number of days that message data is kept. The "unlimited" parameter must be false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>unlimited</i> If true, message data is kept indefinitely.<br/>
        :return: retentionPeriod:  How long, in days, message data is kept for the data store.:<br/>datastoreArn:  The ARN of the data store.<br/>datastoreName:  The name of the data store.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>numberOfDays</i> The number of days that message data is kept. The "unlimited" parameter must be false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>unlimited</i> If true, message data is kept indefinitely.<br/>        """
        pass

    def create_pipeline(self, pipelineActivities: List[PipelineActivity], pipelineName: str, tags: List[Tag]=None) -> CreatePipelineResponse:
        """Creates a pipeline. A pipeline consumes messages from one or more channels and allows you to process the messages before storing them in a data store. You must specify both a <code>channel</code> and a <code>datastore</code> activity and, optionally, as many as 23 additional activities in the <code>pipelineActivities</code> array.

        :param self:
        :param tags:  Metadata which can be used to manage the pipeline.<br/>pipelineActivities:  A list of "PipelineActivity" objects. Activities perform transformations on your messages, such as removing, renaming or adding message attributes; filtering messages based on attribute values; invoking your Lambda functions on messages for advanced processing; or performing mathematical transformations to normalize device data. The list can be 2-25 <b>PipelineActivity</b> objects and must contain both a <code>channel</code> and a <code>datastore</code> activity. Each entry in the list must contain only one activity, for example:  <code>pipelineActivities = [ { "channel": { ... } }, { "lambda": { ... } }, ... ]</code> <br/>pipelineName:  The name of the pipeline.<br/>
        :return: pipelineArn:  The ARN of the pipeline.<br/>pipelineName:  The name of the pipeline.<br/>        """
        pass

    def delete_channel(self, channelName: str):
        """Deletes the specified channel.

        :param self:
        :param channelName:  The name of the channel to delete.<br/>
        :return:         """
        pass

    def delete_dataset(self, datasetName: str):
        """Deletes the specified data set. You do not have to delete the content of the data set before you perform this operation.

        :param self:
        :param datasetName:  The name of the data set to delete.<br/>
        :return:         """
        pass

    def delete_dataset_content(self, datasetName: str, versionId: str=None):
        """Deletes the content of the specified data set.

        :param self:
        :param versionId:  The version of the data set whose content is deleted. You can also use the strings "$LATEST" or "$LATEST_SUCCEEDED" to delete the latest or latest successfully completed data set. If not specified, "$LATEST_SUCCEEDED" is the default.<br/>datasetName:  The name of the data set whose content is deleted.<br/>
        :return:         """
        pass

    def delete_datastore(self, datastoreName: str):
        """Deletes the specified data store.

        :param self:
        :param datastoreName:  The name of the data store to delete.<br/>
        :return:         """
        pass

    def delete_pipeline(self, pipelineName: str):
        """Deletes the specified pipeline.

        :param self:
        :param pipelineName:  The name of the pipeline to delete.<br/>
        :return:         """
        pass

    def describe_channel(self, channelName: str, includeStatistics: IncludeStatisticsFlag=None) -> DescribeChannelResponse:
        """Retrieves information about a channel.

        :param self:
        :param includeStatistics:  If true, additional statistical information about the channel is included in the response.<br/>channelName:  The name of the channel whose information is retrieved.<br/>
        :return: statistics:  Statistics about the channel. Included if the 'includeStatistics' parameter is set to true in the request.:<br/>channel:  An object that contains information about the channel.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdateTime</i> When the channel was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationTime</i> When the channel was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>retentionPeriod</i> How long, in days, message data is kept for the channel.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>retentionPeriod</b>: How long, in days, message data is kept for the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The ARN of the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>storage</i> Where channel data is stored.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>storage</b>: Where channel data is stored.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the channel.<br/>Use this to store channel data in an S3 bucket managed by the AWS IoT Analytics service.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>serviceManagedS3</b>: Use this to store channel data in an S3 bucket managed by the AWS IoT Analytics service.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>numberOfDays</i> The number of days that message data is kept. The "unlimited" parameter must be false.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>unlimited</i> If true, message data is kept indefinitely.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>size</i> The estimated size of the channel.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>size</b>: The estimated size of the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>estimatedOn</i> The time when the estimate of the size of the resource was made.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>estimatedSizeInBytes</i> The estimated size of the resource in bytes.<br/>        """
        pass

    def describe_dataset(self, datasetName: str) -> DescribeDatasetResponse:
        """Retrieves information about a data set.

        :param self:
        :param datasetName:  The name of the data set whose information is retrieved.<br/>
        :return: dataset:  An object that contains information about the data set.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>versioningConfiguration</i> [Optional] How many versions of data set contents are kept. If not specified or set to null, only the latest version plus the latest succeeded version (if they are different) are kept for the time period specified by the "retentionPeriod" parameter. (For more information, see https://docs.aws.amazon.com/iotanalytics/latest/userguide/getting-started.html#aws-iot-analytics-dataset-versions)&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>versioningConfiguration</b>: [Optional] How many versions of data set contents are kept. If not specified or set to null, only the latest version plus the latest succeeded version (if they are different) are kept for the time period specified by the "retentionPeriod" parameter. (For more information, see https://docs.aws.amazon.com/iotanalytics/latest/userguide/getting-started.html#aws-iot-analytics-dataset-versions)<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>retentionPeriod</i> [Optional] How long, in days, message data is kept for the data set.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>retentionPeriod</b>: [Optional] How long, in days, message data is kept for the data set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdateTime</i> The last time the data set was updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationTime</i> When the data set was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the data set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>contentDeliveryRules</i> When data set contents are created they are delivered to destinations specified here.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>triggers</i> The "DatasetTrigger" objects that specify when the data set is automatically updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>actions</i> The "DatasetAction" objects that automatically create the data set contents.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The ARN of the data set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the data set.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>numberOfDays</i> The number of days that message data is kept. The "unlimited" parameter must be false.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>unlimited</i> If true, message data is kept indefinitely.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>maxVersions</i> How many versions of data set contents will be kept. The "unlimited" parameter must be false.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>unlimited</i> If true, unlimited versions of data set contents will be kept.<br/>        """
        pass

    def describe_datastore(self, datastoreName: str, includeStatistics: IncludeStatisticsFlag=None) -> DescribeDatastoreResponse:
        """Retrieves information about a data store.

        :param self:
        :param includeStatistics:  If true, additional statistical information about the datastore is included in the response.<br/>datastoreName:  The name of the data store<br/>
        :return: statistics:  Additional statistical information about the data store. Included if the 'includeStatistics' parameter is set to true in the request.:<br/>datastore:  Information about the data store.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdateTime</i> The last time the data store was updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationTime</i> When the data store was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>retentionPeriod</i> How long, in days, message data is kept for the data store.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>retentionPeriod</b>: How long, in days, message data is kept for the data store.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of a data store: <dl> <dt>CREATING</dt> <dd> The data store is being created. </dd> <dt>ACTIVE</dt> <dd> The data store has been created and can be used. </dd> <dt>DELETING</dt> <dd> The data store is being deleted. </dd> </dl><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The ARN of the data store.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>storage</i> Where data store data is stored.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>storage</b>: Where data store data is stored.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the data store.<br/>Use this to store data store data in an S3 bucket managed by the AWS IoT Analytics service.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>serviceManagedS3</b>: Use this to store data store data in an S3 bucket managed by the AWS IoT Analytics service.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>numberOfDays</i> The number of days that message data is kept. The "unlimited" parameter must be false.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>unlimited</i> If true, message data is kept indefinitely.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>size</i> The estimated size of the data store.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>size</b>: The estimated size of the data store.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>estimatedOn</i> The time when the estimate of the size of the resource was made.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>estimatedSizeInBytes</i> The estimated size of the resource in bytes.<br/>        """
        pass

    def describe_logging_options(self) -> DescribeLoggingOptionsResponse:
        """Retrieves the current settings of the AWS IoT Analytics logging options.

        :param self:
        :param 
        :return: loggingOptions:  The current settings of the AWS IoT Analytics logging options.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enabled</i> If true, logging is enabled for AWS IoT Analytics.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>level</i> The logging level. Currently, only "ERROR" is supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The ARN of the role that grants permission to AWS IoT Analytics to perform logging.<br/>        """
        pass

    def describe_pipeline(self, pipelineName: str) -> DescribePipelineResponse:
        """Retrieves information about a pipeline.

        :param self:
        :param pipelineName:  The name of the pipeline whose information is retrieved.<br/>
        :return: pipeline:  A "Pipeline" object that contains information about the pipeline.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdateTime</i> The last time the pipeline was updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationTime</i> When the pipeline was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>reprocessingSummaries</i> A summary of information about the pipeline reprocessing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>activities</i> The activities that perform transformations on the messages.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The ARN of the pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the pipeline.<br/>        """
        pass

    def get_dataset_content(self, datasetName: str, versionId: str=None) -> GetDatasetContentResponse:
        """Retrieves the contents of a data set as pre-signed URIs.

        :param self:
        :param versionId:  The version of the data set whose contents are retrieved. You can also use the strings "$LATEST" or "$LATEST_SUCCEEDED" to retrieve the contents of the latest or latest successfully completed data set. If not specified, "$LATEST_SUCCEEDED" is the default.<br/>datasetName:  The name of the data set whose contents are retrieved.<br/>
        :return: status:  The status of the data set content.:<br/>timestamp:  The time when the request was made.<br/>entries:  A list of "DatasetEntry" objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>reason</i> The reason the data set contents are in this state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>state</i> The state of the data set contents. Can be one of "READY", "CREATING", "SUCCEEDED" or "FAILED".<br/>        """
        pass

    def list_channels(self, nextToken: str=None, maxResults: int=None) -> ListChannelsResponse:
        """Retrieves a list of channels.

        :param self:
        :param maxResults:  The maximum number of results to return in this request. The default value is 100.<br/>nextToken:  The token for the next set of results.<br/>
        :return: nextToken:  The token to retrieve the next set of results, or <code>null</code> if there are no more results.<br/>channelSummaries:  A list of "ChannelSummary" objects.<br/>        """
        pass

    def list_dataset_contents(self, datasetName: str, nextToken: str=None, maxResults: int=None, scheduledOnOrAfter: Timestamp=None, scheduledBefore: Timestamp=None) -> ListDatasetContentsResponse:
        """Lists information about data set contents that have been created.

        :param self:
        :param scheduledBefore:  A filter to limit results to those data set contents whose creation is scheduled before the given time. See the field <code>triggers.schedule</code> in the CreateDataset request. (timestamp)<br/>scheduledOnOrAfter:  A filter to limit results to those data set contents whose creation is scheduled on or after the given time. See the field <code>triggers.schedule</code> in the CreateDataset request. (timestamp)<br/>maxResults:  The maximum number of results to return in this request.<br/>nextToken:  The token for the next set of results.<br/>datasetName:  The name of the data set whose contents information you want to list.<br/>
        :return: nextToken:  The token to retrieve the next set of results, or <code>null</code> if there are no more results.<br/>datasetContentSummaries:  Summary information about data set contents that have been created.<br/>        """
        pass

    def list_datasets(self, nextToken: str=None, maxResults: int=None) -> ListDatasetsResponse:
        """Retrieves information about data sets.

        :param self:
        :param maxResults:  The maximum number of results to return in this request. The default value is 100.<br/>nextToken:  The token for the next set of results.<br/>
        :return: nextToken:  The token to retrieve the next set of results, or <code>null</code> if there are no more results.<br/>datasetSummaries:  A list of "DatasetSummary" objects.<br/>        """
        pass

    def list_datastores(self, nextToken: str=None, maxResults: int=None) -> ListDatastoresResponse:
        """Retrieves a list of data stores.

        :param self:
        :param maxResults:  The maximum number of results to return in this request. The default value is 100.<br/>nextToken:  The token for the next set of results.<br/>
        :return: nextToken:  The token to retrieve the next set of results, or <code>null</code> if there are no more results.<br/>datastoreSummaries:  A list of "DatastoreSummary" objects.<br/>        """
        pass

    def list_pipelines(self, nextToken: str=None, maxResults: int=None) -> ListPipelinesResponse:
        """Retrieves a list of pipelines.

        :param self:
        :param maxResults:  The maximum number of results to return in this request. The default value is 100.<br/>nextToken:  The token for the next set of results.<br/>
        :return: nextToken:  The token to retrieve the next set of results, or <code>null</code> if there are no more results.<br/>pipelineSummaries:  A list of "PipelineSummary" objects.<br/>        """
        pass

    def list_tags_for_resource(self, resourceArn: str) -> ListTagsForResourceResponse:
        """Lists the tags (metadata) which you have assigned to the resource.

        :param self:
        :param resourceArn:  The ARN of the resource whose tags you want to list.<br/>
        :return: tags:  The tags (metadata) which you have assigned to the resource.<br/>        """
        pass

    def put_logging_options(self, loggingOptions: LoggingOptions):
        """Sets or updates the AWS IoT Analytics logging options. Note that if you update the value of any <code>loggingOptions</code> field, it takes up to one minute for the change to take effect. Also, if you change the policy attached to the role you specified in the roleArn field (for example, to correct an invalid policy) it takes up to 5 minutes for that change to take effect. 

        :param self:
        :param loggingOptions:  The new values of the AWS IoT Analytics logging options.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enabled</i> If true, logging is enabled for AWS IoT Analytics.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>level</i> The logging level. Currently, only "ERROR" is supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The ARN of the role that grants permission to AWS IoT Analytics to perform logging.<br/>
        :return:         """
        pass

    def run_pipeline_activity(self, payloads: List[MessagePayload], pipelineActivity: PipelineActivity) -> RunPipelineActivityResponse:
        """Simulates the results of running a pipeline activity on a message payload.

        :param self:
        :param payloads:  The sample message payloads on which the pipeline activity is run.<br/>pipelineActivity:  The pipeline activity that is run. This must not be a 'channel' activity or a 'datastore' activity because these activities are used in a pipeline only to load the original message and to store the (possibly) transformed message. If a 'lambda' activity is specified, only short-running Lambda functions (those with a timeout of less than 30 seconds or less) can be used.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deviceShadowEnrich</i> Adds information from the AWS IoT Device Shadows service to a message.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>deviceShadowEnrich</b>: Adds information from the AWS IoT Device Shadows service to a message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deviceRegistryEnrich</i> Adds data from the AWS IoT device registry to your message.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>deviceRegistryEnrich</b>: Adds data from the AWS IoT device registry to your message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>math</i> Computes an arithmetic expression using the message's attributes and adds it to the message.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>math</b>: Computes an arithmetic expression using the message's attributes and adds it to the message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>filter</i> Filters a message based on its attributes.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>filter</b>: Filters a message based on its attributes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>selectAttributes</i> Creates a new message using only the specified attributes from the original message. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>selectAttributes</b>: Creates a new message using only the specified attributes from the original message. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>removeAttributes</i> Removes attributes from a message.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>removeAttributes</b>: Removes attributes from a message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>addAttributes</i> Adds other attributes based on existing attributes in the message.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>addAttributes</b>: Adds other attributes based on existing attributes in the message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>datastore</i> Specifies where to store the processed message data.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>datastore</b>: Specifies where to store the processed message data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lambda</i> Runs a Lambda function to modify the message.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>lambda</b>: Runs a Lambda function to modify the message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>channel</i> Determines the source of the messages to be processed.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>channel</b>: Determines the source of the messages to be processed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>next</i> The next activity in the pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>channelName</i> The name of the channel from which the messages are processed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the 'channel' activity.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>next</i> The next activity in the pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>batchSize</i> The number of messages passed to the Lambda function for processing. The AWS Lambda function must be able to process all of these messages within five minutes, which is the maximum timeout duration for Lambda functions.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lambdaName</i> The name of the Lambda function that is run on the message.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the 'lambda' activity.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>datastoreName</i> The name of the data store where processed messages are stored.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the 'datastore' activity.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>next</i> The next activity in the pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>attributes</i> A list of 1-50 "AttributeNameMapping" objects that map an existing attribute to a new attribute. <note> The existing attributes remain in the message, so if you want to remove the originals, use "RemoveAttributeActivity". </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the 'addAttributes' activity.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>next</i> The next activity in the pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>attributes</i> A list of 1-50 attributes to remove from the message.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the 'removeAttributes' activity.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>next</i> The next activity in the pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>attributes</i> A list of the attributes to select from the message.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the 'selectAttributes' activity.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>next</i> The next activity in the pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>filter</i> An expression that looks like a SQL WHERE clause that must return a Boolean value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the 'filter' activity.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>next</i> The next activity in the pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>math</i> An expression that uses one or more existing attributes and must return an integer value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>attribute</i> The name of the attribute that contains the result of the math operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the 'math' activity.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>next</i> The next activity in the pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The ARN of the role that allows access to the device's registry information.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>thingName</i> The name of the IoT device whose registry information is added to the message.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>attribute</i> The name of the attribute that is added to the message.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the 'deviceRegistryEnrich' activity.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>next</i> The next activity in the pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The ARN of the role that allows access to the device's shadow.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>thingName</i> The name of the IoT device whose shadow information is added to the message.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>attribute</i> The name of the attribute that is added to the message.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the 'deviceShadowEnrich' activity.<br/>
        :return: logResult:  In case the pipeline activity fails, the log message that is generated.<br/>payloads:  The enriched or transformed sample message payloads as base64-encoded strings. (The results of running the pipeline activity on each input sample message payload, encoded in base64.)<br/>        """
        pass

    def sample_channel_data(self, channelName: str, maxMessages: int=None, startTime: StartTime=None, endTime: EndTime=None) -> SampleChannelDataResponse:
        """Retrieves a sample of messages from the specified channel ingested during the specified timeframe. Up to 10 messages can be retrieved.

        :param self:
        :param endTime:  The end of the time window from which sample messages are retrieved.<br/>startTime:  The start of the time window from which sample messages are retrieved.<br/>maxMessages:  The number of sample messages to be retrieved. The limit is 10, the default is also 10.<br/>channelName:  The name of the channel whose message samples are retrieved.<br/>
        :return: payloads:  The list of message samples. Each sample message is returned as a base64-encoded string.<br/>        """
        pass

    def start_pipeline_reprocessing(self, pipelineName: str, startTime: StartTime=None, endTime: EndTime=None) -> StartPipelineReprocessingResponse:
        """Starts the reprocessing of raw message data through the pipeline.

        :param self:
        :param endTime:  The end time (exclusive) of raw message data that is reprocessed.<br/>startTime:  The start time (inclusive) of raw message data that is reprocessed.<br/>pipelineName:  The name of the pipeline on which to start reprocessing.<br/>
        :return: reprocessingId:  The ID of the pipeline reprocessing activity that was started.<br/>        """
        pass

    def tag_resource(self, tags: List[Tag], resourceArn: str) -> TagResourceResponse:
        """Adds to or modifies the tags of the given resource. Tags are metadata which can be used to manage a resource.

        :param self:
        :param tags:  The new or modified tags for the resource.<br/>resourceArn:  The ARN of the resource whose tags you want to modify.<br/>
        :return:         """
        pass

    def untag_resource(self, tagKeys: List[TagKey], resourceArn: str) -> UntagResourceResponse:
        """Removes the given tags (metadata) from the resource.

        :param self:
        :param tagKeys:  The keys of those tags which you want to remove.<br/>resourceArn:  The ARN of the resource whose tags you want to remove.<br/>
        :return:         """
        pass

    def update_channel(self, channelName: str, channelStorage: ChannelStorage=None, retentionPeriod: RetentionPeriod=None):
        """Updates the settings of a channel.

        :param self:
        :param retentionPeriod:  How long, in days, message data is kept for the channel.:<br/>channelStorage:  Where channel data is stored.:<br/>channelName:  The name of the channel to be updated.<br/>Use this to store channel data in an S3 bucket managed by the AWS IoT Analytics service.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>serviceManagedS3</b>: Use this to store channel data in an S3 bucket managed by the AWS IoT Analytics service.&nbsp;&nbsp;&nbsp;&nbsp; <i>numberOfDays</i> The number of days that message data is kept. The "unlimited" parameter must be false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>unlimited</i> If true, message data is kept indefinitely.<br/>
        :return:         """
        pass

    def update_dataset(self, actions: List[DatasetAction], datasetName: str, triggers: List[DatasetTrigger]=None, contentDeliveryRules: List[DatasetContentDeliveryRule]=None, retentionPeriod: RetentionPeriod=None, versioningConfiguration: VersioningConfiguration=None):
        """Updates the settings of a data set.

        :param self:
        :param versioningConfiguration:  [Optional] How many versions of data set contents are kept. If not specified or set to null, only the latest version plus the latest succeeded version (if they are different) are kept for the time period specified by the "retentionPeriod" parameter. (For more information, see https://docs.aws.amazon.com/iotanalytics/latest/userguide/getting-started.html#aws-iot-analytics-dataset-versions):<br/>retentionPeriod:  How long, in days, data set contents are kept for the data set.:<br/>contentDeliveryRules:  When data set contents are created they are delivered to destinations specified here.<br/>triggers:  A list of "DatasetTrigger" objects. The list can be empty or can contain up to five <b>DataSetTrigger</b> objects.<br/>actions:  A list of "DatasetAction" objects.<br/>datasetName:  The name of the data set to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>numberOfDays</i> The number of days that message data is kept. The "unlimited" parameter must be false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>unlimited</i> If true, message data is kept indefinitely.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>maxVersions</i> How many versions of data set contents will be kept. The "unlimited" parameter must be false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>unlimited</i> If true, unlimited versions of data set contents will be kept.<br/>
        :return:         """
        pass

    def update_datastore(self, datastoreName: str, retentionPeriod: RetentionPeriod=None, datastoreStorage: DatastoreStorage=None):
        """Updates the settings of a data store.

        :param self:
        :param datastoreStorage:  Where data store data is stored.:<br/>retentionPeriod:  How long, in days, message data is kept for the data store.:<br/>datastoreName:  The name of the data store to be updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>numberOfDays</i> The number of days that message data is kept. The "unlimited" parameter must be false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>unlimited</i> If true, message data is kept indefinitely.<br/>Use this to store data store data in an S3 bucket managed by the AWS IoT Analytics service.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>serviceManagedS3</b>: Use this to store data store data in an S3 bucket managed by the AWS IoT Analytics service.
        :return:         """
        pass

    def update_pipeline(self, pipelineActivities: List[PipelineActivity], pipelineName: str):
        """Updates the settings of a pipeline. You must specify both a <code>channel</code> and a <code>datastore</code> activity and, optionally, as many as 23 additional activities in the <code>pipelineActivities</code> array.

        :param self:
        :param pipelineActivities:  A list of "PipelineActivity" objects. Activities perform transformations on your messages, such as removing, renaming or adding message attributes; filtering messages based on attribute values; invoking your Lambda functions on messages for advanced processing; or performing mathematical transformations to normalize device data. The list can be 2-25 <b>PipelineActivity</b> objects and must contain both a <code>channel</code> and a <code>datastore</code> activity. Each entry in the list must contain only one activity, for example:  <code>pipelineActivities = [ { "channel": { ... } }, { "lambda": { ... } }, ... ]</code> <br/>pipelineName:  The name of the pipeline to update.<br/>
        :return:         """
        pass



class IoTEvents(BaseClient):
    """AWS IoT Events monitors your equipment or device fleets for failures or changes in operation, and triggers actions when such events occur."""




    class CreateDetectorModelRequest(Mapping):
        pass
    
    class CreateDetectorModelResponse(Mapping):
        pass
    
    class DetectorModelDefinition(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreateInputRequest(Mapping):
        pass
    
    class CreateInputResponse(Mapping):
        pass
    
    class InputDefinition(Mapping):
        pass
    
    class DeleteDetectorModelRequest(Mapping):
        pass
    
    class DeleteDetectorModelResponse(Mapping):
        pass
    
    class DeleteInputRequest(Mapping):
        pass
    
    class DeleteInputResponse(Mapping):
        pass
    
    class DescribeDetectorModelRequest(Mapping):
        pass
    
    class DescribeDetectorModelResponse(Mapping):
        pass
    
    class DescribeInputRequest(Mapping):
        pass
    
    class DescribeInputResponse(Mapping):
        pass
    
    class DescribeLoggingOptionsRequest(Mapping):
        pass
    
    class DescribeLoggingOptionsResponse(Mapping):
        pass
    
    class ListDetectorModelVersionsRequest(Mapping):
        pass
    
    class ListDetectorModelVersionsResponse(Mapping):
        pass
    
    class ListDetectorModelsRequest(Mapping):
        pass
    
    class ListDetectorModelsResponse(Mapping):
        pass
    
    class ListInputsRequest(Mapping):
        pass
    
    class ListInputsResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class PutLoggingOptionsRequest(Mapping):
        pass
    
    class LoggingOptions(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateDetectorModelRequest(Mapping):
        pass
    
    class UpdateDetectorModelResponse(Mapping):
        pass
    
    class UpdateInputRequest(Mapping):
        pass
    
    class UpdateInputResponse(Mapping):
        pass
    
    def create_detector_model(self, roleArn: str, detectorModelDefinition: DetectorModelDefinition, detectorModelName: str, detectorModelDescription: str=None, key: str=None, tags: List[Tag]=None) -> CreateDetectorModelResponse:
        """Creates a detector model.

        :param self:
        :param tags:  Metadata which can be used to manage the detector model.<br/>roleArn:  The ARN of the role that grants permission to AWS IoT Events to perform its operations.<br/>key:  The input attribute key used to identify a device or system in order to create a detector (an instance of the detector model) and then to route each input received to the appropriate detector (instance). This parameter uses a JSON-path expression to specify the attribute-value pair in the message payload of each input that is used to identify the device associated with the input.<br/>detectorModelDescription:  A brief description of the detector model.<br/>detectorModelDefinition:  Information that defines how the detectors operate.:<br/>detectorModelName:  The name of the detector model.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>initialStateName</i> The state that is entered at the creation of each detector (instance).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>states</i> Information about the states of the detector.<br/>
        :return: detectorModelConfiguration:  Information about how the detector model is configured.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>key</i> The input attribute key used to identify a device or system in order to create a detector (an instance of the detector model) and then to route each input received to the appropriate detector (instance). This parameter uses a JSON-path expression to specify the attribute-value pair in the message payload of each input that is used to identify the device associated with the input.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the detector model.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdateTime</i> The time the detector model was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationTime</i> The time the detector model was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The ARN of the role that grants permission to AWS IoT Events to perform its operations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>detectorModelArn</i> The ARN of the detector model.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>detectorModelDescription</i> A brief description of the detector model.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>detectorModelVersion</i> The version of the detector model.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>detectorModelName</i> The name of the detector model.<br/>        """
        pass

    def create_input(self, inputDefinition: InputDefinition, inputName: str, inputDescription: str=None, tags: List[Tag]=None) -> CreateInputResponse:
        """Creates an input.

        :param self:
        :param tags:  Metadata which can be used to manage the input.<br/>inputDefinition:  The definition of the input.:<br/>inputDescription:  A brief description of the input.<br/>inputName:  The name you want to give to the input.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>attributes</i> The attributes from the JSON payload that are made available by the input. Inputs are derived from messages sent to the AWS IoT Events system using <code>BatchPutMessage</code>. Each such message contains a JSON payload, and those attributes (and their paired values) specified here is available for use in the <code>condition</code> expressions used by detectors that monitor this input. <br/>
        :return: inputConfiguration:  Information about the configuration of the input.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the input.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdateTime</i> The last time the input was updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationTime</i> The time the input was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>inputArn</i> The ARN of the input.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>inputDescription</i> A brief description of the input.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>inputName</i> The name of the input.<br/>        """
        pass

    def delete_detector_model(self, detectorModelName: str) -> DeleteDetectorModelResponse:
        """Deletes a detector model. Any active instances of the detector model are also deleted.

        :param self:
        :param detectorModelName:  The name of the detector model to be deleted.<br/>
        :return:         """
        pass

    def delete_input(self, inputName: str) -> DeleteInputResponse:
        """Deletes an input.

        :param self:
        :param inputName:  The name of the input to be deleted.<br/>
        :return:         """
        pass

    def describe_detector_model(self, detectorModelName: str, detectorModelVersion: str=None) -> DescribeDetectorModelResponse:
        """Describes a detector model. If the <code>version</code> parameter is not specified, information about the latest version is returned.

        :param self:
        :param detectorModelVersion:  The version of the detector model.<br/>detectorModelName:  The name of the detector model.<br/>
        :return: detectorModel:  Information about the detector model.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>detectorModelConfiguration</i> Information about how the detector is configured.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>detectorModelConfiguration</b>: Information about how the detector is configured.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>detectorModelDefinition</i> Information that defines how a detector operates.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>detectorModelDefinition</b>: Information that defines how a detector operates.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>initialStateName</i> The state that is entered at the creation of each detector (instance).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>states</i> Information about the states of the detector.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>key</i> The input attribute key used to identify a device or system in order to create a detector (an instance of the detector model) and then to route each input received to the appropriate detector (instance). This parameter uses a JSON-path expression to specify the attribute-value pair in the message payload of each input that is used to identify the device associated with the input.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the detector model.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdateTime</i> The time the detector model was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>creationTime</i> The time the detector model was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The ARN of the role that grants permission to AWS IoT Events to perform its operations.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>detectorModelArn</i> The ARN of the detector model.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>detectorModelDescription</i> A brief description of the detector model.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>detectorModelVersion</i> The version of the detector model.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>detectorModelName</i> The name of the detector model.<br/>        """
        pass

    def describe_input(self, inputName: str) -> DescribeInputResponse:
        """Describes an input.

        :param self:
        :param inputName:  The name of the input.<br/>
        :return: input:  Information about the input.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>inputDefinition</i> The definition of the input.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>inputDefinition</b>: The definition of the input.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>inputConfiguration</i> Information about the configuration of an input.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>inputConfiguration</b>: Information about the configuration of an input.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the input.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdateTime</i> The last time the input was updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>creationTime</i> The time the input was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>inputArn</i> The ARN of the input.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>inputDescription</i> A brief description of the input.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>inputName</i> The name of the input.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>attributes</i> The attributes from the JSON payload that are made available by the input. Inputs are derived from messages sent to the AWS IoT Events system using <code>BatchPutMessage</code>. Each such message contains a JSON payload, and those attributes (and their paired values) specified here is available for use in the <code>condition</code> expressions used by detectors that monitor this input. <br/>        """
        pass

    def describe_logging_options(self) -> DescribeLoggingOptionsResponse:
        """Retrieves the current settings of the AWS IoT Events logging options.

        :param self:
        :param 
        :return: loggingOptions:  The current settings of the AWS IoT Events logging options.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>detectorDebugOptions</i> Information that identifies those detector models and their detectors (instances) for which the logging level is given.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enabled</i> If TRUE, logging is enabled for AWS IoT Events.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>level</i> The logging level. Currently, only "ERROR" is supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The ARN of the role that grants permission to AWS IoT Events to perform logging.<br/>        """
        pass

    def list_detector_model_versions(self, detectorModelName: str, nextToken: str=None, maxResults: int=None) -> ListDetectorModelVersionsResponse:
        """Lists all the versions of a detector model. Only the metadata associated with each detector model version is returned.

        :param self:
        :param maxResults:  The maximum number of results to return at one time.<br/>nextToken:  The token for the next set of results.<br/>detectorModelName:  The name of the detector model whose versions are returned.<br/>
        :return: nextToken:  A token to retrieve the next set of results, or <code>null</code> if there are no additional results.<br/>detectorModelVersionSummaries:  Summary information about the detector model versions.<br/>        """
        pass

    def list_detector_models(self, nextToken: str=None, maxResults: int=None) -> ListDetectorModelsResponse:
        """Lists the detector models you have created. Only the metadata associated with each detector model is returned.

        :param self:
        :param maxResults:  The maximum number of results to return at one time.<br/>nextToken:  The token for the next set of results.<br/>
        :return: nextToken:  A token to retrieve the next set of results, or <code>null</code> if there are no additional results.<br/>detectorModelSummaries:  Summary information about the detector models.<br/>        """
        pass

    def list_inputs(self, nextToken: str=None, maxResults: int=None) -> ListInputsResponse:
        """Lists the inputs you have created.

        :param self:
        :param maxResults:  The maximum number of results to return at one time.<br/>nextToken:  The token for the next set of results.<br/>
        :return: nextToken:  A token to retrieve the next set of results, or <code>null</code> if there are no additional results.<br/>inputSummaries:  Summary information about the inputs.<br/>        """
        pass

    def list_tags_for_resource(self, resourceArn: str) -> ListTagsForResourceResponse:
        """Lists the tags (metadata) you have assigned to the resource.

        :param self:
        :param resourceArn:  The ARN of the resource.<br/>
        :return: tags:  The list of tags assigned to the resource.<br/>        """
        pass

    def put_logging_options(self, loggingOptions: LoggingOptions):
        """Sets or updates the AWS IoT Events logging options. Note that if you update the value of any <code>loggingOptions</code> field, it takes up to one minute for the change to take effect. Also, if you change the policy attached to the role you specified in the roleArn field (for example, to correct an invalid policy) it takes up to five minutes for that change to take effect.

        :param self:
        :param loggingOptions:  The new values of the AWS IoT Events logging options.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>detectorDebugOptions</i> Information that identifies those detector models and their detectors (instances) for which the logging level is given.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>enabled</i> If TRUE, logging is enabled for AWS IoT Events.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>level</i> The logging level. Currently, only "ERROR" is supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The ARN of the role that grants permission to AWS IoT Events to perform logging.<br/>
        :return:         """
        pass

    def tag_resource(self, tags: List[Tag], resourceArn: str) -> TagResourceResponse:
        """Add to or modifies the tags of the given resource. Tags are metadata which can be used to manage a resource.

        :param self:
        :param tags:  The new or modified tags for the resource.<br/>resourceArn:  The ARN of the resource.<br/>
        :return:         """
        pass

    def untag_resource(self, tagKeys: List[TagKey], resourceArn: str) -> UntagResourceResponse:
        """Removes the given tags (metadata) from the resource.

        :param self:
        :param tagKeys:  A list of the keys of the tags to be removed from the resource.<br/>resourceArn:  The ARN of the resource.<br/>
        :return:         """
        pass

    def update_detector_model(self, roleArn: str, detectorModelDefinition: DetectorModelDefinition, detectorModelName: str, detectorModelDescription: str=None) -> UpdateDetectorModelResponse:
        """Updates a detector model. Detectors (instances) spawned by the previous version will be deleted and re-created as new inputs arrive.

        :param self:
        :param roleArn:  The ARN of the role that grants permission to AWS IoT Events to perform its operations.<br/>detectorModelDescription:  A brief description of the detector model.<br/>detectorModelDefinition:  Information that defines how a detector operates.:<br/>detectorModelName:  The name of the detector model to be updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>initialStateName</i> The state that is entered at the creation of each detector (instance).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>states</i> Information about the states of the detector.<br/>
        :return: detectorModelConfiguration:  Information about how the detector model is configured.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>key</i> The input attribute key used to identify a device or system in order to create a detector (an instance of the detector model) and then to route each input received to the appropriate detector (instance). This parameter uses a JSON-path expression to specify the attribute-value pair in the message payload of each input that is used to identify the device associated with the input.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the detector model.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdateTime</i> The time the detector model was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationTime</i> The time the detector model was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The ARN of the role that grants permission to AWS IoT Events to perform its operations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>detectorModelArn</i> The ARN of the detector model.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>detectorModelDescription</i> A brief description of the detector model.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>detectorModelVersion</i> The version of the detector model.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>detectorModelName</i> The name of the detector model.<br/>        """
        pass

    def update_input(self, inputDefinition: InputDefinition, inputName: str, inputDescription: str=None) -> UpdateInputResponse:
        """Updates an input.

        :param self:
        :param inputDefinition:  The definition of the input.:<br/>inputDescription:  A brief description of the input.<br/>inputName:  The name of the input you want to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>attributes</i> The attributes from the JSON payload that are made available by the input. Inputs are derived from messages sent to the AWS IoT Events system using <code>BatchPutMessage</code>. Each such message contains a JSON payload, and those attributes (and their paired values) specified here is available for use in the <code>condition</code> expressions used by detectors that monitor this input. <br/>
        :return: inputConfiguration:  Information about the configuration of the input.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the input.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdateTime</i> The last time the input was updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationTime</i> The time the input was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>inputArn</i> The ARN of the input.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>inputDescription</i> A brief description of the input.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>inputName</i> The name of the input.<br/>        """
        pass



class IoTEventsData(BaseClient):
    """AWS IoT Events monitors your equipment or device fleets for failures or changes in operation, and triggers actions when such events occur. AWS IoT Events Data API commands enable you to send inputs to detectors, list detectors, and view or update a detector's status."""




    class BatchPutMessageRequest(Mapping):
        pass
    
    class BatchPutMessageResponse(Mapping):
        pass
    
    class Message(Mapping):
        pass
    
    class BatchUpdateDetectorRequest(Mapping):
        pass
    
    class BatchUpdateDetectorResponse(Mapping):
        pass
    
    class UpdateDetectorRequest(Mapping):
        pass
    
    class DescribeDetectorRequest(Mapping):
        pass
    
    class DescribeDetectorResponse(Mapping):
        pass
    
    class ListDetectorsRequest(Mapping):
        pass
    
    class ListDetectorsResponse(Mapping):
        pass
    
    def batch_put_message(self, messages: List[Message]) -> BatchPutMessageResponse:
        """Sends a set of messages to the AWS IoT Events system. Each message payload is transformed into the input you specify (<code>"inputName"</code>) and ingested into any detectors that monitor that input. If multiple messages are sent, the order in which the messages are processed isn't guaranteed. To guarantee ordering, you must send messages one at a time and wait for a successful response.

        :param self:
        :param messages:  The list of messages to send. Each message has the following format: <code>'{ "messageId": "string", "inputName": "string", "payload": "string"}'</code> <br/>
        :return: BatchPutMessageErrorEntries:  A list of any errors encountered when sending the messages.<br/>        """
        pass

    def batch_update_detector(self, detectors: List[UpdateDetectorRequest]) -> BatchUpdateDetectorResponse:
        """Updates the state, variable values, and timer settings of one or more detectors (instances) of a specified detector model.

        :param self:
        :param detectors:  The list of detectors (instances) to update, along with the values to update.<br/>
        :return: batchUpdateDetectorErrorEntries:  A list of those detector updates that resulted in errors. (If an error is listed here, the specific update did not occur.)<br/>        """
        pass

    def describe_detector(self, detectorModelName: str, keyValue: str=None) -> DescribeDetectorResponse:
        """Returns information about the specified detector (instance).

        :param self:
        :param keyValue:  A filter used to limit results to detectors (instances) created because of the given key ID.<br/>detectorModelName:  The name of the detector model whose detectors (instances) you want information about.<br/>
        :return: detector:  Information about the detector (instance).:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdateTime</i> The time the detector (instance) was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationTime</i> The time the detector (instance) was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>state</i> The current state of the detector (instance).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>state</b>: The current state of the detector (instance).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>detectorModelVersion</i> The version of the detector model that created this detector (instance).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>keyValue</i> The value of the key (identifying the device or system) that caused the creation of this detector (instance).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>detectorModelName</i> The name of the detector model that created this detector (instance).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>timers</i> The current state of the detector's timers.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>variables</i> The current values of the detector's variables.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>stateName</i> The name of the state.<br/>        """
        pass

    def list_detectors(self, detectorModelName: str, stateName: str=None, nextToken: str=None, maxResults: int=None) -> ListDetectorsResponse:
        """Lists detectors (the instances of a detector model).

        :param self:
        :param maxResults:  The maximum number of results to return at one time.<br/>nextToken:  The token for the next set of results.<br/>stateName:  A filter that limits results to those detectors (instances) in the given state.<br/>detectorModelName:  The name of the detector model whose detectors (instances) are listed.<br/>
        :return: nextToken:  A token to retrieve the next set of results, or <code>null</code> if there are no additional results.<br/>detectorSummaries:  A list of summary information about the detectors (instances).<br/>        """
        pass



class IoTThingsGraph(BaseClient):
    """<fullname>AWS IoT Things Graph</fullname> AWS IoT Things Graph provides an integrated set of tools that enable developers to connect devices and services that use different standards, such as units of measure and communication protocols. AWS IoT Things Graph makes it possible to build IoT applications with little to no code by connecting devices and services and defining how they interact at an abstract level. For more information about how AWS IoT Things Graph works, see the <a href="https://docs.aws.amazon.com/thingsgraph/latest/ug/iot-tg-whatis.html">User Guide</a>."""


    class GetFlowTemplateRevisionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_flow_template_revisions` for valid parameters"""
            pass
    class GetSystemTemplateRevisionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_system_template_revisions` for valid parameters"""
            pass
    class ListFlowExecutionMessagesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_flow_execution_messages` for valid parameters"""
            pass
    class ListTagsForResourcePaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_tags_for_resource` for valid parameters"""
            pass
    class SearchEntitiesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `search_entities` for valid parameters"""
            pass
    class SearchFlowExecutionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `search_flow_executions` for valid parameters"""
            pass
    class SearchFlowTemplatesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `search_flow_templates` for valid parameters"""
            pass
    class SearchSystemInstancesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `search_system_instances` for valid parameters"""
            pass
    class SearchSystemTemplatesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `search_system_templates` for valid parameters"""
            pass
    class SearchThingsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `search_things` for valid parameters"""
            pass


    class AssociateEntityToThingRequest(Mapping):
        pass
    
    class AssociateEntityToThingResponse(Mapping):
        pass
    
    class Version(object):
        pass
    
    class CreateFlowTemplateRequest(Mapping):
        pass
    
    class CreateFlowTemplateResponse(Mapping):
        pass
    
    class DefinitionDocument(Mapping):
        pass
    
    class CreateSystemInstanceRequest(Mapping):
        pass
    
    class CreateSystemInstanceResponse(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class MetricsConfiguration(Mapping):
        pass
    
    class CreateSystemTemplateRequest(Mapping):
        pass
    
    class CreateSystemTemplateResponse(Mapping):
        pass
    
    class DeleteFlowTemplateRequest(Mapping):
        pass
    
    class DeleteFlowTemplateResponse(Mapping):
        pass
    
    class DeleteNamespaceRequest(Mapping):
        pass
    
    class DeleteNamespaceResponse(Mapping):
        pass
    
    class DeleteSystemInstanceRequest(Mapping):
        pass
    
    class DeleteSystemInstanceResponse(Mapping):
        pass
    
    class DeleteSystemTemplateRequest(Mapping):
        pass
    
    class DeleteSystemTemplateResponse(Mapping):
        pass
    
    class DeploySystemInstanceRequest(Mapping):
        pass
    
    class DeploySystemInstanceResponse(Mapping):
        pass
    
    class DeprecateFlowTemplateRequest(Mapping):
        pass
    
    class DeprecateFlowTemplateResponse(Mapping):
        pass
    
    class DeprecateSystemTemplateRequest(Mapping):
        pass
    
    class DeprecateSystemTemplateResponse(Mapping):
        pass
    
    class DescribeNamespaceRequest(Mapping):
        pass
    
    class DescribeNamespaceResponse(Mapping):
        pass
    
    class DissociateEntityFromThingRequest(Mapping):
        pass
    
    class DissociateEntityFromThingResponse(Mapping):
        pass
    
    class GetEntitiesRequest(Mapping):
        pass
    
    class GetEntitiesResponse(Mapping):
        pass
    
    class Urn(object):
        pass
    
    class GetFlowTemplateRequest(Mapping):
        pass
    
    class GetFlowTemplateResponse(Mapping):
        pass
    
    class GetFlowTemplateRevisionsRequest(Mapping):
        pass
    
    class GetFlowTemplateRevisionsResponse(Mapping):
        pass
    
    class GetNamespaceDeletionStatusRequest(Mapping):
        pass
    
    class GetNamespaceDeletionStatusResponse(Mapping):
        pass
    
    class GetSystemInstanceRequest(Mapping):
        pass
    
    class GetSystemInstanceResponse(Mapping):
        pass
    
    class GetSystemTemplateRequest(Mapping):
        pass
    
    class GetSystemTemplateResponse(Mapping):
        pass
    
    class GetSystemTemplateRevisionsRequest(Mapping):
        pass
    
    class GetSystemTemplateRevisionsResponse(Mapping):
        pass
    
    class GetUploadStatusRequest(Mapping):
        pass
    
    class GetUploadStatusResponse(Mapping):
        pass
    
    class ListFlowExecutionMessagesRequest(Mapping):
        pass
    
    class ListFlowExecutionMessagesResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class SearchEntitiesRequest(Mapping):
        pass
    
    class SearchEntitiesResponse(Mapping):
        pass
    
    class EntityType(object):
        pass
    
    class EntityFilter(Mapping):
        pass
    
    class SearchFlowExecutionsRequest(Mapping):
        pass
    
    class SearchFlowExecutionsResponse(Mapping):
        pass
    
    class Timestamp(object):
        pass
    
    class SearchFlowTemplatesRequest(Mapping):
        pass
    
    class SearchFlowTemplatesResponse(Mapping):
        pass
    
    class FlowTemplateFilter(Mapping):
        pass
    
    class SearchSystemInstancesRequest(Mapping):
        pass
    
    class SearchSystemInstancesResponse(Mapping):
        pass
    
    class SystemInstanceFilter(Mapping):
        pass
    
    class SearchSystemTemplatesRequest(Mapping):
        pass
    
    class SearchSystemTemplatesResponse(Mapping):
        pass
    
    class SystemTemplateFilter(Mapping):
        pass
    
    class SearchThingsRequest(Mapping):
        pass
    
    class SearchThingsResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UndeploySystemInstanceRequest(Mapping):
        pass
    
    class UndeploySystemInstanceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateFlowTemplateRequest(Mapping):
        pass
    
    class UpdateFlowTemplateResponse(Mapping):
        pass
    
    class UpdateSystemTemplateRequest(Mapping):
        pass
    
    class UpdateSystemTemplateResponse(Mapping):
        pass
    
    class UploadEntityDefinitionsRequest(Mapping):
        pass
    
    class UploadEntityDefinitionsResponse(Mapping):
        pass
    
    class SyncWithPublicNamespace(object):
        pass
    
    class DeprecateExistingEntities(object):
        pass
    
    def associate_entity_to_thing(self, entityId: str, thingName: str, namespaceVersion: Version=None) -> AssociateEntityToThingResponse:
        """Associates a device with a concrete thing that is in the user's registry. A thing can be associated with only one device at a time. If you associate a thing with a new device id, its previous association will be removed.

        :param self:
        :param namespaceVersion:  The version of the user's namespace. Defaults to the latest version of the user's namespace.<br/>entityId:  The ID of the device to be associated with the thing. The ID should be in the following format.  <code>urn:tdm:REGION/ACCOUNT ID/default:device:DEVICENAME</code> <br/>thingName:  The name of the thing to which the entity is to be associated.<br/>
        :return:         """
        pass

    def create_flow_template(self, definition: DefinitionDocument, compatibleNamespaceVersion: Version=None) -> CreateFlowTemplateResponse:
        """Creates a workflow template. Workflows can be created only in the user's namespace. (The public namespace contains only entities.) The workflow can contain only entities in the specified namespace. The workflow is validated against the entities in the latest version of the user's namespace unless another namespace version is specified in the request.

        :param self:
        :param compatibleNamespaceVersion:  The namespace version in which the workflow is to be created. If no value is specified, the latest version is used by default.<br/>definition:  The workflow <code>DefinitionDocument</code>.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>text</i> The GraphQL text that defines the entity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>language</i> The language used to define the entity. <code>GRAPHQL</code> is the only valid value.<br/>
        :return: summary:  The summary object that describes the created workflow.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The date when the workflow was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>revisionNumber</i> The revision number of the workflow.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The ARN of the workflow.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the workflow.<br/>        """
        pass

    def create_system_instance(self, target: str, definition: DefinitionDocument, tags: List[Tag]=None, greengrassGroupName: str=None, s3BucketName: str=None, metricsConfiguration: MetricsConfiguration=None, flowActionsRoleArn: str=None) -> CreateSystemInstanceResponse:
        """Creates a system instance.  This action validates the system instance, prepares the deployment-related resources. For Greengrass deployments, it updates the Greengrass group that is specified by the <code>greengrassGroupName</code> parameter. It also adds a file to the S3 bucket specified by the <code>s3BucketName</code> parameter. You need to call <code>DeploySystemInstance</code> after running this action. For Greengrass deployments, since this action modifies and adds resources to a Greengrass group and an S3 bucket on the caller's behalf, the calling identity must have write permissions to both the specified Greengrass group and S3 bucket. Otherwise, the call will fail with an authorization error. For cloud deployments, this action requires a <code>flowActionsRoleArn</code> value. This is an IAM role that has permissions to access AWS services, such as AWS Lambda and AWS IoT, that the flow uses when it executes. If the definition document doesn't specify a version of the user's namespace, the latest version will be used by default.

        :param self:
        :param flowActionsRoleArn:  The ARN of the IAM role that AWS IoT Things Graph will assume when it executes the flow. This role must have read and write access to AWS Lambda and AWS IoT and any other AWS services that the flow uses when it executes. This value is required if the value of the <code>target</code> parameter is <code>CLOUD</code>.<br/>metricsConfiguration:  An object that specifies whether cloud metrics are collected in a deployment and, if so, what role is used to collect metrics.:<br/>s3BucketName:  The name of the Amazon Simple Storage Service bucket that will be used to store and deploy the system instance's resource file. This value is required if the value of the <code>target</code> parameter is <code>GREENGRASS</code>.<br/>greengrassGroupName:  The name of the Greengrass group where the system instance will be deployed. This value is required if the value of the <code>target</code> parameter is <code>GREENGRASS</code>.<br/>target:  The target type of the deployment. Valid values are <code>GREENGRASS</code> and <code>CLOUD</code>.<br/>definition:  A document that defines an entity. :<br/>tags:  Metadata, consisting of key-value pairs, that can be used to categorize your system instances.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>text</i> The GraphQL text that defines the entity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>language</i> The language used to define the entity. <code>GRAPHQL</code> is the only valid value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>metricRuleRoleArn</i> The ARN of the role that is used to collect cloud metrics.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cloudMetricEnabled</i> A Boolean that specifies whether cloud metrics are collected.<br/>
        :return: summary:  The summary object that describes the new system instance.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>greengrassGroupVersionId</i> The version of the Greengrass group where the system instance is deployed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>greengrassGroupId</i> The ID of the Greengrass group where the system instance is deployed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>updatedAt</i>  The date and time when the system instance was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The date when the system instance was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>greengrassGroupName</i> The ID of the Greengrass group where the system instance is deployed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>target</i> The target of the system instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the system instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The ARN of the system instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the system instance.<br/>        """
        pass

    def create_system_template(self, definition: DefinitionDocument, compatibleNamespaceVersion: Version=None) -> CreateSystemTemplateResponse:
        """Creates a system. The system is validated against the entities in the latest version of the user's namespace unless another namespace version is specified in the request.

        :param self:
        :param compatibleNamespaceVersion:  The namespace version in which the system is to be created. If no value is specified, the latest version is used by default.<br/>definition:  The <code>DefinitionDocument</code> used to create the system.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>text</i> The GraphQL text that defines the entity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>language</i> The language used to define the entity. <code>GRAPHQL</code> is the only valid value.<br/>
        :return: summary:  The summary object that describes the created system.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The date when the system was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>revisionNumber</i> The revision number of the system.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The ARN of the system.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the system.<br/>        """
        pass

    def delete_flow_template(self, id: str) -> DeleteFlowTemplateResponse:
        """Deletes a workflow. Any new system or deployment that contains this workflow will fail to update or deploy. Existing deployments that contain the workflow will continue to run (since they use a snapshot of the workflow taken at the time of deployment).

        :param self:
        :param id:  The ID of the workflow to be deleted. The ID should be in the following format.  <code>urn:tdm:REGION/ACCOUNT ID/default:workflow:WORKFLOWNAME</code> <br/>
        :return:         """
        pass

    def delete_namespace(self) -> DeleteNamespaceResponse:
        """Deletes the specified namespace. This action deletes all of the entities in the namespace. Delete the systems and flows that use entities in the namespace before performing this action.

        :param self:
        :param 
        :return: namespaceName:  The name of the namespace to be deleted.<br/>namespaceArn:  The ARN of the namespace to be deleted.<br/>        """
        pass

    def delete_system_instance(self, id: str=None) -> DeleteSystemInstanceResponse:
        """Deletes a system instance. Only system instances that have never been deployed, or that have been undeployed can be deleted. Users can create a new system instance that has the same ID as a deleted system instance.

        :param self:
        :param id:  The ID of the system instance to be deleted.<br/>
        :return:         """
        pass

    def delete_system_template(self, id: str) -> DeleteSystemTemplateResponse:
        """Deletes a system. New deployments can't contain the system after its deletion. Existing deployments that contain the system will continue to work because they use a snapshot of the system that is taken when it is deployed.

        :param self:
        :param id:  The ID of the system to be deleted. The ID should be in the following format.  <code>urn:tdm:REGION/ACCOUNT ID/default:system:SYSTEMNAME</code> <br/>
        :return:         """
        pass

    def deploy_system_instance(self, id: str=None) -> DeploySystemInstanceResponse:
        """ <b>Greengrass and Cloud Deployments</b>  Deploys the system instance to the target specified in <code>CreateSystemInstance</code>.   <b>Greengrass Deployments</b>  If the system or any workflows and entities have been updated before this action is called, then the deployment will create a new Amazon Simple Storage Service resource file and then deploy it. Since this action creates a Greengrass deployment on the caller's behalf, the calling identity must have write permissions to the specified Greengrass group. Otherwise, the call will fail with an authorization error. For information about the artifacts that get added to your Greengrass core device when you use this API, see <a href="https://docs.aws.amazon.com/thingsgraph/latest/ug/iot-tg-greengrass.html">AWS IoT Things Graph and AWS IoT Greengrass</a>.

        :param self:
        :param id:  The ID of the system instance. This value is returned by the <code>CreateSystemInstance</code> action. The ID should be in the following format.  <code>urn:tdm:REGION/ACCOUNT ID/default:deployment:DEPLOYMENTNAME</code> <br/>
        :return: greengrassDeploymentId:  The ID of the Greengrass deployment used to deploy the system instance.<br/>summary:  An object that contains summary information about a system instance that was deployed. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>greengrassGroupVersionId</i> The version of the Greengrass group where the system instance is deployed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>greengrassGroupId</i> The ID of the Greengrass group where the system instance is deployed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>updatedAt</i>  The date and time when the system instance was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The date when the system instance was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>greengrassGroupName</i> The ID of the Greengrass group where the system instance is deployed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>target</i> The target of the system instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the system instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The ARN of the system instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the system instance.<br/>        """
        pass

    def deprecate_flow_template(self, id: str) -> DeprecateFlowTemplateResponse:
        """Deprecates the specified workflow. This action marks the workflow for deletion. Deprecated flows can't be deployed, but existing deployments will continue to run.

        :param self:
        :param id:  The ID of the workflow to be deleted. The ID should be in the following format.  <code>urn:tdm:REGION/ACCOUNT ID/default:workflow:WORKFLOWNAME</code> <br/>
        :return:         """
        pass

    def deprecate_system_template(self, id: str) -> DeprecateSystemTemplateResponse:
        """Deprecates the specified system.

        :param self:
        :param id:  The ID of the system to delete. The ID should be in the following format.  <code>urn:tdm:REGION/ACCOUNT ID/default:system:SYSTEMNAME</code> <br/>
        :return:         """
        pass

    def describe_namespace(self, namespaceName: str=None) -> DescribeNamespaceResponse:
        """Gets the latest version of the user's namespace and the public version that it is tracking.

        :param self:
        :param namespaceName:  The name of the user's namespace. Set this to <code>aws</code> to get the public namespace.<br/>
        :return: namespaceVersion:  The version of the user's namespace to describe.<br/>trackingNamespaceVersion:  The version of the public namespace that the latest version is tracking.<br/>trackingNamespaceName:  The name of the public namespace that the latest namespace version is tracking.<br/>namespaceName:  The name of the namespace.<br/>namespaceArn:  The ARN of the namespace.<br/>        """
        pass

    def dissociate_entity_from_thing(self, entityType: str, thingName: str) -> DissociateEntityFromThingResponse:
        """Dissociates a device entity from a concrete thing. The action takes only the type of the entity that you need to dissociate because only one entity of a particular type can be associated with a thing.

        :param self:
        :param entityType:  The entity type from which to disassociate the thing.<br/>thingName:  The name of the thing to disassociate.<br/>
        :return:         """
        pass

    def get_entities(self, ids: List[Urn], namespaceVersion: Version=None) -> GetEntitiesResponse:
        """Gets definitions of the specified entities. Uses the latest version of the user's namespace by default. This API returns the following TDM entities. <ul> <li> Properties </li> <li> States </li> <li> Events </li> <li> Actions </li> <li> Capabilities </li> <li> Mappings </li> <li> Devices </li> <li> Device Models </li> <li> Services </li> </ul> This action doesn't return definitions for systems, flows, and deployments.

        :param self:
        :param namespaceVersion:  The version of the user's namespace. Defaults to the latest version of the user's namespace.<br/>ids:  An array of entity IDs. The IDs should be in the following format.  <code>urn:tdm:REGION/ACCOUNT ID/default:device:DEVICENAME</code> <br/>
        :return: descriptions:  An array of descriptions for the specified entities.<br/>        """
        pass

    def get_flow_template(self, id: str, revisionNumber: Version=None) -> GetFlowTemplateResponse:
        """Gets the latest version of the <code>DefinitionDocument</code> and <code>FlowTemplateSummary</code> for the specified workflow.

        :param self:
        :param revisionNumber:  The number of the workflow revision to retrieve.<br/>id:  The ID of the workflow. The ID should be in the following format.  <code>urn:tdm:REGION/ACCOUNT ID/default:workflow:WORKFLOWNAME</code> <br/>
        :return: description:  The object that describes the specified workflow.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>validatedNamespaceVersion</i> The version of the user's namespace against which the workflow was validated. Use this value in your system instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>definition</i> A workflow's definition document.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>definition</b>: A workflow's definition document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>summary</i> An object that contains summary information about a workflow.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>summary</b>: An object that contains summary information about a workflow.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The date when the workflow was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>revisionNumber</i> The revision number of the workflow.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The ARN of the workflow.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the workflow.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>text</i> The GraphQL text that defines the entity.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>language</i> The language used to define the entity. <code>GRAPHQL</code> is the only valid value.<br/>        """
        pass

    def get_flow_template_revisions(self, id: str, nextToken: str=None, maxResults: int=None) -> GetFlowTemplateRevisionsResponse:
        """Gets revisions of the specified workflow. Only the last 100 revisions are stored. If the workflow has been deprecated, this action will return revisions that occurred before the deprecation. This action won't work for workflows that have been deleted.

        :param self:
        :param maxResults:  The maximum number of results to return in the response.<br/>nextToken:  The string that specifies the next page of results. Use this when you're paginating results.<br/>id:  The ID of the workflow. The ID should be in the following format.  <code>urn:tdm:REGION/ACCOUNT ID/default:workflow:WORKFLOWNAME</code> <br/>
        :return: nextToken:  The string to specify as <code>nextToken</code> when you request the next page of results.<br/>summaries:  An array of objects that provide summary data about each revision.<br/>        """
        pass

    def get_namespace_deletion_status(self) -> GetNamespaceDeletionStatusResponse:
        """Gets the status of a namespace deletion task.

        :param self:
        :param 
        :return: errorMessage:  An error code returned by the namespace deletion task.<br/>errorCode:  An error code returned by the namespace deletion task.<br/>status:  The status of the deletion request.<br/>namespaceName:  The name of the namespace that is being deleted.<br/>namespaceArn:  The ARN of the namespace that is being deleted.<br/>        """
        pass

    def get_system_instance(self, id: str) -> GetSystemInstanceResponse:
        """Gets a system instance.

        :param self:
        :param id:  The ID of the system deployment instance. This value is returned by <code>CreateSystemInstance</code>. The ID should be in the following format.  <code>urn:tdm:REGION/ACCOUNT ID/default:deployment:DEPLOYMENTNAME</code> <br/>
        :return: description:  An object that describes the system instance.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>flowActionsRoleArn</i> The AWS Identity and Access Management (IAM) role that AWS IoT Things Graph assumes during flow execution in a cloud deployment. This role must have read and write permissionss to AWS Lambda and AWS IoT and to any other AWS services that the flow uses.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>validatedDependencyRevisions</i> A list of objects that contain all of the IDs and revision numbers of workflows and systems that are used in a system instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>validatedNamespaceVersion</i> The version of the user's namespace against which the system instance was validated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>metricsConfiguration</i> An object that specifies whether cloud metrics are collected in a deployment and, if so, what role is used to collect metrics.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>metricsConfiguration</b>: An object that specifies whether cloud metrics are collected in a deployment and, if so, what role is used to collect metrics.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>s3BucketName</i> The Amazon Simple Storage Service bucket where information about a system instance is stored.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>definition</i> A document that defines an entity. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>definition</b>: A document that defines an entity. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>summary</i> An object that contains summary information about a system instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>summary</b>: An object that contains summary information about a system instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>greengrassGroupVersionId</i> The version of the Greengrass group where the system instance is deployed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>greengrassGroupId</i> The ID of the Greengrass group where the system instance is deployed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>updatedAt</i>  The date and time when the system instance was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The date when the system instance was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>greengrassGroupName</i> The ID of the Greengrass group where the system instance is deployed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>target</i> The target of the system instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the system instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The ARN of the system instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the system instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>text</i> The GraphQL text that defines the entity.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>language</i> The language used to define the entity. <code>GRAPHQL</code> is the only valid value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>metricRuleRoleArn</i> The ARN of the role that is used to collect cloud metrics.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>cloudMetricEnabled</i> A Boolean that specifies whether cloud metrics are collected.<br/>        """
        pass

    def get_system_template(self, id: str, revisionNumber: Version=None) -> GetSystemTemplateResponse:
        """Gets a system.

        :param self:
        :param revisionNumber:  The number that specifies the revision of the system to get.<br/>id:  The ID of the system to get. This ID must be in the user's namespace. The ID should be in the following format.  <code>urn:tdm:REGION/ACCOUNT ID/default:system:SYSTEMNAME</code> <br/>
        :return: description:  An object that contains summary data about the system.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>validatedNamespaceVersion</i> The namespace version against which the system was validated. Use this value in your system instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>definition</i> The definition document of a system.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>definition</b>: The definition document of a system.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>summary</i> An object that contains summary information about a system.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>summary</b>: An object that contains summary information about a system.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The date when the system was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>revisionNumber</i> The revision number of the system.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The ARN of the system.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the system.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>text</i> The GraphQL text that defines the entity.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>language</i> The language used to define the entity. <code>GRAPHQL</code> is the only valid value.<br/>        """
        pass

    def get_system_template_revisions(self, id: str, nextToken: str=None, maxResults: int=None) -> GetSystemTemplateRevisionsResponse:
        """Gets revisions made to the specified system template. Only the previous 100 revisions are stored. If the system has been deprecated, this action will return the revisions that occurred before its deprecation. This action won't work with systems that have been deleted.

        :param self:
        :param maxResults:  The maximum number of results to return in the response.<br/>nextToken:  The string that specifies the next page of results. Use this when you're paginating results.<br/>id:  The ID of the system template. The ID should be in the following format.  <code>urn:tdm:REGION/ACCOUNT ID/default:system:SYSTEMNAME</code> <br/>
        :return: nextToken:  The string to specify as <code>nextToken</code> when you request the next page of results. <br/>summaries:  An array of objects that contain summary data about the system template revisions.<br/>        """
        pass

    def get_upload_status(self, uploadId: str) -> GetUploadStatusResponse:
        """Gets the status of the specified upload.

        :param self:
        :param uploadId:  The ID of the upload. This value is returned by the <code>UploadEntityDefinitions</code> action.<br/>
        :return: createdDate:  The date at which the upload was created.<br/>failureReason:  The reason for an upload failure.<br/>namespaceVersion:  The version of the user's namespace. Defaults to the latest version of the user's namespace.<br/>namespaceName:  The name of the upload's namespace.<br/>namespaceArn:  The ARN of the upload.<br/>uploadStatus:  The status of the upload. The initial status is <code>IN_PROGRESS</code>. The response show all validation failures if the upload fails.<br/>uploadId:  The ID of the upload.<br/>        """
        pass

    def list_flow_execution_messages(self, flowExecutionId: str, nextToken: str=None, maxResults: int=None) -> ListFlowExecutionMessagesResponse:
        """Returns a list of objects that contain information about events in a flow execution.

        :param self:
        :param maxResults:  The maximum number of results to return in the response.<br/>nextToken:  The string that specifies the next page of results. Use this when you're paginating results.<br/>flowExecutionId:  The ID of the flow execution.<br/>
        :return: nextToken:  The string to specify as <code>nextToken</code> when you request the next page of results. <br/>messages:  A list of objects that contain information about events in the specified flow execution.<br/>        """
        pass

    def list_tags_for_resource(self, resourceArn: str, maxResults: int=None, nextToken: str=None) -> ListTagsForResourceResponse:
        """Lists all tags on an AWS IoT Things Graph resource.

        :param self:
        :param nextToken:  The token that specifies the next page of results to return.<br/>resourceArn:  The Amazon Resource Name (ARN) of the resource whose tags are to be returned.<br/>maxResults:  The maximum number of tags to return.<br/>
        :return: nextToken:  The token that specifies the next page of results to return.<br/>tags:  List of tags returned by the <code>ListTagsForResource</code> operation.<br/>        """
        pass

    def search_entities(self, entityTypes: List[EntityType], filters: List[EntityFilter]=None, nextToken: str=None, maxResults: int=None, namespaceVersion: Version=None) -> SearchEntitiesResponse:
        """Searches for entities of the specified type. You can search for entities in your namespace and the public namespace that you're tracking.

        :param self:
        :param namespaceVersion:  The version of the user's namespace. Defaults to the latest version of the user's namespace.<br/>maxResults:  The maximum number of results to return in the response.<br/>nextToken:  The string that specifies the next page of results. Use this when you're paginating results.<br/>filters:  Optional filter to apply to the search. Valid filters are <code>NAME</code> <code>NAMESPACE</code>, <code>SEMANTIC_TYPE_PATH</code> and <code>REFERENCED_ENTITY_ID</code>. <code>REFERENCED_ENTITY_ID</code> filters on entities that are used by the entity in the result set. For example, you can filter on the ID of a property that is used in a state. Multiple filters function as OR criteria in the query. Multiple values passed inside the filter function as AND criteria.<br/>entityTypes:  The entity types for which to search.<br/>
        :return: nextToken:  The string to specify as <code>nextToken</code> when you request the next page of results.<br/>descriptions:  An array of descriptions for each entity returned in the search result.<br/>        """
        pass

    def search_flow_executions(self, systemInstanceId: str, flowExecutionId: str=None, startTime: Timestamp=None, endTime: Timestamp=None, nextToken: str=None, maxResults: int=None) -> SearchFlowExecutionsResponse:
        """Searches for AWS IoT Things Graph workflow execution instances.

        :param self:
        :param maxResults:  The maximum number of results to return in the response.<br/>nextToken:  The string that specifies the next page of results. Use this when you're paginating results.<br/>endTime:  The date and time of the latest flow execution to return.<br/>startTime:  The date and time of the earliest flow execution to return.<br/>flowExecutionId:  The ID of a flow execution.<br/>systemInstanceId:  The ID of the system instance that contains the flow.<br/>
        :return: nextToken:  The string to specify as <code>nextToken</code> when you request the next page of results.<br/>summaries:  An array of objects that contain summary information about each workflow execution in the result set.<br/>        """
        pass

    def search_flow_templates(self, filters: List[FlowTemplateFilter]=None, nextToken: str=None, maxResults: int=None) -> SearchFlowTemplatesResponse:
        """Searches for summary information about workflows.

        :param self:
        :param maxResults:  The maximum number of results to return in the response.<br/>nextToken:  The string that specifies the next page of results. Use this when you're paginating results.<br/>filters:  An array of objects that limit the result set. The only valid filter is <code>DEVICE_MODEL_ID</code>.<br/>
        :return: nextToken:  The string to specify as <code>nextToken</code> when you request the next page of results.<br/>summaries:  An array of objects that contain summary information about each workflow in the result set.<br/>        """
        pass

    def search_system_instances(self, filters: List[SystemInstanceFilter]=None, nextToken: str=None, maxResults: int=None) -> SearchSystemInstancesResponse:
        """Searches for system instances in the user's account.

        :param self:
        :param maxResults:  The maximum number of results to return in the response.<br/>nextToken:  The string that specifies the next page of results. Use this when you're paginating results.<br/>filters:  Optional filter to apply to the search. Valid filters are <code>SYSTEM_TEMPLATE_ID</code>, <code>STATUS</code>, and <code>GREENGRASS_GROUP_NAME</code>. Multiple filters function as OR criteria in the query. Multiple values passed inside the filter function as AND criteria.<br/>
        :return: nextToken:  The string to specify as <code>nextToken</code> when you request the next page of results. <br/>summaries:  An array of objects that contain summary data abour the system instances in the result set.<br/>        """
        pass

    def search_system_templates(self, filters: List[SystemTemplateFilter]=None, nextToken: str=None, maxResults: int=None) -> SearchSystemTemplatesResponse:
        """Searches for summary information about systems in the user's account. You can filter by the ID of a workflow to return only systems that use the specified workflow.

        :param self:
        :param maxResults:  The maximum number of results to return in the response.<br/>nextToken:  The string that specifies the next page of results. Use this when you're paginating results.<br/>filters:  An array of filters that limit the result set. The only valid filter is <code>FLOW_TEMPLATE_ID</code>.<br/>
        :return: nextToken:  The string to specify as <code>nextToken</code> when you request the next page of results.<br/>summaries:  An array of objects that contain summary information about each system deployment in the result set.<br/>        """
        pass

    def search_things(self, entityId: str, nextToken: str=None, maxResults: int=None, namespaceVersion: Version=None) -> SearchThingsResponse:
        """Searches for things associated with the specified entity. You can search by both device and device model. For example, if two different devices, camera1 and camera2, implement the camera device model, the user can associate thing1 to camera1 and thing2 to camera2. <code>SearchThings(camera2)</code> will return only thing2, but <code>SearchThings(camera)</code> will return both thing1 and thing2. This action searches for exact matches and doesn't perform partial text matching.

        :param self:
        :param namespaceVersion:  The version of the user's namespace. Defaults to the latest version of the user's namespace.<br/>maxResults:  The maximum number of results to return in the response.<br/>nextToken:  The string that specifies the next page of results. Use this when you're paginating results.<br/>entityId:  The ID of the entity to which the things are associated. The IDs should be in the following format.  <code>urn:tdm:REGION/ACCOUNT ID/default:device:DEVICENAME</code> <br/>
        :return: nextToken:  The string to specify as <code>nextToken</code> when you request the next page of results.<br/>things:  An array of things in the result set.<br/>        """
        pass

    def tag_resource(self, tags: List[Tag], resourceArn: str) -> TagResourceResponse:
        """Creates a tag for the specified resource.

        :param self:
        :param tags:  A list of tags to add to the resource.&gt;<br/>resourceArn:  The Amazon Resource Name (ARN) of the resource whose tags are returned.<br/>
        :return:         """
        pass

    def undeploy_system_instance(self, id: str=None) -> UndeploySystemInstanceResponse:
        """Removes a system instance from its target (Cloud or Greengrass).

        :param self:
        :param id:  The ID of the system instance to remove from its target.<br/>
        :return: summary:  An object that contains summary information about the system instance that was removed from its target.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>greengrassGroupVersionId</i> The version of the Greengrass group where the system instance is deployed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>greengrassGroupId</i> The ID of the Greengrass group where the system instance is deployed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>updatedAt</i>  The date and time when the system instance was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The date when the system instance was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>greengrassGroupName</i> The ID of the Greengrass group where the system instance is deployed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>target</i> The target of the system instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the system instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The ARN of the system instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the system instance.<br/>        """
        pass

    def untag_resource(self, tagKeys: List[TagKey], resourceArn: str) -> UntagResourceResponse:
        """Removes a tag from the specified resource.

        :param self:
        :param tagKeys:  A list of tag key names to remove from the resource. You don't specify the value. Both the key and its associated value are removed.  This parameter to the API requires a JSON text string argument. For information on how to format a JSON parameter for the various command line tool environments, see <a href="https://docs.aws.amazon.com/cli/latest/userguide/cli-usage-parameters.html#cli-using-param-json">Using JSON for Parameters</a> in the <i>AWS CLI User Guide</i>. <br/>resourceArn:  The Amazon Resource Name (ARN) of the resource whose tags are to be removed.<br/>
        :return:         """
        pass

    def update_flow_template(self, definition: DefinitionDocument, id: str, compatibleNamespaceVersion: Version=None) -> UpdateFlowTemplateResponse:
        """Updates the specified workflow. All deployed systems and system instances that use the workflow will see the changes in the flow when it is redeployed. If you don't want this behavior, copy the workflow (creating a new workflow with a different ID), and update the copy. The workflow can contain only entities in the specified namespace. 

        :param self:
        :param compatibleNamespaceVersion:  The version of the user's namespace. If no value is specified, the latest version is used by default. Use the <code>GetFlowTemplateRevisions</code> if you want to find earlier revisions of the flow to update.<br/>definition:  The <code>DefinitionDocument</code> that contains the updated workflow definition.:<br/>id:  The ID of the workflow to be updated. The ID should be in the following format.  <code>urn:tdm:REGION/ACCOUNT ID/default:workflow:WORKFLOWNAME</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>text</i> The GraphQL text that defines the entity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>language</i> The language used to define the entity. <code>GRAPHQL</code> is the only valid value.<br/>
        :return: summary:  An object containing summary information about the updated workflow.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The date when the workflow was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>revisionNumber</i> The revision number of the workflow.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The ARN of the workflow.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the workflow.<br/>        """
        pass

    def update_system_template(self, definition: DefinitionDocument, id: str, compatibleNamespaceVersion: Version=None) -> UpdateSystemTemplateResponse:
        """Updates the specified system. You don't need to run this action after updating a workflow. Any deployment that uses the system will see the changes in the system when it is redeployed.

        :param self:
        :param compatibleNamespaceVersion:  The version of the user's namespace. Defaults to the latest version of the user's namespace. If no value is specified, the latest version is used by default.<br/>definition:  The <code>DefinitionDocument</code> that contains the updated system definition.:<br/>id:  The ID of the system to be updated. The ID should be in the following format.  <code>urn:tdm:REGION/ACCOUNT ID/default:system:SYSTEMNAME</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>text</i> The GraphQL text that defines the entity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>language</i> The language used to define the entity. <code>GRAPHQL</code> is the only valid value.<br/>
        :return: summary:  An object containing summary information about the updated system.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The date when the system was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>revisionNumber</i> The revision number of the system.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The ARN of the system.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the system.<br/>        """
        pass

    def upload_entity_definitions(self, document: DefinitionDocument=None, syncWithPublicNamespace: SyncWithPublicNamespace=None, deprecateExistingEntities: DeprecateExistingEntities=None) -> UploadEntityDefinitionsResponse:
        """Asynchronously uploads one or more entity definitions to the user's namespace. The <code>document</code> parameter is required if <code>syncWithPublicNamespace</code> and <code>deleteExistingEntites</code> are false. If the <code>syncWithPublicNamespace</code> parameter is set to <code>true</code>, the user's namespace will synchronize with the latest version of the public namespace. If <code>deprecateExistingEntities</code> is set to true, all entities in the latest version will be deleted before the new <code>DefinitionDocument</code> is uploaded. When a user uploads entity definitions for the first time, the service creates a new namespace for the user. The new namespace tracks the public namespace. Currently users can have only one namespace. The namespace version increments whenever a user uploads entity definitions that are backwards-incompatible and whenever a user sets the <code>syncWithPublicNamespace</code> parameter or the <code>deprecateExistingEntities</code> parameter to <code>true</code>. The IDs for all of the entities should be in URN format. Each entity must be in the user's namespace. Users can't create entities in the public namespace, but entity definitions can refer to entities in the public namespace. Valid entities are <code>Device</code>, <code>DeviceModel</code>, <code>Service</code>, <code>Capability</code>, <code>State</code>, <code>Action</code>, <code>Event</code>, <code>Property</code>, <code>Mapping</code>, <code>Enum</code>. 

        :param self:
        :param deprecateExistingEntities:  A Boolean that specifies whether to deprecate all entities in the latest version before uploading the new <code>DefinitionDocument</code>. If set to <code>true</code>, the upload will create a new namespace version.<br/>syncWithPublicNamespace:  A Boolean that specifies whether to synchronize with the latest version of the public namespace. If set to <code>true</code>, the upload will create a new namespace version.<br/>document:  The <code>DefinitionDocument</code> that defines the updated entities.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>text</i> The GraphQL text that defines the entity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>language</i> The language used to define the entity. <code>GRAPHQL</code> is the only valid value.<br/>
        :return: uploadId:  The ID that specifies the upload action. You can use this to track the status of the upload.<br/>        """
        pass



class Kafka(BaseClient):
    """
               The operations for managing an Amazon MSK cluster.
            """


    class ListClustersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_clusters` for valid parameters"""
            pass
    class ListNodesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_nodes` for valid parameters"""
            pass
    class ListConfigurationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_configurations` for valid parameters"""
            pass
    class ListClusterOperationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_cluster_operations` for valid parameters"""
            pass
    class ListConfigurationRevisionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_configuration_revisions` for valid parameters"""
            pass


    class CreateClusterRequest(Mapping):
        pass
    
    class CreateClusterResponse(Mapping):
        pass
    
    class BrokerNodeGroupInfo(Mapping):
        pass
    
    class ClientAuthentication(Mapping):
        pass
    
    class ConfigurationInfo(Mapping):
        pass
    
    class EncryptionInfo(Mapping):
        pass
    
    class __mapOf__string(object):
        pass
    
    class CreateConfigurationRequest(Mapping):
        pass
    
    class CreateConfigurationResponse(Mapping):
        pass
    
    class __string(object):
        pass
    
    class __blob(object):
        pass
    
    class DeleteClusterRequest(Mapping):
        pass
    
    class DeleteClusterResponse(Mapping):
        pass
    
    class DescribeClusterRequest(Mapping):
        pass
    
    class DescribeClusterResponse(Mapping):
        pass
    
    class DescribeClusterOperationRequest(Mapping):
        pass
    
    class DescribeClusterOperationResponse(Mapping):
        pass
    
    class DescribeConfigurationRequest(Mapping):
        pass
    
    class DescribeConfigurationResponse(Mapping):
        pass
    
    class DescribeConfigurationRevisionRequest(Mapping):
        pass
    
    class DescribeConfigurationRevisionResponse(Mapping):
        pass
    
    class __long(object):
        pass
    
    class GetBootstrapBrokersRequest(Mapping):
        pass
    
    class GetBootstrapBrokersResponse(Mapping):
        pass
    
    class ListClusterOperationsRequest(Mapping):
        pass
    
    class ListClusterOperationsResponse(Mapping):
        pass
    
    class ListClustersRequest(Mapping):
        pass
    
    class ListClustersResponse(Mapping):
        pass
    
    class ListConfigurationRevisionsRequest(Mapping):
        pass
    
    class ListConfigurationRevisionsResponse(Mapping):
        pass
    
    class ListConfigurationsRequest(Mapping):
        pass
    
    class ListConfigurationsResponse(Mapping):
        pass
    
    class ListNodesRequest(Mapping):
        pass
    
    class ListNodesResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UpdateBrokerStorageRequest(Mapping):
        pass
    
    class UpdateBrokerStorageResponse(Mapping):
        pass
    
    class BrokerEBSVolumeInfo(Mapping):
        pass
    
    class UpdateClusterConfigurationRequest(Mapping):
        pass
    
    class UpdateClusterConfigurationResponse(Mapping):
        pass
    
    def create_cluster(self, NumberOfBrokerNodes: int, KafkaVersion: str, ClusterName: str, BrokerNodeGroupInfo: BrokerNodeGroupInfo, ClientAuthentication: ClientAuthentication=None, ConfigurationInfo: ConfigurationInfo=None, EncryptionInfo: EncryptionInfo=None, EnhancedMonitoring: str=None, Tags: __mapOf__string=None) -> CreateClusterResponse:
        """
            Creates a new MSK cluster.
         

        :param self:
        :param Tags:  
            Create tags when creating the cluster.
         <br/>NumberOfBrokerNodes:  
            The number of broker nodes in the cluster.
         <br/>KafkaVersion:  
            The version of Apache Kafka.
         <br/>EnhancedMonitoring:  
            Specifies the level of monitoring for the MSK cluster. The possible values are DEFAULT, PER_BROKER, and PER_TOPIC_PER_BROKER.
         <br/>EncryptionInfo:  
            Includes all encryption-related information.
         :<br/>ConfigurationInfo:  
            Represents the configuration that you want MSK to use for the brokers in a cluster.
         :<br/>ClusterName:  
            The name of the cluster.
         <br/>ClientAuthentication:  
            Includes all client authentication related information.
         :<br/>BrokerNodeGroupInfo:  
            Information about the broker nodes in the cluster.
         :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageInfo</i> 
            Contains information about storage volumes attached to MSK broker nodes.
         &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StorageInfo</b>: 
            Contains information about storage volumes attached to MSK broker nodes.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroups</i> 
            The AWS security groups to associate with the elastic network interfaces in order to specify who can connect to and communicate with the Amazon MSK cluster. If you don't specify a security group, Amazon MSK uses the default security group associated with the VPC.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i> 
            The type of Amazon EC2 instances to use for Kafka brokers. The following instance types are allowed: kafka.m5.large, kafka.m5.xlarge, kafka.m5.2xlarge,
kafka.m5.4xlarge, kafka.m5.12xlarge, and kafka.m5.24xlarge.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientSubnets</i> 
            The list of subnets to connect to in the client virtual private cloud (VPC). AWS creates elastic network interfaces inside these subnets. Client applications use elastic network interfaces to produce and consume data. Client subnets can't be in Availability Zone us-east-1e.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BrokerAZDistribution</i> 
            The distribution of broker nodes across Availability Zones.
         <br/>
            EBS volume information.
         &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EbsStorageInfo</b>: 
            EBS volume information.
         &nbsp;&nbsp;&nbsp;&nbsp; <i>Tls</i> 
            Details for ClientAuthentication using TLS.
         &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Tls</b>: 
            Details for ClientAuthentication using TLS.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CertificateAuthorityArnList</i> 
            List of ACM Certificate Authority ARNs.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Revision</i> 
            The revision of the configuration to use.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> 
            ARN of the configuration to use.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionInTransit</i> 
            The details for encryption in transit.
         &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncryptionInTransit</b>: 
            The details for encryption in transit.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionAtRest</i> 
            The data-volume encryption details.
         &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncryptionAtRest</b>: 
            The data-volume encryption details.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DataVolumeKMSKeyId</i> 
            The ARN of the AWS KMS key for encrypting data at rest. If you don't specify a KMS key, MSK creates one for you and uses it.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InCluster</i> 
            When set to true, it indicates that data communication among the broker nodes of the cluster is encrypted. When set to false, the communication happens in plaintext.
            The default value is true.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientBroker</i> 
            Indicates the encryption setting for data in transit between clients and brokers. The following are the possible values.
            
               TLS means that client-broker communication is enabled with TLS only.
            
               TLS_PLAINTEXT means that client-broker communication is enabled for both TLS-encrypted, as well as plaintext data.
            
               PLAINTEXT means that client-broker communication is enabled in plaintext only.
            The default value is TLS_PLAINTEXT.
         <br/>
        :return: State:  
            The state of the cluster. The possible states are CREATING, ACTIVE, and FAILED.
         <br/>ClusterName:  
            The name of the MSK cluster.
         <br/>ClusterArn:  
            The Amazon Resource Name (ARN) of the cluster.
         <br/>        """
        pass

    def create_configuration(self, ServerProperties: __blob, Name: str, KafkaVersions: List[__string], Description: str=None) -> CreateConfigurationResponse:
        """
            Creates a new MSK configuration.
         

        :param self:
        :param ServerProperties:  
            Contents of the <filename>server.properties</filename> file. When using the API, you must ensure that the contents of the file are base64 encoded. 
               When using the AWS Management Console, the SDK, or the AWS CLI, the contents of <filename>server.properties</filename> can be in plaintext.
         <br/>Name:  
            The name of the configuration.
         <br/>KafkaVersions:  
            The versions of Apache Kafka with which you can use this MSK configuration.
         <br/>Description:  
            The description of the configuration.
         <br/>
        :return: Name:  
            The name of the configuration.
         <br/>LatestRevision:  
            Latest revision of the configuration.
         :<br/>CreationTime:  
            The time when the configuration was created.
         <br/>Arn:  
            The Amazon Resource Name (ARN) of the configuration.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Revision</i> 
            The revision number.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> 
            The description of the configuration revision.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> 
            The time when the configuration revision was created.
         <br/>        """
        pass

    def delete_cluster(self, ClusterArn: str, CurrentVersion: str=None) -> DeleteClusterResponse:
        """
            Deletes the MSK cluster specified by the Amazon Resource Name (ARN) in the request.
         

        :param self:
        :param CurrentVersion:  
            The current version of the MSK cluster.
         <br/>ClusterArn:  
            The Amazon Resource Name (ARN) that uniquely identifies the cluster.
         <br/>
        :return: State:  
            The state of the cluster. The possible states are CREATING, ACTIVE, and FAILED.
         <br/>ClusterArn:  
            The Amazon Resource Name (ARN) of the cluster.
         <br/>        """
        pass

    def describe_cluster(self, ClusterArn: str) -> DescribeClusterResponse:
        """
            Returns a description of the MSK cluster whose Amazon Resource Name (ARN) is specified in the request.
         

        :param self:
        :param ClusterArn:  
            The Amazon Resource Name (ARN) that uniquely identifies the cluster.
         <br/>
        :return: ClusterInfo:  
            The cluster information.
         :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ZookeeperConnectString</i> 
            The connection string to use to connect to the Apache ZooKeeper cluster.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> 
            Tags attached to the cluster.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> 
            The state of the cluster. The possible states are CREATING, ACTIVE, and FAILED.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfBrokerNodes</i> 
            The number of broker nodes in the cluster.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedMonitoring</i> 
            Specifies which metrics are gathered for the MSK cluster. This property has three possible values: DEFAULT, PER_BROKER, and PER_TOPIC_PER_BROKER. For a list of the metrics associated with each of these three levels of monitoring, see <a href="https://docs.aws.amazon.com/msk/latest/developerguide/monitoring.html">Monitoring</a>.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionInfo</i> 
            Includes all encryption-related information.
         &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncryptionInfo</b>: 
            Includes all encryption-related information.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentVersion</i> 
            The current version of the MSK cluster.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentBrokerSoftwareInfo</i> 
            Information about the version of software currently deployed on the Kafka brokers in the cluster.
         &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CurrentBrokerSoftwareInfo</b>: 
            Information about the version of software currently deployed on the Kafka brokers in the cluster.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> 
            The time when the cluster was created.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterName</i> 
            The name of the cluster.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterArn</i> 
            The Amazon Resource Name (ARN) that uniquely identifies the cluster.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientAuthentication</i> 
            Includes all client authentication information.
         &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ClientAuthentication</b>: 
            Includes all client authentication information.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BrokerNodeGroupInfo</i> 
            Information about the broker nodes.
         &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BrokerNodeGroupInfo</b>: 
            Information about the broker nodes.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActiveOperationArn</i> 
            Arn of active cluster operation.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroups</i> 
            The AWS security groups to associate with the elastic network interfaces in order to specify who can connect to and communicate with the Amazon MSK cluster. If you don't specify a security group, Amazon MSK uses the default security group associated with the VPC.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i> 
            The type of Amazon EC2 instances to use for Kafka brokers. The following instance types are allowed: kafka.m5.large, kafka.m5.xlarge, kafka.m5.2xlarge,
kafka.m5.4xlarge, kafka.m5.12xlarge, and kafka.m5.24xlarge.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientSubnets</i> 
            The list of subnets to connect to in the client virtual private cloud (VPC). AWS creates elastic network interfaces inside these subnets. Client applications use elastic network interfaces to produce and consume data. Client subnets can't be in Availability Zone us-east-1e.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BrokerAZDistribution</i> 
            The distribution of broker nodes across Availability Zones.
         <br/>
            Contains information about storage volumes attached to MSK broker nodes.
         &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StorageInfo</b>: 
            Contains information about storage volumes attached to MSK broker nodes.
         
            Details for ClientAuthentication using TLS.
         &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Tls</b>: 
            Details for ClientAuthentication using TLS.
         &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KafkaVersion</i> 
            The version of Apache Kafka.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigurationRevision</i> 
            The revision of the configuration to use. This field isn't visible in this preview release.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigurationArn</i> 
            The Amazon Resource Name (ARN) of the configuration used for the cluster. This field isn't visible in this preview release.
         <br/>
            The data-volume encryption details.
         &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncryptionAtRest</b>: 
            The data-volume encryption details.
                 """
        pass

    def describe_cluster_operation(self, ClusterOperationArn: str) -> DescribeClusterOperationResponse:
        """
            Returns a description of the cluster operation specified by the ARN.
         

        :param self:
        :param ClusterOperationArn:  
            The Amazon Resource Name (ARN) that uniquely identifies the MSK cluster operation.
         <br/>
        :return: ClusterOperationInfo:  
            Cluster operation information
         :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetClusterInfo</i> 
            Information about cluster attributes after a cluster is updated.
         &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TargetClusterInfo</b>: 
            Information about cluster attributes after a cluster is updated.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceClusterInfo</i> 
            Information about cluster attributes before a cluster is updated.
         &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SourceClusterInfo</b>: 
            Information about cluster attributes before a cluster is updated.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OperationType</i> 
            Type of the cluster operation.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OperationState</i> 
            State of the cluster operation.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OperationArn</i> 
            ARN of the cluster operation.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorInfo</i> 
            Describes the error if the operation fails.
         &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ErrorInfo</b>: 
            Describes the error if the operation fails.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> 
            The time at which the operation finished.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> 
            The time that the operation was created.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterArn</i> 
            ARN of the cluster.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientRequestId</i> 
            The ID of the API request that triggered this operation.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorString</i> 
            An optional field to provide more details about the error.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorCode</i> 
            A number describing the error programmatically.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BrokerEBSVolumeInfo</i> 
            Specifies the size of the EBS volume and the ID of the associated broker.
         <br/>
            Information about the changes in the configuration of the brokers.
         &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ConfigurationInfo</b>: 
            Information about the changes in the configuration of the brokers.
         &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BrokerEBSVolumeInfo</i> 
            Specifies the size of the EBS volume and the ID of the associated broker.
         <br/>
            Information about the changes in the configuration of the brokers.
         &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ConfigurationInfo</b>: 
            Information about the changes in the configuration of the brokers.
                 """
        pass

    def describe_configuration(self, Arn: str) -> DescribeConfigurationResponse:
        """
            Returns a description of this MSK configuration.
         

        :param self:
        :param Arn:  
            The Amazon Resource Name (ARN) that uniquely identifies an MSK configuration and all of its revisions.
         <br/>
        :return: Name:  
            The name of the configuration.
         <br/>LatestRevision:  
            Latest revision of the configuration.
         :<br/>KafkaVersions:  
            The versions of Apache Kafka with which you can use this MSK configuration.
         <br/>Description:  
            The description of the configuration.
         <br/>CreationTime:  
            The time when the configuration was created.
         <br/>Arn:  
            The Amazon Resource Name (ARN) of the configuration.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Revision</i> 
            The revision number.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> 
            The description of the configuration revision.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> 
            The time when the configuration revision was created.
         <br/>        """
        pass

    def describe_configuration_revision(self, Revision: __long, Arn: str) -> DescribeConfigurationRevisionResponse:
        """
            Returns a description of this revision of the configuration.
         

        :param self:
        :param Revision:  
            A string that uniquely identifies a revision of an MSK configuration.
         <br/>Arn:  
            The Amazon Resource Name (ARN) that uniquely identifies an MSK configuration and all of its revisions.
         <br/>
        :return: ServerProperties:  
            Contents of the <filename>server.properties</filename> file. When using the API, you must ensure that the contents of the file are base64 encoded. 
               When using the AWS Management Console, the SDK, or the AWS CLI, the contents of <filename>server.properties</filename> can be in plaintext.
         <br/>Revision:  
            The revision number.
         <br/>Description:  
            The description of the configuration.
         <br/>CreationTime:  
            The time when the configuration was created.
         <br/>Arn:  
            The Amazon Resource Name (ARN) of the configuration.
         <br/>        """
        pass

    def get_bootstrap_brokers(self, ClusterArn: str) -> GetBootstrapBrokersResponse:
        """
            A list of brokers that a client application can use to bootstrap.
         

        :param self:
        :param ClusterArn:  
            The Amazon Resource Name (ARN) that uniquely identifies the cluster.
         <br/>
        :return: BootstrapBrokerStringTls:  
            A string containing one or more DNS names (or IP) and TLS port pairs.
         <br/>BootstrapBrokerString:  
            A string containing one or more hostname:port pairs.
         <br/>        """
        pass

    def list_cluster_operations(self, ClusterArn: str, MaxResults: int=None, NextToken: str=None) -> ListClusterOperationsResponse:
        """
            Returns a list of all the operations that have been performed on the specified MSK cluster.
         

        :param self:
        :param NextToken:  
            The paginated results marker. When the result of the operation is truncated, the call returns NextToken in the response. 
            To get the next batch, provide this token in your next request.
         <br/>MaxResults:  
            The maximum number of results to return in the response. If there are more results, the response includes a NextToken parameter.
         <br/>ClusterArn:  
            The Amazon Resource Name (ARN) that uniquely identifies the cluster.
         <br/>
        :return: NextToken:  
            If the response of ListClusterOperations is truncated, it returns a NextToken in the response. This Nexttoken should be sent in the subsequent request to ListClusterOperations.
         <br/>ClusterOperationInfoList:  
            An array of cluster operation information objects.
         <br/>        """
        pass

    def list_clusters(self, ClusterNameFilter: str=None, MaxResults: int=None, NextToken: str=None) -> ListClustersResponse:
        """
            Returns a list of all the MSK clusters in the current Region.
         

        :param self:
        :param NextToken:  
            The paginated results marker. When the result of the operation is truncated, the call returns NextToken in the response. 
            To get the next batch, provide this token in your next request.
         <br/>MaxResults:  
            The maximum number of results to return in the response. If there are more results, the response includes a NextToken parameter.
         <br/>ClusterNameFilter:  
            Specify a prefix of the name of the clusters that you want to list. The service lists all the clusters whose names start with this prefix.
         <br/>
        :return: NextToken:  
            The paginated results marker. When the result of a ListClusters operation is truncated, the call returns NextToken in the response. 
               To get another batch of clusters, provide this token in your next request.
         <br/>ClusterInfoList:  
            Information on each of the MSK clusters in the response.
         <br/>        """
        pass

    def list_configuration_revisions(self, Arn: str, MaxResults: int=None, NextToken: str=None) -> ListConfigurationRevisionsResponse:
        """
            Returns a list of all the MSK configurations in this Region.
         

        :param self:
        :param NextToken:  
            The paginated results marker. When the result of the operation is truncated, the call returns NextToken in the response. 
            To get the next batch, provide this token in your next request.
         <br/>MaxResults:  
            The maximum number of results to return in the response. If there are more results, the response includes a NextToken parameter.
         <br/>Arn:  
            The Amazon Resource Name (ARN) that uniquely identifies an MSK configuration and all of its revisions.
         <br/>
        :return: Revisions:  
            List of ConfigurationRevision objects.
         <br/>NextToken:  
            Paginated results marker.
         <br/>        """
        pass

    def list_configurations(self, MaxResults: int=None, NextToken: str=None) -> ListConfigurationsResponse:
        """
            Returns a list of all the MSK configurations in this Region.
         

        :param self:
        :param NextToken:  
            The paginated results marker. When the result of the operation is truncated, the call returns NextToken in the response. 
            To get the next batch, provide this token in your next request.
         <br/>MaxResults:  
            The maximum number of results to return in the response. If there are more results, the response includes a NextToken parameter.
         <br/>
        :return: NextToken:  
            The paginated results marker. When the result of a ListConfigurations operation is truncated, the call returns NextToken in the response. 
               To get another batch of configurations, provide this token in your next request.
         <br/>Configurations:  
            An array of MSK configurations.
         <br/>        """
        pass

    def list_nodes(self, ClusterArn: str, MaxResults: int=None, NextToken: str=None) -> ListNodesResponse:
        """
            Returns a list of the broker nodes in the cluster.
         

        :param self:
        :param NextToken:  
            The paginated results marker. When the result of the operation is truncated, the call returns NextToken in the response. 
            To get the next batch, provide this token in your next request.
         <br/>MaxResults:  
            The maximum number of results to return in the response. If there are more results, the response includes a NextToken parameter.
         <br/>ClusterArn:  
            The Amazon Resource Name (ARN) that uniquely identifies the cluster.
         <br/>
        :return: NodeInfoList:  
            List containing a NodeInfo object.
         <br/>NextToken:  
            The paginated results marker. When the result of a ListNodes operation is truncated, the call returns NextToken in the response. 
               To get another batch of nodes, provide this token in your next request.
         <br/>        """
        pass

    def list_tags_for_resource(self, ResourceArn: str) -> ListTagsForResourceResponse:
        """
            Returns a list of the tags associated with the specified resource.
         

        :param self:
        :param ResourceArn:  
            The Amazon Resource Name (ARN) that uniquely identifies the resource that's associated with the tags.
         <br/>
        :return: Tags:  
            The key-value pair for the resource tag.
         <br/>        """
        pass

    def tag_resource(self, Tags: __mapOf__string, ResourceArn: str):
        """
            Adds tags to the specified MSK resource.
         

        :param self:
        :param Tags:  
            The key-value pair for the resource tag.
         <br/>ResourceArn:  
            The Amazon Resource Name (ARN) that uniquely identifies the resource that's associated with the tags.
         <br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[__string], ResourceArn: str):
        """
            Removes the tags associated with the keys that are provided in the query.
         

        :param self:
        :param TagKeys:  
            Tag keys must be unique for a given cluster. In addition, the following restrictions apply:
            <ul>
               <li>
                  Each tag key must be unique. If you add a tag with a key that's already in
                  use, your new tag overwrites the existing key-value pair. 
               </li>
               <li>
                  You can't start a tag key with aws: because this prefix is reserved for use
                  by  AWS.  AWS creates tags that begin with this prefix on your behalf, but
                  you can't edit or delete them.
               </li>
               <li>
                  Tag keys must be between 1 and 128 Unicode characters in length.
               </li>
               <li>
                  Tag keys must consist of the following characters: Unicode letters, digits,
                  white space, and the following special characters: _ . / = + -
                     @.
               </li>
            </ul>
         <br/>ResourceArn:  
            The Amazon Resource Name (ARN) that uniquely identifies the resource that's associated with the tags.
         <br/>
        :return:         """
        pass

    def update_broker_storage(self, TargetBrokerEBSVolumeInfo: List[BrokerEBSVolumeInfo], CurrentVersion: str, ClusterArn: str) -> UpdateBrokerStorageResponse:
        """
            Updates the EBS storage associated with MSK brokers.
         

        :param self:
        :param TargetBrokerEBSVolumeInfo:  
            Describes the target volume size and the ID of the broker to apply the update to.
         <br/>CurrentVersion:  
            The version of cluster to update from. A successful operation will then generate a new version.
         <br/>ClusterArn:  
            The Amazon Resource Name (ARN) that uniquely identifies the cluster.
         <br/>
        :return: ClusterOperationArn:  
            The Amazon Resource Name (ARN) of the cluster operation.
         <br/>ClusterArn:  
            The Amazon Resource Name (ARN) of the cluster.
         <br/>        """
        pass

    def update_cluster_configuration(self, CurrentVersion: str, ConfigurationInfo: ConfigurationInfo, ClusterArn: str) -> UpdateClusterConfigurationResponse:
        """
            Updates the cluster with the configuration that is specified in the request body.
         

        :param self:
        :param CurrentVersion:  
            The version of the cluster that needs to be updated.
         <br/>ConfigurationInfo:  
            Represents the configuration that you want MSK to use for the brokers in a cluster.
         :<br/>ClusterArn:  
            The Amazon Resource Name (ARN) that uniquely identifies the cluster.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Revision</i> 
            The revision of the configuration to use.
         <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> 
            ARN of the configuration to use.
         <br/>
        :return: ClusterOperationArn:  
            The Amazon Resource Name (ARN) of the cluster operation.
         <br/>ClusterArn:  
            The Amazon Resource Name (ARN) of the cluster.
         <br/>        """
        pass



class Kinesis(BaseClient):
    """<fullname>Amazon Kinesis Data Streams Service API Reference</fullname> Amazon Kinesis Data Streams is a managed service that scales elastically for real-time processing of streaming big data."""

    class StreamExistsWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_stream` for valid parameters"""
            pass
    class StreamNotExistsWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_stream` for valid parameters"""
            pass


    class DescribeStreamPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_stream` for valid parameters"""
            pass
    class ListStreamsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_streams` for valid parameters"""
            pass
    class ListShardsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_shards` for valid parameters"""
            pass
    class ListStreamConsumersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_stream_consumers` for valid parameters"""
            pass


    class AddTagsToStreamInput(Mapping):
        pass
    
    class TagMap(object):
        pass
    
    class CreateStreamInput(Mapping):
        pass
    
    class DecreaseStreamRetentionPeriodInput(Mapping):
        pass
    
    class DeleteStreamInput(Mapping):
        pass
    
    class BooleanObject(object):
        pass
    
    class DeregisterStreamConsumerInput(Mapping):
        pass
    
    class DescribeLimitsInput(Mapping):
        pass
    
    class DescribeLimitsOutput(Mapping):
        pass
    
    class DescribeStreamInput(Mapping):
        pass
    
    class DescribeStreamOutput(Mapping):
        pass
    
    class DescribeStreamConsumerInput(Mapping):
        pass
    
    class DescribeStreamConsumerOutput(Mapping):
        pass
    
    class DescribeStreamSummaryInput(Mapping):
        pass
    
    class DescribeStreamSummaryOutput(Mapping):
        pass
    
    class DisableEnhancedMonitoringInput(Mapping):
        pass
    
    class EnhancedMonitoringOutput(Mapping):
        pass
    
    class MetricsName(object):
        pass
    
    class EnableEnhancedMonitoringInput(Mapping):
        pass
    
    class GetRecordsInput(Mapping):
        pass
    
    class GetRecordsOutput(Mapping):
        pass
    
    class GetShardIteratorInput(Mapping):
        pass
    
    class GetShardIteratorOutput(Mapping):
        pass
    
    class Timestamp(object):
        pass
    
    class IncreaseStreamRetentionPeriodInput(Mapping):
        pass
    
    class ListShardsInput(Mapping):
        pass
    
    class ListShardsOutput(Mapping):
        pass
    
    class ListStreamConsumersInput(Mapping):
        pass
    
    class ListStreamConsumersOutput(Mapping):
        pass
    
    class ListStreamsInput(Mapping):
        pass
    
    class ListStreamsOutput(Mapping):
        pass
    
    class ListTagsForStreamInput(Mapping):
        pass
    
    class ListTagsForStreamOutput(Mapping):
        pass
    
    class MergeShardsInput(Mapping):
        pass
    
    class PutRecordInput(Mapping):
        pass
    
    class PutRecordOutput(Mapping):
        pass
    
    class Data(object):
        pass
    
    class PutRecordsInput(Mapping):
        pass
    
    class PutRecordsOutput(Mapping):
        pass
    
    class PutRecordsRequestEntry(Mapping):
        pass
    
    class RegisterStreamConsumerInput(Mapping):
        pass
    
    class RegisterStreamConsumerOutput(Mapping):
        pass
    
    class RemoveTagsFromStreamInput(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class SplitShardInput(Mapping):
        pass
    
    class StartStreamEncryptionInput(Mapping):
        pass
    
    class StopStreamEncryptionInput(Mapping):
        pass
    
    class SubscribeToShardInput(Mapping):
        pass
    
    class SubscribeToShardOutput(Mapping):
        pass
    
    class StartingPosition(Mapping):
        pass
    
    class UpdateShardCountInput(Mapping):
        pass
    
    class UpdateShardCountOutput(Mapping):
        pass
    
    def add_tags_to_stream(self, Tags: TagMap, StreamName: str):
        """Adds or updates tags for the specified Kinesis data stream. Each time you invoke this operation, you can specify up to 10 tags. If you want to add more than 10 tags to your stream, you can invoke this operation multiple times. In total, each stream can have up to 50 tags. If tags have already been assigned to the stream, <code>AddTagsToStream</code> overwrites any existing tags that correspond to the specified tag keys.  <a>AddTagsToStream</a> has a limit of five transactions per second per account.

        :param self:
        :param Tags:  A set of up to 10 key-value pairs to use to create the tags.<br/>StreamName:  The name of the stream.<br/>
        :return:         """
        pass

    def create_stream(self, ShardCount: int, StreamName: str):
        """Creates a Kinesis data stream. A stream captures and transports data records that are continuously emitted from different data sources or <i>producers</i>. Scale-out within a stream is explicitly supported by means of shards, which are uniquely identified groups of data records in a stream. You specify and control the number of shards that a stream is composed of. Each shard can support reads up to five transactions per second, up to a maximum data read total of 2 MB per second. Each shard can support writes up to 1,000 records per second, up to a maximum data write total of 1 MB per second. If the amount of data input increases or decreases, you can add or remove shards. The stream name identifies the stream. The name is scoped to the AWS account used by the application. It is also scoped by AWS Region. That is, two streams in two different accounts can have the same name, and two streams in the same account, but in two different Regions, can have the same name.  <code>CreateStream</code> is an asynchronous operation. Upon receiving a <code>CreateStream</code> request, Kinesis Data Streams immediately returns and sets the stream status to <code>CREATING</code>. After the stream is created, Kinesis Data Streams sets the stream status to <code>ACTIVE</code>. You should perform read and write operations only on an <code>ACTIVE</code> stream.  You receive a <code>LimitExceededException</code> when making a <code>CreateStream</code> request when you try to do one of the following: <ul> <li> Have more than five streams in the <code>CREATING</code> state at any point in time. </li> <li> Create more shards than are authorized for your account. </li> </ul> For the default shard limit for an AWS account, see <a href="http://docs.aws.amazon.com/kinesis/latest/dev/service-sizes-and-limits.html">Amazon Kinesis Data Streams Limits</a> in the <i>Amazon Kinesis Data Streams Developer Guide</i>. To increase this limit, <a href="http://docs.aws.amazon.com/general/latest/gr/aws_service_limits.html">contact AWS Support</a>. You can use <code>DescribeStream</code> to check the stream status, which is returned in <code>StreamStatus</code>.  <a>CreateStream</a> has a limit of five transactions per second per account.

        :param self:
        :param ShardCount:  The number of shards that the stream will use. The throughput of the stream is a function of the number of shards; more shards are required for greater provisioned throughput. DefaultShardLimit;<br/>StreamName:  A name to identify the stream. The stream name is scoped to the AWS account used by the application that creates the stream. It is also scoped by AWS Region. That is, two streams in two different AWS accounts can have the same name. Two streams in the same AWS account but in two different Regions can also have the same name.<br/>
        :return:         """
        pass

    def decrease_stream_retention_period(self, RetentionPeriodHours: int, StreamName: str):
        """Decreases the Kinesis data stream's retention period, which is the length of time data records are accessible after they are added to the stream. The minimum value of a stream's retention period is 24 hours. This operation may result in lost data. For example, if the stream's retention period is 48 hours and is decreased to 24 hours, any data already in the stream that is older than 24 hours is inaccessible.

        :param self:
        :param RetentionPeriodHours:  The new retention period of the stream, in hours. Must be less than the current retention period.<br/>StreamName:  The name of the stream to modify.<br/>
        :return:         """
        pass

    def delete_stream(self, StreamName: str, EnforceConsumerDeletion: BooleanObject=None):
        """Deletes a Kinesis data stream and all its shards and data. You must shut down any applications that are operating on the stream before you delete the stream. If an application attempts to operate on a deleted stream, it receives the exception <code>ResourceNotFoundException</code>. If the stream is in the <code>ACTIVE</code> state, you can delete it. After a <code>DeleteStream</code> request, the specified stream is in the <code>DELETING</code> state until Kinesis Data Streams completes the deletion.  <b>Note:</b> Kinesis Data Streams might continue to accept data read and write operations, such as <a>PutRecord</a>, <a>PutRecords</a>, and <a>GetRecords</a>, on a stream in the <code>DELETING</code> state until the stream deletion is complete. When you delete a stream, any shards in that stream are also deleted, and any tags are dissociated from the stream. You can use the <a>DescribeStream</a> operation to check the state of the stream, which is returned in <code>StreamStatus</code>.  <a>DeleteStream</a> has a limit of five transactions per second per account.

        :param self:
        :param EnforceConsumerDeletion:  If this parameter is unset (<code>null</code>) or if you set it to <code>false</code>, and the stream has registered consumers, the call to <code>DeleteStream</code> fails with a <code>ResourceInUseException</code>. <br/>StreamName:  The name of the stream to delete.<br/>
        :return:         """
        pass

    def deregister_stream_consumer(self, StreamARN: str=None, ConsumerName: str=None, ConsumerARN: str=None):
        """To deregister a consumer, provide its ARN. Alternatively, you can provide the ARN of the data stream and the name you gave the consumer when you registered it. You may also provide all three parameters, as long as they don't conflict with each other. If you don't know the name or ARN of the consumer that you want to deregister, you can use the <a>ListStreamConsumers</a> operation to get a list of the descriptions of all the consumers that are currently registered with a given data stream. The description of a consumer contains its name and ARN. This operation has a limit of five transactions per second per account.

        :param self:
        :param ConsumerARN:  The ARN returned by Kinesis Data Streams when you registered the consumer. If you don't know the ARN of the consumer that you want to deregister, you can use the ListStreamConsumers operation to get a list of the descriptions of all the consumers that are currently registered with a given data stream. The description of a consumer contains its ARN.<br/>ConsumerName:  The name that you gave to the consumer.<br/>StreamARN:  The ARN of the Kinesis data stream that the consumer is registered with. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#arn-syntax-kinesis-streams">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>
        :return:         """
        pass

    def describe_limits(self) -> DescribeLimitsOutput:
        """Describes the shard limits and usage for the account. If you update your account limits, the old limits might be returned for a few minutes. This operation has a limit of one transaction per second per account.

        :param self:
        :param 
        :return: OpenShardCount:  The number of open shards.<br/>ShardLimit:  The maximum number of shards.<br/>        """
        pass

    def describe_stream(self, StreamName: str, Limit: int=None, ExclusiveStartShardId: str=None) -> DescribeStreamOutput:
        """Describes the specified Kinesis data stream. The information returned includes the stream name, Amazon Resource Name (ARN), creation time, enhanced metric configuration, and shard map. The shard map is an array of shard objects. For each shard object, there is the hash key and sequence number ranges that the shard spans, and the IDs of any earlier shards that played in a role in creating the shard. Every record ingested in the stream is identified by a sequence number, which is assigned when the record is put into the stream. You can limit the number of shards returned by each call. For more information, see <a href="http://docs.aws.amazon.com/kinesis/latest/dev/kinesis-using-sdk-java-retrieve-shards.html">Retrieving Shards from a Stream</a> in the <i>Amazon Kinesis Data Streams Developer Guide</i>. There are no guarantees about the chronological order shards returned. To process shards in chronological order, use the ID of the parent shard to track the lineage to the oldest shard. This operation has a limit of 10 transactions per second per account.

        :param self:
        :param ExclusiveStartShardId:  The shard ID of the shard to start with.<br/>Limit:  The maximum number of shards to return in a single call. The default value is 100. If you specify a value greater than 100, at most 100 shards are returned.<br/>StreamName:  The name of the stream to describe.<br/>
        :return: StreamDescription:  The current status of the stream, the stream Amazon Resource Name (ARN), an array of shard objects that comprise the stream, and whether there are more shards available.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyId</i> The GUID for the customer-managed AWS KMS key to use for encryption. This value can be a globally unique identifier, a fully specified ARN to either an alias or a key, or an alias name prefixed by "alias/".You can also use a master key owned by Kinesis Data Streams by specifying the alias <code>aws/kinesis</code>. <ul> <li> Key ARN example: <code>arn:aws:kms:us-east-1:123456789012:key/12345678-1234-1234-1234-123456789012</code>  </li> <li> Alias ARN example: <code>arn:aws:kms:us-east-1:123456789012:alias/MyAliasName</code>  </li> <li> Globally unique key ID example: <code>12345678-1234-1234-1234-123456789012</code>  </li> <li> Alias name example: <code>alias/MyAliasName</code>  </li> <li> Master key owned by Kinesis Data Streams: <code>alias/aws/kinesis</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionType</i> The server-side encryption type used on the stream. This parameter can be one of the following values: <ul> <li>  <code>NONE</code>: Do not encrypt the records in the stream. </li> <li>  <code>KMS</code>: Use server-side encryption on the records in the stream using a customer-managed AWS KMS key. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedMonitoring</i> Represents the current enhanced monitoring settings of the stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamCreationTimestamp</i> The approximate time that the stream was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RetentionPeriodHours</i> The current retention period, in hours.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasMoreShards</i> If set to <code>true</code>, more shards in the stream are available to describe.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Shards</i> The shards that comprise the stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamStatus</i> The current status of the stream being described. The stream status is one of the following states: <ul> <li>  <code>CREATING</code> - The stream is being created. Kinesis Data Streams immediately returns and sets <code>StreamStatus</code> to <code>CREATING</code>. </li> <li>  <code>DELETING</code> - The stream is being deleted. The specified stream is in the <code>DELETING</code> state until Kinesis Data Streams completes the deletion. </li> <li>  <code>ACTIVE</code> - The stream exists and is ready for read and write operations or deletion. You should perform read and write operations only on an <code>ACTIVE</code> stream. </li> <li>  <code>UPDATING</code> - Shards in the stream are being merged or split. Read and write operations continue to work while the stream is in the <code>UPDATING</code> state. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamARN</i> The Amazon Resource Name (ARN) for the stream being described.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamName</i> The name of the stream being described.<br/>        """
        pass

    def describe_stream_consumer(self, StreamARN: str=None, ConsumerName: str=None, ConsumerARN: str=None) -> DescribeStreamConsumerOutput:
        """To get the description of a registered consumer, provide the ARN of the consumer. Alternatively, you can provide the ARN of the data stream and the name you gave the consumer when you registered it. You may also provide all three parameters, as long as they don't conflict with each other. If you don't know the name or ARN of the consumer that you want to describe, you can use the <a>ListStreamConsumers</a> operation to get a list of the descriptions of all the consumers that are currently registered with a given data stream. This operation has a limit of 20 transactions per second per account.

        :param self:
        :param ConsumerARN:  The ARN returned by Kinesis Data Streams when you registered the consumer.<br/>ConsumerName:  The name that you gave to the consumer.<br/>StreamARN:  The ARN of the Kinesis data stream that the consumer is registered with. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#arn-syntax-kinesis-streams">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>
        :return: ConsumerDescription:  An object that represents the details of the consumer.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamARN</i> The ARN of the stream with which you registered the consumer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConsumerCreationTimestamp</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConsumerStatus</i> A consumer can't read data while in the <code>CREATING</code> or <code>DELETING</code> states.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConsumerARN</i> When you register a consumer, Kinesis Data Streams generates an ARN for it. You need this ARN to be able to call <a>SubscribeToShard</a>. If you delete a consumer and then create a new one with the same name, it won't have the same ARN. That's because consumer ARNs contain the creation timestamp. This is important to keep in mind if you have IAM policies that reference consumer ARNs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConsumerName</i> The name of the consumer is something you choose when you register the consumer.<br/>        """
        pass

    def describe_stream_summary(self, StreamName: str) -> DescribeStreamSummaryOutput:
        """Provides a summarized description of the specified Kinesis data stream without the shard list. The information returned includes the stream name, Amazon Resource Name (ARN), status, record retention period, approximate creation time, monitoring, encryption details, and open shard count. 

        :param self:
        :param StreamName:  The name of the stream to describe.<br/>
        :return: StreamDescriptionSummary:  A <a>StreamDescriptionSummary</a> containing information about the stream.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConsumerCount</i> The number of enhanced fan-out consumers registered with the stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OpenShardCount</i> The number of open shards in the stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyId</i> The GUID for the customer-managed AWS KMS key to use for encryption. This value can be a globally unique identifier, a fully specified ARN to either an alias or a key, or an alias name prefixed by "alias/".You can also use a master key owned by Kinesis Data Streams by specifying the alias <code>aws/kinesis</code>. <ul> <li> Key ARN example: <code>arn:aws:kms:us-east-1:123456789012:key/12345678-1234-1234-1234-123456789012</code>  </li> <li> Alias ARN example: <code> arn:aws:kms:us-east-1:123456789012:alias/MyAliasName</code>  </li> <li> Globally unique key ID example: <code>12345678-1234-1234-1234-123456789012</code>  </li> <li> Alias name example: <code>alias/MyAliasName</code>  </li> <li> Master key owned by Kinesis Data Streams: <code>alias/aws/kinesis</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionType</i> The encryption type used. This value is one of the following: <ul> <li>  <code>KMS</code>  </li> <li>  <code>NONE</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedMonitoring</i> Represents the current enhanced monitoring settings of the stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamCreationTimestamp</i> The approximate time that the stream was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RetentionPeriodHours</i> The current retention period, in hours.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamStatus</i> The current status of the stream being described. The stream status is one of the following states: <ul> <li>  <code>CREATING</code> - The stream is being created. Kinesis Data Streams immediately returns and sets <code>StreamStatus</code> to <code>CREATING</code>. </li> <li>  <code>DELETING</code> - The stream is being deleted. The specified stream is in the <code>DELETING</code> state until Kinesis Data Streams completes the deletion. </li> <li>  <code>ACTIVE</code> - The stream exists and is ready for read and write operations or deletion. You should perform read and write operations only on an <code>ACTIVE</code> stream. </li> <li>  <code>UPDATING</code> - Shards in the stream are being merged or split. Read and write operations continue to work while the stream is in the <code>UPDATING</code> state. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamARN</i> The Amazon Resource Name (ARN) for the stream being described.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamName</i> The name of the stream being described.<br/>        """
        pass

    def disable_enhanced_monitoring(self, ShardLevelMetrics: List[MetricsName], StreamName: str) -> EnhancedMonitoringOutput:
        """Disables enhanced monitoring.

        :param self:
        :param ShardLevelMetrics:  List of shard-level metrics to disable. The following are the valid shard-level metrics. The value "<code>ALL</code>" disables every metric. <ul> <li>  <code>IncomingBytes</code>  </li> <li>  <code>IncomingRecords</code>  </li> <li>  <code>OutgoingBytes</code>  </li> <li>  <code>OutgoingRecords</code>  </li> <li>  <code>WriteProvisionedThroughputExceeded</code>  </li> <li>  <code>ReadProvisionedThroughputExceeded</code>  </li> <li>  <code>IteratorAgeMilliseconds</code>  </li> <li>  <code>ALL</code>  </li> </ul> For more information, see <a href="http://docs.aws.amazon.com/kinesis/latest/dev/monitoring-with-cloudwatch.html">Monitoring the Amazon Kinesis Data Streams Service with Amazon CloudWatch</a> in the <i>Amazon Kinesis Data Streams Developer Guide</i>.<br/>StreamName:  The name of the Kinesis data stream for which to disable enhanced monitoring.<br/>
        :return: DesiredShardLevelMetrics:  Represents the list of all the metrics that would be in the enhanced state after the operation.<br/>CurrentShardLevelMetrics:  Represents the current state of the metrics that are in the enhanced state before the operation.<br/>StreamName:  The name of the Kinesis data stream.<br/>        """
        pass

    def enable_enhanced_monitoring(self, ShardLevelMetrics: List[MetricsName], StreamName: str) -> EnhancedMonitoringOutput:
        """Enables enhanced Kinesis data stream monitoring for shard-level metrics.

        :param self:
        :param ShardLevelMetrics:  List of shard-level metrics to enable. The following are the valid shard-level metrics. The value "<code>ALL</code>" enables every metric. <ul> <li>  <code>IncomingBytes</code>  </li> <li>  <code>IncomingRecords</code>  </li> <li>  <code>OutgoingBytes</code>  </li> <li>  <code>OutgoingRecords</code>  </li> <li>  <code>WriteProvisionedThroughputExceeded</code>  </li> <li>  <code>ReadProvisionedThroughputExceeded</code>  </li> <li>  <code>IteratorAgeMilliseconds</code>  </li> <li>  <code>ALL</code>  </li> </ul> For more information, see <a href="http://docs.aws.amazon.com/kinesis/latest/dev/monitoring-with-cloudwatch.html">Monitoring the Amazon Kinesis Data Streams Service with Amazon CloudWatch</a> in the <i>Amazon Kinesis Data Streams Developer Guide</i>.<br/>StreamName:  The name of the stream for which to enable enhanced monitoring.<br/>
        :return: DesiredShardLevelMetrics:  Represents the list of all the metrics that would be in the enhanced state after the operation.<br/>CurrentShardLevelMetrics:  Represents the current state of the metrics that are in the enhanced state before the operation.<br/>StreamName:  The name of the Kinesis data stream.<br/>        """
        pass

    def get_records(self, ShardIterator: str, Limit: int=None) -> GetRecordsOutput:
        """Gets data records from a Kinesis data stream's shard. Specify a shard iterator using the <code>ShardIterator</code> parameter. The shard iterator specifies the position in the shard from which you want to start reading data records sequentially. If there are no records available in the portion of the shard that the iterator points to, <a>GetRecords</a> returns an empty list. It might take multiple calls to get to a portion of the shard that contains records. You can scale by provisioning multiple shards per stream while considering service limits (for more information, see <a href="http://docs.aws.amazon.com/kinesis/latest/dev/service-sizes-and-limits.html">Amazon Kinesis Data Streams Limits</a> in the <i>Amazon Kinesis Data Streams Developer Guide</i>). Your application should have one thread per shard, each reading continuously from its stream. To read from a stream continually, call <a>GetRecords</a> in a loop. Use <a>GetShardIterator</a> to get the shard iterator to specify in the first <a>GetRecords</a> call. <a>GetRecords</a> returns a new shard iterator in <code>NextShardIterator</code>. Specify the shard iterator returned in <code>NextShardIterator</code> in subsequent calls to <a>GetRecords</a>. If the shard has been closed, the shard iterator can't return more data and <a>GetRecords</a> returns <code>null</code> in <code>NextShardIterator</code>. You can terminate the loop when the shard is closed, or when the shard iterator reaches the record with the sequence number or other attribute that marks it as the last record to process. Each data record can be up to 1 MiB in size, and each shard can read up to 2 MiB per second. You can ensure that your calls don't exceed the maximum supported size or throughput by using the <code>Limit</code> parameter to specify the maximum number of records that <a>GetRecords</a> can return. Consider your average record size when determining this limit. The maximum number of records that can be returned per call is 10,000. The size of the data returned by <a>GetRecords</a> varies depending on the utilization of the shard. The maximum size of data that <a>GetRecords</a> can return is 10 MiB. If a call returns this amount of data, subsequent calls made within the next 5 seconds throw <code>ProvisionedThroughputExceededException</code>. If there is insufficient provisioned throughput on the stream, subsequent calls made within the next 1 second throw <code>ProvisionedThroughputExceededException</code>. <a>GetRecords</a> doesn't return any data when it throws an exception. For this reason, we recommend that you wait 1 second between calls to <a>GetRecords</a>. However, it's possible that the application will get exceptions for longer than 1 second. To detect whether the application is falling behind in processing, you can use the <code>MillisBehindLatest</code> response attribute. You can also monitor the stream using CloudWatch metrics and other mechanisms (see <a href="http://docs.aws.amazon.com/kinesis/latest/dev/monitoring.html">Monitoring</a> in the <i>Amazon Kinesis Data Streams Developer Guide</i>). Each Amazon Kinesis record includes a value, <code>ApproximateArrivalTimestamp</code>, that is set when a stream successfully receives and stores a record. This is commonly referred to as a server-side time stamp, whereas a client-side time stamp is set when a data producer creates or sends the record to a stream (a data producer is any data source putting data records into a stream, for example with <a>PutRecords</a>). The time stamp has millisecond precision. There are no guarantees about the time stamp accuracy, or that the time stamp is always increasing. For example, records in a shard or across a stream might have time stamps that are out of order. This operation has a limit of five transactions per second per account.

        :param self:
        :param Limit:  The maximum number of records to return. Specify a value of up to 10,000. If you specify a value that is greater than 10,000, <a>GetRecords</a> throws <code>InvalidArgumentException</code>.<br/>ShardIterator:  The position in the shard from which you want to start sequentially reading data records. A shard iterator specifies this position using the sequence number of a data record in the shard.<br/>
        :return: MillisBehindLatest:  The number of milliseconds the <a>GetRecords</a> response is from the tip of the stream, indicating how far behind current time the consumer is. A value of zero indicates that record processing is caught up, and there are no new records to process at this moment.<br/>NextShardIterator:  The next position in the shard from which to start sequentially reading data records. If set to <code>null</code>, the shard has been closed and the requested iterator does not return any more data. <br/>Records:  The data records retrieved from the shard.<br/>        """
        pass

    def get_shard_iterator(self, ShardIteratorType: str, ShardId: str, StreamName: str, StartingSequenceNumber: str=None, Timestamp: Timestamp=None) -> GetShardIteratorOutput:
        """Gets an Amazon Kinesis shard iterator. A shard iterator expires 5 minutes after it is returned to the requester. A shard iterator specifies the shard position from which to start reading data records sequentially. The position is specified using the sequence number of a data record in a shard. A sequence number is the identifier associated with every record ingested in the stream, and is assigned when a record is put into the stream. Each stream has one or more shards. You must specify the shard iterator type. For example, you can set the <code>ShardIteratorType</code> parameter to read exactly from the position denoted by a specific sequence number by using the <code>AT_SEQUENCE_NUMBER</code> shard iterator type. Alternatively, the parameter can read right after the sequence number by using the <code>AFTER_SEQUENCE_NUMBER</code> shard iterator type, using sequence numbers returned by earlier calls to <a>PutRecord</a>, <a>PutRecords</a>, <a>GetRecords</a>, or <a>DescribeStream</a>. In the request, you can specify the shard iterator type <code>AT_TIMESTAMP</code> to read records from an arbitrary point in time, <code>TRIM_HORIZON</code> to cause <code>ShardIterator</code> to point to the last untrimmed record in the shard in the system (the oldest data record in the shard), or <code>LATEST</code> so that you always read the most recent data in the shard.  When you read repeatedly from a stream, use a <a>GetShardIterator</a> request to get the first shard iterator for use in your first <a>GetRecords</a> request and for subsequent reads use the shard iterator returned by the <a>GetRecords</a> request in <code>NextShardIterator</code>. A new shard iterator is returned by every <a>GetRecords</a> request in <code>NextShardIterator</code>, which you use in the <code>ShardIterator</code> parameter of the next <a>GetRecords</a> request.  If a <a>GetShardIterator</a> request is made too often, you receive a <code>ProvisionedThroughputExceededException</code>. For more information about throughput limits, see <a>GetRecords</a>, and <a href="http://docs.aws.amazon.com/kinesis/latest/dev/service-sizes-and-limits.html">Streams Limits</a> in the <i>Amazon Kinesis Data Streams Developer Guide</i>. If the shard is closed, <a>GetShardIterator</a> returns a valid iterator for the last sequence number of the shard. A shard can be closed as a result of using <a>SplitShard</a> or <a>MergeShards</a>.  <a>GetShardIterator</a> has a limit of five transactions per second per account per open shard.

        :param self:
        :param Timestamp:  The time stamp of the data record from which to start reading. Used with shard iterator type AT_TIMESTAMP. A time stamp is the Unix epoch date with precision in milliseconds. For example, <code>2016-04-04T19:58:46.480-00:00</code> or <code>1459799926.480</code>. If a record with this exact time stamp does not exist, the iterator returned is for the next (later) record. If the time stamp is older than the current trim horizon, the iterator returned is for the oldest untrimmed data record (TRIM_HORIZON).<br/>StartingSequenceNumber:  The sequence number of the data record in the shard from which to start reading. Used with shard iterator type AT_SEQUENCE_NUMBER and AFTER_SEQUENCE_NUMBER.<br/>ShardIteratorType:  Determines how the shard iterator is used to start reading data records from the shard. The following are the valid Amazon Kinesis shard iterator types: <ul> <li> AT_SEQUENCE_NUMBER - Start reading from the position denoted by a specific sequence number, provided in the value <code>StartingSequenceNumber</code>. </li> <li> AFTER_SEQUENCE_NUMBER - Start reading right after the position denoted by a specific sequence number, provided in the value <code>StartingSequenceNumber</code>. </li> <li> AT_TIMESTAMP - Start reading from the position denoted by a specific time stamp, provided in the value <code>Timestamp</code>. </li> <li> TRIM_HORIZON - Start reading at the last untrimmed record in the shard in the system, which is the oldest data record in the shard. </li> <li> LATEST - Start reading just after the most recent record in the shard, so that you always read the most recent data in the shard. </li> </ul><br/>ShardId:  The shard ID of the Kinesis Data Streams shard to get the iterator for.<br/>StreamName:  The name of the Amazon Kinesis data stream.<br/>
        :return: ShardIterator:  The position in the shard from which to start reading data records sequentially. A shard iterator specifies this position using the sequence number of a data record in a shard.<br/>        """
        pass

    def increase_stream_retention_period(self, RetentionPeriodHours: int, StreamName: str):
        """Increases the Kinesis data stream's retention period, which is the length of time data records are accessible after they are added to the stream. The maximum value of a stream's retention period is 168 hours (7 days). If you choose a longer stream retention period, this operation increases the time period during which records that have not yet expired are accessible. However, it does not make previous, expired data (older than the stream's previous retention period) accessible after the operation has been called. For example, if a stream's retention period is set to 24 hours and is increased to 168 hours, any data that is older than 24 hours remains inaccessible to consumer applications.

        :param self:
        :param RetentionPeriodHours:  The new retention period of the stream, in hours. Must be more than the current retention period.<br/>StreamName:  The name of the stream to modify.<br/>
        :return:         """
        pass

    def list_shards(self, StreamName: str=None, NextToken: str=None, ExclusiveStartShardId: str=None, MaxResults: int=None, StreamCreationTimestamp: Timestamp=None) -> ListShardsOutput:
        """Lists the shards in a stream and provides information about each shard. This operation has a limit of 100 transactions per second per data stream. <important> This API is a new operation that is used by the Amazon Kinesis Client Library (KCL). If you have a fine-grained IAM policy that only allows specific operations, you must update your policy to allow calls to this API. For more information, see <a href="https://docs.aws.amazon.com/streams/latest/dev/controlling-access.html">Controlling Access to Amazon Kinesis Data Streams Resources Using IAM</a>. </important>

        :param self:
        :param StreamCreationTimestamp:  Specify this input parameter to distinguish data streams that have the same name. For example, if you create a data stream and then delete it, and you later create another data stream with the same name, you can use this input parameter to specify which of the two streams you want to list the shards for. You cannot specify this parameter if you specify the <code>NextToken</code> parameter.<br/>MaxResults:  The maximum number of shards to return in a single call to <code>ListShards</code>. The minimum value you can specify for this parameter is 1, and the maximum is 1,000, which is also the default. When the number of shards to be listed is greater than the value of <code>MaxResults</code>, the response contains a <code>NextToken</code> value that you can use in a subsequent call to <code>ListShards</code> to list the next set of shards.<br/>ExclusiveStartShardId:  Specify this parameter to indicate that you want to list the shards starting with the shard whose ID immediately follows <code>ExclusiveStartShardId</code>. If you don't specify this parameter, the default behavior is for <code>ListShards</code> to list the shards starting with the first one in the stream. You cannot specify this parameter if you specify <code>NextToken</code>.<br/>NextToken:  When the number of shards in the data stream is greater than the default value for the <code>MaxResults</code> parameter, or if you explicitly specify a value for <code>MaxResults</code> that is less than the number of shards in the data stream, the response includes a pagination token named <code>NextToken</code>. You can specify this <code>NextToken</code> value in a subsequent call to <code>ListShards</code> to list the next set of shards. Don't specify <code>StreamName</code> or <code>StreamCreationTimestamp</code> if you specify <code>NextToken</code> because the latter unambiguously identifies the stream. You can optionally specify a value for the <code>MaxResults</code> parameter when you specify <code>NextToken</code>. If you specify a <code>MaxResults</code> value that is less than the number of shards that the operation returns if you don't specify <code>MaxResults</code>, the response will contain a new <code>NextToken</code> value. You can use the new <code>NextToken</code> value in a subsequent call to the <code>ListShards</code> operation. <important> Tokens expire after 300 seconds. When you obtain a value for <code>NextToken</code> in the response to a call to <code>ListShards</code>, you have 300 seconds to use that value. If you specify an expired token in a call to <code>ListShards</code>, you get <code>ExpiredNextTokenException</code>. </important><br/>StreamName:  The name of the data stream whose shards you want to list.  You cannot specify this parameter if you specify the <code>NextToken</code> parameter.<br/>
        :return: NextToken:  When the number of shards in the data stream is greater than the default value for the <code>MaxResults</code> parameter, or if you explicitly specify a value for <code>MaxResults</code> that is less than the number of shards in the data stream, the response includes a pagination token named <code>NextToken</code>. You can specify this <code>NextToken</code> value in a subsequent call to <code>ListShards</code> to list the next set of shards. For more information about the use of this pagination token when calling the <code>ListShards</code> operation, see <a>ListShardsInput$NextToken</a>. <important> Tokens expire after 300 seconds. When you obtain a value for <code>NextToken</code> in the response to a call to <code>ListShards</code>, you have 300 seconds to use that value. If you specify an expired token in a call to <code>ListShards</code>, you get <code>ExpiredNextTokenException</code>. </important><br/>Shards:  An array of JSON objects. Each object represents one shard and specifies the IDs of the shard, the shard's parent, and the shard that's adjacent to the shard's parent. Each object also contains the starting and ending hash keys and the starting and ending sequence numbers for the shard.<br/>        """
        pass

    def list_stream_consumers(self, StreamARN: str, NextToken: str=None, MaxResults: int=None, StreamCreationTimestamp: Timestamp=None) -> ListStreamConsumersOutput:
        """Lists the consumers registered to receive data from a stream using enhanced fan-out, and provides information about each consumer. This operation has a limit of 10 transactions per second per account.

        :param self:
        :param StreamCreationTimestamp:  Specify this input parameter to distinguish data streams that have the same name. For example, if you create a data stream and then delete it, and you later create another data stream with the same name, you can use this input parameter to specify which of the two streams you want to list the consumers for.  You can't specify this parameter if you specify the NextToken parameter. <br/>MaxResults:  The maximum number of consumers that you want a single call of <code>ListStreamConsumers</code> to return.<br/>NextToken:  When the number of consumers that are registered with the data stream is greater than the default value for the <code>MaxResults</code> parameter, or if you explicitly specify a value for <code>MaxResults</code> that is less than the number of consumers that are registered with the data stream, the response includes a pagination token named <code>NextToken</code>. You can specify this <code>NextToken</code> value in a subsequent call to <code>ListStreamConsumers</code> to list the next set of registered consumers. Don't specify <code>StreamName</code> or <code>StreamCreationTimestamp</code> if you specify <code>NextToken</code> because the latter unambiguously identifies the stream. You can optionally specify a value for the <code>MaxResults</code> parameter when you specify <code>NextToken</code>. If you specify a <code>MaxResults</code> value that is less than the number of consumers that the operation returns if you don't specify <code>MaxResults</code>, the response will contain a new <code>NextToken</code> value. You can use the new <code>NextToken</code> value in a subsequent call to the <code>ListStreamConsumers</code> operation to list the next set of consumers. <important> Tokens expire after 300 seconds. When you obtain a value for <code>NextToken</code> in the response to a call to <code>ListStreamConsumers</code>, you have 300 seconds to use that value. If you specify an expired token in a call to <code>ListStreamConsumers</code>, you get <code>ExpiredNextTokenException</code>. </important><br/>StreamARN:  The ARN of the Kinesis data stream for which you want to list the registered consumers. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#arn-syntax-kinesis-streams">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>
        :return: NextToken:  When the number of consumers that are registered with the data stream is greater than the default value for the <code>MaxResults</code> parameter, or if you explicitly specify a value for <code>MaxResults</code> that is less than the number of registered consumers, the response includes a pagination token named <code>NextToken</code>. You can specify this <code>NextToken</code> value in a subsequent call to <code>ListStreamConsumers</code> to list the next set of registered consumers. For more information about the use of this pagination token when calling the <code>ListStreamConsumers</code> operation, see <a>ListStreamConsumersInput$NextToken</a>. <important> Tokens expire after 300 seconds. When you obtain a value for <code>NextToken</code> in the response to a call to <code>ListStreamConsumers</code>, you have 300 seconds to use that value. If you specify an expired token in a call to <code>ListStreamConsumers</code>, you get <code>ExpiredNextTokenException</code>. </important><br/>Consumers:  An array of JSON objects. Each object represents one registered consumer.<br/>        """
        pass

    def list_streams(self, Limit: int=None, ExclusiveStartStreamName: str=None) -> ListStreamsOutput:
        """Lists your Kinesis data streams. The number of streams may be too large to return from a single call to <code>ListStreams</code>. You can limit the number of returned streams using the <code>Limit</code> parameter. If you do not specify a value for the <code>Limit</code> parameter, Kinesis Data Streams uses the default limit, which is currently 10. You can detect if there are more streams available to list by using the <code>HasMoreStreams</code> flag from the returned output. If there are more streams available, you can request more streams by using the name of the last stream returned by the <code>ListStreams</code> request in the <code>ExclusiveStartStreamName</code> parameter in a subsequent request to <code>ListStreams</code>. The group of stream names returned by the subsequent request is then added to the list. You can continue this process until all the stream names have been collected in the list.   <a>ListStreams</a> has a limit of five transactions per second per account.

        :param self:
        :param ExclusiveStartStreamName:  The name of the stream to start the list with.<br/>Limit:  The maximum number of streams to list.<br/>
        :return: HasMoreStreams:  If set to <code>true</code>, there are more streams available to list.<br/>StreamNames:  The names of the streams that are associated with the AWS account making the <code>ListStreams</code> request.<br/>        """
        pass

    def list_tags_for_stream(self, StreamName: str, ExclusiveStartTagKey: str=None, Limit: int=None) -> ListTagsForStreamOutput:
        """Lists the tags for the specified Kinesis data stream. This operation has a limit of five transactions per second per account.

        :param self:
        :param Limit:  The number of tags to return. If this number is less than the total number of tags associated with the stream, <code>HasMoreTags</code> is set to <code>true</code>. To list additional tags, set <code>ExclusiveStartTagKey</code> to the last key in the response.<br/>ExclusiveStartTagKey:  The key to use as the starting point for the list of tags. If this parameter is set, <code>ListTagsForStream</code> gets all tags that occur after <code>ExclusiveStartTagKey</code>. <br/>StreamName:  The name of the stream.<br/>
        :return: HasMoreTags:  If set to <code>true</code>, more tags are available. To request additional tags, set <code>ExclusiveStartTagKey</code> to the key of the last tag returned.<br/>Tags:  A list of tags associated with <code>StreamName</code>, starting with the first tag after <code>ExclusiveStartTagKey</code> and up to the specified <code>Limit</code>. <br/>        """
        pass

    def merge_shards(self, AdjacentShardToMerge: str, ShardToMerge: str, StreamName: str):
        """Merges two adjacent shards in a Kinesis data stream and combines them into a single shard to reduce the stream's capacity to ingest and transport data. Two shards are considered adjacent if the union of the hash key ranges for the two shards form a contiguous set with no gaps. For example, if you have two shards, one with a hash key range of 276...381 and the other with a hash key range of 382...454, then you could merge these two shards into a single shard that would have a hash key range of 276...454. After the merge, the single child shard receives data for all hash key values covered by the two parent shards.  <code>MergeShards</code> is called when there is a need to reduce the overall capacity of a stream because of excess capacity that is not being used. You must specify the shard to be merged and the adjacent shard for a stream. For more information about merging shards, see <a href="http://docs.aws.amazon.com/kinesis/latest/dev/kinesis-using-sdk-java-resharding-merge.html">Merge Two Shards</a> in the <i>Amazon Kinesis Data Streams Developer Guide</i>. If the stream is in the <code>ACTIVE</code> state, you can call <code>MergeShards</code>. If a stream is in the <code>CREATING</code>, <code>UPDATING</code>, or <code>DELETING</code> state, <code>MergeShards</code> returns a <code>ResourceInUseException</code>. If the specified stream does not exist, <code>MergeShards</code> returns a <code>ResourceNotFoundException</code>.  You can use <a>DescribeStream</a> to check the state of the stream, which is returned in <code>StreamStatus</code>.  <code>MergeShards</code> is an asynchronous operation. Upon receiving a <code>MergeShards</code> request, Amazon Kinesis Data Streams immediately returns a response and sets the <code>StreamStatus</code> to <code>UPDATING</code>. After the operation is completed, Kinesis Data Streams sets the <code>StreamStatus</code> to <code>ACTIVE</code>. Read and write operations continue to work while the stream is in the <code>UPDATING</code> state.  You use <a>DescribeStream</a> to determine the shard IDs that are specified in the <code>MergeShards</code> request.  If you try to operate on too many streams in parallel using <a>CreateStream</a>, <a>DeleteStream</a>, <code>MergeShards</code>, or <a>SplitShard</a>, you receive a <code>LimitExceededException</code>.   <code>MergeShards</code> has a limit of five transactions per second per account.

        :param self:
        :param AdjacentShardToMerge:  The shard ID of the adjacent shard for the merge.<br/>ShardToMerge:  The shard ID of the shard to combine with the adjacent shard for the merge.<br/>StreamName:  The name of the stream for the merge.<br/>
        :return:         """
        pass

    def put_record(self, PartitionKey: str, Data: Data, StreamName: str, ExplicitHashKey: str=None, SequenceNumberForOrdering: str=None) -> PutRecordOutput:
        """Writes a single data record into an Amazon Kinesis data stream. Call <code>PutRecord</code> to send data into the stream for real-time ingestion and subsequent processing, one record at a time. Each shard can support writes up to 1,000 records per second, up to a maximum data write total of 1 MB per second. You must specify the name of the stream that captures, stores, and transports the data; a partition key; and the data blob itself. The data blob can be any type of data; for example, a segment from a log file, geographic/location data, website clickstream data, and so on. The partition key is used by Kinesis Data Streams to distribute data across shards. Kinesis Data Streams segregates the data records that belong to a stream into multiple shards, using the partition key associated with each data record to determine the shard to which a given data record belongs. Partition keys are Unicode strings, with a maximum length limit of 256 characters for each key. An MD5 hash function is used to map partition keys to 128-bit integer values and to map associated data records to shards using the hash key ranges of the shards. You can override hashing the partition key to determine the shard by explicitly specifying a hash value using the <code>ExplicitHashKey</code> parameter. For more information, see <a href="http://docs.aws.amazon.com/kinesis/latest/dev/developing-producers-with-sdk.html#kinesis-using-sdk-java-add-data-to-stream">Adding Data to a Stream</a> in the <i>Amazon Kinesis Data Streams Developer Guide</i>.  <code>PutRecord</code> returns the shard ID of where the data record was placed and the sequence number that was assigned to the data record. Sequence numbers increase over time and are specific to a shard within a stream, not across all shards within a stream. To guarantee strictly increasing ordering, write serially to a shard and use the <code>SequenceNumberForOrdering</code> parameter. For more information, see <a href="http://docs.aws.amazon.com/kinesis/latest/dev/developing-producers-with-sdk.html#kinesis-using-sdk-java-add-data-to-stream">Adding Data to a Stream</a> in the <i>Amazon Kinesis Data Streams Developer Guide</i>. If a <code>PutRecord</code> request cannot be processed because of insufficient provisioned throughput on the shard involved in the request, <code>PutRecord</code> throws <code>ProvisionedThroughputExceededException</code>.  By default, data records are accessible for 24 hours from the time that they are added to a stream. You can use <a>IncreaseStreamRetentionPeriod</a> or <a>DecreaseStreamRetentionPeriod</a> to modify this retention period.

        :param self:
        :param SequenceNumberForOrdering:  Guarantees strictly increasing sequence numbers, for puts from the same client and to the same partition key. Usage: set the <code>SequenceNumberForOrdering</code> of record <i>n</i> to the sequence number of record <i>n-1</i> (as returned in the result when putting record <i>n-1</i>). If this parameter is not set, records are coarsely ordered based on arrival time.<br/>ExplicitHashKey:  The hash value used to explicitly determine the shard the data record is assigned to by overriding the partition key hash.<br/>PartitionKey:  Determines which shard in the stream the data record is assigned to. Partition keys are Unicode strings with a maximum length limit of 256 characters for each key. Amazon Kinesis Data Streams uses the partition key as input to a hash function that maps the partition key and associated data to a specific shard. Specifically, an MD5 hash function is used to map partition keys to 128-bit integer values and to map associated data records to shards. As a result of this hashing mechanism, all data records with the same partition key map to the same shard within the stream.<br/>Data:  The data blob to put into the record, which is base64-encoded when the blob is serialized. When the data blob (the payload before base64-encoding) is added to the partition key size, the total size must not exceed the maximum record size (1 MB).<br/>StreamName:  The name of the stream to put the data record into.<br/>
        :return: EncryptionType:  The encryption type to use on the record. This parameter can be one of the following values: <ul> <li>  <code>NONE</code>: Do not encrypt the records in the stream. </li> <li>  <code>KMS</code>: Use server-side encryption on the records in the stream using a customer-managed AWS KMS key. </li> </ul><br/>SequenceNumber:  The sequence number identifier that was assigned to the put data record. The sequence number for the record is unique across all records in the stream. A sequence number is the identifier associated with every record put into the stream.<br/>ShardId:  The shard ID of the shard where the data record was placed.<br/>        """
        pass

    def put_records(self, StreamName: str, Records: List[PutRecordsRequestEntry]) -> PutRecordsOutput:
        """Writes multiple data records into a Kinesis data stream in a single call (also referred to as a <code>PutRecords</code> request). Use this operation to send data into the stream for data ingestion and processing.  Each <code>PutRecords</code> request can support up to 500 records. Each record in the request can be as large as 1 MB, up to a limit of 5 MB for the entire request, including partition keys. Each shard can support writes up to 1,000 records per second, up to a maximum data write total of 1 MB per second. You must specify the name of the stream that captures, stores, and transports the data; and an array of request <code>Records</code>, with each record in the array requiring a partition key and data blob. The record size limit applies to the total size of the partition key and data blob. The data blob can be any type of data; for example, a segment from a log file, geographic/location data, website clickstream data, and so on. The partition key is used by Kinesis Data Streams as input to a hash function that maps the partition key and associated data to a specific shard. An MD5 hash function is used to map partition keys to 128-bit integer values and to map associated data records to shards. As a result of this hashing mechanism, all data records with the same partition key map to the same shard within the stream. For more information, see <a href="http://docs.aws.amazon.com/kinesis/latest/dev/developing-producers-with-sdk.html#kinesis-using-sdk-java-add-data-to-stream">Adding Data to a Stream</a> in the <i>Amazon Kinesis Data Streams Developer Guide</i>. Each record in the <code>Records</code> array may include an optional parameter, <code>ExplicitHashKey</code>, which overrides the partition key to shard mapping. This parameter allows a data producer to determine explicitly the shard where the record is stored. For more information, see <a href="http://docs.aws.amazon.com/kinesis/latest/dev/developing-producers-with-sdk.html#kinesis-using-sdk-java-putrecords">Adding Multiple Records with PutRecords</a> in the <i>Amazon Kinesis Data Streams Developer Guide</i>. The <code>PutRecords</code> response includes an array of response <code>Records</code>. Each record in the response array directly correlates with a record in the request array using natural ordering, from the top to the bottom of the request and response. The response <code>Records</code> array always includes the same number of records as the request array. The response <code>Records</code> array includes both successfully and unsuccessfully processed records. Kinesis Data Streams attempts to process all records in each <code>PutRecords</code> request. A single record failure does not stop the processing of subsequent records. A successfully processed record includes <code>ShardId</code> and <code>SequenceNumber</code> values. The <code>ShardId</code> parameter identifies the shard in the stream where the record is stored. The <code>SequenceNumber</code> parameter is an identifier assigned to the put record, unique to all records in the stream. An unsuccessfully processed record includes <code>ErrorCode</code> and <code>ErrorMessage</code> values. <code>ErrorCode</code> reflects the type of error and can be one of the following values: <code>ProvisionedThroughputExceededException</code> or <code>InternalFailure</code>. <code>ErrorMessage</code> provides more detailed information about the <code>ProvisionedThroughputExceededException</code> exception including the account ID, stream name, and shard ID of the record that was throttled. For more information about partially successful responses, see <a href="http://docs.aws.amazon.com/kinesis/latest/dev/kinesis-using-sdk-java-add-data-to-stream.html#kinesis-using-sdk-java-putrecords">Adding Multiple Records with PutRecords</a> in the <i>Amazon Kinesis Data Streams Developer Guide</i>. By default, data records are accessible for 24 hours from the time that they are added to a stream. You can use <a>IncreaseStreamRetentionPeriod</a> or <a>DecreaseStreamRetentionPeriod</a> to modify this retention period.

        :param self:
        :param StreamName:  The stream name associated with the request.<br/>Records:  The records associated with the request.<br/>
        :return: EncryptionType:  The encryption type used on the records. This parameter can be one of the following values: <ul> <li>  <code>NONE</code>: Do not encrypt the records. </li> <li>  <code>KMS</code>: Use server-side encryption on the records using a customer-managed AWS KMS key. </li> </ul><br/>Records:  An array of successfully and unsuccessfully processed record results, correlated with the request by natural ordering. A record that is successfully added to a stream includes <code>SequenceNumber</code> and <code>ShardId</code> in the result. A record that fails to be added to a stream includes <code>ErrorCode</code> and <code>ErrorMessage</code> in the result.<br/>FailedRecordCount:  The number of unsuccessfully processed records in a <code>PutRecords</code> request.<br/>        """
        pass

    def register_stream_consumer(self, ConsumerName: str, StreamARN: str) -> RegisterStreamConsumerOutput:
        """Registers a consumer with a Kinesis data stream. When you use this operation, the consumer you register can read data from the stream at a rate of up to 2 MiB per second. This rate is unaffected by the total number of consumers that read from the same stream. You can register up to 5 consumers per stream. A given consumer can only be registered with one stream. This operation has a limit of five transactions per second per account.

        :param self:
        :param ConsumerName:  For a given Kinesis data stream, each consumer must have a unique name. However, consumer names don't have to be unique across data streams.<br/>StreamARN:  The ARN of the Kinesis data stream that you want to register the consumer with. For more info, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#arn-syntax-kinesis-streams">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>.<br/>
        :return: Consumer:  An object that represents the details of the consumer you registered. When you register a consumer, it gets an ARN that is generated by Kinesis Data Streams.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConsumerCreationTimestamp</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConsumerStatus</i> A consumer can't read data while in the <code>CREATING</code> or <code>DELETING</code> states.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConsumerARN</i> When you register a consumer, Kinesis Data Streams generates an ARN for it. You need this ARN to be able to call <a>SubscribeToShard</a>. If you delete a consumer and then create a new one with the same name, it won't have the same ARN. That's because consumer ARNs contain the creation timestamp. This is important to keep in mind if you have IAM policies that reference consumer ARNs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConsumerName</i> The name of the consumer is something you choose when you register the consumer.<br/>        """
        pass

    def remove_tags_from_stream(self, TagKeys: List[TagKey], StreamName: str):
        """Removes tags from the specified Kinesis data stream. Removed tags are deleted and cannot be recovered after this operation successfully completes. If you specify a tag that does not exist, it is ignored.  <a>RemoveTagsFromStream</a> has a limit of five transactions per second per account.

        :param self:
        :param TagKeys:  A list of tag keys. Each corresponding tag is removed from the stream.<br/>StreamName:  The name of the stream.<br/>
        :return:         """
        pass

    def split_shard(self, NewStartingHashKey: str, ShardToSplit: str, StreamName: str):
        """Splits a shard into two new shards in the Kinesis data stream, to increase the stream's capacity to ingest and transport data. <code>SplitShard</code> is called when there is a need to increase the overall capacity of a stream because of an expected increase in the volume of data records being ingested.  You can also use <code>SplitShard</code> when a shard appears to be approaching its maximum utilization; for example, the producers sending data into the specific shard are suddenly sending more than previously anticipated. You can also call <code>SplitShard</code> to increase stream capacity, so that more Kinesis Data Streams applications can simultaneously read data from the stream for real-time processing.  You must specify the shard to be split and the new hash key, which is the position in the shard where the shard gets split in two. In many cases, the new hash key might be the average of the beginning and ending hash key, but it can be any hash key value in the range being mapped into the shard. For more information, see <a href="http://docs.aws.amazon.com/kinesis/latest/dev/kinesis-using-sdk-java-resharding-split.html">Split a Shard</a> in the <i>Amazon Kinesis Data Streams Developer Guide</i>. You can use <a>DescribeStream</a> to determine the shard ID and hash key values for the <code>ShardToSplit</code> and <code>NewStartingHashKey</code> parameters that are specified in the <code>SplitShard</code> request.  <code>SplitShard</code> is an asynchronous operation. Upon receiving a <code>SplitShard</code> request, Kinesis Data Streams immediately returns a response and sets the stream status to <code>UPDATING</code>. After the operation is completed, Kinesis Data Streams sets the stream status to <code>ACTIVE</code>. Read and write operations continue to work while the stream is in the <code>UPDATING</code> state.  You can use <code>DescribeStream</code> to check the status of the stream, which is returned in <code>StreamStatus</code>. If the stream is in the <code>ACTIVE</code> state, you can call <code>SplitShard</code>. If a stream is in <code>CREATING</code> or <code>UPDATING</code> or <code>DELETING</code> states, <code>DescribeStream</code> returns a <code>ResourceInUseException</code>. If the specified stream does not exist, <code>DescribeStream</code> returns a <code>ResourceNotFoundException</code>. If you try to create more shards than are authorized for your account, you receive a <code>LimitExceededException</code>.  For the default shard limit for an AWS account, see <a href="http://docs.aws.amazon.com/kinesis/latest/dev/service-sizes-and-limits.html">Kinesis Data Streams Limits</a> in the <i>Amazon Kinesis Data Streams Developer Guide</i>. To increase this limit, <a href="http://docs.aws.amazon.com/general/latest/gr/aws_service_limits.html">contact AWS Support</a>. If you try to operate on too many streams simultaneously using <a>CreateStream</a>, <a>DeleteStream</a>, <a>MergeShards</a>, and/or <a>SplitShard</a>, you receive a <code>LimitExceededException</code>.   <code>SplitShard</code> has a limit of five transactions per second per account.

        :param self:
        :param NewStartingHashKey:  A hash key value for the starting hash key of one of the child shards created by the split. The hash key range for a given shard constitutes a set of ordered contiguous positive integers. The value for <code>NewStartingHashKey</code> must be in the range of hash keys being mapped into the shard. The <code>NewStartingHashKey</code> hash key value and all higher hash key values in hash key range are distributed to one of the child shards. All the lower hash key values in the range are distributed to the other child shard.<br/>ShardToSplit:  The shard ID of the shard to split.<br/>StreamName:  The name of the stream for the shard split.<br/>
        :return:         """
        pass

    def start_stream_encryption(self, KeyId: str, EncryptionType: str, StreamName: str):
        """Enables or updates server-side encryption using an AWS KMS key for a specified stream.  Starting encryption is an asynchronous operation. Upon receiving the request, Kinesis Data Streams returns immediately and sets the status of the stream to <code>UPDATING</code>. After the update is complete, Kinesis Data Streams sets the status of the stream back to <code>ACTIVE</code>. Updating or applying encryption normally takes a few seconds to complete, but it can take minutes. You can continue to read and write data to your stream while its status is <code>UPDATING</code>. Once the status of the stream is <code>ACTIVE</code>, encryption begins for records written to the stream.  API Limits: You can successfully apply a new AWS KMS key for server-side encryption 25 times in a rolling 24-hour period. Note: It can take up to 5 seconds after the stream is in an <code>ACTIVE</code> status before all records written to the stream are encrypted. After you enable encryption, you can verify that encryption is applied by inspecting the API response from <code>PutRecord</code> or <code>PutRecords</code>.

        :param self:
        :param KeyId:  The GUID for the customer-managed AWS KMS key to use for encryption. This value can be a globally unique identifier, a fully specified Amazon Resource Name (ARN) to either an alias or a key, or an alias name prefixed by "alias/".You can also use a master key owned by Kinesis Data Streams by specifying the alias <code>aws/kinesis</code>. <ul> <li> Key ARN example: <code>arn:aws:kms:us-east-1:123456789012:key/12345678-1234-1234-1234-123456789012</code>  </li> <li> Alias ARN example: <code>arn:aws:kms:us-east-1:123456789012:alias/MyAliasName</code>  </li> <li> Globally unique key ID example: <code>12345678-1234-1234-1234-123456789012</code>  </li> <li> Alias name example: <code>alias/MyAliasName</code>  </li> <li> Master key owned by Kinesis Data Streams: <code>alias/aws/kinesis</code>  </li> </ul><br/>EncryptionType:  The encryption type to use. The only valid value is <code>KMS</code>.<br/>StreamName:  The name of the stream for which to start encrypting records.<br/>
        :return:         """
        pass

    def stop_stream_encryption(self, KeyId: str, EncryptionType: str, StreamName: str):
        """Disables server-side encryption for a specified stream.  Stopping encryption is an asynchronous operation. Upon receiving the request, Kinesis Data Streams returns immediately and sets the status of the stream to <code>UPDATING</code>. After the update is complete, Kinesis Data Streams sets the status of the stream back to <code>ACTIVE</code>. Stopping encryption normally takes a few seconds to complete, but it can take minutes. You can continue to read and write data to your stream while its status is <code>UPDATING</code>. Once the status of the stream is <code>ACTIVE</code>, records written to the stream are no longer encrypted by Kinesis Data Streams.  API Limits: You can successfully disable server-side encryption 25 times in a rolling 24-hour period.  Note: It can take up to 5 seconds after the stream is in an <code>ACTIVE</code> status before all records written to the stream are no longer subject to encryption. After you disabled encryption, you can verify that encryption is not applied by inspecting the API response from <code>PutRecord</code> or <code>PutRecords</code>.

        :param self:
        :param KeyId:  The GUID for the customer-managed AWS KMS key to use for encryption. This value can be a globally unique identifier, a fully specified Amazon Resource Name (ARN) to either an alias or a key, or an alias name prefixed by "alias/".You can also use a master key owned by Kinesis Data Streams by specifying the alias <code>aws/kinesis</code>. <ul> <li> Key ARN example: <code>arn:aws:kms:us-east-1:123456789012:key/12345678-1234-1234-1234-123456789012</code>  </li> <li> Alias ARN example: <code>arn:aws:kms:us-east-1:123456789012:alias/MyAliasName</code>  </li> <li> Globally unique key ID example: <code>12345678-1234-1234-1234-123456789012</code>  </li> <li> Alias name example: <code>alias/MyAliasName</code>  </li> <li> Master key owned by Kinesis Data Streams: <code>alias/aws/kinesis</code>  </li> </ul><br/>EncryptionType:  The encryption type. The only valid value is <code>KMS</code>.<br/>StreamName:  The name of the stream on which to stop encrypting records.<br/>
        :return:         """
        pass

    def subscribe_to_shard(self, StartingPosition: StartingPosition, ShardId: str, ConsumerARN: str) -> SubscribeToShardOutput:
        """Call this operation from your consumer after you call <a>RegisterStreamConsumer</a> to register the consumer with Kinesis Data Streams. If the call succeeds, your consumer starts receiving events of type <a>SubscribeToShardEvent</a> for up to 5 minutes, after which time you need to call <code>SubscribeToShard</code> again to renew the subscription if you want to continue to receive records. You can make one call to <code>SubscribeToShard</code> per second per <code>ConsumerARN</code>. If your call succeeds, and then you call the operation again less than 5 seconds later, the second call generates a <a>ResourceInUseException</a>. If you call the operation a second time more than 5 seconds after the first call succeeds, the second call succeeds and the first connection gets shut down.

        :param self:
        :param StartingPosition:  :<br/>ShardId:  The ID of the shard you want to subscribe to. To see a list of all the shards for a given stream, use <a>ListShards</a>.<br/>ConsumerARN:  For this parameter, use the value you obtained when you called <a>RegisterStreamConsumer</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timestamp</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SequenceNumber</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> <br/>
        :return: EventStream:  The event stream that your consumer can use to read records from the shard.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InternalFailureException</i> &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InternalFailureException</b>: <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KMSThrottlingException</i> The request was denied due to request throttling. For more information about throttling, see <a href="http://docs.aws.amazon.com/kms/latest/developerguide/limits.html#requests-per-second">Limits</a> in the <i>AWS Key Management Service Developer Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KMSThrottlingException</b>: The request was denied due to request throttling. For more information about throttling, see <a href="http://docs.aws.amazon.com/kms/latest/developerguide/limits.html#requests-per-second">Limits</a> in the <i>AWS Key Management Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KMSOptInRequired</i> The AWS access key ID needs a subscription for the service.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KMSOptInRequired</b>: The AWS access key ID needs a subscription for the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KMSNotFoundException</i> The request was rejected because the specified entity or resource can't be found.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KMSNotFoundException</b>: The request was rejected because the specified entity or resource can't be found.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KMSAccessDeniedException</i> The ciphertext references a key that doesn't exist or that you don't have access to.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KMSAccessDeniedException</b>: The ciphertext references a key that doesn't exist or that you don't have access to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KMSInvalidStateException</i> The request was rejected because the state of the specified resource isn't valid for this request. For more information, see <a href="http://docs.aws.amazon.com/kms/latest/developerguide/key-state.html">How Key State Affects Use of a Customer Master Key</a> in the <i>AWS Key Management Service Developer Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KMSInvalidStateException</b>: The request was rejected because the state of the specified resource isn't valid for this request. For more information, see <a href="http://docs.aws.amazon.com/kms/latest/developerguide/key-state.html">How Key State Affects Use of a Customer Master Key</a> in the <i>AWS Key Management Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KMSDisabledException</i> The request was rejected because the specified customer master key (CMK) isn't enabled.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KMSDisabledException</b>: The request was rejected because the specified customer master key (CMK) isn't enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceInUseException</i> The resource is not available for this operation. For successful operation, the resource must be in the <code>ACTIVE</code> state.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResourceInUseException</b>: The resource is not available for this operation. For successful operation, the resource must be in the <code>ACTIVE</code> state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceNotFoundException</i> The requested resource could not be found. The stream might not be specified correctly.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResourceNotFoundException</b>: The requested resource could not be found. The stream might not be specified correctly.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubscribeToShardEvent</i> After you call <a>SubscribeToShard</a>, Kinesis Data Streams sends events of this type to your consumer. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SubscribeToShardEvent</b>: After you call <a>SubscribeToShard</a>, Kinesis Data Streams sends events of this type to your consumer. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MillisBehindLatest</i> The number of milliseconds the read records are from the tip of the stream, indicating how far behind current time the consumer is. A value of zero indicates that record processing is caught up, and there are no new records to process at this moment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ContinuationSequenceNumber</i> Use this as <code>StartingSequenceNumber</code> in the next call to <a>SubscribeToShard</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Records</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>message</i> A message that provides information about the error.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>message</i> A message that provides information about the error.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>message</i> A message that provides information about the error.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>message</i> A message that provides information about the error.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>message</i> A message that provides information about the error.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>message</i> A message that provides information about the error.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>message</i> A message that provides information about the error.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>message</i> A message that provides information about the error.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>message</i> <br/>        """
        pass

    def update_shard_count(self, ScalingType: str, TargetShardCount: int, StreamName: str) -> UpdateShardCountOutput:
        """Updates the shard count of the specified stream to the specified number of shards. Updating the shard count is an asynchronous operation. Upon receiving the request, Kinesis Data Streams returns immediately and sets the status of the stream to <code>UPDATING</code>. After the update is complete, Kinesis Data Streams sets the status of the stream back to <code>ACTIVE</code>. Depending on the size of the stream, the scaling action could take a few minutes to complete. You can continue to read and write data to your stream while its status is <code>UPDATING</code>. To update the shard count, Kinesis Data Streams performs splits or merges on individual shards. This can cause short-lived shards to be created, in addition to the final shards. We recommend that you double or halve the shard count, as this results in the fewest number of splits or merges. This operation has the following default limits. By default, you cannot do the following: <ul> <li> Scale more than twice per rolling 24-hour period per stream </li> <li> Scale up to more than double your current shard count for a stream </li> <li> Scale down below half your current shard count for a stream </li> <li> Scale up to more than 500 shards in a stream </li> <li> Scale a stream with more than 500 shards down unless the result is less than 500 shards </li> <li> Scale up to more than the shard limit for your account </li> </ul> For the default limits for an AWS account, see <a href="http://docs.aws.amazon.com/kinesis/latest/dev/service-sizes-and-limits.html">Streams Limits</a> in the <i>Amazon Kinesis Data Streams Developer Guide</i>. To request an increase in the call rate limit, the shard limit for this API, or your overall shard limit, use the <a href="https://console.aws.amazon.com/support/v1#/case/create?issueType=service-limit-increase&amp;limitType=service-code-kinesis">limits form</a>.

        :param self:
        :param ScalingType:  The scaling type. Uniform scaling creates shards of equal size.<br/>TargetShardCount:  The new number of shards.<br/>StreamName:  The name of the stream.<br/>
        :return: TargetShardCount:  The updated number of shards.<br/>CurrentShardCount:  The current number of shards.<br/>StreamName:  The name of the stream.<br/>        """
        pass



class KinesisVideoArchivedMedia(BaseClient):
    """<p/>"""


    class ListFragmentsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_fragments` for valid parameters"""
            pass


    class GetDASHStreamingSessionURLInput(Mapping):
        pass
    
    class GetDASHStreamingSessionURLOutput(Mapping):
        pass
    
    class DASHFragmentSelector(Mapping):
        pass
    
    class PageLimit(object):
        pass
    
    class GetHLSStreamingSessionURLInput(Mapping):
        pass
    
    class GetHLSStreamingSessionURLOutput(Mapping):
        pass
    
    class HLSFragmentSelector(Mapping):
        pass
    
    class GetMediaForFragmentListInput(Mapping):
        pass
    
    class GetMediaForFragmentListOutput(Mapping):
        pass
    
    class FragmentNumberString(object):
        pass
    
    class ListFragmentsInput(Mapping):
        pass
    
    class ListFragmentsOutput(Mapping):
        pass
    
    class FragmentSelector(Mapping):
        pass
    
    def get_dash_streaming_session_url(self, StreamName: str=None, StreamARN: str=None, PlaybackMode: str=None, DisplayFragmentTimestamp: str=None, DisplayFragmentNumber: str=None, DASHFragmentSelector: DASHFragmentSelector=None, Expires: int=None, MaxManifestFragmentResults: PageLimit=None) -> GetDASHStreamingSessionURLOutput:
        """Retrieves an MPEG Dynamic Adaptive Streaming over HTTP (DASH) URL for the stream. You can then open the URL in a media player to view the stream contents. Both the <code>StreamName</code> and the <code>StreamARN</code> parameters are optional, but you must specify either the <code>StreamName</code> or the <code>StreamARN</code> when invoking this API operation. An Amazon Kinesis video stream has the following requirements for providing data through MPEG-DASH: <ul> <li> The media must contain h.264 or h.265 encoded video and, optionally, AAC or G.711 encoded audio. Specifically, the codec id of track 1 should be <code>V_MPEG/ISO/AVC</code> (for h.264) or V_MPEGH/ISO/HEVC (for H.265). Optionally, the codec id of track 2 should be <code>A_AAC</code> (for AAC) or A_MS/ACM (for G.711). </li> <li> Data retention must be greater than 0. </li> <li> The video track of each fragment must contain codec private data in the Advanced Video Coding (AVC) for H.264 format and HEVC for H.265 format. For more information, see <a href="https://www.iso.org/standard/55980.html">MPEG-4 specification ISO/IEC 14496-15</a>. For information about adapting stream data to a given format, see <a href="http://docs.aws.amazon.com/kinesisvideostreams/latest/dg/producer-reference-nal.html">NAL Adaptation Flags</a>. </li> <li> The audio track (if present) of each fragment must contain codec private data in the AAC format (<a href="https://www.iso.org/standard/43345.html">AAC specification ISO/IEC 13818-7</a>) or the <a href="http://www-mmsp.ece.mcgill.ca/Documents/AudioFormats/WAVE/WAVE.html">MS Wave format</a>. </li> </ul> The following procedure shows how to use MPEG-DASH with Kinesis Video Streams: <ol> <li> Get an endpoint using <a href="http://docs.aws.amazon.com/kinesisvideostreams/latest/dg/API_GetDataEndpoint.html">GetDataEndpoint</a>, specifying <code>GET_DASH_STREAMING_SESSION_URL</code> for the <code>APIName</code> parameter. </li> <li> Retrieve the MPEG-DASH URL using <code>GetDASHStreamingSessionURL</code>. Kinesis Video Streams creates an MPEG-DASH streaming session to be used for accessing content in a stream using the MPEG-DASH protocol. <code>GetDASHStreamingSessionURL</code> returns an authenticated URL (that includes an encrypted session token) for the session's MPEG-DASH <i>manifest</i> (the root resource needed for streaming with MPEG-DASH). <note> Don't share or store this token where an unauthorized entity could access it. The token provides access to the content of the stream. Safeguard the token with the same measures that you would use with your AWS credentials. </note> The media that is made available through the manifest consists only of the requested stream, time range, and format. No other media data (such as frames outside the requested window or alternate bitrates) is made available. </li> <li> Provide the URL (containing the encrypted session token) for the MPEG-DASH manifest to a media player that supports the MPEG-DASH protocol. Kinesis Video Streams makes the initialization fragment, and media fragments available through the manifest URL. The initialization fragment contains the codec private data for the stream, and other data needed to set up the video or audio decoder and renderer. The media fragments contain encoded video frames or encoded audio samples. </li> <li> The media player receives the authenticated URL and requests stream metadata and media data normally. When the media player requests data, it calls the following actions: <ul> <li>  <b>GetDASHManifest:</b> Retrieves an MPEG DASH manifest, which contains the metadata for the media that you want to playback. </li> <li>  <b>GetMP4InitFragment:</b> Retrieves the MP4 initialization fragment. The media player typically loads the initialization fragment before loading any media fragments. This fragment contains the "<code>fytp</code>" and "<code>moov</code>" MP4 atoms, and the child atoms that are needed to initialize the media player decoder. The initialization fragment does not correspond to a fragment in a Kinesis video stream. It contains only the codec private data for the stream and respective track, which the media player needs to decode the media frames. </li> <li>  <b>GetMP4MediaFragment:</b> Retrieves MP4 media fragments. These fragments contain the "<code>moof</code>" and "<code>mdat</code>" MP4 atoms and their child atoms, containing the encoded fragment's media frames and their timestamps.  <note> After the first media fragment is made available in a streaming session, any fragments that don't contain the same codec private data cause an error to be returned when those different media fragments are loaded. Therefore, the codec private data should not change between fragments in a session. This also means that the session fails if the fragments in a stream change from having only video to having both audio and video. </note> Data retrieved with this action is billable. See <a href="https://aws.amazon.com/kinesis/video-streams/pricing/">Pricing</a> for details. </li> </ul> </li> </ol> <note> The following restrictions apply to MPEG-DASH sessions: <ul> <li> A streaming session URL should not be shared between players. The service might throttle a session if multiple media players are sharing it. For connection limits, see <a href="http://docs.aws.amazon.com/kinesisvideostreams/latest/dg/limits.html">Kinesis Video Streams Limits</a>. </li> <li> A Kinesis video stream can have a maximum of ten active MPEG-DASH streaming sessions. If a new session is created when the maximum number of sessions is already active, the oldest (earliest created) session is closed. The number of active <code>GetMedia</code> connections on a Kinesis video stream does not count against this limit, and the number of active MPEG-DASH sessions does not count against the active <code>GetMedia</code> connection limit. <note> The maximum limits for active HLS and MPEG-DASH streaming sessions are independent of each other.  </note> </li> </ul> </note> You can monitor the amount of data that the media player consumes by monitoring the <code>GetMP4MediaFragment.OutgoingBytes</code> Amazon CloudWatch metric. For information about using CloudWatch to monitor Kinesis Video Streams, see <a href="http://docs.aws.amazon.com/kinesisvideostreams/latest/dg/monitoring.html">Monitoring Kinesis Video Streams</a>. For pricing information, see <a href="https://aws.amazon.com/kinesis/video-streams/pricing/">Amazon Kinesis Video Streams Pricing</a> and <a href="https://aws.amazon.com/pricing/">AWS Pricing</a>. Charges for both HLS sessions and outgoing AWS data apply. For more information about HLS, see <a href="https://developer.apple.com/streaming/">HTTP Live Streaming</a> on the <a href="https://developer.apple.com">Apple Developer site</a>. <important> If an error is thrown after invoking a Kinesis Video Streams archived media API, in addition to the HTTP status code and the response body, it includes the following pieces of information:  <ul> <li>  <code>x-amz-ErrorType</code> HTTP header – contains a more specific error type in addition to what the HTTP status code provides.  </li> <li>  <code>x-amz-RequestId</code> HTTP header – if you want to report an issue to AWS, the support team can better diagnose the problem if given the Request Id. </li> </ul> Both the HTTP status code and the ErrorType header can be utilized to make programmatic decisions about whether errors are retry-able and under what conditions, as well as provide information on what actions the client programmer might need to take in order to successfully try again. For more information, see the <b>Errors</b> section at the bottom of this topic, as well as <a href="https://docs.aws.amazon.com/kinesisvideostreams/latest/dg/CommonErrors.html">Common Errors</a>.  </important>

        :param self:
        :param MaxManifestFragmentResults:  The maximum number of fragments that are returned in the MPEG-DASH manifest. When the <code>PlaybackMode</code> is <code>LIVE</code>, the most recent fragments are returned up to this value. When the <code>PlaybackMode</code> is <code>ON_DEMAND</code>, the oldest fragments are returned, up to this maximum number. When there are a higher number of fragments available in a live MPEG-DASH manifest, video players often buffer content before starting playback. Increasing the buffer size increases the playback latency, but it decreases the likelihood that rebuffering will occur during playback. We recommend that a live MPEG-DASH manifest have a minimum of 3 fragments and a maximum of 10 fragments. The default is 5 fragments if <code>PlaybackMode</code> is <code>LIVE</code> or <code>LIVE_REPLAY</code>, and 1,000 if <code>PlaybackMode</code> is <code>ON_DEMAND</code>.  The maximum value of 1,000 fragments corresponds to more than 16 minutes of video on streams with 1-second fragments, and more than 2 1/2 hours of video on streams with 10-second fragments.<br/>Expires:  The time in seconds until the requested session expires. This value can be between 300 (5 minutes) and 43200 (12 hours). When a session expires, no new calls to <code>GetDashManifest</code>, <code>GetMP4InitFragment</code>, or <code>GetMP4MediaFragment</code> can be made for that session. The default is 300 (5 minutes).<br/>DASHFragmentSelector:  The time range of the requested fragment, and the source of the timestamps. This parameter is required if <code>PlaybackMode</code> is <code>ON_DEMAND</code> or <code>LIVE_REPLAY</code>. This parameter is optional if PlaybackMode is<code/> <code>LIVE</code>. If <code>PlaybackMode</code> is <code>LIVE</code>, the <code>FragmentSelectorType</code> can be set, but the <code>TimestampRange</code> should not be set. If <code>PlaybackMode</code> is <code>ON_DEMAND</code> or <code>LIVE_REPLAY</code>, both <code>FragmentSelectorType</code> and <code>TimestampRange</code> must be set.:<br/>DisplayFragmentNumber:  Fragments are identified in the manifest file based on their sequence number in the session. If DisplayFragmentNumber is set to <code>ALWAYS</code>, the Kinesis Video Streams fragment number is added to each S element in the manifest file with the attribute name “kvs:fn”. These fragment numbers can be used for logging or for use with other APIs (e.g. <code>GetMedia</code> and <code>GetMediaForFragmentList</code>). A custom MPEG-DASH media player is necessary to leverage these this custom attribute. The default value is <code>NEVER</code>.<br/>DisplayFragmentTimestamp:  Per the MPEG-DASH specification, the wall-clock time of fragments in the manifest file can be derived using attributes in the manifest itself. However, typically, MPEG-DASH compatible media players do not properly handle gaps in the media timeline. Kinesis Video Streams adjusts the media timeline in the manifest file to enable playback of media with discontinuities. Therefore, the wall-clock time derived from the manifest file may be inaccurate. If DisplayFragmentTimestamp is set to <code>ALWAYS</code>, the accurate fragment timestamp is added to each S element in the manifest file with the attribute name “kvs:ts”. A custom MPEG-DASH media player is necessary to leverage this custom attribute. The default value is <code>NEVER</code>. When <a>DASHFragmentSelector</a> is <code>SERVER_TIMESTAMP</code>, the timestamps will be the server start timestamps. Similarly, when <a>DASHFragmentSelector</a> is <code>PRODUCER_TIMESTAMP</code>, the timestamps will be the producer start timestamps. <br/>PlaybackMode:  Whether to retrieve live, live replay, or archived, on-demand data. Features of the three types of sessions include the following: <ul> <li>  <b> <code>LIVE</code> </b>: For sessions of this type, the MPEG-DASH manifest is continually updated with the latest fragments as they become available. We recommend that the media player retrieve a new manifest on a one-second interval. When this type of session is played in a media player, the user interface typically displays a "live" notification, with no scrubber control for choosing the position in the playback window to display. <note> In <code>LIVE</code> mode, the newest available fragments are included in an MPEG-DASH manifest, even if there is a gap between fragments (that is, if a fragment is missing). A gap like this might cause a media player to halt or cause a jump in playback. In this mode, fragments are not added to the MPEG-DASH manifest if they are older than the newest fragment in the playlist. If the missing fragment becomes available after a subsequent fragment is added to the manifest, the older fragment is not added, and the gap is not filled. </note> </li> <li>  <b> <code>LIVE_REPLAY</code> </b>: For sessions of this type, the MPEG-DASH manifest is updated similarly to how it is updated for <code>LIVE</code> mode except that it starts by including fragments from a given start time. Instead of fragments being added as they are ingested, fragments are added as the duration of the next fragment elapses. For example, if the fragments in the session are two seconds long, then a new fragment is added to the manifest every two seconds. This mode is useful to be able to start playback from when an event is detected and continue live streaming media that has not yet been ingested as of the time of the session creation. This mode is also useful to stream previously archived media without being limited by the 1,000 fragment limit in the <code>ON_DEMAND</code> mode.  </li> <li>  <b> <code>ON_DEMAND</code> </b>: For sessions of this type, the MPEG-DASH manifest contains all the fragments for the session, up to the number that is specified in <code>MaxMediaPlaylistFragmentResults</code>. The manifest must be retrieved only once for each session. When this type of session is played in a media player, the user interface typically displays a scrubber control for choosing the position in the playback window to display. </li> </ul> In all playback modes, if <code>FragmentSelectorType</code> is <code>PRODUCER_TIMESTAMP</code>, and if there are multiple fragments with the same start timestamp, the fragment that has the larger fragment number (that is, the newer fragment) is included in the MPEG-DASH manifest. The other fragments are not included. Fragments that have different timestamps but have overlapping durations are still included in the MPEG-DASH manifest. This can lead to unexpected behavior in the media player. The default is <code>LIVE</code>.<br/>StreamARN:  The Amazon Resource Name (ARN) of the stream for which to retrieve the MPEG-DASH manifest URL. You must specify either the <code>StreamName</code> or the <code>StreamARN</code>.<br/>StreamName:  The name of the stream for which to retrieve the MPEG-DASH manifest URL. You must specify either the <code>StreamName</code> or the <code>StreamARN</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimestampRange</i> The start and end of the timestamp range for the requested media. This value should not be present if <code>PlaybackType</code> is <code>LIVE</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TimestampRange</b>: The start and end of the timestamp range for the requested media. This value should not be present if <code>PlaybackType</code> is <code>LIVE</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FragmentSelectorType</i> The source of the timestamps for the requested media. When <code>FragmentSelectorType</code> is set to <code>PRODUCER_TIMESTAMP</code> and <a>GetDASHStreamingSessionURLInput$PlaybackMode</a> is <code>ON_DEMAND</code> or <code>LIVE_REPLAY</code>, the first fragment ingested with a producer timestamp within the specified <a>FragmentSelector$TimestampRange</a> is included in the media playlist. In addition, the fragments with producer timestamps within the <code>TimestampRange</code> ingested immediately following the first fragment (up to the <a>GetDASHStreamingSessionURLInput$MaxManifestFragmentResults</a> value) are included.  Fragments that have duplicate producer timestamps are deduplicated. This means that if producers are producing a stream of fragments with producer timestamps that are approximately equal to the true clock time, the MPEG-DASH manifest will contain all of the fragments within the requested timestamp range. If some fragments are ingested within the same time range and very different points in time, only the oldest ingested collection of fragments are returned. When <code>FragmentSelectorType</code> is set to <code>PRODUCER_TIMESTAMP</code> and <a>GetDASHStreamingSessionURLInput$PlaybackMode</a> is <code>LIVE</code>, the producer timestamps are used in the MP4 fragments and for deduplication. But the most recently ingested fragments based on server timestamps are included in the MPEG-DASH manifest. This means that even if fragments ingested in the past have producer timestamps with values now, they are not included in the HLS media playlist. The default is <code>SERVER_TIMESTAMP</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTimestamp</i> The end of the timestamp range for the requested media. This value must be within 3 hours of the specified <code>StartTimestamp</code>, and it must be later than the <code>StartTimestamp</code> value. If <code>FragmentSelectorType</code> for the request is <code>SERVER_TIMESTAMP</code>, this value must be in the past. The <code>EndTimestamp</code> value is required for <code>ON_DEMAND</code> mode, but optional for <code>LIVE_REPLAY</code> mode. If the <code>EndTimestamp</code> is not set for <code>LIVE_REPLAY</code> mode then the session will continue to include newly ingested fragments until the session expires. <note> This value is inclusive. The <code>EndTimestamp</code> is compared to the (starting) timestamp of the fragment. Fragments that start before the <code>EndTimestamp</code> value and continue past it are included in the session. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTimestamp</i> The start of the timestamp range for the requested media. If the <code>DASHTimestampRange</code> value is specified, the <code>StartTimestamp</code> value is required. <note> This value is inclusive. Fragments that start before the <code>StartTimestamp</code> and continue past it are included in the session. If <code>FragmentSelectorType</code> is <code>SERVER_TIMESTAMP</code>, the <code>StartTimestamp</code> must be later than the stream head. </note><br/>
        :return: DASHStreamingSessionURL:  The URL (containing the session token) that a media player can use to retrieve the MPEG-DASH manifest.<br/>        """
        pass

    def get_hls_streaming_session_url(self, StreamName: str=None, StreamARN: str=None, PlaybackMode: str=None, HLSFragmentSelector: HLSFragmentSelector=None, ContainerFormat: str=None, DiscontinuityMode: str=None, DisplayFragmentTimestamp: str=None, Expires: int=None, MaxMediaPlaylistFragmentResults: PageLimit=None) -> GetHLSStreamingSessionURLOutput:
        """Retrieves an HTTP Live Streaming (HLS) URL for the stream. You can then open the URL in a browser or media player to view the stream contents. Both the <code>StreamName</code> and the <code>StreamARN</code> parameters are optional, but you must specify either the <code>StreamName</code> or the <code>StreamARN</code> when invoking this API operation. An Amazon Kinesis video stream has the following requirements for providing data through HLS: <ul> <li> The media must contain h.264 or h.265 encoded video and, optionally, AAC encoded audio. Specifically, the codec id of track 1 should be <code>V_MPEG/ISO/AVC</code> (for h.264) or <code>V_MPEG/ISO/HEVC</code> (for h.265). Optionally, the codec id of track 2 should be <code>A_AAC</code>. </li> <li> Data retention must be greater than 0. </li> <li> The video track of each fragment must contain codec private data in the Advanced Video Coding (AVC) for H.264 format or HEVC for H.265 format (<a href="https://www.iso.org/standard/55980.html">MPEG-4 specification ISO/IEC 14496-15</a>). For information about adapting stream data to a given format, see <a href="http://docs.aws.amazon.com/kinesisvideostreams/latest/dg/producer-reference-nal.html">NAL Adaptation Flags</a>. </li> <li> The audio track (if present) of each fragment must contain codec private data in the AAC format (<a href="https://www.iso.org/standard/43345.html">AAC specification ISO/IEC 13818-7</a>). </li> </ul> Kinesis Video Streams HLS sessions contain fragments in the fragmented MPEG-4 form (also called fMP4 or CMAF) or the MPEG-2 form (also called TS chunks, which the HLS specification also supports). For more information about HLS fragment types, see the <a href="https://tools.ietf.org/html/draft-pantos-http-live-streaming-23">HLS specification</a>. The following procedure shows how to use HLS with Kinesis Video Streams: <ol> <li> Get an endpoint using <a href="http://docs.aws.amazon.com/kinesisvideostreams/latest/dg/API_GetDataEndpoint.html">GetDataEndpoint</a>, specifying <code>GET_HLS_STREAMING_SESSION_URL</code> for the <code>APIName</code> parameter. </li> <li> Retrieve the HLS URL using <code>GetHLSStreamingSessionURL</code>. Kinesis Video Streams creates an HLS streaming session to be used for accessing content in a stream using the HLS protocol. <code>GetHLSStreamingSessionURL</code> returns an authenticated URL (that includes an encrypted session token) for the session's HLS <i>master playlist</i> (the root resource needed for streaming with HLS). <note> Don't share or store this token where an unauthorized entity could access it. The token provides access to the content of the stream. Safeguard the token with the same measures that you would use with your AWS credentials. </note> The media that is made available through the playlist consists only of the requested stream, time range, and format. No other media data (such as frames outside the requested window or alternate bitrates) is made available. </li> <li> Provide the URL (containing the encrypted session token) for the HLS master playlist to a media player that supports the HLS protocol. Kinesis Video Streams makes the HLS media playlist, initialization fragment, and media fragments available through the master playlist URL. The initialization fragment contains the codec private data for the stream, and other data needed to set up the video or audio decoder and renderer. The media fragments contain H.264-encoded video frames or AAC-encoded audio samples. </li> <li> The media player receives the authenticated URL and requests stream metadata and media data normally. When the media player requests data, it calls the following actions: <ul> <li>  <b>GetHLSMasterPlaylist:</b> Retrieves an HLS master playlist, which contains a URL for the <code>GetHLSMediaPlaylist</code> action for each track, and additional metadata for the media player, including estimated bitrate and resolution. </li> <li>  <b>GetHLSMediaPlaylist:</b> Retrieves an HLS media playlist, which contains a URL to access the MP4 initialization fragment with the <code>GetMP4InitFragment</code> action, and URLs to access the MP4 media fragments with the <code>GetMP4MediaFragment</code> actions. The HLS media playlist also contains metadata about the stream that the player needs to play it, such as whether the <code>PlaybackMode</code> is <code>LIVE</code> or <code>ON_DEMAND</code>. The HLS media playlist is typically static for sessions with a <code>PlaybackType</code> of <code>ON_DEMAND</code>. The HLS media playlist is continually updated with new fragments for sessions with a <code>PlaybackType</code> of <code>LIVE</code>. There is a distinct HLS media playlist for the video track and the audio track (if applicable) that contains MP4 media URLs for the specific track.  </li> <li>  <b>GetMP4InitFragment:</b> Retrieves the MP4 initialization fragment. The media player typically loads the initialization fragment before loading any media fragments. This fragment contains the "<code>fytp</code>" and "<code>moov</code>" MP4 atoms, and the child atoms that are needed to initialize the media player decoder. The initialization fragment does not correspond to a fragment in a Kinesis video stream. It contains only the codec private data for the stream and respective track, which the media player needs to decode the media frames. </li> <li>  <b>GetMP4MediaFragment:</b> Retrieves MP4 media fragments. These fragments contain the "<code>moof</code>" and "<code>mdat</code>" MP4 atoms and their child atoms, containing the encoded fragment's media frames and their timestamps.  <note> After the first media fragment is made available in a streaming session, any fragments that don't contain the same codec private data cause an error to be returned when those different media fragments are loaded. Therefore, the codec private data should not change between fragments in a session. This also means that the session fails if the fragments in a stream change from having only video to having both audio and video. </note> Data retrieved with this action is billable. See <a href="https://aws.amazon.com/kinesis/video-streams/pricing/">Pricing</a> for details. </li> <li>  <b>GetTSFragment:</b> Retrieves MPEG TS fragments containing both initialization and media data for all tracks in the stream. <note> If the <code>ContainerFormat</code> is <code>MPEG_TS</code>, this API is used instead of <code>GetMP4InitFragment</code> and <code>GetMP4MediaFragment</code> to retrieve stream media. </note> Data retrieved with this action is billable. For more information, see <a href="https://aws.amazon.com/kinesis/video-streams/pricing/">Kinesis Video Streams pricing</a>. </li> </ul> </li> </ol> <note> The following restrictions apply to HLS sessions: <ul> <li> A streaming session URL should not be shared between players. The service might throttle a session if multiple media players are sharing it. For connection limits, see <a href="http://docs.aws.amazon.com/kinesisvideostreams/latest/dg/limits.html">Kinesis Video Streams Limits</a>. </li> <li> A Kinesis video stream can have a maximum of ten active HLS streaming sessions. If a new session is created when the maximum number of sessions is already active, the oldest (earliest created) session is closed. The number of active <code>GetMedia</code> connections on a Kinesis video stream does not count against this limit, and the number of active HLS sessions does not count against the active <code>GetMedia</code> connection limit. <note> The maximum limits for active HLS and MPEG-DASH streaming sessions are independent of each other. </note> </li> </ul> </note> You can monitor the amount of data that the media player consumes by monitoring the <code>GetMP4MediaFragment.OutgoingBytes</code> Amazon CloudWatch metric. For information about using CloudWatch to monitor Kinesis Video Streams, see <a href="http://docs.aws.amazon.com/kinesisvideostreams/latest/dg/monitoring.html">Monitoring Kinesis Video Streams</a>. For pricing information, see <a href="https://aws.amazon.com/kinesis/video-streams/pricing/">Amazon Kinesis Video Streams Pricing</a> and <a href="https://aws.amazon.com/pricing/">AWS Pricing</a>. Charges for both HLS sessions and outgoing AWS data apply. For more information about HLS, see <a href="https://developer.apple.com/streaming/">HTTP Live Streaming</a> on the <a href="https://developer.apple.com">Apple Developer site</a>. <important> If an error is thrown after invoking a Kinesis Video Streams archived media API, in addition to the HTTP status code and the response body, it includes the following pieces of information:  <ul> <li>  <code>x-amz-ErrorType</code> HTTP header – contains a more specific error type in addition to what the HTTP status code provides.  </li> <li>  <code>x-amz-RequestId</code> HTTP header – if you want to report an issue to AWS, the support team can better diagnose the problem if given the Request Id. </li> </ul> Both the HTTP status code and the ErrorType header can be utilized to make programmatic decisions about whether errors are retry-able and under what conditions, as well as provide information on what actions the client programmer might need to take in order to successfully try again. For more information, see the <b>Errors</b> section at the bottom of this topic, as well as <a href="https://docs.aws.amazon.com/kinesisvideostreams/latest/dg/CommonErrors.html">Common Errors</a>.  </important>

        :param self:
        :param MaxMediaPlaylistFragmentResults:  The maximum number of fragments that are returned in the HLS media playlists. When the <code>PlaybackMode</code> is <code>LIVE</code>, the most recent fragments are returned up to this value. When the <code>PlaybackMode</code> is <code>ON_DEMAND</code>, the oldest fragments are returned, up to this maximum number. When there are a higher number of fragments available in a live HLS media playlist, video players often buffer content before starting playback. Increasing the buffer size increases the playback latency, but it decreases the likelihood that rebuffering will occur during playback. We recommend that a live HLS media playlist have a minimum of 3 fragments and a maximum of 10 fragments. The default is 5 fragments if <code>PlaybackMode</code> is <code>LIVE</code> or <code>LIVE_REPLAY</code>, and 1,000 if <code>PlaybackMode</code> is <code>ON_DEMAND</code>.  The maximum value of 1,000 fragments corresponds to more than 16 minutes of video on streams with 1-second fragments, and more than 2 1/2 hours of video on streams with 10-second fragments.<br/>Expires:  The time in seconds until the requested session expires. This value can be between 300 (5 minutes) and 43200 (12 hours). When a session expires, no new calls to <code>GetHLSMasterPlaylist</code>, <code>GetHLSMediaPlaylist</code>, <code>GetMP4InitFragment</code>, <code>GetMP4MediaFragment</code>, or <code>GetTSFragment</code> can be made for that session. The default is 300 (5 minutes).<br/>DisplayFragmentTimestamp:  Specifies when the fragment start timestamps should be included in the HLS media playlist. Typically, media players report the playhead position as a time relative to the start of the first fragment in the playback session. However, when the start timestamps are included in the HLS media playlist, some media players might report the current playhead as an absolute time based on the fragment timestamps. This can be useful for creating a playback experience that shows viewers the wall-clock time of the media. The default is <code>NEVER</code>. When <a>HLSFragmentSelector</a> is <code>SERVER_TIMESTAMP</code>, the timestamps will be the server start timestamps. Similarly, when <a>HLSFragmentSelector</a> is <code>PRODUCER_TIMESTAMP</code>, the timestamps will be the producer start timestamps. <br/>DiscontinuityMode:  Specifies when flags marking discontinuities between fragments will be added to the media playlists. The default is <code>ALWAYS</code> when <a>HLSFragmentSelector</a> is <code>SERVER_TIMESTAMP</code>, and <code>NEVER</code> when it is <code>PRODUCER_TIMESTAMP</code>. Media players typically build a timeline of media content to play, based on the timestamps of each fragment. This means that if there is any overlap between fragments (as is typical if <a>HLSFragmentSelector</a> is <code>SERVER_TIMESTAMP</code>), the media player timeline has small gaps between fragments in some places, and overwrites frames in other places. When there are discontinuity flags between fragments, the media player is expected to reset the timeline, resulting in the fragment being played immediately after the previous fragment. We recommend that you always have discontinuity flags between fragments if the fragment timestamps are not accurate or if fragments might be missing. You should not place discontinuity flags between fragments for the player timeline to accurately map to the producer timestamps.<br/>ContainerFormat:  Specifies which format should be used for packaging the media. Specifying the <code>FRAGMENTED_MP4</code> container format packages the media into MP4 fragments (fMP4 or CMAF). This is the recommended packaging because there is minimal packaging overhead. The other container format option is <code>MPEG_TS</code>. HLS has supported MPEG TS chunks since it was released and is sometimes the only supported packaging on older HLS players. MPEG TS typically has a 5-25 percent packaging overhead. This means MPEG TS typically requires 5-25 percent more bandwidth and cost than fMP4. The default is <code>FRAGMENTED_MP4</code>.<br/>HLSFragmentSelector:  The time range of the requested fragment, and the source of the timestamps. This parameter is required if <code>PlaybackMode</code> is <code>ON_DEMAND</code> or <code>LIVE_REPLAY</code>. This parameter is optional if PlaybackMode is<code/> <code>LIVE</code>. If <code>PlaybackMode</code> is <code>LIVE</code>, the <code>FragmentSelectorType</code> can be set, but the <code>TimestampRange</code> should not be set. If <code>PlaybackMode</code> is <code>ON_DEMAND</code> or <code>LIVE_REPLAY</code>, both <code>FragmentSelectorType</code> and <code>TimestampRange</code> must be set.:<br/>PlaybackMode:  Whether to retrieve live, live replay, or archived, on-demand data. Features of the three types of sessions include the following: <ul> <li>  <b> <code>LIVE</code> </b>: For sessions of this type, the HLS media playlist is continually updated with the latest fragments as they become available. We recommend that the media player retrieve a new playlist on a one-second interval. When this type of session is played in a media player, the user interface typically displays a "live" notification, with no scrubber control for choosing the position in the playback window to display. <note> In <code>LIVE</code> mode, the newest available fragments are included in an HLS media playlist, even if there is a gap between fragments (that is, if a fragment is missing). A gap like this might cause a media player to halt or cause a jump in playback. In this mode, fragments are not added to the HLS media playlist if they are older than the newest fragment in the playlist. If the missing fragment becomes available after a subsequent fragment is added to the playlist, the older fragment is not added, and the gap is not filled. </note> </li> <li>  <b> <code>LIVE_REPLAY</code> </b>: For sessions of this type, the HLS media playlist is updated similarly to how it is updated for <code>LIVE</code> mode except that it starts by including fragments from a given start time. Instead of fragments being added as they are ingested, fragments are added as the duration of the next fragment elapses. For example, if the fragments in the session are two seconds long, then a new fragment is added to the media playlist every two seconds. This mode is useful to be able to start playback from when an event is detected and continue live streaming media that has not yet been ingested as of the time of the session creation. This mode is also useful to stream previously archived media without being limited by the 1,000 fragment limit in the <code>ON_DEMAND</code> mode.  </li> <li>  <b> <code>ON_DEMAND</code> </b>: For sessions of this type, the HLS media playlist contains all the fragments for the session, up to the number that is specified in <code>MaxMediaPlaylistFragmentResults</code>. The playlist must be retrieved only once for each session. When this type of session is played in a media player, the user interface typically displays a scrubber control for choosing the position in the playback window to display. </li> </ul> In all playback modes, if <code>FragmentSelectorType</code> is <code>PRODUCER_TIMESTAMP</code>, and if there are multiple fragments with the same start timestamp, the fragment that has the larger fragment number (that is, the newer fragment) is included in the HLS media playlist. The other fragments are not included. Fragments that have different timestamps but have overlapping durations are still included in the HLS media playlist. This can lead to unexpected behavior in the media player. The default is <code>LIVE</code>.<br/>StreamARN:  The Amazon Resource Name (ARN) of the stream for which to retrieve the HLS master playlist URL. You must specify either the <code>StreamName</code> or the <code>StreamARN</code>.<br/>StreamName:  The name of the stream for which to retrieve the HLS master playlist URL. You must specify either the <code>StreamName</code> or the <code>StreamARN</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimestampRange</i> The start and end of the timestamp range for the requested media. This value should not be present if <code>PlaybackType</code> is <code>LIVE</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TimestampRange</b>: The start and end of the timestamp range for the requested media. This value should not be present if <code>PlaybackType</code> is <code>LIVE</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FragmentSelectorType</i> The source of the timestamps for the requested media. When <code>FragmentSelectorType</code> is set to <code>PRODUCER_TIMESTAMP</code> and <a>GetHLSStreamingSessionURLInput$PlaybackMode</a> is <code>ON_DEMAND</code> or <code>LIVE_REPLAY</code>, the first fragment ingested with a producer timestamp within the specified <a>FragmentSelector$TimestampRange</a> is included in the media playlist. In addition, the fragments with producer timestamps within the <code>TimestampRange</code> ingested immediately following the first fragment (up to the <a>GetHLSStreamingSessionURLInput$MaxMediaPlaylistFragmentResults</a> value) are included.  Fragments that have duplicate producer timestamps are deduplicated. This means that if producers are producing a stream of fragments with producer timestamps that are approximately equal to the true clock time, the HLS media playlists will contain all of the fragments within the requested timestamp range. If some fragments are ingested within the same time range and very different points in time, only the oldest ingested collection of fragments are returned. When <code>FragmentSelectorType</code> is set to <code>PRODUCER_TIMESTAMP</code> and <a>GetHLSStreamingSessionURLInput$PlaybackMode</a> is <code>LIVE</code>, the producer timestamps are used in the MP4 fragments and for deduplication. But the most recently ingested fragments based on server timestamps are included in the HLS media playlist. This means that even if fragments ingested in the past have producer timestamps with values now, they are not included in the HLS media playlist. The default is <code>SERVER_TIMESTAMP</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTimestamp</i> The end of the timestamp range for the requested media. This value must be within 3 hours of the specified <code>StartTimestamp</code>, and it must be later than the <code>StartTimestamp</code> value. If <code>FragmentSelectorType</code> for the request is <code>SERVER_TIMESTAMP</code>, this value must be in the past. The <code>EndTimestamp</code> value is required for <code>ON_DEMAND</code> mode, but optional for <code>LIVE_REPLAY</code> mode. If the <code>EndTimestamp</code> is not set for <code>LIVE_REPLAY</code> mode then the session will continue to include newly ingested fragments until the session expires. <note> This value is inclusive. The <code>EndTimestamp</code> is compared to the (starting) timestamp of the fragment. Fragments that start before the <code>EndTimestamp</code> value and continue past it are included in the session. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTimestamp</i> The start of the timestamp range for the requested media. If the <code>HLSTimestampRange</code> value is specified, the <code>StartTimestamp</code> value is required. <note> This value is inclusive. Fragments that start before the <code>StartTimestamp</code> and continue past it are included in the session. If <code>FragmentSelectorType</code> is <code>SERVER_TIMESTAMP</code>, the <code>StartTimestamp</code> must be later than the stream head. </note><br/>
        :return: HLSStreamingSessionURL:  The URL (containing the session token) that a media player can use to retrieve the HLS master playlist.<br/>        """
        pass

    def get_media_for_fragment_list(self, Fragments: List[FragmentNumberString], StreamName: str) -> GetMediaForFragmentListOutput:
        """Gets media for a list of fragments (specified by fragment number) from the archived data in an Amazon Kinesis video stream. <note> You must first call the <code>GetDataEndpoint</code> API to get an endpoint. Then send the <code>GetMediaForFragmentList</code> requests to this endpoint using the <a href="https://docs.aws.amazon.com/cli/latest/reference/">--endpoint-url parameter</a>.  </note> The following limits apply when using the <code>GetMediaForFragmentList</code> API: <ul> <li> A client can call <code>GetMediaForFragmentList</code> up to five times per second per stream.  </li> <li> Kinesis Video Streams sends media data at a rate of up to 25 megabytes per second (or 200 megabits per second) during a <code>GetMediaForFragmentList</code> session.  </li> </ul> <important> If an error is thrown after invoking a Kinesis Video Streams archived media API, in addition to the HTTP status code and the response body, it includes the following pieces of information:  <ul> <li>  <code>x-amz-ErrorType</code> HTTP header – contains a more specific error type in addition to what the HTTP status code provides.  </li> <li>  <code>x-amz-RequestId</code> HTTP header – if you want to report an issue to AWS, the support team can better diagnose the problem if given the Request Id. </li> </ul> Both the HTTP status code and the ErrorType header can be utilized to make programmatic decisions about whether errors are retry-able and under what conditions, as well as provide information on what actions the client programmer might need to take in order to successfully try again. For more information, see the <b>Errors</b> section at the bottom of this topic, as well as <a href="https://docs.aws.amazon.com/kinesisvideostreams/latest/dg/CommonErrors.html">Common Errors</a>.  </important>

        :param self:
        :param Fragments:  A list of the numbers of fragments for which to retrieve media. You retrieve these values with <a>ListFragments</a>.<br/>StreamName:  The name of the stream from which to retrieve fragment media.<br/>
        :return: Payload:  The payload that Kinesis Video Streams returns is a sequence of chunks from the specified stream. For information about the chunks, see <a href="http://docs.aws.amazon.com/kinesisvideostreams/latest/dg/API_dataplane_PutMedia.html">PutMedia</a>. The chunks that Kinesis Video Streams returns in the <code>GetMediaForFragmentList</code> call also include the following additional Matroska (MKV) tags:  <ul> <li> AWS_KINESISVIDEO_FRAGMENT_NUMBER - Fragment number returned in the chunk. </li> <li> AWS_KINESISVIDEO_SERVER_SIDE_TIMESTAMP - Server-side timestamp of the fragment. </li> <li> AWS_KINESISVIDEO_PRODUCER_SIDE_TIMESTAMP - Producer-side timestamp of the fragment. </li> </ul> The following tags will be included if an exception occurs: <ul> <li> AWS_KINESISVIDEO_FRAGMENT_NUMBER - The number of the fragment that threw the exception </li> <li> AWS_KINESISVIDEO_EXCEPTION_ERROR_CODE - The integer code of the exception </li> <li> AWS_KINESISVIDEO_EXCEPTION_MESSAGE - A text description of the exception </li> </ul><br/>ContentType:  The content type of the requested media.<br/>        """
        pass

    def list_fragments(self, StreamName: str, MaxResults: PageLimit=None, NextToken: str=None, FragmentSelector: FragmentSelector=None) -> ListFragmentsOutput:
        """Returns a list of <a>Fragment</a> objects from the specified stream and timestamp range within the archived data. Listing fragments is eventually consistent. This means that even if the producer receives an acknowledgment that a fragment is persisted, the result might not be returned immediately from a request to <code>ListFragments</code>. However, results are typically available in less than one second. <note> You must first call the <code>GetDataEndpoint</code> API to get an endpoint. Then send the <code>ListFragments</code> requests to this endpoint using the <a href="https://docs.aws.amazon.com/cli/latest/reference/">--endpoint-url parameter</a>.  </note> <important> If an error is thrown after invoking a Kinesis Video Streams archived media API, in addition to the HTTP status code and the response body, it includes the following pieces of information:  <ul> <li>  <code>x-amz-ErrorType</code> HTTP header – contains a more specific error type in addition to what the HTTP status code provides.  </li> <li>  <code>x-amz-RequestId</code> HTTP header – if you want to report an issue to AWS, the support team can better diagnose the problem if given the Request Id. </li> </ul> Both the HTTP status code and the ErrorType header can be utilized to make programmatic decisions about whether errors are retry-able and under what conditions, as well as provide information on what actions the client programmer might need to take in order to successfully try again. For more information, see the <b>Errors</b> section at the bottom of this topic, as well as <a href="https://docs.aws.amazon.com/kinesisvideostreams/latest/dg/CommonErrors.html">Common Errors</a>.  </important>

        :param self:
        :param FragmentSelector:  Describes the timestamp range and timestamp origin for the range of fragments to return.:<br/>NextToken:  A token to specify where to start paginating. This is the <a>ListFragmentsOutput$NextToken</a> from a previously truncated response.<br/>MaxResults:  The total number of fragments to return. If the total number of fragments available is more than the value specified in <code>max-results</code>, then a <a>ListFragmentsOutput$NextToken</a> is provided in the output that you can use to resume pagination.<br/>StreamName:  The name of the stream from which to retrieve a fragment list.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimestampRange</i> The range of timestamps to return.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TimestampRange</b>: The range of timestamps to return.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FragmentSelectorType</i> The origin of the timestamps to use (Server or Producer).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTimestamp</i> The ending timestamp in the range of timestamps for which to return fragments.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTimestamp</i> The starting timestamp in the range of timestamps for which to return fragments.<br/>
        :return: NextToken:  If the returned list is truncated, the operation returns this token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>Fragments:  A list of archived <a>Fragment</a> objects from the stream that meet the selector criteria. Results are in no specific order, even across pages.<br/>        """
        pass



class KinesisVideoMedia(BaseClient):
    """<p/>"""




    class GetMediaInput(Mapping):
        pass
    
    class GetMediaOutput(Mapping):
        pass
    
    class StartSelector(Mapping):
        pass
    
    def get_media(self, StartSelector: StartSelector, StreamName: str=None, StreamARN: str=None) -> GetMediaOutput:
        """ Use this API to retrieve media content from a Kinesis video stream. In the request, you identify the stream name or stream Amazon Resource Name (ARN), and the starting chunk. Kinesis Video Streams then returns a stream of chunks in order by fragment number. <note> You must first call the <code>GetDataEndpoint</code> API to get an endpoint. Then send the <code>GetMedia</code> requests to this endpoint using the <a href="https://docs.aws.amazon.com/cli/latest/reference/">--endpoint-url parameter</a>.  </note> When you put media data (fragments) on a stream, Kinesis Video Streams stores each incoming fragment and related metadata in what is called a "chunk." For more information, see <a href="https://docs.aws.amazon.com/kinesisvideostreams/latest/dg/API_dataplane_PutMedia.html">PutMedia</a>. The <code>GetMedia</code> API returns a stream of these chunks starting from the chunk that you specify in the request.  The following limits apply when using the <code>GetMedia</code> API: <ul> <li> A client can call <code>GetMedia</code> up to five times per second per stream.  </li> <li> Kinesis Video Streams sends media data at a rate of up to 25 megabytes per second (or 200 megabits per second) during a <code>GetMedia</code> session.  </li> </ul> <note> If an error is thrown after invoking a Kinesis Video Streams media API, in addition to the HTTP status code and the response body, it includes the following pieces of information:  <ul> <li>  <code>x-amz-ErrorType</code> HTTP header – contains a more specific error type in addition to what the HTTP status code provides.  </li> <li>  <code>x-amz-RequestId</code> HTTP header – if you want to report an issue to AWS, the support team can better diagnose the problem if given the Request Id. </li> </ul> Both the HTTP status code and the ErrorType header can be utilized to make programmatic decisions about whether errors are retry-able and under what conditions, as well as provide information on what actions the client programmer might need to take in order to successfully try again. For more information, see the <b>Errors</b> section at the bottom of this topic, as well as <a href="https://docs.aws.amazon.com/kinesisvideostreams/latest/dg/CommonErrors.html">Common Errors</a>.  </note>

        :param self:
        :param StartSelector:  Identifies the starting chunk to get from the specified stream. :<br/>StreamARN:  The ARN of the stream from where you want to get the media content. If you don't specify the <code>streamARN</code>, you must specify the <code>streamName</code>.<br/>StreamName:  The Kinesis video stream name from where you want to get the media content. If you don't specify the <code>streamName</code>, you must specify the <code>streamARN</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContinuationToken</i> Continuation token that Kinesis Video Streams returned in the previous <code>GetMedia</code> response. The <code>GetMedia</code> API then starts with the chunk identified by the continuation token.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTimestamp</i> A timestamp value. This value is required if you choose the PRODUCER_TIMESTAMP or the SERVER_TIMESTAMP as the <code>startSelectorType</code>. The <code>GetMedia</code> API then starts with the chunk containing the fragment that has the specified timestamp.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AfterFragmentNumber</i> Specifies the fragment number from where you want the <code>GetMedia</code> API to start returning the fragments. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartSelectorType</i> Identifies the fragment on the Kinesis video stream where you want to start getting the data from. <ul> <li> NOW - Start with the latest chunk on the stream. </li> <li> EARLIEST - Start with earliest available chunk on the stream. </li> <li> FRAGMENT_NUMBER - Start with the chunk after a specific fragment. You must also specify the <code>AfterFragmentNumber</code> parameter. </li> <li> PRODUCER_TIMESTAMP or SERVER_TIMESTAMP - Start with the chunk containing a fragment with the specified producer or server timestamp. You specify the timestamp by adding <code>StartTimestamp</code>. </li> <li>  CONTINUATION_TOKEN - Read using the specified continuation token.  </li> </ul> <note> If you choose the NOW, EARLIEST, or CONTINUATION_TOKEN as the <code>startSelectorType</code>, you don't provide any additional information in the <code>startSelector</code>. </note><br/>
        :return: Payload:   The payload Kinesis Video Streams returns is a sequence of chunks from the specified stream. For information about the chunks, see . The chunks that Kinesis Video Streams returns in the <code>GetMedia</code> call also include the following additional Matroska (MKV) tags:  <ul> <li> AWS_KINESISVIDEO_CONTINUATION_TOKEN (UTF-8 string) - In the event your <code>GetMedia</code> call terminates, you can use this continuation token in your next request to get the next chunk where the last request terminated. </li> <li> AWS_KINESISVIDEO_MILLIS_BEHIND_NOW (UTF-8 string) - Client applications can use this tag value to determine how far behind the chunk returned in the response is from the latest chunk on the stream.  </li> <li> AWS_KINESISVIDEO_FRAGMENT_NUMBER - Fragment number returned in the chunk. </li> <li> AWS_KINESISVIDEO_SERVER_TIMESTAMP - Server timestamp of the fragment. </li> <li> AWS_KINESISVIDEO_PRODUCER_TIMESTAMP - Producer timestamp of the fragment. </li> </ul> The following tags will be present if an error occurs: <ul> <li> AWS_KINESISVIDEO_ERROR_CODE - String description of an error that caused GetMedia to stop. </li> <li> AWS_KINESISVIDEO_ERROR_ID: Integer code of the error. </li> </ul> The error codes are as follows: <ul> <li> 3002 - Error writing to the stream </li> <li> 4000 - Requested fragment is not found </li> <li> 4500 - Access denied for the stream's KMS key </li> <li> 4501 - Stream's KMS key is disabled </li> <li> 4502 - Validation error on the stream's KMS key </li> <li> 4503 - KMS key specified in the stream is unavailable </li> <li> 4504 - Invalid usage of the KMS key specified in the stream </li> <li> 4505 - Invalid state of the KMS key specified in the stream </li> <li> 4506 - Unable to find the KMS key specified in the stream </li> <li> 5000 - Internal error </li> </ul><br/>ContentType:  The content type of the requested media.<br/>        """
        pass



class KinesisAnalytics(BaseClient):
    """<fullname>Amazon Kinesis Analytics</fullname>  <b>Overview</b>  <note> This documentation is for version 1 of the Amazon Kinesis Data Analytics API, which only supports SQL applications. Version 2 of the API supports SQL and Java applications. For more information about version 2, see <a href="/kinesisanalytics/latest/apiv2/Welcome.html">Amazon Kinesis Data Analytics API V2 Documentation</a>. </note> This is the <i>Amazon Kinesis Analytics v1 API Reference</i>. The Amazon Kinesis Analytics Developer Guide provides additional information. """




    class AddApplicationCloudWatchLoggingOptionRequest(Mapping):
        pass
    
    class AddApplicationCloudWatchLoggingOptionResponse(Mapping):
        pass
    
    class ApplicationVersionId(object):
        pass
    
    class CloudWatchLoggingOption(Mapping):
        pass
    
    class AddApplicationInputRequest(Mapping):
        pass
    
    class AddApplicationInputResponse(Mapping):
        pass
    
    class Input(Mapping):
        pass
    
    class AddApplicationInputProcessingConfigurationRequest(Mapping):
        pass
    
    class AddApplicationInputProcessingConfigurationResponse(Mapping):
        pass
    
    class InputProcessingConfiguration(Mapping):
        pass
    
    class AddApplicationOutputRequest(Mapping):
        pass
    
    class AddApplicationOutputResponse(Mapping):
        pass
    
    class Output(Mapping):
        pass
    
    class AddApplicationReferenceDataSourceRequest(Mapping):
        pass
    
    class AddApplicationReferenceDataSourceResponse(Mapping):
        pass
    
    class ReferenceDataSource(Mapping):
        pass
    
    class CreateApplicationRequest(Mapping):
        pass
    
    class CreateApplicationResponse(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class DeleteApplicationRequest(Mapping):
        pass
    
    class DeleteApplicationResponse(Mapping):
        pass
    
    class Timestamp(object):
        pass
    
    class DeleteApplicationCloudWatchLoggingOptionRequest(Mapping):
        pass
    
    class DeleteApplicationCloudWatchLoggingOptionResponse(Mapping):
        pass
    
    class DeleteApplicationInputProcessingConfigurationRequest(Mapping):
        pass
    
    class DeleteApplicationInputProcessingConfigurationResponse(Mapping):
        pass
    
    class DeleteApplicationOutputRequest(Mapping):
        pass
    
    class DeleteApplicationOutputResponse(Mapping):
        pass
    
    class DeleteApplicationReferenceDataSourceRequest(Mapping):
        pass
    
    class DeleteApplicationReferenceDataSourceResponse(Mapping):
        pass
    
    class DescribeApplicationRequest(Mapping):
        pass
    
    class DescribeApplicationResponse(Mapping):
        pass
    
    class DiscoverInputSchemaRequest(Mapping):
        pass
    
    class DiscoverInputSchemaResponse(Mapping):
        pass
    
    class InputStartingPositionConfiguration(Mapping):
        pass
    
    class S3Configuration(Mapping):
        pass
    
    class ListApplicationsRequest(Mapping):
        pass
    
    class ListApplicationsResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class StartApplicationRequest(Mapping):
        pass
    
    class StartApplicationResponse(Mapping):
        pass
    
    class InputConfiguration(Mapping):
        pass
    
    class StopApplicationRequest(Mapping):
        pass
    
    class StopApplicationResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateApplicationRequest(Mapping):
        pass
    
    class UpdateApplicationResponse(Mapping):
        pass
    
    class ApplicationUpdate(Mapping):
        pass
    
    def add_application_cloud_watch_logging_option(self, CloudWatchLoggingOption: CloudWatchLoggingOption, CurrentApplicationVersionId: ApplicationVersionId, ApplicationName: str) -> AddApplicationCloudWatchLoggingOptionResponse:
        """<note> This documentation is for version 1 of the Amazon Kinesis Data Analytics API, which only supports SQL applications. Version 2 of the API supports SQL and Java applications. For more information about version 2, see <a href="/kinesisanalytics/latest/apiv2/Welcome.html">Amazon Kinesis Data Analytics API V2 Documentation</a>. </note> Adds a CloudWatch log stream to monitor application configuration errors. For more information about using CloudWatch log streams with Amazon Kinesis Analytics applications, see <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/cloudwatch-logs.html">Working with Amazon CloudWatch Logs</a>.

        :param self:
        :param CloudWatchLoggingOption:  Provides the CloudWatch log stream Amazon Resource Name (ARN) and the IAM role ARN. Note: To write application messages to CloudWatch, the IAM role that is used must have the <code>PutLogEvents</code> policy action enabled.:<br/>CurrentApplicationVersionId:  The version ID of the Kinesis Analytics application.<br/>ApplicationName:  The Kinesis Analytics application name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> IAM ARN of the role to use to send application messages. Note: To write application messages to CloudWatch, the IAM role that is used must have the <code>PutLogEvents</code> policy action enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogStreamARN</i> ARN of the CloudWatch log to receive application messages.<br/>
        :return:         """
        pass

    def add_application_input(self, Input: Input, CurrentApplicationVersionId: ApplicationVersionId, ApplicationName: str) -> AddApplicationInputResponse:
        """<note> This documentation is for version 1 of the Amazon Kinesis Data Analytics API, which only supports SQL applications. Version 2 of the API supports SQL and Java applications. For more information about version 2, see <a href="/kinesisanalytics/latest/apiv2/Welcome.html">Amazon Kinesis Data Analytics API V2 Documentation</a>. </note>  Adds a streaming source to your Amazon Kinesis application. For conceptual information, see <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/how-it-works-input.html">Configuring Application Input</a>.  You can add a streaming source either when you create an application or you can use this operation to add a streaming source after you create an application. For more information, see <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_CreateApplication.html">CreateApplication</a>. Any configuration update, including adding a streaming source using this operation, results in a new version of the application. You can use the <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_DescribeApplication.html">DescribeApplication</a> operation to find the current application version.  This operation requires permissions to perform the <code>kinesisanalytics:AddApplicationInput</code> action.

        :param self:
        :param Input:  The <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_Input.html">Input</a> to add.:<br/>CurrentApplicationVersionId:  Current version of your Amazon Kinesis Analytics application. You can use the <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_DescribeApplication.html">DescribeApplication</a> operation to find the current application version.<br/>ApplicationName:  Name of your existing Amazon Kinesis Analytics application to which you want to add the streaming source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputSchema</i> Describes the format of the data in the streaming source, and how each data element maps to corresponding columns in the in-application stream that is being created. Also used to describe the format of the reference data source.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputSchema</b>: Describes the format of the data in the streaming source, and how each data element maps to corresponding columns in the in-application stream that is being created. Also used to describe the format of the reference data source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputParallelism</i> Describes the number of in-application streams to create.  Data from your source is routed to these in-application input streams.  (see <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/how-it-works-input.html">Configuring Application Input</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputParallelism</b>: Describes the number of in-application streams to create.  Data from your source is routed to these in-application input streams.  (see <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/how-it-works-input.html">Configuring Application Input</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KinesisFirehoseInput</i> If the streaming source is an Amazon Kinesis Firehose delivery stream, identifies the delivery stream's ARN and an IAM role that enables Amazon Kinesis Analytics to access the stream on your behalf. Note: Either <code>KinesisStreamsInput</code> or <code>KinesisFirehoseInput</code> is required.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KinesisFirehoseInput</b>: If the streaming source is an Amazon Kinesis Firehose delivery stream, identifies the delivery stream's ARN and an IAM role that enables Amazon Kinesis Analytics to access the stream on your behalf. Note: Either <code>KinesisStreamsInput</code> or <code>KinesisFirehoseInput</code> is required.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KinesisStreamsInput</i> If the streaming source is an Amazon Kinesis stream, identifies the stream's Amazon Resource Name (ARN) and an IAM role that enables Amazon Kinesis Analytics to access the stream on your behalf. Note: Either <code>KinesisStreamsInput</code> or <code>KinesisFirehoseInput</code> is required.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KinesisStreamsInput</b>: If the streaming source is an Amazon Kinesis stream, identifies the stream's Amazon Resource Name (ARN) and an IAM role that enables Amazon Kinesis Analytics to access the stream on your behalf. Note: Either <code>KinesisStreamsInput</code> or <code>KinesisFirehoseInput</code> is required.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputProcessingConfiguration</i> The <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_InputProcessingConfiguration.html">InputProcessingConfiguration</a> for the input. An input processor transforms records as they are received from the stream, before the application's SQL code executes. Currently, the only input processing configuration available is <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_InputLambdaProcessor.html">InputLambdaProcessor</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputProcessingConfiguration</b>: The <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_InputProcessingConfiguration.html">InputProcessingConfiguration</a> for the input. An input processor transforms records as they are received from the stream, before the application's SQL code executes. Currently, the only input processing configuration available is <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_InputLambdaProcessor.html">InputLambdaProcessor</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NamePrefix</i> Name prefix to use when creating an in-application stream. Suppose that you specify a prefix "MyInApplicationStream." Amazon Kinesis Analytics then creates one or more (as per the <code>InputParallelism</code> count you specified) in-application streams with names "MyInApplicationStream_001," "MyInApplicationStream_002," and so on. <br/>The <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_InputLambdaProcessor.html">InputLambdaProcessor</a> that is used to preprocess the records in the stream before being processed by your application code.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputLambdaProcessor</b>: The <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_InputLambdaProcessor.html">InputLambdaProcessor</a> that is used to preprocess the records in the stream before being processed by your application code.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> ARN of the IAM role that Amazon Kinesis Analytics can assume to access the stream on your behalf. You need to grant the necessary permissions to this role.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceARN</i> ARN of the input Amazon Kinesis stream to read.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> ARN of the IAM role that Amazon Kinesis Analytics can assume to access the stream on your behalf. You need to make sure that the role has the necessary permissions to access the stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceARN</i> ARN of the input delivery stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Count</i> Number of in-application streams to create. For more information, see <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/limits.html">Limits</a>. <br/>Specifies the format of the records on the streaming source.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RecordFormat</b>: Specifies the format of the records on the streaming source.
        :return:         """
        pass

    def add_application_input_processing_configuration(self, InputProcessingConfiguration: InputProcessingConfiguration, InputId: str, CurrentApplicationVersionId: ApplicationVersionId, ApplicationName: str) -> AddApplicationInputProcessingConfigurationResponse:
        """<note> This documentation is for version 1 of the Amazon Kinesis Data Analytics API, which only supports SQL applications. Version 2 of the API supports SQL and Java applications. For more information about version 2, see <a href="/kinesisanalytics/latest/apiv2/Welcome.html">Amazon Kinesis Data Analytics API V2 Documentation</a>. </note> Adds an <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_InputProcessingConfiguration.html">InputProcessingConfiguration</a> to an application. An input processor preprocesses records on the input stream before the application's SQL code executes. Currently, the only input processor available is <a href="https://docs.aws.amazon.com/lambda/">AWS Lambda</a>.

        :param self:
        :param InputProcessingConfiguration:  The <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_InputProcessingConfiguration.html">InputProcessingConfiguration</a> to add to the application.:<br/>InputId:  The ID of the input configuration to add the input processing configuration to. You can get a list of the input IDs for an application using the <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_DescribeApplication.html">DescribeApplication</a> operation.<br/>CurrentApplicationVersionId:  Version of the application to which you want to add the input processing configuration. You can use the <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_DescribeApplication.html">DescribeApplication</a> operation to get the current application version. If the version specified is not the current version, the <code>ConcurrentModificationException</code> is returned.<br/>ApplicationName:  Name of the application to which you want to add the input processing configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputLambdaProcessor</i> The <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_InputLambdaProcessor.html">InputLambdaProcessor</a> that is used to preprocess the records in the stream before being processed by your application code.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputLambdaProcessor</b>: The <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_InputLambdaProcessor.html">InputLambdaProcessor</a> that is used to preprocess the records in the stream before being processed by your application code.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> The ARN of the IAM role that is used to access the AWS Lambda function.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceARN</i> The ARN of the <a href="https://docs.aws.amazon.com/lambda/">AWS Lambda</a> function that operates on records in the stream. <note> To specify an earlier version of the Lambda function than the latest, include the Lambda function version in the Lambda function ARN. For more information about Lambda ARNs, see <a href="/general/latest/gr/aws-arns-and-namespaces.html#arn-syntax-lambda">Example ARNs: AWS Lambda</a>  </note><br/>
        :return:         """
        pass

    def add_application_output(self, Output: Output, CurrentApplicationVersionId: ApplicationVersionId, ApplicationName: str) -> AddApplicationOutputResponse:
        """<note> This documentation is for version 1 of the Amazon Kinesis Data Analytics API, which only supports SQL applications. Version 2 of the API supports SQL and Java applications. For more information about version 2, see <a href="/kinesisanalytics/latest/apiv2/Welcome.html">Amazon Kinesis Data Analytics API V2 Documentation</a>. </note> Adds an external destination to your Amazon Kinesis Analytics application. If you want Amazon Kinesis Analytics to deliver data from an in-application stream within your application to an external destination (such as an Amazon Kinesis stream, an Amazon Kinesis Firehose delivery stream, or an AWS Lambda function), you add the relevant configuration to your application using this operation. You can configure one or more outputs for your application. Each output configuration maps an in-application stream and an external destination.  You can use one of the output configurations to deliver data from your in-application error stream to an external destination so that you can analyze the errors. For more information, see <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/how-it-works-output.html">Understanding Application Output (Destination)</a>.   Any configuration update, including adding a streaming source using this operation, results in a new version of the application. You can use the <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_DescribeApplication.html">DescribeApplication</a> operation to find the current application version. For the limits on the number of application inputs and outputs you can configure, see <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/limits.html">Limits</a>. This operation requires permissions to perform the <code>kinesisanalytics:AddApplicationOutput</code> action.

        :param self:
        :param Output:  An array of objects, each describing one output configuration. In the output configuration, you specify the name of an in-application stream, a destination (that is, an Amazon Kinesis stream, an Amazon Kinesis Firehose delivery stream, or an AWS Lambda function), and record the formation to use when writing to the destination.:<br/>CurrentApplicationVersionId:  Version of the application to which you want to add the output configuration. You can use the <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_DescribeApplication.html">DescribeApplication</a> operation to get the current application version. If the version specified is not the current version, the <code>ConcurrentModificationException</code> is returned. <br/>ApplicationName:  Name of the application to which you want to add the output configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationSchema</i> Describes the data format when records are written to the destination. For more information, see <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/how-it-works-output.html">Configuring Application Output</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DestinationSchema</b>: Describes the data format when records are written to the destination. For more information, see <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/how-it-works-output.html">Configuring Application Output</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LambdaOutput</i> Identifies an AWS Lambda function as the destination.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LambdaOutput</b>: Identifies an AWS Lambda function as the destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KinesisFirehoseOutput</i> Identifies an Amazon Kinesis Firehose delivery stream as the destination.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KinesisFirehoseOutput</b>: Identifies an Amazon Kinesis Firehose delivery stream as the destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KinesisStreamsOutput</i> Identifies an Amazon Kinesis stream as the destination.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KinesisStreamsOutput</b>: Identifies an Amazon Kinesis stream as the destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Name of the in-application stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> ARN of the IAM role that Amazon Kinesis Analytics can assume to write to the destination stream on your behalf. You need to grant the necessary permissions to this role.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceARN</i> ARN of the destination Amazon Kinesis stream to write to.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> ARN of the IAM role that Amazon Kinesis Analytics can assume to write to the destination stream on your behalf. You need to grant the necessary permissions to this role.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceARN</i> ARN of the destination Amazon Kinesis Firehose delivery stream to write to.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> ARN of the IAM role that Amazon Kinesis Analytics can assume to write to the destination function on your behalf. You need to grant the necessary permissions to this role. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceARN</i> Amazon Resource Name (ARN) of the destination Lambda function to write to. <note> To specify an earlier version of the Lambda function than the latest, include the Lambda function version in the Lambda function ARN. For more information about Lambda ARNs, see <a href="/general/latest/gr/aws-arns-and-namespaces.html#arn-syntax-lambda">Example ARNs: AWS Lambda</a>  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordFormatType</i> Specifies the format of the records on the output stream.<br/>
        :return:         """
        pass

    def add_application_reference_data_source(self, ReferenceDataSource: ReferenceDataSource, CurrentApplicationVersionId: ApplicationVersionId, ApplicationName: str) -> AddApplicationReferenceDataSourceResponse:
        """<note> This documentation is for version 1 of the Amazon Kinesis Data Analytics API, which only supports SQL applications. Version 2 of the API supports SQL and Java applications. For more information about version 2, see <a href="/kinesisanalytics/latest/apiv2/Welcome.html">Amazon Kinesis Data Analytics API V2 Documentation</a>. </note> Adds a reference data source to an existing application. Amazon Kinesis Analytics reads reference data (that is, an Amazon S3 object) and creates an in-application table within your application. In the request, you provide the source (S3 bucket name and object key name), name of the in-application table to create, and the necessary mapping information that describes how data in Amazon S3 object maps to columns in the resulting in-application table.  For conceptual information, see <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/how-it-works-input.html">Configuring Application Input</a>. For the limits on data sources you can add to your application, see <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/limits.html">Limits</a>.   This operation requires permissions to perform the <code>kinesisanalytics:AddApplicationOutput</code> action. 

        :param self:
        :param ReferenceDataSource:  The reference data source can be an object in your Amazon S3 bucket. Amazon Kinesis Analytics reads the object and copies the data into the in-application table that is created. You provide an S3 bucket, object key name, and the resulting in-application table that is created. You must also provide an IAM role with the necessary permissions that Amazon Kinesis Analytics can assume to read the object from your S3 bucket on your behalf.:<br/>CurrentApplicationVersionId:  Version of the application for which you are adding the reference data source. You can use the <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_DescribeApplication.html">DescribeApplication</a> operation to get the current application version. If the version specified is not the current version, the <code>ConcurrentModificationException</code> is returned.<br/>ApplicationName:  Name of an existing application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReferenceSchema</i> Describes the format of the data in the streaming source, and how each data element maps to corresponding columns created in the in-application stream.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ReferenceSchema</b>: Describes the format of the data in the streaming source, and how each data element maps to corresponding columns created in the in-application stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3ReferenceDataSource</i> Identifies the S3 bucket and object that contains the reference data. Also identifies the IAM role Amazon Kinesis Analytics can assume to read this object on your behalf. An Amazon Kinesis Analytics application loads reference data only once. If the data changes, you call the <code>UpdateApplication</code> operation to trigger reloading of data into your application. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3ReferenceDataSource</b>: Identifies the S3 bucket and object that contains the reference data. Also identifies the IAM role Amazon Kinesis Analytics can assume to read this object on your behalf. An Amazon Kinesis Analytics application loads reference data only once. If the data changes, you call the <code>UpdateApplication</code> operation to trigger reloading of data into your application. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableName</i> Name of the in-application table to create.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReferenceRoleARN</i> ARN of the IAM role that the service can assume to read data on your behalf. This role must have permission for the <code>s3:GetObject</code> action on the object and trust policy that allows Amazon Kinesis Analytics service principal to assume this role.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FileKey</i> Object key name containing reference data.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketARN</i> Amazon Resource Name (ARN) of the S3 bucket.<br/>Specifies the format of the records on the streaming source.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RecordFormat</b>: Specifies the format of the records on the streaming source.
        :return:         """
        pass

    def create_application(self, ApplicationName: str, ApplicationDescription: str=None, Inputs: List[Input]=None, Outputs: List[Output]=None, CloudWatchLoggingOptions: List[CloudWatchLoggingOption]=None, ApplicationCode: str=None, Tags: List[Tag]=None) -> CreateApplicationResponse:
        """<note> This documentation is for version 1 of the Amazon Kinesis Data Analytics API, which only supports SQL applications. Version 2 of the API supports SQL and Java applications. For more information about version 2, see <a href="/kinesisanalytics/latest/apiv2/Welcome.html">Amazon Kinesis Data Analytics API V2 Documentation</a>. </note>  Creates an Amazon Kinesis Analytics application. You can configure each application with one streaming source as input, application code to process the input, and up to three destinations where you want Amazon Kinesis Analytics to write the output data from your application. For an overview, see <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/how-it-works.html">How it Works</a>.  In the input configuration, you map the streaming source to an in-application stream, which you can think of as a constantly updating table. In the mapping, you must provide a schema for the in-application stream and map each data column in the in-application stream to a data element in the streaming source. Your application code is one or more SQL statements that read input data, transform it, and generate output. Your application code can create one or more SQL artifacts like SQL streams or pumps. In the output configuration, you can configure the application to write data from in-application streams created in your applications to up to three destinations.  To read data from your source stream or write data to destination streams, Amazon Kinesis Analytics needs your permissions. You grant these permissions by creating IAM roles. This operation requires permissions to perform the <code>kinesisanalytics:CreateApplication</code> action.   For introductory exercises to create an Amazon Kinesis Analytics application, see <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/getting-started.html">Getting Started</a>. 

        :param self:
        :param Tags:  A list of one or more tags to assign to the application. A tag is a key-value pair that identifies an application. Note that the maximum number of application tags includes system tags. The maximum number of user-defined application tags is 50. For more information, see <a href="https://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/cost-alloc-tags.html">Using Cost Allocation Tags</a> in the <i>AWS Billing and Cost Management Guide</i>.<br/>ApplicationCode:  One or more SQL statements that read input data, transform it, and generate output. For example, you can write a SQL statement that reads data from one in-application stream, generates a running average of the number of advertisement clicks by vendor, and insert resulting rows in another in-application stream using pumps. For more information about the typical pattern, see <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/how-it-works-app-code.html">Application Code</a>.  You can provide such series of SQL statements, where output of one statement can be used as the input for the next statement. You store intermediate results by creating in-application streams and pumps. Note that the application code must create the streams with names specified in the <code>Outputs</code>. For example, if your <code>Outputs</code> defines output streams named <code>ExampleOutputStream1</code> and <code>ExampleOutputStream2</code>, then your application code must create these streams. <br/>CloudWatchLoggingOptions:  Use this parameter to configure a CloudWatch log stream to monitor application configuration errors. For more information, see <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/cloudwatch-logs.html">Working with Amazon CloudWatch Logs</a>.<br/>Outputs:  You can configure application output to write data from any of the in-application streams to up to three destinations. These destinations can be Amazon Kinesis streams, Amazon Kinesis Firehose delivery streams, AWS Lambda destinations, or any combination of the three. In the configuration, you specify the in-application stream name, the destination stream or Lambda function Amazon Resource Name (ARN), and the format to use when writing data. You must also provide an IAM role that Amazon Kinesis Analytics can assume to write to the destination stream or Lambda function on your behalf. In the output configuration, you also provide the output stream or Lambda function ARN. For stream destinations, you provide the format of data in the stream (for example, JSON, CSV). You also must provide an IAM role that Amazon Kinesis Analytics can assume to write to the stream or Lambda function on your behalf.<br/>Inputs:  Use this parameter to configure the application input. You can configure your application to receive input from a single streaming source. In this configuration, you map this streaming source to an in-application stream that is created. Your application code can then query the in-application stream like a table (you can think of it as a constantly updating table). For the streaming source, you provide its Amazon Resource Name (ARN) and format of data on the stream (for example, JSON, CSV, etc.). You also must provide an IAM role that Amazon Kinesis Analytics can assume to read this stream on your behalf. To create the in-application stream, you need to specify a schema to transform your data into a schematized version used in SQL. In the schema, you provide the necessary mapping of the data elements in the streaming source to record columns in the in-app stream.<br/>ApplicationDescription:  Summary description of the application.<br/>ApplicationName:  Name of your Amazon Kinesis Analytics application (for example, <code>sample-app</code>).<br/>
        :return: ApplicationSummary:  In response to your <code>CreateApplication</code> request, Amazon Kinesis Analytics returns a response with a summary of the application it created, including the application Amazon Resource Name (ARN), name, and status.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationStatus</i> Status of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationARN</i> ARN of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationName</i> Name of the application.<br/>        """
        pass

    def delete_application(self, CreateTimestamp: Timestamp, ApplicationName: str) -> DeleteApplicationResponse:
        """<note> This documentation is for version 1 of the Amazon Kinesis Data Analytics API, which only supports SQL applications. Version 2 of the API supports SQL and Java applications. For more information about version 2, see <a href="/kinesisanalytics/latest/apiv2/Welcome.html">Amazon Kinesis Data Analytics API V2 Documentation</a>. </note> Deletes the specified application. Amazon Kinesis Analytics halts application execution and deletes the application, including any application artifacts (such as in-application streams, reference table, and application code). This operation requires permissions to perform the <code>kinesisanalytics:DeleteApplication</code> action.

        :param self:
        :param CreateTimestamp:   You can use the <code>DescribeApplication</code> operation to get this value. <br/>ApplicationName:  Name of the Amazon Kinesis Analytics application to delete.<br/>
        :return:         """
        pass

    def delete_application_cloud_watch_logging_option(self, CloudWatchLoggingOptionId: str, CurrentApplicationVersionId: ApplicationVersionId, ApplicationName: str) -> DeleteApplicationCloudWatchLoggingOptionResponse:
        """<note> This documentation is for version 1 of the Amazon Kinesis Data Analytics API, which only supports SQL applications. Version 2 of the API supports SQL and Java applications. For more information about version 2, see <a href="/kinesisanalytics/latest/apiv2/Welcome.html">Amazon Kinesis Data Analytics API V2 Documentation</a>. </note> Deletes a CloudWatch log stream from an application. For more information about using CloudWatch log streams with Amazon Kinesis Analytics applications, see <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/cloudwatch-logs.html">Working with Amazon CloudWatch Logs</a>.

        :param self:
        :param CloudWatchLoggingOptionId:  The <code>CloudWatchLoggingOptionId</code> of the CloudWatch logging option to delete. You can get the <code>CloudWatchLoggingOptionId</code> by using the <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_DescribeApplication.html">DescribeApplication</a> operation. <br/>CurrentApplicationVersionId:  The version ID of the Kinesis Analytics application.<br/>ApplicationName:  The Kinesis Analytics application name.<br/>
        :return:         """
        pass

    def delete_application_input_processing_configuration(self, InputId: str, CurrentApplicationVersionId: ApplicationVersionId, ApplicationName: str) -> DeleteApplicationInputProcessingConfigurationResponse:
        """<note> This documentation is for version 1 of the Amazon Kinesis Data Analytics API, which only supports SQL applications. Version 2 of the API supports SQL and Java applications. For more information about version 2, see <a href="/kinesisanalytics/latest/apiv2/Welcome.html">Amazon Kinesis Data Analytics API V2 Documentation</a>. </note> Deletes an <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_InputProcessingConfiguration.html">InputProcessingConfiguration</a> from an input.

        :param self:
        :param InputId:  The ID of the input configuration from which to delete the input processing configuration. You can get a list of the input IDs for an application by using the <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_DescribeApplication.html">DescribeApplication</a> operation.<br/>CurrentApplicationVersionId:  The version ID of the Kinesis Analytics application.<br/>ApplicationName:  The Kinesis Analytics application name.<br/>
        :return:         """
        pass

    def delete_application_output(self, OutputId: str, CurrentApplicationVersionId: ApplicationVersionId, ApplicationName: str) -> DeleteApplicationOutputResponse:
        """<note> This documentation is for version 1 of the Amazon Kinesis Data Analytics API, which only supports SQL applications. Version 2 of the API supports SQL and Java applications. For more information about version 2, see <a href="/kinesisanalytics/latest/apiv2/Welcome.html">Amazon Kinesis Data Analytics API V2 Documentation</a>. </note> Deletes output destination configuration from your application configuration. Amazon Kinesis Analytics will no longer write data from the corresponding in-application stream to the external output destination. This operation requires permissions to perform the <code>kinesisanalytics:DeleteApplicationOutput</code> action.

        :param self:
        :param OutputId:  The ID of the configuration to delete. Each output configuration that is added to the application, either when the application is created or later using the <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_AddApplicationOutput.html">AddApplicationOutput</a> operation, has a unique ID. You need to provide the ID to uniquely identify the output configuration that you want to delete from the application configuration. You can use the <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_DescribeApplication.html">DescribeApplication</a> operation to get the specific <code>OutputId</code>. <br/>CurrentApplicationVersionId:  Amazon Kinesis Analytics application version. You can use the <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_DescribeApplication.html">DescribeApplication</a> operation to get the current application version. If the version specified is not the current version, the <code>ConcurrentModificationException</code> is returned. <br/>ApplicationName:  Amazon Kinesis Analytics application name.<br/>
        :return:         """
        pass

    def delete_application_reference_data_source(self, ReferenceId: str, CurrentApplicationVersionId: ApplicationVersionId, ApplicationName: str) -> DeleteApplicationReferenceDataSourceResponse:
        """<note> This documentation is for version 1 of the Amazon Kinesis Data Analytics API, which only supports SQL applications. Version 2 of the API supports SQL and Java applications. For more information about version 2, see <a href="/kinesisanalytics/latest/apiv2/Welcome.html">Amazon Kinesis Data Analytics API V2 Documentation</a>. </note> Deletes a reference data source configuration from the specified application configuration. If the application is running, Amazon Kinesis Analytics immediately removes the in-application table that you created using the <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_AddApplicationReferenceDataSource.html">AddApplicationReferenceDataSource</a> operation.  This operation requires permissions to perform the <code>kinesisanalytics.DeleteApplicationReferenceDataSource</code> action.

        :param self:
        :param ReferenceId:  ID of the reference data source. When you add a reference data source to your application using the <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_AddApplicationReferenceDataSource.html">AddApplicationReferenceDataSource</a>, Amazon Kinesis Analytics assigns an ID. You can use the <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_DescribeApplication.html">DescribeApplication</a> operation to get the reference ID. <br/>CurrentApplicationVersionId:  Version of the application. You can use the <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_DescribeApplication.html">DescribeApplication</a> operation to get the current application version. If the version specified is not the current version, the <code>ConcurrentModificationException</code> is returned.<br/>ApplicationName:  Name of an existing application.<br/>
        :return:         """
        pass

    def describe_application(self, ApplicationName: str) -> DescribeApplicationResponse:
        """<note> This documentation is for version 1 of the Amazon Kinesis Data Analytics API, which only supports SQL applications. Version 2 of the API supports SQL and Java applications. For more information about version 2, see <a href="/kinesisanalytics/latest/apiv2/Welcome.html">Amazon Kinesis Data Analytics API V2 Documentation</a>. </note> Returns information about a specific Amazon Kinesis Analytics application. If you want to retrieve a list of all applications in your account, use the <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_ListApplications.html">ListApplications</a> operation. This operation requires permissions to perform the <code>kinesisanalytics:DescribeApplication</code> action. You can use <code>DescribeApplication</code> to get the current application versionId, which you need to call other operations such as <code>Update</code>. 

        :param self:
        :param ApplicationName:  Name of the application.<br/>
        :return: ApplicationDetail:  Provides a description of the application, such as the application Amazon Resource Name (ARN), status, latest version, and input and output configuration details.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationVersionId</i> Provides the current application version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationCode</i> Returns the application code that you provided to perform data analysis on any of the in-application streams in your application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchLoggingOptionDescriptions</i> Describes the CloudWatch log streams that are configured to receive application messages. For more information about using CloudWatch log streams with Amazon Kinesis Analytics applications, see <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/cloudwatch-logs.html">Working with Amazon CloudWatch Logs</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReferenceDataSourceDescriptions</i> Describes reference data sources configured for the application. For more information, see <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/how-it-works-input.html">Configuring Application Input</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputDescriptions</i> Describes the application output configuration. For more information, see <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/how-it-works-output.html">Configuring Application Output</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputDescriptions</i> Describes the application input configuration. For more information, see <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/how-it-works-input.html">Configuring Application Input</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdateTimestamp</i> Time stamp when the application was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateTimestamp</i> Time stamp when the application version was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationStatus</i> Status of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationARN</i> ARN of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationDescription</i> Description of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationName</i> Name of the application.<br/>        """
        pass

    def discover_input_schema(self, ResourceARN: str=None, RoleARN: str=None, InputStartingPositionConfiguration: InputStartingPositionConfiguration=None, S3Configuration: S3Configuration=None, InputProcessingConfiguration: InputProcessingConfiguration=None) -> DiscoverInputSchemaResponse:
        """<note> This documentation is for version 1 of the Amazon Kinesis Data Analytics API, which only supports SQL applications. Version 2 of the API supports SQL and Java applications. For more information about version 2, see <a href="/kinesisanalytics/latest/apiv2/Welcome.html">Amazon Kinesis Data Analytics API V2 Documentation</a>. </note> Infers a schema by evaluating sample records on the specified streaming source (Amazon Kinesis stream or Amazon Kinesis Firehose delivery stream) or S3 object. In the response, the operation returns the inferred schema and also the sample records that the operation used to infer the schema.  You can use the inferred schema when configuring a streaming source for your application. For conceptual information, see <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/how-it-works-input.html">Configuring Application Input</a>. Note that when you create an application using the Amazon Kinesis Analytics console, the console uses this operation to infer a schema and show it in the console user interface.   This operation requires permissions to perform the <code>kinesisanalytics:DiscoverInputSchema</code> action. 

        :param self:
        :param InputProcessingConfiguration:  The <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_InputProcessingConfiguration.html">InputProcessingConfiguration</a> to use to preprocess the records before discovering the schema of the records.:<br/>S3Configuration:  Specify this parameter to discover a schema from data in an Amazon S3 object.:<br/>InputStartingPositionConfiguration:  Point at which you want Amazon Kinesis Analytics to start reading records from the specified streaming source discovery purposes.:<br/>RoleARN:  ARN of the IAM role that Amazon Kinesis Analytics can assume to access the stream on your behalf.<br/>ResourceARN:  Amazon Resource Name (ARN) of the streaming source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputStartingPosition</i> The starting position on the stream. <ul> <li>  <code>NOW</code> - Start reading just after the most recent record in the stream, start at the request time stamp that the customer issued. </li> <li>  <code>TRIM_HORIZON</code> - Start reading at the last untrimmed record in the stream, which is the oldest record available in the stream. This option is not available for an Amazon Kinesis Firehose delivery stream. </li> <li>  <code>LAST_STOPPED_POINT</code> - Resume reading from where the application last stopped reading. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FileKey</i> The name of the object that contains the data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketARN</i> ARN of the S3 bucket that contains the data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> IAM ARN of the role used to access the data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputLambdaProcessor</i> The <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_InputLambdaProcessor.html">InputLambdaProcessor</a> that is used to preprocess the records in the stream before being processed by your application code.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputLambdaProcessor</b>: The <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_InputLambdaProcessor.html">InputLambdaProcessor</a> that is used to preprocess the records in the stream before being processed by your application code.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> The ARN of the IAM role that is used to access the AWS Lambda function.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceARN</i> The ARN of the <a href="https://docs.aws.amazon.com/lambda/">AWS Lambda</a> function that operates on records in the stream. <note> To specify an earlier version of the Lambda function than the latest, include the Lambda function version in the Lambda function ARN. For more information about Lambda ARNs, see <a href="/general/latest/gr/aws-arns-and-namespaces.html#arn-syntax-lambda">Example ARNs: AWS Lambda</a>  </note><br/>
        :return: RawInputRecords:  Raw stream data that was sampled to infer the schema.<br/>ProcessedInputRecords:  Stream data that was modified by the processor specified in the <code>InputProcessingConfiguration</code> parameter.<br/>ParsedInputRecords:  An array of elements, where each element corresponds to a row in a stream record (a stream record can have more than one row).<br/>InputSchema:  Schema inferred from the streaming source. It identifies the format of the data in the streaming source and how each data element maps to corresponding columns in the in-application stream that you can create.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordColumns</i> A list of <code>RecordColumn</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordEncoding</i> Specifies the encoding of the records in the streaming source. For example, UTF-8.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordFormat</i> Specifies the format of the records on the streaming source.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RecordFormat</b>: Specifies the format of the records on the streaming source.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordFormatType</i> The type of record format.<br/>When configuring application input at the time of creating or updating an application, provides additional mapping information specific to the record format (such as JSON, CSV, or record fields delimited by some delimiter) on the streaming source.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>MappingParameters</b>: When configuring application input at the time of creating or updating an application, provides additional mapping information specific to the record format (such as JSON, CSV, or record fields delimited by some delimiter) on the streaming source.        """
        pass

    def list_applications(self, Limit: int=None, ExclusiveStartApplicationName: str=None) -> ListApplicationsResponse:
        """<note> This documentation is for version 1 of the Amazon Kinesis Data Analytics API, which only supports SQL applications. Version 2 of the API supports SQL and Java applications. For more information about version 2, see <a href="/kinesisanalytics/latest/apiv2/Welcome.html">Amazon Kinesis Data Analytics API V2 Documentation</a>. </note> Returns a list of Amazon Kinesis Analytics applications in your account. For each application, the response includes the application name, Amazon Resource Name (ARN), and status. If the response returns the <code>HasMoreApplications</code> value as true, you can send another request by adding the <code>ExclusiveStartApplicationName</code> in the request body, and set the value of this to the last application name from the previous response.  If you want detailed information about a specific application, use <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_DescribeApplication.html">DescribeApplication</a>. This operation requires permissions to perform the <code>kinesisanalytics:ListApplications</code> action.

        :param self:
        :param ExclusiveStartApplicationName:  Name of the application to start the list with. When using pagination to retrieve the list, you don't need to specify this parameter in the first request. However, in subsequent requests, you add the last application name from the previous response to get the next page of applications.<br/>Limit:  Maximum number of applications to list.<br/>
        :return: HasMoreApplications:  Returns true if there are more applications to retrieve.<br/>ApplicationSummaries:  List of <code>ApplicationSummary</code> objects. <br/>        """
        pass

    def list_tags_for_resource(self, ResourceARN: str) -> ListTagsForResourceResponse:
        """Retrieves the list of key-value tags assigned to the application.

        :param self:
        :param ResourceARN:  The ARN of the application for which to retrieve tags.<br/>
        :return: Tags:  The key-value tags assigned to the application.<br/>        """
        pass

    def start_application(self, InputConfigurations: List[InputConfiguration], ApplicationName: str) -> StartApplicationResponse:
        """<note> This documentation is for version 1 of the Amazon Kinesis Data Analytics API, which only supports SQL applications. Version 2 of the API supports SQL and Java applications. For more information about version 2, see <a href="/kinesisanalytics/latest/apiv2/Welcome.html">Amazon Kinesis Data Analytics API V2 Documentation</a>. </note> Starts the specified Amazon Kinesis Analytics application. After creating an application, you must exclusively call this operation to start your application. After the application starts, it begins consuming the input data, processes it, and writes the output to the configured destination.  The application status must be <code>READY</code> for you to start an application. You can get the application status in the console or using the <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_DescribeApplication.html">DescribeApplication</a> operation. After you start the application, you can stop the application from processing the input by calling the <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_StopApplication.html">StopApplication</a> operation. This operation requires permissions to perform the <code>kinesisanalytics:StartApplication</code> action.

        :param self:
        :param InputConfigurations:  Identifies the specific input, by ID, that the application starts consuming. Amazon Kinesis Analytics starts reading the streaming source associated with the input. You can also specify where in the streaming source you want Amazon Kinesis Analytics to start reading.<br/>ApplicationName:  Name of the application.<br/>
        :return:         """
        pass

    def stop_application(self, ApplicationName: str) -> StopApplicationResponse:
        """<note> This documentation is for version 1 of the Amazon Kinesis Data Analytics API, which only supports SQL applications. Version 2 of the API supports SQL and Java applications. For more information about version 2, see <a href="/kinesisanalytics/latest/apiv2/Welcome.html">Amazon Kinesis Data Analytics API V2 Documentation</a>. </note> Stops the application from processing input data. You can stop an application only if it is in the running state. You can use the <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_DescribeApplication.html">DescribeApplication</a> operation to find the application state. After the application is stopped, Amazon Kinesis Analytics stops reading data from the input, the application stops processing data, and there is no output written to the destination.  This operation requires permissions to perform the <code>kinesisanalytics:StopApplication</code> action.

        :param self:
        :param ApplicationName:  Name of the running application to stop.<br/>
        :return:         """
        pass

    def tag_resource(self, Tags: List[Tag], ResourceARN: str) -> TagResourceResponse:
        """Adds one or more key-value tags to a Kinesis Analytics application. Note that the maximum number of application tags includes system tags. The maximum number of user-defined application tags is 50.

        :param self:
        :param Tags:  The key-value tags to assign to the application.<br/>ResourceARN:  The ARN of the application to assign the tags.<br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[TagKey], ResourceARN: str) -> UntagResourceResponse:
        """Removes one or more tags from a Kinesis Analytics application.

        :param self:
        :param TagKeys:  A list of keys of tags to remove from the specified application.<br/>ResourceARN:  The ARN of the Kinesis Analytics application from which to remove the tags.<br/>
        :return:         """
        pass

    def update_application(self, ApplicationUpdate: ApplicationUpdate, CurrentApplicationVersionId: ApplicationVersionId, ApplicationName: str) -> UpdateApplicationResponse:
        """<note> This documentation is for version 1 of the Amazon Kinesis Data Analytics API, which only supports SQL applications. Version 2 of the API supports SQL and Java applications. For more information about version 2, see <a href="/kinesisanalytics/latest/apiv2/Welcome.html">Amazon Kinesis Data Analytics API V2 Documentation</a>. </note> Updates an existing Amazon Kinesis Analytics application. Using this API, you can update application code, input configuration, and output configuration.  Note that Amazon Kinesis Analytics updates the <code>CurrentApplicationVersionId</code> each time you update your application.  This operation requires permission for the <code>kinesisanalytics:UpdateApplication</code> action.

        :param self:
        :param ApplicationUpdate:  Describes application updates.:<br/>CurrentApplicationVersionId:  The current application version ID. You can use the <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_DescribeApplication.html">DescribeApplication</a> operation to get this value.<br/>ApplicationName:  Name of the Amazon Kinesis Analytics application to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchLoggingOptionUpdates</i> Describes application CloudWatch logging option updates.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReferenceDataSourceUpdates</i> Describes application reference data source updates.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputUpdates</i> Describes application output configuration updates.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationCodeUpdate</i> Describes application code updates.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputUpdates</i> Describes application input configuration updates.<br/>
        :return:         """
        pass



class KinesisAnalyticsV2(BaseClient):
    """Amazon Kinesis Data Analytics is a fully managed service that you can use to process and analyze streaming data using SQL or Java. The service enables you to quickly author and run SQL or Java code against streaming sources to perform time series analytics, feed real-time dashboards, and create real-time metrics."""


    class ListApplicationSnapshotsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_application_snapshots` for valid parameters"""
            pass
    class ListApplicationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_applications` for valid parameters"""
            pass


    class AddApplicationCloudWatchLoggingOptionRequest(Mapping):
        pass
    
    class AddApplicationCloudWatchLoggingOptionResponse(Mapping):
        pass
    
    class ApplicationVersionId(object):
        pass
    
    class CloudWatchLoggingOption(Mapping):
        pass
    
    class AddApplicationInputRequest(Mapping):
        pass
    
    class AddApplicationInputResponse(Mapping):
        pass
    
    class Input(Mapping):
        pass
    
    class AddApplicationInputProcessingConfigurationRequest(Mapping):
        pass
    
    class AddApplicationInputProcessingConfigurationResponse(Mapping):
        pass
    
    class InputProcessingConfiguration(Mapping):
        pass
    
    class AddApplicationOutputRequest(Mapping):
        pass
    
    class AddApplicationOutputResponse(Mapping):
        pass
    
    class Output(Mapping):
        pass
    
    class AddApplicationReferenceDataSourceRequest(Mapping):
        pass
    
    class AddApplicationReferenceDataSourceResponse(Mapping):
        pass
    
    class ReferenceDataSource(Mapping):
        pass
    
    class CreateApplicationRequest(Mapping):
        pass
    
    class CreateApplicationResponse(Mapping):
        pass
    
    class ApplicationConfiguration(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreateApplicationSnapshotRequest(Mapping):
        pass
    
    class CreateApplicationSnapshotResponse(Mapping):
        pass
    
    class DeleteApplicationRequest(Mapping):
        pass
    
    class DeleteApplicationResponse(Mapping):
        pass
    
    class Timestamp(object):
        pass
    
    class DeleteApplicationCloudWatchLoggingOptionRequest(Mapping):
        pass
    
    class DeleteApplicationCloudWatchLoggingOptionResponse(Mapping):
        pass
    
    class DeleteApplicationInputProcessingConfigurationRequest(Mapping):
        pass
    
    class DeleteApplicationInputProcessingConfigurationResponse(Mapping):
        pass
    
    class DeleteApplicationOutputRequest(Mapping):
        pass
    
    class DeleteApplicationOutputResponse(Mapping):
        pass
    
    class DeleteApplicationReferenceDataSourceRequest(Mapping):
        pass
    
    class DeleteApplicationReferenceDataSourceResponse(Mapping):
        pass
    
    class DeleteApplicationSnapshotRequest(Mapping):
        pass
    
    class DeleteApplicationSnapshotResponse(Mapping):
        pass
    
    class DescribeApplicationRequest(Mapping):
        pass
    
    class DescribeApplicationResponse(Mapping):
        pass
    
    class BooleanObject(object):
        pass
    
    class DescribeApplicationSnapshotRequest(Mapping):
        pass
    
    class DescribeApplicationSnapshotResponse(Mapping):
        pass
    
    class DiscoverInputSchemaRequest(Mapping):
        pass
    
    class DiscoverInputSchemaResponse(Mapping):
        pass
    
    class InputStartingPositionConfiguration(Mapping):
        pass
    
    class S3Configuration(Mapping):
        pass
    
    class ListApplicationSnapshotsRequest(Mapping):
        pass
    
    class ListApplicationSnapshotsResponse(Mapping):
        pass
    
    class ListApplicationsRequest(Mapping):
        pass
    
    class ListApplicationsResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class StartApplicationRequest(Mapping):
        pass
    
    class StartApplicationResponse(Mapping):
        pass
    
    class RunConfiguration(Mapping):
        pass
    
    class StopApplicationRequest(Mapping):
        pass
    
    class StopApplicationResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateApplicationRequest(Mapping):
        pass
    
    class UpdateApplicationResponse(Mapping):
        pass
    
    class ApplicationConfigurationUpdate(Mapping):
        pass
    
    class RunConfigurationUpdate(Mapping):
        pass
    
    class CloudWatchLoggingOptionUpdate(Mapping):
        pass
    
    def add_application_cloud_watch_logging_option(self, CloudWatchLoggingOption: CloudWatchLoggingOption, CurrentApplicationVersionId: ApplicationVersionId, ApplicationName: str) -> AddApplicationCloudWatchLoggingOptionResponse:
        """Adds an Amazon CloudWatch log stream to monitor application configuration errors.

        :param self:
        :param CloudWatchLoggingOption:  Provides the Amazon CloudWatch log stream Amazon Resource Name (ARN). :<br/>CurrentApplicationVersionId:  The version ID of the Kinesis Data Analytics application. You can retrieve the application version ID using <a>DescribeApplication</a>.<br/>ApplicationName:  The Kinesis Data Analytics application name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogStreamARN</i> The ARN of the CloudWatch log to receive application messages.<br/>
        :return: CloudWatchLoggingOptionDescriptions:  The descriptions of the current CloudWatch logging options for the Kinesis Data Analytics application.<br/>ApplicationVersionId:  The new version ID of the Kinesis Data Analytics application. Kinesis Data Analytics updates the <code>ApplicationVersionId</code> each time you change the CloudWatch logging options. <br/>ApplicationARN:  The application's ARN.<br/>        """
        pass

    def add_application_input(self, Input: Input, CurrentApplicationVersionId: ApplicationVersionId, ApplicationName: str) -> AddApplicationInputResponse:
        """ Adds a streaming source to your SQL-based Amazon Kinesis Data Analytics application.  You can add a streaming source when you create an application, or you can use this operation to add a streaming source after you create an application. For more information, see <a>CreateApplication</a>. Any configuration update, including adding a streaming source using this operation, results in a new version of the application. You can use the <a>DescribeApplication</a> operation to find the current application version. 

        :param self:
        :param Input:  The <a>Input</a> to add.:<br/>CurrentApplicationVersionId:  The current version of your application. You can use the <a>DescribeApplication</a> operation to find the current application version.<br/>ApplicationName:  The name of your existing application to which you want to add the streaming source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputSchema</i> Describes the format of the data in the streaming source, and how each data element maps to corresponding columns in the in-application stream that is being created. Also used to describe the format of the reference data source.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputSchema</b>: Describes the format of the data in the streaming source, and how each data element maps to corresponding columns in the in-application stream that is being created. Also used to describe the format of the reference data source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputParallelism</i> Describes the number of in-application streams to create. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputParallelism</b>: Describes the number of in-application streams to create. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KinesisFirehoseInput</i> If the streaming source is an Amazon Kinesis Data Firehose delivery stream, identifies the delivery stream's ARN.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KinesisFirehoseInput</b>: If the streaming source is an Amazon Kinesis Data Firehose delivery stream, identifies the delivery stream's ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KinesisStreamsInput</i> If the streaming source is an Amazon Kinesis data stream, identifies the stream's Amazon Resource Name (ARN). &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KinesisStreamsInput</b>: If the streaming source is an Amazon Kinesis data stream, identifies the stream's Amazon Resource Name (ARN). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputProcessingConfiguration</i> The <a>InputProcessingConfiguration</a> for the input. An input processor transforms records as they are received from the stream, before the application's SQL code executes. Currently, the only input processing configuration available is <a>InputLambdaProcessor</a>. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputProcessingConfiguration</b>: The <a>InputProcessingConfiguration</a> for the input. An input processor transforms records as they are received from the stream, before the application's SQL code executes. Currently, the only input processing configuration available is <a>InputLambdaProcessor</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NamePrefix</i> The name prefix to use when creating an in-application stream. Suppose that you specify a prefix "<code>MyInApplicationStream</code>." Kinesis Data Analytics then creates one or more (as per the <code>InputParallelism</code> count you specified) in-application streams with the names "<code>MyInApplicationStream_001</code>," "<code>MyInApplicationStream_002</code>," and so on. <br/>The <a>InputLambdaProcessor</a> that is used to preprocess the records in the stream before being processed by your application code.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputLambdaProcessor</b>: The <a>InputLambdaProcessor</a> that is used to preprocess the records in the stream before being processed by your application code.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceARN</i> The ARN of the input Kinesis data stream to read.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceARN</i> The Amazon Resource Name (ARN) of the delivery stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Count</i> The number of in-application streams to create.<br/>Specifies the format of the records on the streaming source.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RecordFormat</b>: Specifies the format of the records on the streaming source.
        :return: InputDescriptions:  Describes the application input configuration. <br/>ApplicationVersionId:  Provides the current application version.<br/>ApplicationARN:  The Amazon Resource Name (ARN) of the application.<br/>        """
        pass

    def add_application_input_processing_configuration(self, InputProcessingConfiguration: InputProcessingConfiguration, InputId: str, CurrentApplicationVersionId: ApplicationVersionId, ApplicationName: str) -> AddApplicationInputProcessingConfigurationResponse:
        """Adds an <a>InputProcessingConfiguration</a> to an SQL-based Kinesis Data Analytics application. An input processor pre-processes records on the input stream before the application's SQL code executes. Currently, the only input processor available is <a href="https://aws.amazon.com/documentation/lambda/">AWS Lambda</a>.

        :param self:
        :param InputProcessingConfiguration:  The <a>InputProcessingConfiguration</a> to add to the application.:<br/>InputId:  The ID of the input configuration to add the input processing configuration to. You can get a list of the input IDs for an application using the <a>DescribeApplication</a> operation.<br/>CurrentApplicationVersionId:  The version of the application to which you want to add the input processing configuration. You can use the <a>DescribeApplication</a> operation to get the current application version. If the version specified is not the current version, the <code>ConcurrentModificationException</code> is returned.<br/>ApplicationName:  The name of the application to which you want to add the input processing configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputLambdaProcessor</i> The <a>InputLambdaProcessor</a> that is used to preprocess the records in the stream before being processed by your application code.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputLambdaProcessor</b>: The <a>InputLambdaProcessor</a> that is used to preprocess the records in the stream before being processed by your application code.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceARN</i> The ARN of the AWS Lambda function that operates on records in the stream. <note> To specify an earlier version of the Lambda function than the latest, include the Lambda function version in the Lambda function ARN. For more information about Lambda ARNs, see <a href="/general/latest/gr/aws-arns-and-namespaces.html#arn-syntax-lambda">Example ARNs: AWS Lambda</a>  </note><br/>
        :return: InputProcessingConfigurationDescription:  The description of the preprocessor that executes on records in this input before the application's code is run.:<br/>InputId:  The input ID that is associated with the application input. This is the ID that Amazon Kinesis Data Analytics assigns to each input configuration that you add to your application.<br/>ApplicationVersionId:  Provides the current application version.<br/>ApplicationARN:  The Amazon Resource Name (ARN) of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputLambdaProcessorDescription</i> Provides configuration information about the associated <a>InputLambdaProcessorDescription</a> &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputLambdaProcessorDescription</b>: Provides configuration information about the associated <a>InputLambdaProcessorDescription</a> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> The ARN of the IAM role that is used to access the AWS Lambda function. <note> Provided for backward compatibility. Applications that are created with the current API version have an application-level service execution role rather than a resource-level role. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceARN</i> The ARN of the AWS Lambda function that is used to preprocess the records in the stream. <note> To specify an earlier version of the Lambda function than the latest, include the Lambda function version in the Lambda function ARN. For more information about Lambda ARNs, see <a href="/general/latest/gr/aws-arns-and-namespaces.html#arn-syntax-lambda">Example ARNs: AWS Lambda</a>  </note><br/>        """
        pass

    def add_application_output(self, Output: Output, CurrentApplicationVersionId: ApplicationVersionId, ApplicationName: str) -> AddApplicationOutputResponse:
        """Adds an external destination to your SQL-based Amazon Kinesis Data Analytics application. If you want Kinesis Data Analytics to deliver data from an in-application stream within your application to an external destination (such as an Kinesis data stream, a Kinesis Data Firehose delivery stream, or an AWS Lambda function), you add the relevant configuration to your application using this operation. You can configure one or more outputs for your application. Each output configuration maps an in-application stream and an external destination.  You can use one of the output configurations to deliver data from your in-application error stream to an external destination so that you can analyze the errors.   Any configuration update, including adding a streaming source using this operation, results in a new version of the application. You can use the <a>DescribeApplication</a> operation to find the current application version.

        :param self:
        :param Output:  An array of objects, each describing one output configuration. In the output configuration, you specify the name of an in-application stream, a destination (that is, a Kinesis data stream, a Kinesis Data Firehose delivery stream, or an AWS Lambda function), and record the formation to use when writing to the destination.:<br/>CurrentApplicationVersionId:  The version of the application to which you want to add the output configuration. You can use the <a>DescribeApplication</a> operation to get the current application version. If the version specified is not the current version, the <code>ConcurrentModificationException</code> is returned. <br/>ApplicationName:  The name of the application to which you want to add the output configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationSchema</i> Describes the data format when records are written to the destination. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DestinationSchema</b>: Describes the data format when records are written to the destination. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LambdaOutput</i> Identifies an AWS Lambda function as the destination.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LambdaOutput</b>: Identifies an AWS Lambda function as the destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KinesisFirehoseOutput</i> Identifies an Amazon Kinesis Data Firehose delivery stream as the destination.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KinesisFirehoseOutput</b>: Identifies an Amazon Kinesis Data Firehose delivery stream as the destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KinesisStreamsOutput</i> Identifies an Amazon Kinesis data stream as the destination.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KinesisStreamsOutput</b>: Identifies an Amazon Kinesis data stream as the destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the in-application stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceARN</i> The ARN of the destination Kinesis data stream to write to.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceARN</i> The ARN of the destination delivery stream to write to.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceARN</i> The Amazon Resource Name (ARN) of the destination Lambda function to write to. <note> To specify an earlier version of the Lambda function than the latest, include the Lambda function version in the Lambda function ARN. For more information about Lambda ARNs, see <a href="/general/latest/gr/aws-arns-and-namespaces.html#arn-syntax-lambda">Example ARNs: AWS Lambda</a>  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordFormatType</i> Specifies the format of the records on the output stream.<br/>
        :return: OutputDescriptions:  Describes the application output configuration. For more information, see <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/dev/how-it-works-output.html">Configuring Application Output</a>. <br/>ApplicationVersionId:  The updated application version ID. Kinesis Data Analytics increments this ID when the application is updated.<br/>ApplicationARN:  The application Amazon Resource Name (ARN).<br/>        """
        pass

    def add_application_reference_data_source(self, ReferenceDataSource: ReferenceDataSource, CurrentApplicationVersionId: ApplicationVersionId, ApplicationName: str) -> AddApplicationReferenceDataSourceResponse:
        """Adds a reference data source to an existing SQL-based Amazon Kinesis Data Analytics application. Kinesis Data Analytics reads reference data (that is, an Amazon S3 object) and creates an in-application table within your application. In the request, you provide the source (S3 bucket name and object key name), name of the in-application table to create, and the necessary mapping information that describes how data in an Amazon S3 object maps to columns in the resulting in-application table.

        :param self:
        :param ReferenceDataSource:  The reference data source can be an object in your Amazon S3 bucket. Kinesis Data Analytics reads the object and copies the data into the in-application table that is created. You provide an S3 bucket, object key name, and the resulting in-application table that is created. :<br/>CurrentApplicationVersionId:  The version of the application for which you are adding the reference data source. You can use the <a>DescribeApplication</a> operation to get the current application version. If the version specified is not the current version, the <code>ConcurrentModificationException</code> is returned.<br/>ApplicationName:  The name of an existing application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReferenceSchema</i> Describes the format of the data in the streaming source, and how each data element maps to corresponding columns created in the in-application stream.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ReferenceSchema</b>: Describes the format of the data in the streaming source, and how each data element maps to corresponding columns created in the in-application stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3ReferenceDataSource</i> Identifies the S3 bucket and object that contains the reference data. A Kinesis Data Analytics application loads reference data only once. If the data changes, you call the <a>UpdateApplication</a> operation to trigger reloading of data into your application. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3ReferenceDataSource</b>: Identifies the S3 bucket and object that contains the reference data. A Kinesis Data Analytics application loads reference data only once. If the data changes, you call the <a>UpdateApplication</a> operation to trigger reloading of data into your application. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableName</i> The name of the in-application table to create.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FileKey</i> The object key name containing the reference data.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketARN</i> The Amazon Resource Name (ARN) of the S3 bucket.<br/>Specifies the format of the records on the streaming source.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RecordFormat</b>: Specifies the format of the records on the streaming source.
        :return: ReferenceDataSourceDescriptions:  Describes reference data sources configured for the application. <br/>ApplicationVersionId:  The updated application version ID. Amazon Kinesis Data Analytics increments this ID when the application is updated.<br/>ApplicationARN:  The application Amazon Resource Name (ARN).<br/>        """
        pass

    def create_application(self, ServiceExecutionRole: str, RuntimeEnvironment: str, ApplicationName: str, ApplicationDescription: str=None, ApplicationConfiguration: ApplicationConfiguration=None, CloudWatchLoggingOptions: List[CloudWatchLoggingOption]=None, Tags: List[Tag]=None) -> CreateApplicationResponse:
        """Creates an Amazon Kinesis Data Analytics application. For information about creating a Kinesis Data Analytics application, see <a href="https://docs.aws.amazon.com/kinesisanalytics/latest/java/getting-started.html">Creating an Application</a>. 

        :param self:
        :param Tags:  A list of one or more tags to assign to the application. A tag is a key-value pair that identifies an application. Note that the maximum number of application tags includes system tags. The maximum number of user-defined application tags is 50. For more information, see <a href="https://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/cost-alloc-tags.html">Using Cost Allocation Tags</a> in the <i>AWS Billing and Cost Management Guide</i>.<br/>CloudWatchLoggingOptions:  Use this parameter to configure an Amazon CloudWatch log stream to monitor application configuration errors. <br/>ApplicationConfiguration:  Use this parameter to configure the application.:<br/>ServiceExecutionRole:  The IAM role used by the application to access Kinesis data streams, Kinesis Data Firehose delivery streams, Amazon S3 objects, and other external resources.<br/>RuntimeEnvironment:  The runtime environment for the application (<code>SQL-1.0</code> or <code>FLINK-1_6</code>).<br/>ApplicationDescription:  A summary description of the application.<br/>ApplicationName:  The name of your application (for example, <code>sample-app</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationSnapshotConfiguration</i> Describes whether snapshots are enabled for a Java-based Kinesis Data Analytics application.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ApplicationSnapshotConfiguration</b>: Describes whether snapshots are enabled for a Java-based Kinesis Data Analytics application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationCodeConfiguration</i> The code location and type parameters for a Java-based Kinesis Data Analytics application.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ApplicationCodeConfiguration</b>: The code location and type parameters for a Java-based Kinesis Data Analytics application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnvironmentProperties</i> Describes execution properties for a Java-based Kinesis Data Analytics application.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EnvironmentProperties</b>: Describes execution properties for a Java-based Kinesis Data Analytics application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FlinkApplicationConfiguration</i> The creation and update parameters for a Java-based Kinesis Data Analytics application.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>FlinkApplicationConfiguration</b>: The creation and update parameters for a Java-based Kinesis Data Analytics application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SqlApplicationConfiguration</i> The creation and update parameters for an SQL-based Kinesis Data Analytics application.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SqlApplicationConfiguration</b>: The creation and update parameters for an SQL-based Kinesis Data Analytics application.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReferenceDataSources</i> The array of <a>ReferenceDataSource</a> objects describing the reference data sources used by the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Outputs</i> The array of <a>Output</a> objects describing the destination streams used by the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Inputs</i> The array of <a>Input</a> objects describing the input streams used by the application.<br/>Describes an application's checkpointing configuration. Checkpointing is the process of persisting application state for fault tolerance. For more information, see <a href="https://ci.apache.org/projects/flink/flink-docs-release-1.6/concepts/programming-model.html#checkpoints-for-fault-tolerance"> Checkpoints for Fault Tolerance</a> in the <a href="https://ci.apache.org/projects/flink/flink-docs-release-1.6/">Apache Flink Documentation</a>. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CheckpointConfiguration</b>: Describes an application's checkpointing configuration. Checkpointing is the process of persisting application state for fault tolerance. For more information, see <a href="https://ci.apache.org/projects/flink/flink-docs-release-1.6/concepts/programming-model.html#checkpoints-for-fault-tolerance"> Checkpoints for Fault Tolerance</a> in the <a href="https://ci.apache.org/projects/flink/flink-docs-release-1.6/">Apache Flink Documentation</a>. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PropertyGroups</i> Describes the execution property groups.<br/>The location and type of the application code.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CodeContent</b>: The location and type of the application code.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotsEnabled</i> Describes whether snapshots are enabled for a Java-based Kinesis Data Analytics application.<br/>
        :return: ApplicationDetail:  In response to your <code>CreateApplication</code> request, Kinesis Data Analytics returns a response with details of the application it created.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchLoggingOptionDescriptions</i> Describes the application Amazon CloudWatch logging options.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationConfigurationDescription</i> Provides details about the application's SQL or Java code and starting parameters.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ApplicationConfigurationDescription</b>: Provides details about the application's SQL or Java code and starting parameters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdateTimestamp</i> The current timestamp when the application was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateTimestamp</i> The current timestamp when the application was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationVersionId</i> Provides the current application version. Kinesis Data Analytics updates the <code>ApplicationVersionId</code> each time you update the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationStatus</i> The status of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceExecutionRole</i> Specifies the IAM role that the application uses to access external resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuntimeEnvironment</i> The runtime environment for the application (<code>SQL-1.0</code> or <code>FLINK-1_6</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationName</i> The name of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationDescription</i> The description of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationARN</i> The ARN of the application.<br/>The details about inputs, outputs, and reference data sources for an SQL-based Kinesis Data Analytics application.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SqlApplicationConfigurationDescription</b>: The details about inputs, outputs, and reference data sources for an SQL-based Kinesis Data Analytics application.        """
        pass

    def create_application_snapshot(self, SnapshotName: str, ApplicationName: str) -> CreateApplicationSnapshotResponse:
        """Creates a snapshot of the application's state data.

        :param self:
        :param SnapshotName:  An identifier for the application snapshot.<br/>ApplicationName:  The name of an existing application<br/>
        :return:         """
        pass

    def delete_application(self, CreateTimestamp: Timestamp, ApplicationName: str) -> DeleteApplicationResponse:
        """Deletes the specified application. Kinesis Data Analytics halts application execution and deletes the application.

        :param self:
        :param CreateTimestamp:  Use the <code>DescribeApplication</code> operation to get this value.<br/>ApplicationName:  The name of the application to delete.<br/>
        :return:         """
        pass

    def delete_application_cloud_watch_logging_option(self, CloudWatchLoggingOptionId: str, CurrentApplicationVersionId: ApplicationVersionId, ApplicationName: str) -> DeleteApplicationCloudWatchLoggingOptionResponse:
        """Deletes an Amazon CloudWatch log stream from an Amazon Kinesis Data Analytics application. 

        :param self:
        :param CloudWatchLoggingOptionId:  The <code>CloudWatchLoggingOptionId</code> of the Amazon CloudWatch logging option to delete. You can get the <code>CloudWatchLoggingOptionId</code> by using the <a>DescribeApplication</a> operation. <br/>CurrentApplicationVersionId:  The version ID of the application. You can retrieve the application version ID using <a>DescribeApplication</a>.<br/>ApplicationName:  The application name.<br/>
        :return: CloudWatchLoggingOptionDescriptions:  The descriptions of the remaining CloudWatch logging options for the application.<br/>ApplicationVersionId:  The version ID of the application. Kinesis Data Analytics updates the <code>ApplicationVersionId</code> each time you change the CloudWatch logging options.<br/>ApplicationARN:  The application's Amazon Resource Name (ARN).<br/>        """
        pass

    def delete_application_input_processing_configuration(self, InputId: str, CurrentApplicationVersionId: ApplicationVersionId, ApplicationName: str) -> DeleteApplicationInputProcessingConfigurationResponse:
        """Deletes an <a>InputProcessingConfiguration</a> from an input.

        :param self:
        :param InputId:  The ID of the input configuration from which to delete the input processing configuration. You can get a list of the input IDs for an application by using the <a>DescribeApplication</a> operation.<br/>CurrentApplicationVersionId:  The application version. You can use the <a>DescribeApplication</a> operation to get the current application version. If the version specified is not the current version, the <code>ConcurrentModificationException</code> is returned. <br/>ApplicationName:  The name of the application.<br/>
        :return: ApplicationVersionId:  The current application version ID.<br/>ApplicationARN:  The Amazon Resource Name (ARN) of the application.<br/>        """
        pass

    def delete_application_output(self, OutputId: str, CurrentApplicationVersionId: ApplicationVersionId, ApplicationName: str) -> DeleteApplicationOutputResponse:
        """Deletes the output destination configuration from your SQL-based Amazon Kinesis Data Analytics application's configuration. Kinesis Data Analytics will no longer write data from the corresponding in-application stream to the external output destination.

        :param self:
        :param OutputId:  The ID of the configuration to delete. Each output configuration that is added to the application (either when the application is created or later) using the <a>AddApplicationOutput</a> operation has a unique ID. You need to provide the ID to uniquely identify the output configuration that you want to delete from the application configuration. You can use the <a>DescribeApplication</a> operation to get the specific <code>OutputId</code>. <br/>CurrentApplicationVersionId:  The application version. You can use the <a>DescribeApplication</a> operation to get the current application version. If the version specified is not the current version, the <code>ConcurrentModificationException</code> is returned. <br/>ApplicationName:  The application name.<br/>
        :return: ApplicationVersionId:  The current application version ID.<br/>ApplicationARN:  The application Amazon Resource Name (ARN).<br/>        """
        pass

    def delete_application_reference_data_source(self, ReferenceId: str, CurrentApplicationVersionId: ApplicationVersionId, ApplicationName: str) -> DeleteApplicationReferenceDataSourceResponse:
        """Deletes a reference data source configuration from the specified SQL-based Amazon Kinesis Data Analytics application's configuration. If the application is running, Kinesis Data Analytics immediately removes the in-application table that you created using the <a>AddApplicationReferenceDataSource</a> operation. 

        :param self:
        :param ReferenceId:  The ID of the reference data source. When you add a reference data source to your application using the <a>AddApplicationReferenceDataSource</a>, Kinesis Data Analytics assigns an ID. You can use the <a>DescribeApplication</a> operation to get the reference ID. <br/>CurrentApplicationVersionId:  The current application version. You can use the <a>DescribeApplication</a> operation to get the current application version. If the version specified is not the current version, the <code>ConcurrentModificationException</code> is returned.<br/>ApplicationName:  The name of an existing application.<br/>
        :return: ApplicationVersionId:  The updated version ID of the application.<br/>ApplicationARN:  The application Amazon Resource Name (ARN).<br/>        """
        pass

    def delete_application_snapshot(self, SnapshotCreationTimestamp: Timestamp, SnapshotName: str, ApplicationName: str) -> DeleteApplicationSnapshotResponse:
        """Deletes a snapshot of application state.

        :param self:
        :param SnapshotCreationTimestamp:  The creation timestamp of the application snapshot to delete. You can retrieve this value using or .<br/>SnapshotName:  The identifier for the snapshot delete.<br/>ApplicationName:  The name of an existing application.<br/>
        :return:         """
        pass

    def describe_application(self, ApplicationName: str, IncludeAdditionalDetails: BooleanObject=None) -> DescribeApplicationResponse:
        """Returns information about a specific Amazon Kinesis Data Analytics application. If you want to retrieve a list of all applications in your account, use the <a>ListApplications</a> operation.

        :param self:
        :param IncludeAdditionalDetails:  Displays verbose information about a Kinesis Data Analytics application, including the application's job plan.<br/>ApplicationName:  The name of the application.<br/>
        :return: ApplicationDetail:  Provides a description of the application, such as the application's Amazon Resource Name (ARN), status, and latest version.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchLoggingOptionDescriptions</i> Describes the application Amazon CloudWatch logging options.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationConfigurationDescription</i> Provides details about the application's SQL or Java code and starting parameters.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ApplicationConfigurationDescription</b>: Provides details about the application's SQL or Java code and starting parameters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdateTimestamp</i> The current timestamp when the application was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateTimestamp</i> The current timestamp when the application was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationVersionId</i> Provides the current application version. Kinesis Data Analytics updates the <code>ApplicationVersionId</code> each time you update the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationStatus</i> The status of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceExecutionRole</i> Specifies the IAM role that the application uses to access external resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuntimeEnvironment</i> The runtime environment for the application (<code>SQL-1.0</code> or <code>FLINK-1_6</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationName</i> The name of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationDescription</i> The description of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationARN</i> The ARN of the application.<br/>The details about inputs, outputs, and reference data sources for an SQL-based Kinesis Data Analytics application.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SqlApplicationConfigurationDescription</b>: The details about inputs, outputs, and reference data sources for an SQL-based Kinesis Data Analytics application.        """
        pass

    def describe_application_snapshot(self, SnapshotName: str, ApplicationName: str) -> DescribeApplicationSnapshotResponse:
        """Returns information about a snapshot of application state data.

        :param self:
        :param SnapshotName:  The identifier of an application snapshot. You can retrieve this value using .<br/>ApplicationName:  The name of an existing application.<br/>
        :return: SnapshotDetails:  An object containing information about the application snapshot.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCreationTimestamp</i> The timestamp of the application snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationVersionId</i> The current application version ID when the snapshot was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotStatus</i> The status of the application snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotName</i> The identifier for the application snapshot.<br/>        """
        pass

    def discover_input_schema(self, ServiceExecutionRole: str, ResourceARN: str=None, InputStartingPositionConfiguration: InputStartingPositionConfiguration=None, S3Configuration: S3Configuration=None, InputProcessingConfiguration: InputProcessingConfiguration=None) -> DiscoverInputSchemaResponse:
        """Infers a schema for an SQL-based Amazon Kinesis Data Analytics application by evaluating sample records on the specified streaming source (Kinesis data stream or Kinesis Data Firehose delivery stream) or Amazon S3 object. In the response, the operation returns the inferred schema and also the sample records that the operation used to infer the schema.  You can use the inferred schema when configuring a streaming source for your application. When you create an application using the Kinesis Data Analytics console, the console uses this operation to infer a schema and show it in the console user interface. 

        :param self:
        :param InputProcessingConfiguration:  The <a>InputProcessingConfiguration</a> to use to preprocess the records before discovering the schema of the records.:<br/>S3Configuration:  Specify this parameter to discover a schema from data in an Amazon S3 object.:<br/>InputStartingPositionConfiguration:  The point at which you want Kinesis Data Analytics to start reading records from the specified streaming source discovery purposes.:<br/>ServiceExecutionRole:  The ARN of the role that is used to access the streaming source.<br/>ResourceARN:  The Amazon Resource Name (ARN) of the streaming source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputStartingPosition</i> The starting position on the stream. <ul> <li>  <code>NOW</code> - Start reading just after the most recent record in the stream, and start at the request timestamp that the customer issued. </li> <li>  <code>TRIM_HORIZON</code> - Start reading at the last untrimmed record in the stream, which is the oldest record available in the stream. This option is not available for an Amazon Kinesis Data Firehose delivery stream. </li> <li>  <code>LAST_STOPPED_POINT</code> - Resume reading from where the application last stopped reading. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FileKey</i> The name of the object that contains the data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketARN</i> The ARN of the S3 bucket that contains the data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputLambdaProcessor</i> The <a>InputLambdaProcessor</a> that is used to preprocess the records in the stream before being processed by your application code.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputLambdaProcessor</b>: The <a>InputLambdaProcessor</a> that is used to preprocess the records in the stream before being processed by your application code.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceARN</i> The ARN of the AWS Lambda function that operates on records in the stream. <note> To specify an earlier version of the Lambda function than the latest, include the Lambda function version in the Lambda function ARN. For more information about Lambda ARNs, see <a href="/general/latest/gr/aws-arns-and-namespaces.html#arn-syntax-lambda">Example ARNs: AWS Lambda</a>  </note><br/>
        :return: RawInputRecords:  The raw stream data that was sampled to infer the schema.<br/>ProcessedInputRecords:  The stream data that was modified by the processor specified in the <code>InputProcessingConfiguration</code> parameter.<br/>ParsedInputRecords:  An array of elements, where each element corresponds to a row in a stream record (a stream record can have more than one row).<br/>InputSchema:  The schema inferred from the streaming source. It identifies the format of the data in the streaming source and how each data element maps to corresponding columns in the in-application stream that you can create.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordColumns</i> A list of <code>RecordColumn</code> objects. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordEncoding</i> Specifies the encoding of the records in the streaming source. For example, UTF-8.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordFormat</i> Specifies the format of the records on the streaming source.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RecordFormat</b>: Specifies the format of the records on the streaming source.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordFormatType</i> The type of record format.<br/>When you configure application input at the time of creating or updating an application, provides additional mapping information specific to the record format (such as JSON, CSV, or record fields delimited by some delimiter) on the streaming source.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>MappingParameters</b>: When you configure application input at the time of creating or updating an application, provides additional mapping information specific to the record format (such as JSON, CSV, or record fields delimited by some delimiter) on the streaming source.        """
        pass

    def list_application_snapshots(self, ApplicationName: str, Limit: int=None, NextToken: str=None) -> ListApplicationSnapshotsResponse:
        """Lists information about the current application snapshots.

        :param self:
        :param NextToken:  Use this parameter if you receive a <code>NextToken</code> response in a previous request that indicates that there is more output available. Set it to the value of the previous call's <code>NextToken</code> response to indicate where the output should continue from. <br/>Limit:  The maximum number of application snapshots to list.<br/>ApplicationName:  The name of an existing application.<br/>
        :return: NextToken:  The token for the next set of results, or <code>null</code> if there are no additional results.<br/>SnapshotSummaries:  A collection of objects containing information about the application snapshots.<br/>        """
        pass

    def list_applications(self, Limit: int=None, NextToken: str=None) -> ListApplicationsResponse:
        """Returns a list of Amazon Kinesis Data Analytics applications in your account. For each application, the response includes the application name, Amazon Resource Name (ARN), and status.  If you want detailed information about a specific application, use <a>DescribeApplication</a>.

        :param self:
        :param NextToken:  If a previous command returned a pagination token, pass it into this value to retrieve the next set of results. For more information about pagination, see <a href="https://docs.aws.amazon.com/cli/latest/userguide/pagination.html">Using the AWS Command Line Interface's Pagination Options</a>.<br/>Limit:  The maximum number of applications to list.<br/>
        :return: NextToken:  The pagination token for the next set of results, or <code>null</code> if there are no additional results. Pass this token into a subsequent command to retrieve the next set of items For more information about pagination, see <a href="https://docs.aws.amazon.com/cli/latest/userguide/pagination.html">Using the AWS Command Line Interface's Pagination Options</a>.<br/>ApplicationSummaries:  A list of <code>ApplicationSummary</code> objects.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceARN: str) -> ListTagsForResourceResponse:
        """Retrieves the list of key-value tags assigned to the application.

        :param self:
        :param ResourceARN:  The ARN of the application for which to retrieve tags.<br/>
        :return: Tags:  The key-value tags assigned to the application.<br/>        """
        pass

    def start_application(self, RunConfiguration: RunConfiguration, ApplicationName: str) -> StartApplicationResponse:
        """Starts the specified Amazon Kinesis Data Analytics application. After creating an application, you must exclusively call this operation to start your application.

        :param self:
        :param RunConfiguration:  Identifies the run configuration (start parameters) of a Kinesis Data Analytics application.:<br/>ApplicationName:  The name of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationRestoreConfiguration</i> Describes the restore behavior of a restarting application.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ApplicationRestoreConfiguration</b>: Describes the restore behavior of a restarting application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SqlRunConfigurations</i> Describes the starting parameters for an SQL-based Kinesis Data Analytics application.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotName</i> The identifier of an existing snapshot of application state to use to restart an application. The application uses this value if <code>RESTORE_FROM_CUSTOM_SNAPSHOT</code> is specified for the <code>ApplicationRestoreType</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationRestoreType</i> Specifies how the application should be restored.<br/>
        :return:         """
        pass

    def stop_application(self, ApplicationName: str) -> StopApplicationResponse:
        """Stops the application from processing data. You can stop an application only if it is in the running state. You can use the <a>DescribeApplication</a> operation to find the application state. 

        :param self:
        :param ApplicationName:  The name of the running application to stop.<br/>
        :return:         """
        pass

    def tag_resource(self, Tags: List[Tag], ResourceARN: str) -> TagResourceResponse:
        """Adds one or more key-value tags to a Kinesis Analytics application. Note that the maximum number of application tags includes system tags. The maximum number of user-defined application tags is 50.

        :param self:
        :param Tags:  The key-value tags to assign to the application.<br/>ResourceARN:  The ARN of the application to assign the tags.<br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[TagKey], ResourceARN: str) -> UntagResourceResponse:
        """Removes one or more tags from a Kinesis Analytics application.

        :param self:
        :param TagKeys:  A list of keys of tags to remove from the specified application.<br/>ResourceARN:  The ARN of the Kinesis Analytics application from which to remove the tags.<br/>
        :return:         """
        pass

    def update_application(self, CurrentApplicationVersionId: ApplicationVersionId, ApplicationName: str, ApplicationConfigurationUpdate: ApplicationConfigurationUpdate=None, ServiceExecutionRoleUpdate: str=None, RunConfigurationUpdate: RunConfigurationUpdate=None, CloudWatchLoggingOptionUpdates: List[CloudWatchLoggingOptionUpdate]=None) -> UpdateApplicationResponse:
        """Updates an existing Amazon Kinesis Data Analytics application. Using this operation, you can update application code, input configuration, and output configuration.  Kinesis Data Analytics updates the <code>ApplicationVersionId</code> each time you update your application. 

        :param self:
        :param CloudWatchLoggingOptionUpdates:  Describes application Amazon CloudWatch logging option updates. You can only update existing CloudWatch logging options with this action. To add a new CloudWatch logging option, use <a>AddApplicationCloudWatchLoggingOption</a>.<br/>RunConfigurationUpdate:  Describes updates to the application's starting parameters.:<br/>ServiceExecutionRoleUpdate:  Describes updates to the service execution role.<br/>ApplicationConfigurationUpdate:  Describes application configuration updates.:<br/>CurrentApplicationVersionId:  The current application version ID. You can retrieve the application version ID using <a>DescribeApplication</a>.<br/>ApplicationName:  The name of the application to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationSnapshotConfigurationUpdate</i> Describes whether snapshots are enabled for a Java-based Kinesis Data Analytics application.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ApplicationSnapshotConfigurationUpdate</b>: Describes whether snapshots are enabled for a Java-based Kinesis Data Analytics application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnvironmentPropertyUpdates</i> Describes updates to the environment properties for a Java-based Kinesis Data Analytics application.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EnvironmentPropertyUpdates</b>: Describes updates to the environment properties for a Java-based Kinesis Data Analytics application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FlinkApplicationConfigurationUpdate</i> Describes updates to a Java-based Kinesis Data Analytics application's configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>FlinkApplicationConfigurationUpdate</b>: Describes updates to a Java-based Kinesis Data Analytics application's configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationCodeConfigurationUpdate</i> Describes updates to a Java-based Kinesis Data Analytics application's code configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ApplicationCodeConfigurationUpdate</b>: Describes updates to a Java-based Kinesis Data Analytics application's code configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SqlApplicationConfigurationUpdate</i> Describes updates to an SQL-based Kinesis Data Analytics application's configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SqlApplicationConfigurationUpdate</b>: Describes updates to an SQL-based Kinesis Data Analytics application's configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReferenceDataSourceUpdates</i> The array of <a>ReferenceDataSourceUpdate</a> objects describing the new reference data sources used by the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputUpdates</i> The array of <a>OutputUpdate</a> objects describing the new destination streams used by the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InputUpdates</i> The array of <a>InputUpdate</a> objects describing the new input streams used by the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CodeContentTypeUpdate</i> Describes updates to the code content type.<br/>Describes updates to the code content of an application.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CodeContentUpdate</b>: Describes updates to the code content of an application.Describes updates to an application's checkpointing configuration. Checkpointing is the process of persisting application state for fault tolerance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CheckpointConfigurationUpdate</b>: Describes updates to an application's checkpointing configuration. Checkpointing is the process of persisting application state for fault tolerance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PropertyGroups</i> Describes updates to the execution property groups.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotsEnabledUpdate</i> Describes updates to whether snapshots are enabled for a Java-based Kinesis Data Analytics application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationRestoreConfiguration</i> Describes updates to the restore behavior of a restarting application.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ApplicationRestoreConfiguration</b>: Describes updates to the restore behavior of a restarting application.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotName</i> The identifier of an existing snapshot of application state to use to restart an application. The application uses this value if <code>RESTORE_FROM_CUSTOM_SNAPSHOT</code> is specified for the <code>ApplicationRestoreType</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationRestoreType</i> Specifies how the application should be restored.<br/>
        :return: ApplicationDetail:  Describes application updates.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchLoggingOptionDescriptions</i> Describes the application Amazon CloudWatch logging options.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationConfigurationDescription</i> Provides details about the application's SQL or Java code and starting parameters.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ApplicationConfigurationDescription</b>: Provides details about the application's SQL or Java code and starting parameters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdateTimestamp</i> The current timestamp when the application was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateTimestamp</i> The current timestamp when the application was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationVersionId</i> Provides the current application version. Kinesis Data Analytics updates the <code>ApplicationVersionId</code> each time you update the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationStatus</i> The status of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceExecutionRole</i> Specifies the IAM role that the application uses to access external resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuntimeEnvironment</i> The runtime environment for the application (<code>SQL-1.0</code> or <code>FLINK-1_6</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationName</i> The name of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationDescription</i> The description of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationARN</i> The ARN of the application.<br/>The details about inputs, outputs, and reference data sources for an SQL-based Kinesis Data Analytics application.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SqlApplicationConfigurationDescription</b>: The details about inputs, outputs, and reference data sources for an SQL-based Kinesis Data Analytics application.        """
        pass



class KinesisVideo(BaseClient):
    """<p/>"""


    class ListStreamsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_streams` for valid parameters"""
            pass


    class CreateStreamInput(Mapping):
        pass
    
    class CreateStreamOutput(Mapping):
        pass
    
    class ResourceTags(object):
        pass
    
    class DeleteStreamInput(Mapping):
        pass
    
    class DeleteStreamOutput(Mapping):
        pass
    
    class DescribeStreamInput(Mapping):
        pass
    
    class DescribeStreamOutput(Mapping):
        pass
    
    class GetDataEndpointInput(Mapping):
        pass
    
    class GetDataEndpointOutput(Mapping):
        pass
    
    class ListStreamsInput(Mapping):
        pass
    
    class ListStreamsOutput(Mapping):
        pass
    
    class StreamNameCondition(Mapping):
        pass
    
    class ListTagsForStreamInput(Mapping):
        pass
    
    class ListTagsForStreamOutput(Mapping):
        pass
    
    class TagStreamInput(Mapping):
        pass
    
    class TagStreamOutput(Mapping):
        pass
    
    class UntagStreamInput(Mapping):
        pass
    
    class UntagStreamOutput(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateDataRetentionInput(Mapping):
        pass
    
    class UpdateDataRetentionOutput(Mapping):
        pass
    
    class UpdateStreamInput(Mapping):
        pass
    
    class UpdateStreamOutput(Mapping):
        pass
    
    def create_stream(self, StreamName: str, DeviceName: str=None, MediaType: str=None, KmsKeyId: str=None, DataRetentionInHours: int=None, Tags: ResourceTags=None) -> CreateStreamOutput:
        """Creates a new Kinesis video stream.  When you create a new stream, Kinesis Video Streams assigns it a version number. When you change the stream's metadata, Kinesis Video Streams updates the version.   <code>CreateStream</code> is an asynchronous operation. For information about how the service works, see <a href="https://docs.aws.amazon.com/kinesisvideostreams/latest/dg/how-it-works.html">How it Works</a>.  You must have permissions for the <code>KinesisVideo:CreateStream</code> action.

        :param self:
        :param Tags:  A list of tags to associate with the specified stream. Each tag is a key-value pair (the value is optional).<br/>DataRetentionInHours:  The number of hours that you want to retain the data in the stream. Kinesis Video Streams retains the data in a data store that is associated with the stream. The default value is 0, indicating that the stream does not persist data. When the <code>DataRetentionInHours</code> value is 0, consumers can still consume the fragments that remain in the service host buffer, which has a retention time limit of 5 minutes and a retention memory limit of 200 MB. Fragments are removed from the buffer when either limit is reached.<br/>KmsKeyId:  The ID of the AWS Key Management Service (AWS KMS) key that you want Kinesis Video Streams to use to encrypt stream data. If no key ID is specified, the default, Kinesis Video-managed key (<code>aws/kinesisvideo</code>) is used.  For more information, see <a href="https://docs.aws.amazon.com/kms/latest/APIReference/API_DescribeKey.html#API_DescribeKey_RequestParameters">DescribeKey</a>. <br/>MediaType:  The media type of the stream. Consumers of the stream can use this information when processing the stream. For more information about media types, see <a href="http://www.iana.org/assignments/media-types/media-types.xhtml">Media Types</a>. If you choose to specify the <code>MediaType</code>, see <a href="https://tools.ietf.org/html/rfc6838#section-4.2">Naming Requirements</a> for guidelines. This parameter is optional; the default value is <code>null</code> (or empty in JSON).<br/>StreamName:  A name for the stream that you are creating. The stream name is an identifier for the stream, and must be unique for each account and region.<br/>DeviceName:  The name of the device that is writing to the stream.  <note> In the current implementation, Kinesis Video Streams does not use this name. </note><br/>
        :return: StreamARN:  The Amazon Resource Name (ARN) of the stream.<br/>        """
        pass

    def delete_stream(self, StreamARN: str, CurrentVersion: str=None) -> DeleteStreamOutput:
        """Deletes a Kinesis video stream and the data contained in the stream.  This method marks the stream for deletion, and makes the data in the stream inaccessible immediately.    To ensure that you have the latest version of the stream before deleting it, you can specify the stream version. Kinesis Video Streams assigns a version to each stream. When you update a stream, Kinesis Video Streams assigns a new version number. To get the latest stream version, use the <code>DescribeStream</code> API.  This operation requires permission for the <code>KinesisVideo:DeleteStream</code> action.

        :param self:
        :param CurrentVersion:  Optional: The version of the stream that you want to delete.  Specify the version as a safeguard to ensure that your are deleting the correct stream. To get the stream version, use the <code>DescribeStream</code> API. If not specified, only the <code>CreationTime</code> is checked before deleting the stream.<br/>StreamARN:  The Amazon Resource Name (ARN) of the stream that you want to delete. <br/>
        :return:         """
        pass

    def describe_stream(self, StreamName: str=None, StreamARN: str=None) -> DescribeStreamOutput:
        """Returns the most current information about the specified stream. You must specify either the <code>StreamName</code> or the <code>StreamARN</code>. 

        :param self:
        :param StreamARN:  The Amazon Resource Name (ARN) of the stream.<br/>StreamName:  The name of the stream.<br/>
        :return: StreamInfo:  An object that describes the stream.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataRetentionInHours</i> How long the stream retains data, in hours.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> A time stamp that indicates when the stream was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version of the stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The ID of the AWS Key Management Service (AWS KMS) key that Kinesis Video Streams uses to encrypt data on the stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MediaType</i> The <code>MediaType</code> of the stream. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamARN</i> The Amazon Resource Name (ARN) of the stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamName</i> The name of the stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceName</i> The name of the device that is associated with the stream.<br/>        """
        pass

    def get_data_endpoint(self, APIName: str, StreamName: str=None, StreamARN: str=None) -> GetDataEndpointOutput:
        """Gets an endpoint for a specified stream for either reading or writing. Use this endpoint in your application to read from the specified stream (using the <code>GetMedia</code> or <code>GetMediaForFragmentList</code> operations) or write to it (using the <code>PutMedia</code> operation).  <note> The returned endpoint does not have the API name appended. The client needs to add the API name to the returned endpoint. </note> In the request, specify the stream either by <code>StreamName</code> or <code>StreamARN</code>.

        :param self:
        :param APIName:  The name of the API action for which to get an endpoint.<br/>StreamARN:  The Amazon Resource Name (ARN) of the stream that you want to get the endpoint for. You must specify either this parameter or a <code>StreamName</code> in the request. <br/>StreamName:  The name of the stream that you want to get the endpoint for. You must specify either this parameter or a <code>StreamARN</code> in the request.<br/>
        :return: DataEndpoint:  The endpoint value. To read data from the stream or to write data to it, specify this endpoint in your application.<br/>        """
        pass

    def list_streams(self, MaxResults: int=None, NextToken: str=None, StreamNameCondition: StreamNameCondition=None) -> ListStreamsOutput:
        """Returns an array of <code>StreamInfo</code> objects. Each object describes a stream. To retrieve only streams that satisfy a specific condition, you can specify a <code>StreamNameCondition</code>. 

        :param self:
        :param StreamNameCondition:  Optional: Returns only streams that satisfy a specific condition. Currently, you can specify only the prefix of a stream name as a condition. :<br/>NextToken:  If you specify this parameter, when the result of a <code>ListStreams</code> operation is truncated, the call returns the <code>NextToken</code> in the response. To get another batch of streams, provide this token in your next request.<br/>MaxResults:  The maximum number of streams to return in the response. The default is 10,000.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ComparisonValue</i> A value to compare.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ComparisonOperator</i> A comparison operator. Currently, you can specify only the <code>BEGINS_WITH</code> operator, which finds streams whose names start with a given prefix.<br/>
        :return: NextToken:  If the response is truncated, the call returns this element with a token. To get the next batch of streams, use this token in your next request. <br/>StreamInfoList:  An array of <code>StreamInfo</code> objects.<br/>        """
        pass

    def list_tags_for_stream(self, NextToken: str=None, StreamARN: str=None, StreamName: str=None) -> ListTagsForStreamOutput:
        """Returns a list of tags associated with the specified stream. In the request, you must specify either the <code>StreamName</code> or the <code>StreamARN</code>. 

        :param self:
        :param StreamName:  The name of the stream that you want to list tags for.<br/>StreamARN:  The Amazon Resource Name (ARN) of the stream that you want to list tags for.<br/>NextToken:  If you specify this parameter and the result of a <code>ListTagsForStream</code> call is truncated, the response includes a token that you can use in the next request to fetch the next batch of tags.<br/>
        :return: Tags:  A map of tag keys and values associated with the specified stream.<br/>NextToken:  If you specify this parameter and the result of a <code>ListTags</code> call is truncated, the response includes a token that you can use in the next request to fetch the next set of tags.<br/>        """
        pass

    def tag_stream(self, Tags: ResourceTags, StreamARN: str=None, StreamName: str=None) -> TagStreamOutput:
        """Adds one or more tags to a stream. A <i>tag</i> is a key-value pair (the value is optional) that you can define and assign to AWS resources. If you specify a tag that already exists, the tag value is replaced with the value that you specify in the request. For more information, see <a href="https://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/cost-alloc-tags.html">Using Cost Allocation Tags</a> in the <i>AWS Billing and Cost Management User Guide</i>.  You must provide either the <code>StreamName</code> or the <code>StreamARN</code>. This operation requires permission for the <code>KinesisVideo:TagStream</code> action. Kinesis video streams support up to 50 tags.

        :param self:
        :param Tags:  A list of tags to associate with the specified stream. Each tag is a key-value pair (the value is optional).<br/>StreamName:  The name of the stream that you want to add the tag or tags to.<br/>StreamARN:  The Amazon Resource Name (ARN) of the resource that you want to add the tag or tags to.<br/>
        :return:         """
        pass

    def untag_stream(self, TagKeyList: List[TagKey], StreamARN: str=None, StreamName: str=None) -> UntagStreamOutput:
        """Removes one or more tags from a stream. In the request, specify only a tag key or keys; don't specify the value. If you specify a tag key that does not exist, it's ignored. In the request, you must provide the <code>StreamName</code> or <code>StreamARN</code>.

        :param self:
        :param TagKeyList:  A list of the keys of the tags that you want to remove.<br/>StreamName:  The name of the stream that you want to remove tags from.<br/>StreamARN:  The Amazon Resource Name (ARN) of the stream that you want to remove tags from.<br/>
        :return:         """
        pass

    def update_data_retention(self, DataRetentionChangeInHours: int, Operation: str, CurrentVersion: str, StreamName: str=None, StreamARN: str=None) -> UpdateDataRetentionOutput:
        """ Increases or decreases the stream's data retention period by the value that you specify. To indicate whether you want to increase or decrease the data retention period, specify the <code>Operation</code> parameter in the request body. In the request, you must specify either the <code>StreamName</code> or the <code>StreamARN</code>.  <note> The retention period that you specify replaces the current value. </note> This operation requires permission for the <code>KinesisVideo:UpdateDataRetention</code> action. Changing the data retention period affects the data in the stream as follows: <ul> <li> If the data retention period is increased, existing data is retained for the new retention period. For example, if the data retention period is increased from one hour to seven hours, all existing data is retained for seven hours. </li> <li> If the data retention period is decreased, existing data is retained for the new retention period. For example, if the data retention period is decreased from seven hours to one hour, all existing data is retained for one hour, and any data older than one hour is deleted immediately. </li> </ul>

        :param self:
        :param DataRetentionChangeInHours:  The retention period, in hours. The value you specify replaces the current value. The maximum value for this parameter is 87600 (ten years).<br/>Operation:  Indicates whether you want to increase or decrease the retention period.<br/>CurrentVersion:  The version of the stream whose retention period you want to change. To get the version, call either the <code>DescribeStream</code> or the <code>ListStreams</code> API.<br/>StreamARN:  The Amazon Resource Name (ARN) of the stream whose retention period you want to change.<br/>StreamName:  The name of the stream whose retention period you want to change.<br/>
        :return:         """
        pass

    def update_stream(self, CurrentVersion: str, StreamName: str=None, StreamARN: str=None, DeviceName: str=None, MediaType: str=None) -> UpdateStreamOutput:
        """Updates stream metadata, such as the device name and media type. You must provide the stream name or the Amazon Resource Name (ARN) of the stream. To make sure that you have the latest version of the stream before updating it, you can specify the stream version. Kinesis Video Streams assigns a version to each stream. When you update a stream, Kinesis Video Streams assigns a new version number. To get the latest stream version, use the <code>DescribeStream</code> API.   <code>UpdateStream</code> is an asynchronous operation, and takes time to complete.

        :param self:
        :param MediaType:  The stream's media type. Use <code>MediaType</code> to specify the type of content that the stream contains to the consumers of the stream. For more information about media types, see <a href="http://www.iana.org/assignments/media-types/media-types.xhtml">Media Types</a>. If you choose to specify the <code>MediaType</code>, see <a href="https://tools.ietf.org/html/rfc6838#section-4.2">Naming Requirements</a>. To play video on the console, you must specify the correct video type. For example, if the video in the stream is H.264, specify <code>video/h264</code> as the <code>MediaType</code>.<br/>DeviceName:  The name of the device that is writing to the stream.  <note>  In the current implementation, Kinesis Video Streams does not use this name.  </note><br/>CurrentVersion:  The version of the stream whose metadata you want to update.<br/>StreamARN:  The ARN of the stream whose metadata you want to update.<br/>StreamName:  The name of the stream whose metadata you want to update. The stream name is an identifier for the stream, and must be unique for each account and region.<br/>
        :return:         """
        pass



class KMS(BaseClient):
    """<fullname>AWS Key Management Service</fullname> AWS Key Management Service (AWS KMS) is an encryption and key management web service. This guide describes the AWS KMS operations that you can call programmatically. For general information about AWS KMS, see the <a href="https://docs.aws.amazon.com/kms/latest/developerguide/"> <i>AWS Key Management Service Developer Guide</i> </a>. <note> AWS provides SDKs that consist of libraries and sample code for various programming languages and platforms (Java, Ruby, .Net, macOS, Android, etc.). The SDKs provide a convenient way to create programmatic access to AWS KMS and other AWS services. For example, the SDKs take care of tasks such as signing requests (see below), managing errors, and retrying requests automatically. For more information about the AWS SDKs, including how to download and install them, see <a href="http://aws.amazon.com/tools/">Tools for Amazon Web Services</a>. </note> We recommend that you use the AWS SDKs to make programmatic API calls to AWS KMS. Clients must support TLS (Transport Layer Security) 1.0. We recommend TLS 1.2. Clients must also support cipher suites with Perfect Forward Secrecy (PFS) such as Ephemeral Diffie-Hellman (DHE) or Elliptic Curve Ephemeral Diffie-Hellman (ECDHE). Most modern systems such as Java 7 and later support these modes.  <b>Signing Requests</b>  Requests must be signed by using an access key ID and a secret access key. We strongly recommend that you <i>do not</i> use your AWS account (root) access key ID and secret key for everyday work with AWS KMS. Instead, use the access key ID and secret access key for an IAM user. You can also use the AWS Security Token Service to generate temporary security credentials that you can use to sign requests. All AWS KMS operations require <a href="https://docs.aws.amazon.com/general/latest/gr/signature-version-4.html">Signature Version 4</a>.  <b>Logging API Requests</b>  AWS KMS supports AWS CloudTrail, a service that logs AWS API calls and related events for your AWS account and delivers them to an Amazon S3 bucket that you specify. By using the information collected by CloudTrail, you can determine what requests were made to AWS KMS, who made the request, when it was made, and so on. To learn more about CloudTrail, including how to turn it on and find your log files, see the <a href="https://docs.aws.amazon.com/awscloudtrail/latest/userguide/">AWS CloudTrail User Guide</a>.  <b>Additional Resources</b>  For more information about credentials and request signing, see the following: <ul> <li>  <a href="https://docs.aws.amazon.com/general/latest/gr/aws-security-credentials.html">AWS Security Credentials</a> - This topic provides general information about the types of credentials used for accessing AWS. </li> <li>  <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_temp.html">Temporary Security Credentials</a> - This section of the <i>IAM User Guide</i> describes how to create and use temporary security credentials. </li> <li>  <a href="https://docs.aws.amazon.com/general/latest/gr/signature-version-4.html">Signature Version 4 Signing Process</a> - This set of topics walks you through the process of signing a request using an access key ID and a secret access key. </li> </ul>  <b>Commonly Used API Operations</b>  Of the API operations discussed in this guide, the following will prove the most useful for most applications. You will likely perform operations other than these, such as creating keys and assigning policies, by using the console. <ul> <li>  <a>Encrypt</a>  </li> <li>  <a>Decrypt</a>  </li> <li>  <a>GenerateDataKey</a>  </li> <li>  <a>GenerateDataKeyWithoutPlaintext</a>  </li> </ul>"""


    class ListAliasesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_aliases` for valid parameters"""
            pass
    class ListGrantsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_grants` for valid parameters"""
            pass
    class ListKeyPoliciesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_key_policies` for valid parameters"""
            pass
    class ListKeysPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_keys` for valid parameters"""
            pass


    class CancelKeyDeletionRequest(Mapping):
        pass
    
    class CancelKeyDeletionResponse(Mapping):
        pass
    
    class ConnectCustomKeyStoreRequest(Mapping):
        pass
    
    class ConnectCustomKeyStoreResponse(Mapping):
        pass
    
    class CreateAliasRequest(Mapping):
        pass
    
    class CreateCustomKeyStoreRequest(Mapping):
        pass
    
    class CreateCustomKeyStoreResponse(Mapping):
        pass
    
    class CreateGrantRequest(Mapping):
        pass
    
    class CreateGrantResponse(Mapping):
        pass
    
    class GrantOperation(object):
        pass
    
    class GrantConstraints(Mapping):
        pass
    
    class GrantTokenType(object):
        pass
    
    class CreateKeyRequest(Mapping):
        pass
    
    class CreateKeyResponse(Mapping):
        pass
    
    class BooleanType(object):
        pass
    
    class Tag(Mapping):
        pass
    
    class DecryptRequest(Mapping):
        pass
    
    class DecryptResponse(Mapping):
        pass
    
    class CiphertextType(object):
        pass
    
    class EncryptionContextType(object):
        pass
    
    class DeleteAliasRequest(Mapping):
        pass
    
    class DeleteCustomKeyStoreRequest(Mapping):
        pass
    
    class DeleteCustomKeyStoreResponse(Mapping):
        pass
    
    class DeleteImportedKeyMaterialRequest(Mapping):
        pass
    
    class DescribeCustomKeyStoresRequest(Mapping):
        pass
    
    class DescribeCustomKeyStoresResponse(Mapping):
        pass
    
    class DescribeKeyRequest(Mapping):
        pass
    
    class DescribeKeyResponse(Mapping):
        pass
    
    class DisableKeyRequest(Mapping):
        pass
    
    class DisableKeyRotationRequest(Mapping):
        pass
    
    class DisconnectCustomKeyStoreRequest(Mapping):
        pass
    
    class DisconnectCustomKeyStoreResponse(Mapping):
        pass
    
    class EnableKeyRequest(Mapping):
        pass
    
    class EnableKeyRotationRequest(Mapping):
        pass
    
    class EncryptRequest(Mapping):
        pass
    
    class EncryptResponse(Mapping):
        pass
    
    class PlaintextType(object):
        pass
    
    class GenerateDataKeyRequest(Mapping):
        pass
    
    class GenerateDataKeyResponse(Mapping):
        pass
    
    class GenerateDataKeyWithoutPlaintextRequest(Mapping):
        pass
    
    class GenerateDataKeyWithoutPlaintextResponse(Mapping):
        pass
    
    class GenerateRandomRequest(Mapping):
        pass
    
    class GenerateRandomResponse(Mapping):
        pass
    
    class GetKeyPolicyRequest(Mapping):
        pass
    
    class GetKeyPolicyResponse(Mapping):
        pass
    
    class GetKeyRotationStatusRequest(Mapping):
        pass
    
    class GetKeyRotationStatusResponse(Mapping):
        pass
    
    class GetParametersForImportRequest(Mapping):
        pass
    
    class GetParametersForImportResponse(Mapping):
        pass
    
    class ImportKeyMaterialRequest(Mapping):
        pass
    
    class ImportKeyMaterialResponse(Mapping):
        pass
    
    class DateType(object):
        pass
    
    class ListAliasesRequest(Mapping):
        pass
    
    class ListAliasesResponse(Mapping):
        pass
    
    class ListGrantsRequest(Mapping):
        pass
    
    class ListGrantsResponse(Mapping):
        pass
    
    class ListKeyPoliciesRequest(Mapping):
        pass
    
    class ListKeyPoliciesResponse(Mapping):
        pass
    
    class ListKeysRequest(Mapping):
        pass
    
    class ListKeysResponse(Mapping):
        pass
    
    class ListResourceTagsRequest(Mapping):
        pass
    
    class ListResourceTagsResponse(Mapping):
        pass
    
    class ListRetirableGrantsRequest(Mapping):
        pass
    
    class PutKeyPolicyRequest(Mapping):
        pass
    
    class ReEncryptRequest(Mapping):
        pass
    
    class ReEncryptResponse(Mapping):
        pass
    
    class RetireGrantRequest(Mapping):
        pass
    
    class RevokeGrantRequest(Mapping):
        pass
    
    class ScheduleKeyDeletionRequest(Mapping):
        pass
    
    class ScheduleKeyDeletionResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class TagKeyType(object):
        pass
    
    class UpdateAliasRequest(Mapping):
        pass
    
    class UpdateCustomKeyStoreRequest(Mapping):
        pass
    
    class UpdateCustomKeyStoreResponse(Mapping):
        pass
    
    class UpdateKeyDescriptionRequest(Mapping):
        pass
    
    def cancel_key_deletion(self, KeyId: str) -> CancelKeyDeletionResponse:
        """Cancels the deletion of a customer master key (CMK). When this operation is successful, the CMK is set to the <code>Disabled</code> state. To enable a CMK, use <a>EnableKey</a>. You cannot perform this operation on a CMK in a different AWS account. For more information about scheduling and canceling deletion of a CMK, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/deleting-keys.html">Deleting Customer Master Keys</a> in the <i>AWS Key Management Service Developer Guide</i>. The result of this operation varies with the key state of the CMK. For details, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-state.html">How Key State Affects Use of a Customer Master Key</a> in the <i>AWS Key Management Service Developer Guide</i>.

        :param self:
        :param KeyId:  The unique identifier for the customer master key (CMK) for which to cancel deletion. Specify the key ID or the Amazon Resource Name (ARN) of the CMK. For example: <ul> <li> Key ID: <code>1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Key ARN: <code>arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> </ul> To get the key ID and key ARN for a CMK, use <a>ListKeys</a> or <a>DescribeKey</a>.<br/>
        :return: KeyId:  The unique identifier of the master key for which deletion is canceled.<br/>        """
        pass

    def connect_custom_key_store(self, CustomKeyStoreId: str) -> ConnectCustomKeyStoreResponse:
        """Connects or reconnects a <a href="https://docs.aws.amazon.com/kms/latest/developerguide/custom-key-store-overview.html">custom key store</a> to its associated AWS CloudHSM cluster. The custom key store must be connected before you can create customer master keys (CMKs) in the key store or use the CMKs it contains. You can disconnect and reconnect a custom key store at any time. To connect a custom key store, its associated AWS CloudHSM cluster must have at least one active HSM. To get the number of active HSMs in a cluster, use the <a href="https://docs.aws.amazon.com/cloudhsm/latest/APIReference/API_DescribeClusters.html">DescribeClusters</a> operation. To add HSMs to the cluster, use the <a href="https://docs.aws.amazon.com/cloudhsm/latest/APIReference/API_CreateHsm.html">CreateHsm</a> operation. The connection process can take an extended amount of time to complete; up to 20 minutes. This operation starts the connection process, but it does not wait for it to complete. When it succeeds, this operation quickly returns an HTTP 200 response and a JSON object with no properties. However, this response does not indicate that the custom key store is connected. To get the connection state of the custom key store, use the <a>DescribeCustomKeyStores</a> operation. During the connection process, AWS KMS finds the AWS CloudHSM cluster that is associated with the custom key store, creates the connection infrastructure, connects to the cluster, logs into the AWS CloudHSM client as the <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-store-concepts.html#concept-kmsuser"> <code>kmsuser</code> crypto user</a> (CU), and rotates its password. The <code>ConnectCustomKeyStore</code> operation might fail for various reasons. To find the reason, use the <a>DescribeCustomKeyStores</a> operation and see the <code>ConnectionErrorCode</code> in the response. For help interpreting the <code>ConnectionErrorCode</code>, see <a>CustomKeyStoresListEntry</a>. To fix the failure, use the <a>DisconnectCustomKeyStore</a> operation to disconnect the custom key store, correct the error, use the <a>UpdateCustomKeyStore</a> operation if necessary, and then use <code>ConnectCustomKeyStore</code> again. If you are having trouble connecting or disconnecting a custom key store, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/fix-keystore.html">Troubleshooting a Custom Key Store</a> in the <i>AWS Key Management Service Developer Guide</i>.

        :param self:
        :param CustomKeyStoreId:  Enter the key store ID of the custom key store that you want to connect. To find the ID of a custom key store, use the <a>DescribeCustomKeyStores</a> operation.<br/>
        :return:         """
        pass

    def create_alias(self, TargetKeyId: str, AliasName: str):
        """Creates a display name for a customer managed customer master key (CMK). You can use an alias to identify a CMK in selected operations, such as <a>Encrypt</a> and <a>GenerateDataKey</a>.  Each CMK can have multiple aliases, but each alias points to only one CMK. The alias name must be unique in the AWS account and region. To simplify code that runs in multiple regions, use the same alias name, but point it to a different CMK in each region.  Because an alias is not a property of a CMK, you can delete and change the aliases of a CMK without affecting the CMK. Also, aliases do not appear in the response from the <a>DescribeKey</a> operation. To get the aliases of all CMKs, use the <a>ListAliases</a> operation. The alias name must begin with <code>alias/</code> followed by a name, such as <code>alias/ExampleAlias</code>. It can contain only alphanumeric characters, forward slashes (/), underscores (_), and dashes (-). The alias name cannot begin with <code>alias/aws/</code>. The <code>alias/aws/</code> prefix is reserved for <a href="https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#aws-managed-cmk">AWS managed CMKs</a>.  The alias and the CMK it is mapped to must be in the same AWS account and the same region. You cannot perform this operation on an alias in a different AWS account. To map an existing alias to a different CMK, call <a>UpdateAlias</a>. The result of this operation varies with the key state of the CMK. For details, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-state.html">How Key State Affects Use of a Customer Master Key</a> in the <i>AWS Key Management Service Developer Guide</i>.

        :param self:
        :param TargetKeyId:  Identifies the CMK to which the alias refers. Specify the key ID or the Amazon Resource Name (ARN) of the CMK. You cannot specify another alias. For help finding the key ID and ARN, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/viewing-keys.html#find-cmk-id-arn">Finding the Key ID and ARN</a> in the <i>AWS Key Management Service Developer Guide</i>.<br/>AliasName:  Specifies the alias name. This value must begin with <code>alias/</code> followed by a name, such as <code>alias/ExampleAlias</code>. The alias name cannot begin with <code>alias/aws/</code>. The <code>alias/aws/</code> prefix is reserved for AWS managed CMKs.<br/>
        :return:         """
        pass

    def create_custom_key_store(self, KeyStorePassword: str, TrustAnchorCertificate: str, CloudHsmClusterId: str, CustomKeyStoreName: str) -> CreateCustomKeyStoreResponse:
        """Creates a <a href="https://docs.aws.amazon.com/kms/latest/developerguide/custom-key-store-overview.html">custom key store</a> that is associated with an <a href="https://docs.aws.amazon.com/cloudhsm/latest/userguide/clusters.html">AWS CloudHSM cluster</a> that you own and manage. This operation is part of the <a href="https://docs.aws.amazon.com/kms/latest/developerguide/custom-key-store-overview.html">Custom Key Store feature</a> feature in AWS KMS, which combines the convenience and extensive integration of AWS KMS with the isolation and control of a single-tenant key store. Before you create the custom key store, you must assemble the required elements, including an AWS CloudHSM cluster that fulfills the requirements for a custom key store. For details about the required elements, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/create-keystore.html#before-keystore">Assemble the Prerequisites</a> in the <i>AWS Key Management Service Developer Guide</i>. When the operation completes successfully, it returns the ID of the new custom key store. Before you can use your new custom key store, you need to use the <a>ConnectCustomKeyStore</a> operation to connect the new key store to its AWS CloudHSM cluster. Even if you are not going to use your custom key store immediately, you might want to connect it to verify that all settings are correct and then disconnect it until you are ready to use it. For help with failures, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/fix-keystore.html">Troubleshooting a Custom Key Store</a> in the <i>AWS Key Management Service Developer Guide</i>.

        :param self:
        :param KeyStorePassword:  Enter the password of the <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-store-concepts.html#concept-kmsuser"> <code>kmsuser</code> crypto user (CU) account</a> in the specified AWS CloudHSM cluster. AWS KMS logs into the cluster as this user to manage key material on your behalf. This parameter tells AWS KMS the <code>kmsuser</code> account password; it does not change the password in the AWS CloudHSM cluster.<br/>TrustAnchorCertificate:  Enter the content of the trust anchor certificate for the cluster. This is the content of the <code>customerCA.crt</code> file that you created when you <a href="https://docs.aws.amazon.com/cloudhsm/latest/userguide/initialize-cluster.html">initialized the cluster</a>.<br/>CloudHsmClusterId:  Identifies the AWS CloudHSM cluster for the custom key store. Enter the cluster ID of any active AWS CloudHSM cluster that is not already associated with a custom key store. To find the cluster ID, use the <a href="https://docs.aws.amazon.com/cloudhsm/latest/APIReference/API_DescribeClusters.html">DescribeClusters</a> operation.<br/>CustomKeyStoreName:  Specifies a friendly name for the custom key store. The name must be unique in your AWS account.<br/>
        :return: CustomKeyStoreId:  A unique identifier for the new custom key store.<br/>        """
        pass

    def create_grant(self, Operations: List[GrantOperation], GranteePrincipal: str, KeyId: str, RetiringPrincipal: str=None, Constraints: GrantConstraints=None, GrantTokens: List[GrantTokenType]=None, Name: str=None) -> CreateGrantResponse:
        """Adds a grant to a customer master key (CMK). The grant allows the grantee principal to use the CMK when the conditions specified in the grant are met. When setting permissions, grants are an alternative to key policies.  To create a grant that allows a cryptographic operation only when the encryption context in the operation request matches or includes a specified encryption context, use the <code>Constraints</code> parameter. For details, see <a>GrantConstraints</a>. To perform this operation on a CMK in a different AWS account, specify the key ARN in the value of the <code>KeyId</code> parameter. For more information about grants, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/grants.html">Grants</a> in the <i> <i>AWS Key Management Service Developer Guide</i> </i>. The result of this operation varies with the key state of the CMK. For details, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-state.html">How Key State Affects Use of a Customer Master Key</a> in the <i>AWS Key Management Service Developer Guide</i>.

        :param self:
        :param Name:  A friendly name for identifying the grant. Use this value to prevent the unintended creation of duplicate grants when retrying this request. When this value is absent, all <code>CreateGrant</code> requests result in a new grant with a unique <code>GrantId</code> even if all the supplied parameters are identical. This can result in unintended duplicates when you retry the <code>CreateGrant</code> request. When this value is present, you can retry a <code>CreateGrant</code> request with identical parameters; if the grant already exists, the original <code>GrantId</code> is returned without creating a new grant. Note that the returned grant token is unique with every <code>CreateGrant</code> request, even when a duplicate <code>GrantId</code> is returned. All grant tokens obtained in this way can be used interchangeably.<br/>GrantTokens:  A list of grant tokens. For more information, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#grant_token">Grant Tokens</a> in the <i>AWS Key Management Service Developer Guide</i>.<br/>Constraints:  Allows a cryptographic operation only when the encryption context matches or includes the encryption context specified in this structure. For more information about encryption context, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#encrypt_context">Encryption Context</a> in the <i> <i>AWS Key Management Service Developer Guide</i> </i>.:<br/>Operations:  A list of operations that the grant permits.<br/>RetiringPrincipal:  The principal that is given permission to retire the grant by using <a>RetireGrant</a> operation. To specify the principal, use the <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Name (ARN)</a> of an AWS principal. Valid AWS principals include AWS accounts (root), IAM users, federated users, and assumed role users. For examples of the ARN syntax to use for specifying a principal, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#arn-syntax-iam">AWS Identity and Access Management (IAM)</a> in the Example ARNs section of the <i>AWS General Reference</i>.<br/>GranteePrincipal:  The principal that is given permission to perform the operations that the grant permits. To specify the principal, use the <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Name (ARN)</a> of an AWS principal. Valid AWS principals include AWS accounts (root), IAM users, IAM roles, federated users, and assumed role users. For examples of the ARN syntax to use for specifying a principal, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#arn-syntax-iam">AWS Identity and Access Management (IAM)</a> in the Example ARNs section of the <i>AWS General Reference</i>.<br/>KeyId:  The unique identifier for the customer master key (CMK) that the grant applies to. Specify the key ID or the Amazon Resource Name (ARN) of the CMK. To specify a CMK in a different AWS account, you must use the key ARN. For example: <ul> <li> Key ID: <code>1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Key ARN: <code>arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> </ul> To get the key ID and key ARN for a CMK, use <a>ListKeys</a> or <a>DescribeKey</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionContextEquals</i> A list of key-value pairs that must match the encryption context in the cryptographic operation request. The grant allows the operation only when the encryption context in the request is the same as the encryption context specified in this constraint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionContextSubset</i> A list of key-value pairs that must be included in the encryption context of the cryptographic operation request. The grant allows the cryptographic operation only when the encryption context in the request includes the key-value pairs specified in this constraint, although it can include additional key-value pairs.<br/>
        :return: GrantId:  The unique identifier for the grant. You can use the <code>GrantId</code> in a subsequent <a>RetireGrant</a> or <a>RevokeGrant</a> operation.<br/>GrantToken:  The grant token. For more information, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#grant_token">Grant Tokens</a> in the <i>AWS Key Management Service Developer Guide</i>.<br/>        """
        pass

    def create_key(self, Policy: str=None, Description: str=None, KeyUsage: str=None, Origin: str=None, CustomKeyStoreId: str=None, BypassPolicyLockoutSafetyCheck: BooleanType=None, Tags: List[Tag]=None) -> CreateKeyResponse:
        """Creates a customer managed <a href="https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#master_keys">customer master key</a> (CMK) in your AWS account. You can use a CMK to encrypt small amounts of data (up to 4096 bytes) directly. But CMKs are more commonly used to encrypt the <a href="https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#data-keys">data keys</a> that are used to encrypt data. To create a CMK for imported key material, use the <code>Origin</code> parameter with a value of <code>EXTERNAL</code>. To create a CMK in a <a href="https://docs.aws.amazon.com/kms/latest/developerguide/custom-key-store-overview.html">custom key store</a>, use the <code>CustomKeyStoreId</code> parameter to specify the custom key store. You must also use the <code>Origin</code> parameter with a value of <code>AWS_CLOUDHSM</code>. The AWS CloudHSM cluster that is associated with the custom key store must have at least two active HSMs in different Availability Zones in the AWS Region. You cannot use this operation to create a CMK in a different AWS account.

        :param self:
        :param Tags:  One or more tags. Each tag consists of a tag key and a tag value. Tag keys and tag values are both required, but tag values can be empty (null) strings. Use this parameter to tag the CMK when it is created. Alternately, you can omit this parameter and instead tag the CMK after it is created using <a>TagResource</a>.<br/>BypassPolicyLockoutSafetyCheck:  A flag to indicate whether to bypass the key policy lockout safety check. <important> Setting this value to true increases the risk that the CMK becomes unmanageable. Do not set this value to true indiscriminately. For more information, refer to the scenario in the <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-policies.html#key-policy-default-allow-root-enable-iam">Default Key Policy</a> section in the <i> <i>AWS Key Management Service Developer Guide</i> </i>. </important> Use this parameter only when you include a policy in the request and you intend to prevent the principal that is making the request from making a subsequent <a>PutKeyPolicy</a> request on the CMK. The default value is false.<br/>CustomKeyStoreId:  Creates the CMK in the specified <a href="https://docs.aws.amazon.com/kms/latest/developerguide/custom-key-store-overview.html">custom key store</a> and the key material in its associated AWS CloudHSM cluster. To create a CMK in a custom key store, you must also specify the <code>Origin</code> parameter with a value of <code>AWS_CLOUDHSM</code>. The AWS CloudHSM cluster that is associated with the custom key store must have at least two active HSMs, each in a different Availability Zone in the Region. To find the ID of a custom key store, use the <a>DescribeCustomKeyStores</a> operation. The response includes the custom key store ID and the ID of the AWS CloudHSM cluster. This operation is part of the <a href="https://docs.aws.amazon.com/kms/latest/developerguide/custom-key-store-overview.html">Custom Key Store feature</a> feature in AWS KMS, which combines the convenience and extensive integration of AWS KMS with the isolation and control of a single-tenant key store.<br/>Origin:  The source of the key material for the CMK. You cannot change the origin after you create the CMK. The default is <code>AWS_KMS</code>, which means AWS KMS creates the key material in its own key store. When the parameter value is <code>EXTERNAL</code>, AWS KMS creates a CMK without key material so that you can import key material from your existing key management infrastructure. For more information about importing key material into AWS KMS, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/importing-keys.html">Importing Key Material</a> in the <i>AWS Key Management Service Developer Guide</i>. When the parameter value is <code>AWS_CLOUDHSM</code>, AWS KMS creates the CMK in an AWS KMS <a href="https://docs.aws.amazon.com/kms/latest/developerguide/custom-key-store-overview.html">custom key store</a> and creates its key material in the associated AWS CloudHSM cluster. You must also use the <code>CustomKeyStoreId</code> parameter to identify the custom key store.<br/>KeyUsage:  The cryptographic operations for which you can use the CMK. The only valid value is <code>ENCRYPT_DECRYPT</code>, which means you can use the CMK to encrypt and decrypt data.<br/>Description:  A description of the CMK. Use a description that helps you decide whether the CMK is appropriate for a task.<br/>Policy:  The key policy to attach to the CMK. If you provide a key policy, it must meet the following criteria: <ul> <li> If you don't set <code>BypassPolicyLockoutSafetyCheck</code> to true, the key policy must allow the principal that is making the <code>CreateKey</code> request to make a subsequent <a>PutKeyPolicy</a> request on the CMK. This reduces the risk that the CMK becomes unmanageable. For more information, refer to the scenario in the <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-policies.html#key-policy-default-allow-root-enable-iam">Default Key Policy</a> section of the <i> <i>AWS Key Management Service Developer Guide</i> </i>. </li> <li> Each statement in the key policy must contain one or more principals. The principals in the key policy must exist and be visible to AWS KMS. When you create a new AWS principal (for example, an IAM user or role), you might need to enforce a delay before including the new principal in a key policy because the new principal might not be immediately visible to AWS KMS. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/troubleshoot_general.html#troubleshoot_general_eventual-consistency">Changes that I make are not always immediately visible</a> in the <i>AWS Identity and Access Management User Guide</i>. </li> </ul> If you do not provide a key policy, AWS KMS attaches a default key policy to the CMK. For more information, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-policies.html#key-policy-default">Default Key Policy</a> in the <i>AWS Key Management Service Developer Guide</i>. The key policy size limit is 32 kilobytes (32768 bytes).<br/>
        :return: KeyMetadata:  Metadata associated with the CMK.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyManager</i> The manager of the CMK. CMKs in your AWS account are either customer managed or AWS managed. For more information about the difference, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#master_keys">Customer Master Keys</a> in the <i>AWS Key Management Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpirationModel</i> Specifies whether the CMK's key material expires. This value is present only when <code>Origin</code> is <code>EXTERNAL</code>, otherwise this value is omitted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudHsmClusterId</i> The cluster ID of the AWS CloudHSM cluster that contains the key material for the CMK. When you create a CMK in a <a href="https://docs.aws.amazon.com/kms/latest/developerguide/custom-key-store-overview.html">custom key store</a>, AWS KMS creates the key material for the CMK in the associated AWS CloudHSM cluster. This value is present only when the CMK is created in a custom key store.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomKeyStoreId</i> A unique identifier for the <a href="https://docs.aws.amazon.com/kms/latest/developerguide/custom-key-store-overview.html">custom key store</a> that contains the CMK. This value is present only when the CMK is created in a custom key store.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Origin</i> The source of the CMK's key material. When this value is <code>AWS_KMS</code>, AWS KMS created the key material. When this value is <code>EXTERNAL</code>, the key material was imported from your existing key management infrastructure or the CMK lacks key material. When this value is <code>AWS_CLOUDHSM</code>, the key material was created in the AWS CloudHSM cluster associated with a custom key store.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ValidTo</i> The time at which the imported key material expires. When the key material expires, AWS KMS deletes the key material and the CMK becomes unusable. This value is present only for CMKs whose <code>Origin</code> is <code>EXTERNAL</code> and whose <code>ExpirationModel</code> is <code>KEY_MATERIAL_EXPIRES</code>, otherwise this value is omitted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionDate</i> The date and time after which AWS KMS deletes the CMK. This value is present only when <code>KeyState</code> is <code>PendingDeletion</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyState</i> The state of the CMK. For more information about how key state affects the use of a CMK, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-state.html">How Key State Affects the Use of a Customer Master Key</a> in the <i>AWS Key Management Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyUsage</i> The cryptographic operations for which you can use the CMK. The only valid value is <code>ENCRYPT_DECRYPT</code>, which means you can use the CMK to encrypt and decrypt data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the CMK.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the CMK is enabled. When <code>KeyState</code> is <code>Enabled</code> this value is true, otherwise it is false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time when the CMK was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the CMK. For examples, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#arn-syntax-kms">AWS Key Management Service (AWS KMS)</a> in the Example ARNs section of the <i>AWS General Reference</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyId</i> The globally unique identifier for the CMK.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AWSAccountId</i> The twelve-digit account ID of the AWS account that owns the CMK.<br/>        """
        pass

    def decrypt(self, CiphertextBlob: CiphertextType, EncryptionContext: EncryptionContextType=None, GrantTokens: List[GrantTokenType]=None) -> DecryptResponse:
        """Decrypts ciphertext. Ciphertext is plaintext that has been previously encrypted by using any of the following operations: <ul> <li>  <a>GenerateDataKey</a>  </li> <li>  <a>GenerateDataKeyWithoutPlaintext</a>  </li> <li>  <a>Encrypt</a>  </li> </ul> Whenever possible, use key policies to give users permission to call the Decrypt operation on the CMK, instead of IAM policies. Otherwise, you might create an IAM user policy that gives the user Decrypt permission on all CMKs. This user could decrypt ciphertext that was encrypted by CMKs in other accounts if the key policy for the cross-account CMK permits it. If you must use an IAM policy for <code>Decrypt</code> permissions, limit the user to particular CMKs or particular trusted accounts. The result of this operation varies with the key state of the CMK. For details, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-state.html">How Key State Affects Use of a Customer Master Key</a> in the <i>AWS Key Management Service Developer Guide</i>.

        :param self:
        :param GrantTokens:  A list of grant tokens. For more information, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#grant_token">Grant Tokens</a> in the <i>AWS Key Management Service Developer Guide</i>.<br/>EncryptionContext:  The encryption context. If this was specified in the <a>Encrypt</a> function, it must be specified here or the decryption operation will fail. For more information, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#encrypt_context">Encryption Context</a>.<br/>CiphertextBlob:  Ciphertext to be decrypted. The blob includes metadata.<br/>
        :return: Plaintext:  Decrypted plaintext data. When you use the HTTP API or the AWS CLI, the value is Base64-encoded. Otherwise, it is not encoded.<br/>KeyId:  ARN of the key used to perform the decryption. This value is returned if no errors are encountered during the operation.<br/>        """
        pass

    def delete_alias(self, AliasName: str):
        """Deletes the specified alias. You cannot perform this operation on an alias in a different AWS account.  Because an alias is not a property of a CMK, you can delete and change the aliases of a CMK without affecting the CMK. Also, aliases do not appear in the response from the <a>DescribeKey</a> operation. To get the aliases of all CMKs, use the <a>ListAliases</a> operation.  Each CMK can have multiple aliases. To change the alias of a CMK, use <a>DeleteAlias</a> to delete the current alias and <a>CreateAlias</a> to create a new alias. To associate an existing alias with a different customer master key (CMK), call <a>UpdateAlias</a>.

        :param self:
        :param AliasName:  The alias to be deleted. The alias name must begin with <code>alias/</code> followed by the alias name, such as <code>alias/ExampleAlias</code>.<br/>
        :return:         """
        pass

    def delete_custom_key_store(self, CustomKeyStoreId: str) -> DeleteCustomKeyStoreResponse:
        """Deletes a <a href="https://docs.aws.amazon.com/kms/latest/developerguide/custom-key-store-overview.html">custom key store</a>. This operation does not delete the AWS CloudHSM cluster that is associated with the custom key store, or affect any users or keys in the cluster. The custom key store that you delete cannot contain any AWS KMS <a href="https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#master_keys">customer master keys (CMKs)</a>. Before deleting the key store, verify that you will never need to use any of the CMKs in the key store for any cryptographic operations. Then, use <a>ScheduleKeyDeletion</a> to delete the AWS KMS customer master keys (CMKs) from the key store. When the scheduled waiting period expires, the <code>ScheduleKeyDeletion</code> operation deletes the CMKs. Then it makes a best effort to delete the key material from the associated cluster. However, you might need to manually <a href="https://docs.aws.amazon.com/kms/latest/developerguide/fix-keystore.html#fix-keystore-orphaned-key">delete the orphaned key material</a> from the cluster and its backups. After all CMKs are deleted from AWS KMS, use <a>DisconnectCustomKeyStore</a> to disconnect the key store from AWS KMS. Then, you can delete the custom key store. Instead of deleting the custom key store, consider using <a>DisconnectCustomKeyStore</a> to disconnect it from AWS KMS. While the key store is disconnected, you cannot create or use the CMKs in the key store. But, you do not need to delete CMKs and you can reconnect a disconnected custom key store at any time. If the operation succeeds, it returns a JSON object with no properties. This operation is part of the <a href="https://docs.aws.amazon.com/kms/latest/developerguide/custom-key-store-overview.html">Custom Key Store feature</a> feature in AWS KMS, which combines the convenience and extensive integration of AWS KMS with the isolation and control of a single-tenant key store.

        :param self:
        :param CustomKeyStoreId:  Enter the ID of the custom key store you want to delete. To find the ID of a custom key store, use the <a>DescribeCustomKeyStores</a> operation.<br/>
        :return:         """
        pass

    def delete_imported_key_material(self, KeyId: str):
        """Deletes key material that you previously imported. This operation makes the specified customer master key (CMK) unusable. For more information about importing key material into AWS KMS, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/importing-keys.html">Importing Key Material</a> in the <i>AWS Key Management Service Developer Guide</i>. You cannot perform this operation on a CMK in a different AWS account. When the specified CMK is in the <code>PendingDeletion</code> state, this operation does not change the CMK's state. Otherwise, it changes the CMK's state to <code>PendingImport</code>. After you delete key material, you can use <a>ImportKeyMaterial</a> to reimport the same key material into the CMK. The result of this operation varies with the key state of the CMK. For details, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-state.html">How Key State Affects Use of a Customer Master Key</a> in the <i>AWS Key Management Service Developer Guide</i>.

        :param self:
        :param KeyId:  Identifies the CMK from which you are deleting imported key material. The <code>Origin</code> of the CMK must be <code>EXTERNAL</code>. Specify the key ID or the Amazon Resource Name (ARN) of the CMK. For example: <ul> <li> Key ID: <code>1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Key ARN: <code>arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> </ul> To get the key ID and key ARN for a CMK, use <a>ListKeys</a> or <a>DescribeKey</a>.<br/>
        :return:         """
        pass

    def describe_custom_key_stores(self, CustomKeyStoreId: str=None, CustomKeyStoreName: str=None, Limit: int=None, Marker: str=None) -> DescribeCustomKeyStoresResponse:
        """Gets information about <a href="https://docs.aws.amazon.com/kms/latest/developerguide/custom-key-store-overview.html">custom key stores</a> in the account and region. This operation is part of the <a href="https://docs.aws.amazon.com/kms/latest/developerguide/custom-key-store-overview.html">Custom Key Store feature</a> feature in AWS KMS, which combines the convenience and extensive integration of AWS KMS with the isolation and control of a single-tenant key store. By default, this operation returns information about all custom key stores in the account and region. To get only information about a particular custom key store, use either the <code>CustomKeyStoreName</code> or <code>CustomKeyStoreId</code> parameter (but not both). To determine whether the custom key store is connected to its AWS CloudHSM cluster, use the <code>ConnectionState</code> element in the response. If an attempt to connect the custom key store failed, the <code>ConnectionState</code> value is <code>FAILED</code> and the <code>ConnectionErrorCode</code> element in the response indicates the cause of the failure. For help interpreting the <code>ConnectionErrorCode</code>, see <a>CustomKeyStoresListEntry</a>. Custom key stores have a <code>DISCONNECTED</code> connection state if the key store has never been connected or you use the <a>DisconnectCustomKeyStore</a> operation to disconnect it. If your custom key store state is <code>CONNECTED</code> but you are having trouble using it, make sure that its associated AWS CloudHSM cluster is active and contains the minimum number of HSMs required for the operation, if any.  For help repairing your custom key store, see the <a href="https://docs.aws.amazon.com/kms/latest/developerguide/fix-keystore.html">Troubleshooting Custom Key Stores</a> topic in the <i>AWS Key Management Service Developer Guide</i>.

        :param self:
        :param Marker:  Use this parameter in a subsequent request after you receive a response with truncated results. Set it to the value of <code>NextMarker</code> from the truncated response you just received.<br/>Limit:  Use this parameter to specify the maximum number of items to return. When this value is present, AWS KMS does not return more than the specified number of items, but it might return fewer.<br/>CustomKeyStoreName:  Gets only information about the specified custom key store. Enter the friendly name of the custom key store. By default, this operation gets information about all custom key stores in the account and region. To limit the output to a particular custom key store, you can use either the <code>CustomKeyStoreId</code> or <code>CustomKeyStoreName</code> parameter, but not both.<br/>CustomKeyStoreId:  Gets only information about the specified custom key store. Enter the key store ID. By default, this operation gets information about all custom key stores in the account and region. To limit the output to a particular custom key store, you can use either the <code>CustomKeyStoreId</code> or <code>CustomKeyStoreName</code> parameter, but not both.<br/>
        :return: Truncated:  A flag that indicates whether there are more items in the list. When this value is true, the list in this response is truncated. To get more items, pass the value of the <code>NextMarker</code> element in thisresponse to the <code>Marker</code> parameter in a subsequent request.<br/>NextMarker:  When <code>Truncated</code> is true, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent request.<br/>CustomKeyStores:  Contains metadata about each custom key store.<br/>        """
        pass

    def describe_key(self, KeyId: str, GrantTokens: List[GrantTokenType]=None) -> DescribeKeyResponse:
        """Provides detailed information about the specified customer master key (CMK). You can use <code>DescribeKey</code> on a predefined AWS alias, that is, an AWS alias with no key ID. When you do, AWS KMS associates the alias with an <a href="https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#master_keys">AWS managed CMK</a> and returns its <code>KeyId</code> and <code>Arn</code> in the response. To perform this operation on a CMK in a different AWS account, specify the key ARN or alias ARN in the value of the KeyId parameter.

        :param self:
        :param GrantTokens:  A list of grant tokens. For more information, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#grant_token">Grant Tokens</a> in the <i>AWS Key Management Service Developer Guide</i>.<br/>KeyId:  Describes the specified customer master key (CMK).  If you specify a predefined AWS alias (an AWS alias with no key ID), KMS associates the alias with an <a href="https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#master_keys">AWS managed CMK</a> and returns its <code>KeyId</code> and <code>Arn</code> in the response. To specify a CMK, use its key ID, Amazon Resource Name (ARN), alias name, or alias ARN. When using an alias name, prefix it with <code>"alias/"</code>. To specify a CMK in a different AWS account, you must use the key ARN or alias ARN. For example: <ul> <li> Key ID: <code>1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Key ARN: <code>arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Alias name: <code>alias/ExampleAlias</code>  </li> <li> Alias ARN: <code>arn:aws:kms:us-east-2:111122223333:alias/ExampleAlias</code>  </li> </ul> To get the key ID and key ARN for a CMK, use <a>ListKeys</a> or <a>DescribeKey</a>. To get the alias name and alias ARN, use <a>ListAliases</a>.<br/>
        :return: KeyMetadata:  Metadata associated with the key.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyManager</i> The manager of the CMK. CMKs in your AWS account are either customer managed or AWS managed. For more information about the difference, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#master_keys">Customer Master Keys</a> in the <i>AWS Key Management Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpirationModel</i> Specifies whether the CMK's key material expires. This value is present only when <code>Origin</code> is <code>EXTERNAL</code>, otherwise this value is omitted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudHsmClusterId</i> The cluster ID of the AWS CloudHSM cluster that contains the key material for the CMK. When you create a CMK in a <a href="https://docs.aws.amazon.com/kms/latest/developerguide/custom-key-store-overview.html">custom key store</a>, AWS KMS creates the key material for the CMK in the associated AWS CloudHSM cluster. This value is present only when the CMK is created in a custom key store.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomKeyStoreId</i> A unique identifier for the <a href="https://docs.aws.amazon.com/kms/latest/developerguide/custom-key-store-overview.html">custom key store</a> that contains the CMK. This value is present only when the CMK is created in a custom key store.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Origin</i> The source of the CMK's key material. When this value is <code>AWS_KMS</code>, AWS KMS created the key material. When this value is <code>EXTERNAL</code>, the key material was imported from your existing key management infrastructure or the CMK lacks key material. When this value is <code>AWS_CLOUDHSM</code>, the key material was created in the AWS CloudHSM cluster associated with a custom key store.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ValidTo</i> The time at which the imported key material expires. When the key material expires, AWS KMS deletes the key material and the CMK becomes unusable. This value is present only for CMKs whose <code>Origin</code> is <code>EXTERNAL</code> and whose <code>ExpirationModel</code> is <code>KEY_MATERIAL_EXPIRES</code>, otherwise this value is omitted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionDate</i> The date and time after which AWS KMS deletes the CMK. This value is present only when <code>KeyState</code> is <code>PendingDeletion</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyState</i> The state of the CMK. For more information about how key state affects the use of a CMK, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-state.html">How Key State Affects the Use of a Customer Master Key</a> in the <i>AWS Key Management Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyUsage</i> The cryptographic operations for which you can use the CMK. The only valid value is <code>ENCRYPT_DECRYPT</code>, which means you can use the CMK to encrypt and decrypt data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the CMK.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the CMK is enabled. When <code>KeyState</code> is <code>Enabled</code> this value is true, otherwise it is false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time when the CMK was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the CMK. For examples, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#arn-syntax-kms">AWS Key Management Service (AWS KMS)</a> in the Example ARNs section of the <i>AWS General Reference</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyId</i> The globally unique identifier for the CMK.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AWSAccountId</i> The twelve-digit account ID of the AWS account that owns the CMK.<br/>        """
        pass

    def disable_key(self, KeyId: str):
        """Sets the state of a customer master key (CMK) to disabled, thereby preventing its use for cryptographic operations. You cannot perform this operation on a CMK in a different AWS account. For more information about how key state affects the use of a CMK, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-state.html">How Key State Affects the Use of a Customer Master Key</a> in the <i> <i>AWS Key Management Service Developer Guide</i> </i>. The result of this operation varies with the key state of the CMK. For details, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-state.html">How Key State Affects Use of a Customer Master Key</a> in the <i>AWS Key Management Service Developer Guide</i>.

        :param self:
        :param KeyId:  A unique identifier for the customer master key (CMK). Specify the key ID or the Amazon Resource Name (ARN) of the CMK. For example: <ul> <li> Key ID: <code>1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Key ARN: <code>arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> </ul> To get the key ID and key ARN for a CMK, use <a>ListKeys</a> or <a>DescribeKey</a>.<br/>
        :return:         """
        pass

    def disable_key_rotation(self, KeyId: str):
        """Disables <a href="https://docs.aws.amazon.com/kms/latest/developerguide/rotate-keys.html">automatic rotation of the key material</a> for the specified customer master key (CMK). You cannot perform this operation on a CMK in a different AWS account. The result of this operation varies with the key state of the CMK. For details, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-state.html">How Key State Affects Use of a Customer Master Key</a> in the <i>AWS Key Management Service Developer Guide</i>.

        :param self:
        :param KeyId:  A unique identifier for the customer master key (CMK). Specify the key ID or the Amazon Resource Name (ARN) of the CMK. For example: <ul> <li> Key ID: <code>1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Key ARN: <code>arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> </ul> To get the key ID and key ARN for a CMK, use <a>ListKeys</a> or <a>DescribeKey</a>.<br/>
        :return:         """
        pass

    def disconnect_custom_key_store(self, CustomKeyStoreId: str) -> DisconnectCustomKeyStoreResponse:
        """Disconnects the <a href="https://docs.aws.amazon.com/kms/latest/developerguide/custom-key-store-overview.html">custom key store</a> from its associated AWS CloudHSM cluster. While a custom key store is disconnected, you can manage the custom key store and its customer master keys (CMKs), but you cannot create or use CMKs in the custom key store. You can reconnect the custom key store at any time. <note> While a custom key store is disconnected, all attempts to create customer master keys (CMKs) in the custom key store or to use existing CMKs in cryptographic operations will fail. This action can prevent users from storing and accessing sensitive data. </note> <p/> To find the connection state of a custom key store, use the <a>DescribeCustomKeyStores</a> operation. To reconnect a custom key store, use the <a>ConnectCustomKeyStore</a> operation. If the operation succeeds, it returns a JSON object with no properties. This operation is part of the <a href="https://docs.aws.amazon.com/kms/latest/developerguide/custom-key-store-overview.html">Custom Key Store feature</a> feature in AWS KMS, which combines the convenience and extensive integration of AWS KMS with the isolation and control of a single-tenant key store.

        :param self:
        :param CustomKeyStoreId:  Enter the ID of the custom key store you want to disconnect. To find the ID of a custom key store, use the <a>DescribeCustomKeyStores</a> operation.<br/>
        :return:         """
        pass

    def enable_key(self, KeyId: str):
        """Sets the key state of a customer master key (CMK) to enabled. This allows you to use the CMK for cryptographic operations. You cannot perform this operation on a CMK in a different AWS account. The result of this operation varies with the key state of the CMK. For details, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-state.html">How Key State Affects Use of a Customer Master Key</a> in the <i>AWS Key Management Service Developer Guide</i>.

        :param self:
        :param KeyId:  A unique identifier for the customer master key (CMK). Specify the key ID or the Amazon Resource Name (ARN) of the CMK. For example: <ul> <li> Key ID: <code>1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Key ARN: <code>arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> </ul> To get the key ID and key ARN for a CMK, use <a>ListKeys</a> or <a>DescribeKey</a>.<br/>
        :return:         """
        pass

    def enable_key_rotation(self, KeyId: str):
        """Enables <a href="https://docs.aws.amazon.com/kms/latest/developerguide/rotate-keys.html">automatic rotation of the key material</a> for the specified customer master key (CMK). You cannot perform this operation on a CMK in a different AWS account. You cannot enable automatic rotation of CMKs with imported key material or CMKs in a <a href="https://docs.aws.amazon.com/kms/latest/developerguide/custom-key-store-overview.html">custom key store</a>. The result of this operation varies with the key state of the CMK. For details, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-state.html">How Key State Affects Use of a Customer Master Key</a> in the <i>AWS Key Management Service Developer Guide</i>.

        :param self:
        :param KeyId:  A unique identifier for the customer master key (CMK). Specify the key ID or the Amazon Resource Name (ARN) of the CMK. For example: <ul> <li> Key ID: <code>1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Key ARN: <code>arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> </ul> To get the key ID and key ARN for a CMK, use <a>ListKeys</a> or <a>DescribeKey</a>.<br/>
        :return:         """
        pass

    def encrypt(self, Plaintext: PlaintextType, KeyId: str, EncryptionContext: EncryptionContextType=None, GrantTokens: List[GrantTokenType]=None) -> EncryptResponse:
        """Encrypts plaintext into ciphertext by using a customer master key (CMK). The <code>Encrypt</code> operation has two primary use cases: <ul> <li> You can encrypt up to 4 kilobytes (4096 bytes) of arbitrary data such as an RSA key, a database password, or other sensitive information. </li> <li> You can use the <code>Encrypt</code> operation to move encrypted data from one AWS region to another. In the first region, generate a data key and use the plaintext key to encrypt the data. Then, in the new region, call the <code>Encrypt</code> method on same plaintext data key. Now, you can safely move the encrypted data and encrypted data key to the new region, and decrypt in the new region when necessary. </li> </ul> You don't need use this operation to encrypt a data key within a region. The <a>GenerateDataKey</a> and <a>GenerateDataKeyWithoutPlaintext</a> operations return an encrypted data key. Also, you don't need to use this operation to encrypt data in your application. You can use the plaintext and encrypted data keys that the <code>GenerateDataKey</code> operation returns. The result of this operation varies with the key state of the CMK. For details, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-state.html">How Key State Affects Use of a Customer Master Key</a> in the <i>AWS Key Management Service Developer Guide</i>. To perform this operation on a CMK in a different AWS account, specify the key ARN or alias ARN in the value of the KeyId parameter.

        :param self:
        :param GrantTokens:  A list of grant tokens. For more information, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#grant_token">Grant Tokens</a> in the <i>AWS Key Management Service Developer Guide</i>.<br/>EncryptionContext:  Name-value pair that specifies the encryption context to be used for authenticated encryption. If used here, the same value must be supplied to the <code>Decrypt</code> API or decryption will fail. For more information, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#encrypt_context">Encryption Context</a>.<br/>Plaintext:  Data to be encrypted.<br/>KeyId:  A unique identifier for the customer master key (CMK). To specify a CMK, use its key ID, Amazon Resource Name (ARN), alias name, or alias ARN. When using an alias name, prefix it with <code>"alias/"</code>. To specify a CMK in a different AWS account, you must use the key ARN or alias ARN. For example: <ul> <li> Key ID: <code>1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Key ARN: <code>arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Alias name: <code>alias/ExampleAlias</code>  </li> <li> Alias ARN: <code>arn:aws:kms:us-east-2:111122223333:alias/ExampleAlias</code>  </li> </ul> To get the key ID and key ARN for a CMK, use <a>ListKeys</a> or <a>DescribeKey</a>. To get the alias name and alias ARN, use <a>ListAliases</a>.<br/>
        :return: KeyId:  The ID of the key used during encryption.<br/>CiphertextBlob:  The encrypted plaintext. When you use the HTTP API or the AWS CLI, the value is Base64-encoded. Otherwise, it is not encoded.<br/>        """
        pass

    def generate_data_key(self, KeyId: str, EncryptionContext: EncryptionContextType=None, NumberOfBytes: int=None, KeySpec: str=None, GrantTokens: List[GrantTokenType]=None) -> GenerateDataKeyResponse:
        """Generates a unique data key. This operation returns a plaintext copy of the data key and a copy that is encrypted under a customer master key (CMK) that you specify. You can use the plaintext key to encrypt your data outside of KMS and store the encrypted data key with the encrypted data.  <code>GenerateDataKey</code> returns a unique data key for each request. The bytes in the key are not related to the caller or CMK that is used to encrypt the data key. To generate a data key, you need to specify the customer master key (CMK) that will be used to encrypt the data key. You must also specify the length of the data key using either the <code>KeySpec</code> or <code>NumberOfBytes</code> field (but not both). For common key lengths (128-bit and 256-bit symmetric keys), we recommend that you use <code>KeySpec</code>. To perform this operation on a CMK in a different AWS account, specify the key ARN or alias ARN in the value of the KeyId parameter. You will find the plaintext copy of the data key in the <code>Plaintext</code> field of the response, and the encrypted copy of the data key in the <code>CiphertextBlob</code> field. We recommend that you use the following pattern to encrypt data locally in your application: <ol> <li> Use the <code>GenerateDataKey</code> operation to get a data encryption key. </li> <li> Use the plaintext data key (returned in the <code>Plaintext</code> field of the response) to encrypt data locally, then erase the plaintext data key from memory. </li> <li> Store the encrypted data key (returned in the <code>CiphertextBlob</code> field of the response) alongside the locally encrypted data. </li> </ol> To decrypt data locally: <ol> <li> Use the <a>Decrypt</a> operation to decrypt the encrypted data key. The operation returns a plaintext copy of the data key. </li> <li> Use the plaintext data key to decrypt data locally, then erase the plaintext data key from memory. </li> </ol> To get only an encrypted copy of the data key, use <a>GenerateDataKeyWithoutPlaintext</a>. To get a cryptographically secure random byte string, use <a>GenerateRandom</a>. You can use the optional encryption context to add additional security to your encryption operation. When you specify an <code>EncryptionContext</code> in the <code>GenerateDataKey</code> operation, you must specify the same encryption context (a case-sensitive exact match) in your request to <a>Decrypt</a> the data key. Otherwise, the request to decrypt fails with an <code>InvalidCiphertextException</code>. For more information, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#encrypt_context">Encryption Context</a> in the <i> <i>AWS Key Management Service Developer Guide</i> </i>. The result of this operation varies with the key state of the CMK. For details, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-state.html">How Key State Affects Use of a Customer Master Key</a> in the <i>AWS Key Management Service Developer Guide</i>.

        :param self:
        :param GrantTokens:  A list of grant tokens. For more information, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#grant_token">Grant Tokens</a> in the <i>AWS Key Management Service Developer Guide</i>.<br/>KeySpec:  The length of the data key. Use <code>AES_128</code> to generate a 128-bit symmetric key, or <code>AES_256</code> to generate a 256-bit symmetric key.<br/>NumberOfBytes:  The length of the data key in bytes. For example, use the value 64 to generate a 512-bit data key (64 bytes is 512 bits). For common key lengths (128-bit and 256-bit symmetric keys), we recommend that you use the <code>KeySpec</code> field instead of this one.<br/>EncryptionContext:  A set of key-value pairs that represents additional authenticated data. For more information, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#encrypt_context">Encryption Context</a> in the <i>AWS Key Management Service Developer Guide</i>.<br/>KeyId:  An identifier for the CMK that encrypts the data key. To specify a CMK, use its key ID, Amazon Resource Name (ARN), alias name, or alias ARN. When using an alias name, prefix it with <code>"alias/"</code>. To specify a CMK in a different AWS account, you must use the key ARN or alias ARN. For example: <ul> <li> Key ID: <code>1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Key ARN: <code>arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Alias name: <code>alias/ExampleAlias</code>  </li> <li> Alias ARN: <code>arn:aws:kms:us-east-2:111122223333:alias/ExampleAlias</code>  </li> </ul> To get the key ID and key ARN for a CMK, use <a>ListKeys</a> or <a>DescribeKey</a>. To get the alias name and alias ARN, use <a>ListAliases</a>.<br/>
        :return: KeyId:  The identifier of the CMK that encrypted the data key.<br/>Plaintext:  The plaintext data key. When you use the HTTP API or the AWS CLI, the value is Base64-encoded. Otherwise, it is not encoded. Use this data key to encrypt your data outside of KMS. Then, remove it from memory as soon as possible.<br/>CiphertextBlob:  The encrypted copy of the data key. When you use the HTTP API or the AWS CLI, the value is Base64-encoded. Otherwise, it is not encoded.<br/>        """
        pass

    def generate_data_key_without_plaintext(self, KeyId: str, EncryptionContext: EncryptionContextType=None, KeySpec: str=None, NumberOfBytes: int=None, GrantTokens: List[GrantTokenType]=None) -> GenerateDataKeyWithoutPlaintextResponse:
        """Generates a unique data key. This operation returns a data key that is encrypted under a customer master key (CMK) that you specify. <code>GenerateDataKeyWithoutPlaintext</code> is identical to <a>GenerateDataKey</a> except that returns only the encrypted copy of the data key. Like <code>GenerateDataKey</code>, <code>GenerateDataKeyWithoutPlaintext</code> returns a unique data key for each request. The bytes in the key are not related to the caller or CMK that is used to encrypt the data key. This operation is useful for systems that need to encrypt data at some point, but not immediately. When you need to encrypt the data, you call the <a>Decrypt</a> operation on the encrypted copy of the key. It's also useful in distributed systems with different levels of trust. For example, you might store encrypted data in containers. One component of your system creates new containers and stores an encrypted data key with each container. Then, a different component puts the data into the containers. That component first decrypts the data key, uses the plaintext data key to encrypt data, puts the encrypted data into the container, and then destroys the plaintext data key. In this system, the component that creates the containers never sees the plaintext data key. The result of this operation varies with the key state of the CMK. For details, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-state.html">How Key State Affects Use of a Customer Master Key</a> in the <i>AWS Key Management Service Developer Guide</i>.

        :param self:
        :param GrantTokens:  A list of grant tokens. For more information, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#grant_token">Grant Tokens</a> in the <i>AWS Key Management Service Developer Guide</i>.<br/>NumberOfBytes:  The length of the data key in bytes. For example, use the value 64 to generate a 512-bit data key (64 bytes is 512 bits). For common key lengths (128-bit and 256-bit symmetric keys), we recommend that you use the <code>KeySpec</code> field instead of this one.<br/>KeySpec:  The length of the data key. Use <code>AES_128</code> to generate a 128-bit symmetric key, or <code>AES_256</code> to generate a 256-bit symmetric key.<br/>EncryptionContext:  A set of key-value pairs that represents additional authenticated data. For more information, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#encrypt_context">Encryption Context</a> in the <i>AWS Key Management Service Developer Guide</i>.<br/>KeyId:  The identifier of the customer master key (CMK) that encrypts the data key. To specify a CMK, use its key ID, Amazon Resource Name (ARN), alias name, or alias ARN. When using an alias name, prefix it with <code>"alias/"</code>. To specify a CMK in a different AWS account, you must use the key ARN or alias ARN. For example: <ul> <li> Key ID: <code>1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Key ARN: <code>arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Alias name: <code>alias/ExampleAlias</code>  </li> <li> Alias ARN: <code>arn:aws:kms:us-east-2:111122223333:alias/ExampleAlias</code>  </li> </ul> To get the key ID and key ARN for a CMK, use <a>ListKeys</a> or <a>DescribeKey</a>. To get the alias name and alias ARN, use <a>ListAliases</a>.<br/>
        :return: KeyId:  The identifier of the CMK that encrypted the data key.<br/>CiphertextBlob:  The encrypted data key. When you use the HTTP API or the AWS CLI, the value is Base64-encoded. Otherwise, it is not encoded.<br/>        """
        pass

    def generate_random(self, NumberOfBytes: int=None, CustomKeyStoreId: str=None) -> GenerateRandomResponse:
        """Returns a random byte string that is cryptographically secure. By default, the random byte string is generated in AWS KMS. To generate the byte string in the AWS CloudHSM cluster that is associated with a <a href="https://docs.aws.amazon.com/kms/latest/developerguide/custom-key-store-overview.html">custom key store</a>, specify the custom key store ID. For more information about entropy and random number generation, see the <a href="https://d0.awsstatic.com/whitepapers/KMS-Cryptographic-Details.pdf">AWS Key Management Service Cryptographic Details</a> whitepaper.

        :param self:
        :param CustomKeyStoreId:  Generates the random byte string in the AWS CloudHSM cluster that is associated with the specified <a href="https://docs.aws.amazon.com/kms/latest/developerguide/custom-key-store-overview.html">custom key store</a>. To find the ID of a custom key store, use the <a>DescribeCustomKeyStores</a> operation.<br/>NumberOfBytes:  The length of the byte string.<br/>
        :return: Plaintext:  The random byte string. When you use the HTTP API or the AWS CLI, the value is Base64-encoded. Otherwise, it is not encoded.<br/>        """
        pass

    def get_key_policy(self, PolicyName: str, KeyId: str) -> GetKeyPolicyResponse:
        """Gets a key policy attached to the specified customer master key (CMK). You cannot perform this operation on a CMK in a different AWS account.

        :param self:
        :param PolicyName:  Specifies the name of the key policy. The only valid name is <code>default</code>. To get the names of key policies, use <a>ListKeyPolicies</a>.<br/>KeyId:  A unique identifier for the customer master key (CMK). Specify the key ID or the Amazon Resource Name (ARN) of the CMK. For example: <ul> <li> Key ID: <code>1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Key ARN: <code>arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> </ul> To get the key ID and key ARN for a CMK, use <a>ListKeys</a> or <a>DescribeKey</a>.<br/>
        :return: Policy:  A key policy document in JSON format.<br/>        """
        pass

    def get_key_rotation_status(self, KeyId: str) -> GetKeyRotationStatusResponse:
        """Gets a Boolean value that indicates whether <a href="https://docs.aws.amazon.com/kms/latest/developerguide/rotate-keys.html">automatic rotation of the key material</a> is enabled for the specified customer master key (CMK). The result of this operation varies with the key state of the CMK. For details, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-state.html">How Key State Affects Use of a Customer Master Key</a> in the <i>AWS Key Management Service Developer Guide</i>. <ul> <li> Disabled: The key rotation status does not change when you disable a CMK. However, while the CMK is disabled, AWS KMS does not rotate the backing key. </li> <li> Pending deletion: While a CMK is pending deletion, its key rotation status is <code>false</code> and AWS KMS does not rotate the backing key. If you cancel the deletion, the original key rotation status is restored. </li> </ul> To perform this operation on a CMK in a different AWS account, specify the key ARN in the value of the <code>KeyId</code> parameter.

        :param self:
        :param KeyId:  A unique identifier for the customer master key (CMK). Specify the key ID or the Amazon Resource Name (ARN) of the CMK. To specify a CMK in a different AWS account, you must use the key ARN. For example: <ul> <li> Key ID: <code>1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Key ARN: <code>arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> </ul> To get the key ID and key ARN for a CMK, use <a>ListKeys</a> or <a>DescribeKey</a>.<br/>
        :return: KeyRotationEnabled:  A Boolean value that specifies whether key rotation is enabled.<br/>        """
        pass

    def get_parameters_for_import(self, WrappingKeySpec: str, WrappingAlgorithm: str, KeyId: str) -> GetParametersForImportResponse:
        """Returns the items you need in order to import key material into AWS KMS from your existing key management infrastructure. For more information about importing key material into AWS KMS, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/importing-keys.html">Importing Key Material</a> in the <i>AWS Key Management Service Developer Guide</i>. You must specify the key ID of the customer master key (CMK) into which you will import key material. This CMK's <code>Origin</code> must be <code>EXTERNAL</code>. You must also specify the wrapping algorithm and type of wrapping key (public key) that you will use to encrypt the key material. You cannot perform this operation on a CMK in a different AWS account. This operation returns a public key and an import token. Use the public key to encrypt the key material. Store the import token to send with a subsequent <a>ImportKeyMaterial</a> request. The public key and import token from the same response must be used together. These items are valid for 24 hours. When they expire, they cannot be used for a subsequent <a>ImportKeyMaterial</a> request. To get new ones, send another <code>GetParametersForImport</code> request. The result of this operation varies with the key state of the CMK. For details, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-state.html">How Key State Affects Use of a Customer Master Key</a> in the <i>AWS Key Management Service Developer Guide</i>.

        :param self:
        :param WrappingKeySpec:  The type of wrapping key (public key) to return in the response. Only 2048-bit RSA public keys are supported.<br/>WrappingAlgorithm:  The algorithm you will use to encrypt the key material before importing it with <a>ImportKeyMaterial</a>. For more information, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/importing-keys-encrypt-key-material.html">Encrypt the Key Material</a> in the <i>AWS Key Management Service Developer Guide</i>.<br/>KeyId:  The identifier of the CMK into which you will import key material. The CMK's <code>Origin</code> must be <code>EXTERNAL</code>. Specify the key ID or the Amazon Resource Name (ARN) of the CMK. For example: <ul> <li> Key ID: <code>1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Key ARN: <code>arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> </ul> To get the key ID and key ARN for a CMK, use <a>ListKeys</a> or <a>DescribeKey</a>.<br/>
        :return: ParametersValidTo:  The time at which the import token and public key are no longer valid. After this time, you cannot use them to make an <a>ImportKeyMaterial</a> request and you must send another <code>GetParametersForImport</code> request to get new ones.<br/>PublicKey:  The public key to use to encrypt the key material before importing it with <a>ImportKeyMaterial</a>.<br/>ImportToken:  The import token to send in a subsequent <a>ImportKeyMaterial</a> request.<br/>KeyId:  The identifier of the CMK to use in a subsequent <a>ImportKeyMaterial</a> request. This is the same CMK specified in the <code>GetParametersForImport</code> request.<br/>        """
        pass

    def import_key_material(self, EncryptedKeyMaterial: CiphertextType, ImportToken: CiphertextType, KeyId: str, ValidTo: DateType=None, ExpirationModel: str=None) -> ImportKeyMaterialResponse:
        """Imports key material into an existing AWS KMS customer master key (CMK) that was created without key material. You cannot perform this operation on a CMK in a different AWS account. For more information about creating CMKs with no key material and then importing key material, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/importing-keys.html">Importing Key Material</a> in the <i>AWS Key Management Service Developer Guide</i>. Before using this operation, call <a>GetParametersForImport</a>. Its response includes a public key and an import token. Use the public key to encrypt the key material. Then, submit the import token from the same <code>GetParametersForImport</code> response. When calling this operation, you must specify the following values: <ul> <li> The key ID or key ARN of a CMK with no key material. Its <code>Origin</code> must be <code>EXTERNAL</code>. To create a CMK with no key material, call <a>CreateKey</a> and set the value of its <code>Origin</code> parameter to <code>EXTERNAL</code>. To get the <code>Origin</code> of a CMK, call <a>DescribeKey</a>.) </li> <li> The encrypted key material. To get the public key to encrypt the key material, call <a>GetParametersForImport</a>. </li> <li> The import token that <a>GetParametersForImport</a> returned. This token and the public key used to encrypt the key material must have come from the same response. </li> <li> Whether the key material expires and if so, when. If you set an expiration date, you can change it only by reimporting the same key material and specifying a new expiration date. If the key material expires, AWS KMS deletes the key material and the CMK becomes unusable. To use the CMK again, you must reimport the same key material. </li> </ul> When this operation is successful, the key state of the CMK changes from <code>PendingImport</code> to <code>Enabled</code>, and you can use the CMK. After you successfully import key material into a CMK, you can reimport the same key material into that CMK, but you cannot import different key material. The result of this operation varies with the key state of the CMK. For details, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-state.html">How Key State Affects Use of a Customer Master Key</a> in the <i>AWS Key Management Service Developer Guide</i>.

        :param self:
        :param ExpirationModel:  Specifies whether the key material expires. The default is <code>KEY_MATERIAL_EXPIRES</code>, in which case you must include the <code>ValidTo</code> parameter. When this parameter is set to <code>KEY_MATERIAL_DOES_NOT_EXPIRE</code>, you must omit the <code>ValidTo</code> parameter.<br/>ValidTo:  The time at which the imported key material expires. When the key material expires, AWS KMS deletes the key material and the CMK becomes unusable. You must omit this parameter when the <code>ExpirationModel</code> parameter is set to <code>KEY_MATERIAL_DOES_NOT_EXPIRE</code>. Otherwise it is required.<br/>EncryptedKeyMaterial:  The encrypted key material to import. It must be encrypted with the public key that you received in the response to a previous <a>GetParametersForImport</a> request, using the wrapping algorithm that you specified in that request.<br/>ImportToken:  The import token that you received in the response to a previous <a>GetParametersForImport</a> request. It must be from the same response that contained the public key that you used to encrypt the key material.<br/>KeyId:  The identifier of the CMK to import the key material into. The CMK's <code>Origin</code> must be <code>EXTERNAL</code>. Specify the key ID or the Amazon Resource Name (ARN) of the CMK. For example: <ul> <li> Key ID: <code>1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Key ARN: <code>arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> </ul> To get the key ID and key ARN for a CMK, use <a>ListKeys</a> or <a>DescribeKey</a>.<br/>
        :return:         """
        pass

    def list_aliases(self, KeyId: str=None, Limit: int=None, Marker: str=None) -> ListAliasesResponse:
        """Gets a list of aliases in the caller's AWS account and region. You cannot list aliases in other accounts. For more information about aliases, see <a>CreateAlias</a>. By default, the ListAliases command returns all aliases in the account and region. To get only the aliases that point to a particular customer master key (CMK), use the <code>KeyId</code> parameter. The <code>ListAliases</code> response can include aliases that you created and associated with your customer managed CMKs, and aliases that AWS created and associated with AWS managed CMKs in your account. You can recognize AWS aliases because their names have the format <code>aws/&lt;service-name&gt;</code>, such as <code>aws/dynamodb</code>. The response might also include aliases that have no <code>TargetKeyId</code> field. These are predefined aliases that AWS has created but has not yet associated with a CMK. Aliases that AWS creates in your account, including predefined aliases, do not count against your <a href="https://docs.aws.amazon.com/kms/latest/developerguide/limits.html#aliases-limit">AWS KMS aliases limit</a>.

        :param self:
        :param Marker:  Use this parameter in a subsequent request after you receive a response with truncated results. Set it to the value of <code>NextMarker</code> from the truncated response you just received.<br/>Limit:  Use this parameter to specify the maximum number of items to return. When this value is present, AWS KMS does not return more than the specified number of items, but it might return fewer. This value is optional. If you include a value, it must be between 1 and 100, inclusive. If you do not include a value, it defaults to 50.<br/>KeyId:  Lists only aliases that refer to the specified CMK. The value of this parameter can be the ID or Amazon Resource Name (ARN) of a CMK in the caller's account and region. You cannot use an alias name or alias ARN in this value. This parameter is optional. If you omit it, <code>ListAliases</code> returns all aliases in the account and region.<br/>
        :return: Truncated:  A flag that indicates whether there are more items in the list. When this value is true, the list in this response is truncated. To get more items, pass the value of the <code>NextMarker</code> element in thisresponse to the <code>Marker</code> parameter in a subsequent request.<br/>NextMarker:  When <code>Truncated</code> is true, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent request.<br/>Aliases:  A list of aliases.<br/>        """
        pass

    def list_grants(self, KeyId: str, Limit: int=None, Marker: str=None) -> ListGrantsResponse:
        """Gets a list of all grants for the specified customer master key (CMK). To perform this operation on a CMK in a different AWS account, specify the key ARN in the value of the <code>KeyId</code> parameter.

        :param self:
        :param KeyId:  A unique identifier for the customer master key (CMK). Specify the key ID or the Amazon Resource Name (ARN) of the CMK. To specify a CMK in a different AWS account, you must use the key ARN. For example: <ul> <li> Key ID: <code>1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Key ARN: <code>arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> </ul> To get the key ID and key ARN for a CMK, use <a>ListKeys</a> or <a>DescribeKey</a>.<br/>Marker:  Use this parameter in a subsequent request after you receive a response with truncated results. Set it to the value of <code>NextMarker</code> from the truncated response you just received.<br/>Limit:  Use this parameter to specify the maximum number of items to return. When this value is present, AWS KMS does not return more than the specified number of items, but it might return fewer. This value is optional. If you include a value, it must be between 1 and 100, inclusive. If you do not include a value, it defaults to 50.<br/>
        :return: Truncated:  A flag that indicates whether there are more items in the list. When this value is true, the list in this response is truncated. To get more items, pass the value of the <code>NextMarker</code> element in thisresponse to the <code>Marker</code> parameter in a subsequent request.<br/>NextMarker:  When <code>Truncated</code> is true, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent request.<br/>Grants:  A list of grants.<br/>        """
        pass

    def list_key_policies(self, KeyId: str, Limit: int=None, Marker: str=None) -> ListKeyPoliciesResponse:
        """Gets the names of the key policies that are attached to a customer master key (CMK). This operation is designed to get policy names that you can use in a <a>GetKeyPolicy</a> operation. However, the only valid policy name is <code>default</code>. You cannot perform this operation on a CMK in a different AWS account.

        :param self:
        :param Marker:  Use this parameter in a subsequent request after you receive a response with truncated results. Set it to the value of <code>NextMarker</code> from the truncated response you just received.<br/>Limit:  Use this parameter to specify the maximum number of items to return. When this value is present, AWS KMS does not return more than the specified number of items, but it might return fewer. This value is optional. If you include a value, it must be between 1 and 1000, inclusive. If you do not include a value, it defaults to 100. Only one policy can be attached to a key.<br/>KeyId:  A unique identifier for the customer master key (CMK). Specify the key ID or the Amazon Resource Name (ARN) of the CMK. For example: <ul> <li> Key ID: <code>1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Key ARN: <code>arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> </ul> To get the key ID and key ARN for a CMK, use <a>ListKeys</a> or <a>DescribeKey</a>.<br/>
        :return: Truncated:  A flag that indicates whether there are more items in the list. When this value is true, the list in this response is truncated. To get more items, pass the value of the <code>NextMarker</code> element in thisresponse to the <code>Marker</code> parameter in a subsequent request.<br/>NextMarker:  When <code>Truncated</code> is true, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent request.<br/>PolicyNames:  A list of key policy names. The only valid value is <code>default</code>.<br/>        """
        pass

    def list_keys(self, Limit: int=None, Marker: str=None) -> ListKeysResponse:
        """Gets a list of all customer master keys (CMKs) in the caller's AWS account and region.

        :param self:
        :param Marker:  Use this parameter in a subsequent request after you receive a response with truncated results. Set it to the value of <code>NextMarker</code> from the truncated response you just received.<br/>Limit:  Use this parameter to specify the maximum number of items to return. When this value is present, AWS KMS does not return more than the specified number of items, but it might return fewer. This value is optional. If you include a value, it must be between 1 and 1000, inclusive. If you do not include a value, it defaults to 100.<br/>
        :return: Truncated:  A flag that indicates whether there are more items in the list. When this value is true, the list in this response is truncated. To get more items, pass the value of the <code>NextMarker</code> element in thisresponse to the <code>Marker</code> parameter in a subsequent request.<br/>NextMarker:  When <code>Truncated</code> is true, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent request.<br/>Keys:  A list of customer master keys (CMKs).<br/>        """
        pass

    def list_resource_tags(self, KeyId: str, Limit: int=None, Marker: str=None) -> ListResourceTagsResponse:
        """Returns a list of all tags for the specified customer master key (CMK). You cannot perform this operation on a CMK in a different AWS account.

        :param self:
        :param Marker:  Use this parameter in a subsequent request after you receive a response with truncated results. Set it to the value of <code>NextMarker</code> from the truncated response you just received. Do not attempt to construct this value. Use only the value of <code>NextMarker</code> from the truncated response you just received.<br/>Limit:  Use this parameter to specify the maximum number of items to return. When this value is present, AWS KMS does not return more than the specified number of items, but it might return fewer. This value is optional. If you include a value, it must be between 1 and 50, inclusive. If you do not include a value, it defaults to 50.<br/>KeyId:  A unique identifier for the customer master key (CMK). Specify the key ID or the Amazon Resource Name (ARN) of the CMK. For example: <ul> <li> Key ID: <code>1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Key ARN: <code>arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> </ul> To get the key ID and key ARN for a CMK, use <a>ListKeys</a> or <a>DescribeKey</a>.<br/>
        :return: Truncated:  A flag that indicates whether there are more items in the list. When this value is true, the list in this response is truncated. To get more items, pass the value of the <code>NextMarker</code> element in thisresponse to the <code>Marker</code> parameter in a subsequent request.<br/>NextMarker:  When <code>Truncated</code> is true, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent request. Do not assume or infer any information from this value.<br/>Tags:  A list of tags. Each tag consists of a tag key and a tag value.<br/>        """
        pass

    def list_retirable_grants(self, RetiringPrincipal: str, Limit: int=None, Marker: str=None) -> ListGrantsResponse:
        """Returns a list of all grants for which the grant's <code>RetiringPrincipal</code> matches the one specified. A typical use is to list all grants that you are able to retire. To retire a grant, use <a>RetireGrant</a>.

        :param self:
        :param RetiringPrincipal:  The retiring principal for which to list grants. To specify the retiring principal, use the <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Name (ARN)</a> of an AWS principal. Valid AWS principals include AWS accounts (root), IAM users, federated users, and assumed role users. For examples of the ARN syntax for specifying a principal, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#arn-syntax-iam">AWS Identity and Access Management (IAM)</a> in the Example ARNs section of the <i>Amazon Web Services General Reference</i>.<br/>Marker:  Use this parameter in a subsequent request after you receive a response with truncated results. Set it to the value of <code>NextMarker</code> from the truncated response you just received.<br/>Limit:  Use this parameter to specify the maximum number of items to return. When this value is present, AWS KMS does not return more than the specified number of items, but it might return fewer. This value is optional. If you include a value, it must be between 1 and 100, inclusive. If you do not include a value, it defaults to 50.<br/>
        :return: Truncated:  A flag that indicates whether there are more items in the list. When this value is true, the list in this response is truncated. To get more items, pass the value of the <code>NextMarker</code> element in thisresponse to the <code>Marker</code> parameter in a subsequent request.<br/>NextMarker:  When <code>Truncated</code> is true, this element is present and contains the value to use for the <code>Marker</code> parameter in a subsequent request.<br/>Grants:  A list of grants.<br/>        """
        pass

    def put_key_policy(self, Policy: str, PolicyName: str, KeyId: str, BypassPolicyLockoutSafetyCheck: BooleanType=None):
        """Attaches a key policy to the specified customer master key (CMK). You cannot perform this operation on a CMK in a different AWS account. For more information about key policies, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-policies.html">Key Policies</a> in the <i>AWS Key Management Service Developer Guide</i>.

        :param self:
        :param BypassPolicyLockoutSafetyCheck:  A flag to indicate whether to bypass the key policy lockout safety check. <important> Setting this value to true increases the risk that the CMK becomes unmanageable. Do not set this value to true indiscriminately. For more information, refer to the scenario in the <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-policies.html#key-policy-default-allow-root-enable-iam">Default Key Policy</a> section in the <i>AWS Key Management Service Developer Guide</i>. </important> Use this parameter only when you intend to prevent the principal that is making the request from making a subsequent <code>PutKeyPolicy</code> request on the CMK. The default value is false.<br/>Policy:  The key policy to attach to the CMK. The key policy must meet the following criteria: <ul> <li> If you don't set <code>BypassPolicyLockoutSafetyCheck</code> to true, the key policy must allow the principal that is making the <code>PutKeyPolicy</code> request to make a subsequent <code>PutKeyPolicy</code> request on the CMK. This reduces the risk that the CMK becomes unmanageable. For more information, refer to the scenario in the <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-policies.html#key-policy-default-allow-root-enable-iam">Default Key Policy</a> section of the <i>AWS Key Management Service Developer Guide</i>. </li> <li> Each statement in the key policy must contain one or more principals. The principals in the key policy must exist and be visible to AWS KMS. When you create a new AWS principal (for example, an IAM user or role), you might need to enforce a delay before including the new principal in a key policy because the new principal might not be immediately visible to AWS KMS. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/troubleshoot_general.html#troubleshoot_general_eventual-consistency">Changes that I make are not always immediately visible</a> in the <i>AWS Identity and Access Management User Guide</i>. </li> </ul> The key policy size limit is 32 kilobytes (32768 bytes).<br/>PolicyName:  The name of the key policy. The only valid value is <code>default</code>.<br/>KeyId:  A unique identifier for the customer master key (CMK). Specify the key ID or the Amazon Resource Name (ARN) of the CMK. For example: <ul> <li> Key ID: <code>1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Key ARN: <code>arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> </ul> To get the key ID and key ARN for a CMK, use <a>ListKeys</a> or <a>DescribeKey</a>.<br/>
        :return:         """
        pass

    def re_encrypt(self, DestinationKeyId: str, CiphertextBlob: CiphertextType, SourceEncryptionContext: EncryptionContextType=None, DestinationEncryptionContext: EncryptionContextType=None, GrantTokens: List[GrantTokenType]=None) -> ReEncryptResponse:
        """Encrypts data on the server side with a new customer master key (CMK) without exposing the plaintext of the data on the client side. The data is first decrypted and then reencrypted. You can also use this operation to change the encryption context of a ciphertext.  You can reencrypt data using CMKs in different AWS accounts. Unlike other operations, <code>ReEncrypt</code> is authorized twice, once as <code>ReEncryptFrom</code> on the source CMK and once as <code>ReEncryptTo</code> on the destination CMK. We recommend that you include the <code>"kms:ReEncrypt*"</code> permission in your <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-policies.html">key policies</a> to permit reencryption from or to the CMK. This permission is automatically included in the key policy when you create a CMK through the console. But you must include it manually when you create a CMK programmatically or when you set a key policy with the <a>PutKeyPolicy</a> operation. The result of this operation varies with the key state of the CMK. For details, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-state.html">How Key State Affects Use of a Customer Master Key</a> in the <i>AWS Key Management Service Developer Guide</i>.

        :param self:
        :param GrantTokens:  A list of grant tokens. For more information, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#grant_token">Grant Tokens</a> in the <i>AWS Key Management Service Developer Guide</i>.<br/>DestinationEncryptionContext:  Encryption context to use when the data is reencrypted.<br/>DestinationKeyId:  A unique identifier for the CMK that is used to reencrypt the data. To specify a CMK, use its key ID, Amazon Resource Name (ARN), alias name, or alias ARN. When using an alias name, prefix it with <code>"alias/"</code>. To specify a CMK in a different AWS account, you must use the key ARN or alias ARN. For example: <ul> <li> Key ID: <code>1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Key ARN: <code>arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Alias name: <code>alias/ExampleAlias</code>  </li> <li> Alias ARN: <code>arn:aws:kms:us-east-2:111122223333:alias/ExampleAlias</code>  </li> </ul> To get the key ID and key ARN for a CMK, use <a>ListKeys</a> or <a>DescribeKey</a>. To get the alias name and alias ARN, use <a>ListAliases</a>.<br/>SourceEncryptionContext:  Encryption context used to encrypt and decrypt the data specified in the <code>CiphertextBlob</code> parameter.<br/>CiphertextBlob:  Ciphertext of the data to reencrypt.<br/>
        :return: KeyId:  Unique identifier of the CMK used to reencrypt the data.<br/>SourceKeyId:  Unique identifier of the CMK used to originally encrypt the data.<br/>CiphertextBlob:  The reencrypted data. When you use the HTTP API or the AWS CLI, the value is Base64-encoded. Otherwise, it is not encoded.<br/>        """
        pass

    def retire_grant(self, GrantToken: str=None, KeyId: str=None, GrantId: str=None):
        """Retires a grant. To clean up, you can retire a grant when you're done using it. You should revoke a grant when you intend to actively deny operations that depend on it. The following are permitted to call this API: <ul> <li> The AWS account (root user) under which the grant was created </li> <li> The <code>RetiringPrincipal</code>, if present in the grant </li> <li> The <code>GranteePrincipal</code>, if <code>RetireGrant</code> is an operation specified in the grant </li> </ul> You must identify the grant to retire by its grant token or by a combination of the grant ID and the Amazon Resource Name (ARN) of the customer master key (CMK). A grant token is a unique variable-length base64-encoded string. A grant ID is a 64 character unique identifier of a grant. The <a>CreateGrant</a> operation returns both.

        :param self:
        :param GrantId:  Unique identifier of the grant to retire. The grant ID is returned in the response to a <code>CreateGrant</code> operation. <ul> <li> Grant ID Example - 0123456789012345678901234567890123456789012345678901234567890123 </li> </ul><br/>KeyId:  The Amazon Resource Name (ARN) of the CMK associated with the grant.  For example: <code>arn:aws:kms:us-east-2:444455556666:key/1234abcd-12ab-34cd-56ef-1234567890ab</code> <br/>GrantToken:  Token that identifies the grant to be retired.<br/>
        :return:         """
        pass

    def revoke_grant(self, GrantId: str, KeyId: str):
        """Revokes the specified grant for the specified customer master key (CMK). You can revoke a grant to actively deny operations that depend on it. To perform this operation on a CMK in a different AWS account, specify the key ARN in the value of the <code>KeyId</code> parameter.

        :param self:
        :param GrantId:  Identifier of the grant to be revoked.<br/>KeyId:  A unique identifier for the customer master key associated with the grant. Specify the key ID or the Amazon Resource Name (ARN) of the CMK. To specify a CMK in a different AWS account, you must use the key ARN. For example: <ul> <li> Key ID: <code>1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Key ARN: <code>arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> </ul> To get the key ID and key ARN for a CMK, use <a>ListKeys</a> or <a>DescribeKey</a>.<br/>
        :return:         """
        pass

    def schedule_key_deletion(self, KeyId: str, PendingWindowInDays: int=None) -> ScheduleKeyDeletionResponse:
        """Schedules the deletion of a customer master key (CMK). You may provide a waiting period, specified in days, before deletion occurs. If you do not provide a waiting period, the default period of 30 days is used. When this operation is successful, the key state of the CMK changes to <code>PendingDeletion</code>. Before the waiting period ends, you can use <a>CancelKeyDeletion</a> to cancel the deletion of the CMK. After the waiting period ends, AWS KMS deletes the CMK and all AWS KMS data associated with it, including all aliases that refer to it. <important> Deleting a CMK is a destructive and potentially dangerous operation. When a CMK is deleted, all data that was encrypted under the CMK is unrecoverable. To prevent the use of a CMK without deleting it, use <a>DisableKey</a>. </important> If you schedule deletion of a CMK from a <a href="https://docs.aws.amazon.com/kms/latest/developerguide/custom-key-store-overview.html">custom key store</a>, when the waiting period expires, <code>ScheduleKeyDeletion</code> deletes the CMK from AWS KMS. Then AWS KMS makes a best effort to delete the key material from the associated AWS CloudHSM cluster. However, you might need to manually <a href="https://docs.aws.amazon.com/kms/latest/developerguide/fix-keystore.html#fix-keystore-orphaned-key">delete the orphaned key material</a> from the cluster and its backups. You cannot perform this operation on a CMK in a different AWS account. For more information about scheduling a CMK for deletion, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/deleting-keys.html">Deleting Customer Master Keys</a> in the <i>AWS Key Management Service Developer Guide</i>. The result of this operation varies with the key state of the CMK. For details, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-state.html">How Key State Affects Use of a Customer Master Key</a> in the <i>AWS Key Management Service Developer Guide</i>.

        :param self:
        :param PendingWindowInDays:  The waiting period, specified in number of days. After the waiting period ends, AWS KMS deletes the customer master key (CMK). This value is optional. If you include a value, it must be between 7 and 30, inclusive. If you do not include a value, it defaults to 30.<br/>KeyId:  The unique identifier of the customer master key (CMK) to delete. Specify the key ID or the Amazon Resource Name (ARN) of the CMK. For example: <ul> <li> Key ID: <code>1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Key ARN: <code>arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> </ul> To get the key ID and key ARN for a CMK, use <a>ListKeys</a> or <a>DescribeKey</a>.<br/>
        :return: DeletionDate:  The date and time after which AWS KMS deletes the customer master key (CMK).<br/>KeyId:  The unique identifier of the customer master key (CMK) for which deletion is scheduled.<br/>        """
        pass

    def tag_resource(self, Tags: List[Tag], KeyId: str):
        """Adds or edits tags for a customer master key (CMK). You cannot perform this operation on a CMK in a different AWS account. Each tag consists of a tag key and a tag value. Tag keys and tag values are both required, but tag values can be empty (null) strings. You can only use a tag key once for each CMK. If you use the tag key again, AWS KMS replaces the current tag value with the specified value. For information about the rules that apply to tag keys and tag values, see <a href="https://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/allocation-tag-restrictions.html">User-Defined Tag Restrictions</a> in the <i>AWS Billing and Cost Management User Guide</i>. The result of this operation varies with the key state of the CMK. For details, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-state.html">How Key State Affects Use of a Customer Master Key</a> in the <i>AWS Key Management Service Developer Guide</i>.

        :param self:
        :param Tags:  One or more tags. Each tag consists of a tag key and a tag value.<br/>KeyId:  A unique identifier for the CMK you are tagging. Specify the key ID or the Amazon Resource Name (ARN) of the CMK. For example: <ul> <li> Key ID: <code>1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Key ARN: <code>arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> </ul> To get the key ID and key ARN for a CMK, use <a>ListKeys</a> or <a>DescribeKey</a>.<br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[TagKeyType], KeyId: str):
        """Removes the specified tags from the specified customer master key (CMK). You cannot perform this operation on a CMK in a different AWS account. To remove a tag, specify the tag key. To change the tag value of an existing tag key, use <a>TagResource</a>. The result of this operation varies with the key state of the CMK. For details, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-state.html">How Key State Affects Use of a Customer Master Key</a> in the <i>AWS Key Management Service Developer Guide</i>.

        :param self:
        :param TagKeys:  One or more tag keys. Specify only the tag keys, not the tag values.<br/>KeyId:  A unique identifier for the CMK from which you are removing tags. Specify the key ID or the Amazon Resource Name (ARN) of the CMK. For example: <ul> <li> Key ID: <code>1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Key ARN: <code>arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> </ul> To get the key ID and key ARN for a CMK, use <a>ListKeys</a> or <a>DescribeKey</a>.<br/>
        :return:         """
        pass

    def update_alias(self, TargetKeyId: str, AliasName: str):
        """Associates an existing alias with a different customer master key (CMK). Each CMK can have multiple aliases, but the aliases must be unique within the account and region. You cannot perform this operation on an alias in a different AWS account. This operation works only on existing aliases. To change the alias of a CMK to a new value, use <a>CreateAlias</a> to create a new alias and <a>DeleteAlias</a> to delete the old alias. Because an alias is not a property of a CMK, you can create, update, and delete the aliases of a CMK without affecting the CMK. Also, aliases do not appear in the response from the <a>DescribeKey</a> operation. To get the aliases of all CMKs in the account, use the <a>ListAliases</a> operation.  The alias name must begin with <code>alias/</code> followed by a name, such as <code>alias/ExampleAlias</code>. It can contain only alphanumeric characters, forward slashes (/), underscores (_), and dashes (-). The alias name cannot begin with <code>alias/aws/</code>. The <code>alias/aws/</code> prefix is reserved for <a href="https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#aws-managed-cmk">AWS managed CMKs</a>.  The result of this operation varies with the key state of the CMK. For details, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-state.html">How Key State Affects Use of a Customer Master Key</a> in the <i>AWS Key Management Service Developer Guide</i>.

        :param self:
        :param TargetKeyId:  Unique identifier of the customer master key (CMK) to be mapped to the alias. When the update operation completes, the alias will point to this CMK. Specify the key ID or the Amazon Resource Name (ARN) of the CMK. For example: <ul> <li> Key ID: <code>1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Key ARN: <code>arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> </ul> To get the key ID and key ARN for a CMK, use <a>ListKeys</a> or <a>DescribeKey</a>. To verify that the alias is mapped to the correct CMK, use <a>ListAliases</a>.<br/>AliasName:  Specifies the name of the alias to change. This value must begin with <code>alias/</code> followed by the alias name, such as <code>alias/ExampleAlias</code>.<br/>
        :return:         """
        pass

    def update_custom_key_store(self, CustomKeyStoreId: str, NewCustomKeyStoreName: str=None, KeyStorePassword: str=None, CloudHsmClusterId: str=None) -> UpdateCustomKeyStoreResponse:
        """Changes the properties of a custom key store. Use the <code>CustomKeyStoreId</code> parameter to identify the custom key store you want to edit. Use the remaining parameters to change the properties of the custom key store. You can only update a custom key store that is disconnected. To disconnect the custom key store, use <a>DisconnectCustomKeyStore</a>. To reconnect the custom key store after the update completes, use <a>ConnectCustomKeyStore</a>. To find the connection state of a custom key store, use the <a>DescribeCustomKeyStores</a> operation. Use the parameters of <code>UpdateCustomKeyStore</code> to edit your keystore settings. <ul> <li> Use the <b>NewCustomKeyStoreName</b> parameter to change the friendly name of the custom key store to the value that you specify.   </li> <li> Use the <b>KeyStorePassword</b> parameter tell AWS KMS the current password of the <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-store-concepts.html#concept-kmsuser"> <code>kmsuser</code> crypto user (CU)</a> in the associated AWS CloudHSM cluster. You can use this parameter to <a href="https://docs.aws.amazon.com/kms/latest/developerguide/fix-keystore.html#fix-keystore-password">fix connection failures</a> that occur when AWS KMS cannot log into the associated cluster because the <code>kmsuser</code> password has changed. This value does not change the password in the AWS CloudHSM cluster.   </li> <li> Use the <b>CloudHsmClusterId</b> parameter to associate the custom key store with a different, but related, AWS CloudHSM cluster. You can use this parameter to repair a custom key store if its AWS CloudHSM cluster becomes corrupted or is deleted, or when you need to create or restore a cluster from a backup.  </li> </ul> If the operation succeeds, it returns a JSON object with no properties. This operation is part of the <a href="https://docs.aws.amazon.com/kms/latest/developerguide/custom-key-store-overview.html">Custom Key Store feature</a> feature in AWS KMS, which combines the convenience and extensive integration of AWS KMS with the isolation and control of a single-tenant key store.

        :param self:
        :param CloudHsmClusterId:  Associates the custom key store with a related AWS CloudHSM cluster.  Enter the cluster ID of the cluster that you used to create the custom key store or a cluster that shares a backup history and has the same cluster certificate as the original cluster. You cannot use this parameter to associate a custom key store with an unrelated cluster. In addition, the replacement cluster must <a href="https://docs.aws.amazon.com/kms/latest/developerguide/create-keystore.html#before-keystore">fulfill the requirements</a> for a cluster associated with a custom key store. To view the cluster certificate of a cluster, use the <a href="https://docs.aws.amazon.com/cloudhsm/latest/APIReference/API_DescribeClusters.html">DescribeClusters</a> operation.<br/>KeyStorePassword:  Enter the current password of the <code>kmsuser</code> crypto user (CU) in the AWS CloudHSM cluster that is associated with the custom key store. This parameter tells AWS KMS the current password of the <code>kmsuser</code> crypto user (CU). It does not set or change the password of any users in the AWS CloudHSM cluster.<br/>NewCustomKeyStoreName:  Changes the friendly name of the custom key store to the value that you specify. The custom key store name must be unique in the AWS account.<br/>CustomKeyStoreId:  Identifies the custom key store that you want to update. Enter the ID of the custom key store. To find the ID of a custom key store, use the <a>DescribeCustomKeyStores</a> operation.<br/>
        :return:         """
        pass

    def update_key_description(self, Description: str, KeyId: str):
        """Updates the description of a customer master key (CMK). To see the description of a CMK, use <a>DescribeKey</a>.  You cannot perform this operation on a CMK in a different AWS account. The result of this operation varies with the key state of the CMK. For details, see <a href="https://docs.aws.amazon.com/kms/latest/developerguide/key-state.html">How Key State Affects Use of a Customer Master Key</a> in the <i>AWS Key Management Service Developer Guide</i>.

        :param self:
        :param Description:  New description for the CMK.<br/>KeyId:  A unique identifier for the customer master key (CMK). Specify the key ID or the Amazon Resource Name (ARN) of the CMK. For example: <ul> <li> Key ID: <code>1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> <li> Key ARN: <code>arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>  </li> </ul> To get the key ID and key ARN for a CMK, use <a>ListKeys</a> or <a>DescribeKey</a>.<br/>
        :return:         """
        pass



class Lambda(BaseClient):
    """<fullname>AWS Lambda</fullname>  <b>Overview</b>  This is the <i>AWS Lambda API Reference</i>. The AWS Lambda Developer Guide provides additional information. For the service overview, see <a href="https://docs.aws.amazon.com/lambda/latest/dg/welcome.html">What is AWS Lambda</a>, and for information about how the service works, see <a href="https://docs.aws.amazon.com/lambda/latest/dg/lambda-introduction.html">AWS Lambda: How it Works</a> in the <b>AWS Lambda Developer Guide</b>."""

    class FunctionExistsWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `get_function` for valid parameters"""
            pass


    class ListEventSourceMappingsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_event_source_mappings` for valid parameters"""
            pass
    class ListFunctionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_functions` for valid parameters"""
            pass
    class ListAliasesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_aliases` for valid parameters"""
            pass
    class ListLayerVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_layer_versions` for valid parameters"""
            pass
    class ListLayersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_layers` for valid parameters"""
            pass
    class ListVersionsByFunctionPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_versions_by_function` for valid parameters"""
            pass


    class AddLayerVersionPermissionRequest(Mapping):
        pass
    
    class AddLayerVersionPermissionResponse(Mapping):
        pass
    
    class LayerVersionNumber(object):
        pass
    
    class AddPermissionRequest(Mapping):
        pass
    
    class AddPermissionResponse(Mapping):
        pass
    
    class CreateAliasRequest(Mapping):
        pass
    
    class AliasConfiguration(Mapping):
        pass
    
    class AliasRoutingConfiguration(Mapping):
        pass
    
    class CreateEventSourceMappingRequest(Mapping):
        pass
    
    class EventSourceMappingConfiguration(Mapping):
        pass
    
    class Enabled(object):
        pass
    
    class Date(object):
        pass
    
    class CreateFunctionRequest(Mapping):
        pass
    
    class FunctionConfiguration(Mapping):
        pass
    
    class FunctionCode(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class VpcConfig(Mapping):
        pass
    
    class DeadLetterConfig(Mapping):
        pass
    
    class Environment(Mapping):
        pass
    
    class TracingConfig(Mapping):
        pass
    
    class Tags(object):
        pass
    
    class LayerVersionArn(object):
        pass
    
    class DeleteAliasRequest(Mapping):
        pass
    
    class DeleteEventSourceMappingRequest(Mapping):
        pass
    
    class DeleteFunctionRequest(Mapping):
        pass
    
    class DeleteFunctionConcurrencyRequest(Mapping):
        pass
    
    class DeleteLayerVersionRequest(Mapping):
        pass
    
    class GetAccountSettingsRequest(Mapping):
        pass
    
    class GetAccountSettingsResponse(Mapping):
        pass
    
    class GetAliasRequest(Mapping):
        pass
    
    class GetEventSourceMappingRequest(Mapping):
        pass
    
    class GetFunctionRequest(Mapping):
        pass
    
    class GetFunctionResponse(Mapping):
        pass
    
    class GetFunctionConfigurationRequest(Mapping):
        pass
    
    class GetLayerVersionRequest(Mapping):
        pass
    
    class GetLayerVersionResponse(Mapping):
        pass
    
    class GetLayerVersionByArnRequest(Mapping):
        pass
    
    class GetLayerVersionPolicyRequest(Mapping):
        pass
    
    class GetLayerVersionPolicyResponse(Mapping):
        pass
    
    class GetPolicyRequest(Mapping):
        pass
    
    class GetPolicyResponse(Mapping):
        pass
    
    class InvocationRequest(Mapping):
        pass
    
    class InvocationResponse(Mapping):
        pass
    
    class Blob(object):
        pass
    
    class InvokeAsyncRequest(Mapping):
        pass
    
    class InvokeAsyncResponse(Mapping):
        pass
    
    class BlobStream(object):
        pass
    
    class ListAliasesRequest(Mapping):
        pass
    
    class ListAliasesResponse(Mapping):
        pass
    
    class ListEventSourceMappingsRequest(Mapping):
        pass
    
    class ListEventSourceMappingsResponse(Mapping):
        pass
    
    class ListFunctionsRequest(Mapping):
        pass
    
    class ListFunctionsResponse(Mapping):
        pass
    
    class ListLayerVersionsRequest(Mapping):
        pass
    
    class ListLayerVersionsResponse(Mapping):
        pass
    
    class ListLayersRequest(Mapping):
        pass
    
    class ListLayersResponse(Mapping):
        pass
    
    class ListTagsRequest(Mapping):
        pass
    
    class ListTagsResponse(Mapping):
        pass
    
    class ListVersionsByFunctionRequest(Mapping):
        pass
    
    class ListVersionsByFunctionResponse(Mapping):
        pass
    
    class PublishLayerVersionRequest(Mapping):
        pass
    
    class PublishLayerVersionResponse(Mapping):
        pass
    
    class LayerVersionContentInput(Mapping):
        pass
    
    class Runtime(object):
        pass
    
    class PublishVersionRequest(Mapping):
        pass
    
    class PutFunctionConcurrencyRequest(Mapping):
        pass
    
    class Concurrency(Mapping):
        pass
    
    class RemoveLayerVersionPermissionRequest(Mapping):
        pass
    
    class RemovePermissionRequest(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateAliasRequest(Mapping):
        pass
    
    class UpdateEventSourceMappingRequest(Mapping):
        pass
    
    class UpdateFunctionCodeRequest(Mapping):
        pass
    
    class UpdateFunctionConfigurationRequest(Mapping):
        pass
    
    def add_layer_version_permission(self, Principal: str, Action: str, StatementId: str, VersionNumber: LayerVersionNumber, LayerName: str, OrganizationId: str=None, RevisionId: str=None) -> AddLayerVersionPermissionResponse:
        """Adds permissions to the resource-based policy of a version of an <a href="https://docs.aws.amazon.com/lambda/latest/dg/configuration-layers.html">AWS Lambda layer</a>. Use this action to grant layer usage permission to other accounts. You can grant permission to a single account, all AWS accounts, or all accounts in an organization. To revoke permission, call <a>RemoveLayerVersionPermission</a> with the statement ID that you specified when you added it.

        :param self:
        :param RevisionId:  Only update the policy if the revision ID matches the ID specified. Use this option to avoid modifying a policy that has changed since you last read it.<br/>OrganizationId:  With the principal set to <code>*</code>, grant permission to all accounts in the specified organization.<br/>Principal:  An account ID, or <code>*</code> to grant permission to all AWS accounts.<br/>Action:  The API action that grants access to the layer. For example, <code>lambda:GetLayerVersion</code>.<br/>StatementId:  An identifier that distinguishes the policy from others on the same layer version.<br/>VersionNumber:  The version number.<br/>LayerName:  The name or Amazon Resource Name (ARN) of the layer.<br/>
        :return: RevisionId:  A unique identifier for the current revision of the policy.<br/>Statement:  The permission statement.<br/>        """
        pass

    def add_permission(self, Principal: str, Action: str, StatementId: str, FunctionName: str, SourceArn: str=None, SourceAccount: str=None, EventSourceToken: str=None, Qualifier: str=None, RevisionId: str=None) -> AddPermissionResponse:
        """Grants an AWS service or another account permission to use a function. You can apply the policy at the function level, or specify a qualifier to restrict access to a single version or alias. If you use a qualifier, the invoker must use the full Amazon Resource Name (ARN) of that version or alias to invoke the function. To grant permission to another account, specify the account ID as the <code>Principal</code>. For AWS services, the principal is a domain-style identifier defined by the service, like <code>s3.amazonaws.com</code> or <code>sns.amazonaws.com</code>. For AWS services, you can also specify the ARN or owning account of the associated resource as the <code>SourceArn</code> or <code>SourceAccount</code>. If you grant permission to a service principal without specifying the source, other accounts could potentially configure resources in their account to invoke your Lambda function. This action adds a statement to a resource-based permission policy for the function. For more information about function policies, see <a href="https://docs.aws.amazon.com/lambda/latest/dg/access-control-resource-based.html">Lambda Function Policies</a>. 

        :param self:
        :param RevisionId:  Only update the policy if the revision ID matches the ID that's specified. Use this option to avoid modifying a policy that has changed since you last read it.<br/>Qualifier:  Specify a version or alias to add permissions to a published version of the function.<br/>EventSourceToken:  For Alexa Smart Home functions, a token that must be supplied by the invoker.<br/>SourceAccount:  For AWS services, the ID of the account that owns the resource. Use this instead of <code>SourceArn</code> to grant permission to resources that are owned by another account (for example, all of an account's Amazon S3 buckets). Or use it together with <code>SourceArn</code> to ensure that the resource is owned by the specified account. For example, an Amazon S3 bucket could be deleted by its owner and recreated by another account.<br/>SourceArn:  For AWS services, the ARN of the AWS resource that invokes the function. For example, an Amazon S3 bucket or Amazon SNS topic.<br/>Principal:  The AWS service or account that invokes the function. If you specify a service, use <code>SourceArn</code> or <code>SourceAccount</code> to limit who can invoke the function through that service.<br/>Action:  The action that the principal can use on the function. For example, <code>lambda:InvokeFunction</code> or <code>lambda:GetFunction</code>.<br/>StatementId:  A statement identifier that differentiates the statement from others in the same policy.<br/>FunctionName:  The name of the Lambda function, version, or alias. <p class="title"> <b>Name formats</b>  <ul> <li>  <b>Function name</b> - <code>my-function</code> (name-only), <code>my-function:v1</code> (with alias). </li> <li>  <b>Function ARN</b> - <code>arn:aws:lambda:us-west-2:123456789012:function:my-function</code>. </li> <li>  <b>Partial ARN</b> - <code>123456789012:function:my-function</code>. </li> </ul> You can append a version number or alias to any of the formats. The length constraint applies only to the full ARN. If you specify only the function name, it is limited to 64 characters in length.<br/>
        :return: Statement:  The permission statement that's added to the function policy.<br/>        """
        pass

    def create_alias(self, FunctionVersion: str, Name: str, FunctionName: str, Description: str=None, RoutingConfig: AliasRoutingConfiguration=None) -> AliasConfiguration:
        """Creates an <a href="https://docs.aws.amazon.com/lambda/latest/dg/versioning-aliases.html">alias</a> for a Lambda function version. Use aliases to provide clients with a function identifier that you can update to invoke a different version. You can also map an alias to split invocation requests between two versions. Use the <code>RoutingConfig</code> parameter to specify a second version and the percentage of invocation requests that it receives.

        :param self:
        :param RoutingConfig:  The <a href="https://docs.aws.amazon.com/lambda/latest/dg/lambda-traffic-shifting-using-aliases.html">routing configuration</a> of the alias.:<br/>Description:  A description of the alias.<br/>FunctionVersion:  The function version that the alias invokes.<br/>Name:  The name of the alias.<br/>FunctionName:  The name of the Lambda function. <p class="title"> <b>Name formats</b>  <ul> <li>  <b>Function name</b> - <code>MyFunction</code>. </li> <li>  <b>Function ARN</b> - <code>arn:aws:lambda:us-west-2:123456789012:function:MyFunction</code>. </li> <li>  <b>Partial ARN</b> - <code>123456789012:function:MyFunction</code>. </li> </ul> The length constraint applies only to the full ARN. If you specify only the function name, it is limited to 64 characters in length.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdditionalVersionWeights</i> The name of the second alias, and the percentage of traffic that's routed to it.<br/>
        :return: RevisionId:  A unique identifier that changes when you update the alias.<br/>RoutingConfig:  The <a href="https://docs.aws.amazon.com/lambda/latest/dg/lambda-traffic-shifting-using-aliases.html">routing configuration</a> of the alias.:<br/>Description:  A description of the alias.<br/>FunctionVersion:  The function version that the alias invokes.<br/>Name:  The name of the alias.<br/>AliasArn:  The Amazon Resource Name (ARN) of the alias.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdditionalVersionWeights</i> The name of the second alias, and the percentage of traffic that's routed to it.<br/>        """
        pass

    def create_event_source_mapping(self, FunctionName: str, EventSourceArn: str, Enabled: Enabled=None, BatchSize: int=None, StartingPosition: str=None, StartingPositionTimestamp: Date=None) -> EventSourceMappingConfiguration:
        """Creates a mapping between an event source and an AWS Lambda function. Lambda reads items from the event source and triggers the function. For details about each event source type, see the following topics. <ul> <li>  <a href="https://docs.aws.amazon.com/lambda/latest/dg/with-kinesis.html">Using AWS Lambda with Amazon Kinesis</a>  </li> <li>  <a href="https://docs.aws.amazon.com/lambda/latest/dg/with-sqs.html">Using AWS Lambda with Amazon SQS</a>  </li> <li>  <a href="https://docs.aws.amazon.com/lambda/latest/dg/with-ddb.html">Using AWS Lambda with Amazon DynamoDB</a>  </li> </ul>

        :param self:
        :param StartingPositionTimestamp:  With <code>StartingPosition</code> set to <code>AT_TIMESTAMP</code>, the time from which to start reading.<br/>StartingPosition:  The position in a stream from which to start reading. Required for Amazon Kinesis and Amazon DynamoDB Streams sources. <code>AT_TIMESTAMP</code> is only supported for Amazon Kinesis streams.<br/>BatchSize:  The maximum number of items to retrieve in a single batch. <ul> <li>  <b>Amazon Kinesis</b> - Default 100. Max 10,000. </li> <li>  <b>Amazon DynamoDB Streams</b> - Default 100. Max 1,000. </li> <li>  <b>Amazon Simple Queue Service</b> - Default 10. Max 10. </li> </ul><br/>Enabled:  Disables the event source mapping to pause polling and invocation.<br/>FunctionName:  The name of the Lambda function. <p class="title"> <b>Name formats</b>  <ul> <li>  <b>Function name</b> - <code>MyFunction</code>. </li> <li>  <b>Function ARN</b> - <code>arn:aws:lambda:us-west-2:123456789012:function:MyFunction</code>. </li> <li>  <b>Version or Alias ARN</b> - <code>arn:aws:lambda:us-west-2:123456789012:function:MyFunction:PROD</code>. </li> <li>  <b>Partial ARN</b> - <code>123456789012:function:MyFunction</code>. </li> </ul> The length constraint applies only to the full ARN. If you specify only the function name, it's limited to 64 characters in length.<br/>EventSourceArn:  The Amazon Resource Name (ARN) of the event source. <ul> <li>  <b>Amazon Kinesis</b> - The ARN of the data stream or a stream consumer. </li> <li>  <b>Amazon DynamoDB Streams</b> - The ARN of the stream. </li> <li>  <b>Amazon Simple Queue Service</b> - The ARN of the queue. </li> </ul><br/>
        :return: StateTransitionReason:  The cause of the last state change, either <code>User initiated</code> or <code>Lambda initiated</code>.<br/>State:  The state of the event source mapping. It can be one of the following: <code>Creating</code>, <code>Enabling</code>, <code>Enabled</code>, <code>Disabling</code>, <code>Disabled</code>, <code>Updating</code>, or <code>Deleting</code>.<br/>LastProcessingResult:  The result of the last AWS Lambda invocation of your Lambda function.<br/>LastModified:  The date that the event source mapping was last updated.<br/>FunctionArn:  The ARN of the Lambda function.<br/>EventSourceArn:  The Amazon Resource Name (ARN) of the event source.<br/>BatchSize:  The maximum number of items to retrieve in a single batch.<br/>UUID:  The identifier of the event source mapping.<br/>        """
        pass

    def create_function(self, Code: FunctionCode, Handler: str, Role: str, Runtime: str, FunctionName: str, Description: str=None, Timeout: int=None, MemorySize: int=None, Publish: Boolean=None, VpcConfig: VpcConfig=None, DeadLetterConfig: DeadLetterConfig=None, Environment: Environment=None, KMSKeyArn: str=None, TracingConfig: TracingConfig=None, Tags: Tags=None, Layers: List[LayerVersionArn]=None) -> FunctionConfiguration:
        """Creates a Lambda function. To create a function, you need a <a href="https://docs.aws.amazon.com/lambda/latest/dg/deployment-package-v2.html">deployment package</a> and an <a href="https://docs.aws.amazon.com/lambda/latest/dg/intro-permission-model.html#lambda-intro-execution-role">execution role</a>. The deployment package contains your function code. The execution role grants the function permission to use AWS services, such as Amazon CloudWatch Logs for log streaming and AWS X-Ray for request tracing. A function has an unpublished version, and can have published versions and aliases. The unpublished version changes when you update your function's code and configuration. A published version is a snapshot of your function code and configuration that can't be changed. An alias is a named resource that maps to a version, and can be changed to map to a different version. Use the <code>Publish</code> parameter to create version <code>1</code> of your function from its initial configuration. The other parameters let you configure version-specific and function-level settings. You can modify version-specific settings later with <a>UpdateFunctionConfiguration</a>. Function-level settings apply to both the unpublished and published versions of the function, and include tags (<a>TagResource</a>) and per-function concurrency limits (<a>PutFunctionConcurrency</a>). If another account or an AWS service invokes your function, use <a>AddPermission</a> to grant permission by creating a resource-based IAM policy. You can grant permissions at the function level, on a version, or on an alias. To invoke your function directly, use <a>Invoke</a>. To invoke your function in response to events in other AWS services, create an event source mapping (<a>CreateEventSourceMapping</a>), or configure a function trigger in the other service. For more information, see <a href="https://docs.aws.amazon.com/lambda/latest/dg/invoking-lambda-functions.html">Invoking Functions</a>.

        :param self:
        :param Layers:  A list of <a href="https://docs.aws.amazon.com/lambda/latest/dg/configuration-layers.html">function layers</a> to add to the function's execution environment. Specify each layer by its ARN, including the version.<br/>Tags:  A list of <a href="https://docs.aws.amazon.com/lambda/latest/dg/tagging.html">tags</a> to apply to the function.<br/>TracingConfig:  Set <code>Mode</code> to <code>Active</code> to sample and trace a subset of incoming requests with AWS X-Ray.:<br/>KMSKeyArn:  The ARN of the AWS Key Management Service (AWS KMS) key that's used to encrypt your function's environment variables. If it's not provided, AWS Lambda uses a default service key.<br/>Environment:  Environment variables that are accessible from function code during execution.:<br/>DeadLetterConfig:  A dead letter queue configuration that specifies the queue or topic where Lambda sends asynchronous events when they fail processing. For more information, see <a href="https://docs.aws.amazon.com/lambda/latest/dg/dlq.html">Dead Letter Queues</a>.:<br/>VpcConfig:  For network connectivity to AWS resources in a VPC, specify a list of security groups and subnets in the VPC. When you connect a function to a VPC, it can only access resources and the internet through that VPC. For more information, see <a href="https://docs.aws.amazon.com/lambda/latest/dg/vpc.html">VPC Settings</a>.:<br/>Publish:  Set to true to publish the first version of the function during creation.<br/>MemorySize:  The amount of memory that your function has access to. Increasing the function's memory also increases its CPU allocation. The default value is 128 MB. The value must be a multiple of 64 MB.<br/>Timeout:  The amount of time that Lambda allows a function to run before stopping it. The default is 3 seconds. The maximum allowed value is 900 seconds.<br/>Description:  A description of the function.<br/>Code:  The code for the function.:<br/>Handler:  The name of the method within your code that Lambda calls to execute your function. The format includes the file name. It can also include namespaces and other qualifiers, depending on the runtime. For more information, see <a href="https://docs.aws.amazon.com/lambda/latest/dg/programming-model-v2.html">Programming Model</a>.<br/>Role:  The Amazon Resource Name (ARN) of the function's execution role.<br/>Runtime:  The identifier of the function's <a href="https://docs.aws.amazon.com/lambda/latest/dg/lambda-runtimes.html">runtime</a>.<br/>FunctionName:  The name of the Lambda function. <p class="title"> <b>Name formats</b>  <ul> <li>  <b>Function name</b> - <code>my-function</code>. </li> <li>  <b>Function ARN</b> - <code>arn:aws:lambda:us-west-2:123456789012:function:my-function</code>. </li> <li>  <b>Partial ARN</b> - <code>123456789012:function:my-function</code>. </li> </ul> The length constraint applies only to the full ARN. If you specify only the function name, it is limited to 64 characters in length.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3ObjectVersion</i> For versioned objects, the version of the deployment package object to use.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Key</i> The Amazon S3 key of the deployment package.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Bucket</i> An Amazon S3 bucket in the same AWS Region as your function. The bucket can be in a different AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ZipFile</i> The base64-encoded contents of the deployment package. AWS SDK and AWS CLI clients handle the encoding for you.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> A list of VPC security groups IDs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> A list of VPC subnet IDs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetArn</i> The Amazon Resource Name (ARN) of an Amazon SQS queue or Amazon SNS topic.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Variables</i> Environment variable key-value pairs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> The tracing mode.<br/>
        :return: Layers:  The function's <a href="https://docs.aws.amazon.com/lambda/latest/dg/configuration-layers.html"> layers</a>.<br/>RevisionId:  The latest updated revision of the function or alias.<br/>MasterArn:  For Lambda@Edge functions, the ARN of the master function.<br/>TracingConfig:  The function's AWS X-Ray tracing configuration.:<br/>KMSKeyArn:  The KMS key that's used to encrypt the function's environment variables. This key is only returned if you've configured a customer-managed CMK.<br/>Environment:  The function's environment variables.:<br/>DeadLetterConfig:  The function's dead letter queue.:<br/>VpcConfig:  The function's networking configuration.:<br/>Version:  The version of the Lambda function.<br/>CodeSha256:  The SHA256 hash of the function's deployment package.<br/>LastModified:  The date and time that the function was last updated, in <a href="https://www.w3.org/TR/NOTE-datetime">ISO-8601 format</a> (YYYY-MM-DDThh:mm:ss.sTZD).<br/>MemorySize:  The memory that's allocated to the function.<br/>Timeout:  The amount of time that Lambda allows a function to run before stopping it.<br/>Description:  The function's description.<br/>CodeSize:  The size of the function's deployment package, in bytes.<br/>Handler:  The function that Lambda calls to begin executing your function.<br/>Role:  The function's execution role.<br/>Runtime:  The runtime environment for the Lambda function.<br/>FunctionArn:  The function's Amazon Resource Name (ARN).<br/>FunctionName:  The name of the function.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> A list of VPC security groups IDs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> A list of VPC subnet IDs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetArn</i> The Amazon Resource Name (ARN) of an Amazon SQS queue or Amazon SNS topic.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Error</i> Error messages for environment variables that couldn't be applied.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Error</b>: Error messages for environment variables that couldn't be applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Variables</i> Environment variable key-value pairs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> The error message.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorCode</i> The error code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> The tracing mode.<br/>        """
        pass

    def delete_alias(self, Name: str, FunctionName: str):
        """Deletes a Lambda function <a href="https://docs.aws.amazon.com/lambda/latest/dg/versioning-aliases.html">alias</a>.

        :param self:
        :param Name:  The name of the alias.<br/>FunctionName:  The name of the Lambda function. <p class="title"> <b>Name formats</b>  <ul> <li>  <b>Function name</b> - <code>MyFunction</code>. </li> <li>  <b>Function ARN</b> - <code>arn:aws:lambda:us-west-2:123456789012:function:MyFunction</code>. </li> <li>  <b>Partial ARN</b> - <code>123456789012:function:MyFunction</code>. </li> </ul> The length constraint applies only to the full ARN. If you specify only the function name, it is limited to 64 characters in length.<br/>
        :return:         """
        pass

    def delete_event_source_mapping(self, UUID: str) -> EventSourceMappingConfiguration:
        """Deletes an <a href="https://docs.aws.amazon.com/lambda/latest/dg/intro-invocation-modes.html">event source mapping</a>. You can get the identifier of a mapping from the output of <a>ListEventSourceMappings</a>.

        :param self:
        :param UUID:  The identifier of the event source mapping.<br/>
        :return: StateTransitionReason:  The cause of the last state change, either <code>User initiated</code> or <code>Lambda initiated</code>.<br/>State:  The state of the event source mapping. It can be one of the following: <code>Creating</code>, <code>Enabling</code>, <code>Enabled</code>, <code>Disabling</code>, <code>Disabled</code>, <code>Updating</code>, or <code>Deleting</code>.<br/>LastProcessingResult:  The result of the last AWS Lambda invocation of your Lambda function.<br/>LastModified:  The date that the event source mapping was last updated.<br/>FunctionArn:  The ARN of the Lambda function.<br/>EventSourceArn:  The Amazon Resource Name (ARN) of the event source.<br/>BatchSize:  The maximum number of items to retrieve in a single batch.<br/>UUID:  The identifier of the event source mapping.<br/>        """
        pass

    def delete_function(self, FunctionName: str, Qualifier: str=None):
        """Deletes a Lambda function. To delete a specific function version, use the <code>Qualifier</code> parameter. Otherwise, all versions and aliases are deleted. To delete Lambda event source mappings that invoke a function, use <a>DeleteEventSourceMapping</a>. For AWS services and resources that invoke your function directly, delete the trigger in the service where you originally configured it.

        :param self:
        :param Qualifier:  Specify a version to delete. You can't delete a version that's referenced by an alias.<br/>FunctionName:  The name of the Lambda function or version. <p class="title"> <b>Name formats</b>  <ul> <li>  <b>Function name</b> - <code>my-function</code> (name-only), <code>my-function:1</code> (with version). </li> <li>  <b>Function ARN</b> - <code>arn:aws:lambda:us-west-2:123456789012:function:my-function</code>. </li> <li>  <b>Partial ARN</b> - <code>123456789012:function:my-function</code>. </li> </ul> You can append a version number or alias to any of the formats. The length constraint applies only to the full ARN. If you specify only the function name, it is limited to 64 characters in length.<br/>
        :return:         """
        pass

    def delete_function_concurrency(self, FunctionName: str):
        """Removes a concurrent execution limit from a function.

        :param self:
        :param FunctionName:  The name of the Lambda function. <p class="title"> <b>Name formats</b>  <ul> <li>  <b>Function name</b> - <code>my-function</code>. </li> <li>  <b>Function ARN</b> - <code>arn:aws:lambda:us-west-2:123456789012:function:my-function</code>. </li> <li>  <b>Partial ARN</b> - <code>123456789012:function:my-function</code>. </li> </ul> The length constraint applies only to the full ARN. If you specify only the function name, it is limited to 64 characters in length.<br/>
        :return:         """
        pass

    def delete_layer_version(self, VersionNumber: LayerVersionNumber, LayerName: str):
        """Deletes a version of an <a href="https://docs.aws.amazon.com/lambda/latest/dg/configuration-layers.html">AWS Lambda layer</a>. Deleted versions can no longer be viewed or added to functions. To avoid breaking functions, a copy of the version remains in Lambda until no functions refer to it.

        :param self:
        :param VersionNumber:  The version number.<br/>LayerName:  The name or Amazon Resource Name (ARN) of the layer.<br/>
        :return:         """
        pass

    def get_account_settings(self) -> GetAccountSettingsResponse:
        """Retrieves details about your account's <a href="https://docs.aws.amazon.com/lambda/latest/dg/limits.html">limits</a> and usage in an AWS Region.

        :param self:
        :param 
        :return: AccountUsage:  The number of functions and amount of storage in use.:<br/>AccountLimit:  Limits that are related to concurrency and code storage.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UnreservedConcurrentExecutions</i> The maximum number of simultaneous function executions, minus the capacity that's reserved for individual functions with <a>PutFunctionConcurrency</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConcurrentExecutions</i> The maximum number of simultaneous function executions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CodeSizeZipped</i> The maximum size of a deployment package when it's uploaded directly to AWS Lambda. Use Amazon S3 for larger files.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CodeSizeUnzipped</i> The maximum size of your function's code and layers when they're extracted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalCodeSize</i> The amount of storage space that you can use for all deployment packages and layer archives.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FunctionCount</i> The number of Lambda functions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalCodeSize</i> The amount of storage space, in bytes, that's being used by deployment packages and layer archives.<br/>        """
        pass

    def get_alias(self, Name: str, FunctionName: str) -> AliasConfiguration:
        """Returns details about a Lambda function <a href="https://docs.aws.amazon.com/lambda/latest/dg/versioning-aliases.html">alias</a>.

        :param self:
        :param Name:  The name of the alias.<br/>FunctionName:  The name of the Lambda function. <p class="title"> <b>Name formats</b>  <ul> <li>  <b>Function name</b> - <code>MyFunction</code>. </li> <li>  <b>Function ARN</b> - <code>arn:aws:lambda:us-west-2:123456789012:function:MyFunction</code>. </li> <li>  <b>Partial ARN</b> - <code>123456789012:function:MyFunction</code>. </li> </ul> The length constraint applies only to the full ARN. If you specify only the function name, it is limited to 64 characters in length.<br/>
        :return: RevisionId:  A unique identifier that changes when you update the alias.<br/>RoutingConfig:  The <a href="https://docs.aws.amazon.com/lambda/latest/dg/lambda-traffic-shifting-using-aliases.html">routing configuration</a> of the alias.:<br/>Description:  A description of the alias.<br/>FunctionVersion:  The function version that the alias invokes.<br/>Name:  The name of the alias.<br/>AliasArn:  The Amazon Resource Name (ARN) of the alias.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdditionalVersionWeights</i> The name of the second alias, and the percentage of traffic that's routed to it.<br/>        """
        pass

    def get_event_source_mapping(self, UUID: str) -> EventSourceMappingConfiguration:
        """Returns details about an event source mapping. You can get the identifier of a mapping from the output of <a>ListEventSourceMappings</a>.

        :param self:
        :param UUID:  The identifier of the event source mapping.<br/>
        :return: StateTransitionReason:  The cause of the last state change, either <code>User initiated</code> or <code>Lambda initiated</code>.<br/>State:  The state of the event source mapping. It can be one of the following: <code>Creating</code>, <code>Enabling</code>, <code>Enabled</code>, <code>Disabling</code>, <code>Disabled</code>, <code>Updating</code>, or <code>Deleting</code>.<br/>LastProcessingResult:  The result of the last AWS Lambda invocation of your Lambda function.<br/>LastModified:  The date that the event source mapping was last updated.<br/>FunctionArn:  The ARN of the Lambda function.<br/>EventSourceArn:  The Amazon Resource Name (ARN) of the event source.<br/>BatchSize:  The maximum number of items to retrieve in a single batch.<br/>UUID:  The identifier of the event source mapping.<br/>        """
        pass

    def get_function(self, FunctionName: str, Qualifier: str=None) -> GetFunctionResponse:
        """Returns information about the function or function version, with a link to download the deployment package that's valid for 10 minutes. If you specify a function version, only details that are specific to that version are returned.

        :param self:
        :param Qualifier:  Specify a version or alias to get details about a published version of the function.<br/>FunctionName:  The name of the Lambda function, version, or alias. <p class="title"> <b>Name formats</b>  <ul> <li>  <b>Function name</b> - <code>my-function</code> (name-only), <code>my-function:v1</code> (with alias). </li> <li>  <b>Function ARN</b> - <code>arn:aws:lambda:us-west-2:123456789012:function:my-function</code>. </li> <li>  <b>Partial ARN</b> - <code>123456789012:function:my-function</code>. </li> </ul> You can append a version number or alias to any of the formats. The length constraint applies only to the full ARN. If you specify only the function name, it is limited to 64 characters in length.<br/>
        :return: Concurrency:  The function's <a href="https://docs.aws.amazon.com/lambda/latest/dg/concurrent-executions.html">reserved concurrency</a>.:<br/>Tags:  The function's <a href="https://docs.aws.amazon.com/lambda/latest/dg/tagging.html">tags</a>.<br/>Code:  The deployment package of the function or version.:<br/>Configuration:  The configuration of the function or version.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Layers</i> The function's <a href="https://docs.aws.amazon.com/lambda/latest/dg/configuration-layers.html"> layers</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RevisionId</i> The latest updated revision of the function or alias.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterArn</i> For Lambda@Edge functions, the ARN of the master function.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TracingConfig</i> The function's AWS X-Ray tracing configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TracingConfig</b>: The function's AWS X-Ray tracing configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KMSKeyArn</i> The KMS key that's used to encrypt the function's environment variables. This key is only returned if you've configured a customer-managed CMK.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Environment</i> The function's environment variables.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Environment</b>: The function's environment variables.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeadLetterConfig</i> The function's dead letter queue.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DeadLetterConfig</b>: The function's dead letter queue.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcConfig</i> The function's networking configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VpcConfig</b>: The function's networking configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version of the Lambda function.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CodeSha256</i> The SHA256 hash of the function's deployment package.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModified</i> The date and time that the function was last updated, in <a href="https://www.w3.org/TR/NOTE-datetime">ISO-8601 format</a> (YYYY-MM-DDThh:mm:ss.sTZD).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MemorySize</i> The memory that's allocated to the function.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timeout</i> The amount of time that Lambda allows a function to run before stopping it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The function's description.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CodeSize</i> The size of the function's deployment package, in bytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Handler</i> The function that Lambda calls to begin executing your function.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Role</i> The function's execution role.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Runtime</i> The runtime environment for the Lambda function.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FunctionArn</i> The function's Amazon Resource Name (ARN).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FunctionName</i> The name of the function.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> A list of VPC security groups IDs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> A list of VPC subnet IDs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetArn</i> The Amazon Resource Name (ARN) of an Amazon SQS queue or Amazon SNS topic.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Variables</i> Environment variable key-value pairs.<br/>Error messages for environment variables that couldn't be applied.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Error</b>: Error messages for environment variables that couldn't be applied.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> The tracing mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Location</i> A presigned URL that you can use to download the deployment package.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RepositoryType</i> The service that's hosting the file.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservedConcurrentExecutions</i> The number of concurrent executions that are reserved for this function. For more information, see <a href="https://docs.aws.amazon.com/lambda/latest/dg/concurrent-executions.html">Managing Concurrency</a>.<br/>        """
        pass

    def get_function_configuration(self, FunctionName: str, Qualifier: str=None) -> FunctionConfiguration:
        """Returns the version-specific settings of a Lambda function or version. The output includes only options that can vary between versions of a function. To modify these settings, use <a>UpdateFunctionConfiguration</a>. To get all of a function's details, including function-level settings, use <a>GetFunction</a>.

        :param self:
        :param Qualifier:  Specify a version or alias to get details about a published version of the function.<br/>FunctionName:  The name of the Lambda function, version, or alias. <p class="title"> <b>Name formats</b>  <ul> <li>  <b>Function name</b> - <code>my-function</code> (name-only), <code>my-function:v1</code> (with alias). </li> <li>  <b>Function ARN</b> - <code>arn:aws:lambda:us-west-2:123456789012:function:my-function</code>. </li> <li>  <b>Partial ARN</b> - <code>123456789012:function:my-function</code>. </li> </ul> You can append a version number or alias to any of the formats. The length constraint applies only to the full ARN. If you specify only the function name, it is limited to 64 characters in length.<br/>
        :return: Layers:  The function's <a href="https://docs.aws.amazon.com/lambda/latest/dg/configuration-layers.html"> layers</a>.<br/>RevisionId:  The latest updated revision of the function or alias.<br/>MasterArn:  For Lambda@Edge functions, the ARN of the master function.<br/>TracingConfig:  The function's AWS X-Ray tracing configuration.:<br/>KMSKeyArn:  The KMS key that's used to encrypt the function's environment variables. This key is only returned if you've configured a customer-managed CMK.<br/>Environment:  The function's environment variables.:<br/>DeadLetterConfig:  The function's dead letter queue.:<br/>VpcConfig:  The function's networking configuration.:<br/>Version:  The version of the Lambda function.<br/>CodeSha256:  The SHA256 hash of the function's deployment package.<br/>LastModified:  The date and time that the function was last updated, in <a href="https://www.w3.org/TR/NOTE-datetime">ISO-8601 format</a> (YYYY-MM-DDThh:mm:ss.sTZD).<br/>MemorySize:  The memory that's allocated to the function.<br/>Timeout:  The amount of time that Lambda allows a function to run before stopping it.<br/>Description:  The function's description.<br/>CodeSize:  The size of the function's deployment package, in bytes.<br/>Handler:  The function that Lambda calls to begin executing your function.<br/>Role:  The function's execution role.<br/>Runtime:  The runtime environment for the Lambda function.<br/>FunctionArn:  The function's Amazon Resource Name (ARN).<br/>FunctionName:  The name of the function.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> A list of VPC security groups IDs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> A list of VPC subnet IDs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetArn</i> The Amazon Resource Name (ARN) of an Amazon SQS queue or Amazon SNS topic.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Error</i> Error messages for environment variables that couldn't be applied.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Error</b>: Error messages for environment variables that couldn't be applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Variables</i> Environment variable key-value pairs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> The error message.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorCode</i> The error code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> The tracing mode.<br/>        """
        pass

    def get_layer_version(self, VersionNumber: LayerVersionNumber, LayerName: str) -> GetLayerVersionResponse:
        """Returns information about a version of an <a href="https://docs.aws.amazon.com/lambda/latest/dg/configuration-layers.html">AWS Lambda layer</a>, with a link to download the layer archive that's valid for 10 minutes.

        :param self:
        :param VersionNumber:  The version number.<br/>LayerName:  The name or Amazon Resource Name (ARN) of the layer.<br/>
        :return: LicenseInfo:  The layer's software license.<br/>CompatibleRuntimes:  The layer's compatible runtimes.<br/>Version:  The version number.<br/>CreatedDate:  The date that the layer version was created, in <a href="https://www.w3.org/TR/NOTE-datetime">ISO-8601 format</a> (YYYY-MM-DDThh:mm:ss.sTZD).<br/>Description:  The description of the version.<br/>LayerVersionArn:  The ARN of the layer version.<br/>LayerArn:  The ARN of the layer.<br/>Content:  Details about the layer version.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CodeSize</i> The size of the layer archive in bytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CodeSha256</i> The SHA-256 hash of the layer archive.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Location</i> A link to the layer archive in Amazon S3 that is valid for 10 minutes.<br/>        """
        pass

    def get_layer_version_by_arn(self, Arn: str) -> GetLayerVersionResponse:
        """Returns information about a version of an <a href="https://docs.aws.amazon.com/lambda/latest/dg/configuration-layers.html">AWS Lambda layer</a>, with a link to download the layer archive that's valid for 10 minutes.

        :param self:
        :param Arn:  The ARN of the layer version.<br/>
        :return: LicenseInfo:  The layer's software license.<br/>CompatibleRuntimes:  The layer's compatible runtimes.<br/>Version:  The version number.<br/>CreatedDate:  The date that the layer version was created, in <a href="https://www.w3.org/TR/NOTE-datetime">ISO-8601 format</a> (YYYY-MM-DDThh:mm:ss.sTZD).<br/>Description:  The description of the version.<br/>LayerVersionArn:  The ARN of the layer version.<br/>LayerArn:  The ARN of the layer.<br/>Content:  Details about the layer version.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CodeSize</i> The size of the layer archive in bytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CodeSha256</i> The SHA-256 hash of the layer archive.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Location</i> A link to the layer archive in Amazon S3 that is valid for 10 minutes.<br/>        """
        pass

    def get_layer_version_policy(self, VersionNumber: LayerVersionNumber, LayerName: str) -> GetLayerVersionPolicyResponse:
        """Returns the permission policy for a version of an <a href="https://docs.aws.amazon.com/lambda/latest/dg/configuration-layers.html">AWS Lambda layer</a>. For more information, see <a>AddLayerVersionPermission</a>.

        :param self:
        :param VersionNumber:  The version number.<br/>LayerName:  The name or Amazon Resource Name (ARN) of the layer.<br/>
        :return: RevisionId:  A unique identifier for the current revision of the policy.<br/>Policy:  The policy document.<br/>        """
        pass

    def get_policy(self, FunctionName: str, Qualifier: str=None) -> GetPolicyResponse:
        """Returns the <a href="https://docs.aws.amazon.com/lambda/latest/dg/access-control-resource-based.html">resource-based IAM policy</a> for a function, version, or alias.

        :param self:
        :param Qualifier:  Specify a version or alias to get the policy for that resource.<br/>FunctionName:  The name of the Lambda function, version, or alias. <p class="title"> <b>Name formats</b>  <ul> <li>  <b>Function name</b> - <code>my-function</code> (name-only), <code>my-function:v1</code> (with alias). </li> <li>  <b>Function ARN</b> - <code>arn:aws:lambda:us-west-2:123456789012:function:my-function</code>. </li> <li>  <b>Partial ARN</b> - <code>123456789012:function:my-function</code>. </li> </ul> You can append a version number or alias to any of the formats. The length constraint applies only to the full ARN. If you specify only the function name, it is limited to 64 characters in length.<br/>
        :return: RevisionId:  A unique identifier for the current revision of the policy.<br/>Policy:  The resource-based policy.<br/>        """
        pass

    def invoke(self, FunctionName: str, InvocationType: str=None, LogType: str=None, ClientContext: str=None, Payload: Blob=None, Qualifier: str=None) -> InvocationResponse:
        """Invokes a Lambda function. You can invoke a function synchronously (and wait for the response), or asynchronously. To invoke a function asynchronously, set <code>InvocationType</code> to <code>Event</code>. For synchronous invocation, details about the function response, including errors, are included in the response body and headers. For either invocation type, you can find more information in the <a href="https://docs.aws.amazon.com/lambda/latest/dg/monitoring-functions.html">execution log</a> and <a href="https://docs.aws.amazon.com/lambda/latest/dg/dlq.html">trace</a>. To record function errors for asynchronous invocations, configure your function with a <a href="https://docs.aws.amazon.com/lambda/latest/dg/dlq.html">dead letter queue</a>. When an error occurs, your function may be invoked multiple times. Retry behavior varies by error type, client, event source, and invocation type. For example, if you invoke a function asynchronously and it returns an error, Lambda executes the function up to two more times. For more information, see <a href="https://docs.aws.amazon.com/lambda/latest/dg/retries-on-errors.html">Retry Behavior</a>. The status code in the API response doesn't reflect function errors. Error codes are reserved for errors that prevent your function from executing, such as permissions errors, <a href="https://docs.aws.amazon.com/lambda/latest/dg/limits.html">limit errors</a>, or issues with your function's code and configuration. For example, Lambda returns <code>TooManyRequestsException</code> if executing the function would cause you to exceed a concurrency limit at either the account level (<code>ConcurrentInvocationLimitExceeded</code>) or function level (<code>ReservedFunctionConcurrentInvocationLimitExceeded</code>). For functions with a long timeout, your client might be disconnected during synchronous invocation while it waits for a response. Configure your HTTP client, SDK, firewall, proxy, or operating system to allow for long connections with timeout or keep-alive settings. This operation requires permission for the <code>lambda:InvokeFunction</code> action.

        :param self:
        :param Qualifier:  Specify a version or alias to invoke a published version of the function.<br/>Payload:  The JSON that you want to provide to your Lambda function as input.<br/>ClientContext:  Up to 3583 bytes of base64-encoded data about the invoking client to pass to the function in the context object.<br/>LogType:  Set to <code>Tail</code> to include the execution log in the response.<br/>InvocationType:  Choose from the following options. <ul> <li>  <code>RequestResponse</code> (default) - Invoke the function synchronously. Keep the connection open until the function returns a response or times out. The API response includes the function response and additional data. </li> <li>  <code>Event</code> - Invoke the function asynchronously. Send events that fail multiple times to the function's dead-letter queue (if it's configured). The API response only includes a status code. </li> <li>  <code>DryRun</code> - Validate parameter values and verify that the user or role has permission to invoke the function. </li> </ul><br/>FunctionName:  The name of the Lambda function, version, or alias. <p class="title"> <b>Name formats</b>  <ul> <li>  <b>Function name</b> - <code>my-function</code> (name-only), <code>my-function:v1</code> (with alias). </li> <li>  <b>Function ARN</b> - <code>arn:aws:lambda:us-west-2:123456789012:function:my-function</code>. </li> <li>  <b>Partial ARN</b> - <code>123456789012:function:my-function</code>. </li> </ul> You can append a version number or alias to any of the formats. The length constraint applies only to the full ARN. If you specify only the function name, it is limited to 64 characters in length.<br/>
        :return: ExecutedVersion:  The version of the function that executed. When you invoke a function with an alias, this indicates which version the alias resolved to.<br/>Payload:  The response from the function, or an error object.<br/>LogResult:  The last 4 KB of the execution log, which is base64 encoded.<br/>FunctionError:  If present, indicates that an error occurred during function execution. Details about the error are included in the response payload. <ul> <li>  <code>Handled</code> - The runtime caught an error thrown by the function and formatted it into a JSON document. </li> <li>  <code>Unhandled</code> - The runtime didn't handle the error. For example, the function ran out of memory or timed out. </li> </ul><br/>StatusCode:  The HTTP status code is in the 200 range for a successful request. For the <code>RequestResponse</code> invocation type, this status code is 200. For the <code>Event</code> invocation type, this status code is 202. For the <code>DryRun</code> invocation type, the status code is 204.<br/>        """
        pass

    def invoke_async(self, InvokeArgs: BlobStream, FunctionName: str) -> InvokeAsyncResponse:
        """<important> For asynchronous function invocation, use <a>Invoke</a>. </important> Invokes a function asynchronously.

        :param self:
        :param InvokeArgs:  The JSON that you want to provide to your Lambda function as input.<br/>FunctionName:  The name of the Lambda function. <p class="title"> <b>Name formats</b>  <ul> <li>  <b>Function name</b> - <code>my-function</code>. </li> <li>  <b>Function ARN</b> - <code>arn:aws:lambda:us-west-2:123456789012:function:my-function</code>. </li> <li>  <b>Partial ARN</b> - <code>123456789012:function:my-function</code>. </li> </ul> The length constraint applies only to the full ARN. If you specify only the function name, it is limited to 64 characters in length.<br/>
        :return: Status:  The status code.<br/>        """
        pass

    def list_aliases(self, FunctionName: str, FunctionVersion: str=None, Marker: str=None, MaxItems: int=None) -> ListAliasesResponse:
        """Returns a list of <a href="https://docs.aws.amazon.com/lambda/latest/dg/versioning-aliases.html">aliases</a> for a Lambda function.

        :param self:
        :param MaxItems:  Limit the number of aliases returned.<br/>Marker:  Specify the pagination token that's returned by a previous request to retrieve the next page of results.<br/>FunctionVersion:  Specify a function version to only list aliases that invoke that version.<br/>FunctionName:  The name of the Lambda function. <p class="title"> <b>Name formats</b>  <ul> <li>  <b>Function name</b> - <code>MyFunction</code>. </li> <li>  <b>Function ARN</b> - <code>arn:aws:lambda:us-west-2:123456789012:function:MyFunction</code>. </li> <li>  <b>Partial ARN</b> - <code>123456789012:function:MyFunction</code>. </li> </ul> The length constraint applies only to the full ARN. If you specify only the function name, it is limited to 64 characters in length.<br/>
        :return: Aliases:  A list of aliases.<br/>NextMarker:  The pagination token that's included if more results are available.<br/>        """
        pass

    def list_event_source_mappings(self, EventSourceArn: str=None, FunctionName: str=None, Marker: str=None, MaxItems: int=None) -> ListEventSourceMappingsResponse:
        """Lists event source mappings. Specify an <code>EventSourceArn</code> to only show event source mappings for a single event source.

        :param self:
        :param MaxItems:  The maximum number of event source mappings to return.<br/>Marker:  A pagination token returned by a previous call.<br/>FunctionName:  The name of the Lambda function. <p class="title"> <b>Name formats</b>  <ul> <li>  <b>Function name</b> - <code>MyFunction</code>. </li> <li>  <b>Function ARN</b> - <code>arn:aws:lambda:us-west-2:123456789012:function:MyFunction</code>. </li> <li>  <b>Version or Alias ARN</b> - <code>arn:aws:lambda:us-west-2:123456789012:function:MyFunction:PROD</code>. </li> <li>  <b>Partial ARN</b> - <code>123456789012:function:MyFunction</code>. </li> </ul> The length constraint applies only to the full ARN. If you specify only the function name, it's limited to 64 characters in length.<br/>EventSourceArn:  The Amazon Resource Name (ARN) of the event source. <ul> <li>  <b>Amazon Kinesis</b> - The ARN of the data stream or a stream consumer. </li> <li>  <b>Amazon DynamoDB Streams</b> - The ARN of the stream. </li> <li>  <b>Amazon Simple Queue Service</b> - The ARN of the queue. </li> </ul><br/>
        :return: EventSourceMappings:  A list of event source mappings.<br/>NextMarker:  A pagination token that's returned when the response doesn't contain all event source mappings.<br/>        """
        pass

    def list_functions(self, MasterRegion: str=None, FunctionVersion: str=None, Marker: str=None, MaxItems: int=None) -> ListFunctionsResponse:
        """Returns a list of Lambda functions, with the version-specific configuration of each. Set <code>FunctionVersion</code> to <code>ALL</code> to include all published versions of each function in addition to the unpublished version. To get more information about a function or version, use <a>GetFunction</a>.

        :param self:
        :param MaxItems:  Specify a value between 1 and 50 to limit the number of functions in the response.<br/>Marker:  Specify the pagination token that's returned by a previous request to retrieve the next page of results.<br/>FunctionVersion:  Set to <code>ALL</code> to include entries for all published versions of each function.<br/>MasterRegion:  For Lambda@Edge functions, the AWS Region of the master function. For example, <code>us-east-2</code> or <code>ALL</code>. If specified, you must set <code>FunctionVersion</code> to <code>ALL</code>.<br/>
        :return: Functions:  A list of Lambda functions.<br/>NextMarker:  The pagination token that's included if more results are available.<br/>        """
        pass

    def list_layer_versions(self, LayerName: str, CompatibleRuntime: str=None, Marker: str=None, MaxItems: int=None) -> ListLayerVersionsResponse:
        """Lists the versions of an <a href="https://docs.aws.amazon.com/lambda/latest/dg/configuration-layers.html">AWS Lambda layer</a>. Versions that have been deleted aren't listed. Specify a <a href="https://docs.aws.amazon.com/lambda/latest/dg/lambda-runtimes.html">runtime identifier</a> to list only versions that indicate that they're compatible with that runtime.

        :param self:
        :param MaxItems:  The maximum number of versions to return.<br/>Marker:  A pagination token returned by a previous call.<br/>LayerName:  The name or Amazon Resource Name (ARN) of the layer.<br/>CompatibleRuntime:  A runtime identifier. For example, <code>go1.x</code>.<br/>
        :return: LayerVersions:  A list of versions.<br/>NextMarker:  A pagination token returned when the response doesn't contain all versions.<br/>        """
        pass

    def list_layers(self, CompatibleRuntime: str=None, Marker: str=None, MaxItems: int=None) -> ListLayersResponse:
        """Lists <a href="https://docs.aws.amazon.com/lambda/latest/dg/configuration-layers.html">AWS Lambda layers</a> and shows information about the latest version of each. Specify a <a href="https://docs.aws.amazon.com/lambda/latest/dg/lambda-runtimes.html">runtime identifier</a> to list only layers that indicate that they're compatible with that runtime.

        :param self:
        :param MaxItems:  The maximum number of layers to return.<br/>Marker:  A pagination token returned by a previous call.<br/>CompatibleRuntime:  A runtime identifier. For example, <code>go1.x</code>.<br/>
        :return: Layers:  A list of function layers.<br/>NextMarker:  A pagination token returned when the response doesn't contain all layers.<br/>        """
        pass

    def list_tags(self, Resource: str) -> ListTagsResponse:
        """Returns a function's <a href="https://docs.aws.amazon.com/lambda/latest/dg/tagging.html">tags</a>. You can also view tags with <a>GetFunction</a>.

        :param self:
        :param Resource:  The function's Amazon Resource Name (ARN).<br/>
        :return: Tags:  The function's tags.<br/>        """
        pass

    def list_versions_by_function(self, FunctionName: str, Marker: str=None, MaxItems: int=None) -> ListVersionsByFunctionResponse:
        """Returns a list of <a href="https://docs.aws.amazon.com/lambda/latest/dg/versioning-aliases.html">versions</a>, with the version-specific configuration of each. 

        :param self:
        :param MaxItems:  Limit the number of versions that are returned.<br/>Marker:  Specify the pagination token that's returned by a previous request to retrieve the next page of results.<br/>FunctionName:  The name of the Lambda function. <p class="title"> <b>Name formats</b>  <ul> <li>  <b>Function name</b> - <code>MyFunction</code>. </li> <li>  <b>Function ARN</b> - <code>arn:aws:lambda:us-west-2:123456789012:function:MyFunction</code>. </li> <li>  <b>Partial ARN</b> - <code>123456789012:function:MyFunction</code>. </li> </ul> The length constraint applies only to the full ARN. If you specify only the function name, it is limited to 64 characters in length.<br/>
        :return: Versions:  A list of Lambda function versions.<br/>NextMarker:  The pagination token that's included if more results are available.<br/>        """
        pass

    def publish_layer_version(self, Content: LayerVersionContentInput, LayerName: str, Description: str=None, CompatibleRuntimes: List[Runtime]=None, LicenseInfo: str=None) -> PublishLayerVersionResponse:
        """Creates an <a href="https://docs.aws.amazon.com/lambda/latest/dg/configuration-layers.html">AWS Lambda layer</a> from a ZIP archive. Each time you call <code>PublishLayerVersion</code> with the same version name, a new version is created. Add layers to your function with <a>CreateFunction</a> or <a>UpdateFunctionConfiguration</a>.

        :param self:
        :param LicenseInfo:  The layer's software license. It can be any of the following: <ul> <li> An <a href="https://spdx.org/licenses/">SPDX license identifier</a>. For example, <code>MIT</code>. </li> <li> The URL of a license hosted on the internet. For example, <code>https://opensource.org/licenses/MIT</code>. </li> <li> The full text of the license. </li> </ul><br/>CompatibleRuntimes:  A list of compatible <a href="https://docs.aws.amazon.com/lambda/latest/dg/lambda-runtimes.html">function runtimes</a>. Used for filtering with <a>ListLayers</a> and <a>ListLayerVersions</a>.<br/>Content:  The function layer archive.:<br/>Description:  The description of the version.<br/>LayerName:  The name or Amazon Resource Name (ARN) of the layer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ZipFile</i> The base64-encoded contents of the layer archive. AWS SDK and AWS CLI clients handle the encoding for you.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3ObjectVersion</i> For versioned objects, the version of the layer archive object to use.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Key</i> The Amazon S3 key of the layer archive.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Bucket</i> The Amazon S3 bucket of the layer archive.<br/>
        :return: LicenseInfo:  The layer's software license.<br/>CompatibleRuntimes:  The layer's compatible runtimes.<br/>Version:  The version number.<br/>CreatedDate:  The date that the layer version was created, in <a href="https://www.w3.org/TR/NOTE-datetime">ISO-8601 format</a> (YYYY-MM-DDThh:mm:ss.sTZD).<br/>Description:  The description of the version.<br/>LayerVersionArn:  The ARN of the layer version.<br/>LayerArn:  The ARN of the layer.<br/>Content:  Details about the layer version.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CodeSize</i> The size of the layer archive in bytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CodeSha256</i> The SHA-256 hash of the layer archive.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Location</i> A link to the layer archive in Amazon S3 that is valid for 10 minutes.<br/>        """
        pass

    def publish_version(self, FunctionName: str, CodeSha256: str=None, Description: str=None, RevisionId: str=None) -> FunctionConfiguration:
        """Creates a <a href="https://docs.aws.amazon.com/lambda/latest/dg/versioning-aliases.html">version</a> from the current code and configuration of a function. Use versions to create a snapshot of your function code and configuration that doesn't change. AWS Lambda doesn't publish a version if the function's configuration and code haven't changed since the last version. Use <a>UpdateFunctionCode</a> or <a>UpdateFunctionConfiguration</a> to update the function before publishing a version. Clients can invoke versions directly or with an alias. To create an alias, use <a>CreateAlias</a>.

        :param self:
        :param RevisionId:  Only update the function if the revision ID matches the ID that's specified. Use this option to avoid publishing a version if the function configuration has changed since you last updated it.<br/>Description:  A description for the version to override the description in the function configuration.<br/>CodeSha256:  Only publish a version if the hash value matches the value that's specified. Use this option to avoid publishing a version if the function code has changed since you last updated it. You can get the hash for the version that you uploaded from the output of <a>UpdateFunctionCode</a>.<br/>FunctionName:  The name of the Lambda function. <p class="title"> <b>Name formats</b>  <ul> <li>  <b>Function name</b> - <code>MyFunction</code>. </li> <li>  <b>Function ARN</b> - <code>arn:aws:lambda:us-west-2:123456789012:function:MyFunction</code>. </li> <li>  <b>Partial ARN</b> - <code>123456789012:function:MyFunction</code>. </li> </ul> The length constraint applies only to the full ARN. If you specify only the function name, it is limited to 64 characters in length.<br/>
        :return: Layers:  The function's <a href="https://docs.aws.amazon.com/lambda/latest/dg/configuration-layers.html"> layers</a>.<br/>RevisionId:  The latest updated revision of the function or alias.<br/>MasterArn:  For Lambda@Edge functions, the ARN of the master function.<br/>TracingConfig:  The function's AWS X-Ray tracing configuration.:<br/>KMSKeyArn:  The KMS key that's used to encrypt the function's environment variables. This key is only returned if you've configured a customer-managed CMK.<br/>Environment:  The function's environment variables.:<br/>DeadLetterConfig:  The function's dead letter queue.:<br/>VpcConfig:  The function's networking configuration.:<br/>Version:  The version of the Lambda function.<br/>CodeSha256:  The SHA256 hash of the function's deployment package.<br/>LastModified:  The date and time that the function was last updated, in <a href="https://www.w3.org/TR/NOTE-datetime">ISO-8601 format</a> (YYYY-MM-DDThh:mm:ss.sTZD).<br/>MemorySize:  The memory that's allocated to the function.<br/>Timeout:  The amount of time that Lambda allows a function to run before stopping it.<br/>Description:  The function's description.<br/>CodeSize:  The size of the function's deployment package, in bytes.<br/>Handler:  The function that Lambda calls to begin executing your function.<br/>Role:  The function's execution role.<br/>Runtime:  The runtime environment for the Lambda function.<br/>FunctionArn:  The function's Amazon Resource Name (ARN).<br/>FunctionName:  The name of the function.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> A list of VPC security groups IDs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> A list of VPC subnet IDs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetArn</i> The Amazon Resource Name (ARN) of an Amazon SQS queue or Amazon SNS topic.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Error</i> Error messages for environment variables that couldn't be applied.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Error</b>: Error messages for environment variables that couldn't be applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Variables</i> Environment variable key-value pairs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> The error message.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorCode</i> The error code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> The tracing mode.<br/>        """
        pass

    def put_function_concurrency(self, ReservedConcurrentExecutions: int, FunctionName: str) -> Concurrency:
        """Sets the maximum number of simultaneous executions for a function, and reserves capacity for that concurrency level. Concurrency settings apply to the function as a whole, including all published versions and the unpublished version. Reserving concurrency both ensures that your function has capacity to process the specified number of events simultaneously, and prevents it from scaling beyond that level. Use <a>GetFunction</a> to see the current setting for a function. Use <a>GetAccountSettings</a> to see your regional concurrency limit. You can reserve concurrency for as many functions as you like, as long as you leave at least 100 simultaneous executions unreserved for functions that aren't configured with a per-function limit. For more information, see <a href="https://docs.aws.amazon.com/lambda/latest/dg/concurrent-executions.html">Managing Concurrency</a>.

        :param self:
        :param ReservedConcurrentExecutions:  The number of simultaneous executions to reserve for the function.<br/>FunctionName:  The name of the Lambda function. <p class="title"> <b>Name formats</b>  <ul> <li>  <b>Function name</b> - <code>my-function</code>. </li> <li>  <b>Function ARN</b> - <code>arn:aws:lambda:us-west-2:123456789012:function:my-function</code>. </li> <li>  <b>Partial ARN</b> - <code>123456789012:function:my-function</code>. </li> </ul> The length constraint applies only to the full ARN. If you specify only the function name, it is limited to 64 characters in length.<br/>
        :return: ReservedConcurrentExecutions:  The number of concurrent executions that are reserved for this function. For more information, see <a href="https://docs.aws.amazon.com/lambda/latest/dg/concurrent-executions.html">Managing Concurrency</a>.<br/>        """
        pass

    def remove_layer_version_permission(self, StatementId: str, VersionNumber: LayerVersionNumber, LayerName: str, RevisionId: str=None):
        """Removes a statement from the permissions policy for a version of an <a href="https://docs.aws.amazon.com/lambda/latest/dg/configuration-layers.html">AWS Lambda layer</a>. For more information, see <a>AddLayerVersionPermission</a>.

        :param self:
        :param RevisionId:  Only update the policy if the revision ID matches the ID specified. Use this option to avoid modifying a policy that has changed since you last read it.<br/>StatementId:  The identifier that was specified when the statement was added.<br/>VersionNumber:  The version number.<br/>LayerName:  The name or Amazon Resource Name (ARN) of the layer.<br/>
        :return:         """
        pass

    def remove_permission(self, StatementId: str, FunctionName: str, Qualifier: str=None, RevisionId: str=None):
        """Revokes function-use permission from an AWS service or another account. You can get the ID of the statement from the output of <a>GetPolicy</a>.

        :param self:
        :param RevisionId:  Only update the policy if the revision ID matches the ID that's specified. Use this option to avoid modifying a policy that has changed since you last read it.<br/>Qualifier:  Specify a version or alias to remove permissions from a published version of the function.<br/>StatementId:  Statement ID of the permission to remove.<br/>FunctionName:  The name of the Lambda function, version, or alias. <p class="title"> <b>Name formats</b>  <ul> <li>  <b>Function name</b> - <code>my-function</code> (name-only), <code>my-function:v1</code> (with alias). </li> <li>  <b>Function ARN</b> - <code>arn:aws:lambda:us-west-2:123456789012:function:my-function</code>. </li> <li>  <b>Partial ARN</b> - <code>123456789012:function:my-function</code>. </li> </ul> You can append a version number or alias to any of the formats. The length constraint applies only to the full ARN. If you specify only the function name, it is limited to 64 characters in length.<br/>
        :return:         """
        pass

    def tag_resource(self, Tags: Tags, Resource: str):
        """Adds <a href="https://docs.aws.amazon.com/lambda/latest/dg/tagging.html">tags</a> to a function.

        :param self:
        :param Tags:  A list of tags to apply to the function.<br/>Resource:  The function's Amazon Resource Name (ARN).<br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[TagKey], Resource: str):
        """Removes <a href="https://docs.aws.amazon.com/lambda/latest/dg/tagging.html">tags</a> from a function.

        :param self:
        :param TagKeys:  A list of tag keys to remove from the function.<br/>Resource:  The function's Amazon Resource Name (ARN).<br/>
        :return:         """
        pass

    def update_alias(self, Name: str, FunctionName: str, FunctionVersion: str=None, Description: str=None, RoutingConfig: AliasRoutingConfiguration=None, RevisionId: str=None) -> AliasConfiguration:
        """Updates the configuration of a Lambda function <a href="https://docs.aws.amazon.com/lambda/latest/dg/versioning-aliases.html">alias</a>.

        :param self:
        :param RevisionId:  Only update the alias if the revision ID matches the ID that's specified. Use this option to avoid modifying an alias that has changed since you last read it.<br/>RoutingConfig:  The <a href="https://docs.aws.amazon.com/lambda/latest/dg/lambda-traffic-shifting-using-aliases.html">routing configuration</a> of the alias.:<br/>Description:  A description of the alias.<br/>FunctionVersion:  The function version that the alias invokes.<br/>Name:  The name of the alias.<br/>FunctionName:  The name of the Lambda function. <p class="title"> <b>Name formats</b>  <ul> <li>  <b>Function name</b> - <code>MyFunction</code>. </li> <li>  <b>Function ARN</b> - <code>arn:aws:lambda:us-west-2:123456789012:function:MyFunction</code>. </li> <li>  <b>Partial ARN</b> - <code>123456789012:function:MyFunction</code>. </li> </ul> The length constraint applies only to the full ARN. If you specify only the function name, it is limited to 64 characters in length.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdditionalVersionWeights</i> The name of the second alias, and the percentage of traffic that's routed to it.<br/>
        :return: RevisionId:  A unique identifier that changes when you update the alias.<br/>RoutingConfig:  The <a href="https://docs.aws.amazon.com/lambda/latest/dg/lambda-traffic-shifting-using-aliases.html">routing configuration</a> of the alias.:<br/>Description:  A description of the alias.<br/>FunctionVersion:  The function version that the alias invokes.<br/>Name:  The name of the alias.<br/>AliasArn:  The Amazon Resource Name (ARN) of the alias.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdditionalVersionWeights</i> The name of the second alias, and the percentage of traffic that's routed to it.<br/>        """
        pass

    def update_event_source_mapping(self, UUID: str, FunctionName: str=None, Enabled: Enabled=None, BatchSize: int=None) -> EventSourceMappingConfiguration:
        """Updates an event source mapping. You can change the function that AWS Lambda invokes, or pause invocation and resume later from the same location.

        :param self:
        :param BatchSize:  The maximum number of items to retrieve in a single batch. <ul> <li>  <b>Amazon Kinesis</b> - Default 100. Max 10,000. </li> <li>  <b>Amazon DynamoDB Streams</b> - Default 100. Max 1,000. </li> <li>  <b>Amazon Simple Queue Service</b> - Default 10. Max 10. </li> </ul><br/>Enabled:  Disables the event source mapping to pause polling and invocation.<br/>FunctionName:  The name of the Lambda function. <p class="title"> <b>Name formats</b>  <ul> <li>  <b>Function name</b> - <code>MyFunction</code>. </li> <li>  <b>Function ARN</b> - <code>arn:aws:lambda:us-west-2:123456789012:function:MyFunction</code>. </li> <li>  <b>Version or Alias ARN</b> - <code>arn:aws:lambda:us-west-2:123456789012:function:MyFunction:PROD</code>. </li> <li>  <b>Partial ARN</b> - <code>123456789012:function:MyFunction</code>. </li> </ul> The length constraint applies only to the full ARN. If you specify only the function name, it's limited to 64 characters in length.<br/>UUID:  The identifier of the event source mapping.<br/>
        :return: StateTransitionReason:  The cause of the last state change, either <code>User initiated</code> or <code>Lambda initiated</code>.<br/>State:  The state of the event source mapping. It can be one of the following: <code>Creating</code>, <code>Enabling</code>, <code>Enabled</code>, <code>Disabling</code>, <code>Disabled</code>, <code>Updating</code>, or <code>Deleting</code>.<br/>LastProcessingResult:  The result of the last AWS Lambda invocation of your Lambda function.<br/>LastModified:  The date that the event source mapping was last updated.<br/>FunctionArn:  The ARN of the Lambda function.<br/>EventSourceArn:  The Amazon Resource Name (ARN) of the event source.<br/>BatchSize:  The maximum number of items to retrieve in a single batch.<br/>UUID:  The identifier of the event source mapping.<br/>        """
        pass

    def update_function_code(self, FunctionName: str, ZipFile: Blob=None, S3Bucket: str=None, S3Key: str=None, S3ObjectVersion: str=None, Publish: Boolean=None, DryRun: Boolean=None, RevisionId: str=None) -> FunctionConfiguration:
        """Updates a Lambda function's code. The function's code is locked when you publish a version. You can't modify the code of a published version, only the unpublished version.

        :param self:
        :param RevisionId:  Only update the function if the revision ID matches the ID that's specified. Use this option to avoid modifying a function that has changed since you last read it.<br/>DryRun:  Set to true to validate the request parameters and access permissions without modifying the function code.<br/>Publish:  Set to true to publish a new version of the function after updating the code. This has the same effect as calling <a>PublishVersion</a> separately.<br/>S3ObjectVersion:  For versioned objects, the version of the deployment package object to use.<br/>S3Key:  The Amazon S3 key of the deployment package.<br/>S3Bucket:  An Amazon S3 bucket in the same AWS Region as your function. The bucket can be in a different AWS account.<br/>ZipFile:  The base64-encoded contents of the deployment package. AWS SDK and AWS CLI clients handle the encoding for you.<br/>FunctionName:  The name of the Lambda function. <p class="title"> <b>Name formats</b>  <ul> <li>  <b>Function name</b> - <code>my-function</code>. </li> <li>  <b>Function ARN</b> - <code>arn:aws:lambda:us-west-2:123456789012:function:my-function</code>. </li> <li>  <b>Partial ARN</b> - <code>123456789012:function:my-function</code>. </li> </ul> The length constraint applies only to the full ARN. If you specify only the function name, it is limited to 64 characters in length.<br/>
        :return: Layers:  The function's <a href="https://docs.aws.amazon.com/lambda/latest/dg/configuration-layers.html"> layers</a>.<br/>RevisionId:  The latest updated revision of the function or alias.<br/>MasterArn:  For Lambda@Edge functions, the ARN of the master function.<br/>TracingConfig:  The function's AWS X-Ray tracing configuration.:<br/>KMSKeyArn:  The KMS key that's used to encrypt the function's environment variables. This key is only returned if you've configured a customer-managed CMK.<br/>Environment:  The function's environment variables.:<br/>DeadLetterConfig:  The function's dead letter queue.:<br/>VpcConfig:  The function's networking configuration.:<br/>Version:  The version of the Lambda function.<br/>CodeSha256:  The SHA256 hash of the function's deployment package.<br/>LastModified:  The date and time that the function was last updated, in <a href="https://www.w3.org/TR/NOTE-datetime">ISO-8601 format</a> (YYYY-MM-DDThh:mm:ss.sTZD).<br/>MemorySize:  The memory that's allocated to the function.<br/>Timeout:  The amount of time that Lambda allows a function to run before stopping it.<br/>Description:  The function's description.<br/>CodeSize:  The size of the function's deployment package, in bytes.<br/>Handler:  The function that Lambda calls to begin executing your function.<br/>Role:  The function's execution role.<br/>Runtime:  The runtime environment for the Lambda function.<br/>FunctionArn:  The function's Amazon Resource Name (ARN).<br/>FunctionName:  The name of the function.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> A list of VPC security groups IDs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> A list of VPC subnet IDs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetArn</i> The Amazon Resource Name (ARN) of an Amazon SQS queue or Amazon SNS topic.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Error</i> Error messages for environment variables that couldn't be applied.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Error</b>: Error messages for environment variables that couldn't be applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Variables</i> Environment variable key-value pairs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> The error message.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorCode</i> The error code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> The tracing mode.<br/>        """
        pass

    def update_function_configuration(self, FunctionName: str, Role: str=None, Handler: str=None, Description: str=None, Timeout: int=None, MemorySize: int=None, VpcConfig: VpcConfig=None, Environment: Environment=None, Runtime: str=None, DeadLetterConfig: DeadLetterConfig=None, KMSKeyArn: str=None, TracingConfig: TracingConfig=None, RevisionId: str=None, Layers: List[LayerVersionArn]=None) -> FunctionConfiguration:
        """Modify the version-specific settings of a Lambda function. These settings can vary between versions of a function and are locked when you publish a version. You can't modify the configuration of a published version, only the unpublished version. To configure function concurrency, use <a>PutFunctionConcurrency</a>. To grant invoke permissions to an account or AWS service, use <a>AddPermission</a>.

        :param self:
        :param Layers:  A list of <a href="https://docs.aws.amazon.com/lambda/latest/dg/configuration-layers.html">function layers</a> to add to the function's execution environment. Specify each layer by its ARN, including the version.<br/>RevisionId:  Only update the function if the revision ID matches the ID that's specified. Use this option to avoid modifying a function that has changed since you last read it.<br/>TracingConfig:  Set <code>Mode</code> to <code>Active</code> to sample and trace a subset of incoming requests with AWS X-Ray.:<br/>KMSKeyArn:  The ARN of the AWS Key Management Service (AWS KMS) key that's used to encrypt your function's environment variables. If it's not provided, AWS Lambda uses a default service key.<br/>DeadLetterConfig:  A dead letter queue configuration that specifies the queue or topic where Lambda sends asynchronous events when they fail processing. For more information, see <a href="https://docs.aws.amazon.com/lambda/latest/dg/dlq.html">Dead Letter Queues</a>.:<br/>Runtime:  The identifier of the function's <a href="https://docs.aws.amazon.com/lambda/latest/dg/lambda-runtimes.html">runtime</a>.<br/>Environment:  Environment variables that are accessible from function code during execution.:<br/>VpcConfig:  For network connectivity to AWS resources in a VPC, specify a list of security groups and subnets in the VPC. When you connect a function to a VPC, it can only access resources and the internet through that VPC. For more information, see <a href="https://docs.aws.amazon.com/lambda/latest/dg/vpc.html">VPC Settings</a>.:<br/>MemorySize:  The amount of memory that your function has access to. Increasing the function's memory also increases its CPU allocation. The default value is 128 MB. The value must be a multiple of 64 MB.<br/>Timeout:  The amount of time that Lambda allows a function to run before stopping it. The default is 3 seconds. The maximum allowed value is 900 seconds.<br/>Description:  A description of the function.<br/>Handler:  The name of the method within your code that Lambda calls to execute your function. The format includes the file name. It can also include namespaces and other qualifiers, depending on the runtime. For more information, see <a href="https://docs.aws.amazon.com/lambda/latest/dg/programming-model-v2.html">Programming Model</a>.<br/>Role:  The Amazon Resource Name (ARN) of the function's execution role.<br/>FunctionName:  The name of the Lambda function. <p class="title"> <b>Name formats</b>  <ul> <li>  <b>Function name</b> - <code>my-function</code>. </li> <li>  <b>Function ARN</b> - <code>arn:aws:lambda:us-west-2:123456789012:function:my-function</code>. </li> <li>  <b>Partial ARN</b> - <code>123456789012:function:my-function</code>. </li> </ul> The length constraint applies only to the full ARN. If you specify only the function name, it is limited to 64 characters in length.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> A list of VPC security groups IDs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> A list of VPC subnet IDs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Variables</i> Environment variable key-value pairs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetArn</i> The Amazon Resource Name (ARN) of an Amazon SQS queue or Amazon SNS topic.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> The tracing mode.<br/>
        :return: Layers:  The function's <a href="https://docs.aws.amazon.com/lambda/latest/dg/configuration-layers.html"> layers</a>.<br/>RevisionId:  The latest updated revision of the function or alias.<br/>MasterArn:  For Lambda@Edge functions, the ARN of the master function.<br/>TracingConfig:  The function's AWS X-Ray tracing configuration.:<br/>KMSKeyArn:  The KMS key that's used to encrypt the function's environment variables. This key is only returned if you've configured a customer-managed CMK.<br/>Environment:  The function's environment variables.:<br/>DeadLetterConfig:  The function's dead letter queue.:<br/>VpcConfig:  The function's networking configuration.:<br/>Version:  The version of the Lambda function.<br/>CodeSha256:  The SHA256 hash of the function's deployment package.<br/>LastModified:  The date and time that the function was last updated, in <a href="https://www.w3.org/TR/NOTE-datetime">ISO-8601 format</a> (YYYY-MM-DDThh:mm:ss.sTZD).<br/>MemorySize:  The memory that's allocated to the function.<br/>Timeout:  The amount of time that Lambda allows a function to run before stopping it.<br/>Description:  The function's description.<br/>CodeSize:  The size of the function's deployment package, in bytes.<br/>Handler:  The function that Lambda calls to begin executing your function.<br/>Role:  The function's execution role.<br/>Runtime:  The runtime environment for the Lambda function.<br/>FunctionArn:  The function's Amazon Resource Name (ARN).<br/>FunctionName:  The name of the function.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The ID of the VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> A list of VPC security groups IDs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> A list of VPC subnet IDs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetArn</i> The Amazon Resource Name (ARN) of an Amazon SQS queue or Amazon SNS topic.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Error</i> Error messages for environment variables that couldn't be applied.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Error</b>: Error messages for environment variables that couldn't be applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Variables</i> Environment variable key-value pairs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> The error message.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorCode</i> The error code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> The tracing mode.<br/>        """
        pass



class LexModelBuildingService(BaseClient):
    """<fullname>Amazon Lex Build-Time Actions</fullname>  Amazon Lex is an AWS service for building conversational voice and text interfaces. Use these actions to create, update, and delete conversational bots for new and existing client applications. """


    class GetSlotTypeVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_slot_type_versions` for valid parameters"""
            pass
    class GetSlotTypesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_slot_types` for valid parameters"""
            pass
    class GetIntentsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_intents` for valid parameters"""
            pass
    class GetBotChannelAssociationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_bot_channel_associations` for valid parameters"""
            pass
    class GetBotsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_bots` for valid parameters"""
            pass
    class GetBuiltinSlotTypesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_builtin_slot_types` for valid parameters"""
            pass
    class GetIntentVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_intent_versions` for valid parameters"""
            pass
    class GetBotAliasesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_bot_aliases` for valid parameters"""
            pass
    class GetBuiltinIntentsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_builtin_intents` for valid parameters"""
            pass
    class GetBotVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_bot_versions` for valid parameters"""
            pass


    class CreateBotVersionRequest(Mapping):
        pass
    
    class CreateBotVersionResponse(Mapping):
        pass
    
    class CreateIntentVersionRequest(Mapping):
        pass
    
    class CreateIntentVersionResponse(Mapping):
        pass
    
    class CreateSlotTypeVersionRequest(Mapping):
        pass
    
    class CreateSlotTypeVersionResponse(Mapping):
        pass
    
    class DeleteBotRequest(Mapping):
        pass
    
    class DeleteBotAliasRequest(Mapping):
        pass
    
    class DeleteBotChannelAssociationRequest(Mapping):
        pass
    
    class DeleteBotVersionRequest(Mapping):
        pass
    
    class DeleteIntentRequest(Mapping):
        pass
    
    class DeleteIntentVersionRequest(Mapping):
        pass
    
    class DeleteSlotTypeRequest(Mapping):
        pass
    
    class DeleteSlotTypeVersionRequest(Mapping):
        pass
    
    class DeleteUtterancesRequest(Mapping):
        pass
    
    class GetBotRequest(Mapping):
        pass
    
    class GetBotResponse(Mapping):
        pass
    
    class GetBotAliasRequest(Mapping):
        pass
    
    class GetBotAliasResponse(Mapping):
        pass
    
    class GetBotAliasesRequest(Mapping):
        pass
    
    class GetBotAliasesResponse(Mapping):
        pass
    
    class GetBotChannelAssociationRequest(Mapping):
        pass
    
    class GetBotChannelAssociationResponse(Mapping):
        pass
    
    class GetBotChannelAssociationsRequest(Mapping):
        pass
    
    class GetBotChannelAssociationsResponse(Mapping):
        pass
    
    class GetBotVersionsRequest(Mapping):
        pass
    
    class GetBotVersionsResponse(Mapping):
        pass
    
    class GetBotsRequest(Mapping):
        pass
    
    class GetBotsResponse(Mapping):
        pass
    
    class GetBuiltinIntentRequest(Mapping):
        pass
    
    class GetBuiltinIntentResponse(Mapping):
        pass
    
    class GetBuiltinIntentsRequest(Mapping):
        pass
    
    class GetBuiltinIntentsResponse(Mapping):
        pass
    
    class GetBuiltinSlotTypesRequest(Mapping):
        pass
    
    class GetBuiltinSlotTypesResponse(Mapping):
        pass
    
    class GetExportRequest(Mapping):
        pass
    
    class GetExportResponse(Mapping):
        pass
    
    class GetImportRequest(Mapping):
        pass
    
    class GetImportResponse(Mapping):
        pass
    
    class GetIntentRequest(Mapping):
        pass
    
    class GetIntentResponse(Mapping):
        pass
    
    class GetIntentVersionsRequest(Mapping):
        pass
    
    class GetIntentVersionsResponse(Mapping):
        pass
    
    class GetIntentsRequest(Mapping):
        pass
    
    class GetIntentsResponse(Mapping):
        pass
    
    class GetSlotTypeRequest(Mapping):
        pass
    
    class GetSlotTypeResponse(Mapping):
        pass
    
    class GetSlotTypeVersionsRequest(Mapping):
        pass
    
    class GetSlotTypeVersionsResponse(Mapping):
        pass
    
    class GetSlotTypesRequest(Mapping):
        pass
    
    class GetSlotTypesResponse(Mapping):
        pass
    
    class GetUtterancesViewRequest(Mapping):
        pass
    
    class GetUtterancesViewResponse(Mapping):
        pass
    
    class Version(object):
        pass
    
    class PutBotRequest(Mapping):
        pass
    
    class PutBotResponse(Mapping):
        pass
    
    class Intent(Mapping):
        pass
    
    class Prompt(Mapping):
        pass
    
    class Statement(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class PutBotAliasRequest(Mapping):
        pass
    
    class PutBotAliasResponse(Mapping):
        pass
    
    class PutIntentRequest(Mapping):
        pass
    
    class PutIntentResponse(Mapping):
        pass
    
    class Slot(Mapping):
        pass
    
    class Utterance(object):
        pass
    
    class FollowUpPrompt(Mapping):
        pass
    
    class CodeHook(Mapping):
        pass
    
    class FulfillmentActivity(Mapping):
        pass
    
    class PutSlotTypeRequest(Mapping):
        pass
    
    class PutSlotTypeResponse(Mapping):
        pass
    
    class EnumerationValue(Mapping):
        pass
    
    class StartImportRequest(Mapping):
        pass
    
    class StartImportResponse(Mapping):
        pass
    
    class Blob(object):
        pass
    
    def create_bot_version(self, name: str, checksum: str=None) -> CreateBotVersionResponse:
        """Creates a new version of the bot based on the <code>$LATEST</code> version. If the <code>$LATEST</code> version of this resource hasn't changed since you created the last version, Amazon Lex doesn't create a new version. It returns the last created version. <note> You can update only the <code>$LATEST</code> version of the bot. You can't update the numbered versions that you create with the <code>CreateBotVersion</code> operation. </note>  When you create the first version of a bot, Amazon Lex sets the version to 1. Subsequent versions increment by 1. For more information, see <a>versioning-intro</a>.   This operation requires permission for the <code>lex:CreateBotVersion</code> action. 

        :param self:
        :param checksum:  Identifies a specific revision of the <code>$LATEST</code> version of the bot. If you specify a checksum and the <code>$LATEST</code> version of the bot has a different checksum, a <code>PreconditionFailedException</code> exception is returned and Amazon Lex doesn't publish a new version. If you don't specify a checksum, Amazon Lex publishes the <code>$LATEST</code> version.<br/>name:  The name of the bot that you want to create a new version of. The name is case sensitive. <br/>
        :return: childDirected:  For each Amazon Lex bot created with the Amazon Lex Model Building Service, you must specify whether your use of Amazon Lex is related to a website, program, or other application that is directed or targeted, in whole or in part, to children under age 13 and subject to the Children's Online Privacy Protection Act (COPPA) by specifying <code>true</code> or <code>false</code> in the <code>childDirected</code> field. By specifying <code>true</code> in the <code>childDirected</code> field, you confirm that your use of Amazon Lex <b>is</b> related to a website, program, or other application that is directed or targeted, in whole or in part, to children under age 13 and subject to COPPA. By specifying <code>false</code> in the <code>childDirected</code> field, you confirm that your use of Amazon Lex <b>is not</b> related to a website, program, or other application that is directed or targeted, in whole or in part, to children under age 13 and subject to COPPA. You may not specify a default value for the <code>childDirected</code> field that does not accurately reflect whether your use of Amazon Lex is related to a website, program, or other application that is directed or targeted, in whole or in part, to children under age 13 and subject to COPPA. If your use of Amazon Lex relates to a website, program, or other application that is directed in whole or in part, to children under age 13, you must obtain any required verifiable parental consent under COPPA. For information regarding the use of Amazon Lex in connection with websites, programs, or other applications that are directed or targeted, in whole or in part, to children under age 13, see the <a href="https://aws.amazon.com/lex/faqs#data-security">Amazon Lex FAQ.</a> <br/>locale:   Specifies the target locale for the bot. <br/>version:  The version of the bot. <br/>checksum:  Checksum identifying the version of the bot that was created.<br/>voiceId:  The Amazon Polly voice ID that Amazon Lex uses for voice interactions with the user.<br/>idleSessionTTLInSeconds:  The maximum time in seconds that Amazon Lex retains the data gathered in a conversation. For more information, see <a>PutBot</a>.<br/>createdDate:  The date when the bot version was created.<br/>lastUpdatedDate:  The date when the <code>$LATEST</code> version of this bot was updated. <br/>failureReason:  If <code>status</code> is <code>FAILED</code>, Amazon Lex provides the reason that it failed to build the bot.<br/>status:   When you send a request to create or update a bot, Amazon Lex sets the <code>status</code> response element to <code>BUILDING</code>. After Amazon Lex builds the bot, it sets <code>status</code> to <code>READY</code>. If Amazon Lex can't build the bot, it sets <code>status</code> to <code>FAILED</code>. Amazon Lex returns the reason for the failure in the <code>failureReason</code> response element. <br/>abortStatement:  The message that Amazon Lex uses to abort a conversation. For more information, see <a>PutBot</a>.:<br/>clarificationPrompt:  The message that Amazon Lex uses when it doesn't understand the user's request. For more information, see <a>PutBot</a>. :<br/>intents:  An array of <code>Intent</code> objects. For more information, see <a>PutBot</a>.<br/>description:  A description of the bot.<br/>name:  The name of the bot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>responseCard</i> A response card. Amazon Lex uses this prompt at runtime, in the <code>PostText</code> API response. It substitutes session attributes and slot values for placeholders in the response card. For more information, see <a>ex-resp-card</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>maxAttempts</i> The number of times to prompt the user for information.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>messages</i> An array of objects, each of which provides a message string and its type. You can specify the message string in plain text or in Speech Synthesis Markup Language (SSML).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>responseCard</i>  At runtime, if the client is using the <a href="http://docs.aws.amazon.com/lex/latest/dg/API_runtime_PostText.html">PostText</a> API, Amazon Lex includes the response card in the response. It substitutes all of the session attributes and slot values for placeholders in the response card. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>messages</i> A collection of message objects.<br/>        """
        pass

    def create_intent_version(self, name: str, checksum: str=None) -> CreateIntentVersionResponse:
        """Creates a new version of an intent based on the <code>$LATEST</code> version of the intent. If the <code>$LATEST</code> version of this intent hasn't changed since you last updated it, Amazon Lex doesn't create a new version. It returns the last version you created. <note> You can update only the <code>$LATEST</code> version of the intent. You can't update the numbered versions that you create with the <code>CreateIntentVersion</code> operation. </note>  When you create a version of an intent, Amazon Lex sets the version to 1. Subsequent versions increment by 1. For more information, see <a>versioning-intro</a>.  This operation requires permissions to perform the <code>lex:CreateIntentVersion</code> action. 

        :param self:
        :param checksum:  Checksum of the <code>$LATEST</code> version of the intent that should be used to create the new version. If you specify a checksum and the <code>$LATEST</code> version of the intent has a different checksum, Amazon Lex returns a <code>PreconditionFailedException</code> exception and doesn't publish a new version. If you don't specify a checksum, Amazon Lex publishes the <code>$LATEST</code> version.<br/>name:  The name of the intent that you want to create a new version of. The name is case sensitive. <br/>
        :return: checksum:  Checksum of the intent version created.<br/>version:  The version number assigned to the new version of the intent.<br/>createdDate:  The date that the intent was created.<br/>lastUpdatedDate:  The date that the intent was updated. <br/>parentIntentSignature:  A unique identifier for a built-in intent.<br/>fulfillmentActivity:   Describes how the intent is fulfilled. :<br/>dialogCodeHook:  If defined, Amazon Lex invokes this Lambda function for each user input.:<br/>conclusionStatement:  After the Lambda function specified in the <code>fulfillmentActivity</code> field fulfills the intent, Amazon Lex conveys this statement to the user. :<br/>followUpPrompt:  If defined, Amazon Lex uses this prompt to solicit additional user activity after the intent is fulfilled. :<br/>rejectionStatement:  If the user answers "no" to the question defined in <code>confirmationPrompt</code>, Amazon Lex responds with this statement to acknowledge that the intent was canceled. :<br/>confirmationPrompt:  If defined, the prompt that Amazon Lex uses to confirm the user's intent before fulfilling it. :<br/>sampleUtterances:  An array of sample utterances configured for the intent. <br/>slots:  An array of slot types that defines the information required to fulfill the intent.<br/>description:  A description of the intent.<br/>name:  The name of the intent.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>responseCard</i> A response card. Amazon Lex uses this prompt at runtime, in the <code>PostText</code> API response. It substitutes session attributes and slot values for placeholders in the response card. For more information, see <a>ex-resp-card</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>maxAttempts</i> The number of times to prompt the user for information.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>messages</i> An array of objects, each of which provides a message string and its type. You can specify the message string in plain text or in Speech Synthesis Markup Language (SSML).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>responseCard</i>  At runtime, if the client is using the <a href="http://docs.aws.amazon.com/lex/latest/dg/API_runtime_PostText.html">PostText</a> API, Amazon Lex includes the response card in the response. It substitutes all of the session attributes and slot values for placeholders in the response card. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>messages</i> A collection of message objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>rejectionStatement</i> If the user answers "no" to the question defined in the <code>prompt</code> field, Amazon Lex responds with this statement to acknowledge that the intent was canceled. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>rejectionStatement</b>: If the user answers "no" to the question defined in the <code>prompt</code> field, Amazon Lex responds with this statement to acknowledge that the intent was canceled. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>prompt</i> Prompts for information from the user. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>prompt</b>: Prompts for information from the user. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>responseCard</i> A response card. Amazon Lex uses this prompt at runtime, in the <code>PostText</code> API response. It substitutes session attributes and slot values for placeholders in the response card. For more information, see <a>ex-resp-card</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>maxAttempts</i> The number of times to prompt the user for information.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>messages</i> An array of objects, each of which provides a message string and its type. You can specify the message string in plain text or in Speech Synthesis Markup Language (SSML).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>responseCard</i>  At runtime, if the client is using the <a href="http://docs.aws.amazon.com/lex/latest/dg/API_runtime_PostText.html">PostText</a> API, Amazon Lex includes the response card in the response. It substitutes all of the session attributes and slot values for placeholders in the response card. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>messages</i> A collection of message objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>responseCard</i>  At runtime, if the client is using the <a href="http://docs.aws.amazon.com/lex/latest/dg/API_runtime_PostText.html">PostText</a> API, Amazon Lex includes the response card in the response. It substitutes all of the session attributes and slot values for placeholders in the response card. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>messages</i> A collection of message objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>messageVersion</i> The version of the request-response that you want Amazon Lex to use to invoke your Lambda function. For more information, see <a>using-lambda</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>uri</i> The Amazon Resource Name (ARN) of the Lambda function.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>codeHook</i>  A description of the Lambda function that is run to fulfill the intent. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>codeHook</b>:  A description of the Lambda function that is run to fulfill the intent. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i>  How the intent should be fulfilled, either by running a Lambda function or by returning the slot data to the client application. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>messageVersion</i> The version of the request-response that you want Amazon Lex to use to invoke your Lambda function. For more information, see <a>using-lambda</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uri</i> The Amazon Resource Name (ARN) of the Lambda function.<br/>        """
        pass

    def create_slot_type_version(self, name: str, checksum: str=None) -> CreateSlotTypeVersionResponse:
        """Creates a new version of a slot type based on the <code>$LATEST</code> version of the specified slot type. If the <code>$LATEST</code> version of this resource has not changed since the last version that you created, Amazon Lex doesn't create a new version. It returns the last version that you created.  <note> You can update only the <code>$LATEST</code> version of a slot type. You can't update the numbered versions that you create with the <code>CreateSlotTypeVersion</code> operation. </note> When you create a version of a slot type, Amazon Lex sets the version to 1. Subsequent versions increment by 1. For more information, see <a>versioning-intro</a>.  This operation requires permissions for the <code>lex:CreateSlotTypeVersion</code> action.

        :param self:
        :param checksum:  Checksum for the <code>$LATEST</code> version of the slot type that you want to publish. If you specify a checksum and the <code>$LATEST</code> version of the slot type has a different checksum, Amazon Lex returns a <code>PreconditionFailedException</code> exception and doesn't publish the new version. If you don't specify a checksum, Amazon Lex publishes the <code>$LATEST</code> version.<br/>name:  The name of the slot type that you want to create a new version for. The name is case sensitive. <br/>
        :return: valueSelectionStrategy:  The strategy that Amazon Lex uses to determine the value of the slot. For more information, see <a>PutSlotType</a>.<br/>checksum:  Checksum of the <code>$LATEST</code> version of the slot type.<br/>version:  The version assigned to the new slot type version. <br/>createdDate:  The date that the slot type was created.<br/>lastUpdatedDate:  The date that the slot type was updated. When you create a resource, the creation date and last update date are the same.<br/>enumerationValues:  A list of <code>EnumerationValue</code> objects that defines the values that the slot type can take.<br/>description:  A description of the slot type.<br/>name:  The name of the slot type.<br/>        """
        pass

    def delete_bot(self, name: str):
        """Deletes all versions of the bot, including the <code>$LATEST</code> version. To delete a specific version of the bot, use the <a>DeleteBotVersion</a> operation. If a bot has an alias, you can't delete it. Instead, the <code>DeleteBot</code> operation returns a <code>ResourceInUseException</code> exception that includes a reference to the alias that refers to the bot. To remove the reference to the bot, delete the alias. If you get the same exception again, delete the referring alias until the <code>DeleteBot</code> operation is successful. This operation requires permissions for the <code>lex:DeleteBot</code> action.

        :param self:
        :param name:  The name of the bot. The name is case sensitive. <br/>
        :return:         """
        pass

    def delete_bot_alias(self, botName: str, name: str):
        """Deletes an alias for the specified bot.  You can't delete an alias that is used in the association between a bot and a messaging channel. If an alias is used in a channel association, the <code>DeleteBot</code> operation returns a <code>ResourceInUseException</code> exception that includes a reference to the channel association that refers to the bot. You can remove the reference to the alias by deleting the channel association. If you get the same exception again, delete the referring association until the <code>DeleteBotAlias</code> operation is successful.

        :param self:
        :param botName:  The name of the bot that the alias points to.<br/>name:  The name of the alias to delete. The name is case sensitive. <br/>
        :return:         """
        pass

    def delete_bot_channel_association(self, botAlias: str, botName: str, name: str):
        """Deletes the association between an Amazon Lex bot and a messaging platform. This operation requires permission for the <code>lex:DeleteBotChannelAssociation</code> action.

        :param self:
        :param botAlias:  An alias that points to the specific version of the Amazon Lex bot to which this association is being made.<br/>botName:  The name of the Amazon Lex bot.<br/>name:  The name of the association. The name is case sensitive. <br/>
        :return:         """
        pass

    def delete_bot_version(self, version: str, name: str):
        """Deletes a specific version of a bot. To delete all versions of a bot, use the <a>DeleteBot</a> operation.  This operation requires permissions for the <code>lex:DeleteBotVersion</code> action.

        :param self:
        :param version:  The version of the bot to delete. You cannot delete the <code>$LATEST</code> version of the bot. To delete the <code>$LATEST</code> version, use the <a>DeleteBot</a> operation.<br/>name:  The name of the bot.<br/>
        :return:         """
        pass

    def delete_intent(self, name: str):
        """Deletes all versions of the intent, including the <code>$LATEST</code> version. To delete a specific version of the intent, use the <a>DeleteIntentVersion</a> operation.  You can delete a version of an intent only if it is not referenced. To delete an intent that is referred to in one or more bots (see <a>how-it-works</a>), you must remove those references first.  <note>  If you get the <code>ResourceInUseException</code> exception, it provides an example reference that shows where the intent is referenced. To remove the reference to the intent, either update the bot or delete it. If you get the same exception when you attempt to delete the intent again, repeat until the intent has no references and the call to <code>DeleteIntent</code> is successful.  </note>  This operation requires permission for the <code>lex:DeleteIntent</code> action. 

        :param self:
        :param name:  The name of the intent. The name is case sensitive. <br/>
        :return:         """
        pass

    def delete_intent_version(self, version: str, name: str):
        """Deletes a specific version of an intent. To delete all versions of a intent, use the <a>DeleteIntent</a> operation.  This operation requires permissions for the <code>lex:DeleteIntentVersion</code> action.

        :param self:
        :param version:  The version of the intent to delete. You cannot delete the <code>$LATEST</code> version of the intent. To delete the <code>$LATEST</code> version, use the <a>DeleteIntent</a> operation.<br/>name:  The name of the intent.<br/>
        :return:         """
        pass

    def delete_slot_type(self, name: str):
        """Deletes all versions of the slot type, including the <code>$LATEST</code> version. To delete a specific version of the slot type, use the <a>DeleteSlotTypeVersion</a> operation.  You can delete a version of a slot type only if it is not referenced. To delete a slot type that is referred to in one or more intents, you must remove those references first.  <note>  If you get the <code>ResourceInUseException</code> exception, the exception provides an example reference that shows the intent where the slot type is referenced. To remove the reference to the slot type, either update the intent or delete it. If you get the same exception when you attempt to delete the slot type again, repeat until the slot type has no references and the <code>DeleteSlotType</code> call is successful.  </note> This operation requires permission for the <code>lex:DeleteSlotType</code> action.

        :param self:
        :param name:  The name of the slot type. The name is case sensitive. <br/>
        :return:         """
        pass

    def delete_slot_type_version(self, version: str, name: str):
        """Deletes a specific version of a slot type. To delete all versions of a slot type, use the <a>DeleteSlotType</a> operation.  This operation requires permissions for the <code>lex:DeleteSlotTypeVersion</code> action.

        :param self:
        :param version:  The version of the slot type to delete. You cannot delete the <code>$LATEST</code> version of the slot type. To delete the <code>$LATEST</code> version, use the <a>DeleteSlotType</a> operation.<br/>name:  The name of the slot type.<br/>
        :return:         """
        pass

    def delete_utterances(self, userId: str, botName: str):
        """Deletes stored utterances. Amazon Lex stores the utterances that users send to your bot. Utterances are stored for 15 days for use with the <a>GetUtterancesView</a> operation, and then stored indefinitely for use in improving the ability of your bot to respond to user input. Use the <code>DeleteStoredUtterances</code> operation to manually delete stored utterances for a specific user. This operation requires permissions for the <code>lex:DeleteUtterances</code> action.

        :param self:
        :param userId:   The unique identifier for the user that made the utterances. This is the user ID that was sent in the <a href="http://docs.aws.amazon.com/lex/latest/dg/API_runtime_PostContent.html">PostContent</a> or <a href="http://docs.aws.amazon.com/lex/latest/dg/API_runtime_PostText.html">PostText</a> operation request that contained the utterance.<br/>botName:  The name of the bot that stored the utterances.<br/>
        :return:         """
        pass

    def get_bot(self, versionOrAlias: str, name: str) -> GetBotResponse:
        """Returns metadata information for a specific bot. You must provide the bot name and the bot version or alias.   This operation requires permissions for the <code>lex:GetBot</code> action. 

        :param self:
        :param versionOrAlias:  The version or alias of the bot.<br/>name:  The name of the bot. The name is case sensitive. <br/>
        :return: childDirected:  For each Amazon Lex bot created with the Amazon Lex Model Building Service, you must specify whether your use of Amazon Lex is related to a website, program, or other application that is directed or targeted, in whole or in part, to children under age 13 and subject to the Children's Online Privacy Protection Act (COPPA) by specifying <code>true</code> or <code>false</code> in the <code>childDirected</code> field. By specifying <code>true</code> in the <code>childDirected</code> field, you confirm that your use of Amazon Lex <b>is</b> related to a website, program, or other application that is directed or targeted, in whole or in part, to children under age 13 and subject to COPPA. By specifying <code>false</code> in the <code>childDirected</code> field, you confirm that your use of Amazon Lex <b>is not</b> related to a website, program, or other application that is directed or targeted, in whole or in part, to children under age 13 and subject to COPPA. You may not specify a default value for the <code>childDirected</code> field that does not accurately reflect whether your use of Amazon Lex is related to a website, program, or other application that is directed or targeted, in whole or in part, to children under age 13 and subject to COPPA. If your use of Amazon Lex relates to a website, program, or other application that is directed in whole or in part, to children under age 13, you must obtain any required verifiable parental consent under COPPA. For information regarding the use of Amazon Lex in connection with websites, programs, or other applications that are directed or targeted, in whole or in part, to children under age 13, see the <a href="https://aws.amazon.com/lex/faqs#data-security">Amazon Lex FAQ.</a> <br/>locale:   The target locale for the bot. <br/>version:  The version of the bot. For a new bot, the version is always <code>$LATEST</code>.<br/>checksum:  Checksum of the bot used to identify a specific revision of the bot's <code>$LATEST</code> version.<br/>voiceId:  The Amazon Polly voice ID that Amazon Lex uses for voice interaction with the user. For more information, see <a>PutBot</a>.<br/>idleSessionTTLInSeconds:  The maximum time in seconds that Amazon Lex retains the data gathered in a conversation. For more information, see <a>PutBot</a>.<br/>createdDate:  The date that the bot was created.<br/>lastUpdatedDate:  The date that the bot was updated. When you create a resource, the creation date and last updated date are the same. <br/>failureReason:  If <code>status</code> is <code>FAILED</code>, Amazon Lex explains why it failed to build the bot.<br/>status:  The status of the bot. If the bot is ready to run, the status is <code>READY</code>. If there was a problem with building the bot, the status is <code>FAILED</code> and the <code>failureReason</code> explains why the bot did not build. If the bot was saved but not built, the status is <code>NOT BUILT</code>.<br/>abortStatement:  The message that Amazon Lex returns when the user elects to end the conversation without completing it. For more information, see <a>PutBot</a>.:<br/>clarificationPrompt:  The message Amazon Lex uses when it doesn't understand the user's request. For more information, see <a>PutBot</a>. :<br/>intents:  An array of <code>intent</code> objects. For more information, see <a>PutBot</a>.<br/>description:  A description of the bot.<br/>name:  The name of the bot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>responseCard</i> A response card. Amazon Lex uses this prompt at runtime, in the <code>PostText</code> API response. It substitutes session attributes and slot values for placeholders in the response card. For more information, see <a>ex-resp-card</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>maxAttempts</i> The number of times to prompt the user for information.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>messages</i> An array of objects, each of which provides a message string and its type. You can specify the message string in plain text or in Speech Synthesis Markup Language (SSML).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>responseCard</i>  At runtime, if the client is using the <a href="http://docs.aws.amazon.com/lex/latest/dg/API_runtime_PostText.html">PostText</a> API, Amazon Lex includes the response card in the response. It substitutes all of the session attributes and slot values for placeholders in the response card. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>messages</i> A collection of message objects.<br/>        """
        pass

    def get_bot_alias(self, botName: str, name: str) -> GetBotAliasResponse:
        """Returns information about an Amazon Lex bot alias. For more information about aliases, see <a>versioning-aliases</a>. This operation requires permissions for the <code>lex:GetBotAlias</code> action.

        :param self:
        :param botName:  The name of the bot.<br/>name:  The name of the bot alias. The name is case sensitive.<br/>
        :return: checksum:  Checksum of the bot alias.<br/>createdDate:  The date that the bot alias was created.<br/>lastUpdatedDate:  The date that the bot alias was updated. When you create a resource, the creation date and the last updated date are the same.<br/>botName:  The name of the bot that the alias points to.<br/>botVersion:  The version of the bot that the alias points to.<br/>description:  A description of the bot alias.<br/>name:  The name of the bot alias.<br/>        """
        pass

    def get_bot_aliases(self, botName: str, nextToken: str=None, maxResults: int=None, nameContains: str=None) -> GetBotAliasesResponse:
        """Returns a list of aliases for a specified Amazon Lex bot. This operation requires permissions for the <code>lex:GetBotAliases</code> action.

        :param self:
        :param nameContains:  Substring to match in bot alias names. An alias will be returned if any part of its name matches the substring. For example, "xyz" matches both "xyzabc" and "abcxyz."<br/>maxResults:  The maximum number of aliases to return in the response. The default is 50. . <br/>nextToken:  A pagination token for fetching the next page of aliases. If the response to this call is truncated, Amazon Lex returns a pagination token in the response. To fetch the next page of aliases, specify the pagination token in the next request. <br/>botName:  The name of the bot.<br/>
        :return: nextToken:  A pagination token for fetching next page of aliases. If the response to this call is truncated, Amazon Lex returns a pagination token in the response. To fetch the next page of aliases, specify the pagination token in the next request. <br/>BotAliases:  An array of <code>BotAliasMetadata</code> objects, each describing a bot alias.<br/>        """
        pass

    def get_bot_channel_association(self, botAlias: str, botName: str, name: str) -> GetBotChannelAssociationResponse:
        """Returns information about the association between an Amazon Lex bot and a messaging platform. This operation requires permissions for the <code>lex:GetBotChannelAssociation</code> action.

        :param self:
        :param botAlias:  An alias pointing to the specific version of the Amazon Lex bot to which this association is being made.<br/>botName:  The name of the Amazon Lex bot.<br/>name:  The name of the association between the bot and the channel. The name is case sensitive. <br/>
        :return: failureReason:  If <code>status</code> is <code>FAILED</code>, Amazon Lex provides the reason that it failed to create the association.<br/>status:  The status of the bot channel.  <ul> <li>  <code>CREATED</code> - The channel has been created and is ready for use. </li> <li>  <code>IN_PROGRESS</code> - Channel creation is in progress. </li> <li>  <code>FAILED</code> - There was an error creating the channel. For information about the reason for the failure, see the <code>failureReason</code> field. </li> </ul><br/>botConfiguration:  Provides information that the messaging platform needs to communicate with the Amazon Lex bot.<br/>type:  The type of the messaging platform.<br/>createdDate:  The date that the association between the bot and the channel was created.<br/>botName:  The name of the Amazon Lex bot.<br/>botAlias:  An alias pointing to the specific version of the Amazon Lex bot to which this association is being made.<br/>description:  A description of the association between the bot and the channel.<br/>name:  The name of the association between the bot and the channel.<br/>        """
        pass

    def get_bot_channel_associations(self, botAlias: str, botName: str, nextToken: str=None, maxResults: int=None, nameContains: str=None) -> GetBotChannelAssociationsResponse:
        """ Returns a list of all of the channels associated with the specified bot.  The <code>GetBotChannelAssociations</code> operation requires permissions for the <code>lex:GetBotChannelAssociations</code> action.

        :param self:
        :param nameContains:  Substring to match in channel association names. An association will be returned if any part of its name matches the substring. For example, "xyz" matches both "xyzabc" and "abcxyz." To return all bot channel associations, use a hyphen ("-") as the <code>nameContains</code> parameter.<br/>maxResults:  The maximum number of associations to return in the response. The default is 50. <br/>nextToken:  A pagination token for fetching the next page of associations. If the response to this call is truncated, Amazon Lex returns a pagination token in the response. To fetch the next page of associations, specify the pagination token in the next request. <br/>botAlias:  An alias pointing to the specific version of the Amazon Lex bot to which this association is being made.<br/>botName:  The name of the Amazon Lex bot in the association.<br/>
        :return: nextToken:  A pagination token that fetches the next page of associations. If the response to this call is truncated, Amazon Lex returns a pagination token in the response. To fetch the next page of associations, specify the pagination token in the next request. <br/>botChannelAssociations:  An array of objects, one for each association, that provides information about the Amazon Lex bot and its association with the channel. <br/>        """
        pass

    def get_bot_versions(self, name: str, nextToken: str=None, maxResults: int=None) -> GetBotVersionsResponse:
        """Gets information about all of the versions of a bot. The <code>GetBotVersions</code> operation returns a <code>BotMetadata</code> object for each version of a bot. For example, if a bot has three numbered versions, the <code>GetBotVersions</code> operation returns four <code>BotMetadata</code> objects in the response, one for each numbered version and one for the <code>$LATEST</code> version.  The <code>GetBotVersions</code> operation always returns at least one version, the <code>$LATEST</code> version. This operation requires permissions for the <code>lex:GetBotVersions</code> action.

        :param self:
        :param maxResults:  The maximum number of bot versions to return in the response. The default is 10.<br/>nextToken:  A pagination token for fetching the next page of bot versions. If the response to this call is truncated, Amazon Lex returns a pagination token in the response. To fetch the next page of versions, specify the pagination token in the next request. <br/>name:  The name of the bot for which versions should be returned.<br/>
        :return: nextToken:  A pagination token for fetching the next page of bot versions. If the response to this call is truncated, Amazon Lex returns a pagination token in the response. To fetch the next page of versions, specify the pagination token in the next request. <br/>bots:  An array of <code>BotMetadata</code> objects, one for each numbered version of the bot plus one for the <code>$LATEST</code> version.<br/>        """
        pass

    def get_bots(self, nextToken: str=None, maxResults: int=None, nameContains: str=None) -> GetBotsResponse:
        """Returns bot information as follows:  <ul> <li> If you provide the <code>nameContains</code> field, the response includes information for the <code>$LATEST</code> version of all bots whose name contains the specified string. </li> <li> If you don't specify the <code>nameContains</code> field, the operation returns information about the <code>$LATEST</code> version of all of your bots. </li> </ul> This operation requires permission for the <code>lex:GetBots</code> action.

        :param self:
        :param nameContains:  Substring to match in bot names. A bot will be returned if any part of its name matches the substring. For example, "xyz" matches both "xyzabc" and "abcxyz."<br/>maxResults:  The maximum number of bots to return in the response that the request will return. The default is 10.<br/>nextToken:  A pagination token that fetches the next page of bots. If the response to this call is truncated, Amazon Lex returns a pagination token in the response. To fetch the next page of bots, specify the pagination token in the next request. <br/>
        :return: nextToken:  If the response is truncated, it includes a pagination token that you can specify in your next request to fetch the next page of bots. <br/>bots:  An array of <code>botMetadata</code> objects, with one entry for each bot. <br/>        """
        pass

    def get_builtin_intent(self, signature: str) -> GetBuiltinIntentResponse:
        """Returns information about a built-in intent. This operation requires permission for the <code>lex:GetBuiltinIntent</code> action.

        :param self:
        :param signature:  The unique identifier for a built-in intent. To find the signature for an intent, see <a href="https://developer.amazon.com/public/solutions/alexa/alexa-skills-kit/docs/built-in-intent-ref/standard-intents">Standard Built-in Intents</a> in the <i>Alexa Skills Kit</i>.<br/>
        :return: slots:  An array of <code>BuiltinIntentSlot</code> objects, one entry for each slot type in the intent.<br/>supportedLocales:  A list of locales that the intent supports.<br/>signature:  The unique identifier for a built-in intent.<br/>        """
        pass

    def get_builtin_intents(self, locale: str=None, signatureContains: str=None, nextToken: str=None, maxResults: int=None) -> GetBuiltinIntentsResponse:
        """Gets a list of built-in intents that meet the specified criteria. This operation requires permission for the <code>lex:GetBuiltinIntents</code> action.

        :param self:
        :param maxResults:  The maximum number of intents to return in the response. The default is 10.<br/>nextToken:  A pagination token that fetches the next page of intents. If this API call is truncated, Amazon Lex returns a pagination token in the response. To fetch the next page of intents, use the pagination token in the next request.<br/>signatureContains:  Substring to match in built-in intent signatures. An intent will be returned if any part of its signature matches the substring. For example, "xyz" matches both "xyzabc" and "abcxyz." To find the signature for an intent, see <a href="https://developer.amazon.com/public/solutions/alexa/alexa-skills-kit/docs/built-in-intent-ref/standard-intents">Standard Built-in Intents</a> in the <i>Alexa Skills Kit</i>.<br/>locale:  A list of locales that the intent supports.<br/>
        :return: nextToken:  A pagination token that fetches the next page of intents. If the response to this API call is truncated, Amazon Lex returns a pagination token in the response. To fetch the next page of intents, specify the pagination token in the next request.<br/>intents:  An array of <code>builtinIntentMetadata</code> objects, one for each intent in the response.<br/>        """
        pass

    def get_builtin_slot_types(self, locale: str=None, signatureContains: str=None, nextToken: str=None, maxResults: int=None) -> GetBuiltinSlotTypesResponse:
        """Gets a list of built-in slot types that meet the specified criteria. For a list of built-in slot types, see <a href="https://developer.amazon.com/public/solutions/alexa/alexa-skills-kit/docs/built-in-intent-ref/slot-type-reference">Slot Type Reference</a> in the <i>Alexa Skills Kit</i>. This operation requires permission for the <code>lex:GetBuiltInSlotTypes</code> action.

        :param self:
        :param maxResults:  The maximum number of slot types to return in the response. The default is 10.<br/>nextToken:  A pagination token that fetches the next page of slot types. If the response to this API call is truncated, Amazon Lex returns a pagination token in the response. To fetch the next page of slot types, specify the pagination token in the next request.<br/>signatureContains:  Substring to match in built-in slot type signatures. A slot type will be returned if any part of its signature matches the substring. For example, "xyz" matches both "xyzabc" and "abcxyz."<br/>locale:  A list of locales that the slot type supports.<br/>
        :return: nextToken:  If the response is truncated, the response includes a pagination token that you can use in your next request to fetch the next page of slot types.<br/>slotTypes:  An array of <code>BuiltInSlotTypeMetadata</code> objects, one entry for each slot type returned.<br/>        """
        pass

    def get_export(self, exportType: str, resourceType: str, version: str, name: str) -> GetExportResponse:
        """Exports the contents of a Amazon Lex resource in a specified format. 

        :param self:
        :param exportType:  The format of the exported data.<br/>resourceType:  The type of resource to export. <br/>version:  The version of the bot to export.<br/>name:  The name of the bot to export.<br/>
        :return: url:  An S3 pre-signed URL that provides the location of the exported resource. The exported resource is a ZIP archive that contains the exported resource in JSON format. The structure of the archive may change. Your code should not rely on the archive structure.<br/>failureReason:  If <code>status</code> is <code>FAILED</code>, Amazon Lex provides the reason that it failed to export the resource.<br/>exportStatus:  The status of the export.  <ul> <li>  <code>IN_PROGRESS</code> - The export is in progress. </li> <li>  <code>READY</code> - The export is complete. </li> <li>  <code>FAILED</code> - The export could not be completed. </li> </ul><br/>exportType:  The format of the exported data.<br/>resourceType:  The type of the exported resource.<br/>version:  The version of the bot being exported.<br/>name:  The name of the bot being exported.<br/>        """
        pass

    def get_import(self, importId: str) -> GetImportResponse:
        """Gets information about an import job started with the <code>StartImport</code> operation.

        :param self:
        :param importId:  The identifier of the import job information to return.<br/>
        :return: createdDate:  A timestamp for the date and time that the import job was created.<br/>failureReason:  A string that describes why an import job failed to complete.<br/>importStatus:  The status of the import job. If the status is <code>FAILED</code>, you can get the reason for the failure from the <code>failureReason</code> field.<br/>importId:  The identifier for the specific import job.<br/>mergeStrategy:  The action taken when there was a conflict between an existing resource and a resource in the import file.<br/>resourceType:  The type of resource imported.<br/>name:  The name given to the import job.<br/>        """
        pass

    def get_intent(self, version: str, name: str) -> GetIntentResponse:
        """ Returns information about an intent. In addition to the intent name, you must specify the intent version.   This operation requires permissions to perform the <code>lex:GetIntent</code> action. 

        :param self:
        :param version:  The version of the intent.<br/>name:  The name of the intent. The name is case sensitive. <br/>
        :return: checksum:  Checksum of the intent.<br/>version:  The version of the intent.<br/>createdDate:  The date that the intent was created.<br/>lastUpdatedDate:  The date that the intent was updated. When you create a resource, the creation date and the last updated date are the same. <br/>parentIntentSignature:  A unique identifier for a built-in intent.<br/>fulfillmentActivity:  Describes how the intent is fulfilled. For more information, see <a>PutIntent</a>. :<br/>dialogCodeHook:  If defined in the bot, Amazon Amazon Lex invokes this Lambda function for each user input. For more information, see <a>PutIntent</a>. :<br/>conclusionStatement:  After the Lambda function specified in the <code>fulfillmentActivity</code> element fulfills the intent, Amazon Lex conveys this statement to the user.:<br/>followUpPrompt:  If defined in the bot, Amazon Lex uses this prompt to solicit additional user activity after the intent is fulfilled. For more information, see <a>PutIntent</a>.:<br/>rejectionStatement:  If the user answers "no" to the question defined in <code>confirmationPrompt</code>, Amazon Lex responds with this statement to acknowledge that the intent was canceled. :<br/>confirmationPrompt:  If defined in the bot, Amazon Lex uses prompt to confirm the intent before fulfilling the user's request. For more information, see <a>PutIntent</a>. :<br/>sampleUtterances:  An array of sample utterances configured for the intent.<br/>slots:  An array of intent slots configured for the intent.<br/>description:  A description of the intent.<br/>name:  The name of the intent.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>responseCard</i> A response card. Amazon Lex uses this prompt at runtime, in the <code>PostText</code> API response. It substitutes session attributes and slot values for placeholders in the response card. For more information, see <a>ex-resp-card</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>maxAttempts</i> The number of times to prompt the user for information.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>messages</i> An array of objects, each of which provides a message string and its type. You can specify the message string in plain text or in Speech Synthesis Markup Language (SSML).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>responseCard</i>  At runtime, if the client is using the <a href="http://docs.aws.amazon.com/lex/latest/dg/API_runtime_PostText.html">PostText</a> API, Amazon Lex includes the response card in the response. It substitutes all of the session attributes and slot values for placeholders in the response card. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>messages</i> A collection of message objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>rejectionStatement</i> If the user answers "no" to the question defined in the <code>prompt</code> field, Amazon Lex responds with this statement to acknowledge that the intent was canceled. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>rejectionStatement</b>: If the user answers "no" to the question defined in the <code>prompt</code> field, Amazon Lex responds with this statement to acknowledge that the intent was canceled. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>prompt</i> Prompts for information from the user. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>prompt</b>: Prompts for information from the user. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>responseCard</i> A response card. Amazon Lex uses this prompt at runtime, in the <code>PostText</code> API response. It substitutes session attributes and slot values for placeholders in the response card. For more information, see <a>ex-resp-card</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>maxAttempts</i> The number of times to prompt the user for information.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>messages</i> An array of objects, each of which provides a message string and its type. You can specify the message string in plain text or in Speech Synthesis Markup Language (SSML).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>responseCard</i>  At runtime, if the client is using the <a href="http://docs.aws.amazon.com/lex/latest/dg/API_runtime_PostText.html">PostText</a> API, Amazon Lex includes the response card in the response. It substitutes all of the session attributes and slot values for placeholders in the response card. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>messages</i> A collection of message objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>responseCard</i>  At runtime, if the client is using the <a href="http://docs.aws.amazon.com/lex/latest/dg/API_runtime_PostText.html">PostText</a> API, Amazon Lex includes the response card in the response. It substitutes all of the session attributes and slot values for placeholders in the response card. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>messages</i> A collection of message objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>messageVersion</i> The version of the request-response that you want Amazon Lex to use to invoke your Lambda function. For more information, see <a>using-lambda</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>uri</i> The Amazon Resource Name (ARN) of the Lambda function.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>codeHook</i>  A description of the Lambda function that is run to fulfill the intent. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>codeHook</b>:  A description of the Lambda function that is run to fulfill the intent. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i>  How the intent should be fulfilled, either by running a Lambda function or by returning the slot data to the client application. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>messageVersion</i> The version of the request-response that you want Amazon Lex to use to invoke your Lambda function. For more information, see <a>using-lambda</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uri</i> The Amazon Resource Name (ARN) of the Lambda function.<br/>        """
        pass

    def get_intent_versions(self, name: str, nextToken: str=None, maxResults: int=None) -> GetIntentVersionsResponse:
        """Gets information about all of the versions of an intent. The <code>GetIntentVersions</code> operation returns an <code>IntentMetadata</code> object for each version of an intent. For example, if an intent has three numbered versions, the <code>GetIntentVersions</code> operation returns four <code>IntentMetadata</code> objects in the response, one for each numbered version and one for the <code>$LATEST</code> version.  The <code>GetIntentVersions</code> operation always returns at least one version, the <code>$LATEST</code> version. This operation requires permissions for the <code>lex:GetIntentVersions</code> action.

        :param self:
        :param maxResults:  The maximum number of intent versions to return in the response. The default is 10.<br/>nextToken:  A pagination token for fetching the next page of intent versions. If the response to this call is truncated, Amazon Lex returns a pagination token in the response. To fetch the next page of versions, specify the pagination token in the next request. <br/>name:  The name of the intent for which versions should be returned.<br/>
        :return: nextToken:  A pagination token for fetching the next page of intent versions. If the response to this call is truncated, Amazon Lex returns a pagination token in the response. To fetch the next page of versions, specify the pagination token in the next request. <br/>intents:  An array of <code>IntentMetadata</code> objects, one for each numbered version of the intent plus one for the <code>$LATEST</code> version.<br/>        """
        pass

    def get_intents(self, nextToken: str=None, maxResults: int=None, nameContains: str=None) -> GetIntentsResponse:
        """Returns intent information as follows:  <ul> <li> If you specify the <code>nameContains</code> field, returns the <code>$LATEST</code> version of all intents that contain the specified string. </li> <li>  If you don't specify the <code>nameContains</code> field, returns information about the <code>$LATEST</code> version of all intents.  </li> </ul>  The operation requires permission for the <code>lex:GetIntents</code> action. 

        :param self:
        :param nameContains:  Substring to match in intent names. An intent will be returned if any part of its name matches the substring. For example, "xyz" matches both "xyzabc" and "abcxyz."<br/>maxResults:  The maximum number of intents to return in the response. The default is 10.<br/>nextToken:  A pagination token that fetches the next page of intents. If the response to this API call is truncated, Amazon Lex returns a pagination token in the response. To fetch the next page of intents, specify the pagination token in the next request. <br/>
        :return: nextToken:  If the response is truncated, the response includes a pagination token that you can specify in your next request to fetch the next page of intents. <br/>intents:  An array of <code>Intent</code> objects. For more information, see <a>PutBot</a>.<br/>        """
        pass

    def get_slot_type(self, version: str, name: str) -> GetSlotTypeResponse:
        """Returns information about a specific version of a slot type. In addition to specifying the slot type name, you must specify the slot type version. This operation requires permissions for the <code>lex:GetSlotType</code> action.

        :param self:
        :param version:  The version of the slot type. <br/>name:  The name of the slot type. The name is case sensitive. <br/>
        :return: valueSelectionStrategy:  The strategy that Amazon Lex uses to determine the value of the slot. For more information, see <a>PutSlotType</a>.<br/>checksum:  Checksum of the <code>$LATEST</code> version of the slot type.<br/>version:  The version of the slot type.<br/>createdDate:  The date that the slot type was created.<br/>lastUpdatedDate:  The date that the slot type was updated. When you create a resource, the creation date and last update date are the same.<br/>enumerationValues:  A list of <code>EnumerationValue</code> objects that defines the values that the slot type can take.<br/>description:  A description of the slot type.<br/>name:  The name of the slot type.<br/>        """
        pass

    def get_slot_type_versions(self, name: str, nextToken: str=None, maxResults: int=None) -> GetSlotTypeVersionsResponse:
        """Gets information about all versions of a slot type. The <code>GetSlotTypeVersions</code> operation returns a <code>SlotTypeMetadata</code> object for each version of a slot type. For example, if a slot type has three numbered versions, the <code>GetSlotTypeVersions</code> operation returns four <code>SlotTypeMetadata</code> objects in the response, one for each numbered version and one for the <code>$LATEST</code> version.  The <code>GetSlotTypeVersions</code> operation always returns at least one version, the <code>$LATEST</code> version. This operation requires permissions for the <code>lex:GetSlotTypeVersions</code> action.

        :param self:
        :param maxResults:  The maximum number of slot type versions to return in the response. The default is 10.<br/>nextToken:  A pagination token for fetching the next page of slot type versions. If the response to this call is truncated, Amazon Lex returns a pagination token in the response. To fetch the next page of versions, specify the pagination token in the next request. <br/>name:  The name of the slot type for which versions should be returned.<br/>
        :return: nextToken:  A pagination token for fetching the next page of slot type versions. If the response to this call is truncated, Amazon Lex returns a pagination token in the response. To fetch the next page of versions, specify the pagination token in the next request. <br/>slotTypes:  An array of <code>SlotTypeMetadata</code> objects, one for each numbered version of the slot type plus one for the <code>$LATEST</code> version.<br/>        """
        pass

    def get_slot_types(self, nextToken: str=None, maxResults: int=None, nameContains: str=None) -> GetSlotTypesResponse:
        """Returns slot type information as follows:  <ul> <li> If you specify the <code>nameContains</code> field, returns the <code>$LATEST</code> version of all slot types that contain the specified string. </li> <li>  If you don't specify the <code>nameContains</code> field, returns information about the <code>$LATEST</code> version of all slot types.  </li> </ul>  The operation requires permission for the <code>lex:GetSlotTypes</code> action. 

        :param self:
        :param nameContains:  Substring to match in slot type names. A slot type will be returned if any part of its name matches the substring. For example, "xyz" matches both "xyzabc" and "abcxyz."<br/>maxResults:  The maximum number of slot types to return in the response. The default is 10.<br/>nextToken:  A pagination token that fetches the next page of slot types. If the response to this API call is truncated, Amazon Lex returns a pagination token in the response. To fetch next page of slot types, specify the pagination token in the next request.<br/>
        :return: nextToken:  If the response is truncated, it includes a pagination token that you can specify in your next request to fetch the next page of slot types.<br/>slotTypes:  An array of objects, one for each slot type, that provides information such as the name of the slot type, the version, and a description.<br/>        """
        pass

    def get_utterances_view(self, statusType: str, botVersions: List[Version], botName: str) -> GetUtterancesViewResponse:
        """Use the <code>GetUtterancesView</code> operation to get information about the utterances that your users have made to your bot. You can use this list to tune the utterances that your bot responds to. For example, say that you have created a bot to order flowers. After your users have used your bot for a while, use the <code>GetUtterancesView</code> operation to see the requests that they have made and whether they have been successful. You might find that the utterance "I want flowers" is not being recognized. You could add this utterance to the <code>OrderFlowers</code> intent so that your bot recognizes that utterance. After you publish a new version of a bot, you can get information about the old version and the new so that you can compare the performance across the two versions.  <note> Utterance statistics are generated once a day. Data is available for the last 15 days. You can request information for up to 5 versions in each request. The response contains information about a maximum of 100 utterances for each version. </note> This operation requires permissions for the <code>lex:GetUtterancesView</code> action.

        :param self:
        :param statusType:  To return utterances that were recognized and handled, use<code>Detected</code>. To return utterances that were not recognized, use <code>Missed</code>.<br/>botVersions:  An array of bot versions for which utterance information should be returned. The limit is 5 versions per request.<br/>botName:  The name of the bot for which utterance information should be returned.<br/>
        :return: utterances:  An array of <a>UtteranceList</a> objects, each containing a list of <a>UtteranceData</a> objects describing the utterances that were processed by your bot. The response contains a maximum of 100 <code>UtteranceData</code> objects for each version.<br/>botName:  The name of the bot for which utterance information was returned.<br/>        """
        pass

    def put_bot(self, childDirected: Boolean, locale: str, name: str, description: str=None, intents: List[Intent]=None, clarificationPrompt: Prompt=None, abortStatement: Statement=None, idleSessionTTLInSeconds: int=None, voiceId: str=None, checksum: str=None, processBehavior: str=None, createVersion: Boolean=None) -> PutBotResponse:
        """Creates an Amazon Lex conversational bot or replaces an existing bot. When you create or update a bot you are only required to specify a name, a locale, and whether the bot is directed toward children under age 13. You can use this to add intents later, or to remove intents from an existing bot. When you create a bot with the minimum information, the bot is created or updated but Amazon Lex returns the <code/> response <code>FAILED</code>. You can build the bot after you add one or more intents. For more information about Amazon Lex bots, see <a>how-it-works</a>.  If you specify the name of an existing bot, the fields in the request replace the existing values in the <code>$LATEST</code> version of the bot. Amazon Lex removes any fields that you don't provide values for in the request, except for the <code>idleTTLInSeconds</code> and <code>privacySettings</code> fields, which are set to their default values. If you don't specify values for required fields, Amazon Lex throws an exception. This operation requires permissions for the <code>lex:PutBot</code> action. For more information, see <a>auth-and-access-control</a>.

        :param self:
        :param createVersion:  <br/>childDirected:  For each Amazon Lex bot created with the Amazon Lex Model Building Service, you must specify whether your use of Amazon Lex is related to a website, program, or other application that is directed or targeted, in whole or in part, to children under age 13 and subject to the Children's Online Privacy Protection Act (COPPA) by specifying <code>true</code> or <code>false</code> in the <code>childDirected</code> field. By specifying <code>true</code> in the <code>childDirected</code> field, you confirm that your use of Amazon Lex <b>is</b> related to a website, program, or other application that is directed or targeted, in whole or in part, to children under age 13 and subject to COPPA. By specifying <code>false</code> in the <code>childDirected</code> field, you confirm that your use of Amazon Lex <b>is not</b> related to a website, program, or other application that is directed or targeted, in whole or in part, to children under age 13 and subject to COPPA. You may not specify a default value for the <code>childDirected</code> field that does not accurately reflect whether your use of Amazon Lex is related to a website, program, or other application that is directed or targeted, in whole or in part, to children under age 13 and subject to COPPA. If your use of Amazon Lex relates to a website, program, or other application that is directed in whole or in part, to children under age 13, you must obtain any required verifiable parental consent under COPPA. For information regarding the use of Amazon Lex in connection with websites, programs, or other applications that are directed or targeted, in whole or in part, to children under age 13, see the <a href="https://aws.amazon.com/lex/faqs#data-security">Amazon Lex FAQ.</a> <br/>locale:   Specifies the target locale for the bot. Any intent used in the bot must be compatible with the locale of the bot.  The default is <code>en-US</code>.<br/>processBehavior:  If you set the <code>processBehavior</code> element to <code>BUILD</code>, Amazon Lex builds the bot so that it can be run. If you set the element to <code>SAVE</code> Amazon Lex saves the bot, but doesn't build it.  If you don't specify this value, the default value is <code>BUILD</code>.<br/>checksum:  Identifies a specific revision of the <code>$LATEST</code> version. When you create a new bot, leave the <code>checksum</code> field blank. If you specify a checksum you get a <code>BadRequestException</code> exception. When you want to update a bot, set the <code>checksum</code> field to the checksum of the most recent revision of the <code>$LATEST</code> version. If you don't specify the <code> checksum</code> field, or if the checksum does not match the <code>$LATEST</code> version, you get a <code>PreconditionFailedException</code> exception.<br/>voiceId:  The Amazon Polly voice ID that you want Amazon Lex to use for voice interactions with the user. The locale configured for the voice must match the locale of the bot. For more information, see <a href="http://docs.aws.amazon.com/polly/latest/dg/voicelist.html">Available Voices</a> in the <i>Amazon Polly Developer Guide</i>.<br/>idleSessionTTLInSeconds:  The maximum time in seconds that Amazon Lex retains the data gathered in a conversation. A user interaction session remains active for the amount of time specified. If no conversation occurs during this time, the session expires and Amazon Lex deletes any data provided before the timeout. For example, suppose that a user chooses the OrderPizza intent, but gets sidetracked halfway through placing an order. If the user doesn't complete the order within the specified time, Amazon Lex discards the slot information that it gathered, and the user must start over. If you don't include the <code>idleSessionTTLInSeconds</code> element in a <code>PutBot</code> operation request, Amazon Lex uses the default value. This is also true if the request replaces an existing bot. The default is 300 seconds (5 minutes).<br/>abortStatement:  When Amazon Lex can't understand the user's input in context, it tries to elicit the information a few times. After that, Amazon Lex sends the message defined in <code>abortStatement</code> to the user, and then aborts the conversation. To set the number of retries, use the <code>valueElicitationPrompt</code> field for the slot type.  For example, in a pizza ordering bot, Amazon Lex might ask a user "What type of crust would you like?" If the user's response is not one of the expected responses (for example, "thin crust, "deep dish," etc.), Amazon Lex tries to elicit a correct response a few more times.  For example, in a pizza ordering application, <code>OrderPizza</code> might be one of the intents. This intent might require the <code>CrustType</code> slot. You specify the <code>valueElicitationPrompt</code> field when you create the <code>CrustType</code> slot.:<br/>clarificationPrompt:  When Amazon Lex doesn't understand the user's intent, it uses this message to get clarification. To specify how many times Amazon Lex should repeate the clarification prompt, use the <code>maxAttempts</code> field. If Amazon Lex still doesn't understand, it sends the message in the <code>abortStatement</code> field.  When you create a clarification prompt, make sure that it suggests the correct response from the user. for example, for a bot that orders pizza and drinks, you might create this clarification prompt: "What would you like to do? You can say 'Order a pizza' or 'Order a drink.'":<br/>intents:  An array of <code>Intent</code> objects. Each intent represents a command that a user can express. For example, a pizza ordering bot might support an OrderPizza intent. For more information, see <a>how-it-works</a>.<br/>description:  A description of the bot.<br/>name:  The name of the bot. The name is <i>not</i> case sensitive. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>responseCard</i> A response card. Amazon Lex uses this prompt at runtime, in the <code>PostText</code> API response. It substitutes session attributes and slot values for placeholders in the response card. For more information, see <a>ex-resp-card</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>maxAttempts</i> The number of times to prompt the user for information.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>messages</i> An array of objects, each of which provides a message string and its type. You can specify the message string in plain text or in Speech Synthesis Markup Language (SSML).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>responseCard</i>  At runtime, if the client is using the <a href="http://docs.aws.amazon.com/lex/latest/dg/API_runtime_PostText.html">PostText</a> API, Amazon Lex includes the response card in the response. It substitutes all of the session attributes and slot values for placeholders in the response card. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>messages</i> A collection of message objects.<br/>
        :return: createVersion:  <br/>childDirected:  For each Amazon Lex bot created with the Amazon Lex Model Building Service, you must specify whether your use of Amazon Lex is related to a website, program, or other application that is directed or targeted, in whole or in part, to children under age 13 and subject to the Children's Online Privacy Protection Act (COPPA) by specifying <code>true</code> or <code>false</code> in the <code>childDirected</code> field. By specifying <code>true</code> in the <code>childDirected</code> field, you confirm that your use of Amazon Lex <b>is</b> related to a website, program, or other application that is directed or targeted, in whole or in part, to children under age 13 and subject to COPPA. By specifying <code>false</code> in the <code>childDirected</code> field, you confirm that your use of Amazon Lex <b>is not</b> related to a website, program, or other application that is directed or targeted, in whole or in part, to children under age 13 and subject to COPPA. You may not specify a default value for the <code>childDirected</code> field that does not accurately reflect whether your use of Amazon Lex is related to a website, program, or other application that is directed or targeted, in whole or in part, to children under age 13 and subject to COPPA. If your use of Amazon Lex relates to a website, program, or other application that is directed in whole or in part, to children under age 13, you must obtain any required verifiable parental consent under COPPA. For information regarding the use of Amazon Lex in connection with websites, programs, or other applications that are directed or targeted, in whole or in part, to children under age 13, see the <a href="https://aws.amazon.com/lex/faqs#data-security">Amazon Lex FAQ.</a> <br/>locale:   The target locale for the bot. <br/>version:  The version of the bot. For a new bot, the version is always <code>$LATEST</code>.<br/>checksum:  Checksum of the bot that you created.<br/>voiceId:  The Amazon Polly voice ID that Amazon Lex uses for voice interaction with the user. For more information, see <a>PutBot</a>.<br/>idleSessionTTLInSeconds:  The maximum length of time that Amazon Lex retains the data gathered in a conversation. For more information, see <a>PutBot</a>.<br/>createdDate:  The date that the bot was created.<br/>lastUpdatedDate:  The date that the bot was updated. When you create a resource, the creation date and last updated date are the same.<br/>failureReason:  If <code>status</code> is <code>FAILED</code>, Amazon Lex provides the reason that it failed to build the bot.<br/>status:   When you send a request to create a bot with <code>processBehavior</code> set to <code>BUILD</code>, Amazon Lex sets the <code>status</code> response element to <code>BUILDING</code>. After Amazon Lex builds the bot, it sets <code>status</code> to <code>READY</code>. If Amazon Lex can't build the bot, Amazon Lex sets <code>status</code> to <code>FAILED</code>. Amazon Lex returns the reason for the failure in the <code>failureReason</code> response element.  When you set <code>processBehavior</code>to <code>SAVE</code>, Amazon Lex sets the status code to <code>NOT BUILT</code>.<br/>abortStatement:  The message that Amazon Lex uses to abort a conversation. For more information, see <a>PutBot</a>.:<br/>clarificationPrompt:   The prompts that Amazon Lex uses when it doesn't understand the user's intent. For more information, see <a>PutBot</a>. :<br/>intents:  An array of <code>Intent</code> objects. For more information, see <a>PutBot</a>.<br/>description:  A description of the bot.<br/>name:  The name of the bot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>responseCard</i> A response card. Amazon Lex uses this prompt at runtime, in the <code>PostText</code> API response. It substitutes session attributes and slot values for placeholders in the response card. For more information, see <a>ex-resp-card</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>maxAttempts</i> The number of times to prompt the user for information.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>messages</i> An array of objects, each of which provides a message string and its type. You can specify the message string in plain text or in Speech Synthesis Markup Language (SSML).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>responseCard</i>  At runtime, if the client is using the <a href="http://docs.aws.amazon.com/lex/latest/dg/API_runtime_PostText.html">PostText</a> API, Amazon Lex includes the response card in the response. It substitutes all of the session attributes and slot values for placeholders in the response card. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>messages</i> A collection of message objects.<br/>        """
        pass

    def put_bot_alias(self, botName: str, botVersion: str, name: str, description: str=None, checksum: str=None) -> PutBotAliasResponse:
        """Creates an alias for the specified version of the bot or replaces an alias for the specified bot. To change the version of the bot that the alias points to, replace the alias. For more information about aliases, see <a>versioning-aliases</a>. This operation requires permissions for the <code>lex:PutBotAlias</code> action. 

        :param self:
        :param checksum:  Identifies a specific revision of the <code>$LATEST</code> version. When you create a new bot alias, leave the <code>checksum</code> field blank. If you specify a checksum you get a <code>BadRequestException</code> exception. When you want to update a bot alias, set the <code>checksum</code> field to the checksum of the most recent revision of the <code>$LATEST</code> version. If you don't specify the <code> checksum</code> field, or if the checksum does not match the <code>$LATEST</code> version, you get a <code>PreconditionFailedException</code> exception.<br/>botName:  The name of the bot.<br/>botVersion:  The version of the bot.<br/>description:  A description of the alias.<br/>name:  The name of the alias. The name is <i>not</i> case sensitive.<br/>
        :return: checksum:  The checksum for the current version of the alias.<br/>createdDate:  The date that the bot alias was created.<br/>lastUpdatedDate:  The date that the bot alias was updated. When you create a resource, the creation date and the last updated date are the same.<br/>botName:  The name of the bot that the alias points to.<br/>botVersion:  The version of the bot that the alias points to.<br/>description:  A description of the alias.<br/>name:  The name of the alias.<br/>        """
        pass

    def put_intent(self, name: str, description: str=None, slots: List[Slot]=None, sampleUtterances: List[Utterance]=None, confirmationPrompt: Prompt=None, rejectionStatement: Statement=None, followUpPrompt: FollowUpPrompt=None, conclusionStatement: Statement=None, dialogCodeHook: CodeHook=None, fulfillmentActivity: FulfillmentActivity=None, parentIntentSignature: str=None, checksum: str=None, createVersion: Boolean=None) -> PutIntentResponse:
        """Creates an intent or replaces an existing intent. To define the interaction between the user and your bot, you use one or more intents. For a pizza ordering bot, for example, you would create an <code>OrderPizza</code> intent.  To create an intent or replace an existing intent, you must provide the following: <ul> <li> Intent name. For example, <code>OrderPizza</code>. </li> <li> Sample utterances. For example, "Can I order a pizza, please." and "I want to order a pizza." </li> <li> Information to be gathered. You specify slot types for the information that your bot will request from the user. You can specify standard slot types, such as a date or a time, or custom slot types such as the size and crust of a pizza. </li> <li> How the intent will be fulfilled. You can provide a Lambda function or configure the intent to return the intent information to the client application. If you use a Lambda function, when all of the intent information is available, Amazon Lex invokes your Lambda function. If you configure your intent to return the intent information to the client application.  </li> </ul> You can specify other optional information in the request, such as: <ul> <li> A confirmation prompt to ask the user to confirm an intent. For example, "Shall I order your pizza?" </li> <li> A conclusion statement to send to the user after the intent has been fulfilled. For example, "I placed your pizza order." </li> <li> A follow-up prompt that asks the user for additional activity. For example, asking "Do you want to order a drink with your pizza?" </li> </ul> If you specify an existing intent name to update the intent, Amazon Lex replaces the values in the <code>$LATEST</code> version of the intent with the values in the request. Amazon Lex removes fields that you don't provide in the request. If you don't specify the required fields, Amazon Lex throws an exception. When you update the <code>$LATEST</code> version of an intent, the <code>status</code> field of any bot that uses the <code>$LATEST</code> version of the intent is set to <code>NOT_BUILT</code>. For more information, see <a>how-it-works</a>. This operation requires permissions for the <code>lex:PutIntent</code> action.

        :param self:
        :param createVersion:  <br/>checksum:  Identifies a specific revision of the <code>$LATEST</code> version. When you create a new intent, leave the <code>checksum</code> field blank. If you specify a checksum you get a <code>BadRequestException</code> exception. When you want to update a intent, set the <code>checksum</code> field to the checksum of the most recent revision of the <code>$LATEST</code> version. If you don't specify the <code> checksum</code> field, or if the checksum does not match the <code>$LATEST</code> version, you get a <code>PreconditionFailedException</code> exception.<br/>parentIntentSignature:  A unique identifier for the built-in intent to base this intent on. To find the signature for an intent, see <a href="https://developer.amazon.com/public/solutions/alexa/alexa-skills-kit/docs/built-in-intent-ref/standard-intents">Standard Built-in Intents</a> in the <i>Alexa Skills Kit</i>.<br/>fulfillmentActivity:  Required. Describes how the intent is fulfilled. For example, after a user provides all of the information for a pizza order, <code>fulfillmentActivity</code> defines how the bot places an order with a local pizza store.   You might configure Amazon Lex to return all of the intent information to the client application, or direct it to invoke a Lambda function that can process the intent (for example, place an order with a pizzeria). :<br/>dialogCodeHook:   Specifies a Lambda function to invoke for each user input. You can invoke this Lambda function to personalize user interaction.  For example, suppose your bot determines that the user is John. Your Lambda function might retrieve John's information from a backend database and prepopulate some of the values. For example, if you find that John is gluten intolerant, you might set the corresponding intent slot, <code>GlutenIntolerant</code>, to true. You might find John's phone number and set the corresponding session attribute. :<br/>conclusionStatement:   The statement that you want Amazon Lex to convey to the user after the intent is successfully fulfilled by the Lambda function.  This element is relevant only if you provide a Lambda function in the <code>fulfillmentActivity</code>. If you return the intent to the client application, you can't specify this element. <note> The <code>followUpPrompt</code> and <code>conclusionStatement</code> are mutually exclusive. You can specify only one. </note>:<br/>followUpPrompt:  Amazon Lex uses this prompt to solicit additional activity after fulfilling an intent. For example, after the <code>OrderPizza</code> intent is fulfilled, you might prompt the user to order a drink. The action that Amazon Lex takes depends on the user's response, as follows: <ul> <li> If the user says "Yes" it responds with the clarification prompt that is configured for the bot. </li> <li> if the user says "Yes" and continues with an utterance that triggers an intent it starts a conversation for the intent. </li> <li> If the user says "No" it responds with the rejection statement configured for the the follow-up prompt. </li> <li> If it doesn't recognize the utterance it repeats the follow-up prompt again. </li> </ul> The <code>followUpPrompt</code> field and the <code>conclusionStatement</code> field are mutually exclusive. You can specify only one. :<br/>rejectionStatement:  When the user answers "no" to the question defined in <code>confirmationPrompt</code>, Amazon Lex responds with this statement to acknowledge that the intent was canceled.  <note> You must provide both the <code>rejectionStatement</code> and the <code>confirmationPrompt</code>, or neither. </note>:<br/>confirmationPrompt:  Prompts the user to confirm the intent. This question should have a yes or no answer. Amazon Lex uses this prompt to ensure that the user acknowledges that the intent is ready for fulfillment. For example, with the <code>OrderPizza</code> intent, you might want to confirm that the order is correct before placing it. For other intents, such as intents that simply respond to user questions, you might not need to ask the user for confirmation before providing the information.  <note> You you must provide both the <code>rejectionStatement</code> and the <code>confirmationPrompt</code>, or neither. </note>:<br/>sampleUtterances:  An array of utterances (strings) that a user might say to signal the intent. For example, "I want {PizzaSize} pizza", "Order {Quantity} {PizzaSize} pizzas".  In each utterance, a slot name is enclosed in curly braces. <br/>slots:  An array of intent slots. At runtime, Amazon Lex elicits required slot values from the user using prompts defined in the slots. For more information, see <a>how-it-works</a>. <br/>description:  A description of the intent.<br/>name:  The name of the intent. The name is <i>not</i> case sensitive.  The name can't match a built-in intent name, or a built-in intent name with "AMAZON." removed. For example, because there is a built-in intent called <code>AMAZON.HelpIntent</code>, you can't create a custom intent called <code>HelpIntent</code>. For a list of built-in intents, see <a href="https://developer.amazon.com/public/solutions/alexa/alexa-skills-kit/docs/built-in-intent-ref/standard-intents">Standard Built-in Intents</a> in the <i>Alexa Skills Kit</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>responseCard</i> A response card. Amazon Lex uses this prompt at runtime, in the <code>PostText</code> API response. It substitutes session attributes and slot values for placeholders in the response card. For more information, see <a>ex-resp-card</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>maxAttempts</i> The number of times to prompt the user for information.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>messages</i> An array of objects, each of which provides a message string and its type. You can specify the message string in plain text or in Speech Synthesis Markup Language (SSML).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>responseCard</i>  At runtime, if the client is using the <a href="http://docs.aws.amazon.com/lex/latest/dg/API_runtime_PostText.html">PostText</a> API, Amazon Lex includes the response card in the response. It substitutes all of the session attributes and slot values for placeholders in the response card. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>messages</i> A collection of message objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>rejectionStatement</i> If the user answers "no" to the question defined in the <code>prompt</code> field, Amazon Lex responds with this statement to acknowledge that the intent was canceled. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>rejectionStatement</b>: If the user answers "no" to the question defined in the <code>prompt</code> field, Amazon Lex responds with this statement to acknowledge that the intent was canceled. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>prompt</i> Prompts for information from the user. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>prompt</b>: Prompts for information from the user. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>responseCard</i> A response card. Amazon Lex uses this prompt at runtime, in the <code>PostText</code> API response. It substitutes session attributes and slot values for placeholders in the response card. For more information, see <a>ex-resp-card</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>maxAttempts</i> The number of times to prompt the user for information.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>messages</i> An array of objects, each of which provides a message string and its type. You can specify the message string in plain text or in Speech Synthesis Markup Language (SSML).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>responseCard</i>  At runtime, if the client is using the <a href="http://docs.aws.amazon.com/lex/latest/dg/API_runtime_PostText.html">PostText</a> API, Amazon Lex includes the response card in the response. It substitutes all of the session attributes and slot values for placeholders in the response card. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>messages</i> A collection of message objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>responseCard</i>  At runtime, if the client is using the <a href="http://docs.aws.amazon.com/lex/latest/dg/API_runtime_PostText.html">PostText</a> API, Amazon Lex includes the response card in the response. It substitutes all of the session attributes and slot values for placeholders in the response card. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>messages</i> A collection of message objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>messageVersion</i> The version of the request-response that you want Amazon Lex to use to invoke your Lambda function. For more information, see <a>using-lambda</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>uri</i> The Amazon Resource Name (ARN) of the Lambda function.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>codeHook</i>  A description of the Lambda function that is run to fulfill the intent. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>codeHook</b>:  A description of the Lambda function that is run to fulfill the intent. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i>  How the intent should be fulfilled, either by running a Lambda function or by returning the slot data to the client application. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>messageVersion</i> The version of the request-response that you want Amazon Lex to use to invoke your Lambda function. For more information, see <a>using-lambda</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uri</i> The Amazon Resource Name (ARN) of the Lambda function.<br/>
        :return: createVersion:  <br/>checksum:  Checksum of the <code>$LATEST</code>version of the intent created or updated.<br/>version:  The version of the intent. For a new intent, the version is always <code>$LATEST</code>.<br/>createdDate:  The date that the intent was created.<br/>lastUpdatedDate:  The date that the intent was updated. When you create a resource, the creation date and last update dates are the same.<br/>parentIntentSignature:  A unique identifier for the built-in intent that this intent is based on.<br/>fulfillmentActivity:  If defined in the intent, Amazon Lex invokes this Lambda function to fulfill the intent after the user provides all of the information required by the intent.:<br/>dialogCodeHook:  If defined in the intent, Amazon Lex invokes this Lambda function for each user input.:<br/>conclusionStatement:  After the Lambda function specified in the<code>fulfillmentActivity</code>intent fulfills the intent, Amazon Lex conveys this statement to the user.:<br/>followUpPrompt:  If defined in the intent, Amazon Lex uses this prompt to solicit additional user activity after the intent is fulfilled.:<br/>rejectionStatement:  If the user answers "no" to the question defined in <code>confirmationPrompt</code> Amazon Lex responds with this statement to acknowledge that the intent was canceled. :<br/>confirmationPrompt:  If defined in the intent, Amazon Lex prompts the user to confirm the intent before fulfilling it.:<br/>sampleUtterances:   An array of sample utterances that are configured for the intent. <br/>slots:  An array of intent slots that are configured for the intent.<br/>description:  A description of the intent.<br/>name:  The name of the intent.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>responseCard</i> A response card. Amazon Lex uses this prompt at runtime, in the <code>PostText</code> API response. It substitutes session attributes and slot values for placeholders in the response card. For more information, see <a>ex-resp-card</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>maxAttempts</i> The number of times to prompt the user for information.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>messages</i> An array of objects, each of which provides a message string and its type. You can specify the message string in plain text or in Speech Synthesis Markup Language (SSML).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>responseCard</i>  At runtime, if the client is using the <a href="http://docs.aws.amazon.com/lex/latest/dg/API_runtime_PostText.html">PostText</a> API, Amazon Lex includes the response card in the response. It substitutes all of the session attributes and slot values for placeholders in the response card. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>messages</i> A collection of message objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>rejectionStatement</i> If the user answers "no" to the question defined in the <code>prompt</code> field, Amazon Lex responds with this statement to acknowledge that the intent was canceled. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>rejectionStatement</b>: If the user answers "no" to the question defined in the <code>prompt</code> field, Amazon Lex responds with this statement to acknowledge that the intent was canceled. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>prompt</i> Prompts for information from the user. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>prompt</b>: Prompts for information from the user. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>responseCard</i> A response card. Amazon Lex uses this prompt at runtime, in the <code>PostText</code> API response. It substitutes session attributes and slot values for placeholders in the response card. For more information, see <a>ex-resp-card</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>maxAttempts</i> The number of times to prompt the user for information.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>messages</i> An array of objects, each of which provides a message string and its type. You can specify the message string in plain text or in Speech Synthesis Markup Language (SSML).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>responseCard</i>  At runtime, if the client is using the <a href="http://docs.aws.amazon.com/lex/latest/dg/API_runtime_PostText.html">PostText</a> API, Amazon Lex includes the response card in the response. It substitutes all of the session attributes and slot values for placeholders in the response card. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>messages</i> A collection of message objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>responseCard</i>  At runtime, if the client is using the <a href="http://docs.aws.amazon.com/lex/latest/dg/API_runtime_PostText.html">PostText</a> API, Amazon Lex includes the response card in the response. It substitutes all of the session attributes and slot values for placeholders in the response card. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>messages</i> A collection of message objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>messageVersion</i> The version of the request-response that you want Amazon Lex to use to invoke your Lambda function. For more information, see <a>using-lambda</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>uri</i> The Amazon Resource Name (ARN) of the Lambda function.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>codeHook</i>  A description of the Lambda function that is run to fulfill the intent. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>codeHook</b>:  A description of the Lambda function that is run to fulfill the intent. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i>  How the intent should be fulfilled, either by running a Lambda function or by returning the slot data to the client application. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>messageVersion</i> The version of the request-response that you want Amazon Lex to use to invoke your Lambda function. For more information, see <a>using-lambda</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>uri</i> The Amazon Resource Name (ARN) of the Lambda function.<br/>        """
        pass

    def put_slot_type(self, name: str, description: str=None, enumerationValues: List[EnumerationValue]=None, checksum: str=None, valueSelectionStrategy: str=None, createVersion: Boolean=None) -> PutSlotTypeResponse:
        """Creates a custom slot type or replaces an existing custom slot type. To create a custom slot type, specify a name for the slot type and a set of enumeration values, which are the values that a slot of this type can assume. For more information, see <a>how-it-works</a>. If you specify the name of an existing slot type, the fields in the request replace the existing values in the <code>$LATEST</code> version of the slot type. Amazon Lex removes the fields that you don't provide in the request. If you don't specify required fields, Amazon Lex throws an exception. When you update the <code>$LATEST</code> version of a slot type, if a bot uses the <code>$LATEST</code> version of an intent that contains the slot type, the bot's <code>status</code> field is set to <code>NOT_BUILT</code>. This operation requires permissions for the <code>lex:PutSlotType</code> action.

        :param self:
        :param createVersion:  <br/>valueSelectionStrategy:  Determines the slot resolution strategy that Amazon Lex uses to return slot type values. The field can be set to one of the following values: <ul> <li>  <code>ORIGINAL_VALUE</code> - Returns the value entered by the user, if the user value is similar to the slot value. </li> <li>  <code>TOP_RESOLUTION</code> - If there is a resolution list for the slot, return the first value in the resolution list as the slot type value. If there is no resolution list, null is returned. </li> </ul> If you don't specify the <code>valueSelectionStrategy</code>, the default is <code>ORIGINAL_VALUE</code>.<br/>checksum:  Identifies a specific revision of the <code>$LATEST</code> version. When you create a new slot type, leave the <code>checksum</code> field blank. If you specify a checksum you get a <code>BadRequestException</code> exception. When you want to update a slot type, set the <code>checksum</code> field to the checksum of the most recent revision of the <code>$LATEST</code> version. If you don't specify the <code> checksum</code> field, or if the checksum does not match the <code>$LATEST</code> version, you get a <code>PreconditionFailedException</code> exception.<br/>enumerationValues:  A list of <code>EnumerationValue</code> objects that defines the values that the slot type can take. Each value can have a list of <code>synonyms</code>, which are additional values that help train the machine learning model about the values that it resolves for a slot.  When Amazon Lex resolves a slot value, it generates a resolution list that contains up to five possible values for the slot. If you are using a Lambda function, this resolution list is passed to the function. If you are not using a Lambda function you can choose to return the value that the user entered or the first value in the resolution list as the slot value. The <code>valueSelectionStrategy</code> field indicates the option to use. <br/>description:  A description of the slot type.<br/>name:  The name of the slot type. The name is <i>not</i> case sensitive.  The name can't match a built-in slot type name, or a built-in slot type name with "AMAZON." removed. For example, because there is a built-in slot type called <code>AMAZON.DATE</code>, you can't create a custom slot type called <code>DATE</code>. For a list of built-in slot types, see <a href="https://developer.amazon.com/public/solutions/alexa/alexa-skills-kit/docs/built-in-intent-ref/slot-type-reference">Slot Type Reference</a> in the <i>Alexa Skills Kit</i>.<br/>
        :return: createVersion:  <br/>valueSelectionStrategy:  The slot resolution strategy that Amazon Lex uses to determine the value of the slot. For more information, see <a>PutSlotType</a>.<br/>checksum:  Checksum of the <code>$LATEST</code> version of the slot type.<br/>version:  The version of the slot type. For a new slot type, the version is always <code>$LATEST</code>. <br/>createdDate:  The date that the slot type was created.<br/>lastUpdatedDate:  The date that the slot type was updated. When you create a slot type, the creation date and last update date are the same.<br/>enumerationValues:  A list of <code>EnumerationValue</code> objects that defines the values that the slot type can take.<br/>description:  A description of the slot type.<br/>name:  The name of the slot type.<br/>        """
        pass

    def start_import(self, mergeStrategy: str, resourceType: str, payload: Blob) -> StartImportResponse:
        """Starts a job to import a resource to Amazon Lex.

        :param self:
        :param mergeStrategy:  Specifies the action that the <code>StartImport</code> operation should take when there is an existing resource with the same name. <ul> <li> FAIL_ON_CONFLICT - The import operation is stopped on the first conflict between a resource in the import file and an existing resource. The name of the resource causing the conflict is in the <code>failureReason</code> field of the response to the <code>GetImport</code> operation. OVERWRITE_LATEST - The import operation proceeds even if there is a conflict with an existing resource. The $LASTEST version of the existing resource is overwritten with the data from the import file. </li> </ul><br/>resourceType:  Specifies the type of resource to export. Each resource also exports any resources that it depends on.  <ul> <li> A bot exports dependent intents. </li> <li> An intent exports dependent slot types. </li> </ul><br/>payload:  A zip archive in binary format. The archive should contain one file, a JSON file containing the resource to import. The resource should match the type specified in the <code>resourceType</code> field.<br/>
        :return: createdDate:  A timestamp for the date and time that the import job was requested.<br/>importStatus:  The status of the import job. If the status is <code>FAILED</code>, you can get the reason for the failure using the <code>GetImport</code> operation.<br/>importId:  The identifier for the specific import job.<br/>mergeStrategy:  The action to take when there is a merge conflict.<br/>resourceType:  The type of resource to import.<br/>name:  The name given to the import job.<br/>        """
        pass



class LexRuntimeService(BaseClient):
    """Amazon Lex provides both build and runtime endpoints. Each endpoint provides a set of operations (API). Your conversational bot uses the runtime API to understand user utterances (user input text or voice). For example, suppose a user says "I want pizza", your bot sends this input to Amazon Lex using the runtime API. Amazon Lex recognizes that the user request is for the OrderPizza intent (one of the intents defined in the bot). Then Amazon Lex engages in user conversation on behalf of the bot to elicit required information (slot values, such as pizza size and crust type), and then performs fulfillment activity (that you configured when you created the bot). You use the build-time API to create and manage your Amazon Lex bot. For a list of build-time operations, see the build-time API, . """




    class PostContentRequest(Mapping):
        pass
    
    class PostContentResponse(Mapping):
        pass
    
    class BlobStream(object):
        pass
    
    class PostTextRequest(Mapping):
        pass
    
    class PostTextResponse(Mapping):
        pass
    
    class StringMap(object):
        pass
    
    def post_content(self, inputStream: BlobStream, contentType: str, userId: str, botAlias: str, botName: str, sessionAttributes: str=None, requestAttributes: str=None, accept: str=None) -> PostContentResponse:
        """ Sends user input (text or speech) to Amazon Lex. Clients use this API to send text and audio requests to Amazon Lex at runtime. Amazon Lex interprets the user input using the machine learning model that it built for the bot.  The <code>PostContent</code> operation supports audio input at 8kHz and 16kHz. You can use 8kHz audio to achieve higher speech recognition accuracy in telephone audio applications.   In response, Amazon Lex returns the next message to convey to the user. Consider the following example messages:  <ul> <li>  For a user input "I would like a pizza," Amazon Lex might return a response with a message eliciting slot data (for example, <code>PizzaSize</code>): "What size pizza would you like?".  </li> <li>  After the user provides all of the pizza order information, Amazon Lex might return a response with a message to get user confirmation: "Order the pizza?".  </li> <li>  After the user replies "Yes" to the confirmation prompt, Amazon Lex might return a conclusion statement: "Thank you, your cheese pizza has been ordered.".  </li> </ul>  Not all Amazon Lex messages require a response from the user. For example, conclusion statements do not require a response. Some messages require only a yes or no response. In addition to the <code>message</code>, Amazon Lex provides additional context about the message in the response that you can use to enhance client behavior, such as displaying the appropriate client user interface. Consider the following examples:  <ul> <li>  If the message is to elicit slot data, Amazon Lex returns the following context information:  <ul> <li>  <code>x-amz-lex-dialog-state</code> header set to <code>ElicitSlot</code>  </li> <li>  <code>x-amz-lex-intent-name</code> header set to the intent name in the current context  </li> <li>  <code>x-amz-lex-slot-to-elicit</code> header set to the slot name for which the <code>message</code> is eliciting information  </li> <li>  <code>x-amz-lex-slots</code> header set to a map of slots configured for the intent with their current values  </li> </ul> </li> <li>  If the message is a confirmation prompt, the <code>x-amz-lex-dialog-state</code> header is set to <code>Confirmation</code> and the <code>x-amz-lex-slot-to-elicit</code> header is omitted.  </li> <li>  If the message is a clarification prompt configured for the intent, indicating that the user intent is not understood, the <code>x-amz-dialog-state</code> header is set to <code>ElicitIntent</code> and the <code>x-amz-slot-to-elicit</code> header is omitted.  </li> </ul>  In addition, Amazon Lex also returns your application-specific <code>sessionAttributes</code>. For more information, see <a href="http://docs.aws.amazon.com/lex/latest/dg/context-mgmt.html">Managing Conversation Context</a>. 

        :param self:
        :param inputStream:   User input in PCM or Opus audio format or text format as described in the <code>Content-Type</code> HTTP header.  You can stream audio data to Amazon Lex or you can create a local buffer that captures all of the audio data before sending. In general, you get better performance if you stream audio data rather than buffering the data locally.<br/>accept:   You pass this value as the <code>Accept</code> HTTP header.   The message Amazon Lex returns in the response can be either text or speech based on the <code>Accept</code> HTTP header value in the request.  <ul> <li>  If the value is <code>text/plain; charset=utf-8</code>, Amazon Lex returns text in the response.  </li> <li>  If the value begins with <code>audio/</code>, Amazon Lex returns speech in the response. Amazon Lex uses Amazon Polly to generate the speech (using the configuration you specified in the <code>Accept</code> header). For example, if you specify <code>audio/mpeg</code> as the value, Amazon Lex returns speech in the MPEG format. The following are the accepted values: <ul> <li> audio/mpeg </li> <li> audio/ogg </li> <li> audio/pcm </li> <li> text/plain; charset=utf-8 </li> <li> audio/* (defaults to mpeg) </li> </ul> </li> </ul><br/>contentType:   You pass this value as the <code>Content-Type</code> HTTP header.   Indicates the audio format or text. The header value must start with one of the following prefixes:  <ul> <li> PCM format, audio data must be in little-endian byte order. <ul> <li> audio/l16; rate=16000; channels=1 </li> <li> audio/x-l16; sample-rate=16000; channel-count=1 </li> <li> audio/lpcm; sample-rate=8000; sample-size-bits=16; channel-count=1; is-big-endian=false  </li> </ul> </li> <li> Opus format <ul> <li> audio/x-cbr-opus-with-preamble; preamble-size=0; bit-rate=256000; frame-size-milliseconds=4 </li> </ul> </li> <li> Text format <ul> <li> text/plain; charset=utf-8 </li> </ul> </li> </ul><br/>requestAttributes:  You pass this value as the <code>x-amz-lex-request-attributes</code> HTTP header. Request-specific information passed between Amazon Lex and a client application. The value must be a JSON serialized and base64 encoded map with string keys and values. The total size of the <code>requestAttributes</code> and <code>sessionAttributes</code> headers is limited to 12 KB. The namespace <code>x-amz-lex:</code> is reserved for special attributes. Don't create any request attributes with the prefix <code>x-amz-lex:</code>. For more information, see <a href="http://docs.aws.amazon.com/lex/latest/dg/context-mgmt.html#context-mgmt-request-attribs">Setting Request Attributes</a>.<br/>sessionAttributes:  You pass this value as the <code>x-amz-lex-session-attributes</code> HTTP header. Application-specific information passed between Amazon Lex and a client application. The value must be a JSON serialized and base64 encoded map with string keys and values. The total size of the <code>sessionAttributes</code> and <code>requestAttributes</code> headers is limited to 12 KB. For more information, see <a href="http://docs.aws.amazon.com/lex/latest/dg/context-mgmt.html#context-mgmt-session-attribs">Setting Session Attributes</a>.<br/>userId:  The ID of the client application user. Amazon Lex uses this to identify a user's conversation with your bot. At runtime, each request must contain the <code>userID</code> field. To decide the user ID to use for your application, consider the following factors. <ul> <li> The <code>userID</code> field must not contain any personally identifiable information of the user, for example, name, personal identification numbers, or other end user personal information. </li> <li> If you want a user to start a conversation on one device and continue on another device, use a user-specific identifier. </li> <li> If you want the same user to be able to have two independent conversations on two different devices, choose a device-specific identifier. </li> <li> A user can't have two independent conversations with two different versions of the same bot. For example, a user can't have a conversation with the PROD and BETA versions of the same bot. If you anticipate that a user will need to have conversation with two different versions, for example, while testing, include the bot alias in the user ID to separate the two conversations. </li> </ul><br/>botAlias:  Alias of the Amazon Lex bot.<br/>botName:  Name of the Amazon Lex bot.<br/>
        :return: audioStream:  The prompt (or statement) to convey to the user. This is based on the bot configuration and context. For example, if Amazon Lex did not understand the user intent, it sends the <code>clarificationPrompt</code> configured for the bot. If the intent requires confirmation before taking the fulfillment action, it sends the <code>confirmationPrompt</code>. Another example: Suppose that the Lambda function successfully fulfilled the intent, and sent a message to convey to the user. Then Amazon Lex sends that message in the response. <br/>inputTranscript:  The text used to process the request. If the input was an audio stream, the <code>inputTranscript</code> field contains the text extracted from the audio stream. This is the text that is actually processed to recognize intents and slot values. You can use this information to determine if Amazon Lex is correctly processing the audio that you send.<br/>slotToElicit:   If the <code>dialogState</code> value is <code>ElicitSlot</code>, returns the name of the slot for which Amazon Lex is eliciting a value. <br/>dialogState:  Identifies the current state of the user interaction. Amazon Lex returns one of the following values as <code>dialogState</code>. The client can optionally use this information to customize the user interface.  <ul> <li>  <code>ElicitIntent</code> - Amazon Lex wants to elicit the user's intent. Consider the following examples:   For example, a user might utter an intent ("I want to order a pizza"). If Amazon Lex cannot infer the user intent from this utterance, it will return this dialog state.  </li> <li>  <code>ConfirmIntent</code> - Amazon Lex is expecting a "yes" or "no" response.  For example, Amazon Lex wants user confirmation before fulfilling an intent. Instead of a simple "yes" or "no" response, a user might respond with additional information. For example, "yes, but make it a thick crust pizza" or "no, I want to order a drink." Amazon Lex can process such additional information (in these examples, update the crust type slot or change the intent from OrderPizza to OrderDrink).  </li> <li>  <code>ElicitSlot</code> - Amazon Lex is expecting the value of a slot for the current intent.   For example, suppose that in the response Amazon Lex sends this message: "What size pizza would you like?". A user might reply with the slot value (e.g., "medium"). The user might also provide additional information in the response (e.g., "medium thick crust pizza"). Amazon Lex can process such additional information appropriately.  </li> <li>  <code>Fulfilled</code> - Conveys that the Lambda function has successfully fulfilled the intent.  </li> <li>  <code>ReadyForFulfillment</code> - Conveys that the client has to fulfill the request.  </li> <li>  <code>Failed</code> - Conveys that the conversation with the user failed.   This can happen for various reasons, including that the user does not provide an appropriate response to prompts from the service (you can configure how many times Amazon Lex can prompt a user for specific information), or if the Lambda function fails to fulfill the intent.  </li> </ul><br/>messageFormat:  The format of the response message. One of the following values: <ul> <li>  <code>PlainText</code> - The message contains plain UTF-8 text. </li> <li>  <code>CustomPayload</code> - The message is a custom format for the client. </li> <li>  <code>SSML</code> - The message contains text formatted for voice output. </li> <li>  <code>Composite</code> - The message contains an escaped JSON object containing one or more messages from the groups that messages were assigned to when the intent was created. </li> </ul><br/>message:  The message to convey to the user. The message can come from the bot's configuration or from a Lambda function. If the intent is not configured with a Lambda function, or if the Lambda function returned <code>Delegate</code> as the <code>dialogAction.type</code> its response, Amazon Lex decides on the next course of action and selects an appropriate message from the bot's configuration based on the current interaction context. For example, if Amazon Lex isn't able to understand user input, it uses a clarification prompt message. When you create an intent you can assign messages to groups. When messages are assigned to groups Amazon Lex returns one message from each group in the response. The message field is an escaped JSON string containing the messages. For more information about the structure of the JSON string returned, see <a>msg-prompts-formats</a>. If the Lambda function returns a message, Amazon Lex passes it to the client in its response.<br/>sessionAttributes:   Map of key/value pairs representing the session-specific context information. <br/>slots:  Map of zero or more intent slots (name/value pairs) Amazon Lex detected from the user input during the conversation. Amazon Lex creates a resolution list containing likely values for a slot. The value that it returns is determined by the <code>valueSelectionStrategy</code> selected when the slot type was created or updated. If <code>valueSelectionStrategy</code> is set to <code>ORIGINAL_VALUE</code>, the value provided by the user is returned, if the user value is similar to the slot values. If <code>valueSelectionStrategy</code> is set to <code>TOP_RESOLUTION</code> Amazon Lex returns the first value in the resolution list or, if there is no resolution list, null. If you don't specify a <code>valueSelectionStrategy</code>, the default is <code>ORIGINAL_VALUE</code>.<br/>intentName:  Current user intent that Amazon Lex is aware of.<br/>contentType:  Content type as specified in the <code>Accept</code> HTTP header in the request.<br/>        """
        pass

    def post_text(self, inputText: str, userId: str, botAlias: str, botName: str, sessionAttributes: StringMap=None, requestAttributes: StringMap=None) -> PostTextResponse:
        """Sends user input (text-only) to Amazon Lex. Client applications can use this API to send requests to Amazon Lex at runtime. Amazon Lex then interprets the user input using the machine learning model it built for the bot.   In response, Amazon Lex returns the next <code>message</code> to convey to the user an optional <code>responseCard</code> to display. Consider the following example messages:  <ul> <li>  For a user input "I would like a pizza", Amazon Lex might return a response with a message eliciting slot data (for example, PizzaSize): "What size pizza would you like?"  </li> <li>  After the user provides all of the pizza order information, Amazon Lex might return a response with a message to obtain user confirmation "Proceed with the pizza order?".  </li> <li>  After the user replies to a confirmation prompt with a "yes", Amazon Lex might return a conclusion statement: "Thank you, your cheese pizza has been ordered.".  </li> </ul>  Not all Amazon Lex messages require a user response. For example, a conclusion statement does not require a response. Some messages require only a "yes" or "no" user response. In addition to the <code>message</code>, Amazon Lex provides additional context about the message in the response that you might use to enhance client behavior, for example, to display the appropriate client user interface. These are the <code>slotToElicit</code>, <code>dialogState</code>, <code>intentName</code>, and <code>slots</code> fields in the response. Consider the following examples:  <ul> <li> If the message is to elicit slot data, Amazon Lex returns the following context information: <ul> <li>  <code>dialogState</code> set to ElicitSlot  </li> <li>  <code>intentName</code> set to the intent name in the current context  </li> <li>  <code>slotToElicit</code> set to the slot name for which the <code>message</code> is eliciting information  </li> <li>  <code>slots</code> set to a map of slots, configured for the intent, with currently known values  </li> </ul> </li> <li>  If the message is a confirmation prompt, the <code>dialogState</code> is set to ConfirmIntent and <code>SlotToElicit</code> is set to null.  </li> <li> If the message is a clarification prompt (configured for the intent) that indicates that user intent is not understood, the <code>dialogState</code> is set to ElicitIntent and <code>slotToElicit</code> is set to null.  </li> </ul>  In addition, Amazon Lex also returns your application-specific <code>sessionAttributes</code>. For more information, see <a href="http://docs.aws.amazon.com/lex/latest/dg/context-mgmt.html">Managing Conversation Context</a>. 

        :param self:
        :param inputText:  The text that the user entered (Amazon Lex interprets this text).<br/>requestAttributes:  Request-specific information passed between Amazon Lex and a client application. The namespace <code>x-amz-lex:</code> is reserved for special attributes. Don't create any request attributes with the prefix <code>x-amz-lex:</code>. For more information, see <a href="http://docs.aws.amazon.com/lex/latest/dg/context-mgmt.html#context-mgmt-request-attribs">Setting Request Attributes</a>.<br/>sessionAttributes:  Application-specific information passed between Amazon Lex and a client application. For more information, see <a href="http://docs.aws.amazon.com/lex/latest/dg/context-mgmt.html#context-mgmt-session-attribs">Setting Session Attributes</a>.<br/>userId:  The ID of the client application user. Amazon Lex uses this to identify a user's conversation with your bot. At runtime, each request must contain the <code>userID</code> field. To decide the user ID to use for your application, consider the following factors. <ul> <li> The <code>userID</code> field must not contain any personally identifiable information of the user, for example, name, personal identification numbers, or other end user personal information. </li> <li> If you want a user to start a conversation on one device and continue on another device, use a user-specific identifier. </li> <li> If you want the same user to be able to have two independent conversations on two different devices, choose a device-specific identifier. </li> <li> A user can't have two independent conversations with two different versions of the same bot. For example, a user can't have a conversation with the PROD and BETA versions of the same bot. If you anticipate that a user will need to have conversation with two different versions, for example, while testing, include the bot alias in the user ID to separate the two conversations. </li> </ul><br/>botAlias:  The alias of the Amazon Lex bot.<br/>botName:  The name of the Amazon Lex bot.<br/>
        :return: responseCard:  Represents the options that the user has to respond to the current prompt. Response Card can come from the bot configuration (in the Amazon Lex console, choose the settings button next to a slot) or from a code hook (Lambda function). :<br/>slotToElicit:  If the <code>dialogState</code> value is <code>ElicitSlot</code>, returns the name of the slot for which Amazon Lex is eliciting a value. <br/>dialogState:   Identifies the current state of the user interaction. Amazon Lex returns one of the following values as <code>dialogState</code>. The client can optionally use this information to customize the user interface.  <ul> <li>  <code>ElicitIntent</code> - Amazon Lex wants to elicit user intent.  For example, a user might utter an intent ("I want to order a pizza"). If Amazon Lex cannot infer the user intent from this utterance, it will return this dialogState. </li> <li>  <code>ConfirmIntent</code> - Amazon Lex is expecting a "yes" or "no" response.   For example, Amazon Lex wants user confirmation before fulfilling an intent.  Instead of a simple "yes" or "no," a user might respond with additional information. For example, "yes, but make it thick crust pizza" or "no, I want to order a drink". Amazon Lex can process such additional information (in these examples, update the crust type slot value, or change intent from OrderPizza to OrderDrink). </li> <li>  <code>ElicitSlot</code> - Amazon Lex is expecting a slot value for the current intent.  For example, suppose that in the response Amazon Lex sends this message: "What size pizza would you like?". A user might reply with the slot value (e.g., "medium"). The user might also provide additional information in the response (e.g., "medium thick crust pizza"). Amazon Lex can process such additional information appropriately.  </li> <li>  <code>Fulfilled</code> - Conveys that the Lambda function configured for the intent has successfully fulfilled the intent.  </li> <li>  <code>ReadyForFulfillment</code> - Conveys that the client has to fulfill the intent.  </li> <li>  <code>Failed</code> - Conveys that the conversation with the user failed.   This can happen for various reasons including that the user did not provide an appropriate response to prompts from the service (you can configure how many times Amazon Lex can prompt a user for specific information), or the Lambda function failed to fulfill the intent.  </li> </ul><br/>messageFormat:  The format of the response message. One of the following values: <ul> <li>  <code>PlainText</code> - The message contains plain UTF-8 text. </li> <li>  <code>CustomPayload</code> - The message is a custom format defined by the Lambda function. </li> <li>  <code>SSML</code> - The message contains text formatted for voice output. </li> <li>  <code>Composite</code> - The message contains an escaped JSON object containing one or more messages from the groups that messages were assigned to when the intent was created. </li> </ul><br/>message:  The message to convey to the user. The message can come from the bot's configuration or from a Lambda function. If the intent is not configured with a Lambda function, or if the Lambda function returned <code>Delegate</code> as the <code>dialogAction.type</code> its response, Amazon Lex decides on the next course of action and selects an appropriate message from the bot's configuration based on the current interaction context. For example, if Amazon Lex isn't able to understand user input, it uses a clarification prompt message. When you create an intent you can assign messages to groups. When messages are assigned to groups Amazon Lex returns one message from each group in the response. The message field is an escaped JSON string containing the messages. For more information about the structure of the JSON string returned, see <a>msg-prompts-formats</a>. If the Lambda function returns a message, Amazon Lex passes it to the client in its response.<br/>sessionAttributes:  A map of key-value pairs representing the session-specific context information.<br/>slots:   The intent slots that Amazon Lex detected from the user input in the conversation.  Amazon Lex creates a resolution list containing likely values for a slot. The value that it returns is determined by the <code>valueSelectionStrategy</code> selected when the slot type was created or updated. If <code>valueSelectionStrategy</code> is set to <code>ORIGINAL_VALUE</code>, the value provided by the user is returned, if the user value is similar to the slot values. If <code>valueSelectionStrategy</code> is set to <code>TOP_RESOLUTION</code> Amazon Lex returns the first value in the resolution list or, if there is no resolution list, null. If you don't specify a <code>valueSelectionStrategy</code>, the default is <code>ORIGINAL_VALUE</code>.<br/>intentName:  The current user intent that Amazon Lex is aware of.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>genericAttachments</i> An array of attachment objects representing options.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>contentType</i> The content type of the response.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the response card format.<br/>        """
        pass



class LicenseManager(BaseClient):
    """<fullname> AWS License Manager </fullname>  <i>This is the AWS License Manager API Reference.</i> It provides descriptions, syntax, and usage examples for each of the actions and data types for License Manager. The topic for each action shows the Query API request parameters and the XML response. You can also view the XML request elements in the WSDL.   Alternatively, you can use one of the AWS SDKs to access an API that's tailored to the programming language or platform that you're using. For more information, see <a href="http://aws.amazon.com/tools/#SDKs">AWS SDKs</a>. """


    class ListAssociationsForLicenseConfigurationPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_associations_for_license_configuration` for valid parameters"""
            pass
    class ListLicenseConfigurationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_license_configurations` for valid parameters"""
            pass
    class ListLicenseSpecificationsForResourcePaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_license_specifications_for_resource` for valid parameters"""
            pass
    class ListResourceInventoryPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_resource_inventory` for valid parameters"""
            pass
    class ListUsageForLicenseConfigurationPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_usage_for_license_configuration` for valid parameters"""
            pass


    class CreateLicenseConfigurationRequest(Mapping):
        pass
    
    class CreateLicenseConfigurationResponse(Mapping):
        pass
    
    class BoxLong(object):
        pass
    
    class BoxBoolean(object):
        pass
    
    class String(object):
        pass
    
    class Tag(Mapping):
        pass
    
    class DeleteLicenseConfigurationRequest(Mapping):
        pass
    
    class DeleteLicenseConfigurationResponse(Mapping):
        pass
    
    class GetLicenseConfigurationRequest(Mapping):
        pass
    
    class GetLicenseConfigurationResponse(Mapping):
        pass
    
    class GetServiceSettingsRequest(Mapping):
        pass
    
    class GetServiceSettingsResponse(Mapping):
        pass
    
    class ListAssociationsForLicenseConfigurationRequest(Mapping):
        pass
    
    class ListAssociationsForLicenseConfigurationResponse(Mapping):
        pass
    
    class ListLicenseConfigurationsRequest(Mapping):
        pass
    
    class ListLicenseConfigurationsResponse(Mapping):
        pass
    
    class Filter(Mapping):
        pass
    
    class ListLicenseSpecificationsForResourceRequest(Mapping):
        pass
    
    class ListLicenseSpecificationsForResourceResponse(Mapping):
        pass
    
    class ListResourceInventoryRequest(Mapping):
        pass
    
    class ListResourceInventoryResponse(Mapping):
        pass
    
    class InventoryFilter(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class ListUsageForLicenseConfigurationRequest(Mapping):
        pass
    
    class ListUsageForLicenseConfigurationResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class UpdateLicenseConfigurationRequest(Mapping):
        pass
    
    class UpdateLicenseConfigurationResponse(Mapping):
        pass
    
    class UpdateLicenseSpecificationsForResourceRequest(Mapping):
        pass
    
    class UpdateLicenseSpecificationsForResourceResponse(Mapping):
        pass
    
    class LicenseSpecification(Mapping):
        pass
    
    class UpdateServiceSettingsRequest(Mapping):
        pass
    
    class UpdateServiceSettingsResponse(Mapping):
        pass
    
    class OrganizationConfiguration(Mapping):
        pass
    
    def create_license_configuration(self, LicenseCountingType: str, Name: str, Description: str=None, LicenseCount: BoxLong=None, LicenseCountHardLimit: BoxBoolean=None, LicenseRules: List[String]=None, Tags: List[Tag]=None) -> CreateLicenseConfigurationResponse:
        """Creates a new license configuration object. A license configuration is an abstraction of a customer license agreement that can be consumed and enforced by License Manager. Components include specifications for the license type (licensing by instance, socket, CPU, or VCPU), tenancy (shared tenancy, Amazon EC2 Dedicated Instance, Amazon EC2 Dedicated Host, or any of these), host affinity (how long a VM must be associated with a host), the number of licenses purchased and used.

        :param self:
        :param Tags:  The tags to apply to the resources during launch. You can only tag instances and volumes on launch. The specified tags are applied to all instances or volumes that are created during launch. To tag a resource after it has been created, see CreateTags . <p/><br/>LicenseRules:  Array of configured License Manager rules.<br/>LicenseCountHardLimit:  Flag indicating whether hard or soft license enforcement is used. Exceeding a hard limit results in the blocked deployment of new instances.<br/>LicenseCount:  Number of licenses managed by the license configuration.<br/>LicenseCountingType:  Dimension to use to track the license inventory.<br/>Description:  Human-friendly description of the license configuration.<br/>Name:  Name of the license configuration.<br/>
        :return: LicenseConfigurationArn:  ARN of the license configuration object after its creation.<br/>        """
        pass

    def delete_license_configuration(self, LicenseConfigurationArn: str) -> DeleteLicenseConfigurationResponse:
        """Deletes an existing license configuration. This action fails if the configuration is in use.

        :param self:
        :param LicenseConfigurationArn:  Unique ID of the configuration object to delete.<br/>
        :return:         """
        pass

    def get_license_configuration(self, LicenseConfigurationArn: str) -> GetLicenseConfigurationResponse:
        """Returns a detailed description of a license configuration.

        :param self:
        :param LicenseConfigurationArn:  ARN of the license configuration being requested.<br/>
        :return: Tags:  List of tags attached to the license configuration.<br/>ManagedResourceSummaryList:  List of summaries of managed resources.<br/>ConsumedLicenseSummaryList:  List of summaries for consumed licenses used by various resources.<br/>OwnerAccountId:  Owner account ID for the license configuration.<br/>Status:  License configuration status (active, etc.).<br/>ConsumedLicenses:  Number of licenses assigned to resources.<br/>LicenseCountHardLimit:  Sets the number of available licenses as a hard limit.<br/>LicenseCount:  Number of available licenses.<br/>LicenseRules:  List of flexible text strings designating license rules.<br/>LicenseCountingType:  Dimension on which the licenses are counted (for example, instances, cores, sockets, or VCPUs).<br/>Description:  Description of the license configuration.<br/>Name:  Name of the license configuration.<br/>LicenseConfigurationArn:  ARN of the license configuration requested.<br/>LicenseConfigurationId:  Unique ID for the license configuration.<br/>        """
        pass

    def get_service_settings(self) -> GetServiceSettingsResponse:
        """Gets License Manager settings for a region. Exposes the configured S3 bucket, SNS topic, etc., for inspection. 

        :param self:
        :param 
        :return: EnableCrossAccountsDiscovery:  Indicates whether cross-account discovery has been enabled.<br/>OrganizationConfiguration:  Indicates whether AWS Organizations has been integrated with License Manager for cross-account discovery.:<br/>SnsTopicArn:  SNS topic configured to receive notifications from License Manager.<br/>S3BucketArn:  Regional S3 bucket path for storing reports, license trail event data, discovery data, etc.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnableIntegration</i> Flag to activate AWS Organization integration.<br/>        """
        pass

    def list_associations_for_license_configuration(self, LicenseConfigurationArn: str, MaxResults: int=None, NextToken: str=None) -> ListAssociationsForLicenseConfigurationResponse:
        """Lists the resource associations for a license configuration. Resource associations need not consume licenses from a license configuration. For example, an AMI or a stopped instance may not consume a license (depending on the license rules). Use this operation to find all resources associated with a license configuration.

        :param self:
        :param NextToken:  Token for the next set of results.<br/>MaxResults:  Maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value.<br/>LicenseConfigurationArn:  ARN of a <code>LicenseConfiguration</code> object.<br/>
        :return: NextToken:  Token for the next set of results.<br/>LicenseConfigurationAssociations:  Lists association objects for the license configuration, each containing the association time, number of consumed licenses, resource ARN, resource ID, account ID that owns the resource, resource size, and resource type.<br/>        """
        pass

    def list_license_configurations(self, LicenseConfigurationArns: List[String]=None, MaxResults: int=None, NextToken: str=None, Filters: List[Filter]=None) -> ListLicenseConfigurationsResponse:
        """Lists license configuration objects for an account, each containing the name, description, license type, and other license terms modeled from a license agreement.

        :param self:
        :param Filters:  One or more filters.<br/>NextToken:  Token for the next set of results.<br/>MaxResults:  Maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value.<br/>LicenseConfigurationArns:  An array of ARNs for the calling account’s license configurations.<br/>
        :return: NextToken:  Token for the next set of results.<br/>LicenseConfigurations:  Array of license configuration objects.<br/>        """
        pass

    def list_license_specifications_for_resource(self, ResourceArn: str, MaxResults: int=None, NextToken: str=None) -> ListLicenseSpecificationsForResourceResponse:
        """Returns the license configuration for a resource.

        :param self:
        :param NextToken:  Token for the next set of results.<br/>MaxResults:  Maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value.<br/>ResourceArn:  ARN of an AMI or Amazon EC2 instance that has an associated license configuration.<br/>
        :return: NextToken:  Token for the next set of results.<br/>LicenseSpecifications:  License configurations associated with a resource.<br/>        """
        pass

    def list_resource_inventory(self, MaxResults: int=None, NextToken: str=None, Filters: List[InventoryFilter]=None) -> ListResourceInventoryResponse:
        """Returns a detailed list of resources.

        :param self:
        :param Filters:  One or more filters.<br/>NextToken:  Token for the next set of results.<br/>MaxResults:  Maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value.<br/>
        :return: NextToken:  Token for the next set of results.<br/>ResourceInventoryList:  The detailed list of resources.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceArn: str) -> ListTagsForResourceResponse:
        """Lists tags attached to a resource.

        :param self:
        :param ResourceArn:  ARN for the resource.<br/>
        :return: Tags:  List of tags attached to the resource.<br/>        """
        pass

    def list_usage_for_license_configuration(self, LicenseConfigurationArn: str, MaxResults: int=None, NextToken: str=None, Filters: List[Filter]=None) -> ListUsageForLicenseConfigurationResponse:
        """Lists all license usage records for a license configuration, displaying license consumption details by resource at a selected point in time. Use this action to audit the current license consumption for any license inventory and configuration.

        :param self:
        :param Filters:  List of filters to apply.<br/>NextToken:  Token for the next set of results.<br/>MaxResults:  Maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value.<br/>LicenseConfigurationArn:  ARN of the targeted <code>LicenseConfiguration</code> object.<br/>
        :return: NextToken:  Token for the next set of results.<br/>LicenseConfigurationUsageList:  An array of <code>LicenseConfigurationUsage</code> objects.<br/>        """
        pass

    def tag_resource(self, Tags: List[Tag], ResourceArn: str) -> TagResourceResponse:
        """Attach one of more tags to any resource.

        :param self:
        :param Tags:  Names of the tags to attach to the resource.<br/>ResourceArn:  Resource of the ARN to be tagged.<br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[String], ResourceArn: str) -> UntagResourceResponse:
        """Remove tags from a resource.

        :param self:
        :param TagKeys:  List keys identifying tags to remove.<br/>ResourceArn:  ARN of the resource.<br/>
        :return:         """
        pass

    def update_license_configuration(self, LicenseConfigurationArn: str, LicenseConfigurationStatus: str=None, LicenseRules: List[String]=None, LicenseCount: BoxLong=None, LicenseCountHardLimit: BoxBoolean=None, Name: str=None, Description: str=None) -> UpdateLicenseConfigurationResponse:
        """Modifies the attributes of an existing license configuration object. A license configuration is an abstraction of a customer license agreement that can be consumed and enforced by License Manager. Components include specifications for the license type (Instances, cores, sockets, VCPUs), tenancy (shared or Dedicated Host), host affinity (how long a VM is associated with a host), the number of licenses purchased and used.

        :param self:
        :param Description:  New human-friendly description of the license configuration.<br/>Name:  New name of the license configuration.<br/>LicenseCountHardLimit:  Sets the number of available licenses as a hard limit.<br/>LicenseCount:  New number of licenses managed by the license configuration.<br/>LicenseRules:  List of flexible text strings designating license rules.<br/>LicenseConfigurationStatus:  New status of the license configuration (<code>ACTIVE</code> or <code>INACTIVE</code>).<br/>LicenseConfigurationArn:  ARN for a license configuration.<br/>
        :return:         """
        pass

    def update_license_specifications_for_resource(self, ResourceArn: str, AddLicenseSpecifications: List[LicenseSpecification]=None, RemoveLicenseSpecifications: List[LicenseSpecification]=None) -> UpdateLicenseSpecificationsForResourceResponse:
        """Adds or removes license configurations for a specified AWS resource. This operation currently supports updating the license specifications of AMIs, instances, and hosts. Launch templates and AWS CloudFormation templates are not managed from this operation as those resources send the license configurations directly to a resource creation operation, such as <code>RunInstances</code>.

        :param self:
        :param RemoveLicenseSpecifications:  License configuration ARNs to be removed from a resource.<br/>AddLicenseSpecifications:  License configuration ARNs to be added to a resource.<br/>ResourceArn:  ARN for an AWS server resource.<br/>
        :return:         """
        pass

    def update_service_settings(self, S3BucketArn: str=None, SnsTopicArn: str=None, OrganizationConfiguration: OrganizationConfiguration=None, EnableCrossAccountsDiscovery: BoxBoolean=None) -> UpdateServiceSettingsResponse:
        """Updates License Manager service settings.

        :param self:
        :param EnableCrossAccountsDiscovery:  Activates cross-account discovery.<br/>OrganizationConfiguration:  Integrates AWS Organizations with License Manager for cross-account discovery.:<br/>SnsTopicArn:  ARN of the Amazon SNS topic used for License Manager alerts.<br/>S3BucketArn:  ARN of the Amazon S3 bucket where License Manager information is stored.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnableIntegration</i> Flag to activate AWS Organization integration.<br/>
        :return:         """
        pass



class Lightsail(BaseClient):
    """Amazon Lightsail is the easiest way to get started with AWS for developers who just need virtual private servers. Lightsail includes everything you need to launch your project quickly - a virtual machine, a managed database, SSD-based storage, data transfer, DNS management, and a static IP - for a low, predictable price. You manage those Lightsail servers through the Lightsail console or by using the API or command-line interface (CLI). For more information about Lightsail concepts and tasks, see the <a href="https://lightsail.aws.amazon.com/ls/docs/all">Lightsail Dev Guide</a>. To use the Lightsail API or the CLI, you will need to use AWS Identity and Access Management (IAM) to generate access keys. For details about how to set this up, see the <a href="http://lightsail.aws.amazon.com/ls/docs/how-to/article/lightsail-how-to-set-up-access-keys-to-use-sdk-api-cli">Lightsail Dev Guide</a>."""


    class GetActiveNamesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_active_names` for valid parameters"""
            pass
    class GetBlueprintsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_blueprints` for valid parameters"""
            pass
    class GetBundlesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_bundles` for valid parameters"""
            pass
    class GetDomainsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_domains` for valid parameters"""
            pass
    class GetInstanceSnapshotsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_instance_snapshots` for valid parameters"""
            pass
    class GetInstancesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_instances` for valid parameters"""
            pass
    class GetKeyPairsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_key_pairs` for valid parameters"""
            pass
    class GetOperationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_operations` for valid parameters"""
            pass
    class GetStaticIpsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_static_ips` for valid parameters"""
            pass
    class GetCloudFormationStackRecordsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_cloud_formation_stack_records` for valid parameters"""
            pass
    class GetDiskSnapshotsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_disk_snapshots` for valid parameters"""
            pass
    class GetDisksPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_disks` for valid parameters"""
            pass
    class GetExportSnapshotRecordsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_export_snapshot_records` for valid parameters"""
            pass
    class GetLoadBalancersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_load_balancers` for valid parameters"""
            pass
    class GetRelationalDatabaseBlueprintsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_relational_database_blueprints` for valid parameters"""
            pass
    class GetRelationalDatabaseBundlesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_relational_database_bundles` for valid parameters"""
            pass
    class GetRelationalDatabaseEventsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_relational_database_events` for valid parameters"""
            pass
    class GetRelationalDatabaseParametersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_relational_database_parameters` for valid parameters"""
            pass
    class GetRelationalDatabaseSnapshotsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_relational_database_snapshots` for valid parameters"""
            pass
    class GetRelationalDatabasesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_relational_databases` for valid parameters"""
            pass


    class AllocateStaticIpRequest(Mapping):
        pass
    
    class AllocateStaticIpResult(Mapping):
        pass
    
    class AttachDiskRequest(Mapping):
        pass
    
    class AttachDiskResult(Mapping):
        pass
    
    class AttachInstancesToLoadBalancerRequest(Mapping):
        pass
    
    class AttachInstancesToLoadBalancerResult(Mapping):
        pass
    
    class ResourceName(object):
        pass
    
    class AttachLoadBalancerTlsCertificateRequest(Mapping):
        pass
    
    class AttachLoadBalancerTlsCertificateResult(Mapping):
        pass
    
    class AttachStaticIpRequest(Mapping):
        pass
    
    class AttachStaticIpResult(Mapping):
        pass
    
    class CloseInstancePublicPortsRequest(Mapping):
        pass
    
    class CloseInstancePublicPortsResult(Mapping):
        pass
    
    class PortInfo(Mapping):
        pass
    
    class CopySnapshotRequest(Mapping):
        pass
    
    class CopySnapshotResult(Mapping):
        pass
    
    class CreateCloudFormationStackRequest(Mapping):
        pass
    
    class CreateCloudFormationStackResult(Mapping):
        pass
    
    class InstanceEntry(Mapping):
        pass
    
    class CreateDiskRequest(Mapping):
        pass
    
    class CreateDiskResult(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreateDiskFromSnapshotRequest(Mapping):
        pass
    
    class CreateDiskFromSnapshotResult(Mapping):
        pass
    
    class CreateDiskSnapshotRequest(Mapping):
        pass
    
    class CreateDiskSnapshotResult(Mapping):
        pass
    
    class CreateDomainRequest(Mapping):
        pass
    
    class CreateDomainResult(Mapping):
        pass
    
    class CreateDomainEntryRequest(Mapping):
        pass
    
    class CreateDomainEntryResult(Mapping):
        pass
    
    class DomainEntry(Mapping):
        pass
    
    class CreateInstanceSnapshotRequest(Mapping):
        pass
    
    class CreateInstanceSnapshotResult(Mapping):
        pass
    
    class CreateInstancesRequest(Mapping):
        pass
    
    class CreateInstancesResult(Mapping):
        pass
    
    class string(object):
        pass
    
    class CreateInstancesFromSnapshotRequest(Mapping):
        pass
    
    class CreateInstancesFromSnapshotResult(Mapping):
        pass
    
    class AttachedDiskMap(object):
        pass
    
    class CreateKeyPairRequest(Mapping):
        pass
    
    class CreateKeyPairResult(Mapping):
        pass
    
    class CreateLoadBalancerRequest(Mapping):
        pass
    
    class CreateLoadBalancerResult(Mapping):
        pass
    
    class DomainName(object):
        pass
    
    class CreateLoadBalancerTlsCertificateRequest(Mapping):
        pass
    
    class CreateLoadBalancerTlsCertificateResult(Mapping):
        pass
    
    class CreateRelationalDatabaseRequest(Mapping):
        pass
    
    class CreateRelationalDatabaseResult(Mapping):
        pass
    
    class boolean(object):
        pass
    
    class CreateRelationalDatabaseFromSnapshotRequest(Mapping):
        pass
    
    class CreateRelationalDatabaseFromSnapshotResult(Mapping):
        pass
    
    class IsoDate(object):
        pass
    
    class CreateRelationalDatabaseSnapshotRequest(Mapping):
        pass
    
    class CreateRelationalDatabaseSnapshotResult(Mapping):
        pass
    
    class DeleteDiskRequest(Mapping):
        pass
    
    class DeleteDiskResult(Mapping):
        pass
    
    class DeleteDiskSnapshotRequest(Mapping):
        pass
    
    class DeleteDiskSnapshotResult(Mapping):
        pass
    
    class DeleteDomainRequest(Mapping):
        pass
    
    class DeleteDomainResult(Mapping):
        pass
    
    class DeleteDomainEntryRequest(Mapping):
        pass
    
    class DeleteDomainEntryResult(Mapping):
        pass
    
    class DeleteInstanceRequest(Mapping):
        pass
    
    class DeleteInstanceResult(Mapping):
        pass
    
    class DeleteInstanceSnapshotRequest(Mapping):
        pass
    
    class DeleteInstanceSnapshotResult(Mapping):
        pass
    
    class DeleteKeyPairRequest(Mapping):
        pass
    
    class DeleteKeyPairResult(Mapping):
        pass
    
    class DeleteKnownHostKeysRequest(Mapping):
        pass
    
    class DeleteKnownHostKeysResult(Mapping):
        pass
    
    class DeleteLoadBalancerRequest(Mapping):
        pass
    
    class DeleteLoadBalancerResult(Mapping):
        pass
    
    class DeleteLoadBalancerTlsCertificateRequest(Mapping):
        pass
    
    class DeleteLoadBalancerTlsCertificateResult(Mapping):
        pass
    
    class DeleteRelationalDatabaseRequest(Mapping):
        pass
    
    class DeleteRelationalDatabaseResult(Mapping):
        pass
    
    class DeleteRelationalDatabaseSnapshotRequest(Mapping):
        pass
    
    class DeleteRelationalDatabaseSnapshotResult(Mapping):
        pass
    
    class DetachDiskRequest(Mapping):
        pass
    
    class DetachDiskResult(Mapping):
        pass
    
    class DetachInstancesFromLoadBalancerRequest(Mapping):
        pass
    
    class DetachInstancesFromLoadBalancerResult(Mapping):
        pass
    
    class DetachStaticIpRequest(Mapping):
        pass
    
    class DetachStaticIpResult(Mapping):
        pass
    
    class DownloadDefaultKeyPairRequest(Mapping):
        pass
    
    class DownloadDefaultKeyPairResult(Mapping):
        pass
    
    class ExportSnapshotRequest(Mapping):
        pass
    
    class ExportSnapshotResult(Mapping):
        pass
    
    class GetActiveNamesRequest(Mapping):
        pass
    
    class GetActiveNamesResult(Mapping):
        pass
    
    class GetBlueprintsRequest(Mapping):
        pass
    
    class GetBlueprintsResult(Mapping):
        pass
    
    class GetBundlesRequest(Mapping):
        pass
    
    class GetBundlesResult(Mapping):
        pass
    
    class GetCloudFormationStackRecordsRequest(Mapping):
        pass
    
    class GetCloudFormationStackRecordsResult(Mapping):
        pass
    
    class GetDiskRequest(Mapping):
        pass
    
    class GetDiskResult(Mapping):
        pass
    
    class GetDiskSnapshotRequest(Mapping):
        pass
    
    class GetDiskSnapshotResult(Mapping):
        pass
    
    class GetDiskSnapshotsRequest(Mapping):
        pass
    
    class GetDiskSnapshotsResult(Mapping):
        pass
    
    class GetDisksRequest(Mapping):
        pass
    
    class GetDisksResult(Mapping):
        pass
    
    class GetDomainRequest(Mapping):
        pass
    
    class GetDomainResult(Mapping):
        pass
    
    class GetDomainsRequest(Mapping):
        pass
    
    class GetDomainsResult(Mapping):
        pass
    
    class GetExportSnapshotRecordsRequest(Mapping):
        pass
    
    class GetExportSnapshotRecordsResult(Mapping):
        pass
    
    class GetInstanceRequest(Mapping):
        pass
    
    class GetInstanceResult(Mapping):
        pass
    
    class GetInstanceAccessDetailsRequest(Mapping):
        pass
    
    class GetInstanceAccessDetailsResult(Mapping):
        pass
    
    class GetInstanceMetricDataRequest(Mapping):
        pass
    
    class GetInstanceMetricDataResult(Mapping):
        pass
    
    class timestamp(object):
        pass
    
    class MetricStatistic(object):
        pass
    
    class GetInstancePortStatesRequest(Mapping):
        pass
    
    class GetInstancePortStatesResult(Mapping):
        pass
    
    class GetInstanceSnapshotRequest(Mapping):
        pass
    
    class GetInstanceSnapshotResult(Mapping):
        pass
    
    class GetInstanceSnapshotsRequest(Mapping):
        pass
    
    class GetInstanceSnapshotsResult(Mapping):
        pass
    
    class GetInstanceStateRequest(Mapping):
        pass
    
    class GetInstanceStateResult(Mapping):
        pass
    
    class GetInstancesRequest(Mapping):
        pass
    
    class GetInstancesResult(Mapping):
        pass
    
    class GetKeyPairRequest(Mapping):
        pass
    
    class GetKeyPairResult(Mapping):
        pass
    
    class GetKeyPairsRequest(Mapping):
        pass
    
    class GetKeyPairsResult(Mapping):
        pass
    
    class GetLoadBalancerRequest(Mapping):
        pass
    
    class GetLoadBalancerResult(Mapping):
        pass
    
    class GetLoadBalancerMetricDataRequest(Mapping):
        pass
    
    class GetLoadBalancerMetricDataResult(Mapping):
        pass
    
    class GetLoadBalancerTlsCertificatesRequest(Mapping):
        pass
    
    class GetLoadBalancerTlsCertificatesResult(Mapping):
        pass
    
    class GetLoadBalancersRequest(Mapping):
        pass
    
    class GetLoadBalancersResult(Mapping):
        pass
    
    class GetOperationRequest(Mapping):
        pass
    
    class GetOperationResult(Mapping):
        pass
    
    class GetOperationsRequest(Mapping):
        pass
    
    class GetOperationsResult(Mapping):
        pass
    
    class GetOperationsForResourceRequest(Mapping):
        pass
    
    class GetOperationsForResourceResult(Mapping):
        pass
    
    class GetRegionsRequest(Mapping):
        pass
    
    class GetRegionsResult(Mapping):
        pass
    
    class GetRelationalDatabaseRequest(Mapping):
        pass
    
    class GetRelationalDatabaseResult(Mapping):
        pass
    
    class GetRelationalDatabaseBlueprintsRequest(Mapping):
        pass
    
    class GetRelationalDatabaseBlueprintsResult(Mapping):
        pass
    
    class GetRelationalDatabaseBundlesRequest(Mapping):
        pass
    
    class GetRelationalDatabaseBundlesResult(Mapping):
        pass
    
    class GetRelationalDatabaseEventsRequest(Mapping):
        pass
    
    class GetRelationalDatabaseEventsResult(Mapping):
        pass
    
    class GetRelationalDatabaseLogEventsRequest(Mapping):
        pass
    
    class GetRelationalDatabaseLogEventsResult(Mapping):
        pass
    
    class GetRelationalDatabaseLogStreamsRequest(Mapping):
        pass
    
    class GetRelationalDatabaseLogStreamsResult(Mapping):
        pass
    
    class GetRelationalDatabaseMasterUserPasswordRequest(Mapping):
        pass
    
    class GetRelationalDatabaseMasterUserPasswordResult(Mapping):
        pass
    
    class GetRelationalDatabaseMetricDataRequest(Mapping):
        pass
    
    class GetRelationalDatabaseMetricDataResult(Mapping):
        pass
    
    class GetRelationalDatabaseParametersRequest(Mapping):
        pass
    
    class GetRelationalDatabaseParametersResult(Mapping):
        pass
    
    class GetRelationalDatabaseSnapshotRequest(Mapping):
        pass
    
    class GetRelationalDatabaseSnapshotResult(Mapping):
        pass
    
    class GetRelationalDatabaseSnapshotsRequest(Mapping):
        pass
    
    class GetRelationalDatabaseSnapshotsResult(Mapping):
        pass
    
    class GetRelationalDatabasesRequest(Mapping):
        pass
    
    class GetRelationalDatabasesResult(Mapping):
        pass
    
    class GetStaticIpRequest(Mapping):
        pass
    
    class GetStaticIpResult(Mapping):
        pass
    
    class GetStaticIpsRequest(Mapping):
        pass
    
    class GetStaticIpsResult(Mapping):
        pass
    
    class ImportKeyPairRequest(Mapping):
        pass
    
    class ImportKeyPairResult(Mapping):
        pass
    
    class IsVpcPeeredRequest(Mapping):
        pass
    
    class IsVpcPeeredResult(Mapping):
        pass
    
    class OpenInstancePublicPortsRequest(Mapping):
        pass
    
    class OpenInstancePublicPortsResult(Mapping):
        pass
    
    class PeerVpcRequest(Mapping):
        pass
    
    class PeerVpcResult(Mapping):
        pass
    
    class PutInstancePublicPortsRequest(Mapping):
        pass
    
    class PutInstancePublicPortsResult(Mapping):
        pass
    
    class RebootInstanceRequest(Mapping):
        pass
    
    class RebootInstanceResult(Mapping):
        pass
    
    class RebootRelationalDatabaseRequest(Mapping):
        pass
    
    class RebootRelationalDatabaseResult(Mapping):
        pass
    
    class ReleaseStaticIpRequest(Mapping):
        pass
    
    class ReleaseStaticIpResult(Mapping):
        pass
    
    class StartInstanceRequest(Mapping):
        pass
    
    class StartInstanceResult(Mapping):
        pass
    
    class StartRelationalDatabaseRequest(Mapping):
        pass
    
    class StartRelationalDatabaseResult(Mapping):
        pass
    
    class StopInstanceRequest(Mapping):
        pass
    
    class StopInstanceResult(Mapping):
        pass
    
    class StopRelationalDatabaseRequest(Mapping):
        pass
    
    class StopRelationalDatabaseResult(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResult(Mapping):
        pass
    
    class UnpeerVpcRequest(Mapping):
        pass
    
    class UnpeerVpcResult(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResult(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateDomainEntryRequest(Mapping):
        pass
    
    class UpdateDomainEntryResult(Mapping):
        pass
    
    class UpdateLoadBalancerAttributeRequest(Mapping):
        pass
    
    class UpdateLoadBalancerAttributeResult(Mapping):
        pass
    
    class UpdateRelationalDatabaseRequest(Mapping):
        pass
    
    class UpdateRelationalDatabaseResult(Mapping):
        pass
    
    class UpdateRelationalDatabaseParametersRequest(Mapping):
        pass
    
    class UpdateRelationalDatabaseParametersResult(Mapping):
        pass
    
    class RelationalDatabaseParameter(Mapping):
        pass
    
    def allocate_static_ip(self, staticIpName: str) -> AllocateStaticIpResult:
        """Allocates a static IP address.

        :param self:
        :param staticIpName:  The name of the static IP address.<br/>
        :return: operations:  An array of key-value pairs containing information about the static IP address you allocated.<br/>        """
        pass

    def attach_disk(self, diskPath: str, instanceName: str, diskName: str) -> AttachDiskResult:
        """Attaches a block storage disk to a running or stopped Lightsail instance and exposes it to the instance with the specified disk name. The <code>attach disk</code> operation supports tag-based access control via resource tags applied to the resource identified by diskName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param diskPath:  The disk path to expose to the instance (e.g., <code>/dev/xvdf</code>).<br/>instanceName:  The name of the Lightsail instance where you want to utilize the storage disk.<br/>diskName:  The unique Lightsail disk name (e.g., <code>my-disk</code>).<br/>
        :return: operations:  An object describing the API operations.<br/>        """
        pass

    def attach_instances_to_load_balancer(self, instanceNames: List[ResourceName], loadBalancerName: str) -> AttachInstancesToLoadBalancerResult:
        """Attaches one or more Lightsail instances to a load balancer. After some time, the instances are attached to the load balancer and the health check status is available. The <code>attach instances to load balancer</code> operation supports tag-based access control via resource tags applied to the resource identified by loadBalancerName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param instanceNames:  An array of strings representing the instance name(s) you want to attach to your load balancer. An instance must be <code>running</code> before you can attach it to your load balancer. There are no additional limits on the number of instances you can attach to your load balancer, aside from the limit of Lightsail instances you can create in your account (20).<br/>loadBalancerName:  The name of the load balancer.<br/>
        :return: operations:  An object representing the API operations.<br/>        """
        pass

    def attach_load_balancer_tls_certificate(self, certificateName: str, loadBalancerName: str) -> AttachLoadBalancerTlsCertificateResult:
        """Attaches a Transport Layer Security (TLS) certificate to your load balancer. TLS is just an updated, more secure version of Secure Socket Layer (SSL). Once you create and validate your certificate, you can attach it to your load balancer. You can also use this API to rotate the certificates on your account. Use the <code>AttachLoadBalancerTlsCertificate</code> operation with the non-attached certificate, and it will replace the existing one and become the attached certificate. The <code>attach load balancer tls certificate</code> operation supports tag-based access control via resource tags applied to the resource identified by loadBalancerName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param certificateName:  The name of your SSL/TLS certificate.<br/>loadBalancerName:  The name of the load balancer to which you want to associate the SSL/TLS certificate.<br/>
        :return: operations:  An object representing the API operations. These SSL/TLS certificates are only usable by Lightsail load balancers. You can't get the certificate and use it for another purpose.<br/>        """
        pass

    def attach_static_ip(self, instanceName: str, staticIpName: str) -> AttachStaticIpResult:
        """Attaches a static IP address to a specific Amazon Lightsail instance.

        :param self:
        :param instanceName:  The instance name to which you want to attach the static IP address.<br/>staticIpName:  The name of the static IP.<br/>
        :return: operations:  An array of key-value pairs containing information about your API operations.<br/>        """
        pass

    def close_instance_public_ports(self, instanceName: str, portInfo: PortInfo) -> CloseInstancePublicPortsResult:
        """Closes the public ports on a specific Amazon Lightsail instance. The <code>close instance public ports</code> operation supports tag-based access control via resource tags applied to the resource identified by instanceName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param instanceName:  The name of the instance on which you're attempting to close the public ports.<br/>portInfo:  Information about the public port you are trying to close.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>protocol</i> The protocol. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>toPort</i> The last port in the range.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fromPort</i> The first port in the range.<br/>
        :return: operation:  An array of key-value pairs that contains information about the operation.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorDetails</i> The error details.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorCode</i> The error code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusChangedAt</i> The timestamp when the status was changed (e.g., <code>1479816991.349</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>operationType</i> The type of operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>operationDetails</i> Details about the operation (e.g., <code>Debian-1GB-Ohio-1</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>isTerminal</i> A Boolean value indicating whether the operation is terminal.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The region and Availability Zone.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>location</b>: The region and Availability Zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The timestamp when the operation was initialized (e.g., <code>1479816991.349</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceType</i> The resource type. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceName</i> The resource name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>regionName</i> The AWS Region name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>availabilityZone</i> The Availability Zone. Follows the format <code>us-east-2a</code> (case-sensitive).<br/>        """
        pass

    def copy_snapshot(self, sourceRegion: str, targetSnapshotName: str, sourceSnapshotName: str) -> CopySnapshotResult:
        """Copies an instance or disk snapshot from one AWS Region to another in Amazon Lightsail.

        :param self:
        :param sourceRegion:  The AWS Region where the source snapshot is located.<br/>targetSnapshotName:  The name of the new instance or disk snapshot to be created as a copy.<br/>sourceSnapshotName:  The name of the source instance or disk snapshot to be copied.<br/>
        :return: operations:  A list of objects describing the API operation.<br/>        """
        pass

    def create_cloud_formation_stack(self, instances: List[InstanceEntry]) -> CreateCloudFormationStackResult:
        """Creates an AWS CloudFormation stack, which creates a new Amazon EC2 instance from an exported Amazon Lightsail snapshot. This operation results in a CloudFormation stack record that can be used to track the AWS CloudFormation stack created. Use the <code>get cloud formation stack records</code> operation to get a list of the CloudFormation stacks created. <important> Wait until after your new Amazon EC2 instance is created before running the <code>create cloud formation stack</code> operation again with the same export snapshot record. </important>

        :param self:
        :param instances:  An array of parameters that will be used to create the new Amazon EC2 instance. You can only pass one instance entry at a time in this array. You will get an invalid parameter error if you pass more than one instance entry in this array.<br/>
        :return: operations:  A list of objects describing the API operation.<br/>        """
        pass

    def create_disk(self, sizeInGb: int, availabilityZone: str, diskName: str, tags: List[Tag]=None) -> CreateDiskResult:
        """Creates a block storage disk that can be attached to a Lightsail instance in the same Availability Zone (e.g., <code>us-east-2a</code>). The disk is created in the regional endpoint that you send the HTTP request to. For more information, see <a href="https://lightsail.aws.amazon.com/ls/docs/overview/article/understanding-regions-and-availability-zones-in-amazon-lightsail">Regions and Availability Zones in Lightsail</a>. The <code>create disk</code> operation supports tag-based access control via request tags. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param tags:  The tag keys and optional values to add to the resource during create. To tag a resource after it has been created, see the <code>tag resource</code> operation.<br/>sizeInGb:  The size of the disk in GB (e.g., <code>32</code>).<br/>availabilityZone:  The Availability Zone where you want to create the disk (e.g., <code>us-east-2a</code>). Choose the same Availability Zone as the Lightsail instance where you want to create the disk. Use the GetRegions operation to list the Availability Zones where Lightsail is currently available.<br/>diskName:  The unique Lightsail disk name (e.g., <code>my-disk</code>).<br/>
        :return: operations:  An object describing the API operations.<br/>        """
        pass

    def create_disk_from_snapshot(self, sizeInGb: int, availabilityZone: str, diskSnapshotName: str, diskName: str, tags: List[Tag]=None) -> CreateDiskFromSnapshotResult:
        """Creates a block storage disk from a disk snapshot that can be attached to a Lightsail instance in the same Availability Zone (e.g., <code>us-east-2a</code>). The disk is created in the regional endpoint that you send the HTTP request to. For more information, see <a href="https://lightsail.aws.amazon.com/ls/docs/overview/article/understanding-regions-and-availability-zones-in-amazon-lightsail">Regions and Availability Zones in Lightsail</a>. The <code>create disk from snapshot</code> operation supports tag-based access control via request tags and resource tags applied to the resource identified by diskSnapshotName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param tags:  The tag keys and optional values to add to the resource during create. To tag a resource after it has been created, see the <code>tag resource</code> operation.<br/>sizeInGb:  The size of the disk in GB (e.g., <code>32</code>).<br/>availabilityZone:  The Availability Zone where you want to create the disk (e.g., <code>us-east-2a</code>). Choose the same Availability Zone as the Lightsail instance where you want to create the disk. Use the GetRegions operation to list the Availability Zones where Lightsail is currently available.<br/>diskSnapshotName:  The name of the disk snapshot (e.g., <code>my-snapshot</code>) from which to create the new storage disk.<br/>diskName:  The unique Lightsail disk name (e.g., <code>my-disk</code>).<br/>
        :return: operations:  An object describing the API operations.<br/>        """
        pass

    def create_disk_snapshot(self, diskSnapshotName: str, diskName: str=None, instanceName: str=None, tags: List[Tag]=None) -> CreateDiskSnapshotResult:
        """Creates a snapshot of a block storage disk. You can use snapshots for backups, to make copies of disks, and to save data before shutting down a Lightsail instance. You can take a snapshot of an attached disk that is in use; however, snapshots only capture data that has been written to your disk at the time the snapshot command is issued. This may exclude any data that has been cached by any applications or the operating system. If you can pause any file systems on the disk long enough to take a snapshot, your snapshot should be complete. Nevertheless, if you cannot pause all file writes to the disk, you should unmount the disk from within the Lightsail instance, issue the create disk snapshot command, and then remount the disk to ensure a consistent and complete snapshot. You may remount and use your disk while the snapshot status is pending. You can also use this operation to create a snapshot of an instance's system volume. You might want to do this, for example, to recover data from the system volume of a botched instance or to create a backup of the system volume like you would for a block storage disk. To create a snapshot of a system volume, just define the <code>instance name</code> parameter when issuing the snapshot command, and a snapshot of the defined instance's system volume will be created. After the snapshot is available, you can create a block storage disk from the snapshot and attach it to a running instance to access the data on the disk. The <code>create disk snapshot</code> operation supports tag-based access control via request tags. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param tags:  The tag keys and optional values to add to the resource during create. To tag a resource after it has been created, see the <code>tag resource</code> operation.<br/>instanceName:  The unique name of the source instance (e.g., <code>Amazon_Linux-512MB-Virginia-1</code>). When this is defined, a snapshot of the instance's system volume is created. <note> This parameter cannot be defined together with the <code>disk name</code> parameter. The <code>instance name</code> and <code>disk name</code> parameters are mutually exclusive. </note><br/>diskSnapshotName:  The name of the destination disk snapshot (e.g., <code>my-disk-snapshot</code>) based on the source disk.<br/>diskName:  The unique name of the source disk (e.g., <code>Disk-Virginia-1</code>). <note> This parameter cannot be defined together with the <code>instance name</code> parameter. The <code>disk name</code> and <code>instance name</code> parameters are mutually exclusive. </note><br/>
        :return: operations:  An object describing the API operations.<br/>        """
        pass

    def create_domain(self, domainName: str, tags: List[Tag]=None) -> CreateDomainResult:
        """Creates a domain resource for the specified domain (e.g., example.com). The <code>create domain</code> operation supports tag-based access control via request tags. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param tags:  The tag keys and optional values to add to the resource during create. To tag a resource after it has been created, see the <code>tag resource</code> operation.<br/>domainName:  The domain name to manage (e.g., <code>example.com</code>). <note> You cannot register a new domain name using Lightsail. You must register a domain name using Amazon Route 53 or another domain name registrar. If you have already registered your domain, you can enter its name in this parameter to manage the DNS records for that domain. </note><br/>
        :return: operation:  An array of key-value pairs containing information about the domain resource you created.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorDetails</i> The error details.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorCode</i> The error code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusChangedAt</i> The timestamp when the status was changed (e.g., <code>1479816991.349</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>operationType</i> The type of operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>operationDetails</i> Details about the operation (e.g., <code>Debian-1GB-Ohio-1</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>isTerminal</i> A Boolean value indicating whether the operation is terminal.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The region and Availability Zone.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>location</b>: The region and Availability Zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The timestamp when the operation was initialized (e.g., <code>1479816991.349</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceType</i> The resource type. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceName</i> The resource name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>regionName</i> The AWS Region name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>availabilityZone</i> The Availability Zone. Follows the format <code>us-east-2a</code> (case-sensitive).<br/>        """
        pass

    def create_domain_entry(self, domainEntry: DomainEntry, domainName: str) -> CreateDomainEntryResult:
        """Creates one of the following entry records associated with the domain: Address (A), canonical name (CNAME), mail exchanger (MX), name server (NS), start of authority (SOA), service locator (SRV), or text (TXT). The <code>create domain entry</code> operation supports tag-based access control via resource tags applied to the resource identified by domainName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param domainEntry:  An array of key-value pairs containing information about the domain entry request.:<br/>domainName:  The domain name (e.g., <code>example.com</code>) for which you want to create the domain entry.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>options</i> (Deprecated) The options for the domain entry. <note> In releases prior to November 29, 2017, this parameter was not included in the API response. It is now deprecated. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of domain entry, such as address (A), canonical name (CNAME), mail exchanger (MX), name server (NS), start of authority (SOA), service locator (SRV), or text (TXT). The following domain entry types can be used: <ul> <li>  <code>A</code>  </li> <li>  <code>CNAME</code>  </li> <li>  <code>MX</code>  </li> <li>  <code>NS</code>  </li> <li>  <code>SOA</code>  </li> <li>  <code>SRV</code>  </li> <li>  <code>TXT</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>isAlias</i> When <code>true</code>, specifies whether the domain entry is an alias used by the Lightsail load balancer. You can include an alias (A type) record in your request, which points to a load balancer DNS name and routes traffic to your load balancer<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>target</i> The target AWS name server (e.g., <code>ns-111.awsdns-22.com.</code>). For Lightsail load balancers, the value looks like <code>ab1234c56789c6b86aba6fb203d443bc-123456789.us-east-2.elb.amazonaws.com</code>. Be sure to also set <code>isAlias</code> to <code>true</code> when setting up an A record for a load balancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the domain recordset entry.<br/>
        :return: operation:  An array of key-value pairs containing information about the operation.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorDetails</i> The error details.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorCode</i> The error code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusChangedAt</i> The timestamp when the status was changed (e.g., <code>1479816991.349</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>operationType</i> The type of operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>operationDetails</i> Details about the operation (e.g., <code>Debian-1GB-Ohio-1</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>isTerminal</i> A Boolean value indicating whether the operation is terminal.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The region and Availability Zone.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>location</b>: The region and Availability Zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The timestamp when the operation was initialized (e.g., <code>1479816991.349</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceType</i> The resource type. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceName</i> The resource name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>regionName</i> The AWS Region name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>availabilityZone</i> The Availability Zone. Follows the format <code>us-east-2a</code> (case-sensitive).<br/>        """
        pass

    def create_instance_snapshot(self, instanceName: str, instanceSnapshotName: str, tags: List[Tag]=None) -> CreateInstanceSnapshotResult:
        """Creates a snapshot of a specific virtual private server, or <i>instance</i>. You can use a snapshot to create a new instance that is based on that snapshot. The <code>create instance snapshot</code> operation supports tag-based access control via request tags. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param tags:  The tag keys and optional values to add to the resource during create. To tag a resource after it has been created, see the <code>tag resource</code> operation.<br/>instanceName:  The Lightsail instance on which to base your snapshot.<br/>instanceSnapshotName:  The name for your new snapshot.<br/>
        :return: operations:  An array of key-value pairs containing information about the results of your create instances snapshot request.<br/>        """
        pass

    def create_instances(self, bundleId: str, blueprintId: str, availabilityZone: str, instanceNames: List[string], customImageName: str=None, userData: str=None, keyPairName: str=None, tags: List[Tag]=None) -> CreateInstancesResult:
        """Creates one or more Amazon Lightsail virtual private servers, or <i>instances</i>. Create instances using active blueprints. Inactive blueprints are listed to support customers with existing instances but are not necessarily available for launch of new instances. Blueprints are marked inactive when they become outdated due to operating system updates or new application releases. Use the get blueprints operation to return a list of available blueprints. The <code>create instances</code> operation supports tag-based access control via request tags. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param tags:  The tag keys and optional values to add to the resource during create. To tag a resource after it has been created, see the <code>tag resource</code> operation.<br/>keyPairName:  The name of your key pair.<br/>userData:  A launch script you can create that configures a server with additional user data. For example, you might want to run <code>apt-get -y update</code>. <note> Depending on the machine image you choose, the command to get software on your instance varies. Amazon Linux and CentOS use <code>yum</code>, Debian and Ubuntu use <code>apt-get</code>, and FreeBSD uses <code>pkg</code>. For a complete list, see the <a href="https://lightsail.aws.amazon.com/ls/docs/getting-started/article/compare-options-choose-lightsail-instance-image">Dev Guide</a>. </note><br/>bundleId:  The bundle of specification information for your virtual private server (or <i>instance</i>), including the pricing plan (e.g., <code>micro_1_0</code>).<br/>blueprintId:  The ID for a virtual private server image (e.g., <code>app_wordpress_4_4</code> or <code>app_lamp_7_0</code>). Use the get blueprints operation to return a list of available images (or <i>blueprints</i>).<br/>customImageName:  (Deprecated) The name for your custom image. <note> In releases prior to June 12, 2017, this parameter was ignored by the API. It is now deprecated. </note><br/>availabilityZone:  The Availability Zone in which to create your instance. Use the following format: <code>us-east-2a</code> (case sensitive). You can get a list of Availability Zones by using the <a href="http://docs.aws.amazon.com/lightsail/2016-11-28/api-reference/API_GetRegions.html">get regions</a> operation. Be sure to add the <code>include Availability Zones</code> parameter to your request.<br/>instanceNames:  The names to use for your new Lightsail instances. Separate multiple values using quotation marks and commas, for example: <code>["MyFirstInstance","MySecondInstance"]</code> <br/>
        :return: operations:  An array of key-value pairs containing information about the results of your create instances request.<br/>        """
        pass

    def create_instances_from_snapshot(self, bundleId: str, instanceSnapshotName: str, availabilityZone: str, instanceNames: List[string], attachedDiskMapping: AttachedDiskMap=None, userData: str=None, keyPairName: str=None, tags: List[Tag]=None) -> CreateInstancesFromSnapshotResult:
        """Uses a specific snapshot as a blueprint for creating one or more new instances that are based on that identical configuration. The <code>create instances from snapshot</code> operation supports tag-based access control via request tags and resource tags applied to the resource identified by instanceSnapshotName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param tags:  The tag keys and optional values to add to the resource during create. To tag a resource after it has been created, see the <code>tag resource</code> operation.<br/>keyPairName:  The name for your key pair.<br/>userData:  You can create a launch script that configures a server with additional user data. For example, <code>apt-get -y update</code>. <note> Depending on the machine image you choose, the command to get software on your instance varies. Amazon Linux and CentOS use <code>yum</code>, Debian and Ubuntu use <code>apt-get</code>, and FreeBSD uses <code>pkg</code>. For a complete list, see the <a href="https://lightsail.aws.amazon.com/ls/docs/getting-started/article/compare-options-choose-lightsail-instance-image">Dev Guide</a>. </note><br/>bundleId:  The bundle of specification information for your virtual private server (or <i>instance</i>), including the pricing plan (e.g., <code>micro_1_0</code>).<br/>instanceSnapshotName:  The name of the instance snapshot on which you are basing your new instances. Use the get instance snapshots operation to return information about your existing snapshots.<br/>availabilityZone:  The Availability Zone where you want to create your instances. Use the following formatting: <code>us-east-2a</code> (case sensitive). You can get a list of Availability Zones by using the <a href="http://docs.aws.amazon.com/lightsail/2016-11-28/api-reference/API_GetRegions.html">get regions</a> operation. Be sure to add the <code>include Availability Zones</code> parameter to your request.<br/>attachedDiskMapping:  An object containing information about one or more disk mappings.<br/>instanceNames:  The names for your new instances.<br/>
        :return: operations:  An array of key-value pairs containing information about the results of your create instances from snapshot request.<br/>        """
        pass

    def create_key_pair(self, keyPairName: str, tags: List[Tag]=None) -> CreateKeyPairResult:
        """Creates an SSH key pair. The <code>create key pair</code> operation supports tag-based access control via request tags. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param tags:  The tag keys and optional values to add to the resource during create. To tag a resource after it has been created, see the <code>tag resource</code> operation.<br/>keyPairName:  The name for your new key pair.<br/>
        :return: operation:  An array of key-value pairs containing information about the results of your create key pair request.:<br/>privateKeyBase64:  A base64-encoded RSA private key.<br/>publicKeyBase64:  A base64-encoded public key of the <code>ssh-rsa</code> type.<br/>keyPair:  An array of key-value pairs containing information about the new key pair you just created.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fingerprint</i> The RSA fingerprint of the key pair.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> The tag keys and optional values for the resource. For more information about tags in Lightsail, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-tags">Lightsail Dev Guide</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceType</i> The resource type (usually <code>KeyPair</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The region name and Availability Zone where the key pair was created.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>location</b>: The region name and Availability Zone where the key pair was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The timestamp when the key pair was created (e.g., <code>1479816991.349</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>supportCode</i> The support code. Include this code in your email to support when you have questions about an instance or another resource in Lightsail. This code enables our support team to look up your Lightsail information more easily.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the key pair (e.g., <code>arn:aws:lightsail:us-east-2:123456789101:KeyPair/05859e3d-331d-48ba-9034-12345EXAMPLE</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The friendly name of the SSH key pair.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>regionName</i> The AWS Region name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>availabilityZone</i> The Availability Zone. Follows the format <code>us-east-2a</code> (case-sensitive).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorDetails</i> The error details.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorCode</i> The error code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusChangedAt</i> The timestamp when the status was changed (e.g., <code>1479816991.349</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>operationType</i> The type of operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>operationDetails</i> Details about the operation (e.g., <code>Debian-1GB-Ohio-1</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>isTerminal</i> A Boolean value indicating whether the operation is terminal.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The region and Availability Zone.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>location</b>: The region and Availability Zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The timestamp when the operation was initialized (e.g., <code>1479816991.349</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceType</i> The resource type. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceName</i> The resource name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>regionName</i> The AWS Region name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>availabilityZone</i> The Availability Zone. Follows the format <code>us-east-2a</code> (case-sensitive).<br/>        """
        pass

    def create_load_balancer(self, instancePort: int, loadBalancerName: str, healthCheckPath: str=None, certificateName: str=None, certificateDomainName: str=None, certificateAlternativeNames: List[DomainName]=None, tags: List[Tag]=None) -> CreateLoadBalancerResult:
        """Creates a Lightsail load balancer. To learn more about deciding whether to load balance your application, see <a href="https://lightsail.aws.amazon.com/ls/docs/how-to/article/configure-lightsail-instances-for-load-balancing">Configure your Lightsail instances for load balancing</a>. You can create up to 5 load balancers per AWS Region in your account. When you create a load balancer, you can specify a unique name and port settings. To change additional load balancer settings, use the <code>UpdateLoadBalancerAttribute</code> operation. The <code>create load balancer</code> operation supports tag-based access control via request tags. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param tags:  The tag keys and optional values to add to the resource during create. To tag a resource after it has been created, see the <code>tag resource</code> operation.<br/>certificateAlternativeNames:  The optional alternative domains and subdomains to use with your SSL/TLS certificate (e.g., <code>www.example.com</code>, <code>example.com</code>, <code>m.example.com</code>, <code>blog.example.com</code>).<br/>certificateDomainName:  The domain name with which your certificate is associated (e.g., <code>example.com</code>). If you specify <code>certificateDomainName</code>, then <code>certificateName</code> is required (and vice-versa).<br/>certificateName:  The name of the SSL/TLS certificate. If you specify <code>certificateName</code>, then <code>certificateDomainName</code> is required (and vice-versa).<br/>healthCheckPath:  The path you provided to perform the load balancer health check. If you didn't specify a health check path, Lightsail uses the root path of your website (e.g., <code>"/"</code>). You may want to specify a custom health check path other than the root of your application if your home page loads slowly or has a lot of media or scripting on it.<br/>instancePort:  The instance port where you're creating your load balancer.<br/>loadBalancerName:  The name of your load balancer.<br/>
        :return: operations:  An object containing information about the API operations.<br/>        """
        pass

    def create_load_balancer_tls_certificate(self, certificateDomainName: str, certificateName: str, loadBalancerName: str, certificateAlternativeNames: List[DomainName]=None, tags: List[Tag]=None) -> CreateLoadBalancerTlsCertificateResult:
        """Creates a Lightsail load balancer TLS certificate. TLS is just an updated, more secure version of Secure Socket Layer (SSL). The <code>create load balancer tls certificate</code> operation supports tag-based access control via resource tags applied to the resource identified by loadBalancerName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param tags:  The tag keys and optional values to add to the resource during create. To tag a resource after it has been created, see the <code>tag resource</code> operation.<br/>certificateAlternativeNames:  An array of strings listing alternative domains and subdomains for your SSL/TLS certificate. Lightsail will de-dupe the names for you. You can have a maximum of 9 alternative names (in addition to the 1 primary domain). We do not support wildcards (e.g., <code>*.example.com</code>).<br/>certificateDomainName:  The domain name (e.g., <code>example.com</code>) for your SSL/TLS certificate.<br/>certificateName:  The SSL/TLS certificate name. You can have up to 10 certificates in your account at one time. Each Lightsail load balancer can have up to 2 certificates associated with it at one time. There is also an overall limit to the number of certificates that can be issue in a 365-day period. For more information, see <a href="http://docs.aws.amazon.com/acm/latest/userguide/acm-limits.html">Limits</a>.<br/>loadBalancerName:  The load balancer name where you want to create the SSL/TLS certificate.<br/>
        :return: operations:  An object containing information about the API operations.<br/>        """
        pass

    def create_relational_database(self, masterUsername: str, masterDatabaseName: str, relationalDatabaseBundleId: str, relationalDatabaseBlueprintId: str, relationalDatabaseName: str, availabilityZone: str=None, masterUserPassword: str=None, preferredBackupWindow: str=None, preferredMaintenanceWindow: str=None, publiclyAccessible: boolean=None, tags: List[Tag]=None) -> CreateRelationalDatabaseResult:
        """Creates a new database in Amazon Lightsail. The <code>create relational database</code> operation supports tag-based access control via request tags. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param tags:  The tag keys and optional values to add to the resource during create. To tag a resource after it has been created, see the <code>tag resource</code> operation.<br/>publiclyAccessible:  Specifies the accessibility options for your new database. A value of <code>true</code> specifies a database that is available to resources outside of your Lightsail account. A value of <code>false</code> specifies a database that is available only to your Lightsail resources in the same region as your database.<br/>preferredMaintenanceWindow:  The weekly time range during which system maintenance can occur on your new database. The default is a 30-minute window selected at random from an 8-hour block of time for each AWS Region, occurring on a random day of the week. Constraints: <ul> <li> Must be in the <code>ddd:hh24:mi-ddd:hh24:mi</code> format. </li> <li> Valid days: Mon, Tue, Wed, Thu, Fri, Sat, Sun. </li> <li> Must be at least 30 minutes. </li> <li> Specified in Universal Coordinated Time (UTC). </li> <li> Example: <code>Tue:17:00-Tue:17:30</code>  </li> </ul><br/>preferredBackupWindow:  The daily time range during which automated backups are created for your new database if automated backups are enabled. The default is a 30-minute window selected at random from an 8-hour block of time for each AWS Region. For more information about the preferred backup window time blocks for each region, see the <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_WorkingWithAutomatedBackups.html#USER_WorkingWithAutomatedBackups.BackupWindow">Working With Backups</a> guide in the Amazon Relational Database Service (Amazon RDS) documentation. Constraints: <ul> <li> Must be in the <code>hh24:mi-hh24:mi</code> format. Example: <code>16:00-16:30</code>  </li> <li> Specified in Universal Coordinated Time (UTC). </li> <li> Must not conflict with the preferred maintenance window. </li> <li> Must be at least 30 minutes. </li> </ul><br/>masterUserPassword:  The password for the master user of your new database. The password can include any printable ASCII character except "/", triple-quotes, or "@". Constraints: Must contain 8 to 41 characters.<br/>masterUsername:  The master user name for your new database. Constraints: <ul> <li> Master user name is required. </li> <li> Must contain from 1 to 16 alphanumeric characters. </li> <li> The first character must be a letter. </li> <li> Cannot be a reserved word for the database engine you choose. For more information about reserved words in MySQL 5.6 or 5.7, see the Keywords and Reserved Words articles for <a href="https://dev.mysql.com/doc/refman/5.6/en/keywords.html">MySQL 5.6</a> or <a href="https://dev.mysql.com/doc/refman/5.7/en/keywords.html">MySQL 5.7</a> respectively. </li> </ul><br/>masterDatabaseName:  The name of the master database created when the Lightsail database resource is created. Constraints: <ul> <li> Must contain from 1 to 64 alphanumeric characters. </li> <li> Cannot be a word reserved by the specified database engine </li> </ul><br/>relationalDatabaseBundleId:  The bundle ID for your new database. A bundle describes the performance specifications for your database. You can get a list of database bundle IDs by using the <code>get relational database bundles</code> operation.<br/>relationalDatabaseBlueprintId:  The blueprint ID for your new database. A blueprint describes the major engine version of a database. You can get a list of database blueprints IDs by using the <code>get relational database blueprints</code> operation.<br/>availabilityZone:  The Availability Zone in which to create your new database. Use the <code>us-east-2a</code> case-sensitive format. You can get a list of Availability Zones by using the <code>get regions</code> operation. Be sure to add the <code>include relational database Availability Zones</code> parameter to your request.<br/>relationalDatabaseName:  The name to use for your new database. Constraints: <ul> <li> Must contain from 2 to 255 alphanumeric characters, or hyphens. </li> <li> The first and last character must be a letter or number. </li> </ul><br/>
        :return: operations:  An object describing the result of your create relational database request.<br/>        """
        pass

    def create_relational_database_from_snapshot(self, relationalDatabaseName: str, availabilityZone: str=None, publiclyAccessible: boolean=None, relationalDatabaseSnapshotName: str=None, relationalDatabaseBundleId: str=None, sourceRelationalDatabaseName: str=None, restoreTime: IsoDate=None, useLatestRestorableTime: boolean=None, tags: List[Tag]=None) -> CreateRelationalDatabaseFromSnapshotResult:
        """Creates a new database from an existing database snapshot in Amazon Lightsail. You can create a new database from a snapshot in if something goes wrong with your original database, or to change it to a different plan, such as a high availability or standard plan. The <code>create relational database from snapshot</code> operation supports tag-based access control via request tags and resource tags applied to the resource identified by relationalDatabaseSnapshotName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param tags:  The tag keys and optional values to add to the resource during create. To tag a resource after it has been created, see the <code>tag resource</code> operation.<br/>useLatestRestorableTime:  Specifies whether your database is restored from the latest backup time. A value of <code>true</code> restores from the latest backup time.  Default: <code>false</code>  Constraints: Cannot be specified if the <code>restore time</code> parameter is provided.<br/>restoreTime:  The date and time to restore your database from. Constraints: <ul> <li> Must be before the latest restorable time for the database. </li> <li> Cannot be specified if the <code>use latest restorable time</code> parameter is <code>true</code>. </li> <li> Specified in Universal Coordinated Time (UTC). </li> <li> Specified in the Unix time format. For example, if you wish to use a restore time of October 1, 2018, at 8 PM UTC, then you input <code>1538424000</code> as the restore time. </li> </ul><br/>sourceRelationalDatabaseName:  The name of the source database.<br/>relationalDatabaseBundleId:  The bundle ID for your new database. A bundle describes the performance specifications for your database. You can get a list of database bundle IDs by using the <code>get relational database bundles</code> operation. When creating a new database from a snapshot, you cannot choose a bundle that is smaller than the bundle of the source database.<br/>relationalDatabaseSnapshotName:  The name of the database snapshot from which to create your new database.<br/>publiclyAccessible:  Specifies the accessibility options for your new database. A value of <code>true</code> specifies a database that is available to resources outside of your Lightsail account. A value of <code>false</code> specifies a database that is available only to your Lightsail resources in the same region as your database.<br/>availabilityZone:  The Availability Zone in which to create your new database. Use the <code>us-east-2a</code> case-sensitive format. You can get a list of Availability Zones by using the <code>get regions</code> operation. Be sure to add the <code>include relational database Availability Zones</code> parameter to your request.<br/>relationalDatabaseName:  The name to use for your new database. Constraints: <ul> <li> Must contain from 2 to 255 alphanumeric characters, or hyphens. </li> <li> The first and last character must be a letter or number. </li> </ul><br/>
        :return: operations:  An object describing the result of your create relational database from snapshot request.<br/>        """
        pass

    def create_relational_database_snapshot(self, relationalDatabaseSnapshotName: str, relationalDatabaseName: str, tags: List[Tag]=None) -> CreateRelationalDatabaseSnapshotResult:
        """Creates a snapshot of your database in Amazon Lightsail. You can use snapshots for backups, to make copies of a database, and to save data before deleting a database. The <code>create relational database snapshot</code> operation supports tag-based access control via request tags. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param tags:  The tag keys and optional values to add to the resource during create. To tag a resource after it has been created, see the <code>tag resource</code> operation.<br/>relationalDatabaseSnapshotName:  The name for your new database snapshot. Constraints: <ul> <li> Must contain from 2 to 255 alphanumeric characters, or hyphens. </li> <li> The first and last character must be a letter or number. </li> </ul><br/>relationalDatabaseName:  The name of the database on which to base your new snapshot.<br/>
        :return: operations:  An object describing the result of your create relational database snapshot request.<br/>        """
        pass

    def delete_disk(self, diskName: str) -> DeleteDiskResult:
        """Deletes the specified block storage disk. The disk must be in the <code>available</code> state (not attached to a Lightsail instance). <note> The disk may remain in the <code>deleting</code> state for several minutes. </note> The <code>delete disk</code> operation supports tag-based access control via resource tags applied to the resource identified by diskName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param diskName:  The unique name of the disk you want to delete (e.g., <code>my-disk</code>).<br/>
        :return: operations:  An object describing the API operations.<br/>        """
        pass

    def delete_disk_snapshot(self, diskSnapshotName: str) -> DeleteDiskSnapshotResult:
        """Deletes the specified disk snapshot. When you make periodic snapshots of a disk, the snapshots are incremental, and only the blocks on the device that have changed since your last snapshot are saved in the new snapshot. When you delete a snapshot, only the data not needed for any other snapshot is removed. So regardless of which prior snapshots have been deleted, all active snapshots will have access to all the information needed to restore the disk. The <code>delete disk snapshot</code> operation supports tag-based access control via resource tags applied to the resource identified by diskSnapshotName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param diskSnapshotName:  The name of the disk snapshot you want to delete (e.g., <code>my-disk-snapshot</code>).<br/>
        :return: operations:  An object describing the API operations.<br/>        """
        pass

    def delete_domain(self, domainName: str) -> DeleteDomainResult:
        """Deletes the specified domain recordset and all of its domain records. The <code>delete domain</code> operation supports tag-based access control via resource tags applied to the resource identified by domainName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param domainName:  The specific domain name to delete.<br/>
        :return: operation:  An array of key-value pairs containing information about the results of your delete domain request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorDetails</i> The error details.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorCode</i> The error code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusChangedAt</i> The timestamp when the status was changed (e.g., <code>1479816991.349</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>operationType</i> The type of operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>operationDetails</i> Details about the operation (e.g., <code>Debian-1GB-Ohio-1</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>isTerminal</i> A Boolean value indicating whether the operation is terminal.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The region and Availability Zone.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>location</b>: The region and Availability Zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The timestamp when the operation was initialized (e.g., <code>1479816991.349</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceType</i> The resource type. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceName</i> The resource name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>regionName</i> The AWS Region name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>availabilityZone</i> The Availability Zone. Follows the format <code>us-east-2a</code> (case-sensitive).<br/>        """
        pass

    def delete_domain_entry(self, domainEntry: DomainEntry, domainName: str) -> DeleteDomainEntryResult:
        """Deletes a specific domain entry. The <code>delete domain entry</code> operation supports tag-based access control via resource tags applied to the resource identified by domainName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param domainEntry:  An array of key-value pairs containing information about your domain entries.:<br/>domainName:  The name of the domain entry to delete.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>options</i> (Deprecated) The options for the domain entry. <note> In releases prior to November 29, 2017, this parameter was not included in the API response. It is now deprecated. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of domain entry, such as address (A), canonical name (CNAME), mail exchanger (MX), name server (NS), start of authority (SOA), service locator (SRV), or text (TXT). The following domain entry types can be used: <ul> <li>  <code>A</code>  </li> <li>  <code>CNAME</code>  </li> <li>  <code>MX</code>  </li> <li>  <code>NS</code>  </li> <li>  <code>SOA</code>  </li> <li>  <code>SRV</code>  </li> <li>  <code>TXT</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>isAlias</i> When <code>true</code>, specifies whether the domain entry is an alias used by the Lightsail load balancer. You can include an alias (A type) record in your request, which points to a load balancer DNS name and routes traffic to your load balancer<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>target</i> The target AWS name server (e.g., <code>ns-111.awsdns-22.com.</code>). For Lightsail load balancers, the value looks like <code>ab1234c56789c6b86aba6fb203d443bc-123456789.us-east-2.elb.amazonaws.com</code>. Be sure to also set <code>isAlias</code> to <code>true</code> when setting up an A record for a load balancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the domain recordset entry.<br/>
        :return: operation:  An array of key-value pairs containing information about the results of your delete domain entry request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorDetails</i> The error details.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorCode</i> The error code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusChangedAt</i> The timestamp when the status was changed (e.g., <code>1479816991.349</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>operationType</i> The type of operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>operationDetails</i> Details about the operation (e.g., <code>Debian-1GB-Ohio-1</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>isTerminal</i> A Boolean value indicating whether the operation is terminal.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The region and Availability Zone.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>location</b>: The region and Availability Zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The timestamp when the operation was initialized (e.g., <code>1479816991.349</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceType</i> The resource type. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceName</i> The resource name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>regionName</i> The AWS Region name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>availabilityZone</i> The Availability Zone. Follows the format <code>us-east-2a</code> (case-sensitive).<br/>        """
        pass

    def delete_instance(self, instanceName: str) -> DeleteInstanceResult:
        """Deletes a specific Amazon Lightsail virtual private server, or <i>instance</i>. The <code>delete instance</code> operation supports tag-based access control via resource tags applied to the resource identified by instanceName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param instanceName:  The name of the instance to delete.<br/>
        :return: operations:  An array of key-value pairs containing information about the results of your delete instance request.<br/>        """
        pass

    def delete_instance_snapshot(self, instanceSnapshotName: str) -> DeleteInstanceSnapshotResult:
        """Deletes a specific snapshot of a virtual private server (or <i>instance</i>). The <code>delete instance snapshot</code> operation supports tag-based access control via resource tags applied to the resource identified by instanceSnapshotName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param instanceSnapshotName:  The name of the snapshot to delete.<br/>
        :return: operations:  An array of key-value pairs containing information about the results of your delete instance snapshot request.<br/>        """
        pass

    def delete_key_pair(self, keyPairName: str) -> DeleteKeyPairResult:
        """Deletes a specific SSH key pair. The <code>delete key pair</code> operation supports tag-based access control via resource tags applied to the resource identified by keyPairName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param keyPairName:  The name of the key pair to delete.<br/>
        :return: operation:  An array of key-value pairs containing information about the results of your delete key pair request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorDetails</i> The error details.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorCode</i> The error code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusChangedAt</i> The timestamp when the status was changed (e.g., <code>1479816991.349</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>operationType</i> The type of operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>operationDetails</i> Details about the operation (e.g., <code>Debian-1GB-Ohio-1</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>isTerminal</i> A Boolean value indicating whether the operation is terminal.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The region and Availability Zone.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>location</b>: The region and Availability Zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The timestamp when the operation was initialized (e.g., <code>1479816991.349</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceType</i> The resource type. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceName</i> The resource name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>regionName</i> The AWS Region name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>availabilityZone</i> The Availability Zone. Follows the format <code>us-east-2a</code> (case-sensitive).<br/>        """
        pass

    def delete_known_host_keys(self, instanceName: str) -> DeleteKnownHostKeysResult:
        """Deletes the known host key or certificate used by the Amazon Lightsail browser-based SSH or RDP clients to authenticate an instance. This operation enables the Lightsail browser-based SSH or RDP clients to connect to the instance after a host key mismatch. <important> Perform this operation only if you were expecting the host key or certificate mismatch or if you are familiar with the new host key or certificate on the instance. For more information, see <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-troubleshooting-browser-based-ssh-rdp-client-connection">Troubleshooting connection issues when using the Amazon Lightsail browser-based SSH or RDP client</a>. </important>

        :param self:
        :param instanceName:  The name of the instance for which you want to reset the host key or certificate.<br/>
        :return: operations:  A list of objects describing the API operation.<br/>        """
        pass

    def delete_load_balancer(self, loadBalancerName: str) -> DeleteLoadBalancerResult:
        """Deletes a Lightsail load balancer and all its associated SSL/TLS certificates. Once the load balancer is deleted, you will need to create a new load balancer, create a new certificate, and verify domain ownership again. The <code>delete load balancer</code> operation supports tag-based access control via resource tags applied to the resource identified by loadBalancerName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param loadBalancerName:  The name of the load balancer you want to delete.<br/>
        :return: operations:  An object describing the API operations.<br/>        """
        pass

    def delete_load_balancer_tls_certificate(self, certificateName: str, loadBalancerName: str, force: boolean=None) -> DeleteLoadBalancerTlsCertificateResult:
        """Deletes an SSL/TLS certificate associated with a Lightsail load balancer. The <code>delete load balancer tls certificate</code> operation supports tag-based access control via resource tags applied to the resource identified by loadBalancerName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param force:  When <code>true</code>, forces the deletion of an SSL/TLS certificate. There can be two certificates associated with a Lightsail load balancer: the primary and the backup. The <code>force</code> parameter is required when the primary SSL/TLS certificate is in use by an instance attached to the load balancer.<br/>certificateName:  The SSL/TLS certificate name.<br/>loadBalancerName:  The load balancer name.<br/>
        :return: operations:  An object describing the API operations.<br/>        """
        pass

    def delete_relational_database(self, relationalDatabaseName: str, skipFinalSnapshot: boolean=None, finalRelationalDatabaseSnapshotName: str=None) -> DeleteRelationalDatabaseResult:
        """Deletes a database in Amazon Lightsail. The <code>delete relational database</code> operation supports tag-based access control via resource tags applied to the resource identified by relationalDatabaseName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param finalRelationalDatabaseSnapshotName:  The name of the database snapshot created if <code>skip final snapshot</code> is <code>false</code>, which is the default value for that parameter. <note> Specifying this parameter and also specifying the <code>skip final snapshot</code> parameter to <code>true</code> results in an error. </note> Constraints: <ul> <li> Must contain from 2 to 255 alphanumeric characters, or hyphens. </li> <li> The first and last character must be a letter or number. </li> </ul><br/>skipFinalSnapshot:  Determines whether a final database snapshot is created before your database is deleted. If <code>true</code> is specified, no database snapshot is created. If <code>false</code> is specified, a database snapshot is created before your database is deleted. You must specify the <code>final relational database snapshot name</code> parameter if the <code>skip final snapshot</code> parameter is <code>false</code>. Default: <code>false</code> <br/>relationalDatabaseName:  The name of the database that you are deleting.<br/>
        :return: operations:  An object describing the result of your delete relational database request.<br/>        """
        pass

    def delete_relational_database_snapshot(self, relationalDatabaseSnapshotName: str) -> DeleteRelationalDatabaseSnapshotResult:
        """Deletes a database snapshot in Amazon Lightsail. The <code>delete relational database snapshot</code> operation supports tag-based access control via resource tags applied to the resource identified by relationalDatabaseName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param relationalDatabaseSnapshotName:  The name of the database snapshot that you are deleting.<br/>
        :return: operations:  An object describing the result of your delete relational database snapshot request.<br/>        """
        pass

    def detach_disk(self, diskName: str) -> DetachDiskResult:
        """Detaches a stopped block storage disk from a Lightsail instance. Make sure to unmount any file systems on the device within your operating system before stopping the instance and detaching the disk. The <code>detach disk</code> operation supports tag-based access control via resource tags applied to the resource identified by diskName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param diskName:  The unique name of the disk you want to detach from your instance (e.g., <code>my-disk</code>).<br/>
        :return: operations:  An object describing the API operations.<br/>        """
        pass

    def detach_instances_from_load_balancer(self, instanceNames: List[ResourceName], loadBalancerName: str) -> DetachInstancesFromLoadBalancerResult:
        """Detaches the specified instances from a Lightsail load balancer. This operation waits until the instances are no longer needed before they are detached from the load balancer. The <code>detach instances from load balancer</code> operation supports tag-based access control via resource tags applied to the resource identified by loadBalancerName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param instanceNames:  An array of strings containing the names of the instances you want to detach from the load balancer.<br/>loadBalancerName:  The name of the Lightsail load balancer.<br/>
        :return: operations:  An object describing the API operations.<br/>        """
        pass

    def detach_static_ip(self, staticIpName: str) -> DetachStaticIpResult:
        """Detaches a static IP from the Amazon Lightsail instance to which it is attached.

        :param self:
        :param staticIpName:  The name of the static IP to detach from the instance.<br/>
        :return: operations:  An array of key-value pairs containing information about the results of your detach static IP request.<br/>        """
        pass

    def download_default_key_pair(self) -> DownloadDefaultKeyPairResult:
        """Downloads the default SSH key pair from the user's account.

        :param self:
        :param 
        :return: privateKeyBase64:  A base64-encoded RSA private key.<br/>publicKeyBase64:  A base64-encoded public key of the <code>ssh-rsa</code> type.<br/>        """
        pass

    def export_snapshot(self, sourceSnapshotName: str) -> ExportSnapshotResult:
        """Exports an Amazon Lightsail instance or block storage disk snapshot to Amazon Elastic Compute Cloud (Amazon EC2). This operation results in an export snapshot record that can be used with the <code>create cloud formation stack</code> operation to create new Amazon EC2 instances. Exported instance snapshots appear in Amazon EC2 as Amazon Machine Images (AMIs), and the instance system disk appears as an Amazon Elastic Block Store (Amazon EBS) volume. Exported disk snapshots appear in Amazon EC2 as Amazon EBS volumes. Snapshots are exported to the same Amazon Web Services Region in Amazon EC2 as the source Lightsail snapshot. <p/> The <code>export snapshot</code> operation supports tag-based access control via resource tags applied to the resource identified by sourceSnapshotName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>. <note> Use the <code>get instance snapshots</code> or <code>get disk snapshots</code> operations to get a list of snapshots that you can export to Amazon EC2. </note>

        :param self:
        :param sourceSnapshotName:  The name of the instance or disk snapshot to be exported to Amazon EC2.<br/>
        :return: operations:  A list of objects describing the API operation.<br/>        """
        pass

    def get_active_names(self, pageToken: str=None) -> GetActiveNamesResult:
        """Returns the names of all active (not deleted) resources.

        :param self:
        :param pageToken:  A token used for paginating results from your get active names request.<br/>
        :return: nextPageToken:  A token used for advancing to the next page of results from your get active names request.<br/>activeNames:  The list of active names returned by the get active names request.<br/>        """
        pass

    def get_blueprints(self, includeInactive: boolean=None, pageToken: str=None) -> GetBlueprintsResult:
        """Returns the list of available instance images, or <i>blueprints</i>. You can use a blueprint to create a new virtual private server already running a specific operating system, as well as a preinstalled app or development stack. The software each instance is running depends on the blueprint image you choose.

        :param self:
        :param pageToken:  A token used for advancing to the next page of results from your get blueprints request.<br/>includeInactive:  A Boolean value indicating whether to include inactive results in your request.<br/>
        :return: nextPageToken:  A token used for advancing to the next page of results from your get blueprints request.<br/>blueprints:  An array of key-value pairs that contains information about the available blueprints.<br/>        """
        pass

    def get_bundles(self, includeInactive: boolean=None, pageToken: str=None) -> GetBundlesResult:
        """Returns the list of bundles that are available for purchase. A bundle describes the specs for your virtual private server (or <i>instance</i>).

        :param self:
        :param pageToken:  A token used for advancing to the next page of results from your get bundles request.<br/>includeInactive:  A Boolean value that indicates whether to include inactive bundle results in your request.<br/>
        :return: nextPageToken:  A token used for advancing to the next page of results from your get active names request.<br/>bundles:  An array of key-value pairs that contains information about the available bundles.<br/>        """
        pass

    def get_cloud_formation_stack_records(self, pageToken: str=None) -> GetCloudFormationStackRecordsResult:
        """Returns the CloudFormation stack record created as a result of the <code>create cloud formation stack</code> operation. An AWS CloudFormation stack is used to create a new Amazon EC2 instance from an exported Lightsail snapshot.

        :param self:
        :param pageToken:  A token used for advancing to a specific page of results for your <code>get cloud formation stack records</code> request.<br/>
        :return: nextPageToken:  A token used for advancing to the next page of results of your get relational database bundles request.<br/>cloudFormationStackRecords:  A list of objects describing the CloudFormation stack records.<br/>        """
        pass

    def get_disk(self, diskName: str) -> GetDiskResult:
        """Returns information about a specific block storage disk.

        :param self:
        :param diskName:  The name of the disk (e.g., <code>my-disk</code>).<br/>
        :return: disk:  An object containing information about the disk.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>gbInUse</i> (Deprecated) The number of GB in use by the disk. <note> In releases prior to November 14, 2017, this parameter was not included in the API response. It is now deprecated. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>attachmentState</i> (Deprecated) The attachment state of the disk. <note> In releases prior to November 14, 2017, this parameter returned <code>attached</code> for system disks in the API response. It is now deprecated, but still included in the response. Use <code>isAttached</code> instead. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>isAttached</i> A Boolean value indicating whether the disk is attached.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>attachedTo</i> The resources to which the disk is attached.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>state</i> Describes the status of the disk.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>path</i> The disk path.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>iops</i> The input/output operations per second (IOPS) of the disk.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>isSystemDisk</i> A Boolean value indicating whether this disk is a system disk (has an operating system loaded on it).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>sizeInGb</i> The size of the disk in GB.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> The tag keys and optional values for the resource. For more information about tags in Lightsail, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-tags">Lightsail Dev Guide</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceType</i> The Lightsail resource type (e.g., <code>Disk</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The AWS Region and Availability Zone where the disk is located.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>location</b>: The AWS Region and Availability Zone where the disk is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The date when the disk was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>supportCode</i> The support code. Include this code in your email to support when you have questions about an instance or another resource in Lightsail. This code enables our support team to look up your Lightsail information more easily.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the disk.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The unique name of the disk.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>regionName</i> The AWS Region name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>availabilityZone</i> The Availability Zone. Follows the format <code>us-east-2a</code> (case-sensitive).<br/>        """
        pass

    def get_disk_snapshot(self, diskSnapshotName: str) -> GetDiskSnapshotResult:
        """Returns information about a specific block storage disk snapshot.

        :param self:
        :param diskSnapshotName:  The name of the disk snapshot (e.g., <code>my-disk-snapshot</code>).<br/>
        :return: diskSnapshot:  An object containing information about the disk snapshot.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fromInstanceArn</i> The Amazon Resource Name (ARN) of the source instance from which the disk (system volume) snapshot was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fromInstanceName</i> The unique name of the source instance from which the disk (system volume) snapshot was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fromDiskArn</i> The Amazon Resource Name (ARN) of the source disk from which the disk snapshot was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fromDiskName</i> The unique name of the source disk from which the disk snapshot was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>progress</i> The progress of the disk snapshot operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>state</i> The status of the disk snapshot operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>sizeInGb</i> The size of the disk in GB.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> The tag keys and optional values for the resource. For more information about tags in Lightsail, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-tags">Lightsail Dev Guide</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceType</i> The Lightsail resource type (e.g., <code>DiskSnapshot</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The AWS Region and Availability Zone where the disk snapshot was created.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>location</b>: The AWS Region and Availability Zone where the disk snapshot was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The date when the disk snapshot was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>supportCode</i> The support code. Include this code in your email to support when you have questions about an instance or another resource in Lightsail. This code enables our support team to look up your Lightsail information more easily.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the disk snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the disk snapshot (e.g., <code>my-disk-snapshot</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>regionName</i> The AWS Region name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>availabilityZone</i> The Availability Zone. Follows the format <code>us-east-2a</code> (case-sensitive).<br/>        """
        pass

    def get_disk_snapshots(self, pageToken: str=None) -> GetDiskSnapshotsResult:
        """Returns information about all block storage disk snapshots in your AWS account and region. If you are describing a long list of disk snapshots, you can paginate the output to make the list more manageable. You can use the pageToken and nextPageToken values to retrieve the next items in the list.

        :param self:
        :param pageToken:  A token used for advancing to the next page of results from your GetDiskSnapshots request.<br/>
        :return: nextPageToken:  A token used for advancing to the next page of results from your GetDiskSnapshots request.<br/>diskSnapshots:  An array of objects containing information about all block storage disk snapshots.<br/>        """
        pass

    def get_disks(self, pageToken: str=None) -> GetDisksResult:
        """Returns information about all block storage disks in your AWS account and region. If you are describing a long list of disks, you can paginate the output to make the list more manageable. You can use the pageToken and nextPageToken values to retrieve the next items in the list.

        :param self:
        :param pageToken:  A token used for advancing to the next page of results from your GetDisks request.<br/>
        :return: nextPageToken:  A token used for advancing to the next page of results from your GetDisks request.<br/>disks:  An array of objects containing information about all block storage disks.<br/>        """
        pass

    def get_domain(self, domainName: str) -> GetDomainResult:
        """Returns information about a specific domain recordset.

        :param self:
        :param domainName:  The domain name for which your want to return information about.<br/>
        :return: domain:  An array of key-value pairs containing information about your get domain request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>domainEntries</i> An array of key-value pairs containing information about the domain entries.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> The tag keys and optional values for the resource. For more information about tags in Lightsail, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-tags">Lightsail Dev Guide</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceType</i> The resource type. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The AWS Region and Availability Zones where the domain recordset was created.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>location</b>: The AWS Region and Availability Zones where the domain recordset was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The date when the domain recordset was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>supportCode</i> The support code. Include this code in your email to support when you have questions about an instance or another resource in Lightsail. This code enables our support team to look up your Lightsail information more easily.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the domain recordset (e.g., <code>arn:aws:lightsail:global:123456789101:Domain/824cede0-abc7-4f84-8dbc-12345EXAMPLE</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>regionName</i> The AWS Region name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>availabilityZone</i> The Availability Zone. Follows the format <code>us-east-2a</code> (case-sensitive).<br/>        """
        pass

    def get_domains(self, pageToken: str=None) -> GetDomainsResult:
        """Returns a list of all domains in the user's account.

        :param self:
        :param pageToken:  A token used for advancing to the next page of results from your get domains request.<br/>
        :return: nextPageToken:  A token used for advancing to the next page of results from your get active names request.<br/>domains:  An array of key-value pairs containing information about each of the domain entries in the user's account.<br/>        """
        pass

    def get_export_snapshot_records(self, pageToken: str=None) -> GetExportSnapshotRecordsResult:
        """Returns the export snapshot record created as a result of the <code>export snapshot</code> operation. An export snapshot record can be used to create a new Amazon EC2 instance and its related resources with the <code>create cloud formation stack</code> operation.

        :param self:
        :param pageToken:  A token used for advancing to a specific page of results for your <code>get export snapshot records</code> request.<br/>
        :return: nextPageToken:  A token used for advancing to the next page of results of your get relational database bundles request.<br/>exportSnapshotRecords:  A list of objects describing the export snapshot records.<br/>        """
        pass

    def get_instance(self, instanceName: str) -> GetInstanceResult:
        """Returns information about a specific Amazon Lightsail instance, which is a virtual private server.

        :param self:
        :param instanceName:  The name of the instance.<br/>
        :return: instance:  An array of key-value pairs containing information about the specified instance.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>sshKeyName</i> The name of the SSH key being used to connect to the instance (e.g., <code>LightsailDefaultKeyPair</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>username</i> The user name for connecting to the instance (e.g., <code>ec2-user</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>state</i> The status code and the state (e.g., <code>running</code>) for the instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>state</b>: The status code and the state (e.g., <code>running</code>) for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>networking</i> Information about the public ports and monthly data transfer rates for the instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>networking</b>: Information about the public ports and monthly data transfer rates for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>hardware</i> The size of the vCPU and the amount of RAM for the instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>hardware</b>: The size of the vCPU and the amount of RAM for the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ipv6Address</i> The IPv6 address of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>publicIpAddress</i> The public IP address of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>privateIpAddress</i> The private IP address of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>isStaticIp</i> A Boolean value indicating whether this instance has a static IP assigned to it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>bundleId</i> The bundle for the instance (e.g., <code>micro_1_0</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>blueprintName</i> The friendly name of the blueprint (e.g., <code>Amazon Linux</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>blueprintId</i> The blueprint ID (e.g., <code>os_amlinux_2016_03</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> The tag keys and optional values for the resource. For more information about tags in Lightsail, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-tags">Lightsail Dev Guide</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceType</i> The type of resource (usually <code>Instance</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The region name and Availability Zone where the instance is located.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>location</b>: The region name and Availability Zone where the instance is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The timestamp when the instance was created (e.g., <code>1479734909.17</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>supportCode</i> The support code. Include this code in your email to support when you have questions about an instance or another resource in Lightsail. This code enables our support team to look up your Lightsail information more easily.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the instance (e.g., <code>arn:aws:lightsail:us-east-2:123456789101:Instance/244ad76f-8aad-4741-809f-12345EXAMPLE</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name the user gave the instance (e.g., <code>Amazon_Linux-1GB-Ohio-1</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>regionName</i> The AWS Region name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>availabilityZone</i> The Availability Zone. Follows the format <code>us-east-2a</code> (case-sensitive).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ramSizeInGb</i> The amount of RAM in GB on the instance (e.g., <code>1.0</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>disks</i> The disks attached to the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>cpuCount</i> The number of vCPUs the instance has.<br/>The amount of data in GB allocated for monthly data transfers.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>monthlyTransfer</b>: The amount of data in GB allocated for monthly data transfers.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The state of the instance (e.g., <code>running</code> or <code>pending</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>code</i> The status code for the instance.<br/>        """
        pass

    def get_instance_access_details(self, instanceName: str, protocol: str=None) -> GetInstanceAccessDetailsResult:
        """Returns temporary SSH keys you can use to connect to a specific virtual private server, or <i>instance</i>. The <code>get instance access details</code> operation supports tag-based access control via resource tags applied to the resource identified by instanceName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param protocol:  The protocol to use to connect to your instance. Defaults to <code>ssh</code>.<br/>instanceName:  The name of the instance to access.<br/>
        :return: accessDetails:  An array of key-value pairs containing information about a get instance access request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>hostKeys</i> Describes the public SSH host keys or the RDP certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>username</i> The user name to use when logging in to the Amazon Lightsail instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>instanceName</i> The name of this Amazon Lightsail instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>protocol</i> The protocol for these Amazon Lightsail instance access details.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>privateKey</i> For SSH access, the temporary private key. For OpenSSH clients (e.g., command line SSH), you should save this value to <code>tempkey</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>passwordData</i> For a Windows Server-based instance, an object with the data you can use to retrieve your password. This is only needed if <code>password</code> is empty and the instance is not new (and therefore the password is not ready yet). When you create an instance, it can take up to 15 minutes for the instance to be ready.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>passwordData</b>: For a Windows Server-based instance, an object with the data you can use to retrieve your password. This is only needed if <code>password</code> is empty and the instance is not new (and therefore the password is not ready yet). When you create an instance, it can take up to 15 minutes for the instance to be ready.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>password</i> For RDP access, the password for your Amazon Lightsail instance. Password will be an empty string if the password for your new instance is not ready yet. When you create an instance, it can take up to 15 minutes for the instance to be ready. <note> If you create an instance using any key pair other than the default (<code>LightsailDefaultKeyPair</code>), <code>password</code> will always be an empty string. If you change the Administrator password on the instance, Lightsail will continue to return the original password value. When accessing the instance using RDP, you need to manually enter the Administrator password after changing it from the default. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ipAddress</i> The public IP address of the Amazon Lightsail instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>expiresAt</i> For SSH access, the date on which the temporary keys expire.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>certKey</i> For SSH access, the public key to use when accessing your instance For OpenSSH clients (e.g., command line SSH), you should save this value to <code>tempkey-cert.pub</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>keyPairName</i> The name of the key pair that you used when creating your instance. If no key pair name was specified when creating the instance, Lightsail uses the default key pair (<code>LightsailDefaultKeyPair</code>). If you are using a custom key pair, you need to use your own means of decrypting your password using the <code>ciphertext</code>. Lightsail creates the ciphertext by encrypting your password with the public key part of this key pair.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ciphertext</i> The encrypted password. Ciphertext will be an empty string if access to your new instance is not ready yet. When you create an instance, it can take up to 15 minutes for the instance to be ready. <note> If you use the default key pair (<code>LightsailDefaultKeyPair</code>), the decrypted password will be available in the password field. If you are using a custom key pair, you need to use your own means of decryption. If you change the Administrator password on the instance, Lightsail will continue to return the original ciphertext value. When accessing the instance using RDP, you need to manually enter the Administrator password after changing it from the default. </note><br/>        """
        pass

    def get_instance_metric_data(self, statistics: List[MetricStatistic], unit: str, endTime: timestamp, startTime: timestamp, period: int, metricName: str, instanceName: str) -> GetInstanceMetricDataResult:
        """Returns the data points for the specified Amazon Lightsail instance metric, given an instance name.

        :param self:
        :param statistics:  The instance statistics. <br/>unit:  The unit. The list of valid values is below.<br/>endTime:  The end time of the time period.<br/>startTime:  The start time of the time period.<br/>period:  The granularity, in seconds, of the returned data points.<br/>metricName:  The metric name to get data about. <br/>instanceName:  The name of the instance for which you want to get metrics data.<br/>
        :return: metricData:  An array of key-value pairs containing information about the results of your get instance metric data request.<br/>metricName:  The metric name to return data for.<br/>        """
        pass

    def get_instance_port_states(self, instanceName: str) -> GetInstancePortStatesResult:
        """Returns the port states for a specific virtual private server, or <i>instance</i>.

        :param self:
        :param instanceName:  The name of the instance.<br/>
        :return: portStates:  Information about the port states resulting from your request.<br/>        """
        pass

    def get_instance_snapshot(self, instanceSnapshotName: str) -> GetInstanceSnapshotResult:
        """Returns information about a specific instance snapshot.

        :param self:
        :param instanceSnapshotName:  The name of the snapshot for which you are requesting information.<br/>
        :return: instanceSnapshot:  An array of key-value pairs containing information about the results of your get instance snapshot request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>sizeInGb</i> The size in GB of the SSD.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fromBundleId</i> The bundle ID from which you created the snapshot (e.g., <code>micro_1_0</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fromBlueprintId</i> The blueprint ID from which you created the snapshot (e.g., <code>os_debian_8_3</code>). A blueprint is a virtual private server (or <i>instance</i>) image used to create instances quickly.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fromInstanceArn</i> The Amazon Resource Name (ARN) of the instance from which the snapshot was created (e.g., <code>arn:aws:lightsail:us-east-2:123456789101:Instance/64b8404c-ccb1-430b-8daf-12345EXAMPLE</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fromInstanceName</i> The instance from which the snapshot was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fromAttachedDisks</i> An array of disk objects containing information about all block storage disks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>progress</i> The progress of the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>state</i> The state the snapshot is in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> The tag keys and optional values for the resource. For more information about tags in Lightsail, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-tags">Lightsail Dev Guide</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceType</i> The type of resource (usually <code>InstanceSnapshot</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The region name and Availability Zone where you created the snapshot.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>location</b>: The region name and Availability Zone where you created the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The timestamp when the snapshot was created (e.g., <code>1479907467.024</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>supportCode</i> The support code. Include this code in your email to support when you have questions about an instance or another resource in Lightsail. This code enables our support team to look up your Lightsail information more easily.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the snapshot (e.g., <code>arn:aws:lightsail:us-east-2:123456789101:InstanceSnapshot/d23b5706-3322-4d83-81e5-12345EXAMPLE</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>regionName</i> The AWS Region name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>availabilityZone</i> The Availability Zone. Follows the format <code>us-east-2a</code> (case-sensitive).<br/>        """
        pass

    def get_instance_snapshots(self, pageToken: str=None) -> GetInstanceSnapshotsResult:
        """Returns all instance snapshots for the user's account.

        :param self:
        :param pageToken:  A token used for advancing to the next page of results from your get instance snapshots request.<br/>
        :return: nextPageToken:  A token used for advancing to the next page of results from your get instance snapshots request.<br/>instanceSnapshots:  An array of key-value pairs containing information about the results of your get instance snapshots request.<br/>        """
        pass

    def get_instance_state(self, instanceName: str) -> GetInstanceStateResult:
        """Returns the state of a specific instance. Works on one instance at a time.

        :param self:
        :param instanceName:  The name of the instance to get state information about.<br/>
        :return: state:  The state of the instance.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The state of the instance (e.g., <code>running</code> or <code>pending</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>code</i> The status code for the instance.<br/>        """
        pass

    def get_instances(self, pageToken: str=None) -> GetInstancesResult:
        """Returns information about all Amazon Lightsail virtual private servers, or <i>instances</i>.

        :param self:
        :param pageToken:  A token used for advancing to the next page of results from your get instances request.<br/>
        :return: nextPageToken:  A token used for advancing to the next page of results from your get instances request.<br/>instances:  An array of key-value pairs containing information about your instances.<br/>        """
        pass

    def get_key_pair(self, keyPairName: str) -> GetKeyPairResult:
        """Returns information about a specific key pair.

        :param self:
        :param keyPairName:  The name of the key pair for which you are requesting information.<br/>
        :return: keyPair:  An array of key-value pairs containing information about the key pair.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fingerprint</i> The RSA fingerprint of the key pair.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> The tag keys and optional values for the resource. For more information about tags in Lightsail, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-tags">Lightsail Dev Guide</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceType</i> The resource type (usually <code>KeyPair</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The region name and Availability Zone where the key pair was created.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>location</b>: The region name and Availability Zone where the key pair was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The timestamp when the key pair was created (e.g., <code>1479816991.349</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>supportCode</i> The support code. Include this code in your email to support when you have questions about an instance or another resource in Lightsail. This code enables our support team to look up your Lightsail information more easily.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the key pair (e.g., <code>arn:aws:lightsail:us-east-2:123456789101:KeyPair/05859e3d-331d-48ba-9034-12345EXAMPLE</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The friendly name of the SSH key pair.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>regionName</i> The AWS Region name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>availabilityZone</i> The Availability Zone. Follows the format <code>us-east-2a</code> (case-sensitive).<br/>        """
        pass

    def get_key_pairs(self, pageToken: str=None) -> GetKeyPairsResult:
        """Returns information about all key pairs in the user's account.

        :param self:
        :param pageToken:  A token used for advancing to the next page of results from your get key pairs request.<br/>
        :return: nextPageToken:  A token used for advancing to the next page of results from your get key pairs request.<br/>keyPairs:  An array of key-value pairs containing information about the key pairs.<br/>        """
        pass

    def get_load_balancer(self, loadBalancerName: str) -> GetLoadBalancerResult:
        """Returns information about the specified Lightsail load balancer.

        :param self:
        :param loadBalancerName:  The name of the load balancer.<br/>
        :return: loadBalancer:  An object containing information about your load balancer.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>configurationOptions</i> A string to string map of the configuration options for your load balancer. Valid values are listed below.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tlsCertificateSummaries</i> An array of LoadBalancerTlsCertificateSummary objects that provide additional information about the SSL/TLS certificates. For example, if <code>true</code>, the certificate is attached to the load balancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>instanceHealthSummary</i> An array of InstanceHealthSummary objects describing the health of the load balancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>instancePort</i> The port where the load balancer will direct traffic to your Lightsail instances. For HTTP traffic, it's port 80. For HTTPS traffic, it's port 443.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>healthCheckPath</i> The path you specified to perform your health checks. If no path is specified, the load balancer tries to make a request to the default (root) page.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>publicPorts</i> An array of public port settings for your load balancer. For HTTP, use port 80. For HTTPS, use port 443.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>protocol</i> The protocol you have enabled for your load balancer. Valid values are below. You can't just have <code>HTTP_HTTPS</code>, but you can have just <code>HTTP</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>state</i> The status of your load balancer. Valid values are below.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>dnsName</i> The DNS name of your Lightsail load balancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> The tag keys and optional values for the resource. For more information about tags in Lightsail, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-tags">Lightsail Dev Guide</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceType</i> The resource type (e.g., <code>LoadBalancer</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The AWS Region where your load balancer was created (e.g., <code>us-east-2a</code>). Lightsail automatically creates your load balancer across Availability Zones.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>location</b>: The AWS Region where your load balancer was created (e.g., <code>us-east-2a</code>). Lightsail automatically creates your load balancer across Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The date when your load balancer was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>supportCode</i> The support code. Include this code in your email to support when you have questions about your Lightsail load balancer. This code enables our support team to look up your Lightsail information more easily.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the load balancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the load balancer (e.g., <code>my-load-balancer</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>regionName</i> The AWS Region name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>availabilityZone</i> The Availability Zone. Follows the format <code>us-east-2a</code> (case-sensitive).<br/>        """
        pass

    def get_load_balancer_metric_data(self, statistics: List[MetricStatistic], unit: str, endTime: timestamp, startTime: timestamp, period: int, metricName: str, loadBalancerName: str) -> GetLoadBalancerMetricDataResult:
        """Returns information about health metrics for your Lightsail load balancer.

        :param self:
        :param statistics:  An array of statistics that you want to request metrics for. Valid values are listed below. <ul> <li>  <b> <code>SampleCount</code> </b> - The count (number) of data points used for the statistical calculation. </li> <li>  <b> <code>Average</code> </b> - The value of Sum / SampleCount during the specified period. By comparing this statistic with the Minimum and Maximum, you can determine the full scope of a metric and how close the average use is to the Minimum and Maximum. This comparison helps you to know when to increase or decrease your resources as needed. </li> <li>  <b> <code>Sum</code> </b> - All values submitted for the matching metric added together. This statistic can be useful for determining the total volume of a metric. </li> <li>  <b> <code>Minimum</code> </b> - The lowest value observed during the specified period. You can use this value to determine low volumes of activity for your application. </li> <li>  <b> <code>Maximum</code> </b> - The highest value observed during the specified period. You can use this value to determine high volumes of activity for your application. </li> </ul><br/>unit:  The unit for the time period request. Valid values are listed below.<br/>endTime:  The end time of the period.<br/>startTime:  The start time of the period.<br/>period:  The granularity, in seconds, of the returned data points.<br/>metricName:  The metric about which you want to return information. Valid values are listed below, along with the most useful <code>statistics</code> to include in your request. <ul> <li>  <b> <code>ClientTLSNegotiationErrorCount</code> </b> - The number of TLS connections initiated by the client that did not establish a session with the load balancer. Possible causes include a mismatch of ciphers or protocols.  <code>Statistics</code>: The most useful statistic is <code>Sum</code>. </li> <li>  <b> <code>HealthyHostCount</code> </b> - The number of target instances that are considered healthy.  <code>Statistics</code>: The most useful statistic are <code>Average</code>, <code>Minimum</code>, and <code>Maximum</code>. </li> <li>  <b> <code>UnhealthyHostCount</code> </b> - The number of target instances that are considered unhealthy.  <code>Statistics</code>: The most useful statistic are <code>Average</code>, <code>Minimum</code>, and <code>Maximum</code>. </li> <li>  <b> <code>HTTPCode_LB_4XX_Count</code> </b> - The number of HTTP 4XX client error codes that originate from the load balancer. Client errors are generated when requests are malformed or incomplete. These requests have not been received by the target instance. This count does not include any response codes generated by the target instances.  <code>Statistics</code>: The most useful statistic is <code>Sum</code>. Note that <code>Minimum</code>, <code>Maximum</code>, and <code>Average</code> all return <code>1</code>. </li> <li>  <b> <code>HTTPCode_LB_5XX_Count</code> </b> - The number of HTTP 5XX server error codes that originate from the load balancer. This count does not include any response codes generated by the target instances.  <code>Statistics</code>: The most useful statistic is <code>Sum</code>. Note that <code>Minimum</code>, <code>Maximum</code>, and <code>Average</code> all return <code>1</code>. Note that <code>Minimum</code>, <code>Maximum</code>, and <code>Average</code> all return <code>1</code>. </li> <li>  <b> <code>HTTPCode_Instance_2XX_Count</code> </b> - The number of HTTP response codes generated by the target instances. This does not include any response codes generated by the load balancer.  <code>Statistics</code>: The most useful statistic is <code>Sum</code>. Note that <code>Minimum</code>, <code>Maximum</code>, and <code>Average</code> all return <code>1</code>. </li> <li>  <b> <code>HTTPCode_Instance_3XX_Count</code> </b> - The number of HTTP response codes generated by the target instances. This does not include any response codes generated by the load balancer.   <code>Statistics</code>: The most useful statistic is <code>Sum</code>. Note that <code>Minimum</code>, <code>Maximum</code>, and <code>Average</code> all return <code>1</code>. </li> <li>  <b> <code>HTTPCode_Instance_4XX_Count</code> </b> - The number of HTTP response codes generated by the target instances. This does not include any response codes generated by the load balancer.  <code>Statistics</code>: The most useful statistic is <code>Sum</code>. Note that <code>Minimum</code>, <code>Maximum</code>, and <code>Average</code> all return <code>1</code>. </li> <li>  <b> <code>HTTPCode_Instance_5XX_Count</code> </b> - The number of HTTP response codes generated by the target instances. This does not include any response codes generated by the load balancer.  <code>Statistics</code>: The most useful statistic is <code>Sum</code>. Note that <code>Minimum</code>, <code>Maximum</code>, and <code>Average</code> all return <code>1</code>. </li> <li>  <b> <code>InstanceResponseTime</code> </b> - The time elapsed, in seconds, after the request leaves the load balancer until a response from the target instance is received.  <code>Statistics</code>: The most useful statistic is <code>Average</code>. </li> <li>  <b> <code>RejectedConnectionCount</code> </b> - The number of connections that were rejected because the load balancer had reached its maximum number of connections.  <code>Statistics</code>: The most useful statistic is <code>Sum</code>. </li> <li>  <b> <code>RequestCount</code> </b> - The number of requests processed over IPv4. This count includes only the requests with a response generated by a target instance of the load balancer.  <code>Statistics</code>: The most useful statistic is <code>Sum</code>. Note that <code>Minimum</code>, <code>Maximum</code>, and <code>Average</code> all return <code>1</code>. </li> </ul><br/>loadBalancerName:  The name of the load balancer.<br/>
        :return: metricData:  An array of metric datapoint objects.<br/>metricName:  The metric about which you are receiving information. Valid values are listed below, along with the most useful <code>statistics</code> to include in your request. <ul> <li>  <b> <code>ClientTLSNegotiationErrorCount</code> </b> - The number of TLS connections initiated by the client that did not establish a session with the load balancer. Possible causes include a mismatch of ciphers or protocols.  <code>Statistics</code>: The most useful statistic is <code>Sum</code>. </li> <li>  <b> <code>HealthyHostCount</code> </b> - The number of target instances that are considered healthy.  <code>Statistics</code>: The most useful statistic are <code>Average</code>, <code>Minimum</code>, and <code>Maximum</code>. </li> <li>  <b> <code>UnhealthyHostCount</code> </b> - The number of target instances that are considered unhealthy.  <code>Statistics</code>: The most useful statistic are <code>Average</code>, <code>Minimum</code>, and <code>Maximum</code>. </li> <li>  <b> <code>HTTPCode_LB_4XX_Count</code> </b> - The number of HTTP 4XX client error codes that originate from the load balancer. Client errors are generated when requests are malformed or incomplete. These requests have not been received by the target instance. This count does not include any response codes generated by the target instances.  <code>Statistics</code>: The most useful statistic is <code>Sum</code>. Note that <code>Minimum</code>, <code>Maximum</code>, and <code>Average</code> all return <code>1</code>. </li> <li>  <b> <code>HTTPCode_LB_5XX_Count</code> </b> - The number of HTTP 5XX server error codes that originate from the load balancer. This count does not include any response codes generated by the target instances.  <code>Statistics</code>: The most useful statistic is <code>Sum</code>. Note that <code>Minimum</code>, <code>Maximum</code>, and <code>Average</code> all return <code>1</code>. Note that <code>Minimum</code>, <code>Maximum</code>, and <code>Average</code> all return <code>1</code>. </li> <li>  <b> <code>HTTPCode_Instance_2XX_Count</code> </b> - The number of HTTP response codes generated by the target instances. This does not include any response codes generated by the load balancer.  <code>Statistics</code>: The most useful statistic is <code>Sum</code>. Note that <code>Minimum</code>, <code>Maximum</code>, and <code>Average</code> all return <code>1</code>. </li> <li>  <b> <code>HTTPCode_Instance_3XX_Count</code> </b> - The number of HTTP response codes generated by the target instances. This does not include any response codes generated by the load balancer.   <code>Statistics</code>: The most useful statistic is <code>Sum</code>. Note that <code>Minimum</code>, <code>Maximum</code>, and <code>Average</code> all return <code>1</code>. </li> <li>  <b> <code>HTTPCode_Instance_4XX_Count</code> </b> - The number of HTTP response codes generated by the target instances. This does not include any response codes generated by the load balancer.  <code>Statistics</code>: The most useful statistic is <code>Sum</code>. Note that <code>Minimum</code>, <code>Maximum</code>, and <code>Average</code> all return <code>1</code>. </li> <li>  <b> <code>HTTPCode_Instance_5XX_Count</code> </b> - The number of HTTP response codes generated by the target instances. This does not include any response codes generated by the load balancer.  <code>Statistics</code>: The most useful statistic is <code>Sum</code>. Note that <code>Minimum</code>, <code>Maximum</code>, and <code>Average</code> all return <code>1</code>. </li> <li>  <b> <code>InstanceResponseTime</code> </b> - The time elapsed, in seconds, after the request leaves the load balancer until a response from the target instance is received.  <code>Statistics</code>: The most useful statistic is <code>Average</code>. </li> <li>  <b> <code>RejectedConnectionCount</code> </b> - The number of connections that were rejected because the load balancer had reached its maximum number of connections.  <code>Statistics</code>: The most useful statistic is <code>Sum</code>. </li> <li>  <b> <code>RequestCount</code> </b> - The number of requests processed over IPv4. This count includes only the requests with a response generated by a target instance of the load balancer.  <code>Statistics</code>: The most useful statistic is <code>Sum</code>. Note that <code>Minimum</code>, <code>Maximum</code>, and <code>Average</code> all return <code>1</code>. </li> </ul><br/>        """
        pass

    def get_load_balancer_tls_certificates(self, loadBalancerName: str) -> GetLoadBalancerTlsCertificatesResult:
        """Returns information about the TLS certificates that are associated with the specified Lightsail load balancer. TLS is just an updated, more secure version of Secure Socket Layer (SSL). You can have a maximum of 2 certificates associated with a Lightsail load balancer. One is active and the other is inactive.

        :param self:
        :param loadBalancerName:  The name of the load balancer you associated with your SSL/TLS certificate.<br/>
        :return: tlsCertificates:  An array of LoadBalancerTlsCertificate objects describing your SSL/TLS certificates.<br/>        """
        pass

    def get_load_balancers(self, pageToken: str=None) -> GetLoadBalancersResult:
        """Returns information about all load balancers in an account. If you are describing a long list of load balancers, you can paginate the output to make the list more manageable. You can use the pageToken and nextPageToken values to retrieve the next items in the list.

        :param self:
        :param pageToken:  A token used for paginating the results from your GetLoadBalancers request.<br/>
        :return: nextPageToken:  A token used for advancing to the next page of results from your GetLoadBalancers request.<br/>loadBalancers:  An array of LoadBalancer objects describing your load balancers.<br/>        """
        pass

    def get_operation(self, operationId: str) -> GetOperationResult:
        """Returns information about a specific operation. Operations include events such as when you create an instance, allocate a static IP, attach a static IP, and so on.

        :param self:
        :param operationId:  A GUID used to identify the operation.<br/>
        :return: operation:  An array of key-value pairs containing information about the results of your get operation request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorDetails</i> The error details.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorCode</i> The error code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusChangedAt</i> The timestamp when the status was changed (e.g., <code>1479816991.349</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>operationType</i> The type of operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>operationDetails</i> Details about the operation (e.g., <code>Debian-1GB-Ohio-1</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>isTerminal</i> A Boolean value indicating whether the operation is terminal.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The region and Availability Zone.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>location</b>: The region and Availability Zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The timestamp when the operation was initialized (e.g., <code>1479816991.349</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceType</i> The resource type. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceName</i> The resource name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>regionName</i> The AWS Region name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>availabilityZone</i> The Availability Zone. Follows the format <code>us-east-2a</code> (case-sensitive).<br/>        """
        pass

    def get_operations(self, pageToken: str=None) -> GetOperationsResult:
        """Returns information about all operations. Results are returned from oldest to newest, up to a maximum of 200. Results can be paged by making each subsequent call to <code>GetOperations</code> use the maximum (last) <code>statusChangedAt</code> value from the previous request.

        :param self:
        :param pageToken:  A token used for advancing to the next page of results from your get operations request.<br/>
        :return: nextPageToken:  A token used for advancing to the next page of results from your get operations request.<br/>operations:  An array of key-value pairs containing information about the results of your get operations request.<br/>        """
        pass

    def get_operations_for_resource(self, resourceName: str, pageToken: str=None) -> GetOperationsForResourceResult:
        """Gets operations for a specific resource (e.g., an instance or a static IP).

        :param self:
        :param pageToken:  A token used for advancing to the next page of results from your get operations for resource request.<br/>resourceName:  The name of the resource for which you are requesting information.<br/>
        :return: nextPageToken:  An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list.<br/>nextPageCount:  (Deprecated) Returns the number of pages of results that remain. <note> In releases prior to June 12, 2017, this parameter returned <code>null</code> by the API. It is now deprecated, and the API returns the <code>next page token</code> parameter instead. </note><br/>operations:  An array of key-value pairs containing information about the results of your get operations for resource request.<br/>        """
        pass

    def get_regions(self, includeAvailabilityZones: boolean=None, includeRelationalDatabaseAvailabilityZones: boolean=None) -> GetRegionsResult:
        """Returns a list of all valid regions for Amazon Lightsail. Use the <code>include availability zones</code> parameter to also return the Availability Zones in a region.

        :param self:
        :param includeRelationalDatabaseAvailabilityZones:  &gt;A Boolean value indicating whether to also include Availability Zones for databases in your get regions request. Availability Zones are indicated with a letter (e.g., <code>us-east-2a</code>).<br/>includeAvailabilityZones:  A Boolean value indicating whether to also include Availability Zones in your get regions request. Availability Zones are indicated with a letter: e.g., <code>us-east-2a</code>.<br/>
        :return: regions:  An array of key-value pairs containing information about your get regions request.<br/>        """
        pass

    def get_relational_database(self, relationalDatabaseName: str) -> GetRelationalDatabaseResult:
        """Returns information about a specific database in Amazon Lightsail.

        :param self:
        :param relationalDatabaseName:  The name of the database that you are looking up.<br/>
        :return: relationalDatabase:  An object describing the specified database.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pendingMaintenanceActions</i> Describes the pending maintenance actions for the database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>masterEndpoint</i> The master endpoint for the database.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>masterEndpoint</b>: The master endpoint for the database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>publiclyAccessible</i> A Boolean value indicating whether the database is publicly accessible.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>preferredMaintenanceWindow</i> The weekly time range during which system maintenance can occur on the database. In the format <code>ddd:hh24:mi-ddd:hh24:mi</code>. For example, <code>Tue:17:00-Tue:17:30</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>preferredBackupWindow</i> The daily time range during which automated backups are created for the database (for example, <code>16:00-16:30</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>parameterApplyStatus</i> The status of parameter updates for the database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>masterUsername</i> The master user name of the database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>latestRestorableTime</i> The latest point in time to which the database can be restored. Formatted in Unix time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>engineVersion</i> The database engine version (for example, <code>5.7.23</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>engine</i> The database software (for example, <code>MySQL</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>pendingModifiedValues</i> Describes pending database value modifications.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>pendingModifiedValues</b>: Describes pending database value modifications.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>backupRetentionEnabled</i> A Boolean value indicating whether automated backup retention is enabled for the database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>secondaryAvailabilityZone</i> Describes the secondary Availability Zone of a high availability database. The secondary database is used for failover support of a high availability database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>state</i> Describes the current state of the database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>hardware</i> Describes the hardware of the database.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>hardware</b>: Describes the hardware of the database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>masterDatabaseName</i> The name of the master database created when the Lightsail database resource is created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>relationalDatabaseBundleId</i> The bundle ID for the database. A bundle describes the performance specifications for your database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>relationalDatabaseBlueprintId</i> The blueprint ID for the database. A blueprint describes the major engine version of a database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> The tag keys and optional values for the resource. For more information about tags in Lightsail, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-tags">Lightsail Dev Guide</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceType</i> The Lightsail resource type for the database (for example, <code>RelationalDatabase</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The Region name and Availability Zone where the database is located.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>location</b>: The Region name and Availability Zone where the database is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The timestamp when the database was created. Formatted in Unix time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>supportCode</i> The support code for the database. Include this code in your email to support when you have questions about a database in Lightsail. This code enables our support team to look up your Lightsail information more easily.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The unique name of the database resource in Lightsail.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>regionName</i> The AWS Region name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>availabilityZone</i> The Availability Zone. Follows the format <code>us-east-2a</code> (case-sensitive).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ramSizeInGb</i> The amount of RAM in GB for the database.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>diskSizeInGb</i> The size of the disk for the database.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>cpuCount</i> The number of vCPUs for the database.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>backupRetentionEnabled</i> A Boolean value indicating whether automated backup retention is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>engineVersion</i> The database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>masterUserPassword</i> The password for the master user of the database.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>address</i> Specifies the DNS address of the database.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>port</i> Specifies the port that the database is listening on.<br/>        """
        pass

    def get_relational_database_blueprints(self, pageToken: str=None) -> GetRelationalDatabaseBlueprintsResult:
        """Returns a list of available database blueprints in Amazon Lightsail. A blueprint describes the major engine version of a database. You can use a blueprint ID to create a new database that runs a specific database engine.

        :param self:
        :param pageToken:  A token used for advancing to a specific page of results for your <code>get relational database blueprints</code> request.<br/>
        :return: nextPageToken:  A token used for advancing to the next page of results of your get relational database blueprints request.<br/>blueprints:  An object describing the result of your get relational database blueprints request.<br/>        """
        pass

    def get_relational_database_bundles(self, pageToken: str=None) -> GetRelationalDatabaseBundlesResult:
        """Returns the list of bundles that are available in Amazon Lightsail. A bundle describes the performance specifications for a database. You can use a bundle ID to create a new database with explicit performance specifications.

        :param self:
        :param pageToken:  A token used for advancing to a specific page of results for your <code>get relational database bundles</code> request.<br/>
        :return: nextPageToken:  A token used for advancing to the next page of results of your get relational database bundles request.<br/>bundles:  An object describing the result of your get relational database bundles request.<br/>        """
        pass

    def get_relational_database_events(self, relationalDatabaseName: str, durationInMinutes: int=None, pageToken: str=None) -> GetRelationalDatabaseEventsResult:
        """Returns a list of events for a specific database in Amazon Lightsail.

        :param self:
        :param pageToken:  A token used for advancing to a specific page of results from for get relational database events request.<br/>durationInMinutes:  The number of minutes in the past from which to retrieve events. For example, to get all events from the past 2 hours, enter 120. Default: <code>60</code>  The minimum is 1 and the maximum is 14 days (20160 minutes).<br/>relationalDatabaseName:  The name of the database from which to get events.<br/>
        :return: nextPageToken:  A token used for advancing to the next page of results from your get relational database events request.<br/>relationalDatabaseEvents:  An object describing the result of your get relational database events request.<br/>        """
        pass

    def get_relational_database_log_events(self, logStreamName: str, relationalDatabaseName: str, startTime: IsoDate=None, endTime: IsoDate=None, startFromHead: boolean=None, pageToken: str=None) -> GetRelationalDatabaseLogEventsResult:
        """Returns a list of log events for a database in Amazon Lightsail.

        :param self:
        :param pageToken:  A token used for advancing to a specific page of results for your <code>get relational database log events</code> request.<br/>startFromHead:  Parameter to specify if the log should start from head or tail. If <code>true</code> is specified, the log event starts from the head of the log. If <code>false</code> is specified, the log event starts from the tail of the log. Default: <code>false</code> <br/>endTime:  The end of the time interval from which to get log events. Constraints: <ul> <li> Specified in Universal Coordinated Time (UTC). </li> <li> Specified in the Unix time format. For example, if you wish to use an end time of October 1, 2018, at 8 PM UTC, then you input <code>1538424000</code> as the end time. </li> </ul><br/>startTime:  The start of the time interval from which to get log events. Constraints: <ul> <li> Specified in Universal Coordinated Time (UTC). </li> <li> Specified in the Unix time format. For example, if you wish to use a start time of October 1, 2018, at 8 PM UTC, then you input <code>1538424000</code> as the start time. </li> </ul><br/>logStreamName:  The name of the log stream. Use the <code>get relational database log streams</code> operation to get a list of available log streams.<br/>relationalDatabaseName:  The name of your database for which to get log events.<br/>
        :return: nextForwardToken:  A token used for advancing to the next page of results from your get relational database log events request.<br/>nextBackwardToken:  A token used for advancing to the previous page of results from your get relational database log events request.<br/>resourceLogEvents:  An object describing the result of your get relational database log events request.<br/>        """
        pass

    def get_relational_database_log_streams(self, relationalDatabaseName: str) -> GetRelationalDatabaseLogStreamsResult:
        """Returns a list of available log streams for a specific database in Amazon Lightsail.

        :param self:
        :param relationalDatabaseName:  The name of your database for which to get log streams.<br/>
        :return: logStreams:  An object describing the result of your get relational database log streams request.<br/>        """
        pass

    def get_relational_database_master_user_password(self, relationalDatabaseName: str, passwordVersion: str=None) -> GetRelationalDatabaseMasterUserPasswordResult:
        """Returns the current, previous, or pending versions of the master user password for a Lightsail database. The <code>asdf</code> operation GetRelationalDatabaseMasterUserPassword supports tag-based access control via resource tags applied to the resource identified by relationalDatabaseName.

        :param self:
        :param passwordVersion:  The password version to return. Specifying <code>CURRENT</code> or <code>PREVIOUS</code> returns the current or previous passwords respectively. Specifying <code>PENDING</code> returns the newest version of the password that will rotate to <code>CURRENT</code>. After the <code>PENDING</code> password rotates to <code>CURRENT</code>, the <code>PENDING</code> password is no longer available. Default: <code>CURRENT</code> <br/>relationalDatabaseName:  The name of your database for which to get the master user password.<br/>
        :return: createdAt:  The timestamp when the specified version of the master user password was created.<br/>masterUserPassword:  The master user password for the <code>password version</code> specified.<br/>        """
        pass

    def get_relational_database_metric_data(self, statistics: List[MetricStatistic], unit: str, endTime: IsoDate, startTime: IsoDate, period: int, metricName: str, relationalDatabaseName: str) -> GetRelationalDatabaseMetricDataResult:
        """Returns the data points of the specified metric for a database in Amazon Lightsail.

        :param self:
        :param statistics:  The array of statistics for your metric data request.<br/>unit:  The unit for the metric data request.<br/>endTime:  The end of the time interval from which to get metric data. Constraints: <ul> <li> Specified in Universal Coordinated Time (UTC). </li> <li> Specified in the Unix time format. For example, if you wish to use an end time of October 1, 2018, at 8 PM UTC, then you input <code>1538424000</code> as the end time. </li> </ul><br/>startTime:  The start of the time interval from which to get metric data. Constraints: <ul> <li> Specified in Universal Coordinated Time (UTC). </li> <li> Specified in the Unix time format. For example, if you wish to use a start time of October 1, 2018, at 8 PM UTC, then you input <code>1538424000</code> as the start time. </li> </ul><br/>period:  The granularity, in seconds, of the returned data points.<br/>metricName:  The name of the metric data to return.<br/>relationalDatabaseName:  The name of your database from which to get metric data.<br/>
        :return: metricData:  An object describing the result of your get relational database metric data request.<br/>metricName:  The name of the metric.<br/>        """
        pass

    def get_relational_database_parameters(self, relationalDatabaseName: str, pageToken: str=None) -> GetRelationalDatabaseParametersResult:
        """Returns all of the runtime parameters offered by the underlying database software, or engine, for a specific database in Amazon Lightsail. In addition to the parameter names and values, this operation returns other information about each parameter. This information includes whether changes require a reboot, whether the parameter is modifiable, the allowed values, and the data types.

        :param self:
        :param pageToken:  A token used for advancing to a specific page of results for your <code>get relational database parameters</code> request.<br/>relationalDatabaseName:  The name of your database for which to get parameters.<br/>
        :return: nextPageToken:  A token used for advancing to the next page of results from your get static IPs request.<br/>parameters:  An object describing the result of your get relational database parameters request.<br/>        """
        pass

    def get_relational_database_snapshot(self, relationalDatabaseSnapshotName: str) -> GetRelationalDatabaseSnapshotResult:
        """Returns information about a specific database snapshot in Amazon Lightsail.

        :param self:
        :param relationalDatabaseSnapshotName:  The name of the database snapshot for which to get information.<br/>
        :return: relationalDatabaseSnapshot:  An object describing the specified database snapshot.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fromRelationalDatabaseBlueprintId</i> The blueprint ID of the database from which the database snapshot was created. A blueprint describes the major engine version of a database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fromRelationalDatabaseBundleId</i> The bundle ID of the database from which the database snapshot was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fromRelationalDatabaseArn</i> The Amazon Resource Name (ARN) of the database from which the database snapshot was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fromRelationalDatabaseName</i> The name of the source database from which the database snapshot was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>state</i> The state of the database snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>sizeInGb</i> The size of the disk in GB (for example, <code>32</code>) for the database snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>engineVersion</i> The database engine version for the database snapshot (for example, <code>5.7.23</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>engine</i> The software of the database snapshot (for example, <code>MySQL</code>)<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> The tag keys and optional values for the resource. For more information about tags in Lightsail, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-tags">Lightsail Dev Guide</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceType</i> The Lightsail resource type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The Region name and Availability Zone where the database snapshot is located.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>location</b>: The Region name and Availability Zone where the database snapshot is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The timestamp when the database snapshot was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>supportCode</i> The support code for the database snapshot. Include this code in your email to support when you have questions about a database snapshot in Lightsail. This code enables our support team to look up your Lightsail information more easily.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the database snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the database snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>regionName</i> The AWS Region name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>availabilityZone</i> The Availability Zone. Follows the format <code>us-east-2a</code> (case-sensitive).<br/>        """
        pass

    def get_relational_database_snapshots(self, pageToken: str=None) -> GetRelationalDatabaseSnapshotsResult:
        """Returns information about all of your database snapshots in Amazon Lightsail.

        :param self:
        :param pageToken:  A token used for advancing to a specific page of results for your <code>get relational database snapshots</code> request.<br/>
        :return: nextPageToken:  A token used for advancing to the next page of results from your get relational database snapshots request.<br/>relationalDatabaseSnapshots:  An object describing the result of your get relational database snapshots request.<br/>        """
        pass

    def get_relational_databases(self, pageToken: str=None) -> GetRelationalDatabasesResult:
        """Returns information about all of your databases in Amazon Lightsail.

        :param self:
        :param pageToken:  A token used for advancing to a specific page of results for your <code>get relational database</code> request.<br/>
        :return: nextPageToken:  A token used for advancing to the next page of results from your get relational databases request.<br/>relationalDatabases:  An object describing the result of your get relational databases request.<br/>        """
        pass

    def get_static_ip(self, staticIpName: str) -> GetStaticIpResult:
        """Returns information about a specific static IP.

        :param self:
        :param staticIpName:  The name of the static IP in Lightsail.<br/>
        :return: staticIp:  An array of key-value pairs containing information about the requested static IP.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>isAttached</i> A Boolean value indicating whether the static IP is attached.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>attachedTo</i> The instance where the static IP is attached (e.g., <code>Amazon_Linux-1GB-Ohio-1</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ipAddress</i> The static IP address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceType</i> The resource type (usually <code>StaticIp</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The region and Availability Zone where the static IP was created.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>location</b>: The region and Availability Zone where the static IP was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The timestamp when the static IP was created (e.g., <code>1479735304.222</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>supportCode</i> The support code. Include this code in your email to support when you have questions about an instance or another resource in Lightsail. This code enables our support team to look up your Lightsail information more easily.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The Amazon Resource Name (ARN) of the static IP (e.g., <code>arn:aws:lightsail:us-east-2:123456789101:StaticIp/9cbb4a9e-f8e3-4dfe-b57e-12345EXAMPLE</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the static IP (e.g., <code>StaticIP-Ohio-EXAMPLE</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>regionName</i> The AWS Region name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>availabilityZone</i> The Availability Zone. Follows the format <code>us-east-2a</code> (case-sensitive).<br/>        """
        pass

    def get_static_ips(self, pageToken: str=None) -> GetStaticIpsResult:
        """Returns information about all static IPs in the user's account.

        :param self:
        :param pageToken:  A token used for advancing to the next page of results from your get static IPs request.<br/>
        :return: nextPageToken:  A token used for advancing to the next page of results from your get static IPs request.<br/>staticIps:  An array of key-value pairs containing information about your get static IPs request.<br/>        """
        pass

    def import_key_pair(self, publicKeyBase64: str, keyPairName: str) -> ImportKeyPairResult:
        """Imports a public SSH key from a specific key pair.

        :param self:
        :param publicKeyBase64:  A base64-encoded public key of the <code>ssh-rsa</code> type.<br/>keyPairName:  The name of the key pair for which you want to import the public key.<br/>
        :return: operation:  An array of key-value pairs containing information about the request operation.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorDetails</i> The error details.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorCode</i> The error code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusChangedAt</i> The timestamp when the status was changed (e.g., <code>1479816991.349</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>operationType</i> The type of operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>operationDetails</i> Details about the operation (e.g., <code>Debian-1GB-Ohio-1</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>isTerminal</i> A Boolean value indicating whether the operation is terminal.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The region and Availability Zone.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>location</b>: The region and Availability Zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The timestamp when the operation was initialized (e.g., <code>1479816991.349</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceType</i> The resource type. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceName</i> The resource name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>regionName</i> The AWS Region name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>availabilityZone</i> The Availability Zone. Follows the format <code>us-east-2a</code> (case-sensitive).<br/>        """
        pass

    def is_vpc_peered(self) -> IsVpcPeeredResult:
        """Returns a Boolean value indicating whether your Lightsail VPC is peered.

        :param self:
        :param 
        :return: isPeered:  Returns <code>true</code> if the Lightsail VPC is peered; otherwise, <code>false</code>.<br/>        """
        pass

    def open_instance_public_ports(self, instanceName: str, portInfo: PortInfo) -> OpenInstancePublicPortsResult:
        """Adds public ports to an Amazon Lightsail instance. The <code>open instance public ports</code> operation supports tag-based access control via resource tags applied to the resource identified by instanceName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param instanceName:  The name of the instance for which you want to open the public ports.<br/>portInfo:  An array of key-value pairs containing information about the port mappings.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>protocol</i> The protocol. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>toPort</i> The last port in the range.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fromPort</i> The first port in the range.<br/>
        :return: operation:  An array of key-value pairs containing information about the request operation.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorDetails</i> The error details.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorCode</i> The error code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusChangedAt</i> The timestamp when the status was changed (e.g., <code>1479816991.349</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>operationType</i> The type of operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>operationDetails</i> Details about the operation (e.g., <code>Debian-1GB-Ohio-1</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>isTerminal</i> A Boolean value indicating whether the operation is terminal.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The region and Availability Zone.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>location</b>: The region and Availability Zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The timestamp when the operation was initialized (e.g., <code>1479816991.349</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceType</i> The resource type. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceName</i> The resource name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>regionName</i> The AWS Region name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>availabilityZone</i> The Availability Zone. Follows the format <code>us-east-2a</code> (case-sensitive).<br/>        """
        pass

    def peer_vpc(self) -> PeerVpcResult:
        """Tries to peer the Lightsail VPC with the user's default VPC.

        :param self:
        :param 
        :return: operation:  An array of key-value pairs containing information about the request operation.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorDetails</i> The error details.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorCode</i> The error code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusChangedAt</i> The timestamp when the status was changed (e.g., <code>1479816991.349</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>operationType</i> The type of operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>operationDetails</i> Details about the operation (e.g., <code>Debian-1GB-Ohio-1</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>isTerminal</i> A Boolean value indicating whether the operation is terminal.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The region and Availability Zone.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>location</b>: The region and Availability Zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The timestamp when the operation was initialized (e.g., <code>1479816991.349</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceType</i> The resource type. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceName</i> The resource name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>regionName</i> The AWS Region name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>availabilityZone</i> The Availability Zone. Follows the format <code>us-east-2a</code> (case-sensitive).<br/>        """
        pass

    def put_instance_public_ports(self, instanceName: str, portInfos: List[PortInfo]) -> PutInstancePublicPortsResult:
        """Sets the specified open ports for an Amazon Lightsail instance, and closes all ports for every protocol not included in the current request. The <code>put instance public ports</code> operation supports tag-based access control via resource tags applied to the resource identified by instanceName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param instanceName:  The Lightsail instance name of the public port(s) you are setting.<br/>portInfos:  Specifies information about the public port(s).<br/>
        :return: operation:  Describes metadata about the operation you just executed.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorDetails</i> The error details.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorCode</i> The error code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusChangedAt</i> The timestamp when the status was changed (e.g., <code>1479816991.349</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>operationType</i> The type of operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>operationDetails</i> Details about the operation (e.g., <code>Debian-1GB-Ohio-1</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>isTerminal</i> A Boolean value indicating whether the operation is terminal.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The region and Availability Zone.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>location</b>: The region and Availability Zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The timestamp when the operation was initialized (e.g., <code>1479816991.349</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceType</i> The resource type. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceName</i> The resource name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>regionName</i> The AWS Region name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>availabilityZone</i> The Availability Zone. Follows the format <code>us-east-2a</code> (case-sensitive).<br/>        """
        pass

    def reboot_instance(self, instanceName: str) -> RebootInstanceResult:
        """Restarts a specific instance. The <code>reboot instance</code> operation supports tag-based access control via resource tags applied to the resource identified by instanceName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param instanceName:  The name of the instance to reboot.<br/>
        :return: operations:  An array of key-value pairs containing information about the request operations.<br/>        """
        pass

    def reboot_relational_database(self, relationalDatabaseName: str) -> RebootRelationalDatabaseResult:
        """Restarts a specific database in Amazon Lightsail. The <code>reboot relational database</code> operation supports tag-based access control via resource tags applied to the resource identified by relationalDatabaseName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param relationalDatabaseName:  The name of your database to reboot.<br/>
        :return: operations:  An object describing the result of your reboot relational database request.<br/>        """
        pass

    def release_static_ip(self, staticIpName: str) -> ReleaseStaticIpResult:
        """Deletes a specific static IP from your account.

        :param self:
        :param staticIpName:  The name of the static IP to delete.<br/>
        :return: operations:  An array of key-value pairs containing information about the request operation.<br/>        """
        pass

    def start_instance(self, instanceName: str) -> StartInstanceResult:
        """Starts a specific Amazon Lightsail instance from a stopped state. To restart an instance, use the <code>reboot instance</code> operation. <note> When you start a stopped instance, Lightsail assigns a new public IP address to the instance. To use the same IP address after stopping and starting an instance, create a static IP address and attach it to the instance. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/lightsail-create-static-ip">Lightsail Dev Guide</a>. </note> The <code>start instance</code> operation supports tag-based access control via resource tags applied to the resource identified by instanceName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param instanceName:  The name of the instance (a virtual private server) to start.<br/>
        :return: operations:  An array of key-value pairs containing information about the request operation.<br/>        """
        pass

    def start_relational_database(self, relationalDatabaseName: str) -> StartRelationalDatabaseResult:
        """Starts a specific database from a stopped state in Amazon Lightsail. To restart a database, use the <code>reboot relational database</code> operation. The <code>start relational database</code> operation supports tag-based access control via resource tags applied to the resource identified by relationalDatabaseName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param relationalDatabaseName:  The name of your database to start.<br/>
        :return: operations:  An object describing the result of your start relational database request.<br/>        """
        pass

    def stop_instance(self, instanceName: str, force: boolean=None) -> StopInstanceResult:
        """Stops a specific Amazon Lightsail instance that is currently running. <note> When you start a stopped instance, Lightsail assigns a new public IP address to the instance. To use the same IP address after stopping and starting an instance, create a static IP address and attach it to the instance. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/lightsail-create-static-ip">Lightsail Dev Guide</a>. </note> The <code>stop instance</code> operation supports tag-based access control via resource tags applied to the resource identified by instanceName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param force:  When set to <code>True</code>, forces a Lightsail instance that is stuck in a <code>stopping</code> state to stop. <important> Only use the <code>force</code> parameter if your instance is stuck in the <code>stopping</code> state. In any other state, your instance should stop normally without adding this parameter to your API request. </important><br/>instanceName:  The name of the instance (a virtual private server) to stop.<br/>
        :return: operations:  An array of key-value pairs containing information about the request operation.<br/>        """
        pass

    def stop_relational_database(self, relationalDatabaseName: str, relationalDatabaseSnapshotName: str=None) -> StopRelationalDatabaseResult:
        """Stops a specific database that is currently running in Amazon Lightsail. The <code>stop relational database</code> operation supports tag-based access control via resource tags applied to the resource identified by relationalDatabaseName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param relationalDatabaseSnapshotName:  The name of your new database snapshot to be created before stopping your database.<br/>relationalDatabaseName:  The name of your database to stop.<br/>
        :return: operations:  An object describing the result of your stop relational database request.<br/>        """
        pass

    def tag_resource(self, tags: List[Tag], resourceName: str) -> TagResourceResult:
        """Adds one or more tags to the specified Amazon Lightsail resource. Each resource can have a maximum of 50 tags. Each tag consists of a key and an optional value. Tag keys must be unique per resource. For more information about tags, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-tags">Lightsail Dev Guide</a>. The <code>tag resource</code> operation supports tag-based access control via request tags and resource tags applied to the resource identified by resourceName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param tags:  The tag key and optional value.<br/>resourceName:  The name of the resource to which you are adding tags.<br/>
        :return: operations:  A list of objects describing the API operation.<br/>        """
        pass

    def unpeer_vpc(self) -> UnpeerVpcResult:
        """Attempts to unpeer the Lightsail VPC from the user's default VPC.

        :param self:
        :param 
        :return: operation:  An array of key-value pairs containing information about the request operation.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorDetails</i> The error details.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>errorCode</i> The error code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusChangedAt</i> The timestamp when the status was changed (e.g., <code>1479816991.349</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>operationType</i> The type of operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>operationDetails</i> Details about the operation (e.g., <code>Debian-1GB-Ohio-1</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>isTerminal</i> A Boolean value indicating whether the operation is terminal.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>location</i> The region and Availability Zone.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>location</b>: The region and Availability Zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdAt</i> The timestamp when the operation was initialized (e.g., <code>1479816991.349</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceType</i> The resource type. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceName</i> The resource name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>regionName</i> The AWS Region name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>availabilityZone</i> The Availability Zone. Follows the format <code>us-east-2a</code> (case-sensitive).<br/>        """
        pass

    def untag_resource(self, tagKeys: List[TagKey], resourceName: str) -> UntagResourceResult:
        """Deletes the specified set of tag keys and their values from the specified Amazon Lightsail resource. The <code>untag resource</code> operation supports tag-based access control via request tags and resource tags applied to the resource identified by resourceName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param tagKeys:  The tag keys to delete from the specified resource.<br/>resourceName:  The name of the resource from which you are removing a tag.<br/>
        :return: operations:  A list of objects describing the API operation.<br/>        """
        pass

    def update_domain_entry(self, domainEntry: DomainEntry, domainName: str) -> UpdateDomainEntryResult:
        """Updates a domain recordset after it is created. The <code>update domain entry</code> operation supports tag-based access control via resource tags applied to the resource identified by domainName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param domainEntry:  An array of key-value pairs containing information about the domain entry.:<br/>domainName:  The name of the domain recordset to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>options</i> (Deprecated) The options for the domain entry. <note> In releases prior to November 29, 2017, this parameter was not included in the API response. It is now deprecated. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>type</i> The type of domain entry, such as address (A), canonical name (CNAME), mail exchanger (MX), name server (NS), start of authority (SOA), service locator (SRV), or text (TXT). The following domain entry types can be used: <ul> <li>  <code>A</code>  </li> <li>  <code>CNAME</code>  </li> <li>  <code>MX</code>  </li> <li>  <code>NS</code>  </li> <li>  <code>SOA</code>  </li> <li>  <code>SRV</code>  </li> <li>  <code>TXT</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>isAlias</i> When <code>true</code>, specifies whether the domain entry is an alias used by the Lightsail load balancer. You can include an alias (A type) record in your request, which points to a load balancer DNS name and routes traffic to your load balancer<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>target</i> The target AWS name server (e.g., <code>ns-111.awsdns-22.com.</code>). For Lightsail load balancers, the value looks like <code>ab1234c56789c6b86aba6fb203d443bc-123456789.us-east-2.elb.amazonaws.com</code>. Be sure to also set <code>isAlias</code> to <code>true</code> when setting up an A record for a load balancer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>id</i> The ID of the domain recordset entry.<br/>
        :return: operations:  An array of key-value pairs containing information about the request operation.<br/>        """
        pass

    def update_load_balancer_attribute(self, attributeValue: str, attributeName: str, loadBalancerName: str) -> UpdateLoadBalancerAttributeResult:
        """Updates the specified attribute for a load balancer. You can only update one attribute at a time. The <code>update load balancer attribute</code> operation supports tag-based access control via resource tags applied to the resource identified by loadBalancerName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param attributeValue:  The value that you want to specify for the attribute name.<br/>attributeName:  The name of the attribute you want to update. Valid values are below.<br/>loadBalancerName:  The name of the load balancer that you want to modify (e.g., <code>my-load-balancer</code>.<br/>
        :return: operations:  An object describing the API operations.<br/>        """
        pass

    def update_relational_database(self, relationalDatabaseName: str, masterUserPassword: str=None, rotateMasterUserPassword: boolean=None, preferredBackupWindow: str=None, preferredMaintenanceWindow: str=None, enableBackupRetention: boolean=None, disableBackupRetention: boolean=None, publiclyAccessible: boolean=None, applyImmediately: boolean=None) -> UpdateRelationalDatabaseResult:
        """Allows the update of one or more attributes of a database in Amazon Lightsail. Updates are applied immediately, or in cases where the updates could result in an outage, are applied during the database's predefined maintenance window. The <code>update relational database</code> operation supports tag-based access control via resource tags applied to the resource identified by relationalDatabaseName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param applyImmediately:  When <code>true</code>, applies changes immediately. When <code>false</code>, applies changes during the preferred maintenance window. Some changes may cause an outage. Default: <code>false</code> <br/>publiclyAccessible:  Specifies the accessibility options for your database. A value of <code>true</code> specifies a database that is available to resources outside of your Lightsail account. A value of <code>false</code> specifies a database that is available only to your Lightsail resources in the same region as your database.<br/>disableBackupRetention:  When <code>true</code>, disables automated backup retention for your database. Disabling backup retention deletes all automated database backups. Before disabling this, you may want to create a snapshot of your database using the <code>create relational database snapshot</code> operation. Updates are applied during the next maintenance window because this can result in an outage.<br/>enableBackupRetention:  When <code>true</code>, enables automated backup retention for your database. Updates are applied during the next maintenance window because this can result in an outage.<br/>preferredMaintenanceWindow:  The weekly time range during which system maintenance can occur on your database. The default is a 30-minute window selected at random from an 8-hour block of time for each AWS Region, occurring on a random day of the week. Constraints: <ul> <li> Must be in the <code>ddd:hh24:mi-ddd:hh24:mi</code> format. </li> <li> Valid days: Mon, Tue, Wed, Thu, Fri, Sat, Sun. </li> <li> Must be at least 30 minutes. </li> <li> Specified in Universal Coordinated Time (UTC). </li> <li> Example: <code>Tue:17:00-Tue:17:30</code>  </li> </ul><br/>preferredBackupWindow:  The daily time range during which automated backups are created for your database if automated backups are enabled. Constraints: <ul> <li> Must be in the <code>hh24:mi-hh24:mi</code> format. Example: <code>16:00-16:30</code>  </li> <li> Specified in Universal Coordinated Time (UTC). </li> <li> Must not conflict with the preferred maintenance window. </li> <li> Must be at least 30 minutes. </li> </ul><br/>rotateMasterUserPassword:  When <code>true</code>, the master user password is changed to a new strong password generated by Lightsail. Use the <code>get relational database master user password</code> operation to get the new password.<br/>masterUserPassword:  The password for the master user of your database. The password can include any printable ASCII character except "/", triple-quotes, or "@". Constraints: Must contain 8 to 41 characters.<br/>relationalDatabaseName:  The name of your database to update.<br/>
        :return: operations:  An object describing the result of your update relational database request.<br/>        """
        pass

    def update_relational_database_parameters(self, parameters: List[RelationalDatabaseParameter], relationalDatabaseName: str) -> UpdateRelationalDatabaseParametersResult:
        """Allows the update of one or more parameters of a database in Amazon Lightsail. Parameter updates don't cause outages; therefore, their application is not subject to the preferred maintenance window. However, there are two ways in which paramater updates are applied: <code>dynamic</code> or <code>pending-reboot</code>. Parameters marked with a <code>dynamic</code> apply type are applied immediately. Parameters marked with a <code>pending-reboot</code> apply type are applied only after the database is rebooted using the <code>reboot relational database</code> operation. The <code>update relational database parameters</code> operation supports tag-based access control via resource tags applied to the resource identified by relationalDatabaseName. For more information, see the <a href="https://lightsail.aws.amazon.com/ls/docs/en/articles/amazon-lightsail-controlling-access-using-tags">Lightsail Dev Guide</a>.

        :param self:
        :param parameters:  The database parameters to update.<br/>relationalDatabaseName:  The name of your database for which to update parameters.<br/>
        :return: operations:  An object describing the result of your update relational database parameters request.<br/>        """
        pass



class CloudWatchLogs(BaseClient):
    """You can use Amazon CloudWatch Logs to monitor, store, and access your log files from Amazon EC2 instances, AWS CloudTrail, or other sources. You can then retrieve the associated log data from CloudWatch Logs using the CloudWatch console, CloudWatch Logs commands in the AWS CLI, CloudWatch Logs API, or CloudWatch Logs SDK. You can use CloudWatch Logs to: <ul> <li>  <b>Monitor logs from EC2 instances in real-time</b>: You can use CloudWatch Logs to monitor applications and systems using log data. For example, CloudWatch Logs can track the number of errors that occur in your application logs and send you a notification whenever the rate of errors exceeds a threshold that you specify. CloudWatch Logs uses your log data for monitoring; so, no code changes are required. For example, you can monitor application logs for specific literal terms (such as "NullReferenceException") or count the number of occurrences of a literal term at a particular position in log data (such as "404" status codes in an Apache access log). When the term you are searching for is found, CloudWatch Logs reports the data to a CloudWatch metric that you specify. </li> <li>  <b>Monitor AWS CloudTrail logged events</b>: You can create alarms in CloudWatch and receive notifications of particular API activity as captured by CloudTrail and use the notification to perform troubleshooting. </li> <li>  <b>Archive log data</b>: You can use CloudWatch Logs to store your log data in highly durable storage. You can change the log retention setting so that any log events older than this setting are automatically deleted. The CloudWatch Logs agent makes it easy to quickly send both rotated and non-rotated log data off of a host and into the log service. You can then access the raw log data when you need it. </li> </ul>"""


    class DescribeDestinationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_destinations` for valid parameters"""
            pass
    class DescribeLogGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_log_groups` for valid parameters"""
            pass
    class DescribeLogStreamsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_log_streams` for valid parameters"""
            pass
    class DescribeMetricFiltersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_metric_filters` for valid parameters"""
            pass
    class DescribeSubscriptionFiltersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_subscription_filters` for valid parameters"""
            pass
    class FilterLogEventsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `filter_log_events` for valid parameters"""
            pass
    class DescribeExportTasksPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_export_tasks` for valid parameters"""
            pass
    class DescribeQueriesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_queries` for valid parameters"""
            pass
    class DescribeResourcePoliciesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_resource_policies` for valid parameters"""
            pass


    class AssociateKmsKeyRequest(Mapping):
        pass
    
    class CancelExportTaskRequest(Mapping):
        pass
    
    class CreateExportTaskRequest(Mapping):
        pass
    
    class CreateExportTaskResponse(Mapping):
        pass
    
    class Timestamp(object):
        pass
    
    class CreateLogGroupRequest(Mapping):
        pass
    
    class Tags(object):
        pass
    
    class CreateLogStreamRequest(Mapping):
        pass
    
    class DeleteDestinationRequest(Mapping):
        pass
    
    class DeleteLogGroupRequest(Mapping):
        pass
    
    class DeleteLogStreamRequest(Mapping):
        pass
    
    class DeleteMetricFilterRequest(Mapping):
        pass
    
    class DeleteResourcePolicyRequest(Mapping):
        pass
    
    class DeleteRetentionPolicyRequest(Mapping):
        pass
    
    class DeleteSubscriptionFilterRequest(Mapping):
        pass
    
    class DescribeDestinationsRequest(Mapping):
        pass
    
    class DescribeDestinationsResponse(Mapping):
        pass
    
    class DescribeExportTasksRequest(Mapping):
        pass
    
    class DescribeExportTasksResponse(Mapping):
        pass
    
    class DescribeLogGroupsRequest(Mapping):
        pass
    
    class DescribeLogGroupsResponse(Mapping):
        pass
    
    class DescribeLogStreamsRequest(Mapping):
        pass
    
    class DescribeLogStreamsResponse(Mapping):
        pass
    
    class Descending(object):
        pass
    
    class DescribeMetricFiltersRequest(Mapping):
        pass
    
    class DescribeMetricFiltersResponse(Mapping):
        pass
    
    class DescribeQueriesRequest(Mapping):
        pass
    
    class DescribeQueriesResponse(Mapping):
        pass
    
    class DescribeResourcePoliciesRequest(Mapping):
        pass
    
    class DescribeResourcePoliciesResponse(Mapping):
        pass
    
    class DescribeSubscriptionFiltersRequest(Mapping):
        pass
    
    class DescribeSubscriptionFiltersResponse(Mapping):
        pass
    
    class DisassociateKmsKeyRequest(Mapping):
        pass
    
    class FilterLogEventsRequest(Mapping):
        pass
    
    class FilterLogEventsResponse(Mapping):
        pass
    
    class LogStreamName(object):
        pass
    
    class Interleaved(object):
        pass
    
    class GetLogEventsRequest(Mapping):
        pass
    
    class GetLogEventsResponse(Mapping):
        pass
    
    class StartFromHead(object):
        pass
    
    class GetLogGroupFieldsRequest(Mapping):
        pass
    
    class GetLogGroupFieldsResponse(Mapping):
        pass
    
    class GetLogRecordRequest(Mapping):
        pass
    
    class GetLogRecordResponse(Mapping):
        pass
    
    class GetQueryResultsRequest(Mapping):
        pass
    
    class GetQueryResultsResponse(Mapping):
        pass
    
    class ListTagsLogGroupRequest(Mapping):
        pass
    
    class ListTagsLogGroupResponse(Mapping):
        pass
    
    class PutDestinationRequest(Mapping):
        pass
    
    class PutDestinationResponse(Mapping):
        pass
    
    class PutDestinationPolicyRequest(Mapping):
        pass
    
    class PutLogEventsRequest(Mapping):
        pass
    
    class PutLogEventsResponse(Mapping):
        pass
    
    class InputLogEvent(Mapping):
        pass
    
    class PutMetricFilterRequest(Mapping):
        pass
    
    class MetricTransformation(Mapping):
        pass
    
    class PutResourcePolicyRequest(Mapping):
        pass
    
    class PutResourcePolicyResponse(Mapping):
        pass
    
    class PutRetentionPolicyRequest(Mapping):
        pass
    
    class PutSubscriptionFilterRequest(Mapping):
        pass
    
    class StartQueryRequest(Mapping):
        pass
    
    class StartQueryResponse(Mapping):
        pass
    
    class StopQueryRequest(Mapping):
        pass
    
    class StopQueryResponse(Mapping):
        pass
    
    class TagLogGroupRequest(Mapping):
        pass
    
    class TestMetricFilterRequest(Mapping):
        pass
    
    class TestMetricFilterResponse(Mapping):
        pass
    
    class EventMessage(object):
        pass
    
    class UntagLogGroupRequest(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    def associate_kms_key(self, kmsKeyId: str, logGroupName: str):
        """Associates the specified AWS Key Management Service (AWS KMS) customer master key (CMK) with the specified log group. Associating an AWS KMS CMK with a log group overrides any existing associations between the log group and a CMK. After a CMK is associated with a log group, all newly ingested data for the log group is encrypted using the CMK. This association is stored as long as the data encrypted with the CMK is still within Amazon CloudWatch Logs. This enables Amazon CloudWatch Logs to decrypt this data whenever it is requested. Note that it can take up to 5 minutes for this operation to take effect. If you attempt to associate a CMK with a log group but the CMK does not exist or the CMK is disabled, you will receive an <code>InvalidParameterException</code> error. 

        :param self:
        :param kmsKeyId:  The Amazon Resource Name (ARN) of the CMK to use when encrypting log data. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#arn-syntax-kms">Amazon Resource Names - AWS Key Management Service (AWS KMS)</a>.<br/>logGroupName:  The name of the log group.<br/>
        :return:         """
        pass

    def cancel_export_task(self, taskId: str):
        """Cancels the specified export task. The task must be in the <code>PENDING</code> or <code>RUNNING</code> state.

        :param self:
        :param taskId:  The ID of the export task.<br/>
        :return:         """
        pass

    def create_export_task(self, destination: str, to: Timestamp, from_: Timestamp, logGroupName: str, taskName: str=None, logStreamNamePrefix: str=None, destinationPrefix: str=None) -> CreateExportTaskResponse:
        """Creates an export task, which allows you to efficiently export data from a log group to an Amazon S3 bucket. This is an asynchronous call. If all the required information is provided, this operation initiates an export task and responds with the ID of the task. After the task has started, you can use <a>DescribeExportTasks</a> to get the status of the export task. Each account can only have one active (<code>RUNNING</code> or <code>PENDING</code>) export task at a time. To cancel an export task, use <a>CancelExportTask</a>. You can export logs from multiple log groups or multiple time ranges to the same S3 bucket. To separate out log data for each export task, you can specify a prefix to be used as the Amazon S3 key prefix for all exported objects.

        :param self:
        :param destinationPrefix:  The prefix used as the start of the key for every object exported. If you don't specify a value, the default is <code>exportedlogs</code>.<br/>destination:  The name of S3 bucket for the exported log data. The bucket must be in the same AWS region.<br/>to:  The end time of the range for the request, expressed as the number of milliseconds after Jan 1, 1970 00:00:00 UTC. Events with a timestamp later than this time are not exported.<br/>from:  The start time of the range for the request, expressed as the number of milliseconds after Jan 1, 1970 00:00:00 UTC. Events with a timestamp earlier than this time are not exported.<br/>logStreamNamePrefix:  Export only log streams that match the provided prefix. If you don't specify a value, no prefix filter is applied.<br/>logGroupName:  The name of the log group.<br/>taskName:  The name of the export task.<br/>
        :return: taskId:  The ID of the export task.<br/>        """
        pass

    def create_log_group(self, logGroupName: str, kmsKeyId: str=None, tags: Tags=None):
        """Creates a log group with the specified name. You can create up to 5000 log groups per account. You must use the following guidelines when naming a log group: <ul> <li> Log group names must be unique within a region for an AWS account. </li> <li> Log group names can be between 1 and 512 characters long. </li> <li> Log group names consist of the following characters: a-z, A-Z, 0-9, '_' (underscore), '-' (hyphen), '/' (forward slash), and '.' (period). </li> </ul> If you associate a AWS Key Management Service (AWS KMS) customer master key (CMK) with the log group, ingested data is encrypted using the CMK. This association is stored as long as the data encrypted with the CMK is still within Amazon CloudWatch Logs. This enables Amazon CloudWatch Logs to decrypt this data whenever it is requested. If you attempt to associate a CMK with the log group but the CMK does not exist or the CMK is disabled, you will receive an <code>InvalidParameterException</code> error. 

        :param self:
        :param tags:  The key-value pairs to use for the tags.<br/>kmsKeyId:  The Amazon Resource Name (ARN) of the CMK to use when encrypting log data. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#arn-syntax-kms">Amazon Resource Names - AWS Key Management Service (AWS KMS)</a>.<br/>logGroupName:  The name of the log group.<br/>
        :return:         """
        pass

    def create_log_stream(self, logStreamName: str, logGroupName: str):
        """Creates a log stream for the specified log group. There is no limit on the number of log streams that you can create for a log group. You must use the following guidelines when naming a log stream: <ul> <li> Log stream names must be unique within the log group. </li> <li> Log stream names can be between 1 and 512 characters long. </li> <li> The ':' (colon) and '*' (asterisk) characters are not allowed. </li> </ul>

        :param self:
        :param logStreamName:  The name of the log stream.<br/>logGroupName:  The name of the log group.<br/>
        :return:         """
        pass

    def delete_destination(self, destinationName: str):
        """Deletes the specified destination, and eventually disables all the subscription filters that publish to it. This operation does not delete the physical resource encapsulated by the destination.

        :param self:
        :param destinationName:  The name of the destination.<br/>
        :return:         """
        pass

    def delete_log_group(self, logGroupName: str):
        """Deletes the specified log group and permanently deletes all the archived log events associated with the log group.

        :param self:
        :param logGroupName:  The name of the log group.<br/>
        :return:         """
        pass

    def delete_log_stream(self, logStreamName: str, logGroupName: str):
        """Deletes the specified log stream and permanently deletes all the archived log events associated with the log stream.

        :param self:
        :param logStreamName:  The name of the log stream.<br/>logGroupName:  The name of the log group.<br/>
        :return:         """
        pass

    def delete_metric_filter(self, filterName: str, logGroupName: str):
        """Deletes the specified metric filter.

        :param self:
        :param filterName:  The name of the metric filter.<br/>logGroupName:  The name of the log group.<br/>
        :return:         """
        pass

    def delete_resource_policy(self, policyName: str=None):
        """Deletes a resource policy from this account. This revokes the access of the identities in that policy to put log events to this account.

        :param self:
        :param policyName:  The name of the policy to be revoked. This parameter is required.<br/>
        :return:         """
        pass

    def delete_retention_policy(self, logGroupName: str):
        """Deletes the specified retention policy. Log events do not expire if they belong to log groups without a retention policy.

        :param self:
        :param logGroupName:  The name of the log group.<br/>
        :return:         """
        pass

    def delete_subscription_filter(self, filterName: str, logGroupName: str):
        """Deletes the specified subscription filter.

        :param self:
        :param filterName:  The name of the subscription filter.<br/>logGroupName:  The name of the log group.<br/>
        :return:         """
        pass

    def describe_destinations(self, DestinationNamePrefix: str=None, nextToken: str=None, limit: int=None) -> DescribeDestinationsResponse:
        """Lists all your destinations. The results are ASCII-sorted by destination name.

        :param self:
        :param limit:  The maximum number of items returned. If you don't specify a value, the default is up to 50 items.<br/>nextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>DestinationNamePrefix:  The prefix to match. If you don't specify a value, no prefix filter is applied.<br/>
        :return: nextToken:  The token for the next set of items to return. The token expires after 24 hours.<br/>destinations:  The destinations.<br/>        """
        pass

    def describe_export_tasks(self, taskId: str=None, statusCode: str=None, nextToken: str=None, limit: int=None) -> DescribeExportTasksResponse:
        """Lists the specified export tasks. You can list all your export tasks or filter the results based on task ID or task status.

        :param self:
        :param limit:  The maximum number of items returned. If you don't specify a value, the default is up to 50 items.<br/>nextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>statusCode:  The status code of the export task. Specifying a status code filters the results to zero or more export tasks.<br/>taskId:  The ID of the export task. Specifying a task ID filters the results to zero or one export tasks.<br/>
        :return: nextToken:  The token for the next set of items to return. The token expires after 24 hours.<br/>exportTasks:  The export tasks.<br/>        """
        pass

    def describe_log_groups(self, logGroupNamePrefix: str=None, nextToken: str=None, limit: int=None) -> DescribeLogGroupsResponse:
        """Lists the specified log groups. You can list all your log groups or filter the results by prefix. The results are ASCII-sorted by log group name.

        :param self:
        :param limit:  The maximum number of items returned. If you don't specify a value, the default is up to 50 items.<br/>nextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>logGroupNamePrefix:  The prefix to match.<br/>
        :return: nextToken:  The token for the next set of items to return. The token expires after 24 hours.<br/>logGroups:  The log groups.<br/>        """
        pass

    def describe_log_streams(self, logGroupName: str, logStreamNamePrefix: str=None, orderBy: str=None, descending: Descending=None, nextToken: str=None, limit: int=None) -> DescribeLogStreamsResponse:
        """Lists the log streams for the specified log group. You can list all the log streams or filter the results by prefix. You can also control how the results are ordered. This operation has a limit of five transactions per second, after which transactions are throttled.

        :param self:
        :param limit:  The maximum number of items returned. If you don't specify a value, the default is up to 50 items.<br/>nextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>descending:  If the value is true, results are returned in descending order. If the value is to false, results are returned in ascending order. The default value is false.<br/>orderBy:  If the value is <code>LogStreamName</code>, the results are ordered by log stream name. If the value is <code>LastEventTime</code>, the results are ordered by the event time. The default value is <code>LogStreamName</code>. If you order the results by event time, you cannot specify the <code>logStreamNamePrefix</code> parameter. lastEventTimestamp represents the time of the most recent log event in the log stream in CloudWatch Logs. This number is expressed as the number of milliseconds after Jan 1, 1970 00:00:00 UTC. lastEventTimeStamp updates on an eventual consistency basis. It typically updates in less than an hour from ingestion, but may take longer in some rare situations.<br/>logStreamNamePrefix:  The prefix to match. If <code>orderBy</code> is <code>LastEventTime</code>,you cannot specify this parameter.<br/>logGroupName:  The name of the log group.<br/>
        :return: nextToken:  The token for the next set of items to return. The token expires after 24 hours.<br/>logStreams:  The log streams.<br/>        """
        pass

    def describe_metric_filters(self, logGroupName: str=None, filterNamePrefix: str=None, nextToken: str=None, limit: int=None, metricName: str=None, metricNamespace: str=None) -> DescribeMetricFiltersResponse:
        """Lists the specified metric filters. You can list all the metric filters or filter the results by log name, prefix, metric name, or metric namespace. The results are ASCII-sorted by filter name.

        :param self:
        :param metricNamespace:  Filters results to include only those in the specified namespace. If you include this parameter in your request, you must also include the <code>metricName</code> parameter.<br/>metricName:  Filters results to include only those with the specified metric name. If you include this parameter in your request, you must also include the <code>metricNamespace</code> parameter.<br/>limit:  The maximum number of items returned. If you don't specify a value, the default is up to 50 items.<br/>nextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>filterNamePrefix:  The prefix to match.<br/>logGroupName:  The name of the log group.<br/>
        :return: nextToken:  The token for the next set of items to return. The token expires after 24 hours.<br/>metricFilters:  The metric filters.<br/>        """
        pass

    def describe_queries(self, logGroupName: str=None, status: str=None, maxResults: int=None, nextToken: str=None) -> DescribeQueriesResponse:
        """Returns a list of CloudWatch Logs Insights queries that are scheduled, executing, or have been executed recently in this account. You can request all queries, or limit it to queries of a specific log group or queries with a certain status.

        :param self:
        :param nextToken:  The token for the next set of items to return. The token expires after 24 hours.<br/>maxResults:  Limits the number of returned queries to the specified number.<br/>status:  Limits the returned queries to only those that have the specified status. Valid values are <code>Cancelled</code>, <code>Complete</code>, <code>Failed</code>, <code>Running</code>, and <code>Scheduled</code>.<br/>logGroupName:  Limits the returned queries to only those for the specified log group.<br/>
        :return: nextToken:  The token for the next set of items to return. The token expires after 24 hours.<br/>queries:  The list of queries that match the request.<br/>        """
        pass

    def describe_resource_policies(self, nextToken: str=None, limit: int=None) -> DescribeResourcePoliciesResponse:
        """Lists the resource policies in this account.

        :param self:
        :param limit:  The maximum number of resource policies to be displayed with one call of this API.<br/>nextToken:  The token for the next set of items to return. The token expires after 24 hours.<br/>
        :return: nextToken:  The token for the next set of items to return. The token expires after 24 hours.<br/>resourcePolicies:  The resource policies that exist in this account.<br/>        """
        pass

    def describe_subscription_filters(self, logGroupName: str, filterNamePrefix: str=None, nextToken: str=None, limit: int=None) -> DescribeSubscriptionFiltersResponse:
        """Lists the subscription filters for the specified log group. You can list all the subscription filters or filter the results by prefix. The results are ASCII-sorted by filter name.

        :param self:
        :param limit:  The maximum number of items returned. If you don't specify a value, the default is up to 50 items.<br/>nextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>filterNamePrefix:  The prefix to match. If you don't specify a value, no prefix filter is applied.<br/>logGroupName:  The name of the log group.<br/>
        :return: nextToken:  The token for the next set of items to return. The token expires after 24 hours.<br/>subscriptionFilters:  The subscription filters.<br/>        """
        pass

    def disassociate_kms_key(self, logGroupName: str):
        """Disassociates the associated AWS Key Management Service (AWS KMS) customer master key (CMK) from the specified log group. After the AWS KMS CMK is disassociated from the log group, AWS CloudWatch Logs stops encrypting newly ingested data for the log group. All previously ingested data remains encrypted, and AWS CloudWatch Logs requires permissions for the CMK whenever the encrypted data is requested. Note that it can take up to 5 minutes for this operation to take effect.

        :param self:
        :param logGroupName:  The name of the log group.<br/>
        :return:         """
        pass

    def filter_log_events(self, logGroupName: str, logStreamNames: List[LogStreamName]=None, logStreamNamePrefix: str=None, startTime: Timestamp=None, endTime: Timestamp=None, filterPattern: str=None, nextToken: str=None, limit: int=None, interleaved: Interleaved=None) -> FilterLogEventsResponse:
        """Lists log events from the specified log group. You can list all the log events or filter the results using a filter pattern, a time range, and the name of the log stream. By default, this operation returns as many log events as can fit in 1 MB (up to 10,000 log events), or all the events found within the time range that you specify. If the results include a token, then there are more log events available, and you can get additional results by specifying the token in a subsequent call.

        :param self:
        :param interleaved:  If the value is true, the operation makes a best effort to provide responses that contain events from multiple log streams within the log group, interleaved in a single response. If the value is false, all the matched log events in the first log stream are searched first, then those in the next log stream, and so on. The default is false.  <b>IMPORTANT:</b> Starting on June 17, 2019, this parameter will be ignored and the value will be assumed to be true. The response from this operation will always interleave events from multiple log streams within a log group.<br/>limit:  The maximum number of events to return. The default is 10,000 events.<br/>nextToken:  The token for the next set of events to return. (You received this token from a previous call.)<br/>filterPattern:  The filter pattern to use. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudWatch/latest/logs/FilterAndPatternSyntax.html">Filter and Pattern Syntax</a>. If not provided, all the events are matched.<br/>endTime:  The end of the time range, expressed as the number of milliseconds after Jan 1, 1970 00:00:00 UTC. Events with a timestamp later than this time are not returned.<br/>startTime:  The start of the time range, expressed as the number of milliseconds after Jan 1, 1970 00:00:00 UTC. Events with a timestamp before this time are not returned.<br/>logStreamNamePrefix:  Filters the results to include only events from log streams that have names starting with this prefix. If you specify a value for both <code>logStreamNamePrefix</code> and <code>logStreamNames</code>, but the value for <code>logStreamNamePrefix</code> does not match any log stream names specified in <code>logStreamNames</code>, the action returns an <code>InvalidParameterException</code> error.<br/>logStreamNames:  Filters the results to only logs from the log streams in this list. If you specify a value for both <code>logStreamNamePrefix</code> and <code>logStreamNames</code>, the action returns an <code>InvalidParameterException</code> error.<br/>logGroupName:  The name of the log group to search.<br/>
        :return: nextToken:  The token to use when requesting the next set of items. The token expires after 24 hours.<br/>searchedLogStreams:  Indicates which log streams have been searched and whether each has been searched completely.<br/>events:  The matched events.<br/>        """
        pass

    def get_log_events(self, logStreamName: str, logGroupName: str, startTime: Timestamp=None, endTime: Timestamp=None, nextToken: str=None, limit: int=None, startFromHead: StartFromHead=None) -> GetLogEventsResponse:
        """Lists log events from the specified log stream. You can list all the log events or filter using a time range. By default, this operation returns as many log events as can fit in a response size of 1MB (up to 10,000 log events). You can get additional log events by specifying one of the tokens in a subsequent call.

        :param self:
        :param startFromHead:  If the value is true, the earliest log events are returned first. If the value is false, the latest log events are returned first. The default value is false.<br/>limit:  The maximum number of log events returned. If you don't specify a value, the maximum is as many log events as can fit in a response size of 1 MB, up to 10,000 log events.<br/>nextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>endTime:  The end of the time range, expressed as the number of milliseconds after Jan 1, 1970 00:00:00 UTC. Events with a timestamp equal to or later than this time are not included.<br/>startTime:  The start of the time range, expressed as the number of milliseconds after Jan 1, 1970 00:00:00 UTC. Events with a timestamp equal to this time or later than this time are included. Events with a timestamp earlier than this time are not included.<br/>logStreamName:  The name of the log stream.<br/>logGroupName:  The name of the log group.<br/>
        :return: nextBackwardToken:  The token for the next set of items in the backward direction. The token expires after 24 hours. This token will never be null. If you have reached the end of the stream, it will return the same token you passed in.<br/>nextForwardToken:  The token for the next set of items in the forward direction. The token expires after 24 hours. If you have reached the end of the stream, it will return the same token you passed in.<br/>events:  The events.<br/>        """
        pass

    def get_log_group_fields(self, logGroupName: str, time: Timestamp=None) -> GetLogGroupFieldsResponse:
        """Returns a list of the fields that are included in log events in the specified log group, along with the percentage of log events that contain each field. The search is limited to a time period that you specify. In the results, fields that start with @ are fields generated by CloudWatch Logs. For example, <code>@timestamp</code> is the timestamp of each log event. The response results are sorted by the frequency percentage, starting with the highest percentage.

        :param self:
        :param time:  The time to set as the center of the query. If you specify <code>time</code>, the 8 minutes before and 8 minutes after this time are searched. If you omit <code>time</code>, the past 15 minutes are queried. The <code>time</code> value is specified as epoch time, the number of seconds since January 1, 1970, 00:00:00 UTC.<br/>logGroupName:  The name of the log group to search.<br/>
        :return: logGroupFields:  The array of fields found in the query. Each object in the array contains the name of the field, along with the percentage of time it appeared in the log events that were queried.<br/>        """
        pass

    def get_log_record(self, logRecordPointer: str) -> GetLogRecordResponse:
        """Retrieves all the fields and values of a single log event. All fields are retrieved, even if the original query that produced the <code>logRecordPointer</code> retrieved only a subset of fields. Fields are returned as field name/field value pairs. Additionally, the entire unparsed log event is returned within <code>@message</code>.

        :param self:
        :param logRecordPointer:  The pointer corresponding to the log event record you want to retrieve. You get this from the response of a <code>GetQueryResults</code> operation. In that response, the value of the <code>@ptr</code> field for a log event is the value to use as <code>logRecordPointer</code> to retrieve that complete log event record.<br/>
        :return: logRecord:  The requested log event, as a JSON string.<br/>        """
        pass

    def get_query_results(self, queryId: str) -> GetQueryResultsResponse:
        """Returns the results from the specified query. If the query is in progress, partial results of that current execution are returned. Only the fields requested in the query are returned, along with a <code>@ptr</code> field which is the identifier for the log record. You can use the value of <code>@ptr</code> in a operation to get the full log record.  <code>GetQueryResults</code> does not start a query execution. To run a query, use .

        :param self:
        :param queryId:  The ID number of the query.<br/>
        :return: status:  The status of the most recent running of the query. Possible values are <code>Cancelled</code>, <code>Complete</code>, <code>Failed</code>, <code>Running</code>, <code>Scheduled</code>, <code>Timeout</code>, and <code>Unknown</code>. Queries time out after 15 minutes of execution. To avoid having your queries time out, reduce the time range being searched, or partition your query into a number of queries.<br/>statistics:  Includes the number of log events scanned by the query, the number of log events that matched the query criteria, and the total number of bytes in the log events that were scanned.:<br/>results:  The log events that matched the query criteria during the most recent time it ran. The <code>results</code> value is an array of arrays. Each log event is one object in the top-level array. Each of these log event objects is an array of <code>field</code>/<code>value</code> pairs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>bytesScanned</i> The total number of bytes in the log events scanned during the query.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>recordsScanned</i> The total number of log events scanned during the query.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>recordsMatched</i> The number of log events that matched the query string.<br/>        """
        pass

    def list_tags_log_group(self, logGroupName: str) -> ListTagsLogGroupResponse:
        """Lists the tags for the specified log group.

        :param self:
        :param logGroupName:  The name of the log group.<br/>
        :return: tags:  The tags for the log group.<br/>        """
        pass

    def put_destination(self, roleArn: str, targetArn: str, destinationName: str) -> PutDestinationResponse:
        """Creates or updates a destination. A destination encapsulates a physical resource (such as an Amazon Kinesis stream) and enables you to subscribe to a real-time stream of log events for a different account, ingested using <a>PutLogEvents</a>. Currently, the only supported physical resource is a Kinesis stream belonging to the same account as the destination. Through an access policy, a destination controls what is written to its Kinesis stream. By default, <code>PutDestination</code> does not set any access policy with the destination, which means a cross-account user cannot call <a>PutSubscriptionFilter</a> against this destination. To enable this, the destination owner must call <a>PutDestinationPolicy</a> after <code>PutDestination</code>.

        :param self:
        :param roleArn:  The ARN of an IAM role that grants CloudWatch Logs permissions to call the Amazon Kinesis PutRecord operation on the destination stream.<br/>targetArn:  The ARN of an Amazon Kinesis stream to which to deliver matching log events.<br/>destinationName:  A name for the destination.<br/>
        :return: destination:  The destination.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationTime</i> The creation time of the destination, expressed as the number of milliseconds after Jan 1, 1970 00:00:00 UTC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The ARN of this destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>accessPolicy</i> An IAM policy document that governs which AWS accounts can create subscription filters against this destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> A role for impersonation, used when delivering log events to the target.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>targetArn</i> The Amazon Resource Name (ARN) of the physical target to where the log events are delivered (for example, a Kinesis stream).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>destinationName</i> The name of the destination.<br/>        """
        pass

    def put_destination_policy(self, accessPolicy: str, destinationName: str):
        """Creates or updates an access policy associated with an existing destination. An access policy is an <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/policies_overview.html">IAM policy document</a> that is used to authorize claims to register a subscription filter against a given destination.

        :param self:
        :param accessPolicy:  An IAM policy document that authorizes cross-account users to deliver their log events to the associated destination.<br/>destinationName:  A name for an existing destination.<br/>
        :return:         """
        pass

    def put_log_events(self, logEvents: List[InputLogEvent], logStreamName: str, logGroupName: str, sequenceToken: str=None) -> PutLogEventsResponse:
        """Uploads a batch of log events to the specified log stream. You must include the sequence token obtained from the response of the previous call. An upload in a newly created log stream does not require a sequence token. You can also get the sequence token using <a>DescribeLogStreams</a>. If you call <code>PutLogEvents</code> twice within a narrow time period using the same value for <code>sequenceToken</code>, both calls may be successful, or one may be rejected. The batch of events must satisfy the following constraints: <ul> <li> The maximum batch size is 1,048,576 bytes, and this size is calculated as the sum of all event messages in UTF-8, plus 26 bytes for each log event. </li> <li> None of the log events in the batch can be more than 2 hours in the future. </li> <li> None of the log events in the batch can be older than 14 days or older than the retention period of the log group. </li> <li> The log events in the batch must be in chronological ordered by their timestamp. The timestamp is the time the event occurred, expressed as the number of milliseconds after Jan 1, 1970 00:00:00 UTC. (In AWS Tools for PowerShell and the AWS SDK for .NET, the timestamp is specified in .NET format: yyyy-mm-ddThh:mm:ss. For example, 2017-09-15T13:45:30.)  </li> <li> The maximum number of log events in a batch is 10,000. </li> <li> A batch of log events in a single request cannot span more than 24 hours. Otherwise, the operation fails. </li> </ul> If a call to PutLogEvents returns "UnrecognizedClientException" the most likely cause is an invalid AWS access key ID or secret key. 

        :param self:
        :param sequenceToken:  The sequence token obtained from the response of the previous <code>PutLogEvents</code> call. An upload in a newly created log stream does not require a sequence token. You can also get the sequence token using <a>DescribeLogStreams</a>. If you call <code>PutLogEvents</code> twice within a narrow time period using the same value for <code>sequenceToken</code>, both calls may be successful, or one may be rejected.<br/>logEvents:  The log events.<br/>logStreamName:  The name of the log stream.<br/>logGroupName:  The name of the log group.<br/>
        :return: rejectedLogEventsInfo:  The rejected events.:<br/>nextSequenceToken:  The next sequence token.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>expiredLogEventEndIndex</i> The expired log events.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tooOldLogEventEndIndex</i> The log events that are too old.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tooNewLogEventStartIndex</i> The log events that are too new.<br/>        """
        pass

    def put_metric_filter(self, metricTransformations: List[MetricTransformation], filterPattern: str, filterName: str, logGroupName: str):
        """Creates or updates a metric filter and associates it with the specified log group. Metric filters allow you to configure rules to extract metric data from log events ingested through <a>PutLogEvents</a>. The maximum number of metric filters that can be associated with a log group is 100.

        :param self:
        :param metricTransformations:  A collection of information that defines how metric data gets emitted.<br/>filterPattern:  A filter pattern for extracting metric data out of ingested log events.<br/>filterName:  A name for the metric filter.<br/>logGroupName:  The name of the log group.<br/>
        :return:         """
        pass

    def put_resource_policy(self, policyName: str=None, policyDocument: str=None) -> PutResourcePolicyResponse:
        """Creates or updates a resource policy allowing other AWS services to put log events to this account, such as Amazon Route 53. An account can have up to 10 resource policies per region.

        :param self:
        :param policyDocument:  Details of the new policy, including the identity of the principal that is enabled to put logs to this account. This is formatted as a JSON string. This parameter is required. The following example creates a resource policy enabling the Route 53 service to put DNS query logs in to the specified log group. Replace "logArn" with the ARN of your CloudWatch Logs resource, such as a log group or log stream.  <code>{ "Version": "2012-10-17", "Statement": [ { "Sid": "Route53LogsToCloudWatchLogs", "Effect": "Allow", "Principal": { "Service": [ "route53.amazonaws.com" ] }, "Action":"logs:PutLogEvents", "Resource": "logArn" } ] } </code> <br/>policyName:  Name of the new policy. This parameter is required.<br/>
        :return: resourcePolicy:  The new policy.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedTime</i> Timestamp showing when this policy was last updated, expressed as the number of milliseconds after Jan 1, 1970 00:00:00 UTC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>policyDocument</i> The details of the policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>policyName</i> The name of the resource policy.<br/>        """
        pass

    def put_retention_policy(self, retentionInDays: int, logGroupName: str):
        """Sets the retention of the specified log group. A retention policy allows you to configure the number of days for which to retain log events in the specified log group.

        :param self:
        :param retentionInDays:  The number of days to retain the log events in the specified log group. Possible values are: 1, 3, 5, 7, 14, 30, 60, 90, 120, 150, 180, 365, 400, 545, 731, 1827, and 3653.<br/>logGroupName:  The name of the log group.<br/>
        :return:         """
        pass

    def put_subscription_filter(self, destinationArn: str, filterPattern: str, filterName: str, logGroupName: str, roleArn: str=None, distribution: str=None):
        """Creates or updates a subscription filter and associates it with the specified log group. Subscription filters allow you to subscribe to a real-time stream of log events ingested through <a>PutLogEvents</a> and have them delivered to a specific destination. Currently, the supported destinations are: <ul> <li> An Amazon Kinesis stream belonging to the same account as the subscription filter, for same-account delivery. </li> <li> A logical destination that belongs to a different account, for cross-account delivery. </li> <li> An Amazon Kinesis Firehose delivery stream that belongs to the same account as the subscription filter, for same-account delivery. </li> <li> An AWS Lambda function that belongs to the same account as the subscription filter, for same-account delivery. </li> </ul> There can only be one subscription filter associated with a log group. If you are updating an existing filter, you must specify the correct name in <code>filterName</code>. Otherwise, the call fails because you cannot associate a second filter with a log group.

        :param self:
        :param distribution:  The method used to distribute log data to the destination. By default log data is grouped by log stream, but the grouping can be set to random for a more even distribution. This property is only applicable when the destination is an Amazon Kinesis stream. <br/>roleArn:  The ARN of an IAM role that grants CloudWatch Logs permissions to deliver ingested log events to the destination stream. You don't need to provide the ARN when you are working with a logical destination for cross-account delivery.<br/>destinationArn:  The ARN of the destination to deliver matching log events to. Currently, the supported destinations are: <ul> <li> An Amazon Kinesis stream belonging to the same account as the subscription filter, for same-account delivery. </li> <li> A logical destination (specified using an ARN) belonging to a different account, for cross-account delivery. </li> <li> An Amazon Kinesis Firehose delivery stream belonging to the same account as the subscription filter, for same-account delivery. </li> <li> An AWS Lambda function belonging to the same account as the subscription filter, for same-account delivery. </li> </ul><br/>filterPattern:  A filter pattern for subscribing to a filtered stream of log events.<br/>filterName:  A name for the subscription filter. If you are updating an existing filter, you must specify the correct name in <code>filterName</code>. Otherwise, the call fails because you cannot associate a second filter with a log group. To find the name of the filter currently associated with a log group, use <a>DescribeSubscriptionFilters</a>.<br/>logGroupName:  The name of the log group.<br/>
        :return:         """
        pass

    def start_query(self, queryString: str, endTime: Timestamp, startTime: Timestamp, logGroupName: str, limit: int=None) -> StartQueryResponse:
        """Schedules a query of a log group using CloudWatch Logs Insights. You specify the log group and time range to query, and the query string to use. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudWatch/latest/logs/CWL_QuerySyntax.html">CloudWatch Logs Insights Query Syntax</a>. Queries time out after 15 minutes of execution. If your queries are timing out, reduce the time range being searched, or partition your query into a number of queries.

        :param self:
        :param limit:  The maximum number of log events to return in the query. If the query string uses the <code>fields</code> command, only the specified fields and their values are returned.<br/>queryString:  The query string to use. For more information, see <a href="https://docs.aws.amazon.com/AmazonCloudWatch/latest/logs/CWL_QuerySyntax.html">CloudWatch Logs Insights Query Syntax</a>.<br/>endTime:  The end of the time range to query. The range is inclusive, so the specified end time is included in the query. Specified as epoch time, the number of seconds since January 1, 1970, 00:00:00 UTC.<br/>startTime:  The beginning of the time range to query. The range is inclusive, so the specified start time is included in the query. Specified as epoch time, the number of seconds since January 1, 1970, 00:00:00 UTC.<br/>logGroupName:  The log group on which to perform the query.<br/>
        :return: queryId:  The unique ID of the query. <br/>        """
        pass

    def stop_query(self, queryId: str) -> StopQueryResponse:
        """Stops a CloudWatch Logs Insights query that is in progress. If the query has already ended, the operation returns an error indicating that the specified query is not running.

        :param self:
        :param queryId:  The ID number of the query to stop. If necessary, you can use <code>DescribeQueries</code> to find this ID number.<br/>
        :return: success:  This is true if the query was stopped by the <code>StopQuery</code> operation.<br/>        """
        pass

    def tag_log_group(self, tags: Tags, logGroupName: str):
        """Adds or updates the specified tags for the specified log group. To list the tags for a log group, use <a>ListTagsLogGroup</a>. To remove tags, use <a>UntagLogGroup</a>. For more information about tags, see <a href="https://docs.aws.amazon.com/AmazonCloudWatch/latest/logs/log-group-tagging.html">Tag Log Groups in Amazon CloudWatch Logs</a> in the <i>Amazon CloudWatch Logs User Guide</i>.

        :param self:
        :param tags:  The key-value pairs to use for the tags.<br/>logGroupName:  The name of the log group.<br/>
        :return:         """
        pass

    def test_metric_filter(self, logEventMessages: List[EventMessage], filterPattern: str) -> TestMetricFilterResponse:
        """Tests the filter pattern of a metric filter against a sample of log event messages. You can use this operation to validate the correctness of a metric filter pattern.

        :param self:
        :param logEventMessages:  The log event messages to test.<br/>filterPattern:  A symbolic description of how CloudWatch Logs should interpret the data in each log event. For example, a log event may contain timestamps, IP addresses, strings, and so on. You use the filter pattern to specify what to look for in the log event message.<br/>
        :return: matches:  The matched events.<br/>        """
        pass

    def untag_log_group(self, tags: List[TagKey], logGroupName: str):
        """Removes the specified tags from the specified log group. To list the tags for a log group, use <a>ListTagsLogGroup</a>. To add tags, use <a>UntagLogGroup</a>.

        :param self:
        :param tags:  The tag keys. The corresponding tags are removed from the log group.<br/>logGroupName:  The name of the log group.<br/>
        :return:         """
        pass



class MachineLearning(BaseClient):
    """Definition of the public APIs exposed by Amazon Machine Learning"""

    class BatchPredictionAvailableWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_batch_predictions` for valid parameters"""
            pass
    class DataSourceAvailableWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_data_sources` for valid parameters"""
            pass
    class EvaluationAvailableWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_evaluations` for valid parameters"""
            pass
    class MLModelAvailableWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_ml_models` for valid parameters"""
            pass


    class DescribeBatchPredictionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_batch_predictions` for valid parameters"""
            pass
    class DescribeDataSourcesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_data_sources` for valid parameters"""
            pass
    class DescribeEvaluationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_evaluations` for valid parameters"""
            pass
    class DescribeMLModelsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_ml_models` for valid parameters"""
            pass


    class AddTagsInput(Mapping):
        pass
    
    class AddTagsOutput(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreateBatchPredictionInput(Mapping):
        pass
    
    class CreateBatchPredictionOutput(Mapping):
        pass
    
    class CreateDataSourceFromRDSInput(Mapping):
        pass
    
    class CreateDataSourceFromRDSOutput(Mapping):
        pass
    
    class RDSDataSpec(Mapping):
        pass
    
    class ComputeStatistics(object):
        pass
    
    class CreateDataSourceFromRedshiftInput(Mapping):
        pass
    
    class CreateDataSourceFromRedshiftOutput(Mapping):
        pass
    
    class RedshiftDataSpec(Mapping):
        pass
    
    class CreateDataSourceFromS3Input(Mapping):
        pass
    
    class CreateDataSourceFromS3Output(Mapping):
        pass
    
    class S3DataSpec(Mapping):
        pass
    
    class CreateEvaluationInput(Mapping):
        pass
    
    class CreateEvaluationOutput(Mapping):
        pass
    
    class CreateMLModelInput(Mapping):
        pass
    
    class CreateMLModelOutput(Mapping):
        pass
    
    class TrainingParameters(object):
        pass
    
    class CreateRealtimeEndpointInput(Mapping):
        pass
    
    class CreateRealtimeEndpointOutput(Mapping):
        pass
    
    class DeleteBatchPredictionInput(Mapping):
        pass
    
    class DeleteBatchPredictionOutput(Mapping):
        pass
    
    class DeleteDataSourceInput(Mapping):
        pass
    
    class DeleteDataSourceOutput(Mapping):
        pass
    
    class DeleteEvaluationInput(Mapping):
        pass
    
    class DeleteEvaluationOutput(Mapping):
        pass
    
    class DeleteMLModelInput(Mapping):
        pass
    
    class DeleteMLModelOutput(Mapping):
        pass
    
    class DeleteRealtimeEndpointInput(Mapping):
        pass
    
    class DeleteRealtimeEndpointOutput(Mapping):
        pass
    
    class DeleteTagsInput(Mapping):
        pass
    
    class DeleteTagsOutput(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class DescribeBatchPredictionsInput(Mapping):
        pass
    
    class DescribeBatchPredictionsOutput(Mapping):
        pass
    
    class DescribeDataSourcesInput(Mapping):
        pass
    
    class DescribeDataSourcesOutput(Mapping):
        pass
    
    class DescribeEvaluationsInput(Mapping):
        pass
    
    class DescribeEvaluationsOutput(Mapping):
        pass
    
    class DescribeMLModelsInput(Mapping):
        pass
    
    class DescribeMLModelsOutput(Mapping):
        pass
    
    class DescribeTagsInput(Mapping):
        pass
    
    class DescribeTagsOutput(Mapping):
        pass
    
    class GetBatchPredictionInput(Mapping):
        pass
    
    class GetBatchPredictionOutput(Mapping):
        pass
    
    class GetDataSourceInput(Mapping):
        pass
    
    class GetDataSourceOutput(Mapping):
        pass
    
    class Verbose(object):
        pass
    
    class GetEvaluationInput(Mapping):
        pass
    
    class GetEvaluationOutput(Mapping):
        pass
    
    class GetMLModelInput(Mapping):
        pass
    
    class GetMLModelOutput(Mapping):
        pass
    
    class PredictInput(Mapping):
        pass
    
    class PredictOutput(Mapping):
        pass
    
    class Record(object):
        pass
    
    class UpdateBatchPredictionInput(Mapping):
        pass
    
    class UpdateBatchPredictionOutput(Mapping):
        pass
    
    class UpdateDataSourceInput(Mapping):
        pass
    
    class UpdateDataSourceOutput(Mapping):
        pass
    
    class UpdateEvaluationInput(Mapping):
        pass
    
    class UpdateEvaluationOutput(Mapping):
        pass
    
    class UpdateMLModelInput(Mapping):
        pass
    
    class UpdateMLModelOutput(Mapping):
        pass
    
    class ScoreThreshold(object):
        pass
    
    def add_tags(self, ResourceType: str, ResourceId: str, Tags: List[Tag]) -> AddTagsOutput:
        """Adds one or more tags to an object, up to a limit of 10. Each tag consists of a key and an optional value. If you add a tag using a key that is already associated with the ML object, <code>AddTags</code> updates the tag's value.

        :param self:
        :param ResourceType:  The type of the ML object to tag. <br/>ResourceId:  The ID of the ML object to tag. For example, <code>exampleModelId</code>.<br/>Tags:  The key-value pairs to use to create tags. If you specify a key without specifying a value, Amazon ML creates a tag with the specified key and a value of null.<br/>
        :return: ResourceType:  The type of the ML object that was tagged.<br/>ResourceId:  The ID of the ML object that was tagged.<br/>        """
        pass

    def create_batch_prediction(self, OutputUri: str, BatchPredictionDataSourceId: str, MLModelId: str, BatchPredictionId: str, BatchPredictionName: str=None) -> CreateBatchPredictionOutput:
        """Generates predictions for a group of observations. The observations to process exist in one or more data files referenced by a <code>DataSource</code>. This operation creates a new <code>BatchPrediction</code>, and uses an <code>MLModel</code> and the data files referenced by the <code>DataSource</code> as information sources.  <code>CreateBatchPrediction</code> is an asynchronous operation. In response to <code>CreateBatchPrediction</code>, Amazon Machine Learning (Amazon ML) immediately returns and sets the <code>BatchPrediction</code> status to <code>PENDING</code>. After the <code>BatchPrediction</code> completes, Amazon ML sets the status to <code>COMPLETED</code>.  You can poll for status updates by using the <a>GetBatchPrediction</a> operation and checking the <code>Status</code> parameter of the result. After the <code>COMPLETED</code> status appears, the results are available in the location specified by the <code>OutputUri</code> parameter.

        :param self:
        :param OutputUri:  The location of an Amazon Simple Storage Service (Amazon S3) bucket or directory to store the batch prediction results. The following substrings are not allowed in the <code>s3 key</code> portion of the <code>outputURI</code> field: ':', '//', '/./', '/../'. Amazon ML needs permissions to store and retrieve the logs on your behalf. For information about how to set permissions, see the <a href="http://docs.aws.amazon.com/machine-learning/latest/dg">Amazon Machine Learning Developer Guide</a>.<br/>BatchPredictionDataSourceId:  The ID of the <code>DataSource</code> that points to the group of observations to predict.<br/>MLModelId:  The ID of the <code>MLModel</code> that will generate predictions for the group of observations. <br/>BatchPredictionName:  A user-supplied name or description of the <code>BatchPrediction</code>. <code>BatchPredictionName</code> can only use the UTF-8 character set.<br/>BatchPredictionId:  A user-supplied ID that uniquely identifies the <code>BatchPrediction</code>.<br/>
        :return: BatchPredictionId:  A user-supplied ID that uniquely identifies the <code>BatchPrediction</code>. This value is identical to the value of the <code>BatchPredictionId</code> in the request.<br/>        """
        pass

    def create_data_source_from_rds(self, RoleARN: str, RDSData: RDSDataSpec, DataSourceId: str, DataSourceName: str=None, ComputeStatistics: ComputeStatistics=None) -> CreateDataSourceFromRDSOutput:
        """Creates a <code>DataSource</code> object from an <a href="http://aws.amazon.com/rds/"> Amazon Relational Database Service</a> (Amazon RDS). A <code>DataSource</code> references data that can be used to perform <code>CreateMLModel</code>, <code>CreateEvaluation</code>, or <code>CreateBatchPrediction</code> operations. <code>CreateDataSourceFromRDS</code> is an asynchronous operation. In response to <code>CreateDataSourceFromRDS</code>, Amazon Machine Learning (Amazon ML) immediately returns and sets the <code>DataSource</code> status to <code>PENDING</code>. After the <code>DataSource</code> is created and ready for use, Amazon ML sets the <code>Status</code> parameter to <code>COMPLETED</code>. <code>DataSource</code> in the <code>COMPLETED</code> or <code>PENDING</code> state can be used only to perform <code>&gt;CreateMLModel</code>&gt;, <code>CreateEvaluation</code>, or <code>CreateBatchPrediction</code> operations.   If Amazon ML cannot accept the input source, it sets the <code>Status</code> parameter to <code>FAILED</code> and includes an error message in the <code>Message</code> attribute of the <code>GetDataSource</code> operation response. 

        :param self:
        :param ComputeStatistics:  The compute statistics for a <code>DataSource</code>. The statistics are generated from the observation data referenced by a <code>DataSource</code>. Amazon ML uses the statistics internally during <code>MLModel</code> training. This parameter must be set to <code>true</code> if the <code></code>DataSource<code></code> needs to be used for <code>MLModel</code> training. <br/>RoleARN:  The role that Amazon ML assumes on behalf of the user to create and activate a data pipeline in the user's account and copy data using the <code>SelectSqlQuery</code> query from Amazon RDS to Amazon S3.  <br/>RDSData:  The data specification of an Amazon RDS <code>DataSource</code>: <ul> <li>DatabaseInformation - <ul> <li> <code>DatabaseName</code> - The name of the Amazon RDS database.</li> <li> <code>InstanceIdentifier </code> - A unique identifier for the Amazon RDS database instance.</li> </ul> </li> <li>DatabaseCredentials - AWS Identity and Access Management (IAM) credentials that are used to connect to the Amazon RDS database.</li> <li>ResourceRole - A role (DataPipelineDefaultResourceRole) assumed by an EC2 instance to carry out the copy task from Amazon RDS to Amazon Simple Storage Service (Amazon S3). For more information, see <a href="http://docs.aws.amazon.com/datapipeline/latest/DeveloperGuide/dp-iam-roles.html">Role templates</a> for data pipelines.</li> <li>ServiceRole - A role (DataPipelineDefaultRole) assumed by the AWS Data Pipeline service to monitor the progress of the copy task from Amazon RDS to Amazon S3. For more information, see <a href="http://docs.aws.amazon.com/datapipeline/latest/DeveloperGuide/dp-iam-roles.html">Role templates</a> for data pipelines.</li> <li>SecurityInfo - The security information to use to access an RDS DB instance. You need to set up appropriate ingress rules for the security entity IDs provided to allow access to the Amazon RDS instance. Specify a [<code>SubnetId</code>, <code>SecurityGroupIds</code>] pair for a VPC-based RDS DB instance.</li> <li>SelectSqlQuery - A query that is used to retrieve the observation data for the <code>Datasource</code>.</li> <li>S3StagingLocation - The Amazon S3 location for staging Amazon RDS data. The data retrieved from Amazon RDS using <code>SelectSqlQuery</code> is stored in this location.</li> <li>DataSchemaUri - The Amazon S3 location of the <code>DataSchema</code>.</li> <li>DataSchema - A JSON string representing the schema. This is not required if <code>DataSchemaUri</code> is specified. </li> <li> DataRearrangement - A JSON string that represents the splitting and rearrangement requirements for the <code>Datasource</code>.  <br>  Sample - <code> "{\"splitting\":{\"percentBegin\":10,\"percentEnd\":60}}"</code>  </li> </ul>:<br/>DataSourceName:  A user-supplied name or description of the <code>DataSource</code>.<br/>DataSourceId:  A user-supplied ID that uniquely identifies the <code>DataSource</code>. Typically, an Amazon Resource Number (ARN) becomes the ID for a <code>DataSource</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The security group IDs to be used to access a VPC-based RDS DB instance. Ensure that there are appropriate ingress rules set up to allow access to the RDS DB instance. This attribute is used by Data Pipeline to carry out the copy operation from Amazon RDS to an Amazon S3 task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetId</i> The subnet ID to be used to access a VPC-based RDS DB instance. This attribute is used by Data Pipeline to carry out the copy task from Amazon RDS to Amazon S3.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceRole</i> The role (DataPipelineDefaultRole) assumed by AWS Data Pipeline service to monitor the progress of the copy task from Amazon RDS to Amazon S3. For more information, see <a href="http://docs.aws.amazon.com/datapipeline/latest/DeveloperGuide/dp-iam-roles.html">Role templates</a> for data pipelines.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceRole</i> The role (DataPipelineDefaultResourceRole) assumed by an Amazon Elastic Compute Cloud (Amazon EC2) instance to carry out the copy operation from Amazon RDS to an Amazon S3 task. For more information, see <a href="http://docs.aws.amazon.com/datapipeline/latest/DeveloperGuide/dp-iam-roles.html">Role templates</a> for data pipelines.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataSchemaUri</i> The Amazon S3 location of the <code>DataSchema</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataSchema</i> A JSON string that represents the schema for an Amazon RDS <code>DataSource</code>. The <code>DataSchema</code> defines the structure of the observation data in the data file(s) referenced in the <code>DataSource</code>. A <code>DataSchema</code> is not required if you specify a <code>DataSchemaUri</code> Define your <code>DataSchema</code> as a series of key-value pairs. <code>attributes</code> and <code>excludedVariableNames</code> have an array of key-value pairs for their value. Use the following format to define your <code>DataSchema</code>. { "version": "1.0",  "recordAnnotationFieldName": "F1",  "recordWeightFieldName": "F2",  "targetFieldName": "F3",  "dataFormat": "CSV",  "dataFileContainsHeader": true,  "attributes": [  { "fieldName": "F1", "fieldType": "TEXT" }, { "fieldName": "F2", "fieldType": "NUMERIC" }, { "fieldName": "F3", "fieldType": "CATEGORICAL" }, { "fieldName": "F4", "fieldType": "NUMERIC" }, { "fieldName": "F5", "fieldType": "CATEGORICAL" }, { "fieldName": "F6", "fieldType": "TEXT" }, { "fieldName": "F7", "fieldType": "WEIGHTED_INT_SEQUENCE" }, { "fieldName": "F8", "fieldType": "WEIGHTED_STRING_SEQUENCE" } ],  "excludedVariableNames": [ "F6" ] }  <?oxy_insert_end><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataRearrangement</i> A JSON string that represents the splitting and rearrangement processing to be applied to a <code>DataSource</code>. If the <code>DataRearrangement</code> parameter is not provided, all of the input data is used to create the <code>Datasource</code>. There are multiple parameters that control what data is used to create a datasource: <ul> <li><b><code>percentBegin</code></b> Use <code>percentBegin</code> to indicate the beginning of the range of the data used to create the Datasource. If you do not include <code>percentBegin</code> and <code>percentEnd</code>, Amazon ML includes all of the data when creating the datasource.</li> <li><b><code>percentEnd</code></b> Use <code>percentEnd</code> to indicate the end of the range of the data used to create the Datasource. If you do not include <code>percentBegin</code> and <code>percentEnd</code>, Amazon ML includes all of the data when creating the datasource.</li> <li><b><code>complement</code></b> The <code>complement</code> parameter instructs Amazon ML to use the data that is not included in the range of <code>percentBegin</code> to <code>percentEnd</code> to create a datasource. The <code>complement</code> parameter is useful if you need to create complementary datasources for training and evaluation. To create a complementary datasource, use the same values for <code>percentBegin</code> and <code>percentEnd</code>, along with the <code>complement</code> parameter. For example, the following two datasources do not share any data, and can be used to train and evaluate a model. The first datasource has 25 percent of the data, and the second one has 75 percent of the data. Datasource for evaluation: <code>{"splitting":{"percentBegin":0, "percentEnd":25}}</code> Datasource for training: <code>{"splitting":{"percentBegin":0, "percentEnd":25, "complement":"true"}}</code> </li> <li><b><code>strategy</code></b> To change how Amazon ML splits the data for a datasource, use the <code>strategy</code> parameter. The default value for the <code>strategy</code> parameter is <code>sequential</code>, meaning that Amazon ML takes all of the data records between the <code>percentBegin</code> and <code>percentEnd</code> parameters for the datasource, in the order that the records appear in the input data. The following two <code>DataRearrangement</code> lines are examples of sequentially ordered training and evaluation datasources: Datasource for evaluation: <code>{"splitting":{"percentBegin":70, "percentEnd":100, "strategy":"sequential"}}</code> Datasource for training: <code>{"splitting":{"percentBegin":70, "percentEnd":100, "strategy":"sequential", "complement":"true"}}</code> To randomly split the input data into the proportions indicated by the percentBegin and percentEnd parameters, set the <code>strategy</code> parameter to <code>random</code> and provide a string that is used as the seed value for the random data splitting (for example, you can use the S3 path to your data as the random seed string). If you choose the random split strategy, Amazon ML assigns each row of data a pseudo-random number between 0 and 100, and then selects the rows that have an assigned number between <code>percentBegin</code> and <code>percentEnd</code>. Pseudo-random numbers are assigned using both the input seed string value and the byte offset as a seed, so changing the data results in a different split. Any existing ordering is preserved. The random splitting strategy ensures that variables in the training and evaluation data are distributed similarly. It is useful in the cases where the input data may have an implicit sort order, which would otherwise result in training and evaluation datasources containing non-similar data records. The following two <code>DataRearrangement</code> lines are examples of non-sequentially ordered training and evaluation datasources: Datasource for evaluation: <code>{"splitting":{"percentBegin":70, "percentEnd":100, "strategy":"random", "randomSeed"="s3://my_s3_path/bucket/file.csv"}}</code> Datasource for training: <code>{"splitting":{"percentBegin":70, "percentEnd":100, "strategy":"random", "randomSeed"="s3://my_s3_path/bucket/file.csv", "complement":"true"}}</code> </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3StagingLocation</i> The Amazon S3 location for staging Amazon RDS data. The data retrieved from Amazon RDS using <code>SelectSqlQuery</code> is stored in this location.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseCredentials</i> The AWS Identity and Access Management (IAM) credentials that are used connect to the Amazon RDS database.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DatabaseCredentials</b>: The AWS Identity and Access Management (IAM) credentials that are used connect to the Amazon RDS database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SelectSqlQuery</i> The query that is used to retrieve the observation data for the <code>DataSource</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseInformation</i> Describes the <code>DatabaseName</code> and <code>InstanceIdentifier</code> of an Amazon RDS database.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DatabaseInformation</b>: Describes the <code>DatabaseName</code> and <code>InstanceIdentifier</code> of an Amazon RDS database.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> The name of a database hosted on an RDS DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceIdentifier</i> The ID of an RDS DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Password</i> The password to be used by Amazon ML to connect to a database on an RDS DB instance. The password should have sufficient permissions to execute the <code>RDSSelectQuery</code> query.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> The username to be used by Amazon ML to connect to database on an Amazon RDS instance. The username should have sufficient permissions to execute an <code>RDSSelectSqlQuery</code> query.<br/>
        :return: DataSourceId:  A user-supplied ID that uniquely identifies the datasource. This value should be identical to the value of the <code>DataSourceID</code> in the request. <br/>        """
        pass

    def create_data_source_from_redshift(self, RoleARN: str, DataSpec: RedshiftDataSpec, DataSourceId: str, DataSourceName: str=None, ComputeStatistics: ComputeStatistics=None) -> CreateDataSourceFromRedshiftOutput:
        """Creates a <code>DataSource</code> from a database hosted on an Amazon Redshift cluster. A <code>DataSource</code> references data that can be used to perform either <code>CreateMLModel</code>, <code>CreateEvaluation</code>, or <code>CreateBatchPrediction</code> operations. <code>CreateDataSourceFromRedshift</code> is an asynchronous operation. In response to <code>CreateDataSourceFromRedshift</code>, Amazon Machine Learning (Amazon ML) immediately returns and sets the <code>DataSource</code> status to <code>PENDING</code>. After the <code>DataSource</code> is created and ready for use, Amazon ML sets the <code>Status</code> parameter to <code>COMPLETED</code>. <code>DataSource</code> in <code>COMPLETED</code> or <code>PENDING</code> states can be used to perform only <code>CreateMLModel</code>, <code>CreateEvaluation</code>, or <code>CreateBatchPrediction</code> operations.   If Amazon ML can't accept the input source, it sets the <code>Status</code> parameter to <code>FAILED</code> and includes an error message in the <code>Message</code> attribute of the <code>GetDataSource</code> operation response.  The observations should be contained in the database hosted on an Amazon Redshift cluster and should be specified by a <code>SelectSqlQuery</code> query. Amazon ML executes an <code>Unload</code> command in Amazon Redshift to transfer the result set of the <code>SelectSqlQuery</code> query to <code>S3StagingLocation</code>. After the <code>DataSource</code> has been created, it's ready for use in evaluations and batch predictions. If you plan to use the <code>DataSource</code> to train an <code>MLModel</code>, the <code>DataSource</code> also requires a recipe. A recipe describes how each input variable will be used in training an <code>MLModel</code>. Will the variable be included or excluded from training? Will the variable be manipulated; for example, will it be combined with another variable or will it be split apart into word combinations? The recipe provides answers to these questions. <?oxy_insert_start author="laurama" timestamp="20160406T153842-0700">You can't change an existing datasource, but you can copy and modify the settings from an existing Amazon Redshift datasource to create a new datasource. To do so, call <code>GetDataSource</code> for an existing datasource and copy the values to a <code>CreateDataSource</code> call. Change the settings that you want to change and make sure that all required fields have the appropriate values. <?oxy_insert_end>

        :param self:
        :param ComputeStatistics:  The compute statistics for a <code>DataSource</code>. The statistics are generated from the observation data referenced by a <code>DataSource</code>. Amazon ML uses the statistics internally during <code>MLModel</code> training. This parameter must be set to <code>true</code> if the <code>DataSource</code> needs to be used for <code>MLModel</code> training.<br/>RoleARN:  A fully specified role Amazon Resource Name (ARN). Amazon ML assumes the role on behalf of the user to create the following:   <ul> <li>A security group to allow Amazon ML to execute the <code>SelectSqlQuery</code> query on an Amazon Redshift cluster</li> <li>An Amazon S3 bucket policy to grant Amazon ML read/write permissions on the <code>S3StagingLocation</code></li> </ul> <br/>DataSpec:  The data specification of an Amazon Redshift <code>DataSource</code>: <ul> <li>DatabaseInformation - <ul> <li> <code>DatabaseName</code> - The name of the Amazon Redshift database. </li> <li> <code> ClusterIdentifier</code> - The unique ID for the Amazon Redshift cluster.</li> </ul></li> <li>DatabaseCredentials - The AWS Identity and Access Management (IAM) credentials that are used to connect to the Amazon Redshift database.</li> <li>SelectSqlQuery - The query that is used to retrieve the observation data for the <code>Datasource</code>.</li> <li>S3StagingLocation - The Amazon Simple Storage Service (Amazon S3) location for staging Amazon Redshift data. The data retrieved from Amazon Redshift using the <code>SelectSqlQuery</code> query is stored in this location.</li> <li>DataSchemaUri - The Amazon S3 location of the <code>DataSchema</code>.</li> <li>DataSchema - A JSON string representing the schema. This is not required if <code>DataSchemaUri</code> is specified. </li> <li> DataRearrangement - A JSON string that represents the splitting and rearrangement requirements for the <code>DataSource</code>.  Sample - <code> "{\"splitting\":{\"percentBegin\":10,\"percentEnd\":60}}"</code>  </li> </ul>:<br/>DataSourceName:  A user-supplied name or description of the <code>DataSource</code>. <br/>DataSourceId:  A user-supplied ID that uniquely identifies the <code>DataSource</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataSchemaUri</i> Describes the schema location for an Amazon Redshift <code>DataSource</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataSchema</i> A JSON string that represents the schema for an Amazon Redshift <code>DataSource</code>. The <code>DataSchema</code> defines the structure of the observation data in the data file(s) referenced in the <code>DataSource</code>. A <code>DataSchema</code> is not required if you specify a <code>DataSchemaUri</code>. Define your <code>DataSchema</code> as a series of key-value pairs. <code>attributes</code> and <code>excludedVariableNames</code> have an array of key-value pairs for their value. Use the following format to define your <code>DataSchema</code>. { "version": "1.0",  "recordAnnotationFieldName": "F1",  "recordWeightFieldName": "F2",  "targetFieldName": "F3",  "dataFormat": "CSV",  "dataFileContainsHeader": true,  "attributes": [  { "fieldName": "F1", "fieldType": "TEXT" }, { "fieldName": "F2", "fieldType": "NUMERIC" }, { "fieldName": "F3", "fieldType": "CATEGORICAL" }, { "fieldName": "F4", "fieldType": "NUMERIC" }, { "fieldName": "F5", "fieldType": "CATEGORICAL" }, { "fieldName": "F6", "fieldType": "TEXT" }, { "fieldName": "F7", "fieldType": "WEIGHTED_INT_SEQUENCE" }, { "fieldName": "F8", "fieldType": "WEIGHTED_STRING_SEQUENCE" } ],  "excludedVariableNames": [ "F6" ] } <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataRearrangement</i> A JSON string that represents the splitting and rearrangement processing to be applied to a <code>DataSource</code>. If the <code>DataRearrangement</code> parameter is not provided, all of the input data is used to create the <code>Datasource</code>. There are multiple parameters that control what data is used to create a datasource: <ul> <li><b><code>percentBegin</code></b> Use <code>percentBegin</code> to indicate the beginning of the range of the data used to create the Datasource. If you do not include <code>percentBegin</code> and <code>percentEnd</code>, Amazon ML includes all of the data when creating the datasource.</li> <li><b><code>percentEnd</code></b> Use <code>percentEnd</code> to indicate the end of the range of the data used to create the Datasource. If you do not include <code>percentBegin</code> and <code>percentEnd</code>, Amazon ML includes all of the data when creating the datasource.</li> <li><b><code>complement</code></b> The <code>complement</code> parameter instructs Amazon ML to use the data that is not included in the range of <code>percentBegin</code> to <code>percentEnd</code> to create a datasource. The <code>complement</code> parameter is useful if you need to create complementary datasources for training and evaluation. To create a complementary datasource, use the same values for <code>percentBegin</code> and <code>percentEnd</code>, along with the <code>complement</code> parameter. For example, the following two datasources do not share any data, and can be used to train and evaluate a model. The first datasource has 25 percent of the data, and the second one has 75 percent of the data. Datasource for evaluation: <code>{"splitting":{"percentBegin":0, "percentEnd":25}}</code> Datasource for training: <code>{"splitting":{"percentBegin":0, "percentEnd":25, "complement":"true"}}</code> </li> <li><b><code>strategy</code></b> To change how Amazon ML splits the data for a datasource, use the <code>strategy</code> parameter. The default value for the <code>strategy</code> parameter is <code>sequential</code>, meaning that Amazon ML takes all of the data records between the <code>percentBegin</code> and <code>percentEnd</code> parameters for the datasource, in the order that the records appear in the input data. The following two <code>DataRearrangement</code> lines are examples of sequentially ordered training and evaluation datasources: Datasource for evaluation: <code>{"splitting":{"percentBegin":70, "percentEnd":100, "strategy":"sequential"}}</code> Datasource for training: <code>{"splitting":{"percentBegin":70, "percentEnd":100, "strategy":"sequential", "complement":"true"}}</code> To randomly split the input data into the proportions indicated by the percentBegin and percentEnd parameters, set the <code>strategy</code> parameter to <code>random</code> and provide a string that is used as the seed value for the random data splitting (for example, you can use the S3 path to your data as the random seed string). If you choose the random split strategy, Amazon ML assigns each row of data a pseudo-random number between 0 and 100, and then selects the rows that have an assigned number between <code>percentBegin</code> and <code>percentEnd</code>. Pseudo-random numbers are assigned using both the input seed string value and the byte offset as a seed, so changing the data results in a different split. Any existing ordering is preserved. The random splitting strategy ensures that variables in the training and evaluation data are distributed similarly. It is useful in the cases where the input data may have an implicit sort order, which would otherwise result in training and evaluation datasources containing non-similar data records. The following two <code>DataRearrangement</code> lines are examples of non-sequentially ordered training and evaluation datasources: Datasource for evaluation: <code>{"splitting":{"percentBegin":70, "percentEnd":100, "strategy":"random", "randomSeed"="s3://my_s3_path/bucket/file.csv"}}</code> Datasource for training: <code>{"splitting":{"percentBegin":70, "percentEnd":100, "strategy":"random", "randomSeed"="s3://my_s3_path/bucket/file.csv", "complement":"true"}}</code> </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3StagingLocation</i> Describes an Amazon S3 location to store the result set of the <code>SelectSqlQuery</code> query.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseCredentials</i> Describes AWS Identity and Access Management (IAM) credentials that are used connect to the Amazon Redshift database.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DatabaseCredentials</b>: Describes AWS Identity and Access Management (IAM) credentials that are used connect to the Amazon Redshift database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SelectSqlQuery</i> Describes the SQL Query to execute on an Amazon Redshift database for an Amazon Redshift <code>DataSource</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseInformation</i> Describes the <code>DatabaseName</code> and <code>ClusterIdentifier</code> for an Amazon Redshift <code>DataSource</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DatabaseInformation</b>: Describes the <code>DatabaseName</code> and <code>ClusterIdentifier</code> for an Amazon Redshift <code>DataSource</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The ID of an Amazon Redshift cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> The name of a database hosted on an Amazon Redshift cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Password</i> A password to be used by Amazon ML to connect to a database on an Amazon Redshift cluster. The password should have sufficient permissions to execute a <code>RedshiftSelectSqlQuery</code> query. The password should be valid for an Amazon Redshift <a href="http://docs.aws.amazon.com/redshift/latest/dg/r_CREATE_USER.html">USER</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> A username to be used by Amazon Machine Learning (Amazon ML)to connect to a database on an Amazon Redshift cluster. The username should have sufficient permissions to execute the <code>RedshiftSelectSqlQuery</code> query. The username should be valid for an Amazon Redshift <a href="http://docs.aws.amazon.com/redshift/latest/dg/r_CREATE_USER.html">USER</a>.<br/>
        :return: DataSourceId:  A user-supplied ID that uniquely identifies the datasource. This value should be identical to the value of the <code>DataSourceID</code> in the request. <br/>        """
        pass

    def create_data_source_from_s3(self, DataSpec: S3DataSpec, DataSourceId: str, DataSourceName: str=None, ComputeStatistics: ComputeStatistics=None) -> CreateDataSourceFromS3Output:
        """Creates a <code>DataSource</code> object. A <code>DataSource</code> references data that can be used to perform <code>CreateMLModel</code>, <code>CreateEvaluation</code>, or <code>CreateBatchPrediction</code> operations. <code>CreateDataSourceFromS3</code> is an asynchronous operation. In response to <code>CreateDataSourceFromS3</code>, Amazon Machine Learning (Amazon ML) immediately returns and sets the <code>DataSource</code> status to <code>PENDING</code>. After the <code>DataSource</code> has been created and is ready for use, Amazon ML sets the <code>Status</code> parameter to <code>COMPLETED</code>. <code>DataSource</code> in the <code>COMPLETED</code> or <code>PENDING</code> state can be used to perform only <code>CreateMLModel</code>, <code>CreateEvaluation</code> or <code>CreateBatchPrediction</code> operations.   If Amazon ML can't accept the input source, it sets the <code>Status</code> parameter to <code>FAILED</code> and includes an error message in the <code>Message</code> attribute of the <code>GetDataSource</code> operation response.  The observation data used in a <code>DataSource</code> should be ready to use; that is, it should have a consistent structure, and missing data values should be kept to a minimum. The observation data must reside in one or more .csv files in an Amazon Simple Storage Service (Amazon S3) location, along with a schema that describes the data items by name and type. The same schema must be used for all of the data files referenced by the <code>DataSource</code>.  After the <code>DataSource</code> has been created, it's ready to use in evaluations and batch predictions. If you plan to use the <code>DataSource</code> to train an <code>MLModel</code>, the <code>DataSource</code> also needs a recipe. A recipe describes how each input variable will be used in training an <code>MLModel</code>. Will the variable be included or excluded from training? Will the variable be manipulated; for example, will it be combined with another variable or will it be split apart into word combinations? The recipe provides answers to these questions.

        :param self:
        :param ComputeStatistics:  The compute statistics for a <code>DataSource</code>. The statistics are generated from the observation data referenced by a <code>DataSource</code>. Amazon ML uses the statistics internally during <code>MLModel</code> training. This parameter must be set to <code>true</code> if the <code></code>DataSource<code></code> needs to be used for <code>MLModel</code> training.<br/>DataSpec:  The data specification of a <code>DataSource</code>: <ul> <li>DataLocationS3 - The Amazon S3 location of the observation data.</li> <li>DataSchemaLocationS3 - The Amazon S3 location of the <code>DataSchema</code>.</li> <li>DataSchema - A JSON string representing the schema. This is not required if <code>DataSchemaUri</code> is specified. </li> <li> DataRearrangement - A JSON string that represents the splitting and rearrangement requirements for the <code>Datasource</code>.   Sample - <code> "{\"splitting\":{\"percentBegin\":10,\"percentEnd\":60}}"</code>  </li> </ul>:<br/>DataSourceName:  A user-supplied name or description of the <code>DataSource</code>. <br/>DataSourceId:  A user-supplied identifier that uniquely identifies the <code>DataSource</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataSchemaLocationS3</i> Describes the schema location in Amazon S3. You must provide either the <code>DataSchema</code> or the <code>DataSchemaLocationS3</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataSchema</i>  A JSON string that represents the schema for an Amazon S3 <code>DataSource</code>. The <code>DataSchema</code> defines the structure of the observation data in the data file(s) referenced in the <code>DataSource</code>. You must provide either the <code>DataSchema</code> or the <code>DataSchemaLocationS3</code>. Define your <code>DataSchema</code> as a series of key-value pairs. <code>attributes</code> and <code>excludedVariableNames</code> have an array of key-value pairs for their value. Use the following format to define your <code>DataSchema</code>. { "version": "1.0",  "recordAnnotationFieldName": "F1",  "recordWeightFieldName": "F2",  "targetFieldName": "F3",  "dataFormat": "CSV",  "dataFileContainsHeader": true,  "attributes": [  { "fieldName": "F1", "fieldType": "TEXT" }, { "fieldName": "F2", "fieldType": "NUMERIC" }, { "fieldName": "F3", "fieldType": "CATEGORICAL" }, { "fieldName": "F4", "fieldType": "NUMERIC" }, { "fieldName": "F5", "fieldType": "CATEGORICAL" }, { "fieldName": "F6", "fieldType": "TEXT" }, { "fieldName": "F7", "fieldType": "WEIGHTED_INT_SEQUENCE" }, { "fieldName": "F8", "fieldType": "WEIGHTED_STRING_SEQUENCE" } ],  "excludedVariableNames": [ "F6" ] }  <?oxy_insert_end><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataRearrangement</i> A JSON string that represents the splitting and rearrangement processing to be applied to a <code>DataSource</code>. If the <code>DataRearrangement</code> parameter is not provided, all of the input data is used to create the <code>Datasource</code>. There are multiple parameters that control what data is used to create a datasource: <ul> <li><b><code>percentBegin</code></b> Use <code>percentBegin</code> to indicate the beginning of the range of the data used to create the Datasource. If you do not include <code>percentBegin</code> and <code>percentEnd</code>, Amazon ML includes all of the data when creating the datasource.</li> <li><b><code>percentEnd</code></b> Use <code>percentEnd</code> to indicate the end of the range of the data used to create the Datasource. If you do not include <code>percentBegin</code> and <code>percentEnd</code>, Amazon ML includes all of the data when creating the datasource.</li> <li><b><code>complement</code></b> The <code>complement</code> parameter instructs Amazon ML to use the data that is not included in the range of <code>percentBegin</code> to <code>percentEnd</code> to create a datasource. The <code>complement</code> parameter is useful if you need to create complementary datasources for training and evaluation. To create a complementary datasource, use the same values for <code>percentBegin</code> and <code>percentEnd</code>, along with the <code>complement</code> parameter. For example, the following two datasources do not share any data, and can be used to train and evaluate a model. The first datasource has 25 percent of the data, and the second one has 75 percent of the data. Datasource for evaluation: <code>{"splitting":{"percentBegin":0, "percentEnd":25}}</code> Datasource for training: <code>{"splitting":{"percentBegin":0, "percentEnd":25, "complement":"true"}}</code> </li> <li><b><code>strategy</code></b> To change how Amazon ML splits the data for a datasource, use the <code>strategy</code> parameter. The default value for the <code>strategy</code> parameter is <code>sequential</code>, meaning that Amazon ML takes all of the data records between the <code>percentBegin</code> and <code>percentEnd</code> parameters for the datasource, in the order that the records appear in the input data. The following two <code>DataRearrangement</code> lines are examples of sequentially ordered training and evaluation datasources: Datasource for evaluation: <code>{"splitting":{"percentBegin":70, "percentEnd":100, "strategy":"sequential"}}</code> Datasource for training: <code>{"splitting":{"percentBegin":70, "percentEnd":100, "strategy":"sequential", "complement":"true"}}</code> To randomly split the input data into the proportions indicated by the percentBegin and percentEnd parameters, set the <code>strategy</code> parameter to <code>random</code> and provide a string that is used as the seed value for the random data splitting (for example, you can use the S3 path to your data as the random seed string). If you choose the random split strategy, Amazon ML assigns each row of data a pseudo-random number between 0 and 100, and then selects the rows that have an assigned number between <code>percentBegin</code> and <code>percentEnd</code>. Pseudo-random numbers are assigned using both the input seed string value and the byte offset as a seed, so changing the data results in a different split. Any existing ordering is preserved. The random splitting strategy ensures that variables in the training and evaluation data are distributed similarly. It is useful in the cases where the input data may have an implicit sort order, which would otherwise result in training and evaluation datasources containing non-similar data records. The following two <code>DataRearrangement</code> lines are examples of non-sequentially ordered training and evaluation datasources: Datasource for evaluation: <code>{"splitting":{"percentBegin":70, "percentEnd":100, "strategy":"random", "randomSeed"="s3://my_s3_path/bucket/file.csv"}}</code> Datasource for training: <code>{"splitting":{"percentBegin":70, "percentEnd":100, "strategy":"random", "randomSeed"="s3://my_s3_path/bucket/file.csv", "complement":"true"}}</code> </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataLocationS3</i> The location of the data file(s) used by a <code>DataSource</code>. The URI specifies a data file or an Amazon Simple Storage Service (Amazon S3) directory or bucket containing data files.<br/>
        :return: DataSourceId:  A user-supplied ID that uniquely identifies the <code>DataSource</code>. This value should be identical to the value of the <code>DataSourceID</code> in the request. <br/>        """
        pass

    def create_evaluation(self, EvaluationDataSourceId: str, MLModelId: str, EvaluationId: str, EvaluationName: str=None) -> CreateEvaluationOutput:
        """Creates a new <code>Evaluation</code> of an <code>MLModel</code>. An <code>MLModel</code> is evaluated on a set of observations associated to a <code>DataSource</code>. Like a <code>DataSource</code> for an <code>MLModel</code>, the <code>DataSource</code> for an <code>Evaluation</code> contains values for the <code>Target Variable</code>. The <code>Evaluation</code> compares the predicted result for each observation to the actual outcome and provides a summary so that you know how effective the <code>MLModel</code> functions on the test data. Evaluation generates a relevant performance metric, such as BinaryAUC, RegressionRMSE or MulticlassAvgFScore based on the corresponding <code>MLModelType</code>: <code>BINARY</code>, <code>REGRESSION</code> or <code>MULTICLASS</code>.  <code>CreateEvaluation</code> is an asynchronous operation. In response to <code>CreateEvaluation</code>, Amazon Machine Learning (Amazon ML) immediately returns and sets the evaluation status to <code>PENDING</code>. After the <code>Evaluation</code> is created and ready for use, Amazon ML sets the status to <code>COMPLETED</code>.  You can use the <code>GetEvaluation</code> operation to check progress of the evaluation during the creation operation.

        :param self:
        :param EvaluationDataSourceId:  The ID of the <code>DataSource</code> for the evaluation. The schema of the <code>DataSource</code> must match the schema used to create the <code>MLModel</code>.<br/>MLModelId:  The ID of the <code>MLModel</code> to evaluate. The schema used in creating the <code>MLModel</code> must match the schema of the <code>DataSource</code> used in the <code>Evaluation</code>.<br/>EvaluationName:  A user-supplied name or description of the <code>Evaluation</code>.<br/>EvaluationId:  A user-supplied ID that uniquely identifies the <code>Evaluation</code>.<br/>
        :return: EvaluationId:  The user-supplied ID that uniquely identifies the <code>Evaluation</code>. This value should be identical to the value of the <code>EvaluationId</code> in the request.<br/>        """
        pass

    def create_ml_model(self, TrainingDataSourceId: str, MLModelType: str, MLModelId: str, MLModelName: str=None, Parameters: TrainingParameters=None, Recipe: str=None, RecipeUri: str=None) -> CreateMLModelOutput:
        """Creates a new <code>MLModel</code> using the <code>DataSource</code> and the recipe as information sources.  An <code>MLModel</code> is nearly immutable. Users can update only the <code>MLModelName</code> and the <code>ScoreThreshold</code> in an <code>MLModel</code> without creating a new <code>MLModel</code>.  <code>CreateMLModel</code> is an asynchronous operation. In response to <code>CreateMLModel</code>, Amazon Machine Learning (Amazon ML) immediately returns and sets the <code>MLModel</code> status to <code>PENDING</code>. After the <code>MLModel</code> has been created and ready is for use, Amazon ML sets the status to <code>COMPLETED</code>.  You can use the <code>GetMLModel</code> operation to check the progress of the <code>MLModel</code> during the creation operation.  <code>CreateMLModel</code> requires a <code>DataSource</code> with computed statistics, which can be created by setting <code>ComputeStatistics</code> to <code>true</code> in <code>CreateDataSourceFromRDS</code>, <code>CreateDataSourceFromS3</code>, or <code>CreateDataSourceFromRedshift</code> operations. 

        :param self:
        :param RecipeUri:  The Amazon Simple Storage Service (Amazon S3) location and file name that contains the <code>MLModel</code> recipe. You must specify either the recipe or its URI. If you don't specify a recipe or its URI, Amazon ML creates a default.<br/>Recipe:  The data recipe for creating the <code>MLModel</code>. You must specify either the recipe or its URI. If you don't specify a recipe or its URI, Amazon ML creates a default.<br/>TrainingDataSourceId:  The <code>DataSource</code> that points to the training data.<br/>Parameters:  A list of the training parameters in the <code>MLModel</code>. The list is implemented as a map of key-value pairs. The following is the current set of training parameters:  <ul> <li> <code>sgd.maxMLModelSizeInBytes</code> - The maximum allowed size of the model. Depending on the input data, the size of the model might affect its performance.  The value is an integer that ranges from <code>100000</code> to <code>2147483648</code>. The default value is <code>33554432</code>. </li> <li><code>sgd.maxPasses</code> - The number of times that the training process traverses the observations to build the <code>MLModel</code>. The value is an integer that ranges from <code>1</code> to <code>10000</code>. The default value is <code>10</code>.</li> <li> <code>sgd.shuffleType</code> - Whether Amazon ML shuffles the training data. Shuffling the data improves a model's ability to find the optimal solution for a variety of data types. The valid values are <code>auto</code> and <code>none</code>. The default value is <code>none</code>. We <?oxy_insert_start author="laurama" timestamp="20160329T131121-0700">strongly recommend that you shuffle your data.<?oxy_insert_end> </li> <li> <code>sgd.l1RegularizationAmount</code> - The coefficient regularization L1 norm. It controls overfitting the data by penalizing large coefficients. This tends to drive coefficients to zero, resulting in a sparse feature set. If you use this parameter, start by specifying a small value, such as <code>1.0E-08</code>. The value is a double that ranges from <code>0</code> to <code>MAX_DOUBLE</code>. The default is to not use L1 normalization. This parameter can't be used when <code>L2</code> is specified. Use this parameter sparingly. </li> <li> <code>sgd.l2RegularizationAmount</code> - The coefficient regularization L2 norm. It controls overfitting the data by penalizing large coefficients. This tends to drive coefficients to small, nonzero values. If you use this parameter, start by specifying a small value, such as <code>1.0E-08</code>. The value is a double that ranges from <code>0</code> to <code>MAX_DOUBLE</code>. The default is to not use L2 normalization. This parameter can't be used when <code>L1</code> is specified. Use this parameter sparingly. </li> </ul><br/>MLModelType:  The category of supervised learning that this <code>MLModel</code> will address. Choose from the following types: <ul> <li>Choose <code>REGRESSION</code> if the <code>MLModel</code> will be used to predict a numeric value.</li> <li>Choose <code>BINARY</code> if the <code>MLModel</code> result has two possible values.</li> <li>Choose <code>MULTICLASS</code> if the <code>MLModel</code> result has a limited number of values. </li> </ul>  For more information, see the <a href="http://docs.aws.amazon.com/machine-learning/latest/dg">Amazon Machine Learning Developer Guide</a>.<br/>MLModelName:  A user-supplied name or description of the <code>MLModel</code>.<br/>MLModelId:  A user-supplied ID that uniquely identifies the <code>MLModel</code>.<br/>
        :return: MLModelId:  A user-supplied ID that uniquely identifies the <code>MLModel</code>. This value should be identical to the value of the <code>MLModelId</code> in the request. <br/>        """
        pass

    def create_realtime_endpoint(self, MLModelId: str) -> CreateRealtimeEndpointOutput:
        """Creates a real-time endpoint for the <code>MLModel</code>. The endpoint contains the URI of the <code>MLModel</code>; that is, the location to send real-time prediction requests for the specified <code>MLModel</code>.

        :param self:
        :param MLModelId:  The ID assigned to the <code>MLModel</code> during creation.<br/>
        :return: RealtimeEndpointInfo:  The endpoint information of the <code>MLModel</code> :<br/>MLModelId:  A user-supplied ID that uniquely identifies the <code>MLModel</code>. This value should be identical to the value of the <code>MLModelId</code> in the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointStatus</i>  The current status of the real-time endpoint for the <code>MLModel</code>. This element can have one of the following values:  <ul> <li> <code>NONE</code> - Endpoint does not exist or was previously deleted.</li> <li> <code>READY</code> - Endpoint is ready to be used for real-time predictions.</li> <li> <code>UPDATING</code> - Updating/creating the endpoint. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointUrl</i> The URI that specifies where to send real-time prediction requests for the <code>MLModel</code>. <note><title>Note</title> The application must wait until the real-time endpoint is ready before using this URI. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i> The time that the request to create the real-time endpoint for the <code>MLModel</code> was received. The time is expressed in epoch time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PeakRequestsPerSecond</i>  The maximum processing rate for the real-time endpoint for <code>MLModel</code>, measured in incoming requests per second.<br/>        """
        pass

    def delete_batch_prediction(self, BatchPredictionId: str) -> DeleteBatchPredictionOutput:
        """Assigns the DELETED status to a <code>BatchPrediction</code>, rendering it unusable. After using the <code>DeleteBatchPrediction</code> operation, you can use the <a>GetBatchPrediction</a> operation to verify that the status of the <code>BatchPrediction</code> changed to DELETED. <b>Caution:</b> The result of the <code>DeleteBatchPrediction</code> operation is irreversible.

        :param self:
        :param BatchPredictionId:  A user-supplied ID that uniquely identifies the <code>BatchPrediction</code>.<br/>
        :return: BatchPredictionId:  A user-supplied ID that uniquely identifies the <code>BatchPrediction</code>. This value should be identical to the value of the <code>BatchPredictionID</code> in the request.<br/>        """
        pass

    def delete_data_source(self, DataSourceId: str) -> DeleteDataSourceOutput:
        """Assigns the DELETED status to a <code>DataSource</code>, rendering it unusable. After using the <code>DeleteDataSource</code> operation, you can use the <a>GetDataSource</a> operation to verify that the status of the <code>DataSource</code> changed to DELETED. <b>Caution:</b> The results of the <code>DeleteDataSource</code> operation are irreversible.

        :param self:
        :param DataSourceId:  A user-supplied ID that uniquely identifies the <code>DataSource</code>.<br/>
        :return: DataSourceId:  A user-supplied ID that uniquely identifies the <code>DataSource</code>. This value should be identical to the value of the <code>DataSourceID</code> in the request.<br/>        """
        pass

    def delete_evaluation(self, EvaluationId: str) -> DeleteEvaluationOutput:
        """Assigns the <code>DELETED</code> status to an <code>Evaluation</code>, rendering it unusable. After invoking the <code>DeleteEvaluation</code> operation, you can use the <code>GetEvaluation</code> operation to verify that the status of the <code>Evaluation</code> changed to <code>DELETED</code>. <caution><title>Caution</title> The results of the <code>DeleteEvaluation</code> operation are irreversible.</caution>

        :param self:
        :param EvaluationId:  A user-supplied ID that uniquely identifies the <code>Evaluation</code> to delete.<br/>
        :return: EvaluationId:  A user-supplied ID that uniquely identifies the <code>Evaluation</code>. This value should be identical to the value of the <code>EvaluationId</code> in the request.<br/>        """
        pass

    def delete_ml_model(self, MLModelId: str) -> DeleteMLModelOutput:
        """Assigns the <code>DELETED</code> status to an <code>MLModel</code>, rendering it unusable. After using the <code>DeleteMLModel</code> operation, you can use the <code>GetMLModel</code> operation to verify that the status of the <code>MLModel</code> changed to DELETED. <b>Caution:</b> The result of the <code>DeleteMLModel</code> operation is irreversible.

        :param self:
        :param MLModelId:  A user-supplied ID that uniquely identifies the <code>MLModel</code>.<br/>
        :return: MLModelId:  A user-supplied ID that uniquely identifies the <code>MLModel</code>. This value should be identical to the value of the <code>MLModelID</code> in the request.<br/>        """
        pass

    def delete_realtime_endpoint(self, MLModelId: str) -> DeleteRealtimeEndpointOutput:
        """Deletes a real time endpoint of an <code>MLModel</code>.

        :param self:
        :param MLModelId:  The ID assigned to the <code>MLModel</code> during creation.<br/>
        :return: RealtimeEndpointInfo:  The endpoint information of the <code>MLModel</code> :<br/>MLModelId:  A user-supplied ID that uniquely identifies the <code>MLModel</code>. This value should be identical to the value of the <code>MLModelId</code> in the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointStatus</i>  The current status of the real-time endpoint for the <code>MLModel</code>. This element can have one of the following values:  <ul> <li> <code>NONE</code> - Endpoint does not exist or was previously deleted.</li> <li> <code>READY</code> - Endpoint is ready to be used for real-time predictions.</li> <li> <code>UPDATING</code> - Updating/creating the endpoint. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointUrl</i> The URI that specifies where to send real-time prediction requests for the <code>MLModel</code>. <note><title>Note</title> The application must wait until the real-time endpoint is ready before using this URI. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i> The time that the request to create the real-time endpoint for the <code>MLModel</code> was received. The time is expressed in epoch time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PeakRequestsPerSecond</i>  The maximum processing rate for the real-time endpoint for <code>MLModel</code>, measured in incoming requests per second.<br/>        """
        pass

    def delete_tags(self, ResourceType: str, ResourceId: str, TagKeys: List[TagKey]) -> DeleteTagsOutput:
        """Deletes the specified tags associated with an ML object. After this operation is complete, you can't recover deleted tags. If you specify a tag that doesn't exist, Amazon ML ignores it.

        :param self:
        :param ResourceType:  The type of the tagged ML object.<br/>ResourceId:  The ID of the tagged ML object. For example, <code>exampleModelId</code>.<br/>TagKeys:  One or more tags to delete.<br/>
        :return: ResourceType:  The type of the ML object from which tags were deleted.<br/>ResourceId:  The ID of the ML object from which tags were deleted.<br/>        """
        pass

    def describe_batch_predictions(self, FilterVariable: str=None, EQ: str=None, GT: str=None, LT: str=None, GE: str=None, LE: str=None, NE: str=None, Prefix: str=None, SortOrder: str=None, NextToken: str=None, Limit: int=None) -> DescribeBatchPredictionsOutput:
        """Returns a list of <code>BatchPrediction</code> operations that match the search criteria in the request.

        :param self:
        :param Limit:  The number of pages of information to include in the result. The range of acceptable values is <code>1</code> through <code>100</code>. The default value is <code>100</code>.<br/>NextToken:  An ID of the page in the paginated results.<br/>SortOrder:  A two-value parameter that determines the sequence of the resulting list of <code>MLModel</code>s. <ul> <li> <code>asc</code> - Arranges the list in ascending order (A-Z, 0-9).</li> <li> <code>dsc</code> - Arranges the list in descending order (Z-A, 9-0).</li> </ul> Results are sorted by <code>FilterVariable</code>.<br/>Prefix:  A string that is found at the beginning of a variable, such as <code>Name</code> or <code>Id</code>. For example, a <code>Batch Prediction</code> operation could have the <code>Name</code> <code>2014-09-09-HolidayGiftMailer</code>. To search for this <code>BatchPrediction</code>, select <code>Name</code> for the <code>FilterVariable</code> and any of the following strings for the <code>Prefix</code>:  <ul> <li>2014-09</li> <li>2014-09-09</li> <li>2014-09-09-Holiday</li> </ul><br/>NE:  The not equal to operator. The <code>BatchPrediction</code> results will have <code>FilterVariable</code> values not equal to the value specified with <code>NE</code>.<br/>LE:  The less than or equal to operator. The <code>BatchPrediction</code> results will have <code>FilterVariable</code> values that are less than or equal to the value specified with <code>LE</code>.<br/>GE:  The greater than or equal to operator. The <code>BatchPrediction</code> results will have <code>FilterVariable</code> values that are greater than or equal to the value specified with <code>GE</code>. <br/>LT:  The less than operator. The <code>BatchPrediction</code> results will have <code>FilterVariable</code> values that are less than the value specified with <code>LT</code>.<br/>GT:  The greater than operator. The <code>BatchPrediction</code> results will have <code>FilterVariable</code> values that are greater than the value specified with <code>GT</code>.<br/>EQ:  The equal to operator. The <code>BatchPrediction</code> results will have <code>FilterVariable</code> values that exactly match the value specified with <code>EQ</code>.<br/>FilterVariable:  Use one of the following variables to filter a list of <code>BatchPrediction</code>: <ul> <li> <code>CreatedAt</code> - Sets the search criteria to the <code>BatchPrediction</code> creation date.</li> <li> <code>Status</code> - Sets the search criteria to the <code>BatchPrediction</code> status.</li> <li> <code>Name</code> - Sets the search criteria to the contents of the <code>BatchPrediction</code><b> </b> <code>Name</code>.</li> <li> <code>IAMUser</code> - Sets the search criteria to the user account that invoked the <code>BatchPrediction</code> creation.</li> <li> <code>MLModelId</code> - Sets the search criteria to the <code>MLModel</code> used in the <code>BatchPrediction</code>.</li> <li> <code>DataSourceId</code> - Sets the search criteria to the <code>DataSource</code> used in the <code>BatchPrediction</code>.</li> <li> <code>DataURI</code> - Sets the search criteria to the data file(s) used in the <code>BatchPrediction</code>. The URL can identify either a file or an Amazon Simple Storage Solution (Amazon S3) bucket or directory.</li> </ul><br/>
        :return: NextToken:  The ID of the next page in the paginated results that indicates at least one more page follows.<br/>Results:  A list of <code>BatchPrediction</code> objects that meet the search criteria. <br/>        """
        pass

    def describe_data_sources(self, FilterVariable: str=None, EQ: str=None, GT: str=None, LT: str=None, GE: str=None, LE: str=None, NE: str=None, Prefix: str=None, SortOrder: str=None, NextToken: str=None, Limit: int=None) -> DescribeDataSourcesOutput:
        """Returns a list of <code>DataSource</code> that match the search criteria in the request.

        :param self:
        :param Limit:   The maximum number of <code>DataSource</code> to include in the result.<br/>NextToken:  The ID of the page in the paginated results.<br/>SortOrder:  A two-value parameter that determines the sequence of the resulting list of <code>DataSource</code>. <ul> <li> <code>asc</code> - Arranges the list in ascending order (A-Z, 0-9).</li> <li> <code>dsc</code> - Arranges the list in descending order (Z-A, 9-0).</li> </ul> Results are sorted by <code>FilterVariable</code>.<br/>Prefix:  A string that is found at the beginning of a variable, such as <code>Name</code> or <code>Id</code>. For example, a <code>DataSource</code> could have the <code>Name</code> <code>2014-09-09-HolidayGiftMailer</code>. To search for this <code>DataSource</code>, select <code>Name</code> for the <code>FilterVariable</code> and any of the following strings for the <code>Prefix</code>:  <ul> <li>2014-09</li> <li>2014-09-09</li> <li>2014-09-09-Holiday</li> </ul><br/>NE:  The not equal to operator. The <code>DataSource</code> results will have <code>FilterVariable</code> values not equal to the value specified with <code>NE</code>.<br/>LE:  The less than or equal to operator. The <code>DataSource</code> results will have <code>FilterVariable</code> values that are less than or equal to the value specified with <code>LE</code>.<br/>GE:  The greater than or equal to operator. The <code>DataSource</code> results will have <code>FilterVariable</code> values that are greater than or equal to the value specified with <code>GE</code>. <br/>LT:  The less than operator. The <code>DataSource</code> results will have <code>FilterVariable</code> values that are less than the value specified with <code>LT</code>.<br/>GT:  The greater than operator. The <code>DataSource</code> results will have <code>FilterVariable</code> values that are greater than the value specified with <code>GT</code>.<br/>EQ:  The equal to operator. The <code>DataSource</code> results will have <code>FilterVariable</code> values that exactly match the value specified with <code>EQ</code>.<br/>FilterVariable:  Use one of the following variables to filter a list of <code>DataSource</code>: <ul> <li> <code>CreatedAt</code> - Sets the search criteria to <code>DataSource</code> creation dates.</li> <li> <code>Status</code> - Sets the search criteria to <code>DataSource</code> statuses.</li> <li> <code>Name</code> - Sets the search criteria to the contents of <code>DataSource</code> <b> </b> <code>Name</code>.</li> <li> <code>DataUri</code> - Sets the search criteria to the URI of data files used to create the <code>DataSource</code>. The URI can identify either a file or an Amazon Simple Storage Service (Amazon S3) bucket or directory.</li> <li> <code>IAMUser</code> - Sets the search criteria to the user account that invoked the <code>DataSource</code> creation.</li> </ul><br/>
        :return: NextToken:  An ID of the next page in the paginated results that indicates at least one more page follows.<br/>Results:  A list of <code>DataSource</code> that meet the search criteria. <br/>        """
        pass

    def describe_evaluations(self, FilterVariable: str=None, EQ: str=None, GT: str=None, LT: str=None, GE: str=None, LE: str=None, NE: str=None, Prefix: str=None, SortOrder: str=None, NextToken: str=None, Limit: int=None) -> DescribeEvaluationsOutput:
        """Returns a list of <code>DescribeEvaluations</code> that match the search criteria in the request.

        :param self:
        :param Limit:   The maximum number of <code>Evaluation</code> to include in the result.<br/>NextToken:  The ID of the page in the paginated results.<br/>SortOrder:  A two-value parameter that determines the sequence of the resulting list of <code>Evaluation</code>. <ul> <li> <code>asc</code> - Arranges the list in ascending order (A-Z, 0-9).</li> <li> <code>dsc</code> - Arranges the list in descending order (Z-A, 9-0).</li> </ul> Results are sorted by <code>FilterVariable</code>.<br/>Prefix:  A string that is found at the beginning of a variable, such as <code>Name</code> or <code>Id</code>. For example, an <code>Evaluation</code> could have the <code>Name</code> <code>2014-09-09-HolidayGiftMailer</code>. To search for this <code>Evaluation</code>, select <code>Name</code> for the <code>FilterVariable</code> and any of the following strings for the <code>Prefix</code>:  <ul> <li>2014-09</li> <li>2014-09-09</li> <li>2014-09-09-Holiday</li> </ul><br/>NE:  The not equal to operator. The <code>Evaluation</code> results will have <code>FilterVariable</code> values not equal to the value specified with <code>NE</code>.<br/>LE:  The less than or equal to operator. The <code>Evaluation</code> results will have <code>FilterVariable</code> values that are less than or equal to the value specified with <code>LE</code>.<br/>GE:  The greater than or equal to operator. The <code>Evaluation</code> results will have <code>FilterVariable</code> values that are greater than or equal to the value specified with <code>GE</code>. <br/>LT:  The less than operator. The <code>Evaluation</code> results will have <code>FilterVariable</code> values that are less than the value specified with <code>LT</code>.<br/>GT:  The greater than operator. The <code>Evaluation</code> results will have <code>FilterVariable</code> values that are greater than the value specified with <code>GT</code>.<br/>EQ:  The equal to operator. The <code>Evaluation</code> results will have <code>FilterVariable</code> values that exactly match the value specified with <code>EQ</code>.<br/>FilterVariable:  Use one of the following variable to filter a list of <code>Evaluation</code> objects: <ul> <li> <code>CreatedAt</code> - Sets the search criteria to the <code>Evaluation</code> creation date.</li> <li> <code>Status</code> - Sets the search criteria to the <code>Evaluation</code> status.</li> <li> <code>Name</code> - Sets the search criteria to the contents of <code>Evaluation</code> <b> </b> <code>Name</code>.</li> <li> <code>IAMUser</code> - Sets the search criteria to the user account that invoked an <code>Evaluation</code>.</li> <li> <code>MLModelId</code> - Sets the search criteria to the <code>MLModel</code> that was evaluated.</li> <li> <code>DataSourceId</code> - Sets the search criteria to the <code>DataSource</code> used in <code>Evaluation</code>.</li> <li> <code>DataUri</code> - Sets the search criteria to the data file(s) used in <code>Evaluation</code>. The URL can identify either a file or an Amazon Simple Storage Solution (Amazon S3) bucket or directory.</li> </ul><br/>
        :return: NextToken:  The ID of the next page in the paginated results that indicates at least one more page follows.<br/>Results:  A list of <code>Evaluation</code> that meet the search criteria. <br/>        """
        pass

    def describe_ml_models(self, FilterVariable: str=None, EQ: str=None, GT: str=None, LT: str=None, GE: str=None, LE: str=None, NE: str=None, Prefix: str=None, SortOrder: str=None, NextToken: str=None, Limit: int=None) -> DescribeMLModelsOutput:
        """Returns a list of <code>MLModel</code> that match the search criteria in the request.

        :param self:
        :param Limit:  The number of pages of information to include in the result. The range of acceptable values is <code>1</code> through <code>100</code>. The default value is <code>100</code>.<br/>NextToken:  The ID of the page in the paginated results.<br/>SortOrder:  A two-value parameter that determines the sequence of the resulting list of <code>MLModel</code>. <ul> <li> <code>asc</code> - Arranges the list in ascending order (A-Z, 0-9).</li> <li> <code>dsc</code> - Arranges the list in descending order (Z-A, 9-0).</li> </ul> Results are sorted by <code>FilterVariable</code>.<br/>Prefix:  A string that is found at the beginning of a variable, such as <code>Name</code> or <code>Id</code>. For example, an <code>MLModel</code> could have the <code>Name</code> <code>2014-09-09-HolidayGiftMailer</code>. To search for this <code>MLModel</code>, select <code>Name</code> for the <code>FilterVariable</code> and any of the following strings for the <code>Prefix</code>:  <ul> <li>2014-09</li> <li>2014-09-09</li> <li>2014-09-09-Holiday</li> </ul><br/>NE:  The not equal to operator. The <code>MLModel</code> results will have <code>FilterVariable</code> values not equal to the value specified with <code>NE</code>.<br/>LE:  The less than or equal to operator. The <code>MLModel</code> results will have <code>FilterVariable</code> values that are less than or equal to the value specified with <code>LE</code>.<br/>GE:  The greater than or equal to operator. The <code>MLModel</code> results will have <code>FilterVariable</code> values that are greater than or equal to the value specified with <code>GE</code>. <br/>LT:  The less than operator. The <code>MLModel</code> results will have <code>FilterVariable</code> values that are less than the value specified with <code>LT</code>.<br/>GT:  The greater than operator. The <code>MLModel</code> results will have <code>FilterVariable</code> values that are greater than the value specified with <code>GT</code>.<br/>EQ:  The equal to operator. The <code>MLModel</code> results will have <code>FilterVariable</code> values that exactly match the value specified with <code>EQ</code>.<br/>FilterVariable:  Use one of the following variables to filter a list of <code>MLModel</code>: <ul> <li> <code>CreatedAt</code> - Sets the search criteria to <code>MLModel</code> creation date.</li> <li> <code>Status</code> - Sets the search criteria to <code>MLModel</code> status.</li> <li> <code>Name</code> - Sets the search criteria to the contents of <code>MLModel</code><b> </b> <code>Name</code>.</li> <li> <code>IAMUser</code> - Sets the search criteria to the user account that invoked the <code>MLModel</code> creation.</li> <li> <code>TrainingDataSourceId</code> - Sets the search criteria to the <code>DataSource</code> used to train one or more <code>MLModel</code>.</li> <li> <code>RealtimeEndpointStatus</code> - Sets the search criteria to the <code>MLModel</code> real-time endpoint status.</li> <li> <code>MLModelType</code> - Sets the search criteria to <code>MLModel</code> type: binary, regression, or multi-class.</li> <li> <code>Algorithm</code> - Sets the search criteria to the algorithm that the <code>MLModel</code> uses.</li> <li> <code>TrainingDataURI</code> - Sets the search criteria to the data file(s) used in training a <code>MLModel</code>. The URL can identify either a file or an Amazon Simple Storage Service (Amazon S3) bucket or directory.</li> </ul><br/>
        :return: NextToken:  The ID of the next page in the paginated results that indicates at least one more page follows.<br/>Results:  A list of <code>MLModel</code> that meet the search criteria.<br/>        """
        pass

    def describe_tags(self, ResourceType: str, ResourceId: str) -> DescribeTagsOutput:
        """Describes one or more of the tags for your Amazon ML object.

        :param self:
        :param ResourceType:  The type of the ML object.<br/>ResourceId:  The ID of the ML object. For example, <code>exampleModelId</code>. <br/>
        :return: Tags:  A list of tags associated with the ML object.<br/>ResourceType:  The type of the tagged ML object.<br/>ResourceId:  The ID of the tagged ML object.<br/>        """
        pass

    def get_batch_prediction(self, BatchPredictionId: str) -> GetBatchPredictionOutput:
        """Returns a <code>BatchPrediction</code> that includes detailed metadata, status, and data file information for a <code>Batch Prediction</code> request.

        :param self:
        :param BatchPredictionId:  An ID assigned to the <code>BatchPrediction</code> at creation.<br/>
        :return: InvalidRecordCount:  The number of invalid records that Amazon Machine Learning saw while processing the <code>BatchPrediction</code>.<br/>TotalRecordCount:  The number of total records that Amazon Machine Learning saw while processing the <code>BatchPrediction</code>.<br/>StartedAt:  The epoch time when Amazon Machine Learning marked the <code>BatchPrediction</code> as <code>INPROGRESS</code>. <code>StartedAt</code> isn't available if the <code>BatchPrediction</code> is in the <code>PENDING</code> state.<br/>FinishedAt:  The epoch time when Amazon Machine Learning marked the <code>BatchPrediction</code> as <code>COMPLETED</code> or <code>FAILED</code>. <code>FinishedAt</code> is only available when the <code>BatchPrediction</code> is in the <code>COMPLETED</code> or <code>FAILED</code> state.<br/>ComputeTime:  The approximate CPU time in milliseconds that Amazon Machine Learning spent processing the <code>BatchPrediction</code>, normalized and scaled on computation resources. <code>ComputeTime</code> is only available if the <code>BatchPrediction</code> is in the <code>COMPLETED</code> state.<br/>Message:  A description of the most recent details about processing the batch prediction request.<br/>LogUri:  A link to the file that contains logs of the <code>CreateBatchPrediction</code> operation.<br/>OutputUri:  The location of an Amazon S3 bucket or directory to receive the operation results.<br/>Status:  The status of the <code>BatchPrediction</code>, which can be one of the following values: <ul> <li> <code>PENDING</code> - Amazon Machine Learning (Amazon ML) submitted a request to generate batch predictions.</li> <li> <code>INPROGRESS</code> - The batch predictions are in progress.</li> <li> <code>FAILED</code> - The request to perform a batch prediction did not run to completion. It is not usable.</li> <li> <code>COMPLETED</code> - The batch prediction process completed successfully.</li> <li> <code>DELETED</code> - The <code>BatchPrediction</code> is marked as deleted. It is not usable.</li> </ul><br/>Name:  A user-supplied name or description of the <code>BatchPrediction</code>.<br/>LastUpdatedAt:  The time of the most recent edit to <code>BatchPrediction</code>. The time is expressed in epoch time.<br/>CreatedAt:  The time when the <code>BatchPrediction</code> was created. The time is expressed in epoch time.<br/>CreatedByIamUser:  The AWS user account that invoked the <code>BatchPrediction</code>. The account type can be either an AWS root account or an AWS Identity and Access Management (IAM) user account.<br/>InputDataLocationS3:  The location of the data file or directory in Amazon Simple Storage Service (Amazon S3).<br/>BatchPredictionDataSourceId:  The ID of the <code>DataSource</code> that was used to create the <code>BatchPrediction</code>. <br/>MLModelId:  The ID of the <code>MLModel</code> that generated predictions for the <code>BatchPrediction</code> request.<br/>BatchPredictionId:  An ID assigned to the <code>BatchPrediction</code> at creation. This value should be identical to the value of the <code>BatchPredictionID</code> in the request.<br/>        """
        pass

    def get_data_source(self, DataSourceId: str, Verbose: Verbose=None) -> GetDataSourceOutput:
        """Returns a <code>DataSource</code> that includes metadata and data file information, as well as the current status of the <code>DataSource</code>. <code>GetDataSource</code> provides results in normal or verbose format. The verbose format adds the schema description and the list of files pointed to by the DataSource to the normal format.

        :param self:
        :param Verbose:  Specifies whether the <code>GetDataSource</code> operation should return <code>DataSourceSchema</code>. If true, <code>DataSourceSchema</code> is returned. If false, <code>DataSourceSchema</code> is not returned.<br/>DataSourceId:  The ID assigned to the <code>DataSource</code> at creation.<br/>
        :return: DataSourceSchema:  The schema used by all of the data files of this <code>DataSource</code>. <note><title>Note</title> This parameter is provided as part of the verbose format.</note><br/>StartedAt:  The epoch time when Amazon Machine Learning marked the <code>DataSource</code> as <code>INPROGRESS</code>. <code>StartedAt</code> isn't available if the <code>DataSource</code> is in the <code>PENDING</code> state.<br/>FinishedAt:  The epoch time when Amazon Machine Learning marked the <code>DataSource</code> as <code>COMPLETED</code> or <code>FAILED</code>. <code>FinishedAt</code> is only available when the <code>DataSource</code> is in the <code>COMPLETED</code> or <code>FAILED</code> state.<br/>ComputeTime:  The approximate CPU time in milliseconds that Amazon Machine Learning spent processing the <code>DataSource</code>, normalized and scaled on computation resources. <code>ComputeTime</code> is only available if the <code>DataSource</code> is in the <code>COMPLETED</code> state and the <code>ComputeStatistics</code> is set to true.<br/>ComputeStatistics:   The parameter is <code>true</code> if statistics need to be generated from the observation data. <br/>RoleARN:  The Amazon Resource Name (ARN) of an <a href="http://docs.aws.amazon.com/IAM/latest/UserGuide/roles-toplevel.html#roles-about-termsandconcepts">AWS IAM Role</a>, such as the following: arn:aws:iam::account:role/rolename. <br/>RDSMetadata:  The datasource details that are specific to Amazon RDS.:<br/>RedshiftMetadata:  Describes the <code>DataSource</code> details specific to Amazon Redshift.:<br/>Message:  The user-supplied description of the most recent details about creating the <code>DataSource</code>.<br/>LogUri:  A link to the file containing logs of <code>CreateDataSourceFrom*</code> operations.<br/>Status:  The current status of the <code>DataSource</code>. This element can have one of the following values: <ul> <li> <code>PENDING</code> - Amazon ML submitted a request to create a <code>DataSource</code>.</li> <li> <code>INPROGRESS</code> - The creation process is underway.</li> <li> <code>FAILED</code> - The request to create a <code>DataSource</code> did not run to completion. It is not usable.</li> <li> <code>COMPLETED</code> - The creation process completed successfully.</li> <li> <code>DELETED</code> - The <code>DataSource</code> is marked as deleted. It is not usable.</li> </ul><br/>Name:  A user-supplied name or description of the <code>DataSource</code>.<br/>NumberOfFiles:  The number of data files referenced by the <code>DataSource</code>.<br/>DataSizeInBytes:  The total size of observations in the data files.<br/>LastUpdatedAt:  The time of the most recent edit to the <code>DataSource</code>. The time is expressed in epoch time.<br/>CreatedAt:  The time that the <code>DataSource</code> was created. The time is expressed in epoch time.<br/>CreatedByIamUser:  The AWS user account from which the <code>DataSource</code> was created. The account type can be either an AWS root account or an AWS Identity and Access Management (IAM) user account.<br/>DataRearrangement:  A JSON string that represents the splitting and rearrangement requirement used when this <code>DataSource</code> was created.<br/>DataLocationS3:  The location of the data file or directory in Amazon Simple Storage Service (Amazon S3).<br/>DataSourceId:  The ID assigned to the <code>DataSource</code> at creation. This value should be identical to the value of the <code>DataSourceId</code> in the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SelectSqlQuery</i>  The SQL query that is specified during <a>CreateDataSourceFromRedshift</a>. Returns only if <code>Verbose</code> is true in GetDataSourceInput. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseUserName</i> A username to be used by Amazon Machine Learning (Amazon ML)to connect to a database on an Amazon Redshift cluster. The username should have sufficient permissions to execute the <code>RedshiftSelectSqlQuery</code> query. The username should be valid for an Amazon Redshift <a href="http://docs.aws.amazon.com/redshift/latest/dg/r_CREATE_USER.html">USER</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RedshiftDatabase</i> Describes the database details required to connect to an Amazon Redshift database.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RedshiftDatabase</b>: Describes the database details required to connect to an Amazon Redshift database.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The ID of an Amazon Redshift cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> The name of a database hosted on an Amazon Redshift cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataPipelineId</i> The ID of the Data Pipeline instance that is used to carry to copy data from Amazon RDS to Amazon S3. You can use the ID to find details about the instance in the Data Pipeline console.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceRole</i> The role (DataPipelineDefaultRole) assumed by the Data Pipeline service to monitor the progress of the copy task from Amazon RDS to Amazon S3. For more information, see <a href="http://docs.aws.amazon.com/datapipeline/latest/DeveloperGuide/dp-iam-roles.html">Role templates</a> for data pipelines.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceRole</i> The role (DataPipelineDefaultResourceRole) assumed by an Amazon EC2 instance to carry out the copy task from Amazon RDS to Amazon S3. For more information, see <a href="http://docs.aws.amazon.com/datapipeline/latest/DeveloperGuide/dp-iam-roles.html">Role templates</a> for data pipelines.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SelectSqlQuery</i> The SQL query that is supplied during <a>CreateDataSourceFromRDS</a>. Returns only if <code>Verbose</code> is true in <code>GetDataSourceInput</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseUserName</i> The username to be used by Amazon ML to connect to database on an Amazon RDS instance. The username should have sufficient permissions to execute an <code>RDSSelectSqlQuery</code> query.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Database</i> The database details required to connect to an Amazon RDS.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Database</b>: The database details required to connect to an Amazon RDS.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> The name of a database hosted on an RDS DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceIdentifier</i> The ID of an RDS DB instance.<br/>        """
        pass

    def get_evaluation(self, EvaluationId: str) -> GetEvaluationOutput:
        """Returns an <code>Evaluation</code> that includes metadata as well as the current status of the <code>Evaluation</code>.

        :param self:
        :param EvaluationId:  The ID of the <code>Evaluation</code> to retrieve. The evaluation of each <code>MLModel</code> is recorded and cataloged. The ID provides the means to access the information. <br/>
        :return: StartedAt:  The epoch time when Amazon Machine Learning marked the <code>Evaluation</code> as <code>INPROGRESS</code>. <code>StartedAt</code> isn't available if the <code>Evaluation</code> is in the <code>PENDING</code> state.<br/>FinishedAt:  The epoch time when Amazon Machine Learning marked the <code>Evaluation</code> as <code>COMPLETED</code> or <code>FAILED</code>. <code>FinishedAt</code> is only available when the <code>Evaluation</code> is in the <code>COMPLETED</code> or <code>FAILED</code> state.<br/>ComputeTime:  The approximate CPU time in milliseconds that Amazon Machine Learning spent processing the <code>Evaluation</code>, normalized and scaled on computation resources. <code>ComputeTime</code> is only available if the <code>Evaluation</code> is in the <code>COMPLETED</code> state.<br/>Message:  A description of the most recent details about evaluating the <code>MLModel</code>.<br/>LogUri:  A link to the file that contains logs of the <code>CreateEvaluation</code> operation.<br/>PerformanceMetrics:  Measurements of how well the <code>MLModel</code> performed using observations referenced by the <code>DataSource</code>. One of the following metric is returned based on the type of the <code>MLModel</code>:  <ul> <li> BinaryAUC: A binary <code>MLModel</code> uses the Area Under the Curve (AUC) technique to measure performance.  </li> <li> RegressionRMSE: A regression <code>MLModel</code> uses the Root Mean Square Error (RMSE) technique to measure performance. RMSE measures the difference between predicted and actual values for a single variable. </li> <li> MulticlassAvgFScore: A multiclass <code>MLModel</code> uses the F1 score technique to measure performance.  </li> </ul>  For more information about performance metrics, please see the <a href="http://docs.aws.amazon.com/machine-learning/latest/dg">Amazon Machine Learning Developer Guide</a>. :<br/>Status:  The status of the evaluation. This element can have one of the following values: <ul> <li> <code>PENDING</code> - Amazon Machine Language (Amazon ML) submitted a request to evaluate an <code>MLModel</code>.</li> <li> <code>INPROGRESS</code> - The evaluation is underway.</li> <li> <code>FAILED</code> - The request to evaluate an <code>MLModel</code> did not run to completion. It is not usable.</li> <li> <code>COMPLETED</code> - The evaluation process completed successfully.</li> <li> <code>DELETED</code> - The <code>Evaluation</code> is marked as deleted. It is not usable.</li> </ul><br/>Name:  A user-supplied name or description of the <code>Evaluation</code>. <br/>LastUpdatedAt:  The time of the most recent edit to the <code>Evaluation</code>. The time is expressed in epoch time.<br/>CreatedAt:  The time that the <code>Evaluation</code> was created. The time is expressed in epoch time.<br/>CreatedByIamUser:  The AWS user account that invoked the evaluation. The account type can be either an AWS root account or an AWS Identity and Access Management (IAM) user account.<br/>InputDataLocationS3:  The location of the data file or directory in Amazon Simple Storage Service (Amazon S3).<br/>EvaluationDataSourceId:  The <code>DataSource</code> used for this evaluation.<br/>MLModelId:  The ID of the <code>MLModel</code> that was the focus of the evaluation.<br/>EvaluationId:  The evaluation ID which is same as the <code>EvaluationId</code> in the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Properties</i> <br/>        """
        pass

    def get_ml_model(self, MLModelId: str, Verbose: Verbose=None) -> GetMLModelOutput:
        """Returns an <code>MLModel</code> that includes detailed metadata, data source information, and the current status of the <code>MLModel</code>. <code>GetMLModel</code> provides results in normal or verbose format. 

        :param self:
        :param Verbose:  Specifies whether the <code>GetMLModel</code> operation should return <code>Recipe</code>. If true, <code>Recipe</code> is returned. If false, <code>Recipe</code> is not returned.<br/>MLModelId:  The ID assigned to the <code>MLModel</code> at creation.<br/>
        :return: Schema:  The schema used by all of the data files referenced by the <code>DataSource</code>. <note><title>Note</title> This parameter is provided as part of the verbose format.</note><br/>Recipe:  The recipe to use when training the <code>MLModel</code>. The <code>Recipe</code> provides detailed information about the observation data to use during training, and manipulations to perform on the observation data during training. <note><title>Note</title> This parameter is provided as part of the verbose format.</note><br/>StartedAt:  The epoch time when Amazon Machine Learning marked the <code>MLModel</code> as <code>INPROGRESS</code>. <code>StartedAt</code> isn't available if the <code>MLModel</code> is in the <code>PENDING</code> state.<br/>FinishedAt:  The epoch time when Amazon Machine Learning marked the <code>MLModel</code> as <code>COMPLETED</code> or <code>FAILED</code>. <code>FinishedAt</code> is only available when the <code>MLModel</code> is in the <code>COMPLETED</code> or <code>FAILED</code> state.<br/>ComputeTime:  The approximate CPU time in milliseconds that Amazon Machine Learning spent processing the <code>MLModel</code>, normalized and scaled on computation resources. <code>ComputeTime</code> is only available if the <code>MLModel</code> is in the <code>COMPLETED</code> state.<br/>Message:  A description of the most recent details about accessing the <code>MLModel</code>.<br/>LogUri:  A link to the file that contains logs of the <code>CreateMLModel</code> operation.<br/>ScoreThresholdLastUpdatedAt:  The time of the most recent edit to the <code>ScoreThreshold</code>. The time is expressed in epoch time.<br/>ScoreThreshold:  The scoring threshold is used in binary classification <code>MLModel</code><?oxy_insert_start author="laurama" timestamp="20160329T114851-0700"> <?oxy_insert_end>models. It marks the boundary between a positive prediction and a negative prediction. Output values greater than or equal to the threshold receive a positive result from the MLModel, such as <code>true</code>. Output values less than the threshold receive a negative response from the MLModel, such as <code>false</code>.<br/>MLModelType:  Identifies the <code>MLModel</code> category. The following are the available types:  <ul> <li>REGRESSION -- Produces a numeric result. For example, "What price should a house be listed at?"</li> <li>BINARY -- Produces one of two possible results. For example, "Is this an e-commerce website?"</li> <li>MULTICLASS -- Produces one of several possible results. For example, "Is this a HIGH, LOW or MEDIUM risk trade?"</li> </ul><br/>InputDataLocationS3:  The location of the data file or directory in Amazon Simple Storage Service (Amazon S3).<br/>TrainingParameters:  A list of the training parameters in the <code>MLModel</code>. The list is implemented as a map of key-value pairs. The following is the current set of training parameters:  <ul> <li> <code>sgd.maxMLModelSizeInBytes</code> - The maximum allowed size of the model. Depending on the input data, the size of the model might affect its performance.  The value is an integer that ranges from <code>100000</code> to <code>2147483648</code>. The default value is <code>33554432</code>. </li> <li><code>sgd.maxPasses</code> - The number of times that the training process traverses the observations to build the <code>MLModel</code>. The value is an integer that ranges from <code>1</code> to <code>10000</code>. The default value is <code>10</code>.</li> <li><code>sgd.shuffleType</code> - Whether Amazon ML shuffles the training data. Shuffling data improves a model's ability to find the optimal solution for a variety of data types. The valid values are <code>auto</code> and <code>none</code>. The default value is <code>none</code>. We strongly recommend that you shuffle your data.</li> <li> <code>sgd.l1RegularizationAmount</code> - The coefficient regularization L1 norm. It controls overfitting the data by penalizing large coefficients. This tends to drive coefficients to zero, resulting in a sparse feature set. If you use this parameter, start by specifying a small value, such as <code>1.0E-08</code>. The value is a double that ranges from <code>0</code> to <code>MAX_DOUBLE</code>. The default is to not use L1 normalization. This parameter can't be used when <code>L2</code> is specified. Use this parameter sparingly. </li> <li> <code>sgd.l2RegularizationAmount</code> - The coefficient regularization L2 norm. It controls overfitting the data by penalizing large coefficients. This tends to drive coefficients to small, nonzero values. If you use this parameter, start by specifying a small value, such as <code>1.0E-08</code>. The value is a double that ranges from <code>0</code> to <code>MAX_DOUBLE</code>. The default is to not use L2 normalization. This parameter can't be used when <code>L1</code> is specified. Use this parameter sparingly. </li> </ul><br/>EndpointInfo:  The current endpoint of the <code>MLModel</code>:<br/>SizeInBytes:  Long integer type that is a 64-bit signed number.<br/>Status:  The current status of the <code>MLModel</code>. This element can have one of the following values: <ul> <li> <code>PENDING</code> - Amazon Machine Learning (Amazon ML) submitted a request to describe a <code>MLModel</code>.</li> <li> <code>INPROGRESS</code> - The request is processing.</li> <li> <code>FAILED</code> - The request did not run to completion. The ML model isn't usable.</li> <li> <code>COMPLETED</code> - The request completed successfully.</li> <li> <code>DELETED</code> - The <code>MLModel</code> is marked as deleted. It isn't usable.</li> </ul><br/>Name:  A user-supplied name or description of the <code>MLModel</code>.<br/>LastUpdatedAt:  The time of the most recent edit to the <code>MLModel</code>. The time is expressed in epoch time.<br/>CreatedAt:  The time that the <code>MLModel</code> was created. The time is expressed in epoch time.<br/>CreatedByIamUser:  The AWS user account from which the <code>MLModel</code> was created. The account type can be either an AWS root account or an AWS Identity and Access Management (IAM) user account.<br/>TrainingDataSourceId:  The ID of the training <code>DataSource</code>.<br/>MLModelId:  The MLModel ID<?oxy_insert_start author="annbech" timestamp="20160328T151251-0700">,<?oxy_insert_end> which is same as the <code>MLModelId</code> in the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointStatus</i>  The current status of the real-time endpoint for the <code>MLModel</code>. This element can have one of the following values:  <ul> <li> <code>NONE</code> - Endpoint does not exist or was previously deleted.</li> <li> <code>READY</code> - Endpoint is ready to be used for real-time predictions.</li> <li> <code>UPDATING</code> - Updating/creating the endpoint. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointUrl</i> The URI that specifies where to send real-time prediction requests for the <code>MLModel</code>. <note><title>Note</title> The application must wait until the real-time endpoint is ready before using this URI. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i> The time that the request to create the real-time endpoint for the <code>MLModel</code> was received. The time is expressed in epoch time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PeakRequestsPerSecond</i>  The maximum processing rate for the real-time endpoint for <code>MLModel</code>, measured in incoming requests per second.<br/>        """
        pass

    def predict(self, PredictEndpoint: str, Record: Record, MLModelId: str) -> PredictOutput:
        """Generates a prediction for the observation using the specified <code>ML Model</code>. <note><title>Note</title> Not all response parameters will be populated. Whether a response parameter is populated depends on the type of model requested.</note>

        :param self:
        :param PredictEndpoint:  <br/>Record:  A map of variable name-value pairs that represent an observation.<br/>MLModelId:  A unique identifier of the <code>MLModel</code>.<br/>
        :return: Prediction:  The output from a <code>Predict</code> operation:  <ul> <li>  <code>Details</code> - Contains the following attributes: <code>DetailsAttributes.PREDICTIVE_MODEL_TYPE - REGRESSION | BINARY | MULTICLASS</code> <code>DetailsAttributes.ALGORITHM - SGD</code>  </li> <li>  <code>PredictedLabel</code> - Present for either a <code>BINARY</code> or <code>MULTICLASS</code> <code>MLModel</code> request.  </li> <li>  <code>PredictedScores</code> - Contains the raw classification score corresponding to each label.  </li> <li>  <code>PredictedValue</code> - Present for a <code>REGRESSION</code> <code>MLModel</code> request.  </li> </ul>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>details</i> Provides any additional details regarding the prediction.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>predictedScores</i> Provides the raw classification score corresponding to each label.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>predictedValue</i> The prediction value for <code>REGRESSION</code> <code>MLModel</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>predictedLabel</i> The prediction label for either a <code>BINARY</code> or <code>MULTICLASS</code> <code>MLModel</code>.<br/>        """
        pass

    def update_batch_prediction(self, BatchPredictionName: str, BatchPredictionId: str) -> UpdateBatchPredictionOutput:
        """Updates the <code>BatchPredictionName</code> of a <code>BatchPrediction</code>. You can use the <code>GetBatchPrediction</code> operation to view the contents of the updated data element.

        :param self:
        :param BatchPredictionName:  A new user-supplied name or description of the <code>BatchPrediction</code>.<br/>BatchPredictionId:  The ID assigned to the <code>BatchPrediction</code> during creation.<br/>
        :return: BatchPredictionId:  The ID assigned to the <code>BatchPrediction</code> during creation. This value should be identical to the value of the <code>BatchPredictionId</code> in the request.<br/>        """
        pass

    def update_data_source(self, DataSourceName: str, DataSourceId: str) -> UpdateDataSourceOutput:
        """Updates the <code>DataSourceName</code> of a <code>DataSource</code>. You can use the <code>GetDataSource</code> operation to view the contents of the updated data element.

        :param self:
        :param DataSourceName:  A new user-supplied name or description of the <code>DataSource</code> that will replace the current description. <br/>DataSourceId:  The ID assigned to the <code>DataSource</code> during creation.<br/>
        :return: DataSourceId:  The ID assigned to the <code>DataSource</code> during creation. This value should be identical to the value of the <code>DataSourceID</code> in the request.<br/>        """
        pass

    def update_evaluation(self, EvaluationName: str, EvaluationId: str) -> UpdateEvaluationOutput:
        """Updates the <code>EvaluationName</code> of an <code>Evaluation</code>. You can use the <code>GetEvaluation</code> operation to view the contents of the updated data element.

        :param self:
        :param EvaluationName:  A new user-supplied name or description of the <code>Evaluation</code> that will replace the current content. <br/>EvaluationId:  The ID assigned to the <code>Evaluation</code> during creation.<br/>
        :return: EvaluationId:  The ID assigned to the <code>Evaluation</code> during creation. This value should be identical to the value of the <code>Evaluation</code> in the request.<br/>        """
        pass

    def update_ml_model(self, MLModelId: str, MLModelName: str=None, ScoreThreshold: ScoreThreshold=None) -> UpdateMLModelOutput:
        """Updates the <code>MLModelName</code> and the <code>ScoreThreshold</code> of an <code>MLModel</code>. You can use the <code>GetMLModel</code> operation to view the contents of the updated data element.

        :param self:
        :param ScoreThreshold:  The <code>ScoreThreshold</code> used in binary classification <code>MLModel</code> that marks the boundary between a positive prediction and a negative prediction. Output values greater than or equal to the <code>ScoreThreshold</code> receive a positive result from the <code>MLModel</code>, such as <code>true</code>. Output values less than the <code>ScoreThreshold</code> receive a negative response from the <code>MLModel</code>, such as <code>false</code>.<br/>MLModelName:  A user-supplied name or description of the <code>MLModel</code>.<br/>MLModelId:  The ID assigned to the <code>MLModel</code> during creation.<br/>
        :return: MLModelId:  The ID assigned to the <code>MLModel</code> during creation. This value should be identical to the value of the <code>MLModelID</code> in the request.<br/>        """
        pass



class Macie(BaseClient):
    """<fullname>Amazon Macie</fullname> Amazon Macie is a security service that uses machine learning to automatically discover, classify, and protect sensitive data in AWS. Macie recognizes sensitive data such as personally identifiable information (PII) or intellectual property, and provides you with dashboards and alerts that give visibility into how this data is being accessed or moved. For more information, see the <a href="https://docs.aws.amazon.com/macie/latest/userguide/what-is-macie.html">Macie User Guide</a>. """


    class ListMemberAccountsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_member_accounts` for valid parameters"""
            pass
    class ListS3ResourcesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_s3_resources` for valid parameters"""
            pass


    class AssociateMemberAccountRequest(Mapping):
        pass
    
    class AssociateS3ResourcesRequest(Mapping):
        pass
    
    class AssociateS3ResourcesResult(Mapping):
        pass
    
    class S3ResourceClassification(Mapping):
        pass
    
    class DisassociateMemberAccountRequest(Mapping):
        pass
    
    class DisassociateS3ResourcesRequest(Mapping):
        pass
    
    class DisassociateS3ResourcesResult(Mapping):
        pass
    
    class S3Resource(Mapping):
        pass
    
    class ListMemberAccountsRequest(Mapping):
        pass
    
    class ListMemberAccountsResult(Mapping):
        pass
    
    class ListS3ResourcesRequest(Mapping):
        pass
    
    class ListS3ResourcesResult(Mapping):
        pass
    
    class UpdateS3ResourcesRequest(Mapping):
        pass
    
    class UpdateS3ResourcesResult(Mapping):
        pass
    
    class S3ResourceClassificationUpdate(Mapping):
        pass
    
    def associate_member_account(self, memberAccountId: str):
        """Associates a specified AWS account with Amazon Macie as a member account.

        :param self:
        :param memberAccountId:  The ID of the AWS account that you want to associate with Amazon Macie as a member account.<br/>
        :return:         """
        pass

    def associate_s3_resources(self, s3Resources: List[S3ResourceClassification], memberAccountId: str=None) -> AssociateS3ResourcesResult:
        """Associates specified S3 resources with Amazon Macie for monitoring and data classification. If memberAccountId isn't specified, the action associates specified S3 resources with Macie for the current master account. If memberAccountId is specified, the action associates specified S3 resources with Macie for the specified member account. 

        :param self:
        :param s3Resources:  The S3 resources that you want to associate with Amazon Macie for monitoring and data classification. <br/>memberAccountId:  The ID of the Amazon Macie member account whose resources you want to associate with Macie. <br/>
        :return: failedS3Resources:  S3 resources that couldn't be associated with Amazon Macie. An error code and an error message are provided for each failed item. <br/>        """
        pass

    def disassociate_member_account(self, memberAccountId: str):
        """Removes the specified member account from Amazon Macie.

        :param self:
        :param memberAccountId:  The ID of the member account that you want to remove from Amazon Macie.<br/>
        :return:         """
        pass

    def disassociate_s3_resources(self, associatedS3Resources: List[S3Resource], memberAccountId: str=None) -> DisassociateS3ResourcesResult:
        """Removes specified S3 resources from being monitored by Amazon Macie. If memberAccountId isn't specified, the action removes specified S3 resources from Macie for the current master account. If memberAccountId is specified, the action removes specified S3 resources from Macie for the specified member account.

        :param self:
        :param associatedS3Resources:  The S3 resources (buckets or prefixes) that you want to remove from being monitored and classified by Amazon Macie. <br/>memberAccountId:  The ID of the Amazon Macie member account whose resources you want to remove from being monitored by Amazon Macie. <br/>
        :return: failedS3Resources:  S3 resources that couldn't be removed from being monitored and classified by Amazon Macie. An error code and an error message are provided for each failed item. <br/>        """
        pass

    def list_member_accounts(self, nextToken: str=None, maxResults: int=None) -> ListMemberAccountsResult:
        """Lists all Amazon Macie member accounts for the current Amazon Macie master account.

        :param self:
        :param maxResults:  Use this parameter to indicate the maximum number of items that you want in the response. The default value is 250. <br/>nextToken:  Use this parameter when paginating results. Set the value of this parameter to null on your first call to the ListMemberAccounts action. Subsequent calls to the action fill nextToken in the request with the value of nextToken from the previous response to continue listing data. <br/>
        :return: nextToken:  When a response is generated, if there is more data to be listed, this parameter is present in the response and contains the value to use for the nextToken parameter in a subsequent pagination request. If there is no more data to be listed, this parameter is set to null. <br/>memberAccounts:  A list of the Amazon Macie member accounts returned by the action. The current master account is also included in this list. <br/>        """
        pass

    def list_s3_resources(self, memberAccountId: str=None, nextToken: str=None, maxResults: int=None) -> ListS3ResourcesResult:
        """Lists all the S3 resources associated with Amazon Macie. If memberAccountId isn't specified, the action lists the S3 resources associated with Amazon Macie for the current master account. If memberAccountId is specified, the action lists the S3 resources associated with Amazon Macie for the specified member account. 

        :param self:
        :param maxResults:  Use this parameter to indicate the maximum number of items that you want in the response. The default value is 250. <br/>nextToken:  Use this parameter when paginating results. Set its value to null on your first call to the ListS3Resources action. Subsequent calls to the action fill nextToken in the request with the value of nextToken from the previous response to continue listing data. <br/>memberAccountId:  The Amazon Macie member account ID whose associated S3 resources you want to list. <br/>
        :return: nextToken:  When a response is generated, if there is more data to be listed, this parameter is present in the response and contains the value to use for the nextToken parameter in a subsequent pagination request. If there is no more data to be listed, this parameter is set to null. <br/>s3Resources:  A list of the associated S3 resources returned by the action.<br/>        """
        pass

    def update_s3_resources(self, s3ResourcesUpdate: List[S3ResourceClassificationUpdate], memberAccountId: str=None) -> UpdateS3ResourcesResult:
        """Updates the classification types for the specified S3 resources. If memberAccountId isn't specified, the action updates the classification types of the S3 resources associated with Amazon Macie for the current master account. If memberAccountId is specified, the action updates the classification types of the S3 resources associated with Amazon Macie for the specified member account. 

        :param self:
        :param s3ResourcesUpdate:  The S3 resources whose classification types you want to update.<br/>memberAccountId:  The AWS ID of the Amazon Macie member account whose S3 resources' classification types you want to update. <br/>
        :return: failedS3Resources:  The S3 resources whose classification types can't be updated. An error code and an error message are provided for each failed item. <br/>        """
        pass



class ManagedBlockchain(BaseClient):
    """<p/> Amazon Managed Blockchain is a fully managed service for creating and managing blockchain networks using open source frameworks. Blockchain allows you to build applications where multiple parties can securely and transparently run transactions and share data without the need for a trusted, central authority. Currently, Managed Blockchain supports the Hyperledger Fabric open source framework. """




    class CreateMemberInput(Mapping):
        pass
    
    class CreateMemberOutput(Mapping):
        pass
    
    class MemberConfiguration(Mapping):
        pass
    
    class CreateNetworkInput(Mapping):
        pass
    
    class CreateNetworkOutput(Mapping):
        pass
    
    class NetworkFrameworkConfiguration(Mapping):
        pass
    
    class VotingPolicy(Mapping):
        pass
    
    class CreateNodeInput(Mapping):
        pass
    
    class CreateNodeOutput(Mapping):
        pass
    
    class NodeConfiguration(Mapping):
        pass
    
    class CreateProposalInput(Mapping):
        pass
    
    class CreateProposalOutput(Mapping):
        pass
    
    class ProposalActions(Mapping):
        pass
    
    class DeleteMemberInput(Mapping):
        pass
    
    class DeleteMemberOutput(Mapping):
        pass
    
    class DeleteNodeInput(Mapping):
        pass
    
    class DeleteNodeOutput(Mapping):
        pass
    
    class GetMemberInput(Mapping):
        pass
    
    class GetMemberOutput(Mapping):
        pass
    
    class GetNetworkInput(Mapping):
        pass
    
    class GetNetworkOutput(Mapping):
        pass
    
    class GetNodeInput(Mapping):
        pass
    
    class GetNodeOutput(Mapping):
        pass
    
    class GetProposalInput(Mapping):
        pass
    
    class GetProposalOutput(Mapping):
        pass
    
    class ListInvitationsInput(Mapping):
        pass
    
    class ListInvitationsOutput(Mapping):
        pass
    
    class ListMembersInput(Mapping):
        pass
    
    class ListMembersOutput(Mapping):
        pass
    
    class IsOwned(object):
        pass
    
    class ListNetworksInput(Mapping):
        pass
    
    class ListNetworksOutput(Mapping):
        pass
    
    class ListNodesInput(Mapping):
        pass
    
    class ListNodesOutput(Mapping):
        pass
    
    class ListProposalVotesInput(Mapping):
        pass
    
    class ListProposalVotesOutput(Mapping):
        pass
    
    class ListProposalsInput(Mapping):
        pass
    
    class ListProposalsOutput(Mapping):
        pass
    
    class RejectInvitationInput(Mapping):
        pass
    
    class RejectInvitationOutput(Mapping):
        pass
    
    class VoteOnProposalInput(Mapping):
        pass
    
    class VoteOnProposalOutput(Mapping):
        pass
    
    def create_member(self, MemberConfiguration: MemberConfiguration, NetworkId: str, InvitationId: str, ClientRequestToken: str) -> CreateMemberOutput:
        """Creates a member within a Managed Blockchain network.

        :param self:
        :param MemberConfiguration:  Member configuration parameters.:<br/>NetworkId:  The unique identifier of the network in which the member is created.<br/>InvitationId:  The unique identifier of the invitation that is sent to the member to join the network.<br/>ClientRequestToken:  A unique, case-sensitive identifier that you provide to ensure the idempotency of the operation. An idempotent operation completes no more than one time. This identifier is required only if you make a service request directly using an HTTP client. It is generated automatically if you use an AWS SDK or the AWS CLI.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameworkConfiguration</i> Configuration properties of the blockchain framework relevant to the member.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>FrameworkConfiguration</b>: Configuration properties of the blockchain framework relevant to the member.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> An optional description of the member.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the member.<br/>Attributes of Hyperledger Fabric for a member on a Managed Blockchain network that uses Hyperledger Fabric.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Fabric</b>: Attributes of Hyperledger Fabric for a member on a Managed Blockchain network that uses Hyperledger Fabric.
        :return: MemberId:  The unique identifier of the member.<br/>        """
        pass

    def create_network(self, MemberConfiguration: MemberConfiguration, VotingPolicy: VotingPolicy, FrameworkVersion: str, Framework: str, Name: str, ClientRequestToken: str, Description: str=None, FrameworkConfiguration: NetworkFrameworkConfiguration=None) -> CreateNetworkOutput:
        """Creates a new blockchain network using Amazon Managed Blockchain.

        :param self:
        :param MemberConfiguration:  Configuration properties for the first member within the network.:<br/>VotingPolicy:   The voting rules used by the network to determine if a proposal is approved. :<br/>FrameworkConfiguration:   Configuration properties of the blockchain framework relevant to the network configuration. :<br/>FrameworkVersion:  The version of the blockchain framework that the network uses.<br/>Framework:  The blockchain framework that the network uses.<br/>Description:  An optional description for the network.<br/>Name:  The name of the network.<br/>ClientRequestToken:  A unique, case-sensitive identifier that you provide to ensure the idempotency of the operation. An idempotent operation completes no more than one time. This identifier is required only if you make a service request directly using an HTTP client. It is generated automatically if you use an AWS SDK or the AWS CLI.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Fabric</i>  Hyperledger Fabric configuration properties for a Managed Blockchain network that uses Hyperledger Fabric. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Fabric</b>:  Hyperledger Fabric configuration properties for a Managed Blockchain network that uses Hyperledger Fabric. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Edition</i> The edition of Amazon Managed Blockchain that the network uses. For more information, see <a href="https://aws.amazon.com/managed-blockchain/pricing/">Amazon Managed Blockchain Pricing</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApprovalThresholdPolicy</i> Defines the rules for the network for voting on proposals, such as the percentage of <code>YES</code> votes required for the proposal to be approved and the duration of the proposal. The policy applies to all proposals and is specified when the network is created.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ApprovalThresholdPolicy</b>: Defines the rules for the network for voting on proposals, such as the percentage of <code>YES</code> votes required for the proposal to be approved and the duration of the proposal. The policy applies to all proposals and is specified when the network is created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ThresholdComparator</i> Determines whether the vote percentage must be greater than the <code>ThresholdPercentage</code> or must be greater than or equal to the <code>ThreholdPercentage</code> to be approved.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ProposalDurationInHours</i> The duration from the time that a proposal is created until it expires. If members cast neither the required number of <code>YES</code> votes to approve the proposal nor the number of <code>NO</code> votes required to reject it before the duration expires, the proposal is <code>EXPIRED</code> and <code>ProposalActions</code> are not carried out.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ThresholdPercentage</i> The percentage of votes among all members that must be <code>YES</code> for a proposal to be approved. For example, a <code>ThresholdPercentage</code> value of <code>50</code> indicates 50%. The <code>ThresholdComparator</code> determines the precise comparison. If a <code>ThresholdPercentage</code> value of <code>50</code> is specified on a network with 10 members, along with a <code>ThresholdComparator</code> value of <code>GREATER_THAN</code>, this indicates that 6 <code>YES</code> votes are required for the proposal to be approved.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameworkConfiguration</i> Configuration properties of the blockchain framework relevant to the member.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>FrameworkConfiguration</b>: Configuration properties of the blockchain framework relevant to the member.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> An optional description of the member.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the member.<br/>Attributes of Hyperledger Fabric for a member on a Managed Blockchain network that uses Hyperledger Fabric.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Fabric</b>: Attributes of Hyperledger Fabric for a member on a Managed Blockchain network that uses Hyperledger Fabric.
        :return: MemberId:  The unique identifier for the first member within the network.<br/>NetworkId:  The unique identifier for the network.<br/>        """
        pass

    def create_node(self, NodeConfiguration: NodeConfiguration, MemberId: str, NetworkId: str, ClientRequestToken: str) -> CreateNodeOutput:
        """Creates a peer node in a member.

        :param self:
        :param NodeConfiguration:  The properties of a node configuration.:<br/>MemberId:  The unique identifier of the member that owns this node.<br/>NetworkId:  The unique identifier of the network in which this node runs.<br/>ClientRequestToken:  A unique, case-sensitive identifier that you provide to ensure the idempotency of the operation. An idempotent operation completes no more than one time. This identifier is required only if you make a service request directly using an HTTP client. It is generated automatically if you use an AWS SDK or the AWS CLI.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone in which the node exists.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i> The Amazon Managed Blockchain instance type for the node.<br/>
        :return: NodeId:  The unique identifier of the node.<br/>        """
        pass

    def create_proposal(self, Actions: ProposalActions, MemberId: str, NetworkId: str, ClientRequestToken: str, Description: str=None) -> CreateProposalOutput:
        """Creates a proposal for a change to the network that other members of the network can vote on, for example, a proposal to add a new member to the network. Any member can create a proposal.

        :param self:
        :param Description:  A description for the proposal that is visible to voting members, for example, "Proposal to add Example Corp. as member."<br/>Actions:  The type of actions proposed, such as inviting a member or removing a member. The types of <code>Actions</code> in a proposal are mutually exclusive. For example, a proposal with <code>Invitations</code> actions cannot also contain <code>Removals</code> actions.:<br/>MemberId:  The unique identifier of the member that is creating the proposal. This identifier is especially useful for identifying the member making the proposal when multiple members exist in a single AWS account.<br/>NetworkId:   The unique identifier of the network for which the proposal is made.<br/>ClientRequestToken:  A unique, case-sensitive identifier that you provide to ensure the idempotency of the operation. An idempotent operation completes no more than one time. This identifier is required only if you make a service request directly using an HTTP client. It is generated automatically if you use an AWS SDK or the AWS CLI.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Removals</i>  The actions to perform for an <code>APPROVED</code> proposal to remove a member from the network, which deletes the member and all associated member resources from the network. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Invitations</i>  The actions to perform for an <code>APPROVED</code> proposal to invite an AWS account to create a member and join the network. <br/>
        :return: ProposalId:  The unique identifier of the proposal.<br/>        """
        pass

    def delete_member(self, MemberId: str, NetworkId: str) -> DeleteMemberOutput:
        """Deletes a member. Deleting a member removes the member and all associated resources from the network. <code>DeleteMember</code> can only be called for a specified <code>MemberId</code> if the principal performing the action is associated with the AWS account that owns the member. In all other cases, the <code>DeleteMember</code> action is carried out as the result of an approved proposal to remove a member. If <code>MemberId</code> is the last member in a network specified by the last AWS account, the network is deleted also.

        :param self:
        :param MemberId:  The unique identifier of the member to remove.<br/>NetworkId:  The unique identifier of the network from which the member is removed.<br/>
        :return:         """
        pass

    def delete_node(self, NodeId: str, MemberId: str, NetworkId: str) -> DeleteNodeOutput:
        """Deletes a peer node from a member that your AWS account owns. All data on the node is lost and cannot be recovered.

        :param self:
        :param NodeId:  The unique identifier of the node.<br/>MemberId:  The unique identifier of the member that owns this node.<br/>NetworkId:  The unique identifier of the network that the node belongs to.<br/>
        :return:         """
        pass

    def get_member(self, MemberId: str, NetworkId: str) -> GetMemberOutput:
        """Returns detailed information about a member.

        :param self:
        :param MemberId:  The unique identifier of the member.<br/>NetworkId:  The unique identifier of the network to which the member belongs.<br/>
        :return: Member:  The properties of a member.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time that the member was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of a member. <ul> <li>  <code>CREATING</code> - The AWS account is in the process of creating a member. </li> <li>  <code>AVAILABLE</code> - The member has been created and can participate in the network. </li> <li>  <code>CREATE_FAILED</code> - The AWS account attempted to create a member and creation failed. </li> <li>  <code>DELETING</code> - The member and all associated resources are in the process of being deleted. Either the AWS account that owns the member deleted it, or the member is being deleted as the result of an <code>APPROVED</code> <code>PROPOSAL</code> to remove the member. </li> <li>  <code>DELETED</code> - The member can no longer participate on the network and all associated resources are deleted. Either the AWS account that owns the member deleted it, or the member is being deleted as the result of an <code>APPROVED</code> <code>PROPOSAL</code> to remove the member. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameworkAttributes</i> Attributes relevant to a member for the blockchain framework that the Managed Blockchain network uses.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>FrameworkAttributes</b>: Attributes relevant to a member for the blockchain framework that the Managed Blockchain network uses.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> An optional description for the member.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the member.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier of the member.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkId</i> The unique identifier of the network to which the member belongs.<br/>Attributes of Hyperledger Fabric relevant to a member on a Managed Blockchain network that uses Hyperledger Fabric.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Fabric</b>: Attributes of Hyperledger Fabric relevant to a member on a Managed Blockchain network that uses Hyperledger Fabric.        """
        pass

    def get_network(self, NetworkId: str) -> GetNetworkOutput:
        """Returns detailed information about a network.

        :param self:
        :param NetworkId:  The unique identifier of the network to get information about.<br/>
        :return: Network:  An object containing network configuration parameters.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time that the network was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current status of the network.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VotingPolicy</i> The voting rules for the network to decide if a proposal is accepted.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VotingPolicy</b>: The voting rules for the network to decide if a proposal is accepted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcEndpointServiceName</i> The VPC endpoint service name of the VPC endpoint service of the network. Members use the VPC endpoint service name to create a VPC endpoint to access network resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameworkAttributes</i> Attributes of the blockchain framework that the network uses.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>FrameworkAttributes</b>: Attributes of the blockchain framework that the network uses.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameworkVersion</i> The version of the blockchain framework that the network uses.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Framework</i> The blockchain framework that the network uses.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> Attributes of the blockchain framework for the network.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the network.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier of the network.<br/>Attributes of Hyperledger Fabric for a Managed Blockchain network that uses Hyperledger Fabric.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Fabric</b>: Attributes of Hyperledger Fabric for a Managed Blockchain network that uses Hyperledger Fabric.Defines the rules for the network for voting on proposals, such as the percentage of <code>YES</code> votes required for the proposal to be approved and the duration of the proposal. The policy applies to all proposals and is specified when the network is created.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ApprovalThresholdPolicy</b>: Defines the rules for the network for voting on proposals, such as the percentage of <code>YES</code> votes required for the proposal to be approved and the duration of the proposal. The policy applies to all proposals and is specified when the network is created.        """
        pass

    def get_node(self, NodeId: str, MemberId: str, NetworkId: str) -> GetNodeOutput:
        """Returns detailed information about a peer node.

        :param self:
        :param NodeId:  The unique identifier of the node.<br/>MemberId:  The unique identifier of the member that owns the node.<br/>NetworkId:  The unique identifier of the network to which the node belongs.<br/>
        :return: Node:  Properties of the node configuration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time that the node was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameworkAttributes</i> Attributes of the blockchain framework being used.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>FrameworkAttributes</b>: Attributes of the blockchain framework being used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone in which the node exists.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i> The instance type of the node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier of the node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MemberId</i> The unique identifier of the member to which the node belongs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkId</i> The unique identifier of the network that the node is in.<br/>Attributes of Hyperledger Fabric for a peer node on a Managed Blockchain network that uses Hyperledger Fabric.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Fabric</b>: Attributes of Hyperledger Fabric for a peer node on a Managed Blockchain network that uses Hyperledger Fabric.        """
        pass

    def get_proposal(self, ProposalId: str, NetworkId: str) -> GetProposalOutput:
        """Returns detailed information about a proposal.

        :param self:
        :param ProposalId:  The unique identifier of the proposal.<br/>NetworkId:  The unique identifier of the network for which the proposal is made.<br/>
        :return: Proposal:  Information about a proposal.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutstandingVoteCount</i>  The number of votes remaining to be cast on the proposal by members. In other words, the number of members minus the sum of <code>YES</code> votes and <code>NO</code> votes. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NoVoteCount</i>  The current total of <code>NO</code> votes cast on the proposal by members. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>YesVoteCount</i>  The current total of <code>YES</code> votes cast on the proposal by members. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpirationDate</i>  The date and time that the proposal expires. This is the <code>CreationDate</code> plus the <code>ProposalDurationInHours</code> that is specified in the <code>ProposalThresholdPolicy</code>. After this date and time, if members have not cast enough votes to determine the outcome according to the voting policy, the proposal is <code>EXPIRED</code> and <code>Actions</code> are not carried out. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i>  The date and time that the proposal was created. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the proposal. Values are as follows: <ul> <li>  <code>IN_PROGRESS</code> - The proposal is active and open for member voting. </li> <li>  <code>APPROVED</code> - The proposal was approved with sufficient <code>YES</code> votes among members according to the <code>VotingPolicy</code> specified for the <code>Network</code>. The specified proposal actions are carried out. </li> <li>  <code>REJECTED</code> - The proposal was rejected with insufficient <code>YES</code> votes among members according to the <code>VotingPolicy</code> specified for the <code>Network</code>. The specified <code>ProposalActions</code> are not carried out. </li> <li>  <code>EXPIRED</code> - Members did not cast the number of votes required to determine the proposal outcome before the proposal expired. The specified <code>ProposalActions</code> are not carried out. </li> <li>  <code>ACTION_FAILED</code> - One or more of the specified <code>ProposalActions</code> in a proposal that was approved could not be completed because of an error. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProposedByMemberName</i> The name of the member that created the proposal.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProposedByMemberId</i> The unique identifier of the member that created the proposal.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Actions</i> The actions to perform on the network if the proposal is <code>APPROVED</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Actions</b>: The actions to perform on the network if the proposal is <code>APPROVED</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the proposal.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkId</i> The unique identifier of the network for which the proposal is made.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProposalId</i> The unique identifier of the proposal.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Removals</i>  The actions to perform for an <code>APPROVED</code> proposal to remove a member from the network, which deletes the member and all associated member resources from the network. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Invitations</i>  The actions to perform for an <code>APPROVED</code> proposal to invite an AWS account to create a member and join the network. <br/>        """
        pass

    def list_invitations(self, MaxResults: int=None, NextToken: str=None) -> ListInvitationsOutput:
        """Returns a listing of all invitations made on the specified network.

        :param self:
        :param NextToken:  The pagination token that indicates the next set of results to retrieve.<br/>MaxResults:  The maximum number of invitations to return.<br/>
        :return: NextToken:  The pagination token that indicates the next set of results to retrieve.<br/>Invitations:  The invitations for the network.<br/>        """
        pass

    def list_members(self, NetworkId: str, Name: str=None, Status: str=None, IsOwned: IsOwned=None, MaxResults: int=None, NextToken: str=None) -> ListMembersOutput:
        """Returns a listing of the members in a network and properties of their configurations.

        :param self:
        :param NextToken:  The pagination token that indicates the next set of results to retrieve.<br/>MaxResults:  The maximum number of members to return in the request.<br/>IsOwned:  An optional Boolean value. If provided, the request is limited either to members that the current AWS account owns (<code>true</code>) or that other AWS accounts own (<code>false</code>). If omitted, all members are listed.<br/>Status:  An optional status specifier. If provided, only members currently in this status are listed.<br/>Name:  The optional name of the member to list.<br/>NetworkId:  The unique identifier of the network for which to list members.<br/>
        :return: NextToken:  The pagination token that indicates the next set of results to retrieve.<br/>Members:  An array of <code>MemberSummary</code> objects. Each object contains details about a network member.<br/>        """
        pass

    def list_networks(self, Name: str=None, Framework: str=None, Status: str=None, MaxResults: int=None, NextToken: str=None) -> ListNetworksOutput:
        """Returns information about the networks in which the current AWS account has members.

        :param self:
        :param NextToken:  The pagination token that indicates the next set of results to retrieve.<br/>MaxResults:  The maximum number of networks to list.<br/>Status:  An optional status specifier. If provided, only networks currently in this status are listed.<br/>Framework:  An optional framework specifier. If provided, only networks of this framework type are listed.<br/>Name:  The name of the network.<br/>
        :return: NextToken:  The pagination token that indicates the next set of results to retrieve.<br/>Networks:  An array of <code>NetworkSummary</code> objects that contain configuration properties for each network.<br/>        """
        pass

    def list_nodes(self, MemberId: str, NetworkId: str, Status: str=None, MaxResults: int=None, NextToken: str=None) -> ListNodesOutput:
        """Returns information about the nodes within a network.

        :param self:
        :param NextToken:  The pagination token that indicates the next set of results to retrieve.<br/>MaxResults:  The maximum number of nodes to list.<br/>Status:  An optional status specifier. If provided, only nodes currently in this status are listed.<br/>MemberId:  The unique identifier of the member who owns the nodes to list.<br/>NetworkId:  The unique identifier of the network for which to list nodes.<br/>
        :return: NextToken:  The pagination token that indicates the next set of results to retrieve.<br/>Nodes:  An array of <code>NodeSummary</code> objects that contain configuration properties for each node.<br/>        """
        pass

    def list_proposal_votes(self, ProposalId: str, NetworkId: str, MaxResults: int=None, NextToken: str=None) -> ListProposalVotesOutput:
        """Returns the listing of votes for a specified proposal, including the value of each vote and the unique identifier of the member that cast the vote.

        :param self:
        :param NextToken:   The pagination token that indicates the next set of results to retrieve. <br/>MaxResults:   The maximum number of votes to return. <br/>ProposalId:   The unique identifier of the proposal. <br/>NetworkId:   The unique identifier of the network. <br/>
        :return: NextToken:   The pagination token that indicates the next set of results to retrieve. <br/>ProposalVotes:   The listing of votes. <br/>        """
        pass

    def list_proposals(self, NetworkId: str, MaxResults: int=None, NextToken: str=None) -> ListProposalsOutput:
        """Returns a listing of proposals for the network.

        :param self:
        :param NextToken:   The pagination token that indicates the next set of results to retrieve. <br/>MaxResults:   The maximum number of proposals to return. <br/>NetworkId:   The unique identifier of the network. <br/>
        :return: NextToken:  The pagination token that indicates the next set of results to retrieve.<br/>Proposals:  The summary of each proposal made on the network.<br/>        """
        pass

    def reject_invitation(self, InvitationId: str) -> RejectInvitationOutput:
        """Rejects an invitation to join a network. This action can be called by a principal in an AWS account that has received an invitation to create a member and join a network.

        :param self:
        :param InvitationId:  The unique identifier of the invitation to reject.<br/>
        :return:         """
        pass

    def vote_on_proposal(self, Vote: str, VoterMemberId: str, ProposalId: str, NetworkId: str) -> VoteOnProposalOutput:
        """Casts a vote for a specified <code>ProposalId</code> on behalf of a member. The member to vote as, specified by <code>VoterMemberId</code>, must be in the same AWS account as the principal that calls the action.

        :param self:
        :param Vote:   The value of the vote. <br/>VoterMemberId:  The unique identifier of the member casting the vote. <br/>ProposalId:   The unique identifier of the proposal. <br/>NetworkId:   The unique identifier of the network. <br/>
        :return:         """
        pass



class MarketplaceEntitlementService(BaseClient):
    """<fullname>AWS Marketplace Entitlement Service</fullname> This reference provides descriptions of the AWS Marketplace Entitlement Service API. AWS Marketplace Entitlement Service is used to determine the entitlement of a customer to a given product. An entitlement represents capacity in a product owned by the customer. For example, a customer might own some number of users or seats in an SaaS application or some amount of data capacity in a multi-tenant database.  <b>Getting Entitlement Records</b>  <ul> <li>  <i>GetEntitlements</i>- Gets the entitlements for a Marketplace product. </li> </ul>"""


    class GetEntitlementsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_entitlements` for valid parameters"""
            pass


    class GetEntitlementsRequest(Mapping):
        pass
    
    class GetEntitlementsResult(Mapping):
        pass
    
    class GetEntitlementFilters(object):
        pass
    
    def get_entitlements(self, ProductCode: str, Filter: GetEntitlementFilters=None, NextToken: str=None, MaxResults: int=None) -> GetEntitlementsResult:
        """GetEntitlements retrieves entitlement values for a given product. The results can be filtered based on customer identifier or product dimensions.

        :param self:
        :param MaxResults:  The maximum number of items to retrieve from the GetEntitlements operation. For pagination, use the NextToken field in subsequent calls to GetEntitlements.<br/>NextToken:  For paginated calls to GetEntitlements, pass the NextToken from the previous GetEntitlementsResult.<br/>Filter:  Filter is used to return entitlements for a specific customer or for a specific dimension. Filters are described as keys mapped to a lists of values. Filtered requests are <i>unioned</i> for each value in the value list, and then <i>intersected</i> for each filter key.<br/>ProductCode:  Product code is used to uniquely identify a product in AWS Marketplace. The product code will be provided by AWS Marketplace when the product listing is created.<br/>
        :return: NextToken:  For paginated results, use NextToken in subsequent calls to GetEntitlements. If the result contains an empty set of entitlements, NextToken might still be present and should be used.<br/>Entitlements:  The set of entitlements found through the GetEntitlements operation. If the result contains an empty set of entitlements, NextToken might still be present and should be used.<br/>        """
        pass



class MarketplaceCommerceAnalytics(BaseClient):
    """Provides AWS Marketplace business intelligence data on-demand."""




    class GenerateDataSetRequest(Mapping):
        pass
    
    class GenerateDataSetResult(Mapping):
        pass
    
    class DataSetPublicationDate(object):
        pass
    
    class CustomerDefinedValues(object):
        pass
    
    class StartSupportDataExportRequest(Mapping):
        pass
    
    class StartSupportDataExportResult(Mapping):
        pass
    
    class FromDate(object):
        pass
    
    def generate_data_set(self, snsTopicArn: str, destinationS3BucketName: str, roleNameArn: str, dataSetPublicationDate: DataSetPublicationDate, dataSetType: str, destinationS3Prefix: str=None, customerDefinedValues: CustomerDefinedValues=None) -> GenerateDataSetResult:
        """Given a data set type and data set publication date, asynchronously publishes the requested data set to the specified S3 bucket and notifies the specified SNS topic once the data is available. Returns a unique request identifier that can be used to correlate requests with notifications from the SNS topic. Data sets will be published in comma-separated values (CSV) format with the file name {data_set_type}_YYYY-MM-DD.csv. If a file with the same name already exists (e.g. if the same data set is requested twice), the original file will be overwritten by the new file. Requires a Role with an attached permissions policy providing Allow permissions for the following actions: s3:PutObject, s3:GetBucketLocation, sns:GetTopicAttributes, sns:Publish, iam:GetRolePolicy.

        :param self:
        :param customerDefinedValues:  (Optional) Key-value pairs which will be returned, unmodified, in the Amazon SNS notification message and the data set metadata file. These key-value pairs can be used to correlated responses with tracking information from other systems.<br/>snsTopicArn:  Amazon Resource Name (ARN) for the SNS Topic that will be notified when the data set has been published or if an error has occurred.<br/>destinationS3Prefix:  (Optional) The desired S3 prefix for the published data set, similar to a directory path in standard file systems. For example, if given the bucket name "mybucket" and the prefix "myprefix/mydatasets", the output file "outputfile" would be published to "s3://mybucket/myprefix/mydatasets/outputfile". If the prefix directory structure does not exist, it will be created. If no prefix is provided, the data set will be published to the S3 bucket root.<br/>destinationS3BucketName:  The name (friendly name, not ARN) of the destination S3 bucket.<br/>roleNameArn:  The Amazon Resource Name (ARN) of the Role with an attached permissions policy to interact with the provided AWS services.<br/>dataSetPublicationDate:  The date a data set was published. For daily data sets, provide a date with day-level granularity for the desired day. For weekly data sets, provide a date with day-level granularity within the desired week (the day value will be ignored). For monthly data sets, provide a date with month-level granularity for the desired month (the day value will be ignored).<br/>dataSetType:  The desired data set type.  <ul> <li> <strong>customer_subscriber_hourly_monthly_subscriptions</strong> From 2014-07-21 to present: Available daily by 5:00 PM Pacific Time. </li> <li> <strong>customer_subscriber_annual_subscriptions</strong> From 2014-07-21 to present: Available daily by 5:00 PM Pacific Time. </li> <li> <strong>daily_business_usage_by_instance_type</strong> From 2015-01-26 to present: Available daily by 5:00 PM Pacific Time. </li> <li> <strong>daily_business_fees</strong> From 2015-01-26 to present: Available daily by 5:00 PM Pacific Time. </li> <li> <strong>daily_business_free_trial_conversions</strong> From 2015-01-26 to present: Available daily by 5:00 PM Pacific Time. </li> <li> <strong>daily_business_new_instances</strong> From 2015-01-26 to present: Available daily by 5:00 PM Pacific Time. </li> <li> <strong>daily_business_new_product_subscribers</strong> From 2015-01-26 to present: Available daily by 5:00 PM Pacific Time. </li> <li> <strong>daily_business_canceled_product_subscribers</strong> From 2015-01-26 to present: Available daily by 5:00 PM Pacific Time. </li> <li> <strong>monthly_revenue_billing_and_revenue_data</strong> From 2015-02 to 2017-06: Available monthly on the 4th day of the month by 5:00pm Pacific Time. Data includes metered transactions (e.g. hourly) from two months prior. From 2017-07 to present: Available monthly on the 15th day of the month by 5:00pm Pacific Time. Data includes metered transactions (e.g. hourly) from one month prior. </li> <li> <strong>monthly_revenue_annual_subscriptions</strong> From 2015-02 to 2017-06: Available monthly on the 4th day of the month by 5:00pm Pacific Time. Data includes up-front software charges (e.g. annual) from one month prior. From 2017-07 to present: Available monthly on the 15th day of the month by 5:00pm Pacific Time. Data includes up-front software charges (e.g. annual) from one month prior. </li> <li> <strong>disbursed_amount_by_product</strong> From 2015-01-26 to present: Available every 30 days by 5:00 PM Pacific Time. </li> <li> <strong>disbursed_amount_by_product_with_uncollected_funds</strong> From 2012-04-19 to 2015-01-25: Available every 30 days by 5:00 PM Pacific Time. From 2015-01-26 to present: This data set was split into three data sets: disbursed_amount_by_product, disbursed_amount_by_age_of_uncollected_funds, and disbursed_amount_by_age_of_disbursed_funds. </li> <li> <strong>disbursed_amount_by_instance_hours</strong> From 2012-09-04 to present: Available every 30 days by 5:00 PM Pacific Time. </li> <li> <strong>disbursed_amount_by_customer_geo</strong> From 2012-04-19 to present: Available every 30 days by 5:00 PM Pacific Time. </li> <li> <strong>disbursed_amount_by_age_of_uncollected_funds</strong> From 2015-01-26 to present: Available every 30 days by 5:00 PM Pacific Time. </li> <li> <strong>disbursed_amount_by_age_of_disbursed_funds</strong> From 2015-01-26 to present: Available every 30 days by 5:00 PM Pacific Time. </li> <li> <strong>customer_profile_by_industry</strong> From 2015-10-01 to 2017-06-29: Available daily by 5:00 PM Pacific Time. From 2017-06-30 to present: This data set is no longer available. </li> <li> <strong>customer_profile_by_revenue</strong> From 2015-10-01 to 2017-06-29: Available daily by 5:00 PM Pacific Time. From 2017-06-30 to present: This data set is no longer available. </li> <li> <strong>customer_profile_by_geography</strong> From 2015-10-01 to 2017-06-29: Available daily by 5:00 PM Pacific Time. From 2017-06-30 to present: This data set is no longer available. </li> <li> <strong>sales_compensation_billed_revenue</strong> From 2016-12 to 2017-06: Available monthly on the 4th day of the month by 5:00pm Pacific Time. Data includes metered transactions (e.g. hourly) from two months prior, and up-front software charges (e.g. annual) from one month prior. From 2017-06 to present: Available monthly on the 15th day of the month by 5:00pm Pacific Time. Data includes metered transactions (e.g. hourly) from one month prior, and up-front software charges (e.g. annual) from one month prior. </li> <li> <strong>us_sales_and_use_tax_records</strong> From 2017-02-15 to present: Available monthly on the 15th day of the month by 5:00 PM Pacific Time. </li> </ul> <br/>
        :return: dataSetRequestId:  A unique identifier representing a specific request to the GenerateDataSet operation. This identifier can be used to correlate a request with notifications from the SNS topic.<br/>        """
        pass

    def start_support_data_export(self, snsTopicArn: str, destinationS3BucketName: str, roleNameArn: str, fromDate: FromDate, dataSetType: str, destinationS3Prefix: str=None, customerDefinedValues: CustomerDefinedValues=None) -> StartSupportDataExportResult:
        """Given a data set type and a from date, asynchronously publishes the requested customer support data to the specified S3 bucket and notifies the specified SNS topic once the data is available. Returns a unique request identifier that can be used to correlate requests with notifications from the SNS topic. Data sets will be published in comma-separated values (CSV) format with the file name {data_set_type}_YYYY-MM-DD'T'HH-mm-ss'Z'.csv. If a file with the same name already exists (e.g. if the same data set is requested twice), the original file will be overwritten by the new file. Requires a Role with an attached permissions policy providing Allow permissions for the following actions: s3:PutObject, s3:GetBucketLocation, sns:GetTopicAttributes, sns:Publish, iam:GetRolePolicy.

        :param self:
        :param customerDefinedValues:  (Optional) Key-value pairs which will be returned, unmodified, in the Amazon SNS notification message and the data set metadata file.<br/>snsTopicArn:  Amazon Resource Name (ARN) for the SNS Topic that will be notified when the data set has been published or if an error has occurred.<br/>destinationS3Prefix:  (Optional) The desired S3 prefix for the published data set, similar to a directory path in standard file systems. For example, if given the bucket name "mybucket" and the prefix "myprefix/mydatasets", the output file "outputfile" would be published to "s3://mybucket/myprefix/mydatasets/outputfile". If the prefix directory structure does not exist, it will be created. If no prefix is provided, the data set will be published to the S3 bucket root.<br/>destinationS3BucketName:  The name (friendly name, not ARN) of the destination S3 bucket.<br/>roleNameArn:  The Amazon Resource Name (ARN) of the Role with an attached permissions policy to interact with the provided AWS services.<br/>fromDate:  The start date from which to retrieve the data set in UTC. This parameter only affects the customer_support_contacts_data data set type.<br/>dataSetType:   Specifies the data set type to be written to the output csv file. The data set types customer_support_contacts_data and test_customer_support_contacts_data both result in a csv file containing the following fields: Product Id, Product Code, Customer Guid, Subscription Guid, Subscription Start Date, Organization, AWS Account Id, Given Name, Surname, Telephone Number, Email, Title, Country Code, ZIP Code, Operation Type, and Operation Time.   <ul> <li><i>customer_support_contacts_data</i> Customer support contact data. The data set will contain all changes (Creates, Updates, and Deletes) to customer support contact data from the date specified in the from_date parameter.</li> <li><i>test_customer_support_contacts_data</i> An example data set containing static test data in the same format as customer_support_contacts_data</li> </ul> <br/>
        :return: dataSetRequestId:  A unique identifier representing a specific request to the StartSupportDataExport operation. This identifier can be used to correlate a request with notifications from the SNS topic.<br/>        """
        pass



class MediaConnect(BaseClient):
    """API for AWS Elemental MediaConnect"""


    class ListFlowsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_flows` for valid parameters"""
            pass
    class ListEntitlementsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_entitlements` for valid parameters"""
            pass


    class AddFlowOutputsRequest(Mapping):
        pass
    
    class AddFlowOutputsResponse(Mapping):
        pass
    
    class AddOutputRequest(Mapping):
        pass
    
    class CreateFlowRequest(Mapping):
        pass
    
    class CreateFlowResponse(Mapping):
        pass
    
    class GrantEntitlementRequest(Mapping):
        pass
    
    class SetSourceRequest(Mapping):
        pass
    
    class DeleteFlowRequest(Mapping):
        pass
    
    class DeleteFlowResponse(Mapping):
        pass
    
    class DescribeFlowRequest(Mapping):
        pass
    
    class DescribeFlowResponse(Mapping):
        pass
    
    class GrantFlowEntitlementsRequest(Mapping):
        pass
    
    class GrantFlowEntitlementsResponse(Mapping):
        pass
    
    class ListEntitlementsRequest(Mapping):
        pass
    
    class ListEntitlementsResponse(Mapping):
        pass
    
    class ListFlowsRequest(Mapping):
        pass
    
    class ListFlowsResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class RemoveFlowOutputRequest(Mapping):
        pass
    
    class RemoveFlowOutputResponse(Mapping):
        pass
    
    class RevokeFlowEntitlementRequest(Mapping):
        pass
    
    class RevokeFlowEntitlementResponse(Mapping):
        pass
    
    class StartFlowRequest(Mapping):
        pass
    
    class StartFlowResponse(Mapping):
        pass
    
    class StopFlowRequest(Mapping):
        pass
    
    class StopFlowResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class __mapOf__string(object):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class __string(object):
        pass
    
    class UpdateFlowEntitlementRequest(Mapping):
        pass
    
    class UpdateFlowEntitlementResponse(Mapping):
        pass
    
    class UpdateEncryption(Mapping):
        pass
    
    class UpdateFlowOutputRequest(Mapping):
        pass
    
    class UpdateFlowOutputResponse(Mapping):
        pass
    
    class UpdateFlowSourceRequest(Mapping):
        pass
    
    class UpdateFlowSourceResponse(Mapping):
        pass
    
    def add_flow_outputs(self, Outputs: List[AddOutputRequest], FlowArn: str) -> AddFlowOutputsResponse:
        """Adds outputs to an existing flow. You can create up to 20 outputs per flow.

        :param self:
        :param Outputs:  A list of outputs that you want to add.<br/>FlowArn:  The flow that you want to add outputs to.<br/>
        :return: Outputs:  The details of the newly added outputs.<br/>FlowArn:  The ARN of the flow that these outputs were added to.<br/>        """
        pass

    def create_flow(self, Source: SetSourceRequest, Name: str, AvailabilityZone: str=None, Entitlements: List[GrantEntitlementRequest]=None, Outputs: List[AddOutputRequest]=None) -> CreateFlowResponse:
        """Creates a new flow. The request must include one source. The request optionally can include outputs (up to 20) and entitlements (up to 50).

        :param self:
        :param Source:  The settings for the source of the flow.:<br/>Outputs:  The outputs that you want to add to this flow.<br/>Name:  The name of the flow.<br/>Entitlements:  The entitlements that you want to grant on a flow.<br/>AvailabilityZone:  The Availability Zone that you want to create the flow in. These options are limited to the Availability Zones within the current AWS Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WhitelistCidr</i> The range of IP addresses that should be allowed to contribute content to your source. These IP addresses should in the form of a Classless Inter-Domain Routing (CIDR) block; for example, 10.0.0.0/16.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamId</i> The stream ID that you want to use for this transport. This parameter applies only to Zixi-based streams.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Protocol</i> The protocol that is used by the source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxLatency</i> The maximum latency in milliseconds for Zixi-based streams.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxBitrate</i> The smoothing max bitrate for RTP and RTP-FEC streams.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IngestPort</i> The port that the flow will be listening on for incoming content.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EntitlementArn</i> The ARN of the entitlement that allows you to subscribe to this flow. The entitlement is set by the flow originator, and the ARN is generated as part of the originator's flow.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description for the source. This value is not used or seen outside of the current AWS Elemental MediaConnect account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Decryption</i> The type of encryption that is used on the content ingested from this source.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Decryption</b>: The type of encryption that is used on the content ingested from this source.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Url</i> The URL from the API Gateway proxy that you set up to talk to your key server. This parameter is required for SPEKE encryption and is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecretArn</i> The ARN of the secret that you created in AWS Secrets Manager to store the encryption key. This parameter is required for static key encryption and is not valid for SPEKE encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The ARN of the role that you created during setup (when you set up AWS Elemental MediaConnect as a trusted entity).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceId</i> An identifier for the content. The service sends this value to the key server to identify the current endpoint. The resource ID is also known as the content ID. This parameter is required for SPEKE encryption and is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Region</i> The AWS Region that the API Gateway proxy endpoint was created in. This parameter is required for SPEKE encryption and is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyType</i> The type of key that is used for the encryption. If no keyType is provided, the service will use the default setting (static-key).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceId</i> The value of one of the devices that you configured with your digital rights management (DRM) platform key provider. This parameter is required for SPEKE encryption and is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ConstantInitializationVector</i> A 128-bit, 16-byte hex value represented by a 32-character string, to be used with the key for encrypting content. This parameter is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Algorithm</i> The type of algorithm that is used for the encryption (such as aes128, aes192, or aes256).<br/>
        :return: Flow:  The settings for a flow, including its source, outputs, and entitlements.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current status of the flow.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Source</i> The settings for the source of the flow.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Source</b>: The settings for the source of the flow.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Outputs</i> The outputs in this flow.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the flow.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FlowArn</i> The Amazon Resource Name (ARN), a unique identifier for any AWS resource, of the flow.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Entitlements</i> The entitlements in this flow.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EgressIp</i> The IP address from which video will be sent to output destinations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of the flow. This value is not used or seen outside of the current AWS Elemental MediaConnect account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone that you want to create the flow in. These options are limited to the Availability Zones within the current AWS.<br/>The type of encryption that is used on the content ingested from this source.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Decryption</b>: The type of encryption that is used on the content ingested from this source.        """
        pass

    def delete_flow(self, FlowArn: str) -> DeleteFlowResponse:
        """Deletes a flow. Before you can delete a flow, you must stop the flow.

        :param self:
        :param FlowArn:  The ARN of the flow that you want to delete.<br/>
        :return: Status:  The status of the flow when the DeleteFlow process begins.<br/>FlowArn:  The ARN of the flow that was deleted.<br/>        """
        pass

    def describe_flow(self, FlowArn: str) -> DescribeFlowResponse:
        """Displays the details of a flow. The response includes the flow ARN, name, and Availability Zone, as well as details about the source, outputs, and entitlements.

        :param self:
        :param FlowArn:  The ARN of the flow that you want to describe.<br/>
        :return: Messages:  Messages that provide the state of the flow.:<br/>Flow:  The settings for a flow, including its source, outputs, and entitlements.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current status of the flow.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Source</i> The settings for the source of the flow.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Source</b>: The settings for the source of the flow.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Outputs</i> The outputs in this flow.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the flow.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FlowArn</i> The Amazon Resource Name (ARN), a unique identifier for any AWS resource, of the flow.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Entitlements</i> The entitlements in this flow.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EgressIp</i> The IP address from which video will be sent to output destinations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of the flow. This value is not used or seen outside of the current AWS Elemental MediaConnect account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone that you want to create the flow in. These options are limited to the Availability Zones within the current AWS.<br/>The type of encryption that is used on the content ingested from this source.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Decryption</b>: The type of encryption that is used on the content ingested from this source.&nbsp;&nbsp;&nbsp;&nbsp; <i>Errors</i> A list of errors that might have been generated from processes on this flow.<br/>        """
        pass

    def grant_flow_entitlements(self, FlowArn: str, Entitlements: List[GrantEntitlementRequest]) -> GrantFlowEntitlementsResponse:
        """Grants entitlements to an existing flow.

        :param self:
        :param FlowArn:  The flow that you want to grant entitlements on.<br/>Entitlements:  The list of entitlements that you want to grant.<br/>
        :return: FlowArn:  The ARN of the flow that these entitlements were granted to.<br/>Entitlements:  The entitlements that were just granted.<br/>        """
        pass

    def list_entitlements(self, MaxResults: int=None, NextToken: str=None) -> ListEntitlementsResponse:
        """Displays a list of all entitlements that have been granted to this account. This request returns 20 results per page.

        :param self:
        :param NextToken:  The token that identifies which batch of results that you want to see. For example, you submit a ListEntitlements request with MaxResults set at 5. The service returns the first batch of results (up to 5) and a NextToken value. To see the next batch of results, you can submit the ListEntitlements request a second time and specify the NextToken value.<br/>MaxResults:  The maximum number of results to return per API request. For example, you submit a ListEntitlements request with MaxResults set at 5. Although 20 items match your request, the service returns no more than the first 5 items. (The service also returns a NextToken value that you can use to fetch the next batch of results.) The service might return fewer results than the MaxResults value. If MaxResults is not included in the request, the service defaults to pagination with a maximum of 20 results per page.<br/>
        :return: NextToken:  The token that identifies which batch of results that you want to see. For example, you submit a ListEntitlements request with MaxResults set at 5. The service returns the first batch of results (up to 5) and a NextToken value. To see the next batch of results, you can submit the ListEntitlements request a second time and specify the NextToken value.<br/>Entitlements:  A list of entitlements that have been granted to you from other AWS accounts.<br/>        """
        pass

    def list_flows(self, MaxResults: int=None, NextToken: str=None) -> ListFlowsResponse:
        """Displays a list of flows that are associated with this account. This request returns a paginated result.

        :param self:
        :param NextToken:  The token that identifies which batch of results that you want to see. For example, you submit a ListFlows request with MaxResults set at 5. The service returns the first batch of results (up to 5) and a NextToken value. To see the next batch of results, you can submit the ListFlows request a second time and specify the NextToken value.<br/>MaxResults:  The maximum number of results to return per API request. For example, you submit a ListFlows request with MaxResults set at 5. Although 20 items match your request, the service returns no more than the first 5 items. (The service also returns a NextToken value that you can use to fetch the next batch of results.) The service might return fewer results than the MaxResults value. If MaxResults is not included in the request, the service defaults to pagination with a maximum of 10 results per page.<br/>
        :return: NextToken:  The token that identifies which batch of results that you want to see. For example, you submit a ListFlows request with MaxResults set at 5. The service returns the first batch of results (up to 5) and a NextToken value. To see the next batch of results, you can submit the ListFlows request a second time and specify the NextToken value.<br/>Flows:  A list of flow summaries.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceArn: str) -> ListTagsForResourceResponse:
        """List all tags on an AWS Elemental MediaConnect resource

        :param self:
        :param ResourceArn:  The Amazon Resource Name (ARN) that identifies the AWS Elemental MediaConnect resource for which to list the tags.<br/>
        :return: Tags:  A map from tag keys to values. Tag keys can have a maximum character length of 128 characters, and tag values can have a maximum length of 256 characters.<br/>        """
        pass

    def remove_flow_output(self, OutputArn: str, FlowArn: str) -> RemoveFlowOutputResponse:
        """Removes an output from an existing flow. This request can be made only on an output that does not have an entitlement associated with it. If the output has an entitlement, you must revoke the entitlement instead. When an entitlement is revoked from a flow, the service automatically removes the associated output.

        :param self:
        :param OutputArn:  The ARN of the output that you want to remove.<br/>FlowArn:  The flow that you want to remove an output from.<br/>
        :return: OutputArn:  The ARN of the output that was removed.<br/>FlowArn:  The ARN of the flow that is associated with the output you removed.<br/>        """
        pass

    def revoke_flow_entitlement(self, FlowArn: str, EntitlementArn: str) -> RevokeFlowEntitlementResponse:
        """Revokes an entitlement from a flow. Once an entitlement is revoked, the content becomes unavailable to the subscriber and the associated output is removed.

        :param self:
        :param FlowArn:  The flow that you want to revoke an entitlement from.<br/>EntitlementArn:  The ARN of the entitlement that you want to revoke.<br/>
        :return: FlowArn:  The ARN of the flow that the entitlement was revoked from.<br/>EntitlementArn:  The ARN of the entitlement that was revoked.<br/>        """
        pass

    def start_flow(self, FlowArn: str) -> StartFlowResponse:
        """Starts a flow.

        :param self:
        :param FlowArn:  The ARN of the flow that you want to start.<br/>
        :return: Status:  The status of the flow when the StartFlow process begins.<br/>FlowArn:  The ARN of the flow that you started.<br/>        """
        pass

    def stop_flow(self, FlowArn: str) -> StopFlowResponse:
        """Stops a flow.

        :param self:
        :param FlowArn:  The ARN of the flow that you want to stop.<br/>
        :return: Status:  The status of the flow when the StopFlow process begins.<br/>FlowArn:  The ARN of the flow that you stopped.<br/>        """
        pass

    def tag_resource(self, Tags: __mapOf__string, ResourceArn: str):
        """Associates the specified tags to a resource with the specified resourceArn. If existing tags on a resource are not specified in the request parameters, they are not changed. When a resource is deleted, the tags associated with that resource are deleted as well.

        :param self:
        :param Tags:  A map from tag keys to values. Tag keys can have a maximum character length of 128 characters, and tag values can have a maximum length of 256 characters.<br/>ResourceArn:  The Amazon Resource Name (ARN) that identifies the AWS Elemental MediaConnect resource to which to add tags.<br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[__string], ResourceArn: str):
        """Deletes specified tags from a resource.

        :param self:
        :param TagKeys:  The keys of the tags to be removed.<br/>ResourceArn:  The Amazon Resource Name (ARN) that identifies the AWS Elemental MediaConnect resource from which to delete tags.<br/>
        :return:         """
        pass

    def update_flow_entitlement(self, FlowArn: str, EntitlementArn: str, Description: str=None, Encryption: UpdateEncryption=None, Subscribers: List[__string]=None) -> UpdateFlowEntitlementResponse:
        """You can change an entitlement's description, subscribers, and encryption. If you change the subscribers, the service will remove the outputs that are are used by the subscribers that are removed.

        :param self:
        :param Subscribers:  The AWS account IDs that you want to share your content with. The receiving accounts (subscribers) will be allowed to create their own flow using your content as the source.<br/>FlowArn:  The flow that is associated with the entitlement that you want to update.<br/>EntitlementArn:  The ARN of the entitlement that you want to update.<br/>Encryption:  The type of encryption that will be used on the output associated with this entitlement.:<br/>Description:  A description of the entitlement. This description appears only on the AWS Elemental MediaConnect console and will not be seen by the subscriber or end user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Url</i> The URL from the API Gateway proxy that you set up to talk to your key server. This parameter is required for SPEKE encryption and is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecretArn</i> The ARN of the secret that you created in AWS Secrets Manager to store the encryption key. This parameter is required for static key encryption and is not valid for SPEKE encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The ARN of the role that you created during setup (when you set up AWS Elemental MediaConnect as a trusted entity).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceId</i> An identifier for the content. The service sends this value to the key server to identify the current endpoint. The resource ID is also known as the content ID. This parameter is required for SPEKE encryption and is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Region</i> The AWS Region that the API Gateway proxy endpoint was created in. This parameter is required for SPEKE encryption and is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyType</i> The type of key that is used for the encryption. If no keyType is provided, the service will use the default setting (static-key).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceId</i> The value of one of the devices that you configured with your digital rights management (DRM) platform key provider. This parameter is required for SPEKE encryption and is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConstantInitializationVector</i> A 128-bit, 16-byte hex value represented by a 32-character string, to be used with the key for encrypting content. This parameter is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Algorithm</i> The type of algorithm that is used for the encryption (such as aes128, aes192, or aes256).<br/>
        :return: FlowArn:  The ARN of the flow that this entitlement was granted on.<br/>Entitlement:  The settings for a flow entitlement.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subscribers</i> The AWS account IDs that you want to share your content with. The receiving accounts (subscribers) will be allowed to create their own flow using your content as the source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the entitlement.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EntitlementArn</i> The ARN of the entitlement.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> The type of encryption that will be used on the output that is associated with this entitlement.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: The type of encryption that will be used on the output that is associated with this entitlement.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of the entitlement.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Url</i> The URL from the API Gateway proxy that you set up to talk to your key server. This parameter is required for SPEKE encryption and is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecretArn</i> The ARN of the secret that you created in AWS Secrets Manager to store the encryption key. This parameter is required for static key encryption and is not valid for SPEKE encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The ARN of the role that you created during setup (when you set up AWS Elemental MediaConnect as a trusted entity).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceId</i> An identifier for the content. The service sends this value to the key server to identify the current endpoint. The resource ID is also known as the content ID. This parameter is required for SPEKE encryption and is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Region</i> The AWS Region that the API Gateway proxy endpoint was created in. This parameter is required for SPEKE encryption and is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyType</i> The type of key that is used for the encryption. If no keyType is provided, the service will use the default setting (static-key).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceId</i> The value of one of the devices that you configured with your digital rights management (DRM) platform key provider. This parameter is required for SPEKE encryption and is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ConstantInitializationVector</i> A 128-bit, 16-byte hex value represented by a 32-character string, to be used with the key for encrypting content. This parameter is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Algorithm</i> The type of algorithm that is used for the encryption (such as aes128, aes192, or aes256).<br/>        """
        pass

    def update_flow_output(self, OutputArn: str, FlowArn: str, Description: str=None, Destination: str=None, Encryption: UpdateEncryption=None, MaxLatency: int=None, Port: int=None, Protocol: str=None, SmoothingLatency: int=None, StreamId: str=None) -> UpdateFlowOutputResponse:
        """Updates an existing flow output.

        :param self:
        :param StreamId:  The stream ID that you want to use for this transport. This parameter applies only to Zixi-based streams.<br/>SmoothingLatency:  The smoothing latency in milliseconds for RTP and RTP-FEC streams.<br/>Protocol:  The protocol to use for the output.<br/>Port:  The port to use when content is distributed to this output.<br/>OutputArn:  The ARN of the output that you want to update.<br/>MaxLatency:  The maximum latency in milliseconds for Zixi-based streams.<br/>FlowArn:  The flow that is associated with the output that you want to update.<br/>Encryption:  The type of key used for the encryption. If no keyType is provided, the service will use the default setting (static-key).:<br/>Destination:  The IP address where you want to send the output.<br/>Description:  A description of the output. This description appears only on the AWS Elemental MediaConnect console and will not be seen by the end user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Url</i> The URL from the API Gateway proxy that you set up to talk to your key server. This parameter is required for SPEKE encryption and is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecretArn</i> The ARN of the secret that you created in AWS Secrets Manager to store the encryption key. This parameter is required for static key encryption and is not valid for SPEKE encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The ARN of the role that you created during setup (when you set up AWS Elemental MediaConnect as a trusted entity).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceId</i> An identifier for the content. The service sends this value to the key server to identify the current endpoint. The resource ID is also known as the content ID. This parameter is required for SPEKE encryption and is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Region</i> The AWS Region that the API Gateway proxy endpoint was created in. This parameter is required for SPEKE encryption and is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyType</i> The type of key that is used for the encryption. If no keyType is provided, the service will use the default setting (static-key).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceId</i> The value of one of the devices that you configured with your digital rights management (DRM) platform key provider. This parameter is required for SPEKE encryption and is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConstantInitializationVector</i> A 128-bit, 16-byte hex value represented by a 32-character string, to be used with the key for encrypting content. This parameter is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Algorithm</i> The type of algorithm that is used for the encryption (such as aes128, aes192, or aes256).<br/>
        :return: Output:  The settings for an output.:<br/>FlowArn:  The ARN of the flow that is associated with the updated output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Transport</i> Attributes related to the transport stream that are used in the output.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Transport</b>: Attributes related to the transport stream that are used in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port to use when content is distributed to this output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputArn</i> The ARN of the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the output. This value must be unique within the current flow.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MediaLiveInputArn</i> The input ARN of the AWS Elemental MediaLive channel. This parameter is relevant only for outputs that were added by creating a MediaLive input.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EntitlementArn</i> The ARN of the entitlement on the originator''s flow. This value is relevant only on entitled flows.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> The type of key used for the encryption. If no keyType is provided, the service will use the default setting (static-key).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: The type of key used for the encryption. If no keyType is provided, the service will use the default setting (static-key).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Destination</i> The address where you want to send the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Url</i> The URL from the API Gateway proxy that you set up to talk to your key server. This parameter is required for SPEKE encryption and is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecretArn</i> The ARN of the secret that you created in AWS Secrets Manager to store the encryption key. This parameter is required for static key encryption and is not valid for SPEKE encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The ARN of the role that you created during setup (when you set up AWS Elemental MediaConnect as a trusted entity).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceId</i> An identifier for the content. The service sends this value to the key server to identify the current endpoint. The resource ID is also known as the content ID. This parameter is required for SPEKE encryption and is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Region</i> The AWS Region that the API Gateway proxy endpoint was created in. This parameter is required for SPEKE encryption and is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyType</i> The type of key that is used for the encryption. If no keyType is provided, the service will use the default setting (static-key).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceId</i> The value of one of the devices that you configured with your digital rights management (DRM) platform key provider. This parameter is required for SPEKE encryption and is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ConstantInitializationVector</i> A 128-bit, 16-byte hex value represented by a 32-character string, to be used with the key for encrypting content. This parameter is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Algorithm</i> The type of algorithm that is used for the encryption (such as aes128, aes192, or aes256).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamId</i> The stream ID that you want to use for this transport. This parameter applies only to Zixi-based streams.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SmoothingLatency</i> The smoothing latency in milliseconds for RTP and RTP-FEC streams.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Protocol</i> The protocol that is used by the source or output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxLatency</i> The maximum latency in milliseconds for Zixi-based streams.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxBitrate</i> The smoothing max bitrate for RTP and RTP-FEC streams.<br/>        """
        pass

    def update_flow_source(self, SourceArn: str, FlowArn: str, Decryption: UpdateEncryption=None, Description: str=None, EntitlementArn: str=None, IngestPort: int=None, MaxBitrate: int=None, MaxLatency: int=None, Protocol: str=None, StreamId: str=None, WhitelistCidr: str=None) -> UpdateFlowSourceResponse:
        """Updates the source of a flow.

        :param self:
        :param WhitelistCidr:  The range of IP addresses that should be allowed to contribute content to your source. These IP addresses should in the form of a Classless Inter-Domain Routing (CIDR) block; for example, 10.0.0.0/16.<br/>StreamId:  The stream ID that you want to use for this transport. This parameter applies only to Zixi-based streams.<br/>SourceArn:  The ARN of the source that you want to update.<br/>Protocol:  The protocol that is used by the source.<br/>MaxLatency:  The maximum latency in milliseconds for Zixi-based streams.<br/>MaxBitrate:  The smoothing max bitrate for RTP and RTP-FEC streams.<br/>IngestPort:  The port that the flow will be listening on for incoming content.<br/>FlowArn:  The flow that is associated with the source that you want to update.<br/>EntitlementArn:  The ARN of the entitlement that allows you to subscribe to this flow. The entitlement is set by the flow originator, and the ARN is generated as part of the originator's flow.<br/>Description:  A description for the source. This value is not used or seen outside of the current AWS Elemental MediaConnect account.<br/>Decryption:  The type of encryption used on the content ingested from this source.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Url</i> The URL from the API Gateway proxy that you set up to talk to your key server. This parameter is required for SPEKE encryption and is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecretArn</i> The ARN of the secret that you created in AWS Secrets Manager to store the encryption key. This parameter is required for static key encryption and is not valid for SPEKE encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The ARN of the role that you created during setup (when you set up AWS Elemental MediaConnect as a trusted entity).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceId</i> An identifier for the content. The service sends this value to the key server to identify the current endpoint. The resource ID is also known as the content ID. This parameter is required for SPEKE encryption and is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Region</i> The AWS Region that the API Gateway proxy endpoint was created in. This parameter is required for SPEKE encryption and is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyType</i> The type of key that is used for the encryption. If no keyType is provided, the service will use the default setting (static-key).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceId</i> The value of one of the devices that you configured with your digital rights management (DRM) platform key provider. This parameter is required for SPEKE encryption and is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConstantInitializationVector</i> A 128-bit, 16-byte hex value represented by a 32-character string, to be used with the key for encrypting content. This parameter is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Algorithm</i> The type of algorithm that is used for the encryption (such as aes128, aes192, or aes256).<br/>
        :return: Source:  The settings for the source of the flow.:<br/>FlowArn:  The ARN of the flow that you want to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WhitelistCidr</i> The range of IP addresses that should be allowed to contribute content to your source. These IP addresses should in the form of a Classless Inter-Domain Routing (CIDR) block; for example, 10.0.0.0/16.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Transport</i> Attributes related to the transport stream that are used in the source.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Transport</b>: Attributes related to the transport stream that are used in the source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceArn</i> The ARN of the source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IngestPort</i> The port that the flow will be listening on for incoming content.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IngestIp</i> The IP address that the flow will be listening on for incoming content.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EntitlementArn</i> The ARN of the entitlement that allows you to subscribe to content that comes from another AWS account. The entitlement is set by the content originator and the ARN is generated as part of the originator's flow.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description for the source. This value is not used or seen outside of the current AWS Elemental MediaConnect account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Decryption</i> The type of encryption that is used on the content ingested from this source.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Decryption</b>: The type of encryption that is used on the content ingested from this source.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Url</i> The URL from the API Gateway proxy that you set up to talk to your key server. This parameter is required for SPEKE encryption and is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecretArn</i> The ARN of the secret that you created in AWS Secrets Manager to store the encryption key. This parameter is required for static key encryption and is not valid for SPEKE encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The ARN of the role that you created during setup (when you set up AWS Elemental MediaConnect as a trusted entity).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceId</i> An identifier for the content. The service sends this value to the key server to identify the current endpoint. The resource ID is also known as the content ID. This parameter is required for SPEKE encryption and is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Region</i> The AWS Region that the API Gateway proxy endpoint was created in. This parameter is required for SPEKE encryption and is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyType</i> The type of key that is used for the encryption. If no keyType is provided, the service will use the default setting (static-key).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DeviceId</i> The value of one of the devices that you configured with your digital rights management (DRM) platform key provider. This parameter is required for SPEKE encryption and is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ConstantInitializationVector</i> A 128-bit, 16-byte hex value represented by a 32-character string, to be used with the key for encrypting content. This parameter is not valid for static key encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Algorithm</i> The type of algorithm that is used for the encryption (such as aes128, aes192, or aes256).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamId</i> The stream ID that you want to use for this transport. This parameter applies only to Zixi-based streams.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SmoothingLatency</i> The smoothing latency in milliseconds for RTP and RTP-FEC streams.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Protocol</i> The protocol that is used by the source or output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxLatency</i> The maximum latency in milliseconds for Zixi-based streams.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxBitrate</i> The smoothing max bitrate for RTP and RTP-FEC streams.<br/>        """
        pass



class MediaConvert(BaseClient):
    """AWS Elemental MediaConvert"""


    class DescribeEndpointsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_endpoints` for valid parameters"""
            pass
    class ListJobsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_jobs` for valid parameters"""
            pass
    class ListPresetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_presets` for valid parameters"""
            pass
    class ListJobTemplatesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_job_templates` for valid parameters"""
            pass
    class ListQueuesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_queues` for valid parameters"""
            pass


    class AssociateCertificateRequest(Mapping):
        pass
    
    class AssociateCertificateResponse(Mapping):
        pass
    
    class CancelJobRequest(Mapping):
        pass
    
    class CancelJobResponse(Mapping):
        pass
    
    class CreateJobRequest(Mapping):
        pass
    
    class CreateJobResponse(Mapping):
        pass
    
    class AccelerationSettings(Mapping):
        pass
    
    class JobSettings(Mapping):
        pass
    
    class __mapOf__string(object):
        pass
    
    class CreateJobTemplateRequest(Mapping):
        pass
    
    class CreateJobTemplateResponse(Mapping):
        pass
    
    class JobTemplateSettings(Mapping):
        pass
    
    class CreatePresetRequest(Mapping):
        pass
    
    class CreatePresetResponse(Mapping):
        pass
    
    class PresetSettings(Mapping):
        pass
    
    class CreateQueueRequest(Mapping):
        pass
    
    class CreateQueueResponse(Mapping):
        pass
    
    class ReservationPlanSettings(Mapping):
        pass
    
    class DeleteJobTemplateRequest(Mapping):
        pass
    
    class DeleteJobTemplateResponse(Mapping):
        pass
    
    class DeletePresetRequest(Mapping):
        pass
    
    class DeletePresetResponse(Mapping):
        pass
    
    class DeleteQueueRequest(Mapping):
        pass
    
    class DeleteQueueResponse(Mapping):
        pass
    
    class DescribeEndpointsRequest(Mapping):
        pass
    
    class DescribeEndpointsResponse(Mapping):
        pass
    
    class DisassociateCertificateRequest(Mapping):
        pass
    
    class DisassociateCertificateResponse(Mapping):
        pass
    
    class GetJobRequest(Mapping):
        pass
    
    class GetJobResponse(Mapping):
        pass
    
    class GetJobTemplateRequest(Mapping):
        pass
    
    class GetJobTemplateResponse(Mapping):
        pass
    
    class GetPresetRequest(Mapping):
        pass
    
    class GetPresetResponse(Mapping):
        pass
    
    class GetQueueRequest(Mapping):
        pass
    
    class GetQueueResponse(Mapping):
        pass
    
    class ListJobTemplatesRequest(Mapping):
        pass
    
    class ListJobTemplatesResponse(Mapping):
        pass
    
    class ListJobsRequest(Mapping):
        pass
    
    class ListJobsResponse(Mapping):
        pass
    
    class ListPresetsRequest(Mapping):
        pass
    
    class ListPresetsResponse(Mapping):
        pass
    
    class ListQueuesRequest(Mapping):
        pass
    
    class ListQueuesResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class __string(object):
        pass
    
    class UpdateJobTemplateRequest(Mapping):
        pass
    
    class UpdateJobTemplateResponse(Mapping):
        pass
    
    class UpdatePresetRequest(Mapping):
        pass
    
    class UpdatePresetResponse(Mapping):
        pass
    
    class UpdateQueueRequest(Mapping):
        pass
    
    class UpdateQueueResponse(Mapping):
        pass
    
    def associate_certificate(self, Arn: str) -> AssociateCertificateResponse:
        """Associates an AWS Certificate Manager (ACM) Amazon Resource Name (ARN) with AWS Elemental MediaConvert.

        :param self:
        :param Arn:  The ARN of the ACM certificate that you want to associate with your MediaConvert resource.<br/>
        :return:         """
        pass

    def cancel_job(self, Id: str) -> CancelJobResponse:
        """Permanently cancel a job. Once you have canceled a job, you can't start it again.

        :param self:
        :param Id:  The Job ID of the job to be cancelled.<br/>
        :return:         """
        pass

    def create_job(self, Settings: JobSettings, Role: str, AccelerationSettings: AccelerationSettings=None, BillingTagsSource: str=None, ClientRequestToken: str=None, JobTemplate: str=None, Queue: str=None, StatusUpdateInterval: str=None, UserMetadata: __mapOf__string=None) -> CreateJobResponse:
        """Create a new transcoding job. For information about jobs and job settings, see the User Guide at http://docs.aws.amazon.com/mediaconvert/latest/ug/what-is.html

        :param self:
        :param UserMetadata:  User-defined metadata that you want to associate with an MediaConvert job. You specify metadata in key/value pairs.<br/>StatusUpdateInterval:  Specify how often MediaConvert sends STATUS_UPDATE events to Amazon CloudWatch Events. Set the interval, in seconds, between status updates. MediaConvert sends an update at this interval from the time the service begins processing your job to the time it completes the transcode or encounters an error.<br/>Settings:  JobSettings contains all the transcode settings for a job.:<br/>Role:  Required. The IAM role you use for creating this job. For details about permissions, see the User Guide topic at the User Guide at http://docs.aws.amazon.com/mediaconvert/latest/ug/iam-role.html.<br/>Queue:  Optional. When you create a job, you can specify a queue to send it to. If you don't specify, the job will go to the default queue. For more about queues, see the User Guide topic at http://docs.aws.amazon.com/mediaconvert/latest/ug/what-is.html.<br/>JobTemplate:  When you create a job, you can either specify a job template or specify the transcoding settings individually<br/>ClientRequestToken:  Idempotency token for CreateJob operation.<br/>BillingTagsSource:  Optional. Choose a tag type that AWS Billing and Cost Management will use to sort your AWS Elemental MediaConvert costs on any billing report that you set up. Any transcoding outputs that don't have an associated tag will appear in your billing report unsorted. If you don't choose a valid value for this field, your job outputs will appear on the billing report unsorted.<br/>AccelerationSettings:  Accelerated transcoding can significantly speed up jobs with long, visually complex content. Outputs that use this feature incur pro-tier pricing. For information about feature limitations, see the AWS Elemental MediaConvert User Guide.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> Acceleration configuration for the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimedMetadataInsertion</i> Enable Timed metadata insertion (TimedMetadataInsertion) to include ID3 tags in your job. To include timed metadata, you must enable it here, enable it in each output container, and specify tags and timecodes in ID3 insertion (Id3Insertion) objects.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TimedMetadataInsertion</b>: Enable Timed metadata insertion (TimedMetadataInsertion) to include ID3 tags in your job. To include timed metadata, you must enable it here, enable it in each output container, and specify tags and timecodes in ID3 insertion (Id3Insertion) objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimecodeConfig</i> Contains settings used to acquire and adjust timecode information from inputs.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TimecodeConfig</b>: Contains settings used to acquire and adjust timecode information from inputs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputGroups</i> (OutputGroups) contains one group of settings for each set of outputs that share a common package type. All unpackaged files (MPEG-4, MPEG-2 TS, Quicktime, MXF, and no container) are grouped in a single output group as well. Required in (OutputGroups) is a group of settings that apply to the whole group. This required object depends on the value you set for (Type) under (OutputGroups)>(OutputGroupSettings). Type, settings object pairs are as follows. * FILE_GROUP_SETTINGS, FileGroupSettings * HLS_GROUP_SETTINGS, HlsGroupSettings * DASH_ISO_GROUP_SETTINGS, DashIsoGroupSettings * MS_SMOOTH_GROUP_SETTINGS, MsSmoothGroupSettings * CMAF_GROUP_SETTINGS, CmafGroupSettings<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NielsenConfiguration</i> Settings for Nielsen Configuration&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NielsenConfiguration</b>: Settings for Nielsen Configuration<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MotionImageInserter</i> Overlay motion graphics on top of your video. The motion graphics that you specify here appear on all outputs in all output groups.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>MotionImageInserter</b>: Overlay motion graphics on top of your video. The motion graphics that you specify here appear on all outputs in all output groups.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Inputs</i> Use Inputs (inputs) to define source file used in the transcode job. There can be multiple inputs add in a job. These inputs will be concantenated together to create the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Esam</i> Settings for Event Signaling And Messaging (ESAM).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Esam</b>: Settings for Event Signaling And Messaging (ESAM).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailBlanking</i> Settings for ad avail blanking.  Video can be blanked or overlaid with an image, and audio muted during SCTE-35 triggered ad avails.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailBlanking</b>: Settings for ad avail blanking.  Video can be blanked or overlaid with an image, and audio muted during SCTE-35 triggered ad avails.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdAvailOffset</i> When specified, this offset (in milliseconds) is added to the input Ad Avail PTS time.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailBlankingImage</i> Blanking image to be used. Leave empty for solid black. Only bmp and png images are supported.<br/>Specifies an ESAM ManifestConfirmConditionNotification XML as per OC-SP-ESAM-API-I03-131025. The transcoder uses the manifest conditioning instructions that you provide in the setting MCC XML (mccXml).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ManifestConfirmConditionNotification</b>: Specifies an ESAM ManifestConfirmConditionNotification XML as per OC-SP-ESAM-API-I03-131025. The transcoder uses the manifest conditioning instructions that you provide in the setting MCC XML (mccXml).If your motion graphic asset is a .mov file, keep this setting unspecified. If your motion graphic asset is a series of .png files, specify the frame rate of the overlay in frames per second, as a fraction. For example, specify 24 fps as 24/1. Make sure that the number of images in your series matches the frame rate and your intended overlay duration. For example, if you want a 30-second overlay at 30 fps, you should have 900 .png images. This overlay frame rate doesn't need to match the frame rate of the underlying video.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Framerate</b>: If your motion graphic asset is a .mov file, keep this setting unspecified. If your motion graphic asset is a series of .png files, specify the frame rate of the overlay in frames per second, as a fraction. For example, specify 24 fps as 24/1. Make sure that the number of images in your series matches the frame rate and your intended overlay duration. For example, if you want a 30-second overlay at 30 fps, you should have 900 .png images. This overlay frame rate doesn't need to match the frame rate of the underlying video.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DistributorId</i> Use Distributor ID (DistributorID) to specify the distributor ID that is assigned to your organization by Neilsen.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BreakoutCode</i> Use Nielsen Configuration (NielsenConfiguration) to set the Nielsen measurement system breakout code. Supported values are 0, 3, 7, and 9.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TimestampOffset</i> Only applies to outputs that support program-date-time stamp. Use Timestamp offset (TimestampOffset) to overwrite the timecode date without affecting the time and frame number. Provide the new date as a string in the format "yyyy-mm-dd".  To use Time stamp offset, you must also enable Insert program-date-time (InsertProgramDateTime) in the output settings. For example, if the date part of your timecodes is 2002-1-25 and you want to change it to one year later, set Timestamp offset (TimestampOffset) to 2003-1-25.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Start</i> Only use when you set Source (TimecodeSource) to Specified start (SPECIFIEDSTART). Use Start timecode (Start) to specify the timecode for the initial frame. Use 24-hour format with frame number, (HH:MM:SS:FF) or (HH:MM:SS;FF).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Source</i> Use Source (TimecodeSource) to set how timecodes are handled within this job. To make sure that your video, audio, captions, and markers are synchronized and that time-based features, such as image inserter, work correctly, choose the Timecode source option that matches your assets. All timecodes are in a 24-hour format with frame number (HH:MM:SS:FF). * Embedded (EMBEDDED) - Use the timecode that is in the input video. If no embedded timecode is in the source, the service will use Start at 0 (ZEROBASED) instead. * Start at 0 (ZEROBASED) - Set the timecode of the initial frame to 00:00:00:00. * Specified Start (SPECIFIEDSTART) - Set the timecode of the initial frame to a value other than zero. You use Start timecode (Start) to provide this value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Anchor</i> If you use an editing platform that relies on an anchor timecode, use Anchor Timecode (Anchor) to specify a timecode that will match the input video frame to the output video frame. Use 24-hour format with frame number, (HH:MM:SS:FF) or (HH:MM:SS;FF). This setting ignores frame rate conversion. System behavior for Anchor Timecode varies depending on your setting for Source (TimecodeSource). * If Source (TimecodeSource) is set to Specified Start (SPECIFIEDSTART), the first input frame is the specified value in Start Timecode (Start). Anchor Timecode (Anchor) and Start Timecode (Start) are used calculate output timecode. * If Source (TimecodeSource) is set to Start at 0 (ZEROBASED)  the  first frame is 00:00:00:00. * If Source (TimecodeSource) is set to Embedded (EMBEDDED), the  first frame is the timecode value on the first input frame of the input.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id3Insertions</i> Id3Insertions contains the array of Id3Insertion instances.<br/>
        :return: Job:  Each job converts an input file into an output file or files. For more information, see the User Guide at http://docs.aws.amazon.com/mediaconvert/latest/ug/what-is.html:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserMetadata</i> User-defined metadata that you want to associate with an MediaConvert job. You specify metadata in key/value pairs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timing</i> Information about when jobs are submitted, started, and finished is specified in Unix epoch format in seconds.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Timing</b>: Information about when jobs are submitted, started, and finished is specified in Unix epoch format in seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusUpdateInterval</i> Specify how often MediaConvert sends STATUS_UPDATE events to Amazon CloudWatch Events. Set the interval, in seconds, between status updates. MediaConvert sends an update at this interval from the time the service begins processing your job to the time it completes the transcode or encounters an error.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> A job's status can be SUBMITTED, PROGRESSING, COMPLETE, CANCELED, or ERROR.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Settings</i> JobSettings contains all the transcode settings for a job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Settings</b>: JobSettings contains all the transcode settings for a job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Role</i> The IAM role you use for creating this job. For details about permissions, see the User Guide topic at the User Guide at http://docs.aws.amazon.com/mediaconvert/latest/ug/iam-role.html<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RetryCount</i> The number of times that the service automatically attempted to process your job after encountering an error.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Queue</i> Optional. When you create a job, you can specify a queue to send it to. If you don't specify, the job will go to the default queue. For more about queues, see the User Guide topic at http://docs.aws.amazon.com/mediaconvert/latest/ug/what-is.html<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputGroupDetails</i> List of output group details<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobTemplate</i> The job template that the job is created from, if it is created from a job template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobPercentComplete</i> An estimate of how far your job has progressed. This estimate is shown as a percentage of the total time from when your job leaves its queue to when your output files appear in your output Amazon S3 bucket. AWS Elemental MediaConvert provides jobPercentComplete in CloudWatch STATUS_UPDATE events and in the response to GetJob and ListJobs requests. The jobPercentComplete estimate is reliable for the following input containers: Quicktime, Transport Stream, MP4, and MXF. For some jobs, including audio-only jobs and jobs that use input clipping, the service can't provide information about job progress. In those cases, jobPercentComplete returns a null value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> A portion of the job's ARN, unique within your AWS Elemental MediaConvert resources<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorMessage</i> Error message of Job<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorCode</i> Error code for the job<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentPhase</i> A job's phase can be PROBING, TRANSCODING OR UPLOADING<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i> The time, in Unix epoch format in seconds, when the job got created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BillingTagsSource</i> Optional. Choose a tag type that AWS Billing and Cost Management will use to sort your AWS Elemental MediaConvert costs on any billing report that you set up. Any transcoding outputs that don't have an associated tag will appear in your billing report unsorted. If you don't choose a valid value for this field, your job outputs will appear on the billing report unsorted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> An identifier for this resource that is unique within all of AWS.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccelerationSettings</i> Accelerated transcoding can significantly speed up jobs with long, visually complex content.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AccelerationSettings</b>: Accelerated transcoding can significantly speed up jobs with long, visually complex content.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> Acceleration configuration for the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AdAvailOffset</i> When specified, this offset (in milliseconds) is added to the input Ad Avail PTS time.<br/>Settings for ad avail blanking.  Video can be blanked or overlaid with an image, and audio muted during SCTE-35 triggered ad avails.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailBlanking</b>: Settings for ad avail blanking.  Video can be blanked or overlaid with an image, and audio muted during SCTE-35 triggered ad avails.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmitTime</i> The time, in Unix epoch format, that you submitted the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTime</i> The time, in Unix epoch format, that transcoding for the job began.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FinishTime</i> The time, in Unix epoch format, that the transcoding job finished<br/>        """
        pass

    def create_job_template(self, Settings: JobTemplateSettings, Name: str, AccelerationSettings: AccelerationSettings=None, Category: str=None, Description: str=None, Queue: str=None, StatusUpdateInterval: str=None, Tags: __mapOf__string=None) -> CreateJobTemplateResponse:
        """Create a new job template. For information about job templates see the User Guide at http://docs.aws.amazon.com/mediaconvert/latest/ug/what-is.html

        :param self:
        :param Tags:  The tags that you want to add to the resource. You can tag resources with a key-value pair or with only a key.<br/>StatusUpdateInterval:  Specify how often MediaConvert sends STATUS_UPDATE events to Amazon CloudWatch Events. Set the interval, in seconds, between status updates. MediaConvert sends an update at this interval from the time the service begins processing your job to the time it completes the transcode or encounters an error.<br/>Settings:  JobTemplateSettings contains all the transcode settings saved in the template that will be applied to jobs created from it.:<br/>Queue:  Optional. The queue that jobs created from this template are assigned to. If you don't specify this, jobs will go to the default queue.<br/>Name:  The name of the job template you are creating.<br/>Description:  Optional. A description of the job template you are creating.<br/>Category:  Optional. A category for the job template you are creating<br/>AccelerationSettings:  Accelerated transcoding can significantly speed up jobs with long, visually complex content. Outputs that use this feature incur pro-tier pricing. For information about feature limitations, see the AWS Elemental MediaConvert User Guide.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> Acceleration configuration for the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimedMetadataInsertion</i> Enable Timed metadata insertion (TimedMetadataInsertion) to include ID3 tags in your job. To include timed metadata, you must enable it here, enable it in each output container, and specify tags and timecodes in ID3 insertion (Id3Insertion) objects.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TimedMetadataInsertion</b>: Enable Timed metadata insertion (TimedMetadataInsertion) to include ID3 tags in your job. To include timed metadata, you must enable it here, enable it in each output container, and specify tags and timecodes in ID3 insertion (Id3Insertion) objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimecodeConfig</i> Contains settings used to acquire and adjust timecode information from inputs.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TimecodeConfig</b>: Contains settings used to acquire and adjust timecode information from inputs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputGroups</i> (OutputGroups) contains one group of settings for each set of outputs that share a common package type. All unpackaged files (MPEG-4, MPEG-2 TS, Quicktime, MXF, and no container) are grouped in a single output group as well. Required in (OutputGroups) is a group of settings that apply to the whole group. This required object depends on the value you set for (Type) under (OutputGroups)>(OutputGroupSettings). Type, settings object pairs are as follows. * FILE_GROUP_SETTINGS, FileGroupSettings * HLS_GROUP_SETTINGS, HlsGroupSettings * DASH_ISO_GROUP_SETTINGS, DashIsoGroupSettings * MS_SMOOTH_GROUP_SETTINGS, MsSmoothGroupSettings * CMAF_GROUP_SETTINGS, CmafGroupSettings<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NielsenConfiguration</i> Settings for Nielsen Configuration&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NielsenConfiguration</b>: Settings for Nielsen Configuration<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MotionImageInserter</i> Overlay motion graphics on top of your video. The motion graphics that you specify here appear on all outputs in all output groups.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>MotionImageInserter</b>: Overlay motion graphics on top of your video. The motion graphics that you specify here appear on all outputs in all output groups.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Inputs</i> Use Inputs (inputs) to define the source file used in the transcode job. There can only be one input in a job template.  Using the API, you can include multiple inputs when referencing a job template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Esam</i> Settings for Event Signaling And Messaging (ESAM).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Esam</b>: Settings for Event Signaling And Messaging (ESAM).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailBlanking</i> Settings for ad avail blanking.  Video can be blanked or overlaid with an image, and audio muted during SCTE-35 triggered ad avails.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailBlanking</b>: Settings for ad avail blanking.  Video can be blanked or overlaid with an image, and audio muted during SCTE-35 triggered ad avails.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdAvailOffset</i> When specified, this offset (in milliseconds) is added to the input Ad Avail PTS time.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailBlankingImage</i> Blanking image to be used. Leave empty for solid black. Only bmp and png images are supported.<br/>Specifies an ESAM ManifestConfirmConditionNotification XML as per OC-SP-ESAM-API-I03-131025. The transcoder uses the manifest conditioning instructions that you provide in the setting MCC XML (mccXml).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ManifestConfirmConditionNotification</b>: Specifies an ESAM ManifestConfirmConditionNotification XML as per OC-SP-ESAM-API-I03-131025. The transcoder uses the manifest conditioning instructions that you provide in the setting MCC XML (mccXml).If your motion graphic asset is a .mov file, keep this setting unspecified. If your motion graphic asset is a series of .png files, specify the frame rate of the overlay in frames per second, as a fraction. For example, specify 24 fps as 24/1. Make sure that the number of images in your series matches the frame rate and your intended overlay duration. For example, if you want a 30-second overlay at 30 fps, you should have 900 .png images. This overlay frame rate doesn't need to match the frame rate of the underlying video.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Framerate</b>: If your motion graphic asset is a .mov file, keep this setting unspecified. If your motion graphic asset is a series of .png files, specify the frame rate of the overlay in frames per second, as a fraction. For example, specify 24 fps as 24/1. Make sure that the number of images in your series matches the frame rate and your intended overlay duration. For example, if you want a 30-second overlay at 30 fps, you should have 900 .png images. This overlay frame rate doesn't need to match the frame rate of the underlying video.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DistributorId</i> Use Distributor ID (DistributorID) to specify the distributor ID that is assigned to your organization by Neilsen.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BreakoutCode</i> Use Nielsen Configuration (NielsenConfiguration) to set the Nielsen measurement system breakout code. Supported values are 0, 3, 7, and 9.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TimestampOffset</i> Only applies to outputs that support program-date-time stamp. Use Timestamp offset (TimestampOffset) to overwrite the timecode date without affecting the time and frame number. Provide the new date as a string in the format "yyyy-mm-dd".  To use Time stamp offset, you must also enable Insert program-date-time (InsertProgramDateTime) in the output settings. For example, if the date part of your timecodes is 2002-1-25 and you want to change it to one year later, set Timestamp offset (TimestampOffset) to 2003-1-25.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Start</i> Only use when you set Source (TimecodeSource) to Specified start (SPECIFIEDSTART). Use Start timecode (Start) to specify the timecode for the initial frame. Use 24-hour format with frame number, (HH:MM:SS:FF) or (HH:MM:SS;FF).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Source</i> Use Source (TimecodeSource) to set how timecodes are handled within this job. To make sure that your video, audio, captions, and markers are synchronized and that time-based features, such as image inserter, work correctly, choose the Timecode source option that matches your assets. All timecodes are in a 24-hour format with frame number (HH:MM:SS:FF). * Embedded (EMBEDDED) - Use the timecode that is in the input video. If no embedded timecode is in the source, the service will use Start at 0 (ZEROBASED) instead. * Start at 0 (ZEROBASED) - Set the timecode of the initial frame to 00:00:00:00. * Specified Start (SPECIFIEDSTART) - Set the timecode of the initial frame to a value other than zero. You use Start timecode (Start) to provide this value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Anchor</i> If you use an editing platform that relies on an anchor timecode, use Anchor Timecode (Anchor) to specify a timecode that will match the input video frame to the output video frame. Use 24-hour format with frame number, (HH:MM:SS:FF) or (HH:MM:SS;FF). This setting ignores frame rate conversion. System behavior for Anchor Timecode varies depending on your setting for Source (TimecodeSource). * If Source (TimecodeSource) is set to Specified Start (SPECIFIEDSTART), the first input frame is the specified value in Start Timecode (Start). Anchor Timecode (Anchor) and Start Timecode (Start) are used calculate output timecode. * If Source (TimecodeSource) is set to Start at 0 (ZEROBASED)  the  first frame is 00:00:00:00. * If Source (TimecodeSource) is set to Embedded (EMBEDDED), the  first frame is the timecode value on the first input frame of the input.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id3Insertions</i> Id3Insertions contains the array of Id3Insertion instances.<br/>
        :return: JobTemplate:  A job template is a pre-made set of encoding instructions that you can use to quickly create a job.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> A job template can be of two types: system or custom. System or built-in job templates can't be modified or deleted by the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusUpdateInterval</i> Specify how often MediaConvert sends STATUS_UPDATE events to Amazon CloudWatch Events. Set the interval, in seconds, between status updates. MediaConvert sends an update at this interval from the time the service begins processing your job to the time it completes the transcode or encounters an error.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Settings</i> JobTemplateSettings contains all the transcode settings saved in the template that will be applied to jobs created from it.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Settings</b>: JobTemplateSettings contains all the transcode settings saved in the template that will be applied to jobs created from it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Queue</i> Optional. The queue that jobs created from this template are assigned to. If you don't specify this, jobs will go to the default queue.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A name you create for each job template. Each name must be unique within your account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdated</i> The timestamp in epoch seconds when the Job template was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> An optional description you create for each job template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i> The timestamp in epoch seconds for Job template creation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Category</i> An optional category you create to organize your job templates.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> An identifier for this resource that is unique within all of AWS.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccelerationSettings</i> Accelerated transcoding is currently in private preview. Contact AWS for more information.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AccelerationSettings</b>: Accelerated transcoding is currently in private preview. Contact AWS for more information.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> Acceleration configuration for the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AdAvailOffset</i> When specified, this offset (in milliseconds) is added to the input Ad Avail PTS time.<br/>Settings for ad avail blanking.  Video can be blanked or overlaid with an image, and audio muted during SCTE-35 triggered ad avails.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailBlanking</b>: Settings for ad avail blanking.  Video can be blanked or overlaid with an image, and audio muted during SCTE-35 triggered ad avails.        """
        pass

    def create_preset(self, Settings: PresetSettings, Name: str, Category: str=None, Description: str=None, Tags: __mapOf__string=None) -> CreatePresetResponse:
        """Create a new preset. For information about job templates see the User Guide at http://docs.aws.amazon.com/mediaconvert/latest/ug/what-is.html

        :param self:
        :param Tags:  The tags that you want to add to the resource. You can tag resources with a key-value pair or with only a key.<br/>Settings:  Settings for preset:<br/>Name:  The name of the preset you are creating.<br/>Description:  Optional. A description of the preset you are creating.<br/>Category:  Optional. A category for the preset you are creating.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VideoDescription</i> (VideoDescription) contains a group of video encoding settings. The specific video settings depend on the video codec you choose when you specify a value for Video codec (codec). Include one instance of (VideoDescription) per output.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VideoDescription</b>: (VideoDescription) contains a group of video encoding settings. The specific video settings depend on the video codec you choose when you specify a value for Video codec (codec). Include one instance of (VideoDescription) per output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContainerSettings</i> Container specific settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ContainerSettings</b>: Container specific settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CaptionDescriptions</i> Caption settings for this preset. There can be multiple caption settings in a single output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AudioDescriptions</i> (AudioDescriptions) contains groups of audio encoding settings organized by audio codec. Include one instance of (AudioDescriptions) per output. (AudioDescriptions) can contain multiple groups of encoding settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Container</i> Container for this output. Some containers require a container settings object. If not specified, the default object will be created.<br/>Settings for F4v container&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>F4vSettings</b>: Settings for F4v container&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AntiAlias</i> The service automatically applies the anti-alias filter to all outputs. The service no longer accepts the value DISABLED for AntiAlias. If you specify that in your job, the service will ignore the setting.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AfdSignaling</i> This setting only applies to H.264, H.265, and MPEG2 outputs. Use Insert AFD signaling (AfdSignaling) to specify whether the service includes AFD values in the output video data and what those values are. * Choose None to remove all AFD values from this output. * Choose Fixed to ignore input AFD values and instead encode the value specified in the job. * Choose Auto to calculate output AFD values based on the input AFD scaler data.<br/>Video codec settings, (CodecSettings) under (VideoDescription), contains the group of settings related to video encoding. The settings in this group vary depending on the value you choose for Video codec (Codec). For each codec enum you choose, define the corresponding settings object. The following lists the codec enum, settings object pairs. * H_264, H264Settings * H_265, H265Settings * MPEG2, Mpeg2Settings * PRORES, ProresSettings * FRAME_CAPTURE, FrameCaptureSettings&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CodecSettings</b>: Video codec settings, (CodecSettings) under (VideoDescription), contains the group of settings related to video encoding. The settings in this group vary depending on the value you choose for Video codec (Codec). For each codec enum you choose, define the corresponding settings object. The following lists the codec enum, settings object pairs. * H_264, H264Settings * H_265, H265Settings * MPEG2, Mpeg2Settings * PRORES, ProresSettings * FRAME_CAPTURE, FrameCaptureSettings
        :return: Preset:  A preset is a collection of preconfigured media conversion settings that you want MediaConvert to apply to the output during the conversion process.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> A preset can be of two types: system or custom. System or built-in preset can't be modified or deleted by the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Settings</i> Settings for preset&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Settings</b>: Settings for preset<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A name you create for each preset. Each name must be unique within your account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdated</i> The timestamp in epoch seconds when the preset was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> An optional description you create for each preset.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i> The timestamp in epoch seconds for preset creation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Category</i> An optional category you create to organize your presets.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> An identifier for this resource that is unique within all of AWS.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CaptionDescriptions</i> Caption settings for this preset. There can be multiple caption settings in a single output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AudioDescriptions</i> (AudioDescriptions) contains groups of audio encoding settings organized by audio codec. Include one instance of (AudioDescriptions) per output. (AudioDescriptions) can contain multiple groups of encoding settings.<br/>Container specific settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ContainerSettings</b>: Container specific settings.        """
        pass

    def create_queue(self, Name: str, Description: str=None, PricingPlan: str=None, ReservationPlanSettings: ReservationPlanSettings=None, Tags: __mapOf__string=None) -> CreateQueueResponse:
        """Create a new transcoding queue. For information about queues, see Working With Queues in the User Guide at https://docs.aws.amazon.com/mediaconvert/latest/ug/working-with-queues.html

        :param self:
        :param Tags:  The tags that you want to add to the resource. You can tag resources with a key-value pair or with only a key.<br/>ReservationPlanSettings:  Details about the pricing plan for your reserved queue. Required for reserved queues and not applicable to on-demand queues.:<br/>PricingPlan:  Specifies whether the pricing plan for the queue is on-demand or reserved. For on-demand, you pay per minute, billed in increments of .01 minute. For reserved, you pay for the transcoding capacity of the entire queue, regardless of how much or how little you use it. Reserved pricing requires a 12-month commitment. When you use the API to create a queue, the default is on-demand.<br/>Name:  The name of the queue that you are creating.<br/>Description:  Optional. A description of the queue that you are creating.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservedSlots</i> Specifies the number of reserved transcode slots (RTS) for this queue. The number of RTS determines how many jobs the queue can process in parallel; each RTS can process one job at a time. You can't decrease the number of RTS in your reserved queue. You can increase the number of RTS by extending your existing commitment with a new 12-month commitment for the larger number. The new commitment begins when you purchase the additional capacity. You can't cancel your commitment or revert to your original commitment after you increase the capacity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RenewalType</i> Specifies whether the term of your reserved queue pricing plan is automatically extended (AUTO_RENEW) or expires (EXPIRE) at the end of the term. When your term is auto renewed, you extend your commitment by 12 months from the auto renew date. You can cancel this commitment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Commitment</i> The length of the term of your reserved queue pricing plan commitment.<br/>
        :return: Queue:  You can use queues to manage the resources that are available to your AWS account for running multiple transcoding jobs at the same time. If you don't specify a queue, the service sends all jobs through the default queue. For more information, see https://docs.aws.amazon.com/mediaconvert/latest/ug/working-with-queues.html.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> Specifies whether this on-demand queue is system or custom. System queues are built in. You can't modify or delete system queues. You can create and modify custom queues.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmittedJobsCount</i> The estimated number of jobs with a SUBMITTED status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Queues can be ACTIVE or PAUSED. If you pause a queue, the service won't begin processing jobs in that queue. Jobs that are running when you pause the queue continue to run until they finish or result in an error.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservationPlan</i> Details about the pricing plan for your reserved queue. Required for reserved queues and not applicable to on-demand queues.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ReservationPlan</b>: Details about the pricing plan for your reserved queue. Required for reserved queues and not applicable to on-demand queues.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProgressingJobsCount</i> The estimated number of jobs with a PROGRESSING status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PricingPlan</i> Specifies whether the pricing plan for the queue is on-demand or reserved. For on-demand, you pay per minute, billed in increments of .01 minute. For reserved, you pay for the transcoding capacity of the entire queue, regardless of how much or how little you use it. Reserved pricing requires a 12-month commitment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A name that you create for each queue. Each name must be unique within your account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdated</i> The timestamp in epoch seconds for when you most recently updated the queue.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> An optional description that you create for each queue.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i> The timestamp in epoch seconds for when you created the queue.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> An identifier for this resource that is unique within all of AWS.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies whether the pricing plan for your reserved queue is ACTIVE or EXPIRED.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservedSlots</i> Specifies the number of reserved transcode slots (RTS) for this queue. The number of RTS determines how many jobs the queue can process in parallel; each RTS can process one job at a time. When you increase this number, you extend your existing commitment with a new 12-month commitment for a larger number of RTS. The new commitment begins when you purchase the additional capacity. You can't decrease the number of RTS in your reserved queue.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RenewalType</i> Specifies whether the term of your reserved queue pricing plan is automatically extended (AUTO_RENEW) or expires (EXPIRE) at the end of the term.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PurchasedAt</i> The timestamp in epoch seconds for when you set up the current pricing plan for this reserved queue.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpiresAt</i> The timestamp in epoch seconds for when the current pricing plan term for this reserved queue expires.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Commitment</i> The length of the term of your reserved queue pricing plan commitment.<br/>        """
        pass

    def delete_job_template(self, Name: str) -> DeleteJobTemplateResponse:
        """Permanently delete a job template you have created.

        :param self:
        :param Name:  The name of the job template to be deleted.<br/>
        :return:         """
        pass

    def delete_preset(self, Name: str) -> DeletePresetResponse:
        """Permanently delete a preset you have created.

        :param self:
        :param Name:  The name of the preset to be deleted.<br/>
        :return:         """
        pass

    def delete_queue(self, Name: str) -> DeleteQueueResponse:
        """Permanently delete a queue you have created.

        :param self:
        :param Name:  The name of the queue that you want to delete.<br/>
        :return:         """
        pass

    def describe_endpoints(self, MaxResults: int=None, Mode: str=None, NextToken: str=None) -> DescribeEndpointsResponse:
        """Send an request with an empty body to the regional API endpoint to get your account API endpoint.

        :param self:
        :param NextToken:  Use this string, provided with the response to a previous request, to request the next batch of endpoints.<br/>Mode:  Optional field, defaults to DEFAULT. Specify DEFAULT for this operation to return your endpoints if any exist, or to create an endpoint for you and return it if one doesn't already exist. Specify GET_ONLY to return your endpoints if any exist, or an empty list if none exist.<br/>MaxResults:  Optional. Max number of endpoints, up to twenty, that will be returned at one time.<br/>
        :return: NextToken:  Use this string to request the next batch of endpoints.<br/>Endpoints:  List of endpoints<br/>        """
        pass

    def disassociate_certificate(self, Arn: str) -> DisassociateCertificateResponse:
        """Removes an association between the Amazon Resource Name (ARN) of an AWS Certificate Manager (ACM) certificate and an AWS Elemental MediaConvert resource.

        :param self:
        :param Arn:  The ARN of the ACM certificate that you want to disassociate from your MediaConvert resource.<br/>
        :return:         """
        pass

    def get_job(self, Id: str) -> GetJobResponse:
        """Retrieve the JSON for a specific completed transcoding job.

        :param self:
        :param Id:  the job ID of the job.<br/>
        :return: Job:  Each job converts an input file into an output file or files. For more information, see the User Guide at http://docs.aws.amazon.com/mediaconvert/latest/ug/what-is.html:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserMetadata</i> User-defined metadata that you want to associate with an MediaConvert job. You specify metadata in key/value pairs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timing</i> Information about when jobs are submitted, started, and finished is specified in Unix epoch format in seconds.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Timing</b>: Information about when jobs are submitted, started, and finished is specified in Unix epoch format in seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusUpdateInterval</i> Specify how often MediaConvert sends STATUS_UPDATE events to Amazon CloudWatch Events. Set the interval, in seconds, between status updates. MediaConvert sends an update at this interval from the time the service begins processing your job to the time it completes the transcode or encounters an error.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> A job's status can be SUBMITTED, PROGRESSING, COMPLETE, CANCELED, or ERROR.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Settings</i> JobSettings contains all the transcode settings for a job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Settings</b>: JobSettings contains all the transcode settings for a job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Role</i> The IAM role you use for creating this job. For details about permissions, see the User Guide topic at the User Guide at http://docs.aws.amazon.com/mediaconvert/latest/ug/iam-role.html<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RetryCount</i> The number of times that the service automatically attempted to process your job after encountering an error.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Queue</i> Optional. When you create a job, you can specify a queue to send it to. If you don't specify, the job will go to the default queue. For more about queues, see the User Guide topic at http://docs.aws.amazon.com/mediaconvert/latest/ug/what-is.html<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputGroupDetails</i> List of output group details<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobTemplate</i> The job template that the job is created from, if it is created from a job template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobPercentComplete</i> An estimate of how far your job has progressed. This estimate is shown as a percentage of the total time from when your job leaves its queue to when your output files appear in your output Amazon S3 bucket. AWS Elemental MediaConvert provides jobPercentComplete in CloudWatch STATUS_UPDATE events and in the response to GetJob and ListJobs requests. The jobPercentComplete estimate is reliable for the following input containers: Quicktime, Transport Stream, MP4, and MXF. For some jobs, including audio-only jobs and jobs that use input clipping, the service can't provide information about job progress. In those cases, jobPercentComplete returns a null value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> A portion of the job's ARN, unique within your AWS Elemental MediaConvert resources<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorMessage</i> Error message of Job<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorCode</i> Error code for the job<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentPhase</i> A job's phase can be PROBING, TRANSCODING OR UPLOADING<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i> The time, in Unix epoch format in seconds, when the job got created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BillingTagsSource</i> Optional. Choose a tag type that AWS Billing and Cost Management will use to sort your AWS Elemental MediaConvert costs on any billing report that you set up. Any transcoding outputs that don't have an associated tag will appear in your billing report unsorted. If you don't choose a valid value for this field, your job outputs will appear on the billing report unsorted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> An identifier for this resource that is unique within all of AWS.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccelerationSettings</i> Accelerated transcoding can significantly speed up jobs with long, visually complex content.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AccelerationSettings</b>: Accelerated transcoding can significantly speed up jobs with long, visually complex content.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> Acceleration configuration for the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AdAvailOffset</i> When specified, this offset (in milliseconds) is added to the input Ad Avail PTS time.<br/>Settings for ad avail blanking.  Video can be blanked or overlaid with an image, and audio muted during SCTE-35 triggered ad avails.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailBlanking</b>: Settings for ad avail blanking.  Video can be blanked or overlaid with an image, and audio muted during SCTE-35 triggered ad avails.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmitTime</i> The time, in Unix epoch format, that you submitted the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTime</i> The time, in Unix epoch format, that transcoding for the job began.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FinishTime</i> The time, in Unix epoch format, that the transcoding job finished<br/>        """
        pass

    def get_job_template(self, Name: str) -> GetJobTemplateResponse:
        """Retrieve the JSON for a specific job template.

        :param self:
        :param Name:  The name of the job template.<br/>
        :return: JobTemplate:  A job template is a pre-made set of encoding instructions that you can use to quickly create a job.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> A job template can be of two types: system or custom. System or built-in job templates can't be modified or deleted by the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusUpdateInterval</i> Specify how often MediaConvert sends STATUS_UPDATE events to Amazon CloudWatch Events. Set the interval, in seconds, between status updates. MediaConvert sends an update at this interval from the time the service begins processing your job to the time it completes the transcode or encounters an error.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Settings</i> JobTemplateSettings contains all the transcode settings saved in the template that will be applied to jobs created from it.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Settings</b>: JobTemplateSettings contains all the transcode settings saved in the template that will be applied to jobs created from it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Queue</i> Optional. The queue that jobs created from this template are assigned to. If you don't specify this, jobs will go to the default queue.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A name you create for each job template. Each name must be unique within your account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdated</i> The timestamp in epoch seconds when the Job template was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> An optional description you create for each job template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i> The timestamp in epoch seconds for Job template creation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Category</i> An optional category you create to organize your job templates.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> An identifier for this resource that is unique within all of AWS.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccelerationSettings</i> Accelerated transcoding is currently in private preview. Contact AWS for more information.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AccelerationSettings</b>: Accelerated transcoding is currently in private preview. Contact AWS for more information.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> Acceleration configuration for the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AdAvailOffset</i> When specified, this offset (in milliseconds) is added to the input Ad Avail PTS time.<br/>Settings for ad avail blanking.  Video can be blanked or overlaid with an image, and audio muted during SCTE-35 triggered ad avails.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailBlanking</b>: Settings for ad avail blanking.  Video can be blanked or overlaid with an image, and audio muted during SCTE-35 triggered ad avails.        """
        pass

    def get_preset(self, Name: str) -> GetPresetResponse:
        """Retrieve the JSON for a specific preset.

        :param self:
        :param Name:  The name of the preset.<br/>
        :return: Preset:  A preset is a collection of preconfigured media conversion settings that you want MediaConvert to apply to the output during the conversion process.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> A preset can be of two types: system or custom. System or built-in preset can't be modified or deleted by the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Settings</i> Settings for preset&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Settings</b>: Settings for preset<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A name you create for each preset. Each name must be unique within your account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdated</i> The timestamp in epoch seconds when the preset was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> An optional description you create for each preset.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i> The timestamp in epoch seconds for preset creation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Category</i> An optional category you create to organize your presets.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> An identifier for this resource that is unique within all of AWS.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CaptionDescriptions</i> Caption settings for this preset. There can be multiple caption settings in a single output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AudioDescriptions</i> (AudioDescriptions) contains groups of audio encoding settings organized by audio codec. Include one instance of (AudioDescriptions) per output. (AudioDescriptions) can contain multiple groups of encoding settings.<br/>Container specific settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ContainerSettings</b>: Container specific settings.        """
        pass

    def get_queue(self, Name: str) -> GetQueueResponse:
        """Retrieve the JSON for a specific queue.

        :param self:
        :param Name:  The name of the queue that you want information about.<br/>
        :return: Queue:  You can use queues to manage the resources that are available to your AWS account for running multiple transcoding jobs at the same time. If you don't specify a queue, the service sends all jobs through the default queue. For more information, see https://docs.aws.amazon.com/mediaconvert/latest/ug/working-with-queues.html.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> Specifies whether this on-demand queue is system or custom. System queues are built in. You can't modify or delete system queues. You can create and modify custom queues.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmittedJobsCount</i> The estimated number of jobs with a SUBMITTED status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Queues can be ACTIVE or PAUSED. If you pause a queue, the service won't begin processing jobs in that queue. Jobs that are running when you pause the queue continue to run until they finish or result in an error.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservationPlan</i> Details about the pricing plan for your reserved queue. Required for reserved queues and not applicable to on-demand queues.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ReservationPlan</b>: Details about the pricing plan for your reserved queue. Required for reserved queues and not applicable to on-demand queues.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProgressingJobsCount</i> The estimated number of jobs with a PROGRESSING status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PricingPlan</i> Specifies whether the pricing plan for the queue is on-demand or reserved. For on-demand, you pay per minute, billed in increments of .01 minute. For reserved, you pay for the transcoding capacity of the entire queue, regardless of how much or how little you use it. Reserved pricing requires a 12-month commitment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A name that you create for each queue. Each name must be unique within your account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdated</i> The timestamp in epoch seconds for when you most recently updated the queue.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> An optional description that you create for each queue.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i> The timestamp in epoch seconds for when you created the queue.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> An identifier for this resource that is unique within all of AWS.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies whether the pricing plan for your reserved queue is ACTIVE or EXPIRED.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservedSlots</i> Specifies the number of reserved transcode slots (RTS) for this queue. The number of RTS determines how many jobs the queue can process in parallel; each RTS can process one job at a time. When you increase this number, you extend your existing commitment with a new 12-month commitment for a larger number of RTS. The new commitment begins when you purchase the additional capacity. You can't decrease the number of RTS in your reserved queue.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RenewalType</i> Specifies whether the term of your reserved queue pricing plan is automatically extended (AUTO_RENEW) or expires (EXPIRE) at the end of the term.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PurchasedAt</i> The timestamp in epoch seconds for when you set up the current pricing plan for this reserved queue.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpiresAt</i> The timestamp in epoch seconds for when the current pricing plan term for this reserved queue expires.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Commitment</i> The length of the term of your reserved queue pricing plan commitment.<br/>        """
        pass

    def list_job_templates(self, Category: str=None, ListBy: str=None, MaxResults: int=None, NextToken: str=None, Order: str=None) -> ListJobTemplatesResponse:
        """Retrieve a JSON array of up to twenty of your job templates. This will return the templates themselves, not just a list of them. To retrieve the next twenty templates, use the nextToken string returned with the array

        :param self:
        :param Order:  When you request lists of resources, you can optionally specify whether they are sorted in ASCENDING or DESCENDING order. Default varies by resource.<br/>NextToken:  Use this string, provided with the response to a previous request, to request the next batch of job templates.<br/>MaxResults:  Optional. Number of job templates, up to twenty, that will be returned at one time.<br/>ListBy:  Optional. When you request a list of job templates, you can choose to list them alphabetically by NAME or chronologically by CREATION_DATE. If you don't specify, the service will list them by name.<br/>Category:  Optionally, specify a job template category to limit responses to only job templates from that category.<br/>
        :return: NextToken:  Use this string to request the next batch of job templates.<br/>JobTemplates:  List of Job templates.<br/>        """
        pass

    def list_jobs(self, MaxResults: int=None, NextToken: str=None, Order: str=None, Queue: str=None, Status: str=None) -> ListJobsResponse:
        """Retrieve a JSON array of up to twenty of your most recently created jobs. This array includes in-process, completed, and errored jobs. This will return the jobs themselves, not just a list of the jobs. To retrieve the twenty next most recent jobs, use the nextToken string returned with the array.

        :param self:
        :param Status:  A job's status can be SUBMITTED, PROGRESSING, COMPLETE, CANCELED, or ERROR.<br/>Queue:  Provide a queue name to get back only jobs from that queue.<br/>Order:  When you request lists of resources, you can optionally specify whether they are sorted in ASCENDING or DESCENDING order. Default varies by resource.<br/>NextToken:  Use this string, provided with the response to a previous request, to request the next batch of jobs.<br/>MaxResults:  Optional. Number of jobs, up to twenty, that will be returned at one time.<br/>
        :return: NextToken:  Use this string to request the next batch of jobs.<br/>Jobs:  List of jobs<br/>        """
        pass

    def list_presets(self, Category: str=None, ListBy: str=None, MaxResults: int=None, NextToken: str=None, Order: str=None) -> ListPresetsResponse:
        """Retrieve a JSON array of up to twenty of your presets. This will return the presets themselves, not just a list of them. To retrieve the next twenty presets, use the nextToken string returned with the array.

        :param self:
        :param Order:  When you request lists of resources, you can optionally specify whether they are sorted in ASCENDING or DESCENDING order. Default varies by resource.<br/>NextToken:  Use this string, provided with the response to a previous request, to request the next batch of presets.<br/>MaxResults:  Optional. Number of presets, up to twenty, that will be returned at one time<br/>ListBy:  Optional. When you request a list of presets, you can choose to list them alphabetically by NAME or chronologically by CREATION_DATE. If you don't specify, the service will list them by name.<br/>Category:  Optionally, specify a preset category to limit responses to only presets from that category.<br/>
        :return: Presets:  List of presets<br/>NextToken:  Use this string to request the next batch of presets.<br/>        """
        pass

    def list_queues(self, ListBy: str=None, MaxResults: int=None, NextToken: str=None, Order: str=None) -> ListQueuesResponse:
        """Retrieve a JSON array of up to twenty of your queues. This will return the queues themselves, not just a list of them. To retrieve the next twenty queues, use the nextToken string returned with the array.

        :param self:
        :param Order:  When you request lists of resources, you can optionally specify whether they are sorted in ASCENDING or DESCENDING order. Default varies by resource.<br/>NextToken:  Use this string, provided with the response to a previous request, to request the next batch of queues.<br/>MaxResults:  Optional. Number of queues, up to twenty, that will be returned at one time.<br/>ListBy:  Optional. When you request a list of queues, you can choose to list them alphabetically by NAME or chronologically by CREATION_DATE. If you don't specify, the service will list them by creation date.<br/>
        :return: Queues:  List of queues.<br/>NextToken:  Use this string to request the next batch of queues.<br/>        """
        pass

    def list_tags_for_resource(self, Arn: str) -> ListTagsForResourceResponse:
        """Retrieve the tags for a MediaConvert resource.

        :param self:
        :param Arn:  The Amazon Resource Name (ARN) of the resource that you want to list tags for. To get the ARN, send a GET request with the resource name.<br/>
        :return: ResourceTags:  The Amazon Resource Name (ARN) and tags for an AWS Elemental MediaConvert resource.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The tags for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the resource.<br/>        """
        pass

    def tag_resource(self, Tags: __mapOf__string, Arn: str) -> TagResourceResponse:
        """Add tags to a MediaConvert queue, preset, or job template. For information about tagging, see the User Guide at https://docs.aws.amazon.com/mediaconvert/latest/ug/tagging-resources.html

        :param self:
        :param Tags:  The tags that you want to add to the resource. You can tag resources with a key-value pair or with only a key.<br/>Arn:  The Amazon Resource Name (ARN) of the resource that you want to tag. To get the ARN, send a GET request with the resource name.<br/>
        :return:         """
        pass

    def untag_resource(self, Arn: str, TagKeys: List[__string]=None) -> UntagResourceResponse:
        """Remove tags from a MediaConvert queue, preset, or job template. For information about tagging, see the User Guide at https://docs.aws.amazon.com/mediaconvert/latest/ug/tagging-resources.html

        :param self:
        :param TagKeys:  The keys of the tags that you want to remove from the resource.<br/>Arn:  The Amazon Resource Name (ARN) of the resource that you want to remove tags from. To get the ARN, send a GET request with the resource name.<br/>
        :return:         """
        pass

    def update_job_template(self, Name: str, AccelerationSettings: AccelerationSettings=None, Category: str=None, Description: str=None, Queue: str=None, Settings: JobTemplateSettings=None, StatusUpdateInterval: str=None) -> UpdateJobTemplateResponse:
        """Modify one of your existing job templates.

        :param self:
        :param StatusUpdateInterval:  Specify how often MediaConvert sends STATUS_UPDATE events to Amazon CloudWatch Events. Set the interval, in seconds, between status updates. MediaConvert sends an update at this interval from the time the service begins processing your job to the time it completes the transcode or encounters an error.<br/>Settings:  JobTemplateSettings contains all the transcode settings saved in the template that will be applied to jobs created from it.:<br/>Queue:  The new queue for the job template, if you are changing it.<br/>Name:  The name of the job template you are modifying<br/>Description:  The new description for the job template, if you are changing it.<br/>Category:  The new category for the job template, if you are changing it.<br/>AccelerationSettings:  Accelerated transcoding can significantly speed up jobs with long, visually complex content. Outputs that use this feature incur pro-tier pricing. For information about feature limitations, see the AWS Elemental MediaConvert User Guide.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> Acceleration configuration for the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimedMetadataInsertion</i> Enable Timed metadata insertion (TimedMetadataInsertion) to include ID3 tags in your job. To include timed metadata, you must enable it here, enable it in each output container, and specify tags and timecodes in ID3 insertion (Id3Insertion) objects.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TimedMetadataInsertion</b>: Enable Timed metadata insertion (TimedMetadataInsertion) to include ID3 tags in your job. To include timed metadata, you must enable it here, enable it in each output container, and specify tags and timecodes in ID3 insertion (Id3Insertion) objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimecodeConfig</i> Contains settings used to acquire and adjust timecode information from inputs.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TimecodeConfig</b>: Contains settings used to acquire and adjust timecode information from inputs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputGroups</i> (OutputGroups) contains one group of settings for each set of outputs that share a common package type. All unpackaged files (MPEG-4, MPEG-2 TS, Quicktime, MXF, and no container) are grouped in a single output group as well. Required in (OutputGroups) is a group of settings that apply to the whole group. This required object depends on the value you set for (Type) under (OutputGroups)>(OutputGroupSettings). Type, settings object pairs are as follows. * FILE_GROUP_SETTINGS, FileGroupSettings * HLS_GROUP_SETTINGS, HlsGroupSettings * DASH_ISO_GROUP_SETTINGS, DashIsoGroupSettings * MS_SMOOTH_GROUP_SETTINGS, MsSmoothGroupSettings * CMAF_GROUP_SETTINGS, CmafGroupSettings<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NielsenConfiguration</i> Settings for Nielsen Configuration&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NielsenConfiguration</b>: Settings for Nielsen Configuration<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MotionImageInserter</i> Overlay motion graphics on top of your video. The motion graphics that you specify here appear on all outputs in all output groups.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>MotionImageInserter</b>: Overlay motion graphics on top of your video. The motion graphics that you specify here appear on all outputs in all output groups.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Inputs</i> Use Inputs (inputs) to define the source file used in the transcode job. There can only be one input in a job template.  Using the API, you can include multiple inputs when referencing a job template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Esam</i> Settings for Event Signaling And Messaging (ESAM).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Esam</b>: Settings for Event Signaling And Messaging (ESAM).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailBlanking</i> Settings for ad avail blanking.  Video can be blanked or overlaid with an image, and audio muted during SCTE-35 triggered ad avails.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailBlanking</b>: Settings for ad avail blanking.  Video can be blanked or overlaid with an image, and audio muted during SCTE-35 triggered ad avails.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdAvailOffset</i> When specified, this offset (in milliseconds) is added to the input Ad Avail PTS time.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailBlankingImage</i> Blanking image to be used. Leave empty for solid black. Only bmp and png images are supported.<br/>Specifies an ESAM ManifestConfirmConditionNotification XML as per OC-SP-ESAM-API-I03-131025. The transcoder uses the manifest conditioning instructions that you provide in the setting MCC XML (mccXml).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ManifestConfirmConditionNotification</b>: Specifies an ESAM ManifestConfirmConditionNotification XML as per OC-SP-ESAM-API-I03-131025. The transcoder uses the manifest conditioning instructions that you provide in the setting MCC XML (mccXml).If your motion graphic asset is a .mov file, keep this setting unspecified. If your motion graphic asset is a series of .png files, specify the frame rate of the overlay in frames per second, as a fraction. For example, specify 24 fps as 24/1. Make sure that the number of images in your series matches the frame rate and your intended overlay duration. For example, if you want a 30-second overlay at 30 fps, you should have 900 .png images. This overlay frame rate doesn't need to match the frame rate of the underlying video.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Framerate</b>: If your motion graphic asset is a .mov file, keep this setting unspecified. If your motion graphic asset is a series of .png files, specify the frame rate of the overlay in frames per second, as a fraction. For example, specify 24 fps as 24/1. Make sure that the number of images in your series matches the frame rate and your intended overlay duration. For example, if you want a 30-second overlay at 30 fps, you should have 900 .png images. This overlay frame rate doesn't need to match the frame rate of the underlying video.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DistributorId</i> Use Distributor ID (DistributorID) to specify the distributor ID that is assigned to your organization by Neilsen.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BreakoutCode</i> Use Nielsen Configuration (NielsenConfiguration) to set the Nielsen measurement system breakout code. Supported values are 0, 3, 7, and 9.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TimestampOffset</i> Only applies to outputs that support program-date-time stamp. Use Timestamp offset (TimestampOffset) to overwrite the timecode date without affecting the time and frame number. Provide the new date as a string in the format "yyyy-mm-dd".  To use Time stamp offset, you must also enable Insert program-date-time (InsertProgramDateTime) in the output settings. For example, if the date part of your timecodes is 2002-1-25 and you want to change it to one year later, set Timestamp offset (TimestampOffset) to 2003-1-25.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Start</i> Only use when you set Source (TimecodeSource) to Specified start (SPECIFIEDSTART). Use Start timecode (Start) to specify the timecode for the initial frame. Use 24-hour format with frame number, (HH:MM:SS:FF) or (HH:MM:SS;FF).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Source</i> Use Source (TimecodeSource) to set how timecodes are handled within this job. To make sure that your video, audio, captions, and markers are synchronized and that time-based features, such as image inserter, work correctly, choose the Timecode source option that matches your assets. All timecodes are in a 24-hour format with frame number (HH:MM:SS:FF). * Embedded (EMBEDDED) - Use the timecode that is in the input video. If no embedded timecode is in the source, the service will use Start at 0 (ZEROBASED) instead. * Start at 0 (ZEROBASED) - Set the timecode of the initial frame to 00:00:00:00. * Specified Start (SPECIFIEDSTART) - Set the timecode of the initial frame to a value other than zero. You use Start timecode (Start) to provide this value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Anchor</i> If you use an editing platform that relies on an anchor timecode, use Anchor Timecode (Anchor) to specify a timecode that will match the input video frame to the output video frame. Use 24-hour format with frame number, (HH:MM:SS:FF) or (HH:MM:SS;FF). This setting ignores frame rate conversion. System behavior for Anchor Timecode varies depending on your setting for Source (TimecodeSource). * If Source (TimecodeSource) is set to Specified Start (SPECIFIEDSTART), the first input frame is the specified value in Start Timecode (Start). Anchor Timecode (Anchor) and Start Timecode (Start) are used calculate output timecode. * If Source (TimecodeSource) is set to Start at 0 (ZEROBASED)  the  first frame is 00:00:00:00. * If Source (TimecodeSource) is set to Embedded (EMBEDDED), the  first frame is the timecode value on the first input frame of the input.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id3Insertions</i> Id3Insertions contains the array of Id3Insertion instances.<br/>
        :return: JobTemplate:  A job template is a pre-made set of encoding instructions that you can use to quickly create a job.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> A job template can be of two types: system or custom. System or built-in job templates can't be modified or deleted by the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusUpdateInterval</i> Specify how often MediaConvert sends STATUS_UPDATE events to Amazon CloudWatch Events. Set the interval, in seconds, between status updates. MediaConvert sends an update at this interval from the time the service begins processing your job to the time it completes the transcode or encounters an error.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Settings</i> JobTemplateSettings contains all the transcode settings saved in the template that will be applied to jobs created from it.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Settings</b>: JobTemplateSettings contains all the transcode settings saved in the template that will be applied to jobs created from it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Queue</i> Optional. The queue that jobs created from this template are assigned to. If you don't specify this, jobs will go to the default queue.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A name you create for each job template. Each name must be unique within your account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdated</i> The timestamp in epoch seconds when the Job template was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> An optional description you create for each job template.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i> The timestamp in epoch seconds for Job template creation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Category</i> An optional category you create to organize your job templates.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> An identifier for this resource that is unique within all of AWS.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccelerationSettings</i> Accelerated transcoding is currently in private preview. Contact AWS for more information.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AccelerationSettings</b>: Accelerated transcoding is currently in private preview. Contact AWS for more information.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> Acceleration configuration for the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AdAvailOffset</i> When specified, this offset (in milliseconds) is added to the input Ad Avail PTS time.<br/>Settings for ad avail blanking.  Video can be blanked or overlaid with an image, and audio muted during SCTE-35 triggered ad avails.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailBlanking</b>: Settings for ad avail blanking.  Video can be blanked or overlaid with an image, and audio muted during SCTE-35 triggered ad avails.        """
        pass

    def update_preset(self, Name: str, Category: str=None, Description: str=None, Settings: PresetSettings=None) -> UpdatePresetResponse:
        """Modify one of your existing presets.

        :param self:
        :param Settings:  Settings for preset:<br/>Name:  The name of the preset you are modifying.<br/>Description:  The new description for the preset, if you are changing it.<br/>Category:  The new category for the preset, if you are changing it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VideoDescription</i> (VideoDescription) contains a group of video encoding settings. The specific video settings depend on the video codec you choose when you specify a value for Video codec (codec). Include one instance of (VideoDescription) per output.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VideoDescription</b>: (VideoDescription) contains a group of video encoding settings. The specific video settings depend on the video codec you choose when you specify a value for Video codec (codec). Include one instance of (VideoDescription) per output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContainerSettings</i> Container specific settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ContainerSettings</b>: Container specific settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CaptionDescriptions</i> Caption settings for this preset. There can be multiple caption settings in a single output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AudioDescriptions</i> (AudioDescriptions) contains groups of audio encoding settings organized by audio codec. Include one instance of (AudioDescriptions) per output. (AudioDescriptions) can contain multiple groups of encoding settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Container</i> Container for this output. Some containers require a container settings object. If not specified, the default object will be created.<br/>Settings for F4v container&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>F4vSettings</b>: Settings for F4v container&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AntiAlias</i> The service automatically applies the anti-alias filter to all outputs. The service no longer accepts the value DISABLED for AntiAlias. If you specify that in your job, the service will ignore the setting.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AfdSignaling</i> This setting only applies to H.264, H.265, and MPEG2 outputs. Use Insert AFD signaling (AfdSignaling) to specify whether the service includes AFD values in the output video data and what those values are. * Choose None to remove all AFD values from this output. * Choose Fixed to ignore input AFD values and instead encode the value specified in the job. * Choose Auto to calculate output AFD values based on the input AFD scaler data.<br/>Video codec settings, (CodecSettings) under (VideoDescription), contains the group of settings related to video encoding. The settings in this group vary depending on the value you choose for Video codec (Codec). For each codec enum you choose, define the corresponding settings object. The following lists the codec enum, settings object pairs. * H_264, H264Settings * H_265, H265Settings * MPEG2, Mpeg2Settings * PRORES, ProresSettings * FRAME_CAPTURE, FrameCaptureSettings&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CodecSettings</b>: Video codec settings, (CodecSettings) under (VideoDescription), contains the group of settings related to video encoding. The settings in this group vary depending on the value you choose for Video codec (Codec). For each codec enum you choose, define the corresponding settings object. The following lists the codec enum, settings object pairs. * H_264, H264Settings * H_265, H265Settings * MPEG2, Mpeg2Settings * PRORES, ProresSettings * FRAME_CAPTURE, FrameCaptureSettings
        :return: Preset:  A preset is a collection of preconfigured media conversion settings that you want MediaConvert to apply to the output during the conversion process.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> A preset can be of two types: system or custom. System or built-in preset can't be modified or deleted by the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Settings</i> Settings for preset&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Settings</b>: Settings for preset<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A name you create for each preset. Each name must be unique within your account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdated</i> The timestamp in epoch seconds when the preset was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> An optional description you create for each preset.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i> The timestamp in epoch seconds for preset creation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Category</i> An optional category you create to organize your presets.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> An identifier for this resource that is unique within all of AWS.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CaptionDescriptions</i> Caption settings for this preset. There can be multiple caption settings in a single output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AudioDescriptions</i> (AudioDescriptions) contains groups of audio encoding settings organized by audio codec. Include one instance of (AudioDescriptions) per output. (AudioDescriptions) can contain multiple groups of encoding settings.<br/>Container specific settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ContainerSettings</b>: Container specific settings.        """
        pass

    def update_queue(self, Name: str, Description: str=None, ReservationPlanSettings: ReservationPlanSettings=None, Status: str=None) -> UpdateQueueResponse:
        """Modify one of your existing queues.

        :param self:
        :param Status:  Pause or activate a queue by changing its status between ACTIVE and PAUSED. If you pause a queue, jobs in that queue won't begin. Jobs that are running when you pause the queue continue to run until they finish or result in an error.<br/>ReservationPlanSettings:  The new details of your pricing plan for your reserved queue. When you set up a new pricing plan to replace an expired one, you enter into another 12-month commitment. When you add capacity to your queue by increasing the number of RTS, you extend the term of your commitment to 12 months from when you add capacity. After you make these commitments, you can't cancel them.:<br/>Name:  The name of the queue that you are modifying.<br/>Description:  The new description for the queue, if you are changing it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservedSlots</i> Specifies the number of reserved transcode slots (RTS) for this queue. The number of RTS determines how many jobs the queue can process in parallel; each RTS can process one job at a time. You can't decrease the number of RTS in your reserved queue. You can increase the number of RTS by extending your existing commitment with a new 12-month commitment for the larger number. The new commitment begins when you purchase the additional capacity. You can't cancel your commitment or revert to your original commitment after you increase the capacity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RenewalType</i> Specifies whether the term of your reserved queue pricing plan is automatically extended (AUTO_RENEW) or expires (EXPIRE) at the end of the term. When your term is auto renewed, you extend your commitment by 12 months from the auto renew date. You can cancel this commitment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Commitment</i> The length of the term of your reserved queue pricing plan commitment.<br/>
        :return: Queue:  You can use queues to manage the resources that are available to your AWS account for running multiple transcoding jobs at the same time. If you don't specify a queue, the service sends all jobs through the default queue. For more information, see https://docs.aws.amazon.com/mediaconvert/latest/ug/working-with-queues.html.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> Specifies whether this on-demand queue is system or custom. System queues are built in. You can't modify or delete system queues. You can create and modify custom queues.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmittedJobsCount</i> The estimated number of jobs with a SUBMITTED status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Queues can be ACTIVE or PAUSED. If you pause a queue, the service won't begin processing jobs in that queue. Jobs that are running when you pause the queue continue to run until they finish or result in an error.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservationPlan</i> Details about the pricing plan for your reserved queue. Required for reserved queues and not applicable to on-demand queues.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ReservationPlan</b>: Details about the pricing plan for your reserved queue. Required for reserved queues and not applicable to on-demand queues.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProgressingJobsCount</i> The estimated number of jobs with a PROGRESSING status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PricingPlan</i> Specifies whether the pricing plan for the queue is on-demand or reserved. For on-demand, you pay per minute, billed in increments of .01 minute. For reserved, you pay for the transcoding capacity of the entire queue, regardless of how much or how little you use it. Reserved pricing requires a 12-month commitment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A name that you create for each queue. Each name must be unique within your account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdated</i> The timestamp in epoch seconds for when you most recently updated the queue.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> An optional description that you create for each queue.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i> The timestamp in epoch seconds for when you created the queue.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> An identifier for this resource that is unique within all of AWS.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies whether the pricing plan for your reserved queue is ACTIVE or EXPIRED.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservedSlots</i> Specifies the number of reserved transcode slots (RTS) for this queue. The number of RTS determines how many jobs the queue can process in parallel; each RTS can process one job at a time. When you increase this number, you extend your existing commitment with a new 12-month commitment for a larger number of RTS. The new commitment begins when you purchase the additional capacity. You can't decrease the number of RTS in your reserved queue.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RenewalType</i> Specifies whether the term of your reserved queue pricing plan is automatically extended (AUTO_RENEW) or expires (EXPIRE) at the end of the term.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PurchasedAt</i> The timestamp in epoch seconds for when you set up the current pricing plan for this reserved queue.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpiresAt</i> The timestamp in epoch seconds for when the current pricing plan term for this reserved queue expires.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Commitment</i> The length of the term of your reserved queue pricing plan commitment.<br/>        """
        pass



class MediaLive(BaseClient):
    """API for AWS Elemental MediaLive"""

    class ChannelCreatedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_channel` for valid parameters"""
            pass
    class ChannelDeletedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_channel` for valid parameters"""
            pass
    class ChannelRunningWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_channel` for valid parameters"""
            pass
    class ChannelStoppedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_channel` for valid parameters"""
            pass


    class ListInputsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_inputs` for valid parameters"""
            pass
    class ListChannelsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_channels` for valid parameters"""
            pass
    class ListInputSecurityGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_input_security_groups` for valid parameters"""
            pass
    class ListOfferingsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_offerings` for valid parameters"""
            pass
    class ListReservationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_reservations` for valid parameters"""
            pass
    class DescribeSchedulePaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_schedule` for valid parameters"""
            pass


    class BatchUpdateScheduleRequest(Mapping):
        pass
    
    class BatchUpdateScheduleResponse(Mapping):
        pass
    
    class BatchScheduleActionCreateRequest(Mapping):
        pass
    
    class BatchScheduleActionDeleteRequest(Mapping):
        pass
    
    class CreateChannelRequest(Mapping):
        pass
    
    class CreateChannelResponse(Mapping):
        pass
    
    class OutputDestination(Mapping):
        pass
    
    class EncoderSettings(Mapping):
        pass
    
    class InputAttachment(Mapping):
        pass
    
    class InputSpecification(Mapping):
        pass
    
    class Tags(object):
        pass
    
    class CreateInputRequest(Mapping):
        pass
    
    class CreateInputResponse(Mapping):
        pass
    
    class InputDestinationRequest(Mapping):
        pass
    
    class __string(object):
        pass
    
    class MediaConnectFlowRequest(Mapping):
        pass
    
    class InputSourceRequest(Mapping):
        pass
    
    class InputVpcRequest(Mapping):
        pass
    
    class CreateInputSecurityGroupRequest(Mapping):
        pass
    
    class CreateInputSecurityGroupResponse(Mapping):
        pass
    
    class InputWhitelistRuleCidr(Mapping):
        pass
    
    class CreateTagsRequest(Mapping):
        pass
    
    class DeleteChannelRequest(Mapping):
        pass
    
    class DeleteChannelResponse(Mapping):
        pass
    
    class DeleteInputRequest(Mapping):
        pass
    
    class DeleteInputResponse(Mapping):
        pass
    
    class DeleteInputSecurityGroupRequest(Mapping):
        pass
    
    class DeleteInputSecurityGroupResponse(Mapping):
        pass
    
    class DeleteReservationRequest(Mapping):
        pass
    
    class DeleteReservationResponse(Mapping):
        pass
    
    class DeleteScheduleRequest(Mapping):
        pass
    
    class DeleteScheduleResponse(Mapping):
        pass
    
    class DeleteTagsRequest(Mapping):
        pass
    
    class DescribeChannelRequest(Mapping):
        pass
    
    class DescribeChannelResponse(Mapping):
        pass
    
    class DescribeInputRequest(Mapping):
        pass
    
    class DescribeInputResponse(Mapping):
        pass
    
    class DescribeInputSecurityGroupRequest(Mapping):
        pass
    
    class DescribeInputSecurityGroupResponse(Mapping):
        pass
    
    class DescribeOfferingRequest(Mapping):
        pass
    
    class DescribeOfferingResponse(Mapping):
        pass
    
    class DescribeReservationRequest(Mapping):
        pass
    
    class DescribeReservationResponse(Mapping):
        pass
    
    class DescribeScheduleRequest(Mapping):
        pass
    
    class DescribeScheduleResponse(Mapping):
        pass
    
    class ListChannelsRequest(Mapping):
        pass
    
    class ListChannelsResponse(Mapping):
        pass
    
    class ListInputSecurityGroupsRequest(Mapping):
        pass
    
    class ListInputSecurityGroupsResponse(Mapping):
        pass
    
    class ListInputsRequest(Mapping):
        pass
    
    class ListInputsResponse(Mapping):
        pass
    
    class ListOfferingsRequest(Mapping):
        pass
    
    class ListOfferingsResponse(Mapping):
        pass
    
    class ListReservationsRequest(Mapping):
        pass
    
    class ListReservationsResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class PurchaseOfferingRequest(Mapping):
        pass
    
    class PurchaseOfferingResponse(Mapping):
        pass
    
    class StartChannelRequest(Mapping):
        pass
    
    class StartChannelResponse(Mapping):
        pass
    
    class StopChannelRequest(Mapping):
        pass
    
    class StopChannelResponse(Mapping):
        pass
    
    class UpdateChannelRequest(Mapping):
        pass
    
    class UpdateChannelResponse(Mapping):
        pass
    
    class UpdateChannelClassRequest(Mapping):
        pass
    
    class UpdateChannelClassResponse(Mapping):
        pass
    
    class UpdateInputRequest(Mapping):
        pass
    
    class UpdateInputResponse(Mapping):
        pass
    
    class UpdateInputSecurityGroupRequest(Mapping):
        pass
    
    class UpdateInputSecurityGroupResponse(Mapping):
        pass
    
    class UpdateReservationRequest(Mapping):
        pass
    
    class UpdateReservationResponse(Mapping):
        pass
    
    def batch_update_schedule(self, ChannelId: str, Creates: BatchScheduleActionCreateRequest=None, Deletes: BatchScheduleActionDeleteRequest=None) -> BatchUpdateScheduleResponse:
        """Update a channel schedule

        :param self:
        :param Deletes:  Schedule actions to delete from the schedule.:<br/>Creates:  Schedule actions to create in the schedule.:<br/>ChannelId:  Id of the channel whose schedule is being updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ScheduleActions</i> A list of schedule actions to create.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActionNames</i> A list of schedule actions to delete.<br/>
        :return: Deletes:  Schedule actions deleted from the schedule.:<br/>Creates:  Schedule actions created in the schedule.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ScheduleActions</i> List of actions that have been created in the schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ScheduleActions</i> List of actions that have been deleted from the schedule.<br/>        """
        pass

    def create_channel(self, ChannelClass: str=None, Destinations: List[OutputDestination]=None, EncoderSettings: EncoderSettings=None, InputAttachments: List[InputAttachment]=None, InputSpecification: InputSpecification=None, LogLevel: str=None, Name: str=None, RequestId: str=None, Reserved: str=None, RoleArn: str=None, Tags: Tags=None) -> CreateChannelResponse:
        """Creates a new channel

        :param self:
        :param Tags:  A collection of key-value pairs.<br/>RoleArn:  An optional Amazon Resource Name (ARN) of the role to assume when running the Channel.<br/>Reserved:  Deprecated field that's only usable by whitelisted customers.<br/>RequestId:  Unique request ID to be specified. This is needed to prevent retries from
creating multiple resources.
<br/>Name:  Name of channel.<br/>LogLevel:  The log level to write to CloudWatch Logs.<br/>InputSpecification:  Specification of input for this channel (max. bitrate, resolution, codec, etc.):<br/>InputAttachments:  List of input attachments for channel.<br/>EncoderSettings:  Encoder Settings:<br/>Destinations:  Placeholder documentation for __listOfOutputDestination<br/>ChannelClass:  The class for this channel. STANDARD for a channel with two pipelines or SINGLE_PIPELINE for a channel with one pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VideoDescriptions</i> Placeholder documentation for __listOfVideoDescription<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimecodeConfig</i> Contains settings used to acquire and adjust timecode information from inputs.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TimecodeConfig</b>: Contains settings used to acquire and adjust timecode information from inputs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputGroups</i> Placeholder documentation for __listOfOutputGroup<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalConfiguration</i> Configuration settings that apply to the event as a whole.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>GlobalConfiguration</b>: Configuration settings that apply to the event as a whole.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CaptionDescriptions</i> Settings for caption decriptions<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BlackoutSlate</i> Settings for blackout slate.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BlackoutSlate</b>: Settings for blackout slate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailConfiguration</i> Event-wide configuration settings for ad avail insertion.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailConfiguration</b>: Event-wide configuration settings for ad avail insertion.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailBlanking</i> Settings for ad avail blanking.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailBlanking</b>: Settings for ad avail blanking.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AudioDescriptions</i> Placeholder documentation for __listOfAudioDescription<br/>Blanking image to be used. Leave empty for solid black. Only bmp and png images are supported.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailBlankingImage</b>: Blanking image to be used. Leave empty for solid black. Only bmp and png images are supported.Ad avail settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailSettings</b>: Ad avail settings.Blackout slate image to be used. Leave empty for solid black. Only bmp and png images are supported.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BlackoutSlateImage</b>: Blackout slate image to be used. Leave empty for solid black. Only bmp and png images are supported.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InputEndAction</i> Indicates the action to take when the current input completes (e.g. end-of-file). When switchAndLoopInputs is configured the encoder will restart at the beginning of the first input.  When "none" is configured the encoder will transcode either black, a solid color, or a user specified slate images per the "Input Loss Behavior" configuration until the next input switch occurs (which is controlled through the Channel Schedule API).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InitialAudioGain</i> Value to set the initial audio gain for the Live Event.<br/>Settings for system actions when input is lost.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputLossBehavior</b>: Settings for system actions when input is lost.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SyncThreshold</i> Threshold in frames beyond which output timecode is resynchronized to the input timecode. Discrepancies below this threshold are permitted to avoid unnecessary discontinuities in the output timecode. No timecode sync when this is not specified.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Source</i> Identifies the source for the timecode that will be associated with the events outputs.
-Embedded (embedded): Initialize the output timecode with timecode from the the source.  If no embedded timecode is detected in the source, the system falls back to using "Start at 0" (zerobased).
-System Clock (systemclock): Use the UTC time.
-Start at 0 (zerobased): The time of the first frame of the event will be 00:00:00:00.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Resolution</i> Input resolution, categorized coarsely<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumBitrate</i> Maximum input bitrate, categorized coarsely<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Codec</i> Input codec<br/>
        :return: Channel:  Placeholder documentation for Channel:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> A collection of key-value pairs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> Placeholder documentation for ChannelState<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The Amazon Resource Name (ARN) of the role assumed when running the Channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PipelinesRunningCount</i> The number of currently healthy pipelines.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the channel. (user-mutable)<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogLevel</i> The log level being written to CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputSpecification</i> Placeholder documentation for InputSpecification&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputSpecification</b>: Placeholder documentation for InputSpecification<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputAttachments</i> List of input attachments for channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique id of the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncoderSettings</i> Encoder Settings&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncoderSettings</b>: Encoder Settings<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EgressEndpoints</i> The endpoints where outgoing connections initiate from<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Destinations</i> A list of destinations of the channel. For UDP outputs, there is one
destination per output. For other types (HLS, for example), there is
one destination per packager.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ChannelClass</i> The class for this channel. STANDARD for a channel with two pipelines or SINGLE_PIPELINE for a channel with one pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The unique arn of the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AudioDescriptions</i> Placeholder documentation for __listOfAudioDescription<br/>Settings for ad avail blanking.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailBlanking</b>: Settings for ad avail blanking.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Resolution</i> Input resolution, categorized coarsely<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumBitrate</i> Maximum input bitrate, categorized coarsely<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Codec</i> Input codec<br/>        """
        pass

    def create_input(self, Destinations: List[InputDestinationRequest]=None, InputSecurityGroups: List[__string]=None, MediaConnectFlows: List[MediaConnectFlowRequest]=None, Name: str=None, RequestId: str=None, RoleArn: str=None, Sources: List[InputSourceRequest]=None, Tags: Tags=None, Type: str=None, Vpc: InputVpcRequest=None) -> CreateInputResponse:
        """Create an input

        :param self:
        :param Vpc:  Settings for a private VPC Input.
When this property is specified, the input destination addresses will be created in a VPC rather than with public Internet addresses.
This property requires setting the roleArn property on Input creation.
Not compatible with the inputSecurityGroups property.
:<br/>Type:  Placeholder documentation for InputType<br/>Tags:  A collection of key-value pairs.<br/>Sources:  The source URLs for a PULL-type input. Every PULL type input needs
exactly two source URLs for redundancy.
Only specify sources for PULL type Inputs. Leave Destinations empty.
<br/>RoleArn:  The Amazon Resource Name (ARN) of the role this input assumes during and after creation.<br/>RequestId:  Unique identifier of the request to ensure the request is handled
exactly once in case of retries.
<br/>Name:  Name of the input.<br/>MediaConnectFlows:  A list of the MediaConnect Flows that you want to use in this input. You can specify as few as one
Flow and presently, as many as two. The only requirement is when you have more than one is that each Flow is in a
separate Availability Zone as this ensures your EML input is redundant to AZ issues.
<br/>InputSecurityGroups:  A list of security groups referenced by IDs to attach to the input.<br/>Destinations:  Destination settings for PUSH type inputs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i> A list of 2 VPC subnet IDs from the same VPC.
Subnet IDs must be mapped to two unique availability zones (AZ).
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> A list of up to 5 EC2 VPC security group IDs to attach to the Input VPC network interfaces.
Requires subnetIds. If none are specified then the VPC default security group will be used.
<br/>
        :return: Input:  Placeholder documentation for Input:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> Placeholder documentation for InputType<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> A collection of key-value pairs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> Placeholder documentation for InputState<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Sources</i> A list of the sources of the input (PULL-type).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroups</i> A list of IDs for all the Input Security Groups attached to the input.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The Amazon Resource Name (ARN) of the role this input assumes during and after creation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The user-assigned name (This is a mutable value).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MediaConnectFlows</i> A list of MediaConnect Flows for this input.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputClass</i> STANDARD - MediaLive expects two sources to be connected to this input. If the channel is also STANDARD, both sources will be ingested. If the channel is SINGLE_PIPELINE, only the first source will be ingested; the second source will always be ignored, even if the first source fails.
SINGLE_PIPELINE - You can connect only one source to this input. If the ChannelClass is also  SINGLE_PIPELINE, this value is valid. If the ChannelClass is STANDARD, this value is not valid because the channel requires two sources in the input.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The generated ID of the input (unique for user account, immutable).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Destinations</i> A list of the destinations of the input (PUSH-type).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttachedChannels</i> A list of channel IDs that that input is attached to (currently an input can only be attached to one channel).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Unique ARN of the input (generated, immutable).<br/>        """
        pass

    def create_input_security_group(self, Tags: Tags=None, WhitelistRules: List[InputWhitelistRuleCidr]=None) -> CreateInputSecurityGroupResponse:
        """Creates a Input Security Group

        :param self:
        :param WhitelistRules:  List of IPv4 CIDR addresses to whitelist<br/>Tags:  A collection of key-value pairs.<br/>
        :return: SecurityGroup:  An Input Security Group:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WhitelistRules</i> Whitelist rules and their sync status<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> A collection of key-value pairs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The current state of the Input Security Group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Inputs</i> The list of inputs currently using this Input Security Group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The Id of the Input Security Group<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> Unique ARN of Input Security Group<br/>        """
        pass

    def create_tags(self, ResourceArn: str, Tags: Tags=None):
        """Create tags for a resource

        :param self:
        :param Tags:  Placeholder documentation for Tags<br/>ResourceArn:  Placeholder documentation for __string<br/>
        :return:         """
        pass

    def delete_channel(self, ChannelId: str) -> DeleteChannelResponse:
        """Starts deletion of channel. The associated outputs are also deleted.

        :param self:
        :param ChannelId:  Unique ID of the channel.<br/>
        :return: Tags:  A collection of key-value pairs.<br/>State:  Placeholder documentation for ChannelState<br/>RoleArn:  The Amazon Resource Name (ARN) of the role assumed when running the Channel.<br/>PipelinesRunningCount:  The number of currently healthy pipelines.<br/>Name:  The name of the channel. (user-mutable)<br/>LogLevel:  The log level being written to CloudWatch Logs.<br/>InputSpecification:  Placeholder documentation for InputSpecification:<br/>InputAttachments:  List of input attachments for channel.<br/>Id:  The unique id of the channel.<br/>EncoderSettings:  Encoder Settings:<br/>EgressEndpoints:  The endpoints where outgoing connections initiate from<br/>Destinations:  A list of destinations of the channel. For UDP outputs, there is one
destination per output. For other types (HLS, for example), there is
one destination per packager.
<br/>ChannelClass:  The class for this channel. STANDARD for a channel with two pipelines or SINGLE_PIPELINE for a channel with one pipeline.<br/>Arn:  The unique arn of the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VideoDescriptions</i> Placeholder documentation for __listOfVideoDescription<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimecodeConfig</i> Contains settings used to acquire and adjust timecode information from inputs.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TimecodeConfig</b>: Contains settings used to acquire and adjust timecode information from inputs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputGroups</i> Placeholder documentation for __listOfOutputGroup<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalConfiguration</i> Configuration settings that apply to the event as a whole.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>GlobalConfiguration</b>: Configuration settings that apply to the event as a whole.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CaptionDescriptions</i> Settings for caption decriptions<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BlackoutSlate</i> Settings for blackout slate.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BlackoutSlate</b>: Settings for blackout slate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailConfiguration</i> Event-wide configuration settings for ad avail insertion.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailConfiguration</b>: Event-wide configuration settings for ad avail insertion.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailBlanking</i> Settings for ad avail blanking.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailBlanking</b>: Settings for ad avail blanking.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AudioDescriptions</i> Placeholder documentation for __listOfAudioDescription<br/>Blanking image to be used. Leave empty for solid black. Only bmp and png images are supported.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailBlankingImage</b>: Blanking image to be used. Leave empty for solid black. Only bmp and png images are supported.Ad avail settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailSettings</b>: Ad avail settings.Blackout slate image to be used. Leave empty for solid black. Only bmp and png images are supported.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BlackoutSlateImage</b>: Blackout slate image to be used. Leave empty for solid black. Only bmp and png images are supported.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InputEndAction</i> Indicates the action to take when the current input completes (e.g. end-of-file). When switchAndLoopInputs is configured the encoder will restart at the beginning of the first input.  When "none" is configured the encoder will transcode either black, a solid color, or a user specified slate images per the "Input Loss Behavior" configuration until the next input switch occurs (which is controlled through the Channel Schedule API).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InitialAudioGain</i> Value to set the initial audio gain for the Live Event.<br/>Settings for system actions when input is lost.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputLossBehavior</b>: Settings for system actions when input is lost.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SyncThreshold</i> Threshold in frames beyond which output timecode is resynchronized to the input timecode. Discrepancies below this threshold are permitted to avoid unnecessary discontinuities in the output timecode. No timecode sync when this is not specified.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Source</i> Identifies the source for the timecode that will be associated with the events outputs.
-Embedded (embedded): Initialize the output timecode with timecode from the the source.  If no embedded timecode is detected in the source, the system falls back to using "Start at 0" (zerobased).
-System Clock (systemclock): Use the UTC time.
-Start at 0 (zerobased): The time of the first frame of the event will be 00:00:00:00.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Resolution</i> Input resolution, categorized coarsely<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumBitrate</i> Maximum input bitrate, categorized coarsely<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Codec</i> Input codec<br/>        """
        pass

    def delete_input(self, InputId: str) -> DeleteInputResponse:
        """Deletes the input end point

        :param self:
        :param InputId:  Unique ID of the input<br/>
        :return:         """
        pass

    def delete_input_security_group(self, InputSecurityGroupId: str) -> DeleteInputSecurityGroupResponse:
        """Deletes an Input Security Group

        :param self:
        :param InputSecurityGroupId:  The Input Security Group to delete<br/>
        :return:         """
        pass

    def delete_reservation(self, ReservationId: str) -> DeleteReservationResponse:
        """Delete an expired reservation.

        :param self:
        :param ReservationId:  Unique reservation ID, e.g. '1234567'<br/>
        :return: UsagePrice:  Recurring usage charge for each reserved resource, e.g. '157.0'<br/>Tags:  A collection of key-value pairs<br/>State:  Current state of reservation, e.g. 'ACTIVE'<br/>Start:  Reservation UTC start date and time in ISO-8601 format, e.g. '2018-03-01T00:00:00'<br/>ResourceSpecification:  Resource configuration details:<br/>ReservationId:  Unique reservation ID, e.g. '1234567'<br/>Region:  AWS region, e.g. 'us-west-2'<br/>OfferingType:  Offering type, e.g. 'NO_UPFRONT'<br/>OfferingId:  Unique offering ID, e.g. '87654321'<br/>OfferingDescription:  Offering description, e.g. 'HD AVC output at 10-20 Mbps, 30 fps, and standard VQ in US West (Oregon)'<br/>Name:  User specified reservation name<br/>FixedPrice:  One-time charge for each reserved resource, e.g. '0.0' for a NO_UPFRONT offering<br/>End:  Reservation UTC end date and time in ISO-8601 format, e.g. '2019-03-01T00:00:00'<br/>DurationUnits:  Units for duration, e.g. 'MONTHS'<br/>Duration:  Lease duration, e.g. '12'<br/>CurrencyCode:  Currency code for usagePrice and fixedPrice in ISO-4217 format, e.g. 'USD'<br/>Count:  Number of reserved resources<br/>Arn:  Unique reservation ARN, e.g. 'arn:aws:medialive:us-west-2:123456789012:reservation:1234567'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VideoQuality</i> Video quality, e.g. 'STANDARD' (Outputs only)<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SpecialFeature</i> Special feature, e.g. 'AUDIO_NORMALIZATION' (Channels only)<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceType</i> Resource type, 'INPUT', 'OUTPUT', or 'CHANNEL'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Resolution</i> Resolution, e.g. 'HD'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumFramerate</i> Maximum framerate, e.g. 'MAX_30_FPS' (Outputs only)<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumBitrate</i> Maximum bitrate, e.g. 'MAX_20_MBPS'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Codec</i> Codec, e.g. 'AVC'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ChannelClass</i> Channel class, e.g. 'STANDARD'<br/>        """
        pass

    def delete_schedule(self, ChannelId: str) -> DeleteScheduleResponse:
        """Delete all schedule actions on a channel.

        :param self:
        :param ChannelId:  Id of the channel whose schedule is being deleted.<br/>
        :return:         """
        pass

    def delete_tags(self, TagKeys: List[__string], ResourceArn: str):
        """Removes tags for a resource

        :param self:
        :param TagKeys:  An array of tag keys to delete<br/>ResourceArn:  Placeholder documentation for __string<br/>
        :return:         """
        pass

    def describe_channel(self, ChannelId: str) -> DescribeChannelResponse:
        """Gets details about a channel

        :param self:
        :param ChannelId:  channel ID<br/>
        :return: Tags:  A collection of key-value pairs.<br/>State:  Placeholder documentation for ChannelState<br/>RoleArn:  The Amazon Resource Name (ARN) of the role assumed when running the Channel.<br/>PipelinesRunningCount:  The number of currently healthy pipelines.<br/>Name:  The name of the channel. (user-mutable)<br/>LogLevel:  The log level being written to CloudWatch Logs.<br/>InputSpecification:  Placeholder documentation for InputSpecification:<br/>InputAttachments:  List of input attachments for channel.<br/>Id:  The unique id of the channel.<br/>EncoderSettings:  Encoder Settings:<br/>EgressEndpoints:  The endpoints where outgoing connections initiate from<br/>Destinations:  A list of destinations of the channel. For UDP outputs, there is one
destination per output. For other types (HLS, for example), there is
one destination per packager.
<br/>ChannelClass:  The class for this channel. STANDARD for a channel with two pipelines or SINGLE_PIPELINE for a channel with one pipeline.<br/>Arn:  The unique arn of the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VideoDescriptions</i> Placeholder documentation for __listOfVideoDescription<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimecodeConfig</i> Contains settings used to acquire and adjust timecode information from inputs.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TimecodeConfig</b>: Contains settings used to acquire and adjust timecode information from inputs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputGroups</i> Placeholder documentation for __listOfOutputGroup<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalConfiguration</i> Configuration settings that apply to the event as a whole.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>GlobalConfiguration</b>: Configuration settings that apply to the event as a whole.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CaptionDescriptions</i> Settings for caption decriptions<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BlackoutSlate</i> Settings for blackout slate.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BlackoutSlate</b>: Settings for blackout slate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailConfiguration</i> Event-wide configuration settings for ad avail insertion.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailConfiguration</b>: Event-wide configuration settings for ad avail insertion.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailBlanking</i> Settings for ad avail blanking.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailBlanking</b>: Settings for ad avail blanking.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AudioDescriptions</i> Placeholder documentation for __listOfAudioDescription<br/>Blanking image to be used. Leave empty for solid black. Only bmp and png images are supported.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailBlankingImage</b>: Blanking image to be used. Leave empty for solid black. Only bmp and png images are supported.Ad avail settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailSettings</b>: Ad avail settings.Blackout slate image to be used. Leave empty for solid black. Only bmp and png images are supported.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BlackoutSlateImage</b>: Blackout slate image to be used. Leave empty for solid black. Only bmp and png images are supported.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InputEndAction</i> Indicates the action to take when the current input completes (e.g. end-of-file). When switchAndLoopInputs is configured the encoder will restart at the beginning of the first input.  When "none" is configured the encoder will transcode either black, a solid color, or a user specified slate images per the "Input Loss Behavior" configuration until the next input switch occurs (which is controlled through the Channel Schedule API).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InitialAudioGain</i> Value to set the initial audio gain for the Live Event.<br/>Settings for system actions when input is lost.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputLossBehavior</b>: Settings for system actions when input is lost.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SyncThreshold</i> Threshold in frames beyond which output timecode is resynchronized to the input timecode. Discrepancies below this threshold are permitted to avoid unnecessary discontinuities in the output timecode. No timecode sync when this is not specified.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Source</i> Identifies the source for the timecode that will be associated with the events outputs.
-Embedded (embedded): Initialize the output timecode with timecode from the the source.  If no embedded timecode is detected in the source, the system falls back to using "Start at 0" (zerobased).
-System Clock (systemclock): Use the UTC time.
-Start at 0 (zerobased): The time of the first frame of the event will be 00:00:00:00.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Resolution</i> Input resolution, categorized coarsely<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumBitrate</i> Maximum input bitrate, categorized coarsely<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Codec</i> Input codec<br/>        """
        pass

    def describe_input(self, InputId: str) -> DescribeInputResponse:
        """Produces details about an input

        :param self:
        :param InputId:  Unique ID of the input<br/>
        :return: Type:  Placeholder documentation for InputType<br/>Tags:  A collection of key-value pairs.<br/>State:  Placeholder documentation for InputState<br/>Sources:  A list of the sources of the input (PULL-type).<br/>SecurityGroups:  A list of IDs for all the Input Security Groups attached to the input.<br/>RoleArn:  The Amazon Resource Name (ARN) of the role this input assumes during and after creation.<br/>Name:  The user-assigned name (This is a mutable value).<br/>MediaConnectFlows:  A list of MediaConnect Flows for this input.<br/>InputClass:  STANDARD - MediaLive expects two sources to be connected to this input. If the channel is also STANDARD, both sources will be ingested. If the channel is SINGLE_PIPELINE, only the first source will be ingested; the second source will always be ignored, even if the first source fails.
SINGLE_PIPELINE - You can connect only one source to this input. If the ChannelClass is also  SINGLE_PIPELINE, this value is valid. If the ChannelClass is STANDARD, this value is not valid because the channel requires two sources in the input.
<br/>Id:  The generated ID of the input (unique for user account, immutable).<br/>Destinations:  A list of the destinations of the input (PUSH-type).<br/>AttachedChannels:  A list of channel IDs that that input is attached to (currently an input can only be attached to one channel).<br/>Arn:  The Unique ARN of the input (generated, immutable).<br/>        """
        pass

    def describe_input_security_group(self, InputSecurityGroupId: str) -> DescribeInputSecurityGroupResponse:
        """Produces a summary of an Input Security Group

        :param self:
        :param InputSecurityGroupId:  The id of the Input Security Group to describe<br/>
        :return: WhitelistRules:  Whitelist rules and their sync status<br/>Tags:  A collection of key-value pairs.<br/>State:  The current state of the Input Security Group.<br/>Inputs:  The list of inputs currently using this Input Security Group.<br/>Id:  The Id of the Input Security Group<br/>Arn:  Unique ARN of Input Security Group<br/>        """
        pass

    def describe_offering(self, OfferingId: str) -> DescribeOfferingResponse:
        """Get details for an offering.

        :param self:
        :param OfferingId:  Unique offering ID, e.g. '87654321'<br/>
        :return: UsagePrice:  Recurring usage charge for each reserved resource, e.g. '157.0'<br/>ResourceSpecification:  Resource configuration details:<br/>Region:  AWS region, e.g. 'us-west-2'<br/>OfferingType:  Offering type, e.g. 'NO_UPFRONT'<br/>OfferingId:  Unique offering ID, e.g. '87654321'<br/>OfferingDescription:  Offering description, e.g. 'HD AVC output at 10-20 Mbps, 30 fps, and standard VQ in US West (Oregon)'<br/>FixedPrice:  One-time charge for each reserved resource, e.g. '0.0' for a NO_UPFRONT offering<br/>DurationUnits:  Units for duration, e.g. 'MONTHS'<br/>Duration:  Lease duration, e.g. '12'<br/>CurrencyCode:  Currency code for usagePrice and fixedPrice in ISO-4217 format, e.g. 'USD'<br/>Arn:  Unique offering ARN, e.g. 'arn:aws:medialive:us-west-2:123456789012:offering:87654321'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VideoQuality</i> Video quality, e.g. 'STANDARD' (Outputs only)<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SpecialFeature</i> Special feature, e.g. 'AUDIO_NORMALIZATION' (Channels only)<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceType</i> Resource type, 'INPUT', 'OUTPUT', or 'CHANNEL'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Resolution</i> Resolution, e.g. 'HD'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumFramerate</i> Maximum framerate, e.g. 'MAX_30_FPS' (Outputs only)<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumBitrate</i> Maximum bitrate, e.g. 'MAX_20_MBPS'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Codec</i> Codec, e.g. 'AVC'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ChannelClass</i> Channel class, e.g. 'STANDARD'<br/>        """
        pass

    def describe_reservation(self, ReservationId: str) -> DescribeReservationResponse:
        """Get details for a reservation.

        :param self:
        :param ReservationId:  Unique reservation ID, e.g. '1234567'<br/>
        :return: UsagePrice:  Recurring usage charge for each reserved resource, e.g. '157.0'<br/>Tags:  A collection of key-value pairs<br/>State:  Current state of reservation, e.g. 'ACTIVE'<br/>Start:  Reservation UTC start date and time in ISO-8601 format, e.g. '2018-03-01T00:00:00'<br/>ResourceSpecification:  Resource configuration details:<br/>ReservationId:  Unique reservation ID, e.g. '1234567'<br/>Region:  AWS region, e.g. 'us-west-2'<br/>OfferingType:  Offering type, e.g. 'NO_UPFRONT'<br/>OfferingId:  Unique offering ID, e.g. '87654321'<br/>OfferingDescription:  Offering description, e.g. 'HD AVC output at 10-20 Mbps, 30 fps, and standard VQ in US West (Oregon)'<br/>Name:  User specified reservation name<br/>FixedPrice:  One-time charge for each reserved resource, e.g. '0.0' for a NO_UPFRONT offering<br/>End:  Reservation UTC end date and time in ISO-8601 format, e.g. '2019-03-01T00:00:00'<br/>DurationUnits:  Units for duration, e.g. 'MONTHS'<br/>Duration:  Lease duration, e.g. '12'<br/>CurrencyCode:  Currency code for usagePrice and fixedPrice in ISO-4217 format, e.g. 'USD'<br/>Count:  Number of reserved resources<br/>Arn:  Unique reservation ARN, e.g. 'arn:aws:medialive:us-west-2:123456789012:reservation:1234567'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VideoQuality</i> Video quality, e.g. 'STANDARD' (Outputs only)<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SpecialFeature</i> Special feature, e.g. 'AUDIO_NORMALIZATION' (Channels only)<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceType</i> Resource type, 'INPUT', 'OUTPUT', or 'CHANNEL'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Resolution</i> Resolution, e.g. 'HD'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumFramerate</i> Maximum framerate, e.g. 'MAX_30_FPS' (Outputs only)<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumBitrate</i> Maximum bitrate, e.g. 'MAX_20_MBPS'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Codec</i> Codec, e.g. 'AVC'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ChannelClass</i> Channel class, e.g. 'STANDARD'<br/>        """
        pass

    def describe_schedule(self, ChannelId: str, MaxResults: int=None, NextToken: str=None) -> DescribeScheduleResponse:
        """Get a channel schedule

        :param self:
        :param NextToken:  Placeholder documentation for __string<br/>MaxResults:  Placeholder documentation for MaxResults<br/>ChannelId:  Id of the channel whose schedule is being updated.<br/>
        :return: ScheduleActions:  The list of actions in the schedule.<br/>NextToken:  The next token; for use in pagination.<br/>        """
        pass

    def list_channels(self, MaxResults: int=None, NextToken: str=None) -> ListChannelsResponse:
        """Produces list of channels that have been created

        :param self:
        :param NextToken:  Placeholder documentation for __string<br/>MaxResults:  Placeholder documentation for MaxResults<br/>
        :return: NextToken:  Placeholder documentation for __string<br/>Channels:  Placeholder documentation for __listOfChannelSummary<br/>        """
        pass

    def list_input_security_groups(self, MaxResults: int=None, NextToken: str=None) -> ListInputSecurityGroupsResponse:
        """Produces a list of Input Security Groups for an account

        :param self:
        :param NextToken:  Placeholder documentation for __string<br/>MaxResults:  Placeholder documentation for MaxResults<br/>
        :return: NextToken:  Placeholder documentation for __string<br/>InputSecurityGroups:  List of input security groups<br/>        """
        pass

    def list_inputs(self, MaxResults: int=None, NextToken: str=None) -> ListInputsResponse:
        """Produces list of inputs that have been created

        :param self:
        :param NextToken:  Placeholder documentation for __string<br/>MaxResults:  Placeholder documentation for MaxResults<br/>
        :return: NextToken:  Placeholder documentation for __string<br/>Inputs:  Placeholder documentation for __listOfInput<br/>        """
        pass

    def list_offerings(self, ChannelClass: str=None, ChannelConfiguration: str=None, Codec: str=None, MaxResults: int=None, MaximumBitrate: str=None, MaximumFramerate: str=None, NextToken: str=None, Resolution: str=None, ResourceType: str=None, SpecialFeature: str=None, VideoQuality: str=None) -> ListOfferingsResponse:
        """List offerings available for purchase.

        :param self:
        :param VideoQuality:  Filter by video quality, 'STANDARD', 'ENHANCED', or 'PREMIUM'
<br/>SpecialFeature:  Filter by special feature, 'ADVANCED_AUDIO' or 'AUDIO_NORMALIZATION'
<br/>ResourceType:  Filter by resource type, 'INPUT', 'OUTPUT', or 'CHANNEL'<br/>Resolution:  Filter by resolution, 'SD', 'HD', or 'UHD'<br/>NextToken:  Placeholder documentation for __string<br/>MaximumFramerate:  Filter by framerate, 'MAX_30_FPS' or 'MAX_60_FPS'<br/>MaximumBitrate:  Filter by bitrate, 'MAX_10_MBPS', 'MAX_20_MBPS', or 'MAX_50_MBPS'
<br/>MaxResults:  Placeholder documentation for MaxResults<br/>Codec:  Filter by codec, 'AVC', 'HEVC', 'MPEG2', or 'AUDIO'<br/>ChannelConfiguration:  Filter to offerings that match the configuration of an existing channel, e.g. '2345678' (a channel ID)
<br/>ChannelClass:  Filter by channel class, 'STANDARD' or 'SINGLE_PIPELINE'
<br/>
        :return: Offerings:  List of offerings<br/>NextToken:  Token to retrieve the next page of results<br/>        """
        pass

    def list_reservations(self, ChannelClass: str=None, Codec: str=None, MaxResults: int=None, MaximumBitrate: str=None, MaximumFramerate: str=None, NextToken: str=None, Resolution: str=None, ResourceType: str=None, SpecialFeature: str=None, VideoQuality: str=None) -> ListReservationsResponse:
        """List purchased reservations.

        :param self:
        :param VideoQuality:  Filter by video quality, 'STANDARD', 'ENHANCED', or 'PREMIUM'
<br/>SpecialFeature:  Filter by special feature, 'ADVANCED_AUDIO' or 'AUDIO_NORMALIZATION'
<br/>ResourceType:  Filter by resource type, 'INPUT', 'OUTPUT', or 'CHANNEL'<br/>Resolution:  Filter by resolution, 'SD', 'HD', or 'UHD'<br/>NextToken:  Placeholder documentation for __string<br/>MaximumFramerate:  Filter by framerate, 'MAX_30_FPS' or 'MAX_60_FPS'<br/>MaximumBitrate:  Filter by bitrate, 'MAX_10_MBPS', 'MAX_20_MBPS', or 'MAX_50_MBPS'
<br/>MaxResults:  Placeholder documentation for MaxResults<br/>Codec:  Filter by codec, 'AVC', 'HEVC', 'MPEG2', or 'AUDIO'<br/>ChannelClass:  Filter by channel class, 'STANDARD' or 'SINGLE_PIPELINE'
<br/>
        :return: Reservations:  List of reservations<br/>NextToken:  Token to retrieve the next page of results<br/>        """
        pass

    def list_tags_for_resource(self, ResourceArn: str) -> ListTagsForResourceResponse:
        """Produces list of tags that have been created for a resource

        :param self:
        :param ResourceArn:  Placeholder documentation for __string<br/>
        :return: Tags:  Placeholder documentation for Tags<br/>        """
        pass

    def purchase_offering(self, OfferingId: str, Count: int, Name: str=None, RequestId: str=None, Start: str=None, Tags: Tags=None) -> PurchaseOfferingResponse:
        """Purchase an offering and create a reservation.

        :param self:
        :param Tags:  A collection of key-value pairs<br/>Start:  Requested reservation start time (UTC) in ISO-8601 format. The specified time must be between the first day of the current month and one year from now. If no value is given, the default is now.<br/>RequestId:  Unique request ID to be specified. This is needed to prevent retries from creating multiple resources.<br/>OfferingId:  Offering to purchase, e.g. '87654321'<br/>Name:  Name for the new reservation<br/>Count:  Number of resources<br/>
        :return: Reservation:  Reserved resources available to use:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UsagePrice</i> Recurring usage charge for each reserved resource, e.g. '157.0'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> A collection of key-value pairs<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> Current state of reservation, e.g. 'ACTIVE'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Start</i> Reservation UTC start date and time in ISO-8601 format, e.g. '2018-03-01T00:00:00'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceSpecification</i> Resource configuration details&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResourceSpecification</b>: Resource configuration details<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservationId</i> Unique reservation ID, e.g. '1234567'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Region</i> AWS region, e.g. 'us-west-2'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OfferingType</i> Offering type, e.g. 'NO_UPFRONT'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OfferingId</i> Unique offering ID, e.g. '87654321'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OfferingDescription</i> Offering description, e.g. 'HD AVC output at 10-20 Mbps, 30 fps, and standard VQ in US West (Oregon)'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> User specified reservation name<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FixedPrice</i> One-time charge for each reserved resource, e.g. '0.0' for a NO_UPFRONT offering<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>End</i> Reservation UTC end date and time in ISO-8601 format, e.g. '2019-03-01T00:00:00'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DurationUnits</i> Units for duration, e.g. 'MONTHS'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Duration</i> Lease duration, e.g. '12'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrencyCode</i> Currency code for usagePrice and fixedPrice in ISO-4217 format, e.g. 'USD'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Count</i> Number of reserved resources<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> Unique reservation ARN, e.g. 'arn:aws:medialive:us-west-2:123456789012:reservation:1234567'<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VideoQuality</i> Video quality, e.g. 'STANDARD' (Outputs only)<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SpecialFeature</i> Special feature, e.g. 'AUDIO_NORMALIZATION' (Channels only)<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceType</i> Resource type, 'INPUT', 'OUTPUT', or 'CHANNEL'<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Resolution</i> Resolution, e.g. 'HD'<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumFramerate</i> Maximum framerate, e.g. 'MAX_30_FPS' (Outputs only)<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumBitrate</i> Maximum bitrate, e.g. 'MAX_20_MBPS'<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Codec</i> Codec, e.g. 'AVC'<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ChannelClass</i> Channel class, e.g. 'STANDARD'<br/>        """
        pass

    def start_channel(self, ChannelId: str) -> StartChannelResponse:
        """Starts an existing channel

        :param self:
        :param ChannelId:  A request to start a channel<br/>
        :return: Tags:  A collection of key-value pairs.<br/>State:  Placeholder documentation for ChannelState<br/>RoleArn:  The Amazon Resource Name (ARN) of the role assumed when running the Channel.<br/>PipelinesRunningCount:  The number of currently healthy pipelines.<br/>Name:  The name of the channel. (user-mutable)<br/>LogLevel:  The log level being written to CloudWatch Logs.<br/>InputSpecification:  Placeholder documentation for InputSpecification:<br/>InputAttachments:  List of input attachments for channel.<br/>Id:  The unique id of the channel.<br/>EncoderSettings:  Encoder Settings:<br/>EgressEndpoints:  The endpoints where outgoing connections initiate from<br/>Destinations:  A list of destinations of the channel. For UDP outputs, there is one
destination per output. For other types (HLS, for example), there is
one destination per packager.
<br/>ChannelClass:  The class for this channel. STANDARD for a channel with two pipelines or SINGLE_PIPELINE for a channel with one pipeline.<br/>Arn:  The unique arn of the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VideoDescriptions</i> Placeholder documentation for __listOfVideoDescription<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimecodeConfig</i> Contains settings used to acquire and adjust timecode information from inputs.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TimecodeConfig</b>: Contains settings used to acquire and adjust timecode information from inputs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputGroups</i> Placeholder documentation for __listOfOutputGroup<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalConfiguration</i> Configuration settings that apply to the event as a whole.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>GlobalConfiguration</b>: Configuration settings that apply to the event as a whole.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CaptionDescriptions</i> Settings for caption decriptions<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BlackoutSlate</i> Settings for blackout slate.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BlackoutSlate</b>: Settings for blackout slate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailConfiguration</i> Event-wide configuration settings for ad avail insertion.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailConfiguration</b>: Event-wide configuration settings for ad avail insertion.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailBlanking</i> Settings for ad avail blanking.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailBlanking</b>: Settings for ad avail blanking.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AudioDescriptions</i> Placeholder documentation for __listOfAudioDescription<br/>Blanking image to be used. Leave empty for solid black. Only bmp and png images are supported.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailBlankingImage</b>: Blanking image to be used. Leave empty for solid black. Only bmp and png images are supported.Ad avail settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailSettings</b>: Ad avail settings.Blackout slate image to be used. Leave empty for solid black. Only bmp and png images are supported.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BlackoutSlateImage</b>: Blackout slate image to be used. Leave empty for solid black. Only bmp and png images are supported.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InputEndAction</i> Indicates the action to take when the current input completes (e.g. end-of-file). When switchAndLoopInputs is configured the encoder will restart at the beginning of the first input.  When "none" is configured the encoder will transcode either black, a solid color, or a user specified slate images per the "Input Loss Behavior" configuration until the next input switch occurs (which is controlled through the Channel Schedule API).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InitialAudioGain</i> Value to set the initial audio gain for the Live Event.<br/>Settings for system actions when input is lost.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputLossBehavior</b>: Settings for system actions when input is lost.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SyncThreshold</i> Threshold in frames beyond which output timecode is resynchronized to the input timecode. Discrepancies below this threshold are permitted to avoid unnecessary discontinuities in the output timecode. No timecode sync when this is not specified.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Source</i> Identifies the source for the timecode that will be associated with the events outputs.
-Embedded (embedded): Initialize the output timecode with timecode from the the source.  If no embedded timecode is detected in the source, the system falls back to using "Start at 0" (zerobased).
-System Clock (systemclock): Use the UTC time.
-Start at 0 (zerobased): The time of the first frame of the event will be 00:00:00:00.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Resolution</i> Input resolution, categorized coarsely<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumBitrate</i> Maximum input bitrate, categorized coarsely<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Codec</i> Input codec<br/>        """
        pass

    def stop_channel(self, ChannelId: str) -> StopChannelResponse:
        """Stops a running channel

        :param self:
        :param ChannelId:  A request to stop a running channel<br/>
        :return: Tags:  A collection of key-value pairs.<br/>State:  Placeholder documentation for ChannelState<br/>RoleArn:  The Amazon Resource Name (ARN) of the role assumed when running the Channel.<br/>PipelinesRunningCount:  The number of currently healthy pipelines.<br/>Name:  The name of the channel. (user-mutable)<br/>LogLevel:  The log level being written to CloudWatch Logs.<br/>InputSpecification:  Placeholder documentation for InputSpecification:<br/>InputAttachments:  List of input attachments for channel.<br/>Id:  The unique id of the channel.<br/>EncoderSettings:  Encoder Settings:<br/>EgressEndpoints:  The endpoints where outgoing connections initiate from<br/>Destinations:  A list of destinations of the channel. For UDP outputs, there is one
destination per output. For other types (HLS, for example), there is
one destination per packager.
<br/>ChannelClass:  The class for this channel. STANDARD for a channel with two pipelines or SINGLE_PIPELINE for a channel with one pipeline.<br/>Arn:  The unique arn of the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VideoDescriptions</i> Placeholder documentation for __listOfVideoDescription<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimecodeConfig</i> Contains settings used to acquire and adjust timecode information from inputs.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TimecodeConfig</b>: Contains settings used to acquire and adjust timecode information from inputs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputGroups</i> Placeholder documentation for __listOfOutputGroup<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalConfiguration</i> Configuration settings that apply to the event as a whole.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>GlobalConfiguration</b>: Configuration settings that apply to the event as a whole.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CaptionDescriptions</i> Settings for caption decriptions<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BlackoutSlate</i> Settings for blackout slate.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BlackoutSlate</b>: Settings for blackout slate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailConfiguration</i> Event-wide configuration settings for ad avail insertion.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailConfiguration</b>: Event-wide configuration settings for ad avail insertion.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailBlanking</i> Settings for ad avail blanking.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailBlanking</b>: Settings for ad avail blanking.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AudioDescriptions</i> Placeholder documentation for __listOfAudioDescription<br/>Blanking image to be used. Leave empty for solid black. Only bmp and png images are supported.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailBlankingImage</b>: Blanking image to be used. Leave empty for solid black. Only bmp and png images are supported.Ad avail settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailSettings</b>: Ad avail settings.Blackout slate image to be used. Leave empty for solid black. Only bmp and png images are supported.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BlackoutSlateImage</b>: Blackout slate image to be used. Leave empty for solid black. Only bmp and png images are supported.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InputEndAction</i> Indicates the action to take when the current input completes (e.g. end-of-file). When switchAndLoopInputs is configured the encoder will restart at the beginning of the first input.  When "none" is configured the encoder will transcode either black, a solid color, or a user specified slate images per the "Input Loss Behavior" configuration until the next input switch occurs (which is controlled through the Channel Schedule API).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InitialAudioGain</i> Value to set the initial audio gain for the Live Event.<br/>Settings for system actions when input is lost.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputLossBehavior</b>: Settings for system actions when input is lost.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SyncThreshold</i> Threshold in frames beyond which output timecode is resynchronized to the input timecode. Discrepancies below this threshold are permitted to avoid unnecessary discontinuities in the output timecode. No timecode sync when this is not specified.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Source</i> Identifies the source for the timecode that will be associated with the events outputs.
-Embedded (embedded): Initialize the output timecode with timecode from the the source.  If no embedded timecode is detected in the source, the system falls back to using "Start at 0" (zerobased).
-System Clock (systemclock): Use the UTC time.
-Start at 0 (zerobased): The time of the first frame of the event will be 00:00:00:00.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Resolution</i> Input resolution, categorized coarsely<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumBitrate</i> Maximum input bitrate, categorized coarsely<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Codec</i> Input codec<br/>        """
        pass

    def update_channel(self, ChannelId: str, Destinations: List[OutputDestination]=None, EncoderSettings: EncoderSettings=None, InputAttachments: List[InputAttachment]=None, InputSpecification: InputSpecification=None, LogLevel: str=None, Name: str=None, RoleArn: str=None) -> UpdateChannelResponse:
        """Updates a channel.

        :param self:
        :param RoleArn:  An optional Amazon Resource Name (ARN) of the role to assume when running the Channel. If you do not specify this on an update call but the role was previously set that role will be removed.<br/>Name:  The name of the channel.<br/>LogLevel:  The log level to write to CloudWatch Logs.<br/>InputSpecification:  Specification of input for this channel (max. bitrate, resolution, codec, etc.):<br/>InputAttachments:  Placeholder documentation for __listOfInputAttachment<br/>EncoderSettings:  The encoder settings for this channel.:<br/>Destinations:  A list of output destinations for this channel.<br/>ChannelId:  channel ID<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VideoDescriptions</i> Placeholder documentation for __listOfVideoDescription<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimecodeConfig</i> Contains settings used to acquire and adjust timecode information from inputs.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TimecodeConfig</b>: Contains settings used to acquire and adjust timecode information from inputs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputGroups</i> Placeholder documentation for __listOfOutputGroup<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalConfiguration</i> Configuration settings that apply to the event as a whole.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>GlobalConfiguration</b>: Configuration settings that apply to the event as a whole.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CaptionDescriptions</i> Settings for caption decriptions<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BlackoutSlate</i> Settings for blackout slate.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BlackoutSlate</b>: Settings for blackout slate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailConfiguration</i> Event-wide configuration settings for ad avail insertion.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailConfiguration</b>: Event-wide configuration settings for ad avail insertion.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailBlanking</i> Settings for ad avail blanking.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailBlanking</b>: Settings for ad avail blanking.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AudioDescriptions</i> Placeholder documentation for __listOfAudioDescription<br/>Blanking image to be used. Leave empty for solid black. Only bmp and png images are supported.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailBlankingImage</b>: Blanking image to be used. Leave empty for solid black. Only bmp and png images are supported.Ad avail settings.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailSettings</b>: Ad avail settings.Blackout slate image to be used. Leave empty for solid black. Only bmp and png images are supported.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BlackoutSlateImage</b>: Blackout slate image to be used. Leave empty for solid black. Only bmp and png images are supported.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InputEndAction</i> Indicates the action to take when the current input completes (e.g. end-of-file). When switchAndLoopInputs is configured the encoder will restart at the beginning of the first input.  When "none" is configured the encoder will transcode either black, a solid color, or a user specified slate images per the "Input Loss Behavior" configuration until the next input switch occurs (which is controlled through the Channel Schedule API).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InitialAudioGain</i> Value to set the initial audio gain for the Live Event.<br/>Settings for system actions when input is lost.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputLossBehavior</b>: Settings for system actions when input is lost.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SyncThreshold</i> Threshold in frames beyond which output timecode is resynchronized to the input timecode. Discrepancies below this threshold are permitted to avoid unnecessary discontinuities in the output timecode. No timecode sync when this is not specified.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Source</i> Identifies the source for the timecode that will be associated with the events outputs.
-Embedded (embedded): Initialize the output timecode with timecode from the the source.  If no embedded timecode is detected in the source, the system falls back to using "Start at 0" (zerobased).
-System Clock (systemclock): Use the UTC time.
-Start at 0 (zerobased): The time of the first frame of the event will be 00:00:00:00.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Resolution</i> Input resolution, categorized coarsely<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumBitrate</i> Maximum input bitrate, categorized coarsely<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Codec</i> Input codec<br/>
        :return: Channel:  Placeholder documentation for Channel:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> A collection of key-value pairs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> Placeholder documentation for ChannelState<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The Amazon Resource Name (ARN) of the role assumed when running the Channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PipelinesRunningCount</i> The number of currently healthy pipelines.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the channel. (user-mutable)<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogLevel</i> The log level being written to CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputSpecification</i> Placeholder documentation for InputSpecification&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputSpecification</b>: Placeholder documentation for InputSpecification<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputAttachments</i> List of input attachments for channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique id of the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncoderSettings</i> Encoder Settings&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncoderSettings</b>: Encoder Settings<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EgressEndpoints</i> The endpoints where outgoing connections initiate from<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Destinations</i> A list of destinations of the channel. For UDP outputs, there is one
destination per output. For other types (HLS, for example), there is
one destination per packager.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ChannelClass</i> The class for this channel. STANDARD for a channel with two pipelines or SINGLE_PIPELINE for a channel with one pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The unique arn of the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AudioDescriptions</i> Placeholder documentation for __listOfAudioDescription<br/>Settings for ad avail blanking.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailBlanking</b>: Settings for ad avail blanking.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Resolution</i> Input resolution, categorized coarsely<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumBitrate</i> Maximum input bitrate, categorized coarsely<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Codec</i> Input codec<br/>        """
        pass

    def update_channel_class(self, ChannelId: str, ChannelClass: str, Destinations: List[OutputDestination]=None) -> UpdateChannelClassResponse:
        """Changes the class of the channel.

        :param self:
        :param Destinations:  A list of output destinations for this channel.<br/>ChannelId:  Channel Id of the channel whose class should be updated.<br/>ChannelClass:  The channel class that you wish to update this channel to use.<br/>
        :return: Channel:  Placeholder documentation for Channel:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> A collection of key-value pairs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> Placeholder documentation for ChannelState<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The Amazon Resource Name (ARN) of the role assumed when running the Channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PipelinesRunningCount</i> The number of currently healthy pipelines.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the channel. (user-mutable)<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogLevel</i> The log level being written to CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputSpecification</i> Placeholder documentation for InputSpecification&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputSpecification</b>: Placeholder documentation for InputSpecification<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputAttachments</i> List of input attachments for channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique id of the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncoderSettings</i> Encoder Settings&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncoderSettings</b>: Encoder Settings<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EgressEndpoints</i> The endpoints where outgoing connections initiate from<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Destinations</i> A list of destinations of the channel. For UDP outputs, there is one
destination per output. For other types (HLS, for example), there is
one destination per packager.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ChannelClass</i> The class for this channel. STANDARD for a channel with two pipelines or SINGLE_PIPELINE for a channel with one pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The unique arn of the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AudioDescriptions</i> Placeholder documentation for __listOfAudioDescription<br/>Settings for ad avail blanking.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AvailBlanking</b>: Settings for ad avail blanking.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Resolution</i> Input resolution, categorized coarsely<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumBitrate</i> Maximum input bitrate, categorized coarsely<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Codec</i> Input codec<br/>        """
        pass

    def update_input(self, InputId: str, Destinations: List[InputDestinationRequest]=None, InputSecurityGroups: List[__string]=None, MediaConnectFlows: List[MediaConnectFlowRequest]=None, Name: str=None, RoleArn: str=None, Sources: List[InputSourceRequest]=None) -> UpdateInputResponse:
        """Updates an input.

        :param self:
        :param Sources:  The source URLs for a PULL-type input. Every PULL type input needs
exactly two source URLs for redundancy.
Only specify sources for PULL type Inputs. Leave Destinations empty.
<br/>RoleArn:  The Amazon Resource Name (ARN) of the role this input assumes during and after creation.<br/>Name:  Name of the input.<br/>MediaConnectFlows:  A list of the MediaConnect Flow ARNs that you want to use as the source of the input. You can specify as few as one
Flow and presently, as many as two. The only requirement is when you have more than one is that each Flow is in a
separate Availability Zone as this ensures your EML input is redundant to AZ issues.
<br/>InputSecurityGroups:  A list of security groups referenced by IDs to attach to the input.<br/>InputId:  Unique ID of the input.<br/>Destinations:  Destination settings for PUSH type inputs.<br/>
        :return: Input:  Placeholder documentation for Input:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> Placeholder documentation for InputType<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> A collection of key-value pairs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> Placeholder documentation for InputState<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Sources</i> A list of the sources of the input (PULL-type).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroups</i> A list of IDs for all the Input Security Groups attached to the input.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The Amazon Resource Name (ARN) of the role this input assumes during and after creation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The user-assigned name (This is a mutable value).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MediaConnectFlows</i> A list of MediaConnect Flows for this input.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputClass</i> STANDARD - MediaLive expects two sources to be connected to this input. If the channel is also STANDARD, both sources will be ingested. If the channel is SINGLE_PIPELINE, only the first source will be ingested; the second source will always be ignored, even if the first source fails.
SINGLE_PIPELINE - You can connect only one source to this input. If the ChannelClass is also  SINGLE_PIPELINE, this value is valid. If the ChannelClass is STANDARD, this value is not valid because the channel requires two sources in the input.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The generated ID of the input (unique for user account, immutable).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Destinations</i> A list of the destinations of the input (PUSH-type).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttachedChannels</i> A list of channel IDs that that input is attached to (currently an input can only be attached to one channel).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Unique ARN of the input (generated, immutable).<br/>        """
        pass

    def update_input_security_group(self, InputSecurityGroupId: str, Tags: Tags=None, WhitelistRules: List[InputWhitelistRuleCidr]=None) -> UpdateInputSecurityGroupResponse:
        """Update an Input Security Group's Whilelists.

        :param self:
        :param WhitelistRules:  List of IPv4 CIDR addresses to whitelist<br/>Tags:  A collection of key-value pairs.<br/>InputSecurityGroupId:  The id of the Input Security Group to update.<br/>
        :return: SecurityGroup:  An Input Security Group:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WhitelistRules</i> Whitelist rules and their sync status<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> A collection of key-value pairs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The current state of the Input Security Group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Inputs</i> The list of inputs currently using this Input Security Group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The Id of the Input Security Group<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> Unique ARN of Input Security Group<br/>        """
        pass

    def update_reservation(self, ReservationId: str, Name: str=None) -> UpdateReservationResponse:
        """Update reservation.

        :param self:
        :param ReservationId:  Unique reservation ID, e.g. '1234567'<br/>Name:  Name of the reservation<br/>
        :return: Reservation:  Reserved resources available to use:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UsagePrice</i> Recurring usage charge for each reserved resource, e.g. '157.0'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> A collection of key-value pairs<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> Current state of reservation, e.g. 'ACTIVE'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Start</i> Reservation UTC start date and time in ISO-8601 format, e.g. '2018-03-01T00:00:00'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceSpecification</i> Resource configuration details&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResourceSpecification</b>: Resource configuration details<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservationId</i> Unique reservation ID, e.g. '1234567'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Region</i> AWS region, e.g. 'us-west-2'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OfferingType</i> Offering type, e.g. 'NO_UPFRONT'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OfferingId</i> Unique offering ID, e.g. '87654321'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OfferingDescription</i> Offering description, e.g. 'HD AVC output at 10-20 Mbps, 30 fps, and standard VQ in US West (Oregon)'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> User specified reservation name<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FixedPrice</i> One-time charge for each reserved resource, e.g. '0.0' for a NO_UPFRONT offering<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>End</i> Reservation UTC end date and time in ISO-8601 format, e.g. '2019-03-01T00:00:00'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DurationUnits</i> Units for duration, e.g. 'MONTHS'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Duration</i> Lease duration, e.g. '12'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrencyCode</i> Currency code for usagePrice and fixedPrice in ISO-4217 format, e.g. 'USD'<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Count</i> Number of reserved resources<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> Unique reservation ARN, e.g. 'arn:aws:medialive:us-west-2:123456789012:reservation:1234567'<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VideoQuality</i> Video quality, e.g. 'STANDARD' (Outputs only)<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SpecialFeature</i> Special feature, e.g. 'AUDIO_NORMALIZATION' (Channels only)<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceType</i> Resource type, 'INPUT', 'OUTPUT', or 'CHANNEL'<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Resolution</i> Resolution, e.g. 'HD'<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumFramerate</i> Maximum framerate, e.g. 'MAX_30_FPS' (Outputs only)<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumBitrate</i> Maximum bitrate, e.g. 'MAX_20_MBPS'<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Codec</i> Codec, e.g. 'AVC'<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ChannelClass</i> Channel class, e.g. 'STANDARD'<br/>        """
        pass



class MediaPackage(BaseClient):
    """AWS Elemental MediaPackage"""


    class ListChannelsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_channels` for valid parameters"""
            pass
    class ListOriginEndpointsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_origin_endpoints` for valid parameters"""
            pass


    class CreateChannelRequest(Mapping):
        pass
    
    class CreateChannelResponse(Mapping):
        pass
    
    class Tags(object):
        pass
    
    class CreateOriginEndpointRequest(Mapping):
        pass
    
    class CreateOriginEndpointResponse(Mapping):
        pass
    
    class CmafPackageCreateOrUpdateParameters(Mapping):
        pass
    
    class DashPackage(Mapping):
        pass
    
    class HlsPackage(Mapping):
        pass
    
    class MssPackage(Mapping):
        pass
    
    class __string(object):
        pass
    
    class DeleteChannelRequest(Mapping):
        pass
    
    class DeleteChannelResponse(Mapping):
        pass
    
    class DeleteOriginEndpointRequest(Mapping):
        pass
    
    class DeleteOriginEndpointResponse(Mapping):
        pass
    
    class DescribeChannelRequest(Mapping):
        pass
    
    class DescribeChannelResponse(Mapping):
        pass
    
    class DescribeOriginEndpointRequest(Mapping):
        pass
    
    class DescribeOriginEndpointResponse(Mapping):
        pass
    
    class ListChannelsRequest(Mapping):
        pass
    
    class ListChannelsResponse(Mapping):
        pass
    
    class ListOriginEndpointsRequest(Mapping):
        pass
    
    class ListOriginEndpointsResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class RotateChannelCredentialsRequest(Mapping):
        pass
    
    class RotateChannelCredentialsResponse(Mapping):
        pass
    
    class RotateIngestEndpointCredentialsRequest(Mapping):
        pass
    
    class RotateIngestEndpointCredentialsResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class __mapOf__string(object):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UpdateChannelRequest(Mapping):
        pass
    
    class UpdateChannelResponse(Mapping):
        pass
    
    class UpdateOriginEndpointRequest(Mapping):
        pass
    
    class UpdateOriginEndpointResponse(Mapping):
        pass
    
    def create_channel(self, Id: str, Description: str=None, Tags: Tags=None) -> CreateChannelResponse:
        """Creates a new Channel.

        :param self:
        :param Tags:  A collection of tags associated with a resource<br/>Id:  The ID of the Channel. The ID must be unique within the region and it
cannot be changed after a Channel is created.
<br/>Description:  A short text description of the Channel.<br/>
        :return: Tags:  A collection of tags associated with a resource<br/>Id:  The ID of the Channel.<br/>HlsIngest:  An HTTP Live Streaming (HLS) ingest resource configuration.:<br/>Description:  A short text description of the Channel.<br/>Arn:  The Amazon Resource Name (ARN) assigned to the Channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IngestEndpoints</i> A list of endpoints to which the source stream should be sent.<br/>        """
        pass

    def create_origin_endpoint(self, Id: str, ChannelId: str, CmafPackage: CmafPackageCreateOrUpdateParameters=None, DashPackage: DashPackage=None, Description: str=None, HlsPackage: HlsPackage=None, ManifestName: str=None, MssPackage: MssPackage=None, StartoverWindowSeconds: int=None, Tags: Tags=None, TimeDelaySeconds: int=None, Whitelist: List[__string]=None) -> CreateOriginEndpointResponse:
        """Creates a new OriginEndpoint record.

        :param self:
        :param Whitelist:  A list of source IP CIDR blocks that will be allowed to access the OriginEndpoint.<br/>TimeDelaySeconds:  Amount of delay (seconds) to enforce on the playback of live content.
If not specified, there will be no time delay in effect for the OriginEndpoint.
<br/>Tags:  A collection of tags associated with a resource<br/>StartoverWindowSeconds:  Maximum duration (seconds) of content to retain for startover playback.
If not specified, startover playback will be disabled for the OriginEndpoint.
<br/>MssPackage:  A Microsoft Smooth Streaming (MSS) packaging configuration.:<br/>ManifestName:  A short string that will be used as the filename of the OriginEndpoint URL (defaults to "index").<br/>Id:  The ID of the OriginEndpoint.  The ID must be unique within the region
and it cannot be changed after the OriginEndpoint is created.
<br/>HlsPackage:  An HTTP Live Streaming (HLS) packaging configuration.:<br/>Description:  A short text description of the OriginEndpoint.<br/>DashPackage:  A Dynamic Adaptive Streaming over HTTP (DASH) packaging configuration.:<br/>CmafPackage:  A Common Media Application Format (CMAF) packaging configuration.:<br/>ChannelId:  The ID of the Channel that the OriginEndpoint will be associated with.
This cannot be changed after the OriginEndpoint is created.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamSelection</i> A StreamSelection configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamSelection</b>: A StreamSelection configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentPrefix</i> An optional custom string that is prepended to the name of each segment. If not specified, it defaults to the ChannelId.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> Duration (in seconds) of each segment. Actual segments will be
rounded to the nearest multiple of the source segment duration.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HlsManifests</i> A list of HLS manifest configurations<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> A Common Media Application Format (CMAF) encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: A Common Media Application Format (CMAF) encryption configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyRotationIntervalSeconds</i> Time (in seconds) between each encryption key rotation.<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamOrder</i> A directive that determines the order of streams in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinVideoBitsPerSecond</i> The minimum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxVideoBitsPerSecond</i> The maximum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SuggestedPresentationDelaySeconds</i> Duration (in seconds) to delay live content before presentation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamSelection</i> A StreamSelection configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamSelection</b>: A StreamSelection configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentTemplateFormat</i> Determines the type of SegmentTemplate included in the Media Presentation Description (MPD).  When set to NUMBER_WITH_TIMELINE, a full timeline is presented in each SegmentTemplate, with $Number$ media URLs.  When set to TIME_WITH_TIMELINE, a full timeline is presented in each SegmentTemplate, with $Time$ media URLs. When set to NUMBER_WITH_DURATION, only a duration is included in each SegmentTemplate, with $Number$ media URLs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> Duration (in seconds) of each segment. Actual segments will be
rounded to the nearest multiple of the source segment duration.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Profile</i> The Dynamic Adaptive Streaming over HTTP (DASH) profile type.  When set to "HBBTV_1_5", HbbTV 1.5 compliant output is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PeriodTriggers</i> A list of triggers that controls when the outgoing Dynamic Adaptive Streaming over HTTP (DASH)
Media Presentation Description (MPD) will be partitioned into multiple periods. If empty, the content will not
be partitioned into more than one period. If the list contains "ADS", new periods will be created where
the Channel source contains SCTE-35 ad markers.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MinUpdatePeriodSeconds</i> Minimum duration (in seconds) between potential changes to the Dynamic Adaptive Streaming over HTTP (DASH) Media Presentation Description (MPD).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MinBufferTimeSeconds</i> Minimum duration (in seconds) that a player will buffer media before starting the presentation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManifestWindowSeconds</i> Time window (in seconds) contained in each manifest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManifestLayout</i> Determines the position of some tags in the Media Presentation Description (MPD).  When set to FULL, elements like SegmentTemplate and ContentProtection are included in each Representation.  When set to COMPACT, duplicate elements are combined and presented at the AdaptationSet level.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> A Dynamic Adaptive Streaming over HTTP (DASH) encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: A Dynamic Adaptive Streaming over HTTP (DASH) encryption configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdsOnDeliveryRestrictions</i> This setting allows the delivery restriction flags on SCTE-35 segmentation descriptors to
determine whether a message signals an ad.  Choosing "NONE" means no SCTE-35 messages become
ads.  Choosing "RESTRICTED" means SCTE-35 messages of the types specified in AdTriggers that
contain delivery restrictions will be treated as ads.  Choosing "UNRESTRICTED" means SCTE-35
messages of the types specified in AdTriggers that do not contain delivery restrictions will
be treated as ads.  Choosing "BOTH" means all SCTE-35 messages of the types specified in
AdTriggers will be treated as ads.  Note that Splice Insert messages do not have these flags
and are always treated as ads if specified in AdTriggers.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdTriggers</i> A list of SCTE-35 message types that are treated as ad markers in the output.  If empty, no
ad markers are output.  Specify multiple items to create ad markers for all of the included
message types.
<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyRotationIntervalSeconds</i> Time (in seconds) between each encryption key rotation.<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamOrder</i> A directive that determines the order of streams in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinVideoBitsPerSecond</i> The minimum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxVideoBitsPerSecond</i> The maximum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UseAudioRenditionGroup</i> When enabled, audio streams will be placed in rendition groups in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamSelection</i> A StreamSelection configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamSelection</b>: A StreamSelection configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> Duration (in seconds) of each fragment. Actual fragments will be
rounded to the nearest multiple of the source fragment duration.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProgramDateTimeIntervalSeconds</i> The interval (in seconds) between each EXT-X-PROGRAM-DATE-TIME tag
inserted into manifests. Additionally, when an interval is specified
ID3Timed Metadata messages will be generated every 5 seconds using the
ingest time of the content.
If the interval is not specified, or set to 0, then
no EXT-X-PROGRAM-DATE-TIME tags will be inserted into manifests and no
ID3Timed Metadata messages will be generated. Note that irrespective
of this parameter, if any ID3 Timed Metadata is found in HTTP Live Streaming (HLS) input,
it will be passed through to HLS output.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlaylistWindowSeconds</i> Time window (in seconds) contained in each parent manifest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlaylistType</i> The HTTP Live Streaming (HLS) playlist type.
When either "EVENT" or "VOD" is specified, a corresponding EXT-X-PLAYLIST-TYPE
entry will be included in the media playlist.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeIframeOnlyStream</i> When enabled, an I-Frame only stream will be included in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> An HTTP Live Streaming (HLS) encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: An HTTP Live Streaming (HLS) encryption configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdsOnDeliveryRestrictions</i> This setting allows the delivery restriction flags on SCTE-35 segmentation descriptors to
determine whether a message signals an ad.  Choosing "NONE" means no SCTE-35 messages become
ads.  Choosing "RESTRICTED" means SCTE-35 messages of the types specified in AdTriggers that
contain delivery restrictions will be treated as ads.  Choosing "UNRESTRICTED" means SCTE-35
messages of the types specified in AdTriggers that do not contain delivery restrictions will
be treated as ads.  Choosing "BOTH" means all SCTE-35 messages of the types specified in
AdTriggers will be treated as ads.  Note that Splice Insert messages do not have these flags
and are always treated as ads if specified in AdTriggers.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdTriggers</i> A list of SCTE-35 message types that are treated as ad markers in the output.  If empty, no
ad markers are output.  Specify multiple items to create ad markers for all of the included
message types.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdMarkers</i> This setting controls how ad markers are included in the packaged OriginEndpoint.
"NONE" will omit all SCTE-35 ad markers from the output.
"PASSTHROUGH" causes the manifest to contain a copy of the SCTE-35 ad
markers (comments) taken directly from the input HTTP Live Streaming (HLS) manifest.
"SCTE35_ENHANCED" generates ad markers and blackout tags based on SCTE-35
messages in the input source.
<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RepeatExtXKey</i> When enabled, the EXT-X-KEY tag will be repeated in output manifests.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyRotationIntervalSeconds</i> Interval (in seconds) between each encryption key rotation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionMethod</i> The encryption method to use.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ConstantInitializationVector</i> A constant initialization vector for encryption (optional).
When not specified the initialization vector will be periodically rotated.
<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamOrder</i> A directive that determines the order of streams in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinVideoBitsPerSecond</i> The minimum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxVideoBitsPerSecond</i> The maximum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamSelection</i> A StreamSelection configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamSelection</b>: A StreamSelection configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> The duration (in seconds) of each segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManifestWindowSeconds</i> The time window (in seconds) contained in each manifest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> A Microsoft Smooth Streaming (MSS) encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: A Microsoft Smooth Streaming (MSS) encryption configuration.<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamOrder</i> A directive that determines the order of streams in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinVideoBitsPerSecond</i> The minimum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxVideoBitsPerSecond</i> The maximum video bitrate (bps) to include in output.<br/>
        :return: Whitelist:  A list of source IP CIDR blocks that will be allowed to access the OriginEndpoint.<br/>Url:  The URL of the packaged OriginEndpoint for consumption.<br/>TimeDelaySeconds:  Amount of delay (seconds) to enforce on the playback of live content.
If not specified, there will be no time delay in effect for the OriginEndpoint.
<br/>Tags:  A collection of tags associated with a resource<br/>StartoverWindowSeconds:  Maximum duration (seconds) of content to retain for startover playback.
If not specified, startover playback will be disabled for the OriginEndpoint.
<br/>MssPackage:  A Microsoft Smooth Streaming (MSS) packaging configuration.:<br/>ManifestName:  A short string appended to the end of the OriginEndpoint URL.<br/>Id:  The ID of the OriginEndpoint.<br/>HlsPackage:  An HTTP Live Streaming (HLS) packaging configuration.:<br/>Description:  A short text description of the OriginEndpoint.<br/>DashPackage:  A Dynamic Adaptive Streaming over HTTP (DASH) packaging configuration.:<br/>CmafPackage:  A Common Media Application Format (CMAF) packaging configuration.:<br/>ChannelId:  The ID of the Channel the OriginEndpoint is associated with.<br/>Arn:  The Amazon Resource Name (ARN) assigned to the OriginEndpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamSelection</i> A StreamSelection configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamSelection</b>: A StreamSelection configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentPrefix</i> An optional custom string that is prepended to the name of each segment. If not specified, it defaults to the ChannelId.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> Duration (in seconds) of each segment. Actual segments will be
rounded to the nearest multiple of the source segment duration.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HlsManifests</i> A list of HLS manifest configurations<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> A Common Media Application Format (CMAF) encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: A Common Media Application Format (CMAF) encryption configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyRotationIntervalSeconds</i> Time (in seconds) between each encryption key rotation.<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamOrder</i> A directive that determines the order of streams in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinVideoBitsPerSecond</i> The minimum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxVideoBitsPerSecond</i> The maximum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SuggestedPresentationDelaySeconds</i> Duration (in seconds) to delay live content before presentation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamSelection</i> A StreamSelection configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamSelection</b>: A StreamSelection configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentTemplateFormat</i> Determines the type of SegmentTemplate included in the Media Presentation Description (MPD).  When set to NUMBER_WITH_TIMELINE, a full timeline is presented in each SegmentTemplate, with $Number$ media URLs.  When set to TIME_WITH_TIMELINE, a full timeline is presented in each SegmentTemplate, with $Time$ media URLs. When set to NUMBER_WITH_DURATION, only a duration is included in each SegmentTemplate, with $Number$ media URLs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> Duration (in seconds) of each segment. Actual segments will be
rounded to the nearest multiple of the source segment duration.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Profile</i> The Dynamic Adaptive Streaming over HTTP (DASH) profile type.  When set to "HBBTV_1_5", HbbTV 1.5 compliant output is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PeriodTriggers</i> A list of triggers that controls when the outgoing Dynamic Adaptive Streaming over HTTP (DASH)
Media Presentation Description (MPD) will be partitioned into multiple periods. If empty, the content will not
be partitioned into more than one period. If the list contains "ADS", new periods will be created where
the Channel source contains SCTE-35 ad markers.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MinUpdatePeriodSeconds</i> Minimum duration (in seconds) between potential changes to the Dynamic Adaptive Streaming over HTTP (DASH) Media Presentation Description (MPD).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MinBufferTimeSeconds</i> Minimum duration (in seconds) that a player will buffer media before starting the presentation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManifestWindowSeconds</i> Time window (in seconds) contained in each manifest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManifestLayout</i> Determines the position of some tags in the Media Presentation Description (MPD).  When set to FULL, elements like SegmentTemplate and ContentProtection are included in each Representation.  When set to COMPACT, duplicate elements are combined and presented at the AdaptationSet level.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> A Dynamic Adaptive Streaming over HTTP (DASH) encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: A Dynamic Adaptive Streaming over HTTP (DASH) encryption configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdsOnDeliveryRestrictions</i> This setting allows the delivery restriction flags on SCTE-35 segmentation descriptors to
determine whether a message signals an ad.  Choosing "NONE" means no SCTE-35 messages become
ads.  Choosing "RESTRICTED" means SCTE-35 messages of the types specified in AdTriggers that
contain delivery restrictions will be treated as ads.  Choosing "UNRESTRICTED" means SCTE-35
messages of the types specified in AdTriggers that do not contain delivery restrictions will
be treated as ads.  Choosing "BOTH" means all SCTE-35 messages of the types specified in
AdTriggers will be treated as ads.  Note that Splice Insert messages do not have these flags
and are always treated as ads if specified in AdTriggers.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdTriggers</i> A list of SCTE-35 message types that are treated as ad markers in the output.  If empty, no
ad markers are output.  Specify multiple items to create ad markers for all of the included
message types.
<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyRotationIntervalSeconds</i> Time (in seconds) between each encryption key rotation.<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamOrder</i> A directive that determines the order of streams in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinVideoBitsPerSecond</i> The minimum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxVideoBitsPerSecond</i> The maximum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UseAudioRenditionGroup</i> When enabled, audio streams will be placed in rendition groups in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamSelection</i> A StreamSelection configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamSelection</b>: A StreamSelection configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> Duration (in seconds) of each fragment. Actual fragments will be
rounded to the nearest multiple of the source fragment duration.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProgramDateTimeIntervalSeconds</i> The interval (in seconds) between each EXT-X-PROGRAM-DATE-TIME tag
inserted into manifests. Additionally, when an interval is specified
ID3Timed Metadata messages will be generated every 5 seconds using the
ingest time of the content.
If the interval is not specified, or set to 0, then
no EXT-X-PROGRAM-DATE-TIME tags will be inserted into manifests and no
ID3Timed Metadata messages will be generated. Note that irrespective
of this parameter, if any ID3 Timed Metadata is found in HTTP Live Streaming (HLS) input,
it will be passed through to HLS output.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlaylistWindowSeconds</i> Time window (in seconds) contained in each parent manifest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlaylistType</i> The HTTP Live Streaming (HLS) playlist type.
When either "EVENT" or "VOD" is specified, a corresponding EXT-X-PLAYLIST-TYPE
entry will be included in the media playlist.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeIframeOnlyStream</i> When enabled, an I-Frame only stream will be included in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> An HTTP Live Streaming (HLS) encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: An HTTP Live Streaming (HLS) encryption configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdsOnDeliveryRestrictions</i> This setting allows the delivery restriction flags on SCTE-35 segmentation descriptors to
determine whether a message signals an ad.  Choosing "NONE" means no SCTE-35 messages become
ads.  Choosing "RESTRICTED" means SCTE-35 messages of the types specified in AdTriggers that
contain delivery restrictions will be treated as ads.  Choosing "UNRESTRICTED" means SCTE-35
messages of the types specified in AdTriggers that do not contain delivery restrictions will
be treated as ads.  Choosing "BOTH" means all SCTE-35 messages of the types specified in
AdTriggers will be treated as ads.  Note that Splice Insert messages do not have these flags
and are always treated as ads if specified in AdTriggers.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdTriggers</i> A list of SCTE-35 message types that are treated as ad markers in the output.  If empty, no
ad markers are output.  Specify multiple items to create ad markers for all of the included
message types.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdMarkers</i> This setting controls how ad markers are included in the packaged OriginEndpoint.
"NONE" will omit all SCTE-35 ad markers from the output.
"PASSTHROUGH" causes the manifest to contain a copy of the SCTE-35 ad
markers (comments) taken directly from the input HTTP Live Streaming (HLS) manifest.
"SCTE35_ENHANCED" generates ad markers and blackout tags based on SCTE-35
messages in the input source.
<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RepeatExtXKey</i> When enabled, the EXT-X-KEY tag will be repeated in output manifests.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyRotationIntervalSeconds</i> Interval (in seconds) between each encryption key rotation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionMethod</i> The encryption method to use.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ConstantInitializationVector</i> A constant initialization vector for encryption (optional).
When not specified the initialization vector will be periodically rotated.
<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamOrder</i> A directive that determines the order of streams in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinVideoBitsPerSecond</i> The minimum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxVideoBitsPerSecond</i> The maximum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamSelection</i> A StreamSelection configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamSelection</b>: A StreamSelection configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> The duration (in seconds) of each segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManifestWindowSeconds</i> The time window (in seconds) contained in each manifest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> A Microsoft Smooth Streaming (MSS) encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: A Microsoft Smooth Streaming (MSS) encryption configuration.<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamOrder</i> A directive that determines the order of streams in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinVideoBitsPerSecond</i> The minimum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxVideoBitsPerSecond</i> The maximum video bitrate (bps) to include in output.<br/>        """
        pass

    def delete_channel(self, Id: str) -> DeleteChannelResponse:
        """Deletes an existing Channel.

        :param self:
        :param Id:  The ID of the Channel to delete.<br/>
        :return:         """
        pass

    def delete_origin_endpoint(self, Id: str) -> DeleteOriginEndpointResponse:
        """Deletes an existing OriginEndpoint.

        :param self:
        :param Id:  The ID of the OriginEndpoint to delete.<br/>
        :return:         """
        pass

    def describe_channel(self, Id: str) -> DescribeChannelResponse:
        """Gets details about a Channel.

        :param self:
        :param Id:  The ID of a Channel.<br/>
        :return: Tags:  A collection of tags associated with a resource<br/>Id:  The ID of the Channel.<br/>HlsIngest:  An HTTP Live Streaming (HLS) ingest resource configuration.:<br/>Description:  A short text description of the Channel.<br/>Arn:  The Amazon Resource Name (ARN) assigned to the Channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IngestEndpoints</i> A list of endpoints to which the source stream should be sent.<br/>        """
        pass

    def describe_origin_endpoint(self, Id: str) -> DescribeOriginEndpointResponse:
        """Gets details about an existing OriginEndpoint.

        :param self:
        :param Id:  The ID of the OriginEndpoint.<br/>
        :return: Whitelist:  A list of source IP CIDR blocks that will be allowed to access the OriginEndpoint.<br/>Url:  The URL of the packaged OriginEndpoint for consumption.<br/>TimeDelaySeconds:  Amount of delay (seconds) to enforce on the playback of live content.
If not specified, there will be no time delay in effect for the OriginEndpoint.
<br/>Tags:  A collection of tags associated with a resource<br/>StartoverWindowSeconds:  Maximum duration (seconds) of content to retain for startover playback.
If not specified, startover playback will be disabled for the OriginEndpoint.
<br/>MssPackage:  A Microsoft Smooth Streaming (MSS) packaging configuration.:<br/>ManifestName:  A short string appended to the end of the OriginEndpoint URL.<br/>Id:  The ID of the OriginEndpoint.<br/>HlsPackage:  An HTTP Live Streaming (HLS) packaging configuration.:<br/>Description:  A short text description of the OriginEndpoint.<br/>DashPackage:  A Dynamic Adaptive Streaming over HTTP (DASH) packaging configuration.:<br/>CmafPackage:  A Common Media Application Format (CMAF) packaging configuration.:<br/>ChannelId:  The ID of the Channel the OriginEndpoint is associated with.<br/>Arn:  The Amazon Resource Name (ARN) assigned to the OriginEndpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamSelection</i> A StreamSelection configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamSelection</b>: A StreamSelection configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentPrefix</i> An optional custom string that is prepended to the name of each segment. If not specified, it defaults to the ChannelId.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> Duration (in seconds) of each segment. Actual segments will be
rounded to the nearest multiple of the source segment duration.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HlsManifests</i> A list of HLS manifest configurations<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> A Common Media Application Format (CMAF) encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: A Common Media Application Format (CMAF) encryption configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyRotationIntervalSeconds</i> Time (in seconds) between each encryption key rotation.<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamOrder</i> A directive that determines the order of streams in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinVideoBitsPerSecond</i> The minimum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxVideoBitsPerSecond</i> The maximum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SuggestedPresentationDelaySeconds</i> Duration (in seconds) to delay live content before presentation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamSelection</i> A StreamSelection configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamSelection</b>: A StreamSelection configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentTemplateFormat</i> Determines the type of SegmentTemplate included in the Media Presentation Description (MPD).  When set to NUMBER_WITH_TIMELINE, a full timeline is presented in each SegmentTemplate, with $Number$ media URLs.  When set to TIME_WITH_TIMELINE, a full timeline is presented in each SegmentTemplate, with $Time$ media URLs. When set to NUMBER_WITH_DURATION, only a duration is included in each SegmentTemplate, with $Number$ media URLs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> Duration (in seconds) of each segment. Actual segments will be
rounded to the nearest multiple of the source segment duration.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Profile</i> The Dynamic Adaptive Streaming over HTTP (DASH) profile type.  When set to "HBBTV_1_5", HbbTV 1.5 compliant output is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PeriodTriggers</i> A list of triggers that controls when the outgoing Dynamic Adaptive Streaming over HTTP (DASH)
Media Presentation Description (MPD) will be partitioned into multiple periods. If empty, the content will not
be partitioned into more than one period. If the list contains "ADS", new periods will be created where
the Channel source contains SCTE-35 ad markers.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MinUpdatePeriodSeconds</i> Minimum duration (in seconds) between potential changes to the Dynamic Adaptive Streaming over HTTP (DASH) Media Presentation Description (MPD).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MinBufferTimeSeconds</i> Minimum duration (in seconds) that a player will buffer media before starting the presentation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManifestWindowSeconds</i> Time window (in seconds) contained in each manifest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManifestLayout</i> Determines the position of some tags in the Media Presentation Description (MPD).  When set to FULL, elements like SegmentTemplate and ContentProtection are included in each Representation.  When set to COMPACT, duplicate elements are combined and presented at the AdaptationSet level.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> A Dynamic Adaptive Streaming over HTTP (DASH) encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: A Dynamic Adaptive Streaming over HTTP (DASH) encryption configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdsOnDeliveryRestrictions</i> This setting allows the delivery restriction flags on SCTE-35 segmentation descriptors to
determine whether a message signals an ad.  Choosing "NONE" means no SCTE-35 messages become
ads.  Choosing "RESTRICTED" means SCTE-35 messages of the types specified in AdTriggers that
contain delivery restrictions will be treated as ads.  Choosing "UNRESTRICTED" means SCTE-35
messages of the types specified in AdTriggers that do not contain delivery restrictions will
be treated as ads.  Choosing "BOTH" means all SCTE-35 messages of the types specified in
AdTriggers will be treated as ads.  Note that Splice Insert messages do not have these flags
and are always treated as ads if specified in AdTriggers.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdTriggers</i> A list of SCTE-35 message types that are treated as ad markers in the output.  If empty, no
ad markers are output.  Specify multiple items to create ad markers for all of the included
message types.
<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyRotationIntervalSeconds</i> Time (in seconds) between each encryption key rotation.<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamOrder</i> A directive that determines the order of streams in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinVideoBitsPerSecond</i> The minimum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxVideoBitsPerSecond</i> The maximum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UseAudioRenditionGroup</i> When enabled, audio streams will be placed in rendition groups in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamSelection</i> A StreamSelection configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamSelection</b>: A StreamSelection configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> Duration (in seconds) of each fragment. Actual fragments will be
rounded to the nearest multiple of the source fragment duration.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProgramDateTimeIntervalSeconds</i> The interval (in seconds) between each EXT-X-PROGRAM-DATE-TIME tag
inserted into manifests. Additionally, when an interval is specified
ID3Timed Metadata messages will be generated every 5 seconds using the
ingest time of the content.
If the interval is not specified, or set to 0, then
no EXT-X-PROGRAM-DATE-TIME tags will be inserted into manifests and no
ID3Timed Metadata messages will be generated. Note that irrespective
of this parameter, if any ID3 Timed Metadata is found in HTTP Live Streaming (HLS) input,
it will be passed through to HLS output.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlaylistWindowSeconds</i> Time window (in seconds) contained in each parent manifest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlaylistType</i> The HTTP Live Streaming (HLS) playlist type.
When either "EVENT" or "VOD" is specified, a corresponding EXT-X-PLAYLIST-TYPE
entry will be included in the media playlist.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeIframeOnlyStream</i> When enabled, an I-Frame only stream will be included in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> An HTTP Live Streaming (HLS) encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: An HTTP Live Streaming (HLS) encryption configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdsOnDeliveryRestrictions</i> This setting allows the delivery restriction flags on SCTE-35 segmentation descriptors to
determine whether a message signals an ad.  Choosing "NONE" means no SCTE-35 messages become
ads.  Choosing "RESTRICTED" means SCTE-35 messages of the types specified in AdTriggers that
contain delivery restrictions will be treated as ads.  Choosing "UNRESTRICTED" means SCTE-35
messages of the types specified in AdTriggers that do not contain delivery restrictions will
be treated as ads.  Choosing "BOTH" means all SCTE-35 messages of the types specified in
AdTriggers will be treated as ads.  Note that Splice Insert messages do not have these flags
and are always treated as ads if specified in AdTriggers.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdTriggers</i> A list of SCTE-35 message types that are treated as ad markers in the output.  If empty, no
ad markers are output.  Specify multiple items to create ad markers for all of the included
message types.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdMarkers</i> This setting controls how ad markers are included in the packaged OriginEndpoint.
"NONE" will omit all SCTE-35 ad markers from the output.
"PASSTHROUGH" causes the manifest to contain a copy of the SCTE-35 ad
markers (comments) taken directly from the input HTTP Live Streaming (HLS) manifest.
"SCTE35_ENHANCED" generates ad markers and blackout tags based on SCTE-35
messages in the input source.
<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RepeatExtXKey</i> When enabled, the EXT-X-KEY tag will be repeated in output manifests.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyRotationIntervalSeconds</i> Interval (in seconds) between each encryption key rotation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionMethod</i> The encryption method to use.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ConstantInitializationVector</i> A constant initialization vector for encryption (optional).
When not specified the initialization vector will be periodically rotated.
<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamOrder</i> A directive that determines the order of streams in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinVideoBitsPerSecond</i> The minimum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxVideoBitsPerSecond</i> The maximum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamSelection</i> A StreamSelection configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamSelection</b>: A StreamSelection configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> The duration (in seconds) of each segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManifestWindowSeconds</i> The time window (in seconds) contained in each manifest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> A Microsoft Smooth Streaming (MSS) encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: A Microsoft Smooth Streaming (MSS) encryption configuration.<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamOrder</i> A directive that determines the order of streams in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinVideoBitsPerSecond</i> The minimum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxVideoBitsPerSecond</i> The maximum video bitrate (bps) to include in output.<br/>        """
        pass

    def list_channels(self, MaxResults: int=None, NextToken: str=None) -> ListChannelsResponse:
        """Returns a collection of Channels.

        :param self:
        :param NextToken:  A token used to resume pagination from the end of a previous request.<br/>MaxResults:  Upper bound on number of records to return.<br/>
        :return: NextToken:  A token that can be used to resume pagination from the end of the collection.<br/>Channels:  A list of Channel records.<br/>        """
        pass

    def list_origin_endpoints(self, ChannelId: str=None, MaxResults: int=None, NextToken: str=None) -> ListOriginEndpointsResponse:
        """Returns a collection of OriginEndpoint records.

        :param self:
        :param NextToken:  A token used to resume pagination from the end of a previous request.<br/>MaxResults:  The upper bound on the number of records to return.<br/>ChannelId:  When specified, the request will return only OriginEndpoints associated with the given Channel ID.<br/>
        :return: OriginEndpoints:  A list of OriginEndpoint records.<br/>NextToken:  A token that can be used to resume pagination from the end of the collection.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceArn: str) -> ListTagsForResourceResponse:
        """

        :param self:
        :param ResourceArn:  <br/>
        :return: Tags:  <br/>        """
        pass

    def rotate_channel_credentials(self, Id: str) -> RotateChannelCredentialsResponse:
        """Changes the Channel's first IngestEndpoint's username and password. WARNING - This API is deprecated. Please use RotateIngestEndpointCredentials instead

        :param self:
        :param Id:  The ID of the channel to update.<br/>
        :return: Tags:  A collection of tags associated with a resource<br/>Id:  The ID of the Channel.<br/>HlsIngest:  An HTTP Live Streaming (HLS) ingest resource configuration.:<br/>Description:  A short text description of the Channel.<br/>Arn:  The Amazon Resource Name (ARN) assigned to the Channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IngestEndpoints</i> A list of endpoints to which the source stream should be sent.<br/>        """
        pass

    def rotate_ingest_endpoint_credentials(self, IngestEndpointId: str, Id: str) -> RotateIngestEndpointCredentialsResponse:
        """Rotate the IngestEndpoint's username and password, as specified by the IngestEndpoint's id.

        :param self:
        :param IngestEndpointId:  The id of the IngestEndpoint whose credentials should be rotated<br/>Id:  The ID of the channel the IngestEndpoint is on.<br/>
        :return: Tags:  A collection of tags associated with a resource<br/>Id:  The ID of the Channel.<br/>HlsIngest:  An HTTP Live Streaming (HLS) ingest resource configuration.:<br/>Description:  A short text description of the Channel.<br/>Arn:  The Amazon Resource Name (ARN) assigned to the Channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IngestEndpoints</i> A list of endpoints to which the source stream should be sent.<br/>        """
        pass

    def tag_resource(self, Tags: __mapOf__string, ResourceArn: str):
        """

        :param self:
        :param Tags:  <br/>ResourceArn:  <br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[__string], ResourceArn: str):
        """

        :param self:
        :param TagKeys:  The key(s) of tag to be deleted<br/>ResourceArn:  <br/>
        :return:         """
        pass

    def update_channel(self, Id: str, Description: str=None) -> UpdateChannelResponse:
        """Updates an existing Channel.

        :param self:
        :param Id:  The ID of the Channel to update.<br/>Description:  A short text description of the Channel.<br/>
        :return: Tags:  A collection of tags associated with a resource<br/>Id:  The ID of the Channel.<br/>HlsIngest:  An HTTP Live Streaming (HLS) ingest resource configuration.:<br/>Description:  A short text description of the Channel.<br/>Arn:  The Amazon Resource Name (ARN) assigned to the Channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IngestEndpoints</i> A list of endpoints to which the source stream should be sent.<br/>        """
        pass

    def update_origin_endpoint(self, Id: str, CmafPackage: CmafPackageCreateOrUpdateParameters=None, DashPackage: DashPackage=None, Description: str=None, HlsPackage: HlsPackage=None, ManifestName: str=None, MssPackage: MssPackage=None, StartoverWindowSeconds: int=None, TimeDelaySeconds: int=None, Whitelist: List[__string]=None) -> UpdateOriginEndpointResponse:
        """Updates an existing OriginEndpoint.

        :param self:
        :param Whitelist:  A list of source IP CIDR blocks that will be allowed to access the OriginEndpoint.<br/>TimeDelaySeconds:  Amount of delay (in seconds) to enforce on the playback of live content.
If not specified, there will be no time delay in effect for the OriginEndpoint.
<br/>StartoverWindowSeconds:  Maximum duration (in seconds) of content to retain for startover playback.
If not specified, startover playback will be disabled for the OriginEndpoint.
<br/>MssPackage:  A Microsoft Smooth Streaming (MSS) packaging configuration.:<br/>ManifestName:  A short string that will be appended to the end of the Endpoint URL.<br/>Id:  The ID of the OriginEndpoint to update.<br/>HlsPackage:  An HTTP Live Streaming (HLS) packaging configuration.:<br/>Description:  A short text description of the OriginEndpoint.<br/>DashPackage:  A Dynamic Adaptive Streaming over HTTP (DASH) packaging configuration.:<br/>CmafPackage:  A Common Media Application Format (CMAF) packaging configuration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamSelection</i> A StreamSelection configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamSelection</b>: A StreamSelection configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentPrefix</i> An optional custom string that is prepended to the name of each segment. If not specified, it defaults to the ChannelId.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> Duration (in seconds) of each segment. Actual segments will be
rounded to the nearest multiple of the source segment duration.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HlsManifests</i> A list of HLS manifest configurations<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> A Common Media Application Format (CMAF) encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: A Common Media Application Format (CMAF) encryption configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyRotationIntervalSeconds</i> Time (in seconds) between each encryption key rotation.<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamOrder</i> A directive that determines the order of streams in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinVideoBitsPerSecond</i> The minimum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxVideoBitsPerSecond</i> The maximum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SuggestedPresentationDelaySeconds</i> Duration (in seconds) to delay live content before presentation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamSelection</i> A StreamSelection configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamSelection</b>: A StreamSelection configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentTemplateFormat</i> Determines the type of SegmentTemplate included in the Media Presentation Description (MPD).  When set to NUMBER_WITH_TIMELINE, a full timeline is presented in each SegmentTemplate, with $Number$ media URLs.  When set to TIME_WITH_TIMELINE, a full timeline is presented in each SegmentTemplate, with $Time$ media URLs. When set to NUMBER_WITH_DURATION, only a duration is included in each SegmentTemplate, with $Number$ media URLs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> Duration (in seconds) of each segment. Actual segments will be
rounded to the nearest multiple of the source segment duration.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Profile</i> The Dynamic Adaptive Streaming over HTTP (DASH) profile type.  When set to "HBBTV_1_5", HbbTV 1.5 compliant output is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PeriodTriggers</i> A list of triggers that controls when the outgoing Dynamic Adaptive Streaming over HTTP (DASH)
Media Presentation Description (MPD) will be partitioned into multiple periods. If empty, the content will not
be partitioned into more than one period. If the list contains "ADS", new periods will be created where
the Channel source contains SCTE-35 ad markers.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MinUpdatePeriodSeconds</i> Minimum duration (in seconds) between potential changes to the Dynamic Adaptive Streaming over HTTP (DASH) Media Presentation Description (MPD).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MinBufferTimeSeconds</i> Minimum duration (in seconds) that a player will buffer media before starting the presentation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManifestWindowSeconds</i> Time window (in seconds) contained in each manifest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManifestLayout</i> Determines the position of some tags in the Media Presentation Description (MPD).  When set to FULL, elements like SegmentTemplate and ContentProtection are included in each Representation.  When set to COMPACT, duplicate elements are combined and presented at the AdaptationSet level.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> A Dynamic Adaptive Streaming over HTTP (DASH) encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: A Dynamic Adaptive Streaming over HTTP (DASH) encryption configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdsOnDeliveryRestrictions</i> This setting allows the delivery restriction flags on SCTE-35 segmentation descriptors to
determine whether a message signals an ad.  Choosing "NONE" means no SCTE-35 messages become
ads.  Choosing "RESTRICTED" means SCTE-35 messages of the types specified in AdTriggers that
contain delivery restrictions will be treated as ads.  Choosing "UNRESTRICTED" means SCTE-35
messages of the types specified in AdTriggers that do not contain delivery restrictions will
be treated as ads.  Choosing "BOTH" means all SCTE-35 messages of the types specified in
AdTriggers will be treated as ads.  Note that Splice Insert messages do not have these flags
and are always treated as ads if specified in AdTriggers.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdTriggers</i> A list of SCTE-35 message types that are treated as ad markers in the output.  If empty, no
ad markers are output.  Specify multiple items to create ad markers for all of the included
message types.
<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyRotationIntervalSeconds</i> Time (in seconds) between each encryption key rotation.<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamOrder</i> A directive that determines the order of streams in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinVideoBitsPerSecond</i> The minimum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxVideoBitsPerSecond</i> The maximum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UseAudioRenditionGroup</i> When enabled, audio streams will be placed in rendition groups in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamSelection</i> A StreamSelection configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamSelection</b>: A StreamSelection configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> Duration (in seconds) of each fragment. Actual fragments will be
rounded to the nearest multiple of the source fragment duration.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProgramDateTimeIntervalSeconds</i> The interval (in seconds) between each EXT-X-PROGRAM-DATE-TIME tag
inserted into manifests. Additionally, when an interval is specified
ID3Timed Metadata messages will be generated every 5 seconds using the
ingest time of the content.
If the interval is not specified, or set to 0, then
no EXT-X-PROGRAM-DATE-TIME tags will be inserted into manifests and no
ID3Timed Metadata messages will be generated. Note that irrespective
of this parameter, if any ID3 Timed Metadata is found in HTTP Live Streaming (HLS) input,
it will be passed through to HLS output.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlaylistWindowSeconds</i> Time window (in seconds) contained in each parent manifest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlaylistType</i> The HTTP Live Streaming (HLS) playlist type.
When either "EVENT" or "VOD" is specified, a corresponding EXT-X-PLAYLIST-TYPE
entry will be included in the media playlist.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeIframeOnlyStream</i> When enabled, an I-Frame only stream will be included in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> An HTTP Live Streaming (HLS) encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: An HTTP Live Streaming (HLS) encryption configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdsOnDeliveryRestrictions</i> This setting allows the delivery restriction flags on SCTE-35 segmentation descriptors to
determine whether a message signals an ad.  Choosing "NONE" means no SCTE-35 messages become
ads.  Choosing "RESTRICTED" means SCTE-35 messages of the types specified in AdTriggers that
contain delivery restrictions will be treated as ads.  Choosing "UNRESTRICTED" means SCTE-35
messages of the types specified in AdTriggers that do not contain delivery restrictions will
be treated as ads.  Choosing "BOTH" means all SCTE-35 messages of the types specified in
AdTriggers will be treated as ads.  Note that Splice Insert messages do not have these flags
and are always treated as ads if specified in AdTriggers.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdTriggers</i> A list of SCTE-35 message types that are treated as ad markers in the output.  If empty, no
ad markers are output.  Specify multiple items to create ad markers for all of the included
message types.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdMarkers</i> This setting controls how ad markers are included in the packaged OriginEndpoint.
"NONE" will omit all SCTE-35 ad markers from the output.
"PASSTHROUGH" causes the manifest to contain a copy of the SCTE-35 ad
markers (comments) taken directly from the input HTTP Live Streaming (HLS) manifest.
"SCTE35_ENHANCED" generates ad markers and blackout tags based on SCTE-35
messages in the input source.
<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RepeatExtXKey</i> When enabled, the EXT-X-KEY tag will be repeated in output manifests.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyRotationIntervalSeconds</i> Interval (in seconds) between each encryption key rotation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionMethod</i> The encryption method to use.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ConstantInitializationVector</i> A constant initialization vector for encryption (optional).
When not specified the initialization vector will be periodically rotated.
<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamOrder</i> A directive that determines the order of streams in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinVideoBitsPerSecond</i> The minimum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxVideoBitsPerSecond</i> The maximum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamSelection</i> A StreamSelection configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamSelection</b>: A StreamSelection configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> The duration (in seconds) of each segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManifestWindowSeconds</i> The time window (in seconds) contained in each manifest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> A Microsoft Smooth Streaming (MSS) encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: A Microsoft Smooth Streaming (MSS) encryption configuration.<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamOrder</i> A directive that determines the order of streams in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinVideoBitsPerSecond</i> The minimum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxVideoBitsPerSecond</i> The maximum video bitrate (bps) to include in output.<br/>
        :return: Whitelist:  A list of source IP CIDR blocks that will be allowed to access the OriginEndpoint.<br/>Url:  The URL of the packaged OriginEndpoint for consumption.<br/>TimeDelaySeconds:  Amount of delay (seconds) to enforce on the playback of live content.
If not specified, there will be no time delay in effect for the OriginEndpoint.
<br/>Tags:  A collection of tags associated with a resource<br/>StartoverWindowSeconds:  Maximum duration (seconds) of content to retain for startover playback.
If not specified, startover playback will be disabled for the OriginEndpoint.
<br/>MssPackage:  A Microsoft Smooth Streaming (MSS) packaging configuration.:<br/>ManifestName:  A short string appended to the end of the OriginEndpoint URL.<br/>Id:  The ID of the OriginEndpoint.<br/>HlsPackage:  An HTTP Live Streaming (HLS) packaging configuration.:<br/>Description:  A short text description of the OriginEndpoint.<br/>DashPackage:  A Dynamic Adaptive Streaming over HTTP (DASH) packaging configuration.:<br/>CmafPackage:  A Common Media Application Format (CMAF) packaging configuration.:<br/>ChannelId:  The ID of the Channel the OriginEndpoint is associated with.<br/>Arn:  The Amazon Resource Name (ARN) assigned to the OriginEndpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamSelection</i> A StreamSelection configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamSelection</b>: A StreamSelection configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentPrefix</i> An optional custom string that is prepended to the name of each segment. If not specified, it defaults to the ChannelId.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> Duration (in seconds) of each segment. Actual segments will be
rounded to the nearest multiple of the source segment duration.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HlsManifests</i> A list of HLS manifest configurations<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> A Common Media Application Format (CMAF) encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: A Common Media Application Format (CMAF) encryption configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyRotationIntervalSeconds</i> Time (in seconds) between each encryption key rotation.<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamOrder</i> A directive that determines the order of streams in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinVideoBitsPerSecond</i> The minimum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxVideoBitsPerSecond</i> The maximum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SuggestedPresentationDelaySeconds</i> Duration (in seconds) to delay live content before presentation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamSelection</i> A StreamSelection configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamSelection</b>: A StreamSelection configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentTemplateFormat</i> Determines the type of SegmentTemplate included in the Media Presentation Description (MPD).  When set to NUMBER_WITH_TIMELINE, a full timeline is presented in each SegmentTemplate, with $Number$ media URLs.  When set to TIME_WITH_TIMELINE, a full timeline is presented in each SegmentTemplate, with $Time$ media URLs. When set to NUMBER_WITH_DURATION, only a duration is included in each SegmentTemplate, with $Number$ media URLs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> Duration (in seconds) of each segment. Actual segments will be
rounded to the nearest multiple of the source segment duration.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Profile</i> The Dynamic Adaptive Streaming over HTTP (DASH) profile type.  When set to "HBBTV_1_5", HbbTV 1.5 compliant output is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PeriodTriggers</i> A list of triggers that controls when the outgoing Dynamic Adaptive Streaming over HTTP (DASH)
Media Presentation Description (MPD) will be partitioned into multiple periods. If empty, the content will not
be partitioned into more than one period. If the list contains "ADS", new periods will be created where
the Channel source contains SCTE-35 ad markers.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MinUpdatePeriodSeconds</i> Minimum duration (in seconds) between potential changes to the Dynamic Adaptive Streaming over HTTP (DASH) Media Presentation Description (MPD).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MinBufferTimeSeconds</i> Minimum duration (in seconds) that a player will buffer media before starting the presentation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManifestWindowSeconds</i> Time window (in seconds) contained in each manifest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManifestLayout</i> Determines the position of some tags in the Media Presentation Description (MPD).  When set to FULL, elements like SegmentTemplate and ContentProtection are included in each Representation.  When set to COMPACT, duplicate elements are combined and presented at the AdaptationSet level.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> A Dynamic Adaptive Streaming over HTTP (DASH) encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: A Dynamic Adaptive Streaming over HTTP (DASH) encryption configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdsOnDeliveryRestrictions</i> This setting allows the delivery restriction flags on SCTE-35 segmentation descriptors to
determine whether a message signals an ad.  Choosing "NONE" means no SCTE-35 messages become
ads.  Choosing "RESTRICTED" means SCTE-35 messages of the types specified in AdTriggers that
contain delivery restrictions will be treated as ads.  Choosing "UNRESTRICTED" means SCTE-35
messages of the types specified in AdTriggers that do not contain delivery restrictions will
be treated as ads.  Choosing "BOTH" means all SCTE-35 messages of the types specified in
AdTriggers will be treated as ads.  Note that Splice Insert messages do not have these flags
and are always treated as ads if specified in AdTriggers.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdTriggers</i> A list of SCTE-35 message types that are treated as ad markers in the output.  If empty, no
ad markers are output.  Specify multiple items to create ad markers for all of the included
message types.
<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyRotationIntervalSeconds</i> Time (in seconds) between each encryption key rotation.<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamOrder</i> A directive that determines the order of streams in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinVideoBitsPerSecond</i> The minimum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxVideoBitsPerSecond</i> The maximum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UseAudioRenditionGroup</i> When enabled, audio streams will be placed in rendition groups in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamSelection</i> A StreamSelection configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamSelection</b>: A StreamSelection configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> Duration (in seconds) of each fragment. Actual fragments will be
rounded to the nearest multiple of the source fragment duration.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProgramDateTimeIntervalSeconds</i> The interval (in seconds) between each EXT-X-PROGRAM-DATE-TIME tag
inserted into manifests. Additionally, when an interval is specified
ID3Timed Metadata messages will be generated every 5 seconds using the
ingest time of the content.
If the interval is not specified, or set to 0, then
no EXT-X-PROGRAM-DATE-TIME tags will be inserted into manifests and no
ID3Timed Metadata messages will be generated. Note that irrespective
of this parameter, if any ID3 Timed Metadata is found in HTTP Live Streaming (HLS) input,
it will be passed through to HLS output.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlaylistWindowSeconds</i> Time window (in seconds) contained in each parent manifest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlaylistType</i> The HTTP Live Streaming (HLS) playlist type.
When either "EVENT" or "VOD" is specified, a corresponding EXT-X-PLAYLIST-TYPE
entry will be included in the media playlist.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludeIframeOnlyStream</i> When enabled, an I-Frame only stream will be included in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> An HTTP Live Streaming (HLS) encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: An HTTP Live Streaming (HLS) encryption configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdsOnDeliveryRestrictions</i> This setting allows the delivery restriction flags on SCTE-35 segmentation descriptors to
determine whether a message signals an ad.  Choosing "NONE" means no SCTE-35 messages become
ads.  Choosing "RESTRICTED" means SCTE-35 messages of the types specified in AdTriggers that
contain delivery restrictions will be treated as ads.  Choosing "UNRESTRICTED" means SCTE-35
messages of the types specified in AdTriggers that do not contain delivery restrictions will
be treated as ads.  Choosing "BOTH" means all SCTE-35 messages of the types specified in
AdTriggers will be treated as ads.  Note that Splice Insert messages do not have these flags
and are always treated as ads if specified in AdTriggers.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdTriggers</i> A list of SCTE-35 message types that are treated as ad markers in the output.  If empty, no
ad markers are output.  Specify multiple items to create ad markers for all of the included
message types.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdMarkers</i> This setting controls how ad markers are included in the packaged OriginEndpoint.
"NONE" will omit all SCTE-35 ad markers from the output.
"PASSTHROUGH" causes the manifest to contain a copy of the SCTE-35 ad
markers (comments) taken directly from the input HTTP Live Streaming (HLS) manifest.
"SCTE35_ENHANCED" generates ad markers and blackout tags based on SCTE-35
messages in the input source.
<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RepeatExtXKey</i> When enabled, the EXT-X-KEY tag will be repeated in output manifests.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyRotationIntervalSeconds</i> Interval (in seconds) between each encryption key rotation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionMethod</i> The encryption method to use.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ConstantInitializationVector</i> A constant initialization vector for encryption (optional).
When not specified the initialization vector will be periodically rotated.
<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamOrder</i> A directive that determines the order of streams in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinVideoBitsPerSecond</i> The minimum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxVideoBitsPerSecond</i> The maximum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamSelection</i> A StreamSelection configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StreamSelection</b>: A StreamSelection configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> The duration (in seconds) of each segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManifestWindowSeconds</i> The time window (in seconds) contained in each manifest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> A Microsoft Smooth Streaming (MSS) encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: A Microsoft Smooth Streaming (MSS) encryption configuration.<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StreamOrder</i> A directive that determines the order of streams in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinVideoBitsPerSecond</i> The minimum video bitrate (bps) to include in output.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxVideoBitsPerSecond</i> The maximum video bitrate (bps) to include in output.<br/>        """
        pass



class MediaPackageVod(BaseClient):
    """AWS Elemental MediaPackage VOD"""


    class ListAssetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_assets` for valid parameters"""
            pass
    class ListPackagingConfigurationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_packaging_configurations` for valid parameters"""
            pass
    class ListPackagingGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_packaging_groups` for valid parameters"""
            pass


    class CreateAssetRequest(Mapping):
        pass
    
    class CreateAssetResponse(Mapping):
        pass
    
    class CreatePackagingConfigurationRequest(Mapping):
        pass
    
    class CreatePackagingConfigurationResponse(Mapping):
        pass
    
    class CmafPackage(Mapping):
        pass
    
    class DashPackage(Mapping):
        pass
    
    class HlsPackage(Mapping):
        pass
    
    class MssPackage(Mapping):
        pass
    
    class CreatePackagingGroupRequest(Mapping):
        pass
    
    class CreatePackagingGroupResponse(Mapping):
        pass
    
    class DeleteAssetRequest(Mapping):
        pass
    
    class DeleteAssetResponse(Mapping):
        pass
    
    class DeletePackagingConfigurationRequest(Mapping):
        pass
    
    class DeletePackagingConfigurationResponse(Mapping):
        pass
    
    class DeletePackagingGroupRequest(Mapping):
        pass
    
    class DeletePackagingGroupResponse(Mapping):
        pass
    
    class DescribeAssetRequest(Mapping):
        pass
    
    class DescribeAssetResponse(Mapping):
        pass
    
    class DescribePackagingConfigurationRequest(Mapping):
        pass
    
    class DescribePackagingConfigurationResponse(Mapping):
        pass
    
    class DescribePackagingGroupRequest(Mapping):
        pass
    
    class DescribePackagingGroupResponse(Mapping):
        pass
    
    class ListAssetsRequest(Mapping):
        pass
    
    class ListAssetsResponse(Mapping):
        pass
    
    class ListPackagingConfigurationsRequest(Mapping):
        pass
    
    class ListPackagingConfigurationsResponse(Mapping):
        pass
    
    class ListPackagingGroupsRequest(Mapping):
        pass
    
    class ListPackagingGroupsResponse(Mapping):
        pass
    
    def create_asset(self, SourceRoleArn: str, SourceArn: str, PackagingGroupId: str, Id: str, ResourceId: str=None) -> CreateAssetResponse:
        """Creates a new MediaPackage VOD Asset resource.

        :param self:
        :param SourceRoleArn:  The IAM role ARN used to access the source S3 bucket.<br/>SourceArn:  ARN of the source object in S3.<br/>ResourceId:  The resource ID to include in SPEKE key requests.<br/>PackagingGroupId:  The ID of the PackagingGroup for the Asset.<br/>Id:  The unique identifier for the Asset.<br/>
        :return: SourceRoleArn:  The IAM role_arn used to access the source S3 bucket.<br/>SourceArn:  ARN of the source object in S3.<br/>ResourceId:  The resource ID to include in SPEKE key requests.<br/>PackagingGroupId:  The ID of the PackagingGroup for the Asset.<br/>Id:  The unique identifier for the Asset.<br/>EgressEndpoints:  The list of egress endpoints available for the Asset.<br/>Arn:  The ARN of the Asset.<br/>        """
        pass

    def create_packaging_configuration(self, PackagingGroupId: str, Id: str, CmafPackage: CmafPackage=None, DashPackage: DashPackage=None, HlsPackage: HlsPackage=None, MssPackage: MssPackage=None) -> CreatePackagingConfigurationResponse:
        """Creates a new MediaPackage VOD PackagingConfiguration resource.

        :param self:
        :param PackagingGroupId:  The ID of a PackagingGroup.<br/>MssPackage:  A Microsoft Smooth Streaming (MSS) PackagingConfiguration.:<br/>Id:  The ID of the PackagingConfiguration.<br/>HlsPackage:  An HTTP Live Streaming (HLS) packaging configuration.:<br/>DashPackage:  A Dynamic Adaptive Streaming over HTTP (DASH) packaging configuration.:<br/>CmafPackage:  A CMAF packaging configuration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> Duration (in seconds) of each fragment. Actual fragments will be
rounded to the nearest multiple of the source fragment duration.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HlsManifests</i> A list of HLS manifest configurations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> A CMAF encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: A CMAF encryption configuration.<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> Duration (in seconds) of each segment. Actual segments will be
rounded to the nearest multiple of the source segment duration.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> A Dynamic Adaptive Streaming over HTTP (DASH) encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: A Dynamic Adaptive Streaming over HTTP (DASH) encryption configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DashManifests</i> A list of DASH manifest configurations.<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp; <i>UseAudioRenditionGroup</i> When enabled, audio streams will be placed in rendition groups in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> Duration (in seconds) of each fragment. Actual fragments will be
rounded to the nearest multiple of the source fragment duration.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HlsManifests</i> A list of HLS manifest configurations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> An HTTP Live Streaming (HLS) encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: An HTTP Live Streaming (HLS) encryption configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionMethod</i> The encryption method to use.<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> The duration (in seconds) of each segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MssManifests</i> A list of MSS manifest configurations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> A Microsoft Smooth Streaming (MSS) encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: A Microsoft Smooth Streaming (MSS) encryption configuration.<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.
        :return: PackagingGroupId:  The ID of a PackagingGroup.<br/>MssPackage:  A Microsoft Smooth Streaming (MSS) PackagingConfiguration.:<br/>Id:  The ID of the PackagingConfiguration.<br/>HlsPackage:  An HTTP Live Streaming (HLS) packaging configuration.:<br/>DashPackage:  A Dynamic Adaptive Streaming over HTTP (DASH) packaging configuration.:<br/>CmafPackage:  A CMAF packaging configuration.:<br/>Arn:  The ARN of the PackagingConfiguration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> Duration (in seconds) of each fragment. Actual fragments will be
rounded to the nearest multiple of the source fragment duration.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HlsManifests</i> A list of HLS manifest configurations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> A CMAF encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: A CMAF encryption configuration.<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> Duration (in seconds) of each segment. Actual segments will be
rounded to the nearest multiple of the source segment duration.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> A Dynamic Adaptive Streaming over HTTP (DASH) encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: A Dynamic Adaptive Streaming over HTTP (DASH) encryption configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DashManifests</i> A list of DASH manifest configurations.<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp; <i>UseAudioRenditionGroup</i> When enabled, audio streams will be placed in rendition groups in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> Duration (in seconds) of each fragment. Actual fragments will be
rounded to the nearest multiple of the source fragment duration.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HlsManifests</i> A list of HLS manifest configurations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> An HTTP Live Streaming (HLS) encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: An HTTP Live Streaming (HLS) encryption configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionMethod</i> The encryption method to use.<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> The duration (in seconds) of each segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MssManifests</i> A list of MSS manifest configurations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> A Microsoft Smooth Streaming (MSS) encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: A Microsoft Smooth Streaming (MSS) encryption configuration.<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.        """
        pass

    def create_packaging_group(self, Id: str) -> CreatePackagingGroupResponse:
        """Creates a new MediaPackage VOD PackagingGroup resource.

        :param self:
        :param Id:  The ID of the PackagingGroup.<br/>
        :return: Id:  The ID of the PackagingGroup.<br/>Arn:  The ARN of the PackagingGroup.<br/>        """
        pass

    def delete_asset(self, Id: str) -> DeleteAssetResponse:
        """Deletes an existing MediaPackage VOD Asset resource.

        :param self:
        :param Id:  The ID of the MediaPackage VOD Asset resource to delete.<br/>
        :return:         """
        pass

    def delete_packaging_configuration(self, Id: str) -> DeletePackagingConfigurationResponse:
        """Deletes a MediaPackage VOD PackagingConfiguration resource.

        :param self:
        :param Id:  The ID of the MediaPackage VOD PackagingConfiguration resource to delete.<br/>
        :return:         """
        pass

    def delete_packaging_group(self, Id: str) -> DeletePackagingGroupResponse:
        """Deletes a MediaPackage VOD PackagingGroup resource.

        :param self:
        :param Id:  The ID of the MediaPackage VOD PackagingGroup resource to delete.<br/>
        :return:         """
        pass

    def describe_asset(self, Id: str) -> DescribeAssetResponse:
        """Returns a description of a MediaPackage VOD Asset resource.

        :param self:
        :param Id:  The ID of an MediaPackage VOD Asset resource.<br/>
        :return: SourceRoleArn:  The IAM role_arn used to access the source S3 bucket.<br/>SourceArn:  ARN of the source object in S3.<br/>ResourceId:  The resource ID to include in SPEKE key requests.<br/>PackagingGroupId:  The ID of the PackagingGroup for the Asset.<br/>Id:  The unique identifier for the Asset.<br/>EgressEndpoints:  The list of egress endpoints available for the Asset.<br/>Arn:  The ARN of the Asset.<br/>        """
        pass

    def describe_packaging_configuration(self, Id: str) -> DescribePackagingConfigurationResponse:
        """Returns a description of a MediaPackage VOD PackagingConfiguration resource.

        :param self:
        :param Id:  The ID of a MediaPackage VOD PackagingConfiguration resource.<br/>
        :return: PackagingGroupId:  The ID of a PackagingGroup.<br/>MssPackage:  A Microsoft Smooth Streaming (MSS) PackagingConfiguration.:<br/>Id:  The ID of the PackagingConfiguration.<br/>HlsPackage:  An HTTP Live Streaming (HLS) packaging configuration.:<br/>DashPackage:  A Dynamic Adaptive Streaming over HTTP (DASH) packaging configuration.:<br/>CmafPackage:  A CMAF packaging configuration.:<br/>Arn:  The ARN of the PackagingConfiguration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> Duration (in seconds) of each fragment. Actual fragments will be
rounded to the nearest multiple of the source fragment duration.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HlsManifests</i> A list of HLS manifest configurations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> A CMAF encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: A CMAF encryption configuration.<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> Duration (in seconds) of each segment. Actual segments will be
rounded to the nearest multiple of the source segment duration.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> A Dynamic Adaptive Streaming over HTTP (DASH) encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: A Dynamic Adaptive Streaming over HTTP (DASH) encryption configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DashManifests</i> A list of DASH manifest configurations.<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp; <i>UseAudioRenditionGroup</i> When enabled, audio streams will be placed in rendition groups in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> Duration (in seconds) of each fragment. Actual fragments will be
rounded to the nearest multiple of the source fragment duration.
<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HlsManifests</i> A list of HLS manifest configurations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> An HTTP Live Streaming (HLS) encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: An HTTP Live Streaming (HLS) encryption configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionMethod</i> The encryption method to use.<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentDurationSeconds</i> The duration (in seconds) of each segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MssManifests</i> A list of MSS manifest configurations.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encryption</i> A Microsoft Smooth Streaming (MSS) encryption configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Encryption</b>: A Microsoft Smooth Streaming (MSS) encryption configuration.<br/>A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SpekeKeyProvider</b>: A configuration for accessing an external Secure Packager and Encoder Key Exchange (SPEKE) service that will provide encryption keys.        """
        pass

    def describe_packaging_group(self, Id: str) -> DescribePackagingGroupResponse:
        """Returns a description of a MediaPackage VOD PackagingGroup resource.

        :param self:
        :param Id:  The ID of a MediaPackage VOD PackagingGroup resource.<br/>
        :return: Id:  The ID of the PackagingGroup.<br/>Arn:  The ARN of the PackagingGroup.<br/>        """
        pass

    def list_assets(self, MaxResults: int=None, NextToken: str=None, PackagingGroupId: str=None) -> ListAssetsResponse:
        """Returns a collection of MediaPackage VOD Asset resources.

        :param self:
        :param PackagingGroupId:  Returns Assets associated with the specified PackagingGroup.<br/>NextToken:  A token used to resume pagination from the end of a previous request.<br/>MaxResults:  Upper bound on number of records to return.<br/>
        :return: NextToken:  A token that can be used to resume pagination from the end of the collection.<br/>Assets:  A list of MediaPackage VOD Asset resources.<br/>        """
        pass

    def list_packaging_configurations(self, MaxResults: int=None, NextToken: str=None, PackagingGroupId: str=None) -> ListPackagingConfigurationsResponse:
        """Returns a collection of MediaPackage VOD PackagingConfiguration resources.

        :param self:
        :param PackagingGroupId:  Returns MediaPackage VOD PackagingConfigurations associated with the specified PackagingGroup.<br/>NextToken:  A token used to resume pagination from the end of a previous request.<br/>MaxResults:  Upper bound on number of records to return.<br/>
        :return: PackagingConfigurations:  A list of MediaPackage VOD PackagingConfiguration resources.<br/>NextToken:  A token that can be used to resume pagination from the end of the collection.<br/>        """
        pass

    def list_packaging_groups(self, MaxResults: int=None, NextToken: str=None) -> ListPackagingGroupsResponse:
        """Returns a collection of MediaPackage VOD PackagingGroup resources.

        :param self:
        :param NextToken:  A token used to resume pagination from the end of a previous request.<br/>MaxResults:  Upper bound on number of records to return.<br/>
        :return: PackagingGroups:  A list of MediaPackage VOD PackagingGroup resources.<br/>NextToken:  A token that can be used to resume pagination from the end of the collection.<br/>        """
        pass



class MediaStore(BaseClient):
    """An AWS Elemental MediaStore container is a namespace that holds folders and objects. You use a container endpoint to create, read, and delete objects. """


    class ListContainersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_containers` for valid parameters"""
            pass


    class CreateContainerInput(Mapping):
        pass
    
    class CreateContainerOutput(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class DeleteContainerInput(Mapping):
        pass
    
    class DeleteContainerOutput(Mapping):
        pass
    
    class DeleteContainerPolicyInput(Mapping):
        pass
    
    class DeleteContainerPolicyOutput(Mapping):
        pass
    
    class DeleteCorsPolicyInput(Mapping):
        pass
    
    class DeleteCorsPolicyOutput(Mapping):
        pass
    
    class DeleteLifecyclePolicyInput(Mapping):
        pass
    
    class DeleteLifecyclePolicyOutput(Mapping):
        pass
    
    class DescribeContainerInput(Mapping):
        pass
    
    class DescribeContainerOutput(Mapping):
        pass
    
    class GetContainerPolicyInput(Mapping):
        pass
    
    class GetContainerPolicyOutput(Mapping):
        pass
    
    class GetCorsPolicyInput(Mapping):
        pass
    
    class GetCorsPolicyOutput(Mapping):
        pass
    
    class GetLifecyclePolicyInput(Mapping):
        pass
    
    class GetLifecyclePolicyOutput(Mapping):
        pass
    
    class ListContainersInput(Mapping):
        pass
    
    class ListContainersOutput(Mapping):
        pass
    
    class ListTagsForResourceInput(Mapping):
        pass
    
    class ListTagsForResourceOutput(Mapping):
        pass
    
    class PutContainerPolicyInput(Mapping):
        pass
    
    class PutContainerPolicyOutput(Mapping):
        pass
    
    class PutCorsPolicyInput(Mapping):
        pass
    
    class PutCorsPolicyOutput(Mapping):
        pass
    
    class CorsRule(Mapping):
        pass
    
    class PutLifecyclePolicyInput(Mapping):
        pass
    
    class PutLifecyclePolicyOutput(Mapping):
        pass
    
    class StartAccessLoggingInput(Mapping):
        pass
    
    class StartAccessLoggingOutput(Mapping):
        pass
    
    class StopAccessLoggingInput(Mapping):
        pass
    
    class StopAccessLoggingOutput(Mapping):
        pass
    
    class TagResourceInput(Mapping):
        pass
    
    class TagResourceOutput(Mapping):
        pass
    
    class UntagResourceInput(Mapping):
        pass
    
    class UntagResourceOutput(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    def create_container(self, ContainerName: str, Tags: List[Tag]=None) -> CreateContainerOutput:
        """Creates a storage container to hold objects. A container is similar to a bucket in the Amazon S3 service.

        :param self:
        :param Tags:  An array of key:value pairs that you define. These values can be anything that you want. Typically, the tag key represents a category (such as "environment") and the tag value represents a specific value within that category (such as "test," "development," or "production"). You can add up to 50 tags to each container. For more information about tagging, including naming and usage conventions, see <a href="https://aws.amazon.com/documentation/mediastore/tagging">Tagging Resources in MediaStore</a>.<br/>ContainerName:  The name for the container. The name must be from 1 to 255 characters. Container names must be unique to your AWS account within a specific region. As an example, you could create a container named <code>movies</code> in every region, as long as you don’t have an existing container with that name.<br/>
        :return: Container:  ContainerARN: The Amazon Resource Name (ARN) of the newly created container. The ARN has the following format: arn:aws:&lt;region&gt;:&lt;account that owns this container&gt;:container/&lt;name of container&gt;. For example: arn:aws:mediastore:us-west-2:111122223333:container/movies  ContainerName: The container name as specified in the request. CreationTime: Unix time stamp. Status: The status of container creation or deletion. The status is one of the following: <code>CREATING</code>, <code>ACTIVE</code>, or <code>DELETING</code>. While the service is creating the container, the status is <code>CREATING</code>. When an endpoint is available, the status changes to <code>ACTIVE</code>. The return value does not include the container's endpoint. To make downstream requests, you must obtain this value by using <a>DescribeContainer</a> or <a>ListContainers</a>.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccessLoggingEnabled</i> The state of access logging on the container. This value is <code>false</code> by default, indicating that AWS Elemental MediaStore does not send access logs to Amazon CloudWatch Logs. When you enable access logging on the container, MediaStore changes this value to <code>true</code>, indicating that the service delivers access logs for objects stored in that container to CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of container creation or deletion. The status is one of the following: <code>CREATING</code>, <code>ACTIVE</code>, or <code>DELETING</code>. While the service is creating the container, the status is <code>CREATING</code>. When the endpoint is available, the status changes to <code>ACTIVE</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the container.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ARN</i> The Amazon Resource Name (ARN) of the container. The ARN has the following format: arn:aws:&lt;region&gt;:&lt;account that owns this container&gt;:container/&lt;name of container&gt;  For example: arn:aws:mediastore:us-west-2:111122223333:container/movies <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> Unix timestamp.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> The DNS endpoint of the container. Use the endpoint to identify the specific container when sending requests to the data plane. The service assigns this value when the container is created. Once the value has been assigned, it does not change.<br/>        """
        pass

    def delete_container(self, ContainerName: str) -> DeleteContainerOutput:
        """Deletes the specified container. Before you make a <code>DeleteContainer</code> request, delete any objects in the container or in any folders in the container. You can delete only empty containers. 

        :param self:
        :param ContainerName:  The name of the container to delete. <br/>
        :return:         """
        pass

    def delete_container_policy(self, ContainerName: str) -> DeleteContainerPolicyOutput:
        """Deletes the access policy that is associated with the specified container.

        :param self:
        :param ContainerName:  The name of the container that holds the policy.<br/>
        :return:         """
        pass

    def delete_cors_policy(self, ContainerName: str) -> DeleteCorsPolicyOutput:
        """Deletes the cross-origin resource sharing (CORS) configuration information that is set for the container. To use this operation, you must have permission to perform the <code>MediaStore:DeleteCorsPolicy</code> action. The container owner has this permission by default and can grant this permission to others.

        :param self:
        :param ContainerName:  The name of the container to remove the policy from.<br/>
        :return:         """
        pass

    def delete_lifecycle_policy(self, ContainerName: str) -> DeleteLifecyclePolicyOutput:
        """Removes an object lifecycle policy from a container. It takes up to 20 minutes for the change to take effect.

        :param self:
        :param ContainerName:  The name of the container that holds the object lifecycle policy.<br/>
        :return:         """
        pass

    def describe_container(self, ContainerName: str=None) -> DescribeContainerOutput:
        """Retrieves the properties of the requested container. This request is commonly used to retrieve the endpoint of a container. An endpoint is a value assigned by the service when a new container is created. A container's endpoint does not change after it has been assigned. The <code>DescribeContainer</code> request returns a single <code>Container</code> object based on <code>ContainerName</code>. To return all <code>Container</code> objects that are associated with a specified AWS account, use <a>ListContainers</a>.

        :param self:
        :param ContainerName:  The name of the container to query.<br/>
        :return: Container:  The name of the queried container.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccessLoggingEnabled</i> The state of access logging on the container. This value is <code>false</code> by default, indicating that AWS Elemental MediaStore does not send access logs to Amazon CloudWatch Logs. When you enable access logging on the container, MediaStore changes this value to <code>true</code>, indicating that the service delivers access logs for objects stored in that container to CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of container creation or deletion. The status is one of the following: <code>CREATING</code>, <code>ACTIVE</code>, or <code>DELETING</code>. While the service is creating the container, the status is <code>CREATING</code>. When the endpoint is available, the status changes to <code>ACTIVE</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the container.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ARN</i> The Amazon Resource Name (ARN) of the container. The ARN has the following format: arn:aws:&lt;region&gt;:&lt;account that owns this container&gt;:container/&lt;name of container&gt;  For example: arn:aws:mediastore:us-west-2:111122223333:container/movies <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> Unix timestamp.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> The DNS endpoint of the container. Use the endpoint to identify the specific container when sending requests to the data plane. The service assigns this value when the container is created. Once the value has been assigned, it does not change.<br/>        """
        pass

    def get_container_policy(self, ContainerName: str) -> GetContainerPolicyOutput:
        """Retrieves the access policy for the specified container. For information about the data that is included in an access policy, see the <a href="https://aws.amazon.com/documentation/iam/">AWS Identity and Access Management User Guide</a>.

        :param self:
        :param ContainerName:  The name of the container. <br/>
        :return: Policy:  The contents of the access policy.<br/>        """
        pass

    def get_cors_policy(self, ContainerName: str) -> GetCorsPolicyOutput:
        """Returns the cross-origin resource sharing (CORS) configuration information that is set for the container. To use this operation, you must have permission to perform the <code>MediaStore:GetCorsPolicy</code> action. By default, the container owner has this permission and can grant it to others.

        :param self:
        :param ContainerName:  The name of the container that the policy is assigned to.<br/>
        :return: CorsPolicy:  The CORS policy assigned to the container.<br/>        """
        pass

    def get_lifecycle_policy(self, ContainerName: str) -> GetLifecyclePolicyOutput:
        """Retrieves the object lifecycle policy that is assigned to a container.

        :param self:
        :param ContainerName:  The name of the container that the object lifecycle policy is assigned to.<br/>
        :return: LifecyclePolicy:  The object lifecycle policy that is assigned to the container.<br/>        """
        pass

    def list_containers(self, NextToken: str=None, MaxResults: int=None) -> ListContainersOutput:
        """Lists the properties of all containers in AWS Elemental MediaStore.  You can query to receive all the containers in one response. Or you can include the <code>MaxResults</code> parameter to receive a limited number of containers in each response. In this case, the response includes a token. To get the next set of containers, send the command again, this time with the <code>NextToken</code> parameter (with the returned token as its value). The next set of responses appears, with a token if there are still more containers to receive.  See also <a>DescribeContainer</a>, which gets the properties of one container. 

        :param self:
        :param MaxResults:  Enter the maximum number of containers in the response. Use from 1 to 255 characters. <br/>NextToken:  Only if you used <code>MaxResults</code> in the first command, enter the token (which was included in the previous response) to obtain the next set of containers. This token is included in a response only if there actually are more containers to list.<br/>
        :return: NextToken:   <code>NextToken</code> is the token to use in the next call to <code>ListContainers</code>. This token is returned only if you included the <code>MaxResults</code> tag in the original command, and only if there are still containers to return. <br/>Containers:  The names of the containers.<br/>        """
        pass

    def list_tags_for_resource(self, Resource: str) -> ListTagsForResourceOutput:
        """Returns a list of the tags assigned to the specified container. 

        :param self:
        :param Resource:  The Amazon Resource Name (ARN) for the container.<br/>
        :return: Tags:  An array of key:value pairs that are assigned to the container.<br/>        """
        pass

    def put_container_policy(self, Policy: str, ContainerName: str) -> PutContainerPolicyOutput:
        """Creates an access policy for the specified container to restrict the users and clients that can access it. For information about the data that is included in an access policy, see the <a href="https://aws.amazon.com/documentation/iam/">AWS Identity and Access Management User Guide</a>. For this release of the REST API, you can create only one policy for a container. If you enter <code>PutContainerPolicy</code> twice, the second command modifies the existing policy. 

        :param self:
        :param Policy:  The contents of the policy, which includes the following:  <ul> <li> One <code>Version</code> tag </li> <li> One <code>Statement</code> tag that contains the standard tags for the policy. </li> </ul><br/>ContainerName:  The name of the container.<br/>
        :return:         """
        pass

    def put_cors_policy(self, CorsPolicy: List[CorsRule], ContainerName: str) -> PutCorsPolicyOutput:
        """Sets the cross-origin resource sharing (CORS) configuration on a container so that the container can service cross-origin requests. For example, you might want to enable a request whose origin is http://www.example.com to access your AWS Elemental MediaStore container at my.example.container.com by using the browser's XMLHttpRequest capability. To enable CORS on a container, you attach a CORS policy to the container. In the CORS policy, you configure rules that identify origins and the HTTP methods that can be executed on your container. The policy can contain up to 398,000 characters. You can add up to 100 rules to a CORS policy. If more than one rule applies, the service uses the first applicable rule listed. To learn more about CORS, see <a href="https://docs.aws.amazon.com/mediastore/latest/ug/cors-policy.html">Cross-Origin Resource Sharing (CORS) in AWS Elemental MediaStore</a>.

        :param self:
        :param CorsPolicy:  The CORS policy to apply to the container. <br/>ContainerName:  The name of the container that you want to assign the CORS policy to.<br/>
        :return:         """
        pass

    def put_lifecycle_policy(self, LifecyclePolicy: str, ContainerName: str) -> PutLifecyclePolicyOutput:
        """Writes an object lifecycle policy to a container. If the container already has an object lifecycle policy, the service replaces the existing policy with the new policy. It takes up to 20 minutes for the change to take effect. For information about how to construct an object lifecycle policy, see <a href="https://docs.aws.amazon.com/mediastore/latest/ug/policies-object-lifecycle-components.html">Components of an Object Lifecycle Policy</a>.

        :param self:
        :param LifecyclePolicy:  The object lifecycle policy to apply to the container.<br/>ContainerName:  The name of the container that you want to assign the object lifecycle policy to.<br/>
        :return:         """
        pass

    def start_access_logging(self, ContainerName: str) -> StartAccessLoggingOutput:
        """Starts access logging on the specified container. When you enable access logging on a container, MediaStore delivers access logs for objects stored in that container to Amazon CloudWatch Logs.

        :param self:
        :param ContainerName:  The name of the container that you want to start access logging on.<br/>
        :return:         """
        pass

    def stop_access_logging(self, ContainerName: str) -> StopAccessLoggingOutput:
        """Stops access logging on the specified container. When you stop access logging on a container, MediaStore stops sending access logs to Amazon CloudWatch Logs. These access logs are not saved and are not retrievable.

        :param self:
        :param ContainerName:  The name of the container that you want to stop access logging on.<br/>
        :return:         """
        pass

    def tag_resource(self, Tags: List[Tag], Resource: str) -> TagResourceOutput:
        """Adds tags to the specified AWS Elemental MediaStore container. Tags are key:value pairs that you can associate with AWS resources. For example, the tag key might be "customer" and the tag value might be "companyA." You can specify one or more tags to add to each container. You can add up to 50 tags to each container. For more information about tagging, including naming and usage conventions, see <a href="https://aws.amazon.com/documentation/mediastore/tagging">Tagging Resources in MediaStore</a>.

        :param self:
        :param Tags:  An array of key:value pairs that you want to add to the container. You need to specify only the tags that you want to add or update. For example, suppose a container already has two tags (customer:CompanyA and priority:High). You want to change the priority tag and also add a third tag (type:Contract). For TagResource, you specify the following tags: priority:Medium, type:Contract. The result is that your container has three tags: customer:CompanyA, priority:Medium, and type:Contract.<br/>Resource:  The Amazon Resource Name (ARN) for the container. <br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[TagKey], Resource: str) -> UntagResourceOutput:
        """Removes tags from the specified container. You can specify one or more tags to remove. 

        :param self:
        :param TagKeys:  A comma-separated list of keys for tags that you want to remove from the container. For example, if your container has two tags (customer:CompanyA and priority:High) and you want to remove one of the tags (priority:High), you specify the key for the tag that you want to remove (priority).<br/>Resource:  The Amazon Resource Name (ARN) for the container.<br/>
        :return:         """
        pass



class MediaStoreData(BaseClient):
    """An AWS Elemental MediaStore asset is an object, similar to an object in the Amazon S3 service. Objects are the fundamental entities that are stored in AWS Elemental MediaStore."""


    class ListItemsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_items` for valid parameters"""
            pass


    class DeleteObjectRequest(Mapping):
        pass
    
    class DeleteObjectResponse(Mapping):
        pass
    
    class DescribeObjectRequest(Mapping):
        pass
    
    class DescribeObjectResponse(Mapping):
        pass
    
    class GetObjectRequest(Mapping):
        pass
    
    class GetObjectResponse(Mapping):
        pass
    
    class ListItemsRequest(Mapping):
        pass
    
    class ListItemsResponse(Mapping):
        pass
    
    class PutObjectRequest(Mapping):
        pass
    
    class PutObjectResponse(Mapping):
        pass
    
    class PayloadBlob(object):
        pass
    
    def delete_object(self, Path: str) -> DeleteObjectResponse:
        """Deletes an object at the specified path.

        :param self:
        :param Path:  The path (including the file name) where the object is stored in the container. Format: &lt;folder name&gt;/&lt;folder name&gt;/&lt;file name&gt;<br/>
        :return:         """
        pass

    def describe_object(self, Path: str) -> DescribeObjectResponse:
        """Gets the headers for an object at the specified path.

        :param self:
        :param Path:  The path (including the file name) where the object is stored in the container. Format: &lt;folder name&gt;/&lt;folder name&gt;/&lt;file name&gt;<br/>
        :return: LastModified:  The date and time that the object was last modified.<br/>CacheControl:  An optional <code>CacheControl</code> header that allows the caller to control the object's cache behavior. Headers can be passed in as specified in the HTTP at <a href="https://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.9">https://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.9</a>. Headers with a custom user-defined value are also accepted.<br/>ContentLength:  The length of the object in bytes.<br/>ContentType:  The content type of the object.<br/>ETag:  The ETag that represents a unique instance of the object.<br/>        """
        pass

    def get_object(self, Path: str, Range: str=None) -> GetObjectResponse:
        """Downloads the object at the specified path. If the object’s upload availability is set to <code>streaming</code>, AWS Elemental MediaStore downloads the object even if it’s still uploading the object.

        :param self:
        :param Range:  The range bytes of an object to retrieve. For more information about the <code>Range</code> header, see <a href="http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.35">http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.35</a>. AWS Elemental MediaStore ignores this header for partially uploaded objects that have streaming upload availability.<br/>Path:  The path (including the file name) where the object is stored in the container. Format: &lt;folder name&gt;/&lt;folder name&gt;/&lt;file name&gt; For example, to upload the file <code>mlaw.avi</code> to the folder path <code>premium\canada</code> in the container <code>movies</code>, enter the path <code>premium/canada/mlaw.avi</code>. Do not include the container name in this path. If the path includes any folders that don't exist yet, the service creates them. For example, suppose you have an existing <code>premium/usa</code> subfolder. If you specify <code>premium/canada</code>, the service creates a <code>canada</code> subfolder in the <code>premium</code> folder. You then have two subfolders, <code>usa</code> and <code>canada</code>, in the <code>premium</code> folder.  There is no correlation between the path to the source and the path (folders) in the container in AWS Elemental MediaStore. For more information about folders and how they exist in a container, see the <a href="http://docs.aws.amazon.com/mediastore/latest/ug/">AWS Elemental MediaStore User Guide</a>. The file name is the name that is assigned to the file that you upload. The file can have the same name inside and outside of AWS Elemental MediaStore, or it can have the same name. The file name can include or omit an extension. <br/>
        :return: StatusCode:  The HTML status code of the request. Status codes ranging from 200 to 299 indicate success. All other status codes indicate the type of error that occurred.<br/>LastModified:  The date and time that the object was last modified.<br/>ETag:  The ETag that represents a unique instance of the object.<br/>ContentType:  The content type of the object.<br/>ContentLength:  The length of the object in bytes.<br/>ContentRange:  The range of bytes to retrieve.<br/>CacheControl:  An optional <code>CacheControl</code> header that allows the caller to control the object's cache behavior. Headers can be passed in as specified in the HTTP spec at <a href="https://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.9">https://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.9</a>. Headers with a custom user-defined value are also accepted.<br/>Body:  The bytes of the object. <br/>        """
        pass

    def list_items(self, Path: str=None, MaxResults: int=None, NextToken: str=None) -> ListItemsResponse:
        """Provides a list of metadata entries about folders and objects in the specified folder.

        :param self:
        :param NextToken:  The token that identifies which batch of results that you want to see. For example, you submit a <code>ListItems</code> request with <code>MaxResults</code> set at 500. The service returns the first batch of results (up to 500) and a <code>NextToken</code> value. To see the next batch of results, you can submit the <code>ListItems</code> request a second time and specify the <code>NextToken</code> value. Tokens expire after 15 minutes.<br/>MaxResults:  The maximum number of results to return per API request. For example, you submit a <code>ListItems</code> request with <code>MaxResults</code> set at 500. Although 2,000 items match your request, the service returns no more than the first 500 items. (The service also returns a <code>NextToken</code> value that you can use to fetch the next batch of results.) The service might return fewer results than the <code>MaxResults</code> value. If <code>MaxResults</code> is not included in the request, the service defaults to pagination with a maximum of 1,000 results per page.<br/>Path:  The path in the container from which to retrieve items. Format: &lt;folder name&gt;/&lt;folder name&gt;/&lt;file name&gt;<br/>
        :return: NextToken:  The token that can be used in a request to view the next set of results. For example, you submit a <code>ListItems</code> request that matches 2,000 items with <code>MaxResults</code> set at 500. The service returns the first batch of results (up to 500) and a <code>NextToken</code> value that can be used to fetch the next batch of results.<br/>Items:  The metadata entries for the folders and objects at the requested path.<br/>        """
        pass

    def put_object(self, Path: str, Body: PayloadBlob, ContentType: str=None, CacheControl: str=None, StorageClass: str=None, UploadAvailability: str=None) -> PutObjectResponse:
        """Uploads an object to the specified path. Object sizes are limited to 25 MB for standard upload availability and 10 MB for streaming upload availability.

        :param self:
        :param UploadAvailability:  Indicates the availability of an object while it is still uploading. If the value is set to <code>streaming</code>, the object is available for downloading after some initial buffering but before the object is uploaded completely. If the value is set to <code>standard</code>, the object is available for downloading only when it is uploaded completely. The default value for this header is <code>standard</code>. To use this header, you must also set the HTTP <code>Transfer-Encoding</code> header to <code>chunked</code>.<br/>StorageClass:  Indicates the storage class of a <code>Put</code> request. Defaults to high-performance temporal storage class, and objects are persisted into durable storage shortly after being received.<br/>CacheControl:  An optional <code>CacheControl</code> header that allows the caller to control the object's cache behavior. Headers can be passed in as specified in the HTTP at <a href="https://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.9">https://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.9</a>. Headers with a custom user-defined value are also accepted.<br/>ContentType:  The content type of the object.<br/>Path:  The path (including the file name) where the object is stored in the container. Format: &lt;folder name&gt;/&lt;folder name&gt;/&lt;file name&gt; For example, to upload the file <code>mlaw.avi</code> to the folder path <code>premium\canada</code> in the container <code>movies</code>, enter the path <code>premium/canada/mlaw.avi</code>. Do not include the container name in this path. If the path includes any folders that don't exist yet, the service creates them. For example, suppose you have an existing <code>premium/usa</code> subfolder. If you specify <code>premium/canada</code>, the service creates a <code>canada</code> subfolder in the <code>premium</code> folder. You then have two subfolders, <code>usa</code> and <code>canada</code>, in the <code>premium</code> folder.  There is no correlation between the path to the source and the path (folders) in the container in AWS Elemental MediaStore. For more information about folders and how they exist in a container, see the <a href="http://docs.aws.amazon.com/mediastore/latest/ug/">AWS Elemental MediaStore User Guide</a>. The file name is the name that is assigned to the file that you upload. The file can have the same name inside and outside of AWS Elemental MediaStore, or it can have the same name. The file name can include or omit an extension. <br/>Body:  The bytes to be stored. <br/>
        :return: StorageClass:  The storage class where the object was persisted. The class should be “Temporal”.<br/>ETag:  Unique identifier of the object in the container.<br/>ContentSHA256:  The SHA256 digest of the object that is persisted.<br/>        """
        pass



class MediaTailor(BaseClient):
    """Use the AWS Elemental MediaTailor SDK to configure scalable ad insertion for your live and VOD content. With AWS Elemental MediaTailor, you can serve targeted ads to viewers while maintaining broadcast quality in over-the-top (OTT) video applications. For information about using the service, including detailed information about the settings covered in this guide, see the AWS Elemental MediaTailor User Guide.Through the SDK, you manage AWS Elemental MediaTailor configurations the same as you do through the console. For example, you specify ad insertion behavior and mapping information for the origin server and the ad decision server (ADS)."""


    class ListPlaybackConfigurationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_playback_configurations` for valid parameters"""
            pass


    class DeletePlaybackConfigurationRequest(Mapping):
        pass
    
    class DeletePlaybackConfigurationResponse(Mapping):
        pass
    
    class GetPlaybackConfigurationRequest(Mapping):
        pass
    
    class GetPlaybackConfigurationResponse(Mapping):
        pass
    
    class ListPlaybackConfigurationsRequest(Mapping):
        pass
    
    class ListPlaybackConfigurationsResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class PutPlaybackConfigurationRequest(Mapping):
        pass
    
    class PutPlaybackConfigurationResponse(Mapping):
        pass
    
    class CdnConfiguration(Mapping):
        pass
    
    class DashConfigurationForPut(Mapping):
        pass
    
    class __mapOf__string(object):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class __string(object):
        pass
    
    def delete_playback_configuration(self, Name: str) -> DeletePlaybackConfigurationResponse:
        """Deletes the playback configuration for the specified name. 

        :param self:
        :param Name:  The identifier for the playback configuration.<br/>
        :return:         """
        pass

    def get_playback_configuration(self, Name: str) -> GetPlaybackConfigurationResponse:
        """Returns the playback configuration for the specified name. 

        :param self:
        :param Name:  The identifier for the playback configuration.<br/>
        :return: VideoContentSourceUrl:  The URL prefix for the master playlist for the stream, minus the asset ID. The maximum length is 512 characters.<br/>TranscodeProfileName:  The name that is used to associate this playback configuration with a custom transcode profile. This overrides the dynamic transcoding defaults of MediaTailor. Use this only if you have already set up custom profiles with the help of AWS Support.<br/>Tags:  The tags assigned to the playback configuration. <br/>SlateAdUrl:  The URL for a high-quality video asset to transcode and use to fill in time that's not used by ads. AWS Elemental MediaTailor shows the slate to fill in gaps in media content. Configuring the slate is optional for non-VPAID playback configurations. For VPAID, the slate is required because MediaTailor provides it in the slots designated for dynamic ad content. The slate must be a high-quality asset that contains both audio and video. <br/>SessionInitializationEndpointPrefix:  The URL that the player uses to initialize a session that uses client-side reporting. <br/>PlaybackEndpointPrefix:  The URL that the player accesses to get a manifest from AWS Elemental MediaTailor. This session will use server-side reporting. <br/>PlaybackConfigurationArn:  The Amazon Resource Name (ARN) for the playback configuration. <br/>Name:  The identifier for the playback configuration.<br/>HlsConfiguration:  The configuration for HLS content. :<br/>DashConfiguration:  The configuration for DASH content. :<br/>CdnConfiguration:  The configuration for using a content delivery network (CDN), like Amazon CloudFront, for content and ad segment management. :<br/>AdDecisionServerUrl:  The URL for the ad decision server (ADS). This includes the specification of static parameters and placeholders for dynamic parameters. AWS Elemental MediaTailor substitutes player-specific and session-specific parameters as needed when calling the ADS. Alternately, for testing, you can provide a static VAST URL. The maximum length is 25,000 characters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContentSegmentUrlPrefix</i> A content delivery network (CDN) to cache content segments, so that content requests don’t always have to go to the origin server. First, create a rule in your CDN for the content segment origin server. Then specify the rule's name in this ContentSegmentUrlPrefix. When AWS Elemental MediaTailor serves a manifest, it reports your CDN as the source for content segments.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdSegmentUrlPrefix</i> A non-default content delivery network (CDN) to serve ad segments. By default, AWS Elemental MediaTailor uses Amazon CloudFront with default cache settings as its CDN for ad segments. To set up an alternate CDN, create a rule in your CDN for the following origin: ads.mediatailor.&lt;region>.amazonaws.com. Then specify the rule's name in this AdSegmentUrlPrefix. When AWS Elemental MediaTailor serves a manifest, it reports your CDN as the source for ad segments.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OriginManifestType</i> The setting that controls whether MediaTailor handles manifests from the origin server as multi-period manifests or single-period manifests. If your origin server produces single-period manifests, set this to SINGLE_PERIOD. The default setting is MULTI_PERIOD. For multi-period manifests, omit this setting or set it to MULTI_PERIOD. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MpdLocation</i> The setting that controls whether MediaTailor includes the Location tag in DASH manifests. MediaTailor populates the Location tag with the URL for manifest update requests, to be used by players that don't support sticky redirects. Disable this if you have CDN routing rules set up for accessing MediaTailor manifests, and you are either using client-side reporting or your players support sticky HTTP redirects. Valid values are DISABLED and EMT_DEFAULT. The EMT_DEFAULT setting enables the inclusion of the tag and is the default value. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManifestEndpointPrefix</i> The URL generated by MediaTailor to initiate a playback session. The session uses server-side reporting. This setting is ignored in PUT operations. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManifestEndpointPrefix</i> The URL that is used to initiate a playback session for devices that support Apple HLS. The session uses server-side reporting.<br/>        """
        pass

    def list_playback_configurations(self, MaxResults: int=None, NextToken: str=None) -> ListPlaybackConfigurationsResponse:
        """Returns a list of the playback configurations defined in AWS Elemental MediaTailor. You can specify a maximum number of configurations to return at a time. The default maximum is 50. Results are returned in pagefuls. If MediaTailor has more configurations than the specified maximum, it provides parameters in the response that you can use to retrieve the next pageful. 

        :param self:
        :param NextToken:  Pagination token returned by the GET list request when results exceed the maximum allowed. Use the token to fetch the next page of results.<br/>MaxResults:  Maximum number of records to return. <br/>
        :return: NextToken:  Pagination token returned by the GET list request when results exceed the maximum allowed. Use the token to fetch the next page of results.<br/>Items:  Array of playback configurations. This might be all the available configurations or a subset, depending on the settings that you provide and the total number of configurations stored. <br/>        """
        pass

    def list_tags_for_resource(self, ResourceArn: str) -> ListTagsForResourceResponse:
        """Returns a list of the tags assigned to the specified playback configuration resource. 

        :param self:
        :param ResourceArn:  The Amazon Resource Name (ARN) for the playback configuration. You can get this from the response to any playback configuration request. <br/>
        :return: Tags:  A comma-separated list of tag key:value pairs. For example: 
 {
 "Key1": "Value1",
 "Key2": "Value2"
 }
 <br/>        """
        pass

    def put_playback_configuration(self, AdDecisionServerUrl: str=None, CdnConfiguration: CdnConfiguration=None, DashConfiguration: DashConfigurationForPut=None, Name: str=None, SlateAdUrl: str=None, Tags: __mapOf__string=None, TranscodeProfileName: str=None, VideoContentSourceUrl: str=None) -> PutPlaybackConfigurationResponse:
        """Adds a new playback configuration to AWS Elemental MediaTailor. 

        :param self:
        :param VideoContentSourceUrl:  The URL prefix for the master playlist for the stream, minus the asset ID. The maximum length is 512 characters.<br/>TranscodeProfileName:  The name that is used to associate this playback configuration with a custom transcode profile. This overrides the dynamic transcoding defaults of MediaTailor. Use this only if you have already set up custom profiles with the help of AWS Support.<br/>Tags:  The tags to assign to the playback configuration. <br/>SlateAdUrl:  The URL for a high-quality video asset to transcode and use to fill in time that's not used by ads. AWS Elemental MediaTailor shows the slate to fill in gaps in media content. Configuring the slate is optional for non-VPAID configurations. For VPAID, the slate is required because MediaTailor provides it in the slots that are designated for dynamic ad content. The slate must be a high-quality asset that contains both audio and video. <br/>Name:  The identifier for the playback configuration.<br/>DashConfiguration:  The configuration for DASH content. :<br/>CdnConfiguration:  The configuration for using a content delivery network (CDN), like Amazon CloudFront, for content and ad segment management. :<br/>AdDecisionServerUrl:  The URL for the ad decision server (ADS). This includes the specification of static parameters and placeholders for dynamic parameters. AWS Elemental MediaTailor substitutes player-specific and session-specific parameters as needed when calling the ADS. Alternately, for testing you can provide a static VAST URL. The maximum length is 25,000 characters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContentSegmentUrlPrefix</i> A content delivery network (CDN) to cache content segments, so that content requests don’t always have to go to the origin server. First, create a rule in your CDN for the content segment origin server. Then specify the rule's name in this ContentSegmentUrlPrefix. When AWS Elemental MediaTailor serves a manifest, it reports your CDN as the source for content segments.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdSegmentUrlPrefix</i> A non-default content delivery network (CDN) to serve ad segments. By default, AWS Elemental MediaTailor uses Amazon CloudFront with default cache settings as its CDN for ad segments. To set up an alternate CDN, create a rule in your CDN for the following origin: ads.mediatailor.&lt;region>.amazonaws.com. Then specify the rule's name in this AdSegmentUrlPrefix. When AWS Elemental MediaTailor serves a manifest, it reports your CDN as the source for ad segments.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OriginManifestType</i> The setting that controls whether MediaTailor handles manifests from the origin server as multi-period manifests or single-period manifests. If your origin server produces single-period manifests, set this to SINGLE_PERIOD. The default setting is MULTI_PERIOD. For multi-period manifests, omit this setting or set it to MULTI_PERIOD. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MpdLocation</i> The setting that controls whether MediaTailor includes the Location tag in DASH manifests. MediaTailor populates the Location tag with the URL for manifest update requests, to be used by players that don't support sticky redirects. Disable this if you have CDN routing rules set up for accessing MediaTailor manifests, and you are either using client-side reporting or your players support sticky HTTP redirects. Valid values are DISABLED and EMT_DEFAULT. The EMT_DEFAULT setting enables the inclusion of the tag and is the default value. <br/>
        :return: VideoContentSourceUrl:  The URL prefix for the master playlist for the stream, minus the asset ID. The maximum length is 512 characters.<br/>TranscodeProfileName:  The name that is used to associate this playback configuration with a custom transcode profile. This overrides the dynamic transcoding defaults of MediaTailor. Use this only if you have already set up custom profiles with the help of AWS Support.<br/>Tags:  The tags assigned to the playback configuration. <br/>SlateAdUrl:  The URL for a high-quality video asset to transcode and use to fill in time that's not used by ads. AWS Elemental MediaTailor shows the slate to fill in gaps in media content. Configuring the slate is optional for non-VPAID playback configurations. For VPAID, the slate is required because MediaTailor provides it in the slots designated for dynamic ad content. The slate must be a high-quality asset that contains both audio and video. <br/>SessionInitializationEndpointPrefix:  The URL that the player uses to initialize a session that uses client-side reporting. <br/>PlaybackEndpointPrefix:  The URL that the player accesses to get a manifest from AWS Elemental MediaTailor. This session will use server-side reporting. <br/>PlaybackConfigurationArn:  The Amazon Resource Name (ARN) for the playback configuration. <br/>Name:  The identifier for the playback configuration.<br/>HlsConfiguration:  The configuration for HLS content. :<br/>DashConfiguration:  The configuration for DASH content. :<br/>CdnConfiguration:  The configuration for using a content delivery network (CDN), like Amazon CloudFront, for content and ad segment management. :<br/>AdDecisionServerUrl:  The URL for the ad decision server (ADS). This includes the specification of static parameters and placeholders for dynamic parameters. AWS Elemental MediaTailor substitutes player-specific and session-specific parameters as needed when calling the ADS. Alternately, for testing, you can provide a static VAST URL. The maximum length is 25,000 characters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContentSegmentUrlPrefix</i> A content delivery network (CDN) to cache content segments, so that content requests don’t always have to go to the origin server. First, create a rule in your CDN for the content segment origin server. Then specify the rule's name in this ContentSegmentUrlPrefix. When AWS Elemental MediaTailor serves a manifest, it reports your CDN as the source for content segments.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdSegmentUrlPrefix</i> A non-default content delivery network (CDN) to serve ad segments. By default, AWS Elemental MediaTailor uses Amazon CloudFront with default cache settings as its CDN for ad segments. To set up an alternate CDN, create a rule in your CDN for the following origin: ads.mediatailor.&lt;region>.amazonaws.com. Then specify the rule's name in this AdSegmentUrlPrefix. When AWS Elemental MediaTailor serves a manifest, it reports your CDN as the source for ad segments.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OriginManifestType</i> The setting that controls whether MediaTailor handles manifests from the origin server as multi-period manifests or single-period manifests. If your origin server produces single-period manifests, set this to SINGLE_PERIOD. The default setting is MULTI_PERIOD. For multi-period manifests, omit this setting or set it to MULTI_PERIOD. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MpdLocation</i> The setting that controls whether MediaTailor includes the Location tag in DASH manifests. MediaTailor populates the Location tag with the URL for manifest update requests, to be used by players that don't support sticky redirects. Disable this if you have CDN routing rules set up for accessing MediaTailor manifests, and you are either using client-side reporting or your players support sticky HTTP redirects. Valid values are DISABLED and EMT_DEFAULT. The EMT_DEFAULT setting enables the inclusion of the tag and is the default value. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManifestEndpointPrefix</i> The URL generated by MediaTailor to initiate a playback session. The session uses server-side reporting. This setting is ignored in PUT operations. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManifestEndpointPrefix</i> The URL that is used to initiate a playback session for devices that support Apple HLS. The session uses server-side reporting.<br/>        """
        pass

    def tag_resource(self, Tags: __mapOf__string, ResourceArn: str):
        """Adds tags to the specified playback configuration resource. You can specify one or more tags to add. 

        :param self:
        :param Tags:  A comma-separated list of tag key:value pairs. For example: 
 {
 "Key1": "Value1",
 "Key2": "Value2"
 }
 <br/>ResourceArn:  The Amazon Resource Name (ARN) for the playback configuration. You can get this from the response to any playback configuration request. <br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[__string], ResourceArn: str):
        """Removes tags from the specified playback configuration resource. You can specify one or more tags to remove. 

        :param self:
        :param TagKeys:  A comma-separated list of the tag keys to remove from the playback configuration. <br/>ResourceArn:  The Amazon Resource Name (ARN) for the playback configuration. You can get this from the response to any playback configuration request. <br/>
        :return:         """
        pass



class MarketplaceMetering(BaseClient):
    """<fullname>AWS Marketplace Metering Service</fullname> This reference provides descriptions of the low-level AWS Marketplace Metering Service API. AWS Marketplace sellers can use this API to submit usage data for custom usage dimensions.  <b>Submitting Metering Records</b>  <ul> <li>  <i>MeterUsage</i>- Submits the metering record for a Marketplace product. MeterUsage is called from an EC2 instance. </li> <li>  <i>BatchMeterUsage</i>- Submits the metering record for a set of customers. BatchMeterUsage is called from a software-as-a-service (SaaS) application. </li> </ul>  <b>Accepting New Customers</b>  <ul> <li>  <i>ResolveCustomer</i>- Called by a SaaS application during the registration process. When a buyer visits your website during the registration process, the buyer submits a Registration Token through the browser. The Registration Token is resolved through this API to obtain a CustomerIdentifier and Product Code. </li> </ul>  <b>Entitlement and Metering for Paid Container Products</b>  <ul> <li>  Paid container software products sold through AWS Marketplace must integrate with the AWS Marketplace Metering Service and call the RegisterUsage operation for software entitlement and metering. Calling RegisterUsage from containers running outside of Amazon Elastic Container Service (Amazon ECR) isn't supported. Free and BYOL products for ECS aren't required to call RegisterUsage, but you can do so if you want to receive usage data in your seller reports. For more information on using the RegisterUsage operation, see <a href="https://docs.aws.amazon.com/marketplace/latest/userguide/container-based-products.html">Container-Based Products</a>.  </li> </ul> BatchMeterUsage API calls are captured by AWS CloudTrail. You can use Cloudtrail to verify that the SaaS metering records that you sent are accurate by searching for records with the eventName of BatchMeterUsage. You can also use CloudTrail to audit records over time. For more information, see the <i> <a href="http://docs.aws.amazon.com/awscloudtrail/latest/userguide/cloudtrail-concepts.html">AWS CloudTrail User Guide</a> </i>."""




    class BatchMeterUsageRequest(Mapping):
        pass
    
    class BatchMeterUsageResult(Mapping):
        pass
    
    class UsageRecord(Mapping):
        pass
    
    class MeterUsageRequest(Mapping):
        pass
    
    class MeterUsageResult(Mapping):
        pass
    
    class Timestamp(object):
        pass
    
    class Boolean(object):
        pass
    
    class RegisterUsageRequest(Mapping):
        pass
    
    class RegisterUsageResult(Mapping):
        pass
    
    class ResolveCustomerRequest(Mapping):
        pass
    
    class ResolveCustomerResult(Mapping):
        pass
    
    def batch_meter_usage(self, ProductCode: str, UsageRecords: List[UsageRecord]) -> BatchMeterUsageResult:
        """BatchMeterUsage is called from a SaaS application listed on the AWS Marketplace to post metering records for a set of customers. For identical requests, the API is idempotent; requests can be retried with the same records or a subset of the input records. Every request to BatchMeterUsage is for one product. If you need to meter usage for multiple products, you must make multiple calls to BatchMeterUsage. BatchMeterUsage can process up to 25 UsageRecords at a time.

        :param self:
        :param ProductCode:  Product code is used to uniquely identify a product in AWS Marketplace. The product code should be the same as the one used during the publishing of a new product.<br/>UsageRecords:  The set of UsageRecords to submit. BatchMeterUsage accepts up to 25 UsageRecords at a time.<br/>
        :return: UnprocessedRecords:  Contains all UsageRecords that were not processed by BatchMeterUsage. This is a list of UsageRecords. You can retry the failed request by making another BatchMeterUsage call with this list as input in the BatchMeterUsageRequest.<br/>Results:  Contains all UsageRecords processed by BatchMeterUsage. These records were either honored by AWS Marketplace Metering Service or were invalid.<br/>        """
        pass

    def meter_usage(self, UsageDimension: str, Timestamp: Timestamp, ProductCode: str, UsageQuantity: int=None, DryRun: Boolean=None) -> MeterUsageResult:
        """API to emit metering records. For identical requests, the API is idempotent. It simply returns the metering record ID. MeterUsage is authenticated on the buyer's AWS account, generally when running from an EC2 instance on the AWS Marketplace.

        :param self:
        :param DryRun:  Checks whether you have the permissions required for the action, but does not make the request. If you have the permissions, the request returns DryRunOperation; otherwise, it returns UnauthorizedException. Defaults to <code>false</code> if not specified.<br/>UsageQuantity:  Consumption value for the hour. Defaults to <code>0</code> if not specified.<br/>UsageDimension:  It will be one of the fcp dimension name provided during the publishing of the product.<br/>Timestamp:  Timestamp, in UTC, for which the usage is being reported. Your application can meter usage for up to one hour in the past. Make sure the timestamp value is not before the start of the software usage.<br/>ProductCode:  Product code is used to uniquely identify a product in AWS Marketplace. The product code should be the same as the one used during the publishing of a new product.<br/>
        :return: MeteringRecordId:  Metering record id.<br/>        """
        pass

    def register_usage(self, PublicKeyVersion: int, ProductCode: str, Nonce: str=None) -> RegisterUsageResult:
        """Paid container software products sold through AWS Marketplace must integrate with the AWS Marketplace Metering Service and call the RegisterUsage operation for software entitlement and metering. Calling RegisterUsage from containers running outside of ECS is not currently supported. Free and BYOL products for ECS aren't required to call RegisterUsage, but you may choose to do so if you would like to receive usage data in your seller reports. The sections below explain the behavior of RegisterUsage. RegisterUsage performs two primary functions: metering and entitlement. <ul> <li>  <i>Entitlement</i>: RegisterUsage allows you to verify that the customer running your paid software is subscribed to your product on AWS Marketplace, enabling you to guard against unauthorized use. Your container image that integrates with RegisterUsage is only required to guard against unauthorized use at container startup, as such a CustomerNotSubscribedException/PlatformNotSupportedException will only be thrown on the initial call to RegisterUsage. Subsequent calls from the same Amazon ECS task instance (e.g. task-id) will not throw a CustomerNotSubscribedException, even if the customer unsubscribes while the Amazon ECS task is still running. </li> <li>  <i>Metering</i>: RegisterUsage meters software use per ECS task, per hour, with usage prorated to the second. A minimum of 1 minute of usage applies to tasks that are short lived. For example, if a customer has a 10 node ECS cluster and creates an ECS service configured as a Daemon Set, then ECS will launch a task on all 10 cluster nodes and the customer will be charged: (10 * hourly_rate). Metering for software use is automatically handled by the AWS Marketplace Metering Control Plane -- your software is not required to perform any metering specific actions, other than call RegisterUsage once for metering of software use to commence. The AWS Marketplace Metering Control Plane will also continue to bill customers for running ECS tasks, regardless of the customers subscription state, removing the need for your software to perform entitlement checks at runtime. </li> </ul>

        :param self:
        :param Nonce:  (Optional) To scope down the registration to a specific running software instance and guard against replay attacks.<br/>PublicKeyVersion:  Public Key Version provided by AWS Marketplace<br/>ProductCode:  Product code is used to uniquely identify a product in AWS Marketplace. The product code should be the same as the one used during the publishing of a new product.<br/>
        :return: Signature:  JWT Token<br/>PublicKeyRotationTimestamp:  (Optional) Only included when public key version has expired<br/>        """
        pass

    def resolve_customer(self, RegistrationToken: str) -> ResolveCustomerResult:
        """ResolveCustomer is called by a SaaS application during the registration process. When a buyer visits your website during the registration process, the buyer submits a registration token through their browser. The registration token is resolved through this API to obtain a CustomerIdentifier and product code.

        :param self:
        :param RegistrationToken:  When a buyer visits your website during the registration process, the buyer submits a registration token through the browser. The registration token is resolved to obtain a CustomerIdentifier and product code.<br/>
        :return: ProductCode:  The product code is returned to confirm that the buyer is registering for your product. Subsequent BatchMeterUsage calls should be made using this product code.<br/>CustomerIdentifier:  The CustomerIdentifier is used to identify an individual customer in your application. Calls to BatchMeterUsage require CustomerIdentifiers for each UsageRecord.<br/>        """
        pass



class MigrationHub(BaseClient):
    """The AWS Migration Hub API methods help to obtain server and application migration status and integrate your resource-specific migration tool by providing a programmatic interface to Migration Hub. """


    class ListCreatedArtifactsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_created_artifacts` for valid parameters"""
            pass
    class ListDiscoveredResourcesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_discovered_resources` for valid parameters"""
            pass
    class ListMigrationTasksPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_migration_tasks` for valid parameters"""
            pass
    class ListProgressUpdateStreamsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_progress_update_streams` for valid parameters"""
            pass


    class AssociateCreatedArtifactRequest(Mapping):
        pass
    
    class AssociateCreatedArtifactResult(Mapping):
        pass
    
    class CreatedArtifact(Mapping):
        pass
    
    class DryRun(object):
        pass
    
    class AssociateDiscoveredResourceRequest(Mapping):
        pass
    
    class AssociateDiscoveredResourceResult(Mapping):
        pass
    
    class DiscoveredResource(Mapping):
        pass
    
    class CreateProgressUpdateStreamRequest(Mapping):
        pass
    
    class CreateProgressUpdateStreamResult(Mapping):
        pass
    
    class DeleteProgressUpdateStreamRequest(Mapping):
        pass
    
    class DeleteProgressUpdateStreamResult(Mapping):
        pass
    
    class DescribeApplicationStateRequest(Mapping):
        pass
    
    class DescribeApplicationStateResult(Mapping):
        pass
    
    class DescribeMigrationTaskRequest(Mapping):
        pass
    
    class DescribeMigrationTaskResult(Mapping):
        pass
    
    class DisassociateCreatedArtifactRequest(Mapping):
        pass
    
    class DisassociateCreatedArtifactResult(Mapping):
        pass
    
    class DisassociateDiscoveredResourceRequest(Mapping):
        pass
    
    class DisassociateDiscoveredResourceResult(Mapping):
        pass
    
    class ImportMigrationTaskRequest(Mapping):
        pass
    
    class ImportMigrationTaskResult(Mapping):
        pass
    
    class ListCreatedArtifactsRequest(Mapping):
        pass
    
    class ListCreatedArtifactsResult(Mapping):
        pass
    
    class ListDiscoveredResourcesRequest(Mapping):
        pass
    
    class ListDiscoveredResourcesResult(Mapping):
        pass
    
    class ListMigrationTasksRequest(Mapping):
        pass
    
    class ListMigrationTasksResult(Mapping):
        pass
    
    class ListProgressUpdateStreamsRequest(Mapping):
        pass
    
    class ListProgressUpdateStreamsResult(Mapping):
        pass
    
    class NotifyApplicationStateRequest(Mapping):
        pass
    
    class NotifyApplicationStateResult(Mapping):
        pass
    
    class NotifyMigrationTaskStateRequest(Mapping):
        pass
    
    class NotifyMigrationTaskStateResult(Mapping):
        pass
    
    class Task(Mapping):
        pass
    
    class UpdateDateTime(object):
        pass
    
    class PutResourceAttributesRequest(Mapping):
        pass
    
    class PutResourceAttributesResult(Mapping):
        pass
    
    class ResourceAttribute(Mapping):
        pass
    
    def associate_created_artifact(self, CreatedArtifact: CreatedArtifact, MigrationTaskName: str, ProgressUpdateStream: str, DryRun: DryRun=None) -> AssociateCreatedArtifactResult:
        """Associates a created artifact of an AWS cloud resource, the target receiving the migration, with the migration task performed by a migration tool. This API has the following traits: <ul> <li> Migration tools can call the <code>AssociateCreatedArtifact</code> operation to indicate which AWS artifact is associated with a migration task. </li> <li> The created artifact name must be provided in ARN (Amazon Resource Name) format which will contain information about type and region; for example: <code>arn:aws:ec2:us-east-1:488216288981:image/ami-6d0ba87b</code>. </li> <li> Examples of the AWS resource behind the created artifact are, AMI's, EC2 instance, or DMS endpoint, etc. </li> </ul>

        :param self:
        :param DryRun:  Optional boolean flag to indicate whether any effect should take place. Used to test if the caller has permission to make the call.<br/>CreatedArtifact:  An ARN of the AWS resource related to the migration (e.g., AMI, EC2 instance, RDS instance, etc.) :<br/>MigrationTaskName:  Unique identifier that references the migration task.<br/>ProgressUpdateStream:  The name of the ProgressUpdateStream. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description that can be free-form text to record additional detail about the artifact for clarity or for later reference.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> An ARN that uniquely identifies the result of a migration task.<br/>
        :return:         """
        pass

    def associate_discovered_resource(self, DiscoveredResource: DiscoveredResource, MigrationTaskName: str, ProgressUpdateStream: str, DryRun: DryRun=None) -> AssociateDiscoveredResourceResult:
        """Associates a discovered resource ID from Application Discovery Service (ADS) with a migration task.

        :param self:
        :param DryRun:  Optional boolean flag to indicate whether any effect should take place. Used to test if the caller has permission to make the call.<br/>DiscoveredResource:  Object representing a Resource.:<br/>MigrationTaskName:  The identifier given to the MigrationTask.<br/>ProgressUpdateStream:  The name of the ProgressUpdateStream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description that can be free-form text to record additional detail about the discovered resource for clarity or later reference.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigurationId</i> The configurationId in ADS that uniquely identifies the on-premise resource.<br/>
        :return:         """
        pass

    def create_progress_update_stream(self, ProgressUpdateStreamName: str, DryRun: DryRun=None) -> CreateProgressUpdateStreamResult:
        """Creates a progress update stream which is an AWS resource used for access control as well as a namespace for migration task names that is implicitly linked to your AWS account. It must uniquely identify the migration tool as it is used for all updates made by the tool; however, it does not need to be unique for each AWS account because it is scoped to the AWS account.

        :param self:
        :param DryRun:  Optional boolean flag to indicate whether any effect should take place. Used to test if the caller has permission to make the call.<br/>ProgressUpdateStreamName:  The name of the ProgressUpdateStream. <br/>
        :return:         """
        pass

    def delete_progress_update_stream(self, ProgressUpdateStreamName: str, DryRun: DryRun=None) -> DeleteProgressUpdateStreamResult:
        """Deletes a progress update stream, including all of its tasks, which was previously created as an AWS resource used for access control. This API has the following traits: <ul> <li> The only parameter needed for <code>DeleteProgressUpdateStream</code> is the stream name (same as a <code>CreateProgressUpdateStream</code> call). </li> <li> The call will return, and a background process will asynchronously delete the stream and all of its resources (tasks, associated resources, resource attributes, created artifacts). </li> <li> If the stream takes time to be deleted, it might still show up on a <code>ListProgressUpdateStreams</code> call. </li> <li>  <code>CreateProgressUpdateStream</code>, <code>ImportMigrationTask</code>, <code>NotifyMigrationTaskState</code>, and all Associate[*] APIs realted to the tasks belonging to the stream will throw "InvalidInputException" if the stream of the same name is in the process of being deleted. </li> <li> Once the stream and all of its resources are deleted, <code>CreateProgressUpdateStream</code> for a stream of the same name will succeed, and that stream will be an entirely new logical resource (without any resources associated with the old stream). </li> </ul>

        :param self:
        :param DryRun:  Optional boolean flag to indicate whether any effect should take place. Used to test if the caller has permission to make the call.<br/>ProgressUpdateStreamName:  The name of the ProgressUpdateStream. <br/>
        :return:         """
        pass

    def describe_application_state(self, ApplicationId: str) -> DescribeApplicationStateResult:
        """Gets the migration status of an application.

        :param self:
        :param ApplicationId:  The configurationId in ADS that uniquely identifies the grouped application.<br/>
        :return: LastUpdatedTime:  The timestamp when the application status was last updated.<br/>ApplicationStatus:  Status of the application - Not Started, In-Progress, Complete.<br/>        """
        pass

    def describe_migration_task(self, MigrationTaskName: str, ProgressUpdateStream: str) -> DescribeMigrationTaskResult:
        """Retrieves a list of all attributes associated with a specific migration task.

        :param self:
        :param MigrationTaskName:  The identifier given to the MigrationTask.<br/>ProgressUpdateStream:  The name of the ProgressUpdateStream. <br/>
        :return: MigrationTask:  Object encapsulating information about the migration task.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAttributeList</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateDateTime</i> The timestamp when the task was gathered.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Task</i> Task object encapsulating task information.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Task</b>: Task object encapsulating task information.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MigrationTaskName</i> Unique identifier that references the migration task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProgressUpdateStream</i> A name that identifies the vendor of the migration tool being used.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ProgressPercent</i> Indication of the percentage completion of the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusDetail</i> Details of task status as notified by a migration tool. A tool might use this field to provide clarifying information about the status that is unique to that tool or that explains an error state.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Status of the task - Not Started, In-Progress, Complete.<br/>        """
        pass

    def disassociate_created_artifact(self, CreatedArtifactName: str, MigrationTaskName: str, ProgressUpdateStream: str, DryRun: DryRun=None) -> DisassociateCreatedArtifactResult:
        """Disassociates a created artifact of an AWS resource with a migration task performed by a migration tool that was previously associated. This API has the following traits: <ul> <li> A migration user can call the <code>DisassociateCreatedArtifacts</code> operation to disassociate a created AWS Artifact from a migration task. </li> <li> The created artifact name must be provided in ARN (Amazon Resource Name) format which will contain information about type and region; for example: <code>arn:aws:ec2:us-east-1:488216288981:image/ami-6d0ba87b</code>. </li> <li> Examples of the AWS resource behind the created artifact are, AMI's, EC2 instance, or RDS instance, etc. </li> </ul>

        :param self:
        :param DryRun:  Optional boolean flag to indicate whether any effect should take place. Used to test if the caller has permission to make the call.<br/>CreatedArtifactName:  An ARN of the AWS resource related to the migration (e.g., AMI, EC2 instance, RDS instance, etc.)<br/>MigrationTaskName:  Unique identifier that references the migration task to be disassociated with the artifact.<br/>ProgressUpdateStream:  The name of the ProgressUpdateStream. <br/>
        :return:         """
        pass

    def disassociate_discovered_resource(self, ConfigurationId: str, MigrationTaskName: str, ProgressUpdateStream: str, DryRun: DryRun=None) -> DisassociateDiscoveredResourceResult:
        """Disassociate an Application Discovery Service (ADS) discovered resource from a migration task.

        :param self:
        :param DryRun:  Optional boolean flag to indicate whether any effect should take place. Used to test if the caller has permission to make the call.<br/>ConfigurationId:  ConfigurationId of the ADS resource to be disassociated.<br/>MigrationTaskName:  The identifier given to the MigrationTask.<br/>ProgressUpdateStream:  The name of the ProgressUpdateStream.<br/>
        :return:         """
        pass

    def import_migration_task(self, MigrationTaskName: str, ProgressUpdateStream: str, DryRun: DryRun=None) -> ImportMigrationTaskResult:
        """Registers a new migration task which represents a server, database, etc., being migrated to AWS by a migration tool. This API is a prerequisite to calling the <code>NotifyMigrationTaskState</code> API as the migration tool must first register the migration task with Migration Hub.

        :param self:
        :param DryRun:  Optional boolean flag to indicate whether any effect should take place. Used to test if the caller has permission to make the call.<br/>MigrationTaskName:  Unique identifier that references the migration task.<br/>ProgressUpdateStream:  The name of the ProgressUpdateStream. <br/>
        :return:         """
        pass

    def list_created_artifacts(self, MigrationTaskName: str, ProgressUpdateStream: str, NextToken: str=None, MaxResults: int=None) -> ListCreatedArtifactsResult:
        """Lists the created artifacts attached to a given migration task in an update stream. This API has the following traits: <ul> <li> Gets the list of the created artifacts while migration is taking place. </li> <li> Shows the artifacts created by the migration tool that was associated by the <code>AssociateCreatedArtifact</code> API.  </li> <li> Lists created artifacts in a paginated interface.  </li> </ul>

        :param self:
        :param MaxResults:  Maximum number of results to be returned per page.<br/>NextToken:  If a <code>NextToken</code> was returned by a previous call, there are more results available. To retrieve the next page of results, make the call again using the returned token in <code>NextToken</code>.<br/>MigrationTaskName:  Unique identifier that references the migration task.<br/>ProgressUpdateStream:  The name of the ProgressUpdateStream. <br/>
        :return: CreatedArtifactList:  List of created artifacts up to the maximum number of results specified in the request.<br/>NextToken:  If there are more created artifacts than the max result, return the next token to be passed to the next call as a bookmark of where to start from.<br/>        """
        pass

    def list_discovered_resources(self, MigrationTaskName: str, ProgressUpdateStream: str, NextToken: str=None, MaxResults: int=None) -> ListDiscoveredResourcesResult:
        """Lists discovered resources associated with the given <code>MigrationTask</code>.

        :param self:
        :param MaxResults:  The maximum number of results returned per page.<br/>NextToken:  If a <code>NextToken</code> was returned by a previous call, there are more results available. To retrieve the next page of results, make the call again using the returned token in <code>NextToken</code>.<br/>MigrationTaskName:  The name of the MigrationTask.<br/>ProgressUpdateStream:  The name of the ProgressUpdateStream.<br/>
        :return: DiscoveredResourceList:  Returned list of discovered resources associated with the given MigrationTask.<br/>NextToken:  If there are more discovered resources than the max result, return the next token to be passed to the next call as a bookmark of where to start from.<br/>        """
        pass

    def list_migration_tasks(self, NextToken: str=None, MaxResults: int=None, ResourceName: str=None) -> ListMigrationTasksResult:
        """Lists all, or filtered by resource name, migration tasks associated with the user account making this call. This API has the following traits: <ul> <li> Can show a summary list of the most recent migration tasks. </li> <li> Can show a summary list of migration tasks associated with a given discovered resource. </li> <li> Lists migration tasks in a paginated interface. </li> </ul>

        :param self:
        :param ResourceName:  Filter migration tasks by discovered resource name.<br/>MaxResults:  Value to specify how many results are returned per page.<br/>NextToken:  If a <code>NextToken</code> was returned by a previous call, there are more results available. To retrieve the next page of results, make the call again using the returned token in <code>NextToken</code>.<br/>
        :return: MigrationTaskSummaryList:  Lists the migration task's summary which includes: <code>MigrationTaskName</code>, <code>ProgressPercent</code>, <code>ProgressUpdateStream</code>, <code>Status</code>, and the <code>UpdateDateTime</code> for each task.<br/>NextToken:  If there are more migration tasks than the max result, return the next token to be passed to the next call as a bookmark of where to start from.<br/>        """
        pass

    def list_progress_update_streams(self, NextToken: str=None, MaxResults: int=None) -> ListProgressUpdateStreamsResult:
        """Lists progress update streams associated with the user account making this call.

        :param self:
        :param MaxResults:  Filter to limit the maximum number of results to list per page.<br/>NextToken:  If a <code>NextToken</code> was returned by a previous call, there are more results available. To retrieve the next page of results, make the call again using the returned token in <code>NextToken</code>.<br/>
        :return: NextToken:  If there are more streams created than the max result, return the next token to be passed to the next call as a bookmark of where to start from.<br/>ProgressUpdateStreamSummaryList:  List of progress update streams up to the max number of results passed in the input.<br/>        """
        pass

    def notify_application_state(self, Status: str, ApplicationId: str, DryRun: DryRun=None) -> NotifyApplicationStateResult:
        """Sets the migration state of an application. For a given application identified by the value passed to <code>ApplicationId</code>, its status is set or updated by passing one of three values to <code>Status</code>: <code>NOT_STARTED | IN_PROGRESS | COMPLETED</code>.

        :param self:
        :param DryRun:  Optional boolean flag to indicate whether any effect should take place. Used to test if the caller has permission to make the call.<br/>Status:  Status of the application - Not Started, In-Progress, Complete.<br/>ApplicationId:  The configurationId in ADS that uniquely identifies the grouped application.<br/>
        :return:         """
        pass

    def notify_migration_task_state(self, NextUpdateSeconds: int, UpdateDateTime: UpdateDateTime, Task: Task, MigrationTaskName: str, ProgressUpdateStream: str, DryRun: DryRun=None) -> NotifyMigrationTaskStateResult:
        """Notifies Migration Hub of the current status, progress, or other detail regarding a migration task. This API has the following traits: <ul> <li> Migration tools will call the <code>NotifyMigrationTaskState</code> API to share the latest progress and status. </li> <li>  <code>MigrationTaskName</code> is used for addressing updates to the correct target. </li> <li>  <code>ProgressUpdateStream</code> is used for access control and to provide a namespace for each migration tool. </li> </ul>

        :param self:
        :param DryRun:  Optional boolean flag to indicate whether any effect should take place. Used to test if the caller has permission to make the call.<br/>NextUpdateSeconds:  Number of seconds after the UpdateDateTime within which the Migration Hub can expect an update. If Migration Hub does not receive an update within the specified interval, then the migration task will be considered stale.<br/>UpdateDateTime:  The timestamp when the task was gathered.<br/>Task:  Information about the task's progress and status.:<br/>MigrationTaskName:  Unique identifier that references the migration task.<br/>ProgressUpdateStream:  The name of the ProgressUpdateStream. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProgressPercent</i> Indication of the percentage completion of the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusDetail</i> Details of task status as notified by a migration tool. A tool might use this field to provide clarifying information about the status that is unique to that tool or that explains an error state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Status of the task - Not Started, In-Progress, Complete.<br/>
        :return:         """
        pass

    def put_resource_attributes(self, ResourceAttributeList: List[ResourceAttribute], MigrationTaskName: str, ProgressUpdateStream: str, DryRun: DryRun=None) -> PutResourceAttributesResult:
        """Provides identifying details of the resource being migrated so that it can be associated in the Application Discovery Service (ADS)'s repository. This association occurs asynchronously after <code>PutResourceAttributes</code> returns. <important> <ul> <li> Keep in mind that subsequent calls to PutResourceAttributes will override previously stored attributes. For example, if it is first called with a MAC address, but later, it is desired to <i>add</i> an IP address, it will then be required to call it with <i>both</i> the IP and MAC addresses to prevent overiding the MAC address. </li> <li> Note the instructions regarding the special use case of the <a href="https://docs.aws.amazon.com/migrationhub/latest/ug/API_PutResourceAttributes.html#migrationhub-PutResourceAttributes-request-ResourceAttributeList"> <code>ResourceAttributeList</code> </a> parameter when specifying any "VM" related value.  </li> </ul> </important> <note> Because this is an asynchronous call, it will always return 200, whether an association occurs or not. To confirm if an association was found based on the provided details, call <code>ListDiscoveredResources</code>. </note>

        :param self:
        :param DryRun:  Optional boolean flag to indicate whether any effect should take place. Used to test if the caller has permission to make the call.<br/>ResourceAttributeList:  Information about the resource that is being migrated. This data will be used to map the task to a resource in the Application Discovery Service (ADS)'s repository. <note> Takes the object array of <code>ResourceAttribute</code> where the <code>Type</code> field is reserved for the following values: <code>IPV4_ADDRESS | IPV6_ADDRESS | MAC_ADDRESS | FQDN | VM_MANAGER_ID | VM_MANAGED_OBJECT_REFERENCE | VM_NAME | VM_PATH | BIOS_ID | MOTHERBOARD_SERIAL_NUMBER</code> where the identifying value can be a string up to 256 characters. </note> <important> <ul> <li> If any "VM" related value is set for a <code>ResourceAttribute</code> object, it is required that <code>VM_MANAGER_ID</code>, as a minimum, is always set. If <code>VM_MANAGER_ID</code> is not set, then all "VM" fields will be discarded and "VM" fields will not be used for matching the migration task to a server in Application Discovery Service (ADS)'s repository. See the <a href="https://docs.aws.amazon.com/migrationhub/latest/ug/API_PutResourceAttributes.html#API_PutResourceAttributes_Examples">Example</a> section below for a use case of specifying "VM" related values. </li> <li>  If a server you are trying to match has multiple IP or MAC addresses, you should provide as many as you know in separate type/value pairs passed to the <code>ResourceAttributeList</code> parameter to maximize the chances of matching. </li> </ul> </important><br/>MigrationTaskName:  Unique identifier that references the migration task.<br/>ProgressUpdateStream:  The name of the ProgressUpdateStream. <br/>
        :return:         """
        pass



class Mobile(BaseClient):
    """ AWS Mobile Service provides mobile app and website developers with capabilities required to configure AWS resources and bootstrap their developer desktop projects with the necessary SDKs, constants, tools and samples to make use of those resources. """


    class ListBundlesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_bundles` for valid parameters"""
            pass
    class ListProjectsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_projects` for valid parameters"""
            pass


    class CreateProjectRequest(Mapping):
        pass
    
    class CreateProjectResult(Mapping):
        pass
    
    class Contents(object):
        pass
    
    class DeleteProjectRequest(Mapping):
        pass
    
    class DeleteProjectResult(Mapping):
        pass
    
    class DescribeBundleRequest(Mapping):
        pass
    
    class DescribeBundleResult(Mapping):
        pass
    
    class DescribeProjectRequest(Mapping):
        pass
    
    class DescribeProjectResult(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class ExportBundleRequest(Mapping):
        pass
    
    class ExportBundleResult(Mapping):
        pass
    
    class ExportProjectRequest(Mapping):
        pass
    
    class ExportProjectResult(Mapping):
        pass
    
    class ListBundlesRequest(Mapping):
        pass
    
    class ListBundlesResult(Mapping):
        pass
    
    class ListProjectsRequest(Mapping):
        pass
    
    class ListProjectsResult(Mapping):
        pass
    
    class UpdateProjectRequest(Mapping):
        pass
    
    class UpdateProjectResult(Mapping):
        pass
    
    def create_project(self, name: str=None, region: str=None, contents: Contents=None, snapshotId: str=None) -> CreateProjectResult:
        """ Creates an AWS Mobile Hub project. 

        :param self:
        :param snapshotId:   Unique identifier for an exported snapshot of project configuration. This snapshot identifier is included in the share URL when a project is exported. <br/>contents:   ZIP or YAML file which contains configuration settings to be used when creating the project. This may be the contents of the file downloaded from the URL provided in an export project operation. <br/>region:   Default region where project resources should be created. <br/>name:   Name of the project. <br/>
        :return: details:   Detailed information about the created AWS Mobile Hub project. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resources</i>  List of AWS resources associated with a project. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>consoleUrl</i>  Website URL for this project in the AWS Mobile Hub console. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedDate</i>  Date of the last modification of the project. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdDate</i>  Date the project was created. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>state</i>  Synchronization state for a project. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>region</i>  Default region to use for AWS resource creation in the AWS Mobile Hub project. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>projectId</i>  Unique project identifier. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i>  Name of the project. <br/>        """
        pass

    def delete_project(self, projectId: str) -> DeleteProjectResult:
        """ Delets a project in AWS Mobile Hub. 

        :param self:
        :param projectId:   Unique project identifier. <br/>
        :return: orphanedResources:   Resources which were not deleted, due to a risk of losing potentially important data or files. <br/>deletedResources:   Resources which were deleted. <br/>        """
        pass

    def describe_bundle(self, bundleId: str) -> DescribeBundleResult:
        """ Get the bundle details for the requested bundle id. 

        :param self:
        :param bundleId:   Unique bundle identifier. <br/>
        :return: details:   The details of the bundle. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>availablePlatforms</i>  Developer desktop or mobile app or website platforms. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>iconUrl</i>  Icon for the download bundle. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i>  Description of the download bundle. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i>  Version of the download bundle. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>title</i>  Title of the download bundle. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>bundleId</i>  Unique bundle identifier. <br/>        """
        pass

    def describe_project(self, projectId: str, syncFromResources: Boolean=None) -> DescribeProjectResult:
        """ Gets details about a project in AWS Mobile Hub. 

        :param self:
        :param syncFromResources:   If set to true, causes AWS Mobile Hub to synchronize information from other services, e.g., update state of AWS CloudFormation stacks in the AWS Mobile Hub project. <br/>projectId:   Unique project identifier. <br/>
        :return: details:   Detailed information about an AWS Mobile Hub project. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resources</i>  List of AWS resources associated with a project. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>consoleUrl</i>  Website URL for this project in the AWS Mobile Hub console. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedDate</i>  Date of the last modification of the project. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdDate</i>  Date the project was created. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>state</i>  Synchronization state for a project. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>region</i>  Default region to use for AWS resource creation in the AWS Mobile Hub project. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>projectId</i>  Unique project identifier. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i>  Name of the project. <br/>        """
        pass

    def export_bundle(self, bundleId: str, projectId: str=None, platform: str=None) -> ExportBundleResult:
        """ Generates customized software development kit (SDK) and or tool packages used to integrate mobile web or mobile app clients with backend AWS resources. 

        :param self:
        :param platform:   Developer desktop or target application platform. <br/>projectId:   Unique project identifier. <br/>bundleId:   Unique bundle identifier. <br/>
        :return: downloadUrl:   URL which contains the custom-generated SDK and tool packages used to integrate the client mobile app or web app with the AWS resources created by the AWS Mobile Hub project. <br/>        """
        pass

    def export_project(self, projectId: str) -> ExportProjectResult:
        """ Exports project configuration to a snapshot which can be downloaded and shared. Note that mobile app push credentials are encrypted in exported projects, so they can only be shared successfully within the same AWS account. 

        :param self:
        :param projectId:   Unique project identifier. <br/>
        :return: snapshotId:   Unique identifier for the exported snapshot of the project configuration. This snapshot identifier is included in the share URL. <br/>shareUrl:   URL which can be shared to allow other AWS users to create their own project in AWS Mobile Hub with the same configuration as the specified project. This URL pertains to a snapshot in time of the project configuration that is created when this API is called. If you want to share additional changes to your project configuration, then you will need to create and share a new snapshot by calling this method again. <br/>downloadUrl:   URL which can be used to download the exported project configuation file(s). <br/>        """
        pass

    def list_bundles(self, maxResults: int=None, nextToken: str=None) -> ListBundlesResult:
        """ List all available bundles. 

        :param self:
        :param nextToken:   Pagination token. Set to null to start listing bundles from start. If non-null pagination token is returned in a result, then pass its value in here in another request to list more bundles. <br/>maxResults:   Maximum number of records to list in a single response. <br/>
        :return: nextToken:   Pagination token. If non-null pagination token is returned in a result, then pass its value in another request to fetch more entries. <br/>bundleList:   A list of bundles. <br/>        """
        pass

    def list_projects(self, maxResults: int=None, nextToken: str=None) -> ListProjectsResult:
        """ Lists projects in AWS Mobile Hub. 

        :param self:
        :param nextToken:   Pagination token. Set to null to start listing projects from start. If non-null pagination token is returned in a result, then pass its value in here in another request to list more projects. <br/>maxResults:   Maximum number of records to list in a single response. <br/>
        :return: nextToken:   Pagination token. Set to null to start listing records from start. If non-null pagination token is returned in a result, then pass its value in here in another request to list more entries. <br/>projects:   List of projects. <br/>        """
        pass

    def update_project(self, projectId: str, contents: Contents=None) -> UpdateProjectResult:
        """ Update an existing project. 

        :param self:
        :param projectId:   Unique project identifier. <br/>contents:   ZIP or YAML file which contains project configuration to be updated. This should be the contents of the file downloaded from the URL provided in an export project operation. <br/>
        :return: details:   Detailed information about the updated AWS Mobile Hub project. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resources</i>  List of AWS resources associated with a project. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>consoleUrl</i>  Website URL for this project in the AWS Mobile Hub console. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedDate</i>  Date of the last modification of the project. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>createdDate</i>  Date the project was created. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>state</i>  Synchronization state for a project. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>region</i>  Default region to use for AWS resource creation in the AWS Mobile Hub project. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>projectId</i>  Unique project identifier. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i>  Name of the project. <br/>        """
        pass



class MQ(BaseClient):
    """Amazon MQ is a managed message broker service for Apache ActiveMQ that makes it easy to set up and operate message brokers in the cloud. A message broker allows software applications and components to communicate using various programming languages, operating systems, and formal messaging protocols."""


    class ListBrokersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_brokers` for valid parameters"""
            pass


    class CreateBrokerRequest(Mapping):
        pass
    
    class CreateBrokerResponse(Mapping):
        pass
    
    class __boolean(object):
        pass
    
    class ConfigurationId(Mapping):
        pass
    
    class Logs(Mapping):
        pass
    
    class WeeklyStartTime(Mapping):
        pass
    
    class __string(object):
        pass
    
    class __mapOf__string(object):
        pass
    
    class User(Mapping):
        pass
    
    class CreateConfigurationRequest(Mapping):
        pass
    
    class CreateConfigurationResponse(Mapping):
        pass
    
    class CreateTagsRequest(Mapping):
        pass
    
    class CreateUserRequest(Mapping):
        pass
    
    class CreateUserResponse(Mapping):
        pass
    
    class DeleteBrokerRequest(Mapping):
        pass
    
    class DeleteBrokerResponse(Mapping):
        pass
    
    class DeleteTagsRequest(Mapping):
        pass
    
    class DeleteUserRequest(Mapping):
        pass
    
    class DeleteUserResponse(Mapping):
        pass
    
    class DescribeBrokerRequest(Mapping):
        pass
    
    class DescribeBrokerResponse(Mapping):
        pass
    
    class DescribeBrokerEngineTypesRequest(Mapping):
        pass
    
    class DescribeBrokerEngineTypesResponse(Mapping):
        pass
    
    class DescribeBrokerInstanceOptionsRequest(Mapping):
        pass
    
    class DescribeBrokerInstanceOptionsResponse(Mapping):
        pass
    
    class DescribeConfigurationRequest(Mapping):
        pass
    
    class DescribeConfigurationResponse(Mapping):
        pass
    
    class DescribeConfigurationRevisionRequest(Mapping):
        pass
    
    class DescribeConfigurationRevisionResponse(Mapping):
        pass
    
    class DescribeUserRequest(Mapping):
        pass
    
    class DescribeUserResponse(Mapping):
        pass
    
    class ListBrokersRequest(Mapping):
        pass
    
    class ListBrokersResponse(Mapping):
        pass
    
    class ListConfigurationRevisionsRequest(Mapping):
        pass
    
    class ListConfigurationRevisionsResponse(Mapping):
        pass
    
    class ListConfigurationsRequest(Mapping):
        pass
    
    class ListConfigurationsResponse(Mapping):
        pass
    
    class ListTagsRequest(Mapping):
        pass
    
    class ListTagsResponse(Mapping):
        pass
    
    class ListUsersRequest(Mapping):
        pass
    
    class ListUsersResponse(Mapping):
        pass
    
    class RebootBrokerRequest(Mapping):
        pass
    
    class RebootBrokerResponse(Mapping):
        pass
    
    class UpdateBrokerRequest(Mapping):
        pass
    
    class UpdateBrokerResponse(Mapping):
        pass
    
    class UpdateConfigurationRequest(Mapping):
        pass
    
    class UpdateConfigurationResponse(Mapping):
        pass
    
    class UpdateUserRequest(Mapping):
        pass
    
    class UpdateUserResponse(Mapping):
        pass
    
    def create_broker(self, AutoMinorVersionUpgrade: __boolean=None, BrokerName: str=None, Configuration: ConfigurationId=None, CreatorRequestId: str=None, DeploymentMode: str=None, EngineType: str=None, EngineVersion: str=None, HostInstanceType: str=None, Logs: Logs=None, MaintenanceWindowStartTime: WeeklyStartTime=None, PubliclyAccessible: __boolean=None, SecurityGroups: List[__string]=None, SubnetIds: List[__string]=None, Tags: __mapOf__string=None, Users: List[User]=None) -> CreateBrokerResponse:
        """Creates a broker. Note: This API is asynchronous.

        :param self:
        :param Users:  Required. The list of ActiveMQ users (persons or applications) who can access queues and topics. This value can contain only alphanumeric characters, dashes, periods, underscores, and tildes (- . _ ~). This value must be 2-100 characters long.<br/>Tags:  Create tags when creating the broker.<br/>SubnetIds:  The list of groups (2 maximum) that define which subnets and IP ranges the broker can use from different Availability Zones. A SINGLE_INSTANCE deployment requires one subnet (for example, the default subnet). An ACTIVE_STANDBY_MULTI_AZ deployment requires two subnets.<br/>SecurityGroups:  The list of rules (1 minimum, 125 maximum) that authorize connections to brokers.<br/>PubliclyAccessible:  Required. Enables connections from applications outside of the VPC that hosts the broker's subnets.<br/>MaintenanceWindowStartTime:  The parameters that determine the WeeklyStartTime.:<br/>Logs:  Enables Amazon CloudWatch logging for brokers.:<br/>HostInstanceType:  Required. The broker's instance type.<br/>EngineVersion:  Required. The version of the broker engine. For a list of supported engine versions, see https://docs.aws.amazon.com/amazon-mq/latest/developer-guide/broker-engine.html<br/>EngineType:  Required. The type of broker engine. Note: Currently, Amazon MQ supports only ACTIVEMQ.<br/>DeploymentMode:  Required. The deployment mode of the broker.<br/>CreatorRequestId:  The unique ID that the requester receives for the created broker. Amazon MQ passes your ID with the API action. Note: We recommend using a Universally Unique Identifier (UUID) for the creatorRequestId. You may omit the creatorRequestId if your application doesn't require idempotency.<br/>Configuration:  A list of information about the configuration.:<br/>BrokerName:  Required. The name of the broker. This value must be unique in your AWS account, 1-50 characters long, must contain only letters, numbers, dashes, and underscores, and must not contain whitespaces, brackets, wildcard characters, or special characters.<br/>AutoMinorVersionUpgrade:  Required. Enables automatic upgrades to new minor versions for brokers, as Apache releases the versions. The automatic upgrades occur during the maintenance window of the broker or after a manual broker reboot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Revision</i> The revision number of the configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> Required. The unique ID that Amazon MQ generates for the configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>General</i> Enables general logging.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Audit</i> Enables audit logging. Every user management action made using JMX or the ActiveMQ Web Console is logged.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeZone</i> The time zone, UTC by default, in either the Country/City format, or the UTC offset format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeOfDay</i> Required. The time, in 24-hour format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DayOfWeek</i> Required. The day of the week.<br/>
        :return: BrokerId:  The unique ID that Amazon MQ generates for the broker.<br/>BrokerArn:  The Amazon Resource Name (ARN) of the broker.<br/>        """
        pass

    def create_configuration(self, EngineType: str=None, EngineVersion: str=None, Name: str=None, Tags: __mapOf__string=None) -> CreateConfigurationResponse:
        """Creates a new configuration for the specified configuration name. Amazon MQ uses the default configuration (the engine type and version).

        :param self:
        :param Tags:  Create tags when creating the configuration.<br/>Name:  Required. The name of the configuration. This value can contain only alphanumeric characters, dashes, periods, underscores, and tildes (- . _ ~). This value must be 1-150 characters long.<br/>EngineVersion:  Required. The version of the broker engine. For a list of supported engine versions, see https://docs.aws.amazon.com/amazon-mq/latest/developer-guide/broker-engine.html<br/>EngineType:  Required. The type of broker engine. Note: Currently, Amazon MQ supports only ACTIVEMQ.<br/>
        :return: Name:  Required. The name of the configuration. This value can contain only alphanumeric characters, dashes, periods, underscores, and tildes (- . _ ~). This value must be 1-150 characters long.<br/>LatestRevision:  The latest revision of the configuration.:<br/>Id:  Required. The unique ID that Amazon MQ generates for the configuration.<br/>Created:  Required. The date and time of the configuration.<br/>Arn:  Required. The Amazon Resource Name (ARN) of the configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Revision</i> Required. The revision number of the configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the configuration revision.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Created</i> Required. The date and time of the configuration revision.<br/>        """
        pass

    def create_tags(self, ResourceArn: str, Tags: __mapOf__string=None):
        """Add a tag to a resource.

        :param self:
        :param Tags:  The key-value pair for the resource tag.<br/>ResourceArn:  The Amazon Resource Name (ARN) of the resource tag.<br/>
        :return:         """
        pass

    def create_user(self, Username: str, BrokerId: str, ConsoleAccess: __boolean=None, Groups: List[__string]=None, Password: str=None) -> CreateUserResponse:
        """Creates an ActiveMQ user.

        :param self:
        :param Username:  The username of the ActiveMQ user. This value can contain only alphanumeric characters, dashes, periods, underscores, and tildes (- . _ ~). This value must be 2-100 characters long.<br/>Password:  Required. The password of the user. This value must be at least 12 characters long, must contain at least 4 unique characters, and must not contain commas.<br/>Groups:  The list of groups (20 maximum) to which the ActiveMQ user belongs. This value can contain only alphanumeric characters, dashes, periods, underscores, and tildes (- . _ ~). This value must be 2-100 characters long.<br/>ConsoleAccess:  Enables access to the the ActiveMQ Web Console for the ActiveMQ user.<br/>BrokerId:  The unique ID that Amazon MQ generates for the broker.<br/>
        :return:         """
        pass

    def delete_broker(self, BrokerId: str) -> DeleteBrokerResponse:
        """Deletes a broker. Note: This API is asynchronous.

        :param self:
        :param BrokerId:  The name of the broker. This value must be unique in your AWS account, 1-50 characters long, must contain only letters, numbers, dashes, and underscores, and must not contain whitespaces, brackets, wildcard characters, or special characters.<br/>
        :return: BrokerId:  The unique ID that Amazon MQ generates for the broker.<br/>        """
        pass

    def delete_tags(self, TagKeys: List[__string], ResourceArn: str):
        """Removes a tag from a resource.

        :param self:
        :param TagKeys:  An array of tag keys to delete<br/>ResourceArn:  The Amazon Resource Name (ARN) of the resource tag.<br/>
        :return:         """
        pass

    def delete_user(self, Username: str, BrokerId: str) -> DeleteUserResponse:
        """Deletes an ActiveMQ user.

        :param self:
        :param Username:  The username of the ActiveMQ user. This value can contain only alphanumeric characters, dashes, periods, underscores, and tildes (- . _ ~). This value must be 2-100 characters long.<br/>BrokerId:  The unique ID that Amazon MQ generates for the broker.<br/>
        :return:         """
        pass

    def describe_broker(self, BrokerId: str) -> DescribeBrokerResponse:
        """Returns information about the specified broker.

        :param self:
        :param BrokerId:  The name of the broker. This value must be unique in your AWS account, 1-50 characters long, must contain only letters, numbers, dashes, and underscores, and must not contain whitespaces, brackets, wildcard characters, or special characters.<br/>
        :return: Users:  The list of all ActiveMQ usernames for the specified broker.<br/>Tags:  The list of all tags associated with this broker.<br/>SubnetIds:  The list of groups (2 maximum) that define which subnets and IP ranges the broker can use from different Availability Zones. A SINGLE_INSTANCE deployment requires one subnet (for example, the default subnet). An ACTIVE_STANDBY_MULTI_AZ deployment requires two subnets.<br/>SecurityGroups:  Required. The list of rules (1 minimum, 125 maximum) that authorize connections to brokers.<br/>PubliclyAccessible:  Required. Enables connections from applications outside of the VPC that hosts the broker's subnets.<br/>PendingEngineVersion:  The version of the broker engine to upgrade to. For a list of supported engine versions, see https://docs.aws.amazon.com/amazon-mq/latest/developer-guide/broker-engine.html<br/>MaintenanceWindowStartTime:  The parameters that determine the WeeklyStartTime.:<br/>Logs:  The list of information about logs currently enabled and pending to be deployed for the specified broker.:<br/>HostInstanceType:  The broker's instance type.<br/>EngineVersion:  The version of the broker engine. For a list of supported engine versions, see https://docs.aws.amazon.com/amazon-mq/latest/developer-guide/broker-engine.html<br/>EngineType:  Required. The type of broker engine. Note: Currently, Amazon MQ supports only ACTIVEMQ.<br/>DeploymentMode:  Required. The deployment mode of the broker.<br/>Created:  The time when the broker was created.<br/>Configurations:  The list of all revisions for the specified configuration.:<br/>BrokerState:  The status of the broker.<br/>BrokerName:  The name of the broker. This value must be unique in your AWS account, 1-50 characters long, must contain only letters, numbers, dashes, and underscores, and must not contain whitespaces, brackets, wildcard characters, or special characters.<br/>BrokerInstances:  A list of information about allocated brokers.<br/>BrokerId:  The unique ID that Amazon MQ generates for the broker.<br/>BrokerArn:  The Amazon Resource Name (ARN) of the broker.<br/>AutoMinorVersionUpgrade:  Required. Enables automatic upgrades to new minor versions for brokers, as Apache releases the versions. The automatic upgrades occur during the maintenance window of the broker or after a manual broker reboot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Pending</i> The pending configuration of the broker.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Pending</b>: The pending configuration of the broker.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>History</i> The history of configurations applied to the broker.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Current</i> The current configuration of the broker.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Current</b>: The current configuration of the broker.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Revision</i> The revision number of the configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> Required. The unique ID that Amazon MQ generates for the configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Revision</i> The revision number of the configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> Required. The unique ID that Amazon MQ generates for the configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Pending</i> The list of information about logs pending to be deployed for the specified broker.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Pending</b>: The list of information about logs pending to be deployed for the specified broker.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GeneralLogGroup</i> The location of the CloudWatch Logs log group where general logs are sent.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>General</i> Enables general logging.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AuditLogGroup</i> The location of the CloudWatch Logs log group where audit logs are sent.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Audit</i> Enables audit logging. Every user management action made using JMX or the ActiveMQ Web Console is logged.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>General</i> Enables general logging.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Audit</i> Enables audit logging. Every user management action made using JMX or the ActiveMQ Web Console is logged.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeZone</i> The time zone, UTC by default, in either the Country/City format, or the UTC offset format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeOfDay</i> Required. The time, in 24-hour format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DayOfWeek</i> Required. The day of the week.<br/>        """
        pass

    def describe_broker_engine_types(self, EngineType: str=None, MaxResults: int=None, NextToken: str=None) -> DescribeBrokerEngineTypesResponse:
        """Describe available engine types and versions.

        :param self:
        :param NextToken:  The token that specifies the next page of results Amazon MQ should return. To request the first page, leave nextToken empty.<br/>MaxResults:  The maximum number of engine types that Amazon MQ can return per page (20 by default). This value must be an integer from 5 to 100.<br/>EngineType:  Filter response by engine type.<br/>
        :return: NextToken:  The token that specifies the next page of results Amazon MQ should return. To request the first page, leave nextToken empty.<br/>MaxResults:  Required. The maximum number of engine types that can be returned per page (20 by default). This value must be an integer from 5 to 100.<br/>BrokerEngineTypes:  List of available engine types and versions.<br/>        """
        pass

    def describe_broker_instance_options(self, EngineType: str=None, HostInstanceType: str=None, MaxResults: int=None, NextToken: str=None) -> DescribeBrokerInstanceOptionsResponse:
        """Describe available broker instance options.

        :param self:
        :param NextToken:  The token that specifies the next page of results Amazon MQ should return. To request the first page, leave nextToken empty.<br/>MaxResults:  The maximum number of instance options that Amazon MQ can return per page (20 by default). This value must be an integer from 5 to 100.<br/>HostInstanceType:  Filter response by host instance type.<br/>EngineType:  Filter response by engine type.<br/>
        :return: NextToken:  The token that specifies the next page of results Amazon MQ should return. To request the first page, leave nextToken empty.<br/>MaxResults:  Required. The maximum number of instance options that can be returned per page (20 by default). This value must be an integer from 5 to 100.<br/>BrokerInstanceOptions:  List of available broker instance options.<br/>        """
        pass

    def describe_configuration(self, ConfigurationId: str) -> DescribeConfigurationResponse:
        """Returns information about the specified configuration.

        :param self:
        :param ConfigurationId:  The unique ID that Amazon MQ generates for the configuration.<br/>
        :return: Tags:  The list of all tags associated with this configuration.<br/>Name:  Required. The name of the configuration. This value can contain only alphanumeric characters, dashes, periods, underscores, and tildes (- . _ ~). This value must be 1-150 characters long.<br/>LatestRevision:  Required. The latest revision of the configuration.:<br/>Id:  Required. The unique ID that Amazon MQ generates for the configuration.<br/>EngineVersion:  Required. The version of the broker engine. For a list of supported engine versions, see https://docs.aws.amazon.com/amazon-mq/latest/developer-guide/broker-engine.html<br/>EngineType:  Required. The type of broker engine. Note: Currently, Amazon MQ supports only ACTIVEMQ.<br/>Description:  Required. The description of the configuration.<br/>Created:  Required. The date and time of the configuration revision.<br/>Arn:  Required. The ARN of the configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Revision</i> Required. The revision number of the configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the configuration revision.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Created</i> Required. The date and time of the configuration revision.<br/>        """
        pass

    def describe_configuration_revision(self, ConfigurationRevision: str, ConfigurationId: str) -> DescribeConfigurationRevisionResponse:
        """Returns the specified configuration revision for the specified configuration.

        :param self:
        :param ConfigurationRevision:  The revision of the configuration.<br/>ConfigurationId:  The unique ID that Amazon MQ generates for the configuration.<br/>
        :return: Description:  The description of the configuration.<br/>Data:  Required. The base64-encoded XML configuration.<br/>Created:  Required. The date and time of the configuration.<br/>ConfigurationId:  Required. The unique ID that Amazon MQ generates for the configuration.<br/>        """
        pass

    def describe_user(self, Username: str, BrokerId: str) -> DescribeUserResponse:
        """Returns information about an ActiveMQ user.

        :param self:
        :param Username:  The username of the ActiveMQ user. This value can contain only alphanumeric characters, dashes, periods, underscores, and tildes (- . _ ~). This value must be 2-100 characters long.<br/>BrokerId:  The unique ID that Amazon MQ generates for the broker.<br/>
        :return: Username:  Required. The username of the ActiveMQ user. This value can contain only alphanumeric characters, dashes, periods, underscores, and tildes (- . _ ~). This value must be 2-100 characters long.<br/>Pending:  The status of the changes pending for the ActiveMQ user.:<br/>Groups:  The list of groups (20 maximum) to which the ActiveMQ user belongs. This value can contain only alphanumeric characters, dashes, periods, underscores, and tildes (- . _ ~). This value must be 2-100 characters long.<br/>ConsoleAccess:  Enables access to the the ActiveMQ Web Console for the ActiveMQ user.<br/>BrokerId:  Required. The unique ID that Amazon MQ generates for the broker.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingChange</i> Required. The type of change pending for the ActiveMQ user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Groups</i> The list of groups (20 maximum) to which the ActiveMQ user belongs. This value can contain only alphanumeric characters, dashes, periods, underscores, and tildes (- . _ ~). This value must be 2-100 characters long.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConsoleAccess</i> Enables access to the the ActiveMQ Web Console for the ActiveMQ user.<br/>        """
        pass

    def list_brokers(self, MaxResults: int=None, NextToken: str=None) -> ListBrokersResponse:
        """Returns a list of all brokers.

        :param self:
        :param NextToken:  The token that specifies the next page of results Amazon MQ should return. To request the first page, leave nextToken empty.<br/>MaxResults:  The maximum number of brokers that Amazon MQ can return per page (20 by default). This value must be an integer from 5 to 100.<br/>
        :return: NextToken:  The token that specifies the next page of results Amazon MQ should return. To request the first page, leave nextToken empty.<br/>BrokerSummaries:  A list of information about all brokers.<br/>        """
        pass

    def list_configuration_revisions(self, ConfigurationId: str, MaxResults: int=None, NextToken: str=None) -> ListConfigurationRevisionsResponse:
        """Returns a list of all revisions for the specified configuration.

        :param self:
        :param NextToken:  The token that specifies the next page of results Amazon MQ should return. To request the first page, leave nextToken empty.<br/>MaxResults:  The maximum number of configurations that Amazon MQ can return per page (20 by default). This value must be an integer from 5 to 100.<br/>ConfigurationId:  The unique ID that Amazon MQ generates for the configuration.<br/>
        :return: Revisions:  The list of all revisions for the specified configuration.<br/>NextToken:  The token that specifies the next page of results Amazon MQ should return. To request the first page, leave nextToken empty.<br/>MaxResults:  The maximum number of configuration revisions that can be returned per page (20 by default). This value must be an integer from 5 to 100.<br/>ConfigurationId:  The unique ID that Amazon MQ generates for the configuration.<br/>        """
        pass

    def list_configurations(self, MaxResults: int=None, NextToken: str=None) -> ListConfigurationsResponse:
        """Returns a list of all configurations.

        :param self:
        :param NextToken:  The token that specifies the next page of results Amazon MQ should return. To request the first page, leave nextToken empty.<br/>MaxResults:  The maximum number of configurations that Amazon MQ can return per page (20 by default). This value must be an integer from 5 to 100.<br/>
        :return: NextToken:  The token that specifies the next page of results Amazon MQ should return. To request the first page, leave nextToken empty.<br/>MaxResults:  The maximum number of configurations that Amazon MQ can return per page (20 by default). This value must be an integer from 5 to 100.<br/>Configurations:  The list of all revisions for the specified configuration.<br/>        """
        pass

    def list_tags(self, ResourceArn: str) -> ListTagsResponse:
        """Lists tags for a resource.

        :param self:
        :param ResourceArn:  The Amazon Resource Name (ARN) of the resource tag.<br/>
        :return: Tags:  The key-value pair for the resource tag.<br/>        """
        pass

    def list_users(self, BrokerId: str, MaxResults: int=None, NextToken: str=None) -> ListUsersResponse:
        """Returns a list of all ActiveMQ users.

        :param self:
        :param NextToken:  The token that specifies the next page of results Amazon MQ should return. To request the first page, leave nextToken empty.<br/>MaxResults:  The maximum number of ActiveMQ users that can be returned per page (20 by default). This value must be an integer from 5 to 100.<br/>BrokerId:  The unique ID that Amazon MQ generates for the broker.<br/>
        :return: Users:  Required. The list of all ActiveMQ usernames for the specified broker.<br/>NextToken:  The token that specifies the next page of results Amazon MQ should return. To request the first page, leave nextToken empty.<br/>MaxResults:  Required. The maximum number of ActiveMQ users that can be returned per page (20 by default). This value must be an integer from 5 to 100.<br/>BrokerId:  Required. The unique ID that Amazon MQ generates for the broker.<br/>        """
        pass

    def reboot_broker(self, BrokerId: str) -> RebootBrokerResponse:
        """Reboots a broker. Note: This API is asynchronous.

        :param self:
        :param BrokerId:  The unique ID that Amazon MQ generates for the broker.<br/>
        :return:         """
        pass

    def update_broker(self, BrokerId: str, AutoMinorVersionUpgrade: __boolean=None, Configuration: ConfigurationId=None, EngineVersion: str=None, Logs: Logs=None) -> UpdateBrokerResponse:
        """Adds a pending configuration change to a broker.

        :param self:
        :param Logs:  Enables Amazon CloudWatch logging for brokers.:<br/>EngineVersion:  The version of the broker engine. For a list of supported engine versions, see https://docs.aws.amazon.com/amazon-mq/latest/developer-guide/broker-engine.html<br/>Configuration:  A list of information about the configuration.:<br/>BrokerId:  The name of the broker. This value must be unique in your AWS account, 1-50 characters long, must contain only letters, numbers, dashes, and underscores, and must not contain whitespaces, brackets, wildcard characters, or special characters.<br/>AutoMinorVersionUpgrade:  Enables automatic upgrades to new minor versions for brokers, as Apache releases the versions. The automatic upgrades occur during the maintenance window of the broker or after a manual broker reboot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Revision</i> The revision number of the configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> Required. The unique ID that Amazon MQ generates for the configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>General</i> Enables general logging.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Audit</i> Enables audit logging. Every user management action made using JMX or the ActiveMQ Web Console is logged.<br/>
        :return: Logs:  The list of information about logs to be enabled for the specified broker.:<br/>EngineVersion:  The version of the broker engine to upgrade to. For a list of supported engine versions, see https://docs.aws.amazon.com/amazon-mq/latest/developer-guide/broker-engine.html<br/>Configuration:  The ID of the updated configuration.:<br/>BrokerId:  Required. The unique ID that Amazon MQ generates for the broker.<br/>AutoMinorVersionUpgrade:  The new value of automatic upgrades to new minor version for brokers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Revision</i> The revision number of the configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> Required. The unique ID that Amazon MQ generates for the configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>General</i> Enables general logging.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Audit</i> Enables audit logging. Every user management action made using JMX or the ActiveMQ Web Console is logged.<br/>        """
        pass

    def update_configuration(self, ConfigurationId: str, Data: str=None, Description: str=None) -> UpdateConfigurationResponse:
        """Updates the specified configuration.

        :param self:
        :param Description:  The description of the configuration.<br/>Data:  Required. The base64-encoded XML configuration.<br/>ConfigurationId:  The unique ID that Amazon MQ generates for the configuration.<br/>
        :return: Warnings:  The list of the first 20 warnings about the configuration XML elements or attributes that were sanitized.<br/>Name:  Required. The name of the configuration. This value can contain only alphanumeric characters, dashes, periods, underscores, and tildes (- . _ ~). This value must be 1-150 characters long.<br/>LatestRevision:  The latest revision of the configuration.:<br/>Id:  Required. The unique ID that Amazon MQ generates for the configuration.<br/>Created:  Required. The date and time of the configuration.<br/>Arn:  Required. The Amazon Resource Name (ARN) of the configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Revision</i> Required. The revision number of the configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the configuration revision.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Created</i> Required. The date and time of the configuration revision.<br/>        """
        pass

    def update_user(self, Username: str, BrokerId: str, ConsoleAccess: __boolean=None, Groups: List[__string]=None, Password: str=None) -> UpdateUserResponse:
        """Updates the information for an ActiveMQ user.

        :param self:
        :param Username:  Required. The username of the ActiveMQ user. This value can contain only alphanumeric characters, dashes, periods, underscores, and tildes (- . _ ~). This value must be 2-100 characters long.<br/>Password:  The password of the user. This value must be at least 12 characters long, must contain at least 4 unique characters, and must not contain commas.<br/>Groups:  The list of groups (20 maximum) to which the ActiveMQ user belongs. This value can contain only alphanumeric characters, dashes, periods, underscores, and tildes (- . _ ~). This value must be 2-100 characters long.<br/>ConsoleAccess:  Enables access to the the ActiveMQ Web Console for the ActiveMQ user.<br/>BrokerId:  The unique ID that Amazon MQ generates for the broker.<br/>
        :return:         """
        pass



class MTurk(BaseClient):
    """<fullname>Amazon Mechanical Turk API Reference</fullname>"""


    class ListAssignmentsForHITPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_assignments_for_hit` for valid parameters"""
            pass
    class ListQualificationTypesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_qualification_types` for valid parameters"""
            pass
    class ListHITsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_hits` for valid parameters"""
            pass
    class ListWorkerBlocksPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_worker_blocks` for valid parameters"""
            pass
    class ListReviewableHITsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_reviewable_hits` for valid parameters"""
            pass
    class ListHITsForQualificationTypePaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_hits_for_qualification_type` for valid parameters"""
            pass
    class ListQualificationRequestsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_qualification_requests` for valid parameters"""
            pass
    class ListWorkersWithQualificationTypePaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_workers_with_qualification_type` for valid parameters"""
            pass
    class ListBonusPaymentsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_bonus_payments` for valid parameters"""
            pass


    class AcceptQualificationRequestRequest(Mapping):
        pass
    
    class AcceptQualificationRequestResponse(Mapping):
        pass
    
    class ApproveAssignmentRequest(Mapping):
        pass
    
    class ApproveAssignmentResponse(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class AssociateQualificationWithWorkerRequest(Mapping):
        pass
    
    class AssociateQualificationWithWorkerResponse(Mapping):
        pass
    
    class CreateAdditionalAssignmentsForHITRequest(Mapping):
        pass
    
    class CreateAdditionalAssignmentsForHITResponse(Mapping):
        pass
    
    class CreateHITRequest(Mapping):
        pass
    
    class CreateHITResponse(Mapping):
        pass
    
    class Long(object):
        pass
    
    class QualificationRequirement(Mapping):
        pass
    
    class ReviewPolicy(Mapping):
        pass
    
    class HITLayoutParameter(Mapping):
        pass
    
    class CreateHITTypeRequest(Mapping):
        pass
    
    class CreateHITTypeResponse(Mapping):
        pass
    
    class CreateHITWithHITTypeRequest(Mapping):
        pass
    
    class CreateHITWithHITTypeResponse(Mapping):
        pass
    
    class CreateQualificationTypeRequest(Mapping):
        pass
    
    class CreateQualificationTypeResponse(Mapping):
        pass
    
    class CreateWorkerBlockRequest(Mapping):
        pass
    
    class CreateWorkerBlockResponse(Mapping):
        pass
    
    class DeleteHITRequest(Mapping):
        pass
    
    class DeleteHITResponse(Mapping):
        pass
    
    class DeleteQualificationTypeRequest(Mapping):
        pass
    
    class DeleteQualificationTypeResponse(Mapping):
        pass
    
    class DeleteWorkerBlockRequest(Mapping):
        pass
    
    class DeleteWorkerBlockResponse(Mapping):
        pass
    
    class DisassociateQualificationFromWorkerRequest(Mapping):
        pass
    
    class DisassociateQualificationFromWorkerResponse(Mapping):
        pass
    
    class GetAccountBalanceRequest(Mapping):
        pass
    
    class GetAccountBalanceResponse(Mapping):
        pass
    
    class GetAssignmentRequest(Mapping):
        pass
    
    class GetAssignmentResponse(Mapping):
        pass
    
    class GetFileUploadURLRequest(Mapping):
        pass
    
    class GetFileUploadURLResponse(Mapping):
        pass
    
    class GetHITRequest(Mapping):
        pass
    
    class GetHITResponse(Mapping):
        pass
    
    class GetQualificationScoreRequest(Mapping):
        pass
    
    class GetQualificationScoreResponse(Mapping):
        pass
    
    class GetQualificationTypeRequest(Mapping):
        pass
    
    class GetQualificationTypeResponse(Mapping):
        pass
    
    class ListAssignmentsForHITRequest(Mapping):
        pass
    
    class ListAssignmentsForHITResponse(Mapping):
        pass
    
    class AssignmentStatus(object):
        pass
    
    class ListBonusPaymentsRequest(Mapping):
        pass
    
    class ListBonusPaymentsResponse(Mapping):
        pass
    
    class ListHITsRequest(Mapping):
        pass
    
    class ListHITsResponse(Mapping):
        pass
    
    class ListHITsForQualificationTypeRequest(Mapping):
        pass
    
    class ListHITsForQualificationTypeResponse(Mapping):
        pass
    
    class ListQualificationRequestsRequest(Mapping):
        pass
    
    class ListQualificationRequestsResponse(Mapping):
        pass
    
    class ListQualificationTypesRequest(Mapping):
        pass
    
    class ListQualificationTypesResponse(Mapping):
        pass
    
    class ListReviewPolicyResultsForHITRequest(Mapping):
        pass
    
    class ListReviewPolicyResultsForHITResponse(Mapping):
        pass
    
    class ReviewPolicyLevel(object):
        pass
    
    class ListReviewableHITsRequest(Mapping):
        pass
    
    class ListReviewableHITsResponse(Mapping):
        pass
    
    class ListWorkerBlocksRequest(Mapping):
        pass
    
    class ListWorkerBlocksResponse(Mapping):
        pass
    
    class ListWorkersWithQualificationTypeRequest(Mapping):
        pass
    
    class ListWorkersWithQualificationTypeResponse(Mapping):
        pass
    
    class NotifyWorkersRequest(Mapping):
        pass
    
    class NotifyWorkersResponse(Mapping):
        pass
    
    class CustomerId(object):
        pass
    
    class RejectAssignmentRequest(Mapping):
        pass
    
    class RejectAssignmentResponse(Mapping):
        pass
    
    class RejectQualificationRequestRequest(Mapping):
        pass
    
    class RejectQualificationRequestResponse(Mapping):
        pass
    
    class SendBonusRequest(Mapping):
        pass
    
    class SendBonusResponse(Mapping):
        pass
    
    class SendTestEventNotificationRequest(Mapping):
        pass
    
    class SendTestEventNotificationResponse(Mapping):
        pass
    
    class NotificationSpecification(Mapping):
        pass
    
    class UpdateExpirationForHITRequest(Mapping):
        pass
    
    class UpdateExpirationForHITResponse(Mapping):
        pass
    
    class Timestamp(object):
        pass
    
    class UpdateHITReviewStatusRequest(Mapping):
        pass
    
    class UpdateHITReviewStatusResponse(Mapping):
        pass
    
    class UpdateHITTypeOfHITRequest(Mapping):
        pass
    
    class UpdateHITTypeOfHITResponse(Mapping):
        pass
    
    class UpdateNotificationSettingsRequest(Mapping):
        pass
    
    class UpdateNotificationSettingsResponse(Mapping):
        pass
    
    class UpdateQualificationTypeRequest(Mapping):
        pass
    
    class UpdateQualificationTypeResponse(Mapping):
        pass
    
    def accept_qualification_request(self, QualificationRequestId: str, IntegerValue: int=None) -> AcceptQualificationRequestResponse:
        """ The <code>AcceptQualificationRequest</code> operation approves a Worker's request for a Qualification.   Only the owner of the Qualification type can grant a Qualification request for that type.   A successful request for the <code>AcceptQualificationRequest</code> operation returns with no errors and an empty body. 

        :param self:
        :param IntegerValue:   The value of the Qualification. You can omit this value if you are using the presence or absence of the Qualification as the basis for a HIT requirement. <br/>QualificationRequestId:  The ID of the Qualification request, as returned by the <code>GetQualificationRequests</code> operation.<br/>
        :return:         """
        pass

    def approve_assignment(self, AssignmentId: str, RequesterFeedback: str=None, OverrideRejection: Boolean=None) -> ApproveAssignmentResponse:
        """ The <code>ApproveAssignment</code> operation approves the results of a completed assignment.   Approving an assignment initiates two payments from the Requester's Amazon.com account  <ul> <li>  The Worker who submitted the results is paid the reward specified in the HIT.  </li> <li>  Amazon Mechanical Turk fees are debited.  </li> </ul>  If the Requester's account does not have adequate funds for these payments, the call to ApproveAssignment returns an exception, and the approval is not processed. You can include an optional feedback message with the approval, which the Worker can see in the Status section of the web site.   You can also call this operation for assignments that were previous rejected and approve them by explicitly overriding the previous rejection. This only works on rejected assignments that were submitted within the previous 30 days and only if the assignment's related HIT has not been deleted. 

        :param self:
        :param OverrideRejection:   A flag indicating that an assignment should be approved even if it was previously rejected. Defaults to <code>False</code>. <br/>RequesterFeedback:   A message for the Worker, which the Worker can see in the Status section of the web site. <br/>AssignmentId:   The ID of the assignment. The assignment must correspond to a HIT created by the Requester. <br/>
        :return:         """
        pass

    def associate_qualification_with_worker(self, WorkerId: str, QualificationTypeId: str, IntegerValue: int=None, SendNotification: Boolean=None) -> AssociateQualificationWithWorkerResponse:
        """ The <code>AssociateQualificationWithWorker</code> operation gives a Worker a Qualification. <code>AssociateQualificationWithWorker</code> does not require that the Worker submit a Qualification request. It gives the Qualification directly to the Worker.   You can only assign a Qualification of a Qualification type that you created (using the <code>CreateQualificationType</code> operation).  <note>  Note: <code>AssociateQualificationWithWorker</code> does not affect any pending Qualification requests for the Qualification by the Worker. If you assign a Qualification to a Worker, then later grant a Qualification request made by the Worker, the granting of the request may modify the Qualification score. To resolve a pending Qualification request without affecting the Qualification the Worker already has, reject the request with the <code>RejectQualificationRequest</code> operation.  </note>

        :param self:
        :param SendNotification:   Specifies whether to send a notification email message to the Worker saying that the qualification was assigned to the Worker. Note: this is true by default. <br/>IntegerValue:  The value of the Qualification to assign.<br/>WorkerId:   The ID of the Worker to whom the Qualification is being assigned. Worker IDs are included with submitted HIT assignments and Qualification requests. <br/>QualificationTypeId:  The ID of the Qualification type to use for the assigned Qualification.<br/>
        :return:         """
        pass

    def create_additional_assignments_for_hit(self, NumberOfAdditionalAssignments: int, HITId: str, UniqueRequestToken: str=None) -> CreateAdditionalAssignmentsForHITResponse:
        """ The <code>CreateAdditionalAssignmentsForHIT</code> operation increases the maximum number of assignments of an existing HIT.   To extend the maximum number of assignments, specify the number of additional assignments. <note> <ul> <li> HITs created with fewer than 10 assignments cannot be extended to have 10 or more assignments. Attempting to add assignments in a way that brings the total number of assignments for a HIT from fewer than 10 assignments to 10 or more assignments will result in an <code>AWS.MechanicalTurk.InvalidMaximumAssignmentsIncrease</code> exception. </li> <li> HITs that were created before July 22, 2015 cannot be extended. Attempting to extend HITs that were created before July 22, 2015 will result in an <code>AWS.MechanicalTurk.HITTooOldForExtension</code> exception.  </li> </ul> </note>

        :param self:
        :param UniqueRequestToken:   A unique identifier for this request, which allows you to retry the call on error without extending the HIT multiple times. This is useful in cases such as network timeouts where it is unclear whether or not the call succeeded on the server. If the extend HIT already exists in the system from a previous call using the same <code>UniqueRequestToken</code>, subsequent calls will return an error with a message containing the request ID. <br/>NumberOfAdditionalAssignments:  The number of additional assignments to request for this HIT.<br/>HITId:  The ID of the HIT to extend.<br/>
        :return:         """
        pass

    def create_hit(self, Description: str, Title: str, Reward: str, AssignmentDurationInSeconds: Long, LifetimeInSeconds: Long, MaxAssignments: int=None, AutoApprovalDelayInSeconds: Long=None, Keywords: str=None, Question: str=None, RequesterAnnotation: str=None, QualificationRequirements: List[QualificationRequirement]=None, UniqueRequestToken: str=None, AssignmentReviewPolicy: ReviewPolicy=None, HITReviewPolicy: ReviewPolicy=None, HITLayoutId: str=None, HITLayoutParameters: List[HITLayoutParameter]=None) -> CreateHITResponse:
        """The <code>CreateHIT</code> operation creates a new Human Intelligence Task (HIT). The new HIT is made available for Workers to find and accept on the Amazon Mechanical Turk website.   This operation allows you to specify a new HIT by passing in values for the properties of the HIT, such as its title, reward amount and number of assignments. When you pass these values to <code>CreateHIT</code>, a new HIT is created for you, with a new <code>HITTypeID</code>. The HITTypeID can be used to create additional HITs in the future without needing to specify common parameters such as the title, description and reward amount each time.  An alternative way to create HITs is to first generate a HITTypeID using the <code>CreateHITType</code> operation and then call the <code>CreateHITWithHITType</code> operation. This is the recommended best practice for Requesters who are creating large numbers of HITs.  CreateHIT also supports several ways to provide question data: by providing a value for the <code>Question</code> parameter that fully specifies the contents of the HIT, or by providing a <code>HitLayoutId</code> and associated <code>HitLayoutParameters</code>.  <note>  If a HIT is created with 10 or more maximum assignments, there is an additional fee. For more information, see <a href="https://requester.mturk.com/pricing">Amazon Mechanical Turk Pricing</a>. </note>

        :param self:
        :param HITLayoutParameters:   If the HITLayoutId is provided, any placeholder values must be filled in with values using the HITLayoutParameter structure. For more information, see HITLayout. <br/>HITLayoutId:   The HITLayoutId allows you to use a pre-existing HIT design with placeholder values and create an additional HIT by providing those values as HITLayoutParameters.   Constraints: Either a Question parameter or a HITLayoutId parameter must be provided. <br/>HITReviewPolicy:   The HIT-level Review Policy applies to the HIT. You can specify for Mechanical Turk to take various actions based on the policy. :<br/>AssignmentReviewPolicy:   The Assignment-level Review Policy applies to the assignments under the HIT. You can specify for Mechanical Turk to take various actions based on the policy. :<br/>UniqueRequestToken:   A unique identifier for this request which allows you to retry the call on error without creating duplicate HITs. This is useful in cases such as network timeouts where it is unclear whether or not the call succeeded on the server. If the HIT already exists in the system from a previous call using the same UniqueRequestToken, subsequent calls will return a AWS.MechanicalTurk.HitAlreadyExists error with a message containing the HITId.  <note>  Note: It is your responsibility to ensure uniqueness of the token. The unique token expires after 24 hours. Subsequent calls using the same UniqueRequestToken made after the 24 hour limit could create duplicate HITs.  </note><br/>QualificationRequirements:   Conditions that a Worker's Qualifications must meet in order to accept the HIT. A HIT can have between zero and ten Qualification requirements. All requirements must be met in order for a Worker to accept the HIT. Additionally, other actions can be restricted using the <code>ActionsGuarded</code> field on each <code>QualificationRequirement</code> structure. <br/>RequesterAnnotation:   An arbitrary data field. The RequesterAnnotation parameter lets your application attach arbitrary data to the HIT for tracking purposes. For example, this parameter could be an identifier internal to the Requester's application that corresponds with the HIT.   The RequesterAnnotation parameter for a HIT is only visible to the Requester who created the HIT. It is not shown to the Worker, or any other Requester.   The RequesterAnnotation parameter may be different for each HIT you submit. It does not affect how your HITs are grouped. <br/>Question:   The data the person completing the HIT uses to produce the results.   Constraints: Must be a QuestionForm data structure, an ExternalQuestion data structure, or an HTMLQuestion data structure. The XML question data must not be larger than 64 kilobytes (65,535 bytes) in size, including whitespace.  Either a Question parameter or a HITLayoutId parameter must be provided.<br/>Description:   A general description of the HIT. A description includes detailed information about the kind of task the HIT contains. On the Amazon Mechanical Turk web site, the HIT description appears in the expanded view of search results, and in the HIT and assignment screens. A good description gives the user enough information to evaluate the HIT before accepting it. <br/>Keywords:   One or more words or phrases that describe the HIT, separated by commas. These words are used in searches to find HITs. <br/>Title:   The title of the HIT. A title should be short and descriptive about the kind of task the HIT contains. On the Amazon Mechanical Turk web site, the HIT title appears in search results, and everywhere the HIT is mentioned. <br/>Reward:   The amount of money the Requester will pay a Worker for successfully completing the HIT. <br/>AssignmentDurationInSeconds:   The amount of time, in seconds, that a Worker has to complete the HIT after accepting it. If a Worker does not complete the assignment within the specified duration, the assignment is considered abandoned. If the HIT is still active (that is, its lifetime has not elapsed), the assignment becomes available for other users to find and accept. <br/>LifetimeInSeconds:   An amount of time, in seconds, after which the HIT is no longer available for users to accept. After the lifetime of the HIT elapses, the HIT no longer appears in HIT searches, even if not all of the assignments for the HIT have been accepted. <br/>AutoApprovalDelayInSeconds:   The number of seconds after an assignment for the HIT has been submitted, after which the assignment is considered Approved automatically unless the Requester explicitly rejects it. <br/>MaxAssignments:   The number of times the HIT can be accepted and completed before the HIT becomes unavailable. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> Name of the parameter from the Review policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyName</i>  Name of a Review Policy: SimplePlurality/2011-09-01 or ScoreMyKnownAnswers/2011-09-01 <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> Name of the parameter from the Review policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyName</i>  Name of a Review Policy: SimplePlurality/2011-09-01 or ScoreMyKnownAnswers/2011-09-01 <br/>
        :return: HIT:   Contains the newly created HIT data. For a description of the HIT data structure as it appears in responses, see the HIT Data Structure documentation. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfAssignmentsCompleted</i>  The number of assignments for this HIT that have been approved or rejected.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfAssignmentsAvailable</i>  The number of assignments for this HIT that are available for Workers to accept.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfAssignmentsPending</i>  The number of assignments for this HIT that are being previewed or have been accepted by Workers, but have not yet been submitted, returned, or abandoned.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HITReviewStatus</i>  Indicates the review status of the HIT. Valid Values are NotReviewed | MarkedForReview | ReviewedAppropriate | ReviewedInappropriate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QualificationRequirements</i>  Conditions that a Worker's Qualifications must meet in order to accept the HIT. A HIT can have between zero and ten Qualification requirements. All requirements must be met in order for a Worker to accept the HIT. Additionally, other actions can be restricted using the <code>ActionsGuarded</code> field on each <code>QualificationRequirement</code> structure. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequesterAnnotation</i>  An arbitrary data field the Requester who created the HIT can use. This field is visible only to the creator of the HIT.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssignmentDurationInSeconds</i>  The length of time, in seconds, that a Worker has to complete the HIT after accepting it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Expiration</i> The date and time the HIT expires.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoApprovalDelayInSeconds</i> The amount of time, in seconds, after the Worker submits an assignment for the HIT that the results are automatically approved by Amazon Mechanical Turk. This is the amount of time the Requester has to reject an assignment submitted by a Worker before the assignment is auto-approved and the Worker is paid. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Reward</i> A string representing a currency amount.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxAssignments</i> The number of times the HIT can be accepted and completed before the HIT becomes unavailable. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HITStatus</i> The status of the HIT and its assignments. Valid Values are Assignable | Unassignable | Reviewable | Reviewing | Disposed. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Keywords</i>  One or more words or phrases that describe the HIT, separated by commas. Search terms similar to the keywords of a HIT are more likely to have the HIT in the search results.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Question</i>  The data the Worker completing the HIT uses produce the results. This is either either a QuestionForm, HTMLQuestion or an ExternalQuestion data structure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i>  A general description of the HIT.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Title</i>  The title of the HIT.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i>  The date and time the HIT was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HITLayoutId</i>  The ID of the HIT Layout of this HIT.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HITGroupId</i>  The ID of the HIT Group of this HIT.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HITTypeId</i> The ID of the HIT type of this HIT<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HITId</i>  A unique identifier for the HIT.<br/>        """
        pass

    def create_hit_type(self, Description: str, Title: str, Reward: str, AssignmentDurationInSeconds: Long, AutoApprovalDelayInSeconds: Long=None, Keywords: str=None, QualificationRequirements: List[QualificationRequirement]=None) -> CreateHITTypeResponse:
        """ The <code>CreateHITType</code> operation creates a new HIT type. This operation allows you to define a standard set of HIT properties to use when creating HITs. If you register a HIT type with values that match an existing HIT type, the HIT type ID of the existing type will be returned. 

        :param self:
        :param QualificationRequirements:   Conditions that a Worker's Qualifications must meet in order to accept the HIT. A HIT can have between zero and ten Qualification requirements. All requirements must be met in order for a Worker to accept the HIT. Additionally, other actions can be restricted using the <code>ActionsGuarded</code> field on each <code>QualificationRequirement</code> structure. <br/>Description:   A general description of the HIT. A description includes detailed information about the kind of task the HIT contains. On the Amazon Mechanical Turk web site, the HIT description appears in the expanded view of search results, and in the HIT and assignment screens. A good description gives the user enough information to evaluate the HIT before accepting it. <br/>Keywords:   One or more words or phrases that describe the HIT, separated by commas. These words are used in searches to find HITs. <br/>Title:   The title of the HIT. A title should be short and descriptive about the kind of task the HIT contains. On the Amazon Mechanical Turk web site, the HIT title appears in search results, and everywhere the HIT is mentioned. <br/>Reward:   The amount of money the Requester will pay a Worker for successfully completing the HIT. <br/>AssignmentDurationInSeconds:   The amount of time, in seconds, that a Worker has to complete the HIT after accepting it. If a Worker does not complete the assignment within the specified duration, the assignment is considered abandoned. If the HIT is still active (that is, its lifetime has not elapsed), the assignment becomes available for other users to find and accept. <br/>AutoApprovalDelayInSeconds:   The number of seconds after an assignment for the HIT has been submitted, after which the assignment is considered Approved automatically unless the Requester explicitly rejects it. <br/>
        :return: HITTypeId:   The ID of the newly registered HIT type.<br/>        """
        pass

    def create_hit_with_hit_type(self, LifetimeInSeconds: Long, HITTypeId: str, MaxAssignments: int=None, Question: str=None, RequesterAnnotation: str=None, UniqueRequestToken: str=None, AssignmentReviewPolicy: ReviewPolicy=None, HITReviewPolicy: ReviewPolicy=None, HITLayoutId: str=None, HITLayoutParameters: List[HITLayoutParameter]=None) -> CreateHITWithHITTypeResponse:
        """ The <code>CreateHITWithHITType</code> operation creates a new Human Intelligence Task (HIT) using an existing HITTypeID generated by the <code>CreateHITType</code> operation.   This is an alternative way to create HITs from the <code>CreateHIT</code> operation. This is the recommended best practice for Requesters who are creating large numbers of HITs.  CreateHITWithHITType also supports several ways to provide question data: by providing a value for the <code>Question</code> parameter that fully specifies the contents of the HIT, or by providing a <code>HitLayoutId</code> and associated <code>HitLayoutParameters</code>.  <note>  If a HIT is created with 10 or more maximum assignments, there is an additional fee. For more information, see <a href="https://requester.mturk.com/pricing">Amazon Mechanical Turk Pricing</a>.  </note>

        :param self:
        :param HITLayoutParameters:   If the HITLayoutId is provided, any placeholder values must be filled in with values using the HITLayoutParameter structure. For more information, see HITLayout. <br/>HITLayoutId:   The HITLayoutId allows you to use a pre-existing HIT design with placeholder values and create an additional HIT by providing those values as HITLayoutParameters.   Constraints: Either a Question parameter or a HITLayoutId parameter must be provided. <br/>HITReviewPolicy:   The HIT-level Review Policy applies to the HIT. You can specify for Mechanical Turk to take various actions based on the policy. :<br/>AssignmentReviewPolicy:   The Assignment-level Review Policy applies to the assignments under the HIT. You can specify for Mechanical Turk to take various actions based on the policy. :<br/>UniqueRequestToken:   A unique identifier for this request which allows you to retry the call on error without creating duplicate HITs. This is useful in cases such as network timeouts where it is unclear whether or not the call succeeded on the server. If the HIT already exists in the system from a previous call using the same UniqueRequestToken, subsequent calls will return a AWS.MechanicalTurk.HitAlreadyExists error with a message containing the HITId.  <note>  Note: It is your responsibility to ensure uniqueness of the token. The unique token expires after 24 hours. Subsequent calls using the same UniqueRequestToken made after the 24 hour limit could create duplicate HITs.  </note><br/>RequesterAnnotation:   An arbitrary data field. The RequesterAnnotation parameter lets your application attach arbitrary data to the HIT for tracking purposes. For example, this parameter could be an identifier internal to the Requester's application that corresponds with the HIT.   The RequesterAnnotation parameter for a HIT is only visible to the Requester who created the HIT. It is not shown to the Worker, or any other Requester.   The RequesterAnnotation parameter may be different for each HIT you submit. It does not affect how your HITs are grouped. <br/>Question:   The data the person completing the HIT uses to produce the results.   Constraints: Must be a QuestionForm data structure, an ExternalQuestion data structure, or an HTMLQuestion data structure. The XML question data must not be larger than 64 kilobytes (65,535 bytes) in size, including whitespace.  Either a Question parameter or a HITLayoutId parameter must be provided.<br/>LifetimeInSeconds:   An amount of time, in seconds, after which the HIT is no longer available for users to accept. After the lifetime of the HIT elapses, the HIT no longer appears in HIT searches, even if not all of the assignments for the HIT have been accepted. <br/>MaxAssignments:   The number of times the HIT can be accepted and completed before the HIT becomes unavailable. <br/>HITTypeId:  The HIT type ID you want to create this HIT with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> Name of the parameter from the Review policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyName</i>  Name of a Review Policy: SimplePlurality/2011-09-01 or ScoreMyKnownAnswers/2011-09-01 <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> Name of the parameter from the Review policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyName</i>  Name of a Review Policy: SimplePlurality/2011-09-01 or ScoreMyKnownAnswers/2011-09-01 <br/>
        :return: HIT:   Contains the newly created HIT data. For a description of the HIT data structure as it appears in responses, see the HIT Data Structure documentation. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfAssignmentsCompleted</i>  The number of assignments for this HIT that have been approved or rejected.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfAssignmentsAvailable</i>  The number of assignments for this HIT that are available for Workers to accept.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfAssignmentsPending</i>  The number of assignments for this HIT that are being previewed or have been accepted by Workers, but have not yet been submitted, returned, or abandoned.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HITReviewStatus</i>  Indicates the review status of the HIT. Valid Values are NotReviewed | MarkedForReview | ReviewedAppropriate | ReviewedInappropriate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QualificationRequirements</i>  Conditions that a Worker's Qualifications must meet in order to accept the HIT. A HIT can have between zero and ten Qualification requirements. All requirements must be met in order for a Worker to accept the HIT. Additionally, other actions can be restricted using the <code>ActionsGuarded</code> field on each <code>QualificationRequirement</code> structure. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequesterAnnotation</i>  An arbitrary data field the Requester who created the HIT can use. This field is visible only to the creator of the HIT.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssignmentDurationInSeconds</i>  The length of time, in seconds, that a Worker has to complete the HIT after accepting it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Expiration</i> The date and time the HIT expires.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoApprovalDelayInSeconds</i> The amount of time, in seconds, after the Worker submits an assignment for the HIT that the results are automatically approved by Amazon Mechanical Turk. This is the amount of time the Requester has to reject an assignment submitted by a Worker before the assignment is auto-approved and the Worker is paid. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Reward</i> A string representing a currency amount.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxAssignments</i> The number of times the HIT can be accepted and completed before the HIT becomes unavailable. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HITStatus</i> The status of the HIT and its assignments. Valid Values are Assignable | Unassignable | Reviewable | Reviewing | Disposed. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Keywords</i>  One or more words or phrases that describe the HIT, separated by commas. Search terms similar to the keywords of a HIT are more likely to have the HIT in the search results.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Question</i>  The data the Worker completing the HIT uses produce the results. This is either either a QuestionForm, HTMLQuestion or an ExternalQuestion data structure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i>  A general description of the HIT.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Title</i>  The title of the HIT.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i>  The date and time the HIT was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HITLayoutId</i>  The ID of the HIT Layout of this HIT.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HITGroupId</i>  The ID of the HIT Group of this HIT.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HITTypeId</i> The ID of the HIT type of this HIT<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HITId</i>  A unique identifier for the HIT.<br/>        """
        pass

    def create_qualification_type(self, QualificationTypeStatus: str, Description: str, Name: str, Keywords: str=None, RetryDelayInSeconds: Long=None, Test: str=None, AnswerKey: str=None, TestDurationInSeconds: Long=None, AutoGranted: Boolean=None, AutoGrantedValue: int=None) -> CreateQualificationTypeResponse:
        """ The <code>CreateQualificationType</code> operation creates a new Qualification type, which is represented by a <code>QualificationType</code> data structure. 

        :param self:
        :param AutoGrantedValue:  The Qualification value to use for automatically granted Qualifications. This parameter is used only if the AutoGranted parameter is true.<br/>AutoGranted:  Specifies whether requests for the Qualification type are granted immediately, without prompting the Worker with a Qualification test. Constraints: If the Test parameter is specified, this parameter cannot be true.<br/>TestDurationInSeconds:  The number of seconds the Worker has to complete the Qualification test, starting from the time the Worker requests the Qualification.<br/>AnswerKey:  The answers to the Qualification test specified in the Test parameter, in the form of an AnswerKey data structure. Constraints: Must not be longer than 65535 bytes. Constraints: None. If not specified, you must process Qualification requests manually.<br/>Test:   The questions for the Qualification test a Worker must answer correctly to obtain a Qualification of this type. If this parameter is specified, <code>TestDurationInSeconds</code> must also be specified.  Constraints: Must not be longer than 65535 bytes. Must be a QuestionForm data structure. This parameter cannot be specified if AutoGranted is true. Constraints: None. If not specified, the Worker may request the Qualification without answering any questions.<br/>RetryDelayInSeconds:  The number of seconds that a Worker must wait after requesting a Qualification of the Qualification type before the worker can retry the Qualification request. Constraints: None. If not specified, retries are disabled and Workers can request a Qualification of this type only once, even if the Worker has not been granted the Qualification. It is not possible to disable retries for a Qualification type after it has been created with retries enabled. If you want to disable retries, you must delete existing retry-enabled Qualification type and then create a new Qualification type with retries disabled.<br/>QualificationTypeStatus:  The initial status of the Qualification type. Constraints: Valid values are: Active | Inactive<br/>Description:  A long description for the Qualification type. On the Amazon Mechanical Turk website, the long description is displayed when a Worker examines a Qualification type.<br/>Keywords:  One or more words or phrases that describe the Qualification type, separated by commas. The keywords of a type make the type easier to find during a search.<br/>Name:   The name you give to the Qualification type. The type name is used to represent the Qualification to Workers, and to find the type using a Qualification type search. It must be unique across all of your Qualification types.<br/>
        :return: QualificationType:  The created Qualification type, returned as a QualificationType data structure.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoGrantedValue</i>  The Qualification integer value to use for automatically granted Qualifications, if AutoGranted is true. This is 1 by default. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoGranted</i> Specifies that requests for the Qualification type are granted immediately, without prompting the Worker with a Qualification test. Valid values are True | False.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsRequestable</i>  Specifies whether the Qualification type is one that a user can request through the Amazon Mechanical Turk web site, such as by taking a Qualification test. This value is False for Qualifications assigned automatically by the system. Valid values are True | False. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RetryDelayInSeconds</i>  The amount of time, in seconds, Workers must wait after taking the Qualification test before they can take it again. Workers can take a Qualification test multiple times if they were not granted the Qualification from a previous attempt, or if the test offers a gradient score and they want a better score. If not specified, retries are disabled and Workers can request a Qualification only once. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AnswerKey</i> The answers to the Qualification test specified in the Test parameter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TestDurationInSeconds</i>  The amount of time, in seconds, given to a Worker to complete the Qualification test, beginning from the time the Worker requests the Qualification. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Test</i>  The questions for a Qualification test associated with this Qualification type that a user can take to obtain a Qualification of this type. This parameter must be specified if AnswerKey is present. A Qualification type cannot have both a specified Test parameter and an AutoGranted value of true. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QualificationTypeStatus</i>  The status of the Qualification type. A Qualification type's status determines if users can apply to receive a Qualification of this type, and if HITs can be created with requirements based on this type. Valid values are Active | Inactive. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Keywords</i>  One or more words or phrases that describe theQualification type, separated by commas. The Keywords make the type easier to find using a search. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i>  A long description for the Qualification type. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i>  The name of the Qualification type. The type name is used to identify the type, and to find the type using a Qualification type search. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i>  The date and time the Qualification type was created. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QualificationTypeId</i>  A unique identifier for the Qualification type. A Qualification type is given a Qualification type ID when you call the CreateQualificationType operation. <br/>        """
        pass

    def create_worker_block(self, Reason: str, WorkerId: str) -> CreateWorkerBlockResponse:
        """The <code>CreateWorkerBlock</code> operation allows you to prevent a Worker from working on your HITs. For example, you can block a Worker who is producing poor quality work. You can block up to 100,000 Workers.

        :param self:
        :param Reason:  A message explaining the reason for blocking the Worker. This parameter enables you to keep track of your Workers. The Worker does not see this message.<br/>WorkerId:  The ID of the Worker to block.<br/>
        :return:         """
        pass

    def delete_hit(self, HITId: str) -> DeleteHITResponse:
        """ The <code>DeleteHIT</code> operation is used to delete HIT that is no longer needed. Only the Requester who created the HIT can delete it.   You can only dispose of HITs that are in the <code>Reviewable</code> state, with all of their submitted assignments already either approved or rejected. If you call the DeleteHIT operation on a HIT that is not in the <code>Reviewable</code> state (for example, that has not expired, or still has active assignments), or on a HIT that is Reviewable but without all of its submitted assignments already approved or rejected, the service will return an error.  <note> <ul> <li>  HITs are automatically disposed of after 120 days.  </li> <li>  After you dispose of a HIT, you can no longer approve the HIT's rejected assignments.  </li> <li>  Disposed HITs are not returned in results for the ListHITs operation.  </li> <li>  Disposing HITs can improve the performance of operations such as ListReviewableHITs and ListHITs.  </li> </ul> </note>

        :param self:
        :param HITId:  The ID of the HIT to be deleted.<br/>
        :return:         """
        pass

    def delete_qualification_type(self, QualificationTypeId: str) -> DeleteQualificationTypeResponse:
        """ The <code>DeleteQualificationType</code> deletes a Qualification type and deletes any HIT types that are associated with the Qualification type.  This operation does not revoke Qualifications already assigned to Workers because the Qualifications might be needed for active HITs. If there are any pending requests for the Qualification type, Amazon Mechanical Turk rejects those requests. After you delete a Qualification type, you can no longer use it to create HITs or HIT types. <note> DeleteQualificationType must wait for all the HITs that use the deleted Qualification type to be deleted before completing. It may take up to 48 hours before DeleteQualificationType completes and the unique name of the Qualification type is available for reuse with CreateQualificationType. </note>

        :param self:
        :param QualificationTypeId:  The ID of the QualificationType to dispose.<br/>
        :return:         """
        pass

    def delete_worker_block(self, WorkerId: str, Reason: str=None) -> DeleteWorkerBlockResponse:
        """The <code>DeleteWorkerBlock</code> operation allows you to reinstate a blocked Worker to work on your HITs. This operation reverses the effects of the CreateWorkerBlock operation. You need the Worker ID to use this operation. If the Worker ID is missing or invalid, this operation fails and returns the message “WorkerId is invalid.” If the specified Worker is not blocked, this operation returns successfully.

        :param self:
        :param Reason:  A message that explains the reason for unblocking the Worker. The Worker does not see this message.<br/>WorkerId:  The ID of the Worker to unblock.<br/>
        :return:         """
        pass

    def disassociate_qualification_from_worker(self, QualificationTypeId: str, WorkerId: str, Reason: str=None) -> DisassociateQualificationFromWorkerResponse:
        """ The <code>DisassociateQualificationFromWorker</code> revokes a previously granted Qualification from a user.   You can provide a text message explaining why the Qualification was revoked. The user who had the Qualification can see this message. 

        :param self:
        :param Reason:  A text message that explains why the Qualification was revoked. The user who had the Qualification sees this message.<br/>QualificationTypeId:  The ID of the Qualification type of the Qualification to be revoked.<br/>WorkerId:  The ID of the Worker who possesses the Qualification to be revoked.<br/>
        :return:         """
        pass

    def get_account_balance(self) -> GetAccountBalanceResponse:
        """The <code>GetAccountBalance</code> operation retrieves the amount of money in your Amazon Mechanical Turk account.

        :param self:
        :param 
        :return: OnHoldBalance:  A string representing a currency amount.<br/>AvailableBalance:  A string representing a currency amount.<br/>        """
        pass

    def get_assignment(self, AssignmentId: str) -> GetAssignmentResponse:
        """ The <code>GetAssignment</code> operation retrieves the details of the specified Assignment. 

        :param self:
        :param AssignmentId:  The ID of the Assignment to be retrieved.<br/>
        :return: HIT:   The HIT associated with this assignment. The response includes one HIT element.:<br/>Assignment:   The assignment. The response includes one Assignment element. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequesterFeedback</i>  The feedback string included with the call to the ApproveAssignment operation or the RejectAssignment operation, if the Requester approved or rejected the assignment and specified feedback.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Answer</i>  The Worker's answers submitted for the HIT contained in a QuestionFormAnswers document, if the Worker provides an answer. If the Worker does not provide any answers, Answer may contain a QuestionFormAnswers document, or Answer may be empty.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Deadline</i>  The date and time of the deadline for the assignment. This value is derived from the deadline specification for the HIT and the date and time the Worker accepted the HIT.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RejectionTime</i>  If the Worker has submitted results and the Requester has rejected the results, RejectionTime is the date and time the Requester rejected the results.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApprovalTime</i>  If the Worker has submitted results and the Requester has approved the results, ApprovalTime is the date and time the Requester approved the results. This value is omitted from the assignment if the Requester has not yet approved the results.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmitTime</i>  If the Worker has submitted results, SubmitTime is the date and time the assignment was submitted. This value is omitted from the assignment if the Worker has not yet submitted results.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AcceptTime</i>  The date and time the Worker accepted the assignment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoApprovalTime</i>  If results have been submitted, AutoApprovalTime is the date and time the results of the assignment results are considered Approved automatically if they have not already been explicitly approved or rejected by the Requester. This value is derived from the auto-approval delay specified by the Requester in the HIT. This value is omitted from the assignment if the Worker has not yet submitted results.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssignmentStatus</i>  The status of the assignment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HITId</i>  The ID of the HIT.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WorkerId</i>  The ID of the Worker who accepted the HIT.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssignmentId</i>  A unique identifier for the assignment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfAssignmentsCompleted</i>  The number of assignments for this HIT that have been approved or rejected.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfAssignmentsAvailable</i>  The number of assignments for this HIT that are available for Workers to accept.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfAssignmentsPending</i>  The number of assignments for this HIT that are being previewed or have been accepted by Workers, but have not yet been submitted, returned, or abandoned.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HITReviewStatus</i>  Indicates the review status of the HIT. Valid Values are NotReviewed | MarkedForReview | ReviewedAppropriate | ReviewedInappropriate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QualificationRequirements</i>  Conditions that a Worker's Qualifications must meet in order to accept the HIT. A HIT can have between zero and ten Qualification requirements. All requirements must be met in order for a Worker to accept the HIT. Additionally, other actions can be restricted using the <code>ActionsGuarded</code> field on each <code>QualificationRequirement</code> structure. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequesterAnnotation</i>  An arbitrary data field the Requester who created the HIT can use. This field is visible only to the creator of the HIT.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssignmentDurationInSeconds</i>  The length of time, in seconds, that a Worker has to complete the HIT after accepting it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Expiration</i> The date and time the HIT expires.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoApprovalDelayInSeconds</i> The amount of time, in seconds, after the Worker submits an assignment for the HIT that the results are automatically approved by Amazon Mechanical Turk. This is the amount of time the Requester has to reject an assignment submitted by a Worker before the assignment is auto-approved and the Worker is paid. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Reward</i> A string representing a currency amount.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxAssignments</i> The number of times the HIT can be accepted and completed before the HIT becomes unavailable. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HITStatus</i> The status of the HIT and its assignments. Valid Values are Assignable | Unassignable | Reviewable | Reviewing | Disposed. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Keywords</i>  One or more words or phrases that describe the HIT, separated by commas. Search terms similar to the keywords of a HIT are more likely to have the HIT in the search results.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Question</i>  The data the Worker completing the HIT uses produce the results. This is either either a QuestionForm, HTMLQuestion or an ExternalQuestion data structure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i>  A general description of the HIT.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Title</i>  The title of the HIT.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i>  The date and time the HIT was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HITLayoutId</i>  The ID of the HIT Layout of this HIT.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HITGroupId</i>  The ID of the HIT Group of this HIT.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HITTypeId</i> The ID of the HIT type of this HIT<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HITId</i>  A unique identifier for the HIT.<br/>        """
        pass

    def get_file_upload_url(self, QuestionIdentifier: str, AssignmentId: str) -> GetFileUploadURLResponse:
        """ The <code>GetFileUploadURL</code> operation generates and returns a temporary URL. You use the temporary URL to retrieve a file uploaded by a Worker as an answer to a FileUploadAnswer question for a HIT. The temporary URL is generated the instant the GetFileUploadURL operation is called, and is valid for 60 seconds. You can get a temporary file upload URL any time until the HIT is disposed. After the HIT is disposed, any uploaded files are deleted, and cannot be retrieved. Pending Deprecation on December 12, 2017. The Answer Specification structure will no longer support the <code>FileUploadAnswer</code> element to be used for the QuestionForm data structure. Instead, we recommend that Requesters who want to create HITs asking Workers to upload files to use Amazon S3. 

        :param self:
        :param QuestionIdentifier:  The identifier of the question with a FileUploadAnswer, as specified in the QuestionForm of the HIT.<br/>AssignmentId:  The ID of the assignment that contains the question with a FileUploadAnswer.<br/>
        :return: FileUploadURL:   A temporary URL for the file that the Worker uploaded for the answer. <br/>        """
        pass

    def get_hit(self, HITId: str) -> GetHITResponse:
        """ The <code>GetHIT</code> operation retrieves the details of the specified HIT. 

        :param self:
        :param HITId:  The ID of the HIT to be retrieved.<br/>
        :return: HIT:   Contains the requested HIT data.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfAssignmentsCompleted</i>  The number of assignments for this HIT that have been approved or rejected.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfAssignmentsAvailable</i>  The number of assignments for this HIT that are available for Workers to accept.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfAssignmentsPending</i>  The number of assignments for this HIT that are being previewed or have been accepted by Workers, but have not yet been submitted, returned, or abandoned.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HITReviewStatus</i>  Indicates the review status of the HIT. Valid Values are NotReviewed | MarkedForReview | ReviewedAppropriate | ReviewedInappropriate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QualificationRequirements</i>  Conditions that a Worker's Qualifications must meet in order to accept the HIT. A HIT can have between zero and ten Qualification requirements. All requirements must be met in order for a Worker to accept the HIT. Additionally, other actions can be restricted using the <code>ActionsGuarded</code> field on each <code>QualificationRequirement</code> structure. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequesterAnnotation</i>  An arbitrary data field the Requester who created the HIT can use. This field is visible only to the creator of the HIT.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssignmentDurationInSeconds</i>  The length of time, in seconds, that a Worker has to complete the HIT after accepting it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Expiration</i> The date and time the HIT expires.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoApprovalDelayInSeconds</i> The amount of time, in seconds, after the Worker submits an assignment for the HIT that the results are automatically approved by Amazon Mechanical Turk. This is the amount of time the Requester has to reject an assignment submitted by a Worker before the assignment is auto-approved and the Worker is paid. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Reward</i> A string representing a currency amount.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxAssignments</i> The number of times the HIT can be accepted and completed before the HIT becomes unavailable. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HITStatus</i> The status of the HIT and its assignments. Valid Values are Assignable | Unassignable | Reviewable | Reviewing | Disposed. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Keywords</i>  One or more words or phrases that describe the HIT, separated by commas. Search terms similar to the keywords of a HIT are more likely to have the HIT in the search results.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Question</i>  The data the Worker completing the HIT uses produce the results. This is either either a QuestionForm, HTMLQuestion or an ExternalQuestion data structure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i>  A general description of the HIT.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Title</i>  The title of the HIT.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i>  The date and time the HIT was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HITLayoutId</i>  The ID of the HIT Layout of this HIT.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HITGroupId</i>  The ID of the HIT Group of this HIT.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HITTypeId</i> The ID of the HIT type of this HIT<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HITId</i>  A unique identifier for the HIT.<br/>        """
        pass

    def get_qualification_score(self, WorkerId: str, QualificationTypeId: str) -> GetQualificationScoreResponse:
        """ The <code>GetQualificationScore</code> operation returns the value of a Worker's Qualification for a given Qualification type.   To get a Worker's Qualification, you must know the Worker's ID. The Worker's ID is included in the assignment data returned by the <code>ListAssignmentsForHIT</code> operation.  Only the owner of a Qualification type can query the value of a Worker's Qualification of that type.

        :param self:
        :param WorkerId:  The ID of the Worker whose Qualification is being updated.<br/>QualificationTypeId:  The ID of the QualificationType.<br/>
        :return: Qualification:   The Qualification data structure of the Qualification assigned to a user, including the Qualification type and the value (score). :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i>  The status of the Qualification. Valid values are Granted | Revoked.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LocaleValue</i> The Locale data structure represents a geographical region or location.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LocaleValue</b>: The Locale data structure represents a geographical region or location.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IntegerValue</i>  The value (score) of the Qualification, if the Qualification has an integer value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GrantTime</i>  The date and time the Qualification was granted to the Worker. If the Worker's Qualification was revoked, and then re-granted based on a new Qualification request, GrantTime is the date and time of the last call to the AcceptQualificationRequest operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WorkerId</i>  The ID of the Worker who possesses the Qualification. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QualificationTypeId</i>  The ID of the Qualification type for the Qualification.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subdivision</i> The state or subdivision of the locale. A valid ISO 3166-2 subdivision code. For example, the code WA refers to the state of Washington.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Country</i>  The country of the locale. Must be a valid ISO 3166 country code. For example, the code US refers to the United States of America. <br/>        """
        pass

    def get_qualification_type(self, QualificationTypeId: str) -> GetQualificationTypeResponse:
        """ The <code>GetQualificationType</code>operation retrieves information about a Qualification type using its ID. 

        :param self:
        :param QualificationTypeId:  The ID of the QualificationType.<br/>
        :return: QualificationType:   The returned Qualification Type:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoGrantedValue</i>  The Qualification integer value to use for automatically granted Qualifications, if AutoGranted is true. This is 1 by default. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoGranted</i> Specifies that requests for the Qualification type are granted immediately, without prompting the Worker with a Qualification test. Valid values are True | False.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsRequestable</i>  Specifies whether the Qualification type is one that a user can request through the Amazon Mechanical Turk web site, such as by taking a Qualification test. This value is False for Qualifications assigned automatically by the system. Valid values are True | False. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RetryDelayInSeconds</i>  The amount of time, in seconds, Workers must wait after taking the Qualification test before they can take it again. Workers can take a Qualification test multiple times if they were not granted the Qualification from a previous attempt, or if the test offers a gradient score and they want a better score. If not specified, retries are disabled and Workers can request a Qualification only once. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AnswerKey</i> The answers to the Qualification test specified in the Test parameter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TestDurationInSeconds</i>  The amount of time, in seconds, given to a Worker to complete the Qualification test, beginning from the time the Worker requests the Qualification. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Test</i>  The questions for a Qualification test associated with this Qualification type that a user can take to obtain a Qualification of this type. This parameter must be specified if AnswerKey is present. A Qualification type cannot have both a specified Test parameter and an AutoGranted value of true. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QualificationTypeStatus</i>  The status of the Qualification type. A Qualification type's status determines if users can apply to receive a Qualification of this type, and if HITs can be created with requirements based on this type. Valid values are Active | Inactive. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Keywords</i>  One or more words or phrases that describe theQualification type, separated by commas. The Keywords make the type easier to find using a search. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i>  A long description for the Qualification type. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i>  The name of the Qualification type. The type name is used to identify the type, and to find the type using a Qualification type search. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i>  The date and time the Qualification type was created. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QualificationTypeId</i>  A unique identifier for the Qualification type. A Qualification type is given a Qualification type ID when you call the CreateQualificationType operation. <br/>        """
        pass

    def list_assignments_for_hit(self, HITId: str, NextToken: str=None, MaxResults: int=None, AssignmentStatuses: List[AssignmentStatus]=None) -> ListAssignmentsForHITResponse:
        """ The <code>ListAssignmentsForHIT</code> operation retrieves completed assignments for a HIT. You can use this operation to retrieve the results for a HIT.   You can get assignments for a HIT at any time, even if the HIT is not yet Reviewable. If a HIT requested multiple assignments, and has received some results but has not yet become Reviewable, you can still retrieve the partial results with this operation.   Use the AssignmentStatus parameter to control which set of assignments for a HIT are returned. The ListAssignmentsForHIT operation can return submitted assignments awaiting approval, or it can return assignments that have already been approved or rejected. You can set AssignmentStatus=Approved,Rejected to get assignments that have already been approved and rejected together in one result set.   Only the Requester who created the HIT can retrieve the assignments for that HIT.   Results are sorted and divided into numbered pages and the operation returns a single page of results. You can use the parameters of the operation to control sorting and pagination. 

        :param self:
        :param AssignmentStatuses:  The status of the assignments to return: Submitted | Approved | Rejected<br/>MaxResults:  <br/>NextToken:  Pagination token<br/>HITId:  The ID of the HIT.<br/>
        :return: Assignments:   The collection of Assignment data structures returned by this call.<br/>NumResults:   The number of assignments on the page in the filtered results list, equivalent to the number of assignments returned by this call.<br/>NextToken:  If the previous response was incomplete (because there is more data to retrieve), Amazon Mechanical Turk returns a pagination token in the response. You can use this pagination token to retrieve the next set of results. <br/>        """
        pass

    def list_bonus_payments(self, HITId: str=None, AssignmentId: str=None, NextToken: str=None, MaxResults: int=None) -> ListBonusPaymentsResponse:
        """ The <code>ListBonusPayments</code> operation retrieves the amounts of bonuses you have paid to Workers for a given HIT or assignment. 

        :param self:
        :param MaxResults:  <br/>NextToken:  Pagination token<br/>AssignmentId:  The ID of the assignment associated with the bonus payments to retrieve. If specified, only bonus payments for the given assignment are returned. Either the HITId parameter or the AssignmentId parameter must be specified<br/>HITId:  The ID of the HIT associated with the bonus payments to retrieve. If not specified, all bonus payments for all assignments for the given HIT are returned. Either the HITId parameter or the AssignmentId parameter must be specified<br/>
        :return: BonusPayments:  A successful request to the ListBonusPayments operation returns a list of BonusPayment objects. <br/>NextToken:  If the previous response was incomplete (because there is more data to retrieve), Amazon Mechanical Turk returns a pagination token in the response. You can use this pagination token to retrieve the next set of results. <br/>NumResults:  The number of bonus payments on this page in the filtered results list, equivalent to the number of bonus payments being returned by this call. <br/>        """
        pass

    def list_hits(self, NextToken: str=None, MaxResults: int=None) -> ListHITsResponse:
        """ The <code>ListHITs</code> operation returns all of a Requester's HITs. The operation returns HITs of any status, except for HITs that have been deleted of with the DeleteHIT operation or that have been auto-deleted. 

        :param self:
        :param MaxResults:  <br/>NextToken:  Pagination token<br/>
        :return: HITs:   The list of HIT elements returned by the query.<br/>NumResults:  The number of HITs on this page in the filtered results list, equivalent to the number of HITs being returned by this call.<br/>NextToken:  If the previous response was incomplete (because there is more data to retrieve), Amazon Mechanical Turk returns a pagination token in the response. You can use this pagination token to retrieve the next set of results. <br/>        """
        pass

    def list_hits_for_qualification_type(self, QualificationTypeId: str, NextToken: str=None, MaxResults: int=None) -> ListHITsForQualificationTypeResponse:
        """ The <code>ListHITsForQualificationType</code> operation returns the HITs that use the given Qualification type for a Qualification requirement. The operation returns HITs of any status, except for HITs that have been deleted with the <code>DeleteHIT</code> operation or that have been auto-deleted. 

        :param self:
        :param MaxResults:   Limit the number of results returned. <br/>NextToken:  Pagination Token<br/>QualificationTypeId:   The ID of the Qualification type to use when querying HITs. <br/>
        :return: HITs:   The list of HIT elements returned by the query.<br/>NumResults:   The number of HITs on this page in the filtered results list, equivalent to the number of HITs being returned by this call. <br/>NextToken:  If the previous response was incomplete (because there is more data to retrieve), Amazon Mechanical Turk returns a pagination token in the response. You can use this pagination token to retrieve the next set of results. <br/>        """
        pass

    def list_qualification_requests(self, QualificationTypeId: str=None, NextToken: str=None, MaxResults: int=None) -> ListQualificationRequestsResponse:
        """ The <code>ListQualificationRequests</code> operation retrieves requests for Qualifications of a particular Qualification type. The owner of the Qualification type calls this operation to poll for pending requests, and accepts them using the AcceptQualification operation. 

        :param self:
        :param MaxResults:   The maximum number of results to return in a single call. <br/>NextToken:  If the previous response was incomplete (because there is more data to retrieve), Amazon Mechanical Turk returns a pagination token in the response. You can use this pagination token to retrieve the next set of results. <br/>QualificationTypeId:  The ID of the QualificationType.<br/>
        :return: QualificationRequests:  The Qualification request. The response includes one QualificationRequest element for each Qualification request returned by the query.<br/>NextToken:  If the previous response was incomplete (because there is more data to retrieve), Amazon Mechanical Turk returns a pagination token in the response. You can use this pagination token to retrieve the next set of results. <br/>NumResults:  The number of Qualification requests on this page in the filtered results list, equivalent to the number of Qualification requests being returned by this call.<br/>        """
        pass

    def list_qualification_types(self, MustBeRequestable: Boolean, Query: str=None, MustBeOwnedByCaller: Boolean=None, NextToken: str=None, MaxResults: int=None) -> ListQualificationTypesResponse:
        """ The <code>ListQualificationTypes</code> operation returns a list of Qualification types, filtered by an optional search term. 

        :param self:
        :param MaxResults:   The maximum number of results to return in a single call. <br/>NextToken:  If the previous response was incomplete (because there is more data to retrieve), Amazon Mechanical Turk returns a pagination token in the response. You can use this pagination token to retrieve the next set of results. <br/>MustBeOwnedByCaller:   Specifies that only Qualification types that the Requester created are returned. If false, the operation returns all Qualification types. <br/>MustBeRequestable:  Specifies that only Qualification types that a user can request through the Amazon Mechanical Turk web site, such as by taking a Qualification test, are returned as results of the search. Some Qualification types, such as those assigned automatically by the system, cannot be requested directly by users. If false, all Qualification types, including those managed by the system, are considered. Valid values are True | False. <br/>Query:   A text query against all of the searchable attributes of Qualification types. <br/>
        :return: QualificationTypes:   The list of QualificationType elements returned by the query. <br/>NextToken:  If the previous response was incomplete (because there is more data to retrieve), Amazon Mechanical Turk returns a pagination token in the response. You can use this pagination token to retrieve the next set of results. <br/>NumResults:   The number of Qualification types on this page in the filtered results list, equivalent to the number of types this operation returns. <br/>        """
        pass

    def list_review_policy_results_for_hit(self, HITId: str, PolicyLevels: List[ReviewPolicyLevel]=None, RetrieveActions: Boolean=None, RetrieveResults: Boolean=None, NextToken: str=None, MaxResults: int=None) -> ListReviewPolicyResultsForHITResponse:
        """ The <code>ListReviewPolicyResultsForHIT</code> operation retrieves the computed results and the actions taken in the course of executing your Review Policies for a given HIT. For information about how to specify Review Policies when you call CreateHIT, see Review Policies. The ListReviewPolicyResultsForHIT operation can return results for both Assignment-level and HIT-level review results. 

        :param self:
        :param MaxResults:  Limit the number of results returned.<br/>NextToken:  Pagination token<br/>RetrieveResults:   Specify if the operation should retrieve a list of the results computed by the Review Policies. <br/>RetrieveActions:   Specify if the operation should retrieve a list of the actions taken executing the Review Policies and their outcomes. <br/>PolicyLevels:   The Policy Level(s) to retrieve review results for - HIT or Assignment. If omitted, the default behavior is to retrieve all data for both policy levels. For a list of all the described policies, see Review Policies. <br/>HITId:  The unique identifier of the HIT to retrieve review results for.<br/>
        :return: NextToken:  If the previous response was incomplete (because there is more data to retrieve), Amazon Mechanical Turk returns a pagination token in the response. You can use this pagination token to retrieve the next set of results. <br/>HITReviewReport:  Contains both ReviewResult and ReviewAction elements for a particular HIT. :<br/>AssignmentReviewReport:   Contains both ReviewResult and ReviewAction elements for an Assignment. :<br/>HITReviewPolicy:  The name of the HIT-level Review Policy. This contains only the PolicyName element.:<br/>AssignmentReviewPolicy:   The name of the Assignment-level Review Policy. This contains only the PolicyName element. :<br/>HITId:  The HITId of the HIT for which results have been returned.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> Name of the parameter from the Review policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyName</i>  Name of a Review Policy: SimplePlurality/2011-09-01 or ScoreMyKnownAnswers/2011-09-01 <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> Name of the parameter from the Review policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyName</i>  Name of a Review Policy: SimplePlurality/2011-09-01 or ScoreMyKnownAnswers/2011-09-01 <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReviewActions</i>  A list of ReviewAction objects for each action specified in the Review Policy. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReviewResults</i>  A list of ReviewResults objects for each action specified in the Review Policy. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReviewActions</i>  A list of ReviewAction objects for each action specified in the Review Policy. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReviewResults</i>  A list of ReviewResults objects for each action specified in the Review Policy. <br/>        """
        pass

    def list_reviewable_hits(self, HITTypeId: str=None, Status: str=None, NextToken: str=None, MaxResults: int=None) -> ListReviewableHITsResponse:
        """ The <code>ListReviewableHITs</code> operation retrieves the HITs with Status equal to Reviewable or Status equal to Reviewing that belong to the Requester calling the operation. 

        :param self:
        :param MaxResults:   Limit the number of results returned. <br/>NextToken:  Pagination Token<br/>Status:   Can be either <code>Reviewable</code> or <code>Reviewing</code>. Reviewable is the default value. <br/>HITTypeId:   The ID of the HIT type of the HITs to consider for the query. If not specified, all HITs for the Reviewer are considered <br/>
        :return: HITs:   The list of HIT elements returned by the query.<br/>NumResults:   The number of HITs on this page in the filtered results list, equivalent to the number of HITs being returned by this call. <br/>NextToken:  If the previous response was incomplete (because there is more data to retrieve), Amazon Mechanical Turk returns a pagination token in the response. You can use this pagination token to retrieve the next set of results. <br/>        """
        pass

    def list_worker_blocks(self, NextToken: str=None, MaxResults: int=None) -> ListWorkerBlocksResponse:
        """The <code>ListWorkersBlocks</code> operation retrieves a list of Workers who are blocked from working on your HITs.

        :param self:
        :param MaxResults:  <br/>NextToken:  Pagination token<br/>
        :return: WorkerBlocks:   The list of WorkerBlocks, containing the collection of Worker IDs and reasons for blocking.<br/>NumResults:   The number of assignments on the page in the filtered results list, equivalent to the number of assignments returned by this call.<br/>NextToken:  If the previous response was incomplete (because there is more data to retrieve), Amazon Mechanical Turk returns a pagination token in the response. You can use this pagination token to retrieve the next set of results. <br/>        """
        pass

    def list_workers_with_qualification_type(self, QualificationTypeId: str, Status: str=None, NextToken: str=None, MaxResults: int=None) -> ListWorkersWithQualificationTypeResponse:
        """ The <code>ListWorkersWithQualificationType</code> operation returns all of the Workers that have been associated with a given Qualification type. 

        :param self:
        :param MaxResults:   Limit the number of results returned. <br/>NextToken:  Pagination Token<br/>Status:   The status of the Qualifications to return. Can be <code>Granted | Revoked</code>. <br/>QualificationTypeId:  The ID of the Qualification type of the Qualifications to return.<br/>
        :return: Qualifications:   The list of Qualification elements returned by this call. <br/>NumResults:   The number of Qualifications on this page in the filtered results list, equivalent to the number of Qualifications being returned by this call.<br/>NextToken:  If the previous response was incomplete (because there is more data to retrieve), Amazon Mechanical Turk returns a pagination token in the response. You can use this pagination token to retrieve the next set of results. <br/>        """
        pass

    def notify_workers(self, WorkerIds: List[CustomerId], MessageText: str, Subject: str) -> NotifyWorkersResponse:
        """ The <code>NotifyWorkers</code> operation sends an email to one or more Workers that you specify with the Worker ID. You can specify up to 100 Worker IDs to send the same message with a single call to the NotifyWorkers operation. The NotifyWorkers operation will send a notification email to a Worker only if you have previously approved or rejected work from the Worker. 

        :param self:
        :param WorkerIds:  A list of Worker IDs you wish to notify. You can notify upto 100 Workers at a time.<br/>MessageText:  The text of the email message to send. Can include up to 4,096 characters<br/>Subject:  The subject line of the email message to send. Can include up to 200 characters.<br/>
        :return: NotifyWorkersFailureStatuses:   When MTurk sends notifications to the list of Workers, it returns back any failures it encounters in this list of NotifyWorkersFailureStatus objects. <br/>        """
        pass

    def reject_assignment(self, RequesterFeedback: str, AssignmentId: str) -> RejectAssignmentResponse:
        """ The <code>RejectAssignment</code> operation rejects the results of a completed assignment.   You can include an optional feedback message with the rejection, which the Worker can see in the Status section of the web site. When you include a feedback message with the rejection, it helps the Worker understand why the assignment was rejected, and can improve the quality of the results the Worker submits in the future.   Only the Requester who created the HIT can reject an assignment for the HIT. 

        :param self:
        :param RequesterFeedback:   A message for the Worker, which the Worker can see in the Status section of the web site. <br/>AssignmentId:   The ID of the assignment. The assignment must correspond to a HIT created by the Requester. <br/>
        :return:         """
        pass

    def reject_qualification_request(self, QualificationRequestId: str, Reason: str=None) -> RejectQualificationRequestResponse:
        """ The <code>RejectQualificationRequest</code> operation rejects a user's request for a Qualification.   You can provide a text message explaining why the request was rejected. The Worker who made the request can see this message.

        :param self:
        :param Reason:  A text message explaining why the request was rejected, to be shown to the Worker who made the request.<br/>QualificationRequestId:   The ID of the Qualification request, as returned by the <code>ListQualificationRequests</code> operation. <br/>
        :return:         """
        pass

    def send_bonus(self, Reason: str, AssignmentId: str, BonusAmount: str, WorkerId: str, UniqueRequestToken: str=None) -> SendBonusResponse:
        """ The <code>SendBonus</code> operation issues a payment of money from your account to a Worker. This payment happens separately from the reward you pay to the Worker when you approve the Worker's assignment. The SendBonus operation requires the Worker's ID and the assignment ID as parameters to initiate payment of the bonus. You must include a message that explains the reason for the bonus payment, as the Worker may not be expecting the payment. Amazon Mechanical Turk collects a fee for bonus payments, similar to the HIT listing fee. This operation fails if your account does not have enough funds to pay for both the bonus and the fees. 

        :param self:
        :param UniqueRequestToken:  A unique identifier for this request, which allows you to retry the call on error without granting multiple bonuses. This is useful in cases such as network timeouts where it is unclear whether or not the call succeeded on the server. If the bonus already exists in the system from a previous call using the same UniqueRequestToken, subsequent calls will return an error with a message containing the request ID.<br/>Reason:  A message that explains the reason for the bonus payment. The Worker receiving the bonus can see this message.<br/>AssignmentId:  The ID of the assignment for which this bonus is paid.<br/>BonusAmount:   The Bonus amount is a US Dollar amount specified using a string (for example, "5" represents $5.00 USD and "101.42" represents $101.42 USD). Do not include currency symbols or currency codes. <br/>WorkerId:  The ID of the Worker being paid the bonus.<br/>
        :return:         """
        pass

    def send_test_event_notification(self, TestEventType: str, Notification: NotificationSpecification) -> SendTestEventNotificationResponse:
        """ The <code>SendTestEventNotification</code> operation causes Amazon Mechanical Turk to send a notification message as if a HIT event occurred, according to the provided notification specification. This allows you to test notifications without setting up notifications for a real HIT type and trying to trigger them using the website. When you call this operation, the service attempts to send the test notification immediately. 

        :param self:
        :param TestEventType:   The event to simulate to test the notification specification. This event is included in the test message even if the notification specification does not include the event type. The notification specification does not filter out the test event. <br/>Notification:   The notification specification to test. This value is identical to the value you would provide to the UpdateNotificationSettings operation when you establish the notification specification for a HIT type. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EventTypes</i>  The list of events that should cause notifications to be sent. Valid Values: AssignmentAccepted | AssignmentAbandoned | AssignmentReturned | AssignmentSubmitted | AssignmentRejected | AssignmentApproved | HITCreated | HITExtended | HITDisposed | HITReviewable | HITExpired | Ping. The Ping event is only valid for the SendTestEventNotification operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version of the Notification API to use. Valid value is 2006-05-05.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Transport</i>  The method Amazon Mechanical Turk uses to send the notification. Valid Values: Email | SQS | SNS. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Destination</i>  The target for notification messages. The Destination’s format is determined by the specified Transport:  <ul> <li> When Transport is Email, the Destination is your email address. </li> <li> When Transport is SQS, the Destination is your queue URL. </li> <li> When Transport is SNS, the Destination is the ARN of your topic. </li> </ul><br/>
        :return:         """
        pass

    def update_expiration_for_hit(self, ExpireAt: Timestamp, HITId: str) -> UpdateExpirationForHITResponse:
        """ The <code>UpdateExpirationForHIT</code> operation allows you update the expiration time of a HIT. If you update it to a time in the past, the HIT will be immediately expired. 

        :param self:
        :param ExpireAt:   The date and time at which you want the HIT to expire <br/>HITId:   The HIT to update. <br/>
        :return:         """
        pass

    def update_hit_review_status(self, HITId: str, Revert: Boolean=None) -> UpdateHITReviewStatusResponse:
        """ The <code>UpdateHITReviewStatus</code> operation updates the status of a HIT. If the status is Reviewable, this operation can update the status to Reviewing, or it can revert a Reviewing HIT back to the Reviewable status. 

        :param self:
        :param Revert:   Specifies how to update the HIT status. Default is <code>False</code>.  <ul> <li>  Setting this to false will only transition a HIT from <code>Reviewable</code> to <code>Reviewing</code>  </li> <li>  Setting this to true will only transition a HIT from <code>Reviewing</code> to <code>Reviewable</code>  </li> </ul><br/>HITId:   The ID of the HIT to update. <br/>
        :return:         """
        pass

    def update_hit_type_of_hit(self, HITTypeId: str, HITId: str) -> UpdateHITTypeOfHITResponse:
        """ The <code>UpdateHITTypeOfHIT</code> operation allows you to change the HITType properties of a HIT. This operation disassociates the HIT from its old HITType properties and associates it with the new HITType properties. The HIT takes on the properties of the new HITType in place of the old ones. 

        :param self:
        :param HITTypeId:  The ID of the new HIT type.<br/>HITId:  The HIT to update.<br/>
        :return:         """
        pass

    def update_notification_settings(self, HITTypeId: str, Notification: NotificationSpecification=None, Active: Boolean=None) -> UpdateNotificationSettingsResponse:
        """ The <code>UpdateNotificationSettings</code> operation creates, updates, disables or re-enables notifications for a HIT type. If you call the UpdateNotificationSettings operation for a HIT type that already has a notification specification, the operation replaces the old specification with a new one. You can call the UpdateNotificationSettings operation to enable or disable notifications for the HIT type, without having to modify the notification specification itself by providing updates to the Active status without specifying a new notification specification. To change the Active status of a HIT type's notifications, the HIT type must already have a notification specification, or one must be provided in the same call to <code>UpdateNotificationSettings</code>. 

        :param self:
        :param Active:   Specifies whether notifications are sent for HITs of this HIT type, according to the notification specification. You must specify either the Notification parameter or the Active parameter for the call to UpdateNotificationSettings to succeed. <br/>Notification:   The notification specification for the HIT type. :<br/>HITTypeId:   The ID of the HIT type whose notification specification is being updated. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EventTypes</i>  The list of events that should cause notifications to be sent. Valid Values: AssignmentAccepted | AssignmentAbandoned | AssignmentReturned | AssignmentSubmitted | AssignmentRejected | AssignmentApproved | HITCreated | HITExtended | HITDisposed | HITReviewable | HITExpired | Ping. The Ping event is only valid for the SendTestEventNotification operation. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version of the Notification API to use. Valid value is 2006-05-05.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Transport</i>  The method Amazon Mechanical Turk uses to send the notification. Valid Values: Email | SQS | SNS. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Destination</i>  The target for notification messages. The Destination’s format is determined by the specified Transport:  <ul> <li> When Transport is Email, the Destination is your email address. </li> <li> When Transport is SQS, the Destination is your queue URL. </li> <li> When Transport is SNS, the Destination is the ARN of your topic. </li> </ul><br/>
        :return:         """
        pass

    def update_qualification_type(self, QualificationTypeId: str, Description: str=None, QualificationTypeStatus: str=None, Test: str=None, AnswerKey: str=None, TestDurationInSeconds: Long=None, RetryDelayInSeconds: Long=None, AutoGranted: Boolean=None, AutoGrantedValue: int=None) -> UpdateQualificationTypeResponse:
        """ The <code>UpdateQualificationType</code> operation modifies the attributes of an existing Qualification type, which is represented by a QualificationType data structure. Only the owner of a Qualification type can modify its attributes.   Most attributes of a Qualification type can be changed after the type has been created. However, the Name and Keywords fields cannot be modified. The RetryDelayInSeconds parameter can be modified or added to change the delay or to enable retries, but RetryDelayInSeconds cannot be used to disable retries.   You can use this operation to update the test for a Qualification type. The test is updated based on the values specified for the Test, TestDurationInSeconds and AnswerKey parameters. All three parameters specify the updated test. If you are updating the test for a type, you must specify the Test and TestDurationInSeconds parameters. The AnswerKey parameter is optional; omitting it specifies that the updated test does not have an answer key.   If you omit the Test parameter, the test for the Qualification type is unchanged. There is no way to remove a test from a Qualification type that has one. If the type already has a test, you cannot update it to be AutoGranted. If the Qualification type does not have a test and one is provided by an update, the type will henceforth have a test.   If you want to update the test duration or answer key for an existing test without changing the questions, you must specify a Test parameter with the original questions, along with the updated values.   If you provide an updated Test but no AnswerKey, the new test will not have an answer key. Requests for such Qualifications must be granted manually.   You can also update the AutoGranted and AutoGrantedValue attributes of the Qualification type.

        :param self:
        :param AutoGrantedValue:  The Qualification value to use for automatically granted Qualifications. This parameter is used only if the AutoGranted parameter is true.<br/>AutoGranted:  Specifies whether requests for the Qualification type are granted immediately, without prompting the Worker with a Qualification test. Constraints: If the Test parameter is specified, this parameter cannot be true.<br/>RetryDelayInSeconds:  The amount of time, in seconds, that Workers must wait after requesting a Qualification of the specified Qualification type before they can retry the Qualification request. It is not possible to disable retries for a Qualification type after it has been created with retries enabled. If you want to disable retries, you must dispose of the existing retry-enabled Qualification type using DisposeQualificationType and then create a new Qualification type with retries disabled using CreateQualificationType.<br/>TestDurationInSeconds:  The number of seconds the Worker has to complete the Qualification test, starting from the time the Worker requests the Qualification.<br/>AnswerKey:  The answers to the Qualification test specified in the Test parameter, in the form of an AnswerKey data structure.<br/>Test:  The questions for the Qualification test a Worker must answer correctly to obtain a Qualification of this type. If this parameter is specified, <code>TestDurationInSeconds</code> must also be specified. Constraints: Must not be longer than 65535 bytes. Must be a QuestionForm data structure. This parameter cannot be specified if AutoGranted is true. Constraints: None. If not specified, the Worker may request the Qualification without answering any questions.<br/>QualificationTypeStatus:  The new status of the Qualification type - Active | Inactive<br/>Description:  The new description of the Qualification type.<br/>QualificationTypeId:  The ID of the Qualification type to update.<br/>
        :return: QualificationType:   Contains a QualificationType data structure.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoGrantedValue</i>  The Qualification integer value to use for automatically granted Qualifications, if AutoGranted is true. This is 1 by default. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoGranted</i> Specifies that requests for the Qualification type are granted immediately, without prompting the Worker with a Qualification test. Valid values are True | False.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsRequestable</i>  Specifies whether the Qualification type is one that a user can request through the Amazon Mechanical Turk web site, such as by taking a Qualification test. This value is False for Qualifications assigned automatically by the system. Valid values are True | False. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RetryDelayInSeconds</i>  The amount of time, in seconds, Workers must wait after taking the Qualification test before they can take it again. Workers can take a Qualification test multiple times if they were not granted the Qualification from a previous attempt, or if the test offers a gradient score and they want a better score. If not specified, retries are disabled and Workers can request a Qualification only once. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AnswerKey</i> The answers to the Qualification test specified in the Test parameter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TestDurationInSeconds</i>  The amount of time, in seconds, given to a Worker to complete the Qualification test, beginning from the time the Worker requests the Qualification. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Test</i>  The questions for a Qualification test associated with this Qualification type that a user can take to obtain a Qualification of this type. This parameter must be specified if AnswerKey is present. A Qualification type cannot have both a specified Test parameter and an AutoGranted value of true. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QualificationTypeStatus</i>  The status of the Qualification type. A Qualification type's status determines if users can apply to receive a Qualification of this type, and if HITs can be created with requirements based on this type. Valid values are Active | Inactive. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Keywords</i>  One or more words or phrases that describe theQualification type, separated by commas. The Keywords make the type easier to find using a search. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i>  A long description for the Qualification type. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i>  The name of the Qualification type. The type name is used to identify the type, and to find the type using a Qualification type search. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i>  The date and time the Qualification type was created. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QualificationTypeId</i>  A unique identifier for the Qualification type. A Qualification type is given a Qualification type ID when you call the CreateQualificationType operation. <br/>        """
        pass



class Neptune(BaseClient):
    """<fullname>Amazon Neptune</fullname> Amazon Neptune is a fast, reliable, fully-managed graph database service that makes it easy to build and run applications that work with highly connected datasets. The core of Amazon Neptune is a purpose-built, high-performance graph database engine optimized for storing billions of relationships and querying the graph with milliseconds latency. Amazon Neptune supports popular graph models Property Graph and W3C's RDF, and their respective query languages Apache TinkerPop Gremlin and SPARQL, allowing you to easily build queries that efficiently navigate highly connected datasets. Neptune powers graph use cases such as recommendation engines, fraud detection, knowledge graphs, drug discovery, and network security. This interface reference for Amazon Neptune contains documentation for a programming or command line interface you can use to manage Amazon Neptune. Note that Amazon Neptune is asynchronous, which means that some interfaces might require techniques such as polling or callback functions to determine when a command has been applied. In this reference, the parameter descriptions indicate whether a command is applied immediately, on the next instance reboot, or during the maintenance window. The reference structure is as follows, and we list following some related topics from the user guide."""

    class DBInstanceAvailableWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_db_instances` for valid parameters"""
            pass
    class DBInstanceDeletedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_db_instances` for valid parameters"""
            pass


    class DescribeDBEngineVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_db_engine_versions` for valid parameters"""
            pass
    class DescribeDBInstancesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_db_instances` for valid parameters"""
            pass
    class DescribeDBParameterGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_db_parameter_groups` for valid parameters"""
            pass
    class DescribeDBParametersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_db_parameters` for valid parameters"""
            pass
    class DescribeDBSubnetGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_db_subnet_groups` for valid parameters"""
            pass
    class DescribeEngineDefaultParametersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_engine_default_parameters` for valid parameters"""
            pass
    class DescribeEventSubscriptionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_event_subscriptions` for valid parameters"""
            pass
    class DescribeEventsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_events` for valid parameters"""
            pass
    class DescribeOrderableDBInstanceOptionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_orderable_db_instance_options` for valid parameters"""
            pass
    class DescribeDBClusterParameterGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_db_cluster_parameter_groups` for valid parameters"""
            pass
    class DescribeDBClusterParametersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_db_cluster_parameters` for valid parameters"""
            pass
    class DescribeDBClusterSnapshotsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_db_cluster_snapshots` for valid parameters"""
            pass
    class DescribeDBClustersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_db_clusters` for valid parameters"""
            pass
    class DescribePendingMaintenanceActionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_pending_maintenance_actions` for valid parameters"""
            pass


    class AddRoleToDBClusterMessage(Mapping):
        pass
    
    class AddSourceIdentifierToSubscriptionMessage(Mapping):
        pass
    
    class AddSourceIdentifierToSubscriptionResult(Mapping):
        pass
    
    class AddTagsToResourceMessage(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class ApplyPendingMaintenanceActionMessage(Mapping):
        pass
    
    class ApplyPendingMaintenanceActionResult(Mapping):
        pass
    
    class CopyDBClusterParameterGroupMessage(Mapping):
        pass
    
    class CopyDBClusterParameterGroupResult(Mapping):
        pass
    
    class CopyDBClusterSnapshotMessage(Mapping):
        pass
    
    class CopyDBClusterSnapshotResult(Mapping):
        pass
    
    class BooleanOptional(object):
        pass
    
    class CopyDBParameterGroupMessage(Mapping):
        pass
    
    class CopyDBParameterGroupResult(Mapping):
        pass
    
    class CreateDBClusterMessage(Mapping):
        pass
    
    class CreateDBClusterResult(Mapping):
        pass
    
    class String(object):
        pass
    
    class CreateDBClusterParameterGroupMessage(Mapping):
        pass
    
    class CreateDBClusterParameterGroupResult(Mapping):
        pass
    
    class CreateDBClusterSnapshotMessage(Mapping):
        pass
    
    class CreateDBClusterSnapshotResult(Mapping):
        pass
    
    class CreateDBInstanceMessage(Mapping):
        pass
    
    class CreateDBInstanceResult(Mapping):
        pass
    
    class CreateDBParameterGroupMessage(Mapping):
        pass
    
    class CreateDBParameterGroupResult(Mapping):
        pass
    
    class CreateDBSubnetGroupMessage(Mapping):
        pass
    
    class CreateDBSubnetGroupResult(Mapping):
        pass
    
    class CreateEventSubscriptionMessage(Mapping):
        pass
    
    class CreateEventSubscriptionResult(Mapping):
        pass
    
    class DeleteDBClusterMessage(Mapping):
        pass
    
    class DeleteDBClusterResult(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class DeleteDBClusterParameterGroupMessage(Mapping):
        pass
    
    class DeleteDBClusterSnapshotMessage(Mapping):
        pass
    
    class DeleteDBClusterSnapshotResult(Mapping):
        pass
    
    class DeleteDBInstanceMessage(Mapping):
        pass
    
    class DeleteDBInstanceResult(Mapping):
        pass
    
    class DeleteDBParameterGroupMessage(Mapping):
        pass
    
    class DeleteDBSubnetGroupMessage(Mapping):
        pass
    
    class DeleteEventSubscriptionMessage(Mapping):
        pass
    
    class DeleteEventSubscriptionResult(Mapping):
        pass
    
    class DescribeDBClusterParameterGroupsMessage(Mapping):
        pass
    
    class DBClusterParameterGroupsMessage(Mapping):
        pass
    
    class Filter(Mapping):
        pass
    
    class DescribeDBClusterParametersMessage(Mapping):
        pass
    
    class DBClusterParameterGroupDetails(Mapping):
        pass
    
    class DescribeDBClusterSnapshotAttributesMessage(Mapping):
        pass
    
    class DescribeDBClusterSnapshotAttributesResult(Mapping):
        pass
    
    class DescribeDBClusterSnapshotsMessage(Mapping):
        pass
    
    class DBClusterSnapshotMessage(Mapping):
        pass
    
    class DescribeDBClustersMessage(Mapping):
        pass
    
    class DBClusterMessage(Mapping):
        pass
    
    class DescribeDBEngineVersionsMessage(Mapping):
        pass
    
    class DBEngineVersionMessage(Mapping):
        pass
    
    class DescribeDBInstancesMessage(Mapping):
        pass
    
    class DBInstanceMessage(Mapping):
        pass
    
    class DescribeDBParameterGroupsMessage(Mapping):
        pass
    
    class DBParameterGroupsMessage(Mapping):
        pass
    
    class DescribeDBParametersMessage(Mapping):
        pass
    
    class DBParameterGroupDetails(Mapping):
        pass
    
    class DescribeDBSubnetGroupsMessage(Mapping):
        pass
    
    class DBSubnetGroupMessage(Mapping):
        pass
    
    class DescribeEngineDefaultClusterParametersMessage(Mapping):
        pass
    
    class DescribeEngineDefaultClusterParametersResult(Mapping):
        pass
    
    class DescribeEngineDefaultParametersMessage(Mapping):
        pass
    
    class DescribeEngineDefaultParametersResult(Mapping):
        pass
    
    class DescribeEventCategoriesMessage(Mapping):
        pass
    
    class EventCategoriesMessage(Mapping):
        pass
    
    class DescribeEventSubscriptionsMessage(Mapping):
        pass
    
    class EventSubscriptionsMessage(Mapping):
        pass
    
    class DescribeEventsMessage(Mapping):
        pass
    
    class EventsMessage(Mapping):
        pass
    
    class TStamp(object):
        pass
    
    class DescribeOrderableDBInstanceOptionsMessage(Mapping):
        pass
    
    class OrderableDBInstanceOptionsMessage(Mapping):
        pass
    
    class DescribePendingMaintenanceActionsMessage(Mapping):
        pass
    
    class PendingMaintenanceActionsMessage(Mapping):
        pass
    
    class DescribeValidDBInstanceModificationsMessage(Mapping):
        pass
    
    class DescribeValidDBInstanceModificationsResult(Mapping):
        pass
    
    class FailoverDBClusterMessage(Mapping):
        pass
    
    class FailoverDBClusterResult(Mapping):
        pass
    
    class ListTagsForResourceMessage(Mapping):
        pass
    
    class TagListMessage(Mapping):
        pass
    
    class ModifyDBClusterMessage(Mapping):
        pass
    
    class ModifyDBClusterResult(Mapping):
        pass
    
    class CloudwatchLogsExportConfiguration(Mapping):
        pass
    
    class ModifyDBClusterParameterGroupMessage(Mapping):
        pass
    
    class DBClusterParameterGroupNameMessage(Mapping):
        pass
    
    class Parameter(Mapping):
        pass
    
    class ModifyDBClusterSnapshotAttributeMessage(Mapping):
        pass
    
    class ModifyDBClusterSnapshotAttributeResult(Mapping):
        pass
    
    class ModifyDBInstanceMessage(Mapping):
        pass
    
    class ModifyDBInstanceResult(Mapping):
        pass
    
    class ModifyDBParameterGroupMessage(Mapping):
        pass
    
    class DBParameterGroupNameMessage(Mapping):
        pass
    
    class ModifyDBSubnetGroupMessage(Mapping):
        pass
    
    class ModifyDBSubnetGroupResult(Mapping):
        pass
    
    class ModifyEventSubscriptionMessage(Mapping):
        pass
    
    class ModifyEventSubscriptionResult(Mapping):
        pass
    
    class PromoteReadReplicaDBClusterMessage(Mapping):
        pass
    
    class PromoteReadReplicaDBClusterResult(Mapping):
        pass
    
    class RebootDBInstanceMessage(Mapping):
        pass
    
    class RebootDBInstanceResult(Mapping):
        pass
    
    class RemoveRoleFromDBClusterMessage(Mapping):
        pass
    
    class RemoveSourceIdentifierFromSubscriptionMessage(Mapping):
        pass
    
    class RemoveSourceIdentifierFromSubscriptionResult(Mapping):
        pass
    
    class RemoveTagsFromResourceMessage(Mapping):
        pass
    
    class ResetDBClusterParameterGroupMessage(Mapping):
        pass
    
    class ResetDBParameterGroupMessage(Mapping):
        pass
    
    class RestoreDBClusterFromSnapshotMessage(Mapping):
        pass
    
    class RestoreDBClusterFromSnapshotResult(Mapping):
        pass
    
    class RestoreDBClusterToPointInTimeMessage(Mapping):
        pass
    
    class RestoreDBClusterToPointInTimeResult(Mapping):
        pass
    
    def add_role_to_db_cluster(self, RoleArn: str, DBClusterIdentifier: str):
        """Associates an Identity and Access Management (IAM) role from an Neptune DB cluster.

        :param self:
        :param RoleArn:  The Amazon Resource Name (ARN) of the IAM role to associate with the Neptune DB cluster, for example <code>arn:aws:iam::123456789012:role/NeptuneAccessRole</code>.<br/>DBClusterIdentifier:  The name of the DB cluster to associate the IAM role with.<br/>
        :return:         """
        pass

    def add_source_identifier_to_subscription(self, SourceIdentifier: str, SubscriptionName: str) -> AddSourceIdentifierToSubscriptionResult:
        """Adds a source identifier to an existing event notification subscription.

        :param self:
        :param SourceIdentifier:  The identifier of the event source to be added. Constraints: <ul> <li> If the source type is a DB instance, then a <code>DBInstanceIdentifier</code> must be supplied. </li> <li> If the source type is a DB security group, a <code>DBSecurityGroupName</code> must be supplied. </li> <li> If the source type is a DB parameter group, a <code>DBParameterGroupName</code> must be supplied. </li> <li> If the source type is a DB snapshot, a <code>DBSnapshotIdentifier</code> must be supplied. </li> </ul><br/>SubscriptionName:  The name of the event notification subscription you want to add a source identifier to.<br/>
        :return: EventSubscription:  Contains the results of a successful invocation of the <a>DescribeEventSubscriptions</a> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EventSubscriptionArn</i> The Amazon Resource Name (ARN) for the event subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> A Boolean value indicating if the subscription is enabled. True indicates the subscription is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EventCategoriesList</i> A list of event categories for the event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceIdsList</i> A list of source IDs for the event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceType</i> The source type for the event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubscriptionCreationTime</i> The time the event notification subscription was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the event notification subscription. Constraints: Can be one of the following: creating | modifying | deleting | active | no-permission | topic-not-exist The status "no-permission" indicates that Neptune no longer has permission to post to the SNS topic. The status "topic-not-exist" indicates that the topic was deleted after the subscription was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsTopicArn</i> The topic ARN of the event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustSubscriptionId</i> The event notification subscription Id.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomerAwsId</i> The AWS customer account associated with the event notification subscription.<br/>        """
        pass

    def add_tags_to_resource(self, Tags: List[Tag], ResourceName: str):
        """Adds metadata tags to an Amazon Neptune resource. These tags can also be used with cost allocation reporting to track cost associated with Amazon Neptune resources, or used in a Condition statement in an IAM policy for Amazon Neptune.

        :param self:
        :param Tags:  The tags to be assigned to the Amazon Neptune resource.<br/>ResourceName:  The Amazon Neptune resource that the tags are added to. This value is an Amazon Resource Name (ARN). For information about creating an ARN, see <a href="https://docs.aws.amazon.com/neptune/latest/UserGuide/tagging.ARN.html#tagging.ARN.Constructing"> Constructing an Amazon Resource Name (ARN)</a>.<br/>
        :return:         """
        pass

    def apply_pending_maintenance_action(self, OptInType: str, ApplyAction: str, ResourceIdentifier: str) -> ApplyPendingMaintenanceActionResult:
        """Applies a pending maintenance action to a resource (for example, to a DB instance).

        :param self:
        :param OptInType:  A value that specifies the type of opt-in request, or undoes an opt-in request. An opt-in request of type <code>immediate</code> can't be undone. Valid values: <ul> <li>  <code>immediate</code> - Apply the maintenance action immediately. </li> <li>  <code>next-maintenance</code> - Apply the maintenance action during the next maintenance window for the resource. </li> <li>  <code>undo-opt-in</code> - Cancel any existing <code>next-maintenance</code> opt-in requests. </li> </ul><br/>ApplyAction:  The pending maintenance action to apply to this resource. Valid values: <code>system-update</code>, <code>db-upgrade</code> <br/>ResourceIdentifier:  The Amazon Resource Name (ARN) of the resource that the pending maintenance action applies to. For information about creating an ARN, see <a href="https://docs.aws.amazon.com/neptune/latest/UserGuide/tagging.ARN.html#tagging.ARN.Constructing"> Constructing an Amazon Resource Name (ARN)</a>.<br/>
        :return: ResourcePendingMaintenanceActions:  Describes the pending maintenance actions for a resource.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingMaintenanceActionDetails</i> A list that provides details about the pending maintenance actions for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceIdentifier</i> The ARN of the resource that has pending maintenance actions.<br/>        """
        pass

    def copy_db_cluster_parameter_group(self, TargetDBClusterParameterGroupDescription: str, TargetDBClusterParameterGroupIdentifier: str, SourceDBClusterParameterGroupIdentifier: str, Tags: List[Tag]=None) -> CopyDBClusterParameterGroupResult:
        """Copies the specified DB cluster parameter group.

        :param self:
        :param Tags:  The tags to be assigned to the copied DB cluster parameter group.<br/>TargetDBClusterParameterGroupDescription:  A description for the copied DB cluster parameter group.<br/>TargetDBClusterParameterGroupIdentifier:  The identifier for the copied DB cluster parameter group. Constraints: <ul> <li> Cannot be null, empty, or blank </li> <li> Must contain from 1 to 255 letters, numbers, or hyphens </li> <li> First character must be a letter </li> <li> Cannot end with a hyphen or contain two consecutive hyphens </li> </ul> Example: <code>my-cluster-param-group1</code> <br/>SourceDBClusterParameterGroupIdentifier:  The identifier or Amazon Resource Name (ARN) for the source DB cluster parameter group. For information about creating an ARN, see <a href="https://docs.aws.amazon.com/neptune/latest/UserGuide/tagging.ARN.html#tagging.ARN.Constructing"> Constructing an Amazon Resource Name (ARN)</a>. Constraints: <ul> <li> Must specify a valid DB cluster parameter group. </li> <li> If the source DB cluster parameter group is in the same AWS Region as the copy, specify a valid DB parameter group identifier, for example <code>my-db-cluster-param-group</code>, or a valid ARN. </li> <li> If the source DB parameter group is in a different AWS Region than the copy, specify a valid DB cluster parameter group ARN, for example <code>arn:aws:rds:us-east-1:123456789012:cluster-pg:custom-cluster-group1</code>. </li> </ul><br/>
        :return: DBClusterParameterGroup:  Contains the details of an Amazon Neptune DB cluster parameter group. This data type is used as a response element in the <a>DescribeDBClusterParameterGroups</a> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroupArn</i> The Amazon Resource Name (ARN) for the DB cluster parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> Provides the customer-specified description for this DB cluster parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroupFamily</i> Provides the name of the DB parameter group family that this DB cluster parameter group is compatible with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroupName</i> Provides the name of the DB cluster parameter group.<br/>        """
        pass

    def copy_db_cluster_snapshot(self, TargetDBClusterSnapshotIdentifier: str, SourceDBClusterSnapshotIdentifier: str, KmsKeyId: str=None, PreSignedUrl: str=None, CopyTags: BooleanOptional=None, Tags: List[Tag]=None, SourceRegion: str=None) -> CopyDBClusterSnapshotResult:
        """Copies a snapshot of a DB cluster. To copy a DB cluster snapshot from a shared manual DB cluster snapshot, <code>SourceDBClusterSnapshotIdentifier</code> must be the Amazon Resource Name (ARN) of the shared DB cluster snapshot. You can't copy from one AWS Region to another.

        :param self:
        :param SourceRegion:  The ID of the region that contains the snapshot to be copied.<br/>Tags:  The tags to assign to the new DB cluster snapshot copy.<br/>CopyTags:  True to copy all tags from the source DB cluster snapshot to the target DB cluster snapshot, and otherwise false. The default is false.<br/>PreSignedUrl:  Not currently supported.<br/>KmsKeyId:  The AWS AWS KMS key ID for an encrypted DB cluster snapshot. The KMS key ID is the Amazon Resource Name (ARN), KMS key identifier, or the KMS key alias for the KMS encryption key. If you copy an unencrypted DB cluster snapshot and specify a value for the <code>KmsKeyId</code> parameter, Amazon Neptune encrypts the target DB cluster snapshot using the specified KMS encryption key. If you copy an encrypted DB cluster snapshot from your AWS account, you can specify a value for <code>KmsKeyId</code> to encrypt the copy with a new KMS encryption key. If you don't specify a value for <code>KmsKeyId</code>, then the copy of the DB cluster snapshot is encrypted with the same KMS key as the source DB cluster snapshot. If you copy an encrypted DB cluster snapshot that is shared from another AWS account, then you must specify a value for <code>KmsKeyId</code>.  KMS encryption keys are specific to the AWS Region that they are created in, and you can't use encryption keys from one AWS Region in another AWS Region.<br/>TargetDBClusterSnapshotIdentifier:  The identifier of the new DB cluster snapshot to create from the source DB cluster snapshot. This parameter is not case-sensitive. Constraints: <ul> <li> Must contain from 1 to 63 letters, numbers, or hyphens. </li> <li> First character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul> Example: <code>my-cluster-snapshot2</code> <br/>SourceDBClusterSnapshotIdentifier:  The identifier of the DB cluster snapshot to copy. This parameter is not case-sensitive. You can't copy from one AWS Region to another. Constraints: <ul> <li> Must specify a valid system snapshot in the "available" state. </li> <li> Specify a valid DB snapshot identifier. </li> </ul> Example: <code>my-cluster-snapshot1</code> <br/>
        :return: DBClusterSnapshot:  Contains the details for an Amazon Neptune DB cluster snapshot This data type is used as a response element in the <a>DescribeDBClusterSnapshots</a> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceDBClusterSnapshotArn</i> If the DB cluster snapshot was copied from a source DB cluster snapshot, the Amazon Resource Name (ARN) for the source DB cluster snapshot, otherwise, a null value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotArn</i> The Amazon Resource Name (ARN) for the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is true, the AWS KMS key identifier for the encrypted DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster snapshot is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the percentage of the estimated data that has been transferred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotType</i> Provides the type of the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> Provides the license model information for this DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Provides the version of the database engine for this DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Provides the master username for the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VPC ID associated with the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the DB cluster was listening on at the time of the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the status of this DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> Specifies the allocated storage size in gibibytes (GiB).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Specifies the name of the database engine.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCreateTime</i> Provides the time when the snapshot was taken, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Specifies the DB cluster identifier of the DB cluster that this DB cluster snapshot was created from.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotIdentifier</i> Specifies the identifier for the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of EC2 Availability Zones that instances in the DB cluster snapshot can be restored in.<br/>        """
        pass

    def copy_db_parameter_group(self, TargetDBParameterGroupDescription: str, TargetDBParameterGroupIdentifier: str, SourceDBParameterGroupIdentifier: str, Tags: List[Tag]=None) -> CopyDBParameterGroupResult:
        """Copies the specified DB parameter group.

        :param self:
        :param Tags:  The tags to be assigned to the copied DB parameter group.<br/>TargetDBParameterGroupDescription:  A description for the copied DB parameter group.<br/>TargetDBParameterGroupIdentifier:  The identifier for the copied DB parameter group. Constraints: <ul> <li> Cannot be null, empty, or blank. </li> <li> Must contain from 1 to 255 letters, numbers, or hyphens. </li> <li> First character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul> Example: <code>my-db-parameter-group</code> <br/>SourceDBParameterGroupIdentifier:  The identifier or ARN for the source DB parameter group. For information about creating an ARN, see <a href="https://docs.aws.amazon.com/neptune/latest/UserGuide/tagging.ARN.html#tagging.ARN.Constructing"> Constructing an Amazon Resource Name (ARN)</a>. Constraints: <ul> <li> Must specify a valid DB parameter group. </li> <li> Must specify a valid DB parameter group identifier, for example <code>my-db-param-group</code>, or a valid ARN. </li> </ul><br/>
        :return: DBParameterGroup:  Contains the details of an Amazon Neptune DB parameter group. This data type is used as a response element in the <a>DescribeDBParameterGroups</a> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroupArn</i> The Amazon Resource Name (ARN) for the DB parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> Provides the customer-specified description for this DB parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroupFamily</i> Provides the name of the DB parameter group family that this DB parameter group is compatible with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroupName</i> Provides the name of the DB parameter group.<br/>        """
        pass

    def create_db_cluster(self, Engine: str, DBClusterIdentifier: str, AvailabilityZones: List[String]=None, BackupRetentionPeriod: int=None, CharacterSetName: str=None, DatabaseName: str=None, DBClusterParameterGroupName: str=None, VpcSecurityGroupIds: List[String]=None, DBSubnetGroupName: str=None, EngineVersion: str=None, Port: int=None, MasterUsername: str=None, MasterUserPassword: str=None, OptionGroupName: str=None, PreferredBackupWindow: str=None, PreferredMaintenanceWindow: str=None, ReplicationSourceIdentifier: str=None, Tags: List[Tag]=None, StorageEncrypted: BooleanOptional=None, KmsKeyId: str=None, PreSignedUrl: str=None, EnableIAMDatabaseAuthentication: BooleanOptional=None, EnableCloudwatchLogsExports: List[String]=None, SourceRegion: str=None) -> CreateDBClusterResult:
        """Creates a new Amazon Neptune DB cluster. You can use the <code>ReplicationSourceIdentifier</code> parameter to create the DB cluster as a Read Replica of another DB cluster or Amazon Neptune DB instance.

        :param self:
        :param SourceRegion:  The ID of the region that contains the source for the db cluster.<br/>EnableCloudwatchLogsExports:  The list of log types that need to be enabled for exporting to CloudWatch Logs.<br/>EnableIAMDatabaseAuthentication:  True to enable mapping of AWS Identity and Access Management (IAM) accounts to database accounts, and otherwise false. Default: <code>false</code> <br/>PreSignedUrl:  This parameter is not currently supported.<br/>KmsKeyId:  The AWS KMS key identifier for an encrypted DB cluster. The KMS key identifier is the Amazon Resource Name (ARN) for the KMS encryption key. If you are creating a DB cluster with the same AWS account that owns the KMS encryption key used to encrypt the new DB cluster, then you can use the KMS key alias instead of the ARN for the KMS encryption key. If an encryption key is not specified in <code>KmsKeyId</code>: <ul> <li> If <code>ReplicationSourceIdentifier</code> identifies an encrypted source, then Amazon Neptune will use the encryption key used to encrypt the source. Otherwise, Amazon Neptune will use your default encryption key. </li> <li> If the <code>StorageEncrypted</code> parameter is true and <code>ReplicationSourceIdentifier</code> is not specified, then Amazon Neptune will use your default encryption key. </li> </ul> AWS KMS creates the default encryption key for your AWS account. Your AWS account has a different default encryption key for each AWS Region. If you create a Read Replica of an encrypted DB cluster in another AWS Region, you must set <code>KmsKeyId</code> to a KMS key ID that is valid in the destination AWS Region. This key is used to encrypt the Read Replica in that AWS Region.<br/>StorageEncrypted:  Specifies whether the DB cluster is encrypted.<br/>Tags:  The tags to assign to the new DB cluster.<br/>ReplicationSourceIdentifier:  The Amazon Resource Name (ARN) of the source DB instance or DB cluster if this DB cluster is created as a Read Replica.<br/>PreferredMaintenanceWindow:  The weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC). Format: <code>ddd:hh24:mi-ddd:hh24:mi</code>  The default is a 30-minute window selected at random from an 8-hour block of time for each AWS Region, occurring on a random day of the week. To see the time blocks available, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/AdjustingTheMaintenanceWindow.html"> Adjusting the Preferred Maintenance Window</a> in the <i>Amazon Neptune User Guide.</i>  Valid Days: Mon, Tue, Wed, Thu, Fri, Sat, Sun. Constraints: Minimum 30-minute window.<br/>PreferredBackupWindow:  The daily time range during which automated backups are created if automated backups are enabled using the <code>BackupRetentionPeriod</code> parameter. The default is a 30-minute window selected at random from an 8-hour block of time for each AWS Region. To see the time blocks available, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/AdjustingTheMaintenanceWindow.html"> Adjusting the Preferred Maintenance Window</a> in the <i>Amazon Neptune User Guide.</i>  Constraints: <ul> <li> Must be in the format <code>hh24:mi-hh24:mi</code>. </li> <li> Must be in Universal Coordinated Time (UTC). </li> <li> Must not conflict with the preferred maintenance window. </li> <li> Must be at least 30 minutes. </li> </ul><br/>OptionGroupName:  A value that indicates that the DB cluster should be associated with the specified option group. Permanent options can't be removed from an option group. The option group can't be removed from a DB cluster once it is associated with a DB cluster.<br/>MasterUserPassword:  The password for the master database user. This password can contain any printable ASCII character except "/", triple-quotes, or "@". Constraints: Must contain from 8 to 41 characters.<br/>MasterUsername:  The name of the master user for the DB cluster. Constraints: <ul> <li> Must be 1 to 16 letters or numbers. </li> <li> First character must be a letter. </li> <li> Cannot be a reserved word for the chosen database engine. </li> </ul><br/>Port:  The port number on which the instances in the DB cluster accept connections.  Default: <code>8182</code> <br/>EngineVersion:  The version number of the database engine to use. Example: <code>1.0.1</code> <br/>Engine:  The name of the database engine to be used for this DB cluster. Valid Values: <code>neptune</code> <br/>DBSubnetGroupName:  A DB subnet group to associate with this DB cluster. Constraints: Must match the name of an existing DBSubnetGroup. Must not be default. Example: <code>mySubnetgroup</code> <br/>VpcSecurityGroupIds:  A list of EC2 VPC security groups to associate with this DB cluster.<br/>DBClusterParameterGroupName:   The name of the DB cluster parameter group to associate with this DB cluster. If this argument is omitted, the default is used. Constraints: <ul> <li> If supplied, must match the name of an existing DBClusterParameterGroup. </li> </ul><br/>DBClusterIdentifier:  The DB cluster identifier. This parameter is stored as a lowercase string. Constraints: <ul> <li> Must contain from 1 to 63 letters, numbers, or hyphens. </li> <li> First character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul> Example: <code>my-cluster1</code> <br/>DatabaseName:  The name for your database of up to 64 alpha-numeric characters. If you do not provide a name, Amazon Neptune will not create a database in the DB cluster you are creating.<br/>CharacterSetName:  A value that indicates that the DB cluster should be associated with the specified CharacterSet.<br/>BackupRetentionPeriod:  The number of days for which automated backups are retained. You must specify a minimum value of 1. Default: 1 Constraints: <ul> <li> Must be a value from 1 to 35 </li> </ul><br/>AvailabilityZones:  A list of EC2 Availability Zones that instances in the DB cluster can be created in.<br/>
        :return: DBCluster:  Contains the details of an Amazon Neptune DB cluster. This data type is used as a response element in the <a>DescribeDBClusters</a> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB cluster is configured to export to CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloneGroupId</i> Identifies the clone group to which the DB cluster is associated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i> Provides a list of the AWS Identity and Access Management (IAM) roles that are associated with the DB cluster. IAM roles that are associated with a DB cluster grant permission for the DB cluster to access other AWS services on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterArn</i> The Amazon Resource Name (ARN) for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbClusterResourceId</i> The AWS Region-unique, immutable identifier for the DB cluster. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB cluster is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is true, the AWS KMS key identifier for the encrypted DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security groups that the DB cluster belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterMembers</i> Provides the list of instances that make up the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSourceIdentifier</i> Not supported by Neptune.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i> Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterOptionGroupMemberships</i> Provides the list of option group memberships for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies whether the DB cluster has instances in multiple Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReaderEndpoint</i> The reader endpoint for the DB cluster. The reader endpoint for a DB cluster load-balances connections across the Read Replicas that are available in a DB cluster. As clients request new connections to the reader endpoint, Neptune distributes the connection requests among the Read Replicas in the DB cluster. This functionality can help balance your read workload across multiple Read Replicas in your DB cluster. If a failover occurs, and the Read Replica that you are connected to is promoted to be the primary instance, your connection is dropped. To continue sending your read workload to other Read Replicas in the cluster, you can then reconnect to the reader endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint for the primary instance of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestRestorableTime</i> Specifies the earliest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the progress of the operation as a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the current state of this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB cluster, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroup</i> Specifies the name of the DB cluster parameter group for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Contains a user-supplied DB cluster identifier. This identifier is the unique key that identifies a DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> Contains the name of the initial database of this DB cluster that was provided at create time, if one was specified when the DB cluster was created. This same name is returned for the life of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this cluster is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of EC2 Availability Zones that instances in the DB cluster can be created in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i>  <code>AllocatedStorage</code> always returns 1, because Neptune DB cluster storage size is not fixed, but instead automatically adjusts as needed.<br/>        """
        pass

    def create_db_cluster_parameter_group(self, Description: str, DBParameterGroupFamily: str, DBClusterParameterGroupName: str, Tags: List[Tag]=None) -> CreateDBClusterParameterGroupResult:
        """Creates a new DB cluster parameter group. Parameters in a DB cluster parameter group apply to all of the instances in a DB cluster.  A DB cluster parameter group is initially created with the default parameters for the database engine used by instances in the DB cluster. To provide custom values for any of the parameters, you must modify the group after creating it using <a>ModifyDBClusterParameterGroup</a>. Once you've created a DB cluster parameter group, you need to associate it with your DB cluster using <a>ModifyDBCluster</a>. When you associate a new DB cluster parameter group with a running DB cluster, you need to reboot the DB instances in the DB cluster without failover for the new DB cluster parameter group and associated settings to take effect. <important> After you create a DB cluster parameter group, you should wait at least 5 minutes before creating your first DB cluster that uses that DB cluster parameter group as the default parameter group. This allows Amazon Neptune to fully complete the create action before the DB cluster parameter group is used as the default for a new DB cluster. This is especially important for parameters that are critical when creating the default database for a DB cluster, such as the character set for the default database defined by the <code>character_set_database</code> parameter. You can use the <i>Parameter Groups</i> option of the <a href="https://console.aws.amazon.com/rds/">Amazon Neptune console</a> or the <a>DescribeDBClusterParameters</a> command to verify that your DB cluster parameter group has been created or modified. </important>

        :param self:
        :param Tags:  The tags to be assigned to the new DB cluster parameter group.<br/>Description:  The description for the DB cluster parameter group.<br/>DBParameterGroupFamily:  The DB cluster parameter group family name. A DB cluster parameter group can be associated with one and only one DB cluster parameter group family, and can be applied only to a DB cluster running a database engine and engine version compatible with that DB cluster parameter group family.<br/>DBClusterParameterGroupName:  The name of the DB cluster parameter group. Constraints: <ul> <li> Must match the name of an existing DBClusterParameterGroup. </li> </ul> <note> This value is stored as a lowercase string. </note><br/>
        :return: DBClusterParameterGroup:  Contains the details of an Amazon Neptune DB cluster parameter group. This data type is used as a response element in the <a>DescribeDBClusterParameterGroups</a> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroupArn</i> The Amazon Resource Name (ARN) for the DB cluster parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> Provides the customer-specified description for this DB cluster parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroupFamily</i> Provides the name of the DB parameter group family that this DB cluster parameter group is compatible with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroupName</i> Provides the name of the DB cluster parameter group.<br/>        """
        pass

    def create_db_cluster_snapshot(self, DBClusterIdentifier: str, DBClusterSnapshotIdentifier: str, Tags: List[Tag]=None) -> CreateDBClusterSnapshotResult:
        """Creates a snapshot of a DB cluster.

        :param self:
        :param Tags:  The tags to be assigned to the DB cluster snapshot.<br/>DBClusterIdentifier:  The identifier of the DB cluster to create a snapshot for. This parameter is not case-sensitive. Constraints: <ul> <li> Must match the identifier of an existing DBCluster. </li> </ul> Example: <code>my-cluster1</code> <br/>DBClusterSnapshotIdentifier:  The identifier of the DB cluster snapshot. This parameter is stored as a lowercase string. Constraints: <ul> <li> Must contain from 1 to 63 letters, numbers, or hyphens. </li> <li> First character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul> Example: <code>my-cluster1-snapshot1</code> <br/>
        :return: DBClusterSnapshot:  Contains the details for an Amazon Neptune DB cluster snapshot This data type is used as a response element in the <a>DescribeDBClusterSnapshots</a> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceDBClusterSnapshotArn</i> If the DB cluster snapshot was copied from a source DB cluster snapshot, the Amazon Resource Name (ARN) for the source DB cluster snapshot, otherwise, a null value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotArn</i> The Amazon Resource Name (ARN) for the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is true, the AWS KMS key identifier for the encrypted DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster snapshot is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the percentage of the estimated data that has been transferred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotType</i> Provides the type of the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> Provides the license model information for this DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Provides the version of the database engine for this DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Provides the master username for the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VPC ID associated with the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the DB cluster was listening on at the time of the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the status of this DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> Specifies the allocated storage size in gibibytes (GiB).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Specifies the name of the database engine.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCreateTime</i> Provides the time when the snapshot was taken, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Specifies the DB cluster identifier of the DB cluster that this DB cluster snapshot was created from.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotIdentifier</i> Specifies the identifier for the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of EC2 Availability Zones that instances in the DB cluster snapshot can be restored in.<br/>        """
        pass

    def create_db_instance(self, Engine: str, DBInstanceClass: str, DBInstanceIdentifier: str, DBName: str=None, AllocatedStorage: int=None, MasterUsername: str=None, MasterUserPassword: str=None, DBSecurityGroups: List[String]=None, VpcSecurityGroupIds: List[String]=None, AvailabilityZone: str=None, DBSubnetGroupName: str=None, PreferredMaintenanceWindow: str=None, DBParameterGroupName: str=None, BackupRetentionPeriod: int=None, PreferredBackupWindow: str=None, Port: int=None, MultiAZ: BooleanOptional=None, EngineVersion: str=None, AutoMinorVersionUpgrade: BooleanOptional=None, LicenseModel: str=None, Iops: int=None, OptionGroupName: str=None, CharacterSetName: str=None, PubliclyAccessible: BooleanOptional=None, Tags: List[Tag]=None, DBClusterIdentifier: str=None, StorageType: str=None, TdeCredentialArn: str=None, TdeCredentialPassword: str=None, StorageEncrypted: BooleanOptional=None, KmsKeyId: str=None, Domain: str=None, CopyTagsToSnapshot: BooleanOptional=None, MonitoringInterval: int=None, MonitoringRoleArn: str=None, DomainIAMRoleName: str=None, PromotionTier: int=None, Timezone: str=None, EnableIAMDatabaseAuthentication: BooleanOptional=None, EnablePerformanceInsights: BooleanOptional=None, PerformanceInsightsKMSKeyId: str=None, EnableCloudwatchLogsExports: List[String]=None) -> CreateDBInstanceResult:
        """Creates a new DB instance.

        :param self:
        :param EnableCloudwatchLogsExports:  The list of log types that need to be enabled for exporting to CloudWatch Logs.<br/>PerformanceInsightsKMSKeyId:  The AWS KMS key identifier for encryption of Performance Insights data. The KMS key ID is the Amazon Resource Name (ARN), KMS key identifier, or the KMS key alias for the KMS encryption key.<br/>EnablePerformanceInsights:  True to enable Performance Insights for the DB instance, and otherwise false.<br/>EnableIAMDatabaseAuthentication:  True to enable AWS Identity and Access Management (IAM) authentication for Neptune. Default: <code>false</code> <br/>Timezone:  The time zone of the DB instance.<br/>PromotionTier:  A value that specifies the order in which an Read Replica is promoted to the primary instance after a failure of the existing primary instance.  Default: 1 Valid Values: 0 - 15<br/>DomainIAMRoleName:  Specify the name of the IAM role to be used when making API calls to the Directory Service.<br/>MonitoringRoleArn:  The ARN for the IAM role that permits Neptune to send enhanced monitoring metrics to Amazon CloudWatch Logs. For example, <code>arn:aws:iam:123456789012:role/emaccess</code>. If <code>MonitoringInterval</code> is set to a value other than 0, then you must supply a <code>MonitoringRoleArn</code> value.<br/>MonitoringInterval:  The interval, in seconds, between points when Enhanced Monitoring metrics are collected for the DB instance. To disable collecting Enhanced Monitoring metrics, specify 0. The default is 0. If <code>MonitoringRoleArn</code> is specified, then you must also set <code>MonitoringInterval</code> to a value other than 0. Valid Values: <code>0, 1, 5, 10, 15, 30, 60</code> <br/>CopyTagsToSnapshot:  True to copy all tags from the DB instance to snapshots of the DB instance, and otherwise false. The default is false.<br/>Domain:  Specify the Active Directory Domain to create the instance in.<br/>KmsKeyId:  The AWS KMS key identifier for an encrypted DB instance. The KMS key identifier is the Amazon Resource Name (ARN) for the KMS encryption key. If you are creating a DB instance with the same AWS account that owns the KMS encryption key used to encrypt the new DB instance, then you can use the KMS key alias instead of the ARN for the KM encryption key. Not applicable. The KMS key identifier is managed by the DB cluster. For more information, see <a>CreateDBCluster</a>. If the <code>StorageEncrypted</code> parameter is true, and you do not specify a value for the <code>KmsKeyId</code> parameter, then Amazon Neptune will use your default encryption key. AWS KMS creates the default encryption key for your AWS account. Your AWS account has a different default encryption key for each AWS Region.<br/>StorageEncrypted:  Specifies whether the DB instance is encrypted. Not applicable. The encryption for DB instances is managed by the DB cluster. For more information, see <a>CreateDBCluster</a>. Default: false<br/>TdeCredentialPassword:  The password for the given ARN from the key store in order to access the device.<br/>TdeCredentialArn:  The ARN from the key store with which to associate the instance for TDE encryption.<br/>StorageType:  Specifies the storage type to be associated with the DB instance. Not applicable. Storage is managed by the DB Cluster.<br/>DBClusterIdentifier:  The identifier of the DB cluster that the instance will belong to. For information on creating a DB cluster, see <a>CreateDBCluster</a>. Type: String<br/>Tags:  The tags to assign to the new instance.<br/>PubliclyAccessible:  This flag should no longer be used.<br/>CharacterSetName:  Indicates that the DB instance should be associated with the specified CharacterSet. Not applicable. The character set is managed by the DB cluster. For more information, see <a>CreateDBCluster</a>.<br/>OptionGroupName:  Indicates that the DB instance should be associated with the specified option group. Permanent options, such as the TDE option for Oracle Advanced Security TDE, can't be removed from an option group, and that option group can't be removed from a DB instance once it is associated with a DB instance<br/>Iops:  The amount of Provisioned IOPS (input/output operations per second) to be initially allocated for the DB instance.<br/>LicenseModel:  License model information for this DB instance.  Valid values: <code>license-included</code> | <code>bring-your-own-license</code> | <code>general-public-license</code> <br/>AutoMinorVersionUpgrade:  Indicates that minor engine upgrades are applied automatically to the DB instance during the maintenance window. Default: <code>true</code> <br/>EngineVersion:  The version number of the database engine to use.<br/>MultiAZ:  Specifies if the DB instance is a Multi-AZ deployment. You can't set the AvailabilityZone parameter if the MultiAZ parameter is set to true.<br/>Port:  The port number on which the database accepts connections. Not applicable. The port is managed by the DB cluster. For more information, see <a>CreateDBCluster</a>.  Default: <code>8182</code>  Type: Integer<br/>PreferredBackupWindow:   The daily time range during which automated backups are created. Not applicable. The daily time range for creating automated backups is managed by the DB cluster. For more information, see <a>CreateDBCluster</a>.<br/>BackupRetentionPeriod:  The number of days for which automated backups are retained. Not applicable. The retention period for automated backups is managed by the DB cluster. For more information, see <a>CreateDBCluster</a>. Default: 1 Constraints: <ul> <li> Must be a value from 0 to 35 </li> <li> Cannot be set to 0 if the DB instance is a source to Read Replicas </li> </ul><br/>DBParameterGroupName:  The name of the DB parameter group to associate with this DB instance. If this argument is omitted, the default DBParameterGroup for the specified engine is used. Constraints: <ul> <li> Must be 1 to 255 letters, numbers, or hyphens. </li> <li> First character must be a letter </li> <li> Cannot end with a hyphen or contain two consecutive hyphens </li> </ul><br/>PreferredMaintenanceWindow:  The time range each week during which system maintenance can occur, in Universal Coordinated Time (UTC).  Format: <code>ddd:hh24:mi-ddd:hh24:mi</code>  The default is a 30-minute window selected at random from an 8-hour block of time for each AWS Region, occurring on a random day of the week. Valid Days: Mon, Tue, Wed, Thu, Fri, Sat, Sun. Constraints: Minimum 30-minute window.<br/>DBSubnetGroupName:  A DB subnet group to associate with this DB instance. If there is no DB subnet group, then it is a non-VPC DB instance.<br/>AvailabilityZone:   The EC2 Availability Zone that the DB instance is created in Default: A random, system-chosen Availability Zone in the endpoint's AWS Region.  Example: <code>us-east-1d</code>   Constraint: The AvailabilityZone parameter can't be specified if the MultiAZ parameter is set to <code>true</code>. The specified Availability Zone must be in the same AWS Region as the current endpoint.<br/>VpcSecurityGroupIds:  A list of EC2 VPC security groups to associate with this DB instance. Not applicable. The associated list of EC2 VPC security groups is managed by the DB cluster. For more information, see <a>CreateDBCluster</a>. Default: The default EC2 VPC security group for the DB subnet group's VPC.<br/>DBSecurityGroups:  A list of DB security groups to associate with this DB instance. Default: The default DB security group for the database engine.<br/>MasterUserPassword:  The password for the master user. The password can include any printable ASCII character except "/", triple-quotes, or "@".  Not used.<br/>MasterUsername:  The name for the master user. Not used.<br/>Engine:  The name of the database engine to be used for this instance. Valid Values: <code>neptune</code> <br/>DBInstanceClass:  The compute and memory capacity of the DB instance, for example, <code>db.m4.large</code>. Not all DB instance classes are available in all AWS Regions.<br/>AllocatedStorage:  The amount of storage (in gibibytes) to allocate for the DB instance. Type: Integer Not applicable. Neptune cluster volumes automatically grow as the amount of data in your database increases, though you are only charged for the space that you use in a Neptune cluster volume.<br/>DBInstanceIdentifier:  The DB instance identifier. This parameter is stored as a lowercase string. Constraints: <ul> <li> Must contain from 1 to 63 letters, numbers, or hyphens. </li> <li> First character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul> Example: <code>mydbinstance</code> <br/>DBName:  Not supported.<br/>
        :return: DBInstance:  Contains the details of an Amazon Neptune DB instance. This data type is used as a response element in the <a>DescribeDBInstances</a> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB instance is configured to export to CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsKMSKeyId</i> The AWS KMS key identifier for encryption of Performance Insights data. The KMS key ID is the Amazon Resource Name (ARN), KMS key identifier, or the KMS key alias for the KMS encryption key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsEnabled</i> True if Performance Insights is enabled for the DB instance, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if AWS Identity and Access Management (IAM) authentication is enabled, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timezone</i> Not supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceArn</i> The Amazon Resource Name (ARN) for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PromotionTier</i> A value that specifies the order in which a Read Replica is promoted to the primary instance after a failure of the existing primary instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringRoleArn</i> The ARN for the IAM role that permits Neptune to send Enhanced Monitoring metrics to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedMonitoringResourceArn</i> The Amazon Resource Name (ARN) of the Amazon CloudWatch Logs log stream that receives the Enhanced Monitoring metrics data for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringInterval</i> The interval, in seconds, between points when Enhanced Monitoring metrics are collected for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyTagsToSnapshot</i> Specifies whether tags are copied from the DB instance to snapshots of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainMemberships</i> Not supported<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> The identifier of the CA certificate for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbiResourceId</i> The AWS Region-unique, immutable identifier for the DB instance. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB instance is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i>  Not supported: The encryption for DB instances is managed by the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Not supported: The encryption for DB instances is managed by the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> If the DB instance is a member of a DB cluster, contains the name of the DB cluster that the DB instance is a member of.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbInstancePort</i> Specifies the port that the DB instance listens on. If the DB instance is part of a DB cluster, this can be a different port than the DB cluster port.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TdeCredentialArn</i> The ARN from the key store with which the instance is associated for TDE encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type associated with DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusInfos</i> The status of a Read Replica. If the instance is not a Read Replica, this is blank.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> This flag should no longer be used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondaryAvailabilityZone</i> If present, specifies the name of the secondary Availability Zone for a DB instance with multi-AZ support.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this instance is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionGroupMemberships</i> Provides the list of option group memberships for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the Provisioned IOPS (I/O operations per second) value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> License model information for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBClusterIdentifiers</i> Contains one or more identifiers of DB clusters that are Read Replicas of this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBInstanceIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaSourceDBInstanceIdentifier</i> Contains the identifier of the source DB instance if this DB instance is a Read Replica.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> Indicates that minor version patches are applied automatically.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies if the DB instance is a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DBSubnetGroup</b>: Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> Specifies the name of the Availability Zone the DB instance is located in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroups</i> Provides the list of DB parameter groups applied to this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security group elements that the DB instance belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSecurityGroups</i>  Provides List of DB security group elements containing only <code>DBSecurityGroup.Name</code> and <code>DBSecurityGroup.Status</code> subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i>  Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCreateTime</i> Provides the date and time the DB instance was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> Specifies the allocated storage size specified in gibibytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: Specifies the connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The database name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceStatus</i> Specifies the current state of this database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i> Contains the name of the compute and memory capacity class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i> Contains a user-supplied database identifier. This identifier is the unique key that identifies a DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupArn</i> The Amazon Resource Name (ARN) for the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i>  Contains a list of <a>Subnet</a> elements.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> Provides the status of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VpcId of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupDescription</i> Provides the description of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The name of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The new DB subnet group for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> Specifies the identifier of the CA certificate for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type to be associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i>  Contains the new <code>DBInstanceIdentifier</code> for the DB instance that will be applied or is currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the new Provisioned IOPS value for the DB instance that will be applied or is currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> The license model for the DB instance. Valid values: <code>license-included</code> | <code>bring-your-own-license</code> | <code>general-public-license</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Indicates that the Single-AZ DB instance is to change to a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the pending number of days for which automated backups are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the pending port for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> Contains the pending or currently-in-progress change of the master credentials for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i>  Contains the new <code>AllocatedStorage</code> size for the DB instance that will be applied or is currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i>  Contains the new <code>DBInstanceClass</code> for the DB instance that will be applied or is currently being applied.<br/>Specifies the CloudWatch logs to be exported.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingCloudwatchLogsExports</b>: Specifies the CloudWatch logs to be exported.        """
        pass

    def create_db_parameter_group(self, Description: str, DBParameterGroupFamily: str, DBParameterGroupName: str, Tags: List[Tag]=None) -> CreateDBParameterGroupResult:
        """Creates a new DB parameter group. A DB parameter group is initially created with the default parameters for the database engine used by the DB instance. To provide custom values for any of the parameters, you must modify the group after creating it using <i>ModifyDBParameterGroup</i>. Once you've created a DB parameter group, you need to associate it with your DB instance using <i>ModifyDBInstance</i>. When you associate a new DB parameter group with a running DB instance, you need to reboot the DB instance without failover for the new DB parameter group and associated settings to take effect. <important> After you create a DB parameter group, you should wait at least 5 minutes before creating your first DB instance that uses that DB parameter group as the default parameter group. This allows Amazon Neptune to fully complete the create action before the parameter group is used as the default for a new DB instance. This is especially important for parameters that are critical when creating the default database for a DB instance, such as the character set for the default database defined by the <code>character_set_database</code> parameter. You can use the <i>Parameter Groups</i> option of the Amazon Neptune console or the <i>DescribeDBParameters</i> command to verify that your DB parameter group has been created or modified. </important>

        :param self:
        :param Tags:  The tags to be assigned to the new DB parameter group.<br/>Description:  The description for the DB parameter group.<br/>DBParameterGroupFamily:  The DB parameter group family name. A DB parameter group can be associated with one and only one DB parameter group family, and can be applied only to a DB instance running a database engine and engine version compatible with that DB parameter group family.<br/>DBParameterGroupName:  The name of the DB parameter group. Constraints: <ul> <li> Must be 1 to 255 letters, numbers, or hyphens. </li> <li> First character must be a letter </li> <li> Cannot end with a hyphen or contain two consecutive hyphens </li> </ul> <note> This value is stored as a lowercase string. </note><br/>
        :return: DBParameterGroup:  Contains the details of an Amazon Neptune DB parameter group. This data type is used as a response element in the <a>DescribeDBParameterGroups</a> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroupArn</i> The Amazon Resource Name (ARN) for the DB parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> Provides the customer-specified description for this DB parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroupFamily</i> Provides the name of the DB parameter group family that this DB parameter group is compatible with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroupName</i> Provides the name of the DB parameter group.<br/>        """
        pass

    def create_db_subnet_group(self, SubnetIds: List[String], DBSubnetGroupDescription: str, DBSubnetGroupName: str, Tags: List[Tag]=None) -> CreateDBSubnetGroupResult:
        """Creates a new DB subnet group. DB subnet groups must contain at least one subnet in at least two AZs in the AWS Region.

        :param self:
        :param Tags:  The tags to be assigned to the new DB subnet group.<br/>SubnetIds:  The EC2 Subnet IDs for the DB subnet group.<br/>DBSubnetGroupDescription:  The description for the DB subnet group.<br/>DBSubnetGroupName:  The name for the DB subnet group. This value is stored as a lowercase string. Constraints: Must contain no more than 255 letters, numbers, periods, underscores, spaces, or hyphens. Must not be default. Example: <code>mySubnetgroup</code> <br/>
        :return: DBSubnetGroup:  Contains the details of an Amazon Neptune DB subnet group. This data type is used as a response element in the <a>DescribeDBSubnetGroups</a> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupArn</i> The Amazon Resource Name (ARN) for the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i>  Contains a list of <a>Subnet</a> elements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> Provides the status of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VpcId of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupDescription</i> Provides the description of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The name of the DB subnet group.<br/>        """
        pass

    def create_event_subscription(self, SnsTopicArn: str, SubscriptionName: str, SourceType: str=None, EventCategories: List[String]=None, SourceIds: List[String]=None, Enabled: BooleanOptional=None, Tags: List[Tag]=None) -> CreateEventSubscriptionResult:
        """Creates an event notification subscription. This action requires a topic ARN (Amazon Resource Name) created by either the Neptune console, the SNS console, or the SNS API. To obtain an ARN with SNS, you must create a topic in Amazon SNS and subscribe to the topic. The ARN is displayed in the SNS console. You can specify the type of source (SourceType) you want to be notified of, provide a list of Neptune sources (SourceIds) that triggers the events, and provide a list of event categories (EventCategories) for events you want to be notified of. For example, you can specify SourceType = db-instance, SourceIds = mydbinstance1, mydbinstance2 and EventCategories = Availability, Backup. If you specify both the SourceType and SourceIds, such as SourceType = db-instance and SourceIdentifier = myDBInstance1, you are notified of all the db-instance events for the specified source. If you specify a SourceType but do not specify a SourceIdentifier, you receive notice of the events for that source type for all your Neptune sources. If you do not specify either the SourceType nor the SourceIdentifier, you are notified of events generated from all Neptune sources belonging to your customer account.

        :param self:
        :param Tags:  The tags to be applied to the new event subscription.<br/>Enabled:   A Boolean value; set to <b>true</b> to activate the subscription, set to <b>false</b> to create the subscription but not active it.<br/>SourceIds:  The list of identifiers of the event sources for which events are returned. If not specified, then all sources are included in the response. An identifier must begin with a letter and must contain only ASCII letters, digits, and hyphens; it can't end with a hyphen or contain two consecutive hyphens. Constraints: <ul> <li> If SourceIds are supplied, SourceType must also be provided. </li> <li> If the source type is a DB instance, then a <code>DBInstanceIdentifier</code> must be supplied. </li> <li> If the source type is a DB security group, a <code>DBSecurityGroupName</code> must be supplied. </li> <li> If the source type is a DB parameter group, a <code>DBParameterGroupName</code> must be supplied. </li> <li> If the source type is a DB snapshot, a <code>DBSnapshotIdentifier</code> must be supplied. </li> </ul><br/>EventCategories:   A list of event categories for a SourceType that you want to subscribe to. You can see a list of the categories for a given SourceType by using the <b>DescribeEventCategories</b> action.<br/>SourceType:  The type of source that is generating the events. For example, if you want to be notified of events generated by a DB instance, you would set this parameter to db-instance. if this value is not specified, all events are returned. Valid values: <code>db-instance</code> | <code>db-cluster</code> | <code>db-parameter-group</code> | <code>db-security-group</code> | <code>db-snapshot</code> | <code>db-cluster-snapshot</code> <br/>SnsTopicArn:  The Amazon Resource Name (ARN) of the SNS topic created for event notification. The ARN is created by Amazon SNS when you create a topic and subscribe to it.<br/>SubscriptionName:  The name of the subscription. Constraints: The name must be less than 255 characters.<br/>
        :return: EventSubscription:  Contains the results of a successful invocation of the <a>DescribeEventSubscriptions</a> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EventSubscriptionArn</i> The Amazon Resource Name (ARN) for the event subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> A Boolean value indicating if the subscription is enabled. True indicates the subscription is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EventCategoriesList</i> A list of event categories for the event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceIdsList</i> A list of source IDs for the event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceType</i> The source type for the event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubscriptionCreationTime</i> The time the event notification subscription was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the event notification subscription. Constraints: Can be one of the following: creating | modifying | deleting | active | no-permission | topic-not-exist The status "no-permission" indicates that Neptune no longer has permission to post to the SNS topic. The status "topic-not-exist" indicates that the topic was deleted after the subscription was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsTopicArn</i> The topic ARN of the event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustSubscriptionId</i> The event notification subscription Id.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomerAwsId</i> The AWS customer account associated with the event notification subscription.<br/>        """
        pass

    def delete_db_cluster(self, DBClusterIdentifier: str, SkipFinalSnapshot: Boolean=None, FinalDBSnapshotIdentifier: str=None) -> DeleteDBClusterResult:
        """The DeleteDBCluster action deletes a previously provisioned DB cluster. When you delete a DB cluster, all automated backups for that DB cluster are deleted and can't be recovered. Manual DB cluster snapshots of the specified DB cluster are not deleted.

        :param self:
        :param FinalDBSnapshotIdentifier:   The DB cluster snapshot identifier of the new DB cluster snapshot created when <code>SkipFinalSnapshot</code> is set to <code>false</code>. <note>  Specifying this parameter and also setting the <code>SkipFinalShapshot</code> parameter to true results in an error. </note> Constraints: <ul> <li> Must be 1 to 255 letters, numbers, or hyphens. </li> <li> First character must be a letter </li> <li> Cannot end with a hyphen or contain two consecutive hyphens </li> </ul><br/>SkipFinalSnapshot:   Determines whether a final DB cluster snapshot is created before the DB cluster is deleted. If <code>true</code> is specified, no DB cluster snapshot is created. If <code>false</code> is specified, a DB cluster snapshot is created before the DB cluster is deleted. <note> You must specify a <code>FinalDBSnapshotIdentifier</code> parameter if <code>SkipFinalSnapshot</code> is <code>false</code>. </note> Default: <code>false</code> <br/>DBClusterIdentifier:  The DB cluster identifier for the DB cluster to be deleted. This parameter isn't case-sensitive. Constraints: <ul> <li> Must match an existing DBClusterIdentifier. </li> </ul><br/>
        :return: DBCluster:  Contains the details of an Amazon Neptune DB cluster. This data type is used as a response element in the <a>DescribeDBClusters</a> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB cluster is configured to export to CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloneGroupId</i> Identifies the clone group to which the DB cluster is associated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i> Provides a list of the AWS Identity and Access Management (IAM) roles that are associated with the DB cluster. IAM roles that are associated with a DB cluster grant permission for the DB cluster to access other AWS services on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterArn</i> The Amazon Resource Name (ARN) for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbClusterResourceId</i> The AWS Region-unique, immutable identifier for the DB cluster. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB cluster is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is true, the AWS KMS key identifier for the encrypted DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security groups that the DB cluster belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterMembers</i> Provides the list of instances that make up the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSourceIdentifier</i> Not supported by Neptune.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i> Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterOptionGroupMemberships</i> Provides the list of option group memberships for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies whether the DB cluster has instances in multiple Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReaderEndpoint</i> The reader endpoint for the DB cluster. The reader endpoint for a DB cluster load-balances connections across the Read Replicas that are available in a DB cluster. As clients request new connections to the reader endpoint, Neptune distributes the connection requests among the Read Replicas in the DB cluster. This functionality can help balance your read workload across multiple Read Replicas in your DB cluster. If a failover occurs, and the Read Replica that you are connected to is promoted to be the primary instance, your connection is dropped. To continue sending your read workload to other Read Replicas in the cluster, you can then reconnect to the reader endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint for the primary instance of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestRestorableTime</i> Specifies the earliest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the progress of the operation as a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the current state of this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB cluster, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroup</i> Specifies the name of the DB cluster parameter group for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Contains a user-supplied DB cluster identifier. This identifier is the unique key that identifies a DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> Contains the name of the initial database of this DB cluster that was provided at create time, if one was specified when the DB cluster was created. This same name is returned for the life of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this cluster is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of EC2 Availability Zones that instances in the DB cluster can be created in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i>  <code>AllocatedStorage</code> always returns 1, because Neptune DB cluster storage size is not fixed, but instead automatically adjusts as needed.<br/>        """
        pass

    def delete_db_cluster_parameter_group(self, DBClusterParameterGroupName: str):
        """Deletes a specified DB cluster parameter group. The DB cluster parameter group to be deleted can't be associated with any DB clusters.

        :param self:
        :param DBClusterParameterGroupName:  The name of the DB cluster parameter group. Constraints: <ul> <li> Must be the name of an existing DB cluster parameter group. </li> <li> You can't delete a default DB cluster parameter group. </li> <li> Cannot be associated with any DB clusters. </li> </ul><br/>
        :return:         """
        pass

    def delete_db_cluster_snapshot(self, DBClusterSnapshotIdentifier: str) -> DeleteDBClusterSnapshotResult:
        """Deletes a DB cluster snapshot. If the snapshot is being copied, the copy operation is terminated. <note> The DB cluster snapshot must be in the <code>available</code> state to be deleted. </note>

        :param self:
        :param DBClusterSnapshotIdentifier:  The identifier of the DB cluster snapshot to delete. Constraints: Must be the name of an existing DB cluster snapshot in the <code>available</code> state.<br/>
        :return: DBClusterSnapshot:  Contains the details for an Amazon Neptune DB cluster snapshot This data type is used as a response element in the <a>DescribeDBClusterSnapshots</a> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceDBClusterSnapshotArn</i> If the DB cluster snapshot was copied from a source DB cluster snapshot, the Amazon Resource Name (ARN) for the source DB cluster snapshot, otherwise, a null value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotArn</i> The Amazon Resource Name (ARN) for the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is true, the AWS KMS key identifier for the encrypted DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster snapshot is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the percentage of the estimated data that has been transferred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotType</i> Provides the type of the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> Provides the license model information for this DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Provides the version of the database engine for this DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Provides the master username for the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VPC ID associated with the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the DB cluster was listening on at the time of the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the status of this DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> Specifies the allocated storage size in gibibytes (GiB).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Specifies the name of the database engine.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCreateTime</i> Provides the time when the snapshot was taken, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Specifies the DB cluster identifier of the DB cluster that this DB cluster snapshot was created from.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotIdentifier</i> Specifies the identifier for the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of EC2 Availability Zones that instances in the DB cluster snapshot can be restored in.<br/>        """
        pass

    def delete_db_instance(self, DBInstanceIdentifier: str, SkipFinalSnapshot: Boolean=None, FinalDBSnapshotIdentifier: str=None) -> DeleteDBInstanceResult:
        """The DeleteDBInstance action deletes a previously provisioned DB instance. When you delete a DB instance, all automated backups for that instance are deleted and can't be recovered. Manual DB snapshots of the DB instance to be deleted by <code>DeleteDBInstance</code> are not deleted.  If you request a final DB snapshot the status of the Amazon Neptune DB instance is <code>deleting</code> until the DB snapshot is created. The API action <code>DescribeDBInstance</code> is used to monitor the status of this operation. The action can't be canceled or reverted once submitted. Note that when a DB instance is in a failure state and has a status of <code>failed</code>, <code>incompatible-restore</code>, or <code>incompatible-network</code>, you can only delete it when the <code>SkipFinalSnapshot</code> parameter is set to <code>true</code>. You can't delete a DB instance if it is the only instance in the DB cluster.

        :param self:
        :param FinalDBSnapshotIdentifier:   The DBSnapshotIdentifier of the new DBSnapshot created when SkipFinalSnapshot is set to <code>false</code>. <note> Specifying this parameter and also setting the SkipFinalShapshot parameter to true results in an error. </note> Constraints: <ul> <li> Must be 1 to 255 letters or numbers. </li> <li> First character must be a letter </li> <li> Cannot end with a hyphen or contain two consecutive hyphens </li> <li> Cannot be specified when deleting a Read Replica. </li> </ul><br/>SkipFinalSnapshot:   Determines whether a final DB snapshot is created before the DB instance is deleted. If <code>true</code> is specified, no DBSnapshot is created. If <code>false</code> is specified, a DB snapshot is created before the DB instance is deleted. Note that when a DB instance is in a failure state and has a status of 'failed', 'incompatible-restore', or 'incompatible-network', it can only be deleted when the SkipFinalSnapshot parameter is set to "true". Specify <code>true</code> when deleting a Read Replica. <note> The FinalDBSnapshotIdentifier parameter must be specified if SkipFinalSnapshot is <code>false</code>. </note> Default: <code>false</code> <br/>DBInstanceIdentifier:  The DB instance identifier for the DB instance to be deleted. This parameter isn't case-sensitive. Constraints: <ul> <li> Must match the name of an existing DB instance. </li> </ul><br/>
        :return: DBInstance:  Contains the details of an Amazon Neptune DB instance. This data type is used as a response element in the <a>DescribeDBInstances</a> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB instance is configured to export to CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsKMSKeyId</i> The AWS KMS key identifier for encryption of Performance Insights data. The KMS key ID is the Amazon Resource Name (ARN), KMS key identifier, or the KMS key alias for the KMS encryption key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsEnabled</i> True if Performance Insights is enabled for the DB instance, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if AWS Identity and Access Management (IAM) authentication is enabled, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timezone</i> Not supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceArn</i> The Amazon Resource Name (ARN) for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PromotionTier</i> A value that specifies the order in which a Read Replica is promoted to the primary instance after a failure of the existing primary instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringRoleArn</i> The ARN for the IAM role that permits Neptune to send Enhanced Monitoring metrics to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedMonitoringResourceArn</i> The Amazon Resource Name (ARN) of the Amazon CloudWatch Logs log stream that receives the Enhanced Monitoring metrics data for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringInterval</i> The interval, in seconds, between points when Enhanced Monitoring metrics are collected for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyTagsToSnapshot</i> Specifies whether tags are copied from the DB instance to snapshots of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainMemberships</i> Not supported<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> The identifier of the CA certificate for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbiResourceId</i> The AWS Region-unique, immutable identifier for the DB instance. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB instance is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i>  Not supported: The encryption for DB instances is managed by the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Not supported: The encryption for DB instances is managed by the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> If the DB instance is a member of a DB cluster, contains the name of the DB cluster that the DB instance is a member of.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbInstancePort</i> Specifies the port that the DB instance listens on. If the DB instance is part of a DB cluster, this can be a different port than the DB cluster port.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TdeCredentialArn</i> The ARN from the key store with which the instance is associated for TDE encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type associated with DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusInfos</i> The status of a Read Replica. If the instance is not a Read Replica, this is blank.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> This flag should no longer be used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondaryAvailabilityZone</i> If present, specifies the name of the secondary Availability Zone for a DB instance with multi-AZ support.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this instance is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionGroupMemberships</i> Provides the list of option group memberships for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the Provisioned IOPS (I/O operations per second) value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> License model information for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBClusterIdentifiers</i> Contains one or more identifiers of DB clusters that are Read Replicas of this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBInstanceIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaSourceDBInstanceIdentifier</i> Contains the identifier of the source DB instance if this DB instance is a Read Replica.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> Indicates that minor version patches are applied automatically.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies if the DB instance is a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DBSubnetGroup</b>: Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> Specifies the name of the Availability Zone the DB instance is located in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroups</i> Provides the list of DB parameter groups applied to this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security group elements that the DB instance belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSecurityGroups</i>  Provides List of DB security group elements containing only <code>DBSecurityGroup.Name</code> and <code>DBSecurityGroup.Status</code> subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i>  Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCreateTime</i> Provides the date and time the DB instance was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> Specifies the allocated storage size specified in gibibytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: Specifies the connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The database name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceStatus</i> Specifies the current state of this database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i> Contains the name of the compute and memory capacity class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i> Contains a user-supplied database identifier. This identifier is the unique key that identifies a DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupArn</i> The Amazon Resource Name (ARN) for the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i>  Contains a list of <a>Subnet</a> elements.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> Provides the status of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VpcId of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupDescription</i> Provides the description of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The name of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The new DB subnet group for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> Specifies the identifier of the CA certificate for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type to be associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i>  Contains the new <code>DBInstanceIdentifier</code> for the DB instance that will be applied or is currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the new Provisioned IOPS value for the DB instance that will be applied or is currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> The license model for the DB instance. Valid values: <code>license-included</code> | <code>bring-your-own-license</code> | <code>general-public-license</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Indicates that the Single-AZ DB instance is to change to a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the pending number of days for which automated backups are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the pending port for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> Contains the pending or currently-in-progress change of the master credentials for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i>  Contains the new <code>AllocatedStorage</code> size for the DB instance that will be applied or is currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i>  Contains the new <code>DBInstanceClass</code> for the DB instance that will be applied or is currently being applied.<br/>Specifies the CloudWatch logs to be exported.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingCloudwatchLogsExports</b>: Specifies the CloudWatch logs to be exported.        """
        pass

    def delete_db_parameter_group(self, DBParameterGroupName: str):
        """Deletes a specified DBParameterGroup. The DBParameterGroup to be deleted can't be associated with any DB instances.

        :param self:
        :param DBParameterGroupName:  The name of the DB parameter group. Constraints: <ul> <li> Must be the name of an existing DB parameter group </li> <li> You can't delete a default DB parameter group </li> <li> Cannot be associated with any DB instances </li> </ul><br/>
        :return:         """
        pass

    def delete_db_subnet_group(self, DBSubnetGroupName: str):
        """Deletes a DB subnet group. <note> The specified database subnet group must not be associated with any DB instances. </note>

        :param self:
        :param DBSubnetGroupName:  The name of the database subnet group to delete. <note> You can't delete the default subnet group. </note> Constraints: Constraints: Must match the name of an existing DBSubnetGroup. Must not be default. Example: <code>mySubnetgroup</code> <br/>
        :return:         """
        pass

    def delete_event_subscription(self, SubscriptionName: str) -> DeleteEventSubscriptionResult:
        """Deletes an event notification subscription.

        :param self:
        :param SubscriptionName:  The name of the event notification subscription you want to delete.<br/>
        :return: EventSubscription:  Contains the results of a successful invocation of the <a>DescribeEventSubscriptions</a> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EventSubscriptionArn</i> The Amazon Resource Name (ARN) for the event subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> A Boolean value indicating if the subscription is enabled. True indicates the subscription is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EventCategoriesList</i> A list of event categories for the event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceIdsList</i> A list of source IDs for the event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceType</i> The source type for the event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubscriptionCreationTime</i> The time the event notification subscription was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the event notification subscription. Constraints: Can be one of the following: creating | modifying | deleting | active | no-permission | topic-not-exist The status "no-permission" indicates that Neptune no longer has permission to post to the SNS topic. The status "topic-not-exist" indicates that the topic was deleted after the subscription was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsTopicArn</i> The topic ARN of the event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustSubscriptionId</i> The event notification subscription Id.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomerAwsId</i> The AWS customer account associated with the event notification subscription.<br/>        """
        pass

    def describe_db_cluster_parameter_groups(self, DBClusterParameterGroupName: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DBClusterParameterGroupsMessage:
        """ Returns a list of <code>DBClusterParameterGroup</code> descriptions. If a <code>DBClusterParameterGroupName</code> parameter is specified, the list will contain only the description of the specified DB cluster parameter group.

        :param self:
        :param Marker:   An optional pagination token provided by a previous <code>DescribeDBClusterParameterGroups</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>DBClusterParameterGroupName:  The name of a specific DB cluster parameter group to return details for. Constraints: <ul> <li> If supplied, must match the name of an existing DBClusterParameterGroup. </li> </ul><br/>
        :return: DBClusterParameterGroups:  A list of DB cluster parameter groups.<br/>Marker:   An optional pagination token provided by a previous <code>DescribeDBClusterParameterGroups</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>        """
        pass

    def describe_db_cluster_parameters(self, DBClusterParameterGroupName: str, Source: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DBClusterParameterGroupDetails:
        """Returns the detailed parameter list for a particular DB cluster parameter group.

        :param self:
        :param Marker:   An optional pagination token provided by a previous <code>DescribeDBClusterParameters</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>Source:   A value that indicates to return only parameters for a specific source. Parameter sources can be <code>engine</code>, <code>service</code>, or <code>customer</code>.<br/>DBClusterParameterGroupName:  The name of a specific DB cluster parameter group to return parameter details for. Constraints: <ul> <li> If supplied, must match the name of an existing DBClusterParameterGroup. </li> </ul><br/>
        :return: Marker:   An optional pagination token provided by a previous DescribeDBClusterParameters request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code> .<br/>Parameters:  Provides a list of parameters for the DB cluster parameter group.<br/>        """
        pass

    def describe_db_cluster_snapshot_attributes(self, DBClusterSnapshotIdentifier: str) -> DescribeDBClusterSnapshotAttributesResult:
        """Returns a list of DB cluster snapshot attribute names and values for a manual DB cluster snapshot. When sharing snapshots with other AWS accounts, <code>DescribeDBClusterSnapshotAttributes</code> returns the <code>restore</code> attribute and a list of IDs for the AWS accounts that are authorized to copy or restore the manual DB cluster snapshot. If <code>all</code> is included in the list of values for the <code>restore</code> attribute, then the manual DB cluster snapshot is public and can be copied or restored by all AWS accounts. To add or remove access for an AWS account to copy or restore a manual DB cluster snapshot, or to make the manual DB cluster snapshot public or private, use the <a>ModifyDBClusterSnapshotAttribute</a> API action.

        :param self:
        :param DBClusterSnapshotIdentifier:  The identifier for the DB cluster snapshot to describe the attributes for.<br/>
        :return: DBClusterSnapshotAttributesResult:  Contains the results of a successful call to the <a>DescribeDBClusterSnapshotAttributes</a> API action. Manual DB cluster snapshot attributes are used to authorize other AWS accounts to copy or restore a manual DB cluster snapshot. For more information, see the <a>ModifyDBClusterSnapshotAttribute</a> API action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotAttributes</i> The list of attributes and values for the manual DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotIdentifier</i> The identifier of the manual DB cluster snapshot that the attributes apply to.<br/>        """
        pass

    def describe_db_cluster_snapshots(self, DBClusterIdentifier: str=None, DBClusterSnapshotIdentifier: str=None, SnapshotType: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None, IncludeShared: Boolean=None, IncludePublic: Boolean=None) -> DBClusterSnapshotMessage:
        """Returns information about DB cluster snapshots. This API action supports pagination.

        :param self:
        :param IncludePublic:  True to include manual DB cluster snapshots that are public and can be copied or restored by any AWS account, and otherwise false. The default is <code>false</code>. The default is false. You can share a manual DB cluster snapshot as public by using the <a>ModifyDBClusterSnapshotAttribute</a> API action.<br/>IncludeShared:  True to include shared manual DB cluster snapshots from other AWS accounts that this AWS account has been given permission to copy or restore, and otherwise false. The default is <code>false</code>. You can give an AWS account permission to restore a manual DB cluster snapshot from another AWS account by the <a>ModifyDBClusterSnapshotAttribute</a> API action.<br/>Marker:  An optional pagination token provided by a previous <code>DescribeDBClusterSnapshots</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:  The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>SnapshotType:  The type of DB cluster snapshots to be returned. You can specify one of the following values: <ul> <li>  <code>automated</code> - Return all DB cluster snapshots that have been automatically taken by Amazon Neptune for my AWS account. </li> <li>  <code>manual</code> - Return all DB cluster snapshots that have been taken by my AWS account. </li> <li>  <code>shared</code> - Return all manual DB cluster snapshots that have been shared to my AWS account. </li> <li>  <code>public</code> - Return all DB cluster snapshots that have been marked as public. </li> </ul> If you don't specify a <code>SnapshotType</code> value, then both automated and manual DB cluster snapshots are returned. You can include shared DB cluster snapshots with these results by setting the <code>IncludeShared</code> parameter to <code>true</code>. You can include public DB cluster snapshots with these results by setting the <code>IncludePublic</code> parameter to <code>true</code>. The <code>IncludeShared</code> and <code>IncludePublic</code> parameters don't apply for <code>SnapshotType</code> values of <code>manual</code> or <code>automated</code>. The <code>IncludePublic</code> parameter doesn't apply when <code>SnapshotType</code> is set to <code>shared</code>. The <code>IncludeShared</code> parameter doesn't apply when <code>SnapshotType</code> is set to <code>public</code>.<br/>DBClusterSnapshotIdentifier:  A specific DB cluster snapshot identifier to describe. This parameter can't be used in conjunction with the <code>DBClusterIdentifier</code> parameter. This value is stored as a lowercase string. Constraints: <ul> <li> If supplied, must match the identifier of an existing DBClusterSnapshot. </li> <li> If this identifier is for an automated snapshot, the <code>SnapshotType</code> parameter must also be specified. </li> </ul><br/>DBClusterIdentifier:  The ID of the DB cluster to retrieve the list of DB cluster snapshots for. This parameter can't be used in conjunction with the <code>DBClusterSnapshotIdentifier</code> parameter. This parameter is not case-sensitive. Constraints: <ul> <li> If supplied, must match the identifier of an existing DBCluster. </li> </ul><br/>
        :return: DBClusterSnapshots:  Provides a list of DB cluster snapshots for the user.<br/>Marker:   An optional pagination token provided by a previous <a>DescribeDBClusterSnapshots</a> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>        """
        pass

    def describe_db_clusters(self, DBClusterIdentifier: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DBClusterMessage:
        """Returns information about provisioned DB clusters. This API supports pagination.

        :param self:
        :param Marker:  An optional pagination token provided by a previous <a>DescribeDBClusters</a> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:  The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  A filter that specifies one or more DB clusters to describe. Supported filters: <ul> <li>  <code>db-cluster-id</code> - Accepts DB cluster identifiers and DB cluster Amazon Resource Names (ARNs). The results list will only include information about the DB clusters identified by these ARNs. </li> </ul><br/>DBClusterIdentifier:  The user-supplied DB cluster identifier. If this parameter is specified, information from only the specific DB cluster is returned. This parameter isn't case-sensitive. Constraints: <ul> <li> If supplied, must match an existing DBClusterIdentifier. </li> </ul><br/>
        :return: DBClusters:  Contains a list of DB clusters for the user.<br/>Marker:  A pagination token that can be used in a subsequent DescribeDBClusters request.<br/>        """
        pass

    def describe_db_engine_versions(self, Engine: str=None, EngineVersion: str=None, DBParameterGroupFamily: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None, DefaultOnly: Boolean=None, ListSupportedCharacterSets: BooleanOptional=None, ListSupportedTimezones: BooleanOptional=None) -> DBEngineVersionMessage:
        """Returns a list of the available DB engines.

        :param self:
        :param ListSupportedTimezones:  If this parameter is specified and the requested engine supports the <code>TimeZone</code> parameter for <code>CreateDBInstance</code>, the response includes a list of supported time zones for each engine version.<br/>ListSupportedCharacterSets:  If this parameter is specified and the requested engine supports the <code>CharacterSetName</code> parameter for <code>CreateDBInstance</code>, the response includes a list of supported character sets for each engine version.<br/>DefaultOnly:  Indicates that only the default version of the specified engine or engine and major version combination is returned.<br/>Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:   The maximum number of records to include in the response. If more than the <code>MaxRecords</code> value is available, a pagination token called a marker is included in the response so that the following results can be retrieved. Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  Not currently supported.<br/>DBParameterGroupFamily:  The name of a specific DB parameter group family to return details for. Constraints: <ul> <li> If supplied, must match an existing DBParameterGroupFamily. </li> </ul><br/>EngineVersion:  The database engine version to return. Example: <code>5.1.49</code> <br/>Engine:  The database engine to return.<br/>
        :return: DBEngineVersions:   A list of <code>DBEngineVersion</code> elements.<br/>Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>        """
        pass

    def describe_db_instances(self, DBInstanceIdentifier: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DBInstanceMessage:
        """Returns information about provisioned instances. This API supports pagination.

        :param self:
        :param Marker:   An optional pagination token provided by a previous <code>DescribeDBInstances</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  A filter that specifies one or more DB instances to describe. Supported filters: <ul> <li>  <code>db-cluster-id</code> - Accepts DB cluster identifiers and DB cluster Amazon Resource Names (ARNs). The results list will only include information about the DB instances associated with the DB clusters identified by these ARNs. </li> <li>  <code>db-instance-id</code> - Accepts DB instance identifiers and DB instance Amazon Resource Names (ARNs). The results list will only include information about the DB instances identified by these ARNs. </li> </ul><br/>DBInstanceIdentifier:  The user-supplied instance identifier. If this parameter is specified, information from only the specific DB instance is returned. This parameter isn't case-sensitive. Constraints: <ul> <li> If supplied, must match the identifier of an existing DBInstance. </li> </ul><br/>
        :return: DBInstances:   A list of <a>DBInstance</a> instances.<br/>Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code> .<br/>        """
        pass

    def describe_db_parameter_groups(self, DBParameterGroupName: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DBParameterGroupsMessage:
        """Returns a list of <code>DBParameterGroup</code> descriptions. If a <code>DBParameterGroupName</code> is specified, the list will contain only the description of the specified DB parameter group.

        :param self:
        :param Marker:  An optional pagination token provided by a previous <code>DescribeDBParameterGroups</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:  The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>DBParameterGroupName:  The name of a specific DB parameter group to return details for. Constraints: <ul> <li> If supplied, must match the name of an existing DBClusterParameterGroup. </li> </ul><br/>
        :return: DBParameterGroups:  A list of <a>DBParameterGroup</a> instances.<br/>Marker:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>        """
        pass

    def describe_db_parameters(self, DBParameterGroupName: str, Source: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DBParameterGroupDetails:
        """Returns the detailed parameter list for a particular DB parameter group.

        :param self:
        :param Marker:  An optional pagination token provided by a previous <code>DescribeDBParameters</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:  The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>Source:  The parameter types to return. Default: All parameter types returned Valid Values: <code>user | system | engine-default</code> <br/>DBParameterGroupName:  The name of a specific DB parameter group to return details for. Constraints: <ul> <li> If supplied, must match the name of an existing DBParameterGroup. </li> </ul><br/>
        :return: Marker:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>Parameters:  A list of <a>Parameter</a> values.<br/>        """
        pass

    def describe_db_subnet_groups(self, DBSubnetGroupName: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DBSubnetGroupMessage:
        """Returns a list of DBSubnetGroup descriptions. If a DBSubnetGroupName is specified, the list will contain only the descriptions of the specified DBSubnetGroup. For an overview of CIDR ranges, go to the <a href="http://en.wikipedia.org/wiki/Classless_Inter-Domain_Routing">Wikipedia Tutorial</a>.

        :param self:
        :param Marker:   An optional pagination token provided by a previous DescribeDBSubnetGroups request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>DBSubnetGroupName:  The name of the DB subnet group to return details for.<br/>
        :return: DBSubnetGroups:   A list of <a>DBSubnetGroup</a> instances.<br/>Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>        """
        pass

    def describe_engine_default_cluster_parameters(self, DBParameterGroupFamily: str, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DescribeEngineDefaultClusterParametersResult:
        """Returns the default engine and system parameter information for the cluster database engine.

        :param self:
        :param Marker:   An optional pagination token provided by a previous <code>DescribeEngineDefaultClusterParameters</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>DBParameterGroupFamily:  The name of the DB cluster parameter group family to return engine parameter information for.<br/>
        :return: EngineDefaults:   Contains the result of a successful invocation of the <a>DescribeEngineDefaultParameters</a> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> Contains a list of engine default parameters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Marker</i>  An optional pagination token provided by a previous EngineDefaults request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code> .<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroupFamily</i> Specifies the name of the DB parameter group family that the engine default parameters apply to.<br/>        """
        pass

    def describe_engine_default_parameters(self, DBParameterGroupFamily: str, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DescribeEngineDefaultParametersResult:
        """Returns the default engine and system parameter information for the specified database engine.

        :param self:
        :param Marker:   An optional pagination token provided by a previous <code>DescribeEngineDefaultParameters</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  Not currently supported.<br/>DBParameterGroupFamily:  The name of the DB parameter group family.<br/>
        :return: EngineDefaults:   Contains the result of a successful invocation of the <a>DescribeEngineDefaultParameters</a> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> Contains a list of engine default parameters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Marker</i>  An optional pagination token provided by a previous EngineDefaults request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code> .<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroupFamily</i> Specifies the name of the DB parameter group family that the engine default parameters apply to.<br/>        """
        pass

    def describe_event_categories(self, SourceType: str=None, Filters: List[Filter]=None) -> EventCategoriesMessage:
        """Displays a list of categories for all event source types, or, if specified, for a specified source type.

        :param self:
        :param Filters:  This parameter is not currently supported.<br/>SourceType:  The type of source that is generating the events. Valid values: db-instance | db-parameter-group | db-security-group | db-snapshot<br/>
        :return: EventCategoriesMapList:  A list of EventCategoriesMap data types.<br/>        """
        pass

    def describe_event_subscriptions(self, SubscriptionName: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> EventSubscriptionsMessage:
        """Lists all the subscription descriptions for a customer account. The description for a subscription includes SubscriptionName, SNSTopicARN, CustomerID, SourceType, SourceID, CreationTime, and Status. If you specify a SubscriptionName, lists the description for that subscription.

        :param self:
        :param Marker:   An optional pagination token provided by a previous DescribeOrderableDBInstanceOptions request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code> .<br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>SubscriptionName:  The name of the event notification subscription you want to describe.<br/>
        :return: EventSubscriptionsList:  A list of EventSubscriptions data types.<br/>Marker:   An optional pagination token provided by a previous DescribeOrderableDBInstanceOptions request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>        """
        pass

    def describe_events(self, SourceIdentifier: str=None, SourceType: str=None, StartTime: TStamp=None, EndTime: TStamp=None, Duration: int=None, EventCategories: List[String]=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> EventsMessage:
        """Returns events related to DB instances, DB security groups, DB snapshots, and DB parameter groups for the past 14 days. Events specific to a particular DB instance, DB security group, database snapshot, or DB parameter group can be obtained by providing the name as a parameter. By default, the past hour of events are returned.

        :param self:
        :param Marker:   An optional pagination token provided by a previous DescribeEvents request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>EventCategories:  A list of event categories that trigger notifications for a event notification subscription.<br/>Duration:  The number of minutes to retrieve events for. Default: 60<br/>EndTime:   The end of the time interval for which to retrieve events, specified in ISO 8601 format. For more information about ISO 8601, go to the <a href="http://en.wikipedia.org/wiki/ISO_8601">ISO8601 Wikipedia page.</a>  Example: 2009-07-08T18:00Z<br/>StartTime:   The beginning of the time interval to retrieve events for, specified in ISO 8601 format. For more information about ISO 8601, go to the <a href="http://en.wikipedia.org/wiki/ISO_8601">ISO8601 Wikipedia page.</a>  Example: 2009-07-08T18:00Z<br/>SourceType:  The event source to retrieve events for. If no value is specified, all events are returned.<br/>SourceIdentifier:  The identifier of the event source for which events are returned. If not specified, then all sources are included in the response. Constraints: <ul> <li> If SourceIdentifier is supplied, SourceType must also be provided. </li> <li> If the source type is <code>DBInstance</code>, then a <code>DBInstanceIdentifier</code> must be supplied. </li> <li> If the source type is <code>DBSecurityGroup</code>, a <code>DBSecurityGroupName</code> must be supplied. </li> <li> If the source type is <code>DBParameterGroup</code>, a <code>DBParameterGroupName</code> must be supplied. </li> <li> If the source type is <code>DBSnapshot</code>, a <code>DBSnapshotIdentifier</code> must be supplied. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul><br/>
        :return: Events:   A list of <a>Event</a> instances.<br/>Marker:   An optional pagination token provided by a previous Events request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code> .<br/>        """
        pass

    def describe_orderable_db_instance_options(self, Engine: str, EngineVersion: str=None, DBInstanceClass: str=None, LicenseModel: str=None, Vpc: BooleanOptional=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> OrderableDBInstanceOptionsMessage:
        """Returns a list of orderable DB instance options for the specified engine.

        :param self:
        :param Marker:   An optional pagination token provided by a previous DescribeOrderableDBInstanceOptions request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code> .<br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>Vpc:  The VPC filter value. Specify this parameter to show only the available VPC or non-VPC offerings.<br/>LicenseModel:  The license model filter value. Specify this parameter to show only the available offerings matching the specified license model.<br/>DBInstanceClass:  The DB instance class filter value. Specify this parameter to show only the available offerings matching the specified DB instance class.<br/>EngineVersion:  The engine version filter value. Specify this parameter to show only the available offerings matching the specified engine version.<br/>Engine:  The name of the engine to retrieve DB instance options for.<br/>
        :return: Marker:   An optional pagination token provided by a previous OrderableDBInstanceOptions request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code> .<br/>OrderableDBInstanceOptions:  An <a>OrderableDBInstanceOption</a> structure containing information about orderable options for the DB instance.<br/>        """
        pass

    def describe_pending_maintenance_actions(self, ResourceIdentifier: str=None, Filters: List[Filter]=None, Marker: str=None, MaxRecords: int=None) -> PendingMaintenanceActionsMessage:
        """Returns a list of resources (for example, DB instances) that have at least one pending maintenance action.

        :param self:
        :param MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: Minimum 20, maximum 100.<br/>Marker:   An optional pagination token provided by a previous <code>DescribePendingMaintenanceActions</code> request. If this parameter is specified, the response includes only records beyond the marker, up to a number of records specified by <code>MaxRecords</code>.<br/>Filters:  A filter that specifies one or more resources to return pending maintenance actions for. Supported filters: <ul> <li>  <code>db-cluster-id</code> - Accepts DB cluster identifiers and DB cluster Amazon Resource Names (ARNs). The results list will only include pending maintenance actions for the DB clusters identified by these ARNs. </li> <li>  <code>db-instance-id</code> - Accepts DB instance identifiers and DB instance ARNs. The results list will only include pending maintenance actions for the DB instances identified by these ARNs. </li> </ul><br/>ResourceIdentifier:  The ARN of a resource to return pending maintenance actions for.<br/>
        :return: Marker:   An optional pagination token provided by a previous <code>DescribePendingMaintenanceActions</code> request. If this parameter is specified, the response includes only records beyond the marker, up to a number of records specified by <code>MaxRecords</code>.<br/>PendingMaintenanceActions:  A list of the pending maintenance actions for the resource.<br/>        """
        pass

    def describe_valid_db_instance_modifications(self, DBInstanceIdentifier: str) -> DescribeValidDBInstanceModificationsResult:
        """You can call <a>DescribeValidDBInstanceModifications</a> to learn what modifications you can make to your DB instance. You can use this information when you call <a>ModifyDBInstance</a>.

        :param self:
        :param DBInstanceIdentifier:  The customer identifier or the ARN of your DB instance.<br/>
        :return: ValidDBInstanceModificationsMessage:  Information about valid modifications that you can make to your DB instance. Contains the result of a successful call to the <a>DescribeValidDBInstanceModifications</a> action. You can use this information when you call <a>ModifyDBInstance</a>. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Storage</i> Valid storage options for your DB instance.<br/>        """
        pass

    def failover_db_cluster(self, DBClusterIdentifier: str=None, TargetDBInstanceIdentifier: str=None) -> FailoverDBClusterResult:
        """Forces a failover for a DB cluster. A failover for a DB cluster promotes one of the Read Replicas (read-only instances) in the DB cluster to be the primary instance (the cluster writer). Amazon Neptune will automatically fail over to a Read Replica, if one exists, when the primary instance fails. You can force a failover when you want to simulate a failure of a primary instance for testing. Because each instance in a DB cluster has its own endpoint address, you will need to clean up and re-establish any existing connections that use those endpoint addresses when the failover is complete.

        :param self:
        :param TargetDBInstanceIdentifier:  The name of the instance to promote to the primary instance. You must specify the instance identifier for an Read Replica in the DB cluster. For example, <code>mydbcluster-replica1</code>.<br/>DBClusterIdentifier:  A DB cluster identifier to force a failover for. This parameter is not case-sensitive. Constraints: <ul> <li> Must match the identifier of an existing DBCluster. </li> </ul><br/>
        :return: DBCluster:  Contains the details of an Amazon Neptune DB cluster. This data type is used as a response element in the <a>DescribeDBClusters</a> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB cluster is configured to export to CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloneGroupId</i> Identifies the clone group to which the DB cluster is associated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i> Provides a list of the AWS Identity and Access Management (IAM) roles that are associated with the DB cluster. IAM roles that are associated with a DB cluster grant permission for the DB cluster to access other AWS services on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterArn</i> The Amazon Resource Name (ARN) for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbClusterResourceId</i> The AWS Region-unique, immutable identifier for the DB cluster. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB cluster is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is true, the AWS KMS key identifier for the encrypted DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security groups that the DB cluster belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterMembers</i> Provides the list of instances that make up the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSourceIdentifier</i> Not supported by Neptune.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i> Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterOptionGroupMemberships</i> Provides the list of option group memberships for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies whether the DB cluster has instances in multiple Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReaderEndpoint</i> The reader endpoint for the DB cluster. The reader endpoint for a DB cluster load-balances connections across the Read Replicas that are available in a DB cluster. As clients request new connections to the reader endpoint, Neptune distributes the connection requests among the Read Replicas in the DB cluster. This functionality can help balance your read workload across multiple Read Replicas in your DB cluster. If a failover occurs, and the Read Replica that you are connected to is promoted to be the primary instance, your connection is dropped. To continue sending your read workload to other Read Replicas in the cluster, you can then reconnect to the reader endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint for the primary instance of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestRestorableTime</i> Specifies the earliest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the progress of the operation as a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the current state of this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB cluster, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroup</i> Specifies the name of the DB cluster parameter group for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Contains a user-supplied DB cluster identifier. This identifier is the unique key that identifies a DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> Contains the name of the initial database of this DB cluster that was provided at create time, if one was specified when the DB cluster was created. This same name is returned for the life of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this cluster is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of EC2 Availability Zones that instances in the DB cluster can be created in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i>  <code>AllocatedStorage</code> always returns 1, because Neptune DB cluster storage size is not fixed, but instead automatically adjusts as needed.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceName: str, Filters: List[Filter]=None) -> TagListMessage:
        """Lists all tags on an Amazon Neptune resource.

        :param self:
        :param Filters:  This parameter is not currently supported.<br/>ResourceName:  The Amazon Neptune resource with tags to be listed. This value is an Amazon Resource Name (ARN). For information about creating an ARN, see <a href="https://docs.aws.amazon.com/neptune/latest/UserGuide/tagging.ARN.html#tagging.ARN.Constructing"> Constructing an Amazon Resource Name (ARN)</a>.<br/>
        :return: TagList:  List of tags returned by the ListTagsForResource operation.<br/>        """
        pass

    def modify_db_cluster(self, DBClusterIdentifier: str, NewDBClusterIdentifier: str=None, ApplyImmediately: Boolean=None, BackupRetentionPeriod: int=None, DBClusterParameterGroupName: str=None, VpcSecurityGroupIds: List[String]=None, Port: int=None, MasterUserPassword: str=None, OptionGroupName: str=None, PreferredBackupWindow: str=None, PreferredMaintenanceWindow: str=None, EnableIAMDatabaseAuthentication: BooleanOptional=None, CloudwatchLogsExportConfiguration: CloudwatchLogsExportConfiguration=None, EngineVersion: str=None) -> ModifyDBClusterResult:
        """Modify a setting for a DB cluster. You can change one or more database configuration parameters by specifying these parameters and the new values in the request.

        :param self:
        :param EngineVersion:  The version number of the database engine to which you want to upgrade. Changing this parameter results in an outage. The change is applied during the next maintenance window unless the ApplyImmediately parameter is set to true. For a list of valid engine versions, see <a>CreateDBInstance</a>, or call <a>DescribeDBEngineVersions</a>.<br/>CloudwatchLogsExportConfiguration:  The configuration setting for the log types to be enabled for export to CloudWatch Logs for a specific DB cluster.:<br/>EnableIAMDatabaseAuthentication:  True to enable mapping of AWS Identity and Access Management (IAM) accounts to database accounts, and otherwise false. Default: <code>false</code> <br/>PreferredMaintenanceWindow:  The weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC). Format: <code>ddd:hh24:mi-ddd:hh24:mi</code>  The default is a 30-minute window selected at random from an 8-hour block of time for each AWS Region, occurring on a random day of the week. Valid Days: Mon, Tue, Wed, Thu, Fri, Sat, Sun. Constraints: Minimum 30-minute window.<br/>PreferredBackupWindow:  The daily time range during which automated backups are created if automated backups are enabled, using the <code>BackupRetentionPeriod</code> parameter. The default is a 30-minute window selected at random from an 8-hour block of time for each AWS Region. Constraints: <ul> <li> Must be in the format <code>hh24:mi-hh24:mi</code>. </li> <li> Must be in Universal Coordinated Time (UTC). </li> <li> Must not conflict with the preferred maintenance window. </li> <li> Must be at least 30 minutes. </li> </ul><br/>OptionGroupName:  A value that indicates that the DB cluster should be associated with the specified option group. Changing this parameter doesn't result in an outage except in the following case, and the change is applied during the next maintenance window unless the <code>ApplyImmediately</code> parameter is set to <code>true</code> for this request. If the parameter change results in an option group that enables OEM, this change can cause a brief (sub-second) period during which new connections are rejected but existing connections are not interrupted. Permanent options can't be removed from an option group. The option group can't be removed from a DB cluster once it is associated with a DB cluster.<br/>MasterUserPassword:  The new password for the master database user. This password can contain any printable ASCII character except "/", triple-quotes, or "@". Constraints: Must contain from 8 to 41 characters.<br/>Port:  The port number on which the DB cluster accepts connections. Constraints: Value must be <code>1150-65535</code>  Default: The same port as the original DB cluster.<br/>VpcSecurityGroupIds:  A list of VPC security groups that the DB cluster will belong to.<br/>DBClusterParameterGroupName:  The name of the DB cluster parameter group to use for the DB cluster.<br/>BackupRetentionPeriod:  The number of days for which automated backups are retained. You must specify a minimum value of 1. Default: 1 Constraints: <ul> <li> Must be a value from 1 to 35 </li> </ul><br/>ApplyImmediately:  A value that specifies whether the modifications in this request and any pending modifications are asynchronously applied as soon as possible, regardless of the <code>PreferredMaintenanceWindow</code> setting for the DB cluster. If this parameter is set to <code>false</code>, changes to the DB cluster are applied during the next maintenance window. The <code>ApplyImmediately</code> parameter only affects the <code>NewDBClusterIdentifier</code> and <code>MasterUserPassword</code> values. If you set the <code>ApplyImmediately</code> parameter value to false, then changes to the <code>NewDBClusterIdentifier</code> and <code>MasterUserPassword</code> values are applied during the next maintenance window. All other changes are applied immediately, regardless of the value of the <code>ApplyImmediately</code> parameter. Default: <code>false</code> <br/>NewDBClusterIdentifier:  The new DB cluster identifier for the DB cluster when renaming a DB cluster. This value is stored as a lowercase string. Constraints: <ul> <li> Must contain from 1 to 63 letters, numbers, or hyphens </li> <li> The first character must be a letter </li> <li> Cannot end with a hyphen or contain two consecutive hyphens </li> </ul> Example: <code>my-cluster2</code> <br/>DBClusterIdentifier:  The DB cluster identifier for the cluster being modified. This parameter is not case-sensitive. Constraints: <ul> <li> Must match the identifier of an existing DBCluster. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisableLogTypes</i> The list of log types to disable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnableLogTypes</i> The list of log types to enable.<br/>
        :return: DBCluster:  Contains the details of an Amazon Neptune DB cluster. This data type is used as a response element in the <a>DescribeDBClusters</a> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB cluster is configured to export to CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloneGroupId</i> Identifies the clone group to which the DB cluster is associated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i> Provides a list of the AWS Identity and Access Management (IAM) roles that are associated with the DB cluster. IAM roles that are associated with a DB cluster grant permission for the DB cluster to access other AWS services on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterArn</i> The Amazon Resource Name (ARN) for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbClusterResourceId</i> The AWS Region-unique, immutable identifier for the DB cluster. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB cluster is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is true, the AWS KMS key identifier for the encrypted DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security groups that the DB cluster belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterMembers</i> Provides the list of instances that make up the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSourceIdentifier</i> Not supported by Neptune.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i> Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterOptionGroupMemberships</i> Provides the list of option group memberships for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies whether the DB cluster has instances in multiple Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReaderEndpoint</i> The reader endpoint for the DB cluster. The reader endpoint for a DB cluster load-balances connections across the Read Replicas that are available in a DB cluster. As clients request new connections to the reader endpoint, Neptune distributes the connection requests among the Read Replicas in the DB cluster. This functionality can help balance your read workload across multiple Read Replicas in your DB cluster. If a failover occurs, and the Read Replica that you are connected to is promoted to be the primary instance, your connection is dropped. To continue sending your read workload to other Read Replicas in the cluster, you can then reconnect to the reader endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint for the primary instance of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestRestorableTime</i> Specifies the earliest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the progress of the operation as a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the current state of this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB cluster, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroup</i> Specifies the name of the DB cluster parameter group for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Contains a user-supplied DB cluster identifier. This identifier is the unique key that identifies a DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> Contains the name of the initial database of this DB cluster that was provided at create time, if one was specified when the DB cluster was created. This same name is returned for the life of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this cluster is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of EC2 Availability Zones that instances in the DB cluster can be created in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i>  <code>AllocatedStorage</code> always returns 1, because Neptune DB cluster storage size is not fixed, but instead automatically adjusts as needed.<br/>        """
        pass

    def modify_db_cluster_parameter_group(self, Parameters: List[Parameter], DBClusterParameterGroupName: str) -> DBClusterParameterGroupNameMessage:
        """ Modifies the parameters of a DB cluster parameter group. To modify more than one parameter, submit a list of the following: <code>ParameterName</code>, <code>ParameterValue</code>, and <code>ApplyMethod</code>. A maximum of 20 parameters can be modified in a single request. <note> Changes to dynamic parameters are applied immediately. Changes to static parameters require a reboot without failover to the DB cluster associated with the parameter group before the change can take effect. </note> <important> After you create a DB cluster parameter group, you should wait at least 5 minutes before creating your first DB cluster that uses that DB cluster parameter group as the default parameter group. This allows Amazon Neptune to fully complete the create action before the parameter group is used as the default for a new DB cluster. This is especially important for parameters that are critical when creating the default database for a DB cluster, such as the character set for the default database defined by the <code>character_set_database</code> parameter. You can use the <i>Parameter Groups</i> option of the Amazon Neptune console or the <a>DescribeDBClusterParameters</a> command to verify that your DB cluster parameter group has been created or modified. </important>

        :param self:
        :param Parameters:  A list of parameters in the DB cluster parameter group to modify.<br/>DBClusterParameterGroupName:  The name of the DB cluster parameter group to modify.<br/>
        :return: DBClusterParameterGroupName:  The name of the DB cluster parameter group. Constraints: <ul> <li> Must be 1 to 255 letters or numbers. </li> <li> First character must be a letter </li> <li> Cannot end with a hyphen or contain two consecutive hyphens </li> </ul> <note> This value is stored as a lowercase string. </note><br/>        """
        pass

    def modify_db_cluster_snapshot_attribute(self, AttributeName: str, DBClusterSnapshotIdentifier: str, ValuesToAdd: List[String]=None, ValuesToRemove: List[String]=None) -> ModifyDBClusterSnapshotAttributeResult:
        """Adds an attribute and values to, or removes an attribute and values from, a manual DB cluster snapshot. To share a manual DB cluster snapshot with other AWS accounts, specify <code>restore</code> as the <code>AttributeName</code> and use the <code>ValuesToAdd</code> parameter to add a list of IDs of the AWS accounts that are authorized to restore the manual DB cluster snapshot. Use the value <code>all</code> to make the manual DB cluster snapshot public, which means that it can be copied or restored by all AWS accounts. Do not add the <code>all</code> value for any manual DB cluster snapshots that contain private information that you don't want available to all AWS accounts. If a manual DB cluster snapshot is encrypted, it can be shared, but only by specifying a list of authorized AWS account IDs for the <code>ValuesToAdd</code> parameter. You can't use <code>all</code> as a value for that parameter in this case. To view which AWS accounts have access to copy or restore a manual DB cluster snapshot, or whether a manual DB cluster snapshot public or private, use the <a>DescribeDBClusterSnapshotAttributes</a> API action.

        :param self:
        :param ValuesToRemove:  A list of DB cluster snapshot attributes to remove from the attribute specified by <code>AttributeName</code>. To remove authorization for other AWS accounts to copy or restore a manual DB cluster snapshot, set this list to include one or more AWS account identifiers, or <code>all</code> to remove authorization for any AWS account to copy or restore the DB cluster snapshot. If you specify <code>all</code>, an AWS account whose account ID is explicitly added to the <code>restore</code> attribute can still copy or restore a manual DB cluster snapshot.<br/>ValuesToAdd:  A list of DB cluster snapshot attributes to add to the attribute specified by <code>AttributeName</code>. To authorize other AWS accounts to copy or restore a manual DB cluster snapshot, set this list to include one or more AWS account IDs, or <code>all</code> to make the manual DB cluster snapshot restorable by any AWS account. Do not add the <code>all</code> value for any manual DB cluster snapshots that contain private information that you don't want available to all AWS accounts.<br/>AttributeName:  The name of the DB cluster snapshot attribute to modify. To manage authorization for other AWS accounts to copy or restore a manual DB cluster snapshot, set this value to <code>restore</code>.<br/>DBClusterSnapshotIdentifier:  The identifier for the DB cluster snapshot to modify the attributes for.<br/>
        :return: DBClusterSnapshotAttributesResult:  Contains the results of a successful call to the <a>DescribeDBClusterSnapshotAttributes</a> API action. Manual DB cluster snapshot attributes are used to authorize other AWS accounts to copy or restore a manual DB cluster snapshot. For more information, see the <a>ModifyDBClusterSnapshotAttribute</a> API action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotAttributes</i> The list of attributes and values for the manual DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotIdentifier</i> The identifier of the manual DB cluster snapshot that the attributes apply to.<br/>        """
        pass

    def modify_db_instance(self, DBInstanceIdentifier: str, AllocatedStorage: int=None, DBInstanceClass: str=None, DBSubnetGroupName: str=None, DBSecurityGroups: List[String]=None, VpcSecurityGroupIds: List[String]=None, ApplyImmediately: Boolean=None, MasterUserPassword: str=None, DBParameterGroupName: str=None, BackupRetentionPeriod: int=None, PreferredBackupWindow: str=None, PreferredMaintenanceWindow: str=None, MultiAZ: BooleanOptional=None, EngineVersion: str=None, AllowMajorVersionUpgrade: Boolean=None, AutoMinorVersionUpgrade: BooleanOptional=None, LicenseModel: str=None, Iops: int=None, OptionGroupName: str=None, NewDBInstanceIdentifier: str=None, StorageType: str=None, TdeCredentialArn: str=None, TdeCredentialPassword: str=None, CACertificateIdentifier: str=None, Domain: str=None, CopyTagsToSnapshot: BooleanOptional=None, MonitoringInterval: int=None, DBPortNumber: int=None, PubliclyAccessible: BooleanOptional=None, MonitoringRoleArn: str=None, DomainIAMRoleName: str=None, PromotionTier: int=None, EnableIAMDatabaseAuthentication: BooleanOptional=None, EnablePerformanceInsights: BooleanOptional=None, PerformanceInsightsKMSKeyId: str=None, CloudwatchLogsExportConfiguration: CloudwatchLogsExportConfiguration=None) -> ModifyDBInstanceResult:
        """Modifies settings for a DB instance. You can change one or more database configuration parameters by specifying these parameters and the new values in the request. To learn what modifications you can make to your DB instance, call <a>DescribeValidDBInstanceModifications</a> before you call <a>ModifyDBInstance</a>.

        :param self:
        :param CloudwatchLogsExportConfiguration:  The configuration setting for the log types to be enabled for export to CloudWatch Logs for a specific DB instance or DB cluster.:<br/>PerformanceInsightsKMSKeyId:  Not supported.<br/>EnablePerformanceInsights:  Not supported.<br/>EnableIAMDatabaseAuthentication:  True to enable mapping of AWS Identity and Access Management (IAM) accounts to database accounts, and otherwise false. You can enable IAM database authentication for the following database engines Not applicable. Mapping AWS IAM accounts to database accounts is managed by the DB cluster. For more information, see <a>ModifyDBCluster</a>. Default: <code>false</code> <br/>PromotionTier:  A value that specifies the order in which a Read Replica is promoted to the primary instance after a failure of the existing primary instance. Default: 1 Valid Values: 0 - 15<br/>DomainIAMRoleName:  Not supported<br/>MonitoringRoleArn:  The ARN for the IAM role that permits Neptune to send enhanced monitoring metrics to Amazon CloudWatch Logs. For example, <code>arn:aws:iam:123456789012:role/emaccess</code>. If <code>MonitoringInterval</code> is set to a value other than 0, then you must supply a <code>MonitoringRoleArn</code> value.<br/>PubliclyAccessible:  This flag should no longer be used.<br/>DBPortNumber:  The port number on which the database accepts connections. The value of the <code>DBPortNumber</code> parameter must not match any of the port values specified for options in the option group for the DB instance. Your database will restart when you change the <code>DBPortNumber</code> value regardless of the value of the <code>ApplyImmediately</code> parameter.  Default: <code>8182</code> <br/>MonitoringInterval:  The interval, in seconds, between points when Enhanced Monitoring metrics are collected for the DB instance. To disable collecting Enhanced Monitoring metrics, specify 0. The default is 0. If <code>MonitoringRoleArn</code> is specified, then you must also set <code>MonitoringInterval</code> to a value other than 0. Valid Values: <code>0, 1, 5, 10, 15, 30, 60</code> <br/>CopyTagsToSnapshot:  True to copy all tags from the DB instance to snapshots of the DB instance, and otherwise false. The default is false.<br/>Domain:  Not supported.<br/>CACertificateIdentifier:  Indicates the certificate that needs to be associated with the instance.<br/>TdeCredentialPassword:  The password for the given ARN from the key store in order to access the device.<br/>TdeCredentialArn:  The ARN from the key store with which to associate the instance for TDE encryption.<br/>StorageType:  Not supported.<br/>NewDBInstanceIdentifier:   The new DB instance identifier for the DB instance when renaming a DB instance. When you change the DB instance identifier, an instance reboot will occur immediately if you set <code>Apply Immediately</code> to true, or will occur during the next maintenance window if <code>Apply Immediately</code> to false. This value is stored as a lowercase string. Constraints: <ul> <li> Must contain from 1 to 63 letters, numbers, or hyphens. </li> <li> The first character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul> Example: <code>mydbinstance</code> <br/>OptionGroupName:   Indicates that the DB instance should be associated with the specified option group. Changing this parameter doesn't result in an outage except in the following case and the change is applied during the next maintenance window unless the <code>ApplyImmediately</code> parameter is set to <code>true</code> for this request. If the parameter change results in an option group that enables OEM, this change can cause a brief (sub-second) period during which new connections are rejected but existing connections are not interrupted. Permanent options, such as the TDE option for Oracle Advanced Security TDE, can't be removed from an option group, and that option group can't be removed from a DB instance once it is associated with a DB instance<br/>Iops:  The new Provisioned IOPS (I/O operations per second) value for the instance. Changing this setting doesn't result in an outage and the change is applied during the next maintenance window unless the <code>ApplyImmediately</code> parameter is set to <code>true</code> for this request. Default: Uses existing setting<br/>LicenseModel:  Not supported.<br/>AutoMinorVersionUpgrade:   Indicates that minor version upgrades are applied automatically to the DB instance during the maintenance window. Changing this parameter doesn't result in an outage except in the following case and the change is asynchronously applied as soon as possible. An outage will result if this parameter is set to <code>true</code> during the maintenance window, and a newer minor version is available, and Neptune has enabled auto patching for that engine version.<br/>AllowMajorVersionUpgrade:  Indicates that major version upgrades are allowed. Changing this parameter doesn't result in an outage and the change is asynchronously applied as soon as possible. Constraints: This parameter must be set to true when specifying a value for the EngineVersion parameter that is a different major version than the DB instance's current version.<br/>EngineVersion:   The version number of the database engine to upgrade to. Changing this parameter results in an outage and the change is applied during the next maintenance window unless the <code>ApplyImmediately</code> parameter is set to <code>true</code> for this request. For major version upgrades, if a nondefault DB parameter group is currently in use, a new DB parameter group in the DB parameter group family for the new engine version must be specified. The new DB parameter group can be the default for that DB parameter group family.<br/>MultiAZ:  Specifies if the DB instance is a Multi-AZ deployment. Changing this parameter doesn't result in an outage and the change is applied during the next maintenance window unless the <code>ApplyImmediately</code> parameter is set to <code>true</code> for this request.<br/>PreferredMaintenanceWindow:  The weekly time range (in UTC) during which system maintenance can occur, which might result in an outage. Changing this parameter doesn't result in an outage, except in the following situation, and the change is asynchronously applied as soon as possible. If there are pending actions that cause a reboot, and the maintenance window is changed to include the current time, then changing this parameter will cause a reboot of the DB instance. If moving this window to the current time, there must be at least 30 minutes between the current time and end of the window to ensure pending changes are applied. Default: Uses existing setting Format: ddd:hh24:mi-ddd:hh24:mi Valid Days: Mon | Tue | Wed | Thu | Fri | Sat | Sun Constraints: Must be at least 30 minutes<br/>PreferredBackupWindow:   The daily time range during which automated backups are created if automated backups are enabled. Not applicable. The daily time range for creating automated backups is managed by the DB cluster. For more information, see <a>ModifyDBCluster</a>. Constraints: <ul> <li> Must be in the format hh24:mi-hh24:mi </li> <li> Must be in Universal Time Coordinated (UTC) </li> <li> Must not conflict with the preferred maintenance window </li> <li> Must be at least 30 minutes </li> </ul><br/>BackupRetentionPeriod:  Not applicable. The retention period for automated backups is managed by the DB cluster. For more information, see <a>ModifyDBCluster</a>. Default: Uses existing setting<br/>DBParameterGroupName:  The name of the DB parameter group to apply to the DB instance. Changing this setting doesn't result in an outage. The parameter group name itself is changed immediately, but the actual parameter changes are not applied until you reboot the instance without failover. The db instance will NOT be rebooted automatically and the parameter changes will NOT be applied during the next maintenance window. Default: Uses existing setting Constraints: The DB parameter group must be in the same DB parameter group family as this DB instance.<br/>MasterUserPassword:  Not applicable.<br/>ApplyImmediately:  Specifies whether the modifications in this request and any pending modifications are asynchronously applied as soon as possible, regardless of the <code>PreferredMaintenanceWindow</code> setting for the DB instance.  If this parameter is set to <code>false</code>, changes to the DB instance are applied during the next maintenance window. Some parameter changes can cause an outage and are applied on the next call to <a>RebootDBInstance</a>, or the next failure reboot. Default: <code>false</code> <br/>VpcSecurityGroupIds:  A list of EC2 VPC security groups to authorize on this DB instance. This change is asynchronously applied as soon as possible. Not applicable. The associated list of EC2 VPC security groups is managed by the DB cluster. For more information, see <a>ModifyDBCluster</a>. Constraints: <ul> <li> If supplied, must match existing VpcSecurityGroupIds. </li> </ul><br/>DBSecurityGroups:  A list of DB security groups to authorize on this DB instance. Changing this setting doesn't result in an outage and the change is asynchronously applied as soon as possible. Constraints: <ul> <li> If supplied, must match existing DBSecurityGroups. </li> </ul><br/>DBSubnetGroupName:  The new DB subnet group for the DB instance. You can use this parameter to move your DB instance to a different VPC. Changing the subnet group causes an outage during the change. The change is applied during the next maintenance window, unless you specify <code>true</code> for the <code>ApplyImmediately</code> parameter. Constraints: If supplied, must match the name of an existing DBSubnetGroup. Example: <code>mySubnetGroup</code> <br/>DBInstanceClass:  The new compute and memory capacity of the DB instance, for example, <code>db.m4.large</code>. Not all DB instance classes are available in all AWS Regions. If you modify the DB instance class, an outage occurs during the change. The change is applied during the next maintenance window, unless <code>ApplyImmediately</code> is specified as <code>true</code> for this request. Default: Uses existing setting<br/>AllocatedStorage:  The new amount of storage (in gibibytes) to allocate for the DB instance. Not applicable. Storage is managed by the DB Cluster.<br/>DBInstanceIdentifier:  The DB instance identifier. This value is stored as a lowercase string. Constraints: <ul> <li> Must match the identifier of an existing DBInstance. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisableLogTypes</i> The list of log types to disable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnableLogTypes</i> The list of log types to enable.<br/>
        :return: DBInstance:  Contains the details of an Amazon Neptune DB instance. This data type is used as a response element in the <a>DescribeDBInstances</a> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB instance is configured to export to CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsKMSKeyId</i> The AWS KMS key identifier for encryption of Performance Insights data. The KMS key ID is the Amazon Resource Name (ARN), KMS key identifier, or the KMS key alias for the KMS encryption key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsEnabled</i> True if Performance Insights is enabled for the DB instance, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if AWS Identity and Access Management (IAM) authentication is enabled, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timezone</i> Not supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceArn</i> The Amazon Resource Name (ARN) for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PromotionTier</i> A value that specifies the order in which a Read Replica is promoted to the primary instance after a failure of the existing primary instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringRoleArn</i> The ARN for the IAM role that permits Neptune to send Enhanced Monitoring metrics to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedMonitoringResourceArn</i> The Amazon Resource Name (ARN) of the Amazon CloudWatch Logs log stream that receives the Enhanced Monitoring metrics data for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringInterval</i> The interval, in seconds, between points when Enhanced Monitoring metrics are collected for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyTagsToSnapshot</i> Specifies whether tags are copied from the DB instance to snapshots of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainMemberships</i> Not supported<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> The identifier of the CA certificate for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbiResourceId</i> The AWS Region-unique, immutable identifier for the DB instance. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB instance is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i>  Not supported: The encryption for DB instances is managed by the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Not supported: The encryption for DB instances is managed by the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> If the DB instance is a member of a DB cluster, contains the name of the DB cluster that the DB instance is a member of.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbInstancePort</i> Specifies the port that the DB instance listens on. If the DB instance is part of a DB cluster, this can be a different port than the DB cluster port.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TdeCredentialArn</i> The ARN from the key store with which the instance is associated for TDE encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type associated with DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusInfos</i> The status of a Read Replica. If the instance is not a Read Replica, this is blank.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> This flag should no longer be used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondaryAvailabilityZone</i> If present, specifies the name of the secondary Availability Zone for a DB instance with multi-AZ support.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this instance is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionGroupMemberships</i> Provides the list of option group memberships for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the Provisioned IOPS (I/O operations per second) value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> License model information for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBClusterIdentifiers</i> Contains one or more identifiers of DB clusters that are Read Replicas of this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBInstanceIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaSourceDBInstanceIdentifier</i> Contains the identifier of the source DB instance if this DB instance is a Read Replica.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> Indicates that minor version patches are applied automatically.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies if the DB instance is a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DBSubnetGroup</b>: Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> Specifies the name of the Availability Zone the DB instance is located in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroups</i> Provides the list of DB parameter groups applied to this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security group elements that the DB instance belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSecurityGroups</i>  Provides List of DB security group elements containing only <code>DBSecurityGroup.Name</code> and <code>DBSecurityGroup.Status</code> subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i>  Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCreateTime</i> Provides the date and time the DB instance was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> Specifies the allocated storage size specified in gibibytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: Specifies the connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The database name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceStatus</i> Specifies the current state of this database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i> Contains the name of the compute and memory capacity class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i> Contains a user-supplied database identifier. This identifier is the unique key that identifies a DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupArn</i> The Amazon Resource Name (ARN) for the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i>  Contains a list of <a>Subnet</a> elements.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> Provides the status of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VpcId of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupDescription</i> Provides the description of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The name of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The new DB subnet group for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> Specifies the identifier of the CA certificate for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type to be associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i>  Contains the new <code>DBInstanceIdentifier</code> for the DB instance that will be applied or is currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the new Provisioned IOPS value for the DB instance that will be applied or is currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> The license model for the DB instance. Valid values: <code>license-included</code> | <code>bring-your-own-license</code> | <code>general-public-license</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Indicates that the Single-AZ DB instance is to change to a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the pending number of days for which automated backups are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the pending port for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> Contains the pending or currently-in-progress change of the master credentials for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i>  Contains the new <code>AllocatedStorage</code> size for the DB instance that will be applied or is currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i>  Contains the new <code>DBInstanceClass</code> for the DB instance that will be applied or is currently being applied.<br/>Specifies the CloudWatch logs to be exported.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingCloudwatchLogsExports</b>: Specifies the CloudWatch logs to be exported.        """
        pass

    def modify_db_parameter_group(self, Parameters: List[Parameter], DBParameterGroupName: str) -> DBParameterGroupNameMessage:
        """Modifies the parameters of a DB parameter group. To modify more than one parameter, submit a list of the following: <code>ParameterName</code>, <code>ParameterValue</code>, and <code>ApplyMethod</code>. A maximum of 20 parameters can be modified in a single request. <note> Changes to dynamic parameters are applied immediately. Changes to static parameters require a reboot without failover to the DB instance associated with the parameter group before the change can take effect. </note> <important> After you modify a DB parameter group, you should wait at least 5 minutes before creating your first DB instance that uses that DB parameter group as the default parameter group. This allows Amazon Neptune to fully complete the modify action before the parameter group is used as the default for a new DB instance. This is especially important for parameters that are critical when creating the default database for a DB instance, such as the character set for the default database defined by the <code>character_set_database</code> parameter. You can use the <i>Parameter Groups</i> option of the Amazon Neptune console or the <i>DescribeDBParameters</i> command to verify that your DB parameter group has been created or modified. </important>

        :param self:
        :param Parameters:  An array of parameter names, values, and the apply method for the parameter update. At least one parameter name, value, and apply method must be supplied; subsequent arguments are optional. A maximum of 20 parameters can be modified in a single request. Valid Values (for the application method): <code>immediate | pending-reboot</code>  <note> You can use the immediate value with dynamic parameters only. You can use the pending-reboot value for both dynamic and static parameters, and changes are applied when you reboot the DB instance without failover. </note><br/>DBParameterGroupName:  The name of the DB parameter group. Constraints: <ul> <li> If supplied, must match the name of an existing DBParameterGroup. </li> </ul><br/>
        :return: DBParameterGroupName:  Provides the name of the DB parameter group.<br/>        """
        pass

    def modify_db_subnet_group(self, SubnetIds: List[String], DBSubnetGroupName: str, DBSubnetGroupDescription: str=None) -> ModifyDBSubnetGroupResult:
        """Modifies an existing DB subnet group. DB subnet groups must contain at least one subnet in at least two AZs in the AWS Region.

        :param self:
        :param SubnetIds:  The EC2 subnet IDs for the DB subnet group.<br/>DBSubnetGroupDescription:  The description for the DB subnet group.<br/>DBSubnetGroupName:  The name for the DB subnet group. This value is stored as a lowercase string. You can't modify the default subnet group. Constraints: Must match the name of an existing DBSubnetGroup. Must not be default. Example: <code>mySubnetgroup</code> <br/>
        :return: DBSubnetGroup:  Contains the details of an Amazon Neptune DB subnet group. This data type is used as a response element in the <a>DescribeDBSubnetGroups</a> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupArn</i> The Amazon Resource Name (ARN) for the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i>  Contains a list of <a>Subnet</a> elements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> Provides the status of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VpcId of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupDescription</i> Provides the description of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The name of the DB subnet group.<br/>        """
        pass

    def modify_event_subscription(self, SubscriptionName: str, SnsTopicArn: str=None, SourceType: str=None, EventCategories: List[String]=None, Enabled: BooleanOptional=None) -> ModifyEventSubscriptionResult:
        """Modifies an existing event notification subscription. Note that you can't modify the source identifiers using this call; to change source identifiers for a subscription, use the <a>AddSourceIdentifierToSubscription</a> and <a>RemoveSourceIdentifierFromSubscription</a> calls. You can see a list of the event categories for a given SourceType by using the <b>DescribeEventCategories</b> action.

        :param self:
        :param Enabled:   A Boolean value; set to <b>true</b> to activate the subscription.<br/>EventCategories:   A list of event categories for a SourceType that you want to subscribe to. You can see a list of the categories for a given SourceType by using the <b>DescribeEventCategories</b> action.<br/>SourceType:  The type of source that is generating the events. For example, if you want to be notified of events generated by a DB instance, you would set this parameter to db-instance. if this value is not specified, all events are returned. Valid values: db-instance | db-parameter-group | db-security-group | db-snapshot<br/>SnsTopicArn:  The Amazon Resource Name (ARN) of the SNS topic created for event notification. The ARN is created by Amazon SNS when you create a topic and subscribe to it.<br/>SubscriptionName:  The name of the event notification subscription.<br/>
        :return: EventSubscription:  Contains the results of a successful invocation of the <a>DescribeEventSubscriptions</a> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EventSubscriptionArn</i> The Amazon Resource Name (ARN) for the event subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> A Boolean value indicating if the subscription is enabled. True indicates the subscription is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EventCategoriesList</i> A list of event categories for the event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceIdsList</i> A list of source IDs for the event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceType</i> The source type for the event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubscriptionCreationTime</i> The time the event notification subscription was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the event notification subscription. Constraints: Can be one of the following: creating | modifying | deleting | active | no-permission | topic-not-exist The status "no-permission" indicates that Neptune no longer has permission to post to the SNS topic. The status "topic-not-exist" indicates that the topic was deleted after the subscription was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsTopicArn</i> The topic ARN of the event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustSubscriptionId</i> The event notification subscription Id.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomerAwsId</i> The AWS customer account associated with the event notification subscription.<br/>        """
        pass

    def promote_read_replica_db_cluster(self, DBClusterIdentifier: str) -> PromoteReadReplicaDBClusterResult:
        """Not supported.

        :param self:
        :param DBClusterIdentifier:  Not supported.<br/>
        :return: DBCluster:  Contains the details of an Amazon Neptune DB cluster. This data type is used as a response element in the <a>DescribeDBClusters</a> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB cluster is configured to export to CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloneGroupId</i> Identifies the clone group to which the DB cluster is associated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i> Provides a list of the AWS Identity and Access Management (IAM) roles that are associated with the DB cluster. IAM roles that are associated with a DB cluster grant permission for the DB cluster to access other AWS services on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterArn</i> The Amazon Resource Name (ARN) for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbClusterResourceId</i> The AWS Region-unique, immutable identifier for the DB cluster. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB cluster is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is true, the AWS KMS key identifier for the encrypted DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security groups that the DB cluster belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterMembers</i> Provides the list of instances that make up the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSourceIdentifier</i> Not supported by Neptune.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i> Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterOptionGroupMemberships</i> Provides the list of option group memberships for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies whether the DB cluster has instances in multiple Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReaderEndpoint</i> The reader endpoint for the DB cluster. The reader endpoint for a DB cluster load-balances connections across the Read Replicas that are available in a DB cluster. As clients request new connections to the reader endpoint, Neptune distributes the connection requests among the Read Replicas in the DB cluster. This functionality can help balance your read workload across multiple Read Replicas in your DB cluster. If a failover occurs, and the Read Replica that you are connected to is promoted to be the primary instance, your connection is dropped. To continue sending your read workload to other Read Replicas in the cluster, you can then reconnect to the reader endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint for the primary instance of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestRestorableTime</i> Specifies the earliest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the progress of the operation as a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the current state of this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB cluster, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroup</i> Specifies the name of the DB cluster parameter group for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Contains a user-supplied DB cluster identifier. This identifier is the unique key that identifies a DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> Contains the name of the initial database of this DB cluster that was provided at create time, if one was specified when the DB cluster was created. This same name is returned for the life of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this cluster is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of EC2 Availability Zones that instances in the DB cluster can be created in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i>  <code>AllocatedStorage</code> always returns 1, because Neptune DB cluster storage size is not fixed, but instead automatically adjusts as needed.<br/>        """
        pass

    def reboot_db_instance(self, DBInstanceIdentifier: str, ForceFailover: BooleanOptional=None) -> RebootDBInstanceResult:
        """You might need to reboot your DB instance, usually for maintenance reasons. For example, if you make certain modifications, or if you change the DB parameter group associated with the DB instance, you must reboot the instance for the changes to take effect. Rebooting a DB instance restarts the database engine service. Rebooting a DB instance results in a momentary outage, during which the DB instance status is set to rebooting.

        :param self:
        :param ForceFailover:   When <code>true</code>, the reboot is conducted through a MultiAZ failover. Constraint: You can't specify <code>true</code> if the instance is not configured for MultiAZ.<br/>DBInstanceIdentifier:  The DB instance identifier. This parameter is stored as a lowercase string. Constraints: <ul> <li> Must match the identifier of an existing DBInstance. </li> </ul><br/>
        :return: DBInstance:  Contains the details of an Amazon Neptune DB instance. This data type is used as a response element in the <a>DescribeDBInstances</a> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB instance is configured to export to CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsKMSKeyId</i> The AWS KMS key identifier for encryption of Performance Insights data. The KMS key ID is the Amazon Resource Name (ARN), KMS key identifier, or the KMS key alias for the KMS encryption key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsEnabled</i> True if Performance Insights is enabled for the DB instance, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if AWS Identity and Access Management (IAM) authentication is enabled, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timezone</i> Not supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceArn</i> The Amazon Resource Name (ARN) for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PromotionTier</i> A value that specifies the order in which a Read Replica is promoted to the primary instance after a failure of the existing primary instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringRoleArn</i> The ARN for the IAM role that permits Neptune to send Enhanced Monitoring metrics to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedMonitoringResourceArn</i> The Amazon Resource Name (ARN) of the Amazon CloudWatch Logs log stream that receives the Enhanced Monitoring metrics data for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringInterval</i> The interval, in seconds, between points when Enhanced Monitoring metrics are collected for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyTagsToSnapshot</i> Specifies whether tags are copied from the DB instance to snapshots of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainMemberships</i> Not supported<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> The identifier of the CA certificate for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbiResourceId</i> The AWS Region-unique, immutable identifier for the DB instance. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB instance is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i>  Not supported: The encryption for DB instances is managed by the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Not supported: The encryption for DB instances is managed by the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> If the DB instance is a member of a DB cluster, contains the name of the DB cluster that the DB instance is a member of.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbInstancePort</i> Specifies the port that the DB instance listens on. If the DB instance is part of a DB cluster, this can be a different port than the DB cluster port.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TdeCredentialArn</i> The ARN from the key store with which the instance is associated for TDE encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type associated with DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusInfos</i> The status of a Read Replica. If the instance is not a Read Replica, this is blank.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> This flag should no longer be used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondaryAvailabilityZone</i> If present, specifies the name of the secondary Availability Zone for a DB instance with multi-AZ support.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this instance is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionGroupMemberships</i> Provides the list of option group memberships for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the Provisioned IOPS (I/O operations per second) value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> License model information for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBClusterIdentifiers</i> Contains one or more identifiers of DB clusters that are Read Replicas of this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBInstanceIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaSourceDBInstanceIdentifier</i> Contains the identifier of the source DB instance if this DB instance is a Read Replica.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> Indicates that minor version patches are applied automatically.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies if the DB instance is a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DBSubnetGroup</b>: Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> Specifies the name of the Availability Zone the DB instance is located in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroups</i> Provides the list of DB parameter groups applied to this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security group elements that the DB instance belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSecurityGroups</i>  Provides List of DB security group elements containing only <code>DBSecurityGroup.Name</code> and <code>DBSecurityGroup.Status</code> subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i>  Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCreateTime</i> Provides the date and time the DB instance was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> Specifies the allocated storage size specified in gibibytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: Specifies the connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The database name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceStatus</i> Specifies the current state of this database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i> Contains the name of the compute and memory capacity class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i> Contains a user-supplied database identifier. This identifier is the unique key that identifies a DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupArn</i> The Amazon Resource Name (ARN) for the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i>  Contains a list of <a>Subnet</a> elements.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> Provides the status of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VpcId of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupDescription</i> Provides the description of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The name of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The new DB subnet group for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> Specifies the identifier of the CA certificate for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type to be associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i>  Contains the new <code>DBInstanceIdentifier</code> for the DB instance that will be applied or is currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the new Provisioned IOPS value for the DB instance that will be applied or is currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> The license model for the DB instance. Valid values: <code>license-included</code> | <code>bring-your-own-license</code> | <code>general-public-license</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Indicates that the Single-AZ DB instance is to change to a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the pending number of days for which automated backups are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the pending port for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> Contains the pending or currently-in-progress change of the master credentials for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i>  Contains the new <code>AllocatedStorage</code> size for the DB instance that will be applied or is currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i>  Contains the new <code>DBInstanceClass</code> for the DB instance that will be applied or is currently being applied.<br/>Specifies the CloudWatch logs to be exported.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingCloudwatchLogsExports</b>: Specifies the CloudWatch logs to be exported.        """
        pass

    def remove_role_from_db_cluster(self, RoleArn: str, DBClusterIdentifier: str):
        """Disassociates an Identity and Access Management (IAM) role from a DB cluster.

        :param self:
        :param RoleArn:  The Amazon Resource Name (ARN) of the IAM role to disassociate from the DB cluster, for example <code>arn:aws:iam::123456789012:role/NeptuneAccessRole</code>.<br/>DBClusterIdentifier:  The name of the DB cluster to disassociate the IAM role from.<br/>
        :return:         """
        pass

    def remove_source_identifier_from_subscription(self, SourceIdentifier: str, SubscriptionName: str) -> RemoveSourceIdentifierFromSubscriptionResult:
        """Removes a source identifier from an existing event notification subscription.

        :param self:
        :param SourceIdentifier:   The source identifier to be removed from the subscription, such as the <b>DB instance identifier</b> for a DB instance or the name of a security group.<br/>SubscriptionName:  The name of the event notification subscription you want to remove a source identifier from.<br/>
        :return: EventSubscription:  Contains the results of a successful invocation of the <a>DescribeEventSubscriptions</a> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EventSubscriptionArn</i> The Amazon Resource Name (ARN) for the event subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> A Boolean value indicating if the subscription is enabled. True indicates the subscription is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EventCategoriesList</i> A list of event categories for the event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceIdsList</i> A list of source IDs for the event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceType</i> The source type for the event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubscriptionCreationTime</i> The time the event notification subscription was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the event notification subscription. Constraints: Can be one of the following: creating | modifying | deleting | active | no-permission | topic-not-exist The status "no-permission" indicates that Neptune no longer has permission to post to the SNS topic. The status "topic-not-exist" indicates that the topic was deleted after the subscription was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsTopicArn</i> The topic ARN of the event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustSubscriptionId</i> The event notification subscription Id.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomerAwsId</i> The AWS customer account associated with the event notification subscription.<br/>        """
        pass

    def remove_tags_from_resource(self, TagKeys: List[String], ResourceName: str):
        """Removes metadata tags from an Amazon Neptune resource.

        :param self:
        :param TagKeys:  The tag key (name) of the tag to be removed.<br/>ResourceName:  The Amazon Neptune resource that the tags are removed from. This value is an Amazon Resource Name (ARN). For information about creating an ARN, see <a href="https://docs.aws.amazon.com/neptune/latest/UserGuide/tagging.ARN.html#tagging.ARN.Constructing"> Constructing an Amazon Resource Name (ARN)</a>.<br/>
        :return:         """
        pass

    def reset_db_cluster_parameter_group(self, DBClusterParameterGroupName: str, ResetAllParameters: Boolean=None, Parameters: List[Parameter]=None) -> DBClusterParameterGroupNameMessage:
        """ Modifies the parameters of a DB cluster parameter group to the default value. To reset specific parameters submit a list of the following: <code>ParameterName</code> and <code>ApplyMethod</code>. To reset the entire DB cluster parameter group, specify the <code>DBClusterParameterGroupName</code> and <code>ResetAllParameters</code> parameters.  When resetting the entire group, dynamic parameters are updated immediately and static parameters are set to <code>pending-reboot</code> to take effect on the next DB instance restart or <a>RebootDBInstance</a> request. You must call <a>RebootDBInstance</a> for every DB instance in your DB cluster that you want the updated static parameter to apply to.

        :param self:
        :param Parameters:  A list of parameter names in the DB cluster parameter group to reset to the default values. You can't use this parameter if the <code>ResetAllParameters</code> parameter is set to <code>true</code>.<br/>ResetAllParameters:  A value that is set to <code>true</code> to reset all parameters in the DB cluster parameter group to their default values, and <code>false</code> otherwise. You can't use this parameter if there is a list of parameter names specified for the <code>Parameters</code> parameter.<br/>DBClusterParameterGroupName:  The name of the DB cluster parameter group to reset.<br/>
        :return: DBClusterParameterGroupName:  The name of the DB cluster parameter group. Constraints: <ul> <li> Must be 1 to 255 letters or numbers. </li> <li> First character must be a letter </li> <li> Cannot end with a hyphen or contain two consecutive hyphens </li> </ul> <note> This value is stored as a lowercase string. </note><br/>        """
        pass

    def reset_db_parameter_group(self, DBParameterGroupName: str, ResetAllParameters: Boolean=None, Parameters: List[Parameter]=None) -> DBParameterGroupNameMessage:
        """Modifies the parameters of a DB parameter group to the engine/system default value. To reset specific parameters, provide a list of the following: <code>ParameterName</code> and <code>ApplyMethod</code>. To reset the entire DB parameter group, specify the <code>DBParameterGroup</code> name and <code>ResetAllParameters</code> parameters. When resetting the entire group, dynamic parameters are updated immediately and static parameters are set to <code>pending-reboot</code> to take effect on the next DB instance restart or <code>RebootDBInstance</code> request.

        :param self:
        :param Parameters:  To reset the entire DB parameter group, specify the <code>DBParameterGroup</code> name and <code>ResetAllParameters</code> parameters. To reset specific parameters, provide a list of the following: <code>ParameterName</code> and <code>ApplyMethod</code>. A maximum of 20 parameters can be modified in a single request. Valid Values (for Apply method): <code>pending-reboot</code> <br/>ResetAllParameters:  Specifies whether (<code>true</code>) or not (<code>false</code>) to reset all parameters in the DB parameter group to default values. Default: <code>true</code> <br/>DBParameterGroupName:  The name of the DB parameter group. Constraints: <ul> <li> Must match the name of an existing DBParameterGroup. </li> </ul><br/>
        :return: DBParameterGroupName:  Provides the name of the DB parameter group.<br/>        """
        pass

    def restore_db_cluster_from_snapshot(self, Engine: str, SnapshotIdentifier: str, DBClusterIdentifier: str, AvailabilityZones: List[String]=None, EngineVersion: str=None, Port: int=None, DBSubnetGroupName: str=None, DatabaseName: str=None, OptionGroupName: str=None, VpcSecurityGroupIds: List[String]=None, Tags: List[Tag]=None, KmsKeyId: str=None, EnableIAMDatabaseAuthentication: BooleanOptional=None, EnableCloudwatchLogsExports: List[String]=None, DBClusterParameterGroupName: str=None) -> RestoreDBClusterFromSnapshotResult:
        """Creates a new DB cluster from a DB snapshot or DB cluster snapshot. If a DB snapshot is specified, the target DB cluster is created from the source DB snapshot with a default configuration and default security group. If a DB cluster snapshot is specified, the target DB cluster is created from the source DB cluster restore point with the same configuration as the original source DB cluster, except that the new DB cluster is created with the default security group.

        :param self:
        :param DBClusterParameterGroupName:  The name of the DB cluster parameter group to associate with the new DB cluster. Constraints: <ul> <li> If supplied, must match the name of an existing DBClusterParameterGroup. </li> </ul><br/>EnableCloudwatchLogsExports:  The list of logs that the restored DB cluster is to export to Amazon CloudWatch Logs.<br/>EnableIAMDatabaseAuthentication:  True to enable mapping of AWS Identity and Access Management (IAM) accounts to database accounts, and otherwise false. Default: <code>false</code> <br/>KmsKeyId:  The AWS KMS key identifier to use when restoring an encrypted DB cluster from a DB snapshot or DB cluster snapshot. The KMS key identifier is the Amazon Resource Name (ARN) for the KMS encryption key. If you are restoring a DB cluster with the same AWS account that owns the KMS encryption key used to encrypt the new DB cluster, then you can use the KMS key alias instead of the ARN for the KMS encryption key. If you do not specify a value for the <code>KmsKeyId</code> parameter, then the following will occur: <ul> <li> If the DB snapshot or DB cluster snapshot in <code>SnapshotIdentifier</code> is encrypted, then the restored DB cluster is encrypted using the KMS key that was used to encrypt the DB snapshot or DB cluster snapshot. </li> <li> If the DB snapshot or DB cluster snapshot in <code>SnapshotIdentifier</code> is not encrypted, then the restored DB cluster is not encrypted. </li> </ul><br/>Tags:  The tags to be assigned to the restored DB cluster.<br/>VpcSecurityGroupIds:  A list of VPC security groups that the new DB cluster will belong to.<br/>OptionGroupName:  The name of the option group to use for the restored DB cluster.<br/>DatabaseName:  Not supported.<br/>DBSubnetGroupName:  The name of the DB subnet group to use for the new DB cluster. Constraints: If supplied, must match the name of an existing DBSubnetGroup. Example: <code>mySubnetgroup</code> <br/>Port:  The port number on which the new DB cluster accepts connections. Constraints: Value must be <code>1150-65535</code>  Default: The same port as the original DB cluster.<br/>EngineVersion:  The version of the database engine to use for the new DB cluster.<br/>Engine:  The database engine to use for the new DB cluster. Default: The same as source Constraint: Must be compatible with the engine of the source<br/>SnapshotIdentifier:  The identifier for the DB snapshot or DB cluster snapshot to restore from. You can use either the name or the Amazon Resource Name (ARN) to specify a DB cluster snapshot. However, you can use only the ARN to specify a DB snapshot. Constraints: <ul> <li> Must match the identifier of an existing Snapshot. </li> </ul><br/>DBClusterIdentifier:  The name of the DB cluster to create from the DB snapshot or DB cluster snapshot. This parameter isn't case-sensitive. Constraints: <ul> <li> Must contain from 1 to 63 letters, numbers, or hyphens </li> <li> First character must be a letter </li> <li> Cannot end with a hyphen or contain two consecutive hyphens </li> </ul> Example: <code>my-snapshot-id</code> <br/>AvailabilityZones:  Provides the list of EC2 Availability Zones that instances in the restored DB cluster can be created in.<br/>
        :return: DBCluster:  Contains the details of an Amazon Neptune DB cluster. This data type is used as a response element in the <a>DescribeDBClusters</a> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB cluster is configured to export to CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloneGroupId</i> Identifies the clone group to which the DB cluster is associated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i> Provides a list of the AWS Identity and Access Management (IAM) roles that are associated with the DB cluster. IAM roles that are associated with a DB cluster grant permission for the DB cluster to access other AWS services on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterArn</i> The Amazon Resource Name (ARN) for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbClusterResourceId</i> The AWS Region-unique, immutable identifier for the DB cluster. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB cluster is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is true, the AWS KMS key identifier for the encrypted DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security groups that the DB cluster belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterMembers</i> Provides the list of instances that make up the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSourceIdentifier</i> Not supported by Neptune.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i> Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterOptionGroupMemberships</i> Provides the list of option group memberships for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies whether the DB cluster has instances in multiple Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReaderEndpoint</i> The reader endpoint for the DB cluster. The reader endpoint for a DB cluster load-balances connections across the Read Replicas that are available in a DB cluster. As clients request new connections to the reader endpoint, Neptune distributes the connection requests among the Read Replicas in the DB cluster. This functionality can help balance your read workload across multiple Read Replicas in your DB cluster. If a failover occurs, and the Read Replica that you are connected to is promoted to be the primary instance, your connection is dropped. To continue sending your read workload to other Read Replicas in the cluster, you can then reconnect to the reader endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint for the primary instance of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestRestorableTime</i> Specifies the earliest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the progress of the operation as a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the current state of this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB cluster, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroup</i> Specifies the name of the DB cluster parameter group for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Contains a user-supplied DB cluster identifier. This identifier is the unique key that identifies a DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> Contains the name of the initial database of this DB cluster that was provided at create time, if one was specified when the DB cluster was created. This same name is returned for the life of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this cluster is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of EC2 Availability Zones that instances in the DB cluster can be created in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i>  <code>AllocatedStorage</code> always returns 1, because Neptune DB cluster storage size is not fixed, but instead automatically adjusts as needed.<br/>        """
        pass

    def restore_db_cluster_to_point_in_time(self, SourceDBClusterIdentifier: str, DBClusterIdentifier: str, RestoreType: str=None, RestoreToTime: TStamp=None, UseLatestRestorableTime: Boolean=None, Port: int=None, DBSubnetGroupName: str=None, OptionGroupName: str=None, VpcSecurityGroupIds: List[String]=None, Tags: List[Tag]=None, KmsKeyId: str=None, EnableIAMDatabaseAuthentication: BooleanOptional=None, EnableCloudwatchLogsExports: List[String]=None, DBClusterParameterGroupName: str=None) -> RestoreDBClusterToPointInTimeResult:
        """Restores a DB cluster to an arbitrary point in time. Users can restore to any point in time before <code>LatestRestorableTime</code> for up to <code>BackupRetentionPeriod</code> days. The target DB cluster is created from the source DB cluster with the same configuration as the original DB cluster, except that the new DB cluster is created with the default DB security group. <note> This action only restores the DB cluster, not the DB instances for that DB cluster. You must invoke the <a>CreateDBInstance</a> action to create DB instances for the restored DB cluster, specifying the identifier of the restored DB cluster in <code>DBClusterIdentifier</code>. You can create DB instances only after the <code>RestoreDBClusterToPointInTime</code> action has completed and the DB cluster is available. </note>

        :param self:
        :param DBClusterParameterGroupName:  The name of the DB cluster parameter group to associate with the new DB cluster. Constraints: <ul> <li> If supplied, must match the name of an existing DBClusterParameterGroup. </li> </ul><br/>EnableCloudwatchLogsExports:  The list of logs that the restored DB cluster is to export to CloudWatch Logs.<br/>EnableIAMDatabaseAuthentication:  True to enable mapping of AWS Identity and Access Management (IAM) accounts to database accounts, and otherwise false. Default: <code>false</code> <br/>KmsKeyId:  The AWS KMS key identifier to use when restoring an encrypted DB cluster from an encrypted DB cluster. The KMS key identifier is the Amazon Resource Name (ARN) for the KMS encryption key. If you are restoring a DB cluster with the same AWS account that owns the KMS encryption key used to encrypt the new DB cluster, then you can use the KMS key alias instead of the ARN for the KMS encryption key. You can restore to a new DB cluster and encrypt the new DB cluster with a KMS key that is different than the KMS key used to encrypt the source DB cluster. The new DB cluster is encrypted with the KMS key identified by the <code>KmsKeyId</code> parameter. If you do not specify a value for the <code>KmsKeyId</code> parameter, then the following will occur: <ul> <li> If the DB cluster is encrypted, then the restored DB cluster is encrypted using the KMS key that was used to encrypt the source DB cluster. </li> <li> If the DB cluster is not encrypted, then the restored DB cluster is not encrypted. </li> </ul> If <code>DBClusterIdentifier</code> refers to a DB cluster that is not encrypted, then the restore request is rejected.<br/>Tags:  The tags to be applied to the restored DB cluster.<br/>VpcSecurityGroupIds:  A list of VPC security groups that the new DB cluster belongs to.<br/>OptionGroupName:  The name of the option group for the new DB cluster.<br/>DBSubnetGroupName:  The DB subnet group name to use for the new DB cluster. Constraints: If supplied, must match the name of an existing DBSubnetGroup. Example: <code>mySubnetgroup</code> <br/>Port:  The port number on which the new DB cluster accepts connections. Constraints: Value must be <code>1150-65535</code>  Default: The same port as the original DB cluster.<br/>UseLatestRestorableTime:  A value that is set to <code>true</code> to restore the DB cluster to the latest restorable backup time, and <code>false</code> otherwise. Default: <code>false</code>  Constraints: Cannot be specified if <code>RestoreToTime</code> parameter is provided.<br/>RestoreToTime:  The date and time to restore the DB cluster to. Valid Values: Value must be a time in Universal Coordinated Time (UTC) format Constraints: <ul> <li> Must be before the latest restorable time for the DB instance </li> <li> Must be specified if <code>UseLatestRestorableTime</code> parameter is not provided </li> <li> Cannot be specified if <code>UseLatestRestorableTime</code> parameter is true </li> <li> Cannot be specified if <code>RestoreType</code> parameter is <code>copy-on-write</code>  </li> </ul> Example: <code>2015-03-07T23:45:00Z</code> <br/>SourceDBClusterIdentifier:  The identifier of the source DB cluster from which to restore. Constraints: <ul> <li> Must match the identifier of an existing DBCluster. </li> </ul><br/>RestoreType:  The type of restore to be performed. You can specify one of the following values: <ul> <li>  <code>full-copy</code> - The new DB cluster is restored as a full copy of the source DB cluster. </li> <li>  <code>copy-on-write</code> - The new DB cluster is restored as a clone of the source DB cluster. </li> </ul> If you don't specify a <code>RestoreType</code> value, then the new DB cluster is restored as a full copy of the source DB cluster.<br/>DBClusterIdentifier:  The name of the new DB cluster to be created. Constraints: <ul> <li> Must contain from 1 to 63 letters, numbers, or hyphens </li> <li> First character must be a letter </li> <li> Cannot end with a hyphen or contain two consecutive hyphens </li> </ul><br/>
        :return: DBCluster:  Contains the details of an Amazon Neptune DB cluster. This data type is used as a response element in the <a>DescribeDBClusters</a> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB cluster is configured to export to CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloneGroupId</i> Identifies the clone group to which the DB cluster is associated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i> Provides a list of the AWS Identity and Access Management (IAM) roles that are associated with the DB cluster. IAM roles that are associated with a DB cluster grant permission for the DB cluster to access other AWS services on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterArn</i> The Amazon Resource Name (ARN) for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbClusterResourceId</i> The AWS Region-unique, immutable identifier for the DB cluster. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB cluster is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is true, the AWS KMS key identifier for the encrypted DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security groups that the DB cluster belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterMembers</i> Provides the list of instances that make up the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSourceIdentifier</i> Not supported by Neptune.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i> Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterOptionGroupMemberships</i> Provides the list of option group memberships for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies whether the DB cluster has instances in multiple Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReaderEndpoint</i> The reader endpoint for the DB cluster. The reader endpoint for a DB cluster load-balances connections across the Read Replicas that are available in a DB cluster. As clients request new connections to the reader endpoint, Neptune distributes the connection requests among the Read Replicas in the DB cluster. This functionality can help balance your read workload across multiple Read Replicas in your DB cluster. If a failover occurs, and the Read Replica that you are connected to is promoted to be the primary instance, your connection is dropped. To continue sending your read workload to other Read Replicas in the cluster, you can then reconnect to the reader endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint for the primary instance of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestRestorableTime</i> Specifies the earliest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the progress of the operation as a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the current state of this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB cluster, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroup</i> Specifies the name of the DB cluster parameter group for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Contains a user-supplied DB cluster identifier. This identifier is the unique key that identifies a DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> Contains the name of the initial database of this DB cluster that was provided at create time, if one was specified when the DB cluster was created. This same name is returned for the life of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this cluster is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of EC2 Availability Zones that instances in the DB cluster can be created in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i>  <code>AllocatedStorage</code> always returns 1, because Neptune DB cluster storage size is not fixed, but instead automatically adjusts as needed.<br/>        """
        pass



class OpsWorks(BaseClient):
    """<fullname>AWS OpsWorks</fullname> Welcome to the <i>AWS OpsWorks Stacks API Reference</i>. This guide provides descriptions, syntax, and usage examples for AWS OpsWorks Stacks actions and data types, including common parameters and error codes.  AWS OpsWorks Stacks is an application management service that provides an integrated experience for overseeing the complete application lifecycle. For information about this product, go to the <a href="http://aws.amazon.com/opsworks/">AWS OpsWorks</a> details page.   <b>SDKs and CLI</b>  The most common way to use the AWS OpsWorks Stacks API is by using the AWS Command Line Interface (CLI) or by using one of the AWS SDKs to implement applications in your preferred language. For more information, see: <ul> <li>  <a href="https://docs.aws.amazon.com/cli/latest/userguide/cli-chap-welcome.html">AWS CLI</a>  </li> <li>  <a href="https://docs.aws.amazon.com/AWSJavaSDK/latest/javadoc/com/amazonaws/services/opsworks/AWSOpsWorksClient.html">AWS SDK for Java</a>  </li> <li>  <a href="https://docs.aws.amazon.com/sdkfornet/latest/apidocs/html/N_Amazon_OpsWorks.htm">AWS SDK for .NET</a>  </li> <li>  <a href="https://docs.aws.amazon.com/aws-sdk-php-2/latest/class-Aws.OpsWorks.OpsWorksClient.html">AWS SDK for PHP 2</a>  </li> <li>  <a href="http://docs.aws.amazon.com/sdkforruby/api/">AWS SDK for Ruby</a>  </li> <li>  <a href="http://aws.amazon.com/documentation/sdkforjavascript/">AWS SDK for Node.js</a>  </li> <li>  <a href="http://docs.pythonboto.org/en/latest/ref/opsworks.html">AWS SDK for Python(Boto)</a>  </li> </ul>  <b>Endpoints</b>  AWS OpsWorks Stacks supports the following endpoints, all HTTPS. You must connect to one of the following endpoints. Stacks can only be accessed or managed within the endpoint in which they are created. <ul> <li> opsworks.us-east-1.amazonaws.com </li> <li> opsworks.us-east-2.amazonaws.com </li> <li> opsworks.us-west-1.amazonaws.com </li> <li> opsworks.us-west-2.amazonaws.com </li> <li> opsworks.ca-central-1.amazonaws.com (API only; not available in the AWS console) </li> <li> opsworks.eu-west-1.amazonaws.com </li> <li> opsworks.eu-west-2.amazonaws.com </li> <li> opsworks.eu-west-3.amazonaws.com </li> <li> opsworks.eu-central-1.amazonaws.com </li> <li> opsworks.ap-northeast-1.amazonaws.com </li> <li> opsworks.ap-northeast-2.amazonaws.com </li> <li> opsworks.ap-south-1.amazonaws.com </li> <li> opsworks.ap-southeast-1.amazonaws.com </li> <li> opsworks.ap-southeast-2.amazonaws.com </li> <li> opsworks.sa-east-1.amazonaws.com </li> </ul>  <b>Chef Versions</b>  When you call <a>CreateStack</a>, <a>CloneStack</a>, or <a>UpdateStack</a> we recommend you use the <code>ConfigurationManager</code> parameter to specify the Chef version. The recommended and default value for Linux stacks is currently 12. Windows stacks use Chef 12.2. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingcookbook-chef11.html">Chef Versions</a>. <note> You can specify Chef 12, 11.10, or 11.4 for your Linux stack. We recommend migrating your existing Linux stacks to Chef 12 as soon as possible. </note>"""

    class AppExistsWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_apps` for valid parameters"""
            pass
    class DeploymentSuccessfulWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_deployments` for valid parameters"""
            pass
    class InstanceOnlineWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_instances` for valid parameters"""
            pass
    class InstanceRegisteredWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_instances` for valid parameters"""
            pass
    class InstanceStoppedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_instances` for valid parameters"""
            pass
    class InstanceTerminatedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_instances` for valid parameters"""
            pass


    class DescribeEcsClustersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_ecs_clusters` for valid parameters"""
            pass


    class AssignInstanceRequest(Mapping):
        pass
    
    class String(object):
        pass
    
    class AssignVolumeRequest(Mapping):
        pass
    
    class AssociateElasticIpRequest(Mapping):
        pass
    
    class AttachElasticLoadBalancerRequest(Mapping):
        pass
    
    class CloneStackRequest(Mapping):
        pass
    
    class CloneStackResult(Mapping):
        pass
    
    class StackAttributes(object):
        pass
    
    class StackConfigurationManager(Mapping):
        pass
    
    class ChefConfiguration(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class Source(Mapping):
        pass
    
    class CreateAppRequest(Mapping):
        pass
    
    class CreateAppResult(Mapping):
        pass
    
    class DataSource(Mapping):
        pass
    
    class SslConfiguration(Mapping):
        pass
    
    class AppAttributes(object):
        pass
    
    class EnvironmentVariable(Mapping):
        pass
    
    class CreateDeploymentRequest(Mapping):
        pass
    
    class CreateDeploymentResult(Mapping):
        pass
    
    class DeploymentCommand(Mapping):
        pass
    
    class CreateInstanceRequest(Mapping):
        pass
    
    class CreateInstanceResult(Mapping):
        pass
    
    class BlockDeviceMapping(Mapping):
        pass
    
    class CreateLayerRequest(Mapping):
        pass
    
    class CreateLayerResult(Mapping):
        pass
    
    class LayerAttributes(object):
        pass
    
    class CloudWatchLogsConfiguration(Mapping):
        pass
    
    class VolumeConfiguration(Mapping):
        pass
    
    class Recipes(Mapping):
        pass
    
    class LifecycleEventConfiguration(Mapping):
        pass
    
    class CreateStackRequest(Mapping):
        pass
    
    class CreateStackResult(Mapping):
        pass
    
    class CreateUserProfileRequest(Mapping):
        pass
    
    class CreateUserProfileResult(Mapping):
        pass
    
    class DeleteAppRequest(Mapping):
        pass
    
    class DeleteInstanceRequest(Mapping):
        pass
    
    class DeleteLayerRequest(Mapping):
        pass
    
    class DeleteStackRequest(Mapping):
        pass
    
    class DeleteUserProfileRequest(Mapping):
        pass
    
    class DeregisterEcsClusterRequest(Mapping):
        pass
    
    class DeregisterElasticIpRequest(Mapping):
        pass
    
    class DeregisterInstanceRequest(Mapping):
        pass
    
    class DeregisterRdsDbInstanceRequest(Mapping):
        pass
    
    class DeregisterVolumeRequest(Mapping):
        pass
    
    class DescribeAgentVersionsRequest(Mapping):
        pass
    
    class DescribeAgentVersionsResult(Mapping):
        pass
    
    class DescribeAppsRequest(Mapping):
        pass
    
    class DescribeAppsResult(Mapping):
        pass
    
    class DescribeCommandsRequest(Mapping):
        pass
    
    class DescribeCommandsResult(Mapping):
        pass
    
    class DescribeDeploymentsRequest(Mapping):
        pass
    
    class DescribeDeploymentsResult(Mapping):
        pass
    
    class DescribeEcsClustersRequest(Mapping):
        pass
    
    class DescribeEcsClustersResult(Mapping):
        pass
    
    class DescribeElasticIpsRequest(Mapping):
        pass
    
    class DescribeElasticIpsResult(Mapping):
        pass
    
    class DescribeElasticLoadBalancersRequest(Mapping):
        pass
    
    class DescribeElasticLoadBalancersResult(Mapping):
        pass
    
    class DescribeInstancesRequest(Mapping):
        pass
    
    class DescribeInstancesResult(Mapping):
        pass
    
    class DescribeLayersRequest(Mapping):
        pass
    
    class DescribeLayersResult(Mapping):
        pass
    
    class DescribeLoadBasedAutoScalingRequest(Mapping):
        pass
    
    class DescribeLoadBasedAutoScalingResult(Mapping):
        pass
    
    class DescribeMyUserProfileResult(Mapping):
        pass
    
    class DescribeOperatingSystemsResponse(Mapping):
        pass
    
    class DescribePermissionsRequest(Mapping):
        pass
    
    class DescribePermissionsResult(Mapping):
        pass
    
    class DescribeRaidArraysRequest(Mapping):
        pass
    
    class DescribeRaidArraysResult(Mapping):
        pass
    
    class DescribeRdsDbInstancesRequest(Mapping):
        pass
    
    class DescribeRdsDbInstancesResult(Mapping):
        pass
    
    class DescribeServiceErrorsRequest(Mapping):
        pass
    
    class DescribeServiceErrorsResult(Mapping):
        pass
    
    class DescribeStackProvisioningParametersRequest(Mapping):
        pass
    
    class DescribeStackProvisioningParametersResult(Mapping):
        pass
    
    class DescribeStackSummaryRequest(Mapping):
        pass
    
    class DescribeStackSummaryResult(Mapping):
        pass
    
    class DescribeStacksRequest(Mapping):
        pass
    
    class DescribeStacksResult(Mapping):
        pass
    
    class DescribeTimeBasedAutoScalingRequest(Mapping):
        pass
    
    class DescribeTimeBasedAutoScalingResult(Mapping):
        pass
    
    class DescribeUserProfilesRequest(Mapping):
        pass
    
    class DescribeUserProfilesResult(Mapping):
        pass
    
    class DescribeVolumesRequest(Mapping):
        pass
    
    class DescribeVolumesResult(Mapping):
        pass
    
    class DetachElasticLoadBalancerRequest(Mapping):
        pass
    
    class DisassociateElasticIpRequest(Mapping):
        pass
    
    class GetHostnameSuggestionRequest(Mapping):
        pass
    
    class GetHostnameSuggestionResult(Mapping):
        pass
    
    class GrantAccessRequest(Mapping):
        pass
    
    class GrantAccessResult(Mapping):
        pass
    
    class ListTagsRequest(Mapping):
        pass
    
    class ListTagsResult(Mapping):
        pass
    
    class RebootInstanceRequest(Mapping):
        pass
    
    class RegisterEcsClusterRequest(Mapping):
        pass
    
    class RegisterEcsClusterResult(Mapping):
        pass
    
    class RegisterElasticIpRequest(Mapping):
        pass
    
    class RegisterElasticIpResult(Mapping):
        pass
    
    class RegisterInstanceRequest(Mapping):
        pass
    
    class RegisterInstanceResult(Mapping):
        pass
    
    class InstanceIdentity(Mapping):
        pass
    
    class RegisterRdsDbInstanceRequest(Mapping):
        pass
    
    class RegisterVolumeRequest(Mapping):
        pass
    
    class RegisterVolumeResult(Mapping):
        pass
    
    class SetLoadBasedAutoScalingRequest(Mapping):
        pass
    
    class AutoScalingThresholds(Mapping):
        pass
    
    class SetPermissionRequest(Mapping):
        pass
    
    class SetTimeBasedAutoScalingRequest(Mapping):
        pass
    
    class WeeklyAutoScalingSchedule(Mapping):
        pass
    
    class StartInstanceRequest(Mapping):
        pass
    
    class StartStackRequest(Mapping):
        pass
    
    class StopInstanceRequest(Mapping):
        pass
    
    class StopStackRequest(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class Tags(object):
        pass
    
    class UnassignInstanceRequest(Mapping):
        pass
    
    class UnassignVolumeRequest(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateAppRequest(Mapping):
        pass
    
    class UpdateElasticIpRequest(Mapping):
        pass
    
    class UpdateInstanceRequest(Mapping):
        pass
    
    class UpdateLayerRequest(Mapping):
        pass
    
    class UpdateMyUserProfileRequest(Mapping):
        pass
    
    class UpdateRdsDbInstanceRequest(Mapping):
        pass
    
    class UpdateStackRequest(Mapping):
        pass
    
    class UpdateUserProfileRequest(Mapping):
        pass
    
    class UpdateVolumeRequest(Mapping):
        pass
    
    def assign_instance(self, LayerIds: List[String], InstanceId: str):
        """Assign a registered instance to a layer. <ul> <li> You can assign registered on-premises instances to any layer type. </li> <li> You can assign registered Amazon EC2 instances only to custom layers. </li> <li> You cannot use this action with instances that were created with AWS OpsWorks Stacks. </li> </ul>  <b>Required Permissions</b>: To use this action, an AWS Identity and Access Management (IAM) user must have a Manage permissions level for the stack or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param LayerIds:  The layer ID, which must correspond to a custom layer. You cannot assign a registered instance to a built-in layer.<br/>InstanceId:  The instance ID.<br/>
        :return:         """
        pass

    def assign_volume(self, VolumeId: str, InstanceId: str=None):
        """Assigns one of the stack's registered Amazon EBS volumes to a specified instance. The volume must first be registered with the stack by calling <a>RegisterVolume</a>. After you register the volume, you must call <a>UpdateVolume</a> to specify a mount point before calling <code>AssignVolume</code>. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/resources.html">Resource Management</a>.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param InstanceId:  The instance ID.<br/>VolumeId:  The volume ID.<br/>
        :return:         """
        pass

    def associate_elastic_ip(self, ElasticIp: str, InstanceId: str=None):
        """Associates one of the stack's registered Elastic IP addresses with a specified instance. The address must first be registered with the stack by calling <a>RegisterElasticIp</a>. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/resources.html">Resource Management</a>.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param InstanceId:  The instance ID.<br/>ElasticIp:  The Elastic IP address.<br/>
        :return:         """
        pass

    def attach_elastic_load_balancer(self, LayerId: str, ElasticLoadBalancerName: str):
        """Attaches an Elastic Load Balancing load balancer to a specified layer. AWS OpsWorks Stacks does not support Application Load Balancer. You can only use Classic Load Balancer with AWS OpsWorks Stacks. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/layers-elb.html">Elastic Load Balancing</a>. <note> You must create the Elastic Load Balancing instance separately, by using the Elastic Load Balancing console, API, or CLI. For more information, see <a href="https://docs.aws.amazon.com/ElasticLoadBalancing/latest/DeveloperGuide/Welcome.html"> Elastic Load Balancing Developer Guide</a>. </note>  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param LayerId:  The ID of the layer to which the Elastic Load Balancing instance is to be attached.<br/>ElasticLoadBalancerName:  The Elastic Load Balancing instance's name.<br/>
        :return:         """
        pass

    def clone_stack(self, ServiceRoleArn: str, SourceStackId: str, Name: str=None, Region: str=None, VpcId: str=None, Attributes: StackAttributes=None, DefaultInstanceProfileArn: str=None, DefaultOs: str=None, HostnameTheme: str=None, DefaultAvailabilityZone: str=None, DefaultSubnetId: str=None, CustomJson: str=None, ConfigurationManager: StackConfigurationManager=None, ChefConfiguration: ChefConfiguration=None, UseCustomCookbooks: Boolean=None, UseOpsworksSecurityGroups: Boolean=None, CustomCookbooksSource: Source=None, DefaultSshKeyName: str=None, ClonePermissions: Boolean=None, CloneAppIds: List[String]=None, DefaultRootDeviceType: str=None, AgentVersion: str=None) -> CloneStackResult:
        """Creates a clone of a specified stack. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingstacks-cloning.html">Clone a Stack</a>. By default, all parameters are set to the values used by the parent stack.  <b>Required Permissions</b>: To use this action, an IAM user must have an attached policy that explicitly grants permissions. For more information about user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param AgentVersion:  The default AWS OpsWorks Stacks agent version. You have the following options: <ul> <li> Auto-update - Set this parameter to <code>LATEST</code>. AWS OpsWorks Stacks automatically installs new agent versions on the stack's instances as soon as they are available. </li> <li> Fixed version - Set this parameter to your preferred agent version. To update the agent version, you must edit the stack configuration and specify a new version. AWS OpsWorks Stacks then automatically installs that version on the stack's instances. </li> </ul> The default setting is <code>LATEST</code>. To specify an agent version, you must use the complete version number, not the abbreviated number shown on the console. For a list of available agent version numbers, call <a>DescribeAgentVersions</a>. AgentVersion cannot be set to Chef 12.2. <note> You can also specify an agent version when you create or update an instance, which overrides the stack's default setting. </note><br/>DefaultRootDeviceType:  The default root device type. This value is used by default for all instances in the cloned stack, but you can override it when you create an instance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ComponentsAMIs.html#storage-for-the-root-device">Storage for the Root Device</a>.<br/>CloneAppIds:  A list of source stack app IDs to be included in the cloned stack.<br/>ClonePermissions:  Whether to clone the source stack's permissions.<br/>DefaultSshKeyName:  A default Amazon EC2 key pair name. The default value is none. If you specify a key pair name, AWS OpsWorks installs the public key on the instance and you can use the private key with an SSH client to log in to the instance. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinginstances-ssh.html"> Using SSH to Communicate with an Instance</a> and <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/security-ssh-access.html"> Managing SSH Access</a>. You can override this setting by specifying a different key pair, or no key pair, when you <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinginstances-add.html"> create an instance</a>. <br/>CustomCookbooksSource:  Contains the information required to retrieve an app or cookbook from a repository. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingapps-creating.html">Adding Apps</a> or <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingcookbook.html">Cookbooks and Recipes</a>.:<br/>UseOpsworksSecurityGroups:  Whether to associate the AWS OpsWorks Stacks built-in security groups with the stack's layers. AWS OpsWorks Stacks provides a standard set of built-in security groups, one for each layer, which are associated with layers by default. With <code>UseOpsworksSecurityGroups</code> you can instead provide your own custom security groups. <code>UseOpsworksSecurityGroups</code> has the following settings:  <ul> <li> True - AWS OpsWorks Stacks automatically associates the appropriate built-in security group with each layer (default setting). You can associate additional security groups with a layer after you create it but you cannot delete the built-in security group. </li> <li> False - AWS OpsWorks Stacks does not associate built-in security groups with layers. You must create appropriate Amazon Elastic Compute Cloud (Amazon EC2) security groups and associate a security group with each layer that you create. However, you can still manually associate a built-in security group with a layer on creation; custom security groups are required only for those layers that need custom settings. </li> </ul> For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingstacks-creating.html">Create a New Stack</a>.<br/>UseCustomCookbooks:  Whether to use custom cookbooks.<br/>ChefConfiguration:  A <code>ChefConfiguration</code> object that specifies whether to enable Berkshelf and the Berkshelf version on Chef 11.10 stacks. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingstacks-creating.html">Create a New Stack</a>.:<br/>ConfigurationManager:  The configuration manager. When you clone a stack we recommend that you use the configuration manager to specify the Chef version: 12, 11.10, or 11.4 for Linux stacks, or 12.2 for Windows stacks. The default value for Linux stacks is currently 12.:<br/>CustomJson:  A string that contains user-defined, custom JSON. It is used to override the corresponding default stack configuration JSON values. The string should be in the following format:  <code>"{\"key1\": \"value1\", \"key2\": \"value2\",...}"</code>  For more information about custom JSON, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingstacks-json.html">Use Custom JSON to Modify the Stack Configuration Attributes</a> <br/>DefaultSubnetId:  The stack's default VPC subnet ID. This parameter is required if you specify a value for the <code>VpcId</code> parameter. All instances are launched into this subnet unless you specify otherwise when you create the instance. If you also specify a value for <code>DefaultAvailabilityZone</code>, the subnet must be in that zone. For information on default values and when this parameter is required, see the <code>VpcId</code> parameter description. <br/>DefaultAvailabilityZone:  The cloned stack's default Availability Zone, which must be in the specified region. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/rande.html">Regions and Endpoints</a>. If you also specify a value for <code>DefaultSubnetId</code>, the subnet must be in the same zone. For more information, see the <code>VpcId</code> parameter description. <br/>HostnameTheme:  The stack's host name theme, with spaces are replaced by underscores. The theme is used to generate host names for the stack's instances. By default, <code>HostnameTheme</code> is set to <code>Layer_Dependent</code>, which creates host names by appending integers to the layer's short name. The other themes are: <ul> <li>  <code>Baked_Goods</code>  </li> <li>  <code>Clouds</code>  </li> <li>  <code>Europe_Cities</code>  </li> <li>  <code>Fruits</code>  </li> <li>  <code>Greek_Deities_and_Titans</code>  </li> <li>  <code>Legendary_creatures_from_Japan</code>  </li> <li>  <code>Planets_and_Moons</code>  </li> <li>  <code>Roman_Deities</code>  </li> <li>  <code>Scottish_Islands</code>  </li> <li>  <code>US_Cities</code>  </li> <li>  <code>Wild_Cats</code>  </li> </ul> To obtain a generated host name, call <code>GetHostNameSuggestion</code>, which returns a host name based on the current theme.<br/>DefaultOs:  The stack's operating system, which must be set to one of the following. <ul> <li> A supported Linux operating system: An Amazon Linux version, such as <code>Amazon Linux 2018.03</code>, <code>Amazon Linux 2017.09</code>, <code>Amazon Linux 2017.03</code>, <code>Amazon Linux 2016.09</code>, <code>Amazon Linux 2016.03</code>, <code>Amazon Linux 2015.09</code>, or <code>Amazon Linux 2015.03</code>. </li> <li> A supported Ubuntu operating system, such as <code>Ubuntu 16.04 LTS</code>, <code>Ubuntu 14.04 LTS</code>, or <code>Ubuntu 12.04 LTS</code>. </li> <li>  <code>CentOS Linux 7</code>  </li> <li>  <code>Red Hat Enterprise Linux 7</code>  </li> <li>  <code>Microsoft Windows Server 2012 R2 Base</code>, <code>Microsoft Windows Server 2012 R2 with SQL Server Express</code>, <code>Microsoft Windows Server 2012 R2 with SQL Server Standard</code>, or <code>Microsoft Windows Server 2012 R2 with SQL Server Web</code>. </li> <li> A custom AMI: <code>Custom</code>. You specify the custom AMI you want to use when you create instances. For more information about how to use custom AMIs with OpsWorks, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinginstances-custom-ami.html">Using Custom AMIs</a>. </li> </ul> The default option is the parent stack's operating system. For more information about supported operating systems, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinginstances-os.html">AWS OpsWorks Stacks Operating Systems</a>. <note> You can specify a different Linux operating system for the cloned stack, but you cannot change from Linux to Windows or Windows to Linux. </note><br/>DefaultInstanceProfileArn:  The Amazon Resource Name (ARN) of an IAM profile that is the default profile for all of the stack's EC2 instances. For more information about IAM ARNs, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">Using Identifiers</a>.<br/>ServiceRoleArn:  The stack AWS Identity and Access Management (IAM) role, which allows AWS OpsWorks Stacks to work with AWS resources on your behalf. You must set this parameter to the Amazon Resource Name (ARN) for an existing IAM role. If you create a stack by using the AWS OpsWorks Stacks console, it creates the role for you. You can obtain an existing stack's IAM ARN programmatically by calling <a>DescribePermissions</a>. For more information about IAM ARNs, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">Using Identifiers</a>. <note> You must set this parameter to a valid service role ARN or the action will fail; there is no default value. You can specify the source stack's service role ARN, if you prefer, but you must do so explicitly. </note><br/>Attributes:  A list of stack attributes and values as key/value pairs to be added to the cloned stack.<br/>VpcId:  The ID of the VPC that the cloned stack is to be launched into. It must be in the specified region. All instances are launched into this VPC, and you cannot change the ID later. <ul> <li> If your account supports EC2 Classic, the default value is no VPC. </li> <li> If your account does not support EC2 Classic, the default value is the default VPC for the specified region. </li> </ul> If the VPC ID corresponds to a default VPC and you have specified either the <code>DefaultAvailabilityZone</code> or the <code>DefaultSubnetId</code> parameter only, AWS OpsWorks Stacks infers the value of the other parameter. If you specify neither parameter, AWS OpsWorks Stacks sets these parameters to the first valid Availability Zone for the specified region and the corresponding default VPC subnet ID, respectively.  If you specify a nondefault VPC ID, note the following: <ul> <li> It must belong to a VPC in your account that is in the specified region. </li> <li> You must specify a value for <code>DefaultSubnetId</code>. </li> </ul> For more information about how to use AWS OpsWorks Stacks with a VPC, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingstacks-vpc.html">Running a Stack in a VPC</a>. For more information about default VPC and EC2 Classic, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-supported-platforms.html">Supported Platforms</a>. <br/>Region:  The cloned stack AWS region, such as "ap-northeast-2". For more information about AWS regions, see <a href="https://docs.aws.amazon.com/general/latest/gr/rande.html">Regions and Endpoints</a>.<br/>Name:  The cloned stack name.<br/>SourceStackId:  The source stack ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The Chef version. This parameter must be set to 12, 11.10, or 11.4 for Linux stacks, and to 12.2 for Windows stacks. The default value for Linux stacks is 11.4.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name. This parameter must be set to "Chef".<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BerkshelfVersion</i> The Berkshelf version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManageBerkshelf</i> Whether to enable Berkshelf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Revision</i> The application's version. AWS OpsWorks Stacks enables you to easily deploy new versions of an application. One of the simplest approaches is to have branches or revisions in your repository that represent different versions that can potentially be deployed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SshKey</i> In requests, the repository's SSH key. In responses, AWS OpsWorks Stacks returns <code>*****FILTERED*****</code> instead of the actual value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Password</i> When included in a request, the parameter depends on the repository type. <ul> <li> For Amazon S3 bundles, set <code>Password</code> to the appropriate IAM secret access key. </li> <li> For HTTP bundles and Subversion repositories, set <code>Password</code> to the password. </li> </ul> For more information on how to safely handle IAM credentials, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-access-keys-best-practices.html">https://docs.aws.amazon.com/general/latest/gr/aws-access-keys-best-practices.html</a>. In responses, AWS OpsWorks Stacks returns <code>*****FILTERED*****</code> instead of the actual value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> This parameter depends on the repository type. <ul> <li> For Amazon S3 bundles, set <code>Username</code> to the appropriate IAM access key ID. </li> <li> For HTTP bundles, Git repositories, and Subversion repositories, set <code>Username</code> to the user name. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Url</i> The source URL. The following is an example of an Amazon S3 source URL: <code>https://s3.amazonaws.com/opsworks-demo-bucket/opsworks_cookbook_demo.tar.gz</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The repository type.<br/>
        :return: StackId:  The cloned stack ID.<br/>        """
        pass

    def create_app(self, Type: str, Name: str, StackId: str, Shortname: str=None, Description: str=None, DataSources: List[DataSource]=None, AppSource: Source=None, Domains: List[String]=None, EnableSsl: Boolean=None, SslConfiguration: SslConfiguration=None, Attributes: AppAttributes=None, Environment: List[EnvironmentVariable]=None) -> CreateAppResult:
        """Creates an app for a specified stack. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingapps-creating.html">Creating Apps</a>.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param Environment:  An array of <code>EnvironmentVariable</code> objects that specify environment variables to be associated with the app. After you deploy the app, these variables are defined on the associated app server instance. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingapps-creating.html#workingapps-creating-environment"> Environment Variables</a>. There is no specific limit on the number of environment variables. However, the size of the associated data structure - which includes the variables' names, values, and protected flag values - cannot exceed 20 KB. This limit should accommodate most if not all use cases. Exceeding it will cause an exception with the message, "Environment: is too large (maximum is 20KB)." <note> If you have specified one or more environment variables, you cannot modify the stack's Chef version. </note><br/>Attributes:  One or more user-defined key/value pairs to be added to the stack attributes.<br/>SslConfiguration:  An <code>SslConfiguration</code> object with the SSL configuration.:<br/>EnableSsl:  Whether to enable SSL for the app.<br/>Domains:  The app virtual host settings, with multiple domains separated by commas. For example: <code>'www.example.com, example.com'</code> <br/>AppSource:  A <code>Source</code> object that specifies the app repository.:<br/>Type:  The app type. Each supported type is associated with a particular layer. For example, PHP applications are associated with a PHP layer. AWS OpsWorks Stacks deploys an application to those instances that are members of the corresponding layer. If your app isn't one of the standard types, or you prefer to implement your own Deploy recipes, specify <code>other</code>.<br/>DataSources:  The app's data source.<br/>Description:  A description of the app.<br/>Name:  The app name.<br/>Shortname:  The app's short name.<br/>StackId:  The stack ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Revision</i> The application's version. AWS OpsWorks Stacks enables you to easily deploy new versions of an application. One of the simplest approaches is to have branches or revisions in your repository that represent different versions that can potentially be deployed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SshKey</i> In requests, the repository's SSH key. In responses, AWS OpsWorks Stacks returns <code>*****FILTERED*****</code> instead of the actual value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Password</i> When included in a request, the parameter depends on the repository type. <ul> <li> For Amazon S3 bundles, set <code>Password</code> to the appropriate IAM secret access key. </li> <li> For HTTP bundles and Subversion repositories, set <code>Password</code> to the password. </li> </ul> For more information on how to safely handle IAM credentials, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-access-keys-best-practices.html">https://docs.aws.amazon.com/general/latest/gr/aws-access-keys-best-practices.html</a>. In responses, AWS OpsWorks Stacks returns <code>*****FILTERED*****</code> instead of the actual value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> This parameter depends on the repository type. <ul> <li> For Amazon S3 bundles, set <code>Username</code> to the appropriate IAM access key ID. </li> <li> For HTTP bundles, Git repositories, and Subversion repositories, set <code>Username</code> to the user name. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Url</i> The source URL. The following is an example of an Amazon S3 source URL: <code>https://s3.amazonaws.com/opsworks-demo-bucket/opsworks_cookbook_demo.tar.gz</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The repository type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Chain</i> Optional. Can be used to specify an intermediate certificate authority key or client authentication.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrivateKey</i> The private key; the contents of the certificate's domain.kex file.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Certificate</i> The contents of the certificate's domain.crt file.<br/>
        :return: AppId:  The app ID.<br/>        """
        pass

    def create_deployment(self, Command: DeploymentCommand, StackId: str, AppId: str=None, InstanceIds: List[String]=None, LayerIds: List[String]=None, Comment: str=None, CustomJson: str=None) -> CreateDeploymentResult:
        """Runs deployment or stack commands. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingapps-deploying.html">Deploying Apps</a> and <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingstacks-commands.html">Run Stack Commands</a>.  <b>Required Permissions</b>: To use this action, an IAM user must have a Deploy or Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param CustomJson:  A string that contains user-defined, custom JSON. You can use this parameter to override some corresponding default stack configuration JSON values. The string should be in the following format:  <code>"{\"key1\": \"value1\", \"key2\": \"value2\",...}"</code>  For more information about custom JSON, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingstacks-json.html">Use Custom JSON to Modify the Stack Configuration Attributes</a> and <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingcookbook-json-override.html">Overriding Attributes With Custom JSON</a>.<br/>Comment:  A user-defined comment.<br/>Command:  A <code>DeploymentCommand</code> object that specifies the deployment command and any associated arguments.:<br/>LayerIds:  The layer IDs for the deployment targets.<br/>InstanceIds:  The instance IDs for the deployment targets.<br/>AppId:  The app ID. This parameter is required for app deployments, but not for other deployment commands.<br/>StackId:  The stack ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Args</i> The arguments of those commands that take arguments. It should be set to a JSON object with the following format:  <code>{"arg_name1" : ["value1", "value2", ...], "arg_name2" : ["value1", "value2", ...], ...}</code>  The <code>update_dependencies</code> command takes two arguments: <ul> <li>  <code>upgrade_os_to</code> - Specifies the desired Amazon Linux version for instances whose OS you want to upgrade, such as <code>Amazon Linux 2016.09</code>. You must also set the <code>allow_reboot</code> argument to true. </li> <li>  <code>allow_reboot</code> - Specifies whether to allow AWS OpsWorks Stacks to reboot the instances if necessary, after installing the updates. This argument can be set to either <code>true</code> or <code>false</code>. The default value is <code>false</code>. </li> </ul> For example, to upgrade an instance to Amazon Linux 2016.09, set <code>Args</code> to the following.  <code> { "upgrade_os_to":["Amazon Linux 2016.09"], "allow_reboot":["true"] } </code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Specifies the operation. You can specify only one command. For stacks, the following commands are available: <ul> <li>  <code>execute_recipes</code>: Execute one or more recipes. To specify the recipes, set an <code>Args</code> parameter named <code>recipes</code> to the list of recipes to be executed. For example, to execute <code>phpapp::appsetup</code>, set <code>Args</code> to <code>{"recipes":["phpapp::appsetup"]}</code>. </li> <li>  <code>install_dependencies</code>: Install the stack's dependencies. </li> <li>  <code>update_custom_cookbooks</code>: Update the stack's custom cookbooks. </li> <li>  <code>update_dependencies</code>: Update the stack's dependencies. </li> </ul> <note> The update_dependencies and install_dependencies commands are supported only for Linux instances. You can run the commands successfully on Windows instances, but they do nothing. </note> For apps, the following commands are available: <ul> <li>  <code>deploy</code>: Deploy an app. Ruby on Rails apps have an optional <code>Args</code> parameter named <code>migrate</code>. Set <code>Args</code> to {"migrate":["true"]} to migrate the database. The default setting is {"migrate":["false"]}. </li> <li>  <code>rollback</code> Roll the app back to the previous version. When you update an app, AWS OpsWorks Stacks stores the previous version, up to a maximum of five versions. You can use this command to roll an app back as many as four versions. </li> <li>  <code>start</code>: Start the app's web or application server. </li> <li>  <code>stop</code>: Stop the app's web or application server. </li> <li>  <code>restart</code>: Restart the app's web or application server. </li> <li>  <code>undeploy</code>: Undeploy the app. </li> </ul><br/>
        :return: DeploymentId:  The deployment ID, which can be used with other requests to identify the deployment.<br/>        """
        pass

    def create_instance(self, InstanceType: str, LayerIds: List[String], StackId: str, AutoScalingType: str=None, Hostname: str=None, Os: str=None, AmiId: str=None, SshKeyName: str=None, AvailabilityZone: str=None, VirtualizationType: str=None, SubnetId: str=None, Architecture: str=None, RootDeviceType: str=None, BlockDeviceMappings: List[BlockDeviceMapping]=None, InstallUpdatesOnBoot: Boolean=None, EbsOptimized: Boolean=None, AgentVersion: str=None, Tenancy: str=None) -> CreateInstanceResult:
        """Creates an instance in a specified stack. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinginstances-add.html">Adding an Instance to a Layer</a>.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param Tenancy:  The instance's tenancy option. The default option is no tenancy, or if the instance is running in a VPC, inherit tenancy settings from the VPC. The following are valid values for this parameter: <code>dedicated</code>, <code>default</code>, or <code>host</code>. Because there are costs associated with changes in tenancy options, we recommend that you research tenancy options before choosing them for your instances. For more information about dedicated hosts, see <a href="http://aws.amazon.com/ec2/dedicated-hosts/">Dedicated Hosts Overview</a> and <a href="http://aws.amazon.com/ec2/dedicated-hosts/">Amazon EC2 Dedicated Hosts</a>. For more information about dedicated instances, see <a href="https://docs.aws.amazon.com/AmazonVPC/latest/UserGuide/dedicated-instance.html">Dedicated Instances</a> and <a href="http://aws.amazon.com/ec2/purchasing-options/dedicated-instances/">Amazon EC2 Dedicated Instances</a>.<br/>AgentVersion:  The default AWS OpsWorks Stacks agent version. You have the following options: <ul> <li>  <code>INHERIT</code> - Use the stack's default agent version setting. </li> <li>  <i>version_number</i> - Use the specified agent version. This value overrides the stack's default setting. To update the agent version, edit the instance configuration and specify a new version. AWS OpsWorks Stacks then automatically installs that version on the instance. </li> </ul> The default setting is <code>INHERIT</code>. To specify an agent version, you must use the complete version number, not the abbreviated number shown on the console. For a list of available agent version numbers, call <a>DescribeAgentVersions</a>. AgentVersion cannot be set to Chef 12.2.<br/>EbsOptimized:  Whether to create an Amazon EBS-optimized instance.<br/>InstallUpdatesOnBoot:  Whether to install operating system and package updates when the instance boots. The default value is <code>true</code>. To control when updates are installed, set this value to <code>false</code>. You must then update your instances manually by using <a>CreateDeployment</a> to run the <code>update_dependencies</code> stack command or by manually running <code>yum</code> (Amazon Linux) or <code>apt-get</code> (Ubuntu) on the instances.  <note> We strongly recommend using the default value of <code>true</code> to ensure that your instances have the latest security updates. </note><br/>BlockDeviceMappings:  An array of <code>BlockDeviceMapping</code> objects that specify the instance's block devices. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/block-device-mapping-concepts.html">Block Device Mapping</a>. Note that block device mappings are not supported for custom AMIs.<br/>RootDeviceType:  The instance root device type. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ComponentsAMIs.html#storage-for-the-root-device">Storage for the Root Device</a>.<br/>Architecture:  The instance architecture. The default option is <code>x86_64</code>. Instance types do not necessarily support both architectures. For a list of the architectures that are supported by the different instance types, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html">Instance Families and Types</a>.<br/>SubnetId:  The ID of the instance's subnet. If the stack is running in a VPC, you can use this parameter to override the stack's default subnet ID value and direct AWS OpsWorks Stacks to launch the instance in a different subnet.<br/>VirtualizationType:  The instance's virtualization type, <code>paravirtual</code> or <code>hvm</code>.<br/>AvailabilityZone:  The instance Availability Zone. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/rande.html">Regions and Endpoints</a>.<br/>SshKeyName:  The instance's Amazon EC2 key-pair name.<br/>AmiId:  A custom AMI ID to be used to create the instance. The AMI should be based on one of the supported operating systems. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinginstances-custom-ami.html">Using Custom AMIs</a>. <note> If you specify a custom AMI, you must set <code>Os</code> to <code>Custom</code>. </note><br/>Os:  The instance's operating system, which must be set to one of the following. <ul> <li> A supported Linux operating system: An Amazon Linux version, such as <code>Amazon Linux 2018.03</code>, <code>Amazon Linux 2017.09</code>, <code>Amazon Linux 2017.03</code>, <code>Amazon Linux 2016.09</code>, <code>Amazon Linux 2016.03</code>, <code>Amazon Linux 2015.09</code>, or <code>Amazon Linux 2015.03</code>. </li> <li> A supported Ubuntu operating system, such as <code>Ubuntu 16.04 LTS</code>, <code>Ubuntu 14.04 LTS</code>, or <code>Ubuntu 12.04 LTS</code>. </li> <li>  <code>CentOS Linux 7</code>  </li> <li>  <code>Red Hat Enterprise Linux 7</code>  </li> <li> A supported Windows operating system, such as <code>Microsoft Windows Server 2012 R2 Base</code>, <code>Microsoft Windows Server 2012 R2 with SQL Server Express</code>, <code>Microsoft Windows Server 2012 R2 with SQL Server Standard</code>, or <code>Microsoft Windows Server 2012 R2 with SQL Server Web</code>. </li> <li> A custom AMI: <code>Custom</code>. </li> </ul> For more information about the supported operating systems, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinginstances-os.html">AWS OpsWorks Stacks Operating Systems</a>. The default option is the current Amazon Linux version. If you set this parameter to <code>Custom</code>, you must use the <a>CreateInstance</a> action's AmiId parameter to specify the custom AMI that you want to use. Block device mappings are not supported if the value is <code>Custom</code>. For more information about supported operating systems, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinginstances-os.html">Operating Systems</a>For more information about how to use custom AMIs with AWS OpsWorks Stacks, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinginstances-custom-ami.html">Using Custom AMIs</a>.<br/>Hostname:  The instance host name.<br/>AutoScalingType:  For load-based or time-based instances, the type. Windows stacks can use only time-based instances.<br/>InstanceType:  The instance type, such as <code>t2.micro</code>. For a list of supported instance types, open the stack in the console, choose <b>Instances</b>, and choose <b>+ Instance</b>. The <b>Size</b> list contains the currently supported types. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html">Instance Families and Types</a>. The parameter values that you use to specify the various types are in the <b>API Name</b> column of the <b>Available Instance Types</b> table.<br/>LayerIds:  An array that contains the instance's layer IDs.<br/>StackId:  The stack ID.<br/>
        :return: InstanceId:  The instance ID.<br/>        """
        pass

    def create_layer(self, Shortname: str, Name: str, Type: str, StackId: str, Attributes: LayerAttributes=None, CloudWatchLogsConfiguration: CloudWatchLogsConfiguration=None, CustomInstanceProfileArn: str=None, CustomJson: str=None, CustomSecurityGroupIds: List[String]=None, Packages: List[String]=None, VolumeConfigurations: List[VolumeConfiguration]=None, EnableAutoHealing: Boolean=None, AutoAssignElasticIps: Boolean=None, AutoAssignPublicIps: Boolean=None, CustomRecipes: Recipes=None, InstallUpdatesOnBoot: Boolean=None, UseEbsOptimizedInstances: Boolean=None, LifecycleEventConfiguration: LifecycleEventConfiguration=None) -> CreateLayerResult:
        """Creates a layer. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinglayers-basics-create.html">How to Create a Layer</a>. <note> You should use <b>CreateLayer</b> for noncustom layer types such as PHP App Server only if the stack does not have an existing layer of that type. A stack can have at most one instance of each noncustom layer; if you attempt to create a second instance, <b>CreateLayer</b> fails. A stack can have an arbitrary number of custom layers, so you can call <b>CreateLayer</b> as many times as you like for that layer type. </note>  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param LifecycleEventConfiguration:  A <code>LifeCycleEventConfiguration</code> object that you can use to configure the Shutdown event to specify an execution timeout and enable or disable Elastic Load Balancer connection draining.:<br/>UseEbsOptimizedInstances:  Whether to use Amazon EBS-optimized instances.<br/>InstallUpdatesOnBoot:  Whether to install operating system and package updates when the instance boots. The default value is <code>true</code>. To control when updates are installed, set this value to <code>false</code>. You must then update your instances manually by using <a>CreateDeployment</a> to run the <code>update_dependencies</code> stack command or by manually running <code>yum</code> (Amazon Linux) or <code>apt-get</code> (Ubuntu) on the instances.  <note> To ensure that your instances have the latest security updates, we strongly recommend using the default value of <code>true</code>. </note><br/>CustomRecipes:  A <code>LayerCustomRecipes</code> object that specifies the layer custom recipes.:<br/>AutoAssignPublicIps:  For stacks that are running in a VPC, whether to automatically assign a public IP address to the layer's instances. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinglayers-basics-edit.html">How to Edit a Layer</a>.<br/>AutoAssignElasticIps:  Whether to automatically assign an <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/elastic-ip-addresses-eip.html">Elastic IP address</a> to the layer's instances. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinglayers-basics-edit.html">How to Edit a Layer</a>.<br/>EnableAutoHealing:  Whether to disable auto healing for the layer.<br/>VolumeConfigurations:  A <code>VolumeConfigurations</code> object that describes the layer's Amazon EBS volumes.<br/>Packages:  An array of <code>Package</code> objects that describes the layer packages.<br/>CustomSecurityGroupIds:  An array containing the layer custom security group IDs.<br/>CustomJson:  A JSON-formatted string containing custom stack configuration and deployment attributes to be installed on the layer's instances. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingcookbook-json-override.html"> Using Custom JSON</a>. This feature is supported as of version 1.7.42 of the AWS CLI. <br/>CustomInstanceProfileArn:  The ARN of an IAM profile to be used for the layer's EC2 instances. For more information about IAM ARNs, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">Using Identifiers</a>.<br/>CloudWatchLogsConfiguration:  Specifies CloudWatch Logs configuration options for the layer. For more information, see <a>CloudWatchLogsLogStream</a>.:<br/>Attributes:  One or more user-defined key-value pairs to be added to the stack attributes. To create a cluster layer, set the <code>EcsClusterArn</code> attribute to the cluster's ARN.<br/>Shortname:  For custom layers only, use this parameter to specify the layer's short name, which is used internally by AWS OpsWorks Stacks and by Chef recipes. The short name is also used as the name for the directory where your app files are installed. It can have a maximum of 200 characters, which are limited to the alphanumeric characters, '-', '_', and '.'. The built-in layers' short names are defined by AWS OpsWorks Stacks. For more information, see the <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/layers.html">Layer Reference</a>.<br/>Name:  The layer name, which is used by the console.<br/>Type:  The layer type. A stack cannot have more than one built-in layer of the same type. It can have any number of custom layers. Built-in layers are not available in Chef 12 stacks.<br/>StackId:  The layer stack ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogStreams</i> A list of configuration options for CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Whether CloudWatch Logs is enabled for a layer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Shutdown</i> An array of custom recipe names to be run following a <code>shutdown</code> event.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Undeploy</i> An array of custom recipe names to be run following a <code>undeploy</code> event.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Deploy</i> An array of custom recipe names to be run following a <code>deploy</code> event.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Configure</i> An array of custom recipe names to be run following a <code>configure</code> event.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Setup</i> An array of custom recipe names to be run following a <code>setup</code> event.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Shutdown</i> A <code>ShutdownEventConfiguration</code> object that specifies the Shutdown event configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Shutdown</b>: A <code>ShutdownEventConfiguration</code> object that specifies the Shutdown event configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DelayUntilElbConnectionsDrained</i> Whether to enable Elastic Load Balancing connection draining. For more information, see <a href="https://docs.aws.amazon.com/ElasticLoadBalancing/latest/DeveloperGuide/TerminologyandKeyConcepts.html#conn-drain">Connection Draining</a> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExecutionTimeout</i> The time, in seconds, that AWS OpsWorks Stacks will wait after triggering a Shutdown event before shutting down an instance.<br/>
        :return: LayerId:  The layer ID.<br/>        """
        pass

    def create_stack(self, DefaultInstanceProfileArn: str, ServiceRoleArn: str, Region: str, Name: str, VpcId: str=None, Attributes: StackAttributes=None, DefaultOs: str=None, HostnameTheme: str=None, DefaultAvailabilityZone: str=None, DefaultSubnetId: str=None, CustomJson: str=None, ConfigurationManager: StackConfigurationManager=None, ChefConfiguration: ChefConfiguration=None, UseCustomCookbooks: Boolean=None, UseOpsworksSecurityGroups: Boolean=None, CustomCookbooksSource: Source=None, DefaultSshKeyName: str=None, DefaultRootDeviceType: str=None, AgentVersion: str=None) -> CreateStackResult:
        """Creates a new stack. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingstacks-edit.html">Create a New Stack</a>.  <b>Required Permissions</b>: To use this action, an IAM user must have an attached policy that explicitly grants permissions. For more information about user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param AgentVersion:  The default AWS OpsWorks Stacks agent version. You have the following options: <ul> <li> Auto-update - Set this parameter to <code>LATEST</code>. AWS OpsWorks Stacks automatically installs new agent versions on the stack's instances as soon as they are available. </li> <li> Fixed version - Set this parameter to your preferred agent version. To update the agent version, you must edit the stack configuration and specify a new version. AWS OpsWorks Stacks then automatically installs that version on the stack's instances. </li> </ul> The default setting is the most recent release of the agent. To specify an agent version, you must use the complete version number, not the abbreviated number shown on the console. For a list of available agent version numbers, call <a>DescribeAgentVersions</a>. AgentVersion cannot be set to Chef 12.2. <note> You can also specify an agent version when you create or update an instance, which overrides the stack's default setting. </note><br/>DefaultRootDeviceType:  The default root device type. This value is the default for all instances in the stack, but you can override it when you create an instance. The default option is <code>instance-store</code>. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ComponentsAMIs.html#storage-for-the-root-device">Storage for the Root Device</a>.<br/>DefaultSshKeyName:  A default Amazon EC2 key pair name. The default value is none. If you specify a key pair name, AWS OpsWorks installs the public key on the instance and you can use the private key with an SSH client to log in to the instance. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinginstances-ssh.html"> Using SSH to Communicate with an Instance</a> and <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/security-ssh-access.html"> Managing SSH Access</a>. You can override this setting by specifying a different key pair, or no key pair, when you <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinginstances-add.html"> create an instance</a>. <br/>CustomCookbooksSource:  Contains the information required to retrieve an app or cookbook from a repository. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingapps-creating.html">Adding Apps</a> or <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingcookbook.html">Cookbooks and Recipes</a>.:<br/>UseOpsworksSecurityGroups:  Whether to associate the AWS OpsWorks Stacks built-in security groups with the stack's layers. AWS OpsWorks Stacks provides a standard set of built-in security groups, one for each layer, which are associated with layers by default. With <code>UseOpsworksSecurityGroups</code> you can instead provide your own custom security groups. <code>UseOpsworksSecurityGroups</code> has the following settings:  <ul> <li> True - AWS OpsWorks Stacks automatically associates the appropriate built-in security group with each layer (default setting). You can associate additional security groups with a layer after you create it, but you cannot delete the built-in security group. </li> <li> False - AWS OpsWorks Stacks does not associate built-in security groups with layers. You must create appropriate EC2 security groups and associate a security group with each layer that you create. However, you can still manually associate a built-in security group with a layer on creation; custom security groups are required only for those layers that need custom settings. </li> </ul> For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingstacks-creating.html">Create a New Stack</a>.<br/>UseCustomCookbooks:  Whether the stack uses custom cookbooks.<br/>ChefConfiguration:  A <code>ChefConfiguration</code> object that specifies whether to enable Berkshelf and the Berkshelf version on Chef 11.10 stacks. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingstacks-creating.html">Create a New Stack</a>.:<br/>ConfigurationManager:  The configuration manager. When you create a stack we recommend that you use the configuration manager to specify the Chef version: 12, 11.10, or 11.4 for Linux stacks, or 12.2 for Windows stacks. The default value for Linux stacks is currently 12.:<br/>CustomJson:  A string that contains user-defined, custom JSON. It can be used to override the corresponding default stack configuration attribute values or to pass data to recipes. The string should be in the following format:  <code>"{\"key1\": \"value1\", \"key2\": \"value2\",...}"</code>  For more information about custom JSON, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingstacks-json.html">Use Custom JSON to Modify the Stack Configuration Attributes</a>.<br/>DefaultSubnetId:  The stack's default VPC subnet ID. This parameter is required if you specify a value for the <code>VpcId</code> parameter. All instances are launched into this subnet unless you specify otherwise when you create the instance. If you also specify a value for <code>DefaultAvailabilityZone</code>, the subnet must be in that zone. For information on default values and when this parameter is required, see the <code>VpcId</code> parameter description. <br/>DefaultAvailabilityZone:  The stack's default Availability Zone, which must be in the specified region. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/rande.html">Regions and Endpoints</a>. If you also specify a value for <code>DefaultSubnetId</code>, the subnet must be in the same zone. For more information, see the <code>VpcId</code> parameter description. <br/>HostnameTheme:  The stack's host name theme, with spaces replaced by underscores. The theme is used to generate host names for the stack's instances. By default, <code>HostnameTheme</code> is set to <code>Layer_Dependent</code>, which creates host names by appending integers to the layer's short name. The other themes are: <ul> <li>  <code>Baked_Goods</code>  </li> <li>  <code>Clouds</code>  </li> <li>  <code>Europe_Cities</code>  </li> <li>  <code>Fruits</code>  </li> <li>  <code>Greek_Deities_and_Titans</code>  </li> <li>  <code>Legendary_creatures_from_Japan</code>  </li> <li>  <code>Planets_and_Moons</code>  </li> <li>  <code>Roman_Deities</code>  </li> <li>  <code>Scottish_Islands</code>  </li> <li>  <code>US_Cities</code>  </li> <li>  <code>Wild_Cats</code>  </li> </ul> To obtain a generated host name, call <code>GetHostNameSuggestion</code>, which returns a host name based on the current theme.<br/>DefaultOs:  The stack's default operating system, which is installed on every instance unless you specify a different operating system when you create the instance. You can specify one of the following. <ul> <li> A supported Linux operating system: An Amazon Linux version, such as <code>Amazon Linux 2018.03</code>, <code>Amazon Linux 2017.09</code>, <code>Amazon Linux 2017.03</code>, <code>Amazon Linux 2016.09</code>, <code>Amazon Linux 2016.03</code>, <code>Amazon Linux 2015.09</code>, or <code>Amazon Linux 2015.03</code>. </li> <li> A supported Ubuntu operating system, such as <code>Ubuntu 16.04 LTS</code>, <code>Ubuntu 14.04 LTS</code>, or <code>Ubuntu 12.04 LTS</code>. </li> <li>  <code>CentOS Linux 7</code>  </li> <li>  <code>Red Hat Enterprise Linux 7</code>  </li> <li> A supported Windows operating system, such as <code>Microsoft Windows Server 2012 R2 Base</code>, <code>Microsoft Windows Server 2012 R2 with SQL Server Express</code>, <code>Microsoft Windows Server 2012 R2 with SQL Server Standard</code>, or <code>Microsoft Windows Server 2012 R2 with SQL Server Web</code>. </li> <li> A custom AMI: <code>Custom</code>. You specify the custom AMI you want to use when you create instances. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinginstances-custom-ami.html"> Using Custom AMIs</a>. </li> </ul> The default option is the current Amazon Linux version. For more information about supported operating systems, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinginstances-os.html">AWS OpsWorks Stacks Operating Systems</a>.<br/>DefaultInstanceProfileArn:  The Amazon Resource Name (ARN) of an IAM profile that is the default profile for all of the stack's EC2 instances. For more information about IAM ARNs, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">Using Identifiers</a>.<br/>ServiceRoleArn:  The stack's AWS Identity and Access Management (IAM) role, which allows AWS OpsWorks Stacks to work with AWS resources on your behalf. You must set this parameter to the Amazon Resource Name (ARN) for an existing IAM role. For more information about IAM ARNs, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">Using Identifiers</a>.<br/>Attributes:  One or more user-defined key-value pairs to be added to the stack attributes.<br/>VpcId:  The ID of the VPC that the stack is to be launched into. The VPC must be in the stack's region. All instances are launched into this VPC. You cannot change the ID later. <ul> <li> If your account supports EC2-Classic, the default value is <code>no VPC</code>. </li> <li> If your account does not support EC2-Classic, the default value is the default VPC for the specified region. </li> </ul> If the VPC ID corresponds to a default VPC and you have specified either the <code>DefaultAvailabilityZone</code> or the <code>DefaultSubnetId</code> parameter only, AWS OpsWorks Stacks infers the value of the other parameter. If you specify neither parameter, AWS OpsWorks Stacks sets these parameters to the first valid Availability Zone for the specified region and the corresponding default VPC subnet ID, respectively. If you specify a nondefault VPC ID, note the following: <ul> <li> It must belong to a VPC in your account that is in the specified region. </li> <li> You must specify a value for <code>DefaultSubnetId</code>. </li> </ul> For more information about how to use AWS OpsWorks Stacks with a VPC, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingstacks-vpc.html">Running a Stack in a VPC</a>. For more information about default VPC and EC2-Classic, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-supported-platforms.html">Supported Platforms</a>. <br/>Region:  The stack's AWS region, such as <code>ap-south-1</code>. For more information about Amazon regions, see <a href="https://docs.aws.amazon.com/general/latest/gr/rande.html">Regions and Endpoints</a>. <note> In the AWS CLI, this API maps to the <code>--stack-region</code> parameter. If the <code>--stack-region</code> parameter and the AWS CLI common parameter <code>--region</code> are set to the same value, the stack uses a <i>regional</i> endpoint. If the <code>--stack-region</code> parameter is not set, but the AWS CLI <code>--region</code> parameter is, this also results in a stack with a <i>regional</i> endpoint. However, if the <code>--region</code> parameter is set to <code>us-east-1</code>, and the <code>--stack-region</code> parameter is set to one of the following, then the stack uses a legacy or <i>classic</i> region: <code>us-west-1, us-west-2, sa-east-1, eu-central-1, eu-west-1, ap-northeast-1, ap-southeast-1, ap-southeast-2</code>. In this case, the actual API endpoint of the stack is in <code>us-east-1</code>. Only the preceding regions are supported as classic regions in the <code>us-east-1</code> API endpoint. Because it is a best practice to choose the regional endpoint that is closest to where you manage AWS, we recommend that you use regional endpoints for new stacks. The AWS CLI common <code>--region</code> parameter always specifies a regional API endpoint; it cannot be used to specify a classic AWS OpsWorks Stacks region. </note><br/>Name:  The stack name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The Chef version. This parameter must be set to 12, 11.10, or 11.4 for Linux stacks, and to 12.2 for Windows stacks. The default value for Linux stacks is 11.4.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name. This parameter must be set to "Chef".<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BerkshelfVersion</i> The Berkshelf version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManageBerkshelf</i> Whether to enable Berkshelf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Revision</i> The application's version. AWS OpsWorks Stacks enables you to easily deploy new versions of an application. One of the simplest approaches is to have branches or revisions in your repository that represent different versions that can potentially be deployed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SshKey</i> In requests, the repository's SSH key. In responses, AWS OpsWorks Stacks returns <code>*****FILTERED*****</code> instead of the actual value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Password</i> When included in a request, the parameter depends on the repository type. <ul> <li> For Amazon S3 bundles, set <code>Password</code> to the appropriate IAM secret access key. </li> <li> For HTTP bundles and Subversion repositories, set <code>Password</code> to the password. </li> </ul> For more information on how to safely handle IAM credentials, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-access-keys-best-practices.html">https://docs.aws.amazon.com/general/latest/gr/aws-access-keys-best-practices.html</a>. In responses, AWS OpsWorks Stacks returns <code>*****FILTERED*****</code> instead of the actual value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> This parameter depends on the repository type. <ul> <li> For Amazon S3 bundles, set <code>Username</code> to the appropriate IAM access key ID. </li> <li> For HTTP bundles, Git repositories, and Subversion repositories, set <code>Username</code> to the user name. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Url</i> The source URL. The following is an example of an Amazon S3 source URL: <code>https://s3.amazonaws.com/opsworks-demo-bucket/opsworks_cookbook_demo.tar.gz</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The repository type.<br/>
        :return: StackId:  The stack ID, which is an opaque string that you use to identify the stack when performing actions such as <code>DescribeStacks</code>.<br/>        """
        pass

    def create_user_profile(self, IamUserArn: str, SshUsername: str=None, SshPublicKey: str=None, AllowSelfManagement: Boolean=None) -> CreateUserProfileResult:
        """Creates a new user profile.  <b>Required Permissions</b>: To use this action, an IAM user must have an attached policy that explicitly grants permissions. For more information about user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param AllowSelfManagement:  Whether users can specify their own SSH public key through the My Settings page. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/security-settingsshkey.html">Setting an IAM User's Public SSH Key</a>.<br/>SshPublicKey:  The user's public SSH key.<br/>SshUsername:  The user's SSH user name. The allowable characters are [a-z], [A-Z], [0-9], '-', and '_'. If the specified name includes other punctuation marks, AWS OpsWorks Stacks removes them. For example, <code>my.name</code> will be changed to <code>myname</code>. If you do not specify an SSH user name, AWS OpsWorks Stacks generates one from the IAM user name. <br/>IamUserArn:  The user's IAM ARN; this can also be a federated user's ARN.<br/>
        :return: IamUserArn:  The user's IAM ARN.<br/>        """
        pass

    def delete_app(self, AppId: str):
        """Deletes a specified app.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param AppId:  The app ID.<br/>
        :return:         """
        pass

    def delete_instance(self, InstanceId: str, DeleteElasticIp: Boolean=None, DeleteVolumes: Boolean=None):
        """Deletes a specified instance, which terminates the associated Amazon EC2 instance. You must stop an instance before you can delete it. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinginstances-delete.html">Deleting Instances</a>.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param DeleteVolumes:  Whether to delete the instance's Amazon EBS volumes.<br/>DeleteElasticIp:  Whether to delete the instance Elastic IP address.<br/>InstanceId:  The instance ID.<br/>
        :return:         """
        pass

    def delete_layer(self, LayerId: str):
        """Deletes a specified layer. You must first stop and then delete all associated instances or unassign registered instances. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinglayers-basics-delete.html">How to Delete a Layer</a>.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param LayerId:  The layer ID.<br/>
        :return:         """
        pass

    def delete_stack(self, StackId: str):
        """Deletes a specified stack. You must first delete all instances, layers, and apps or deregister registered instances. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingstacks-shutting.html">Shut Down a Stack</a>.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param StackId:  The stack ID.<br/>
        :return:         """
        pass

    def delete_user_profile(self, IamUserArn: str):
        """Deletes a user profile.  <b>Required Permissions</b>: To use this action, an IAM user must have an attached policy that explicitly grants permissions. For more information about user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param IamUserArn:  The user's IAM ARN. This can also be a federated user's ARN.<br/>
        :return:         """
        pass

    def deregister_ecs_cluster(self, EcsClusterArn: str):
        """Deregisters a specified Amazon ECS cluster from a stack. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinglayers-ecscluster.html#workinglayers-ecscluster-delete"> Resource Management</a>.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html</a>.

        :param self:
        :param EcsClusterArn:  The cluster's Amazon Resource Number (ARN).<br/>
        :return:         """
        pass

    def deregister_elastic_ip(self, ElasticIp: str):
        """Deregisters a specified Elastic IP address. The address can then be registered by another stack. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/resources.html">Resource Management</a>.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param ElasticIp:  The Elastic IP address.<br/>
        :return:         """
        pass

    def deregister_instance(self, InstanceId: str):
        """Deregister a registered Amazon EC2 or on-premises instance. This action removes the instance from the stack and returns it to your control. This action cannot be used with instances that were created with AWS OpsWorks Stacks.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param InstanceId:  The instance ID.<br/>
        :return:         """
        pass

    def deregister_rds_db_instance(self, RdsDbInstanceArn: str):
        """Deregisters an Amazon RDS instance.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param RdsDbInstanceArn:  The Amazon RDS instance's ARN.<br/>
        :return:         """
        pass

    def deregister_volume(self, VolumeId: str):
        """Deregisters an Amazon EBS volume. The volume can then be registered by another stack. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/resources.html">Resource Management</a>.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param VolumeId:  The AWS OpsWorks Stacks volume ID, which is the GUID that AWS OpsWorks Stacks assigned to the instance when you registered the volume with the stack, not the Amazon EC2 volume ID.<br/>
        :return:         """
        pass

    def describe_agent_versions(self, StackId: str=None, ConfigurationManager: StackConfigurationManager=None) -> DescribeAgentVersionsResult:
        """Describes the available AWS OpsWorks Stacks agent versions. You must specify a stack ID or a configuration manager. <code>DescribeAgentVersions</code> returns a list of available agent versions for the specified stack or configuration manager.

        :param self:
        :param ConfigurationManager:  The configuration manager.:<br/>StackId:  The stack ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The Chef version. This parameter must be set to 12, 11.10, or 11.4 for Linux stacks, and to 12.2 for Windows stacks. The default value for Linux stacks is 11.4.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name. This parameter must be set to "Chef".<br/>
        :return: AgentVersions:  The agent versions for the specified stack or configuration manager. Note that this value is the complete version number, not the abbreviated number used by the console.<br/>        """
        pass

    def describe_apps(self, StackId: str=None, AppIds: List[String]=None) -> DescribeAppsResult:
        """Requests a description of a specified set of apps. <note> This call accepts only one resource-identifying parameter. </note>  <b>Required Permissions</b>: To use this action, an IAM user must have a Show, Deploy, or Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information about user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param AppIds:  An array of app IDs for the apps to be described. If you use this parameter, <code>DescribeApps</code> returns a description of the specified apps. Otherwise, it returns a description of every app.<br/>StackId:  The app stack ID. If you use this parameter, <code>DescribeApps</code> returns a description of the apps in the specified stack.<br/>
        :return: Apps:  An array of <code>App</code> objects that describe the specified apps. <br/>        """
        pass

    def describe_commands(self, DeploymentId: str=None, InstanceId: str=None, CommandIds: List[String]=None) -> DescribeCommandsResult:
        """Describes the results of specified commands. <note> This call accepts only one resource-identifying parameter. </note>  <b>Required Permissions</b>: To use this action, an IAM user must have a Show, Deploy, or Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information about user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param CommandIds:  An array of command IDs. If you include this parameter, <code>DescribeCommands</code> returns a description of the specified commands. Otherwise, it returns a description of every command.<br/>InstanceId:  The instance ID. If you include this parameter, <code>DescribeCommands</code> returns a description of the commands associated with the specified instance.<br/>DeploymentId:  The deployment ID. If you include this parameter, <code>DescribeCommands</code> returns a description of the commands associated with the specified deployment.<br/>
        :return: Commands:  An array of <code>Command</code> objects that describe each of the specified commands.<br/>        """
        pass

    def describe_deployments(self, StackId: str=None, AppId: str=None, DeploymentIds: List[String]=None) -> DescribeDeploymentsResult:
        """Requests a description of a specified set of deployments. <note> This call accepts only one resource-identifying parameter. </note>  <b>Required Permissions</b>: To use this action, an IAM user must have a Show, Deploy, or Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information about user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param DeploymentIds:  An array of deployment IDs to be described. If you include this parameter, the command returns a description of the specified deployments. Otherwise, it returns a description of every deployment.<br/>AppId:  The app ID. If you include this parameter, the command returns a description of the commands associated with the specified app.<br/>StackId:  The stack ID. If you include this parameter, the command returns a description of the commands associated with the specified stack.<br/>
        :return: Deployments:  An array of <code>Deployment</code> objects that describe the deployments.<br/>        """
        pass

    def describe_ecs_clusters(self, EcsClusterArns: List[String]=None, StackId: str=None, NextToken: str=None, MaxResults: int=None) -> DescribeEcsClustersResult:
        """Describes Amazon ECS clusters that are registered with a stack. If you specify only a stack ID, you can use the <code>MaxResults</code> and <code>NextToken</code> parameters to paginate the response. However, AWS OpsWorks Stacks currently supports only one cluster per layer, so the result set has a maximum of one element.  <b>Required Permissions</b>: To use this action, an IAM user must have a Show, Deploy, or Manage permissions level for the stack or an attached policy that explicitly grants permission. For more information about user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>. This call accepts only one resource-identifying parameter.

        :param self:
        :param MaxResults:  To receive a paginated response, use this parameter to specify the maximum number of results to be returned with a single call. If the number of available results exceeds this maximum, the response includes a <code>NextToken</code> value that you can assign to the <code>NextToken</code> request parameter to get the next set of results.<br/>NextToken:  If the previous paginated request did not return all of the remaining results, the response object's<code>NextToken</code> parameter value is set to a token. To retrieve the next set of results, call <code>DescribeEcsClusters</code> again and assign that token to the request object's <code>NextToken</code> parameter. If there are no remaining results, the previous response object's <code>NextToken</code> parameter is set to <code>null</code>.<br/>StackId:  A stack ID. <code>DescribeEcsClusters</code> returns a description of the cluster that is registered with the stack.<br/>EcsClusterArns:  A list of ARNs, one for each cluster to be described.<br/>
        :return: NextToken:  If a paginated request does not return all of the remaining results, this parameter is set to a token that you can assign to the request object's <code>NextToken</code> parameter to retrieve the next set of results. If the previous paginated request returned all of the remaining results, this parameter is set to <code>null</code>.<br/>EcsClusters:  A list of <code>EcsCluster</code> objects containing the cluster descriptions.<br/>        """
        pass

    def describe_elastic_ips(self, InstanceId: str=None, StackId: str=None, Ips: List[String]=None) -> DescribeElasticIpsResult:
        """Describes <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/elastic-ip-addresses-eip.html">Elastic IP addresses</a>. <note> This call accepts only one resource-identifying parameter. </note>  <b>Required Permissions</b>: To use this action, an IAM user must have a Show, Deploy, or Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information about user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param Ips:  An array of Elastic IP addresses to be described. If you include this parameter, <code>DescribeElasticIps</code> returns a description of the specified Elastic IP addresses. Otherwise, it returns a description of every Elastic IP address.<br/>StackId:  A stack ID. If you include this parameter, <code>DescribeElasticIps</code> returns a description of the Elastic IP addresses that are registered with the specified stack.<br/>InstanceId:  The instance ID. If you include this parameter, <code>DescribeElasticIps</code> returns a description of the Elastic IP addresses associated with the specified instance.<br/>
        :return: ElasticIps:  An <code>ElasticIps</code> object that describes the specified Elastic IP addresses.<br/>        """
        pass

    def describe_elastic_load_balancers(self, StackId: str=None, LayerIds: List[String]=None) -> DescribeElasticLoadBalancersResult:
        """Describes a stack's Elastic Load Balancing instances. <note> This call accepts only one resource-identifying parameter. </note>  <b>Required Permissions</b>: To use this action, an IAM user must have a Show, Deploy, or Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information about user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param LayerIds:  A list of layer IDs. The action describes the Elastic Load Balancing instances for the specified layers.<br/>StackId:  A stack ID. The action describes the stack's Elastic Load Balancing instances.<br/>
        :return: ElasticLoadBalancers:  A list of <code>ElasticLoadBalancer</code> objects that describe the specified Elastic Load Balancing instances.<br/>        """
        pass

    def describe_instances(self, StackId: str=None, LayerId: str=None, InstanceIds: List[String]=None) -> DescribeInstancesResult:
        """Requests a description of a set of instances. <note> This call accepts only one resource-identifying parameter. </note>  <b>Required Permissions</b>: To use this action, an IAM user must have a Show, Deploy, or Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information about user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param InstanceIds:  An array of instance IDs to be described. If you use this parameter, <code>DescribeInstances</code> returns a description of the specified instances. Otherwise, it returns a description of every instance.<br/>LayerId:  A layer ID. If you use this parameter, <code>DescribeInstances</code> returns descriptions of the instances associated with the specified layer.<br/>StackId:  A stack ID. If you use this parameter, <code>DescribeInstances</code> returns descriptions of the instances associated with the specified stack.<br/>
        :return: Instances:  An array of <code>Instance</code> objects that describe the instances.<br/>        """
        pass

    def describe_layers(self, StackId: str=None, LayerIds: List[String]=None) -> DescribeLayersResult:
        """Requests a description of one or more layers in a specified stack. <note> This call accepts only one resource-identifying parameter. </note>  <b>Required Permissions</b>: To use this action, an IAM user must have a Show, Deploy, or Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information about user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param LayerIds:  An array of layer IDs that specify the layers to be described. If you omit this parameter, <code>DescribeLayers</code> returns a description of every layer in the specified stack.<br/>StackId:  The stack ID.<br/>
        :return: Layers:  An array of <code>Layer</code> objects that describe the layers.<br/>        """
        pass

    def describe_load_based_auto_scaling(self, LayerIds: List[String]) -> DescribeLoadBasedAutoScalingResult:
        """Describes load-based auto scaling configurations for specified layers. <note> You must specify at least one of the parameters. </note>  <b>Required Permissions</b>: To use this action, an IAM user must have a Show, Deploy, or Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information about user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param LayerIds:  An array of layer IDs.<br/>
        :return: LoadBasedAutoScalingConfigurations:  An array of <code>LoadBasedAutoScalingConfiguration</code> objects that describe each layer's configuration.<br/>        """
        pass

    def describe_my_user_profile(self) -> DescribeMyUserProfileResult:
        """Describes a user's SSH information.  <b>Required Permissions</b>: To use this action, an IAM user must have self-management enabled or an attached policy that explicitly grants permissions. For more information about user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param 
        :return: UserProfile:  A <code>UserProfile</code> object that describes the user's SSH information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SshPublicKey</i> The user's SSH public key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SshUsername</i> The user's SSH user name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The user's name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamUserArn</i> The user's IAM ARN.<br/>        """
        pass

    def describe_operating_systems(self) -> DescribeOperatingSystemsResponse:
        """Describes the operating systems that are supported by AWS OpsWorks Stacks.

        :param self:
        :param 
        :return: OperatingSystems:  Contains information in response to a <code>DescribeOperatingSystems</code> request.<br/>        """
        pass

    def describe_permissions(self, IamUserArn: str=None, StackId: str=None) -> DescribePermissionsResult:
        """Describes the permissions for a specified stack.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param StackId:  The stack ID.<br/>IamUserArn:  The user's IAM ARN. This can also be a federated user's ARN. For more information about IAM ARNs, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">Using Identifiers</a>.<br/>
        :return: Permissions:  An array of <code>Permission</code> objects that describe the stack permissions. <ul> <li> If the request object contains only a stack ID, the array contains a <code>Permission</code> object with permissions for each of the stack IAM ARNs. </li> <li> If the request object contains only an IAM ARN, the array contains a <code>Permission</code> object with permissions for each of the user's stack IDs. </li> <li> If the request contains a stack ID and an IAM ARN, the array contains a single <code>Permission</code> object with permissions for the specified stack and IAM ARN. </li> </ul><br/>        """
        pass

    def describe_raid_arrays(self, InstanceId: str=None, StackId: str=None, RaidArrayIds: List[String]=None) -> DescribeRaidArraysResult:
        """Describe an instance's RAID arrays. <note> This call accepts only one resource-identifying parameter. </note>  <b>Required Permissions</b>: To use this action, an IAM user must have a Show, Deploy, or Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information about user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param RaidArrayIds:  An array of RAID array IDs. If you use this parameter, <code>DescribeRaidArrays</code> returns descriptions of the specified arrays. Otherwise, it returns a description of every array.<br/>StackId:  The stack ID.<br/>InstanceId:  The instance ID. If you use this parameter, <code>DescribeRaidArrays</code> returns descriptions of the RAID arrays associated with the specified instance. <br/>
        :return: RaidArrays:  A <code>RaidArrays</code> object that describes the specified RAID arrays.<br/>        """
        pass

    def describe_rds_db_instances(self, StackId: str, RdsDbInstanceArns: List[String]=None) -> DescribeRdsDbInstancesResult:
        """Describes Amazon RDS instances.  <b>Required Permissions</b>: To use this action, an IAM user must have a Show, Deploy, or Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information about user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>. This call accepts only one resource-identifying parameter.

        :param self:
        :param RdsDbInstanceArns:  An array containing the ARNs of the instances to be described.<br/>StackId:  The ID of the stack with which the instances are registered. The operation returns descriptions of all registered Amazon RDS instances.<br/>
        :return: RdsDbInstances:  An a array of <code>RdsDbInstance</code> objects that describe the instances.<br/>        """
        pass

    def describe_service_errors(self, StackId: str=None, InstanceId: str=None, ServiceErrorIds: List[String]=None) -> DescribeServiceErrorsResult:
        """Describes AWS OpsWorks Stacks service errors.  <b>Required Permissions</b>: To use this action, an IAM user must have a Show, Deploy, or Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information about user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>. This call accepts only one resource-identifying parameter.

        :param self:
        :param ServiceErrorIds:  An array of service error IDs. If you use this parameter, <code>DescribeServiceErrors</code> returns descriptions of the specified errors. Otherwise, it returns a description of every error.<br/>InstanceId:  The instance ID. If you use this parameter, <code>DescribeServiceErrors</code> returns descriptions of the errors associated with the specified instance.<br/>StackId:  The stack ID. If you use this parameter, <code>DescribeServiceErrors</code> returns descriptions of the errors associated with the specified stack.<br/>
        :return: ServiceErrors:  An array of <code>ServiceError</code> objects that describe the specified service errors.<br/>        """
        pass

    def describe_stack_provisioning_parameters(self, StackId: str) -> DescribeStackProvisioningParametersResult:
        """Requests a description of a stack's provisioning parameters.  <b>Required Permissions</b>: To use this action, an IAM user must have a Show, Deploy, or Manage permissions level for the stack or an attached policy that explicitly grants permissions. For more information about user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param StackId:  The stack ID.<br/>
        :return: Parameters:  An embedded object that contains the provisioning parameters.<br/>AgentInstallerUrl:  The AWS OpsWorks Stacks agent installer's URL.<br/>        """
        pass

    def describe_stack_summary(self, StackId: str) -> DescribeStackSummaryResult:
        """Describes the number of layers and apps in a specified stack, and the number of instances in each state, such as <code>running_setup</code> or <code>online</code>.  <b>Required Permissions</b>: To use this action, an IAM user must have a Show, Deploy, or Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information about user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param StackId:  The stack ID.<br/>
        :return: StackSummary:  A <code>StackSummary</code> object that contains the results.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstancesCount</i> An <code>InstancesCount</code> object with the number of instances in each status.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InstancesCount</b>: An <code>InstancesCount</code> object with the number of instances in each status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AppsCount</i> The number of apps.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LayersCount</i> The number of layers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The stack's ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The stack name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StackId</i> The stack ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Unassigning</i> The number of instances in the Unassigning state.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Terminating</i> The number of instances with <code>terminating</code> status.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Terminated</i> The number of instances with <code>terminated</code> status.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Stopping</i> The number of instances with <code>stopping</code> status.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Stopped</i> The number of instances with <code>stopped</code> status.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StopFailed</i> The number of instances with <code>stop_failed</code> status.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StartFailed</i> The number of instances with <code>start_failed</code> status.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ShuttingDown</i> The number of instances with <code>shutting_down</code> status.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SetupFailed</i> The number of instances with <code>setup_failed</code> status.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RunningSetup</i> The number of instances with <code>running_setup</code> status.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Requested</i> The number of instances with <code>requested</code> status.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Registering</i> The number of instances in the Registering state.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Registered</i> The number of instances in the Registered state.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Rebooting</i> The number of instances with <code>rebooting</code> status.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Pending</i> The number of instances with <code>pending</code> status.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Online</i> The number of instances with <code>online</code> status.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Deregistering</i> The number of instances in the Deregistering state.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ConnectionLost</i> The number of instances with <code>connection_lost</code> status.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Booting</i> The number of instances with <code>booting</code> status.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Assigning</i> The number of instances in the Assigning state.<br/>        """
        pass

    def describe_stacks(self, StackIds: List[String]=None) -> DescribeStacksResult:
        """Requests a description of one or more stacks.  <b>Required Permissions</b>: To use this action, an IAM user must have a Show, Deploy, or Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information about user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param StackIds:  An array of stack IDs that specify the stacks to be described. If you omit this parameter, <code>DescribeStacks</code> returns a description of every stack.<br/>
        :return: Stacks:  An array of <code>Stack</code> objects that describe the stacks.<br/>        """
        pass

    def describe_time_based_auto_scaling(self, InstanceIds: List[String]) -> DescribeTimeBasedAutoScalingResult:
        """Describes time-based auto scaling configurations for specified instances. <note> You must specify at least one of the parameters. </note>  <b>Required Permissions</b>: To use this action, an IAM user must have a Show, Deploy, or Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information about user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param InstanceIds:  An array of instance IDs.<br/>
        :return: TimeBasedAutoScalingConfigurations:  An array of <code>TimeBasedAutoScalingConfiguration</code> objects that describe the configuration for the specified instances.<br/>        """
        pass

    def describe_user_profiles(self, IamUserArns: List[String]=None) -> DescribeUserProfilesResult:
        """Describe specified users.  <b>Required Permissions</b>: To use this action, an IAM user must have an attached policy that explicitly grants permissions. For more information about user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param IamUserArns:  An array of IAM or federated user ARNs that identify the users to be described.<br/>
        :return: UserProfiles:  A <code>Users</code> object that describes the specified users.<br/>        """
        pass

    def describe_volumes(self, InstanceId: str=None, StackId: str=None, RaidArrayId: str=None, VolumeIds: List[String]=None) -> DescribeVolumesResult:
        """Describes an instance's Amazon EBS volumes. <note> This call accepts only one resource-identifying parameter. </note>  <b>Required Permissions</b>: To use this action, an IAM user must have a Show, Deploy, or Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information about user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param VolumeIds:  Am array of volume IDs. If you use this parameter, <code>DescribeVolumes</code> returns descriptions of the specified volumes. Otherwise, it returns a description of every volume.<br/>RaidArrayId:  The RAID array ID. If you use this parameter, <code>DescribeVolumes</code> returns descriptions of the volumes associated with the specified RAID array.<br/>StackId:  A stack ID. The action describes the stack's registered Amazon EBS volumes.<br/>InstanceId:  The instance ID. If you use this parameter, <code>DescribeVolumes</code> returns descriptions of the volumes associated with the specified instance.<br/>
        :return: Volumes:  An array of volume IDs.<br/>        """
        pass

    def detach_elastic_load_balancer(self, LayerId: str, ElasticLoadBalancerName: str):
        """Detaches a specified Elastic Load Balancing instance from its layer.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param LayerId:  The ID of the layer that the Elastic Load Balancing instance is attached to.<br/>ElasticLoadBalancerName:  The Elastic Load Balancing instance's name.<br/>
        :return:         """
        pass

    def disassociate_elastic_ip(self, ElasticIp: str):
        """Disassociates an Elastic IP address from its instance. The address remains registered with the stack. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/resources.html">Resource Management</a>.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param ElasticIp:  The Elastic IP address.<br/>
        :return:         """
        pass

    def get_hostname_suggestion(self, LayerId: str) -> GetHostnameSuggestionResult:
        """Gets a generated host name for the specified layer, based on the current host name theme.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param LayerId:  The layer ID.<br/>
        :return: Hostname:  The generated host name.<br/>LayerId:  The layer ID.<br/>        """
        pass

    def grant_access(self, InstanceId: str, ValidForInMinutes: int=None) -> GrantAccessResult:
        """<note> This action can be used only with Windows stacks. </note> Grants RDP access to a Windows instance for a specified time period.

        :param self:
        :param ValidForInMinutes:  The length of time (in minutes) that the grant is valid. When the grant expires at the end of this period, the user will no longer be able to use the credentials to log in. If the user is logged in at the time, he or she automatically will be logged out.<br/>InstanceId:  The instance's AWS OpsWorks Stacks ID.<br/>
        :return: TemporaryCredential:  A <code>TemporaryCredential</code> object that contains the data needed to log in to the instance by RDP clients, such as the Microsoft Remote Desktop Connection.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceId</i> The instance's AWS OpsWorks Stacks ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ValidForInMinutes</i> The length of time (in minutes) that the grant is valid. When the grant expires, at the end of this period, the user will no longer be able to use the credentials to log in. If they are logged in at the time, they will be automatically logged out.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Password</i> The password.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> The user name.<br/>        """
        pass

    def list_tags(self, ResourceArn: str, MaxResults: int=None, NextToken: str=None) -> ListTagsResult:
        """Returns a list of tags that are applied to the specified stack or layer.

        :param self:
        :param NextToken:  Do not use. A validation exception occurs if you add a <code>NextToken</code> parameter to a <code>ListTagsRequest</code> call. <br/>MaxResults:  Do not use. A validation exception occurs if you add a <code>MaxResults</code> parameter to a <code>ListTagsRequest</code> call. <br/>ResourceArn:  The stack or layer's Amazon Resource Number (ARN).<br/>
        :return: NextToken:  If a paginated request does not return all of the remaining results, this parameter is set to a token that you can assign to the request object's <code>NextToken</code> parameter to get the next set of results. If the previous paginated request returned all of the remaining results, this parameter is set to <code>null</code>. <br/>Tags:  A set of key-value pairs that contain tag keys and tag values that are attached to a stack or layer.<br/>        """
        pass

    def reboot_instance(self, InstanceId: str):
        """Reboots a specified instance. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinginstances-starting.html">Starting, Stopping, and Rebooting Instances</a>.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param InstanceId:  The instance ID.<br/>
        :return:         """
        pass

    def register_ecs_cluster(self, StackId: str, EcsClusterArn: str) -> RegisterEcsClusterResult:
        """Registers a specified Amazon ECS cluster with a stack. You can register only one cluster with a stack. A cluster can be registered with only one stack. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinglayers-ecscluster.html"> Resource Management</a>.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html"> Managing User Permissions</a>.

        :param self:
        :param StackId:  The stack ID.<br/>EcsClusterArn:  The cluster's ARN.<br/>
        :return: EcsClusterArn:  The cluster's ARN.<br/>        """
        pass

    def register_elastic_ip(self, StackId: str, ElasticIp: str) -> RegisterElasticIpResult:
        """Registers an Elastic IP address with a specified stack. An address can be registered with only one stack at a time. If the address is already registered, you must first deregister it by calling <a>DeregisterElasticIp</a>. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/resources.html">Resource Management</a>.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param StackId:  The stack ID.<br/>ElasticIp:  The Elastic IP address.<br/>
        :return: ElasticIp:  The Elastic IP address.<br/>        """
        pass

    def register_instance(self, StackId: str, Hostname: str=None, PublicIp: str=None, PrivateIp: str=None, RsaPublicKey: str=None, RsaPublicKeyFingerprint: str=None, InstanceIdentity: InstanceIdentity=None) -> RegisterInstanceResult:
        """Registers instances that were created outside of AWS OpsWorks Stacks with a specified stack. <note> We do not recommend using this action to register instances. The complete registration operation includes two tasks: installing the AWS OpsWorks Stacks agent on the instance, and registering the instance with the stack. <code>RegisterInstance</code> handles only the second step. You should instead use the AWS CLI <code>register</code> command, which performs the entire registration operation. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/registered-instances-register.html"> Registering an Instance with an AWS OpsWorks Stacks Stack</a>. </note> Registered instances have the same requirements as instances that are created by using the <a>CreateInstance</a> API. For example, registered instances must be running a supported Linux-based operating system, and they must have a supported instance type. For more information about requirements for instances that you want to register, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/registered-instances-register-registering-preparer.html"> Preparing the Instance</a>.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param InstanceIdentity:  An InstanceIdentity object that contains the instance's identity.:<br/>RsaPublicKeyFingerprint:  The instances public RSA key fingerprint.<br/>RsaPublicKey:  The instances public RSA key. This key is used to encrypt communication between the instance and the service.<br/>PrivateIp:  The instance's private IP address.<br/>PublicIp:  The instance's public IP address.<br/>Hostname:  The instance's hostname.<br/>StackId:  The ID of the stack that the instance is to be registered with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Signature</i> A signature that can be used to verify the document's accuracy and authenticity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Document</i> A JSON document that contains the metadata.<br/>
        :return: InstanceId:  The registered instance's AWS OpsWorks Stacks ID.<br/>        """
        pass

    def register_rds_db_instance(self, DbPassword: str, DbUser: str, RdsDbInstanceArn: str, StackId: str):
        """Registers an Amazon RDS instance with a stack.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param DbPassword:  The database password.<br/>DbUser:  The database's master user name.<br/>RdsDbInstanceArn:  The Amazon RDS instance's ARN.<br/>StackId:  The stack ID.<br/>
        :return:         """
        pass

    def register_volume(self, StackId: str, Ec2VolumeId: str=None) -> RegisterVolumeResult:
        """Registers an Amazon EBS volume with a specified stack. A volume can be registered with only one stack at a time. If the volume is already registered, you must first deregister it by calling <a>DeregisterVolume</a>. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/resources.html">Resource Management</a>.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param StackId:  The stack ID.<br/>Ec2VolumeId:  The Amazon EBS volume ID.<br/>
        :return: VolumeId:  The volume ID.<br/>        """
        pass

    def set_load_based_auto_scaling(self, LayerId: str, Enable: Boolean=None, UpScaling: AutoScalingThresholds=None, DownScaling: AutoScalingThresholds=None):
        """Specify the load-based auto scaling configuration for a specified layer. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinginstances-autoscaling.html">Managing Load with Time-based and Load-based Instances</a>. <note> To use load-based auto scaling, you must create a set of load-based auto scaling instances. Load-based auto scaling operates only on the instances from that set, so you must ensure that you have created enough instances to handle the maximum anticipated load. </note>  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param DownScaling:  An <code>AutoScalingThresholds</code> object with the downscaling threshold configuration. If the load falls below these thresholds for a specified amount of time, AWS OpsWorks Stacks stops a specified number of instances.:<br/>UpScaling:  An <code>AutoScalingThresholds</code> object with the upscaling threshold configuration. If the load exceeds these thresholds for a specified amount of time, AWS OpsWorks Stacks starts a specified number of instances.:<br/>Enable:  Enables load-based auto scaling for the layer.<br/>LayerId:  The layer ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Alarms</i> Custom Cloudwatch auto scaling alarms, to be used as thresholds. This parameter takes a list of up to five alarm names, which are case sensitive and must be in the same region as the stack. <note> To use custom alarms, you must update your service role to allow <code>cloudwatch:DescribeAlarms</code>. You can either have AWS OpsWorks Stacks update the role for you when you first use this feature or you can edit the role manually. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-servicerole.html">Allowing AWS OpsWorks Stacks to Act on Your Behalf</a>. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LoadThreshold</i> The load threshold. A value of -1 disables the threshold. For more information about how load is computed, see <a href="http://en.wikipedia.org/wiki/Load_%28computing%29">Load (computing)</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MemoryThreshold</i> The memory utilization threshold, as a percent of the available memory. A value of -1 disables the threshold.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CpuThreshold</i> The CPU utilization threshold, as a percent of the available CPU. A value of -1 disables the threshold.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IgnoreMetricsTime</i> The amount of time (in minutes) after a scaling event occurs that AWS OpsWorks Stacks should ignore metrics and suppress additional scaling events. For example, AWS OpsWorks Stacks adds new instances following an upscaling event but the instances won't start reducing the load until they have been booted and configured. There is no point in raising additional scaling events during that operation, which typically takes several minutes. <code>IgnoreMetricsTime</code> allows you to direct AWS OpsWorks Stacks to suppress scaling events long enough to get the new instances online.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThresholdsWaitTime</i> The amount of time, in minutes, that the load must exceed a threshold before more instances are added or removed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCount</i> The number of instances to add or remove when the load exceeds a threshold.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Alarms</i> Custom Cloudwatch auto scaling alarms, to be used as thresholds. This parameter takes a list of up to five alarm names, which are case sensitive and must be in the same region as the stack. <note> To use custom alarms, you must update your service role to allow <code>cloudwatch:DescribeAlarms</code>. You can either have AWS OpsWorks Stacks update the role for you when you first use this feature or you can edit the role manually. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-servicerole.html">Allowing AWS OpsWorks Stacks to Act on Your Behalf</a>. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LoadThreshold</i> The load threshold. A value of -1 disables the threshold. For more information about how load is computed, see <a href="http://en.wikipedia.org/wiki/Load_%28computing%29">Load (computing)</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MemoryThreshold</i> The memory utilization threshold, as a percent of the available memory. A value of -1 disables the threshold.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CpuThreshold</i> The CPU utilization threshold, as a percent of the available CPU. A value of -1 disables the threshold.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IgnoreMetricsTime</i> The amount of time (in minutes) after a scaling event occurs that AWS OpsWorks Stacks should ignore metrics and suppress additional scaling events. For example, AWS OpsWorks Stacks adds new instances following an upscaling event but the instances won't start reducing the load until they have been booted and configured. There is no point in raising additional scaling events during that operation, which typically takes several minutes. <code>IgnoreMetricsTime</code> allows you to direct AWS OpsWorks Stacks to suppress scaling events long enough to get the new instances online.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThresholdsWaitTime</i> The amount of time, in minutes, that the load must exceed a threshold before more instances are added or removed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCount</i> The number of instances to add or remove when the load exceeds a threshold.<br/>
        :return:         """
        pass

    def set_permission(self, IamUserArn: str, StackId: str, AllowSsh: Boolean=None, AllowSudo: Boolean=None, Level: str=None):
        """Specifies a user's permissions. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingsecurity.html">Security and Permissions</a>.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param Level:  The user's permission level, which must be set to one of the following strings. You cannot set your own permissions level. <ul> <li>  <code>deny</code>  </li> <li>  <code>show</code>  </li> <li>  <code>deploy</code>  </li> <li>  <code>manage</code>  </li> <li>  <code>iam_only</code>  </li> </ul> For more information about the permissions associated with these levels, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.<br/>AllowSudo:  The user is allowed to use <b>sudo</b> to elevate privileges.<br/>AllowSsh:  The user is allowed to use SSH to communicate with the instance.<br/>IamUserArn:  The user's IAM ARN. This can also be a federated user's ARN.<br/>StackId:  The stack ID.<br/>
        :return:         """
        pass

    def set_time_based_auto_scaling(self, InstanceId: str, AutoScalingSchedule: WeeklyAutoScalingSchedule=None):
        """Specify the time-based auto scaling configuration for a specified instance. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinginstances-autoscaling.html">Managing Load with Time-based and Load-based Instances</a>.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param AutoScalingSchedule:  An <code>AutoScalingSchedule</code> with the instance schedule.:<br/>InstanceId:  The instance ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Sunday</i> The schedule for Sunday.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Saturday</i> The schedule for Saturday.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Friday</i> The schedule for Friday.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Thursday</i> The schedule for Thursday.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Wednesday</i> The schedule for Wednesday.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tuesday</i> The schedule for Tuesday.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Monday</i> The schedule for Monday.<br/>
        :return:         """
        pass

    def start_instance(self, InstanceId: str):
        """Starts a specified instance. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinginstances-starting.html">Starting, Stopping, and Rebooting Instances</a>.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param InstanceId:  The instance ID.<br/>
        :return:         """
        pass

    def start_stack(self, StackId: str):
        """Starts a stack's instances.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param StackId:  The stack ID.<br/>
        :return:         """
        pass

    def stop_instance(self, InstanceId: str, Force: Boolean=None):
        """Stops a specified instance. When you stop a standard instance, the data disappears and must be reinstalled when you restart the instance. You can stop an Amazon EBS-backed instance without losing data. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinginstances-starting.html">Starting, Stopping, and Rebooting Instances</a>.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param Force:  Specifies whether to force an instance to stop. If the instance's root device type is <code>ebs</code>, or EBS-backed, adding the <code>Force</code> parameter to the <code>StopInstances</code> API call disassociates the AWS OpsWorks Stacks instance from EC2, and forces deletion of <i>only</i> the OpsWorks Stacks instance. You must also delete the formerly-associated instance in EC2 after troubleshooting and replacing the AWS OpsWorks Stacks instance with a new one.<br/>InstanceId:  The instance ID.<br/>
        :return:         """
        pass

    def stop_stack(self, StackId: str):
        """Stops a specified stack.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param StackId:  The stack ID.<br/>
        :return:         """
        pass

    def tag_resource(self, Tags: Tags, ResourceArn: str):
        """Apply cost-allocation tags to a specified stack or layer in AWS OpsWorks Stacks. For more information about how tagging works, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/tagging.html">Tags</a> in the AWS OpsWorks User Guide.

        :param self:
        :param Tags:  A map that contains tag keys and tag values that are attached to a stack or layer. <ul> <li> The key cannot be empty. </li> <li> The key can be a maximum of 127 characters, and can contain only Unicode letters, numbers, or separators, or the following special characters: <code>+ - = . _ : /</code>  </li> <li> The value can be a maximum 255 characters, and contain only Unicode letters, numbers, or separators, or the following special characters: <code>+ - = . _ : /</code>  </li> <li> Leading and trailing white spaces are trimmed from both the key and value. </li> <li> A maximum of 40 tags is allowed for any resource. </li> </ul><br/>ResourceArn:  The stack or layer's Amazon Resource Number (ARN).<br/>
        :return:         """
        pass

    def unassign_instance(self, InstanceId: str):
        """Unassigns a registered instance from all layers that are using the instance. The instance remains in the stack as an unassigned instance, and can be assigned to another layer as needed. You cannot use this action with instances that were created with AWS OpsWorks Stacks.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack or an attached policy that explicitly grants permissions. For more information about user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param InstanceId:  The instance ID.<br/>
        :return:         """
        pass

    def unassign_volume(self, VolumeId: str):
        """Unassigns an assigned Amazon EBS volume. The volume remains registered with the stack. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/resources.html">Resource Management</a>.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param VolumeId:  The volume ID.<br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[TagKey], ResourceArn: str):
        """Removes tags from a specified stack or layer.

        :param self:
        :param TagKeys:  A list of the keys of tags to be removed from a stack or layer.<br/>ResourceArn:  The stack or layer's Amazon Resource Number (ARN).<br/>
        :return:         """
        pass

    def update_app(self, AppId: str, Name: str=None, Description: str=None, DataSources: List[DataSource]=None, Type: str=None, AppSource: Source=None, Domains: List[String]=None, EnableSsl: Boolean=None, SslConfiguration: SslConfiguration=None, Attributes: AppAttributes=None, Environment: List[EnvironmentVariable]=None):
        """Updates a specified app.  <b>Required Permissions</b>: To use this action, an IAM user must have a Deploy or Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param Environment:  An array of <code>EnvironmentVariable</code> objects that specify environment variables to be associated with the app. After you deploy the app, these variables are defined on the associated app server instances.For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingapps-creating.html#workingapps-creating-environment"> Environment Variables</a>. There is no specific limit on the number of environment variables. However, the size of the associated data structure - which includes the variables' names, values, and protected flag values - cannot exceed 20 KB. This limit should accommodate most if not all use cases. Exceeding it will cause an exception with the message, "Environment: is too large (maximum is 20 KB)." <note> If you have specified one or more environment variables, you cannot modify the stack's Chef version. </note><br/>Attributes:  One or more user-defined key/value pairs to be added to the stack attributes.<br/>SslConfiguration:  An <code>SslConfiguration</code> object with the SSL configuration.:<br/>EnableSsl:  Whether SSL is enabled for the app.<br/>Domains:  The app's virtual host settings, with multiple domains separated by commas. For example: <code>'www.example.com, example.com'</code> <br/>AppSource:  A <code>Source</code> object that specifies the app repository.:<br/>Type:  The app type.<br/>DataSources:  The app's data sources.<br/>Description:  A description of the app.<br/>Name:  The app name.<br/>AppId:  The app ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Revision</i> The application's version. AWS OpsWorks Stacks enables you to easily deploy new versions of an application. One of the simplest approaches is to have branches or revisions in your repository that represent different versions that can potentially be deployed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SshKey</i> In requests, the repository's SSH key. In responses, AWS OpsWorks Stacks returns <code>*****FILTERED*****</code> instead of the actual value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Password</i> When included in a request, the parameter depends on the repository type. <ul> <li> For Amazon S3 bundles, set <code>Password</code> to the appropriate IAM secret access key. </li> <li> For HTTP bundles and Subversion repositories, set <code>Password</code> to the password. </li> </ul> For more information on how to safely handle IAM credentials, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-access-keys-best-practices.html">https://docs.aws.amazon.com/general/latest/gr/aws-access-keys-best-practices.html</a>. In responses, AWS OpsWorks Stacks returns <code>*****FILTERED*****</code> instead of the actual value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> This parameter depends on the repository type. <ul> <li> For Amazon S3 bundles, set <code>Username</code> to the appropriate IAM access key ID. </li> <li> For HTTP bundles, Git repositories, and Subversion repositories, set <code>Username</code> to the user name. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Url</i> The source URL. The following is an example of an Amazon S3 source URL: <code>https://s3.amazonaws.com/opsworks-demo-bucket/opsworks_cookbook_demo.tar.gz</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The repository type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Chain</i> Optional. Can be used to specify an intermediate certificate authority key or client authentication.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrivateKey</i> The private key; the contents of the certificate's domain.kex file.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Certificate</i> The contents of the certificate's domain.crt file.<br/>
        :return:         """
        pass

    def update_elastic_ip(self, ElasticIp: str, Name: str=None):
        """Updates a registered Elastic IP address's name. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/resources.html">Resource Management</a>.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param Name:  The new name.<br/>ElasticIp:  The IP address for which you want to update the name.<br/>
        :return:         """
        pass

    def update_instance(self, InstanceId: str, LayerIds: List[String]=None, InstanceType: str=None, AutoScalingType: str=None, Hostname: str=None, Os: str=None, AmiId: str=None, SshKeyName: str=None, Architecture: str=None, InstallUpdatesOnBoot: Boolean=None, EbsOptimized: Boolean=None, AgentVersion: str=None):
        """Updates a specified instance.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param AgentVersion:  The default AWS OpsWorks Stacks agent version. You have the following options: <ul> <li>  <code>INHERIT</code> - Use the stack's default agent version setting. </li> <li>  <i>version_number</i> - Use the specified agent version. This value overrides the stack's default setting. To update the agent version, you must edit the instance configuration and specify a new version. AWS OpsWorks Stacks then automatically installs that version on the instance. </li> </ul> The default setting is <code>INHERIT</code>. To specify an agent version, you must use the complete version number, not the abbreviated number shown on the console. For a list of available agent version numbers, call <a>DescribeAgentVersions</a>. AgentVersion cannot be set to Chef 12.2.<br/>EbsOptimized:  This property cannot be updated.<br/>InstallUpdatesOnBoot:  Whether to install operating system and package updates when the instance boots. The default value is <code>true</code>. To control when updates are installed, set this value to <code>false</code>. You must then update your instances manually by using <a>CreateDeployment</a> to run the <code>update_dependencies</code> stack command or by manually running <code>yum</code> (Amazon Linux) or <code>apt-get</code> (Ubuntu) on the instances.  <note> We strongly recommend using the default value of <code>true</code>, to ensure that your instances have the latest security updates. </note><br/>Architecture:  The instance architecture. Instance types do not necessarily support both architectures. For a list of the architectures that are supported by the different instance types, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html">Instance Families and Types</a>.<br/>SshKeyName:  The instance's Amazon EC2 key name.<br/>AmiId:  The ID of the AMI that was used to create the instance. The value of this parameter must be the same AMI ID that the instance is already using. You cannot apply a new AMI to an instance by running UpdateInstance. UpdateInstance does not work on instances that are using custom AMIs. <br/>Os:  The instance's operating system, which must be set to one of the following. You cannot update an instance that is using a custom AMI. <ul> <li> A supported Linux operating system: An Amazon Linux version, such as <code>Amazon Linux 2018.03</code>, <code>Amazon Linux 2017.09</code>, <code>Amazon Linux 2017.03</code>, <code>Amazon Linux 2016.09</code>, <code>Amazon Linux 2016.03</code>, <code>Amazon Linux 2015.09</code>, or <code>Amazon Linux 2015.03</code>. </li> <li> A supported Ubuntu operating system, such as <code>Ubuntu 16.04 LTS</code>, <code>Ubuntu 14.04 LTS</code>, or <code>Ubuntu 12.04 LTS</code>. </li> <li>  <code>CentOS Linux 7</code>  </li> <li>  <code>Red Hat Enterprise Linux 7</code>  </li> <li> A supported Windows operating system, such as <code>Microsoft Windows Server 2012 R2 Base</code>, <code>Microsoft Windows Server 2012 R2 with SQL Server Express</code>, <code>Microsoft Windows Server 2012 R2 with SQL Server Standard</code>, or <code>Microsoft Windows Server 2012 R2 with SQL Server Web</code>. </li> </ul> For more information about supported operating systems, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinginstances-os.html">AWS OpsWorks Stacks Operating Systems</a>. The default option is the current Amazon Linux version. If you set this parameter to <code>Custom</code>, you must use the AmiId parameter to specify the custom AMI that you want to use. For more information about supported operating systems, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinginstances-os.html">Operating Systems</a>. For more information about how to use custom AMIs with OpsWorks, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinginstances-custom-ami.html">Using Custom AMIs</a>. <note> You can specify a different Linux operating system for the updated stack, but you cannot change from Linux to Windows or Windows to Linux. </note><br/>Hostname:  The instance host name.<br/>AutoScalingType:  For load-based or time-based instances, the type. Windows stacks can use only time-based instances.<br/>InstanceType:  The instance type, such as <code>t2.micro</code>. For a list of supported instance types, open the stack in the console, choose <b>Instances</b>, and choose <b>+ Instance</b>. The <b>Size</b> list contains the currently supported types. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html">Instance Families and Types</a>. The parameter values that you use to specify the various types are in the <b>API Name</b> column of the <b>Available Instance Types</b> table.<br/>LayerIds:  The instance's layer IDs.<br/>InstanceId:  The instance ID.<br/>
        :return:         """
        pass

    def update_layer(self, LayerId: str, Name: str=None, Shortname: str=None, Attributes: LayerAttributes=None, CloudWatchLogsConfiguration: CloudWatchLogsConfiguration=None, CustomInstanceProfileArn: str=None, CustomJson: str=None, CustomSecurityGroupIds: List[String]=None, Packages: List[String]=None, VolumeConfigurations: List[VolumeConfiguration]=None, EnableAutoHealing: Boolean=None, AutoAssignElasticIps: Boolean=None, AutoAssignPublicIps: Boolean=None, CustomRecipes: Recipes=None, InstallUpdatesOnBoot: Boolean=None, UseEbsOptimizedInstances: Boolean=None, LifecycleEventConfiguration: LifecycleEventConfiguration=None):
        """Updates a specified layer.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param LifecycleEventConfiguration:  <p/>:<br/>UseEbsOptimizedInstances:  Whether to use Amazon EBS-optimized instances.<br/>InstallUpdatesOnBoot:  Whether to install operating system and package updates when the instance boots. The default value is <code>true</code>. To control when updates are installed, set this value to <code>false</code>. You must then update your instances manually by using <a>CreateDeployment</a> to run the <code>update_dependencies</code> stack command or manually running <code>yum</code> (Amazon Linux) or <code>apt-get</code> (Ubuntu) on the instances.  <note> We strongly recommend using the default value of <code>true</code>, to ensure that your instances have the latest security updates. </note><br/>CustomRecipes:  A <code>LayerCustomRecipes</code> object that specifies the layer's custom recipes.:<br/>AutoAssignPublicIps:  For stacks that are running in a VPC, whether to automatically assign a public IP address to the layer's instances. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinglayers-basics-edit.html">How to Edit a Layer</a>.<br/>AutoAssignElasticIps:  Whether to automatically assign an <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/elastic-ip-addresses-eip.html">Elastic IP address</a> to the layer's instances. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinglayers-basics-edit.html">How to Edit a Layer</a>.<br/>EnableAutoHealing:  Whether to disable auto healing for the layer.<br/>VolumeConfigurations:  A <code>VolumeConfigurations</code> object that describes the layer's Amazon EBS volumes.<br/>Packages:  An array of <code>Package</code> objects that describe the layer's packages.<br/>CustomSecurityGroupIds:  An array containing the layer's custom security group IDs.<br/>CustomJson:  A JSON-formatted string containing custom stack configuration and deployment attributes to be installed on the layer's instances. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingcookbook-json-override.html"> Using Custom JSON</a>. <br/>CustomInstanceProfileArn:  The ARN of an IAM profile to be used for all of the layer's EC2 instances. For more information about IAM ARNs, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">Using Identifiers</a>.<br/>CloudWatchLogsConfiguration:  Specifies CloudWatch Logs configuration options for the layer. For more information, see <a>CloudWatchLogsLogStream</a>.:<br/>Attributes:  One or more user-defined key/value pairs to be added to the stack attributes.<br/>Shortname:  For custom layers only, use this parameter to specify the layer's short name, which is used internally by AWS OpsWorks Stacks and by Chef. The short name is also used as the name for the directory where your app files are installed. It can have a maximum of 200 characters and must be in the following format: /\A[a-z0-9\-\_\.]+\Z/. The built-in layers' short names are defined by AWS OpsWorks Stacks. For more information, see the <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/layers.html">Layer Reference</a> <br/>Name:  The layer name, which is used by the console.<br/>LayerId:  The layer ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogStreams</i> A list of configuration options for CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Whether CloudWatch Logs is enabled for a layer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Shutdown</i> An array of custom recipe names to be run following a <code>shutdown</code> event.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Undeploy</i> An array of custom recipe names to be run following a <code>undeploy</code> event.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Deploy</i> An array of custom recipe names to be run following a <code>deploy</code> event.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Configure</i> An array of custom recipe names to be run following a <code>configure</code> event.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Setup</i> An array of custom recipe names to be run following a <code>setup</code> event.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Shutdown</i> A <code>ShutdownEventConfiguration</code> object that specifies the Shutdown event configuration.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Shutdown</b>: A <code>ShutdownEventConfiguration</code> object that specifies the Shutdown event configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DelayUntilElbConnectionsDrained</i> Whether to enable Elastic Load Balancing connection draining. For more information, see <a href="https://docs.aws.amazon.com/ElasticLoadBalancing/latest/DeveloperGuide/TerminologyandKeyConcepts.html#conn-drain">Connection Draining</a> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExecutionTimeout</i> The time, in seconds, that AWS OpsWorks Stacks will wait after triggering a Shutdown event before shutting down an instance.<br/>
        :return:         """
        pass

    def update_my_user_profile(self, SshPublicKey: str=None):
        """Updates a user's SSH public key.  <b>Required Permissions</b>: To use this action, an IAM user must have self-management enabled or an attached policy that explicitly grants permissions. For more information about user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param SshPublicKey:  The user's SSH public key.<br/>
        :return:         """
        pass

    def update_rds_db_instance(self, RdsDbInstanceArn: str, DbUser: str=None, DbPassword: str=None):
        """Updates an Amazon RDS instance.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param DbPassword:  The database password.<br/>DbUser:  The master user name.<br/>RdsDbInstanceArn:  The Amazon RDS instance's ARN.<br/>
        :return:         """
        pass

    def update_stack(self, StackId: str, Name: str=None, Attributes: StackAttributes=None, ServiceRoleArn: str=None, DefaultInstanceProfileArn: str=None, DefaultOs: str=None, HostnameTheme: str=None, DefaultAvailabilityZone: str=None, DefaultSubnetId: str=None, CustomJson: str=None, ConfigurationManager: StackConfigurationManager=None, ChefConfiguration: ChefConfiguration=None, UseCustomCookbooks: Boolean=None, CustomCookbooksSource: Source=None, DefaultSshKeyName: str=None, DefaultRootDeviceType: str=None, UseOpsworksSecurityGroups: Boolean=None, AgentVersion: str=None):
        """Updates a specified stack.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param AgentVersion:  The default AWS OpsWorks Stacks agent version. You have the following options: <ul> <li> Auto-update - Set this parameter to <code>LATEST</code>. AWS OpsWorks Stacks automatically installs new agent versions on the stack's instances as soon as they are available. </li> <li> Fixed version - Set this parameter to your preferred agent version. To update the agent version, you must edit the stack configuration and specify a new version. AWS OpsWorks Stacks then automatically installs that version on the stack's instances. </li> </ul> The default setting is <code>LATEST</code>. To specify an agent version, you must use the complete version number, not the abbreviated number shown on the console. For a list of available agent version numbers, call <a>DescribeAgentVersions</a>. AgentVersion cannot be set to Chef 12.2. <note> You can also specify an agent version when you create or update an instance, which overrides the stack's default setting. </note><br/>UseOpsworksSecurityGroups:  Whether to associate the AWS OpsWorks Stacks built-in security groups with the stack's layers. AWS OpsWorks Stacks provides a standard set of built-in security groups, one for each layer, which are associated with layers by default. <code>UseOpsworksSecurityGroups</code> allows you to provide your own custom security groups instead of using the built-in groups. <code>UseOpsworksSecurityGroups</code> has the following settings:  <ul> <li> True - AWS OpsWorks Stacks automatically associates the appropriate built-in security group with each layer (default setting). You can associate additional security groups with a layer after you create it, but you cannot delete the built-in security group. </li> <li> False - AWS OpsWorks Stacks does not associate built-in security groups with layers. You must create appropriate EC2 security groups and associate a security group with each layer that you create. However, you can still manually associate a built-in security group with a layer on. Custom security groups are required only for those layers that need custom settings. </li> </ul> For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingstacks-creating.html">Create a New Stack</a>.<br/>DefaultRootDeviceType:  The default root device type. This value is used by default for all instances in the stack, but you can override it when you create an instance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ComponentsAMIs.html#storage-for-the-root-device">Storage for the Root Device</a>.<br/>DefaultSshKeyName:  A default Amazon EC2 key-pair name. The default value is <code>none</code>. If you specify a key-pair name, AWS OpsWorks Stacks installs the public key on the instance and you can use the private key with an SSH client to log in to the instance. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinginstances-ssh.html"> Using SSH to Communicate with an Instance</a> and <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/security-ssh-access.html"> Managing SSH Access</a>. You can override this setting by specifying a different key pair, or no key pair, when you <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinginstances-add.html"> create an instance</a>. <br/>CustomCookbooksSource:  Contains the information required to retrieve an app or cookbook from a repository. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingapps-creating.html">Adding Apps</a> or <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingcookbook.html">Cookbooks and Recipes</a>.:<br/>UseCustomCookbooks:  Whether the stack uses custom cookbooks.<br/>ChefConfiguration:  A <code>ChefConfiguration</code> object that specifies whether to enable Berkshelf and the Berkshelf version on Chef 11.10 stacks. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingstacks-creating.html">Create a New Stack</a>.:<br/>ConfigurationManager:  The configuration manager. When you update a stack, we recommend that you use the configuration manager to specify the Chef version: 12, 11.10, or 11.4 for Linux stacks, or 12.2 for Windows stacks. The default value for Linux stacks is currently 12.:<br/>CustomJson:  A string that contains user-defined, custom JSON. It can be used to override the corresponding default stack configuration JSON values or to pass data to recipes. The string should be in the following format:  <code>"{\"key1\": \"value1\", \"key2\": \"value2\",...}"</code>  For more information about custom JSON, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workingstacks-json.html">Use Custom JSON to Modify the Stack Configuration Attributes</a>.<br/>DefaultSubnetId:  The stack's default VPC subnet ID. This parameter is required if you specify a value for the <code>VpcId</code> parameter. All instances are launched into this subnet unless you specify otherwise when you create the instance. If you also specify a value for <code>DefaultAvailabilityZone</code>, the subnet must be in that zone. For information on default values and when this parameter is required, see the <code>VpcId</code> parameter description. <br/>DefaultAvailabilityZone:  The stack's default Availability Zone, which must be in the stack's region. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/rande.html">Regions and Endpoints</a>. If you also specify a value for <code>DefaultSubnetId</code>, the subnet must be in the same zone. For more information, see <a>CreateStack</a>. <br/>HostnameTheme:  The stack's new host name theme, with spaces replaced by underscores. The theme is used to generate host names for the stack's instances. By default, <code>HostnameTheme</code> is set to <code>Layer_Dependent</code>, which creates host names by appending integers to the layer's short name. The other themes are: <ul> <li>  <code>Baked_Goods</code>  </li> <li>  <code>Clouds</code>  </li> <li>  <code>Europe_Cities</code>  </li> <li>  <code>Fruits</code>  </li> <li>  <code>Greek_Deities_and_Titans</code>  </li> <li>  <code>Legendary_creatures_from_Japan</code>  </li> <li>  <code>Planets_and_Moons</code>  </li> <li>  <code>Roman_Deities</code>  </li> <li>  <code>Scottish_Islands</code>  </li> <li>  <code>US_Cities</code>  </li> <li>  <code>Wild_Cats</code>  </li> </ul> To obtain a generated host name, call <code>GetHostNameSuggestion</code>, which returns a host name based on the current theme.<br/>DefaultOs:  The stack's operating system, which must be set to one of the following: <ul> <li> A supported Linux operating system: An Amazon Linux version, such as <code>Amazon Linux 2018.03</code>, <code>Amazon Linux 2017.09</code>, <code>Amazon Linux 2017.03</code>, <code>Amazon Linux 2016.09</code>, <code>Amazon Linux 2016.03</code>, <code>Amazon Linux 2015.09</code>, or <code>Amazon Linux 2015.03</code>. </li> <li> A supported Ubuntu operating system, such as <code>Ubuntu 16.04 LTS</code>, <code>Ubuntu 14.04 LTS</code>, or <code>Ubuntu 12.04 LTS</code>. </li> <li>  <code>CentOS Linux 7</code>  </li> <li>  <code>Red Hat Enterprise Linux 7</code>  </li> <li> A supported Windows operating system, such as <code>Microsoft Windows Server 2012 R2 Base</code>, <code>Microsoft Windows Server 2012 R2 with SQL Server Express</code>, <code>Microsoft Windows Server 2012 R2 with SQL Server Standard</code>, or <code>Microsoft Windows Server 2012 R2 with SQL Server Web</code>. </li> <li> A custom AMI: <code>Custom</code>. You specify the custom AMI you want to use when you create instances. For more information about how to use custom AMIs with OpsWorks, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinginstances-custom-ami.html">Using Custom AMIs</a>. </li> </ul> The default option is the stack's current operating system. For more information about supported operating systems, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/workinginstances-os.html">AWS OpsWorks Stacks Operating Systems</a>.<br/>DefaultInstanceProfileArn:  The ARN of an IAM profile that is the default profile for all of the stack's EC2 instances. For more information about IAM ARNs, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/Using_Identifiers.html">Using Identifiers</a>.<br/>ServiceRoleArn:  Do not use this parameter. You cannot update a stack's service role.<br/>Attributes:  One or more user-defined key-value pairs to be added to the stack attributes.<br/>Name:  The stack's new name.<br/>StackId:  The stack ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The Chef version. This parameter must be set to 12, 11.10, or 11.4 for Linux stacks, and to 12.2 for Windows stacks. The default value for Linux stacks is 11.4.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name. This parameter must be set to "Chef".<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BerkshelfVersion</i> The Berkshelf version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManageBerkshelf</i> Whether to enable Berkshelf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Revision</i> The application's version. AWS OpsWorks Stacks enables you to easily deploy new versions of an application. One of the simplest approaches is to have branches or revisions in your repository that represent different versions that can potentially be deployed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SshKey</i> In requests, the repository's SSH key. In responses, AWS OpsWorks Stacks returns <code>*****FILTERED*****</code> instead of the actual value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Password</i> When included in a request, the parameter depends on the repository type. <ul> <li> For Amazon S3 bundles, set <code>Password</code> to the appropriate IAM secret access key. </li> <li> For HTTP bundles and Subversion repositories, set <code>Password</code> to the password. </li> </ul> For more information on how to safely handle IAM credentials, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws-access-keys-best-practices.html">https://docs.aws.amazon.com/general/latest/gr/aws-access-keys-best-practices.html</a>. In responses, AWS OpsWorks Stacks returns <code>*****FILTERED*****</code> instead of the actual value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> This parameter depends on the repository type. <ul> <li> For Amazon S3 bundles, set <code>Username</code> to the appropriate IAM access key ID. </li> <li> For HTTP bundles, Git repositories, and Subversion repositories, set <code>Username</code> to the user name. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Url</i> The source URL. The following is an example of an Amazon S3 source URL: <code>https://s3.amazonaws.com/opsworks-demo-bucket/opsworks_cookbook_demo.tar.gz</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The repository type.<br/>
        :return:         """
        pass

    def update_user_profile(self, IamUserArn: str, SshUsername: str=None, SshPublicKey: str=None, AllowSelfManagement: Boolean=None):
        """Updates a specified user profile.  <b>Required Permissions</b>: To use this action, an IAM user must have an attached policy that explicitly grants permissions. For more information about user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param AllowSelfManagement:  Whether users can specify their own SSH public key through the My Settings page. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/security-settingsshkey.html">Managing User Permissions</a>.<br/>SshPublicKey:  The user's new SSH public key.<br/>SshUsername:  The user's SSH user name. The allowable characters are [a-z], [A-Z], [0-9], '-', and '_'. If the specified name includes other punctuation marks, AWS OpsWorks Stacks removes them. For example, <code>my.name</code> will be changed to <code>myname</code>. If you do not specify an SSH user name, AWS OpsWorks Stacks generates one from the IAM user name. <br/>IamUserArn:  The user IAM ARN. This can also be a federated user's ARN.<br/>
        :return:         """
        pass

    def update_volume(self, VolumeId: str, Name: str=None, MountPoint: str=None):
        """Updates an Amazon EBS volume's name or mount point. For more information, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/resources.html">Resource Management</a>.  <b>Required Permissions</b>: To use this action, an IAM user must have a Manage permissions level for the stack, or an attached policy that explicitly grants permissions. For more information on user permissions, see <a href="https://docs.aws.amazon.com/opsworks/latest/userguide/opsworks-security-users.html">Managing User Permissions</a>.

        :param self:
        :param MountPoint:  The new mount point.<br/>Name:  The new name.<br/>VolumeId:  The volume ID.<br/>
        :return:         """
        pass

    class OpsworksResource:
        
        class Layer:
            """Creates a Layer resource.::

  layer = opsworks.Layer('id')

:type id: string
:param id: The Layer's id identifier. This **must** be set.

:rtype: :py:class:`OpsWorks.Layer`
:returns: A Layer resource"""

            def __init__(self, id):
                pass
            
            class LayerAttributes(object):
                pass

            class Recipes(Mapping):
                pass

            class String(object):
                pass

            class LifecycleEventConfiguration(Mapping):
                pass

            class CloudWatchLogsConfiguration(Mapping):
                pass

            class Boolean(object):
                pass

            class VolumeConfiguration(Mapping):
                pass
        
            arn: str=None
            stack_id: str=None
            layer_id: str=None
            type: str=None
            name: str=None
            shortname: str=None
            attributes: LayerAttributes=None
            cloud_watch_logs_configuration: CloudWatchLogsConfiguration=None
            custom_instance_profile_arn: str=None
            custom_json: str=None
            custom_security_group_ids: List[String]=None
            default_security_group_names: List[String]=None
            packages: List[String]=None
            volume_configurations: List[VolumeConfiguration]=None
            enable_auto_healing: Boolean=None
            auto_assign_elastic_ips: Boolean=None
            auto_assign_public_ips: Boolean=None
            default_recipes: Recipes=None
            custom_recipes: Recipes=None
            created_at: str=None
            install_updates_on_boot: Boolean=None
            use_ebs_optimized_instances: Boolean=None
            lifecycle_event_configuration: LifecycleEventConfiguration=None
            
            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class Stack:
            """Creates a Stack resource.::

  stack = opsworks.Stack('id')

:type id: string
:param id: The Stack's id identifier. This **must** be set.

:rtype: :py:class:`OpsWorks.Stack`
:returns: A Stack resource"""

            def __init__(self, id):
                pass
            
            class ChefConfiguration(Mapping):
                pass

            class StackAttributes(object):
                pass

            class Source(Mapping):
                pass

            class StackConfigurationManager(Mapping):
                pass

            class Boolean(object):
                pass
        
            stack_id: str=None
            name: str=None
            arn: str=None
            region: str=None
            vpc_id: str=None
            attributes: StackAttributes=None
            service_role_arn: str=None
            default_instance_profile_arn: str=None
            default_os: str=None
            hostname_theme: str=None
            default_availability_zone: str=None
            default_subnet_id: str=None
            custom_json: str=None
            configuration_manager: StackConfigurationManager=None
            chef_configuration: ChefConfiguration=None
            use_custom_cookbooks: Boolean=None
            use_opsworks_security_groups: Boolean=None
            custom_cookbooks_source: Source=None
            default_ssh_key_name: str=None
            created_at: str=None
            default_root_device_type: str=None
            agent_version: str=None
            
            def create_layer(self, **kwargs):
                # type: (object) -> Layer
                return Layer()

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class StackSummary:
            """Creates a StackSummary resource.::

  stack_summary = opsworks.StackSummary('stack_id')

:type stack_id: string
:param stack_id: The StackSummary's stack_id identifier. This **must** be set.

:rtype: :py:class:`OpsWorks.StackSummary`
:returns: A StackSummary resource"""

            def __init__(self, stack_id):
                pass
            
            class InstancesCount(Mapping):
                pass
        
            stack_id: str=None
            name: str=None
            arn: str=None
            layers_count: int=None
            apps_count: int=None
            instances_count: InstancesCount=None
            

        def create_stack(self, **kwargs):
            # type: (object) -> Stack
            pass

        def get_available_subresources(self) -> List[str]:
            pass

        class StackResourceCollection(List[Stack], ResourceCollection):
            pass

        stacks: StackResourceCollection = None




class OpsWorksCM(BaseClient):
    """<fullname>AWS OpsWorks CM</fullname> AWS OpsWorks for configuration management (CM) is a service that runs and manages configuration management servers. You can use AWS OpsWorks CM to create and manage AWS OpsWorks for Chef Automate and AWS OpsWorks for Puppet Enterprise servers, and add or remove nodes for the servers to manage.  <b>Glossary of terms</b>  <ul> <li>  <b>Server</b>: A configuration management server that can be highly-available. The configuration management server runs on an Amazon Elastic Compute Cloud (EC2) instance, and may use various other AWS services, such as Amazon Relational Database Service (RDS) and Elastic Load Balancing. A server is a generic abstraction over the configuration manager that you want to use, much like Amazon RDS. In AWS OpsWorks CM, you do not start or stop servers. After you create servers, they continue to run until they are deleted. </li> <li>  <b>Engine</b>: The engine is the specific configuration manager that you want to use. Valid values in this release include <code>ChefAutomate</code> and <code>Puppet</code>. </li> <li>  <b>Backup</b>: This is an application-level backup of the data that the configuration manager stores. AWS OpsWorks CM creates an S3 bucket for backups when you launch the first server. A backup maintains a snapshot of a server's configuration-related attributes at the time the backup starts. </li> <li>  <b>Events</b>: Events are always related to a server. Events are written during server creation, when health checks run, when backups are created, when system maintenance is performed, etc. When you delete a server, the server's events are also deleted. </li> <li>  <b>Account attributes</b>: Every account has attributes that are assigned in the AWS OpsWorks CM database. These attributes store information about configuration limits (servers, backups, etc.) and your customer account.  </li> </ul>  <b>Endpoints</b>  AWS OpsWorks CM supports the following endpoints, all HTTPS. You must connect to one of the following endpoints. Your servers can only be accessed or managed within the endpoint in which they are created. <ul> <li> opsworks-cm.us-east-1.amazonaws.com </li> <li> opsworks-cm.us-east-2.amazonaws.com </li> <li> opsworks-cm.us-west-1.amazonaws.com </li> <li> opsworks-cm.us-west-2.amazonaws.com </li> <li> opsworks-cm.ap-northeast-1.amazonaws.com </li> <li> opsworks-cm.ap-southeast-1.amazonaws.com </li> <li> opsworks-cm.ap-southeast-2.amazonaws.com </li> <li> opsworks-cm.eu-central-1.amazonaws.com </li> <li> opsworks-cm.eu-west-1.amazonaws.com </li> </ul>  <b>Throttling limits</b>  All API operations allow for five requests per second with a burst of 10 requests per second."""

    class NodeAssociatedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_node_association_status` for valid parameters"""
            pass


    class DescribeBackupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_backups` for valid parameters"""
            pass
    class DescribeEventsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_events` for valid parameters"""
            pass
    class DescribeServersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_servers` for valid parameters"""
            pass


    class AssociateNodeRequest(Mapping):
        pass
    
    class AssociateNodeResponse(Mapping):
        pass
    
    class EngineAttribute(Mapping):
        pass
    
    class CreateBackupRequest(Mapping):
        pass
    
    class CreateBackupResponse(Mapping):
        pass
    
    class CreateServerRequest(Mapping):
        pass
    
    class CreateServerResponse(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class String(object):
        pass
    
    class DeleteBackupRequest(Mapping):
        pass
    
    class DeleteBackupResponse(Mapping):
        pass
    
    class DeleteServerRequest(Mapping):
        pass
    
    class DeleteServerResponse(Mapping):
        pass
    
    class DescribeAccountAttributesRequest(Mapping):
        pass
    
    class DescribeAccountAttributesResponse(Mapping):
        pass
    
    class DescribeBackupsRequest(Mapping):
        pass
    
    class DescribeBackupsResponse(Mapping):
        pass
    
    class DescribeEventsRequest(Mapping):
        pass
    
    class DescribeEventsResponse(Mapping):
        pass
    
    class DescribeNodeAssociationStatusRequest(Mapping):
        pass
    
    class DescribeNodeAssociationStatusResponse(Mapping):
        pass
    
    class DescribeServersRequest(Mapping):
        pass
    
    class DescribeServersResponse(Mapping):
        pass
    
    class DisassociateNodeRequest(Mapping):
        pass
    
    class DisassociateNodeResponse(Mapping):
        pass
    
    class ExportServerEngineAttributeRequest(Mapping):
        pass
    
    class ExportServerEngineAttributeResponse(Mapping):
        pass
    
    class RestoreServerRequest(Mapping):
        pass
    
    class RestoreServerResponse(Mapping):
        pass
    
    class StartMaintenanceRequest(Mapping):
        pass
    
    class StartMaintenanceResponse(Mapping):
        pass
    
    class UpdateServerRequest(Mapping):
        pass
    
    class UpdateServerResponse(Mapping):
        pass
    
    class UpdateServerEngineAttributesRequest(Mapping):
        pass
    
    class UpdateServerEngineAttributesResponse(Mapping):
        pass
    
    def associate_node(self, EngineAttributes: List[EngineAttribute], NodeName: str, ServerName: str) -> AssociateNodeResponse:
        """ Associates a new node with the server. For more information about how to disassociate a node, see <a>DisassociateNode</a>.  On a Chef server: This command is an alternative to <code>knife bootstrap</code>.  Example (Chef): <code>aws opsworks-cm associate-node --server-name <i>MyServer</i> --node-name <i>MyManagedNode</i> --engine-attributes "Name=<i>CHEF_ORGANIZATION</i>,Value=default" "Name=<i>CHEF_NODE_PUBLIC_KEY</i>,Value=<i>public-key-pem</i>"</code>   On a Puppet server, this command is an alternative to the <code>puppet cert sign</code> command that signs a Puppet node CSR.   Example (Chef): <code>aws opsworks-cm associate-node --server-name <i>MyServer</i> --node-name <i>MyManagedNode</i> --engine-attributes "Name=<i>PUPPET_NODE_CSR</i>,Value=<i>csr-pem</i>"</code>   A node can can only be associated with servers that are in a <code>HEALTHY</code> state. Otherwise, an <code>InvalidStateException</code> is thrown. A <code>ResourceNotFoundException</code> is thrown when the server does not exist. A <code>ValidationException</code> is raised when parameters of the request are not valid. The AssociateNode API call can be integrated into Auto Scaling configurations, AWS Cloudformation templates, or the user data of a server's instance. 

        :param self:
        :param EngineAttributes:  Engine attributes used for associating the node.  <p class="title"> <b>Attributes accepted in a AssociateNode request for Chef</b>  <ul> <li>  <code>CHEF_ORGANIZATION</code>: The Chef organization with which the node is associated. By default only one organization named <code>default</code> can exist.  </li> <li>  <code>CHEF_NODE_PUBLIC_KEY</code>: A PEM-formatted public key. This key is required for the <code>chef-client</code> agent to access the Chef API.  </li> </ul> <p class="title"> <b>Attributes accepted in a AssociateNode request for Puppet</b>  <ul> <li>  <code>PUPPET_NODE_CSR</code>: A PEM-formatted certificate-signing request (CSR) that is created by the node.  </li> </ul><br/>NodeName:  The name of the node. <br/>ServerName:  The name of the server with which to associate the node. <br/>
        :return: NodeAssociationStatusToken:  Contains a token which can be passed to the <code>DescribeNodeAssociationStatus</code> API call to get the status of the association request. <br/>        """
        pass

    def create_backup(self, ServerName: str, Description: str=None) -> CreateBackupResponse:
        """ Creates an application-level backup of a server. While the server is in the <code>BACKING_UP</code> state, the server cannot be changed, and no additional backup can be created.   Backups can be created for servers in <code>RUNNING</code>, <code>HEALTHY</code>, and <code>UNHEALTHY</code> states. By default, you can create a maximum of 50 manual backups.   This operation is asynchronous.   A <code>LimitExceededException</code> is thrown when the maximum number of manual backups is reached. An <code>InvalidStateException</code> is thrown when the server is not in any of the following states: RUNNING, HEALTHY, or UNHEALTHY. A <code>ResourceNotFoundException</code> is thrown when the server is not found. A <code>ValidationException</code> is thrown when parameters of the request are not valid. 

        :param self:
        :param Description:   A user-defined description of the backup. <br/>ServerName:  The name of the server that you want to back up. <br/>
        :return: Backup:  Backup created by request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserArn</i>  The IAM user ARN of the requester for manual backups. This field is empty for automated backups. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ToolsVersion</i>  The version of AWS OpsWorks CM-specific tools that is obtained from the server when the backup is created. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i>  The subnet IDs that are obtained from the server when the backup is created. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusDescription</i>  An informational message about backup status. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of a backup while in progress. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceRoleArn</i>  The service role ARN that is obtained from the server when the backup is created. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerName</i>  The name of the server from which the backup was made. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i>  The security group IDs that are obtained from the server when the backup is created. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3LogUrl</i>  The Amazon S3 URL of the backup's log file. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3DataUrl</i>  This field is deprecated and is no longer used. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3DataSize</i>  This field is deprecated and is no longer used. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i>  The preferred maintenance period that is obtained from the server when the backup is created. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i>  The preferred backup period that is obtained from the server when the backup is created. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyPair</i>  The key pair that is obtained from the server when the backup is created. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i>  The instance type that is obtained from the server when the backup is created. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceProfileArn</i>  The EC2 instance profile ARN that is obtained from the server when the backup is created. Because this value is stored, you are not required to provide the InstanceProfileArn again if you restore a backup. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i>  The engine version that is obtained from the server when the backup is created. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineModel</i>  The engine model that is obtained from the server when the backup is created. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i>  The engine type that is obtained from the server when the backup is created. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i>  A user-provided description for a manual backup. This field is empty for automated backups. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i>  The time stamp when the backup was created in the database. Example: <code>2016-07-29T13:38:47.520Z</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupType</i>  The backup type. Valid values are <code>automated</code> or <code>manual</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupId</i>  The generated ID of the backup. Example: <code>myServerName-yyyyMMddHHmmssSSS</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupArn</i> The ARN of the backup. <br/>        """
        pass

    def create_server(self, ServiceRoleArn: str, InstanceType: str, InstanceProfileArn: str, ServerName: str, AssociatePublicIpAddress: Boolean=None, DisableAutomatedBackup: Boolean=None, Engine: str=None, EngineModel: str=None, EngineVersion: str=None, EngineAttributes: List[EngineAttribute]=None, BackupRetentionCount: int=None, KeyPair: str=None, PreferredMaintenanceWindow: str=None, PreferredBackupWindow: str=None, SecurityGroupIds: List[String]=None, SubnetIds: List[String]=None, BackupId: str=None) -> CreateServerResponse:
        """ Creates and immedately starts a new server. The server is ready to use when it is in the <code>HEALTHY</code> state. By default, you can create a maximum of 10 servers.   This operation is asynchronous.   A <code>LimitExceededException</code> is thrown when you have created the maximum number of servers (10). A <code>ResourceAlreadyExistsException</code> is thrown when a server with the same name already exists in the account. A <code>ResourceNotFoundException</code> is thrown when you specify a backup ID that is not valid or is for a backup that does not exist. A <code>ValidationException</code> is thrown when parameters of the request are not valid.   If you do not specify a security group by adding the <code>SecurityGroupIds</code> parameter, AWS OpsWorks creates a new security group.   <i>Chef Automate:</i> The default security group opens the Chef server to the world on TCP port 443. If a KeyName is present, AWS OpsWorks enables SSH access. SSH is also open to the world on TCP port 22.   <i>Puppet Enterprise:</i> The default security group opens TCP ports 22, 443, 4433, 8140, 8142, 8143, and 8170. If a KeyName is present, AWS OpsWorks enables SSH access. SSH is also open to the world on TCP port 22.  By default, your server is accessible from any IP address. We recommend that you update your security group rules to allow access from known IP addresses and address ranges only. To edit security group rules, open Security Groups in the navigation pane of the EC2 management console. 

        :param self:
        :param BackupId:   If you specify this field, AWS OpsWorks CM creates the server by using the backup represented by BackupId. <br/>SubnetIds:   The IDs of subnets in which to launch the server EC2 instance.   Amazon EC2-Classic customers: This field is required. All servers must run within a VPC. The VPC must have "Auto Assign Public IP" enabled.   EC2-VPC customers: This field is optional. If you do not specify subnet IDs, your EC2 instances are created in a default subnet that is selected by Amazon EC2. If you specify subnet IDs, the VPC must have "Auto Assign Public IP" enabled.  For more information about supported Amazon EC2 platforms, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-supported-platforms.html">Supported Platforms</a>.<br/>ServiceRoleArn:   The service role that the AWS OpsWorks CM service backend uses to work with your account. Although the AWS OpsWorks management console typically creates the service role for you, if you are using the AWS CLI or API commands, run the service-role-creation.yaml AWS CloudFormation template, located at https://s3.amazonaws.com/opsworks-cm-us-east-1-prod-default-assets/misc/opsworks-cm-roles.yaml. This template creates a CloudFormation stack that includes the service role and instance profile that you need. <br/>SecurityGroupIds:   A list of security group IDs to attach to the Amazon EC2 instance. If you add this parameter, the specified security groups must be within the VPC that is specified by <code>SubnetIds</code>.   If you do not specify this parameter, AWS OpsWorks CM creates one new security group that uses TCP ports 22 and 443, open to 0.0.0.0/0 (everyone). <br/>PreferredBackupWindow:   The start time for a one-hour period during which AWS OpsWorks CM backs up application-level data on your server if automated backups are enabled. Valid values must be specified in one of the following formats:  <ul> <li>  <code>HH:MM</code> for daily backups </li> <li>  <code>DDD:HH:MM</code> for weekly backups </li> </ul> The specified time is in coordinated universal time (UTC). The default value is a random, daily start time.  <b>Example:</b> <code>08:00</code>, which represents a daily start time of 08:00 UTC.  <b>Example:</b> <code>Mon:08:00</code>, which represents a start time of every Monday at 08:00 UTC. (8:00 a.m.)<br/>PreferredMaintenanceWindow:   The start time for a one-hour period each week during which AWS OpsWorks CM performs maintenance on the instance. Valid values must be specified in the following format: <code>DDD:HH:MM</code>. The specified time is in coordinated universal time (UTC). The default value is a random one-hour period on Tuesday, Wednesday, or Friday. See <code>TimeWindowDefinition</code> for more information.   <b>Example:</b> <code>Mon:08:00</code>, which represents a start time of every Monday at 08:00 UTC. (8:00 a.m.) <br/>KeyPair:   The Amazon EC2 key pair to set for the instance. This parameter is optional; if desired, you may specify this parameter to connect to your instances by using SSH. <br/>InstanceType:   The Amazon EC2 instance type to use. For example, <code>m5.large</code>. <br/>InstanceProfileArn:   The ARN of the instance profile that your Amazon EC2 instances use. Although the AWS OpsWorks console typically creates the instance profile for you, if you are using API commands instead, run the service-role-creation.yaml AWS CloudFormation template, located at https://s3.amazonaws.com/opsworks-cm-us-east-1-prod-default-assets/misc/opsworks-cm-roles.yaml. This template creates a CloudFormation stack that includes the instance profile you need. <br/>ServerName:   The name of the server. The server name must be unique within your AWS account, within each region. Server names must start with a letter; then letters, numbers, or hyphens (-) are allowed, up to a maximum of 40 characters. <br/>BackupRetentionCount:   The number of automated backups that you want to keep. Whenever a new backup is created, AWS OpsWorks CM deletes the oldest backups if this number is exceeded. The default value is <code>1</code>. <br/>EngineAttributes:  Optional engine attributes on a specified server.  <p class="title"> <b>Attributes accepted in a Chef createServer request:</b>  <ul> <li>  <code>CHEF_AUTOMATE_PIVOTAL_KEY</code>: A base64-encoded RSA public key. The corresponding private key is required to access the Chef API. When no CHEF_AUTOMATE_PIVOTAL_KEY is set, a private key is generated and returned in the response.  </li> <li>  <code>CHEF_AUTOMATE_ADMIN_PASSWORD</code>: The password for the administrative user in the Chef Automate web-based dashboard. The password length is a minimum of eight characters, and a maximum of 32. The password can contain letters, numbers, and special characters (!/@#$%^&amp;+=_). The password must contain at least one lower case letter, one upper case letter, one number, and one special character. When no CHEF_AUTOMATE_ADMIN_PASSWORD is set, one is generated and returned in the response. </li> </ul> <p class="title"> <b>Attributes accepted in a Puppet createServer request:</b>  <ul> <li>  <code>PUPPET_ADMIN_PASSWORD</code>: To work with the Puppet Enterprise console, a password must use ASCII characters. </li> <li>  <code>PUPPET_R10K_REMOTE</code>: The r10k remote is the URL of your control repository (for example, ssh://git@your.git-repo.com:user/control-repo.git). Specifying an r10k remote opens TCP port 8170. </li> <li>  <code>PUPPET_R10K_PRIVATE_KEY</code>: If you are using a private Git repository, add PUPPET_R10K_PRIVATE_KEY to specify a PEM-encoded private SSH key. </li> </ul><br/>EngineVersion:   The major release version of the engine that you want to use. For a Chef server, the valid value for EngineVersion is currently <code>12</code>. For a Puppet server, the valid value is <code>2017</code>. <br/>EngineModel:   The engine model of the server. Valid values in this release include <code>Monolithic</code> for Puppet and <code>Single</code> for Chef. <br/>Engine:   The configuration management engine to use. Valid values include <code>ChefAutomate</code> and <code>Puppet</code>. <br/>DisableAutomatedBackup:   Enable or disable scheduled backups. Valid values are <code>true</code> or <code>false</code>. The default value is <code>true</code>. <br/>AssociatePublicIpAddress:   Associate a public IP address with a server that you are launching. Valid values are <code>true</code> or <code>false</code>. The default value is <code>true</code>. <br/>
        :return: Server:  The server that is created by the request. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerArn</i> The ARN of the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i>  The subnet IDs specified in a CreateServer request. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusReason</i>  Depending on the server status, this field has either a human-readable message (such as a create or backup error), or an escaped block of JSON (used for health check results). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i>  The server's status. This field displays the states of actions in progress, such as creating, running, or backing up the server, as well as the server's health state. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceRoleArn</i> The service role ARN used to create the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i>  The security group IDs for the server, as specified in the CloudFormation stack. These might not be the same security groups that are shown in the EC2 console. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i> The preferred backup period specified for the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> The preferred maintenance period specified for the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceStatus</i> The status of the most recent server maintenance run. Shows <code>SUCCESS</code> or <code>FAILED</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyPair</i> The key pair associated with the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i>  The instance type for the server, as specified in the CloudFormation stack. This might not be the same instance type that is shown in the EC2 console. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceProfileArn</i> The instance profile ARN of the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> The engine version of the server. For a Chef server, the valid value for EngineVersion is currently <code>12</code>. For a Puppet server, the valid value is <code>2017</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineAttributes</i> The response of a createServer() request returns the master credential to access the server in EngineAttributes. These credentials are not stored by AWS OpsWorks CM; they are returned only as part of the result of createServer().  <p class="title"> <b>Attributes returned in a createServer response for Chef</b>  <ul> <li>  <code>CHEF_AUTOMATE_PIVOTAL_KEY</code>: A base64-encoded RSA private key that is generated by AWS OpsWorks for Chef Automate. This private key is required to access the Chef API. </li> <li>  <code>CHEF_STARTER_KIT</code>: A base64-encoded ZIP file. The ZIP file contains a Chef starter kit, which includes a README, a configuration file, and the required RSA private key. Save this file, unzip it, and then change to the directory where you've unzipped the file contents. From this directory, you can run Knife commands. </li> </ul> <p class="title"> <b>Attributes returned in a createServer response for Puppet</b>  <ul> <li>  <code>PUPPET_STARTER_KIT</code>: A base64-encoded ZIP file. The ZIP file contains a Puppet starter kit, including a README and a required private key. Save this file, unzip it, and then change to the directory where you've unzipped the file contents. </li> <li>  <code>PUPPET_ADMIN_PASSWORD</code>: An administrator password that you can use to sign in to the Puppet Enterprise console after the server is online. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineModel</i> The engine model of the server. Valid values in this release include <code>Monolithic</code> for Puppet and <code>Single</code> for Chef. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> The engine type of the server. Valid values in this release include <code>ChefAutomate</code> and <code>Puppet</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i>  A DNS name that can be used to access the engine. Example: <code>myserver-asdfghjkl.us-east-1.opsworks.io</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisableAutomatedBackup</i> Disables automated backups. The number of stored backups is dependent on the value of PreferredBackupCount. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudFormationStackArn</i> The ARN of the CloudFormation stack that was used to create the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i> Time stamp of server creation. Example <code>2016-07-29T13:38:47.520Z</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerName</i> The name of the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionCount</i> The number of automated backups to keep. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatePublicIpAddress</i> Associate a public IP address with a server that you are launching. <br/>        """
        pass

    def delete_backup(self, BackupId: str) -> DeleteBackupResponse:
        """ Deletes a backup. You can delete both manual and automated backups. This operation is asynchronous.   An <code>InvalidStateException</code> is thrown when a backup deletion is already in progress. A <code>ResourceNotFoundException</code> is thrown when the backup does not exist. A <code>ValidationException</code> is thrown when parameters of the request are not valid. 

        :param self:
        :param BackupId:  The ID of the backup to delete. Run the DescribeBackups command to get a list of backup IDs. Backup IDs are in the format <code>ServerName-yyyyMMddHHmmssSSS</code>. <br/>
        :return:         """
        pass

    def delete_server(self, ServerName: str) -> DeleteServerResponse:
        """ Deletes the server and the underlying AWS CloudFormation stacks (including the server's EC2 instance). When you run this command, the server state is updated to <code>DELETING</code>. After the server is deleted, it is no longer returned by <code>DescribeServer</code> requests. If the AWS CloudFormation stack cannot be deleted, the server cannot be deleted.   This operation is asynchronous.   An <code>InvalidStateException</code> is thrown when a server deletion is already in progress. A <code>ResourceNotFoundException</code> is thrown when the server does not exist. A <code>ValidationException</code> is raised when parameters of the request are not valid.   

        :param self:
        :param ServerName:  The ID of the server to delete.<br/>
        :return:         """
        pass

    def describe_account_attributes(self) -> DescribeAccountAttributesResponse:
        """ Describes your OpsWorks-CM account attributes.   This operation is synchronous. 

        :param self:
        :param 
        :return: Attributes:   The attributes that are currently set for the account. <br/>        """
        pass

    def describe_backups(self, BackupId: str=None, ServerName: str=None, NextToken: str=None, MaxResults: int=None) -> DescribeBackupsResponse:
        """ Describes backups. The results are ordered by time, with newest backups first. If you do not specify a BackupId or ServerName, the command returns all backups.   This operation is synchronous.   A <code>ResourceNotFoundException</code> is thrown when the backup does not exist. A <code>ValidationException</code> is raised when parameters of the request are not valid. 

        :param self:
        :param MaxResults:  This is not currently implemented for <code>DescribeBackups</code> requests.<br/>NextToken:  This is not currently implemented for <code>DescribeBackups</code> requests.<br/>ServerName:  Returns backups for the server with the specified ServerName. <br/>BackupId:  Describes a single backup. <br/>
        :return: NextToken:  This is not currently implemented for <code>DescribeBackups</code> requests.<br/>Backups:  Contains the response to a <code>DescribeBackups</code> request. <br/>        """
        pass

    def describe_events(self, ServerName: str, NextToken: str=None, MaxResults: int=None) -> DescribeEventsResponse:
        """ Describes events for a specified server. Results are ordered by time, with newest events first.   This operation is synchronous.   A <code>ResourceNotFoundException</code> is thrown when the server does not exist. A <code>ValidationException</code> is raised when parameters of the request are not valid. 

        :param self:
        :param MaxResults:  To receive a paginated response, use this parameter to specify the maximum number of results to be returned with a single call. If the number of available results exceeds this maximum, the response includes a <code>NextToken</code> value that you can assign to the <code>NextToken</code> request parameter to get the next set of results. <br/>NextToken:  NextToken is a string that is returned in some command responses. It indicates that not all entries have been returned, and that you must run at least one more request to get remaining items. To get remaining results, call <code>DescribeEvents</code> again, and assign the token from the previous results as the value of the <code>nextToken</code> parameter. If there are no more results, the response object's <code>nextToken</code> parameter value is <code>null</code>. Setting a <code>nextToken</code> value that was not returned in your previous results causes an <code>InvalidNextTokenException</code> to occur. <br/>ServerName:  The name of the server for which you want to view events.<br/>
        :return: NextToken:  NextToken is a string that is returned in some command responses. It indicates that not all entries have been returned, and that you must run at least one more request to get remaining items. To get remaining results, call <code>DescribeEvents</code> again, and assign the token from the previous results as the value of the <code>nextToken</code> parameter. If there are no more results, the response object's <code>nextToken</code> parameter value is <code>null</code>. Setting a <code>nextToken</code> value that was not returned in your previous results causes an <code>InvalidNextTokenException</code> to occur. <br/>ServerEvents:  Contains the response to a <code>DescribeEvents</code> request. <br/>        """
        pass

    def describe_node_association_status(self, ServerName: str, NodeAssociationStatusToken: str) -> DescribeNodeAssociationStatusResponse:
        """ Returns the current status of an existing association or disassociation request.   A <code>ResourceNotFoundException</code> is thrown when no recent association or disassociation request with the specified token is found, or when the server does not exist. A <code>ValidationException</code> is raised when parameters of the request are not valid. 

        :param self:
        :param ServerName:  The name of the server from which to disassociate the node. <br/>NodeAssociationStatusToken:  The token returned in either the AssociateNodeResponse or the DisassociateNodeResponse. <br/>
        :return: EngineAttributes:  Attributes specific to the node association. In Puppet, the attibute PUPPET_NODE_CERT contains the signed certificate (the result of the CSR). <br/>NodeAssociationStatus:  The status of the association or disassociation request.  <p class="title"> <b>Possible values:</b>  <ul> <li>  <code>SUCCESS</code>: The association or disassociation succeeded.  </li> <li>  <code>FAILED</code>: The association or disassociation failed.  </li> <li>  <code>IN_PROGRESS</code>: The association or disassociation is still in progress.  </li> </ul><br/>        """
        pass

    def describe_servers(self, ServerName: str=None, NextToken: str=None, MaxResults: int=None) -> DescribeServersResponse:
        """ Lists all configuration management servers that are identified with your account. Only the stored results from Amazon DynamoDB are returned. AWS OpsWorks CM does not query other services.   This operation is synchronous.   A <code>ResourceNotFoundException</code> is thrown when the server does not exist. A <code>ValidationException</code> is raised when parameters of the request are not valid. 

        :param self:
        :param MaxResults:  This is not currently implemented for <code>DescribeServers</code> requests. <br/>NextToken:  This is not currently implemented for <code>DescribeServers</code> requests. <br/>ServerName:  Describes the server with the specified ServerName.<br/>
        :return: NextToken:  This is not currently implemented for <code>DescribeServers</code> requests. <br/>Servers:  Contains the response to a <code>DescribeServers</code> request.  <i>For Puppet Server:</i> <code>DescribeServersResponse$Servers$EngineAttributes</code> contains PUPPET_API_CA_CERT. This is the PEM-encoded CA certificate that is used by the Puppet API over TCP port number 8140. The CA certificate is also used to sign node certificates.<br/>        """
        pass

    def disassociate_node(self, NodeName: str, ServerName: str, EngineAttributes: List[EngineAttribute]=None) -> DisassociateNodeResponse:
        """ Disassociates a node from an AWS OpsWorks CM server, and removes the node from the server's managed nodes. After a node is disassociated, the node key pair is no longer valid for accessing the configuration manager's API. For more information about how to associate a node, see <a>AssociateNode</a>.  A node can can only be disassociated from a server that is in a <code>HEALTHY</code> state. Otherwise, an <code>InvalidStateException</code> is thrown. A <code>ResourceNotFoundException</code> is thrown when the server does not exist. A <code>ValidationException</code> is raised when parameters of the request are not valid. 

        :param self:
        :param EngineAttributes:  Engine attributes that are used for disassociating the node. No attributes are required for Puppet.  <p class="title"> <b>Attributes required in a DisassociateNode request for Chef</b>  <ul> <li>  <code>CHEF_ORGANIZATION</code>: The Chef organization with which the node was associated. By default only one organization named <code>default</code> can exist.  </li> </ul><br/>NodeName:  The name of the client node. <br/>ServerName:  The name of the server from which to disassociate the node. <br/>
        :return: NodeAssociationStatusToken:  Contains a token which can be passed to the <code>DescribeNodeAssociationStatus</code> API call to get the status of the disassociation request. <br/>        """
        pass

    def export_server_engine_attribute(self, ServerName: str, ExportAttributeName: str, InputAttributes: List[EngineAttribute]=None) -> ExportServerEngineAttributeResponse:
        """ Exports a specified server engine attribute as a base64-encoded string. For example, you can export user data that you can use in EC2 to associate nodes with a server.   This operation is synchronous.   A <code>ValidationException</code> is raised when parameters of the request are not valid. A <code>ResourceNotFoundException</code> is thrown when the server does not exist. An <code>InvalidStateException</code> is thrown when the server is in any of the following states: CREATING, TERMINATED, FAILED or DELETING. 

        :param self:
        :param InputAttributes:  The list of engine attributes. The list type is <code>EngineAttribute</code>. An <code>EngineAttribute</code> list item is a pair that includes an attribute name and its value. For the <code>Userdata</code> ExportAttributeName, the following are supported engine attribute names. <ul> <li>  <b>RunList</b> In Chef, a list of roles or recipes that are run in the specified order. In Puppet, this parameter is ignored. </li> <li>  <b>OrganizationName</b> In Chef, an organization name. AWS OpsWorks for Chef Automate always creates the organization <code>default</code>. In Puppet, this parameter is ignored. </li> <li>  <b>NodeEnvironment</b> In Chef, a node environment (for example, development, staging, or one-box). In Puppet, this parameter is ignored. </li> <li>  <b>NodeClientVersion</b> In Chef, the version of the Chef engine (three numbers separated by dots, such as 13.8.5). If this attribute is empty, OpsWorks for Chef Automate uses the most current version. In Puppet, this parameter is ignored. </li> </ul><br/>ServerName:  The name of the server from which you are exporting the attribute.<br/>ExportAttributeName:  The name of the export attribute. Currently, the supported export attribute is <code>Userdata</code>. This exports a user data script that includes parameters and values provided in the <code>InputAttributes</code> list.<br/>
        :return: ServerName:  The server name used in the request.<br/>EngineAttribute:  The requested engine attribute pair with attribute name and value.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The value of the engine attribute. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the engine attribute. <br/>        """
        pass

    def restore_server(self, ServerName: str, BackupId: str, InstanceType: str=None, KeyPair: str=None) -> RestoreServerResponse:
        """ Restores a backup to a server that is in a <code>CONNECTION_LOST</code>, <code>HEALTHY</code>, <code>RUNNING</code>, <code>UNHEALTHY</code>, or <code>TERMINATED</code> state. When you run RestoreServer, the server's EC2 instance is deleted, and a new EC2 instance is configured. RestoreServer maintains the existing server endpoint, so configuration management of the server's client devices (nodes) should continue to work.   This operation is asynchronous.   An <code>InvalidStateException</code> is thrown when the server is not in a valid state. A <code>ResourceNotFoundException</code> is thrown when the server does not exist. A <code>ValidationException</code> is raised when parameters of the request are not valid. 

        :param self:
        :param KeyPair:   The name of the key pair to set on the new EC2 instance. This can be helpful if the administrator no longer has the SSH key. <br/>InstanceType:   The type of the instance to create. Valid values must be specified in the following format: <code>^([cm][34]|t2).*</code> For example, <code>m5.large</code>. Valid values are <code>m5.large</code>, <code>r5.xlarge</code>, and <code>r5.2xlarge</code>. If you do not specify this parameter, RestoreServer uses the instance type from the specified backup. <br/>ServerName:   The name of the server that you want to restore. <br/>BackupId:   The ID of the backup that you want to use to restore a server. <br/>
        :return:         """
        pass

    def start_maintenance(self, ServerName: str, EngineAttributes: List[EngineAttribute]=None) -> StartMaintenanceResponse:
        """ Manually starts server maintenance. This command can be useful if an earlier maintenance attempt failed, and the underlying cause of maintenance failure has been resolved. The server is in an <code>UNDER_MAINTENANCE</code> state while maintenance is in progress.   Maintenance can only be started on servers in <code>HEALTHY</code> and <code>UNHEALTHY</code> states. Otherwise, an <code>InvalidStateException</code> is thrown. A <code>ResourceNotFoundException</code> is thrown when the server does not exist. A <code>ValidationException</code> is raised when parameters of the request are not valid. 

        :param self:
        :param EngineAttributes:  Engine attributes that are specific to the server on which you want to run maintenance. <br/>ServerName:  The name of the server on which to run maintenance. <br/>
        :return: Server:  Contains the response to a <code>StartMaintenance</code> request. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerArn</i> The ARN of the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i>  The subnet IDs specified in a CreateServer request. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusReason</i>  Depending on the server status, this field has either a human-readable message (such as a create or backup error), or an escaped block of JSON (used for health check results). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i>  The server's status. This field displays the states of actions in progress, such as creating, running, or backing up the server, as well as the server's health state. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceRoleArn</i> The service role ARN used to create the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i>  The security group IDs for the server, as specified in the CloudFormation stack. These might not be the same security groups that are shown in the EC2 console. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i> The preferred backup period specified for the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> The preferred maintenance period specified for the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceStatus</i> The status of the most recent server maintenance run. Shows <code>SUCCESS</code> or <code>FAILED</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyPair</i> The key pair associated with the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i>  The instance type for the server, as specified in the CloudFormation stack. This might not be the same instance type that is shown in the EC2 console. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceProfileArn</i> The instance profile ARN of the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> The engine version of the server. For a Chef server, the valid value for EngineVersion is currently <code>12</code>. For a Puppet server, the valid value is <code>2017</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineAttributes</i> The response of a createServer() request returns the master credential to access the server in EngineAttributes. These credentials are not stored by AWS OpsWorks CM; they are returned only as part of the result of createServer().  <p class="title"> <b>Attributes returned in a createServer response for Chef</b>  <ul> <li>  <code>CHEF_AUTOMATE_PIVOTAL_KEY</code>: A base64-encoded RSA private key that is generated by AWS OpsWorks for Chef Automate. This private key is required to access the Chef API. </li> <li>  <code>CHEF_STARTER_KIT</code>: A base64-encoded ZIP file. The ZIP file contains a Chef starter kit, which includes a README, a configuration file, and the required RSA private key. Save this file, unzip it, and then change to the directory where you've unzipped the file contents. From this directory, you can run Knife commands. </li> </ul> <p class="title"> <b>Attributes returned in a createServer response for Puppet</b>  <ul> <li>  <code>PUPPET_STARTER_KIT</code>: A base64-encoded ZIP file. The ZIP file contains a Puppet starter kit, including a README and a required private key. Save this file, unzip it, and then change to the directory where you've unzipped the file contents. </li> <li>  <code>PUPPET_ADMIN_PASSWORD</code>: An administrator password that you can use to sign in to the Puppet Enterprise console after the server is online. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineModel</i> The engine model of the server. Valid values in this release include <code>Monolithic</code> for Puppet and <code>Single</code> for Chef. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> The engine type of the server. Valid values in this release include <code>ChefAutomate</code> and <code>Puppet</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i>  A DNS name that can be used to access the engine. Example: <code>myserver-asdfghjkl.us-east-1.opsworks.io</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisableAutomatedBackup</i> Disables automated backups. The number of stored backups is dependent on the value of PreferredBackupCount. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudFormationStackArn</i> The ARN of the CloudFormation stack that was used to create the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i> Time stamp of server creation. Example <code>2016-07-29T13:38:47.520Z</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerName</i> The name of the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionCount</i> The number of automated backups to keep. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatePublicIpAddress</i> Associate a public IP address with a server that you are launching. <br/>        """
        pass

    def update_server(self, ServerName: str, DisableAutomatedBackup: Boolean=None, BackupRetentionCount: int=None, PreferredMaintenanceWindow: str=None, PreferredBackupWindow: str=None) -> UpdateServerResponse:
        """ Updates settings for a server.   This operation is synchronous. 

        :param self:
        :param PreferredBackupWindow:   <code>DDD:HH:MM</code> (weekly start time) or <code>HH:MM</code> (daily start time).   Time windows always use coordinated universal time (UTC). Valid strings for day of week (<code>DDD</code>) are: <code>Mon</code>, <code>Tue</code>, <code>Wed</code>, <code>Thr</code>, <code>Fri</code>, <code>Sat</code>, or <code>Sun</code>.<br/>PreferredMaintenanceWindow:   <code>DDD:HH:MM</code> (weekly start time) or <code>HH:MM</code> (daily start time).   Time windows always use coordinated universal time (UTC). Valid strings for day of week (<code>DDD</code>) are: <code>Mon</code>, <code>Tue</code>, <code>Wed</code>, <code>Thr</code>, <code>Fri</code>, <code>Sat</code>, or <code>Sun</code>.<br/>ServerName:  The name of the server to update. <br/>BackupRetentionCount:  Sets the number of automated backups that you want to keep. <br/>DisableAutomatedBackup:  Setting DisableAutomatedBackup to <code>true</code> disables automated or scheduled backups. Automated backups are enabled by default. <br/>
        :return: Server:  Contains the response to a <code>UpdateServer</code> request. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerArn</i> The ARN of the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i>  The subnet IDs specified in a CreateServer request. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusReason</i>  Depending on the server status, this field has either a human-readable message (such as a create or backup error), or an escaped block of JSON (used for health check results). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i>  The server's status. This field displays the states of actions in progress, such as creating, running, or backing up the server, as well as the server's health state. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceRoleArn</i> The service role ARN used to create the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i>  The security group IDs for the server, as specified in the CloudFormation stack. These might not be the same security groups that are shown in the EC2 console. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i> The preferred backup period specified for the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> The preferred maintenance period specified for the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceStatus</i> The status of the most recent server maintenance run. Shows <code>SUCCESS</code> or <code>FAILED</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyPair</i> The key pair associated with the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i>  The instance type for the server, as specified in the CloudFormation stack. This might not be the same instance type that is shown in the EC2 console. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceProfileArn</i> The instance profile ARN of the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> The engine version of the server. For a Chef server, the valid value for EngineVersion is currently <code>12</code>. For a Puppet server, the valid value is <code>2017</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineAttributes</i> The response of a createServer() request returns the master credential to access the server in EngineAttributes. These credentials are not stored by AWS OpsWorks CM; they are returned only as part of the result of createServer().  <p class="title"> <b>Attributes returned in a createServer response for Chef</b>  <ul> <li>  <code>CHEF_AUTOMATE_PIVOTAL_KEY</code>: A base64-encoded RSA private key that is generated by AWS OpsWorks for Chef Automate. This private key is required to access the Chef API. </li> <li>  <code>CHEF_STARTER_KIT</code>: A base64-encoded ZIP file. The ZIP file contains a Chef starter kit, which includes a README, a configuration file, and the required RSA private key. Save this file, unzip it, and then change to the directory where you've unzipped the file contents. From this directory, you can run Knife commands. </li> </ul> <p class="title"> <b>Attributes returned in a createServer response for Puppet</b>  <ul> <li>  <code>PUPPET_STARTER_KIT</code>: A base64-encoded ZIP file. The ZIP file contains a Puppet starter kit, including a README and a required private key. Save this file, unzip it, and then change to the directory where you've unzipped the file contents. </li> <li>  <code>PUPPET_ADMIN_PASSWORD</code>: An administrator password that you can use to sign in to the Puppet Enterprise console after the server is online. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineModel</i> The engine model of the server. Valid values in this release include <code>Monolithic</code> for Puppet and <code>Single</code> for Chef. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> The engine type of the server. Valid values in this release include <code>ChefAutomate</code> and <code>Puppet</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i>  A DNS name that can be used to access the engine. Example: <code>myserver-asdfghjkl.us-east-1.opsworks.io</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisableAutomatedBackup</i> Disables automated backups. The number of stored backups is dependent on the value of PreferredBackupCount. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudFormationStackArn</i> The ARN of the CloudFormation stack that was used to create the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i> Time stamp of server creation. Example <code>2016-07-29T13:38:47.520Z</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerName</i> The name of the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionCount</i> The number of automated backups to keep. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatePublicIpAddress</i> Associate a public IP address with a server that you are launching. <br/>        """
        pass

    def update_server_engine_attributes(self, AttributeName: str, ServerName: str, AttributeValue: str=None) -> UpdateServerEngineAttributesResponse:
        """ Updates engine-specific attributes on a specified server. The server enters the <code>MODIFYING</code> state when this operation is in progress. Only one update can occur at a time. You can use this command to reset a Chef server's public key (<code>CHEF_PIVOTAL_KEY</code>) or a Puppet server's admin password (<code>PUPPET_ADMIN_PASSWORD</code>).   This operation is asynchronous.   This operation can only be called for servers in <code>HEALTHY</code> or <code>UNHEALTHY</code> states. Otherwise, an <code>InvalidStateException</code> is raised. A <code>ResourceNotFoundException</code> is thrown when the server does not exist. A <code>ValidationException</code> is raised when parameters of the request are not valid. 

        :param self:
        :param AttributeValue:  The value to set for the attribute. <br/>AttributeName:  The name of the engine attribute to update. <br/>ServerName:  The name of the server to update. <br/>
        :return: Server:  Contains the response to an <code>UpdateServerEngineAttributes</code> request. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerArn</i> The ARN of the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetIds</i>  The subnet IDs specified in a CreateServer request. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusReason</i>  Depending on the server status, this field has either a human-readable message (such as a create or backup error), or an escaped block of JSON (used for health check results). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i>  The server's status. This field displays the states of actions in progress, such as creating, running, or backing up the server, as well as the server's health state. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceRoleArn</i> The service role ARN used to create the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i>  The security group IDs for the server, as specified in the CloudFormation stack. These might not be the same security groups that are shown in the EC2 console. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i> The preferred backup period specified for the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> The preferred maintenance period specified for the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceStatus</i> The status of the most recent server maintenance run. Shows <code>SUCCESS</code> or <code>FAILED</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyPair</i> The key pair associated with the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i>  The instance type for the server, as specified in the CloudFormation stack. This might not be the same instance type that is shown in the EC2 console. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceProfileArn</i> The instance profile ARN of the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> The engine version of the server. For a Chef server, the valid value for EngineVersion is currently <code>12</code>. For a Puppet server, the valid value is <code>2017</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineAttributes</i> The response of a createServer() request returns the master credential to access the server in EngineAttributes. These credentials are not stored by AWS OpsWorks CM; they are returned only as part of the result of createServer().  <p class="title"> <b>Attributes returned in a createServer response for Chef</b>  <ul> <li>  <code>CHEF_AUTOMATE_PIVOTAL_KEY</code>: A base64-encoded RSA private key that is generated by AWS OpsWorks for Chef Automate. This private key is required to access the Chef API. </li> <li>  <code>CHEF_STARTER_KIT</code>: A base64-encoded ZIP file. The ZIP file contains a Chef starter kit, which includes a README, a configuration file, and the required RSA private key. Save this file, unzip it, and then change to the directory where you've unzipped the file contents. From this directory, you can run Knife commands. </li> </ul> <p class="title"> <b>Attributes returned in a createServer response for Puppet</b>  <ul> <li>  <code>PUPPET_STARTER_KIT</code>: A base64-encoded ZIP file. The ZIP file contains a Puppet starter kit, including a README and a required private key. Save this file, unzip it, and then change to the directory where you've unzipped the file contents. </li> <li>  <code>PUPPET_ADMIN_PASSWORD</code>: An administrator password that you can use to sign in to the Puppet Enterprise console after the server is online. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineModel</i> The engine model of the server. Valid values in this release include <code>Monolithic</code> for Puppet and <code>Single</code> for Chef. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> The engine type of the server. Valid values in this release include <code>ChefAutomate</code> and <code>Puppet</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i>  A DNS name that can be used to access the engine. Example: <code>myserver-asdfghjkl.us-east-1.opsworks.io</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisableAutomatedBackup</i> Disables automated backups. The number of stored backups is dependent on the value of PreferredBackupCount. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudFormationStackArn</i> The ARN of the CloudFormation stack that was used to create the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i> Time stamp of server creation. Example <code>2016-07-29T13:38:47.520Z</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerName</i> The name of the server. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionCount</i> The number of automated backups to keep. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatePublicIpAddress</i> Associate a public IP address with a server that you are launching. <br/>        """
        pass



class Organizations(BaseClient):
    """<fullname>AWS Organizations API Reference</fullname> AWS Organizations is a web service that enables you to consolidate your multiple AWS accounts into an <i>organization</i> and centrally manage your accounts and their resources. This guide provides descriptions of the Organizations API. For more information about using this service, see the <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_introduction.html">AWS Organizations User Guide</a>.  <b>API Version</b>  This version of the Organizations API Reference documents the Organizations API version 2016-11-28. <note> As an alternative to using the API directly, you can use one of the AWS SDKs, which consist of libraries and sample code for various programming languages and platforms (Java, Ruby, .NET, iOS, Android, and more). The SDKs provide a convenient way to create programmatic access to AWS Organizations. For example, the SDKs take care of cryptographically signing requests, managing errors, and retrying requests automatically. For more information about the AWS SDKs, including how to download and install them, see <a href="http://aws.amazon.com/tools/">Tools for Amazon Web Services</a>. </note> We recommend that you use the AWS SDKs to make programmatic API calls to Organizations. However, you also can use the Organizations Query API to make direct calls to the Organizations web service. To learn more about the Organizations Query API, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_query-requests.html">Making Query Requests</a> in the <i>AWS Organizations User Guide</i>. Organizations supports GET and POST requests for all actions. That is, the API does not require you to use GET for some actions and POST for others. However, GET requests are subject to the limitation size of a URL. Therefore, for operations that require larger sizes, use a POST request.  <b>Signing Requests</b>  When you send HTTP requests to AWS, you must sign the requests so that AWS can identify who sent them. You sign requests with your AWS access key, which consists of an access key ID and a secret access key. We strongly recommend that you do not create an access key for your root account. Anyone who has the access key for your root account has unrestricted access to all the resources in your account. Instead, create an access key for an IAM user account that has administrative privileges. As another option, use AWS Security Token Service to generate temporary security credentials, and use those credentials to sign requests.  To sign requests, we recommend that you use <a href="https://docs.aws.amazon.com/general/latest/gr/signature-version-4.html">Signature Version 4</a>. If you have an existing application that uses Signature Version 2, you do not have to update it to use Signature Version 4. However, some operations now require Signature Version 4. The documentation for operations that require version 4 indicate this requirement.  When you use the AWS Command Line Interface (AWS CLI) or one of the AWS SDKs to make requests to AWS, these tools automatically sign the requests for you with the access key that you specify when you configure the tools. In this release, each organization can have only one root. In a future release, a single organization will support multiple roots.  <b>Support and Feedback for AWS Organizations</b>  We welcome your feedback. Send your comments to <a href="mailto:feedback-awsorganizations@amazon.com">feedback-awsorganizations@amazon.com</a> or post your feedback and questions in the <a href="http://forums.aws.amazon.com/forum.jspa?forumID=219">AWS Organizations support forum</a>. For more information about the AWS support forums, see <a href="http://forums.aws.amazon.com/help.jspa">Forums Help</a>.  <b>Endpoint to Call When Using the CLI or the AWS API</b>  For the current release of Organizations, you must specify the <code>us-east-1</code> region for all AWS API and CLI calls. You can do this in the CLI by using these parameters and commands: <ul> <li> Use the following parameter with each command to specify both the endpoint and its region:  <code>--endpoint-url https://organizations.us-east-1.amazonaws.com</code>  </li> <li> Use the default endpoint, but configure your default region with this command:  <code>aws configure set default.region us-east-1</code>  </li> <li> Use the following parameter with each command to specify the endpoint:  <code>--region us-east-1</code>  </li> </ul> For the various SDKs used to call the APIs, see the documentation for the SDK of interest to learn how to direct the requests to a specific endpoint. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/rande.html#sts_region">Regions and Endpoints</a> in the <i>AWS General Reference</i>.   <b>How examples are presented</b>  The JSON returned by the AWS Organizations service as response to your requests is returned as a single long string without line breaks or formatting whitespace. Both line breaks and whitespace are included in the examples in this guide to improve readability. When example input parameters also would result in long strings that would extend beyond the screen, we insert line breaks to enhance readability. You should always submit the input as a single JSON text string.  <b>Recording API Requests</b>  AWS Organizations supports AWS CloudTrail, a service that records AWS API calls for your AWS account and delivers log files to an Amazon S3 bucket. By using information collected by AWS CloudTrail, you can determine which requests were successfully made to Organizations, who made the request, when it was made, and so on. For more about AWS Organizations and its support for AWS CloudTrail, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_monitoring.html#orgs_cloudtrail-integration">Logging AWS Organizations Events with AWS CloudTrail</a> in the <i>AWS Organizations User Guide</i>. To learn more about CloudTrail, including how to turn it on and find your log files, see the <a href="https://docs.aws.amazon.com/awscloudtrail/latest/userguide/what_is_cloud_trail_top_level.html">AWS CloudTrail User Guide</a>."""


    class ListAccountsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_accounts` for valid parameters"""
            pass
    class ListAccountsForParentPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_accounts_for_parent` for valid parameters"""
            pass
    class ListChildrenPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_children` for valid parameters"""
            pass
    class ListCreateAccountStatusPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_create_account_status` for valid parameters"""
            pass
    class ListHandshakesForAccountPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_handshakes_for_account` for valid parameters"""
            pass
    class ListHandshakesForOrganizationPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_handshakes_for_organization` for valid parameters"""
            pass
    class ListOrganizationalUnitsForParentPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_organizational_units_for_parent` for valid parameters"""
            pass
    class ListParentsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_parents` for valid parameters"""
            pass
    class ListPoliciesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_policies` for valid parameters"""
            pass
    class ListPoliciesForTargetPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_policies_for_target` for valid parameters"""
            pass
    class ListRootsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_roots` for valid parameters"""
            pass
    class ListTargetsForPolicyPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_targets_for_policy` for valid parameters"""
            pass
    class ListAWSServiceAccessForOrganizationPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_aws_service_access_for_organization` for valid parameters"""
            pass
    class ListTagsForResourcePaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_tags_for_resource` for valid parameters"""
            pass


    class AcceptHandshakeRequest(Mapping):
        pass
    
    class AcceptHandshakeResponse(Mapping):
        pass
    
    class AttachPolicyRequest(Mapping):
        pass
    
    class CancelHandshakeRequest(Mapping):
        pass
    
    class CancelHandshakeResponse(Mapping):
        pass
    
    class CreateAccountRequest(Mapping):
        pass
    
    class CreateAccountResponse(Mapping):
        pass
    
    class CreateGovCloudAccountRequest(Mapping):
        pass
    
    class CreateGovCloudAccountResponse(Mapping):
        pass
    
    class CreateOrganizationRequest(Mapping):
        pass
    
    class CreateOrganizationResponse(Mapping):
        pass
    
    class CreateOrganizationalUnitRequest(Mapping):
        pass
    
    class CreateOrganizationalUnitResponse(Mapping):
        pass
    
    class CreatePolicyRequest(Mapping):
        pass
    
    class CreatePolicyResponse(Mapping):
        pass
    
    class DeclineHandshakeRequest(Mapping):
        pass
    
    class DeclineHandshakeResponse(Mapping):
        pass
    
    class DeleteOrganizationalUnitRequest(Mapping):
        pass
    
    class DeletePolicyRequest(Mapping):
        pass
    
    class DescribeAccountRequest(Mapping):
        pass
    
    class DescribeAccountResponse(Mapping):
        pass
    
    class DescribeCreateAccountStatusRequest(Mapping):
        pass
    
    class DescribeCreateAccountStatusResponse(Mapping):
        pass
    
    class DescribeHandshakeRequest(Mapping):
        pass
    
    class DescribeHandshakeResponse(Mapping):
        pass
    
    class DescribeOrganizationResponse(Mapping):
        pass
    
    class DescribeOrganizationalUnitRequest(Mapping):
        pass
    
    class DescribeOrganizationalUnitResponse(Mapping):
        pass
    
    class DescribePolicyRequest(Mapping):
        pass
    
    class DescribePolicyResponse(Mapping):
        pass
    
    class DetachPolicyRequest(Mapping):
        pass
    
    class DisableAWSServiceAccessRequest(Mapping):
        pass
    
    class DisablePolicyTypeRequest(Mapping):
        pass
    
    class DisablePolicyTypeResponse(Mapping):
        pass
    
    class EnableAWSServiceAccessRequest(Mapping):
        pass
    
    class EnableAllFeaturesRequest(Mapping):
        pass
    
    class EnableAllFeaturesResponse(Mapping):
        pass
    
    class EnablePolicyTypeRequest(Mapping):
        pass
    
    class EnablePolicyTypeResponse(Mapping):
        pass
    
    class InviteAccountToOrganizationRequest(Mapping):
        pass
    
    class InviteAccountToOrganizationResponse(Mapping):
        pass
    
    class HandshakeParty(Mapping):
        pass
    
    class ListAWSServiceAccessForOrganizationRequest(Mapping):
        pass
    
    class ListAWSServiceAccessForOrganizationResponse(Mapping):
        pass
    
    class ListAccountsRequest(Mapping):
        pass
    
    class ListAccountsResponse(Mapping):
        pass
    
    class ListAccountsForParentRequest(Mapping):
        pass
    
    class ListAccountsForParentResponse(Mapping):
        pass
    
    class ListChildrenRequest(Mapping):
        pass
    
    class ListChildrenResponse(Mapping):
        pass
    
    class ListCreateAccountStatusRequest(Mapping):
        pass
    
    class ListCreateAccountStatusResponse(Mapping):
        pass
    
    class CreateAccountState(object):
        pass
    
    class ListHandshakesForAccountRequest(Mapping):
        pass
    
    class ListHandshakesForAccountResponse(Mapping):
        pass
    
    class HandshakeFilter(Mapping):
        pass
    
    class ListHandshakesForOrganizationRequest(Mapping):
        pass
    
    class ListHandshakesForOrganizationResponse(Mapping):
        pass
    
    class ListOrganizationalUnitsForParentRequest(Mapping):
        pass
    
    class ListOrganizationalUnitsForParentResponse(Mapping):
        pass
    
    class ListParentsRequest(Mapping):
        pass
    
    class ListParentsResponse(Mapping):
        pass
    
    class ListPoliciesRequest(Mapping):
        pass
    
    class ListPoliciesResponse(Mapping):
        pass
    
    class ListPoliciesForTargetRequest(Mapping):
        pass
    
    class ListPoliciesForTargetResponse(Mapping):
        pass
    
    class ListRootsRequest(Mapping):
        pass
    
    class ListRootsResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class ListTargetsForPolicyRequest(Mapping):
        pass
    
    class ListTargetsForPolicyResponse(Mapping):
        pass
    
    class MoveAccountRequest(Mapping):
        pass
    
    class RemoveAccountFromOrganizationRequest(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateOrganizationalUnitRequest(Mapping):
        pass
    
    class UpdateOrganizationalUnitResponse(Mapping):
        pass
    
    class UpdatePolicyRequest(Mapping):
        pass
    
    class UpdatePolicyResponse(Mapping):
        pass
    
    def accept_handshake(self, HandshakeId: str) -> AcceptHandshakeResponse:
        """Sends a response to the originator of a handshake agreeing to the action proposed by the handshake request.  This operation can be called only by the following principals when they also have the relevant IAM permissions: <ul> <li>  <b>Invitation to join</b> or <b>Approve all features request</b> handshakes: only a principal from the member account.  The user who calls the API for an invitation to join must have the <code>organizations:AcceptHandshake</code> permission. If you enabled all features in the organization, the user must also have the <code>iam:CreateServiceLinkedRole</code> permission so that AWS Organizations can create the required service-linked role named <code>AWSServiceRoleForOrganizations</code>. For more information, see <a href="http://docs.aws.amazon.com/organizations/latest/userguide/orgs_integration_services.html#orgs_integration_service-linked-roles">AWS Organizations and Service-Linked Roles</a> in the <i>AWS Organizations User Guide</i>. </li> <li>  <b>Enable all features final confirmation</b> handshake: only a principal from the master account. For more information about invitations, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_manage_accounts_invites.html">Inviting an AWS Account to Join Your Organization</a> in the <i>AWS Organizations User Guide.</i> For more information about requests to enable all features in the organization, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_manage_org_support-all-features.html">Enabling All Features in Your Organization</a> in the <i>AWS Organizations User Guide.</i>  </li> </ul> After you accept a handshake, it continues to appear in the results of relevant APIs for only 30 days. After that, it's deleted.

        :param self:
        :param HandshakeId:  The unique identifier (ID) of the handshake that you want to accept. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for handshake ID string requires "h-" followed by from 8 to 32 lower-case letters or digits.<br/>
        :return: Handshake:  A structure that contains details about the accepted handshake.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Resources</i> Additional information that is needed to process the handshake.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Action</i> The type of handshake, indicating what action occurs when the recipient accepts the handshake. The following handshake types are supported: <ul> <li>  <b>INVITE</b>: This type of handshake represents a request to join an organization. It is always sent from the master account to only non-member accounts. </li> <li>  <b>ENABLE_ALL_FEATURES</b>: This type of handshake represents a request to enable all features in an organization. It is always sent from the master account to only <i>invited</i> member accounts. Created accounts do not receive this because those accounts were created by the organization's master account and approval is inferred. </li> <li>  <b>APPROVE_ALL_FEATURES</b>: This type of handshake is sent from the Organizations service when all member accounts have approved the <code>ENABLE_ALL_FEATURES</code> invitation. It is sent only to the master account and signals the master that it can finalize the process to enable all features. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpirationTimestamp</i> The date and time that the handshake expires. If the recipient of the handshake request fails to respond before the specified date and time, the handshake becomes inactive and is no longer valid.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestedTimestamp</i> The date and time that the handshake request was made.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The current state of the handshake. Use the state to trace the flow of the handshake through the process from its creation to its acceptance. The meaning of each of the valid values is as follows: <ul> <li>  <b>REQUESTED</b>: This handshake was sent to multiple recipients (applicable to only some handshake types) and not all recipients have responded yet. The request stays in this state until all recipients respond. </li> <li>  <b>OPEN</b>: This handshake was sent to multiple recipients (applicable to only some policy types) and all recipients have responded, allowing the originator to complete the handshake action. </li> <li>  <b>CANCELED</b>: This handshake is no longer active because it was canceled by the originating account. </li> <li>  <b>ACCEPTED</b>: This handshake is complete because it has been accepted by the recipient. </li> <li>  <b>DECLINED</b>: This handshake is no longer active because it was declined by the recipient account. </li> <li>  <b>EXPIRED</b>: This handshake is no longer active because the originator did not receive a response of any kind from the recipient before the expiration time (15 days). </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parties</i> Information about the two accounts that are participating in the handshake.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of a handshake. For more information about ARNs in Organizations, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_permissions.html#orgs-permissions-arns">ARN Formats Supported by Organizations</a> in the <i>AWS Organizations User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier (ID) of a handshake. The originating account creates the ID when it initiates the handshake. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for handshake ID string requires "h-" followed by from 8 to 32 lower-case letters or digits.<br/>        """
        pass

    def attach_policy(self, TargetId: str, PolicyId: str):
        """Attaches a policy to a root, an organizational unit (OU), or an individual account. How the policy affects accounts depends on the type of policy: <ul> <li>  <b>Service control policy (SCP)</b> - An SCP specifies what permissions can be delegated to users in affected member accounts. The scope of influence for a policy depends on what you attach the policy to: <ul> <li> If you attach an SCP to a root, it affects all accounts in the organization </li> <li> If you attach an SCP to an OU, it affects all accounts in that OU and in any child OUs </li> <li> If you attach the policy directly to an account, it affects only that account </li> </ul> SCPs are JSON policies that specify the maximum permissions for an organization or organizational unit (OU). When you attach one SCP to a higher level root or OU, and you also attach a different SCP to a child OU or to an account, the child policy can further restrict only the permissions that pass through the parent filter and are available to the child. An SCP that is attached to a child can't grant a permission that the paren't hasn't already granted. For example, imagine that the parent SCP allows permissions A, B, C, D, and E. The child SCP allows C, D, E, F, and G. The result is that the accounts affected by the child SCP are allowed to use only C, D, and E. They can't use A or B because the child OU filtered them out. They also can't use F and G because the parent OU filtered them out. They can't be granted back by the child SCP; child SCPs can only filter the permissions they receive from the parent SCP. AWS Organizations attaches a default SCP named <code>"FullAWSAccess</code> to every root, OU, and account. This default SCP allows all services and actions, enabling any new child OU or account to inherit the permissions of the parent root or OU. If you detach the default policy, you must replace it with a policy that specifies the permissions that you want to allow in that OU or account. For more information about how AWS Organizations policies permissions work, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_manage_policies_scp.html">Using Service Control Policies</a> in the <i>AWS Organizations User Guide.</i>  </li> </ul> This operation can be called only from the organization's master account.

        :param self:
        :param TargetId:  The unique identifier (ID) of the root, OU, or account that you want to attach the policy to. You can get the ID by calling the <a>ListRoots</a>, <a>ListOrganizationalUnitsForParent</a>, or <a>ListAccounts</a> operations. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for a target ID string requires one of the following: <ul> <li> Root: a string that begins with "r-" followed by from 4 to 32 lower-case letters or digits. </li> <li> Account: a string that consists of exactly 12 digits. </li> <li> Organizational unit (OU): a string that begins with "ou-" followed by from 4 to 32 lower-case letters or digits (the ID of the root that the OU is in) followed by a second "-" dash and from 8 to 32 additional lower-case letters or digits. </li> </ul><br/>PolicyId:  The unique identifier (ID) of the policy that you want to attach to the target. You can get the ID for the policy by calling the <a>ListPolicies</a> operation. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for a policy ID string requires "p-" followed by from 8 to 128 lower-case letters or digits.<br/>
        :return:         """
        pass

    def cancel_handshake(self, HandshakeId: str) -> CancelHandshakeResponse:
        """Cancels a handshake. Canceling a handshake sets the handshake state to <code>CANCELED</code>.  This operation can be called only from the account that originated the handshake. The recipient of the handshake can't cancel it, but can use <a>DeclineHandshake</a> instead. After a handshake is canceled, the recipient can no longer respond to that handshake. After you cancel a handshake, it continues to appear in the results of relevant APIs for only 30 days. After that, it's deleted.

        :param self:
        :param HandshakeId:  The unique identifier (ID) of the handshake that you want to cancel. You can get the ID from the <a>ListHandshakesForOrganization</a> operation. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for handshake ID string requires "h-" followed by from 8 to 32 lower-case letters or digits.<br/>
        :return: Handshake:  A structure that contains details about the handshake that you canceled.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Resources</i> Additional information that is needed to process the handshake.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Action</i> The type of handshake, indicating what action occurs when the recipient accepts the handshake. The following handshake types are supported: <ul> <li>  <b>INVITE</b>: This type of handshake represents a request to join an organization. It is always sent from the master account to only non-member accounts. </li> <li>  <b>ENABLE_ALL_FEATURES</b>: This type of handshake represents a request to enable all features in an organization. It is always sent from the master account to only <i>invited</i> member accounts. Created accounts do not receive this because those accounts were created by the organization's master account and approval is inferred. </li> <li>  <b>APPROVE_ALL_FEATURES</b>: This type of handshake is sent from the Organizations service when all member accounts have approved the <code>ENABLE_ALL_FEATURES</code> invitation. It is sent only to the master account and signals the master that it can finalize the process to enable all features. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpirationTimestamp</i> The date and time that the handshake expires. If the recipient of the handshake request fails to respond before the specified date and time, the handshake becomes inactive and is no longer valid.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestedTimestamp</i> The date and time that the handshake request was made.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The current state of the handshake. Use the state to trace the flow of the handshake through the process from its creation to its acceptance. The meaning of each of the valid values is as follows: <ul> <li>  <b>REQUESTED</b>: This handshake was sent to multiple recipients (applicable to only some handshake types) and not all recipients have responded yet. The request stays in this state until all recipients respond. </li> <li>  <b>OPEN</b>: This handshake was sent to multiple recipients (applicable to only some policy types) and all recipients have responded, allowing the originator to complete the handshake action. </li> <li>  <b>CANCELED</b>: This handshake is no longer active because it was canceled by the originating account. </li> <li>  <b>ACCEPTED</b>: This handshake is complete because it has been accepted by the recipient. </li> <li>  <b>DECLINED</b>: This handshake is no longer active because it was declined by the recipient account. </li> <li>  <b>EXPIRED</b>: This handshake is no longer active because the originator did not receive a response of any kind from the recipient before the expiration time (15 days). </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parties</i> Information about the two accounts that are participating in the handshake.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of a handshake. For more information about ARNs in Organizations, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_permissions.html#orgs-permissions-arns">ARN Formats Supported by Organizations</a> in the <i>AWS Organizations User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier (ID) of a handshake. The originating account creates the ID when it initiates the handshake. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for handshake ID string requires "h-" followed by from 8 to 32 lower-case letters or digits.<br/>        """
        pass

    def create_account(self, AccountName: str, Email: str, RoleName: str=None, IamUserAccessToBilling: str=None) -> CreateAccountResponse:
        """Creates an AWS account that is automatically a member of the organization whose credentials made the request. This is an asynchronous request that AWS performs in the background. Because <code>CreateAccount</code> operates asynchronously, it can return a successful completion message even though account initialization might still be in progress. You might need to wait a few minutes before you can successfully access the account. To check the status of the request, do one of the following: <ul> <li> Use the <code>OperationId</code> response element from this operation to provide as a parameter to the <a>DescribeCreateAccountStatus</a> operation. </li> <li> Check the AWS CloudTrail log for the <code>CreateAccountResult</code> event. For information on using AWS CloudTrail with AWS Organizations, see <a href="http://docs.aws.amazon.com/organizations/latest/userguide/orgs_monitoring.html">Monitoring the Activity in Your Organization</a> in the <i>AWS Organizations User Guide.</i>  </li> </ul> <p/> The user who calls the API to create an account must have the <code>organizations:CreateAccount</code> permission. If you enabled all features in the organization, AWS Organizations will create the required service-linked role named <code>AWSServiceRoleForOrganizations</code>. For more information, see <a href="http://docs.aws.amazon.com/organizations/latest/userguide/orgs_integrate_services.html#orgs_integrate_services-using_slrs">AWS Organizations and Service-Linked Roles</a> in the <i>AWS Organizations User Guide</i>. AWS Organizations preconfigures the new member account with a role (named <code>OrganizationAccountAccessRole</code> by default) that grants users in the master account administrator permissions in the new member account. Principals in the master account can assume the role. AWS Organizations clones the company name and address information for the new account from the organization's master account. This operation can be called only from the organization's master account. For more information about creating accounts, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_manage_accounts_create.html">Creating an AWS Account in Your Organization</a> in the <i>AWS Organizations User Guide.</i>  <important> <ul> <li> When you create an account in an organization using the AWS Organizations console, API, or CLI commands, the information required for the account to operate as a standalone account, such as a payment method and signing the end user license agreement (EULA) is <i>not</i> automatically collected. If you must remove an account from your organization later, you can do so only after you provide the missing information. Follow the steps at <a href="http://docs.aws.amazon.com/organizations/latest/userguide/orgs_manage_accounts_remove.html#leave-without-all-info"> To leave an organization as a member account</a> in the <i>AWS Organizations User Guide</i>. </li> <li> If you get an exception that indicates that you exceeded your account limits for the organization, contact <a href="https://console.aws.amazon.com/support/home#/">AWS Support</a>. </li> <li> If you get an exception that indicates that the operation failed because your organization is still initializing, wait one hour and then try again. If the error persists, contact <a href="https://console.aws.amazon.com/support/home#/">AWS Support</a>. </li> <li> Using <code>CreateAccount</code> to create multiple temporary accounts isn't recommended. You can only close an account from the Billing and Cost Management Console, and you must be signed in as the root user. For information on the requirements and process for closing an account, see <a href="http://docs.aws.amazon.com/organizations/latest/userguide/orgs_manage_accounts_close.html">Closing an AWS Account</a> in the <i>AWS Organizations User Guide</i>. </li> </ul> </important> <note> When you create a member account with this operation, you can choose whether to create the account with the <b>IAM User and Role Access to Billing Information</b> switch enabled. If you enable it, IAM users and roles that have appropriate permissions can view billing information for the account. If you disable it, only the account root user can access billing information. For information about how to disable this switch for an account, see <a href="https://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/grantaccess.html">Granting Access to Your Billing Information and Tools</a>. </note>

        :param self:
        :param IamUserAccessToBilling:  If set to <code>ALLOW</code>, the new account enables IAM users to access account billing information <i>if</i> they have the required permissions. If set to <code>DENY</code>, only the root user of the new account can access account billing information. For more information, see <a href="https://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/grantaccess.html#ControllingAccessWebsite-Activate">Activating Access to the Billing and Cost Management Console</a> in the <i>AWS Billing and Cost Management User Guide</i>. If you don't specify this parameter, the value defaults to <code>ALLOW</code>, and IAM users and roles with the required permissions can access billing information for the new account.<br/>RoleName:  (Optional) The name of an IAM role that AWS Organizations automatically preconfigures in the new member account. This role trusts the master account, allowing users in the master account to assume the role, as permitted by the master account administrator. The role has administrator permissions in the new member account. If you don't specify this parameter, the role name defaults to <code>OrganizationAccountAccessRole</code>. For more information about how to use this role to access the member account, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_manage_accounts_access.html#orgs_manage_accounts_create-cross-account-role">Accessing and Administering the Member Accounts in Your Organization</a> in the <i>AWS Organizations User Guide</i>, and steps 2 and 3 in <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/tutorial_cross-account-with-roles.html">Tutorial: Delegate Access Across AWS Accounts Using IAM Roles</a> in the <i>IAM User Guide.</i>  The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> that is used to validate this parameter is a string of characters that can consist of uppercase letters, lowercase letters, digits with no spaces, and any of the following characters: =,.@-<br/>AccountName:  The friendly name of the member account.<br/>Email:  The email address of the owner to assign to the new member account. This email address must not already be associated with another AWS account. You must use a valid email address to complete account creation. You can't access the root user of the account or remove an account that was created with an invalid email address.<br/>
        :return: CreateAccountStatus:  A structure that contains details about the request to create an account. This response structure might not be fully populated when you first receive it because account creation is an asynchronous process. You can pass the returned <code>CreateAccountStatus</code> ID as a parameter to <a>DescribeCreateAccountStatus</a> to get status about the progress of the request at later times. You can also check the AWS CloudTrail log for the <code>CreateAccountResult</code> event. For more information, see <a href="http://docs.aws.amazon.com/organizations/latest/userguide/orgs_monitoring.html">Monitoring the Activity in Your Organization</a> in the <i>AWS Organizations User Guide</i>.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureReason</i> If the request failed, a description of the reason for the failure. <ul> <li> ACCOUNT_LIMIT_EXCEEDED: The account could not be created because you have reached the limit on the number of accounts in your organization. </li> <li> EMAIL_ALREADY_EXISTS: The account could not be created because another AWS account with that email address already exists. </li> <li> INVALID_ADDRESS: The account could not be created because the address you provided is not valid. </li> <li> INVALID_EMAIL: The account could not be created because the email address you provided is not valid. </li> <li> INTERNAL_FAILURE: The account could not be created because of an internal failure. Try again later. If the problem persists, contact Customer Support. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GovCloudAccountId</i> If the account was created successfully, the unique identifier (ID) of the new account in the AWS GovCloud (US) Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountId</i> If the account was created successfully, the unique identifier (ID) of the new account. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for an account ID string requires exactly 12 digits.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompletedTimestamp</i> The date and time that the account was created and the request completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestedTimestamp</i> The date and time that the request was made for the account creation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The status of the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountName</i> The account name given to the account when it was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier (ID) that references this request. You get this value from the response of the initial <a>CreateAccount</a> request to create the account. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for an create account request ID string requires "car-" followed by from 8 to 32 lower-case letters or digits.<br/>        """
        pass

    def create_gov_cloud_account(self, AccountName: str, Email: str, RoleName: str=None, IamUserAccessToBilling: str=None) -> CreateGovCloudAccountResponse:
        """This action is available if all of the following are true: <ul> <li> You're authorized to create accounts in the AWS GovCloud (US) Region. For more information on the AWS GovCloud (US) Region, see the <a href="http://docs.aws.amazon.com/govcloud-us/latest/UserGuide/welcome.html"> <i>AWS GovCloud User Guide</i>.</a>  </li> <li> You already have an account in the AWS GovCloud (US) Region that is associated with your master account in the commercial Region.  </li> <li> You call this action from the master account of your organization in the commercial Region. </li> <li> You have the <code>organizations:CreateGovCloudAccount</code> permission. AWS Organizations creates the required service-linked role named <code>AWSServiceRoleForOrganizations</code>. For more information, see <a href="http://docs.aws.amazon.com/organizations/latest/userguide/orgs_integrate_services.html#orgs_integrate_services-using_slrs">AWS Organizations and Service-Linked Roles</a> in the <i>AWS Organizations User Guide.</i>  </li> </ul> AWS automatically enables AWS CloudTrail for AWS GovCloud (US) accounts, but you should also do the following: <ul> <li> Verify that AWS CloudTrail is enabled to store logs. </li> <li> Create an S3 bucket for AWS CloudTrail log storage. For more information, see <a href="http://docs.aws.amazon.com/govcloud-us/latest/UserGuide/verifying-cloudtrail.html">Verifying AWS CloudTrail Is Enabled</a> in the <i>AWS GovCloud User Guide</i>.  </li> </ul> You call this action from the master account of your organization in the commercial Region to create a standalone AWS account in the AWS GovCloud (US) Region. After the account is created, the master account of an organization in the AWS GovCloud (US) Region can invite it to that organization. For more information on inviting standalone accounts in the AWS GovCloud (US) to join an organization, see <a href="http://docs.aws.amazon.com/govcloud-us/latest/UserGuide/govcloud-organizations.html">AWS Organizations</a> in the <i>AWS GovCloud User Guide.</i>  Calling <code>CreateGovCloudAccount</code> is an asynchronous request that AWS performs in the background. Because <code>CreateGovCloudAccount</code> operates asynchronously, it can return a successful completion message even though account initialization might still be in progress. You might need to wait a few minutes before you can successfully access the account. To check the status of the request, do one of the following: <ul> <li> Use the <code>OperationId</code> response element from this operation to provide as a parameter to the <a>DescribeCreateAccountStatus</a> operation. </li> <li> Check the AWS CloudTrail log for the <code>CreateAccountResult</code> event. For information on using AWS CloudTrail with Organizations, see <a href="http://docs.aws.amazon.com/organizations/latest/userguide/orgs_monitoring.html">Monitoring the Activity in Your Organization</a> in the <i>AWS Organizations User Guide.</i>  </li> </ul> <p/> When you call the <code>CreateGovCloudAccount</code> action, you create two accounts: a standalone account in the AWS GovCloud (US) Region and an associated account in the commercial Region for billing and support purposes. The account in the commercial Region is automatically a member of the organization whose credentials made the request. Both accounts are associated with the same email address. A role is created in the new account in the commercial Region that allows the master account in the organization in the commercial Region to assume it. An AWS GovCloud (US) account is then created and associated with the commercial account that you just created. A role is created in the new AWS GovCloud (US) account that can be assumed by the AWS GovCloud (US) account that is associated with the master account of the commercial organization. For more information and to view a diagram that explains how account access works, see <a href="http://docs.aws.amazon.com/govcloud-us/latest/UserGuide/govcloud-organizations.html">AWS Organizations</a> in the <i>AWS GovCloud User Guide.</i>  For more information about creating accounts, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_manage_accounts_create.html">Creating an AWS Account in Your Organization</a> in the <i>AWS Organizations User Guide.</i>  <important> <ul> <li> When you create an account in an organization using the AWS Organizations console, API, or CLI commands, the information required for the account to operate as a standalone account, such as a payment method and signing the end user license agreement (EULA) is <i>not</i> automatically collected. If you must remove an account from your organization later, you can do so only after you provide the missing information. Follow the steps at <a href="http://docs.aws.amazon.com/organizations/latest/userguide/orgs_manage_accounts_remove.html#leave-without-all-info"> To leave an organization as a member account</a> in the <i>AWS Organizations User Guide.</i>  </li> <li> If you get an exception that indicates that you exceeded your account limits for the organization, contact <a href="https://console.aws.amazon.com/support/home#/">AWS Support</a>. </li> <li> If you get an exception that indicates that the operation failed because your organization is still initializing, wait one hour and then try again. If the error persists, contact <a href="https://console.aws.amazon.com/support/home#/">AWS Support</a>. </li> <li> Using <code>CreateGovCloudAccount</code> to create multiple temporary accounts isn't recommended. You can only close an account from the AWS Billing and Cost Management console, and you must be signed in as the root user. For information on the requirements and process for closing an account, see <a href="http://docs.aws.amazon.com/organizations/latest/userguide/orgs_manage_accounts_close.html">Closing an AWS Account</a> in the <i>AWS Organizations User Guide</i>. </li> </ul> </important> <note> When you create a member account with this operation, you can choose whether to create the account with the <b>IAM User and Role Access to Billing Information</b> switch enabled. If you enable it, IAM users and roles that have appropriate permissions can view billing information for the account. If you disable it, only the account root user can access billing information. For information about how to disable this switch for an account, see <a href="https://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/grantaccess.html">Granting Access to Your Billing Information and Tools</a>. </note>

        :param self:
        :param IamUserAccessToBilling:  If set to <code>ALLOW</code>, the new linked account in the commercial Region enables IAM users to access account billing information <i>if</i> they have the required permissions. If set to <code>DENY</code>, only the root user of the new account can access account billing information. For more information, see <a href="https://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/grantaccess.html#ControllingAccessWebsite-Activate">Activating Access to the Billing and Cost Management Console</a> in the <i>AWS Billing and Cost Management User Guide.</i>  If you don't specify this parameter, the value defaults to <code>ALLOW</code>, and IAM users and roles with the required permissions can access billing information for the new account.<br/>RoleName:  (Optional) The name of an IAM role that AWS Organizations automatically preconfigures in the new member accounts in both the AWS GovCloud (US) Region and in the commercial Region. This role trusts the master account, allowing users in the master account to assume the role, as permitted by the master account administrator. The role has administrator permissions in the new member account. If you don't specify this parameter, the role name defaults to <code>OrganizationAccountAccessRole</code>. For more information about how to use this role to access the member account, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_manage_accounts_access.html#orgs_manage_accounts_create-cross-account-role">Accessing and Administering the Member Accounts in Your Organization</a> in the <i>AWS Organizations User Guide</i> and steps 2 and 3 in <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/tutorial_cross-account-with-roles.html">Tutorial: Delegate Access Across AWS Accounts Using IAM Roles</a> in the <i>IAM User Guide.</i>  The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> that is used to validate this parameter is a string of characters that can consist of uppercase letters, lowercase letters, digits with no spaces, and any of the following characters: =,.@-<br/>AccountName:  The friendly name of the member account.<br/>Email:  The email address of the owner to assign to the new member account in the commercial Region. This email address must not already be associated with another AWS account. You must use a valid email address to complete account creation. You can't access the root user of the account or remove an account that was created with an invalid email address. Like all request parameters for <code>CreateGovCloudAccount</code>, the request for the email address for the AWS GovCloud (US) account originates from the commercial Region, not from the AWS GovCloud (US) Region.<br/>
        :return: CreateAccountStatus:  Contains the status about a <a>CreateAccount</a> or <a>CreateGovCloudAccount</a> request to create an AWS account or an AWS GovCloud (US) account in an organization.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureReason</i> If the request failed, a description of the reason for the failure. <ul> <li> ACCOUNT_LIMIT_EXCEEDED: The account could not be created because you have reached the limit on the number of accounts in your organization. </li> <li> EMAIL_ALREADY_EXISTS: The account could not be created because another AWS account with that email address already exists. </li> <li> INVALID_ADDRESS: The account could not be created because the address you provided is not valid. </li> <li> INVALID_EMAIL: The account could not be created because the email address you provided is not valid. </li> <li> INTERNAL_FAILURE: The account could not be created because of an internal failure. Try again later. If the problem persists, contact Customer Support. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GovCloudAccountId</i> If the account was created successfully, the unique identifier (ID) of the new account in the AWS GovCloud (US) Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountId</i> If the account was created successfully, the unique identifier (ID) of the new account. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for an account ID string requires exactly 12 digits.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompletedTimestamp</i> The date and time that the account was created and the request completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestedTimestamp</i> The date and time that the request was made for the account creation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The status of the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountName</i> The account name given to the account when it was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier (ID) that references this request. You get this value from the response of the initial <a>CreateAccount</a> request to create the account. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for an create account request ID string requires "car-" followed by from 8 to 32 lower-case letters or digits.<br/>        """
        pass

    def create_organization(self, FeatureSet: str=None) -> CreateOrganizationResponse:
        """Creates an AWS organization. The account whose user is calling the <code>CreateOrganization</code> operation automatically becomes the <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/orgs_getting-started_concepts.html#account">master account</a> of the new organization. This operation must be called using credentials from the account that is to become the new organization's master account. The principal must also have the relevant IAM permissions. By default (or if you set the <code>FeatureSet</code> parameter to <code>ALL</code>), the new organization is created with all features enabled and service control policies automatically enabled in the root. If you instead choose to create the organization supporting only the consolidated billing features by setting the <code>FeatureSet</code> parameter to <code>CONSOLIDATED_BILLING"</code>, no policy types are enabled by default, and you can't use organization policies.

        :param self:
        :param FeatureSet:  Specifies the feature set supported by the new organization. Each feature set supports different levels of functionality. <ul> <li>  <code>CONSOLIDATED_BILLING</code>: All member accounts have their bills consolidated to and paid by the master account. For more information, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_getting-started_concepts.html#feature-set-cb-only">Consolidated billing</a> in the <i>AWS Organizations User Guide.</i>   The consolidated billing feature subset isn't available for organizations in the AWS GovCloud (US) Region. </li> <li>  <code>ALL</code>: In addition to all the features supported by the consolidated billing feature set, the master account can also apply any type of policy to any member account in the organization. For more information, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_getting-started_concepts.html#feature-set-all">All features</a> in the <i>AWS Organizations User Guide.</i>  </li> </ul><br/>
        :return: Organization:  A structure that contains details about the newly created organization.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailablePolicyTypes</i> A list of policy types that are enabled for this organization. For example, if your organization has all features enabled, then service control policies (SCPs) are included in the list. <note> Even if a policy type is shown as available in the organization, you can separately enable and disable them at the root level by using <a>EnablePolicyType</a> and <a>DisablePolicyType</a>. Use <a>ListRoots</a> to see the status of a policy type in that root. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterAccountEmail</i> The email address that is associated with the AWS account that is designated as the master account for the organization.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterAccountId</i> The unique identifier (ID) of the master account of an organization. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for an account ID string requires exactly 12 digits.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterAccountArn</i> The Amazon Resource Name (ARN) of the account that is designated as the master account for the organization. For more information about ARNs in Organizations, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_permissions.html#orgs-permissions-arns">ARN Formats Supported by Organizations</a> in the <i>AWS Organizations User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FeatureSet</i> Specifies the functionality that currently is available to the organization. If set to "ALL", then all features are enabled and policies can be applied to accounts in the organization. If set to "CONSOLIDATED_BILLING", then only consolidated billing functionality is available. For more information, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_manage_org_support-all-features.html">Enabling All Features in Your Organization</a> in the <i>AWS Organizations User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of an organization. For more information about ARNs in Organizations, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_permissions.html#orgs-permissions-arns">ARN Formats Supported by Organizations</a> in the <i>AWS Organizations User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier (ID) of an organization. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for an organization ID string requires "o-" followed by from 10 to 32 lower-case letters or digits.<br/>        """
        pass

    def create_organizational_unit(self, Name: str, ParentId: str) -> CreateOrganizationalUnitResponse:
        """Creates an organizational unit (OU) within a root or parent OU. An OU is a container for accounts that enables you to organize your accounts to apply policies according to your business requirements. The number of levels deep that you can nest OUs is dependent upon the policy types enabled for that root. For service control policies, the limit is five.  For more information about OUs, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_manage_ous.html">Managing Organizational Units</a> in the <i>AWS Organizations User Guide.</i>  This operation can be called only from the organization's master account.

        :param self:
        :param Name:  The friendly name to assign to the new OU.<br/>ParentId:  The unique identifier (ID) of the parent root or OU that you want to create the new OU in. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for a parent ID string requires one of the following: <ul> <li> Root: a string that begins with "r-" followed by from 4 to 32 lower-case letters or digits. </li> <li> Organizational unit (OU): a string that begins with "ou-" followed by from 4 to 32 lower-case letters or digits (the ID of the root that the OU is in) followed by a second "-" dash and from 8 to 32 additional lower-case letters or digits. </li> </ul><br/>
        :return: OrganizationalUnit:  A structure that contains details about the newly created OU.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The friendly name of this OU. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> that is used to validate this parameter is a string of any of the characters in the ASCII character range.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of this OU. For more information about ARNs in Organizations, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_permissions.html#orgs-permissions-arns">ARN Formats Supported by Organizations</a> in the <i>AWS Organizations User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier (ID) associated with this OU. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for an organizational unit ID string requires "ou-" followed by from 4 to 32 lower-case letters or digits (the ID of the root that contains the OU) followed by a second "-" dash and from 8 to 32 additional lower-case letters or digits.<br/>        """
        pass

    def create_policy(self, Type: str, Name: str, Description: str, Content: str) -> CreatePolicyResponse:
        """Creates a policy of a specified type that you can attach to a root, an organizational unit (OU), or an individual AWS account. For more information about policies and their use, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_manage_policies.html">Managing Organization Policies</a>. This operation can be called only from the organization's master account.

        :param self:
        :param Type:  The type of policy to create. <note> In the current release, the only type of policy that you can create is a service control policy (SCP). </note><br/>Name:  The friendly name to assign to the policy. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> that is used to validate this parameter is a string of any of the characters in the ASCII character range.<br/>Description:  An optional description to assign to the policy.<br/>Content:  The policy content to add to the new policy. For example, if you create a <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_manage_policies_scp.html">service control policy</a> (SCP), this string must be JSON text that specifies the permissions that admins in attached accounts can delegate to their users, groups, and roles. For more information about the SCP syntax, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_reference_scp-syntax.html">Service Control Policy Syntax</a> in the <i>AWS Organizations User Guide.</i> <br/>
        :return: Policy:  A structure that contains details about the newly created policy.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Content</i> The text content of the policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicySummary</i> A structure that contains additional details about the policy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PolicySummary</b>: A structure that contains additional details about the policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AwsManaged</i> A boolean value that indicates whether the specified policy is an AWS managed policy. If true, then you can attach the policy to roots, OUs, or accounts, but you cannot edit it.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The friendly name of the policy. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> that is used to validate this parameter is a string of any of the characters in the ASCII character range.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the policy. For more information about ARNs in Organizations, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_permissions.html#orgs-permissions-arns">ARN Formats Supported by Organizations</a> in the <i>AWS Organizations User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier (ID) of the policy. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for a policy ID string requires "p-" followed by from 8 to 128 lower-case letters or digits.<br/>        """
        pass

    def decline_handshake(self, HandshakeId: str) -> DeclineHandshakeResponse:
        """Declines a handshake request. This sets the handshake state to <code>DECLINED</code> and effectively deactivates the request. This operation can be called only from the account that received the handshake. The originator of the handshake can use <a>CancelHandshake</a> instead. The originator can't reactivate a declined request, but can reinitiate the process with a new handshake request. After you decline a handshake, it continues to appear in the results of relevant APIs for only 30 days. After that, it's deleted.

        :param self:
        :param HandshakeId:  The unique identifier (ID) of the handshake that you want to decline. You can get the ID from the <a>ListHandshakesForAccount</a> operation. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for handshake ID string requires "h-" followed by from 8 to 32 lower-case letters or digits.<br/>
        :return: Handshake:  A structure that contains details about the declined handshake. The state is updated to show the value <code>DECLINED</code>.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Resources</i> Additional information that is needed to process the handshake.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Action</i> The type of handshake, indicating what action occurs when the recipient accepts the handshake. The following handshake types are supported: <ul> <li>  <b>INVITE</b>: This type of handshake represents a request to join an organization. It is always sent from the master account to only non-member accounts. </li> <li>  <b>ENABLE_ALL_FEATURES</b>: This type of handshake represents a request to enable all features in an organization. It is always sent from the master account to only <i>invited</i> member accounts. Created accounts do not receive this because those accounts were created by the organization's master account and approval is inferred. </li> <li>  <b>APPROVE_ALL_FEATURES</b>: This type of handshake is sent from the Organizations service when all member accounts have approved the <code>ENABLE_ALL_FEATURES</code> invitation. It is sent only to the master account and signals the master that it can finalize the process to enable all features. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpirationTimestamp</i> The date and time that the handshake expires. If the recipient of the handshake request fails to respond before the specified date and time, the handshake becomes inactive and is no longer valid.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestedTimestamp</i> The date and time that the handshake request was made.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The current state of the handshake. Use the state to trace the flow of the handshake through the process from its creation to its acceptance. The meaning of each of the valid values is as follows: <ul> <li>  <b>REQUESTED</b>: This handshake was sent to multiple recipients (applicable to only some handshake types) and not all recipients have responded yet. The request stays in this state until all recipients respond. </li> <li>  <b>OPEN</b>: This handshake was sent to multiple recipients (applicable to only some policy types) and all recipients have responded, allowing the originator to complete the handshake action. </li> <li>  <b>CANCELED</b>: This handshake is no longer active because it was canceled by the originating account. </li> <li>  <b>ACCEPTED</b>: This handshake is complete because it has been accepted by the recipient. </li> <li>  <b>DECLINED</b>: This handshake is no longer active because it was declined by the recipient account. </li> <li>  <b>EXPIRED</b>: This handshake is no longer active because the originator did not receive a response of any kind from the recipient before the expiration time (15 days). </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parties</i> Information about the two accounts that are participating in the handshake.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of a handshake. For more information about ARNs in Organizations, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_permissions.html#orgs-permissions-arns">ARN Formats Supported by Organizations</a> in the <i>AWS Organizations User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier (ID) of a handshake. The originating account creates the ID when it initiates the handshake. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for handshake ID string requires "h-" followed by from 8 to 32 lower-case letters or digits.<br/>        """
        pass

    def delete_organization(self):
        """Deletes the organization. You can delete an organization only by using credentials from the master account. The organization must be empty of member accounts.

        :param self:
        :param 
        :return:         """
        pass

    def delete_organizational_unit(self, OrganizationalUnitId: str):
        """Deletes an organizational unit (OU) from a root or another OU. You must first remove all accounts and child OUs from the OU that you want to delete. This operation can be called only from the organization's master account.

        :param self:
        :param OrganizationalUnitId:  The unique identifier (ID) of the organizational unit that you want to delete. You can get the ID from the <a>ListOrganizationalUnitsForParent</a> operation. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for an organizational unit ID string requires "ou-" followed by from 4 to 32 lower-case letters or digits (the ID of the root that contains the OU) followed by a second "-" dash and from 8 to 32 additional lower-case letters or digits.<br/>
        :return:         """
        pass

    def delete_policy(self, PolicyId: str):
        """Deletes the specified policy from your organization. Before you perform this operation, you must first detach the policy from all organizational units (OUs), roots, and accounts. This operation can be called only from the organization's master account.

        :param self:
        :param PolicyId:  The unique identifier (ID) of the policy that you want to delete. You can get the ID from the <a>ListPolicies</a> or <a>ListPoliciesForTarget</a> operations. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for a policy ID string requires "p-" followed by from 8 to 128 lower-case letters or digits.<br/>
        :return:         """
        pass

    def describe_account(self, AccountId: str) -> DescribeAccountResponse:
        """Retrieves AWS Organizations-related information about the specified account. This operation can be called only from the organization's master account.

        :param self:
        :param AccountId:  The unique identifier (ID) of the AWS account that you want information about. You can get the ID from the <a>ListAccounts</a> or <a>ListAccountsForParent</a> operations. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for an account ID string requires exactly 12 digits.<br/>
        :return: Account:  A structure that contains information about the requested account.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JoinedTimestamp</i> The date the account became a part of the organization.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JoinedMethod</i> The method by which the account joined the organization.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the account in the organization.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The friendly name of the account. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> that is used to validate this parameter is a string of any of the characters in the ASCII character range.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Email</i> The email address associated with the AWS account. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for this parameter is a string of characters that represents a standard Internet email address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the account. For more information about ARNs in Organizations, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_permissions.html#orgs-permissions-arns">ARN Formats Supported by Organizations</a> in the <i>AWS Organizations User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier (ID) of the account. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for an account ID string requires exactly 12 digits.<br/>        """
        pass

    def describe_create_account_status(self, CreateAccountRequestId: str) -> DescribeCreateAccountStatusResponse:
        """Retrieves the current status of an asynchronous request to create an account. This operation can be called only from the organization's master account.

        :param self:
        :param CreateAccountRequestId:  Specifies the <code>operationId</code> that uniquely identifies the request. You can get the ID from the response to an earlier <a>CreateAccount</a> request, or from the <a>ListCreateAccountStatus</a> operation. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for an create account request ID string requires "car-" followed by from 8 to 32 lower-case letters or digits.<br/>
        :return: CreateAccountStatus:  A structure that contains the current status of an account creation request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureReason</i> If the request failed, a description of the reason for the failure. <ul> <li> ACCOUNT_LIMIT_EXCEEDED: The account could not be created because you have reached the limit on the number of accounts in your organization. </li> <li> EMAIL_ALREADY_EXISTS: The account could not be created because another AWS account with that email address already exists. </li> <li> INVALID_ADDRESS: The account could not be created because the address you provided is not valid. </li> <li> INVALID_EMAIL: The account could not be created because the email address you provided is not valid. </li> <li> INTERNAL_FAILURE: The account could not be created because of an internal failure. Try again later. If the problem persists, contact Customer Support. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GovCloudAccountId</i> If the account was created successfully, the unique identifier (ID) of the new account in the AWS GovCloud (US) Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountId</i> If the account was created successfully, the unique identifier (ID) of the new account. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for an account ID string requires exactly 12 digits.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompletedTimestamp</i> The date and time that the account was created and the request completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestedTimestamp</i> The date and time that the request was made for the account creation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The status of the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountName</i> The account name given to the account when it was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier (ID) that references this request. You get this value from the response of the initial <a>CreateAccount</a> request to create the account. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for an create account request ID string requires "car-" followed by from 8 to 32 lower-case letters or digits.<br/>        """
        pass

    def describe_handshake(self, HandshakeId: str) -> DescribeHandshakeResponse:
        """Retrieves information about a previously requested handshake. The handshake ID comes from the response to the original <a>InviteAccountToOrganization</a> operation that generated the handshake. You can access handshakes that are <code>ACCEPTED</code>, <code>DECLINED</code>, or <code>CANCELED</code> for only 30 days after they change to that state. They're then deleted and no longer accessible. This operation can be called from any account in the organization.

        :param self:
        :param HandshakeId:  The unique identifier (ID) of the handshake that you want information about. You can get the ID from the original call to <a>InviteAccountToOrganization</a>, or from a call to <a>ListHandshakesForAccount</a> or <a>ListHandshakesForOrganization</a>. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for handshake ID string requires "h-" followed by from 8 to 32 lower-case letters or digits.<br/>
        :return: Handshake:  A structure that contains information about the specified handshake.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Resources</i> Additional information that is needed to process the handshake.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Action</i> The type of handshake, indicating what action occurs when the recipient accepts the handshake. The following handshake types are supported: <ul> <li>  <b>INVITE</b>: This type of handshake represents a request to join an organization. It is always sent from the master account to only non-member accounts. </li> <li>  <b>ENABLE_ALL_FEATURES</b>: This type of handshake represents a request to enable all features in an organization. It is always sent from the master account to only <i>invited</i> member accounts. Created accounts do not receive this because those accounts were created by the organization's master account and approval is inferred. </li> <li>  <b>APPROVE_ALL_FEATURES</b>: This type of handshake is sent from the Organizations service when all member accounts have approved the <code>ENABLE_ALL_FEATURES</code> invitation. It is sent only to the master account and signals the master that it can finalize the process to enable all features. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpirationTimestamp</i> The date and time that the handshake expires. If the recipient of the handshake request fails to respond before the specified date and time, the handshake becomes inactive and is no longer valid.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestedTimestamp</i> The date and time that the handshake request was made.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The current state of the handshake. Use the state to trace the flow of the handshake through the process from its creation to its acceptance. The meaning of each of the valid values is as follows: <ul> <li>  <b>REQUESTED</b>: This handshake was sent to multiple recipients (applicable to only some handshake types) and not all recipients have responded yet. The request stays in this state until all recipients respond. </li> <li>  <b>OPEN</b>: This handshake was sent to multiple recipients (applicable to only some policy types) and all recipients have responded, allowing the originator to complete the handshake action. </li> <li>  <b>CANCELED</b>: This handshake is no longer active because it was canceled by the originating account. </li> <li>  <b>ACCEPTED</b>: This handshake is complete because it has been accepted by the recipient. </li> <li>  <b>DECLINED</b>: This handshake is no longer active because it was declined by the recipient account. </li> <li>  <b>EXPIRED</b>: This handshake is no longer active because the originator did not receive a response of any kind from the recipient before the expiration time (15 days). </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parties</i> Information about the two accounts that are participating in the handshake.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of a handshake. For more information about ARNs in Organizations, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_permissions.html#orgs-permissions-arns">ARN Formats Supported by Organizations</a> in the <i>AWS Organizations User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier (ID) of a handshake. The originating account creates the ID when it initiates the handshake. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for handshake ID string requires "h-" followed by from 8 to 32 lower-case letters or digits.<br/>        """
        pass

    def describe_organization(self) -> DescribeOrganizationResponse:
        """Retrieves information about the organization that the user's account belongs to. This operation can be called from any account in the organization. <note> Even if a policy type is shown as available in the organization, you can disable it separately at the root level with <a>DisablePolicyType</a>. Use <a>ListRoots</a> to see the status of policy types for a specified root. </note>

        :param self:
        :param 
        :return: Organization:  A structure that contains information about the organization.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailablePolicyTypes</i> A list of policy types that are enabled for this organization. For example, if your organization has all features enabled, then service control policies (SCPs) are included in the list. <note> Even if a policy type is shown as available in the organization, you can separately enable and disable them at the root level by using <a>EnablePolicyType</a> and <a>DisablePolicyType</a>. Use <a>ListRoots</a> to see the status of a policy type in that root. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterAccountEmail</i> The email address that is associated with the AWS account that is designated as the master account for the organization.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterAccountId</i> The unique identifier (ID) of the master account of an organization. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for an account ID string requires exactly 12 digits.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterAccountArn</i> The Amazon Resource Name (ARN) of the account that is designated as the master account for the organization. For more information about ARNs in Organizations, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_permissions.html#orgs-permissions-arns">ARN Formats Supported by Organizations</a> in the <i>AWS Organizations User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FeatureSet</i> Specifies the functionality that currently is available to the organization. If set to "ALL", then all features are enabled and policies can be applied to accounts in the organization. If set to "CONSOLIDATED_BILLING", then only consolidated billing functionality is available. For more information, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_manage_org_support-all-features.html">Enabling All Features in Your Organization</a> in the <i>AWS Organizations User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of an organization. For more information about ARNs in Organizations, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_permissions.html#orgs-permissions-arns">ARN Formats Supported by Organizations</a> in the <i>AWS Organizations User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier (ID) of an organization. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for an organization ID string requires "o-" followed by from 10 to 32 lower-case letters or digits.<br/>        """
        pass

    def describe_organizational_unit(self, OrganizationalUnitId: str) -> DescribeOrganizationalUnitResponse:
        """Retrieves information about an organizational unit (OU). This operation can be called only from the organization's master account.

        :param self:
        :param OrganizationalUnitId:  The unique identifier (ID) of the organizational unit that you want details about. You can get the ID from the <a>ListOrganizationalUnitsForParent</a> operation. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for an organizational unit ID string requires "ou-" followed by from 4 to 32 lower-case letters or digits (the ID of the root that contains the OU) followed by a second "-" dash and from 8 to 32 additional lower-case letters or digits.<br/>
        :return: OrganizationalUnit:  A structure that contains details about the specified OU.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The friendly name of this OU. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> that is used to validate this parameter is a string of any of the characters in the ASCII character range.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of this OU. For more information about ARNs in Organizations, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_permissions.html#orgs-permissions-arns">ARN Formats Supported by Organizations</a> in the <i>AWS Organizations User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier (ID) associated with this OU. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for an organizational unit ID string requires "ou-" followed by from 4 to 32 lower-case letters or digits (the ID of the root that contains the OU) followed by a second "-" dash and from 8 to 32 additional lower-case letters or digits.<br/>        """
        pass

    def describe_policy(self, PolicyId: str) -> DescribePolicyResponse:
        """Retrieves information about a policy. This operation can be called only from the organization's master account.

        :param self:
        :param PolicyId:  The unique identifier (ID) of the policy that you want details about. You can get the ID from the <a>ListPolicies</a> or <a>ListPoliciesForTarget</a> operations. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for a policy ID string requires "p-" followed by from 8 to 128 lower-case letters or digits.<br/>
        :return: Policy:  A structure that contains details about the specified policy.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Content</i> The text content of the policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicySummary</i> A structure that contains additional details about the policy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PolicySummary</b>: A structure that contains additional details about the policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AwsManaged</i> A boolean value that indicates whether the specified policy is an AWS managed policy. If true, then you can attach the policy to roots, OUs, or accounts, but you cannot edit it.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The friendly name of the policy. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> that is used to validate this parameter is a string of any of the characters in the ASCII character range.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the policy. For more information about ARNs in Organizations, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_permissions.html#orgs-permissions-arns">ARN Formats Supported by Organizations</a> in the <i>AWS Organizations User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier (ID) of the policy. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for a policy ID string requires "p-" followed by from 8 to 128 lower-case letters or digits.<br/>        """
        pass

    def detach_policy(self, TargetId: str, PolicyId: str):
        """Detaches a policy from a target root, organizational unit (OU), or account. If the policy being detached is a service control policy (SCP), the changes to permissions for IAM users and roles in affected accounts are immediate.  <b>Note:</b> Every root, OU, and account must have at least one SCP attached. If you want to replace the default <code>FullAWSAccess</code> policy with one that limits the permissions that can be delegated, you must attach the replacement policy before you can remove the default one. This is the authorization strategy of <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_manage_policies_about-scps.html#orgs_policies_whitelist">whitelisting</a>. If you instead attach a second SCP and leave the <code>FullAWSAccess</code> SCP still attached, and specify <code>"Effect": "Deny"</code> in the second SCP to override the <code>"Effect": "Allow"</code> in the <code>FullAWSAccess</code> policy (or any other attached SCP), you're using the authorization strategy of <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_manage_policies_about-scps.html#orgs_policies_blacklist">blacklisting</a>.  This operation can be called only from the organization's master account.

        :param self:
        :param TargetId:  The unique identifier (ID) of the root, OU, or account that you want to detach the policy from. You can get the ID from the <a>ListRoots</a>, <a>ListOrganizationalUnitsForParent</a>, or <a>ListAccounts</a> operations. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for a target ID string requires one of the following: <ul> <li> Root: a string that begins with "r-" followed by from 4 to 32 lower-case letters or digits. </li> <li> Account: a string that consists of exactly 12 digits. </li> <li> Organizational unit (OU): a string that begins with "ou-" followed by from 4 to 32 lower-case letters or digits (the ID of the root that the OU is in) followed by a second "-" dash and from 8 to 32 additional lower-case letters or digits. </li> </ul><br/>PolicyId:  The unique identifier (ID) of the policy you want to detach. You can get the ID from the <a>ListPolicies</a> or <a>ListPoliciesForTarget</a> operations. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for a policy ID string requires "p-" followed by from 8 to 128 lower-case letters or digits.<br/>
        :return:         """
        pass

    def disable_aws_service_access(self, ServicePrincipal: str):
        """Disables the integration of an AWS service (the service that is specified by <code>ServicePrincipal</code>) with AWS Organizations. When you disable integration, the specified service no longer can create a <a href="http://docs.aws.amazon.com/IAM/latest/UserGuide/using-service-linked-roles.html">service-linked role</a> in <i>new</i> accounts in your organization. This means the service can't perform operations on your behalf on any new accounts in your organization. The service can still perform operations in older accounts until the service completes its clean-up from AWS Organizations. <p/> <important> We recommend that you disable integration between AWS Organizations and the specified AWS service by using the console or commands that are provided by the specified service. Doing so ensures that the other service is aware that it can clean up any resources that are required only for the integration. How the service cleans up its resources in the organization's accounts depends on that service. For more information, see the documentation for the other AWS service. </important> After you perform the <code>DisableAWSServiceAccess</code> operation, the specified service can no longer perform operations in your organization's accounts unless the operations are explicitly permitted by the IAM policies that are attached to your roles.  For more information about integrating other services with AWS Organizations, including the list of services that work with Organizations, see <a href="http://docs.aws.amazon.com/organizations/latest/userguide/orgs_integrate_services.html">Integrating AWS Organizations with Other AWS Services</a> in the <i>AWS Organizations User Guide.</i>  This operation can be called only from the organization's master account.

        :param self:
        :param ServicePrincipal:  The service principal name of the AWS service for which you want to disable integration with your organization. This is typically in the form of a URL, such as <code> <i>service-abbreviation</i>.amazonaws.com</code>.<br/>
        :return:         """
        pass

    def disable_policy_type(self, PolicyType: str, RootId: str) -> DisablePolicyTypeResponse:
        """Disables an organizational control policy type in a root. A policy of a certain type can be attached to entities in a root only if that type is enabled in the root. After you perform this operation, you no longer can attach policies of the specified type to that root or to any organizational unit (OU) or account in that root. You can undo this by using the <a>EnablePolicyType</a> operation. This operation can be called only from the organization's master account. <note> If you disable a policy type for a root, it still shows as enabled for the organization if all features are enabled in that organization. Use <a>ListRoots</a> to see the status of policy types for a specified root. Use <a>DescribeOrganization</a> to see the status of policy types in the organization. </note>

        :param self:
        :param PolicyType:  The policy type that you want to disable in this root.<br/>RootId:  The unique identifier (ID) of the root in which you want to disable a policy type. You can get the ID from the <a>ListRoots</a> operation. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for a root ID string requires "r-" followed by from 4 to 32 lower-case letters or digits.<br/>
        :return: Root:  A structure that shows the root with the updated list of enabled policy types.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyTypes</i> The types of policies that are currently enabled for the root and therefore can be attached to the root or to its OUs or accounts. <note> Even if a policy type is shown as available in the organization, you can separately enable and disable them at the root level by using <a>EnablePolicyType</a> and <a>DisablePolicyType</a>. Use <a>DescribeOrganization</a> to see the availability of the policy types in that organization. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The friendly name of the root. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> that is used to validate this parameter is a string of any of the characters in the ASCII character range.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the root. For more information about ARNs in Organizations, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_permissions.html#orgs-permissions-arns">ARN Formats Supported by Organizations</a> in the <i>AWS Organizations User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier (ID) for the root. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for a root ID string requires "r-" followed by from 4 to 32 lower-case letters or digits.<br/>        """
        pass

    def enable_aws_service_access(self, ServicePrincipal: str):
        """Enables the integration of an AWS service (the service that is specified by <code>ServicePrincipal</code>) with AWS Organizations. When you enable integration, you allow the specified service to create a <a href="http://docs.aws.amazon.com/IAM/latest/UserGuide/using-service-linked-roles.html">service-linked role</a> in all the accounts in your organization. This allows the service to perform operations on your behalf in your organization and its accounts. <important> We recommend that you enable integration between AWS Organizations and the specified AWS service by using the console or commands that are provided by the specified service. Doing so ensures that the service is aware that it can create the resources that are required for the integration. How the service creates those resources in the organization's accounts depends on that service. For more information, see the documentation for the other AWS service. </important> For more information about enabling services to integrate with AWS Organizations, see <a href="http://docs.aws.amazon.com/organizations/latest/userguide/orgs_integrate_services.html">Integrating AWS Organizations with Other AWS Services</a> in the <i>AWS Organizations User Guide.</i>  This operation can be called only from the organization's master account and only if the organization has <a href="http://docs.aws.amazon.com/organizations/latest/userguide/orgs_manage_org_support-all-features.html">enabled all features</a>.

        :param self:
        :param ServicePrincipal:  The service principal name of the AWS service for which you want to enable integration with your organization. This is typically in the form of a URL, such as <code> <i>service-abbreviation</i>.amazonaws.com</code>.<br/>
        :return:         """
        pass

    def enable_all_features(self) -> EnableAllFeaturesResponse:
        """Enables all features in an organization. This enables the use of organization policies that can restrict the services and actions that can be called in each account. Until you enable all features, you have access only to consolidated billing, and you can't use any of the advanced account administration features that AWS Organizations supports. For more information, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_manage_org_support-all-features.html">Enabling All Features in Your Organization</a> in the <i>AWS Organizations User Guide.</i>  <important> This operation is required only for organizations that were created explicitly with only the consolidated billing features enabled. Calling this operation sends a handshake to every invited account in the organization. The feature set change can be finalized and the additional features enabled only after all administrators in the invited accounts approve the change by accepting the handshake. </important> After you enable all features, you can separately enable or disable individual policy types in a root using <a>EnablePolicyType</a> and <a>DisablePolicyType</a>. To see the status of policy types in a root, use <a>ListRoots</a>. After all invited member accounts accept the handshake, you finalize the feature set change by accepting the handshake that contains <code>"Action": "ENABLE_ALL_FEATURES"</code>. This completes the change. After you enable all features in your organization, the master account in the organization can apply policies on all member accounts. These policies can restrict what users and even administrators in those accounts can do. The master account can apply policies that prevent accounts from leaving the organization. Ensure that your account administrators are aware of this. This operation can be called only from the organization's master account. 

        :param self:
        :param 
        :return: Handshake:  A structure that contains details about the handshake created to support this request to enable all features in the organization.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Resources</i> Additional information that is needed to process the handshake.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Action</i> The type of handshake, indicating what action occurs when the recipient accepts the handshake. The following handshake types are supported: <ul> <li>  <b>INVITE</b>: This type of handshake represents a request to join an organization. It is always sent from the master account to only non-member accounts. </li> <li>  <b>ENABLE_ALL_FEATURES</b>: This type of handshake represents a request to enable all features in an organization. It is always sent from the master account to only <i>invited</i> member accounts. Created accounts do not receive this because those accounts were created by the organization's master account and approval is inferred. </li> <li>  <b>APPROVE_ALL_FEATURES</b>: This type of handshake is sent from the Organizations service when all member accounts have approved the <code>ENABLE_ALL_FEATURES</code> invitation. It is sent only to the master account and signals the master that it can finalize the process to enable all features. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpirationTimestamp</i> The date and time that the handshake expires. If the recipient of the handshake request fails to respond before the specified date and time, the handshake becomes inactive and is no longer valid.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestedTimestamp</i> The date and time that the handshake request was made.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The current state of the handshake. Use the state to trace the flow of the handshake through the process from its creation to its acceptance. The meaning of each of the valid values is as follows: <ul> <li>  <b>REQUESTED</b>: This handshake was sent to multiple recipients (applicable to only some handshake types) and not all recipients have responded yet. The request stays in this state until all recipients respond. </li> <li>  <b>OPEN</b>: This handshake was sent to multiple recipients (applicable to only some policy types) and all recipients have responded, allowing the originator to complete the handshake action. </li> <li>  <b>CANCELED</b>: This handshake is no longer active because it was canceled by the originating account. </li> <li>  <b>ACCEPTED</b>: This handshake is complete because it has been accepted by the recipient. </li> <li>  <b>DECLINED</b>: This handshake is no longer active because it was declined by the recipient account. </li> <li>  <b>EXPIRED</b>: This handshake is no longer active because the originator did not receive a response of any kind from the recipient before the expiration time (15 days). </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parties</i> Information about the two accounts that are participating in the handshake.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of a handshake. For more information about ARNs in Organizations, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_permissions.html#orgs-permissions-arns">ARN Formats Supported by Organizations</a> in the <i>AWS Organizations User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier (ID) of a handshake. The originating account creates the ID when it initiates the handshake. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for handshake ID string requires "h-" followed by from 8 to 32 lower-case letters or digits.<br/>        """
        pass

    def enable_policy_type(self, PolicyType: str, RootId: str) -> EnablePolicyTypeResponse:
        """Enables a policy type in a root. After you enable a policy type in a root, you can attach policies of that type to the root, any organizational unit (OU), or account in that root. You can undo this by using the <a>DisablePolicyType</a> operation. This operation can be called only from the organization's master account. You can enable a policy type in a root only if that policy type is available in the organization. Use <a>DescribeOrganization</a> to view the status of available policy types in the organization. To view the status of policy type in a root, use <a>ListRoots</a>.

        :param self:
        :param PolicyType:  The policy type that you want to enable.<br/>RootId:  The unique identifier (ID) of the root in which you want to enable a policy type. You can get the ID from the <a>ListRoots</a> operation. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for a root ID string requires "r-" followed by from 4 to 32 lower-case letters or digits.<br/>
        :return: Root:  A structure that shows the root with the updated list of enabled policy types.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicyTypes</i> The types of policies that are currently enabled for the root and therefore can be attached to the root or to its OUs or accounts. <note> Even if a policy type is shown as available in the organization, you can separately enable and disable them at the root level by using <a>EnablePolicyType</a> and <a>DisablePolicyType</a>. Use <a>DescribeOrganization</a> to see the availability of the policy types in that organization. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The friendly name of the root. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> that is used to validate this parameter is a string of any of the characters in the ASCII character range.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the root. For more information about ARNs in Organizations, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_permissions.html#orgs-permissions-arns">ARN Formats Supported by Organizations</a> in the <i>AWS Organizations User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier (ID) for the root. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for a root ID string requires "r-" followed by from 4 to 32 lower-case letters or digits.<br/>        """
        pass

    def invite_account_to_organization(self, Target: HandshakeParty, Notes: str=None) -> InviteAccountToOrganizationResponse:
        """Sends an invitation to another account to join your organization as a member account. AWS Organizations sends email on your behalf to the email address that is associated with the other account's owner. The invitation is implemented as a <a>Handshake</a> whose details are in the response. <important> <ul> <li> You can invite AWS accounts only from the same seller as the master account. For example, if your organization's master account was created by Amazon Internet Services Pvt. Ltd (AISPL), an AWS seller in India, you can invite only other AISPL accounts to your organization. You can't combine accounts from AISPL and AWS or from any other AWS seller. For more information, see <a href="http://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/useconsolidatedbilliing-India.html">Consolidated Billing in India</a>. </li> <li> If you receive an exception that indicates that you exceeded your account limits for the organization or that the operation failed because your organization is still initializing, wait one hour and then try again. If the error persists after an hour, contact <a href="https://console.aws.amazon.com/support/home#/">AWS Support</a>. </li> </ul> </important> This operation can be called only from the organization's master account.

        :param self:
        :param Notes:  Additional information that you want to include in the generated email to the recipient account owner.<br/>Target:  The identifier (ID) of the AWS account that you want to invite to join your organization. This is a JSON object that contains the following elements:   <code>{ "Type": "ACCOUNT", "Id": "&lt;<i> <b>account id number</b> </i>&gt;" }</code>  If you use the AWS CLI, you can submit this as a single string, similar to the following example:  <code>--target Id=123456789012,Type=ACCOUNT</code>  If you specify <code>"Type": "ACCOUNT"</code>, you must provide the AWS account ID number as the <code>Id</code>. If you specify <code>"Type": "EMAIL"</code>, you must specify the email address that is associated with the account.  <code>--target Id=diego@example.com,Type=EMAIL</code> :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of party.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier (ID) for the party. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for handshake ID string requires "h-" followed by from 8 to 32 lower-case letters or digits.<br/>
        :return: Handshake:  A structure that contains details about the handshake that is created to support this invitation request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Resources</i> Additional information that is needed to process the handshake.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Action</i> The type of handshake, indicating what action occurs when the recipient accepts the handshake. The following handshake types are supported: <ul> <li>  <b>INVITE</b>: This type of handshake represents a request to join an organization. It is always sent from the master account to only non-member accounts. </li> <li>  <b>ENABLE_ALL_FEATURES</b>: This type of handshake represents a request to enable all features in an organization. It is always sent from the master account to only <i>invited</i> member accounts. Created accounts do not receive this because those accounts were created by the organization's master account and approval is inferred. </li> <li>  <b>APPROVE_ALL_FEATURES</b>: This type of handshake is sent from the Organizations service when all member accounts have approved the <code>ENABLE_ALL_FEATURES</code> invitation. It is sent only to the master account and signals the master that it can finalize the process to enable all features. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpirationTimestamp</i> The date and time that the handshake expires. If the recipient of the handshake request fails to respond before the specified date and time, the handshake becomes inactive and is no longer valid.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestedTimestamp</i> The date and time that the handshake request was made.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The current state of the handshake. Use the state to trace the flow of the handshake through the process from its creation to its acceptance. The meaning of each of the valid values is as follows: <ul> <li>  <b>REQUESTED</b>: This handshake was sent to multiple recipients (applicable to only some handshake types) and not all recipients have responded yet. The request stays in this state until all recipients respond. </li> <li>  <b>OPEN</b>: This handshake was sent to multiple recipients (applicable to only some policy types) and all recipients have responded, allowing the originator to complete the handshake action. </li> <li>  <b>CANCELED</b>: This handshake is no longer active because it was canceled by the originating account. </li> <li>  <b>ACCEPTED</b>: This handshake is complete because it has been accepted by the recipient. </li> <li>  <b>DECLINED</b>: This handshake is no longer active because it was declined by the recipient account. </li> <li>  <b>EXPIRED</b>: This handshake is no longer active because the originator did not receive a response of any kind from the recipient before the expiration time (15 days). </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parties</i> Information about the two accounts that are participating in the handshake.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of a handshake. For more information about ARNs in Organizations, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_permissions.html#orgs-permissions-arns">ARN Formats Supported by Organizations</a> in the <i>AWS Organizations User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier (ID) of a handshake. The originating account creates the ID when it initiates the handshake. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for handshake ID string requires "h-" followed by from 8 to 32 lower-case letters or digits.<br/>        """
        pass

    def leave_organization(self):
        """Removes a member account from its parent organization. This version of the operation is performed by the account that wants to leave. To remove a member account as a user in the master account, use <a>RemoveAccountFromOrganization</a> instead. This operation can be called only from a member account in the organization. <important> <ul> <li> The master account in an organization with all features enabled can set service control policies (SCPs) that can restrict what administrators of member accounts can do, including preventing them from successfully calling <code>LeaveOrganization</code> and leaving the organization.  </li> <li> You can leave an organization as a member account only if the account is configured with the information required to operate as a standalone account. When you create an account in an organization using the AWS Organizations console, API, or CLI commands, the information required of standalone accounts is <i>not</i> automatically collected. For each account that you want to make standalone, you must accept the end user license agreement (EULA), choose a support plan, provide and verify the required contact information, and provide a current payment method. AWS uses the payment method to charge for any billable (not free tier) AWS activity that occurs while the account isn't attached to an organization. Follow the steps at <a href="http://docs.aws.amazon.com/organizations/latest/userguide/orgs_manage_accounts_remove.html#leave-without-all-info"> To leave an organization when all required account information has not yet been provided</a> in the <i>AWS Organizations User Guide.</i>  </li> <li> You can leave an organization only after you enable IAM user access to billing in your account. For more information, see <a href="http://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/grantaccess.html#ControllingAccessWebsite-Activate">Activating Access to the Billing and Cost Management Console</a> in the <i>AWS Billing and Cost Management User Guide.</i>  </li> </ul> </important>

        :param self:
        :param 
        :return:         """
        pass

    def list_aws_service_access_for_organization(self, NextToken: str=None, MaxResults: int=None) -> ListAWSServiceAccessForOrganizationResponse:
        """Returns a list of the AWS services that you enabled to integrate with your organization. After a service on this list creates the resources that it requires for the integration, it can perform operations on your organization and its accounts. For more information about integrating other services with AWS Organizations, including the list of services that currently work with Organizations, see <a href="http://docs.aws.amazon.com/organizations/latest/userguide/orgs_integrate_services.html">Integrating AWS Organizations with Other AWS Services</a> in the <i>AWS Organizations User Guide.</i>  This operation can be called only from the organization's master account.

        :param self:
        :param MaxResults:  (Optional) Use this to limit the number of results you want included per page in the response. If you do not include this parameter, it defaults to a value that is specific to the operation. If additional items exist beyond the maximum you specify, the <code>NextToken</code> response element is present and has a value (is not null). Include that value as the <code>NextToken</code> request parameter in the next call to the operation to get the next part of the results. Note that Organizations might return fewer results than the maximum even when there are more results available. You should check <code>NextToken</code> after every operation to ensure that you receive all of the results.<br/>NextToken:  Use this parameter if you receive a <code>NextToken</code> response in a previous request that indicates that there is more output available. Set it to the value of the previous call's <code>NextToken</code> response to indicate where the output should continue from.<br/>
        :return: NextToken:  If present, this value indicates that there is more output available than is included in the current response. Use this value in the <code>NextToken</code> request parameter in a subsequent call to the operation to get the next part of the output. You should repeat this until the <code>NextToken</code> response element comes back as <code>null</code>.<br/>EnabledServicePrincipals:  A list of the service principals for the services that are enabled to integrate with your organization. Each principal is a structure that includes the name and the date that it was enabled for integration with AWS Organizations.<br/>        """
        pass

    def list_accounts(self, NextToken: str=None, MaxResults: int=None) -> ListAccountsResponse:
        """Lists all the accounts in the organization. To request only the accounts in a specified root or organizational unit (OU), use the <a>ListAccountsForParent</a> operation instead. <note> Always check the <code>NextToken</code> response parameter for a <code>null</code> value when calling a <code>List*</code> operation. These operations can occasionally return an empty set of results even when there are more results available. The <code>NextToken</code> response parameter value is <code>null</code> <i>only</i> when there are no more results to display. </note> This operation can be called only from the organization's master account.

        :param self:
        :param MaxResults:  (Optional) Use this to limit the number of results you want included per page in the response. If you do not include this parameter, it defaults to a value that is specific to the operation. If additional items exist beyond the maximum you specify, the <code>NextToken</code> response element is present and has a value (is not null). Include that value as the <code>NextToken</code> request parameter in the next call to the operation to get the next part of the results. Note that Organizations might return fewer results than the maximum even when there are more results available. You should check <code>NextToken</code> after every operation to ensure that you receive all of the results.<br/>NextToken:  Use this parameter if you receive a <code>NextToken</code> response in a previous request that indicates that there is more output available. Set it to the value of the previous call's <code>NextToken</code> response to indicate where the output should continue from.<br/>
        :return: NextToken:  If present, this value indicates that there is more output available than is included in the current response. Use this value in the <code>NextToken</code> request parameter in a subsequent call to the operation to get the next part of the output. You should repeat this until the <code>NextToken</code> response element comes back as <code>null</code>.<br/>Accounts:  A list of objects in the organization.<br/>        """
        pass

    def list_accounts_for_parent(self, ParentId: str, NextToken: str=None, MaxResults: int=None) -> ListAccountsForParentResponse:
        """Lists the accounts in an organization that are contained by the specified target root or organizational unit (OU). If you specify the root, you get a list of all the accounts that aren't in any OU. If you specify an OU, you get a list of all the accounts in only that OU and not in any child OUs. To get a list of all accounts in the organization, use the <a>ListAccounts</a> operation. <note> Always check the <code>NextToken</code> response parameter for a <code>null</code> value when calling a <code>List*</code> operation. These operations can occasionally return an empty set of results even when there are more results available. The <code>NextToken</code> response parameter value is <code>null</code> <i>only</i> when there are no more results to display. </note> This operation can be called only from the organization's master account.

        :param self:
        :param MaxResults:  (Optional) Use this to limit the number of results you want included per page in the response. If you do not include this parameter, it defaults to a value that is specific to the operation. If additional items exist beyond the maximum you specify, the <code>NextToken</code> response element is present and has a value (is not null). Include that value as the <code>NextToken</code> request parameter in the next call to the operation to get the next part of the results. Note that Organizations might return fewer results than the maximum even when there are more results available. You should check <code>NextToken</code> after every operation to ensure that you receive all of the results.<br/>NextToken:  Use this parameter if you receive a <code>NextToken</code> response in a previous request that indicates that there is more output available. Set it to the value of the previous call's <code>NextToken</code> response to indicate where the output should continue from.<br/>ParentId:  The unique identifier (ID) for the parent root or organization unit (OU) whose accounts you want to list.<br/>
        :return: NextToken:  If present, this value indicates that there is more output available than is included in the current response. Use this value in the <code>NextToken</code> request parameter in a subsequent call to the operation to get the next part of the output. You should repeat this until the <code>NextToken</code> response element comes back as <code>null</code>.<br/>Accounts:  A list of the accounts in the specified root or OU.<br/>        """
        pass

    def list_children(self, ChildType: str, ParentId: str, NextToken: str=None, MaxResults: int=None) -> ListChildrenResponse:
        """Lists all of the organizational units (OUs) or accounts that are contained in the specified parent OU or root. This operation, along with <a>ListParents</a> enables you to traverse the tree structure that makes up this root. <note> Always check the <code>NextToken</code> response parameter for a <code>null</code> value when calling a <code>List*</code> operation. These operations can occasionally return an empty set of results even when there are more results available. The <code>NextToken</code> response parameter value is <code>null</code> <i>only</i> when there are no more results to display. </note> This operation can be called only from the organization's master account.

        :param self:
        :param MaxResults:  (Optional) Use this to limit the number of results you want included per page in the response. If you do not include this parameter, it defaults to a value that is specific to the operation. If additional items exist beyond the maximum you specify, the <code>NextToken</code> response element is present and has a value (is not null). Include that value as the <code>NextToken</code> request parameter in the next call to the operation to get the next part of the results. Note that Organizations might return fewer results than the maximum even when there are more results available. You should check <code>NextToken</code> after every operation to ensure that you receive all of the results.<br/>NextToken:  Use this parameter if you receive a <code>NextToken</code> response in a previous request that indicates that there is more output available. Set it to the value of the previous call's <code>NextToken</code> response to indicate where the output should continue from.<br/>ChildType:  Filters the output to include only the specified child type.<br/>ParentId:  The unique identifier (ID) for the parent root or OU whose children you want to list. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for a parent ID string requires one of the following: <ul> <li> Root: a string that begins with "r-" followed by from 4 to 32 lower-case letters or digits. </li> <li> Organizational unit (OU): a string that begins with "ou-" followed by from 4 to 32 lower-case letters or digits (the ID of the root that the OU is in) followed by a second "-" dash and from 8 to 32 additional lower-case letters or digits. </li> </ul><br/>
        :return: NextToken:  If present, this value indicates that there is more output available than is included in the current response. Use this value in the <code>NextToken</code> request parameter in a subsequent call to the operation to get the next part of the output. You should repeat this until the <code>NextToken</code> response element comes back as <code>null</code>.<br/>Children:  The list of children of the specified parent container.<br/>        """
        pass

    def list_create_account_status(self, States: List[CreateAccountState]=None, NextToken: str=None, MaxResults: int=None) -> ListCreateAccountStatusResponse:
        """Lists the account creation requests that match the specified status that is currently being tracked for the organization. <note> Always check the <code>NextToken</code> response parameter for a <code>null</code> value when calling a <code>List*</code> operation. These operations can occasionally return an empty set of results even when there are more results available. The <code>NextToken</code> response parameter value is <code>null</code> <i>only</i> when there are no more results to display. </note> This operation can be called only from the organization's master account.

        :param self:
        :param MaxResults:  (Optional) Use this to limit the number of results you want included per page in the response. If you do not include this parameter, it defaults to a value that is specific to the operation. If additional items exist beyond the maximum you specify, the <code>NextToken</code> response element is present and has a value (is not null). Include that value as the <code>NextToken</code> request parameter in the next call to the operation to get the next part of the results. Note that Organizations might return fewer results than the maximum even when there are more results available. You should check <code>NextToken</code> after every operation to ensure that you receive all of the results.<br/>NextToken:  Use this parameter if you receive a <code>NextToken</code> response in a previous request that indicates that there is more output available. Set it to the value of the previous call's <code>NextToken</code> response to indicate where the output should continue from.<br/>States:  A list of one or more states that you want included in the response. If this parameter isn't present, all requests are included in the response.<br/>
        :return: NextToken:  If present, this value indicates that there is more output available than is included in the current response. Use this value in the <code>NextToken</code> request parameter in a subsequent call to the operation to get the next part of the output. You should repeat this until the <code>NextToken</code> response element comes back as <code>null</code>.<br/>CreateAccountStatuses:  A list of objects with details about the requests. Certain elements, such as the accountId number, are present in the output only after the account has been successfully created.<br/>        """
        pass

    def list_handshakes_for_account(self, Filter: HandshakeFilter=None, NextToken: str=None, MaxResults: int=None) -> ListHandshakesForAccountResponse:
        """Lists the current handshakes that are associated with the account of the requesting user. Handshakes that are <code>ACCEPTED</code>, <code>DECLINED</code>, or <code>CANCELED</code> appear in the results of this API for only 30 days after changing to that state. After that, they're deleted and no longer accessible. <note> Always check the <code>NextToken</code> response parameter for a <code>null</code> value when calling a <code>List*</code> operation. These operations can occasionally return an empty set of results even when there are more results available. The <code>NextToken</code> response parameter value is <code>null</code> <i>only</i> when there are no more results to display. </note> This operation can be called from any account in the organization.

        :param self:
        :param MaxResults:  (Optional) Use this to limit the number of results you want included per page in the response. If you do not include this parameter, it defaults to a value that is specific to the operation. If additional items exist beyond the maximum you specify, the <code>NextToken</code> response element is present and has a value (is not null). Include that value as the <code>NextToken</code> request parameter in the next call to the operation to get the next part of the results. Note that Organizations might return fewer results than the maximum even when there are more results available. You should check <code>NextToken</code> after every operation to ensure that you receive all of the results.<br/>NextToken:  Use this parameter if you receive a <code>NextToken</code> response in a previous request that indicates that there is more output available. Set it to the value of the previous call's <code>NextToken</code> response to indicate where the output should continue from.<br/>Filter:  Filters the handshakes that you want included in the response. The default is all types. Use the <code>ActionType</code> element to limit the output to only a specified type, such as <code>INVITE</code>, <code>ENABLE_ALL_FEATURES</code>, or <code>APPROVE_ALL_FEATURES</code>. Alternatively, for the <code>ENABLE_ALL_FEATURES</code> handshake that generates a separate child handshake for each member account, you can specify <code>ParentHandshakeId</code> to see only the handshakes that were generated by that parent request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParentHandshakeId</i> Specifies the parent handshake. Only used for handshake types that are a child of another type. If you specify <code>ParentHandshakeId</code>, you cannot also specify <code>ActionType</code>. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for handshake ID string requires "h-" followed by from 8 to 32 lower-case letters or digits.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActionType</i> Specifies the type of handshake action. If you specify <code>ActionType</code>, you cannot also specify <code>ParentHandshakeId</code>.<br/>
        :return: NextToken:  If present, this value indicates that there is more output available than is included in the current response. Use this value in the <code>NextToken</code> request parameter in a subsequent call to the operation to get the next part of the output. You should repeat this until the <code>NextToken</code> response element comes back as <code>null</code>.<br/>Handshakes:  A list of <a>Handshake</a> objects with details about each of the handshakes that is associated with the specified account.<br/>        """
        pass

    def list_handshakes_for_organization(self, Filter: HandshakeFilter=None, NextToken: str=None, MaxResults: int=None) -> ListHandshakesForOrganizationResponse:
        """Lists the handshakes that are associated with the organization that the requesting user is part of. The <code>ListHandshakesForOrganization</code> operation returns a list of handshake structures. Each structure contains details and status about a handshake. Handshakes that are <code>ACCEPTED</code>, <code>DECLINED</code>, or <code>CANCELED</code> appear in the results of this API for only 30 days after changing to that state. After that, they're deleted and no longer accessible. <note> Always check the <code>NextToken</code> response parameter for a <code>null</code> value when calling a <code>List*</code> operation. These operations can occasionally return an empty set of results even when there are more results available. The <code>NextToken</code> response parameter value is <code>null</code> <i>only</i> when there are no more results to display. </note> This operation can be called only from the organization's master account.

        :param self:
        :param MaxResults:  (Optional) Use this to limit the number of results you want included per page in the response. If you do not include this parameter, it defaults to a value that is specific to the operation. If additional items exist beyond the maximum you specify, the <code>NextToken</code> response element is present and has a value (is not null). Include that value as the <code>NextToken</code> request parameter in the next call to the operation to get the next part of the results. Note that Organizations might return fewer results than the maximum even when there are more results available. You should check <code>NextToken</code> after every operation to ensure that you receive all of the results.<br/>NextToken:  Use this parameter if you receive a <code>NextToken</code> response in a previous request that indicates that there is more output available. Set it to the value of the previous call's <code>NextToken</code> response to indicate where the output should continue from.<br/>Filter:  A filter of the handshakes that you want included in the response. The default is all types. Use the <code>ActionType</code> element to limit the output to only a specified type, such as <code>INVITE</code>, <code>ENABLE-ALL-FEATURES</code>, or <code>APPROVE-ALL-FEATURES</code>. Alternatively, for the <code>ENABLE-ALL-FEATURES</code> handshake that generates a separate child handshake for each member account, you can specify the <code>ParentHandshakeId</code> to see only the handshakes that were generated by that parent request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParentHandshakeId</i> Specifies the parent handshake. Only used for handshake types that are a child of another type. If you specify <code>ParentHandshakeId</code>, you cannot also specify <code>ActionType</code>. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for handshake ID string requires "h-" followed by from 8 to 32 lower-case letters or digits.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActionType</i> Specifies the type of handshake action. If you specify <code>ActionType</code>, you cannot also specify <code>ParentHandshakeId</code>.<br/>
        :return: NextToken:  If present, this value indicates that there is more output available than is included in the current response. Use this value in the <code>NextToken</code> request parameter in a subsequent call to the operation to get the next part of the output. You should repeat this until the <code>NextToken</code> response element comes back as <code>null</code>.<br/>Handshakes:  A list of <a>Handshake</a> objects with details about each of the handshakes that are associated with an organization.<br/>        """
        pass

    def list_organizational_units_for_parent(self, ParentId: str, NextToken: str=None, MaxResults: int=None) -> ListOrganizationalUnitsForParentResponse:
        """Lists the organizational units (OUs) in a parent organizational unit or root. <note> Always check the <code>NextToken</code> response parameter for a <code>null</code> value when calling a <code>List*</code> operation. These operations can occasionally return an empty set of results even when there are more results available. The <code>NextToken</code> response parameter value is <code>null</code> <i>only</i> when there are no more results to display. </note> This operation can be called only from the organization's master account.

        :param self:
        :param MaxResults:  (Optional) Use this to limit the number of results you want included per page in the response. If you do not include this parameter, it defaults to a value that is specific to the operation. If additional items exist beyond the maximum you specify, the <code>NextToken</code> response element is present and has a value (is not null). Include that value as the <code>NextToken</code> request parameter in the next call to the operation to get the next part of the results. Note that Organizations might return fewer results than the maximum even when there are more results available. You should check <code>NextToken</code> after every operation to ensure that you receive all of the results.<br/>NextToken:  Use this parameter if you receive a <code>NextToken</code> response in a previous request that indicates that there is more output available. Set it to the value of the previous call's <code>NextToken</code> response to indicate where the output should continue from.<br/>ParentId:  The unique identifier (ID) of the root or OU whose child OUs you want to list. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for a parent ID string requires one of the following: <ul> <li> Root: a string that begins with "r-" followed by from 4 to 32 lower-case letters or digits. </li> <li> Organizational unit (OU): a string that begins with "ou-" followed by from 4 to 32 lower-case letters or digits (the ID of the root that the OU is in) followed by a second "-" dash and from 8 to 32 additional lower-case letters or digits. </li> </ul><br/>
        :return: NextToken:  If present, this value indicates that there is more output available than is included in the current response. Use this value in the <code>NextToken</code> request parameter in a subsequent call to the operation to get the next part of the output. You should repeat this until the <code>NextToken</code> response element comes back as <code>null</code>.<br/>OrganizationalUnits:  A list of the OUs in the specified root or parent OU.<br/>        """
        pass

    def list_parents(self, ChildId: str, NextToken: str=None, MaxResults: int=None) -> ListParentsResponse:
        """Lists the root or organizational units (OUs) that serve as the immediate parent of the specified child OU or account. This operation, along with <a>ListChildren</a> enables you to traverse the tree structure that makes up this root. <note> Always check the <code>NextToken</code> response parameter for a <code>null</code> value when calling a <code>List*</code> operation. These operations can occasionally return an empty set of results even when there are more results available. The <code>NextToken</code> response parameter value is <code>null</code> <i>only</i> when there are no more results to display. </note> This operation can be called only from the organization's master account. <note> In the current release, a child can have only a single parent.  </note>

        :param self:
        :param MaxResults:  (Optional) Use this to limit the number of results you want included per page in the response. If you do not include this parameter, it defaults to a value that is specific to the operation. If additional items exist beyond the maximum you specify, the <code>NextToken</code> response element is present and has a value (is not null). Include that value as the <code>NextToken</code> request parameter in the next call to the operation to get the next part of the results. Note that Organizations might return fewer results than the maximum even when there are more results available. You should check <code>NextToken</code> after every operation to ensure that you receive all of the results.<br/>NextToken:  Use this parameter if you receive a <code>NextToken</code> response in a previous request that indicates that there is more output available. Set it to the value of the previous call's <code>NextToken</code> response to indicate where the output should continue from.<br/>ChildId:  The unique identifier (ID) of the OU or account whose parent containers you want to list. Don't specify a root. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for a child ID string requires one of the following: <ul> <li> Account: a string that consists of exactly 12 digits. </li> <li> Organizational unit (OU): a string that begins with "ou-" followed by from 4 to 32 lower-case letters or digits (the ID of the root that contains the OU) followed by a second "-" dash and from 8 to 32 additional lower-case letters or digits. </li> </ul><br/>
        :return: NextToken:  If present, this value indicates that there is more output available than is included in the current response. Use this value in the <code>NextToken</code> request parameter in a subsequent call to the operation to get the next part of the output. You should repeat this until the <code>NextToken</code> response element comes back as <code>null</code>.<br/>Parents:  A list of parents for the specified child account or OU.<br/>        """
        pass

    def list_policies(self, Filter: str, NextToken: str=None, MaxResults: int=None) -> ListPoliciesResponse:
        """Retrieves the list of all policies in an organization of a specified type. <note> Always check the <code>NextToken</code> response parameter for a <code>null</code> value when calling a <code>List*</code> operation. These operations can occasionally return an empty set of results even when there are more results available. The <code>NextToken</code> response parameter value is <code>null</code> <i>only</i> when there are no more results to display. </note> This operation can be called only from the organization's master account.

        :param self:
        :param MaxResults:  (Optional) Use this to limit the number of results you want included per page in the response. If you do not include this parameter, it defaults to a value that is specific to the operation. If additional items exist beyond the maximum you specify, the <code>NextToken</code> response element is present and has a value (is not null). Include that value as the <code>NextToken</code> request parameter in the next call to the operation to get the next part of the results. Note that Organizations might return fewer results than the maximum even when there are more results available. You should check <code>NextToken</code> after every operation to ensure that you receive all of the results.<br/>NextToken:  Use this parameter if you receive a <code>NextToken</code> response in a previous request that indicates that there is more output available. Set it to the value of the previous call's <code>NextToken</code> response to indicate where the output should continue from.<br/>Filter:  Specifies the type of policy that you want to include in the response.<br/>
        :return: NextToken:  If present, this value indicates that there is more output available than is included in the current response. Use this value in the <code>NextToken</code> request parameter in a subsequent call to the operation to get the next part of the output. You should repeat this until the <code>NextToken</code> response element comes back as <code>null</code>.<br/>Policies:  A list of policies that match the filter criteria in the request. The output list doesn't include the policy contents. To see the content for a policy, see <a>DescribePolicy</a>.<br/>        """
        pass

    def list_policies_for_target(self, Filter: str, TargetId: str, NextToken: str=None, MaxResults: int=None) -> ListPoliciesForTargetResponse:
        """Lists the policies that are directly attached to the specified target root, organizational unit (OU), or account. You must specify the policy type that you want included in the returned list. <note> Always check the <code>NextToken</code> response parameter for a <code>null</code> value when calling a <code>List*</code> operation. These operations can occasionally return an empty set of results even when there are more results available. The <code>NextToken</code> response parameter value is <code>null</code> <i>only</i> when there are no more results to display. </note> This operation can be called only from the organization's master account.

        :param self:
        :param MaxResults:  (Optional) Use this to limit the number of results you want included per page in the response. If you do not include this parameter, it defaults to a value that is specific to the operation. If additional items exist beyond the maximum you specify, the <code>NextToken</code> response element is present and has a value (is not null). Include that value as the <code>NextToken</code> request parameter in the next call to the operation to get the next part of the results. Note that Organizations might return fewer results than the maximum even when there are more results available. You should check <code>NextToken</code> after every operation to ensure that you receive all of the results.<br/>NextToken:  Use this parameter if you receive a <code>NextToken</code> response in a previous request that indicates that there is more output available. Set it to the value of the previous call's <code>NextToken</code> response to indicate where the output should continue from.<br/>Filter:  The type of policy that you want to include in the returned list.<br/>TargetId:  The unique identifier (ID) of the root, organizational unit, or account whose policies you want to list. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for a target ID string requires one of the following: <ul> <li> Root: a string that begins with "r-" followed by from 4 to 32 lower-case letters or digits. </li> <li> Account: a string that consists of exactly 12 digits. </li> <li> Organizational unit (OU): a string that begins with "ou-" followed by from 4 to 32 lower-case letters or digits (the ID of the root that the OU is in) followed by a second "-" dash and from 8 to 32 additional lower-case letters or digits. </li> </ul><br/>
        :return: NextToken:  If present, this value indicates that there is more output available than is included in the current response. Use this value in the <code>NextToken</code> request parameter in a subsequent call to the operation to get the next part of the output. You should repeat this until the <code>NextToken</code> response element comes back as <code>null</code>.<br/>Policies:  The list of policies that match the criteria in the request.<br/>        """
        pass

    def list_roots(self, NextToken: str=None, MaxResults: int=None) -> ListRootsResponse:
        """Lists the roots that are defined in the current organization. <note> Always check the <code>NextToken</code> response parameter for a <code>null</code> value when calling a <code>List*</code> operation. These operations can occasionally return an empty set of results even when there are more results available. The <code>NextToken</code> response parameter value is <code>null</code> <i>only</i> when there are no more results to display. </note> This operation can be called only from the organization's master account. <note> Policy types can be enabled and disabled in roots. This is distinct from whether they're available in the organization. When you enable all features, you make policy types available for use in that organization. Individual policy types can then be enabled and disabled in a root. To see the availability of a policy type in an organization, use <a>DescribeOrganization</a>. </note>

        :param self:
        :param MaxResults:  (Optional) Use this to limit the number of results you want included per page in the response. If you do not include this parameter, it defaults to a value that is specific to the operation. If additional items exist beyond the maximum you specify, the <code>NextToken</code> response element is present and has a value (is not null). Include that value as the <code>NextToken</code> request parameter in the next call to the operation to get the next part of the results. Note that Organizations might return fewer results than the maximum even when there are more results available. You should check <code>NextToken</code> after every operation to ensure that you receive all of the results.<br/>NextToken:  Use this parameter if you receive a <code>NextToken</code> response in a previous request that indicates that there is more output available. Set it to the value of the previous call's <code>NextToken</code> response to indicate where the output should continue from.<br/>
        :return: NextToken:  If present, this value indicates that there is more output available than is included in the current response. Use this value in the <code>NextToken</code> request parameter in a subsequent call to the operation to get the next part of the output. You should repeat this until the <code>NextToken</code> response element comes back as <code>null</code>.<br/>Roots:  A list of roots that are defined in an organization.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceId: str, NextToken: str=None) -> ListTagsForResourceResponse:
        """Lists tags for the specified resource.  Currently, you can list tags on an account in AWS Organizations.

        :param self:
        :param NextToken:  Use this parameter if you receive a <code>NextToken</code> response in a previous request that indicates that there is more output available. Set it to the value of the previous call's <code>NextToken</code> response to indicate where the output should continue from.<br/>ResourceId:  The ID of the resource that you want to retrieve tags for. <br/>
        :return: NextToken:  If present, this value indicates that there is more output available than is included in the current response. Use this value in the <code>NextToken</code> request parameter in a subsequent call to the operation to get the next part of the output. You should repeat this until the <code>NextToken</code> response element comes back as <code>null</code>.<br/>Tags:  The tags that are assigned to the resource.<br/>        """
        pass

    def list_targets_for_policy(self, PolicyId: str, NextToken: str=None, MaxResults: int=None) -> ListTargetsForPolicyResponse:
        """Lists all the roots, organizational units (OUs), and accounts that the specified policy is attached to. <note> Always check the <code>NextToken</code> response parameter for a <code>null</code> value when calling a <code>List*</code> operation. These operations can occasionally return an empty set of results even when there are more results available. The <code>NextToken</code> response parameter value is <code>null</code> <i>only</i> when there are no more results to display. </note> This operation can be called only from the organization's master account.

        :param self:
        :param MaxResults:  (Optional) Use this to limit the number of results you want included per page in the response. If you do not include this parameter, it defaults to a value that is specific to the operation. If additional items exist beyond the maximum you specify, the <code>NextToken</code> response element is present and has a value (is not null). Include that value as the <code>NextToken</code> request parameter in the next call to the operation to get the next part of the results. Note that Organizations might return fewer results than the maximum even when there are more results available. You should check <code>NextToken</code> after every operation to ensure that you receive all of the results.<br/>NextToken:  Use this parameter if you receive a <code>NextToken</code> response in a previous request that indicates that there is more output available. Set it to the value of the previous call's <code>NextToken</code> response to indicate where the output should continue from.<br/>PolicyId:  The unique identifier (ID) of the policy whose attachments you want to know. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for a policy ID string requires "p-" followed by from 8 to 128 lower-case letters or digits.<br/>
        :return: NextToken:  If present, this value indicates that there is more output available than is included in the current response. Use this value in the <code>NextToken</code> request parameter in a subsequent call to the operation to get the next part of the output. You should repeat this until the <code>NextToken</code> response element comes back as <code>null</code>.<br/>Targets:  A list of structures, each of which contains details about one of the entities to which the specified policy is attached.<br/>        """
        pass

    def move_account(self, DestinationParentId: str, SourceParentId: str, AccountId: str):
        """Moves an account from its current source parent root or organizational unit (OU) to the specified destination parent root or OU. This operation can be called only from the organization's master account.

        :param self:
        :param DestinationParentId:  The unique identifier (ID) of the root or organizational unit that you want to move the account to. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for a parent ID string requires one of the following: <ul> <li> Root: a string that begins with "r-" followed by from 4 to 32 lower-case letters or digits. </li> <li> Organizational unit (OU): a string that begins with "ou-" followed by from 4 to 32 lower-case letters or digits (the ID of the root that the OU is in) followed by a second "-" dash and from 8 to 32 additional lower-case letters or digits. </li> </ul><br/>SourceParentId:  The unique identifier (ID) of the root or organizational unit that you want to move the account from. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for a parent ID string requires one of the following: <ul> <li> Root: a string that begins with "r-" followed by from 4 to 32 lower-case letters or digits. </li> <li> Organizational unit (OU): a string that begins with "ou-" followed by from 4 to 32 lower-case letters or digits (the ID of the root that the OU is in) followed by a second "-" dash and from 8 to 32 additional lower-case letters or digits. </li> </ul><br/>AccountId:  The unique identifier (ID) of the account that you want to move. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for an account ID string requires exactly 12 digits.<br/>
        :return:         """
        pass

    def remove_account_from_organization(self, AccountId: str):
        """Removes the specified account from the organization. The removed account becomes a standalone account that isn't a member of any organization. It's no longer subject to any policies and is responsible for its own bill payments. The organization's master account is no longer charged for any expenses accrued by the member account after it's removed from the organization. This operation can be called only from the organization's master account. Member accounts can remove themselves with <a>LeaveOrganization</a> instead. <important> You can remove an account from your organization only if the account is configured with the information required to operate as a standalone account. When you create an account in an organization using the AWS Organizations console, API, or CLI commands, the information required of standalone accounts is <i>not</i> automatically collected. For an account that you want to make standalone, you must accept the end user license agreement (EULA), choose a support plan, provide and verify the required contact information, and provide a current payment method. AWS uses the payment method to charge for any billable (not free tier) AWS activity that occurs while the account isn't attached to an organization. To remove an account that doesn't yet have this information, you must sign in as the member account and follow the steps at <a href="http://docs.aws.amazon.com/organizations/latest/userguide/orgs_manage_accounts_remove.html#leave-without-all-info"> To leave an organization when all required account information has not yet been provided</a> in the <i>AWS Organizations User Guide.</i>  </important>

        :param self:
        :param AccountId:  The unique identifier (ID) of the member account that you want to remove from the organization. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for an account ID string requires exactly 12 digits.<br/>
        :return:         """
        pass

    def tag_resource(self, Tags: List[Tag], ResourceId: str):
        """Adds one or more tags to the specified resource. Currently, you can tag and untag accounts in AWS Organizations.

        :param self:
        :param Tags:  The tag to add to the specified resource. Specifying the tag key is required. You can set the value of a tag to an empty string, but you can't set the value of a tag to null.<br/>ResourceId:  The ID of the resource to add a tag to.<br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[TagKey], ResourceId: str):
        """Removes a tag from the specified resource.  Currently, you can tag and untag accounts in AWS Organizations.

        :param self:
        :param TagKeys:  The tag to remove from the specified resource.<br/>ResourceId:  The ID of the resource to remove the tag from.<br/>
        :return:         """
        pass

    def update_organizational_unit(self, OrganizationalUnitId: str, Name: str=None) -> UpdateOrganizationalUnitResponse:
        """Renames the specified organizational unit (OU). The ID and ARN don't change. The child OUs and accounts remain in place, and any attached policies of the OU remain attached.  This operation can be called only from the organization's master account.

        :param self:
        :param Name:  The new name that you want to assign to the OU. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> that is used to validate this parameter is a string of any of the characters in the ASCII character range.<br/>OrganizationalUnitId:  The unique identifier (ID) of the OU that you want to rename. You can get the ID from the <a>ListOrganizationalUnitsForParent</a> operation. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for an organizational unit ID string requires "ou-" followed by from 4 to 32 lower-case letters or digits (the ID of the root that contains the OU) followed by a second "-" dash and from 8 to 32 additional lower-case letters or digits.<br/>
        :return: OrganizationalUnit:  A structure that contains the details about the specified OU, including its new name.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The friendly name of this OU. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> that is used to validate this parameter is a string of any of the characters in the ASCII character range.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of this OU. For more information about ARNs in Organizations, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_permissions.html#orgs-permissions-arns">ARN Formats Supported by Organizations</a> in the <i>AWS Organizations User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier (ID) associated with this OU. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for an organizational unit ID string requires "ou-" followed by from 4 to 32 lower-case letters or digits (the ID of the root that contains the OU) followed by a second "-" dash and from 8 to 32 additional lower-case letters or digits.<br/>        """
        pass

    def update_policy(self, PolicyId: str, Name: str=None, Description: str=None, Content: str=None) -> UpdatePolicyResponse:
        """Updates an existing policy with a new name, description, or content. If you don't supply any parameter, that value remains unchanged. You can't change a policy's type. This operation can be called only from the organization's master account.

        :param self:
        :param Content:  If provided, the new content for the policy. The text must be correctly formatted JSON that complies with the syntax for the policy's type. For more information, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_reference_scp-syntax.html">Service Control Policy Syntax</a> in the <i>AWS Organizations User Guide.</i> <br/>Description:  If provided, the new description for the policy.<br/>Name:  If provided, the new name for the policy. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> that is used to validate this parameter is a string of any of the characters in the ASCII character range.<br/>PolicyId:  The unique identifier (ID) of the policy that you want to update. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for a policy ID string requires "p-" followed by from 8 to 128 lower-case letters or digits.<br/>
        :return: Policy:  A structure that contains details about the updated policy, showing the requested changes.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Content</i> The text content of the policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PolicySummary</i> A structure that contains additional details about the policy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PolicySummary</b>: A structure that contains additional details about the policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AwsManaged</i> A boolean value that indicates whether the specified policy is an AWS managed policy. If true, then you can attach the policy to roots, OUs, or accounts, but you cannot edit it.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The friendly name of the policy. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> that is used to validate this parameter is a string of any of the characters in the ASCII character range.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the policy. For more information about ARNs in Organizations, see <a href="https://docs.aws.amazon.com/organizations/latest/userguide/orgs_permissions.html#orgs-permissions-arns">ARN Formats Supported by Organizations</a> in the <i>AWS Organizations User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier (ID) of the policy. The <a href="http://wikipedia.org/wiki/regex">regex pattern</a> for a policy ID string requires "p-" followed by from 8 to 128 lower-case letters or digits.<br/>        """
        pass



class Personalize(BaseClient):
    """Amazon Personalize is a machine learning service that makes it easy to add individualized recommendations to customers."""


    class ListCampaignsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_campaigns` for valid parameters"""
            pass
    class ListDatasetGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_dataset_groups` for valid parameters"""
            pass
    class ListDatasetImportJobsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_dataset_import_jobs` for valid parameters"""
            pass
    class ListDatasetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_datasets` for valid parameters"""
            pass
    class ListEventTrackersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_event_trackers` for valid parameters"""
            pass
    class ListRecipesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_recipes` for valid parameters"""
            pass
    class ListSchemasPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_schemas` for valid parameters"""
            pass
    class ListSolutionVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_solution_versions` for valid parameters"""
            pass
    class ListSolutionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_solutions` for valid parameters"""
            pass


    class CreateCampaignRequest(Mapping):
        pass
    
    class CreateCampaignResponse(Mapping):
        pass
    
    class CreateDatasetRequest(Mapping):
        pass
    
    class CreateDatasetResponse(Mapping):
        pass
    
    class CreateDatasetGroupRequest(Mapping):
        pass
    
    class CreateDatasetGroupResponse(Mapping):
        pass
    
    class CreateDatasetImportJobRequest(Mapping):
        pass
    
    class CreateDatasetImportJobResponse(Mapping):
        pass
    
    class DataSource(Mapping):
        pass
    
    class CreateEventTrackerRequest(Mapping):
        pass
    
    class CreateEventTrackerResponse(Mapping):
        pass
    
    class CreateSchemaRequest(Mapping):
        pass
    
    class CreateSchemaResponse(Mapping):
        pass
    
    class CreateSolutionRequest(Mapping):
        pass
    
    class CreateSolutionResponse(Mapping):
        pass
    
    class PerformHPO(object):
        pass
    
    class PerformAutoML(object):
        pass
    
    class SolutionConfig(Mapping):
        pass
    
    class CreateSolutionVersionRequest(Mapping):
        pass
    
    class CreateSolutionVersionResponse(Mapping):
        pass
    
    class DeleteCampaignRequest(Mapping):
        pass
    
    class DeleteDatasetRequest(Mapping):
        pass
    
    class DeleteDatasetGroupRequest(Mapping):
        pass
    
    class DeleteEventTrackerRequest(Mapping):
        pass
    
    class DeleteSchemaRequest(Mapping):
        pass
    
    class DeleteSolutionRequest(Mapping):
        pass
    
    class DescribeAlgorithmRequest(Mapping):
        pass
    
    class DescribeAlgorithmResponse(Mapping):
        pass
    
    class DescribeCampaignRequest(Mapping):
        pass
    
    class DescribeCampaignResponse(Mapping):
        pass
    
    class DescribeDatasetRequest(Mapping):
        pass
    
    class DescribeDatasetResponse(Mapping):
        pass
    
    class DescribeDatasetGroupRequest(Mapping):
        pass
    
    class DescribeDatasetGroupResponse(Mapping):
        pass
    
    class DescribeDatasetImportJobRequest(Mapping):
        pass
    
    class DescribeDatasetImportJobResponse(Mapping):
        pass
    
    class DescribeEventTrackerRequest(Mapping):
        pass
    
    class DescribeEventTrackerResponse(Mapping):
        pass
    
    class DescribeFeatureTransformationRequest(Mapping):
        pass
    
    class DescribeFeatureTransformationResponse(Mapping):
        pass
    
    class DescribeRecipeRequest(Mapping):
        pass
    
    class DescribeRecipeResponse(Mapping):
        pass
    
    class DescribeSchemaRequest(Mapping):
        pass
    
    class DescribeSchemaResponse(Mapping):
        pass
    
    class DescribeSolutionRequest(Mapping):
        pass
    
    class DescribeSolutionResponse(Mapping):
        pass
    
    class DescribeSolutionVersionRequest(Mapping):
        pass
    
    class DescribeSolutionVersionResponse(Mapping):
        pass
    
    class GetSolutionMetricsRequest(Mapping):
        pass
    
    class GetSolutionMetricsResponse(Mapping):
        pass
    
    class ListCampaignsRequest(Mapping):
        pass
    
    class ListCampaignsResponse(Mapping):
        pass
    
    class ListDatasetGroupsRequest(Mapping):
        pass
    
    class ListDatasetGroupsResponse(Mapping):
        pass
    
    class ListDatasetImportJobsRequest(Mapping):
        pass
    
    class ListDatasetImportJobsResponse(Mapping):
        pass
    
    class ListDatasetsRequest(Mapping):
        pass
    
    class ListDatasetsResponse(Mapping):
        pass
    
    class ListEventTrackersRequest(Mapping):
        pass
    
    class ListEventTrackersResponse(Mapping):
        pass
    
    class ListRecipesRequest(Mapping):
        pass
    
    class ListRecipesResponse(Mapping):
        pass
    
    class ListSchemasRequest(Mapping):
        pass
    
    class ListSchemasResponse(Mapping):
        pass
    
    class ListSolutionVersionsRequest(Mapping):
        pass
    
    class ListSolutionVersionsResponse(Mapping):
        pass
    
    class ListSolutionsRequest(Mapping):
        pass
    
    class ListSolutionsResponse(Mapping):
        pass
    
    class UpdateCampaignRequest(Mapping):
        pass
    
    class UpdateCampaignResponse(Mapping):
        pass
    
    def create_campaign(self, minProvisionedTPS: int, solutionVersionArn: str, name: str) -> CreateCampaignResponse:
        """Creates a campaign by deploying a solution version. When a client calls the <a href="https://docs.aws.amazon.com/personalize/latest/dg/API_RS_GetRecommendations.html">GetRecommendations</a> and <a href="https://docs.aws.amazon.com/personalize/latest/dg/API_RS_GetPersonalizedRanking.html">GetPersonalizedRanking</a> APIs, a campaign is specified in the request.  <b>Minimum Provisioned TPS and Auto-Scaling</b>  A transaction is a single <code>GetRecommendations</code> or <code>GetPersonalizedRanking</code> call. Transactions per second (TPS) is the throughput and unit of billing for Amazon Personalize. The minimum provisioned TPS (<code>minProvisionedTPS</code>) specifies the baseline throughput provisioned by Amazon Personalize, and thus, the minimum billing charge. If your TPS increases beyond <code>minProvisionedTPS</code>, Amazon Personalize auto-scales the provisioned capacity up and down, but never below <code>minProvisionedTPS</code>, to maintain a 70% utilization. There's a short time delay while the capacity is increased that might cause loss of transactions. It's recommended to start with a low <code>minProvisionedTPS</code>, track your usage using Amazon CloudWatch metrics, and then increase the <code>minProvisionedTPS</code> as necessary.  <b>Status</b>  A campaign can be in one of the following states: <ul> <li> CREATE PENDING &gt; CREATE IN_PROGRESS &gt; ACTIVE -or- CREATE FAILED </li> <li> DELETE PENDING &gt; DELETE IN_PROGRESS </li> </ul> To get the campaign status, call <a>DescribeCampaign</a>. <note> Wait until the <code>status</code> of the campaign is <code>ACTIVE</code> before asking the campaign for recommendations. </note> <p class="title"> <b>Related APIs</b>  <ul> <li>  <a>ListCampaigns</a>  </li> <li>  <a>DescribeCampaign</a>  </li> <li>  <a>UpdateCampaign</a>  </li> <li>  <a>DeleteCampaign</a>  </li> </ul>

        :param self:
        :param minProvisionedTPS:  Specifies the requested minimum provisioned transactions (recommendations) per second that Amazon Personalize will support.<br/>solutionVersionArn:  The Amazon Resource Name (ARN) of the solution version to deploy.<br/>name:  A name for the new campaign. The campaign name must be unique within your account.<br/>
        :return: campaignArn:  The Amazon Resource Name (ARN) of the campaign.<br/>        """
        pass

    def create_dataset(self, datasetType: str, datasetGroupArn: str, schemaArn: str, name: str) -> CreateDatasetResponse:
        """Creates an empty dataset and adds it to the specified dataset group. Use <a>CreateDatasetImportJob</a> to import your training data to a dataset. There are three types of datasets: <ul> <li> Interactions </li> <li> Items </li> <li> Users </li> </ul> Each dataset type has an associated schema with required field types. Only the <code>Interactions</code> dataset is required in order to train a model (also referred to as creating a solution). A dataset can be in one of the following states: <ul> <li> CREATE PENDING &gt; CREATE IN_PROGRESS &gt; ACTIVE -or- CREATE FAILED </li> <li> DELETE PENDING &gt; DELETE IN_PROGRESS </li> </ul> To get the status of the dataset, call <a>DescribeDataset</a>. <p class="title"> <b>Related APIs</b>  <ul> <li>  <a>CreateDatasetGroup</a>  </li> <li>  <a>ListDatasets</a>  </li> <li>  <a>DescribeDataset</a>  </li> <li>  <a>DeleteDataset</a>  </li> </ul>

        :param self:
        :param datasetType:  The type of dataset. One of the following (case insensitive) values: <ul> <li> Interactions </li> <li> Items </li> <li> Users </li> </ul><br/>datasetGroupArn:  The Amazon Resource Name (ARN) of the dataset group to add the dataset to.<br/>schemaArn:  The ARN of the schema to associate with the dataset. The schema defines the dataset fields.<br/>name:  The name for the dataset.<br/>
        :return: datasetArn:  The ARN of the dataset.<br/>        """
        pass

    def create_dataset_group(self, name: str, roleArn: str=None, kmsKeyArn: str=None) -> CreateDatasetGroupResponse:
        """Creates an empty dataset group. A dataset group contains related datasets that supply data for training a model. A dataset group can contain at most three datasets, one for each type of dataset: <ul> <li> Interactions </li> <li> Items </li> <li> Users </li> </ul> To train a model (create a solution), a dataset group that contains an <code>Interactions</code> dataset is required. Call <a>CreateDataset</a> to add a dataset to the group. A dataset group can be in one of the following states: <ul> <li> CREATE PENDING &gt; CREATE IN_PROGRESS &gt; ACTIVE -or- CREATE FAILED </li> <li> DELETE PENDING </li> </ul> To get the status of the dataset group, call <a>DescribeDatasetGroup</a>. If the status shows as CREATE FAILED, the response includes a <code>failureReason</code> key, which describes why the creation failed. <note> You must wait until the <code>status</code> of the dataset group is <code>ACTIVE</code> before adding a dataset to the group. </note> You can specify an AWS Key Management Service (KMS) key to encrypt the datasets in the group. If you specify a KMS key, you must also include an AWS Identity and Access Management (IAM) role that has permission to access the key. <p class="title"> <b>APIs that require a dataset group ARN in the request</b>  <ul> <li>  <a>CreateDataset</a>  </li> <li>  <a>CreateEventTracker</a>  </li> <li>  <a>CreateSolution</a>  </li> </ul> <p class="title"> <b>Related APIs</b>  <ul> <li>  <a>ListDatasetGroups</a>  </li> <li>  <a>DescribeDatasetGroup</a>  </li> <li>  <a>DeleteDatasetGroup</a>  </li> </ul>

        :param self:
        :param kmsKeyArn:  The Amazon Resource Name (ARN) of a KMS key used to encrypt the datasets.<br/>roleArn:  The ARN of the IAM role that has permissions to access the KMS key. Supplying an IAM role is only valid when also specifying a KMS key.<br/>name:  The name for the new dataset group.<br/>
        :return: datasetGroupArn:  The Amazon Resource Name (ARN) of the new dataset group.<br/>        """
        pass

    def create_dataset_import_job(self, roleArn: str, dataSource: DataSource, datasetArn: str, jobName: str) -> CreateDatasetImportJobResponse:
        """Creates a job that imports training data from your data source (an Amazon S3 bucket) to an Amazon Personalize dataset. To allow Amazon Personalize to import the training data, you must specify an AWS Identity and Access Management (IAM) role that has permission to read from the data source. <important> The dataset import job replaces any previous data in the dataset. </important>  <b>Status</b>  A dataset import job can be in one of the following states: <ul> <li> CREATE PENDING &gt; CREATE IN_PROGRESS &gt; ACTIVE -or- CREATE FAILED </li> </ul> To get the status of the import job, call <a>DescribeDatasetImportJob</a>, providing the Amazon Resource Name (ARN) of the dataset import job. The dataset import is complete when the status shows as ACTIVE. If the status shows as CREATE FAILED, the response includes a <code>failureReason</code> key, which describes why the job failed. <note> Importing takes time. You must wait until the status shows as ACTIVE before training a model using the dataset. </note> <p class="title"> <b>Related APIs</b>  <ul> <li>  <a>ListDatasetImportJobs</a>  </li> <li>  <a>DescribeDatasetImportJob</a>  </li> </ul>

        :param self:
        :param roleArn:  The ARN of the IAM role that has permissions to read from the Amazon S3 data source.<br/>dataSource:  The Amazon S3 bucket that contains the training data to import.:<br/>datasetArn:  The ARN of the dataset that receives the imported data.<br/>jobName:  The name for the dataset import job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>dataLocation</i> The path to the Amazon S3 bucket where the data that you want to upload to your dataset is stored. For example:   <code>s3://bucket-name/training-data.csv</code> <br/>
        :return: datasetImportJobArn:  The ARN of the dataset import job.<br/>        """
        pass

    def create_event_tracker(self, datasetGroupArn: str, name: str) -> CreateEventTrackerResponse:
        """Creates an event tracker that you use when sending event data to the specified dataset group using the <a href="https://docs.aws.amazon.com/personalize/latest/dg/API_UBS_PutEvents.html">PutEvents</a> API. When Amazon Personalize creates an event tracker, it also creates an <i>event-interactions</i> dataset in the dataset group associated with the event tracker. The event-interactions dataset stores the event data from the <code>PutEvents</code> call. The contents of this dataset are not available to the user. <note> Only one event tracker can be associated with a dataset group. You will get an error if you call <code>CreateEventTracker</code> using the same dataset group as an existing event tracker. </note> When you send event data you include your tracking ID. The tracking ID identifies the customer and authorizes the customer to send the data. The event tracker can be in one of the following states: <ul> <li> CREATE PENDING &gt; CREATE IN_PROGRESS &gt; ACTIVE -or- CREATE FAILED </li> <li> DELETE PENDING &gt; DELETE IN_PROGRESS </li> </ul> To get the status of the event tracker, call <a>DescribeEventTracker</a>. <note> The event tracker must be in the ACTIVE state before using the tracking ID. </note> <p class="title"> <b>Related APIs</b>  <ul> <li>  <a>ListEventTrackers</a>  </li> <li>  <a>DescribeEventTracker</a>  </li> <li>  <a>DeleteEventTracker</a>  </li> </ul>

        :param self:
        :param datasetGroupArn:  The Amazon Resource Name (ARN) of the dataset group that receives the event data.<br/>name:  The name for the event tracker.<br/>
        :return: trackingId:  The ID of the event tracker. Include this ID in requests to the <a href="https://docs.aws.amazon.com/personalize/latest/dg/API_UBS_PutEvents.html">PutEvents</a> API.<br/>eventTrackerArn:  The ARN of the event tracker.<br/>        """
        pass

    def create_schema(self, schema: str, name: str) -> CreateSchemaResponse:
        """Creates an Amazon Personalize schema from the specified schema string. The schema you create must be in Avro JSON format. Amazon Personalize recognizes three schema variants. Each schema is associated with a dataset type and has a set of required field and keywords. You specify a schema when you call <a>CreateDataset</a>. <p class="title"> <b>Related APIs</b>  <ul> <li>  <a>ListSchemas</a>  </li> <li>  <a>DescribeSchema</a>  </li> <li>  <a>DeleteSchema</a>  </li> </ul>

        :param self:
        :param schema:  A schema in Avro JSON format.<br/>name:  The name for the schema.<br/>
        :return: schemaArn:  The Amazon Resource Name (ARN) of the created schema.<br/>        """
        pass

    def create_solution(self, datasetGroupArn: str, name: str, performHPO: PerformHPO=None, performAutoML: PerformAutoML=None, recipeArn: str=None, eventType: str=None, solutionConfig: SolutionConfig=None) -> CreateSolutionResponse:
        """Creates the configuration for training a model. A trained model is known as a solution. After the configuration is created, you train the model (create a solution) by calling the <a>CreateSolutionVersion</a> operation. Every time you call <code>CreateSolutionVersion</code>, a new version of the solution is created. After creating a solution version, you check its accuracy by calling <a>GetSolutionMetrics</a>. When you are satisfied with the version, you deploy it using <a>CreateCampaign</a>. The campaign provides recommendations to a client through the <a href="https://docs.aws.amazon.com/personalize/latest/dg/API_RS_GetRecommendations.html">GetRecommendations</a> API. To train a model, Amazon Personalize requires training data and a recipe. The training data comes from the dataset group that you provide in the request. A recipe specifies the training algorithm and a feature transformation. You can specify one of the predefined recipes provided by Amazon Personalize. Alternatively, you can specify <code>performAutoML</code> and Amazon Personalize will analyze your data and select the optimum USER_PERSONALIZATION recipe for you.  <b>Status</b>  A solution can be in one of the following states: <ul> <li> CREATE PENDING &gt; CREATE IN_PROGRESS &gt; ACTIVE -or- CREATE FAILED </li> <li> DELETE PENDING &gt; DELETE IN_PROGRESS </li> </ul> To get the status of the solution, call <a>DescribeSolution</a>. Wait until the status shows as ACTIVE before calling <code>CreateSolutionVersion</code>. <p class="title"> <b>Related APIs</b>  <ul> <li>  <a>ListSolutions</a>  </li> <li>  <a>CreateSolutionVersion</a>  </li> <li>  <a>DescribeSolution</a>  </li> <li>  <a>DeleteSolution</a>  </li> </ul> <ul> <li>  <a>ListSolutionVersions</a>  </li> <li>  <a>DescribeSolutionVersion</a>  </li> </ul>

        :param self:
        :param solutionConfig:  The configuration to use with the solution. When <code>performAutoML</code> is set to true, Amazon Personalize only evaluates the <code>autoMLConfig</code> section of the solution configuration.:<br/>eventType:  When your have multiple event types (using an <code>EVENT_TYPE</code> schema field), this parameter specifies which event type (for example, 'click' or 'like') is used for training the model.<br/>datasetGroupArn:  The Amazon Resource Name (ARN) of the dataset group that provides the training data.<br/>recipeArn:  The ARN of the recipe to use for model training. Only specified when <code>performAutoML</code> is false.<br/>performAutoML:  Whether to perform automated machine learning (AutoML). The default is <code>false</code>. For this case, you must specify <code>recipeArn</code>. When set to <code>true</code>, Amazon Personalize analyzes your training data and selects the optimal USER_PERSONALIZATION recipe and hyperparameters. In this case, you must omit <code>recipeArn</code>. Amazon Personalize determines the optimal recipe by running tests with different values for the hyperparameters. AutoML lengthens the training process as compared to selecting a specific recipe.<br/>performHPO:  Whether to perform hyperparameter optimization (HPO) on the specified or selected recipe. The default is <code>false</code>. When performing AutoML, this parameter is always <code>true</code> and you should not set it to <code>false</code>.<br/>name:  The name for the solution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>autoMLConfig</i> The <a>AutoMLConfig</a> object containing a list of recipes to search when AutoML is performed.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>autoMLConfig</b>: The <a>AutoMLConfig</a> object containing a list of recipes to search when AutoML is performed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>featureTransformationParameters</i> Lists the feature transformation parameters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>algorithmHyperParameters</i> Lists the hyperparameter names and ranges.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>hpoConfig</i> Describes the properties for hyperparameter optimization (HPO). For use with the bring-your-own-recipe feature. Not used with Amazon Personalize predefined recipes.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>hpoConfig</b>: Describes the properties for hyperparameter optimization (HPO). For use with the bring-your-own-recipe feature. Not used with Amazon Personalize predefined recipes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>eventValueThreshold</i> Only events with a value greater than or equal to this threshold are used for training a model.<br/>The metric to optimize during HPO.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>hpoObjective</b>: The metric to optimize during HPO.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>recipeList</i> The list of candidate recipes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>metricName</i> The metric to optimize.<br/>
        :return: solutionArn:  The ARN of the solution.<br/>        """
        pass

    def create_solution_version(self, solutionArn: str) -> CreateSolutionVersionResponse:
        """Trains or retrains an active solution. A solution is created using the <a>CreateSolution</a> operation and must be in the ACTIVE state before calling <code>CreateSolutionVersion</code>. A new version of the solution is created every time you call this operation.  <b>Status</b>  A solution version can be in one of the following states: <ul> <li> CREATE PENDING &gt; CREATE IN_PROGRESS &gt; ACTIVE -or- CREATE FAILED </li> </ul> To get the status of the version, call <a>DescribeSolutionVersion</a>. Wait until the status shows as ACTIVE before calling <code>CreateCampaign</code>. If the status shows as CREATE FAILED, the response includes a <code>failureReason</code> key, which describes why the job failed. <p class="title"> <b>Related APIs</b>  <ul> <li>  <a>ListSolutionVersions</a>  </li> <li>  <a>DescribeSolutionVersion</a>  </li> </ul> <ul> <li>  <a>ListSolutions</a>  </li> <li>  <a>CreateSolution</a>  </li> <li>  <a>DescribeSolution</a>  </li> <li>  <a>DeleteSolution</a>  </li> </ul>

        :param self:
        :param solutionArn:  The Amazon Resource Name (ARN) of the solution containing the training configuration information.<br/>
        :return: solutionVersionArn:  The ARN of the new solution version.<br/>        """
        pass

    def delete_campaign(self, campaignArn: str):
        """Removes a campaign by deleting the solution deployment. The solution that the campaign is based on is not deleted and can be redeployed when needed. A deleted campaign can no longer be specified in a <a href="https://docs.aws.amazon.com/personalize/latest/dg/API_RS_GetRecommendations.html">GetRecommendations</a> request. For more information on campaigns, see <a>CreateCampaign</a>.

        :param self:
        :param campaignArn:  The Amazon Resource Name (ARN) of the campaign to delete.<br/>
        :return:         """
        pass

    def delete_dataset(self, datasetArn: str):
        """Deletes a dataset. You can't delete a dataset if an associated <code>DatasetImportJob</code> or <code>SolutionVersion</code> is in the CREATE PENDING or IN PROGRESS state. For more information on datasets, see <a>CreateDataset</a>.

        :param self:
        :param datasetArn:  The Amazon Resource Name (ARN) of the dataset to delete.<br/>
        :return:         """
        pass

    def delete_dataset_group(self, datasetGroupArn: str):
        """Deletes a dataset group. Before you delete a dataset group, you must delete the following: <ul> <li> All associated event trackers. </li> <li> All associated solutions. </li> <li> All datasets in the dataset group. </li> </ul>

        :param self:
        :param datasetGroupArn:  The ARN of the dataset group to delete.<br/>
        :return:         """
        pass

    def delete_event_tracker(self, eventTrackerArn: str):
        """Deletes the event tracker. Does not delete the event-interactions dataset from the associated dataset group. For more information on event trackers, see <a>CreateEventTracker</a>.

        :param self:
        :param eventTrackerArn:  The Amazon Resource Name (ARN) of the event tracker to delete.<br/>
        :return:         """
        pass

    def delete_schema(self, schemaArn: str):
        """Deletes a schema. Before deleting a schema, you must delete all datasets referencing the schema. For more information on schemas, see <a>CreateSchema</a>.

        :param self:
        :param schemaArn:  The Amazon Resource Name (ARN) of the schema to delete.<br/>
        :return:         """
        pass

    def delete_solution(self, solutionArn: str):
        """Deletes all versions of a solution and the <code>Solution</code> object itself. Before deleting a solution, you must delete all campaigns based on the solution. To determine what campaigns are using the solution, call <a>ListCampaigns</a> and supply the Amazon Resource Name (ARN) of the solution. You can't delete a solution if an associated <code>SolutionVersion</code> is in the CREATE PENDING or IN PROGRESS state. For more information on solutions, see <a>CreateSolution</a>.

        :param self:
        :param solutionArn:  The ARN of the solution to delete.<br/>
        :return:         """
        pass

    def describe_algorithm(self, algorithmArn: str) -> DescribeAlgorithmResponse:
        """Describes the given algorithm.

        :param self:
        :param algorithmArn:  The Amazon Resource Name (ARN) of the algorithm to describe.<br/>
        :return: algorithm:  A listing of the properties of the algorithm.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedDateTime</i> The date and time (in Unix time) that the algorithm was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDateTime</i> The date and time (in Unix time) that the algorithm was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The Amazon Resource Name (ARN) of the role.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>trainingInputMode</i> The training input mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultResourceConfig</i> Specifies the default maximum number of training jobs and parallel training jobs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultHyperParameterRanges</i> Specifies the default hyperparameters, their ranges, and whether they are tunable. A tunable hyperparameter can have its value determined during hyperparameter optimization (HPO).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>defaultHyperParameterRanges</b>: Specifies the default hyperparameters, their ranges, and whether they are tunable. A tunable hyperparameter can have its value determined during hyperparameter optimization (HPO).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultHyperParameters</i> Specifies the default hyperparameters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>algorithmImage</i> The URI of the Docker container for the algorithm image.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>algorithmImage</b>: The URI of the Docker container for the algorithm image.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>algorithmArn</i> The Amazon Resource Name (ARN) of the algorithm.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the algorithm.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>dockerURI</i> The URI of the Docker container for the algorithm image.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the algorithm image.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>categoricalHyperParameterRanges</i> The categorical hyperparameters and their default ranges.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>continuousHyperParameterRanges</i> The continuous hyperparameters and their default ranges.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>integerHyperParameterRanges</i> The integer-valued hyperparameters and their default ranges.<br/>        """
        pass

    def describe_campaign(self, campaignArn: str) -> DescribeCampaignResponse:
        """Describes the given campaign, including its status. A campaign can be in one of the following states: <ul> <li> CREATE PENDING &gt; CREATE IN_PROGRESS &gt; ACTIVE -or- CREATE FAILED </li> <li> DELETE PENDING &gt; DELETE IN_PROGRESS </li> </ul> When the <code>status</code> is <code>CREATE FAILED</code>, the response includes the <code>failureReason</code> key, which describes why. For more information on campaigns, see <a>CreateCampaign</a>.

        :param self:
        :param campaignArn:  The Amazon Resource Name (ARN) of the campaign.<br/>
        :return: campaign:  The properties of the campaign.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>latestCampaignUpdate</i> Provides a summary of the properties of a campaign update. For a complete listing, call the <a>DescribeCampaign</a> API.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>latestCampaignUpdate</b>: Provides a summary of the properties of a campaign update. For a complete listing, call the <a>DescribeCampaign</a> API.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedDateTime</i> The date and time (in Unix format) that the campaign was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDateTime</i> The date and time (in Unix format) that the campaign was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>failureReason</i> If a campaign fails, the reason behind the failure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the campaign. A campaign can be in one of the following states: <ul> <li> CREATE PENDING &gt; CREATE IN_PROGRESS &gt; ACTIVE -or- CREATE FAILED </li> <li> DELETE PENDING &gt; DELETE IN_PROGRESS </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>minProvisionedTPS</i> Specifies the requested minimum provisioned transactions (recommendations) per second.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>solutionVersionArn</i> The Amazon Resource Name (ARN) of a specific version of the solution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>campaignArn</i> The Amazon Resource Name (ARN) of the campaign. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedDateTime</i> The date and time (in Unix time) that the campaign update was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDateTime</i> The date and time (in Unix time) that the campaign update was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>failureReason</i> If a campaign update fails, the reason behind the failure.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the campaign update. A campaign update can be in one of the following states: <ul> <li> CREATE PENDING &gt; CREATE IN_PROGRESS &gt; ACTIVE -or- CREATE FAILED </li> <li> DELETE PENDING &gt; DELETE IN_PROGRESS </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>minProvisionedTPS</i> Specifies the requested minimum provisioned transactions (recommendations) per second that Amazon Personalize will support.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>solutionVersionArn</i> The Amazon Resource Name (ARN) of the deployed solution version.<br/>        """
        pass

    def describe_dataset(self, datasetArn: str) -> DescribeDatasetResponse:
        """Describes the given dataset. For more information on datasets, see <a>CreateDataset</a>.

        :param self:
        :param datasetArn:  The Amazon Resource Name (ARN) of the dataset to describe.<br/>
        :return: dataset:  A listing of the dataset's properties.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedDateTime</i> A time stamp that shows when the dataset was updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDateTime</i> The creation date and time (in Unix time) of the dataset.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the dataset. A dataset can be in one of the following states: <ul> <li> CREATE PENDING &gt; CREATE IN_PROGRESS &gt; ACTIVE -or- CREATE FAILED </li> <li> DELETE PENDING &gt; DELETE IN_PROGRESS </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>schemaArn</i> The ARN of the associated schema.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>datasetType</i> One of the following values: <ul> <li> Interactions </li> <li> Items </li> <li> Users </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>datasetGroupArn</i> The Amazon Resource Name (ARN) of the dataset group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>datasetArn</i> The Amazon Resource Name (ARN) of the dataset that you want metadata for.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the dataset.<br/>        """
        pass

    def describe_dataset_group(self, datasetGroupArn: str) -> DescribeDatasetGroupResponse:
        """Describes the given dataset group. For more information on dataset groups, see <a>CreateDatasetGroup</a>.

        :param self:
        :param datasetGroupArn:  The Amazon Resource Name (ARN) of the dataset group to describe.<br/>
        :return: datasetGroup:  A listing of the dataset group's properties.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>failureReason</i> If creating a dataset group fails, provides the reason why.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedDateTime</i> The last update date and time (in Unix time) of the dataset group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDateTime</i> The creation date and time (in Unix time) of the dataset group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>kmsKeyArn</i> The Amazon Resource Name (ARN) of the KMS key used to encrypt the datasets.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The ARN of the IAM role that has permissions to create the dataset group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the dataset group. A dataset group can be in one of the following states: <ul> <li> CREATE PENDING &gt; CREATE IN_PROGRESS &gt; ACTIVE -or- CREATE FAILED </li> <li> DELETE PENDING </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>datasetGroupArn</i> The Amazon Resource Name (ARN) of the dataset group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the dataset group.<br/>        """
        pass

    def describe_dataset_import_job(self, datasetImportJobArn: str) -> DescribeDatasetImportJobResponse:
        """Describes the dataset import job created by <a>CreateDatasetImportJob</a>, including the import job status.

        :param self:
        :param datasetImportJobArn:  The Amazon Resource Name (ARN) of the dataset import job to describe.<br/>
        :return: datasetImportJob:  Information about the dataset import job, including the status. The status is one of the following values: <ul> <li> CREATE PENDING </li> <li> CREATE IN_PROGRESS </li> <li> ACTIVE </li> <li> CREATE FAILED </li> </ul>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>failureReason</i> If a dataset import job fails, provides the reason why.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedDateTime</i> The date and time (in Unix time) the dataset was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDateTime</i> The creation date and time (in Unix time) of the dataset import job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the dataset import job. A dataset import job can be in one of the following states: <ul> <li> CREATE PENDING &gt; CREATE IN_PROGRESS &gt; ACTIVE -or- CREATE FAILED </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleArn</i> The ARN of the AWS Identity and Access Management (IAM) role that has permissions to read from the Amazon S3 data source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>dataSource</i> The Amazon S3 bucket that contains the training data to import.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>dataSource</b>: The Amazon S3 bucket that contains the training data to import.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>datasetArn</i> The Amazon Resource Name (ARN) of the dataset that receives the imported data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>datasetImportJobArn</i> The ARN of the dataset import job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>jobName</i> The name of the import job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>dataLocation</i> The path to the Amazon S3 bucket where the data that you want to upload to your dataset is stored. For example:   <code>s3://bucket-name/training-data.csv</code> <br/>        """
        pass

    def describe_event_tracker(self, eventTrackerArn: str) -> DescribeEventTrackerResponse:
        """Describes an event tracker. The response includes the <code>trackingId</code> and <code>status</code> of the event tracker. For more information on event trackers, see <a>CreateEventTracker</a>.

        :param self:
        :param eventTrackerArn:  The Amazon Resource Name (ARN) of the event tracker to describe.<br/>
        :return: eventTracker:  An object that describes the event tracker.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedDateTime</i> The date and time (in Unix time) that the event tracker was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDateTime</i> The date and time (in Unix format) that the event tracker was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the event tracker. An event tracker can be in one of the following states: <ul> <li> CREATE PENDING &gt; CREATE IN_PROGRESS &gt; ACTIVE -or- CREATE FAILED </li> <li> DELETE PENDING &gt; DELETE IN_PROGRESS </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>datasetGroupArn</i> The Amazon Resource Name (ARN) of the dataset group that receives the event data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>trackingId</i> The ID of the event tracker. Include this ID in requests to the <a href="https://docs.aws.amazon.com/personalize/latest/dg/API_UBS_PutEvents.html">PutEvents</a> API.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>accountId</i> The Amazon AWS account that owns the event tracker.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>eventTrackerArn</i> The ARN of the event tracker.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the event tracker.<br/>        """
        pass

    def describe_feature_transformation(self, featureTransformationArn: str) -> DescribeFeatureTransformationResponse:
        """Describes the given feature transformation.

        :param self:
        :param featureTransformationArn:  The Amazon Resource Name (ARN) of the feature transformation to describe.<br/>
        :return: featureTransformation:  A listing of the FeatureTransformation properties.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the feature transformation. A feature transformation can be in one of the following states: <ul> <li> CREATE PENDING &gt; CREATE IN_PROGRESS &gt; ACTIVE -or- CREATE FAILED </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedDateTime</i> The last update date and time (in Unix time) of the feature transformation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDateTime</i> The creation date and time (in Unix time) of the feature transformation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultParameters</i> Provides the default parameters for feature transformation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>featureTransformationArn</i> The Amazon Resource Name (ARN) of the FeatureTransformation object.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the feature transformation.<br/>        """
        pass

    def describe_recipe(self, recipeArn: str) -> DescribeRecipeResponse:
        """Describes a recipe. A recipe contains three items: <ul> <li> An algorithm that trains a model. </li> <li> Hyperparameters that govern the training. </li> <li> Feature transformation information for modifying the input data before training. </li> </ul> Amazon Personalize provides a set of predefined recipes. You specify a recipe when you create a solution with the <a>CreateSolution</a> API. <code>CreateSolution</code> trains a model by using the algorithm in the specified recipe and a training dataset. The solution, when deployed as a campaign, can provide recommendations using the <a href="https://docs.aws.amazon.com/personalize/latest/dg/API_RS_GetRecommendations.html">GetRecommendations</a> API.

        :param self:
        :param recipeArn:  The Amazon Resource Name (ARN) of the recipe to describe.<br/>
        :return: recipe:  An object that describes the recipe.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedDateTime</i> The date and time (in Unix format) that the recipe was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>recipeType</i> One of the following values: <ul> <li> PERSONALIZED_RANKING </li> <li> RELATED_ITEMS </li> <li> USER_PERSONALIZATION </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDateTime</i> The date and time (in Unix format) that the recipe was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The description of the recipe.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the recipe.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>featureTransformationArn</i> The ARN of the FeatureTransformation object.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>algorithmArn</i> The Amazon Resource Name (ARN) of the algorithm that Amazon Personalize uses to train the model.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>recipeArn</i> The Amazon Resource Name (ARN) of the recipe.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the recipe.<br/>        """
        pass

    def describe_schema(self, schemaArn: str) -> DescribeSchemaResponse:
        """Describes a schema. For more information on schemas, see <a>CreateSchema</a>.

        :param self:
        :param schemaArn:  The Amazon Resource Name (ARN) of the schema to retrieve.<br/>
        :return: schema:  The requested schema.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedDateTime</i> The date and time (in Unix time) that the schema was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDateTime</i> The date and time (in Unix time) that the schema was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>schema</i> The schema.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>schemaArn</i> The Amazon Resource Name (ARN) of the schema.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the schema.<br/>        """
        pass

    def describe_solution(self, solutionArn: str) -> DescribeSolutionResponse:
        """Describes a solution. For more information on solutions, see <a>CreateSolution</a>.

        :param self:
        :param solutionArn:  The Amazon Resource Name (ARN) of the solution to describe.<br/>
        :return: solution:  An object that describes the solution.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>latestSolutionVersion</i> Describes the latest version of the solution, including the status and the ARN.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>latestSolutionVersion</b>: Describes the latest version of the solution, including the status and the ARN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedDateTime</i> The date and time (in Unix time) that the solution was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDateTime</i> The creation date and time (in Unix time) of the solution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the solution. A solution can be in one of the following states: <ul> <li> CREATE PENDING &gt; CREATE IN_PROGRESS &gt; ACTIVE -or- CREATE FAILED </li> <li> DELETE PENDING &gt; DELETE IN_PROGRESS </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>autoMLResult</i> When <code>performAutoML</code> is true, specifies the best recipe found.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>autoMLResult</b>: When <code>performAutoML</code> is true, specifies the best recipe found.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>solutionConfig</i> Describes the configuration properties for the solution.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>solutionConfig</b>: Describes the configuration properties for the solution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>eventType</i> The event type (for example, 'click' or 'like') that is used for training the model.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>datasetGroupArn</i> The Amazon Resource Name (ARN) of the dataset group that provides the training data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>recipeArn</i> The ARN of the recipe used to create the solution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>performAutoML</i> When true, Amazon Personalize performs a search for the best USER_PERSONALIZATION recipe from the list specified in the solution configuration (<code>recipeArn</code> must not be specified). When false (the default), Amazon Personalize uses <code>recipeArn</code> for training.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>performHPO</i> Whether to perform hyperparameter optimization (HPO) on the chosen recipe. The default is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>solutionArn</i> The ARN of the solution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the solution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>eventValueThreshold</i> Only events with a value greater than or equal to this threshold are used for training a model.<br/>Describes the properties for hyperparameter optimization (HPO). For use with the bring-your-own-recipe feature. Not used with Amazon Personalize predefined recipes.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>hpoConfig</b>: Describes the properties for hyperparameter optimization (HPO). For use with the bring-your-own-recipe feature. Not used with Amazon Personalize predefined recipes.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>bestRecipeArn</i> The Amazon Resource Name (ARN) of the best recipe.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>failureReason</i> If a solution version fails, the reason behind the failure.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedDateTime</i> The date and time (in Unix time) that the solution version was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDateTime</i> The date and time (in Unix time) that this version of a solution was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the solution version. A solution version can be in one of the following states: <ul> <li> CREATE PENDING &gt; CREATE IN_PROGRESS &gt; ACTIVE -or- CREATE FAILED </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>solutionVersionArn</i> The Amazon Resource Name (ARN) of the solution version.<br/>        """
        pass

    def describe_solution_version(self, solutionVersionArn: str) -> DescribeSolutionVersionResponse:
        """Describes a specific version of a solution. For more information on solutions, see <a>CreateSolution</a>.

        :param self:
        :param solutionVersionArn:  The Amazon Resource Name (ARN) of the solution version.<br/>
        :return: solutionVersion:  The solution version.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedDateTime</i> The date and time (in Unix time) that the solution was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDateTime</i> The date and time (in Unix time) that this version of the solution was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>failureReason</i> If training a solution version fails, the reason behind the failure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the solution version. A solution version can be in one of the following states: <ul> <li> CREATE PENDING &gt; CREATE IN_PROGRESS &gt; ACTIVE -or- CREATE FAILED </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>solutionConfig</i> Describes the configuration properties for the solution.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>solutionConfig</b>: Describes the configuration properties for the solution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>datasetGroupArn</i> The Amazon Resource Name (ARN) of the dataset group providing the training data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>eventType</i> The event type (for example, 'click' or 'like') that is used for training the model.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>recipeArn</i> The ARN of the recipe used in the solution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>performAutoML</i> When true, Amazon Personalize performs a search for the most optimal recipe according to the solution configuration. When false (the default), Amazon Personalize uses <code>recipeArn</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>performHPO</i> Whether to perform hyperparameter optimization (HPO) on the chosen recipe. The default is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>solutionArn</i> The ARN of the solution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>solutionVersionArn</i> The ARN of the solution version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>eventValueThreshold</i> Only events with a value greater than or equal to this threshold are used for training a model.<br/>Describes the properties for hyperparameter optimization (HPO). For use with the bring-your-own-recipe feature. Not used with Amazon Personalize predefined recipes.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>hpoConfig</b>: Describes the properties for hyperparameter optimization (HPO). For use with the bring-your-own-recipe feature. Not used with Amazon Personalize predefined recipes.        """
        pass

    def get_solution_metrics(self, solutionVersionArn: str) -> GetSolutionMetricsResponse:
        """Gets the metrics for the specified solution version.

        :param self:
        :param solutionVersionArn:  The Amazon Resource Name (ARN) of the solution version for which to get metrics.<br/>
        :return: metrics:  The metrics for the solution version.<br/>solutionVersionArn:  The same solution version ARN as specified in the request.<br/>        """
        pass

    def list_campaigns(self, solutionArn: str=None, nextToken: str=None, maxResults: int=None) -> ListCampaignsResponse:
        """Returns a list of campaigns that use the given solution. When a solution is not specified, all the campaigns associated with the account are listed. The response provides the properties for each campaign, including the Amazon Resource Name (ARN). For more information on campaigns, see <a>CreateCampaign</a>.

        :param self:
        :param maxResults:  The maximum number of campaigns to return.<br/>nextToken:  A token returned from the previous call to <code>ListCampaigns</code> for getting the next set of campaigns (if they exist).<br/>solutionArn:  The Amazon Resource Name (ARN) of the solution to list the campaigns for. When a solution is not specified, all the campaigns associated with the account are listed.<br/>
        :return: nextToken:  A token for getting the next set of campaigns (if they exist).<br/>campaigns:  A list of the campaigns.<br/>        """
        pass

    def list_dataset_groups(self, nextToken: str=None, maxResults: int=None) -> ListDatasetGroupsResponse:
        """Returns a list of dataset groups. The response provides the properties for each dataset group, including the Amazon Resource Name (ARN). For more information on dataset groups, see <a>CreateDatasetGroup</a>.

        :param self:
        :param maxResults:  The maximum number of dataset groups to return.<br/>nextToken:  A token returned from the previous call to <code>ListDatasetGroups</code> for getting the next set of dataset groups (if they exist).<br/>
        :return: nextToken:  A token for getting the next set of dataset groups (if they exist).<br/>datasetGroups:  The list of your dataset groups.<br/>        """
        pass

    def list_dataset_import_jobs(self, datasetArn: str=None, nextToken: str=None, maxResults: int=None) -> ListDatasetImportJobsResponse:
        """Returns a list of dataset import jobs that use the given dataset. When a dataset is not specified, all the dataset import jobs associated with the account are listed. The response provides the properties for each dataset import job, including the Amazon Resource Name (ARN). For more information on dataset import jobs, see <a>CreateDatasetImportJob</a>. For more information on datasets, see <a>CreateDataset</a>.

        :param self:
        :param maxResults:  The maximum number of dataset import jobs to return.<br/>nextToken:  A token returned from the previous call to <code>ListDatasetImportJobs</code> for getting the next set of dataset import jobs (if they exist).<br/>datasetArn:  The Amazon Resource Name (ARN) of the dataset to list the dataset import jobs for.<br/>
        :return: nextToken:  A token for getting the next set of dataset import jobs (if they exist).<br/>datasetImportJobs:  The list of dataset import jobs.<br/>        """
        pass

    def list_datasets(self, datasetGroupArn: str=None, nextToken: str=None, maxResults: int=None) -> ListDatasetsResponse:
        """Returns the list of datasets contained in the given dataset group. The response provides the properties for each dataset, including the Amazon Resource Name (ARN). For more information on datasets, see <a>CreateDataset</a>.

        :param self:
        :param maxResults:  The maximum number of datasets to return.<br/>nextToken:  A token returned from the previous call to <code>ListDatasetImportJobs</code> for getting the next set of dataset import jobs (if they exist).<br/>datasetGroupArn:  The Amazon Resource Name (ARN) of the dataset group that contains the datasets to list.<br/>
        :return: nextToken:  A token for getting the next set of datasets (if they exist).<br/>datasets:  An array of <code>Dataset</code> objects. Each object provides metadata information.<br/>        """
        pass

    def list_event_trackers(self, datasetGroupArn: str=None, nextToken: str=None, maxResults: int=None) -> ListEventTrackersResponse:
        """Returns the list of event trackers associated with the account. The response provides the properties for each event tracker, including the Amazon Resource Name (ARN) and tracking ID. For more information on event trackers, see <a>CreateEventTracker</a>.

        :param self:
        :param maxResults:  The maximum number of event trackers to return.<br/>nextToken:  A token returned from the previous call to <code>ListEventTrackers</code> for getting the next set of event trackers (if they exist).<br/>datasetGroupArn:  The ARN of a dataset group used to filter the response.<br/>
        :return: nextToken:  A token for getting the next set of event trackers (if they exist).<br/>eventTrackers:  A list of event trackers.<br/>        """
        pass

    def list_recipes(self, recipeProvider: str=None, nextToken: str=None, maxResults: int=None) -> ListRecipesResponse:
        """Returns a list of available recipes. The response provides the properties for each recipe, including the recipe's Amazon Resource Name (ARN).

        :param self:
        :param maxResults:  The maximum number of recipes to return.<br/>nextToken:  A token returned from the previous call to <code>ListRecipes</code> for getting the next set of recipes (if they exist).<br/>recipeProvider:  The default is <code>SERVICE</code>.<br/>
        :return: nextToken:  A token for getting the next set of recipes.<br/>recipes:  The list of available recipes.<br/>        """
        pass

    def list_schemas(self, nextToken: str=None, maxResults: int=None) -> ListSchemasResponse:
        """Returns the list of schemas associated with the account. The response provides the properties for each schema, including the Amazon Resource Name (ARN). For more information on schemas, see <a>CreateSchema</a>.

        :param self:
        :param maxResults:  The maximum number of schemas to return.<br/>nextToken:  A token returned from the previous call to <code>ListSchemas</code> for getting the next set of schemas (if they exist).<br/>
        :return: nextToken:  A token used to get the next set of schemas (if they exist).<br/>schemas:  A list of schemas.<br/>        """
        pass

    def list_solution_versions(self, solutionArn: str=None, nextToken: str=None, maxResults: int=None) -> ListSolutionVersionsResponse:
        """Returns a list of solution versions for the given solution. When a solution is not specified, all the solution versions associated with the account are listed. The response provides the properties for each solution version, including the Amazon Resource Name (ARN). For more information on solutions, see <a>CreateSolution</a>.

        :param self:
        :param maxResults:  The maximum number of solution versions to return.<br/>nextToken:  A token returned from the previous call to <code>ListSolutionVersions</code> for getting the next set of solution versions (if they exist).<br/>solutionArn:  The Amazon Resource Name (ARN) of the solution.<br/>
        :return: nextToken:  A token for getting the next set of solution versions (if they exist).<br/>solutionVersions:  A list of solution versions describing the version properties.<br/>        """
        pass

    def list_solutions(self, datasetGroupArn: str=None, nextToken: str=None, maxResults: int=None) -> ListSolutionsResponse:
        """Returns a list of solutions that use the given dataset group. When a dataset group is not specified, all the solutions associated with the account are listed. The response provides the properties for each solution, including the Amazon Resource Name (ARN). For more information on solutions, see <a>CreateSolution</a>.

        :param self:
        :param maxResults:  The maximum number of solutions to return.<br/>nextToken:  A token returned from the previous call to <code>ListSolutions</code> for getting the next set of solutions (if they exist).<br/>datasetGroupArn:  The Amazon Resource Name (ARN) of the dataset group.<br/>
        :return: nextToken:  A token for getting the next set of solutions (if they exist).<br/>solutions:  A list of the current solutions.<br/>        """
        pass

    def update_campaign(self, campaignArn: str, solutionVersionArn: str=None, minProvisionedTPS: int=None) -> UpdateCampaignResponse:
        """Updates a campaign by either deploying a new solution or changing the value of the campaign's <code>minProvisionedTPS</code> parameter. To update a campaign, the campaign status must be ACTIVE or CREATE FAILED. Check the campaign status using the <a>DescribeCampaign</a> API. <note> You must wait until the <code>status</code> of the updated campaign is <code>ACTIVE</code> before asking the campaign for recommendations. </note> For more information on campaigns, see <a>CreateCampaign</a>.

        :param self:
        :param minProvisionedTPS:  Specifies the requested minimum provisioned transactions (recommendations) per second that Amazon Personalize will support.<br/>solutionVersionArn:  The ARN of a new solution version to deploy.<br/>campaignArn:  The Amazon Resource Name (ARN) of the campaign.<br/>
        :return: campaignArn:  The same campaign ARN as given in the request.<br/>        """
        pass



class PersonalizeEvents(BaseClient):
    """<p/>"""




    class PutEventsRequest(Mapping):
        pass
    
    class Event(Mapping):
        pass
    
    def put_events(self, eventList: List[Event], sessionId: str, trackingId: str, userId: str=None):
        """Records user interaction event data.

        :param self:
        :param eventList:  A list of event data from the session.<br/>sessionId:  The session ID associated with the user's visit.<br/>userId:  The user associated with the event.<br/>trackingId:  The tracking ID for the event. The ID is generated by a call to the <a href="https://docs.aws.amazon.com/personalize/latest/dg/API_CreateEventTracker.html">CreateEventTracker</a> API.<br/>
        :return:         """
        pass



class PersonalizeRuntime(BaseClient):
    """<p/>"""




    class GetPersonalizedRankingRequest(Mapping):
        pass
    
    class GetPersonalizedRankingResponse(Mapping):
        pass
    
    class ItemID(object):
        pass
    
    class GetRecommendationsRequest(Mapping):
        pass
    
    class GetRecommendationsResponse(Mapping):
        pass
    
    def get_personalized_ranking(self, userId: str, inputList: List[ItemID], campaignArn: str) -> GetPersonalizedRankingResponse:
        """Re-ranks a list of recommended items for the given user. The first item in the list is deemed the most likely item to be of interest to the user. <note> The solution backing the campaign must have been created using a recipe of type PERSONALIZED_RANKING. </note>

        :param self:
        :param userId:  The user for which you want the campaign to provide a personalized ranking.<br/>inputList:  A list of items (itemId's) to rank. If an item was not included in the training dataset, the item is appended to the end of the reranked list.<br/>campaignArn:  The Amazon Resource Name (ARN) of the campaign to use for generating the personalized ranking.<br/>
        :return: personalizedRanking:  A list of items in order of most likely interest to the user.<br/>        """
        pass

    def get_recommendations(self, campaignArn: str, itemId: str=None, userId: str=None, numResults: int=None) -> GetRecommendationsResponse:
        """Returns a list of recommended items. The required input depends on the recipe type used to create the solution backing the campaign, as follows: <ul> <li> RELATED_ITEMS - <code>itemId</code> required, <code>userId</code> not used </li> <li> USER_PERSONALIZATION - <code>itemId</code> optional, <code>userId</code> required </li> </ul> <note> Campaigns that are backed by a solution created using a recipe of type PERSONALIZED_RANKING use the API. </note>

        :param self:
        :param numResults:  The number of results to return. The default is 25. The maximum is 100.<br/>userId:  The user ID to provide recommendations for. Required for <code>USER_PERSONALIZATION</code> recipe type.<br/>itemId:  The item ID to provide recommendations for. Required for <code>RELATED_ITEMS</code> recipe type.<br/>campaignArn:  The Amazon Resource Name (ARN) of the campaign to use for getting recommendations.<br/>
        :return: itemList:  A list of recommendations.<br/>        """
        pass



class PI(BaseClient):
    """AWS Performance Insights enables you to monitor and explore different dimensions of database load based on data captured from a running RDS instance. The guide provides detailed information about Performance Insights data types, parameters and errors. For more information about Performance Insights capabilities see <a href="http://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_PerfInsights.html">Using Amazon RDS Performance Insights </a> in the <i>Amazon RDS User Guide</i>.  The AWS Performance Insights API provides visibility into the performance of your RDS instance, when Performance Insights is enabled for supported engine types. While Amazon CloudWatch provides the authoritative source for AWS service vended monitoring metrics, AWS Performance Insights offers a domain-specific view of database load measured as Average Active Sessions and provided to API consumers as a 2-dimensional time-series dataset. The time dimension of the data provides DB load data for each time point in the queried time range, and each time point decomposes overall load in relation to the requested dimensions, such as SQL, Wait-event, User or Host, measured at that time point."""




    class DescribeDimensionKeysRequest(Mapping):
        pass
    
    class DescribeDimensionKeysResponse(Mapping):
        pass
    
    class ISOTimestamp(object):
        pass
    
    class DimensionGroup(Mapping):
        pass
    
    class MetricQueryFilterMap(object):
        pass
    
    class GetResourceMetricsRequest(Mapping):
        pass
    
    class GetResourceMetricsResponse(Mapping):
        pass
    
    class MetricQuery(Mapping):
        pass
    
    def describe_dimension_keys(self, GroupBy: DimensionGroup, Metric: str, EndTime: ISOTimestamp, StartTime: ISOTimestamp, Identifier: str, ServiceType: str, PeriodInSeconds: int=None, PartitionBy: DimensionGroup=None, Filter: MetricQueryFilterMap=None, MaxResults: int=None, NextToken: str=None) -> DescribeDimensionKeysResponse:
        """For a specific time period, retrieve the top <code>N</code> dimension keys for a metric.

        :param self:
        :param NextToken:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the token, up to the value specified by <code>MaxRecords</code>.<br/>MaxResults:  The maximum number of items to return in the response. If more items exist than the specified <code>MaxRecords</code> value, a pagination token is included in the response so that the remaining results can be retrieved. <br/>Filter:  One or more filters to apply in the request. Restrictions: <ul> <li> Any number of filters by the same dimension, as specified in the <code>GroupBy</code> or <code>Partition</code> parameters. </li> <li> A single filter for any other dimension in this dimension group. </li> </ul><br/>PartitionBy:  For each dimension specified in <code>GroupBy</code>, specify a secondary dimension to further subdivide the partition keys in the response.:<br/>GroupBy:  A specification for how to aggregate the data points from a query result. You must specify a valid dimension group. Performance Insights will return all of the dimensions within that group, unless you provide the names of specific dimensions within that group. You can also request that Performance Insights return a limited number of values for a dimension.:<br/>PeriodInSeconds:  The granularity, in seconds, of the data points returned from Performance Insights. A period can be as short as one second, or as long as one day (86400 seconds). Valid values are: <ul> <li>  <code>1</code> (one second) </li> <li>  <code>60</code> (one minute) </li> <li>  <code>300</code> (five minutes) </li> <li>  <code>3600</code> (one hour) </li> <li>  <code>86400</code> (twenty-four hours) </li> </ul> If you don't specify <code>PeriodInSeconds</code>, then Performance Insights will choose a value for you, with a goal of returning roughly 100-200 data points in the response.<br/>Metric:  The name of a Performance Insights metric to be measured. Valid values for <code>Metric</code> are: <ul> <li>  <code>db.load.avg</code> - a scaled representation of the number of active sessions for the database engine. </li> <li>  <code>db.sampledload.avg</code> - the raw number of active sessions for the database engine. </li> </ul><br/>EndTime:  The date and time specifying the end of the requested time series data. The value specified is <i>exclusive</i> - data points less than (but not equal to) <code>EndTime</code> will be returned. The value for <code>EndTime</code> must be later than the value for <code>StartTime</code>.<br/>StartTime:  The date and time specifying the beginning of the requested time series data. You can't specify a <code>StartTime</code> that's earlier than 7 days ago. The value specified is <i>inclusive</i> - data points equal to or greater than <code>StartTime</code> will be returned. The value for <code>StartTime</code> must be earlier than the value for <code>EndTime</code>.<br/>Identifier:  An immutable, AWS Region-unique identifier for a data source. Performance Insights gathers metrics from this data source. To use an Amazon RDS instance as a data source, you specify its <code>DbiResourceId</code> value - for example: <code>db-FAIHNTYBKTGAUSUZQYPDS2GW4A</code> <br/>ServiceType:  The AWS service for which Performance Insights will return metrics. The only valid value for <i>ServiceType</i> is: <code>RDS</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Limit</i> The maximum number of items to fetch for this dimension group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Dimensions</i> A list of specific dimensions from a dimension group. If this parameter is not present, then it signifies that all of the dimensions in the group were requested, or are present in the response. Valid values for elements in the <code>Dimensions</code> array are: <ul> <li> db.user.id </li> <li> db.user.name </li> <li> db.host.id </li> <li> db.host.name </li> <li> db.sql.id </li> <li> db.sql.db_id </li> <li> db.sql.statement </li> <li> db.sql.tokenized_id </li> <li> db.sql_tokenized.id </li> <li> db.sql_tokenized.db_id </li> <li> db.sql_tokenized.statement </li> <li> db.wait_event.name </li> <li> db.wait_event.type </li> <li> db.wait_event_type.name </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Group</i> The name of the dimension group. Valid values are: <ul> <li>  <code>db.user</code>  </li> <li>  <code>db.host</code>  </li> <li>  <code>db.sql</code>  </li> <li>  <code>db.sql_tokenized</code>  </li> <li>  <code>db.wait_event</code>  </li> <li>  <code>db.wait_event_type</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Limit</i> The maximum number of items to fetch for this dimension group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Dimensions</i> A list of specific dimensions from a dimension group. If this parameter is not present, then it signifies that all of the dimensions in the group were requested, or are present in the response. Valid values for elements in the <code>Dimensions</code> array are: <ul> <li> db.user.id </li> <li> db.user.name </li> <li> db.host.id </li> <li> db.host.name </li> <li> db.sql.id </li> <li> db.sql.db_id </li> <li> db.sql.statement </li> <li> db.sql.tokenized_id </li> <li> db.sql_tokenized.id </li> <li> db.sql_tokenized.db_id </li> <li> db.sql_tokenized.statement </li> <li> db.wait_event.name </li> <li> db.wait_event.type </li> <li> db.wait_event_type.name </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Group</i> The name of the dimension group. Valid values are: <ul> <li>  <code>db.user</code>  </li> <li>  <code>db.host</code>  </li> <li>  <code>db.sql</code>  </li> <li>  <code>db.sql_tokenized</code>  </li> <li>  <code>db.wait_event</code>  </li> <li>  <code>db.wait_event_type</code>  </li> </ul><br/>
        :return: NextToken:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the token, up to the value specified by <code>MaxRecords</code>.<br/>Keys:  The dimension keys that were requested.<br/>PartitionKeys:  If <code>PartitionBy</code> was present in the request, <code>PartitionKeys</code> contains the breakdown of dimension keys by the specified partitions.<br/>AlignedEndTime:  The end time for the returned dimension keys, after alignment to a granular boundary (as specified by <code>PeriodInSeconds</code>). <code>AlignedEndTime</code> will be greater than or equal to the value of the user-specified <code>Endtime</code>.<br/>AlignedStartTime:  The start time for the returned dimension keys, after alignment to a granular boundary (as specified by <code>PeriodInSeconds</code>). <code>AlignedStartTime</code> will be less than or equal to the value of the user-specified <code>StartTime</code>.<br/>        """
        pass

    def get_resource_metrics(self, EndTime: ISOTimestamp, StartTime: ISOTimestamp, MetricQueries: List[MetricQuery], Identifier: str, ServiceType: str, PeriodInSeconds: int=None, MaxResults: int=None, NextToken: str=None) -> GetResourceMetricsResponse:
        """Retrieve Performance Insights metrics for a set of data sources, over a time period. You can provide specific dimension groups and dimensions, and provide aggregation and filtering criteria for each group.

        :param self:
        :param NextToken:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the token, up to the value specified by <code>MaxRecords</code>.<br/>MaxResults:  The maximum number of items to return in the response. If more items exist than the specified <code>MaxRecords</code> value, a pagination token is included in the response so that the remaining results can be retrieved. <br/>PeriodInSeconds:  The granularity, in seconds, of the data points returned from Performance Insights. A period can be as short as one second, or as long as one day (86400 seconds). Valid values are: <ul> <li>  <code>1</code> (one second) </li> <li>  <code>60</code> (one minute) </li> <li>  <code>300</code> (five minutes) </li> <li>  <code>3600</code> (one hour) </li> <li>  <code>86400</code> (twenty-four hours) </li> </ul> If you don't specify <code>PeriodInSeconds</code>, then Performance Insights will choose a value for you, with a goal of returning roughly 100-200 data points in the response.<br/>EndTime:  The date and time specifiying the end of the requested time series data. The value specified is <i>exclusive</i> - data points less than (but not equal to) <code>EndTime</code> will be returned. The value for <code>EndTime</code> must be later than the value for <code>StartTime</code>.<br/>StartTime:  The date and time specifying the beginning of the requested time series data. You can't specify a <code>StartTime</code> that's earlier than 7 days ago. The value specified is <i>inclusive</i> - data points equal to or greater than <code>StartTime</code> will be returned. The value for <code>StartTime</code> must be earlier than the value for <code>EndTime</code>.<br/>MetricQueries:  An array of one or more queries to perform. Each query must specify a Performance Insights metric, and can optionally specify aggregation and filtering criteria.<br/>Identifier:  An immutable, AWS Region-unique identifier for a data source. Performance Insights gathers metrics from this data source. To use an Amazon RDS instance as a data source, you specify its <code>DbiResourceId</code> value - for example: <code>db-FAIHNTYBKTGAUSUZQYPDS2GW4A</code> <br/>ServiceType:  The AWS service for which Performance Insights will return metrics. The only valid value for <i>ServiceType</i> is: <code>RDS</code> <br/>
        :return: NextToken:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the token, up to the value specified by <code>MaxRecords</code>.<br/>MetricList:  An array of metric results,, where each array element contains all of the data points for a particular dimension.<br/>Identifier:  An immutable, AWS Region-unique identifier for a data source. Performance Insights gathers metrics from this data source. To use an Amazon RDS instance as a data source, you specify its <code>DbiResourceId</code> value - for example: <code>db-FAIHNTYBKTGAUSUZQYPDS2GW4A</code> <br/>AlignedEndTime:  The end time for the returned metrics, after alignment to a granular boundary (as specified by <code>PeriodInSeconds</code>). <code>AlignedEndTime</code> will be greater than or equal to the value of the user-specified <code>Endtime</code>.<br/>AlignedStartTime:  The start time for the returned metrics, after alignment to a granular boundary (as specified by <code>PeriodInSeconds</code>). <code>AlignedStartTime</code> will be less than or equal to the value of the user-specified <code>StartTime</code>.<br/>        """
        pass



class Pinpoint(BaseClient):
    """Doc Engage API - Amazon Pinpoint API"""



    class CreateAppRequest(Mapping):
        pass
    
    class CreateAppResponse(Mapping):
        pass
    
    class CreateApplicationRequest(Mapping):
        pass
    
    class CreateCampaignRequest(Mapping):
        pass
    
    class CreateCampaignResponse(Mapping):
        pass
    
    class WriteCampaignRequest(Mapping):
        pass
    
    class CreateExportJobRequest(Mapping):
        pass
    
    class CreateExportJobResponse(Mapping):
        pass
    
    class ExportJobRequest(Mapping):
        pass
    
    class CreateImportJobRequest(Mapping):
        pass
    
    class CreateImportJobResponse(Mapping):
        pass
    
    class ImportJobRequest(Mapping):
        pass
    
    class CreateSegmentRequest(Mapping):
        pass
    
    class CreateSegmentResponse(Mapping):
        pass
    
    class WriteSegmentRequest(Mapping):
        pass
    
    class DeleteAdmChannelRequest(Mapping):
        pass
    
    class DeleteAdmChannelResponse(Mapping):
        pass
    
    class DeleteApnsChannelRequest(Mapping):
        pass
    
    class DeleteApnsChannelResponse(Mapping):
        pass
    
    class DeleteApnsSandboxChannelRequest(Mapping):
        pass
    
    class DeleteApnsSandboxChannelResponse(Mapping):
        pass
    
    class DeleteApnsVoipChannelRequest(Mapping):
        pass
    
    class DeleteApnsVoipChannelResponse(Mapping):
        pass
    
    class DeleteApnsVoipSandboxChannelRequest(Mapping):
        pass
    
    class DeleteApnsVoipSandboxChannelResponse(Mapping):
        pass
    
    class DeleteAppRequest(Mapping):
        pass
    
    class DeleteAppResponse(Mapping):
        pass
    
    class DeleteBaiduChannelRequest(Mapping):
        pass
    
    class DeleteBaiduChannelResponse(Mapping):
        pass
    
    class DeleteCampaignRequest(Mapping):
        pass
    
    class DeleteCampaignResponse(Mapping):
        pass
    
    class DeleteEmailChannelRequest(Mapping):
        pass
    
    class DeleteEmailChannelResponse(Mapping):
        pass
    
    class DeleteEndpointRequest(Mapping):
        pass
    
    class DeleteEndpointResponse(Mapping):
        pass
    
    class DeleteEventStreamRequest(Mapping):
        pass
    
    class DeleteEventStreamResponse(Mapping):
        pass
    
    class DeleteGcmChannelRequest(Mapping):
        pass
    
    class DeleteGcmChannelResponse(Mapping):
        pass
    
    class DeleteSegmentRequest(Mapping):
        pass
    
    class DeleteSegmentResponse(Mapping):
        pass
    
    class DeleteSmsChannelRequest(Mapping):
        pass
    
    class DeleteSmsChannelResponse(Mapping):
        pass
    
    class DeleteUserEndpointsRequest(Mapping):
        pass
    
    class DeleteUserEndpointsResponse(Mapping):
        pass
    
    class DeleteVoiceChannelRequest(Mapping):
        pass
    
    class DeleteVoiceChannelResponse(Mapping):
        pass
    
    class GetAdmChannelRequest(Mapping):
        pass
    
    class GetAdmChannelResponse(Mapping):
        pass
    
    class GetApnsChannelRequest(Mapping):
        pass
    
    class GetApnsChannelResponse(Mapping):
        pass
    
    class GetApnsSandboxChannelRequest(Mapping):
        pass
    
    class GetApnsSandboxChannelResponse(Mapping):
        pass
    
    class GetApnsVoipChannelRequest(Mapping):
        pass
    
    class GetApnsVoipChannelResponse(Mapping):
        pass
    
    class GetApnsVoipSandboxChannelRequest(Mapping):
        pass
    
    class GetApnsVoipSandboxChannelResponse(Mapping):
        pass
    
    class GetAppRequest(Mapping):
        pass
    
    class GetAppResponse(Mapping):
        pass
    
    class GetApplicationSettingsRequest(Mapping):
        pass
    
    class GetApplicationSettingsResponse(Mapping):
        pass
    
    class GetAppsRequest(Mapping):
        pass
    
    class GetAppsResponse(Mapping):
        pass
    
    class GetBaiduChannelRequest(Mapping):
        pass
    
    class GetBaiduChannelResponse(Mapping):
        pass
    
    class GetCampaignRequest(Mapping):
        pass
    
    class GetCampaignResponse(Mapping):
        pass
    
    class GetCampaignActivitiesRequest(Mapping):
        pass
    
    class GetCampaignActivitiesResponse(Mapping):
        pass
    
    class GetCampaignVersionRequest(Mapping):
        pass
    
    class GetCampaignVersionResponse(Mapping):
        pass
    
    class GetCampaignVersionsRequest(Mapping):
        pass
    
    class GetCampaignVersionsResponse(Mapping):
        pass
    
    class GetCampaignsRequest(Mapping):
        pass
    
    class GetCampaignsResponse(Mapping):
        pass
    
    class GetChannelsRequest(Mapping):
        pass
    
    class GetChannelsResponse(Mapping):
        pass
    
    class GetEmailChannelRequest(Mapping):
        pass
    
    class GetEmailChannelResponse(Mapping):
        pass
    
    class GetEndpointRequest(Mapping):
        pass
    
    class GetEndpointResponse(Mapping):
        pass
    
    class GetEventStreamRequest(Mapping):
        pass
    
    class GetEventStreamResponse(Mapping):
        pass
    
    class GetExportJobRequest(Mapping):
        pass
    
    class GetExportJobResponse(Mapping):
        pass
    
    class GetExportJobsRequest(Mapping):
        pass
    
    class GetExportJobsResponse(Mapping):
        pass
    
    class GetGcmChannelRequest(Mapping):
        pass
    
    class GetGcmChannelResponse(Mapping):
        pass
    
    class GetImportJobRequest(Mapping):
        pass
    
    class GetImportJobResponse(Mapping):
        pass
    
    class GetImportJobsRequest(Mapping):
        pass
    
    class GetImportJobsResponse(Mapping):
        pass
    
    class GetSegmentRequest(Mapping):
        pass
    
    class GetSegmentResponse(Mapping):
        pass
    
    class GetSegmentExportJobsRequest(Mapping):
        pass
    
    class GetSegmentExportJobsResponse(Mapping):
        pass
    
    class GetSegmentImportJobsRequest(Mapping):
        pass
    
    class GetSegmentImportJobsResponse(Mapping):
        pass
    
    class GetSegmentVersionRequest(Mapping):
        pass
    
    class GetSegmentVersionResponse(Mapping):
        pass
    
    class GetSegmentVersionsRequest(Mapping):
        pass
    
    class GetSegmentVersionsResponse(Mapping):
        pass
    
    class GetSegmentsRequest(Mapping):
        pass
    
    class GetSegmentsResponse(Mapping):
        pass
    
    class GetSmsChannelRequest(Mapping):
        pass
    
    class GetSmsChannelResponse(Mapping):
        pass
    
    class GetUserEndpointsRequest(Mapping):
        pass
    
    class GetUserEndpointsResponse(Mapping):
        pass
    
    class GetVoiceChannelRequest(Mapping):
        pass
    
    class GetVoiceChannelResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class PhoneNumberValidateRequest(Mapping):
        pass
    
    class PhoneNumberValidateResponse(Mapping):
        pass
    
    class NumberValidateRequest(Mapping):
        pass
    
    class PutEventStreamRequest(Mapping):
        pass
    
    class PutEventStreamResponse(Mapping):
        pass
    
    class WriteEventStream(Mapping):
        pass
    
    class PutEventsRequest(Mapping):
        pass
    
    class PutEventsResponse(Mapping):
        pass
    
    class EventsRequest(Mapping):
        pass
    
    class RemoveAttributesRequest(Mapping):
        pass
    
    class RemoveAttributesResponse(Mapping):
        pass
    
    class UpdateAttributesRequest(Mapping):
        pass
    
    class SendMessagesRequest(Mapping):
        pass
    
    class SendMessagesResponse(Mapping):
        pass
    
    class MessageRequest(Mapping):
        pass
    
    class SendUsersMessagesRequest(Mapping):
        pass
    
    class SendUsersMessagesResponse(Mapping):
        pass
    
    class SendUsersMessageRequest(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagsModel(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class __string(object):
        pass
    
    class UpdateAdmChannelRequest(Mapping):
        pass
    
    class UpdateAdmChannelResponse(Mapping):
        pass
    
    class ADMChannelRequest(Mapping):
        pass
    
    class UpdateApnsChannelRequest(Mapping):
        pass
    
    class UpdateApnsChannelResponse(Mapping):
        pass
    
    class APNSChannelRequest(Mapping):
        pass
    
    class UpdateApnsSandboxChannelRequest(Mapping):
        pass
    
    class UpdateApnsSandboxChannelResponse(Mapping):
        pass
    
    class APNSSandboxChannelRequest(Mapping):
        pass
    
    class UpdateApnsVoipChannelRequest(Mapping):
        pass
    
    class UpdateApnsVoipChannelResponse(Mapping):
        pass
    
    class APNSVoipChannelRequest(Mapping):
        pass
    
    class UpdateApnsVoipSandboxChannelRequest(Mapping):
        pass
    
    class UpdateApnsVoipSandboxChannelResponse(Mapping):
        pass
    
    class APNSVoipSandboxChannelRequest(Mapping):
        pass
    
    class UpdateApplicationSettingsRequest(Mapping):
        pass
    
    class UpdateApplicationSettingsResponse(Mapping):
        pass
    
    class WriteApplicationSettingsRequest(Mapping):
        pass
    
    class UpdateBaiduChannelRequest(Mapping):
        pass
    
    class UpdateBaiduChannelResponse(Mapping):
        pass
    
    class BaiduChannelRequest(Mapping):
        pass
    
    class UpdateCampaignRequest(Mapping):
        pass
    
    class UpdateCampaignResponse(Mapping):
        pass
    
    class UpdateEmailChannelRequest(Mapping):
        pass
    
    class UpdateEmailChannelResponse(Mapping):
        pass
    
    class EmailChannelRequest(Mapping):
        pass
    
    class UpdateEndpointRequest(Mapping):
        pass
    
    class UpdateEndpointResponse(Mapping):
        pass
    
    class EndpointRequest(Mapping):
        pass
    
    class UpdateEndpointsBatchRequest(Mapping):
        pass
    
    class UpdateEndpointsBatchResponse(Mapping):
        pass
    
    class EndpointBatchRequest(Mapping):
        pass
    
    class UpdateGcmChannelRequest(Mapping):
        pass
    
    class UpdateGcmChannelResponse(Mapping):
        pass
    
    class GCMChannelRequest(Mapping):
        pass
    
    class UpdateSegmentRequest(Mapping):
        pass
    
    class UpdateSegmentResponse(Mapping):
        pass
    
    class UpdateSmsChannelRequest(Mapping):
        pass
    
    class UpdateSmsChannelResponse(Mapping):
        pass
    
    class SMSChannelRequest(Mapping):
        pass
    
    class UpdateVoiceChannelRequest(Mapping):
        pass
    
    class UpdateVoiceChannelResponse(Mapping):
        pass
    
    class VoiceChannelRequest(Mapping):
        pass
    
    def create_app(self, CreateApplicationRequest: CreateApplicationRequest) -> CreateAppResponse:
        """ Creates an application.

        :param self:
        :param CreateApplicationRequest:  Specifies the display name of an application and the tags to associate with the application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> A string-to-string map of key-value pairs that defines the tags to associate with the application. Each tag consists of a required tag key and an associated tag value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The display name of the application. This name is displayed as the <b>Project name</b> on the Amazon Pinpoint console.<br/>
        :return: ApplicationResponse:  Provides information about an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> A string-to-string map of key-value pairs that identifies the tags that are associated with the application. Each tag consists of a required tag key and an associated tag value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The display name of the application. This name is displayed as the <b>Project name</b> on the Amazon Pinpoint console.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the application.<br/>        """
        pass

    def create_campaign(self, WriteCampaignRequest: WriteCampaignRequest, ApplicationId: str) -> CreateCampaignResponse:
        """Creates a new campaign for an application or updates the settings of an existing campaign for an application.

        :param self:
        :param WriteCampaignRequest:  Specifies the configuration and other settings for a campaign.:<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TreatmentName</i> The custom name of a variation of the campaign to use for A/B testing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TreatmentDescription</i> The custom description of a variation of the campaign to use for A/B testing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> A string-to-string map of key-value pairs that defines the tags to associate with the campaign. Each tag consists of a required tag key and an associated tag value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentVersion</i> The version of the segment to associate with the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentId</i> The unique identifier for the segment to associate with the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Schedule</i> The schedule settings for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Schedule</b>: The schedule settings for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The custom name of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MessageConfiguration</i> The message configuration settings for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>MessageConfiguration</b>: The message configuration settings for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Limits</i> The messaging limits for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Limits</b>: The messaging limits for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsPaused</i> Specifies whether to pause the campaign. A paused campaign doesn't run unless you resume it by setting this value to false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Hook</i> The settings for the AWS Lambda function to use as a code hook for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Hook</b>: The settings for the AWS Lambda function to use as a code hook for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HoldoutPercent</i> The allocated percentage of users (segment members) who shouldn't receive messages from the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The custom description of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdditionalTreatments</i> An array of requests that defines additional treatments for the campaign, in addition to the default treatment for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WebUrl</i>  The web URL that Amazon Pinpoint calls to invoke the AWS Lambda function over HTTPS.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> Specifies which Lambda mode to use when invoking the AWS Lambda function.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LambdaFunctionName</i> The name or Amazon Resource Name (ARN) of the AWS Lambda function that Amazon Pinpoint invokes to send messages for a campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Total</i> The maximum number of messages that a campaign can send to a single endpoint during the course of the campaign. The maximum value is 100.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MessagesPerSecond</i> The maximum number of messages that a campaign can send each second. The minimum value is 50. The maximum value is 20,000.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumDuration</i> The maximum amount of time, in seconds, that a campaign can attempt to deliver a message after the scheduled start time for the campaign. The minimum value is 60 seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Daily</i> The maximum number of messages that a campaign can send to a single endpoint during a 24-hour period. The maximum value is 100.<br/>The message that the campaign sends through the ADM (Amazon Device Messaging) channel. This message overrides the default message.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ADMMessage</b>: The message that the campaign sends through the ADM (Amazon Device Messaging) channel. This message overrides the default message.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> The scheduled time, in ISO 8601 format, for the campaign to end.<br/>The type of event that causes the campaign to be sent, if the value of the Frequency property is EVENT.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EventFilter</b>: The type of event that causes the campaign to be sent, if the value of the Frequency property is EVENT.
        :return: CampaignResponse:  Provides information about the status, configuration, and other settings for a campaign.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version number of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TreatmentName</i> The custom name of a variation of the campaign that's used for A/B testing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TreatmentDescription</i> The custom description of a variation of the campaign that's used for A/B testing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> A string-to-string map of key-value pairs that identifies the tags that are associated with the campaign. Each tag consists of a required tag key and an associated tag value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The current status of the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>State</b>: The current status of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentVersion</i> The version number of the segment that's associated with the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentId</i> The unique identifier for the segment that's associated with the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Schedule</i> The schedule settings for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Schedule</b>: The schedule settings for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MessageConfiguration</i> The message configuration settings for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>MessageConfiguration</b>: The message configuration settings for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Limits</i> The messaging limits for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Limits</b>: The messaging limits for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date, in ISO 8601 format, when the campaign was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsPaused</i> Specifies whether the campaign is paused. A paused campaign doesn't run unless you resume it by changing this value to false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Hook</i> The settings for the AWS Lambda function to use as a code hook for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Hook</b>: The settings for the AWS Lambda function to use as a code hook for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HoldoutPercent</i> The allocated percentage of users (segment members) who shouldn't receive messages from the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The custom description of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultState</i> The current status of the campaign's default treatment. This value exists only for campaigns that have more than one treatment, to support A/B testing.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DefaultState</b>: The current status of the campaign's default treatment. This value exists only for campaigns that have more than one treatment, to support A/B testing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date, ISO 8601 format, when the campaign was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the campaign applies to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdditionalTreatments</i> An array of responses, one for each treatment that you defined for the campaign, in addition to the default treatment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CampaignStatus</i> The status of the campaign, or the status of a treatment that belongs to an A/B test campaign. If a campaign uses A/B testing, the campaign has a status of COMPLETED only when all campaign treatments have a status of COMPLETED.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WebUrl</i>  The web URL that Amazon Pinpoint calls to invoke the AWS Lambda function over HTTPS.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> Specifies which Lambda mode to use when invoking the AWS Lambda function.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LambdaFunctionName</i> The name or Amazon Resource Name (ARN) of the AWS Lambda function that Amazon Pinpoint invokes to send messages for a campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Total</i> The maximum number of messages that a campaign can send to a single endpoint during the course of the campaign. The maximum value is 100.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MessagesPerSecond</i> The maximum number of messages that a campaign can send each second. The minimum value is 50. The maximum value is 20,000.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumDuration</i> The maximum amount of time, in seconds, that a campaign can attempt to deliver a message after the scheduled start time for the campaign. The minimum value is 60 seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Daily</i> The maximum number of messages that a campaign can send to a single endpoint during a 24-hour period. The maximum value is 100.<br/>The message that the campaign sends through the ADM (Amazon Device Messaging) channel. This message overrides the default message.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ADMMessage</b>: The message that the campaign sends through the ADM (Amazon Device Messaging) channel. This message overrides the default message.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> The scheduled time, in ISO 8601 format, for the campaign to end.<br/>The type of event that causes the campaign to be sent, if the value of the Frequency property is EVENT.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EventFilter</b>: The type of event that causes the campaign to be sent, if the value of the Frequency property is EVENT.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CampaignStatus</i> The status of the campaign, or the status of a treatment that belongs to an A/B test campaign. If a campaign uses A/B testing, the campaign has a status of COMPLETED only when all campaign treatments have a status of COMPLETED.<br/>        """
        pass

    def create_export_job(self, ExportJobRequest: ExportJobRequest, ApplicationId: str) -> CreateExportJobResponse:
        """Creates a new export job for an application.

        :param self:
        :param ExportJobRequest:  Specifies the settings for a job that exports endpoint definitions to an Amazon Simple Storage Service (Amazon S3) bucket.:<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentVersion</i> The version of the segment to export endpoint definitions from, if specified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentId</i> The identifier for the segment to export endpoint definitions from. If you don't specify this value, Amazon Pinpoint exports definitions for all the endpoints that are associated with the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3UrlPrefix</i> The URL of the location in an Amazon Simple Storage Service (Amazon S3) bucket where you want to export endpoint definitions to. This location is typically a folder that contains multiple files. The URL should be in the following format: s3://<replaceable>bucket-name</replaceable>/<replaceable>folder-name</replaceable>/.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The Amazon Resource Name (ARN) of the AWS Identity and Access Management (IAM) role that authorizes Amazon Pinpoint to access the Amazon S3 location where you want to export endpoint definitions to.<br/>
        :return: ExportJobResponse:  Provides information about the status and settings of a job that exports endpoint definitions to a file. The file can be added directly to an Amazon Simple Storage Service (Amazon S3) bucket by using the Amazon Pinpoint API or downloaded directly to a computer by using the Amazon Pinpoint console.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The job type. This value is EXPORT for export jobs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalProcessed</i> The total number of endpoint definitions that were processed by the export job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalPieces</i> The total number of pieces that must be processed to complete the export job. Each piece consists of an approximately equal portion of the endpoint definitions that are part of the export job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalFailures</i> The total number of endpoint definitions that weren't processed successfully (failed) by the export job, typically because an error, such as a syntax error, occurred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobStatus</i> The status of the export job. The job status is FAILED if Amazon Pinpoint wasn't able to process one or more pieces in the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier for the export job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Failures</i> An array of entries, one for each of the first 100 entries that weren't processed successfully (failed) by the export job, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailedPieces</i> The number of pieces that weren't processed successfully (failed) by the export job, as of the time of the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Definition</i> The resource settings that apply to the export job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Definition</b>: The resource settings that apply to the export job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date, in ISO 8601 format, when the export job was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompletionDate</i> The date, in ISO 8601 format, when the export job was completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompletedPieces</i> The number of pieces that were processed successfully (completed) by the export job, as of the time of the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that's associated with the export job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentVersion</i> The version of the segment that the endpoint definitions were exported from.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentId</i> The identifier for the segment that the endpoint definitions were exported from. If this value isn't present, Amazon Pinpoint exported definitions for all the endpoints that are associated with the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3UrlPrefix</i> The URL of the location in an Amazon Simple Storage Service (Amazon S3) bucket where the endpoint definitions were exported to. This location is typically a folder that contains multiple files. The URL should be in the following format: s3://<replaceable>bucket-name</replaceable>/<replaceable>folder-name</replaceable>/.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The Amazon Resource Name (ARN) of the AWS Identity and Access Management (IAM) role that authorized Amazon Pinpoint to access the Amazon S3 location where the endpoint definitions were exported to.<br/>        """
        pass

    def create_import_job(self, ImportJobRequest: ImportJobRequest, ApplicationId: str) -> CreateImportJobResponse:
        """Creates a new import job for an application.

        :param self:
        :param ImportJobRequest:  Specifies the settings for a job that imports endpoint definitions from an Amazon Simple Storage Service (Amazon S3) bucket.:<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentName</i> The custom name for the segment that's created by the import job, if the value of the DefineSegment property is true.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentId</i> The identifier for the segment to update or add the imported endpoint definitions to, if the import job is meant to update an existing segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Url</i> The URL of the Amazon Simple Storage Service (Amazon S3) bucket that contains the endpoint definitions to import. This location can be a folder or a single file. If the location is a folder, Amazon Pinpoint imports endpoint definitions from the files in this location, including any subfolders that the folder contains. The URL should be in the following format: s3://<replaceable>bucket-name</replaceable>/<replaceable>folder-name</replaceable>/<replaceable>file-name</replaceable>. The location can end with the key for an individual object or a prefix that qualifies multiple objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The Amazon Resource Name (ARN) of the AWS Identity and Access Management (IAM) role that authorizes Amazon Pinpoint to access the Amazon S3 location to import endpoint definitions from.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RegisterEndpoints</i> Specifies whether to register the endpoints with Amazon Pinpoint, when the endpoint definitions are imported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> The format of the files that contain the endpoint definitions to import. Valid values are: CSV, for comma-separated values format; and, JSON, for newline-delimited JSON format. If the Amazon S3 location stores multiple files that use different formats, Amazon Pinpoint imports data only from the files that use the specified format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalId</i> (Deprecated) Your AWS account ID, which you assigned to an external ID key in an IAM trust policy. Amazon Pinpoint previously used this value to assume an IAM role when importing endpoint definitions, but we removed this requirement. We don't recommend use of external IDs for IAM roles that are assumed by Amazon Pinpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefineSegment</i> Specifies whether to create a segment that contains the endpoints, when the endpoint definitions are imported.<br/>
        :return: ImportJobResponse:  Provides information about the status and settings of a job that imports endpoint definitions from one or more files. The files can be stored in an Amazon Simple Storage Service (Amazon S3) bucket or uploaded directly from a computer by using the Amazon Pinpoint console.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The job type. This value is IMPORT for import jobs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalProcessed</i> The total number of endpoint definitions that were processed by the import job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalPieces</i> The total number of pieces that must be processed to complete the import job. Each piece consists of an approximately equal portion of the endpoint definitions that are part of the import job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalFailures</i> The total number of endpoint definitions that weren't processed successfully (failed) by the import job, typically because an error, such as a syntax error, occurred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobStatus</i> The status of the import job. The job status is FAILED if Amazon Pinpoint wasn't able to process one or more pieces in the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier for the import job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Failures</i> An array of entries, one for each of the first 100 entries that weren't processed successfully (failed) by the import job, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailedPieces</i> The number of pieces that weren't processed successfully (failed) by the import job, as of the time of the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Definition</i> The resource settings that apply to the import job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Definition</b>: The resource settings that apply to the import job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date, in ISO 8601 format, when the import job was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompletionDate</i> The date, in ISO 8601 format, when the import job was completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompletedPieces</i> The number of pieces that were processed successfully (completed) by the import job, as of the time of the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that's associated with the import job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentName</i> The custom name for the segment that's created by the import job, if the value of the DefineSegment property is true.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentId</i> The identifier for the segment that the import job updates or adds endpoint definitions to, if the import job updates an existing segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Url</i> The URL of the Amazon Simple Storage Service (Amazon S3) bucket that contains the endpoint definitions to import. This location can be a folder or a single file. If the location is a folder, Amazon Pinpoint imports endpoint definitions from the files in this location, including any subfolders that the folder contains. The URL should be in the following format: s3://<replaceable>bucket-name</replaceable>/<replaceable>folder-name</replaceable>/<replaceable>file-name</replaceable>. The location can end with the key for an individual object or a prefix that qualifies multiple objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The Amazon Resource Name (ARN) of the AWS Identity and Access Management (IAM) role that authorizes Amazon Pinpoint to access the Amazon S3 location to import endpoint definitions from.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RegisterEndpoints</i> Specifies whether the import job registers the endpoints with Amazon Pinpoint, when the endpoint definitions are imported.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> The format of the files that contain the endpoint definitions to import. Valid values are: CSV, for comma-separated values format; and, JSON, for newline-delimited JSON format. If the files are stored in an Amazon S3 location and that location contains multiple files that use different formats, Amazon Pinpoint imports data only from the files that use the specified format.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalId</i> (Deprecated) Your AWS account ID, which you assigned to an external ID key in an IAM trust policy. Amazon Pinpoint previously used this value to assume an IAM role when importing endpoint definitions, but we removed this requirement. We don't recommend use of external IDs for IAM roles that are assumed by Amazon Pinpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DefineSegment</i> Specifies whether the import job creates a segment that contains the endpoints, when the endpoint definitions are imported.<br/>        """
        pass

    def create_segment(self, WriteSegmentRequest: WriteSegmentRequest, ApplicationId: str) -> CreateSegmentResponse:
        """Creates a new segment for an application or updates the configuration, dimension, and other settings for an existing segment that's associated with an application.

        :param self:
        :param WriteSegmentRequest:  Specifies the configuration, dimension, and other settings for a segment. A WriteSegmentRequest object can include a Dimensions object or a SegmentGroups object, but not both.:<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> A string-to-string map of key-value pairs that defines the tags to associate with the segment. Each tag consists of a required tag key and an associated tag value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentGroups</i> The segment group to use and the dimensions to apply to the group's base segments in order to build the segment. A segment group can consist of zero or more base segments. Your request can include only one segment group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SegmentGroups</b>: The segment group to use and the dimensions to apply to the group's base segments in order to build the segment. A segment group can consist of zero or more base segments. Your request can include only one segment group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Dimensions</i> The criteria that define the dimensions for the segment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Dimensions</b>: The criteria that define the dimensions for the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Attributes</i> One or more custom attributes to use as criteria for the segment.<br/>The behavior-based criteria, such as how recently users have used your app, for the segment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Behavior</b>: The behavior-based criteria, such as how recently users have used your app, for the segment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Include</i> Specifies how to handle multiple segment groups for the segment. For example, if the segment includes three segment groups, whether the resulting segment includes endpoints that match all, any, or none of the segment groups.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Groups</i> An array that defines the set of segment criteria to evaluate when handling segment groups for the segment.<br/>
        :return: SegmentResponse:  Provides information about the configuration, dimension, and other settings for a segment.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version number of the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> A string-to-string map of key-value pairs that identifies the tags that are associated with the segment. Each tag consists of a required tag key and an associated tag value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentType</i> The segment type. Valid values are: <ul><li>DIMENSIONAL - A dynamic segment, which is a segment that uses selection criteria that you specify and is based on endpoint data that's reported by your app. Dynamic segments can change over time.</li> <li>IMPORT - A static segment, which is a segment that uses selection criteria that you specify and is based on endpoint definitions that you import from a file. Imported segments are static; they don't change over time.</li></ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentGroups</i> A list of one or more segment groups that apply to the segment. Each segment group consists of zero or more base segments and the dimensions that are applied to those base segments.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SegmentGroups</b>: A list of one or more segment groups that apply to the segment. Each segment group consists of zero or more base segments and the dimensions that are applied to those base segments.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time when the segment was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImportDefinition</i> The settings for the import job that's associated with the segment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ImportDefinition</b>: The settings for the import job that's associated with the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier for the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Dimensions</i> The dimension settings for the segment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Dimensions</b>: The dimension settings for the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time when the segment was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the segment is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Attributes</i> One or more custom attributes to use as criteria for the segment.<br/>The behavior-based criteria, such as how recently users have used your app, for the segment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Behavior</b>: The behavior-based criteria, such as how recently users have used your app, for the segment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Size</i> The number of endpoint definitions that were imported successfully to create the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Url</i> The URL of the Amazon Simple Storage Service (Amazon S3) bucket that the endpoint definitions were imported from to create the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The Amazon Resource Name (ARN) of the AWS Identity and Access Management (IAM) role that authorized Amazon Pinpoint to access the Amazon S3 location to import endpoint definitions from.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> The format of the files that were imported to create the segment. Valid values are: CSV, for comma-separated values format; and, JSON, for newline-delimited JSON format.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalId</i> (Deprecated) Your AWS account ID, which you assigned to an external ID key in an IAM trust policy. Amazon Pinpoint previously used this value to assume an IAM role when importing endpoint definitions, but we removed this requirement. We don't recommend use of external IDs for IAM roles that are assumed by Amazon Pinpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ChannelCounts</i> The number of channel types in the endpoint definitions that were imported to create the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Include</i> Specifies how to handle multiple segment groups for the segment. For example, if the segment includes three segment groups, whether the resulting segment includes endpoints that match all, any, or none of the segment groups.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Groups</i> An array that defines the set of segment criteria to evaluate when handling segment groups for the segment.<br/>        """
        pass

    def delete_adm_channel(self, ApplicationId: str) -> DeleteAdmChannelResponse:
        """Disables the ADM channel for an application and deletes any existing settings for the channel.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: ADMChannelResponse:  Provides information about the status and settings of the ADM (Amazon Device Messaging) channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the ADM channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The type of messaging or notification platform for the channel. For the ADM channel, this value is ADM.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time when the ADM channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the ADM channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the ADM channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the ADM channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the ADM channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time when the ADM channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the ADM channel applies to.<br/>        """
        pass

    def delete_apns_channel(self, ApplicationId: str) -> DeleteApnsChannelResponse:
        """Disables the APNs channel for an application and deletes any existing settings for the channel.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: APNSChannelResponse:  Provides information about the status and settings of the APNs (Apple Push Notification service) channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the APNs channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i>  The type of messaging or notification platform for the channel. For the APNs channel, this value is APNS.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time when the APNs channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the APNs channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the APNs channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the APNs channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasTokenKey</i> Specifies whether the APNs channel is configured to communicate with APNs by using APNs tokens. To provide an authentication key for APNs tokens, set the TokenKey property of the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the APNs channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultAuthenticationMethod</i> The default authentication method that Amazon Pinpoint uses to authenticate with APNs for this channel, key or certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time when the APNs channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the APNs channel applies to.<br/>        """
        pass

    def delete_apns_sandbox_channel(self, ApplicationId: str) -> DeleteApnsSandboxChannelResponse:
        """Disables the APNs sandbox channel for an application and deletes any existing settings for the channel.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: APNSSandboxChannelResponse:  Provides information about the status and settings of the APNs (Apple Push Notification service) sandbox channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the APNs sandbox channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The type of messaging or notification platform for the channel. For the APNs sandbox channel, this value is APNS_SANDBOX.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time when the APNs sandbox channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the APNs sandbox channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the APNs sandbox channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the APNs sandbox channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasTokenKey</i> Specifies whether the APNs sandbox channel is configured to communicate with APNs by using APNs tokens. To provide an authentication key for APNs tokens, set the TokenKey property of the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the APNs sandbox channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultAuthenticationMethod</i> The default authentication method that Amazon Pinpoint uses to authenticate with the APNs sandbox environment for this channel, key or certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time when the APNs sandbox channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the APNs sandbox channel applies to.<br/>        """
        pass

    def delete_apns_voip_channel(self, ApplicationId: str) -> DeleteApnsVoipChannelResponse:
        """Disables the APNs VoIP channel for an application and deletes any existing settings for the channel.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: APNSVoipChannelResponse:  Provides information about the status and settings of the APNs (Apple Push Notification service) VoIP channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the APNs VoIP channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The type of messaging or notification platform for the channel. For the APNs VoIP channel, this value is APNS_VOIP.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time when the APNs VoIP channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the APNs VoIP channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the APNs VoIP channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the APNs VoIP channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasTokenKey</i> Specifies whether the APNs VoIP channel is configured to communicate with APNs by using APNs tokens. To provide an authentication key for APNs tokens, set the TokenKey property of the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the APNs VoIP channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultAuthenticationMethod</i> The default authentication method that Amazon Pinpoint uses to authenticate with APNs for this channel, key or certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time when the APNs VoIP channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the APNs VoIP channel applies to.<br/>        """
        pass

    def delete_apns_voip_sandbox_channel(self, ApplicationId: str) -> DeleteApnsVoipSandboxChannelResponse:
        """Disables the APNs VoIP sandbox channel for an application and deletes any existing settings for the channel.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: APNSVoipSandboxChannelResponse:  Provides information about the status and settings of the APNs (Apple Push Notification service) VoIP sandbox channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the APNs VoIP sandbox channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The type of messaging or notification platform for the channel. For the APNs VoIP sandbox channel, this value is APNS_VOIP_SANDBOX.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time when the APNs VoIP sandbox channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the APNs VoIP sandbox channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the APNs VoIP sandbox channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the APNs VoIP sandbox channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasTokenKey</i> Specifies whether the APNs VoIP sandbox channel is configured to communicate with APNs by using APNs tokens. To provide an authentication key for APNs tokens, set the TokenKey property of the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the APNs VoIP sandbox channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultAuthenticationMethod</i> The default authentication method that Amazon Pinpoint uses to authenticate with the APNs sandbox environment for this channel, key or certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time when the APNs VoIP sandbox channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the APNs VoIP sandbox channel applies to.<br/>        """
        pass

    def delete_app(self, ApplicationId: str) -> DeleteAppResponse:
        """Deletes an application.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: ApplicationResponse:  Provides information about an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> A string-to-string map of key-value pairs that identifies the tags that are associated with the application. Each tag consists of a required tag key and an associated tag value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The display name of the application. This name is displayed as the <b>Project name</b> on the Amazon Pinpoint console.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the application.<br/>        """
        pass

    def delete_baidu_channel(self, ApplicationId: str) -> DeleteBaiduChannelResponse:
        """Disables the Baidu channel for an application and deletes any existing settings for the channel.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: BaiduChannelResponse:  Provides information about the status and settings of the Baidu (Baidu Cloud Push) channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the Baidu channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The type of messaging or notification platform for the channel. For the Baidu channel, this value is BAIDU.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time when the Baidu channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the Baidu channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the Baidu channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the Baidu channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the Baidu channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Credential</i> The API key that you received from the Baidu Cloud Push service to communicate with the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time when the Baidu channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the Baidu channel applies to.<br/>        """
        pass

    def delete_campaign(self, CampaignId: str, ApplicationId: str) -> DeleteCampaignResponse:
        """Deletes a campaign from an application.

        :param self:
        :param CampaignId:  The unique identifier for the campaign.<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: CampaignResponse:  Provides information about the status, configuration, and other settings for a campaign.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version number of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TreatmentName</i> The custom name of a variation of the campaign that's used for A/B testing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TreatmentDescription</i> The custom description of a variation of the campaign that's used for A/B testing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> A string-to-string map of key-value pairs that identifies the tags that are associated with the campaign. Each tag consists of a required tag key and an associated tag value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The current status of the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>State</b>: The current status of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentVersion</i> The version number of the segment that's associated with the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentId</i> The unique identifier for the segment that's associated with the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Schedule</i> The schedule settings for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Schedule</b>: The schedule settings for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MessageConfiguration</i> The message configuration settings for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>MessageConfiguration</b>: The message configuration settings for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Limits</i> The messaging limits for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Limits</b>: The messaging limits for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date, in ISO 8601 format, when the campaign was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsPaused</i> Specifies whether the campaign is paused. A paused campaign doesn't run unless you resume it by changing this value to false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Hook</i> The settings for the AWS Lambda function to use as a code hook for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Hook</b>: The settings for the AWS Lambda function to use as a code hook for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HoldoutPercent</i> The allocated percentage of users (segment members) who shouldn't receive messages from the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The custom description of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultState</i> The current status of the campaign's default treatment. This value exists only for campaigns that have more than one treatment, to support A/B testing.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DefaultState</b>: The current status of the campaign's default treatment. This value exists only for campaigns that have more than one treatment, to support A/B testing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date, ISO 8601 format, when the campaign was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the campaign applies to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdditionalTreatments</i> An array of responses, one for each treatment that you defined for the campaign, in addition to the default treatment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CampaignStatus</i> The status of the campaign, or the status of a treatment that belongs to an A/B test campaign. If a campaign uses A/B testing, the campaign has a status of COMPLETED only when all campaign treatments have a status of COMPLETED.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WebUrl</i>  The web URL that Amazon Pinpoint calls to invoke the AWS Lambda function over HTTPS.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> Specifies which Lambda mode to use when invoking the AWS Lambda function.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LambdaFunctionName</i> The name or Amazon Resource Name (ARN) of the AWS Lambda function that Amazon Pinpoint invokes to send messages for a campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Total</i> The maximum number of messages that a campaign can send to a single endpoint during the course of the campaign. The maximum value is 100.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MessagesPerSecond</i> The maximum number of messages that a campaign can send each second. The minimum value is 50. The maximum value is 20,000.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumDuration</i> The maximum amount of time, in seconds, that a campaign can attempt to deliver a message after the scheduled start time for the campaign. The minimum value is 60 seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Daily</i> The maximum number of messages that a campaign can send to a single endpoint during a 24-hour period. The maximum value is 100.<br/>The message that the campaign sends through the ADM (Amazon Device Messaging) channel. This message overrides the default message.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ADMMessage</b>: The message that the campaign sends through the ADM (Amazon Device Messaging) channel. This message overrides the default message.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> The scheduled time, in ISO 8601 format, for the campaign to end.<br/>The type of event that causes the campaign to be sent, if the value of the Frequency property is EVENT.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EventFilter</b>: The type of event that causes the campaign to be sent, if the value of the Frequency property is EVENT.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CampaignStatus</i> The status of the campaign, or the status of a treatment that belongs to an A/B test campaign. If a campaign uses A/B testing, the campaign has a status of COMPLETED only when all campaign treatments have a status of COMPLETED.<br/>        """
        pass

    def delete_email_channel(self, ApplicationId: str) -> DeleteEmailChannelResponse:
        """Disables the email channel for an application and deletes any existing settings for the channel.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: EmailChannelResponse:  Provides information about the status and settings of the email channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the email channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i>  The ARN of the AWS Identity and Access Management (IAM) role that Amazon Pinpoint uses to submit email-related event data for the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The type of messaging or notification platform for the channel. For the email channel, this value is EMAIL.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MessagesPerSecond</i> The maximum number of emails that you can send through the channel each second.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time, in ISO 8601 format, when the email channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the email channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the email channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Identity</i>  The Amazon Resource Name (ARN) of the identity, verified with Amazon Simple Email Service (Amazon SES), that you use when you send email through the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the email channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FromAddress</i> The verified email address that you send email from when you send email through the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the email channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time, in ISO 8601 format, when the email channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigurationSet</i> The configuration set that's applied to email that's sent through the channel by using the <a href="emailAPIreference.html">Amazon Pinpoint Email API</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the email channel applies to.<br/>        """
        pass

    def delete_endpoint(self, EndpointId: str, ApplicationId: str) -> DeleteEndpointResponse:
        """Deletes an endpoint from an application.

        :param self:
        :param EndpointId:  The unique identifier for the endpoint.<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: EndpointResponse:  Provides information about the channel type and other settings for an endpoint.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>User</i> One or more custom user attributes that your app reports to Amazon Pinpoint for the user who's associated with the endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>User</b>: One or more custom user attributes that your app reports to Amazon Pinpoint for the user who's associated with the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestId</i> The unique identifier for the most recent request to update the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptOut</i> Specifies whether the user who's associated with the endpoint has opted out of receiving messages and push notifications from you. Possible values are: ALL, the user has opted out and doesn't want to receive any messages or push notifications; and, NONE, the user hasn't opted out and wants to receive all messages and push notifications.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Metrics</i> One or more custom metrics that your app reports to Amazon Pinpoint for the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Location</i> The geographic information for the endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Location</b>: The geographic information for the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier that you assigned to the endpoint. The identifier should be a globally unique identifier (GUID) to ensure that it doesn't conflict with other endpoint identifiers that are associated with the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointStatus</i> Not used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EffectiveDate</i> The date and time, in ISO 8601 format, when the endpoint was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Demographic</i> The demographic information for the endpoint, such as the time zone and platform.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Demographic</b>: The demographic information for the endpoint, such as the time zone and platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time, in ISO 8601 format, when the endpoint was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CohortId</i> A number from 0-99 that represents the cohort that the endpoint is assigned to. Endpoints are grouped into cohorts randomly, and each cohort contains approximately 1 percent of the endpoints for an application. Amazon Pinpoint assigns cohorts to the holdout or treatment allocations for campaigns.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ChannelType</i> The channel that's used when sending messages or push notifications to the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Attributes</i> One or more custom attributes that describe the endpoint by associating a name with an array of values. For example, the value of a custom attribute named Interests might be: ["science", "music", "travel"]. You can use these attributes as filter criteria when you create segments.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that's associated with the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The destination address for messages or push notifications that you send to the endpoint. The address varies by channel. For example, the address for a push-notification channel is typically the token provided by a push notification service, such as an Apple Push Notification service (APNs) device token or a Firebase Cloud Messaging (FCM) registration token. The address for the SMS channel is a phone number in E.164 format, such as +12065550100. The address for the email channel is an email address.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Timezone</i> The time zone of the endpoint, specified as a tz database name value, such as America/Los_Angeles.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PlatformVersion</i> The platform version of the endpoint device.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The platform of the endpoint device, such as iOS or Android.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ModelVersion</i> The model version of the endpoint device.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Model</i> The model name or number of the endpoint device, such as iPhone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Make</i> The manufacturer of the endpoint device, such as Apple or Samsung.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Locale</i> The locale of the endpoint, in the following format: the ISO 639-1 alpha-2 code, followed by an underscore (_), followed by an ISO 3166-1 alpha-2 value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AppVersion</i> The version of the app that's associated with the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Region</i> The name of the region where the endpoint is located. For locations in the United States, this value is the name of a state.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PostalCode</i> The postal or ZIP code for the area where the endpoint is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Longitude</i> The longitude coordinate of the endpoint location, rounded to one decimal place.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Latitude</i> The latitude coordinate of the endpoint location, rounded to one decimal place.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Country</i> The two-character code, in ISO 3166-1 alpha-2 format, for the country or region where the endpoint is located. For example, US for the United States.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>City</i> The name of the city where the endpoint is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UserId</i> The unique identifier for the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UserAttributes</i> One or more custom attributes that describe the user by associating a name with an array of values. For example, the value of an attribute named Interests might be: ["science", "music", "travel"]. You can use these attributes as filter criteria when you create segments. When you define the name of a custom attribute, avoid using the following characters: number sign (#), colon (:), question mark (?), backslash (\), and slash (/). The Amazon Pinpoint console can't display attribute names that contain these characters. This limitation doesn't apply to attribute values.<br/>        """
        pass

    def delete_event_stream(self, ApplicationId: str) -> DeleteEventStreamResponse:
        """Deletes the event stream for an application.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: EventStream:  Specifies settings for publishing event data to an Amazon Kinesis data stream or an Amazon Kinesis Data Firehose delivery stream.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The AWS Identity and Access Management (IAM) role that authorizes Amazon Pinpoint to publish event data to the stream in your AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdatedBy</i> The IAM user who last modified the event stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date, in ISO 8601 format, when the event stream was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalId</i> (Deprecated) Your AWS account ID, which you assigned to an external ID key in an IAM trust policy. Amazon Pinpoint previously used this value to assume an IAM role when publishing event data, but we removed this requirement. We don't recommend use of external IDs for IAM roles that are assumed by Amazon Pinpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationStreamArn</i> The Amazon Resource Name (ARN) of the Amazon Kinesis data stream or Amazon Kinesis Data Firehose delivery stream to publish event data to. For a Kinesis data stream, the ARN format is: arn:aws:kinesis:<replaceable>region</replaceable>:<replaceable>account-id</replaceable>:stream/<replaceable>stream_name</replaceable>
                For a Kinesis Data Firehose delivery stream, the ARN format is: arn:aws:firehose:<replaceable>region</replaceable>:<replaceable>account-id</replaceable>:deliverystream/<replaceable>stream_name</replaceable>
               <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application to publish event data for.<br/>        """
        pass

    def delete_gcm_channel(self, ApplicationId: str) -> DeleteGcmChannelResponse:
        """Disables the GCM channel for an application and deletes any existing settings for the channel.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: GCMChannelResponse:  Provides information about the status and settings of the GCM channel for an application. The GCM channel enables Amazon Pinpoint to send push notifications through the Firebase Cloud Messaging (FCM), formerly Google Cloud Messaging (GCM), service.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the GCM channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The type of messaging or notification platform for the channel. For the GCM channel, this value is GCM.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time when the GCM channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the GCM channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the GCM channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the GCM channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the GCM channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Credential</i> The API key, also referred to as a <i>server key</i>, that you received from Google to communicate with Google services.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time when the GCM channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the GCM channel applies to.<br/>        """
        pass

    def delete_segment(self, SegmentId: str, ApplicationId: str) -> DeleteSegmentResponse:
        """Deletes a segment from an application.

        :param self:
        :param SegmentId:  The unique identifier for the segment.<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: SegmentResponse:  Provides information about the configuration, dimension, and other settings for a segment.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version number of the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> A string-to-string map of key-value pairs that identifies the tags that are associated with the segment. Each tag consists of a required tag key and an associated tag value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentType</i> The segment type. Valid values are: <ul><li>DIMENSIONAL - A dynamic segment, which is a segment that uses selection criteria that you specify and is based on endpoint data that's reported by your app. Dynamic segments can change over time.</li> <li>IMPORT - A static segment, which is a segment that uses selection criteria that you specify and is based on endpoint definitions that you import from a file. Imported segments are static; they don't change over time.</li></ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentGroups</i> A list of one or more segment groups that apply to the segment. Each segment group consists of zero or more base segments and the dimensions that are applied to those base segments.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SegmentGroups</b>: A list of one or more segment groups that apply to the segment. Each segment group consists of zero or more base segments and the dimensions that are applied to those base segments.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time when the segment was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImportDefinition</i> The settings for the import job that's associated with the segment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ImportDefinition</b>: The settings for the import job that's associated with the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier for the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Dimensions</i> The dimension settings for the segment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Dimensions</b>: The dimension settings for the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time when the segment was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the segment is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Attributes</i> One or more custom attributes to use as criteria for the segment.<br/>The behavior-based criteria, such as how recently users have used your app, for the segment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Behavior</b>: The behavior-based criteria, such as how recently users have used your app, for the segment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Size</i> The number of endpoint definitions that were imported successfully to create the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Url</i> The URL of the Amazon Simple Storage Service (Amazon S3) bucket that the endpoint definitions were imported from to create the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The Amazon Resource Name (ARN) of the AWS Identity and Access Management (IAM) role that authorized Amazon Pinpoint to access the Amazon S3 location to import endpoint definitions from.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> The format of the files that were imported to create the segment. Valid values are: CSV, for comma-separated values format; and, JSON, for newline-delimited JSON format.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalId</i> (Deprecated) Your AWS account ID, which you assigned to an external ID key in an IAM trust policy. Amazon Pinpoint previously used this value to assume an IAM role when importing endpoint definitions, but we removed this requirement. We don't recommend use of external IDs for IAM roles that are assumed by Amazon Pinpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ChannelCounts</i> The number of channel types in the endpoint definitions that were imported to create the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Include</i> Specifies how to handle multiple segment groups for the segment. For example, if the segment includes three segment groups, whether the resulting segment includes endpoints that match all, any, or none of the segment groups.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Groups</i> An array that defines the set of segment criteria to evaluate when handling segment groups for the segment.<br/>        """
        pass

    def delete_sms_channel(self, ApplicationId: str) -> DeleteSmsChannelResponse:
        """Disables the SMS channel for an application and deletes any existing settings for the channel.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: SMSChannelResponse:  Provides information about the status and settings of the SMS channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the SMS channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransactionalMessagesPerSecond</i> The maximum number of transactional messages that you can send through the SMS channel each second.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ShortCode</i> The registered short code to use when you send messages through the SMS channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SenderId</i> The identity that displays on recipients' devices when they receive messages from the SMS channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PromotionalMessagesPerSecond</i> The maximum number of promotional messages that you can send through the SMS channel each second.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The type of messaging or notification platform for the channel. For the SMS channel, this value is SMS.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time, in ISO 8601 format, when the SMS channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the SMS channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the SMS channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the SMS channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the SMS channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time, in ISO 8601 format, when the SMS channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the SMS channel applies to.<br/>        """
        pass

    def delete_user_endpoints(self, UserId: str, ApplicationId: str) -> DeleteUserEndpointsResponse:
        """Deletes all the endpoints that are associated with a specific user ID.

        :param self:
        :param UserId:  The unique identifier for the user.<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: EndpointsResponse:  Provides information about all the endpoints that are associated with a user ID.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Item</i> An array of responses, one for each endpoint that's associated with the user ID.<br/>        """
        pass

    def delete_voice_channel(self, ApplicationId: str) -> DeleteVoiceChannelResponse:
        """Disables the voice channel for an application and deletes any existing settings for the channel.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: VoiceChannelResponse:  Provides information about the status and settings of the voice channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the voice channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The type of messaging or notification platform for the channel. For the voice channel, this value is VOICE.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time, in ISO 8601 format, when the voice channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the voice channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the voice channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the voice channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the voice channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time, in ISO 8601 format, when the voice channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the voice channel applies to.<br/>        """
        pass

    def get_adm_channel(self, ApplicationId: str) -> GetAdmChannelResponse:
        """Retrieves information about the status and settings of the ADM channel for an application.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: ADMChannelResponse:  Provides information about the status and settings of the ADM (Amazon Device Messaging) channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the ADM channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The type of messaging or notification platform for the channel. For the ADM channel, this value is ADM.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time when the ADM channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the ADM channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the ADM channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the ADM channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the ADM channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time when the ADM channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the ADM channel applies to.<br/>        """
        pass

    def get_apns_channel(self, ApplicationId: str) -> GetApnsChannelResponse:
        """Retrieves information about the status and settings of the APNs channel for an application.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: APNSChannelResponse:  Provides information about the status and settings of the APNs (Apple Push Notification service) channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the APNs channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i>  The type of messaging or notification platform for the channel. For the APNs channel, this value is APNS.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time when the APNs channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the APNs channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the APNs channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the APNs channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasTokenKey</i> Specifies whether the APNs channel is configured to communicate with APNs by using APNs tokens. To provide an authentication key for APNs tokens, set the TokenKey property of the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the APNs channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultAuthenticationMethod</i> The default authentication method that Amazon Pinpoint uses to authenticate with APNs for this channel, key or certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time when the APNs channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the APNs channel applies to.<br/>        """
        pass

    def get_apns_sandbox_channel(self, ApplicationId: str) -> GetApnsSandboxChannelResponse:
        """Retrieves information about the status and settings of the APNs sandbox channel for an application.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: APNSSandboxChannelResponse:  Provides information about the status and settings of the APNs (Apple Push Notification service) sandbox channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the APNs sandbox channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The type of messaging or notification platform for the channel. For the APNs sandbox channel, this value is APNS_SANDBOX.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time when the APNs sandbox channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the APNs sandbox channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the APNs sandbox channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the APNs sandbox channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasTokenKey</i> Specifies whether the APNs sandbox channel is configured to communicate with APNs by using APNs tokens. To provide an authentication key for APNs tokens, set the TokenKey property of the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the APNs sandbox channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultAuthenticationMethod</i> The default authentication method that Amazon Pinpoint uses to authenticate with the APNs sandbox environment for this channel, key or certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time when the APNs sandbox channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the APNs sandbox channel applies to.<br/>        """
        pass

    def get_apns_voip_channel(self, ApplicationId: str) -> GetApnsVoipChannelResponse:
        """Retrieves information about the status and settings of the APNs VoIP channel for an application.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: APNSVoipChannelResponse:  Provides information about the status and settings of the APNs (Apple Push Notification service) VoIP channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the APNs VoIP channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The type of messaging or notification platform for the channel. For the APNs VoIP channel, this value is APNS_VOIP.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time when the APNs VoIP channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the APNs VoIP channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the APNs VoIP channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the APNs VoIP channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasTokenKey</i> Specifies whether the APNs VoIP channel is configured to communicate with APNs by using APNs tokens. To provide an authentication key for APNs tokens, set the TokenKey property of the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the APNs VoIP channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultAuthenticationMethod</i> The default authentication method that Amazon Pinpoint uses to authenticate with APNs for this channel, key or certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time when the APNs VoIP channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the APNs VoIP channel applies to.<br/>        """
        pass

    def get_apns_voip_sandbox_channel(self, ApplicationId: str) -> GetApnsVoipSandboxChannelResponse:
        """Retrieves information about the status and settings of the APNs VoIP sandbox channel for an application.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: APNSVoipSandboxChannelResponse:  Provides information about the status and settings of the APNs (Apple Push Notification service) VoIP sandbox channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the APNs VoIP sandbox channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The type of messaging or notification platform for the channel. For the APNs VoIP sandbox channel, this value is APNS_VOIP_SANDBOX.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time when the APNs VoIP sandbox channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the APNs VoIP sandbox channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the APNs VoIP sandbox channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the APNs VoIP sandbox channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasTokenKey</i> Specifies whether the APNs VoIP sandbox channel is configured to communicate with APNs by using APNs tokens. To provide an authentication key for APNs tokens, set the TokenKey property of the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the APNs VoIP sandbox channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultAuthenticationMethod</i> The default authentication method that Amazon Pinpoint uses to authenticate with the APNs sandbox environment for this channel, key or certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time when the APNs VoIP sandbox channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the APNs VoIP sandbox channel applies to.<br/>        """
        pass

    def get_app(self, ApplicationId: str) -> GetAppResponse:
        """Retrieves information about an application.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: ApplicationResponse:  Provides information about an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> A string-to-string map of key-value pairs that identifies the tags that are associated with the application. Each tag consists of a required tag key and an associated tag value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The display name of the application. This name is displayed as the <b>Project name</b> on the Amazon Pinpoint console.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the application.<br/>        """
        pass

    def get_application_settings(self, ApplicationId: str) -> GetApplicationSettingsResponse:
        """Retrieves information about the settings for an application.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: ApplicationSettingsResource:  Provides information about an application, including the default settings for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QuietTime</i> The default quiet time for campaigns in the application. Quiet time is a specific time range when campaigns don't send messages to endpoints, if all the following conditions are met: <ul><li>The EndpointDemographic.Timezone property of the endpoint is set to a valid value.</li> <li>The current time in the endpoint's time zone is later than or equal to the time specified by the QuietTime.Start property for the application (or a campaign that has custom quiet time settings).</li> <li>The current time in the endpoint's time zone is earlier than or equal to the time specified by the QuietTime.End property for the application (or a campaign that has custom quiet time settings).</li></ul> If any of the preceding conditions isn't met, the endpoint will receive messages from a campaign, even if quiet time is enabled.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>QuietTime</b>: The default quiet time for campaigns in the application. Quiet time is a specific time range when campaigns don't send messages to endpoints, if all the following conditions are met: <ul><li>The EndpointDemographic.Timezone property of the endpoint is set to a valid value.</li> <li>The current time in the endpoint's time zone is later than or equal to the time specified by the QuietTime.Start property for the application (or a campaign that has custom quiet time settings).</li> <li>The current time in the endpoint's time zone is earlier than or equal to the time specified by the QuietTime.End property for the application (or a campaign that has custom quiet time settings).</li></ul> If any of the preceding conditions isn't met, the endpoint will receive messages from a campaign, even if quiet time is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Limits</i> The default sending limits for campaigns in the application.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Limits</b>: The default sending limits for campaigns in the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time, in ISO 8601 format, when the application's settings were last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CampaignHook</i> The settings for the AWS Lambda function to use by default as a code hook for campaigns in the application.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CampaignHook</b>: The settings for the AWS Lambda function to use by default as a code hook for campaigns in the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WebUrl</i>  The web URL that Amazon Pinpoint calls to invoke the AWS Lambda function over HTTPS.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> Specifies which Lambda mode to use when invoking the AWS Lambda function.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LambdaFunctionName</i> The name or Amazon Resource Name (ARN) of the AWS Lambda function that Amazon Pinpoint invokes to send messages for a campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Total</i> The maximum number of messages that a campaign can send to a single endpoint during the course of the campaign. The maximum value is 100.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MessagesPerSecond</i> The maximum number of messages that a campaign can send each second. The minimum value is 50. The maximum value is 20,000.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumDuration</i> The maximum amount of time, in seconds, that a campaign can attempt to deliver a message after the scheduled start time for the campaign. The minimum value is 60 seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Daily</i> The maximum number of messages that a campaign can send to a single endpoint during a 24-hour period. The maximum value is 100.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Start</i> The specific time when quiet time begins. This value has to use 24-hour notation and be in HH:MM format, where HH is the hour (with a leading zero, if applicable) and MM is the minutes. For example, use 02:30 to represent 2:30 AM, or 14:30 to represent 2:30 PM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>End</i> The specific time when quiet time ends. This value has to use 24-hour notation and be in HH:MM format, where HH is the hour (with a leading zero, if applicable) and MM is the minutes. For example, use 02:30 to represent 2:30 AM, or 14:30 to represent 2:30 PM.<br/>        """
        pass

    def get_apps(self, PageSize: str=None, Token: str=None) -> GetAppsResponse:
        """Retrieves information about all of your applications.

        :param self:
        :param Token:  The NextToken string that specifies which page of results to return in a paginated response.<br/>PageSize:  The maximum number of items to include on each page in a paginated response.<br/>
        :return: ApplicationsResponse:  Provides information about all of your applications.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NextToken</i> The string to use in a subsequent request to get the next page of results in a paginated response. This value is null if there are no additional pages.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Item</i> An array of responses, one for each application that was returned.<br/>        """
        pass

    def get_baidu_channel(self, ApplicationId: str) -> GetBaiduChannelResponse:
        """Retrieves information about the status and settings of the Baidu Cloud Push channel for an application.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: BaiduChannelResponse:  Provides information about the status and settings of the Baidu (Baidu Cloud Push) channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the Baidu channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The type of messaging or notification platform for the channel. For the Baidu channel, this value is BAIDU.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time when the Baidu channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the Baidu channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the Baidu channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the Baidu channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the Baidu channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Credential</i> The API key that you received from the Baidu Cloud Push service to communicate with the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time when the Baidu channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the Baidu channel applies to.<br/>        """
        pass

    def get_campaign(self, CampaignId: str, ApplicationId: str) -> GetCampaignResponse:
        """Retrieves information about the status, configuration, and other settings for a campaign.

        :param self:
        :param CampaignId:  The unique identifier for the campaign.<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: CampaignResponse:  Provides information about the status, configuration, and other settings for a campaign.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version number of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TreatmentName</i> The custom name of a variation of the campaign that's used for A/B testing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TreatmentDescription</i> The custom description of a variation of the campaign that's used for A/B testing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> A string-to-string map of key-value pairs that identifies the tags that are associated with the campaign. Each tag consists of a required tag key and an associated tag value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The current status of the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>State</b>: The current status of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentVersion</i> The version number of the segment that's associated with the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentId</i> The unique identifier for the segment that's associated with the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Schedule</i> The schedule settings for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Schedule</b>: The schedule settings for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MessageConfiguration</i> The message configuration settings for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>MessageConfiguration</b>: The message configuration settings for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Limits</i> The messaging limits for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Limits</b>: The messaging limits for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date, in ISO 8601 format, when the campaign was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsPaused</i> Specifies whether the campaign is paused. A paused campaign doesn't run unless you resume it by changing this value to false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Hook</i> The settings for the AWS Lambda function to use as a code hook for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Hook</b>: The settings for the AWS Lambda function to use as a code hook for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HoldoutPercent</i> The allocated percentage of users (segment members) who shouldn't receive messages from the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The custom description of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultState</i> The current status of the campaign's default treatment. This value exists only for campaigns that have more than one treatment, to support A/B testing.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DefaultState</b>: The current status of the campaign's default treatment. This value exists only for campaigns that have more than one treatment, to support A/B testing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date, ISO 8601 format, when the campaign was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the campaign applies to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdditionalTreatments</i> An array of responses, one for each treatment that you defined for the campaign, in addition to the default treatment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CampaignStatus</i> The status of the campaign, or the status of a treatment that belongs to an A/B test campaign. If a campaign uses A/B testing, the campaign has a status of COMPLETED only when all campaign treatments have a status of COMPLETED.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WebUrl</i>  The web URL that Amazon Pinpoint calls to invoke the AWS Lambda function over HTTPS.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> Specifies which Lambda mode to use when invoking the AWS Lambda function.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LambdaFunctionName</i> The name or Amazon Resource Name (ARN) of the AWS Lambda function that Amazon Pinpoint invokes to send messages for a campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Total</i> The maximum number of messages that a campaign can send to a single endpoint during the course of the campaign. The maximum value is 100.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MessagesPerSecond</i> The maximum number of messages that a campaign can send each second. The minimum value is 50. The maximum value is 20,000.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumDuration</i> The maximum amount of time, in seconds, that a campaign can attempt to deliver a message after the scheduled start time for the campaign. The minimum value is 60 seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Daily</i> The maximum number of messages that a campaign can send to a single endpoint during a 24-hour period. The maximum value is 100.<br/>The message that the campaign sends through the ADM (Amazon Device Messaging) channel. This message overrides the default message.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ADMMessage</b>: The message that the campaign sends through the ADM (Amazon Device Messaging) channel. This message overrides the default message.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> The scheduled time, in ISO 8601 format, for the campaign to end.<br/>The type of event that causes the campaign to be sent, if the value of the Frequency property is EVENT.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EventFilter</b>: The type of event that causes the campaign to be sent, if the value of the Frequency property is EVENT.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CampaignStatus</i> The status of the campaign, or the status of a treatment that belongs to an A/B test campaign. If a campaign uses A/B testing, the campaign has a status of COMPLETED only when all campaign treatments have a status of COMPLETED.<br/>        """
        pass

    def get_campaign_activities(self, CampaignId: str, ApplicationId: str, PageSize: str=None, Token: str=None) -> GetCampaignActivitiesResponse:
        """Retrieves information about the activity performed by a campaign.

        :param self:
        :param Token:  The NextToken string that specifies which page of results to return in a paginated response.<br/>PageSize:  The maximum number of items to include on each page in a paginated response.<br/>CampaignId:  The unique identifier for the campaign.<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: ActivitiesResponse:  Provides information about the activities that were performed by a campaign.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NextToken</i> The string to use in a subsequent request to get the next page of results in a paginated response. This value is null if there are no additional pages.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Item</i> An array of responses, one for each activity that was performed by the campaign.<br/>        """
        pass

    def get_campaign_version(self, Version: str, CampaignId: str, ApplicationId: str) -> GetCampaignVersionResponse:
        """Retrieves information about the status, configuration, and other settings for a specific version of a campaign.

        :param self:
        :param Version:  The unique version number (Version property) for the campaign version.<br/>CampaignId:  The unique identifier for the campaign.<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: CampaignResponse:  Provides information about the status, configuration, and other settings for a campaign.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version number of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TreatmentName</i> The custom name of a variation of the campaign that's used for A/B testing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TreatmentDescription</i> The custom description of a variation of the campaign that's used for A/B testing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> A string-to-string map of key-value pairs that identifies the tags that are associated with the campaign. Each tag consists of a required tag key and an associated tag value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The current status of the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>State</b>: The current status of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentVersion</i> The version number of the segment that's associated with the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentId</i> The unique identifier for the segment that's associated with the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Schedule</i> The schedule settings for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Schedule</b>: The schedule settings for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MessageConfiguration</i> The message configuration settings for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>MessageConfiguration</b>: The message configuration settings for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Limits</i> The messaging limits for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Limits</b>: The messaging limits for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date, in ISO 8601 format, when the campaign was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsPaused</i> Specifies whether the campaign is paused. A paused campaign doesn't run unless you resume it by changing this value to false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Hook</i> The settings for the AWS Lambda function to use as a code hook for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Hook</b>: The settings for the AWS Lambda function to use as a code hook for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HoldoutPercent</i> The allocated percentage of users (segment members) who shouldn't receive messages from the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The custom description of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultState</i> The current status of the campaign's default treatment. This value exists only for campaigns that have more than one treatment, to support A/B testing.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DefaultState</b>: The current status of the campaign's default treatment. This value exists only for campaigns that have more than one treatment, to support A/B testing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date, ISO 8601 format, when the campaign was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the campaign applies to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdditionalTreatments</i> An array of responses, one for each treatment that you defined for the campaign, in addition to the default treatment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CampaignStatus</i> The status of the campaign, or the status of a treatment that belongs to an A/B test campaign. If a campaign uses A/B testing, the campaign has a status of COMPLETED only when all campaign treatments have a status of COMPLETED.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WebUrl</i>  The web URL that Amazon Pinpoint calls to invoke the AWS Lambda function over HTTPS.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> Specifies which Lambda mode to use when invoking the AWS Lambda function.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LambdaFunctionName</i> The name or Amazon Resource Name (ARN) of the AWS Lambda function that Amazon Pinpoint invokes to send messages for a campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Total</i> The maximum number of messages that a campaign can send to a single endpoint during the course of the campaign. The maximum value is 100.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MessagesPerSecond</i> The maximum number of messages that a campaign can send each second. The minimum value is 50. The maximum value is 20,000.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumDuration</i> The maximum amount of time, in seconds, that a campaign can attempt to deliver a message after the scheduled start time for the campaign. The minimum value is 60 seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Daily</i> The maximum number of messages that a campaign can send to a single endpoint during a 24-hour period. The maximum value is 100.<br/>The message that the campaign sends through the ADM (Amazon Device Messaging) channel. This message overrides the default message.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ADMMessage</b>: The message that the campaign sends through the ADM (Amazon Device Messaging) channel. This message overrides the default message.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> The scheduled time, in ISO 8601 format, for the campaign to end.<br/>The type of event that causes the campaign to be sent, if the value of the Frequency property is EVENT.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EventFilter</b>: The type of event that causes the campaign to be sent, if the value of the Frequency property is EVENT.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CampaignStatus</i> The status of the campaign, or the status of a treatment that belongs to an A/B test campaign. If a campaign uses A/B testing, the campaign has a status of COMPLETED only when all campaign treatments have a status of COMPLETED.<br/>        """
        pass

    def get_campaign_versions(self, CampaignId: str, ApplicationId: str, PageSize: str=None, Token: str=None) -> GetCampaignVersionsResponse:
        """Retrieves information about the status, configuration, and other settings for all versions of a specific campaign.

        :param self:
        :param Token:  The NextToken string that specifies which page of results to return in a paginated response.<br/>PageSize:  The maximum number of items to include on each page in a paginated response.<br/>CampaignId:  The unique identifier for the campaign.<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: CampaignsResponse:  Provides information about the configuration and other settings for all the campaigns that are associated with an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NextToken</i> The string to use in a subsequent request to get the next page of results in a paginated response. This value is null if there are no additional pages.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Item</i> An array of responses, one for each campaign that's associated with the application.<br/>        """
        pass

    def get_campaigns(self, ApplicationId: str, PageSize: str=None, Token: str=None) -> GetCampaignsResponse:
        """Retrieves information about the status, configuration, and other settings for all the campaigns that are associated with an application.

        :param self:
        :param Token:  The NextToken string that specifies which page of results to return in a paginated response.<br/>PageSize:  The maximum number of items to include on each page in a paginated response.<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: CampaignsResponse:  Provides information about the configuration and other settings for all the campaigns that are associated with an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NextToken</i> The string to use in a subsequent request to get the next page of results in a paginated response. This value is null if there are no additional pages.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Item</i> An array of responses, one for each campaign that's associated with the application.<br/>        """
        pass

    def get_channels(self, ApplicationId: str) -> GetChannelsResponse:
        """Retrieves information about the history and status of each channel for an application.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: ChannelsResponse:  Provides information about the general settings and status of all channels for an application, including channels that aren't enabled for the application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Channels</i> A map that contains a multipart response for each channel. For each item in this object, the ChannelType is the key and the Channel is the value.<br/>        """
        pass

    def get_email_channel(self, ApplicationId: str) -> GetEmailChannelResponse:
        """Retrieves information about the status and settings of the email channel for an application.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: EmailChannelResponse:  Provides information about the status and settings of the email channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the email channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i>  The ARN of the AWS Identity and Access Management (IAM) role that Amazon Pinpoint uses to submit email-related event data for the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The type of messaging or notification platform for the channel. For the email channel, this value is EMAIL.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MessagesPerSecond</i> The maximum number of emails that you can send through the channel each second.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time, in ISO 8601 format, when the email channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the email channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the email channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Identity</i>  The Amazon Resource Name (ARN) of the identity, verified with Amazon Simple Email Service (Amazon SES), that you use when you send email through the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the email channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FromAddress</i> The verified email address that you send email from when you send email through the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the email channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time, in ISO 8601 format, when the email channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigurationSet</i> The configuration set that's applied to email that's sent through the channel by using the <a href="emailAPIreference.html">Amazon Pinpoint Email API</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the email channel applies to.<br/>        """
        pass

    def get_endpoint(self, EndpointId: str, ApplicationId: str) -> GetEndpointResponse:
        """Retrieves information about the settings and attributes of a specific endpoint for an application.

        :param self:
        :param EndpointId:  The unique identifier for the endpoint.<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: EndpointResponse:  Provides information about the channel type and other settings for an endpoint.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>User</i> One or more custom user attributes that your app reports to Amazon Pinpoint for the user who's associated with the endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>User</b>: One or more custom user attributes that your app reports to Amazon Pinpoint for the user who's associated with the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestId</i> The unique identifier for the most recent request to update the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptOut</i> Specifies whether the user who's associated with the endpoint has opted out of receiving messages and push notifications from you. Possible values are: ALL, the user has opted out and doesn't want to receive any messages or push notifications; and, NONE, the user hasn't opted out and wants to receive all messages and push notifications.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Metrics</i> One or more custom metrics that your app reports to Amazon Pinpoint for the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Location</i> The geographic information for the endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Location</b>: The geographic information for the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier that you assigned to the endpoint. The identifier should be a globally unique identifier (GUID) to ensure that it doesn't conflict with other endpoint identifiers that are associated with the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointStatus</i> Not used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EffectiveDate</i> The date and time, in ISO 8601 format, when the endpoint was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Demographic</i> The demographic information for the endpoint, such as the time zone and platform.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Demographic</b>: The demographic information for the endpoint, such as the time zone and platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time, in ISO 8601 format, when the endpoint was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CohortId</i> A number from 0-99 that represents the cohort that the endpoint is assigned to. Endpoints are grouped into cohorts randomly, and each cohort contains approximately 1 percent of the endpoints for an application. Amazon Pinpoint assigns cohorts to the holdout or treatment allocations for campaigns.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ChannelType</i> The channel that's used when sending messages or push notifications to the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Attributes</i> One or more custom attributes that describe the endpoint by associating a name with an array of values. For example, the value of a custom attribute named Interests might be: ["science", "music", "travel"]. You can use these attributes as filter criteria when you create segments.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that's associated with the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The destination address for messages or push notifications that you send to the endpoint. The address varies by channel. For example, the address for a push-notification channel is typically the token provided by a push notification service, such as an Apple Push Notification service (APNs) device token or a Firebase Cloud Messaging (FCM) registration token. The address for the SMS channel is a phone number in E.164 format, such as +12065550100. The address for the email channel is an email address.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Timezone</i> The time zone of the endpoint, specified as a tz database name value, such as America/Los_Angeles.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PlatformVersion</i> The platform version of the endpoint device.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The platform of the endpoint device, such as iOS or Android.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ModelVersion</i> The model version of the endpoint device.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Model</i> The model name or number of the endpoint device, such as iPhone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Make</i> The manufacturer of the endpoint device, such as Apple or Samsung.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Locale</i> The locale of the endpoint, in the following format: the ISO 639-1 alpha-2 code, followed by an underscore (_), followed by an ISO 3166-1 alpha-2 value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AppVersion</i> The version of the app that's associated with the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Region</i> The name of the region where the endpoint is located. For locations in the United States, this value is the name of a state.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PostalCode</i> The postal or ZIP code for the area where the endpoint is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Longitude</i> The longitude coordinate of the endpoint location, rounded to one decimal place.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Latitude</i> The latitude coordinate of the endpoint location, rounded to one decimal place.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Country</i> The two-character code, in ISO 3166-1 alpha-2 format, for the country or region where the endpoint is located. For example, US for the United States.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>City</i> The name of the city where the endpoint is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UserId</i> The unique identifier for the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UserAttributes</i> One or more custom attributes that describe the user by associating a name with an array of values. For example, the value of an attribute named Interests might be: ["science", "music", "travel"]. You can use these attributes as filter criteria when you create segments. When you define the name of a custom attribute, avoid using the following characters: number sign (#), colon (:), question mark (?), backslash (\), and slash (/). The Amazon Pinpoint console can't display attribute names that contain these characters. This limitation doesn't apply to attribute values.<br/>        """
        pass

    def get_event_stream(self, ApplicationId: str) -> GetEventStreamResponse:
        """Retrieves information about the event stream settings for an application.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: EventStream:  Specifies settings for publishing event data to an Amazon Kinesis data stream or an Amazon Kinesis Data Firehose delivery stream.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The AWS Identity and Access Management (IAM) role that authorizes Amazon Pinpoint to publish event data to the stream in your AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdatedBy</i> The IAM user who last modified the event stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date, in ISO 8601 format, when the event stream was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalId</i> (Deprecated) Your AWS account ID, which you assigned to an external ID key in an IAM trust policy. Amazon Pinpoint previously used this value to assume an IAM role when publishing event data, but we removed this requirement. We don't recommend use of external IDs for IAM roles that are assumed by Amazon Pinpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationStreamArn</i> The Amazon Resource Name (ARN) of the Amazon Kinesis data stream or Amazon Kinesis Data Firehose delivery stream to publish event data to. For a Kinesis data stream, the ARN format is: arn:aws:kinesis:<replaceable>region</replaceable>:<replaceable>account-id</replaceable>:stream/<replaceable>stream_name</replaceable>
                For a Kinesis Data Firehose delivery stream, the ARN format is: arn:aws:firehose:<replaceable>region</replaceable>:<replaceable>account-id</replaceable>:deliverystream/<replaceable>stream_name</replaceable>
               <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application to publish event data for.<br/>        """
        pass

    def get_export_job(self, JobId: str, ApplicationId: str) -> GetExportJobResponse:
        """Retrieves information about the status and settings of a specific export job for an application.

        :param self:
        :param JobId:  The unique identifier for the job.<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: ExportJobResponse:  Provides information about the status and settings of a job that exports endpoint definitions to a file. The file can be added directly to an Amazon Simple Storage Service (Amazon S3) bucket by using the Amazon Pinpoint API or downloaded directly to a computer by using the Amazon Pinpoint console.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The job type. This value is EXPORT for export jobs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalProcessed</i> The total number of endpoint definitions that were processed by the export job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalPieces</i> The total number of pieces that must be processed to complete the export job. Each piece consists of an approximately equal portion of the endpoint definitions that are part of the export job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalFailures</i> The total number of endpoint definitions that weren't processed successfully (failed) by the export job, typically because an error, such as a syntax error, occurred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobStatus</i> The status of the export job. The job status is FAILED if Amazon Pinpoint wasn't able to process one or more pieces in the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier for the export job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Failures</i> An array of entries, one for each of the first 100 entries that weren't processed successfully (failed) by the export job, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailedPieces</i> The number of pieces that weren't processed successfully (failed) by the export job, as of the time of the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Definition</i> The resource settings that apply to the export job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Definition</b>: The resource settings that apply to the export job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date, in ISO 8601 format, when the export job was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompletionDate</i> The date, in ISO 8601 format, when the export job was completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompletedPieces</i> The number of pieces that were processed successfully (completed) by the export job, as of the time of the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that's associated with the export job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentVersion</i> The version of the segment that the endpoint definitions were exported from.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentId</i> The identifier for the segment that the endpoint definitions were exported from. If this value isn't present, Amazon Pinpoint exported definitions for all the endpoints that are associated with the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3UrlPrefix</i> The URL of the location in an Amazon Simple Storage Service (Amazon S3) bucket where the endpoint definitions were exported to. This location is typically a folder that contains multiple files. The URL should be in the following format: s3://<replaceable>bucket-name</replaceable>/<replaceable>folder-name</replaceable>/.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The Amazon Resource Name (ARN) of the AWS Identity and Access Management (IAM) role that authorized Amazon Pinpoint to access the Amazon S3 location where the endpoint definitions were exported to.<br/>        """
        pass

    def get_export_jobs(self, ApplicationId: str, PageSize: str=None, Token: str=None) -> GetExportJobsResponse:
        """Retrieves information about the status and settings of all the export jobs for an application.

        :param self:
        :param Token:  The NextToken string that specifies which page of results to return in a paginated response.<br/>PageSize:  The maximum number of items to include on each page in a paginated response.<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: ExportJobsResponse:  Provides information about all the export jobs that are associated with an application or segment. An export job is a job that exports endpoint definitions to a file.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NextToken</i> The string to use in a subsequent request to get the next page of results in a paginated response. This value is null if there are no additional pages.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Item</i> An array of responses, one for each export job that's associated with the application (Export Jobs resource) or segment (Segment Export Jobs resource).<br/>        """
        pass

    def get_gcm_channel(self, ApplicationId: str) -> GetGcmChannelResponse:
        """Retrieves information about the status and settings of the GCM channel for an application.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: GCMChannelResponse:  Provides information about the status and settings of the GCM channel for an application. The GCM channel enables Amazon Pinpoint to send push notifications through the Firebase Cloud Messaging (FCM), formerly Google Cloud Messaging (GCM), service.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the GCM channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The type of messaging or notification platform for the channel. For the GCM channel, this value is GCM.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time when the GCM channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the GCM channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the GCM channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the GCM channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the GCM channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Credential</i> The API key, also referred to as a <i>server key</i>, that you received from Google to communicate with Google services.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time when the GCM channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the GCM channel applies to.<br/>        """
        pass

    def get_import_job(self, JobId: str, ApplicationId: str) -> GetImportJobResponse:
        """Retrieves information about the status and settings of a specific import job for an application.

        :param self:
        :param JobId:  The unique identifier for the job.<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: ImportJobResponse:  Provides information about the status and settings of a job that imports endpoint definitions from one or more files. The files can be stored in an Amazon Simple Storage Service (Amazon S3) bucket or uploaded directly from a computer by using the Amazon Pinpoint console.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The job type. This value is IMPORT for import jobs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalProcessed</i> The total number of endpoint definitions that were processed by the import job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalPieces</i> The total number of pieces that must be processed to complete the import job. Each piece consists of an approximately equal portion of the endpoint definitions that are part of the import job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalFailures</i> The total number of endpoint definitions that weren't processed successfully (failed) by the import job, typically because an error, such as a syntax error, occurred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobStatus</i> The status of the import job. The job status is FAILED if Amazon Pinpoint wasn't able to process one or more pieces in the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier for the import job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Failures</i> An array of entries, one for each of the first 100 entries that weren't processed successfully (failed) by the import job, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailedPieces</i> The number of pieces that weren't processed successfully (failed) by the import job, as of the time of the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Definition</i> The resource settings that apply to the import job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Definition</b>: The resource settings that apply to the import job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date, in ISO 8601 format, when the import job was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompletionDate</i> The date, in ISO 8601 format, when the import job was completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompletedPieces</i> The number of pieces that were processed successfully (completed) by the import job, as of the time of the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that's associated with the import job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentName</i> The custom name for the segment that's created by the import job, if the value of the DefineSegment property is true.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentId</i> The identifier for the segment that the import job updates or adds endpoint definitions to, if the import job updates an existing segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Url</i> The URL of the Amazon Simple Storage Service (Amazon S3) bucket that contains the endpoint definitions to import. This location can be a folder or a single file. If the location is a folder, Amazon Pinpoint imports endpoint definitions from the files in this location, including any subfolders that the folder contains. The URL should be in the following format: s3://<replaceable>bucket-name</replaceable>/<replaceable>folder-name</replaceable>/<replaceable>file-name</replaceable>. The location can end with the key for an individual object or a prefix that qualifies multiple objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The Amazon Resource Name (ARN) of the AWS Identity and Access Management (IAM) role that authorizes Amazon Pinpoint to access the Amazon S3 location to import endpoint definitions from.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RegisterEndpoints</i> Specifies whether the import job registers the endpoints with Amazon Pinpoint, when the endpoint definitions are imported.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> The format of the files that contain the endpoint definitions to import. Valid values are: CSV, for comma-separated values format; and, JSON, for newline-delimited JSON format. If the files are stored in an Amazon S3 location and that location contains multiple files that use different formats, Amazon Pinpoint imports data only from the files that use the specified format.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalId</i> (Deprecated) Your AWS account ID, which you assigned to an external ID key in an IAM trust policy. Amazon Pinpoint previously used this value to assume an IAM role when importing endpoint definitions, but we removed this requirement. We don't recommend use of external IDs for IAM roles that are assumed by Amazon Pinpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DefineSegment</i> Specifies whether the import job creates a segment that contains the endpoints, when the endpoint definitions are imported.<br/>        """
        pass

    def get_import_jobs(self, ApplicationId: str, PageSize: str=None, Token: str=None) -> GetImportJobsResponse:
        """Retrieves information about the status and settings of all the import jobs for an application.

        :param self:
        :param Token:  The NextToken string that specifies which page of results to return in a paginated response.<br/>PageSize:  The maximum number of items to include on each page in a paginated response.<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: ImportJobsResponse:  Provides information about the status and settings of all the import jobs that are associated with an application or segment. An import job is a job that imports endpoint definitions from one or more files.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NextToken</i> The string to use in a subsequent request to get the next page of results in a paginated response. This value is null if there are no additional pages.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Item</i> An array of responses, one for each import job that's associated with the application (Import Jobs resource) or segment (Segment Import Jobs resource).<br/>        """
        pass

    def get_segment(self, SegmentId: str, ApplicationId: str) -> GetSegmentResponse:
        """Retrieves information about the configuration, dimension, and other settings for a specific segment that's associated with an application.

        :param self:
        :param SegmentId:  The unique identifier for the segment.<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: SegmentResponse:  Provides information about the configuration, dimension, and other settings for a segment.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version number of the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> A string-to-string map of key-value pairs that identifies the tags that are associated with the segment. Each tag consists of a required tag key and an associated tag value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentType</i> The segment type. Valid values are: <ul><li>DIMENSIONAL - A dynamic segment, which is a segment that uses selection criteria that you specify and is based on endpoint data that's reported by your app. Dynamic segments can change over time.</li> <li>IMPORT - A static segment, which is a segment that uses selection criteria that you specify and is based on endpoint definitions that you import from a file. Imported segments are static; they don't change over time.</li></ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentGroups</i> A list of one or more segment groups that apply to the segment. Each segment group consists of zero or more base segments and the dimensions that are applied to those base segments.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SegmentGroups</b>: A list of one or more segment groups that apply to the segment. Each segment group consists of zero or more base segments and the dimensions that are applied to those base segments.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time when the segment was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImportDefinition</i> The settings for the import job that's associated with the segment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ImportDefinition</b>: The settings for the import job that's associated with the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier for the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Dimensions</i> The dimension settings for the segment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Dimensions</b>: The dimension settings for the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time when the segment was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the segment is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Attributes</i> One or more custom attributes to use as criteria for the segment.<br/>The behavior-based criteria, such as how recently users have used your app, for the segment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Behavior</b>: The behavior-based criteria, such as how recently users have used your app, for the segment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Size</i> The number of endpoint definitions that were imported successfully to create the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Url</i> The URL of the Amazon Simple Storage Service (Amazon S3) bucket that the endpoint definitions were imported from to create the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The Amazon Resource Name (ARN) of the AWS Identity and Access Management (IAM) role that authorized Amazon Pinpoint to access the Amazon S3 location to import endpoint definitions from.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> The format of the files that were imported to create the segment. Valid values are: CSV, for comma-separated values format; and, JSON, for newline-delimited JSON format.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalId</i> (Deprecated) Your AWS account ID, which you assigned to an external ID key in an IAM trust policy. Amazon Pinpoint previously used this value to assume an IAM role when importing endpoint definitions, but we removed this requirement. We don't recommend use of external IDs for IAM roles that are assumed by Amazon Pinpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ChannelCounts</i> The number of channel types in the endpoint definitions that were imported to create the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Include</i> Specifies how to handle multiple segment groups for the segment. For example, if the segment includes three segment groups, whether the resulting segment includes endpoints that match all, any, or none of the segment groups.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Groups</i> An array that defines the set of segment criteria to evaluate when handling segment groups for the segment.<br/>        """
        pass

    def get_segment_export_jobs(self, SegmentId: str, ApplicationId: str, PageSize: str=None, Token: str=None) -> GetSegmentExportJobsResponse:
        """Retrieves information about the status and settings of the export jobs for a segment.

        :param self:
        :param Token:  The NextToken string that specifies which page of results to return in a paginated response.<br/>SegmentId:  The unique identifier for the segment.<br/>PageSize:  The maximum number of items to include on each page in a paginated response.<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: ExportJobsResponse:  Provides information about all the export jobs that are associated with an application or segment. An export job is a job that exports endpoint definitions to a file.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NextToken</i> The string to use in a subsequent request to get the next page of results in a paginated response. This value is null if there are no additional pages.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Item</i> An array of responses, one for each export job that's associated with the application (Export Jobs resource) or segment (Segment Export Jobs resource).<br/>        """
        pass

    def get_segment_import_jobs(self, SegmentId: str, ApplicationId: str, PageSize: str=None, Token: str=None) -> GetSegmentImportJobsResponse:
        """Retrieves information about the status and settings of the import jobs for a segment.

        :param self:
        :param Token:  The NextToken string that specifies which page of results to return in a paginated response.<br/>SegmentId:  The unique identifier for the segment.<br/>PageSize:  The maximum number of items to include on each page in a paginated response.<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: ImportJobsResponse:  Provides information about the status and settings of all the import jobs that are associated with an application or segment. An import job is a job that imports endpoint definitions from one or more files.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NextToken</i> The string to use in a subsequent request to get the next page of results in a paginated response. This value is null if there are no additional pages.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Item</i> An array of responses, one for each import job that's associated with the application (Import Jobs resource) or segment (Segment Import Jobs resource).<br/>        """
        pass

    def get_segment_version(self, Version: str, SegmentId: str, ApplicationId: str) -> GetSegmentVersionResponse:
        """Retrieves information about the configuration, dimension, and other settings for a specific version of a segment that's associated with an application.

        :param self:
        :param Version:  The unique version number (Version property) for the campaign version.<br/>SegmentId:  The unique identifier for the segment.<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: SegmentResponse:  Provides information about the configuration, dimension, and other settings for a segment.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version number of the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> A string-to-string map of key-value pairs that identifies the tags that are associated with the segment. Each tag consists of a required tag key and an associated tag value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentType</i> The segment type. Valid values are: <ul><li>DIMENSIONAL - A dynamic segment, which is a segment that uses selection criteria that you specify and is based on endpoint data that's reported by your app. Dynamic segments can change over time.</li> <li>IMPORT - A static segment, which is a segment that uses selection criteria that you specify and is based on endpoint definitions that you import from a file. Imported segments are static; they don't change over time.</li></ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentGroups</i> A list of one or more segment groups that apply to the segment. Each segment group consists of zero or more base segments and the dimensions that are applied to those base segments.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SegmentGroups</b>: A list of one or more segment groups that apply to the segment. Each segment group consists of zero or more base segments and the dimensions that are applied to those base segments.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time when the segment was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImportDefinition</i> The settings for the import job that's associated with the segment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ImportDefinition</b>: The settings for the import job that's associated with the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier for the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Dimensions</i> The dimension settings for the segment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Dimensions</b>: The dimension settings for the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time when the segment was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the segment is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Attributes</i> One or more custom attributes to use as criteria for the segment.<br/>The behavior-based criteria, such as how recently users have used your app, for the segment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Behavior</b>: The behavior-based criteria, such as how recently users have used your app, for the segment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Size</i> The number of endpoint definitions that were imported successfully to create the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Url</i> The URL of the Amazon Simple Storage Service (Amazon S3) bucket that the endpoint definitions were imported from to create the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The Amazon Resource Name (ARN) of the AWS Identity and Access Management (IAM) role that authorized Amazon Pinpoint to access the Amazon S3 location to import endpoint definitions from.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> The format of the files that were imported to create the segment. Valid values are: CSV, for comma-separated values format; and, JSON, for newline-delimited JSON format.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalId</i> (Deprecated) Your AWS account ID, which you assigned to an external ID key in an IAM trust policy. Amazon Pinpoint previously used this value to assume an IAM role when importing endpoint definitions, but we removed this requirement. We don't recommend use of external IDs for IAM roles that are assumed by Amazon Pinpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ChannelCounts</i> The number of channel types in the endpoint definitions that were imported to create the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Include</i> Specifies how to handle multiple segment groups for the segment. For example, if the segment includes three segment groups, whether the resulting segment includes endpoints that match all, any, or none of the segment groups.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Groups</i> An array that defines the set of segment criteria to evaluate when handling segment groups for the segment.<br/>        """
        pass

    def get_segment_versions(self, SegmentId: str, ApplicationId: str, PageSize: str=None, Token: str=None) -> GetSegmentVersionsResponse:
        """Retrieves information about the configuration, dimension, and other settings for all versions of a specific segment that's associated with an application.

        :param self:
        :param Token:  The NextToken string that specifies which page of results to return in a paginated response.<br/>SegmentId:  The unique identifier for the segment.<br/>PageSize:  The maximum number of items to include on each page in a paginated response.<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: SegmentsResponse:  Provides information about all the segments that are associated with an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NextToken</i> The string to use in a subsequent request to get the next page of results in a paginated response. This value is null if there are no additional pages.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Item</i> An array of responses, one for each segment that's associated with the application (Segments resource) or each version of a segment that's associated with the application (Segment Versions resource).<br/>        """
        pass

    def get_segments(self, ApplicationId: str, PageSize: str=None, Token: str=None) -> GetSegmentsResponse:
        """Retrieves information about the configuration, dimension, and other settings for all the segments that are associated with an application.

        :param self:
        :param Token:  The NextToken string that specifies which page of results to return in a paginated response.<br/>PageSize:  The maximum number of items to include on each page in a paginated response.<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: SegmentsResponse:  Provides information about all the segments that are associated with an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NextToken</i> The string to use in a subsequent request to get the next page of results in a paginated response. This value is null if there are no additional pages.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Item</i> An array of responses, one for each segment that's associated with the application (Segments resource) or each version of a segment that's associated with the application (Segment Versions resource).<br/>        """
        pass

    def get_sms_channel(self, ApplicationId: str) -> GetSmsChannelResponse:
        """Retrieves information about the status and settings of the SMS channel for an application.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: SMSChannelResponse:  Provides information about the status and settings of the SMS channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the SMS channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransactionalMessagesPerSecond</i> The maximum number of transactional messages that you can send through the SMS channel each second.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ShortCode</i> The registered short code to use when you send messages through the SMS channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SenderId</i> The identity that displays on recipients' devices when they receive messages from the SMS channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PromotionalMessagesPerSecond</i> The maximum number of promotional messages that you can send through the SMS channel each second.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The type of messaging or notification platform for the channel. For the SMS channel, this value is SMS.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time, in ISO 8601 format, when the SMS channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the SMS channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the SMS channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the SMS channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the SMS channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time, in ISO 8601 format, when the SMS channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the SMS channel applies to.<br/>        """
        pass

    def get_user_endpoints(self, UserId: str, ApplicationId: str) -> GetUserEndpointsResponse:
        """Retrieves information about all the endpoints that are associated with a specific user ID.

        :param self:
        :param UserId:  The unique identifier for the user.<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: EndpointsResponse:  Provides information about all the endpoints that are associated with a user ID.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Item</i> An array of responses, one for each endpoint that's associated with the user ID.<br/>        """
        pass

    def get_voice_channel(self, ApplicationId: str) -> GetVoiceChannelResponse:
        """Retrieves information about the status and settings of the voice channel for an application.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>
        :return: VoiceChannelResponse:  Provides information about the status and settings of the voice channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the voice channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The type of messaging or notification platform for the channel. For the voice channel, this value is VOICE.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time, in ISO 8601 format, when the voice channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the voice channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the voice channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the voice channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the voice channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time, in ISO 8601 format, when the voice channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the voice channel applies to.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceArn: str) -> ListTagsForResourceResponse:
        """Retrieves all the tags (keys and values) that are associated with an application, campaign, or segment.

        :param self:
        :param ResourceArn:  The Amazon Resource Name (ARN) of the application, campaign, or segment.<br/>
        :return: TagsModel:  Specifies the tags (keys and values) for an application, campaign, or segment.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> A string-to-string map of key-value pairs that defines the tags for an application, campaign, or segment. A project, campaign, or segment can have a maximum of 50 tags. Each tag consists of a required tag key and an associated tag value. The maximum length of a tag key is 128 characters. The maximum length of a tag value is 256 characters.<br/>        """
        pass

    def phone_number_validate(self, NumberValidateRequest: NumberValidateRequest) -> PhoneNumberValidateResponse:
        """Retrieves information about a phone number.

        :param self:
        :param NumberValidateRequest:  Specifies a phone number to validate and retrieve information about.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneNumber</i> The phone number to retrieve information about. The phone number that you provide should include a valid numeric country code. Otherwise, the operation might result in an error.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsoCountryCode</i> The two-character code, in ISO 3166-1 alpha-2 format, for the country or region where the phone number was originally registered.<br/>
        :return: NumberValidateResponse:  Provides information about a phone number.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ZipCode</i> The postal or ZIP code for the location where the phone number was originally registered.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timezone</i> The time zone for the location where the phone number was originally registered.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneTypeCode</i> The phone type, represented by an integer. Valid values are: 0 (mobile), 1 (landline), 2 (VoIP), 3 (invalid), 4 (other), and 5 (prepaid).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneType</i> The description of the phone type. Valid values are: MOBILE, LANDLINE, VOIP,
                  INVALID, PREPAID, and OTHER.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OriginalPhoneNumber</i> The phone number that was sent in the request body.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OriginalCountryCodeIso2</i> The two-character code, in ISO 3166-1 alpha-2 format, that was sent in the request body.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>County</i> The name of the county where the phone number was originally registered.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CountryCodeNumeric</i> The numeric code for the country or region where the phone number was originally registered.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CountryCodeIso2</i> The two-character code, in ISO 3166-1 alpha-2 format, for the country or region where the phone number was originally registered.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Country</i> The name of the country or region where the phone number was originally registered.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CleansedPhoneNumberNational</i> The cleansed phone number, in the format for the location where the phone number was originally registered.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CleansedPhoneNumberE164</i> The cleansed phone number, in E.164 format, for the location where the phone number was originally registered.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>City</i> The name of the city where the phone number was originally registered.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Carrier</i> The carrier or service provider that the phone number is currently registered with.<br/>        """
        pass

    def put_event_stream(self, WriteEventStream: WriteEventStream, ApplicationId: str) -> PutEventStreamResponse:
        """Creates a new event stream for an application or updates the settings of an existing event stream for an application.

        :param self:
        :param WriteEventStream:  Specifies the Amazon Resource Name (ARN) of an event stream to publish events to and the AWS Identity and Access Management (IAM) role to use when publishing those events.:<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The AWS Identity and Access Management (IAM) role that authorizes Amazon Pinpoint to publish event data to the stream in your AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationStreamArn</i> The Amazon Resource Name (ARN) of the Amazon Kinesis data stream or Amazon Kinesis Data Firehose delivery stream that you want to publish event data to. For a Kinesis data stream, the ARN format is: arn:aws:kinesis:<replaceable>region</replaceable>:<replaceable>account-id</replaceable>:stream/<replaceable>stream_name</replaceable>
                For a Kinesis Data Firehose delivery stream, the ARN format is: arn:aws:firehose:<replaceable>region</replaceable>:<replaceable>account-id</replaceable>:deliverystream/<replaceable>stream_name</replaceable>
               <br/>
        :return: EventStream:  Specifies settings for publishing event data to an Amazon Kinesis data stream or an Amazon Kinesis Data Firehose delivery stream.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The AWS Identity and Access Management (IAM) role that authorizes Amazon Pinpoint to publish event data to the stream in your AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdatedBy</i> The IAM user who last modified the event stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date, in ISO 8601 format, when the event stream was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalId</i> (Deprecated) Your AWS account ID, which you assigned to an external ID key in an IAM trust policy. Amazon Pinpoint previously used this value to assume an IAM role when publishing event data, but we removed this requirement. We don't recommend use of external IDs for IAM roles that are assumed by Amazon Pinpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationStreamArn</i> The Amazon Resource Name (ARN) of the Amazon Kinesis data stream or Amazon Kinesis Data Firehose delivery stream to publish event data to. For a Kinesis data stream, the ARN format is: arn:aws:kinesis:<replaceable>region</replaceable>:<replaceable>account-id</replaceable>:stream/<replaceable>stream_name</replaceable>
                For a Kinesis Data Firehose delivery stream, the ARN format is: arn:aws:firehose:<replaceable>region</replaceable>:<replaceable>account-id</replaceable>:deliverystream/<replaceable>stream_name</replaceable>
               <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application to publish event data for.<br/>        """
        pass

    def put_events(self, EventsRequest: EventsRequest, ApplicationId: str) -> PutEventsResponse:
        """Creates a new event to record for endpoints, or creates or updates endpoint data that existing events are associated with.

        :param self:
        :param EventsRequest:  Specifies a batch of events to process.:<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BatchItem</i> The batch of events to process. For each item in a batch, the endpoint ID acts as a key that has an EventsBatch object as its value.<br/>
        :return: EventsResponse:  Provides information about endpoints and the events that they're associated with.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Results</i> A map that contains a multipart response for each endpoint. For each item in this object, the endpoint ID is the key and the item response is the value. If no item response exists, the value can also be one of the following: 202, the request was processed successfully; or 400, the payload wasn't valid or required fields were missing.<br/>        """
        pass

    def remove_attributes(self, UpdateAttributesRequest: UpdateAttributesRequest, AttributeType: str, ApplicationId: str) -> RemoveAttributesResponse:
        """Removes one or more attributes, of the same attribute type, from all the endpoints that are associated with an application.

        :param self:
        :param UpdateAttributesRequest:  Specifies one or more attributes to remove from all the endpoints that are associated with an application.:<br/>AttributeType:   The type of attribute or attributes to remove. Valid values are: <ul><li>endpoint-custom-attributes - Custom attributes that describe endpoints, such as the date when an associated user opted in or out of receiving communications from you through a specific type of channel.</li> <li>endpoint-custom-metrics - Custom metrics that your app reports to Amazon Pinpoint for endpoints, such as the number of app sessions or the number of items left in a cart.</li> <li>endpoint-user-attributes - Custom attributes that describe users, such as first name, last name, and age.</li></ul><br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Blacklist</i> An array of the attributes to remove from all the endpoints that are associated with the application. The array can specify the complete, exact name of each attribute to remove or it can specify a glob pattern that an attribute name must match in order for the attribute to be removed.<br/>
        :return: AttributesResource:  Provides information about the type and the names of attributes that were removed from all the endpoints that are associated with an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Attributes</i> An array that specifies the names of the attributes that were removed from the endpoints.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttributeType</i> The type of attribute or attributes that were removed from the endpoints. Valid values are: <ul><li>endpoint-custom-attributes - Custom attributes that describe endpoints</li> <li>endpoint-custom-metrics - Custom metrics that your app reports to Amazon Pinpoint for endpoints</li> <li>endpoint-user-attributes - Custom attributes that describe users</li></ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application.<br/>        """
        pass

    def send_messages(self, MessageRequest: MessageRequest, ApplicationId: str) -> SendMessagesResponse:
        """Creates and sends a direct message.

        :param self:
        :param MessageRequest:  Specifies the objects that define configuration and other settings for a message.:<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TraceId</i> The unique identifier for tracing the message. This identifier is visible to message recipients.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MessageConfiguration</i> The set of properties that defines the configuration settings for the message.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>MessageConfiguration</b>: The set of properties that defines the configuration settings for the message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoints</i> A map of key-value pairs, where each key is an endpoint ID and each value is an EndpointSendConfiguration object. You can use an EndpointSendConfiguration object to tailor the message for an endpoint by specifying settings such as content overrides and message variables.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Context</i> A map of custom attributes to attach to the message. For a push notification, this payload is added to the data.pinpoint object. For an email or text message, this payload is added to email/SMS delivery receipt event attributes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Addresses</i> A map of key-value pairs, where each key is an address and each value is an AddressConfiguration object. An address can be a push notification token, a phone number, or an email address. You can use an AddressConfiguration object to tailor the message for an address by specifying settings such as content overrides and message variables.<br/>The default push notification message for the ADM (Amazon Device Messaging) channel. This message overrides the default push notification message (DefaultPushNotificationMessage).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ADMMessage</b>: The default push notification message for the ADM (Amazon Device Messaging) channel. This message overrides the default push notification message (DefaultPushNotificationMessage).
        :return: MessageResponse:  Provides information about the results of a request to send a message to an endpoint address.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Result</i> A map that contains a multipart response for each address (email address, phone number, or push notification token) that the message was sent to. In the map, the address is the key and the result is the value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestId</i> The identifier for the original request that the message was delivered for.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointResult</i> A map that contains a multipart response for each address that the message was sent to. In the map, the endpoint ID is the key and the result is the value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that was used to send the message.<br/>        """
        pass

    def send_users_messages(self, SendUsersMessageRequest: SendUsersMessageRequest, ApplicationId: str) -> SendUsersMessagesResponse:
        """Creates and sends a message to a list of users.

        :param self:
        :param SendUsersMessageRequest:  Specifies the configuration and other settings for a message to send to all the endpoints that are associated with a list of users.:<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Users</i> A map that associates user IDs with EndpointSendConfiguration objects. You can use an EndpointSendConfiguration object to tailor the message for a user by specifying settings such as content overrides and message variables.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TraceId</i> The unique identifier for tracing the message. This identifier is visible to message recipients.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MessageConfiguration</i> The message definitions for the default message and any default messages that you defined for specific channels.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>MessageConfiguration</b>: The message definitions for the default message and any default messages that you defined for specific channels.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Context</i> A map of custom attribute-value pairs. For a push notification, Amazon Pinpoint adds these attributes to the data.pinpoint object in the body of the notification payload. Amazon Pinpoint also provides these attributes in the events that it generates for users-messages deliveries.<br/>The default push notification message for the ADM (Amazon Device Messaging) channel. This message overrides the default push notification message (DefaultPushNotificationMessage).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ADMMessage</b>: The default push notification message for the ADM (Amazon Device Messaging) channel. This message overrides the default push notification message (DefaultPushNotificationMessage).
        :return: SendUsersMessageResponse:  Provides information about which users and endpoints a message was sent to.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Result</i> An object that indicates which endpoints the message was sent to, for each user. The object lists user IDs and, for each user ID, provides the endpoint IDs that the message was sent to. For each endpoint ID, it provides an EndpointMessageResult object.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestId</i> The unique identifier that was assigned to the message request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that was used to send the message.<br/>        """
        pass

    def tag_resource(self, TagsModel: TagsModel, ResourceArn: str):
        """Adds one or more tags (keys and values) to an application, campaign, or segment.

        :param self:
        :param TagsModel:  Specifies the tags (keys and values) for an application, campaign, or segment.:<br/>ResourceArn:  The Amazon Resource Name (ARN) of the application, campaign, or segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> A string-to-string map of key-value pairs that defines the tags for an application, campaign, or segment. A project, campaign, or segment can have a maximum of 50 tags. Each tag consists of a required tag key and an associated tag value. The maximum length of a tag key is 128 characters. The maximum length of a tag value is 256 characters.<br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[__string], ResourceArn: str):
        """Removes one or more tags (keys and values) from an application, campaign, or segment.

        :param self:
        :param TagKeys:  The key of the tag to remove from the application, campaign, or segment. To remove multiple tags, append the tagKeys parameter and argument for each additional tag to remove, separated by an ampersand (&amp;).<br/>ResourceArn:  The Amazon Resource Name (ARN) of the application, campaign, or segment.<br/>
        :return:         """
        pass

    def update_adm_channel(self, ApplicationId: str, ADMChannelRequest: ADMChannelRequest) -> UpdateAdmChannelResponse:
        """Updates the ADM channel settings for an application.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>ADMChannelRequest:  Specifies the status and settings of the ADM (Amazon Device Messaging) channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether to enable the ADM channel for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientSecret</i> The Client Secret that you received from Amazon to send messages by using ADM.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientId</i> The Client ID that you received from Amazon to send messages by using ADM.<br/>
        :return: ADMChannelResponse:  Provides information about the status and settings of the ADM (Amazon Device Messaging) channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the ADM channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The type of messaging or notification platform for the channel. For the ADM channel, this value is ADM.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time when the ADM channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the ADM channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the ADM channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the ADM channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the ADM channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time when the ADM channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the ADM channel applies to.<br/>        """
        pass

    def update_apns_channel(self, ApplicationId: str, APNSChannelRequest: APNSChannelRequest) -> UpdateApnsChannelResponse:
        """Updates the APNs channel settings for an application.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>APNSChannelRequest:  Specifies the status and settings of the APNs (Apple Push Notification service) channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TokenKeyId</i> The key identifier that's assigned to your APNs signing key, if you want Amazon Pinpoint to communicate with APNs by using APNs tokens.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TokenKey</i> The authentication key to use for APNs tokens.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TeamId</i> The identifier that's assigned to your Apple developer account team. This identifier is used for APNs tokens.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrivateKey</i> The private key for the APNs client certificate that you want Amazon Pinpoint to use to communicate with APNs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether to enable the APNs channel for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultAuthenticationMethod</i> The default authentication method that you want Amazon Pinpoint to use when authenticating with APNs, key or certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Certificate</i> The APNs client certificate that you received from Apple, if you want Amazon Pinpoint to communicate with APNs by using an APNs certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BundleId</i> The bundle identifier that's assigned to your iOS app. This identifier is used for APNs tokens.<br/>
        :return: APNSChannelResponse:  Provides information about the status and settings of the APNs (Apple Push Notification service) channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the APNs channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i>  The type of messaging or notification platform for the channel. For the APNs channel, this value is APNS.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time when the APNs channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the APNs channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the APNs channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the APNs channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasTokenKey</i> Specifies whether the APNs channel is configured to communicate with APNs by using APNs tokens. To provide an authentication key for APNs tokens, set the TokenKey property of the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the APNs channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultAuthenticationMethod</i> The default authentication method that Amazon Pinpoint uses to authenticate with APNs for this channel, key or certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time when the APNs channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the APNs channel applies to.<br/>        """
        pass

    def update_apns_sandbox_channel(self, ApplicationId: str, APNSSandboxChannelRequest: APNSSandboxChannelRequest) -> UpdateApnsSandboxChannelResponse:
        """Updates the APNs sandbox channel settings for an application.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>APNSSandboxChannelRequest:  Specifies the status and settings of the APNs (Apple Push Notification service) sandbox channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TokenKeyId</i> The key identifier that's assigned to your APNs signing key, if you want Amazon Pinpoint to communicate with the APNs sandbox environment by using APNs tokens.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TokenKey</i> The authentication key to use for APNs tokens.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TeamId</i> The identifier that's assigned to your Apple developer account team. This identifier is used for APNs tokens.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrivateKey</i> The private key for the APNs client certificate that you want Amazon Pinpoint to use to communicate with the APNs sandbox environment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether to enable the APNs sandbox channel for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultAuthenticationMethod</i> The default authentication method that you want Amazon Pinpoint to use when authenticating with the APNs sandbox environment, key or certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Certificate</i> The APNs client certificate that you received from Apple, if you want Amazon Pinpoint to communicate with the APNs sandbox environment by using an APNs certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BundleId</i> The bundle identifier that's assigned to your iOS app. This identifier is used for APNs tokens.<br/>
        :return: APNSSandboxChannelResponse:  Provides information about the status and settings of the APNs (Apple Push Notification service) sandbox channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the APNs sandbox channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The type of messaging or notification platform for the channel. For the APNs sandbox channel, this value is APNS_SANDBOX.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time when the APNs sandbox channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the APNs sandbox channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the APNs sandbox channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the APNs sandbox channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasTokenKey</i> Specifies whether the APNs sandbox channel is configured to communicate with APNs by using APNs tokens. To provide an authentication key for APNs tokens, set the TokenKey property of the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the APNs sandbox channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultAuthenticationMethod</i> The default authentication method that Amazon Pinpoint uses to authenticate with the APNs sandbox environment for this channel, key or certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time when the APNs sandbox channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the APNs sandbox channel applies to.<br/>        """
        pass

    def update_apns_voip_channel(self, ApplicationId: str, APNSVoipChannelRequest: APNSVoipChannelRequest) -> UpdateApnsVoipChannelResponse:
        """Updates the APNs VoIP channel settings for an application.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>APNSVoipChannelRequest:  Specifies the status and settings of the APNs (Apple Push Notification service) VoIP channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TokenKeyId</i> The key identifier that's assigned to your APNs signing key, if you want Amazon Pinpoint to communicate with APNs by using APNs tokens.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TokenKey</i> The authentication key to use for APNs tokens.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TeamId</i> The identifier that's assigned to your Apple developer account team. This identifier is used for APNs tokens.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrivateKey</i> The private key for the APNs client certificate that you want Amazon Pinpoint to use to communicate with APNs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether to enable the APNs VoIP channel for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultAuthenticationMethod</i> The default authentication method that you want Amazon Pinpoint to use when authenticating with APNs, key or certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Certificate</i> The APNs client certificate that you received from Apple, if you want Amazon Pinpoint to communicate with APNs by using an APNs certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BundleId</i> The bundle identifier that's assigned to your iOS app. This identifier is used for APNs tokens.<br/>
        :return: APNSVoipChannelResponse:  Provides information about the status and settings of the APNs (Apple Push Notification service) VoIP channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the APNs VoIP channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The type of messaging or notification platform for the channel. For the APNs VoIP channel, this value is APNS_VOIP.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time when the APNs VoIP channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the APNs VoIP channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the APNs VoIP channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the APNs VoIP channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasTokenKey</i> Specifies whether the APNs VoIP channel is configured to communicate with APNs by using APNs tokens. To provide an authentication key for APNs tokens, set the TokenKey property of the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the APNs VoIP channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultAuthenticationMethod</i> The default authentication method that Amazon Pinpoint uses to authenticate with APNs for this channel, key or certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time when the APNs VoIP channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the APNs VoIP channel applies to.<br/>        """
        pass

    def update_apns_voip_sandbox_channel(self, ApplicationId: str, APNSVoipSandboxChannelRequest: APNSVoipSandboxChannelRequest) -> UpdateApnsVoipSandboxChannelResponse:
        """Updates the settings for the APNs VoIP sandbox channel for an application.

        :param self:
        :param ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>APNSVoipSandboxChannelRequest:  Specifies the status and settings of the APNs (Apple Push Notification service) VoIP sandbox channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TokenKeyId</i> The key identifier that's assigned to your APNs signing key, if you want Amazon Pinpoint to communicate with the APNs sandbox environment by using APNs tokens.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TokenKey</i> The authentication key to use for APNs tokens.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TeamId</i> The identifier that's assigned to your Apple developer account team. This identifier is used for APNs tokens.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrivateKey</i> The private key for the APNs client certificate that you want Amazon Pinpoint to use to communicate with the APNs sandbox environment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the APNs VoIP sandbox channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultAuthenticationMethod</i> The default authentication method that you want Amazon Pinpoint to use when authenticating with the APNs sandbox environment for this channel, key or certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Certificate</i> The APNs client certificate that you received from Apple, if you want Amazon Pinpoint to communicate with the APNs sandbox environment by using an APNs certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BundleId</i> The bundle identifier that's assigned to your iOS app. This identifier is used for APNs tokens.<br/>
        :return: APNSVoipSandboxChannelResponse:  Provides information about the status and settings of the APNs (Apple Push Notification service) VoIP sandbox channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the APNs VoIP sandbox channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The type of messaging or notification platform for the channel. For the APNs VoIP sandbox channel, this value is APNS_VOIP_SANDBOX.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time when the APNs VoIP sandbox channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the APNs VoIP sandbox channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the APNs VoIP sandbox channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the APNs VoIP sandbox channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasTokenKey</i> Specifies whether the APNs VoIP sandbox channel is configured to communicate with APNs by using APNs tokens. To provide an authentication key for APNs tokens, set the TokenKey property of the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the APNs VoIP sandbox channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultAuthenticationMethod</i> The default authentication method that Amazon Pinpoint uses to authenticate with the APNs sandbox environment for this channel, key or certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time when the APNs VoIP sandbox channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the APNs VoIP sandbox channel applies to.<br/>        """
        pass

    def update_application_settings(self, WriteApplicationSettingsRequest: WriteApplicationSettingsRequest, ApplicationId: str) -> UpdateApplicationSettingsResponse:
        """Updates the settings for an application.

        :param self:
        :param WriteApplicationSettingsRequest:  Specifies the default settings for an application.:<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QuietTime</i> The default quiet time for campaigns in the application. Quiet time is a specific time range when campaigns don't send messages to endpoints, if all the following conditions are met: <ul><li>The EndpointDemographic.Timezone property of the endpoint is set to a valid value.</li> <li>The current time in the endpoint's time zone is later than or equal to the time specified by the QuietTime.Start property for the application (or a campaign that has custom quiet time settings).</li> <li>The current time in the endpoint's time zone is earlier than or equal to the time specified by the QuietTime.End property for the application (or a campaign that has custom quiet time settings).</li></ul> If any of the preceding conditions isn't met, the endpoint will receive messages from a campaign, even if quiet time is enabled. To override the default quiet time settings for a specific campaign, use the <link  linkend="apps-application-id-campaigns-campaign-id">Campaign</link> resource to define a custom quiet time for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>QuietTime</b>: The default quiet time for campaigns in the application. Quiet time is a specific time range when campaigns don't send messages to endpoints, if all the following conditions are met: <ul><li>The EndpointDemographic.Timezone property of the endpoint is set to a valid value.</li> <li>The current time in the endpoint's time zone is later than or equal to the time specified by the QuietTime.Start property for the application (or a campaign that has custom quiet time settings).</li> <li>The current time in the endpoint's time zone is earlier than or equal to the time specified by the QuietTime.End property for the application (or a campaign that has custom quiet time settings).</li></ul> If any of the preceding conditions isn't met, the endpoint will receive messages from a campaign, even if quiet time is enabled. To override the default quiet time settings for a specific campaign, use the <link  linkend="apps-application-id-campaigns-campaign-id">Campaign</link> resource to define a custom quiet time for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Limits</i> The default sending limits for campaigns in the application. To override these limits for a specific campaign, use the <link  linkend="apps-application-id-campaigns-campaign-id">Campaign</link> resource to define custom limits for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Limits</b>: The default sending limits for campaigns in the application. To override these limits for a specific campaign, use the <link  linkend="apps-application-id-campaigns-campaign-id">Campaign</link> resource to define custom limits for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchMetricsEnabled</i> Specifies whether to enable application-related alarms in Amazon CloudWatch.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CampaignHook</i> The settings for the AWS Lambda function to use by default as a code hook for campaigns in the application. To override these settings for a specific campaign, use the <link  linkend="apps-application-id-campaigns-campaign-id">Campaign</link> resource to define custom Lambda function settings for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CampaignHook</b>: The settings for the AWS Lambda function to use by default as a code hook for campaigns in the application. To override these settings for a specific campaign, use the <link  linkend="apps-application-id-campaigns-campaign-id">Campaign</link> resource to define custom Lambda function settings for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WebUrl</i>  The web URL that Amazon Pinpoint calls to invoke the AWS Lambda function over HTTPS.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> Specifies which Lambda mode to use when invoking the AWS Lambda function.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LambdaFunctionName</i> The name or Amazon Resource Name (ARN) of the AWS Lambda function that Amazon Pinpoint invokes to send messages for a campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Total</i> The maximum number of messages that a campaign can send to a single endpoint during the course of the campaign. The maximum value is 100.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MessagesPerSecond</i> The maximum number of messages that a campaign can send each second. The minimum value is 50. The maximum value is 20,000.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumDuration</i> The maximum amount of time, in seconds, that a campaign can attempt to deliver a message after the scheduled start time for the campaign. The minimum value is 60 seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Daily</i> The maximum number of messages that a campaign can send to a single endpoint during a 24-hour period. The maximum value is 100.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Start</i> The specific time when quiet time begins. This value has to use 24-hour notation and be in HH:MM format, where HH is the hour (with a leading zero, if applicable) and MM is the minutes. For example, use 02:30 to represent 2:30 AM, or 14:30 to represent 2:30 PM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>End</i> The specific time when quiet time ends. This value has to use 24-hour notation and be in HH:MM format, where HH is the hour (with a leading zero, if applicable) and MM is the minutes. For example, use 02:30 to represent 2:30 AM, or 14:30 to represent 2:30 PM.<br/>
        :return: ApplicationSettingsResource:  Provides information about an application, including the default settings for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QuietTime</i> The default quiet time for campaigns in the application. Quiet time is a specific time range when campaigns don't send messages to endpoints, if all the following conditions are met: <ul><li>The EndpointDemographic.Timezone property of the endpoint is set to a valid value.</li> <li>The current time in the endpoint's time zone is later than or equal to the time specified by the QuietTime.Start property for the application (or a campaign that has custom quiet time settings).</li> <li>The current time in the endpoint's time zone is earlier than or equal to the time specified by the QuietTime.End property for the application (or a campaign that has custom quiet time settings).</li></ul> If any of the preceding conditions isn't met, the endpoint will receive messages from a campaign, even if quiet time is enabled.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>QuietTime</b>: The default quiet time for campaigns in the application. Quiet time is a specific time range when campaigns don't send messages to endpoints, if all the following conditions are met: <ul><li>The EndpointDemographic.Timezone property of the endpoint is set to a valid value.</li> <li>The current time in the endpoint's time zone is later than or equal to the time specified by the QuietTime.Start property for the application (or a campaign that has custom quiet time settings).</li> <li>The current time in the endpoint's time zone is earlier than or equal to the time specified by the QuietTime.End property for the application (or a campaign that has custom quiet time settings).</li></ul> If any of the preceding conditions isn't met, the endpoint will receive messages from a campaign, even if quiet time is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Limits</i> The default sending limits for campaigns in the application.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Limits</b>: The default sending limits for campaigns in the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time, in ISO 8601 format, when the application's settings were last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CampaignHook</i> The settings for the AWS Lambda function to use by default as a code hook for campaigns in the application.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CampaignHook</b>: The settings for the AWS Lambda function to use by default as a code hook for campaigns in the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WebUrl</i>  The web URL that Amazon Pinpoint calls to invoke the AWS Lambda function over HTTPS.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> Specifies which Lambda mode to use when invoking the AWS Lambda function.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LambdaFunctionName</i> The name or Amazon Resource Name (ARN) of the AWS Lambda function that Amazon Pinpoint invokes to send messages for a campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Total</i> The maximum number of messages that a campaign can send to a single endpoint during the course of the campaign. The maximum value is 100.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MessagesPerSecond</i> The maximum number of messages that a campaign can send each second. The minimum value is 50. The maximum value is 20,000.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumDuration</i> The maximum amount of time, in seconds, that a campaign can attempt to deliver a message after the scheduled start time for the campaign. The minimum value is 60 seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Daily</i> The maximum number of messages that a campaign can send to a single endpoint during a 24-hour period. The maximum value is 100.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Start</i> The specific time when quiet time begins. This value has to use 24-hour notation and be in HH:MM format, where HH is the hour (with a leading zero, if applicable) and MM is the minutes. For example, use 02:30 to represent 2:30 AM, or 14:30 to represent 2:30 PM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>End</i> The specific time when quiet time ends. This value has to use 24-hour notation and be in HH:MM format, where HH is the hour (with a leading zero, if applicable) and MM is the minutes. For example, use 02:30 to represent 2:30 AM, or 14:30 to represent 2:30 PM.<br/>        """
        pass

    def update_baidu_channel(self, BaiduChannelRequest: BaiduChannelRequest, ApplicationId: str) -> UpdateBaiduChannelResponse:
        """Updates the settings of the Baidu channel for an application.

        :param self:
        :param BaiduChannelRequest:  Specifies the status and settings of the Baidu (Baidu Cloud Push) channel for an application.:<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecretKey</i> The secret key that you received from the Baidu Cloud Push service to communicate with the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether to enable the Baidu channel for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApiKey</i> The API key that you received from the Baidu Cloud Push service to communicate with the service.<br/>
        :return: BaiduChannelResponse:  Provides information about the status and settings of the Baidu (Baidu Cloud Push) channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the Baidu channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The type of messaging or notification platform for the channel. For the Baidu channel, this value is BAIDU.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time when the Baidu channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the Baidu channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the Baidu channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the Baidu channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the Baidu channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Credential</i> The API key that you received from the Baidu Cloud Push service to communicate with the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time when the Baidu channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the Baidu channel applies to.<br/>        """
        pass

    def update_campaign(self, WriteCampaignRequest: WriteCampaignRequest, CampaignId: str, ApplicationId: str) -> UpdateCampaignResponse:
        """Updates the settings for a campaign.

        :param self:
        :param WriteCampaignRequest:  Specifies the configuration and other settings for a campaign.:<br/>CampaignId:  The unique identifier for the campaign.<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TreatmentName</i> The custom name of a variation of the campaign to use for A/B testing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TreatmentDescription</i> The custom description of a variation of the campaign to use for A/B testing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> A string-to-string map of key-value pairs that defines the tags to associate with the campaign. Each tag consists of a required tag key and an associated tag value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentVersion</i> The version of the segment to associate with the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentId</i> The unique identifier for the segment to associate with the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Schedule</i> The schedule settings for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Schedule</b>: The schedule settings for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The custom name of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MessageConfiguration</i> The message configuration settings for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>MessageConfiguration</b>: The message configuration settings for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Limits</i> The messaging limits for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Limits</b>: The messaging limits for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsPaused</i> Specifies whether to pause the campaign. A paused campaign doesn't run unless you resume it by setting this value to false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Hook</i> The settings for the AWS Lambda function to use as a code hook for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Hook</b>: The settings for the AWS Lambda function to use as a code hook for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HoldoutPercent</i> The allocated percentage of users (segment members) who shouldn't receive messages from the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The custom description of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdditionalTreatments</i> An array of requests that defines additional treatments for the campaign, in addition to the default treatment for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WebUrl</i>  The web URL that Amazon Pinpoint calls to invoke the AWS Lambda function over HTTPS.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> Specifies which Lambda mode to use when invoking the AWS Lambda function.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LambdaFunctionName</i> The name or Amazon Resource Name (ARN) of the AWS Lambda function that Amazon Pinpoint invokes to send messages for a campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Total</i> The maximum number of messages that a campaign can send to a single endpoint during the course of the campaign. The maximum value is 100.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MessagesPerSecond</i> The maximum number of messages that a campaign can send each second. The minimum value is 50. The maximum value is 20,000.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumDuration</i> The maximum amount of time, in seconds, that a campaign can attempt to deliver a message after the scheduled start time for the campaign. The minimum value is 60 seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Daily</i> The maximum number of messages that a campaign can send to a single endpoint during a 24-hour period. The maximum value is 100.<br/>The message that the campaign sends through the ADM (Amazon Device Messaging) channel. This message overrides the default message.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ADMMessage</b>: The message that the campaign sends through the ADM (Amazon Device Messaging) channel. This message overrides the default message.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> The scheduled time, in ISO 8601 format, for the campaign to end.<br/>The type of event that causes the campaign to be sent, if the value of the Frequency property is EVENT.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EventFilter</b>: The type of event that causes the campaign to be sent, if the value of the Frequency property is EVENT.
        :return: CampaignResponse:  Provides information about the status, configuration, and other settings for a campaign.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version number of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TreatmentName</i> The custom name of a variation of the campaign that's used for A/B testing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TreatmentDescription</i> The custom description of a variation of the campaign that's used for A/B testing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> A string-to-string map of key-value pairs that identifies the tags that are associated with the campaign. Each tag consists of a required tag key and an associated tag value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The current status of the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>State</b>: The current status of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentVersion</i> The version number of the segment that's associated with the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentId</i> The unique identifier for the segment that's associated with the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Schedule</i> The schedule settings for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Schedule</b>: The schedule settings for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MessageConfiguration</i> The message configuration settings for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>MessageConfiguration</b>: The message configuration settings for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Limits</i> The messaging limits for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Limits</b>: The messaging limits for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date, in ISO 8601 format, when the campaign was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsPaused</i> Specifies whether the campaign is paused. A paused campaign doesn't run unless you resume it by changing this value to false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Hook</i> The settings for the AWS Lambda function to use as a code hook for the campaign.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Hook</b>: The settings for the AWS Lambda function to use as a code hook for the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HoldoutPercent</i> The allocated percentage of users (segment members) who shouldn't receive messages from the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The custom description of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultState</i> The current status of the campaign's default treatment. This value exists only for campaigns that have more than one treatment, to support A/B testing.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DefaultState</b>: The current status of the campaign's default treatment. This value exists only for campaigns that have more than one treatment, to support A/B testing.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date, ISO 8601 format, when the campaign was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the campaign applies to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdditionalTreatments</i> An array of responses, one for each treatment that you defined for the campaign, in addition to the default treatment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CampaignStatus</i> The status of the campaign, or the status of a treatment that belongs to an A/B test campaign. If a campaign uses A/B testing, the campaign has a status of COMPLETED only when all campaign treatments have a status of COMPLETED.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>WebUrl</i>  The web URL that Amazon Pinpoint calls to invoke the AWS Lambda function over HTTPS.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> Specifies which Lambda mode to use when invoking the AWS Lambda function.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LambdaFunctionName</i> The name or Amazon Resource Name (ARN) of the AWS Lambda function that Amazon Pinpoint invokes to send messages for a campaign.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Total</i> The maximum number of messages that a campaign can send to a single endpoint during the course of the campaign. The maximum value is 100.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MessagesPerSecond</i> The maximum number of messages that a campaign can send each second. The minimum value is 50. The maximum value is 20,000.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaximumDuration</i> The maximum amount of time, in seconds, that a campaign can attempt to deliver a message after the scheduled start time for the campaign. The minimum value is 60 seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Daily</i> The maximum number of messages that a campaign can send to a single endpoint during a 24-hour period. The maximum value is 100.<br/>The message that the campaign sends through the ADM (Amazon Device Messaging) channel. This message overrides the default message.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ADMMessage</b>: The message that the campaign sends through the ADM (Amazon Device Messaging) channel. This message overrides the default message.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> The scheduled time, in ISO 8601 format, for the campaign to end.<br/>The type of event that causes the campaign to be sent, if the value of the Frequency property is EVENT.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EventFilter</b>: The type of event that causes the campaign to be sent, if the value of the Frequency property is EVENT.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CampaignStatus</i> The status of the campaign, or the status of a treatment that belongs to an A/B test campaign. If a campaign uses A/B testing, the campaign has a status of COMPLETED only when all campaign treatments have a status of COMPLETED.<br/>        """
        pass

    def update_email_channel(self, EmailChannelRequest: EmailChannelRequest, ApplicationId: str) -> UpdateEmailChannelResponse:
        """Updates the status and settings of the email channel for an application.

        :param self:
        :param EmailChannelRequest:  Specifies the status and settings of the email channel for an application.:<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i>  The ARN of the AWS Identity and Access Management (IAM) role that you want Amazon Pinpoint to use when it submits email-related event data for the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Identity</i>  The Amazon Resource Name (ARN) of the identity, verified with Amazon Simple Email Service (Amazon SES), that you want to use when you send email through the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FromAddress</i>  The verified email address that you want to send email from when you send email through the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether to enable the email channel for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigurationSet</i> The configuration set that you want to apply to email that you send through the channel by using the <a href="emailAPIreference.html">Amazon Pinpoint Email API</a>.<br/>
        :return: EmailChannelResponse:  Provides information about the status and settings of the email channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the email channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i>  The ARN of the AWS Identity and Access Management (IAM) role that Amazon Pinpoint uses to submit email-related event data for the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The type of messaging or notification platform for the channel. For the email channel, this value is EMAIL.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MessagesPerSecond</i> The maximum number of emails that you can send through the channel each second.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time, in ISO 8601 format, when the email channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the email channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the email channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Identity</i>  The Amazon Resource Name (ARN) of the identity, verified with Amazon Simple Email Service (Amazon SES), that you use when you send email through the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the email channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FromAddress</i> The verified email address that you send email from when you send email through the channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the email channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time, in ISO 8601 format, when the email channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfigurationSet</i> The configuration set that's applied to email that's sent through the channel by using the <a href="emailAPIreference.html">Amazon Pinpoint Email API</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the email channel applies to.<br/>        """
        pass

    def update_endpoint(self, EndpointRequest: EndpointRequest, EndpointId: str, ApplicationId: str) -> UpdateEndpointResponse:
        """Creates a new endpoint for an application or updates the settings and attributes of an existing endpoint for an application. You can also use this operation to define custom attributes (Attributes, Metrics, and UserAttributes properties) for an endpoint.

        :param self:
        :param EndpointRequest:  Specifies the channel type and other settings for an endpoint.:<br/>EndpointId:  The unique identifier for the endpoint.<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>User</i> One or more custom user attributes that describe the user who's associated with the endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>User</b>: One or more custom user attributes that describe the user who's associated with the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestId</i> The unique identifier for the most recent request to update the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptOut</i> Specifies whether the user who's associated with the endpoint has opted out of receiving messages and push notifications from you. Possible values are: ALL, the user has opted out and doesn't want to receive any messages or push notifications; and, NONE, the user hasn't opted out and wants to receive all messages and push notifications.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Metrics</i> One or more custom metrics that your app reports to Amazon Pinpoint for the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Location</i> The geographic information for the endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Location</b>: The geographic information for the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointStatus</i> Not used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EffectiveDate</i> The date and time, in ISO 8601 format, when the endpoint is updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Demographic</i> The demographic information for the endpoint, such as the time zone and platform.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Demographic</b>: The demographic information for the endpoint, such as the time zone and platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ChannelType</i> The channel to use when sending messages or push notifications to the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Attributes</i> One or more custom attributes that describe the endpoint by associating a name with an array of values. For example, the value of a custom attribute named Interests might be: ["science", "music", "travel"]. You can use these attributes as filter criteria when you create segments. When you define the name of a custom attribute, avoid using the following characters: number sign (#), colon (:), question mark (?), backslash (\), and slash (/). The Amazon Pinpoint console can't display attribute names that contain these characters. This limitation doesn't apply to attribute values.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The destination address for messages or push notifications that you send to the endpoint. The address varies by channel. For a push-notification channel, use the token provided by the push notification service, such as an Apple Push Notification service (APNs) device token or a Firebase Cloud Messaging (FCM) registration token. For the SMS channel, use a phone number in E.164 format, such as +12065550100. For the email channel, use an email address.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Timezone</i> The time zone of the endpoint, specified as a tz database name value, such as America/Los_Angeles.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PlatformVersion</i> The platform version of the endpoint device.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The platform of the endpoint device, such as iOS or Android.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ModelVersion</i> The model version of the endpoint device.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Model</i> The model name or number of the endpoint device, such as iPhone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Make</i> The manufacturer of the endpoint device, such as Apple or Samsung.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Locale</i> The locale of the endpoint, in the following format: the ISO 639-1 alpha-2 code, followed by an underscore (_), followed by an ISO 3166-1 alpha-2 value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AppVersion</i> The version of the app that's associated with the endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Region</i> The name of the region where the endpoint is located. For locations in the United States, this value is the name of a state.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PostalCode</i> The postal or ZIP code for the area where the endpoint is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Longitude</i> The longitude coordinate of the endpoint location, rounded to one decimal place.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Latitude</i> The latitude coordinate of the endpoint location, rounded to one decimal place.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Country</i> The two-character code, in ISO 3166-1 alpha-2 format, for the country or region where the endpoint is located. For example, US for the United States.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>City</i> The name of the city where the endpoint is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UserId</i> The unique identifier for the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UserAttributes</i> One or more custom attributes that describe the user by associating a name with an array of values. For example, the value of an attribute named Interests might be: ["science", "music", "travel"]. You can use these attributes as filter criteria when you create segments. When you define the name of a custom attribute, avoid using the following characters: number sign (#), colon (:), question mark (?), backslash (\), and slash (/). The Amazon Pinpoint console can't display attribute names that contain these characters. This limitation doesn't apply to attribute values.<br/>
        :return: MessageBody:  Provides information about an API request or response.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestID</i> The unique identifier for the request or response.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> The message that's returned from the API.<br/>        """
        pass

    def update_endpoints_batch(self, EndpointBatchRequest: EndpointBatchRequest, ApplicationId: str) -> UpdateEndpointsBatchResponse:
        """ Creates a new batch of endpoints for an application or updates the settings and attributes of a batch of existing endpoints for an application. You can also use this operation to define custom attributes (Attributes, Metrics, and UserAttributes properties) for a batch of endpoints.

        :param self:
        :param EndpointBatchRequest:  Specifies a batch of endpoints to create or update and the settings and attributes to set or change for each endpoint.:<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Item</i> An array that defines the endpoints to create or update and, for each endpoint, the property values to set or change. An array can contain a maximum of 100 items.<br/>
        :return: MessageBody:  Provides information about an API request or response.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestID</i> The unique identifier for the request or response.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> The message that's returned from the API.<br/>        """
        pass

    def update_gcm_channel(self, GCMChannelRequest: GCMChannelRequest, ApplicationId: str) -> UpdateGcmChannelResponse:
        """Updates the status and settings of the GCM channel for an application.

        :param self:
        :param GCMChannelRequest:  Specifies the status and settings of the GCM channel for an application. This channel enables Amazon Pinpoint to send push notifications through the Firebase Cloud Messaging (FCM), formerly Google Cloud Messaging (GCM), service.:<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether to enable the GCM channel for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApiKey</i> The API key, also referred to as a <i>server key</i>, that you received from Google to communicate with Google services.<br/>
        :return: GCMChannelResponse:  Provides information about the status and settings of the GCM channel for an application. The GCM channel enables Amazon Pinpoint to send push notifications through the Firebase Cloud Messaging (FCM), formerly Google Cloud Messaging (GCM), service.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the GCM channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The type of messaging or notification platform for the channel. For the GCM channel, this value is GCM.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time when the GCM channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the GCM channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the GCM channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the GCM channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the GCM channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Credential</i> The API key, also referred to as a <i>server key</i>, that you received from Google to communicate with Google services.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time when the GCM channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the GCM channel applies to.<br/>        """
        pass

    def update_segment(self, WriteSegmentRequest: WriteSegmentRequest, SegmentId: str, ApplicationId: str) -> UpdateSegmentResponse:
        """Creates a new segment for an application or updates the configuration, dimension, and other settings for an existing segment that's associated with an application.

        :param self:
        :param WriteSegmentRequest:  Specifies the configuration, dimension, and other settings for a segment. A WriteSegmentRequest object can include a Dimensions object or a SegmentGroups object, but not both.:<br/>SegmentId:  The unique identifier for the segment.<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> A string-to-string map of key-value pairs that defines the tags to associate with the segment. Each tag consists of a required tag key and an associated tag value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentGroups</i> The segment group to use and the dimensions to apply to the group's base segments in order to build the segment. A segment group can consist of zero or more base segments. Your request can include only one segment group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SegmentGroups</b>: The segment group to use and the dimensions to apply to the group's base segments in order to build the segment. A segment group can consist of zero or more base segments. Your request can include only one segment group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Dimensions</i> The criteria that define the dimensions for the segment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Dimensions</b>: The criteria that define the dimensions for the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Attributes</i> One or more custom attributes to use as criteria for the segment.<br/>The behavior-based criteria, such as how recently users have used your app, for the segment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Behavior</b>: The behavior-based criteria, such as how recently users have used your app, for the segment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Include</i> Specifies how to handle multiple segment groups for the segment. For example, if the segment includes three segment groups, whether the resulting segment includes endpoints that match all, any, or none of the segment groups.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Groups</i> An array that defines the set of segment criteria to evaluate when handling segment groups for the segment.<br/>
        :return: SegmentResponse:  Provides information about the configuration, dimension, and other settings for a segment.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version number of the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> A string-to-string map of key-value pairs that identifies the tags that are associated with the segment. Each tag consists of a required tag key and an associated tag value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentType</i> The segment type. Valid values are: <ul><li>DIMENSIONAL - A dynamic segment, which is a segment that uses selection criteria that you specify and is based on endpoint data that's reported by your app. Dynamic segments can change over time.</li> <li>IMPORT - A static segment, which is a segment that uses selection criteria that you specify and is based on endpoint definitions that you import from a file. Imported segments are static; they don't change over time.</li></ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SegmentGroups</i> A list of one or more segment groups that apply to the segment. Each segment group consists of zero or more base segments and the dimensions that are applied to those base segments.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SegmentGroups</b>: A list of one or more segment groups that apply to the segment. Each segment group consists of zero or more base segments and the dimensions that are applied to those base segments.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time when the segment was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImportDefinition</i> The settings for the import job that's associated with the segment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ImportDefinition</b>: The settings for the import job that's associated with the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier for the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Dimensions</i> The dimension settings for the segment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Dimensions</b>: The dimension settings for the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time when the segment was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) of the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the segment is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Attributes</i> One or more custom attributes to use as criteria for the segment.<br/>The behavior-based criteria, such as how recently users have used your app, for the segment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Behavior</b>: The behavior-based criteria, such as how recently users have used your app, for the segment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Size</i> The number of endpoint definitions that were imported successfully to create the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Url</i> The URL of the Amazon Simple Storage Service (Amazon S3) bucket that the endpoint definitions were imported from to create the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The Amazon Resource Name (ARN) of the AWS Identity and Access Management (IAM) role that authorized Amazon Pinpoint to access the Amazon S3 location to import endpoint definitions from.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> The format of the files that were imported to create the segment. Valid values are: CSV, for comma-separated values format; and, JSON, for newline-delimited JSON format.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExternalId</i> (Deprecated) Your AWS account ID, which you assigned to an external ID key in an IAM trust policy. Amazon Pinpoint previously used this value to assume an IAM role when importing endpoint definitions, but we removed this requirement. We don't recommend use of external IDs for IAM roles that are assumed by Amazon Pinpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ChannelCounts</i> The number of channel types in the endpoint definitions that were imported to create the segment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Include</i> Specifies how to handle multiple segment groups for the segment. For example, if the segment includes three segment groups, whether the resulting segment includes endpoints that match all, any, or none of the segment groups.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Groups</i> An array that defines the set of segment criteria to evaluate when handling segment groups for the segment.<br/>        """
        pass

    def update_sms_channel(self, SMSChannelRequest: SMSChannelRequest, ApplicationId: str) -> UpdateSmsChannelResponse:
        """Updates the status and settings of the SMS channel for an application.

        :param self:
        :param SMSChannelRequest:  Specifies the status and settings of the SMS channel for an application.:<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ShortCode</i> The registered short code that you want to use when you send messages through the SMS channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SenderId</i> The identity that you want to display on recipients' devices when they receive messages from the SMS channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether to enable the SMS channel for the application.<br/>
        :return: SMSChannelResponse:  Provides information about the status and settings of the SMS channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the SMS channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TransactionalMessagesPerSecond</i> The maximum number of transactional messages that you can send through the SMS channel each second.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ShortCode</i> The registered short code to use when you send messages through the SMS channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SenderId</i> The identity that displays on recipients' devices when they receive messages from the SMS channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PromotionalMessagesPerSecond</i> The maximum number of promotional messages that you can send through the SMS channel each second.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The type of messaging or notification platform for the channel. For the SMS channel, this value is SMS.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time, in ISO 8601 format, when the SMS channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the SMS channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the SMS channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the SMS channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the SMS channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time, in ISO 8601 format, when the SMS channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the SMS channel applies to.<br/>        """
        pass

    def update_voice_channel(self, VoiceChannelRequest: VoiceChannelRequest, ApplicationId: str) -> UpdateVoiceChannelResponse:
        """Updates the status and settings of the voice channel for an application.

        :param self:
        :param VoiceChannelRequest:  Specifies the status and settings of the voice channel for an application.:<br/>ApplicationId:  The unique identifier for the application. This identifier is displayed as the <b>Project ID</b> on the Amazon Pinpoint console.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether to enable the voice channel for the application.<br/>
        :return: VoiceChannelResponse:  Provides information about the status and settings of the voice channel for an application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The current version of the voice channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Platform</i> The type of messaging or notification platform for the channel. For the voice channel, this value is VOICE.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The date and time, in ISO 8601 format, when the voice channel was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The user who last modified the voice channel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsArchived</i> Specifies whether the voice channel is archived.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> (Deprecated) An identifier for the voice channel. This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasCredential</i> (Not used) This property is retained only for backward compatibility.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether the voice channel is enabled for the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The date and time, in ISO 8601 format, when the voice channel was enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The unique identifier for the application that the voice channel applies to.<br/>        """
        pass



class PinpointEmail(BaseClient):
    """<fullname>Amazon Pinpoint Email Service</fullname> This document contains reference information for the <a href="https://aws.amazon.com/pinpoint">Amazon Pinpoint</a> Email API, version 1.0. This document is best used in conjunction with the <a href="https://docs.aws.amazon.com/pinpoint/latest/developerguide/welcome.html">Amazon Pinpoint Developer Guide</a>. The Amazon Pinpoint Email API is available in several AWS Regions and it provides an endpoint for each of these Regions. For a list of all the Regions and endpoints where the API is currently available, see <a href="https://docs.aws.amazon.com/general/latest/gr/rande.html#pinpoint_region">AWS Regions and Endpoints</a> in the <i>Amazon Web Services General Reference</i>. In each Region, AWS maintains multiple Availability Zones. These Availability Zones are physically isolated from each other, but are united by private, low-latency, high-throughput, and highly redundant network connections. These Availability Zones enable us to provide very high levels of availability and redundancy, while also minimizing latency. To learn more about the number of Availability Zones that are available in each Region, see <a href="http://aws.amazon.com/about-aws/global-infrastructure/">AWS Global Infrastructure</a>."""


    class GetDedicatedIpsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_dedicated_ips` for valid parameters"""
            pass
    class ListConfigurationSetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_configuration_sets` for valid parameters"""
            pass
    class ListDedicatedIpPoolsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_dedicated_ip_pools` for valid parameters"""
            pass
    class ListDeliverabilityTestReportsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_deliverability_test_reports` for valid parameters"""
            pass
    class ListEmailIdentitiesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_email_identities` for valid parameters"""
            pass


    class CreateConfigurationSetRequest(Mapping):
        pass
    
    class CreateConfigurationSetResponse(Mapping):
        pass
    
    class TrackingOptions(Mapping):
        pass
    
    class DeliveryOptions(Mapping):
        pass
    
    class ReputationOptions(Mapping):
        pass
    
    class SendingOptions(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreateConfigurationSetEventDestinationRequest(Mapping):
        pass
    
    class CreateConfigurationSetEventDestinationResponse(Mapping):
        pass
    
    class EventDestinationDefinition(Mapping):
        pass
    
    class CreateDedicatedIpPoolRequest(Mapping):
        pass
    
    class CreateDedicatedIpPoolResponse(Mapping):
        pass
    
    class CreateDeliverabilityTestReportRequest(Mapping):
        pass
    
    class CreateDeliverabilityTestReportResponse(Mapping):
        pass
    
    class EmailContent(Mapping):
        pass
    
    class CreateEmailIdentityRequest(Mapping):
        pass
    
    class CreateEmailIdentityResponse(Mapping):
        pass
    
    class DeleteConfigurationSetRequest(Mapping):
        pass
    
    class DeleteConfigurationSetResponse(Mapping):
        pass
    
    class DeleteConfigurationSetEventDestinationRequest(Mapping):
        pass
    
    class DeleteConfigurationSetEventDestinationResponse(Mapping):
        pass
    
    class DeleteDedicatedIpPoolRequest(Mapping):
        pass
    
    class DeleteDedicatedIpPoolResponse(Mapping):
        pass
    
    class DeleteEmailIdentityRequest(Mapping):
        pass
    
    class DeleteEmailIdentityResponse(Mapping):
        pass
    
    class GetAccountRequest(Mapping):
        pass
    
    class GetAccountResponse(Mapping):
        pass
    
    class GetBlacklistReportsRequest(Mapping):
        pass
    
    class GetBlacklistReportsResponse(Mapping):
        pass
    
    class BlacklistItemName(object):
        pass
    
    class GetConfigurationSetRequest(Mapping):
        pass
    
    class GetConfigurationSetResponse(Mapping):
        pass
    
    class GetConfigurationSetEventDestinationsRequest(Mapping):
        pass
    
    class GetConfigurationSetEventDestinationsResponse(Mapping):
        pass
    
    class GetDedicatedIpRequest(Mapping):
        pass
    
    class GetDedicatedIpResponse(Mapping):
        pass
    
    class GetDedicatedIpsRequest(Mapping):
        pass
    
    class GetDedicatedIpsResponse(Mapping):
        pass
    
    class GetDeliverabilityDashboardOptionsRequest(Mapping):
        pass
    
    class GetDeliverabilityDashboardOptionsResponse(Mapping):
        pass
    
    class GetDeliverabilityTestReportRequest(Mapping):
        pass
    
    class GetDeliverabilityTestReportResponse(Mapping):
        pass
    
    class GetDomainDeliverabilityCampaignRequest(Mapping):
        pass
    
    class GetDomainDeliverabilityCampaignResponse(Mapping):
        pass
    
    class GetDomainStatisticsReportRequest(Mapping):
        pass
    
    class GetDomainStatisticsReportResponse(Mapping):
        pass
    
    class Timestamp(object):
        pass
    
    class GetEmailIdentityRequest(Mapping):
        pass
    
    class GetEmailIdentityResponse(Mapping):
        pass
    
    class ListConfigurationSetsRequest(Mapping):
        pass
    
    class ListConfigurationSetsResponse(Mapping):
        pass
    
    class ListDedicatedIpPoolsRequest(Mapping):
        pass
    
    class ListDedicatedIpPoolsResponse(Mapping):
        pass
    
    class ListDeliverabilityTestReportsRequest(Mapping):
        pass
    
    class ListDeliverabilityTestReportsResponse(Mapping):
        pass
    
    class ListDomainDeliverabilityCampaignsRequest(Mapping):
        pass
    
    class ListDomainDeliverabilityCampaignsResponse(Mapping):
        pass
    
    class ListEmailIdentitiesRequest(Mapping):
        pass
    
    class ListEmailIdentitiesResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class PutAccountDedicatedIpWarmupAttributesRequest(Mapping):
        pass
    
    class PutAccountDedicatedIpWarmupAttributesResponse(Mapping):
        pass
    
    class Enabled(object):
        pass
    
    class PutAccountSendingAttributesRequest(Mapping):
        pass
    
    class PutAccountSendingAttributesResponse(Mapping):
        pass
    
    class PutConfigurationSetDeliveryOptionsRequest(Mapping):
        pass
    
    class PutConfigurationSetDeliveryOptionsResponse(Mapping):
        pass
    
    class PutConfigurationSetReputationOptionsRequest(Mapping):
        pass
    
    class PutConfigurationSetReputationOptionsResponse(Mapping):
        pass
    
    class PutConfigurationSetSendingOptionsRequest(Mapping):
        pass
    
    class PutConfigurationSetSendingOptionsResponse(Mapping):
        pass
    
    class PutConfigurationSetTrackingOptionsRequest(Mapping):
        pass
    
    class PutConfigurationSetTrackingOptionsResponse(Mapping):
        pass
    
    class PutDedicatedIpInPoolRequest(Mapping):
        pass
    
    class PutDedicatedIpInPoolResponse(Mapping):
        pass
    
    class PutDedicatedIpWarmupAttributesRequest(Mapping):
        pass
    
    class PutDedicatedIpWarmupAttributesResponse(Mapping):
        pass
    
    class PutDeliverabilityDashboardOptionRequest(Mapping):
        pass
    
    class PutDeliverabilityDashboardOptionResponse(Mapping):
        pass
    
    class DomainDeliverabilityTrackingOption(Mapping):
        pass
    
    class PutEmailIdentityDkimAttributesRequest(Mapping):
        pass
    
    class PutEmailIdentityDkimAttributesResponse(Mapping):
        pass
    
    class PutEmailIdentityFeedbackAttributesRequest(Mapping):
        pass
    
    class PutEmailIdentityFeedbackAttributesResponse(Mapping):
        pass
    
    class PutEmailIdentityMailFromAttributesRequest(Mapping):
        pass
    
    class PutEmailIdentityMailFromAttributesResponse(Mapping):
        pass
    
    class SendEmailRequest(Mapping):
        pass
    
    class SendEmailResponse(Mapping):
        pass
    
    class Destination(Mapping):
        pass
    
    class EmailAddress(object):
        pass
    
    class MessageTag(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateConfigurationSetEventDestinationRequest(Mapping):
        pass
    
    class UpdateConfigurationSetEventDestinationResponse(Mapping):
        pass
    
    def create_configuration_set(self, ConfigurationSetName: str, TrackingOptions: TrackingOptions=None, DeliveryOptions: DeliveryOptions=None, ReputationOptions: ReputationOptions=None, SendingOptions: SendingOptions=None, Tags: List[Tag]=None) -> CreateConfigurationSetResponse:
        """Create a configuration set. <i>Configuration sets</i> are groups of rules that you can apply to the emails you send using Amazon Pinpoint. You apply a configuration set to an email by including a reference to the configuration set in the headers of the email. When you apply a configuration set to an email, all of the rules in that configuration set are applied to the email. 

        :param self:
        :param Tags:  An array of objects that define the tags (keys and values) that you want to associate with the configuration set.<br/>SendingOptions:  An object that defines whether or not Amazon Pinpoint can send email that you send using the configuration set.:<br/>ReputationOptions:  An object that defines whether or not Amazon Pinpoint collects reputation metrics for the emails that you send that use the configuration set.:<br/>DeliveryOptions:  An object that defines the dedicated IP pool that is used to send emails that you send using the configuration set.:<br/>TrackingOptions:  An object that defines the open and click tracking options for emails that you send using the configuration set.:<br/>ConfigurationSetName:  The name of the configuration set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomRedirectDomain</i> The domain that you want to use for tracking open and click events.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SendingPoolName</i> The name of the dedicated IP pool that you want to associate with the configuration set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TlsPolicy</i> Specifies whether Amazon Pinpoint should require that incoming email is delivered over a connection that’s encrypted by using Transport Layer Security (TLS). If this value is set to <code>Require</code>, Amazon Pinpoint will bounce email messages that cannot be delivered over TLS. The default value is <code>Optional</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastFreshStart</i> The date and time (in Unix time) when the reputation metrics were last given a fresh start. When your account is given a fresh start, your reputation metrics are calculated starting from the date of the fresh start.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReputationMetricsEnabled</i> If <code>true</code>, tracking of reputation metrics is enabled for the configuration set. If <code>false</code>, tracking of reputation metrics is disabled for the configuration set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SendingEnabled</i> If <code>true</code>, email sending is enabled for the configuration set. If <code>false</code>, email sending is disabled for the configuration set.<br/>
        :return:         """
        pass

    def create_configuration_set_event_destination(self, EventDestination: EventDestinationDefinition, EventDestinationName: str, ConfigurationSetName: str) -> CreateConfigurationSetEventDestinationResponse:
        """Create an event destination. In Amazon Pinpoint, <i>events</i> include message sends, deliveries, opens, clicks, bounces, and complaints. <i>Event destinations</i> are places that you can send information about these events to. For example, you can send event data to Amazon SNS to receive notifications when you receive bounces or complaints, or you can use Amazon Kinesis Data Firehose to stream data to Amazon S3 for long-term storage. A single configuration set can include more than one event destination.

        :param self:
        :param EventDestination:  An object that defines the event destination.:<br/>EventDestinationName:  A name that identifies the event destination within the configuration set.<br/>ConfigurationSetName:  The name of the configuration set that you want to add an event destination to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PinpointDestination</i> An object that defines a Amazon Pinpoint destination for email events. You can use Amazon Pinpoint events to create attributes in Amazon Pinpoint projects. You can use these attributes to create segments for your campaigns.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PinpointDestination</b>: An object that defines a Amazon Pinpoint destination for email events. You can use Amazon Pinpoint events to create attributes in Amazon Pinpoint projects. You can use these attributes to create segments for your campaigns.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsDestination</i> An object that defines an Amazon SNS destination for email events. You can use Amazon SNS to send notification when certain email events occur.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SnsDestination</b>: An object that defines an Amazon SNS destination for email events. You can use Amazon SNS to send notification when certain email events occur.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchDestination</i> An object that defines an Amazon CloudWatch destination for email events. You can use Amazon CloudWatch to monitor and gain insights on your email sending metrics.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CloudWatchDestination</b>: An object that defines an Amazon CloudWatch destination for email events. You can use Amazon CloudWatch to monitor and gain insights on your email sending metrics.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KinesisFirehoseDestination</i> An object that defines an Amazon Kinesis Data Firehose destination for email events. You can use Amazon Kinesis Data Firehose to stream data to other services, such as Amazon S3 and Amazon Redshift.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KinesisFirehoseDestination</b>: An object that defines an Amazon Kinesis Data Firehose destination for email events. You can use Amazon Kinesis Data Firehose to stream data to other services, such as Amazon S3 and Amazon Redshift.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MatchingEventTypes</i> An array that specifies which events Amazon Pinpoint should send to the destinations in this <code>EventDestinationDefinition</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> If <code>true</code>, the event destination is enabled. When the event destination is enabled, the specified event types are sent to the destinations in this <code>EventDestinationDefinition</code>. If <code>false</code>, the event destination is disabled. When the event destination is disabled, events aren't sent to the specified destinations.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DeliveryStreamArn</i> The Amazon Resource Name (ARN) of the Amazon Kinesis Data Firehose stream that Amazon Pinpoint sends email events to.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoleArn</i> The Amazon Resource Name (ARN) of the IAM role that Amazon Pinpoint uses when sending email events to the Amazon Kinesis Data Firehose stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DimensionConfigurations</i> An array of objects that define the dimensions to use when you send email events to Amazon CloudWatch.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicArn</i> The Amazon Resource Name (ARN) of the Amazon SNS topic that you want to publish email events to. For more information about Amazon SNS topics, see the <a href="https://docs.aws.amazon.com/sns/latest/dg/CreateTopic.html">Amazon SNS Developer Guide</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationArn</i> The Amazon Resource Name (ARN) of the Amazon Pinpoint project that you want to send email events to.<br/>
        :return:         """
        pass

    def create_dedicated_ip_pool(self, PoolName: str, Tags: List[Tag]=None) -> CreateDedicatedIpPoolResponse:
        """Create a new pool of dedicated IP addresses. A pool can include one or more dedicated IP addresses that are associated with your Amazon Pinpoint account. You can associate a pool with a configuration set. When you send an email that uses that configuration set, Amazon Pinpoint sends it using only the IP addresses in the associated pool.

        :param self:
        :param Tags:  An object that defines the tags (keys and values) that you want to associate with the pool.<br/>PoolName:  The name of the dedicated IP pool.<br/>
        :return:         """
        pass

    def create_deliverability_test_report(self, Content: EmailContent, FromEmailAddress: str, ReportName: str=None, Tags: List[Tag]=None) -> CreateDeliverabilityTestReportResponse:
        """Create a new predictive inbox placement test. Predictive inbox placement tests can help you predict how your messages will be handled by various email providers around the world. When you perform a predictive inbox placement test, you provide a sample message that contains the content that you plan to send to your customers. Amazon Pinpoint then sends that message to special email addresses spread across several major email providers. After about 24 hours, the test is complete, and you can use the <code>GetDeliverabilityTestReport</code> operation to view the results of the test.

        :param self:
        :param Tags:  An array of objects that define the tags (keys and values) that you want to associate with the predictive inbox placement test.<br/>Content:  The HTML body of the message that you sent when you performed the predictive inbox placement test.:<br/>FromEmailAddress:  The email address that the predictive inbox placement test email was sent from.<br/>ReportName:  A unique name that helps you to identify the predictive inbox placement test when you retrieve the results.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Raw</i> The raw email message. The message has to meet the following criteria: <ul> <li> The message has to contain a header and a body, separated by one blank line. </li> <li> All of the required header fields must be present in the message. </li> <li> Each part of a multipart MIME message must be formatted properly. </li> <li> If you include attachments, they must be in a file format that Amazon Pinpoint supports.  </li> <li> The entire message must be Base64 encoded. </li> <li> If any of the MIME parts in your message contain content that is outside of the 7-bit ASCII character range, you should encode that content to ensure that recipients' email clients render the message properly. </li> <li> The length of any single line of text in the message can't exceed 1,000 characters. This restriction is defined in <a href="https://tools.ietf.org/html/rfc5321">RFC 5321</a>. </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Raw</b>: The raw email message. The message has to meet the following criteria: <ul> <li> The message has to contain a header and a body, separated by one blank line. </li> <li> All of the required header fields must be present in the message. </li> <li> Each part of a multipart MIME message must be formatted properly. </li> <li> If you include attachments, they must be in a file format that Amazon Pinpoint supports.  </li> <li> The entire message must be Base64 encoded. </li> <li> If any of the MIME parts in your message contain content that is outside of the 7-bit ASCII character range, you should encode that content to ensure that recipients' email clients render the message properly. </li> <li> The length of any single line of text in the message can't exceed 1,000 characters. This restriction is defined in <a href="https://tools.ietf.org/html/rfc5321">RFC 5321</a>. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Simple</i> The simple email message. The message consists of a subject and a message body.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Simple</b>: The simple email message. The message consists of a subject and a message body.<br/>The subject line of the email. The subject line can only contain 7-bit ASCII characters. However, you can specify non-ASCII characters in the subject line by using encoded-word syntax, as described in <a href="https://tools.ietf.org/html/rfc2047">RFC 2047</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Subject</b>: The subject line of the email. The subject line can only contain 7-bit ASCII characters. However, you can specify non-ASCII characters in the subject line by using encoded-word syntax, as described in <a href="https://tools.ietf.org/html/rfc2047">RFC 2047</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Data</i> The raw email message. The message has to meet the following criteria: <ul> <li> The message has to contain a header and a body, separated by one blank line. </li> <li> All of the required header fields must be present in the message. </li> <li> Each part of a multipart MIME message must be formatted properly. </li> <li> Attachments must be in a file format that Amazon Pinpoint supports.  </li> <li> The entire message must be Base64 encoded. </li> <li> If any of the MIME parts in your message contain content that is outside of the 7-bit ASCII character range, you should encode that content to ensure that recipients' email clients render the message properly. </li> <li> The length of any single line of text in the message can't exceed 1,000 characters. This restriction is defined in <a href="https://tools.ietf.org/html/rfc5321">RFC 5321</a>. </li> </ul><br/>
        :return: DeliverabilityTestStatus:  The status of the predictive inbox placement test. If the status is <code>IN_PROGRESS</code>, then the predictive inbox placement test is currently running. Predictive inbox placement tests are usually complete within 24 hours of creating the test. If the status is <code>COMPLETE</code>, then the test is finished, and you can use the <code>GetDeliverabilityTestReport</code> to view the results of the test.<br/>ReportId:  A unique string that identifies the predictive inbox placement test.<br/>        """
        pass

    def create_email_identity(self, EmailIdentity: str, Tags: List[Tag]=None) -> CreateEmailIdentityResponse:
        """Verifies an email identity for use with Amazon Pinpoint. In Amazon Pinpoint, an identity is an email address or domain that you use when you send email. Before you can use an identity to send email with Amazon Pinpoint, you first have to verify it. By verifying an address, you demonstrate that you're the owner of the address, and that you've given Amazon Pinpoint permission to send email from the address. When you verify an email address, Amazon Pinpoint sends an email to the address. Your email address is verified as soon as you follow the link in the verification email.  When you verify a domain, this operation provides a set of DKIM tokens, which you can convert into CNAME tokens. You add these CNAME tokens to the DNS configuration for your domain. Your domain is verified when Amazon Pinpoint detects these records in the DNS configuration for your domain. It usually takes around 72 hours to complete the domain verification process.

        :param self:
        :param Tags:  An array of objects that define the tags (keys and values) that you want to associate with the email identity.<br/>EmailIdentity:  The email address or domain that you want to verify.<br/>
        :return: DkimAttributes:  An object that contains information about the DKIM attributes for the identity. This object includes the tokens that you use to create the CNAME records that are required to complete the DKIM verification process.:<br/>VerifiedForSendingStatus:  Specifies whether or not the identity is verified. In Amazon Pinpoint, you can only send email from verified email addresses or domains. For more information about verifying identities, see the <a href="https://docs.aws.amazon.com/pinpoint/latest/userguide/channels-email-manage-verify.html">Amazon Pinpoint User Guide</a>.<br/>IdentityType:  The email identity type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tokens</i> A set of unique strings that you use to create a set of CNAME records that you add to the DNS configuration for your domain. When Amazon Pinpoint detects these records in the DNS configuration for your domain, the DKIM authentication process is complete. Amazon Pinpoint usually detects these records within about 72 hours of adding them to the DNS configuration for your domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Describes whether or not Amazon Pinpoint has successfully located the DKIM records in the DNS records for the domain. The status can be one of the following: <ul> <li>  <code>PENDING</code> – Amazon Pinpoint hasn't yet located the DKIM records in the DNS configuration for the domain, but will continue to attempt to locate them. </li> <li>  <code>SUCCESS</code> – Amazon Pinpoint located the DKIM records in the DNS configuration for the domain and determined that they're correct. Amazon Pinpoint can now send DKIM-signed email from the identity. </li> <li>  <code>FAILED</code> – Amazon Pinpoint was unable to locate the DKIM records in the DNS settings for the domain, and won't continue to search for them. </li> <li>  <code>TEMPORARY_FAILURE</code> – A temporary issue occurred, which prevented Amazon Pinpoint from determining the DKIM status for the domain. </li> <li>  <code>NOT_STARTED</code> – Amazon Pinpoint hasn't yet started searching for the DKIM records in the DKIM records for the domain. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SigningEnabled</i> If the value is <code>true</code>, then the messages that Amazon Pinpoint sends from the identity are DKIM-signed. If the value is <code>false</code>, then the messages that Amazon Pinpoint sends from the identity aren't DKIM-signed.<br/>        """
        pass

    def delete_configuration_set(self, ConfigurationSetName: str) -> DeleteConfigurationSetResponse:
        """Delete an existing configuration set. In Amazon Pinpoint, <i>configuration sets</i> are groups of rules that you can apply to the emails you send. You apply a configuration set to an email by including a reference to the configuration set in the headers of the email. When you apply a configuration set to an email, all of the rules in that configuration set are applied to the email.

        :param self:
        :param ConfigurationSetName:  The name of the configuration set that you want to delete.<br/>
        :return:         """
        pass

    def delete_configuration_set_event_destination(self, EventDestinationName: str, ConfigurationSetName: str) -> DeleteConfigurationSetEventDestinationResponse:
        """Delete an event destination. In Amazon Pinpoint, <i>events</i> include message sends, deliveries, opens, clicks, bounces, and complaints. <i>Event destinations</i> are places that you can send information about these events to. For example, you can send event data to Amazon SNS to receive notifications when you receive bounces or complaints, or you can use Amazon Kinesis Data Firehose to stream data to Amazon S3 for long-term storage.

        :param self:
        :param EventDestinationName:  The name of the event destination that you want to delete.<br/>ConfigurationSetName:  The name of the configuration set that contains the event destination that you want to delete.<br/>
        :return:         """
        pass

    def delete_dedicated_ip_pool(self, PoolName: str) -> DeleteDedicatedIpPoolResponse:
        """Delete a dedicated IP pool.

        :param self:
        :param PoolName:  The name of the dedicated IP pool that you want to delete.<br/>
        :return:         """
        pass

    def delete_email_identity(self, EmailIdentity: str) -> DeleteEmailIdentityResponse:
        """Deletes an email identity that you previously verified for use with Amazon Pinpoint. An identity can be either an email address or a domain name.

        :param self:
        :param EmailIdentity:  The identity (that is, the email address or domain) that you want to delete from your Amazon Pinpoint account.<br/>
        :return:         """
        pass

    def get_account(self) -> GetAccountResponse:
        """Obtain information about the email-sending status and capabilities of your Amazon Pinpoint account in the current AWS Region.

        :param self:
        :param 
        :return: ProductionAccessEnabled:  Indicates whether or not your account has production access in the current AWS Region. If the value is <code>false</code>, then your account is in the <i>sandbox</i>. When your account is in the sandbox, you can only send email to verified identities. Additionally, the maximum number of emails you can send in a 24-hour period (your sending quota) is 200, and the maximum number of emails you can send per second (your maximum sending rate) is 1. If the value is <code>true</code>, then your account has production access. When your account has production access, you can send email to any address. The sending quota and maximum sending rate for your account vary based on your specific use case.<br/>EnforcementStatus:  The reputation status of your Amazon Pinpoint account. The status can be one of the following: <ul> <li>  <code>HEALTHY</code> – There are no reputation-related issues that currently impact your account. </li> <li>  <code>PROBATION</code> – We've identified some issues with your Amazon Pinpoint account. We're placing your account under review while you work on correcting these issues. </li> <li>  <code>SHUTDOWN</code> – Your account's ability to send email is currently paused because of an issue with the email sent from your account. When you correct the issue, you can contact us and request that your account's ability to send email is resumed. </li> </ul><br/>DedicatedIpAutoWarmupEnabled:  Indicates whether or not the automatic warm-up feature is enabled for dedicated IP addresses that are associated with your account.<br/>SendingEnabled:  Indicates whether or not email sending is enabled for your Amazon Pinpoint account in the current AWS Region.<br/>SendQuota:  An object that contains information about the per-day and per-second sending limits for your Amazon Pinpoint account in the current AWS Region.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SentLast24Hours</i> The number of emails sent from your Amazon Pinpoint account in the current AWS Region over the past 24 hours.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxSendRate</i> The maximum number of emails that you can send per second in the current AWS Region. This value is also called your <i>maximum sending rate</i> or your <i>maximum TPS (transactions per second) rate</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Max24HourSend</i> The maximum number of emails that you can send in the current AWS Region over a 24-hour period. This value is also called your <i>sending quota</i>.<br/>        """
        pass

    def get_blacklist_reports(self, BlacklistItemNames: List[BlacklistItemName]) -> GetBlacklistReportsResponse:
        """Retrieve a list of the blacklists that your dedicated IP addresses appear on.

        :param self:
        :param BlacklistItemNames:  A list of IP addresses that you want to retrieve blacklist information about. You can only specify the dedicated IP addresses that you use to send email using Amazon Pinpoint or Amazon SES.<br/>
        :return: BlacklistReport:  An object that contains information about a blacklist that one of your dedicated IP addresses appears on.<br/>        """
        pass

    def get_configuration_set(self, ConfigurationSetName: str) -> GetConfigurationSetResponse:
        """Get information about an existing configuration set, including the dedicated IP pool that it's associated with, whether or not it's enabled for sending email, and more. In Amazon Pinpoint, <i>configuration sets</i> are groups of rules that you can apply to the emails you send. You apply a configuration set to an email by including a reference to the configuration set in the headers of the email. When you apply a configuration set to an email, all of the rules in that configuration set are applied to the email.

        :param self:
        :param ConfigurationSetName:  The name of the configuration set that you want to obtain more information about.<br/>
        :return: Tags:  An array of objects that define the tags (keys and values) that are associated with the configuration set.<br/>SendingOptions:  An object that defines whether or not Amazon Pinpoint can send email that you send using the configuration set.:<br/>ReputationOptions:  An object that defines whether or not Amazon Pinpoint collects reputation metrics for the emails that you send that use the configuration set.:<br/>DeliveryOptions:  An object that defines the dedicated IP pool that is used to send emails that you send using the configuration set.:<br/>TrackingOptions:  An object that defines the open and click tracking options for emails that you send using the configuration set.:<br/>ConfigurationSetName:  The name of the configuration set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomRedirectDomain</i> The domain that you want to use for tracking open and click events.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SendingPoolName</i> The name of the dedicated IP pool that you want to associate with the configuration set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TlsPolicy</i> Specifies whether Amazon Pinpoint should require that incoming email is delivered over a connection that’s encrypted by using Transport Layer Security (TLS). If this value is set to <code>Require</code>, Amazon Pinpoint will bounce email messages that cannot be delivered over TLS. The default value is <code>Optional</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastFreshStart</i> The date and time (in Unix time) when the reputation metrics were last given a fresh start. When your account is given a fresh start, your reputation metrics are calculated starting from the date of the fresh start.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReputationMetricsEnabled</i> If <code>true</code>, tracking of reputation metrics is enabled for the configuration set. If <code>false</code>, tracking of reputation metrics is disabled for the configuration set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SendingEnabled</i> If <code>true</code>, email sending is enabled for the configuration set. If <code>false</code>, email sending is disabled for the configuration set.<br/>        """
        pass

    def get_configuration_set_event_destinations(self, ConfigurationSetName: str) -> GetConfigurationSetEventDestinationsResponse:
        """Retrieve a list of event destinations that are associated with a configuration set. In Amazon Pinpoint, <i>events</i> include message sends, deliveries, opens, clicks, bounces, and complaints. <i>Event destinations</i> are places that you can send information about these events to. For example, you can send event data to Amazon SNS to receive notifications when you receive bounces or complaints, or you can use Amazon Kinesis Data Firehose to stream data to Amazon S3 for long-term storage.

        :param self:
        :param ConfigurationSetName:  The name of the configuration set that contains the event destination.<br/>
        :return: EventDestinations:  An array that includes all of the events destinations that have been configured for the configuration set.<br/>        """
        pass

    def get_dedicated_ip(self, Ip: str) -> GetDedicatedIpResponse:
        """Get information about a dedicated IP address, including the name of the dedicated IP pool that it's associated with, as well information about the automatic warm-up process for the address.

        :param self:
        :param Ip:  The IP address that you want to obtain more information about. The value you specify has to be a dedicated IP address that's assocaited with your Amazon Pinpoint account.<br/>
        :return: DedicatedIp:  An object that contains information about a dedicated IP address.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PoolName</i> The name of the dedicated IP pool that the IP address is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WarmupPercentage</i> Indicates how complete the dedicated IP warm-up process is. When this value equals 1, the address has completed the warm-up process and is ready for use.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WarmupStatus</i> The warm-up status of a dedicated IP address. The status can have one of the following values: <ul> <li>  <code>IN_PROGRESS</code> – The IP address isn't ready to use because the dedicated IP warm-up process is ongoing. </li> <li>  <code>DONE</code> – The dedicated IP warm-up process is complete, and the IP address is ready to use. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Ip</i> An IP address that is reserved for use by your Amazon Pinpoint account.<br/>        """
        pass

    def get_dedicated_ips(self, PoolName: str=None, NextToken: str=None, PageSize: int=None) -> GetDedicatedIpsResponse:
        """List the dedicated IP addresses that are associated with your Amazon Pinpoint account.

        :param self:
        :param PageSize:  The number of results to show in a single call to <code>GetDedicatedIpsRequest</code>. If the number of results is larger than the number you specified in this parameter, then the response includes a <code>NextToken</code> element, which you can use to obtain additional results.<br/>NextToken:  A token returned from a previous call to <code>GetDedicatedIps</code> to indicate the position of the dedicated IP pool in the list of IP pools.<br/>PoolName:  The name of the IP pool that the dedicated IP address is associated with.<br/>
        :return: NextToken:  A token that indicates that there are additional dedicated IP addresses to list. To view additional addresses, issue another request to <code>GetDedicatedIps</code>, passing this token in the <code>NextToken</code> parameter.<br/>DedicatedIps:  A list of dedicated IP addresses that are reserved for use by your Amazon Pinpoint account.<br/>        """
        pass

    def get_deliverability_dashboard_options(self) -> GetDeliverabilityDashboardOptionsResponse:
        """Retrieve information about the status of the Deliverability dashboard for your Amazon Pinpoint account. When the Deliverability dashboard is enabled, you gain access to reputation, deliverability, and other metrics for the domains that you use to send email using Amazon Pinpoint. You also gain the ability to perform predictive inbox placement tests. When you use the Deliverability dashboard, you pay a monthly subscription charge, in addition to any other fees that you accrue by using Amazon Pinpoint. For more information about the features and cost of a Deliverability dashboard subscription, see <a href="http://aws.amazon.com/pinpoint/pricing/">Amazon Pinpoint Pricing</a>.

        :param self:
        :param 
        :return: PendingExpirationSubscribedDomains:  An array of objects, one for each verified domain that you use to send email and currently has an active Deliverability dashboard subscription that's scheduled to expire at the end of the current calendar month.<br/>ActiveSubscribedDomains:  An array of objects, one for each verified domain that you use to send email and currently has an active Deliverability dashboard subscription that isn’t scheduled to expire at the end of the current calendar month.<br/>AccountStatus:  The current status of your Deliverability dashboard subscription. If this value is <code>PENDING_EXPIRATION</code>, your subscription is scheduled to expire at the end of the current calendar month.<br/>SubscriptionExpiryDate:  The date, in Unix time format, when your current subscription to the Deliverability dashboard is scheduled to expire, if your subscription is scheduled to expire at the end of the current calendar month. This value is null if you have an active subscription that isn’t due to expire at the end of the month.<br/>DashboardEnabled:  Specifies whether the Deliverability dashboard is enabled for your Amazon Pinpoint account. If this value is <code>true</code>, the dashboard is enabled.<br/>        """
        pass

    def get_deliverability_test_report(self, ReportId: str) -> GetDeliverabilityTestReportResponse:
        """Retrieve the results of a predictive inbox placement test.

        :param self:
        :param ReportId:  A unique string that identifies the predictive inbox placement test.<br/>
        :return: Tags:  An array of objects that define the tags (keys and values) that are associated with the predictive inbox placement test.<br/>Message:  An object that contains the message that you sent when you performed this predictive inbox placement test.<br/>IspPlacements:  An object that describes how the test email was handled by several email providers, including Gmail, Hotmail, Yahoo, AOL, and others.<br/>OverallPlacement:  An object that specifies how many test messages that were sent during the predictive inbox placement test were delivered to recipients' inboxes, how many were sent to recipients' spam folders, and how many weren't delivered.:<br/>DeliverabilityTestReport:  An object that contains the results of the predictive inbox placement test.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeliverabilityTestStatus</i> The status of the predictive inbox placement test. If the status is <code>IN_PROGRESS</code>, then the predictive inbox placement test is currently running. Predictive inbox placement tests are usually complete within 24 hours of creating the test. If the status is <code>COMPLETE</code>, then the test is finished, and you can use the <code>GetDeliverabilityTestReport</code> to view the results of the test.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateDate</i> The date and time when the predictive inbox placement test was created, in Unix time format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FromEmailAddress</i> The sender address that you specified for the predictive inbox placement test.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subject</i> The subject line for an email that you submitted in a predictive inbox placement test.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReportName</i> A name that helps you identify a predictive inbox placement test report.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReportId</i> A unique string that identifies the predictive inbox placement test.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DkimPercentage</i> The percentage of emails that were authenticated by using DomainKeys Identified Mail (DKIM) during the predictive inbox placement test.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SpfPercentage</i> The percentage of emails that were authenticated by using Sender Policy Framework (SPF) during the predictive inbox placement test.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MissingPercentage</i> The percentage of emails that didn't arrive in recipients' inboxes at all during the predictive inbox placement test.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SpamPercentage</i> The percentage of emails that arrived in recipients' spam or junk mail folders during the predictive inbox placement test.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InboxPercentage</i> The percentage of emails that arrived in recipients' inboxes during the predictive inbox placement test.<br/>        """
        pass

    def get_domain_deliverability_campaign(self, CampaignId: str) -> GetDomainDeliverabilityCampaignResponse:
        """Retrieve all the deliverability data for a specific campaign. This data is available for a campaign only if the campaign sent email by using a domain that the Deliverability dashboard is enabled for (<code>PutDeliverabilityDashboardOption</code> operation).

        :param self:
        :param CampaignId:  The unique identifier for the campaign. Amazon Pinpoint automatically generates and assigns this identifier to a campaign. This value is not the same as the campaign identifier that Amazon Pinpoint assigns to campaigns that you create and manage by using the Amazon Pinpoint API or the Amazon Pinpoint console.<br/>
        :return: DomainDeliverabilityCampaign:  An object that contains the deliverability data for the campaign.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Esps</i> The major email providers who handled the email message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProjectedVolume</i> The projected number of recipients that the email message was sent to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadDeleteRate</i> The percentage of email messages that were opened and then deleted by recipients. Due to technical limitations, this value only includes recipients who opened the message by using an email client that supports images.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeleteRate</i> The percentage of email messages that were deleted by recipients, without being opened first. Due to technical limitations, this value only includes recipients who opened the message by using an email client that supports images.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadRate</i> The percentage of email messages that were opened by recipients. Due to technical limitations, this value only includes recipients who opened the message by using an email client that supports images.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SpamCount</i> The number of email messages that were delivered to recipients' spam or junk mail folders.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InboxCount</i> The number of email messages that were delivered to recipients’ inboxes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastSeenDateTime</i> The last time, in Unix time format, when the email message was delivered to any recipient's inbox. This value can help you determine how long it took for a campaign to deliver an email message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FirstSeenDateTime</i> The first time, in Unix time format, when the email message was delivered to any recipient's inbox. This value can help you determine how long it took for a campaign to deliver an email message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SendingIps</i> The IP addresses that were used to send the email message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FromAddress</i> The verified email address that the email message was sent from.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subject</i> The subject line, or title, of the email message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImageUrl</i> The URL of an image that contains a snapshot of the email message that was sent.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CampaignId</i> The unique identifier for the campaign. Amazon Pinpoint automatically generates and assigns this identifier to a campaign. This value is not the same as the campaign identifier that Amazon Pinpoint assigns to campaigns that you create and manage by using the Amazon Pinpoint API or the Amazon Pinpoint console.<br/>        """
        pass

    def get_domain_statistics_report(self, EndDate: Timestamp, StartDate: Timestamp, Domain: str) -> GetDomainStatisticsReportResponse:
        """Retrieve inbox placement and engagement rates for the domains that you use to send email.

        :param self:
        :param EndDate:  The last day (in Unix time) that you want to obtain domain deliverability metrics for. The <code>EndDate</code> that you specify has to be less than or equal to 30 days after the <code>StartDate</code>.<br/>StartDate:  The first day (in Unix time) that you want to obtain domain deliverability metrics for.<br/>Domain:  The domain that you want to obtain deliverability metrics for.<br/>
        :return: DailyVolumes:  An object that contains deliverability metrics for the domain that you specified. This object contains data for each day, starting on the <code>StartDate</code> and ending on the <code>EndDate</code>.<br/>OverallVolume:  An object that contains deliverability metrics for the domain that you specified. The data in this object is a summary of all of the data that was collected from the <code>StartDate</code> to the <code>EndDate</code>.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainIspPlacements</i> An object that contains inbox and junk mail placement metrics for individual email providers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadRatePercent</i> The percentage of emails that were sent from the domain that were read by their recipients.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeStatistics</i> An object that contains information about the numbers of messages that arrived in recipients' inboxes and junk mail folders.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VolumeStatistics</b>: An object that contains information about the numbers of messages that arrived in recipients' inboxes and junk mail folders.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ProjectedSpam</i> An estimate of the percentage of emails sent from the current domain that will arrive in recipients' spam or junk mail folders.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ProjectedInbox</i> An estimate of the percentage of emails sent from the current domain that will arrive in recipients' inboxes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SpamRawCount</i> The total number of emails that arrived in recipients' spam or junk mail folders.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InboxRawCount</i> The total number of emails that arrived in recipients' inboxes.<br/>        """
        pass

    def get_email_identity(self, EmailIdentity: str) -> GetEmailIdentityResponse:
        """Provides information about a specific identity associated with your Amazon Pinpoint account, including the identity's verification status, its DKIM authentication status, and its custom Mail-From settings.

        :param self:
        :param EmailIdentity:  The email identity that you want to retrieve details for.<br/>
        :return: Tags:  An array of objects that define the tags (keys and values) that are associated with the email identity.<br/>MailFromAttributes:  An object that contains information about the Mail-From attributes for the email identity.:<br/>DkimAttributes:  An object that contains information about the DKIM attributes for the identity. This object includes the tokens that you use to create the CNAME records that are required to complete the DKIM verification process.:<br/>VerifiedForSendingStatus:  Specifies whether or not the identity is verified. In Amazon Pinpoint, you can only send email from verified email addresses or domains. For more information about verifying identities, see the <a href="https://docs.aws.amazon.com/pinpoint/latest/userguide/channels-email-manage-verify.html">Amazon Pinpoint User Guide</a>.<br/>FeedbackForwardingStatus:  The feedback forwarding configuration for the identity. If the value is <code>true</code>, Amazon Pinpoint sends you email notifications when bounce or complaint events occur. Amazon Pinpoint sends this notification to the address that you specified in the Return-Path header of the original email. When you set this value to <code>false</code>, Amazon Pinpoint sends notifications through other mechanisms, such as by notifying an Amazon SNS topic or another event destination. You're required to have a method of tracking bounces and complaints. If you haven't set up another mechanism for receiving bounce or complaint notifications, Amazon Pinpoint sends an email notification when these events occur (even if this setting is disabled).<br/>IdentityType:  The email identity type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tokens</i> A set of unique strings that you use to create a set of CNAME records that you add to the DNS configuration for your domain. When Amazon Pinpoint detects these records in the DNS configuration for your domain, the DKIM authentication process is complete. Amazon Pinpoint usually detects these records within about 72 hours of adding them to the DNS configuration for your domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Describes whether or not Amazon Pinpoint has successfully located the DKIM records in the DNS records for the domain. The status can be one of the following: <ul> <li>  <code>PENDING</code> – Amazon Pinpoint hasn't yet located the DKIM records in the DNS configuration for the domain, but will continue to attempt to locate them. </li> <li>  <code>SUCCESS</code> – Amazon Pinpoint located the DKIM records in the DNS configuration for the domain and determined that they're correct. Amazon Pinpoint can now send DKIM-signed email from the identity. </li> <li>  <code>FAILED</code> – Amazon Pinpoint was unable to locate the DKIM records in the DNS settings for the domain, and won't continue to search for them. </li> <li>  <code>TEMPORARY_FAILURE</code> – A temporary issue occurred, which prevented Amazon Pinpoint from determining the DKIM status for the domain. </li> <li>  <code>NOT_STARTED</code> – Amazon Pinpoint hasn't yet started searching for the DKIM records in the DKIM records for the domain. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SigningEnabled</i> If the value is <code>true</code>, then the messages that Amazon Pinpoint sends from the identity are DKIM-signed. If the value is <code>false</code>, then the messages that Amazon Pinpoint sends from the identity aren't DKIM-signed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BehaviorOnMxFailure</i> The action that Amazon Pinpoint to takes if it can't read the required MX record for a custom MAIL FROM domain. When you set this value to <code>UseDefaultValue</code>, Amazon Pinpoint uses <i>amazonses.com</i> as the MAIL FROM domain. When you set this value to <code>RejectMessage</code>, Amazon Pinpoint returns a <code>MailFromDomainNotVerified</code> error, and doesn't attempt to deliver the email. These behaviors are taken when the custom MAIL FROM domain configuration is in the <code>Pending</code>, <code>Failed</code>, and <code>TemporaryFailure</code> states.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MailFromDomainStatus</i> The status of the MAIL FROM domain. This status can have the following values: <ul> <li>  <code>PENDING</code> – Amazon Pinpoint hasn't started searching for the MX record yet. </li> <li>  <code>SUCCESS</code> – Amazon Pinpoint detected the required MX record for the MAIL FROM domain. </li> <li>  <code>FAILED</code> – Amazon Pinpoint can't find the required MX record, or the record no longer exists. </li> <li>  <code>TEMPORARY_FAILURE</code> – A temporary issue occurred, which prevented Amazon Pinpoint from determining the status of the MAIL FROM domain. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MailFromDomain</i> The name of a domain that an email identity uses as a custom MAIL FROM domain.<br/>        """
        pass

    def list_configuration_sets(self, NextToken: str=None, PageSize: int=None) -> ListConfigurationSetsResponse:
        """List all of the configuration sets associated with your Amazon Pinpoint account in the current region. In Amazon Pinpoint, <i>configuration sets</i> are groups of rules that you can apply to the emails you send. You apply a configuration set to an email by including a reference to the configuration set in the headers of the email. When you apply a configuration set to an email, all of the rules in that configuration set are applied to the email.

        :param self:
        :param PageSize:  The number of results to show in a single call to <code>ListConfigurationSets</code>. If the number of results is larger than the number you specified in this parameter, then the response includes a <code>NextToken</code> element, which you can use to obtain additional results.<br/>NextToken:  A token returned from a previous call to <code>ListConfigurationSets</code> to indicate the position in the list of configuration sets.<br/>
        :return: NextToken:  A token that indicates that there are additional configuration sets to list. To view additional configuration sets, issue another request to <code>ListConfigurationSets</code>, and pass this token in the <code>NextToken</code> parameter.<br/>ConfigurationSets:  An array that contains all of the configuration sets in your Amazon Pinpoint account in the current AWS Region.<br/>        """
        pass

    def list_dedicated_ip_pools(self, NextToken: str=None, PageSize: int=None) -> ListDedicatedIpPoolsResponse:
        """List all of the dedicated IP pools that exist in your Amazon Pinpoint account in the current AWS Region.

        :param self:
        :param PageSize:  The number of results to show in a single call to <code>ListDedicatedIpPools</code>. If the number of results is larger than the number you specified in this parameter, then the response includes a <code>NextToken</code> element, which you can use to obtain additional results.<br/>NextToken:  A token returned from a previous call to <code>ListDedicatedIpPools</code> to indicate the position in the list of dedicated IP pools.<br/>
        :return: NextToken:  A token that indicates that there are additional IP pools to list. To view additional IP pools, issue another request to <code>ListDedicatedIpPools</code>, passing this token in the <code>NextToken</code> parameter.<br/>DedicatedIpPools:  A list of all of the dedicated IP pools that are associated with your Amazon Pinpoint account.<br/>        """
        pass

    def list_deliverability_test_reports(self, NextToken: str=None, PageSize: int=None) -> ListDeliverabilityTestReportsResponse:
        """Show a list of the predictive inbox placement tests that you've performed, regardless of their statuses. For predictive inbox placement tests that are complete, you can use the <code>GetDeliverabilityTestReport</code> operation to view the results.

        :param self:
        :param PageSize:  The number of results to show in a single call to <code>ListDeliverabilityTestReports</code>. If the number of results is larger than the number you specified in this parameter, then the response includes a <code>NextToken</code> element, which you can use to obtain additional results. The value you specify has to be at least 0, and can be no more than 1000.<br/>NextToken:  A token returned from a previous call to <code>ListDeliverabilityTestReports</code> to indicate the position in the list of predictive inbox placement tests.<br/>
        :return: NextToken:  A token that indicates that there are additional predictive inbox placement tests to list. To view additional predictive inbox placement tests, issue another request to <code>ListDeliverabilityTestReports</code>, and pass this token in the <code>NextToken</code> parameter.<br/>DeliverabilityTestReports:  An object that contains a lists of predictive inbox placement tests that you've performed.<br/>        """
        pass

    def list_domain_deliverability_campaigns(self, SubscribedDomain: str, EndDate: Timestamp, StartDate: Timestamp, NextToken: str=None, PageSize: int=None) -> ListDomainDeliverabilityCampaignsResponse:
        """Retrieve deliverability data for all the campaigns that used a specific domain to send email during a specified time range. This data is available for a domain only if you enabled the Deliverability dashboard (<code>PutDeliverabilityDashboardOption</code> operation) for the domain.

        :param self:
        :param PageSize:  The maximum number of results to include in response to a single call to the <code>ListDomainDeliverabilityCampaigns</code> operation. If the number of results is larger than the number that you specify in this parameter, the response includes a <code>NextToken</code> element, which you can use to obtain additional results.<br/>NextToken:  A token that’s returned from a previous call to the <code>ListDomainDeliverabilityCampaigns</code> operation. This token indicates the position of a campaign in the list of campaigns.<br/>SubscribedDomain:  The domain to obtain deliverability data for.<br/>EndDate:  The last day, in Unix time format, that you want to obtain deliverability data for. This value has to be less than or equal to 30 days after the value of the <code>StartDate</code> parameter.<br/>StartDate:  The first day, in Unix time format, that you want to obtain deliverability data for.<br/>
        :return: NextToken:  A token that’s returned from a previous call to the <code>ListDomainDeliverabilityCampaigns</code> operation. This token indicates the position of the campaign in the list of campaigns.<br/>DomainDeliverabilityCampaigns:  An array of responses, one for each campaign that used the domain to send email during the specified time range.<br/>        """
        pass

    def list_email_identities(self, NextToken: str=None, PageSize: int=None) -> ListEmailIdentitiesResponse:
        """Returns a list of all of the email identities that are associated with your Amazon Pinpoint account. An identity can be either an email address or a domain. This operation returns identities that are verified as well as those that aren't.

        :param self:
        :param PageSize:  The number of results to show in a single call to <code>ListEmailIdentities</code>. If the number of results is larger than the number you specified in this parameter, then the response includes a <code>NextToken</code> element, which you can use to obtain additional results. The value you specify has to be at least 0, and can be no more than 1000.<br/>NextToken:  A token returned from a previous call to <code>ListEmailIdentities</code> to indicate the position in the list of identities.<br/>
        :return: NextToken:  A token that indicates that there are additional configuration sets to list. To view additional configuration sets, issue another request to <code>ListEmailIdentities</code>, and pass this token in the <code>NextToken</code> parameter.<br/>EmailIdentities:  An array that includes all of the identities associated with your Amazon Pinpoint account.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceArn: str) -> ListTagsForResourceResponse:
        """Retrieve a list of the tags (keys and values) that are associated with a specified resource. A <i>tag</i> is a label that you optionally define and associate with a resource in Amazon Pinpoint. Each tag consists of a required <i>tag key</i> and an optional associated <i>tag value</i>. A tag key is a general label that acts as a category for more specific tag values. A tag value acts as a descriptor within a tag key.

        :param self:
        :param ResourceArn:  The Amazon Resource Name (ARN) of the resource that you want to retrieve tag information for.<br/>
        :return: Tags:  An array that lists all the tags that are associated with the resource. Each tag consists of a required tag key (<code>Key</code>) and an associated tag value (<code>Value</code>)<br/>        """
        pass

    def put_account_dedicated_ip_warmup_attributes(self, AutoWarmupEnabled: Enabled=None) -> PutAccountDedicatedIpWarmupAttributesResponse:
        """Enable or disable the automatic warm-up feature for dedicated IP addresses.

        :param self:
        :param AutoWarmupEnabled:  Enables or disables the automatic warm-up feature for dedicated IP addresses that are associated with your Amazon Pinpoint account in the current AWS Region. Set to <code>true</code> to enable the automatic warm-up feature, or set to <code>false</code> to disable it.<br/>
        :return:         """
        pass

    def put_account_sending_attributes(self, SendingEnabled: Enabled=None) -> PutAccountSendingAttributesResponse:
        """Enable or disable the ability of your account to send email.

        :param self:
        :param SendingEnabled:  Enables or disables your account's ability to send email. Set to <code>true</code> to enable email sending, or set to <code>false</code> to disable email sending. <note> If AWS paused your account's ability to send email, you can't use this operation to resume your account's ability to send email. </note><br/>
        :return:         """
        pass

    def put_configuration_set_delivery_options(self, ConfigurationSetName: str, TlsPolicy: str=None, SendingPoolName: str=None) -> PutConfigurationSetDeliveryOptionsResponse:
        """Associate a configuration set with a dedicated IP pool. You can use dedicated IP pools to create groups of dedicated IP addresses for sending specific types of email.

        :param self:
        :param SendingPoolName:  The name of the dedicated IP pool that you want to associate with the configuration set.<br/>TlsPolicy:  Whether Amazon Pinpoint should require that incoming email is delivered over a connection encrypted with Transport Layer Security (TLS).<br/>ConfigurationSetName:  The name of the configuration set that you want to associate with a dedicated IP pool.<br/>
        :return:         """
        pass

    def put_configuration_set_reputation_options(self, ConfigurationSetName: str, ReputationMetricsEnabled: Enabled=None) -> PutConfigurationSetReputationOptionsResponse:
        """Enable or disable collection of reputation metrics for emails that you send using a particular configuration set in a specific AWS Region.

        :param self:
        :param ReputationMetricsEnabled:  If <code>true</code>, tracking of reputation metrics is enabled for the configuration set. If <code>false</code>, tracking of reputation metrics is disabled for the configuration set.<br/>ConfigurationSetName:  The name of the configuration set that you want to enable or disable reputation metric tracking for.<br/>
        :return:         """
        pass

    def put_configuration_set_sending_options(self, ConfigurationSetName: str, SendingEnabled: Enabled=None) -> PutConfigurationSetSendingOptionsResponse:
        """Enable or disable email sending for messages that use a particular configuration set in a specific AWS Region.

        :param self:
        :param SendingEnabled:  If <code>true</code>, email sending is enabled for the configuration set. If <code>false</code>, email sending is disabled for the configuration set.<br/>ConfigurationSetName:  The name of the configuration set that you want to enable or disable email sending for.<br/>
        :return:         """
        pass

    def put_configuration_set_tracking_options(self, ConfigurationSetName: str, CustomRedirectDomain: str=None) -> PutConfigurationSetTrackingOptionsResponse:
        """Specify a custom domain to use for open and click tracking elements in email that you send using Amazon Pinpoint.

        :param self:
        :param CustomRedirectDomain:  The domain that you want to use to track open and click events.<br/>ConfigurationSetName:  The name of the configuration set that you want to add a custom tracking domain to.<br/>
        :return:         """
        pass

    def put_dedicated_ip_in_pool(self, DestinationPoolName: str, Ip: str) -> PutDedicatedIpInPoolResponse:
        """Move a dedicated IP address to an existing dedicated IP pool. <note> The dedicated IP address that you specify must already exist, and must be associated with your Amazon Pinpoint account.  The dedicated IP pool you specify must already exist. You can create a new pool by using the <code>CreateDedicatedIpPool</code> operation. </note>

        :param self:
        :param DestinationPoolName:  The name of the IP pool that you want to add the dedicated IP address to. You have to specify an IP pool that already exists.<br/>Ip:  The IP address that you want to move to the dedicated IP pool. The value you specify has to be a dedicated IP address that's associated with your Amazon Pinpoint account.<br/>
        :return:         """
        pass

    def put_dedicated_ip_warmup_attributes(self, WarmupPercentage: int, Ip: str) -> PutDedicatedIpWarmupAttributesResponse:
        """<p/>

        :param self:
        :param WarmupPercentage:  The warm-up percentage that you want to associate with the dedicated IP address.<br/>Ip:  The dedicated IP address that you want to update the warm-up attributes for.<br/>
        :return:         """
        pass

    def put_deliverability_dashboard_option(self, DashboardEnabled: Enabled, SubscribedDomains: List[DomainDeliverabilityTrackingOption]=None) -> PutDeliverabilityDashboardOptionResponse:
        """Enable or disable the Deliverability dashboard for your Amazon Pinpoint account. When you enable the Deliverability dashboard, you gain access to reputation, deliverability, and other metrics for the domains that you use to send email using Amazon Pinpoint. You also gain the ability to perform predictive inbox placement tests. When you use the Deliverability dashboard, you pay a monthly subscription charge, in addition to any other fees that you accrue by using Amazon Pinpoint. For more information about the features and cost of a Deliverability dashboard subscription, see <a href="http://aws.amazon.com/pinpoint/pricing/">Amazon Pinpoint Pricing</a>.

        :param self:
        :param SubscribedDomains:  An array of objects, one for each verified domain that you use to send email and enabled the Deliverability dashboard for.<br/>DashboardEnabled:  Specifies whether to enable the Deliverability dashboard for your Amazon Pinpoint account. To enable the dashboard, set this value to <code>true</code>.<br/>
        :return:         """
        pass

    def put_email_identity_dkim_attributes(self, EmailIdentity: str, SigningEnabled: Enabled=None) -> PutEmailIdentityDkimAttributesResponse:
        """Used to enable or disable DKIM authentication for an email identity.

        :param self:
        :param SigningEnabled:  Sets the DKIM signing configuration for the identity. When you set this value <code>true</code>, then the messages that Amazon Pinpoint sends from the identity are DKIM-signed. When you set this value to <code>false</code>, then the messages that Amazon Pinpoint sends from the identity aren't DKIM-signed.<br/>EmailIdentity:  The email identity that you want to change the DKIM settings for.<br/>
        :return:         """
        pass

    def put_email_identity_feedback_attributes(self, EmailIdentity: str, EmailForwardingEnabled: Enabled=None) -> PutEmailIdentityFeedbackAttributesResponse:
        """Used to enable or disable feedback forwarding for an identity. This setting determines what happens when an identity is used to send an email that results in a bounce or complaint event. When you enable feedback forwarding, Amazon Pinpoint sends you email notifications when bounce or complaint events occur. Amazon Pinpoint sends this notification to the address that you specified in the Return-Path header of the original email. When you disable feedback forwarding, Amazon Pinpoint sends notifications through other mechanisms, such as by notifying an Amazon SNS topic. You're required to have a method of tracking bounces and complaints. If you haven't set up another mechanism for receiving bounce or complaint notifications, Amazon Pinpoint sends an email notification when these events occur (even if this setting is disabled).

        :param self:
        :param EmailForwardingEnabled:  Sets the feedback forwarding configuration for the identity. If the value is <code>true</code>, Amazon Pinpoint sends you email notifications when bounce or complaint events occur. Amazon Pinpoint sends this notification to the address that you specified in the Return-Path header of the original email. When you set this value to <code>false</code>, Amazon Pinpoint sends notifications through other mechanisms, such as by notifying an Amazon SNS topic or another event destination. You're required to have a method of tracking bounces and complaints. If you haven't set up another mechanism for receiving bounce or complaint notifications, Amazon Pinpoint sends an email notification when these events occur (even if this setting is disabled).<br/>EmailIdentity:  The email identity that you want to configure bounce and complaint feedback forwarding for.<br/>
        :return:         """
        pass

    def put_email_identity_mail_from_attributes(self, EmailIdentity: str, MailFromDomain: str=None, BehaviorOnMxFailure: str=None) -> PutEmailIdentityMailFromAttributesResponse:
        """Used to enable or disable the custom Mail-From domain configuration for an email identity.

        :param self:
        :param BehaviorOnMxFailure:  The action that you want Amazon Pinpoint to take if it can't read the required MX record when you send an email. When you set this value to <code>UseDefaultValue</code>, Amazon Pinpoint uses <i>amazonses.com</i> as the MAIL FROM domain. When you set this value to <code>RejectMessage</code>, Amazon Pinpoint returns a <code>MailFromDomainNotVerified</code> error, and doesn't attempt to deliver the email. These behaviors are taken when the custom MAIL FROM domain configuration is in the <code>Pending</code>, <code>Failed</code>, and <code>TemporaryFailure</code> states.<br/>MailFromDomain:   The custom MAIL FROM domain that you want the verified identity to use. The MAIL FROM domain must meet the following criteria: <ul> <li> It has to be a subdomain of the verified identity. </li> <li> It can't be used to receive email. </li> <li> It can't be used in a "From" address if the MAIL FROM domain is a destination for feedback forwarding emails. </li> </ul><br/>EmailIdentity:  The verified email identity that you want to set up the custom MAIL FROM domain for.<br/>
        :return:         """
        pass

    def send_email(self, Content: EmailContent, Destination: Destination, FromEmailAddress: str=None, ReplyToAddresses: List[EmailAddress]=None, FeedbackForwardingEmailAddress: str=None, EmailTags: List[MessageTag]=None, ConfigurationSetName: str=None) -> SendEmailResponse:
        """Sends an email message. You can use the Amazon Pinpoint Email API to send two types of messages: <ul> <li>  <b>Simple</b> – A standard email message. When you create this type of message, you specify the sender, the recipient, and the message body, and Amazon Pinpoint assembles the message for you. </li> <li>  <b>Raw</b> – A raw, MIME-formatted email message. When you send this type of email, you have to specify all of the message headers, as well as the message body. You can use this message type to send messages that contain attachments. The message that you specify has to be a valid MIME message. </li> </ul>

        :param self:
        :param ConfigurationSetName:  The name of the configuration set that you want to use when sending the email.<br/>EmailTags:  A list of tags, in the form of name/value pairs, to apply to an email that you send using the <code>SendEmail</code> operation. Tags correspond to characteristics of the email that you define, so that you can publish email sending events. <br/>Content:  An object that contains the body of the message. You can send either a Simple message or a Raw message.:<br/>FeedbackForwardingEmailAddress:  The address that Amazon Pinpoint should send bounce and complaint notifications to.<br/>ReplyToAddresses:  The "Reply-to" email addresses for the message. When the recipient replies to the message, each Reply-to address receives the reply.<br/>Destination:  An object that contains the recipients of the email message.:<br/>FromEmailAddress:  The email address that you want to use as the "From" address for the email. The address that you specify has to be verified. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BccAddresses</i> An array that contains the email addresses of the "BCC" (blind carbon copy) recipients for the email.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CcAddresses</i> An array that contains the email addresses of the "CC" (carbon copy) recipients for the email.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ToAddresses</i> An array that contains the email addresses of the "To" recipients for the email.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Raw</i> The raw email message. The message has to meet the following criteria: <ul> <li> The message has to contain a header and a body, separated by one blank line. </li> <li> All of the required header fields must be present in the message. </li> <li> Each part of a multipart MIME message must be formatted properly. </li> <li> If you include attachments, they must be in a file format that Amazon Pinpoint supports.  </li> <li> The entire message must be Base64 encoded. </li> <li> If any of the MIME parts in your message contain content that is outside of the 7-bit ASCII character range, you should encode that content to ensure that recipients' email clients render the message properly. </li> <li> The length of any single line of text in the message can't exceed 1,000 characters. This restriction is defined in <a href="https://tools.ietf.org/html/rfc5321">RFC 5321</a>. </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Raw</b>: The raw email message. The message has to meet the following criteria: <ul> <li> The message has to contain a header and a body, separated by one blank line. </li> <li> All of the required header fields must be present in the message. </li> <li> Each part of a multipart MIME message must be formatted properly. </li> <li> If you include attachments, they must be in a file format that Amazon Pinpoint supports.  </li> <li> The entire message must be Base64 encoded. </li> <li> If any of the MIME parts in your message contain content that is outside of the 7-bit ASCII character range, you should encode that content to ensure that recipients' email clients render the message properly. </li> <li> The length of any single line of text in the message can't exceed 1,000 characters. This restriction is defined in <a href="https://tools.ietf.org/html/rfc5321">RFC 5321</a>. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Simple</i> The simple email message. The message consists of a subject and a message body.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Simple</b>: The simple email message. The message consists of a subject and a message body.<br/>The subject line of the email. The subject line can only contain 7-bit ASCII characters. However, you can specify non-ASCII characters in the subject line by using encoded-word syntax, as described in <a href="https://tools.ietf.org/html/rfc2047">RFC 2047</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Subject</b>: The subject line of the email. The subject line can only contain 7-bit ASCII characters. However, you can specify non-ASCII characters in the subject line by using encoded-word syntax, as described in <a href="https://tools.ietf.org/html/rfc2047">RFC 2047</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Data</i> The raw email message. The message has to meet the following criteria: <ul> <li> The message has to contain a header and a body, separated by one blank line. </li> <li> All of the required header fields must be present in the message. </li> <li> Each part of a multipart MIME message must be formatted properly. </li> <li> Attachments must be in a file format that Amazon Pinpoint supports.  </li> <li> The entire message must be Base64 encoded. </li> <li> If any of the MIME parts in your message contain content that is outside of the 7-bit ASCII character range, you should encode that content to ensure that recipients' email clients render the message properly. </li> <li> The length of any single line of text in the message can't exceed 1,000 characters. This restriction is defined in <a href="https://tools.ietf.org/html/rfc5321">RFC 5321</a>. </li> </ul><br/>
        :return: MessageId:  A unique identifier for the message that is generated when Amazon Pinpoint accepts the message. <note> It is possible for Amazon Pinpoint to accept a message without sending it. This can happen when the message you're trying to send has an attachment doesn't pass a virus check, or when you send a templated email that contains invalid personalization content, for example. </note><br/>        """
        pass

    def tag_resource(self, Tags: List[Tag], ResourceArn: str) -> TagResourceResponse:
        """Add one or more tags (keys and values) to a specified resource. A <i>tag</i> is a label that you optionally define and associate with a resource in Amazon Pinpoint. Tags can help you categorize and manage resources in different ways, such as by purpose, owner, environment, or other criteria. A resource can have as many as 50 tags. Each tag consists of a required <i>tag key</i> and an associated <i>tag value</i>, both of which you define. A tag key is a general label that acts as a category for more specific tag values. A tag value acts as a descriptor within a tag key.

        :param self:
        :param Tags:  A list of the tags that you want to add to the resource. A tag consists of a required tag key (<code>Key</code>) and an associated tag value (<code>Value</code>). The maximum length of a tag key is 128 characters. The maximum length of a tag value is 256 characters.<br/>ResourceArn:  The Amazon Resource Name (ARN) of the resource that you want to add one or more tags to.<br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[TagKey], ResourceArn: str) -> UntagResourceResponse:
        """Remove one or more tags (keys and values) from a specified resource.

        :param self:
        :param TagKeys:  The tags (tag keys) that you want to remove from the resource. When you specify a tag key, the action removes both that key and its associated tag value. To remove more than one tag from the resource, append the <code>TagKeys</code> parameter and argument for each additional tag to remove, separated by an ampersand. For example: <code>/v1/email/tags?ResourceArn=ResourceArn&amp;TagKeys=Key1&amp;TagKeys=Key2</code> <br/>ResourceArn:  The Amazon Resource Name (ARN) of the resource that you want to remove one or more tags from.<br/>
        :return:         """
        pass

    def update_configuration_set_event_destination(self, EventDestination: EventDestinationDefinition, EventDestinationName: str, ConfigurationSetName: str) -> UpdateConfigurationSetEventDestinationResponse:
        """Update the configuration of an event destination for a configuration set. In Amazon Pinpoint, <i>events</i> include message sends, deliveries, opens, clicks, bounces, and complaints. <i>Event destinations</i> are places that you can send information about these events to. For example, you can send event data to Amazon SNS to receive notifications when you receive bounces or complaints, or you can use Amazon Kinesis Data Firehose to stream data to Amazon S3 for long-term storage.

        :param self:
        :param EventDestination:  An object that defines the event destination.:<br/>EventDestinationName:  The name of the event destination that you want to modify.<br/>ConfigurationSetName:  The name of the configuration set that contains the event destination that you want to modify.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PinpointDestination</i> An object that defines a Amazon Pinpoint destination for email events. You can use Amazon Pinpoint events to create attributes in Amazon Pinpoint projects. You can use these attributes to create segments for your campaigns.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PinpointDestination</b>: An object that defines a Amazon Pinpoint destination for email events. You can use Amazon Pinpoint events to create attributes in Amazon Pinpoint projects. You can use these attributes to create segments for your campaigns.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsDestination</i> An object that defines an Amazon SNS destination for email events. You can use Amazon SNS to send notification when certain email events occur.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SnsDestination</b>: An object that defines an Amazon SNS destination for email events. You can use Amazon SNS to send notification when certain email events occur.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchDestination</i> An object that defines an Amazon CloudWatch destination for email events. You can use Amazon CloudWatch to monitor and gain insights on your email sending metrics.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CloudWatchDestination</b>: An object that defines an Amazon CloudWatch destination for email events. You can use Amazon CloudWatch to monitor and gain insights on your email sending metrics.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KinesisFirehoseDestination</i> An object that defines an Amazon Kinesis Data Firehose destination for email events. You can use Amazon Kinesis Data Firehose to stream data to other services, such as Amazon S3 and Amazon Redshift.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KinesisFirehoseDestination</b>: An object that defines an Amazon Kinesis Data Firehose destination for email events. You can use Amazon Kinesis Data Firehose to stream data to other services, such as Amazon S3 and Amazon Redshift.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MatchingEventTypes</i> An array that specifies which events Amazon Pinpoint should send to the destinations in this <code>EventDestinationDefinition</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> If <code>true</code>, the event destination is enabled. When the event destination is enabled, the specified event types are sent to the destinations in this <code>EventDestinationDefinition</code>. If <code>false</code>, the event destination is disabled. When the event destination is disabled, events aren't sent to the specified destinations.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DeliveryStreamArn</i> The Amazon Resource Name (ARN) of the Amazon Kinesis Data Firehose stream that Amazon Pinpoint sends email events to.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoleArn</i> The Amazon Resource Name (ARN) of the IAM role that Amazon Pinpoint uses when sending email events to the Amazon Kinesis Data Firehose stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DimensionConfigurations</i> An array of objects that define the dimensions to use when you send email events to Amazon CloudWatch.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicArn</i> The Amazon Resource Name (ARN) of the Amazon SNS topic that you want to publish email events to. For more information about Amazon SNS topics, see the <a href="https://docs.aws.amazon.com/sns/latest/dg/CreateTopic.html">Amazon SNS Developer Guide</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationArn</i> The Amazon Resource Name (ARN) of the Amazon Pinpoint project that you want to send email events to.<br/>
        :return:         """
        pass



class PinpointSMSVoice(BaseClient):
    """Pinpoint SMS and Voice Messaging public facing APIs"""



    class CreateConfigurationSetRequest(Mapping):
        pass
    
    class CreateConfigurationSetResponse(Mapping):
        pass
    
    class CreateConfigurationSetEventDestinationRequest(Mapping):
        pass
    
    class CreateConfigurationSetEventDestinationResponse(Mapping):
        pass
    
    class EventDestinationDefinition(Mapping):
        pass
    
    class DeleteConfigurationSetRequest(Mapping):
        pass
    
    class DeleteConfigurationSetResponse(Mapping):
        pass
    
    class DeleteConfigurationSetEventDestinationRequest(Mapping):
        pass
    
    class DeleteConfigurationSetEventDestinationResponse(Mapping):
        pass
    
    class GetConfigurationSetEventDestinationsRequest(Mapping):
        pass
    
    class GetConfigurationSetEventDestinationsResponse(Mapping):
        pass
    
    class SendVoiceMessageRequest(Mapping):
        pass
    
    class SendVoiceMessageResponse(Mapping):
        pass
    
    class VoiceMessageContent(Mapping):
        pass
    
    class UpdateConfigurationSetEventDestinationRequest(Mapping):
        pass
    
    class UpdateConfigurationSetEventDestinationResponse(Mapping):
        pass
    
    def create_configuration_set(self, ConfigurationSetName: str=None) -> CreateConfigurationSetResponse:
        """Create a new configuration set. After you create the configuration set, you can add one or more event destinations to it.

        :param self:
        :param ConfigurationSetName:  The name that you want to give the configuration set.<br/>
        :return:         """
        pass

    def create_configuration_set_event_destination(self, ConfigurationSetName: str, EventDestination: EventDestinationDefinition=None, EventDestinationName: str=None) -> CreateConfigurationSetEventDestinationResponse:
        """Create a new event destination in a configuration set.

        :param self:
        :param EventDestinationName:  A name that identifies the event destination.<br/>EventDestination:  An object that defines a single event destination.:<br/>ConfigurationSetName:  ConfigurationSetName<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsDestination</i> An object that contains information about an event destination that sends data to Amazon SNS.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SnsDestination</b>: An object that contains information about an event destination that sends data to Amazon SNS.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MatchingEventTypes</i> An array of EventDestination objects. Each EventDestination object includes ARNs and other information that define an event destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KinesisFirehoseDestination</i> An object that contains information about an event destination that sends data to Amazon Kinesis Data Firehose.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KinesisFirehoseDestination</b>: An object that contains information about an event destination that sends data to Amazon Kinesis Data Firehose.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Indicates whether or not the event destination is enabled. If the event destination is enabled, then Amazon Pinpoint sends response data to the specified event destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchLogsDestination</i> An object that contains information about an event destination that sends data to Amazon CloudWatch Logs.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CloudWatchLogsDestination</b>: An object that contains information about an event destination that sends data to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LogGroupArn</i> The name of the Amazon CloudWatch Log Group that you want to record events in.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoleArn</i> The Amazon Resource Name (ARN) of an Amazon Identity and Access Management (IAM) role that is able to write event data to an Amazon CloudWatch destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoleArn</i> The Amazon Resource Name (ARN) of the Amazon Kinesis Data Firehose destination that you want to use in the event destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DeliveryStreamArn</i> The Amazon Resource Name (ARN) of an IAM role that can write data to an Amazon Kinesis Data Firehose stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicArn</i> The Amazon Resource Name (ARN) of the Amazon SNS topic that you want to publish events to.<br/>
        :return:         """
        pass

    def delete_configuration_set(self, ConfigurationSetName: str) -> DeleteConfigurationSetResponse:
        """Deletes an existing configuration set.

        :param self:
        :param ConfigurationSetName:  ConfigurationSetName<br/>
        :return:         """
        pass

    def delete_configuration_set_event_destination(self, EventDestinationName: str, ConfigurationSetName: str) -> DeleteConfigurationSetEventDestinationResponse:
        """Deletes an event destination in a configuration set.

        :param self:
        :param EventDestinationName:  EventDestinationName<br/>ConfigurationSetName:  ConfigurationSetName<br/>
        :return:         """
        pass

    def get_configuration_set_event_destinations(self, ConfigurationSetName: str) -> GetConfigurationSetEventDestinationsResponse:
        """Obtain information about an event destination, including the types of events it reports, the Amazon Resource Name (ARN) of the destination, and the name of the event destination.

        :param self:
        :param ConfigurationSetName:  ConfigurationSetName<br/>
        :return: EventDestinations:  An array of EventDestination objects. Each EventDestination object includes ARNs and other information that define an event destination.<br/>        """
        pass

    def send_voice_message(self, CallerId: str=None, ConfigurationSetName: str=None, Content: VoiceMessageContent=None, DestinationPhoneNumber: str=None, OriginationPhoneNumber: str=None) -> SendVoiceMessageResponse:
        """Create a new voice message and send it to a recipient's phone number.

        :param self:
        :param OriginationPhoneNumber:  The phone number that Amazon Pinpoint should use to send the voice message. This isn't necessarily the phone number that appears on recipients' devices when they receive the message, because you can specify a CallerId parameter in the request.<br/>DestinationPhoneNumber:  The phone number that you want to send the voice message to.<br/>Content:  An object that contains a voice message and information about the recipient that you want to send it to.:<br/>ConfigurationSetName:  The name of the configuration set that you want to use to send the message.<br/>CallerId:  The phone number that appears on recipients' devices when they receive the message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SSMLMessage</i> An object that defines a message that contains SSML-formatted text.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SSMLMessage</b>: An object that defines a message that contains SSML-formatted text.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlainTextMessage</i> An object that defines a message that contains unformatted text.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PlainTextMessage</b>: An object that defines a message that contains unformatted text.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallInstructionsMessage</i> An object that defines a message that contains text formatted using Amazon Pinpoint Voice Instructions markup.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CallInstructionsMessage</b>: An object that defines a message that contains text formatted using Amazon Pinpoint Voice Instructions markup.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Text</i> The language to use when delivering the message. For a complete list of supported languages, see the Amazon Polly Developer Guide.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VoiceId</i> The name of the voice that you want to use to deliver the message. For a complete list of supported voices, see the Amazon Polly Developer Guide.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Text</i> The plain (not SSML-formatted) text to deliver to the recipient.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LanguageCode</i> The language to use when delivering the message. For a complete list of supported languages, see the Amazon Polly Developer Guide.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VoiceId</i> The name of the voice that you want to use to deliver the message. For a complete list of supported voices, see the Amazon Polly Developer Guide.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Text</i> The SSML-formatted text to deliver to the recipient.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LanguageCode</i> The language to use when delivering the message. For a complete list of supported languages, see the Amazon Polly Developer Guide.<br/>
        :return: MessageId:  A unique identifier for the voice message.<br/>        """
        pass

    def update_configuration_set_event_destination(self, EventDestinationName: str, ConfigurationSetName: str, EventDestination: EventDestinationDefinition=None) -> UpdateConfigurationSetEventDestinationResponse:
        """Update an event destination in a configuration set. An event destination is a location that you publish information about your voice calls to. For example, you can log an event to an Amazon CloudWatch destination when a call fails.

        :param self:
        :param EventDestinationName:  EventDestinationName<br/>EventDestination:  An object that defines a single event destination.:<br/>ConfigurationSetName:  ConfigurationSetName<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsDestination</i> An object that contains information about an event destination that sends data to Amazon SNS.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SnsDestination</b>: An object that contains information about an event destination that sends data to Amazon SNS.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MatchingEventTypes</i> An array of EventDestination objects. Each EventDestination object includes ARNs and other information that define an event destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KinesisFirehoseDestination</i> An object that contains information about an event destination that sends data to Amazon Kinesis Data Firehose.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KinesisFirehoseDestination</b>: An object that contains information about an event destination that sends data to Amazon Kinesis Data Firehose.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Indicates whether or not the event destination is enabled. If the event destination is enabled, then Amazon Pinpoint sends response data to the specified event destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchLogsDestination</i> An object that contains information about an event destination that sends data to Amazon CloudWatch Logs.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CloudWatchLogsDestination</b>: An object that contains information about an event destination that sends data to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LogGroupArn</i> The name of the Amazon CloudWatch Log Group that you want to record events in.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoleArn</i> The Amazon Resource Name (ARN) of an Amazon Identity and Access Management (IAM) role that is able to write event data to an Amazon CloudWatch destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoleArn</i> The Amazon Resource Name (ARN) of the Amazon Kinesis Data Firehose destination that you want to use in the event destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DeliveryStreamArn</i> The Amazon Resource Name (ARN) of an IAM role that can write data to an Amazon Kinesis Data Firehose stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicArn</i> The Amazon Resource Name (ARN) of the Amazon SNS topic that you want to publish events to.<br/>
        :return:         """
        pass



class Polly(BaseClient):
    """Amazon Polly is a web service that makes it easy to synthesize speech from text. The Amazon Polly service provides API operations for synthesizing high-quality speech from plain text and Speech Synthesis Markup Language (SSML), along with managing pronunciations lexicons that enable you to get the best results for your application domain."""


    class DescribeVoicesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_voices` for valid parameters"""
            pass
    class ListLexiconsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_lexicons` for valid parameters"""
            pass
    class ListSpeechSynthesisTasksPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_speech_synthesis_tasks` for valid parameters"""
            pass


    class DeleteLexiconInput(Mapping):
        pass
    
    class DeleteLexiconOutput(Mapping):
        pass
    
    class DescribeVoicesInput(Mapping):
        pass
    
    class DescribeVoicesOutput(Mapping):
        pass
    
    class IncludeAdditionalLanguageCodes(object):
        pass
    
    class GetLexiconInput(Mapping):
        pass
    
    class GetLexiconOutput(Mapping):
        pass
    
    class GetSpeechSynthesisTaskInput(Mapping):
        pass
    
    class GetSpeechSynthesisTaskOutput(Mapping):
        pass
    
    class ListLexiconsInput(Mapping):
        pass
    
    class ListLexiconsOutput(Mapping):
        pass
    
    class ListSpeechSynthesisTasksInput(Mapping):
        pass
    
    class ListSpeechSynthesisTasksOutput(Mapping):
        pass
    
    class PutLexiconInput(Mapping):
        pass
    
    class PutLexiconOutput(Mapping):
        pass
    
    class StartSpeechSynthesisTaskInput(Mapping):
        pass
    
    class StartSpeechSynthesisTaskOutput(Mapping):
        pass
    
    class LexiconName(object):
        pass
    
    class SpeechMarkType(object):
        pass
    
    class SynthesizeSpeechInput(Mapping):
        pass
    
    class SynthesizeSpeechOutput(Mapping):
        pass
    
    def delete_lexicon(self, Name: str) -> DeleteLexiconOutput:
        """Deletes the specified pronunciation lexicon stored in an AWS Region. A lexicon which has been deleted is not available for speech synthesis, nor is it possible to retrieve it using either the <code>GetLexicon</code> or <code>ListLexicon</code> APIs. For more information, see <a href="http://docs.aws.amazon.com/polly/latest/dg/managing-lexicons.html">Managing Lexicons</a>.

        :param self:
        :param Name:  The name of the lexicon to delete. Must be an existing lexicon in the region.<br/>
        :return:         """
        pass

    def describe_voices(self, LanguageCode: str=None, IncludeAdditionalLanguageCodes: IncludeAdditionalLanguageCodes=None, NextToken: str=None) -> DescribeVoicesOutput:
        """Returns the list of voices that are available for use when requesting speech synthesis. Each voice speaks a specified language, is either male or female, and is identified by an ID, which is the ASCII version of the voice name.  When synthesizing speech ( <code>SynthesizeSpeech</code> ), you provide the voice ID for the voice you want from the list of voices returned by <code>DescribeVoices</code>. For example, you want your news reader application to read news in a specific language, but giving a user the option to choose the voice. Using the <code>DescribeVoices</code> operation you can provide the user with a list of available voices to select from.  You can optionally specify a language code to filter the available voices. For example, if you specify <code>en-US</code>, the operation returns a list of all available US English voices.  This operation requires permissions to perform the <code>polly:DescribeVoices</code> action.

        :param self:
        :param NextToken:  An opaque pagination token returned from the previous <code>DescribeVoices</code> operation. If present, this indicates where to continue the listing.<br/>IncludeAdditionalLanguageCodes:  Boolean value indicating whether to return any bilingual voices that use the specified language as an additional language. For instance, if you request all languages that use US English (es-US), and there is an Italian voice that speaks both Italian (it-IT) and US English, that voice will be included if you specify <code>yes</code> but not if you specify <code>no</code>.<br/>LanguageCode:   The language identification tag (ISO 639 code for the language name-ISO 3166 country code) for filtering the list of voices returned. If you don't specify this optional parameter, all available voices are returned. <br/>
        :return: NextToken:  The pagination token to use in the next request to continue the listing of voices. <code>NextToken</code> is returned only if the response is truncated.<br/>Voices:  A list of voices with their properties.<br/>        """
        pass

    def get_lexicon(self, Name: str) -> GetLexiconOutput:
        """Returns the content of the specified pronunciation lexicon stored in an AWS Region. For more information, see <a href="http://docs.aws.amazon.com/polly/latest/dg/managing-lexicons.html">Managing Lexicons</a>.

        :param self:
        :param Name:  Name of the lexicon.<br/>
        :return: LexiconAttributes:  Metadata of the lexicon, including phonetic alphabetic used, language code, lexicon ARN, number of lexemes defined in the lexicon, and size of lexicon in bytes.:<br/>Lexicon:  Lexicon object that provides name and the string content of the lexicon. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> Name of the lexicon.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Content</i> Lexicon content in string format. The content of a lexicon must be in PLS format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Size</i> Total size of the lexicon, in characters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LexemesCount</i> Number of lexemes in the lexicon.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LexiconArn</i> Amazon Resource Name (ARN) of the lexicon.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModified</i> Date lexicon was last modified (a timestamp value).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LanguageCode</i> Language code that the lexicon applies to. A lexicon with a language code such as "en" would be applied to all English languages (en-GB, en-US, en-AUS, en-WLS, and so on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Alphabet</i> Phonetic alphabet used in the lexicon. Valid values are <code>ipa</code> and <code>x-sampa</code>.<br/>        """
        pass

    def get_speech_synthesis_task(self, TaskId: str) -> GetSpeechSynthesisTaskOutput:
        """Retrieves a specific SpeechSynthesisTask object based on its TaskID. This object contains information about the given speech synthesis task, including the status of the task, and a link to the S3 bucket containing the output of the task.

        :param self:
        :param TaskId:  The Amazon Polly generated identifier for a speech synthesis task.<br/>
        :return: SynthesisTask:  SynthesisTask object that provides information from the requested task, including output format, creation time, task status, and so on.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LanguageCode</i> Optional language code for a synthesis task. This is only necessary if using a bilingual voice, such as Aditi, which can be used for either Indian English (en-IN) or Hindi (hi-IN).  If a bilingual voice is used and no language code is specified, Amazon Polly will use the default language of the bilingual voice. The default language for any voice is the one returned by the <a href="https://docs.aws.amazon.com/polly/latest/dg/API_DescribeVoices.html">DescribeVoices</a> operation for the <code>LanguageCode</code> parameter. For example, if no language code is specified, Aditi will use Indian English rather than Hindi.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VoiceId</i> Voice ID to use for the synthesis. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TextType</i> Specifies whether the input text is plain text or SSML. The default value is plain text. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SpeechMarkTypes</i> The type of speech marks returned for the input text.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SampleRate</i> The audio frequency specified in Hz. The valid values for mp3 and ogg_vorbis are "8000", "16000", and "22050". The default value is "22050". Valid values for pcm are "8000" and "16000" The default value is "16000". <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputFormat</i> The format in which the returned output will be encoded. For audio stream, this will be mp3, ogg_vorbis, or pcm. For speech marks, this will be json. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LexiconNames</i> List of one or more pronunciation lexicon names you want the service to apply during synthesis. Lexicons are applied only if the language of the lexicon is the same as the language of the voice. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsTopicArn</i> ARN for the SNS topic optionally used for providing status notification for a speech synthesis task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestCharacters</i> Number of billable characters synthesized.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> Timestamp for the time the synthesis task was started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputUri</i> Pathway for the output speech file.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TaskStatusReason</i> Reason for the current status of a specific speech synthesis task, including errors if the task has failed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TaskStatus</i> Current status of the individual speech synthesis task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TaskId</i> The Amazon Polly generated identifier for a speech synthesis task.<br/>        """
        pass

    def list_lexicons(self, NextToken: str=None) -> ListLexiconsOutput:
        """Returns a list of pronunciation lexicons stored in an AWS Region. For more information, see <a href="http://docs.aws.amazon.com/polly/latest/dg/managing-lexicons.html">Managing Lexicons</a>.

        :param self:
        :param NextToken:  An opaque pagination token returned from previous <code>ListLexicons</code> operation. If present, indicates where to continue the list of lexicons.<br/>
        :return: NextToken:  The pagination token to use in the next request to continue the listing of lexicons. <code>NextToken</code> is returned only if the response is truncated.<br/>Lexicons:  A list of lexicon names and attributes.<br/>        """
        pass

    def list_speech_synthesis_tasks(self, MaxResults: int=None, NextToken: str=None, Status: str=None) -> ListSpeechSynthesisTasksOutput:
        """Returns a list of SpeechSynthesisTask objects ordered by their creation date. This operation can filter the tasks by their status, for example, allowing users to list only tasks that are completed.

        :param self:
        :param Status:  Status of the speech synthesis tasks returned in a List operation<br/>NextToken:  The pagination token to use in the next request to continue the listing of speech synthesis tasks. <br/>MaxResults:  Maximum number of speech synthesis tasks returned in a List operation.<br/>
        :return: SynthesisTasks:  List of SynthesisTask objects that provides information from the specified task in the list request, including output format, creation time, task status, and so on.<br/>NextToken:  An opaque pagination token returned from the previous List operation in this request. If present, this indicates where to continue the listing.<br/>        """
        pass

    def put_lexicon(self, Content: str, Name: str) -> PutLexiconOutput:
        """Stores a pronunciation lexicon in an AWS Region. If a lexicon with the same name already exists in the region, it is overwritten by the new lexicon. Lexicon operations have eventual consistency, therefore, it might take some time before the lexicon is available to the SynthesizeSpeech operation. For more information, see <a href="http://docs.aws.amazon.com/polly/latest/dg/managing-lexicons.html">Managing Lexicons</a>.

        :param self:
        :param Content:  Content of the PLS lexicon as string data.<br/>Name:  Name of the lexicon. The name must follow the regular express format [0-9A-Za-z]{1,20}. That is, the name is a case-sensitive alphanumeric string up to 20 characters long. <br/>
        :return:         """
        pass

    def start_speech_synthesis_task(self, VoiceId: str, Text: str, OutputS3BucketName: str, OutputFormat: str, LexiconNames: List[LexiconName]=None, OutputS3KeyPrefix: str=None, SampleRate: str=None, SnsTopicArn: str=None, SpeechMarkTypes: List[SpeechMarkType]=None, TextType: str=None, LanguageCode: str=None) -> StartSpeechSynthesisTaskOutput:
        """Allows the creation of an asynchronous synthesis task, by starting a new <code>SpeechSynthesisTask</code>. This operation requires all the standard information needed for speech synthesis, plus the name of an Amazon S3 bucket for the service to store the output of the synthesis task and two optional parameters (OutputS3KeyPrefix and SnsTopicArn). Once the synthesis task is created, this operation will return a SpeechSynthesisTask object, which will include an identifier of this task as well as the current status.

        :param self:
        :param LanguageCode:  Optional language code for the Speech Synthesis request. This is only necessary if using a bilingual voice, such as Aditi, which can be used for either Indian English (en-IN) or Hindi (hi-IN).  If a bilingual voice is used and no language code is specified, Amazon Polly will use the default language of the bilingual voice. The default language for any voice is the one returned by the <a href="https://docs.aws.amazon.com/polly/latest/dg/API_DescribeVoices.html">DescribeVoices</a> operation for the <code>LanguageCode</code> parameter. For example, if no language code is specified, Aditi will use Indian English rather than Hindi.<br/>VoiceId:  Voice ID to use for the synthesis. <br/>TextType:  Specifies whether the input text is plain text or SSML. The default value is plain text. <br/>Text:  The input text to synthesize. If you specify ssml as the TextType, follow the SSML format for the input text. <br/>SpeechMarkTypes:  The type of speech marks returned for the input text.<br/>SnsTopicArn:  ARN for the SNS topic optionally used for providing status notification for a speech synthesis task.<br/>SampleRate:  The audio frequency specified in Hz. The valid values for mp3 and ogg_vorbis are "8000", "16000", and "22050". The default value is "22050". Valid values for pcm are "8000" and "16000" The default value is "16000". <br/>OutputS3KeyPrefix:  The Amazon S3 key prefix for the output speech file.<br/>OutputS3BucketName:  Amazon S3 bucket name to which the output file will be saved.<br/>OutputFormat:  The format in which the returned output will be encoded. For audio stream, this will be mp3, ogg_vorbis, or pcm. For speech marks, this will be json. <br/>LexiconNames:  List of one or more pronunciation lexicon names you want the service to apply during synthesis. Lexicons are applied only if the language of the lexicon is the same as the language of the voice. <br/>
        :return: SynthesisTask:  SynthesisTask object that provides information and attributes about a newly submitted speech synthesis task.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LanguageCode</i> Optional language code for a synthesis task. This is only necessary if using a bilingual voice, such as Aditi, which can be used for either Indian English (en-IN) or Hindi (hi-IN).  If a bilingual voice is used and no language code is specified, Amazon Polly will use the default language of the bilingual voice. The default language for any voice is the one returned by the <a href="https://docs.aws.amazon.com/polly/latest/dg/API_DescribeVoices.html">DescribeVoices</a> operation for the <code>LanguageCode</code> parameter. For example, if no language code is specified, Aditi will use Indian English rather than Hindi.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VoiceId</i> Voice ID to use for the synthesis. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TextType</i> Specifies whether the input text is plain text or SSML. The default value is plain text. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SpeechMarkTypes</i> The type of speech marks returned for the input text.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SampleRate</i> The audio frequency specified in Hz. The valid values for mp3 and ogg_vorbis are "8000", "16000", and "22050". The default value is "22050". Valid values for pcm are "8000" and "16000" The default value is "16000". <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputFormat</i> The format in which the returned output will be encoded. For audio stream, this will be mp3, ogg_vorbis, or pcm. For speech marks, this will be json. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LexiconNames</i> List of one or more pronunciation lexicon names you want the service to apply during synthesis. Lexicons are applied only if the language of the lexicon is the same as the language of the voice. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsTopicArn</i> ARN for the SNS topic optionally used for providing status notification for a speech synthesis task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestCharacters</i> Number of billable characters synthesized.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> Timestamp for the time the synthesis task was started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputUri</i> Pathway for the output speech file.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TaskStatusReason</i> Reason for the current status of a specific speech synthesis task, including errors if the task has failed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TaskStatus</i> Current status of the individual speech synthesis task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TaskId</i> The Amazon Polly generated identifier for a speech synthesis task.<br/>        """
        pass

    def synthesize_speech(self, VoiceId: str, Text: str, OutputFormat: str, LexiconNames: List[LexiconName]=None, SampleRate: str=None, SpeechMarkTypes: List[SpeechMarkType]=None, TextType: str=None, LanguageCode: str=None) -> SynthesizeSpeechOutput:
        """Synthesizes UTF-8 input, plain text or SSML, to a stream of bytes. SSML input must be valid, well-formed SSML. Some alphabets might not be available with all the voices (for example, Cyrillic might not be read at all by English voices) unless phoneme mapping is used. For more information, see <a href="http://docs.aws.amazon.com/polly/latest/dg/how-text-to-speech-works.html">How it Works</a>.

        :param self:
        :param LanguageCode:  Optional language code for the Synthesize Speech request. This is only necessary if using a bilingual voice, such as Aditi, which can be used for either Indian English (en-IN) or Hindi (hi-IN).  If a bilingual voice is used and no language code is specified, Amazon Polly will use the default language of the bilingual voice. The default language for any voice is the one returned by the <a href="https://docs.aws.amazon.com/polly/latest/dg/API_DescribeVoices.html">DescribeVoices</a> operation for the <code>LanguageCode</code> parameter. For example, if no language code is specified, Aditi will use Indian English rather than Hindi.<br/>VoiceId:   Voice ID to use for the synthesis. You can get a list of available voice IDs by calling the <a href="http://docs.aws.amazon.com/polly/latest/dg/API_DescribeVoices.html">DescribeVoices</a> operation. <br/>TextType:   Specifies whether the input text is plain text or SSML. The default value is plain text. For more information, see <a href="http://docs.aws.amazon.com/polly/latest/dg/ssml.html">Using SSML</a>.<br/>Text:   Input text to synthesize. If you specify <code>ssml</code> as the <code>TextType</code>, follow the SSML format for the input text. <br/>SpeechMarkTypes:  The type of speech marks returned for the input text.<br/>SampleRate:   The audio frequency specified in Hz.  The valid values for <code>mp3</code> and <code>ogg_vorbis</code> are "8000", "16000", and "22050". The default value is "22050".   Valid values for <code>pcm</code> are "8000" and "16000" The default value is "16000". <br/>OutputFormat:   The format in which the returned output will be encoded. For audio stream, this will be mp3, ogg_vorbis, or pcm. For speech marks, this will be json.  When pcm is used, the content returned is audio/pcm in a signed 16-bit, 1 channel (mono), little-endian format. <br/>LexiconNames:  List of one or more pronunciation lexicon names you want the service to apply during synthesis. Lexicons are applied only if the language of the lexicon is the same as the language of the voice. For information about storing lexicons, see <a href="http://docs.aws.amazon.com/polly/latest/dg/API_PutLexicon.html">PutLexicon</a>.<br/>
        :return: RequestCharacters:  Number of characters synthesized.<br/>ContentType:   Specifies the type audio stream. This should reflect the <code>OutputFormat</code> parameter in your request.  <ul> <li>  If you request <code>mp3</code> as the <code>OutputFormat</code>, the <code>ContentType</code> returned is audio/mpeg.  </li> <li>  If you request <code>ogg_vorbis</code> as the <code>OutputFormat</code>, the <code>ContentType</code> returned is audio/ogg.  </li> <li>  If you request <code>pcm</code> as the <code>OutputFormat</code>, the <code>ContentType</code> returned is audio/pcm in a signed 16-bit, 1 channel (mono), little-endian format.  </li> <li> If you request <code>json</code> as the <code>OutputFormat</code>, the <code>ContentType</code> returned is audio/json. </li> </ul>  <br/>AudioStream:   Stream containing the synthesized speech. <br/>        """
        pass



class Pricing(BaseClient):
    """AWS Price List Service API (AWS Price List Service) is a centralized and convenient way to programmatically query Amazon Web Services for services, products, and pricing information. The AWS Price List Service uses standardized product attributes such as <code>Location</code>, <code>Storage Class</code>, and <code>Operating System</code>, and provides prices at the SKU level. You can use the AWS Price List Service to build cost control and scenario planning tools, reconcile billing data, forecast future spend for budgeting purposes, and provide cost benefit analysis that compare your internal workloads with AWS. Use <code>GetServices</code> without a service code to retrieve the service codes for all AWS services, then <code>GetServices</code> with a service code to retreive the attribute names for that service. After you have the service code and attribute names, you can use <code>GetAttributeValues</code> to see what values are available for an attribute. With the service code and an attribute name and value, you can use <code>GetProducts</code> to find specific products that you're interested in, such as an <code>AmazonEC2</code> instance, with a <code>Provisioned IOPS</code> <code>volumeType</code>. Service Endpoint AWS Price List Service API provides the following two endpoints: <ul> <li> https://api.pricing.us-east-1.amazonaws.com </li> <li> https://api.pricing.ap-south-1.amazonaws.com </li> </ul>"""


    class DescribeServicesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_services` for valid parameters"""
            pass
    class GetAttributeValuesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_attribute_values` for valid parameters"""
            pass
    class GetProductsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_products` for valid parameters"""
            pass


    class DescribeServicesRequest(Mapping):
        pass
    
    class DescribeServicesResponse(Mapping):
        pass
    
    class GetAttributeValuesRequest(Mapping):
        pass
    
    class GetAttributeValuesResponse(Mapping):
        pass
    
    class GetProductsRequest(Mapping):
        pass
    
    class GetProductsResponse(Mapping):
        pass
    
    class Filter(Mapping):
        pass
    
    def describe_services(self, ServiceCode: str=None, FormatVersion: str=None, NextToken: str=None, MaxResults: int=None) -> DescribeServicesResponse:
        """Returns the metadata for one service or a list of the metadata for all services. Use this without a service code to get the service codes for all services. Use it with a service code, such as <code>AmazonEC2</code>, to get information specific to that service, such as the attribute names available for that service. For example, some of the attribute names available for EC2 are <code>volumeType</code>, <code>maxIopsVolume</code>, <code>operation</code>, <code>locationType</code>, and <code>instanceCapacity10xlarge</code>.

        :param self:
        :param MaxResults:  The maximum number of results that you want returned in the response.<br/>NextToken:  The pagination token that indicates the next set of results that you want to retrieve.<br/>FormatVersion:  The format version that you want the response to be in. Valid values are: <code>aws_v1</code> <br/>ServiceCode:  The code for the service whose information you want to retrieve, such as <code>AmazonEC2</code>. You can use the <code>ServiceCode</code> to filter the results in a <code>GetProducts</code> call. To retrieve a list of all services, leave this blank.<br/>
        :return: NextToken:  The pagination token for the next set of retreivable results.<br/>FormatVersion:  The format version of the response. For example, <code>aws_v1</code>.<br/>Services:  The service metadata for the service or services in the response.<br/>        """
        pass

    def get_attribute_values(self, AttributeName: str, ServiceCode: str, NextToken: str=None, MaxResults: int=None) -> GetAttributeValuesResponse:
        """Returns a list of attribute values. Attibutes are similar to the details in a Price List API offer file. For a list of available attributes, see <a href="http://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/reading-an-offer.html#pps-defs">Offer File Definitions</a> in the <a href="http://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/billing-what-is.html">AWS Billing and Cost Management User Guide</a>.

        :param self:
        :param MaxResults:  The maximum number of results to return in response.<br/>NextToken:  The pagination token that indicates the next set of results that you want to retrieve.<br/>AttributeName:  The name of the attribute that you want to retrieve the values for, such as <code>volumeType</code>.<br/>ServiceCode:  The service code for the service whose attributes you want to retrieve. For example, if you want the retrieve an EC2 attribute, use <code>AmazonEC2</code>.<br/>
        :return: NextToken:  The pagination token that indicates the next set of results to retrieve.<br/>AttributeValues:  The list of values for an attribute. For example, <code>Throughput Optimized HDD</code> and <code>Provisioned IOPS</code> are two available values for the <code>AmazonEC2</code> <code>volumeType</code>.<br/>        """
        pass

    def get_products(self, ServiceCode: str=None, Filters: List[Filter]=None, FormatVersion: str=None, NextToken: str=None, MaxResults: int=None) -> GetProductsResponse:
        """Returns a list of all products that match the filter criteria.

        :param self:
        :param MaxResults:  The maximum number of results to return in the response.<br/>NextToken:  The pagination token that indicates the next set of results that you want to retrieve.<br/>FormatVersion:  The format version that you want the response to be in. Valid values are: <code>aws_v1</code> <br/>Filters:  The list of filters that limit the returned products. only products that match all filters are returned.<br/>ServiceCode:  The code for the service whose products you want to retrieve. <br/>
        :return: NextToken:  The pagination token that indicates the next set of results to retrieve.<br/>PriceList:  The list of products that match your filters. The list contains both the product metadata and the price information.<br/>FormatVersion:  The format version of the response. For example, aws_v1.<br/>        """
        pass



class QuickSight(BaseClient):
    """<fullname>Amazon QuickSight API Reference</fullname> Amazon QuickSight is a fully managed, serverless, cloud business intelligence service that makes it easy to extend data and insights to every user in your organization. This API interface reference contains documentation for a programming interface that you can use to manage Amazon QuickSight. """




    class CreateGroupRequest(Mapping):
        pass
    
    class CreateGroupResponse(Mapping):
        pass
    
    class CreateGroupMembershipRequest(Mapping):
        pass
    
    class CreateGroupMembershipResponse(Mapping):
        pass
    
    class DeleteGroupRequest(Mapping):
        pass
    
    class DeleteGroupResponse(Mapping):
        pass
    
    class DeleteGroupMembershipRequest(Mapping):
        pass
    
    class DeleteGroupMembershipResponse(Mapping):
        pass
    
    class DeleteUserRequest(Mapping):
        pass
    
    class DeleteUserResponse(Mapping):
        pass
    
    class DeleteUserByPrincipalIdRequest(Mapping):
        pass
    
    class DeleteUserByPrincipalIdResponse(Mapping):
        pass
    
    class DescribeGroupRequest(Mapping):
        pass
    
    class DescribeGroupResponse(Mapping):
        pass
    
    class DescribeUserRequest(Mapping):
        pass
    
    class DescribeUserResponse(Mapping):
        pass
    
    class GetDashboardEmbedUrlRequest(Mapping):
        pass
    
    class GetDashboardEmbedUrlResponse(Mapping):
        pass
    
    class SessionLifetimeInMinutes(object):
        pass
    
    class boolean(object):
        pass
    
    class ListGroupMembershipsRequest(Mapping):
        pass
    
    class ListGroupMembershipsResponse(Mapping):
        pass
    
    class ListGroupsRequest(Mapping):
        pass
    
    class ListGroupsResponse(Mapping):
        pass
    
    class ListUserGroupsRequest(Mapping):
        pass
    
    class ListUserGroupsResponse(Mapping):
        pass
    
    class ListUsersRequest(Mapping):
        pass
    
    class ListUsersResponse(Mapping):
        pass
    
    class RegisterUserRequest(Mapping):
        pass
    
    class RegisterUserResponse(Mapping):
        pass
    
    class UpdateGroupRequest(Mapping):
        pass
    
    class UpdateGroupResponse(Mapping):
        pass
    
    class UpdateUserRequest(Mapping):
        pass
    
    class UpdateUserResponse(Mapping):
        pass
    
    def create_group(self, Namespace: str, AwsAccountId: str, GroupName: str, Description: str=None) -> CreateGroupResponse:
        """Creates an Amazon QuickSight group. The permissions resource is <code>arn:aws:quicksight:us-east-1:<i>&lt;relevant-aws-account-id&gt;</i>:group/default/<i>&lt;group-name&gt;</i> </code>. The response is a group object.  <b>CLI Sample:</b>   <code>aws quicksight create-group --aws-account-id=111122223333 --namespace=default --group-name="Sales-Management" --description="Sales Management - Forecasting" </code> 

        :param self:
        :param Namespace:  The namespace. Currently, you should set this to <code>default</code>.<br/>AwsAccountId:  The ID for the AWS account that the group is in. Currently, you use the ID for the AWS account that contains your Amazon QuickSight account.<br/>Description:  A description for the group that you want to create.<br/>GroupName:  A name for the group that you want to create.<br/>
        :return: Status:  The http status of the request.<br/>RequestId:  The AWS request ID for this operation.<br/>Group:  The name of the group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrincipalId</i> The principal ID of the group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The group description.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupName</i> The name of the group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) for the group.<br/>        """
        pass

    def create_group_membership(self, Namespace: str, AwsAccountId: str, GroupName: str, MemberName: str) -> CreateGroupMembershipResponse:
        """Adds an Amazon QuickSight user to an Amazon QuickSight group.  The permissions resource is <code>arn:aws:quicksight:us-east-1:<i>&lt;aws-account-id&gt;</i>:group/default/<i>&lt;group-name&gt;</i> </code>. The condition resource is the user name. The condition key is <code>quicksight:UserName</code>. The response is the group member object.  <b>CLI Sample:</b>   <code>aws quicksight create-group-membership --aws-account-id=111122223333 --namespace=default --group-name=Sales --member-name=Pat </code> 

        :param self:
        :param Namespace:  The namespace. Currently, you should set this to <code>default</code>.<br/>AwsAccountId:  The ID for the AWS account that the group is in. Currently, you use the ID for the AWS account that contains your Amazon QuickSight account.<br/>GroupName:  The name of the group that you want to add the user to.<br/>MemberName:  The name of the user that you want to add to the group membership.<br/>
        :return: Status:  The http status of the request.<br/>RequestId:  The AWS request ID for this operation.<br/>GroupMember:  The group member.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MemberName</i> The name of the group member (user).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) for the group member (user).<br/>        """
        pass

    def delete_group(self, Namespace: str, AwsAccountId: str, GroupName: str) -> DeleteGroupResponse:
        """Removes a user group from Amazon QuickSight.  The permissions resource is <code>arn:aws:quicksight:us-east-1:<i>&lt;aws-account-id&gt;</i>:group/default/<i>&lt;group-name&gt;</i> </code>.  <b>CLI Sample:</b>   <code>aws quicksight delete-group -\-aws-account-id=111122223333 -\-namespace=default -\-group-name=Sales-Management </code> 

        :param self:
        :param Namespace:  The namespace. Currently, you should set this to <code>default</code>.<br/>AwsAccountId:  The ID for the AWS account that the group is in. Currently, you use the ID for the AWS account that contains your Amazon QuickSight account.<br/>GroupName:  The name of the group that you want to delete.<br/>
        :return: Status:  The http status of the request.<br/>RequestId:  The AWS request ID for this operation.<br/>        """
        pass

    def delete_group_membership(self, Namespace: str, AwsAccountId: str, GroupName: str, MemberName: str) -> DeleteGroupMembershipResponse:
        """Removes a user from a group so that the user is no longer a member of the group. The permissions resource is <code>arn:aws:quicksight:us-east-1:<i>&lt;aws-account-id&gt;</i>:group/default/<i>&lt;group-name&gt;</i> </code>. The condition resource is the user name. The condition key is <code>quicksight:UserName</code>.  <b>CLI Sample:</b>   <code>aws quicksight delete-group-membership --aws-account-id=111122223333 --namespace=default --group-name=Sales-Management --member-name=Charlie </code> 

        :param self:
        :param Namespace:  The namespace. Currently, you should set this to <code>default</code>.<br/>AwsAccountId:  The ID for the AWS account that the group is in. Currently, you use the ID for the AWS account that contains your Amazon QuickSight account.<br/>GroupName:  The name of the group that you want to delete the user from.<br/>MemberName:  The name of the user that you want to delete from the group membership.<br/>
        :return: Status:  The http status of the request.<br/>RequestId:  The AWS request ID for this operation.<br/>        """
        pass

    def delete_user(self, Namespace: str, AwsAccountId: str, UserName: str) -> DeleteUserResponse:
        """Deletes the Amazon QuickSight user that is associated with the identity of the AWS Identity and Access Management (IAM) user or role that's making the call. The IAM user isn't deleted as a result of this call.  The permission resource is <code>arn:aws:quicksight:us-east-1:<i>&lt;aws-account-id&gt;</i>:user/default/<i>&lt;user-name&gt; </i> </code>.  <b>CLI Sample:</b>   <code>aws quicksight delete-user --aws-account-id=111122223333 --namespace=default --user-name=Pat </code> 

        :param self:
        :param Namespace:  The namespace. Currently, you should set this to <code>default</code>.<br/>AwsAccountId:  The ID for the AWS account that the user is in. Currently, you use the ID for the AWS account that contains your Amazon QuickSight account.<br/>UserName:  The name of the user that you want to delete.<br/>
        :return: Status:  The http status of the request.<br/>RequestId:  The AWS request ID for this operation.<br/>        """
        pass

    def delete_user_by_principal_id(self, Namespace: str, AwsAccountId: str, PrincipalId: str) -> DeleteUserByPrincipalIdResponse:
        """Deletes a user identified by its principal ID.  The permission resource is <code>arn:aws:quicksight:us-east-1:<i>&lt;aws-account-id&gt;</i>:user/default/<i>&lt;user-name&gt; </i> </code>.  <b>CLI Sample:</b>   <code>aws quicksight delete-user-by-principal-id --aws-account-id=111122223333 --namespace=default --principal-id=ABCDEFJA26JLI7EUUOEHS </code> 

        :param self:
        :param Namespace:  The namespace. Currently, you should set this to <code>default</code>.<br/>AwsAccountId:  The ID for the AWS account that the user is in. Currently, you use the ID for the AWS account that contains your Amazon QuickSight account.<br/>PrincipalId:  The principal ID of the user.<br/>
        :return: Status:  The http status of the request.<br/>RequestId:  The AWS request ID for this operation.<br/>        """
        pass

    def describe_group(self, Namespace: str, AwsAccountId: str, GroupName: str) -> DescribeGroupResponse:
        """Returns an Amazon QuickSight group's description and Amazon Resource Name (ARN).  The permissions resource is <code>arn:aws:quicksight:us-east-1:<i>&lt;relevant-aws-account-id&gt;</i>:group/default/<i>&lt;group-name&gt;</i> </code>. The response is the group object.   <b>CLI Sample:</b>   <code>aws quicksight describe-group -\-aws-account-id=11112222333 -\-namespace=default -\-group-name=Sales </code> 

        :param self:
        :param Namespace:  The namespace. Currently, you should set this to <code>default</code>.<br/>AwsAccountId:  The ID for the AWS account that the group is in. Currently, you use the ID for the AWS account that contains your Amazon QuickSight account.<br/>GroupName:  The name of the group that you want to describe.<br/>
        :return: Status:  The http status of the request.<br/>RequestId:  The AWS request ID for this operation.<br/>Group:  The name of the group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrincipalId</i> The principal ID of the group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The group description.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupName</i> The name of the group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) for the group.<br/>        """
        pass

    def describe_user(self, Namespace: str, AwsAccountId: str, UserName: str) -> DescribeUserResponse:
        """Returns information about a user, given the user name.  The permission resource is <code>arn:aws:quicksight:us-east-1:<i>&lt;aws-account-id&gt;</i>:user/default/<i>&lt;user-name&gt;</i> </code>.  The response is a user object that contains the user's Amazon Resource Name (ARN), AWS Identity and Access Management (IAM) role, and email address.   <b>CLI Sample:</b>   <code>aws quicksight describe-user --aws-account-id=111122223333 --namespace=default --user-name=Pat </code> 

        :param self:
        :param Namespace:  The namespace. Currently, you should set this to <code>default</code>.<br/>AwsAccountId:  The ID for the AWS account that the user is in. Currently, you use the ID for the AWS account that contains your Amazon QuickSight account.<br/>UserName:  The name of the user that you want to describe.<br/>
        :return: Status:  The http status of the request.<br/>RequestId:  The AWS request ID for this operation.<br/>User:  The user name.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrincipalId</i> The principal ID of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Active</i> Active status of user. When you create an Amazon QuickSight user that’s not an IAM user or an AD user, that user is inactive until they sign in and provide a password<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IdentityType</i> The type of identity authentication used by the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Role</i> The Amazon QuickSight role for the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Email</i> The user's email address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserName</i> The user's user name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) for the user.<br/>        """
        pass

    def get_dashboard_embed_url(self, IdentityType: str, DashboardId: str, AwsAccountId: str, SessionLifetimeInMinutes: SessionLifetimeInMinutes=None, UndoRedoDisabled: boolean=None, ResetDisabled: boolean=None, UserArn: str=None) -> GetDashboardEmbedUrlResponse:
        """Generates a server-side embeddable URL and authorization code. Before this can work properly, first you need to configure the dashboards and user permissions. For more information, see <a href="https://docs.aws.amazon.com/en_us/quicksight/latest/user/embedding.html"> Embedding Amazon QuickSight Dashboards</a>. Currently, you can use <code>GetDashboardEmbedURL</code> only from the server, not from the user’s browser.  <b>CLI Sample:</b>  Assume the role with permissions enabled for actions: <code>quickSight:RegisterUser</code> and <code>quicksight:GetDashboardEmbedURL</code>. You can use assume-role, assume-role-with-web-identity, or assume-role-with-saml.   <code>aws sts assume-role --role-arn "arn:aws:iam::111122223333:role/embedding_quicksight_dashboard_role" --role-session-name embeddingsession</code>  If the user does not exist in QuickSight, register the user:  <code>aws quicksight register-user --aws-account-id 111122223333 --namespace default --identity-type IAM --iam-arn "arn:aws:iam::111122223333:role/embedding_quicksight_dashboard_role" --user-role READER --session-name "embeddingsession" --email user123@example.com --region us-east-1</code>  Get the URL for the embedded dashboard  <code>aws quicksight get-dashboard-embed-url --aws-account-id 111122223333 --dashboard-id 1a1ac2b2-3fc3-4b44-5e5d-c6db6778df89 --identity-type IAM</code> 

        :param self:
        :param UserArn:  The Amazon QuickSight user's ARN, for use with <code>QUICKSIGHT</code> identity type. You can use this for any of the following: <ul> <li> Amazon QuickSight users in your account (readers, authors, or admins) </li> <li> AD users </li> <li> Invited non-federated users </li> <li> Federated IAM users </li> <li> Federated IAM role-based sessions </li> </ul><br/>ResetDisabled:  Remove the reset button on embedded dashboard. The default is FALSE, which allows the reset button.<br/>UndoRedoDisabled:  Remove the undo/redo button on embedded dashboard. The default is FALSE, which enables the undo/redo button.<br/>SessionLifetimeInMinutes:  How many minutes the session is valid. The session lifetime must be between 15 and 600 minutes.<br/>IdentityType:  The authentication method the user uses to sign in (IAM only).<br/>DashboardId:  The ID for the dashboard, also added to IAM policy<br/>AwsAccountId:  AWS account ID that contains the dashboard you are embedding.<br/>
        :return: RequestId:  The AWS request ID for this operation.<br/>Status:  The http status of the request.<br/>EmbedUrl:  URL that you can put into your server-side webpage to embed your dashboard. This URL is valid for 5 minutes, and the resulting session is valid for 10 hours. The API provides the URL with an auth_code that enables a single-signon session. <br/>        """
        pass

    def list_group_memberships(self, Namespace: str, AwsAccountId: str, GroupName: str, NextToken: str=None, MaxResults: int=None) -> ListGroupMembershipsResponse:
        """Lists member users in a group. The permissions resource is <code>arn:aws:quicksight:us-east-1:<i>&lt;aws-account-id&gt;</i>:group/default/<i>&lt;group-name&gt;</i> </code>. The response is a list of group member objects.  <b>CLI Sample:</b>   <code>aws quicksight list-group-memberships -\-aws-account-id=111122223333 -\-namespace=default </code> 

        :param self:
        :param Namespace:  The namespace. Currently, you should set this to <code>default</code>.<br/>AwsAccountId:  The ID for the AWS account that the group is in. Currently, you use the ID for the AWS account that contains your Amazon QuickSight account.<br/>MaxResults:  The maximum number of results to return from this request.<br/>NextToken:  A pagination token that can be used in a subsequent request.<br/>GroupName:  The name of the group that you want to see a membership list of.<br/>
        :return: Status:  The http status of the request.<br/>RequestId:  The AWS request ID for this operation.<br/>NextToken:  A pagination token that can be used in a subsequent request.<br/>GroupMemberList:  The list of the members of the group.<br/>        """
        pass

    def list_groups(self, Namespace: str, AwsAccountId: str, NextToken: str=None, MaxResults: int=None) -> ListGroupsResponse:
        """Lists all user groups in Amazon QuickSight.  The permissions resource is <code>arn:aws:quicksight:us-east-1:<i>&lt;aws-account-id&gt;</i>:group/default/*</code>. The response is a list of group objects.   <b>CLI Sample:</b>   <code>aws quicksight list-groups -\-aws-account-id=111122223333 -\-namespace=default </code> 

        :param self:
        :param Namespace:  The namespace. Currently, you should set this to <code>default</code>.<br/>MaxResults:  The maximum number of results to return.<br/>NextToken:  A pagination token that can be used in a subsequent request.<br/>AwsAccountId:  The ID for the AWS account that the group is in. Currently, you use the ID for the AWS account that contains your Amazon QuickSight account.<br/>
        :return: Status:  The http status of the request.<br/>RequestId:  The AWS request ID for this operation.<br/>NextToken:  A pagination token that can be used in a subsequent request.<br/>GroupList:  The list of the groups.<br/>        """
        pass

    def list_user_groups(self, Namespace: str, AwsAccountId: str, UserName: str, NextToken: str=None, MaxResults: int=None) -> ListUserGroupsResponse:
        """Lists the Amazon QuickSight groups that an Amazon QuickSight user is a member of. The permission resource is <code>arn:aws:quicksight:us-east-1:<i>&lt;aws-account-id&gt;</i>:user/default/<i>&lt;user-name&gt;</i> </code>.  The response is a one or more group objects.   <b>CLI Sample:</b>   <code>aws quicksight list-user-groups -\-user-name=Pat -\-aws-account-id=111122223333 -\-namespace=default -\-region=us-east-1 </code> 

        :param self:
        :param MaxResults:  The maximum number of results to return from this request.<br/>NextToken:  A pagination token that can be used in a subsequent request.<br/>Namespace:  The namespace. Currently, you should set this to <code>default</code>.<br/>AwsAccountId:  The AWS Account ID that the user is in. Currently, you use the ID for the AWS account that contains your Amazon QuickSight account.<br/>UserName:  The Amazon QuickSight user name that you want to list group memberships for.<br/>
        :return: Status:  The HTTP status of the request.<br/>RequestId:  The AWS request ID for this operation.<br/>NextToken:  A pagination token that can be used in a subsequent request.<br/>GroupList:  The list of groups the user is a member of.<br/>        """
        pass

    def list_users(self, Namespace: str, AwsAccountId: str, NextToken: str=None, MaxResults: int=None) -> ListUsersResponse:
        """Returns a list of all of the Amazon QuickSight users belonging to this account.  The permission resource is <code>arn:aws:quicksight:us-east-1:<i>&lt;aws-account-id&gt;</i>:user/default/<i>*</i> </code>. The response is a list of user objects, containing each user's Amazon Resource Name (ARN), AWS Identity and Access Management (IAM) role, and email address.   <b>CLI Sample:</b>   <code>aws quicksight list-users --aws-account-id=111122223333 --namespace=default </code> 

        :param self:
        :param Namespace:  The namespace. Currently, you should set this to <code>default</code>.<br/>MaxResults:  The maximum number of results to return from this request.<br/>NextToken:  A pagination token that can be used in a subsequent request.<br/>AwsAccountId:  The ID for the AWS account that the user is in. Currently, you use the ID for the AWS account that contains your Amazon QuickSight account.<br/>
        :return: Status:  The http status of the request.<br/>RequestId:  The AWS request ID for this operation.<br/>NextToken:  A pagination token that can be used in a subsequent request.<br/>UserList:  The list of users.<br/>        """
        pass

    def register_user(self, Namespace: str, AwsAccountId: str, UserRole: str, Email: str, IdentityType: str, IamArn: str=None, SessionName: str=None, UserName: str=None) -> RegisterUserResponse:
        """Creates an Amazon QuickSight user, whose identity is associated with the AWS Identity and Access Management (IAM) identity or role specified in the request.  The permission resource is <code>arn:aws:quicksight:us-east-1:<i>&lt;aws-account-id&gt;</i>:user/default/<i>&lt;user-name&gt;</i> </code>. The condition resource is the Amazon Resource Name (ARN) for the IAM user or role, and the session name.  The condition keys are <code>quicksight:IamArn</code> and <code>quicksight:SessionName</code>.   <b>CLI Sample:</b>   <code>aws quicksight register-user -\-aws-account-id=111122223333 -\-namespace=default -\-email=pat@example.com -\-identity-type=IAM -\-user-role=AUTHOR -\-iam-arn=arn:aws:iam::111122223333:user/Pat </code> 

        :param self:
        :param UserName:  The Amazon QuickSight user name that you want to create for the user you are registering.<br/>Namespace:  The namespace. Currently, you should set this to <code>default</code>.<br/>AwsAccountId:  The ID for the AWS account that the user is in. Currently, you use the ID for the AWS account that contains your Amazon QuickSight account.<br/>SessionName:  You need to use this parameter only when you register one or more users using an assumed IAM role. You don't need to provide the session name for other scenarios, for example when you are registering an IAM user or an Amazon QuickSight user. You can register multiple users using the same IAM role if each user has a different session name. For more information on assuming IAM roles, see <a href="https://docs.aws.amazon.com/cli/latest/reference/sts/assume-role.html"> <code>assume-role</code> </a> in the <i>AWS CLI Reference.</i> <br/>IamArn:  The ARN of the IAM user or role that you are registering with Amazon QuickSight. <br/>UserRole:  The Amazon QuickSight role of the user. The user role can be one of the following: <ul> <li>  <code>READER</code>: A user who has read-only access to dashboards. </li> <li>  <code>AUTHOR</code>: A user who can create data sources, data sets, analyses, and dashboards. </li> <li>  <code>ADMIN</code>: A user who is an author, who can also manage Amazon QuickSight settings. </li> </ul><br/>Email:  The email address of the user that you want to register.<br/>IdentityType:  Amazon QuickSight supports several ways of managing the identity of users. This parameter accepts two values: <ul> <li>  <code>IAM</code>: A user whose identity maps to an existing IAM user or role.  </li> <li>  <code>QUICKSIGHT</code>: A user whose identity is owned and managed internally by Amazon QuickSight.  </li> </ul><br/>
        :return: Status:  The http status of the request.<br/>RequestId:  The AWS request ID for this operation.<br/>UserInvitationUrl:  The URL the user visits to complete registration and provide a password. This is returned only for users with an identity type of <code>QUICKSIGHT</code>.<br/>User:  The user name.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrincipalId</i> The principal ID of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Active</i> Active status of user. When you create an Amazon QuickSight user that’s not an IAM user or an AD user, that user is inactive until they sign in and provide a password<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IdentityType</i> The type of identity authentication used by the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Role</i> The Amazon QuickSight role for the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Email</i> The user's email address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserName</i> The user's user name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) for the user.<br/>        """
        pass

    def update_group(self, Namespace: str, AwsAccountId: str, GroupName: str, Description: str=None) -> UpdateGroupResponse:
        """Changes a group description.  The permissions resource is <code>arn:aws:quicksight:us-east-1:<i>&lt;aws-account-id&gt;</i>:group/default/<i>&lt;group-name&gt;</i> </code>. The response is a group object.  <b>CLI Sample:</b>   <code>aws quicksight update-group --aws-account-id=111122223333 --namespace=default --group-name=Sales --description="Sales BI Dashboards" </code> 

        :param self:
        :param Namespace:  The namespace. Currently, you should set this to <code>default</code>.<br/>AwsAccountId:  The ID for the AWS account that the group is in. Currently, you use the ID for the AWS account that contains your Amazon QuickSight account.<br/>Description:  The description for the group that you want to update.<br/>GroupName:  The name of the group that you want to update.<br/>
        :return: Status:  The http status of the request.<br/>RequestId:  The AWS request ID for this operation.<br/>Group:  The name of the group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrincipalId</i> The principal ID of the group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The group description.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupName</i> The name of the group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) for the group.<br/>        """
        pass

    def update_user(self, Role: str, Email: str, Namespace: str, AwsAccountId: str, UserName: str) -> UpdateUserResponse:
        """Updates an Amazon QuickSight user. The permission resource is <code>arn:aws:quicksight:us-east-1:<i>&lt;aws-account-id&gt;</i>:user/default/<i>&lt;user-name&gt;</i> </code>.  The response is a user object that contains the user's Amazon QuickSight user name, email address, active or inactive status in Amazon QuickSight, Amazon QuickSight role, and Amazon Resource Name (ARN).   <b>CLI Sample:</b>   <code>aws quicksight update-user --user-name=Pat --role=ADMIN --email=new_address@amazon.com --aws-account-id=111122223333 --namespace=default --region=us-east-1 </code> 

        :param self:
        :param Role:  The Amazon QuickSight role of the user. The user role can be one of the following: <ul> <li>  <code>READER</code>: A user who has read-only access to dashboards. </li> <li>  <code>AUTHOR</code>: A user who can create data sources, data sets, analyses, and dashboards. </li> <li>  <code>ADMIN</code>: A user who is an author, who can also manage Amazon QuickSight settings. </li> </ul><br/>Email:  The email address of the user that you want to update.<br/>Namespace:  The namespace. Currently, you should set this to <code>default</code>.<br/>AwsAccountId:  The ID for the AWS account that the user is in. Currently, you use the ID for the AWS account that contains your Amazon QuickSight account.<br/>UserName:  The Amazon QuickSight user name that you want to update.<br/>
        :return: Status:  The http status of the request.<br/>RequestId:  The AWS request ID for this operation.<br/>User:  The Amazon QuickSight user.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrincipalId</i> The principal ID of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Active</i> Active status of user. When you create an Amazon QuickSight user that’s not an IAM user or an AD user, that user is inactive until they sign in and provide a password<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IdentityType</i> The type of identity authentication used by the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Role</i> The Amazon QuickSight role for the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Email</i> The user's email address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserName</i> The user's user name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) for the user.<br/>        """
        pass



class RAM(BaseClient):
    """Use AWS Resource Access Manager to share AWS resources between AWS accounts. To share a resource, you create a resource share, associate the resource with the resource share, and specify the principals that can access the resource. The following principals are supported: <ul> <li> The ID of an AWS account </li> <li> The Amazon Resource Name (ARN) of an OU from AWS Organizations </li> <li> The Amazon Resource Name (ARN) of an organization from AWS Organizations </li> </ul> If you specify an AWS account that doesn't exist in the same organization as the account that owns the resource share, the owner of the specified account receives an invitation to accept the resource share. After the owner accepts the invitation, they can access the resources in the resource share. An administrator of the specified account can use IAM policies to restrict access resources in the resource share."""


    class GetResourcePoliciesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_resource_policies` for valid parameters"""
            pass
    class GetResourceShareAssociationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_resource_share_associations` for valid parameters"""
            pass
    class GetResourceShareInvitationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_resource_share_invitations` for valid parameters"""
            pass
    class GetResourceSharesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_resource_shares` for valid parameters"""
            pass
    class ListPrincipalsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_principals` for valid parameters"""
            pass
    class ListResourcesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_resources` for valid parameters"""
            pass


    class AcceptResourceShareInvitationRequest(Mapping):
        pass
    
    class AcceptResourceShareInvitationResponse(Mapping):
        pass
    
    class AssociateResourceShareRequest(Mapping):
        pass
    
    class AssociateResourceShareResponse(Mapping):
        pass
    
    class String(object):
        pass
    
    class CreateResourceShareRequest(Mapping):
        pass
    
    class CreateResourceShareResponse(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class DeleteResourceShareRequest(Mapping):
        pass
    
    class DeleteResourceShareResponse(Mapping):
        pass
    
    class DisassociateResourceShareRequest(Mapping):
        pass
    
    class DisassociateResourceShareResponse(Mapping):
        pass
    
    class EnableSharingWithAwsOrganizationRequest(Mapping):
        pass
    
    class EnableSharingWithAwsOrganizationResponse(Mapping):
        pass
    
    class GetResourcePoliciesRequest(Mapping):
        pass
    
    class GetResourcePoliciesResponse(Mapping):
        pass
    
    class GetResourceShareAssociationsRequest(Mapping):
        pass
    
    class GetResourceShareAssociationsResponse(Mapping):
        pass
    
    class GetResourceShareInvitationsRequest(Mapping):
        pass
    
    class GetResourceShareInvitationsResponse(Mapping):
        pass
    
    class GetResourceSharesRequest(Mapping):
        pass
    
    class GetResourceSharesResponse(Mapping):
        pass
    
    class TagFilter(Mapping):
        pass
    
    class ListPrincipalsRequest(Mapping):
        pass
    
    class ListPrincipalsResponse(Mapping):
        pass
    
    class ListResourcesRequest(Mapping):
        pass
    
    class ListResourcesResponse(Mapping):
        pass
    
    class RejectResourceShareInvitationRequest(Mapping):
        pass
    
    class RejectResourceShareInvitationResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateResourceShareRequest(Mapping):
        pass
    
    class UpdateResourceShareResponse(Mapping):
        pass
    
    def accept_resource_share_invitation(self, resourceShareInvitationArn: str, clientToken: str=None) -> AcceptResourceShareInvitationResponse:
        """Accepts an invitation to a resource share from another AWS account.

        :param self:
        :param clientToken:  A unique, case-sensitive identifier that you provide to ensure the idempotency of the request.<br/>resourceShareInvitationArn:  The Amazon Resource Name (ARN) of the invitation.<br/>
        :return: clientToken:  A unique, case-sensitive identifier that you provide to ensure the idempotency of the request.<br/>resourceShareInvitation:  Information about the invitation.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceShareAssociations</i> The resources associated with the resource share.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the invitation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>invitationTimestamp</i> The date and time when the invitation was sent.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>receiverAccountId</i> The ID of the AWS account that received the invitation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>senderAccountId</i> The ID of the AWS account that sent the invitation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceShareArn</i> The Amazon Resource Name (ARN) of the resource share.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceShareName</i> The name of the resource share.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceShareInvitationArn</i> The Amazon Resource Name (ARN) of the invitation.<br/>        """
        pass

    def associate_resource_share(self, resourceShareArn: str, resourceArns: List[String]=None, principals: List[String]=None, clientToken: str=None) -> AssociateResourceShareResponse:
        """Associates the specified resource share with the specified principals and resources.

        :param self:
        :param clientToken:  A unique, case-sensitive identifier that you provide to ensure the idempotency of the request.<br/>principals:  The principals.<br/>resourceArns:  The Amazon Resource Names (ARN) of the resources.<br/>resourceShareArn:  The Amazon Resource Name (ARN) of the resource share.<br/>
        :return: clientToken:  A unique, case-sensitive identifier that you provide to ensure the idempotency of the request.<br/>resourceShareAssociations:  Information about the associations.<br/>        """
        pass

    def create_resource_share(self, name: str, resourceArns: List[String]=None, principals: List[String]=None, tags: List[Tag]=None, allowExternalPrincipals: Boolean=None, clientToken: str=None) -> CreateResourceShareResponse:
        """Creates a resource share.

        :param self:
        :param clientToken:  A unique, case-sensitive identifier that you provide to ensure the idempotency of the request.<br/>allowExternalPrincipals:  Indicates whether principals outside your organization can be associated with a resource share.<br/>tags:  One or more tags.<br/>principals:  The principals to associate with the resource share. The possible values are IDs of AWS accounts, the ARN of an OU or organization from AWS Organizations.<br/>resourceArns:  The Amazon Resource Names (ARN) of the resources to associate with the resource share.<br/>name:  The name of the resource share.<br/>
        :return: clientToken:  A unique, case-sensitive identifier that you provide to ensure the idempotency of the request.<br/>resourceShare:  Information about the resource share.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedTime</i> The time when the resource share was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationTime</i> The time when the resource share was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> The tags for the resource share.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusMessage</i> A message about the status of the resource share.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the resource share.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>allowExternalPrincipals</i> Indicates whether principals outside your organization can be associated with a resource share.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>owningAccountId</i> The ID of the AWS account that owns the resource share.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the resource share.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceShareArn</i> The Amazon Resource Name (ARN) of the resource share.<br/>        """
        pass

    def delete_resource_share(self, resourceShareArn: str, clientToken: str=None) -> DeleteResourceShareResponse:
        """Deletes the specified resource share.

        :param self:
        :param clientToken:  A unique, case-sensitive identifier that you provide to ensure the idempotency of the request.<br/>resourceShareArn:  The Amazon Resource Name (ARN) of the resource share.<br/>
        :return: clientToken:  A unique, case-sensitive identifier that you provide to ensure the idempotency of the request.<br/>returnValue:  Indicates whether the request succeeded.<br/>        """
        pass

    def disassociate_resource_share(self, resourceShareArn: str, resourceArns: List[String]=None, principals: List[String]=None, clientToken: str=None) -> DisassociateResourceShareResponse:
        """Disassociates the specified principals or resources from the specified resource share.

        :param self:
        :param clientToken:  A unique, case-sensitive identifier that you provide to ensure the idempotency of the request.<br/>principals:  The principals.<br/>resourceArns:  The Amazon Resource Names (ARN) of the resources.<br/>resourceShareArn:  The Amazon Resource Name (ARN) of the resource share.<br/>
        :return: clientToken:  A unique, case-sensitive identifier that you provide to ensure the idempotency of the request.<br/>resourceShareAssociations:  Information about the associations.<br/>        """
        pass

    def enable_sharing_with_aws_organization(self) -> EnableSharingWithAwsOrganizationResponse:
        """Enables resource sharing within your organization.

        :param self:
        :param 
        :return: returnValue:  Indicates whether the request succeeded.<br/>        """
        pass

    def get_resource_policies(self, resourceArns: List[String], principal: str=None, nextToken: str=None, maxResults: int=None) -> GetResourcePoliciesResponse:
        """Gets the policies for the specifies resources.

        :param self:
        :param maxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>nextToken:  The token for the next page of results.<br/>principal:  The principal.<br/>resourceArns:  The Amazon Resource Names (ARN) of the resources.<br/>
        :return: nextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>policies:  A key policy document, in JSON format.<br/>        """
        pass

    def get_resource_share_associations(self, associationType: str, resourceShareArns: List[String]=None, resourceArn: str=None, principal: str=None, associationStatus: str=None, nextToken: str=None, maxResults: int=None) -> GetResourceShareAssociationsResponse:
        """Gets the associations for the specified resource share.

        :param self:
        :param maxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>nextToken:  The token for the next page of results.<br/>associationStatus:  The status of the association.<br/>principal:  The principal.<br/>resourceArn:  The Amazon Resource Name (ARN) of the resource.<br/>resourceShareArns:  The Amazon Resource Names (ARN) of the resource shares.<br/>associationType:  The association type.<br/>
        :return: nextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>resourceShareAssociations:  Information about the association.<br/>        """
        pass

    def get_resource_share_invitations(self, resourceShareInvitationArns: List[String]=None, resourceShareArns: List[String]=None, nextToken: str=None, maxResults: int=None) -> GetResourceShareInvitationsResponse:
        """Gets the specified invitations for resource sharing.

        :param self:
        :param maxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>nextToken:  The token for the next page of results.<br/>resourceShareArns:  The Amazon Resource Names (ARN) of the resource shares.<br/>resourceShareInvitationArns:  The Amazon Resource Names (ARN) of the invitations.<br/>
        :return: nextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>resourceShareInvitations:  Information about the invitations.<br/>        """
        pass

    def get_resource_shares(self, resourceOwner: str, resourceShareArns: List[String]=None, resourceShareStatus: str=None, name: str=None, tagFilters: List[TagFilter]=None, nextToken: str=None, maxResults: int=None) -> GetResourceSharesResponse:
        """Gets the specified resource shares or all of your resource shares.

        :param self:
        :param maxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>nextToken:  The token for the next page of results.<br/>tagFilters:  One or more tag filters.<br/>name:  The name of the resource share.<br/>resourceOwner:  The type of owner.<br/>resourceShareStatus:  The status of the resource share.<br/>resourceShareArns:  The Amazon Resource Names (ARN) of the resource shares.<br/>
        :return: nextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>resourceShares:  Information about the resource shares.<br/>        """
        pass

    def list_principals(self, resourceOwner: str, resourceArn: str=None, principals: List[String]=None, resourceType: str=None, resourceShareArns: List[String]=None, nextToken: str=None, maxResults: int=None) -> ListPrincipalsResponse:
        """Lists the principals with access to the specified resource.

        :param self:
        :param maxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>nextToken:  The token for the next page of results.<br/>resourceShareArns:  The Amazon Resource Names (ARN) of the resource shares.<br/>resourceType:  The resource type.<br/>principals:  The principals.<br/>resourceArn:  The Amazon Resource Name (ARN) of the resource.<br/>resourceOwner:  The type of owner.<br/>
        :return: nextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>principals:  The principals.<br/>        """
        pass

    def list_resources(self, resourceOwner: str, principal: str=None, resourceType: str=None, resourceArns: List[String]=None, resourceShareArns: List[String]=None, nextToken: str=None, maxResults: int=None) -> ListResourcesResponse:
        """Lists the resources that the specified principal can access.

        :param self:
        :param maxResults:  The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.<br/>nextToken:  The token for the next page of results.<br/>resourceShareArns:  The Amazon Resource Names (ARN) of the resource shares.<br/>resourceArns:  The Amazon Resource Names (ARN) of the resources.<br/>resourceType:  The resource type.<br/>principal:  The principal.<br/>resourceOwner:  The type of owner.<br/>
        :return: nextToken:  The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.<br/>resources:  Information about the resources.<br/>        """
        pass

    def reject_resource_share_invitation(self, resourceShareInvitationArn: str, clientToken: str=None) -> RejectResourceShareInvitationResponse:
        """Rejects an invitation to a resource share from another AWS account.

        :param self:
        :param clientToken:  A unique, case-sensitive identifier that you provide to ensure the idempotency of the request.<br/>resourceShareInvitationArn:  The Amazon Resource Name (ARN) of the invitation.<br/>
        :return: clientToken:  A unique, case-sensitive identifier that you provide to ensure the idempotency of the request.<br/>resourceShareInvitation:  Information about the invitation.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceShareAssociations</i> The resources associated with the resource share.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the invitation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>invitationTimestamp</i> The date and time when the invitation was sent.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>receiverAccountId</i> The ID of the AWS account that received the invitation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>senderAccountId</i> The ID of the AWS account that sent the invitation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceShareArn</i> The Amazon Resource Name (ARN) of the resource share.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceShareName</i> The name of the resource share.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceShareInvitationArn</i> The Amazon Resource Name (ARN) of the invitation.<br/>        """
        pass

    def tag_resource(self, tags: List[Tag], resourceShareArn: str) -> TagResourceResponse:
        """Adds the specified tags to the specified resource share.

        :param self:
        :param tags:  One or more tags.<br/>resourceShareArn:  The Amazon Resource Name (ARN) of the resource share.<br/>
        :return:         """
        pass

    def untag_resource(self, tagKeys: List[TagKey], resourceShareArn: str) -> UntagResourceResponse:
        """Removes the specified tags from the specified resource share.

        :param self:
        :param tagKeys:  The tag keys of the tags to remove.<br/>resourceShareArn:  The Amazon Resource Name (ARN) of the resource share.<br/>
        :return:         """
        pass

    def update_resource_share(self, resourceShareArn: str, name: str=None, allowExternalPrincipals: Boolean=None, clientToken: str=None) -> UpdateResourceShareResponse:
        """Updates the specified resource share.

        :param self:
        :param clientToken:  A unique, case-sensitive identifier that you provide to ensure the idempotency of the request.<br/>allowExternalPrincipals:  Indicates whether principals outside your organization can be associated with a resource share.<br/>name:  The name of the resource share.<br/>resourceShareArn:  The Amazon Resource Name (ARN) of the resource share.<br/>
        :return: clientToken:  A unique, case-sensitive identifier that you provide to ensure the idempotency of the request.<br/>resourceShare:  Information about the resource share.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastUpdatedTime</i> The time when the resource share was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationTime</i> The time when the resource share was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tags</i> The tags for the resource share.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusMessage</i> A message about the status of the resource share.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the resource share.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>allowExternalPrincipals</i> Indicates whether principals outside your organization can be associated with a resource share.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>owningAccountId</i> The ID of the AWS account that owns the resource share.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the resource share.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourceShareArn</i> The Amazon Resource Name (ARN) of the resource share.<br/>        """
        pass



class RDS(BaseClient):
    """<fullname>Amazon Relational Database Service</fullname>   Amazon Relational Database Service (Amazon RDS) is a web service that makes it easier to set up, operate, and scale a relational database in the cloud. It provides cost-efficient, resizable capacity for an industry-standard relational database and manages common database administration tasks, freeing up developers to focus on what makes their applications and businesses unique. Amazon RDS gives you access to the capabilities of a MySQL, MariaDB, PostgreSQL, Microsoft SQL Server, Oracle, or Amazon Aurora database server. These capabilities mean that the code, applications, and tools you already use today with your existing databases work with Amazon RDS without modification. Amazon RDS automatically backs up your database and maintains the database software that powers your DB instance. Amazon RDS is flexible: you can scale your DB instance's compute resources and storage capacity to meet your application's demand. As with all Amazon Web Services, there are no up-front investments, and you pay only for the resources you use. This interface reference for Amazon RDS contains documentation for a programming or command line interface you can use to manage Amazon RDS. Note that Amazon RDS is asynchronous, which means that some interfaces might require techniques such as polling or callback functions to determine when a command has been applied. In this reference, the parameter descriptions indicate whether a command is applied immediately, on the next instance reboot, or during the maintenance window. The reference structure is as follows, and we list following some related topics from the user guide.  <b>Amazon RDS API Reference</b>  <ul> <li> For the alphabetical list of API actions, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/APIReference/API_Operations.html">API Actions</a>. </li> <li> For the alphabetical list of data types, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/APIReference/API_Types.html">Data Types</a>. </li> <li> For a list of common query parameters, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/APIReference/CommonParameters.html">Common Parameters</a>. </li> <li> For descriptions of the error codes, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/APIReference/CommonErrors.html">Common Errors</a>. </li> </ul>  <b>Amazon RDS User Guide</b>  <ul> <li> For a summary of the Amazon RDS interfaces, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/Welcome.html#Welcome.Interfaces">Available RDS Interfaces</a>. </li> <li> For more information about how to use the Query API, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/Using_the_Query_API.html">Using the Query API</a>. </li> </ul>"""

    class DBInstanceAvailableWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_db_instances` for valid parameters"""
            pass
    class DBInstanceDeletedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_db_instances` for valid parameters"""
            pass
    class DBSnapshotAvailableWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_db_snapshots` for valid parameters"""
            pass
    class DBSnapshotCompletedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_db_snapshots` for valid parameters"""
            pass
    class DBSnapshotDeletedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_db_snapshots` for valid parameters"""
            pass


    class DescribeCertificatesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_certificates` for valid parameters"""
            pass
    class DescribeDBClusterBacktracksPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_db_cluster_backtracks` for valid parameters"""
            pass
    class DescribeDBClusterParameterGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_db_cluster_parameter_groups` for valid parameters"""
            pass
    class DescribeDBClusterParametersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_db_cluster_parameters` for valid parameters"""
            pass
    class DescribeDBClusterSnapshotsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_db_cluster_snapshots` for valid parameters"""
            pass
    class DescribeDBClustersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_db_clusters` for valid parameters"""
            pass
    class DescribeDBEngineVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_db_engine_versions` for valid parameters"""
            pass
    class DescribeDBInstanceAutomatedBackupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_db_instance_automated_backups` for valid parameters"""
            pass
    class DescribeDBInstancesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_db_instances` for valid parameters"""
            pass
    class DescribeDBLogFilesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_db_log_files` for valid parameters"""
            pass
    class DescribeDBParameterGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_db_parameter_groups` for valid parameters"""
            pass
    class DescribeDBParametersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_db_parameters` for valid parameters"""
            pass
    class DescribeDBSecurityGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_db_security_groups` for valid parameters"""
            pass
    class DescribeDBSnapshotsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_db_snapshots` for valid parameters"""
            pass
    class DescribeDBSubnetGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_db_subnet_groups` for valid parameters"""
            pass
    class DescribeEngineDefaultClusterParametersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_engine_default_cluster_parameters` for valid parameters"""
            pass
    class DescribeEngineDefaultParametersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_engine_default_parameters` for valid parameters"""
            pass
    class DescribeEventSubscriptionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_event_subscriptions` for valid parameters"""
            pass
    class DescribeEventsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_events` for valid parameters"""
            pass
    class DescribeGlobalClustersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_global_clusters` for valid parameters"""
            pass
    class DescribeOptionGroupOptionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_option_group_options` for valid parameters"""
            pass
    class DescribeOptionGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_option_groups` for valid parameters"""
            pass
    class DescribeOrderableDBInstanceOptionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_orderable_db_instance_options` for valid parameters"""
            pass
    class DescribePendingMaintenanceActionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_pending_maintenance_actions` for valid parameters"""
            pass
    class DescribeReservedDBInstancesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_reserved_db_instances` for valid parameters"""
            pass
    class DescribeReservedDBInstancesOfferingsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_reserved_db_instances_offerings` for valid parameters"""
            pass
    class DescribeSourceRegionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_source_regions` for valid parameters"""
            pass
    class DownloadDBLogFilePortionPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `download_db_log_file_portion` for valid parameters"""
            pass
    class DescribeDBClusterEndpointsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_db_cluster_endpoints` for valid parameters"""
            pass


    class AddRoleToDBClusterMessage(Mapping):
        pass
    
    class AddRoleToDBInstanceMessage(Mapping):
        pass
    
    class AddSourceIdentifierToSubscriptionMessage(Mapping):
        pass
    
    class AddSourceIdentifierToSubscriptionResult(Mapping):
        pass
    
    class AddTagsToResourceMessage(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class ApplyPendingMaintenanceActionMessage(Mapping):
        pass
    
    class ApplyPendingMaintenanceActionResult(Mapping):
        pass
    
    class AuthorizeDBSecurityGroupIngressMessage(Mapping):
        pass
    
    class AuthorizeDBSecurityGroupIngressResult(Mapping):
        pass
    
    class BacktrackDBClusterMessage(Mapping):
        pass
    
    class DBClusterBacktrack(Mapping):
        pass
    
    class TStamp(object):
        pass
    
    class BooleanOptional(object):
        pass
    
    class CopyDBClusterParameterGroupMessage(Mapping):
        pass
    
    class CopyDBClusterParameterGroupResult(Mapping):
        pass
    
    class CopyDBClusterSnapshotMessage(Mapping):
        pass
    
    class CopyDBClusterSnapshotResult(Mapping):
        pass
    
    class CopyDBParameterGroupMessage(Mapping):
        pass
    
    class CopyDBParameterGroupResult(Mapping):
        pass
    
    class CopyDBSnapshotMessage(Mapping):
        pass
    
    class CopyDBSnapshotResult(Mapping):
        pass
    
    class CopyOptionGroupMessage(Mapping):
        pass
    
    class CopyOptionGroupResult(Mapping):
        pass
    
    class CreateDBClusterMessage(Mapping):
        pass
    
    class CreateDBClusterResult(Mapping):
        pass
    
    class String(object):
        pass
    
    class LongOptional(object):
        pass
    
    class ScalingConfiguration(Mapping):
        pass
    
    class CreateDBClusterEndpointMessage(Mapping):
        pass
    
    class DBClusterEndpoint(Mapping):
        pass
    
    class CreateDBClusterParameterGroupMessage(Mapping):
        pass
    
    class CreateDBClusterParameterGroupResult(Mapping):
        pass
    
    class CreateDBClusterSnapshotMessage(Mapping):
        pass
    
    class CreateDBClusterSnapshotResult(Mapping):
        pass
    
    class CreateDBInstanceMessage(Mapping):
        pass
    
    class CreateDBInstanceResult(Mapping):
        pass
    
    class ProcessorFeature(Mapping):
        pass
    
    class CreateDBInstanceReadReplicaMessage(Mapping):
        pass
    
    class CreateDBInstanceReadReplicaResult(Mapping):
        pass
    
    class CreateDBParameterGroupMessage(Mapping):
        pass
    
    class CreateDBParameterGroupResult(Mapping):
        pass
    
    class CreateDBSecurityGroupMessage(Mapping):
        pass
    
    class CreateDBSecurityGroupResult(Mapping):
        pass
    
    class CreateDBSnapshotMessage(Mapping):
        pass
    
    class CreateDBSnapshotResult(Mapping):
        pass
    
    class CreateDBSubnetGroupMessage(Mapping):
        pass
    
    class CreateDBSubnetGroupResult(Mapping):
        pass
    
    class CreateEventSubscriptionMessage(Mapping):
        pass
    
    class CreateEventSubscriptionResult(Mapping):
        pass
    
    class CreateGlobalClusterMessage(Mapping):
        pass
    
    class CreateGlobalClusterResult(Mapping):
        pass
    
    class CreateOptionGroupMessage(Mapping):
        pass
    
    class CreateOptionGroupResult(Mapping):
        pass
    
    class DeleteDBClusterMessage(Mapping):
        pass
    
    class DeleteDBClusterResult(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class DeleteDBClusterEndpointMessage(Mapping):
        pass
    
    class DeleteDBClusterParameterGroupMessage(Mapping):
        pass
    
    class DeleteDBClusterSnapshotMessage(Mapping):
        pass
    
    class DeleteDBClusterSnapshotResult(Mapping):
        pass
    
    class DeleteDBInstanceMessage(Mapping):
        pass
    
    class DeleteDBInstanceResult(Mapping):
        pass
    
    class DeleteDBInstanceAutomatedBackupMessage(Mapping):
        pass
    
    class DeleteDBInstanceAutomatedBackupResult(Mapping):
        pass
    
    class DeleteDBParameterGroupMessage(Mapping):
        pass
    
    class DeleteDBSecurityGroupMessage(Mapping):
        pass
    
    class DeleteDBSnapshotMessage(Mapping):
        pass
    
    class DeleteDBSnapshotResult(Mapping):
        pass
    
    class DeleteDBSubnetGroupMessage(Mapping):
        pass
    
    class DeleteEventSubscriptionMessage(Mapping):
        pass
    
    class DeleteEventSubscriptionResult(Mapping):
        pass
    
    class DeleteGlobalClusterMessage(Mapping):
        pass
    
    class DeleteGlobalClusterResult(Mapping):
        pass
    
    class DeleteOptionGroupMessage(Mapping):
        pass
    
    class DescribeAccountAttributesMessage(Mapping):
        pass
    
    class AccountAttributesMessage(Mapping):
        pass
    
    class DescribeCertificatesMessage(Mapping):
        pass
    
    class CertificateMessage(Mapping):
        pass
    
    class Filter(Mapping):
        pass
    
    class DescribeDBClusterBacktracksMessage(Mapping):
        pass
    
    class DBClusterBacktrackMessage(Mapping):
        pass
    
    class DescribeDBClusterEndpointsMessage(Mapping):
        pass
    
    class DBClusterEndpointMessage(Mapping):
        pass
    
    class DescribeDBClusterParameterGroupsMessage(Mapping):
        pass
    
    class DBClusterParameterGroupsMessage(Mapping):
        pass
    
    class DescribeDBClusterParametersMessage(Mapping):
        pass
    
    class DBClusterParameterGroupDetails(Mapping):
        pass
    
    class DescribeDBClusterSnapshotAttributesMessage(Mapping):
        pass
    
    class DescribeDBClusterSnapshotAttributesResult(Mapping):
        pass
    
    class DescribeDBClusterSnapshotsMessage(Mapping):
        pass
    
    class DBClusterSnapshotMessage(Mapping):
        pass
    
    class DescribeDBClustersMessage(Mapping):
        pass
    
    class DBClusterMessage(Mapping):
        pass
    
    class DescribeDBEngineVersionsMessage(Mapping):
        pass
    
    class DBEngineVersionMessage(Mapping):
        pass
    
    class DescribeDBInstanceAutomatedBackupsMessage(Mapping):
        pass
    
    class DBInstanceAutomatedBackupMessage(Mapping):
        pass
    
    class DescribeDBInstancesMessage(Mapping):
        pass
    
    class DBInstanceMessage(Mapping):
        pass
    
    class DescribeDBLogFilesMessage(Mapping):
        pass
    
    class DescribeDBLogFilesResponse(Mapping):
        pass
    
    class Long(object):
        pass
    
    class DescribeDBParameterGroupsMessage(Mapping):
        pass
    
    class DBParameterGroupsMessage(Mapping):
        pass
    
    class DescribeDBParametersMessage(Mapping):
        pass
    
    class DBParameterGroupDetails(Mapping):
        pass
    
    class DescribeDBSecurityGroupsMessage(Mapping):
        pass
    
    class DBSecurityGroupMessage(Mapping):
        pass
    
    class DescribeDBSnapshotAttributesMessage(Mapping):
        pass
    
    class DescribeDBSnapshotAttributesResult(Mapping):
        pass
    
    class DescribeDBSnapshotsMessage(Mapping):
        pass
    
    class DBSnapshotMessage(Mapping):
        pass
    
    class DescribeDBSubnetGroupsMessage(Mapping):
        pass
    
    class DBSubnetGroupMessage(Mapping):
        pass
    
    class DescribeEngineDefaultClusterParametersMessage(Mapping):
        pass
    
    class DescribeEngineDefaultClusterParametersResult(Mapping):
        pass
    
    class DescribeEngineDefaultParametersMessage(Mapping):
        pass
    
    class DescribeEngineDefaultParametersResult(Mapping):
        pass
    
    class DescribeEventCategoriesMessage(Mapping):
        pass
    
    class EventCategoriesMessage(Mapping):
        pass
    
    class DescribeEventSubscriptionsMessage(Mapping):
        pass
    
    class EventSubscriptionsMessage(Mapping):
        pass
    
    class DescribeEventsMessage(Mapping):
        pass
    
    class EventsMessage(Mapping):
        pass
    
    class DescribeGlobalClustersMessage(Mapping):
        pass
    
    class GlobalClustersMessage(Mapping):
        pass
    
    class DescribeOptionGroupOptionsMessage(Mapping):
        pass
    
    class OptionGroupOptionsMessage(Mapping):
        pass
    
    class DescribeOptionGroupsMessage(Mapping):
        pass
    
    class OptionGroups(Mapping):
        pass
    
    class DescribeOrderableDBInstanceOptionsMessage(Mapping):
        pass
    
    class OrderableDBInstanceOptionsMessage(Mapping):
        pass
    
    class DescribePendingMaintenanceActionsMessage(Mapping):
        pass
    
    class PendingMaintenanceActionsMessage(Mapping):
        pass
    
    class DescribeReservedDBInstancesMessage(Mapping):
        pass
    
    class ReservedDBInstanceMessage(Mapping):
        pass
    
    class DescribeReservedDBInstancesOfferingsMessage(Mapping):
        pass
    
    class ReservedDBInstancesOfferingMessage(Mapping):
        pass
    
    class DescribeSourceRegionsMessage(Mapping):
        pass
    
    class SourceRegionMessage(Mapping):
        pass
    
    class DescribeValidDBInstanceModificationsMessage(Mapping):
        pass
    
    class DescribeValidDBInstanceModificationsResult(Mapping):
        pass
    
    class DownloadDBLogFilePortionMessage(Mapping):
        pass
    
    class DownloadDBLogFilePortionDetails(Mapping):
        pass
    
    class FailoverDBClusterMessage(Mapping):
        pass
    
    class FailoverDBClusterResult(Mapping):
        pass
    
    class ListTagsForResourceMessage(Mapping):
        pass
    
    class TagListMessage(Mapping):
        pass
    
    class ModifyCurrentDBClusterCapacityMessage(Mapping):
        pass
    
    class DBClusterCapacityInfo(Mapping):
        pass
    
    class ModifyDBClusterMessage(Mapping):
        pass
    
    class ModifyDBClusterResult(Mapping):
        pass
    
    class CloudwatchLogsExportConfiguration(Mapping):
        pass
    
    class ModifyDBClusterEndpointMessage(Mapping):
        pass
    
    class ModifyDBClusterParameterGroupMessage(Mapping):
        pass
    
    class DBClusterParameterGroupNameMessage(Mapping):
        pass
    
    class Parameter(Mapping):
        pass
    
    class ModifyDBClusterSnapshotAttributeMessage(Mapping):
        pass
    
    class ModifyDBClusterSnapshotAttributeResult(Mapping):
        pass
    
    class ModifyDBInstanceMessage(Mapping):
        pass
    
    class ModifyDBInstanceResult(Mapping):
        pass
    
    class ModifyDBParameterGroupMessage(Mapping):
        pass
    
    class DBParameterGroupNameMessage(Mapping):
        pass
    
    class ModifyDBSnapshotMessage(Mapping):
        pass
    
    class ModifyDBSnapshotResult(Mapping):
        pass
    
    class ModifyDBSnapshotAttributeMessage(Mapping):
        pass
    
    class ModifyDBSnapshotAttributeResult(Mapping):
        pass
    
    class ModifyDBSubnetGroupMessage(Mapping):
        pass
    
    class ModifyDBSubnetGroupResult(Mapping):
        pass
    
    class ModifyEventSubscriptionMessage(Mapping):
        pass
    
    class ModifyEventSubscriptionResult(Mapping):
        pass
    
    class ModifyGlobalClusterMessage(Mapping):
        pass
    
    class ModifyGlobalClusterResult(Mapping):
        pass
    
    class ModifyOptionGroupMessage(Mapping):
        pass
    
    class ModifyOptionGroupResult(Mapping):
        pass
    
    class OptionConfiguration(Mapping):
        pass
    
    class PromoteReadReplicaMessage(Mapping):
        pass
    
    class PromoteReadReplicaResult(Mapping):
        pass
    
    class PromoteReadReplicaDBClusterMessage(Mapping):
        pass
    
    class PromoteReadReplicaDBClusterResult(Mapping):
        pass
    
    class PurchaseReservedDBInstancesOfferingMessage(Mapping):
        pass
    
    class PurchaseReservedDBInstancesOfferingResult(Mapping):
        pass
    
    class RebootDBInstanceMessage(Mapping):
        pass
    
    class RebootDBInstanceResult(Mapping):
        pass
    
    class RemoveFromGlobalClusterMessage(Mapping):
        pass
    
    class RemoveFromGlobalClusterResult(Mapping):
        pass
    
    class RemoveRoleFromDBClusterMessage(Mapping):
        pass
    
    class RemoveRoleFromDBInstanceMessage(Mapping):
        pass
    
    class RemoveSourceIdentifierFromSubscriptionMessage(Mapping):
        pass
    
    class RemoveSourceIdentifierFromSubscriptionResult(Mapping):
        pass
    
    class RemoveTagsFromResourceMessage(Mapping):
        pass
    
    class ResetDBClusterParameterGroupMessage(Mapping):
        pass
    
    class ResetDBParameterGroupMessage(Mapping):
        pass
    
    class RestoreDBClusterFromS3Message(Mapping):
        pass
    
    class RestoreDBClusterFromS3Result(Mapping):
        pass
    
    class RestoreDBClusterFromSnapshotMessage(Mapping):
        pass
    
    class RestoreDBClusterFromSnapshotResult(Mapping):
        pass
    
    class RestoreDBClusterToPointInTimeMessage(Mapping):
        pass
    
    class RestoreDBClusterToPointInTimeResult(Mapping):
        pass
    
    class RestoreDBInstanceFromDBSnapshotMessage(Mapping):
        pass
    
    class RestoreDBInstanceFromDBSnapshotResult(Mapping):
        pass
    
    class RestoreDBInstanceFromS3Message(Mapping):
        pass
    
    class RestoreDBInstanceFromS3Result(Mapping):
        pass
    
    class RestoreDBInstanceToPointInTimeMessage(Mapping):
        pass
    
    class RestoreDBInstanceToPointInTimeResult(Mapping):
        pass
    
    class RevokeDBSecurityGroupIngressMessage(Mapping):
        pass
    
    class RevokeDBSecurityGroupIngressResult(Mapping):
        pass
    
    class StartActivityStreamRequest(Mapping):
        pass
    
    class StartActivityStreamResponse(Mapping):
        pass
    
    class StartDBClusterMessage(Mapping):
        pass
    
    class StartDBClusterResult(Mapping):
        pass
    
    class StartDBInstanceMessage(Mapping):
        pass
    
    class StartDBInstanceResult(Mapping):
        pass
    
    class StopActivityStreamRequest(Mapping):
        pass
    
    class StopActivityStreamResponse(Mapping):
        pass
    
    class StopDBClusterMessage(Mapping):
        pass
    
    class StopDBClusterResult(Mapping):
        pass
    
    class StopDBInstanceMessage(Mapping):
        pass
    
    class StopDBInstanceResult(Mapping):
        pass
    
    def add_role_to_db_cluster(self, RoleArn: str, DBClusterIdentifier: str, FeatureName: str=None):
        """Associates an Identity and Access Management (IAM) role from an Amazon Aurora DB cluster. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/AuroraMySQL.Integrating.Authorizing.html">Authorizing Amazon Aurora MySQL to Access Other AWS Services on Your Behalf</a> in the <i>Amazon Aurora User Guide</i>. <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param FeatureName:  The name of the feature for the DB cluster that the IAM role is to be associated with. For the list of supported feature names, see <a>DBEngineVersion</a>.<br/>RoleArn:  The Amazon Resource Name (ARN) of the IAM role to associate with the Aurora DB cluster, for example <code>arn:aws:iam::123456789012:role/AuroraAccessRole</code>.<br/>DBClusterIdentifier:  The name of the DB cluster to associate the IAM role with.<br/>
        :return:         """
        pass

    def add_role_to_db_instance(self, FeatureName: str, RoleArn: str, DBInstanceIdentifier: str):
        """Associates an AWS Identity and Access Management (IAM) role with a DB instance. <note> To add a role to a DB instance, the status of the DB instance must be <code>available</code>. </note>

        :param self:
        :param FeatureName:  The name of the feature for the DB instance that the IAM role is to be associated with. For the list of supported feature names, see <code>DBEngineVersion</code>. <br/>RoleArn:  The Amazon Resource Name (ARN) of the IAM role to associate with the DB instance, for example <code>arn:aws:iam::123456789012:role/AccessRole</code>. <br/>DBInstanceIdentifier:  The name of the DB instance to associate the IAM role with.<br/>
        :return:         """
        pass

    def add_source_identifier_to_subscription(self, SourceIdentifier: str, SubscriptionName: str) -> AddSourceIdentifierToSubscriptionResult:
        """Adds a source identifier to an existing RDS event notification subscription.

        :param self:
        :param SourceIdentifier:  The identifier of the event source to be added. Constraints: <ul> <li> If the source type is a DB instance, then a <code>DBInstanceIdentifier</code> must be supplied. </li> <li> If the source type is a DB security group, a <code>DBSecurityGroupName</code> must be supplied. </li> <li> If the source type is a DB parameter group, a <code>DBParameterGroupName</code> must be supplied. </li> <li> If the source type is a DB snapshot, a <code>DBSnapshotIdentifier</code> must be supplied. </li> </ul><br/>SubscriptionName:  The name of the RDS event notification subscription you want to add a source identifier to.<br/>
        :return: EventSubscription:  Contains the results of a successful invocation of the <code>DescribeEventSubscriptions</code> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EventSubscriptionArn</i> The Amazon Resource Name (ARN) for the event subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> A Boolean value indicating if the subscription is enabled. True indicates the subscription is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EventCategoriesList</i> A list of event categories for the RDS event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceIdsList</i> A list of source IDs for the RDS event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceType</i> The source type for the RDS event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubscriptionCreationTime</i> The time the RDS event notification subscription was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the RDS event notification subscription. Constraints: Can be one of the following: creating | modifying | deleting | active | no-permission | topic-not-exist The status "no-permission" indicates that RDS no longer has permission to post to the SNS topic. The status "topic-not-exist" indicates that the topic was deleted after the subscription was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsTopicArn</i> The topic ARN of the RDS event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustSubscriptionId</i> The RDS event notification subscription Id.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomerAwsId</i> The AWS customer account associated with the RDS event notification subscription.<br/>        """
        pass

    def add_tags_to_resource(self, Tags: List[Tag], ResourceName: str):
        """Adds metadata tags to an Amazon RDS resource. These tags can also be used with cost allocation reporting to track cost associated with Amazon RDS resources, or used in a Condition statement in an IAM policy for Amazon RDS. For an overview on tagging Amazon RDS resources, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/Overview.Tagging.html">Tagging Amazon RDS Resources</a>.

        :param self:
        :param Tags:  The tags to be assigned to the Amazon RDS resource.<br/>ResourceName:  The Amazon RDS resource that the tags are added to. This value is an Amazon Resource Name (ARN). For information about creating an ARN, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Tagging.ARN.html#USER_Tagging.ARN.Constructing"> Constructing an RDS Amazon Resource Name (ARN)</a>.<br/>
        :return:         """
        pass

    def apply_pending_maintenance_action(self, OptInType: str, ApplyAction: str, ResourceIdentifier: str) -> ApplyPendingMaintenanceActionResult:
        """Applies a pending maintenance action to a resource (for example, to a DB instance).

        :param self:
        :param OptInType:  A value that specifies the type of opt-in request, or undoes an opt-in request. An opt-in request of type <code>immediate</code> can't be undone. Valid values: <ul> <li>  <code>immediate</code> - Apply the maintenance action immediately. </li> <li>  <code>next-maintenance</code> - Apply the maintenance action during the next maintenance window for the resource. </li> <li>  <code>undo-opt-in</code> - Cancel any existing <code>next-maintenance</code> opt-in requests. </li> </ul><br/>ApplyAction:  The pending maintenance action to apply to this resource. Valid values: <code>system-update</code>, <code>db-upgrade</code>, <code>hardware-maintenance</code> <br/>ResourceIdentifier:  The RDS Amazon Resource Name (ARN) of the resource that the pending maintenance action applies to. For information about creating an ARN, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Tagging.ARN.html#USER_Tagging.ARN.Constructing"> Constructing an RDS Amazon Resource Name (ARN)</a>.<br/>
        :return: ResourcePendingMaintenanceActions:  Describes the pending maintenance actions for a resource.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingMaintenanceActionDetails</i> A list that provides details about the pending maintenance actions for the resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceIdentifier</i> The ARN of the resource that has pending maintenance actions.<br/>        """
        pass

    def authorize_db_security_group_ingress(self, DBSecurityGroupName: str, CIDRIP: str=None, EC2SecurityGroupName: str=None, EC2SecurityGroupId: str=None, EC2SecurityGroupOwnerId: str=None) -> AuthorizeDBSecurityGroupIngressResult:
        """Enables ingress to a DBSecurityGroup using one of two forms of authorization. First, EC2 or VPC security groups can be added to the DBSecurityGroup if the application using the database is running on EC2 or VPC instances. Second, IP ranges are available if the application accessing your database is running on the Internet. Required parameters for this API are one of CIDR range, EC2SecurityGroupId for VPC, or (EC2SecurityGroupOwnerId and either EC2SecurityGroupName or EC2SecurityGroupId for non-VPC). <note> You can't authorize ingress from an EC2 security group in one AWS Region to an Amazon RDS DB instance in another. You can't authorize ingress from a VPC security group in one VPC to an Amazon RDS DB instance in another. </note> For an overview of CIDR ranges, go to the <a href="http://en.wikipedia.org/wiki/Classless_Inter-Domain_Routing">Wikipedia Tutorial</a>. 

        :param self:
        :param EC2SecurityGroupOwnerId:   AWS account number of the owner of the EC2 security group specified in the <code>EC2SecurityGroupName</code> parameter. The AWS Access Key ID is not an acceptable value. For VPC DB security groups, <code>EC2SecurityGroupId</code> must be provided. Otherwise, <code>EC2SecurityGroupOwnerId</code> and either <code>EC2SecurityGroupName</code> or <code>EC2SecurityGroupId</code> must be provided. <br/>EC2SecurityGroupId:   Id of the EC2 security group to authorize. For VPC DB security groups, <code>EC2SecurityGroupId</code> must be provided. Otherwise, <code>EC2SecurityGroupOwnerId</code> and either <code>EC2SecurityGroupName</code> or <code>EC2SecurityGroupId</code> must be provided. <br/>EC2SecurityGroupName:   Name of the EC2 security group to authorize. For VPC DB security groups, <code>EC2SecurityGroupId</code> must be provided. Otherwise, <code>EC2SecurityGroupOwnerId</code> and either <code>EC2SecurityGroupName</code> or <code>EC2SecurityGroupId</code> must be provided. <br/>CIDRIP:  The IP range to authorize.<br/>DBSecurityGroupName:  The name of the DB security group to add authorization to.<br/>
        :return: DBSecurityGroup:  Contains the details for an Amazon RDS DB security group.  This data type is used as a response element in the <code>DescribeDBSecurityGroups</code> action. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSecurityGroupArn</i> The Amazon Resource Name (ARN) for the DB security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IPRanges</i>  Contains a list of <code>IPRange</code> elements. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EC2SecurityGroups</i>  Contains a list of <code>EC2SecurityGroup</code> elements. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VpcId of the DB security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSecurityGroupDescription</i> Provides the description of the DB security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSecurityGroupName</i> Specifies the name of the DB security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> Provides the AWS ID of the owner of a specific DB security group.<br/>        """
        pass

    def backtrack_db_cluster(self, BacktrackTo: TStamp, DBClusterIdentifier: str, Force: BooleanOptional=None, UseEarliestTimeOnPointInTimeUnavailable: BooleanOptional=None) -> DBClusterBacktrack:
        """Backtracks a DB cluster to a specific time, without creating a new DB cluster. For more information on backtracking, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/AuroraMySQL.Managing.Backtrack.html"> Backtracking an Aurora DB Cluster</a> in the <i>Amazon Aurora User Guide.</i>  <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param UseEarliestTimeOnPointInTimeUnavailable:  A value that indicates whether to backtrack the DB cluster to the earliest possible backtrack time when <i>BacktrackTo</i> is set to a timestamp earlier than the earliest backtrack time. When this parameter is disabled and <i>BacktrackTo</i> is set to a timestamp earlier than the earliest backtrack time, an error occurs.<br/>Force:  A value that indicates whether to force the DB cluster to backtrack when binary logging is enabled. Otherwise, an error occurs when binary logging is enabled.<br/>BacktrackTo:  The timestamp of the time to backtrack the DB cluster to, specified in ISO 8601 format. For more information about ISO 8601, see the <a href="http://en.wikipedia.org/wiki/ISO_8601">ISO8601 Wikipedia page.</a>  <note> If the specified time is not a consistent time for the DB cluster, Aurora automatically chooses the nearest possible consistent time for the DB cluster. </note> Constraints: <ul> <li> Must contain a valid ISO 8601 timestamp. </li> <li> Can't contain a timestamp set in the future. </li> </ul> Example: <code>2017-07-08T18:00Z</code> <br/>DBClusterIdentifier:  The DB cluster identifier of the DB cluster to be backtracked. This parameter is stored as a lowercase string. Constraints: <ul> <li> Must contain from 1 to 63 alphanumeric characters or hyphens. </li> <li> First character must be a letter. </li> <li> Can't end with a hyphen or contain two consecutive hyphens. </li> </ul> Example: <code>my-cluster1</code> <br/>
        :return: Status:  The status of the backtrack. This property returns one of the following values: <ul> <li>  <code>applying</code> - The backtrack is currently being applied to or rolled back from the DB cluster. </li> <li>  <code>completed</code> - The backtrack has successfully been applied to or rolled back from the DB cluster. </li> <li>  <code>failed</code> - An error occurred while the backtrack was applied to or rolled back from the DB cluster. </li> <li>  <code>pending</code> - The backtrack is currently pending application to or rollback from the DB cluster. </li> </ul><br/>BacktrackRequestCreationTime:  The timestamp of the time at which the backtrack was requested.<br/>BacktrackedFrom:  The timestamp of the time from which the DB cluster was backtracked.<br/>BacktrackTo:  The timestamp of the time to which the DB cluster was backtracked.<br/>BacktrackIdentifier:  Contains the backtrack identifier.<br/>DBClusterIdentifier:  Contains a user-supplied DB cluster identifier. This identifier is the unique key that identifies a DB cluster.<br/>        """
        pass

    def copy_db_cluster_parameter_group(self, TargetDBClusterParameterGroupDescription: str, TargetDBClusterParameterGroupIdentifier: str, SourceDBClusterParameterGroupIdentifier: str, Tags: List[Tag]=None) -> CopyDBClusterParameterGroupResult:
        """Copies the specified DB cluster parameter group. <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param Tags:  A list of tags. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Tagging.html">Tagging Amazon RDS Resources</a> in the <i>Amazon RDS User Guide.</i> <br/>TargetDBClusterParameterGroupDescription:  A description for the copied DB cluster parameter group.<br/>TargetDBClusterParameterGroupIdentifier:  The identifier for the copied DB cluster parameter group. Constraints: <ul> <li> Can't be null, empty, or blank </li> <li> Must contain from 1 to 255 letters, numbers, or hyphens </li> <li> First character must be a letter </li> <li> Can't end with a hyphen or contain two consecutive hyphens </li> </ul> Example: <code>my-cluster-param-group1</code> <br/>SourceDBClusterParameterGroupIdentifier:  The identifier or Amazon Resource Name (ARN) for the source DB cluster parameter group. For information about creating an ARN, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/USER_Tagging.ARN.html#USER_Tagging.ARN.Constructing"> Constructing an ARN for Amazon RDS</a> in the <i>Amazon Aurora User Guide</i>.  Constraints: <ul> <li> Must specify a valid DB cluster parameter group. </li> <li> If the source DB cluster parameter group is in the same AWS Region as the copy, specify a valid DB parameter group identifier, for example <code>my-db-cluster-param-group</code>, or a valid ARN. </li> <li> If the source DB parameter group is in a different AWS Region than the copy, specify a valid DB cluster parameter group ARN, for example <code>arn:aws:rds:us-east-1:123456789012:cluster-pg:custom-cluster-group1</code>. </li> </ul><br/>
        :return: DBClusterParameterGroup:  Contains the details of an Amazon RDS DB cluster parameter group.  This data type is used as a response element in the <code>DescribeDBClusterParameterGroups</code> action. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroupArn</i> The Amazon Resource Name (ARN) for the DB cluster parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> Provides the customer-specified description for this DB cluster parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroupFamily</i> Provides the name of the DB parameter group family that this DB cluster parameter group is compatible with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroupName</i> Provides the name of the DB cluster parameter group.<br/>        """
        pass

    def copy_db_cluster_snapshot(self, TargetDBClusterSnapshotIdentifier: str, SourceDBClusterSnapshotIdentifier: str, KmsKeyId: str=None, PreSignedUrl: str=None, CopyTags: BooleanOptional=None, Tags: List[Tag]=None, SourceRegion: str=None) -> CopyDBClusterSnapshotResult:
        """Copies a snapshot of a DB cluster. To copy a DB cluster snapshot from a shared manual DB cluster snapshot, <code>SourceDBClusterSnapshotIdentifier</code> must be the Amazon Resource Name (ARN) of the shared DB cluster snapshot. You can copy an encrypted DB cluster snapshot from another AWS Region. In that case, the AWS Region where you call the <code>CopyDBClusterSnapshot</code> action is the destination AWS Region for the encrypted DB cluster snapshot to be copied to. To copy an encrypted DB cluster snapshot from another AWS Region, you must provide the following values: <ul> <li>  <code>KmsKeyId</code> - The AWS Key Management System (AWS KMS) key identifier for the key to use to encrypt the copy of the DB cluster snapshot in the destination AWS Region. </li> <li>  <code>PreSignedUrl</code> - A URL that contains a Signature Version 4 signed request for the <code>CopyDBClusterSnapshot</code> action to be called in the source AWS Region where the DB cluster snapshot is copied from. The pre-signed URL must be a valid request for the <code>CopyDBClusterSnapshot</code> API action that can be executed in the source AWS Region that contains the encrypted DB cluster snapshot to be copied. The pre-signed URL request must contain the following parameter values: <ul> <li>  <code>KmsKeyId</code> - The KMS key identifier for the key to use to encrypt the copy of the DB cluster snapshot in the destination AWS Region. This is the same identifier for both the <code>CopyDBClusterSnapshot</code> action that is called in the destination AWS Region, and the action contained in the pre-signed URL. </li> <li>  <code>DestinationRegion</code> - The name of the AWS Region that the DB cluster snapshot will be created in. </li> <li>  <code>SourceDBClusterSnapshotIdentifier</code> - The DB cluster snapshot identifier for the encrypted DB cluster snapshot to be copied. This identifier must be in the Amazon Resource Name (ARN) format for the source AWS Region. For example, if you are copying an encrypted DB cluster snapshot from the us-west-2 AWS Region, then your <code>SourceDBClusterSnapshotIdentifier</code> looks like the following example: <code>arn:aws:rds:us-west-2:123456789012:cluster-snapshot:aurora-cluster1-snapshot-20161115</code>. </li> </ul> To learn how to generate a Signature Version 4 signed request, see <a href="https://docs.aws.amazon.com/AmazonS3/latest/API/sigv4-query-string-auth.html"> Authenticating Requests: Using Query Parameters (AWS Signature Version 4)</a> and <a href="https://docs.aws.amazon.com/general/latest/gr/signature-version-4.html"> Signature Version 4 Signing Process</a>. </li> <li>  <code>TargetDBClusterSnapshotIdentifier</code> - The identifier for the new copy of the DB cluster snapshot in the destination AWS Region. </li> <li>  <code>SourceDBClusterSnapshotIdentifier</code> - The DB cluster snapshot identifier for the encrypted DB cluster snapshot to be copied. This identifier must be in the ARN format for the source AWS Region and is the same value as the <code>SourceDBClusterSnapshotIdentifier</code> in the pre-signed URL.  </li> </ul> To cancel the copy operation once it is in progress, delete the target DB cluster snapshot identified by <code>TargetDBClusterSnapshotIdentifier</code> while that DB cluster snapshot is in "copying" status. For more information on copying encrypted DB cluster snapshots from one AWS Region to another, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/USER_CopySnapshot.html"> Copying a Snapshot</a> in the <i>Amazon Aurora User Guide.</i>  For more information on Amazon Aurora, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/CHAP_AuroraOverview.html"> What Is Amazon Aurora?</a> in the <i>Amazon Aurora User Guide.</i>  <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param SourceRegion:  The ID of the region that contains the snapshot to be copied.<br/>Tags:  A list of tags. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Tagging.html">Tagging Amazon RDS Resources</a> in the <i>Amazon RDS User Guide.</i> <br/>CopyTags:  A value that indicates whether to copy all tags from the source DB cluster snapshot to the target DB cluster snapshot. By default, tags are not copied.<br/>PreSignedUrl:  The URL that contains a Signature Version 4 signed request for the <code>CopyDBClusterSnapshot</code> API action in the AWS Region that contains the source DB cluster snapshot to copy. The <code>PreSignedUrl</code> parameter must be used when copying an encrypted DB cluster snapshot from another AWS Region. The pre-signed URL must be a valid request for the <code>CopyDBSClusterSnapshot</code> API action that can be executed in the source AWS Region that contains the encrypted DB cluster snapshot to be copied. The pre-signed URL request must contain the following parameter values: <ul> <li>  <code>KmsKeyId</code> - The AWS KMS key identifier for the key to use to encrypt the copy of the DB cluster snapshot in the destination AWS Region. This is the same identifier for both the <code>CopyDBClusterSnapshot</code> action that is called in the destination AWS Region, and the action contained in the pre-signed URL. </li> <li>  <code>DestinationRegion</code> - The name of the AWS Region that the DB cluster snapshot will be created in. </li> <li>  <code>SourceDBClusterSnapshotIdentifier</code> - The DB cluster snapshot identifier for the encrypted DB cluster snapshot to be copied. This identifier must be in the Amazon Resource Name (ARN) format for the source AWS Region. For example, if you are copying an encrypted DB cluster snapshot from the us-west-2 AWS Region, then your <code>SourceDBClusterSnapshotIdentifier</code> looks like the following example: <code>arn:aws:rds:us-west-2:123456789012:cluster-snapshot:aurora-cluster1-snapshot-20161115</code>. </li> </ul> To learn how to generate a Signature Version 4 signed request, see <a href="https://docs.aws.amazon.com/AmazonS3/latest/API/sigv4-query-string-auth.html"> Authenticating Requests: Using Query Parameters (AWS Signature Version 4)</a> and <a href="https://docs.aws.amazon.com/general/latest/gr/signature-version-4.html"> Signature Version 4 Signing Process</a>.<br/>KmsKeyId:  The AWS AWS KMS key ID for an encrypted DB cluster snapshot. The KMS key ID is the Amazon Resource Name (ARN), KMS key identifier, or the KMS key alias for the KMS encryption key.  If you copy an encrypted DB cluster snapshot from your AWS account, you can specify a value for <code>KmsKeyId</code> to encrypt the copy with a new KMS encryption key. If you don't specify a value for <code>KmsKeyId</code>, then the copy of the DB cluster snapshot is encrypted with the same KMS key as the source DB cluster snapshot.  If you copy an encrypted DB cluster snapshot that is shared from another AWS account, then you must specify a value for <code>KmsKeyId</code>.  To copy an encrypted DB cluster snapshot to another AWS Region, you must set <code>KmsKeyId</code> to the KMS key ID you want to use to encrypt the copy of the DB cluster snapshot in the destination AWS Region. KMS encryption keys are specific to the AWS Region that they are created in, and you can't use encryption keys from one AWS Region in another AWS Region. If you copy an unencrypted DB cluster snapshot and specify a value for the <code>KmsKeyId</code> parameter, an error is returned.<br/>TargetDBClusterSnapshotIdentifier:  The identifier of the new DB cluster snapshot to create from the source DB cluster snapshot. This parameter is not case-sensitive. Constraints: <ul> <li> Must contain from 1 to 63 letters, numbers, or hyphens. </li> <li> First character must be a letter. </li> <li> Can't end with a hyphen or contain two consecutive hyphens. </li> </ul> Example: <code>my-cluster-snapshot2</code> <br/>SourceDBClusterSnapshotIdentifier:  The identifier of the DB cluster snapshot to copy. This parameter is not case-sensitive. You can't copy an encrypted, shared DB cluster snapshot from one AWS Region to another. Constraints: <ul> <li> Must specify a valid system snapshot in the "available" state. </li> <li> If the source snapshot is in the same AWS Region as the copy, specify a valid DB snapshot identifier. </li> <li> If the source snapshot is in a different AWS Region than the copy, specify a valid DB cluster snapshot ARN. For more information, go to <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/USER_CopySnapshot.html#USER_CopySnapshot.AcrossRegions"> Copying Snapshots Across AWS Regions</a> in the <i>Amazon Aurora User Guide.</i>  </li> </ul> Example: <code>my-cluster-snapshot1</code> <br/>
        :return: DBClusterSnapshot:  Contains the details for an Amazon RDS DB cluster snapshot  This data type is used as a response element in the <code>DescribeDBClusterSnapshots</code> action. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceDBClusterSnapshotArn</i> If the DB cluster snapshot was copied from a source DB cluster snapshot, the Amazon Resource Name (ARN) for the source DB cluster snapshot, otherwise, a null value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotArn</i> The Amazon Resource Name (ARN) for the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is true, the AWS KMS key identifier for the encrypted DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster snapshot is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the percentage of the estimated data that has been transferred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotType</i> Provides the type of the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> Provides the license model information for this DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Provides the version of the database engine for this DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Provides the master username for the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VPC ID associated with the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the DB cluster was listening on at the time of the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the status of this DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> Specifies the allocated storage size in gibibytes (GiB).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Specifies the name of the database engine.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCreateTime</i> Provides the time when the snapshot was taken, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Specifies the DB cluster identifier of the DB cluster that this DB cluster snapshot was created from.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotIdentifier</i> Specifies the identifier for the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of Availability Zones (AZs) where instances in the DB cluster snapshot can be restored.<br/>        """
        pass

    def copy_db_parameter_group(self, TargetDBParameterGroupDescription: str, TargetDBParameterGroupIdentifier: str, SourceDBParameterGroupIdentifier: str, Tags: List[Tag]=None) -> CopyDBParameterGroupResult:
        """Copies the specified DB parameter group.

        :param self:
        :param Tags:  A list of tags. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Tagging.html">Tagging Amazon RDS Resources</a> in the <i>Amazon RDS User Guide.</i> <br/>TargetDBParameterGroupDescription:  A description for the copied DB parameter group.<br/>TargetDBParameterGroupIdentifier:  The identifier for the copied DB parameter group. Constraints: <ul> <li> Can't be null, empty, or blank </li> <li> Must contain from 1 to 255 letters, numbers, or hyphens </li> <li> First character must be a letter </li> <li> Can't end with a hyphen or contain two consecutive hyphens </li> </ul> Example: <code>my-db-parameter-group</code> <br/>SourceDBParameterGroupIdentifier:   The identifier or ARN for the source DB parameter group. For information about creating an ARN, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Tagging.ARN.html#USER_Tagging.ARN.Constructing"> Constructing an ARN for Amazon RDS</a> in the <i>Amazon RDS User Guide</i>.  Constraints: <ul> <li> Must specify a valid DB parameter group. </li> <li> Must specify a valid DB parameter group identifier, for example <code>my-db-param-group</code>, or a valid ARN. </li> </ul><br/>
        :return: DBParameterGroup:  Contains the details of an Amazon RDS DB parameter group.  This data type is used as a response element in the <code>DescribeDBParameterGroups</code> action. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroupArn</i> The Amazon Resource Name (ARN) for the DB parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> Provides the customer-specified description for this DB parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroupFamily</i> Provides the name of the DB parameter group family that this DB parameter group is compatible with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroupName</i> Provides the name of the DB parameter group.<br/>        """
        pass

    def copy_db_snapshot(self, TargetDBSnapshotIdentifier: str, SourceDBSnapshotIdentifier: str, KmsKeyId: str=None, Tags: List[Tag]=None, CopyTags: BooleanOptional=None, PreSignedUrl: str=None, OptionGroupName: str=None, SourceRegion: str=None) -> CopyDBSnapshotResult:
        """Copies the specified DB snapshot. The source DB snapshot must be in the "available" state. You can copy a snapshot from one AWS Region to another. In that case, the AWS Region where you call the <code>CopyDBSnapshot</code> action is the destination AWS Region for the DB snapshot copy.  For more information about copying snapshots, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_CopyDBSnapshot.html">Copying a DB Snapshot</a> in the <i>Amazon RDS User Guide.</i> 

        :param self:
        :param SourceRegion:  The ID of the region that contains the snapshot to be copied.<br/>OptionGroupName:  The name of an option group to associate with the copy of the snapshot. Specify this option if you are copying a snapshot from one AWS Region to another, and your DB instance uses a nondefault option group. If your source DB instance uses Transparent Data Encryption for Oracle or Microsoft SQL Server, you must specify this option when copying across AWS Regions. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_CopySnapshot.html#USER_CopySnapshot.Options">Option Group Considerations</a> in the <i>Amazon RDS User Guide.</i> <br/>PreSignedUrl:  The URL that contains a Signature Version 4 signed request for the <code>CopyDBSnapshot</code> API action in the source AWS Region that contains the source DB snapshot to copy.  You must specify this parameter when you copy an encrypted DB snapshot from another AWS Region by using the Amazon RDS API. You can specify the <code>--source-region</code> option instead of this parameter when you copy an encrypted DB snapshot from another AWS Region by using the AWS CLI.  The presigned URL must be a valid request for the <code>CopyDBSnapshot</code> API action that can be executed in the source AWS Region that contains the encrypted DB snapshot to be copied. The presigned URL request must contain the following parameter values:  <ul> <li>  <code>DestinationRegion</code> - The AWS Region that the encrypted DB snapshot is copied to. This AWS Region is the same one where the <code>CopyDBSnapshot</code> action is called that contains this presigned URL.  For example, if you copy an encrypted DB snapshot from the us-west-2 AWS Region to the us-east-1 AWS Region, then you call the <code>CopyDBSnapshot</code> action in the us-east-1 AWS Region and provide a presigned URL that contains a call to the <code>CopyDBSnapshot</code> action in the us-west-2 AWS Region. For this example, the <code>DestinationRegion</code> in the presigned URL must be set to the us-east-1 AWS Region.  </li> <li>  <code>KmsKeyId</code> - The AWS KMS key identifier for the key to use to encrypt the copy of the DB snapshot in the destination AWS Region. This is the same identifier for both the <code>CopyDBSnapshot</code> action that is called in the destination AWS Region, and the action contained in the presigned URL.  </li> <li>  <code>SourceDBSnapshotIdentifier</code> - The DB snapshot identifier for the encrypted snapshot to be copied. This identifier must be in the Amazon Resource Name (ARN) format for the source AWS Region. For example, if you are copying an encrypted DB snapshot from the us-west-2 AWS Region, then your <code>SourceDBSnapshotIdentifier</code> looks like the following example: <code>arn:aws:rds:us-west-2:123456789012:snapshot:mysql-instance1-snapshot-20161115</code>.  </li> </ul> To learn how to generate a Signature Version 4 signed request, see <a href="https://docs.aws.amazon.com/AmazonS3/latest/API/sigv4-query-string-auth.html">Authenticating Requests: Using Query Parameters (AWS Signature Version 4)</a> and <a href="https://docs.aws.amazon.com/general/latest/gr/signature-version-4.html">Signature Version 4 Signing Process</a>. <br/>CopyTags:  A value that indicates whether to copy all tags from the source DB snapshot to the target DB snapshot. By default, tags are not copied.<br/>Tags:  A list of tags. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Tagging.html">Tagging Amazon RDS Resources</a> in the <i>Amazon RDS User Guide.</i> <br/>KmsKeyId:  The AWS KMS key ID for an encrypted DB snapshot. The KMS key ID is the Amazon Resource Name (ARN), KMS key identifier, or the KMS key alias for the KMS encryption key.  If you copy an encrypted DB snapshot from your AWS account, you can specify a value for this parameter to encrypt the copy with a new KMS encryption key. If you don't specify a value for this parameter, then the copy of the DB snapshot is encrypted with the same KMS key as the source DB snapshot.  If you copy an encrypted DB snapshot that is shared from another AWS account, then you must specify a value for this parameter.  If you specify this parameter when you copy an unencrypted snapshot, the copy is encrypted.  If you copy an encrypted snapshot to a different AWS Region, then you must specify a KMS key for the destination AWS Region. KMS encryption keys are specific to the AWS Region that they are created in, and you can't use encryption keys from one AWS Region in another AWS Region. <br/>TargetDBSnapshotIdentifier:  The identifier for the copy of the snapshot.  Constraints: <ul> <li> Can't be null, empty, or blank </li> <li> Must contain from 1 to 255 letters, numbers, or hyphens </li> <li> First character must be a letter </li> <li> Can't end with a hyphen or contain two consecutive hyphens </li> </ul> Example: <code>my-db-snapshot</code> <br/>SourceDBSnapshotIdentifier:  The identifier for the source DB snapshot. If the source snapshot is in the same AWS Region as the copy, specify a valid DB snapshot identifier. For example, you might specify <code>rds:mysql-instance1-snapshot-20130805</code>.  If the source snapshot is in a different AWS Region than the copy, specify a valid DB snapshot ARN. For example, you might specify <code>arn:aws:rds:us-west-2:123456789012:snapshot:mysql-instance1-snapshot-20130805</code>.  If you are copying from a shared manual DB snapshot, this parameter must be the Amazon Resource Name (ARN) of the shared DB snapshot.  If you are copying an encrypted snapshot this parameter must be in the ARN format for the source AWS Region, and must match the <code>SourceDBSnapshotIdentifier</code> in the <code>PreSignedUrl</code> parameter.  Constraints: <ul> <li> Must specify a valid system snapshot in the "available" state. </li> </ul> Example: <code>rds:mydb-2012-04-02-00-01</code>  Example: <code>arn:aws:rds:us-west-2:123456789012:snapshot:mysql-instance1-snapshot-20130805</code> <br/>
        :return: DBSnapshot:  Contains the details of an Amazon RDS DB snapshot.  This data type is used as a response element in the <code>DescribeDBSnapshots</code> action. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbiResourceId</i> The identifier for the source DB instance, which can't be changed and which is unique to an AWS Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessorFeatures</i> The number of CPU cores and the number of threads per core for the DB instance class of the DB instance when the DB snapshot was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timezone</i> The time zone of the DB snapshot. In most cases, the <code>Timezone</code> element is empty. <code>Timezone</code> content appears only for snapshots taken from Microsoft SQL Server DB instances that were created with a time zone specified. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSnapshotArn</i> The Amazon Resource Name (ARN) for the DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i>  If <code>Encrypted</code> is true, the AWS KMS key identifier for the encrypted DB snapshot. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encrypted</i> Specifies whether the DB snapshot is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TdeCredentialArn</i> The ARN from the key store with which to associate the instance for TDE encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type associated with DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceDBSnapshotIdentifier</i> The DB snapshot Amazon Resource Name (ARN) that the DB snapshot was copied from. It only has value in case of cross-customer or cross-region copy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceRegion</i> The AWS Region that the DB snapshot was created in or copied from.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> The percentage of the estimated data that has been transferred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionGroupName</i> Provides the option group name for the DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the Provisioned IOPS (I/O operations per second) value of the DB instance at the time of the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotType</i> Provides the type of the DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> License model information for the restored DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Specifies the version of the database engine.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Provides the master username for the DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCreateTime</i> Specifies the time when the snapshot was taken, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VPC ID associated with the DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> Specifies the name of the Availability Zone the DB instance was located in at the time of the DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine was listening on at the time of the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the status of this DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> Specifies the allocated storage size in gibibytes (GiB).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Specifies the name of the database engine.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCreateTime</i> Provides the time when the snapshot was taken, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i> Specifies the DB instance identifier of the DB instance this DB snapshot was created from.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSnapshotIdentifier</i> Specifies the identifier for the DB snapshot.<br/>        """
        pass

    def copy_option_group(self, TargetOptionGroupDescription: str, TargetOptionGroupIdentifier: str, SourceOptionGroupIdentifier: str, Tags: List[Tag]=None) -> CopyOptionGroupResult:
        """Copies the specified option group.

        :param self:
        :param Tags:  A list of tags. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Tagging.html">Tagging Amazon RDS Resources</a> in the <i>Amazon RDS User Guide.</i> <br/>TargetOptionGroupDescription:  The description for the copied option group.<br/>TargetOptionGroupIdentifier:  The identifier for the copied option group. Constraints: <ul> <li> Can't be null, empty, or blank </li> <li> Must contain from 1 to 255 letters, numbers, or hyphens </li> <li> First character must be a letter </li> <li> Can't end with a hyphen or contain two consecutive hyphens </li> </ul> Example: <code>my-option-group</code> <br/>SourceOptionGroupIdentifier:  The identifier or ARN for the source option group. For information about creating an ARN, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Tagging.ARN.html#USER_Tagging.ARN.Constructing"> Constructing an ARN for Amazon RDS</a> in the <i>Amazon RDS User Guide</i>.  Constraints: <ul> <li> Must specify a valid option group. </li> <li> If the source option group is in the same AWS Region as the copy, specify a valid option group identifier, for example <code>my-option-group</code>, or a valid ARN. </li> <li> If the source option group is in a different AWS Region than the copy, specify a valid option group ARN, for example <code>arn:aws:rds:us-west-2:123456789012:og:special-options</code>. </li> </ul><br/>
        :return: OptionGroup:  <p/>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionGroupArn</i> The Amazon Resource Name (ARN) for the option group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> If <b>AllowsVpcAndNonVpcInstanceMemberships</b> is <code>false</code>, this field is blank. If <b>AllowsVpcAndNonVpcInstanceMemberships</b> is <code>true</code> and this field is blank, then this option group can be applied to both VPC and non-VPC instances. If this field contains a value, then this option group can only be applied to instances that are in the VPC indicated by this field. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowsVpcAndNonVpcInstanceMemberships</i> Indicates whether this option group can be applied to both VPC and non-VPC instances. The value <code>true</code> indicates the option group can be applied to both VPC and non-VPC instances. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> Indicates what options are available in the option group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MajorEngineVersion</i> Indicates the major engine version associated with this option group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineName</i> Indicates the name of the engine that this option group can be applied to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionGroupDescription</i> Provides a description of the option group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionGroupName</i> Specifies the name of the option group.<br/>        """
        pass

    def create_db_cluster(self, Engine: str, DBClusterIdentifier: str, AvailabilityZones: List[String]=None, BackupRetentionPeriod: int=None, CharacterSetName: str=None, DatabaseName: str=None, DBClusterParameterGroupName: str=None, VpcSecurityGroupIds: List[String]=None, DBSubnetGroupName: str=None, EngineVersion: str=None, Port: int=None, MasterUsername: str=None, MasterUserPassword: str=None, OptionGroupName: str=None, PreferredBackupWindow: str=None, PreferredMaintenanceWindow: str=None, ReplicationSourceIdentifier: str=None, Tags: List[Tag]=None, StorageEncrypted: BooleanOptional=None, KmsKeyId: str=None, PreSignedUrl: str=None, EnableIAMDatabaseAuthentication: BooleanOptional=None, BacktrackWindow: LongOptional=None, EnableCloudwatchLogsExports: List[String]=None, EngineMode: str=None, ScalingConfiguration: ScalingConfiguration=None, DeletionProtection: BooleanOptional=None, GlobalClusterIdentifier: str=None, CopyTagsToSnapshot: BooleanOptional=None, SourceRegion: str=None) -> CreateDBClusterResult:
        """Creates a new Amazon Aurora DB cluster. You can use the <code>ReplicationSourceIdentifier</code> parameter to create the DB cluster as a Read Replica of another DB cluster or Amazon RDS MySQL DB instance. For cross-region replication where the DB cluster identified by <code>ReplicationSourceIdentifier</code> is encrypted, you must also specify the <code>PreSignedUrl</code> parameter. For more information on Amazon Aurora, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/CHAP_AuroraOverview.html"> What Is Amazon Aurora?</a> in the <i>Amazon Aurora User Guide.</i>  <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param SourceRegion:  The ID of the region that contains the source for the db cluster.<br/>CopyTagsToSnapshot:  A value that indicates whether to copy all tags from the DB cluster to snapshots of the DB cluster. The default is not to copy them.<br/>GlobalClusterIdentifier:   The global cluster ID of an Aurora cluster that becomes the primary cluster in the new global database cluster. <br/>DeletionProtection:  A value that indicates whether the DB cluster has deletion protection enabled. The database can't be deleted when deletion protection is enabled. By default, deletion protection is disabled.<br/>ScalingConfiguration:  For DB clusters in <code>serverless</code> DB engine mode, the scaling properties of the DB cluster.:<br/>EngineMode:  The DB engine mode of the DB cluster, either <code>provisioned</code>, <code>serverless</code>, <code>parallelquery</code>, or <code>global</code>.<br/>EnableCloudwatchLogsExports:  The list of log types that need to be enabled for exporting to CloudWatch Logs. The values in the list depend on the DB engine being used. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/USER_LogAccess.html#USER_LogAccess.Procedural.UploadtoCloudWatch">Publishing Database Logs to Amazon CloudWatch Logs</a> in the <i>Amazon Aurora User Guide</i>.<br/>BacktrackWindow:  The target backtrack window, in seconds. To disable backtracking, set this value to 0.  Default: 0 Constraints: <ul> <li> If specified, this value must be set to a number from 0 to 259,200 (72 hours). </li> </ul><br/>EnableIAMDatabaseAuthentication:  A value that indicates whether to enable mapping of AWS Identity and Access Management (IAM) accounts to database accounts. By default, mapping is disabled.<br/>PreSignedUrl:  A URL that contains a Signature Version 4 signed request for the <code>CreateDBCluster</code> action to be called in the source AWS Region where the DB cluster is replicated from. You only need to specify <code>PreSignedUrl</code> when you are performing cross-region replication from an encrypted DB cluster. The pre-signed URL must be a valid request for the <code>CreateDBCluster</code> API action that can be executed in the source AWS Region that contains the encrypted DB cluster to be copied. The pre-signed URL request must contain the following parameter values: <ul> <li>  <code>KmsKeyId</code> - The AWS KMS key identifier for the key to use to encrypt the copy of the DB cluster in the destination AWS Region. This should refer to the same KMS key for both the <code>CreateDBCluster</code> action that is called in the destination AWS Region, and the action contained in the pre-signed URL. </li> <li>  <code>DestinationRegion</code> - The name of the AWS Region that Aurora Read Replica will be created in. </li> <li>  <code>ReplicationSourceIdentifier</code> - The DB cluster identifier for the encrypted DB cluster to be copied. This identifier must be in the Amazon Resource Name (ARN) format for the source AWS Region. For example, if you are copying an encrypted DB cluster from the us-west-2 AWS Region, then your <code>ReplicationSourceIdentifier</code> would look like Example: <code>arn:aws:rds:us-west-2:123456789012:cluster:aurora-cluster1</code>. </li> </ul> To learn how to generate a Signature Version 4 signed request, see <a href="https://docs.aws.amazon.com/AmazonS3/latest/API/sigv4-query-string-auth.html"> Authenticating Requests: Using Query Parameters (AWS Signature Version 4)</a> and <a href="https://docs.aws.amazon.com/general/latest/gr/signature-version-4.html"> Signature Version 4 Signing Process</a>.<br/>KmsKeyId:  The AWS KMS key identifier for an encrypted DB cluster. The KMS key identifier is the Amazon Resource Name (ARN) for the KMS encryption key. If you are creating a DB cluster with the same AWS account that owns the KMS encryption key used to encrypt the new DB cluster, then you can use the KMS key alias instead of the ARN for the KMS encryption key. If an encryption key is not specified in <code>KmsKeyId</code>: <ul> <li> If <code>ReplicationSourceIdentifier</code> identifies an encrypted source, then Amazon RDS will use the encryption key used to encrypt the source. Otherwise, Amazon RDS will use your default encryption key.  </li> <li> If the <code>StorageEncrypted</code> parameter is enabled and <code>ReplicationSourceIdentifier</code> is not specified, then Amazon RDS will use your default encryption key. </li> </ul> AWS KMS creates the default encryption key for your AWS account. Your AWS account has a different default encryption key for each AWS Region. If you create a Read Replica of an encrypted DB cluster in another AWS Region, you must set <code>KmsKeyId</code> to a KMS key ID that is valid in the destination AWS Region. This key is used to encrypt the Read Replica in that AWS Region.<br/>StorageEncrypted:  A value that indicates whether the DB cluster is encrypted.<br/>Tags:  Tags to assign to the DB cluster.<br/>ReplicationSourceIdentifier:  The Amazon Resource Name (ARN) of the source DB instance or DB cluster if this DB cluster is created as a Read Replica.<br/>PreferredMaintenanceWindow:  The weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC). Format: <code>ddd:hh24:mi-ddd:hh24:mi</code>  The default is a 30-minute window selected at random from an 8-hour block of time for each AWS Region, occurring on a random day of the week. To see the time blocks available, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/USER_UpgradeDBInstance.Maintenance.html#AdjustingTheMaintenanceWindow.Aurora"> Adjusting the Preferred DB Cluster Maintenance Window</a> in the <i>Amazon Aurora User Guide.</i>  Valid Days: Mon, Tue, Wed, Thu, Fri, Sat, Sun. Constraints: Minimum 30-minute window.<br/>PreferredBackupWindow:  The daily time range during which automated backups are created if automated backups are enabled using the <code>BackupRetentionPeriod</code> parameter.  The default is a 30-minute window selected at random from an 8-hour block of time for each AWS Region. To see the time blocks available, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/USER_UpgradeDBInstance.Maintenance.html#AdjustingTheMaintenanceWindow.Aurora"> Adjusting the Preferred DB Cluster Maintenance Window</a> in the <i>Amazon Aurora User Guide.</i>  Constraints: <ul> <li> Must be in the format <code>hh24:mi-hh24:mi</code>. </li> <li> Must be in Universal Coordinated Time (UTC). </li> <li> Must not conflict with the preferred maintenance window. </li> <li> Must be at least 30 minutes. </li> </ul><br/>OptionGroupName:  A value that indicates that the DB cluster should be associated with the specified option group. Permanent options can't be removed from an option group. The option group can't be removed from a DB cluster once it is associated with a DB cluster.<br/>MasterUserPassword:  The password for the master database user. This password can contain any printable ASCII character except "/", triple-quotes, or "@". Constraints: Must contain from 8 to 41 characters.<br/>MasterUsername:  The name of the master user for the DB cluster. Constraints: <ul> <li> Must be 1 to 16 letters or numbers. </li> <li> First character must be a letter. </li> <li> Can't be a reserved word for the chosen database engine. </li> </ul><br/>Port:  The port number on which the instances in the DB cluster accept connections.  Default: <code>3306</code> if engine is set as aurora or <code>5432</code> if set to aurora-postgresql. <br/>EngineVersion:  The version number of the database engine to use.  <b>Aurora MySQL</b>  Example: <code>5.6.10a</code>, <code>5.7.12</code>   <b>Aurora PostgreSQL</b>  Example: <code>9.6.3</code> <br/>Engine:  The name of the database engine to be used for this DB cluster. Valid Values: <code>aurora</code> (for MySQL 5.6-compatible Aurora), <code>aurora-mysql</code> (for MySQL 5.7-compatible Aurora), and <code>aurora-postgresql</code> <br/>DBSubnetGroupName:  A DB subnet group to associate with this DB cluster. Constraints: Must match the name of an existing DBSubnetGroup. Must not be default. Example: <code>mySubnetgroup</code> <br/>VpcSecurityGroupIds:  A list of EC2 VPC security groups to associate with this DB cluster.<br/>DBClusterParameterGroupName:   The name of the DB cluster parameter group to associate with this DB cluster. If this argument is omitted, <code>default.aurora5.6</code> is used.  Constraints: <ul> <li> If supplied, must match the name of an existing DB cluster parameter group. </li> </ul><br/>DBClusterIdentifier:  The DB cluster identifier. This parameter is stored as a lowercase string. Constraints: <ul> <li> Must contain from 1 to 63 letters, numbers, or hyphens. </li> <li> First character must be a letter. </li> <li> Can't end with a hyphen or contain two consecutive hyphens. </li> </ul> Example: <code>my-cluster1</code> <br/>DatabaseName:  The name for your database of up to 64 alpha-numeric characters. If you do not provide a name, Amazon RDS will not create a database in the DB cluster you are creating.<br/>CharacterSetName:  A value that indicates that the DB cluster should be associated with the specified CharacterSet.<br/>BackupRetentionPeriod:  The number of days for which automated backups are retained. Default: 1 Constraints: <ul> <li> Must be a value from 1 to 35 </li> </ul><br/>AvailabilityZones:  A list of Availability Zones (AZs) where instances in the DB cluster can be created. For information on AWS Regions and Availability Zones, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/Concepts.RegionsAndAvailabilityZones.html">Choosing the Regions and Availability Zones</a> in the <i>Amazon Aurora User Guide</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeoutAction</i> The action to take when the timeout is reached, either <code>ForceApplyCapacityChange</code> or <code>RollbackCapacityChange</code>.  <code>ForceApplyCapacityChange</code> sets the capacity to the specified value as soon as possible.  <code>RollbackCapacityChange</code>, the default, ignores the capacity change if a scaling point is not found in the timeout period. <important> If you specify <code>ForceApplyCapacityChange</code>, connections that prevent Aurora Serverless from finding a scaling point might be dropped. </important> For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.how-it-works.html#aurora-serverless.how-it-works.auto-scaling"> Autoscaling for Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondsUntilAutoPause</i> The time, in seconds, before an Aurora DB cluster in <code>serverless</code> mode is paused.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoPause</i> A value that indicates whether to allow or disallow automatic pause for an Aurora DB cluster in <code>serverless</code> DB engine mode. A DB cluster can be paused only when it's idle (it has no connections). <note> If a DB cluster is paused for more than seven days, the DB cluster might be backed up with a snapshot. In this case, the DB cluster is restored when there is a request to connect to it.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxCapacity</i> The maximum capacity for an Aurora DB cluster in <code>serverless</code> DB engine mode. Valid capacity values are <code>1</code>, <code>2</code>, <code>4</code>, <code>8</code>, <code>16</code>, <code>32</code>, <code>64</code>, <code>128</code>, and <code>256</code>. The maximum capacity must be greater than or equal to the minimum capacity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MinCapacity</i> The minimum capacity for an Aurora DB cluster in <code>serverless</code> DB engine mode. Valid capacity values are <code>1</code>, <code>2</code>, <code>4</code>, <code>8</code>, <code>16</code>, <code>32</code>, <code>64</code>, <code>128</code>, and <code>256</code>. The minimum capacity must be less than or equal to the maximum capacity.<br/>
        :return: DBCluster:  Contains the details of an Amazon Aurora DB cluster.  This data type is used as a response element in the <code>DescribeDBClusters</code>, <code>StopDBCluster</code>, and <code>StartDBCluster</code> actions. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CrossAccountClone</i> Specifies whether the DB cluster is a clone of a DB cluster owned by a different AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyTagsToSnapshot</i> Specifies whether tags are copied from the DB cluster to snapshots of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamKinesisStreamName</i> The name of the Amazon Kinesis data stream used for the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamKmsKeyId</i> The AWS KMS key identifier used for encrypting messages in the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamStatus</i> The status of the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamMode</i> The mode of the database activity stream. Database events such as a change or access generate an activity stream event. The database session can handle these events either synchronously or asynchronously. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HttpEndpointEnabled</i> A value that indicates whether the HTTP endpoint for an Aurora Serverless DB cluster is enabled. When enabled, the HTTP endpoint provides a connectionless web service API for running SQL queries on the Aurora Serverless DB cluster. You can also query your database from inside the RDS console with the query editor. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/data-api.html">Using the Data API for Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i> Indicates if the DB cluster has deletion protection enabled. The database can't be deleted when deletion protection is enabled. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ScalingConfigurationInfo</i> Shows the scaling configuration for an Aurora DB cluster in <code>serverless</code> DB engine mode. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ScalingConfigurationInfo</b>: Shows the scaling configuration for an Aurora DB cluster in <code>serverless</code> DB engine mode. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineMode</i> The DB engine mode of the DB cluster, either <code>provisioned</code>, <code>serverless</code>, or <code>parallelquery</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Capacity</i> The current capacity of an Aurora Serverless DB cluster. The capacity is 0 (zero) when the cluster is paused. For more information about Aurora Serverless, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB cluster is configured to export to CloudWatch Logs. Log types vary by DB engine. For information about the log types for each DB engine, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/USER_LogAccess.html">Amazon RDS Database Log Files</a> in the <i>Amazon Aurora User Guide.</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BacktrackConsumedChangeRecords</i> The number of change records stored for Backtrack.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BacktrackWindow</i> The target backtrack window, in seconds. If this value is set to 0, backtracking is disabled for the DB cluster. Otherwise, backtracking is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestBacktrackTime</i> The earliest time to which a DB cluster can be backtracked.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloneGroupId</i> Identifies the clone group to which the DB cluster is associated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> A value that indicates whether the mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i> Provides a list of the AWS Identity and Access Management (IAM) roles that are associated with the DB cluster. IAM roles that are associated with a DB cluster grant permission for the DB cluster to access other AWS services on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterArn</i> The Amazon Resource Name (ARN) for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbClusterResourceId</i> The AWS Region-unique, immutable identifier for the DB cluster. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB cluster is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is enabled, the AWS KMS key identifier for the encrypted DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security groups that the DB cluster belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterMembers</i> Provides the list of instances that make up the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSourceIdentifier</i> Contains the identifier of the source DB cluster if this DB cluster is a Read Replica.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i> Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterOptionGroupMemberships</i> Provides the list of option group memberships for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies whether the DB cluster has instances in multiple Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomEndpoints</i> Identifies all custom endpoints associated with the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReaderEndpoint</i> The reader endpoint for the DB cluster. The reader endpoint for a DB cluster load-balances connections across the Aurora Replicas that are available in a DB cluster. As clients request new connections to the reader endpoint, Aurora distributes the connection requests among the Aurora Replicas in the DB cluster. This functionality can help balance your read workload across multiple Aurora Replicas in your DB cluster.  If a failover occurs, and the Aurora Replica that you are connected to is promoted to be the primary instance, your connection is dropped. To continue sending your read workload to other Aurora Replicas in the cluster, you can then reconnect to the reader endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint for the primary instance of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestRestorableTime</i> The earliest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the progress of the operation as a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the current state of this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB cluster, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroup</i> Specifies the name of the DB cluster parameter group for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Contains a user-supplied DB cluster identifier. This identifier is the unique key that identifies a DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> Contains the name of the initial database of this DB cluster that was provided at create time, if one was specified when the DB cluster was created. This same name is returned for the life of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this cluster is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of Availability Zones (AZs) where instances in the DB cluster can be created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> For all database engines except Amazon Aurora, <code>AllocatedStorage</code> specifies the allocated storage size in gibibytes (GiB). For Aurora, <code>AllocatedStorage</code> always returns 1, because Aurora DB cluster storage size is not fixed, but instead automatically adjusts as needed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeoutAction</i> The timeout action of a call to <code>ModifyCurrentDBClusterCapacity</code>, either <code>ForceApplyCapacityChange</code> or <code>RollbackCapacityChange</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondsUntilAutoPause</i> The remaining amount of time, in seconds, before the Aurora DB cluster in <code>serverless</code> mode is paused. A DB cluster can be paused only when it's idle (it has no connections).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoPause</i> A value that indicates whether automatic pause is allowed for the Aurora DB cluster in <code>serverless</code> DB engine mode. When the value is set to false for an Aurora Serverless DB cluster, the DB cluster automatically resumes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxCapacity</i> The maximum capacity for an Aurora DB cluster in <code>serverless</code> DB engine mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinCapacity</i> The maximum capacity for the Aurora DB cluster in <code>serverless</code> DB engine mode.<br/>        """
        pass

    def create_db_cluster_endpoint(self, EndpointType: str, DBClusterEndpointIdentifier: str, DBClusterIdentifier: str, StaticMembers: List[String]=None, ExcludedMembers: List[String]=None) -> DBClusterEndpoint:
        """Creates a new custom endpoint and associates it with an Amazon Aurora DB cluster. <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param ExcludedMembers:  List of DB instance identifiers that aren't part of the custom endpoint group. All other eligible instances are reachable through the custom endpoint. Only relevant if the list of static members is empty.<br/>StaticMembers:  List of DB instance identifiers that are part of the custom endpoint group.<br/>EndpointType:  The type of the endpoint. One of: <code>READER</code>, <code>ANY</code>. <br/>DBClusterEndpointIdentifier:  The identifier to use for the new endpoint. This parameter is stored as a lowercase string.<br/>DBClusterIdentifier:  The DB cluster identifier of the DB cluster associated with the endpoint. This parameter is stored as a lowercase string.<br/>
        :return: DBClusterEndpointArn:  The Amazon Resource Name (ARN) for the endpoint.<br/>ExcludedMembers:  List of DB instance identifiers that aren't part of the custom endpoint group. All other eligible instances are reachable through the custom endpoint. Only relevant if the list of static members is empty.<br/>StaticMembers:  List of DB instance identifiers that are part of the custom endpoint group.<br/>CustomEndpointType:  The type associated with a custom endpoint. One of: <code>READER</code>, <code>ANY</code>.<br/>EndpointType:  The type of the endpoint. One of: <code>READER</code>, <code>WRITER</code>, <code>CUSTOM</code>.<br/>Status:  The current status of the endpoint. One of: <code>creating</code>, <code>available</code>, <code>deleting</code>, <code>modifying</code>.<br/>Endpoint:  The DNS address of the endpoint.<br/>DBClusterEndpointResourceIdentifier:  A unique system-generated identifier for an endpoint. It remains the same for the whole life of the endpoint.<br/>DBClusterIdentifier:  The DB cluster identifier of the DB cluster associated with the endpoint. This parameter is stored as a lowercase string.<br/>DBClusterEndpointIdentifier:  The identifier associated with the endpoint. This parameter is stored as a lowercase string.<br/>        """
        pass

    def create_db_cluster_parameter_group(self, Description: str, DBParameterGroupFamily: str, DBClusterParameterGroupName: str, Tags: List[Tag]=None) -> CreateDBClusterParameterGroupResult:
        """Creates a new DB cluster parameter group. Parameters in a DB cluster parameter group apply to all of the instances in a DB cluster.  A DB cluster parameter group is initially created with the default parameters for the database engine used by instances in the DB cluster. To provide custom values for any of the parameters, you must modify the group after creating it using <code>ModifyDBClusterParameterGroup</code>. Once you've created a DB cluster parameter group, you need to associate it with your DB cluster using <code>ModifyDBCluster</code>. When you associate a new DB cluster parameter group with a running DB cluster, you need to reboot the DB instances in the DB cluster without failover for the new DB cluster parameter group and associated settings to take effect.  <important> After you create a DB cluster parameter group, you should wait at least 5 minutes before creating your first DB cluster that uses that DB cluster parameter group as the default parameter group. This allows Amazon RDS to fully complete the create action before the DB cluster parameter group is used as the default for a new DB cluster. This is especially important for parameters that are critical when creating the default database for a DB cluster, such as the character set for the default database defined by the <code>character_set_database</code> parameter. You can use the <i>Parameter Groups</i> option of the <a href="https://console.aws.amazon.com/rds/">Amazon RDS console</a> or the <code>DescribeDBClusterParameters</code> action to verify that your DB cluster parameter group has been created or modified. </important> For more information on Amazon Aurora, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/CHAP_AuroraOverview.html"> What Is Amazon Aurora?</a> in the <i>Amazon Aurora User Guide.</i>  <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param Tags:  Tags to assign to the DB cluster parameter group.<br/>Description:  The description for the DB cluster parameter group.<br/>DBParameterGroupFamily:  The DB cluster parameter group family name. A DB cluster parameter group can be associated with one and only one DB cluster parameter group family, and can be applied only to a DB cluster running a database engine and engine version compatible with that DB cluster parameter group family.  <b>Aurora MySQL</b>  Example: <code>aurora5.6</code>, <code>aurora-mysql5.7</code>   <b>Aurora PostgreSQL</b>  Example: <code>aurora-postgresql9.6</code> <br/>DBClusterParameterGroupName:  The name of the DB cluster parameter group. Constraints: <ul> <li> Must match the name of an existing DB cluster parameter group. </li> </ul> <note> This value is stored as a lowercase string. </note><br/>
        :return: DBClusterParameterGroup:  Contains the details of an Amazon RDS DB cluster parameter group.  This data type is used as a response element in the <code>DescribeDBClusterParameterGroups</code> action. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroupArn</i> The Amazon Resource Name (ARN) for the DB cluster parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> Provides the customer-specified description for this DB cluster parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroupFamily</i> Provides the name of the DB parameter group family that this DB cluster parameter group is compatible with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroupName</i> Provides the name of the DB cluster parameter group.<br/>        """
        pass

    def create_db_cluster_snapshot(self, DBClusterIdentifier: str, DBClusterSnapshotIdentifier: str, Tags: List[Tag]=None) -> CreateDBClusterSnapshotResult:
        """Creates a snapshot of a DB cluster. For more information on Amazon Aurora, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/CHAP_AuroraOverview.html"> What Is Amazon Aurora?</a> in the <i>Amazon Aurora User Guide.</i>  <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param Tags:  The tags to be assigned to the DB cluster snapshot.<br/>DBClusterIdentifier:  The identifier of the DB cluster to create a snapshot for. This parameter is not case-sensitive. Constraints: <ul> <li> Must match the identifier of an existing DBCluster. </li> </ul> Example: <code>my-cluster1</code> <br/>DBClusterSnapshotIdentifier:  The identifier of the DB cluster snapshot. This parameter is stored as a lowercase string. Constraints: <ul> <li> Must contain from 1 to 63 letters, numbers, or hyphens. </li> <li> First character must be a letter. </li> <li> Can't end with a hyphen or contain two consecutive hyphens. </li> </ul> Example: <code>my-cluster1-snapshot1</code> <br/>
        :return: DBClusterSnapshot:  Contains the details for an Amazon RDS DB cluster snapshot  This data type is used as a response element in the <code>DescribeDBClusterSnapshots</code> action. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceDBClusterSnapshotArn</i> If the DB cluster snapshot was copied from a source DB cluster snapshot, the Amazon Resource Name (ARN) for the source DB cluster snapshot, otherwise, a null value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotArn</i> The Amazon Resource Name (ARN) for the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is true, the AWS KMS key identifier for the encrypted DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster snapshot is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the percentage of the estimated data that has been transferred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotType</i> Provides the type of the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> Provides the license model information for this DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Provides the version of the database engine for this DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Provides the master username for the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VPC ID associated with the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the DB cluster was listening on at the time of the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the status of this DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> Specifies the allocated storage size in gibibytes (GiB).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Specifies the name of the database engine.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCreateTime</i> Provides the time when the snapshot was taken, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Specifies the DB cluster identifier of the DB cluster that this DB cluster snapshot was created from.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotIdentifier</i> Specifies the identifier for the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of Availability Zones (AZs) where instances in the DB cluster snapshot can be restored.<br/>        """
        pass

    def create_db_instance(self, Engine: str, DBInstanceClass: str, DBInstanceIdentifier: str, DBName: str=None, AllocatedStorage: int=None, MasterUsername: str=None, MasterUserPassword: str=None, DBSecurityGroups: List[String]=None, VpcSecurityGroupIds: List[String]=None, AvailabilityZone: str=None, DBSubnetGroupName: str=None, PreferredMaintenanceWindow: str=None, DBParameterGroupName: str=None, BackupRetentionPeriod: int=None, PreferredBackupWindow: str=None, Port: int=None, MultiAZ: BooleanOptional=None, EngineVersion: str=None, AutoMinorVersionUpgrade: BooleanOptional=None, LicenseModel: str=None, Iops: int=None, OptionGroupName: str=None, CharacterSetName: str=None, PubliclyAccessible: BooleanOptional=None, Tags: List[Tag]=None, DBClusterIdentifier: str=None, StorageType: str=None, TdeCredentialArn: str=None, TdeCredentialPassword: str=None, StorageEncrypted: BooleanOptional=None, KmsKeyId: str=None, Domain: str=None, CopyTagsToSnapshot: BooleanOptional=None, MonitoringInterval: int=None, MonitoringRoleArn: str=None, DomainIAMRoleName: str=None, PromotionTier: int=None, Timezone: str=None, EnableIAMDatabaseAuthentication: BooleanOptional=None, EnablePerformanceInsights: BooleanOptional=None, PerformanceInsightsKMSKeyId: str=None, PerformanceInsightsRetentionPeriod: int=None, EnableCloudwatchLogsExports: List[String]=None, ProcessorFeatures: List[ProcessorFeature]=None, DeletionProtection: BooleanOptional=None, MaxAllocatedStorage: int=None) -> CreateDBInstanceResult:
        """Creates a new DB instance.

        :param self:
        :param MaxAllocatedStorage:  The upper limit to which Amazon RDS can automatically scale the storage of the DB instance.<br/>DeletionProtection:  A value that indicates whether the DB instance has deletion protection enabled. The database can't be deleted when deletion protection is enabled. By default, deletion protection is disabled. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_DeleteInstance.html"> Deleting a DB Instance</a>. <br/>ProcessorFeatures:  The number of CPU cores and the number of threads per core for the DB instance class of the DB instance.<br/>EnableCloudwatchLogsExports:  The list of log types that need to be enabled for exporting to CloudWatch Logs. The values in the list depend on the DB engine being used. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_LogAccess.html#USER_LogAccess.Procedural.UploadtoCloudWatch">Publishing Database Logs to Amazon CloudWatch Logs </a> in the <i>Amazon Relational Database Service User Guide</i>.<br/>PerformanceInsightsRetentionPeriod:  The amount of time, in days, to retain Performance Insights data. Valid values are 7 or 731 (2 years). <br/>PerformanceInsightsKMSKeyId:  The AWS KMS key identifier for encryption of Performance Insights data. The KMS key ID is the Amazon Resource Name (ARN), KMS key identifier, or the KMS key alias for the KMS encryption key. If you do not specify a value for <code>PerformanceInsightsKMSKeyId</code>, then Amazon RDS uses your default encryption key. AWS KMS creates the default encryption key for your AWS account. Your AWS account has a different default encryption key for each AWS Region.<br/>EnablePerformanceInsights:  A value that indicates whether to enable Performance Insights for the DB instance.  For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_PerfInsights.html">Using Amazon Performance Insights</a> in the <i>Amazon Relational Database Service User Guide</i>. <br/>EnableIAMDatabaseAuthentication:  A value that indicates whether to enable mapping of AWS Identity and Access Management (IAM) accounts to database accounts. By default, mapping is disabled. You can enable IAM database authentication for the following database engines:  <b>Amazon Aurora</b>  Not applicable. Mapping AWS IAM accounts to database accounts is managed by the DB cluster.  <b>MySQL</b>  <ul> <li> For MySQL 5.6, minor version 5.6.34 or higher </li> <li> For MySQL 5.7, minor version 5.7.16 or higher </li> </ul><br/>Timezone:  The time zone of the DB instance. The time zone parameter is currently supported only by <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/CHAP_SQLServer.html#SQLServer.Concepts.General.TimeZone">Microsoft SQL Server</a>. <br/>PromotionTier:  A value that specifies the order in which an Aurora Replica is promoted to the primary instance after a failure of the existing primary instance. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/Aurora.Managing.Backups.html#Aurora.Managing.FaultTolerance"> Fault Tolerance for an Aurora DB Cluster</a> in the <i>Amazon Aurora User Guide</i>.  Default: 1 Valid Values: 0 - 15<br/>DomainIAMRoleName:  Specify the name of the IAM role to be used when making API calls to the Directory Service.<br/>MonitoringRoleArn:  The ARN for the IAM role that permits RDS to send enhanced monitoring metrics to Amazon CloudWatch Logs. For example, <code>arn:aws:iam:123456789012:role/emaccess</code>. For information on creating a monitoring role, go to <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Monitoring.OS.html#USER_Monitoring.OS.Enabling">Setting Up and Enabling Enhanced Monitoring</a> in the <i>Amazon RDS User Guide</i>. If <code>MonitoringInterval</code> is set to a value other than 0, then you must supply a <code>MonitoringRoleArn</code> value.<br/>MonitoringInterval:  The interval, in seconds, between points when Enhanced Monitoring metrics are collected for the DB instance. To disable collecting Enhanced Monitoring metrics, specify 0. The default is 0. If <code>MonitoringRoleArn</code> is specified, then you must also set <code>MonitoringInterval</code> to a value other than 0. Valid Values: <code>0, 1, 5, 10, 15, 30, 60</code> <br/>CopyTagsToSnapshot:  A value that indicates whether to copy tags from the DB instance to snapshots of the DB instance. By default, tags are not copied.  <b>Amazon Aurora</b>  Not applicable. Copying tags to snapshots is managed by the DB cluster. Setting this value for an Aurora DB instance has no effect on the DB cluster setting.<br/>Domain:  For an Amazon RDS DB instance that's running Microsoft SQL Server, this parameter specifies the Active Directory directory ID to create the instance in. Amazon RDS uses Windows Authentication to authenticate users that connect to the DB instance. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/DeveloperGuide/USER_SQLServerWinAuth.html">Using Windows Authentication with an Amazon RDS DB Instance Running Microsoft SQL Server</a> in the <i>Amazon RDS User Guide</i>. <br/>KmsKeyId:  The AWS KMS key identifier for an encrypted DB instance. The KMS key identifier is the Amazon Resource Name (ARN) for the KMS encryption key. If you are creating a DB instance with the same AWS account that owns the KMS encryption key used to encrypt the new DB instance, then you can use the KMS key alias instead of the ARN for the KM encryption key.  <b>Amazon Aurora</b>  Not applicable. The KMS key identifier is managed by the DB cluster. For more information, see <code>CreateDBCluster</code>. If <code>StorageEncrypted</code> is enabled, and you do not specify a value for the <code>KmsKeyId</code> parameter, then Amazon RDS will use your default encryption key. AWS KMS creates the default encryption key for your AWS account. Your AWS account has a different default encryption key for each AWS Region.<br/>StorageEncrypted:  A value that indicates whether the DB instance is encrypted. By default, it is not encrypted.  <b>Amazon Aurora</b>  Not applicable. The encryption for DB instances is managed by the DB cluster.<br/>TdeCredentialPassword:  The password for the given ARN from the key store in order to access the device.<br/>TdeCredentialArn:  The ARN from the key store with which to associate the instance for TDE encryption.<br/>StorageType:  Specifies the storage type to be associated with the DB instance.  Valid values: <code>standard | gp2 | io1</code>   If you specify <code>io1</code>, you must also include a value for the <code>Iops</code> parameter.   Default: <code>io1</code> if the <code>Iops</code> parameter is specified, otherwise <code>gp2</code> <br/>DBClusterIdentifier:  The identifier of the DB cluster that the instance will belong to.<br/>Tags:  Tags to assign to the DB instance.<br/>PubliclyAccessible:  A value that indicates whether the DB instance is publicly accessible. When the DB instance is publicly accessible, it is an Internet-facing instance with a publicly resolvable DNS name, which resolves to a public IP address. When the DB instance is not publicly accessible, it is an internal instance with a DNS name that resolves to a private IP address. Default: The default behavior varies depending on whether <code>DBSubnetGroupName</code> is specified. If <code>DBSubnetGroupName</code> is not specified, and <code>PubliclyAccessible</code> is not specified, the following applies: <ul> <li> If the default VPC in the target region doesn’t have an Internet gateway attached to it, the DB instance is private. </li> <li> If the default VPC in the target region has an Internet gateway attached to it, the DB instance is public. </li> </ul> If <code>DBSubnetGroupName</code> is specified, and <code>PubliclyAccessible</code> is not specified, the following applies: <ul> <li> If the subnets are part of a VPC that doesn’t have an Internet gateway attached to it, the DB instance is private. </li> <li> If the subnets are part of a VPC that has an Internet gateway attached to it, the DB instance is public. </li> </ul><br/>CharacterSetName:  For supported engines, indicates that the DB instance should be associated with the specified CharacterSet.  <b>Amazon Aurora</b>  Not applicable. The character set is managed by the DB cluster. For more information, see <code>CreateDBCluster</code>.<br/>OptionGroupName:  Indicates that the DB instance should be associated with the specified option group. Permanent options, such as the TDE option for Oracle Advanced Security TDE, can't be removed from an option group, and that option group can't be removed from a DB instance once it is associated with a DB instance<br/>Iops:  The amount of Provisioned IOPS (input/output operations per second) to be initially allocated for the DB instance. For information about valid Iops values, see see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/CHAP_Storage.html#USER_PIOPS">Amazon RDS Provisioned IOPS Storage to Improve Performance</a> in the <i>Amazon RDS User Guide</i>.  Constraints: Must be a multiple between 1 and 50 of the storage amount for the DB instance. <br/>LicenseModel:  License model information for this DB instance.  Valid values: <code>license-included</code> | <code>bring-your-own-license</code> | <code>general-public-license</code> <br/>AutoMinorVersionUpgrade:  A value that indicates whether minor engine upgrades are applied automatically to the DB instance during the maintenance window. By default, minor engine upgrades are applied automatically.<br/>EngineVersion:  The version number of the database engine to use. For a list of valid engine versions, use the <code>DescribeDBEngineVersions</code> action. The following are the database engines and links to information about the major and minor versions that are available with Amazon RDS. Not every database engine is available for every AWS Region.  <b>Amazon Aurora</b>  Not applicable. The version number of the database engine to be used by the DB instance is managed by the DB cluster.  <b>MariaDB</b>  See <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/CHAP_MariaDB.html#MariaDB.Concepts.VersionMgmt">MariaDB on Amazon RDS Versions</a> in the <i>Amazon RDS User Guide.</i>   <b>Microsoft SQL Server</b>  See <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/CHAP_SQLServer.html#SQLServer.Concepts.General.FeatureSupport">Version and Feature Support on Amazon RDS</a> in the <i>Amazon RDS User Guide.</i>   <b>MySQL</b>  See <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/CHAP_MySQL.html#MySQL.Concepts.VersionMgmt">MySQL on Amazon RDS Versions</a> in the <i>Amazon RDS User Guide.</i>   <b>Oracle</b>  See <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/Appendix.Oracle.PatchComposition.html">Oracle Database Engine Release Notes</a> in the <i>Amazon RDS User Guide.</i>   <b>PostgreSQL</b>  See <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/CHAP_PostgreSQL.html#PostgreSQL.Concepts.General.DBVersions">Supported PostgreSQL Database Versions</a> in the <i>Amazon RDS User Guide.</i> <br/>MultiAZ:  A value that indicates whether the DB instance is a Multi-AZ deployment. You can't set the <code>AvailabilityZone</code> parameter if the DB instance is a Multi-AZ deployment.<br/>Port:  The port number on which the database accepts connections.  <b>MySQL</b>   Default: <code>3306</code>   Valid Values: <code>1150-65535</code>  Type: Integer  <b>MariaDB</b>   Default: <code>3306</code>   Valid Values: <code>1150-65535</code>  Type: Integer  <b>PostgreSQL</b>   Default: <code>5432</code>   Valid Values: <code>1150-65535</code>  Type: Integer  <b>Oracle</b>   Default: <code>1521</code>   Valid Values: <code>1150-65535</code>   <b>SQL Server</b>   Default: <code>1433</code>   Valid Values: <code>1150-65535</code> except for <code>1434</code>, <code>3389</code>, <code>47001</code>, <code>49152</code>, and <code>49152</code> through <code>49156</code>.   <b>Amazon Aurora</b>   Default: <code>3306</code>   Valid Values: <code>1150-65535</code>  Type: Integer<br/>PreferredBackupWindow:   The daily time range during which automated backups are created if automated backups are enabled, using the <code>BackupRetentionPeriod</code> parameter. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_WorkingWithAutomatedBackups.html#USER_WorkingWithAutomatedBackups.BackupWindow">The Backup Window</a> in the <i>Amazon RDS User Guide</i>.   <b>Amazon Aurora</b>  Not applicable. The daily time range for creating automated backups is managed by the DB cluster.  The default is a 30-minute window selected at random from an 8-hour block of time for each AWS Region. To see the time blocks available, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_UpgradeDBInstance.Maintenance.html#AdjustingTheMaintenanceWindow"> Adjusting the Preferred DB Instance Maintenance Window</a> in the <i>Amazon RDS User Guide</i>.  Constraints: <ul> <li> Must be in the format <code>hh24:mi-hh24:mi</code>. </li> <li> Must be in Universal Coordinated Time (UTC). </li> <li> Must not conflict with the preferred maintenance window. </li> <li> Must be at least 30 minutes. </li> </ul><br/>BackupRetentionPeriod:  The number of days for which automated backups are retained. Setting this parameter to a positive number enables backups. Setting this parameter to 0 disables automated backups.  <b>Amazon Aurora</b>  Not applicable. The retention period for automated backups is managed by the DB cluster. Default: 1 Constraints: <ul> <li> Must be a value from 0 to 35 </li> <li> Can't be set to 0 if the DB instance is a source to Read Replicas </li> </ul><br/>DBParameterGroupName:  The name of the DB parameter group to associate with this DB instance. If this argument is omitted, the default DBParameterGroup for the specified engine is used. Constraints: <ul> <li> Must be 1 to 255 letters, numbers, or hyphens. </li> <li> First character must be a letter </li> <li> Can't end with a hyphen or contain two consecutive hyphens </li> </ul><br/>PreferredMaintenanceWindow:  The time range each week during which system maintenance can occur, in Universal Coordinated Time (UTC). For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_UpgradeDBInstance.Maintenance.html#Concepts.DBMaintenance">Amazon RDS Maintenance Window</a>.   Format: <code>ddd:hh24:mi-ddd:hh24:mi</code>  The default is a 30-minute window selected at random from an 8-hour block of time for each AWS Region, occurring on a random day of the week.  Valid Days: Mon, Tue, Wed, Thu, Fri, Sat, Sun. Constraints: Minimum 30-minute window.<br/>DBSubnetGroupName:  A DB subnet group to associate with this DB instance. If there is no DB subnet group, then it is a non-VPC DB instance.<br/>AvailabilityZone:   The Availability Zone (AZ) where the database will be created. For information on AWS Regions and Availability Zones, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/Concepts.RegionsAndAvailabilityZones.html">Regions and Availability Zones</a>.  Default: A random, system-chosen Availability Zone in the endpoint's AWS Region.  Example: <code>us-east-1d</code>   Constraint: The <code>AvailabilityZone</code> parameter can't be specified if the DB instance is a Multi-AZ deployment. The specified Availability Zone must be in the same AWS Region as the current endpoint. <br/>VpcSecurityGroupIds:  A list of Amazon EC2 VPC security groups to associate with this DB instance.  <b>Amazon Aurora</b>  Not applicable. The associated list of EC2 VPC security groups is managed by the DB cluster. Default: The default EC2 VPC security group for the DB subnet group's VPC.<br/>DBSecurityGroups:  A list of DB security groups to associate with this DB instance. Default: The default DB security group for the database engine.<br/>MasterUserPassword:  The password for the master user. The password can include any printable ASCII character except "/", triple-quotes, or "@".  <b>Amazon Aurora</b>  Not applicable. The password for the master user is managed by the DB cluster.  <b>MariaDB</b>  Constraints: Must contain from 8 to 41 characters.  <b>Microsoft SQL Server</b>  Constraints: Must contain from 8 to 128 characters.  <b>MySQL</b>  Constraints: Must contain from 8 to 41 characters.  <b>Oracle</b>  Constraints: Must contain from 8 to 30 characters.  <b>PostgreSQL</b>  Constraints: Must contain from 8 to 128 characters.<br/>MasterUsername:  The name for the master user.  <b>Amazon Aurora</b>  Not applicable. The name for the master user is managed by the DB cluster.   <b>MariaDB</b>  Constraints: <ul> <li> Required for MariaDB. </li> <li> Must be 1 to 16 letters or numbers. </li> <li> Can't be a reserved word for the chosen database engine. </li> </ul>  <b>Microsoft SQL Server</b>  Constraints: <ul> <li> Required for SQL Server. </li> <li> Must be 1 to 128 letters or numbers. </li> <li> The first character must be a letter. </li> <li> Can't be a reserved word for the chosen database engine. </li> </ul>  <b>MySQL</b>  Constraints: <ul> <li> Required for MySQL. </li> <li> Must be 1 to 16 letters or numbers. </li> <li> First character must be a letter. </li> <li> Can't be a reserved word for the chosen database engine. </li> </ul>  <b>Oracle</b>  Constraints: <ul> <li> Required for Oracle. </li> <li> Must be 1 to 30 letters or numbers. </li> <li> First character must be a letter. </li> <li> Can't be a reserved word for the chosen database engine. </li> </ul>  <b>PostgreSQL</b>  Constraints: <ul> <li> Required for PostgreSQL. </li> <li> Must be 1 to 63 letters or numbers. </li> <li> First character must be a letter. </li> <li> Can't be a reserved word for the chosen database engine. </li> </ul><br/>Engine:  The name of the database engine to be used for this instance.  Not every database engine is available for every AWS Region.  Valid Values:  <ul> <li>  <code>aurora</code> (for MySQL 5.6-compatible Aurora) </li> <li>  <code>aurora-mysql</code> (for MySQL 5.7-compatible Aurora) </li> <li>  <code>aurora-postgresql</code>  </li> <li>  <code>mariadb</code>  </li> <li>  <code>mysql</code>  </li> <li>  <code>oracle-ee</code>  </li> <li>  <code>oracle-se2</code>  </li> <li>  <code>oracle-se1</code>  </li> <li>  <code>oracle-se</code>  </li> <li>  <code>postgres</code>  </li> <li>  <code>sqlserver-ee</code>  </li> <li>  <code>sqlserver-se</code>  </li> <li>  <code>sqlserver-ex</code>  </li> <li>  <code>sqlserver-web</code>  </li> </ul><br/>DBInstanceClass:  The compute and memory capacity of the DB instance, for example, <code>db.m4.large</code>. Not all DB instance classes are available in all AWS Regions, or for all database engines. For the full list of DB instance classes, and availability for your engine, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/Concepts.DBInstanceClass.html">DB Instance Class</a> in the <i>Amazon RDS User Guide.</i> <br/>AllocatedStorage:  The amount of storage (in gibibytes) to allocate for the DB instance. Type: Integer  <b>Amazon Aurora</b>  Not applicable. Aurora cluster volumes automatically grow as the amount of data in your database increases, though you are only charged for the space that you use in an Aurora cluster volume.  <b>MySQL</b>  Constraints to the amount of storage for each storage type are the following:  <ul> <li> General Purpose (SSD) storage (gp2): Must be an integer from 20 to 65536. </li> <li> Provisioned IOPS storage (io1): Must be an integer from 100 to 65536. </li> <li> Magnetic storage (standard): Must be an integer from 5 to 3072. </li> </ul>  <b>MariaDB</b>  Constraints to the amount of storage for each storage type are the following:  <ul> <li> General Purpose (SSD) storage (gp2): Must be an integer from 20 to 65536. </li> <li> Provisioned IOPS storage (io1): Must be an integer from 100 to 65536. </li> <li> Magnetic storage (standard): Must be an integer from 5 to 3072. </li> </ul>  <b>PostgreSQL</b>  Constraints to the amount of storage for each storage type are the following:  <ul> <li> General Purpose (SSD) storage (gp2): Must be an integer from 20 to 65536. </li> <li> Provisioned IOPS storage (io1): Must be an integer from 100 to 65536. </li> <li> Magnetic storage (standard): Must be an integer from 5 to 3072. </li> </ul>  <b>Oracle</b>  Constraints to the amount of storage for each storage type are the following:  <ul> <li> General Purpose (SSD) storage (gp2): Must be an integer from 20 to 65536. </li> <li> Provisioned IOPS storage (io1): Must be an integer from 100 to 65536. </li> <li> Magnetic storage (standard): Must be an integer from 10 to 3072. </li> </ul>  <b>SQL Server</b>  Constraints to the amount of storage for each storage type are the following:  <ul> <li> General Purpose (SSD) storage (gp2): <ul> <li> Enterprise and Standard editions: Must be an integer from 200 to 16384. </li> <li> Web and Express editions: Must be an integer from 20 to 16384. </li> </ul> </li> <li> Provisioned IOPS storage (io1): <ul> <li> Enterprise and Standard editions: Must be an integer from 200 to 16384. </li> <li> Web and Express editions: Must be an integer from 100 to 16384. </li> </ul> </li> <li> Magnetic storage (standard): <ul> <li> Enterprise and Standard editions: Must be an integer from 200 to 1024. </li> <li> Web and Express editions: Must be an integer from 20 to 1024. </li> </ul> </li> </ul><br/>DBInstanceIdentifier:  The DB instance identifier. This parameter is stored as a lowercase string. Constraints: <ul> <li> Must contain from 1 to 63 letters, numbers, or hyphens. </li> <li> First character must be a letter. </li> <li> Can't end with a hyphen or contain two consecutive hyphens. </li> </ul> Example: <code>mydbinstance</code> <br/>DBName:  The meaning of this parameter differs according to the database engine you use.  <b>MySQL</b>  The name of the database to create when the DB instance is created. If this parameter is not specified, no database is created in the DB instance. Constraints: <ul> <li> Must contain 1 to 64 letters or numbers. </li> <li> Can't be a word reserved by the specified database engine </li> </ul>  <b>MariaDB</b>  The name of the database to create when the DB instance is created. If this parameter is not specified, no database is created in the DB instance. Constraints: <ul> <li> Must contain 1 to 64 letters or numbers. </li> <li> Can't be a word reserved by the specified database engine </li> </ul>  <b>PostgreSQL</b>  The name of the database to create when the DB instance is created. If this parameter is not specified, the default "postgres" database is created in the DB instance. Constraints: <ul> <li> Must contain 1 to 63 letters, numbers, or underscores. </li> <li> Must begin with a letter or an underscore. Subsequent characters can be letters, underscores, or digits (0-9). </li> <li> Can't be a word reserved by the specified database engine </li> </ul>  <b>Oracle</b>  The Oracle System ID (SID) of the created DB instance. If you specify <code>null</code>, the default value <code>ORCL</code> is used. You can't specify the string NULL, or any other reserved word, for <code>DBName</code>.  Default: <code>ORCL</code>  Constraints: <ul> <li> Can't be longer than 8 characters </li> </ul>  <b>SQL Server</b>  Not applicable. Must be null.  <b>Amazon Aurora</b>  The name of the database to create when the primary instance of the DB cluster is created. If this parameter is not specified, no database is created in the DB instance. Constraints: <ul> <li> Must contain 1 to 64 letters or numbers. </li> <li> Can't be a word reserved by the specified database engine </li> </ul><br/>
        :return: DBInstance:  Contains the details of an Amazon RDS DB instance.  This data type is used as a response element in the <code>DescribeDBInstances</code> action. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxAllocatedStorage</i> The upper limit to which Amazon RDS can automatically scale the storage of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ListenerEndpoint</i> Specifies the listener connection endpoint for SQL Server Always On.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ListenerEndpoint</b>: Specifies the listener connection endpoint for SQL Server Always On.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i>  The AWS Identity and Access Management (IAM) roles associated with the DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i> Indicates if the DB instance has deletion protection enabled. The database can't be deleted when deletion protection is enabled. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_DeleteInstance.html"> Deleting a DB Instance</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessorFeatures</i> The number of CPU cores and the number of threads per core for the DB instance class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB instance is configured to export to CloudWatch Logs. Log types vary by DB engine. For information about the log types for each DB engine, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_LogAccess.html">Amazon RDS Database Log Files</a> in the <i>Amazon RDS User Guide.</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsRetentionPeriod</i> The amount of time, in days, to retain Performance Insights data. Valid values are 7 or 731 (2 years). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsKMSKeyId</i> The AWS KMS key identifier for encryption of Performance Insights data. The KMS key ID is the Amazon Resource Name (ARN), KMS key identifier, or the KMS key alias for the KMS encryption key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsEnabled</i> True if Performance Insights is enabled for the DB instance, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled, and otherwise false. IAM database authentication can be enabled for the following database engines <ul> <li> For MySQL 5.6, minor version 5.6.34 or higher </li> <li> For MySQL 5.7, minor version 5.7.16 or higher </li> <li> Aurora 5.6 or higher. To enable IAM database authentication for Aurora, see DBCluster Type. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timezone</i> The time zone of the DB instance. In most cases, the <code>Timezone</code> element is empty. <code>Timezone</code> content appears only for Microsoft SQL Server DB instances that were created with a time zone specified. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceArn</i> The Amazon Resource Name (ARN) for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PromotionTier</i> A value that specifies the order in which an Aurora Replica is promoted to the primary instance after a failure of the existing primary instance. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/Aurora.Managing.Backups.html#Aurora.Managing.FaultTolerance"> Fault Tolerance for an Aurora DB Cluster</a> in the <i>Amazon Aurora User Guide</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringRoleArn</i> The ARN for the IAM role that permits RDS to send Enhanced Monitoring metrics to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedMonitoringResourceArn</i> The Amazon Resource Name (ARN) of the Amazon CloudWatch Logs log stream that receives the Enhanced Monitoring metrics data for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringInterval</i> The interval, in seconds, between points when Enhanced Monitoring metrics are collected for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyTagsToSnapshot</i> Specifies whether tags are copied from the DB instance to snapshots of the DB instance.  <b>Amazon Aurora</b>  Not applicable. Copying tags to snapshots is managed by the DB cluster. Setting this value for an Aurora DB instance has no effect on the DB cluster setting. For more information, see <code>DBCluster</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainMemberships</i> The Active Directory Domain membership records associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> The identifier of the CA certificate for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbiResourceId</i> The AWS Region-unique, immutable identifier for the DB instance. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB instance is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i>  If <code>StorageEncrypted</code> is true, the AWS KMS key identifier for the encrypted DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB instance is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> If the DB instance is a member of a DB cluster, contains the name of the DB cluster that the DB instance is a member of.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbInstancePort</i> Specifies the port that the DB instance listens on. If the DB instance is part of a DB cluster, this can be a different port than the DB cluster port.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TdeCredentialArn</i> The ARN from the key store with which the instance is associated for TDE encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type associated with DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusInfos</i> The status of a Read Replica. If the instance is not a Read Replica, this is blank.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> Specifies the accessibility options for the DB instance. A value of true specifies an Internet-facing instance with a publicly resolvable DNS name, which resolves to a public IP address. A value of false specifies an internal instance with a DNS name that resolves to a private IP address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondaryAvailabilityZone</i> If present, specifies the name of the secondary Availability Zone for a DB instance with multi-AZ support.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this instance is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionGroupMemberships</i> Provides the list of option group memberships for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the Provisioned IOPS (I/O operations per second) value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> License model information for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBClusterIdentifiers</i> Contains one or more identifiers of Aurora DB clusters to which the RDS DB instance is replicated as a Read Replica. For example, when you create an Aurora Read Replica of an RDS MySQL DB instance, the Aurora MySQL DB cluster for the Aurora Read Replica is shown. This output does not contain information about cross region Aurora Read Replicas. <note> Currently, each RDS DB instance can have only one Aurora Read Replica. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBInstanceIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaSourceDBInstanceIdentifier</i> Contains the identifier of the source DB instance if this DB instance is a Read Replica.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> Indicates that minor version patches are applied automatically.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies if the DB instance is a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DBSubnetGroup</b>: Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> Specifies the name of the Availability Zone the DB instance is located in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroups</i> Provides the list of DB parameter groups applied to this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security group elements that the DB instance belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSecurityGroups</i>  A list of DB security group elements containing <code>DBSecurityGroup.Name</code> and <code>DBSecurityGroup.Status</code> subelements. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i>  Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCreateTime</i> Provides the date and time the DB instance was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> Specifies the allocated storage size specified in gibibytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: Specifies the connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The meaning of this parameter differs according to the database engine you use.  <b>MySQL, MariaDB, SQL Server, PostgreSQL</b>  Contains the name of the initial database of this instance that was provided at create time, if one was specified when the DB instance was created. This same name is returned for the life of the DB instance. Type: String  <b>Oracle</b>  Contains the Oracle System ID (SID) of the created DB instance. Not shown when the returned parameters do not apply to an Oracle DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceStatus</i> Specifies the current state of this database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i> Contains the name of the compute and memory capacity class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i> Contains a user-supplied database identifier. This identifier is the unique key that identifies a DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupArn</i> The Amazon Resource Name (ARN) for the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i>  Contains a list of <code>Subnet</code> elements. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> Provides the status of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VpcId of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupDescription</i> Provides the description of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The name of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The new DB subnet group for the DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> Specifies the identifier of the CA certificate for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type to be associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i>  Contains the new <code>DBInstanceIdentifier</code> for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the new Provisioned IOPS value for the DB instance that will be applied or is currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> The license model for the DB instance. Valid values: <code>license-included</code> | <code>bring-your-own-license</code> | <code>general-public-license</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Indicates that the Single-AZ DB instance is to change to a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the pending number of days for which automated backups are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the pending port for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> Contains the pending or currently-in-progress change of the master credentials for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i>  Contains the new <code>AllocatedStorage</code> size for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i>  Contains the new <code>DBInstanceClass</code> for the DB instance that will be applied or is currently being applied. <br/>A list of the log types whose configuration is still pending. In other words, these log types are in the process of being activated or deactivated.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingCloudwatchLogsExports</b>: A list of the log types whose configuration is still pending. In other words, these log types are in the process of being activated or deactivated.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>        """
        pass

    def create_db_instance_read_replica(self, SourceDBInstanceIdentifier: str, DBInstanceIdentifier: str, DBInstanceClass: str=None, AvailabilityZone: str=None, Port: int=None, MultiAZ: BooleanOptional=None, AutoMinorVersionUpgrade: BooleanOptional=None, Iops: int=None, OptionGroupName: str=None, PubliclyAccessible: BooleanOptional=None, Tags: List[Tag]=None, DBSubnetGroupName: str=None, VpcSecurityGroupIds: List[String]=None, StorageType: str=None, CopyTagsToSnapshot: BooleanOptional=None, MonitoringInterval: int=None, MonitoringRoleArn: str=None, KmsKeyId: str=None, PreSignedUrl: str=None, EnableIAMDatabaseAuthentication: BooleanOptional=None, EnablePerformanceInsights: BooleanOptional=None, PerformanceInsightsKMSKeyId: str=None, PerformanceInsightsRetentionPeriod: int=None, EnableCloudwatchLogsExports: List[String]=None, ProcessorFeatures: List[ProcessorFeature]=None, UseDefaultProcessorFeatures: BooleanOptional=None, DeletionProtection: BooleanOptional=None, SourceRegion: str=None) -> CreateDBInstanceReadReplicaResult:
        """Creates a new DB instance that acts as a Read Replica for an existing source DB instance. You can create a Read Replica for a DB instance running MySQL, MariaDB, Oracle, or PostgreSQL. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_ReadRepl.html">Working with Read Replicas</a> in the <i>Amazon RDS User Guide</i>.  Amazon Aurora doesn't support this action. You must call the <code>CreateDBInstance</code> action to create a DB instance for an Aurora DB cluster.  All Read Replica DB instances are created with backups disabled. All other DB instance attributes (including DB security groups and DB parameter groups) are inherited from the source DB instance, except as specified following.  <important> Your source DB instance must have backup retention enabled.  </important>

        :param self:
        :param SourceRegion:  The ID of the region that contains the source for the read replica.<br/>DeletionProtection:  A value that indicates whether the DB instance has deletion protection enabled. The database can't be deleted when deletion protection is enabled. By default, deletion protection is disabled. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_DeleteInstance.html"> Deleting a DB Instance</a>. <br/>UseDefaultProcessorFeatures:  A value that indicates whether the DB instance class of the DB instance uses its default processor features.<br/>ProcessorFeatures:  The number of CPU cores and the number of threads per core for the DB instance class of the DB instance.<br/>EnableCloudwatchLogsExports:  The list of logs that the new DB instance is to export to CloudWatch Logs. The values in the list depend on the DB engine being used. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_LogAccess.html#USER_LogAccess.Procedural.UploadtoCloudWatch">Publishing Database Logs to Amazon CloudWatch Logs </a> in the <i>Amazon RDS User Guide</i>.<br/>PerformanceInsightsRetentionPeriod:  The amount of time, in days, to retain Performance Insights data. Valid values are 7 or 731 (2 years). <br/>PerformanceInsightsKMSKeyId:  The AWS KMS key identifier for encryption of Performance Insights data. The KMS key ID is the Amazon Resource Name (ARN), KMS key identifier, or the KMS key alias for the KMS encryption key. If you do not specify a value for <code>PerformanceInsightsKMSKeyId</code>, then Amazon RDS uses your default encryption key. AWS KMS creates the default encryption key for your AWS account. Your AWS account has a different default encryption key for each AWS Region.<br/>EnablePerformanceInsights:  A value that indicates whether to enable Performance Insights for the Read Replica.  For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_PerfInsights.html">Using Amazon Performance Insights</a> in the <i>Amazon RDS User Guide</i>. <br/>EnableIAMDatabaseAuthentication:  A value that indicates whether to enable mapping of AWS Identity and Access Management (IAM) accounts to database accounts. By default, mapping is disabled. You can enable IAM database authentication for the following database engines <ul> <li> For MySQL 5.6, minor version 5.6.34 or higher </li> <li> For MySQL 5.7, minor version 5.7.16 or higher </li> <li> Aurora MySQL 5.6 or higher </li> </ul><br/>PreSignedUrl:  The URL that contains a Signature Version 4 signed request for the <code>CreateDBInstanceReadReplica</code> API action in the source AWS Region that contains the source DB instance.  You must specify this parameter when you create an encrypted Read Replica from another AWS Region by using the Amazon RDS API. You can specify the <code>--source-region</code> option instead of this parameter when you create an encrypted Read Replica from another AWS Region by using the AWS CLI.  The presigned URL must be a valid request for the <code>CreateDBInstanceReadReplica</code> API action that can be executed in the source AWS Region that contains the encrypted source DB instance. The presigned URL request must contain the following parameter values:  <ul> <li>  <code>DestinationRegion</code> - The AWS Region that the encrypted Read Replica is created in. This AWS Region is the same one where the <code>CreateDBInstanceReadReplica</code> action is called that contains this presigned URL.  For example, if you create an encrypted DB instance in the us-west-1 AWS Region, from a source DB instance in the us-east-2 AWS Region, then you call the <code>CreateDBInstanceReadReplica</code> action in the us-east-1 AWS Region and provide a presigned URL that contains a call to the <code>CreateDBInstanceReadReplica</code> action in the us-west-2 AWS Region. For this example, the <code>DestinationRegion</code> in the presigned URL must be set to the us-east-1 AWS Region.  </li> <li>  <code>KmsKeyId</code> - The AWS KMS key identifier for the key to use to encrypt the Read Replica in the destination AWS Region. This is the same identifier for both the <code>CreateDBInstanceReadReplica</code> action that is called in the destination AWS Region, and the action contained in the presigned URL.  </li> <li>  <code>SourceDBInstanceIdentifier</code> - The DB instance identifier for the encrypted DB instance to be replicated. This identifier must be in the Amazon Resource Name (ARN) format for the source AWS Region. For example, if you are creating an encrypted Read Replica from a DB instance in the us-west-2 AWS Region, then your <code>SourceDBInstanceIdentifier</code> looks like the following example: <code>arn:aws:rds:us-west-2:123456789012:instance:mysql-instance1-20161115</code>.  </li> </ul> To learn how to generate a Signature Version 4 signed request, see <a href="https://docs.aws.amazon.com/AmazonS3/latest/API/sigv4-query-string-auth.html">Authenticating Requests: Using Query Parameters (AWS Signature Version 4)</a> and <a href="https://docs.aws.amazon.com/general/latest/gr/signature-version-4.html">Signature Version 4 Signing Process</a>. <br/>KmsKeyId:  The AWS KMS key ID for an encrypted Read Replica. The KMS key ID is the Amazon Resource Name (ARN), KMS key identifier, or the KMS key alias for the KMS encryption key.  If you create an encrypted Read Replica in the same AWS Region as the source DB instance, then you do not have to specify a value for this parameter. The Read Replica is encrypted with the same KMS key as the source DB instance.  If you create an encrypted Read Replica in a different AWS Region, then you must specify a KMS key for the destination AWS Region. KMS encryption keys are specific to the AWS Region that they are created in, and you can't use encryption keys from one AWS Region in another AWS Region.  You can't create an encrypted Read Replica from an unencrypted DB instance. <br/>MonitoringRoleArn:  The ARN for the IAM role that permits RDS to send enhanced monitoring metrics to Amazon CloudWatch Logs. For example, <code>arn:aws:iam:123456789012:role/emaccess</code>. For information on creating a monitoring role, go to <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Monitoring.html#USER_Monitoring.OS.IAMRole">To create an IAM role for Amazon RDS Enhanced Monitoring</a> in the <i>Amazon RDS User Guide</i>. If <code>MonitoringInterval</code> is set to a value other than 0, then you must supply a <code>MonitoringRoleArn</code> value.<br/>MonitoringInterval:  The interval, in seconds, between points when Enhanced Monitoring metrics are collected for the Read Replica. To disable collecting Enhanced Monitoring metrics, specify 0. The default is 0. If <code>MonitoringRoleArn</code> is specified, then you must also set <code>MonitoringInterval</code> to a value other than 0. Valid Values: <code>0, 1, 5, 10, 15, 30, 60</code> <br/>CopyTagsToSnapshot:  A value that indicates whether to copy all tags from the Read Replica to snapshots of the Read Replica. By default, tags are not copied.<br/>StorageType:  Specifies the storage type to be associated with the Read Replica.  Valid values: <code>standard | gp2 | io1</code>   If you specify <code>io1</code>, you must also include a value for the <code>Iops</code> parameter.   Default: <code>io1</code> if the <code>Iops</code> parameter is specified, otherwise <code>gp2</code> <br/>VpcSecurityGroupIds:   A list of EC2 VPC security groups to associate with the Read Replica.   Default: The default EC2 VPC security group for the DB subnet group's VPC. <br/>DBSubnetGroupName:  Specifies a DB subnet group for the DB instance. The new DB instance is created in the VPC associated with the DB subnet group. If no DB subnet group is specified, then the new DB instance is not created in a VPC. Constraints: <ul> <li> Can only be specified if the source DB instance identifier specifies a DB instance in another AWS Region. </li> <li> If supplied, must match the name of an existing DBSubnetGroup. </li> <li> The specified DB subnet group must be in the same AWS Region in which the operation is running. </li> <li> All Read Replicas in one AWS Region that are created from the same source DB instance must either:&gt; <ul> <li> Specify DB subnet groups from the same VPC. All these Read Replicas are created in the same VPC. </li> <li> Not specify a DB subnet group. All these Read Replicas are created outside of any VPC. </li> </ul> </li> </ul> Example: <code>mySubnetgroup</code> <br/>Tags:  A list of tags. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Tagging.html">Tagging Amazon RDS Resources</a> in the <i>Amazon RDS User Guide.</i> <br/>PubliclyAccessible:  A value that indicates whether the DB instance is publicly accessible. When the DB instance is publicly accessible, it is an Internet-facing instance with a publicly resolvable DNS name, which resolves to a public IP address. When the DB instance is not publicly accessible, it is an internal instance with a DNS name that resolves to a private IP address. For more information, see <a>CreateDBInstance</a>.<br/>OptionGroupName:  The option group the DB instance is associated with. If omitted, the option group associated with the source instance is used.<br/>Iops:  The amount of Provisioned IOPS (input/output operations per second) to be initially allocated for the DB instance.<br/>AutoMinorVersionUpgrade:  A value that indicates whether minor engine upgrades are applied automatically to the Read Replica during the maintenance window. Default: Inherits from the source DB instance<br/>MultiAZ:  A value that indicates whether the Read Replica is in a Multi-AZ deployment.  You can create a Read Replica as a Multi-AZ DB instance. RDS creates a standby of your replica in another Availability Zone for failover support for the replica. Creating your Read Replica as a Multi-AZ DB instance is independent of whether the source database is a Multi-AZ DB instance. <br/>Port:  The port number that the DB instance uses for connections. Default: Inherits from the source DB instance Valid Values: <code>1150-65535</code> <br/>AvailabilityZone:  The Availability Zone (AZ) where the Read Replica will be created. Default: A random, system-chosen Availability Zone in the endpoint's AWS Region.  Example: <code>us-east-1d</code> <br/>DBInstanceClass:  The compute and memory capacity of the Read Replica, for example, <code>db.m4.large</code>. Not all DB instance classes are available in all AWS Regions, or for all database engines. For the full list of DB instance classes, and availability for your engine, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/Concepts.DBInstanceClass.html">DB Instance Class</a> in the <i>Amazon RDS User Guide.</i>  Default: Inherits from the source DB instance.<br/>SourceDBInstanceIdentifier:  The identifier of the DB instance that will act as the source for the Read Replica. Each DB instance can have up to five Read Replicas. Constraints: <ul> <li> Must be the identifier of an existing MySQL, MariaDB, Oracle, or PostgreSQL DB instance. </li> <li> Can specify a DB instance that is a MySQL Read Replica only if the source is running MySQL 5.6 or later. </li> <li> For the limitations of Oracle Read Replicas, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/oracle-read-replicas.html">Read Replica Limitations with Oracle</a> in the <i>Amazon RDS User Guide</i>. </li> <li> Can specify a DB instance that is a PostgreSQL DB instance only if the source is running PostgreSQL 9.3.5 or later (9.4.7 and higher for cross-region replication). </li> <li> The specified DB instance must have automatic backups enabled, its backup retention period must be greater than 0. </li> <li> If the source DB instance is in the same AWS Region as the Read Replica, specify a valid DB instance identifier. </li> <li> If the source DB instance is in a different AWS Region than the Read Replica, specify a valid DB instance ARN. For more information, go to <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Tagging.ARN.html#USER_Tagging.ARN.Constructing"> Constructing an ARN for Amazon RDS</a> in the <i>Amazon RDS User Guide</i>. </li> </ul><br/>DBInstanceIdentifier:  The DB instance identifier of the Read Replica. This identifier is the unique key that identifies a DB instance. This parameter is stored as a lowercase string.<br/>
        :return: DBInstance:  Contains the details of an Amazon RDS DB instance.  This data type is used as a response element in the <code>DescribeDBInstances</code> action. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxAllocatedStorage</i> The upper limit to which Amazon RDS can automatically scale the storage of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ListenerEndpoint</i> Specifies the listener connection endpoint for SQL Server Always On.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ListenerEndpoint</b>: Specifies the listener connection endpoint for SQL Server Always On.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i>  The AWS Identity and Access Management (IAM) roles associated with the DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i> Indicates if the DB instance has deletion protection enabled. The database can't be deleted when deletion protection is enabled. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_DeleteInstance.html"> Deleting a DB Instance</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessorFeatures</i> The number of CPU cores and the number of threads per core for the DB instance class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB instance is configured to export to CloudWatch Logs. Log types vary by DB engine. For information about the log types for each DB engine, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_LogAccess.html">Amazon RDS Database Log Files</a> in the <i>Amazon RDS User Guide.</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsRetentionPeriod</i> The amount of time, in days, to retain Performance Insights data. Valid values are 7 or 731 (2 years). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsKMSKeyId</i> The AWS KMS key identifier for encryption of Performance Insights data. The KMS key ID is the Amazon Resource Name (ARN), KMS key identifier, or the KMS key alias for the KMS encryption key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsEnabled</i> True if Performance Insights is enabled for the DB instance, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled, and otherwise false. IAM database authentication can be enabled for the following database engines <ul> <li> For MySQL 5.6, minor version 5.6.34 or higher </li> <li> For MySQL 5.7, minor version 5.7.16 or higher </li> <li> Aurora 5.6 or higher. To enable IAM database authentication for Aurora, see DBCluster Type. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timezone</i> The time zone of the DB instance. In most cases, the <code>Timezone</code> element is empty. <code>Timezone</code> content appears only for Microsoft SQL Server DB instances that were created with a time zone specified. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceArn</i> The Amazon Resource Name (ARN) for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PromotionTier</i> A value that specifies the order in which an Aurora Replica is promoted to the primary instance after a failure of the existing primary instance. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/Aurora.Managing.Backups.html#Aurora.Managing.FaultTolerance"> Fault Tolerance for an Aurora DB Cluster</a> in the <i>Amazon Aurora User Guide</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringRoleArn</i> The ARN for the IAM role that permits RDS to send Enhanced Monitoring metrics to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedMonitoringResourceArn</i> The Amazon Resource Name (ARN) of the Amazon CloudWatch Logs log stream that receives the Enhanced Monitoring metrics data for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringInterval</i> The interval, in seconds, between points when Enhanced Monitoring metrics are collected for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyTagsToSnapshot</i> Specifies whether tags are copied from the DB instance to snapshots of the DB instance.  <b>Amazon Aurora</b>  Not applicable. Copying tags to snapshots is managed by the DB cluster. Setting this value for an Aurora DB instance has no effect on the DB cluster setting. For more information, see <code>DBCluster</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainMemberships</i> The Active Directory Domain membership records associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> The identifier of the CA certificate for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbiResourceId</i> The AWS Region-unique, immutable identifier for the DB instance. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB instance is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i>  If <code>StorageEncrypted</code> is true, the AWS KMS key identifier for the encrypted DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB instance is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> If the DB instance is a member of a DB cluster, contains the name of the DB cluster that the DB instance is a member of.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbInstancePort</i> Specifies the port that the DB instance listens on. If the DB instance is part of a DB cluster, this can be a different port than the DB cluster port.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TdeCredentialArn</i> The ARN from the key store with which the instance is associated for TDE encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type associated with DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusInfos</i> The status of a Read Replica. If the instance is not a Read Replica, this is blank.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> Specifies the accessibility options for the DB instance. A value of true specifies an Internet-facing instance with a publicly resolvable DNS name, which resolves to a public IP address. A value of false specifies an internal instance with a DNS name that resolves to a private IP address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondaryAvailabilityZone</i> If present, specifies the name of the secondary Availability Zone for a DB instance with multi-AZ support.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this instance is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionGroupMemberships</i> Provides the list of option group memberships for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the Provisioned IOPS (I/O operations per second) value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> License model information for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBClusterIdentifiers</i> Contains one or more identifiers of Aurora DB clusters to which the RDS DB instance is replicated as a Read Replica. For example, when you create an Aurora Read Replica of an RDS MySQL DB instance, the Aurora MySQL DB cluster for the Aurora Read Replica is shown. This output does not contain information about cross region Aurora Read Replicas. <note> Currently, each RDS DB instance can have only one Aurora Read Replica. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBInstanceIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaSourceDBInstanceIdentifier</i> Contains the identifier of the source DB instance if this DB instance is a Read Replica.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> Indicates that minor version patches are applied automatically.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies if the DB instance is a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DBSubnetGroup</b>: Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> Specifies the name of the Availability Zone the DB instance is located in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroups</i> Provides the list of DB parameter groups applied to this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security group elements that the DB instance belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSecurityGroups</i>  A list of DB security group elements containing <code>DBSecurityGroup.Name</code> and <code>DBSecurityGroup.Status</code> subelements. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i>  Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCreateTime</i> Provides the date and time the DB instance was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> Specifies the allocated storage size specified in gibibytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: Specifies the connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The meaning of this parameter differs according to the database engine you use.  <b>MySQL, MariaDB, SQL Server, PostgreSQL</b>  Contains the name of the initial database of this instance that was provided at create time, if one was specified when the DB instance was created. This same name is returned for the life of the DB instance. Type: String  <b>Oracle</b>  Contains the Oracle System ID (SID) of the created DB instance. Not shown when the returned parameters do not apply to an Oracle DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceStatus</i> Specifies the current state of this database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i> Contains the name of the compute and memory capacity class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i> Contains a user-supplied database identifier. This identifier is the unique key that identifies a DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupArn</i> The Amazon Resource Name (ARN) for the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i>  Contains a list of <code>Subnet</code> elements. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> Provides the status of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VpcId of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupDescription</i> Provides the description of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The name of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The new DB subnet group for the DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> Specifies the identifier of the CA certificate for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type to be associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i>  Contains the new <code>DBInstanceIdentifier</code> for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the new Provisioned IOPS value for the DB instance that will be applied or is currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> The license model for the DB instance. Valid values: <code>license-included</code> | <code>bring-your-own-license</code> | <code>general-public-license</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Indicates that the Single-AZ DB instance is to change to a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the pending number of days for which automated backups are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the pending port for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> Contains the pending or currently-in-progress change of the master credentials for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i>  Contains the new <code>AllocatedStorage</code> size for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i>  Contains the new <code>DBInstanceClass</code> for the DB instance that will be applied or is currently being applied. <br/>A list of the log types whose configuration is still pending. In other words, these log types are in the process of being activated or deactivated.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingCloudwatchLogsExports</b>: A list of the log types whose configuration is still pending. In other words, these log types are in the process of being activated or deactivated.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>        """
        pass

    def create_db_parameter_group(self, Description: str, DBParameterGroupFamily: str, DBParameterGroupName: str, Tags: List[Tag]=None) -> CreateDBParameterGroupResult:
        """Creates a new DB parameter group.  A DB parameter group is initially created with the default parameters for the database engine used by the DB instance. To provide custom values for any of the parameters, you must modify the group after creating it using <i>ModifyDBParameterGroup</i>. Once you've created a DB parameter group, you need to associate it with your DB instance using <i>ModifyDBInstance</i>. When you associate a new DB parameter group with a running DB instance, you need to reboot the DB instance without failover for the new DB parameter group and associated settings to take effect.  <important> After you create a DB parameter group, you should wait at least 5 minutes before creating your first DB instance that uses that DB parameter group as the default parameter group. This allows Amazon RDS to fully complete the create action before the parameter group is used as the default for a new DB instance. This is especially important for parameters that are critical when creating the default database for a DB instance, such as the character set for the default database defined by the <code>character_set_database</code> parameter. You can use the <i>Parameter Groups</i> option of the <a href="https://console.aws.amazon.com/rds/">Amazon RDS console</a> or the <i>DescribeDBParameters</i> command to verify that your DB parameter group has been created or modified. </important>

        :param self:
        :param Tags:  Tags to assign to the DB parameter group.<br/>Description:  The description for the DB parameter group.<br/>DBParameterGroupFamily:  The DB parameter group family name. A DB parameter group can be associated with one and only one DB parameter group family, and can be applied only to a DB instance running a database engine and engine version compatible with that DB parameter group family. To list all of the available parameter group families, use the following command:  <code>aws rds describe-db-engine-versions --query "DBEngineVersions[].DBParameterGroupFamily"</code>  <note> The output contains duplicates. </note><br/>DBParameterGroupName:  The name of the DB parameter group. Constraints: <ul> <li> Must be 1 to 255 letters, numbers, or hyphens. </li> <li> First character must be a letter </li> <li> Can't end with a hyphen or contain two consecutive hyphens </li> </ul> <note> This value is stored as a lowercase string. </note><br/>
        :return: DBParameterGroup:  Contains the details of an Amazon RDS DB parameter group.  This data type is used as a response element in the <code>DescribeDBParameterGroups</code> action. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroupArn</i> The Amazon Resource Name (ARN) for the DB parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> Provides the customer-specified description for this DB parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroupFamily</i> Provides the name of the DB parameter group family that this DB parameter group is compatible with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroupName</i> Provides the name of the DB parameter group.<br/>        """
        pass

    def create_db_security_group(self, DBSecurityGroupDescription: str, DBSecurityGroupName: str, Tags: List[Tag]=None) -> CreateDBSecurityGroupResult:
        """Creates a new DB security group. DB security groups control access to a DB instance. <note> A DB security group controls access to EC2-Classic DB instances that are not in a VPC. </note>

        :param self:
        :param Tags:  Tags to assign to the DB security group.<br/>DBSecurityGroupDescription:  The description for the DB security group.<br/>DBSecurityGroupName:  The name for the DB security group. This value is stored as a lowercase string. Constraints: <ul> <li> Must be 1 to 255 letters, numbers, or hyphens. </li> <li> First character must be a letter </li> <li> Can't end with a hyphen or contain two consecutive hyphens </li> <li> Must not be "Default" </li> </ul> Example: <code>mysecuritygroup</code> <br/>
        :return: DBSecurityGroup:  Contains the details for an Amazon RDS DB security group.  This data type is used as a response element in the <code>DescribeDBSecurityGroups</code> action. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSecurityGroupArn</i> The Amazon Resource Name (ARN) for the DB security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IPRanges</i>  Contains a list of <code>IPRange</code> elements. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EC2SecurityGroups</i>  Contains a list of <code>EC2SecurityGroup</code> elements. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VpcId of the DB security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSecurityGroupDescription</i> Provides the description of the DB security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSecurityGroupName</i> Specifies the name of the DB security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> Provides the AWS ID of the owner of a specific DB security group.<br/>        """
        pass

    def create_db_snapshot(self, DBInstanceIdentifier: str, DBSnapshotIdentifier: str, Tags: List[Tag]=None) -> CreateDBSnapshotResult:
        """Creates a DBSnapshot. The source DBInstance must be in "available" state.

        :param self:
        :param Tags:  A list of tags. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Tagging.html">Tagging Amazon RDS Resources</a> in the <i>Amazon RDS User Guide.</i> <br/>DBInstanceIdentifier:  The identifier of the DB instance that you want to create the snapshot of. Constraints: <ul> <li> Must match the identifier of an existing DBInstance. </li> </ul><br/>DBSnapshotIdentifier:  The identifier for the DB snapshot. Constraints: <ul> <li> Can't be null, empty, or blank </li> <li> Must contain from 1 to 255 letters, numbers, or hyphens </li> <li> First character must be a letter </li> <li> Can't end with a hyphen or contain two consecutive hyphens </li> </ul> Example: <code>my-snapshot-id</code> <br/>
        :return: DBSnapshot:  Contains the details of an Amazon RDS DB snapshot.  This data type is used as a response element in the <code>DescribeDBSnapshots</code> action. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbiResourceId</i> The identifier for the source DB instance, which can't be changed and which is unique to an AWS Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessorFeatures</i> The number of CPU cores and the number of threads per core for the DB instance class of the DB instance when the DB snapshot was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timezone</i> The time zone of the DB snapshot. In most cases, the <code>Timezone</code> element is empty. <code>Timezone</code> content appears only for snapshots taken from Microsoft SQL Server DB instances that were created with a time zone specified. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSnapshotArn</i> The Amazon Resource Name (ARN) for the DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i>  If <code>Encrypted</code> is true, the AWS KMS key identifier for the encrypted DB snapshot. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encrypted</i> Specifies whether the DB snapshot is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TdeCredentialArn</i> The ARN from the key store with which to associate the instance for TDE encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type associated with DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceDBSnapshotIdentifier</i> The DB snapshot Amazon Resource Name (ARN) that the DB snapshot was copied from. It only has value in case of cross-customer or cross-region copy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceRegion</i> The AWS Region that the DB snapshot was created in or copied from.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> The percentage of the estimated data that has been transferred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionGroupName</i> Provides the option group name for the DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the Provisioned IOPS (I/O operations per second) value of the DB instance at the time of the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotType</i> Provides the type of the DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> License model information for the restored DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Specifies the version of the database engine.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Provides the master username for the DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCreateTime</i> Specifies the time when the snapshot was taken, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VPC ID associated with the DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> Specifies the name of the Availability Zone the DB instance was located in at the time of the DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine was listening on at the time of the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the status of this DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> Specifies the allocated storage size in gibibytes (GiB).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Specifies the name of the database engine.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCreateTime</i> Provides the time when the snapshot was taken, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i> Specifies the DB instance identifier of the DB instance this DB snapshot was created from.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSnapshotIdentifier</i> Specifies the identifier for the DB snapshot.<br/>        """
        pass

    def create_db_subnet_group(self, SubnetIds: List[String], DBSubnetGroupDescription: str, DBSubnetGroupName: str, Tags: List[Tag]=None) -> CreateDBSubnetGroupResult:
        """Creates a new DB subnet group. DB subnet groups must contain at least one subnet in at least two AZs in the AWS Region.

        :param self:
        :param Tags:  Tags to assign to the DB subnet group.<br/>SubnetIds:  The EC2 Subnet IDs for the DB subnet group.<br/>DBSubnetGroupDescription:  The description for the DB subnet group.<br/>DBSubnetGroupName:  The name for the DB subnet group. This value is stored as a lowercase string. Constraints: Must contain no more than 255 letters, numbers, periods, underscores, spaces, or hyphens. Must not be default. Example: <code>mySubnetgroup</code> <br/>
        :return: DBSubnetGroup:  Contains the details of an Amazon RDS DB subnet group.  This data type is used as a response element in the <code>DescribeDBSubnetGroups</code> action. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupArn</i> The Amazon Resource Name (ARN) for the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i>  Contains a list of <code>Subnet</code> elements. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> Provides the status of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VpcId of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupDescription</i> Provides the description of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The name of the DB subnet group.<br/>        """
        pass

    def create_event_subscription(self, SnsTopicArn: str, SubscriptionName: str, SourceType: str=None, EventCategories: List[String]=None, SourceIds: List[String]=None, Enabled: BooleanOptional=None, Tags: List[Tag]=None) -> CreateEventSubscriptionResult:
        """Creates an RDS event notification subscription. This action requires a topic ARN (Amazon Resource Name) created by either the RDS console, the SNS console, or the SNS API. To obtain an ARN with SNS, you must create a topic in Amazon SNS and subscribe to the topic. The ARN is displayed in the SNS console. You can specify the type of source (SourceType) you want to be notified of, provide a list of RDS sources (SourceIds) that triggers the events, and provide a list of event categories (EventCategories) for events you want to be notified of. For example, you can specify SourceType = db-instance, SourceIds = mydbinstance1, mydbinstance2 and EventCategories = Availability, Backup. If you specify both the SourceType and SourceIds, such as SourceType = db-instance and SourceIdentifier = myDBInstance1, you are notified of all the db-instance events for the specified source. If you specify a SourceType but do not specify a SourceIdentifier, you receive notice of the events for that source type for all your RDS sources. If you do not specify either the SourceType nor the SourceIdentifier, you are notified of events generated from all RDS sources belonging to your customer account.

        :param self:
        :param Tags:  A list of tags. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Tagging.html">Tagging Amazon RDS Resources</a> in the <i>Amazon RDS User Guide.</i> <br/>Enabled:   A value that indicates whether to activate the subscription. If the event notification subscription is not activated, the subscription is created but not active. <br/>SourceIds:  The list of identifiers of the event sources for which events are returned. If not specified, then all sources are included in the response. An identifier must begin with a letter and must contain only ASCII letters, digits, and hyphens; it can't end with a hyphen or contain two consecutive hyphens. Constraints: <ul> <li> If SourceIds are supplied, SourceType must also be provided. </li> <li> If the source type is a DB instance, then a <code>DBInstanceIdentifier</code> must be supplied. </li> <li> If the source type is a DB security group, a <code>DBSecurityGroupName</code> must be supplied. </li> <li> If the source type is a DB parameter group, a <code>DBParameterGroupName</code> must be supplied. </li> <li> If the source type is a DB snapshot, a <code>DBSnapshotIdentifier</code> must be supplied. </li> </ul><br/>EventCategories:   A list of event categories for a SourceType that you want to subscribe to. You can see a list of the categories for a given SourceType in the <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Events.html">Events</a> topic in the <i>Amazon RDS User Guide</i> or by using the <b>DescribeEventCategories</b> action. <br/>SourceType:  The type of source that is generating the events. For example, if you want to be notified of events generated by a DB instance, you would set this parameter to db-instance. if this value is not specified, all events are returned. Valid values: <code>db-instance</code> | <code>db-cluster</code> | <code>db-parameter-group</code> | <code>db-security-group</code> | <code>db-snapshot</code> | <code>db-cluster-snapshot</code> <br/>SnsTopicArn:  The Amazon Resource Name (ARN) of the SNS topic created for event notification. The ARN is created by Amazon SNS when you create a topic and subscribe to it.<br/>SubscriptionName:  The name of the subscription. Constraints: The name must be less than 255 characters.<br/>
        :return: EventSubscription:  Contains the results of a successful invocation of the <code>DescribeEventSubscriptions</code> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EventSubscriptionArn</i> The Amazon Resource Name (ARN) for the event subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> A Boolean value indicating if the subscription is enabled. True indicates the subscription is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EventCategoriesList</i> A list of event categories for the RDS event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceIdsList</i> A list of source IDs for the RDS event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceType</i> The source type for the RDS event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubscriptionCreationTime</i> The time the RDS event notification subscription was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the RDS event notification subscription. Constraints: Can be one of the following: creating | modifying | deleting | active | no-permission | topic-not-exist The status "no-permission" indicates that RDS no longer has permission to post to the SNS topic. The status "topic-not-exist" indicates that the topic was deleted after the subscription was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsTopicArn</i> The topic ARN of the RDS event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustSubscriptionId</i> The RDS event notification subscription Id.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomerAwsId</i> The AWS customer account associated with the RDS event notification subscription.<br/>        """
        pass

    def create_global_cluster(self, GlobalClusterIdentifier: str=None, SourceDBClusterIdentifier: str=None, Engine: str=None, EngineVersion: str=None, DeletionProtection: BooleanOptional=None, DatabaseName: str=None, StorageEncrypted: BooleanOptional=None) -> CreateGlobalClusterResult:
        """   Creates an Aurora global database spread across multiple regions. The global database contains a single primary cluster with read-write capability, and a read-only secondary cluster that receives data from the primary cluster through high-speed replication performed by the Aurora storage subsystem.   You can create a global database that is initially empty, and then add a primary cluster and a secondary cluster to it. Or you can specify an existing Aurora cluster during the create operation, and this cluster becomes the primary cluster of the global database.  <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param StorageEncrypted:   The storage encryption setting for the new global database cluster. <br/>DatabaseName:   The name for your database of up to 64 alpha-numeric characters. If you do not provide a name, Amazon Aurora will not create a database in the global database cluster you are creating. <br/>DeletionProtection:   The deletion protection setting for the new global database. The global database can't be deleted when deletion protection is enabled. <br/>EngineVersion:  The engine version of the Aurora global database.<br/>Engine:  Provides the name of the database engine to be used for this DB cluster.<br/>SourceDBClusterIdentifier:   The Amazon Resource Name (ARN) to use as the primary cluster of the global database. This parameter is optional. <br/>GlobalClusterIdentifier:  The cluster identifier of the new global database cluster.<br/>
        :return: GlobalCluster:  A data type representing an Aurora global database.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalClusterMembers</i>  The list of cluster IDs for secondary clusters within the global database cluster. Currently limited to 1 item. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i>  The deletion protection setting for the new global database cluster. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i>  The storage encryption setting for the global database cluster. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i>  The default database name within the new global database cluster. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i>  The Aurora database engine used by the global database cluster. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the current state of this global database cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalClusterArn</i> The Amazon Resource Name (ARN) for the global database cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalClusterResourceId</i>  The AWS Region-unique, immutable identifier for the global database cluster. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB cluster is accessed. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalClusterIdentifier</i>  Contains a user-supplied global database cluster identifier. This identifier is the unique key that identifies a global database cluster. <br/>        """
        pass

    def create_option_group(self, OptionGroupDescription: str, MajorEngineVersion: str, EngineName: str, OptionGroupName: str, Tags: List[Tag]=None) -> CreateOptionGroupResult:
        """Creates a new option group. You can create up to 20 option groups.

        :param self:
        :param Tags:  Tags to assign to the option group.<br/>OptionGroupDescription:  The description of the option group.<br/>MajorEngineVersion:  Specifies the major version of the engine that this option group should be associated with.<br/>EngineName:  Specifies the name of the engine that this option group should be associated with.<br/>OptionGroupName:  Specifies the name of the option group to be created. Constraints: <ul> <li> Must be 1 to 255 letters, numbers, or hyphens </li> <li> First character must be a letter </li> <li> Can't end with a hyphen or contain two consecutive hyphens </li> </ul> Example: <code>myoptiongroup</code> <br/>
        :return: OptionGroup:  <p/>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionGroupArn</i> The Amazon Resource Name (ARN) for the option group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> If <b>AllowsVpcAndNonVpcInstanceMemberships</b> is <code>false</code>, this field is blank. If <b>AllowsVpcAndNonVpcInstanceMemberships</b> is <code>true</code> and this field is blank, then this option group can be applied to both VPC and non-VPC instances. If this field contains a value, then this option group can only be applied to instances that are in the VPC indicated by this field. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowsVpcAndNonVpcInstanceMemberships</i> Indicates whether this option group can be applied to both VPC and non-VPC instances. The value <code>true</code> indicates the option group can be applied to both VPC and non-VPC instances. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> Indicates what options are available in the option group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MajorEngineVersion</i> Indicates the major engine version associated with this option group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineName</i> Indicates the name of the engine that this option group can be applied to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionGroupDescription</i> Provides a description of the option group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionGroupName</i> Specifies the name of the option group.<br/>        """
        pass

    def delete_db_cluster(self, DBClusterIdentifier: str, SkipFinalSnapshot: Boolean=None, FinalDBSnapshotIdentifier: str=None) -> DeleteDBClusterResult:
        """The DeleteDBCluster action deletes a previously provisioned DB cluster. When you delete a DB cluster, all automated backups for that DB cluster are deleted and can't be recovered. Manual DB cluster snapshots of the specified DB cluster are not deleted. <p/> For more information on Amazon Aurora, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/CHAP_AuroraOverview.html"> What Is Amazon Aurora?</a> in the <i>Amazon Aurora User Guide.</i>  <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param FinalDBSnapshotIdentifier:   The DB cluster snapshot identifier of the new DB cluster snapshot created when <code>SkipFinalSnapshot</code> is disabled.  <note>  Specifying this parameter and also skipping the creation of a final DB cluster snapshot with the <code>SkipFinalShapshot</code> parameter results in an error. </note> Constraints: <ul> <li> Must be 1 to 255 letters, numbers, or hyphens. </li> <li> First character must be a letter </li> <li> Can't end with a hyphen or contain two consecutive hyphens </li> </ul><br/>SkipFinalSnapshot:  A value that indicates whether to skip the creation of a final DB cluster snapshot before the DB cluster is deleted. If skip is specified, no DB cluster snapshot is created. If skip is not specified, a DB cluster snapshot is created before the DB cluster is deleted. By default, skip is not specified, and the DB cluster snapshot is created. By default, this parameter is disabled. <note> You must specify a <code>FinalDBSnapshotIdentifier</code> parameter if <code>SkipFinalSnapshot</code> is disabled. </note><br/>DBClusterIdentifier:  The DB cluster identifier for the DB cluster to be deleted. This parameter isn't case-sensitive. Constraints: <ul> <li> Must match an existing DBClusterIdentifier. </li> </ul><br/>
        :return: DBCluster:  Contains the details of an Amazon Aurora DB cluster.  This data type is used as a response element in the <code>DescribeDBClusters</code>, <code>StopDBCluster</code>, and <code>StartDBCluster</code> actions. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CrossAccountClone</i> Specifies whether the DB cluster is a clone of a DB cluster owned by a different AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyTagsToSnapshot</i> Specifies whether tags are copied from the DB cluster to snapshots of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamKinesisStreamName</i> The name of the Amazon Kinesis data stream used for the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamKmsKeyId</i> The AWS KMS key identifier used for encrypting messages in the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamStatus</i> The status of the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamMode</i> The mode of the database activity stream. Database events such as a change or access generate an activity stream event. The database session can handle these events either synchronously or asynchronously. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HttpEndpointEnabled</i> A value that indicates whether the HTTP endpoint for an Aurora Serverless DB cluster is enabled. When enabled, the HTTP endpoint provides a connectionless web service API for running SQL queries on the Aurora Serverless DB cluster. You can also query your database from inside the RDS console with the query editor. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/data-api.html">Using the Data API for Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i> Indicates if the DB cluster has deletion protection enabled. The database can't be deleted when deletion protection is enabled. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ScalingConfigurationInfo</i> Shows the scaling configuration for an Aurora DB cluster in <code>serverless</code> DB engine mode. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ScalingConfigurationInfo</b>: Shows the scaling configuration for an Aurora DB cluster in <code>serverless</code> DB engine mode. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineMode</i> The DB engine mode of the DB cluster, either <code>provisioned</code>, <code>serverless</code>, or <code>parallelquery</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Capacity</i> The current capacity of an Aurora Serverless DB cluster. The capacity is 0 (zero) when the cluster is paused. For more information about Aurora Serverless, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB cluster is configured to export to CloudWatch Logs. Log types vary by DB engine. For information about the log types for each DB engine, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/USER_LogAccess.html">Amazon RDS Database Log Files</a> in the <i>Amazon Aurora User Guide.</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BacktrackConsumedChangeRecords</i> The number of change records stored for Backtrack.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BacktrackWindow</i> The target backtrack window, in seconds. If this value is set to 0, backtracking is disabled for the DB cluster. Otherwise, backtracking is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestBacktrackTime</i> The earliest time to which a DB cluster can be backtracked.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloneGroupId</i> Identifies the clone group to which the DB cluster is associated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> A value that indicates whether the mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i> Provides a list of the AWS Identity and Access Management (IAM) roles that are associated with the DB cluster. IAM roles that are associated with a DB cluster grant permission for the DB cluster to access other AWS services on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterArn</i> The Amazon Resource Name (ARN) for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbClusterResourceId</i> The AWS Region-unique, immutable identifier for the DB cluster. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB cluster is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is enabled, the AWS KMS key identifier for the encrypted DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security groups that the DB cluster belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterMembers</i> Provides the list of instances that make up the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSourceIdentifier</i> Contains the identifier of the source DB cluster if this DB cluster is a Read Replica.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i> Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterOptionGroupMemberships</i> Provides the list of option group memberships for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies whether the DB cluster has instances in multiple Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomEndpoints</i> Identifies all custom endpoints associated with the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReaderEndpoint</i> The reader endpoint for the DB cluster. The reader endpoint for a DB cluster load-balances connections across the Aurora Replicas that are available in a DB cluster. As clients request new connections to the reader endpoint, Aurora distributes the connection requests among the Aurora Replicas in the DB cluster. This functionality can help balance your read workload across multiple Aurora Replicas in your DB cluster.  If a failover occurs, and the Aurora Replica that you are connected to is promoted to be the primary instance, your connection is dropped. To continue sending your read workload to other Aurora Replicas in the cluster, you can then reconnect to the reader endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint for the primary instance of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestRestorableTime</i> The earliest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the progress of the operation as a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the current state of this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB cluster, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroup</i> Specifies the name of the DB cluster parameter group for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Contains a user-supplied DB cluster identifier. This identifier is the unique key that identifies a DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> Contains the name of the initial database of this DB cluster that was provided at create time, if one was specified when the DB cluster was created. This same name is returned for the life of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this cluster is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of Availability Zones (AZs) where instances in the DB cluster can be created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> For all database engines except Amazon Aurora, <code>AllocatedStorage</code> specifies the allocated storage size in gibibytes (GiB). For Aurora, <code>AllocatedStorage</code> always returns 1, because Aurora DB cluster storage size is not fixed, but instead automatically adjusts as needed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeoutAction</i> The timeout action of a call to <code>ModifyCurrentDBClusterCapacity</code>, either <code>ForceApplyCapacityChange</code> or <code>RollbackCapacityChange</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondsUntilAutoPause</i> The remaining amount of time, in seconds, before the Aurora DB cluster in <code>serverless</code> mode is paused. A DB cluster can be paused only when it's idle (it has no connections).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoPause</i> A value that indicates whether automatic pause is allowed for the Aurora DB cluster in <code>serverless</code> DB engine mode. When the value is set to false for an Aurora Serverless DB cluster, the DB cluster automatically resumes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxCapacity</i> The maximum capacity for an Aurora DB cluster in <code>serverless</code> DB engine mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinCapacity</i> The maximum capacity for the Aurora DB cluster in <code>serverless</code> DB engine mode.<br/>        """
        pass

    def delete_db_cluster_endpoint(self, DBClusterEndpointIdentifier: str) -> DBClusterEndpoint:
        """Deletes a custom endpoint and removes it from an Amazon Aurora DB cluster. <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param DBClusterEndpointIdentifier:  The identifier associated with the custom endpoint. This parameter is stored as a lowercase string.<br/>
        :return: DBClusterEndpointArn:  The Amazon Resource Name (ARN) for the endpoint.<br/>ExcludedMembers:  List of DB instance identifiers that aren't part of the custom endpoint group. All other eligible instances are reachable through the custom endpoint. Only relevant if the list of static members is empty.<br/>StaticMembers:  List of DB instance identifiers that are part of the custom endpoint group.<br/>CustomEndpointType:  The type associated with a custom endpoint. One of: <code>READER</code>, <code>ANY</code>.<br/>EndpointType:  The type of the endpoint. One of: <code>READER</code>, <code>WRITER</code>, <code>CUSTOM</code>.<br/>Status:  The current status of the endpoint. One of: <code>creating</code>, <code>available</code>, <code>deleting</code>, <code>modifying</code>.<br/>Endpoint:  The DNS address of the endpoint.<br/>DBClusterEndpointResourceIdentifier:  A unique system-generated identifier for an endpoint. It remains the same for the whole life of the endpoint.<br/>DBClusterIdentifier:  The DB cluster identifier of the DB cluster associated with the endpoint. This parameter is stored as a lowercase string.<br/>DBClusterEndpointIdentifier:  The identifier associated with the endpoint. This parameter is stored as a lowercase string.<br/>        """
        pass

    def delete_db_cluster_parameter_group(self, DBClusterParameterGroupName: str):
        """Deletes a specified DB cluster parameter group. The DB cluster parameter group to be deleted can't be associated with any DB clusters. For more information on Amazon Aurora, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/CHAP_AuroraOverview.html"> What Is Amazon Aurora?</a> in the <i>Amazon Aurora User Guide.</i>  <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param DBClusterParameterGroupName:  The name of the DB cluster parameter group. Constraints: <ul> <li> Must be the name of an existing DB cluster parameter group. </li> <li> You can't delete a default DB cluster parameter group. </li> <li> Can't be associated with any DB clusters. </li> </ul><br/>
        :return:         """
        pass

    def delete_db_cluster_snapshot(self, DBClusterSnapshotIdentifier: str) -> DeleteDBClusterSnapshotResult:
        """Deletes a DB cluster snapshot. If the snapshot is being copied, the copy operation is terminated. <note> The DB cluster snapshot must be in the <code>available</code> state to be deleted. </note> For more information on Amazon Aurora, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/CHAP_AuroraOverview.html"> What Is Amazon Aurora?</a> in the <i>Amazon Aurora User Guide.</i>  <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param DBClusterSnapshotIdentifier:  The identifier of the DB cluster snapshot to delete. Constraints: Must be the name of an existing DB cluster snapshot in the <code>available</code> state.<br/>
        :return: DBClusterSnapshot:  Contains the details for an Amazon RDS DB cluster snapshot  This data type is used as a response element in the <code>DescribeDBClusterSnapshots</code> action. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceDBClusterSnapshotArn</i> If the DB cluster snapshot was copied from a source DB cluster snapshot, the Amazon Resource Name (ARN) for the source DB cluster snapshot, otherwise, a null value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotArn</i> The Amazon Resource Name (ARN) for the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is true, the AWS KMS key identifier for the encrypted DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster snapshot is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the percentage of the estimated data that has been transferred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotType</i> Provides the type of the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> Provides the license model information for this DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Provides the version of the database engine for this DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Provides the master username for the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VPC ID associated with the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the DB cluster was listening on at the time of the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the status of this DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> Specifies the allocated storage size in gibibytes (GiB).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Specifies the name of the database engine.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCreateTime</i> Provides the time when the snapshot was taken, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Specifies the DB cluster identifier of the DB cluster that this DB cluster snapshot was created from.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotIdentifier</i> Specifies the identifier for the DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of Availability Zones (AZs) where instances in the DB cluster snapshot can be restored.<br/>        """
        pass

    def delete_db_instance(self, DBInstanceIdentifier: str, SkipFinalSnapshot: Boolean=None, FinalDBSnapshotIdentifier: str=None, DeleteAutomatedBackups: BooleanOptional=None) -> DeleteDBInstanceResult:
        """The DeleteDBInstance action deletes a previously provisioned DB instance. When you delete a DB instance, all automated backups for that instance are deleted and can't be recovered. Manual DB snapshots of the DB instance to be deleted by <code>DeleteDBInstance</code> are not deleted.  If you request a final DB snapshot the status of the Amazon RDS DB instance is <code>deleting</code> until the DB snapshot is created. The API action <code>DescribeDBInstance</code> is used to monitor the status of this operation. The action can't be canceled or reverted once submitted.  Note that when a DB instance is in a failure state and has a status of <code>failed</code>, <code>incompatible-restore</code>, or <code>incompatible-network</code>, you can only delete it when you skip creation of the final snapshot with the <code>SkipFinalSnapshot</code> parameter. If the specified DB instance is part of an Amazon Aurora DB cluster, you can't delete the DB instance if both of the following conditions are true: <ul> <li> The DB cluster is a Read Replica of another Amazon Aurora DB cluster. </li> <li> The DB instance is the only instance in the DB cluster. </li> </ul> To delete a DB instance in this case, first call the <code>PromoteReadReplicaDBCluster</code> API action to promote the DB cluster so it's no longer a Read Replica. After the promotion completes, then call the <code>DeleteDBInstance</code> API action to delete the final instance in the DB cluster.

        :param self:
        :param DeleteAutomatedBackups:  A value that indicates whether to remove automated backups immediately after the DB instance is deleted. This parameter isn't case-sensitive. The default is to remove automated backups immediately after the DB instance is deleted.<br/>FinalDBSnapshotIdentifier:   The <code>DBSnapshotIdentifier</code> of the new <code>DBSnapshot</code> created when the <code>SkipFinalSnapshot</code> parameter is disabled.  <note> Specifying this parameter and also specifying to skip final DB snapshot creation in SkipFinalShapshot results in an error. </note> Constraints: <ul> <li> Must be 1 to 255 letters or numbers. </li> <li> First character must be a letter. </li> <li> Can't end with a hyphen or contain two consecutive hyphens. </li> <li> Can't be specified when deleting a Read Replica. </li> </ul><br/>SkipFinalSnapshot:  A value that indicates whether to skip the creation of a final DB snapshot before the DB instance is deleted. If skip is specified, no DB snapshot is created. If skip is not specified, a DB snapshot is created before the DB instance is deleted. By default, skip is not specified, and the DB snapshot is created. Note that when a DB instance is in a failure state and has a status of 'failed', 'incompatible-restore', or 'incompatible-network', it can only be deleted when skip is specified. Specify skip when deleting a Read Replica. <note> The FinalDBSnapshotIdentifier parameter must be specified if skip is not specified. </note><br/>DBInstanceIdentifier:  The DB instance identifier for the DB instance to be deleted. This parameter isn't case-sensitive. Constraints: <ul> <li> Must match the name of an existing DB instance. </li> </ul><br/>
        :return: DBInstance:  Contains the details of an Amazon RDS DB instance.  This data type is used as a response element in the <code>DescribeDBInstances</code> action. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxAllocatedStorage</i> The upper limit to which Amazon RDS can automatically scale the storage of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ListenerEndpoint</i> Specifies the listener connection endpoint for SQL Server Always On.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ListenerEndpoint</b>: Specifies the listener connection endpoint for SQL Server Always On.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i>  The AWS Identity and Access Management (IAM) roles associated with the DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i> Indicates if the DB instance has deletion protection enabled. The database can't be deleted when deletion protection is enabled. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_DeleteInstance.html"> Deleting a DB Instance</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessorFeatures</i> The number of CPU cores and the number of threads per core for the DB instance class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB instance is configured to export to CloudWatch Logs. Log types vary by DB engine. For information about the log types for each DB engine, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_LogAccess.html">Amazon RDS Database Log Files</a> in the <i>Amazon RDS User Guide.</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsRetentionPeriod</i> The amount of time, in days, to retain Performance Insights data. Valid values are 7 or 731 (2 years). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsKMSKeyId</i> The AWS KMS key identifier for encryption of Performance Insights data. The KMS key ID is the Amazon Resource Name (ARN), KMS key identifier, or the KMS key alias for the KMS encryption key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsEnabled</i> True if Performance Insights is enabled for the DB instance, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled, and otherwise false. IAM database authentication can be enabled for the following database engines <ul> <li> For MySQL 5.6, minor version 5.6.34 or higher </li> <li> For MySQL 5.7, minor version 5.7.16 or higher </li> <li> Aurora 5.6 or higher. To enable IAM database authentication for Aurora, see DBCluster Type. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timezone</i> The time zone of the DB instance. In most cases, the <code>Timezone</code> element is empty. <code>Timezone</code> content appears only for Microsoft SQL Server DB instances that were created with a time zone specified. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceArn</i> The Amazon Resource Name (ARN) for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PromotionTier</i> A value that specifies the order in which an Aurora Replica is promoted to the primary instance after a failure of the existing primary instance. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/Aurora.Managing.Backups.html#Aurora.Managing.FaultTolerance"> Fault Tolerance for an Aurora DB Cluster</a> in the <i>Amazon Aurora User Guide</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringRoleArn</i> The ARN for the IAM role that permits RDS to send Enhanced Monitoring metrics to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedMonitoringResourceArn</i> The Amazon Resource Name (ARN) of the Amazon CloudWatch Logs log stream that receives the Enhanced Monitoring metrics data for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringInterval</i> The interval, in seconds, between points when Enhanced Monitoring metrics are collected for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyTagsToSnapshot</i> Specifies whether tags are copied from the DB instance to snapshots of the DB instance.  <b>Amazon Aurora</b>  Not applicable. Copying tags to snapshots is managed by the DB cluster. Setting this value for an Aurora DB instance has no effect on the DB cluster setting. For more information, see <code>DBCluster</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainMemberships</i> The Active Directory Domain membership records associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> The identifier of the CA certificate for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbiResourceId</i> The AWS Region-unique, immutable identifier for the DB instance. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB instance is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i>  If <code>StorageEncrypted</code> is true, the AWS KMS key identifier for the encrypted DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB instance is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> If the DB instance is a member of a DB cluster, contains the name of the DB cluster that the DB instance is a member of.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbInstancePort</i> Specifies the port that the DB instance listens on. If the DB instance is part of a DB cluster, this can be a different port than the DB cluster port.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TdeCredentialArn</i> The ARN from the key store with which the instance is associated for TDE encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type associated with DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusInfos</i> The status of a Read Replica. If the instance is not a Read Replica, this is blank.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> Specifies the accessibility options for the DB instance. A value of true specifies an Internet-facing instance with a publicly resolvable DNS name, which resolves to a public IP address. A value of false specifies an internal instance with a DNS name that resolves to a private IP address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondaryAvailabilityZone</i> If present, specifies the name of the secondary Availability Zone for a DB instance with multi-AZ support.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this instance is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionGroupMemberships</i> Provides the list of option group memberships for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the Provisioned IOPS (I/O operations per second) value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> License model information for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBClusterIdentifiers</i> Contains one or more identifiers of Aurora DB clusters to which the RDS DB instance is replicated as a Read Replica. For example, when you create an Aurora Read Replica of an RDS MySQL DB instance, the Aurora MySQL DB cluster for the Aurora Read Replica is shown. This output does not contain information about cross region Aurora Read Replicas. <note> Currently, each RDS DB instance can have only one Aurora Read Replica. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBInstanceIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaSourceDBInstanceIdentifier</i> Contains the identifier of the source DB instance if this DB instance is a Read Replica.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> Indicates that minor version patches are applied automatically.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies if the DB instance is a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DBSubnetGroup</b>: Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> Specifies the name of the Availability Zone the DB instance is located in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroups</i> Provides the list of DB parameter groups applied to this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security group elements that the DB instance belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSecurityGroups</i>  A list of DB security group elements containing <code>DBSecurityGroup.Name</code> and <code>DBSecurityGroup.Status</code> subelements. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i>  Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCreateTime</i> Provides the date and time the DB instance was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> Specifies the allocated storage size specified in gibibytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: Specifies the connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The meaning of this parameter differs according to the database engine you use.  <b>MySQL, MariaDB, SQL Server, PostgreSQL</b>  Contains the name of the initial database of this instance that was provided at create time, if one was specified when the DB instance was created. This same name is returned for the life of the DB instance. Type: String  <b>Oracle</b>  Contains the Oracle System ID (SID) of the created DB instance. Not shown when the returned parameters do not apply to an Oracle DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceStatus</i> Specifies the current state of this database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i> Contains the name of the compute and memory capacity class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i> Contains a user-supplied database identifier. This identifier is the unique key that identifies a DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupArn</i> The Amazon Resource Name (ARN) for the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i>  Contains a list of <code>Subnet</code> elements. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> Provides the status of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VpcId of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupDescription</i> Provides the description of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The name of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The new DB subnet group for the DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> Specifies the identifier of the CA certificate for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type to be associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i>  Contains the new <code>DBInstanceIdentifier</code> for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the new Provisioned IOPS value for the DB instance that will be applied or is currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> The license model for the DB instance. Valid values: <code>license-included</code> | <code>bring-your-own-license</code> | <code>general-public-license</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Indicates that the Single-AZ DB instance is to change to a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the pending number of days for which automated backups are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the pending port for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> Contains the pending or currently-in-progress change of the master credentials for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i>  Contains the new <code>AllocatedStorage</code> size for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i>  Contains the new <code>DBInstanceClass</code> for the DB instance that will be applied or is currently being applied. <br/>A list of the log types whose configuration is still pending. In other words, these log types are in the process of being activated or deactivated.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingCloudwatchLogsExports</b>: A list of the log types whose configuration is still pending. In other words, these log types are in the process of being activated or deactivated.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>        """
        pass

    def delete_db_instance_automated_backup(self, DbiResourceId: str) -> DeleteDBInstanceAutomatedBackupResult:
        """Deletes automated backups based on the source instance's <code>DbiResourceId</code> value or the restorable instance's resource ID.

        :param self:
        :param DbiResourceId:  The identifier for the source DB instance, which can't be changed and which is unique to an AWS Region.<br/>
        :return: DBInstanceAutomatedBackup:  An automated backup of a DB instance. It it consists of system backups, transaction logs, and the database instance properties that existed at the time you deleted the source instance. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timezone</i> The time zone of the automated backup. In most cases, the <code>Timezone</code> element is empty. <code>Timezone</code> content appears only for Microsoft SQL Server DB instances that were created with a time zone specified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS KMS key ID for an automated backup. The KMS key ID is the Amazon Resource Name (ARN), KMS key identifier, or the KMS key alias for the KMS encryption key. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type associated with the automated backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encrypted</i> Specifies whether the automated backup is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TdeCredentialArn</i> The ARN from the key store with which the automated backup is associated for TDE encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionGroupName</i> The option group the automated backup is associated with. If omitted, the default option group for the engine specified is used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> The IOPS (I/O operations per second) value for the automated backup. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> License model information for the automated backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> The version of the database engine for the automated backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> The name of the database engine for this automated backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> The license model of an automated backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCreateTime</i> Provides the date and time that the DB instance was created. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VPC ID associated with the DB instance<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone that the automated backup was created in. For information on AWS Regions and Availability Zones, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/Concepts.RegionsAndAvailabilityZones.html">Regions and Availability Zones</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port number that the automated backup used for connections. Default: Inherits from the source DB instance Valid Values: <code>1150-65535</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Provides a list of status information for an automated backup: <ul> <li>  <code>active</code> - automated backups for current instances </li> <li>  <code>retained</code> - automated backups for deleted instances </li> <li>  <code>creating</code> - automated backups that are waiting for the first automated snapshot to be available. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> Specifies the allocated storage size in gibibytes (GiB).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreWindow</i> Earliest and latest time an instance can be restored to.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RestoreWindow</b>: Earliest and latest time an instance can be restored to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i> The customer id of the instance that is/was associated with the automated backup. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Region</i> The AWS Region associated with the automated backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbiResourceId</i> The identifier for the source DB instance, which can't be changed and which is unique to an AWS Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceArn</i> The Amazon Resource Name (ARN) for the automated backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestTime</i> The latest time you can restore an instance to.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestTime</i> The earliest time you can restore an instance to.<br/>        """
        pass

    def delete_db_parameter_group(self, DBParameterGroupName: str):
        """Deletes a specified DB parameter group. The DB parameter group to be deleted can't be associated with any DB instances.

        :param self:
        :param DBParameterGroupName:  The name of the DB parameter group. Constraints: <ul> <li> Must be the name of an existing DB parameter group </li> <li> You can't delete a default DB parameter group </li> <li> Can't be associated with any DB instances </li> </ul><br/>
        :return:         """
        pass

    def delete_db_security_group(self, DBSecurityGroupName: str):
        """Deletes a DB security group. <note> The specified DB security group must not be associated with any DB instances. </note>

        :param self:
        :param DBSecurityGroupName:  The name of the DB security group to delete. <note> You can't delete the default DB security group. </note> Constraints: <ul> <li> Must be 1 to 255 letters, numbers, or hyphens. </li> <li> First character must be a letter </li> <li> Can't end with a hyphen or contain two consecutive hyphens </li> <li> Must not be "Default" </li> </ul><br/>
        :return:         """
        pass

    def delete_db_snapshot(self, DBSnapshotIdentifier: str) -> DeleteDBSnapshotResult:
        """Deletes a DB snapshot. If the snapshot is being copied, the copy operation is terminated. <note> The DB snapshot must be in the <code>available</code> state to be deleted. </note>

        :param self:
        :param DBSnapshotIdentifier:  The DB snapshot identifier. Constraints: Must be the name of an existing DB snapshot in the <code>available</code> state.<br/>
        :return: DBSnapshot:  Contains the details of an Amazon RDS DB snapshot.  This data type is used as a response element in the <code>DescribeDBSnapshots</code> action. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbiResourceId</i> The identifier for the source DB instance, which can't be changed and which is unique to an AWS Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessorFeatures</i> The number of CPU cores and the number of threads per core for the DB instance class of the DB instance when the DB snapshot was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timezone</i> The time zone of the DB snapshot. In most cases, the <code>Timezone</code> element is empty. <code>Timezone</code> content appears only for snapshots taken from Microsoft SQL Server DB instances that were created with a time zone specified. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSnapshotArn</i> The Amazon Resource Name (ARN) for the DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i>  If <code>Encrypted</code> is true, the AWS KMS key identifier for the encrypted DB snapshot. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encrypted</i> Specifies whether the DB snapshot is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TdeCredentialArn</i> The ARN from the key store with which to associate the instance for TDE encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type associated with DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceDBSnapshotIdentifier</i> The DB snapshot Amazon Resource Name (ARN) that the DB snapshot was copied from. It only has value in case of cross-customer or cross-region copy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceRegion</i> The AWS Region that the DB snapshot was created in or copied from.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> The percentage of the estimated data that has been transferred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionGroupName</i> Provides the option group name for the DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the Provisioned IOPS (I/O operations per second) value of the DB instance at the time of the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotType</i> Provides the type of the DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> License model information for the restored DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Specifies the version of the database engine.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Provides the master username for the DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCreateTime</i> Specifies the time when the snapshot was taken, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VPC ID associated with the DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> Specifies the name of the Availability Zone the DB instance was located in at the time of the DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine was listening on at the time of the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the status of this DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> Specifies the allocated storage size in gibibytes (GiB).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Specifies the name of the database engine.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCreateTime</i> Provides the time when the snapshot was taken, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i> Specifies the DB instance identifier of the DB instance this DB snapshot was created from.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSnapshotIdentifier</i> Specifies the identifier for the DB snapshot.<br/>        """
        pass

    def delete_db_subnet_group(self, DBSubnetGroupName: str):
        """Deletes a DB subnet group. <note> The specified database subnet group must not be associated with any DB instances. </note>

        :param self:
        :param DBSubnetGroupName:  The name of the database subnet group to delete. <note> You can't delete the default subnet group. </note> Constraints: Constraints: Must match the name of an existing DBSubnetGroup. Must not be default. Example: <code>mySubnetgroup</code> <br/>
        :return:         """
        pass

    def delete_event_subscription(self, SubscriptionName: str) -> DeleteEventSubscriptionResult:
        """Deletes an RDS event notification subscription.

        :param self:
        :param SubscriptionName:  The name of the RDS event notification subscription you want to delete.<br/>
        :return: EventSubscription:  Contains the results of a successful invocation of the <code>DescribeEventSubscriptions</code> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EventSubscriptionArn</i> The Amazon Resource Name (ARN) for the event subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> A Boolean value indicating if the subscription is enabled. True indicates the subscription is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EventCategoriesList</i> A list of event categories for the RDS event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceIdsList</i> A list of source IDs for the RDS event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceType</i> The source type for the RDS event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubscriptionCreationTime</i> The time the RDS event notification subscription was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the RDS event notification subscription. Constraints: Can be one of the following: creating | modifying | deleting | active | no-permission | topic-not-exist The status "no-permission" indicates that RDS no longer has permission to post to the SNS topic. The status "topic-not-exist" indicates that the topic was deleted after the subscription was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsTopicArn</i> The topic ARN of the RDS event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustSubscriptionId</i> The RDS event notification subscription Id.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomerAwsId</i> The AWS customer account associated with the RDS event notification subscription.<br/>        """
        pass

    def delete_global_cluster(self, GlobalClusterIdentifier: str) -> DeleteGlobalClusterResult:
        """ Deletes a global database cluster. The primary and secondary clusters must already be detached or destroyed first.  <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param GlobalClusterIdentifier:   The cluster identifier of the global database cluster being deleted. <br/>
        :return: GlobalCluster:  A data type representing an Aurora global database.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalClusterMembers</i>  The list of cluster IDs for secondary clusters within the global database cluster. Currently limited to 1 item. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i>  The deletion protection setting for the new global database cluster. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i>  The storage encryption setting for the global database cluster. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i>  The default database name within the new global database cluster. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i>  The Aurora database engine used by the global database cluster. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the current state of this global database cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalClusterArn</i> The Amazon Resource Name (ARN) for the global database cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalClusterResourceId</i>  The AWS Region-unique, immutable identifier for the global database cluster. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB cluster is accessed. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalClusterIdentifier</i>  Contains a user-supplied global database cluster identifier. This identifier is the unique key that identifies a global database cluster. <br/>        """
        pass

    def delete_option_group(self, OptionGroupName: str):
        """Deletes an existing option group.

        :param self:
        :param OptionGroupName:  The name of the option group to be deleted. <note> You can't delete default option groups. </note><br/>
        :return:         """
        pass

    def describe_account_attributes(self) -> AccountAttributesMessage:
        """Lists all of the attributes for a customer account. The attributes include Amazon RDS quotas for the account, such as the number of DB instances allowed. The description for a quota includes the quota name, current usage toward that quota, and the quota's maximum value. This command doesn't take any parameters.

        :param self:
        :param 
        :return: AccountQuotas:  A list of <code>AccountQuota</code> objects. Within this list, each quota has a name, a count of usage toward the quota maximum, and a maximum value for the quota.<br/>        """
        pass

    def describe_certificates(self, CertificateIdentifier: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> CertificateMessage:
        """Lists the set of CA certificates provided by Amazon RDS for this AWS account.

        :param self:
        :param Marker:   An optional pagination token provided by a previous <code>DescribeCertificates</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>CertificateIdentifier:  The user-supplied certificate identifier. If this parameter is specified, information for only the identified certificate is returned. This parameter isn't case-sensitive. Constraints: <ul> <li> Must match an existing CertificateIdentifier. </li> </ul><br/>
        :return: Marker:   An optional pagination token provided by a previous <code>DescribeCertificates</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code> . <br/>Certificates:  The list of <code>Certificate</code> objects for the AWS account.<br/>        """
        pass

    def describe_db_cluster_backtracks(self, DBClusterIdentifier: str, BacktrackIdentifier: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DBClusterBacktrackMessage:
        """Returns information about backtracks for a DB cluster. For more information on Amazon Aurora, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/CHAP_AuroraOverview.html"> What Is Amazon Aurora?</a> in the <i>Amazon Aurora User Guide.</i>  <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param Marker:   An optional pagination token provided by a previous <code>DescribeDBClusterBacktracks</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:  The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  A filter that specifies one or more DB clusters to describe. Supported filters include the following: <ul> <li>  <code>db-cluster-backtrack-id</code> - Accepts backtrack identifiers. The results list includes information about only the backtracks identified by these identifiers. </li> <li>  <code>db-cluster-backtrack-status</code> - Accepts any of the following backtrack status values: <ul> <li>  <code>applying</code>  </li> <li>  <code>completed</code>  </li> <li>  <code>failed</code>  </li> <li>  <code>pending</code>  </li> </ul> The results list includes information about only the backtracks identified by these values. </li> </ul><br/>BacktrackIdentifier:  If specified, this value is the backtrack identifier of the backtrack to be described. Constraints: <ul> <li> Must contain a valid universally unique identifier (UUID). For more information about UUIDs, see <a href="http://www.ietf.org/rfc/rfc4122.txt">A Universally Unique Identifier (UUID) URN Namespace</a>. </li> </ul> Example: <code>123e4567-e89b-12d3-a456-426655440000</code> <br/>DBClusterIdentifier:  The DB cluster identifier of the DB cluster to be described. This parameter is stored as a lowercase string. Constraints: <ul> <li> Must contain from 1 to 63 alphanumeric characters or hyphens. </li> <li> First character must be a letter. </li> <li> Can't end with a hyphen or contain two consecutive hyphens. </li> </ul> Example: <code>my-cluster1</code> <br/>
        :return: DBClusterBacktracks:  Contains a list of backtracks for the user.<br/>Marker:  A pagination token that can be used in a subsequent <code>DescribeDBClusterBacktracks</code> request.<br/>        """
        pass

    def describe_db_cluster_endpoints(self, DBClusterIdentifier: str=None, DBClusterEndpointIdentifier: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DBClusterEndpointMessage:
        """Returns information about endpoints for an Amazon Aurora DB cluster. <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param Marker:   An optional pagination token provided by a previous <code>DescribeDBClusterEndpoints</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:  The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  A set of name-value pairs that define which endpoints to include in the output. The filters are specified as name-value pairs, in the format <code>Name=<i>endpoint_type</i>,Values=<i>endpoint_type1</i>,<i>endpoint_type2</i>,...</code>. <code>Name</code> can be one of: <code>db-cluster-endpoint-type</code>, <code>db-cluster-endpoint-custom-type</code>, <code>db-cluster-endpoint-id</code>, <code>db-cluster-endpoint-status</code>. <code>Values</code> for the <code> db-cluster-endpoint-type</code> filter can be one or more of: <code>reader</code>, <code>writer</code>, <code>custom</code>. <code>Values</code> for the <code>db-cluster-endpoint-custom-type</code> filter can be one or more of: <code>reader</code>, <code>any</code>. <code>Values</code> for the <code>db-cluster-endpoint-status</code> filter can be one or more of: <code>available</code>, <code>creating</code>, <code>deleting</code>, <code>modifying</code>. <br/>DBClusterEndpointIdentifier:  The identifier of the endpoint to describe. This parameter is stored as a lowercase string.<br/>DBClusterIdentifier:  The DB cluster identifier of the DB cluster associated with the endpoint. This parameter is stored as a lowercase string.<br/>
        :return: DBClusterEndpoints:  Contains the details of the endpoints associated with the cluster and matching any filter conditions.<br/>Marker:   An optional pagination token provided by a previous <code>DescribeDBClusterEndpoints</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>        """
        pass

    def describe_db_cluster_parameter_groups(self, DBClusterParameterGroupName: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DBClusterParameterGroupsMessage:
        """ Returns a list of <code>DBClusterParameterGroup</code> descriptions. If a <code>DBClusterParameterGroupName</code> parameter is specified, the list will contain only the description of the specified DB cluster parameter group.  For more information on Amazon Aurora, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/CHAP_AuroraOverview.html"> What Is Amazon Aurora?</a> in the <i>Amazon Aurora User Guide.</i>  <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param Marker:   An optional pagination token provided by a previous <code>DescribeDBClusterParameterGroups</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>DBClusterParameterGroupName:  The name of a specific DB cluster parameter group to return details for. Constraints: <ul> <li> If supplied, must match the name of an existing DBClusterParameterGroup. </li> </ul><br/>
        :return: DBClusterParameterGroups:  A list of DB cluster parameter groups.<br/>Marker:   An optional pagination token provided by a previous <code>DescribeDBClusterParameterGroups</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>        """
        pass

    def describe_db_cluster_parameters(self, DBClusterParameterGroupName: str, Source: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DBClusterParameterGroupDetails:
        """Returns the detailed parameter list for a particular DB cluster parameter group. For more information on Amazon Aurora, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/CHAP_AuroraOverview.html"> What Is Amazon Aurora?</a> in the <i>Amazon Aurora User Guide.</i>  <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param Marker:   An optional pagination token provided by a previous <code>DescribeDBClusterParameters</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>Source:   A value that indicates to return only parameters for a specific source. Parameter sources can be <code>engine</code>, <code>service</code>, or <code>customer</code>. <br/>DBClusterParameterGroupName:  The name of a specific DB cluster parameter group to return parameter details for. Constraints: <ul> <li> If supplied, must match the name of an existing DBClusterParameterGroup. </li> </ul><br/>
        :return: Marker:   An optional pagination token provided by a previous DescribeDBClusterParameters request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code> . <br/>Parameters:  Provides a list of parameters for the DB cluster parameter group.<br/>        """
        pass

    def describe_db_cluster_snapshot_attributes(self, DBClusterSnapshotIdentifier: str) -> DescribeDBClusterSnapshotAttributesResult:
        """Returns a list of DB cluster snapshot attribute names and values for a manual DB cluster snapshot. When sharing snapshots with other AWS accounts, <code>DescribeDBClusterSnapshotAttributes</code> returns the <code>restore</code> attribute and a list of IDs for the AWS accounts that are authorized to copy or restore the manual DB cluster snapshot. If <code>all</code> is included in the list of values for the <code>restore</code> attribute, then the manual DB cluster snapshot is public and can be copied or restored by all AWS accounts. To add or remove access for an AWS account to copy or restore a manual DB cluster snapshot, or to make the manual DB cluster snapshot public or private, use the <code>ModifyDBClusterSnapshotAttribute</code> API action. <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param DBClusterSnapshotIdentifier:  The identifier for the DB cluster snapshot to describe the attributes for.<br/>
        :return: DBClusterSnapshotAttributesResult:  Contains the results of a successful call to the <code>DescribeDBClusterSnapshotAttributes</code> API action. Manual DB cluster snapshot attributes are used to authorize other AWS accounts to copy or restore a manual DB cluster snapshot. For more information, see the <code>ModifyDBClusterSnapshotAttribute</code> API action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotAttributes</i> The list of attributes and values for the manual DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotIdentifier</i> The identifier of the manual DB cluster snapshot that the attributes apply to.<br/>        """
        pass

    def describe_db_cluster_snapshots(self, DBClusterIdentifier: str=None, DBClusterSnapshotIdentifier: str=None, SnapshotType: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None, IncludeShared: Boolean=None, IncludePublic: Boolean=None) -> DBClusterSnapshotMessage:
        """Returns information about DB cluster snapshots. This API action supports pagination. For more information on Amazon Aurora, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/CHAP_AuroraOverview.html"> What Is Amazon Aurora?</a> in the <i>Amazon Aurora User Guide.</i>  <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param IncludePublic:  A value that indicates whether to include manual DB cluster snapshots that are public and can be copied or restored by any AWS account. By default, the public snapshots are not included. You can share a manual DB cluster snapshot as public by using the <a>ModifyDBClusterSnapshotAttribute</a> API action.<br/>IncludeShared:  A value that indicates whether to include shared manual DB cluster snapshots from other AWS accounts that this AWS account has been given permission to copy or restore. By default, these snapshots are not included. You can give an AWS account permission to restore a manual DB cluster snapshot from another AWS account by the <code>ModifyDBClusterSnapshotAttribute</code> API action.<br/>Marker:  An optional pagination token provided by a previous <code>DescribeDBClusterSnapshots</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:  The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>SnapshotType:  The type of DB cluster snapshots to be returned. You can specify one of the following values: <ul> <li>  <code>automated</code> - Return all DB cluster snapshots that have been automatically taken by Amazon RDS for my AWS account. </li> <li>  <code>manual</code> - Return all DB cluster snapshots that have been taken by my AWS account. </li> <li>  <code>shared</code> - Return all manual DB cluster snapshots that have been shared to my AWS account. </li> <li>  <code>public</code> - Return all DB cluster snapshots that have been marked as public. </li> </ul> If you don't specify a <code>SnapshotType</code> value, then both automated and manual DB cluster snapshots are returned. You can include shared DB cluster snapshots with these results by enabling the <code>IncludeShared</code> parameter. You can include public DB cluster snapshots with these results by enabling the <code>IncludePublic</code> parameter. The <code>IncludeShared</code> and <code>IncludePublic</code> parameters don't apply for <code>SnapshotType</code> values of <code>manual</code> or <code>automated</code>. The <code>IncludePublic</code> parameter doesn't apply when <code>SnapshotType</code> is set to <code>shared</code>. The <code>IncludeShared</code> parameter doesn't apply when <code>SnapshotType</code> is set to <code>public</code>.<br/>DBClusterSnapshotIdentifier:  A specific DB cluster snapshot identifier to describe. This parameter can't be used in conjunction with the <code>DBClusterIdentifier</code> parameter. This value is stored as a lowercase string.  Constraints: <ul> <li> If supplied, must match the identifier of an existing DBClusterSnapshot. </li> <li> If this identifier is for an automated snapshot, the <code>SnapshotType</code> parameter must also be specified. </li> </ul><br/>DBClusterIdentifier:  The ID of the DB cluster to retrieve the list of DB cluster snapshots for. This parameter can't be used in conjunction with the <code>DBClusterSnapshotIdentifier</code> parameter. This parameter is not case-sensitive.  Constraints: <ul> <li> If supplied, must match the identifier of an existing DBCluster. </li> </ul><br/>
        :return: DBClusterSnapshots:  Provides a list of DB cluster snapshots for the user.<br/>Marker:   An optional pagination token provided by a previous <code>DescribeDBClusterSnapshots</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>        """
        pass

    def describe_db_clusters(self, DBClusterIdentifier: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None, IncludeShared: Boolean=None) -> DBClusterMessage:
        """Returns information about provisioned Aurora DB clusters. This API supports pagination. For more information on Amazon Aurora, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/CHAP_AuroraOverview.html"> What Is Amazon Aurora?</a> in the <i>Amazon Aurora User Guide.</i>  <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param IncludeShared:  Optional Boolean parameter that specifies whether the output includes information about clusters shared from other AWS accounts.<br/>Marker:  An optional pagination token provided by a previous <code>DescribeDBClusters</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:  The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  A filter that specifies one or more DB clusters to describe. Supported filters: <ul> <li>  <code>db-cluster-id</code> - Accepts DB cluster identifiers and DB cluster Amazon Resource Names (ARNs). The results list will only include information about the DB clusters identified by these ARNs. </li> </ul><br/>DBClusterIdentifier:  The user-supplied DB cluster identifier. If this parameter is specified, information from only the specific DB cluster is returned. This parameter isn't case-sensitive. Constraints: <ul> <li> If supplied, must match an existing DBClusterIdentifier. </li> </ul><br/>
        :return: DBClusters:  Contains a list of DB clusters for the user.<br/>Marker:  A pagination token that can be used in a subsequent DescribeDBClusters request.<br/>        """
        pass

    def describe_db_engine_versions(self, Engine: str=None, EngineVersion: str=None, DBParameterGroupFamily: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None, DefaultOnly: Boolean=None, ListSupportedCharacterSets: BooleanOptional=None, ListSupportedTimezones: BooleanOptional=None, IncludeAll: BooleanOptional=None) -> DBEngineVersionMessage:
        """Returns a list of the available DB engines.

        :param self:
        :param IncludeAll:  A value that indicates whether to include engine versions that aren't available in the list. The default is to list only available engine versions.<br/>ListSupportedTimezones:  A value that indicates whether to list the supported time zones for each engine version. If this parameter is enabled and the requested engine supports the <code>TimeZone</code> parameter for <code>CreateDBInstance</code>, the response includes a list of supported time zones for each engine version. <br/>ListSupportedCharacterSets:  A value that indicates whether to list the supported character sets for each engine version. If this parameter is enabled and the requested engine supports the <code>CharacterSetName</code> parameter for <code>CreateDBInstance</code>, the response includes a list of supported character sets for each engine version. <br/>DefaultOnly:  A value that indicates whether only the default version of the specified engine or engine and major version combination is returned.<br/>Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more than the <code>MaxRecords</code> value is available, a pagination token called a marker is included in the response so that the following results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>DBParameterGroupFamily:  The name of a specific DB parameter group family to return details for. Constraints: <ul> <li> If supplied, must match an existing DBParameterGroupFamily. </li> </ul><br/>EngineVersion:  The database engine version to return. Example: <code>5.1.49</code> <br/>Engine:  The database engine to return.<br/>
        :return: DBEngineVersions:   A list of <code>DBEngineVersion</code> elements. <br/>Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>        """
        pass

    def describe_db_instance_automated_backups(self, DbiResourceId: str=None, DBInstanceIdentifier: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DBInstanceAutomatedBackupMessage:
        """Displays backups for both current and deleted instances. For example, use this operation to find details about automated backups for previously deleted instances. Current instances with retention periods greater than zero (0) are returned for both the <code>DescribeDBInstanceAutomatedBackups</code> and <code>DescribeDBInstances</code> operations. All parameters are optional.

        :param self:
        :param Marker:  The pagination token provided in the previous request. If this parameter is specified the response includes only records beyond the marker, up to <code>MaxRecords</code>.<br/>MaxRecords:  The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.<br/>Filters:  A filter that specifies which resources to return based on status. Supported filters are the following: <ul> <li>  <code>status</code>  <ul> <li>  <code>active</code> - automated backups for current instances </li> <li>  <code>retained</code> - automated backups for deleted instances </li> <li>  <code>creating</code> - automated backups that are waiting for the first automated snapshot to be available </li> </ul> </li> <li>  <code>db-instance-id</code> - Accepts DB instance identifiers and Amazon Resource Names (ARNs) for DB instances. The results list includes only information about the DB instance automated backupss identified by these ARNs. </li> <li>  <code>dbi-resource-id</code> - Accepts DB instance resource identifiers and DB Amazon Resource Names (ARNs) for DB instances. The results list includes only information about the DB instance resources identified by these ARNs. </li> </ul> Returns all resources by default. The status for each resource is specified in the response.<br/>DBInstanceIdentifier:  (Optional) The user-supplied instance identifier. If this parameter is specified, it must match the identifier of an existing DB instance. It returns information from the specific DB instance' automated backup. This parameter isn't case-sensitive. <br/>DbiResourceId:  The resource ID of the DB instance that is the source of the automated backup. This parameter isn't case-sensitive. <br/>
        :return: DBInstanceAutomatedBackups:   A list of <code>DBInstanceAutomatedBackup</code> instances. <br/>Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code> . <br/>        """
        pass

    def describe_db_instances(self, DBInstanceIdentifier: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DBInstanceMessage:
        """Returns information about provisioned RDS instances. This API supports pagination.

        :param self:
        :param Marker:   An optional pagination token provided by a previous <code>DescribeDBInstances</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  A filter that specifies one or more DB instances to describe. Supported filters: <ul> <li>  <code>db-cluster-id</code> - Accepts DB cluster identifiers and DB cluster Amazon Resource Names (ARNs). The results list will only include information about the DB instances associated with the DB clusters identified by these ARNs. </li> <li>  <code>db-instance-id</code> - Accepts DB instance identifiers and DB instance Amazon Resource Names (ARNs). The results list will only include information about the DB instances identified by these ARNs. </li> </ul><br/>DBInstanceIdentifier:  The user-supplied instance identifier. If this parameter is specified, information from only the specific DB instance is returned. This parameter isn't case-sensitive. Constraints: <ul> <li> If supplied, must match the identifier of an existing DBInstance. </li> </ul><br/>
        :return: DBInstances:   A list of <code>DBInstance</code> instances. <br/>Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code> . <br/>        """
        pass

    def describe_db_log_files(self, DBInstanceIdentifier: str, FilenameContains: str=None, FileLastWritten: Long=None, FileSize: Long=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DescribeDBLogFilesResponse:
        """Returns a list of DB log files for the DB instance.

        :param self:
        :param Marker:  The pagination token provided in the previous request. If this parameter is specified the response includes only records beyond the marker, up to MaxRecords.<br/>MaxRecords:  The maximum number of records to include in the response. If more records exist than the specified MaxRecords value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.<br/>Filters:  This parameter is not currently supported.<br/>FileSize:  Filters the available log files for files larger than the specified size.<br/>FileLastWritten:  Filters the available log files for files written since the specified date, in POSIX timestamp format with milliseconds.<br/>FilenameContains:  Filters the available log files for log file names that contain the specified string.<br/>DBInstanceIdentifier:  The customer-assigned name of the DB instance that contains the log files you want to list. Constraints: <ul> <li> Must match the identifier of an existing DBInstance. </li> </ul><br/>
        :return: Marker:  A pagination token that can be used in a subsequent DescribeDBLogFiles request.<br/>DescribeDBLogFiles:  The DB log files returned.<br/>        """
        pass

    def describe_db_parameter_groups(self, DBParameterGroupName: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DBParameterGroupsMessage:
        """ Returns a list of <code>DBParameterGroup</code> descriptions. If a <code>DBParameterGroupName</code> is specified, the list will contain only the description of the specified DB parameter group. 

        :param self:
        :param Marker:   An optional pagination token provided by a previous <code>DescribeDBParameterGroups</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>DBParameterGroupName:  The name of a specific DB parameter group to return details for. Constraints: <ul> <li> If supplied, must match the name of an existing DBClusterParameterGroup. </li> </ul><br/>
        :return: DBParameterGroups:   A list of <code>DBParameterGroup</code> instances. <br/>Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>        """
        pass

    def describe_db_parameters(self, DBParameterGroupName: str, Source: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DBParameterGroupDetails:
        """Returns the detailed parameter list for a particular DB parameter group.

        :param self:
        :param Marker:   An optional pagination token provided by a previous <code>DescribeDBParameters</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>Source:  The parameter types to return. Default: All parameter types returned Valid Values: <code>user | system | engine-default</code> <br/>DBParameterGroupName:  The name of a specific DB parameter group to return details for. Constraints: <ul> <li> If supplied, must match the name of an existing DBParameterGroup. </li> </ul><br/>
        :return: Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>Parameters:   A list of <code>Parameter</code> values. <br/>        """
        pass

    def describe_db_security_groups(self, DBSecurityGroupName: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DBSecurityGroupMessage:
        """ Returns a list of <code>DBSecurityGroup</code> descriptions. If a <code>DBSecurityGroupName</code> is specified, the list will contain only the descriptions of the specified DB security group. 

        :param self:
        :param Marker:   An optional pagination token provided by a previous <code>DescribeDBSecurityGroups</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>DBSecurityGroupName:  The name of the DB security group to return details for.<br/>
        :return: DBSecurityGroups:   A list of <code>DBSecurityGroup</code> instances. <br/>Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>        """
        pass

    def describe_db_snapshot_attributes(self, DBSnapshotIdentifier: str) -> DescribeDBSnapshotAttributesResult:
        """Returns a list of DB snapshot attribute names and values for a manual DB snapshot. When sharing snapshots with other AWS accounts, <code>DescribeDBSnapshotAttributes</code> returns the <code>restore</code> attribute and a list of IDs for the AWS accounts that are authorized to copy or restore the manual DB snapshot. If <code>all</code> is included in the list of values for the <code>restore</code> attribute, then the manual DB snapshot is public and can be copied or restored by all AWS accounts. To add or remove access for an AWS account to copy or restore a manual DB snapshot, or to make the manual DB snapshot public or private, use the <code>ModifyDBSnapshotAttribute</code> API action.

        :param self:
        :param DBSnapshotIdentifier:  The identifier for the DB snapshot to describe the attributes for.<br/>
        :return: DBSnapshotAttributesResult:  Contains the results of a successful call to the <code>DescribeDBSnapshotAttributes</code> API action. Manual DB snapshot attributes are used to authorize other AWS accounts to copy or restore a manual DB snapshot. For more information, see the <code>ModifyDBSnapshotAttribute</code> API action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSnapshotAttributes</i> The list of attributes and values for the manual DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSnapshotIdentifier</i> The identifier of the manual DB snapshot that the attributes apply to.<br/>        """
        pass

    def describe_db_snapshots(self, DBInstanceIdentifier: str=None, DBSnapshotIdentifier: str=None, SnapshotType: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None, IncludeShared: Boolean=None, IncludePublic: Boolean=None, DbiResourceId: str=None) -> DBSnapshotMessage:
        """Returns information about DB snapshots. This API action supports pagination.

        :param self:
        :param DbiResourceId:  A specific DB resource ID to describe.<br/>IncludePublic:  A value that indicates whether to include manual DB cluster snapshots that are public and can be copied or restored by any AWS account. By default, the public snapshots are not included. You can share a manual DB snapshot as public by using the <a>ModifyDBSnapshotAttribute</a> API.<br/>IncludeShared:  A value that indicates whether to include shared manual DB cluster snapshots from other AWS accounts that this AWS account has been given permission to copy or restore. By default, these snapshots are not included. You can give an AWS account permission to restore a manual DB snapshot from another AWS account by using the <code>ModifyDBSnapshotAttribute</code> API action.<br/>Marker:   An optional pagination token provided by a previous <code>DescribeDBSnapshots</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>SnapshotType:  The type of snapshots to be returned. You can specify one of the following values: <ul> <li>  <code>automated</code> - Return all DB snapshots that have been automatically taken by Amazon RDS for my AWS account. </li> <li>  <code>manual</code> - Return all DB snapshots that have been taken by my AWS account. </li> <li>  <code>shared</code> - Return all manual DB snapshots that have been shared to my AWS account. </li> <li>  <code>public</code> - Return all DB snapshots that have been marked as public. </li> <li>  <code>awsbackup</code> - Return the DB snapshots managed by the AWS Backup service. For information about AWS Backup, see the <a href="https://docs.aws.amazon.com/aws-backup/latest/devguide/whatisbackup.html"> <i>AWS Backup Developer Guide.</i> </a>  The <code>awsbackup</code> type does not apply to Aurora. </li> </ul> If you don't specify a <code>SnapshotType</code> value, then both automated and manual snapshots are returned. Shared and public DB snapshots are not included in the returned results by default. You can include shared snapshots with these results by enabling the <code>IncludeShared</code> parameter. You can include public snapshots with these results by enabling the <code>IncludePublic</code> parameter. The <code>IncludeShared</code> and <code>IncludePublic</code> parameters don't apply for <code>SnapshotType</code> values of <code>manual</code> or <code>automated</code>. The <code>IncludePublic</code> parameter doesn't apply when <code>SnapshotType</code> is set to <code>shared</code>. The <code>IncludeShared</code> parameter doesn't apply when <code>SnapshotType</code> is set to <code>public</code>.<br/>DBSnapshotIdentifier:   A specific DB snapshot identifier to describe. This parameter can't be used in conjunction with <code>DBInstanceIdentifier</code>. This value is stored as a lowercase string.  Constraints: <ul> <li> If supplied, must match the identifier of an existing DBSnapshot. </li> <li> If this identifier is for an automated snapshot, the <code>SnapshotType</code> parameter must also be specified. </li> </ul><br/>DBInstanceIdentifier:  The ID of the DB instance to retrieve the list of DB snapshots for. This parameter can't be used in conjunction with <code>DBSnapshotIdentifier</code>. This parameter is not case-sensitive.  Constraints: <ul> <li> If supplied, must match the identifier of an existing DBInstance. </li> </ul><br/>
        :return: DBSnapshots:   A list of <code>DBSnapshot</code> instances. <br/>Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>        """
        pass

    def describe_db_subnet_groups(self, DBSubnetGroupName: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DBSubnetGroupMessage:
        """Returns a list of DBSubnetGroup descriptions. If a DBSubnetGroupName is specified, the list will contain only the descriptions of the specified DBSubnetGroup. For an overview of CIDR ranges, go to the <a href="http://en.wikipedia.org/wiki/Classless_Inter-Domain_Routing">Wikipedia Tutorial</a>. 

        :param self:
        :param Marker:   An optional pagination token provided by a previous DescribeDBSubnetGroups request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>DBSubnetGroupName:  The name of the DB subnet group to return details for.<br/>
        :return: DBSubnetGroups:   A list of <code>DBSubnetGroup</code> instances. <br/>Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>        """
        pass

    def describe_engine_default_cluster_parameters(self, DBParameterGroupFamily: str, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DescribeEngineDefaultClusterParametersResult:
        """Returns the default engine and system parameter information for the cluster database engine. For more information on Amazon Aurora, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/CHAP_AuroraOverview.html"> What Is Amazon Aurora?</a> in the <i>Amazon Aurora User Guide.</i> 

        :param self:
        :param Marker:   An optional pagination token provided by a previous <code>DescribeEngineDefaultClusterParameters</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>DBParameterGroupFamily:  The name of the DB cluster parameter group family to return engine parameter information for.<br/>
        :return: EngineDefaults:   Contains the result of a successful invocation of the <code>DescribeEngineDefaultParameters</code> action. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> Contains a list of engine default parameters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Marker</i>  An optional pagination token provided by a previous EngineDefaults request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code> . <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroupFamily</i> Specifies the name of the DB parameter group family that the engine default parameters apply to.<br/>        """
        pass

    def describe_engine_default_parameters(self, DBParameterGroupFamily: str, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> DescribeEngineDefaultParametersResult:
        """Returns the default engine and system parameter information for the specified database engine.

        :param self:
        :param Marker:   An optional pagination token provided by a previous <code>DescribeEngineDefaultParameters</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>DBParameterGroupFamily:  The name of the DB parameter group family.<br/>
        :return: EngineDefaults:   Contains the result of a successful invocation of the <code>DescribeEngineDefaultParameters</code> action. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> Contains a list of engine default parameters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Marker</i>  An optional pagination token provided by a previous EngineDefaults request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code> . <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroupFamily</i> Specifies the name of the DB parameter group family that the engine default parameters apply to.<br/>        """
        pass

    def describe_event_categories(self, SourceType: str=None, Filters: List[Filter]=None) -> EventCategoriesMessage:
        """Displays a list of categories for all event source types, or, if specified, for a specified source type. You can see a list of the event categories and source types in the <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Events.html"> Events</a> topic in the <i>Amazon RDS User Guide.</i> 

        :param self:
        :param Filters:  This parameter is not currently supported.<br/>SourceType:  The type of source that is generating the events. Valid values: db-instance | db-parameter-group | db-security-group | db-snapshot<br/>
        :return: EventCategoriesMapList:  A list of EventCategoriesMap data types.<br/>        """
        pass

    def describe_event_subscriptions(self, SubscriptionName: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> EventSubscriptionsMessage:
        """Lists all the subscription descriptions for a customer account. The description for a subscription includes SubscriptionName, SNSTopicARN, CustomerID, SourceType, SourceID, CreationTime, and Status. If you specify a SubscriptionName, lists the description for that subscription.

        :param self:
        :param Marker:   An optional pagination token provided by a previous DescribeOrderableDBInstanceOptions request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code> . <br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>SubscriptionName:  The name of the RDS event notification subscription you want to describe.<br/>
        :return: EventSubscriptionsList:  A list of EventSubscriptions data types.<br/>Marker:   An optional pagination token provided by a previous DescribeOrderableDBInstanceOptions request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>        """
        pass

    def describe_events(self, SourceIdentifier: str=None, SourceType: str=None, StartTime: TStamp=None, EndTime: TStamp=None, Duration: int=None, EventCategories: List[String]=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> EventsMessage:
        """Returns events related to DB instances, DB security groups, DB snapshots, and DB parameter groups for the past 14 days. Events specific to a particular DB instance, DB security group, database snapshot, or DB parameter group can be obtained by providing the name as a parameter. By default, the past hour of events are returned.

        :param self:
        :param Marker:   An optional pagination token provided by a previous DescribeEvents request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>EventCategories:  A list of event categories that trigger notifications for a event notification subscription.<br/>Duration:  The number of minutes to retrieve events for. Default: 60<br/>EndTime:   The end of the time interval for which to retrieve events, specified in ISO 8601 format. For more information about ISO 8601, go to the <a href="http://en.wikipedia.org/wiki/ISO_8601">ISO8601 Wikipedia page.</a>  Example: 2009-07-08T18:00Z<br/>StartTime:   The beginning of the time interval to retrieve events for, specified in ISO 8601 format. For more information about ISO 8601, go to the <a href="http://en.wikipedia.org/wiki/ISO_8601">ISO8601 Wikipedia page.</a>  Example: 2009-07-08T18:00Z<br/>SourceType:  The event source to retrieve events for. If no value is specified, all events are returned.<br/>SourceIdentifier:  The identifier of the event source for which events are returned. If not specified, then all sources are included in the response. Constraints: <ul> <li> If SourceIdentifier is supplied, SourceType must also be provided. </li> <li> If the source type is <code>DBInstance</code>, then a <code>DBInstanceIdentifier</code> must be supplied. </li> <li> If the source type is <code>DBSecurityGroup</code>, a <code>DBSecurityGroupName</code> must be supplied. </li> <li> If the source type is <code>DBParameterGroup</code>, a <code>DBParameterGroupName</code> must be supplied. </li> <li> If the source type is <code>DBSnapshot</code>, a <code>DBSnapshotIdentifier</code> must be supplied. </li> <li> Can't end with a hyphen or contain two consecutive hyphens. </li> </ul><br/>
        :return: Events:   A list of <code>Event</code> instances. <br/>Marker:   An optional pagination token provided by a previous Events request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code> . <br/>        """
        pass

    def describe_global_clusters(self, GlobalClusterIdentifier: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> GlobalClustersMessage:
        """ Returns information about Aurora global database clusters. This API supports pagination.   For more information on Amazon Aurora, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/CHAP_AuroraOverview.html"> What Is Amazon Aurora?</a> in the <i>Amazon Aurora User Guide.</i>  <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param Marker:   An optional pagination token provided by a previous <code>DescribeGlobalClusters</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  A filter that specifies one or more global DB clusters to describe. Supported filters: <ul> <li>  <code>db-cluster-id</code> - Accepts DB cluster identifiers and DB cluster Amazon Resource Names (ARNs). The results list will only include information about the DB clusters identified by these ARNs. </li> </ul><br/>GlobalClusterIdentifier:   The user-supplied DB cluster identifier. If this parameter is specified, information from only the specific DB cluster is returned. This parameter isn't case-sensitive.  Constraints: <ul> <li> If supplied, must match an existing DBClusterIdentifier. </li> </ul><br/>
        :return: GlobalClusters:   The list of global clusters returned by this request. <br/>Marker:   An optional pagination token provided by a previous <code>DescribeGlobalClusters</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>        """
        pass

    def describe_option_group_options(self, EngineName: str, MajorEngineVersion: str=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> OptionGroupOptionsMessage:
        """Describes all available options.

        :param self:
        :param Marker:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>MajorEngineVersion:  If specified, filters the results to include only options for the specified major engine version.<br/>EngineName:  A required parameter. Options available for the given engine name are described.<br/>
        :return: Marker:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>OptionGroupOptions:  List of available option group options.<br/>        """
        pass

    def describe_option_groups(self, OptionGroupName: str=None, Filters: List[Filter]=None, Marker: str=None, MaxRecords: int=None, EngineName: str=None, MajorEngineVersion: str=None) -> OptionGroups:
        """Describes the available option groups.

        :param self:
        :param MajorEngineVersion:  Filters the list of option groups to only include groups associated with a specific database engine version. If specified, then EngineName must also be specified.<br/>EngineName:  Filters the list of option groups to only include groups associated with a specific database engine.<br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Marker:   An optional pagination token provided by a previous DescribeOptionGroups request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>Filters:  This parameter is not currently supported.<br/>OptionGroupName:  The name of the option group to describe. Can't be supplied together with EngineName or MajorEngineVersion.<br/>
        :return: Marker:  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>OptionGroupsList:  List of option groups.<br/>        """
        pass

    def describe_orderable_db_instance_options(self, Engine: str, EngineVersion: str=None, DBInstanceClass: str=None, LicenseModel: str=None, Vpc: BooleanOptional=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> OrderableDBInstanceOptionsMessage:
        """Returns a list of orderable DB instance options for the specified engine.

        :param self:
        :param Marker:   An optional pagination token provided by a previous DescribeOrderableDBInstanceOptions request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code> . <br/>MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>Vpc:  A value that indicates whether to show only VPC or non-VPC offerings.<br/>LicenseModel:  The license model filter value. Specify this parameter to show only the available offerings matching the specified license model.<br/>DBInstanceClass:  The DB instance class filter value. Specify this parameter to show only the available offerings matching the specified DB instance class.<br/>EngineVersion:  The engine version filter value. Specify this parameter to show only the available offerings matching the specified engine version.<br/>Engine:  The name of the engine to retrieve DB instance options for.<br/>
        :return: Marker:   An optional pagination token provided by a previous OrderableDBInstanceOptions request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code> . <br/>OrderableDBInstanceOptions:  An <code>OrderableDBInstanceOption</code> structure containing information about orderable options for the DB instance.<br/>        """
        pass

    def describe_pending_maintenance_actions(self, ResourceIdentifier: str=None, Filters: List[Filter]=None, Marker: str=None, MaxRecords: int=None) -> PendingMaintenanceActionsMessage:
        """Returns a list of resources (for example, DB instances) that have at least one pending maintenance action.

        :param self:
        :param MaxRecords:   The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Marker:   An optional pagination token provided by a previous <code>DescribePendingMaintenanceActions</code> request. If this parameter is specified, the response includes only records beyond the marker, up to a number of records specified by <code>MaxRecords</code>. <br/>Filters:  A filter that specifies one or more resources to return pending maintenance actions for. Supported filters: <ul> <li>  <code>db-cluster-id</code> - Accepts DB cluster identifiers and DB cluster Amazon Resource Names (ARNs). The results list will only include pending maintenance actions for the DB clusters identified by these ARNs. </li> <li>  <code>db-instance-id</code> - Accepts DB instance identifiers and DB instance ARNs. The results list will only include pending maintenance actions for the DB instances identified by these ARNs. </li> </ul><br/>ResourceIdentifier:  The ARN of a resource to return pending maintenance actions for.<br/>
        :return: Marker:   An optional pagination token provided by a previous <code>DescribePendingMaintenanceActions</code> request. If this parameter is specified, the response includes only records beyond the marker, up to a number of records specified by <code>MaxRecords</code>. <br/>PendingMaintenanceActions:  A list of the pending maintenance actions for the resource.<br/>        """
        pass

    def describe_reserved_db_instances(self, ReservedDBInstanceId: str=None, ReservedDBInstancesOfferingId: str=None, DBInstanceClass: str=None, Duration: str=None, ProductDescription: str=None, OfferingType: str=None, MultiAZ: BooleanOptional=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> ReservedDBInstanceMessage:
        """Returns information about reserved DB instances for this account, or about a specified reserved DB instance.

        :param self:
        :param Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more than the <code>MaxRecords</code> value is available, a pagination token called a marker is included in the response so that the following results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>MultiAZ:  A value that indicates whether to show only those reservations that support Multi-AZ.<br/>OfferingType:  The offering type filter value. Specify this parameter to show only the available offerings matching the specified offering type. Valid Values: <code>"Partial Upfront" | "All Upfront" | "No Upfront" </code> <br/>ProductDescription:  The product description filter value. Specify this parameter to show only those reservations matching the specified product description.<br/>Duration:  The duration filter value, specified in years or seconds. Specify this parameter to show only reservations for this duration. Valid Values: <code>1 | 3 | 31536000 | 94608000</code> <br/>DBInstanceClass:  The DB instance class filter value. Specify this parameter to show only those reservations matching the specified DB instances class.<br/>ReservedDBInstancesOfferingId:  The offering identifier filter value. Specify this parameter to show only purchased reservations matching the specified offering identifier.<br/>ReservedDBInstanceId:  The reserved DB instance identifier filter value. Specify this parameter to show only the reservation that matches the specified reservation ID.<br/>
        :return: ReservedDBInstances:  A list of reserved DB instances.<br/>Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>        """
        pass

    def describe_reserved_db_instances_offerings(self, ReservedDBInstancesOfferingId: str=None, DBInstanceClass: str=None, Duration: str=None, ProductDescription: str=None, OfferingType: str=None, MultiAZ: BooleanOptional=None, Filters: List[Filter]=None, MaxRecords: int=None, Marker: str=None) -> ReservedDBInstancesOfferingMessage:
        """Lists available reserved DB instance offerings.

        :param self:
        :param Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>MaxRecords:   The maximum number of records to include in the response. If more than the <code>MaxRecords</code> value is available, a pagination token called a marker is included in the response so that the following results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>Filters:  This parameter is not currently supported.<br/>MultiAZ:  A value that indicates whether to show only those reservations that support Multi-AZ.<br/>OfferingType:  The offering type filter value. Specify this parameter to show only the available offerings matching the specified offering type. Valid Values: <code>"Partial Upfront" | "All Upfront" | "No Upfront" </code> <br/>ProductDescription:  Product description filter value. Specify this parameter to show only the available offerings that contain the specified product description. <note> The results show offerings that partially match the filter value. </note><br/>Duration:  Duration filter value, specified in years or seconds. Specify this parameter to show only reservations for this duration. Valid Values: <code>1 | 3 | 31536000 | 94608000</code> <br/>DBInstanceClass:  The DB instance class filter value. Specify this parameter to show only the available offerings matching the specified DB instance class.<br/>ReservedDBInstancesOfferingId:  The offering identifier filter value. Specify this parameter to show only the available offering that matches the specified reservation identifier. Example: <code>438012d3-4052-4cc7-b2e3-8d3372e0e706</code> <br/>
        :return: ReservedDBInstancesOfferings:  A list of reserved DB instance offerings.<br/>Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>        """
        pass

    def describe_source_regions(self, RegionName: str=None, MaxRecords: int=None, Marker: str=None, Filters: List[Filter]=None) -> SourceRegionMessage:
        """Returns a list of the source AWS Regions where the current AWS Region can create a Read Replica or copy a DB snapshot from. This API action supports pagination.

        :param self:
        :param Filters:  This parameter is not currently supported.<br/>Marker:  An optional pagination token provided by a previous <code>DescribeSourceRegions</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>.<br/>MaxRecords:  The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100.<br/>RegionName:  The source AWS Region name. For example, <code>us-east-1</code>. Constraints: <ul> <li> Must specify a valid AWS Region name. </li> </ul><br/>
        :return: SourceRegions:  A list of SourceRegion instances that contains each source AWS Region that the current AWS Region can get a Read Replica or a DB snapshot from.<br/>Marker:   An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by <code>MaxRecords</code>. <br/>        """
        pass

    def describe_valid_db_instance_modifications(self, DBInstanceIdentifier: str) -> DescribeValidDBInstanceModificationsResult:
        """You can call <code>DescribeValidDBInstanceModifications</code> to learn what modifications you can make to your DB instance. You can use this information when you call <code>ModifyDBInstance</code>. 

        :param self:
        :param DBInstanceIdentifier:  The customer identifier or the ARN of your DB instance. <br/>
        :return: ValidDBInstanceModificationsMessage:  Information about valid modifications that you can make to your DB instance. Contains the result of a successful call to the <code>DescribeValidDBInstanceModifications</code> action. You can use this information when you call <code>ModifyDBInstance</code>. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ValidProcessorFeatures</i> Valid processor features for your DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Storage</i> Valid storage options for your DB instance. <br/>        """
        pass

    def download_db_log_file_portion(self, LogFileName: str, DBInstanceIdentifier: str, Marker: str=None, NumberOfLines: int=None) -> DownloadDBLogFilePortionDetails:
        """Downloads all or a portion of the specified log file, up to 1 MB in size.

        :param self:
        :param NumberOfLines:  The number of lines to download. If the number of lines specified results in a file over 1 MB in size, the file is truncated at 1 MB in size. If the NumberOfLines parameter is specified, then the block of lines returned can be from the beginning or the end of the log file, depending on the value of the Marker parameter. <ul> <li> If neither Marker or NumberOfLines are specified, the entire log file is returned up to a maximum of 10000 lines, starting with the most recent log entries first. </li> <li> If NumberOfLines is specified and Marker is not specified, then the most recent lines from the end of the log file are returned. </li> <li> If Marker is specified as "0", then the specified number of lines from the beginning of the log file are returned. </li> <li> You can download the log file in blocks of lines by specifying the size of the block using the NumberOfLines parameter, and by specifying a value of "0" for the Marker parameter in your first request. Include the Marker value returned in the response as the Marker value for the next request, continuing until the AdditionalDataPending response element returns false. </li> </ul><br/>Marker:  The pagination token provided in the previous request or "0". If the Marker parameter is specified the response includes only records beyond the marker until the end of the file or up to NumberOfLines.<br/>LogFileName:  The name of the log file to be downloaded.<br/>DBInstanceIdentifier:  The customer-assigned name of the DB instance that contains the log files you want to list. Constraints: <ul> <li> Must match the identifier of an existing DBInstance. </li> </ul><br/>
        :return: AdditionalDataPending:  Boolean value that if true, indicates there is more data to be downloaded.<br/>Marker:  A pagination token that can be used in a subsequent DownloadDBLogFilePortion request.<br/>LogFileData:  Entries from the specified log file.<br/>        """
        pass

    def failover_db_cluster(self, DBClusterIdentifier: str, TargetDBInstanceIdentifier: str=None) -> FailoverDBClusterResult:
        """Forces a failover for a DB cluster. A failover for a DB cluster promotes one of the Aurora Replicas (read-only instances) in the DB cluster to be the primary instance (the cluster writer). Amazon Aurora will automatically fail over to an Aurora Replica, if one exists, when the primary instance fails. You can force a failover when you want to simulate a failure of a primary instance for testing. Because each instance in a DB cluster has its own endpoint address, you will need to clean up and re-establish any existing connections that use those endpoint addresses when the failover is complete. For more information on Amazon Aurora, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/CHAP_AuroraOverview.html"> What Is Amazon Aurora?</a> in the <i>Amazon Aurora User Guide.</i>  <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param TargetDBInstanceIdentifier:  The name of the instance to promote to the primary instance. You must specify the instance identifier for an Aurora Replica in the DB cluster. For example, <code>mydbcluster-replica1</code>.<br/>DBClusterIdentifier:  A DB cluster identifier to force a failover for. This parameter is not case-sensitive. Constraints: <ul> <li> Must match the identifier of an existing DBCluster. </li> </ul><br/>
        :return: DBCluster:  Contains the details of an Amazon Aurora DB cluster.  This data type is used as a response element in the <code>DescribeDBClusters</code>, <code>StopDBCluster</code>, and <code>StartDBCluster</code> actions. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CrossAccountClone</i> Specifies whether the DB cluster is a clone of a DB cluster owned by a different AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyTagsToSnapshot</i> Specifies whether tags are copied from the DB cluster to snapshots of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamKinesisStreamName</i> The name of the Amazon Kinesis data stream used for the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamKmsKeyId</i> The AWS KMS key identifier used for encrypting messages in the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamStatus</i> The status of the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamMode</i> The mode of the database activity stream. Database events such as a change or access generate an activity stream event. The database session can handle these events either synchronously or asynchronously. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HttpEndpointEnabled</i> A value that indicates whether the HTTP endpoint for an Aurora Serverless DB cluster is enabled. When enabled, the HTTP endpoint provides a connectionless web service API for running SQL queries on the Aurora Serverless DB cluster. You can also query your database from inside the RDS console with the query editor. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/data-api.html">Using the Data API for Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i> Indicates if the DB cluster has deletion protection enabled. The database can't be deleted when deletion protection is enabled. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ScalingConfigurationInfo</i> Shows the scaling configuration for an Aurora DB cluster in <code>serverless</code> DB engine mode. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ScalingConfigurationInfo</b>: Shows the scaling configuration for an Aurora DB cluster in <code>serverless</code> DB engine mode. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineMode</i> The DB engine mode of the DB cluster, either <code>provisioned</code>, <code>serverless</code>, or <code>parallelquery</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Capacity</i> The current capacity of an Aurora Serverless DB cluster. The capacity is 0 (zero) when the cluster is paused. For more information about Aurora Serverless, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB cluster is configured to export to CloudWatch Logs. Log types vary by DB engine. For information about the log types for each DB engine, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/USER_LogAccess.html">Amazon RDS Database Log Files</a> in the <i>Amazon Aurora User Guide.</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BacktrackConsumedChangeRecords</i> The number of change records stored for Backtrack.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BacktrackWindow</i> The target backtrack window, in seconds. If this value is set to 0, backtracking is disabled for the DB cluster. Otherwise, backtracking is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestBacktrackTime</i> The earliest time to which a DB cluster can be backtracked.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloneGroupId</i> Identifies the clone group to which the DB cluster is associated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> A value that indicates whether the mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i> Provides a list of the AWS Identity and Access Management (IAM) roles that are associated with the DB cluster. IAM roles that are associated with a DB cluster grant permission for the DB cluster to access other AWS services on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterArn</i> The Amazon Resource Name (ARN) for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbClusterResourceId</i> The AWS Region-unique, immutable identifier for the DB cluster. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB cluster is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is enabled, the AWS KMS key identifier for the encrypted DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security groups that the DB cluster belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterMembers</i> Provides the list of instances that make up the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSourceIdentifier</i> Contains the identifier of the source DB cluster if this DB cluster is a Read Replica.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i> Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterOptionGroupMemberships</i> Provides the list of option group memberships for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies whether the DB cluster has instances in multiple Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomEndpoints</i> Identifies all custom endpoints associated with the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReaderEndpoint</i> The reader endpoint for the DB cluster. The reader endpoint for a DB cluster load-balances connections across the Aurora Replicas that are available in a DB cluster. As clients request new connections to the reader endpoint, Aurora distributes the connection requests among the Aurora Replicas in the DB cluster. This functionality can help balance your read workload across multiple Aurora Replicas in your DB cluster.  If a failover occurs, and the Aurora Replica that you are connected to is promoted to be the primary instance, your connection is dropped. To continue sending your read workload to other Aurora Replicas in the cluster, you can then reconnect to the reader endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint for the primary instance of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestRestorableTime</i> The earliest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the progress of the operation as a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the current state of this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB cluster, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroup</i> Specifies the name of the DB cluster parameter group for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Contains a user-supplied DB cluster identifier. This identifier is the unique key that identifies a DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> Contains the name of the initial database of this DB cluster that was provided at create time, if one was specified when the DB cluster was created. This same name is returned for the life of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this cluster is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of Availability Zones (AZs) where instances in the DB cluster can be created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> For all database engines except Amazon Aurora, <code>AllocatedStorage</code> specifies the allocated storage size in gibibytes (GiB). For Aurora, <code>AllocatedStorage</code> always returns 1, because Aurora DB cluster storage size is not fixed, but instead automatically adjusts as needed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeoutAction</i> The timeout action of a call to <code>ModifyCurrentDBClusterCapacity</code>, either <code>ForceApplyCapacityChange</code> or <code>RollbackCapacityChange</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondsUntilAutoPause</i> The remaining amount of time, in seconds, before the Aurora DB cluster in <code>serverless</code> mode is paused. A DB cluster can be paused only when it's idle (it has no connections).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoPause</i> A value that indicates whether automatic pause is allowed for the Aurora DB cluster in <code>serverless</code> DB engine mode. When the value is set to false for an Aurora Serverless DB cluster, the DB cluster automatically resumes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxCapacity</i> The maximum capacity for an Aurora DB cluster in <code>serverless</code> DB engine mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinCapacity</i> The maximum capacity for the Aurora DB cluster in <code>serverless</code> DB engine mode.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceName: str, Filters: List[Filter]=None) -> TagListMessage:
        """Lists all tags on an Amazon RDS resource. For an overview on tagging an Amazon RDS resource, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/Overview.Tagging.html">Tagging Amazon RDS Resources</a> in the <i>Amazon RDS User Guide</i>.

        :param self:
        :param Filters:  This parameter is not currently supported.<br/>ResourceName:  The Amazon RDS resource with tags to be listed. This value is an Amazon Resource Name (ARN). For information about creating an ARN, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Tagging.ARN.html#USER_Tagging.ARN.Constructing"> Constructing an ARN for Amazon RDS</a> in the <i>Amazon RDS User Guide</i>.<br/>
        :return: TagList:  List of tags returned by the ListTagsForResource operation.<br/>        """
        pass

    def modify_current_db_cluster_capacity(self, DBClusterIdentifier: str, Capacity: int=None, SecondsBeforeTimeout: int=None, TimeoutAction: str=None) -> DBClusterCapacityInfo:
        """Set the capacity of an Aurora Serverless DB cluster to a specific value. Aurora Serverless scales seamlessly based on the workload on the DB cluster. In some cases, the capacity might not scale fast enough to meet a sudden change in workload, such as a large number of new transactions. Call <code>ModifyCurrentDBClusterCapacity</code> to set the capacity explicitly. After this call sets the DB cluster capacity, Aurora Serverless can automatically scale the DB cluster based on the cooldown period for scaling up and the cooldown period for scaling down. For more information about Aurora Serverless, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>. <important> If you call <code>ModifyCurrentDBClusterCapacity</code> with the default <code>TimeoutAction</code>, connections that prevent Aurora Serverless from finding a scaling point might be dropped. For more information about scaling points, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.how-it-works.html#aurora-serverless.how-it-works.auto-scaling"> Autoscaling for Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>. </important> <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param TimeoutAction:  The action to take when the timeout is reached, either <code>ForceApplyCapacityChange</code> or <code>RollbackCapacityChange</code>.  <code>ForceApplyCapacityChange</code>, the default, sets the capacity to the specified value as soon as possible.  <code>RollbackCapacityChange</code> ignores the capacity change if a scaling point is not found in the timeout period.<br/>SecondsBeforeTimeout:  The amount of time, in seconds, that Aurora Serverless tries to find a scaling point to perform seamless scaling before enforcing the timeout action. The default is 300. <ul> <li> Value must be from 10 through 600. </li> </ul><br/>Capacity:  The DB cluster capacity. When you change the capacity of a paused Aurora Serverless DB cluster, it automatically resumes. Constraints: <ul> <li> Value must be <code>1</code>, <code>2</code>, <code>4</code>, <code>8</code>, <code>16</code>, <code>32</code>, <code>64</code>, <code>128</code>, or <code>256</code>. </li> </ul><br/>DBClusterIdentifier:  The DB cluster identifier for the cluster being modified. This parameter is not case-sensitive. Constraints: <ul> <li> Must match the identifier of an existing DB cluster. </li> </ul><br/>
        :return: TimeoutAction:  The timeout action of a call to <code>ModifyCurrentDBClusterCapacity</code>, either <code>ForceApplyCapacityChange</code> or <code>RollbackCapacityChange</code>.<br/>SecondsBeforeTimeout:  The number of seconds before a call to <code>ModifyCurrentDBClusterCapacity</code> times out.<br/>CurrentCapacity:  The current capacity of the DB cluster.<br/>PendingCapacity:  A value that specifies the capacity that the DB cluster scales to next.<br/>DBClusterIdentifier:  A user-supplied DB cluster identifier. This identifier is the unique key that identifies a DB cluster. <br/>        """
        pass

    def modify_db_cluster(self, DBClusterIdentifier: str, NewDBClusterIdentifier: str=None, ApplyImmediately: Boolean=None, BackupRetentionPeriod: int=None, DBClusterParameterGroupName: str=None, VpcSecurityGroupIds: List[String]=None, Port: int=None, MasterUserPassword: str=None, OptionGroupName: str=None, PreferredBackupWindow: str=None, PreferredMaintenanceWindow: str=None, EnableIAMDatabaseAuthentication: BooleanOptional=None, BacktrackWindow: LongOptional=None, CloudwatchLogsExportConfiguration: CloudwatchLogsExportConfiguration=None, EngineVersion: str=None, AllowMajorVersionUpgrade: Boolean=None, DBInstanceParameterGroupName: str=None, ScalingConfiguration: ScalingConfiguration=None, DeletionProtection: BooleanOptional=None, EnableHttpEndpoint: BooleanOptional=None, CopyTagsToSnapshot: BooleanOptional=None) -> ModifyDBClusterResult:
        """Modify a setting for an Amazon Aurora DB cluster. You can change one or more database configuration parameters by specifying these parameters and the new values in the request. For more information on Amazon Aurora, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/CHAP_AuroraOverview.html"> What Is Amazon Aurora?</a> in the <i>Amazon Aurora User Guide.</i>  <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param CopyTagsToSnapshot:  A value that indicates whether to copy all tags from the DB cluster to snapshots of the DB cluster. The default is not to copy them.<br/>EnableHttpEndpoint:  A value that indicates whether to enable the HTTP endpoint for an Aurora Serverless DB cluster. By default, the HTTP endpoint is disabled. When enabled, the HTTP endpoint provides a connectionless web service API for running SQL queries on the Aurora Serverless DB cluster. You can also query your database from inside the RDS console with the query editor. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/data-api.html">Using the Data API for Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>DeletionProtection:  A value that indicates whether the DB cluster has deletion protection enabled. The database can't be deleted when deletion protection is enabled. By default, deletion protection is disabled. <br/>ScalingConfiguration:  The scaling properties of the DB cluster. You can only modify scaling properties for DB clusters in <code>serverless</code> DB engine mode.:<br/>DBInstanceParameterGroupName:  The name of the DB parameter group to apply to all instances of the DB cluster.  <note> When you apply a parameter group using the <code>DBInstanceParameterGroupName</code> parameter, the DB cluster isn't rebooted automatically. Also, parameter changes aren't applied during the next maintenance window but instead are applied immediately. </note> Default: The existing name setting Constraints: <ul> <li> The DB parameter group must be in the same DB parameter group family as this DB cluster. </li> <li> The <code>DBInstanceParameterGroupName</code> parameter is only valid in combination with the <code>AllowMajorVersionUpgrade</code> parameter. </li> </ul><br/>AllowMajorVersionUpgrade:  A value that indicates whether major version upgrades are allowed. Constraints: You must allow major version upgrades when specifying a value for the <code>EngineVersion</code> parameter that is a different major version than the DB cluster's current version.<br/>EngineVersion:  The version number of the database engine to which you want to upgrade. Changing this parameter results in an outage. The change is applied during the next maintenance window unless <code>ApplyImmediately</code> is enabled. For a list of valid engine versions, use <a>DescribeDBEngineVersions</a>.<br/>CloudwatchLogsExportConfiguration:  The configuration setting for the log types to be enabled for export to CloudWatch Logs for a specific DB cluster.:<br/>BacktrackWindow:  The target backtrack window, in seconds. To disable backtracking, set this value to 0. Default: 0 Constraints: <ul> <li> If specified, this value must be set to a number from 0 to 259,200 (72 hours). </li> </ul><br/>EnableIAMDatabaseAuthentication:  A value that indicates whether to enable mapping of AWS Identity and Access Management (IAM) accounts to database accounts. By default, mapping is disabled.<br/>PreferredMaintenanceWindow:  The weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC). Format: <code>ddd:hh24:mi-ddd:hh24:mi</code>  The default is a 30-minute window selected at random from an 8-hour block of time for each AWS Region, occurring on a random day of the week. To see the time blocks available, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/USER_UpgradeDBInstance.Maintenance.html#AdjustingTheMaintenanceWindow.Aurora"> Adjusting the Preferred DB Cluster Maintenance Window</a> in the <i>Amazon Aurora User Guide.</i>  Valid Days: Mon, Tue, Wed, Thu, Fri, Sat, Sun. Constraints: Minimum 30-minute window.<br/>PreferredBackupWindow:  The daily time range during which automated backups are created if automated backups are enabled, using the <code>BackupRetentionPeriod</code> parameter.  The default is a 30-minute window selected at random from an 8-hour block of time for each AWS Region. To see the time blocks available, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/USER_UpgradeDBInstance.Maintenance.html#AdjustingTheMaintenanceWindow.Aurora"> Adjusting the Preferred DB Cluster Maintenance Window</a> in the <i>Amazon Aurora User Guide.</i>  Constraints: <ul> <li> Must be in the format <code>hh24:mi-hh24:mi</code>. </li> <li> Must be in Universal Coordinated Time (UTC). </li> <li> Must not conflict with the preferred maintenance window. </li> <li> Must be at least 30 minutes. </li> </ul><br/>OptionGroupName:  A value that indicates that the DB cluster should be associated with the specified option group. Changing this parameter doesn't result in an outage except in the following case, and the change is applied during the next maintenance window unless the <code>ApplyImmediately</code> is enabled for this request. If the parameter change results in an option group that enables OEM, this change can cause a brief (sub-second) period during which new connections are rejected but existing connections are not interrupted.  Permanent options can't be removed from an option group. The option group can't be removed from a DB cluster once it is associated with a DB cluster.<br/>MasterUserPassword:  The new password for the master database user. This password can contain any printable ASCII character except "/", triple-quotes, or "@". Constraints: Must contain from 8 to 41 characters.<br/>Port:  The port number on which the DB cluster accepts connections. Constraints: Value must be <code>1150-65535</code>  Default: The same port as the original DB cluster.<br/>VpcSecurityGroupIds:  A list of VPC security groups that the DB cluster will belong to.<br/>DBClusterParameterGroupName:  The name of the DB cluster parameter group to use for the DB cluster.<br/>BackupRetentionPeriod:  The number of days for which automated backups are retained. You must specify a minimum value of 1. Default: 1 Constraints: <ul> <li> Must be a value from 1 to 35 </li> </ul><br/>ApplyImmediately:  A value that indicates whether the modifications in this request and any pending modifications are asynchronously applied as soon as possible, regardless of the <code>PreferredMaintenanceWindow</code> setting for the DB cluster. If this parameter is disabled, changes to the DB cluster are applied during the next maintenance window. The <code>ApplyImmediately</code> parameter only affects the <code>EnableIAMDatabaseAuthentication</code>, <code>MasterUserPassword</code>, and <code>NewDBClusterIdentifier</code> values. If the <code>ApplyImmediately</code> parameter is disabled, then changes to the <code>EnableIAMDatabaseAuthentication</code>, <code>MasterUserPassword</code>, and <code>NewDBClusterIdentifier</code> values are applied during the next maintenance window. All other changes are applied immediately, regardless of the value of the <code>ApplyImmediately</code> parameter. By default, this parameter is disabled.<br/>NewDBClusterIdentifier:  The new DB cluster identifier for the DB cluster when renaming a DB cluster. This value is stored as a lowercase string. Constraints: <ul> <li> Must contain from 1 to 63 letters, numbers, or hyphens </li> <li> The first character must be a letter </li> <li> Can't end with a hyphen or contain two consecutive hyphens </li> </ul> Example: <code>my-cluster2</code> <br/>DBClusterIdentifier:  The DB cluster identifier for the cluster being modified. This parameter is not case-sensitive. Constraints: This identifier must match the identifier of an existing DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisableLogTypes</i> The list of log types to disable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnableLogTypes</i> The list of log types to enable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeoutAction</i> The action to take when the timeout is reached, either <code>ForceApplyCapacityChange</code> or <code>RollbackCapacityChange</code>.  <code>ForceApplyCapacityChange</code> sets the capacity to the specified value as soon as possible.  <code>RollbackCapacityChange</code>, the default, ignores the capacity change if a scaling point is not found in the timeout period. <important> If you specify <code>ForceApplyCapacityChange</code>, connections that prevent Aurora Serverless from finding a scaling point might be dropped. </important> For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.how-it-works.html#aurora-serverless.how-it-works.auto-scaling"> Autoscaling for Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondsUntilAutoPause</i> The time, in seconds, before an Aurora DB cluster in <code>serverless</code> mode is paused.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoPause</i> A value that indicates whether to allow or disallow automatic pause for an Aurora DB cluster in <code>serverless</code> DB engine mode. A DB cluster can be paused only when it's idle (it has no connections). <note> If a DB cluster is paused for more than seven days, the DB cluster might be backed up with a snapshot. In this case, the DB cluster is restored when there is a request to connect to it.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxCapacity</i> The maximum capacity for an Aurora DB cluster in <code>serverless</code> DB engine mode. Valid capacity values are <code>1</code>, <code>2</code>, <code>4</code>, <code>8</code>, <code>16</code>, <code>32</code>, <code>64</code>, <code>128</code>, and <code>256</code>. The maximum capacity must be greater than or equal to the minimum capacity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MinCapacity</i> The minimum capacity for an Aurora DB cluster in <code>serverless</code> DB engine mode. Valid capacity values are <code>1</code>, <code>2</code>, <code>4</code>, <code>8</code>, <code>16</code>, <code>32</code>, <code>64</code>, <code>128</code>, and <code>256</code>. The minimum capacity must be less than or equal to the maximum capacity.<br/>
        :return: DBCluster:  Contains the details of an Amazon Aurora DB cluster.  This data type is used as a response element in the <code>DescribeDBClusters</code>, <code>StopDBCluster</code>, and <code>StartDBCluster</code> actions. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CrossAccountClone</i> Specifies whether the DB cluster is a clone of a DB cluster owned by a different AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyTagsToSnapshot</i> Specifies whether tags are copied from the DB cluster to snapshots of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamKinesisStreamName</i> The name of the Amazon Kinesis data stream used for the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamKmsKeyId</i> The AWS KMS key identifier used for encrypting messages in the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamStatus</i> The status of the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamMode</i> The mode of the database activity stream. Database events such as a change or access generate an activity stream event. The database session can handle these events either synchronously or asynchronously. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HttpEndpointEnabled</i> A value that indicates whether the HTTP endpoint for an Aurora Serverless DB cluster is enabled. When enabled, the HTTP endpoint provides a connectionless web service API for running SQL queries on the Aurora Serverless DB cluster. You can also query your database from inside the RDS console with the query editor. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/data-api.html">Using the Data API for Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i> Indicates if the DB cluster has deletion protection enabled. The database can't be deleted when deletion protection is enabled. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ScalingConfigurationInfo</i> Shows the scaling configuration for an Aurora DB cluster in <code>serverless</code> DB engine mode. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ScalingConfigurationInfo</b>: Shows the scaling configuration for an Aurora DB cluster in <code>serverless</code> DB engine mode. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineMode</i> The DB engine mode of the DB cluster, either <code>provisioned</code>, <code>serverless</code>, or <code>parallelquery</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Capacity</i> The current capacity of an Aurora Serverless DB cluster. The capacity is 0 (zero) when the cluster is paused. For more information about Aurora Serverless, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB cluster is configured to export to CloudWatch Logs. Log types vary by DB engine. For information about the log types for each DB engine, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/USER_LogAccess.html">Amazon RDS Database Log Files</a> in the <i>Amazon Aurora User Guide.</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BacktrackConsumedChangeRecords</i> The number of change records stored for Backtrack.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BacktrackWindow</i> The target backtrack window, in seconds. If this value is set to 0, backtracking is disabled for the DB cluster. Otherwise, backtracking is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestBacktrackTime</i> The earliest time to which a DB cluster can be backtracked.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloneGroupId</i> Identifies the clone group to which the DB cluster is associated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> A value that indicates whether the mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i> Provides a list of the AWS Identity and Access Management (IAM) roles that are associated with the DB cluster. IAM roles that are associated with a DB cluster grant permission for the DB cluster to access other AWS services on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterArn</i> The Amazon Resource Name (ARN) for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbClusterResourceId</i> The AWS Region-unique, immutable identifier for the DB cluster. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB cluster is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is enabled, the AWS KMS key identifier for the encrypted DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security groups that the DB cluster belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterMembers</i> Provides the list of instances that make up the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSourceIdentifier</i> Contains the identifier of the source DB cluster if this DB cluster is a Read Replica.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i> Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterOptionGroupMemberships</i> Provides the list of option group memberships for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies whether the DB cluster has instances in multiple Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomEndpoints</i> Identifies all custom endpoints associated with the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReaderEndpoint</i> The reader endpoint for the DB cluster. The reader endpoint for a DB cluster load-balances connections across the Aurora Replicas that are available in a DB cluster. As clients request new connections to the reader endpoint, Aurora distributes the connection requests among the Aurora Replicas in the DB cluster. This functionality can help balance your read workload across multiple Aurora Replicas in your DB cluster.  If a failover occurs, and the Aurora Replica that you are connected to is promoted to be the primary instance, your connection is dropped. To continue sending your read workload to other Aurora Replicas in the cluster, you can then reconnect to the reader endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint for the primary instance of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestRestorableTime</i> The earliest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the progress of the operation as a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the current state of this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB cluster, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroup</i> Specifies the name of the DB cluster parameter group for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Contains a user-supplied DB cluster identifier. This identifier is the unique key that identifies a DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> Contains the name of the initial database of this DB cluster that was provided at create time, if one was specified when the DB cluster was created. This same name is returned for the life of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this cluster is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of Availability Zones (AZs) where instances in the DB cluster can be created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> For all database engines except Amazon Aurora, <code>AllocatedStorage</code> specifies the allocated storage size in gibibytes (GiB). For Aurora, <code>AllocatedStorage</code> always returns 1, because Aurora DB cluster storage size is not fixed, but instead automatically adjusts as needed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeoutAction</i> The timeout action of a call to <code>ModifyCurrentDBClusterCapacity</code>, either <code>ForceApplyCapacityChange</code> or <code>RollbackCapacityChange</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondsUntilAutoPause</i> The remaining amount of time, in seconds, before the Aurora DB cluster in <code>serverless</code> mode is paused. A DB cluster can be paused only when it's idle (it has no connections).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoPause</i> A value that indicates whether automatic pause is allowed for the Aurora DB cluster in <code>serverless</code> DB engine mode. When the value is set to false for an Aurora Serverless DB cluster, the DB cluster automatically resumes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxCapacity</i> The maximum capacity for an Aurora DB cluster in <code>serverless</code> DB engine mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinCapacity</i> The maximum capacity for the Aurora DB cluster in <code>serverless</code> DB engine mode.<br/>        """
        pass

    def modify_db_cluster_endpoint(self, DBClusterEndpointIdentifier: str, EndpointType: str=None, StaticMembers: List[String]=None, ExcludedMembers: List[String]=None) -> DBClusterEndpoint:
        """Modifies the properties of an endpoint in an Amazon Aurora DB cluster. <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param ExcludedMembers:  List of DB instance identifiers that aren't part of the custom endpoint group. All other eligible instances are reachable through the custom endpoint. Only relevant if the list of static members is empty.<br/>StaticMembers:  List of DB instance identifiers that are part of the custom endpoint group.<br/>EndpointType:  The type of the endpoint. One of: <code>READER</code>, <code>ANY</code>. <br/>DBClusterEndpointIdentifier:  The identifier of the endpoint to modify. This parameter is stored as a lowercase string.<br/>
        :return: DBClusterEndpointArn:  The Amazon Resource Name (ARN) for the endpoint.<br/>ExcludedMembers:  List of DB instance identifiers that aren't part of the custom endpoint group. All other eligible instances are reachable through the custom endpoint. Only relevant if the list of static members is empty.<br/>StaticMembers:  List of DB instance identifiers that are part of the custom endpoint group.<br/>CustomEndpointType:  The type associated with a custom endpoint. One of: <code>READER</code>, <code>ANY</code>.<br/>EndpointType:  The type of the endpoint. One of: <code>READER</code>, <code>WRITER</code>, <code>CUSTOM</code>.<br/>Status:  The current status of the endpoint. One of: <code>creating</code>, <code>available</code>, <code>deleting</code>, <code>modifying</code>.<br/>Endpoint:  The DNS address of the endpoint.<br/>DBClusterEndpointResourceIdentifier:  A unique system-generated identifier for an endpoint. It remains the same for the whole life of the endpoint.<br/>DBClusterIdentifier:  The DB cluster identifier of the DB cluster associated with the endpoint. This parameter is stored as a lowercase string.<br/>DBClusterEndpointIdentifier:  The identifier associated with the endpoint. This parameter is stored as a lowercase string.<br/>        """
        pass

    def modify_db_cluster_parameter_group(self, Parameters: List[Parameter], DBClusterParameterGroupName: str) -> DBClusterParameterGroupNameMessage:
        """ Modifies the parameters of a DB cluster parameter group. To modify more than one parameter, submit a list of the following: <code>ParameterName</code>, <code>ParameterValue</code>, and <code>ApplyMethod</code>. A maximum of 20 parameters can be modified in a single request.  For more information on Amazon Aurora, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/CHAP_AuroraOverview.html"> What Is Amazon Aurora?</a> in the <i>Amazon Aurora User Guide.</i>  <note> Changes to dynamic parameters are applied immediately. Changes to static parameters require a reboot without failover to the DB cluster associated with the parameter group before the change can take effect. </note> <important> After you create a DB cluster parameter group, you should wait at least 5 minutes before creating your first DB cluster that uses that DB cluster parameter group as the default parameter group. This allows Amazon RDS to fully complete the create action before the parameter group is used as the default for a new DB cluster. This is especially important for parameters that are critical when creating the default database for a DB cluster, such as the character set for the default database defined by the <code>character_set_database</code> parameter. You can use the <i>Parameter Groups</i> option of the <a href="https://console.aws.amazon.com/rds/">Amazon RDS console</a> or the <code>DescribeDBClusterParameters</code> action to verify that your DB cluster parameter group has been created or modified. </important> <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param Parameters:  A list of parameters in the DB cluster parameter group to modify.<br/>DBClusterParameterGroupName:  The name of the DB cluster parameter group to modify.<br/>
        :return: DBClusterParameterGroupName:  The name of the DB cluster parameter group. Constraints: <ul> <li> Must be 1 to 255 letters or numbers. </li> <li> First character must be a letter </li> <li> Can't end with a hyphen or contain two consecutive hyphens </li> </ul> <note> This value is stored as a lowercase string. </note><br/>        """
        pass

    def modify_db_cluster_snapshot_attribute(self, AttributeName: str, DBClusterSnapshotIdentifier: str, ValuesToAdd: List[String]=None, ValuesToRemove: List[String]=None) -> ModifyDBClusterSnapshotAttributeResult:
        """Adds an attribute and values to, or removes an attribute and values from, a manual DB cluster snapshot. To share a manual DB cluster snapshot with other AWS accounts, specify <code>restore</code> as the <code>AttributeName</code> and use the <code>ValuesToAdd</code> parameter to add a list of IDs of the AWS accounts that are authorized to restore the manual DB cluster snapshot. Use the value <code>all</code> to make the manual DB cluster snapshot public, which means that it can be copied or restored by all AWS accounts. Do not add the <code>all</code> value for any manual DB cluster snapshots that contain private information that you don't want available to all AWS accounts. If a manual DB cluster snapshot is encrypted, it can be shared, but only by specifying a list of authorized AWS account IDs for the <code>ValuesToAdd</code> parameter. You can't use <code>all</code> as a value for that parameter in this case. To view which AWS accounts have access to copy or restore a manual DB cluster snapshot, or whether a manual DB cluster snapshot public or private, use the <code>DescribeDBClusterSnapshotAttributes</code> API action. <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param ValuesToRemove:  A list of DB cluster snapshot attributes to remove from the attribute specified by <code>AttributeName</code>. To remove authorization for other AWS accounts to copy or restore a manual DB cluster snapshot, set this list to include one or more AWS account identifiers, or <code>all</code> to remove authorization for any AWS account to copy or restore the DB cluster snapshot. If you specify <code>all</code>, an AWS account whose account ID is explicitly added to the <code>restore</code> attribute can still copy or restore a manual DB cluster snapshot.<br/>ValuesToAdd:  A list of DB cluster snapshot attributes to add to the attribute specified by <code>AttributeName</code>. To authorize other AWS accounts to copy or restore a manual DB cluster snapshot, set this list to include one or more AWS account IDs, or <code>all</code> to make the manual DB cluster snapshot restorable by any AWS account. Do not add the <code>all</code> value for any manual DB cluster snapshots that contain private information that you don't want available to all AWS accounts.<br/>AttributeName:  The name of the DB cluster snapshot attribute to modify. To manage authorization for other AWS accounts to copy or restore a manual DB cluster snapshot, set this value to <code>restore</code>.<br/>DBClusterSnapshotIdentifier:  The identifier for the DB cluster snapshot to modify the attributes for.<br/>
        :return: DBClusterSnapshotAttributesResult:  Contains the results of a successful call to the <code>DescribeDBClusterSnapshotAttributes</code> API action. Manual DB cluster snapshot attributes are used to authorize other AWS accounts to copy or restore a manual DB cluster snapshot. For more information, see the <code>ModifyDBClusterSnapshotAttribute</code> API action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotAttributes</i> The list of attributes and values for the manual DB cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterSnapshotIdentifier</i> The identifier of the manual DB cluster snapshot that the attributes apply to.<br/>        """
        pass

    def modify_db_instance(self, DBInstanceIdentifier: str, AllocatedStorage: int=None, DBInstanceClass: str=None, DBSubnetGroupName: str=None, DBSecurityGroups: List[String]=None, VpcSecurityGroupIds: List[String]=None, ApplyImmediately: Boolean=None, MasterUserPassword: str=None, DBParameterGroupName: str=None, BackupRetentionPeriod: int=None, PreferredBackupWindow: str=None, PreferredMaintenanceWindow: str=None, MultiAZ: BooleanOptional=None, EngineVersion: str=None, AllowMajorVersionUpgrade: Boolean=None, AutoMinorVersionUpgrade: BooleanOptional=None, LicenseModel: str=None, Iops: int=None, OptionGroupName: str=None, NewDBInstanceIdentifier: str=None, StorageType: str=None, TdeCredentialArn: str=None, TdeCredentialPassword: str=None, CACertificateIdentifier: str=None, Domain: str=None, CopyTagsToSnapshot: BooleanOptional=None, MonitoringInterval: int=None, DBPortNumber: int=None, PubliclyAccessible: BooleanOptional=None, MonitoringRoleArn: str=None, DomainIAMRoleName: str=None, PromotionTier: int=None, EnableIAMDatabaseAuthentication: BooleanOptional=None, EnablePerformanceInsights: BooleanOptional=None, PerformanceInsightsKMSKeyId: str=None, PerformanceInsightsRetentionPeriod: int=None, CloudwatchLogsExportConfiguration: CloudwatchLogsExportConfiguration=None, ProcessorFeatures: List[ProcessorFeature]=None, UseDefaultProcessorFeatures: BooleanOptional=None, DeletionProtection: BooleanOptional=None, MaxAllocatedStorage: int=None) -> ModifyDBInstanceResult:
        """Modifies settings for a DB instance. You can change one or more database configuration parameters by specifying these parameters and the new values in the request. To learn what modifications you can make to your DB instance, call <code>DescribeValidDBInstanceModifications</code> before you call <code>ModifyDBInstance</code>. 

        :param self:
        :param MaxAllocatedStorage:  The upper limit to which Amazon RDS can automatically scale the storage of the DB instance.<br/>DeletionProtection:  A value that indicates whether the DB instance has deletion protection enabled. The database can't be deleted when deletion protection is enabled. By default, deletion protection is disabled. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_DeleteInstance.html"> Deleting a DB Instance</a>. <br/>UseDefaultProcessorFeatures:  A value that indicates whether the DB instance class of the DB instance uses its default processor features.<br/>ProcessorFeatures:  The number of CPU cores and the number of threads per core for the DB instance class of the DB instance.<br/>CloudwatchLogsExportConfiguration:  The configuration setting for the log types to be enabled for export to CloudWatch Logs for a specific DB instance. A change to the <code>CloudwatchLogsExportConfiguration</code> parameter is always applied to the DB instance immediately. Therefore, the <code>ApplyImmediately</code> parameter has no effect.:<br/>PerformanceInsightsRetentionPeriod:  The amount of time, in days, to retain Performance Insights data. Valid values are 7 or 731 (2 years). <br/>PerformanceInsightsKMSKeyId:  The AWS KMS key identifier for encryption of Performance Insights data. The KMS key ID is the Amazon Resource Name (ARN), KMS key identifier, or the KMS key alias for the KMS encryption key. If you do not specify a value for <code>PerformanceInsightsKMSKeyId</code>, then Amazon RDS uses your default encryption key. AWS KMS creates the default encryption key for your AWS account. Your AWS account has a different default encryption key for each AWS Region.<br/>EnablePerformanceInsights:  A value that indicates whether to enable Performance Insights for the DB instance. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_PerfInsights.html">Using Amazon Performance Insights</a> in the <i>Amazon Relational Database Service User Guide</i>. <br/>EnableIAMDatabaseAuthentication:  A value that indicates whether to enable mapping of AWS Identity and Access Management (IAM) accounts to database accounts. By default, mapping is disabled. You can enable IAM database authentication for the following database engines  <b>Amazon Aurora</b>  Not applicable. Mapping AWS IAM accounts to database accounts is managed by the DB cluster. For more information, see <code>ModifyDBCluster</code>.  <b>MySQL</b>  <ul> <li> For MySQL 5.6, minor version 5.6.34 or higher </li> <li> For MySQL 5.7, minor version 5.7.16 or higher </li> </ul><br/>PromotionTier:  A value that specifies the order in which an Aurora Replica is promoted to the primary instance after a failure of the existing primary instance. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/Aurora.Managing.Backups.html#Aurora.Managing.FaultTolerance"> Fault Tolerance for an Aurora DB Cluster</a> in the <i>Amazon Aurora User Guide</i>.  Default: 1 Valid Values: 0 - 15<br/>DomainIAMRoleName:  The name of the IAM role to use when making API calls to the Directory Service.<br/>MonitoringRoleArn:  The ARN for the IAM role that permits RDS to send enhanced monitoring metrics to Amazon CloudWatch Logs. For example, <code>arn:aws:iam:123456789012:role/emaccess</code>. For information on creating a monitoring role, go to <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Monitoring.html#USER_Monitoring.OS.IAMRole">To create an IAM role for Amazon RDS Enhanced Monitoring</a> in the <i>Amazon RDS User Guide.</i>  If <code>MonitoringInterval</code> is set to a value other than 0, then you must supply a <code>MonitoringRoleArn</code> value.<br/>PubliclyAccessible:  A value that indicates whether the DB instance is publicly accessible. When the DB instance is publicly accessible, it is an Internet-facing instance with a publicly resolvable DNS name, which resolves to a public IP address. When the DB instance is not publicly accessible, it is an internal instance with a DNS name that resolves to a private IP address.   <code>PubliclyAccessible</code> only applies to DB instances in a VPC. The DB instance must be part of a public subnet and <code>PubliclyAccessible</code> must be enabled for it to be publicly accessible.  Changes to the <code>PubliclyAccessible</code> parameter are applied immediately regardless of the value of the <code>ApplyImmediately</code> parameter.<br/>DBPortNumber:  The port number on which the database accepts connections. The value of the <code>DBPortNumber</code> parameter must not match any of the port values specified for options in the option group for the DB instance. Your database will restart when you change the <code>DBPortNumber</code> value regardless of the value of the <code>ApplyImmediately</code> parameter.  <b>MySQL</b>   Default: <code>3306</code>   Valid Values: <code>1150-65535</code>   <b>MariaDB</b>   Default: <code>3306</code>   Valid Values: <code>1150-65535</code>   <b>PostgreSQL</b>   Default: <code>5432</code>   Valid Values: <code>1150-65535</code>  Type: Integer  <b>Oracle</b>   Default: <code>1521</code>   Valid Values: <code>1150-65535</code>   <b>SQL Server</b>   Default: <code>1433</code>   Valid Values: <code>1150-65535</code> except for <code>1434</code>, <code>3389</code>, <code>47001</code>, <code>49152</code>, and <code>49152</code> through <code>49156</code>.   <b>Amazon Aurora</b>   Default: <code>3306</code>   Valid Values: <code>1150-65535</code> <br/>MonitoringInterval:  The interval, in seconds, between points when Enhanced Monitoring metrics are collected for the DB instance. To disable collecting Enhanced Monitoring metrics, specify 0. The default is 0. If <code>MonitoringRoleArn</code> is specified, then you must also set <code>MonitoringInterval</code> to a value other than 0. Valid Values: <code>0, 1, 5, 10, 15, 30, 60</code> <br/>CopyTagsToSnapshot:  A value that indicates whether to copy all tags from the DB instance to snapshots of the DB instance. By default, tags are not copied.  <b>Amazon Aurora</b>  Not applicable. Copying tags to snapshots is managed by the DB cluster. Setting this value for an Aurora DB instance has no effect on the DB cluster setting. For more information, see <code>ModifyDBCluster</code>.<br/>Domain:  The Active Directory Domain to move the instance to. Specify <code>none</code> to remove the instance from its current domain. The domain must be created prior to this operation. Currently only a Microsoft SQL Server instance can be created in a Active Directory Domain. <br/>CACertificateIdentifier:  Indicates the certificate that needs to be associated with the instance.<br/>TdeCredentialPassword:  The password for the given ARN from the key store in order to access the device.<br/>TdeCredentialArn:  The ARN from the key store with which to associate the instance for TDE encryption.<br/>StorageType:  Specifies the storage type to be associated with the DB instance.  If you specify Provisioned IOPS (<code>io1</code>), you must also include a value for the <code>Iops</code> parameter.  If you choose to migrate your DB instance from using standard storage to using Provisioned IOPS, or from using Provisioned IOPS to using standard storage, the process can take time. The duration of the migration depends on several factors such as database load, storage size, storage type (standard or Provisioned IOPS), amount of IOPS provisioned (if any), and the number of prior scale storage operations. Typical migration times are under 24 hours, but the process can take up to several days in some cases. During the migration, the DB instance is available for use, but might experience performance degradation. While the migration takes place, nightly backups for the instance are suspended. No other Amazon RDS operations can take place for the instance, including modifying the instance, rebooting the instance, deleting the instance, creating a Read Replica for the instance, and creating a DB snapshot of the instance.   Valid values: <code>standard | gp2 | io1</code>  Default: <code>io1</code> if the <code>Iops</code> parameter is specified, otherwise <code>gp2</code> <br/>NewDBInstanceIdentifier:   The new DB instance identifier for the DB instance when renaming a DB instance. When you change the DB instance identifier, an instance reboot occurs immediately if you enable <code>ApplyImmediately</code>, or will occur during the next maintenance window if you disable Apply Immediately. This value is stored as a lowercase string.  Constraints: <ul> <li> Must contain from 1 to 63 letters, numbers, or hyphens. </li> <li> The first character must be a letter. </li> <li> Can't end with a hyphen or contain two consecutive hyphens. </li> </ul> Example: <code>mydbinstance</code> <br/>OptionGroupName:   Indicates that the DB instance should be associated with the specified option group. Changing this parameter doesn't result in an outage except in the following case and the change is applied during the next maintenance window unless the <code>ApplyImmediately</code> parameter is enabled for this request. If the parameter change results in an option group that enables OEM, this change can cause a brief (sub-second) period during which new connections are rejected but existing connections are not interrupted.  Permanent options, such as the TDE option for Oracle Advanced Security TDE, can't be removed from an option group, and that option group can't be removed from a DB instance once it is associated with a DB instance<br/>Iops:  The new Provisioned IOPS (I/O operations per second) value for the RDS instance.  Changing this setting doesn't result in an outage and the change is applied during the next maintenance window unless the <code>ApplyImmediately</code> parameter is enabled for this request. If you are migrating from Provisioned IOPS to standard storage, set this value to 0. The DB instance will require a reboot for the change in storage type to take effect.  If you choose to migrate your DB instance from using standard storage to using Provisioned IOPS, or from using Provisioned IOPS to using standard storage, the process can take time. The duration of the migration depends on several factors such as database load, storage size, storage type (standard or Provisioned IOPS), amount of IOPS provisioned (if any), and the number of prior scale storage operations. Typical migration times are under 24 hours, but the process can take up to several days in some cases. During the migration, the DB instance is available for use, but might experience performance degradation. While the migration takes place, nightly backups for the instance are suspended. No other Amazon RDS operations can take place for the instance, including modifying the instance, rebooting the instance, deleting the instance, creating a Read Replica for the instance, and creating a DB snapshot of the instance.  Constraints: For MariaDB, MySQL, Oracle, and PostgreSQL, the value supplied must be at least 10% greater than the current value. Values that are not at least 10% greater than the existing value are rounded up so that they are 10% greater than the current value.  Default: Uses existing setting<br/>LicenseModel:  The license model for the DB instance. Valid values: <code>license-included</code> | <code>bring-your-own-license</code> | <code>general-public-license</code> <br/>AutoMinorVersionUpgrade:   A value that indicates whether minor version upgrades are applied automatically to the DB instance during the maintenance window. Changing this parameter doesn't result in an outage except in the following case and the change is asynchronously applied as soon as possible. An outage results if this parameter is enabled during the maintenance window, and a newer minor version is available, and RDS has enabled auto patching for that engine version. <br/>AllowMajorVersionUpgrade:  A value that indicates whether major version upgrades are allowed. Changing this parameter doesn't result in an outage and the change is asynchronously applied as soon as possible. Constraints: Major version upgrades must be allowed when specifying a value for the EngineVersion parameter that is a different major version than the DB instance's current version.<br/>EngineVersion:   The version number of the database engine to upgrade to. Changing this parameter results in an outage and the change is applied during the next maintenance window unless the <code>ApplyImmediately</code> parameter is eanbled for this request.  For major version upgrades, if a nondefault DB parameter group is currently in use, a new DB parameter group in the DB parameter group family for the new engine version must be specified. The new DB parameter group can be the default for that DB parameter group family. For information about valid engine versions, see <code>CreateDBInstance</code>, or call <code>DescribeDBEngineVersions</code>.<br/>MultiAZ:  A value that indicates whether the DB instance is a Multi-AZ deployment. Changing this parameter doesn't result in an outage and the change is applied during the next maintenance window unless the <code>ApplyImmediately</code> parameter is enabled for this request. <br/>PreferredMaintenanceWindow:  The weekly time range (in UTC) during which system maintenance can occur, which might result in an outage. Changing this parameter doesn't result in an outage, except in the following situation, and the change is asynchronously applied as soon as possible. If there are pending actions that cause a reboot, and the maintenance window is changed to include the current time, then changing this parameter will cause a reboot of the DB instance. If moving this window to the current time, there must be at least 30 minutes between the current time and end of the window to ensure pending changes are applied. Default: Uses existing setting Format: ddd:hh24:mi-ddd:hh24:mi Valid Days: Mon | Tue | Wed | Thu | Fri | Sat | Sun Constraints: Must be at least 30 minutes<br/>PreferredBackupWindow:   The daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code> parameter. Changing this parameter doesn't result in an outage and the change is asynchronously applied as soon as possible.   <b>Amazon Aurora</b>  Not applicable. The daily time range for creating automated backups is managed by the DB cluster. For more information, see <code>ModifyDBCluster</code>. Constraints: <ul> <li> Must be in the format hh24:mi-hh24:mi </li> <li> Must be in Universal Time Coordinated (UTC) </li> <li> Must not conflict with the preferred maintenance window </li> <li> Must be at least 30 minutes </li> </ul><br/>BackupRetentionPeriod:  The number of days to retain automated backups. Setting this parameter to a positive number enables backups. Setting this parameter to 0 disables automated backups. Changing this parameter can result in an outage if you change from 0 to a non-zero value or from a non-zero value to 0. These changes are applied during the next maintenance window unless the <code>ApplyImmediately</code> parameter is enabled for this request. If you change the parameter from one non-zero value to another non-zero value, the change is asynchronously applied as soon as possible.  <b>Amazon Aurora</b>  Not applicable. The retention period for automated backups is managed by the DB cluster. For more information, see <code>ModifyDBCluster</code>. Default: Uses existing setting Constraints: <ul> <li> Must be a value from 0 to 35 </li> <li> Can be specified for a MySQL Read Replica only if the source is running MySQL 5.6 or later </li> <li> Can be specified for a PostgreSQL Read Replica only if the source is running PostgreSQL 9.3.5 </li> <li> Can't be set to 0 if the DB instance is a source to Read Replicas </li> </ul><br/>DBParameterGroupName:  The name of the DB parameter group to apply to the DB instance. Changing this setting doesn't result in an outage. The parameter group name itself is changed immediately, but the actual parameter changes are not applied until you reboot the instance without failover. In this case, the DB instance isn't rebooted automatically and the parameter changes isn't applied during the next maintenance window. Default: Uses existing setting Constraints: The DB parameter group must be in the same DB parameter group family as this DB instance.<br/>MasterUserPassword:  The new password for the master user. The password can include any printable ASCII character except "/", triple-quotes, or "@".  Changing this parameter doesn't result in an outage and the change is asynchronously applied as soon as possible. Between the time of the request and the completion of the request, the <code>MasterUserPassword</code> element exists in the <code>PendingModifiedValues</code> element of the operation response.   <b>Amazon Aurora</b>  Not applicable. The password for the master user is managed by the DB cluster. For more information, see <code>ModifyDBCluster</code>.  Default: Uses existing setting  <b>MariaDB</b>  Constraints: Must contain from 8 to 41 characters.  <b>Microsoft SQL Server</b>  Constraints: Must contain from 8 to 128 characters.  <b>MySQL</b>  Constraints: Must contain from 8 to 41 characters.  <b>Oracle</b>  Constraints: Must contain from 8 to 30 characters.  <b>PostgreSQL</b>  Constraints: Must contain from 8 to 128 characters. <note> Amazon RDS API actions never return the password, so this action provides a way to regain access to a primary instance user if the password is lost. This includes restoring privileges that might have been accidentally revoked.  </note><br/>ApplyImmediately:  A value that indicates whether the modifications in this request and any pending modifications are asynchronously applied as soon as possible, regardless of the <code>PreferredMaintenanceWindow</code> setting for the DB instance. By default, this parameter is disabled.   If this parameter is disabled, changes to the DB instance are applied during the next maintenance window. Some parameter changes can cause an outage and are applied on the next call to <a>RebootDBInstance</a>, or the next failure reboot. Review the table of parameters in <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/Overview.DBInstance.Modifying.html">Modifying a DB Instance</a> in the <i>Amazon RDS User Guide.</i> to see the impact of enabling or disabling <code>ApplyImmediately</code> for each modified parameter and to determine when the changes are applied. <br/>VpcSecurityGroupIds:  A list of EC2 VPC security groups to authorize on this DB instance. This change is asynchronously applied as soon as possible.  <b>Amazon Aurora</b>  Not applicable. The associated list of EC2 VPC security groups is managed by the DB cluster. For more information, see <code>ModifyDBCluster</code>. Constraints: <ul> <li> If supplied, must match existing VpcSecurityGroupIds. </li> </ul><br/>DBSecurityGroups:  A list of DB security groups to authorize on this DB instance. Changing this setting doesn't result in an outage and the change is asynchronously applied as soon as possible. Constraints: <ul> <li> If supplied, must match existing DBSecurityGroups. </li> </ul><br/>DBSubnetGroupName:  The new DB subnet group for the DB instance. You can use this parameter to move your DB instance to a different VPC. If your DB instance is not in a VPC, you can also use this parameter to move your DB instance into a VPC. For more information, see <a href="http://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_VPC.WorkingWithRDSInstanceinaVPC.html#USER_VPC.Non-VPC2VPC">Updating the VPC for a DB Instance</a> in the <i>Amazon RDS User Guide.</i>  Changing the subnet group causes an outage during the change. The change is applied during the next maintenance window, unless you enable <code>ApplyImmediately</code>.  Constraints: If supplied, must match the name of an existing DBSubnetGroup. Example: <code>mySubnetGroup</code> <br/>DBInstanceClass:  The new compute and memory capacity of the DB instance, for example, <code>db.m4.large</code>. Not all DB instance classes are available in all AWS Regions, or for all database engines. For the full list of DB instance classes, and availability for your engine, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/Concepts.DBInstanceClass.html">DB Instance Class</a> in the <i>Amazon RDS User Guide.</i>  If you modify the DB instance class, an outage occurs during the change. The change is applied during the next maintenance window, unless <code>ApplyImmediately</code> is enabled for this request.  Default: Uses existing setting<br/>AllocatedStorage:  The new amount of storage (in gibibytes) to allocate for the DB instance.  For MariaDB, MySQL, Oracle, and PostgreSQL, the value supplied must be at least 10% greater than the current value. Values that are not at least 10% greater than the existing value are rounded up so that they are 10% greater than the current value.  For the valid values for allocated storage for each engine, see <code>CreateDBInstance</code>. <br/>DBInstanceIdentifier:  The DB instance identifier. This value is stored as a lowercase string. Constraints: <ul> <li> Must match the identifier of an existing DBInstance. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisableLogTypes</i> The list of log types to disable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnableLogTypes</i> The list of log types to enable.<br/>
        :return: DBInstance:  Contains the details of an Amazon RDS DB instance.  This data type is used as a response element in the <code>DescribeDBInstances</code> action. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxAllocatedStorage</i> The upper limit to which Amazon RDS can automatically scale the storage of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ListenerEndpoint</i> Specifies the listener connection endpoint for SQL Server Always On.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ListenerEndpoint</b>: Specifies the listener connection endpoint for SQL Server Always On.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i>  The AWS Identity and Access Management (IAM) roles associated with the DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i> Indicates if the DB instance has deletion protection enabled. The database can't be deleted when deletion protection is enabled. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_DeleteInstance.html"> Deleting a DB Instance</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessorFeatures</i> The number of CPU cores and the number of threads per core for the DB instance class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB instance is configured to export to CloudWatch Logs. Log types vary by DB engine. For information about the log types for each DB engine, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_LogAccess.html">Amazon RDS Database Log Files</a> in the <i>Amazon RDS User Guide.</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsRetentionPeriod</i> The amount of time, in days, to retain Performance Insights data. Valid values are 7 or 731 (2 years). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsKMSKeyId</i> The AWS KMS key identifier for encryption of Performance Insights data. The KMS key ID is the Amazon Resource Name (ARN), KMS key identifier, or the KMS key alias for the KMS encryption key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsEnabled</i> True if Performance Insights is enabled for the DB instance, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled, and otherwise false. IAM database authentication can be enabled for the following database engines <ul> <li> For MySQL 5.6, minor version 5.6.34 or higher </li> <li> For MySQL 5.7, minor version 5.7.16 or higher </li> <li> Aurora 5.6 or higher. To enable IAM database authentication for Aurora, see DBCluster Type. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timezone</i> The time zone of the DB instance. In most cases, the <code>Timezone</code> element is empty. <code>Timezone</code> content appears only for Microsoft SQL Server DB instances that were created with a time zone specified. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceArn</i> The Amazon Resource Name (ARN) for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PromotionTier</i> A value that specifies the order in which an Aurora Replica is promoted to the primary instance after a failure of the existing primary instance. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/Aurora.Managing.Backups.html#Aurora.Managing.FaultTolerance"> Fault Tolerance for an Aurora DB Cluster</a> in the <i>Amazon Aurora User Guide</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringRoleArn</i> The ARN for the IAM role that permits RDS to send Enhanced Monitoring metrics to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedMonitoringResourceArn</i> The Amazon Resource Name (ARN) of the Amazon CloudWatch Logs log stream that receives the Enhanced Monitoring metrics data for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringInterval</i> The interval, in seconds, between points when Enhanced Monitoring metrics are collected for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyTagsToSnapshot</i> Specifies whether tags are copied from the DB instance to snapshots of the DB instance.  <b>Amazon Aurora</b>  Not applicable. Copying tags to snapshots is managed by the DB cluster. Setting this value for an Aurora DB instance has no effect on the DB cluster setting. For more information, see <code>DBCluster</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainMemberships</i> The Active Directory Domain membership records associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> The identifier of the CA certificate for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbiResourceId</i> The AWS Region-unique, immutable identifier for the DB instance. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB instance is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i>  If <code>StorageEncrypted</code> is true, the AWS KMS key identifier for the encrypted DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB instance is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> If the DB instance is a member of a DB cluster, contains the name of the DB cluster that the DB instance is a member of.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbInstancePort</i> Specifies the port that the DB instance listens on. If the DB instance is part of a DB cluster, this can be a different port than the DB cluster port.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TdeCredentialArn</i> The ARN from the key store with which the instance is associated for TDE encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type associated with DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusInfos</i> The status of a Read Replica. If the instance is not a Read Replica, this is blank.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> Specifies the accessibility options for the DB instance. A value of true specifies an Internet-facing instance with a publicly resolvable DNS name, which resolves to a public IP address. A value of false specifies an internal instance with a DNS name that resolves to a private IP address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondaryAvailabilityZone</i> If present, specifies the name of the secondary Availability Zone for a DB instance with multi-AZ support.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this instance is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionGroupMemberships</i> Provides the list of option group memberships for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the Provisioned IOPS (I/O operations per second) value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> License model information for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBClusterIdentifiers</i> Contains one or more identifiers of Aurora DB clusters to which the RDS DB instance is replicated as a Read Replica. For example, when you create an Aurora Read Replica of an RDS MySQL DB instance, the Aurora MySQL DB cluster for the Aurora Read Replica is shown. This output does not contain information about cross region Aurora Read Replicas. <note> Currently, each RDS DB instance can have only one Aurora Read Replica. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBInstanceIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaSourceDBInstanceIdentifier</i> Contains the identifier of the source DB instance if this DB instance is a Read Replica.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> Indicates that minor version patches are applied automatically.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies if the DB instance is a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DBSubnetGroup</b>: Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> Specifies the name of the Availability Zone the DB instance is located in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroups</i> Provides the list of DB parameter groups applied to this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security group elements that the DB instance belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSecurityGroups</i>  A list of DB security group elements containing <code>DBSecurityGroup.Name</code> and <code>DBSecurityGroup.Status</code> subelements. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i>  Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCreateTime</i> Provides the date and time the DB instance was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> Specifies the allocated storage size specified in gibibytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: Specifies the connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The meaning of this parameter differs according to the database engine you use.  <b>MySQL, MariaDB, SQL Server, PostgreSQL</b>  Contains the name of the initial database of this instance that was provided at create time, if one was specified when the DB instance was created. This same name is returned for the life of the DB instance. Type: String  <b>Oracle</b>  Contains the Oracle System ID (SID) of the created DB instance. Not shown when the returned parameters do not apply to an Oracle DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceStatus</i> Specifies the current state of this database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i> Contains the name of the compute and memory capacity class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i> Contains a user-supplied database identifier. This identifier is the unique key that identifies a DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupArn</i> The Amazon Resource Name (ARN) for the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i>  Contains a list of <code>Subnet</code> elements. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> Provides the status of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VpcId of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupDescription</i> Provides the description of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The name of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The new DB subnet group for the DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> Specifies the identifier of the CA certificate for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type to be associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i>  Contains the new <code>DBInstanceIdentifier</code> for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the new Provisioned IOPS value for the DB instance that will be applied or is currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> The license model for the DB instance. Valid values: <code>license-included</code> | <code>bring-your-own-license</code> | <code>general-public-license</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Indicates that the Single-AZ DB instance is to change to a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the pending number of days for which automated backups are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the pending port for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> Contains the pending or currently-in-progress change of the master credentials for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i>  Contains the new <code>AllocatedStorage</code> size for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i>  Contains the new <code>DBInstanceClass</code> for the DB instance that will be applied or is currently being applied. <br/>A list of the log types whose configuration is still pending. In other words, these log types are in the process of being activated or deactivated.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingCloudwatchLogsExports</b>: A list of the log types whose configuration is still pending. In other words, these log types are in the process of being activated or deactivated.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>        """
        pass

    def modify_db_parameter_group(self, Parameters: List[Parameter], DBParameterGroupName: str) -> DBParameterGroupNameMessage:
        """ Modifies the parameters of a DB parameter group. To modify more than one parameter, submit a list of the following: <code>ParameterName</code>, <code>ParameterValue</code>, and <code>ApplyMethod</code>. A maximum of 20 parameters can be modified in a single request.  <note> Changes to dynamic parameters are applied immediately. Changes to static parameters require a reboot without failover to the DB instance associated with the parameter group before the change can take effect. </note> <important> After you modify a DB parameter group, you should wait at least 5 minutes before creating your first DB instance that uses that DB parameter group as the default parameter group. This allows Amazon RDS to fully complete the modify action before the parameter group is used as the default for a new DB instance. This is especially important for parameters that are critical when creating the default database for a DB instance, such as the character set for the default database defined by the <code>character_set_database</code> parameter. You can use the <i>Parameter Groups</i> option of the <a href="https://console.aws.amazon.com/rds/">Amazon RDS console</a> or the <i>DescribeDBParameters</i> command to verify that your DB parameter group has been created or modified. </important>

        :param self:
        :param Parameters:  An array of parameter names, values, and the apply method for the parameter update. At least one parameter name, value, and apply method must be supplied; subsequent arguments are optional. A maximum of 20 parameters can be modified in a single request. Valid Values (for the application method): <code>immediate | pending-reboot</code>  <note> You can use the immediate value with dynamic parameters only. You can use the pending-reboot value for both dynamic and static parameters, and changes are applied when you reboot the DB instance without failover. </note><br/>DBParameterGroupName:  The name of the DB parameter group. Constraints: <ul> <li> If supplied, must match the name of an existing DBParameterGroup. </li> </ul><br/>
        :return: DBParameterGroupName:  Provides the name of the DB parameter group.<br/>        """
        pass

    def modify_db_snapshot(self, DBSnapshotIdentifier: str, EngineVersion: str=None, OptionGroupName: str=None) -> ModifyDBSnapshotResult:
        """Updates a manual DB snapshot, which can be encrypted or not encrypted, with a new engine version.  Amazon RDS supports upgrading DB snapshots for MySQL and Oracle. 

        :param self:
        :param OptionGroupName:  The option group to identify with the upgraded DB snapshot.  You can specify this parameter when you upgrade an Oracle DB snapshot. The same option group considerations apply when upgrading a DB snapshot as when upgrading a DB instance. For more information, see <a href="http://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_UpgradeDBInstance.Oracle.html#USER_UpgradeDBInstance.Oracle.OGPG.OG">Option Group Considerations</a> in the <i>Amazon RDS User Guide.</i> <br/>EngineVersion:  The engine version to upgrade the DB snapshot to.  The following are the database engines and engine versions that are available when you upgrade a DB snapshot.   <b>MySQL</b>  <ul> <li>  <code>5.5.46</code> (supported for 5.1 DB snapshots) </li> </ul>  <b>Oracle</b>  <ul> <li>  <code>12.1.0.2.v8</code> (supported for 12.1.0.1 DB snapshots) </li> <li>  <code>11.2.0.4.v12</code> (supported for 11.2.0.2 DB snapshots) </li> <li>  <code>11.2.0.4.v11</code> (supported for 11.2.0.3 DB snapshots) </li> </ul><br/>DBSnapshotIdentifier:  The identifier of the DB snapshot to modify.<br/>
        :return: DBSnapshot:  Contains the details of an Amazon RDS DB snapshot.  This data type is used as a response element in the <code>DescribeDBSnapshots</code> action. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbiResourceId</i> The identifier for the source DB instance, which can't be changed and which is unique to an AWS Region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessorFeatures</i> The number of CPU cores and the number of threads per core for the DB instance class of the DB instance when the DB snapshot was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timezone</i> The time zone of the DB snapshot. In most cases, the <code>Timezone</code> element is empty. <code>Timezone</code> content appears only for snapshots taken from Microsoft SQL Server DB instances that were created with a time zone specified. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSnapshotArn</i> The Amazon Resource Name (ARN) for the DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i>  If <code>Encrypted</code> is true, the AWS KMS key identifier for the encrypted DB snapshot. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encrypted</i> Specifies whether the DB snapshot is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TdeCredentialArn</i> The ARN from the key store with which to associate the instance for TDE encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type associated with DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceDBSnapshotIdentifier</i> The DB snapshot Amazon Resource Name (ARN) that the DB snapshot was copied from. It only has value in case of cross-customer or cross-region copy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceRegion</i> The AWS Region that the DB snapshot was created in or copied from.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> The percentage of the estimated data that has been transferred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionGroupName</i> Provides the option group name for the DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the Provisioned IOPS (I/O operations per second) value of the DB instance at the time of the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotType</i> Provides the type of the DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> License model information for the restored DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Specifies the version of the database engine.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Provides the master username for the DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCreateTime</i> Specifies the time when the snapshot was taken, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VPC ID associated with the DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> Specifies the name of the Availability Zone the DB instance was located in at the time of the DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine was listening on at the time of the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the status of this DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> Specifies the allocated storage size in gibibytes (GiB).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Specifies the name of the database engine.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCreateTime</i> Provides the time when the snapshot was taken, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i> Specifies the DB instance identifier of the DB instance this DB snapshot was created from.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSnapshotIdentifier</i> Specifies the identifier for the DB snapshot.<br/>        """
        pass

    def modify_db_snapshot_attribute(self, AttributeName: str, DBSnapshotIdentifier: str, ValuesToAdd: List[String]=None, ValuesToRemove: List[String]=None) -> ModifyDBSnapshotAttributeResult:
        """Adds an attribute and values to, or removes an attribute and values from, a manual DB snapshot. To share a manual DB snapshot with other AWS accounts, specify <code>restore</code> as the <code>AttributeName</code> and use the <code>ValuesToAdd</code> parameter to add a list of IDs of the AWS accounts that are authorized to restore the manual DB snapshot. Uses the value <code>all</code> to make the manual DB snapshot public, which means it can be copied or restored by all AWS accounts. Do not add the <code>all</code> value for any manual DB snapshots that contain private information that you don't want available to all AWS accounts. If the manual DB snapshot is encrypted, it can be shared, but only by specifying a list of authorized AWS account IDs for the <code>ValuesToAdd</code> parameter. You can't use <code>all</code> as a value for that parameter in this case. To view which AWS accounts have access to copy or restore a manual DB snapshot, or whether a manual DB snapshot public or private, use the <code>DescribeDBSnapshotAttributes</code> API action.

        :param self:
        :param ValuesToRemove:  A list of DB snapshot attributes to remove from the attribute specified by <code>AttributeName</code>. To remove authorization for other AWS accounts to copy or restore a manual snapshot, set this list to include one or more AWS account identifiers, or <code>all</code> to remove authorization for any AWS account to copy or restore the DB snapshot. If you specify <code>all</code>, an AWS account whose account ID is explicitly added to the <code>restore</code> attribute can still copy or restore the manual DB snapshot.<br/>ValuesToAdd:  A list of DB snapshot attributes to add to the attribute specified by <code>AttributeName</code>. To authorize other AWS accounts to copy or restore a manual snapshot, set this list to include one or more AWS account IDs, or <code>all</code> to make the manual DB snapshot restorable by any AWS account. Do not add the <code>all</code> value for any manual DB snapshots that contain private information that you don't want available to all AWS accounts.<br/>AttributeName:  The name of the DB snapshot attribute to modify. To manage authorization for other AWS accounts to copy or restore a manual DB snapshot, set this value to <code>restore</code>.<br/>DBSnapshotIdentifier:  The identifier for the DB snapshot to modify the attributes for.<br/>
        :return: DBSnapshotAttributesResult:  Contains the results of a successful call to the <code>DescribeDBSnapshotAttributes</code> API action. Manual DB snapshot attributes are used to authorize other AWS accounts to copy or restore a manual DB snapshot. For more information, see the <code>ModifyDBSnapshotAttribute</code> API action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSnapshotAttributes</i> The list of attributes and values for the manual DB snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSnapshotIdentifier</i> The identifier of the manual DB snapshot that the attributes apply to.<br/>        """
        pass

    def modify_db_subnet_group(self, SubnetIds: List[String], DBSubnetGroupName: str, DBSubnetGroupDescription: str=None) -> ModifyDBSubnetGroupResult:
        """Modifies an existing DB subnet group. DB subnet groups must contain at least one subnet in at least two AZs in the AWS Region.

        :param self:
        :param SubnetIds:  The EC2 subnet IDs for the DB subnet group.<br/>DBSubnetGroupDescription:  The description for the DB subnet group.<br/>DBSubnetGroupName:  The name for the DB subnet group. This value is stored as a lowercase string. You can't modify the default subnet group.  Constraints: Must match the name of an existing DBSubnetGroup. Must not be default. Example: <code>mySubnetgroup</code> <br/>
        :return: DBSubnetGroup:  Contains the details of an Amazon RDS DB subnet group.  This data type is used as a response element in the <code>DescribeDBSubnetGroups</code> action. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupArn</i> The Amazon Resource Name (ARN) for the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i>  Contains a list of <code>Subnet</code> elements. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> Provides the status of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VpcId of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupDescription</i> Provides the description of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The name of the DB subnet group.<br/>        """
        pass

    def modify_event_subscription(self, SubscriptionName: str, SnsTopicArn: str=None, SourceType: str=None, EventCategories: List[String]=None, Enabled: BooleanOptional=None) -> ModifyEventSubscriptionResult:
        """Modifies an existing RDS event notification subscription. Note that you can't modify the source identifiers using this call; to change source identifiers for a subscription, use the <code>AddSourceIdentifierToSubscription</code> and <code>RemoveSourceIdentifierFromSubscription</code> calls. You can see a list of the event categories for a given SourceType in the <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Events.html">Events</a> topic in the <i>Amazon RDS User Guide</i> or by using the <b>DescribeEventCategories</b> action.

        :param self:
        :param Enabled:   A value that indicates whether to activate the subscription. <br/>EventCategories:   A list of event categories for a SourceType that you want to subscribe to. You can see a list of the categories for a given SourceType in the <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Events.html">Events</a> topic in the <i>Amazon RDS User Guide</i> or by using the <b>DescribeEventCategories</b> action. <br/>SourceType:  The type of source that is generating the events. For example, if you want to be notified of events generated by a DB instance, you would set this parameter to db-instance. if this value is not specified, all events are returned. Valid values: db-instance | db-parameter-group | db-security-group | db-snapshot<br/>SnsTopicArn:  The Amazon Resource Name (ARN) of the SNS topic created for event notification. The ARN is created by Amazon SNS when you create a topic and subscribe to it.<br/>SubscriptionName:  The name of the RDS event notification subscription.<br/>
        :return: EventSubscription:  Contains the results of a successful invocation of the <code>DescribeEventSubscriptions</code> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EventSubscriptionArn</i> The Amazon Resource Name (ARN) for the event subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> A Boolean value indicating if the subscription is enabled. True indicates the subscription is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EventCategoriesList</i> A list of event categories for the RDS event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceIdsList</i> A list of source IDs for the RDS event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceType</i> The source type for the RDS event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubscriptionCreationTime</i> The time the RDS event notification subscription was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the RDS event notification subscription. Constraints: Can be one of the following: creating | modifying | deleting | active | no-permission | topic-not-exist The status "no-permission" indicates that RDS no longer has permission to post to the SNS topic. The status "topic-not-exist" indicates that the topic was deleted after the subscription was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsTopicArn</i> The topic ARN of the RDS event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustSubscriptionId</i> The RDS event notification subscription Id.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomerAwsId</i> The AWS customer account associated with the RDS event notification subscription.<br/>        """
        pass

    def modify_global_cluster(self, GlobalClusterIdentifier: str=None, NewGlobalClusterIdentifier: str=None, DeletionProtection: BooleanOptional=None) -> ModifyGlobalClusterResult:
        """ Modify a setting for an Amazon Aurora global cluster. You can change one or more database configuration parameters by specifying these parameters and the new values in the request. For more information on Amazon Aurora, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/CHAP_AuroraOverview.html"> What Is Amazon Aurora?</a> in the <i>Amazon Aurora User Guide.</i>  <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param DeletionProtection:   Indicates if the global database cluster has deletion protection enabled. The global database cluster can't be deleted when deletion protection is enabled. <br/>NewGlobalClusterIdentifier:   The new cluster identifier for the global database cluster when modifying a global database cluster. This value is stored as a lowercase string.  Constraints: <ul> <li> Must contain from 1 to 63 letters, numbers, or hyphens </li> <li> The first character must be a letter </li> <li> Can't end with a hyphen or contain two consecutive hyphens </li> </ul> Example: <code>my-cluster2</code> <br/>GlobalClusterIdentifier:   The DB cluster identifier for the global cluster being modified. This parameter is not case-sensitive.  Constraints: <ul> <li> Must match the identifier of an existing global database cluster. </li> </ul><br/>
        :return: GlobalCluster:  A data type representing an Aurora global database.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalClusterMembers</i>  The list of cluster IDs for secondary clusters within the global database cluster. Currently limited to 1 item. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i>  The deletion protection setting for the new global database cluster. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i>  The storage encryption setting for the global database cluster. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i>  The default database name within the new global database cluster. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i>  The Aurora database engine used by the global database cluster. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the current state of this global database cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalClusterArn</i> The Amazon Resource Name (ARN) for the global database cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalClusterResourceId</i>  The AWS Region-unique, immutable identifier for the global database cluster. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB cluster is accessed. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalClusterIdentifier</i>  Contains a user-supplied global database cluster identifier. This identifier is the unique key that identifies a global database cluster. <br/>        """
        pass

    def modify_option_group(self, OptionGroupName: str, OptionsToInclude: List[OptionConfiguration]=None, OptionsToRemove: List[String]=None, ApplyImmediately: Boolean=None) -> ModifyOptionGroupResult:
        """Modifies an existing option group.

        :param self:
        :param ApplyImmediately:  A value that indicates whether to apply the change immediately or during the next maintenance window for each instance associated with the option group.<br/>OptionsToRemove:  Options in this list are removed from the option group.<br/>OptionsToInclude:  Options in this list are added to the option group or, if already present, the specified configuration is used to update the existing configuration.<br/>OptionGroupName:  The name of the option group to be modified. Permanent options, such as the TDE option for Oracle Advanced Security TDE, can't be removed from an option group, and that option group can't be removed from a DB instance once it is associated with a DB instance<br/>
        :return: OptionGroup:  <p/>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionGroupArn</i> The Amazon Resource Name (ARN) for the option group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> If <b>AllowsVpcAndNonVpcInstanceMemberships</b> is <code>false</code>, this field is blank. If <b>AllowsVpcAndNonVpcInstanceMemberships</b> is <code>true</code> and this field is blank, then this option group can be applied to both VPC and non-VPC instances. If this field contains a value, then this option group can only be applied to instances that are in the VPC indicated by this field. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowsVpcAndNonVpcInstanceMemberships</i> Indicates whether this option group can be applied to both VPC and non-VPC instances. The value <code>true</code> indicates the option group can be applied to both VPC and non-VPC instances. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Options</i> Indicates what options are available in the option group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MajorEngineVersion</i> Indicates the major engine version associated with this option group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineName</i> Indicates the name of the engine that this option group can be applied to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionGroupDescription</i> Provides a description of the option group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionGroupName</i> Specifies the name of the option group.<br/>        """
        pass

    def promote_read_replica(self, DBInstanceIdentifier: str, BackupRetentionPeriod: int=None, PreferredBackupWindow: str=None) -> PromoteReadReplicaResult:
        """Promotes a Read Replica DB instance to a standalone DB instance. <note> <ul> <li> Backup duration is a function of the amount of changes to the database since the previous backup. If you plan to promote a Read Replica to a standalone instance, we recommend that you enable backups and complete at least one backup prior to promotion. In addition, a Read Replica cannot be promoted to a standalone instance when it is in the <code>backing-up</code> status. If you have enabled backups on your Read Replica, configure the automated backup window so that daily backups do not interfere with Read Replica promotion. </li> <li> This command doesn't apply to Aurora MySQL and Aurora PostgreSQL. </li> </ul> </note>

        :param self:
        :param PreferredBackupWindow:   The daily time range during which automated backups are created if automated backups are enabled, using the <code>BackupRetentionPeriod</code> parameter.   The default is a 30-minute window selected at random from an 8-hour block of time for each AWS Region. To see the time blocks available, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/AdjustingTheMaintenanceWindow.html"> Adjusting the Preferred Maintenance Window</a> in the <i>Amazon RDS User Guide.</i>  Constraints: <ul> <li> Must be in the format <code>hh24:mi-hh24:mi</code>. </li> <li> Must be in Universal Coordinated Time (UTC). </li> <li> Must not conflict with the preferred maintenance window. </li> <li> Must be at least 30 minutes. </li> </ul><br/>BackupRetentionPeriod:  The number of days to retain automated backups. Setting this parameter to a positive number enables backups. Setting this parameter to 0 disables automated backups. Default: 1 Constraints: <ul> <li> Must be a value from 0 to 8 </li> </ul><br/>DBInstanceIdentifier:  The DB instance identifier. This value is stored as a lowercase string. Constraints: <ul> <li> Must match the identifier of an existing Read Replica DB instance. </li> </ul> Example: <code>mydbinstance</code> <br/>
        :return: DBInstance:  Contains the details of an Amazon RDS DB instance.  This data type is used as a response element in the <code>DescribeDBInstances</code> action. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxAllocatedStorage</i> The upper limit to which Amazon RDS can automatically scale the storage of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ListenerEndpoint</i> Specifies the listener connection endpoint for SQL Server Always On.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ListenerEndpoint</b>: Specifies the listener connection endpoint for SQL Server Always On.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i>  The AWS Identity and Access Management (IAM) roles associated with the DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i> Indicates if the DB instance has deletion protection enabled. The database can't be deleted when deletion protection is enabled. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_DeleteInstance.html"> Deleting a DB Instance</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessorFeatures</i> The number of CPU cores and the number of threads per core for the DB instance class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB instance is configured to export to CloudWatch Logs. Log types vary by DB engine. For information about the log types for each DB engine, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_LogAccess.html">Amazon RDS Database Log Files</a> in the <i>Amazon RDS User Guide.</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsRetentionPeriod</i> The amount of time, in days, to retain Performance Insights data. Valid values are 7 or 731 (2 years). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsKMSKeyId</i> The AWS KMS key identifier for encryption of Performance Insights data. The KMS key ID is the Amazon Resource Name (ARN), KMS key identifier, or the KMS key alias for the KMS encryption key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsEnabled</i> True if Performance Insights is enabled for the DB instance, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled, and otherwise false. IAM database authentication can be enabled for the following database engines <ul> <li> For MySQL 5.6, minor version 5.6.34 or higher </li> <li> For MySQL 5.7, minor version 5.7.16 or higher </li> <li> Aurora 5.6 or higher. To enable IAM database authentication for Aurora, see DBCluster Type. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timezone</i> The time zone of the DB instance. In most cases, the <code>Timezone</code> element is empty. <code>Timezone</code> content appears only for Microsoft SQL Server DB instances that were created with a time zone specified. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceArn</i> The Amazon Resource Name (ARN) for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PromotionTier</i> A value that specifies the order in which an Aurora Replica is promoted to the primary instance after a failure of the existing primary instance. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/Aurora.Managing.Backups.html#Aurora.Managing.FaultTolerance"> Fault Tolerance for an Aurora DB Cluster</a> in the <i>Amazon Aurora User Guide</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringRoleArn</i> The ARN for the IAM role that permits RDS to send Enhanced Monitoring metrics to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedMonitoringResourceArn</i> The Amazon Resource Name (ARN) of the Amazon CloudWatch Logs log stream that receives the Enhanced Monitoring metrics data for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringInterval</i> The interval, in seconds, between points when Enhanced Monitoring metrics are collected for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyTagsToSnapshot</i> Specifies whether tags are copied from the DB instance to snapshots of the DB instance.  <b>Amazon Aurora</b>  Not applicable. Copying tags to snapshots is managed by the DB cluster. Setting this value for an Aurora DB instance has no effect on the DB cluster setting. For more information, see <code>DBCluster</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainMemberships</i> The Active Directory Domain membership records associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> The identifier of the CA certificate for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbiResourceId</i> The AWS Region-unique, immutable identifier for the DB instance. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB instance is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i>  If <code>StorageEncrypted</code> is true, the AWS KMS key identifier for the encrypted DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB instance is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> If the DB instance is a member of a DB cluster, contains the name of the DB cluster that the DB instance is a member of.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbInstancePort</i> Specifies the port that the DB instance listens on. If the DB instance is part of a DB cluster, this can be a different port than the DB cluster port.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TdeCredentialArn</i> The ARN from the key store with which the instance is associated for TDE encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type associated with DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusInfos</i> The status of a Read Replica. If the instance is not a Read Replica, this is blank.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> Specifies the accessibility options for the DB instance. A value of true specifies an Internet-facing instance with a publicly resolvable DNS name, which resolves to a public IP address. A value of false specifies an internal instance with a DNS name that resolves to a private IP address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondaryAvailabilityZone</i> If present, specifies the name of the secondary Availability Zone for a DB instance with multi-AZ support.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this instance is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionGroupMemberships</i> Provides the list of option group memberships for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the Provisioned IOPS (I/O operations per second) value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> License model information for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBClusterIdentifiers</i> Contains one or more identifiers of Aurora DB clusters to which the RDS DB instance is replicated as a Read Replica. For example, when you create an Aurora Read Replica of an RDS MySQL DB instance, the Aurora MySQL DB cluster for the Aurora Read Replica is shown. This output does not contain information about cross region Aurora Read Replicas. <note> Currently, each RDS DB instance can have only one Aurora Read Replica. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBInstanceIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaSourceDBInstanceIdentifier</i> Contains the identifier of the source DB instance if this DB instance is a Read Replica.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> Indicates that minor version patches are applied automatically.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies if the DB instance is a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DBSubnetGroup</b>: Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> Specifies the name of the Availability Zone the DB instance is located in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroups</i> Provides the list of DB parameter groups applied to this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security group elements that the DB instance belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSecurityGroups</i>  A list of DB security group elements containing <code>DBSecurityGroup.Name</code> and <code>DBSecurityGroup.Status</code> subelements. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i>  Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCreateTime</i> Provides the date and time the DB instance was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> Specifies the allocated storage size specified in gibibytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: Specifies the connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The meaning of this parameter differs according to the database engine you use.  <b>MySQL, MariaDB, SQL Server, PostgreSQL</b>  Contains the name of the initial database of this instance that was provided at create time, if one was specified when the DB instance was created. This same name is returned for the life of the DB instance. Type: String  <b>Oracle</b>  Contains the Oracle System ID (SID) of the created DB instance. Not shown when the returned parameters do not apply to an Oracle DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceStatus</i> Specifies the current state of this database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i> Contains the name of the compute and memory capacity class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i> Contains a user-supplied database identifier. This identifier is the unique key that identifies a DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupArn</i> The Amazon Resource Name (ARN) for the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i>  Contains a list of <code>Subnet</code> elements. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> Provides the status of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VpcId of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupDescription</i> Provides the description of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The name of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The new DB subnet group for the DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> Specifies the identifier of the CA certificate for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type to be associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i>  Contains the new <code>DBInstanceIdentifier</code> for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the new Provisioned IOPS value for the DB instance that will be applied or is currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> The license model for the DB instance. Valid values: <code>license-included</code> | <code>bring-your-own-license</code> | <code>general-public-license</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Indicates that the Single-AZ DB instance is to change to a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the pending number of days for which automated backups are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the pending port for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> Contains the pending or currently-in-progress change of the master credentials for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i>  Contains the new <code>AllocatedStorage</code> size for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i>  Contains the new <code>DBInstanceClass</code> for the DB instance that will be applied or is currently being applied. <br/>A list of the log types whose configuration is still pending. In other words, these log types are in the process of being activated or deactivated.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingCloudwatchLogsExports</b>: A list of the log types whose configuration is still pending. In other words, these log types are in the process of being activated or deactivated.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>        """
        pass

    def promote_read_replica_db_cluster(self, DBClusterIdentifier: str) -> PromoteReadReplicaDBClusterResult:
        """Promotes a Read Replica DB cluster to a standalone DB cluster. <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param DBClusterIdentifier:  The identifier of the DB cluster Read Replica to promote. This parameter is not case-sensitive.  Constraints: <ul> <li> Must match the identifier of an existing DBCluster Read Replica. </li> </ul> Example: <code>my-cluster-replica1</code> <br/>
        :return: DBCluster:  Contains the details of an Amazon Aurora DB cluster.  This data type is used as a response element in the <code>DescribeDBClusters</code>, <code>StopDBCluster</code>, and <code>StartDBCluster</code> actions. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CrossAccountClone</i> Specifies whether the DB cluster is a clone of a DB cluster owned by a different AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyTagsToSnapshot</i> Specifies whether tags are copied from the DB cluster to snapshots of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamKinesisStreamName</i> The name of the Amazon Kinesis data stream used for the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamKmsKeyId</i> The AWS KMS key identifier used for encrypting messages in the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamStatus</i> The status of the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamMode</i> The mode of the database activity stream. Database events such as a change or access generate an activity stream event. The database session can handle these events either synchronously or asynchronously. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HttpEndpointEnabled</i> A value that indicates whether the HTTP endpoint for an Aurora Serverless DB cluster is enabled. When enabled, the HTTP endpoint provides a connectionless web service API for running SQL queries on the Aurora Serverless DB cluster. You can also query your database from inside the RDS console with the query editor. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/data-api.html">Using the Data API for Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i> Indicates if the DB cluster has deletion protection enabled. The database can't be deleted when deletion protection is enabled. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ScalingConfigurationInfo</i> Shows the scaling configuration for an Aurora DB cluster in <code>serverless</code> DB engine mode. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ScalingConfigurationInfo</b>: Shows the scaling configuration for an Aurora DB cluster in <code>serverless</code> DB engine mode. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineMode</i> The DB engine mode of the DB cluster, either <code>provisioned</code>, <code>serverless</code>, or <code>parallelquery</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Capacity</i> The current capacity of an Aurora Serverless DB cluster. The capacity is 0 (zero) when the cluster is paused. For more information about Aurora Serverless, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB cluster is configured to export to CloudWatch Logs. Log types vary by DB engine. For information about the log types for each DB engine, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/USER_LogAccess.html">Amazon RDS Database Log Files</a> in the <i>Amazon Aurora User Guide.</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BacktrackConsumedChangeRecords</i> The number of change records stored for Backtrack.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BacktrackWindow</i> The target backtrack window, in seconds. If this value is set to 0, backtracking is disabled for the DB cluster. Otherwise, backtracking is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestBacktrackTime</i> The earliest time to which a DB cluster can be backtracked.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloneGroupId</i> Identifies the clone group to which the DB cluster is associated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> A value that indicates whether the mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i> Provides a list of the AWS Identity and Access Management (IAM) roles that are associated with the DB cluster. IAM roles that are associated with a DB cluster grant permission for the DB cluster to access other AWS services on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterArn</i> The Amazon Resource Name (ARN) for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbClusterResourceId</i> The AWS Region-unique, immutable identifier for the DB cluster. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB cluster is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is enabled, the AWS KMS key identifier for the encrypted DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security groups that the DB cluster belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterMembers</i> Provides the list of instances that make up the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSourceIdentifier</i> Contains the identifier of the source DB cluster if this DB cluster is a Read Replica.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i> Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterOptionGroupMemberships</i> Provides the list of option group memberships for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies whether the DB cluster has instances in multiple Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomEndpoints</i> Identifies all custom endpoints associated with the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReaderEndpoint</i> The reader endpoint for the DB cluster. The reader endpoint for a DB cluster load-balances connections across the Aurora Replicas that are available in a DB cluster. As clients request new connections to the reader endpoint, Aurora distributes the connection requests among the Aurora Replicas in the DB cluster. This functionality can help balance your read workload across multiple Aurora Replicas in your DB cluster.  If a failover occurs, and the Aurora Replica that you are connected to is promoted to be the primary instance, your connection is dropped. To continue sending your read workload to other Aurora Replicas in the cluster, you can then reconnect to the reader endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint for the primary instance of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestRestorableTime</i> The earliest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the progress of the operation as a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the current state of this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB cluster, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroup</i> Specifies the name of the DB cluster parameter group for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Contains a user-supplied DB cluster identifier. This identifier is the unique key that identifies a DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> Contains the name of the initial database of this DB cluster that was provided at create time, if one was specified when the DB cluster was created. This same name is returned for the life of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this cluster is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of Availability Zones (AZs) where instances in the DB cluster can be created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> For all database engines except Amazon Aurora, <code>AllocatedStorage</code> specifies the allocated storage size in gibibytes (GiB). For Aurora, <code>AllocatedStorage</code> always returns 1, because Aurora DB cluster storage size is not fixed, but instead automatically adjusts as needed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeoutAction</i> The timeout action of a call to <code>ModifyCurrentDBClusterCapacity</code>, either <code>ForceApplyCapacityChange</code> or <code>RollbackCapacityChange</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondsUntilAutoPause</i> The remaining amount of time, in seconds, before the Aurora DB cluster in <code>serverless</code> mode is paused. A DB cluster can be paused only when it's idle (it has no connections).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoPause</i> A value that indicates whether automatic pause is allowed for the Aurora DB cluster in <code>serverless</code> DB engine mode. When the value is set to false for an Aurora Serverless DB cluster, the DB cluster automatically resumes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxCapacity</i> The maximum capacity for an Aurora DB cluster in <code>serverless</code> DB engine mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinCapacity</i> The maximum capacity for the Aurora DB cluster in <code>serverless</code> DB engine mode.<br/>        """
        pass

    def purchase_reserved_db_instances_offering(self, ReservedDBInstancesOfferingId: str, ReservedDBInstanceId: str=None, DBInstanceCount: int=None, Tags: List[Tag]=None) -> PurchaseReservedDBInstancesOfferingResult:
        """Purchases a reserved DB instance offering.

        :param self:
        :param Tags:  A list of tags. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Tagging.html">Tagging Amazon RDS Resources</a> in the <i>Amazon RDS User Guide.</i> <br/>DBInstanceCount:  The number of instances to reserve. Default: <code>1</code> <br/>ReservedDBInstanceId:  Customer-specified identifier to track this reservation. Example: myreservationID<br/>ReservedDBInstancesOfferingId:  The ID of the Reserved DB instance offering to purchase. Example: 438012d3-4052-4cc7-b2e3-8d3372e0e706<br/>
        :return: ReservedDBInstance:   This data type is used as a response element in the <code>DescribeReservedDBInstances</code> and <code>PurchaseReservedDBInstancesOffering</code> actions. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservedDBInstanceArn</i> The Amazon Resource Name (ARN) for the reserved DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecurringCharges</i> The recurring price charged to run this reserved DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the reserved DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Indicates if the reservation applies to Multi-AZ deployments.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OfferingType</i> The offering type of this reserved DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductDescription</i> The description of the reserved DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceCount</i> The number of reserved DB instances.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrencyCode</i> The currency code for the reserved DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UsagePrice</i> The hourly price charged for this reserved DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FixedPrice</i> The fixed price charged for this reserved DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Duration</i> The duration of the reservation in seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTime</i> The time the reservation started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i> The DB instance class for the reserved DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservedDBInstancesOfferingId</i> The offering identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservedDBInstanceId</i> The unique identifier for the reservation.<br/>        """
        pass

    def reboot_db_instance(self, DBInstanceIdentifier: str, ForceFailover: BooleanOptional=None) -> RebootDBInstanceResult:
        """You might need to reboot your DB instance, usually for maintenance reasons. For example, if you make certain modifications, or if you change the DB parameter group associated with the DB instance, you must reboot the instance for the changes to take effect.  Rebooting a DB instance restarts the database engine service. Rebooting a DB instance results in a momentary outage, during which the DB instance status is set to rebooting.  For more information about rebooting, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_RebootInstance.html">Rebooting a DB Instance</a> in the <i>Amazon RDS User Guide.</i> 

        :param self:
        :param ForceFailover:   A value that indicates whether the reboot is conducted through a Multi-AZ failover.  Constraint: You can't enable force failover if the instance is not configured for Multi-AZ.<br/>DBInstanceIdentifier:  The DB instance identifier. This parameter is stored as a lowercase string. Constraints: <ul> <li> Must match the identifier of an existing DBInstance. </li> </ul><br/>
        :return: DBInstance:  Contains the details of an Amazon RDS DB instance.  This data type is used as a response element in the <code>DescribeDBInstances</code> action. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxAllocatedStorage</i> The upper limit to which Amazon RDS can automatically scale the storage of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ListenerEndpoint</i> Specifies the listener connection endpoint for SQL Server Always On.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ListenerEndpoint</b>: Specifies the listener connection endpoint for SQL Server Always On.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i>  The AWS Identity and Access Management (IAM) roles associated with the DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i> Indicates if the DB instance has deletion protection enabled. The database can't be deleted when deletion protection is enabled. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_DeleteInstance.html"> Deleting a DB Instance</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessorFeatures</i> The number of CPU cores and the number of threads per core for the DB instance class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB instance is configured to export to CloudWatch Logs. Log types vary by DB engine. For information about the log types for each DB engine, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_LogAccess.html">Amazon RDS Database Log Files</a> in the <i>Amazon RDS User Guide.</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsRetentionPeriod</i> The amount of time, in days, to retain Performance Insights data. Valid values are 7 or 731 (2 years). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsKMSKeyId</i> The AWS KMS key identifier for encryption of Performance Insights data. The KMS key ID is the Amazon Resource Name (ARN), KMS key identifier, or the KMS key alias for the KMS encryption key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsEnabled</i> True if Performance Insights is enabled for the DB instance, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled, and otherwise false. IAM database authentication can be enabled for the following database engines <ul> <li> For MySQL 5.6, minor version 5.6.34 or higher </li> <li> For MySQL 5.7, minor version 5.7.16 or higher </li> <li> Aurora 5.6 or higher. To enable IAM database authentication for Aurora, see DBCluster Type. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timezone</i> The time zone of the DB instance. In most cases, the <code>Timezone</code> element is empty. <code>Timezone</code> content appears only for Microsoft SQL Server DB instances that were created with a time zone specified. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceArn</i> The Amazon Resource Name (ARN) for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PromotionTier</i> A value that specifies the order in which an Aurora Replica is promoted to the primary instance after a failure of the existing primary instance. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/Aurora.Managing.Backups.html#Aurora.Managing.FaultTolerance"> Fault Tolerance for an Aurora DB Cluster</a> in the <i>Amazon Aurora User Guide</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringRoleArn</i> The ARN for the IAM role that permits RDS to send Enhanced Monitoring metrics to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedMonitoringResourceArn</i> The Amazon Resource Name (ARN) of the Amazon CloudWatch Logs log stream that receives the Enhanced Monitoring metrics data for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringInterval</i> The interval, in seconds, between points when Enhanced Monitoring metrics are collected for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyTagsToSnapshot</i> Specifies whether tags are copied from the DB instance to snapshots of the DB instance.  <b>Amazon Aurora</b>  Not applicable. Copying tags to snapshots is managed by the DB cluster. Setting this value for an Aurora DB instance has no effect on the DB cluster setting. For more information, see <code>DBCluster</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainMemberships</i> The Active Directory Domain membership records associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> The identifier of the CA certificate for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbiResourceId</i> The AWS Region-unique, immutable identifier for the DB instance. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB instance is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i>  If <code>StorageEncrypted</code> is true, the AWS KMS key identifier for the encrypted DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB instance is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> If the DB instance is a member of a DB cluster, contains the name of the DB cluster that the DB instance is a member of.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbInstancePort</i> Specifies the port that the DB instance listens on. If the DB instance is part of a DB cluster, this can be a different port than the DB cluster port.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TdeCredentialArn</i> The ARN from the key store with which the instance is associated for TDE encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type associated with DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusInfos</i> The status of a Read Replica. If the instance is not a Read Replica, this is blank.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> Specifies the accessibility options for the DB instance. A value of true specifies an Internet-facing instance with a publicly resolvable DNS name, which resolves to a public IP address. A value of false specifies an internal instance with a DNS name that resolves to a private IP address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondaryAvailabilityZone</i> If present, specifies the name of the secondary Availability Zone for a DB instance with multi-AZ support.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this instance is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionGroupMemberships</i> Provides the list of option group memberships for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the Provisioned IOPS (I/O operations per second) value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> License model information for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBClusterIdentifiers</i> Contains one or more identifiers of Aurora DB clusters to which the RDS DB instance is replicated as a Read Replica. For example, when you create an Aurora Read Replica of an RDS MySQL DB instance, the Aurora MySQL DB cluster for the Aurora Read Replica is shown. This output does not contain information about cross region Aurora Read Replicas. <note> Currently, each RDS DB instance can have only one Aurora Read Replica. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBInstanceIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaSourceDBInstanceIdentifier</i> Contains the identifier of the source DB instance if this DB instance is a Read Replica.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> Indicates that minor version patches are applied automatically.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies if the DB instance is a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DBSubnetGroup</b>: Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> Specifies the name of the Availability Zone the DB instance is located in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroups</i> Provides the list of DB parameter groups applied to this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security group elements that the DB instance belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSecurityGroups</i>  A list of DB security group elements containing <code>DBSecurityGroup.Name</code> and <code>DBSecurityGroup.Status</code> subelements. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i>  Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCreateTime</i> Provides the date and time the DB instance was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> Specifies the allocated storage size specified in gibibytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: Specifies the connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The meaning of this parameter differs according to the database engine you use.  <b>MySQL, MariaDB, SQL Server, PostgreSQL</b>  Contains the name of the initial database of this instance that was provided at create time, if one was specified when the DB instance was created. This same name is returned for the life of the DB instance. Type: String  <b>Oracle</b>  Contains the Oracle System ID (SID) of the created DB instance. Not shown when the returned parameters do not apply to an Oracle DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceStatus</i> Specifies the current state of this database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i> Contains the name of the compute and memory capacity class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i> Contains a user-supplied database identifier. This identifier is the unique key that identifies a DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupArn</i> The Amazon Resource Name (ARN) for the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i>  Contains a list of <code>Subnet</code> elements. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> Provides the status of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VpcId of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupDescription</i> Provides the description of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The name of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The new DB subnet group for the DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> Specifies the identifier of the CA certificate for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type to be associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i>  Contains the new <code>DBInstanceIdentifier</code> for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the new Provisioned IOPS value for the DB instance that will be applied or is currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> The license model for the DB instance. Valid values: <code>license-included</code> | <code>bring-your-own-license</code> | <code>general-public-license</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Indicates that the Single-AZ DB instance is to change to a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the pending number of days for which automated backups are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the pending port for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> Contains the pending or currently-in-progress change of the master credentials for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i>  Contains the new <code>AllocatedStorage</code> size for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i>  Contains the new <code>DBInstanceClass</code> for the DB instance that will be applied or is currently being applied. <br/>A list of the log types whose configuration is still pending. In other words, these log types are in the process of being activated or deactivated.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingCloudwatchLogsExports</b>: A list of the log types whose configuration is still pending. In other words, these log types are in the process of being activated or deactivated.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>        """
        pass

    def remove_from_global_cluster(self, GlobalClusterIdentifier: str=None, DbClusterIdentifier: str=None) -> RemoveFromGlobalClusterResult:
        """ Detaches an Aurora secondary cluster from an Aurora global database cluster. The cluster becomes a standalone cluster with read-write capability instead of being read-only and receiving data from a primary cluster in a different region.  <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param DbClusterIdentifier:   The Amazon Resource Name (ARN) identifying the cluster that was detached from the Aurora global database cluster. <br/>GlobalClusterIdentifier:   The cluster identifier to detach from the Aurora global database cluster. <br/>
        :return: GlobalCluster:  A data type representing an Aurora global database.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalClusterMembers</i>  The list of cluster IDs for secondary clusters within the global database cluster. Currently limited to 1 item. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i>  The deletion protection setting for the new global database cluster. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i>  The storage encryption setting for the global database cluster. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i>  The default database name within the new global database cluster. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i>  The Aurora database engine used by the global database cluster. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the current state of this global database cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalClusterArn</i> The Amazon Resource Name (ARN) for the global database cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalClusterResourceId</i>  The AWS Region-unique, immutable identifier for the global database cluster. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB cluster is accessed. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalClusterIdentifier</i>  Contains a user-supplied global database cluster identifier. This identifier is the unique key that identifies a global database cluster. <br/>        """
        pass

    def remove_role_from_db_cluster(self, RoleArn: str, DBClusterIdentifier: str, FeatureName: str=None):
        """Disassociates an AWS Identity and Access Management (IAM) role from an Amazon Aurora DB cluster. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/AuroraMySQL.Integrating.Authorizing.html">Authorizing Amazon Aurora MySQL to Access Other AWS Services on Your Behalf </a> in the <i>Amazon Aurora User Guide</i>. <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param FeatureName:  The name of the feature for the DB cluster that the IAM role is to be disassociated from. For the list of supported feature names, see <a>DBEngineVersion</a>.<br/>RoleArn:  The Amazon Resource Name (ARN) of the IAM role to disassociate from the Aurora DB cluster, for example <code>arn:aws:iam::123456789012:role/AuroraAccessRole</code>.<br/>DBClusterIdentifier:  The name of the DB cluster to disassociate the IAM role from.<br/>
        :return:         """
        pass

    def remove_role_from_db_instance(self, FeatureName: str, RoleArn: str, DBInstanceIdentifier: str):
        """Disassociates an AWS Identity and Access Management (IAM) role from a DB instance.

        :param self:
        :param FeatureName:  The name of the feature for the DB instance that the IAM role is to be disassociated from. For the list of supported feature names, see <code>DBEngineVersion</code>. <br/>RoleArn:  The Amazon Resource Name (ARN) of the IAM role to disassociate from the DB instance, for example <code>arn:aws:iam::123456789012:role/AccessRole</code>.<br/>DBInstanceIdentifier:  The name of the DB instance to disassociate the IAM role from.<br/>
        :return:         """
        pass

    def remove_source_identifier_from_subscription(self, SourceIdentifier: str, SubscriptionName: str) -> RemoveSourceIdentifierFromSubscriptionResult:
        """Removes a source identifier from an existing RDS event notification subscription.

        :param self:
        :param SourceIdentifier:   The source identifier to be removed from the subscription, such as the <b>DB instance identifier</b> for a DB instance or the name of a security group. <br/>SubscriptionName:  The name of the RDS event notification subscription you want to remove a source identifier from.<br/>
        :return: EventSubscription:  Contains the results of a successful invocation of the <code>DescribeEventSubscriptions</code> action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EventSubscriptionArn</i> The Amazon Resource Name (ARN) for the event subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> A Boolean value indicating if the subscription is enabled. True indicates the subscription is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EventCategoriesList</i> A list of event categories for the RDS event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceIdsList</i> A list of source IDs for the RDS event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceType</i> The source type for the RDS event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubscriptionCreationTime</i> The time the RDS event notification subscription was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the RDS event notification subscription. Constraints: Can be one of the following: creating | modifying | deleting | active | no-permission | topic-not-exist The status "no-permission" indicates that RDS no longer has permission to post to the SNS topic. The status "topic-not-exist" indicates that the topic was deleted after the subscription was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsTopicArn</i> The topic ARN of the RDS event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustSubscriptionId</i> The RDS event notification subscription Id.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomerAwsId</i> The AWS customer account associated with the RDS event notification subscription.<br/>        """
        pass

    def remove_tags_from_resource(self, TagKeys: List[String], ResourceName: str):
        """Removes metadata tags from an Amazon RDS resource. For an overview on tagging an Amazon RDS resource, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/Overview.Tagging.html">Tagging Amazon RDS Resources</a> in the <i>Amazon RDS User Guide.</i> 

        :param self:
        :param TagKeys:  The tag key (name) of the tag to be removed.<br/>ResourceName:  The Amazon RDS resource that the tags are removed from. This value is an Amazon Resource Name (ARN). For information about creating an ARN, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Tagging.ARN.html#USER_Tagging.ARN.Constructing"> Constructing an ARN for Amazon RDS</a> in the <i>Amazon RDS User Guide.</i> <br/>
        :return:         """
        pass

    def reset_db_cluster_parameter_group(self, DBClusterParameterGroupName: str, ResetAllParameters: Boolean=None, Parameters: List[Parameter]=None) -> DBClusterParameterGroupNameMessage:
        """ Modifies the parameters of a DB cluster parameter group to the default value. To reset specific parameters submit a list of the following: <code>ParameterName</code> and <code>ApplyMethod</code>. To reset the entire DB cluster parameter group, specify the <code>DBClusterParameterGroupName</code> and <code>ResetAllParameters</code> parameters.   When resetting the entire group, dynamic parameters are updated immediately and static parameters are set to <code>pending-reboot</code> to take effect on the next DB instance restart or <code>RebootDBInstance</code> request. You must call <code>RebootDBInstance</code> for every DB instance in your DB cluster that you want the updated static parameter to apply to. For more information on Amazon Aurora, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/CHAP_AuroraOverview.html"> What Is Amazon Aurora?</a> in the <i>Amazon Aurora User Guide.</i>  <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param Parameters:  A list of parameter names in the DB cluster parameter group to reset to the default values. You can't use this parameter if the <code>ResetAllParameters</code> parameter is enabled.<br/>ResetAllParameters:  A value that indicates whether to reset all parameters in the DB cluster parameter group to their default values. You can't use this parameter if there is a list of parameter names specified for the <code>Parameters</code> parameter.<br/>DBClusterParameterGroupName:  The name of the DB cluster parameter group to reset.<br/>
        :return: DBClusterParameterGroupName:  The name of the DB cluster parameter group. Constraints: <ul> <li> Must be 1 to 255 letters or numbers. </li> <li> First character must be a letter </li> <li> Can't end with a hyphen or contain two consecutive hyphens </li> </ul> <note> This value is stored as a lowercase string. </note><br/>        """
        pass

    def reset_db_parameter_group(self, DBParameterGroupName: str, ResetAllParameters: Boolean=None, Parameters: List[Parameter]=None) -> DBParameterGroupNameMessage:
        """Modifies the parameters of a DB parameter group to the engine/system default value. To reset specific parameters, provide a list of the following: <code>ParameterName</code> and <code>ApplyMethod</code>. To reset the entire DB parameter group, specify the <code>DBParameterGroup</code> name and <code>ResetAllParameters</code> parameters. When resetting the entire group, dynamic parameters are updated immediately and static parameters are set to <code>pending-reboot</code> to take effect on the next DB instance restart or <code>RebootDBInstance</code> request. 

        :param self:
        :param Parameters:  To reset the entire DB parameter group, specify the <code>DBParameterGroup</code> name and <code>ResetAllParameters</code> parameters. To reset specific parameters, provide a list of the following: <code>ParameterName</code> and <code>ApplyMethod</code>. A maximum of 20 parameters can be modified in a single request.  <b>MySQL</b>  Valid Values (for Apply method): <code>immediate</code> | <code>pending-reboot</code>  You can use the immediate value with dynamic parameters only. You can use the <code>pending-reboot</code> value for both dynamic and static parameters, and changes are applied when DB instance reboots.  <b>MariaDB</b>  Valid Values (for Apply method): <code>immediate</code> | <code>pending-reboot</code>  You can use the immediate value with dynamic parameters only. You can use the <code>pending-reboot</code> value for both dynamic and static parameters, and changes are applied when DB instance reboots.  <b>Oracle</b>  Valid Values (for Apply method): <code>pending-reboot</code> <br/>ResetAllParameters:   A value that indicates whether to reset all parameters in the DB parameter group to default values. By default, all parameters in the DB parameter group are reset to default values. <br/>DBParameterGroupName:  The name of the DB parameter group. Constraints: <ul> <li> Must match the name of an existing DBParameterGroup. </li> </ul><br/>
        :return: DBParameterGroupName:  Provides the name of the DB parameter group.<br/>        """
        pass

    def restore_db_cluster_from_s3(self, S3IngestionRoleArn: str, S3BucketName: str, SourceEngineVersion: str, SourceEngine: str, MasterUserPassword: str, MasterUsername: str, Engine: str, DBClusterIdentifier: str, AvailabilityZones: List[String]=None, BackupRetentionPeriod: int=None, CharacterSetName: str=None, DatabaseName: str=None, DBClusterParameterGroupName: str=None, VpcSecurityGroupIds: List[String]=None, DBSubnetGroupName: str=None, EngineVersion: str=None, Port: int=None, OptionGroupName: str=None, PreferredBackupWindow: str=None, PreferredMaintenanceWindow: str=None, Tags: List[Tag]=None, StorageEncrypted: BooleanOptional=None, KmsKeyId: str=None, EnableIAMDatabaseAuthentication: BooleanOptional=None, S3Prefix: str=None, BacktrackWindow: LongOptional=None, EnableCloudwatchLogsExports: List[String]=None, DeletionProtection: BooleanOptional=None, CopyTagsToSnapshot: BooleanOptional=None) -> RestoreDBClusterFromS3Result:
        """Creates an Amazon Aurora DB cluster from data stored in an Amazon S3 bucket. Amazon RDS must be authorized to access the Amazon S3 bucket and the data must be created using the Percona XtraBackup utility as described in <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/AuroraMySQL.Migrating.html"> Migrating Data to an Amazon Aurora MySQL DB Cluster</a> in the <i>Amazon Aurora User Guide</i>. <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param CopyTagsToSnapshot:  A value that indicates whether to copy all tags from the restored DB cluster to snapshots of the restored DB cluster. The default is not to copy them.<br/>DeletionProtection:  A value that indicates whether the DB cluster has deletion protection enabled. The database can't be deleted when deletion protection is enabled. By default, deletion protection is disabled. <br/>EnableCloudwatchLogsExports:  The list of logs that the restored DB cluster is to export to CloudWatch Logs. The values in the list depend on the DB engine being used. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/USER_LogAccess.html#USER_LogAccess.Procedural.UploadtoCloudWatch">Publishing Database Logs to Amazon CloudWatch Logs</a> in the <i>Amazon Aurora User Guide</i>.<br/>BacktrackWindow:  The target backtrack window, in seconds. To disable backtracking, set this value to 0. Default: 0 Constraints: <ul> <li> If specified, this value must be set to a number from 0 to 259,200 (72 hours). </li> </ul><br/>S3IngestionRoleArn:  The Amazon Resource Name (ARN) of the AWS Identity and Access Management (IAM) role that authorizes Amazon RDS to access the Amazon S3 bucket on your behalf.<br/>S3Prefix:  The prefix for all of the file names that contain the data used to create the Amazon Aurora DB cluster. If you do not specify a <b>SourceS3Prefix</b> value, then the Amazon Aurora DB cluster is created by using all of the files in the Amazon S3 bucket.<br/>S3BucketName:  The name of the Amazon S3 bucket that contains the data used to create the Amazon Aurora DB cluster.<br/>SourceEngineVersion:  The version of the database that the backup files were created from. MySQL version 5.5 and 5.6 are supported.  Example: <code>5.6.22</code> <br/>SourceEngine:  The identifier for the database engine that was backed up to create the files stored in the Amazon S3 bucket.  Valid values: <code>mysql</code> <br/>EnableIAMDatabaseAuthentication:  A value that indicates whether to enable mapping of AWS Identity and Access Management (IAM) accounts to database accounts. By default, mapping is disabled.<br/>KmsKeyId:  The AWS KMS key identifier for an encrypted DB cluster. The KMS key identifier is the Amazon Resource Name (ARN) for the KMS encryption key. If you are creating a DB cluster with the same AWS account that owns the KMS encryption key used to encrypt the new DB cluster, then you can use the KMS key alias instead of the ARN for the KM encryption key. If the StorageEncrypted parameter is enabled, and you do not specify a value for the <code>KmsKeyId</code> parameter, then Amazon RDS will use your default encryption key. AWS KMS creates the default encryption key for your AWS account. Your AWS account has a different default encryption key for each AWS Region.<br/>StorageEncrypted:  A value that indicates whether the restored DB cluster is encrypted.<br/>Tags:  A list of tags. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Tagging.html">Tagging Amazon RDS Resources</a> in the <i>Amazon RDS User Guide.</i> <br/>PreferredMaintenanceWindow:  The weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC). Format: <code>ddd:hh24:mi-ddd:hh24:mi</code>  The default is a 30-minute window selected at random from an 8-hour block of time for each AWS Region, occurring on a random day of the week. To see the time blocks available, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/USER_UpgradeDBInstance.Maintenance.html#AdjustingTheMaintenanceWindow.Aurora"> Adjusting the Preferred Maintenance Window</a> in the <i>Amazon Aurora User Guide.</i>  Valid Days: Mon, Tue, Wed, Thu, Fri, Sat, Sun. Constraints: Minimum 30-minute window.<br/>PreferredBackupWindow:  The daily time range during which automated backups are created if automated backups are enabled using the <code>BackupRetentionPeriod</code> parameter.  The default is a 30-minute window selected at random from an 8-hour block of time for each AWS Region. To see the time blocks available, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/USER_UpgradeDBInstance.Maintenance.html#AdjustingTheMaintenanceWindow.Aurora"> Adjusting the Preferred Maintenance Window</a> in the <i>Amazon Aurora User Guide.</i>  Constraints: <ul> <li> Must be in the format <code>hh24:mi-hh24:mi</code>. </li> <li> Must be in Universal Coordinated Time (UTC). </li> <li> Must not conflict with the preferred maintenance window. </li> <li> Must be at least 30 minutes. </li> </ul><br/>OptionGroupName:  A value that indicates that the restored DB cluster should be associated with the specified option group. Permanent options can't be removed from an option group. An option group can't be removed from a DB cluster once it is associated with a DB cluster.<br/>MasterUserPassword:  The password for the master database user. This password can contain any printable ASCII character except "/", triple-quotes, or "@". Constraints: Must contain from 8 to 41 characters.<br/>MasterUsername:  The name of the master user for the restored DB cluster. Constraints: <ul> <li> Must be 1 to 16 letters or numbers. </li> <li> First character must be a letter. </li> <li> Can't be a reserved word for the chosen database engine. </li> </ul><br/>Port:  The port number on which the instances in the restored DB cluster accept connections.  Default: <code>3306</code> <br/>EngineVersion:  The version number of the database engine to use.  <b>Aurora MySQL</b>  Example: <code>5.6.10a</code>   <b>Aurora PostgreSQL</b>  Example: <code>9.6.3</code> <br/>Engine:  The name of the database engine to be used for the restored DB cluster. Valid Values: <code>aurora</code>, <code>aurora-postgresql</code> <br/>DBSubnetGroupName:  A DB subnet group to associate with the restored DB cluster. Constraints: If supplied, must match the name of an existing DBSubnetGroup.  Example: <code>mySubnetgroup</code> <br/>VpcSecurityGroupIds:  A list of EC2 VPC security groups to associate with the restored DB cluster.<br/>DBClusterParameterGroupName:  The name of the DB cluster parameter group to associate with the restored DB cluster. If this argument is omitted, <code>default.aurora5.6</code> is used.  Constraints: <ul> <li> If supplied, must match the name of an existing DBClusterParameterGroup. </li> </ul><br/>DBClusterIdentifier:  The name of the DB cluster to create from the source data in the Amazon S3 bucket. This parameter is isn't case-sensitive. Constraints: <ul> <li> Must contain from 1 to 63 letters, numbers, or hyphens. </li> <li> First character must be a letter. </li> <li> Can't end with a hyphen or contain two consecutive hyphens. </li> </ul> Example: <code>my-cluster1</code> <br/>DatabaseName:  The database name for the restored DB cluster.<br/>CharacterSetName:  A value that indicates that the restored DB cluster should be associated with the specified CharacterSet.<br/>BackupRetentionPeriod:  The number of days for which automated backups of the restored DB cluster are retained. You must specify a minimum value of 1. Default: 1 Constraints: <ul> <li> Must be a value from 1 to 35 </li> </ul><br/>AvailabilityZones:  A list of Availability Zones (AZs) where instances in the restored DB cluster can be created.<br/>
        :return: DBCluster:  Contains the details of an Amazon Aurora DB cluster.  This data type is used as a response element in the <code>DescribeDBClusters</code>, <code>StopDBCluster</code>, and <code>StartDBCluster</code> actions. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CrossAccountClone</i> Specifies whether the DB cluster is a clone of a DB cluster owned by a different AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyTagsToSnapshot</i> Specifies whether tags are copied from the DB cluster to snapshots of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamKinesisStreamName</i> The name of the Amazon Kinesis data stream used for the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamKmsKeyId</i> The AWS KMS key identifier used for encrypting messages in the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamStatus</i> The status of the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamMode</i> The mode of the database activity stream. Database events such as a change or access generate an activity stream event. The database session can handle these events either synchronously or asynchronously. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HttpEndpointEnabled</i> A value that indicates whether the HTTP endpoint for an Aurora Serverless DB cluster is enabled. When enabled, the HTTP endpoint provides a connectionless web service API for running SQL queries on the Aurora Serverless DB cluster. You can also query your database from inside the RDS console with the query editor. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/data-api.html">Using the Data API for Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i> Indicates if the DB cluster has deletion protection enabled. The database can't be deleted when deletion protection is enabled. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ScalingConfigurationInfo</i> Shows the scaling configuration for an Aurora DB cluster in <code>serverless</code> DB engine mode. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ScalingConfigurationInfo</b>: Shows the scaling configuration for an Aurora DB cluster in <code>serverless</code> DB engine mode. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineMode</i> The DB engine mode of the DB cluster, either <code>provisioned</code>, <code>serverless</code>, or <code>parallelquery</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Capacity</i> The current capacity of an Aurora Serverless DB cluster. The capacity is 0 (zero) when the cluster is paused. For more information about Aurora Serverless, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB cluster is configured to export to CloudWatch Logs. Log types vary by DB engine. For information about the log types for each DB engine, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/USER_LogAccess.html">Amazon RDS Database Log Files</a> in the <i>Amazon Aurora User Guide.</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BacktrackConsumedChangeRecords</i> The number of change records stored for Backtrack.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BacktrackWindow</i> The target backtrack window, in seconds. If this value is set to 0, backtracking is disabled for the DB cluster. Otherwise, backtracking is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestBacktrackTime</i> The earliest time to which a DB cluster can be backtracked.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloneGroupId</i> Identifies the clone group to which the DB cluster is associated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> A value that indicates whether the mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i> Provides a list of the AWS Identity and Access Management (IAM) roles that are associated with the DB cluster. IAM roles that are associated with a DB cluster grant permission for the DB cluster to access other AWS services on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterArn</i> The Amazon Resource Name (ARN) for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbClusterResourceId</i> The AWS Region-unique, immutable identifier for the DB cluster. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB cluster is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is enabled, the AWS KMS key identifier for the encrypted DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security groups that the DB cluster belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterMembers</i> Provides the list of instances that make up the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSourceIdentifier</i> Contains the identifier of the source DB cluster if this DB cluster is a Read Replica.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i> Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterOptionGroupMemberships</i> Provides the list of option group memberships for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies whether the DB cluster has instances in multiple Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomEndpoints</i> Identifies all custom endpoints associated with the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReaderEndpoint</i> The reader endpoint for the DB cluster. The reader endpoint for a DB cluster load-balances connections across the Aurora Replicas that are available in a DB cluster. As clients request new connections to the reader endpoint, Aurora distributes the connection requests among the Aurora Replicas in the DB cluster. This functionality can help balance your read workload across multiple Aurora Replicas in your DB cluster.  If a failover occurs, and the Aurora Replica that you are connected to is promoted to be the primary instance, your connection is dropped. To continue sending your read workload to other Aurora Replicas in the cluster, you can then reconnect to the reader endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint for the primary instance of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestRestorableTime</i> The earliest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the progress of the operation as a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the current state of this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB cluster, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroup</i> Specifies the name of the DB cluster parameter group for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Contains a user-supplied DB cluster identifier. This identifier is the unique key that identifies a DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> Contains the name of the initial database of this DB cluster that was provided at create time, if one was specified when the DB cluster was created. This same name is returned for the life of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this cluster is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of Availability Zones (AZs) where instances in the DB cluster can be created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> For all database engines except Amazon Aurora, <code>AllocatedStorage</code> specifies the allocated storage size in gibibytes (GiB). For Aurora, <code>AllocatedStorage</code> always returns 1, because Aurora DB cluster storage size is not fixed, but instead automatically adjusts as needed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeoutAction</i> The timeout action of a call to <code>ModifyCurrentDBClusterCapacity</code>, either <code>ForceApplyCapacityChange</code> or <code>RollbackCapacityChange</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondsUntilAutoPause</i> The remaining amount of time, in seconds, before the Aurora DB cluster in <code>serverless</code> mode is paused. A DB cluster can be paused only when it's idle (it has no connections).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoPause</i> A value that indicates whether automatic pause is allowed for the Aurora DB cluster in <code>serverless</code> DB engine mode. When the value is set to false for an Aurora Serverless DB cluster, the DB cluster automatically resumes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxCapacity</i> The maximum capacity for an Aurora DB cluster in <code>serverless</code> DB engine mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinCapacity</i> The maximum capacity for the Aurora DB cluster in <code>serverless</code> DB engine mode.<br/>        """
        pass

    def restore_db_cluster_from_snapshot(self, Engine: str, SnapshotIdentifier: str, DBClusterIdentifier: str, AvailabilityZones: List[String]=None, EngineVersion: str=None, Port: int=None, DBSubnetGroupName: str=None, DatabaseName: str=None, OptionGroupName: str=None, VpcSecurityGroupIds: List[String]=None, Tags: List[Tag]=None, KmsKeyId: str=None, EnableIAMDatabaseAuthentication: BooleanOptional=None, BacktrackWindow: LongOptional=None, EnableCloudwatchLogsExports: List[String]=None, EngineMode: str=None, ScalingConfiguration: ScalingConfiguration=None, DBClusterParameterGroupName: str=None, DeletionProtection: BooleanOptional=None, CopyTagsToSnapshot: BooleanOptional=None) -> RestoreDBClusterFromSnapshotResult:
        """Creates a new DB cluster from a DB snapshot or DB cluster snapshot. If a DB snapshot is specified, the target DB cluster is created from the source DB snapshot with a default configuration and default security group. If a DB cluster snapshot is specified, the target DB cluster is created from the source DB cluster restore point with the same configuration as the original source DB cluster, except that the new DB cluster is created with the default security group. For more information on Amazon Aurora, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/CHAP_AuroraOverview.html"> What Is Amazon Aurora?</a> in the <i>Amazon Aurora User Guide.</i>  <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param CopyTagsToSnapshot:  A value that indicates whether to copy all tags from the restored DB cluster to snapshots of the restored DB cluster. The default is not to copy them.<br/>DeletionProtection:  A value that indicates whether the DB cluster has deletion protection enabled. The database can't be deleted when deletion protection is enabled. By default, deletion protection is disabled. <br/>DBClusterParameterGroupName:  The name of the DB cluster parameter group to associate with this DB cluster. If this argument is omitted, the default DB cluster parameter group for the specified engine is used. Constraints: <ul> <li> If supplied, must match the name of an existing default DB cluster parameter group. </li> <li> Must be 1 to 255 letters, numbers, or hyphens. </li> <li> First character must be a letter. </li> <li> Can't end with a hyphen or contain two consecutive hyphens. </li> </ul><br/>ScalingConfiguration:  For DB clusters in <code>serverless</code> DB engine mode, the scaling properties of the DB cluster.:<br/>EngineMode:  The DB engine mode of the DB cluster, either <code>provisioned</code>, <code>serverless</code>, or <code>parallelquery</code>.<br/>EnableCloudwatchLogsExports:  The list of logs that the restored DB cluster is to export to Amazon CloudWatch Logs. The values in the list depend on the DB engine being used. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/USER_LogAccess.html#USER_LogAccess.Procedural.UploadtoCloudWatch">Publishing Database Logs to Amazon CloudWatch Logs </a> in the <i>Amazon Aurora User Guide</i>.<br/>BacktrackWindow:  The target backtrack window, in seconds. To disable backtracking, set this value to 0. Default: 0 Constraints: <ul> <li> If specified, this value must be set to a number from 0 to 259,200 (72 hours). </li> </ul><br/>EnableIAMDatabaseAuthentication:  A value that indicates whether to enable mapping of AWS Identity and Access Management (IAM) accounts to database accounts. By default, mapping is disabled.<br/>KmsKeyId:  The AWS KMS key identifier to use when restoring an encrypted DB cluster from a DB snapshot or DB cluster snapshot. The KMS key identifier is the Amazon Resource Name (ARN) for the KMS encryption key. If you are restoring a DB cluster with the same AWS account that owns the KMS encryption key used to encrypt the new DB cluster, then you can use the KMS key alias instead of the ARN for the KMS encryption key. If you don't specify a value for the <code>KmsKeyId</code> parameter, then the following occurs: <ul> <li> If the DB snapshot or DB cluster snapshot in <code>SnapshotIdentifier</code> is encrypted, then the restored DB cluster is encrypted using the KMS key that was used to encrypt the DB snapshot or DB cluster snapshot. </li> <li> If the DB snapshot or DB cluster snapshot in <code>SnapshotIdentifier</code> is not encrypted, then the restored DB cluster is not encrypted. </li> </ul><br/>Tags:  The tags to be assigned to the restored DB cluster.<br/>VpcSecurityGroupIds:  A list of VPC security groups that the new DB cluster will belong to.<br/>OptionGroupName:  The name of the option group to use for the restored DB cluster.<br/>DatabaseName:  The database name for the restored DB cluster.<br/>DBSubnetGroupName:  The name of the DB subnet group to use for the new DB cluster. Constraints: If supplied, must match the name of an existing DB subnet group. Example: <code>mySubnetgroup</code> <br/>Port:  The port number on which the new DB cluster accepts connections. Constraints: This value must be <code>1150-65535</code>  Default: The same port as the original DB cluster.<br/>EngineVersion:  The version of the database engine to use for the new DB cluster.<br/>Engine:  The database engine to use for the new DB cluster. Default: The same as source Constraint: Must be compatible with the engine of the source<br/>SnapshotIdentifier:  The identifier for the DB snapshot or DB cluster snapshot to restore from. You can use either the name or the Amazon Resource Name (ARN) to specify a DB cluster snapshot. However, you can use only the ARN to specify a DB snapshot. Constraints: <ul> <li> Must match the identifier of an existing Snapshot. </li> </ul><br/>DBClusterIdentifier:  The name of the DB cluster to create from the DB snapshot or DB cluster snapshot. This parameter isn't case-sensitive. Constraints: <ul> <li> Must contain from 1 to 63 letters, numbers, or hyphens </li> <li> First character must be a letter </li> <li> Can't end with a hyphen or contain two consecutive hyphens </li> </ul> Example: <code>my-snapshot-id</code> <br/>AvailabilityZones:  Provides the list of Availability Zones (AZs) where instances in the restored DB cluster can be created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeoutAction</i> The action to take when the timeout is reached, either <code>ForceApplyCapacityChange</code> or <code>RollbackCapacityChange</code>.  <code>ForceApplyCapacityChange</code> sets the capacity to the specified value as soon as possible.  <code>RollbackCapacityChange</code>, the default, ignores the capacity change if a scaling point is not found in the timeout period. <important> If you specify <code>ForceApplyCapacityChange</code>, connections that prevent Aurora Serverless from finding a scaling point might be dropped. </important> For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.how-it-works.html#aurora-serverless.how-it-works.auto-scaling"> Autoscaling for Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondsUntilAutoPause</i> The time, in seconds, before an Aurora DB cluster in <code>serverless</code> mode is paused.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoPause</i> A value that indicates whether to allow or disallow automatic pause for an Aurora DB cluster in <code>serverless</code> DB engine mode. A DB cluster can be paused only when it's idle (it has no connections). <note> If a DB cluster is paused for more than seven days, the DB cluster might be backed up with a snapshot. In this case, the DB cluster is restored when there is a request to connect to it.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxCapacity</i> The maximum capacity for an Aurora DB cluster in <code>serverless</code> DB engine mode. Valid capacity values are <code>1</code>, <code>2</code>, <code>4</code>, <code>8</code>, <code>16</code>, <code>32</code>, <code>64</code>, <code>128</code>, and <code>256</code>. The maximum capacity must be greater than or equal to the minimum capacity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MinCapacity</i> The minimum capacity for an Aurora DB cluster in <code>serverless</code> DB engine mode. Valid capacity values are <code>1</code>, <code>2</code>, <code>4</code>, <code>8</code>, <code>16</code>, <code>32</code>, <code>64</code>, <code>128</code>, and <code>256</code>. The minimum capacity must be less than or equal to the maximum capacity.<br/>
        :return: DBCluster:  Contains the details of an Amazon Aurora DB cluster.  This data type is used as a response element in the <code>DescribeDBClusters</code>, <code>StopDBCluster</code>, and <code>StartDBCluster</code> actions. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CrossAccountClone</i> Specifies whether the DB cluster is a clone of a DB cluster owned by a different AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyTagsToSnapshot</i> Specifies whether tags are copied from the DB cluster to snapshots of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamKinesisStreamName</i> The name of the Amazon Kinesis data stream used for the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamKmsKeyId</i> The AWS KMS key identifier used for encrypting messages in the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamStatus</i> The status of the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamMode</i> The mode of the database activity stream. Database events such as a change or access generate an activity stream event. The database session can handle these events either synchronously or asynchronously. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HttpEndpointEnabled</i> A value that indicates whether the HTTP endpoint for an Aurora Serverless DB cluster is enabled. When enabled, the HTTP endpoint provides a connectionless web service API for running SQL queries on the Aurora Serverless DB cluster. You can also query your database from inside the RDS console with the query editor. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/data-api.html">Using the Data API for Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i> Indicates if the DB cluster has deletion protection enabled. The database can't be deleted when deletion protection is enabled. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ScalingConfigurationInfo</i> Shows the scaling configuration for an Aurora DB cluster in <code>serverless</code> DB engine mode. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ScalingConfigurationInfo</b>: Shows the scaling configuration for an Aurora DB cluster in <code>serverless</code> DB engine mode. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineMode</i> The DB engine mode of the DB cluster, either <code>provisioned</code>, <code>serverless</code>, or <code>parallelquery</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Capacity</i> The current capacity of an Aurora Serverless DB cluster. The capacity is 0 (zero) when the cluster is paused. For more information about Aurora Serverless, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB cluster is configured to export to CloudWatch Logs. Log types vary by DB engine. For information about the log types for each DB engine, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/USER_LogAccess.html">Amazon RDS Database Log Files</a> in the <i>Amazon Aurora User Guide.</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BacktrackConsumedChangeRecords</i> The number of change records stored for Backtrack.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BacktrackWindow</i> The target backtrack window, in seconds. If this value is set to 0, backtracking is disabled for the DB cluster. Otherwise, backtracking is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestBacktrackTime</i> The earliest time to which a DB cluster can be backtracked.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloneGroupId</i> Identifies the clone group to which the DB cluster is associated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> A value that indicates whether the mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i> Provides a list of the AWS Identity and Access Management (IAM) roles that are associated with the DB cluster. IAM roles that are associated with a DB cluster grant permission for the DB cluster to access other AWS services on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterArn</i> The Amazon Resource Name (ARN) for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbClusterResourceId</i> The AWS Region-unique, immutable identifier for the DB cluster. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB cluster is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is enabled, the AWS KMS key identifier for the encrypted DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security groups that the DB cluster belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterMembers</i> Provides the list of instances that make up the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSourceIdentifier</i> Contains the identifier of the source DB cluster if this DB cluster is a Read Replica.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i> Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterOptionGroupMemberships</i> Provides the list of option group memberships for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies whether the DB cluster has instances in multiple Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomEndpoints</i> Identifies all custom endpoints associated with the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReaderEndpoint</i> The reader endpoint for the DB cluster. The reader endpoint for a DB cluster load-balances connections across the Aurora Replicas that are available in a DB cluster. As clients request new connections to the reader endpoint, Aurora distributes the connection requests among the Aurora Replicas in the DB cluster. This functionality can help balance your read workload across multiple Aurora Replicas in your DB cluster.  If a failover occurs, and the Aurora Replica that you are connected to is promoted to be the primary instance, your connection is dropped. To continue sending your read workload to other Aurora Replicas in the cluster, you can then reconnect to the reader endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint for the primary instance of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestRestorableTime</i> The earliest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the progress of the operation as a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the current state of this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB cluster, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroup</i> Specifies the name of the DB cluster parameter group for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Contains a user-supplied DB cluster identifier. This identifier is the unique key that identifies a DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> Contains the name of the initial database of this DB cluster that was provided at create time, if one was specified when the DB cluster was created. This same name is returned for the life of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this cluster is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of Availability Zones (AZs) where instances in the DB cluster can be created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> For all database engines except Amazon Aurora, <code>AllocatedStorage</code> specifies the allocated storage size in gibibytes (GiB). For Aurora, <code>AllocatedStorage</code> always returns 1, because Aurora DB cluster storage size is not fixed, but instead automatically adjusts as needed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeoutAction</i> The timeout action of a call to <code>ModifyCurrentDBClusterCapacity</code>, either <code>ForceApplyCapacityChange</code> or <code>RollbackCapacityChange</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondsUntilAutoPause</i> The remaining amount of time, in seconds, before the Aurora DB cluster in <code>serverless</code> mode is paused. A DB cluster can be paused only when it's idle (it has no connections).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoPause</i> A value that indicates whether automatic pause is allowed for the Aurora DB cluster in <code>serverless</code> DB engine mode. When the value is set to false for an Aurora Serverless DB cluster, the DB cluster automatically resumes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxCapacity</i> The maximum capacity for an Aurora DB cluster in <code>serverless</code> DB engine mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinCapacity</i> The maximum capacity for the Aurora DB cluster in <code>serverless</code> DB engine mode.<br/>        """
        pass

    def restore_db_cluster_to_point_in_time(self, SourceDBClusterIdentifier: str, DBClusterIdentifier: str, RestoreType: str=None, RestoreToTime: TStamp=None, UseLatestRestorableTime: Boolean=None, Port: int=None, DBSubnetGroupName: str=None, OptionGroupName: str=None, VpcSecurityGroupIds: List[String]=None, Tags: List[Tag]=None, KmsKeyId: str=None, EnableIAMDatabaseAuthentication: BooleanOptional=None, BacktrackWindow: LongOptional=None, EnableCloudwatchLogsExports: List[String]=None, DBClusterParameterGroupName: str=None, DeletionProtection: BooleanOptional=None, CopyTagsToSnapshot: BooleanOptional=None) -> RestoreDBClusterToPointInTimeResult:
        """Restores a DB cluster to an arbitrary point in time. Users can restore to any point in time before <code>LatestRestorableTime</code> for up to <code>BackupRetentionPeriod</code> days. The target DB cluster is created from the source DB cluster with the same configuration as the original DB cluster, except that the new DB cluster is created with the default DB security group.  <note> This action only restores the DB cluster, not the DB instances for that DB cluster. You must invoke the <code>CreateDBInstance</code> action to create DB instances for the restored DB cluster, specifying the identifier of the restored DB cluster in <code>DBClusterIdentifier</code>. You can create DB instances only after the <code>RestoreDBClusterToPointInTime</code> action has completed and the DB cluster is available. </note> For more information on Amazon Aurora, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/CHAP_AuroraOverview.html"> What Is Amazon Aurora?</a> in the <i>Amazon Aurora User Guide.</i>  <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param CopyTagsToSnapshot:  A value that indicates whether to copy all tags from the restored DB cluster to snapshots of the restored DB cluster. The default is not to copy them.<br/>DeletionProtection:  A value that indicates whether the DB cluster has deletion protection enabled. The database can't be deleted when deletion protection is enabled. By default, deletion protection is disabled. <br/>DBClusterParameterGroupName:  The name of the DB cluster parameter group to associate with this DB cluster. If this argument is omitted, the default DB cluster parameter group for the specified engine is used. Constraints: <ul> <li> If supplied, must match the name of an existing DB cluster parameter group. </li> <li> Must be 1 to 255 letters, numbers, or hyphens. </li> <li> First character must be a letter. </li> <li> Can't end with a hyphen or contain two consecutive hyphens. </li> </ul><br/>EnableCloudwatchLogsExports:  The list of logs that the restored DB cluster is to export to CloudWatch Logs. The values in the list depend on the DB engine being used. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/USER_LogAccess.html#USER_LogAccess.Procedural.UploadtoCloudWatch">Publishing Database Logs to Amazon CloudWatch Logs</a> in the <i>Amazon Aurora User Guide</i>.<br/>BacktrackWindow:  The target backtrack window, in seconds. To disable backtracking, set this value to 0. Default: 0 Constraints: <ul> <li> If specified, this value must be set to a number from 0 to 259,200 (72 hours). </li> </ul><br/>EnableIAMDatabaseAuthentication:  A value that indicates whether to enable mapping of AWS Identity and Access Management (IAM) accounts to database accounts. By default, mapping is disabled.<br/>KmsKeyId:  The AWS KMS key identifier to use when restoring an encrypted DB cluster from an encrypted DB cluster. The KMS key identifier is the Amazon Resource Name (ARN) for the KMS encryption key. If you are restoring a DB cluster with the same AWS account that owns the KMS encryption key used to encrypt the new DB cluster, then you can use the KMS key alias instead of the ARN for the KMS encryption key. You can restore to a new DB cluster and encrypt the new DB cluster with a KMS key that is different than the KMS key used to encrypt the source DB cluster. The new DB cluster is encrypted with the KMS key identified by the <code>KmsKeyId</code> parameter. If you don't specify a value for the <code>KmsKeyId</code> parameter, then the following occurs: <ul> <li> If the DB cluster is encrypted, then the restored DB cluster is encrypted using the KMS key that was used to encrypt the source DB cluster. </li> <li> If the DB cluster is not encrypted, then the restored DB cluster is not encrypted. </li> </ul> If <code>DBClusterIdentifier</code> refers to a DB cluster that is not encrypted, then the restore request is rejected.<br/>Tags:  A list of tags. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Tagging.html">Tagging Amazon RDS Resources</a> in the <i>Amazon RDS User Guide.</i> <br/>VpcSecurityGroupIds:  A list of VPC security groups that the new DB cluster belongs to.<br/>OptionGroupName:  The name of the option group for the new DB cluster.<br/>DBSubnetGroupName:  The DB subnet group name to use for the new DB cluster. Constraints: If supplied, must match the name of an existing DBSubnetGroup. Example: <code>mySubnetgroup</code> <br/>Port:  The port number on which the new DB cluster accepts connections. Constraints: A value from <code>1150-65535</code>.  Default: The default port for the engine.<br/>UseLatestRestorableTime:  A value that indicates whether to restore the DB cluster to the latest restorable backup time. By default, the DB cluster is not restored to the latest restorable backup time.  Constraints: Can't be specified if <code>RestoreToTime</code> parameter is provided.<br/>RestoreToTime:  The date and time to restore the DB cluster to. Valid Values: Value must be a time in Universal Coordinated Time (UTC) format Constraints: <ul> <li> Must be before the latest restorable time for the DB instance </li> <li> Must be specified if <code>UseLatestRestorableTime</code> parameter is not provided </li> <li> Can't be specified if the <code>UseLatestRestorableTime</code> parameter is enabled </li> <li> Can't be specified if the <code>RestoreType</code> parameter is <code>copy-on-write</code>  </li> </ul> Example: <code>2015-03-07T23:45:00Z</code> <br/>SourceDBClusterIdentifier:  The identifier of the source DB cluster from which to restore. Constraints: <ul> <li> Must match the identifier of an existing DBCluster. </li> </ul><br/>RestoreType:  The type of restore to be performed. You can specify one of the following values: <ul> <li>  <code>full-copy</code> - The new DB cluster is restored as a full copy of the source DB cluster. </li> <li>  <code>copy-on-write</code> - The new DB cluster is restored as a clone of the source DB cluster. </li> </ul> Constraints: You can't specify <code>copy-on-write</code> if the engine version of the source DB cluster is earlier than 1.11. If you don't specify a <code>RestoreType</code> value, then the new DB cluster is restored as a full copy of the source DB cluster.<br/>DBClusterIdentifier:  The name of the new DB cluster to be created. Constraints: <ul> <li> Must contain from 1 to 63 letters, numbers, or hyphens </li> <li> First character must be a letter </li> <li> Can't end with a hyphen or contain two consecutive hyphens </li> </ul><br/>
        :return: DBCluster:  Contains the details of an Amazon Aurora DB cluster.  This data type is used as a response element in the <code>DescribeDBClusters</code>, <code>StopDBCluster</code>, and <code>StartDBCluster</code> actions. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CrossAccountClone</i> Specifies whether the DB cluster is a clone of a DB cluster owned by a different AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyTagsToSnapshot</i> Specifies whether tags are copied from the DB cluster to snapshots of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamKinesisStreamName</i> The name of the Amazon Kinesis data stream used for the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamKmsKeyId</i> The AWS KMS key identifier used for encrypting messages in the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamStatus</i> The status of the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamMode</i> The mode of the database activity stream. Database events such as a change or access generate an activity stream event. The database session can handle these events either synchronously or asynchronously. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HttpEndpointEnabled</i> A value that indicates whether the HTTP endpoint for an Aurora Serverless DB cluster is enabled. When enabled, the HTTP endpoint provides a connectionless web service API for running SQL queries on the Aurora Serverless DB cluster. You can also query your database from inside the RDS console with the query editor. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/data-api.html">Using the Data API for Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i> Indicates if the DB cluster has deletion protection enabled. The database can't be deleted when deletion protection is enabled. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ScalingConfigurationInfo</i> Shows the scaling configuration for an Aurora DB cluster in <code>serverless</code> DB engine mode. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ScalingConfigurationInfo</b>: Shows the scaling configuration for an Aurora DB cluster in <code>serverless</code> DB engine mode. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineMode</i> The DB engine mode of the DB cluster, either <code>provisioned</code>, <code>serverless</code>, or <code>parallelquery</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Capacity</i> The current capacity of an Aurora Serverless DB cluster. The capacity is 0 (zero) when the cluster is paused. For more information about Aurora Serverless, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB cluster is configured to export to CloudWatch Logs. Log types vary by DB engine. For information about the log types for each DB engine, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/USER_LogAccess.html">Amazon RDS Database Log Files</a> in the <i>Amazon Aurora User Guide.</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BacktrackConsumedChangeRecords</i> The number of change records stored for Backtrack.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BacktrackWindow</i> The target backtrack window, in seconds. If this value is set to 0, backtracking is disabled for the DB cluster. Otherwise, backtracking is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestBacktrackTime</i> The earliest time to which a DB cluster can be backtracked.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloneGroupId</i> Identifies the clone group to which the DB cluster is associated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> A value that indicates whether the mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i> Provides a list of the AWS Identity and Access Management (IAM) roles that are associated with the DB cluster. IAM roles that are associated with a DB cluster grant permission for the DB cluster to access other AWS services on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterArn</i> The Amazon Resource Name (ARN) for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbClusterResourceId</i> The AWS Region-unique, immutable identifier for the DB cluster. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB cluster is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is enabled, the AWS KMS key identifier for the encrypted DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security groups that the DB cluster belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterMembers</i> Provides the list of instances that make up the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSourceIdentifier</i> Contains the identifier of the source DB cluster if this DB cluster is a Read Replica.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i> Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterOptionGroupMemberships</i> Provides the list of option group memberships for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies whether the DB cluster has instances in multiple Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomEndpoints</i> Identifies all custom endpoints associated with the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReaderEndpoint</i> The reader endpoint for the DB cluster. The reader endpoint for a DB cluster load-balances connections across the Aurora Replicas that are available in a DB cluster. As clients request new connections to the reader endpoint, Aurora distributes the connection requests among the Aurora Replicas in the DB cluster. This functionality can help balance your read workload across multiple Aurora Replicas in your DB cluster.  If a failover occurs, and the Aurora Replica that you are connected to is promoted to be the primary instance, your connection is dropped. To continue sending your read workload to other Aurora Replicas in the cluster, you can then reconnect to the reader endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint for the primary instance of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestRestorableTime</i> The earliest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the progress of the operation as a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the current state of this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB cluster, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroup</i> Specifies the name of the DB cluster parameter group for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Contains a user-supplied DB cluster identifier. This identifier is the unique key that identifies a DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> Contains the name of the initial database of this DB cluster that was provided at create time, if one was specified when the DB cluster was created. This same name is returned for the life of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this cluster is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of Availability Zones (AZs) where instances in the DB cluster can be created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> For all database engines except Amazon Aurora, <code>AllocatedStorage</code> specifies the allocated storage size in gibibytes (GiB). For Aurora, <code>AllocatedStorage</code> always returns 1, because Aurora DB cluster storage size is not fixed, but instead automatically adjusts as needed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeoutAction</i> The timeout action of a call to <code>ModifyCurrentDBClusterCapacity</code>, either <code>ForceApplyCapacityChange</code> or <code>RollbackCapacityChange</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondsUntilAutoPause</i> The remaining amount of time, in seconds, before the Aurora DB cluster in <code>serverless</code> mode is paused. A DB cluster can be paused only when it's idle (it has no connections).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoPause</i> A value that indicates whether automatic pause is allowed for the Aurora DB cluster in <code>serverless</code> DB engine mode. When the value is set to false for an Aurora Serverless DB cluster, the DB cluster automatically resumes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxCapacity</i> The maximum capacity for an Aurora DB cluster in <code>serverless</code> DB engine mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinCapacity</i> The maximum capacity for the Aurora DB cluster in <code>serverless</code> DB engine mode.<br/>        """
        pass

    def restore_db_instance_from_db_snapshot(self, DBSnapshotIdentifier: str, DBInstanceIdentifier: str, DBInstanceClass: str=None, Port: int=None, AvailabilityZone: str=None, DBSubnetGroupName: str=None, MultiAZ: BooleanOptional=None, PubliclyAccessible: BooleanOptional=None, AutoMinorVersionUpgrade: BooleanOptional=None, LicenseModel: str=None, DBName: str=None, Engine: str=None, Iops: int=None, OptionGroupName: str=None, Tags: List[Tag]=None, StorageType: str=None, TdeCredentialArn: str=None, TdeCredentialPassword: str=None, VpcSecurityGroupIds: List[String]=None, Domain: str=None, CopyTagsToSnapshot: BooleanOptional=None, DomainIAMRoleName: str=None, EnableIAMDatabaseAuthentication: BooleanOptional=None, EnableCloudwatchLogsExports: List[String]=None, ProcessorFeatures: List[ProcessorFeature]=None, UseDefaultProcessorFeatures: BooleanOptional=None, DBParameterGroupName: str=None, DeletionProtection: BooleanOptional=None) -> RestoreDBInstanceFromDBSnapshotResult:
        """Creates a new DB instance from a DB snapshot. The target database is created from the source database restore point with the most of original configuration with the default security group and the default DB parameter group. By default, the new DB instance is created as a single-AZ deployment except when the instance is a SQL Server instance that has an option group that is associated with mirroring; in this case, the instance becomes a mirrored AZ deployment and not a single-AZ deployment. If your intent is to replace your original DB instance with the new, restored DB instance, then rename your original DB instance before you call the RestoreDBInstanceFromDBSnapshot action. RDS doesn't allow two DB instances with the same name. Once you have renamed your original DB instance with a different identifier, then you can pass the original name of the DB instance as the DBInstanceIdentifier in the call to the RestoreDBInstanceFromDBSnapshot action. The result is that you will replace the original DB instance with the DB instance created from the snapshot. If you are restoring from a shared manual DB snapshot, the <code>DBSnapshotIdentifier</code> must be the ARN of the shared DB snapshot. <note> This command doesn't apply to Aurora MySQL and Aurora PostgreSQL. For Aurora, use <code>RestoreDBClusterFromSnapshot</code>. </note>

        :param self:
        :param DeletionProtection:  A value that indicates whether the DB instance has deletion protection enabled. The database can't be deleted when deletion protection is enabled. By default, deletion protection is disabled. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_DeleteInstance.html"> Deleting a DB Instance</a>. <br/>DBParameterGroupName:  The name of the DB parameter group to associate with this DB instance. If this argument is omitted, the default DBParameterGroup for the specified engine is used. Constraints: <ul> <li> If supplied, must match the name of an existing DBParameterGroup. </li> <li> Must be 1 to 255 letters, numbers, or hyphens. </li> <li> First character must be a letter. </li> <li> Can't end with a hyphen or contain two consecutive hyphens. </li> </ul><br/>UseDefaultProcessorFeatures:  A value that indicates whether the DB instance class of the DB instance uses its default processor features.<br/>ProcessorFeatures:  The number of CPU cores and the number of threads per core for the DB instance class of the DB instance.<br/>EnableCloudwatchLogsExports:  The list of logs that the restored DB instance is to export to CloudWatch Logs. The values in the list depend on the DB engine being used. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/USER_LogAccess.html#USER_LogAccess.Procedural.UploadtoCloudWatch">Publishing Database Logs to Amazon CloudWatch Logs</a> in the <i>Amazon Aurora User Guide</i>.<br/>EnableIAMDatabaseAuthentication:  A value that indicates whether to enable mapping of AWS Identity and Access Management (IAM) accounts to database accounts. By default, mapping is disabled. You can enable IAM database authentication for the following database engines <ul> <li> For MySQL 5.6, minor version 5.6.34 or higher </li> <li> For MySQL 5.7, minor version 5.7.16 or higher </li> </ul><br/>DomainIAMRoleName:  Specify the name of the IAM role to be used when making API calls to the Directory Service.<br/>CopyTagsToSnapshot:  A value that indicates whether to copy all tags from the restored DB instance to snapshots of the DB instance. By default, tags are not copied.<br/>Domain:  Specify the Active Directory Domain to restore the instance in.<br/>VpcSecurityGroupIds:   A list of EC2 VPC security groups to associate with this DB instance.   Default: The default EC2 VPC security group for the DB subnet group's VPC. <br/>TdeCredentialPassword:  The password for the given ARN from the key store in order to access the device.<br/>TdeCredentialArn:  The ARN from the key store with which to associate the instance for TDE encryption.<br/>StorageType:  Specifies the storage type to be associated with the DB instance.  Valid values: <code>standard | gp2 | io1</code>   If you specify <code>io1</code>, you must also include a value for the <code>Iops</code> parameter.   Default: <code>io1</code> if the <code>Iops</code> parameter is specified, otherwise <code>gp2</code> <br/>Tags:  A list of tags. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Tagging.html">Tagging Amazon RDS Resources</a> in the <i>Amazon RDS User Guide.</i> <br/>OptionGroupName:  The name of the option group to be used for the restored DB instance. Permanent options, such as the TDE option for Oracle Advanced Security TDE, can't be removed from an option group, and that option group can't be removed from a DB instance once it is associated with a DB instance<br/>Iops:  Specifies the amount of provisioned IOPS for the DB instance, expressed in I/O operations per second. If this parameter is not specified, the IOPS value is taken from the backup. If this parameter is set to 0, the new instance is converted to a non-PIOPS instance. The conversion takes additional time, though your DB instance is available for connections before the conversion starts.  The provisioned IOPS value must follow the requirements for your database engine. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/CHAP_Storage.html#USER_PIOPS">Amazon RDS Provisioned IOPS Storage to Improve Performance</a> in the <i>Amazon RDS User Guide.</i>  Constraints: Must be an integer greater than 1000.<br/>Engine:  The database engine to use for the new instance. Default: The same as source Constraint: Must be compatible with the engine of the source. For example, you can restore a MariaDB 10.1 DB instance from a MySQL 5.6 snapshot. Valid Values: <ul> <li>  <code>mariadb</code>  </li> <li>  <code>mysql</code>  </li> <li>  <code>oracle-ee</code>  </li> <li>  <code>oracle-se2</code>  </li> <li>  <code>oracle-se1</code>  </li> <li>  <code>oracle-se</code>  </li> <li>  <code>postgres</code>  </li> <li>  <code>sqlserver-ee</code>  </li> <li>  <code>sqlserver-se</code>  </li> <li>  <code>sqlserver-ex</code>  </li> <li>  <code>sqlserver-web</code>  </li> </ul><br/>DBName:  The database name for the restored DB instance. <note> This parameter doesn't apply to the MySQL, PostgreSQL, or MariaDB engines. </note><br/>LicenseModel:  License model information for the restored DB instance. Default: Same as source.  Valid values: <code>license-included</code> | <code>bring-your-own-license</code> | <code>general-public-license</code> <br/>AutoMinorVersionUpgrade:  A value that indicates whether minor version upgrades are applied automatically to the DB instance during the maintenance window.<br/>PubliclyAccessible:  A value that indicates whether the DB instance is publicly accessible. When the DB instance is publicly accessible, it is an Internet-facing instance with a publicly resolvable DNS name, which resolves to a public IP address. When the DB instance is not publicly accessible, it is an internal instance with a DNS name that resolves to a private IP address. For more information, see <a>CreateDBInstance</a>.<br/>MultiAZ:  A value that indicates whether the DB instance is a Multi-AZ deployment. Constraint: You can't specify the <code>AvailabilityZone</code> parameter if the DB instance is a Multi-AZ deployment.<br/>DBSubnetGroupName:  The DB subnet group name to use for the new instance. Constraints: If supplied, must match the name of an existing DBSubnetGroup. Example: <code>mySubnetgroup</code> <br/>AvailabilityZone:  The Availability Zone (AZ) where the DB instance will be created. Default: A random, system-chosen Availability Zone. Constraint: You can't specify the <code>AvailabilityZone</code> parameter if the DB instance is a Multi-AZ deployment. Example: <code>us-east-1a</code> <br/>Port:  The port number on which the database accepts connections. Default: The same port as the original DB instance Constraints: Value must be <code>1150-65535</code> <br/>DBInstanceClass:  The compute and memory capacity of the Amazon RDS DB instance, for example, <code>db.m4.large</code>. Not all DB instance classes are available in all AWS Regions, or for all database engines. For the full list of DB instance classes, and availability for your engine, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/Concepts.DBInstanceClass.html">DB Instance Class</a> in the <i>Amazon RDS User Guide.</i>  Default: The same DBInstanceClass as the original DB instance.<br/>DBSnapshotIdentifier:  The identifier for the DB snapshot to restore from. Constraints: <ul> <li> Must match the identifier of an existing DBSnapshot. </li> <li> If you are restoring from a shared manual DB snapshot, the <code>DBSnapshotIdentifier</code> must be the ARN of the shared DB snapshot. </li> </ul><br/>DBInstanceIdentifier:  Name of the DB instance to create from the DB snapshot. This parameter isn't case-sensitive. Constraints: <ul> <li> Must contain from 1 to 63 numbers, letters, or hyphens </li> <li> First character must be a letter </li> <li> Can't end with a hyphen or contain two consecutive hyphens </li> </ul> Example: <code>my-snapshot-id</code> <br/>
        :return: DBInstance:  Contains the details of an Amazon RDS DB instance.  This data type is used as a response element in the <code>DescribeDBInstances</code> action. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxAllocatedStorage</i> The upper limit to which Amazon RDS can automatically scale the storage of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ListenerEndpoint</i> Specifies the listener connection endpoint for SQL Server Always On.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ListenerEndpoint</b>: Specifies the listener connection endpoint for SQL Server Always On.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i>  The AWS Identity and Access Management (IAM) roles associated with the DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i> Indicates if the DB instance has deletion protection enabled. The database can't be deleted when deletion protection is enabled. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_DeleteInstance.html"> Deleting a DB Instance</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessorFeatures</i> The number of CPU cores and the number of threads per core for the DB instance class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB instance is configured to export to CloudWatch Logs. Log types vary by DB engine. For information about the log types for each DB engine, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_LogAccess.html">Amazon RDS Database Log Files</a> in the <i>Amazon RDS User Guide.</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsRetentionPeriod</i> The amount of time, in days, to retain Performance Insights data. Valid values are 7 or 731 (2 years). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsKMSKeyId</i> The AWS KMS key identifier for encryption of Performance Insights data. The KMS key ID is the Amazon Resource Name (ARN), KMS key identifier, or the KMS key alias for the KMS encryption key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsEnabled</i> True if Performance Insights is enabled for the DB instance, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled, and otherwise false. IAM database authentication can be enabled for the following database engines <ul> <li> For MySQL 5.6, minor version 5.6.34 or higher </li> <li> For MySQL 5.7, minor version 5.7.16 or higher </li> <li> Aurora 5.6 or higher. To enable IAM database authentication for Aurora, see DBCluster Type. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timezone</i> The time zone of the DB instance. In most cases, the <code>Timezone</code> element is empty. <code>Timezone</code> content appears only for Microsoft SQL Server DB instances that were created with a time zone specified. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceArn</i> The Amazon Resource Name (ARN) for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PromotionTier</i> A value that specifies the order in which an Aurora Replica is promoted to the primary instance after a failure of the existing primary instance. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/Aurora.Managing.Backups.html#Aurora.Managing.FaultTolerance"> Fault Tolerance for an Aurora DB Cluster</a> in the <i>Amazon Aurora User Guide</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringRoleArn</i> The ARN for the IAM role that permits RDS to send Enhanced Monitoring metrics to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedMonitoringResourceArn</i> The Amazon Resource Name (ARN) of the Amazon CloudWatch Logs log stream that receives the Enhanced Monitoring metrics data for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringInterval</i> The interval, in seconds, between points when Enhanced Monitoring metrics are collected for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyTagsToSnapshot</i> Specifies whether tags are copied from the DB instance to snapshots of the DB instance.  <b>Amazon Aurora</b>  Not applicable. Copying tags to snapshots is managed by the DB cluster. Setting this value for an Aurora DB instance has no effect on the DB cluster setting. For more information, see <code>DBCluster</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainMemberships</i> The Active Directory Domain membership records associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> The identifier of the CA certificate for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbiResourceId</i> The AWS Region-unique, immutable identifier for the DB instance. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB instance is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i>  If <code>StorageEncrypted</code> is true, the AWS KMS key identifier for the encrypted DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB instance is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> If the DB instance is a member of a DB cluster, contains the name of the DB cluster that the DB instance is a member of.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbInstancePort</i> Specifies the port that the DB instance listens on. If the DB instance is part of a DB cluster, this can be a different port than the DB cluster port.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TdeCredentialArn</i> The ARN from the key store with which the instance is associated for TDE encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type associated with DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusInfos</i> The status of a Read Replica. If the instance is not a Read Replica, this is blank.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> Specifies the accessibility options for the DB instance. A value of true specifies an Internet-facing instance with a publicly resolvable DNS name, which resolves to a public IP address. A value of false specifies an internal instance with a DNS name that resolves to a private IP address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondaryAvailabilityZone</i> If present, specifies the name of the secondary Availability Zone for a DB instance with multi-AZ support.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this instance is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionGroupMemberships</i> Provides the list of option group memberships for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the Provisioned IOPS (I/O operations per second) value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> License model information for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBClusterIdentifiers</i> Contains one or more identifiers of Aurora DB clusters to which the RDS DB instance is replicated as a Read Replica. For example, when you create an Aurora Read Replica of an RDS MySQL DB instance, the Aurora MySQL DB cluster for the Aurora Read Replica is shown. This output does not contain information about cross region Aurora Read Replicas. <note> Currently, each RDS DB instance can have only one Aurora Read Replica. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBInstanceIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaSourceDBInstanceIdentifier</i> Contains the identifier of the source DB instance if this DB instance is a Read Replica.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> Indicates that minor version patches are applied automatically.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies if the DB instance is a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DBSubnetGroup</b>: Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> Specifies the name of the Availability Zone the DB instance is located in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroups</i> Provides the list of DB parameter groups applied to this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security group elements that the DB instance belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSecurityGroups</i>  A list of DB security group elements containing <code>DBSecurityGroup.Name</code> and <code>DBSecurityGroup.Status</code> subelements. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i>  Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCreateTime</i> Provides the date and time the DB instance was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> Specifies the allocated storage size specified in gibibytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: Specifies the connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The meaning of this parameter differs according to the database engine you use.  <b>MySQL, MariaDB, SQL Server, PostgreSQL</b>  Contains the name of the initial database of this instance that was provided at create time, if one was specified when the DB instance was created. This same name is returned for the life of the DB instance. Type: String  <b>Oracle</b>  Contains the Oracle System ID (SID) of the created DB instance. Not shown when the returned parameters do not apply to an Oracle DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceStatus</i> Specifies the current state of this database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i> Contains the name of the compute and memory capacity class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i> Contains a user-supplied database identifier. This identifier is the unique key that identifies a DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupArn</i> The Amazon Resource Name (ARN) for the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i>  Contains a list of <code>Subnet</code> elements. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> Provides the status of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VpcId of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupDescription</i> Provides the description of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The name of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The new DB subnet group for the DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> Specifies the identifier of the CA certificate for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type to be associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i>  Contains the new <code>DBInstanceIdentifier</code> for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the new Provisioned IOPS value for the DB instance that will be applied or is currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> The license model for the DB instance. Valid values: <code>license-included</code> | <code>bring-your-own-license</code> | <code>general-public-license</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Indicates that the Single-AZ DB instance is to change to a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the pending number of days for which automated backups are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the pending port for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> Contains the pending or currently-in-progress change of the master credentials for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i>  Contains the new <code>AllocatedStorage</code> size for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i>  Contains the new <code>DBInstanceClass</code> for the DB instance that will be applied or is currently being applied. <br/>A list of the log types whose configuration is still pending. In other words, these log types are in the process of being activated or deactivated.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingCloudwatchLogsExports</b>: A list of the log types whose configuration is still pending. In other words, these log types are in the process of being activated or deactivated.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>        """
        pass

    def restore_db_instance_from_s3(self, S3IngestionRoleArn: str, S3BucketName: str, SourceEngineVersion: str, SourceEngine: str, Engine: str, DBInstanceClass: str, DBInstanceIdentifier: str, DBName: str=None, AllocatedStorage: int=None, MasterUsername: str=None, MasterUserPassword: str=None, DBSecurityGroups: List[String]=None, VpcSecurityGroupIds: List[String]=None, AvailabilityZone: str=None, DBSubnetGroupName: str=None, PreferredMaintenanceWindow: str=None, DBParameterGroupName: str=None, BackupRetentionPeriod: int=None, PreferredBackupWindow: str=None, Port: int=None, MultiAZ: BooleanOptional=None, EngineVersion: str=None, AutoMinorVersionUpgrade: BooleanOptional=None, LicenseModel: str=None, Iops: int=None, OptionGroupName: str=None, PubliclyAccessible: BooleanOptional=None, Tags: List[Tag]=None, StorageType: str=None, StorageEncrypted: BooleanOptional=None, KmsKeyId: str=None, CopyTagsToSnapshot: BooleanOptional=None, MonitoringInterval: int=None, MonitoringRoleArn: str=None, EnableIAMDatabaseAuthentication: BooleanOptional=None, S3Prefix: str=None, EnablePerformanceInsights: BooleanOptional=None, PerformanceInsightsKMSKeyId: str=None, PerformanceInsightsRetentionPeriod: int=None, EnableCloudwatchLogsExports: List[String]=None, ProcessorFeatures: List[ProcessorFeature]=None, UseDefaultProcessorFeatures: BooleanOptional=None, DeletionProtection: BooleanOptional=None) -> RestoreDBInstanceFromS3Result:
        """Amazon Relational Database Service (Amazon RDS) supports importing MySQL databases by using backup files. You can create a backup of your on-premises database, store it on Amazon Simple Storage Service (Amazon S3), and then restore the backup file onto a new Amazon RDS DB instance running MySQL. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/MySQL.Procedural.Importing.html">Importing Data into an Amazon RDS MySQL DB Instance</a> in the <i>Amazon RDS User Guide.</i> 

        :param self:
        :param DeletionProtection:  A value that indicates whether the DB instance has deletion protection enabled. The database can't be deleted when deletion protection is enabled. By default, deletion protection is disabled. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_DeleteInstance.html"> Deleting a DB Instance</a>. <br/>UseDefaultProcessorFeatures:  A value that indicates whether the DB instance class of the DB instance uses its default processor features.<br/>ProcessorFeatures:  The number of CPU cores and the number of threads per core for the DB instance class of the DB instance.<br/>EnableCloudwatchLogsExports:  The list of logs that the restored DB instance is to export to CloudWatch Logs. The values in the list depend on the DB engine being used. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_LogAccess.html#USER_LogAccess.Procedural.UploadtoCloudWatch">Publishing Database Logs to Amazon CloudWatch Logs</a> in the <i>Amazon RDS User Guide</i>.<br/>PerformanceInsightsRetentionPeriod:  The amount of time, in days, to retain Performance Insights data. Valid values are 7 or 731 (2 years). <br/>PerformanceInsightsKMSKeyId:  The AWS KMS key identifier for encryption of Performance Insights data. The KMS key ID is the Amazon Resource Name (ARN), the KMS key identifier, or the KMS key alias for the KMS encryption key.  If you do not specify a value for <code>PerformanceInsightsKMSKeyId</code>, then Amazon RDS uses your default encryption key. AWS KMS creates the default encryption key for your AWS account. Your AWS account has a different default encryption key for each AWS Region.<br/>EnablePerformanceInsights:  A value that indicates whether to enable Performance Insights for the DB instance.  For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_PerfInsights.html">Using Amazon Performance Insights</a> in the <i>Amazon Relational Database Service User Guide</i>. <br/>S3IngestionRoleArn:  An AWS Identity and Access Management (IAM) role to allow Amazon RDS to access your Amazon S3 bucket. <br/>S3Prefix:  The prefix of your Amazon S3 bucket. <br/>S3BucketName:  The name of your Amazon S3 bucket that contains your database backup file. <br/>SourceEngineVersion:  The engine version of your source database.  Valid Values: <code>5.6</code> <br/>SourceEngine:  The name of the engine of your source database.  Valid Values: <code>mysql</code> <br/>EnableIAMDatabaseAuthentication:  A value that indicates whether to enable mapping of AWS Identity and Access Management (IAM) accounts to database accounts. By default, mapping is disabled.<br/>MonitoringRoleArn:  The ARN for the IAM role that permits RDS to send enhanced monitoring metrics to Amazon CloudWatch Logs. For example, <code>arn:aws:iam:123456789012:role/emaccess</code>. For information on creating a monitoring role, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Monitoring.OS.html#USER_Monitoring.OS.Enabling">Setting Up and Enabling Enhanced Monitoring</a> in the <i>Amazon RDS User Guide.</i>  If <code>MonitoringInterval</code> is set to a value other than 0, then you must supply a <code>MonitoringRoleArn</code> value. <br/>MonitoringInterval:  The interval, in seconds, between points when Enhanced Monitoring metrics are collected for the DB instance. To disable collecting Enhanced Monitoring metrics, specify 0.  If <code>MonitoringRoleArn</code> is specified, then you must also set <code>MonitoringInterval</code> to a value other than 0.  Valid Values: 0, 1, 5, 10, 15, 30, 60  Default: <code>0</code> <br/>CopyTagsToSnapshot:  A value that indicates whether to copy all tags from the DB instance to snapshots of the DB instance. By default, tags are not copied. <br/>KmsKeyId:  The AWS KMS key identifier for an encrypted DB instance.  The KMS key identifier is the Amazon Resource Name (ARN) for the KMS encryption key. If you are creating a DB instance with the same AWS account that owns the KMS encryption key used to encrypt the new DB instance, then you can use the KMS key alias instead of the ARN for the KM encryption key.  If the <code>StorageEncrypted</code> parameter is enabled, and you do not specify a value for the <code>KmsKeyId</code> parameter, then Amazon RDS will use your default encryption key. AWS KMS creates the default encryption key for your AWS account. Your AWS account has a different default encryption key for each AWS Region. <br/>StorageEncrypted:  A value that indicates whether the new DB instance is encrypted or not. <br/>StorageType:  Specifies the storage type to be associated with the DB instance.  Valid values: <code>standard</code> | <code>gp2</code> | <code>io1</code>  If you specify <code>io1</code>, you must also include a value for the <code>Iops</code> parameter.  Default: <code>io1</code> if the <code>Iops</code> parameter is specified; otherwise <code>gp2</code> <br/>Tags:  A list of tags to associate with this DB instance. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Tagging.html">Tagging Amazon RDS Resources</a> in the <i>Amazon RDS User Guide.</i> <br/>PubliclyAccessible:  A value that indicates whether the DB instance is publicly accessible. When the DB instance is publicly accessible, it is an Internet-facing instance with a publicly resolvable DNS name, which resolves to a public IP address. When the DB instance is not publicly accessible, it is an internal instance with a DNS name that resolves to a private IP address. For more information, see <a>CreateDBInstance</a>.<br/>OptionGroupName:  The name of the option group to associate with this DB instance. If this argument is omitted, the default option group for the specified engine is used. <br/>Iops:  The amount of Provisioned IOPS (input/output operations per second) to allocate initially for the DB instance. For information about valid Iops values, see see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/CHAP_Storage.html#USER_PIOPS">Amazon RDS Provisioned IOPS Storage to Improve Performance</a> in the <i>Amazon RDS User Guide.</i> <br/>LicenseModel:  The license model for this DB instance. Use <code>general-public-license</code>. <br/>AutoMinorVersionUpgrade:  A value that indicates whether minor engine upgrades are applied automatically to the DB instance during the maintenance window. By default, minor engine upgrades are not applied automatically. <br/>EngineVersion:  The version number of the database engine to use. Choose the latest minor version of your database engine. For information about engine versions, see <code>CreateDBInstance</code>, or call <code>DescribeDBEngineVersions</code>. <br/>MultiAZ:  A value that indicates whether the DB instance is a Multi-AZ deployment. If the DB instance is a Multi-AZ deployment, you can't set the <code>AvailabilityZone</code> parameter. <br/>Port:  The port number on which the database accepts connections.  Type: Integer  Valid Values: <code>1150</code>-<code>65535</code>  Default: <code>3306</code> <br/>PreferredBackupWindow:  The time range each day during which automated backups are created if automated backups are enabled. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_WorkingWithAutomatedBackups.html#USER_WorkingWithAutomatedBackups.BackupWindow">The Backup Window</a> in the <i>Amazon RDS User Guide.</i>  Constraints: <ul> <li> Must be in the format <code>hh24:mi-hh24:mi</code>. </li> <li> Must be in Universal Coordinated Time (UTC). </li> <li> Must not conflict with the preferred maintenance window. </li> <li> Must be at least 30 minutes. </li> </ul><br/>BackupRetentionPeriod:  The number of days for which automated backups are retained. Setting this parameter to a positive number enables backups. For more information, see <code>CreateDBInstance</code>. <br/>DBParameterGroupName:  The name of the DB parameter group to associate with this DB instance. If this argument is omitted, the default parameter group for the specified engine is used. <br/>PreferredMaintenanceWindow:  The time range each week during which system maintenance can occur, in Universal Coordinated Time (UTC). For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_UpgradeDBInstance.Maintenance.html#Concepts.DBMaintenance">Amazon RDS Maintenance Window</a> in the <i>Amazon RDS User Guide.</i>  Constraints: <ul> <li> Must be in the format <code>ddd:hh24:mi-ddd:hh24:mi</code>. </li> <li> Valid Days: Mon, Tue, Wed, Thu, Fri, Sat, Sun. </li> <li> Must be in Universal Coordinated Time (UTC). </li> <li> Must not conflict with the preferred backup window. </li> <li> Must be at least 30 minutes. </li> </ul><br/>DBSubnetGroupName:  A DB subnet group to associate with this DB instance.<br/>AvailabilityZone:  The Availability Zone that the DB instance is created in. For information about AWS Regions and Availability Zones, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/Concepts.RegionsAndAvailabilityZones.html">Regions and Availability Zones</a> in the <i>Amazon RDS User Guide.</i>  Default: A random, system-chosen Availability Zone in the endpoint's AWS Region.   Example: <code>us-east-1d</code>  Constraint: The <code>AvailabilityZone</code> parameter can't be specified if the DB instance is a Multi-AZ deployment. The specified Availability Zone must be in the same AWS Region as the current endpoint. <br/>VpcSecurityGroupIds:  A list of VPC security groups to associate with this DB instance. <br/>DBSecurityGroups:  A list of DB security groups to associate with this DB instance. Default: The default DB security group for the database engine.<br/>MasterUserPassword:  The password for the master user. The password can include any printable ASCII character except "/", triple-quotes, or "@".  Constraints: Must contain from 8 to 41 characters.<br/>MasterUsername:  The name for the master user.  Constraints:  <ul> <li> Must be 1 to 16 letters or numbers. </li> <li> First character must be a letter. </li> <li> Can't be a reserved word for the chosen database engine. </li> </ul><br/>Engine:  The name of the database engine to be used for this instance.  Valid Values: <code>mysql</code> <br/>DBInstanceClass:  The compute and memory capacity of the DB instance, for example, <code>db.m4.large</code>. Not all DB instance classes are available in all AWS Regions, or for all database engines. For the full list of DB instance classes, and availability for your engine, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/Concepts.DBInstanceClass.html">DB Instance Class</a> in the <i>Amazon RDS User Guide.</i>  Importing from Amazon S3 is not supported on the db.t2.micro DB instance class. <br/>AllocatedStorage:  The amount of storage (in gigabytes) to allocate initially for the DB instance. Follow the allocation rules specified in <code>CreateDBInstance</code>.  <note> Be sure to allocate enough memory for your new DB instance so that the restore operation can succeed. You can also allocate additional memory for future growth.  </note><br/>DBInstanceIdentifier:  The DB instance identifier. This parameter is stored as a lowercase string.  Constraints: <ul> <li> Must contain from 1 to 63 letters, numbers, or hyphens. </li> <li> First character must be a letter. </li> <li> Can't end with a hyphen or contain two consecutive hyphens. </li> </ul> Example: <code>mydbinstance</code> <br/>DBName:  The name of the database to create when the DB instance is created. Follow the naming rules specified in <code>CreateDBInstance</code>. <br/>
        :return: DBInstance:  Contains the details of an Amazon RDS DB instance.  This data type is used as a response element in the <code>DescribeDBInstances</code> action. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxAllocatedStorage</i> The upper limit to which Amazon RDS can automatically scale the storage of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ListenerEndpoint</i> Specifies the listener connection endpoint for SQL Server Always On.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ListenerEndpoint</b>: Specifies the listener connection endpoint for SQL Server Always On.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i>  The AWS Identity and Access Management (IAM) roles associated with the DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i> Indicates if the DB instance has deletion protection enabled. The database can't be deleted when deletion protection is enabled. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_DeleteInstance.html"> Deleting a DB Instance</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessorFeatures</i> The number of CPU cores and the number of threads per core for the DB instance class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB instance is configured to export to CloudWatch Logs. Log types vary by DB engine. For information about the log types for each DB engine, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_LogAccess.html">Amazon RDS Database Log Files</a> in the <i>Amazon RDS User Guide.</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsRetentionPeriod</i> The amount of time, in days, to retain Performance Insights data. Valid values are 7 or 731 (2 years). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsKMSKeyId</i> The AWS KMS key identifier for encryption of Performance Insights data. The KMS key ID is the Amazon Resource Name (ARN), KMS key identifier, or the KMS key alias for the KMS encryption key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsEnabled</i> True if Performance Insights is enabled for the DB instance, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled, and otherwise false. IAM database authentication can be enabled for the following database engines <ul> <li> For MySQL 5.6, minor version 5.6.34 or higher </li> <li> For MySQL 5.7, minor version 5.7.16 or higher </li> <li> Aurora 5.6 or higher. To enable IAM database authentication for Aurora, see DBCluster Type. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timezone</i> The time zone of the DB instance. In most cases, the <code>Timezone</code> element is empty. <code>Timezone</code> content appears only for Microsoft SQL Server DB instances that were created with a time zone specified. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceArn</i> The Amazon Resource Name (ARN) for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PromotionTier</i> A value that specifies the order in which an Aurora Replica is promoted to the primary instance after a failure of the existing primary instance. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/Aurora.Managing.Backups.html#Aurora.Managing.FaultTolerance"> Fault Tolerance for an Aurora DB Cluster</a> in the <i>Amazon Aurora User Guide</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringRoleArn</i> The ARN for the IAM role that permits RDS to send Enhanced Monitoring metrics to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedMonitoringResourceArn</i> The Amazon Resource Name (ARN) of the Amazon CloudWatch Logs log stream that receives the Enhanced Monitoring metrics data for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringInterval</i> The interval, in seconds, between points when Enhanced Monitoring metrics are collected for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyTagsToSnapshot</i> Specifies whether tags are copied from the DB instance to snapshots of the DB instance.  <b>Amazon Aurora</b>  Not applicable. Copying tags to snapshots is managed by the DB cluster. Setting this value for an Aurora DB instance has no effect on the DB cluster setting. For more information, see <code>DBCluster</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainMemberships</i> The Active Directory Domain membership records associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> The identifier of the CA certificate for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbiResourceId</i> The AWS Region-unique, immutable identifier for the DB instance. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB instance is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i>  If <code>StorageEncrypted</code> is true, the AWS KMS key identifier for the encrypted DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB instance is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> If the DB instance is a member of a DB cluster, contains the name of the DB cluster that the DB instance is a member of.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbInstancePort</i> Specifies the port that the DB instance listens on. If the DB instance is part of a DB cluster, this can be a different port than the DB cluster port.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TdeCredentialArn</i> The ARN from the key store with which the instance is associated for TDE encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type associated with DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusInfos</i> The status of a Read Replica. If the instance is not a Read Replica, this is blank.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> Specifies the accessibility options for the DB instance. A value of true specifies an Internet-facing instance with a publicly resolvable DNS name, which resolves to a public IP address. A value of false specifies an internal instance with a DNS name that resolves to a private IP address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondaryAvailabilityZone</i> If present, specifies the name of the secondary Availability Zone for a DB instance with multi-AZ support.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this instance is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionGroupMemberships</i> Provides the list of option group memberships for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the Provisioned IOPS (I/O operations per second) value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> License model information for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBClusterIdentifiers</i> Contains one or more identifiers of Aurora DB clusters to which the RDS DB instance is replicated as a Read Replica. For example, when you create an Aurora Read Replica of an RDS MySQL DB instance, the Aurora MySQL DB cluster for the Aurora Read Replica is shown. This output does not contain information about cross region Aurora Read Replicas. <note> Currently, each RDS DB instance can have only one Aurora Read Replica. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBInstanceIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaSourceDBInstanceIdentifier</i> Contains the identifier of the source DB instance if this DB instance is a Read Replica.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> Indicates that minor version patches are applied automatically.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies if the DB instance is a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DBSubnetGroup</b>: Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> Specifies the name of the Availability Zone the DB instance is located in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroups</i> Provides the list of DB parameter groups applied to this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security group elements that the DB instance belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSecurityGroups</i>  A list of DB security group elements containing <code>DBSecurityGroup.Name</code> and <code>DBSecurityGroup.Status</code> subelements. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i>  Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCreateTime</i> Provides the date and time the DB instance was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> Specifies the allocated storage size specified in gibibytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: Specifies the connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The meaning of this parameter differs according to the database engine you use.  <b>MySQL, MariaDB, SQL Server, PostgreSQL</b>  Contains the name of the initial database of this instance that was provided at create time, if one was specified when the DB instance was created. This same name is returned for the life of the DB instance. Type: String  <b>Oracle</b>  Contains the Oracle System ID (SID) of the created DB instance. Not shown when the returned parameters do not apply to an Oracle DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceStatus</i> Specifies the current state of this database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i> Contains the name of the compute and memory capacity class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i> Contains a user-supplied database identifier. This identifier is the unique key that identifies a DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupArn</i> The Amazon Resource Name (ARN) for the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i>  Contains a list of <code>Subnet</code> elements. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> Provides the status of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VpcId of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupDescription</i> Provides the description of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The name of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The new DB subnet group for the DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> Specifies the identifier of the CA certificate for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type to be associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i>  Contains the new <code>DBInstanceIdentifier</code> for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the new Provisioned IOPS value for the DB instance that will be applied or is currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> The license model for the DB instance. Valid values: <code>license-included</code> | <code>bring-your-own-license</code> | <code>general-public-license</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Indicates that the Single-AZ DB instance is to change to a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the pending number of days for which automated backups are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the pending port for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> Contains the pending or currently-in-progress change of the master credentials for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i>  Contains the new <code>AllocatedStorage</code> size for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i>  Contains the new <code>DBInstanceClass</code> for the DB instance that will be applied or is currently being applied. <br/>A list of the log types whose configuration is still pending. In other words, these log types are in the process of being activated or deactivated.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingCloudwatchLogsExports</b>: A list of the log types whose configuration is still pending. In other words, these log types are in the process of being activated or deactivated.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>        """
        pass

    def restore_db_instance_to_point_in_time(self, TargetDBInstanceIdentifier: str, SourceDBInstanceIdentifier: str=None, RestoreTime: TStamp=None, UseLatestRestorableTime: Boolean=None, DBInstanceClass: str=None, Port: int=None, AvailabilityZone: str=None, DBSubnetGroupName: str=None, MultiAZ: BooleanOptional=None, PubliclyAccessible: BooleanOptional=None, AutoMinorVersionUpgrade: BooleanOptional=None, LicenseModel: str=None, DBName: str=None, Engine: str=None, Iops: int=None, OptionGroupName: str=None, CopyTagsToSnapshot: BooleanOptional=None, Tags: List[Tag]=None, StorageType: str=None, TdeCredentialArn: str=None, TdeCredentialPassword: str=None, VpcSecurityGroupIds: List[String]=None, Domain: str=None, DomainIAMRoleName: str=None, EnableIAMDatabaseAuthentication: BooleanOptional=None, EnableCloudwatchLogsExports: List[String]=None, ProcessorFeatures: List[ProcessorFeature]=None, UseDefaultProcessorFeatures: BooleanOptional=None, DBParameterGroupName: str=None, DeletionProtection: BooleanOptional=None, SourceDbiResourceId: str=None) -> RestoreDBInstanceToPointInTimeResult:
        """Restores a DB instance to an arbitrary point in time. You can restore to any point in time before the time identified by the LatestRestorableTime property. You can restore to a point up to the number of days specified by the BackupRetentionPeriod property. The target database is created with most of the original configuration, but in a system-selected Availability Zone, with the default security group, the default subnet group, and the default DB parameter group. By default, the new DB instance is created as a single-AZ deployment except when the instance is a SQL Server instance that has an option group that is associated with mirroring; in this case, the instance becomes a mirrored deployment and not a single-AZ deployment. <note> This command doesn't apply to Aurora MySQL and Aurora PostgreSQL. For Aurora, use <code>RestoreDBClusterToPointInTime</code>. </note>

        :param self:
        :param SourceDbiResourceId:  The resource ID of the source DB instance from which to restore.<br/>DeletionProtection:  A value that indicates whether the DB instance has deletion protection enabled. The database can't be deleted when deletion protection is enabled. By default, deletion protection is disabled. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_DeleteInstance.html"> Deleting a DB Instance</a>. <br/>DBParameterGroupName:  The name of the DB parameter group to associate with this DB instance. If this argument is omitted, the default DBParameterGroup for the specified engine is used. Constraints: <ul> <li> If supplied, must match the name of an existing DBParameterGroup. </li> <li> Must be 1 to 255 letters, numbers, or hyphens. </li> <li> First character must be a letter. </li> <li> Can't end with a hyphen or contain two consecutive hyphens. </li> </ul><br/>UseDefaultProcessorFeatures:  A value that indicates whether the DB instance class of the DB instance uses its default processor features.<br/>ProcessorFeatures:  The number of CPU cores and the number of threads per core for the DB instance class of the DB instance.<br/>EnableCloudwatchLogsExports:  The list of logs that the restored DB instance is to export to CloudWatch Logs. The values in the list depend on the DB engine being used. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_LogAccess.html#USER_LogAccess.Procedural.UploadtoCloudWatch">Publishing Database Logs to Amazon CloudWatch Logs</a> in the <i>Amazon RDS User Guide</i>.<br/>EnableIAMDatabaseAuthentication:  A value that indicates whether to enable mapping of AWS Identity and Access Management (IAM) accounts to database accounts. By default, mapping is disabled. You can enable IAM database authentication for the following database engines <ul> <li> For MySQL 5.6, minor version 5.6.34 or higher </li> <li> For MySQL 5.7, minor version 5.7.16 or higher </li> </ul><br/>DomainIAMRoleName:  Specify the name of the IAM role to be used when making API calls to the Directory Service.<br/>Domain:  Specify the Active Directory Domain to restore the instance in.<br/>VpcSecurityGroupIds:   A list of EC2 VPC security groups to associate with this DB instance.   Default: The default EC2 VPC security group for the DB subnet group's VPC. <br/>TdeCredentialPassword:  The password for the given ARN from the key store in order to access the device.<br/>TdeCredentialArn:  The ARN from the key store with which to associate the instance for TDE encryption.<br/>StorageType:  Specifies the storage type to be associated with the DB instance.  Valid values: <code>standard | gp2 | io1</code>   If you specify <code>io1</code>, you must also include a value for the <code>Iops</code> parameter.   Default: <code>io1</code> if the <code>Iops</code> parameter is specified, otherwise <code>gp2</code> <br/>Tags:  A list of tags. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Tagging.html">Tagging Amazon RDS Resources</a> in the <i>Amazon RDS User Guide.</i> <br/>CopyTagsToSnapshot:  A value that indicates whether to copy all tags from the restored DB instance to snapshots of the DB instance. By default, tags are not copied.<br/>OptionGroupName:  The name of the option group to be used for the restored DB instance. Permanent options, such as the TDE option for Oracle Advanced Security TDE, can't be removed from an option group, and that option group can't be removed from a DB instance once it is associated with a DB instance<br/>Iops:  The amount of Provisioned IOPS (input/output operations per second) to be initially allocated for the DB instance. Constraints: Must be an integer greater than 1000.  <b>SQL Server</b>  Setting the IOPS value for the SQL Server database engine is not supported.<br/>Engine:  The database engine to use for the new instance. Default: The same as source Constraint: Must be compatible with the engine of the source Valid Values: <ul> <li>  <code>mariadb</code>  </li> <li>  <code>mysql</code>  </li> <li>  <code>oracle-ee</code>  </li> <li>  <code>oracle-se2</code>  </li> <li>  <code>oracle-se1</code>  </li> <li>  <code>oracle-se</code>  </li> <li>  <code>postgres</code>  </li> <li>  <code>sqlserver-ee</code>  </li> <li>  <code>sqlserver-se</code>  </li> <li>  <code>sqlserver-ex</code>  </li> <li>  <code>sqlserver-web</code>  </li> </ul><br/>DBName:  The database name for the restored DB instance. <note> This parameter is not used for the MySQL or MariaDB engines. </note><br/>LicenseModel:  License model information for the restored DB instance. Default: Same as source.  Valid values: <code>license-included</code> | <code>bring-your-own-license</code> | <code>general-public-license</code> <br/>AutoMinorVersionUpgrade:  A value that indicates whether minor version upgrades are applied automatically to the DB instance during the maintenance window.<br/>PubliclyAccessible:  A value that indicates whether the DB instance is publicly accessible. When the DB instance is publicly accessible, it is an Internet-facing instance with a publicly resolvable DNS name, which resolves to a public IP address. When the DB instance is not publicly accessible, it is an internal instance with a DNS name that resolves to a private IP address. For more information, see <a>CreateDBInstance</a>.<br/>MultiAZ:  A value that indicates whether the DB instance is a Multi-AZ deployment. Constraint: You can't specify the <code>AvailabilityZone</code> parameter if the DB instance is a Multi-AZ deployment.<br/>DBSubnetGroupName:  The DB subnet group name to use for the new instance. Constraints: If supplied, must match the name of an existing DBSubnetGroup. Example: <code>mySubnetgroup</code> <br/>AvailabilityZone:  The Availability Zone (AZ) where the DB instance will be created. Default: A random, system-chosen Availability Zone. Constraint: You can't specify the <code>AvailabilityZone</code> parameter if the DB instance is a Multi-AZ deployment. Example: <code>us-east-1a</code> <br/>Port:  The port number on which the database accepts connections. Constraints: Value must be <code>1150-65535</code>  Default: The same port as the original DB instance.<br/>DBInstanceClass:  The compute and memory capacity of the Amazon RDS DB instance, for example, <code>db.m4.large</code>. Not all DB instance classes are available in all AWS Regions, or for all database engines. For the full list of DB instance classes, and availability for your engine, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/Concepts.DBInstanceClass.html">DB Instance Class</a> in the <i>Amazon RDS User Guide.</i>  Default: The same DBInstanceClass as the original DB instance.<br/>UseLatestRestorableTime:   A value that indicates whether the DB instance is restored from the latest backup time. By default, the DB instance is not restored from the latest backup time.  Constraints: Can't be specified if the <code>RestoreTime</code> parameter is provided.<br/>RestoreTime:  The date and time to restore from. Valid Values: Value must be a time in Universal Coordinated Time (UTC) format Constraints: <ul> <li> Must be before the latest restorable time for the DB instance </li> <li> Can't be specified if the <code>UseLatestRestorableTime</code> parameter is enabled </li> </ul> Example: <code>2009-09-07T23:45:00Z</code> <br/>TargetDBInstanceIdentifier:  The name of the new DB instance to be created. Constraints: <ul> <li> Must contain from 1 to 63 letters, numbers, or hyphens </li> <li> First character must be a letter </li> <li> Can't end with a hyphen or contain two consecutive hyphens </li> </ul><br/>SourceDBInstanceIdentifier:  The identifier of the source DB instance from which to restore. Constraints: <ul> <li> Must match the identifier of an existing DB instance. </li> </ul><br/>
        :return: DBInstance:  Contains the details of an Amazon RDS DB instance.  This data type is used as a response element in the <code>DescribeDBInstances</code> action. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxAllocatedStorage</i> The upper limit to which Amazon RDS can automatically scale the storage of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ListenerEndpoint</i> Specifies the listener connection endpoint for SQL Server Always On.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ListenerEndpoint</b>: Specifies the listener connection endpoint for SQL Server Always On.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i>  The AWS Identity and Access Management (IAM) roles associated with the DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i> Indicates if the DB instance has deletion protection enabled. The database can't be deleted when deletion protection is enabled. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_DeleteInstance.html"> Deleting a DB Instance</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessorFeatures</i> The number of CPU cores and the number of threads per core for the DB instance class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB instance is configured to export to CloudWatch Logs. Log types vary by DB engine. For information about the log types for each DB engine, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_LogAccess.html">Amazon RDS Database Log Files</a> in the <i>Amazon RDS User Guide.</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsRetentionPeriod</i> The amount of time, in days, to retain Performance Insights data. Valid values are 7 or 731 (2 years). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsKMSKeyId</i> The AWS KMS key identifier for encryption of Performance Insights data. The KMS key ID is the Amazon Resource Name (ARN), KMS key identifier, or the KMS key alias for the KMS encryption key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsEnabled</i> True if Performance Insights is enabled for the DB instance, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled, and otherwise false. IAM database authentication can be enabled for the following database engines <ul> <li> For MySQL 5.6, minor version 5.6.34 or higher </li> <li> For MySQL 5.7, minor version 5.7.16 or higher </li> <li> Aurora 5.6 or higher. To enable IAM database authentication for Aurora, see DBCluster Type. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timezone</i> The time zone of the DB instance. In most cases, the <code>Timezone</code> element is empty. <code>Timezone</code> content appears only for Microsoft SQL Server DB instances that were created with a time zone specified. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceArn</i> The Amazon Resource Name (ARN) for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PromotionTier</i> A value that specifies the order in which an Aurora Replica is promoted to the primary instance after a failure of the existing primary instance. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/Aurora.Managing.Backups.html#Aurora.Managing.FaultTolerance"> Fault Tolerance for an Aurora DB Cluster</a> in the <i>Amazon Aurora User Guide</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringRoleArn</i> The ARN for the IAM role that permits RDS to send Enhanced Monitoring metrics to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedMonitoringResourceArn</i> The Amazon Resource Name (ARN) of the Amazon CloudWatch Logs log stream that receives the Enhanced Monitoring metrics data for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringInterval</i> The interval, in seconds, between points when Enhanced Monitoring metrics are collected for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyTagsToSnapshot</i> Specifies whether tags are copied from the DB instance to snapshots of the DB instance.  <b>Amazon Aurora</b>  Not applicable. Copying tags to snapshots is managed by the DB cluster. Setting this value for an Aurora DB instance has no effect on the DB cluster setting. For more information, see <code>DBCluster</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainMemberships</i> The Active Directory Domain membership records associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> The identifier of the CA certificate for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbiResourceId</i> The AWS Region-unique, immutable identifier for the DB instance. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB instance is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i>  If <code>StorageEncrypted</code> is true, the AWS KMS key identifier for the encrypted DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB instance is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> If the DB instance is a member of a DB cluster, contains the name of the DB cluster that the DB instance is a member of.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbInstancePort</i> Specifies the port that the DB instance listens on. If the DB instance is part of a DB cluster, this can be a different port than the DB cluster port.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TdeCredentialArn</i> The ARN from the key store with which the instance is associated for TDE encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type associated with DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusInfos</i> The status of a Read Replica. If the instance is not a Read Replica, this is blank.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> Specifies the accessibility options for the DB instance. A value of true specifies an Internet-facing instance with a publicly resolvable DNS name, which resolves to a public IP address. A value of false specifies an internal instance with a DNS name that resolves to a private IP address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondaryAvailabilityZone</i> If present, specifies the name of the secondary Availability Zone for a DB instance with multi-AZ support.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this instance is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionGroupMemberships</i> Provides the list of option group memberships for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the Provisioned IOPS (I/O operations per second) value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> License model information for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBClusterIdentifiers</i> Contains one or more identifiers of Aurora DB clusters to which the RDS DB instance is replicated as a Read Replica. For example, when you create an Aurora Read Replica of an RDS MySQL DB instance, the Aurora MySQL DB cluster for the Aurora Read Replica is shown. This output does not contain information about cross region Aurora Read Replicas. <note> Currently, each RDS DB instance can have only one Aurora Read Replica. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBInstanceIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaSourceDBInstanceIdentifier</i> Contains the identifier of the source DB instance if this DB instance is a Read Replica.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> Indicates that minor version patches are applied automatically.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies if the DB instance is a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DBSubnetGroup</b>: Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> Specifies the name of the Availability Zone the DB instance is located in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroups</i> Provides the list of DB parameter groups applied to this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security group elements that the DB instance belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSecurityGroups</i>  A list of DB security group elements containing <code>DBSecurityGroup.Name</code> and <code>DBSecurityGroup.Status</code> subelements. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i>  Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCreateTime</i> Provides the date and time the DB instance was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> Specifies the allocated storage size specified in gibibytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: Specifies the connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The meaning of this parameter differs according to the database engine you use.  <b>MySQL, MariaDB, SQL Server, PostgreSQL</b>  Contains the name of the initial database of this instance that was provided at create time, if one was specified when the DB instance was created. This same name is returned for the life of the DB instance. Type: String  <b>Oracle</b>  Contains the Oracle System ID (SID) of the created DB instance. Not shown when the returned parameters do not apply to an Oracle DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceStatus</i> Specifies the current state of this database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i> Contains the name of the compute and memory capacity class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i> Contains a user-supplied database identifier. This identifier is the unique key that identifies a DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupArn</i> The Amazon Resource Name (ARN) for the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i>  Contains a list of <code>Subnet</code> elements. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> Provides the status of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VpcId of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupDescription</i> Provides the description of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The name of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The new DB subnet group for the DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> Specifies the identifier of the CA certificate for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type to be associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i>  Contains the new <code>DBInstanceIdentifier</code> for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the new Provisioned IOPS value for the DB instance that will be applied or is currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> The license model for the DB instance. Valid values: <code>license-included</code> | <code>bring-your-own-license</code> | <code>general-public-license</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Indicates that the Single-AZ DB instance is to change to a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the pending number of days for which automated backups are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the pending port for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> Contains the pending or currently-in-progress change of the master credentials for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i>  Contains the new <code>AllocatedStorage</code> size for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i>  Contains the new <code>DBInstanceClass</code> for the DB instance that will be applied or is currently being applied. <br/>A list of the log types whose configuration is still pending. In other words, these log types are in the process of being activated or deactivated.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingCloudwatchLogsExports</b>: A list of the log types whose configuration is still pending. In other words, these log types are in the process of being activated or deactivated.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>        """
        pass

    def revoke_db_security_group_ingress(self, DBSecurityGroupName: str, CIDRIP: str=None, EC2SecurityGroupName: str=None, EC2SecurityGroupId: str=None, EC2SecurityGroupOwnerId: str=None) -> RevokeDBSecurityGroupIngressResult:
        """Revokes ingress from a DBSecurityGroup for previously authorized IP ranges or EC2 or VPC Security Groups. Required parameters for this API are one of CIDRIP, EC2SecurityGroupId for VPC, or (EC2SecurityGroupOwnerId and either EC2SecurityGroupName or EC2SecurityGroupId).

        :param self:
        :param EC2SecurityGroupOwnerId:   The AWS Account Number of the owner of the EC2 security group specified in the <code>EC2SecurityGroupName</code> parameter. The AWS Access Key ID is not an acceptable value. For VPC DB security groups, <code>EC2SecurityGroupId</code> must be provided. Otherwise, EC2SecurityGroupOwnerId and either <code>EC2SecurityGroupName</code> or <code>EC2SecurityGroupId</code> must be provided. <br/>EC2SecurityGroupId:   The id of the EC2 security group to revoke access from. For VPC DB security groups, <code>EC2SecurityGroupId</code> must be provided. Otherwise, EC2SecurityGroupOwnerId and either <code>EC2SecurityGroupName</code> or <code>EC2SecurityGroupId</code> must be provided. <br/>EC2SecurityGroupName:   The name of the EC2 security group to revoke access from. For VPC DB security groups, <code>EC2SecurityGroupId</code> must be provided. Otherwise, EC2SecurityGroupOwnerId and either <code>EC2SecurityGroupName</code> or <code>EC2SecurityGroupId</code> must be provided. <br/>CIDRIP:   The IP range to revoke access from. Must be a valid CIDR range. If <code>CIDRIP</code> is specified, <code>EC2SecurityGroupName</code>, <code>EC2SecurityGroupId</code> and <code>EC2SecurityGroupOwnerId</code> can't be provided. <br/>DBSecurityGroupName:  The name of the DB security group to revoke ingress from.<br/>
        :return: DBSecurityGroup:  Contains the details for an Amazon RDS DB security group.  This data type is used as a response element in the <code>DescribeDBSecurityGroups</code> action. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSecurityGroupArn</i> The Amazon Resource Name (ARN) for the DB security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IPRanges</i>  Contains a list of <code>IPRange</code> elements. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EC2SecurityGroups</i>  Contains a list of <code>EC2SecurityGroup</code> elements. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VpcId of the DB security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSecurityGroupDescription</i> Provides the description of the DB security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSecurityGroupName</i> Specifies the name of the DB security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> Provides the AWS ID of the owner of a specific DB security group.<br/>        """
        pass

    def start_activity_stream(self, KmsKeyId: str, Mode: str, ResourceArn: str, ApplyImmediately: BooleanOptional=None) -> StartActivityStreamResponse:
        """Starts a database activity stream to monitor activity on the database. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/DBActivityStreams.html">Database Activity Streams</a> in the <i>Amazon Aurora User Guide</i>.

        :param self:
        :param ApplyImmediately:  Specifies whether or not the database activity stream is to start as soon as possible, regardless of the maintenance window for the database.<br/>KmsKeyId:  The AWS KMS key identifier for encrypting messages in the database activity stream. The key identifier can be either a key ID, a key ARN, or a key alias.<br/>Mode:  Specifies the mode of the database activity stream. Database events such as a change or access generate an activity stream event. The database session can handle these events either synchronously or asynchronously. <br/>ResourceArn:  The Amazon Resource Name (ARN) of the DB cluster, for example <code>arn:aws:rds:us-east-1:12345667890:cluster:das-cluster</code>.<br/>
        :return: ApplyImmediately:  Indicates whether or not the database activity stream will start as soon as possible, regardless of the maintenance window for the database.<br/>Mode:  The mode of the database activity stream.<br/>Status:  The status of the database activity stream.<br/>KinesisStreamName:  The name of the Amazon Kinesis data stream to be used for the database activity stream.<br/>KmsKeyId:  The AWS KMS key identifier for encryption of messages in the database activity stream.<br/>        """
        pass

    def start_db_cluster(self, DBClusterIdentifier: str) -> StartDBClusterResult:
        """Starts an Amazon Aurora DB cluster that was stopped using the AWS console, the stop-db-cluster AWS CLI command, or the StopDBCluster action. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-cluster-stop-start.html"> Stopping and Starting an Aurora Cluster</a> in the <i>Amazon Aurora User Guide.</i>  <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param DBClusterIdentifier:  The DB cluster identifier of the Amazon Aurora DB cluster to be started. This parameter is stored as a lowercase string.<br/>
        :return: DBCluster:  Contains the details of an Amazon Aurora DB cluster.  This data type is used as a response element in the <code>DescribeDBClusters</code>, <code>StopDBCluster</code>, and <code>StartDBCluster</code> actions. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CrossAccountClone</i> Specifies whether the DB cluster is a clone of a DB cluster owned by a different AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyTagsToSnapshot</i> Specifies whether tags are copied from the DB cluster to snapshots of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamKinesisStreamName</i> The name of the Amazon Kinesis data stream used for the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamKmsKeyId</i> The AWS KMS key identifier used for encrypting messages in the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamStatus</i> The status of the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamMode</i> The mode of the database activity stream. Database events such as a change or access generate an activity stream event. The database session can handle these events either synchronously or asynchronously. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HttpEndpointEnabled</i> A value that indicates whether the HTTP endpoint for an Aurora Serverless DB cluster is enabled. When enabled, the HTTP endpoint provides a connectionless web service API for running SQL queries on the Aurora Serverless DB cluster. You can also query your database from inside the RDS console with the query editor. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/data-api.html">Using the Data API for Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i> Indicates if the DB cluster has deletion protection enabled. The database can't be deleted when deletion protection is enabled. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ScalingConfigurationInfo</i> Shows the scaling configuration for an Aurora DB cluster in <code>serverless</code> DB engine mode. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ScalingConfigurationInfo</b>: Shows the scaling configuration for an Aurora DB cluster in <code>serverless</code> DB engine mode. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineMode</i> The DB engine mode of the DB cluster, either <code>provisioned</code>, <code>serverless</code>, or <code>parallelquery</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Capacity</i> The current capacity of an Aurora Serverless DB cluster. The capacity is 0 (zero) when the cluster is paused. For more information about Aurora Serverless, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB cluster is configured to export to CloudWatch Logs. Log types vary by DB engine. For information about the log types for each DB engine, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/USER_LogAccess.html">Amazon RDS Database Log Files</a> in the <i>Amazon Aurora User Guide.</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BacktrackConsumedChangeRecords</i> The number of change records stored for Backtrack.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BacktrackWindow</i> The target backtrack window, in seconds. If this value is set to 0, backtracking is disabled for the DB cluster. Otherwise, backtracking is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestBacktrackTime</i> The earliest time to which a DB cluster can be backtracked.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloneGroupId</i> Identifies the clone group to which the DB cluster is associated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> A value that indicates whether the mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i> Provides a list of the AWS Identity and Access Management (IAM) roles that are associated with the DB cluster. IAM roles that are associated with a DB cluster grant permission for the DB cluster to access other AWS services on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterArn</i> The Amazon Resource Name (ARN) for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbClusterResourceId</i> The AWS Region-unique, immutable identifier for the DB cluster. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB cluster is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is enabled, the AWS KMS key identifier for the encrypted DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security groups that the DB cluster belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterMembers</i> Provides the list of instances that make up the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSourceIdentifier</i> Contains the identifier of the source DB cluster if this DB cluster is a Read Replica.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i> Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterOptionGroupMemberships</i> Provides the list of option group memberships for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies whether the DB cluster has instances in multiple Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomEndpoints</i> Identifies all custom endpoints associated with the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReaderEndpoint</i> The reader endpoint for the DB cluster. The reader endpoint for a DB cluster load-balances connections across the Aurora Replicas that are available in a DB cluster. As clients request new connections to the reader endpoint, Aurora distributes the connection requests among the Aurora Replicas in the DB cluster. This functionality can help balance your read workload across multiple Aurora Replicas in your DB cluster.  If a failover occurs, and the Aurora Replica that you are connected to is promoted to be the primary instance, your connection is dropped. To continue sending your read workload to other Aurora Replicas in the cluster, you can then reconnect to the reader endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint for the primary instance of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestRestorableTime</i> The earliest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the progress of the operation as a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the current state of this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB cluster, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroup</i> Specifies the name of the DB cluster parameter group for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Contains a user-supplied DB cluster identifier. This identifier is the unique key that identifies a DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> Contains the name of the initial database of this DB cluster that was provided at create time, if one was specified when the DB cluster was created. This same name is returned for the life of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this cluster is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of Availability Zones (AZs) where instances in the DB cluster can be created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> For all database engines except Amazon Aurora, <code>AllocatedStorage</code> specifies the allocated storage size in gibibytes (GiB). For Aurora, <code>AllocatedStorage</code> always returns 1, because Aurora DB cluster storage size is not fixed, but instead automatically adjusts as needed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeoutAction</i> The timeout action of a call to <code>ModifyCurrentDBClusterCapacity</code>, either <code>ForceApplyCapacityChange</code> or <code>RollbackCapacityChange</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondsUntilAutoPause</i> The remaining amount of time, in seconds, before the Aurora DB cluster in <code>serverless</code> mode is paused. A DB cluster can be paused only when it's idle (it has no connections).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoPause</i> A value that indicates whether automatic pause is allowed for the Aurora DB cluster in <code>serverless</code> DB engine mode. When the value is set to false for an Aurora Serverless DB cluster, the DB cluster automatically resumes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxCapacity</i> The maximum capacity for an Aurora DB cluster in <code>serverless</code> DB engine mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinCapacity</i> The maximum capacity for the Aurora DB cluster in <code>serverless</code> DB engine mode.<br/>        """
        pass

    def start_db_instance(self, DBInstanceIdentifier: str) -> StartDBInstanceResult:
        """ Starts an Amazon RDS DB instance that was stopped using the AWS console, the stop-db-instance AWS CLI command, or the StopDBInstance action.  For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_StartInstance.html"> Starting an Amazon RDS DB instance That Was Previously Stopped</a> in the <i>Amazon RDS User Guide.</i>  <note>  This command doesn't apply to Aurora MySQL and Aurora PostgreSQL. For Aurora DB clusters, use <code>StartDBCluster</code> instead.  </note>

        :param self:
        :param DBInstanceIdentifier:   The user-supplied instance identifier. <br/>
        :return: DBInstance:  Contains the details of an Amazon RDS DB instance.  This data type is used as a response element in the <code>DescribeDBInstances</code> action. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxAllocatedStorage</i> The upper limit to which Amazon RDS can automatically scale the storage of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ListenerEndpoint</i> Specifies the listener connection endpoint for SQL Server Always On.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ListenerEndpoint</b>: Specifies the listener connection endpoint for SQL Server Always On.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i>  The AWS Identity and Access Management (IAM) roles associated with the DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i> Indicates if the DB instance has deletion protection enabled. The database can't be deleted when deletion protection is enabled. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_DeleteInstance.html"> Deleting a DB Instance</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessorFeatures</i> The number of CPU cores and the number of threads per core for the DB instance class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB instance is configured to export to CloudWatch Logs. Log types vary by DB engine. For information about the log types for each DB engine, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_LogAccess.html">Amazon RDS Database Log Files</a> in the <i>Amazon RDS User Guide.</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsRetentionPeriod</i> The amount of time, in days, to retain Performance Insights data. Valid values are 7 or 731 (2 years). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsKMSKeyId</i> The AWS KMS key identifier for encryption of Performance Insights data. The KMS key ID is the Amazon Resource Name (ARN), KMS key identifier, or the KMS key alias for the KMS encryption key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsEnabled</i> True if Performance Insights is enabled for the DB instance, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled, and otherwise false. IAM database authentication can be enabled for the following database engines <ul> <li> For MySQL 5.6, minor version 5.6.34 or higher </li> <li> For MySQL 5.7, minor version 5.7.16 or higher </li> <li> Aurora 5.6 or higher. To enable IAM database authentication for Aurora, see DBCluster Type. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timezone</i> The time zone of the DB instance. In most cases, the <code>Timezone</code> element is empty. <code>Timezone</code> content appears only for Microsoft SQL Server DB instances that were created with a time zone specified. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceArn</i> The Amazon Resource Name (ARN) for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PromotionTier</i> A value that specifies the order in which an Aurora Replica is promoted to the primary instance after a failure of the existing primary instance. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/Aurora.Managing.Backups.html#Aurora.Managing.FaultTolerance"> Fault Tolerance for an Aurora DB Cluster</a> in the <i>Amazon Aurora User Guide</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringRoleArn</i> The ARN for the IAM role that permits RDS to send Enhanced Monitoring metrics to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedMonitoringResourceArn</i> The Amazon Resource Name (ARN) of the Amazon CloudWatch Logs log stream that receives the Enhanced Monitoring metrics data for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringInterval</i> The interval, in seconds, between points when Enhanced Monitoring metrics are collected for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyTagsToSnapshot</i> Specifies whether tags are copied from the DB instance to snapshots of the DB instance.  <b>Amazon Aurora</b>  Not applicable. Copying tags to snapshots is managed by the DB cluster. Setting this value for an Aurora DB instance has no effect on the DB cluster setting. For more information, see <code>DBCluster</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainMemberships</i> The Active Directory Domain membership records associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> The identifier of the CA certificate for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbiResourceId</i> The AWS Region-unique, immutable identifier for the DB instance. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB instance is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i>  If <code>StorageEncrypted</code> is true, the AWS KMS key identifier for the encrypted DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB instance is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> If the DB instance is a member of a DB cluster, contains the name of the DB cluster that the DB instance is a member of.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbInstancePort</i> Specifies the port that the DB instance listens on. If the DB instance is part of a DB cluster, this can be a different port than the DB cluster port.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TdeCredentialArn</i> The ARN from the key store with which the instance is associated for TDE encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type associated with DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusInfos</i> The status of a Read Replica. If the instance is not a Read Replica, this is blank.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> Specifies the accessibility options for the DB instance. A value of true specifies an Internet-facing instance with a publicly resolvable DNS name, which resolves to a public IP address. A value of false specifies an internal instance with a DNS name that resolves to a private IP address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondaryAvailabilityZone</i> If present, specifies the name of the secondary Availability Zone for a DB instance with multi-AZ support.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this instance is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionGroupMemberships</i> Provides the list of option group memberships for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the Provisioned IOPS (I/O operations per second) value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> License model information for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBClusterIdentifiers</i> Contains one or more identifiers of Aurora DB clusters to which the RDS DB instance is replicated as a Read Replica. For example, when you create an Aurora Read Replica of an RDS MySQL DB instance, the Aurora MySQL DB cluster for the Aurora Read Replica is shown. This output does not contain information about cross region Aurora Read Replicas. <note> Currently, each RDS DB instance can have only one Aurora Read Replica. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBInstanceIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaSourceDBInstanceIdentifier</i> Contains the identifier of the source DB instance if this DB instance is a Read Replica.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> Indicates that minor version patches are applied automatically.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies if the DB instance is a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DBSubnetGroup</b>: Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> Specifies the name of the Availability Zone the DB instance is located in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroups</i> Provides the list of DB parameter groups applied to this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security group elements that the DB instance belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSecurityGroups</i>  A list of DB security group elements containing <code>DBSecurityGroup.Name</code> and <code>DBSecurityGroup.Status</code> subelements. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i>  Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCreateTime</i> Provides the date and time the DB instance was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> Specifies the allocated storage size specified in gibibytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: Specifies the connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The meaning of this parameter differs according to the database engine you use.  <b>MySQL, MariaDB, SQL Server, PostgreSQL</b>  Contains the name of the initial database of this instance that was provided at create time, if one was specified when the DB instance was created. This same name is returned for the life of the DB instance. Type: String  <b>Oracle</b>  Contains the Oracle System ID (SID) of the created DB instance. Not shown when the returned parameters do not apply to an Oracle DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceStatus</i> Specifies the current state of this database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i> Contains the name of the compute and memory capacity class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i> Contains a user-supplied database identifier. This identifier is the unique key that identifies a DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupArn</i> The Amazon Resource Name (ARN) for the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i>  Contains a list of <code>Subnet</code> elements. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> Provides the status of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VpcId of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupDescription</i> Provides the description of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The name of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The new DB subnet group for the DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> Specifies the identifier of the CA certificate for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type to be associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i>  Contains the new <code>DBInstanceIdentifier</code> for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the new Provisioned IOPS value for the DB instance that will be applied or is currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> The license model for the DB instance. Valid values: <code>license-included</code> | <code>bring-your-own-license</code> | <code>general-public-license</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Indicates that the Single-AZ DB instance is to change to a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the pending number of days for which automated backups are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the pending port for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> Contains the pending or currently-in-progress change of the master credentials for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i>  Contains the new <code>AllocatedStorage</code> size for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i>  Contains the new <code>DBInstanceClass</code> for the DB instance that will be applied or is currently being applied. <br/>A list of the log types whose configuration is still pending. In other words, these log types are in the process of being activated or deactivated.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingCloudwatchLogsExports</b>: A list of the log types whose configuration is still pending. In other words, these log types are in the process of being activated or deactivated.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>        """
        pass

    def stop_activity_stream(self, ResourceArn: str, ApplyImmediately: BooleanOptional=None) -> StopActivityStreamResponse:
        """Stops a database activity stream that was started using the AWS console, the <code>start-activity-stream</code> AWS CLI command, or the <code>StartActivityStream</code> action. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/DBActivityStreams.html">Database Activity Streams</a> in the <i>Amazon Aurora User Guide</i>.

        :param self:
        :param ApplyImmediately:  Specifies whether or not the database activity stream is to stop as soon as possible, regardless of the maintenance window for the database.<br/>ResourceArn:  The Amazon Resource Name (ARN) of the DB cluster for the database activity stream. For example, <code>arn:aws:rds:us-east-1:12345667890:cluster:das-cluster</code>. <br/>
        :return: Status:  The status of the database activity stream.<br/>KinesisStreamName:  The name of the Amazon Kinesis data stream used for the database activity stream.<br/>KmsKeyId:  The AWS KMS key identifier used for encrypting messages in the database activity stream.<br/>        """
        pass

    def stop_db_cluster(self, DBClusterIdentifier: str) -> StopDBClusterResult:
        """ Stops an Amazon Aurora DB cluster. When you stop a DB cluster, Aurora retains the DB cluster's metadata, including its endpoints and DB parameter groups. Aurora also retains the transaction logs so you can do a point-in-time restore if necessary.  For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-cluster-stop-start.html"> Stopping and Starting an Aurora Cluster</a> in the <i>Amazon Aurora User Guide.</i>  <note> This action only applies to Aurora DB clusters. </note>

        :param self:
        :param DBClusterIdentifier:  The DB cluster identifier of the Amazon Aurora DB cluster to be stopped. This parameter is stored as a lowercase string.<br/>
        :return: DBCluster:  Contains the details of an Amazon Aurora DB cluster.  This data type is used as a response element in the <code>DescribeDBClusters</code>, <code>StopDBCluster</code>, and <code>StartDBCluster</code> actions. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CrossAccountClone</i> Specifies whether the DB cluster is a clone of a DB cluster owned by a different AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyTagsToSnapshot</i> Specifies whether tags are copied from the DB cluster to snapshots of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamKinesisStreamName</i> The name of the Amazon Kinesis data stream used for the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamKmsKeyId</i> The AWS KMS key identifier used for encrypting messages in the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamStatus</i> The status of the database activity stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActivityStreamMode</i> The mode of the database activity stream. Database events such as a change or access generate an activity stream event. The database session can handle these events either synchronously or asynchronously. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HttpEndpointEnabled</i> A value that indicates whether the HTTP endpoint for an Aurora Serverless DB cluster is enabled. When enabled, the HTTP endpoint provides a connectionless web service API for running SQL queries on the Aurora Serverless DB cluster. You can also query your database from inside the RDS console with the query editor. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/data-api.html">Using the Data API for Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i> Indicates if the DB cluster has deletion protection enabled. The database can't be deleted when deletion protection is enabled. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ScalingConfigurationInfo</i> Shows the scaling configuration for an Aurora DB cluster in <code>serverless</code> DB engine mode. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ScalingConfigurationInfo</b>: Shows the scaling configuration for an Aurora DB cluster in <code>serverless</code> DB engine mode. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineMode</i> The DB engine mode of the DB cluster, either <code>provisioned</code>, <code>serverless</code>, or <code>parallelquery</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Capacity</i> The current capacity of an Aurora Serverless DB cluster. The capacity is 0 (zero) when the cluster is paused. For more information about Aurora Serverless, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/aurora-serverless.html">Using Amazon Aurora Serverless</a> in the <i>Amazon Aurora User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB cluster is configured to export to CloudWatch Logs. Log types vary by DB engine. For information about the log types for each DB engine, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/USER_LogAccess.html">Amazon RDS Database Log Files</a> in the <i>Amazon Aurora User Guide.</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BacktrackConsumedChangeRecords</i> The number of change records stored for Backtrack.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BacktrackWindow</i> The target backtrack window, in seconds. If this value is set to 0, backtracking is disabled for the DB cluster. Otherwise, backtracking is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestBacktrackTime</i> The earliest time to which a DB cluster can be backtracked.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> Specifies the time when the DB cluster was created, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloneGroupId</i> Identifies the clone group to which the DB cluster is associated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> A value that indicates whether the mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i> Provides a list of the AWS Identity and Access Management (IAM) roles that are associated with the DB cluster. IAM roles that are associated with a DB cluster grant permission for the DB cluster to access other AWS services on your behalf.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterArn</i> The Amazon Resource Name (ARN) for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbClusterResourceId</i> The AWS Region-unique, immutable identifier for the DB cluster. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB cluster is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> If <code>StorageEncrypted</code> is enabled, the AWS KMS key identifier for the encrypted DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB cluster is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security groups that the DB cluster belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterMembers</i> Provides the list of instances that make up the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReplicationSourceIdentifier</i> Contains the identifier of the source DB cluster if this DB cluster is a Read Replica.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i> Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterOptionGroupMemberships</i> Provides the list of option group memberships for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies whether the DB cluster has instances in multiple Availability Zones.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomEndpoints</i> Identifies all custom endpoints associated with the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReaderEndpoint</i> The reader endpoint for the DB cluster. The reader endpoint for a DB cluster load-balances connections across the Aurora Replicas that are available in a DB cluster. As clients request new connections to the reader endpoint, Aurora distributes the connection requests among the Aurora Replicas in the DB cluster. This functionality can help balance your read workload across multiple Aurora Replicas in your DB cluster.  If a failover occurs, and the Aurora Replica that you are connected to is promoted to be the primary instance, your connection is dropped. To continue sending your read workload to other Aurora Replicas in the cluster, you can then reconnect to the reader endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint for the primary instance of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EarliestRestorableTime</i> The earliest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PercentProgress</i> Specifies the progress of the operation as a percentage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the current state of this DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB cluster, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterParameterGroup</i> Specifies the name of the DB cluster parameter group for the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> Contains a user-supplied DB cluster identifier. This identifier is the unique key that identifies a DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DatabaseName</i> Contains the name of the initial database of this DB cluster that was provided at create time, if one was specified when the DB cluster was created. This same name is returned for the life of the DB cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this cluster is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZones</i> Provides the list of Availability Zones (AZs) where instances in the DB cluster can be created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> For all database engines except Amazon Aurora, <code>AllocatedStorage</code> specifies the allocated storage size in gibibytes (GiB). For Aurora, <code>AllocatedStorage</code> always returns 1, because Aurora DB cluster storage size is not fixed, but instead automatically adjusts as needed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeoutAction</i> The timeout action of a call to <code>ModifyCurrentDBClusterCapacity</code>, either <code>ForceApplyCapacityChange</code> or <code>RollbackCapacityChange</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondsUntilAutoPause</i> The remaining amount of time, in seconds, before the Aurora DB cluster in <code>serverless</code> mode is paused. A DB cluster can be paused only when it's idle (it has no connections).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoPause</i> A value that indicates whether automatic pause is allowed for the Aurora DB cluster in <code>serverless</code> DB engine mode. When the value is set to false for an Aurora Serverless DB cluster, the DB cluster automatically resumes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxCapacity</i> The maximum capacity for an Aurora DB cluster in <code>serverless</code> DB engine mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MinCapacity</i> The maximum capacity for the Aurora DB cluster in <code>serverless</code> DB engine mode.<br/>        """
        pass

    def stop_db_instance(self, DBInstanceIdentifier: str, DBSnapshotIdentifier: str=None) -> StopDBInstanceResult:
        """ Stops an Amazon RDS DB instance. When you stop a DB instance, Amazon RDS retains the DB instance's metadata, including its endpoint, DB parameter group, and option group membership. Amazon RDS also retains the transaction logs so you can do a point-in-time restore if necessary.  For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_StopInstance.html"> Stopping an Amazon RDS DB Instance Temporarily</a> in the <i>Amazon RDS User Guide.</i>  <note>  This command doesn't apply to Aurora MySQL and Aurora PostgreSQL. For Aurora clusters, use <code>StopDBCluster</code> instead.  </note>

        :param self:
        :param DBSnapshotIdentifier:   The user-supplied instance identifier of the DB Snapshot created immediately before the DB instance is stopped. <br/>DBInstanceIdentifier:   The user-supplied instance identifier. <br/>
        :return: DBInstance:  Contains the details of an Amazon RDS DB instance.  This data type is used as a response element in the <code>DescribeDBInstances</code> action. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxAllocatedStorage</i> The upper limit to which Amazon RDS can automatically scale the storage of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ListenerEndpoint</i> Specifies the listener connection endpoint for SQL Server Always On.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ListenerEndpoint</b>: Specifies the listener connection endpoint for SQL Server Always On.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociatedRoles</i>  The AWS Identity and Access Management (IAM) roles associated with the DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeletionProtection</i> Indicates if the DB instance has deletion protection enabled. The database can't be deleted when deletion protection is enabled. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_DeleteInstance.html"> Deleting a DB Instance</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessorFeatures</i> The number of CPU cores and the number of threads per core for the DB instance class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnabledCloudwatchLogsExports</i> A list of log types that this DB instance is configured to export to CloudWatch Logs. Log types vary by DB engine. For information about the log types for each DB engine, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_LogAccess.html">Amazon RDS Database Log Files</a> in the <i>Amazon RDS User Guide.</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsRetentionPeriod</i> The amount of time, in days, to retain Performance Insights data. Valid values are 7 or 731 (2 years). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsKMSKeyId</i> The AWS KMS key identifier for encryption of Performance Insights data. The KMS key ID is the Amazon Resource Name (ARN), KMS key identifier, or the KMS key alias for the KMS encryption key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PerformanceInsightsEnabled</i> True if Performance Insights is enabled for the DB instance, and otherwise false.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMDatabaseAuthenticationEnabled</i> True if mapping of AWS Identity and Access Management (IAM) accounts to database accounts is enabled, and otherwise false. IAM database authentication can be enabled for the following database engines <ul> <li> For MySQL 5.6, minor version 5.6.34 or higher </li> <li> For MySQL 5.7, minor version 5.7.16 or higher </li> <li> Aurora 5.6 or higher. To enable IAM database authentication for Aurora, see DBCluster Type. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Timezone</i> The time zone of the DB instance. In most cases, the <code>Timezone</code> element is empty. <code>Timezone</code> content appears only for Microsoft SQL Server DB instances that were created with a time zone specified. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceArn</i> The Amazon Resource Name (ARN) for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PromotionTier</i> A value that specifies the order in which an Aurora Replica is promoted to the primary instance after a failure of the existing primary instance. For more information, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/Aurora.Managing.Backups.html#Aurora.Managing.FaultTolerance"> Fault Tolerance for an Aurora DB Cluster</a> in the <i>Amazon Aurora User Guide</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringRoleArn</i> The ARN for the IAM role that permits RDS to send Enhanced Monitoring metrics to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedMonitoringResourceArn</i> The Amazon Resource Name (ARN) of the Amazon CloudWatch Logs log stream that receives the Enhanced Monitoring metrics data for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringInterval</i> The interval, in seconds, between points when Enhanced Monitoring metrics are collected for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CopyTagsToSnapshot</i> Specifies whether tags are copied from the DB instance to snapshots of the DB instance.  <b>Amazon Aurora</b>  Not applicable. Copying tags to snapshots is managed by the DB cluster. Setting this value for an Aurora DB instance has no effect on the DB cluster setting. For more information, see <code>DBCluster</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainMemberships</i> The Active Directory Domain membership records associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> The identifier of the CA certificate for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbiResourceId</i> The AWS Region-unique, immutable identifier for the DB instance. This identifier is found in AWS CloudTrail log entries whenever the AWS KMS key for the DB instance is accessed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i>  If <code>StorageEncrypted</code> is true, the AWS KMS key identifier for the encrypted DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageEncrypted</i> Specifies whether the DB instance is encrypted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBClusterIdentifier</i> If the DB instance is a member of a DB cluster, contains the name of the DB cluster that the DB instance is a member of.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DbInstancePort</i> Specifies the port that the DB instance listens on. If the DB instance is part of a DB cluster, this can be a different port than the DB cluster port.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TdeCredentialArn</i> The ARN from the key store with which the instance is associated for TDE encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type associated with DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusInfos</i> The status of a Read Replica. If the instance is not a Read Replica, this is blank.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> Specifies the accessibility options for the DB instance. A value of true specifies an Internet-facing instance with a publicly resolvable DNS name, which resolves to a public IP address. A value of false specifies an internal instance with a DNS name that resolves to a private IP address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecondaryAvailabilityZone</i> If present, specifies the name of the secondary Availability Zone for a DB instance with multi-AZ support.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CharacterSetName</i> If present, specifies the name of the character set that this instance is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionGroupMemberships</i> Provides the list of option group memberships for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the Provisioned IOPS (I/O operations per second) value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> License model information for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBClusterIdentifiers</i> Contains one or more identifiers of Aurora DB clusters to which the RDS DB instance is replicated as a Read Replica. For example, when you create an Aurora Read Replica of an RDS MySQL DB instance, the Aurora MySQL DB cluster for the Aurora Read Replica is shown. This output does not contain information about cross region Aurora Read Replicas. <note> Currently, each RDS DB instance can have only one Aurora Read Replica. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaDBInstanceIdentifiers</i> Contains one or more identifiers of the Read Replicas associated with this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReadReplicaSourceDBInstanceIdentifier</i> Contains the identifier of the source DB instance if this DB instance is a Read Replica.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoMinorVersionUpgrade</i> Indicates that minor version patches are applied automatically.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Specifies if the DB instance is a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestRestorableTime</i> Specifies the latest time to which a database can be restored with point-in-time restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: Specifies that changes to the DB instance are pending. This element is only included when changes are pending. Specific changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> Specifies the weekly time range during which system maintenance can occur, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroup</i> Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DBSubnetGroup</b>: Specifies information on the subnet group associated with the DB instance, including the name, description, and subnets in the subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> Specifies the name of the Availability Zone the DB instance is located in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBParameterGroups</i> Provides the list of DB parameter groups applied to this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> Provides a list of VPC security group elements that the DB instance belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSecurityGroups</i>  A list of DB security group elements containing <code>DBSecurityGroup.Name</code> and <code>DBSecurityGroup.Status</code> subelements. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the number of days for which automatic DB snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredBackupWindow</i>  Specifies the daily time range during which automated backups are created if automated backups are enabled, as determined by the <code>BackupRetentionPeriod</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCreateTime</i> Provides the date and time the DB instance was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i> Specifies the allocated storage size specified in gibibytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> Specifies the connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: Specifies the connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The meaning of this parameter differs according to the database engine you use.  <b>MySQL, MariaDB, SQL Server, PostgreSQL</b>  Contains the name of the initial database of this instance that was provided at create time, if one was specified when the DB instance was created. This same name is returned for the life of the DB instance. Type: String  <b>Oracle</b>  Contains the Oracle System ID (SID) of the created DB instance. Not shown when the returned parameters do not apply to an Oracle DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> Contains the master username for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceStatus</i> Specifies the current state of this database.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Engine</i> Provides the name of the database engine to be used for this DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i> Contains the name of the compute and memory capacity class of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i> Contains a user-supplied database identifier. This identifier is the unique key that identifies a DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupArn</i> The Amazon Resource Name (ARN) for the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i>  Contains a list of <code>Subnet</code> elements. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> Provides the status of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> Provides the VpcId of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupDescription</i> Provides the description of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The name of the DB subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBSubnetGroupName</i> The new DB subnet group for the DB instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CACertificateIdentifier</i> Specifies the identifier of the CA certificate for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> Specifies the storage type to be associated with the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceIdentifier</i>  Contains the new <code>DBInstanceIdentifier</code> for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Iops</i> Specifies the new Provisioned IOPS value for the DB instance that will be applied or is currently being applied.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LicenseModel</i> The license model for the DB instance. Valid values: <code>license-included</code> | <code>bring-your-own-license</code> | <code>general-public-license</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EngineVersion</i> Indicates the database engine version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MultiAZ</i> Indicates that the Single-AZ DB instance is to change to a Multi-AZ deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupRetentionPeriod</i> Specifies the pending number of days for which automated backups are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the pending port for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> Contains the pending or currently-in-progress change of the master credentials for the DB instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllocatedStorage</i>  Contains the new <code>AllocatedStorage</code> size for the DB instance that will be applied or is currently being applied. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DBInstanceClass</i>  Contains the new <code>DBInstanceClass</code> for the DB instance that will be applied or is currently being applied. <br/>A list of the log types whose configuration is still pending. In other words, these log types are in the process of being activated or deactivated.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingCloudwatchLogsExports</b>: A list of the log types whose configuration is still pending. In other words, these log types are in the process of being activated or deactivated.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> Specifies the port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> Specifies the DNS address of the DB instance.<br/>        """
        pass



class RDSDataService(BaseClient):
    """<fullname>Amazon RDS Data Service</fullname>
        Amazon RDS provides an HTTP endpoint to run SQL statements on an Amazon Aurora
            Serverless DB cluster. To run these statements, you work with the Data Service
            API.
        For more information about the Data Service API, see <a href="https://docs.aws.amazon.com/AmazonRDS/latest/AuroraUserGuide/data-api.html">Using the Data API for Aurora
                Serverless</a> in the <i>Amazon Aurora User Guide</i>."""




    class BatchExecuteStatementRequest(Mapping):
        pass
    
    class BatchExecuteStatementResponse(Mapping):
        pass
    
    class SqlParametersList(object):
        pass
    
    class BeginTransactionRequest(Mapping):
        pass
    
    class BeginTransactionResponse(Mapping):
        pass
    
    class CommitTransactionRequest(Mapping):
        pass
    
    class CommitTransactionResponse(Mapping):
        pass
    
    class ExecuteSqlRequest(Mapping):
        pass
    
    class ExecuteSqlResponse(Mapping):
        pass
    
    class ExecuteStatementRequest(Mapping):
        pass
    
    class ExecuteStatementResponse(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class SqlParameter(Mapping):
        pass
    
    class RollbackTransactionRequest(Mapping):
        pass
    
    class RollbackTransactionResponse(Mapping):
        pass
    
    def batch_execute_statement(self, sql: str, secretArn: str, resourceArn: str, database: str=None, parameterSets: List[SqlParametersList]=None, schema: str=None, transactionId: str=None) -> BatchExecuteStatementResponse:
        """Runs a batch SQL statement over an array of data.
        You can run bulk update and insert operations for multiple records using a DML 
            statement with different parameter sets. Bulk operations can provide a significant 
            performance improvement over individual insert and update operations.
        <important>    
            If a call isn't part of a transaction because it doesn't include the
                    <code>transactionID</code> parameter, changes that result from the call are
                committed automatically.    
        </important>

        :param self:
        :param transactionId:  The identifier of a transaction that was started by using the
                <code>BeginTransaction</code> operation. Specify the transaction ID of the
            transaction that you want to include the SQL statement in.
        If the SQL statement is not part of a transaction, don't set this
            parameter.<br/>sql:  The SQL statement to run.<br/>secretArn:  The name or ARN of the secret that enables access to the DB cluster.<br/>schema:  The name of the database schema.<br/>resourceArn:  The Amazon Resource Name (ARN) of the Aurora Serverless DB cluster.<br/>parameterSets:  The parameter set for the batch operation.<br/>database:  The name of the database.<br/>
        :return: updateResults:  The execution results of each batch entry.<br/>        """
        pass

    def begin_transaction(self, secretArn: str, resourceArn: str, database: str=None, schema: str=None) -> BeginTransactionResponse:
        """Starts a SQL transaction.
        
        <important>
            A transaction can run for a maximum of 24 hours. A transaction is terminated and 
                rolled back automatically after 24 hours.
            A transaction times out if no calls use its transaction ID in three minutes. 
                If a transaction times out before it's committed, it's rolled back
                automatically.
            DDL statements inside a transaction cause an implicit commit. We recommend 
                that you run each DDL statement in a separate <code>ExecuteStatement</code> call with 
                <code>continueAfterTimeout</code> enabled.
        </important>

        :param self:
        :param secretArn:  The name or ARN of the secret that enables access to the DB cluster.<br/>schema:  The name of the database schema.<br/>resourceArn:  The Amazon Resource Name (ARN) of the Aurora Serverless DB cluster.<br/>database:  The name of the database.<br/>
        :return: transactionId:  The transaction ID of the transaction started by the call.<br/>        """
        pass

    def commit_transaction(self, transactionId: str, secretArn: str, resourceArn: str) -> CommitTransactionResponse:
        """Ends a SQL transaction started with the <code>BeginTransaction</code> operation and
            commits the changes.

        :param self:
        :param transactionId:  The identifier of the transaction to end and commit.<br/>secretArn:  The name or ARN of the secret that enables access to the DB cluster.<br/>resourceArn:  The Amazon Resource Name (ARN) of the Aurora Serverless DB cluster.<br/>
        :return: transactionStatus:  The status of the commit operation.<br/>        """
        pass

    def execute_sql(self, sqlStatements: str, dbClusterOrInstanceArn: str, awsSecretStoreArn: str, database: str=None, schema: str=None) -> ExecuteSqlResponse:
        """Runs one or more SQL statements.
        <important>
            This operation is deprecated. Use the <code>BatchExecuteStatement</code> or
                    <code>ExecuteStatement</code> operation.
        </important>

        :param self:
        :param sqlStatements:  One or more SQL statements to run on the DB cluster.
        You can separate SQL statements from each other with a semicolon (;). Any valid SQL
            statement is permitted, including data definition, data manipulation, and commit
            statements. <br/>schema:  The name of the database schema.<br/>dbClusterOrInstanceArn:  The ARN of the Aurora Serverless DB cluster.<br/>database:  The name of the database.<br/>awsSecretStoreArn:  The Amazon Resource Name (ARN) of the secret that enables access to the DB cluster.<br/>
        :return: sqlStatementResults:  The results of the SQL statement or statements.<br/>        """
        pass

    def execute_statement(self, sql: str, secretArn: str, resourceArn: str, continueAfterTimeout: Boolean=None, database: str=None, includeResultMetadata: Boolean=None, parameters: List[SqlParameter]=None, schema: str=None, transactionId: str=None) -> ExecuteStatementResponse:
        """Runs a SQL statement against a database.
        <important>    
            If a call isn't part of a transaction because it doesn't include the
                    <code>transactionID</code> parameter, changes that result from the call are
                committed automatically.    
        </important>
        The response size limit is 1 MB or 1,000 records. If the call returns more than 1 MB of response data or over 1,000 records, the call is terminated.

        :param self:
        :param transactionId:  The identifier of a transaction that was started by using the
                <code>BeginTransaction</code> operation. Specify the transaction ID of the
            transaction that you want to include the SQL statement in.
        If the SQL statement is not part of a transaction, don't set this parameter.<br/>sql:  The SQL statement to run.<br/>secretArn:  The name or ARN of the secret that enables access to the DB cluster.<br/>schema:  The name of the database schema.<br/>resourceArn:  The Amazon Resource Name (ARN) of the Aurora Serverless DB cluster.<br/>parameters:  The parameters for the SQL statement.<br/>includeResultMetadata:  A value that indicates whether to include metadata in the results.<br/>database:  The name of the database.<br/>continueAfterTimeout:  A value that indicates whether to continue running the statement after 
            the call times out. By default, the statement stops running when the call 
            times out.
        <important>
            For DDL statements, we recommend continuing to run the statement after 
               the call times out. When a DDL statement terminates before it is finished 
               running, it can result in errors and possibly corrupted data structures.
        </important><br/>
        :return: records:  The records returned by the SQL statement.<br/>numberOfRecordsUpdated:  The number of records updated by the request.<br/>generatedFields:  Values for fields generated during the request.<br/>columnMetadata:  Metadata for the columns included in the results.<br/>        """
        pass

    def rollback_transaction(self, transactionId: str, secretArn: str, resourceArn: str) -> RollbackTransactionResponse:
        """Performs a rollback of a transaction. Rolling back a transaction cancels its changes.

        :param self:
        :param transactionId:  The identifier of the transaction to roll back.<br/>secretArn:  The name or ARN of the secret that enables access to the DB cluster.<br/>resourceArn:  The Amazon Resource Name (ARN) of the Aurora Serverless DB cluster.<br/>
        :return: transactionStatus:  The status of the rollback operation.<br/>        """
        pass



class Redshift(BaseClient):
    """<fullname>Amazon Redshift</fullname>  <b>Overview</b>  This is an interface reference for Amazon Redshift. It contains documentation for one of the programming or command line interfaces you can use to manage Amazon Redshift clusters. Note that Amazon Redshift is asynchronous, which means that some interfaces may require techniques, such as polling or asynchronous callback handlers, to determine when a command has been applied. In this reference, the parameter descriptions indicate whether a change is applied immediately, on the next instance reboot, or during the next maintenance window. For a summary of the Amazon Redshift cluster management interfaces, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/using-aws-sdk.html">Using the Amazon Redshift Management Interfaces</a>. Amazon Redshift manages all the work of setting up, operating, and scaling a data warehouse: provisioning capacity, monitoring and backing up the cluster, and applying patches and upgrades to the Amazon Redshift engine. You can focus on using your data to acquire new insights for your business and customers. If you are a first-time user of Amazon Redshift, we recommend that you begin by reading the <a href="https://docs.aws.amazon.com/redshift/latest/gsg/getting-started.html">Amazon Redshift Getting Started Guide</a>. If you are a database developer, the <a href="https://docs.aws.amazon.com/redshift/latest/dg/welcome.html">Amazon Redshift Database Developer Guide</a> explains how to design, build, query, and maintain the databases that make up your data warehouse. """

    class ClusterAvailableWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_clusters` for valid parameters"""
            pass
    class ClusterDeletedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_clusters` for valid parameters"""
            pass
    class ClusterRestoredWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_clusters` for valid parameters"""
            pass
    class SnapshotAvailableWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_cluster_snapshots` for valid parameters"""
            pass


    class DescribeClusterParameterGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_cluster_parameter_groups` for valid parameters"""
            pass
    class DescribeClusterParametersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_cluster_parameters` for valid parameters"""
            pass
    class DescribeClusterSecurityGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_cluster_security_groups` for valid parameters"""
            pass
    class DescribeClusterSnapshotsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_cluster_snapshots` for valid parameters"""
            pass
    class DescribeClusterSubnetGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_cluster_subnet_groups` for valid parameters"""
            pass
    class DescribeClusterVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_cluster_versions` for valid parameters"""
            pass
    class DescribeClustersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_clusters` for valid parameters"""
            pass
    class DescribeDefaultClusterParametersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_default_cluster_parameters` for valid parameters"""
            pass
    class DescribeEventSubscriptionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_event_subscriptions` for valid parameters"""
            pass
    class DescribeEventsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_events` for valid parameters"""
            pass
    class DescribeHsmClientCertificatesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_hsm_client_certificates` for valid parameters"""
            pass
    class DescribeHsmConfigurationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_hsm_configurations` for valid parameters"""
            pass
    class DescribeOrderableClusterOptionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_orderable_cluster_options` for valid parameters"""
            pass
    class DescribeReservedNodeOfferingsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_reserved_node_offerings` for valid parameters"""
            pass
    class DescribeReservedNodesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_reserved_nodes` for valid parameters"""
            pass
    class DescribeClusterDbRevisionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_cluster_db_revisions` for valid parameters"""
            pass
    class DescribeClusterTracksPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_cluster_tracks` for valid parameters"""
            pass
    class DescribeSnapshotCopyGrantsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_snapshot_copy_grants` for valid parameters"""
            pass
    class DescribeSnapshotSchedulesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_snapshot_schedules` for valid parameters"""
            pass
    class DescribeTableRestoreStatusPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_table_restore_status` for valid parameters"""
            pass
    class DescribeTagsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_tags` for valid parameters"""
            pass
    class GetReservedNodeExchangeOfferingsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_reserved_node_exchange_offerings` for valid parameters"""
            pass


    class AcceptReservedNodeExchangeInputMessage(Mapping):
        pass
    
    class AcceptReservedNodeExchangeOutputMessage(Mapping):
        pass
    
    class AuthorizeClusterSecurityGroupIngressMessage(Mapping):
        pass
    
    class AuthorizeClusterSecurityGroupIngressResult(Mapping):
        pass
    
    class AuthorizeSnapshotAccessMessage(Mapping):
        pass
    
    class AuthorizeSnapshotAccessResult(Mapping):
        pass
    
    class BatchDeleteClusterSnapshotsRequest(Mapping):
        pass
    
    class BatchDeleteClusterSnapshotsResult(Mapping):
        pass
    
    class DeleteClusterSnapshotMessage(Mapping):
        pass
    
    class BatchModifyClusterSnapshotsMessage(Mapping):
        pass
    
    class BatchModifyClusterSnapshotsOutputMessage(Mapping):
        pass
    
    class String(object):
        pass
    
    class Boolean(object):
        pass
    
    class CancelResizeMessage(Mapping):
        pass
    
    class ResizeProgressMessage(Mapping):
        pass
    
    class CopyClusterSnapshotMessage(Mapping):
        pass
    
    class CopyClusterSnapshotResult(Mapping):
        pass
    
    class CreateClusterMessage(Mapping):
        pass
    
    class CreateClusterResult(Mapping):
        pass
    
    class BooleanOptional(object):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreateClusterParameterGroupMessage(Mapping):
        pass
    
    class CreateClusterParameterGroupResult(Mapping):
        pass
    
    class CreateClusterSecurityGroupMessage(Mapping):
        pass
    
    class CreateClusterSecurityGroupResult(Mapping):
        pass
    
    class CreateClusterSnapshotMessage(Mapping):
        pass
    
    class CreateClusterSnapshotResult(Mapping):
        pass
    
    class CreateClusterSubnetGroupMessage(Mapping):
        pass
    
    class CreateClusterSubnetGroupResult(Mapping):
        pass
    
    class CreateEventSubscriptionMessage(Mapping):
        pass
    
    class CreateEventSubscriptionResult(Mapping):
        pass
    
    class CreateHsmClientCertificateMessage(Mapping):
        pass
    
    class CreateHsmClientCertificateResult(Mapping):
        pass
    
    class CreateHsmConfigurationMessage(Mapping):
        pass
    
    class CreateHsmConfigurationResult(Mapping):
        pass
    
    class CreateSnapshotCopyGrantMessage(Mapping):
        pass
    
    class CreateSnapshotCopyGrantResult(Mapping):
        pass
    
    class CreateSnapshotScheduleMessage(Mapping):
        pass
    
    class SnapshotSchedule(Mapping):
        pass
    
    class CreateTagsMessage(Mapping):
        pass
    
    class DeleteClusterMessage(Mapping):
        pass
    
    class DeleteClusterResult(Mapping):
        pass
    
    class DeleteClusterParameterGroupMessage(Mapping):
        pass
    
    class DeleteClusterSecurityGroupMessage(Mapping):
        pass
    
    class DeleteClusterSnapshotResult(Mapping):
        pass
    
    class DeleteClusterSubnetGroupMessage(Mapping):
        pass
    
    class DeleteEventSubscriptionMessage(Mapping):
        pass
    
    class DeleteHsmClientCertificateMessage(Mapping):
        pass
    
    class DeleteHsmConfigurationMessage(Mapping):
        pass
    
    class DeleteSnapshotCopyGrantMessage(Mapping):
        pass
    
    class DeleteSnapshotScheduleMessage(Mapping):
        pass
    
    class DeleteTagsMessage(Mapping):
        pass
    
    class DescribeAccountAttributesMessage(Mapping):
        pass
    
    class AccountAttributeList(Mapping):
        pass
    
    class DescribeClusterDbRevisionsMessage(Mapping):
        pass
    
    class ClusterDbRevisionsMessage(Mapping):
        pass
    
    class DescribeClusterParameterGroupsMessage(Mapping):
        pass
    
    class ClusterParameterGroupsMessage(Mapping):
        pass
    
    class DescribeClusterParametersMessage(Mapping):
        pass
    
    class ClusterParameterGroupDetails(Mapping):
        pass
    
    class DescribeClusterSecurityGroupsMessage(Mapping):
        pass
    
    class ClusterSecurityGroupMessage(Mapping):
        pass
    
    class DescribeClusterSnapshotsMessage(Mapping):
        pass
    
    class SnapshotMessage(Mapping):
        pass
    
    class TStamp(object):
        pass
    
    class SnapshotSortingEntity(Mapping):
        pass
    
    class DescribeClusterSubnetGroupsMessage(Mapping):
        pass
    
    class ClusterSubnetGroupMessage(Mapping):
        pass
    
    class DescribeClusterTracksMessage(Mapping):
        pass
    
    class TrackListMessage(Mapping):
        pass
    
    class DescribeClusterVersionsMessage(Mapping):
        pass
    
    class ClusterVersionsMessage(Mapping):
        pass
    
    class DescribeClustersMessage(Mapping):
        pass
    
    class ClustersMessage(Mapping):
        pass
    
    class DescribeDefaultClusterParametersMessage(Mapping):
        pass
    
    class DescribeDefaultClusterParametersResult(Mapping):
        pass
    
    class DescribeEventCategoriesMessage(Mapping):
        pass
    
    class EventCategoriesMessage(Mapping):
        pass
    
    class DescribeEventSubscriptionsMessage(Mapping):
        pass
    
    class EventSubscriptionsMessage(Mapping):
        pass
    
    class DescribeEventsMessage(Mapping):
        pass
    
    class EventsMessage(Mapping):
        pass
    
    class DescribeHsmClientCertificatesMessage(Mapping):
        pass
    
    class HsmClientCertificateMessage(Mapping):
        pass
    
    class DescribeHsmConfigurationsMessage(Mapping):
        pass
    
    class HsmConfigurationMessage(Mapping):
        pass
    
    class DescribeLoggingStatusMessage(Mapping):
        pass
    
    class LoggingStatus(Mapping):
        pass
    
    class DescribeOrderableClusterOptionsMessage(Mapping):
        pass
    
    class OrderableClusterOptionsMessage(Mapping):
        pass
    
    class DescribeReservedNodeOfferingsMessage(Mapping):
        pass
    
    class ReservedNodeOfferingsMessage(Mapping):
        pass
    
    class DescribeReservedNodesMessage(Mapping):
        pass
    
    class ReservedNodesMessage(Mapping):
        pass
    
    class DescribeResizeMessage(Mapping):
        pass
    
    class DescribeSnapshotCopyGrantsMessage(Mapping):
        pass
    
    class SnapshotCopyGrantMessage(Mapping):
        pass
    
    class DescribeSnapshotSchedulesMessage(Mapping):
        pass
    
    class DescribeSnapshotSchedulesOutputMessage(Mapping):
        pass
    
    class CustomerStorageMessage(Mapping):
        pass
    
    class DescribeTableRestoreStatusMessage(Mapping):
        pass
    
    class TableRestoreStatusMessage(Mapping):
        pass
    
    class DescribeTagsMessage(Mapping):
        pass
    
    class TaggedResourceListMessage(Mapping):
        pass
    
    class DisableLoggingMessage(Mapping):
        pass
    
    class DisableSnapshotCopyMessage(Mapping):
        pass
    
    class DisableSnapshotCopyResult(Mapping):
        pass
    
    class EnableLoggingMessage(Mapping):
        pass
    
    class EnableSnapshotCopyMessage(Mapping):
        pass
    
    class EnableSnapshotCopyResult(Mapping):
        pass
    
    class GetClusterCredentialsMessage(Mapping):
        pass
    
    class ClusterCredentials(Mapping):
        pass
    
    class GetReservedNodeExchangeOfferingsInputMessage(Mapping):
        pass
    
    class GetReservedNodeExchangeOfferingsOutputMessage(Mapping):
        pass
    
    class ModifyClusterMessage(Mapping):
        pass
    
    class ModifyClusterResult(Mapping):
        pass
    
    class ModifyClusterDbRevisionMessage(Mapping):
        pass
    
    class ModifyClusterDbRevisionResult(Mapping):
        pass
    
    class ModifyClusterIamRolesMessage(Mapping):
        pass
    
    class ModifyClusterIamRolesResult(Mapping):
        pass
    
    class ModifyClusterMaintenanceMessage(Mapping):
        pass
    
    class ModifyClusterMaintenanceResult(Mapping):
        pass
    
    class ModifyClusterParameterGroupMessage(Mapping):
        pass
    
    class ClusterParameterGroupNameMessage(Mapping):
        pass
    
    class Parameter(Mapping):
        pass
    
    class ModifyClusterSnapshotMessage(Mapping):
        pass
    
    class ModifyClusterSnapshotResult(Mapping):
        pass
    
    class ModifyClusterSnapshotScheduleMessage(Mapping):
        pass
    
    class ModifyClusterSubnetGroupMessage(Mapping):
        pass
    
    class ModifyClusterSubnetGroupResult(Mapping):
        pass
    
    class ModifyEventSubscriptionMessage(Mapping):
        pass
    
    class ModifyEventSubscriptionResult(Mapping):
        pass
    
    class ModifySnapshotCopyRetentionPeriodMessage(Mapping):
        pass
    
    class ModifySnapshotCopyRetentionPeriodResult(Mapping):
        pass
    
    class ModifySnapshotScheduleMessage(Mapping):
        pass
    
    class PurchaseReservedNodeOfferingMessage(Mapping):
        pass
    
    class PurchaseReservedNodeOfferingResult(Mapping):
        pass
    
    class RebootClusterMessage(Mapping):
        pass
    
    class RebootClusterResult(Mapping):
        pass
    
    class ResetClusterParameterGroupMessage(Mapping):
        pass
    
    class ResizeClusterMessage(Mapping):
        pass
    
    class ResizeClusterResult(Mapping):
        pass
    
    class RestoreFromClusterSnapshotMessage(Mapping):
        pass
    
    class RestoreFromClusterSnapshotResult(Mapping):
        pass
    
    class RestoreTableFromClusterSnapshotMessage(Mapping):
        pass
    
    class RestoreTableFromClusterSnapshotResult(Mapping):
        pass
    
    class RevokeClusterSecurityGroupIngressMessage(Mapping):
        pass
    
    class RevokeClusterSecurityGroupIngressResult(Mapping):
        pass
    
    class RevokeSnapshotAccessMessage(Mapping):
        pass
    
    class RevokeSnapshotAccessResult(Mapping):
        pass
    
    class RotateEncryptionKeyMessage(Mapping):
        pass
    
    class RotateEncryptionKeyResult(Mapping):
        pass
    
    def accept_reserved_node_exchange(self, TargetReservedNodeOfferingId: str, ReservedNodeId: str) -> AcceptReservedNodeExchangeOutputMessage:
        """Exchanges a DC1 Reserved Node for a DC2 Reserved Node with no changes to the configuration (term, payment type, or number of nodes) and no additional costs. 

        :param self:
        :param TargetReservedNodeOfferingId:  The unique identifier of the DC2 Reserved Node offering to be used for the exchange. You can obtain the value for the parameter by calling <a>GetReservedNodeExchangeOfferings</a> <br/>ReservedNodeId:  A string representing the node identifier of the DC1 Reserved Node to be exchanged.<br/>
        :return: ExchangedReservedNode:  <p/>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservedNodeOfferingType</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecurringCharges</i> The recurring charges for the reserved node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OfferingType</i> The anticipated utilization of the reserved node, as defined in the reserved node offering.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the reserved compute node. Possible Values: <ul> <li> pending-payment-This reserved node has recently been purchased, and the sale has been approved, but payment has not yet been confirmed. </li> <li> active-This reserved node is owned by the caller and is available for use. </li> <li> payment-failed-Payment failed for the purchase attempt. </li> <li> retired-The reserved node is no longer available.  </li> <li> exchanging-The owner is exchanging the reserved node for another reserved node. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeCount</i> The number of reserved compute nodes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrencyCode</i> The currency code for the reserved cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UsagePrice</i> The hourly rate Amazon Redshift charges you for this reserved node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FixedPrice</i> The fixed cost Amazon Redshift charges you for this reserved node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Duration</i> The duration of the node reservation in seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTime</i> The time the reservation started. You purchase a reserved node offering for a duration. This is the start time of that duration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The node type of the reserved node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservedNodeOfferingId</i> The identifier for the reserved node offering.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservedNodeId</i> The unique identifier for the reservation.<br/>        """
        pass

    def authorize_cluster_security_group_ingress(self, ClusterSecurityGroupName: str, CIDRIP: str=None, EC2SecurityGroupName: str=None, EC2SecurityGroupOwnerId: str=None) -> AuthorizeClusterSecurityGroupIngressResult:
        """Adds an inbound (ingress) rule to an Amazon Redshift security group. Depending on whether the application accessing your cluster is running on the Internet or an Amazon EC2 instance, you can authorize inbound access to either a Classless Interdomain Routing (CIDR)/Internet Protocol (IP) range or to an Amazon EC2 security group. You can add as many as 20 ingress rules to an Amazon Redshift security group. If you authorize access to an Amazon EC2 security group, specify <i>EC2SecurityGroupName</i> and <i>EC2SecurityGroupOwnerId</i>. The Amazon EC2 security group and Amazon Redshift cluster must be in the same AWS Region.  If you authorize access to a CIDR/IP address range, specify <i>CIDRIP</i>. For an overview of CIDR blocks, see the Wikipedia article on <a href="http://en.wikipedia.org/wiki/Classless_Inter-Domain_Routing">Classless Inter-Domain Routing</a>.  You must also associate the security group with a cluster so that clients running on these IP addresses or the EC2 instance are authorized to connect to the cluster. For information about managing security groups, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-security-groups.html">Working with Security Groups</a> in the <i>Amazon Redshift Cluster Management Guide</i>.

        :param self:
        :param EC2SecurityGroupOwnerId:  The AWS account number of the owner of the security group specified by the <i>EC2SecurityGroupName</i> parameter. The AWS Access Key ID is not an acceptable value.  Example: <code>111122223333</code> <br/>EC2SecurityGroupName:  The EC2 security group to be added the Amazon Redshift security group.<br/>CIDRIP:  The IP range to be added the Amazon Redshift security group.<br/>ClusterSecurityGroupName:  The name of the security group to which the ingress rule is added.<br/>
        :return: ClusterSecurityGroup:  Describes a security group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The list of tags for the cluster security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IPRanges</i> A list of IP ranges (CIDR blocks) that are permitted to access clusters associated with this cluster security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EC2SecurityGroups</i> A list of EC2 security groups that are permitted to access clusters associated with this cluster security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of the security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSecurityGroupName</i> The name of the cluster security group to which the operation was applied.<br/>        """
        pass

    def authorize_snapshot_access(self, AccountWithRestoreAccess: str, SnapshotIdentifier: str, SnapshotClusterIdentifier: str=None) -> AuthorizeSnapshotAccessResult:
        """Authorizes the specified AWS customer account to restore the specified snapshot.  For more information about working with snapshots, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-snapshots.html">Amazon Redshift Snapshots</a> in the <i>Amazon Redshift Cluster Management Guide</i>.

        :param self:
        :param AccountWithRestoreAccess:  The identifier of the AWS customer account authorized to restore the specified snapshot. To share a snapshot with AWS support, specify amazon-redshift-support.<br/>SnapshotClusterIdentifier:  The identifier of the cluster the snapshot was created from. This parameter is required if your IAM user has a policy containing a snapshot resource element that specifies anything other than * for the cluster name.<br/>SnapshotIdentifier:  The identifier of the snapshot the account is authorized to restore.<br/>
        :return: Snapshot:  Describes a snapshot.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotRetentionStartTime</i> A timestamp representing the start of the retention period for the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRemainingDays</i> The number of days until a manual snapshot will pass its retention period.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The number of days that a manual snapshot is retained. If the value is -1, the manual snapshot is retained indefinitely.  The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track for the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestorableNodeTypes</i> The list of node types that this cluster snapshot is able to restore into.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The list of tags for the cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceRegion</i> The source region from which the snapshot was copied.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> The amount of time an in-progress snapshot backup has been running, or the amount of time it took a completed backup to finish.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedSecondsToCompletion</i> The estimate of the time remaining before the snapshot backup will complete. Returns <code>0</code> for a completed backup. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentBackupRateInMegaBytesPerSecond</i> The number of megabytes per second being transferred to the snapshot backup. Returns <code>0</code> for a completed backup. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupProgressInMegaBytes</i> The number of megabytes that have been transferred to the snapshot backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActualIncrementalBackupSizeInMegaBytes</i> The size of the incremental backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalBackupSizeInMegaBytes</i> The size of the complete set of backup data that would be used to restore the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerAccount</i> For manual snapshots, the AWS customer account used to create or copy the snapshot. For automatic snapshots, the owner of the cluster. The owner can perform all snapshot actions, such as sharing a manual snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountsWithRestoreAccess</i> A list of the AWS customer accounts authorized to restore the snapshot. Returns <code>null</code> if no accounts are authorized. Visible only to the snapshot owner. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptedWithHSM</i> A boolean that indicates whether the snapshot data is encrypted using the HSM keys of the source cluster. <code>true</code> indicates that the data is encrypted using HSM keys.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS Key Management Service (KMS) key ID of the encryption key that was used to encrypt data in the cluster from which the snapshot was taken.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encrypted</i> If <code>true</code>, the data in the snapshot is encrypted at rest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The VPC identifier of the cluster if the snapshot is from a cluster in a VPC. Otherwise, this field is not in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The name of the database that was created when the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The number of nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The node type of the nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotType</i> The snapshot type. Snapshots created using <a>CreateClusterSnapshot</a> and <a>CopyClusterSnapshot</a> are of type "manual". <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The version ID of the Amazon Redshift engine that is running on the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> The master user name for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> The time (UTC) when the cluster was originally created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone in which the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port that the cluster is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The snapshot status. The value of the status depends on the API operation used:  <ul> <li>  <a>CreateClusterSnapshot</a> and <a>CopyClusterSnapshot</a> returns status as "creating".  </li> <li>  <a>DescribeClusterSnapshots</a> returns status as "creating", "available", "final snapshot", or "failed". </li> <li>  <a>DeleteClusterSnapshot</a> returns status as "deleted". </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCreateTime</i> The time (in UTC format) when Amazon Redshift began the snapshot. A snapshot contains a copy of the cluster data as of this exact time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The identifier of the cluster for which the snapshot was taken.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotIdentifier</i> The snapshot identifier that is provided in the request.<br/>        """
        pass

    def batch_delete_cluster_snapshots(self, Identifiers: List[DeleteClusterSnapshotMessage]) -> BatchDeleteClusterSnapshotsResult:
        """Deletes a set of cluster snapshots.

        :param self:
        :param Identifiers:  A list of identifiers for the snapshots that you want to delete.<br/>
        :return: Errors:  A list of any errors returned.<br/>Resources:  A list of the snapshot identifiers that were deleted. <br/>        """
        pass

    def batch_modify_cluster_snapshots(self, SnapshotIdentifierList: List[String], ManualSnapshotRetentionPeriod: int=None, Force: Boolean=None) -> BatchModifyClusterSnapshotsOutputMessage:
        """Modifies the settings for a list of snapshots.

        :param self:
        :param Force:  A boolean value indicating whether to override an exception if the retention period has passed. <br/>ManualSnapshotRetentionPeriod:  The number of days that a manual snapshot is retained. If you specify the value -1, the manual snapshot is retained indefinitely. The number must be either -1 or an integer between 1 and 3,653. If you decrease the manual snapshot retention period from its current value, existing manual snapshots that fall outside of the new retention period will return an error. If you want to suppress the errors and delete the snapshots, use the force option. <br/>SnapshotIdentifierList:  A list of snapshot identifiers you want to modify.<br/>
        :return: Errors:  A list of any errors returned.<br/>Resources:  A list of the snapshots that were modified.<br/>        """
        pass

    def cancel_resize(self, ClusterIdentifier: str) -> ResizeProgressMessage:
        """Cancels a resize operation.

        :param self:
        :param ClusterIdentifier:  The unique identifier for the cluster that you want to cancel a resize operation for.<br/>
        :return: DataTransferProgressPercent:  The percent of data transferred from source cluster to target cluster.<br/>TargetEncryptionType:  The type of encryption for the cluster after the resize is complete. Possible values are <code>KMS</code> and <code>None</code>. In the China region possible values are: <code>Legacy</code> and <code>None</code>.<br/>Message:  An optional string to provide additional details about the resize action.<br/>ResizeType:  An enum with possible values of <code>ClassicResize</code> and <code>ElasticResize</code>. These values describe the type of resize operation being performed. <br/>EstimatedTimeToCompletionInSeconds:  The estimated time remaining, in seconds, until the resize operation is complete. This value is calculated based on the average resize rate and the estimated amount of data remaining to be processed. Once the resize operation is complete, this value will be 0.<br/>ElapsedTimeInSeconds:  The amount of seconds that have elapsed since the resize operation began. After the resize operation completes, this value shows the total actual time, in seconds, for the resize operation.<br/>ProgressInMegaBytes:  While the resize operation is in progress, this value shows the current amount of data, in megabytes, that has been processed so far. When the resize operation is complete, this value shows the total amount of data, in megabytes, on the cluster, which may be more or less than TotalResizeDataInMegaBytes (the estimated total amount of data before resize).<br/>TotalResizeDataInMegaBytes:  The estimated total amount of data, in megabytes, on the cluster before the resize operation began.<br/>AvgResizeRateInMegaBytesPerSecond:  The average rate of the resize operation over the last few minutes, measured in megabytes per second. After the resize operation completes, this value shows the average rate of the entire resize operation.<br/>ImportTablesNotStarted:  The names of tables that have not been yet imported. Valid Values: List of table names<br/>ImportTablesInProgress:  The names of tables that are being currently imported. Valid Values: List of table names.<br/>ImportTablesCompleted:  The names of tables that have been completely imported . Valid Values: List of table names.<br/>Status:  The status of the resize operation. Valid Values: <code>NONE</code> | <code>IN_PROGRESS</code> | <code>FAILED</code> | <code>SUCCEEDED</code> | <code>CANCELLING</code> <br/>TargetClusterType:  The cluster type after the resize operation is complete. Valid Values: <code>multi-node</code> | <code>single-node</code> <br/>TargetNumberOfNodes:  The number of nodes that the cluster will have after the resize operation is complete.<br/>TargetNodeType:  The node type that the cluster will have after the resize operation is complete.<br/>        """
        pass

    def copy_cluster_snapshot(self, TargetSnapshotIdentifier: str, SourceSnapshotIdentifier: str, SourceSnapshotClusterIdentifier: str=None, ManualSnapshotRetentionPeriod: int=None) -> CopyClusterSnapshotResult:
        """Copies the specified automated cluster snapshot to a new manual cluster snapshot. The source must be an automated snapshot and it must be in the available state. When you delete a cluster, Amazon Redshift deletes any automated snapshots of the cluster. Also, when the retention period of the snapshot expires, Amazon Redshift automatically deletes it. If you want to keep an automated snapshot for a longer period, you can make a manual copy of the snapshot. Manual snapshots are retained until you delete them.  For more information about working with snapshots, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-snapshots.html">Amazon Redshift Snapshots</a> in the <i>Amazon Redshift Cluster Management Guide</i>.

        :param self:
        :param ManualSnapshotRetentionPeriod:  The number of days that a manual snapshot is retained. If the value is -1, the manual snapshot is retained indefinitely.  The value must be either -1 or an integer between 1 and 3,653. The default value is -1.<br/>TargetSnapshotIdentifier:  The identifier given to the new manual snapshot. Constraints: <ul> <li> Cannot be null, empty, or blank. </li> <li> Must contain from 1 to 255 alphanumeric characters or hyphens. </li> <li> First character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> <li> Must be unique for the AWS account that is making the request. </li> </ul><br/>SourceSnapshotClusterIdentifier:  The identifier of the cluster the source snapshot was created from. This parameter is required if your IAM user has a policy containing a snapshot resource element that specifies anything other than * for the cluster name. Constraints: <ul> <li> Must be the identifier for a valid cluster. </li> </ul><br/>SourceSnapshotIdentifier:  The identifier for the source snapshot. Constraints: <ul> <li> Must be the identifier for a valid automated snapshot whose state is <code>available</code>. </li> </ul><br/>
        :return: Snapshot:  Describes a snapshot.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotRetentionStartTime</i> A timestamp representing the start of the retention period for the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRemainingDays</i> The number of days until a manual snapshot will pass its retention period.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The number of days that a manual snapshot is retained. If the value is -1, the manual snapshot is retained indefinitely.  The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track for the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestorableNodeTypes</i> The list of node types that this cluster snapshot is able to restore into.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The list of tags for the cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceRegion</i> The source region from which the snapshot was copied.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> The amount of time an in-progress snapshot backup has been running, or the amount of time it took a completed backup to finish.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedSecondsToCompletion</i> The estimate of the time remaining before the snapshot backup will complete. Returns <code>0</code> for a completed backup. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentBackupRateInMegaBytesPerSecond</i> The number of megabytes per second being transferred to the snapshot backup. Returns <code>0</code> for a completed backup. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupProgressInMegaBytes</i> The number of megabytes that have been transferred to the snapshot backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActualIncrementalBackupSizeInMegaBytes</i> The size of the incremental backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalBackupSizeInMegaBytes</i> The size of the complete set of backup data that would be used to restore the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerAccount</i> For manual snapshots, the AWS customer account used to create or copy the snapshot. For automatic snapshots, the owner of the cluster. The owner can perform all snapshot actions, such as sharing a manual snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountsWithRestoreAccess</i> A list of the AWS customer accounts authorized to restore the snapshot. Returns <code>null</code> if no accounts are authorized. Visible only to the snapshot owner. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptedWithHSM</i> A boolean that indicates whether the snapshot data is encrypted using the HSM keys of the source cluster. <code>true</code> indicates that the data is encrypted using HSM keys.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS Key Management Service (KMS) key ID of the encryption key that was used to encrypt data in the cluster from which the snapshot was taken.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encrypted</i> If <code>true</code>, the data in the snapshot is encrypted at rest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The VPC identifier of the cluster if the snapshot is from a cluster in a VPC. Otherwise, this field is not in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The name of the database that was created when the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The number of nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The node type of the nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotType</i> The snapshot type. Snapshots created using <a>CreateClusterSnapshot</a> and <a>CopyClusterSnapshot</a> are of type "manual". <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The version ID of the Amazon Redshift engine that is running on the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> The master user name for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> The time (UTC) when the cluster was originally created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone in which the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port that the cluster is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The snapshot status. The value of the status depends on the API operation used:  <ul> <li>  <a>CreateClusterSnapshot</a> and <a>CopyClusterSnapshot</a> returns status as "creating".  </li> <li>  <a>DescribeClusterSnapshots</a> returns status as "creating", "available", "final snapshot", or "failed". </li> <li>  <a>DeleteClusterSnapshot</a> returns status as "deleted". </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCreateTime</i> The time (in UTC format) when Amazon Redshift began the snapshot. A snapshot contains a copy of the cluster data as of this exact time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The identifier of the cluster for which the snapshot was taken.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotIdentifier</i> The snapshot identifier that is provided in the request.<br/>        """
        pass

    def create_cluster(self, MasterUserPassword: str, MasterUsername: str, NodeType: str, ClusterIdentifier: str, DBName: str=None, ClusterType: str=None, ClusterSecurityGroups: List[String]=None, VpcSecurityGroupIds: List[String]=None, ClusterSubnetGroupName: str=None, AvailabilityZone: str=None, PreferredMaintenanceWindow: str=None, ClusterParameterGroupName: str=None, AutomatedSnapshotRetentionPeriod: int=None, ManualSnapshotRetentionPeriod: int=None, Port: int=None, ClusterVersion: str=None, AllowVersionUpgrade: BooleanOptional=None, NumberOfNodes: int=None, PubliclyAccessible: BooleanOptional=None, Encrypted: BooleanOptional=None, HsmClientCertificateIdentifier: str=None, HsmConfigurationIdentifier: str=None, ElasticIp: str=None, Tags: List[Tag]=None, KmsKeyId: str=None, EnhancedVpcRouting: BooleanOptional=None, AdditionalInfo: str=None, IamRoles: List[String]=None, MaintenanceTrackName: str=None, SnapshotScheduleIdentifier: str=None) -> CreateClusterResult:
        """Creates a new cluster. To create a cluster in Virtual Private Cloud (VPC), you must provide a cluster subnet group name. The cluster subnet group identifies the subnets of your VPC that Amazon Redshift uses when creating the cluster. For more information about managing clusters, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-clusters.html">Amazon Redshift Clusters</a> in the <i>Amazon Redshift Cluster Management Guide</i>.

        :param self:
        :param SnapshotScheduleIdentifier:  A unique identifier for the snapshot schedule.<br/>MaintenanceTrackName:  An optional parameter for the name of the maintenance track for the cluster. If you don't provide a maintenance track name, the cluster is assigned to the <code>current</code> track.<br/>IamRoles:  A list of AWS Identity and Access Management (IAM) roles that can be used by the cluster to access other AWS services. You must supply the IAM roles in their Amazon Resource Name (ARN) format. You can supply up to 10 IAM roles in a single request. A cluster can have up to 10 IAM roles associated with it at any time.<br/>AdditionalInfo:  Reserved.<br/>EnhancedVpcRouting:  An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>KmsKeyId:  The AWS Key Management Service (KMS) key ID of the encryption key that you want to use to encrypt data in the cluster.<br/>Tags:  A list of tag instances.<br/>ElasticIp:  The Elastic IP (EIP) address for the cluster. Constraints: The cluster must be provisioned in EC2-VPC and publicly-accessible through an Internet gateway. For more information about provisioning clusters in EC2-VPC, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-clusters.html#cluster-platforms">Supported Platforms to Launch Your Cluster</a> in the Amazon Redshift Cluster Management Guide.<br/>HsmConfigurationIdentifier:  Specifies the name of the HSM configuration that contains the information the Amazon Redshift cluster can use to retrieve and store keys in an HSM.<br/>HsmClientCertificateIdentifier:  Specifies the name of the HSM client certificate the Amazon Redshift cluster uses to retrieve the data encryption keys stored in an HSM.<br/>Encrypted:  If <code>true</code>, the data in the cluster is encrypted at rest.  Default: false<br/>PubliclyAccessible:  If <code>true</code>, the cluster can be accessed from a public network. <br/>NumberOfNodes:  The number of compute nodes in the cluster. This parameter is required when the <b>ClusterType</b> parameter is specified as <code>multi-node</code>.  For information about determining how many nodes you need, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-clusters.html#how-many-nodes"> Working with Clusters</a> in the <i>Amazon Redshift Cluster Management Guide</i>.  If you don't specify this parameter, you get a single-node cluster. When requesting a multi-node cluster, you must specify the number of nodes that you want in the cluster. Default: <code>1</code>  Constraints: Value must be at least 1 and no more than 100.<br/>AllowVersionUpgrade:  If <code>true</code>, major version upgrades can be applied during the maintenance window to the Amazon Redshift engine that is running on the cluster. When a new major version of the Amazon Redshift engine is released, you can request that the service automatically apply upgrades during the maintenance window to the Amazon Redshift engine that is running on your cluster. Default: <code>true</code> <br/>ClusterVersion:  The version of the Amazon Redshift engine software that you want to deploy on the cluster. The version selected runs on all the nodes in the cluster. Constraints: Only version 1.0 is currently available. Example: <code>1.0</code> <br/>Port:  The port number on which the cluster accepts incoming connections. The cluster is accessible only via the JDBC and ODBC connection strings. Part of the connection string requires the port on which the cluster will listen for incoming connections. Default: <code>5439</code>  Valid Values: <code>1150-65535</code> <br/>ManualSnapshotRetentionPeriod:  The default number of days to retain a manual snapshot. If the value is -1, the snapshot is retained indefinitely. This setting doesn't change the retention period of existing snapshots. The value must be either -1 or an integer between 1 and 3,653.<br/>AutomatedSnapshotRetentionPeriod:  The number of days that automated snapshots are retained. If the value is 0, automated snapshots are disabled. Even if automated snapshots are disabled, you can still create manual snapshots when you want with <a>CreateClusterSnapshot</a>.  Default: <code>1</code>  Constraints: Must be a value from 0 to 35.<br/>ClusterParameterGroupName:  The name of the parameter group to be associated with this cluster. Default: The default Amazon Redshift cluster parameter group. For information about the default parameter group, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-parameter-groups.html">Working with Amazon Redshift Parameter Groups</a>  Constraints: <ul> <li> Must be 1 to 255 alphanumeric characters or hyphens. </li> <li> First character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul><br/>PreferredMaintenanceWindow:  The weekly time range (in UTC) during which automated cluster maintenance can occur.  Format: <code>ddd:hh24:mi-ddd:hh24:mi</code>   Default: A 30-minute window selected at random from an 8-hour block of time per region, occurring on a random day of the week. For more information about the time blocks for each region, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-clusters.html#rs-maintenance-windows">Maintenance Windows</a> in Amazon Redshift Cluster Management Guide. Valid Days: Mon | Tue | Wed | Thu | Fri | Sat | Sun Constraints: Minimum 30-minute window.<br/>AvailabilityZone:  The EC2 Availability Zone (AZ) in which you want Amazon Redshift to provision the cluster. For example, if you have several EC2 instances running in a specific Availability Zone, then you might want the cluster to be provisioned in the same zone in order to decrease network latency. Default: A random, system-chosen Availability Zone in the region that is specified by the endpoint. Example: <code>us-east-1d</code>  Constraint: The specified Availability Zone must be in the same region as the current endpoint.<br/>ClusterSubnetGroupName:  The name of a cluster subnet group to be associated with this cluster. If this parameter is not provided the resulting cluster will be deployed outside virtual private cloud (VPC).<br/>VpcSecurityGroupIds:  A list of Virtual Private Cloud (VPC) security groups to be associated with the cluster. Default: The default VPC security group is associated with the cluster.<br/>ClusterSecurityGroups:  A list of security groups to be associated with this cluster. Default: The default cluster security group for Amazon Redshift.<br/>MasterUserPassword:  The password associated with the master user account for the cluster that is being created. Constraints: <ul> <li> Must be between 8 and 64 characters in length. </li> <li> Must contain at least one uppercase letter. </li> <li> Must contain at least one lowercase letter. </li> <li> Must contain one number. </li> <li> Can be any printable ASCII character (ASCII code 33 to 126) except ' (single quote), " (double quote), \, /, @, or space. </li> </ul><br/>MasterUsername:  The user name associated with the master user account for the cluster that is being created. Constraints: <ul> <li> Must be 1 - 128 alphanumeric characters. The user name can't be <code>PUBLIC</code>. </li> <li> First character must be a letter. </li> <li> Cannot be a reserved word. A list of reserved words can be found in <a href="https://docs.aws.amazon.com/redshift/latest/dg/r_pg_keywords.html">Reserved Words</a> in the Amazon Redshift Database Developer Guide.  </li> </ul><br/>NodeType:  The node type to be provisioned for the cluster. For information about node types, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-clusters.html#how-many-nodes"> Working with Clusters</a> in the <i>Amazon Redshift Cluster Management Guide</i>.  Valid Values: <code>ds2.xlarge</code> | <code>ds2.8xlarge</code> | <code>ds2.xlarge</code> | <code>ds2.8xlarge</code> | <code>dc1.large</code> | <code>dc1.8xlarge</code> | <code>dc2.large</code> | <code>dc2.8xlarge</code> <br/>ClusterType:  The type of the cluster. When cluster type is specified as <ul> <li>  <code>single-node</code>, the <b>NumberOfNodes</b> parameter is not required. </li> <li>  <code>multi-node</code>, the <b>NumberOfNodes</b> parameter is required. </li> </ul> Valid Values: <code>multi-node</code> | <code>single-node</code>  Default: <code>multi-node</code> <br/>ClusterIdentifier:  A unique identifier for the cluster. You use this identifier to refer to the cluster for any subsequent cluster operations such as deleting or modifying. The identifier also appears in the Amazon Redshift console. Constraints: <ul> <li> Must contain from 1 to 63 alphanumeric characters or hyphens. </li> <li> Alphabetic characters must be lowercase. </li> <li> First character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> <li> Must be unique for all clusters within an AWS account. </li> </ul> Example: <code>myexamplecluster</code> <br/>DBName:  The name of the first database to be created when the cluster is created. To create additional databases after the cluster is created, connect to the cluster with a SQL client and use SQL commands to create a database. For more information, go to <a href="https://docs.aws.amazon.com/redshift/latest/dg/t_creating_database.html">Create a Database</a> in the Amazon Redshift Database Developer Guide.  Default: <code>dev</code>  Constraints: <ul> <li> Must contain 1 to 64 alphanumeric characters. </li> <li> Must contain only lowercase letters. </li> <li> Cannot be a word that is reserved by the service. A list of reserved words can be found in <a href="https://docs.aws.amazon.com/redshift/latest/dg/r_pg_keywords.html">Reserved Words</a> in the Amazon Redshift Database Developer Guide.  </li> </ul><br/>
        :return: Cluster:  Describes a cluster.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResizeInfo</i> Returns the following: <ul> <li> AllowCancelResize: a boolean value indicating if the resize operation can be cancelled. </li> <li> ResizeType: Returns ClassicResize </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResizeInfo</b>: Returns the following: <ul> <li> AllowCancelResize: a boolean value indicating if the resize operation can be cancelled. </li> <li> ResizeType: Returns ClassicResize </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotScheduleState</i> The current state of the cluster snapshot schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotScheduleIdentifier</i> A unique identifier for the cluster snapshot schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeferredMaintenanceWindows</i> Describes a group of <code>DeferredMaintenanceWindow</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticResizeNumberOfNodeOptions</i> The number of nodes that you can resize the cluster to with the elastic resize method. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingActions</i> Cluster operations that are waiting to be started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoles</i> A list of AWS Identity and Access Management (IAM) roles that can be used by the cluster to access other AWS services.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS Key Management Service (AWS KMS) key ID of the encryption key used to encrypt data in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The list of tags for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterRevisionNumber</i> The specific revision number of the database in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticIpStatus</i> The status of the elastic IP (EIP) address.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ElasticIpStatus</b>: The status of the elastic IP (EIP) address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterNodes</i> The nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterPublicKey</i> The public key for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSnapshotCopyStatus</i> A value that returns the destination region and retention period that are configured for cross-region snapshot copy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ClusterSnapshotCopyStatus</b>: A value that returns the destination region and retention period that are configured for cross-region snapshot copy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmStatus</i> A value that reports whether the Amazon Redshift cluster has finished applying any hardware security module (HSM) settings changes specified in a modify cluster command. Values: active, applying&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>HsmStatus</b>: A value that reports whether the Amazon Redshift cluster has finished applying any hardware security module (HSM) settings changes specified in a modify cluster command. Values: active, applying<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTransferProgress</i> <p/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DataTransferProgress</b>: <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreStatus</i> A value that describes the status of a cluster restore action. This parameter returns null if the cluster was not created by restoring a snapshot.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RestoreStatus</b>: A value that describes the status of a cluster restore action. This parameter returns null if the cluster was not created by restoring a snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encrypted</i> A boolean value that, if <code>true</code>, indicates that data in the cluster is encrypted at rest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> A boolean value that, if <code>true</code>, indicates that the cluster can be accessed from a public network.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The number of compute nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowVersionUpgrade</i> A boolean value that, if <code>true</code>, indicates that major version upgrades will be applied automatically to the cluster during the maintenance window. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The version ID of the Amazon Redshift engine that is running on the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> A value that, if present, indicates that changes to the cluster are pending. Specific pending changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: A value that, if present, indicates that changes to the cluster are pending. Specific pending changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> The weekly time range, in Universal Coordinated Time (UTC), during which system maintenance can occur.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The name of the Availability Zone in which the cluster is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The identifier of the VPC the cluster is in, if the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSubnetGroupName</i> The name of the subnet group that is associated with the cluster. This parameter is valid only when the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterParameterGroups</i> The list of cluster parameter groups that are associated with this cluster. Each parameter group in the list is returned with its status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> A list of Amazon Virtual Private Cloud (Amazon VPC) security groups that are associated with the cluster. This parameter is returned only if the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSecurityGroups</i> A list of cluster security group that are associated with the cluster. Each security group is represented by an element that contains <code>ClusterSecurityGroup.Name</code> and <code>ClusterSecurityGroup.Status</code> subelements.  Cluster security groups are used when the cluster is not created in an Amazon Virtual Private Cloud (VPC). Clusters that are created in a VPC use VPC security groups, which are listed by the <b>VpcSecurityGroups</b> parameter. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The default number of days to retain a manual snapshot. If the value is -1, the snapshot is retained indefinitely. This setting doesn't change the retention period of existing snapshots. The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotRetentionPeriod</i> The number of days that automatic cluster snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> The date and time that the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> The connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: The connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The name of the initial database that was created when the cluster was created. This same name is returned for the life of the cluster. If an initial database was not specified, a database named <code>dev</code>dev was created by default. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> The master user name for the cluster. This name is used to connect to the database that is specified in the <b>DBName</b> parameter. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModifyStatus</i> The status of a modify operation, if any, initiated for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterAvailabilityStatus</i> The availability status of the cluster for queries. Possible values are the following: <ul> <li> Available - The cluster is available for queries.  </li> <li> Unavailable - The cluster is not available for queries. </li> <li> Maintenance - The cluster is intermittently available for queries due to maintenance activities. </li> <li> Modifying - The cluster is intermittently available for queries due to changes that modify the cluster. </li> <li> Failed - The cluster failed and is not available for queries. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterStatus</i>  The current state of the cluster. Possible values are the following: <ul> <li>  <code>available</code>  </li> <li>  <code>available, prep-for-resize</code>  </li> <li>  <code>available, resize-cleanup</code>  </li> <li>  <code>cancelling-resize</code>  </li> <li>  <code>creating</code>  </li> <li>  <code>deleting</code>  </li> <li>  <code>final-snapshot</code>  </li> <li>  <code>hardware-failure</code>  </li> <li>  <code>incompatible-hsm</code>  </li> <li>  <code>incompatible-network</code>  </li> <li>  <code>incompatible-parameters</code>  </li> <li>  <code>incompatible-restore</code>  </li> <li>  <code>modifying</code>  </li> <li>  <code>rebooting</code>  </li> <li>  <code>renaming</code>  </li> <li>  <code>resizing</code>  </li> <li>  <code>rotating-keys</code>  </li> <li>  <code>storage-full</code>  </li> <li>  <code>updating-hsm</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The node type for the nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The unique identifier of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS address of the Cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionType</i> The encryption type for a cluster. Possible values are: KMS and None. For the China region the possible values are None, and Legacy. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track that the cluster will change to during the next maintenance window.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> The pending or in-progress change of the ability to connect to the cluster from the public network.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The pending or in-progress change of the new identifier for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotRetentionPeriod</i> The pending or in-progress change of the automated snapshot retention period.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The pending or in-progress change of the service version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterType</i> The pending or in-progress change of the cluster type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The pending or in-progress change of the number of nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The pending or in-progress change of the cluster's node type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> The pending or in-progress change of the master user password for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedTimeToCompletionInSeconds</i> The estimate of the time remaining before the restore will complete. Returns 0 for a completed restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> The amount of time an in-progress restore has been running, or the amount of time it took a completed restore to finish.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ProgressInMegaBytes</i> The number of megabytes that have been transferred from snapshot storage.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotSizeInMegaBytes</i> The size of the set of snapshot data used to restore the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentRestoreRateInMegaBytesPerSecond</i> The number of megabytes per second being transferred from the backup storage. Returns the average rate for a completed backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the restore action. Returns starting, restoring, completed, or failed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> Describes the number of seconds that have elapsed during the data transfer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedTimeToCompletionInSeconds</i> Describes the estimated number of seconds remaining to complete the transfer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTransferredInMegaBytes</i> Describes the total amount of data that has been transfered in MB's.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalDataInMegaBytes</i> Describes the total amount of data to be transfered in megabytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentRateInMegaBytesPerSecond</i> Describes the data transfer rate in MB's per second.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Describes the status of the cluster. While the transfer is in progress the status is <code>transferringdata</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Reports whether the Amazon Redshift cluster has finished applying any HSM settings changes specified in a modify cluster command. Values: active, applying<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmConfigurationIdentifier</i> Specifies the name of the HSM configuration that contains the information the Amazon Redshift cluster can use to retrieve and store keys in an HSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmClientCertificateIdentifier</i> Specifies the name of the HSM client certificate the Amazon Redshift cluster uses to retrieve the data encryption keys stored in an HSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCopyGrantName</i> The name of the snapshot copy grant.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The number of days that automated snapshots are retained in the destination region after they are copied from a source region. If the value is -1, the manual snapshot is retained indefinitely.  The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RetentionPeriod</i> The number of days that automated snapshots are retained in the destination region after they are copied from a source region.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationRegion</i> The destination region that snapshots are automatically copied to when cross-region snapshot copy is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the elastic IP (EIP) address.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticIp</i> The elastic IP (EIP) address for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowCancelResize</i> A boolean value indicating if the resize operation can be cancelled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResizeType</i> Returns the value <code>ClassicResize</code>.<br/>        """
        pass

    def create_cluster_parameter_group(self, Description: str, ParameterGroupFamily: str, ParameterGroupName: str, Tags: List[Tag]=None) -> CreateClusterParameterGroupResult:
        """Creates an Amazon Redshift parameter group. Creating parameter groups is independent of creating clusters. You can associate a cluster with a parameter group when you create the cluster. You can also associate an existing cluster with a parameter group after the cluster is created by using <a>ModifyCluster</a>.  Parameters in the parameter group define specific behavior that applies to the databases you create on the cluster. For more information about parameters and parameter groups, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-parameter-groups.html">Amazon Redshift Parameter Groups</a> in the <i>Amazon Redshift Cluster Management Guide</i>.

        :param self:
        :param Tags:  A list of tag instances.<br/>Description:  A description of the parameter group.<br/>ParameterGroupFamily:  The Amazon Redshift engine version to which the cluster parameter group applies. The cluster engine version determines the set of parameters. To get a list of valid parameter group family names, you can call <a>DescribeClusterParameterGroups</a>. By default, Amazon Redshift returns a list of all the parameter groups that are owned by your AWS account, including the default parameter groups for each Amazon Redshift engine version. The parameter group family names associated with the default parameter groups provide you the valid values. For example, a valid family name is "redshift-1.0". <br/>ParameterGroupName:  The name of the cluster parameter group. Constraints: <ul> <li> Must be 1 to 255 alphanumeric characters or hyphens </li> <li> First character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> <li> Must be unique withing your AWS account. </li> </ul> <note> This value is stored as a lower-case string. </note><br/>
        :return: ClusterParameterGroup:  Describes a parameter group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The list of tags for the cluster parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the parameter group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterGroupFamily</i> The name of the cluster parameter group family that this cluster parameter group is compatible with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterGroupName</i> The name of the cluster parameter group.<br/>        """
        pass

    def create_cluster_security_group(self, Description: str, ClusterSecurityGroupName: str, Tags: List[Tag]=None) -> CreateClusterSecurityGroupResult:
        """Creates a new Amazon Redshift security group. You use security groups to control access to non-VPC clusters.  For information about managing security groups, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-security-groups.html">Amazon Redshift Cluster Security Groups</a> in the <i>Amazon Redshift Cluster Management Guide</i>.

        :param self:
        :param Tags:  A list of tag instances.<br/>Description:  A description for the security group.<br/>ClusterSecurityGroupName:  The name for the security group. Amazon Redshift stores the value as a lowercase string. Constraints: <ul> <li> Must contain no more than 255 alphanumeric characters or hyphens. </li> <li> Must not be "Default". </li> <li> Must be unique for all security groups that are created by your AWS account. </li> </ul> Example: <code>examplesecuritygroup</code> <br/>
        :return: ClusterSecurityGroup:  Describes a security group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The list of tags for the cluster security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IPRanges</i> A list of IP ranges (CIDR blocks) that are permitted to access clusters associated with this cluster security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EC2SecurityGroups</i> A list of EC2 security groups that are permitted to access clusters associated with this cluster security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of the security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSecurityGroupName</i> The name of the cluster security group to which the operation was applied.<br/>        """
        pass

    def create_cluster_snapshot(self, ClusterIdentifier: str, SnapshotIdentifier: str, ManualSnapshotRetentionPeriod: int=None, Tags: List[Tag]=None) -> CreateClusterSnapshotResult:
        """Creates a manual snapshot of the specified cluster. The cluster must be in the <code>available</code> state.   For more information about working with snapshots, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-snapshots.html">Amazon Redshift Snapshots</a> in the <i>Amazon Redshift Cluster Management Guide</i>.

        :param self:
        :param Tags:  A list of tag instances.<br/>ManualSnapshotRetentionPeriod:  The number of days that a manual snapshot is retained. If the value is -1, the manual snapshot is retained indefinitely.  The value must be either -1 or an integer between 1 and 3,653. The default value is -1.<br/>ClusterIdentifier:  The cluster identifier for which you want a snapshot.<br/>SnapshotIdentifier:  A unique identifier for the snapshot that you are requesting. This identifier must be unique for all snapshots within the AWS account. Constraints: <ul> <li> Cannot be null, empty, or blank </li> <li> Must contain from 1 to 255 alphanumeric characters or hyphens </li> <li> First character must be a letter </li> <li> Cannot end with a hyphen or contain two consecutive hyphens </li> </ul> Example: <code>my-snapshot-id</code> <br/>
        :return: Snapshot:  Describes a snapshot.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotRetentionStartTime</i> A timestamp representing the start of the retention period for the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRemainingDays</i> The number of days until a manual snapshot will pass its retention period.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The number of days that a manual snapshot is retained. If the value is -1, the manual snapshot is retained indefinitely.  The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track for the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestorableNodeTypes</i> The list of node types that this cluster snapshot is able to restore into.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The list of tags for the cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceRegion</i> The source region from which the snapshot was copied.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> The amount of time an in-progress snapshot backup has been running, or the amount of time it took a completed backup to finish.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedSecondsToCompletion</i> The estimate of the time remaining before the snapshot backup will complete. Returns <code>0</code> for a completed backup. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentBackupRateInMegaBytesPerSecond</i> The number of megabytes per second being transferred to the snapshot backup. Returns <code>0</code> for a completed backup. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupProgressInMegaBytes</i> The number of megabytes that have been transferred to the snapshot backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActualIncrementalBackupSizeInMegaBytes</i> The size of the incremental backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalBackupSizeInMegaBytes</i> The size of the complete set of backup data that would be used to restore the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerAccount</i> For manual snapshots, the AWS customer account used to create or copy the snapshot. For automatic snapshots, the owner of the cluster. The owner can perform all snapshot actions, such as sharing a manual snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountsWithRestoreAccess</i> A list of the AWS customer accounts authorized to restore the snapshot. Returns <code>null</code> if no accounts are authorized. Visible only to the snapshot owner. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptedWithHSM</i> A boolean that indicates whether the snapshot data is encrypted using the HSM keys of the source cluster. <code>true</code> indicates that the data is encrypted using HSM keys.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS Key Management Service (KMS) key ID of the encryption key that was used to encrypt data in the cluster from which the snapshot was taken.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encrypted</i> If <code>true</code>, the data in the snapshot is encrypted at rest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The VPC identifier of the cluster if the snapshot is from a cluster in a VPC. Otherwise, this field is not in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The name of the database that was created when the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The number of nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The node type of the nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotType</i> The snapshot type. Snapshots created using <a>CreateClusterSnapshot</a> and <a>CopyClusterSnapshot</a> are of type "manual". <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The version ID of the Amazon Redshift engine that is running on the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> The master user name for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> The time (UTC) when the cluster was originally created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone in which the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port that the cluster is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The snapshot status. The value of the status depends on the API operation used:  <ul> <li>  <a>CreateClusterSnapshot</a> and <a>CopyClusterSnapshot</a> returns status as "creating".  </li> <li>  <a>DescribeClusterSnapshots</a> returns status as "creating", "available", "final snapshot", or "failed". </li> <li>  <a>DeleteClusterSnapshot</a> returns status as "deleted". </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCreateTime</i> The time (in UTC format) when Amazon Redshift began the snapshot. A snapshot contains a copy of the cluster data as of this exact time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The identifier of the cluster for which the snapshot was taken.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotIdentifier</i> The snapshot identifier that is provided in the request.<br/>        """
        pass

    def create_cluster_subnet_group(self, SubnetIds: List[String], Description: str, ClusterSubnetGroupName: str, Tags: List[Tag]=None) -> CreateClusterSubnetGroupResult:
        """Creates a new Amazon Redshift subnet group. You must provide a list of one or more subnets in your existing Amazon Virtual Private Cloud (Amazon VPC) when creating Amazon Redshift subnet group.  For information about subnet groups, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-cluster-subnet-groups.html">Amazon Redshift Cluster Subnet Groups</a> in the <i>Amazon Redshift Cluster Management Guide</i>.

        :param self:
        :param Tags:  A list of tag instances.<br/>SubnetIds:  An array of VPC subnet IDs. A maximum of 20 subnets can be modified in a single request.<br/>Description:  A description for the subnet group.<br/>ClusterSubnetGroupName:  The name for the subnet group. Amazon Redshift stores the value as a lowercase string. Constraints: <ul> <li> Must contain no more than 255 alphanumeric characters or hyphens. </li> <li> Must not be "Default". </li> <li> Must be unique for all subnet groups that are created by your AWS account. </li> </ul> Example: <code>examplesubnetgroup</code> <br/>
        :return: ClusterSubnetGroup:  Describes a subnet group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The list of tags for the cluster subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> A list of the VPC <a>Subnet</a> elements. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> The status of the cluster subnet group. Possible values are <code>Complete</code>, <code>Incomplete</code> and <code>Invalid</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The VPC ID of the cluster subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the cluster subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSubnetGroupName</i> The name of the cluster subnet group.<br/>        """
        pass

    def create_event_subscription(self, SnsTopicArn: str, SubscriptionName: str, SourceType: str=None, SourceIds: List[String]=None, EventCategories: List[String]=None, Severity: str=None, Enabled: BooleanOptional=None, Tags: List[Tag]=None) -> CreateEventSubscriptionResult:
        """Creates an Amazon Redshift event notification subscription. This action requires an ARN (Amazon Resource Name) of an Amazon SNS topic created by either the Amazon Redshift console, the Amazon SNS console, or the Amazon SNS API. To obtain an ARN with Amazon SNS, you must create a topic in Amazon SNS and subscribe to the topic. The ARN is displayed in the SNS console. You can specify the source type, and lists of Amazon Redshift source IDs, event categories, and event severities. Notifications will be sent for all events you want that match those criteria. For example, you can specify source type = cluster, source ID = my-cluster-1 and mycluster2, event categories = Availability, Backup, and severity = ERROR. The subscription will only send notifications for those ERROR events in the Availability and Backup categories for the specified clusters. If you specify both the source type and source IDs, such as source type = cluster and source identifier = my-cluster-1, notifications will be sent for all the cluster events for my-cluster-1. If you specify a source type but do not specify a source identifier, you will receive notice of the events for the objects of that type in your AWS account. If you do not specify either the SourceType nor the SourceIdentifier, you will be notified of events generated from all Amazon Redshift sources belonging to your AWS account. You must specify a source type if you specify a source ID.

        :param self:
        :param Tags:  A list of tag instances.<br/>Enabled:  A boolean value; set to <code>true</code> to activate the subscription, and set to <code>false</code> to create the subscription but not activate it. <br/>Severity:  Specifies the Amazon Redshift event severity to be published by the event notification subscription. Values: ERROR, INFO<br/>EventCategories:  Specifies the Amazon Redshift event categories to be published by the event notification subscription. Values: configuration, management, monitoring, security<br/>SourceIds:  A list of one or more identifiers of Amazon Redshift source objects. All of the objects must be of the same type as was specified in the source type parameter. The event subscription will return only events generated by the specified objects. If not specified, then events are returned for all objects within the source type specified. Example: my-cluster-1, my-cluster-2 Example: my-snapshot-20131010<br/>SourceType:  The type of source that will be generating the events. For example, if you want to be notified of events generated by a cluster, you would set this parameter to cluster. If this value is not specified, events are returned for all Amazon Redshift objects in your AWS account. You must specify a source type in order to specify source IDs. Valid values: cluster, cluster-parameter-group, cluster-security-group, and cluster-snapshot.<br/>SnsTopicArn:  The Amazon Resource Name (ARN) of the Amazon SNS topic used to transmit the event notifications. The ARN is created by Amazon SNS when you create a topic and subscribe to it.<br/>SubscriptionName:  The name of the event subscription to be created. Constraints: <ul> <li> Cannot be null, empty, or blank. </li> <li> Must contain from 1 to 255 alphanumeric characters or hyphens. </li> <li> First character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul><br/>
        :return: EventSubscription:  Describes event subscriptions.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The list of tags for the event subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> A boolean value indicating whether the subscription is enabled; <code>true</code> indicates that the subscription is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Severity</i> The event severity specified in the Amazon Redshift event notification subscription. Values: ERROR, INFO<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EventCategoriesList</i> The list of Amazon Redshift event categories specified in the event notification subscription. Values: Configuration, Management, Monitoring, Security<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceIdsList</i> A list of the sources that publish events to the Amazon Redshift event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceType</i> The source type of the events returned the Amazon Redshift event notification, such as cluster, or cluster-snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubscriptionCreationTime</i> The date and time the Amazon Redshift event notification subscription was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the Amazon Redshift event notification subscription. Constraints: <ul> <li> Can be one of the following: active | no-permission | topic-not-exist </li> <li> The status "no-permission" indicates that Amazon Redshift no longer has permission to post to the Amazon SNS topic. The status "topic-not-exist" indicates that the topic was deleted after the subscription was created. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsTopicArn</i> The Amazon Resource Name (ARN) of the Amazon SNS topic used by the event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustSubscriptionId</i> The name of the Amazon Redshift event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomerAwsId</i> The AWS customer account associated with the Amazon Redshift event notification subscription.<br/>        """
        pass

    def create_hsm_client_certificate(self, HsmClientCertificateIdentifier: str, Tags: List[Tag]=None) -> CreateHsmClientCertificateResult:
        """Creates an HSM client certificate that an Amazon Redshift cluster will use to connect to the client's HSM in order to store and retrieve the keys used to encrypt the cluster databases. The command returns a public key, which you must store in the HSM. In addition to creating the HSM certificate, you must create an Amazon Redshift HSM configuration that provides a cluster the information needed to store and use encryption keys in the HSM. For more information, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-HSM.html">Hardware Security Modules</a> in the Amazon Redshift Cluster Management Guide.

        :param self:
        :param Tags:  A list of tag instances.<br/>HsmClientCertificateIdentifier:  The identifier to be assigned to the new HSM client certificate that the cluster will use to connect to the HSM to use the database encryption keys.<br/>
        :return: HsmClientCertificate:  Returns information about an HSM client certificate. The certificate is stored in a secure Hardware Storage Module (HSM), and used by the Amazon Redshift cluster to encrypt data files.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The list of tags for the HSM client certificate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmClientCertificatePublicKey</i> The public key that the Amazon Redshift cluster will use to connect to the HSM. You must register the public key in the HSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmClientCertificateIdentifier</i> The identifier of the HSM client certificate.<br/>        """
        pass

    def create_hsm_configuration(self, HsmServerPublicCertificate: str, HsmPartitionPassword: str, HsmPartitionName: str, HsmIpAddress: str, Description: str, HsmConfigurationIdentifier: str, Tags: List[Tag]=None) -> CreateHsmConfigurationResult:
        """Creates an HSM configuration that contains the information required by an Amazon Redshift cluster to store and use database encryption keys in a Hardware Security Module (HSM). After creating the HSM configuration, you can specify it as a parameter when creating a cluster. The cluster will then store its encryption keys in the HSM. In addition to creating an HSM configuration, you must also create an HSM client certificate. For more information, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-HSM.html">Hardware Security Modules</a> in the Amazon Redshift Cluster Management Guide.

        :param self:
        :param Tags:  A list of tag instances.<br/>HsmServerPublicCertificate:  The HSMs public certificate file. When using Cloud HSM, the file name is server.pem.<br/>HsmPartitionPassword:  The password required to access the HSM partition.<br/>HsmPartitionName:  The name of the partition in the HSM where the Amazon Redshift clusters will store their database encryption keys.<br/>HsmIpAddress:  The IP address that the Amazon Redshift cluster must use to access the HSM.<br/>Description:  A text description of the HSM configuration to be created.<br/>HsmConfigurationIdentifier:  The identifier to be assigned to the new Amazon Redshift HSM configuration.<br/>
        :return: HsmConfiguration:  Returns information about an HSM configuration, which is an object that describes to Amazon Redshift clusters the information they require to connect to an HSM where they can store database encryption keys.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The list of tags for the HSM configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmPartitionName</i> The name of the partition in the HSM where the Amazon Redshift clusters will store their database encryption keys.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmIpAddress</i> The IP address that the Amazon Redshift cluster must use to access the HSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A text description of the HSM configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmConfigurationIdentifier</i> The name of the Amazon Redshift HSM configuration.<br/>        """
        pass

    def create_snapshot_copy_grant(self, SnapshotCopyGrantName: str, KmsKeyId: str=None, Tags: List[Tag]=None) -> CreateSnapshotCopyGrantResult:
        """Creates a snapshot copy grant that permits Amazon Redshift to use a customer master key (CMK) from AWS Key Management Service (AWS KMS) to encrypt copied snapshots in a destination region.  For more information about managing snapshot copy grants, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-db-encryption.html">Amazon Redshift Database Encryption</a> in the <i>Amazon Redshift Cluster Management Guide</i>. 

        :param self:
        :param Tags:  A list of tag instances.<br/>KmsKeyId:  The unique identifier of the customer master key (CMK) to which to grant Amazon Redshift permission. If no key is specified, the default key is used.<br/>SnapshotCopyGrantName:  The name of the snapshot copy grant. This name must be unique in the region for the AWS account. Constraints: <ul> <li> Must contain from 1 to 63 alphanumeric characters or hyphens. </li> <li> Alphabetic characters must be lowercase. </li> <li> First character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> <li> Must be unique for all clusters within an AWS account. </li> </ul><br/>
        :return: SnapshotCopyGrant:  The snapshot copy grant that grants Amazon Redshift permission to encrypt copied snapshots with the specified customer master key (CMK) from AWS KMS in the destination region.  For more information about managing snapshot copy grants, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-db-encryption.html">Amazon Redshift Database Encryption</a> in the <i>Amazon Redshift Cluster Management Guide</i>. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> A list of tag instances.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The unique identifier of the customer master key (CMK) in AWS KMS to which Amazon Redshift is granted permission.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCopyGrantName</i> The name of the snapshot copy grant.<br/>        """
        pass

    def create_snapshot_schedule(self, ScheduleDefinitions: List[String]=None, ScheduleIdentifier: str=None, ScheduleDescription: str=None, Tags: List[Tag]=None, DryRun: BooleanOptional=None, NextInvocations: int=None) -> SnapshotSchedule:
        """Creates a new snapshot schedule.

        :param self:
        :param NextInvocations:  <p/><br/>DryRun:  <p/><br/>Tags:  An optional set of tags you can use to search for the schedule.<br/>ScheduleDescription:  The description of the snapshot schedule.<br/>ScheduleIdentifier:  A unique identifier for a snapshot schedule. Only alphanumeric characters are allowed for the identifier.<br/>ScheduleDefinitions:  The definition of the snapshot schedule. The definition is made up of schedule expressions, for example "cron(30 12 *)" or "rate(12 hours)". <br/>
        :return: AssociatedClusters:  A list of clusters associated with the schedule. A maximum of 100 clusters is returned.<br/>AssociatedClusterCount:  The number of clusters associated with the schedule.<br/>NextInvocations:  <p/><br/>Tags:  An optional set of tags describing the schedule.<br/>ScheduleDescription:  The description of the schedule.<br/>ScheduleIdentifier:  A unique identifier for the schedule.<br/>ScheduleDefinitions:  A list of ScheduleDefinitions.<br/>        """
        pass

    def create_tags(self, Tags: List[Tag], ResourceName: str):
        """Adds one or more tags to a specified resource. A resource can have up to 50 tags. If you try to create more than 50 tags for a resource, you will receive an error and the attempt will fail. If you specify a key that already exists for the resource, the value for that key will be updated with the new value.

        :param self:
        :param Tags:  One or more name/value pairs to add as tags to the specified resource. Each tag name is passed in with the parameter <code>Key</code> and the corresponding value is passed in with the parameter <code>Value</code>. The <code>Key</code> and <code>Value</code> parameters are separated by a comma (,). Separate multiple tags with a space. For example, <code>--tags "Key"="owner","Value"="admin" "Key"="environment","Value"="test" "Key"="version","Value"="1.0"</code>. <br/>ResourceName:  The Amazon Resource Name (ARN) to which you want to add the tag or tags. For example, <code>arn:aws:redshift:us-east-1:123456789:cluster:t1</code>. <br/>
        :return:         """
        pass

    def delete_cluster(self, ClusterIdentifier: str, SkipFinalClusterSnapshot: Boolean=None, FinalClusterSnapshotIdentifier: str=None, FinalClusterSnapshotRetentionPeriod: int=None) -> DeleteClusterResult:
        """Deletes a previously provisioned cluster. A successful response from the web service indicates that the request was received correctly. Use <a>DescribeClusters</a> to monitor the status of the deletion. The delete operation cannot be canceled or reverted once submitted. For more information about managing clusters, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-clusters.html">Amazon Redshift Clusters</a> in the <i>Amazon Redshift Cluster Management Guide</i>. If you want to shut down the cluster and retain it for future use, set <i>SkipFinalClusterSnapshot</i> to <code>false</code> and specify a name for <i>FinalClusterSnapshotIdentifier</i>. You can later restore this snapshot to resume using the cluster. If a final cluster snapshot is requested, the status of the cluster will be "final-snapshot" while the snapshot is being taken, then it's "deleting" once Amazon Redshift begins deleting the cluster.   For more information about managing clusters, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-clusters.html">Amazon Redshift Clusters</a> in the <i>Amazon Redshift Cluster Management Guide</i>.

        :param self:
        :param FinalClusterSnapshotRetentionPeriod:  The number of days that a manual snapshot is retained. If the value is -1, the manual snapshot is retained indefinitely. The value must be either -1 or an integer between 1 and 3,653. The default value is -1.<br/>FinalClusterSnapshotIdentifier:  The identifier of the final snapshot that is to be created immediately before deleting the cluster. If this parameter is provided, <i>SkipFinalClusterSnapshot</i> must be <code>false</code>.  Constraints: <ul> <li> Must be 1 to 255 alphanumeric characters. </li> <li> First character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul><br/>SkipFinalClusterSnapshot:  Determines whether a final snapshot of the cluster is created before Amazon Redshift deletes the cluster. If <code>true</code>, a final cluster snapshot is not created. If <code>false</code>, a final cluster snapshot is created before the cluster is deleted.  <note> The <i>FinalClusterSnapshotIdentifier</i> parameter must be specified if <i>SkipFinalClusterSnapshot</i> is <code>false</code>. </note> Default: <code>false</code> <br/>ClusterIdentifier:  The identifier of the cluster to be deleted. Constraints: <ul> <li> Must contain lowercase characters. </li> <li> Must contain from 1 to 63 alphanumeric characters or hyphens. </li> <li> First character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul><br/>
        :return: Cluster:  Describes a cluster.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResizeInfo</i> Returns the following: <ul> <li> AllowCancelResize: a boolean value indicating if the resize operation can be cancelled. </li> <li> ResizeType: Returns ClassicResize </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResizeInfo</b>: Returns the following: <ul> <li> AllowCancelResize: a boolean value indicating if the resize operation can be cancelled. </li> <li> ResizeType: Returns ClassicResize </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotScheduleState</i> The current state of the cluster snapshot schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotScheduleIdentifier</i> A unique identifier for the cluster snapshot schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeferredMaintenanceWindows</i> Describes a group of <code>DeferredMaintenanceWindow</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticResizeNumberOfNodeOptions</i> The number of nodes that you can resize the cluster to with the elastic resize method. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingActions</i> Cluster operations that are waiting to be started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoles</i> A list of AWS Identity and Access Management (IAM) roles that can be used by the cluster to access other AWS services.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS Key Management Service (AWS KMS) key ID of the encryption key used to encrypt data in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The list of tags for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterRevisionNumber</i> The specific revision number of the database in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticIpStatus</i> The status of the elastic IP (EIP) address.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ElasticIpStatus</b>: The status of the elastic IP (EIP) address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterNodes</i> The nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterPublicKey</i> The public key for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSnapshotCopyStatus</i> A value that returns the destination region and retention period that are configured for cross-region snapshot copy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ClusterSnapshotCopyStatus</b>: A value that returns the destination region and retention period that are configured for cross-region snapshot copy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmStatus</i> A value that reports whether the Amazon Redshift cluster has finished applying any hardware security module (HSM) settings changes specified in a modify cluster command. Values: active, applying&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>HsmStatus</b>: A value that reports whether the Amazon Redshift cluster has finished applying any hardware security module (HSM) settings changes specified in a modify cluster command. Values: active, applying<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTransferProgress</i> <p/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DataTransferProgress</b>: <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreStatus</i> A value that describes the status of a cluster restore action. This parameter returns null if the cluster was not created by restoring a snapshot.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RestoreStatus</b>: A value that describes the status of a cluster restore action. This parameter returns null if the cluster was not created by restoring a snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encrypted</i> A boolean value that, if <code>true</code>, indicates that data in the cluster is encrypted at rest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> A boolean value that, if <code>true</code>, indicates that the cluster can be accessed from a public network.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The number of compute nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowVersionUpgrade</i> A boolean value that, if <code>true</code>, indicates that major version upgrades will be applied automatically to the cluster during the maintenance window. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The version ID of the Amazon Redshift engine that is running on the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> A value that, if present, indicates that changes to the cluster are pending. Specific pending changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: A value that, if present, indicates that changes to the cluster are pending. Specific pending changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> The weekly time range, in Universal Coordinated Time (UTC), during which system maintenance can occur.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The name of the Availability Zone in which the cluster is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The identifier of the VPC the cluster is in, if the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSubnetGroupName</i> The name of the subnet group that is associated with the cluster. This parameter is valid only when the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterParameterGroups</i> The list of cluster parameter groups that are associated with this cluster. Each parameter group in the list is returned with its status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> A list of Amazon Virtual Private Cloud (Amazon VPC) security groups that are associated with the cluster. This parameter is returned only if the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSecurityGroups</i> A list of cluster security group that are associated with the cluster. Each security group is represented by an element that contains <code>ClusterSecurityGroup.Name</code> and <code>ClusterSecurityGroup.Status</code> subelements.  Cluster security groups are used when the cluster is not created in an Amazon Virtual Private Cloud (VPC). Clusters that are created in a VPC use VPC security groups, which are listed by the <b>VpcSecurityGroups</b> parameter. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The default number of days to retain a manual snapshot. If the value is -1, the snapshot is retained indefinitely. This setting doesn't change the retention period of existing snapshots. The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotRetentionPeriod</i> The number of days that automatic cluster snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> The date and time that the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> The connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: The connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The name of the initial database that was created when the cluster was created. This same name is returned for the life of the cluster. If an initial database was not specified, a database named <code>dev</code>dev was created by default. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> The master user name for the cluster. This name is used to connect to the database that is specified in the <b>DBName</b> parameter. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModifyStatus</i> The status of a modify operation, if any, initiated for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterAvailabilityStatus</i> The availability status of the cluster for queries. Possible values are the following: <ul> <li> Available - The cluster is available for queries.  </li> <li> Unavailable - The cluster is not available for queries. </li> <li> Maintenance - The cluster is intermittently available for queries due to maintenance activities. </li> <li> Modifying - The cluster is intermittently available for queries due to changes that modify the cluster. </li> <li> Failed - The cluster failed and is not available for queries. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterStatus</i>  The current state of the cluster. Possible values are the following: <ul> <li>  <code>available</code>  </li> <li>  <code>available, prep-for-resize</code>  </li> <li>  <code>available, resize-cleanup</code>  </li> <li>  <code>cancelling-resize</code>  </li> <li>  <code>creating</code>  </li> <li>  <code>deleting</code>  </li> <li>  <code>final-snapshot</code>  </li> <li>  <code>hardware-failure</code>  </li> <li>  <code>incompatible-hsm</code>  </li> <li>  <code>incompatible-network</code>  </li> <li>  <code>incompatible-parameters</code>  </li> <li>  <code>incompatible-restore</code>  </li> <li>  <code>modifying</code>  </li> <li>  <code>rebooting</code>  </li> <li>  <code>renaming</code>  </li> <li>  <code>resizing</code>  </li> <li>  <code>rotating-keys</code>  </li> <li>  <code>storage-full</code>  </li> <li>  <code>updating-hsm</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The node type for the nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The unique identifier of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS address of the Cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionType</i> The encryption type for a cluster. Possible values are: KMS and None. For the China region the possible values are None, and Legacy. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track that the cluster will change to during the next maintenance window.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> The pending or in-progress change of the ability to connect to the cluster from the public network.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The pending or in-progress change of the new identifier for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotRetentionPeriod</i> The pending or in-progress change of the automated snapshot retention period.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The pending or in-progress change of the service version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterType</i> The pending or in-progress change of the cluster type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The pending or in-progress change of the number of nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The pending or in-progress change of the cluster's node type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> The pending or in-progress change of the master user password for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedTimeToCompletionInSeconds</i> The estimate of the time remaining before the restore will complete. Returns 0 for a completed restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> The amount of time an in-progress restore has been running, or the amount of time it took a completed restore to finish.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ProgressInMegaBytes</i> The number of megabytes that have been transferred from snapshot storage.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotSizeInMegaBytes</i> The size of the set of snapshot data used to restore the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentRestoreRateInMegaBytesPerSecond</i> The number of megabytes per second being transferred from the backup storage. Returns the average rate for a completed backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the restore action. Returns starting, restoring, completed, or failed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> Describes the number of seconds that have elapsed during the data transfer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedTimeToCompletionInSeconds</i> Describes the estimated number of seconds remaining to complete the transfer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTransferredInMegaBytes</i> Describes the total amount of data that has been transfered in MB's.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalDataInMegaBytes</i> Describes the total amount of data to be transfered in megabytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentRateInMegaBytesPerSecond</i> Describes the data transfer rate in MB's per second.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Describes the status of the cluster. While the transfer is in progress the status is <code>transferringdata</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Reports whether the Amazon Redshift cluster has finished applying any HSM settings changes specified in a modify cluster command. Values: active, applying<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmConfigurationIdentifier</i> Specifies the name of the HSM configuration that contains the information the Amazon Redshift cluster can use to retrieve and store keys in an HSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmClientCertificateIdentifier</i> Specifies the name of the HSM client certificate the Amazon Redshift cluster uses to retrieve the data encryption keys stored in an HSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCopyGrantName</i> The name of the snapshot copy grant.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The number of days that automated snapshots are retained in the destination region after they are copied from a source region. If the value is -1, the manual snapshot is retained indefinitely.  The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RetentionPeriod</i> The number of days that automated snapshots are retained in the destination region after they are copied from a source region.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationRegion</i> The destination region that snapshots are automatically copied to when cross-region snapshot copy is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the elastic IP (EIP) address.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticIp</i> The elastic IP (EIP) address for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowCancelResize</i> A boolean value indicating if the resize operation can be cancelled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResizeType</i> Returns the value <code>ClassicResize</code>.<br/>        """
        pass

    def delete_cluster_parameter_group(self, ParameterGroupName: str):
        """Deletes a specified Amazon Redshift parameter group. <note> You cannot delete a parameter group if it is associated with a cluster. </note>

        :param self:
        :param ParameterGroupName:  The name of the parameter group to be deleted. Constraints: <ul> <li> Must be the name of an existing cluster parameter group. </li> <li> Cannot delete a default cluster parameter group. </li> </ul><br/>
        :return:         """
        pass

    def delete_cluster_security_group(self, ClusterSecurityGroupName: str):
        """Deletes an Amazon Redshift security group. <note> You cannot delete a security group that is associated with any clusters. You cannot delete the default security group. </note>  For information about managing security groups, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-security-groups.html">Amazon Redshift Cluster Security Groups</a> in the <i>Amazon Redshift Cluster Management Guide</i>.

        :param self:
        :param ClusterSecurityGroupName:  The name of the cluster security group to be deleted.<br/>
        :return:         """
        pass

    def delete_cluster_snapshot(self, SnapshotIdentifier: str, SnapshotClusterIdentifier: str=None) -> DeleteClusterSnapshotResult:
        """Deletes the specified manual snapshot. The snapshot must be in the <code>available</code> state, with no other users authorized to access the snapshot.  Unlike automated snapshots, manual snapshots are retained even after you delete your cluster. Amazon Redshift does not delete your manual snapshots. You must delete manual snapshot explicitly to avoid getting charged. If other accounts are authorized to access the snapshot, you must revoke all of the authorizations before you can delete the snapshot.

        :param self:
        :param SnapshotClusterIdentifier:  The unique identifier of the cluster the snapshot was created from. This parameter is required if your IAM user has a policy containing a snapshot resource element that specifies anything other than * for the cluster name. Constraints: Must be the name of valid cluster.<br/>SnapshotIdentifier:  The unique identifier of the manual snapshot to be deleted. Constraints: Must be the name of an existing snapshot that is in the <code>available</code>, <code>failed</code>, or <code>cancelled</code> state.<br/>
        :return: Snapshot:  Describes a snapshot.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotRetentionStartTime</i> A timestamp representing the start of the retention period for the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRemainingDays</i> The number of days until a manual snapshot will pass its retention period.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The number of days that a manual snapshot is retained. If the value is -1, the manual snapshot is retained indefinitely.  The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track for the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestorableNodeTypes</i> The list of node types that this cluster snapshot is able to restore into.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The list of tags for the cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceRegion</i> The source region from which the snapshot was copied.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> The amount of time an in-progress snapshot backup has been running, or the amount of time it took a completed backup to finish.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedSecondsToCompletion</i> The estimate of the time remaining before the snapshot backup will complete. Returns <code>0</code> for a completed backup. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentBackupRateInMegaBytesPerSecond</i> The number of megabytes per second being transferred to the snapshot backup. Returns <code>0</code> for a completed backup. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupProgressInMegaBytes</i> The number of megabytes that have been transferred to the snapshot backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActualIncrementalBackupSizeInMegaBytes</i> The size of the incremental backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalBackupSizeInMegaBytes</i> The size of the complete set of backup data that would be used to restore the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerAccount</i> For manual snapshots, the AWS customer account used to create or copy the snapshot. For automatic snapshots, the owner of the cluster. The owner can perform all snapshot actions, such as sharing a manual snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountsWithRestoreAccess</i> A list of the AWS customer accounts authorized to restore the snapshot. Returns <code>null</code> if no accounts are authorized. Visible only to the snapshot owner. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptedWithHSM</i> A boolean that indicates whether the snapshot data is encrypted using the HSM keys of the source cluster. <code>true</code> indicates that the data is encrypted using HSM keys.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS Key Management Service (KMS) key ID of the encryption key that was used to encrypt data in the cluster from which the snapshot was taken.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encrypted</i> If <code>true</code>, the data in the snapshot is encrypted at rest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The VPC identifier of the cluster if the snapshot is from a cluster in a VPC. Otherwise, this field is not in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The name of the database that was created when the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The number of nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The node type of the nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotType</i> The snapshot type. Snapshots created using <a>CreateClusterSnapshot</a> and <a>CopyClusterSnapshot</a> are of type "manual". <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The version ID of the Amazon Redshift engine that is running on the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> The master user name for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> The time (UTC) when the cluster was originally created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone in which the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port that the cluster is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The snapshot status. The value of the status depends on the API operation used:  <ul> <li>  <a>CreateClusterSnapshot</a> and <a>CopyClusterSnapshot</a> returns status as "creating".  </li> <li>  <a>DescribeClusterSnapshots</a> returns status as "creating", "available", "final snapshot", or "failed". </li> <li>  <a>DeleteClusterSnapshot</a> returns status as "deleted". </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCreateTime</i> The time (in UTC format) when Amazon Redshift began the snapshot. A snapshot contains a copy of the cluster data as of this exact time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The identifier of the cluster for which the snapshot was taken.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotIdentifier</i> The snapshot identifier that is provided in the request.<br/>        """
        pass

    def delete_cluster_subnet_group(self, ClusterSubnetGroupName: str):
        """Deletes the specified cluster subnet group.

        :param self:
        :param ClusterSubnetGroupName:  The name of the cluster subnet group name to be deleted.<br/>
        :return:         """
        pass

    def delete_event_subscription(self, SubscriptionName: str):
        """Deletes an Amazon Redshift event notification subscription.

        :param self:
        :param SubscriptionName:  The name of the Amazon Redshift event notification subscription to be deleted.<br/>
        :return:         """
        pass

    def delete_hsm_client_certificate(self, HsmClientCertificateIdentifier: str):
        """Deletes the specified HSM client certificate.

        :param self:
        :param HsmClientCertificateIdentifier:  The identifier of the HSM client certificate to be deleted.<br/>
        :return:         """
        pass

    def delete_hsm_configuration(self, HsmConfigurationIdentifier: str):
        """Deletes the specified Amazon Redshift HSM configuration.

        :param self:
        :param HsmConfigurationIdentifier:  The identifier of the Amazon Redshift HSM configuration to be deleted.<br/>
        :return:         """
        pass

    def delete_snapshot_copy_grant(self, SnapshotCopyGrantName: str):
        """Deletes the specified snapshot copy grant.

        :param self:
        :param SnapshotCopyGrantName:  The name of the snapshot copy grant to delete.<br/>
        :return:         """
        pass

    def delete_snapshot_schedule(self, ScheduleIdentifier: str):
        """Deletes a snapshot schedule.

        :param self:
        :param ScheduleIdentifier:  A unique identifier of the snapshot schedule to delete.<br/>
        :return:         """
        pass

    def delete_tags(self, TagKeys: List[String], ResourceName: str):
        """Deletes a tag or tags from a resource. You must provide the ARN of the resource from which you want to delete the tag or tags.

        :param self:
        :param TagKeys:  The tag key that you want to delete.<br/>ResourceName:  The Amazon Resource Name (ARN) from which you want to remove the tag or tags. For example, <code>arn:aws:redshift:us-east-1:123456789:cluster:t1</code>. <br/>
        :return:         """
        pass

    def describe_account_attributes(self, AttributeNames: List[String]=None) -> AccountAttributeList:
        """Returns a list of attributes attached to an account

        :param self:
        :param AttributeNames:  A list of attribute names.<br/>
        :return: AccountAttributes:  A list of attributes assigned to an account.<br/>        """
        pass

    def describe_cluster_db_revisions(self, ClusterIdentifier: str=None, MaxRecords: int=None, Marker: str=None) -> ClusterDbRevisionsMessage:
        """Returns an array of <code>ClusterDbRevision</code> objects.

        :param self:
        :param Marker:  An optional parameter that specifies the starting point for returning a set of response records. When the results of a <code>DescribeClusterDbRevisions</code> request exceed the value specified in <code>MaxRecords</code>, Amazon Redshift returns a value in the <code>marker</code> field of the response. You can retrieve the next set of response records by providing the returned <code>marker</code> value in the <code>marker</code> parameter and retrying the request.  Constraints: You can specify either the <code>ClusterIdentifier</code> parameter, or the <code>marker</code> parameter, but not both.<br/>MaxRecords:  The maximum number of response records to return in each call. If the number of remaining response records exceeds the specified MaxRecords value, a value is returned in the <code>marker</code> field of the response. You can retrieve the next set of response records by providing the returned <code>marker</code> value in the <code>marker</code> parameter and retrying the request.  Default: 100 Constraints: minimum 20, maximum 100.<br/>ClusterIdentifier:  A unique identifier for a cluster whose <code>ClusterDbRevisions</code> you are requesting. This parameter is case sensitive. All clusters defined for an account are returned by default.<br/>
        :return: ClusterDbRevisions:  A list of revisions.<br/>Marker:  A string representing the starting point for the next set of revisions. If a value is returned in a response, you can retrieve the next set of revisions by providing the value in the <code>marker</code> parameter and retrying the command. If the <code>marker</code> field is empty, all revisions have already been returned.<br/>        """
        pass

    def describe_cluster_parameter_groups(self, ParameterGroupName: str=None, MaxRecords: int=None, Marker: str=None, TagKeys: List[String]=None, TagValues: List[String]=None) -> ClusterParameterGroupsMessage:
        """Returns a list of Amazon Redshift parameter groups, including parameter groups you created and the default parameter group. For each parameter group, the response includes the parameter group name, description, and parameter group family name. You can optionally specify a name to retrieve the description of a specific parameter group.  For more information about parameters and parameter groups, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-parameter-groups.html">Amazon Redshift Parameter Groups</a> in the <i>Amazon Redshift Cluster Management Guide</i>. If you specify both tag keys and tag values in the same request, Amazon Redshift returns all parameter groups that match any combination of the specified keys and values. For example, if you have <code>owner</code> and <code>environment</code> for tag keys, and <code>admin</code> and <code>test</code> for tag values, all parameter groups that have any combination of those values are returned. If both tag keys and values are omitted from the request, parameter groups are returned regardless of whether they have tag keys or values associated with them.

        :param self:
        :param TagValues:  A tag value or values for which you want to return all matching cluster parameter groups that are associated with the specified tag value or values. For example, suppose that you have parameter groups that are tagged with values called <code>admin</code> and <code>test</code>. If you specify both of these tag values in the request, Amazon Redshift returns a response with the parameter groups that have either or both of these tag values associated with them.<br/>TagKeys:  A tag key or keys for which you want to return all matching cluster parameter groups that are associated with the specified key or keys. For example, suppose that you have parameter groups that are tagged with keys called <code>owner</code> and <code>environment</code>. If you specify both of these tag keys in the request, Amazon Redshift returns a response with the parameter groups that have either or both of these tag keys associated with them.<br/>Marker:  An optional parameter that specifies the starting point to return a set of response records. When the results of a <a>DescribeClusterParameterGroups</a> request exceed the value specified in <code>MaxRecords</code>, AWS returns a value in the <code>Marker</code> field of the response. You can retrieve the next set of response records by providing the returned marker value in the <code>Marker</code> parameter and retrying the request. <br/>MaxRecords:  The maximum number of response records to return in each call. If the number of remaining response records exceeds the specified <code>MaxRecords</code> value, a value is returned in a <code>marker</code> field of the response. You can retrieve the next set of records by retrying the command with the returned marker value.  Default: <code>100</code>  Constraints: minimum 20, maximum 100.<br/>ParameterGroupName:  The name of a specific parameter group for which to return details. By default, details about all parameter groups and the default parameter group are returned.<br/>
        :return: ParameterGroups:  A list of <a>ClusterParameterGroup</a> instances. Each instance describes one cluster parameter group. <br/>Marker:  A value that indicates the starting point for the next set of response records in a subsequent request. If a value is returned in a response, you can retrieve the next set of records by providing this returned marker value in the <code>Marker</code> parameter and retrying the command. If the <code>Marker</code> field is empty, all response records have been retrieved for the request. <br/>        """
        pass

    def describe_cluster_parameters(self, ParameterGroupName: str, Source: str=None, MaxRecords: int=None, Marker: str=None) -> ClusterParameterGroupDetails:
        """Returns a detailed list of parameters contained within the specified Amazon Redshift parameter group. For each parameter the response includes information such as parameter name, description, data type, value, whether the parameter value is modifiable, and so on. You can specify <i>source</i> filter to retrieve parameters of only specific type. For example, to retrieve parameters that were modified by a user action such as from <a>ModifyClusterParameterGroup</a>, you can specify <i>source</i> equal to <i>user</i>.  For more information about parameters and parameter groups, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-parameter-groups.html">Amazon Redshift Parameter Groups</a> in the <i>Amazon Redshift Cluster Management Guide</i>.

        :param self:
        :param Marker:  An optional parameter that specifies the starting point to return a set of response records. When the results of a <a>DescribeClusterParameters</a> request exceed the value specified in <code>MaxRecords</code>, AWS returns a value in the <code>Marker</code> field of the response. You can retrieve the next set of response records by providing the returned marker value in the <code>Marker</code> parameter and retrying the request. <br/>MaxRecords:  The maximum number of response records to return in each call. If the number of remaining response records exceeds the specified <code>MaxRecords</code> value, a value is returned in a <code>marker</code> field of the response. You can retrieve the next set of records by retrying the command with the returned marker value.  Default: <code>100</code>  Constraints: minimum 20, maximum 100.<br/>Source:  The parameter types to return. Specify <code>user</code> to show parameters that are different form the default. Similarly, specify <code>engine-default</code> to show parameters that are the same as the default parameter group.  Default: All parameter types returned. Valid Values: <code>user</code> | <code>engine-default</code> <br/>ParameterGroupName:  The name of a cluster parameter group for which to return details.<br/>
        :return: Marker:  A value that indicates the starting point for the next set of response records in a subsequent request. If a value is returned in a response, you can retrieve the next set of records by providing this returned marker value in the <code>Marker</code> parameter and retrying the command. If the <code>Marker</code> field is empty, all response records have been retrieved for the request. <br/>Parameters:  A list of <a>Parameter</a> instances. Each instance lists the parameters of one cluster parameter group. <br/>        """
        pass

    def describe_cluster_security_groups(self, ClusterSecurityGroupName: str=None, MaxRecords: int=None, Marker: str=None, TagKeys: List[String]=None, TagValues: List[String]=None) -> ClusterSecurityGroupMessage:
        """Returns information about Amazon Redshift security groups. If the name of a security group is specified, the response will contain only information about only that security group.  For information about managing security groups, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-security-groups.html">Amazon Redshift Cluster Security Groups</a> in the <i>Amazon Redshift Cluster Management Guide</i>. If you specify both tag keys and tag values in the same request, Amazon Redshift returns all security groups that match any combination of the specified keys and values. For example, if you have <code>owner</code> and <code>environment</code> for tag keys, and <code>admin</code> and <code>test</code> for tag values, all security groups that have any combination of those values are returned. If both tag keys and values are omitted from the request, security groups are returned regardless of whether they have tag keys or values associated with them.

        :param self:
        :param TagValues:  A tag value or values for which you want to return all matching cluster security groups that are associated with the specified tag value or values. For example, suppose that you have security groups that are tagged with values called <code>admin</code> and <code>test</code>. If you specify both of these tag values in the request, Amazon Redshift returns a response with the security groups that have either or both of these tag values associated with them.<br/>TagKeys:  A tag key or keys for which you want to return all matching cluster security groups that are associated with the specified key or keys. For example, suppose that you have security groups that are tagged with keys called <code>owner</code> and <code>environment</code>. If you specify both of these tag keys in the request, Amazon Redshift returns a response with the security groups that have either or both of these tag keys associated with them.<br/>Marker:  An optional parameter that specifies the starting point to return a set of response records. When the results of a <a>DescribeClusterSecurityGroups</a> request exceed the value specified in <code>MaxRecords</code>, AWS returns a value in the <code>Marker</code> field of the response. You can retrieve the next set of response records by providing the returned marker value in the <code>Marker</code> parameter and retrying the request.  Constraints: You can specify either the <b>ClusterSecurityGroupName</b> parameter or the <b>Marker</b> parameter, but not both. <br/>MaxRecords:  The maximum number of response records to return in each call. If the number of remaining response records exceeds the specified <code>MaxRecords</code> value, a value is returned in a <code>marker</code> field of the response. You can retrieve the next set of records by retrying the command with the returned marker value.  Default: <code>100</code>  Constraints: minimum 20, maximum 100.<br/>ClusterSecurityGroupName:  The name of a cluster security group for which you are requesting details. You can specify either the <b>Marker</b> parameter or a <b>ClusterSecurityGroupName</b> parameter, but not both.   Example: <code>securitygroup1</code> <br/>
        :return: ClusterSecurityGroups:  A list of <a>ClusterSecurityGroup</a> instances. <br/>Marker:  A value that indicates the starting point for the next set of response records in a subsequent request. If a value is returned in a response, you can retrieve the next set of records by providing this returned marker value in the <code>Marker</code> parameter and retrying the command. If the <code>Marker</code> field is empty, all response records have been retrieved for the request. <br/>        """
        pass

    def describe_cluster_snapshots(self, ClusterIdentifier: str=None, SnapshotIdentifier: str=None, SnapshotType: str=None, StartTime: TStamp=None, EndTime: TStamp=None, MaxRecords: int=None, Marker: str=None, OwnerAccount: str=None, TagKeys: List[String]=None, TagValues: List[String]=None, ClusterExists: BooleanOptional=None, SortingEntities: List[SnapshotSortingEntity]=None) -> SnapshotMessage:
        """Returns one or more snapshot objects, which contain metadata about your cluster snapshots. By default, this operation returns information about all snapshots of all clusters that are owned by you AWS customer account. No information is returned for snapshots owned by inactive AWS customer accounts. If you specify both tag keys and tag values in the same request, Amazon Redshift returns all snapshots that match any combination of the specified keys and values. For example, if you have <code>owner</code> and <code>environment</code> for tag keys, and <code>admin</code> and <code>test</code> for tag values, all snapshots that have any combination of those values are returned. Only snapshots that you own are returned in the response; shared snapshots are not returned with the tag key and tag value request parameters. If both tag keys and values are omitted from the request, snapshots are returned regardless of whether they have tag keys or values associated with them.

        :param self:
        :param SortingEntities:  <p/><br/>ClusterExists:  A value that indicates whether to return snapshots only for an existing cluster. You can perform table-level restore only by using a snapshot of an existing cluster, that is, a cluster that has not been deleted. Values for this parameter work as follows:  <ul> <li> If <code>ClusterExists</code> is set to <code>true</code>, <code>ClusterIdentifier</code> is required. </li> <li> If <code>ClusterExists</code> is set to <code>false</code> and <code>ClusterIdentifier</code> isn't specified, all snapshots associated with deleted clusters (orphaned snapshots) are returned.  </li> <li> If <code>ClusterExists</code> is set to <code>false</code> and <code>ClusterIdentifier</code> is specified for a deleted cluster, snapshots associated with that cluster are returned. </li> <li> If <code>ClusterExists</code> is set to <code>false</code> and <code>ClusterIdentifier</code> is specified for an existing cluster, no snapshots are returned.  </li> </ul><br/>TagValues:  A tag value or values for which you want to return all matching cluster snapshots that are associated with the specified tag value or values. For example, suppose that you have snapshots that are tagged with values called <code>admin</code> and <code>test</code>. If you specify both of these tag values in the request, Amazon Redshift returns a response with the snapshots that have either or both of these tag values associated with them.<br/>TagKeys:  A tag key or keys for which you want to return all matching cluster snapshots that are associated with the specified key or keys. For example, suppose that you have snapshots that are tagged with keys called <code>owner</code> and <code>environment</code>. If you specify both of these tag keys in the request, Amazon Redshift returns a response with the snapshots that have either or both of these tag keys associated with them.<br/>OwnerAccount:  The AWS customer account used to create or copy the snapshot. Use this field to filter the results to snapshots owned by a particular account. To describe snapshots you own, either specify your AWS customer account, or do not specify the parameter.<br/>Marker:  An optional parameter that specifies the starting point to return a set of response records. When the results of a <a>DescribeClusterSnapshots</a> request exceed the value specified in <code>MaxRecords</code>, AWS returns a value in the <code>Marker</code> field of the response. You can retrieve the next set of response records by providing the returned marker value in the <code>Marker</code> parameter and retrying the request. <br/>MaxRecords:  The maximum number of response records to return in each call. If the number of remaining response records exceeds the specified <code>MaxRecords</code> value, a value is returned in a <code>marker</code> field of the response. You can retrieve the next set of records by retrying the command with the returned marker value.  Default: <code>100</code>  Constraints: minimum 20, maximum 100.<br/>EndTime:  A time value that requests only snapshots created at or before the specified time. The time value is specified in ISO 8601 format. For more information about ISO 8601, go to the <a href="http://en.wikipedia.org/wiki/ISO_8601">ISO8601 Wikipedia page.</a>  Example: <code>2012-07-16T18:00:00Z</code> <br/>StartTime:  A value that requests only snapshots created at or after the specified time. The time value is specified in ISO 8601 format. For more information about ISO 8601, go to the <a href="http://en.wikipedia.org/wiki/ISO_8601">ISO8601 Wikipedia page.</a>  Example: <code>2012-07-16T18:00:00Z</code> <br/>SnapshotType:  The type of snapshots for which you are requesting information. By default, snapshots of all types are returned. Valid Values: <code>automated</code> | <code>manual</code> <br/>SnapshotIdentifier:  The snapshot identifier of the snapshot about which to return information.<br/>ClusterIdentifier:  The identifier of the cluster which generated the requested snapshots.<br/>
        :return: Snapshots:  A list of <a>Snapshot</a> instances. <br/>Marker:  A value that indicates the starting point for the next set of response records in a subsequent request. If a value is returned in a response, you can retrieve the next set of records by providing this returned marker value in the <code>Marker</code> parameter and retrying the command. If the <code>Marker</code> field is empty, all response records have been retrieved for the request. <br/>        """
        pass

    def describe_cluster_subnet_groups(self, ClusterSubnetGroupName: str=None, MaxRecords: int=None, Marker: str=None, TagKeys: List[String]=None, TagValues: List[String]=None) -> ClusterSubnetGroupMessage:
        """Returns one or more cluster subnet group objects, which contain metadata about your cluster subnet groups. By default, this operation returns information about all cluster subnet groups that are defined in you AWS account. If you specify both tag keys and tag values in the same request, Amazon Redshift returns all subnet groups that match any combination of the specified keys and values. For example, if you have <code>owner</code> and <code>environment</code> for tag keys, and <code>admin</code> and <code>test</code> for tag values, all subnet groups that have any combination of those values are returned. If both tag keys and values are omitted from the request, subnet groups are returned regardless of whether they have tag keys or values associated with them.

        :param self:
        :param TagValues:  A tag value or values for which you want to return all matching cluster subnet groups that are associated with the specified tag value or values. For example, suppose that you have subnet groups that are tagged with values called <code>admin</code> and <code>test</code>. If you specify both of these tag values in the request, Amazon Redshift returns a response with the subnet groups that have either or both of these tag values associated with them.<br/>TagKeys:  A tag key or keys for which you want to return all matching cluster subnet groups that are associated with the specified key or keys. For example, suppose that you have subnet groups that are tagged with keys called <code>owner</code> and <code>environment</code>. If you specify both of these tag keys in the request, Amazon Redshift returns a response with the subnet groups that have either or both of these tag keys associated with them.<br/>Marker:  An optional parameter that specifies the starting point to return a set of response records. When the results of a <a>DescribeClusterSubnetGroups</a> request exceed the value specified in <code>MaxRecords</code>, AWS returns a value in the <code>Marker</code> field of the response. You can retrieve the next set of response records by providing the returned marker value in the <code>Marker</code> parameter and retrying the request. <br/>MaxRecords:  The maximum number of response records to return in each call. If the number of remaining response records exceeds the specified <code>MaxRecords</code> value, a value is returned in a <code>marker</code> field of the response. You can retrieve the next set of records by retrying the command with the returned marker value.  Default: <code>100</code>  Constraints: minimum 20, maximum 100.<br/>ClusterSubnetGroupName:  The name of the cluster subnet group for which information is requested.<br/>
        :return: ClusterSubnetGroups:  A list of <a>ClusterSubnetGroup</a> instances. <br/>Marker:  A value that indicates the starting point for the next set of response records in a subsequent request. If a value is returned in a response, you can retrieve the next set of records by providing this returned marker value in the <code>Marker</code> parameter and retrying the command. If the <code>Marker</code> field is empty, all response records have been retrieved for the request. <br/>        """
        pass

    def describe_cluster_tracks(self, MaintenanceTrackName: str=None, MaxRecords: int=None, Marker: str=None) -> TrackListMessage:
        """Returns a list of all the available maintenance tracks.

        :param self:
        :param Marker:  An optional parameter that specifies the starting point to return a set of response records. When the results of a <code>DescribeClusterTracks</code> request exceed the value specified in <code>MaxRecords</code>, Amazon Redshift returns a value in the <code>Marker</code> field of the response. You can retrieve the next set of response records by providing the returned marker value in the <code>Marker</code> parameter and retrying the request. <br/>MaxRecords:  An integer value for the maximum number of maintenance tracks to return.<br/>MaintenanceTrackName:  The name of the maintenance track. <br/>
        :return: Marker:  The starting point to return a set of response tracklist records. You can retrieve the next set of response records by providing the returned marker value in the <code>Marker</code> parameter and retrying the request.<br/>MaintenanceTracks:  A list of maintenance tracks output by the <code>DescribeClusterTracks</code> operation. <br/>        """
        pass

    def describe_cluster_versions(self, ClusterVersion: str=None, ClusterParameterGroupFamily: str=None, MaxRecords: int=None, Marker: str=None) -> ClusterVersionsMessage:
        """Returns descriptions of the available Amazon Redshift cluster versions. You can call this operation even before creating any clusters to learn more about the Amazon Redshift versions. For more information about managing clusters, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-clusters.html">Amazon Redshift Clusters</a> in the <i>Amazon Redshift Cluster Management Guide</i>.

        :param self:
        :param Marker:  An optional parameter that specifies the starting point to return a set of response records. When the results of a <a>DescribeClusterVersions</a> request exceed the value specified in <code>MaxRecords</code>, AWS returns a value in the <code>Marker</code> field of the response. You can retrieve the next set of response records by providing the returned marker value in the <code>Marker</code> parameter and retrying the request. <br/>MaxRecords:  The maximum number of response records to return in each call. If the number of remaining response records exceeds the specified <code>MaxRecords</code> value, a value is returned in a <code>marker</code> field of the response. You can retrieve the next set of records by retrying the command with the returned marker value.  Default: <code>100</code>  Constraints: minimum 20, maximum 100.<br/>ClusterParameterGroupFamily:  The name of a specific cluster parameter group family to return details for. Constraints: <ul> <li> Must be 1 to 255 alphanumeric characters </li> <li> First character must be a letter </li> <li> Cannot end with a hyphen or contain two consecutive hyphens </li> </ul><br/>ClusterVersion:  The specific cluster version to return. Example: <code>1.0</code> <br/>
        :return: ClusterVersions:  A list of <code>Version</code> elements. <br/>Marker:  A value that indicates the starting point for the next set of response records in a subsequent request. If a value is returned in a response, you can retrieve the next set of records by providing this returned marker value in the <code>Marker</code> parameter and retrying the command. If the <code>Marker</code> field is empty, all response records have been retrieved for the request. <br/>        """
        pass

    def describe_clusters(self, ClusterIdentifier: str=None, MaxRecords: int=None, Marker: str=None, TagKeys: List[String]=None, TagValues: List[String]=None) -> ClustersMessage:
        """Returns properties of provisioned clusters including general cluster properties, cluster database properties, maintenance and backup properties, and security and access properties. This operation supports pagination. For more information about managing clusters, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-clusters.html">Amazon Redshift Clusters</a> in the <i>Amazon Redshift Cluster Management Guide</i>. If you specify both tag keys and tag values in the same request, Amazon Redshift returns all clusters that match any combination of the specified keys and values. For example, if you have <code>owner</code> and <code>environment</code> for tag keys, and <code>admin</code> and <code>test</code> for tag values, all clusters that have any combination of those values are returned. If both tag keys and values are omitted from the request, clusters are returned regardless of whether they have tag keys or values associated with them.

        :param self:
        :param TagValues:  A tag value or values for which you want to return all matching clusters that are associated with the specified tag value or values. For example, suppose that you have clusters that are tagged with values called <code>admin</code> and <code>test</code>. If you specify both of these tag values in the request, Amazon Redshift returns a response with the clusters that have either or both of these tag values associated with them.<br/>TagKeys:  A tag key or keys for which you want to return all matching clusters that are associated with the specified key or keys. For example, suppose that you have clusters that are tagged with keys called <code>owner</code> and <code>environment</code>. If you specify both of these tag keys in the request, Amazon Redshift returns a response with the clusters that have either or both of these tag keys associated with them.<br/>Marker:  An optional parameter that specifies the starting point to return a set of response records. When the results of a <a>DescribeClusters</a> request exceed the value specified in <code>MaxRecords</code>, AWS returns a value in the <code>Marker</code> field of the response. You can retrieve the next set of response records by providing the returned marker value in the <code>Marker</code> parameter and retrying the request.  Constraints: You can specify either the <b>ClusterIdentifier</b> parameter or the <b>Marker</b> parameter, but not both. <br/>MaxRecords:  The maximum number of response records to return in each call. If the number of remaining response records exceeds the specified <code>MaxRecords</code> value, a value is returned in a <code>marker</code> field of the response. You can retrieve the next set of records by retrying the command with the returned marker value.  Default: <code>100</code>  Constraints: minimum 20, maximum 100.<br/>ClusterIdentifier:  The unique identifier of a cluster whose properties you are requesting. This parameter is case sensitive. The default is that all clusters defined for an account are returned.<br/>
        :return: Clusters:  A list of <code>Cluster</code> objects, where each object describes one cluster. <br/>Marker:  A value that indicates the starting point for the next set of response records in a subsequent request. If a value is returned in a response, you can retrieve the next set of records by providing this returned marker value in the <code>Marker</code> parameter and retrying the command. If the <code>Marker</code> field is empty, all response records have been retrieved for the request. <br/>        """
        pass

    def describe_default_cluster_parameters(self, ParameterGroupFamily: str, MaxRecords: int=None, Marker: str=None) -> DescribeDefaultClusterParametersResult:
        """Returns a list of parameter settings for the specified parameter group family.  For more information about parameters and parameter groups, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-parameter-groups.html">Amazon Redshift Parameter Groups</a> in the <i>Amazon Redshift Cluster Management Guide</i>.

        :param self:
        :param Marker:  An optional parameter that specifies the starting point to return a set of response records. When the results of a <a>DescribeDefaultClusterParameters</a> request exceed the value specified in <code>MaxRecords</code>, AWS returns a value in the <code>Marker</code> field of the response. You can retrieve the next set of response records by providing the returned marker value in the <code>Marker</code> parameter and retrying the request. <br/>MaxRecords:  The maximum number of response records to return in each call. If the number of remaining response records exceeds the specified <code>MaxRecords</code> value, a value is returned in a <code>marker</code> field of the response. You can retrieve the next set of records by retrying the command with the returned marker value.  Default: <code>100</code>  Constraints: minimum 20, maximum 100.<br/>ParameterGroupFamily:  The name of the cluster parameter group family.<br/>
        :return: DefaultClusterParameters:  Describes the default cluster parameters for a parameter group family.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> The list of cluster default parameters.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Marker</i> A value that indicates the starting point for the next set of response records in a subsequent request. If a value is returned in a response, you can retrieve the next set of records by providing this returned marker value in the <code>Marker</code> parameter and retrying the command. If the <code>Marker</code> field is empty, all response records have been retrieved for the request. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterGroupFamily</i> The name of the cluster parameter group family to which the engine default parameters apply.<br/>        """
        pass

    def describe_event_categories(self, SourceType: str=None) -> EventCategoriesMessage:
        """Displays a list of event categories for all event source types, or for a specified source type. For a list of the event categories and source types, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-event-notifications.html">Amazon Redshift Event Notifications</a>.

        :param self:
        :param SourceType:  The source type, such as cluster or parameter group, to which the described event categories apply. Valid values: cluster, cluster-snapshot, cluster-parameter-group, and cluster-security-group.<br/>
        :return: EventCategoriesMapList:  A list of event categories descriptions.<br/>        """
        pass

    def describe_event_subscriptions(self, SubscriptionName: str=None, MaxRecords: int=None, Marker: str=None, TagKeys: List[String]=None, TagValues: List[String]=None) -> EventSubscriptionsMessage:
        """Lists descriptions of all the Amazon Redshift event notification subscriptions for a customer account. If you specify a subscription name, lists the description for that subscription. If you specify both tag keys and tag values in the same request, Amazon Redshift returns all event notification subscriptions that match any combination of the specified keys and values. For example, if you have <code>owner</code> and <code>environment</code> for tag keys, and <code>admin</code> and <code>test</code> for tag values, all subscriptions that have any combination of those values are returned. If both tag keys and values are omitted from the request, subscriptions are returned regardless of whether they have tag keys or values associated with them.

        :param self:
        :param TagValues:  A tag value or values for which you want to return all matching event notification subscriptions that are associated with the specified tag value or values. For example, suppose that you have subscriptions that are tagged with values called <code>admin</code> and <code>test</code>. If you specify both of these tag values in the request, Amazon Redshift returns a response with the subscriptions that have either or both of these tag values associated with them.<br/>TagKeys:  A tag key or keys for which you want to return all matching event notification subscriptions that are associated with the specified key or keys. For example, suppose that you have subscriptions that are tagged with keys called <code>owner</code> and <code>environment</code>. If you specify both of these tag keys in the request, Amazon Redshift returns a response with the subscriptions that have either or both of these tag keys associated with them.<br/>Marker:  An optional parameter that specifies the starting point to return a set of response records. When the results of a DescribeEventSubscriptions request exceed the value specified in <code>MaxRecords</code>, AWS returns a value in the <code>Marker</code> field of the response. You can retrieve the next set of response records by providing the returned marker value in the <code>Marker</code> parameter and retrying the request. <br/>MaxRecords:  The maximum number of response records to return in each call. If the number of remaining response records exceeds the specified <code>MaxRecords</code> value, a value is returned in a <code>marker</code> field of the response. You can retrieve the next set of records by retrying the command with the returned marker value.  Default: <code>100</code>  Constraints: minimum 20, maximum 100.<br/>SubscriptionName:  The name of the Amazon Redshift event notification subscription to be described.<br/>
        :return: EventSubscriptionsList:  A list of event subscriptions.<br/>Marker:  A value that indicates the starting point for the next set of response records in a subsequent request. If a value is returned in a response, you can retrieve the next set of records by providing this returned marker value in the <code>Marker</code> parameter and retrying the command. If the <code>Marker</code> field is empty, all response records have been retrieved for the request. <br/>        """
        pass

    def describe_events(self, SourceIdentifier: str=None, SourceType: str=None, StartTime: TStamp=None, EndTime: TStamp=None, Duration: int=None, MaxRecords: int=None, Marker: str=None) -> EventsMessage:
        """Returns events related to clusters, security groups, snapshots, and parameter groups for the past 14 days. Events specific to a particular cluster, security group, snapshot or parameter group can be obtained by providing the name as a parameter. By default, the past hour of events are returned.

        :param self:
        :param Marker:  An optional parameter that specifies the starting point to return a set of response records. When the results of a <a>DescribeEvents</a> request exceed the value specified in <code>MaxRecords</code>, AWS returns a value in the <code>Marker</code> field of the response. You can retrieve the next set of response records by providing the returned marker value in the <code>Marker</code> parameter and retrying the request. <br/>MaxRecords:  The maximum number of response records to return in each call. If the number of remaining response records exceeds the specified <code>MaxRecords</code> value, a value is returned in a <code>marker</code> field of the response. You can retrieve the next set of records by retrying the command with the returned marker value.  Default: <code>100</code>  Constraints: minimum 20, maximum 100.<br/>Duration:  The number of minutes prior to the time of the request for which to retrieve events. For example, if the request is sent at 18:00 and you specify a duration of 60, then only events which have occurred after 17:00 will be returned. Default: <code>60</code> <br/>EndTime:  The end of the time interval for which to retrieve events, specified in ISO 8601 format. For more information about ISO 8601, go to the <a href="http://en.wikipedia.org/wiki/ISO_8601">ISO8601 Wikipedia page.</a>  Example: <code>2009-07-08T18:00Z</code> <br/>StartTime:  The beginning of the time interval to retrieve events for, specified in ISO 8601 format. For more information about ISO 8601, go to the <a href="http://en.wikipedia.org/wiki/ISO_8601">ISO8601 Wikipedia page.</a>  Example: <code>2009-07-08T18:00Z</code> <br/>SourceType:  The event source to retrieve events for. If no value is specified, all events are returned. Constraints: If <i>SourceType</i> is supplied, <i>SourceIdentifier</i> must also be provided. <ul> <li> Specify <code>cluster</code> when <i>SourceIdentifier</i> is a cluster identifier. </li> <li> Specify <code>cluster-security-group</code> when <i>SourceIdentifier</i> is a cluster security group name. </li> <li> Specify <code>cluster-parameter-group</code> when <i>SourceIdentifier</i> is a cluster parameter group name. </li> <li> Specify <code>cluster-snapshot</code> when <i>SourceIdentifier</i> is a cluster snapshot identifier. </li> </ul><br/>SourceIdentifier:  The identifier of the event source for which events will be returned. If this parameter is not specified, then all sources are included in the response. Constraints: If <i>SourceIdentifier</i> is supplied, <i>SourceType</i> must also be provided. <ul> <li> Specify a cluster identifier when <i>SourceType</i> is <code>cluster</code>. </li> <li> Specify a cluster security group name when <i>SourceType</i> is <code>cluster-security-group</code>. </li> <li> Specify a cluster parameter group name when <i>SourceType</i> is <code>cluster-parameter-group</code>. </li> <li> Specify a cluster snapshot identifier when <i>SourceType</i> is <code>cluster-snapshot</code>. </li> </ul><br/>
        :return: Events:  A list of <code>Event</code> instances. <br/>Marker:  A value that indicates the starting point for the next set of response records in a subsequent request. If a value is returned in a response, you can retrieve the next set of records by providing this returned marker value in the <code>Marker</code> parameter and retrying the command. If the <code>Marker</code> field is empty, all response records have been retrieved for the request. <br/>        """
        pass

    def describe_hsm_client_certificates(self, HsmClientCertificateIdentifier: str=None, MaxRecords: int=None, Marker: str=None, TagKeys: List[String]=None, TagValues: List[String]=None) -> HsmClientCertificateMessage:
        """Returns information about the specified HSM client certificate. If no certificate ID is specified, returns information about all the HSM certificates owned by your AWS customer account. If you specify both tag keys and tag values in the same request, Amazon Redshift returns all HSM client certificates that match any combination of the specified keys and values. For example, if you have <code>owner</code> and <code>environment</code> for tag keys, and <code>admin</code> and <code>test</code> for tag values, all HSM client certificates that have any combination of those values are returned. If both tag keys and values are omitted from the request, HSM client certificates are returned regardless of whether they have tag keys or values associated with them.

        :param self:
        :param TagValues:  A tag value or values for which you want to return all matching HSM client certificates that are associated with the specified tag value or values. For example, suppose that you have HSM client certificates that are tagged with values called <code>admin</code> and <code>test</code>. If you specify both of these tag values in the request, Amazon Redshift returns a response with the HSM client certificates that have either or both of these tag values associated with them.<br/>TagKeys:  A tag key or keys for which you want to return all matching HSM client certificates that are associated with the specified key or keys. For example, suppose that you have HSM client certificates that are tagged with keys called <code>owner</code> and <code>environment</code>. If you specify both of these tag keys in the request, Amazon Redshift returns a response with the HSM client certificates that have either or both of these tag keys associated with them.<br/>Marker:  An optional parameter that specifies the starting point to return a set of response records. When the results of a <a>DescribeHsmClientCertificates</a> request exceed the value specified in <code>MaxRecords</code>, AWS returns a value in the <code>Marker</code> field of the response. You can retrieve the next set of response records by providing the returned marker value in the <code>Marker</code> parameter and retrying the request. <br/>MaxRecords:  The maximum number of response records to return in each call. If the number of remaining response records exceeds the specified <code>MaxRecords</code> value, a value is returned in a <code>marker</code> field of the response. You can retrieve the next set of records by retrying the command with the returned marker value.  Default: <code>100</code>  Constraints: minimum 20, maximum 100.<br/>HsmClientCertificateIdentifier:  The identifier of a specific HSM client certificate for which you want information. If no identifier is specified, information is returned for all HSM client certificates owned by your AWS customer account.<br/>
        :return: HsmClientCertificates:  A list of the identifiers for one or more HSM client certificates used by Amazon Redshift clusters to store and retrieve database encryption keys in an HSM.<br/>Marker:  A value that indicates the starting point for the next set of response records in a subsequent request. If a value is returned in a response, you can retrieve the next set of records by providing this returned marker value in the <code>Marker</code> parameter and retrying the command. If the <code>Marker</code> field is empty, all response records have been retrieved for the request. <br/>        """
        pass

    def describe_hsm_configurations(self, HsmConfigurationIdentifier: str=None, MaxRecords: int=None, Marker: str=None, TagKeys: List[String]=None, TagValues: List[String]=None) -> HsmConfigurationMessage:
        """Returns information about the specified Amazon Redshift HSM configuration. If no configuration ID is specified, returns information about all the HSM configurations owned by your AWS customer account. If you specify both tag keys and tag values in the same request, Amazon Redshift returns all HSM connections that match any combination of the specified keys and values. For example, if you have <code>owner</code> and <code>environment</code> for tag keys, and <code>admin</code> and <code>test</code> for tag values, all HSM connections that have any combination of those values are returned. If both tag keys and values are omitted from the request, HSM connections are returned regardless of whether they have tag keys or values associated with them.

        :param self:
        :param TagValues:  A tag value or values for which you want to return all matching HSM configurations that are associated with the specified tag value or values. For example, suppose that you have HSM configurations that are tagged with values called <code>admin</code> and <code>test</code>. If you specify both of these tag values in the request, Amazon Redshift returns a response with the HSM configurations that have either or both of these tag values associated with them.<br/>TagKeys:  A tag key or keys for which you want to return all matching HSM configurations that are associated with the specified key or keys. For example, suppose that you have HSM configurations that are tagged with keys called <code>owner</code> and <code>environment</code>. If you specify both of these tag keys in the request, Amazon Redshift returns a response with the HSM configurations that have either or both of these tag keys associated with them.<br/>Marker:  An optional parameter that specifies the starting point to return a set of response records. When the results of a <a>DescribeHsmConfigurations</a> request exceed the value specified in <code>MaxRecords</code>, AWS returns a value in the <code>Marker</code> field of the response. You can retrieve the next set of response records by providing the returned marker value in the <code>Marker</code> parameter and retrying the request. <br/>MaxRecords:  The maximum number of response records to return in each call. If the number of remaining response records exceeds the specified <code>MaxRecords</code> value, a value is returned in a <code>marker</code> field of the response. You can retrieve the next set of records by retrying the command with the returned marker value.  Default: <code>100</code>  Constraints: minimum 20, maximum 100.<br/>HsmConfigurationIdentifier:  The identifier of a specific Amazon Redshift HSM configuration to be described. If no identifier is specified, information is returned for all HSM configurations owned by your AWS customer account.<br/>
        :return: HsmConfigurations:  A list of <code>HsmConfiguration</code> objects.<br/>Marker:  A value that indicates the starting point for the next set of response records in a subsequent request. If a value is returned in a response, you can retrieve the next set of records by providing this returned marker value in the <code>Marker</code> parameter and retrying the command. If the <code>Marker</code> field is empty, all response records have been retrieved for the request. <br/>        """
        pass

    def describe_logging_status(self, ClusterIdentifier: str) -> LoggingStatus:
        """Describes whether information, such as queries and connection attempts, is being logged for the specified Amazon Redshift cluster.

        :param self:
        :param ClusterIdentifier:  The identifier of the cluster from which to get the logging status. Example: <code>examplecluster</code> <br/>
        :return: LastFailureMessage:  The message indicating that logs failed to be delivered.<br/>LastFailureTime:  The last time when logs failed to be delivered.<br/>LastSuccessfulDeliveryTime:  The last time that logs were delivered.<br/>S3KeyPrefix:  The prefix applied to the log file names.<br/>BucketName:  The name of the S3 bucket where the log files are stored.<br/>LoggingEnabled:   <code>true</code> if logging is on, <code>false</code> if logging is off.<br/>        """
        pass

    def describe_orderable_cluster_options(self, ClusterVersion: str=None, NodeType: str=None, MaxRecords: int=None, Marker: str=None) -> OrderableClusterOptionsMessage:
        """Returns a list of orderable cluster options. Before you create a new cluster you can use this operation to find what options are available, such as the EC2 Availability Zones (AZ) in the specific AWS Region that you can specify, and the node types you can request. The node types differ by available storage, memory, CPU and price. With the cost involved you might want to obtain a list of cluster options in the specific region and specify values when creating a cluster. For more information about managing clusters, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-clusters.html">Amazon Redshift Clusters</a> in the <i>Amazon Redshift Cluster Management Guide</i>.

        :param self:
        :param Marker:  An optional parameter that specifies the starting point to return a set of response records. When the results of a <a>DescribeOrderableClusterOptions</a> request exceed the value specified in <code>MaxRecords</code>, AWS returns a value in the <code>Marker</code> field of the response. You can retrieve the next set of response records by providing the returned marker value in the <code>Marker</code> parameter and retrying the request. <br/>MaxRecords:  The maximum number of response records to return in each call. If the number of remaining response records exceeds the specified <code>MaxRecords</code> value, a value is returned in a <code>marker</code> field of the response. You can retrieve the next set of records by retrying the command with the returned marker value.  Default: <code>100</code>  Constraints: minimum 20, maximum 100.<br/>NodeType:  The node type filter value. Specify this parameter to show only the available offerings matching the specified node type.<br/>ClusterVersion:  The version filter value. Specify this parameter to show only the available offerings matching the specified version. Default: All versions. Constraints: Must be one of the version returned from <a>DescribeClusterVersions</a>.<br/>
        :return: Marker:  A value that indicates the starting point for the next set of response records in a subsequent request. If a value is returned in a response, you can retrieve the next set of records by providing this returned marker value in the <code>Marker</code> parameter and retrying the command. If the <code>Marker</code> field is empty, all response records have been retrieved for the request. <br/>OrderableClusterOptions:  An <code>OrderableClusterOption</code> structure containing information about orderable options for the cluster.<br/>        """
        pass

    def describe_reserved_node_offerings(self, ReservedNodeOfferingId: str=None, MaxRecords: int=None, Marker: str=None) -> ReservedNodeOfferingsMessage:
        """Returns a list of the available reserved node offerings by Amazon Redshift with their descriptions including the node type, the fixed and recurring costs of reserving the node and duration the node will be reserved for you. These descriptions help you determine which reserve node offering you want to purchase. You then use the unique offering ID in you call to <a>PurchaseReservedNodeOffering</a> to reserve one or more nodes for your Amazon Redshift cluster.   For more information about reserved node offerings, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/purchase-reserved-node-instance.html">Purchasing Reserved Nodes</a> in the <i>Amazon Redshift Cluster Management Guide</i>.

        :param self:
        :param Marker:  An optional parameter that specifies the starting point to return a set of response records. When the results of a <a>DescribeReservedNodeOfferings</a> request exceed the value specified in <code>MaxRecords</code>, AWS returns a value in the <code>Marker</code> field of the response. You can retrieve the next set of response records by providing the returned marker value in the <code>Marker</code> parameter and retrying the request. <br/>MaxRecords:  The maximum number of response records to return in each call. If the number of remaining response records exceeds the specified <code>MaxRecords</code> value, a value is returned in a <code>marker</code> field of the response. You can retrieve the next set of records by retrying the command with the returned marker value.  Default: <code>100</code>  Constraints: minimum 20, maximum 100.<br/>ReservedNodeOfferingId:  The unique identifier for the offering.<br/>
        :return: ReservedNodeOfferings:  A list of <code>ReservedNodeOffering</code> objects.<br/>Marker:  A value that indicates the starting point for the next set of response records in a subsequent request. If a value is returned in a response, you can retrieve the next set of records by providing this returned marker value in the <code>Marker</code> parameter and retrying the command. If the <code>Marker</code> field is empty, all response records have been retrieved for the request. <br/>        """
        pass

    def describe_reserved_nodes(self, ReservedNodeId: str=None, MaxRecords: int=None, Marker: str=None) -> ReservedNodesMessage:
        """Returns the descriptions of the reserved nodes.

        :param self:
        :param Marker:  An optional parameter that specifies the starting point to return a set of response records. When the results of a <a>DescribeReservedNodes</a> request exceed the value specified in <code>MaxRecords</code>, AWS returns a value in the <code>Marker</code> field of the response. You can retrieve the next set of response records by providing the returned marker value in the <code>Marker</code> parameter and retrying the request. <br/>MaxRecords:  The maximum number of response records to return in each call. If the number of remaining response records exceeds the specified <code>MaxRecords</code> value, a value is returned in a <code>marker</code> field of the response. You can retrieve the next set of records by retrying the command with the returned marker value.  Default: <code>100</code>  Constraints: minimum 20, maximum 100.<br/>ReservedNodeId:  Identifier for the node reservation.<br/>
        :return: ReservedNodes:  The list of <code>ReservedNode</code> objects.<br/>Marker:  A value that indicates the starting point for the next set of response records in a subsequent request. If a value is returned in a response, you can retrieve the next set of records by providing this returned marker value in the <code>Marker</code> parameter and retrying the command. If the <code>Marker</code> field is empty, all response records have been retrieved for the request. <br/>        """
        pass

    def describe_resize(self, ClusterIdentifier: str) -> ResizeProgressMessage:
        """Returns information about the last resize operation for the specified cluster. If no resize operation has ever been initiated for the specified cluster, a <code>HTTP 404</code> error is returned. If a resize operation was initiated and completed, the status of the resize remains as <code>SUCCEEDED</code> until the next resize.  A resize operation can be requested using <a>ModifyCluster</a> and specifying a different number or type of nodes for the cluster. 

        :param self:
        :param ClusterIdentifier:  The unique identifier of a cluster whose resize progress you are requesting. This parameter is case-sensitive. By default, resize operations for all clusters defined for an AWS account are returned.<br/>
        :return: DataTransferProgressPercent:  The percent of data transferred from source cluster to target cluster.<br/>TargetEncryptionType:  The type of encryption for the cluster after the resize is complete. Possible values are <code>KMS</code> and <code>None</code>. In the China region possible values are: <code>Legacy</code> and <code>None</code>.<br/>Message:  An optional string to provide additional details about the resize action.<br/>ResizeType:  An enum with possible values of <code>ClassicResize</code> and <code>ElasticResize</code>. These values describe the type of resize operation being performed. <br/>EstimatedTimeToCompletionInSeconds:  The estimated time remaining, in seconds, until the resize operation is complete. This value is calculated based on the average resize rate and the estimated amount of data remaining to be processed. Once the resize operation is complete, this value will be 0.<br/>ElapsedTimeInSeconds:  The amount of seconds that have elapsed since the resize operation began. After the resize operation completes, this value shows the total actual time, in seconds, for the resize operation.<br/>ProgressInMegaBytes:  While the resize operation is in progress, this value shows the current amount of data, in megabytes, that has been processed so far. When the resize operation is complete, this value shows the total amount of data, in megabytes, on the cluster, which may be more or less than TotalResizeDataInMegaBytes (the estimated total amount of data before resize).<br/>TotalResizeDataInMegaBytes:  The estimated total amount of data, in megabytes, on the cluster before the resize operation began.<br/>AvgResizeRateInMegaBytesPerSecond:  The average rate of the resize operation over the last few minutes, measured in megabytes per second. After the resize operation completes, this value shows the average rate of the entire resize operation.<br/>ImportTablesNotStarted:  The names of tables that have not been yet imported. Valid Values: List of table names<br/>ImportTablesInProgress:  The names of tables that are being currently imported. Valid Values: List of table names.<br/>ImportTablesCompleted:  The names of tables that have been completely imported . Valid Values: List of table names.<br/>Status:  The status of the resize operation. Valid Values: <code>NONE</code> | <code>IN_PROGRESS</code> | <code>FAILED</code> | <code>SUCCEEDED</code> | <code>CANCELLING</code> <br/>TargetClusterType:  The cluster type after the resize operation is complete. Valid Values: <code>multi-node</code> | <code>single-node</code> <br/>TargetNumberOfNodes:  The number of nodes that the cluster will have after the resize operation is complete.<br/>TargetNodeType:  The node type that the cluster will have after the resize operation is complete.<br/>        """
        pass

    def describe_snapshot_copy_grants(self, SnapshotCopyGrantName: str=None, MaxRecords: int=None, Marker: str=None, TagKeys: List[String]=None, TagValues: List[String]=None) -> SnapshotCopyGrantMessage:
        """Returns a list of snapshot copy grants owned by the AWS account in the destination region.  For more information about managing snapshot copy grants, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-db-encryption.html">Amazon Redshift Database Encryption</a> in the <i>Amazon Redshift Cluster Management Guide</i>. 

        :param self:
        :param TagValues:  A tag value or values for which you want to return all matching resources that are associated with the specified value or values. For example, suppose that you have resources tagged with values called <code>admin</code> and <code>test</code>. If you specify both of these tag values in the request, Amazon Redshift returns a response with all resources that have either or both of these tag values associated with them.<br/>TagKeys:  A tag key or keys for which you want to return all matching resources that are associated with the specified key or keys. For example, suppose that you have resources tagged with keys called <code>owner</code> and <code>environment</code>. If you specify both of these tag keys in the request, Amazon Redshift returns a response with all resources that have either or both of these tag keys associated with them.<br/>Marker:  An optional parameter that specifies the starting point to return a set of response records. When the results of a <code>DescribeSnapshotCopyGrant</code> request exceed the value specified in <code>MaxRecords</code>, AWS returns a value in the <code>Marker</code> field of the response. You can retrieve the next set of response records by providing the returned marker value in the <code>Marker</code> parameter and retrying the request.  Constraints: You can specify either the <b>SnapshotCopyGrantName</b> parameter or the <b>Marker</b> parameter, but not both. <br/>MaxRecords:  The maximum number of response records to return in each call. If the number of remaining response records exceeds the specified <code>MaxRecords</code> value, a value is returned in a <code>marker</code> field of the response. You can retrieve the next set of records by retrying the command with the returned marker value.  Default: <code>100</code>  Constraints: minimum 20, maximum 100.<br/>SnapshotCopyGrantName:  The name of the snapshot copy grant.<br/>
        :return: SnapshotCopyGrants:  The list of <code>SnapshotCopyGrant</code> objects.<br/>Marker:  An optional parameter that specifies the starting point to return a set of response records. When the results of a <code>DescribeSnapshotCopyGrant</code> request exceed the value specified in <code>MaxRecords</code>, AWS returns a value in the <code>Marker</code> field of the response. You can retrieve the next set of response records by providing the returned marker value in the <code>Marker</code> parameter and retrying the request.  Constraints: You can specify either the <b>SnapshotCopyGrantName</b> parameter or the <b>Marker</b> parameter, but not both. <br/>        """
        pass

    def describe_snapshot_schedules(self, ClusterIdentifier: str=None, ScheduleIdentifier: str=None, TagKeys: List[String]=None, TagValues: List[String]=None, Marker: str=None, MaxRecords: int=None) -> DescribeSnapshotSchedulesOutputMessage:
        """Returns a list of snapshot schedules. 

        :param self:
        :param MaxRecords:  The maximum number or response records to return in each call. If the number of remaining response records exceeds the specified <code>MaxRecords</code> value, a value is returned in a <code>marker</code> field of the response. You can retrieve the next set of records by retrying the command with the returned <code>marker</code> value.<br/>Marker:  A value that indicates the starting point for the next set of response records in a subsequent request. If a value is returned in a response, you can retrieve the next set of records by providing this returned marker value in the <code>marker</code> parameter and retrying the command. If the <code>marker</code> field is empty, all response records have been retrieved for the request.<br/>TagValues:  The value corresponding to the key of the snapshot schedule tag.<br/>TagKeys:  The key value for a snapshot schedule tag.<br/>ScheduleIdentifier:  A unique identifier for a snapshot schedule.<br/>ClusterIdentifier:  The unique identifier for the cluster whose snapshot schedules you want to view.<br/>
        :return: Marker:  A value that indicates the starting point for the next set of response records in a subsequent request. If a value is returned in a response, you can retrieve the next set of records by providing this returned marker value in the <code>marker</code> parameter and retrying the command. If the <code>marker</code> field is empty, all response records have been retrieved for the request.<br/>SnapshotSchedules:  A list of SnapshotSchedules.<br/>        """
        pass

    def describe_storage(self) -> CustomerStorageMessage:
        """Returns the total amount of snapshot usage and provisioned storage for a user in megabytes.

        :param self:
        :param 
        :return: TotalProvisionedStorageInMegaBytes:  The total amount of storage currently provisioned.<br/>TotalBackupSizeInMegaBytes:  The total amount of storage currently used for snapshots.<br/>        """
        pass

    def describe_table_restore_status(self, ClusterIdentifier: str=None, TableRestoreRequestId: str=None, MaxRecords: int=None, Marker: str=None) -> TableRestoreStatusMessage:
        """Lists the status of one or more table restore requests made using the <a>RestoreTableFromClusterSnapshot</a> API action. If you don't specify a value for the <code>TableRestoreRequestId</code> parameter, then <code>DescribeTableRestoreStatus</code> returns the status of all table restore requests ordered by the date and time of the request in ascending order. Otherwise <code>DescribeTableRestoreStatus</code> returns the status of the table specified by <code>TableRestoreRequestId</code>.

        :param self:
        :param Marker:  An optional pagination token provided by a previous <code>DescribeTableRestoreStatus</code> request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by the <code>MaxRecords</code> parameter.<br/>MaxRecords:  The maximum number of records to include in the response. If more records exist than the specified <code>MaxRecords</code> value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.<br/>TableRestoreRequestId:  The identifier of the table restore request to return status for. If you don't specify a <code>TableRestoreRequestId</code> value, then <code>DescribeTableRestoreStatus</code> returns the status of all in-progress table restore requests.<br/>ClusterIdentifier:  The Amazon Redshift cluster that the table is being restored to.<br/>
        :return: Marker:  A pagination token that can be used in a subsequent <a>DescribeTableRestoreStatus</a> request.<br/>TableRestoreStatusDetails:  A list of status details for one or more table restore requests.<br/>        """
        pass

    def describe_tags(self, ResourceName: str=None, ResourceType: str=None, MaxRecords: int=None, Marker: str=None, TagKeys: List[String]=None, TagValues: List[String]=None) -> TaggedResourceListMessage:
        """Returns a list of tags. You can return tags from a specific resource by specifying an ARN, or you can return all tags for a given type of resource, such as clusters, snapshots, and so on. The following are limitations for <code>DescribeTags</code>:  <ul> <li> You cannot specify an ARN and a resource-type value together in the same request. </li> <li> You cannot use the <code>MaxRecords</code> and <code>Marker</code> parameters together with the ARN parameter. </li> <li> The <code>MaxRecords</code> parameter can be a range from 10 to 50 results to return in a request. </li> </ul> If you specify both tag keys and tag values in the same request, Amazon Redshift returns all resources that match any combination of the specified keys and values. For example, if you have <code>owner</code> and <code>environment</code> for tag keys, and <code>admin</code> and <code>test</code> for tag values, all resources that have any combination of those values are returned. If both tag keys and values are omitted from the request, resources are returned regardless of whether they have tag keys or values associated with them.

        :param self:
        :param TagValues:  A tag value or values for which you want to return all matching resources that are associated with the specified value or values. For example, suppose that you have resources tagged with values called <code>admin</code> and <code>test</code>. If you specify both of these tag values in the request, Amazon Redshift returns a response with all resources that have either or both of these tag values associated with them.<br/>TagKeys:  A tag key or keys for which you want to return all matching resources that are associated with the specified key or keys. For example, suppose that you have resources tagged with keys called <code>owner</code> and <code>environment</code>. If you specify both of these tag keys in the request, Amazon Redshift returns a response with all resources that have either or both of these tag keys associated with them.<br/>Marker:  A value that indicates the starting point for the next set of response records in a subsequent request. If a value is returned in a response, you can retrieve the next set of records by providing this returned marker value in the <code>marker</code> parameter and retrying the command. If the <code>marker</code> field is empty, all response records have been retrieved for the request. <br/>MaxRecords:  The maximum number or response records to return in each call. If the number of remaining response records exceeds the specified <code>MaxRecords</code> value, a value is returned in a <code>marker</code> field of the response. You can retrieve the next set of records by retrying the command with the returned <code>marker</code> value. <br/>ResourceType:  The type of resource with which you want to view tags. Valid resource types are:  <ul> <li> Cluster </li> <li> CIDR/IP </li> <li> EC2 security group </li> <li> Snapshot </li> <li> Cluster security group </li> <li> Subnet group </li> <li> HSM connection </li> <li> HSM certificate </li> <li> Parameter group </li> <li> Snapshot copy grant </li> </ul> For more information about Amazon Redshift resource types and constructing ARNs, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/redshift-iam-access-control-overview.html#redshift-iam-access-control-specify-actions">Specifying Policy Elements: Actions, Effects, Resources, and Principals</a> in the Amazon Redshift Cluster Management Guide. <br/>ResourceName:  The Amazon Resource Name (ARN) for which you want to describe the tag or tags. For example, <code>arn:aws:redshift:us-east-1:123456789:cluster:t1</code>. <br/>
        :return: Marker:  A value that indicates the starting point for the next set of response records in a subsequent request. If a value is returned in a response, you can retrieve the next set of records by providing this returned marker value in the <code>Marker</code> parameter and retrying the command. If the <code>Marker</code> field is empty, all response records have been retrieved for the request. <br/>TaggedResources:  A list of tags with their associated resources.<br/>        """
        pass

    def disable_logging(self, ClusterIdentifier: str) -> LoggingStatus:
        """Stops logging information, such as queries and connection attempts, for the specified Amazon Redshift cluster.

        :param self:
        :param ClusterIdentifier:  The identifier of the cluster on which logging is to be stopped. Example: <code>examplecluster</code> <br/>
        :return: LastFailureMessage:  The message indicating that logs failed to be delivered.<br/>LastFailureTime:  The last time when logs failed to be delivered.<br/>LastSuccessfulDeliveryTime:  The last time that logs were delivered.<br/>S3KeyPrefix:  The prefix applied to the log file names.<br/>BucketName:  The name of the S3 bucket where the log files are stored.<br/>LoggingEnabled:   <code>true</code> if logging is on, <code>false</code> if logging is off.<br/>        """
        pass

    def disable_snapshot_copy(self, ClusterIdentifier: str) -> DisableSnapshotCopyResult:
        """Disables the automatic copying of snapshots from one region to another region for a specified cluster. If your cluster and its snapshots are encrypted using a customer master key (CMK) from AWS KMS, use <a>DeleteSnapshotCopyGrant</a> to delete the grant that grants Amazon Redshift permission to the CMK in the destination region. 

        :param self:
        :param ClusterIdentifier:  The unique identifier of the source cluster that you want to disable copying of snapshots to a destination region. Constraints: Must be the valid name of an existing cluster that has cross-region snapshot copy enabled.<br/>
        :return: Cluster:  Describes a cluster.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResizeInfo</i> Returns the following: <ul> <li> AllowCancelResize: a boolean value indicating if the resize operation can be cancelled. </li> <li> ResizeType: Returns ClassicResize </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResizeInfo</b>: Returns the following: <ul> <li> AllowCancelResize: a boolean value indicating if the resize operation can be cancelled. </li> <li> ResizeType: Returns ClassicResize </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotScheduleState</i> The current state of the cluster snapshot schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotScheduleIdentifier</i> A unique identifier for the cluster snapshot schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeferredMaintenanceWindows</i> Describes a group of <code>DeferredMaintenanceWindow</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticResizeNumberOfNodeOptions</i> The number of nodes that you can resize the cluster to with the elastic resize method. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingActions</i> Cluster operations that are waiting to be started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoles</i> A list of AWS Identity and Access Management (IAM) roles that can be used by the cluster to access other AWS services.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS Key Management Service (AWS KMS) key ID of the encryption key used to encrypt data in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The list of tags for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterRevisionNumber</i> The specific revision number of the database in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticIpStatus</i> The status of the elastic IP (EIP) address.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ElasticIpStatus</b>: The status of the elastic IP (EIP) address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterNodes</i> The nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterPublicKey</i> The public key for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSnapshotCopyStatus</i> A value that returns the destination region and retention period that are configured for cross-region snapshot copy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ClusterSnapshotCopyStatus</b>: A value that returns the destination region and retention period that are configured for cross-region snapshot copy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmStatus</i> A value that reports whether the Amazon Redshift cluster has finished applying any hardware security module (HSM) settings changes specified in a modify cluster command. Values: active, applying&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>HsmStatus</b>: A value that reports whether the Amazon Redshift cluster has finished applying any hardware security module (HSM) settings changes specified in a modify cluster command. Values: active, applying<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTransferProgress</i> <p/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DataTransferProgress</b>: <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreStatus</i> A value that describes the status of a cluster restore action. This parameter returns null if the cluster was not created by restoring a snapshot.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RestoreStatus</b>: A value that describes the status of a cluster restore action. This parameter returns null if the cluster was not created by restoring a snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encrypted</i> A boolean value that, if <code>true</code>, indicates that data in the cluster is encrypted at rest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> A boolean value that, if <code>true</code>, indicates that the cluster can be accessed from a public network.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The number of compute nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowVersionUpgrade</i> A boolean value that, if <code>true</code>, indicates that major version upgrades will be applied automatically to the cluster during the maintenance window. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The version ID of the Amazon Redshift engine that is running on the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> A value that, if present, indicates that changes to the cluster are pending. Specific pending changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: A value that, if present, indicates that changes to the cluster are pending. Specific pending changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> The weekly time range, in Universal Coordinated Time (UTC), during which system maintenance can occur.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The name of the Availability Zone in which the cluster is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The identifier of the VPC the cluster is in, if the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSubnetGroupName</i> The name of the subnet group that is associated with the cluster. This parameter is valid only when the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterParameterGroups</i> The list of cluster parameter groups that are associated with this cluster. Each parameter group in the list is returned with its status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> A list of Amazon Virtual Private Cloud (Amazon VPC) security groups that are associated with the cluster. This parameter is returned only if the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSecurityGroups</i> A list of cluster security group that are associated with the cluster. Each security group is represented by an element that contains <code>ClusterSecurityGroup.Name</code> and <code>ClusterSecurityGroup.Status</code> subelements.  Cluster security groups are used when the cluster is not created in an Amazon Virtual Private Cloud (VPC). Clusters that are created in a VPC use VPC security groups, which are listed by the <b>VpcSecurityGroups</b> parameter. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The default number of days to retain a manual snapshot. If the value is -1, the snapshot is retained indefinitely. This setting doesn't change the retention period of existing snapshots. The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotRetentionPeriod</i> The number of days that automatic cluster snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> The date and time that the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> The connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: The connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The name of the initial database that was created when the cluster was created. This same name is returned for the life of the cluster. If an initial database was not specified, a database named <code>dev</code>dev was created by default. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> The master user name for the cluster. This name is used to connect to the database that is specified in the <b>DBName</b> parameter. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModifyStatus</i> The status of a modify operation, if any, initiated for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterAvailabilityStatus</i> The availability status of the cluster for queries. Possible values are the following: <ul> <li> Available - The cluster is available for queries.  </li> <li> Unavailable - The cluster is not available for queries. </li> <li> Maintenance - The cluster is intermittently available for queries due to maintenance activities. </li> <li> Modifying - The cluster is intermittently available for queries due to changes that modify the cluster. </li> <li> Failed - The cluster failed and is not available for queries. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterStatus</i>  The current state of the cluster. Possible values are the following: <ul> <li>  <code>available</code>  </li> <li>  <code>available, prep-for-resize</code>  </li> <li>  <code>available, resize-cleanup</code>  </li> <li>  <code>cancelling-resize</code>  </li> <li>  <code>creating</code>  </li> <li>  <code>deleting</code>  </li> <li>  <code>final-snapshot</code>  </li> <li>  <code>hardware-failure</code>  </li> <li>  <code>incompatible-hsm</code>  </li> <li>  <code>incompatible-network</code>  </li> <li>  <code>incompatible-parameters</code>  </li> <li>  <code>incompatible-restore</code>  </li> <li>  <code>modifying</code>  </li> <li>  <code>rebooting</code>  </li> <li>  <code>renaming</code>  </li> <li>  <code>resizing</code>  </li> <li>  <code>rotating-keys</code>  </li> <li>  <code>storage-full</code>  </li> <li>  <code>updating-hsm</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The node type for the nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The unique identifier of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS address of the Cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionType</i> The encryption type for a cluster. Possible values are: KMS and None. For the China region the possible values are None, and Legacy. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track that the cluster will change to during the next maintenance window.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> The pending or in-progress change of the ability to connect to the cluster from the public network.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The pending or in-progress change of the new identifier for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotRetentionPeriod</i> The pending or in-progress change of the automated snapshot retention period.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The pending or in-progress change of the service version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterType</i> The pending or in-progress change of the cluster type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The pending or in-progress change of the number of nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The pending or in-progress change of the cluster's node type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> The pending or in-progress change of the master user password for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedTimeToCompletionInSeconds</i> The estimate of the time remaining before the restore will complete. Returns 0 for a completed restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> The amount of time an in-progress restore has been running, or the amount of time it took a completed restore to finish.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ProgressInMegaBytes</i> The number of megabytes that have been transferred from snapshot storage.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotSizeInMegaBytes</i> The size of the set of snapshot data used to restore the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentRestoreRateInMegaBytesPerSecond</i> The number of megabytes per second being transferred from the backup storage. Returns the average rate for a completed backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the restore action. Returns starting, restoring, completed, or failed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> Describes the number of seconds that have elapsed during the data transfer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedTimeToCompletionInSeconds</i> Describes the estimated number of seconds remaining to complete the transfer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTransferredInMegaBytes</i> Describes the total amount of data that has been transfered in MB's.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalDataInMegaBytes</i> Describes the total amount of data to be transfered in megabytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentRateInMegaBytesPerSecond</i> Describes the data transfer rate in MB's per second.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Describes the status of the cluster. While the transfer is in progress the status is <code>transferringdata</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Reports whether the Amazon Redshift cluster has finished applying any HSM settings changes specified in a modify cluster command. Values: active, applying<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmConfigurationIdentifier</i> Specifies the name of the HSM configuration that contains the information the Amazon Redshift cluster can use to retrieve and store keys in an HSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmClientCertificateIdentifier</i> Specifies the name of the HSM client certificate the Amazon Redshift cluster uses to retrieve the data encryption keys stored in an HSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCopyGrantName</i> The name of the snapshot copy grant.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The number of days that automated snapshots are retained in the destination region after they are copied from a source region. If the value is -1, the manual snapshot is retained indefinitely.  The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RetentionPeriod</i> The number of days that automated snapshots are retained in the destination region after they are copied from a source region.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationRegion</i> The destination region that snapshots are automatically copied to when cross-region snapshot copy is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the elastic IP (EIP) address.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticIp</i> The elastic IP (EIP) address for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowCancelResize</i> A boolean value indicating if the resize operation can be cancelled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResizeType</i> Returns the value <code>ClassicResize</code>.<br/>        """
        pass

    def enable_logging(self, BucketName: str, ClusterIdentifier: str, S3KeyPrefix: str=None) -> LoggingStatus:
        """Starts logging information, such as queries and connection attempts, for the specified Amazon Redshift cluster.

        :param self:
        :param S3KeyPrefix:  The prefix applied to the log file names. Constraints: <ul> <li> Cannot exceed 512 characters </li> <li> Cannot contain spaces( ), double quotes ("), single quotes ('), a backslash (\), or control characters. The hexadecimal codes for invalid characters are:  <ul> <li> x00 to x20 </li> <li> x22 </li> <li> x27 </li> <li> x5c </li> <li> x7f or larger </li> </ul> </li> </ul><br/>BucketName:  The name of an existing S3 bucket where the log files are to be stored. Constraints: <ul> <li> Must be in the same region as the cluster </li> <li> The cluster must have read bucket and put object permissions </li> </ul><br/>ClusterIdentifier:  The identifier of the cluster on which logging is to be started. Example: <code>examplecluster</code> <br/>
        :return: LastFailureMessage:  The message indicating that logs failed to be delivered.<br/>LastFailureTime:  The last time when logs failed to be delivered.<br/>LastSuccessfulDeliveryTime:  The last time that logs were delivered.<br/>S3KeyPrefix:  The prefix applied to the log file names.<br/>BucketName:  The name of the S3 bucket where the log files are stored.<br/>LoggingEnabled:   <code>true</code> if logging is on, <code>false</code> if logging is off.<br/>        """
        pass

    def enable_snapshot_copy(self, DestinationRegion: str, ClusterIdentifier: str, RetentionPeriod: int=None, SnapshotCopyGrantName: str=None, ManualSnapshotRetentionPeriod: int=None) -> EnableSnapshotCopyResult:
        """Enables the automatic copy of snapshots from one region to another region for a specified cluster.

        :param self:
        :param ManualSnapshotRetentionPeriod:  The number of days to retain newly copied snapshots in the destination AWS Region after they are copied from the source AWS Region. If the value is -1, the manual snapshot is retained indefinitely.  The value must be either -1 or an integer between 1 and 3,653.<br/>SnapshotCopyGrantName:  The name of the snapshot copy grant to use when snapshots of an AWS KMS-encrypted cluster are copied to the destination region.<br/>RetentionPeriod:  The number of days to retain automated snapshots in the destination region after they are copied from the source region. Default: 7. Constraints: Must be at least 1 and no more than 35.<br/>DestinationRegion:  The destination AWS Region that you want to copy snapshots to. Constraints: Must be the name of a valid AWS Region. For more information, see <a href="https://docs.aws.amazon.com/general/latest/gr/rande.html#redshift_region">Regions and Endpoints</a> in the Amazon Web Services General Reference. <br/>ClusterIdentifier:  The unique identifier of the source cluster to copy snapshots from. Constraints: Must be the valid name of an existing cluster that does not already have cross-region snapshot copy enabled.<br/>
        :return: Cluster:  Describes a cluster.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResizeInfo</i> Returns the following: <ul> <li> AllowCancelResize: a boolean value indicating if the resize operation can be cancelled. </li> <li> ResizeType: Returns ClassicResize </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResizeInfo</b>: Returns the following: <ul> <li> AllowCancelResize: a boolean value indicating if the resize operation can be cancelled. </li> <li> ResizeType: Returns ClassicResize </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotScheduleState</i> The current state of the cluster snapshot schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotScheduleIdentifier</i> A unique identifier for the cluster snapshot schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeferredMaintenanceWindows</i> Describes a group of <code>DeferredMaintenanceWindow</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticResizeNumberOfNodeOptions</i> The number of nodes that you can resize the cluster to with the elastic resize method. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingActions</i> Cluster operations that are waiting to be started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoles</i> A list of AWS Identity and Access Management (IAM) roles that can be used by the cluster to access other AWS services.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS Key Management Service (AWS KMS) key ID of the encryption key used to encrypt data in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The list of tags for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterRevisionNumber</i> The specific revision number of the database in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticIpStatus</i> The status of the elastic IP (EIP) address.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ElasticIpStatus</b>: The status of the elastic IP (EIP) address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterNodes</i> The nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterPublicKey</i> The public key for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSnapshotCopyStatus</i> A value that returns the destination region and retention period that are configured for cross-region snapshot copy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ClusterSnapshotCopyStatus</b>: A value that returns the destination region and retention period that are configured for cross-region snapshot copy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmStatus</i> A value that reports whether the Amazon Redshift cluster has finished applying any hardware security module (HSM) settings changes specified in a modify cluster command. Values: active, applying&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>HsmStatus</b>: A value that reports whether the Amazon Redshift cluster has finished applying any hardware security module (HSM) settings changes specified in a modify cluster command. Values: active, applying<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTransferProgress</i> <p/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DataTransferProgress</b>: <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreStatus</i> A value that describes the status of a cluster restore action. This parameter returns null if the cluster was not created by restoring a snapshot.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RestoreStatus</b>: A value that describes the status of a cluster restore action. This parameter returns null if the cluster was not created by restoring a snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encrypted</i> A boolean value that, if <code>true</code>, indicates that data in the cluster is encrypted at rest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> A boolean value that, if <code>true</code>, indicates that the cluster can be accessed from a public network.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The number of compute nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowVersionUpgrade</i> A boolean value that, if <code>true</code>, indicates that major version upgrades will be applied automatically to the cluster during the maintenance window. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The version ID of the Amazon Redshift engine that is running on the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> A value that, if present, indicates that changes to the cluster are pending. Specific pending changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: A value that, if present, indicates that changes to the cluster are pending. Specific pending changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> The weekly time range, in Universal Coordinated Time (UTC), during which system maintenance can occur.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The name of the Availability Zone in which the cluster is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The identifier of the VPC the cluster is in, if the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSubnetGroupName</i> The name of the subnet group that is associated with the cluster. This parameter is valid only when the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterParameterGroups</i> The list of cluster parameter groups that are associated with this cluster. Each parameter group in the list is returned with its status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> A list of Amazon Virtual Private Cloud (Amazon VPC) security groups that are associated with the cluster. This parameter is returned only if the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSecurityGroups</i> A list of cluster security group that are associated with the cluster. Each security group is represented by an element that contains <code>ClusterSecurityGroup.Name</code> and <code>ClusterSecurityGroup.Status</code> subelements.  Cluster security groups are used when the cluster is not created in an Amazon Virtual Private Cloud (VPC). Clusters that are created in a VPC use VPC security groups, which are listed by the <b>VpcSecurityGroups</b> parameter. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The default number of days to retain a manual snapshot. If the value is -1, the snapshot is retained indefinitely. This setting doesn't change the retention period of existing snapshots. The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotRetentionPeriod</i> The number of days that automatic cluster snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> The date and time that the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> The connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: The connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The name of the initial database that was created when the cluster was created. This same name is returned for the life of the cluster. If an initial database was not specified, a database named <code>dev</code>dev was created by default. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> The master user name for the cluster. This name is used to connect to the database that is specified in the <b>DBName</b> parameter. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModifyStatus</i> The status of a modify operation, if any, initiated for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterAvailabilityStatus</i> The availability status of the cluster for queries. Possible values are the following: <ul> <li> Available - The cluster is available for queries.  </li> <li> Unavailable - The cluster is not available for queries. </li> <li> Maintenance - The cluster is intermittently available for queries due to maintenance activities. </li> <li> Modifying - The cluster is intermittently available for queries due to changes that modify the cluster. </li> <li> Failed - The cluster failed and is not available for queries. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterStatus</i>  The current state of the cluster. Possible values are the following: <ul> <li>  <code>available</code>  </li> <li>  <code>available, prep-for-resize</code>  </li> <li>  <code>available, resize-cleanup</code>  </li> <li>  <code>cancelling-resize</code>  </li> <li>  <code>creating</code>  </li> <li>  <code>deleting</code>  </li> <li>  <code>final-snapshot</code>  </li> <li>  <code>hardware-failure</code>  </li> <li>  <code>incompatible-hsm</code>  </li> <li>  <code>incompatible-network</code>  </li> <li>  <code>incompatible-parameters</code>  </li> <li>  <code>incompatible-restore</code>  </li> <li>  <code>modifying</code>  </li> <li>  <code>rebooting</code>  </li> <li>  <code>renaming</code>  </li> <li>  <code>resizing</code>  </li> <li>  <code>rotating-keys</code>  </li> <li>  <code>storage-full</code>  </li> <li>  <code>updating-hsm</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The node type for the nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The unique identifier of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS address of the Cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionType</i> The encryption type for a cluster. Possible values are: KMS and None. For the China region the possible values are None, and Legacy. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track that the cluster will change to during the next maintenance window.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> The pending or in-progress change of the ability to connect to the cluster from the public network.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The pending or in-progress change of the new identifier for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotRetentionPeriod</i> The pending or in-progress change of the automated snapshot retention period.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The pending or in-progress change of the service version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterType</i> The pending or in-progress change of the cluster type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The pending or in-progress change of the number of nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The pending or in-progress change of the cluster's node type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> The pending or in-progress change of the master user password for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedTimeToCompletionInSeconds</i> The estimate of the time remaining before the restore will complete. Returns 0 for a completed restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> The amount of time an in-progress restore has been running, or the amount of time it took a completed restore to finish.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ProgressInMegaBytes</i> The number of megabytes that have been transferred from snapshot storage.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotSizeInMegaBytes</i> The size of the set of snapshot data used to restore the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentRestoreRateInMegaBytesPerSecond</i> The number of megabytes per second being transferred from the backup storage. Returns the average rate for a completed backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the restore action. Returns starting, restoring, completed, or failed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> Describes the number of seconds that have elapsed during the data transfer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedTimeToCompletionInSeconds</i> Describes the estimated number of seconds remaining to complete the transfer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTransferredInMegaBytes</i> Describes the total amount of data that has been transfered in MB's.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalDataInMegaBytes</i> Describes the total amount of data to be transfered in megabytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentRateInMegaBytesPerSecond</i> Describes the data transfer rate in MB's per second.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Describes the status of the cluster. While the transfer is in progress the status is <code>transferringdata</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Reports whether the Amazon Redshift cluster has finished applying any HSM settings changes specified in a modify cluster command. Values: active, applying<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmConfigurationIdentifier</i> Specifies the name of the HSM configuration that contains the information the Amazon Redshift cluster can use to retrieve and store keys in an HSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmClientCertificateIdentifier</i> Specifies the name of the HSM client certificate the Amazon Redshift cluster uses to retrieve the data encryption keys stored in an HSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCopyGrantName</i> The name of the snapshot copy grant.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The number of days that automated snapshots are retained in the destination region after they are copied from a source region. If the value is -1, the manual snapshot is retained indefinitely.  The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RetentionPeriod</i> The number of days that automated snapshots are retained in the destination region after they are copied from a source region.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationRegion</i> The destination region that snapshots are automatically copied to when cross-region snapshot copy is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the elastic IP (EIP) address.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticIp</i> The elastic IP (EIP) address for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowCancelResize</i> A boolean value indicating if the resize operation can be cancelled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResizeType</i> Returns the value <code>ClassicResize</code>.<br/>        """
        pass

    def get_cluster_credentials(self, ClusterIdentifier: str, DbUser: str, DbName: str=None, DurationSeconds: int=None, AutoCreate: BooleanOptional=None, DbGroups: List[String]=None) -> ClusterCredentials:
        """Returns a database user name and temporary password with temporary authorization to log on to an Amazon Redshift database. The action returns the database user name prefixed with <code>IAM:</code> if <code>AutoCreate</code> is <code>False</code> or <code>IAMA:</code> if <code>AutoCreate</code> is <code>True</code>. You can optionally specify one or more database user groups that the user will join at log on. By default, the temporary credentials expire in 900 seconds. You can optionally specify a duration between 900 seconds (15 minutes) and 3600 seconds (60 minutes). For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/generating-user-credentials.html">Using IAM Authentication to Generate Database User Credentials</a> in the Amazon Redshift Cluster Management Guide. The AWS Identity and Access Management (IAM)user or role that executes GetClusterCredentials must have an IAM policy attached that allows access to all necessary actions and resources. For more information about permissions, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/redshift-iam-access-control-identity-based.html#redshift-policy-resources.getclustercredentials-resources">Resource Policies for GetClusterCredentials</a> in the Amazon Redshift Cluster Management Guide. If the <code>DbGroups</code> parameter is specified, the IAM policy must allow the <code>redshift:JoinGroup</code> action with access to the listed <code>dbgroups</code>.  In addition, if the <code>AutoCreate</code> parameter is set to <code>True</code>, then the policy must include the <code>redshift:CreateClusterUser</code> privilege. If the <code>DbName</code> parameter is specified, the IAM policy must allow access to the resource <code>dbname</code> for the specified database name. 

        :param self:
        :param DbGroups:  A list of the names of existing database groups that the user named in <code>DbUser</code> will join for the current session, in addition to any group memberships for an existing user. If not specified, a new user is added only to PUBLIC. Database group name constraints <ul> <li> Must be 1 to 64 alphanumeric characters or hyphens </li> <li> Must contain only lowercase letters, numbers, underscore, plus sign, period (dot), at symbol (@), or hyphen. </li> <li> First character must be a letter. </li> <li> Must not contain a colon ( : ) or slash ( / ).  </li> <li> Cannot be a reserved word. A list of reserved words can be found in <a href="http://docs.aws.amazon.com/redshift/latest/dg/r_pg_keywords.html">Reserved Words</a> in the Amazon Redshift Database Developer Guide. </li> </ul><br/>AutoCreate:  Create a database user with the name specified for the user named in <code>DbUser</code> if one does not exist.<br/>DurationSeconds:  The number of seconds until the returned temporary password expires. Constraint: minimum 900, maximum 3600. Default: 900<br/>ClusterIdentifier:  The unique identifier of the cluster that contains the database for which your are requesting credentials. This parameter is case sensitive.<br/>DbName:  The name of a database that <code>DbUser</code> is authorized to log on to. If <code>DbName</code> is not specified, <code>DbUser</code> can log on to any existing database. Constraints: <ul> <li> Must be 1 to 64 alphanumeric characters or hyphens </li> <li> Must contain only lowercase letters, numbers, underscore, plus sign, period (dot), at symbol (@), or hyphen. </li> <li> First character must be a letter. </li> <li> Must not contain a colon ( : ) or slash ( / ).  </li> <li> Cannot be a reserved word. A list of reserved words can be found in <a href="http://docs.aws.amazon.com/redshift/latest/dg/r_pg_keywords.html">Reserved Words</a> in the Amazon Redshift Database Developer Guide. </li> </ul><br/>DbUser:  The name of a database user. If a user name matching <code>DbUser</code> exists in the database, the temporary user credentials have the same permissions as the existing user. If <code>DbUser</code> doesn't exist in the database and <code>Autocreate</code> is <code>True</code>, a new user is created using the value for <code>DbUser</code> with PUBLIC permissions. If a database user matching the value for <code>DbUser</code> doesn't exist and <code>Autocreate</code> is <code>False</code>, then the command succeeds but the connection attempt will fail because the user doesn't exist in the database. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/dg/r_CREATE_USER.html">CREATE USER</a> in the Amazon Redshift Database Developer Guide.  Constraints: <ul> <li> Must be 1 to 64 alphanumeric characters or hyphens. The user name can't be <code>PUBLIC</code>. </li> <li> Must contain only lowercase letters, numbers, underscore, plus sign, period (dot), at symbol (@), or hyphen. </li> <li> First character must be a letter. </li> <li> Must not contain a colon ( : ) or slash ( / ).  </li> <li> Cannot be a reserved word. A list of reserved words can be found in <a href="http://docs.aws.amazon.com/redshift/latest/dg/r_pg_keywords.html">Reserved Words</a> in the Amazon Redshift Database Developer Guide. </li> </ul><br/>
        :return: Expiration:  The date and time the password in <code>DbPassword</code> expires.<br/>DbPassword:  A temporary password that authorizes the user name returned by <code>DbUser</code> to log on to the database <code>DbName</code>. <br/>DbUser:  A database user name that is authorized to log on to the database <code>DbName</code> using the password <code>DbPassword</code>. If the specified DbUser exists in the database, the new user name has the same database privileges as the the user named in DbUser. By default, the user is added to PUBLIC. If the <code>DbGroups</code> parameter is specifed, <code>DbUser</code> is added to the listed groups for any sessions created using these credentials.<br/>        """
        pass

    def get_reserved_node_exchange_offerings(self, ReservedNodeId: str, MaxRecords: int=None, Marker: str=None) -> GetReservedNodeExchangeOfferingsOutputMessage:
        """Returns an array of DC2 ReservedNodeOfferings that matches the payment type, term, and usage price of the given DC1 reserved node.

        :param self:
        :param Marker:  A value that indicates the starting point for the next set of ReservedNodeOfferings.<br/>MaxRecords:  An integer setting the maximum number of ReservedNodeOfferings to retrieve.<br/>ReservedNodeId:  A string representing the node identifier for the DC1 Reserved Node to be exchanged.<br/>
        :return: ReservedNodeOfferings:  Returns an array of <a>ReservedNodeOffering</a> objects.<br/>Marker:  An optional parameter that specifies the starting point for returning a set of response records. When the results of a <code>GetReservedNodeExchangeOfferings</code> request exceed the value specified in MaxRecords, Amazon Redshift returns a value in the marker field of the response. You can retrieve the next set of response records by providing the returned marker value in the marker parameter and retrying the request. <br/>        """
        pass

    def modify_cluster(self, ClusterIdentifier: str, ClusterType: str=None, NodeType: str=None, NumberOfNodes: int=None, ClusterSecurityGroups: List[String]=None, VpcSecurityGroupIds: List[String]=None, MasterUserPassword: str=None, ClusterParameterGroupName: str=None, AutomatedSnapshotRetentionPeriod: int=None, ManualSnapshotRetentionPeriod: int=None, PreferredMaintenanceWindow: str=None, ClusterVersion: str=None, AllowVersionUpgrade: BooleanOptional=None, HsmClientCertificateIdentifier: str=None, HsmConfigurationIdentifier: str=None, NewClusterIdentifier: str=None, PubliclyAccessible: BooleanOptional=None, ElasticIp: str=None, EnhancedVpcRouting: BooleanOptional=None, MaintenanceTrackName: str=None, Encrypted: BooleanOptional=None, KmsKeyId: str=None) -> ModifyClusterResult:
        """Modifies the settings for a cluster. For example, you can add another security or parameter group, update the preferred maintenance window, or change the master user password. Resetting a cluster password or modifying the security groups associated with a cluster do not need a reboot. However, modifying a parameter group requires a reboot for parameters to take effect. For more information about managing clusters, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-clusters.html">Amazon Redshift Clusters</a> in the <i>Amazon Redshift Cluster Management Guide</i>. You can also change node type and the number of nodes to scale up or down the cluster. When resizing a cluster, you must specify both the number of nodes and the node type even if one of the parameters does not change.

        :param self:
        :param KmsKeyId:  The AWS Key Management Service (KMS) key ID of the encryption key that you want to use to encrypt data in the cluster.<br/>Encrypted:  Indicates whether the cluster is encrypted. If the cluster is encrypted and you provide a value for the <code>KmsKeyId</code> parameter, we will encrypt the cluster with the provided <code>KmsKeyId</code>. If you don't provide a <code>KmsKeyId</code>, we will encrypt with the default key. In the China region we will use legacy encryption if you specify that the cluster is encrypted.<br/>MaintenanceTrackName:  The name for the maintenance track that you want to assign for the cluster. This name change is asynchronous. The new track name stays in the <code>PendingModifiedValues</code> for the cluster until the next maintenance window. When the maintenance track changes, the cluster is switched to the latest cluster release available for the maintenance track. At this point, the maintenance track name is applied.<br/>EnhancedVpcRouting:  An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>ElasticIp:  The Elastic IP (EIP) address for the cluster. Constraints: The cluster must be provisioned in EC2-VPC and publicly-accessible through an Internet gateway. For more information about provisioning clusters in EC2-VPC, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-clusters.html#cluster-platforms">Supported Platforms to Launch Your Cluster</a> in the Amazon Redshift Cluster Management Guide.<br/>PubliclyAccessible:  If <code>true</code>, the cluster can be accessed from a public network. Only clusters in VPCs can be set to be publicly available.<br/>NewClusterIdentifier:  The new identifier for the cluster. Constraints: <ul> <li> Must contain from 1 to 63 alphanumeric characters or hyphens. </li> <li> Alphabetic characters must be lowercase. </li> <li> First character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> <li> Must be unique for all clusters within an AWS account. </li> </ul> Example: <code>examplecluster</code> <br/>HsmConfigurationIdentifier:  Specifies the name of the HSM configuration that contains the information the Amazon Redshift cluster can use to retrieve and store keys in an HSM.<br/>HsmClientCertificateIdentifier:  Specifies the name of the HSM client certificate the Amazon Redshift cluster uses to retrieve the data encryption keys stored in an HSM.<br/>AllowVersionUpgrade:  If <code>true</code>, major version upgrades will be applied automatically to the cluster during the maintenance window.  Default: <code>false</code> <br/>ClusterVersion:  The new version number of the Amazon Redshift engine to upgrade to. For major version upgrades, if a non-default cluster parameter group is currently in use, a new cluster parameter group in the cluster parameter group family for the new version must be specified. The new cluster parameter group can be the default for that cluster parameter group family. For more information about parameters and parameter groups, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-parameter-groups.html">Amazon Redshift Parameter Groups</a> in the <i>Amazon Redshift Cluster Management Guide</i>. Example: <code>1.0</code> <br/>PreferredMaintenanceWindow:  The weekly time range (in UTC) during which system maintenance can occur, if necessary. If system maintenance is necessary during the window, it may result in an outage. This maintenance window change is made immediately. If the new maintenance window indicates the current time, there must be at least 120 minutes between the current time and end of the window in order to ensure that pending changes are applied. Default: Uses existing setting. Format: ddd:hh24:mi-ddd:hh24:mi, for example <code>wed:07:30-wed:08:00</code>. Valid Days: Mon | Tue | Wed | Thu | Fri | Sat | Sun Constraints: Must be at least 30 minutes.<br/>ManualSnapshotRetentionPeriod:  The default for number of days that a newly created manual snapshot is retained. If the value is -1, the manual snapshot is retained indefinitely. This value doesn't retroactively change the retention periods of existing manual snapshots. The value must be either -1 or an integer between 1 and 3,653. The default value is -1.<br/>AutomatedSnapshotRetentionPeriod:  The number of days that automated snapshots are retained. If the value is 0, automated snapshots are disabled. Even if automated snapshots are disabled, you can still create manual snapshots when you want with <a>CreateClusterSnapshot</a>.  If you decrease the automated snapshot retention period from its current value, existing automated snapshots that fall outside of the new retention period will be immediately deleted. Default: Uses existing setting. Constraints: Must be a value from 0 to 35.<br/>ClusterParameterGroupName:  The name of the cluster parameter group to apply to this cluster. This change is applied only after the cluster is rebooted. To reboot a cluster use <a>RebootCluster</a>.  Default: Uses existing setting. Constraints: The cluster parameter group must be in the same parameter group family that matches the cluster version.<br/>MasterUserPassword:  The new password for the cluster master user. This change is asynchronously applied as soon as possible. Between the time of the request and the completion of the request, the <code>MasterUserPassword</code> element exists in the <code>PendingModifiedValues</code> element of the operation response.  <note> Operations never return the password, so this operation provides a way to regain access to the master user account for a cluster if the password is lost. </note> Default: Uses existing setting. Constraints: <ul> <li> Must be between 8 and 64 characters in length. </li> <li> Must contain at least one uppercase letter. </li> <li> Must contain at least one lowercase letter. </li> <li> Must contain one number. </li> <li> Can be any printable ASCII character (ASCII code 33 to 126) except ' (single quote), " (double quote), \, /, @, or space. </li> </ul><br/>VpcSecurityGroupIds:  A list of virtual private cloud (VPC) security groups to be associated with the cluster. This change is asynchronously applied as soon as possible.<br/>ClusterSecurityGroups:  A list of cluster security groups to be authorized on this cluster. This change is asynchronously applied as soon as possible. Security groups currently associated with the cluster, and not in the list of groups to apply, will be revoked from the cluster. Constraints: <ul> <li> Must be 1 to 255 alphanumeric characters or hyphens </li> <li> First character must be a letter </li> <li> Cannot end with a hyphen or contain two consecutive hyphens </li> </ul><br/>NumberOfNodes:  The new number of nodes of the cluster. If you specify a new number of nodes, you must also specify the node type parameter. When you submit your request to resize a cluster, Amazon Redshift sets access permissions for the cluster to read-only. After Amazon Redshift provisions a new cluster according to your resize requirements, there will be a temporary outage while the old cluster is deleted and your connection is switched to the new cluster. When the new connection is complete, the original access permissions for the cluster are restored. You can use <a>DescribeResize</a> to track the progress of the resize request.  Valid Values: Integer greater than <code>0</code>.<br/>NodeType:  The new node type of the cluster. If you specify a new node type, you must also specify the number of nodes parameter. When you submit your request to resize a cluster, Amazon Redshift sets access permissions for the cluster to read-only. After Amazon Redshift provisions a new cluster according to your resize requirements, there will be a temporary outage while the old cluster is deleted and your connection is switched to the new cluster. When the new connection is complete, the original access permissions for the cluster are restored. You can use <a>DescribeResize</a> to track the progress of the resize request.  Valid Values: <code>ds2.xlarge</code> | <code>ds2.8xlarge</code> | <code>dc1.large</code> | <code>dc1.8xlarge</code> | <code>dc2.large</code> | <code>dc2.8xlarge</code> <br/>ClusterType:  The new cluster type. When you submit your cluster resize request, your existing cluster goes into a read-only mode. After Amazon Redshift provisions a new cluster based on your resize requirements, there will be outage for a period while the old cluster is deleted and your connection is switched to the new cluster. You can use <a>DescribeResize</a> to track the progress of the resize request.  Valid Values: <code> multi-node | single-node </code> <br/>ClusterIdentifier:  The unique identifier of the cluster to be modified. Example: <code>examplecluster</code> <br/>
        :return: Cluster:  Describes a cluster.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResizeInfo</i> Returns the following: <ul> <li> AllowCancelResize: a boolean value indicating if the resize operation can be cancelled. </li> <li> ResizeType: Returns ClassicResize </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResizeInfo</b>: Returns the following: <ul> <li> AllowCancelResize: a boolean value indicating if the resize operation can be cancelled. </li> <li> ResizeType: Returns ClassicResize </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotScheduleState</i> The current state of the cluster snapshot schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotScheduleIdentifier</i> A unique identifier for the cluster snapshot schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeferredMaintenanceWindows</i> Describes a group of <code>DeferredMaintenanceWindow</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticResizeNumberOfNodeOptions</i> The number of nodes that you can resize the cluster to with the elastic resize method. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingActions</i> Cluster operations that are waiting to be started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoles</i> A list of AWS Identity and Access Management (IAM) roles that can be used by the cluster to access other AWS services.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS Key Management Service (AWS KMS) key ID of the encryption key used to encrypt data in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The list of tags for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterRevisionNumber</i> The specific revision number of the database in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticIpStatus</i> The status of the elastic IP (EIP) address.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ElasticIpStatus</b>: The status of the elastic IP (EIP) address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterNodes</i> The nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterPublicKey</i> The public key for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSnapshotCopyStatus</i> A value that returns the destination region and retention period that are configured for cross-region snapshot copy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ClusterSnapshotCopyStatus</b>: A value that returns the destination region and retention period that are configured for cross-region snapshot copy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmStatus</i> A value that reports whether the Amazon Redshift cluster has finished applying any hardware security module (HSM) settings changes specified in a modify cluster command. Values: active, applying&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>HsmStatus</b>: A value that reports whether the Amazon Redshift cluster has finished applying any hardware security module (HSM) settings changes specified in a modify cluster command. Values: active, applying<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTransferProgress</i> <p/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DataTransferProgress</b>: <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreStatus</i> A value that describes the status of a cluster restore action. This parameter returns null if the cluster was not created by restoring a snapshot.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RestoreStatus</b>: A value that describes the status of a cluster restore action. This parameter returns null if the cluster was not created by restoring a snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encrypted</i> A boolean value that, if <code>true</code>, indicates that data in the cluster is encrypted at rest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> A boolean value that, if <code>true</code>, indicates that the cluster can be accessed from a public network.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The number of compute nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowVersionUpgrade</i> A boolean value that, if <code>true</code>, indicates that major version upgrades will be applied automatically to the cluster during the maintenance window. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The version ID of the Amazon Redshift engine that is running on the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> A value that, if present, indicates that changes to the cluster are pending. Specific pending changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: A value that, if present, indicates that changes to the cluster are pending. Specific pending changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> The weekly time range, in Universal Coordinated Time (UTC), during which system maintenance can occur.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The name of the Availability Zone in which the cluster is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The identifier of the VPC the cluster is in, if the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSubnetGroupName</i> The name of the subnet group that is associated with the cluster. This parameter is valid only when the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterParameterGroups</i> The list of cluster parameter groups that are associated with this cluster. Each parameter group in the list is returned with its status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> A list of Amazon Virtual Private Cloud (Amazon VPC) security groups that are associated with the cluster. This parameter is returned only if the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSecurityGroups</i> A list of cluster security group that are associated with the cluster. Each security group is represented by an element that contains <code>ClusterSecurityGroup.Name</code> and <code>ClusterSecurityGroup.Status</code> subelements.  Cluster security groups are used when the cluster is not created in an Amazon Virtual Private Cloud (VPC). Clusters that are created in a VPC use VPC security groups, which are listed by the <b>VpcSecurityGroups</b> parameter. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The default number of days to retain a manual snapshot. If the value is -1, the snapshot is retained indefinitely. This setting doesn't change the retention period of existing snapshots. The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotRetentionPeriod</i> The number of days that automatic cluster snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> The date and time that the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> The connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: The connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The name of the initial database that was created when the cluster was created. This same name is returned for the life of the cluster. If an initial database was not specified, a database named <code>dev</code>dev was created by default. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> The master user name for the cluster. This name is used to connect to the database that is specified in the <b>DBName</b> parameter. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModifyStatus</i> The status of a modify operation, if any, initiated for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterAvailabilityStatus</i> The availability status of the cluster for queries. Possible values are the following: <ul> <li> Available - The cluster is available for queries.  </li> <li> Unavailable - The cluster is not available for queries. </li> <li> Maintenance - The cluster is intermittently available for queries due to maintenance activities. </li> <li> Modifying - The cluster is intermittently available for queries due to changes that modify the cluster. </li> <li> Failed - The cluster failed and is not available for queries. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterStatus</i>  The current state of the cluster. Possible values are the following: <ul> <li>  <code>available</code>  </li> <li>  <code>available, prep-for-resize</code>  </li> <li>  <code>available, resize-cleanup</code>  </li> <li>  <code>cancelling-resize</code>  </li> <li>  <code>creating</code>  </li> <li>  <code>deleting</code>  </li> <li>  <code>final-snapshot</code>  </li> <li>  <code>hardware-failure</code>  </li> <li>  <code>incompatible-hsm</code>  </li> <li>  <code>incompatible-network</code>  </li> <li>  <code>incompatible-parameters</code>  </li> <li>  <code>incompatible-restore</code>  </li> <li>  <code>modifying</code>  </li> <li>  <code>rebooting</code>  </li> <li>  <code>renaming</code>  </li> <li>  <code>resizing</code>  </li> <li>  <code>rotating-keys</code>  </li> <li>  <code>storage-full</code>  </li> <li>  <code>updating-hsm</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The node type for the nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The unique identifier of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS address of the Cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionType</i> The encryption type for a cluster. Possible values are: KMS and None. For the China region the possible values are None, and Legacy. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track that the cluster will change to during the next maintenance window.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> The pending or in-progress change of the ability to connect to the cluster from the public network.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The pending or in-progress change of the new identifier for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotRetentionPeriod</i> The pending or in-progress change of the automated snapshot retention period.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The pending or in-progress change of the service version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterType</i> The pending or in-progress change of the cluster type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The pending or in-progress change of the number of nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The pending or in-progress change of the cluster's node type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> The pending or in-progress change of the master user password for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedTimeToCompletionInSeconds</i> The estimate of the time remaining before the restore will complete. Returns 0 for a completed restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> The amount of time an in-progress restore has been running, or the amount of time it took a completed restore to finish.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ProgressInMegaBytes</i> The number of megabytes that have been transferred from snapshot storage.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotSizeInMegaBytes</i> The size of the set of snapshot data used to restore the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentRestoreRateInMegaBytesPerSecond</i> The number of megabytes per second being transferred from the backup storage. Returns the average rate for a completed backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the restore action. Returns starting, restoring, completed, or failed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> Describes the number of seconds that have elapsed during the data transfer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedTimeToCompletionInSeconds</i> Describes the estimated number of seconds remaining to complete the transfer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTransferredInMegaBytes</i> Describes the total amount of data that has been transfered in MB's.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalDataInMegaBytes</i> Describes the total amount of data to be transfered in megabytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentRateInMegaBytesPerSecond</i> Describes the data transfer rate in MB's per second.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Describes the status of the cluster. While the transfer is in progress the status is <code>transferringdata</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Reports whether the Amazon Redshift cluster has finished applying any HSM settings changes specified in a modify cluster command. Values: active, applying<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmConfigurationIdentifier</i> Specifies the name of the HSM configuration that contains the information the Amazon Redshift cluster can use to retrieve and store keys in an HSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmClientCertificateIdentifier</i> Specifies the name of the HSM client certificate the Amazon Redshift cluster uses to retrieve the data encryption keys stored in an HSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCopyGrantName</i> The name of the snapshot copy grant.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The number of days that automated snapshots are retained in the destination region after they are copied from a source region. If the value is -1, the manual snapshot is retained indefinitely.  The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RetentionPeriod</i> The number of days that automated snapshots are retained in the destination region after they are copied from a source region.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationRegion</i> The destination region that snapshots are automatically copied to when cross-region snapshot copy is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the elastic IP (EIP) address.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticIp</i> The elastic IP (EIP) address for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowCancelResize</i> A boolean value indicating if the resize operation can be cancelled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResizeType</i> Returns the value <code>ClassicResize</code>.<br/>        """
        pass

    def modify_cluster_db_revision(self, RevisionTarget: str, ClusterIdentifier: str) -> ModifyClusterDbRevisionResult:
        """Modifies the database revision of a cluster. The database revision is a unique revision of the database running in a cluster.

        :param self:
        :param RevisionTarget:  The identifier of the database revision. You can retrieve this value from the response to the <a>DescribeClusterDbRevisions</a> request.<br/>ClusterIdentifier:  The unique identifier of a cluster whose database revision you want to modify.  Example: <code>examplecluster</code> <br/>
        :return: Cluster:  Describes a cluster.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResizeInfo</i> Returns the following: <ul> <li> AllowCancelResize: a boolean value indicating if the resize operation can be cancelled. </li> <li> ResizeType: Returns ClassicResize </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResizeInfo</b>: Returns the following: <ul> <li> AllowCancelResize: a boolean value indicating if the resize operation can be cancelled. </li> <li> ResizeType: Returns ClassicResize </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotScheduleState</i> The current state of the cluster snapshot schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotScheduleIdentifier</i> A unique identifier for the cluster snapshot schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeferredMaintenanceWindows</i> Describes a group of <code>DeferredMaintenanceWindow</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticResizeNumberOfNodeOptions</i> The number of nodes that you can resize the cluster to with the elastic resize method. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingActions</i> Cluster operations that are waiting to be started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoles</i> A list of AWS Identity and Access Management (IAM) roles that can be used by the cluster to access other AWS services.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS Key Management Service (AWS KMS) key ID of the encryption key used to encrypt data in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The list of tags for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterRevisionNumber</i> The specific revision number of the database in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticIpStatus</i> The status of the elastic IP (EIP) address.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ElasticIpStatus</b>: The status of the elastic IP (EIP) address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterNodes</i> The nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterPublicKey</i> The public key for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSnapshotCopyStatus</i> A value that returns the destination region and retention period that are configured for cross-region snapshot copy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ClusterSnapshotCopyStatus</b>: A value that returns the destination region and retention period that are configured for cross-region snapshot copy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmStatus</i> A value that reports whether the Amazon Redshift cluster has finished applying any hardware security module (HSM) settings changes specified in a modify cluster command. Values: active, applying&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>HsmStatus</b>: A value that reports whether the Amazon Redshift cluster has finished applying any hardware security module (HSM) settings changes specified in a modify cluster command. Values: active, applying<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTransferProgress</i> <p/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DataTransferProgress</b>: <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreStatus</i> A value that describes the status of a cluster restore action. This parameter returns null if the cluster was not created by restoring a snapshot.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RestoreStatus</b>: A value that describes the status of a cluster restore action. This parameter returns null if the cluster was not created by restoring a snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encrypted</i> A boolean value that, if <code>true</code>, indicates that data in the cluster is encrypted at rest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> A boolean value that, if <code>true</code>, indicates that the cluster can be accessed from a public network.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The number of compute nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowVersionUpgrade</i> A boolean value that, if <code>true</code>, indicates that major version upgrades will be applied automatically to the cluster during the maintenance window. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The version ID of the Amazon Redshift engine that is running on the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> A value that, if present, indicates that changes to the cluster are pending. Specific pending changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: A value that, if present, indicates that changes to the cluster are pending. Specific pending changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> The weekly time range, in Universal Coordinated Time (UTC), during which system maintenance can occur.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The name of the Availability Zone in which the cluster is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The identifier of the VPC the cluster is in, if the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSubnetGroupName</i> The name of the subnet group that is associated with the cluster. This parameter is valid only when the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterParameterGroups</i> The list of cluster parameter groups that are associated with this cluster. Each parameter group in the list is returned with its status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> A list of Amazon Virtual Private Cloud (Amazon VPC) security groups that are associated with the cluster. This parameter is returned only if the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSecurityGroups</i> A list of cluster security group that are associated with the cluster. Each security group is represented by an element that contains <code>ClusterSecurityGroup.Name</code> and <code>ClusterSecurityGroup.Status</code> subelements.  Cluster security groups are used when the cluster is not created in an Amazon Virtual Private Cloud (VPC). Clusters that are created in a VPC use VPC security groups, which are listed by the <b>VpcSecurityGroups</b> parameter. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The default number of days to retain a manual snapshot. If the value is -1, the snapshot is retained indefinitely. This setting doesn't change the retention period of existing snapshots. The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotRetentionPeriod</i> The number of days that automatic cluster snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> The date and time that the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> The connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: The connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The name of the initial database that was created when the cluster was created. This same name is returned for the life of the cluster. If an initial database was not specified, a database named <code>dev</code>dev was created by default. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> The master user name for the cluster. This name is used to connect to the database that is specified in the <b>DBName</b> parameter. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModifyStatus</i> The status of a modify operation, if any, initiated for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterAvailabilityStatus</i> The availability status of the cluster for queries. Possible values are the following: <ul> <li> Available - The cluster is available for queries.  </li> <li> Unavailable - The cluster is not available for queries. </li> <li> Maintenance - The cluster is intermittently available for queries due to maintenance activities. </li> <li> Modifying - The cluster is intermittently available for queries due to changes that modify the cluster. </li> <li> Failed - The cluster failed and is not available for queries. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterStatus</i>  The current state of the cluster. Possible values are the following: <ul> <li>  <code>available</code>  </li> <li>  <code>available, prep-for-resize</code>  </li> <li>  <code>available, resize-cleanup</code>  </li> <li>  <code>cancelling-resize</code>  </li> <li>  <code>creating</code>  </li> <li>  <code>deleting</code>  </li> <li>  <code>final-snapshot</code>  </li> <li>  <code>hardware-failure</code>  </li> <li>  <code>incompatible-hsm</code>  </li> <li>  <code>incompatible-network</code>  </li> <li>  <code>incompatible-parameters</code>  </li> <li>  <code>incompatible-restore</code>  </li> <li>  <code>modifying</code>  </li> <li>  <code>rebooting</code>  </li> <li>  <code>renaming</code>  </li> <li>  <code>resizing</code>  </li> <li>  <code>rotating-keys</code>  </li> <li>  <code>storage-full</code>  </li> <li>  <code>updating-hsm</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The node type for the nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The unique identifier of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS address of the Cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionType</i> The encryption type for a cluster. Possible values are: KMS and None. For the China region the possible values are None, and Legacy. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track that the cluster will change to during the next maintenance window.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> The pending or in-progress change of the ability to connect to the cluster from the public network.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The pending or in-progress change of the new identifier for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotRetentionPeriod</i> The pending or in-progress change of the automated snapshot retention period.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The pending or in-progress change of the service version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterType</i> The pending or in-progress change of the cluster type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The pending or in-progress change of the number of nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The pending or in-progress change of the cluster's node type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> The pending or in-progress change of the master user password for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedTimeToCompletionInSeconds</i> The estimate of the time remaining before the restore will complete. Returns 0 for a completed restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> The amount of time an in-progress restore has been running, or the amount of time it took a completed restore to finish.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ProgressInMegaBytes</i> The number of megabytes that have been transferred from snapshot storage.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotSizeInMegaBytes</i> The size of the set of snapshot data used to restore the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentRestoreRateInMegaBytesPerSecond</i> The number of megabytes per second being transferred from the backup storage. Returns the average rate for a completed backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the restore action. Returns starting, restoring, completed, or failed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> Describes the number of seconds that have elapsed during the data transfer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedTimeToCompletionInSeconds</i> Describes the estimated number of seconds remaining to complete the transfer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTransferredInMegaBytes</i> Describes the total amount of data that has been transfered in MB's.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalDataInMegaBytes</i> Describes the total amount of data to be transfered in megabytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentRateInMegaBytesPerSecond</i> Describes the data transfer rate in MB's per second.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Describes the status of the cluster. While the transfer is in progress the status is <code>transferringdata</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Reports whether the Amazon Redshift cluster has finished applying any HSM settings changes specified in a modify cluster command. Values: active, applying<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmConfigurationIdentifier</i> Specifies the name of the HSM configuration that contains the information the Amazon Redshift cluster can use to retrieve and store keys in an HSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmClientCertificateIdentifier</i> Specifies the name of the HSM client certificate the Amazon Redshift cluster uses to retrieve the data encryption keys stored in an HSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCopyGrantName</i> The name of the snapshot copy grant.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The number of days that automated snapshots are retained in the destination region after they are copied from a source region. If the value is -1, the manual snapshot is retained indefinitely.  The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RetentionPeriod</i> The number of days that automated snapshots are retained in the destination region after they are copied from a source region.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationRegion</i> The destination region that snapshots are automatically copied to when cross-region snapshot copy is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the elastic IP (EIP) address.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticIp</i> The elastic IP (EIP) address for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowCancelResize</i> A boolean value indicating if the resize operation can be cancelled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResizeType</i> Returns the value <code>ClassicResize</code>.<br/>        """
        pass

    def modify_cluster_iam_roles(self, ClusterIdentifier: str, AddIamRoles: List[String]=None, RemoveIamRoles: List[String]=None) -> ModifyClusterIamRolesResult:
        """Modifies the list of AWS Identity and Access Management (IAM) roles that can be used by the cluster to access other AWS services. A cluster can have up to 10 IAM roles associated at any time.

        :param self:
        :param RemoveIamRoles:  Zero or more IAM roles in ARN format to disassociate from the cluster. You can disassociate up to 10 IAM roles from a single cluster in a single request.<br/>AddIamRoles:  Zero or more IAM roles to associate with the cluster. The roles must be in their Amazon Resource Name (ARN) format. You can associate up to 10 IAM roles with a single cluster in a single request.<br/>ClusterIdentifier:  The unique identifier of the cluster for which you want to associate or disassociate IAM roles.<br/>
        :return: Cluster:  Describes a cluster.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResizeInfo</i> Returns the following: <ul> <li> AllowCancelResize: a boolean value indicating if the resize operation can be cancelled. </li> <li> ResizeType: Returns ClassicResize </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResizeInfo</b>: Returns the following: <ul> <li> AllowCancelResize: a boolean value indicating if the resize operation can be cancelled. </li> <li> ResizeType: Returns ClassicResize </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotScheduleState</i> The current state of the cluster snapshot schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotScheduleIdentifier</i> A unique identifier for the cluster snapshot schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeferredMaintenanceWindows</i> Describes a group of <code>DeferredMaintenanceWindow</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticResizeNumberOfNodeOptions</i> The number of nodes that you can resize the cluster to with the elastic resize method. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingActions</i> Cluster operations that are waiting to be started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoles</i> A list of AWS Identity and Access Management (IAM) roles that can be used by the cluster to access other AWS services.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS Key Management Service (AWS KMS) key ID of the encryption key used to encrypt data in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The list of tags for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterRevisionNumber</i> The specific revision number of the database in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticIpStatus</i> The status of the elastic IP (EIP) address.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ElasticIpStatus</b>: The status of the elastic IP (EIP) address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterNodes</i> The nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterPublicKey</i> The public key for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSnapshotCopyStatus</i> A value that returns the destination region and retention period that are configured for cross-region snapshot copy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ClusterSnapshotCopyStatus</b>: A value that returns the destination region and retention period that are configured for cross-region snapshot copy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmStatus</i> A value that reports whether the Amazon Redshift cluster has finished applying any hardware security module (HSM) settings changes specified in a modify cluster command. Values: active, applying&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>HsmStatus</b>: A value that reports whether the Amazon Redshift cluster has finished applying any hardware security module (HSM) settings changes specified in a modify cluster command. Values: active, applying<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTransferProgress</i> <p/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DataTransferProgress</b>: <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreStatus</i> A value that describes the status of a cluster restore action. This parameter returns null if the cluster was not created by restoring a snapshot.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RestoreStatus</b>: A value that describes the status of a cluster restore action. This parameter returns null if the cluster was not created by restoring a snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encrypted</i> A boolean value that, if <code>true</code>, indicates that data in the cluster is encrypted at rest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> A boolean value that, if <code>true</code>, indicates that the cluster can be accessed from a public network.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The number of compute nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowVersionUpgrade</i> A boolean value that, if <code>true</code>, indicates that major version upgrades will be applied automatically to the cluster during the maintenance window. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The version ID of the Amazon Redshift engine that is running on the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> A value that, if present, indicates that changes to the cluster are pending. Specific pending changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: A value that, if present, indicates that changes to the cluster are pending. Specific pending changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> The weekly time range, in Universal Coordinated Time (UTC), during which system maintenance can occur.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The name of the Availability Zone in which the cluster is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The identifier of the VPC the cluster is in, if the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSubnetGroupName</i> The name of the subnet group that is associated with the cluster. This parameter is valid only when the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterParameterGroups</i> The list of cluster parameter groups that are associated with this cluster. Each parameter group in the list is returned with its status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> A list of Amazon Virtual Private Cloud (Amazon VPC) security groups that are associated with the cluster. This parameter is returned only if the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSecurityGroups</i> A list of cluster security group that are associated with the cluster. Each security group is represented by an element that contains <code>ClusterSecurityGroup.Name</code> and <code>ClusterSecurityGroup.Status</code> subelements.  Cluster security groups are used when the cluster is not created in an Amazon Virtual Private Cloud (VPC). Clusters that are created in a VPC use VPC security groups, which are listed by the <b>VpcSecurityGroups</b> parameter. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The default number of days to retain a manual snapshot. If the value is -1, the snapshot is retained indefinitely. This setting doesn't change the retention period of existing snapshots. The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotRetentionPeriod</i> The number of days that automatic cluster snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> The date and time that the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> The connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: The connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The name of the initial database that was created when the cluster was created. This same name is returned for the life of the cluster. If an initial database was not specified, a database named <code>dev</code>dev was created by default. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> The master user name for the cluster. This name is used to connect to the database that is specified in the <b>DBName</b> parameter. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModifyStatus</i> The status of a modify operation, if any, initiated for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterAvailabilityStatus</i> The availability status of the cluster for queries. Possible values are the following: <ul> <li> Available - The cluster is available for queries.  </li> <li> Unavailable - The cluster is not available for queries. </li> <li> Maintenance - The cluster is intermittently available for queries due to maintenance activities. </li> <li> Modifying - The cluster is intermittently available for queries due to changes that modify the cluster. </li> <li> Failed - The cluster failed and is not available for queries. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterStatus</i>  The current state of the cluster. Possible values are the following: <ul> <li>  <code>available</code>  </li> <li>  <code>available, prep-for-resize</code>  </li> <li>  <code>available, resize-cleanup</code>  </li> <li>  <code>cancelling-resize</code>  </li> <li>  <code>creating</code>  </li> <li>  <code>deleting</code>  </li> <li>  <code>final-snapshot</code>  </li> <li>  <code>hardware-failure</code>  </li> <li>  <code>incompatible-hsm</code>  </li> <li>  <code>incompatible-network</code>  </li> <li>  <code>incompatible-parameters</code>  </li> <li>  <code>incompatible-restore</code>  </li> <li>  <code>modifying</code>  </li> <li>  <code>rebooting</code>  </li> <li>  <code>renaming</code>  </li> <li>  <code>resizing</code>  </li> <li>  <code>rotating-keys</code>  </li> <li>  <code>storage-full</code>  </li> <li>  <code>updating-hsm</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The node type for the nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The unique identifier of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS address of the Cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionType</i> The encryption type for a cluster. Possible values are: KMS and None. For the China region the possible values are None, and Legacy. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track that the cluster will change to during the next maintenance window.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> The pending or in-progress change of the ability to connect to the cluster from the public network.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The pending or in-progress change of the new identifier for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotRetentionPeriod</i> The pending or in-progress change of the automated snapshot retention period.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The pending or in-progress change of the service version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterType</i> The pending or in-progress change of the cluster type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The pending or in-progress change of the number of nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The pending or in-progress change of the cluster's node type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> The pending or in-progress change of the master user password for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedTimeToCompletionInSeconds</i> The estimate of the time remaining before the restore will complete. Returns 0 for a completed restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> The amount of time an in-progress restore has been running, or the amount of time it took a completed restore to finish.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ProgressInMegaBytes</i> The number of megabytes that have been transferred from snapshot storage.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotSizeInMegaBytes</i> The size of the set of snapshot data used to restore the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentRestoreRateInMegaBytesPerSecond</i> The number of megabytes per second being transferred from the backup storage. Returns the average rate for a completed backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the restore action. Returns starting, restoring, completed, or failed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> Describes the number of seconds that have elapsed during the data transfer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedTimeToCompletionInSeconds</i> Describes the estimated number of seconds remaining to complete the transfer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTransferredInMegaBytes</i> Describes the total amount of data that has been transfered in MB's.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalDataInMegaBytes</i> Describes the total amount of data to be transfered in megabytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentRateInMegaBytesPerSecond</i> Describes the data transfer rate in MB's per second.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Describes the status of the cluster. While the transfer is in progress the status is <code>transferringdata</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Reports whether the Amazon Redshift cluster has finished applying any HSM settings changes specified in a modify cluster command. Values: active, applying<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmConfigurationIdentifier</i> Specifies the name of the HSM configuration that contains the information the Amazon Redshift cluster can use to retrieve and store keys in an HSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmClientCertificateIdentifier</i> Specifies the name of the HSM client certificate the Amazon Redshift cluster uses to retrieve the data encryption keys stored in an HSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCopyGrantName</i> The name of the snapshot copy grant.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The number of days that automated snapshots are retained in the destination region after they are copied from a source region. If the value is -1, the manual snapshot is retained indefinitely.  The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RetentionPeriod</i> The number of days that automated snapshots are retained in the destination region after they are copied from a source region.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationRegion</i> The destination region that snapshots are automatically copied to when cross-region snapshot copy is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the elastic IP (EIP) address.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticIp</i> The elastic IP (EIP) address for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowCancelResize</i> A boolean value indicating if the resize operation can be cancelled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResizeType</i> Returns the value <code>ClassicResize</code>.<br/>        """
        pass

    def modify_cluster_maintenance(self, ClusterIdentifier: str, DeferMaintenance: BooleanOptional=None, DeferMaintenanceIdentifier: str=None, DeferMaintenanceStartTime: TStamp=None, DeferMaintenanceEndTime: TStamp=None, DeferMaintenanceDuration: int=None) -> ModifyClusterMaintenanceResult:
        """Modifies the maintenance settings of a cluster. For example, you can defer a maintenance window. You can also update or cancel a deferment. 

        :param self:
        :param DeferMaintenanceDuration:  An integer indicating the duration of the maintenance window in days. If you specify a duration, you can't specify an end time. The duration must be 45 days or less.<br/>DeferMaintenanceEndTime:  A timestamp indicating end time for the deferred maintenance window. If you specify an end time, you can't specify a duration.<br/>DeferMaintenanceStartTime:  A timestamp indicating the start time for the deferred maintenance window.<br/>DeferMaintenanceIdentifier:  A unique identifier for the deferred maintenance window.<br/>DeferMaintenance:  A boolean indicating whether to enable the deferred maintenance window. <br/>ClusterIdentifier:  A unique identifier for the cluster.<br/>
        :return: Cluster:  Describes a cluster.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResizeInfo</i> Returns the following: <ul> <li> AllowCancelResize: a boolean value indicating if the resize operation can be cancelled. </li> <li> ResizeType: Returns ClassicResize </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResizeInfo</b>: Returns the following: <ul> <li> AllowCancelResize: a boolean value indicating if the resize operation can be cancelled. </li> <li> ResizeType: Returns ClassicResize </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotScheduleState</i> The current state of the cluster snapshot schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotScheduleIdentifier</i> A unique identifier for the cluster snapshot schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeferredMaintenanceWindows</i> Describes a group of <code>DeferredMaintenanceWindow</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticResizeNumberOfNodeOptions</i> The number of nodes that you can resize the cluster to with the elastic resize method. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingActions</i> Cluster operations that are waiting to be started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoles</i> A list of AWS Identity and Access Management (IAM) roles that can be used by the cluster to access other AWS services.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS Key Management Service (AWS KMS) key ID of the encryption key used to encrypt data in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The list of tags for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterRevisionNumber</i> The specific revision number of the database in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticIpStatus</i> The status of the elastic IP (EIP) address.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ElasticIpStatus</b>: The status of the elastic IP (EIP) address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterNodes</i> The nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterPublicKey</i> The public key for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSnapshotCopyStatus</i> A value that returns the destination region and retention period that are configured for cross-region snapshot copy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ClusterSnapshotCopyStatus</b>: A value that returns the destination region and retention period that are configured for cross-region snapshot copy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmStatus</i> A value that reports whether the Amazon Redshift cluster has finished applying any hardware security module (HSM) settings changes specified in a modify cluster command. Values: active, applying&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>HsmStatus</b>: A value that reports whether the Amazon Redshift cluster has finished applying any hardware security module (HSM) settings changes specified in a modify cluster command. Values: active, applying<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTransferProgress</i> <p/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DataTransferProgress</b>: <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreStatus</i> A value that describes the status of a cluster restore action. This parameter returns null if the cluster was not created by restoring a snapshot.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RestoreStatus</b>: A value that describes the status of a cluster restore action. This parameter returns null if the cluster was not created by restoring a snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encrypted</i> A boolean value that, if <code>true</code>, indicates that data in the cluster is encrypted at rest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> A boolean value that, if <code>true</code>, indicates that the cluster can be accessed from a public network.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The number of compute nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowVersionUpgrade</i> A boolean value that, if <code>true</code>, indicates that major version upgrades will be applied automatically to the cluster during the maintenance window. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The version ID of the Amazon Redshift engine that is running on the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> A value that, if present, indicates that changes to the cluster are pending. Specific pending changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: A value that, if present, indicates that changes to the cluster are pending. Specific pending changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> The weekly time range, in Universal Coordinated Time (UTC), during which system maintenance can occur.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The name of the Availability Zone in which the cluster is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The identifier of the VPC the cluster is in, if the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSubnetGroupName</i> The name of the subnet group that is associated with the cluster. This parameter is valid only when the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterParameterGroups</i> The list of cluster parameter groups that are associated with this cluster. Each parameter group in the list is returned with its status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> A list of Amazon Virtual Private Cloud (Amazon VPC) security groups that are associated with the cluster. This parameter is returned only if the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSecurityGroups</i> A list of cluster security group that are associated with the cluster. Each security group is represented by an element that contains <code>ClusterSecurityGroup.Name</code> and <code>ClusterSecurityGroup.Status</code> subelements.  Cluster security groups are used when the cluster is not created in an Amazon Virtual Private Cloud (VPC). Clusters that are created in a VPC use VPC security groups, which are listed by the <b>VpcSecurityGroups</b> parameter. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The default number of days to retain a manual snapshot. If the value is -1, the snapshot is retained indefinitely. This setting doesn't change the retention period of existing snapshots. The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotRetentionPeriod</i> The number of days that automatic cluster snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> The date and time that the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> The connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: The connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The name of the initial database that was created when the cluster was created. This same name is returned for the life of the cluster. If an initial database was not specified, a database named <code>dev</code>dev was created by default. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> The master user name for the cluster. This name is used to connect to the database that is specified in the <b>DBName</b> parameter. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModifyStatus</i> The status of a modify operation, if any, initiated for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterAvailabilityStatus</i> The availability status of the cluster for queries. Possible values are the following: <ul> <li> Available - The cluster is available for queries.  </li> <li> Unavailable - The cluster is not available for queries. </li> <li> Maintenance - The cluster is intermittently available for queries due to maintenance activities. </li> <li> Modifying - The cluster is intermittently available for queries due to changes that modify the cluster. </li> <li> Failed - The cluster failed and is not available for queries. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterStatus</i>  The current state of the cluster. Possible values are the following: <ul> <li>  <code>available</code>  </li> <li>  <code>available, prep-for-resize</code>  </li> <li>  <code>available, resize-cleanup</code>  </li> <li>  <code>cancelling-resize</code>  </li> <li>  <code>creating</code>  </li> <li>  <code>deleting</code>  </li> <li>  <code>final-snapshot</code>  </li> <li>  <code>hardware-failure</code>  </li> <li>  <code>incompatible-hsm</code>  </li> <li>  <code>incompatible-network</code>  </li> <li>  <code>incompatible-parameters</code>  </li> <li>  <code>incompatible-restore</code>  </li> <li>  <code>modifying</code>  </li> <li>  <code>rebooting</code>  </li> <li>  <code>renaming</code>  </li> <li>  <code>resizing</code>  </li> <li>  <code>rotating-keys</code>  </li> <li>  <code>storage-full</code>  </li> <li>  <code>updating-hsm</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The node type for the nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The unique identifier of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS address of the Cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionType</i> The encryption type for a cluster. Possible values are: KMS and None. For the China region the possible values are None, and Legacy. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track that the cluster will change to during the next maintenance window.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> The pending or in-progress change of the ability to connect to the cluster from the public network.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The pending or in-progress change of the new identifier for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotRetentionPeriod</i> The pending or in-progress change of the automated snapshot retention period.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The pending or in-progress change of the service version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterType</i> The pending or in-progress change of the cluster type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The pending or in-progress change of the number of nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The pending or in-progress change of the cluster's node type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> The pending or in-progress change of the master user password for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedTimeToCompletionInSeconds</i> The estimate of the time remaining before the restore will complete. Returns 0 for a completed restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> The amount of time an in-progress restore has been running, or the amount of time it took a completed restore to finish.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ProgressInMegaBytes</i> The number of megabytes that have been transferred from snapshot storage.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotSizeInMegaBytes</i> The size of the set of snapshot data used to restore the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentRestoreRateInMegaBytesPerSecond</i> The number of megabytes per second being transferred from the backup storage. Returns the average rate for a completed backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the restore action. Returns starting, restoring, completed, or failed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> Describes the number of seconds that have elapsed during the data transfer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedTimeToCompletionInSeconds</i> Describes the estimated number of seconds remaining to complete the transfer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTransferredInMegaBytes</i> Describes the total amount of data that has been transfered in MB's.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalDataInMegaBytes</i> Describes the total amount of data to be transfered in megabytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentRateInMegaBytesPerSecond</i> Describes the data transfer rate in MB's per second.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Describes the status of the cluster. While the transfer is in progress the status is <code>transferringdata</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Reports whether the Amazon Redshift cluster has finished applying any HSM settings changes specified in a modify cluster command. Values: active, applying<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmConfigurationIdentifier</i> Specifies the name of the HSM configuration that contains the information the Amazon Redshift cluster can use to retrieve and store keys in an HSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmClientCertificateIdentifier</i> Specifies the name of the HSM client certificate the Amazon Redshift cluster uses to retrieve the data encryption keys stored in an HSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCopyGrantName</i> The name of the snapshot copy grant.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The number of days that automated snapshots are retained in the destination region after they are copied from a source region. If the value is -1, the manual snapshot is retained indefinitely.  The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RetentionPeriod</i> The number of days that automated snapshots are retained in the destination region after they are copied from a source region.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationRegion</i> The destination region that snapshots are automatically copied to when cross-region snapshot copy is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the elastic IP (EIP) address.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticIp</i> The elastic IP (EIP) address for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowCancelResize</i> A boolean value indicating if the resize operation can be cancelled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResizeType</i> Returns the value <code>ClassicResize</code>.<br/>        """
        pass

    def modify_cluster_parameter_group(self, Parameters: List[Parameter], ParameterGroupName: str) -> ClusterParameterGroupNameMessage:
        """Modifies the parameters of a parameter group.  For more information about parameters and parameter groups, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-parameter-groups.html">Amazon Redshift Parameter Groups</a> in the <i>Amazon Redshift Cluster Management Guide</i>.

        :param self:
        :param Parameters:  An array of parameters to be modified. A maximum of 20 parameters can be modified in a single request. For each parameter to be modified, you must supply at least the parameter name and parameter value; other name-value pairs of the parameter are optional. For the workload management (WLM) configuration, you must supply all the name-value pairs in the wlm_json_configuration parameter.<br/>ParameterGroupName:  The name of the parameter group to be modified.<br/>
        :return: ParameterGroupStatus:  The status of the parameter group. For example, if you made a change to a parameter group name-value pair, then the change could be pending a reboot of an associated cluster.<br/>ParameterGroupName:  The name of the cluster parameter group.<br/>        """
        pass

    def modify_cluster_snapshot(self, SnapshotIdentifier: str, ManualSnapshotRetentionPeriod: int=None, Force: Boolean=None) -> ModifyClusterSnapshotResult:
        """Modifies the settings for a snapshot.

        :param self:
        :param Force:  A Boolean option to override an exception if the retention period has already passed.<br/>ManualSnapshotRetentionPeriod:  The number of days that a manual snapshot is retained. If the value is -1, the manual snapshot is retained indefinitely. If the manual snapshot falls outside of the new retention period, you can specify the force option to immediately delete the snapshot. The value must be either -1 or an integer between 1 and 3,653.<br/>SnapshotIdentifier:  The identifier of the snapshot whose setting you want to modify.<br/>
        :return: Snapshot:  Describes a snapshot.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotRetentionStartTime</i> A timestamp representing the start of the retention period for the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRemainingDays</i> The number of days until a manual snapshot will pass its retention period.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The number of days that a manual snapshot is retained. If the value is -1, the manual snapshot is retained indefinitely.  The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track for the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestorableNodeTypes</i> The list of node types that this cluster snapshot is able to restore into.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The list of tags for the cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceRegion</i> The source region from which the snapshot was copied.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> The amount of time an in-progress snapshot backup has been running, or the amount of time it took a completed backup to finish.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedSecondsToCompletion</i> The estimate of the time remaining before the snapshot backup will complete. Returns <code>0</code> for a completed backup. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentBackupRateInMegaBytesPerSecond</i> The number of megabytes per second being transferred to the snapshot backup. Returns <code>0</code> for a completed backup. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupProgressInMegaBytes</i> The number of megabytes that have been transferred to the snapshot backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActualIncrementalBackupSizeInMegaBytes</i> The size of the incremental backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalBackupSizeInMegaBytes</i> The size of the complete set of backup data that would be used to restore the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerAccount</i> For manual snapshots, the AWS customer account used to create or copy the snapshot. For automatic snapshots, the owner of the cluster. The owner can perform all snapshot actions, such as sharing a manual snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountsWithRestoreAccess</i> A list of the AWS customer accounts authorized to restore the snapshot. Returns <code>null</code> if no accounts are authorized. Visible only to the snapshot owner. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptedWithHSM</i> A boolean that indicates whether the snapshot data is encrypted using the HSM keys of the source cluster. <code>true</code> indicates that the data is encrypted using HSM keys.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS Key Management Service (KMS) key ID of the encryption key that was used to encrypt data in the cluster from which the snapshot was taken.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encrypted</i> If <code>true</code>, the data in the snapshot is encrypted at rest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The VPC identifier of the cluster if the snapshot is from a cluster in a VPC. Otherwise, this field is not in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The name of the database that was created when the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The number of nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The node type of the nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotType</i> The snapshot type. Snapshots created using <a>CreateClusterSnapshot</a> and <a>CopyClusterSnapshot</a> are of type "manual". <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The version ID of the Amazon Redshift engine that is running on the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> The master user name for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> The time (UTC) when the cluster was originally created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone in which the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port that the cluster is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The snapshot status. The value of the status depends on the API operation used:  <ul> <li>  <a>CreateClusterSnapshot</a> and <a>CopyClusterSnapshot</a> returns status as "creating".  </li> <li>  <a>DescribeClusterSnapshots</a> returns status as "creating", "available", "final snapshot", or "failed". </li> <li>  <a>DeleteClusterSnapshot</a> returns status as "deleted". </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCreateTime</i> The time (in UTC format) when Amazon Redshift began the snapshot. A snapshot contains a copy of the cluster data as of this exact time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The identifier of the cluster for which the snapshot was taken.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotIdentifier</i> The snapshot identifier that is provided in the request.<br/>        """
        pass

    def modify_cluster_snapshot_schedule(self, ClusterIdentifier: str, ScheduleIdentifier: str=None, DisassociateSchedule: BooleanOptional=None):
        """Modifies a snapshot schedule for a cluster.

        :param self:
        :param DisassociateSchedule:  A boolean to indicate whether to remove the assoiciation between the cluster and the schedule.<br/>ScheduleIdentifier:  A unique alphanumeric identifier for the schedule that you want to associate with the cluster.<br/>ClusterIdentifier:  A unique identifier for the cluster whose snapshot schedule you want to modify. <br/>
        :return:         """
        pass

    def modify_cluster_subnet_group(self, SubnetIds: List[String], ClusterSubnetGroupName: str, Description: str=None) -> ModifyClusterSubnetGroupResult:
        """Modifies a cluster subnet group to include the specified list of VPC subnets. The operation replaces the existing list of subnets with the new list of subnets.

        :param self:
        :param SubnetIds:  An array of VPC subnet IDs. A maximum of 20 subnets can be modified in a single request.<br/>Description:  A text description of the subnet group to be modified.<br/>ClusterSubnetGroupName:  The name of the subnet group to be modified.<br/>
        :return: ClusterSubnetGroup:  Describes a subnet group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The list of tags for the cluster subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> A list of the VPC <a>Subnet</a> elements. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetGroupStatus</i> The status of the cluster subnet group. Possible values are <code>Complete</code>, <code>Incomplete</code> and <code>Invalid</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The VPC ID of the cluster subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the cluster subnet group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSubnetGroupName</i> The name of the cluster subnet group.<br/>        """
        pass

    def modify_event_subscription(self, SubscriptionName: str, SnsTopicArn: str=None, SourceType: str=None, SourceIds: List[String]=None, EventCategories: List[String]=None, Severity: str=None, Enabled: BooleanOptional=None) -> ModifyEventSubscriptionResult:
        """Modifies an existing Amazon Redshift event notification subscription.

        :param self:
        :param Enabled:  A Boolean value indicating if the subscription is enabled. <code>true</code> indicates the subscription is enabled <br/>Severity:  Specifies the Amazon Redshift event severity to be published by the event notification subscription. Values: ERROR, INFO<br/>EventCategories:  Specifies the Amazon Redshift event categories to be published by the event notification subscription. Values: configuration, management, monitoring, security<br/>SourceIds:  A list of one or more identifiers of Amazon Redshift source objects. All of the objects must be of the same type as was specified in the source type parameter. The event subscription will return only events generated by the specified objects. If not specified, then events are returned for all objects within the source type specified. Example: my-cluster-1, my-cluster-2 Example: my-snapshot-20131010<br/>SourceType:  The type of source that will be generating the events. For example, if you want to be notified of events generated by a cluster, you would set this parameter to cluster. If this value is not specified, events are returned for all Amazon Redshift objects in your AWS account. You must specify a source type in order to specify source IDs. Valid values: cluster, cluster-parameter-group, cluster-security-group, and cluster-snapshot.<br/>SnsTopicArn:  The Amazon Resource Name (ARN) of the SNS topic to be used by the event notification subscription.<br/>SubscriptionName:  The name of the modified Amazon Redshift event notification subscription.<br/>
        :return: EventSubscription:  Describes event subscriptions.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The list of tags for the event subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> A boolean value indicating whether the subscription is enabled; <code>true</code> indicates that the subscription is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Severity</i> The event severity specified in the Amazon Redshift event notification subscription. Values: ERROR, INFO<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EventCategoriesList</i> The list of Amazon Redshift event categories specified in the event notification subscription. Values: Configuration, Management, Monitoring, Security<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceIdsList</i> A list of the sources that publish events to the Amazon Redshift event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceType</i> The source type of the events returned the Amazon Redshift event notification, such as cluster, or cluster-snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubscriptionCreationTime</i> The date and time the Amazon Redshift event notification subscription was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the Amazon Redshift event notification subscription. Constraints: <ul> <li> Can be one of the following: active | no-permission | topic-not-exist </li> <li> The status "no-permission" indicates that Amazon Redshift no longer has permission to post to the Amazon SNS topic. The status "topic-not-exist" indicates that the topic was deleted after the subscription was created. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsTopicArn</i> The Amazon Resource Name (ARN) of the Amazon SNS topic used by the event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustSubscriptionId</i> The name of the Amazon Redshift event notification subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomerAwsId</i> The AWS customer account associated with the Amazon Redshift event notification subscription.<br/>        """
        pass

    def modify_snapshot_copy_retention_period(self, RetentionPeriod: int, ClusterIdentifier: str, Manual: Boolean=None) -> ModifySnapshotCopyRetentionPeriodResult:
        """Modifies the number of days to retain snapshots in the destination AWS Region after they are copied from the source AWS Region. By default, this operation only changes the retention period of copied automated snapshots. The retention periods for both new and existing copied automated snapshots are updated with the new retention period. You can set the manual option to change only the retention periods of copied manual snapshots. If you set this option, only newly copied manual snapshots have the new retention period. 

        :param self:
        :param Manual:  Indicates whether to apply the snapshot retention period to newly copied manual snapshots instead of automated snapshots.<br/>RetentionPeriod:  The number of days to retain automated snapshots in the destination AWS Region after they are copied from the source AWS Region. By default, this only changes the retention period of copied automated snapshots.  If you decrease the retention period for automated snapshots that are copied to a destination AWS Region, Amazon Redshift deletes any existing automated snapshots that were copied to the destination AWS Region and that fall outside of the new retention period. Constraints: Must be at least 1 and no more than 35 for automated snapshots.  If you specify the <code>manual</code> option, only newly copied manual snapshots will have the new retention period.  If you specify the value of -1 newly copied manual snapshots are retained indefinitely. Constraints: The number of days must be either -1 or an integer between 1 and 3,653 for manual snapshots.<br/>ClusterIdentifier:  The unique identifier of the cluster for which you want to change the retention period for either automated or manual snapshots that are copied to a destination AWS Region. Constraints: Must be the valid name of an existing cluster that has cross-region snapshot copy enabled.<br/>
        :return: Cluster:  Describes a cluster.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResizeInfo</i> Returns the following: <ul> <li> AllowCancelResize: a boolean value indicating if the resize operation can be cancelled. </li> <li> ResizeType: Returns ClassicResize </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResizeInfo</b>: Returns the following: <ul> <li> AllowCancelResize: a boolean value indicating if the resize operation can be cancelled. </li> <li> ResizeType: Returns ClassicResize </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotScheduleState</i> The current state of the cluster snapshot schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotScheduleIdentifier</i> A unique identifier for the cluster snapshot schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeferredMaintenanceWindows</i> Describes a group of <code>DeferredMaintenanceWindow</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticResizeNumberOfNodeOptions</i> The number of nodes that you can resize the cluster to with the elastic resize method. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingActions</i> Cluster operations that are waiting to be started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoles</i> A list of AWS Identity and Access Management (IAM) roles that can be used by the cluster to access other AWS services.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS Key Management Service (AWS KMS) key ID of the encryption key used to encrypt data in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The list of tags for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterRevisionNumber</i> The specific revision number of the database in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticIpStatus</i> The status of the elastic IP (EIP) address.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ElasticIpStatus</b>: The status of the elastic IP (EIP) address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterNodes</i> The nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterPublicKey</i> The public key for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSnapshotCopyStatus</i> A value that returns the destination region and retention period that are configured for cross-region snapshot copy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ClusterSnapshotCopyStatus</b>: A value that returns the destination region and retention period that are configured for cross-region snapshot copy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmStatus</i> A value that reports whether the Amazon Redshift cluster has finished applying any hardware security module (HSM) settings changes specified in a modify cluster command. Values: active, applying&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>HsmStatus</b>: A value that reports whether the Amazon Redshift cluster has finished applying any hardware security module (HSM) settings changes specified in a modify cluster command. Values: active, applying<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTransferProgress</i> <p/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DataTransferProgress</b>: <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreStatus</i> A value that describes the status of a cluster restore action. This parameter returns null if the cluster was not created by restoring a snapshot.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RestoreStatus</b>: A value that describes the status of a cluster restore action. This parameter returns null if the cluster was not created by restoring a snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encrypted</i> A boolean value that, if <code>true</code>, indicates that data in the cluster is encrypted at rest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> A boolean value that, if <code>true</code>, indicates that the cluster can be accessed from a public network.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The number of compute nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowVersionUpgrade</i> A boolean value that, if <code>true</code>, indicates that major version upgrades will be applied automatically to the cluster during the maintenance window. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The version ID of the Amazon Redshift engine that is running on the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> A value that, if present, indicates that changes to the cluster are pending. Specific pending changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: A value that, if present, indicates that changes to the cluster are pending. Specific pending changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> The weekly time range, in Universal Coordinated Time (UTC), during which system maintenance can occur.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The name of the Availability Zone in which the cluster is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The identifier of the VPC the cluster is in, if the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSubnetGroupName</i> The name of the subnet group that is associated with the cluster. This parameter is valid only when the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterParameterGroups</i> The list of cluster parameter groups that are associated with this cluster. Each parameter group in the list is returned with its status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> A list of Amazon Virtual Private Cloud (Amazon VPC) security groups that are associated with the cluster. This parameter is returned only if the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSecurityGroups</i> A list of cluster security group that are associated with the cluster. Each security group is represented by an element that contains <code>ClusterSecurityGroup.Name</code> and <code>ClusterSecurityGroup.Status</code> subelements.  Cluster security groups are used when the cluster is not created in an Amazon Virtual Private Cloud (VPC). Clusters that are created in a VPC use VPC security groups, which are listed by the <b>VpcSecurityGroups</b> parameter. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The default number of days to retain a manual snapshot. If the value is -1, the snapshot is retained indefinitely. This setting doesn't change the retention period of existing snapshots. The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotRetentionPeriod</i> The number of days that automatic cluster snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> The date and time that the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> The connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: The connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The name of the initial database that was created when the cluster was created. This same name is returned for the life of the cluster. If an initial database was not specified, a database named <code>dev</code>dev was created by default. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> The master user name for the cluster. This name is used to connect to the database that is specified in the <b>DBName</b> parameter. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModifyStatus</i> The status of a modify operation, if any, initiated for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterAvailabilityStatus</i> The availability status of the cluster for queries. Possible values are the following: <ul> <li> Available - The cluster is available for queries.  </li> <li> Unavailable - The cluster is not available for queries. </li> <li> Maintenance - The cluster is intermittently available for queries due to maintenance activities. </li> <li> Modifying - The cluster is intermittently available for queries due to changes that modify the cluster. </li> <li> Failed - The cluster failed and is not available for queries. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterStatus</i>  The current state of the cluster. Possible values are the following: <ul> <li>  <code>available</code>  </li> <li>  <code>available, prep-for-resize</code>  </li> <li>  <code>available, resize-cleanup</code>  </li> <li>  <code>cancelling-resize</code>  </li> <li>  <code>creating</code>  </li> <li>  <code>deleting</code>  </li> <li>  <code>final-snapshot</code>  </li> <li>  <code>hardware-failure</code>  </li> <li>  <code>incompatible-hsm</code>  </li> <li>  <code>incompatible-network</code>  </li> <li>  <code>incompatible-parameters</code>  </li> <li>  <code>incompatible-restore</code>  </li> <li>  <code>modifying</code>  </li> <li>  <code>rebooting</code>  </li> <li>  <code>renaming</code>  </li> <li>  <code>resizing</code>  </li> <li>  <code>rotating-keys</code>  </li> <li>  <code>storage-full</code>  </li> <li>  <code>updating-hsm</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The node type for the nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The unique identifier of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS address of the Cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionType</i> The encryption type for a cluster. Possible values are: KMS and None. For the China region the possible values are None, and Legacy. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track that the cluster will change to during the next maintenance window.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> The pending or in-progress change of the ability to connect to the cluster from the public network.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The pending or in-progress change of the new identifier for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotRetentionPeriod</i> The pending or in-progress change of the automated snapshot retention period.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The pending or in-progress change of the service version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterType</i> The pending or in-progress change of the cluster type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The pending or in-progress change of the number of nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The pending or in-progress change of the cluster's node type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> The pending or in-progress change of the master user password for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedTimeToCompletionInSeconds</i> The estimate of the time remaining before the restore will complete. Returns 0 for a completed restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> The amount of time an in-progress restore has been running, or the amount of time it took a completed restore to finish.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ProgressInMegaBytes</i> The number of megabytes that have been transferred from snapshot storage.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotSizeInMegaBytes</i> The size of the set of snapshot data used to restore the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentRestoreRateInMegaBytesPerSecond</i> The number of megabytes per second being transferred from the backup storage. Returns the average rate for a completed backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the restore action. Returns starting, restoring, completed, or failed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> Describes the number of seconds that have elapsed during the data transfer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedTimeToCompletionInSeconds</i> Describes the estimated number of seconds remaining to complete the transfer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTransferredInMegaBytes</i> Describes the total amount of data that has been transfered in MB's.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalDataInMegaBytes</i> Describes the total amount of data to be transfered in megabytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentRateInMegaBytesPerSecond</i> Describes the data transfer rate in MB's per second.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Describes the status of the cluster. While the transfer is in progress the status is <code>transferringdata</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Reports whether the Amazon Redshift cluster has finished applying any HSM settings changes specified in a modify cluster command. Values: active, applying<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmConfigurationIdentifier</i> Specifies the name of the HSM configuration that contains the information the Amazon Redshift cluster can use to retrieve and store keys in an HSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmClientCertificateIdentifier</i> Specifies the name of the HSM client certificate the Amazon Redshift cluster uses to retrieve the data encryption keys stored in an HSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCopyGrantName</i> The name of the snapshot copy grant.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The number of days that automated snapshots are retained in the destination region after they are copied from a source region. If the value is -1, the manual snapshot is retained indefinitely.  The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RetentionPeriod</i> The number of days that automated snapshots are retained in the destination region after they are copied from a source region.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationRegion</i> The destination region that snapshots are automatically copied to when cross-region snapshot copy is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the elastic IP (EIP) address.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticIp</i> The elastic IP (EIP) address for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowCancelResize</i> A boolean value indicating if the resize operation can be cancelled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResizeType</i> Returns the value <code>ClassicResize</code>.<br/>        """
        pass

    def modify_snapshot_schedule(self, ScheduleDefinitions: List[String], ScheduleIdentifier: str) -> SnapshotSchedule:
        """Modifies a snapshot schedule. Any schedule associated with a cluster is modified asynchronously.

        :param self:
        :param ScheduleDefinitions:  An updated list of schedule definitions. A schedule definition is made up of schedule expressions, for example, "cron(30 12 *)" or "rate(12 hours)".<br/>ScheduleIdentifier:  A unique alphanumeric identifier of the schedule to modify.<br/>
        :return: AssociatedClusters:  A list of clusters associated with the schedule. A maximum of 100 clusters is returned.<br/>AssociatedClusterCount:  The number of clusters associated with the schedule.<br/>NextInvocations:  <p/><br/>Tags:  An optional set of tags describing the schedule.<br/>ScheduleDescription:  The description of the schedule.<br/>ScheduleIdentifier:  A unique identifier for the schedule.<br/>ScheduleDefinitions:  A list of ScheduleDefinitions.<br/>        """
        pass

    def purchase_reserved_node_offering(self, ReservedNodeOfferingId: str, NodeCount: int=None) -> PurchaseReservedNodeOfferingResult:
        """Allows you to purchase reserved nodes. Amazon Redshift offers a predefined set of reserved node offerings. You can purchase one or more of the offerings. You can call the <a>DescribeReservedNodeOfferings</a> API to obtain the available reserved node offerings. You can call this API by providing a specific reserved node offering and the number of nodes you want to reserve.   For more information about reserved node offerings, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/purchase-reserved-node-instance.html">Purchasing Reserved Nodes</a> in the <i>Amazon Redshift Cluster Management Guide</i>.

        :param self:
        :param NodeCount:  The number of reserved nodes that you want to purchase. Default: <code>1</code> <br/>ReservedNodeOfferingId:  The unique identifier of the reserved node offering you want to purchase.<br/>
        :return: ReservedNode:  Describes a reserved node. You can call the <a>DescribeReservedNodeOfferings</a> API to obtain the available reserved node offerings. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservedNodeOfferingType</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecurringCharges</i> The recurring charges for the reserved node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OfferingType</i> The anticipated utilization of the reserved node, as defined in the reserved node offering.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state of the reserved compute node. Possible Values: <ul> <li> pending-payment-This reserved node has recently been purchased, and the sale has been approved, but payment has not yet been confirmed. </li> <li> active-This reserved node is owned by the caller and is available for use. </li> <li> payment-failed-Payment failed for the purchase attempt. </li> <li> retired-The reserved node is no longer available.  </li> <li> exchanging-The owner is exchanging the reserved node for another reserved node. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeCount</i> The number of reserved compute nodes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrencyCode</i> The currency code for the reserved cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UsagePrice</i> The hourly rate Amazon Redshift charges you for this reserved node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FixedPrice</i> The fixed cost Amazon Redshift charges you for this reserved node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Duration</i> The duration of the node reservation in seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTime</i> The time the reservation started. You purchase a reserved node offering for a duration. This is the start time of that duration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The node type of the reserved node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservedNodeOfferingId</i> The identifier for the reserved node offering.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservedNodeId</i> The unique identifier for the reservation.<br/>        """
        pass

    def reboot_cluster(self, ClusterIdentifier: str) -> RebootClusterResult:
        """Reboots a cluster. This action is taken as soon as possible. It results in a momentary outage to the cluster, during which the cluster status is set to <code>rebooting</code>. A cluster event is created when the reboot is completed. Any pending cluster modifications (see <a>ModifyCluster</a>) are applied at this reboot. For more information about managing clusters, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-clusters.html">Amazon Redshift Clusters</a> in the <i>Amazon Redshift Cluster Management Guide</i>. 

        :param self:
        :param ClusterIdentifier:  The cluster identifier.<br/>
        :return: Cluster:  Describes a cluster.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResizeInfo</i> Returns the following: <ul> <li> AllowCancelResize: a boolean value indicating if the resize operation can be cancelled. </li> <li> ResizeType: Returns ClassicResize </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResizeInfo</b>: Returns the following: <ul> <li> AllowCancelResize: a boolean value indicating if the resize operation can be cancelled. </li> <li> ResizeType: Returns ClassicResize </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotScheduleState</i> The current state of the cluster snapshot schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotScheduleIdentifier</i> A unique identifier for the cluster snapshot schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeferredMaintenanceWindows</i> Describes a group of <code>DeferredMaintenanceWindow</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticResizeNumberOfNodeOptions</i> The number of nodes that you can resize the cluster to with the elastic resize method. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingActions</i> Cluster operations that are waiting to be started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoles</i> A list of AWS Identity and Access Management (IAM) roles that can be used by the cluster to access other AWS services.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS Key Management Service (AWS KMS) key ID of the encryption key used to encrypt data in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The list of tags for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterRevisionNumber</i> The specific revision number of the database in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticIpStatus</i> The status of the elastic IP (EIP) address.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ElasticIpStatus</b>: The status of the elastic IP (EIP) address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterNodes</i> The nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterPublicKey</i> The public key for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSnapshotCopyStatus</i> A value that returns the destination region and retention period that are configured for cross-region snapshot copy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ClusterSnapshotCopyStatus</b>: A value that returns the destination region and retention period that are configured for cross-region snapshot copy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmStatus</i> A value that reports whether the Amazon Redshift cluster has finished applying any hardware security module (HSM) settings changes specified in a modify cluster command. Values: active, applying&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>HsmStatus</b>: A value that reports whether the Amazon Redshift cluster has finished applying any hardware security module (HSM) settings changes specified in a modify cluster command. Values: active, applying<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTransferProgress</i> <p/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DataTransferProgress</b>: <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreStatus</i> A value that describes the status of a cluster restore action. This parameter returns null if the cluster was not created by restoring a snapshot.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RestoreStatus</b>: A value that describes the status of a cluster restore action. This parameter returns null if the cluster was not created by restoring a snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encrypted</i> A boolean value that, if <code>true</code>, indicates that data in the cluster is encrypted at rest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> A boolean value that, if <code>true</code>, indicates that the cluster can be accessed from a public network.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The number of compute nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowVersionUpgrade</i> A boolean value that, if <code>true</code>, indicates that major version upgrades will be applied automatically to the cluster during the maintenance window. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The version ID of the Amazon Redshift engine that is running on the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> A value that, if present, indicates that changes to the cluster are pending. Specific pending changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: A value that, if present, indicates that changes to the cluster are pending. Specific pending changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> The weekly time range, in Universal Coordinated Time (UTC), during which system maintenance can occur.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The name of the Availability Zone in which the cluster is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The identifier of the VPC the cluster is in, if the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSubnetGroupName</i> The name of the subnet group that is associated with the cluster. This parameter is valid only when the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterParameterGroups</i> The list of cluster parameter groups that are associated with this cluster. Each parameter group in the list is returned with its status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> A list of Amazon Virtual Private Cloud (Amazon VPC) security groups that are associated with the cluster. This parameter is returned only if the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSecurityGroups</i> A list of cluster security group that are associated with the cluster. Each security group is represented by an element that contains <code>ClusterSecurityGroup.Name</code> and <code>ClusterSecurityGroup.Status</code> subelements.  Cluster security groups are used when the cluster is not created in an Amazon Virtual Private Cloud (VPC). Clusters that are created in a VPC use VPC security groups, which are listed by the <b>VpcSecurityGroups</b> parameter. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The default number of days to retain a manual snapshot. If the value is -1, the snapshot is retained indefinitely. This setting doesn't change the retention period of existing snapshots. The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotRetentionPeriod</i> The number of days that automatic cluster snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> The date and time that the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> The connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: The connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The name of the initial database that was created when the cluster was created. This same name is returned for the life of the cluster. If an initial database was not specified, a database named <code>dev</code>dev was created by default. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> The master user name for the cluster. This name is used to connect to the database that is specified in the <b>DBName</b> parameter. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModifyStatus</i> The status of a modify operation, if any, initiated for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterAvailabilityStatus</i> The availability status of the cluster for queries. Possible values are the following: <ul> <li> Available - The cluster is available for queries.  </li> <li> Unavailable - The cluster is not available for queries. </li> <li> Maintenance - The cluster is intermittently available for queries due to maintenance activities. </li> <li> Modifying - The cluster is intermittently available for queries due to changes that modify the cluster. </li> <li> Failed - The cluster failed and is not available for queries. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterStatus</i>  The current state of the cluster. Possible values are the following: <ul> <li>  <code>available</code>  </li> <li>  <code>available, prep-for-resize</code>  </li> <li>  <code>available, resize-cleanup</code>  </li> <li>  <code>cancelling-resize</code>  </li> <li>  <code>creating</code>  </li> <li>  <code>deleting</code>  </li> <li>  <code>final-snapshot</code>  </li> <li>  <code>hardware-failure</code>  </li> <li>  <code>incompatible-hsm</code>  </li> <li>  <code>incompatible-network</code>  </li> <li>  <code>incompatible-parameters</code>  </li> <li>  <code>incompatible-restore</code>  </li> <li>  <code>modifying</code>  </li> <li>  <code>rebooting</code>  </li> <li>  <code>renaming</code>  </li> <li>  <code>resizing</code>  </li> <li>  <code>rotating-keys</code>  </li> <li>  <code>storage-full</code>  </li> <li>  <code>updating-hsm</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The node type for the nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The unique identifier of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS address of the Cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionType</i> The encryption type for a cluster. Possible values are: KMS and None. For the China region the possible values are None, and Legacy. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track that the cluster will change to during the next maintenance window.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> The pending or in-progress change of the ability to connect to the cluster from the public network.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The pending or in-progress change of the new identifier for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotRetentionPeriod</i> The pending or in-progress change of the automated snapshot retention period.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The pending or in-progress change of the service version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterType</i> The pending or in-progress change of the cluster type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The pending or in-progress change of the number of nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The pending or in-progress change of the cluster's node type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> The pending or in-progress change of the master user password for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedTimeToCompletionInSeconds</i> The estimate of the time remaining before the restore will complete. Returns 0 for a completed restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> The amount of time an in-progress restore has been running, or the amount of time it took a completed restore to finish.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ProgressInMegaBytes</i> The number of megabytes that have been transferred from snapshot storage.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotSizeInMegaBytes</i> The size of the set of snapshot data used to restore the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentRestoreRateInMegaBytesPerSecond</i> The number of megabytes per second being transferred from the backup storage. Returns the average rate for a completed backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the restore action. Returns starting, restoring, completed, or failed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> Describes the number of seconds that have elapsed during the data transfer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedTimeToCompletionInSeconds</i> Describes the estimated number of seconds remaining to complete the transfer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTransferredInMegaBytes</i> Describes the total amount of data that has been transfered in MB's.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalDataInMegaBytes</i> Describes the total amount of data to be transfered in megabytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentRateInMegaBytesPerSecond</i> Describes the data transfer rate in MB's per second.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Describes the status of the cluster. While the transfer is in progress the status is <code>transferringdata</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Reports whether the Amazon Redshift cluster has finished applying any HSM settings changes specified in a modify cluster command. Values: active, applying<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmConfigurationIdentifier</i> Specifies the name of the HSM configuration that contains the information the Amazon Redshift cluster can use to retrieve and store keys in an HSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmClientCertificateIdentifier</i> Specifies the name of the HSM client certificate the Amazon Redshift cluster uses to retrieve the data encryption keys stored in an HSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCopyGrantName</i> The name of the snapshot copy grant.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The number of days that automated snapshots are retained in the destination region after they are copied from a source region. If the value is -1, the manual snapshot is retained indefinitely.  The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RetentionPeriod</i> The number of days that automated snapshots are retained in the destination region after they are copied from a source region.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationRegion</i> The destination region that snapshots are automatically copied to when cross-region snapshot copy is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the elastic IP (EIP) address.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticIp</i> The elastic IP (EIP) address for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowCancelResize</i> A boolean value indicating if the resize operation can be cancelled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResizeType</i> Returns the value <code>ClassicResize</code>.<br/>        """
        pass

    def reset_cluster_parameter_group(self, ParameterGroupName: str, ResetAllParameters: Boolean=None, Parameters: List[Parameter]=None) -> ClusterParameterGroupNameMessage:
        """Sets one or more parameters of the specified parameter group to their default values and sets the source values of the parameters to "engine-default". To reset the entire parameter group specify the <i>ResetAllParameters</i> parameter. For parameter changes to take effect you must reboot any associated clusters. 

        :param self:
        :param Parameters:  An array of names of parameters to be reset. If <i>ResetAllParameters</i> option is not used, then at least one parameter name must be supplied.  Constraints: A maximum of 20 parameters can be reset in a single request.<br/>ResetAllParameters:  If <code>true</code>, all parameters in the specified parameter group will be reset to their default values.  Default: <code>true</code> <br/>ParameterGroupName:  The name of the cluster parameter group to be reset.<br/>
        :return: ParameterGroupStatus:  The status of the parameter group. For example, if you made a change to a parameter group name-value pair, then the change could be pending a reboot of an associated cluster.<br/>ParameterGroupName:  The name of the cluster parameter group.<br/>        """
        pass

    def resize_cluster(self, NumberOfNodes: int, ClusterIdentifier: str, ClusterType: str=None, NodeType: str=None, Classic: BooleanOptional=None) -> ResizeClusterResult:
        """Changes the size of the cluster. You can change the cluster's type, or change the number or type of nodes. The default behavior is to use the elastic resize method. With an elastic resize, your cluster is available for read and write operations more quickly than with the classic resize method.  Elastic resize operations have the following restrictions: <ul> <li> You can only resize clusters of the following types: <ul> <li> dc2.large </li> <li> dc2.8xlarge </li> <li> ds2.xlarge </li> <li> ds2.8xlarge </li> </ul> </li> <li> The type of nodes that you add must match the node type for the cluster. </li> </ul>

        :param self:
        :param Classic:  A boolean value indicating whether the resize operation is using the classic resize process. If you don't provide this parameter or set the value to <code>false</code>, the resize type is elastic. <br/>NumberOfNodes:  The new number of nodes for the cluster.<br/>NodeType:  The new node type for the nodes you are adding.<br/>ClusterType:  The new cluster type for the specified cluster.<br/>ClusterIdentifier:  The unique identifier for the cluster to resize.<br/>
        :return: Cluster:  Describes a cluster.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResizeInfo</i> Returns the following: <ul> <li> AllowCancelResize: a boolean value indicating if the resize operation can be cancelled. </li> <li> ResizeType: Returns ClassicResize </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResizeInfo</b>: Returns the following: <ul> <li> AllowCancelResize: a boolean value indicating if the resize operation can be cancelled. </li> <li> ResizeType: Returns ClassicResize </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotScheduleState</i> The current state of the cluster snapshot schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotScheduleIdentifier</i> A unique identifier for the cluster snapshot schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeferredMaintenanceWindows</i> Describes a group of <code>DeferredMaintenanceWindow</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticResizeNumberOfNodeOptions</i> The number of nodes that you can resize the cluster to with the elastic resize method. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingActions</i> Cluster operations that are waiting to be started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoles</i> A list of AWS Identity and Access Management (IAM) roles that can be used by the cluster to access other AWS services.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS Key Management Service (AWS KMS) key ID of the encryption key used to encrypt data in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The list of tags for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterRevisionNumber</i> The specific revision number of the database in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticIpStatus</i> The status of the elastic IP (EIP) address.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ElasticIpStatus</b>: The status of the elastic IP (EIP) address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterNodes</i> The nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterPublicKey</i> The public key for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSnapshotCopyStatus</i> A value that returns the destination region and retention period that are configured for cross-region snapshot copy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ClusterSnapshotCopyStatus</b>: A value that returns the destination region and retention period that are configured for cross-region snapshot copy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmStatus</i> A value that reports whether the Amazon Redshift cluster has finished applying any hardware security module (HSM) settings changes specified in a modify cluster command. Values: active, applying&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>HsmStatus</b>: A value that reports whether the Amazon Redshift cluster has finished applying any hardware security module (HSM) settings changes specified in a modify cluster command. Values: active, applying<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTransferProgress</i> <p/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DataTransferProgress</b>: <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreStatus</i> A value that describes the status of a cluster restore action. This parameter returns null if the cluster was not created by restoring a snapshot.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RestoreStatus</b>: A value that describes the status of a cluster restore action. This parameter returns null if the cluster was not created by restoring a snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encrypted</i> A boolean value that, if <code>true</code>, indicates that data in the cluster is encrypted at rest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> A boolean value that, if <code>true</code>, indicates that the cluster can be accessed from a public network.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The number of compute nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowVersionUpgrade</i> A boolean value that, if <code>true</code>, indicates that major version upgrades will be applied automatically to the cluster during the maintenance window. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The version ID of the Amazon Redshift engine that is running on the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> A value that, if present, indicates that changes to the cluster are pending. Specific pending changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: A value that, if present, indicates that changes to the cluster are pending. Specific pending changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> The weekly time range, in Universal Coordinated Time (UTC), during which system maintenance can occur.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The name of the Availability Zone in which the cluster is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The identifier of the VPC the cluster is in, if the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSubnetGroupName</i> The name of the subnet group that is associated with the cluster. This parameter is valid only when the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterParameterGroups</i> The list of cluster parameter groups that are associated with this cluster. Each parameter group in the list is returned with its status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> A list of Amazon Virtual Private Cloud (Amazon VPC) security groups that are associated with the cluster. This parameter is returned only if the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSecurityGroups</i> A list of cluster security group that are associated with the cluster. Each security group is represented by an element that contains <code>ClusterSecurityGroup.Name</code> and <code>ClusterSecurityGroup.Status</code> subelements.  Cluster security groups are used when the cluster is not created in an Amazon Virtual Private Cloud (VPC). Clusters that are created in a VPC use VPC security groups, which are listed by the <b>VpcSecurityGroups</b> parameter. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The default number of days to retain a manual snapshot. If the value is -1, the snapshot is retained indefinitely. This setting doesn't change the retention period of existing snapshots. The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotRetentionPeriod</i> The number of days that automatic cluster snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> The date and time that the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> The connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: The connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The name of the initial database that was created when the cluster was created. This same name is returned for the life of the cluster. If an initial database was not specified, a database named <code>dev</code>dev was created by default. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> The master user name for the cluster. This name is used to connect to the database that is specified in the <b>DBName</b> parameter. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModifyStatus</i> The status of a modify operation, if any, initiated for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterAvailabilityStatus</i> The availability status of the cluster for queries. Possible values are the following: <ul> <li> Available - The cluster is available for queries.  </li> <li> Unavailable - The cluster is not available for queries. </li> <li> Maintenance - The cluster is intermittently available for queries due to maintenance activities. </li> <li> Modifying - The cluster is intermittently available for queries due to changes that modify the cluster. </li> <li> Failed - The cluster failed and is not available for queries. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterStatus</i>  The current state of the cluster. Possible values are the following: <ul> <li>  <code>available</code>  </li> <li>  <code>available, prep-for-resize</code>  </li> <li>  <code>available, resize-cleanup</code>  </li> <li>  <code>cancelling-resize</code>  </li> <li>  <code>creating</code>  </li> <li>  <code>deleting</code>  </li> <li>  <code>final-snapshot</code>  </li> <li>  <code>hardware-failure</code>  </li> <li>  <code>incompatible-hsm</code>  </li> <li>  <code>incompatible-network</code>  </li> <li>  <code>incompatible-parameters</code>  </li> <li>  <code>incompatible-restore</code>  </li> <li>  <code>modifying</code>  </li> <li>  <code>rebooting</code>  </li> <li>  <code>renaming</code>  </li> <li>  <code>resizing</code>  </li> <li>  <code>rotating-keys</code>  </li> <li>  <code>storage-full</code>  </li> <li>  <code>updating-hsm</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The node type for the nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The unique identifier of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS address of the Cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionType</i> The encryption type for a cluster. Possible values are: KMS and None. For the China region the possible values are None, and Legacy. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track that the cluster will change to during the next maintenance window.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> The pending or in-progress change of the ability to connect to the cluster from the public network.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The pending or in-progress change of the new identifier for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotRetentionPeriod</i> The pending or in-progress change of the automated snapshot retention period.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The pending or in-progress change of the service version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterType</i> The pending or in-progress change of the cluster type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The pending or in-progress change of the number of nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The pending or in-progress change of the cluster's node type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> The pending or in-progress change of the master user password for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedTimeToCompletionInSeconds</i> The estimate of the time remaining before the restore will complete. Returns 0 for a completed restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> The amount of time an in-progress restore has been running, or the amount of time it took a completed restore to finish.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ProgressInMegaBytes</i> The number of megabytes that have been transferred from snapshot storage.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotSizeInMegaBytes</i> The size of the set of snapshot data used to restore the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentRestoreRateInMegaBytesPerSecond</i> The number of megabytes per second being transferred from the backup storage. Returns the average rate for a completed backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the restore action. Returns starting, restoring, completed, or failed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> Describes the number of seconds that have elapsed during the data transfer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedTimeToCompletionInSeconds</i> Describes the estimated number of seconds remaining to complete the transfer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTransferredInMegaBytes</i> Describes the total amount of data that has been transfered in MB's.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalDataInMegaBytes</i> Describes the total amount of data to be transfered in megabytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentRateInMegaBytesPerSecond</i> Describes the data transfer rate in MB's per second.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Describes the status of the cluster. While the transfer is in progress the status is <code>transferringdata</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Reports whether the Amazon Redshift cluster has finished applying any HSM settings changes specified in a modify cluster command. Values: active, applying<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmConfigurationIdentifier</i> Specifies the name of the HSM configuration that contains the information the Amazon Redshift cluster can use to retrieve and store keys in an HSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmClientCertificateIdentifier</i> Specifies the name of the HSM client certificate the Amazon Redshift cluster uses to retrieve the data encryption keys stored in an HSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCopyGrantName</i> The name of the snapshot copy grant.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The number of days that automated snapshots are retained in the destination region after they are copied from a source region. If the value is -1, the manual snapshot is retained indefinitely.  The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RetentionPeriod</i> The number of days that automated snapshots are retained in the destination region after they are copied from a source region.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationRegion</i> The destination region that snapshots are automatically copied to when cross-region snapshot copy is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the elastic IP (EIP) address.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticIp</i> The elastic IP (EIP) address for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowCancelResize</i> A boolean value indicating if the resize operation can be cancelled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResizeType</i> Returns the value <code>ClassicResize</code>.<br/>        """
        pass

    def restore_from_cluster_snapshot(self, SnapshotIdentifier: str, ClusterIdentifier: str, SnapshotClusterIdentifier: str=None, Port: int=None, AvailabilityZone: str=None, AllowVersionUpgrade: BooleanOptional=None, ClusterSubnetGroupName: str=None, PubliclyAccessible: BooleanOptional=None, OwnerAccount: str=None, HsmClientCertificateIdentifier: str=None, HsmConfigurationIdentifier: str=None, ElasticIp: str=None, ClusterParameterGroupName: str=None, ClusterSecurityGroups: List[String]=None, VpcSecurityGroupIds: List[String]=None, PreferredMaintenanceWindow: str=None, AutomatedSnapshotRetentionPeriod: int=None, ManualSnapshotRetentionPeriod: int=None, KmsKeyId: str=None, NodeType: str=None, EnhancedVpcRouting: BooleanOptional=None, AdditionalInfo: str=None, IamRoles: List[String]=None, MaintenanceTrackName: str=None, SnapshotScheduleIdentifier: str=None) -> RestoreFromClusterSnapshotResult:
        """Creates a new cluster from a snapshot. By default, Amazon Redshift creates the resulting cluster with the same configuration as the original cluster from which the snapshot was created, except that the new cluster is created with the default cluster security and parameter groups. After Amazon Redshift creates the cluster, you can use the <a>ModifyCluster</a> API to associate a different security group and different parameter group with the restored cluster. If you are using a DS node type, you can also choose to change to another DS node type of the same size during restore. If you restore a cluster into a VPC, you must provide a cluster subnet group where you want the cluster restored.  For more information about working with snapshots, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-snapshots.html">Amazon Redshift Snapshots</a> in the <i>Amazon Redshift Cluster Management Guide</i>.

        :param self:
        :param SnapshotScheduleIdentifier:  A unique identifier for the snapshot schedule.<br/>MaintenanceTrackName:  The name of the maintenance track for the restored cluster. When you take a snapshot, the snapshot inherits the <code>MaintenanceTrack</code> value from the cluster. The snapshot might be on a different track than the cluster that was the source for the snapshot. For example, suppose that you take a snapshot of a cluster that is on the current track and then change the cluster to be on the trailing track. In this case, the snapshot and the source cluster are on different tracks.<br/>IamRoles:  A list of AWS Identity and Access Management (IAM) roles that can be used by the cluster to access other AWS services. You must supply the IAM roles in their Amazon Resource Name (ARN) format. You can supply up to 10 IAM roles in a single request. A cluster can have up to 10 IAM roles associated at any time.<br/>AdditionalInfo:  Reserved.<br/>EnhancedVpcRouting:  An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>NodeType:  The node type that the restored cluster will be provisioned with. Default: The node type of the cluster from which the snapshot was taken. You can modify this if you are using any DS node type. In that case, you can choose to restore into another DS node type of the same size. For example, you can restore ds1.8xlarge into ds2.8xlarge, or ds1.xlarge into ds2.xlarge. If you have a DC instance type, you must restore into that same instance type and size. In other words, you can only restore a dc1.large instance type into another dc1.large instance type or dc2.large instance type. You can't restore dc1.8xlarge to dc2.8xlarge. First restore to a dc1.8xlareg cluster, then resize to a dc2.8large cluster. For more information about node types, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-clusters.html#rs-about-clusters-and-nodes"> About Clusters and Nodes</a> in the <i>Amazon Redshift Cluster Management Guide</i>. <br/>KmsKeyId:  The AWS Key Management Service (KMS) key ID of the encryption key that you want to use to encrypt data in the cluster that you restore from a shared snapshot.<br/>ManualSnapshotRetentionPeriod:  The default number of days to retain a manual snapshot. If the value is -1, the snapshot is retained indefinitely. This setting doesn't change the retention period of existing snapshots. The value must be either -1 or an integer between 1 and 3,653.<br/>AutomatedSnapshotRetentionPeriod:  The number of days that automated snapshots are retained. If the value is 0, automated snapshots are disabled. Even if automated snapshots are disabled, you can still create manual snapshots when you want with <a>CreateClusterSnapshot</a>.  Default: The value selected for the cluster from which the snapshot was taken. Constraints: Must be a value from 0 to 35.<br/>PreferredMaintenanceWindow:  The weekly time range (in UTC) during which automated cluster maintenance can occur.  Format: <code>ddd:hh24:mi-ddd:hh24:mi</code>   Default: The value selected for the cluster from which the snapshot was taken. For more information about the time blocks for each region, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-clusters.html#rs-maintenance-windows">Maintenance Windows</a> in Amazon Redshift Cluster Management Guide.  Valid Days: Mon | Tue | Wed | Thu | Fri | Sat | Sun Constraints: Minimum 30-minute window.<br/>VpcSecurityGroupIds:  A list of Virtual Private Cloud (VPC) security groups to be associated with the cluster. Default: The default VPC security group is associated with the cluster. VPC security groups only apply to clusters in VPCs.<br/>ClusterSecurityGroups:  A list of security groups to be associated with this cluster. Default: The default cluster security group for Amazon Redshift. Cluster security groups only apply to clusters outside of VPCs.<br/>ClusterParameterGroupName:  The name of the parameter group to be associated with this cluster. Default: The default Amazon Redshift cluster parameter group. For information about the default parameter group, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-parameter-groups.html">Working with Amazon Redshift Parameter Groups</a>. Constraints: <ul> <li> Must be 1 to 255 alphanumeric characters or hyphens. </li> <li> First character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> </ul><br/>ElasticIp:  The elastic IP (EIP) address for the cluster.<br/>HsmConfigurationIdentifier:  Specifies the name of the HSM configuration that contains the information the Amazon Redshift cluster can use to retrieve and store keys in an HSM.<br/>HsmClientCertificateIdentifier:  Specifies the name of the HSM client certificate the Amazon Redshift cluster uses to retrieve the data encryption keys stored in an HSM.<br/>OwnerAccount:  The AWS customer account used to create or copy the snapshot. Required if you are restoring a snapshot you do not own, optional if you own the snapshot.<br/>PubliclyAccessible:  If <code>true</code>, the cluster can be accessed from a public network. <br/>ClusterSubnetGroupName:  The name of the subnet group where you want to cluster restored. A snapshot of cluster in VPC can be restored only in VPC. Therefore, you must provide subnet group name where you want the cluster restored.<br/>AllowVersionUpgrade:  If <code>true</code>, major version upgrades can be applied during the maintenance window to the Amazon Redshift engine that is running on the cluster.  Default: <code>true</code> <br/>AvailabilityZone:  The Amazon EC2 Availability Zone in which to restore the cluster. Default: A random, system-chosen Availability Zone. Example: <code>us-east-1a</code> <br/>Port:  The port number on which the cluster accepts connections. Default: The same port as the original cluster. Constraints: Must be between <code>1115</code> and <code>65535</code>.<br/>SnapshotClusterIdentifier:  The name of the cluster the source snapshot was created from. This parameter is required if your IAM user has a policy containing a snapshot resource element that specifies anything other than * for the cluster name.<br/>SnapshotIdentifier:  The name of the snapshot from which to create the new cluster. This parameter isn't case sensitive. Example: <code>my-snapshot-id</code> <br/>ClusterIdentifier:  The identifier of the cluster that will be created from restoring the snapshot. Constraints: <ul> <li> Must contain from 1 to 63 alphanumeric characters or hyphens. </li> <li> Alphabetic characters must be lowercase. </li> <li> First character must be a letter. </li> <li> Cannot end with a hyphen or contain two consecutive hyphens. </li> <li> Must be unique for all clusters within an AWS account. </li> </ul><br/>
        :return: Cluster:  Describes a cluster.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResizeInfo</i> Returns the following: <ul> <li> AllowCancelResize: a boolean value indicating if the resize operation can be cancelled. </li> <li> ResizeType: Returns ClassicResize </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResizeInfo</b>: Returns the following: <ul> <li> AllowCancelResize: a boolean value indicating if the resize operation can be cancelled. </li> <li> ResizeType: Returns ClassicResize </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotScheduleState</i> The current state of the cluster snapshot schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotScheduleIdentifier</i> A unique identifier for the cluster snapshot schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeferredMaintenanceWindows</i> Describes a group of <code>DeferredMaintenanceWindow</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticResizeNumberOfNodeOptions</i> The number of nodes that you can resize the cluster to with the elastic resize method. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingActions</i> Cluster operations that are waiting to be started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoles</i> A list of AWS Identity and Access Management (IAM) roles that can be used by the cluster to access other AWS services.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS Key Management Service (AWS KMS) key ID of the encryption key used to encrypt data in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The list of tags for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterRevisionNumber</i> The specific revision number of the database in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticIpStatus</i> The status of the elastic IP (EIP) address.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ElasticIpStatus</b>: The status of the elastic IP (EIP) address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterNodes</i> The nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterPublicKey</i> The public key for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSnapshotCopyStatus</i> A value that returns the destination region and retention period that are configured for cross-region snapshot copy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ClusterSnapshotCopyStatus</b>: A value that returns the destination region and retention period that are configured for cross-region snapshot copy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmStatus</i> A value that reports whether the Amazon Redshift cluster has finished applying any hardware security module (HSM) settings changes specified in a modify cluster command. Values: active, applying&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>HsmStatus</b>: A value that reports whether the Amazon Redshift cluster has finished applying any hardware security module (HSM) settings changes specified in a modify cluster command. Values: active, applying<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTransferProgress</i> <p/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DataTransferProgress</b>: <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreStatus</i> A value that describes the status of a cluster restore action. This parameter returns null if the cluster was not created by restoring a snapshot.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RestoreStatus</b>: A value that describes the status of a cluster restore action. This parameter returns null if the cluster was not created by restoring a snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encrypted</i> A boolean value that, if <code>true</code>, indicates that data in the cluster is encrypted at rest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> A boolean value that, if <code>true</code>, indicates that the cluster can be accessed from a public network.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The number of compute nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowVersionUpgrade</i> A boolean value that, if <code>true</code>, indicates that major version upgrades will be applied automatically to the cluster during the maintenance window. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The version ID of the Amazon Redshift engine that is running on the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> A value that, if present, indicates that changes to the cluster are pending. Specific pending changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: A value that, if present, indicates that changes to the cluster are pending. Specific pending changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> The weekly time range, in Universal Coordinated Time (UTC), during which system maintenance can occur.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The name of the Availability Zone in which the cluster is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The identifier of the VPC the cluster is in, if the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSubnetGroupName</i> The name of the subnet group that is associated with the cluster. This parameter is valid only when the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterParameterGroups</i> The list of cluster parameter groups that are associated with this cluster. Each parameter group in the list is returned with its status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> A list of Amazon Virtual Private Cloud (Amazon VPC) security groups that are associated with the cluster. This parameter is returned only if the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSecurityGroups</i> A list of cluster security group that are associated with the cluster. Each security group is represented by an element that contains <code>ClusterSecurityGroup.Name</code> and <code>ClusterSecurityGroup.Status</code> subelements.  Cluster security groups are used when the cluster is not created in an Amazon Virtual Private Cloud (VPC). Clusters that are created in a VPC use VPC security groups, which are listed by the <b>VpcSecurityGroups</b> parameter. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The default number of days to retain a manual snapshot. If the value is -1, the snapshot is retained indefinitely. This setting doesn't change the retention period of existing snapshots. The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotRetentionPeriod</i> The number of days that automatic cluster snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> The date and time that the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> The connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: The connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The name of the initial database that was created when the cluster was created. This same name is returned for the life of the cluster. If an initial database was not specified, a database named <code>dev</code>dev was created by default. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> The master user name for the cluster. This name is used to connect to the database that is specified in the <b>DBName</b> parameter. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModifyStatus</i> The status of a modify operation, if any, initiated for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterAvailabilityStatus</i> The availability status of the cluster for queries. Possible values are the following: <ul> <li> Available - The cluster is available for queries.  </li> <li> Unavailable - The cluster is not available for queries. </li> <li> Maintenance - The cluster is intermittently available for queries due to maintenance activities. </li> <li> Modifying - The cluster is intermittently available for queries due to changes that modify the cluster. </li> <li> Failed - The cluster failed and is not available for queries. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterStatus</i>  The current state of the cluster. Possible values are the following: <ul> <li>  <code>available</code>  </li> <li>  <code>available, prep-for-resize</code>  </li> <li>  <code>available, resize-cleanup</code>  </li> <li>  <code>cancelling-resize</code>  </li> <li>  <code>creating</code>  </li> <li>  <code>deleting</code>  </li> <li>  <code>final-snapshot</code>  </li> <li>  <code>hardware-failure</code>  </li> <li>  <code>incompatible-hsm</code>  </li> <li>  <code>incompatible-network</code>  </li> <li>  <code>incompatible-parameters</code>  </li> <li>  <code>incompatible-restore</code>  </li> <li>  <code>modifying</code>  </li> <li>  <code>rebooting</code>  </li> <li>  <code>renaming</code>  </li> <li>  <code>resizing</code>  </li> <li>  <code>rotating-keys</code>  </li> <li>  <code>storage-full</code>  </li> <li>  <code>updating-hsm</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The node type for the nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The unique identifier of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS address of the Cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionType</i> The encryption type for a cluster. Possible values are: KMS and None. For the China region the possible values are None, and Legacy. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track that the cluster will change to during the next maintenance window.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> The pending or in-progress change of the ability to connect to the cluster from the public network.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The pending or in-progress change of the new identifier for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotRetentionPeriod</i> The pending or in-progress change of the automated snapshot retention period.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The pending or in-progress change of the service version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterType</i> The pending or in-progress change of the cluster type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The pending or in-progress change of the number of nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The pending or in-progress change of the cluster's node type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> The pending or in-progress change of the master user password for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedTimeToCompletionInSeconds</i> The estimate of the time remaining before the restore will complete. Returns 0 for a completed restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> The amount of time an in-progress restore has been running, or the amount of time it took a completed restore to finish.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ProgressInMegaBytes</i> The number of megabytes that have been transferred from snapshot storage.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotSizeInMegaBytes</i> The size of the set of snapshot data used to restore the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentRestoreRateInMegaBytesPerSecond</i> The number of megabytes per second being transferred from the backup storage. Returns the average rate for a completed backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the restore action. Returns starting, restoring, completed, or failed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> Describes the number of seconds that have elapsed during the data transfer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedTimeToCompletionInSeconds</i> Describes the estimated number of seconds remaining to complete the transfer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTransferredInMegaBytes</i> Describes the total amount of data that has been transfered in MB's.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalDataInMegaBytes</i> Describes the total amount of data to be transfered in megabytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentRateInMegaBytesPerSecond</i> Describes the data transfer rate in MB's per second.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Describes the status of the cluster. While the transfer is in progress the status is <code>transferringdata</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Reports whether the Amazon Redshift cluster has finished applying any HSM settings changes specified in a modify cluster command. Values: active, applying<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmConfigurationIdentifier</i> Specifies the name of the HSM configuration that contains the information the Amazon Redshift cluster can use to retrieve and store keys in an HSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmClientCertificateIdentifier</i> Specifies the name of the HSM client certificate the Amazon Redshift cluster uses to retrieve the data encryption keys stored in an HSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCopyGrantName</i> The name of the snapshot copy grant.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The number of days that automated snapshots are retained in the destination region after they are copied from a source region. If the value is -1, the manual snapshot is retained indefinitely.  The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RetentionPeriod</i> The number of days that automated snapshots are retained in the destination region after they are copied from a source region.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationRegion</i> The destination region that snapshots are automatically copied to when cross-region snapshot copy is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the elastic IP (EIP) address.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticIp</i> The elastic IP (EIP) address for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowCancelResize</i> A boolean value indicating if the resize operation can be cancelled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResizeType</i> Returns the value <code>ClassicResize</code>.<br/>        """
        pass

    def restore_table_from_cluster_snapshot(self, NewTableName: str, SourceTableName: str, SourceDatabaseName: str, SnapshotIdentifier: str, ClusterIdentifier: str, SourceSchemaName: str=None, TargetDatabaseName: str=None, TargetSchemaName: str=None) -> RestoreTableFromClusterSnapshotResult:
        """Creates a new table from a table in an Amazon Redshift cluster snapshot. You must create the new table within the Amazon Redshift cluster that the snapshot was taken from. You cannot use <code>RestoreTableFromClusterSnapshot</code> to restore a table with the same name as an existing table in an Amazon Redshift cluster. That is, you cannot overwrite an existing table in a cluster with a restored table. If you want to replace your original table with a new, restored table, then rename or drop your original table before you call <code>RestoreTableFromClusterSnapshot</code>. When you have renamed your original table, then you can pass the original name of the table as the <code>NewTableName</code> parameter value in the call to <code>RestoreTableFromClusterSnapshot</code>. This way, you can replace the original table with the table created from the snapshot.

        :param self:
        :param NewTableName:  The name of the table to create as a result of the current request.<br/>TargetSchemaName:  The name of the schema to restore the table to.<br/>TargetDatabaseName:  The name of the database to restore the table to.<br/>SourceTableName:  The name of the source table to restore from.<br/>SourceSchemaName:  The name of the source schema that contains the table to restore from. If you do not specify a <code>SourceSchemaName</code> value, the default is <code>public</code>.<br/>SourceDatabaseName:  The name of the source database that contains the table to restore from.<br/>SnapshotIdentifier:  The identifier of the snapshot to restore the table from. This snapshot must have been created from the Amazon Redshift cluster specified by the <code>ClusterIdentifier</code> parameter.<br/>ClusterIdentifier:  The identifier of the Amazon Redshift cluster to restore the table to.<br/>
        :return: TableRestoreStatus:  Describes the status of a <a>RestoreTableFromClusterSnapshot</a> operation.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NewTableName</i> The name of the table to create as a result of the table restore request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetSchemaName</i> The name of the schema to restore the table to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetDatabaseName</i> The name of the database to restore the table to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceTableName</i> The name of the source table being restored.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceSchemaName</i> The name of the source schema that contains the table being restored.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceDatabaseName</i> The name of the source database that contains the table being restored.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotIdentifier</i> The identifier of the snapshot that the table is being restored from.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The identifier of the Amazon Redshift cluster that the table is being restored to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalDataInMegaBytes</i> The total amount of data to restore to the new table, in megabytes (MB).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProgressInMegaBytes</i> The amount of data restored to the new table so far, in megabytes (MB).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestTime</i> The time that the table restore request was made, in Universal Coordinated Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> A description of the status of the table restore request. Status values include <code>SUCCEEDED</code>, <code>FAILED</code>, <code>CANCELED</code>, <code>PENDING</code>, <code>IN_PROGRESS</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> A value that describes the current state of the table restore request. Valid Values: <code>SUCCEEDED</code>, <code>FAILED</code>, <code>CANCELED</code>, <code>PENDING</code>, <code>IN_PROGRESS</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TableRestoreRequestId</i> The unique identifier for the table restore request.<br/>        """
        pass

    def revoke_cluster_security_group_ingress(self, ClusterSecurityGroupName: str, CIDRIP: str=None, EC2SecurityGroupName: str=None, EC2SecurityGroupOwnerId: str=None) -> RevokeClusterSecurityGroupIngressResult:
        """Revokes an ingress rule in an Amazon Redshift security group for a previously authorized IP range or Amazon EC2 security group. To add an ingress rule, see <a>AuthorizeClusterSecurityGroupIngress</a>. For information about managing security groups, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-security-groups.html">Amazon Redshift Cluster Security Groups</a> in the <i>Amazon Redshift Cluster Management Guide</i>. 

        :param self:
        :param EC2SecurityGroupOwnerId:  The AWS account number of the owner of the security group specified in the <code>EC2SecurityGroupName</code> parameter. The AWS access key ID is not an acceptable value. If <code>EC2SecurityGroupOwnerId</code> is specified, <code>EC2SecurityGroupName</code> must also be provided. and <code>CIDRIP</code> cannot be provided.  Example: <code>111122223333</code> <br/>EC2SecurityGroupName:  The name of the EC2 Security Group whose access is to be revoked. If <code>EC2SecurityGroupName</code> is specified, <code>EC2SecurityGroupOwnerId</code> must also be provided and <code>CIDRIP</code> cannot be provided. <br/>CIDRIP:  The IP range for which to revoke access. This range must be a valid Classless Inter-Domain Routing (CIDR) block of IP addresses. If <code>CIDRIP</code> is specified, <code>EC2SecurityGroupName</code> and <code>EC2SecurityGroupOwnerId</code> cannot be provided. <br/>ClusterSecurityGroupName:  The name of the security Group from which to revoke the ingress rule.<br/>
        :return: ClusterSecurityGroup:  Describes a security group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The list of tags for the cluster security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IPRanges</i> A list of IP ranges (CIDR blocks) that are permitted to access clusters associated with this cluster security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EC2SecurityGroups</i> A list of EC2 security groups that are permitted to access clusters associated with this cluster security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of the security group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSecurityGroupName</i> The name of the cluster security group to which the operation was applied.<br/>        """
        pass

    def revoke_snapshot_access(self, AccountWithRestoreAccess: str, SnapshotIdentifier: str, SnapshotClusterIdentifier: str=None) -> RevokeSnapshotAccessResult:
        """Removes the ability of the specified AWS customer account to restore the specified snapshot. If the account is currently restoring the snapshot, the restore will run to completion.  For more information about working with snapshots, go to <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-snapshots.html">Amazon Redshift Snapshots</a> in the <i>Amazon Redshift Cluster Management Guide</i>.

        :param self:
        :param AccountWithRestoreAccess:  The identifier of the AWS customer account that can no longer restore the specified snapshot.<br/>SnapshotClusterIdentifier:  The identifier of the cluster the snapshot was created from. This parameter is required if your IAM user has a policy containing a snapshot resource element that specifies anything other than * for the cluster name.<br/>SnapshotIdentifier:  The identifier of the snapshot that the account can no longer access.<br/>
        :return: Snapshot:  Describes a snapshot.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotRetentionStartTime</i> A timestamp representing the start of the retention period for the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRemainingDays</i> The number of days until a manual snapshot will pass its retention period.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The number of days that a manual snapshot is retained. If the value is -1, the manual snapshot is retained indefinitely.  The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track for the snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestorableNodeTypes</i> The list of node types that this cluster snapshot is able to restore into.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The list of tags for the cluster snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceRegion</i> The source region from which the snapshot was copied.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> The amount of time an in-progress snapshot backup has been running, or the amount of time it took a completed backup to finish.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedSecondsToCompletion</i> The estimate of the time remaining before the snapshot backup will complete. Returns <code>0</code> for a completed backup. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentBackupRateInMegaBytesPerSecond</i> The number of megabytes per second being transferred to the snapshot backup. Returns <code>0</code> for a completed backup. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BackupProgressInMegaBytes</i> The number of megabytes that have been transferred to the snapshot backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ActualIncrementalBackupSizeInMegaBytes</i> The size of the incremental backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalBackupSizeInMegaBytes</i> The size of the complete set of backup data that would be used to restore the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerAccount</i> For manual snapshots, the AWS customer account used to create or copy the snapshot. For automatic snapshots, the owner of the cluster. The owner can perform all snapshot actions, such as sharing a manual snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountsWithRestoreAccess</i> A list of the AWS customer accounts authorized to restore the snapshot. Returns <code>null</code> if no accounts are authorized. Visible only to the snapshot owner. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptedWithHSM</i> A boolean that indicates whether the snapshot data is encrypted using the HSM keys of the source cluster. <code>true</code> indicates that the data is encrypted using HSM keys.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS Key Management Service (KMS) key ID of the encryption key that was used to encrypt data in the cluster from which the snapshot was taken.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encrypted</i> If <code>true</code>, the data in the snapshot is encrypted at rest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The VPC identifier of the cluster if the snapshot is from a cluster in a VPC. Otherwise, this field is not in the output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The name of the database that was created when the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The number of nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The node type of the nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotType</i> The snapshot type. Snapshots created using <a>CreateClusterSnapshot</a> and <a>CopyClusterSnapshot</a> are of type "manual". <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The version ID of the Amazon Redshift engine that is running on the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> The master user name for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> The time (UTC) when the cluster was originally created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The Availability Zone in which the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port that the cluster is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The snapshot status. The value of the status depends on the API operation used:  <ul> <li>  <a>CreateClusterSnapshot</a> and <a>CopyClusterSnapshot</a> returns status as "creating".  </li> <li>  <a>DescribeClusterSnapshots</a> returns status as "creating", "available", "final snapshot", or "failed". </li> <li>  <a>DeleteClusterSnapshot</a> returns status as "deleted". </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCreateTime</i> The time (in UTC format) when Amazon Redshift began the snapshot. A snapshot contains a copy of the cluster data as of this exact time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The identifier of the cluster for which the snapshot was taken.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotIdentifier</i> The snapshot identifier that is provided in the request.<br/>        """
        pass

    def rotate_encryption_key(self, ClusterIdentifier: str) -> RotateEncryptionKeyResult:
        """Rotates the encryption keys for a cluster.

        :param self:
        :param ClusterIdentifier:  The unique identifier of the cluster that you want to rotate the encryption keys for. Constraints: Must be the name of valid cluster that has encryption enabled.<br/>
        :return: Cluster:  Describes a cluster.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResizeInfo</i> Returns the following: <ul> <li> AllowCancelResize: a boolean value indicating if the resize operation can be cancelled. </li> <li> ResizeType: Returns ClassicResize </li> </ul>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResizeInfo</b>: Returns the following: <ul> <li> AllowCancelResize: a boolean value indicating if the resize operation can be cancelled. </li> <li> ResizeType: Returns ClassicResize </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotScheduleState</i> The current state of the cluster snapshot schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotScheduleIdentifier</i> A unique identifier for the cluster snapshot schedule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeferredMaintenanceWindows</i> Describes a group of <code>DeferredMaintenanceWindow</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticResizeNumberOfNodeOptions</i> The number of nodes that you can resize the cluster to with the elastic resize method. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingActions</i> Cluster operations that are waiting to be started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoles</i> A list of AWS Identity and Access Management (IAM) roles that can be used by the cluster to access other AWS services.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS Key Management Service (AWS KMS) key ID of the encryption key used to encrypt data in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The list of tags for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterRevisionNumber</i> The specific revision number of the database in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticIpStatus</i> The status of the elastic IP (EIP) address.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ElasticIpStatus</b>: The status of the elastic IP (EIP) address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterNodes</i> The nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterPublicKey</i> The public key for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSnapshotCopyStatus</i> A value that returns the destination region and retention period that are configured for cross-region snapshot copy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ClusterSnapshotCopyStatus</b>: A value that returns the destination region and retention period that are configured for cross-region snapshot copy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmStatus</i> A value that reports whether the Amazon Redshift cluster has finished applying any hardware security module (HSM) settings changes specified in a modify cluster command. Values: active, applying&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>HsmStatus</b>: A value that reports whether the Amazon Redshift cluster has finished applying any hardware security module (HSM) settings changes specified in a modify cluster command. Values: active, applying<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTransferProgress</i> <p/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DataTransferProgress</b>: <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestoreStatus</i> A value that describes the status of a cluster restore action. This parameter returns null if the cluster was not created by restoring a snapshot.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RestoreStatus</b>: A value that describes the status of a cluster restore action. This parameter returns null if the cluster was not created by restoring a snapshot.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Encrypted</i> A boolean value that, if <code>true</code>, indicates that data in the cluster is encrypted at rest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> A boolean value that, if <code>true</code>, indicates that the cluster can be accessed from a public network.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The number of compute nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowVersionUpgrade</i> A boolean value that, if <code>true</code>, indicates that major version upgrades will be applied automatically to the cluster during the maintenance window. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The version ID of the Amazon Redshift engine that is running on the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PendingModifiedValues</i> A value that, if present, indicates that changes to the cluster are pending. Specific pending changes are identified by subelements.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PendingModifiedValues</b>: A value that, if present, indicates that changes to the cluster are pending. Specific pending changes are identified by subelements.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreferredMaintenanceWindow</i> The weekly time range, in Universal Coordinated Time (UTC), during which system maintenance can occur.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AvailabilityZone</i> The name of the Availability Zone in which the cluster is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcId</i> The identifier of the VPC the cluster is in, if the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSubnetGroupName</i> The name of the subnet group that is associated with the cluster. This parameter is valid only when the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterParameterGroups</i> The list of cluster parameter groups that are associated with this cluster. Each parameter group in the list is returned with its status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcSecurityGroups</i> A list of Amazon Virtual Private Cloud (Amazon VPC) security groups that are associated with the cluster. This parameter is returned only if the cluster is in a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterSecurityGroups</i> A list of cluster security group that are associated with the cluster. Each security group is represented by an element that contains <code>ClusterSecurityGroup.Name</code> and <code>ClusterSecurityGroup.Status</code> subelements.  Cluster security groups are used when the cluster is not created in an Amazon Virtual Private Cloud (VPC). Clusters that are created in a VPC use VPC security groups, which are listed by the <b>VpcSecurityGroups</b> parameter. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The default number of days to retain a manual snapshot. If the value is -1, the snapshot is retained indefinitely. This setting doesn't change the retention period of existing snapshots. The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotRetentionPeriod</i> The number of days that automatic cluster snapshots are retained.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterCreateTime</i> The date and time that the cluster was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Endpoint</i> The connection endpoint.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Endpoint</b>: The connection endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DBName</i> The name of the initial database that was created when the cluster was created. This same name is returned for the life of the cluster. If an initial database was not specified, a database named <code>dev</code>dev was created by default. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUsername</i> The master user name for the cluster. This name is used to connect to the database that is specified in the <b>DBName</b> parameter. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModifyStatus</i> The status of a modify operation, if any, initiated for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterAvailabilityStatus</i> The availability status of the cluster for queries. Possible values are the following: <ul> <li> Available - The cluster is available for queries.  </li> <li> Unavailable - The cluster is not available for queries. </li> <li> Maintenance - The cluster is intermittently available for queries due to maintenance activities. </li> <li> Modifying - The cluster is intermittently available for queries due to changes that modify the cluster. </li> <li> Failed - The cluster failed and is not available for queries. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterStatus</i>  The current state of the cluster. Possible values are the following: <ul> <li>  <code>available</code>  </li> <li>  <code>available, prep-for-resize</code>  </li> <li>  <code>available, resize-cleanup</code>  </li> <li>  <code>cancelling-resize</code>  </li> <li>  <code>creating</code>  </li> <li>  <code>deleting</code>  </li> <li>  <code>final-snapshot</code>  </li> <li>  <code>hardware-failure</code>  </li> <li>  <code>incompatible-hsm</code>  </li> <li>  <code>incompatible-network</code>  </li> <li>  <code>incompatible-parameters</code>  </li> <li>  <code>incompatible-restore</code>  </li> <li>  <code>modifying</code>  </li> <li>  <code>rebooting</code>  </li> <li>  <code>renaming</code>  </li> <li>  <code>resizing</code>  </li> <li>  <code>rotating-keys</code>  </li> <li>  <code>storage-full</code>  </li> <li>  <code>updating-hsm</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The node type for the nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The unique identifier of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port that the database engine is listening on.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Address</i> The DNS address of the Cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionType</i> The encryption type for a cluster. Possible values are: KMS and None. For the China region the possible values are None, and Legacy. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaintenanceTrackName</i> The name of the maintenance track that the cluster will change to during the next maintenance window.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EnhancedVpcRouting</i> An option that specifies whether to create the cluster with enhanced VPC routing enabled. To create a cluster that uses enhanced VPC routing, the cluster must be in a VPC. For more information, see <a href="https://docs.aws.amazon.com/redshift/latest/mgmt/enhanced-vpc-routing.html">Enhanced VPC Routing</a> in the Amazon Redshift Cluster Management Guide. If this option is <code>true</code>, enhanced VPC routing is enabled.  Default: false<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PubliclyAccessible</i> The pending or in-progress change of the ability to connect to the cluster from the public network.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterIdentifier</i> The pending or in-progress change of the new identifier for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomatedSnapshotRetentionPeriod</i> The pending or in-progress change of the automated snapshot retention period.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterVersion</i> The pending or in-progress change of the service version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterType</i> The pending or in-progress change of the cluster type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfNodes</i> The pending or in-progress change of the number of nodes in the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NodeType</i> The pending or in-progress change of the cluster's node type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MasterUserPassword</i> The pending or in-progress change of the master user password for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedTimeToCompletionInSeconds</i> The estimate of the time remaining before the restore will complete. Returns 0 for a completed restore.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> The amount of time an in-progress restore has been running, or the amount of time it took a completed restore to finish.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ProgressInMegaBytes</i> The number of megabytes that have been transferred from snapshot storage.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotSizeInMegaBytes</i> The size of the set of snapshot data used to restore the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentRestoreRateInMegaBytesPerSecond</i> The number of megabytes per second being transferred from the backup storage. Returns the average rate for a completed backup.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the restore action. Returns starting, restoring, completed, or failed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElapsedTimeInSeconds</i> Describes the number of seconds that have elapsed during the data transfer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EstimatedTimeToCompletionInSeconds</i> Describes the estimated number of seconds remaining to complete the transfer.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTransferredInMegaBytes</i> Describes the total amount of data that has been transfered in MB's.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalDataInMegaBytes</i> Describes the total amount of data to be transfered in megabytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentRateInMegaBytesPerSecond</i> Describes the data transfer rate in MB's per second.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Describes the status of the cluster. While the transfer is in progress the status is <code>transferringdata</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Reports whether the Amazon Redshift cluster has finished applying any HSM settings changes specified in a modify cluster command. Values: active, applying<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmConfigurationIdentifier</i> Specifies the name of the HSM configuration that contains the information the Amazon Redshift cluster can use to retrieve and store keys in an HSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HsmClientCertificateIdentifier</i> Specifies the name of the HSM client certificate the Amazon Redshift cluster uses to retrieve the data encryption keys stored in an HSM.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnapshotCopyGrantName</i> The name of the snapshot copy grant.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ManualSnapshotRetentionPeriod</i> The number of days that automated snapshots are retained in the destination region after they are copied from a source region. If the value is -1, the manual snapshot is retained indefinitely.  The value must be either -1 or an integer between 1 and 3,653.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RetentionPeriod</i> The number of days that automated snapshots are retained in the destination region after they are copied from a source region.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DestinationRegion</i> The destination region that snapshots are automatically copied to when cross-region snapshot copy is enabled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the elastic IP (EIP) address.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ElasticIp</i> The elastic IP (EIP) address for the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowCancelResize</i> A boolean value indicating if the resize operation can be cancelled.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResizeType</i> Returns the value <code>ClassicResize</code>.<br/>        """
        pass



class Rekognition(BaseClient):
    """This is the Amazon Rekognition API reference."""


    class ListCollectionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_collections` for valid parameters"""
            pass
    class ListFacesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_faces` for valid parameters"""
            pass
    class ListStreamProcessorsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_stream_processors` for valid parameters"""
            pass


    class CompareFacesRequest(Mapping):
        pass
    
    class CompareFacesResponse(Mapping):
        pass
    
    class Image(Mapping):
        pass
    
    class Percent(object):
        pass
    
    class CreateCollectionRequest(Mapping):
        pass
    
    class CreateCollectionResponse(Mapping):
        pass
    
    class CreateStreamProcessorRequest(Mapping):
        pass
    
    class CreateStreamProcessorResponse(Mapping):
        pass
    
    class StreamProcessorInput(Mapping):
        pass
    
    class StreamProcessorOutput(Mapping):
        pass
    
    class StreamProcessorSettings(Mapping):
        pass
    
    class DeleteCollectionRequest(Mapping):
        pass
    
    class DeleteCollectionResponse(Mapping):
        pass
    
    class DeleteFacesRequest(Mapping):
        pass
    
    class DeleteFacesResponse(Mapping):
        pass
    
    class FaceId(object):
        pass
    
    class DeleteStreamProcessorRequest(Mapping):
        pass
    
    class DeleteStreamProcessorResponse(Mapping):
        pass
    
    class DescribeCollectionRequest(Mapping):
        pass
    
    class DescribeCollectionResponse(Mapping):
        pass
    
    class DescribeStreamProcessorRequest(Mapping):
        pass
    
    class DescribeStreamProcessorResponse(Mapping):
        pass
    
    class DetectFacesRequest(Mapping):
        pass
    
    class DetectFacesResponse(Mapping):
        pass
    
    class Attribute(object):
        pass
    
    class DetectLabelsRequest(Mapping):
        pass
    
    class DetectLabelsResponse(Mapping):
        pass
    
    class DetectModerationLabelsRequest(Mapping):
        pass
    
    class DetectModerationLabelsResponse(Mapping):
        pass
    
    class DetectTextRequest(Mapping):
        pass
    
    class DetectTextResponse(Mapping):
        pass
    
    class GetCelebrityInfoRequest(Mapping):
        pass
    
    class GetCelebrityInfoResponse(Mapping):
        pass
    
    class GetCelebrityRecognitionRequest(Mapping):
        pass
    
    class GetCelebrityRecognitionResponse(Mapping):
        pass
    
    class GetContentModerationRequest(Mapping):
        pass
    
    class GetContentModerationResponse(Mapping):
        pass
    
    class GetFaceDetectionRequest(Mapping):
        pass
    
    class GetFaceDetectionResponse(Mapping):
        pass
    
    class GetFaceSearchRequest(Mapping):
        pass
    
    class GetFaceSearchResponse(Mapping):
        pass
    
    class GetLabelDetectionRequest(Mapping):
        pass
    
    class GetLabelDetectionResponse(Mapping):
        pass
    
    class GetPersonTrackingRequest(Mapping):
        pass
    
    class GetPersonTrackingResponse(Mapping):
        pass
    
    class IndexFacesRequest(Mapping):
        pass
    
    class IndexFacesResponse(Mapping):
        pass
    
    class ListCollectionsRequest(Mapping):
        pass
    
    class ListCollectionsResponse(Mapping):
        pass
    
    class ListFacesRequest(Mapping):
        pass
    
    class ListFacesResponse(Mapping):
        pass
    
    class ListStreamProcessorsRequest(Mapping):
        pass
    
    class ListStreamProcessorsResponse(Mapping):
        pass
    
    class RecognizeCelebritiesRequest(Mapping):
        pass
    
    class RecognizeCelebritiesResponse(Mapping):
        pass
    
    class SearchFacesRequest(Mapping):
        pass
    
    class SearchFacesResponse(Mapping):
        pass
    
    class SearchFacesByImageRequest(Mapping):
        pass
    
    class SearchFacesByImageResponse(Mapping):
        pass
    
    class StartCelebrityRecognitionRequest(Mapping):
        pass
    
    class StartCelebrityRecognitionResponse(Mapping):
        pass
    
    class Video(Mapping):
        pass
    
    class NotificationChannel(Mapping):
        pass
    
    class StartContentModerationRequest(Mapping):
        pass
    
    class StartContentModerationResponse(Mapping):
        pass
    
    class StartFaceDetectionRequest(Mapping):
        pass
    
    class StartFaceDetectionResponse(Mapping):
        pass
    
    class StartFaceSearchRequest(Mapping):
        pass
    
    class StartFaceSearchResponse(Mapping):
        pass
    
    class StartLabelDetectionRequest(Mapping):
        pass
    
    class StartLabelDetectionResponse(Mapping):
        pass
    
    class StartPersonTrackingRequest(Mapping):
        pass
    
    class StartPersonTrackingResponse(Mapping):
        pass
    
    class StartStreamProcessorRequest(Mapping):
        pass
    
    class StartStreamProcessorResponse(Mapping):
        pass
    
    class StopStreamProcessorRequest(Mapping):
        pass
    
    class StopStreamProcessorResponse(Mapping):
        pass
    
    def compare_faces(self, TargetImage: Image, SourceImage: Image, SimilarityThreshold: Percent=None) -> CompareFacesResponse:
        """Compares a face in the <i>source</i> input image with each of the 100 largest faces detected in the <i>target</i> input image.  <note>  If the source image contains multiple faces, the service detects the largest face and compares it with each face detected in the target image.  </note> You pass the input and target images either as base64-encoded image bytes or as references to images in an Amazon S3 bucket. If you use the AWS CLI to call Amazon Rekognition operations, passing image bytes isn't supported. The image must be formatted as a PNG or JPEG file.  In response, the operation returns an array of face matches ordered by similarity score in descending order. For each face match, the response provides a bounding box of the face, facial landmarks, pose details (pitch, role, and yaw), quality (brightness and sharpness), and confidence value (indicating the level of confidence that the bounding box contains a face). The response also provides a similarity score, which indicates how closely the faces match.  <note> By default, only faces with a similarity score of greater than or equal to 80% are returned in the response. You can change this value by specifying the <code>SimilarityThreshold</code> parameter. </note>  <code>CompareFaces</code> also returns an array of faces that don't match the source image. For each face, it returns a bounding box, confidence value, landmarks, pose details, and quality. The response also returns information about the face in the source image, including the bounding box of the face and confidence value. If the image doesn't contain Exif metadata, <code>CompareFaces</code> returns orientation information for the source and target images. Use these values to display the images with the correct image orientation. If no faces are detected in the source or target images, <code>CompareFaces</code> returns an <code>InvalidParameterException</code> error.  <note>  This is a stateless API operation. That is, data returned by this operation doesn't persist. </note> For an example, see Comparing Faces in Images in the Amazon Rekognition Developer Guide. This operation requires permissions to perform the <code>rekognition:CompareFaces</code> action.

        :param self:
        :param SimilarityThreshold:  The minimum level of confidence in the face matches that a match must meet to be included in the <code>FaceMatches</code> array.<br/>TargetImage:  The target image as base64-encoded bytes or an S3 object. If you use the AWS CLI to call Amazon Rekognition operations, passing base64-encoded image bytes is not supported.  If you are using an AWS SDK to call Amazon Rekognition, you might not need to base64-encode image bytes passed using the <code>Bytes</code> field. For more information, see Images in the Amazon Rekognition developer guide.:<br/>SourceImage:  The input image as base64-encoded bytes or an S3 object. If you use the AWS CLI to call Amazon Rekognition operations, passing base64-encoded image bytes is not supported.  If you are using an AWS SDK to call Amazon Rekognition, you might not need to base64-encode image bytes passed using the <code>Bytes</code> field. For more information, see Images in the Amazon Rekognition developer guide.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Object</i> Identifies an S3 object as the image source.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Object</b>: Identifies an S3 object as the image source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Bytes</i> Blob of image bytes up to 5 MBs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> If the bucket is versioning enabled, you can specify the object version. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> S3 object key name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> Name of the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Object</i> Identifies an S3 object as the image source.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Object</b>: Identifies an S3 object as the image source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Bytes</i> Blob of image bytes up to 5 MBs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> If the bucket is versioning enabled, you can specify the object version. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> S3 object key name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> Name of the S3 bucket.<br/>
        :return: TargetImageOrientationCorrection:  The value of <code>TargetImageOrientationCorrection</code> is always null. If the input image is in .jpeg format, it might contain exchangeable image file format (Exif) metadata that includes the image's orientation. Amazon Rekognition uses this orientation information to perform image correction. The bounding box coordinates are translated to represent object locations after the orientation information in the Exif metadata is used to correct the image orientation. Images in .png format don't contain Exif metadata. Amazon Rekognition doesn’t perform image correction for images in .png format and .jpeg images without orientation information in the image Exif metadata. The bounding box coordinates aren't translated and represent the object locations before the image is rotated. <br/>SourceImageOrientationCorrection:  The value of <code>SourceImageOrientationCorrection</code> is always null. If the input image is in .jpeg format, it might contain exchangeable image file format (Exif) metadata that includes the image's orientation. Amazon Rekognition uses this orientation information to perform image correction. The bounding box coordinates are translated to represent object locations after the orientation information in the Exif metadata is used to correct the image orientation. Images in .png format don't contain Exif metadata. Amazon Rekognition doesn’t perform image correction for images in .png format and .jpeg images without orientation information in the image Exif metadata. The bounding box coordinates aren't translated and represent the object locations before the image is rotated. <br/>UnmatchedFaces:  An array of faces in the target image that did not match the source image face.<br/>FaceMatches:  An array of faces in the target image that match the source image face. Each <code>CompareFacesMatch</code> object provides the bounding box, the confidence level that the bounding box contains a face, and the similarity score for the face in the bounding box and the face in the source image.<br/>SourceImageFace:  The face in the source image that was used for comparison.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Confidence</i> Confidence level that the selected bounding box contains a face.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BoundingBox</i> Bounding box of the face.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>BoundingBox</b>: Bounding box of the face.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Top</i> Top coordinate of the bounding box as a ratio of overall image height.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Left</i> Left coordinate of the bounding box as a ratio of overall image width.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Height</i> Height of the bounding box as a ratio of the overall image height.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Width</i> Width of the bounding box as a ratio of the overall image width.<br/>        """
        pass

    def create_collection(self, CollectionId: str) -> CreateCollectionResponse:
        """Creates a collection in an AWS Region. You can add faces to the collection using the <a>IndexFaces</a> operation.  For example, you might create collections, one for each of your application users. A user can then index faces using the <code>IndexFaces</code> operation and persist results in a specific collection. Then, a user can search the collection for faces in the user-specific container.  When you create a collection, it is associated with the latest version of the face model version. <note> Collection names are case-sensitive. </note> This operation requires permissions to perform the <code>rekognition:CreateCollection</code> action.

        :param self:
        :param CollectionId:  ID for the collection that you are creating.<br/>
        :return: FaceModelVersion:  Version number of the face detection model associated with the collection you are creating.<br/>CollectionArn:  Amazon Resource Name (ARN) of the collection. You can use this to manage permissions on your resources. <br/>StatusCode:  HTTP status code indicating the result of the operation.<br/>        """
        pass

    def create_stream_processor(self, RoleArn: str, Settings: StreamProcessorSettings, Name: str, Output: StreamProcessorOutput, Input: StreamProcessorInput) -> CreateStreamProcessorResponse:
        """Creates an Amazon Rekognition stream processor that you can use to detect and recognize faces in a streaming video. Amazon Rekognition Video is a consumer of live video from Amazon Kinesis Video Streams. Amazon Rekognition Video sends analysis results to Amazon Kinesis Data Streams. You provide as input a Kinesis video stream (<code>Input</code>) and a Kinesis data stream (<code>Output</code>) stream. You also specify the face recognition criteria in <code>Settings</code>. For example, the collection containing faces that you want to recognize. Use <code>Name</code> to assign an identifier for the stream processor. You use <code>Name</code> to manage the stream processor. For example, you can start processing the source video by calling <a>StartStreamProcessor</a> with the <code>Name</code> field.  After you have finished analyzing a streaming video, use <a>StopStreamProcessor</a> to stop processing. You can delete the stream processor by calling <a>DeleteStreamProcessor</a>.

        :param self:
        :param RoleArn:  ARN of the IAM role that allows access to the stream processor.<br/>Settings:  Face recognition input parameters to be used by the stream processor. Includes the collection to use for face recognition and the face attributes to detect.:<br/>Name:  An identifier you assign to the stream processor. You can use <code>Name</code> to manage the stream processor. For example, you can get the current status of the stream processor by calling <a>DescribeStreamProcessor</a>. <code>Name</code> is idempotent. <br/>Output:  Kinesis data stream stream to which Amazon Rekognition Video puts the analysis results. If you are using the AWS CLI, the parameter name is <code>StreamProcessorOutput</code>.:<br/>Input:  Kinesis video stream stream that provides the source streaming video. If you are using the AWS CLI, the parameter name is <code>StreamProcessorInput</code>.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KinesisVideoStream</i> The Kinesis video stream input stream for the source streaming video.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KinesisVideoStream</b>: The Kinesis video stream input stream for the source streaming video.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> ARN of the Kinesis video stream stream that streams the source video.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KinesisDataStream</i> The Amazon Kinesis Data Streams stream to which the Amazon Rekognition stream processor streams the analysis results.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KinesisDataStream</b>: The Amazon Kinesis Data Streams stream to which the Amazon Rekognition stream processor streams the analysis results.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> ARN of the output Amazon Kinesis Data Streams stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FaceSearch</i> Face search settings to use on a streaming video. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>FaceSearch</b>: Face search settings to use on a streaming video. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FaceMatchThreshold</i> Minimum face match confidence score that must be met to return a result for a recognized face. Default is 70. 0 is the lowest confidence. 100 is the highest confidence.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CollectionId</i> The ID of a collection that contains faces that you want to search for.<br/>
        :return: StreamProcessorArn:  ARN for the newly create stream processor.<br/>        """
        pass

    def delete_collection(self, CollectionId: str) -> DeleteCollectionResponse:
        """Deletes the specified collection. Note that this operation removes all faces in the collection. For an example, see <a>delete-collection-procedure</a>. This operation requires permissions to perform the <code>rekognition:DeleteCollection</code> action.

        :param self:
        :param CollectionId:  ID of the collection to delete.<br/>
        :return: StatusCode:  HTTP status code that indicates the result of the operation.<br/>        """
        pass

    def delete_faces(self, FaceIds: List[FaceId], CollectionId: str) -> DeleteFacesResponse:
        """Deletes faces from a collection. You specify a collection ID and an array of face IDs to remove from the collection. This operation requires permissions to perform the <code>rekognition:DeleteFaces</code> action.

        :param self:
        :param FaceIds:  An array of face IDs to delete.<br/>CollectionId:  Collection from which to remove the specific faces.<br/>
        :return: DeletedFaces:  An array of strings (face IDs) of the faces that were deleted.<br/>        """
        pass

    def delete_stream_processor(self, Name: str) -> DeleteStreamProcessorResponse:
        """Deletes the stream processor identified by <code>Name</code>. You assign the value for <code>Name</code> when you create the stream processor with <a>CreateStreamProcessor</a>. You might not be able to use the same name for a stream processor for a few seconds after calling <code>DeleteStreamProcessor</code>.

        :param self:
        :param Name:  The name of the stream processor you want to delete.<br/>
        :return:         """
        pass

    def describe_collection(self, CollectionId: str) -> DescribeCollectionResponse:
        """Describes the specified collection. You can use <code>DescribeCollection</code> to get information, such as the number of faces indexed into a collection and the version of the model used by the collection for face detection. For more information, see Describing a Collection in the Amazon Rekognition Developer Guide.

        :param self:
        :param CollectionId:  The ID of the collection to describe.<br/>
        :return: CreationTimestamp:  The number of milliseconds since the Unix epoch time until the creation of the collection. The Unix epoch time is 00:00:00 Coordinated Universal Time (UTC), Thursday, 1 January 1970.<br/>CollectionARN:  The Amazon Resource Name (ARN) of the collection.<br/>FaceModelVersion:  The version of the face model that's used by the collection for face detection. For more information, see Model Versioning in the Amazon Rekognition Developer Guide.<br/>FaceCount:  The number of faces that are indexed into the collection. To index faces into a collection, use <a>IndexFaces</a>.<br/>        """
        pass

    def describe_stream_processor(self, Name: str) -> DescribeStreamProcessorResponse:
        """Provides information about a stream processor created by <a>CreateStreamProcessor</a>. You can get information about the input and output streams, the input parameters for the face recognition being performed, and the current status of the stream processor.

        :param self:
        :param Name:  Name of the stream processor for which you want information.<br/>
        :return: Settings:  Face recognition input parameters that are being used by the stream processor. Includes the collection to use for face recognition and the face attributes to detect.:<br/>RoleArn:  ARN of the IAM role that allows access to the stream processor.<br/>Output:  Kinesis data stream to which Amazon Rekognition Video puts the analysis results.:<br/>Input:  Kinesis video stream that provides the source streaming video.:<br/>LastUpdateTimestamp:  The time, in Unix format, the stream processor was last updated. For example, when the stream processor moves from a running state to a failed state, or when the user starts or stops the stream processor.<br/>CreationTimestamp:  Date and time the stream processor was created<br/>StatusMessage:  Detailed status message about the stream processor.<br/>Status:  Current status of the stream processor.<br/>StreamProcessorArn:  ARN of the stream processor.<br/>Name:  Name of the stream processor. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KinesisVideoStream</i> The Kinesis video stream input stream for the source streaming video.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KinesisVideoStream</b>: The Kinesis video stream input stream for the source streaming video.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> ARN of the Kinesis video stream stream that streams the source video.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KinesisDataStream</i> The Amazon Kinesis Data Streams stream to which the Amazon Rekognition stream processor streams the analysis results.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KinesisDataStream</b>: The Amazon Kinesis Data Streams stream to which the Amazon Rekognition stream processor streams the analysis results.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> ARN of the output Amazon Kinesis Data Streams stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FaceSearch</i> Face search settings to use on a streaming video. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>FaceSearch</b>: Face search settings to use on a streaming video. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FaceMatchThreshold</i> Minimum face match confidence score that must be met to return a result for a recognized face. Default is 70. 0 is the lowest confidence. 100 is the highest confidence.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CollectionId</i> The ID of a collection that contains faces that you want to search for.<br/>        """
        pass

    def detect_faces(self, Image: Image, Attributes: List[Attribute]=None) -> DetectFacesResponse:
        """Detects faces within an image that is provided as input.  <code>DetectFaces</code> detects the 100 largest faces in the image. For each face detected, the operation returns face details. These details include a bounding box of the face, a confidence value (that the bounding box contains a face), and a fixed set of attributes such as facial landmarks (for example, coordinates of eye and mouth), gender, presence of beard, sunglasses, and so on.  The face-detection algorithm is most effective on frontal faces. For non-frontal or obscured faces, the algorithm might not detect the faces or might detect faces with lower confidence.  You pass the input image either as base64-encoded image bytes or as a reference to an image in an Amazon S3 bucket. If you use the to call Amazon Rekognition operations, passing image bytes is not supported. The image must be either a PNG or JPEG formatted file.  <note> This is a stateless API operation. That is, the operation does not persist any data. </note> This operation requires permissions to perform the <code>rekognition:DetectFaces</code> action. 

        :param self:
        :param Attributes:  An array of facial attributes you want to be returned. This can be the default list of attributes or all attributes. If you don't specify a value for <code>Attributes</code> or if you specify <code>["DEFAULT"]</code>, the API returns the following subset of facial attributes: <code>BoundingBox</code>, <code>Confidence</code>, <code>Pose</code>, <code>Quality</code>, and <code>Landmarks</code>. If you provide <code>["ALL"]</code>, all facial attributes are returned, but the operation takes longer to complete. If you provide both, <code>["ALL", "DEFAULT"]</code>, the service uses a logical AND operator to determine which attributes to return (in this case, all attributes). <br/>Image:  The input image as base64-encoded bytes or an S3 object. If you use the AWS CLI to call Amazon Rekognition operations, passing base64-encoded image bytes is not supported.  If you are using an AWS SDK to call Amazon Rekognition, you might not need to base64-encode image bytes passed using the <code>Bytes</code> field. For more information, see Images in the Amazon Rekognition developer guide.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Object</i> Identifies an S3 object as the image source.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Object</b>: Identifies an S3 object as the image source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Bytes</i> Blob of image bytes up to 5 MBs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> If the bucket is versioning enabled, you can specify the object version. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> S3 object key name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> Name of the S3 bucket.<br/>
        :return: OrientationCorrection:  The value of <code>OrientationCorrection</code> is always null. If the input image is in .jpeg format, it might contain exchangeable image file format (Exif) metadata that includes the image's orientation. Amazon Rekognition uses this orientation information to perform image correction. The bounding box coordinates are translated to represent object locations after the orientation information in the Exif metadata is used to correct the image orientation. Images in .png format don't contain Exif metadata. Amazon Rekognition doesn’t perform image correction for images in .png format and .jpeg images without orientation information in the image Exif metadata. The bounding box coordinates aren't translated and represent the object locations before the image is rotated. <br/>FaceDetails:  Details of each face found in the image. <br/>        """
        pass

    def detect_labels(self, Image: Image, MaxLabels: int=None, MinConfidence: Percent=None) -> DetectLabelsResponse:
        """Detects instances of real-world entities within an image (JPEG or PNG) provided as input. This includes objects like flower, tree, and table; events like wedding, graduation, and birthday party; and concepts like landscape, evening, and nature.  For an example, see Analyzing Images Stored in an Amazon S3 Bucket in the Amazon Rekognition Developer Guide. <note>  <code>DetectLabels</code> does not support the detection of activities. However, activity detection is supported for label detection in videos. For more information, see StartLabelDetection in the Amazon Rekognition Developer Guide. </note> You pass the input image as base64-encoded image bytes or as a reference to an image in an Amazon S3 bucket. If you use the AWS CLI to call Amazon Rekognition operations, passing image bytes is not supported. The image must be either a PNG or JPEG formatted file.   For each object, scene, and concept the API returns one or more labels. Each label provides the object name, and the level of confidence that the image contains the object. For example, suppose the input image has a lighthouse, the sea, and a rock. The response includes all three labels, one for each object.   <code>{Name: lighthouse, Confidence: 98.4629}</code>   <code>{Name: rock,Confidence: 79.2097}</code>   <code> {Name: sea,Confidence: 75.061}</code>  In the preceding example, the operation returns one label for each of the three objects. The operation can also return multiple labels for the same object in the image. For example, if the input image shows a flower (for example, a tulip), the operation might return the following three labels.   <code>{Name: flower,Confidence: 99.0562}</code>   <code>{Name: plant,Confidence: 99.0562}</code>   <code>{Name: tulip,Confidence: 99.0562}</code>  In this example, the detection algorithm more precisely identifies the flower as a tulip. In response, the API returns an array of labels. In addition, the response also includes the orientation correction. Optionally, you can specify <code>MinConfidence</code> to control the confidence threshold for the labels returned. The default is 55%. You can also add the <code>MaxLabels</code> parameter to limit the number of labels returned.  <note> If the object detected is a person, the operation doesn't provide the same facial details that the <a>DetectFaces</a> operation provides. </note>  <code>DetectLabels</code> returns bounding boxes for instances of common object labels in an array of <a>Instance</a> objects. An <code>Instance</code> object contains a <a>BoundingBox</a> object, for the location of the label on the image. It also includes the confidence by which the bounding box was detected.  <code>DetectLabels</code> also returns a hierarchical taxonomy of detected labels. For example, a detected car might be assigned the label <i>car</i>. The label <i>car</i> has two parent labels: <i>Vehicle</i> (its parent) and <i>Transportation</i> (its grandparent). The response returns the entire list of ancestors for a label. Each ancestor is a unique label in the response. In the previous example, <i>Car</i>, <i>Vehicle</i>, and <i>Transportation</i> are returned as unique labels in the response.  This is a stateless API operation. That is, the operation does not persist any data. This operation requires permissions to perform the <code>rekognition:DetectLabels</code> action. 

        :param self:
        :param MinConfidence:  Specifies the minimum confidence level for the labels to return. Amazon Rekognition doesn't return any labels with confidence lower than this specified value. If <code>MinConfidence</code> is not specified, the operation returns labels with a confidence values greater than or equal to 55 percent.<br/>MaxLabels:  Maximum number of labels you want the service to return in the response. The service returns the specified number of highest confidence labels. <br/>Image:  The input image as base64-encoded bytes or an S3 object. If you use the AWS CLI to call Amazon Rekognition operations, passing image bytes is not supported. Images stored in an S3 Bucket do not need to be base64-encoded. If you are using an AWS SDK to call Amazon Rekognition, you might not need to base64-encode image bytes passed using the <code>Bytes</code> field. For more information, see Images in the Amazon Rekognition developer guide.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Object</i> Identifies an S3 object as the image source.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Object</b>: Identifies an S3 object as the image source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Bytes</i> Blob of image bytes up to 5 MBs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> If the bucket is versioning enabled, you can specify the object version. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> S3 object key name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> Name of the S3 bucket.<br/>
        :return: LabelModelVersion:  Version number of the label detection model that was used to detect labels.<br/>OrientationCorrection:  The value of <code>OrientationCorrection</code> is always null. If the input image is in .jpeg format, it might contain exchangeable image file format (Exif) metadata that includes the image's orientation. Amazon Rekognition uses this orientation information to perform image correction. The bounding box coordinates are translated to represent object locations after the orientation information in the Exif metadata is used to correct the image orientation. Images in .png format don't contain Exif metadata. Amazon Rekognition doesn’t perform image correction for images in .png format and .jpeg images without orientation information in the image Exif metadata. The bounding box coordinates aren't translated and represent the object locations before the image is rotated. <br/>Labels:  An array of labels for the real-world objects detected. <br/>        """
        pass

    def detect_moderation_labels(self, Image: Image, MinConfidence: Percent=None) -> DetectModerationLabelsResponse:
        """Detects explicit or suggestive adult content in a specified JPEG or PNG format image. Use <code>DetectModerationLabels</code> to moderate images depending on your requirements. For example, you might want to filter images that contain nudity, but not images containing suggestive content. To filter images, use the labels returned by <code>DetectModerationLabels</code> to determine which types of content are appropriate. For information about moderation labels, see Detecting Unsafe Content in the Amazon Rekognition Developer Guide. You pass the input image either as base64-encoded image bytes or as a reference to an image in an Amazon S3 bucket. If you use the AWS CLI to call Amazon Rekognition operations, passing image bytes is not supported. The image must be either a PNG or JPEG formatted file. 

        :param self:
        :param MinConfidence:  Specifies the minimum confidence level for the labels to return. Amazon Rekognition doesn't return any labels with a confidence level lower than this specified value. If you don't specify <code>MinConfidence</code>, the operation returns labels with confidence values greater than or equal to 50 percent.<br/>Image:  The input image as base64-encoded bytes or an S3 object. If you use the AWS CLI to call Amazon Rekognition operations, passing base64-encoded image bytes is not supported.  If you are using an AWS SDK to call Amazon Rekognition, you might not need to base64-encode image bytes passed using the <code>Bytes</code> field. For more information, see Images in the Amazon Rekognition developer guide.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Object</i> Identifies an S3 object as the image source.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Object</b>: Identifies an S3 object as the image source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Bytes</i> Blob of image bytes up to 5 MBs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> If the bucket is versioning enabled, you can specify the object version. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> S3 object key name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> Name of the S3 bucket.<br/>
        :return: ModerationModelVersion:  Version number of the moderation detection model that was used to detect unsafe content.<br/>ModerationLabels:  Array of detected Moderation labels and the time, in millseconds from the start of the video, they were detected.<br/>        """
        pass

    def detect_text(self, Image: Image) -> DetectTextResponse:
        """Detects text in the input image and converts it into machine-readable text. Pass the input image as base64-encoded image bytes or as a reference to an image in an Amazon S3 bucket. If you use the AWS CLI to call Amazon Rekognition operations, you must pass it as a reference to an image in an Amazon S3 bucket. For the AWS CLI, passing image bytes is not supported. The image must be either a .png or .jpeg formatted file.  The <code>DetectText</code> operation returns text in an array of <a>TextDetection</a> elements, <code>TextDetections</code>. Each <code>TextDetection</code> element provides information about a single word or line of text that was detected in the image.  A word is one or more ISO basic latin script characters that are not separated by spaces. <code>DetectText</code> can detect up to 50 words in an image. A line is a string of equally spaced words. A line isn't necessarily a complete sentence. For example, a driver's license number is detected as a line. A line ends when there is no aligned text after it. Also, a line ends when there is a large gap between words, relative to the length of the words. This means, depending on the gap between words, Amazon Rekognition may detect multiple lines in text aligned in the same direction. Periods don't represent the end of a line. If a sentence spans multiple lines, the <code>DetectText</code> operation returns multiple lines. To determine whether a <code>TextDetection</code> element is a line of text or a word, use the <code>TextDetection</code> object <code>Type</code> field.  To be detected, text must be within +/- 90 degrees orientation of the horizontal axis. For more information, see DetectText in the Amazon Rekognition Developer Guide.

        :param self:
        :param Image:  The input image as base64-encoded bytes or an Amazon S3 object. If you use the AWS CLI to call Amazon Rekognition operations, you can't pass image bytes.  If you are using an AWS SDK to call Amazon Rekognition, you might not need to base64-encode image bytes passed using the <code>Bytes</code> field. For more information, see Images in the Amazon Rekognition developer guide.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Object</i> Identifies an S3 object as the image source.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Object</b>: Identifies an S3 object as the image source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Bytes</i> Blob of image bytes up to 5 MBs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> If the bucket is versioning enabled, you can specify the object version. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> S3 object key name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> Name of the S3 bucket.<br/>
        :return: TextDetections:  An array of text that was detected in the input image.<br/>        """
        pass

    def get_celebrity_info(self, Id: str) -> GetCelebrityInfoResponse:
        """Gets the name and additional information about a celebrity based on his or her Amazon Rekognition ID. The additional information is returned as an array of URLs. If there is no additional information about the celebrity, this list is empty. For more information, see Recognizing Celebrities in an Image in the Amazon Rekognition Developer Guide. This operation requires permissions to perform the <code>rekognition:GetCelebrityInfo</code> action. 

        :param self:
        :param Id:  The ID for the celebrity. You get the celebrity ID from a call to the <a>RecognizeCelebrities</a> operation, which recognizes celebrities in an image. <br/>
        :return: Name:  The name of the celebrity.<br/>Urls:  An array of URLs pointing to additional celebrity information. <br/>        """
        pass

    def get_celebrity_recognition(self, JobId: str, MaxResults: int=None, NextToken: str=None, SortBy: str=None) -> GetCelebrityRecognitionResponse:
        """Gets the celebrity recognition results for a Amazon Rekognition Video analysis started by <a>StartCelebrityRecognition</a>. Celebrity recognition in a video is an asynchronous operation. Analysis is started by a call to <a>StartCelebrityRecognition</a> which returns a job identifier (<code>JobId</code>). When the celebrity recognition operation finishes, Amazon Rekognition Video publishes a completion status to the Amazon Simple Notification Service topic registered in the initial call to <code>StartCelebrityRecognition</code>. To get the results of the celebrity recognition analysis, first check that the status value published to the Amazon SNS topic is <code>SUCCEEDED</code>. If so, call <code>GetCelebrityDetection</code> and pass the job identifier (<code>JobId</code>) from the initial call to <code>StartCelebrityDetection</code>.  For more information, see Working With Stored Videos in the Amazon Rekognition Developer Guide.  <code>GetCelebrityRecognition</code> returns detected celebrities and the time(s) they are detected in an array (<code>Celebrities</code>) of <a>CelebrityRecognition</a> objects. Each <code>CelebrityRecognition</code> contains information about the celebrity in a <a>CelebrityDetail</a> object and the time, <code>Timestamp</code>, the celebrity was detected.  <note>  <code>GetCelebrityRecognition</code> only returns the default facial attributes (<code>BoundingBox</code>, <code>Confidence</code>, <code>Landmarks</code>, <code>Pose</code>, and <code>Quality</code>). The other facial attributes listed in the <code>Face</code> object of the following response syntax are not returned. For more information, see FaceDetail in the Amazon Rekognition Developer Guide.  </note> By default, the <code>Celebrities</code> array is sorted by time (milliseconds from the start of the video). You can also sort the array by celebrity by specifying the value <code>ID</code> in the <code>SortBy</code> input parameter. The <code>CelebrityDetail</code> object includes the celebrity identifer and additional information urls. If you don't store the additional information urls, you can get them later by calling <a>GetCelebrityInfo</a> with the celebrity identifer. No information is returned for faces not recognized as celebrities. Use MaxResults parameter to limit the number of labels returned. If there are more results than specified in <code>MaxResults</code>, the value of <code>NextToken</code> in the operation response contains a pagination token for getting the next set of results. To get the next page of results, call <code>GetCelebrityDetection</code> and populate the <code>NextToken</code> request parameter with the token value returned from the previous call to <code>GetCelebrityRecognition</code>.

        :param self:
        :param SortBy:  Sort to use for celebrities returned in <code>Celebrities</code> field. Specify <code>ID</code> to sort by the celebrity identifier, specify <code>TIMESTAMP</code> to sort by the time the celebrity was recognized.<br/>NextToken:  If the previous response was incomplete (because there is more recognized celebrities to retrieve), Amazon Rekognition Video returns a pagination token in the response. You can use this pagination token to retrieve the next set of celebrities. <br/>MaxResults:  Maximum number of results to return per paginated call. The largest value you can specify is 1000. If you specify a value greater than 1000, a maximum of 1000 results is returned. The default value is 1000.<br/>JobId:  Job identifier for the required celebrity recognition analysis. You can get the job identifer from a call to <code>StartCelebrityRecognition</code>.<br/>
        :return: Celebrities:  Array of celebrities recognized in the video.<br/>NextToken:  If the response is truncated, Amazon Rekognition Video returns this token that you can use in the subsequent request to retrieve the next set of celebrities.<br/>VideoMetadata:  Information about a video that Amazon Rekognition Video analyzed. <code>Videometadata</code> is returned in every page of paginated responses from a Amazon Rekognition Video operation.:<br/>StatusMessage:  If the job fails, <code>StatusMessage</code> provides a descriptive error message.<br/>JobStatus:  The current status of the celebrity recognition job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameWidth</i> Horizontal pixel dimension of the video.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameHeight</i> Vertical pixel dimension of the video.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameRate</i> Number of frames per second in the video.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> Format of the analyzed video. Possible values are MP4, MOV and AVI. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DurationMillis</i> Length of the video in milliseconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Codec</i> Type of compression used in the analyzed video. <br/>        """
        pass

    def get_content_moderation(self, JobId: str, MaxResults: int=None, NextToken: str=None, SortBy: str=None) -> GetContentModerationResponse:
        """Gets the content moderation analysis results for a Amazon Rekognition Video analysis started by <a>StartContentModeration</a>. Content moderation analysis of a video is an asynchronous operation. You start analysis by calling <a>StartContentModeration</a> which returns a job identifier (<code>JobId</code>). When analysis finishes, Amazon Rekognition Video publishes a completion status to the Amazon Simple Notification Service topic registered in the initial call to <code>StartContentModeration</code>. To get the results of the content moderation analysis, first check that the status value published to the Amazon SNS topic is <code>SUCCEEDED</code>. If so, call <code>GetContentModeration</code> and pass the job identifier (<code>JobId</code>) from the initial call to <code>StartContentModeration</code>.  For more information, see Working with Stored Videos in the Amazon Rekognition Devlopers Guide.  <code>GetContentModeration</code> returns detected content moderation labels, and the time they are detected, in an array, <code>ModerationLabels</code>, of <a>ContentModerationDetection</a> objects.  By default, the moderated labels are returned sorted by time, in milliseconds from the start of the video. You can also sort them by moderated label by specifying <code>NAME</code> for the <code>SortBy</code> input parameter.  Since video analysis can return a large number of results, use the <code>MaxResults</code> parameter to limit the number of labels returned in a single call to <code>GetContentModeration</code>. If there are more results than specified in <code>MaxResults</code>, the value of <code>NextToken</code> in the operation response contains a pagination token for getting the next set of results. To get the next page of results, call <code>GetContentModeration</code> and populate the <code>NextToken</code> request parameter with the value of <code>NextToken</code> returned from the previous call to <code>GetContentModeration</code>. For more information, see Detecting Unsafe Content in the Amazon Rekognition Developer Guide.

        :param self:
        :param SortBy:  Sort to use for elements in the <code>ModerationLabelDetections</code> array. Use <code>TIMESTAMP</code> to sort array elements by the time labels are detected. Use <code>NAME</code> to alphabetically group elements for a label together. Within each label group, the array element are sorted by detection confidence. The default sort is by <code>TIMESTAMP</code>.<br/>NextToken:  If the previous response was incomplete (because there is more data to retrieve), Amazon Rekognition returns a pagination token in the response. You can use this pagination token to retrieve the next set of content moderation labels.<br/>MaxResults:  Maximum number of results to return per paginated call. The largest value you can specify is 1000. If you specify a value greater than 1000, a maximum of 1000 results is returned. The default value is 1000.<br/>JobId:  The identifier for the content moderation job. Use <code>JobId</code> to identify the job in a subsequent call to <code>GetContentModeration</code>.<br/>
        :return: ModerationModelVersion:  Version number of the moderation detection model that was used to detect unsafe content.<br/>NextToken:  If the response is truncated, Amazon Rekognition Video returns this token that you can use in the subsequent request to retrieve the next set of moderation labels. <br/>ModerationLabels:  The detected moderation labels and the time(s) they were detected.<br/>VideoMetadata:  Information about a video that Amazon Rekognition analyzed. <code>Videometadata</code> is returned in every page of paginated responses from <code>GetContentModeration</code>. :<br/>StatusMessage:  If the job fails, <code>StatusMessage</code> provides a descriptive error message.<br/>JobStatus:  The current status of the content moderation job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameWidth</i> Horizontal pixel dimension of the video.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameHeight</i> Vertical pixel dimension of the video.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameRate</i> Number of frames per second in the video.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> Format of the analyzed video. Possible values are MP4, MOV and AVI. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DurationMillis</i> Length of the video in milliseconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Codec</i> Type of compression used in the analyzed video. <br/>        """
        pass

    def get_face_detection(self, JobId: str, MaxResults: int=None, NextToken: str=None) -> GetFaceDetectionResponse:
        """Gets face detection results for a Amazon Rekognition Video analysis started by <a>StartFaceDetection</a>. Face detection with Amazon Rekognition Video is an asynchronous operation. You start face detection by calling <a>StartFaceDetection</a> which returns a job identifier (<code>JobId</code>). When the face detection operation finishes, Amazon Rekognition Video publishes a completion status to the Amazon Simple Notification Service topic registered in the initial call to <code>StartFaceDetection</code>. To get the results of the face detection operation, first check that the status value published to the Amazon SNS topic is <code>SUCCEEDED</code>. If so, call <a>GetFaceDetection</a> and pass the job identifier (<code>JobId</code>) from the initial call to <code>StartFaceDetection</code>.  <code>GetFaceDetection</code> returns an array of detected faces (<code>Faces</code>) sorted by the time the faces were detected.  Use MaxResults parameter to limit the number of labels returned. If there are more results than specified in <code>MaxResults</code>, the value of <code>NextToken</code> in the operation response contains a pagination token for getting the next set of results. To get the next page of results, call <code>GetFaceDetection</code> and populate the <code>NextToken</code> request parameter with the token value returned from the previous call to <code>GetFaceDetection</code>.

        :param self:
        :param NextToken:  If the previous response was incomplete (because there are more faces to retrieve), Amazon Rekognition Video returns a pagination token in the response. You can use this pagination token to retrieve the next set of faces.<br/>MaxResults:  Maximum number of results to return per paginated call. The largest value you can specify is 1000. If you specify a value greater than 1000, a maximum of 1000 results is returned. The default value is 1000.<br/>JobId:  Unique identifier for the face detection job. The <code>JobId</code> is returned from <code>StartFaceDetection</code>.<br/>
        :return: Faces:  An array of faces detected in the video. Each element contains a detected face's details and the time, in milliseconds from the start of the video, the face was detected. <br/>NextToken:  If the response is truncated, Amazon Rekognition returns this token that you can use in the subsequent request to retrieve the next set of faces. <br/>VideoMetadata:  Information about a video that Amazon Rekognition Video analyzed. <code>Videometadata</code> is returned in every page of paginated responses from a Amazon Rekognition video operation.:<br/>StatusMessage:  If the job fails, <code>StatusMessage</code> provides a descriptive error message.<br/>JobStatus:  The current status of the face detection job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameWidth</i> Horizontal pixel dimension of the video.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameHeight</i> Vertical pixel dimension of the video.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameRate</i> Number of frames per second in the video.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> Format of the analyzed video. Possible values are MP4, MOV and AVI. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DurationMillis</i> Length of the video in milliseconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Codec</i> Type of compression used in the analyzed video. <br/>        """
        pass

    def get_face_search(self, JobId: str, MaxResults: int=None, NextToken: str=None, SortBy: str=None) -> GetFaceSearchResponse:
        """Gets the face search results for Amazon Rekognition Video face search started by <a>StartFaceSearch</a>. The search returns faces in a collection that match the faces of persons detected in a video. It also includes the time(s) that faces are matched in the video. Face search in a video is an asynchronous operation. You start face search by calling to <a>StartFaceSearch</a> which returns a job identifier (<code>JobId</code>). When the search operation finishes, Amazon Rekognition Video publishes a completion status to the Amazon Simple Notification Service topic registered in the initial call to <code>StartFaceSearch</code>. To get the search results, first check that the status value published to the Amazon SNS topic is <code>SUCCEEDED</code>. If so, call <code>GetFaceSearch</code> and pass the job identifier (<code>JobId</code>) from the initial call to <code>StartFaceSearch</code>. For more information, see Searching Faces in a Collection in the Amazon Rekognition Developer Guide. The search results are retured in an array, <code>Persons</code>, of <a>PersonMatch</a> objects. Each<code>PersonMatch</code> element contains details about the matching faces in the input collection, person information (facial attributes, bounding boxes, and person identifer) for the matched person, and the time the person was matched in the video. <note>  <code>GetFaceSearch</code> only returns the default facial attributes (<code>BoundingBox</code>, <code>Confidence</code>, <code>Landmarks</code>, <code>Pose</code>, and <code>Quality</code>). The other facial attributes listed in the <code>Face</code> object of the following response syntax are not returned. For more information, see FaceDetail in the Amazon Rekognition Developer Guide.  </note> By default, the <code>Persons</code> array is sorted by the time, in milliseconds from the start of the video, persons are matched. You can also sort by persons by specifying <code>INDEX</code> for the <code>SORTBY</code> input parameter.

        :param self:
        :param SortBy:  Sort to use for grouping faces in the response. Use <code>TIMESTAMP</code> to group faces by the time that they are recognized. Use <code>INDEX</code> to sort by recognized faces. <br/>NextToken:  If the previous response was incomplete (because there is more search results to retrieve), Amazon Rekognition Video returns a pagination token in the response. You can use this pagination token to retrieve the next set of search results. <br/>MaxResults:  Maximum number of results to return per paginated call. The largest value you can specify is 1000. If you specify a value greater than 1000, a maximum of 1000 results is returned. The default value is 1000.<br/>JobId:  The job identifer for the search request. You get the job identifier from an initial call to <code>StartFaceSearch</code>.<br/>
        :return: Persons:  An array of persons, <a>PersonMatch</a>, in the video whose face(s) match the face(s) in an Amazon Rekognition collection. It also includes time information for when persons are matched in the video. You specify the input collection in an initial call to <code>StartFaceSearch</code>. Each <code>Persons</code> element includes a time the person was matched, face match details (<code>FaceMatches</code>) for matching faces in the collection, and person information (<code>Person</code>) for the matched person. <br/>VideoMetadata:  Information about a video that Amazon Rekognition analyzed. <code>Videometadata</code> is returned in every page of paginated responses from a Amazon Rekognition Video operation. :<br/>NextToken:  If the response is truncated, Amazon Rekognition Video returns this token that you can use in the subsequent request to retrieve the next set of search results. <br/>StatusMessage:  If the job fails, <code>StatusMessage</code> provides a descriptive error message.<br/>JobStatus:  The current status of the face search job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameWidth</i> Horizontal pixel dimension of the video.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameHeight</i> Vertical pixel dimension of the video.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameRate</i> Number of frames per second in the video.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> Format of the analyzed video. Possible values are MP4, MOV and AVI. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DurationMillis</i> Length of the video in milliseconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Codec</i> Type of compression used in the analyzed video. <br/>        """
        pass

    def get_label_detection(self, JobId: str, MaxResults: int=None, NextToken: str=None, SortBy: str=None) -> GetLabelDetectionResponse:
        """Gets the label detection results of a Amazon Rekognition Video analysis started by <a>StartLabelDetection</a>.  The label detection operation is started by a call to <a>StartLabelDetection</a> which returns a job identifier (<code>JobId</code>). When the label detection operation finishes, Amazon Rekognition publishes a completion status to the Amazon Simple Notification Service topic registered in the initial call to <code>StartlabelDetection</code>. To get the results of the label detection operation, first check that the status value published to the Amazon SNS topic is <code>SUCCEEDED</code>. If so, call <a>GetLabelDetection</a> and pass the job identifier (<code>JobId</code>) from the initial call to <code>StartLabelDetection</code>.  <code>GetLabelDetection</code> returns an array of detected labels (<code>Labels</code>) sorted by the time the labels were detected. You can also sort by the label name by specifying <code>NAME</code> for the <code>SortBy</code> input parameter. The labels returned include the label name, the percentage confidence in the accuracy of the detected label, and the time the label was detected in the video. The returned labels also include bounding box information for common objects, a hierarchical taxonomy of detected labels, and the version of the label model used for detection. Use MaxResults parameter to limit the number of labels returned. If there are more results than specified in <code>MaxResults</code>, the value of <code>NextToken</code> in the operation response contains a pagination token for getting the next set of results. To get the next page of results, call <code>GetlabelDetection</code> and populate the <code>NextToken</code> request parameter with the token value returned from the previous call to <code>GetLabelDetection</code>.

        :param self:
        :param SortBy:  Sort to use for elements in the <code>Labels</code> array. Use <code>TIMESTAMP</code> to sort array elements by the time labels are detected. Use <code>NAME</code> to alphabetically group elements for a label together. Within each label group, the array element are sorted by detection confidence. The default sort is by <code>TIMESTAMP</code>.<br/>NextToken:  If the previous response was incomplete (because there are more labels to retrieve), Amazon Rekognition Video returns a pagination token in the response. You can use this pagination token to retrieve the next set of labels. <br/>MaxResults:  Maximum number of results to return per paginated call. The largest value you can specify is 1000. If you specify a value greater than 1000, a maximum of 1000 results is returned. The default value is 1000.<br/>JobId:  Job identifier for the label detection operation for which you want results returned. You get the job identifer from an initial call to <code>StartlabelDetection</code>.<br/>
        :return: LabelModelVersion:  Version number of the label detection model that was used to detect labels.<br/>Labels:  An array of labels detected in the video. Each element contains the detected label and the time, in milliseconds from the start of the video, that the label was detected. <br/>NextToken:  If the response is truncated, Amazon Rekognition Video returns this token that you can use in the subsequent request to retrieve the next set of labels.<br/>VideoMetadata:  Information about a video that Amazon Rekognition Video analyzed. <code>Videometadata</code> is returned in every page of paginated responses from a Amazon Rekognition video operation.:<br/>StatusMessage:  If the job fails, <code>StatusMessage</code> provides a descriptive error message.<br/>JobStatus:  The current status of the label detection job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameWidth</i> Horizontal pixel dimension of the video.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameHeight</i> Vertical pixel dimension of the video.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameRate</i> Number of frames per second in the video.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> Format of the analyzed video. Possible values are MP4, MOV and AVI. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DurationMillis</i> Length of the video in milliseconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Codec</i> Type of compression used in the analyzed video. <br/>        """
        pass

    def get_person_tracking(self, JobId: str, MaxResults: int=None, NextToken: str=None, SortBy: str=None) -> GetPersonTrackingResponse:
        """Gets the path tracking results of a Amazon Rekognition Video analysis started by <a>StartPersonTracking</a>. The person path tracking operation is started by a call to <code>StartPersonTracking</code> which returns a job identifier (<code>JobId</code>). When the operation finishes, Amazon Rekognition Video publishes a completion status to the Amazon Simple Notification Service topic registered in the initial call to <code>StartPersonTracking</code>. To get the results of the person path tracking operation, first check that the status value published to the Amazon SNS topic is <code>SUCCEEDED</code>. If so, call <a>GetPersonTracking</a> and pass the job identifier (<code>JobId</code>) from the initial call to <code>StartPersonTracking</code>.  <code>GetPersonTracking</code> returns an array, <code>Persons</code>, of tracked persons and the time(s) their paths were tracked in the video.  <note>  <code>GetPersonTracking</code> only returns the default facial attributes (<code>BoundingBox</code>, <code>Confidence</code>, <code>Landmarks</code>, <code>Pose</code>, and <code>Quality</code>). The other facial attributes listed in the <code>Face</code> object of the following response syntax are not returned.  For more information, see FaceDetail in the Amazon Rekognition Developer Guide. </note> By default, the array is sorted by the time(s) a person's path is tracked in the video. You can sort by tracked persons by specifying <code>INDEX</code> for the <code>SortBy</code> input parameter. Use the <code>MaxResults</code> parameter to limit the number of items returned. If there are more results than specified in <code>MaxResults</code>, the value of <code>NextToken</code> in the operation response contains a pagination token for getting the next set of results. To get the next page of results, call <code>GetPersonTracking</code> and populate the <code>NextToken</code> request parameter with the token value returned from the previous call to <code>GetPersonTracking</code>.

        :param self:
        :param SortBy:  Sort to use for elements in the <code>Persons</code> array. Use <code>TIMESTAMP</code> to sort array elements by the time persons are detected. Use <code>INDEX</code> to sort by the tracked persons. If you sort by <code>INDEX</code>, the array elements for each person are sorted by detection confidence. The default sort is by <code>TIMESTAMP</code>.<br/>NextToken:  If the previous response was incomplete (because there are more persons to retrieve), Amazon Rekognition Video returns a pagination token in the response. You can use this pagination token to retrieve the next set of persons. <br/>MaxResults:  Maximum number of results to return per paginated call. The largest value you can specify is 1000. If you specify a value greater than 1000, a maximum of 1000 results is returned. The default value is 1000.<br/>JobId:  The identifier for a job that tracks persons in a video. You get the <code>JobId</code> from a call to <code>StartPersonTracking</code>. <br/>
        :return: Persons:  An array of the persons detected in the video and the time(s) their path was tracked throughout the video. An array element will exist for each time a person's path is tracked. <br/>NextToken:  If the response is truncated, Amazon Rekognition Video returns this token that you can use in the subsequent request to retrieve the next set of persons. <br/>VideoMetadata:  Information about a video that Amazon Rekognition Video analyzed. <code>Videometadata</code> is returned in every page of paginated responses from a Amazon Rekognition Video operation.:<br/>StatusMessage:  If the job fails, <code>StatusMessage</code> provides a descriptive error message.<br/>JobStatus:  The current status of the person tracking job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameWidth</i> Horizontal pixel dimension of the video.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameHeight</i> Vertical pixel dimension of the video.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FrameRate</i> Number of frames per second in the video.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> Format of the analyzed video. Possible values are MP4, MOV and AVI. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DurationMillis</i> Length of the video in milliseconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Codec</i> Type of compression used in the analyzed video. <br/>        """
        pass

    def index_faces(self, Image: Image, CollectionId: str, ExternalImageId: str=None, DetectionAttributes: List[Attribute]=None, MaxFaces: int=None, QualityFilter: str=None) -> IndexFacesResponse:
        """Detects faces in the input image and adds them to the specified collection.  Amazon Rekognition doesn't save the actual faces that are detected. Instead, the underlying detection algorithm first detects the faces in the input image. For each face, the algorithm extracts facial features into a feature vector, and stores it in the backend database. Amazon Rekognition uses feature vectors when it performs face match and search operations using the <a>SearchFaces</a> and <a>SearchFacesByImage</a> operations. For more information, see Adding Faces to a Collection in the Amazon Rekognition Developer Guide. To get the number of faces in a collection, call <a>DescribeCollection</a>.  If you're using version 1.0 of the face detection model, <code>IndexFaces</code> indexes the 15 largest faces in the input image. Later versions of the face detection model index the 100 largest faces in the input image.  If you're using version 4 or later of the face model, image orientation information is not returned in the <code>OrientationCorrection</code> field.  To determine which version of the model you're using, call <a>DescribeCollection</a> and supply the collection ID. You can also get the model version from the value of <code>FaceModelVersion</code> in the response from <code>IndexFaces</code>  For more information, see Model Versioning in the Amazon Rekognition Developer Guide. If you provide the optional <code>ExternalImageID</code> for the input image you provided, Amazon Rekognition associates this ID with all faces that it detects. When you call the <a>ListFaces</a> operation, the response returns the external ID. You can use this external image ID to create a client-side index to associate the faces with each image. You can then use the index to find all faces in an image. You can specify the maximum number of faces to index with the <code>MaxFaces</code> input parameter. This is useful when you want to index the largest faces in an image and don't want to index smaller faces, such as those belonging to people standing in the background. The <code>QualityFilter</code> input parameter allows you to filter out detected faces that don’t meet the required quality bar chosen by Amazon Rekognition. The quality bar is based on a variety of common use cases. By default, <code>IndexFaces</code> filters detected faces. You can also explicitly filter detected faces by specifying <code>AUTO</code> for the value of <code>QualityFilter</code>. If you do not want to filter detected faces, specify <code>NONE</code>.  <note> To use quality filtering, you need a collection associated with version 3 of the face model. To get the version of the face model associated with a collection, call <a>DescribeCollection</a>.  </note> Information about faces detected in an image, but not indexed, is returned in an array of <a>UnindexedFace</a> objects, <code>UnindexedFaces</code>. Faces aren't indexed for reasons such as: <ul> <li> The number of faces detected exceeds the value of the <code>MaxFaces</code> request parameter. </li> <li> The face is too small compared to the image dimensions. </li> <li> The face is too blurry. </li> <li> The image is too dark. </li> <li> The face has an extreme pose. </li> </ul> In response, the <code>IndexFaces</code> operation returns an array of metadata for all detected faces, <code>FaceRecords</code>. This includes:  <ul> <li> The bounding box, <code>BoundingBox</code>, of the detected face.  </li> <li> A confidence value, <code>Confidence</code>, which indicates the confidence that the bounding box contains a face. </li> <li> A face ID, <code>FaceId</code>, assigned by the service for each face that's detected and stored. </li> <li> An image ID, <code>ImageId</code>, assigned by the service for the input image. </li> </ul> If you request all facial attributes (by using the <code>detectionAttributes</code> parameter), Amazon Rekognition returns detailed facial attributes, such as facial landmarks (for example, location of eye and mouth) and other facial attributes like gender. If you provide the same image, specify the same collection, and use the same external ID in the <code>IndexFaces</code> operation, Amazon Rekognition doesn't save duplicate face metadata. <p/> The input image is passed either as base64-encoded image bytes, or as a reference to an image in an Amazon S3 bucket. If you use the AWS CLI to call Amazon Rekognition operations, passing image bytes isn't supported. The image must be formatted as a PNG or JPEG file.  This operation requires permissions to perform the <code>rekognition:IndexFaces</code> action.

        :param self:
        :param QualityFilter:  A filter that specifies how much filtering is done to identify faces that are detected with low quality. Filtered faces aren't indexed. If you specify <code>AUTO</code>, filtering prioritizes the identification of faces that don’t meet the required quality bar chosen by Amazon Rekognition. The quality bar is based on a variety of common use cases. Low-quality detections can occur for a number of reasons. Some examples are an object that's misidentified as a face, a face that's too blurry, or a face with a pose that's too extreme to use. If you specify <code>NONE</code>, no filtering is performed. The default value is AUTO. To use quality filtering, the collection you are using must be associated with version 3 of the face model.<br/>MaxFaces:  The maximum number of faces to index. The value of <code>MaxFaces</code> must be greater than or equal to 1. <code>IndexFaces</code> returns no more than 100 detected faces in an image, even if you specify a larger value for <code>MaxFaces</code>. If <code>IndexFaces</code> detects more faces than the value of <code>MaxFaces</code>, the faces with the lowest quality are filtered out first. If there are still more faces than the value of <code>MaxFaces</code>, the faces with the smallest bounding boxes are filtered out (up to the number that's needed to satisfy the value of <code>MaxFaces</code>). Information about the unindexed faces is available in the <code>UnindexedFaces</code> array.  The faces that are returned by <code>IndexFaces</code> are sorted by the largest face bounding box size to the smallest size, in descending order.  <code>MaxFaces</code> can be used with a collection associated with any version of the face model.<br/>DetectionAttributes:  An array of facial attributes that you want to be returned. This can be the default list of attributes or all attributes. If you don't specify a value for <code>Attributes</code> or if you specify <code>["DEFAULT"]</code>, the API returns the following subset of facial attributes: <code>BoundingBox</code>, <code>Confidence</code>, <code>Pose</code>, <code>Quality</code>, and <code>Landmarks</code>. If you provide <code>["ALL"]</code>, all facial attributes are returned, but the operation takes longer to complete. If you provide both, <code>["ALL", "DEFAULT"]</code>, the service uses a logical AND operator to determine which attributes to return (in this case, all attributes). <br/>ExternalImageId:  The ID you want to assign to all the faces detected in the image.<br/>Image:  The input image as base64-encoded bytes or an S3 object. If you use the AWS CLI to call Amazon Rekognition operations, passing base64-encoded image bytes isn't supported.  If you are using an AWS SDK to call Amazon Rekognition, you might not need to base64-encode image bytes passed using the <code>Bytes</code> field. For more information, see Images in the Amazon Rekognition developer guide.:<br/>CollectionId:  The ID of an existing collection to which you want to add the faces that are detected in the input images.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Object</i> Identifies an S3 object as the image source.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Object</b>: Identifies an S3 object as the image source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Bytes</i> Blob of image bytes up to 5 MBs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> If the bucket is versioning enabled, you can specify the object version. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> S3 object key name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> Name of the S3 bucket.<br/>
        :return: UnindexedFaces:  An array of faces that were detected in the image but weren't indexed. They weren't indexed because the quality filter identified them as low quality, or the <code>MaxFaces</code> request parameter filtered them out. To use the quality filter, you specify the <code>QualityFilter</code> request parameter.<br/>FaceModelVersion:  The version number of the face detection model that's associated with the input collection (<code>CollectionId</code>).<br/>OrientationCorrection:  If your collection is associated with a face detection model that's later than version 3.0, the value of <code>OrientationCorrection</code> is always null and no orientation information is returned. If your collection is associated with a face detection model that's version 3.0 or earlier, the following applies: <ul> <li> If the input image is in .jpeg format, it might contain exchangeable image file format (Exif) metadata that includes the image's orientation. Amazon Rekognition uses this orientation information to perform image correction - the bounding box coordinates are translated to represent object locations after the orientation information in the Exif metadata is used to correct the image orientation. Images in .png format don't contain Exif metadata. The value of <code>OrientationCorrection</code> is null. </li> <li> If the image doesn't contain orientation information in its Exif metadata, Amazon Rekognition returns an estimated orientation (ROTATE_0, ROTATE_90, ROTATE_180, ROTATE_270). Amazon Rekognition doesn’t perform image correction for images. The bounding box coordinates aren't translated and represent the object locations before the image is rotated. </li> </ul> Bounding box information is returned in the <code>FaceRecords</code> array. You can get the version of the face detection model by calling <a>DescribeCollection</a>. <br/>FaceRecords:  An array of faces detected and added to the collection. For more information, see Searching Faces in a Collection in the Amazon Rekognition Developer Guide. <br/>        """
        pass

    def list_collections(self, NextToken: str=None, MaxResults: int=None) -> ListCollectionsResponse:
        """Returns list of collection IDs in your account. If the result is truncated, the response also provides a <code>NextToken</code> that you can use in the subsequent request to fetch the next set of collection IDs. For an example, see Listing Collections in the Amazon Rekognition Developer Guide. This operation requires permissions to perform the <code>rekognition:ListCollections</code> action.

        :param self:
        :param MaxResults:  Maximum number of collection IDs to return. <br/>NextToken:  Pagination token from the previous response.<br/>
        :return: FaceModelVersions:  Version numbers of the face detection models associated with the collections in the array <code>CollectionIds</code>. For example, the value of <code>FaceModelVersions[2]</code> is the version number for the face detection model used by the collection in <code>CollectionId[2]</code>.<br/>NextToken:  If the result is truncated, the response provides a <code>NextToken</code> that you can use in the subsequent request to fetch the next set of collection IDs.<br/>CollectionIds:  An array of collection IDs.<br/>        """
        pass

    def list_faces(self, CollectionId: str, NextToken: str=None, MaxResults: int=None) -> ListFacesResponse:
        """Returns metadata for faces in the specified collection. This metadata includes information such as the bounding box coordinates, the confidence (that the bounding box contains a face), and face ID. For an example, see Listing Faces in a Collection in the Amazon Rekognition Developer Guide. This operation requires permissions to perform the <code>rekognition:ListFaces</code> action.

        :param self:
        :param MaxResults:  Maximum number of faces to return.<br/>NextToken:  If the previous response was incomplete (because there is more data to retrieve), Amazon Rekognition returns a pagination token in the response. You can use this pagination token to retrieve the next set of faces.<br/>CollectionId:  ID of the collection from which to list the faces.<br/>
        :return: FaceModelVersion:  Version number of the face detection model associated with the input collection (<code>CollectionId</code>).<br/>NextToken:  If the response is truncated, Amazon Rekognition returns this token that you can use in the subsequent request to retrieve the next set of faces.<br/>Faces:  An array of <code>Face</code> objects. <br/>        """
        pass

    def list_stream_processors(self, NextToken: str=None, MaxResults: int=None) -> ListStreamProcessorsResponse:
        """Gets a list of stream processors that you have created with <a>CreateStreamProcessor</a>. 

        :param self:
        :param MaxResults:  Maximum number of stream processors you want Amazon Rekognition Video to return in the response. The default is 1000. <br/>NextToken:  If the previous response was incomplete (because there are more stream processors to retrieve), Amazon Rekognition Video returns a pagination token in the response. You can use this pagination token to retrieve the next set of stream processors. <br/>
        :return: StreamProcessors:  List of stream processors that you have created.<br/>NextToken:  If the response is truncated, Amazon Rekognition Video returns this token that you can use in the subsequent request to retrieve the next set of stream processors. <br/>        """
        pass

    def recognize_celebrities(self, Image: Image) -> RecognizeCelebritiesResponse:
        """Returns an array of celebrities recognized in the input image. For more information, see Recognizing Celebrities in the Amazon Rekognition Developer Guide.   <code>RecognizeCelebrities</code> returns the 100 largest faces in the image. It lists recognized celebrities in the <code>CelebrityFaces</code> array and unrecognized faces in the <code>UnrecognizedFaces</code> array. <code>RecognizeCelebrities</code> doesn't return celebrities whose faces aren't among the largest 100 faces in the image. For each celebrity recognized, <code>RecognizeCelebrities</code> returns a <code>Celebrity</code> object. The <code>Celebrity</code> object contains the celebrity name, ID, URL links to additional information, match confidence, and a <code>ComparedFace</code> object that you can use to locate the celebrity's face on the image. Amazon Rekognition doesn't retain information about which images a celebrity has been recognized in. Your application must store this information and use the <code>Celebrity</code> ID property as a unique identifier for the celebrity. If you don't store the celebrity name or additional information URLs returned by <code>RecognizeCelebrities</code>, you will need the ID to identify the celebrity in a call to the <a>GetCelebrityInfo</a> operation. You pass the input image either as base64-encoded image bytes or as a reference to an image in an Amazon S3 bucket. If you use the AWS CLI to call Amazon Rekognition operations, passing image bytes is not supported. The image must be either a PNG or JPEG formatted file.  For an example, see Recognizing Celebrities in an Image in the Amazon Rekognition Developer Guide. This operation requires permissions to perform the <code>rekognition:RecognizeCelebrities</code> operation.

        :param self:
        :param Image:  The input image as base64-encoded bytes or an S3 object. If you use the AWS CLI to call Amazon Rekognition operations, passing base64-encoded image bytes is not supported.  If you are using an AWS SDK to call Amazon Rekognition, you might not need to base64-encode image bytes passed using the <code>Bytes</code> field. For more information, see Images in the Amazon Rekognition developer guide.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Object</i> Identifies an S3 object as the image source.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Object</b>: Identifies an S3 object as the image source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Bytes</i> Blob of image bytes up to 5 MBs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> If the bucket is versioning enabled, you can specify the object version. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> S3 object key name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> Name of the S3 bucket.<br/>
        :return: OrientationCorrection:  The orientation of the input image (counterclockwise direction). If your application displays the image, you can use this value to correct the orientation. The bounding box coordinates returned in <code>CelebrityFaces</code> and <code>UnrecognizedFaces</code> represent face locations before the image orientation is corrected.  <note> If the input image is in .jpeg format, it might contain exchangeable image (Exif) metadata that includes the image's orientation. If so, and the Exif metadata for the input image populates the orientation field, the value of <code>OrientationCorrection</code> is null. The <code>CelebrityFaces</code> and <code>UnrecognizedFaces</code> bounding box coordinates represent face locations after Exif metadata is used to correct the image orientation. Images in .png format don't contain Exif metadata.  </note><br/>UnrecognizedFaces:  Details about each unrecognized face in the image.<br/>CelebrityFaces:  Details about each celebrity found in the image. Amazon Rekognition can detect a maximum of 15 celebrities in an image.<br/>        """
        pass

    def search_faces(self, FaceId: str, CollectionId: str, MaxFaces: int=None, FaceMatchThreshold: Percent=None) -> SearchFacesResponse:
        """For a given input face ID, searches for matching faces in the collection the face belongs to. You get a face ID when you add a face to the collection using the <a>IndexFaces</a> operation. The operation compares the features of the input face with faces in the specified collection.  <note> You can also search faces without indexing faces by using the <code>SearchFacesByImage</code> operation. </note>  The operation response returns an array of faces that match, ordered by similarity score with the highest similarity first. More specifically, it is an array of metadata for each face match that is found. Along with the metadata, the response also includes a <code>confidence</code> value for each face match, indicating the confidence that the specific face matches the input face.  For an example, see Searching for a Face Using Its Face ID in the Amazon Rekognition Developer Guide. This operation requires permissions to perform the <code>rekognition:SearchFaces</code> action.

        :param self:
        :param FaceMatchThreshold:  Optional value specifying the minimum confidence in the face match to return. For example, don't return any matches where confidence in matches is less than 70%.<br/>MaxFaces:  Maximum number of faces to return. The operation returns the maximum number of faces with the highest confidence in the match.<br/>FaceId:  ID of a face to find matches for in the collection.<br/>CollectionId:  ID of the collection the face belongs to.<br/>
        :return: FaceModelVersion:  Version number of the face detection model associated with the input collection (<code>CollectionId</code>).<br/>FaceMatches:  An array of faces that matched the input face, along with the confidence in the match.<br/>SearchedFaceId:  ID of the face that was searched for matches in a collection.<br/>        """
        pass

    def search_faces_by_image(self, Image: Image, CollectionId: str, MaxFaces: int=None, FaceMatchThreshold: Percent=None) -> SearchFacesByImageResponse:
        """For a given input image, first detects the largest face in the image, and then searches the specified collection for matching faces. The operation compares the features of the input face with faces in the specified collection.  <note> To search for all faces in an input image, you might first call the <a>IndexFaces</a> operation, and then use the face IDs returned in subsequent calls to the <a>SearchFaces</a> operation.   You can also call the <code>DetectFaces</code> operation and use the bounding boxes in the response to make face crops, which then you can pass in to the <code>SearchFacesByImage</code> operation.  </note> You pass the input image either as base64-encoded image bytes or as a reference to an image in an Amazon S3 bucket. If you use the AWS CLI to call Amazon Rekognition operations, passing image bytes is not supported. The image must be either a PNG or JPEG formatted file.   The response returns an array of faces that match, ordered by similarity score with the highest similarity first. More specifically, it is an array of metadata for each face match found. Along with the metadata, the response also includes a <code>similarity</code> indicating how similar the face is to the input face. In the response, the operation also returns the bounding box (and a confidence level that the bounding box contains a face) of the face that Amazon Rekognition used for the input image.  For an example, Searching for a Face Using an Image in the Amazon Rekognition Developer Guide. This operation requires permissions to perform the <code>rekognition:SearchFacesByImage</code> action.

        :param self:
        :param FaceMatchThreshold:  (Optional) Specifies the minimum confidence in the face match to return. For example, don't return any matches where confidence in matches is less than 70%.<br/>MaxFaces:  Maximum number of faces to return. The operation returns the maximum number of faces with the highest confidence in the match.<br/>Image:  The input image as base64-encoded bytes or an S3 object. If you use the AWS CLI to call Amazon Rekognition operations, passing base64-encoded image bytes is not supported.  If you are using an AWS SDK to call Amazon Rekognition, you might not need to base64-encode image bytes passed using the <code>Bytes</code> field. For more information, see Images in the Amazon Rekognition developer guide.:<br/>CollectionId:  ID of the collection to search.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Object</i> Identifies an S3 object as the image source.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Object</b>: Identifies an S3 object as the image source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Bytes</i> Blob of image bytes up to 5 MBs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> If the bucket is versioning enabled, you can specify the object version. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> S3 object key name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> Name of the S3 bucket.<br/>
        :return: FaceModelVersion:  Version number of the face detection model associated with the input collection (<code>CollectionId</code>).<br/>FaceMatches:  An array of faces that match the input face, along with the confidence in the match.<br/>SearchedFaceConfidence:  The level of confidence that the <code>searchedFaceBoundingBox</code>, contains a face.<br/>SearchedFaceBoundingBox:  The bounding box around the face in the input image that Amazon Rekognition used for the search.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Top</i> Top coordinate of the bounding box as a ratio of overall image height.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Left</i> Left coordinate of the bounding box as a ratio of overall image width.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Height</i> Height of the bounding box as a ratio of the overall image height.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Width</i> Width of the bounding box as a ratio of the overall image width.<br/>        """
        pass

    def start_celebrity_recognition(self, Video: Video, ClientRequestToken: str=None, NotificationChannel: NotificationChannel=None, JobTag: str=None) -> StartCelebrityRecognitionResponse:
        """Starts asynchronous recognition of celebrities in a stored video. Amazon Rekognition Video can detect celebrities in a video must be stored in an Amazon S3 bucket. Use <a>Video</a> to specify the bucket name and the filename of the video. <code>StartCelebrityRecognition</code> returns a job identifier (<code>JobId</code>) which you use to get the results of the analysis. When celebrity recognition analysis is finished, Amazon Rekognition Video publishes a completion status to the Amazon Simple Notification Service topic that you specify in <code>NotificationChannel</code>. To get the results of the celebrity recognition analysis, first check that the status value published to the Amazon SNS topic is <code>SUCCEEDED</code>. If so, call <a>GetCelebrityRecognition</a> and pass the job identifier (<code>JobId</code>) from the initial call to <code>StartCelebrityRecognition</code>.  For more information, see Recognizing Celebrities in the Amazon Rekognition Developer Guide.

        :param self:
        :param JobTag:  Unique identifier you specify to identify the job in the completion status published to the Amazon Simple Notification Service topic. <br/>NotificationChannel:  The Amazon SNS topic ARN that you want Amazon Rekognition Video to publish the completion status of the celebrity recognition analysis to.:<br/>ClientRequestToken:  Idempotent token used to identify the start request. If you use the same token with multiple <code>StartCelebrityRecognition</code> requests, the same <code>JobId</code> is returned. Use <code>ClientRequestToken</code> to prevent the same job from being accidently started more than once. <br/>Video:  The video in which you want to recognize celebrities. The video must be stored in an Amazon S3 bucket.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Object</i> The Amazon S3 bucket name and file name for the video.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Object</b>: The Amazon S3 bucket name and file name for the video.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> If the bucket is versioning enabled, you can specify the object version. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> S3 object key name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> Name of the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The ARN of an IAM role that gives Amazon Rekognition publishing permissions to the Amazon SNS topic. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SNSTopicArn</i> The Amazon SNS topic to which Amazon Rekognition to posts the completion status.<br/>
        :return: JobId:  The identifier for the celebrity recognition analysis job. Use <code>JobId</code> to identify the job in a subsequent call to <code>GetCelebrityRecognition</code>.<br/>        """
        pass

    def start_content_moderation(self, Video: Video, MinConfidence: Percent=None, ClientRequestToken: str=None, NotificationChannel: NotificationChannel=None, JobTag: str=None) -> StartContentModerationResponse:
        """ Starts asynchronous detection of explicit or suggestive adult content in a stored video. Amazon Rekognition Video can moderate content in a video stored in an Amazon S3 bucket. Use <a>Video</a> to specify the bucket name and the filename of the video. <code>StartContentModeration</code> returns a job identifier (<code>JobId</code>) which you use to get the results of the analysis. When content moderation analysis is finished, Amazon Rekognition Video publishes a completion status to the Amazon Simple Notification Service topic that you specify in <code>NotificationChannel</code>. To get the results of the content moderation analysis, first check that the status value published to the Amazon SNS topic is <code>SUCCEEDED</code>. If so, call <a>GetContentModeration</a> and pass the job identifier (<code>JobId</code>) from the initial call to <code>StartContentModeration</code>.  For more information, see Detecting Unsafe Content in the Amazon Rekognition Developer Guide.

        :param self:
        :param JobTag:  Unique identifier you specify to identify the job in the completion status published to the Amazon Simple Notification Service topic. <br/>NotificationChannel:  The Amazon SNS topic ARN that you want Amazon Rekognition Video to publish the completion status of the content moderation analysis to.:<br/>ClientRequestToken:  Idempotent token used to identify the start request. If you use the same token with multiple <code>StartContentModeration</code> requests, the same <code>JobId</code> is returned. Use <code>ClientRequestToken</code> to prevent the same job from being accidently started more than once. <br/>MinConfidence:  Specifies the minimum confidence that Amazon Rekognition must have in order to return a moderated content label. Confidence represents how certain Amazon Rekognition is that the moderated content is correctly identified. 0 is the lowest confidence. 100 is the highest confidence. Amazon Rekognition doesn't return any moderated content labels with a confidence level lower than this specified value. If you don't specify <code>MinConfidence</code>, <code>GetContentModeration</code> returns labels with confidence values greater than or equal to 50 percent.<br/>Video:  The video in which you want to moderate content. The video must be stored in an Amazon S3 bucket.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Object</i> The Amazon S3 bucket name and file name for the video.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Object</b>: The Amazon S3 bucket name and file name for the video.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> If the bucket is versioning enabled, you can specify the object version. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> S3 object key name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> Name of the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The ARN of an IAM role that gives Amazon Rekognition publishing permissions to the Amazon SNS topic. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SNSTopicArn</i> The Amazon SNS topic to which Amazon Rekognition to posts the completion status.<br/>
        :return: JobId:  The identifier for the content moderation analysis job. Use <code>JobId</code> to identify the job in a subsequent call to <code>GetContentModeration</code>.<br/>        """
        pass

    def start_face_detection(self, Video: Video, ClientRequestToken: str=None, NotificationChannel: NotificationChannel=None, FaceAttributes: str=None, JobTag: str=None) -> StartFaceDetectionResponse:
        """Starts asynchronous detection of faces in a stored video. Amazon Rekognition Video can detect faces in a video stored in an Amazon S3 bucket. Use <a>Video</a> to specify the bucket name and the filename of the video. <code>StartFaceDetection</code> returns a job identifier (<code>JobId</code>) that you use to get the results of the operation. When face detection is finished, Amazon Rekognition Video publishes a completion status to the Amazon Simple Notification Service topic that you specify in <code>NotificationChannel</code>. To get the results of the face detection operation, first check that the status value published to the Amazon SNS topic is <code>SUCCEEDED</code>. If so, call <a>GetFaceDetection</a> and pass the job identifier (<code>JobId</code>) from the initial call to <code>StartFaceDetection</code>. For more information, see Detecting Faces in a Stored Video in the Amazon Rekognition Developer Guide.

        :param self:
        :param JobTag:  Unique identifier you specify to identify the job in the completion status published to the Amazon Simple Notification Service topic. <br/>FaceAttributes:  The face attributes you want returned.  <code>DEFAULT</code> - The following subset of facial attributes are returned: BoundingBox, Confidence, Pose, Quality and Landmarks.   <code>ALL</code> - All facial attributes are returned.<br/>NotificationChannel:  The ARN of the Amazon SNS topic to which you want Amazon Rekognition Video to publish the completion status of the face detection operation.:<br/>ClientRequestToken:  Idempotent token used to identify the start request. If you use the same token with multiple <code>StartFaceDetection</code> requests, the same <code>JobId</code> is returned. Use <code>ClientRequestToken</code> to prevent the same job from being accidently started more than once. <br/>Video:  The video in which you want to detect faces. The video must be stored in an Amazon S3 bucket.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Object</i> The Amazon S3 bucket name and file name for the video.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Object</b>: The Amazon S3 bucket name and file name for the video.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> If the bucket is versioning enabled, you can specify the object version. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> S3 object key name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> Name of the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The ARN of an IAM role that gives Amazon Rekognition publishing permissions to the Amazon SNS topic. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SNSTopicArn</i> The Amazon SNS topic to which Amazon Rekognition to posts the completion status.<br/>
        :return: JobId:  The identifier for the face detection job. Use <code>JobId</code> to identify the job in a subsequent call to <code>GetFaceDetection</code>.<br/>        """
        pass

    def start_face_search(self, CollectionId: str, Video: Video, ClientRequestToken: str=None, FaceMatchThreshold: Percent=None, NotificationChannel: NotificationChannel=None, JobTag: str=None) -> StartFaceSearchResponse:
        """Starts the asynchronous search for faces in a collection that match the faces of persons detected in a stored video. The video must be stored in an Amazon S3 bucket. Use <a>Video</a> to specify the bucket name and the filename of the video. <code>StartFaceSearch</code> returns a job identifier (<code>JobId</code>) which you use to get the search results once the search has completed. When searching is finished, Amazon Rekognition Video publishes a completion status to the Amazon Simple Notification Service topic that you specify in <code>NotificationChannel</code>. To get the search results, first check that the status value published to the Amazon SNS topic is <code>SUCCEEDED</code>. If so, call <a>GetFaceSearch</a> and pass the job identifier (<code>JobId</code>) from the initial call to <code>StartFaceSearch</code>. For more information, see <a>procedure-person-search-videos</a>.

        :param self:
        :param JobTag:  Unique identifier you specify to identify the job in the completion status published to the Amazon Simple Notification Service topic. <br/>NotificationChannel:  The ARN of the Amazon SNS topic to which you want Amazon Rekognition Video to publish the completion status of the search. :<br/>CollectionId:  ID of the collection that contains the faces you want to search for.<br/>FaceMatchThreshold:  The minimum confidence in the person match to return. For example, don't return any matches where confidence in matches is less than 70%. <br/>ClientRequestToken:  Idempotent token used to identify the start request. If you use the same token with multiple <code>StartFaceSearch</code> requests, the same <code>JobId</code> is returned. Use <code>ClientRequestToken</code> to prevent the same job from being accidently started more than once. <br/>Video:  The video you want to search. The video must be stored in an Amazon S3 bucket. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Object</i> The Amazon S3 bucket name and file name for the video.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Object</b>: The Amazon S3 bucket name and file name for the video.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> If the bucket is versioning enabled, you can specify the object version. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> S3 object key name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> Name of the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The ARN of an IAM role that gives Amazon Rekognition publishing permissions to the Amazon SNS topic. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SNSTopicArn</i> The Amazon SNS topic to which Amazon Rekognition to posts the completion status.<br/>
        :return: JobId:  The identifier for the search job. Use <code>JobId</code> to identify the job in a subsequent call to <code>GetFaceSearch</code>. <br/>        """
        pass

    def start_label_detection(self, Video: Video, ClientRequestToken: str=None, MinConfidence: Percent=None, NotificationChannel: NotificationChannel=None, JobTag: str=None) -> StartLabelDetectionResponse:
        """Starts asynchronous detection of labels in a stored video. Amazon Rekognition Video can detect labels in a video. Labels are instances of real-world entities. This includes objects like flower, tree, and table; events like wedding, graduation, and birthday party; concepts like landscape, evening, and nature; and activities like a person getting out of a car or a person skiing. The video must be stored in an Amazon S3 bucket. Use <a>Video</a> to specify the bucket name and the filename of the video. <code>StartLabelDetection</code> returns a job identifier (<code>JobId</code>) which you use to get the results of the operation. When label detection is finished, Amazon Rekognition Video publishes a completion status to the Amazon Simple Notification Service topic that you specify in <code>NotificationChannel</code>. To get the results of the label detection operation, first check that the status value published to the Amazon SNS topic is <code>SUCCEEDED</code>. If so, call <a>GetLabelDetection</a> and pass the job identifier (<code>JobId</code>) from the initial call to <code>StartLabelDetection</code>. <p/>

        :param self:
        :param JobTag:  Unique identifier you specify to identify the job in the completion status published to the Amazon Simple Notification Service topic. <br/>NotificationChannel:  The Amazon SNS topic ARN you want Amazon Rekognition Video to publish the completion status of the label detection operation to. :<br/>MinConfidence:  Specifies the minimum confidence that Amazon Rekognition Video must have in order to return a detected label. Confidence represents how certain Amazon Rekognition is that a label is correctly identified.0 is the lowest confidence. 100 is the highest confidence. Amazon Rekognition Video doesn't return any labels with a confidence level lower than this specified value. If you don't specify <code>MinConfidence</code>, the operation returns labels with confidence values greater than or equal to 50 percent.<br/>ClientRequestToken:  Idempotent token used to identify the start request. If you use the same token with multiple <code>StartLabelDetection</code> requests, the same <code>JobId</code> is returned. Use <code>ClientRequestToken</code> to prevent the same job from being accidently started more than once. <br/>Video:  The video in which you want to detect labels. The video must be stored in an Amazon S3 bucket.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Object</i> The Amazon S3 bucket name and file name for the video.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Object</b>: The Amazon S3 bucket name and file name for the video.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> If the bucket is versioning enabled, you can specify the object version. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> S3 object key name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> Name of the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The ARN of an IAM role that gives Amazon Rekognition publishing permissions to the Amazon SNS topic. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SNSTopicArn</i> The Amazon SNS topic to which Amazon Rekognition to posts the completion status.<br/>
        :return: JobId:  The identifier for the label detection job. Use <code>JobId</code> to identify the job in a subsequent call to <code>GetLabelDetection</code>. <br/>        """
        pass

    def start_person_tracking(self, Video: Video, ClientRequestToken: str=None, NotificationChannel: NotificationChannel=None, JobTag: str=None) -> StartPersonTrackingResponse:
        """Starts the asynchronous tracking of a person's path in a stored video. Amazon Rekognition Video can track the path of people in a video stored in an Amazon S3 bucket. Use <a>Video</a> to specify the bucket name and the filename of the video. <code>StartPersonTracking</code> returns a job identifier (<code>JobId</code>) which you use to get the results of the operation. When label detection is finished, Amazon Rekognition publishes a completion status to the Amazon Simple Notification Service topic that you specify in <code>NotificationChannel</code>.  To get the results of the person detection operation, first check that the status value published to the Amazon SNS topic is <code>SUCCEEDED</code>. If so, call <a>GetPersonTracking</a> and pass the job identifier (<code>JobId</code>) from the initial call to <code>StartPersonTracking</code>.

        :param self:
        :param JobTag:  Unique identifier you specify to identify the job in the completion status published to the Amazon Simple Notification Service topic. <br/>NotificationChannel:  The Amazon SNS topic ARN you want Amazon Rekognition Video to publish the completion status of the people detection operation to.:<br/>ClientRequestToken:  Idempotent token used to identify the start request. If you use the same token with multiple <code>StartPersonTracking</code> requests, the same <code>JobId</code> is returned. Use <code>ClientRequestToken</code> to prevent the same job from being accidently started more than once. <br/>Video:  The video in which you want to detect people. The video must be stored in an Amazon S3 bucket.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Object</i> The Amazon S3 bucket name and file name for the video.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Object</b>: The Amazon S3 bucket name and file name for the video.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> If the bucket is versioning enabled, you can specify the object version. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> S3 object key name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> Name of the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The ARN of an IAM role that gives Amazon Rekognition publishing permissions to the Amazon SNS topic. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SNSTopicArn</i> The Amazon SNS topic to which Amazon Rekognition to posts the completion status.<br/>
        :return: JobId:  The identifier for the person detection job. Use <code>JobId</code> to identify the job in a subsequent call to <code>GetPersonTracking</code>.<br/>        """
        pass

    def start_stream_processor(self, Name: str) -> StartStreamProcessorResponse:
        """Starts processing a stream processor. You create a stream processor by calling <a>CreateStreamProcessor</a>. To tell <code>StartStreamProcessor</code> which stream processor to start, use the value of the <code>Name</code> field specified in the call to <code>CreateStreamProcessor</code>.

        :param self:
        :param Name:  The name of the stream processor to start processing.<br/>
        :return:         """
        pass

    def stop_stream_processor(self, Name: str) -> StopStreamProcessorResponse:
        """Stops a running stream processor that was created by <a>CreateStreamProcessor</a>.

        :param self:
        :param Name:  The name of a stream processor created by <a>CreateStreamProcessor</a>.<br/>
        :return:         """
        pass



class ResourceGroups(BaseClient):
    """<fullname>AWS Resource Groups</fullname> AWS Resource Groups lets you organize AWS resources such as Amazon EC2 instances, Amazon Relational Database Service databases, and Amazon S3 buckets into groups using criteria that you define as tags. A resource group is a collection of resources that match the resource types specified in a query, and share one or more tags or portions of tags. You can create a group of resources based on their roles in your cloud infrastructure, lifecycle stages, regions, application layers, or virtually any criteria. Resource groups enable you to automate management tasks, such as those in AWS Systems Manager Automation documents, on tag-related resources in AWS Systems Manager. Groups of tagged resources also let you quickly view a custom console in AWS Systems Manager that shows AWS Config compliance and other monitoring data about member resources. To create a resource group, build a resource query, and specify tags that identify the criteria that members of the group have in common. Tags are key-value pairs. For more information about Resource Groups, see the <a href="https://docs.aws.amazon.com/ARG/latest/userguide/welcome.html">AWS Resource Groups User Guide</a>. AWS Resource Groups uses a REST-compliant API that you can use to perform the following types of operations. <ul> <li> Create, Read, Update, and Delete (CRUD) operations on resource groups and resource query entities </li> <li> Applying, editing, and removing tags from resource groups </li> <li> Resolving resource group member ARNs so they can be returned as search results </li> <li> Getting data about resources that are members of a group </li> <li> Searching AWS resources based on a resource query </li> </ul>"""


    class ListGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_groups` for valid parameters"""
            pass
    class SearchResourcesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `search_resources` for valid parameters"""
            pass
    class ListGroupResourcesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_group_resources` for valid parameters"""
            pass


    class CreateGroupInput(Mapping):
        pass
    
    class CreateGroupOutput(Mapping):
        pass
    
    class ResourceQuery(Mapping):
        pass
    
    class Tags(object):
        pass
    
    class DeleteGroupInput(Mapping):
        pass
    
    class DeleteGroupOutput(Mapping):
        pass
    
    class GetGroupInput(Mapping):
        pass
    
    class GetGroupOutput(Mapping):
        pass
    
    class GetGroupQueryInput(Mapping):
        pass
    
    class GetGroupQueryOutput(Mapping):
        pass
    
    class GetTagsInput(Mapping):
        pass
    
    class GetTagsOutput(Mapping):
        pass
    
    class ListGroupResourcesInput(Mapping):
        pass
    
    class ListGroupResourcesOutput(Mapping):
        pass
    
    class ResourceFilter(Mapping):
        pass
    
    class ListGroupsInput(Mapping):
        pass
    
    class ListGroupsOutput(Mapping):
        pass
    
    class GroupFilter(Mapping):
        pass
    
    class SearchResourcesInput(Mapping):
        pass
    
    class SearchResourcesOutput(Mapping):
        pass
    
    class TagInput(Mapping):
        pass
    
    class TagOutput(Mapping):
        pass
    
    class UntagInput(Mapping):
        pass
    
    class UntagOutput(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateGroupInput(Mapping):
        pass
    
    class UpdateGroupOutput(Mapping):
        pass
    
    class UpdateGroupQueryInput(Mapping):
        pass
    
    class UpdateGroupQueryOutput(Mapping):
        pass
    
    def create_group(self, ResourceQuery: ResourceQuery, Name: str, Description: str=None, Tags: Tags=None) -> CreateGroupOutput:
        """Creates a group with a specified name, description, and resource query.

        :param self:
        :param Tags:  The tags to add to the group. A tag is a string-to-string map of key-value pairs. Tag keys can have a maximum character length of 128 characters, and tag values can have a maximum length of 256 characters.<br/>ResourceQuery:  The resource query that determines which AWS resources are members of this group.:<br/>Description:  The description of the resource group. Descriptions can have a maximum of 511 characters, including letters, numbers, hyphens, underscores, punctuation, and spaces.<br/>Name:  The name of the group, which is the identifier of the group in other operations. A resource group name cannot be updated after it is created. A resource group name can have a maximum of 128 characters, including letters, numbers, hyphens, dots, and underscores. The name cannot start with <code>AWS</code> or <code>aws</code>; these are reserved. A resource group name must be unique within your account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Query</i> The query that defines a group or a search.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of the query. The valid values in this release are <code>TAG_FILTERS_1_0</code> and <code>CLOUDFORMATION_STACK_1_0</code>.  <i> <code>TAG_FILTERS_1_0:</code> </i> A JSON syntax that lets you specify a collection of simple tag filters for resource types and tags, as supported by the AWS Tagging API <a href="https://docs.aws.amazon.com/resourcegroupstagging/latest/APIReference/API_GetResources.html">GetResources</a> operation. If you specify more than one tag key, only resources that match all tag keys, and at least one value of each specified tag key, are returned in your query. If you specify more than one value for a tag key, a resource matches the filter if it has a tag key value that matches <i>any</i> of the specified values. For example, consider the following sample query for resources that have two tags, <code>Stage</code> and <code>Version</code>, with two values each. (<code>[{"Key":"Stage","Values":["Test","Deploy"]},{"Key":"Version","Values":["1","2"]}]</code>) The results of this query might include the following. <ul> <li> An EC2 instance that has the following two tags: <code>{"Key":"Stage","Value":"Deploy"}</code>, and <code>{"Key":"Version","Value":"2"}</code>  </li> <li> An S3 bucket that has the following two tags: {"Key":"Stage","Value":"Test"}, and {"Key":"Version","Value":"1"} </li> </ul> The query would not return the following results, however. The following EC2 instance does not have all tag keys specified in the filter, so it is rejected. The RDS database has all of the tag keys, but no values that match at least one of the specified tag key values in the filter. <ul> <li> An EC2 instance that has only the following tag: <code>{"Key":"Stage","Value":"Deploy"}</code>. </li> <li> An RDS database that has the following two tags: <code>{"Key":"Stage","Value":"Archived"}</code>, and <code>{"Key":"Version","Value":"4"}</code>  </li> </ul>  <i> <code>CLOUDFORMATION_STACK_1_0:</code> </i> A JSON syntax that lets you specify a CloudFormation stack ARN.<br/>
        :return: Tags:  The tags associated with the group.<br/>ResourceQuery:  The resource query associated with the group.:<br/>Group:  A full description of the resource group after it is created.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the resource group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of a resource group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupArn</i> The ARN of a resource group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Query</i> The query that defines a group or a search.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of the query. The valid values in this release are <code>TAG_FILTERS_1_0</code> and <code>CLOUDFORMATION_STACK_1_0</code>.  <i> <code>TAG_FILTERS_1_0:</code> </i> A JSON syntax that lets you specify a collection of simple tag filters for resource types and tags, as supported by the AWS Tagging API <a href="https://docs.aws.amazon.com/resourcegroupstagging/latest/APIReference/API_GetResources.html">GetResources</a> operation. If you specify more than one tag key, only resources that match all tag keys, and at least one value of each specified tag key, are returned in your query. If you specify more than one value for a tag key, a resource matches the filter if it has a tag key value that matches <i>any</i> of the specified values. For example, consider the following sample query for resources that have two tags, <code>Stage</code> and <code>Version</code>, with two values each. (<code>[{"Key":"Stage","Values":["Test","Deploy"]},{"Key":"Version","Values":["1","2"]}]</code>) The results of this query might include the following. <ul> <li> An EC2 instance that has the following two tags: <code>{"Key":"Stage","Value":"Deploy"}</code>, and <code>{"Key":"Version","Value":"2"}</code>  </li> <li> An S3 bucket that has the following two tags: {"Key":"Stage","Value":"Test"}, and {"Key":"Version","Value":"1"} </li> </ul> The query would not return the following results, however. The following EC2 instance does not have all tag keys specified in the filter, so it is rejected. The RDS database has all of the tag keys, but no values that match at least one of the specified tag key values in the filter. <ul> <li> An EC2 instance that has only the following tag: <code>{"Key":"Stage","Value":"Deploy"}</code>. </li> <li> An RDS database that has the following two tags: <code>{"Key":"Stage","Value":"Archived"}</code>, and <code>{"Key":"Version","Value":"4"}</code>  </li> </ul>  <i> <code>CLOUDFORMATION_STACK_1_0:</code> </i> A JSON syntax that lets you specify a CloudFormation stack ARN.<br/>        """
        pass

    def delete_group(self, GroupName: str) -> DeleteGroupOutput:
        """Deletes a specified resource group. Deleting a resource group does not delete resources that are members of the group; it only deletes the group structure.

        :param self:
        :param GroupName:  The name of the resource group to delete.<br/>
        :return: Group:  A full description of the deleted resource group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the resource group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of a resource group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupArn</i> The ARN of a resource group.<br/>        """
        pass

    def get_group(self, GroupName: str) -> GetGroupOutput:
        """Returns information about a specified resource group.

        :param self:
        :param GroupName:  The name of the resource group.<br/>
        :return: Group:  A full description of the resource group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the resource group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of a resource group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupArn</i> The ARN of a resource group.<br/>        """
        pass

    def get_group_query(self, GroupName: str) -> GetGroupQueryOutput:
        """Returns the resource query associated with the specified resource group.

        :param self:
        :param GroupName:  The name of the resource group.<br/>
        :return: GroupQuery:  The resource query associated with the specified group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceQuery</i> The resource query which determines which AWS resources are members of the associated resource group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResourceQuery</b>: The resource query which determines which AWS resources are members of the associated resource group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupName</i> The name of a resource group that is associated with a specific resource query.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Query</i> The query that defines a group or a search.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of the query. The valid values in this release are <code>TAG_FILTERS_1_0</code> and <code>CLOUDFORMATION_STACK_1_0</code>.  <i> <code>TAG_FILTERS_1_0:</code> </i> A JSON syntax that lets you specify a collection of simple tag filters for resource types and tags, as supported by the AWS Tagging API <a href="https://docs.aws.amazon.com/resourcegroupstagging/latest/APIReference/API_GetResources.html">GetResources</a> operation. If you specify more than one tag key, only resources that match all tag keys, and at least one value of each specified tag key, are returned in your query. If you specify more than one value for a tag key, a resource matches the filter if it has a tag key value that matches <i>any</i> of the specified values. For example, consider the following sample query for resources that have two tags, <code>Stage</code> and <code>Version</code>, with two values each. (<code>[{"Key":"Stage","Values":["Test","Deploy"]},{"Key":"Version","Values":["1","2"]}]</code>) The results of this query might include the following. <ul> <li> An EC2 instance that has the following two tags: <code>{"Key":"Stage","Value":"Deploy"}</code>, and <code>{"Key":"Version","Value":"2"}</code>  </li> <li> An S3 bucket that has the following two tags: {"Key":"Stage","Value":"Test"}, and {"Key":"Version","Value":"1"} </li> </ul> The query would not return the following results, however. The following EC2 instance does not have all tag keys specified in the filter, so it is rejected. The RDS database has all of the tag keys, but no values that match at least one of the specified tag key values in the filter. <ul> <li> An EC2 instance that has only the following tag: <code>{"Key":"Stage","Value":"Deploy"}</code>. </li> <li> An RDS database that has the following two tags: <code>{"Key":"Stage","Value":"Archived"}</code>, and <code>{"Key":"Version","Value":"4"}</code>  </li> </ul>  <i> <code>CLOUDFORMATION_STACK_1_0:</code> </i> A JSON syntax that lets you specify a CloudFormation stack ARN.<br/>        """
        pass

    def get_tags(self, Arn: str) -> GetTagsOutput:
        """Returns a list of tags that are associated with a resource group, specified by an ARN.

        :param self:
        :param Arn:  The ARN of the resource group for which you want a list of tags. The resource must exist within the account you are using.<br/>
        :return: Tags:  The tags associated with the specified resource group.<br/>Arn:  The ARN of the tagged resource group.<br/>        """
        pass

    def list_group_resources(self, GroupName: str, Filters: List[ResourceFilter]=None, MaxResults: int=None, NextToken: str=None) -> ListGroupResourcesOutput:
        """Returns a list of ARNs of resources that are members of a specified resource group.

        :param self:
        :param NextToken:  The NextToken value that is returned in a paginated ListGroupResources request. To get the next page of results, run the call again, add the NextToken parameter, and specify the NextToken value.<br/>MaxResults:  The maximum number of group member ARNs that are returned in a single call by ListGroupResources, in paginated output. By default, this number is 50.<br/>Filters:  Filters, formatted as ResourceFilter objects, that you want to apply to a ListGroupResources operation. <ul> <li>  <code>resource-type</code> - Filter resources by their type. Specify up to five resource types in the format AWS::ServiceCode::ResourceType. For example, AWS::EC2::Instance, or AWS::S3::Bucket. </li> </ul><br/>GroupName:  The name of the resource group.<br/>
        :return: QueryErrors:  A list of <code>QueryError</code> objects. Each error is an object that contains <code>ErrorCode</code> and <code>Message</code> structures. Possible values for <code>ErrorCode</code> are <code>CLOUDFORMATION_STACK_INACTIVE</code> and <code>CLOUDFORMATION_STACK_NOT_EXISTING</code>.<br/>NextToken:  The NextToken value to include in a subsequent <code>ListGroupResources</code> request, to get more results.<br/>ResourceIdentifiers:  The ARNs and resource types of resources that are members of the group that you specified.<br/>        """
        pass

    def list_groups(self, Filters: List[GroupFilter]=None, MaxResults: int=None, NextToken: str=None) -> ListGroupsOutput:
        """Returns a list of existing resource groups in your account.

        :param self:
        :param NextToken:  The NextToken value that is returned in a paginated <code>ListGroups</code> request. To get the next page of results, run the call again, add the NextToken parameter, and specify the NextToken value.<br/>MaxResults:  The maximum number of resource group results that are returned by ListGroups in paginated output. By default, this number is 50.<br/>Filters:  Filters, formatted as GroupFilter objects, that you want to apply to a ListGroups operation. <ul> <li>  <code>resource-type</code> - Filter groups by resource type. Specify up to five resource types in the format AWS::ServiceCode::ResourceType. For example, AWS::EC2::Instance, or AWS::S3::Bucket. </li> </ul><br/>
        :return: NextToken:  The NextToken value to include in a subsequent <code>ListGroups</code> request, to get more results.<br/>Groups:  A list of resource groups.<br/>GroupIdentifiers:  A list of GroupIdentifier objects. Each identifier is an object that contains both the GroupName and the GroupArn.<br/>        """
        pass

    def search_resources(self, ResourceQuery: ResourceQuery, MaxResults: int=None, NextToken: str=None) -> SearchResourcesOutput:
        """Returns a list of AWS resource identifiers that matches a specified query. The query uses the same format as a resource query in a CreateGroup or UpdateGroupQuery operation.

        :param self:
        :param NextToken:  The NextToken value that is returned in a paginated <code>SearchResources</code> request. To get the next page of results, run the call again, add the NextToken parameter, and specify the NextToken value.<br/>MaxResults:  The maximum number of group member ARNs returned by <code>SearchResources</code> in paginated output. By default, this number is 50.<br/>ResourceQuery:  The search query, using the same formats that are supported for resource group definition.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Query</i> The query that defines a group or a search.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of the query. The valid values in this release are <code>TAG_FILTERS_1_0</code> and <code>CLOUDFORMATION_STACK_1_0</code>.  <i> <code>TAG_FILTERS_1_0:</code> </i> A JSON syntax that lets you specify a collection of simple tag filters for resource types and tags, as supported by the AWS Tagging API <a href="https://docs.aws.amazon.com/resourcegroupstagging/latest/APIReference/API_GetResources.html">GetResources</a> operation. If you specify more than one tag key, only resources that match all tag keys, and at least one value of each specified tag key, are returned in your query. If you specify more than one value for a tag key, a resource matches the filter if it has a tag key value that matches <i>any</i> of the specified values. For example, consider the following sample query for resources that have two tags, <code>Stage</code> and <code>Version</code>, with two values each. (<code>[{"Key":"Stage","Values":["Test","Deploy"]},{"Key":"Version","Values":["1","2"]}]</code>) The results of this query might include the following. <ul> <li> An EC2 instance that has the following two tags: <code>{"Key":"Stage","Value":"Deploy"}</code>, and <code>{"Key":"Version","Value":"2"}</code>  </li> <li> An S3 bucket that has the following two tags: {"Key":"Stage","Value":"Test"}, and {"Key":"Version","Value":"1"} </li> </ul> The query would not return the following results, however. The following EC2 instance does not have all tag keys specified in the filter, so it is rejected. The RDS database has all of the tag keys, but no values that match at least one of the specified tag key values in the filter. <ul> <li> An EC2 instance that has only the following tag: <code>{"Key":"Stage","Value":"Deploy"}</code>. </li> <li> An RDS database that has the following two tags: <code>{"Key":"Stage","Value":"Archived"}</code>, and <code>{"Key":"Version","Value":"4"}</code>  </li> </ul>  <i> <code>CLOUDFORMATION_STACK_1_0:</code> </i> A JSON syntax that lets you specify a CloudFormation stack ARN.<br/>
        :return: QueryErrors:  A list of <code>QueryError</code> objects. Each error is an object that contains <code>ErrorCode</code> and <code>Message</code> structures. Possible values for <code>ErrorCode</code> are <code>CLOUDFORMATION_STACK_INACTIVE</code> and <code>CLOUDFORMATION_STACK_NOT_EXISTING</code>.<br/>NextToken:  The NextToken value to include in a subsequent <code>SearchResources</code> request, to get more results.<br/>ResourceIdentifiers:  The ARNs and resource types of resources that are members of the group that you specified.<br/>        """
        pass

    def tag(self, Tags: Tags, Arn: str) -> TagOutput:
        """Adds tags to a resource group with the specified ARN. Existing tags on a resource group are not changed if they are not specified in the request parameters.

        :param self:
        :param Tags:  The tags to add to the specified resource. A tag is a string-to-string map of key-value pairs. Tag keys can have a maximum character length of 128 characters, and tag values can have a maximum length of 256 characters.<br/>Arn:  The ARN of the resource to which to add tags.<br/>
        :return: Tags:  The tags that have been added to the specified resource.<br/>Arn:  The ARN of the tagged resource.<br/>        """
        pass

    def untag(self, Keys: List[TagKey], Arn: str) -> UntagOutput:
        """Deletes specified tags from a specified resource.

        :param self:
        :param Keys:  The keys of the tags to be removed.<br/>Arn:  The ARN of the resource from which to remove tags.<br/>
        :return: Keys:  The keys of tags that have been removed.<br/>Arn:  The ARN of the resource from which tags have been removed.<br/>        """
        pass

    def update_group(self, GroupName: str, Description: str=None) -> UpdateGroupOutput:
        """Updates an existing group with a new or changed description. You cannot update the name of a resource group.

        :param self:
        :param Description:  The description of the resource group. Descriptions can have a maximum of 511 characters, including letters, numbers, hyphens, underscores, punctuation, and spaces.<br/>GroupName:  The name of the resource group for which you want to update its description.<br/>
        :return: Group:  The full description of the resource group after it has been updated.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the resource group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of a resource group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupArn</i> The ARN of a resource group.<br/>        """
        pass

    def update_group_query(self, ResourceQuery: ResourceQuery, GroupName: str) -> UpdateGroupQueryOutput:
        """Updates the resource query of a group.

        :param self:
        :param ResourceQuery:  The resource query that determines which AWS resources are members of the resource group.:<br/>GroupName:  The name of the resource group for which you want to edit the query.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Query</i> The query that defines a group or a search.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of the query. The valid values in this release are <code>TAG_FILTERS_1_0</code> and <code>CLOUDFORMATION_STACK_1_0</code>.  <i> <code>TAG_FILTERS_1_0:</code> </i> A JSON syntax that lets you specify a collection of simple tag filters for resource types and tags, as supported by the AWS Tagging API <a href="https://docs.aws.amazon.com/resourcegroupstagging/latest/APIReference/API_GetResources.html">GetResources</a> operation. If you specify more than one tag key, only resources that match all tag keys, and at least one value of each specified tag key, are returned in your query. If you specify more than one value for a tag key, a resource matches the filter if it has a tag key value that matches <i>any</i> of the specified values. For example, consider the following sample query for resources that have two tags, <code>Stage</code> and <code>Version</code>, with two values each. (<code>[{"Key":"Stage","Values":["Test","Deploy"]},{"Key":"Version","Values":["1","2"]}]</code>) The results of this query might include the following. <ul> <li> An EC2 instance that has the following two tags: <code>{"Key":"Stage","Value":"Deploy"}</code>, and <code>{"Key":"Version","Value":"2"}</code>  </li> <li> An S3 bucket that has the following two tags: {"Key":"Stage","Value":"Test"}, and {"Key":"Version","Value":"1"} </li> </ul> The query would not return the following results, however. The following EC2 instance does not have all tag keys specified in the filter, so it is rejected. The RDS database has all of the tag keys, but no values that match at least one of the specified tag key values in the filter. <ul> <li> An EC2 instance that has only the following tag: <code>{"Key":"Stage","Value":"Deploy"}</code>. </li> <li> An RDS database that has the following two tags: <code>{"Key":"Stage","Value":"Archived"}</code>, and <code>{"Key":"Version","Value":"4"}</code>  </li> </ul>  <i> <code>CLOUDFORMATION_STACK_1_0:</code> </i> A JSON syntax that lets you specify a CloudFormation stack ARN.<br/>
        :return: GroupQuery:  The resource query associated with the resource group after the update.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceQuery</i> The resource query which determines which AWS resources are members of the associated resource group.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResourceQuery</b>: The resource query which determines which AWS resources are members of the associated resource group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupName</i> The name of a resource group that is associated with a specific resource query.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Query</i> The query that defines a group or a search.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of the query. The valid values in this release are <code>TAG_FILTERS_1_0</code> and <code>CLOUDFORMATION_STACK_1_0</code>.  <i> <code>TAG_FILTERS_1_0:</code> </i> A JSON syntax that lets you specify a collection of simple tag filters for resource types and tags, as supported by the AWS Tagging API <a href="https://docs.aws.amazon.com/resourcegroupstagging/latest/APIReference/API_GetResources.html">GetResources</a> operation. If you specify more than one tag key, only resources that match all tag keys, and at least one value of each specified tag key, are returned in your query. If you specify more than one value for a tag key, a resource matches the filter if it has a tag key value that matches <i>any</i> of the specified values. For example, consider the following sample query for resources that have two tags, <code>Stage</code> and <code>Version</code>, with two values each. (<code>[{"Key":"Stage","Values":["Test","Deploy"]},{"Key":"Version","Values":["1","2"]}]</code>) The results of this query might include the following. <ul> <li> An EC2 instance that has the following two tags: <code>{"Key":"Stage","Value":"Deploy"}</code>, and <code>{"Key":"Version","Value":"2"}</code>  </li> <li> An S3 bucket that has the following two tags: {"Key":"Stage","Value":"Test"}, and {"Key":"Version","Value":"1"} </li> </ul> The query would not return the following results, however. The following EC2 instance does not have all tag keys specified in the filter, so it is rejected. The RDS database has all of the tag keys, but no values that match at least one of the specified tag key values in the filter. <ul> <li> An EC2 instance that has only the following tag: <code>{"Key":"Stage","Value":"Deploy"}</code>. </li> <li> An RDS database that has the following two tags: <code>{"Key":"Stage","Value":"Archived"}</code>, and <code>{"Key":"Version","Value":"4"}</code>  </li> </ul>  <i> <code>CLOUDFORMATION_STACK_1_0:</code> </i> A JSON syntax that lets you specify a CloudFormation stack ARN.<br/>        """
        pass



class ResourceGroupsTaggingAPI(BaseClient):
    """<fullname>Resource Groups Tagging API</fullname> This guide describes the API operations for the resource groups tagging. A tag is a label that you assign to an AWS resource. A tag consists of a key and a value, both of which you define. For example, if you have two Amazon EC2 instances, you might assign both a tag key of "Stack." But the value of "Stack" might be "Testing" for one and "Production" for the other. Tagging can help you organize your resources and enables you to simplify resource management, access management and cost allocation.  You can use the resource groups tagging API operations to complete the following tasks: <ul> <li> Tag and untag supported resources located in the specified region for the AWS account </li> <li> Use tag-based filters to search for resources located in the specified region for the AWS account </li> <li> List all existing tag keys in the specified region for the AWS account </li> <li> List all existing values for the specified key in the specified region for the AWS account </li> </ul> To make full use of the resource groups tagging API operations, you might need additional IAM permissions, including permission to access the resources of individual services as well as permission to view and apply tags to those resources. For more information, see <a href="http://docs.aws.amazon.com/awsconsolehelpdocs/latest/gsg/obtaining-permissions-for-resource-groups.html">Obtaining Permissions for Resource Groups and Tag Editor</a>. You can use the Resource Groups Tagging API to tag resources for the following AWS services. <ul> <li> Alexa for Business (a4b) </li> <li> API Gateway </li> <li> AWS AppStream </li> <li> AWS AppSync </li> <li> Amazon Athena </li> <li> Amazon Aurora </li> <li> AWS Certificate Manager </li> <li> AWS Certificate Manager Private CA </li> <li> Amazon Cloud Directory </li> <li> AWS CloudFormation </li> <li> Amazon CloudFront </li> <li> AWS CloudHSM </li> <li> AWS CloudTrail </li> <li> Amazon CloudWatch (alarms only) </li> <li> Amazon CloudWatch Events </li> <li> Amazon CloudWatch Logs </li> <li> AWS CodeBuild </li> <li> AWS CodeStar </li> <li> Amazon Cognito Identity </li> <li> Amazon Cognito User Pools </li> <li> Amazon Comprehend </li> <li> AWS Config </li> <li> AWS Data Pipeline </li> <li> AWS Database Migration Service </li> <li> AWS Datasync </li> <li> AWS Direct Connect </li> <li> AWS Directory Service </li> <li> Amazon DynamoDB </li> <li> Amazon EBS </li> <li> Amazon EC2 </li> <li> Amazon ECR </li> <li> Amazon ECS </li> <li> AWS Elastic Beanstalk </li> <li> Amazon Elastic File System </li> <li> Elastic Load Balancing </li> <li> Amazon ElastiCache </li> <li> Amazon Elasticsearch Service </li> <li> AWS Elemental MediaLive </li> <li> AWS Elemental MediaPackage </li> <li> AWS Elemental MediaTailor </li> <li> Amazon EMR </li> <li> Amazon FSx </li> <li> Amazon Glacier </li> <li> AWS Glue </li> <li> Amazon Inspector </li> <li> AWS IoT Analytics </li> <li> AWS IoT Core </li> <li> AWS IoT Device Defender </li> <li> AWS IoT Device Management </li> <li> AWS IoT Greengrass </li> <li> AWS Key Management Service </li> <li> Amazon Kinesis </li> <li> Amazon Kinesis Data Analytics </li> <li> Amazon Kinesis Data Firehose </li> <li> AWS Lambda </li> <li> AWS License Manager </li> <li> Amazon Machine Learning </li> <li> Amazon MQ </li> <li> Amazon MSK </li> <li> Amazon Neptune </li> <li> AWS OpsWorks </li> <li> Amazon RDS </li> <li> Amazon Redshift </li> <li> AWS Resource Access Manager </li> <li> AWS Resource Groups </li> <li> AWS RoboMaker </li> <li> Amazon Route 53 </li> <li> Amazon Route 53 Resolver </li> <li> Amazon S3 (buckets only) </li> <li> Amazon SageMaker </li> <li> AWS Secrets Manager </li> <li> AWS Service Catalog </li> <li> Amazon Simple Notification Service (SNS) </li> <li> Amazon Simple Queue Service (SQS) </li> <li> AWS Simple System Manager (SSM) </li> <li> AWS Step Functions </li> <li> AWS Storage Gateway </li> <li> AWS Transfer for SFTP </li> <li> Amazon VPC </li> <li> Amazon WorkSpaces </li> </ul>"""


    class GetResourcesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_resources` for valid parameters"""
            pass
    class GetTagKeysPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_tag_keys` for valid parameters"""
            pass
    class GetTagValuesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_tag_values` for valid parameters"""
            pass


    class GetResourcesInput(Mapping):
        pass
    
    class GetResourcesOutput(Mapping):
        pass
    
    class TagFilter(Mapping):
        pass
    
    class AmazonResourceType(object):
        pass
    
    class GetTagKeysInput(Mapping):
        pass
    
    class GetTagKeysOutput(Mapping):
        pass
    
    class GetTagValuesInput(Mapping):
        pass
    
    class GetTagValuesOutput(Mapping):
        pass
    
    class TagResourcesInput(Mapping):
        pass
    
    class TagResourcesOutput(Mapping):
        pass
    
    class ResourceARN(object):
        pass
    
    class TagMap(object):
        pass
    
    class UntagResourcesInput(Mapping):
        pass
    
    class UntagResourcesOutput(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    def get_resources(self, PaginationToken: str=None, TagFilters: List[TagFilter]=None, ResourcesPerPage: int=None, TagsPerPage: int=None, ResourceTypeFilters: List[AmazonResourceType]=None) -> GetResourcesOutput:
        """Returns all the tagged or previously tagged resources that are located in the specified region for the AWS account. You can optionally specify <i>filters</i> (tags and resource types) in your request, depending on what information you want returned. The response includes all tags that are associated with the requested resources. <note> You can check the <code>PaginationToken</code> response parameter to determine if a query completed. Queries can occasionally return fewer results on a page than allowed. The <code>PaginationToken</code> response parameter value is <code>null</code> <i>only</i> when there are no more results to display.  </note>

        :param self:
        :param ResourceTypeFilters:  The constraints on the resources that you want returned. The format of each resource type is <code>service[:resourceType]</code>. For example, specifying a resource type of <code>ec2</code> returns all Amazon EC2 resources (which includes EC2 instances). Specifying a resource type of <code>ec2:instance</code> returns only EC2 instances.  The string for each service name and resource type is the same as that embedded in a resource's Amazon Resource Name (ARN). Consult the <i>AWS General Reference</i> for the following: <ul> <li> For a list of service name strings, see <a href="http://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#genref-aws-service-namespaces">AWS Service Namespaces</a>. </li> <li> For resource type strings, see <a href="http://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#arns-syntax">Example ARNs</a>. </li> <li> For more information about ARNs, see <a href="http://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a>. </li> </ul> You can specify multiple resource types by using an array. The array can include up to 100 items. Note that the length constraint requirement applies to each resource type filter. <br/>TagsPerPage:  A limit that restricts the number of tags (key and value pairs) returned by GetResources in paginated output. A resource with no tags is counted as having one tag (one key and value pair).  <code>GetResources</code> does not split a resource and its associated tags across pages. If the specified <code>TagsPerPage</code> would cause such a break, a <code>PaginationToken</code> is returned in place of the affected resource and its tags. Use that token in another request to get the remaining data. For example, if you specify a <code>TagsPerPage</code> of <code>100</code> and the account has 22 resources with 10 tags each (meaning that each resource has 10 key and value pairs), the output will consist of 3 pages, with the first page displaying the first 10 resources, each with its 10 tags, the second page displaying the next 10 resources each with its 10 tags, and the third page displaying the remaining 2 resources, each with its 10 tags. You can set <code>TagsPerPage</code> to a minimum of 100 items and the maximum of 500 items.<br/>ResourcesPerPage:  A limit that restricts the number of resources returned by GetResources in paginated output. You can set ResourcesPerPage to a minimum of 1 item and the maximum of 100 items. <br/>TagFilters:  A list of TagFilters (keys and values). Each TagFilter specified must contain a key with values as optional. A request can include up to 50 keys, and each key can include up to 20 values.  Note the following when deciding how to use TagFilters: <ul> <li> If you <i>do</i> specify a TagFilter, the response returns only those resources that are currently associated with the specified tag.  </li> <li> If you <i>don't</i> specify a TagFilter, the response includes all resources that were ever associated with tags. Resources that currently don't have associated tags are shown with an empty tag set, like this: <code>"Tags": []</code>. </li> <li> If you specify more than one filter in a single request, the response returns only those resources that satisfy all specified filters. </li> <li> If you specify a filter that contains more than one value for a key, the response returns resources that match any of the specified values for that key. </li> <li> If you don't specify any values for a key, the response returns resources that are tagged with that key irrespective of the value. For example, for filters: filter1 = {key1, {value1}}, filter2 = {key2, {value2,value3,value4}} , filter3 = {key3}: <ul> <li> GetResources( {filter1} ) returns resources tagged with key1=value1 </li> <li> GetResources( {filter2} ) returns resources tagged with key2=value2 or key2=value3 or key2=value4 </li> <li> GetResources( {filter3} ) returns resources tagged with any tag containing key3 as its tag key, irrespective of its value </li> <li> GetResources( {filter1,filter2,filter3} ) returns resources tagged with ( key1=value1) and ( key2=value2 or key2=value3 or key2=value4) and (key3, irrespective of the value) </li> </ul> </li> </ul><br/>PaginationToken:  A string that indicates that additional data is available. Leave this value empty for your initial request. If the response includes a <code>PaginationToken</code>, use that string for this value to request an additional page of data.<br/>
        :return: ResourceTagMappingList:  A list of resource ARNs and the tags (keys and values) associated with each.<br/>PaginationToken:  A string that indicates that the response contains more data than can be returned in a single response. To receive additional data, specify this string for the <code>PaginationToken</code> value in a subsequent request.<br/>        """
        pass

    def get_tag_keys(self, PaginationToken: str=None) -> GetTagKeysOutput:
        """Returns all tag keys in the specified region for the AWS account.

        :param self:
        :param PaginationToken:  A string that indicates that additional data is available. Leave this value empty for your initial request. If the response includes a PaginationToken, use that string for this value to request an additional page of data.<br/>
        :return: TagKeys:  A list of all tag keys in the AWS account.<br/>PaginationToken:  A string that indicates that the response contains more data than can be returned in a single response. To receive additional data, specify this string for the <code>PaginationToken</code> value in a subsequent request.<br/>        """
        pass

    def get_tag_values(self, Key: str, PaginationToken: str=None) -> GetTagValuesOutput:
        """Returns all tag values for the specified key in the specified region for the AWS account.

        :param self:
        :param Key:  The key for which you want to list all existing values in the specified region for the AWS account.<br/>PaginationToken:  A string that indicates that additional data is available. Leave this value empty for your initial request. If the response includes a PaginationToken, use that string for this value to request an additional page of data.<br/>
        :return: TagValues:  A list of all tag values for the specified key in the AWS account.<br/>PaginationToken:  A string that indicates that the response contains more data than can be returned in a single response. To receive additional data, specify this string for the <code>PaginationToken</code> value in a subsequent request.<br/>        """
        pass

    def tag_resources(self, Tags: TagMap, ResourceARNList: List[ResourceARN]) -> TagResourcesOutput:
        """Applies one or more tags to the specified resources. Note the following: <ul> <li> Not all resources can have tags. For a list of resources that support tagging, see <a href="http://docs.aws.amazon.com/ARG/latest/userguide/supported-resources.html">Supported Resources</a> in the <i>AWS Resource Groups User Guide</i>. </li> <li> Each resource can have up to 50 tags. For other limits, see <a href="http://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Using_Tags.html#tag-restrictions">Tag Restrictions</a> in the <i>Amazon EC2 User Guide for Linux Instances</i>. </li> <li> You can only tag resources that are located in the specified region for the AWS account. </li> <li> To add tags to a resource, you need the necessary permissions for the service that the resource belongs to as well as permissions for adding tags. For more information, see <a href="http://docs.aws.amazon.com/ARG/latest/userguide/obtaining-permissions-for-tagging.html">Obtaining Permissions for Tagging</a> in the <i>AWS Resource Groups User Guide</i>. </li> </ul>

        :param self:
        :param Tags:  The tags that you want to add to the specified resources. A tag consists of a key and a value that you define.<br/>ResourceARNList:  A list of ARNs. An ARN (Amazon Resource Name) uniquely identifies a resource. You can specify a minimum of 1 and a maximum of 20 ARNs (resources) to tag. An ARN can be set to a maximum of 1600 characters. For more information, see <a href="http://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the <i>AWS General Reference</i>.<br/>
        :return: FailedResourcesMap:  Details of resources that could not be tagged. An error code, status code, and error message are returned for each failed item.<br/>        """
        pass

    def untag_resources(self, TagKeys: List[TagKey], ResourceARNList: List[ResourceARN]) -> UntagResourcesOutput:
        """Removes the specified tags from the specified resources. When you specify a tag key, the action removes both that key and its associated value. The operation succeeds even if you attempt to remove tags from a resource that were already removed. Note the following: <ul> <li> To remove tags from a resource, you need the necessary permissions for the service that the resource belongs to as well as permissions for removing tags. For more information, see <a href="http://docs.aws.amazon.com/ARG/latest/userguide/obtaining-permissions-for-tagging.html">Obtaining Permissions for Tagging</a> in the <i>AWS Resource Groups User Guide</i>. </li> <li> You can only tag resources that are located in the specified region for the AWS account. </li> </ul>

        :param self:
        :param TagKeys:  A list of the tag keys that you want to remove from the specified resources.<br/>ResourceARNList:  A list of ARNs. An ARN (Amazon Resource Name) uniquely identifies a resource. You can specify a minimum of 1 and a maximum of 20 ARNs (resources) to untag. An ARN can be set to a maximum of 1600 characters. For more information, see <a href="http://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the <i>AWS General Reference</i>.<br/>
        :return: FailedResourcesMap:  Details of resources that could not be untagged. An error code, status code, and error message are returned for each failed item.<br/>        """
        pass



class RoboMaker(BaseClient):
    """This section provides documentation for the AWS RoboMaker API operations."""


    class ListDeploymentJobsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_deployment_jobs` for valid parameters"""
            pass
    class ListFleetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_fleets` for valid parameters"""
            pass
    class ListRobotApplicationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_robot_applications` for valid parameters"""
            pass
    class ListRobotsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_robots` for valid parameters"""
            pass
    class ListSimulationApplicationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_simulation_applications` for valid parameters"""
            pass
    class ListSimulationJobsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_simulation_jobs` for valid parameters"""
            pass


    class BatchDescribeSimulationJobRequest(Mapping):
        pass
    
    class BatchDescribeSimulationJobResponse(Mapping):
        pass
    
    class Arn(object):
        pass
    
    class CancelDeploymentJobRequest(Mapping):
        pass
    
    class CancelDeploymentJobResponse(Mapping):
        pass
    
    class CancelSimulationJobRequest(Mapping):
        pass
    
    class CancelSimulationJobResponse(Mapping):
        pass
    
    class CreateDeploymentJobRequest(Mapping):
        pass
    
    class CreateDeploymentJobResponse(Mapping):
        pass
    
    class DeploymentConfig(Mapping):
        pass
    
    class DeploymentApplicationConfig(Mapping):
        pass
    
    class TagMap(object):
        pass
    
    class CreateFleetRequest(Mapping):
        pass
    
    class CreateFleetResponse(Mapping):
        pass
    
    class CreateRobotRequest(Mapping):
        pass
    
    class CreateRobotResponse(Mapping):
        pass
    
    class CreateRobotApplicationRequest(Mapping):
        pass
    
    class CreateRobotApplicationResponse(Mapping):
        pass
    
    class SourceConfig(Mapping):
        pass
    
    class RobotSoftwareSuite(Mapping):
        pass
    
    class CreateRobotApplicationVersionRequest(Mapping):
        pass
    
    class CreateRobotApplicationVersionResponse(Mapping):
        pass
    
    class CreateSimulationApplicationRequest(Mapping):
        pass
    
    class CreateSimulationApplicationResponse(Mapping):
        pass
    
    class SimulationSoftwareSuite(Mapping):
        pass
    
    class RenderingEngine(Mapping):
        pass
    
    class CreateSimulationApplicationVersionRequest(Mapping):
        pass
    
    class CreateSimulationApplicationVersionResponse(Mapping):
        pass
    
    class CreateSimulationJobRequest(Mapping):
        pass
    
    class CreateSimulationJobResponse(Mapping):
        pass
    
    class OutputLocation(Mapping):
        pass
    
    class JobDuration(object):
        pass
    
    class RobotApplicationConfig(Mapping):
        pass
    
    class SimulationApplicationConfig(Mapping):
        pass
    
    class VPCConfig(Mapping):
        pass
    
    class DeleteFleetRequest(Mapping):
        pass
    
    class DeleteFleetResponse(Mapping):
        pass
    
    class DeleteRobotRequest(Mapping):
        pass
    
    class DeleteRobotResponse(Mapping):
        pass
    
    class DeleteRobotApplicationRequest(Mapping):
        pass
    
    class DeleteRobotApplicationResponse(Mapping):
        pass
    
    class DeleteSimulationApplicationRequest(Mapping):
        pass
    
    class DeleteSimulationApplicationResponse(Mapping):
        pass
    
    class DeregisterRobotRequest(Mapping):
        pass
    
    class DeregisterRobotResponse(Mapping):
        pass
    
    class DescribeDeploymentJobRequest(Mapping):
        pass
    
    class DescribeDeploymentJobResponse(Mapping):
        pass
    
    class DescribeFleetRequest(Mapping):
        pass
    
    class DescribeFleetResponse(Mapping):
        pass
    
    class DescribeRobotRequest(Mapping):
        pass
    
    class DescribeRobotResponse(Mapping):
        pass
    
    class DescribeRobotApplicationRequest(Mapping):
        pass
    
    class DescribeRobotApplicationResponse(Mapping):
        pass
    
    class DescribeSimulationApplicationRequest(Mapping):
        pass
    
    class DescribeSimulationApplicationResponse(Mapping):
        pass
    
    class DescribeSimulationJobRequest(Mapping):
        pass
    
    class DescribeSimulationJobResponse(Mapping):
        pass
    
    class ListDeploymentJobsRequest(Mapping):
        pass
    
    class ListDeploymentJobsResponse(Mapping):
        pass
    
    class Filter(Mapping):
        pass
    
    class ListFleetsRequest(Mapping):
        pass
    
    class ListFleetsResponse(Mapping):
        pass
    
    class ListRobotApplicationsRequest(Mapping):
        pass
    
    class ListRobotApplicationsResponse(Mapping):
        pass
    
    class ListRobotsRequest(Mapping):
        pass
    
    class ListRobotsResponse(Mapping):
        pass
    
    class ListSimulationApplicationsRequest(Mapping):
        pass
    
    class ListSimulationApplicationsResponse(Mapping):
        pass
    
    class ListSimulationJobsRequest(Mapping):
        pass
    
    class ListSimulationJobsResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class RegisterRobotRequest(Mapping):
        pass
    
    class RegisterRobotResponse(Mapping):
        pass
    
    class RestartSimulationJobRequest(Mapping):
        pass
    
    class RestartSimulationJobResponse(Mapping):
        pass
    
    class SyncDeploymentJobRequest(Mapping):
        pass
    
    class SyncDeploymentJobResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateRobotApplicationRequest(Mapping):
        pass
    
    class UpdateRobotApplicationResponse(Mapping):
        pass
    
    class UpdateSimulationApplicationRequest(Mapping):
        pass
    
    class UpdateSimulationApplicationResponse(Mapping):
        pass
    
    def batch_describe_simulation_job(self, jobs: List[Arn]) -> BatchDescribeSimulationJobResponse:
        """Describes one or more simulation jobs.

        :param self:
        :param jobs:  A list of Amazon Resource Names (ARNs) of simulation jobs to describe.<br/>
        :return: unprocessedJobs:  A list of unprocessed simulation job Amazon Resource Names (ARNs).<br/>jobs:  A list of simulation jobs.<br/>        """
        pass

    def cancel_deployment_job(self, job: str) -> CancelDeploymentJobResponse:
        """Cancels the specified deployment job.

        :param self:
        :param job:  The deployment job ARN to cancel.<br/>
        :return:         """
        pass

    def cancel_simulation_job(self, job: str) -> CancelSimulationJobResponse:
        """Cancels the specified simulation job.

        :param self:
        :param job:  The simulation job ARN to cancel.<br/>
        :return:         """
        pass

    def create_deployment_job(self, deploymentApplicationConfigs: List[DeploymentApplicationConfig], fleet: str, clientRequestToken: str, deploymentConfig: DeploymentConfig=None, tags: TagMap=None) -> CreateDeploymentJobResponse:
        """Deploys a specific version of a robot application to robots in a fleet. The robot application must have a numbered <code>applicationVersion</code> for consistency reasons. To create a new version, use <code>CreateRobotApplicationVersion</code> or see <a href="https://docs.aws.amazon.com/robomaker/latest/dg/create-robot-application-version.html">Creating a Robot Application Version</a>.  <note> After 90 days, deployment jobs expire and will be deleted. They will no longer be accessible.  </note>

        :param self:
        :param tags:  A map that contains tag keys and tag values that are attached to the deployment job.<br/>deploymentApplicationConfigs:  The deployment application configuration.<br/>fleet:  The Amazon Resource Name (ARN) of the fleet to deploy.<br/>clientRequestToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request.<br/>deploymentConfig:  The requested deployment configuration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>failureThresholdPercentage</i> The percentage of deployments that need to fail before stopping deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>concurrentDeploymentPercentage</i> The percentage of robots receiving the deployment at the same time.<br/>
        :return: tags:  The list of all tags added to the deployment job.<br/>deploymentConfig:  The deployment configuration.:<br/>createdAt:  The time, in milliseconds since the epoch, when the fleet was created.<br/>failureCode:  The failure code of the simulation job if it failed: <dl> <dt>BadPermissionError</dt> <dd> AWS Greengrass requires a service-level role permission to access other services. The role must include the <a href="https://console.aws.amazon.com/iam/home?#/policies/arn:aws:iam::aws:policy/service-role/AWSGreengrassResourceAccessRolePolicy$jsonEditor"> <code>AWSGreengrassResourceAccessRolePolicy</code> managed policy</a>.  </dd> <dt>ExtractingBundleFailure</dt> <dd> The robot application could not be extracted from the bundle. </dd> <dt>FailureThresholdBreached</dt> <dd> The percentage of robots that could not be updated exceeded the percentage set for the deployment. </dd> <dt>GreengrassDeploymentFailed</dt> <dd> The robot application could not be deployed to the robot. </dd> <dt>GreengrassGroupVersionDoesNotExist</dt> <dd> The AWS Greengrass group or version associated with a robot is missing. </dd> <dt>InternalServerError</dt> <dd> An internal error has occurred. Retry your request, but if the problem persists, contact us with details. </dd> <dt>MissingRobotApplicationArchitecture</dt> <dd> The robot application does not have a source that matches the architecture of the robot. </dd> <dt>MissingRobotDeploymentResource</dt> <dd> One or more of the resources specified for the robot application are missing. For example, does the robot application have the correct launch package and launch file? </dd> <dt>PostLaunchFileFailure</dt> <dd> The post-launch script failed. </dd> <dt>PreLaunchFileFailure</dt> <dd> The pre-launch script failed. </dd> <dt>ResourceNotFound</dt> <dd> One or more deployment resources are missing. For example, do robot application source bundles still exist?  </dd> <dt>RobotDeploymentNoResponse</dt> <dd> There is no response from the robot. It might not be powered on or connected to the internet. </dd> </dl><br/>failureReason:  The failure reason of the deployment job if it failed.<br/>deploymentApplicationConfigs:  The deployment application configuration.<br/>status:  The status of the deployment job.<br/>fleet:  The target fleet for the deployment job.<br/>arn:  The Amazon Resource Name (ARN) of the deployment job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>failureThresholdPercentage</i> The percentage of deployments that need to fail before stopping deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>concurrentDeploymentPercentage</i> The percentage of robots receiving the deployment at the same time.<br/>        """
        pass

    def create_fleet(self, name: str, tags: TagMap=None) -> CreateFleetResponse:
        """Creates a fleet, a logical group of robots running the same robot application.

        :param self:
        :param tags:  A map that contains tag keys and tag values that are attached to the fleet.<br/>name:  The name of the fleet.<br/>
        :return: tags:  The list of all tags added to the fleet.<br/>createdAt:  The time, in milliseconds since the epoch, when the fleet was created.<br/>name:  The name of the fleet.<br/>arn:  The Amazon Resource Name (ARN) of the fleet.<br/>        """
        pass

    def create_robot(self, greengrassGroupId: str, architecture: str, name: str, tags: TagMap=None) -> CreateRobotResponse:
        """Creates a robot.

        :param self:
        :param tags:  A map that contains tag keys and tag values that are attached to the robot.<br/>greengrassGroupId:  The Greengrass group id.<br/>architecture:  The target architecture of the robot.<br/>name:  The name for the robot.<br/>
        :return: tags:  The list of all tags added to the robot.<br/>architecture:  The target architecture of the robot.<br/>greengrassGroupId:  The Amazon Resource Name (ARN) of the Greengrass group associated with the robot.<br/>createdAt:  The time, in milliseconds since the epoch, when the robot was created.<br/>name:  The name of the robot.<br/>arn:  The Amazon Resource Name (ARN) of the robot.<br/>        """
        pass

    def create_robot_application(self, robotSoftwareSuite: RobotSoftwareSuite, sources: List[SourceConfig], name: str, tags: TagMap=None) -> CreateRobotApplicationResponse:
        """Creates a robot application. 

        :param self:
        :param tags:  A map that contains tag keys and tag values that are attached to the robot application.<br/>robotSoftwareSuite:  The robot software suite used by the robot application.:<br/>sources:  The sources of the robot application.<br/>name:  The name of the robot application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the robot software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the robot software suite.<br/>
        :return: tags:  The list of all tags added to the robot application.<br/>revisionId:  The revision id of the robot application.<br/>lastUpdatedAt:  The time, in milliseconds since the epoch, when the robot application was last updated.<br/>robotSoftwareSuite:  The robot software suite used by the robot application.:<br/>sources:  The sources of the robot application.<br/>version:  The version of the robot application.<br/>name:  The name of the robot application.<br/>arn:  The Amazon Resource Name (ARN) of the robot application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the robot software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the robot software suite.<br/>        """
        pass

    def create_robot_application_version(self, application: str, currentRevisionId: str=None) -> CreateRobotApplicationVersionResponse:
        """Creates a version of a robot application.

        :param self:
        :param currentRevisionId:  The current revision id for the robot application. If you provide a value and it matches the latest revision ID, a new version will be created.<br/>application:  The application information for the robot application.<br/>
        :return: revisionId:  The revision id of the robot application.<br/>lastUpdatedAt:  The time, in milliseconds since the epoch, when the robot application was last updated.<br/>robotSoftwareSuite:  The robot software suite used by the robot application.:<br/>sources:  The sources of the robot application.<br/>version:  The version of the robot application.<br/>name:  The name of the robot application.<br/>arn:  The Amazon Resource Name (ARN) of the robot application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the robot software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the robot software suite.<br/>        """
        pass

    def create_simulation_application(self, renderingEngine: RenderingEngine, robotSoftwareSuite: RobotSoftwareSuite, simulationSoftwareSuite: SimulationSoftwareSuite, sources: List[SourceConfig], name: str, tags: TagMap=None) -> CreateSimulationApplicationResponse:
        """Creates a simulation application.

        :param self:
        :param tags:  A map that contains tag keys and tag values that are attached to the simulation application.<br/>renderingEngine:  The rendering engine for the simulation application.:<br/>robotSoftwareSuite:  The robot software suite of the simulation application.:<br/>simulationSoftwareSuite:  The simulation software suite used by the simulation application.:<br/>sources:  The sources of the simulation application.<br/>name:  The name of the simulation application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the simulation software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the simulation software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the robot software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the robot software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the rendering engine.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the rendering engine.<br/>
        :return: tags:  The list of all tags added to the simulation application.<br/>revisionId:  The revision id of the simulation application.<br/>lastUpdatedAt:  The time, in milliseconds since the epoch, when the simulation application was last updated.<br/>renderingEngine:  The rendering engine for the simulation application.:<br/>robotSoftwareSuite:  Information about the robot software suite.:<br/>simulationSoftwareSuite:  The simulation software suite used by the simulation application.:<br/>sources:  The sources of the simulation application.<br/>version:  The version of the simulation application.<br/>name:  The name of the simulation application.<br/>arn:  The Amazon Resource Name (ARN) of the simulation application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the simulation software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the simulation software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the robot software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the robot software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the rendering engine.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the rendering engine.<br/>        """
        pass

    def create_simulation_application_version(self, application: str, currentRevisionId: str=None) -> CreateSimulationApplicationVersionResponse:
        """Creates a simulation application with a specific revision id.

        :param self:
        :param currentRevisionId:  The current revision id for the simulation application. If you provide a value and it matches the latest revision ID, a new version will be created.<br/>application:  The application information for the simulation application.<br/>
        :return: revisionId:  The revision ID of the simulation application.<br/>lastUpdatedAt:  The time, in milliseconds since the epoch, when the simulation application was last updated.<br/>renderingEngine:  The rendering engine for the simulation application.:<br/>robotSoftwareSuite:  Information about the robot software suite.:<br/>simulationSoftwareSuite:  The simulation software suite used by the simulation application.:<br/>sources:  The sources of the simulation application.<br/>version:  The version of the simulation application.<br/>name:  The name of the simulation application.<br/>arn:  The Amazon Resource Name (ARN) of the simulation application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the simulation software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the simulation software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the robot software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the robot software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the rendering engine.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the rendering engine.<br/>        """
        pass

    def create_simulation_job(self, iamRole: str, maxJobDurationInSeconds: JobDuration, clientRequestToken: str=None, outputLocation: OutputLocation=None, failureBehavior: str=None, robotApplications: List[RobotApplicationConfig]=None, simulationApplications: List[SimulationApplicationConfig]=None, tags: TagMap=None, vpcConfig: VPCConfig=None) -> CreateSimulationJobResponse:
        """Creates a simulation job. <note> After 90 days, simulation jobs expire and will be deleted. They will no longer be accessible.  </note>

        :param self:
        :param vpcConfig:  If your simulation job accesses resources in a VPC, you provide this parameter identifying the list of security group IDs and subnet IDs. These must belong to the same VPC. You must provide at least one security group and one subnet ID. :<br/>tags:  A map that contains tag keys and tag values that are attached to the simulation job.<br/>simulationApplications:  The simulation application to use in the simulation job.<br/>robotApplications:  The robot application to use in the simulation job.<br/>failureBehavior:  The failure behavior the simulation job. <dl> <dt>Continue</dt> <dd> Restart the simulation job in the same host instance. </dd> <dt>Fail</dt> <dd> Stop the simulation job and terminate the instance. </dd> </dl><br/>iamRole:  The IAM role name that allows the simulation instance to call the AWS APIs that are specified in its associated policies on your behalf. This is how credentials are passed in to your simulation job. <br/>maxJobDurationInSeconds:  The maximum simulation job duration in seconds (up to 14 days or 1,209,600 seconds. When <code>maxJobDurationInSeconds</code> is reached, the simulation job will status will transition to <code>Completed</code>.<br/>outputLocation:  Location for output files generated by the simulation job.:<br/>clientRequestToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>s3Prefix</i> The S3 folder in the <code>s3Bucket</code> where output files will be placed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>s3Bucket</i> The S3 bucket for output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>assignPublicIp</i> A boolean indicating whether to assign a public IP address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>securityGroups</i> A list of one or more security groups IDs in your VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>subnets</i> A list of one or more subnet IDs in your VPC.<br/>
        :return: vpcConfig:  Information about the vpc configuration.:<br/>tags:  The list of all tags added to the simulation job.<br/>simulationApplications:  The simulation application used by the simulation job.<br/>robotApplications:  The robot application used by the simulation job.<br/>iamRole:  The IAM role that allows the simulation job to call the AWS APIs that are specified in its associated policies on your behalf.<br/>simulationTimeMillis:  The simulation job execution duration in milliseconds.<br/>maxJobDurationInSeconds:  The maximum simulation job duration in seconds. <br/>outputLocation:  Simulation job output files location.:<br/>clientRequestToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request.<br/>failureCode:  The failure code of the simulation job if it failed: <dl> <dt>InternalServiceError</dt> <dd> Internal service error. </dd> <dt>RobotApplicationCrash</dt> <dd> Robot application exited abnormally. </dd> <dt>SimulationApplicationCrash</dt> <dd>  Simulation application exited abnormally. </dd> <dt>BadPermissionsRobotApplication</dt> <dd> Robot application bundle could not be downloaded. </dd> <dt>BadPermissionsSimulationApplication</dt> <dd> Simulation application bundle could not be downloaded. </dd> <dt>BadPermissionsS3Output</dt> <dd> Unable to publish outputs to customer-provided S3 bucket. </dd> <dt>BadPermissionsCloudwatchLogs</dt> <dd> Unable to publish logs to customer-provided CloudWatch Logs resource. </dd> <dt>SubnetIpLimitExceeded</dt> <dd> Subnet IP limit exceeded. </dd> <dt>ENILimitExceeded</dt> <dd> ENI limit exceeded. </dd> <dt>BadPermissionsUserCredentials</dt> <dd> Unable to use the Role provided. </dd> <dt>InvalidBundleRobotApplication</dt> <dd> Robot bundle cannot be extracted (invalid format, bundling error, or other issue). </dd> <dt>InvalidBundleSimulationApplication</dt> <dd> Simulation bundle cannot be extracted (invalid format, bundling error, or other issue). </dd> <dt>RobotApplicationVersionMismatchedEtag</dt> <dd> Etag for RobotApplication does not match value during version creation. </dd> <dt>SimulationApplicationVersionMismatchedEtag</dt> <dd> Etag for SimulationApplication does not match value during version creation. </dd> </dl><br/>failureBehavior:  the failure behavior for the simulation job.<br/>lastUpdatedAt:  The time, in milliseconds since the epoch, when the simulation job was last updated.<br/>lastStartedAt:  The time, in milliseconds since the epoch, when the simulation job was last started.<br/>status:  The status of the simulation job.<br/>arn:  The Amazon Resource Name (ARN) of the simulation job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>s3Prefix</i> The S3 folder in the <code>s3Bucket</code> where output files will be placed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>s3Bucket</i> The S3 bucket for output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>assignPublicIp</i> A boolean indicating if a public IP was assigned.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vpcId</i> The VPC ID associated with your simulation job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>securityGroups</i> A list of security group IDs associated with the simulation job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>subnets</i> A list of subnet IDs associated with the simulation job.<br/>        """
        pass

    def delete_fleet(self, fleet: str) -> DeleteFleetResponse:
        """Deletes a fleet.

        :param self:
        :param fleet:  The Amazon Resource Name (ARN) of the fleet.<br/>
        :return:         """
        pass

    def delete_robot(self, robot: str) -> DeleteRobotResponse:
        """Deletes a robot.

        :param self:
        :param robot:  The Amazon Resource Name (ARN) of the robot.<br/>
        :return:         """
        pass

    def delete_robot_application(self, application: str, applicationVersion: str=None) -> DeleteRobotApplicationResponse:
        """Deletes a robot application.

        :param self:
        :param applicationVersion:  The version of the robot application to delete.<br/>application:  The Amazon Resource Name (ARN) of the the robot application.<br/>
        :return:         """
        pass

    def delete_simulation_application(self, application: str, applicationVersion: str=None) -> DeleteSimulationApplicationResponse:
        """Deletes a simulation application.

        :param self:
        :param applicationVersion:  The version of the simulation application to delete.<br/>application:  The application information for the simulation application to delete.<br/>
        :return:         """
        pass

    def deregister_robot(self, robot: str, fleet: str) -> DeregisterRobotResponse:
        """Deregisters a robot.

        :param self:
        :param robot:  The Amazon Resource Name (ARN) of the robot.<br/>fleet:  The Amazon Resource Name (ARN) of the fleet.<br/>
        :return: robot:  The Amazon Resource Name (ARN) of the robot.<br/>fleet:  The Amazon Resource Name (ARN) of the fleet.<br/>        """
        pass

    def describe_deployment_job(self, job: str) -> DescribeDeploymentJobResponse:
        """Describes a deployment job.

        :param self:
        :param job:  The Amazon Resource Name (ARN) of the deployment job.<br/>
        :return: tags:  The list of all tags added to the specified deployment job.<br/>robotDeploymentSummary:  A list of robot deployment summaries.<br/>createdAt:  The time, in milliseconds since the epoch, when the deployment job was created.<br/>failureCode:  The deployment job failure code.<br/>failureReason:  A short description of the reason why the deployment job failed.<br/>deploymentApplicationConfigs:  The deployment application configuration.<br/>deploymentConfig:  The deployment configuration.:<br/>status:  The status of the deployment job.<br/>fleet:  The Amazon Resource Name (ARN) of the fleet.<br/>arn:  The Amazon Resource Name (ARN) of the deployment job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>failureThresholdPercentage</i> The percentage of deployments that need to fail before stopping deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>concurrentDeploymentPercentage</i> The percentage of robots receiving the deployment at the same time.<br/>        """
        pass

    def describe_fleet(self, fleet: str) -> DescribeFleetResponse:
        """Describes a fleet.

        :param self:
        :param fleet:  The Amazon Resource Name (ARN) of the fleet.<br/>
        :return: tags:  The list of all tags added to the specified fleet.<br/>lastDeploymentTime:  The time of the last deployment.<br/>lastDeploymentJob:  The Amazon Resource Name (ARN) of the last deployment job.<br/>lastDeploymentStatus:  The status of the last deployment.<br/>createdAt:  The time, in milliseconds since the epoch, when the fleet was created.<br/>robots:  A list of robots.<br/>arn:  The Amazon Resource Name (ARN) of the fleet.<br/>name:  The name of the fleet.<br/>        """
        pass

    def describe_robot(self, robot: str) -> DescribeRobotResponse:
        """Describes a robot.

        :param self:
        :param robot:  The Amazon Resource Name (ARN) of the robot to be described.<br/>
        :return: tags:  The list of all tags added to the specified robot.<br/>lastDeploymentTime:  The time of the last deployment job.<br/>lastDeploymentJob:  The Amazon Resource Name (ARN) of the last deployment job.<br/>architecture:  The target architecture of the robot application.<br/>createdAt:  The time, in milliseconds since the epoch, when the robot was created.<br/>greengrassGroupId:  The Greengrass group id.<br/>status:  The status of the fleet.<br/>fleetArn:  The Amazon Resource Name (ARN) of the fleet.<br/>name:  The name of the robot.<br/>arn:  The Amazon Resource Name (ARN) of the robot.<br/>        """
        pass

    def describe_robot_application(self, application: str, applicationVersion: str=None) -> DescribeRobotApplicationResponse:
        """Describes a robot application.

        :param self:
        :param applicationVersion:  The version of the robot application to describe.<br/>application:  The Amazon Resource Name (ARN) of the robot application.<br/>
        :return: tags:  The list of all tags added to the specified robot application.<br/>lastUpdatedAt:  The time, in milliseconds since the epoch, when the robot application was last updated.<br/>revisionId:  The revision id of the robot application.<br/>robotSoftwareSuite:  The robot software suite used by the robot application.:<br/>sources:  The sources of the robot application.<br/>version:  The version of the robot application.<br/>name:  The name of the robot application.<br/>arn:  The Amazon Resource Name (ARN) of the robot application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the robot software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the robot software suite.<br/>        """
        pass

    def describe_simulation_application(self, application: str, applicationVersion: str=None) -> DescribeSimulationApplicationResponse:
        """Describes a simulation application.

        :param self:
        :param applicationVersion:  The version of the simulation application to describe.<br/>application:  The application information for the simulation application.<br/>
        :return: tags:  The list of all tags added to the specified simulation application.<br/>lastUpdatedAt:  The time, in milliseconds since the epoch, when the simulation application was last updated.<br/>revisionId:  The revision id of the simulation application.<br/>renderingEngine:  The rendering engine for the simulation application.:<br/>robotSoftwareSuite:  Information about the robot software suite.:<br/>simulationSoftwareSuite:  The simulation software suite used by the simulation application.:<br/>sources:  The sources of the simulation application.<br/>version:  The version of the simulation application.<br/>name:  The name of the simulation application.<br/>arn:  The Amazon Resource Name (ARN) of the robot simulation application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the simulation software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the simulation software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the robot software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the robot software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the rendering engine.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the rendering engine.<br/>        """
        pass

    def describe_simulation_job(self, job: str) -> DescribeSimulationJobResponse:
        """Describes a simulation job.

        :param self:
        :param job:  The Amazon Resource Name (ARN) of the simulation job to be described.<br/>
        :return: vpcConfig:  The VPC configuration.:<br/>tags:  The list of all tags added to the specified simulation job.<br/>simulationApplications:  A list of simulation applications.<br/>robotApplications:  A list of robot applications.<br/>iamRole:  The IAM role that allows the simulation instance to call the AWS APIs that are specified in its associated policies on your behalf.<br/>simulationTimeMillis:  The simulation job execution duration in milliseconds.<br/>maxJobDurationInSeconds:  The maximum job duration in seconds. The value must be 8 days (691,200 seconds) or less.<br/>outputLocation:  Location for output files generated by the simulation job.:<br/>clientRequestToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request.<br/>failureReason:  Details about why the simulation job failed. For more information about troubleshooting, see <a href="https://docs.aws.amazon.com/robomaker/latest/dg/troubleshooting.html">Troubleshooting</a>.<br/>failureCode:  The failure code of the simulation job if it failed: <dl> <dt>InternalServiceError</dt> <dd> Internal service error. </dd> <dt>RobotApplicationCrash</dt> <dd> Robot application exited abnormally. </dd> <dt>SimulationApplicationCrash</dt> <dd>  Simulation application exited abnormally. </dd> <dt>BadPermissionsRobotApplication</dt> <dd> Robot application bundle could not be downloaded. </dd> <dt>BadPermissionsSimulationApplication</dt> <dd> Simulation application bundle could not be downloaded. </dd> <dt>BadPermissionsS3Output</dt> <dd> Unable to publish outputs to customer-provided S3 bucket. </dd> <dt>BadPermissionsCloudwatchLogs</dt> <dd> Unable to publish logs to customer-provided CloudWatch Logs resource. </dd> <dt>SubnetIpLimitExceeded</dt> <dd> Subnet IP limit exceeded. </dd> <dt>ENILimitExceeded</dt> <dd> ENI limit exceeded. </dd> <dt>BadPermissionsUserCredentials</dt> <dd> Unable to use the Role provided. </dd> <dt>InvalidBundleRobotApplication</dt> <dd> Robot bundle cannot be extracted (invalid format, bundling error, or other issue). </dd> <dt>InvalidBundleSimulationApplication</dt> <dd> Simulation bundle cannot be extracted (invalid format, bundling error, or other issue). </dd> <dt>RobotApplicationVersionMismatchedEtag</dt> <dd> Etag for RobotApplication does not match value during version creation. </dd> <dt>SimulationApplicationVersionMismatchedEtag</dt> <dd> Etag for SimulationApplication does not match value during version creation. </dd> </dl><br/>failureBehavior:  The failure behavior for the simulation job.<br/>lastUpdatedAt:  The time, in milliseconds since the epoch, when the simulation job was last updated.<br/>lastStartedAt:  The time, in milliseconds since the epoch, when the simulation job was last started.<br/>status:  The status of the simulation job.<br/>name:  The name of the simulation job.<br/>arn:  The Amazon Resource Name (ARN) of the simulation job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>s3Prefix</i> The S3 folder in the <code>s3Bucket</code> where output files will be placed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>s3Bucket</i> The S3 bucket for output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>assignPublicIp</i> A boolean indicating if a public IP was assigned.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vpcId</i> The VPC ID associated with your simulation job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>securityGroups</i> A list of security group IDs associated with the simulation job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>subnets</i> A list of subnet IDs associated with the simulation job.<br/>        """
        pass

    def list_deployment_jobs(self, filters: List[Filter]=None, nextToken: str=None, maxResults: int=None) -> ListDeploymentJobsResponse:
        """Returns a list of deployment jobs for a fleet. You can optionally provide filters to retrieve specific deployment jobs.  <note>   </note>

        :param self:
        :param maxResults:  The maximum number of deployment job results returned by <code>ListDeploymentJobs</code> in paginated output. When this parameter is used, <code>ListDeploymentJobs</code> only returns <code>maxResults</code> results in a single page along with a <code>nextToken</code> response element. The remaining results of the initial request can be seen by sending another <code>ListDeploymentJobs</code> request with the returned <code>nextToken</code> value. This value can be between 1 and 100. If this parameter is not used, then <code>ListDeploymentJobs</code> returns up to 100 results and a <code>nextToken</code> value if applicable. <br/>nextToken:  The <code>nextToken</code> value returned from a previous paginated <code>ListDeploymentJobs</code> request where <code>maxResults</code> was used and the results exceeded the value of that parameter. Pagination continues from the end of the previous results that returned the <code>nextToken</code> value.  <note> This token should be treated as an opaque identifier that is only used to retrieve the next items in a list and not for other programmatic purposes. </note><br/>filters:  Optional filters to limit results. The filter names <code>status</code> and <code>fleetName</code> are supported. When filtering, you must use the complete value of the filtered item. You can use up to three filters, but they must be for the same named item. For example, if you are looking for items with the status <code>InProgress</code> or the status <code>Pending</code>.<br/>
        :return: nextToken:  The <code>nextToken</code> value to include in a future <code>ListDeploymentJobs</code> request. When the results of a <code>ListDeploymentJobs</code> request exceed <code>maxResults</code>, this value can be used to retrieve the next page of results. This value is <code>null</code> when there are no more results to return. <br/>deploymentJobs:  A list of deployment jobs that meet the criteria of the request.<br/>        """
        pass

    def list_fleets(self, nextToken: str=None, maxResults: int=None, filters: List[Filter]=None) -> ListFleetsResponse:
        """Returns a list of fleets. You can optionally provide filters to retrieve specific fleets. 

        :param self:
        :param filters:  Optional filters to limit results. The filter name <code>name</code> is supported. When filtering, you must use the complete value of the filtered item. You can use up to three filters.<br/>maxResults:  The maximum number of deployment job results returned by <code>ListFleets</code> in paginated output. When this parameter is used, <code>ListFleets</code> only returns <code>maxResults</code> results in a single page along with a <code>nextToken</code> response element. The remaining results of the initial request can be seen by sending another <code>ListFleets</code> request with the returned <code>nextToken</code> value. This value can be between 1 and 100. If this parameter is not used, then <code>ListFleets</code> returns up to 100 results and a <code>nextToken</code> value if applicable. <br/>nextToken:  The <code>nextToken</code> value returned from a previous paginated <code>ListFleets</code> request where <code>maxResults</code> was used and the results exceeded the value of that parameter. Pagination continues from the end of the previous results that returned the <code>nextToken</code> value.  <note> This token should be treated as an opaque identifier that is only used to retrieve the next items in a list and not for other programmatic purposes. </note><br/>
        :return: nextToken:  The <code>nextToken</code> value to include in a future <code>ListDeploymentJobs</code> request. When the results of a <code>ListFleets</code> request exceed <code>maxResults</code>, this value can be used to retrieve the next page of results. This value is <code>null</code> when there are no more results to return. <br/>fleetDetails:  A list of fleet details meeting the request criteria.<br/>        """
        pass

    def list_robot_applications(self, versionQualifier: str=None, nextToken: str=None, maxResults: int=None, filters: List[Filter]=None) -> ListRobotApplicationsResponse:
        """Returns a list of robot application. You can optionally provide filters to retrieve specific robot applications.

        :param self:
        :param filters:  Optional filters to limit results. The filter name <code>name</code> is supported. When filtering, you must use the complete value of the filtered item. You can use up to three filters.<br/>maxResults:  The maximum number of deployment job results returned by <code>ListRobotApplications</code> in paginated output. When this parameter is used, <code>ListRobotApplications</code> only returns <code>maxResults</code> results in a single page along with a <code>nextToken</code> response element. The remaining results of the initial request can be seen by sending another <code>ListRobotApplications</code> request with the returned <code>nextToken</code> value. This value can be between 1 and 100. If this parameter is not used, then <code>ListRobotApplications</code> returns up to 100 results and a <code>nextToken</code> value if applicable. <br/>nextToken:  The <code>nextToken</code> value returned from a previous paginated <code>ListRobotApplications</code> request where <code>maxResults</code> was used and the results exceeded the value of that parameter. Pagination continues from the end of the previous results that returned the <code>nextToken</code> value.  <note> This token should be treated as an opaque identifier that is only used to retrieve the next items in a list and not for other programmatic purposes. </note><br/>versionQualifier:  The version qualifier of the robot application.<br/>
        :return: nextToken:  The <code>nextToken</code> value to include in a future <code>ListRobotApplications</code> request. When the results of a <code>ListRobotApplications</code> request exceed <code>maxResults</code>, this value can be used to retrieve the next page of results. This value is <code>null</code> when there are no more results to return. <br/>robotApplicationSummaries:  A list of robot application summaries that meet the criteria of the request.<br/>        """
        pass

    def list_robots(self, nextToken: str=None, maxResults: int=None, filters: List[Filter]=None) -> ListRobotsResponse:
        """Returns a list of robots. You can optionally provide filters to retrieve specific robots.

        :param self:
        :param filters:  Optional filters to limit results. The filter names <code>status</code> and <code>fleetName</code> are supported. When filtering, you must use the complete value of the filtered item. You can use up to three filters, but they must be for the same named item. For example, if you are looking for items with the status <code>Registered</code> or the status <code>Available</code>.<br/>maxResults:  The maximum number of deployment job results returned by <code>ListRobots</code> in paginated output. When this parameter is used, <code>ListRobots</code> only returns <code>maxResults</code> results in a single page along with a <code>nextToken</code> response element. The remaining results of the initial request can be seen by sending another <code>ListRobots</code> request with the returned <code>nextToken</code> value. This value can be between 1 and 100. If this parameter is not used, then <code>ListRobots</code> returns up to 100 results and a <code>nextToken</code> value if applicable. <br/>nextToken:  The <code>nextToken</code> value returned from a previous paginated <code>ListRobots</code> request where <code>maxResults</code> was used and the results exceeded the value of that parameter. Pagination continues from the end of the previous results that returned the <code>nextToken</code> value.  <note> This token should be treated as an opaque identifier that is only used to retrieve the next items in a list and not for other programmatic purposes. </note><br/>
        :return: nextToken:  The <code>nextToken</code> value to include in a future <code>ListRobots</code> request. When the results of a <code>ListRobot</code> request exceed <code>maxResults</code>, this value can be used to retrieve the next page of results. This value is <code>null</code> when there are no more results to return. <br/>robots:  A list of robots that meet the criteria of the request.<br/>        """
        pass

    def list_simulation_applications(self, versionQualifier: str=None, nextToken: str=None, maxResults: int=None, filters: List[Filter]=None) -> ListSimulationApplicationsResponse:
        """Returns a list of simulation applications. You can optionally provide filters to retrieve specific simulation applications. 

        :param self:
        :param filters:  Optional list of filters to limit results. The filter name <code>name</code> is supported. When filtering, you must use the complete value of the filtered item. You can use up to three filters.<br/>maxResults:  The maximum number of deployment job results returned by <code>ListSimulationApplications</code> in paginated output. When this parameter is used, <code>ListSimulationApplications</code> only returns <code>maxResults</code> results in a single page along with a <code>nextToken</code> response element. The remaining results of the initial request can be seen by sending another <code>ListSimulationApplications</code> request with the returned <code>nextToken</code> value. This value can be between 1 and 100. If this parameter is not used, then <code>ListSimulationApplications</code> returns up to 100 results and a <code>nextToken</code> value if applicable. <br/>nextToken:  The <code>nextToken</code> value returned from a previous paginated <code>ListSimulationApplications</code> request where <code>maxResults</code> was used and the results exceeded the value of that parameter. Pagination continues from the end of the previous results that returned the <code>nextToken</code> value.  <note> This token should be treated as an opaque identifier that is only used to retrieve the next items in a list and not for other programmatic purposes. </note><br/>versionQualifier:  The version qualifier of the simulation application.<br/>
        :return: nextToken:  The <code>nextToken</code> value to include in a future <code>ListSimulationApplications</code> request. When the results of a <code>ListRobot</code> request exceed <code>maxResults</code>, this value can be used to retrieve the next page of results. This value is <code>null</code> when there are no more results to return. <br/>simulationApplicationSummaries:  A list of simulation application summaries that meet the criteria of the request.<br/>        """
        pass

    def list_simulation_jobs(self, nextToken: str=None, maxResults: int=None, filters: List[Filter]=None) -> ListSimulationJobsResponse:
        """Returns a list of simulation jobs. You can optionally provide filters to retrieve specific simulation jobs. 

        :param self:
        :param filters:  Optional filters to limit results. The filter names <code>status</code> and <code>simulationApplicationName</code> and <code>robotApplicationName</code> are supported. When filtering, you must use the complete value of the filtered item. You can use up to three filters, but they must be for the same named item. For example, if you are looking for items with the status <code>Preparing</code> or the status <code>Running</code>.<br/>maxResults:  The maximum number of deployment job results returned by <code>ListSimulationJobs</code> in paginated output. When this parameter is used, <code>ListSimulationJobs</code> only returns <code>maxResults</code> results in a single page along with a <code>nextToken</code> response element. The remaining results of the initial request can be seen by sending another <code>ListSimulationJobs</code> request with the returned <code>nextToken</code> value. This value can be between 1 and 100. If this parameter is not used, then <code>ListSimulationJobs</code> returns up to 100 results and a <code>nextToken</code> value if applicable. <br/>nextToken:  The <code>nextToken</code> value returned from a previous paginated <code>ListSimulationJobs</code> request where <code>maxResults</code> was used and the results exceeded the value of that parameter. Pagination continues from the end of the previous results that returned the <code>nextToken</code> value.  <note> This token should be treated as an opaque identifier that is only used to retrieve the next items in a list and not for other programmatic purposes. </note><br/>
        :return: nextToken:  The <code>nextToken</code> value to include in a future <code>ListSimulationJobs</code> request. When the results of a <code>ListRobot</code> request exceed <code>maxResults</code>, this value can be used to retrieve the next page of results. This value is <code>null</code> when there are no more results to return. <br/>simulationJobSummaries:  A list of simulation job summaries that meet the criteria of the request.<br/>        """
        pass

    def list_tags_for_resource(self, resourceArn: str) -> ListTagsForResourceResponse:
        """Lists all tags on a AWS RoboMaker resource.

        :param self:
        :param resourceArn:  The AWS RoboMaker Amazon Resource Name (ARN) with tags to be listed.<br/>
        :return: tags:  The list of all tags added to the specified resource.<br/>        """
        pass

    def register_robot(self, robot: str, fleet: str) -> RegisterRobotResponse:
        """Registers a robot with a fleet.

        :param self:
        :param robot:  The Amazon Resource Name (ARN) of the robot.<br/>fleet:  The Amazon Resource Name (ARN) of the fleet.<br/>
        :return: robot:  Information about the robot registration.<br/>fleet:  The Amazon Resource Name (ARN) of the fleet that the robot will join.<br/>        """
        pass

    def restart_simulation_job(self, job: str) -> RestartSimulationJobResponse:
        """Restarts a running simulation job.

        :param self:
        :param job:  The Amazon Resource Name (ARN) of the simulation job.<br/>
        :return:         """
        pass

    def sync_deployment_job(self, fleet: str, clientRequestToken: str) -> SyncDeploymentJobResponse:
        """Syncrhonizes robots in a fleet to the latest deployment. This is helpful if robots were added after a deployment.

        :param self:
        :param fleet:  The target fleet for the synchronization.<br/>clientRequestToken:  Unique, case-sensitive identifier that you provide to ensure the idempotency of the request.<br/>
        :return: createdAt:  The time, in milliseconds since the epoch, when the fleet was created.<br/>failureCode:  The failure code if the job fails: <dl> <dt>InternalServiceError</dt> <dd> Internal service error. </dd> <dt>RobotApplicationCrash</dt> <dd> Robot application exited abnormally. </dd> <dt>SimulationApplicationCrash</dt> <dd>  Simulation application exited abnormally. </dd> <dt>BadPermissionsRobotApplication</dt> <dd> Robot application bundle could not be downloaded. </dd> <dt>BadPermissionsSimulationApplication</dt> <dd> Simulation application bundle could not be downloaded. </dd> <dt>BadPermissionsS3Output</dt> <dd> Unable to publish outputs to customer-provided S3 bucket. </dd> <dt>BadPermissionsCloudwatchLogs</dt> <dd> Unable to publish logs to customer-provided CloudWatch Logs resource. </dd> <dt>SubnetIpLimitExceeded</dt> <dd> Subnet IP limit exceeded. </dd> <dt>ENILimitExceeded</dt> <dd> ENI limit exceeded. </dd> <dt>BadPermissionsUserCredentials</dt> <dd> Unable to use the Role provided. </dd> <dt>InvalidBundleRobotApplication</dt> <dd> Robot bundle cannot be extracted (invalid format, bundling error, or other issue). </dd> <dt>InvalidBundleSimulationApplication</dt> <dd> Simulation bundle cannot be extracted (invalid format, bundling error, or other issue). </dd> <dt>RobotApplicationVersionMismatchedEtag</dt> <dd> Etag for RobotApplication does not match value during version creation. </dd> <dt>SimulationApplicationVersionMismatchedEtag</dt> <dd> Etag for SimulationApplication does not match value during version creation. </dd> </dl><br/>failureReason:  The failure reason if the job fails.<br/>deploymentApplicationConfigs:  Information about the deployment application configurations.<br/>deploymentConfig:  Information about the deployment configuration.:<br/>status:  The status of the synchronization job.<br/>fleet:  The Amazon Resource Name (ARN) of the fleet.<br/>arn:  The Amazon Resource Name (ARN) of the synchronization request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>failureThresholdPercentage</i> The percentage of deployments that need to fail before stopping deployment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>concurrentDeploymentPercentage</i> The percentage of robots receiving the deployment at the same time.<br/>        """
        pass

    def tag_resource(self, tags: TagMap, resourceArn: str) -> TagResourceResponse:
        """Adds or edits tags for a AWS RoboMaker resource. Each tag consists of a tag key and a tag value. Tag keys and tag values are both required, but tag values can be empty strings.  For information about the rules that apply to tag keys and tag values, see <a href="https://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/allocation-tag-restrictions.html">User-Defined Tag Restrictions</a> in the <i>AWS Billing and Cost Management User Guide</i>. 

        :param self:
        :param tags:  A map that contains tag keys and tag values that are attached to the resource.<br/>resourceArn:  The Amazon Resource Name (ARN) of the AWS RoboMaker resource you are tagging.<br/>
        :return:         """
        pass

    def untag_resource(self, tagKeys: List[TagKey], resourceArn: str) -> UntagResourceResponse:
        """Removes the specified tags from the specified AWS RoboMaker resource. To remove a tag, specify the tag key. To change the tag value of an existing tag key, use <a href="https://docs.aws.amazon.com/robomaker/latest/dg/API_TagResource.html"> <code>TagResource</code> </a>. 

        :param self:
        :param tagKeys:  A map that contains tag keys and tag values that will be unattached from the resource.<br/>resourceArn:  The Amazon Resource Name (ARN) of the AWS RoboMaker resource you are removing tags.<br/>
        :return:         """
        pass

    def update_robot_application(self, robotSoftwareSuite: RobotSoftwareSuite, sources: List[SourceConfig], application: str, currentRevisionId: str=None) -> UpdateRobotApplicationResponse:
        """Updates a robot application.

        :param self:
        :param currentRevisionId:  The revision id for the robot application.<br/>robotSoftwareSuite:  The robot software suite used by the robot application.:<br/>sources:  The sources of the robot application.<br/>application:  The application information for the robot application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the robot software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the robot software suite.<br/>
        :return: revisionId:  The revision id of the robot application.<br/>lastUpdatedAt:  The time, in milliseconds since the epoch, when the robot application was last updated.<br/>robotSoftwareSuite:  The robot software suite used by the robot application.:<br/>sources:  The sources of the robot application.<br/>version:  The version of the robot application.<br/>name:  The name of the robot application.<br/>arn:  The Amazon Resource Name (ARN) of the updated robot application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the robot software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the robot software suite.<br/>        """
        pass

    def update_simulation_application(self, renderingEngine: RenderingEngine, robotSoftwareSuite: RobotSoftwareSuite, simulationSoftwareSuite: SimulationSoftwareSuite, sources: List[SourceConfig], application: str, currentRevisionId: str=None) -> UpdateSimulationApplicationResponse:
        """Updates a simulation application.

        :param self:
        :param currentRevisionId:  The revision id for the robot application.<br/>renderingEngine:  The rendering engine for the simulation application.:<br/>robotSoftwareSuite:  Information about the robot software suite.:<br/>simulationSoftwareSuite:  The simulation software suite used by the simulation application.:<br/>sources:  The sources of the simulation application.<br/>application:  The application information for the simulation application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the simulation software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the simulation software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the robot software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the robot software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the rendering engine.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the rendering engine.<br/>
        :return: revisionId:  The revision id of the simulation application.<br/>lastUpdatedAt:  The time, in milliseconds since the epoch, when the simulation application was last updated.<br/>renderingEngine:  The rendering engine for the simulation application.:<br/>robotSoftwareSuite:  Information about the robot software suite.:<br/>simulationSoftwareSuite:  The simulation software suite used by the simulation application.:<br/>sources:  The sources of the simulation application.<br/>version:  The version of the robot application.<br/>name:  The name of the simulation application.<br/>arn:  The Amazon Resource Name (ARN) of the updated simulation application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the simulation software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the simulation software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the robot software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the robot software suite.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of the rendering engine.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the rendering engine.<br/>        """
        pass



class Route53(BaseClient):
    """Amazon Route 53 is a highly available and scalable Domain Name System (DNS) web service."""

    class ResourceRecordSetsChangedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `get_change` for valid parameters"""
            pass


    class ListHealthChecksPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_health_checks` for valid parameters"""
            pass
    class ListHostedZonesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_hosted_zones` for valid parameters"""
            pass
    class ListResourceRecordSetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_resource_record_sets` for valid parameters"""
            pass
    class ListVPCAssociationAuthorizationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_vpc_association_authorizations` for valid parameters"""
            pass
    class ListQueryLoggingConfigsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_query_logging_configs` for valid parameters"""
            pass


    class AssociateVPCWithHostedZoneRequest(Mapping):
        pass
    
    class AssociateVPCWithHostedZoneResponse(Mapping):
        pass
    
    class VPC(Mapping):
        pass
    
    class ChangeResourceRecordSetsRequest(Mapping):
        pass
    
    class ChangeResourceRecordSetsResponse(Mapping):
        pass
    
    class ChangeBatch(Mapping):
        pass
    
    class ChangeTagsForResourceRequest(Mapping):
        pass
    
    class ChangeTagsForResourceResponse(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class CreateHealthCheckRequest(Mapping):
        pass
    
    class CreateHealthCheckResponse(Mapping):
        pass
    
    class HealthCheckConfig(Mapping):
        pass
    
    class CreateHostedZoneRequest(Mapping):
        pass
    
    class CreateHostedZoneResponse(Mapping):
        pass
    
    class HostedZoneConfig(Mapping):
        pass
    
    class CreateQueryLoggingConfigRequest(Mapping):
        pass
    
    class CreateQueryLoggingConfigResponse(Mapping):
        pass
    
    class CreateReusableDelegationSetRequest(Mapping):
        pass
    
    class CreateReusableDelegationSetResponse(Mapping):
        pass
    
    class CreateTrafficPolicyRequest(Mapping):
        pass
    
    class CreateTrafficPolicyResponse(Mapping):
        pass
    
    class CreateTrafficPolicyInstanceRequest(Mapping):
        pass
    
    class CreateTrafficPolicyInstanceResponse(Mapping):
        pass
    
    class TTL(object):
        pass
    
    class CreateTrafficPolicyVersionRequest(Mapping):
        pass
    
    class CreateTrafficPolicyVersionResponse(Mapping):
        pass
    
    class CreateVPCAssociationAuthorizationRequest(Mapping):
        pass
    
    class CreateVPCAssociationAuthorizationResponse(Mapping):
        pass
    
    class DeleteHealthCheckRequest(Mapping):
        pass
    
    class DeleteHealthCheckResponse(Mapping):
        pass
    
    class DeleteHostedZoneRequest(Mapping):
        pass
    
    class DeleteHostedZoneResponse(Mapping):
        pass
    
    class DeleteQueryLoggingConfigRequest(Mapping):
        pass
    
    class DeleteQueryLoggingConfigResponse(Mapping):
        pass
    
    class DeleteReusableDelegationSetRequest(Mapping):
        pass
    
    class DeleteReusableDelegationSetResponse(Mapping):
        pass
    
    class DeleteTrafficPolicyRequest(Mapping):
        pass
    
    class DeleteTrafficPolicyResponse(Mapping):
        pass
    
    class DeleteTrafficPolicyInstanceRequest(Mapping):
        pass
    
    class DeleteTrafficPolicyInstanceResponse(Mapping):
        pass
    
    class DeleteVPCAssociationAuthorizationRequest(Mapping):
        pass
    
    class DeleteVPCAssociationAuthorizationResponse(Mapping):
        pass
    
    class DisassociateVPCFromHostedZoneRequest(Mapping):
        pass
    
    class DisassociateVPCFromHostedZoneResponse(Mapping):
        pass
    
    class GetAccountLimitRequest(Mapping):
        pass
    
    class GetAccountLimitResponse(Mapping):
        pass
    
    class GetChangeRequest(Mapping):
        pass
    
    class GetChangeResponse(Mapping):
        pass
    
    class GetCheckerIpRangesRequest(Mapping):
        pass
    
    class GetCheckerIpRangesResponse(Mapping):
        pass
    
    class GetGeoLocationRequest(Mapping):
        pass
    
    class GetGeoLocationResponse(Mapping):
        pass
    
    class GetHealthCheckRequest(Mapping):
        pass
    
    class GetHealthCheckResponse(Mapping):
        pass
    
    class GetHealthCheckCountRequest(Mapping):
        pass
    
    class GetHealthCheckCountResponse(Mapping):
        pass
    
    class GetHealthCheckLastFailureReasonRequest(Mapping):
        pass
    
    class GetHealthCheckLastFailureReasonResponse(Mapping):
        pass
    
    class GetHealthCheckStatusRequest(Mapping):
        pass
    
    class GetHealthCheckStatusResponse(Mapping):
        pass
    
    class GetHostedZoneRequest(Mapping):
        pass
    
    class GetHostedZoneResponse(Mapping):
        pass
    
    class GetHostedZoneCountRequest(Mapping):
        pass
    
    class GetHostedZoneCountResponse(Mapping):
        pass
    
    class GetHostedZoneLimitRequest(Mapping):
        pass
    
    class GetHostedZoneLimitResponse(Mapping):
        pass
    
    class GetQueryLoggingConfigRequest(Mapping):
        pass
    
    class GetQueryLoggingConfigResponse(Mapping):
        pass
    
    class GetReusableDelegationSetRequest(Mapping):
        pass
    
    class GetReusableDelegationSetResponse(Mapping):
        pass
    
    class GetReusableDelegationSetLimitRequest(Mapping):
        pass
    
    class GetReusableDelegationSetLimitResponse(Mapping):
        pass
    
    class GetTrafficPolicyRequest(Mapping):
        pass
    
    class GetTrafficPolicyResponse(Mapping):
        pass
    
    class GetTrafficPolicyInstanceRequest(Mapping):
        pass
    
    class GetTrafficPolicyInstanceResponse(Mapping):
        pass
    
    class GetTrafficPolicyInstanceCountRequest(Mapping):
        pass
    
    class GetTrafficPolicyInstanceCountResponse(Mapping):
        pass
    
    class ListGeoLocationsRequest(Mapping):
        pass
    
    class ListGeoLocationsResponse(Mapping):
        pass
    
    class ListHealthChecksRequest(Mapping):
        pass
    
    class ListHealthChecksResponse(Mapping):
        pass
    
    class ListHostedZonesRequest(Mapping):
        pass
    
    class ListHostedZonesResponse(Mapping):
        pass
    
    class ListHostedZonesByNameRequest(Mapping):
        pass
    
    class ListHostedZonesByNameResponse(Mapping):
        pass
    
    class ListQueryLoggingConfigsRequest(Mapping):
        pass
    
    class ListQueryLoggingConfigsResponse(Mapping):
        pass
    
    class ListResourceRecordSetsRequest(Mapping):
        pass
    
    class ListResourceRecordSetsResponse(Mapping):
        pass
    
    class ListReusableDelegationSetsRequest(Mapping):
        pass
    
    class ListReusableDelegationSetsResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class ListTagsForResourcesRequest(Mapping):
        pass
    
    class ListTagsForResourcesResponse(Mapping):
        pass
    
    class TagResourceId(object):
        pass
    
    class ListTrafficPoliciesRequest(Mapping):
        pass
    
    class ListTrafficPoliciesResponse(Mapping):
        pass
    
    class ListTrafficPolicyInstancesRequest(Mapping):
        pass
    
    class ListTrafficPolicyInstancesResponse(Mapping):
        pass
    
    class ListTrafficPolicyInstancesByHostedZoneRequest(Mapping):
        pass
    
    class ListTrafficPolicyInstancesByHostedZoneResponse(Mapping):
        pass
    
    class ListTrafficPolicyInstancesByPolicyRequest(Mapping):
        pass
    
    class ListTrafficPolicyInstancesByPolicyResponse(Mapping):
        pass
    
    class ListTrafficPolicyVersionsRequest(Mapping):
        pass
    
    class ListTrafficPolicyVersionsResponse(Mapping):
        pass
    
    class ListVPCAssociationAuthorizationsRequest(Mapping):
        pass
    
    class ListVPCAssociationAuthorizationsResponse(Mapping):
        pass
    
    class TestDNSAnswerRequest(Mapping):
        pass
    
    class TestDNSAnswerResponse(Mapping):
        pass
    
    class UpdateHealthCheckRequest(Mapping):
        pass
    
    class UpdateHealthCheckResponse(Mapping):
        pass
    
    class HealthCheckVersion(object):
        pass
    
    class Inverted(object):
        pass
    
    class Disabled(object):
        pass
    
    class HealthCheckId(object):
        pass
    
    class EnableSNI(object):
        pass
    
    class HealthCheckRegion(object):
        pass
    
    class AlarmIdentifier(Mapping):
        pass
    
    class ResettableElementName(object):
        pass
    
    class UpdateHostedZoneCommentRequest(Mapping):
        pass
    
    class UpdateHostedZoneCommentResponse(Mapping):
        pass
    
    class UpdateTrafficPolicyCommentRequest(Mapping):
        pass
    
    class UpdateTrafficPolicyCommentResponse(Mapping):
        pass
    
    class UpdateTrafficPolicyInstanceRequest(Mapping):
        pass
    
    class UpdateTrafficPolicyInstanceResponse(Mapping):
        pass
    
    def associate_vpc_with_hosted_zone(self, VPC: VPC, HostedZoneId: str, Comment: str=None) -> AssociateVPCWithHostedZoneResponse:
        """Associates an Amazon VPC with a private hosted zone.  <important> To perform the association, the VPC and the private hosted zone must already exist. You can't convert a public hosted zone into a private hosted zone. </important> <note> If you want to associate a VPC that was created by using one AWS account with a private hosted zone that was created by using a different account, the AWS account that created the private hosted zone must first submit a <code>CreateVPCAssociationAuthorization</code> request. Then the account that created the VPC must submit an <code>AssociateVPCWithHostedZone</code> request. </note>

        :param self:
        :param Comment:   <i>Optional:</i> A comment about the association request.<br/>VPC:  A complex type that contains information about the VPC that you want to associate with a private hosted zone.:<br/>HostedZoneId:  The ID of the private hosted zone that you want to associate an Amazon VPC with. Note that you can't associate a VPC with a hosted zone that doesn't have an existing VPC association.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VPCId</i> (Private hosted zones only) The ID of an Amazon VPC. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VPCRegion</i> (Private hosted zones only) The region that an Amazon VPC was created in.<br/>
        :return: ChangeInfo:  A complex type that describes the changes made to your hosted zone.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> A complex type that describes change information about changes made to your hosted zone. This element contains an ID that you use when performing a <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_GetChange.html">GetChange</a> action to get detailed information about the change.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmittedAt</i> The date and time that the change request was submitted in <a href="https://en.wikipedia.org/wiki/ISO_8601">ISO 8601 format</a> and Coordinated Universal Time (UTC). For example, the value <code>2017-03-27T17:48:16.751Z</code> represents March 27, 2017 at 17:48:16.751 UTC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current state of the request. <code>PENDING</code> indicates that this request has not yet been applied to all Amazon Route 53 DNS servers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the request.<br/>        """
        pass

    def change_resource_record_sets(self, ChangeBatch: ChangeBatch, HostedZoneId: str) -> ChangeResourceRecordSetsResponse:
        """Creates, changes, or deletes a resource record set, which contains authoritative DNS information for a specified domain name or subdomain name. For example, you can use <code>ChangeResourceRecordSets</code> to create a resource record set that routes traffic for test.example.com to a web server that has an IP address of 192.0.2.44.  <b>Change Batches and Transactional Changes</b>  The request body must include a document with a <code>ChangeResourceRecordSetsRequest</code> element. The request body contains a list of change items, known as a change batch. Change batches are considered transactional changes. When using the Amazon Route 53 API to change resource record sets, Route 53 either makes all or none of the changes in a change batch request. This ensures that Route 53 never partially implements the intended changes to the resource record sets in a hosted zone.  For example, a change batch request that deletes the <code>CNAME</code> record for www.example.com and creates an alias resource record set for www.example.com. Route 53 deletes the first resource record set and creates the second resource record set in a single operation. If either the <code>DELETE</code> or the <code>CREATE</code> action fails, then both changes (plus any other changes in the batch) fail, and the original <code>CNAME</code> record continues to exist. <important> Due to the nature of transactional changes, you can't delete the same resource record set more than once in a single change batch. If you attempt to delete the same change batch more than once, Route 53 returns an <code>InvalidChangeBatch</code> error. </important>  <b>Traffic Flow</b>  To create resource record sets for complex routing configurations, use either the traffic flow visual editor in the Route 53 console or the API actions for traffic policies and traffic policy instances. Save the configuration as a traffic policy, then associate the traffic policy with one or more domain names (such as example.com) or subdomain names (such as www.example.com), in the same hosted zone or in multiple hosted zones. You can roll back the updates if the new configuration isn't performing as expected. For more information, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/traffic-flow.html">Using Traffic Flow to Route DNS Traffic</a> in the <i>Amazon Route 53 Developer Guide</i>.  <b>Create, Delete, and Upsert</b>  Use <code>ChangeResourceRecordsSetsRequest</code> to perform the following actions: <ul> <li>  <code>CREATE</code>: Creates a resource record set that has the specified values. </li> <li>  <code>DELETE</code>: Deletes an existing resource record set that has the specified values. </li> <li>  <code>UPSERT</code>: If a resource record set does not already exist, AWS creates it. If a resource set does exist, Route 53 updates it with the values in the request.  </li> </ul>  <b>Syntaxes for Creating, Updating, and Deleting Resource Record Sets</b>  The syntax for a request depends on the type of resource record set that you want to create, delete, or update, such as weighted, alias, or failover. The XML elements in your request must appear in the order listed in the syntax.  For an example for each type of resource record set, see "Examples." Don't refer to the syntax in the "Parameter Syntax" section, which includes all of the elements for every kind of resource record set that you can create, delete, or update by using <code>ChangeResourceRecordSets</code>.   <b>Change Propagation to Route 53 DNS Servers</b>  When you submit a <code>ChangeResourceRecordSets</code> request, Route 53 propagates your changes to all of the Route 53 authoritative DNS servers. While your changes are propagating, <code>GetChange</code> returns a status of <code>PENDING</code>. When propagation is complete, <code>GetChange</code> returns a status of <code>INSYNC</code>. Changes generally propagate to all Route 53 name servers within 60 seconds. For more information, see <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_GetChange.html">GetChange</a>.  <b>Limits on ChangeResourceRecordSets Requests</b>  For information about the limits on a <code>ChangeResourceRecordSets</code> request, see <a href="https://docs.aws.amazon.com/Route53/latest/DeveloperGuide/DNSLimitations.html">Limits</a> in the <i>Amazon Route 53 Developer Guide</i>.

        :param self:
        :param ChangeBatch:  A complex type that contains an optional comment and the <code>Changes</code> element.:<br/>HostedZoneId:  The ID of the hosted zone that contains the resource record sets that you want to change.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Changes</i> Information about the changes to make to the record sets.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i>  <i>Optional:</i> Any comments you want to include about a change batch request.<br/>
        :return: ChangeInfo:  A complex type that contains information about changes made to your hosted zone. This element contains an ID that you use when performing a <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_GetChange.html">GetChange</a> action to get detailed information about the change.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> A complex type that describes change information about changes made to your hosted zone. This element contains an ID that you use when performing a <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_GetChange.html">GetChange</a> action to get detailed information about the change.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmittedAt</i> The date and time that the change request was submitted in <a href="https://en.wikipedia.org/wiki/ISO_8601">ISO 8601 format</a> and Coordinated Universal Time (UTC). For example, the value <code>2017-03-27T17:48:16.751Z</code> represents March 27, 2017 at 17:48:16.751 UTC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current state of the request. <code>PENDING</code> indicates that this request has not yet been applied to all Amazon Route 53 DNS servers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the request.<br/>        """
        pass

    def change_tags_for_resource(self, ResourceId: str, ResourceType: str, AddTags: List[Tag]=None, RemoveTagKeys: List[TagKey]=None) -> ChangeTagsForResourceResponse:
        """Adds, edits, or deletes tags for a health check or a hosted zone. For information about using tags for cost allocation, see <a href="https://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/cost-alloc-tags.html">Using Cost Allocation Tags</a> in the <i>AWS Billing and Cost Management User Guide</i>.

        :param self:
        :param RemoveTagKeys:  A complex type that contains a list of the tags that you want to delete from the specified health check or hosted zone. You can specify up to 10 keys.<br/>AddTags:  A complex type that contains a list of the tags that you want to add to the specified health check or hosted zone and/or the tags that you want to edit <code>Value</code> for. You can add a maximum of 10 tags to a health check or a hosted zone.<br/>ResourceId:  The ID of the resource for which you want to add, change, or delete tags.<br/>ResourceType:  The type of the resource. <ul> <li> The resource type for health checks is <code>healthcheck</code>. </li> <li> The resource type for hosted zones is <code>hostedzone</code>. </li> </ul><br/>
        :return:         """
        pass

    def create_health_check(self, HealthCheckConfig: HealthCheckConfig, CallerReference: str) -> CreateHealthCheckResponse:
        """Creates a new health check. For information about adding health checks to resource record sets, see <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_ResourceRecordSet.html#Route53-Type-ResourceRecordSet-HealthCheckId">HealthCheckId</a> in <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_ChangeResourceRecordSets.html">ChangeResourceRecordSets</a>.   <b>ELB Load Balancers</b>  If you're registering EC2 instances with an Elastic Load Balancing (ELB) load balancer, do not create Amazon Route 53 health checks for the EC2 instances. When you register an EC2 instance with a load balancer, you configure settings for an ELB health check, which performs a similar function to a Route 53 health check.  <b>Private Hosted Zones</b>  You can associate health checks with failover resource record sets in a private hosted zone. Note the following: <ul> <li> Route 53 health checkers are outside the VPC. To check the health of an endpoint within a VPC by IP address, you must assign a public IP address to the instance in the VPC. </li> <li> You can configure a health checker to check the health of an external resource that the instance relies on, such as a database server. </li> <li> You can create a CloudWatch metric, associate an alarm with the metric, and then create a health check that is based on the state of the alarm. For example, you might create a CloudWatch metric that checks the status of the Amazon EC2 <code>StatusCheckFailed</code> metric, add an alarm to the metric, and then create a health check that is based on the state of the alarm. For information about creating CloudWatch metrics and alarms by using the CloudWatch console, see the <a href="http://docs.aws.amazon.com/AmazonCloudWatch/latest/DeveloperGuide/WhatIsCloudWatch.html">Amazon CloudWatch User Guide</a>. </li> </ul>

        :param self:
        :param HealthCheckConfig:  A complex type that contains settings for a new health check.:<br/>CallerReference:  A unique string that identifies the request and that allows you to retry a failed <code>CreateHealthCheck</code> request without the risk of creating two identical health checks: <ul> <li> If you send a <code>CreateHealthCheck</code> request with the same <code>CallerReference</code> and settings as a previous request, and if the health check doesn't exist, Amazon Route 53 creates the health check. If the health check does exist, Route 53 returns the settings for the existing health check. </li> <li> If you send a <code>CreateHealthCheck</code> request with the same <code>CallerReference</code> as a deleted health check, regardless of the settings, Route 53 returns a <code>HealthCheckAlreadyExists</code> error. </li> <li> If you send a <code>CreateHealthCheck</code> request with the same <code>CallerReference</code> as an existing health check but with different settings, Route 53 returns a <code>HealthCheckAlreadyExists</code> error. </li> <li> If you send a <code>CreateHealthCheck</code> request with a unique <code>CallerReference</code> but settings identical to an existing health check, Route 53 creates the health check. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InsufficientDataHealthStatus</i> When CloudWatch has insufficient data about the metric to determine the alarm state, the status that you want Amazon Route 53 to assign to the health check: <ul> <li>  <code>Healthy</code>: Route 53 considers the health check to be healthy. </li> <li>  <code>Unhealthy</code>: Route 53 considers the health check to be unhealthy. </li> <li>  <code>LastKnownStatus</code>: Route 53 uses the status of the health check from the last time that CloudWatch had sufficient data to determine the alarm state. For new health checks that have no last known status, the default status for the health check is healthy. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AlarmIdentifier</i> A complex type that identifies the CloudWatch alarm that you want Amazon Route 53 health checkers to use to determine whether the specified health check is healthy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AlarmIdentifier</b>: A complex type that identifies the CloudWatch alarm that you want Amazon Route 53 health checkers to use to determine whether the specified health check is healthy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Regions</i> A complex type that contains one <code>Region</code> element for each region from which you want Amazon Route 53 health checkers to check the specified endpoint. If you don't specify any regions, Route 53 health checkers automatically performs checks from all of the regions that are listed under <b>Valid Values</b>. If you update a health check to remove a region that has been performing health checks, Route 53 will briefly continue to perform checks from that region to ensure that some health checkers are always checking the endpoint (for example, if you replace three regions with four different regions). <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnableSNI</i> Specify whether you want Amazon Route 53 to send the value of <code>FullyQualifiedDomainName</code> to the endpoint in the <code>client_hello</code> message during TLS negotiation. This allows the endpoint to respond to <code>HTTPS</code> health check requests with the applicable SSL/TLS certificate. Some endpoints require that <code>HTTPS</code> requests include the host name in the <code>client_hello</code> message. If you don't enable SNI, the status of the health check will be <code>SSL alert handshake_failure</code>. A health check can also have that status for other reasons. If SNI is enabled and you're still getting the error, check the SSL/TLS configuration on your endpoint and confirm that your certificate is valid. The SSL/TLS certificate on your endpoint includes a domain name in the <code>Common Name</code> field and possibly several more in the <code>Subject Alternative Names</code> field. One of the domain names in the certificate should match the value that you specify for <code>FullyQualifiedDomainName</code>. If the endpoint responds to the <code>client_hello</code> message with a certificate that does not include the domain name that you specified in <code>FullyQualifiedDomainName</code>, a health checker will retry the handshake. In the second attempt, the health checker will omit <code>FullyQualifiedDomainName</code> from the <code>client_hello</code> message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ChildHealthChecks</i> (CALCULATED Health Checks Only) A complex type that contains one <code>ChildHealthCheck</code> element for each health check that you want to associate with a <code>CALCULATED</code> health check.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HealthThreshold</i> The number of child health checks that are associated with a <code>CALCULATED</code> health check that Amazon Route 53 must consider healthy for the <code>CALCULATED</code> health check to be considered healthy. To specify the child health checks that you want to associate with a <code>CALCULATED</code> health check, use the <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_UpdateHealthCheck.html#Route53-UpdateHealthCheck-request-ChildHealthChecks">ChildHealthChecks</a> element. Note the following: <ul> <li> If you specify a number greater than the number of child health checks, Route 53 always considers this health check to be unhealthy. </li> <li> If you specify <code>0</code>, Route 53 always considers this health check to be healthy. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Disabled</i> Stops Route 53 from performing health checks. When you disable a health check, here's what happens: <ul> <li>  <b>Health checks that check the health of endpoints:</b> Route 53 stops submitting requests to your application, server, or other resource. </li> <li>  <b>Calculated health checks:</b> Route 53 stops aggregating the status of the referenced health checks. </li> <li>  <b>Health checks that monitor CloudWatch alarms:</b> Route 53 stops monitoring the corresponding CloudWatch metrics. </li> </ul> After you disable a health check, Route 53 considers the status of the health check to always be healthy. If you configured DNS failover, Route 53 continues to route traffic to the corresponding resources. If you want to stop routing traffic to a resource, change the value of <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_UpdateHealthCheck.html#Route53-UpdateHealthCheck-request-Inverted">Inverted</a>.  Charges for a health check still apply when the health check is disabled. For more information, see <a href="http://aws.amazon.com/route53/pricing/">Amazon Route 53 Pricing</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Inverted</i> Specify whether you want Amazon Route 53 to invert the status of a health check, for example, to consider a health check unhealthy when it otherwise would be considered healthy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MeasureLatency</i> Specify whether you want Amazon Route 53 to measure the latency between health checkers in multiple AWS regions and your endpoint, and to display CloudWatch latency graphs on the <b>Health Checks</b> page in the Route 53 console. <important> You can't change the value of <code>MeasureLatency</code> after you create a health check. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureThreshold</i> The number of consecutive health checks that an endpoint must pass or fail for Amazon Route 53 to change the current status of the endpoint from unhealthy to healthy or vice versa. For more information, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/dns-failover-determining-health-of-endpoints.html">How Amazon Route 53 Determines Whether an Endpoint Is Healthy</a> in the <i>Amazon Route 53 Developer Guide</i>. If you don't specify a value for <code>FailureThreshold</code>, the default value is three health checks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestInterval</i> The number of seconds between the time that Amazon Route 53 gets a response from your endpoint and the time that it sends the next health check request. Each Route 53 health checker makes requests at this interval. <important> You can't change the value of <code>RequestInterval</code> after you create a health check. </important> If you don't specify a value for <code>RequestInterval</code>, the default value is <code>30</code> seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SearchString</i> If the value of Type is <code>HTTP_STR_MATCH</code> or <code>HTTP_STR_MATCH</code>, the string that you want Amazon Route 53 to search for in the response body from the specified resource. If the string appears in the response body, Route 53 considers the resource healthy. Route 53 considers case when searching for <code>SearchString</code> in the response body. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FullyQualifiedDomainName</i> Amazon Route 53 behavior depends on whether you specify a value for <code>IPAddress</code>.  <b>If you specify a value for</b> <code>IPAddress</code>: Amazon Route 53 sends health check requests to the specified IPv4 or IPv6 address and passes the value of <code>FullyQualifiedDomainName</code> in the <code>Host</code> header for all health checks except TCP health checks. This is typically the fully qualified DNS name of the endpoint on which you want Route 53 to perform health checks. When Route 53 checks the health of an endpoint, here is how it constructs the <code>Host</code> header: <ul> <li> If you specify a value of <code>80</code> for <code>Port</code> and <code>HTTP</code> or <code>HTTP_STR_MATCH</code> for <code>Type</code>, Route 53 passes the value of <code>FullyQualifiedDomainName</code> to the endpoint in the Host header.  </li> <li> If you specify a value of <code>443</code> for <code>Port</code> and <code>HTTPS</code> or <code>HTTPS_STR_MATCH</code> for <code>Type</code>, Route 53 passes the value of <code>FullyQualifiedDomainName</code> to the endpoint in the <code>Host</code> header. </li> <li> If you specify another value for <code>Port</code> and any value except <code>TCP</code> for <code>Type</code>, Route 53 passes <code>FullyQualifiedDomainName:Port</code> to the endpoint in the <code>Host</code> header. </li> </ul> If you don't specify a value for <code>FullyQualifiedDomainName</code>, Route 53 substitutes the value of <code>IPAddress</code> in the <code>Host</code> header in each of the preceding cases.  <b>If you don't specify a value for <code>IPAddress</code> </b>: Route 53 sends a DNS request to the domain that you specify for <code>FullyQualifiedDomainName</code> at the interval that you specify for <code>RequestInterval</code>. Using an IPv4 address that DNS returns, Route 53 then checks the health of the endpoint. <note> If you don't specify a value for <code>IPAddress</code>, Route 53 uses only IPv4 to send health checks to the endpoint. If there's no resource record set with a type of A for the name that you specify for <code>FullyQualifiedDomainName</code>, the health check fails with a "DNS resolution failed" error. </note> If you want to check the health of weighted, latency, or failover resource record sets and you choose to specify the endpoint only by <code>FullyQualifiedDomainName</code>, we recommend that you create a separate health check for each endpoint. For example, create a health check for each HTTP server that is serving content for www.example.com. For the value of <code>FullyQualifiedDomainName</code>, specify the domain name of the server (such as us-east-2-www.example.com), not the name of the resource record sets (www.example.com). <important> In this configuration, if you create a health check for which the value of <code>FullyQualifiedDomainName</code> matches the name of the resource record sets and you then associate the health check with those resource record sets, health check results will be unpredictable. </important> In addition, if the value that you specify for <code>Type</code> is <code>HTTP</code>, <code>HTTPS</code>, <code>HTTP_STR_MATCH</code>, or <code>HTTPS_STR_MATCH</code>, Route 53 passes the value of <code>FullyQualifiedDomainName</code> in the <code>Host</code> header, as it does when you specify a value for <code>IPAddress</code>. If the value of <code>Type</code> is <code>TCP</code>, Route 53 doesn't pass a <code>Host</code> header.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourcePath</i> The path, if any, that you want Amazon Route 53 to request when performing health checks. The path can be any value for which your endpoint will return an HTTP status code of 2xx or 3xx when the endpoint is healthy, for example, the file /docs/route53-health-check.html. You can also include query string parameters, for example, <code>/welcome.html?language=jp&amp;login=y</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of health check that you want to create, which indicates how Amazon Route 53 determines whether an endpoint is healthy. <important> You can't change the value of <code>Type</code> after you create a health check. </important> You can create the following types of health checks: <ul> <li>  <b>HTTP</b>: Route 53 tries to establish a TCP connection. If successful, Route 53 submits an HTTP request and waits for an HTTP status code of 200 or greater and less than 400. </li> <li>  <b>HTTPS</b>: Route 53 tries to establish a TCP connection. If successful, Route 53 submits an HTTPS request and waits for an HTTP status code of 200 or greater and less than 400. <important> If you specify <code>HTTPS</code> for the value of <code>Type</code>, the endpoint must support TLS v1.0 or later. </important> </li> <li>  <b>HTTP_STR_MATCH</b>: Route 53 tries to establish a TCP connection. If successful, Route 53 submits an HTTP request and searches the first 5,120 bytes of the response body for the string that you specify in <code>SearchString</code>. </li> <li>  <b>HTTPS_STR_MATCH</b>: Route 53 tries to establish a TCP connection. If successful, Route 53 submits an <code>HTTPS</code> request and searches the first 5,120 bytes of the response body for the string that you specify in <code>SearchString</code>. </li> <li>  <b>TCP</b>: Route 53 tries to establish a TCP connection. </li> <li>  <b>CLOUDWATCH_METRIC</b>: The health check is associated with a CloudWatch alarm. If the state of the alarm is <code>OK</code>, the health check is considered healthy. If the state is <code>ALARM</code>, the health check is considered unhealthy. If CloudWatch doesn't have sufficient data to determine whether the state is <code>OK</code> or <code>ALARM</code>, the health check status depends on the setting for <code>InsufficientDataHealthStatus</code>: <code>Healthy</code>, <code>Unhealthy</code>, or <code>LastKnownStatus</code>.  </li> <li>  <b>CALCULATED</b>: For health checks that monitor the status of other health checks, Route 53 adds up the number of health checks that Route 53 health checkers consider to be healthy and compares that number with the value of <code>HealthThreshold</code>.  </li> </ul> For more information, see <a href="https://docs.aws.amazon.com/Route53/latest/DeveloperGuide/dns-failover-determining-health-of-endpoints.html">How Route 53 Determines Whether an Endpoint Is Healthy</a> in the <i>Amazon Route 53 Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port on the endpoint on which you want Amazon Route 53 to perform health checks. Specify a value for <code>Port</code> only when you specify a value for <code>IPAddress</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IPAddress</i> The IPv4 or IPv6 IP address of the endpoint that you want Amazon Route 53 to perform health checks on. If you don't specify a value for <code>IPAddress</code>, Route 53 sends a DNS request to resolve the domain name that you specify in <code>FullyQualifiedDomainName</code> at the interval that you specify in <code>RequestInterval</code>. Using an IP address returned by DNS, Route 53 then checks the health of the endpoint. Use one of the following formats for the value of <code>IPAddress</code>:  <ul> <li>  <b>IPv4 address</b>: four values between 0 and 255, separated by periods (.), for example, <code>192.0.2.44</code>. </li> <li>  <b>IPv6 address</b>: eight groups of four hexadecimal values, separated by colons (:), for example, <code>2001:0db8:85a3:0000:0000:abcd:0001:2345</code>. You can also shorten IPv6 addresses as described in RFC 5952, for example, <code>2001:db8:85a3::abcd:1:2345</code>. </li> </ul> If the endpoint is an EC2 instance, we recommend that you create an Elastic IP address, associate it with your EC2 instance, and specify the Elastic IP address for <code>IPAddress</code>. This ensures that the IP address of your instance will never change. For more information, see <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_UpdateHealthCheck.html#Route53-UpdateHealthCheck-request-FullyQualifiedDomainName">FullyQualifiedDomainName</a>.  Constraints: Route 53 can't check the health of endpoints for which the IP address is in local, private, non-routable, or multicast ranges. For more information about IP addresses for which you can't create health checks, see the following documents: <ul> <li>  <a href="https://tools.ietf.org/html/rfc5735">RFC 5735, Special Use IPv4 Addresses</a>  </li> <li>  <a href="https://tools.ietf.org/html/rfc6598">RFC 6598, IANA-Reserved IPv4 Prefix for Shared Address Space</a>  </li> <li>  <a href="https://tools.ietf.org/html/rfc5156">RFC 5156, Special-Use IPv6 Addresses</a>  </li> </ul> When the value of <code>Type</code> is <code>CALCULATED</code> or <code>CLOUDWATCH_METRIC</code>, omit <code>IPAddress</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the CloudWatch alarm that you want Amazon Route 53 health checkers to use to determine whether this health check is healthy. <note> Route 53 supports CloudWatch alarms with the following features: <ul> <li> Standard-resolution metrics. High-resolution metrics aren't supported. For more information, see <a href="http://docs.aws.amazon.com/AmazonCloudWatch/latest/DeveloperGuide/publishingMetrics.html#high-resolution-metrics">High-Resolution Metrics</a> in the <i>Amazon CloudWatch User Guide</i>. </li> <li> Statistics: Average, Minimum, Maximum, Sum, and SampleCount. Extended statistics aren't supported. </li> </ul> </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Region</i> For the CloudWatch alarm that you want Route 53 health checkers to use to determine whether this health check is healthy, the region that the alarm was created in. For the current list of CloudWatch regions, see <a href="http://docs.aws.amazon.com/general/latest/gr/rande.html#cw_region">Amazon CloudWatch</a> in the <i>AWS Regions and Endpoints</i> chapter of the <i>Amazon Web Services General Reference</i>.<br/>
        :return: Location:  The unique URL representing the new health check.<br/>HealthCheck:  A complex type that contains identifying information about the health check.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchAlarmConfiguration</i> A complex type that contains information about the CloudWatch alarm that Amazon Route 53 is monitoring for this health check.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CloudWatchAlarmConfiguration</b>: A complex type that contains information about the CloudWatch alarm that Amazon Route 53 is monitoring for this health check.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HealthCheckVersion</i> The version of the health check. You can optionally pass this value in a call to <code>UpdateHealthCheck</code> to prevent overwriting another change to the health check.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HealthCheckConfig</i> A complex type that contains detailed information about one health check.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>HealthCheckConfig</b>: A complex type that contains detailed information about one health check.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LinkedService</i> If the health check was created by another service, the service that created the health check. When a health check is created by another service, you can't edit or delete it using Amazon Route 53. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LinkedService</b>: If the health check was created by another service, the service that created the health check. When a health check is created by another service, you can't edit or delete it using Amazon Route 53. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique string that you specified when you created the health check.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier that Amazon Route 53assigned to the health check when you created it. When you add or update a resource record set, you use this value to specify which health check to use. The value can be up to 64 characters long. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> If the health check or hosted zone was created by another service, an optional description that can be provided by the other service. When a resource is created by another service, you can't edit or delete it using Amazon Route 53. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServicePrincipal</i> If the health check or hosted zone was created by another service, the service that created the resource. When a resource is created by another service, you can't edit or delete it using Amazon Route 53. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Regions</i> A complex type that contains one <code>Region</code> element for each region from which you want Amazon Route 53 health checkers to check the specified endpoint. If you don't specify any regions, Route 53 health checkers automatically performs checks from all of the regions that are listed under <b>Valid Values</b>. If you update a health check to remove a region that has been performing health checks, Route 53 will briefly continue to perform checks from that region to ensure that some health checkers are always checking the endpoint (for example, if you replace three regions with four different regions). <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EnableSNI</i> Specify whether you want Amazon Route 53 to send the value of <code>FullyQualifiedDomainName</code> to the endpoint in the <code>client_hello</code> message during TLS negotiation. This allows the endpoint to respond to <code>HTTPS</code> health check requests with the applicable SSL/TLS certificate. Some endpoints require that <code>HTTPS</code> requests include the host name in the <code>client_hello</code> message. If you don't enable SNI, the status of the health check will be <code>SSL alert handshake_failure</code>. A health check can also have that status for other reasons. If SNI is enabled and you're still getting the error, check the SSL/TLS configuration on your endpoint and confirm that your certificate is valid. The SSL/TLS certificate on your endpoint includes a domain name in the <code>Common Name</code> field and possibly several more in the <code>Subject Alternative Names</code> field. One of the domain names in the certificate should match the value that you specify for <code>FullyQualifiedDomainName</code>. If the endpoint responds to the <code>client_hello</code> message with a certificate that does not include the domain name that you specified in <code>FullyQualifiedDomainName</code>, a health checker will retry the handshake. In the second attempt, the health checker will omit <code>FullyQualifiedDomainName</code> from the <code>client_hello</code> message.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ChildHealthChecks</i> (CALCULATED Health Checks Only) A complex type that contains one <code>ChildHealthCheck</code> element for each health check that you want to associate with a <code>CALCULATED</code> health check.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HealthThreshold</i> The number of child health checks that are associated with a <code>CALCULATED</code> health check that Amazon Route 53 must consider healthy for the <code>CALCULATED</code> health check to be considered healthy. To specify the child health checks that you want to associate with a <code>CALCULATED</code> health check, use the <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_UpdateHealthCheck.html#Route53-UpdateHealthCheck-request-ChildHealthChecks">ChildHealthChecks</a> element. Note the following: <ul> <li> If you specify a number greater than the number of child health checks, Route 53 always considers this health check to be unhealthy. </li> <li> If you specify <code>0</code>, Route 53 always considers this health check to be healthy. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Disabled</i> Stops Route 53 from performing health checks. When you disable a health check, here's what happens: <ul> <li>  <b>Health checks that check the health of endpoints:</b> Route 53 stops submitting requests to your application, server, or other resource. </li> <li>  <b>Calculated health checks:</b> Route 53 stops aggregating the status of the referenced health checks. </li> <li>  <b>Health checks that monitor CloudWatch alarms:</b> Route 53 stops monitoring the corresponding CloudWatch metrics. </li> </ul> After you disable a health check, Route 53 considers the status of the health check to always be healthy. If you configured DNS failover, Route 53 continues to route traffic to the corresponding resources. If you want to stop routing traffic to a resource, change the value of <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_UpdateHealthCheck.html#Route53-UpdateHealthCheck-request-Inverted">Inverted</a>.  Charges for a health check still apply when the health check is disabled. For more information, see <a href="http://aws.amazon.com/route53/pricing/">Amazon Route 53 Pricing</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Inverted</i> Specify whether you want Amazon Route 53 to invert the status of a health check, for example, to consider a health check unhealthy when it otherwise would be considered healthy.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MeasureLatency</i> Specify whether you want Amazon Route 53 to measure the latency between health checkers in multiple AWS regions and your endpoint, and to display CloudWatch latency graphs on the <b>Health Checks</b> page in the Route 53 console. <important> You can't change the value of <code>MeasureLatency</code> after you create a health check. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureThreshold</i> The number of consecutive health checks that an endpoint must pass or fail for Amazon Route 53 to change the current status of the endpoint from unhealthy to healthy or vice versa. For more information, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/dns-failover-determining-health-of-endpoints.html">How Amazon Route 53 Determines Whether an Endpoint Is Healthy</a> in the <i>Amazon Route 53 Developer Guide</i>. If you don't specify a value for <code>FailureThreshold</code>, the default value is three health checks.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestInterval</i> The number of seconds between the time that Amazon Route 53 gets a response from your endpoint and the time that it sends the next health check request. Each Route 53 health checker makes requests at this interval. <important> You can't change the value of <code>RequestInterval</code> after you create a health check. </important> If you don't specify a value for <code>RequestInterval</code>, the default value is <code>30</code> seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SearchString</i> If the value of Type is <code>HTTP_STR_MATCH</code> or <code>HTTP_STR_MATCH</code>, the string that you want Amazon Route 53 to search for in the response body from the specified resource. If the string appears in the response body, Route 53 considers the resource healthy. Route 53 considers case when searching for <code>SearchString</code> in the response body. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FullyQualifiedDomainName</i> Amazon Route 53 behavior depends on whether you specify a value for <code>IPAddress</code>.  <b>If you specify a value for</b> <code>IPAddress</code>: Amazon Route 53 sends health check requests to the specified IPv4 or IPv6 address and passes the value of <code>FullyQualifiedDomainName</code> in the <code>Host</code> header for all health checks except TCP health checks. This is typically the fully qualified DNS name of the endpoint on which you want Route 53 to perform health checks. When Route 53 checks the health of an endpoint, here is how it constructs the <code>Host</code> header: <ul> <li> If you specify a value of <code>80</code> for <code>Port</code> and <code>HTTP</code> or <code>HTTP_STR_MATCH</code> for <code>Type</code>, Route 53 passes the value of <code>FullyQualifiedDomainName</code> to the endpoint in the Host header.  </li> <li> If you specify a value of <code>443</code> for <code>Port</code> and <code>HTTPS</code> or <code>HTTPS_STR_MATCH</code> for <code>Type</code>, Route 53 passes the value of <code>FullyQualifiedDomainName</code> to the endpoint in the <code>Host</code> header. </li> <li> If you specify another value for <code>Port</code> and any value except <code>TCP</code> for <code>Type</code>, Route 53 passes <code>FullyQualifiedDomainName:Port</code> to the endpoint in the <code>Host</code> header. </li> </ul> If you don't specify a value for <code>FullyQualifiedDomainName</code>, Route 53 substitutes the value of <code>IPAddress</code> in the <code>Host</code> header in each of the preceding cases.  <b>If you don't specify a value for <code>IPAddress</code> </b>: Route 53 sends a DNS request to the domain that you specify for <code>FullyQualifiedDomainName</code> at the interval that you specify for <code>RequestInterval</code>. Using an IPv4 address that DNS returns, Route 53 then checks the health of the endpoint. <note> If you don't specify a value for <code>IPAddress</code>, Route 53 uses only IPv4 to send health checks to the endpoint. If there's no resource record set with a type of A for the name that you specify for <code>FullyQualifiedDomainName</code>, the health check fails with a "DNS resolution failed" error. </note> If you want to check the health of weighted, latency, or failover resource record sets and you choose to specify the endpoint only by <code>FullyQualifiedDomainName</code>, we recommend that you create a separate health check for each endpoint. For example, create a health check for each HTTP server that is serving content for www.example.com. For the value of <code>FullyQualifiedDomainName</code>, specify the domain name of the server (such as us-east-2-www.example.com), not the name of the resource record sets (www.example.com). <important> In this configuration, if you create a health check for which the value of <code>FullyQualifiedDomainName</code> matches the name of the resource record sets and you then associate the health check with those resource record sets, health check results will be unpredictable. </important> In addition, if the value that you specify for <code>Type</code> is <code>HTTP</code>, <code>HTTPS</code>, <code>HTTP_STR_MATCH</code>, or <code>HTTPS_STR_MATCH</code>, Route 53 passes the value of <code>FullyQualifiedDomainName</code> in the <code>Host</code> header, as it does when you specify a value for <code>IPAddress</code>. If the value of <code>Type</code> is <code>TCP</code>, Route 53 doesn't pass a <code>Host</code> header.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourcePath</i> The path, if any, that you want Amazon Route 53 to request when performing health checks. The path can be any value for which your endpoint will return an HTTP status code of 2xx or 3xx when the endpoint is healthy, for example, the file /docs/route53-health-check.html. You can also include query string parameters, for example, <code>/welcome.html?language=jp&amp;login=y</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of health check that you want to create, which indicates how Amazon Route 53 determines whether an endpoint is healthy. <important> You can't change the value of <code>Type</code> after you create a health check. </important> You can create the following types of health checks: <ul> <li>  <b>HTTP</b>: Route 53 tries to establish a TCP connection. If successful, Route 53 submits an HTTP request and waits for an HTTP status code of 200 or greater and less than 400. </li> <li>  <b>HTTPS</b>: Route 53 tries to establish a TCP connection. If successful, Route 53 submits an HTTPS request and waits for an HTTP status code of 200 or greater and less than 400. <important> If you specify <code>HTTPS</code> for the value of <code>Type</code>, the endpoint must support TLS v1.0 or later. </important> </li> <li>  <b>HTTP_STR_MATCH</b>: Route 53 tries to establish a TCP connection. If successful, Route 53 submits an HTTP request and searches the first 5,120 bytes of the response body for the string that you specify in <code>SearchString</code>. </li> <li>  <b>HTTPS_STR_MATCH</b>: Route 53 tries to establish a TCP connection. If successful, Route 53 submits an <code>HTTPS</code> request and searches the first 5,120 bytes of the response body for the string that you specify in <code>SearchString</code>. </li> <li>  <b>TCP</b>: Route 53 tries to establish a TCP connection. </li> <li>  <b>CLOUDWATCH_METRIC</b>: The health check is associated with a CloudWatch alarm. If the state of the alarm is <code>OK</code>, the health check is considered healthy. If the state is <code>ALARM</code>, the health check is considered unhealthy. If CloudWatch doesn't have sufficient data to determine whether the state is <code>OK</code> or <code>ALARM</code>, the health check status depends on the setting for <code>InsufficientDataHealthStatus</code>: <code>Healthy</code>, <code>Unhealthy</code>, or <code>LastKnownStatus</code>.  </li> <li>  <b>CALCULATED</b>: For health checks that monitor the status of other health checks, Route 53 adds up the number of health checks that Route 53 health checkers consider to be healthy and compares that number with the value of <code>HealthThreshold</code>.  </li> </ul> For more information, see <a href="https://docs.aws.amazon.com/Route53/latest/DeveloperGuide/dns-failover-determining-health-of-endpoints.html">How Route 53 Determines Whether an Endpoint Is Healthy</a> in the <i>Amazon Route 53 Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port on the endpoint on which you want Amazon Route 53 to perform health checks. Specify a value for <code>Port</code> only when you specify a value for <code>IPAddress</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IPAddress</i> The IPv4 or IPv6 IP address of the endpoint that you want Amazon Route 53 to perform health checks on. If you don't specify a value for <code>IPAddress</code>, Route 53 sends a DNS request to resolve the domain name that you specify in <code>FullyQualifiedDomainName</code> at the interval that you specify in <code>RequestInterval</code>. Using an IP address returned by DNS, Route 53 then checks the health of the endpoint. Use one of the following formats for the value of <code>IPAddress</code>:  <ul> <li>  <b>IPv4 address</b>: four values between 0 and 255, separated by periods (.), for example, <code>192.0.2.44</code>. </li> <li>  <b>IPv6 address</b>: eight groups of four hexadecimal values, separated by colons (:), for example, <code>2001:0db8:85a3:0000:0000:abcd:0001:2345</code>. You can also shorten IPv6 addresses as described in RFC 5952, for example, <code>2001:db8:85a3::abcd:1:2345</code>. </li> </ul> If the endpoint is an EC2 instance, we recommend that you create an Elastic IP address, associate it with your EC2 instance, and specify the Elastic IP address for <code>IPAddress</code>. This ensures that the IP address of your instance will never change. For more information, see <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_UpdateHealthCheck.html#Route53-UpdateHealthCheck-request-FullyQualifiedDomainName">FullyQualifiedDomainName</a>.  Constraints: Route 53 can't check the health of endpoints for which the IP address is in local, private, non-routable, or multicast ranges. For more information about IP addresses for which you can't create health checks, see the following documents: <ul> <li>  <a href="https://tools.ietf.org/html/rfc5735">RFC 5735, Special Use IPv4 Addresses</a>  </li> <li>  <a href="https://tools.ietf.org/html/rfc6598">RFC 6598, IANA-Reserved IPv4 Prefix for Shared Address Space</a>  </li> <li>  <a href="https://tools.ietf.org/html/rfc5156">RFC 5156, Special-Use IPv6 Addresses</a>  </li> </ul> When the value of <code>Type</code> is <code>CALCULATED</code> or <code>CLOUDWATCH_METRIC</code>, omit <code>IPAddress</code>.<br/>A complex type that identifies the CloudWatch alarm that you want Amazon Route 53 health checkers to use to determine whether the specified health check is healthy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AlarmIdentifier</b>: A complex type that identifies the CloudWatch alarm that you want Amazon Route 53 health checkers to use to determine whether the specified health check is healthy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Dimensions</i> For the metric that the CloudWatch alarm is associated with, a complex type that contains information about the dimensions for the metric. For information, see <a href="http://docs.aws.amazon.com/AmazonCloudWatch/latest/DeveloperGuide/CW_Support_For_AWS.html">Amazon CloudWatch Namespaces, Dimensions, and Metrics Reference</a> in the <i>Amazon CloudWatch User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Statistic</i> For the metric that the CloudWatch alarm is associated with, the statistic that is applied to the metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Namespace</i> The namespace of the metric that the alarm is associated with. For more information, see <a href="http://docs.aws.amazon.com/AmazonCloudWatch/latest/DeveloperGuide/CW_Support_For_AWS.html">Amazon CloudWatch Namespaces, Dimensions, and Metrics Reference</a> in the <i>Amazon CloudWatch User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricName</i> The name of the CloudWatch metric that the alarm is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Period</i> For the metric that the CloudWatch alarm is associated with, the duration of one evaluation period in seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ComparisonOperator</i> For the metric that the CloudWatch alarm is associated with, the arithmetic operation that is used for the comparison.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Threshold</i> For the metric that the CloudWatch alarm is associated with, the value the metric is compared with.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EvaluationPeriods</i> For the metric that the CloudWatch alarm is associated with, the number of periods that the metric is compared to the threshold.<br/>        """
        pass

    def create_hosted_zone(self, CallerReference: str, Name: str, VPC: VPC=None, HostedZoneConfig: HostedZoneConfig=None, DelegationSetId: str=None) -> CreateHostedZoneResponse:
        """Creates a new public or private hosted zone. You create records in a public hosted zone to define how you want to route traffic on the internet for a domain, such as example.com, and its subdomains (apex.example.com, acme.example.com). You create records in a private hosted zone to define how you want to route traffic for a domain and its subdomains within one or more Amazon Virtual Private Clouds (Amazon VPCs).  <important> You can't convert a public hosted zone to a private hosted zone or vice versa. Instead, you must create a new hosted zone with the same name and create new resource record sets. </important> For more information about charges for hosted zones, see <a href="http://aws.amazon.com/route53/pricing/">Amazon Route 53 Pricing</a>. Note the following: <ul> <li> You can't create a hosted zone for a top-level domain (TLD) such as .com. </li> <li> For public hosted zones, Amazon Route 53 automatically creates a default SOA record and four NS records for the zone. For more information about SOA and NS records, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/SOA-NSrecords.html">NS and SOA Records that Route 53 Creates for a Hosted Zone</a> in the <i>Amazon Route 53 Developer Guide</i>. If you want to use the same name servers for multiple public hosted zones, you can optionally associate a reusable delegation set with the hosted zone. See the <code>DelegationSetId</code> element. </li> <li> If your domain is registered with a registrar other than Route 53, you must update the name servers with your registrar to make Route 53 the DNS service for the domain. For more information, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/MigratingDNS.html">Migrating DNS Service for an Existing Domain to Amazon Route 53</a> in the <i>Amazon Route 53 Developer Guide</i>.  </li> </ul> When you submit a <code>CreateHostedZone</code> request, the initial status of the hosted zone is <code>PENDING</code>. For public hosted zones, this means that the NS and SOA records are not yet available on all Route 53 DNS servers. When the NS and SOA records are available, the status of the zone changes to <code>INSYNC</code>.

        :param self:
        :param DelegationSetId:  If you want to associate a reusable delegation set with this hosted zone, the ID that Amazon Route 53 assigned to the reusable delegation set when you created it. For more information about reusable delegation sets, see <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_CreateReusableDelegationSet.html">CreateReusableDelegationSet</a>.<br/>HostedZoneConfig:  (Optional) A complex type that contains the following optional values: <ul> <li> For public and private hosted zones, an optional comment </li> <li> For private hosted zones, an optional <code>PrivateZone</code> element </li> </ul> If you don't specify a comment or the <code>PrivateZone</code> element, omit <code>HostedZoneConfig</code> and the other elements.:<br/>CallerReference:  A unique string that identifies the request and that allows failed <code>CreateHostedZone</code> requests to be retried without the risk of executing the operation twice. You must use a unique <code>CallerReference</code> string every time you submit a <code>CreateHostedZone</code> request. <code>CallerReference</code> can be any unique string, for example, a date/time stamp.<br/>VPC:  (Private hosted zones only) A complex type that contains information about the Amazon VPC that you're associating with this hosted zone. You can specify only one Amazon VPC when you create a private hosted zone. To associate additional Amazon VPCs with the hosted zone, use <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_AssociateVPCWithHostedZone.html">AssociateVPCWithHostedZone</a> after you create a hosted zone.:<br/>Name:  The name of the domain. Specify a fully qualified domain name, for example, <i>www.example.com</i>. The trailing dot is optional; Amazon Route 53 assumes that the domain name is fully qualified. This means that Route 53 treats <i>www.example.com</i> (without a trailing dot) and <i>www.example.com.</i> (with a trailing dot) as identical. If you're creating a public hosted zone, this is the name you have registered with your DNS registrar. If your domain name is registered with a registrar other than Route 53, change the name servers for your domain to the set of <code>NameServers</code> that <code>CreateHostedZone</code> returns in <code>DelegationSet</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VPCId</i> (Private hosted zones only) The ID of an Amazon VPC. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VPCRegion</i> (Private hosted zones only) The region that an Amazon VPC was created in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrivateZone</i> A value that indicates whether this is a private hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> Any comments that you want to include about the hosted zone.<br/>
        :return: Location:  The unique URL representing the new hosted zone.<br/>VPC:  A complex type that contains information about an Amazon VPC that you associated with this hosted zone.:<br/>DelegationSet:  A complex type that describes the name servers for this hosted zone.:<br/>ChangeInfo:  A complex type that contains information about the <code>CreateHostedZone</code> request.:<br/>HostedZone:  A complex type that contains general information about the hosted zone.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LinkedService</i> If the hosted zone was created by another service, the service that created the hosted zone. When a hosted zone is created by another service, you can't edit or delete it using Route 53. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LinkedService</b>: If the hosted zone was created by another service, the service that created the hosted zone. When a hosted zone is created by another service, you can't edit or delete it using Route 53. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceRecordSetCount</i> The number of resource record sets in the hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Config</i> A complex type that includes the <code>Comment</code> and <code>PrivateZone</code> elements. If you omitted the <code>HostedZoneConfig</code> and <code>Comment</code> elements from the request, the <code>Config</code> and <code>Comment</code> elements don't appear in the response.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Config</b>: A complex type that includes the <code>Comment</code> and <code>PrivateZone</code> elements. If you omitted the <code>HostedZoneConfig</code> and <code>Comment</code> elements from the request, the <code>Config</code> and <code>Comment</code> elements don't appear in the response.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> The value that you specified for <code>CallerReference</code> when you created the hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the domain. For public hosted zones, this is the name that you have registered with your DNS registrar. For information about how to specify characters other than <code>a-z</code>, <code>0-9</code>, and <code>-</code> (hyphen) and how to specify internationalized domain names, see <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_CreateHostedZone.html">CreateHostedZone</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID that Amazon Route 53 assigned to the hosted zone when you created it.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PrivateZone</i> A value that indicates whether this is a private hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> Any comments that you want to include about the hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> If the health check or hosted zone was created by another service, an optional description that can be provided by the other service. When a resource is created by another service, you can't edit or delete it using Amazon Route 53. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServicePrincipal</i> If the health check or hosted zone was created by another service, the service that created the resource. When a resource is created by another service, you can't edit or delete it using Amazon Route 53. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> A complex type that describes change information about changes made to your hosted zone. This element contains an ID that you use when performing a <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_GetChange.html">GetChange</a> action to get detailed information about the change.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmittedAt</i> The date and time that the change request was submitted in <a href="https://en.wikipedia.org/wiki/ISO_8601">ISO 8601 format</a> and Coordinated Universal Time (UTC). For example, the value <code>2017-03-27T17:48:16.751Z</code> represents March 27, 2017 at 17:48:16.751 UTC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current state of the request. <code>PENDING</code> indicates that this request has not yet been applied to all Amazon Route 53 DNS servers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NameServers</i> A complex type that contains a list of the authoritative name servers for a hosted zone or for a reusable delegation set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> The value that you specified for <code>CallerReference</code> when you created the reusable delegation set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID that Amazon Route 53 assigns to a reusable delegation set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VPCId</i> (Private hosted zones only) The ID of an Amazon VPC. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VPCRegion</i> (Private hosted zones only) The region that an Amazon VPC was created in.<br/>        """
        pass

    def create_query_logging_config(self, CloudWatchLogsLogGroupArn: str, HostedZoneId: str) -> CreateQueryLoggingConfigResponse:
        """Creates a configuration for DNS query logging. After you create a query logging configuration, Amazon Route 53 begins to publish log data to an Amazon CloudWatch Logs log group. DNS query logs contain information about the queries that Route 53 receives for a specified public hosted zone, such as the following: <ul> <li> Route 53 edge location that responded to the DNS query </li> <li> Domain or subdomain that was requested </li> <li> DNS record type, such as A or AAAA </li> <li> DNS response code, such as <code>NoError</code> or <code>ServFail</code>  </li> </ul> <dl> <dt>Log Group and Resource Policy</dt> <dd> Before you create a query logging configuration, perform the following operations. <note> If you create a query logging configuration using the Route 53 console, Route 53 performs these operations automatically. </note> <ol> <li> Create a CloudWatch Logs log group, and make note of the ARN, which you specify when you create a query logging configuration. Note the following: <ul> <li> You must create the log group in the us-east-1 region. </li> <li> You must use the same AWS account to create the log group and the hosted zone that you want to configure query logging for. </li> <li> When you create log groups for query logging, we recommend that you use a consistent prefix, for example:  <code>/aws/route53/<i>hosted zone name</i> </code>  In the next step, you'll create a resource policy, which controls access to one or more log groups and the associated AWS resources, such as Route 53 hosted zones. There's a limit on the number of resource policies that you can create, so we recommend that you use a consistent prefix so you can use the same resource policy for all the log groups that you create for query logging. </li> </ul> </li> <li> Create a CloudWatch Logs resource policy, and give it the permissions that Route 53 needs to create log streams and to send query logs to log streams. For the value of <code>Resource</code>, specify the ARN for the log group that you created in the previous step. To use the same resource policy for all the CloudWatch Logs log groups that you created for query logging configurations, replace the hosted zone name with <code>*</code>, for example:  <code>arn:aws:logs:us-east-1:123412341234:log-group:/aws/route53/*</code>  <note> You can't use the CloudWatch console to create or edit a resource policy. You must use the CloudWatch API, one of the AWS SDKs, or the AWS CLI. </note> </li> </ol> </dd> <dt>Log Streams and Edge Locations</dt> <dd> When Route 53 finishes creating the configuration for DNS query logging, it does the following: <ul> <li> Creates a log stream for an edge location the first time that the edge location responds to DNS queries for the specified hosted zone. That log stream is used to log all queries that Route 53 responds to for that edge location. </li> <li> Begins to send query logs to the applicable log stream. </li> </ul> The name of each log stream is in the following format:  <code> <i>hosted zone ID</i>/<i>edge location code</i> </code>  The edge location code is a three-letter code and an arbitrarily assigned number, for example, DFW3. The three-letter code typically corresponds with the International Air Transport Association airport code for an airport near the edge location. (These abbreviations might change in the future.) For a list of edge locations, see "The Route 53 Global Network" on the <a href="http://aws.amazon.com/route53/details/">Route 53 Product Details</a> page. </dd> <dt>Queries That Are Logged</dt> <dd> Query logs contain only the queries that DNS resolvers forward to Route 53. If a DNS resolver has already cached the response to a query (such as the IP address for a load balancer for example.com), the resolver will continue to return the cached response. It doesn't forward another query to Route 53 until the TTL for the corresponding resource record set expires. Depending on how many DNS queries are submitted for a resource record set, and depending on the TTL for that resource record set, query logs might contain information about only one query out of every several thousand queries that are submitted to DNS. For more information about how DNS works, see <a href="https://docs.aws.amazon.com/Route53/latest/DeveloperGuide/welcome-dns-service.html">Routing Internet Traffic to Your Website or Web Application</a> in the <i>Amazon Route 53 Developer Guide</i>. </dd> <dt>Log File Format</dt> <dd> For a list of the values in each query log and the format of each value, see <a href="https://docs.aws.amazon.com/Route53/latest/DeveloperGuide/query-logs.html">Logging DNS Queries</a> in the <i>Amazon Route 53 Developer Guide</i>. </dd> <dt>Pricing</dt> <dd> For information about charges for query logs, see <a href="http://aws.amazon.com/cloudwatch/pricing/">Amazon CloudWatch Pricing</a>. </dd> <dt>How to Stop Logging</dt> <dd> If you want Route 53 to stop sending query logs to CloudWatch Logs, delete the query logging configuration. For more information, see <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_DeleteQueryLoggingConfig.html">DeleteQueryLoggingConfig</a>. </dd> </dl>

        :param self:
        :param CloudWatchLogsLogGroupArn:  The Amazon Resource Name (ARN) for the log group that you want to Amazon Route 53 to send query logs to. This is the format of the ARN: arn:aws:logs:<i>region</i>:<i>account-id</i>:log-group:<i>log_group_name</i>  To get the ARN for a log group, you can use the CloudWatch console, the <a href="https://docs.aws.amazon.com/AmazonCloudWatchLogs/latest/APIReference/API_DescribeLogGroups.html">DescribeLogGroups</a> API action, the <a href="https://docs.aws.amazon.com/cli/latest/reference/logs/describe-log-groups.html">describe-log-groups</a> command, or the applicable command in one of the AWS SDKs.<br/>HostedZoneId:  The ID of the hosted zone that you want to log queries for. You can log queries only for public hosted zones.<br/>
        :return: Location:  The unique URL representing the new query logging configuration.<br/>QueryLoggingConfig:  A complex type that contains the ID for a query logging configuration, the ID of the hosted zone that you want to log queries for, and the ARN for the log group that you want Amazon Route 53 to send query logs to.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchLogsLogGroupArn</i> The Amazon Resource Name (ARN) of the CloudWatch Logs log group that Amazon Route 53 is publishing logs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> The ID of the hosted zone that CloudWatch Logs is logging queries for. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID for a configuration for DNS query logging.<br/>        """
        pass

    def create_reusable_delegation_set(self, CallerReference: str, HostedZoneId: str=None) -> CreateReusableDelegationSetResponse:
        """Creates a delegation set (a group of four name servers) that can be reused by multiple hosted zones. If a hosted zoned ID is specified, <code>CreateReusableDelegationSet</code> marks the delegation set associated with that zone as reusable. <note> You can't associate a reusable delegation set with a private hosted zone. </note> For information about using a reusable delegation set to configure white label name servers, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/white-label-name-servers.html">Configuring White Label Name Servers</a>. The process for migrating existing hosted zones to use a reusable delegation set is comparable to the process for configuring white label name servers. You need to perform the following steps: <ol> <li> Create a reusable delegation set. </li> <li> Recreate hosted zones, and reduce the TTL to 60 seconds or less. </li> <li> Recreate resource record sets in the new hosted zones. </li> <li> Change the registrar's name servers to use the name servers for the new hosted zones. </li> <li> Monitor traffic for the website or application. </li> <li> Change TTLs back to their original values. </li> </ol> If you want to migrate existing hosted zones to use a reusable delegation set, the existing hosted zones can't use any of the name servers that are assigned to the reusable delegation set. If one or more hosted zones do use one or more name servers that are assigned to the reusable delegation set, you can do one of the following: <ul> <li> For small numbers of hosted zones—up to a few hundred—it's relatively easy to create reusable delegation sets until you get one that has four name servers that don't overlap with any of the name servers in your hosted zones. </li> <li> For larger numbers of hosted zones, the easiest solution is to use more than one reusable delegation set. </li> <li> For larger numbers of hosted zones, you can also migrate hosted zones that have overlapping name servers to hosted zones that don't have overlapping name servers, then migrate the hosted zones again to use the reusable delegation set. </li> </ul>

        :param self:
        :param HostedZoneId:  If you want to mark the delegation set for an existing hosted zone as reusable, the ID for that hosted zone.<br/>CallerReference:  A unique string that identifies the request, and that allows you to retry failed <code>CreateReusableDelegationSet</code> requests without the risk of executing the operation twice. You must use a unique <code>CallerReference</code> string every time you submit a <code>CreateReusableDelegationSet</code> request. <code>CallerReference</code> can be any unique string, for example a date/time stamp.<br/>
        :return: Location:  The unique URL representing the new reusable delegation set.<br/>DelegationSet:  A complex type that contains name server information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NameServers</i> A complex type that contains a list of the authoritative name servers for a hosted zone or for a reusable delegation set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> The value that you specified for <code>CallerReference</code> when you created the reusable delegation set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID that Amazon Route 53 assigns to a reusable delegation set.<br/>        """
        pass

    def create_traffic_policy(self, Document: str, Name: str, Comment: str=None) -> CreateTrafficPolicyResponse:
        """Creates a traffic policy, which you use to create multiple DNS resource record sets for one domain name (such as example.com) or one subdomain name (such as www.example.com).

        :param self:
        :param Comment:  (Optional) Any comments that you want to include about the traffic policy.<br/>Document:  The definition of this traffic policy in JSON format. For more information, see <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/api-policies-traffic-policy-document-format.html">Traffic Policy Document Format</a>.<br/>Name:  The name of the traffic policy.<br/>
        :return: Location:  A unique URL that represents a new traffic policy.<br/>TrafficPolicy:  A complex type that contains settings for the new traffic policy.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> The comment that you specify in the <code>CreateTrafficPolicy</code> request, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Document</i> The definition of a traffic policy in JSON format. You specify the JSON document to use for a new traffic policy in the <code>CreateTrafficPolicy</code> request. For more information about the JSON format, see <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/api-policies-traffic-policy-document-format.html">Traffic Policy Document Format</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The DNS type of the resource record sets that Amazon Route 53 creates when you use a traffic policy to create a traffic policy instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name that you specified when you created the traffic policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version number that Amazon Route 53 assigns to a traffic policy. For a new traffic policy, the value of <code>Version</code> is always 1.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID that Amazon Route 53 assigned to a traffic policy when you created it.<br/>        """
        pass

    def create_traffic_policy_instance(self, TrafficPolicyVersion: int, TrafficPolicyId: str, TTL: TTL, Name: str, HostedZoneId: str) -> CreateTrafficPolicyInstanceResponse:
        """Creates resource record sets in a specified hosted zone based on the settings in a specified traffic policy version. In addition, <code>CreateTrafficPolicyInstance</code> associates the resource record sets with a specified domain name (such as example.com) or subdomain name (such as www.example.com). Amazon Route 53 responds to DNS queries for the domain or subdomain name by using the resource record sets that <code>CreateTrafficPolicyInstance</code> created.

        :param self:
        :param TrafficPolicyVersion:  The version of the traffic policy that you want to use to create resource record sets in the specified hosted zone.<br/>TrafficPolicyId:  The ID of the traffic policy that you want to use to create resource record sets in the specified hosted zone.<br/>TTL:  (Optional) The TTL that you want Amazon Route 53 to assign to all of the resource record sets that it creates in the specified hosted zone.<br/>Name:  The domain name (such as example.com) or subdomain name (such as www.example.com) for which Amazon Route 53 responds to DNS queries by using the resource record sets that Route 53 creates for this traffic policy instance.<br/>HostedZoneId:  The ID of the hosted zone that you want Amazon Route 53 to create resource record sets in by using the configuration in a traffic policy.<br/>
        :return: Location:  A unique URL that represents a new traffic policy instance.<br/>TrafficPolicyInstance:  A complex type that contains settings for the new traffic policy instance.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrafficPolicyType</i> The DNS type that Amazon Route 53 assigned to all of the resource record sets that it created for this traffic policy instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrafficPolicyVersion</i> The version of the traffic policy that Amazon Route 53 used to create resource record sets in the specified hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrafficPolicyId</i> The ID of the traffic policy that Amazon Route 53 used to create resource record sets in the specified hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> If <code>State</code> is <code>Failed</code>, an explanation of the reason for the failure. If <code>State</code> is another value, <code>Message</code> is empty.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The value of <code>State</code> is one of the following values: <dl> <dt>Applied</dt> <dd> Amazon Route 53 has finished creating resource record sets, and changes have propagated to all Route 53 edge locations. </dd> <dt>Creating</dt> <dd> Route 53 is creating the resource record sets. Use <code>GetTrafficPolicyInstance</code> to confirm that the <code>CreateTrafficPolicyInstance</code> request completed successfully. </dd> <dt>Failed</dt> <dd> Route 53 wasn't able to create or update the resource record sets. When the value of <code>State</code> is <code>Failed</code>, see <code>Message</code> for an explanation of what caused the request to fail. </dd> </dl><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TTL</i> The TTL that Amazon Route 53 assigned to all of the resource record sets that it created in the specified hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The DNS name, such as www.example.com, for which Amazon Route 53 responds to queries by using the resource record sets that are associated with this traffic policy instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> The ID of the hosted zone that Amazon Route 53 created resource record sets in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID that Amazon Route 53 assigned to the new traffic policy instance.<br/>        """
        pass

    def create_traffic_policy_version(self, Document: str, Id: str, Comment: str=None) -> CreateTrafficPolicyVersionResponse:
        """Creates a new version of an existing traffic policy. When you create a new version of a traffic policy, you specify the ID of the traffic policy that you want to update and a JSON-formatted document that describes the new version. You use traffic policies to create multiple DNS resource record sets for one domain name (such as example.com) or one subdomain name (such as www.example.com). You can create a maximum of 1000 versions of a traffic policy. If you reach the limit and need to create another version, you'll need to start a new traffic policy.

        :param self:
        :param Comment:  The comment that you specified in the <code>CreateTrafficPolicyVersion</code> request, if any.<br/>Document:  The definition of this version of the traffic policy, in JSON format. You specified the JSON in the <code>CreateTrafficPolicyVersion</code> request. For more information about the JSON format, see <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_CreateTrafficPolicy.html">CreateTrafficPolicy</a>.<br/>Id:  The ID of the traffic policy for which you want to create a new version.<br/>
        :return: Location:  A unique URL that represents a new traffic policy version.<br/>TrafficPolicy:  A complex type that contains settings for the new version of the traffic policy.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> The comment that you specify in the <code>CreateTrafficPolicy</code> request, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Document</i> The definition of a traffic policy in JSON format. You specify the JSON document to use for a new traffic policy in the <code>CreateTrafficPolicy</code> request. For more information about the JSON format, see <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/api-policies-traffic-policy-document-format.html">Traffic Policy Document Format</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The DNS type of the resource record sets that Amazon Route 53 creates when you use a traffic policy to create a traffic policy instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name that you specified when you created the traffic policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version number that Amazon Route 53 assigns to a traffic policy. For a new traffic policy, the value of <code>Version</code> is always 1.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID that Amazon Route 53 assigned to a traffic policy when you created it.<br/>        """
        pass

    def create_vpc_association_authorization(self, VPC: VPC, HostedZoneId: str) -> CreateVPCAssociationAuthorizationResponse:
        """Authorizes the AWS account that created a specified VPC to submit an <code>AssociateVPCWithHostedZone</code> request to associate the VPC with a specified hosted zone that was created by a different account. To submit a <code>CreateVPCAssociationAuthorization</code> request, you must use the account that created the hosted zone. After you authorize the association, use the account that created the VPC to submit an <code>AssociateVPCWithHostedZone</code> request. <note> If you want to associate multiple VPCs that you created by using one account with a hosted zone that you created by using a different account, you must submit one authorization request for each VPC. </note>

        :param self:
        :param VPC:  A complex type that contains the VPC ID and region for the VPC that you want to authorize associating with your hosted zone.:<br/>HostedZoneId:  The ID of the private hosted zone that you want to authorize associating a VPC with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VPCId</i> (Private hosted zones only) The ID of an Amazon VPC. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VPCRegion</i> (Private hosted zones only) The region that an Amazon VPC was created in.<br/>
        :return: VPC:  The VPC that you authorized associating with a hosted zone.:<br/>HostedZoneId:  The ID of the hosted zone that you authorized associating a VPC with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VPCId</i> (Private hosted zones only) The ID of an Amazon VPC. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VPCRegion</i> (Private hosted zones only) The region that an Amazon VPC was created in.<br/>        """
        pass

    def delete_health_check(self, HealthCheckId: str) -> DeleteHealthCheckResponse:
        """Deletes a health check. <important> Amazon Route 53 does not prevent you from deleting a health check even if the health check is associated with one or more resource record sets. If you delete a health check and you don't update the associated resource record sets, the future status of the health check can't be predicted and may change. This will affect the routing of DNS queries for your DNS failover configuration. For more information, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/health-checks-creating-deleting.html#health-checks-deleting.html">Replacing and Deleting Health Checks</a> in the <i>Amazon Route 53 Developer Guide</i>. </important>

        :param self:
        :param HealthCheckId:  The ID of the health check that you want to delete.<br/>
        :return:         """
        pass

    def delete_hosted_zone(self, Id: str) -> DeleteHostedZoneResponse:
        """Deletes a hosted zone. If the hosted zone was created by another service, such as AWS Cloud Map, see <a href="https://docs.aws.amazon.com/Route53/latest/DeveloperGuide/DeleteHostedZone.html#delete-public-hosted-zone-created-by-another-service">Deleting Public Hosted Zones That Were Created by Another Service</a> in the <i>Amazon Route 53 Developer Guide</i> for information about how to delete it. (The process is the same for public and private hosted zones that were created by another service.) If you want to keep your domain registration but you want to stop routing internet traffic to your website or web application, we recommend that you delete resource record sets in the hosted zone instead of deleting the hosted zone. <important> If you delete a hosted zone, you can't undelete it. You must create a new hosted zone and update the name servers for your domain registration, which can require up to 48 hours to take effect. (If you delegated responsibility for a subdomain to a hosted zone and you delete the child hosted zone, you must update the name servers in the parent hosted zone.) In addition, if you delete a hosted zone, someone could hijack the domain and route traffic to their own resources using your domain name. </important> If you want to avoid the monthly charge for the hosted zone, you can transfer DNS service for the domain to a free DNS service. When you transfer DNS service, you have to update the name servers for the domain registration. If the domain is registered with Route 53, see <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_domains_UpdateDomainNameservers.html">UpdateDomainNameservers</a> for information about how to replace Route 53 name servers with name servers for the new DNS service. If the domain is registered with another registrar, use the method provided by the registrar to update name servers for the domain registration. For more information, perform an internet search on "free DNS service." You can delete a hosted zone only if it contains only the default SOA record and NS resource record sets. If the hosted zone contains other resource record sets, you must delete them before you can delete the hosted zone. If you try to delete a hosted zone that contains other resource record sets, the request fails, and Route 53 returns a <code>HostedZoneNotEmpty</code> error. For information about deleting records from your hosted zone, see <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_ChangeResourceRecordSets.html">ChangeResourceRecordSets</a>. To verify that the hosted zone has been deleted, do one of the following: <ul> <li> Use the <code>GetHostedZone</code> action to request information about the hosted zone. </li> <li> Use the <code>ListHostedZones</code> action to get a list of the hosted zones associated with the current AWS account. </li> </ul>

        :param self:
        :param Id:  The ID of the hosted zone you want to delete.<br/>
        :return: ChangeInfo:  A complex type that contains the ID, the status, and the date and time of a request to delete a hosted zone.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> A complex type that describes change information about changes made to your hosted zone. This element contains an ID that you use when performing a <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_GetChange.html">GetChange</a> action to get detailed information about the change.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmittedAt</i> The date and time that the change request was submitted in <a href="https://en.wikipedia.org/wiki/ISO_8601">ISO 8601 format</a> and Coordinated Universal Time (UTC). For example, the value <code>2017-03-27T17:48:16.751Z</code> represents March 27, 2017 at 17:48:16.751 UTC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current state of the request. <code>PENDING</code> indicates that this request has not yet been applied to all Amazon Route 53 DNS servers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the request.<br/>        """
        pass

    def delete_query_logging_config(self, Id: str) -> DeleteQueryLoggingConfigResponse:
        """Deletes a configuration for DNS query logging. If you delete a configuration, Amazon Route 53 stops sending query logs to CloudWatch Logs. Route 53 doesn't delete any logs that are already in CloudWatch Logs. For more information about DNS query logs, see <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_CreateQueryLoggingConfig.html">CreateQueryLoggingConfig</a>.

        :param self:
        :param Id:  The ID of the configuration that you want to delete. <br/>
        :return:         """
        pass

    def delete_reusable_delegation_set(self, Id: str) -> DeleteReusableDelegationSetResponse:
        """Deletes a reusable delegation set. <important> You can delete a reusable delegation set only if it isn't associated with any hosted zones. </important> To verify that the reusable delegation set is not associated with any hosted zones, submit a <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_GetReusableDelegationSet.html">GetReusableDelegationSet</a> request and specify the ID of the reusable delegation set that you want to delete.

        :param self:
        :param Id:  The ID of the reusable delegation set that you want to delete.<br/>
        :return:         """
        pass

    def delete_traffic_policy(self, Version: int, Id: str) -> DeleteTrafficPolicyResponse:
        """Deletes a traffic policy.

        :param self:
        :param Version:  The version number of the traffic policy that you want to delete.<br/>Id:  The ID of the traffic policy that you want to delete.<br/>
        :return:         """
        pass

    def delete_traffic_policy_instance(self, Id: str) -> DeleteTrafficPolicyInstanceResponse:
        """Deletes a traffic policy instance and all of the resource record sets that Amazon Route 53 created when you created the instance. <note> In the Route 53 console, traffic policy instances are known as policy records. </note>

        :param self:
        :param Id:  The ID of the traffic policy instance that you want to delete.  <important> When you delete a traffic policy instance, Amazon Route 53 also deletes all of the resource record sets that were created when you created the traffic policy instance. </important><br/>
        :return:         """
        pass

    def delete_vpc_association_authorization(self, VPC: VPC, HostedZoneId: str) -> DeleteVPCAssociationAuthorizationResponse:
        """Removes authorization to submit an <code>AssociateVPCWithHostedZone</code> request to associate a specified VPC with a hosted zone that was created by a different account. You must use the account that created the hosted zone to submit a <code>DeleteVPCAssociationAuthorization</code> request. <important> Sending this request only prevents the AWS account that created the VPC from associating the VPC with the Amazon Route 53 hosted zone in the future. If the VPC is already associated with the hosted zone, <code>DeleteVPCAssociationAuthorization</code> won't disassociate the VPC from the hosted zone. If you want to delete an existing association, use <code>DisassociateVPCFromHostedZone</code>. </important>

        :param self:
        :param VPC:  When removing authorization to associate a VPC that was created by one AWS account with a hosted zone that was created with a different AWS account, a complex type that includes the ID and region of the VPC.:<br/>HostedZoneId:  When removing authorization to associate a VPC that was created by one AWS account with a hosted zone that was created with a different AWS account, the ID of the hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VPCId</i> (Private hosted zones only) The ID of an Amazon VPC. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VPCRegion</i> (Private hosted zones only) The region that an Amazon VPC was created in.<br/>
        :return:         """
        pass

    def disassociate_vpc_from_hosted_zone(self, VPC: VPC, HostedZoneId: str, Comment: str=None) -> DisassociateVPCFromHostedZoneResponse:
        """Disassociates a VPC from a Amazon Route 53 private hosted zone. Note the following: <ul> <li> You can't disassociate the last VPC from a private hosted zone. </li> <li> You can't convert a private hosted zone into a public hosted zone. </li> <li> You can submit a <code>DisassociateVPCFromHostedZone</code> request using either the account that created the hosted zone or the account that created the VPC. </li> </ul>

        :param self:
        :param Comment:   <i>Optional:</i> A comment about the disassociation request.<br/>VPC:  A complex type that contains information about the VPC that you're disassociating from the specified hosted zone.:<br/>HostedZoneId:  The ID of the private hosted zone that you want to disassociate a VPC from.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VPCId</i> (Private hosted zones only) The ID of an Amazon VPC. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VPCRegion</i> (Private hosted zones only) The region that an Amazon VPC was created in.<br/>
        :return: ChangeInfo:  A complex type that describes the changes made to the specified private hosted zone.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> A complex type that describes change information about changes made to your hosted zone. This element contains an ID that you use when performing a <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_GetChange.html">GetChange</a> action to get detailed information about the change.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmittedAt</i> The date and time that the change request was submitted in <a href="https://en.wikipedia.org/wiki/ISO_8601">ISO 8601 format</a> and Coordinated Universal Time (UTC). For example, the value <code>2017-03-27T17:48:16.751Z</code> represents March 27, 2017 at 17:48:16.751 UTC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current state of the request. <code>PENDING</code> indicates that this request has not yet been applied to all Amazon Route 53 DNS servers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the request.<br/>        """
        pass

    def get_account_limit(self, Type: str) -> GetAccountLimitResponse:
        """Gets the specified limit for the current account, for example, the maximum number of health checks that you can create using the account. For the default limit, see <a href="https://docs.aws.amazon.com/Route53/latest/DeveloperGuide/DNSLimitations.html">Limits</a> in the <i>Amazon Route 53 Developer Guide</i>. To request a higher limit, <a href="https://console.aws.amazon.com/support/home#/case/create?issueType=service-limit-increase&amp;limitType=service-code-route53">open a case</a>. <note> You can also view account limits in AWS Trusted Advisor. Sign in to the AWS Management Console and open the Trusted Advisor console at <a href="https://console.aws.amazon.com/trustedadvisor">https://console.aws.amazon.com/trustedadvisor/</a>. Then choose <b>Service limits</b> in the navigation pane. </note>

        :param self:
        :param Type:  The limit that you want to get. Valid values include the following: <ul> <li>  <b>MAX_HEALTH_CHECKS_BY_OWNER</b>: The maximum number of health checks that you can create using the current account. </li> <li>  <b>MAX_HOSTED_ZONES_BY_OWNER</b>: The maximum number of hosted zones that you can create using the current account. </li> <li>  <b>MAX_REUSABLE_DELEGATION_SETS_BY_OWNER</b>: The maximum number of reusable delegation sets that you can create using the current account. </li> <li>  <b>MAX_TRAFFIC_POLICIES_BY_OWNER</b>: The maximum number of traffic policies that you can create using the current account. </li> <li>  <b>MAX_TRAFFIC_POLICY_INSTANCES_BY_OWNER</b>: The maximum number of traffic policy instances that you can create using the current account. (Traffic policy instances are referred to as traffic flow policy records in the Amazon Route 53 console.) </li> </ul><br/>
        :return: Count:  The current number of entities that you have created of the specified type. For example, if you specified <code>MAX_HEALTH_CHECKS_BY_OWNER</code> for the value of <code>Type</code> in the request, the value of <code>Count</code> is the current number of health checks that you have created using the current account.<br/>Limit:  The current setting for the specified limit. For example, if you specified <code>MAX_HEALTH_CHECKS_BY_OWNER</code> for the value of <code>Type</code> in the request, the value of <code>Limit</code> is the maximum number of health checks that you can create using the current account.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The current value for the limit that is specified by <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_AccountLimit.html#Route53-Type-AccountLimit-Type">Type</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The limit that you requested. Valid values include the following: <ul> <li>  <b>MAX_HEALTH_CHECKS_BY_OWNER</b>: The maximum number of health checks that you can create using the current account. </li> <li>  <b>MAX_HOSTED_ZONES_BY_OWNER</b>: The maximum number of hosted zones that you can create using the current account. </li> <li>  <b>MAX_REUSABLE_DELEGATION_SETS_BY_OWNER</b>: The maximum number of reusable delegation sets that you can create using the current account. </li> <li>  <b>MAX_TRAFFIC_POLICIES_BY_OWNER</b>: The maximum number of traffic policies that you can create using the current account. </li> <li>  <b>MAX_TRAFFIC_POLICY_INSTANCES_BY_OWNER</b>: The maximum number of traffic policy instances that you can create using the current account. (Traffic policy instances are referred to as traffic flow policy records in the Amazon Route 53 console.) </li> </ul><br/>        """
        pass

    def get_change(self, Id: str) -> GetChangeResponse:
        """Returns the current status of a change batch request. The status is one of the following values: <ul> <li>  <code>PENDING</code> indicates that the changes in this request have not propagated to all Amazon Route 53 DNS servers. This is the initial status of all change batch requests. </li> <li>  <code>INSYNC</code> indicates that the changes have propagated to all Route 53 DNS servers.  </li> </ul>

        :param self:
        :param Id:  The ID of the change batch request. The value that you specify here is the value that <code>ChangeResourceRecordSets</code> returned in the <code>Id</code> element when you submitted the request.<br/>
        :return: ChangeInfo:  A complex type that contains information about the specified change batch.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> A complex type that describes change information about changes made to your hosted zone. This element contains an ID that you use when performing a <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_GetChange.html">GetChange</a> action to get detailed information about the change.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubmittedAt</i> The date and time that the change request was submitted in <a href="https://en.wikipedia.org/wiki/ISO_8601">ISO 8601 format</a> and Coordinated Universal Time (UTC). For example, the value <code>2017-03-27T17:48:16.751Z</code> represents March 27, 2017 at 17:48:16.751 UTC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current state of the request. <code>PENDING</code> indicates that this request has not yet been applied to all Amazon Route 53 DNS servers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the request.<br/>        """
        pass

    def get_checker_ip_ranges(self) -> GetCheckerIpRangesResponse:
        """<important>  <code>GetCheckerIpRanges</code> still works, but we recommend that you download ip-ranges.json, which includes IP address ranges for all AWS services. For more information, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/route-53-ip-addresses.html">IP Address Ranges of Amazon Route 53 Servers</a> in the <i>Amazon Route 53 Developer Guide</i>. </important>

        :param self:
        :param 
        :return: CheckerIpRanges:  A complex type that contains sorted list of IP ranges in CIDR format for Amazon Route 53 health checkers.<br/>        """
        pass

    def get_geo_location(self, ContinentCode: str=None, CountryCode: str=None, SubdivisionCode: str=None) -> GetGeoLocationResponse:
        """Gets information about whether a specified geographic location is supported for Amazon Route 53 geolocation resource record sets. Use the following syntax to determine whether a continent is supported for geolocation:  <code>GET /2013-04-01/geolocation?continentcode=<i>two-letter abbreviation for a continent</i> </code>  Use the following syntax to determine whether a country is supported for geolocation:  <code>GET /2013-04-01/geolocation?countrycode=<i>two-character country code</i> </code>  Use the following syntax to determine whether a subdivision of a country is supported for geolocation:  <code>GET /2013-04-01/geolocation?countrycode=<i>two-character country code</i>&amp;subdivisioncode=<i>subdivision code</i> </code> 

        :param self:
        :param SubdivisionCode:  Amazon Route 53 uses the one- to three-letter subdivision codes that are specified in <a href="https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2">ISO standard 3166-1 alpha-2</a>. Route 53 doesn't support subdivision codes for all countries. If you specify <code>subdivisioncode</code>, you must also specify <code>countrycode</code>. <br/>CountryCode:  Amazon Route 53 uses the two-letter country codes that are specified in <a href="https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2">ISO standard 3166-1 alpha-2</a>.<br/>ContinentCode:  Amazon Route 53 supports the following continent codes: <ul> <li>  <b>AF</b>: Africa </li> <li>  <b>AN</b>: Antarctica </li> <li>  <b>AS</b>: Asia </li> <li>  <b>EU</b>: Europe </li> <li>  <b>OC</b>: Oceania </li> <li>  <b>NA</b>: North America </li> <li>  <b>SA</b>: South America </li> </ul><br/>
        :return: GeoLocationDetails:  A complex type that contains the codes and full continent, country, and subdivision names for the specified geolocation code.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubdivisionName</i> The full name of the subdivision. Route 53 currently supports only states in the United States.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubdivisionCode</i> The code for the subdivision. Route 53 currently supports only states in the United States.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CountryName</i> The name of the country.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CountryCode</i> The two-letter code for the country.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContinentName</i> The full name of the continent.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContinentCode</i> The two-letter code for the continent.<br/>        """
        pass

    def get_health_check(self, HealthCheckId: str) -> GetHealthCheckResponse:
        """Gets information about a specified health check.

        :param self:
        :param HealthCheckId:  The identifier that Amazon Route 53 assigned to the health check when you created it. When you add or update a resource record set, you use this value to specify which health check to use. The value can be up to 64 characters long.<br/>
        :return: HealthCheck:  A complex type that contains information about one health check that is associated with the current AWS account.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchAlarmConfiguration</i> A complex type that contains information about the CloudWatch alarm that Amazon Route 53 is monitoring for this health check.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CloudWatchAlarmConfiguration</b>: A complex type that contains information about the CloudWatch alarm that Amazon Route 53 is monitoring for this health check.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HealthCheckVersion</i> The version of the health check. You can optionally pass this value in a call to <code>UpdateHealthCheck</code> to prevent overwriting another change to the health check.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HealthCheckConfig</i> A complex type that contains detailed information about one health check.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>HealthCheckConfig</b>: A complex type that contains detailed information about one health check.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LinkedService</i> If the health check was created by another service, the service that created the health check. When a health check is created by another service, you can't edit or delete it using Amazon Route 53. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LinkedService</b>: If the health check was created by another service, the service that created the health check. When a health check is created by another service, you can't edit or delete it using Amazon Route 53. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique string that you specified when you created the health check.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier that Amazon Route 53assigned to the health check when you created it. When you add or update a resource record set, you use this value to specify which health check to use. The value can be up to 64 characters long. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> If the health check or hosted zone was created by another service, an optional description that can be provided by the other service. When a resource is created by another service, you can't edit or delete it using Amazon Route 53. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServicePrincipal</i> If the health check or hosted zone was created by another service, the service that created the resource. When a resource is created by another service, you can't edit or delete it using Amazon Route 53. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Regions</i> A complex type that contains one <code>Region</code> element for each region from which you want Amazon Route 53 health checkers to check the specified endpoint. If you don't specify any regions, Route 53 health checkers automatically performs checks from all of the regions that are listed under <b>Valid Values</b>. If you update a health check to remove a region that has been performing health checks, Route 53 will briefly continue to perform checks from that region to ensure that some health checkers are always checking the endpoint (for example, if you replace three regions with four different regions). <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EnableSNI</i> Specify whether you want Amazon Route 53 to send the value of <code>FullyQualifiedDomainName</code> to the endpoint in the <code>client_hello</code> message during TLS negotiation. This allows the endpoint to respond to <code>HTTPS</code> health check requests with the applicable SSL/TLS certificate. Some endpoints require that <code>HTTPS</code> requests include the host name in the <code>client_hello</code> message. If you don't enable SNI, the status of the health check will be <code>SSL alert handshake_failure</code>. A health check can also have that status for other reasons. If SNI is enabled and you're still getting the error, check the SSL/TLS configuration on your endpoint and confirm that your certificate is valid. The SSL/TLS certificate on your endpoint includes a domain name in the <code>Common Name</code> field and possibly several more in the <code>Subject Alternative Names</code> field. One of the domain names in the certificate should match the value that you specify for <code>FullyQualifiedDomainName</code>. If the endpoint responds to the <code>client_hello</code> message with a certificate that does not include the domain name that you specified in <code>FullyQualifiedDomainName</code>, a health checker will retry the handshake. In the second attempt, the health checker will omit <code>FullyQualifiedDomainName</code> from the <code>client_hello</code> message.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ChildHealthChecks</i> (CALCULATED Health Checks Only) A complex type that contains one <code>ChildHealthCheck</code> element for each health check that you want to associate with a <code>CALCULATED</code> health check.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HealthThreshold</i> The number of child health checks that are associated with a <code>CALCULATED</code> health check that Amazon Route 53 must consider healthy for the <code>CALCULATED</code> health check to be considered healthy. To specify the child health checks that you want to associate with a <code>CALCULATED</code> health check, use the <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_UpdateHealthCheck.html#Route53-UpdateHealthCheck-request-ChildHealthChecks">ChildHealthChecks</a> element. Note the following: <ul> <li> If you specify a number greater than the number of child health checks, Route 53 always considers this health check to be unhealthy. </li> <li> If you specify <code>0</code>, Route 53 always considers this health check to be healthy. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Disabled</i> Stops Route 53 from performing health checks. When you disable a health check, here's what happens: <ul> <li>  <b>Health checks that check the health of endpoints:</b> Route 53 stops submitting requests to your application, server, or other resource. </li> <li>  <b>Calculated health checks:</b> Route 53 stops aggregating the status of the referenced health checks. </li> <li>  <b>Health checks that monitor CloudWatch alarms:</b> Route 53 stops monitoring the corresponding CloudWatch metrics. </li> </ul> After you disable a health check, Route 53 considers the status of the health check to always be healthy. If you configured DNS failover, Route 53 continues to route traffic to the corresponding resources. If you want to stop routing traffic to a resource, change the value of <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_UpdateHealthCheck.html#Route53-UpdateHealthCheck-request-Inverted">Inverted</a>.  Charges for a health check still apply when the health check is disabled. For more information, see <a href="http://aws.amazon.com/route53/pricing/">Amazon Route 53 Pricing</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Inverted</i> Specify whether you want Amazon Route 53 to invert the status of a health check, for example, to consider a health check unhealthy when it otherwise would be considered healthy.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MeasureLatency</i> Specify whether you want Amazon Route 53 to measure the latency between health checkers in multiple AWS regions and your endpoint, and to display CloudWatch latency graphs on the <b>Health Checks</b> page in the Route 53 console. <important> You can't change the value of <code>MeasureLatency</code> after you create a health check. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureThreshold</i> The number of consecutive health checks that an endpoint must pass or fail for Amazon Route 53 to change the current status of the endpoint from unhealthy to healthy or vice versa. For more information, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/dns-failover-determining-health-of-endpoints.html">How Amazon Route 53 Determines Whether an Endpoint Is Healthy</a> in the <i>Amazon Route 53 Developer Guide</i>. If you don't specify a value for <code>FailureThreshold</code>, the default value is three health checks.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestInterval</i> The number of seconds between the time that Amazon Route 53 gets a response from your endpoint and the time that it sends the next health check request. Each Route 53 health checker makes requests at this interval. <important> You can't change the value of <code>RequestInterval</code> after you create a health check. </important> If you don't specify a value for <code>RequestInterval</code>, the default value is <code>30</code> seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SearchString</i> If the value of Type is <code>HTTP_STR_MATCH</code> or <code>HTTP_STR_MATCH</code>, the string that you want Amazon Route 53 to search for in the response body from the specified resource. If the string appears in the response body, Route 53 considers the resource healthy. Route 53 considers case when searching for <code>SearchString</code> in the response body. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FullyQualifiedDomainName</i> Amazon Route 53 behavior depends on whether you specify a value for <code>IPAddress</code>.  <b>If you specify a value for</b> <code>IPAddress</code>: Amazon Route 53 sends health check requests to the specified IPv4 or IPv6 address and passes the value of <code>FullyQualifiedDomainName</code> in the <code>Host</code> header for all health checks except TCP health checks. This is typically the fully qualified DNS name of the endpoint on which you want Route 53 to perform health checks. When Route 53 checks the health of an endpoint, here is how it constructs the <code>Host</code> header: <ul> <li> If you specify a value of <code>80</code> for <code>Port</code> and <code>HTTP</code> or <code>HTTP_STR_MATCH</code> for <code>Type</code>, Route 53 passes the value of <code>FullyQualifiedDomainName</code> to the endpoint in the Host header.  </li> <li> If you specify a value of <code>443</code> for <code>Port</code> and <code>HTTPS</code> or <code>HTTPS_STR_MATCH</code> for <code>Type</code>, Route 53 passes the value of <code>FullyQualifiedDomainName</code> to the endpoint in the <code>Host</code> header. </li> <li> If you specify another value for <code>Port</code> and any value except <code>TCP</code> for <code>Type</code>, Route 53 passes <code>FullyQualifiedDomainName:Port</code> to the endpoint in the <code>Host</code> header. </li> </ul> If you don't specify a value for <code>FullyQualifiedDomainName</code>, Route 53 substitutes the value of <code>IPAddress</code> in the <code>Host</code> header in each of the preceding cases.  <b>If you don't specify a value for <code>IPAddress</code> </b>: Route 53 sends a DNS request to the domain that you specify for <code>FullyQualifiedDomainName</code> at the interval that you specify for <code>RequestInterval</code>. Using an IPv4 address that DNS returns, Route 53 then checks the health of the endpoint. <note> If you don't specify a value for <code>IPAddress</code>, Route 53 uses only IPv4 to send health checks to the endpoint. If there's no resource record set with a type of A for the name that you specify for <code>FullyQualifiedDomainName</code>, the health check fails with a "DNS resolution failed" error. </note> If you want to check the health of weighted, latency, or failover resource record sets and you choose to specify the endpoint only by <code>FullyQualifiedDomainName</code>, we recommend that you create a separate health check for each endpoint. For example, create a health check for each HTTP server that is serving content for www.example.com. For the value of <code>FullyQualifiedDomainName</code>, specify the domain name of the server (such as us-east-2-www.example.com), not the name of the resource record sets (www.example.com). <important> In this configuration, if you create a health check for which the value of <code>FullyQualifiedDomainName</code> matches the name of the resource record sets and you then associate the health check with those resource record sets, health check results will be unpredictable. </important> In addition, if the value that you specify for <code>Type</code> is <code>HTTP</code>, <code>HTTPS</code>, <code>HTTP_STR_MATCH</code>, or <code>HTTPS_STR_MATCH</code>, Route 53 passes the value of <code>FullyQualifiedDomainName</code> in the <code>Host</code> header, as it does when you specify a value for <code>IPAddress</code>. If the value of <code>Type</code> is <code>TCP</code>, Route 53 doesn't pass a <code>Host</code> header.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourcePath</i> The path, if any, that you want Amazon Route 53 to request when performing health checks. The path can be any value for which your endpoint will return an HTTP status code of 2xx or 3xx when the endpoint is healthy, for example, the file /docs/route53-health-check.html. You can also include query string parameters, for example, <code>/welcome.html?language=jp&amp;login=y</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of health check that you want to create, which indicates how Amazon Route 53 determines whether an endpoint is healthy. <important> You can't change the value of <code>Type</code> after you create a health check. </important> You can create the following types of health checks: <ul> <li>  <b>HTTP</b>: Route 53 tries to establish a TCP connection. If successful, Route 53 submits an HTTP request and waits for an HTTP status code of 200 or greater and less than 400. </li> <li>  <b>HTTPS</b>: Route 53 tries to establish a TCP connection. If successful, Route 53 submits an HTTPS request and waits for an HTTP status code of 200 or greater and less than 400. <important> If you specify <code>HTTPS</code> for the value of <code>Type</code>, the endpoint must support TLS v1.0 or later. </important> </li> <li>  <b>HTTP_STR_MATCH</b>: Route 53 tries to establish a TCP connection. If successful, Route 53 submits an HTTP request and searches the first 5,120 bytes of the response body for the string that you specify in <code>SearchString</code>. </li> <li>  <b>HTTPS_STR_MATCH</b>: Route 53 tries to establish a TCP connection. If successful, Route 53 submits an <code>HTTPS</code> request and searches the first 5,120 bytes of the response body for the string that you specify in <code>SearchString</code>. </li> <li>  <b>TCP</b>: Route 53 tries to establish a TCP connection. </li> <li>  <b>CLOUDWATCH_METRIC</b>: The health check is associated with a CloudWatch alarm. If the state of the alarm is <code>OK</code>, the health check is considered healthy. If the state is <code>ALARM</code>, the health check is considered unhealthy. If CloudWatch doesn't have sufficient data to determine whether the state is <code>OK</code> or <code>ALARM</code>, the health check status depends on the setting for <code>InsufficientDataHealthStatus</code>: <code>Healthy</code>, <code>Unhealthy</code>, or <code>LastKnownStatus</code>.  </li> <li>  <b>CALCULATED</b>: For health checks that monitor the status of other health checks, Route 53 adds up the number of health checks that Route 53 health checkers consider to be healthy and compares that number with the value of <code>HealthThreshold</code>.  </li> </ul> For more information, see <a href="https://docs.aws.amazon.com/Route53/latest/DeveloperGuide/dns-failover-determining-health-of-endpoints.html">How Route 53 Determines Whether an Endpoint Is Healthy</a> in the <i>Amazon Route 53 Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port on the endpoint on which you want Amazon Route 53 to perform health checks. Specify a value for <code>Port</code> only when you specify a value for <code>IPAddress</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IPAddress</i> The IPv4 or IPv6 IP address of the endpoint that you want Amazon Route 53 to perform health checks on. If you don't specify a value for <code>IPAddress</code>, Route 53 sends a DNS request to resolve the domain name that you specify in <code>FullyQualifiedDomainName</code> at the interval that you specify in <code>RequestInterval</code>. Using an IP address returned by DNS, Route 53 then checks the health of the endpoint. Use one of the following formats for the value of <code>IPAddress</code>:  <ul> <li>  <b>IPv4 address</b>: four values between 0 and 255, separated by periods (.), for example, <code>192.0.2.44</code>. </li> <li>  <b>IPv6 address</b>: eight groups of four hexadecimal values, separated by colons (:), for example, <code>2001:0db8:85a3:0000:0000:abcd:0001:2345</code>. You can also shorten IPv6 addresses as described in RFC 5952, for example, <code>2001:db8:85a3::abcd:1:2345</code>. </li> </ul> If the endpoint is an EC2 instance, we recommend that you create an Elastic IP address, associate it with your EC2 instance, and specify the Elastic IP address for <code>IPAddress</code>. This ensures that the IP address of your instance will never change. For more information, see <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_UpdateHealthCheck.html#Route53-UpdateHealthCheck-request-FullyQualifiedDomainName">FullyQualifiedDomainName</a>.  Constraints: Route 53 can't check the health of endpoints for which the IP address is in local, private, non-routable, or multicast ranges. For more information about IP addresses for which you can't create health checks, see the following documents: <ul> <li>  <a href="https://tools.ietf.org/html/rfc5735">RFC 5735, Special Use IPv4 Addresses</a>  </li> <li>  <a href="https://tools.ietf.org/html/rfc6598">RFC 6598, IANA-Reserved IPv4 Prefix for Shared Address Space</a>  </li> <li>  <a href="https://tools.ietf.org/html/rfc5156">RFC 5156, Special-Use IPv6 Addresses</a>  </li> </ul> When the value of <code>Type</code> is <code>CALCULATED</code> or <code>CLOUDWATCH_METRIC</code>, omit <code>IPAddress</code>.<br/>A complex type that identifies the CloudWatch alarm that you want Amazon Route 53 health checkers to use to determine whether the specified health check is healthy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AlarmIdentifier</b>: A complex type that identifies the CloudWatch alarm that you want Amazon Route 53 health checkers to use to determine whether the specified health check is healthy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Dimensions</i> For the metric that the CloudWatch alarm is associated with, a complex type that contains information about the dimensions for the metric. For information, see <a href="http://docs.aws.amazon.com/AmazonCloudWatch/latest/DeveloperGuide/CW_Support_For_AWS.html">Amazon CloudWatch Namespaces, Dimensions, and Metrics Reference</a> in the <i>Amazon CloudWatch User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Statistic</i> For the metric that the CloudWatch alarm is associated with, the statistic that is applied to the metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Namespace</i> The namespace of the metric that the alarm is associated with. For more information, see <a href="http://docs.aws.amazon.com/AmazonCloudWatch/latest/DeveloperGuide/CW_Support_For_AWS.html">Amazon CloudWatch Namespaces, Dimensions, and Metrics Reference</a> in the <i>Amazon CloudWatch User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricName</i> The name of the CloudWatch metric that the alarm is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Period</i> For the metric that the CloudWatch alarm is associated with, the duration of one evaluation period in seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ComparisonOperator</i> For the metric that the CloudWatch alarm is associated with, the arithmetic operation that is used for the comparison.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Threshold</i> For the metric that the CloudWatch alarm is associated with, the value the metric is compared with.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EvaluationPeriods</i> For the metric that the CloudWatch alarm is associated with, the number of periods that the metric is compared to the threshold.<br/>        """
        pass

    def get_health_check_count(self) -> GetHealthCheckCountResponse:
        """Retrieves the number of health checks that are associated with the current AWS account.

        :param self:
        :param 
        :return: HealthCheckCount:  The number of health checks associated with the current AWS account.<br/>        """
        pass

    def get_health_check_last_failure_reason(self, HealthCheckId: str) -> GetHealthCheckLastFailureReasonResponse:
        """Gets the reason that a specified health check failed most recently.

        :param self:
        :param HealthCheckId:  The ID for the health check for which you want the last failure reason. When you created the health check, <code>CreateHealthCheck</code> returned the ID in the response, in the <code>HealthCheckId</code> element. <note> If you want to get the last failure reason for a calculated health check, you must use the Amazon Route 53 console or the CloudWatch console. You can't use <code>GetHealthCheckLastFailureReason</code> for a calculated health check. </note><br/>
        :return: HealthCheckObservations:  A list that contains one <code>Observation</code> element for each Amazon Route 53 health checker that is reporting a last failure reason. <br/>        """
        pass

    def get_health_check_status(self, HealthCheckId: str) -> GetHealthCheckStatusResponse:
        """Gets status of a specified health check. 

        :param self:
        :param HealthCheckId:  The ID for the health check that you want the current status for. When you created the health check, <code>CreateHealthCheck</code> returned the ID in the response, in the <code>HealthCheckId</code> element. <note> If you want to check the status of a calculated health check, you must use the Amazon Route 53 console or the CloudWatch console. You can't use <code>GetHealthCheckStatus</code> to get the status of a calculated health check. </note><br/>
        :return: HealthCheckObservations:  A list that contains one <code>HealthCheckObservation</code> element for each Amazon Route 53 health checker that is reporting a status about the health check endpoint.<br/>        """
        pass

    def get_hosted_zone(self, Id: str) -> GetHostedZoneResponse:
        """Gets information about a specified hosted zone including the four name servers assigned to the hosted zone.

        :param self:
        :param Id:  The ID of the hosted zone that you want to get information about.<br/>
        :return: VPCs:  A complex type that contains information about the VPCs that are associated with the specified hosted zone.<br/>DelegationSet:  A complex type that lists the Amazon Route 53 name servers for the specified hosted zone.:<br/>HostedZone:  A complex type that contains general information about the specified hosted zone.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LinkedService</i> If the hosted zone was created by another service, the service that created the hosted zone. When a hosted zone is created by another service, you can't edit or delete it using Route 53. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LinkedService</b>: If the hosted zone was created by another service, the service that created the hosted zone. When a hosted zone is created by another service, you can't edit or delete it using Route 53. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceRecordSetCount</i> The number of resource record sets in the hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Config</i> A complex type that includes the <code>Comment</code> and <code>PrivateZone</code> elements. If you omitted the <code>HostedZoneConfig</code> and <code>Comment</code> elements from the request, the <code>Config</code> and <code>Comment</code> elements don't appear in the response.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Config</b>: A complex type that includes the <code>Comment</code> and <code>PrivateZone</code> elements. If you omitted the <code>HostedZoneConfig</code> and <code>Comment</code> elements from the request, the <code>Config</code> and <code>Comment</code> elements don't appear in the response.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> The value that you specified for <code>CallerReference</code> when you created the hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the domain. For public hosted zones, this is the name that you have registered with your DNS registrar. For information about how to specify characters other than <code>a-z</code>, <code>0-9</code>, and <code>-</code> (hyphen) and how to specify internationalized domain names, see <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_CreateHostedZone.html">CreateHostedZone</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID that Amazon Route 53 assigned to the hosted zone when you created it.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PrivateZone</i> A value that indicates whether this is a private hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> Any comments that you want to include about the hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> If the health check or hosted zone was created by another service, an optional description that can be provided by the other service. When a resource is created by another service, you can't edit or delete it using Amazon Route 53. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServicePrincipal</i> If the health check or hosted zone was created by another service, the service that created the resource. When a resource is created by another service, you can't edit or delete it using Amazon Route 53. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NameServers</i> A complex type that contains a list of the authoritative name servers for a hosted zone or for a reusable delegation set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> The value that you specified for <code>CallerReference</code> when you created the reusable delegation set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID that Amazon Route 53 assigns to a reusable delegation set.<br/>        """
        pass

    def get_hosted_zone_count(self) -> GetHostedZoneCountResponse:
        """Retrieves the number of hosted zones that are associated with the current AWS account.

        :param self:
        :param 
        :return: HostedZoneCount:  The total number of public and private hosted zones that are associated with the current AWS account.<br/>        """
        pass

    def get_hosted_zone_limit(self, HostedZoneId: str, Type: str) -> GetHostedZoneLimitResponse:
        """Gets the specified limit for a specified hosted zone, for example, the maximum number of records that you can create in the hosted zone.  For the default limit, see <a href="https://docs.aws.amazon.com/Route53/latest/DeveloperGuide/DNSLimitations.html">Limits</a> in the <i>Amazon Route 53 Developer Guide</i>. To request a higher limit, <a href="https://console.aws.amazon.com/support/home#/case/create?issueType=service-limit-increase&amp;limitType=service-code-route53">open a case</a>.

        :param self:
        :param HostedZoneId:  The ID of the hosted zone that you want to get a limit for.<br/>Type:  The limit that you want to get. Valid values include the following: <ul> <li>  <b>MAX_RRSETS_BY_ZONE</b>: The maximum number of records that you can create in the specified hosted zone. </li> <li>  <b>MAX_VPCS_ASSOCIATED_BY_ZONE</b>: The maximum number of Amazon VPCs that you can associate with the specified private hosted zone. </li> </ul><br/>
        :return: Count:  The current number of entities that you have created of the specified type. For example, if you specified <code>MAX_RRSETS_BY_ZONE</code> for the value of <code>Type</code> in the request, the value of <code>Count</code> is the current number of records that you have created in the specified hosted zone.<br/>Limit:  The current setting for the specified limit. For example, if you specified <code>MAX_RRSETS_BY_ZONE</code> for the value of <code>Type</code> in the request, the value of <code>Limit</code> is the maximum number of records that you can create in the specified hosted zone.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The current value for the limit that is specified by <code>Type</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The limit that you requested. Valid values include the following: <ul> <li>  <b>MAX_RRSETS_BY_ZONE</b>: The maximum number of records that you can create in the specified hosted zone. </li> <li>  <b>MAX_VPCS_ASSOCIATED_BY_ZONE</b>: The maximum number of Amazon VPCs that you can associate with the specified private hosted zone. </li> </ul><br/>        """
        pass

    def get_query_logging_config(self, Id: str) -> GetQueryLoggingConfigResponse:
        """Gets information about a specified configuration for DNS query logging. For more information about DNS query logs, see <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_CreateQueryLoggingConfig.html">CreateQueryLoggingConfig</a> and <a href="https://docs.aws.amazon.com/Route53/latest/DeveloperGuide/query-logs.html">Logging DNS Queries</a>.

        :param self:
        :param Id:  The ID of the configuration for DNS query logging that you want to get information about.<br/>
        :return: QueryLoggingConfig:  A complex type that contains information about the query logging configuration that you specified in a <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_GetQueryLoggingConfig.html">GetQueryLoggingConfig</a> request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchLogsLogGroupArn</i> The Amazon Resource Name (ARN) of the CloudWatch Logs log group that Amazon Route 53 is publishing logs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> The ID of the hosted zone that CloudWatch Logs is logging queries for. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID for a configuration for DNS query logging.<br/>        """
        pass

    def get_reusable_delegation_set(self, Id: str) -> GetReusableDelegationSetResponse:
        """Retrieves information about a specified reusable delegation set, including the four name servers that are assigned to the delegation set.

        :param self:
        :param Id:  The ID of the reusable delegation set that you want to get a list of name servers for.<br/>
        :return: DelegationSet:  A complex type that contains information about the reusable delegation set.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NameServers</i> A complex type that contains a list of the authoritative name servers for a hosted zone or for a reusable delegation set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> The value that you specified for <code>CallerReference</code> when you created the reusable delegation set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID that Amazon Route 53 assigns to a reusable delegation set.<br/>        """
        pass

    def get_reusable_delegation_set_limit(self, DelegationSetId: str, Type: str) -> GetReusableDelegationSetLimitResponse:
        """Gets the maximum number of hosted zones that you can associate with the specified reusable delegation set. For the default limit, see <a href="https://docs.aws.amazon.com/Route53/latest/DeveloperGuide/DNSLimitations.html">Limits</a> in the <i>Amazon Route 53 Developer Guide</i>. To request a higher limit, <a href="https://console.aws.amazon.com/support/home#/case/create?issueType=service-limit-increase&amp;limitType=service-code-route53">open a case</a>.

        :param self:
        :param DelegationSetId:  The ID of the delegation set that you want to get the limit for.<br/>Type:  Specify <code>MAX_ZONES_BY_REUSABLE_DELEGATION_SET</code> to get the maximum number of hosted zones that you can associate with the specified reusable delegation set.<br/>
        :return: Count:  The current number of hosted zones that you can associate with the specified reusable delegation set.<br/>Limit:  The current setting for the limit on hosted zones that you can associate with the specified reusable delegation set.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The current value for the <code>MAX_ZONES_BY_REUSABLE_DELEGATION_SET</code> limit.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The limit that you requested: <code>MAX_ZONES_BY_REUSABLE_DELEGATION_SET</code>, the maximum number of hosted zones that you can associate with the specified reusable delegation set.<br/>        """
        pass

    def get_traffic_policy(self, Version: int, Id: str) -> GetTrafficPolicyResponse:
        """Gets information about a specific traffic policy version.

        :param self:
        :param Version:  The version number of the traffic policy that you want to get information about.<br/>Id:  The ID of the traffic policy that you want to get information about.<br/>
        :return: TrafficPolicy:  A complex type that contains settings for the specified traffic policy.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> The comment that you specify in the <code>CreateTrafficPolicy</code> request, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Document</i> The definition of a traffic policy in JSON format. You specify the JSON document to use for a new traffic policy in the <code>CreateTrafficPolicy</code> request. For more information about the JSON format, see <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/api-policies-traffic-policy-document-format.html">Traffic Policy Document Format</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The DNS type of the resource record sets that Amazon Route 53 creates when you use a traffic policy to create a traffic policy instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name that you specified when you created the traffic policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version number that Amazon Route 53 assigns to a traffic policy. For a new traffic policy, the value of <code>Version</code> is always 1.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID that Amazon Route 53 assigned to a traffic policy when you created it.<br/>        """
        pass

    def get_traffic_policy_instance(self, Id: str) -> GetTrafficPolicyInstanceResponse:
        """Gets information about a specified traffic policy instance. <note> After you submit a <code>CreateTrafficPolicyInstance</code> or an <code>UpdateTrafficPolicyInstance</code> request, there's a brief delay while Amazon Route 53 creates the resource record sets that are specified in the traffic policy definition. For more information, see the <code>State</code> response element. </note> <note> In the Route 53 console, traffic policy instances are known as policy records. </note>

        :param self:
        :param Id:  The ID of the traffic policy instance that you want to get information about.<br/>
        :return: TrafficPolicyInstance:  A complex type that contains settings for the traffic policy instance.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrafficPolicyType</i> The DNS type that Amazon Route 53 assigned to all of the resource record sets that it created for this traffic policy instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrafficPolicyVersion</i> The version of the traffic policy that Amazon Route 53 used to create resource record sets in the specified hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrafficPolicyId</i> The ID of the traffic policy that Amazon Route 53 used to create resource record sets in the specified hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> If <code>State</code> is <code>Failed</code>, an explanation of the reason for the failure. If <code>State</code> is another value, <code>Message</code> is empty.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The value of <code>State</code> is one of the following values: <dl> <dt>Applied</dt> <dd> Amazon Route 53 has finished creating resource record sets, and changes have propagated to all Route 53 edge locations. </dd> <dt>Creating</dt> <dd> Route 53 is creating the resource record sets. Use <code>GetTrafficPolicyInstance</code> to confirm that the <code>CreateTrafficPolicyInstance</code> request completed successfully. </dd> <dt>Failed</dt> <dd> Route 53 wasn't able to create or update the resource record sets. When the value of <code>State</code> is <code>Failed</code>, see <code>Message</code> for an explanation of what caused the request to fail. </dd> </dl><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TTL</i> The TTL that Amazon Route 53 assigned to all of the resource record sets that it created in the specified hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The DNS name, such as www.example.com, for which Amazon Route 53 responds to queries by using the resource record sets that are associated with this traffic policy instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> The ID of the hosted zone that Amazon Route 53 created resource record sets in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID that Amazon Route 53 assigned to the new traffic policy instance.<br/>        """
        pass

    def get_traffic_policy_instance_count(self) -> GetTrafficPolicyInstanceCountResponse:
        """Gets the number of traffic policy instances that are associated with the current AWS account.

        :param self:
        :param 
        :return: TrafficPolicyInstanceCount:  The number of traffic policy instances that are associated with the current AWS account.<br/>        """
        pass

    def list_geo_locations(self, StartContinentCode: str=None, StartCountryCode: str=None, StartSubdivisionCode: str=None, MaxItems: str=None) -> ListGeoLocationsResponse:
        """Retrieves a list of supported geographic locations. Countries are listed first, and continents are listed last. If Amazon Route 53 supports subdivisions for a country (for example, states or provinces), the subdivisions for that country are listed in alphabetical order immediately after the corresponding country.

        :param self:
        :param MaxItems:  (Optional) The maximum number of geolocations to be included in the response body for this request. If more than <code>maxitems</code> geolocations remain to be listed, then the value of the <code>IsTruncated</code> element in the response is <code>true</code>.<br/>StartSubdivisionCode:  The code for the subdivision (for example, state or province) with which you want to start listing locations that Amazon Route 53 supports for geolocation. If Route 53 has already returned a page or more of results, if <code>IsTruncated</code> is <code>true</code>, and if <code>NextSubdivisionCode</code> from the previous response has a value, enter that value in <code>startsubdivisioncode</code> to return the next page of results. To list subdivisions of a country, you must include both <code>startcountrycode</code> and <code>startsubdivisioncode</code>.<br/>StartCountryCode:  The code for the country with which you want to start listing locations that Amazon Route 53 supports for geolocation. If Route 53 has already returned a page or more of results, if <code>IsTruncated</code> is <code>true</code>, and if <code>NextCountryCode</code> from the previous response has a value, enter that value in <code>startcountrycode</code> to return the next page of results. Route 53 uses the two-letter country codes that are specified in <a href="https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2">ISO standard 3166-1 alpha-2</a>.<br/>StartContinentCode:  The code for the continent with which you want to start listing locations that Amazon Route 53 supports for geolocation. If Route 53 has already returned a page or more of results, if <code>IsTruncated</code> is true, and if <code>NextContinentCode</code> from the previous response has a value, enter that value in <code>startcontinentcode</code> to return the next page of results. Include <code>startcontinentcode</code> only if you want to list continents. Don't include <code>startcontinentcode</code> when you're listing countries or countries with their subdivisions.<br/>
        :return: MaxItems:  The value that you specified for <code>MaxItems</code> in the request.<br/>NextSubdivisionCode:  If <code>IsTruncated</code> is <code>true</code>, you can make a follow-up request to display more locations. Enter the value of <code>NextSubdivisionCode</code> in the <code>startsubdivisioncode</code> parameter in another <code>ListGeoLocations</code> request.<br/>NextCountryCode:  If <code>IsTruncated</code> is <code>true</code>, you can make a follow-up request to display more locations. Enter the value of <code>NextCountryCode</code> in the <code>startcountrycode</code> parameter in another <code>ListGeoLocations</code> request.<br/>NextContinentCode:  If <code>IsTruncated</code> is <code>true</code>, you can make a follow-up request to display more locations. Enter the value of <code>NextContinentCode</code> in the <code>startcontinentcode</code> parameter in another <code>ListGeoLocations</code> request.<br/>IsTruncated:  A value that indicates whether more locations remain to be listed after the last location in this response. If so, the value of <code>IsTruncated</code> is <code>true</code>. To get more values, submit another request and include the values of <code>NextContinentCode</code>, <code>NextCountryCode</code>, and <code>NextSubdivisionCode</code> in the <code>startcontinentcode</code>, <code>startcountrycode</code>, and <code>startsubdivisioncode</code>, as applicable.<br/>GeoLocationDetailsList:  A complex type that contains one <code>GeoLocationDetails</code> element for each location that Amazon Route 53 supports for geolocation.<br/>        """
        pass

    def list_health_checks(self, Marker: str=None, MaxItems: str=None) -> ListHealthChecksResponse:
        """Retrieve a list of the health checks that are associated with the current AWS account. 

        :param self:
        :param MaxItems:  The maximum number of health checks that you want <code>ListHealthChecks</code> to return in response to the current request. Amazon Route 53 returns a maximum of 100 items. If you set <code>MaxItems</code> to a value greater than 100, Route 53 returns only the first 100 health checks. <br/>Marker:  If the value of <code>IsTruncated</code> in the previous response was <code>true</code>, you have more health checks. To get another group, submit another <code>ListHealthChecks</code> request.  For the value of <code>marker</code>, specify the value of <code>NextMarker</code> from the previous response, which is the ID of the first health check that Amazon Route 53 will return if you submit another request. If the value of <code>IsTruncated</code> in the previous response was <code>false</code>, there are no more health checks to get.<br/>
        :return: MaxItems:  The value that you specified for the <code>maxitems</code> parameter in the call to <code>ListHealthChecks</code> that produced the current response.<br/>NextMarker:  If <code>IsTruncated</code> is <code>true</code>, the value of <code>NextMarker</code> identifies the first health check that Amazon Route 53 returns if you submit another <code>ListHealthChecks</code> request and specify the value of <code>NextMarker</code> in the <code>marker</code> parameter.<br/>IsTruncated:  A flag that indicates whether there are more health checks to be listed. If the response was truncated, you can get the next group of health checks by submitting another <code>ListHealthChecks</code> request and specifying the value of <code>NextMarker</code> in the <code>marker</code> parameter.<br/>Marker:  For the second and subsequent calls to <code>ListHealthChecks</code>, <code>Marker</code> is the value that you specified for the <code>marker</code> parameter in the previous request.<br/>HealthChecks:  A complex type that contains one <code>HealthCheck</code> element for each health check that is associated with the current AWS account.<br/>        """
        pass

    def list_hosted_zones(self, Marker: str=None, MaxItems: str=None, DelegationSetId: str=None) -> ListHostedZonesResponse:
        """Retrieves a list of the public and private hosted zones that are associated with the current AWS account. The response includes a <code>HostedZones</code> child element for each hosted zone. Amazon Route 53 returns a maximum of 100 items in each response. If you have a lot of hosted zones, you can use the <code>maxitems</code> parameter to list them in groups of up to 100.

        :param self:
        :param DelegationSetId:  If you're using reusable delegation sets and you want to list all of the hosted zones that are associated with a reusable delegation set, specify the ID of that reusable delegation set. <br/>MaxItems:  (Optional) The maximum number of hosted zones that you want Amazon Route 53 to return. If you have more than <code>maxitems</code> hosted zones, the value of <code>IsTruncated</code> in the response is <code>true</code>, and the value of <code>NextMarker</code> is the hosted zone ID of the first hosted zone that Route 53 will return if you submit another request.<br/>Marker:  If the value of <code>IsTruncated</code> in the previous response was <code>true</code>, you have more hosted zones. To get more hosted zones, submit another <code>ListHostedZones</code> request.  For the value of <code>marker</code>, specify the value of <code>NextMarker</code> from the previous response, which is the ID of the first hosted zone that Amazon Route 53 will return if you submit another request. If the value of <code>IsTruncated</code> in the previous response was <code>false</code>, there are no more hosted zones to get.<br/>
        :return: MaxItems:  The value that you specified for the <code>maxitems</code> parameter in the call to <code>ListHostedZones</code> that produced the current response.<br/>NextMarker:  If <code>IsTruncated</code> is <code>true</code>, the value of <code>NextMarker</code> identifies the first hosted zone in the next group of hosted zones. Submit another <code>ListHostedZones</code> request, and specify the value of <code>NextMarker</code> from the response in the <code>marker</code> parameter. This element is present only if <code>IsTruncated</code> is <code>true</code>.<br/>IsTruncated:  A flag indicating whether there are more hosted zones to be listed. If the response was truncated, you can get more hosted zones by submitting another <code>ListHostedZones</code> request and specifying the value of <code>NextMarker</code> in the <code>marker</code> parameter.<br/>Marker:  For the second and subsequent calls to <code>ListHostedZones</code>, <code>Marker</code> is the value that you specified for the <code>marker</code> parameter in the request that produced the current response.<br/>HostedZones:  A complex type that contains general information about the hosted zone.<br/>        """
        pass

    def list_hosted_zones_by_name(self, DNSName: str=None, HostedZoneId: str=None, MaxItems: str=None) -> ListHostedZonesByNameResponse:
        """Retrieves a list of your hosted zones in lexicographic order. The response includes a <code>HostedZones</code> child element for each hosted zone created by the current AWS account.   <code>ListHostedZonesByName</code> sorts hosted zones by name with the labels reversed. For example:  <code>com.example.www.</code>  Note the trailing dot, which can change the sort order in some circumstances. If the domain name includes escape characters or Punycode, <code>ListHostedZonesByName</code> alphabetizes the domain name using the escaped or Punycoded value, which is the format that Amazon Route 53 saves in its database. For example, to create a hosted zone for exämple.com, you specify ex\344mple.com for the domain name. <code>ListHostedZonesByName</code> alphabetizes it as:  <code>com.ex\344mple.</code>  The labels are reversed and alphabetized using the escaped value. For more information about valid domain name formats, including internationalized domain names, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/DomainNameFormat.html">DNS Domain Name Format</a> in the <i>Amazon Route 53 Developer Guide</i>. Route 53 returns up to 100 items in each response. If you have a lot of hosted zones, use the <code>MaxItems</code> parameter to list them in groups of up to 100. The response includes values that help navigate from one group of <code>MaxItems</code> hosted zones to the next: <ul> <li> The <code>DNSName</code> and <code>HostedZoneId</code> elements in the response contain the values, if any, specified for the <code>dnsname</code> and <code>hostedzoneid</code> parameters in the request that produced the current response. </li> <li> The <code>MaxItems</code> element in the response contains the value, if any, that you specified for the <code>maxitems</code> parameter in the request that produced the current response. </li> <li> If the value of <code>IsTruncated</code> in the response is true, there are more hosted zones associated with the current AWS account.  If <code>IsTruncated</code> is false, this response includes the last hosted zone that is associated with the current account. The <code>NextDNSName</code> element and <code>NextHostedZoneId</code> elements are omitted from the response. </li> <li> The <code>NextDNSName</code> and <code>NextHostedZoneId</code> elements in the response contain the domain name and the hosted zone ID of the next hosted zone that is associated with the current AWS account. If you want to list more hosted zones, make another call to <code>ListHostedZonesByName</code>, and specify the value of <code>NextDNSName</code> and <code>NextHostedZoneId</code> in the <code>dnsname</code> and <code>hostedzoneid</code> parameters, respectively. </li> </ul>

        :param self:
        :param MaxItems:  The maximum number of hosted zones to be included in the response body for this request. If you have more than <code>maxitems</code> hosted zones, then the value of the <code>IsTruncated</code> element in the response is true, and the values of <code>NextDNSName</code> and <code>NextHostedZoneId</code> specify the first hosted zone in the next group of <code>maxitems</code> hosted zones. <br/>HostedZoneId:  (Optional) For your first request to <code>ListHostedZonesByName</code>, do not include the <code>hostedzoneid</code> parameter. If you have more hosted zones than the value of <code>maxitems</code>, <code>ListHostedZonesByName</code> returns only the first <code>maxitems</code> hosted zones. To get the next group of <code>maxitems</code> hosted zones, submit another request to <code>ListHostedZonesByName</code> and include both <code>dnsname</code> and <code>hostedzoneid</code> parameters. For the value of <code>hostedzoneid</code>, specify the value of the <code>NextHostedZoneId</code> element from the previous response.<br/>DNSName:  (Optional) For your first request to <code>ListHostedZonesByName</code>, include the <code>dnsname</code> parameter only if you want to specify the name of the first hosted zone in the response. If you don't include the <code>dnsname</code> parameter, Amazon Route 53 returns all of the hosted zones that were created by the current AWS account, in ASCII order. For subsequent requests, include both <code>dnsname</code> and <code>hostedzoneid</code> parameters. For <code>dnsname</code>, specify the value of <code>NextDNSName</code> from the previous response.<br/>
        :return: MaxItems:  The value that you specified for the <code>maxitems</code> parameter in the call to <code>ListHostedZonesByName</code> that produced the current response.<br/>NextHostedZoneId:  If <code>IsTruncated</code> is <code>true</code>, the value of <code>NextHostedZoneId</code> identifies the first hosted zone in the next group of <code>maxitems</code> hosted zones. Call <code>ListHostedZonesByName</code> again and specify the value of <code>NextDNSName</code> and <code>NextHostedZoneId</code> in the <code>dnsname</code> and <code>hostedzoneid</code> parameters, respectively. This element is present only if <code>IsTruncated</code> is <code>true</code>.<br/>NextDNSName:  If <code>IsTruncated</code> is true, the value of <code>NextDNSName</code> is the name of the first hosted zone in the next group of <code>maxitems</code> hosted zones. Call <code>ListHostedZonesByName</code> again and specify the value of <code>NextDNSName</code> and <code>NextHostedZoneId</code> in the <code>dnsname</code> and <code>hostedzoneid</code> parameters, respectively. This element is present only if <code>IsTruncated</code> is <code>true</code>.<br/>IsTruncated:  A flag that indicates whether there are more hosted zones to be listed. If the response was truncated, you can get the next group of <code>maxitems</code> hosted zones by calling <code>ListHostedZonesByName</code> again and specifying the values of <code>NextDNSName</code> and <code>NextHostedZoneId</code> elements in the <code>dnsname</code> and <code>hostedzoneid</code> parameters.<br/>HostedZoneId:  The ID that Amazon Route 53 assigned to the hosted zone when you created it.<br/>DNSName:  For the second and subsequent calls to <code>ListHostedZonesByName</code>, <code>DNSName</code> is the value that you specified for the <code>dnsname</code> parameter in the request that produced the current response.<br/>HostedZones:  A complex type that contains general information about the hosted zone.<br/>        """
        pass

    def list_query_logging_configs(self, HostedZoneId: str=None, NextToken: str=None, MaxResults: str=None) -> ListQueryLoggingConfigsResponse:
        """Lists the configurations for DNS query logging that are associated with the current AWS account or the configuration that is associated with a specified hosted zone. For more information about DNS query logs, see <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_CreateQueryLoggingConfig.html">CreateQueryLoggingConfig</a>. Additional information, including the format of DNS query logs, appears in <a href="https://docs.aws.amazon.com/Route53/latest/DeveloperGuide/query-logs.html">Logging DNS Queries</a> in the <i>Amazon Route 53 Developer Guide</i>.

        :param self:
        :param MaxResults:  (Optional) The maximum number of query logging configurations that you want Amazon Route 53 to return in response to the current request. If the current AWS account has more than <code>MaxResults</code> configurations, use the value of <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_ListQueryLoggingConfigs.html#API_ListQueryLoggingConfigs_RequestSyntax">NextToken</a> in the response to get the next page of results. If you don't specify a value for <code>MaxResults</code>, Route 53 returns up to 100 configurations.<br/>NextToken:  (Optional) If the current AWS account has more than <code>MaxResults</code> query logging configurations, use <code>NextToken</code> to get the second and subsequent pages of results. For the first <code>ListQueryLoggingConfigs</code> request, omit this value. For the second and subsequent requests, get the value of <code>NextToken</code> from the previous response and specify that value for <code>NextToken</code> in the request.<br/>HostedZoneId:  (Optional) If you want to list the query logging configuration that is associated with a hosted zone, specify the ID in <code>HostedZoneId</code>.  If you don't specify a hosted zone ID, <code>ListQueryLoggingConfigs</code> returns all of the configurations that are associated with the current AWS account.<br/>
        :return: NextToken:  If a response includes the last of the query logging configurations that are associated with the current AWS account, <code>NextToken</code> doesn't appear in the response. If a response doesn't include the last of the configurations, you can get more configurations by submitting another <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_ListQueryLoggingConfigs.html">ListQueryLoggingConfigs</a> request. Get the value of <code>NextToken</code> that Amazon Route 53 returned in the previous response and include it in <code>NextToken</code> in the next request.<br/>QueryLoggingConfigs:  An array that contains one <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_QueryLoggingConfig.html">QueryLoggingConfig</a> element for each configuration for DNS query logging that is associated with the current AWS account.<br/>        """
        pass

    def list_resource_record_sets(self, HostedZoneId: str, StartRecordName: str=None, StartRecordType: str=None, StartRecordIdentifier: str=None, MaxItems: str=None) -> ListResourceRecordSetsResponse:
        """Lists the resource record sets in a specified hosted zone.  <code>ListResourceRecordSets</code> returns up to 100 resource record sets at a time in ASCII order, beginning at a position specified by the <code>name</code> and <code>type</code> elements.  <b>Sort order</b>   <code>ListResourceRecordSets</code> sorts results first by DNS name with the labels reversed, for example:  <code>com.example.www.</code>  Note the trailing dot, which can change the sort order when the record name contains characters that appear before <code>.</code> (decimal 46) in the ASCII table. These characters include the following: <code>! " # $ % &amp; ' ( ) * + , -</code>  When multiple records have the same DNS name, <code>ListResourceRecordSets</code> sorts results by the record type.  <b>Specifying where to start listing records</b>  You can use the name and type elements to specify the resource record set that the list begins with: <dl> <dt>If you do not specify Name or Type</dt> <dd> The results begin with the first resource record set that the hosted zone contains. </dd> <dt>If you specify Name but not Type</dt> <dd> The results begin with the first resource record set in the list whose name is greater than or equal to <code>Name</code>. </dd> <dt>If you specify Type but not Name</dt> <dd> Amazon Route 53 returns the <code>InvalidInput</code> error. </dd> <dt>If you specify both Name and Type</dt> <dd> The results begin with the first resource record set in the list whose name is greater than or equal to <code>Name</code>, and whose type is greater than or equal to <code>Type</code>. </dd> </dl>  <b>Resource record sets that are PENDING</b>  This action returns the most current version of the records. This includes records that are <code>PENDING</code>, and that are not yet available on all Route 53 DNS servers.  <b>Changing resource record sets</b>  To ensure that you get an accurate listing of the resource record sets for a hosted zone at a point in time, do not submit a <code>ChangeResourceRecordSets</code> request while you're paging through the results of a <code>ListResourceRecordSets</code> request. If you do, some pages may display results without the latest changes while other pages display results with the latest changes.  <b>Displaying the next page of results</b>  If a <code>ListResourceRecordSets</code> command returns more than one page of results, the value of <code>IsTruncated</code> is <code>true</code>. To display the next page of results, get the values of <code>NextRecordName</code>, <code>NextRecordType</code>, and <code>NextRecordIdentifier</code> (if any) from the response. Then submit another <code>ListResourceRecordSets</code> request, and specify those values for <code>StartRecordName</code>, <code>StartRecordType</code>, and <code>StartRecordIdentifier</code>.

        :param self:
        :param MaxItems:  (Optional) The maximum number of resource records sets to include in the response body for this request. If the response includes more than <code>maxitems</code> resource record sets, the value of the <code>IsTruncated</code> element in the response is <code>true</code>, and the values of the <code>NextRecordName</code> and <code>NextRecordType</code> elements in the response identify the first resource record set in the next group of <code>maxitems</code> resource record sets.<br/>StartRecordIdentifier:   <i>Weighted resource record sets only:</i> If results were truncated for a given DNS name and type, specify the value of <code>NextRecordIdentifier</code> from the previous response to get the next resource record set that has the current DNS name and type.<br/>StartRecordType:  The type of resource record set to begin the record listing from. Valid values for basic resource record sets: <code>A</code> | <code>AAAA</code> | <code>CAA</code> | <code>CNAME</code> | <code>MX</code> | <code>NAPTR</code> | <code>NS</code> | <code>PTR</code> | <code>SOA</code> | <code>SPF</code> | <code>SRV</code> | <code>TXT</code>  Values for weighted, latency, geolocation, and failover resource record sets: <code>A</code> | <code>AAAA</code> | <code>CAA</code> | <code>CNAME</code> | <code>MX</code> | <code>NAPTR</code> | <code>PTR</code> | <code>SPF</code> | <code>SRV</code> | <code>TXT</code>  Values for alias resource record sets:  <ul> <li>  <b>API Gateway custom regional API or edge-optimized API</b>: A </li> <li>  <b>CloudFront distribution</b>: A or AAAA </li> <li>  <b>Elastic Beanstalk environment that has a regionalized subdomain</b>: A </li> <li>  <b>Elastic Load Balancing load balancer</b>: A | AAAA </li> <li>  <b>Amazon S3 bucket</b>: A </li> <li>  <b>Amazon VPC interface VPC endpoint</b>: A </li> <li>  <b>Another resource record set in this hosted zone:</b> The type of the resource record set that the alias references. </li> </ul> Constraint: Specifying <code>type</code> without specifying <code>name</code> returns an <code>InvalidInput</code> error.<br/>StartRecordName:  The first name in the lexicographic ordering of resource record sets that you want to list.<br/>HostedZoneId:  The ID of the hosted zone that contains the resource record sets that you want to list.<br/>
        :return: MaxItems:  The maximum number of records you requested.<br/>NextRecordIdentifier:   <i>Resource record sets that have a routing policy other than simple:</i> If results were truncated for a given DNS name and type, the value of <code>SetIdentifier</code> for the next resource record set that has the current DNS name and type. For information about routing policies, see <a href="https://docs.aws.amazon.com/Route53/latest/DeveloperGuide/routing-policy.html">Choosing a Routing Policy</a> in the <i>Amazon Route 53 Developer Guide</i>.<br/>NextRecordType:  If the results were truncated, the type of the next record in the list. This element is present only if <code>IsTruncated</code> is true. <br/>NextRecordName:  If the results were truncated, the name of the next record in the list. This element is present only if <code>IsTruncated</code> is true. <br/>IsTruncated:  A flag that indicates whether more resource record sets remain to be listed. If your results were truncated, you can make a follow-up pagination request by using the <code>NextRecordName</code> element.<br/>ResourceRecordSets:  Information about multiple resource record sets.<br/>        """
        pass

    def list_reusable_delegation_sets(self, Marker: str=None, MaxItems: str=None) -> ListReusableDelegationSetsResponse:
        """Retrieves a list of the reusable delegation sets that are associated with the current AWS account.

        :param self:
        :param MaxItems:  The number of reusable delegation sets that you want Amazon Route 53 to return in the response to this request. If you specify a value greater than 100, Route 53 returns only the first 100 reusable delegation sets.<br/>Marker:  If the value of <code>IsTruncated</code> in the previous response was <code>true</code>, you have more reusable delegation sets. To get another group, submit another <code>ListReusableDelegationSets</code> request.  For the value of <code>marker</code>, specify the value of <code>NextMarker</code> from the previous response, which is the ID of the first reusable delegation set that Amazon Route 53 will return if you submit another request. If the value of <code>IsTruncated</code> in the previous response was <code>false</code>, there are no more reusable delegation sets to get.<br/>
        :return: MaxItems:  The value that you specified for the <code>maxitems</code> parameter in the call to <code>ListReusableDelegationSets</code> that produced the current response.<br/>NextMarker:  If <code>IsTruncated</code> is <code>true</code>, the value of <code>NextMarker</code> identifies the next reusable delegation set that Amazon Route 53 will return if you submit another <code>ListReusableDelegationSets</code> request and specify the value of <code>NextMarker</code> in the <code>marker</code> parameter.<br/>IsTruncated:  A flag that indicates whether there are more reusable delegation sets to be listed.<br/>Marker:  For the second and subsequent calls to <code>ListReusableDelegationSets</code>, <code>Marker</code> is the value that you specified for the <code>marker</code> parameter in the request that produced the current response.<br/>DelegationSets:  A complex type that contains one <code>DelegationSet</code> element for each reusable delegation set that was created by the current AWS account.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceId: str, ResourceType: str) -> ListTagsForResourceResponse:
        """Lists tags for one health check or hosted zone.  For information about using tags for cost allocation, see <a href="https://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/cost-alloc-tags.html">Using Cost Allocation Tags</a> in the <i>AWS Billing and Cost Management User Guide</i>.

        :param self:
        :param ResourceId:  The ID of the resource for which you want to retrieve tags.<br/>ResourceType:  The type of the resource. <ul> <li> The resource type for health checks is <code>healthcheck</code>. </li> <li> The resource type for hosted zones is <code>hostedzone</code>. </li> </ul><br/>
        :return: ResourceTagSet:  A <code>ResourceTagSet</code> containing tags associated with the specified resource.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The tags associated with the specified resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceId</i> The ID for the specified resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceType</i> The type of the resource. <ul> <li> The resource type for health checks is <code>healthcheck</code>. </li> <li> The resource type for hosted zones is <code>hostedzone</code>. </li> </ul><br/>        """
        pass

    def list_tags_for_resources(self, ResourceIds: List[TagResourceId], ResourceType: str) -> ListTagsForResourcesResponse:
        """Lists tags for up to 10 health checks or hosted zones. For information about using tags for cost allocation, see <a href="https://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/cost-alloc-tags.html">Using Cost Allocation Tags</a> in the <i>AWS Billing and Cost Management User Guide</i>.

        :param self:
        :param ResourceIds:  A complex type that contains the ResourceId element for each resource for which you want to get a list of tags.<br/>ResourceType:  The type of the resources. <ul> <li> The resource type for health checks is <code>healthcheck</code>. </li> <li> The resource type for hosted zones is <code>hostedzone</code>. </li> </ul><br/>
        :return: ResourceTagSets:  A list of <code>ResourceTagSet</code>s containing tags associated with the specified resources.<br/>        """
        pass

    def list_traffic_policies(self, TrafficPolicyIdMarker: str=None, MaxItems: str=None) -> ListTrafficPoliciesResponse:
        """Gets information about the latest version for every traffic policy that is associated with the current AWS account. Policies are listed in the order that they were created in. 

        :param self:
        :param MaxItems:  (Optional) The maximum number of traffic policies that you want Amazon Route 53 to return in response to this request. If you have more than <code>MaxItems</code> traffic policies, the value of <code>IsTruncated</code> in the response is <code>true</code>, and the value of <code>TrafficPolicyIdMarker</code> is the ID of the first traffic policy that Route 53 will return if you submit another request.<br/>TrafficPolicyIdMarker:  (Conditional) For your first request to <code>ListTrafficPolicies</code>, don't include the <code>TrafficPolicyIdMarker</code> parameter. If you have more traffic policies than the value of <code>MaxItems</code>, <code>ListTrafficPolicies</code> returns only the first <code>MaxItems</code> traffic policies. To get the next group of policies, submit another request to <code>ListTrafficPolicies</code>. For the value of <code>TrafficPolicyIdMarker</code>, specify the value of <code>TrafficPolicyIdMarker</code> that was returned in the previous response.<br/>
        :return: MaxItems:  The value that you specified for the <code>MaxItems</code> parameter in the <code>ListTrafficPolicies</code> request that produced the current response.<br/>TrafficPolicyIdMarker:  If the value of <code>IsTruncated</code> is <code>true</code>, <code>TrafficPolicyIdMarker</code> is the ID of the first traffic policy in the next group of <code>MaxItems</code> traffic policies.<br/>IsTruncated:  A flag that indicates whether there are more traffic policies to be listed. If the response was truncated, you can get the next group of traffic policies by submitting another <code>ListTrafficPolicies</code> request and specifying the value of <code>TrafficPolicyIdMarker</code> in the <code>TrafficPolicyIdMarker</code> request parameter.<br/>TrafficPolicySummaries:  A list that contains one <code>TrafficPolicySummary</code> element for each traffic policy that was created by the current AWS account.<br/>        """
        pass

    def list_traffic_policy_instances(self, HostedZoneIdMarker: str=None, TrafficPolicyInstanceNameMarker: str=None, TrafficPolicyInstanceTypeMarker: str=None, MaxItems: str=None) -> ListTrafficPolicyInstancesResponse:
        """Gets information about the traffic policy instances that you created by using the current AWS account. <note> After you submit an <code>UpdateTrafficPolicyInstance</code> request, there's a brief delay while Amazon Route 53 creates the resource record sets that are specified in the traffic policy definition. For more information, see the <code>State</code> response element. </note> Route 53 returns a maximum of 100 items in each response. If you have a lot of traffic policy instances, you can use the <code>MaxItems</code> parameter to list them in groups of up to 100.

        :param self:
        :param MaxItems:  The maximum number of traffic policy instances that you want Amazon Route 53 to return in response to a <code>ListTrafficPolicyInstances</code> request. If you have more than <code>MaxItems</code> traffic policy instances, the value of the <code>IsTruncated</code> element in the response is <code>true</code>, and the values of <code>HostedZoneIdMarker</code>, <code>TrafficPolicyInstanceNameMarker</code>, and <code>TrafficPolicyInstanceTypeMarker</code> represent the first traffic policy instance in the next group of <code>MaxItems</code> traffic policy instances.<br/>TrafficPolicyInstanceTypeMarker:  If the value of <code>IsTruncated</code> in the previous response was <code>true</code>, you have more traffic policy instances. To get more traffic policy instances, submit another <code>ListTrafficPolicyInstances</code> request. For the value of <code>trafficpolicyinstancetype</code>, specify the value of <code>TrafficPolicyInstanceTypeMarker</code> from the previous response, which is the type of the first traffic policy instance in the next group of traffic policy instances. If the value of <code>IsTruncated</code> in the previous response was <code>false</code>, there are no more traffic policy instances to get.<br/>TrafficPolicyInstanceNameMarker:  If the value of <code>IsTruncated</code> in the previous response was <code>true</code>, you have more traffic policy instances. To get more traffic policy instances, submit another <code>ListTrafficPolicyInstances</code> request. For the value of <code>trafficpolicyinstancename</code>, specify the value of <code>TrafficPolicyInstanceNameMarker</code> from the previous response, which is the name of the first traffic policy instance in the next group of traffic policy instances. If the value of <code>IsTruncated</code> in the previous response was <code>false</code>, there are no more traffic policy instances to get.<br/>HostedZoneIdMarker:  If the value of <code>IsTruncated</code> in the previous response was <code>true</code>, you have more traffic policy instances. To get more traffic policy instances, submit another <code>ListTrafficPolicyInstances</code> request. For the value of <code>HostedZoneId</code>, specify the value of <code>HostedZoneIdMarker</code> from the previous response, which is the hosted zone ID of the first traffic policy instance in the next group of traffic policy instances. If the value of <code>IsTruncated</code> in the previous response was <code>false</code>, there are no more traffic policy instances to get.<br/>
        :return: MaxItems:  The value that you specified for the <code>MaxItems</code> parameter in the call to <code>ListTrafficPolicyInstances</code> that produced the current response.<br/>IsTruncated:  A flag that indicates whether there are more traffic policy instances to be listed. If the response was truncated, you can get more traffic policy instances by calling <code>ListTrafficPolicyInstances</code> again and specifying the values of the <code>HostedZoneIdMarker</code>, <code>TrafficPolicyInstanceNameMarker</code>, and <code>TrafficPolicyInstanceTypeMarker</code> in the corresponding request parameters.<br/>TrafficPolicyInstanceTypeMarker:  If <code>IsTruncated</code> is <code>true</code>, <code>TrafficPolicyInstanceTypeMarker</code> is the DNS type of the resource record sets that are associated with the first traffic policy instance that Amazon Route 53 will return if you submit another <code>ListTrafficPolicyInstances</code> request. <br/>TrafficPolicyInstanceNameMarker:  If <code>IsTruncated</code> is <code>true</code>, <code>TrafficPolicyInstanceNameMarker</code> is the name of the first traffic policy instance that Route 53 will return if you submit another <code>ListTrafficPolicyInstances</code> request. <br/>HostedZoneIdMarker:  If <code>IsTruncated</code> is <code>true</code>, <code>HostedZoneIdMarker</code> is the ID of the hosted zone of the first traffic policy instance that Route 53 will return if you submit another <code>ListTrafficPolicyInstances</code> request. <br/>TrafficPolicyInstances:  A list that contains one <code>TrafficPolicyInstance</code> element for each traffic policy instance that matches the elements in the request.<br/>        """
        pass

    def list_traffic_policy_instances_by_hosted_zone(self, HostedZoneId: str, TrafficPolicyInstanceNameMarker: str=None, TrafficPolicyInstanceTypeMarker: str=None, MaxItems: str=None) -> ListTrafficPolicyInstancesByHostedZoneResponse:
        """Gets information about the traffic policy instances that you created in a specified hosted zone. <note> After you submit a <code>CreateTrafficPolicyInstance</code> or an <code>UpdateTrafficPolicyInstance</code> request, there's a brief delay while Amazon Route 53 creates the resource record sets that are specified in the traffic policy definition. For more information, see the <code>State</code> response element. </note> Route 53 returns a maximum of 100 items in each response. If you have a lot of traffic policy instances, you can use the <code>MaxItems</code> parameter to list them in groups of up to 100.

        :param self:
        :param MaxItems:  The maximum number of traffic policy instances to be included in the response body for this request. If you have more than <code>MaxItems</code> traffic policy instances, the value of the <code>IsTruncated</code> element in the response is <code>true</code>, and the values of <code>HostedZoneIdMarker</code>, <code>TrafficPolicyInstanceNameMarker</code>, and <code>TrafficPolicyInstanceTypeMarker</code> represent the first traffic policy instance that Amazon Route 53 will return if you submit another request.<br/>TrafficPolicyInstanceTypeMarker:  If the value of <code>IsTruncated</code> in the previous response is true, you have more traffic policy instances. To get more traffic policy instances, submit another <code>ListTrafficPolicyInstances</code> request. For the value of <code>trafficpolicyinstancetype</code>, specify the value of <code>TrafficPolicyInstanceTypeMarker</code> from the previous response, which is the type of the first traffic policy instance in the next group of traffic policy instances. If the value of <code>IsTruncated</code> in the previous response was <code>false</code>, there are no more traffic policy instances to get.<br/>TrafficPolicyInstanceNameMarker:  If the value of <code>IsTruncated</code> in the previous response is true, you have more traffic policy instances. To get more traffic policy instances, submit another <code>ListTrafficPolicyInstances</code> request. For the value of <code>trafficpolicyinstancename</code>, specify the value of <code>TrafficPolicyInstanceNameMarker</code> from the previous response, which is the name of the first traffic policy instance in the next group of traffic policy instances. If the value of <code>IsTruncated</code> in the previous response was <code>false</code>, there are no more traffic policy instances to get.<br/>HostedZoneId:  The ID of the hosted zone that you want to list traffic policy instances for.<br/>
        :return: MaxItems:  The value that you specified for the <code>MaxItems</code> parameter in the <code>ListTrafficPolicyInstancesByHostedZone</code> request that produced the current response.<br/>IsTruncated:  A flag that indicates whether there are more traffic policy instances to be listed. If the response was truncated, you can get the next group of traffic policy instances by submitting another <code>ListTrafficPolicyInstancesByHostedZone</code> request and specifying the values of <code>HostedZoneIdMarker</code>, <code>TrafficPolicyInstanceNameMarker</code>, and <code>TrafficPolicyInstanceTypeMarker</code> in the corresponding request parameters.<br/>TrafficPolicyInstanceTypeMarker:  If <code>IsTruncated</code> is true, <code>TrafficPolicyInstanceTypeMarker</code> is the DNS type of the resource record sets that are associated with the first traffic policy instance in the next group of traffic policy instances.<br/>TrafficPolicyInstanceNameMarker:  If <code>IsTruncated</code> is <code>true</code>, <code>TrafficPolicyInstanceNameMarker</code> is the name of the first traffic policy instance in the next group of traffic policy instances.<br/>TrafficPolicyInstances:  A list that contains one <code>TrafficPolicyInstance</code> element for each traffic policy instance that matches the elements in the request. <br/>        """
        pass

    def list_traffic_policy_instances_by_policy(self, TrafficPolicyVersion: int, TrafficPolicyId: str, HostedZoneIdMarker: str=None, TrafficPolicyInstanceNameMarker: str=None, TrafficPolicyInstanceTypeMarker: str=None, MaxItems: str=None) -> ListTrafficPolicyInstancesByPolicyResponse:
        """Gets information about the traffic policy instances that you created by using a specify traffic policy version. <note> After you submit a <code>CreateTrafficPolicyInstance</code> or an <code>UpdateTrafficPolicyInstance</code> request, there's a brief delay while Amazon Route 53 creates the resource record sets that are specified in the traffic policy definition. For more information, see the <code>State</code> response element. </note> Route 53 returns a maximum of 100 items in each response. If you have a lot of traffic policy instances, you can use the <code>MaxItems</code> parameter to list them in groups of up to 100.

        :param self:
        :param MaxItems:  The maximum number of traffic policy instances to be included in the response body for this request. If you have more than <code>MaxItems</code> traffic policy instances, the value of the <code>IsTruncated</code> element in the response is <code>true</code>, and the values of <code>HostedZoneIdMarker</code>, <code>TrafficPolicyInstanceNameMarker</code>, and <code>TrafficPolicyInstanceTypeMarker</code> represent the first traffic policy instance that Amazon Route 53 will return if you submit another request.<br/>TrafficPolicyInstanceTypeMarker:  If the value of <code>IsTruncated</code> in the previous response was <code>true</code>, you have more traffic policy instances. To get more traffic policy instances, submit another <code>ListTrafficPolicyInstancesByPolicy</code> request. For the value of <code>trafficpolicyinstancetype</code>, specify the value of <code>TrafficPolicyInstanceTypeMarker</code> from the previous response, which is the name of the first traffic policy instance that Amazon Route 53 will return if you submit another request. If the value of <code>IsTruncated</code> in the previous response was <code>false</code>, there are no more traffic policy instances to get.<br/>TrafficPolicyInstanceNameMarker:  If the value of <code>IsTruncated</code> in the previous response was <code>true</code>, you have more traffic policy instances. To get more traffic policy instances, submit another <code>ListTrafficPolicyInstancesByPolicy</code> request. For the value of <code>trafficpolicyinstancename</code>, specify the value of <code>TrafficPolicyInstanceNameMarker</code> from the previous response, which is the name of the first traffic policy instance that Amazon Route 53 will return if you submit another request. If the value of <code>IsTruncated</code> in the previous response was <code>false</code>, there are no more traffic policy instances to get.<br/>HostedZoneIdMarker:  If the value of <code>IsTruncated</code> in the previous response was <code>true</code>, you have more traffic policy instances. To get more traffic policy instances, submit another <code>ListTrafficPolicyInstancesByPolicy</code> request.  For the value of <code>hostedzoneid</code>, specify the value of <code>HostedZoneIdMarker</code> from the previous response, which is the hosted zone ID of the first traffic policy instance that Amazon Route 53 will return if you submit another request. If the value of <code>IsTruncated</code> in the previous response was <code>false</code>, there are no more traffic policy instances to get.<br/>TrafficPolicyVersion:  The version of the traffic policy for which you want to list traffic policy instances. The version must be associated with the traffic policy that is specified by <code>TrafficPolicyId</code>.<br/>TrafficPolicyId:  The ID of the traffic policy for which you want to list traffic policy instances.<br/>
        :return: MaxItems:  The value that you specified for the <code>MaxItems</code> parameter in the call to <code>ListTrafficPolicyInstancesByPolicy</code> that produced the current response.<br/>IsTruncated:  A flag that indicates whether there are more traffic policy instances to be listed. If the response was truncated, you can get the next group of traffic policy instances by calling <code>ListTrafficPolicyInstancesByPolicy</code> again and specifying the values of the <code>HostedZoneIdMarker</code>, <code>TrafficPolicyInstanceNameMarker</code>, and <code>TrafficPolicyInstanceTypeMarker</code> elements in the corresponding request parameters.<br/>TrafficPolicyInstanceTypeMarker:  If <code>IsTruncated</code> is <code>true</code>, <code>TrafficPolicyInstanceTypeMarker</code> is the DNS type of the resource record sets that are associated with the first traffic policy instance in the next group of <code>MaxItems</code> traffic policy instances.<br/>TrafficPolicyInstanceNameMarker:  If <code>IsTruncated</code> is <code>true</code>, <code>TrafficPolicyInstanceNameMarker</code> is the name of the first traffic policy instance in the next group of <code>MaxItems</code> traffic policy instances.<br/>HostedZoneIdMarker:  If <code>IsTruncated</code> is <code>true</code>, <code>HostedZoneIdMarker</code> is the ID of the hosted zone of the first traffic policy instance in the next group of traffic policy instances.<br/>TrafficPolicyInstances:  A list that contains one <code>TrafficPolicyInstance</code> element for each traffic policy instance that matches the elements in the request.<br/>        """
        pass

    def list_traffic_policy_versions(self, Id: str, TrafficPolicyVersionMarker: str=None, MaxItems: str=None) -> ListTrafficPolicyVersionsResponse:
        """Gets information about all of the versions for a specified traffic policy. Traffic policy versions are listed in numerical order by <code>VersionNumber</code>.

        :param self:
        :param MaxItems:  The maximum number of traffic policy versions that you want Amazon Route 53 to include in the response body for this request. If the specified traffic policy has more than <code>MaxItems</code> versions, the value of <code>IsTruncated</code> in the response is <code>true</code>, and the value of the <code>TrafficPolicyVersionMarker</code> element is the ID of the first version that Route 53 will return if you submit another request.<br/>TrafficPolicyVersionMarker:  For your first request to <code>ListTrafficPolicyVersions</code>, don't include the <code>TrafficPolicyVersionMarker</code> parameter. If you have more traffic policy versions than the value of <code>MaxItems</code>, <code>ListTrafficPolicyVersions</code> returns only the first group of <code>MaxItems</code> versions. To get more traffic policy versions, submit another <code>ListTrafficPolicyVersions</code> request. For the value of <code>TrafficPolicyVersionMarker</code>, specify the value of <code>TrafficPolicyVersionMarker</code> in the previous response.<br/>Id:  Specify the value of <code>Id</code> of the traffic policy for which you want to list all versions.<br/>
        :return: MaxItems:  The value that you specified for the <code>maxitems</code> parameter in the <code>ListTrafficPolicyVersions</code> request that produced the current response.<br/>TrafficPolicyVersionMarker:  If <code>IsTruncated</code> is <code>true</code>, the value of <code>TrafficPolicyVersionMarker</code> identifies the first traffic policy that Amazon Route 53 will return if you submit another request. Call <code>ListTrafficPolicyVersions</code> again and specify the value of <code>TrafficPolicyVersionMarker</code> in the <code>TrafficPolicyVersionMarker</code> request parameter. This element is present only if <code>IsTruncated</code> is <code>true</code>.<br/>IsTruncated:  A flag that indicates whether there are more traffic policies to be listed. If the response was truncated, you can get the next group of traffic policies by submitting another <code>ListTrafficPolicyVersions</code> request and specifying the value of <code>NextMarker</code> in the <code>marker</code> parameter.<br/>TrafficPolicies:  A list that contains one <code>TrafficPolicy</code> element for each traffic policy version that is associated with the specified traffic policy.<br/>        """
        pass

    def list_vpc_association_authorizations(self, HostedZoneId: str, NextToken: str=None, MaxResults: str=None) -> ListVPCAssociationAuthorizationsResponse:
        """Gets a list of the VPCs that were created by other accounts and that can be associated with a specified hosted zone because you've submitted one or more <code>CreateVPCAssociationAuthorization</code> requests.  The response includes a <code>VPCs</code> element with a <code>VPC</code> child element for each VPC that can be associated with the hosted zone.

        :param self:
        :param MaxResults:   <i>Optional</i>: An integer that specifies the maximum number of VPCs that you want Amazon Route 53 to return. If you don't specify a value for <code>MaxResults</code>, Route 53 returns up to 50 VPCs per page.<br/>NextToken:   <i>Optional</i>: If a response includes a <code>NextToken</code> element, there are more VPCs that can be associated with the specified hosted zone. To get the next page of results, submit another request, and include the value of <code>NextToken</code> from the response in the <code>nexttoken</code> parameter in another <code>ListVPCAssociationAuthorizations</code> request.<br/>HostedZoneId:  The ID of the hosted zone for which you want a list of VPCs that can be associated with the hosted zone.<br/>
        :return: VPCs:  The list of VPCs that are authorized to be associated with the specified hosted zone.<br/>NextToken:  When the response includes a <code>NextToken</code> element, there are more VPCs that can be associated with the specified hosted zone. To get the next page of VPCs, submit another <code>ListVPCAssociationAuthorizations</code> request, and include the value of the <code>NextToken</code> element from the response in the <code>nexttoken</code> request parameter.<br/>HostedZoneId:  The ID of the hosted zone that you can associate the listed VPCs with.<br/>        """
        pass

    def test_dns_answer(self, RecordType: str, RecordName: str, HostedZoneId: str, ResolverIP: str=None, EDNS0ClientSubnetIP: str=None, EDNS0ClientSubnetMask: str=None) -> TestDNSAnswerResponse:
        """Gets the value that Amazon Route 53 returns in response to a DNS request for a specified record name and type. You can optionally specify the IP address of a DNS resolver, an EDNS0 client subnet IP address, and a subnet mask. 

        :param self:
        :param EDNS0ClientSubnetMask:  If you specify an IP address for <code>edns0clientsubnetip</code>, you can optionally specify the number of bits of the IP address that you want the checking tool to include in the DNS query. For example, if you specify <code>192.0.2.44</code> for <code>edns0clientsubnetip</code> and <code>24</code> for <code>edns0clientsubnetmask</code>, the checking tool will simulate a request from 192.0.2.0/24. The default value is 24 bits for IPv4 addresses and 64 bits for IPv6 addresses. The range of valid values depends on whether <code>edns0clientsubnetip</code> is an IPv4 or an IPv6 address: <ul> <li>  <b>IPv4</b>: Specify a value between 0 and 32 </li> <li>  <b>IPv6</b>: Specify a value between 0 and 128 </li> </ul><br/>EDNS0ClientSubnetIP:  If the resolver that you specified for resolverip supports EDNS0, specify the IPv4 or IPv6 address of a client in the applicable location, for example, <code>192.0.2.44</code> or <code>2001:db8:85a3::8a2e:370:7334</code>.<br/>ResolverIP:  If you want to simulate a request from a specific DNS resolver, specify the IP address for that resolver. If you omit this value, <code>TestDnsAnswer</code> uses the IP address of a DNS resolver in the AWS US East (N. Virginia) Region (<code>us-east-1</code>).<br/>RecordType:  The type of the resource record set.<br/>RecordName:  The name of the resource record set that you want Amazon Route 53 to simulate a query for.<br/>HostedZoneId:  The ID of the hosted zone that you want Amazon Route 53 to simulate a query for.<br/>
        :return: Protocol:  The protocol that Amazon Route 53 used to respond to the request, either <code>UDP</code> or <code>TCP</code>. <br/>ResponseCode:  A code that indicates whether the request is valid or not. The most common response code is <code>NOERROR</code>, meaning that the request is valid. If the response is not valid, Amazon Route 53 returns a response code that describes the error. For a list of possible response codes, see <a href="http://www.iana.org/assignments/dns-parameters/dns-parameters.xhtml#dns-parameters-6">DNS RCODES</a> on the IANA website. <br/>RecordData:  A list that contains values that Amazon Route 53 returned for this resource record set.<br/>RecordType:  The type of the resource record set that you submitted a request for.<br/>RecordName:  The name of the resource record set that you submitted a request for.<br/>Nameserver:  The Amazon Route 53 name server used to respond to the request.<br/>        """
        pass

    def update_health_check(self, HealthCheckId: str, HealthCheckVersion: HealthCheckVersion=None, IPAddress: str=None, Port: int=None, ResourcePath: str=None, FullyQualifiedDomainName: str=None, SearchString: str=None, FailureThreshold: int=None, Inverted: Inverted=None, Disabled: Disabled=None, HealthThreshold: int=None, ChildHealthChecks: List[HealthCheckId]=None, EnableSNI: EnableSNI=None, Regions: List[HealthCheckRegion]=None, AlarmIdentifier: AlarmIdentifier=None, InsufficientDataHealthStatus: str=None, ResetElements: List[ResettableElementName]=None) -> UpdateHealthCheckResponse:
        """Updates an existing health check. Note that some values can't be updated.  For more information about updating health checks, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/health-checks-creating-deleting.html">Creating, Updating, and Deleting Health Checks</a> in the <i>Amazon Route 53 Developer Guide</i>.

        :param self:
        :param ResetElements:  A complex type that contains one <code>ResettableElementName</code> element for each element that you want to reset to the default value. Valid values for <code>ResettableElementName</code> include the following: <ul> <li>  <code>ChildHealthChecks</code>: Amazon Route 53 resets <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_HealthCheckConfig.html#Route53-Type-HealthCheckConfig-ChildHealthChecks">ChildHealthChecks</a> to null. </li> <li>  <code>FullyQualifiedDomainName</code>: Route 53 resets <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_UpdateHealthCheck.html#Route53-UpdateHealthCheck-request-FullyQualifiedDomainName">FullyQualifiedDomainName</a>. to null. </li> <li>  <code>Regions</code>: Route 53 resets the <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_HealthCheckConfig.html#Route53-Type-HealthCheckConfig-Regions">Regions</a> list to the default set of regions.  </li> <li>  <code>ResourcePath</code>: Route 53 resets <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_HealthCheckConfig.html#Route53-Type-HealthCheckConfig-ResourcePath">ResourcePath</a> to null. </li> </ul><br/>InsufficientDataHealthStatus:  When CloudWatch has insufficient data about the metric to determine the alarm state, the status that you want Amazon Route 53 to assign to the health check: <ul> <li>  <code>Healthy</code>: Route 53 considers the health check to be healthy. </li> <li>  <code>Unhealthy</code>: Route 53 considers the health check to be unhealthy. </li> <li>  <code>LastKnownStatus</code>: Route 53 uses the status of the health check from the last time CloudWatch had sufficient data to determine the alarm state. For new health checks that have no last known status, the default status for the health check is healthy. </li> </ul><br/>AlarmIdentifier:  A complex type that identifies the CloudWatch alarm that you want Amazon Route 53 health checkers to use to determine whether the specified health check is healthy.:<br/>Regions:  A complex type that contains one <code>Region</code> element for each region that you want Amazon Route 53 health checkers to check the specified endpoint from.<br/>EnableSNI:  Specify whether you want Amazon Route 53 to send the value of <code>FullyQualifiedDomainName</code> to the endpoint in the <code>client_hello</code> message during <code>TLS</code> negotiation. This allows the endpoint to respond to <code>HTTPS</code> health check requests with the applicable SSL/TLS certificate. Some endpoints require that HTTPS requests include the host name in the <code>client_hello</code> message. If you don't enable SNI, the status of the health check will be SSL alert <code>handshake_failure</code>. A health check can also have that status for other reasons. If SNI is enabled and you're still getting the error, check the SSL/TLS configuration on your endpoint and confirm that your certificate is valid. The SSL/TLS certificate on your endpoint includes a domain name in the <code>Common Name</code> field and possibly several more in the <code>Subject Alternative Names</code> field. One of the domain names in the certificate should match the value that you specify for <code>FullyQualifiedDomainName</code>. If the endpoint responds to the <code>client_hello</code> message with a certificate that does not include the domain name that you specified in <code>FullyQualifiedDomainName</code>, a health checker will retry the handshake. In the second attempt, the health checker will omit <code>FullyQualifiedDomainName</code> from the <code>client_hello</code> message.<br/>ChildHealthChecks:  A complex type that contains one <code>ChildHealthCheck</code> element for each health check that you want to associate with a <code>CALCULATED</code> health check.<br/>HealthThreshold:  The number of child health checks that are associated with a <code>CALCULATED</code> health that Amazon Route 53 must consider healthy for the <code>CALCULATED</code> health check to be considered healthy. To specify the child health checks that you want to associate with a <code>CALCULATED</code> health check, use the <code>ChildHealthChecks</code> and <code>ChildHealthCheck</code> elements. Note the following: <ul> <li> If you specify a number greater than the number of child health checks, Route 53 always considers this health check to be unhealthy. </li> <li> If you specify <code>0</code>, Route 53 always considers this health check to be healthy. </li> </ul><br/>Disabled:  Stops Route 53 from performing health checks. When you disable a health check, here's what happens: <ul> <li>  <b>Health checks that check the health of endpoints:</b> Route 53 stops submitting requests to your application, server, or other resource. </li> <li>  <b>Calculated health checks:</b> Route 53 stops aggregating the status of the referenced health checks. </li> <li>  <b>Health checks that monitor CloudWatch alarms:</b> Route 53 stops monitoring the corresponding CloudWatch metrics. </li> </ul> After you disable a health check, Route 53 considers the status of the health check to always be healthy. If you configured DNS failover, Route 53 continues to route traffic to the corresponding resources. If you want to stop routing traffic to a resource, change the value of <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_UpdateHealthCheck.html#Route53-UpdateHealthCheck-request-Inverted">Inverted</a>.  Charges for a health check still apply when the health check is disabled. For more information, see <a href="http://aws.amazon.com/route53/pricing/">Amazon Route 53 Pricing</a>.<br/>Inverted:  Specify whether you want Amazon Route 53 to invert the status of a health check, for example, to consider a health check unhealthy when it otherwise would be considered healthy.<br/>FailureThreshold:  The number of consecutive health checks that an endpoint must pass or fail for Amazon Route 53 to change the current status of the endpoint from unhealthy to healthy or vice versa. For more information, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/dns-failover-determining-health-of-endpoints.html">How Amazon Route 53 Determines Whether an Endpoint Is Healthy</a> in the <i>Amazon Route 53 Developer Guide</i>. If you don't specify a value for <code>FailureThreshold</code>, the default value is three health checks.<br/>SearchString:  If the value of <code>Type</code> is <code>HTTP_STR_MATCH</code> or <code>HTTP_STR_MATCH</code>, the string that you want Amazon Route 53 to search for in the response body from the specified resource. If the string appears in the response body, Route 53 considers the resource healthy. (You can't change the value of <code>Type</code> when you update a health check.)<br/>FullyQualifiedDomainName:  Amazon Route 53 behavior depends on whether you specify a value for <code>IPAddress</code>. <note> If a health check already has a value for <code>IPAddress</code>, you can change the value. However, you can't update an existing health check to add or remove the value of <code>IPAddress</code>.  </note>  <b>If you specify a value for</b> <code>IPAddress</code>: Route 53 sends health check requests to the specified IPv4 or IPv6 address and passes the value of <code>FullyQualifiedDomainName</code> in the <code>Host</code> header for all health checks except TCP health checks. This is typically the fully qualified DNS name of the endpoint on which you want Route 53 to perform health checks. When Route 53 checks the health of an endpoint, here is how it constructs the <code>Host</code> header: <ul> <li> If you specify a value of <code>80</code> for <code>Port</code> and <code>HTTP</code> or <code>HTTP_STR_MATCH</code> for <code>Type</code>, Route 53 passes the value of <code>FullyQualifiedDomainName</code> to the endpoint in the <code>Host</code> header. </li> <li> If you specify a value of <code>443</code> for <code>Port</code> and <code>HTTPS</code> or <code>HTTPS_STR_MATCH</code> for <code>Type</code>, Route 53 passes the value of <code>FullyQualifiedDomainName</code> to the endpoint in the <code>Host</code> header. </li> <li> If you specify another value for <code>Port</code> and any value except <code>TCP</code> for <code>Type</code>, Route 53 passes <i> <code>FullyQualifiedDomainName</code>:<code>Port</code> </i> to the endpoint in the <code>Host</code> header. </li> </ul> If you don't specify a value for <code>FullyQualifiedDomainName</code>, Route 53 substitutes the value of <code>IPAddress</code> in the <code>Host</code> header in each of the above cases.  <b>If you don't specify a value for</b> <code>IPAddress</code>: If you don't specify a value for <code>IPAddress</code>, Route 53 sends a DNS request to the domain that you specify in <code>FullyQualifiedDomainName</code> at the interval you specify in <code>RequestInterval</code>. Using an IPv4 address that is returned by DNS, Route 53 then checks the health of the endpoint. <note> If you don't specify a value for <code>IPAddress</code>, Route 53 uses only IPv4 to send health checks to the endpoint. If there's no resource record set with a type of A for the name that you specify for <code>FullyQualifiedDomainName</code>, the health check fails with a "DNS resolution failed" error. </note> If you want to check the health of weighted, latency, or failover resource record sets and you choose to specify the endpoint only by <code>FullyQualifiedDomainName</code>, we recommend that you create a separate health check for each endpoint. For example, create a health check for each HTTP server that is serving content for www.example.com. For the value of <code>FullyQualifiedDomainName</code>, specify the domain name of the server (such as <code>us-east-2-www.example.com</code>), not the name of the resource record sets (www.example.com). <important> In this configuration, if the value of <code>FullyQualifiedDomainName</code> matches the name of the resource record sets and you then associate the health check with those resource record sets, health check results will be unpredictable. </important> In addition, if the value of <code>Type</code> is <code>HTTP</code>, <code>HTTPS</code>, <code>HTTP_STR_MATCH</code>, or <code>HTTPS_STR_MATCH</code>, Route 53 passes the value of <code>FullyQualifiedDomainName</code> in the <code>Host</code> header, as it does when you specify a value for <code>IPAddress</code>. If the value of <code>Type</code> is <code>TCP</code>, Route 53 doesn't pass a <code>Host</code> header.<br/>ResourcePath:  The path that you want Amazon Route 53 to request when performing health checks. The path can be any value for which your endpoint will return an HTTP status code of 2xx or 3xx when the endpoint is healthy, for example the file /docs/route53-health-check.html. You can also include query string parameters, for example, <code>/welcome.html?language=jp&amp;login=y</code>.  Specify this value only if you want to change it.<br/>Port:  The port on the endpoint on which you want Amazon Route 53 to perform health checks.<br/>IPAddress:  The IPv4 or IPv6 IP address for the endpoint that you want Amazon Route 53 to perform health checks on. If you don't specify a value for <code>IPAddress</code>, Route 53 sends a DNS request to resolve the domain name that you specify in <code>FullyQualifiedDomainName</code> at the interval that you specify in <code>RequestInterval</code>. Using an IP address that is returned by DNS, Route 53 then checks the health of the endpoint. Use one of the following formats for the value of <code>IPAddress</code>:  <ul> <li>  <b>IPv4 address</b>: four values between 0 and 255, separated by periods (.), for example, <code>192.0.2.44</code>. </li> <li>  <b>IPv6 address</b>: eight groups of four hexadecimal values, separated by colons (:), for example, <code>2001:0db8:85a3:0000:0000:abcd:0001:2345</code>. You can also shorten IPv6 addresses as described in RFC 5952, for example, <code>2001:db8:85a3::abcd:1:2345</code>. </li> </ul> If the endpoint is an EC2 instance, we recommend that you create an Elastic IP address, associate it with your EC2 instance, and specify the Elastic IP address for <code>IPAddress</code>. This ensures that the IP address of your instance never changes. For more information, see the applicable documentation: <ul> <li> Linux: <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/elastic-ip-addresses-eip.html">Elastic IP Addresses (EIP)</a> in the <i>Amazon EC2 User Guide for Linux Instances</i>  </li> <li> Windows: <a href="https://docs.aws.amazon.com/AWSEC2/latest/WindowsGuide/elastic-ip-addresses-eip.html">Elastic IP Addresses (EIP)</a> in the <i>Amazon EC2 User Guide for Windows Instances</i>  </li> </ul> <note> If a health check already has a value for <code>IPAddress</code>, you can change the value. However, you can't update an existing health check to add or remove the value of <code>IPAddress</code>.  </note> For more information, see <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_UpdateHealthCheck.html#Route53-UpdateHealthCheck-request-FullyQualifiedDomainName">FullyQualifiedDomainName</a>.  Constraints: Route 53 can't check the health of endpoints for which the IP address is in local, private, non-routable, or multicast ranges. For more information about IP addresses for which you can't create health checks, see the following documents: <ul> <li>  <a href="https://tools.ietf.org/html/rfc5735">RFC 5735, Special Use IPv4 Addresses</a>  </li> <li>  <a href="https://tools.ietf.org/html/rfc6598">RFC 6598, IANA-Reserved IPv4 Prefix for Shared Address Space</a>  </li> <li>  <a href="https://tools.ietf.org/html/rfc5156">RFC 5156, Special-Use IPv6 Addresses</a>  </li> </ul><br/>HealthCheckVersion:  A sequential counter that Amazon Route 53 sets to <code>1</code> when you create a health check and increments by 1 each time you update settings for the health check. We recommend that you use <code>GetHealthCheck</code> or <code>ListHealthChecks</code> to get the current value of <code>HealthCheckVersion</code> for the health check that you want to update, and that you include that value in your <code>UpdateHealthCheck</code> request. This prevents Route 53 from overwriting an intervening update: <ul> <li> If the value in the <code>UpdateHealthCheck</code> request matches the value of <code>HealthCheckVersion</code> in the health check, Route 53 updates the health check with the new settings. </li> <li> If the value of <code>HealthCheckVersion</code> in the health check is greater, the health check was changed after you got the version number. Route 53 does not update the health check, and it returns a <code>HealthCheckVersionMismatch</code> error. </li> </ul><br/>HealthCheckId:  The ID for the health check for which you want detailed information. When you created the health check, <code>CreateHealthCheck</code> returned the ID in the response, in the <code>HealthCheckId</code> element.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the CloudWatch alarm that you want Amazon Route 53 health checkers to use to determine whether this health check is healthy. <note> Route 53 supports CloudWatch alarms with the following features: <ul> <li> Standard-resolution metrics. High-resolution metrics aren't supported. For more information, see <a href="http://docs.aws.amazon.com/AmazonCloudWatch/latest/DeveloperGuide/publishingMetrics.html#high-resolution-metrics">High-Resolution Metrics</a> in the <i>Amazon CloudWatch User Guide</i>. </li> <li> Statistics: Average, Minimum, Maximum, Sum, and SampleCount. Extended statistics aren't supported. </li> </ul> </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Region</i> For the CloudWatch alarm that you want Route 53 health checkers to use to determine whether this health check is healthy, the region that the alarm was created in. For the current list of CloudWatch regions, see <a href="http://docs.aws.amazon.com/general/latest/gr/rande.html#cw_region">Amazon CloudWatch</a> in the <i>AWS Regions and Endpoints</i> chapter of the <i>Amazon Web Services General Reference</i>.<br/>
        :return: HealthCheck:  A complex type that contains the response to an <code>UpdateHealthCheck</code> request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchAlarmConfiguration</i> A complex type that contains information about the CloudWatch alarm that Amazon Route 53 is monitoring for this health check.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CloudWatchAlarmConfiguration</b>: A complex type that contains information about the CloudWatch alarm that Amazon Route 53 is monitoring for this health check.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HealthCheckVersion</i> The version of the health check. You can optionally pass this value in a call to <code>UpdateHealthCheck</code> to prevent overwriting another change to the health check.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HealthCheckConfig</i> A complex type that contains detailed information about one health check.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>HealthCheckConfig</b>: A complex type that contains detailed information about one health check.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LinkedService</i> If the health check was created by another service, the service that created the health check. When a health check is created by another service, you can't edit or delete it using Amazon Route 53. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LinkedService</b>: If the health check was created by another service, the service that created the health check. When a health check is created by another service, you can't edit or delete it using Amazon Route 53. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> A unique string that you specified when you created the health check.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier that Amazon Route 53assigned to the health check when you created it. When you add or update a resource record set, you use this value to specify which health check to use. The value can be up to 64 characters long. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> If the health check or hosted zone was created by another service, an optional description that can be provided by the other service. When a resource is created by another service, you can't edit or delete it using Amazon Route 53. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServicePrincipal</i> If the health check or hosted zone was created by another service, the service that created the resource. When a resource is created by another service, you can't edit or delete it using Amazon Route 53. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Regions</i> A complex type that contains one <code>Region</code> element for each region from which you want Amazon Route 53 health checkers to check the specified endpoint. If you don't specify any regions, Route 53 health checkers automatically performs checks from all of the regions that are listed under <b>Valid Values</b>. If you update a health check to remove a region that has been performing health checks, Route 53 will briefly continue to perform checks from that region to ensure that some health checkers are always checking the endpoint (for example, if you replace three regions with four different regions). <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EnableSNI</i> Specify whether you want Amazon Route 53 to send the value of <code>FullyQualifiedDomainName</code> to the endpoint in the <code>client_hello</code> message during TLS negotiation. This allows the endpoint to respond to <code>HTTPS</code> health check requests with the applicable SSL/TLS certificate. Some endpoints require that <code>HTTPS</code> requests include the host name in the <code>client_hello</code> message. If you don't enable SNI, the status of the health check will be <code>SSL alert handshake_failure</code>. A health check can also have that status for other reasons. If SNI is enabled and you're still getting the error, check the SSL/TLS configuration on your endpoint and confirm that your certificate is valid. The SSL/TLS certificate on your endpoint includes a domain name in the <code>Common Name</code> field and possibly several more in the <code>Subject Alternative Names</code> field. One of the domain names in the certificate should match the value that you specify for <code>FullyQualifiedDomainName</code>. If the endpoint responds to the <code>client_hello</code> message with a certificate that does not include the domain name that you specified in <code>FullyQualifiedDomainName</code>, a health checker will retry the handshake. In the second attempt, the health checker will omit <code>FullyQualifiedDomainName</code> from the <code>client_hello</code> message.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ChildHealthChecks</i> (CALCULATED Health Checks Only) A complex type that contains one <code>ChildHealthCheck</code> element for each health check that you want to associate with a <code>CALCULATED</code> health check.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HealthThreshold</i> The number of child health checks that are associated with a <code>CALCULATED</code> health check that Amazon Route 53 must consider healthy for the <code>CALCULATED</code> health check to be considered healthy. To specify the child health checks that you want to associate with a <code>CALCULATED</code> health check, use the <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_UpdateHealthCheck.html#Route53-UpdateHealthCheck-request-ChildHealthChecks">ChildHealthChecks</a> element. Note the following: <ul> <li> If you specify a number greater than the number of child health checks, Route 53 always considers this health check to be unhealthy. </li> <li> If you specify <code>0</code>, Route 53 always considers this health check to be healthy. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Disabled</i> Stops Route 53 from performing health checks. When you disable a health check, here's what happens: <ul> <li>  <b>Health checks that check the health of endpoints:</b> Route 53 stops submitting requests to your application, server, or other resource. </li> <li>  <b>Calculated health checks:</b> Route 53 stops aggregating the status of the referenced health checks. </li> <li>  <b>Health checks that monitor CloudWatch alarms:</b> Route 53 stops monitoring the corresponding CloudWatch metrics. </li> </ul> After you disable a health check, Route 53 considers the status of the health check to always be healthy. If you configured DNS failover, Route 53 continues to route traffic to the corresponding resources. If you want to stop routing traffic to a resource, change the value of <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_UpdateHealthCheck.html#Route53-UpdateHealthCheck-request-Inverted">Inverted</a>.  Charges for a health check still apply when the health check is disabled. For more information, see <a href="http://aws.amazon.com/route53/pricing/">Amazon Route 53 Pricing</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Inverted</i> Specify whether you want Amazon Route 53 to invert the status of a health check, for example, to consider a health check unhealthy when it otherwise would be considered healthy.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MeasureLatency</i> Specify whether you want Amazon Route 53 to measure the latency between health checkers in multiple AWS regions and your endpoint, and to display CloudWatch latency graphs on the <b>Health Checks</b> page in the Route 53 console. <important> You can't change the value of <code>MeasureLatency</code> after you create a health check. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureThreshold</i> The number of consecutive health checks that an endpoint must pass or fail for Amazon Route 53 to change the current status of the endpoint from unhealthy to healthy or vice versa. For more information, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/dns-failover-determining-health-of-endpoints.html">How Amazon Route 53 Determines Whether an Endpoint Is Healthy</a> in the <i>Amazon Route 53 Developer Guide</i>. If you don't specify a value for <code>FailureThreshold</code>, the default value is three health checks.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestInterval</i> The number of seconds between the time that Amazon Route 53 gets a response from your endpoint and the time that it sends the next health check request. Each Route 53 health checker makes requests at this interval. <important> You can't change the value of <code>RequestInterval</code> after you create a health check. </important> If you don't specify a value for <code>RequestInterval</code>, the default value is <code>30</code> seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SearchString</i> If the value of Type is <code>HTTP_STR_MATCH</code> or <code>HTTP_STR_MATCH</code>, the string that you want Amazon Route 53 to search for in the response body from the specified resource. If the string appears in the response body, Route 53 considers the resource healthy. Route 53 considers case when searching for <code>SearchString</code> in the response body. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FullyQualifiedDomainName</i> Amazon Route 53 behavior depends on whether you specify a value for <code>IPAddress</code>.  <b>If you specify a value for</b> <code>IPAddress</code>: Amazon Route 53 sends health check requests to the specified IPv4 or IPv6 address and passes the value of <code>FullyQualifiedDomainName</code> in the <code>Host</code> header for all health checks except TCP health checks. This is typically the fully qualified DNS name of the endpoint on which you want Route 53 to perform health checks. When Route 53 checks the health of an endpoint, here is how it constructs the <code>Host</code> header: <ul> <li> If you specify a value of <code>80</code> for <code>Port</code> and <code>HTTP</code> or <code>HTTP_STR_MATCH</code> for <code>Type</code>, Route 53 passes the value of <code>FullyQualifiedDomainName</code> to the endpoint in the Host header.  </li> <li> If you specify a value of <code>443</code> for <code>Port</code> and <code>HTTPS</code> or <code>HTTPS_STR_MATCH</code> for <code>Type</code>, Route 53 passes the value of <code>FullyQualifiedDomainName</code> to the endpoint in the <code>Host</code> header. </li> <li> If you specify another value for <code>Port</code> and any value except <code>TCP</code> for <code>Type</code>, Route 53 passes <code>FullyQualifiedDomainName:Port</code> to the endpoint in the <code>Host</code> header. </li> </ul> If you don't specify a value for <code>FullyQualifiedDomainName</code>, Route 53 substitutes the value of <code>IPAddress</code> in the <code>Host</code> header in each of the preceding cases.  <b>If you don't specify a value for <code>IPAddress</code> </b>: Route 53 sends a DNS request to the domain that you specify for <code>FullyQualifiedDomainName</code> at the interval that you specify for <code>RequestInterval</code>. Using an IPv4 address that DNS returns, Route 53 then checks the health of the endpoint. <note> If you don't specify a value for <code>IPAddress</code>, Route 53 uses only IPv4 to send health checks to the endpoint. If there's no resource record set with a type of A for the name that you specify for <code>FullyQualifiedDomainName</code>, the health check fails with a "DNS resolution failed" error. </note> If you want to check the health of weighted, latency, or failover resource record sets and you choose to specify the endpoint only by <code>FullyQualifiedDomainName</code>, we recommend that you create a separate health check for each endpoint. For example, create a health check for each HTTP server that is serving content for www.example.com. For the value of <code>FullyQualifiedDomainName</code>, specify the domain name of the server (such as us-east-2-www.example.com), not the name of the resource record sets (www.example.com). <important> In this configuration, if you create a health check for which the value of <code>FullyQualifiedDomainName</code> matches the name of the resource record sets and you then associate the health check with those resource record sets, health check results will be unpredictable. </important> In addition, if the value that you specify for <code>Type</code> is <code>HTTP</code>, <code>HTTPS</code>, <code>HTTP_STR_MATCH</code>, or <code>HTTPS_STR_MATCH</code>, Route 53 passes the value of <code>FullyQualifiedDomainName</code> in the <code>Host</code> header, as it does when you specify a value for <code>IPAddress</code>. If the value of <code>Type</code> is <code>TCP</code>, Route 53 doesn't pass a <code>Host</code> header.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourcePath</i> The path, if any, that you want Amazon Route 53 to request when performing health checks. The path can be any value for which your endpoint will return an HTTP status code of 2xx or 3xx when the endpoint is healthy, for example, the file /docs/route53-health-check.html. You can also include query string parameters, for example, <code>/welcome.html?language=jp&amp;login=y</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of health check that you want to create, which indicates how Amazon Route 53 determines whether an endpoint is healthy. <important> You can't change the value of <code>Type</code> after you create a health check. </important> You can create the following types of health checks: <ul> <li>  <b>HTTP</b>: Route 53 tries to establish a TCP connection. If successful, Route 53 submits an HTTP request and waits for an HTTP status code of 200 or greater and less than 400. </li> <li>  <b>HTTPS</b>: Route 53 tries to establish a TCP connection. If successful, Route 53 submits an HTTPS request and waits for an HTTP status code of 200 or greater and less than 400. <important> If you specify <code>HTTPS</code> for the value of <code>Type</code>, the endpoint must support TLS v1.0 or later. </important> </li> <li>  <b>HTTP_STR_MATCH</b>: Route 53 tries to establish a TCP connection. If successful, Route 53 submits an HTTP request and searches the first 5,120 bytes of the response body for the string that you specify in <code>SearchString</code>. </li> <li>  <b>HTTPS_STR_MATCH</b>: Route 53 tries to establish a TCP connection. If successful, Route 53 submits an <code>HTTPS</code> request and searches the first 5,120 bytes of the response body for the string that you specify in <code>SearchString</code>. </li> <li>  <b>TCP</b>: Route 53 tries to establish a TCP connection. </li> <li>  <b>CLOUDWATCH_METRIC</b>: The health check is associated with a CloudWatch alarm. If the state of the alarm is <code>OK</code>, the health check is considered healthy. If the state is <code>ALARM</code>, the health check is considered unhealthy. If CloudWatch doesn't have sufficient data to determine whether the state is <code>OK</code> or <code>ALARM</code>, the health check status depends on the setting for <code>InsufficientDataHealthStatus</code>: <code>Healthy</code>, <code>Unhealthy</code>, or <code>LastKnownStatus</code>.  </li> <li>  <b>CALCULATED</b>: For health checks that monitor the status of other health checks, Route 53 adds up the number of health checks that Route 53 health checkers consider to be healthy and compares that number with the value of <code>HealthThreshold</code>.  </li> </ul> For more information, see <a href="https://docs.aws.amazon.com/Route53/latest/DeveloperGuide/dns-failover-determining-health-of-endpoints.html">How Route 53 Determines Whether an Endpoint Is Healthy</a> in the <i>Amazon Route 53 Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Port</i> The port on the endpoint on which you want Amazon Route 53 to perform health checks. Specify a value for <code>Port</code> only when you specify a value for <code>IPAddress</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IPAddress</i> The IPv4 or IPv6 IP address of the endpoint that you want Amazon Route 53 to perform health checks on. If you don't specify a value for <code>IPAddress</code>, Route 53 sends a DNS request to resolve the domain name that you specify in <code>FullyQualifiedDomainName</code> at the interval that you specify in <code>RequestInterval</code>. Using an IP address returned by DNS, Route 53 then checks the health of the endpoint. Use one of the following formats for the value of <code>IPAddress</code>:  <ul> <li>  <b>IPv4 address</b>: four values between 0 and 255, separated by periods (.), for example, <code>192.0.2.44</code>. </li> <li>  <b>IPv6 address</b>: eight groups of four hexadecimal values, separated by colons (:), for example, <code>2001:0db8:85a3:0000:0000:abcd:0001:2345</code>. You can also shorten IPv6 addresses as described in RFC 5952, for example, <code>2001:db8:85a3::abcd:1:2345</code>. </li> </ul> If the endpoint is an EC2 instance, we recommend that you create an Elastic IP address, associate it with your EC2 instance, and specify the Elastic IP address for <code>IPAddress</code>. This ensures that the IP address of your instance will never change. For more information, see <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_UpdateHealthCheck.html#Route53-UpdateHealthCheck-request-FullyQualifiedDomainName">FullyQualifiedDomainName</a>.  Constraints: Route 53 can't check the health of endpoints for which the IP address is in local, private, non-routable, or multicast ranges. For more information about IP addresses for which you can't create health checks, see the following documents: <ul> <li>  <a href="https://tools.ietf.org/html/rfc5735">RFC 5735, Special Use IPv4 Addresses</a>  </li> <li>  <a href="https://tools.ietf.org/html/rfc6598">RFC 6598, IANA-Reserved IPv4 Prefix for Shared Address Space</a>  </li> <li>  <a href="https://tools.ietf.org/html/rfc5156">RFC 5156, Special-Use IPv6 Addresses</a>  </li> </ul> When the value of <code>Type</code> is <code>CALCULATED</code> or <code>CLOUDWATCH_METRIC</code>, omit <code>IPAddress</code>.<br/>A complex type that identifies the CloudWatch alarm that you want Amazon Route 53 health checkers to use to determine whether the specified health check is healthy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AlarmIdentifier</b>: A complex type that identifies the CloudWatch alarm that you want Amazon Route 53 health checkers to use to determine whether the specified health check is healthy.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Dimensions</i> For the metric that the CloudWatch alarm is associated with, a complex type that contains information about the dimensions for the metric. For information, see <a href="http://docs.aws.amazon.com/AmazonCloudWatch/latest/DeveloperGuide/CW_Support_For_AWS.html">Amazon CloudWatch Namespaces, Dimensions, and Metrics Reference</a> in the <i>Amazon CloudWatch User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Statistic</i> For the metric that the CloudWatch alarm is associated with, the statistic that is applied to the metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Namespace</i> The namespace of the metric that the alarm is associated with. For more information, see <a href="http://docs.aws.amazon.com/AmazonCloudWatch/latest/DeveloperGuide/CW_Support_For_AWS.html">Amazon CloudWatch Namespaces, Dimensions, and Metrics Reference</a> in the <i>Amazon CloudWatch User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricName</i> The name of the CloudWatch metric that the alarm is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Period</i> For the metric that the CloudWatch alarm is associated with, the duration of one evaluation period in seconds.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ComparisonOperator</i> For the metric that the CloudWatch alarm is associated with, the arithmetic operation that is used for the comparison.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Threshold</i> For the metric that the CloudWatch alarm is associated with, the value the metric is compared with.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EvaluationPeriods</i> For the metric that the CloudWatch alarm is associated with, the number of periods that the metric is compared to the threshold.<br/>        """
        pass

    def update_hosted_zone_comment(self, Id: str, Comment: str=None) -> UpdateHostedZoneCommentResponse:
        """Updates the comment for a specified hosted zone.

        :param self:
        :param Comment:  The new comment for the hosted zone. If you don't specify a value for <code>Comment</code>, Amazon Route 53 deletes the existing value of the <code>Comment</code> element, if any.<br/>Id:  The ID for the hosted zone that you want to update the comment for.<br/>
        :return: HostedZone:  A complex type that contains the response to the <code>UpdateHostedZoneComment</code> request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LinkedService</i> If the hosted zone was created by another service, the service that created the hosted zone. When a hosted zone is created by another service, you can't edit or delete it using Route 53. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LinkedService</b>: If the hosted zone was created by another service, the service that created the hosted zone. When a hosted zone is created by another service, you can't edit or delete it using Route 53. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceRecordSetCount</i> The number of resource record sets in the hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Config</i> A complex type that includes the <code>Comment</code> and <code>PrivateZone</code> elements. If you omitted the <code>HostedZoneConfig</code> and <code>Comment</code> elements from the request, the <code>Config</code> and <code>Comment</code> elements don't appear in the response.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Config</b>: A complex type that includes the <code>Comment</code> and <code>PrivateZone</code> elements. If you omitted the <code>HostedZoneConfig</code> and <code>Comment</code> elements from the request, the <code>Config</code> and <code>Comment</code> elements don't appear in the response.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallerReference</i> The value that you specified for <code>CallerReference</code> when you created the hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the domain. For public hosted zones, this is the name that you have registered with your DNS registrar. For information about how to specify characters other than <code>a-z</code>, <code>0-9</code>, and <code>-</code> (hyphen) and how to specify internationalized domain names, see <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/API_CreateHostedZone.html">CreateHostedZone</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID that Amazon Route 53 assigned to the hosted zone when you created it.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PrivateZone</i> A value that indicates whether this is a private hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> Any comments that you want to include about the hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> If the health check or hosted zone was created by another service, an optional description that can be provided by the other service. When a resource is created by another service, you can't edit or delete it using Amazon Route 53. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServicePrincipal</i> If the health check or hosted zone was created by another service, the service that created the resource. When a resource is created by another service, you can't edit or delete it using Amazon Route 53. <br/>        """
        pass

    def update_traffic_policy_comment(self, Comment: str, Version: int, Id: str) -> UpdateTrafficPolicyCommentResponse:
        """Updates the comment for a specified traffic policy version.

        :param self:
        :param Comment:  The new comment for the specified traffic policy and version.<br/>Version:  The value of <code>Version</code> for the traffic policy that you want to update the comment for.<br/>Id:  The value of <code>Id</code> for the traffic policy that you want to update the comment for.<br/>
        :return: TrafficPolicy:  A complex type that contains settings for the specified traffic policy.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> The comment that you specify in the <code>CreateTrafficPolicy</code> request, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Document</i> The definition of a traffic policy in JSON format. You specify the JSON document to use for a new traffic policy in the <code>CreateTrafficPolicy</code> request. For more information about the JSON format, see <a href="https://docs.aws.amazon.com/Route53/latest/APIReference/api-policies-traffic-policy-document-format.html">Traffic Policy Document Format</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The DNS type of the resource record sets that Amazon Route 53 creates when you use a traffic policy to create a traffic policy instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name that you specified when you created the traffic policy.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version number that Amazon Route 53 assigns to a traffic policy. For a new traffic policy, the value of <code>Version</code> is always 1.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID that Amazon Route 53 assigned to a traffic policy when you created it.<br/>        """
        pass

    def update_traffic_policy_instance(self, TrafficPolicyVersion: int, TrafficPolicyId: str, TTL: TTL, Id: str) -> UpdateTrafficPolicyInstanceResponse:
        """Updates the resource record sets in a specified hosted zone that were created based on the settings in a specified traffic policy version. When you update a traffic policy instance, Amazon Route 53 continues to respond to DNS queries for the root resource record set name (such as example.com) while it replaces one group of resource record sets with another. Route 53 performs the following operations: <ol> <li> Route 53 creates a new group of resource record sets based on the specified traffic policy. This is true regardless of how significant the differences are between the existing resource record sets and the new resource record sets.  </li> <li> When all of the new resource record sets have been created, Route 53 starts to respond to DNS queries for the root resource record set name (such as example.com) by using the new resource record sets. </li> <li> Route 53 deletes the old group of resource record sets that are associated with the root resource record set name. </li> </ol>

        :param self:
        :param TrafficPolicyVersion:  The version of the traffic policy that you want Amazon Route 53 to use to update resource record sets for the specified traffic policy instance.<br/>TrafficPolicyId:  The ID of the traffic policy that you want Amazon Route 53 to use to update resource record sets for the specified traffic policy instance.<br/>TTL:  The TTL that you want Amazon Route 53 to assign to all of the updated resource record sets.<br/>Id:  The ID of the traffic policy instance that you want to update.<br/>
        :return: TrafficPolicyInstance:  A complex type that contains settings for the updated traffic policy instance.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrafficPolicyType</i> The DNS type that Amazon Route 53 assigned to all of the resource record sets that it created for this traffic policy instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrafficPolicyVersion</i> The version of the traffic policy that Amazon Route 53 used to create resource record sets in the specified hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrafficPolicyId</i> The ID of the traffic policy that Amazon Route 53 used to create resource record sets in the specified hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> If <code>State</code> is <code>Failed</code>, an explanation of the reason for the failure. If <code>State</code> is another value, <code>Message</code> is empty.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The value of <code>State</code> is one of the following values: <dl> <dt>Applied</dt> <dd> Amazon Route 53 has finished creating resource record sets, and changes have propagated to all Route 53 edge locations. </dd> <dt>Creating</dt> <dd> Route 53 is creating the resource record sets. Use <code>GetTrafficPolicyInstance</code> to confirm that the <code>CreateTrafficPolicyInstance</code> request completed successfully. </dd> <dt>Failed</dt> <dd> Route 53 wasn't able to create or update the resource record sets. When the value of <code>State</code> is <code>Failed</code>, see <code>Message</code> for an explanation of what caused the request to fail. </dd> </dl><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TTL</i> The TTL that Amazon Route 53 assigned to all of the resource record sets that it created in the specified hosted zone.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The DNS name, such as www.example.com, for which Amazon Route 53 responds to queries by using the resource record sets that are associated with this traffic policy instance. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostedZoneId</i> The ID of the hosted zone that Amazon Route 53 created resource record sets in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID that Amazon Route 53 assigned to the new traffic policy instance.<br/>        """
        pass



class Route53Domains(BaseClient):
    """Amazon Route 53 API actions let you register domain names and perform related operations."""


    class ListDomainsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_domains` for valid parameters"""
            pass
    class ListOperationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_operations` for valid parameters"""
            pass
    class ViewBillingPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `view_billing` for valid parameters"""
            pass


    class CheckDomainAvailabilityRequest(Mapping):
        pass
    
    class CheckDomainAvailabilityResponse(Mapping):
        pass
    
    class CheckDomainTransferabilityRequest(Mapping):
        pass
    
    class CheckDomainTransferabilityResponse(Mapping):
        pass
    
    class DeleteTagsForDomainRequest(Mapping):
        pass
    
    class DeleteTagsForDomainResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class DisableDomainAutoRenewRequest(Mapping):
        pass
    
    class DisableDomainAutoRenewResponse(Mapping):
        pass
    
    class DisableDomainTransferLockRequest(Mapping):
        pass
    
    class DisableDomainTransferLockResponse(Mapping):
        pass
    
    class EnableDomainAutoRenewRequest(Mapping):
        pass
    
    class EnableDomainAutoRenewResponse(Mapping):
        pass
    
    class EnableDomainTransferLockRequest(Mapping):
        pass
    
    class EnableDomainTransferLockResponse(Mapping):
        pass
    
    class GetContactReachabilityStatusRequest(Mapping):
        pass
    
    class GetContactReachabilityStatusResponse(Mapping):
        pass
    
    class GetDomainDetailRequest(Mapping):
        pass
    
    class GetDomainDetailResponse(Mapping):
        pass
    
    class GetDomainSuggestionsRequest(Mapping):
        pass
    
    class GetDomainSuggestionsResponse(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class GetOperationDetailRequest(Mapping):
        pass
    
    class GetOperationDetailResponse(Mapping):
        pass
    
    class ListDomainsRequest(Mapping):
        pass
    
    class ListDomainsResponse(Mapping):
        pass
    
    class ListOperationsRequest(Mapping):
        pass
    
    class ListOperationsResponse(Mapping):
        pass
    
    class Timestamp(object):
        pass
    
    class ListTagsForDomainRequest(Mapping):
        pass
    
    class ListTagsForDomainResponse(Mapping):
        pass
    
    class RegisterDomainRequest(Mapping):
        pass
    
    class RegisterDomainResponse(Mapping):
        pass
    
    class ContactDetail(Mapping):
        pass
    
    class RenewDomainRequest(Mapping):
        pass
    
    class RenewDomainResponse(Mapping):
        pass
    
    class ResendContactReachabilityEmailRequest(Mapping):
        pass
    
    class ResendContactReachabilityEmailResponse(Mapping):
        pass
    
    class RetrieveDomainAuthCodeRequest(Mapping):
        pass
    
    class RetrieveDomainAuthCodeResponse(Mapping):
        pass
    
    class TransferDomainRequest(Mapping):
        pass
    
    class TransferDomainResponse(Mapping):
        pass
    
    class Nameserver(Mapping):
        pass
    
    class UpdateDomainContactRequest(Mapping):
        pass
    
    class UpdateDomainContactResponse(Mapping):
        pass
    
    class UpdateDomainContactPrivacyRequest(Mapping):
        pass
    
    class UpdateDomainContactPrivacyResponse(Mapping):
        pass
    
    class UpdateDomainNameserversRequest(Mapping):
        pass
    
    class UpdateDomainNameserversResponse(Mapping):
        pass
    
    class UpdateTagsForDomainRequest(Mapping):
        pass
    
    class UpdateTagsForDomainResponse(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class ViewBillingRequest(Mapping):
        pass
    
    class ViewBillingResponse(Mapping):
        pass
    
    def check_domain_availability(self, DomainName: str, IdnLangCode: str=None) -> CheckDomainAvailabilityResponse:
        """This operation checks the availability of one domain name. Note that if the availability status of a domain is pending, you must submit another request to determine the availability of the domain name.

        :param self:
        :param IdnLangCode:  Reserved for future use.<br/>DomainName:  The name of the domain that you want to get availability for. Constraints: The domain name can contain only the letters a through z, the numbers 0 through 9, and hyphen (-). Internationalized Domain Names are not supported.<br/>
        :return: Availability:  Whether the domain name is available for registering. <note> You can register only domains designated as <code>AVAILABLE</code>. </note> Valid values: <dl> <dt>AVAILABLE</dt> <dd> The domain name is available. </dd> <dt>AVAILABLE_RESERVED</dt> <dd> The domain name is reserved under specific conditions. </dd> <dt>AVAILABLE_PREORDER</dt> <dd> The domain name is available and can be preordered. </dd> <dt>DONT_KNOW</dt> <dd> The TLD registry didn't reply with a definitive answer about whether the domain name is available. Amazon Route 53 can return this response for a variety of reasons, for example, the registry is performing maintenance. Try again later. </dd> <dt>PENDING</dt> <dd> The TLD registry didn't return a response in the expected amount of time. When the response is delayed, it usually takes just a few extra seconds. You can resubmit the request immediately. </dd> <dt>RESERVED</dt> <dd> The domain name has been reserved for another person or organization. </dd> <dt>UNAVAILABLE</dt> <dd> The domain name is not available. </dd> <dt>UNAVAILABLE_PREMIUM</dt> <dd> The domain name is not available. </dd> <dt>UNAVAILABLE_RESTRICTED</dt> <dd> The domain name is forbidden. </dd> </dl><br/>        """
        pass

    def check_domain_transferability(self, DomainName: str, AuthCode: str=None) -> CheckDomainTransferabilityResponse:
        """Checks whether a domain name can be transferred to Amazon Route 53. 

        :param self:
        :param AuthCode:  If the registrar for the top-level domain (TLD) requires an authorization code to transfer the domain, the code that you got from the current registrar for the domain.<br/>DomainName:  The name of the domain that you want to transfer to Amazon Route 53. Constraints: The domain name can contain only the letters a through z, the numbers 0 through 9, and hyphen (-). Internationalized Domain Names are not supported.<br/>
        :return: Transferability:  A complex type that contains information about whether the specified domain can be transferred to Amazon Route 53.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Transferable</i> Whether the domain name can be transferred to Amazon Route 53. <note> You can transfer only domains that have a value of <code>TRANSFERABLE</code> for <code>Transferable</code>. </note> Valid values: <dl> <dt>TRANSFERABLE</dt> <dd> The domain name can be transferred to Amazon Route 53. </dd> <dt>UNTRANSFERRABLE</dt> <dd> The domain name can't be transferred to Amazon Route 53. </dd> <dt>DONT_KNOW</dt> <dd> Reserved for future use. </dd> </dl><br/>        """
        pass

    def delete_tags_for_domain(self, TagsToDelete: List[TagKey], DomainName: str) -> DeleteTagsForDomainResponse:
        """This operation deletes the specified tags for a domain. All tag operations are eventually consistent; subsequent operations might not immediately represent all issued operations.

        :param self:
        :param TagsToDelete:  A list of tag keys to delete.<br/>DomainName:  The domain for which you want to delete one or more tags.<br/>
        :return:         """
        pass

    def disable_domain_auto_renew(self, DomainName: str) -> DisableDomainAutoRenewResponse:
        """This operation disables automatic renewal of domain registration for the specified domain.

        :param self:
        :param DomainName:  The name of the domain that you want to disable automatic renewal for.<br/>
        :return:         """
        pass

    def disable_domain_transfer_lock(self, DomainName: str) -> DisableDomainTransferLockResponse:
        """This operation removes the transfer lock on the domain (specifically the <code>clientTransferProhibited</code> status) to allow domain transfers. We recommend you refrain from performing this action unless you intend to transfer the domain to a different registrar. Successful submission returns an operation ID that you can use to track the progress and completion of the action. If the request is not completed successfully, the domain registrant will be notified by email.

        :param self:
        :param DomainName:  The name of the domain that you want to remove the transfer lock for.<br/>
        :return: OperationId:  Identifier for tracking the progress of the request. To use this ID to query the operation status, use <a>GetOperationDetail</a>.<br/>        """
        pass

    def enable_domain_auto_renew(self, DomainName: str) -> EnableDomainAutoRenewResponse:
        """This operation configures Amazon Route 53 to automatically renew the specified domain before the domain registration expires. The cost of renewing your domain registration is billed to your AWS account. The period during which you can renew a domain name varies by TLD. For a list of TLDs and their renewal policies, see <a href="http://wiki.gandi.net/en/domains/renew#renewal_restoration_and_deletion_times">"Renewal, restoration, and deletion times"</a> on the website for our registrar associate, Gandi. Amazon Route 53 requires that you renew before the end of the renewal period that is listed on the Gandi website so we can complete processing before the deadline.

        :param self:
        :param DomainName:  The name of the domain that you want to enable automatic renewal for.<br/>
        :return:         """
        pass

    def enable_domain_transfer_lock(self, DomainName: str) -> EnableDomainTransferLockResponse:
        """This operation sets the transfer lock on the domain (specifically the <code>clientTransferProhibited</code> status) to prevent domain transfers. Successful submission returns an operation ID that you can use to track the progress and completion of the action. If the request is not completed successfully, the domain registrant will be notified by email.

        :param self:
        :param DomainName:  The name of the domain that you want to set the transfer lock for.<br/>
        :return: OperationId:  Identifier for tracking the progress of the request. To use this ID to query the operation status, use GetOperationDetail.<br/>        """
        pass

    def get_contact_reachability_status(self, domainName: str=None) -> GetContactReachabilityStatusResponse:
        """For operations that require confirmation that the email address for the registrant contact is valid, such as registering a new domain, this operation returns information about whether the registrant contact has responded. If you want us to resend the email, use the <code>ResendContactReachabilityEmail</code> operation.

        :param self:
        :param domainName:  The name of the domain for which you want to know whether the registrant contact has confirmed that the email address is valid.<br/>
        :return: status:  Whether the registrant contact has responded. Values include the following: <dl> <dt>PENDING</dt> <dd> We sent the confirmation email and haven't received a response yet. </dd> <dt>DONE</dt> <dd> We sent the email and got confirmation from the registrant contact. </dd> <dt>EXPIRED</dt> <dd> The time limit expired before the registrant contact responded. </dd> </dl><br/>domainName:  The domain name for which you requested the reachability status.<br/>        """
        pass

    def get_domain_detail(self, DomainName: str) -> GetDomainDetailResponse:
        """This operation returns detailed information about a specified domain that is associated with the current AWS account. Contact information for the domain is also returned as part of the output.

        :param self:
        :param DomainName:  The name of the domain that you want to get detailed information about.<br/>
        :return: StatusList:  An array of domain name status codes, also known as Extensible Provisioning Protocol (EPP) status codes. ICANN, the organization that maintains a central database of domain names, has developed a set of domain name status codes that tell you the status of a variety of operations on a domain name, for example, registering a domain name, transferring a domain name to another registrar, renewing the registration for a domain name, and so on. All registrars use this same set of status codes. For a current list of domain name status codes and an explanation of what each code means, go to the <a href="https://www.icann.org/">ICANN website</a> and search for <code>epp status codes</code>. (Search on the ICANN website; web searches sometimes return an old version of the document.)<br/>DnsSec:  Reserved for future use.<br/>Reseller:  Reseller of the domain. Domains registered or transferred using Amazon Route 53 domains will have <code>"Amazon"</code> as the reseller. <br/>ExpirationDate:  The date when the registration for the domain is set to expire. The date and time is in Coordinated Universal time (UTC).<br/>UpdatedDate:  The last updated date of the domain as found in the response to a WHOIS query. The date and time is in Coordinated Universal time (UTC).<br/>CreationDate:  The date when the domain was created as found in the response to a WHOIS query. The date and time is in Coordinated Universal time (UTC).<br/>RegistryDomainId:  Reserved for future use.<br/>AbuseContactPhone:  Phone number for reporting abuse.<br/>AbuseContactEmail:  Email address to contact to report incorrect contact information for a domain, to report that the domain is being used to send spam, to report that someone is cybersquatting on a domain name, or report some other type of abuse.<br/>RegistrarUrl:  Web address of the registrar.<br/>WhoIsServer:  The fully qualified name of the WHOIS server that can answer the WHOIS query for the domain.<br/>RegistrarName:  Name of the registrar of the domain as identified in the registry. Domains with a .com, .net, or .org TLD are registered by Amazon Registrar. All other domains are registered by our registrar associate, Gandi. The value for domains that are registered by Gandi is <code>"GANDI SAS"</code>. <br/>TechPrivacy:  Specifies whether contact information is concealed from WHOIS queries. If the value is <code>true</code>, WHOIS ("who is") queries return contact information either for Amazon Registrar (for .com, .net, and .org domains) or for our registrar associate, Gandi (for all other TLDs). If the value is <code>false</code>, WHOIS queries return the information that you entered for the technical contact.<br/>RegistrantPrivacy:  Specifies whether contact information is concealed from WHOIS queries. If the value is <code>true</code>, WHOIS ("who is") queries return contact information either for Amazon Registrar (for .com, .net, and .org domains) or for our registrar associate, Gandi (for all other TLDs). If the value is <code>false</code>, WHOIS queries return the information that you entered for the registrant contact (domain owner).<br/>AdminPrivacy:  Specifies whether contact information is concealed from WHOIS queries. If the value is <code>true</code>, WHOIS ("who is") queries return contact information either for Amazon Registrar (for .com, .net, and .org domains) or for our registrar associate, Gandi (for all other TLDs). If the value is <code>false</code>, WHOIS queries return the information that you entered for the admin contact.<br/>TechContact:  Provides details about the domain technical contact.:<br/>RegistrantContact:  Provides details about the domain registrant.:<br/>AdminContact:  Provides details about the domain administrative contact.:<br/>AutoRenew:  Specifies whether the domain registration is set to renew automatically.<br/>Nameservers:  The name of the domain.<br/>DomainName:  The name of a domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExtraParams</i> A list of name-value pairs for parameters required by certain top-level domains.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Fax</i> Fax number of the contact. Constraints: Phone number must be specified in the format "+[country dialing code].[number including any area code]". For example, a US phone number might appear as <code>"+1.1234567890"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Email</i> Email address of the contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneNumber</i> The phone number of the contact. Constraints: Phone number must be specified in the format "+[country dialing code].[number including any area code&gt;]". For example, a US phone number might appear as <code>"+1.1234567890"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ZipCode</i> The zip or postal code of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CountryCode</i> Code for the country of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state or province of the contact's city.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>City</i> The city of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressLine2</i> Second line of contact's address, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressLine1</i> First line of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationName</i> Name of the organization for contact types other than <code>PERSON</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContactType</i> Indicates whether the contact is a person, company, association, or public organization. If you choose an option other than <code>PERSON</code>, you must enter an organization name, and you can't enable privacy protection for the contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastName</i> Last name of contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FirstName</i> First name of contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExtraParams</i> A list of name-value pairs for parameters required by certain top-level domains.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Fax</i> Fax number of the contact. Constraints: Phone number must be specified in the format "+[country dialing code].[number including any area code]". For example, a US phone number might appear as <code>"+1.1234567890"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Email</i> Email address of the contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneNumber</i> The phone number of the contact. Constraints: Phone number must be specified in the format "+[country dialing code].[number including any area code&gt;]". For example, a US phone number might appear as <code>"+1.1234567890"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ZipCode</i> The zip or postal code of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CountryCode</i> Code for the country of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state or province of the contact's city.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>City</i> The city of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressLine2</i> Second line of contact's address, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressLine1</i> First line of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationName</i> Name of the organization for contact types other than <code>PERSON</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContactType</i> Indicates whether the contact is a person, company, association, or public organization. If you choose an option other than <code>PERSON</code>, you must enter an organization name, and you can't enable privacy protection for the contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastName</i> Last name of contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FirstName</i> First name of contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExtraParams</i> A list of name-value pairs for parameters required by certain top-level domains.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Fax</i> Fax number of the contact. Constraints: Phone number must be specified in the format "+[country dialing code].[number including any area code]". For example, a US phone number might appear as <code>"+1.1234567890"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Email</i> Email address of the contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneNumber</i> The phone number of the contact. Constraints: Phone number must be specified in the format "+[country dialing code].[number including any area code&gt;]". For example, a US phone number might appear as <code>"+1.1234567890"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ZipCode</i> The zip or postal code of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CountryCode</i> Code for the country of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state or province of the contact's city.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>City</i> The city of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressLine2</i> Second line of contact's address, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressLine1</i> First line of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationName</i> Name of the organization for contact types other than <code>PERSON</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContactType</i> Indicates whether the contact is a person, company, association, or public organization. If you choose an option other than <code>PERSON</code>, you must enter an organization name, and you can't enable privacy protection for the contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastName</i> Last name of contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FirstName</i> First name of contact.<br/>        """
        pass

    def get_domain_suggestions(self, OnlyAvailable: Boolean, SuggestionCount: int, DomainName: str) -> GetDomainSuggestionsResponse:
        """The GetDomainSuggestions operation returns a list of suggested domain names given a string, which can either be a domain name or simply a word or phrase (without spaces).

        :param self:
        :param OnlyAvailable:  If <code>OnlyAvailable</code> is <code>true</code>, Amazon Route 53 returns only domain names that are available. If <code>OnlyAvailable</code> is <code>false</code>, Amazon Route 53 returns domain names without checking whether they're available to be registered. To determine whether the domain is available, you can call <code>checkDomainAvailability</code> for each suggestion.<br/>SuggestionCount:  The number of suggested domain names that you want Amazon Route 53 to return.<br/>DomainName:  A domain name that you want to use as the basis for a list of possible domain names. The domain name must contain a top-level domain (TLD), such as .com, that Amazon Route 53 supports. For a list of TLDs, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/registrar-tld-list.html">Domains that You Can Register with Amazon Route 53</a> in the <i>Amazon Route 53 Developer Guide</i>.<br/>
        :return: SuggestionsList:  A list of possible domain names. If you specified <code>true</code> for <code>OnlyAvailable</code> in the request, the list contains only domains that are available for registration.<br/>        """
        pass

    def get_operation_detail(self, OperationId: str) -> GetOperationDetailResponse:
        """This operation returns the current status of an operation that is not completed.

        :param self:
        :param OperationId:  The identifier for the operation for which you want to get the status. Amazon Route 53 returned the identifier in the response to the original request.<br/>
        :return: SubmittedDate:  The date when the request was submitted.<br/>Type:  The type of operation that was requested.<br/>DomainName:  The name of a domain.<br/>Message:  Detailed information on the status including possible errors.<br/>Status:  The current status of the requested operation in the system.<br/>OperationId:  The identifier for the operation.<br/>        """
        pass

    def list_domains(self, Marker: str=None, MaxItems: int=None) -> ListDomainsResponse:
        """This operation returns all the domain names registered with Amazon Route 53 for the current AWS account.

        :param self:
        :param MaxItems:  Number of domains to be returned. Default: 20<br/>Marker:  For an initial request for a list of domains, omit this element. If the number of domains that are associated with the current AWS account is greater than the value that you specified for <code>MaxItems</code>, you can use <code>Marker</code> to return additional domains. Get the value of <code>NextPageMarker</code> from the previous response, and submit another request that includes the value of <code>NextPageMarker</code> in the <code>Marker</code> element. Constraints: The marker must match the value specified in the previous request.<br/>
        :return: NextPageMarker:  If there are more domains than you specified for <code>MaxItems</code> in the request, submit another request and include the value of <code>NextPageMarker</code> in the value of <code>Marker</code>.<br/>Domains:  A summary of domains.<br/>        """
        pass

    def list_operations(self, SubmittedSince: Timestamp=None, Marker: str=None, MaxItems: int=None) -> ListOperationsResponse:
        """This operation returns the operation IDs of operations that are not yet complete.

        :param self:
        :param MaxItems:  Number of domains to be returned. Default: 20<br/>Marker:  For an initial request for a list of operations, omit this element. If the number of operations that are not yet complete is greater than the value that you specified for <code>MaxItems</code>, you can use <code>Marker</code> to return additional operations. Get the value of <code>NextPageMarker</code> from the previous response, and submit another request that includes the value of <code>NextPageMarker</code> in the <code>Marker</code> element.<br/>SubmittedSince:  An optional parameter that lets you get information about all the operations that you submitted after a specified date and time. Specify the date and time in Coordinated Universal time (UTC).<br/>
        :return: NextPageMarker:  If there are more operations than you specified for <code>MaxItems</code> in the request, submit another request and include the value of <code>NextPageMarker</code> in the value of <code>Marker</code>.<br/>Operations:  Lists summaries of the operations.<br/>        """
        pass

    def list_tags_for_domain(self, DomainName: str) -> ListTagsForDomainResponse:
        """This operation returns all of the tags that are associated with the specified domain. All tag operations are eventually consistent; subsequent operations might not immediately represent all issued operations.

        :param self:
        :param DomainName:  The domain for which you want to get a list of tags.<br/>
        :return: TagList:  A list of the tags that are associated with the specified domain.<br/>        """
        pass

    def register_domain(self, TechContact: ContactDetail, RegistrantContact: ContactDetail, AdminContact: ContactDetail, DurationInYears: int, DomainName: str, IdnLangCode: str=None, AutoRenew: Boolean=None, PrivacyProtectAdminContact: Boolean=None, PrivacyProtectRegistrantContact: Boolean=None, PrivacyProtectTechContact: Boolean=None) -> RegisterDomainResponse:
        """This operation registers a domain. Domains are registered either by Amazon Registrar (for .com, .net, and .org domains) or by our registrar associate, Gandi (for all other domains). For some top-level domains (TLDs), this operation requires extra parameters. When you register a domain, Amazon Route 53 does the following: <ul> <li> Creates a Amazon Route 53 hosted zone that has the same name as the domain. Amazon Route 53 assigns four name servers to your hosted zone and automatically updates your domain registration with the names of these name servers. </li> <li> Enables autorenew, so your domain registration will renew automatically each year. We'll notify you in advance of the renewal date so you can choose whether to renew the registration. </li> <li> Optionally enables privacy protection, so WHOIS queries return contact information either for Amazon Registrar (for .com, .net, and .org domains) or for our registrar associate, Gandi (for all other TLDs). If you don't enable privacy protection, WHOIS queries return the information that you entered for the registrant, admin, and tech contacts. </li> <li> If registration is successful, returns an operation ID that you can use to track the progress and completion of the action. If the request is not completed successfully, the domain registrant is notified by email. </li> <li> Charges your AWS account an amount based on the top-level domain. For more information, see <a href="http://aws.amazon.com/route53/pricing/">Amazon Route 53 Pricing</a>. </li> </ul>

        :param self:
        :param PrivacyProtectTechContact:  Whether you want to conceal contact information from WHOIS queries. If you specify <code>true</code>, WHOIS ("who is") queries return contact information either for Amazon Registrar (for .com, .net, and .org domains) or for our registrar associate, Gandi (for all other TLDs). If you specify <code>false</code>, WHOIS queries return the information that you entered for the technical contact. Default: <code>true</code> <br/>PrivacyProtectRegistrantContact:  Whether you want to conceal contact information from WHOIS queries. If you specify <code>true</code>, WHOIS ("who is") queries return contact information either for Amazon Registrar (for .com, .net, and .org domains) or for our registrar associate, Gandi (for all other TLDs). If you specify <code>false</code>, WHOIS queries return the information that you entered for the registrant contact (the domain owner). Default: <code>true</code> <br/>PrivacyProtectAdminContact:  Whether you want to conceal contact information from WHOIS queries. If you specify <code>true</code>, WHOIS ("who is") queries return contact information either for Amazon Registrar (for .com, .net, and .org domains) or for our registrar associate, Gandi (for all other TLDs). If you specify <code>false</code>, WHOIS queries return the information that you entered for the admin contact. Default: <code>true</code> <br/>TechContact:  Provides detailed contact information.:<br/>RegistrantContact:  Provides detailed contact information.:<br/>AdminContact:  Provides detailed contact information.:<br/>AutoRenew:  Indicates whether the domain will be automatically renewed (<code>true</code>) or not (<code>false</code>). Autorenewal only takes effect after the account is charged. Default: <code>true</code> <br/>DurationInYears:  The number of years that you want to register the domain for. Domains are registered for a minimum of one year. The maximum period depends on the top-level domain. For the range of valid values for your domain, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/registrar-tld-list.html">Domains that You Can Register with Amazon Route 53</a> in the <i>Amazon Route 53 Developer Guide</i>. Default: 1<br/>IdnLangCode:  Reserved for future use.<br/>DomainName:  The domain name that you want to register. Constraints: The domain name can contain only the letters a through z, the numbers 0 through 9, and hyphen (-). Internationalized Domain Names are not supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExtraParams</i> A list of name-value pairs for parameters required by certain top-level domains.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Fax</i> Fax number of the contact. Constraints: Phone number must be specified in the format "+[country dialing code].[number including any area code]". For example, a US phone number might appear as <code>"+1.1234567890"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Email</i> Email address of the contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneNumber</i> The phone number of the contact. Constraints: Phone number must be specified in the format "+[country dialing code].[number including any area code&gt;]". For example, a US phone number might appear as <code>"+1.1234567890"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ZipCode</i> The zip or postal code of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CountryCode</i> Code for the country of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state or province of the contact's city.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>City</i> The city of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressLine2</i> Second line of contact's address, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressLine1</i> First line of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationName</i> Name of the organization for contact types other than <code>PERSON</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContactType</i> Indicates whether the contact is a person, company, association, or public organization. If you choose an option other than <code>PERSON</code>, you must enter an organization name, and you can't enable privacy protection for the contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastName</i> Last name of contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FirstName</i> First name of contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExtraParams</i> A list of name-value pairs for parameters required by certain top-level domains.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Fax</i> Fax number of the contact. Constraints: Phone number must be specified in the format "+[country dialing code].[number including any area code]". For example, a US phone number might appear as <code>"+1.1234567890"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Email</i> Email address of the contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneNumber</i> The phone number of the contact. Constraints: Phone number must be specified in the format "+[country dialing code].[number including any area code&gt;]". For example, a US phone number might appear as <code>"+1.1234567890"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ZipCode</i> The zip or postal code of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CountryCode</i> Code for the country of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state or province of the contact's city.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>City</i> The city of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressLine2</i> Second line of contact's address, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressLine1</i> First line of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationName</i> Name of the organization for contact types other than <code>PERSON</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContactType</i> Indicates whether the contact is a person, company, association, or public organization. If you choose an option other than <code>PERSON</code>, you must enter an organization name, and you can't enable privacy protection for the contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastName</i> Last name of contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FirstName</i> First name of contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExtraParams</i> A list of name-value pairs for parameters required by certain top-level domains.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Fax</i> Fax number of the contact. Constraints: Phone number must be specified in the format "+[country dialing code].[number including any area code]". For example, a US phone number might appear as <code>"+1.1234567890"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Email</i> Email address of the contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneNumber</i> The phone number of the contact. Constraints: Phone number must be specified in the format "+[country dialing code].[number including any area code&gt;]". For example, a US phone number might appear as <code>"+1.1234567890"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ZipCode</i> The zip or postal code of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CountryCode</i> Code for the country of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state or province of the contact's city.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>City</i> The city of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressLine2</i> Second line of contact's address, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressLine1</i> First line of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationName</i> Name of the organization for contact types other than <code>PERSON</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContactType</i> Indicates whether the contact is a person, company, association, or public organization. If you choose an option other than <code>PERSON</code>, you must enter an organization name, and you can't enable privacy protection for the contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastName</i> Last name of contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FirstName</i> First name of contact.<br/>
        :return: OperationId:  Identifier for tracking the progress of the request. To use this ID to query the operation status, use <a>GetOperationDetail</a>.<br/>        """
        pass

    def renew_domain(self, CurrentExpiryYear: int, DomainName: str, DurationInYears: int=None) -> RenewDomainResponse:
        """This operation renews a domain for the specified number of years. The cost of renewing your domain is billed to your AWS account. We recommend that you renew your domain several weeks before the expiration date. Some TLD registries delete domains before the expiration date if you haven't renewed far enough in advance. For more information about renewing domain registration, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/domain-renew.html">Renewing Registration for a Domain</a> in the Amazon Route 53 Developer Guide.

        :param self:
        :param CurrentExpiryYear:  The year when the registration for the domain is set to expire. This value must match the current expiration date for the domain.<br/>DurationInYears:  The number of years that you want to renew the domain for. The maximum number of years depends on the top-level domain. For the range of valid values for your domain, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/registrar-tld-list.html">Domains that You Can Register with Amazon Route 53</a> in the <i>Amazon Route 53 Developer Guide</i>. Default: 1<br/>DomainName:  The name of the domain that you want to renew.<br/>
        :return: OperationId:  The identifier for tracking the progress of the request. To use this ID to query the operation status, use <a>GetOperationDetail</a>.<br/>        """
        pass

    def resend_contact_reachability_email(self, domainName: str=None) -> ResendContactReachabilityEmailResponse:
        """For operations that require confirmation that the email address for the registrant contact is valid, such as registering a new domain, this operation resends the confirmation email to the current email address for the registrant contact.

        :param self:
        :param domainName:  The name of the domain for which you want Amazon Route 53 to resend a confirmation email to the registrant contact.<br/>
        :return: isAlreadyVerified:   <code>True</code> if the email address for the registrant contact has already been verified, and <code>false</code> otherwise. If the email address has already been verified, we don't send another confirmation email.<br/>emailAddress:  The email address for the registrant contact at the time that we sent the verification email.<br/>domainName:  The domain name for which you requested a confirmation email.<br/>        """
        pass

    def retrieve_domain_auth_code(self, DomainName: str) -> RetrieveDomainAuthCodeResponse:
        """This operation returns the AuthCode for the domain. To transfer a domain to another registrar, you provide this value to the new registrar.

        :param self:
        :param DomainName:  The name of the domain that you want to get an authorization code for.<br/>
        :return: AuthCode:  The authorization code for the domain.<br/>        """
        pass

    def transfer_domain(self, TechContact: ContactDetail, RegistrantContact: ContactDetail, AdminContact: ContactDetail, DurationInYears: int, DomainName: str, IdnLangCode: str=None, Nameservers: List[Nameserver]=None, AuthCode: str=None, AutoRenew: Boolean=None, PrivacyProtectAdminContact: Boolean=None, PrivacyProtectRegistrantContact: Boolean=None, PrivacyProtectTechContact: Boolean=None) -> TransferDomainResponse:
        """This operation transfers a domain from another registrar to Amazon Route 53. When the transfer is complete, the domain is registered either with Amazon Registrar (for .com, .net, and .org domains) or with our registrar associate, Gandi (for all other TLDs). For transfer requirements, a detailed procedure, and information about viewing the status of a domain transfer, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/domain-transfer-to-route-53.html">Transferring Registration for a Domain to Amazon Route 53</a> in the <i>Amazon Route 53 Developer Guide</i>. If the registrar for your domain is also the DNS service provider for the domain, we highly recommend that you consider transferring your DNS service to Amazon Route 53 or to another DNS service provider before you transfer your registration. Some registrars provide free DNS service when you purchase a domain registration. When you transfer the registration, the previous registrar will not renew your domain registration and could end your DNS service at any time. <important> If the registrar for your domain is also the DNS service provider for the domain and you don't transfer DNS service to another provider, your website, email, and the web applications associated with the domain might become unavailable. </important> If the transfer is successful, this method returns an operation ID that you can use to track the progress and completion of the action. If the transfer doesn't complete successfully, the domain registrant will be notified by email.

        :param self:
        :param PrivacyProtectTechContact:  Whether you want to conceal contact information from WHOIS queries. If you specify <code>true</code>, WHOIS ("who is") queries return contact information either for Amazon Registrar (for .com, .net, and .org domains) or for our registrar associate, Gandi (for all other TLDs). If you specify <code>false</code>, WHOIS queries return the information that you entered for the technical contact. Default: <code>true</code> <br/>PrivacyProtectRegistrantContact:  Whether you want to conceal contact information from WHOIS queries. If you specify <code>true</code>, WHOIS ("who is") queries return contact information either for Amazon Registrar (for .com, .net, and .org domains) or for our registrar associate, Gandi (for all other TLDs). If you specify <code>false</code>, WHOIS queries return the information that you entered for the registrant contact (domain owner). Default: <code>true</code> <br/>PrivacyProtectAdminContact:  Whether you want to conceal contact information from WHOIS queries. If you specify <code>true</code>, WHOIS ("who is") queries return contact information either for Amazon Registrar (for .com, .net, and .org domains) or for our registrar associate, Gandi (for all other TLDs). If you specify <code>false</code>, WHOIS queries return the information that you entered for the admin contact. Default: <code>true</code> <br/>TechContact:  Provides detailed contact information.:<br/>RegistrantContact:  Provides detailed contact information.:<br/>AdminContact:  Provides detailed contact information.:<br/>AutoRenew:  Indicates whether the domain will be automatically renewed (true) or not (false). Autorenewal only takes effect after the account is charged. Default: true<br/>AuthCode:  The authorization code for the domain. You get this value from the current registrar.<br/>Nameservers:  Contains details for the host and glue IP addresses.<br/>DurationInYears:  The number of years that you want to register the domain for. Domains are registered for a minimum of one year. The maximum period depends on the top-level domain. Default: 1<br/>IdnLangCode:  Reserved for future use.<br/>DomainName:  The name of the domain that you want to transfer to Amazon Route 53. Constraints: The domain name can contain only the letters a through z, the numbers 0 through 9, and hyphen (-). Internationalized Domain Names are not supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExtraParams</i> A list of name-value pairs for parameters required by certain top-level domains.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Fax</i> Fax number of the contact. Constraints: Phone number must be specified in the format "+[country dialing code].[number including any area code]". For example, a US phone number might appear as <code>"+1.1234567890"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Email</i> Email address of the contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneNumber</i> The phone number of the contact. Constraints: Phone number must be specified in the format "+[country dialing code].[number including any area code&gt;]". For example, a US phone number might appear as <code>"+1.1234567890"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ZipCode</i> The zip or postal code of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CountryCode</i> Code for the country of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state or province of the contact's city.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>City</i> The city of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressLine2</i> Second line of contact's address, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressLine1</i> First line of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationName</i> Name of the organization for contact types other than <code>PERSON</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContactType</i> Indicates whether the contact is a person, company, association, or public organization. If you choose an option other than <code>PERSON</code>, you must enter an organization name, and you can't enable privacy protection for the contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastName</i> Last name of contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FirstName</i> First name of contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExtraParams</i> A list of name-value pairs for parameters required by certain top-level domains.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Fax</i> Fax number of the contact. Constraints: Phone number must be specified in the format "+[country dialing code].[number including any area code]". For example, a US phone number might appear as <code>"+1.1234567890"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Email</i> Email address of the contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneNumber</i> The phone number of the contact. Constraints: Phone number must be specified in the format "+[country dialing code].[number including any area code&gt;]". For example, a US phone number might appear as <code>"+1.1234567890"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ZipCode</i> The zip or postal code of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CountryCode</i> Code for the country of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state or province of the contact's city.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>City</i> The city of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressLine2</i> Second line of contact's address, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressLine1</i> First line of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationName</i> Name of the organization for contact types other than <code>PERSON</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContactType</i> Indicates whether the contact is a person, company, association, or public organization. If you choose an option other than <code>PERSON</code>, you must enter an organization name, and you can't enable privacy protection for the contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastName</i> Last name of contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FirstName</i> First name of contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExtraParams</i> A list of name-value pairs for parameters required by certain top-level domains.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Fax</i> Fax number of the contact. Constraints: Phone number must be specified in the format "+[country dialing code].[number including any area code]". For example, a US phone number might appear as <code>"+1.1234567890"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Email</i> Email address of the contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneNumber</i> The phone number of the contact. Constraints: Phone number must be specified in the format "+[country dialing code].[number including any area code&gt;]". For example, a US phone number might appear as <code>"+1.1234567890"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ZipCode</i> The zip or postal code of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CountryCode</i> Code for the country of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state or province of the contact's city.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>City</i> The city of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressLine2</i> Second line of contact's address, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressLine1</i> First line of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationName</i> Name of the organization for contact types other than <code>PERSON</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContactType</i> Indicates whether the contact is a person, company, association, or public organization. If you choose an option other than <code>PERSON</code>, you must enter an organization name, and you can't enable privacy protection for the contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastName</i> Last name of contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FirstName</i> First name of contact.<br/>
        :return: OperationId:  Identifier for tracking the progress of the request. To use this ID to query the operation status, use <a>GetOperationDetail</a>.<br/>        """
        pass

    def update_domain_contact(self, DomainName: str, AdminContact: ContactDetail=None, RegistrantContact: ContactDetail=None, TechContact: ContactDetail=None) -> UpdateDomainContactResponse:
        """This operation updates the contact information for a particular domain. You must specify information for at least one contact: registrant, administrator, or technical. If the update is successful, this method returns an operation ID that you can use to track the progress and completion of the action. If the request is not completed successfully, the domain registrant will be notified by email.

        :param self:
        :param TechContact:  Provides detailed contact information.:<br/>RegistrantContact:  Provides detailed contact information.:<br/>AdminContact:  Provides detailed contact information.:<br/>DomainName:  The name of the domain that you want to update contact information for.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExtraParams</i> A list of name-value pairs for parameters required by certain top-level domains.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Fax</i> Fax number of the contact. Constraints: Phone number must be specified in the format "+[country dialing code].[number including any area code]". For example, a US phone number might appear as <code>"+1.1234567890"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Email</i> Email address of the contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneNumber</i> The phone number of the contact. Constraints: Phone number must be specified in the format "+[country dialing code].[number including any area code&gt;]". For example, a US phone number might appear as <code>"+1.1234567890"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ZipCode</i> The zip or postal code of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CountryCode</i> Code for the country of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state or province of the contact's city.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>City</i> The city of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressLine2</i> Second line of contact's address, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressLine1</i> First line of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationName</i> Name of the organization for contact types other than <code>PERSON</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContactType</i> Indicates whether the contact is a person, company, association, or public organization. If you choose an option other than <code>PERSON</code>, you must enter an organization name, and you can't enable privacy protection for the contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastName</i> Last name of contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FirstName</i> First name of contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExtraParams</i> A list of name-value pairs for parameters required by certain top-level domains.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Fax</i> Fax number of the contact. Constraints: Phone number must be specified in the format "+[country dialing code].[number including any area code]". For example, a US phone number might appear as <code>"+1.1234567890"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Email</i> Email address of the contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneNumber</i> The phone number of the contact. Constraints: Phone number must be specified in the format "+[country dialing code].[number including any area code&gt;]". For example, a US phone number might appear as <code>"+1.1234567890"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ZipCode</i> The zip or postal code of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CountryCode</i> Code for the country of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state or province of the contact's city.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>City</i> The city of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressLine2</i> Second line of contact's address, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressLine1</i> First line of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationName</i> Name of the organization for contact types other than <code>PERSON</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContactType</i> Indicates whether the contact is a person, company, association, or public organization. If you choose an option other than <code>PERSON</code>, you must enter an organization name, and you can't enable privacy protection for the contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastName</i> Last name of contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FirstName</i> First name of contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExtraParams</i> A list of name-value pairs for parameters required by certain top-level domains.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Fax</i> Fax number of the contact. Constraints: Phone number must be specified in the format "+[country dialing code].[number including any area code]". For example, a US phone number might appear as <code>"+1.1234567890"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Email</i> Email address of the contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneNumber</i> The phone number of the contact. Constraints: Phone number must be specified in the format "+[country dialing code].[number including any area code&gt;]". For example, a US phone number might appear as <code>"+1.1234567890"</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ZipCode</i> The zip or postal code of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CountryCode</i> Code for the country of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The state or province of the contact's city.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>City</i> The city of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressLine2</i> Second line of contact's address, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressLine1</i> First line of the contact's address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationName</i> Name of the organization for contact types other than <code>PERSON</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContactType</i> Indicates whether the contact is a person, company, association, or public organization. If you choose an option other than <code>PERSON</code>, you must enter an organization name, and you can't enable privacy protection for the contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastName</i> Last name of contact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FirstName</i> First name of contact.<br/>
        :return: OperationId:  Identifier for tracking the progress of the request. To use this ID to query the operation status, use <a>GetOperationDetail</a>.<br/>        """
        pass

    def update_domain_contact_privacy(self, DomainName: str, AdminPrivacy: Boolean=None, RegistrantPrivacy: Boolean=None, TechPrivacy: Boolean=None) -> UpdateDomainContactPrivacyResponse:
        """This operation updates the specified domain contact's privacy setting. When privacy protection is enabled, contact information such as email address is replaced either with contact information for Amazon Registrar (for .com, .net, and .org domains) or with contact information for our registrar associate, Gandi. This operation affects only the contact information for the specified contact type (registrant, administrator, or tech). If the request succeeds, Amazon Route 53 returns an operation ID that you can use with <a>GetOperationDetail</a> to track the progress and completion of the action. If the request doesn't complete successfully, the domain registrant will be notified by email.

        :param self:
        :param TechPrivacy:  Whether you want to conceal contact information from WHOIS queries. If you specify <code>true</code>, WHOIS ("who is") queries return contact information either for Amazon Registrar (for .com, .net, and .org domains) or for our registrar associate, Gandi (for all other TLDs). If you specify <code>false</code>, WHOIS queries return the information that you entered for the technical contact.<br/>RegistrantPrivacy:  Whether you want to conceal contact information from WHOIS queries. If you specify <code>true</code>, WHOIS ("who is") queries return contact information either for Amazon Registrar (for .com, .net, and .org domains) or for our registrar associate, Gandi (for all other TLDs). If you specify <code>false</code>, WHOIS queries return the information that you entered for the registrant contact (domain owner).<br/>AdminPrivacy:  Whether you want to conceal contact information from WHOIS queries. If you specify <code>true</code>, WHOIS ("who is") queries return contact information either for Amazon Registrar (for .com, .net, and .org domains) or for our registrar associate, Gandi (for all other TLDs). If you specify <code>false</code>, WHOIS queries return the information that you entered for the admin contact.<br/>DomainName:  The name of the domain that you want to update the privacy setting for.<br/>
        :return: OperationId:  Identifier for tracking the progress of the request. To use this ID to query the operation status, use GetOperationDetail.<br/>        """
        pass

    def update_domain_nameservers(self, Nameservers: List[Nameserver], DomainName: str, FIAuthKey: str=None) -> UpdateDomainNameserversResponse:
        """This operation replaces the current set of name servers for the domain with the specified set of name servers. If you use Amazon Route 53 as your DNS service, specify the four name servers in the delegation set for the hosted zone for the domain. If successful, this operation returns an operation ID that you can use to track the progress and completion of the action. If the request is not completed successfully, the domain registrant will be notified by email.

        :param self:
        :param Nameservers:  A list of new name servers for the domain.<br/>FIAuthKey:  The authorization key for .fi domains<br/>DomainName:  The name of the domain that you want to change name servers for.<br/>
        :return: OperationId:  Identifier for tracking the progress of the request. To use this ID to query the operation status, use <a>GetOperationDetail</a>.<br/>        """
        pass

    def update_tags_for_domain(self, DomainName: str, TagsToUpdate: List[Tag]=None) -> UpdateTagsForDomainResponse:
        """This operation adds or updates tags for a specified domain. All tag operations are eventually consistent; subsequent operations might not immediately represent all issued operations.

        :param self:
        :param TagsToUpdate:  A list of the tag keys and values that you want to add or update. If you specify a key that already exists, the corresponding value will be replaced.<br/>DomainName:  The domain for which you want to add or update tags.<br/>
        :return:         """
        pass

    def view_billing(self, Start: Timestamp=None, End: Timestamp=None, Marker: str=None, MaxItems: int=None) -> ViewBillingResponse:
        """Returns all the domain-related billing records for the current AWS account for a specified period

        :param self:
        :param MaxItems:  The number of billing records to be returned. Default: 20<br/>Marker:  For an initial request for a list of billing records, omit this element. If the number of billing records that are associated with the current AWS account during the specified period is greater than the value that you specified for <code>MaxItems</code>, you can use <code>Marker</code> to return additional billing records. Get the value of <code>NextPageMarker</code> from the previous response, and submit another request that includes the value of <code>NextPageMarker</code> in the <code>Marker</code> element.  Constraints: The marker must match the value of <code>NextPageMarker</code> that was returned in the previous response.<br/>End:  The end date and time for the time period for which you want a list of billing records. Specify the date and time in Coordinated Universal time (UTC).<br/>Start:  The beginning date and time for the time period for which you want a list of billing records. Specify the date and time in Coordinated Universal time (UTC).<br/>
        :return: BillingRecords:  A summary of billing records.<br/>NextPageMarker:  If there are more billing records than you specified for <code>MaxItems</code> in the request, submit another request and include the value of <code>NextPageMarker</code> in the value of <code>Marker</code>.<br/>        """
        pass



class Route53Resolver(BaseClient):
    """Here's how you set up to query an Amazon Route 53 private hosted zone from your network: <ol> <li> Connect your network to a VPC using AWS Direct Connect or a VPN. </li> <li> Run the following AWS CLI command to create a Resolver endpoint:  <code>create-resolver-endpoint --name [endpoint_name] --direction INBOUND --creator-request-id [unique_string] --security-group-ids [security_group_with_inbound_rules] --ip-addresses SubnetId=[subnet_id] SubnetId=[subnet_id_in_different_AZ]</code>  Note the resolver endpoint ID that appears in the response. You'll use it in step 3. </li> <li> Get the IP addresses for the Resolver endpoints:  <code>get-resolver-endpoint --resolver-endpoint-id [resolver_endpoint_id]</code>  </li> <li> In your network configuration, define the IP addresses that you got in step 3 as DNS servers. You can now query instance names in your VPCs and the names of records in your private hosted zone. </li> </ol> You can also perform the following operations using the AWS CLI: <ul> <li>  <code>list-resolver-endpoints</code>: List all endpoints. The syntax includes options for pagination and filtering. </li> <li>  <code>update-resolver-endpoints</code>: Add IP addresses to an endpoint or remove IP addresses from an endpoint.  </li> </ul> To delete an endpoint, use the following AWS CLI command:  <code>delete-resolver-endpoint --resolver-endpoint-id [resolver_endpoint_id]</code> """


    class ListTagsForResourcePaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_tags_for_resource` for valid parameters"""
            pass


    class AssociateResolverEndpointIpAddressRequest(Mapping):
        pass
    
    class AssociateResolverEndpointIpAddressResponse(Mapping):
        pass
    
    class IpAddressUpdate(Mapping):
        pass
    
    class AssociateResolverRuleRequest(Mapping):
        pass
    
    class AssociateResolverRuleResponse(Mapping):
        pass
    
    class CreateResolverEndpointRequest(Mapping):
        pass
    
    class CreateResolverEndpointResponse(Mapping):
        pass
    
    class ResourceId(object):
        pass
    
    class IpAddressRequest(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreateResolverRuleRequest(Mapping):
        pass
    
    class CreateResolverRuleResponse(Mapping):
        pass
    
    class TargetAddress(Mapping):
        pass
    
    class DeleteResolverEndpointRequest(Mapping):
        pass
    
    class DeleteResolverEndpointResponse(Mapping):
        pass
    
    class DeleteResolverRuleRequest(Mapping):
        pass
    
    class DeleteResolverRuleResponse(Mapping):
        pass
    
    class DisassociateResolverEndpointIpAddressRequest(Mapping):
        pass
    
    class DisassociateResolverEndpointIpAddressResponse(Mapping):
        pass
    
    class DisassociateResolverRuleRequest(Mapping):
        pass
    
    class DisassociateResolverRuleResponse(Mapping):
        pass
    
    class GetResolverEndpointRequest(Mapping):
        pass
    
    class GetResolverEndpointResponse(Mapping):
        pass
    
    class GetResolverRuleRequest(Mapping):
        pass
    
    class GetResolverRuleResponse(Mapping):
        pass
    
    class GetResolverRuleAssociationRequest(Mapping):
        pass
    
    class GetResolverRuleAssociationResponse(Mapping):
        pass
    
    class GetResolverRulePolicyRequest(Mapping):
        pass
    
    class GetResolverRulePolicyResponse(Mapping):
        pass
    
    class ListResolverEndpointIpAddressesRequest(Mapping):
        pass
    
    class ListResolverEndpointIpAddressesResponse(Mapping):
        pass
    
    class ListResolverEndpointsRequest(Mapping):
        pass
    
    class ListResolverEndpointsResponse(Mapping):
        pass
    
    class Filter(Mapping):
        pass
    
    class ListResolverRuleAssociationsRequest(Mapping):
        pass
    
    class ListResolverRuleAssociationsResponse(Mapping):
        pass
    
    class ListResolverRulesRequest(Mapping):
        pass
    
    class ListResolverRulesResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class PutResolverRulePolicyRequest(Mapping):
        pass
    
    class PutResolverRulePolicyResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateResolverEndpointRequest(Mapping):
        pass
    
    class UpdateResolverEndpointResponse(Mapping):
        pass
    
    class UpdateResolverRuleRequest(Mapping):
        pass
    
    class UpdateResolverRuleResponse(Mapping):
        pass
    
    class ResolverRuleConfig(Mapping):
        pass
    
    def associate_resolver_endpoint_ip_address(self, IpAddress: IpAddressUpdate, ResolverEndpointId: str) -> AssociateResolverEndpointIpAddressResponse:
        """Adds IP addresses to an inbound or an outbound resolver endpoint. If you want to adding more than one IP address, submit one <code>AssociateResolverEndpointIpAddress</code> request for each IP address. To remove an IP address from an endpoint, see <a>DisassociateResolverEndpointIpAddress</a>.

        :param self:
        :param IpAddress:  Either the IPv4 address that you want to add to a resolver endpoint or a subnet ID. If you specify a subnet ID, Resolver chooses an IP address for you from the available IPs in the specified subnet.:<br/>ResolverEndpointId:  The ID of the resolver endpoint that you want to associate IP addresses with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Ip</i> The new IP address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetId</i> The ID of the subnet that includes the IP address that you want to update. To get this ID, use <a>GetResolverEndpoint</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IpId</i>  <i>Only when removing an IP address from a resolver endpoint</i>: The ID of the IP address that you want to remove. To get this ID, use <a>GetResolverEndpoint</a>.<br/>
        :return: ResolverEndpoint:  The response to an <code>AssociateResolverEndpointIpAddress</code> request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModificationTime</i> The date and time that the endpoint was last modified, in Unix time format and Coordinated Universal Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The date and time that the endpoint was created, in Unix time format and Coordinated Universal Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> A detailed description of the status of the resolver endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> A code that specifies the current status of the resolver endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostVPCId</i> The ID of the VPC that you want to create the resolver endpoint in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IpAddressCount</i> The number of IP addresses that the resolver endpoint can use for DNS queries.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Direction</i> Indicates whether the resolver endpoint allows inbound or outbound DNS queries: <ul> <li>  <code>INBOUND</code>: allows DNS queries to your VPC from your network or another VPC </li> <li>  <code>OUTBOUND</code>: allows DNS queries from your VPC to your network or another VPC </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The ID of one or more security groups that control access to this VPC. The security group must include one or more inbound resolver rules.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name that you assigned to the resolver endpoint when you submitted a <a>CreateResolverEndpoint</a> request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN (Amazon Resource Name) for the resolver endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatorRequestId</i> A unique string that identifies the request that created the resolver endpoint. The <code>CreatorRequestId</code> allows failed requests to be retried without the risk of executing the operation twice.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the resolver endpoint.<br/>        """
        pass

    def associate_resolver_rule(self, VPCId: str, ResolverRuleId: str, Name: str=None) -> AssociateResolverRuleResponse:
        """Associates a resolver rule with a VPC. When you associate a rule with a VPC, Resolver forwards all DNS queries for the domain name that is specified in the rule and that originate in the VPC. The queries are forwarded to the IP addresses for the DNS resolvers that are specified in the rule. For more information about rules, see <a>CreateResolverRule</a>. 

        :param self:
        :param VPCId:  The ID of the VPC that you want to associate the resolver rule with.<br/>Name:  A name for the association that you're creating between a resolver rule and a VPC.<br/>ResolverRuleId:  The ID of the resolver rule that you want to associate with the VPC. To list the existing resolver rules, use <a>ListResolverRules</a>.<br/>
        :return: ResolverRuleAssociation:  Information about the <code>AssociateResolverRule</code> request, including the status of the request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> A detailed description of the status of the association between a resolver rule and a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> A code that specifies the current status of the association between a resolver rule and a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VPCId</i> The ID of the VPC that you associated the resolver rule with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of an association between a resolver rule and a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResolverRuleId</i> The ID of the resolver rule that you associated with the VPC that is specified by <code>VPCId</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the association between a resolver rule and a VPC. Resolver assigns this value when you submit an <a>AssociateResolverRule</a> request.<br/>        """
        pass

    def create_resolver_endpoint(self, IpAddresses: List[IpAddressRequest], Direction: str, SecurityGroupIds: List[ResourceId], CreatorRequestId: str, Name: str=None, Tags: List[Tag]=None) -> CreateResolverEndpointResponse:
        """Creates a resolver endpoint. There are two types of resolver endpoints, inbound and outbound: <ul> <li> An <i>inbound resolver endpoint</i> forwards DNS queries to the DNS service for a VPC from your network or another VPC. </li> <li> An <i>outbound resolver endpoint</i> forwards DNS queries from the DNS service for a VPC to your network or another VPC. </li> </ul>

        :param self:
        :param Tags:  A list of the tag keys and values that you want to associate with the endpoint.<br/>IpAddresses:  The subnets and IP addresses in your VPC that you want DNS queries to pass through on the way from your VPCs to your network (for outbound endpoints) or on the way from your network to your VPCs (for inbound resolver endpoints). <br/>Direction:  Specify the applicable value: <ul> <li>  <code>INBOUND</code>: Resolver forwards DNS queries to the DNS service for a VPC from your network or another VPC </li> <li>  <code>OUTBOUND</code>: Resolver forwards DNS queries from the DNS service for a VPC to your network or another VPC </li> </ul><br/>SecurityGroupIds:  The ID of one or more security groups that you want to use to control access to this VPC. The security group that you specify must include one or more inbound rules (for inbound resolver endpoints) or outbound rules (for outbound resolver endpoints).<br/>Name:  A friendly name that lets you easily find a configuration in the Resolver dashboard in the Route 53 console.<br/>CreatorRequestId:  A unique string that identifies the request and that allows failed requests to be retried without the risk of executing the operation twice. <code>CreatorRequestId</code> can be any unique string, for example, a date/time stamp. <br/>
        :return: ResolverEndpoint:  Information about the <code>CreateResolverEndpoint</code> request, including the status of the request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModificationTime</i> The date and time that the endpoint was last modified, in Unix time format and Coordinated Universal Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The date and time that the endpoint was created, in Unix time format and Coordinated Universal Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> A detailed description of the status of the resolver endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> A code that specifies the current status of the resolver endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostVPCId</i> The ID of the VPC that you want to create the resolver endpoint in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IpAddressCount</i> The number of IP addresses that the resolver endpoint can use for DNS queries.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Direction</i> Indicates whether the resolver endpoint allows inbound or outbound DNS queries: <ul> <li>  <code>INBOUND</code>: allows DNS queries to your VPC from your network or another VPC </li> <li>  <code>OUTBOUND</code>: allows DNS queries from your VPC to your network or another VPC </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The ID of one or more security groups that control access to this VPC. The security group must include one or more inbound resolver rules.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name that you assigned to the resolver endpoint when you submitted a <a>CreateResolverEndpoint</a> request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN (Amazon Resource Name) for the resolver endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatorRequestId</i> A unique string that identifies the request that created the resolver endpoint. The <code>CreatorRequestId</code> allows failed requests to be retried without the risk of executing the operation twice.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the resolver endpoint.<br/>        """
        pass

    def create_resolver_rule(self, DomainName: str, RuleType: str, CreatorRequestId: str, Name: str=None, TargetIps: List[TargetAddress]=None, ResolverEndpointId: str=None, Tags: List[Tag]=None) -> CreateResolverRuleResponse:
        """For DNS queries that originate in your VPCs, specifies which resolver endpoint the queries pass through, one domain name that you want to forward to your network, and the IP addresses of the DNS resolvers in your network.

        :param self:
        :param Tags:  A list of the tag keys and values that you want to associate with the endpoint.<br/>ResolverEndpointId:  The ID of the outbound resolver endpoint that you want to use to route DNS queries to the IP addresses that you specify in <code>TargetIps</code>.<br/>TargetIps:  The IPs that you want Resolver to forward DNS queries to. You can specify only IPv4 addresses. Separate IP addresses with a comma.<br/>DomainName:  DNS queries for this domain name are forwarded to the IP addresses that you specify in <code>TargetIps</code>. If a query matches multiple resolver rules (example.com and www.example.com), outbound DNS queries are routed using the resolver rule that contains the most specific domain name (www.example.com).<br/>RuleType:  Specify <code>FORWARD</code>. Other resolver rule types aren't supported.<br/>Name:  A friendly name that lets you easily find a rule in the Resolver dashboard in the Route 53 console.<br/>CreatorRequestId:  A unique string that identifies the request and that allows failed requests to be retried without the risk of executing the operation twice. <code>CreatorRequestId</code> can be any unique string, for example, a date/time stamp. <br/>
        :return: ResolverRule:  Information about the <code>CreateResolverRule</code> request, including the status of the request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ShareStatus</i> Whether the rules is shared and, if so, whether the current account is sharing the rule with another account, or another account is sharing the rule with the current account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> When a rule is shared with another AWS account, the account ID of the account that the rule is shared with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResolverEndpointId</i> The ID of the endpoint that the rule is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetIps</i> An array that contains the IP addresses and ports that you want to forward <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name for the resolver rule, which you specified when you created the resolver rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleType</i> This value is always <code>FORWARD</code>. Other resolver rule types aren't supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> A detailed description of the status of a resolver rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> A code that specifies the current status of the resolver rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainName</i> DNS queries for this domain name are forwarded to the IP addresses that are specified in <code>TargetIps</code>. If a query matches multiple resolver rules (example.com and www.example.com), the query is routed using the resolver rule that contains the most specific domain name (www.example.com).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN (Amazon Resource Name) for the resolver rule specified by <code>Id</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatorRequestId</i> A unique string that you specified when you created the resolver rule. <code>CreatorRequestId</code>identifies the request and allows failed requests to be retried without the risk of executing the operation twice. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID that Resolver assigned to the resolver rule when you created it.<br/>        """
        pass

    def delete_resolver_endpoint(self, ResolverEndpointId: str) -> DeleteResolverEndpointResponse:
        """Deletes a resolver endpoint. The effect of deleting a resolver endpoint depends on whether it's an inbound or an outbound resolver endpoint: <ul> <li>  <b>Inbound</b>: DNS queries from your network or another VPC are no longer routed to the DNS service for the specified VPC. </li> <li>  <b>Outbound</b>: DNS queries from a VPC are no longer routed to your network or to another VPC. </li> </ul>

        :param self:
        :param ResolverEndpointId:  The ID of the resolver endpoint that you want to delete.<br/>
        :return: ResolverEndpoint:  Information about the <code>DeleteResolverEndpoint</code> request, including the status of the request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModificationTime</i> The date and time that the endpoint was last modified, in Unix time format and Coordinated Universal Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The date and time that the endpoint was created, in Unix time format and Coordinated Universal Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> A detailed description of the status of the resolver endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> A code that specifies the current status of the resolver endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostVPCId</i> The ID of the VPC that you want to create the resolver endpoint in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IpAddressCount</i> The number of IP addresses that the resolver endpoint can use for DNS queries.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Direction</i> Indicates whether the resolver endpoint allows inbound or outbound DNS queries: <ul> <li>  <code>INBOUND</code>: allows DNS queries to your VPC from your network or another VPC </li> <li>  <code>OUTBOUND</code>: allows DNS queries from your VPC to your network or another VPC </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The ID of one or more security groups that control access to this VPC. The security group must include one or more inbound resolver rules.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name that you assigned to the resolver endpoint when you submitted a <a>CreateResolverEndpoint</a> request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN (Amazon Resource Name) for the resolver endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatorRequestId</i> A unique string that identifies the request that created the resolver endpoint. The <code>CreatorRequestId</code> allows failed requests to be retried without the risk of executing the operation twice.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the resolver endpoint.<br/>        """
        pass

    def delete_resolver_rule(self, ResolverRuleId: str) -> DeleteResolverRuleResponse:
        """Deletes a resolver rule. Before you can delete a resolver rule, you must disassociate it from all the VPCs that you associated the resolver rule with. For more infomation, see <a>DisassociateResolverRule</a>.

        :param self:
        :param ResolverRuleId:  The ID of the resolver rule that you want to delete.<br/>
        :return: ResolverRule:  Information about the <code>DeleteResolverRule</code> request, including the status of the request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ShareStatus</i> Whether the rules is shared and, if so, whether the current account is sharing the rule with another account, or another account is sharing the rule with the current account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> When a rule is shared with another AWS account, the account ID of the account that the rule is shared with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResolverEndpointId</i> The ID of the endpoint that the rule is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetIps</i> An array that contains the IP addresses and ports that you want to forward <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name for the resolver rule, which you specified when you created the resolver rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleType</i> This value is always <code>FORWARD</code>. Other resolver rule types aren't supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> A detailed description of the status of a resolver rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> A code that specifies the current status of the resolver rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainName</i> DNS queries for this domain name are forwarded to the IP addresses that are specified in <code>TargetIps</code>. If a query matches multiple resolver rules (example.com and www.example.com), the query is routed using the resolver rule that contains the most specific domain name (www.example.com).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN (Amazon Resource Name) for the resolver rule specified by <code>Id</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatorRequestId</i> A unique string that you specified when you created the resolver rule. <code>CreatorRequestId</code>identifies the request and allows failed requests to be retried without the risk of executing the operation twice. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID that Resolver assigned to the resolver rule when you created it.<br/>        """
        pass

    def disassociate_resolver_endpoint_ip_address(self, IpAddress: IpAddressUpdate, ResolverEndpointId: str) -> DisassociateResolverEndpointIpAddressResponse:
        """Removes IP addresses from an inbound or an outbound resolver endpoint. If you want to remove more than one IP address, submit one <code>DisassociateResolverEndpointIpAddress</code> request for each IP address. To add an IP address to an endpoint, see <a>AssociateResolverEndpointIpAddress</a>.

        :param self:
        :param IpAddress:  The IPv4 address that you want to remove from a resolver endpoint.:<br/>ResolverEndpointId:  The ID of the resolver endpoint that you want to disassociate an IP address from.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Ip</i> The new IP address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubnetId</i> The ID of the subnet that includes the IP address that you want to update. To get this ID, use <a>GetResolverEndpoint</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IpId</i>  <i>Only when removing an IP address from a resolver endpoint</i>: The ID of the IP address that you want to remove. To get this ID, use <a>GetResolverEndpoint</a>.<br/>
        :return: ResolverEndpoint:  The response to an <code>DisassociateResolverEndpointIpAddress</code> request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModificationTime</i> The date and time that the endpoint was last modified, in Unix time format and Coordinated Universal Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The date and time that the endpoint was created, in Unix time format and Coordinated Universal Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> A detailed description of the status of the resolver endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> A code that specifies the current status of the resolver endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostVPCId</i> The ID of the VPC that you want to create the resolver endpoint in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IpAddressCount</i> The number of IP addresses that the resolver endpoint can use for DNS queries.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Direction</i> Indicates whether the resolver endpoint allows inbound or outbound DNS queries: <ul> <li>  <code>INBOUND</code>: allows DNS queries to your VPC from your network or another VPC </li> <li>  <code>OUTBOUND</code>: allows DNS queries from your VPC to your network or another VPC </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The ID of one or more security groups that control access to this VPC. The security group must include one or more inbound resolver rules.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name that you assigned to the resolver endpoint when you submitted a <a>CreateResolverEndpoint</a> request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN (Amazon Resource Name) for the resolver endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatorRequestId</i> A unique string that identifies the request that created the resolver endpoint. The <code>CreatorRequestId</code> allows failed requests to be retried without the risk of executing the operation twice.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the resolver endpoint.<br/>        """
        pass

    def disassociate_resolver_rule(self, ResolverRuleId: str, VPCId: str) -> DisassociateResolverRuleResponse:
        """Removes the association between a specified resolver rule and a specified VPC. <important> If you disassociate a resolver rule from a VPC, Resolver stops forwarding DNS queries for the domain name that you specified in the resolver rule.  </important>

        :param self:
        :param ResolverRuleId:  The ID of the resolver rule that you want to disassociate from the specified VPC.<br/>VPCId:  The ID of the VPC that you want to disassociate the resolver rule from.<br/>
        :return: ResolverRuleAssociation:  Information about the <code>DisassociateResolverRule</code> request, including the status of the request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> A detailed description of the status of the association between a resolver rule and a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> A code that specifies the current status of the association between a resolver rule and a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VPCId</i> The ID of the VPC that you associated the resolver rule with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of an association between a resolver rule and a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResolverRuleId</i> The ID of the resolver rule that you associated with the VPC that is specified by <code>VPCId</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the association between a resolver rule and a VPC. Resolver assigns this value when you submit an <a>AssociateResolverRule</a> request.<br/>        """
        pass

    def get_resolver_endpoint(self, ResolverEndpointId: str) -> GetResolverEndpointResponse:
        """Gets information about a specified resolver endpoint, such as whether it's an inbound or an outbound resolver endpoint, and the current status of the endpoint.

        :param self:
        :param ResolverEndpointId:  The ID of the resolver endpoint that you want to get information about.<br/>
        :return: ResolverEndpoint:  Information about the resolver endpoint that you specified in a <code>GetResolverEndpoint</code> request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModificationTime</i> The date and time that the endpoint was last modified, in Unix time format and Coordinated Universal Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The date and time that the endpoint was created, in Unix time format and Coordinated Universal Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> A detailed description of the status of the resolver endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> A code that specifies the current status of the resolver endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostVPCId</i> The ID of the VPC that you want to create the resolver endpoint in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IpAddressCount</i> The number of IP addresses that the resolver endpoint can use for DNS queries.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Direction</i> Indicates whether the resolver endpoint allows inbound or outbound DNS queries: <ul> <li>  <code>INBOUND</code>: allows DNS queries to your VPC from your network or another VPC </li> <li>  <code>OUTBOUND</code>: allows DNS queries from your VPC to your network or another VPC </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The ID of one or more security groups that control access to this VPC. The security group must include one or more inbound resolver rules.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name that you assigned to the resolver endpoint when you submitted a <a>CreateResolverEndpoint</a> request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN (Amazon Resource Name) for the resolver endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatorRequestId</i> A unique string that identifies the request that created the resolver endpoint. The <code>CreatorRequestId</code> allows failed requests to be retried without the risk of executing the operation twice.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the resolver endpoint.<br/>        """
        pass

    def get_resolver_rule(self, ResolverRuleId: str) -> GetResolverRuleResponse:
        """Gets information about a specified resolver rule, such as the domain name that the rule forwards DNS queries for and the ID of the outbound resolver endpoint that the rule is associated with.

        :param self:
        :param ResolverRuleId:  The ID of the resolver rule that you want to get information about.<br/>
        :return: ResolverRule:  Information about the resolver rule that you specified in a <code>GetResolverRule</code> request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ShareStatus</i> Whether the rules is shared and, if so, whether the current account is sharing the rule with another account, or another account is sharing the rule with the current account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> When a rule is shared with another AWS account, the account ID of the account that the rule is shared with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResolverEndpointId</i> The ID of the endpoint that the rule is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetIps</i> An array that contains the IP addresses and ports that you want to forward <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name for the resolver rule, which you specified when you created the resolver rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleType</i> This value is always <code>FORWARD</code>. Other resolver rule types aren't supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> A detailed description of the status of a resolver rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> A code that specifies the current status of the resolver rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainName</i> DNS queries for this domain name are forwarded to the IP addresses that are specified in <code>TargetIps</code>. If a query matches multiple resolver rules (example.com and www.example.com), the query is routed using the resolver rule that contains the most specific domain name (www.example.com).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN (Amazon Resource Name) for the resolver rule specified by <code>Id</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatorRequestId</i> A unique string that you specified when you created the resolver rule. <code>CreatorRequestId</code>identifies the request and allows failed requests to be retried without the risk of executing the operation twice. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID that Resolver assigned to the resolver rule when you created it.<br/>        """
        pass

    def get_resolver_rule_association(self, ResolverRuleAssociationId: str) -> GetResolverRuleAssociationResponse:
        """Gets information about an association between a specified resolver rule and a VPC. You associate a resolver rule and a VPC using <a>AssociateResolverRule</a>. 

        :param self:
        :param ResolverRuleAssociationId:  The ID of the resolver rule association that you want to get information about.<br/>
        :return: ResolverRuleAssociation:  Information about the resolver rule association that you specified in a <code>GetResolverRuleAssociation</code> request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> A detailed description of the status of the association between a resolver rule and a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> A code that specifies the current status of the association between a resolver rule and a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VPCId</i> The ID of the VPC that you associated the resolver rule with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of an association between a resolver rule and a VPC.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResolverRuleId</i> The ID of the resolver rule that you associated with the VPC that is specified by <code>VPCId</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the association between a resolver rule and a VPC. Resolver assigns this value when you submit an <a>AssociateResolverRule</a> request.<br/>        """
        pass

    def get_resolver_rule_policy(self, Arn: str) -> GetResolverRulePolicyResponse:
        """Gets information about a resolver rule policy. A resolver rule policy specifies the Resolver operations and resources that you want to allow another AWS account to be able to use. 

        :param self:
        :param Arn:  The ID of the resolver rule policy that you want to get information about.<br/>
        :return: ResolverRulePolicy:  Information about the resolver rule policy that you specified in a <code>GetResolverRulePolicy</code> request.<br/>        """
        pass

    def list_resolver_endpoint_ip_addresses(self, ResolverEndpointId: str, MaxResults: int=None, NextToken: str=None) -> ListResolverEndpointIpAddressesResponse:
        """Gets the IP addresses for a specified resolver endpoint.

        :param self:
        :param NextToken:  For the first <code>ListResolverEndpointIpAddresses</code> request, omit this value. If the specified resolver endpoint has more than <code>MaxResults</code> IP addresses, you can submit another <code>ListResolverEndpointIpAddresses</code> request to get the next group of IP addresses. In the next request, specify the value of <code>NextToken</code> from the previous response. <br/>MaxResults:  The maximum number of IP addresses that you want to return in the response to a <code>ListResolverEndpointIpAddresses</code> request. If you don't specify a value for <code>MaxResults</code>, Resolver returns up to 100 IP addresses. <br/>ResolverEndpointId:  The ID of the resolver endpoint that you want to get IP addresses for.<br/>
        :return: IpAddresses:  The IP addresses that DNS queries pass through on their way to your network (outbound endpoint) or on the way to Resolver (inbound endpoint).<br/>MaxResults:  The value that you specified for <code>MaxResults</code> in the request.<br/>NextToken:  If the specified endpoint has more than <code>MaxResults</code> IP addresses, you can submit another <code>ListResolverEndpointIpAddresses</code> request to get the next group of IP addresses. In the next request, specify the value of <code>NextToken</code> from the previous response. <br/>        """
        pass

    def list_resolver_endpoints(self, MaxResults: int=None, NextToken: str=None, Filters: List[Filter]=None) -> ListResolverEndpointsResponse:
        """Lists all the resolver endpoints that were created using the current AWS account.

        :param self:
        :param Filters:  An optional specification to return a subset of resolver endpoints, such as all inbound resolver endpoints. <note> If you submit a second or subsequent <code>ListResolverEndpoints</code> request and specify the <code>NextToken</code> parameter, you must use the same values for <code>Filters</code>, if any, as in the previous request. </note><br/>NextToken:  For the first <code>ListResolverEndpoints</code> request, omit this value. If you have more than <code>MaxResults</code> resolver endpoints, you can submit another <code>ListResolverEndpoints</code> request to get the next group of resolver endpoints. In the next request, specify the value of <code>NextToken</code> from the previous response. <br/>MaxResults:  The maximum number of resolver endpoints that you want to return in the response to a <code>ListResolverEndpoints</code> request. If you don't specify a value for <code>MaxResults</code>, Resolver returns up to 100 resolver endpoints. <br/>
        :return: ResolverEndpoints:  The resolver endpoints that were created by using the current AWS account, and that match the specified filters, if any.<br/>MaxResults:  The value that you specified for <code>MaxResults</code> in the request.<br/>NextToken:  If more than <code>MaxResults</code> IP addresses match the specified criteria, you can submit another <code>ListResolverEndpoint</code> request to get the next group of results. In the next request, specify the value of <code>NextToken</code> from the previous response. <br/>        """
        pass

    def list_resolver_rule_associations(self, MaxResults: int=None, NextToken: str=None, Filters: List[Filter]=None) -> ListResolverRuleAssociationsResponse:
        """Lists the associations that were created between resolver rules and VPCs using the current AWS account.

        :param self:
        :param Filters:  An optional specification to return a subset of resolver rules, such as resolver rules that are associated with the same VPC ID. <note> If you submit a second or subsequent <code>ListResolverRuleAssociations</code> request and specify the <code>NextToken</code> parameter, you must use the same values for <code>Filters</code>, if any, as in the previous request. </note><br/>NextToken:  For the first <code>ListResolverRuleAssociation</code> request, omit this value. If you have more than <code>MaxResults</code> rule associations, you can submit another <code>ListResolverRuleAssociation</code> request to get the next group of rule associations. In the next request, specify the value of <code>NextToken</code> from the previous response. <br/>MaxResults:  The maximum number of rule associations that you want to return in the response to a <code>ListResolverRuleAssociations</code> request. If you don't specify a value for <code>MaxResults</code>, Resolver returns up to 100 rule associations. <br/>
        :return: ResolverRuleAssociations:  The associations that were created between resolver rules and VPCs using the current AWS account, and that match the specified filters, if any.<br/>MaxResults:  The value that you specified for <code>MaxResults</code> in the request.<br/>NextToken:  If more than <code>MaxResults</code> rule associations match the specified criteria, you can submit another <code>ListResolverRuleAssociation</code> request to get the next group of results. In the next request, specify the value of <code>NextToken</code> from the previous response. <br/>        """
        pass

    def list_resolver_rules(self, MaxResults: int=None, NextToken: str=None, Filters: List[Filter]=None) -> ListResolverRulesResponse:
        """Lists the resolver rules that were created using the current AWS account.

        :param self:
        :param Filters:  An optional specification to return a subset of resolver rules, such as all resolver rules that are associated with the same resolver endpoint. <note> If you submit a second or subsequent <code>ListResolverRules</code> request and specify the <code>NextToken</code> parameter, you must use the same values for <code>Filters</code>, if any, as in the previous request. </note><br/>NextToken:  For the first <code>ListResolverRules</code> request, omit this value. If you have more than <code>MaxResults</code> resolver rules, you can submit another <code>ListResolverRules</code> request to get the next group of resolver rules. In the next request, specify the value of <code>NextToken</code> from the previous response. <br/>MaxResults:  The maximum number of resolver rules that you want to return in the response to a <code>ListResolverRules</code> request. If you don't specify a value for <code>MaxResults</code>, Resolver returns up to 100 resolver rules.<br/>
        :return: ResolverRules:  The resolver rules that were created using the current AWS account and that match the specified filters, if any.<br/>MaxResults:  The value that you specified for <code>MaxResults</code> in the request.<br/>NextToken:  If more than <code>MaxResults</code> resolver rules match the specified criteria, you can submit another <code>ListResolverRules</code> request to get the next group of results. In the next request, specify the value of <code>NextToken</code> from the previous response. <br/>        """
        pass

    def list_tags_for_resource(self, ResourceArn: str, MaxResults: int=None, NextToken: str=None) -> ListTagsForResourceResponse:
        """Lists the tags that you associated with the specified resource.

        :param self:
        :param NextToken:  For the first <code>ListTagsForResource</code> request, omit this value. If you have more than <code>MaxResults</code> tags, you can submit another <code>ListTagsForResource</code> request to get the next group of tags for the resource. In the next request, specify the value of <code>NextToken</code> from the previous response. <br/>MaxResults:  The maximum number of tags that you want to return in the response to a <code>ListTagsForResource</code> request. If you don't specify a value for <code>MaxResults</code>, Resolver returns up to 100 tags.<br/>ResourceArn:  The Amazon Resource Name (ARN) for the resource that you want to list tags for.<br/>
        :return: NextToken:  If more than <code>MaxResults</code> tags match the specified criteria, you can submit another <code>ListTagsForResource</code> request to get the next group of results. In the next request, specify the value of <code>NextToken</code> from the previous response. <br/>Tags:  The tags that are associated with the resource that you specified in the <code>ListTagsForResource</code> request.<br/>        """
        pass

    def put_resolver_rule_policy(self, ResolverRulePolicy: str, Arn: str) -> PutResolverRulePolicyResponse:
        """Specifies the Resolver operations and resources that you want to allow another AWS account to be able to use.

        :param self:
        :param ResolverRulePolicy:  An AWS Identity and Access Management policy statement that lists the permissions that you want to grant to another AWS account.<br/>Arn:  The Amazon Resource Name (ARN) of the account that you want to grant permissions to.<br/>
        :return: ReturnValue:  Whether the <code>PutResolverRulePolicy</code> request was successful.<br/>        """
        pass

    def tag_resource(self, Tags: List[Tag], ResourceArn: str) -> TagResourceResponse:
        """Adds one or more tags to a specified resource.

        :param self:
        :param Tags:  The tags that you want to add to the specified resource.<br/>ResourceArn:  The Amazon Resource Name (ARN) for the resource that you want to add tags to. To get the ARN for a resource, use the applicable <code>Get</code> or <code>List</code> command:  <ul> <li>  <a>GetResolverEndpoint</a>  </li> <li>  <a>GetResolverRule</a>  </li> <li>  <a>GetResolverRuleAssociation</a>  </li> <li>  <a>ListResolverEndpoints</a>  </li> <li>  <a>ListResolverRuleAssociations</a>  </li> <li>  <a>ListResolverRules</a>  </li> </ul><br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[TagKey], ResourceArn: str) -> UntagResourceResponse:
        """Removes one or more tags from a specified resource.

        :param self:
        :param TagKeys:  The tags that you want to remove to the specified resource.<br/>ResourceArn:  The Amazon Resource Name (ARN) for the resource that you want to remove tags from. To get the ARN for a resource, use the applicable <code>Get</code> or <code>List</code> command:  <ul> <li>  <a>GetResolverEndpoint</a>  </li> <li>  <a>GetResolverRule</a>  </li> <li>  <a>GetResolverRuleAssociation</a>  </li> <li>  <a>ListResolverEndpoints</a>  </li> <li>  <a>ListResolverRuleAssociations</a>  </li> <li>  <a>ListResolverRules</a>  </li> </ul><br/>
        :return:         """
        pass

    def update_resolver_endpoint(self, ResolverEndpointId: str, Name: str=None) -> UpdateResolverEndpointResponse:
        """Updates the name of an inbound or an outbound resolver endpoint. 

        :param self:
        :param Name:  The name of the resolver endpoint that you want to update.<br/>ResolverEndpointId:  The ID of the resolver endpoint that you want to update.<br/>
        :return: ResolverEndpoint:  The response to an <code>UpdateResolverEndpoint</code> request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModificationTime</i> The date and time that the endpoint was last modified, in Unix time format and Coordinated Universal Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The date and time that the endpoint was created, in Unix time format and Coordinated Universal Time (UTC).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> A detailed description of the status of the resolver endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> A code that specifies the current status of the resolver endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostVPCId</i> The ID of the VPC that you want to create the resolver endpoint in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IpAddressCount</i> The number of IP addresses that the resolver endpoint can use for DNS queries.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Direction</i> Indicates whether the resolver endpoint allows inbound or outbound DNS queries: <ul> <li>  <code>INBOUND</code>: allows DNS queries to your VPC from your network or another VPC </li> <li>  <code>OUTBOUND</code>: allows DNS queries from your VPC to your network or another VPC </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The ID of one or more security groups that control access to this VPC. The security group must include one or more inbound resolver rules.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name that you assigned to the resolver endpoint when you submitted a <a>CreateResolverEndpoint</a> request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN (Amazon Resource Name) for the resolver endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatorRequestId</i> A unique string that identifies the request that created the resolver endpoint. The <code>CreatorRequestId</code> allows failed requests to be retried without the risk of executing the operation twice.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the resolver endpoint.<br/>        """
        pass

    def update_resolver_rule(self, Config: ResolverRuleConfig, ResolverRuleId: str) -> UpdateResolverRuleResponse:
        """Updates settings for a specified resolver rule. <code>ResolverRuleId</code> is required, and all other parameters are optional. If you don't specify a parameter, it retains its current value.

        :param self:
        :param Config:  The new settings for the resolver rule.:<br/>ResolverRuleId:  The ID of the resolver rule that you want to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResolverEndpointId</i> The ID of the new outbound resolver endpoint that you want to use to route DNS queries to the IP addresses that you specify in <code>TargetIps</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetIps</i> For DNS queries that originate in your VPC, the new IP addresses that you want to route outbound DNS queries to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The new name for the resolver rule. The name that you specify appears in the Resolver dashboard in the Route 53 console. <br/>
        :return: ResolverRule:  The response to an <code>UpdateResolverRule</code> request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ShareStatus</i> Whether the rules is shared and, if so, whether the current account is sharing the rule with another account, or another account is sharing the rule with the current account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> When a rule is shared with another AWS account, the account ID of the account that the rule is shared with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResolverEndpointId</i> The ID of the endpoint that the rule is associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetIps</i> An array that contains the IP addresses and ports that you want to forward <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name for the resolver rule, which you specified when you created the resolver rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleType</i> This value is always <code>FORWARD</code>. Other resolver rule types aren't supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> A detailed description of the status of a resolver rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> A code that specifies the current status of the resolver rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DomainName</i> DNS queries for this domain name are forwarded to the IP addresses that are specified in <code>TargetIps</code>. If a query matches multiple resolver rules (example.com and www.example.com), the query is routed using the resolver rule that contains the most specific domain name (www.example.com).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN (Amazon Resource Name) for the resolver rule specified by <code>Id</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatorRequestId</i> A unique string that you specified when you created the resolver rule. <code>CreatorRequestId</code>identifies the request and allows failed requests to be retried without the risk of executing the operation twice. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID that Resolver assigned to the resolver rule when you created it.<br/>        """
        pass



class S3(BaseClient):
    """<p/>"""

    class BucketExistsWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `head_bucket` for valid parameters"""
            pass
    class BucketNotExistsWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `head_bucket` for valid parameters"""
            pass
    class ObjectExistsWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `head_object` for valid parameters"""
            pass
    class ObjectNotExistsWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `head_object` for valid parameters"""
            pass


    class ListMultipartUploadsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_multipart_uploads` for valid parameters"""
            pass
    class ListObjectVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_object_versions` for valid parameters"""
            pass
    class ListObjectsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_objects` for valid parameters"""
            pass
    class ListObjectsV2Paginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_objects_v2` for valid parameters"""
            pass
    class ListPartsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_parts` for valid parameters"""
            pass


    class AbortMultipartUploadRequest(Mapping):
        pass
    
    class AbortMultipartUploadOutput(Mapping):
        pass
    
    class CompleteMultipartUploadRequest(Mapping):
        pass
    
    class CompleteMultipartUploadOutput(Mapping):
        pass
    
    class CompletedMultipartUpload(Mapping):
        pass
    
    class CopyObjectRequest(Mapping):
        pass
    
    class CopyObjectOutput(Mapping):
        pass
    
    class CopySourceIfModifiedSince(object):
        pass
    
    class CopySourceIfUnmodifiedSince(object):
        pass
    
    class Expires(object):
        pass
    
    class Metadata(object):
        pass
    
    class ObjectLockRetainUntilDate(object):
        pass
    
    class CreateBucketRequest(Mapping):
        pass
    
    class CreateBucketOutput(Mapping):
        pass
    
    class CreateBucketConfiguration(Mapping):
        pass
    
    class ObjectLockEnabledForBucket(object):
        pass
    
    class CreateMultipartUploadRequest(Mapping):
        pass
    
    class CreateMultipartUploadOutput(Mapping):
        pass
    
    class DeleteBucketRequest(Mapping):
        pass
    
    class DeleteBucketAnalyticsConfigurationRequest(Mapping):
        pass
    
    class DeleteBucketCorsRequest(Mapping):
        pass
    
    class DeleteBucketEncryptionRequest(Mapping):
        pass
    
    class DeleteBucketInventoryConfigurationRequest(Mapping):
        pass
    
    class DeleteBucketLifecycleRequest(Mapping):
        pass
    
    class DeleteBucketMetricsConfigurationRequest(Mapping):
        pass
    
    class DeleteBucketPolicyRequest(Mapping):
        pass
    
    class DeleteBucketReplicationRequest(Mapping):
        pass
    
    class DeleteBucketTaggingRequest(Mapping):
        pass
    
    class DeleteBucketWebsiteRequest(Mapping):
        pass
    
    class DeleteObjectRequest(Mapping):
        pass
    
    class DeleteObjectOutput(Mapping):
        pass
    
    class BypassGovernanceRetention(object):
        pass
    
    class DeleteObjectTaggingRequest(Mapping):
        pass
    
    class DeleteObjectTaggingOutput(Mapping):
        pass
    
    class DeleteObjectsRequest(Mapping):
        pass
    
    class DeleteObjectsOutput(Mapping):
        pass
    
    class Delete(Mapping):
        pass
    
    class DeletePublicAccessBlockRequest(Mapping):
        pass
    
    class GetBucketAccelerateConfigurationRequest(Mapping):
        pass
    
    class GetBucketAccelerateConfigurationOutput(Mapping):
        pass
    
    class GetBucketAclRequest(Mapping):
        pass
    
    class GetBucketAclOutput(Mapping):
        pass
    
    class GetBucketAnalyticsConfigurationRequest(Mapping):
        pass
    
    class GetBucketAnalyticsConfigurationOutput(Mapping):
        pass
    
    class GetBucketCorsRequest(Mapping):
        pass
    
    class GetBucketCorsOutput(Mapping):
        pass
    
    class GetBucketEncryptionRequest(Mapping):
        pass
    
    class GetBucketEncryptionOutput(Mapping):
        pass
    
    class GetBucketInventoryConfigurationRequest(Mapping):
        pass
    
    class GetBucketInventoryConfigurationOutput(Mapping):
        pass
    
    class GetBucketLifecycleRequest(Mapping):
        pass
    
    class GetBucketLifecycleOutput(Mapping):
        pass
    
    class GetBucketLifecycleConfigurationRequest(Mapping):
        pass
    
    class GetBucketLifecycleConfigurationOutput(Mapping):
        pass
    
    class GetBucketLocationRequest(Mapping):
        pass
    
    class GetBucketLocationOutput(Mapping):
        pass
    
    class GetBucketLoggingRequest(Mapping):
        pass
    
    class GetBucketLoggingOutput(Mapping):
        pass
    
    class GetBucketMetricsConfigurationRequest(Mapping):
        pass
    
    class GetBucketMetricsConfigurationOutput(Mapping):
        pass
    
    class GetBucketNotificationConfigurationRequest(Mapping):
        pass
    
    class NotificationConfigurationDeprecated(Mapping):
        pass
    
    class NotificationConfiguration(Mapping):
        pass
    
    class GetBucketPolicyRequest(Mapping):
        pass
    
    class GetBucketPolicyOutput(Mapping):
        pass
    
    class GetBucketPolicyStatusRequest(Mapping):
        pass
    
    class GetBucketPolicyStatusOutput(Mapping):
        pass
    
    class GetBucketReplicationRequest(Mapping):
        pass
    
    class GetBucketReplicationOutput(Mapping):
        pass
    
    class GetBucketRequestPaymentRequest(Mapping):
        pass
    
    class GetBucketRequestPaymentOutput(Mapping):
        pass
    
    class GetBucketTaggingRequest(Mapping):
        pass
    
    class GetBucketTaggingOutput(Mapping):
        pass
    
    class GetBucketVersioningRequest(Mapping):
        pass
    
    class GetBucketVersioningOutput(Mapping):
        pass
    
    class GetBucketWebsiteRequest(Mapping):
        pass
    
    class GetBucketWebsiteOutput(Mapping):
        pass
    
    class GetObjectRequest(Mapping):
        pass
    
    class GetObjectOutput(Mapping):
        pass
    
    class IfModifiedSince(object):
        pass
    
    class IfUnmodifiedSince(object):
        pass
    
    class ResponseExpires(object):
        pass
    
    class GetObjectAclRequest(Mapping):
        pass
    
    class GetObjectAclOutput(Mapping):
        pass
    
    class GetObjectLegalHoldRequest(Mapping):
        pass
    
    class GetObjectLegalHoldOutput(Mapping):
        pass
    
    class GetObjectLockConfigurationRequest(Mapping):
        pass
    
    class GetObjectLockConfigurationOutput(Mapping):
        pass
    
    class GetObjectRetentionRequest(Mapping):
        pass
    
    class GetObjectRetentionOutput(Mapping):
        pass
    
    class GetObjectTaggingRequest(Mapping):
        pass
    
    class GetObjectTaggingOutput(Mapping):
        pass
    
    class GetObjectTorrentRequest(Mapping):
        pass
    
    class GetObjectTorrentOutput(Mapping):
        pass
    
    class GetPublicAccessBlockRequest(Mapping):
        pass
    
    class GetPublicAccessBlockOutput(Mapping):
        pass
    
    class HeadBucketRequest(Mapping):
        pass
    
    class HeadObjectRequest(Mapping):
        pass
    
    class HeadObjectOutput(Mapping):
        pass
    
    class ListBucketAnalyticsConfigurationsRequest(Mapping):
        pass
    
    class ListBucketAnalyticsConfigurationsOutput(Mapping):
        pass
    
    class ListBucketInventoryConfigurationsRequest(Mapping):
        pass
    
    class ListBucketInventoryConfigurationsOutput(Mapping):
        pass
    
    class ListBucketMetricsConfigurationsRequest(Mapping):
        pass
    
    class ListBucketMetricsConfigurationsOutput(Mapping):
        pass
    
    class ListBucketsOutput(Mapping):
        pass
    
    class ListMultipartUploadsRequest(Mapping):
        pass
    
    class ListMultipartUploadsOutput(Mapping):
        pass
    
    class ListObjectVersionsRequest(Mapping):
        pass
    
    class ListObjectVersionsOutput(Mapping):
        pass
    
    class ListObjectsRequest(Mapping):
        pass
    
    class ListObjectsOutput(Mapping):
        pass
    
    class ListObjectsV2Request(Mapping):
        pass
    
    class ListObjectsV2Output(Mapping):
        pass
    
    class FetchOwner(object):
        pass
    
    class ListPartsRequest(Mapping):
        pass
    
    class ListPartsOutput(Mapping):
        pass
    
    class PutBucketAccelerateConfigurationRequest(Mapping):
        pass
    
    class AccelerateConfiguration(Mapping):
        pass
    
    class PutBucketAclRequest(Mapping):
        pass
    
    class AccessControlPolicy(Mapping):
        pass
    
    class PutBucketAnalyticsConfigurationRequest(Mapping):
        pass
    
    class AnalyticsConfiguration(Mapping):
        pass
    
    class PutBucketCorsRequest(Mapping):
        pass
    
    class CORSConfiguration(Mapping):
        pass
    
    class PutBucketEncryptionRequest(Mapping):
        pass
    
    class ServerSideEncryptionConfiguration(Mapping):
        pass
    
    class PutBucketInventoryConfigurationRequest(Mapping):
        pass
    
    class InventoryConfiguration(Mapping):
        pass
    
    class PutBucketLifecycleRequest(Mapping):
        pass
    
    class LifecycleConfiguration(Mapping):
        pass
    
    class PutBucketLifecycleConfigurationRequest(Mapping):
        pass
    
    class BucketLifecycleConfiguration(Mapping):
        pass
    
    class PutBucketLoggingRequest(Mapping):
        pass
    
    class BucketLoggingStatus(Mapping):
        pass
    
    class PutBucketMetricsConfigurationRequest(Mapping):
        pass
    
    class MetricsConfiguration(Mapping):
        pass
    
    class PutBucketNotificationRequest(Mapping):
        pass
    
    class PutBucketNotificationConfigurationRequest(Mapping):
        pass
    
    class PutBucketPolicyRequest(Mapping):
        pass
    
    class ConfirmRemoveSelfBucketAccess(object):
        pass
    
    class PutBucketReplicationRequest(Mapping):
        pass
    
    class ReplicationConfiguration(Mapping):
        pass
    
    class PutBucketRequestPaymentRequest(Mapping):
        pass
    
    class RequestPaymentConfiguration(Mapping):
        pass
    
    class PutBucketTaggingRequest(Mapping):
        pass
    
    class Tagging(Mapping):
        pass
    
    class PutBucketVersioningRequest(Mapping):
        pass
    
    class VersioningConfiguration(Mapping):
        pass
    
    class PutBucketWebsiteRequest(Mapping):
        pass
    
    class WebsiteConfiguration(Mapping):
        pass
    
    class PutObjectRequest(Mapping):
        pass
    
    class PutObjectOutput(Mapping):
        pass
    
    class Body(object):
        pass
    
    class ContentLength(object):
        pass
    
    class PutObjectAclRequest(Mapping):
        pass
    
    class PutObjectAclOutput(Mapping):
        pass
    
    class PutObjectLegalHoldRequest(Mapping):
        pass
    
    class PutObjectLegalHoldOutput(Mapping):
        pass
    
    class ObjectLockLegalHold(Mapping):
        pass
    
    class PutObjectLockConfigurationRequest(Mapping):
        pass
    
    class PutObjectLockConfigurationOutput(Mapping):
        pass
    
    class ObjectLockConfiguration(Mapping):
        pass
    
    class PutObjectRetentionRequest(Mapping):
        pass
    
    class PutObjectRetentionOutput(Mapping):
        pass
    
    class ObjectLockRetention(Mapping):
        pass
    
    class PutObjectTaggingRequest(Mapping):
        pass
    
    class PutObjectTaggingOutput(Mapping):
        pass
    
    class PutPublicAccessBlockRequest(Mapping):
        pass
    
    class PublicAccessBlockConfiguration(Mapping):
        pass
    
    class RestoreObjectRequest(Mapping):
        pass
    
    class RestoreObjectOutput(Mapping):
        pass
    
    class RestoreRequest(Mapping):
        pass
    
    class SelectObjectContentRequest(Mapping):
        pass
    
    class SelectObjectContentOutput(Mapping):
        pass
    
    class RequestProgress(Mapping):
        pass
    
    class InputSerialization(Mapping):
        pass
    
    class OutputSerialization(Mapping):
        pass
    
    class UploadPartRequest(Mapping):
        pass
    
    class UploadPartOutput(Mapping):
        pass
    
    class UploadPartCopyRequest(Mapping):
        pass
    
    class UploadPartCopyOutput(Mapping):
        pass
    
    def abort_multipart_upload(self, UploadId: str, Key: str, Bucket: str, RequestPayer: str=None) -> AbortMultipartUploadOutput:
        """Aborts a multipart upload. To verify that all parts have been removed, so you don't get charged for the part storage, you should call the List Parts operation and ensure the parts list is empty.

        :param self:
        :param RequestPayer:  Confirms that the requester knows that she or he will be charged for the request. Bucket owners need not specify this parameter in their requests. Documentation on downloading objects from requester pays buckets can be found at http://docs.aws.amazon.com/AmazonS3/latest/dev/ObjectsinRequesterPaysBuckets.html<br/>UploadId:  Upload ID that identifies the multipart upload.<br/>Key:  Key of the object for which the multipart upload was initiated.<br/>Bucket:  Name of the bucket to which the multipart upload was initiated.<br/>
        :return: RequestCharged:  If present, indicates that the requester was successfully charged for the request.<br/>        """
        pass

    def complete_multipart_upload(self, UploadId: str, Key: str, Bucket: str, MultipartUpload: CompletedMultipartUpload=None, RequestPayer: str=None) -> CompleteMultipartUploadOutput:
        """Completes a multipart upload by assembling previously uploaded parts.

        :param self:
        :param RequestPayer:  Confirms that the requester knows that she or he will be charged for the request. Bucket owners need not specify this parameter in their requests. Documentation on downloading objects from requester pays buckets can be found at http://docs.aws.amazon.com/AmazonS3/latest/dev/ObjectsinRequesterPaysBuckets.html<br/>UploadId:  <p/><br/>MultipartUpload:  <p/>:<br/>Key:  <p/><br/>Bucket:  <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parts</i> <p/><br/>
        :return: RequestCharged:  If present, indicates that the requester was successfully charged for the request.<br/>SSEKMSKeyId:  If present, specifies the ID of the AWS Key Management Service (KMS) master encryption key that was used for the object.<br/>VersionId:  Version of the object.<br/>ServerSideEncryption:  The Server-side encryption algorithm used when storing this object in S3 (e.g., AES256, aws:kms).<br/>ETag:  Entity tag of the object.<br/>Expiration:  If the object expiration is configured, this will contain the expiration date (expiry-date) and rule ID (rule-id). The value of rule-id is URL encoded.<br/>Key:  <p/><br/>Bucket:  <p/><br/>Location:  <p/><br/>        """
        pass

    def copy_object(self, Key: str, CopySource: str, Bucket: str, ACL: str=None, CacheControl: str=None, ContentDisposition: str=None, ContentEncoding: str=None, ContentLanguage: str=None, ContentType: str=None, CopySourceIfMatch: str=None, CopySourceIfModifiedSince: CopySourceIfModifiedSince=None, CopySourceIfNoneMatch: str=None, CopySourceIfUnmodifiedSince: CopySourceIfUnmodifiedSince=None, Expires: Expires=None, GrantFullControl: str=None, GrantRead: str=None, GrantReadACP: str=None, GrantWriteACP: str=None, Metadata: Metadata=None, MetadataDirective: str=None, TaggingDirective: str=None, ServerSideEncryption: str=None, StorageClass: str=None, WebsiteRedirectLocation: str=None, SSECustomerAlgorithm: str=None, SSECustomerKey: str=None, SSECustomerKeyMD5: str=None, SSEKMSKeyId: str=None, SSEKMSEncryptionContext: str=None, CopySourceSSECustomerAlgorithm: str=None, CopySourceSSECustomerKey: str=None, CopySourceSSECustomerKeyMD5: str=None, RequestPayer: str=None, Tagging: str=None, ObjectLockMode: str=None, ObjectLockRetainUntilDate: ObjectLockRetainUntilDate=None, ObjectLockLegalHoldStatus: str=None) -> CopyObjectOutput:
        """Creates a copy of an object that is already stored in Amazon S3.

        :param self:
        :param ObjectLockLegalHoldStatus:  Specifies whether you want to apply a Legal Hold to the copied object.<br/>ObjectLockRetainUntilDate:  The date and time when you want the copied object's object lock to expire.<br/>ObjectLockMode:  The object lock mode that you want to apply to the copied object.<br/>Tagging:  The tag-set for the object destination object this value must be used in conjunction with the TaggingDirective. The tag-set must be encoded as URL Query parameters<br/>RequestPayer:  Confirms that the requester knows that she or he will be charged for the request. Bucket owners need not specify this parameter in their requests. Documentation on downloading objects from requester pays buckets can be found at http://docs.aws.amazon.com/AmazonS3/latest/dev/ObjectsinRequesterPaysBuckets.html<br/>CopySourceSSECustomerKeyMD5:  Specifies the 128-bit MD5 digest of the encryption key according to RFC 1321. Amazon S3 uses this header for a message integrity check to ensure the encryption key was transmitted without error.<br/>CopySourceSSECustomerKey:  Specifies the customer-provided encryption key for Amazon S3 to use to decrypt the source object. The encryption key provided in this header must be one that was used when the source object was created.<br/>CopySourceSSECustomerAlgorithm:  Specifies the algorithm to use when decrypting the source object (e.g., AES256).<br/>SSEKMSEncryptionContext:  Specifies the AWS KMS Encryption Context to use for object encryption. The value of this header is a base64-encoded UTF-8 string holding JSON with the encryption context key-value pairs.<br/>SSEKMSKeyId:  Specifies the AWS KMS key ID to use for object encryption. All GET and PUT requests for an object protected by AWS KMS will fail if not made via SSL or using SigV4. Documentation on configuring any of the officially supported AWS SDKs and CLI can be found at http://docs.aws.amazon.com/AmazonS3/latest/dev/UsingAWSSDK.html#specify-signature-version<br/>SSECustomerKeyMD5:  Specifies the 128-bit MD5 digest of the encryption key according to RFC 1321. Amazon S3 uses this header for a message integrity check to ensure the encryption key was transmitted without error.<br/>SSECustomerKey:  Specifies the customer-provided encryption key for Amazon S3 to use in encrypting data. This value is used to store the object and then it is discarded; Amazon does not store the encryption key. The key must be appropriate for use with the algorithm specified in the x-amz-server-side​-encryption​-customer-algorithm header.<br/>SSECustomerAlgorithm:  Specifies the algorithm to use to when encrypting the object (e.g., AES256).<br/>WebsiteRedirectLocation:  If the bucket is configured as a website, redirects requests for this object to another object in the same bucket or to an external URL. Amazon S3 stores the value of this header in the object metadata.<br/>StorageClass:  The type of storage to use for the object. Defaults to 'STANDARD'.<br/>ServerSideEncryption:  The Server-side encryption algorithm used when storing this object in S3 (e.g., AES256, aws:kms).<br/>TaggingDirective:  Specifies whether the object tag-set are copied from the source object or replaced with tag-set provided in the request.<br/>MetadataDirective:  Specifies whether the metadata is copied from the source object or replaced with metadata provided in the request.<br/>Metadata:  A map of metadata to store with the object in S3.<br/>Key:  <p/><br/>GrantWriteACP:  Allows grantee to write the ACL for the applicable object.<br/>GrantReadACP:  Allows grantee to read the object ACL.<br/>GrantRead:  Allows grantee to read the object data and its metadata.<br/>GrantFullControl:  Gives the grantee READ, READ_ACP, and WRITE_ACP permissions on the object.<br/>Expires:  The date and time at which the object is no longer cacheable.<br/>CopySourceIfUnmodifiedSince:  Copies the object if it hasn't been modified since the specified time.<br/>CopySourceIfNoneMatch:  Copies the object if its entity tag (ETag) is different than the specified ETag.<br/>CopySourceIfModifiedSince:  Copies the object if it has been modified since the specified time.<br/>CopySourceIfMatch:  Copies the object if its entity tag (ETag) matches the specified tag.<br/>CopySource:  The name of the source bucket and key name of the source object, separated by a slash (/). Must be URL-encoded.<br/>ContentType:  A standard MIME type describing the format of the object data.<br/>ContentLanguage:  The language the content is in.<br/>ContentEncoding:  Specifies what content encodings have been applied to the object and thus what decoding mechanisms must be applied to obtain the media-type referenced by the Content-Type header field.<br/>ContentDisposition:  Specifies presentational information for the object.<br/>CacheControl:  Specifies caching behavior along the request/reply chain.<br/>Bucket:  <p/><br/>ACL:  The canned ACL to apply to the object.<br/>
        :return: RequestCharged:  If present, indicates that the requester was successfully charged for the request.<br/>SSEKMSEncryptionContext:  If present, specifies the AWS KMS Encryption Context to use for object encryption. The value of this header is a base64-encoded UTF-8 string holding JSON with the encryption context key-value pairs.<br/>SSEKMSKeyId:  If present, specifies the ID of the AWS Key Management Service (KMS) master encryption key that was used for the object.<br/>SSECustomerKeyMD5:  If server-side encryption with a customer-provided encryption key was requested, the response will include this header to provide round trip message integrity verification of the customer-provided encryption key.<br/>SSECustomerAlgorithm:  If server-side encryption with a customer-provided encryption key was requested, the response will include this header confirming the encryption algorithm used.<br/>ServerSideEncryption:  The Server-side encryption algorithm used when storing this object in S3 (e.g., AES256, aws:kms).<br/>VersionId:  Version ID of the newly created copy.<br/>CopySourceVersionId:  <p/><br/>Expiration:  If the object expiration is configured, the response includes this header.<br/>CopyObjectResult:  <p/>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModified</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ETag</i> <p/><br/>        """
        pass

    def create_bucket(self, Bucket: str, ACL: str=None, CreateBucketConfiguration: CreateBucketConfiguration=None, GrantFullControl: str=None, GrantRead: str=None, GrantReadACP: str=None, GrantWrite: str=None, GrantWriteACP: str=None, ObjectLockEnabledForBucket: ObjectLockEnabledForBucket=None) -> CreateBucketOutput:
        """Creates a new bucket.

        :param self:
        :param ObjectLockEnabledForBucket:  Specifies whether you want Amazon S3 object lock to be enabled for the new bucket.<br/>GrantWriteACP:  Allows grantee to write the ACL for the applicable bucket.<br/>GrantWrite:  Allows grantee to create, overwrite, and delete any object in the bucket.<br/>GrantReadACP:  Allows grantee to read the bucket ACL.<br/>GrantRead:  Allows grantee to list the objects in the bucket.<br/>GrantFullControl:  Allows grantee the read, write, read ACP, and write ACP permissions on the bucket.<br/>CreateBucketConfiguration:  <p/>:<br/>Bucket:  <p/><br/>ACL:  The canned ACL to apply to the bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LocationConstraint</i> Specifies the region where the bucket will be created. If you don't specify a region, the bucket is created in US East (N. Virginia) Region (us-east-1).<br/>
        :return: Location:  <p/><br/>        """
        pass

    def create_multipart_upload(self, Key: str, Bucket: str, ACL: str=None, CacheControl: str=None, ContentDisposition: str=None, ContentEncoding: str=None, ContentLanguage: str=None, ContentType: str=None, Expires: Expires=None, GrantFullControl: str=None, GrantRead: str=None, GrantReadACP: str=None, GrantWriteACP: str=None, Metadata: Metadata=None, ServerSideEncryption: str=None, StorageClass: str=None, WebsiteRedirectLocation: str=None, SSECustomerAlgorithm: str=None, SSECustomerKey: str=None, SSECustomerKeyMD5: str=None, SSEKMSKeyId: str=None, SSEKMSEncryptionContext: str=None, RequestPayer: str=None, Tagging: str=None, ObjectLockMode: str=None, ObjectLockRetainUntilDate: ObjectLockRetainUntilDate=None, ObjectLockLegalHoldStatus: str=None) -> CreateMultipartUploadOutput:
        """Initiates a multipart upload and returns an upload ID.  <b>Note:</b> After you initiate multipart upload and upload one or more parts, you must either complete or abort multipart upload in order to stop getting charged for storage of the uploaded parts. Only after you either complete or abort multipart upload, Amazon S3 frees up the parts storage and stops charging you for the parts storage.

        :param self:
        :param ObjectLockLegalHoldStatus:  Specifies whether you want to apply a Legal Hold to the uploaded object.<br/>ObjectLockRetainUntilDate:  Specifies the date and time when you want the object lock to expire.<br/>ObjectLockMode:  Specifies the object lock mode that you want to apply to the uploaded object.<br/>Tagging:  The tag-set for the object. The tag-set must be encoded as URL Query parameters<br/>RequestPayer:  Confirms that the requester knows that she or he will be charged for the request. Bucket owners need not specify this parameter in their requests. Documentation on downloading objects from requester pays buckets can be found at http://docs.aws.amazon.com/AmazonS3/latest/dev/ObjectsinRequesterPaysBuckets.html<br/>SSEKMSEncryptionContext:  Specifies the AWS KMS Encryption Context to use for object encryption. The value of this header is a base64-encoded UTF-8 string holding JSON with the encryption context key-value pairs.<br/>SSEKMSKeyId:  Specifies the AWS KMS key ID to use for object encryption. All GET and PUT requests for an object protected by AWS KMS will fail if not made via SSL or using SigV4. Documentation on configuring any of the officially supported AWS SDKs and CLI can be found at http://docs.aws.amazon.com/AmazonS3/latest/dev/UsingAWSSDK.html#specify-signature-version<br/>SSECustomerKeyMD5:  Specifies the 128-bit MD5 digest of the encryption key according to RFC 1321. Amazon S3 uses this header for a message integrity check to ensure the encryption key was transmitted without error.<br/>SSECustomerKey:  Specifies the customer-provided encryption key for Amazon S3 to use in encrypting data. This value is used to store the object and then it is discarded; Amazon does not store the encryption key. The key must be appropriate for use with the algorithm specified in the x-amz-server-side​-encryption​-customer-algorithm header.<br/>SSECustomerAlgorithm:  Specifies the algorithm to use to when encrypting the object (e.g., AES256).<br/>WebsiteRedirectLocation:  If the bucket is configured as a website, redirects requests for this object to another object in the same bucket or to an external URL. Amazon S3 stores the value of this header in the object metadata.<br/>StorageClass:  The type of storage to use for the object. Defaults to 'STANDARD'.<br/>ServerSideEncryption:  The Server-side encryption algorithm used when storing this object in S3 (e.g., AES256, aws:kms).<br/>Metadata:  A map of metadata to store with the object in S3.<br/>Key:  <p/><br/>GrantWriteACP:  Allows grantee to write the ACL for the applicable object.<br/>GrantReadACP:  Allows grantee to read the object ACL.<br/>GrantRead:  Allows grantee to read the object data and its metadata.<br/>GrantFullControl:  Gives the grantee READ, READ_ACP, and WRITE_ACP permissions on the object.<br/>Expires:  The date and time at which the object is no longer cacheable.<br/>ContentType:  A standard MIME type describing the format of the object data.<br/>ContentLanguage:  The language the content is in.<br/>ContentEncoding:  Specifies what content encodings have been applied to the object and thus what decoding mechanisms must be applied to obtain the media-type referenced by the Content-Type header field.<br/>ContentDisposition:  Specifies presentational information for the object.<br/>CacheControl:  Specifies caching behavior along the request/reply chain.<br/>Bucket:  <p/><br/>ACL:  The canned ACL to apply to the object.<br/>
        :return: RequestCharged:  If present, indicates that the requester was successfully charged for the request.<br/>SSEKMSEncryptionContext:  If present, specifies the AWS KMS Encryption Context to use for object encryption. The value of this header is a base64-encoded UTF-8 string holding JSON with the encryption context key-value pairs.<br/>SSEKMSKeyId:  If present, specifies the ID of the AWS Key Management Service (KMS) master encryption key that was used for the object.<br/>SSECustomerKeyMD5:  If server-side encryption with a customer-provided encryption key was requested, the response will include this header to provide round trip message integrity verification of the customer-provided encryption key.<br/>SSECustomerAlgorithm:  If server-side encryption with a customer-provided encryption key was requested, the response will include this header confirming the encryption algorithm used.<br/>ServerSideEncryption:  The Server-side encryption algorithm used when storing this object in S3 (e.g., AES256, aws:kms).<br/>UploadId:  ID for the initiated multipart upload.<br/>Key:  Object key for which the multipart upload was initiated.<br/>Bucket:  Name of the bucket to which the multipart upload was initiated.<br/>AbortRuleId:  Id of the lifecycle rule that makes a multipart upload eligible for abort operation.<br/>AbortDate:  Date when multipart upload will become eligible for abort operation by lifecycle.<br/>        """
        pass

    def delete_bucket(self, Bucket: str):
        """Deletes the bucket. All objects (including all object versions and Delete Markers) in the bucket must be deleted before the bucket itself can be deleted.

        :param self:
        :param Bucket:  <p/><br/>
        :return:         """
        pass

    def delete_bucket_analytics_configuration(self, Id: str, Bucket: str):
        """Deletes an analytics configuration for the bucket (specified by the analytics configuration ID). To use this operation, you must have permissions to perform the s3:PutAnalyticsConfiguration action. The bucket owner has this permission by default. The bucket owner can grant this permission to others. 

        :param self:
        :param Id:  The ID that identifies the analytics configuration.<br/>Bucket:  The name of the bucket from which an analytics configuration is deleted.<br/>
        :return:         """
        pass

    def delete_bucket_cors(self, Bucket: str):
        """Deletes the CORS configuration information set for the bucket.

        :param self:
        :param Bucket:  <p/><br/>
        :return:         """
        pass

    def delete_bucket_encryption(self, Bucket: str):
        """Deletes the server-side encryption configuration from the bucket.

        :param self:
        :param Bucket:  The name of the bucket containing the server-side encryption configuration to delete.<br/>
        :return:         """
        pass

    def delete_bucket_inventory_configuration(self, Id: str, Bucket: str):
        """Deletes an inventory configuration (identified by the inventory ID) from the bucket.

        :param self:
        :param Id:  The ID used to identify the inventory configuration.<br/>Bucket:  The name of the bucket containing the inventory configuration to delete.<br/>
        :return:         """
        pass

    def delete_bucket_lifecycle(self, Bucket: str):
        """Deletes the lifecycle configuration from the bucket.

        :param self:
        :param Bucket:  <p/><br/>
        :return:         """
        pass

    def delete_bucket_metrics_configuration(self, Id: str, Bucket: str):
        """Deletes a metrics configuration (specified by the metrics configuration ID) from the bucket.

        :param self:
        :param Id:  The ID used to identify the metrics configuration.<br/>Bucket:  The name of the bucket containing the metrics configuration to delete.<br/>
        :return:         """
        pass

    def delete_bucket_policy(self, Bucket: str):
        """Deletes the policy from the bucket.

        :param self:
        :param Bucket:  <p/><br/>
        :return:         """
        pass

    def delete_bucket_replication(self, Bucket: str):
        """ Deletes the replication configuration from the bucket. For information about replication configuration, see <a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/crr.html">Cross-Region Replication (CRR)</a> in the <i>Amazon S3 Developer Guide</i>. 

        :param self:
        :param Bucket:   The bucket name.  <note> It can take a while to propagate the deletion of a replication configuration to all Amazon S3 systems. </note><br/>
        :return:         """
        pass

    def delete_bucket_tagging(self, Bucket: str):
        """Deletes the tags from the bucket.

        :param self:
        :param Bucket:  <p/><br/>
        :return:         """
        pass

    def delete_bucket_website(self, Bucket: str):
        """This operation removes the website configuration from the bucket.

        :param self:
        :param Bucket:  <p/><br/>
        :return:         """
        pass

    def delete_object(self, Key: str, Bucket: str, MFA: str=None, VersionId: str=None, RequestPayer: str=None, BypassGovernanceRetention: BypassGovernanceRetention=None) -> DeleteObjectOutput:
        """Removes the null version (if there is one) of an object and inserts a delete marker, which becomes the latest version of the object. If there isn't a null version, Amazon S3 does not remove any objects.

        :param self:
        :param BypassGovernanceRetention:  Indicates whether Amazon S3 object lock should bypass governance-mode restrictions to process this operation.<br/>RequestPayer:  Confirms that the requester knows that she or he will be charged for the request. Bucket owners need not specify this parameter in their requests. Documentation on downloading objects from requester pays buckets can be found at http://docs.aws.amazon.com/AmazonS3/latest/dev/ObjectsinRequesterPaysBuckets.html<br/>VersionId:  VersionId used to reference a specific version of the object.<br/>MFA:  The concatenation of the authentication device's serial number, a space, and the value that is displayed on your authentication device.<br/>Key:  <p/><br/>Bucket:  <p/><br/>
        :return: RequestCharged:  If present, indicates that the requester was successfully charged for the request.<br/>VersionId:  Returns the version ID of the delete marker created as a result of the DELETE operation.<br/>DeleteMarker:  Specifies whether the versioned object that was permanently deleted was (true) or was not (false) a delete marker.<br/>        """
        pass

    def delete_object_tagging(self, Key: str, Bucket: str, VersionId: str=None) -> DeleteObjectTaggingOutput:
        """Removes the tag-set from an existing object.

        :param self:
        :param VersionId:  The versionId of the object that the tag-set will be removed from.<br/>Key:  <p/><br/>Bucket:  <p/><br/>
        :return: VersionId:  The versionId of the object the tag-set was removed from.<br/>        """
        pass

    def delete_objects(self, Delete: Delete, Bucket: str, MFA: str=None, RequestPayer: str=None, BypassGovernanceRetention: BypassGovernanceRetention=None) -> DeleteObjectsOutput:
        """This operation enables you to delete multiple objects from a bucket using a single HTTP request. You may specify up to 1000 keys.

        :param self:
        :param BypassGovernanceRetention:  Specifies whether you want to delete this object even if it has a Governance-type object lock in place. You must have sufficient permissions to perform this operation.<br/>RequestPayer:  Confirms that the requester knows that she or he will be charged for the request. Bucket owners need not specify this parameter in their requests. Documentation on downloading objects from requester pays buckets can be found at http://docs.aws.amazon.com/AmazonS3/latest/dev/ObjectsinRequesterPaysBuckets.html<br/>MFA:  The concatenation of the authentication device's serial number, a space, and the value that is displayed on your authentication device.<br/>Delete:  <p/>:<br/>Bucket:  <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Quiet</i> Element to enable quiet mode for the request. When you add this element, you must set its value to true.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Objects</i> <p/><br/>
        :return: Errors:  <p/><br/>RequestCharged:  If present, indicates that the requester was successfully charged for the request.<br/>Deleted:  <p/><br/>        """
        pass

    def delete_public_access_block(self, Bucket: str):
        """Removes the <code>PublicAccessBlock</code> configuration from an Amazon S3 bucket.

        :param self:
        :param Bucket:  The Amazon S3 bucket whose <code>PublicAccessBlock</code> configuration you want to delete. <br/>
        :return:         """
        pass

    def get_bucket_accelerate_configuration(self, Bucket: str) -> GetBucketAccelerateConfigurationOutput:
        """Returns the accelerate configuration of a bucket.

        :param self:
        :param Bucket:  Name of the bucket for which the accelerate configuration is retrieved.<br/>
        :return: Status:  The accelerate configuration of the bucket.<br/>        """
        pass

    def get_bucket_acl(self, Bucket: str) -> GetBucketAclOutput:
        """Gets the access control policy for the bucket.

        :param self:
        :param Bucket:  <p/><br/>
        :return: Grants:  A list of grants.<br/>Owner:  <p/>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ID</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayName</i> <p/><br/>        """
        pass

    def get_bucket_analytics_configuration(self, Id: str, Bucket: str) -> GetBucketAnalyticsConfigurationOutput:
        """Gets an analytics configuration for the bucket (specified by the analytics configuration ID).

        :param self:
        :param Id:  The ID that identifies the analytics configuration.<br/>Bucket:  The name of the bucket from which an analytics configuration is retrieved.<br/>
        :return: AnalyticsConfiguration:  The configuration and any analyses for the analytics filter.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageClassAnalysis</i>  Contains data related to access patterns to be collected and made available to analyze the tradeoffs between different storage classes. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StorageClassAnalysis</b>:  Contains data related to access patterns to be collected and made available to analyze the tradeoffs between different storage classes. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Filter</i> The filter used to describe a set of objects for analyses. A filter must have exactly one prefix, one tag, or one conjunction (AnalyticsAndOperator). If no filter is provided, all objects will be considered in any analysis.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Filter</b>: The filter used to describe a set of objects for analyses. A filter must have exactly one prefix, one tag, or one conjunction (AnalyticsAndOperator). If no filter is provided, all objects will be considered in any analysis.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID that identifies the analytics configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> The prefix to use when evaluating an analytics filter.<br/>The tag to use when evaluating an analytics filter.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Tag</b>: The tag to use when evaluating an analytics filter.Specifies how data related to the storage class analysis for an Amazon S3 bucket should be exported.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DataExport</b>: Specifies how data related to the storage class analysis for an Amazon S3 bucket should be exported.        """
        pass

    def get_bucket_cors(self, Bucket: str) -> GetBucketCorsOutput:
        """Returns the CORS configuration for the bucket.

        :param self:
        :param Bucket:  <p/><br/>
        :return: CORSRules:  <p/><br/>        """
        pass

    def get_bucket_encryption(self, Bucket: str) -> GetBucketEncryptionOutput:
        """Returns the server-side encryption configuration of a bucket.

        :param self:
        :param Bucket:  The name of the bucket from which the server-side encryption configuration is retrieved.<br/>
        :return: ServerSideEncryptionConfiguration:  <p/>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Rules</i> Container for information about a particular server-side encryption configuration rule.<br/>        """
        pass

    def get_bucket_inventory_configuration(self, Id: str, Bucket: str) -> GetBucketInventoryConfigurationOutput:
        """Returns an inventory configuration (identified by the inventory ID) from the bucket.

        :param self:
        :param Id:  The ID used to identify the inventory configuration.<br/>Bucket:  The name of the bucket containing the inventory configuration to retrieve.<br/>
        :return: InventoryConfiguration:  Specifies the inventory configuration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Schedule</i> Specifies the schedule for generating inventory results.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Schedule</b>: Specifies the schedule for generating inventory results.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionalFields</i> Contains the optional fields that are included in the inventory results.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludedObjectVersions</i> Object versions to include in the inventory list. If set to <code>All</code>, the list includes all the object versions, which adds the version-related fields <code>VersionId</code>, <code>IsLatest</code>, and <code>DeleteMarker</code> to the list. If set to <code>Current</code>, the list does not contain these version-related fields.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID used to identify the inventory configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Filter</i> Specifies an inventory filter. The inventory only includes objects that meet the filter's criteria.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Filter</b>: Specifies an inventory filter. The inventory only includes objects that meet the filter's criteria.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsEnabled</i> Specifies whether the inventory is enabled or disabled. If set to <code>True</code>, an inventory list is generated. If set to <code>False</code>, no inventory list is generated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Destination</i> Contains information about where to publish the inventory results.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Destination</b>: Contains information about where to publish the inventory results.<br/>Contains the bucket name, file format, bucket owner (optional), and prefix (optional) where inventory results are published.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3BucketDestination</b>: Contains the bucket name, file format, bucket owner (optional), and prefix (optional) where inventory results are published.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> The prefix that an object must have to be included in the inventory results.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Frequency</i> Specifies how frequently inventory results are produced.<br/>        """
        pass

    def get_bucket_lifecycle(self, Bucket: str) -> GetBucketLifecycleOutput:
        """ No longer used, see the GetBucketLifecycleConfiguration operation.

        :param self:
        :param Bucket:  <p/><br/>
        :return: Rules:  <p/><br/>        """
        pass

    def get_bucket_lifecycle_configuration(self, Bucket: str) -> GetBucketLifecycleConfigurationOutput:
        """Returns the lifecycle configuration information set on the bucket.

        :param self:
        :param Bucket:  <p/><br/>
        :return: Rules:  <p/><br/>        """
        pass

    def get_bucket_location(self, Bucket: str) -> GetBucketLocationOutput:
        """Returns the region the bucket resides in.

        :param self:
        :param Bucket:  <p/><br/>
        :return: LocationConstraint:  <p/><br/>        """
        pass

    def get_bucket_logging(self, Bucket: str) -> GetBucketLoggingOutput:
        """Returns the logging status of a bucket and the permissions users have to view and modify that status. To use GET, you must be the bucket owner.

        :param self:
        :param Bucket:  <p/><br/>
        :return: LoggingEnabled:  <p/>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetPrefix</i> A prefix for all log object keys. If you store log files from multiple Amazon S3 buckets in a single bucket, you can use a prefix to distinguish which log files came from which bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetGrants</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetBucket</i> Specifies the bucket where you want Amazon S3 to store server access logs. You can have your logs delivered to any bucket that you own, including the same bucket that is being logged. You can also configure multiple buckets to deliver their logs to the same target bucket. In this case you should choose a different TargetPrefix for each source bucket so that the delivered log files can be distinguished by key.<br/>        """
        pass

    def get_bucket_metrics_configuration(self, Id: str, Bucket: str) -> GetBucketMetricsConfigurationOutput:
        """Gets a metrics configuration (specified by the metrics configuration ID) from the bucket.

        :param self:
        :param Id:  The ID used to identify the metrics configuration.<br/>Bucket:  The name of the bucket containing the metrics configuration to retrieve.<br/>
        :return: MetricsConfiguration:  Specifies the metrics configuration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Filter</i> Specifies a metrics configuration filter. The metrics configuration will only include objects that meet the filter's criteria. A filter must be a prefix, a tag, or a conjunction (MetricsAndOperator).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Filter</b>: Specifies a metrics configuration filter. The metrics configuration will only include objects that meet the filter's criteria. A filter must be a prefix, a tag, or a conjunction (MetricsAndOperator).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID used to identify the metrics configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> The prefix used when evaluating a metrics filter.<br/>The tag used when evaluating a metrics filter.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Tag</b>: The tag used when evaluating a metrics filter.        """
        pass

    def get_bucket_notification(self, Bucket: str) -> NotificationConfigurationDeprecated:
        """ No longer used, see the GetBucketNotificationConfiguration operation.

        :param self:
        :param Bucket:  Name of the bucket to get the notification configuration for.<br/>
        :return: CloudFunctionConfiguration:  <p/>:<br/>QueueConfiguration:  <p/>:<br/>TopicConfiguration:  <p/>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Topic</i> Amazon SNS topic to which Amazon S3 will publish a message to report the specified events for the bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Event</i> Bucket event for which to send notifications.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Events</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> An optional unique identifier for configurations in a notification configuration. If you don't provide one, Amazon S3 will assign an ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Queue</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Events</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Event</i> The bucket event for which to send notifications.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> An optional unique identifier for configurations in a notification configuration. If you don't provide one, Amazon S3 will assign an ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InvocationRole</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudFunction</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Events</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Event</i> The bucket event for which to send notifications.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> An optional unique identifier for configurations in a notification configuration. If you don't provide one, Amazon S3 will assign an ID.<br/>        """
        pass

    def get_bucket_notification_configuration(self, Bucket: str) -> NotificationConfiguration:
        """Returns the notification configuration of a bucket.

        :param self:
        :param Bucket:  Name of the bucket to get the notification configuration for.<br/>
        :return: LambdaFunctionConfigurations:  Describes the AWS Lambda functions to invoke and the events for which to invoke them.<br/>QueueConfigurations:  The Amazon Simple Queue Service queues to publish messages to and the events for which to publish messages.<br/>TopicConfigurations:  The topic to which notifications are sent and the events for which notifications are generated.<br/>        """
        pass

    def get_bucket_policy(self, Bucket: str) -> GetBucketPolicyOutput:
        """Returns the policy of a specified bucket.

        :param self:
        :param Bucket:  <p/><br/>
        :return: Policy:  The bucket policy as a JSON document.<br/>        """
        pass

    def get_bucket_policy_status(self, Bucket: str) -> GetBucketPolicyStatusOutput:
        """Retrieves the policy status for an Amazon S3 bucket, indicating whether the bucket is public.

        :param self:
        :param Bucket:  The name of the Amazon S3 bucket whose policy status you want to retrieve.<br/>
        :return: PolicyStatus:  The policy status for the specified bucket.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsPublic</i> The policy status for this bucket. <code>TRUE</code> indicates that this bucket is public. <code>FALSE</code> indicates that the bucket is not public.<br/>        """
        pass

    def get_bucket_replication(self, Bucket: str) -> GetBucketReplicationOutput:
        """Returns the replication configuration of a bucket. <note>  It can take a while to propagate the put or delete a replication configuration to all Amazon S3 systems. Therefore, a get request soon after put or delete can return a wrong result.  </note>

        :param self:
        :param Bucket:  <p/><br/>
        :return: ReplicationConfiguration:  <p/>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Rules</i> A container for one or more replication rules. A replication configuration must have at least one rule and can contain a maximum of 1,000 rules. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Role</i> The Amazon Resource Name (ARN) of the AWS Identity and Access Management (IAM) role that Amazon S3 assumes when replicating objects. For more information, see <a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/crr-how-setup.html">How to Set Up Cross-Region Replication</a> in the <i>Amazon Simple Storage Service Developer Guide</i>.<br/>        """
        pass

    def get_bucket_request_payment(self, Bucket: str) -> GetBucketRequestPaymentOutput:
        """Returns the request payment configuration of a bucket.

        :param self:
        :param Bucket:  <p/><br/>
        :return: Payer:  Specifies who pays for the download and request fees.<br/>        """
        pass

    def get_bucket_tagging(self, Bucket: str) -> GetBucketTaggingOutput:
        """Returns the tag set associated with the bucket.

        :param self:
        :param Bucket:  <p/><br/>
        :return: TagSet:  <p/><br/>        """
        pass

    def get_bucket_versioning(self, Bucket: str) -> GetBucketVersioningOutput:
        """Returns the versioning state of a bucket.

        :param self:
        :param Bucket:  <p/><br/>
        :return: MFADelete:  Specifies whether MFA delete is enabled in the bucket versioning configuration. This element is only returned if the bucket has been configured with MFA delete. If the bucket has never been so configured, this element is not returned.<br/>Status:  The versioning state of the bucket.<br/>        """
        pass

    def get_bucket_website(self, Bucket: str) -> GetBucketWebsiteOutput:
        """Returns the website configuration for a bucket.

        :param self:
        :param Bucket:  <p/><br/>
        :return: RoutingRules:  <p/><br/>ErrorDocument:  <p/>:<br/>IndexDocument:  <p/>:<br/>RedirectAllRequestsTo:  <p/>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Protocol</i> Protocol to use when redirecting requests. The default is the protocol that is used in the original request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostName</i> Name of the host where requests are redirected.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Suffix</i> A suffix that is appended to a request that is for a directory on the website endpoint (e.g. if the suffix is index.html and you make a request to samplebucket/images/ the data that is returned will be for the object with the key name images/index.html) The suffix must not be empty and must not include a slash character.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> The object key name to use when a 4XX class error occurs.<br/>        """
        pass

    def get_object(self, Key: str, Bucket: str, IfMatch: str=None, IfModifiedSince: IfModifiedSince=None, IfNoneMatch: str=None, IfUnmodifiedSince: IfUnmodifiedSince=None, Range: str=None, ResponseCacheControl: str=None, ResponseContentDisposition: str=None, ResponseContentEncoding: str=None, ResponseContentLanguage: str=None, ResponseContentType: str=None, ResponseExpires: ResponseExpires=None, VersionId: str=None, SSECustomerAlgorithm: str=None, SSECustomerKey: str=None, SSECustomerKeyMD5: str=None, RequestPayer: str=None, PartNumber: int=None) -> GetObjectOutput:
        """Retrieves objects from Amazon S3.

        :param self:
        :param PartNumber:  Part number of the object being read. This is a positive integer between 1 and 10,000. Effectively performs a 'ranged' GET request for the part specified. Useful for downloading just a part of an object.<br/>RequestPayer:  Confirms that the requester knows that she or he will be charged for the request. Bucket owners need not specify this parameter in their requests. Documentation on downloading objects from requester pays buckets can be found at http://docs.aws.amazon.com/AmazonS3/latest/dev/ObjectsinRequesterPaysBuckets.html<br/>SSECustomerKeyMD5:  Specifies the 128-bit MD5 digest of the encryption key according to RFC 1321. Amazon S3 uses this header for a message integrity check to ensure the encryption key was transmitted without error.<br/>SSECustomerKey:  Specifies the customer-provided encryption key for Amazon S3 to use in encrypting data. This value is used to store the object and then it is discarded; Amazon does not store the encryption key. The key must be appropriate for use with the algorithm specified in the x-amz-server-side​-encryption​-customer-algorithm header.<br/>SSECustomerAlgorithm:  Specifies the algorithm to use to when encrypting the object (e.g., AES256).<br/>VersionId:  VersionId used to reference a specific version of the object.<br/>ResponseExpires:  Sets the Expires header of the response.<br/>ResponseContentType:  Sets the Content-Type header of the response.<br/>ResponseContentLanguage:  Sets the Content-Language header of the response.<br/>ResponseContentEncoding:  Sets the Content-Encoding header of the response.<br/>ResponseContentDisposition:  Sets the Content-Disposition header of the response<br/>ResponseCacheControl:  Sets the Cache-Control header of the response.<br/>Range:  Downloads the specified range bytes of an object. For more information about the HTTP Range header, go to http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.35.<br/>Key:  <p/><br/>IfUnmodifiedSince:  Return the object only if it has not been modified since the specified time, otherwise return a 412 (precondition failed).<br/>IfNoneMatch:  Return the object only if its entity tag (ETag) is different from the one specified, otherwise return a 304 (not modified).<br/>IfModifiedSince:  Return the object only if it has been modified since the specified time, otherwise return a 304 (not modified).<br/>IfMatch:  Return the object only if its entity tag (ETag) is the same as the one specified, otherwise return a 412 (precondition failed).<br/>Bucket:  <p/><br/>
        :return: ObjectLockLegalHoldStatus:  Indicates whether this object has an active legal hold. This field is only returned if you have permission to view an object's legal hold status.<br/>ObjectLockRetainUntilDate:  The date and time when this object's object lock will expire.<br/>ObjectLockMode:  The object lock mode currently in place for this object.<br/>TagCount:  The number of tags, if any, on the object.<br/>PartsCount:  The count of parts this object has.<br/>ReplicationStatus:  <p/><br/>RequestCharged:  If present, indicates that the requester was successfully charged for the request.<br/>StorageClass:  <p/><br/>SSEKMSKeyId:  If present, specifies the ID of the AWS Key Management Service (KMS) master encryption key that was used for the object.<br/>SSECustomerKeyMD5:  If server-side encryption with a customer-provided encryption key was requested, the response will include this header to provide round trip message integrity verification of the customer-provided encryption key.<br/>SSECustomerAlgorithm:  If server-side encryption with a customer-provided encryption key was requested, the response will include this header confirming the encryption algorithm used.<br/>Metadata:  A map of metadata to store with the object in S3.<br/>ServerSideEncryption:  The Server-side encryption algorithm used when storing this object in S3 (e.g., AES256, aws:kms).<br/>WebsiteRedirectLocation:  If the bucket is configured as a website, redirects requests for this object to another object in the same bucket or to an external URL. Amazon S3 stores the value of this header in the object metadata.<br/>Expires:  The date and time at which the object is no longer cacheable.<br/>ContentType:  A standard MIME type describing the format of the object data.<br/>ContentRange:  The portion of the object returned in the response.<br/>ContentLanguage:  The language the content is in.<br/>ContentEncoding:  Specifies what content encodings have been applied to the object and thus what decoding mechanisms must be applied to obtain the media-type referenced by the Content-Type header field.<br/>ContentDisposition:  Specifies presentational information for the object.<br/>CacheControl:  Specifies caching behavior along the request/reply chain.<br/>VersionId:  Version of the object.<br/>MissingMeta:  This is set to the number of metadata entries not returned in x-amz-meta headers. This can happen if you create metadata using an API like SOAP that supports more flexible metadata than the REST API. For example, using SOAP, you can create metadata whose values are not legal HTTP headers.<br/>ETag:  An ETag is an opaque identifier assigned by a web server to a specific version of a resource found at a URL<br/>ContentLength:  Size of the body in bytes.<br/>LastModified:  Last modified date of the object<br/>Restore:  Provides information about object restoration operation and expiration time of the restored object copy.<br/>Expiration:  If the object expiration is configured (see PUT Bucket lifecycle), the response includes this header. It includes the expiry-date and rule-id key value pairs providing object expiration information. The value of the rule-id is URL encoded.<br/>AcceptRanges:  <p/><br/>DeleteMarker:  Specifies whether the object retrieved was (true) or was not (false) a Delete Marker. If false, this response header does not appear in the response.<br/>Body:  Object data.<br/>        """
        pass

    def get_object_acl(self, Key: str, Bucket: str, VersionId: str=None, RequestPayer: str=None) -> GetObjectAclOutput:
        """Returns the access control list (ACL) of an object.

        :param self:
        :param RequestPayer:  Confirms that the requester knows that she or he will be charged for the request. Bucket owners need not specify this parameter in their requests. Documentation on downloading objects from requester pays buckets can be found at http://docs.aws.amazon.com/AmazonS3/latest/dev/ObjectsinRequesterPaysBuckets.html<br/>VersionId:  VersionId used to reference a specific version of the object.<br/>Key:  <p/><br/>Bucket:  <p/><br/>
        :return: RequestCharged:  If present, indicates that the requester was successfully charged for the request.<br/>Grants:  A list of grants.<br/>Owner:  <p/>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ID</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayName</i> <p/><br/>        """
        pass

    def get_object_legal_hold(self, Key: str, Bucket: str, VersionId: str=None, RequestPayer: str=None) -> GetObjectLegalHoldOutput:
        """Gets an object's current Legal Hold status.

        :param self:
        :param RequestPayer:  Confirms that the requester knows that she or he will be charged for the request. Bucket owners need not specify this parameter in their requests. Documentation on downloading objects from requester pays buckets can be found at http://docs.aws.amazon.com/AmazonS3/latest/dev/ObjectsinRequesterPaysBuckets.html<br/>VersionId:  The version ID of the object whose Legal Hold status you want to retrieve.<br/>Key:  The key name for the object whose Legal Hold status you want to retrieve.<br/>Bucket:  The bucket containing the object whose Legal Hold status you want to retrieve.<br/>
        :return: LegalHold:  The current Legal Hold status for the specified object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Indicates whether the specified object has a Legal Hold in place.<br/>        """
        pass

    def get_object_lock_configuration(self, Bucket: str) -> GetObjectLockConfigurationOutput:
        """Gets the object lock configuration for a bucket. The rule specified in the object lock configuration will be applied by default to every new object placed in the specified bucket.

        :param self:
        :param Bucket:  The bucket whose object lock configuration you want to retrieve.<br/>
        :return: ObjectLockConfiguration:  The specified bucket's object lock configuration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Rule</i> The object lock rule in place for the specified object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Rule</b>: The object lock rule in place for the specified object.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ObjectLockEnabled</i> Indicates whether this bucket has an object lock configuration enabled.<br/>The default retention period that you want to apply to new objects placed in the specified bucket.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DefaultRetention</b>: The default retention period that you want to apply to new objects placed in the specified bucket.        """
        pass

    def get_object_retention(self, Key: str, Bucket: str, VersionId: str=None, RequestPayer: str=None) -> GetObjectRetentionOutput:
        """Retrieves an object's retention settings.

        :param self:
        :param RequestPayer:  Confirms that the requester knows that she or he will be charged for the request. Bucket owners need not specify this parameter in their requests. Documentation on downloading objects from requester pays buckets can be found at http://docs.aws.amazon.com/AmazonS3/latest/dev/ObjectsinRequesterPaysBuckets.html<br/>VersionId:  The version ID for the object whose retention settings you want to retrieve.<br/>Key:  The key name for the object whose retention settings you want to retrieve.<br/>Bucket:  The bucket containing the object whose retention settings you want to retrieve.<br/>
        :return: Retention:  The container element for an object's retention settings.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RetainUntilDate</i> The date on which this object lock retention expires.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> Indicates the Retention mode for the specified object.<br/>        """
        pass

    def get_object_tagging(self, Key: str, Bucket: str, VersionId: str=None) -> GetObjectTaggingOutput:
        """Returns the tag-set of an object.

        :param self:
        :param VersionId:  <p/><br/>Key:  <p/><br/>Bucket:  <p/><br/>
        :return: TagSet:  <p/><br/>VersionId:  <p/><br/>        """
        pass

    def get_object_torrent(self, Key: str, Bucket: str, RequestPayer: str=None) -> GetObjectTorrentOutput:
        """Return torrent files from a bucket.

        :param self:
        :param RequestPayer:  Confirms that the requester knows that she or he will be charged for the request. Bucket owners need not specify this parameter in their requests. Documentation on downloading objects from requester pays buckets can be found at http://docs.aws.amazon.com/AmazonS3/latest/dev/ObjectsinRequesterPaysBuckets.html<br/>Key:  <p/><br/>Bucket:  <p/><br/>
        :return: RequestCharged:  If present, indicates that the requester was successfully charged for the request.<br/>Body:  <p/><br/>        """
        pass

    def get_public_access_block(self, Bucket: str) -> GetPublicAccessBlockOutput:
        """Retrieves the <code>PublicAccessBlock</code> configuration for an Amazon S3 bucket.

        :param self:
        :param Bucket:  The name of the Amazon S3 bucket whose <code>PublicAccessBlock</code> configuration you want to retrieve. <br/>
        :return: PublicAccessBlockConfiguration:  The <code>PublicAccessBlock</code> configuration currently in effect for this Amazon S3 bucket.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestrictPublicBuckets</i> Specifies whether Amazon S3 should restrict public bucket policies for this bucket. Setting this element to <code>TRUE</code> restricts access to this bucket to only AWS services and authorized users within this account if the bucket has a public policy. Enabling this setting doesn't affect previously stored bucket policies, except that public and cross-account access within any public bucket policy, including non-public delegation to specific accounts, is blocked.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BlockPublicPolicy</i> Specifies whether Amazon S3 should block public bucket policies for this bucket. Setting this element to <code>TRUE</code> causes Amazon S3 to reject calls to PUT Bucket policy if the specified bucket policy allows public access.  Enabling this setting doesn't affect existing bucket policies.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IgnorePublicAcls</i> Specifies whether Amazon S3 should ignore public ACLs for this bucket and objects in this bucket. Setting this element to <code>TRUE</code> causes Amazon S3 to ignore all public ACLs on this bucket and objects in this bucket. Enabling this setting doesn't affect the persistence of any existing ACLs and doesn't prevent new public ACLs from being set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BlockPublicAcls</i> Specifies whether Amazon S3 should block public access control lists (ACLs) for this bucket and objects in this bucket. Setting this element to <code>TRUE</code> causes the following behavior: <ul> <li> PUT Bucket acl and PUT Object acl calls fail if the specified ACL is public. </li> <li> PUT Object calls fail if the request includes a public ACL. </li> </ul> Enabling this setting doesn't affect existing policies or ACLs.<br/>        """
        pass

    def head_bucket(self, Bucket: str):
        """This operation is useful to determine if a bucket exists and you have permission to access it.

        :param self:
        :param Bucket:  <p/><br/>
        :return:         """
        pass

    def head_object(self, Key: str, Bucket: str, IfMatch: str=None, IfModifiedSince: IfModifiedSince=None, IfNoneMatch: str=None, IfUnmodifiedSince: IfUnmodifiedSince=None, Range: str=None, VersionId: str=None, SSECustomerAlgorithm: str=None, SSECustomerKey: str=None, SSECustomerKeyMD5: str=None, RequestPayer: str=None, PartNumber: int=None) -> HeadObjectOutput:
        """The HEAD operation retrieves metadata from an object without returning the object itself. This operation is useful if you're only interested in an object's metadata. To use HEAD, you must have READ access to the object.

        :param self:
        :param PartNumber:  Part number of the object being read. This is a positive integer between 1 and 10,000. Effectively performs a 'ranged' HEAD request for the part specified. Useful querying about the size of the part and the number of parts in this object.<br/>RequestPayer:  Confirms that the requester knows that she or he will be charged for the request. Bucket owners need not specify this parameter in their requests. Documentation on downloading objects from requester pays buckets can be found at http://docs.aws.amazon.com/AmazonS3/latest/dev/ObjectsinRequesterPaysBuckets.html<br/>SSECustomerKeyMD5:  Specifies the 128-bit MD5 digest of the encryption key according to RFC 1321. Amazon S3 uses this header for a message integrity check to ensure the encryption key was transmitted without error.<br/>SSECustomerKey:  Specifies the customer-provided encryption key for Amazon S3 to use in encrypting data. This value is used to store the object and then it is discarded; Amazon does not store the encryption key. The key must be appropriate for use with the algorithm specified in the x-amz-server-side​-encryption​-customer-algorithm header.<br/>SSECustomerAlgorithm:  Specifies the algorithm to use to when encrypting the object (e.g., AES256).<br/>VersionId:  VersionId used to reference a specific version of the object.<br/>Range:  Downloads the specified range bytes of an object. For more information about the HTTP Range header, go to http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.35.<br/>Key:  <p/><br/>IfUnmodifiedSince:  Return the object only if it has not been modified since the specified time, otherwise return a 412 (precondition failed).<br/>IfNoneMatch:  Return the object only if its entity tag (ETag) is different from the one specified, otherwise return a 304 (not modified).<br/>IfModifiedSince:  Return the object only if it has been modified since the specified time, otherwise return a 304 (not modified).<br/>IfMatch:  Return the object only if its entity tag (ETag) is the same as the one specified, otherwise return a 412 (precondition failed).<br/>Bucket:  <p/><br/>
        :return: ObjectLockLegalHoldStatus:  The Legal Hold status for the specified object.<br/>ObjectLockRetainUntilDate:  The date and time when this object's object lock expires.<br/>ObjectLockMode:  The object lock mode currently in place for this object.<br/>PartsCount:  The count of parts this object has.<br/>ReplicationStatus:  <p/><br/>RequestCharged:  If present, indicates that the requester was successfully charged for the request.<br/>StorageClass:  <p/><br/>SSEKMSKeyId:  If present, specifies the ID of the AWS Key Management Service (KMS) master encryption key that was used for the object.<br/>SSECustomerKeyMD5:  If server-side encryption with a customer-provided encryption key was requested, the response will include this header to provide round trip message integrity verification of the customer-provided encryption key.<br/>SSECustomerAlgorithm:  If server-side encryption with a customer-provided encryption key was requested, the response will include this header confirming the encryption algorithm used.<br/>Metadata:  A map of metadata to store with the object in S3.<br/>ServerSideEncryption:  The Server-side encryption algorithm used when storing this object in S3 (e.g., AES256, aws:kms).<br/>WebsiteRedirectLocation:  If the bucket is configured as a website, redirects requests for this object to another object in the same bucket or to an external URL. Amazon S3 stores the value of this header in the object metadata.<br/>Expires:  The date and time at which the object is no longer cacheable.<br/>ContentType:  A standard MIME type describing the format of the object data.<br/>ContentLanguage:  The language the content is in.<br/>ContentEncoding:  Specifies what content encodings have been applied to the object and thus what decoding mechanisms must be applied to obtain the media-type referenced by the Content-Type header field.<br/>ContentDisposition:  Specifies presentational information for the object.<br/>CacheControl:  Specifies caching behavior along the request/reply chain.<br/>VersionId:  Version of the object.<br/>MissingMeta:  This is set to the number of metadata entries not returned in x-amz-meta headers. This can happen if you create metadata using an API like SOAP that supports more flexible metadata than the REST API. For example, using SOAP, you can create metadata whose values are not legal HTTP headers.<br/>ETag:  An ETag is an opaque identifier assigned by a web server to a specific version of a resource found at a URL<br/>ContentLength:  Size of the body in bytes.<br/>LastModified:  Last modified date of the object<br/>Restore:  Provides information about object restoration operation and expiration time of the restored object copy.<br/>Expiration:  If the object expiration is configured (see PUT Bucket lifecycle), the response includes this header. It includes the expiry-date and rule-id key value pairs providing object expiration information. The value of the rule-id is URL encoded.<br/>AcceptRanges:  <p/><br/>DeleteMarker:  Specifies whether the object retrieved was (true) or was not (false) a Delete Marker. If false, this response header does not appear in the response.<br/>        """
        pass

    def list_bucket_analytics_configurations(self, Bucket: str, ContinuationToken: str=None) -> ListBucketAnalyticsConfigurationsOutput:
        """Lists the analytics configurations for the bucket.

        :param self:
        :param ContinuationToken:  The ContinuationToken that represents a placeholder from where this request should begin.<br/>Bucket:  The name of the bucket from which analytics configurations are retrieved.<br/>
        :return: AnalyticsConfigurationList:  The list of analytics configurations for a bucket.<br/>NextContinuationToken:  NextContinuationToken is sent when isTruncated is true, which indicates that there are more analytics configurations to list. The next request must include this NextContinuationToken. The token is obfuscated and is not a usable value.<br/>ContinuationToken:  The ContinuationToken that represents where this request began.<br/>IsTruncated:  Indicates whether the returned list of analytics configurations is complete. A value of true indicates that the list is not complete and the NextContinuationToken will be provided for a subsequent request.<br/>        """
        pass

    def list_bucket_inventory_configurations(self, Bucket: str, ContinuationToken: str=None) -> ListBucketInventoryConfigurationsOutput:
        """Returns a list of inventory configurations for the bucket.

        :param self:
        :param ContinuationToken:  The marker used to continue an inventory configuration listing that has been truncated. Use the NextContinuationToken from a previously truncated list response to continue the listing. The continuation token is an opaque value that Amazon S3 understands.<br/>Bucket:  The name of the bucket containing the inventory configurations to retrieve.<br/>
        :return: NextContinuationToken:  The marker used to continue this inventory configuration listing. Use the NextContinuationToken from this response to continue the listing in a subsequent request. The continuation token is an opaque value that Amazon S3 understands.<br/>IsTruncated:  Indicates whether the returned list of inventory configurations is truncated in this response. A value of true indicates that the list is truncated.<br/>InventoryConfigurationList:  The list of inventory configurations for a bucket.<br/>ContinuationToken:  If sent in the request, the marker that is used as a starting point for this inventory configuration list response.<br/>        """
        pass

    def list_bucket_metrics_configurations(self, Bucket: str, ContinuationToken: str=None) -> ListBucketMetricsConfigurationsOutput:
        """Lists the metrics configurations for the bucket.

        :param self:
        :param ContinuationToken:  The marker that is used to continue a metrics configuration listing that has been truncated. Use the NextContinuationToken from a previously truncated list response to continue the listing. The continuation token is an opaque value that Amazon S3 understands.<br/>Bucket:  The name of the bucket containing the metrics configurations to retrieve.<br/>
        :return: MetricsConfigurationList:  The list of metrics configurations for a bucket.<br/>NextContinuationToken:  The marker used to continue a metrics configuration listing that has been truncated. Use the NextContinuationToken from a previously truncated list response to continue the listing. The continuation token is an opaque value that Amazon S3 understands.<br/>ContinuationToken:  The marker that is used as a starting point for this metrics configuration list response. This value is present if it was sent in the request.<br/>IsTruncated:  Indicates whether the returned list of metrics configurations is complete. A value of true indicates that the list is not complete and the NextContinuationToken will be provided for a subsequent request.<br/>        """
        pass

    def list_buckets(self) -> ListBucketsOutput:
        """Returns a list of all buckets owned by the authenticated sender of the request.

        :param self:
        :param 
        :return: Owner:  <p/>:<br/>Buckets:  <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ID</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayName</i> <p/><br/>        """
        pass

    def list_multipart_uploads(self, Bucket: str, Delimiter: str=None, EncodingType: str=None, KeyMarker: str=None, MaxUploads: int=None, Prefix: str=None, UploadIdMarker: str=None) -> ListMultipartUploadsOutput:
        """This operation lists in-progress multipart uploads.

        :param self:
        :param UploadIdMarker:  Together with key-marker, specifies the multipart upload after which listing should begin. If key-marker is not specified, the upload-id-marker parameter is ignored.<br/>Prefix:  Lists in-progress uploads only for those keys that begin with the specified prefix.<br/>MaxUploads:  Sets the maximum number of multipart uploads, from 1 to 1,000, to return in the response body. 1,000 is the maximum number of uploads that can be returned in a response.<br/>KeyMarker:  Together with upload-id-marker, this parameter specifies the multipart upload after which listing should begin.<br/>EncodingType:  Requests Amazon S3 to encode the object keys in the response and specifies the encoding method to use. An object key may contain any Unicode character; however, XML 1.0 parser cannot parse some characters, such as characters with an ASCII value from 0 to 10. For characters that are not supported in XML 1.0, you can add this parameter to request that Amazon S3 encode the keys in the response.<br/>Delimiter:  Character you use to group keys.<br/>Bucket:  <p/><br/>
        :return: EncodingType:  Encoding type used by Amazon S3 to encode object keys in the response.<br/>CommonPrefixes:  <p/><br/>Uploads:  <p/><br/>IsTruncated:  Indicates whether the returned list of multipart uploads is truncated. A value of true indicates that the list was truncated. The list can be truncated if the number of multipart uploads exceeds the limit allowed or specified by max uploads.<br/>MaxUploads:  Maximum number of multipart uploads that could have been included in the response.<br/>NextUploadIdMarker:  When a list is truncated, this element specifies the value that should be used for the upload-id-marker request parameter in a subsequent request.<br/>Delimiter:  <p/><br/>Prefix:  When a prefix is provided in the request, this field contains the specified prefix. The result contains only keys starting with the specified prefix.<br/>NextKeyMarker:  When a list is truncated, this element specifies the value that should be used for the key-marker request parameter in a subsequent request.<br/>UploadIdMarker:  Upload ID after which listing began.<br/>KeyMarker:  The key at or after which the listing began.<br/>Bucket:  Name of the bucket to which the multipart upload was initiated.<br/>        """
        pass

    def list_object_versions(self, Bucket: str, Delimiter: str=None, EncodingType: str=None, KeyMarker: str=None, MaxKeys: int=None, Prefix: str=None, VersionIdMarker: str=None) -> ListObjectVersionsOutput:
        """Returns metadata about all of the versions of objects in a bucket.

        :param self:
        :param VersionIdMarker:  Specifies the object version you want to start listing from.<br/>Prefix:  Limits the response to keys that begin with the specified prefix.<br/>MaxKeys:  Sets the maximum number of keys returned in the response. The response might contain fewer keys but will never contain more.<br/>KeyMarker:  Specifies the key to start with when listing objects in a bucket.<br/>EncodingType:  Requests Amazon S3 to encode the object keys in the response and specifies the encoding method to use. An object key may contain any Unicode character; however, XML 1.0 parser cannot parse some characters, such as characters with an ASCII value from 0 to 10. For characters that are not supported in XML 1.0, you can add this parameter to request that Amazon S3 encode the keys in the response.<br/>Delimiter:  A delimiter is a character you use to group keys.<br/>Bucket:  <p/><br/>
        :return: EncodingType:  Encoding type used by Amazon S3 to encode object keys in the response.<br/>CommonPrefixes:  <p/><br/>MaxKeys:  <p/><br/>Delimiter:  <p/><br/>Prefix:  <p/><br/>Name:  <p/><br/>DeleteMarkers:  <p/><br/>Versions:  <p/><br/>NextVersionIdMarker:  Use this value for the next version id marker parameter in a subsequent request.<br/>NextKeyMarker:  Use this value for the key marker request parameter in a subsequent request.<br/>VersionIdMarker:  <p/><br/>KeyMarker:  Marks the last Key returned in a truncated response.<br/>IsTruncated:  A flag that indicates whether or not Amazon S3 returned all of the results that satisfied the search criteria. If your results were truncated, you can make a follow-up paginated request using the NextKeyMarker and NextVersionIdMarker response parameters as a starting place in another request to return the rest of the results.<br/>        """
        pass

    def list_objects(self, Bucket: str, Delimiter: str=None, EncodingType: str=None, Marker: str=None, MaxKeys: int=None, Prefix: str=None, RequestPayer: str=None) -> ListObjectsOutput:
        """Returns some or all (up to 1000) of the objects in a bucket. You can use the request parameters as selection criteria to return a subset of the objects in a bucket.

        :param self:
        :param RequestPayer:  Confirms that the requester knows that she or he will be charged for the list objects request. Bucket owners need not specify this parameter in their requests.<br/>Prefix:  Limits the response to keys that begin with the specified prefix.<br/>MaxKeys:  Sets the maximum number of keys returned in the response. The response might contain fewer keys but will never contain more.<br/>Marker:  Specifies the key to start with when listing objects in a bucket.<br/>EncodingType:  Requests Amazon S3 to encode the object keys in the response and specifies the encoding method to use. An object key may contain any Unicode character; however, XML 1.0 parser cannot parse some characters, such as characters with an ASCII value from 0 to 10. For characters that are not supported in XML 1.0, you can add this parameter to request that Amazon S3 encode the keys in the response.<br/>Delimiter:  A delimiter is a character you use to group keys.<br/>Bucket:  <p/><br/>
        :return: EncodingType:  Encoding type used by Amazon S3 to encode object keys in the response.<br/>CommonPrefixes:  <p/><br/>MaxKeys:  <p/><br/>Delimiter:  <p/><br/>Prefix:  <p/><br/>Name:  <p/><br/>Contents:  <p/><br/>NextMarker:  When response is truncated (the IsTruncated element value in the response is true), you can use the key name in this field as marker in the subsequent request to get next set of objects. Amazon S3 lists objects in alphabetical order Note: This element is returned only if you have delimiter request parameter specified. If response does not include the NextMaker and it is truncated, you can use the value of the last Key in the response as the marker in the subsequent request to get the next set of object keys.<br/>Marker:  <p/><br/>IsTruncated:  A flag that indicates whether or not Amazon S3 returned all of the results that satisfied the search criteria.<br/>        """
        pass

    def list_objects_v2(self, Bucket: str, Delimiter: str=None, EncodingType: str=None, MaxKeys: int=None, Prefix: str=None, ContinuationToken: str=None, FetchOwner: FetchOwner=None, StartAfter: str=None, RequestPayer: str=None) -> ListObjectsV2Output:
        """Returns some or all (up to 1000) of the objects in a bucket. You can use the request parameters as selection criteria to return a subset of the objects in a bucket. Note: ListObjectsV2 is the revised List Objects API and we recommend you use this revised API for new application development.

        :param self:
        :param RequestPayer:  Confirms that the requester knows that she or he will be charged for the list objects request in V2 style. Bucket owners need not specify this parameter in their requests.<br/>StartAfter:  StartAfter is where you want Amazon S3 to start listing from. Amazon S3 starts listing after this specified key. StartAfter can be any key in the bucket<br/>FetchOwner:  The owner field is not present in listV2 by default, if you want to return owner field with each key in the result then set the fetch owner field to true<br/>ContinuationToken:  ContinuationToken indicates Amazon S3 that the list is being continued on this bucket with a token. ContinuationToken is obfuscated and is not a real key<br/>Prefix:  Limits the response to keys that begin with the specified prefix.<br/>MaxKeys:  Sets the maximum number of keys returned in the response. The response might contain fewer keys but will never contain more.<br/>EncodingType:  Encoding type used by Amazon S3 to encode object keys in the response.<br/>Delimiter:  A delimiter is a character you use to group keys.<br/>Bucket:  Name of the bucket to list.<br/>
        :return: StartAfter:  StartAfter is where you want Amazon S3 to start listing from. Amazon S3 starts listing after this specified key. StartAfter can be any key in the bucket<br/>NextContinuationToken:  NextContinuationToken is sent when isTruncated is true which means there are more keys in the bucket that can be listed. The next list requests to Amazon S3 can be continued with this NextContinuationToken. NextContinuationToken is obfuscated and is not a real key<br/>ContinuationToken:  ContinuationToken indicates Amazon S3 that the list is being continued on this bucket with a token. ContinuationToken is obfuscated and is not a real key<br/>KeyCount:  KeyCount is the number of keys returned with this request. KeyCount will always be less than equals to MaxKeys field. Say you ask for 50 keys, your result will include less than equals 50 keys <br/>EncodingType:  Encoding type used by Amazon S3 to encode object keys in the response.<br/>CommonPrefixes:  CommonPrefixes contains all (if there are any) keys between Prefix and the next occurrence of the string specified by delimiter<br/>MaxKeys:  Sets the maximum number of keys returned in the response. The response might contain fewer keys but will never contain more.<br/>Delimiter:  A delimiter is a character you use to group keys.<br/>Prefix:  Limits the response to keys that begin with the specified prefix.<br/>Name:  Name of the bucket to list.<br/>Contents:  Metadata about each object returned.<br/>IsTruncated:  A flag that indicates whether or not Amazon S3 returned all of the results that satisfied the search criteria.<br/>        """
        pass

    def list_parts(self, UploadId: str, Key: str, Bucket: str, MaxParts: int=None, PartNumberMarker: int=None, RequestPayer: str=None) -> ListPartsOutput:
        """Lists the parts that have been uploaded for a specific multipart upload.

        :param self:
        :param RequestPayer:  Confirms that the requester knows that she or he will be charged for the request. Bucket owners need not specify this parameter in their requests. Documentation on downloading objects from requester pays buckets can be found at http://docs.aws.amazon.com/AmazonS3/latest/dev/ObjectsinRequesterPaysBuckets.html<br/>UploadId:  Upload ID identifying the multipart upload whose parts are being listed.<br/>PartNumberMarker:  Specifies the part after which listing should begin. Only parts with higher part numbers will be listed.<br/>MaxParts:  Sets the maximum number of parts to return.<br/>Key:  <p/><br/>Bucket:  <p/><br/>
        :return: RequestCharged:  If present, indicates that the requester was successfully charged for the request.<br/>StorageClass:  The class of storage used to store the object.<br/>Owner:  <p/>:<br/>Initiator:  Identifies who initiated the multipart upload.:<br/>Parts:  <p/><br/>IsTruncated:  Indicates whether the returned list of parts is truncated.<br/>MaxParts:  Maximum number of parts that were allowed in the response.<br/>NextPartNumberMarker:  When a list is truncated, this element specifies the last part in the list, as well as the value to use for the part-number-marker request parameter in a subsequent request.<br/>PartNumberMarker:  Part number after which listing begins.<br/>UploadId:  Upload ID identifying the multipart upload whose parts are being listed.<br/>Key:  Object key for which the multipart upload was initiated.<br/>Bucket:  Name of the bucket to which the multipart upload was initiated.<br/>AbortRuleId:  Id of the lifecycle rule that makes a multipart upload eligible for abort operation.<br/>AbortDate:  Date when multipart upload will become eligible for abort operation by lifecycle.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayName</i> Name of the Principal.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ID</i> If the principal is an AWS account, it provides the Canonical User ID. If the principal is an IAM User, it provides a user ARN value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ID</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayName</i> <p/><br/>        """
        pass

    def put_bucket_accelerate_configuration(self, AccelerateConfiguration: AccelerateConfiguration, Bucket: str):
        """Sets the accelerate configuration of an existing bucket.

        :param self:
        :param AccelerateConfiguration:  Specifies the Accelerate Configuration you want to set for the bucket.:<br/>Bucket:  Name of the bucket for which the accelerate configuration is set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Specifies the transfer acceleration status of the bucket.<br/>
        :return:         """
        pass

    def put_bucket_acl(self, Bucket: str, ACL: str=None, AccessControlPolicy: AccessControlPolicy=None, ContentMD5: str=None, GrantFullControl: str=None, GrantRead: str=None, GrantReadACP: str=None, GrantWrite: str=None, GrantWriteACP: str=None):
        """Sets the permissions on a bucket using access control lists (ACL).

        :param self:
        :param GrantWriteACP:  Allows grantee to write the ACL for the applicable bucket.<br/>GrantWrite:  Allows grantee to create, overwrite, and delete any object in the bucket.<br/>GrantReadACP:  Allows grantee to read the bucket ACL.<br/>GrantRead:  Allows grantee to list the objects in the bucket.<br/>GrantFullControl:  Allows grantee the read, write, read ACP, and write ACP permissions on the bucket.<br/>ContentMD5:  <p/><br/>Bucket:  <p/><br/>AccessControlPolicy:  Contains the elements that set the ACL permissions for an object per grantee.:<br/>ACL:  The canned ACL to apply to the bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Owner</i> Container for the bucket owner's display name and ID.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Owner</b>: Container for the bucket owner's display name and ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Grants</i> A list of grants.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ID</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayName</i> <p/><br/>
        :return:         """
        pass

    def put_bucket_analytics_configuration(self, AnalyticsConfiguration: AnalyticsConfiguration, Id: str, Bucket: str):
        """Sets an analytics configuration for the bucket (specified by the analytics configuration ID).

        :param self:
        :param AnalyticsConfiguration:  The configuration and any analyses for the analytics filter.:<br/>Id:  The ID that identifies the analytics configuration.<br/>Bucket:  The name of the bucket to which an analytics configuration is stored.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageClassAnalysis</i>  Contains data related to access patterns to be collected and made available to analyze the tradeoffs between different storage classes. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StorageClassAnalysis</b>:  Contains data related to access patterns to be collected and made available to analyze the tradeoffs between different storage classes. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Filter</i> The filter used to describe a set of objects for analyses. A filter must have exactly one prefix, one tag, or one conjunction (AnalyticsAndOperator). If no filter is provided, all objects will be considered in any analysis.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Filter</b>: The filter used to describe a set of objects for analyses. A filter must have exactly one prefix, one tag, or one conjunction (AnalyticsAndOperator). If no filter is provided, all objects will be considered in any analysis.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID that identifies the analytics configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> The prefix to use when evaluating an analytics filter.<br/>The tag to use when evaluating an analytics filter.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Tag</b>: The tag to use when evaluating an analytics filter.Specifies how data related to the storage class analysis for an Amazon S3 bucket should be exported.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DataExport</b>: Specifies how data related to the storage class analysis for an Amazon S3 bucket should be exported.
        :return:         """
        pass

    def put_bucket_cors(self, CORSConfiguration: CORSConfiguration, Bucket: str, ContentMD5: str=None):
        """Sets the CORS configuration for a bucket.

        :param self:
        :param ContentMD5:  <p/><br/>CORSConfiguration:  <p/>:<br/>Bucket:  <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CORSRules</i> A set of allowed origins and methods.<br/>
        :return:         """
        pass

    def put_bucket_encryption(self, ServerSideEncryptionConfiguration: ServerSideEncryptionConfiguration, Bucket: str, ContentMD5: str=None):
        """Creates a new server-side encryption configuration (or replaces an existing one, if present).

        :param self:
        :param ServerSideEncryptionConfiguration:  <p/>:<br/>ContentMD5:  The base64-encoded 128-bit MD5 digest of the server-side encryption configuration. This parameter is auto-populated when using the command from the CLI.<br/>Bucket:  Specifies default encryption for a bucket using server-side encryption with Amazon S3-managed keys (SSE-S3) or AWS KMS-managed keys (SSE-KMS). For information about the Amazon S3 default encryption feature, see <a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/bucket-encryption.html">Amazon S3 Default Bucket Encryption</a> in the <i>Amazon Simple Storage Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Rules</i> Container for information about a particular server-side encryption configuration rule.<br/>
        :return:         """
        pass

    def put_bucket_inventory_configuration(self, InventoryConfiguration: InventoryConfiguration, Id: str, Bucket: str):
        """Adds an inventory configuration (identified by the inventory ID) from the bucket.

        :param self:
        :param InventoryConfiguration:  Specifies the inventory configuration.:<br/>Id:  The ID used to identify the inventory configuration.<br/>Bucket:  The name of the bucket where the inventory configuration will be stored.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Schedule</i> Specifies the schedule for generating inventory results.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Schedule</b>: Specifies the schedule for generating inventory results.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OptionalFields</i> Contains the optional fields that are included in the inventory results.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IncludedObjectVersions</i> Object versions to include in the inventory list. If set to <code>All</code>, the list includes all the object versions, which adds the version-related fields <code>VersionId</code>, <code>IsLatest</code>, and <code>DeleteMarker</code> to the list. If set to <code>Current</code>, the list does not contain these version-related fields.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID used to identify the inventory configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Filter</i> Specifies an inventory filter. The inventory only includes objects that meet the filter's criteria.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Filter</b>: Specifies an inventory filter. The inventory only includes objects that meet the filter's criteria.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsEnabled</i> Specifies whether the inventory is enabled or disabled. If set to <code>True</code>, an inventory list is generated. If set to <code>False</code>, no inventory list is generated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Destination</i> Contains information about where to publish the inventory results.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Destination</b>: Contains information about where to publish the inventory results.<br/>Contains the bucket name, file format, bucket owner (optional), and prefix (optional) where inventory results are published.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3BucketDestination</b>: Contains the bucket name, file format, bucket owner (optional), and prefix (optional) where inventory results are published.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> The prefix that an object must have to be included in the inventory results.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Frequency</i> Specifies how frequently inventory results are produced.<br/>
        :return:         """
        pass

    def put_bucket_lifecycle(self, Bucket: str, ContentMD5: str=None, LifecycleConfiguration: LifecycleConfiguration=None):
        """ No longer used, see the PutBucketLifecycleConfiguration operation.

        :param self:
        :param LifecycleConfiguration:  <p/>:<br/>ContentMD5:  <p/><br/>Bucket:  <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Rules</i> <p/><br/>
        :return:         """
        pass

    def put_bucket_lifecycle_configuration(self, Bucket: str, LifecycleConfiguration: BucketLifecycleConfiguration=None):
        """Sets lifecycle configuration for your bucket. If a lifecycle configuration exists, it replaces it.

        :param self:
        :param LifecycleConfiguration:  <p/>:<br/>Bucket:  <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Rules</i> A lifecycle rule for individual objects in an Amazon S3 bucket.<br/>
        :return:         """
        pass

    def put_bucket_logging(self, BucketLoggingStatus: BucketLoggingStatus, Bucket: str, ContentMD5: str=None):
        """Set the logging parameters for a bucket and to specify permissions for who can view and modify the logging parameters. To set the logging status of a bucket, you must be the bucket owner.

        :param self:
        :param ContentMD5:  <p/><br/>BucketLoggingStatus:  <p/>:<br/>Bucket:  <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LoggingEnabled</i> <p/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LoggingEnabled</b>: <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetPrefix</i> A prefix for all log object keys. If you store log files from multiple Amazon S3 buckets in a single bucket, you can use a prefix to distinguish which log files came from which bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetGrants</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetBucket</i> Specifies the bucket where you want Amazon S3 to store server access logs. You can have your logs delivered to any bucket that you own, including the same bucket that is being logged. You can also configure multiple buckets to deliver their logs to the same target bucket. In this case you should choose a different TargetPrefix for each source bucket so that the delivered log files can be distinguished by key.<br/>
        :return:         """
        pass

    def put_bucket_metrics_configuration(self, MetricsConfiguration: MetricsConfiguration, Id: str, Bucket: str):
        """Sets a metrics configuration (specified by the metrics configuration ID) for the bucket.

        :param self:
        :param MetricsConfiguration:  Specifies the metrics configuration.:<br/>Id:  The ID used to identify the metrics configuration.<br/>Bucket:  The name of the bucket for which the metrics configuration is set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Filter</i> Specifies a metrics configuration filter. The metrics configuration will only include objects that meet the filter's criteria. A filter must be a prefix, a tag, or a conjunction (MetricsAndOperator).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Filter</b>: Specifies a metrics configuration filter. The metrics configuration will only include objects that meet the filter's criteria. A filter must be a prefix, a tag, or a conjunction (MetricsAndOperator).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID used to identify the metrics configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> The prefix used when evaluating a metrics filter.<br/>The tag used when evaluating a metrics filter.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Tag</b>: The tag used when evaluating a metrics filter.
        :return:         """
        pass

    def put_bucket_notification(self, NotificationConfiguration: NotificationConfigurationDeprecated, Bucket: str, ContentMD5: str=None):
        """ No longer used, see the PutBucketNotificationConfiguration operation.

        :param self:
        :param NotificationConfiguration:  <p/>:<br/>ContentMD5:  <p/><br/>Bucket:  <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudFunctionConfiguration</i> <p/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CloudFunctionConfiguration</b>: <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QueueConfiguration</i> <p/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>QueueConfiguration</b>: <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicConfiguration</i> <p/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>TopicConfiguration</b>: <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Topic</i> Amazon SNS topic to which Amazon S3 will publish a message to report the specified events for the bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Event</i> Bucket event for which to send notifications.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Events</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> An optional unique identifier for configurations in a notification configuration. If you don't provide one, Amazon S3 will assign an ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Queue</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Events</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Event</i> The bucket event for which to send notifications.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> An optional unique identifier for configurations in a notification configuration. If you don't provide one, Amazon S3 will assign an ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InvocationRole</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudFunction</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Events</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Event</i> The bucket event for which to send notifications.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> An optional unique identifier for configurations in a notification configuration. If you don't provide one, Amazon S3 will assign an ID.<br/>
        :return:         """
        pass

    def put_bucket_notification_configuration(self, NotificationConfiguration: NotificationConfiguration, Bucket: str):
        """Enables notifications of specified events for a bucket.

        :param self:
        :param NotificationConfiguration:  <p/>:<br/>Bucket:  <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LambdaFunctionConfigurations</i> Describes the AWS Lambda functions to invoke and the events for which to invoke them.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QueueConfigurations</i> The Amazon Simple Queue Service queues to publish messages to and the events for which to publish messages.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicConfigurations</i> The topic to which notifications are sent and the events for which notifications are generated.<br/>
        :return:         """
        pass

    def put_bucket_policy(self, Policy: str, Bucket: str, ContentMD5: str=None, ConfirmRemoveSelfBucketAccess: ConfirmRemoveSelfBucketAccess=None):
        """Applies an Amazon S3 bucket policy to an Amazon S3 bucket.

        :param self:
        :param Policy:  The bucket policy as a JSON document.<br/>ConfirmRemoveSelfBucketAccess:  Set this parameter to true to confirm that you want to remove your permissions to change this bucket policy in the future.<br/>ContentMD5:  <p/><br/>Bucket:  <p/><br/>
        :return:         """
        pass

    def put_bucket_replication(self, ReplicationConfiguration: ReplicationConfiguration, Bucket: str, ContentMD5: str=None, Token: str=None):
        """ Creates a replication configuration or replaces an existing one. For more information, see <a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/crr.html">Cross-Region Replication (CRR)</a> in the <i>Amazon S3 Developer Guide</i>. 

        :param self:
        :param Token:  A token that allows Amazon S3 object lock to be enabled for an existing bucket.<br/>ReplicationConfiguration:  <p/>:<br/>ContentMD5:  The base64-encoded 128-bit MD5 digest of the data. You must use this header as a message integrity check to verify that the request body was not corrupted in transit.<br/>Bucket:  <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Rules</i> A container for one or more replication rules. A replication configuration must have at least one rule and can contain a maximum of 1,000 rules. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Role</i> The Amazon Resource Name (ARN) of the AWS Identity and Access Management (IAM) role that Amazon S3 assumes when replicating objects. For more information, see <a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/crr-how-setup.html">How to Set Up Cross-Region Replication</a> in the <i>Amazon Simple Storage Service Developer Guide</i>.<br/>
        :return:         """
        pass

    def put_bucket_request_payment(self, RequestPaymentConfiguration: RequestPaymentConfiguration, Bucket: str, ContentMD5: str=None):
        """Sets the request payment configuration for a bucket. By default, the bucket owner pays for downloads from the bucket. This configuration parameter enables the bucket owner (only) to specify that the person requesting the download will be charged for the download. Documentation on requester pays buckets can be found at http://docs.aws.amazon.com/AmazonS3/latest/dev/RequesterPaysBuckets.html

        :param self:
        :param RequestPaymentConfiguration:  <p/>:<br/>ContentMD5:  <p/><br/>Bucket:  <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Payer</i> Specifies who pays for the download and request fees.<br/>
        :return:         """
        pass

    def put_bucket_tagging(self, Tagging: Tagging, Bucket: str, ContentMD5: str=None):
        """Sets the tags for a bucket.

        :param self:
        :param Tagging:  <p/>:<br/>ContentMD5:  <p/><br/>Bucket:  <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TagSet</i> <p/><br/>
        :return:         """
        pass

    def put_bucket_versioning(self, VersioningConfiguration: VersioningConfiguration, Bucket: str, ContentMD5: str=None, MFA: str=None):
        """Sets the versioning state of an existing bucket. To set the versioning state, you must be the bucket owner.

        :param self:
        :param VersioningConfiguration:  <p/>:<br/>MFA:  The concatenation of the authentication device's serial number, a space, and the value that is displayed on your authentication device.<br/>ContentMD5:  <p/><br/>Bucket:  <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The versioning state of the bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MFADelete</i> Specifies whether MFA delete is enabled in the bucket versioning configuration. This element is only returned if the bucket has been configured with MFA delete. If the bucket has never been so configured, this element is not returned.<br/>
        :return:         """
        pass

    def put_bucket_website(self, WebsiteConfiguration: WebsiteConfiguration, Bucket: str, ContentMD5: str=None):
        """Set the website configuration for a bucket.

        :param self:
        :param WebsiteConfiguration:  <p/>:<br/>ContentMD5:  <p/><br/>Bucket:  <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoutingRules</i> Rules that define when a redirect is applied and the redirect behavior.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RedirectAllRequestsTo</i> The redirect behavior for every request to this bucket's website endpoint. <important> If you specify this property, you can't specify any other property. </important>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RedirectAllRequestsTo</b>: The redirect behavior for every request to this bucket's website endpoint. <important> If you specify this property, you can't specify any other property. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IndexDocument</i> The name of the index document for the website.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>IndexDocument</b>: The name of the index document for the website.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorDocument</i> The name of the error document for the website.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ErrorDocument</b>: The name of the error document for the website.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> The object key name to use when a 4XX class error occurs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Suffix</i> A suffix that is appended to a request that is for a directory on the website endpoint (e.g. if the suffix is index.html and you make a request to samplebucket/images/ the data that is returned will be for the object with the key name images/index.html) The suffix must not be empty and must not include a slash character.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Protocol</i> Protocol to use when redirecting requests. The default is the protocol that is used in the original request.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HostName</i> Name of the host where requests are redirected.<br/>
        :return:         """
        pass

    def put_object(self, Key: str, Bucket: str, ACL: str=None, Body: Body=None, CacheControl: str=None, ContentDisposition: str=None, ContentEncoding: str=None, ContentLanguage: str=None, ContentLength: ContentLength=None, ContentMD5: str=None, ContentType: str=None, Expires: Expires=None, GrantFullControl: str=None, GrantRead: str=None, GrantReadACP: str=None, GrantWriteACP: str=None, Metadata: Metadata=None, ServerSideEncryption: str=None, StorageClass: str=None, WebsiteRedirectLocation: str=None, SSECustomerAlgorithm: str=None, SSECustomerKey: str=None, SSECustomerKeyMD5: str=None, SSEKMSKeyId: str=None, SSEKMSEncryptionContext: str=None, RequestPayer: str=None, Tagging: str=None, ObjectLockMode: str=None, ObjectLockRetainUntilDate: ObjectLockRetainUntilDate=None, ObjectLockLegalHoldStatus: str=None) -> PutObjectOutput:
        """Adds an object to a bucket.

        :param self:
        :param ObjectLockLegalHoldStatus:  The Legal Hold status that you want to apply to the specified object.<br/>ObjectLockRetainUntilDate:  The date and time when you want this object's object lock to expire.<br/>ObjectLockMode:  The object lock mode that you want to apply to this object.<br/>Tagging:  The tag-set for the object. The tag-set must be encoded as URL Query parameters. (For example, "Key1=Value1")<br/>RequestPayer:  Confirms that the requester knows that she or he will be charged for the request. Bucket owners need not specify this parameter in their requests. Documentation on downloading objects from requester pays buckets can be found at http://docs.aws.amazon.com/AmazonS3/latest/dev/ObjectsinRequesterPaysBuckets.html<br/>SSEKMSEncryptionContext:  Specifies the AWS KMS Encryption Context to use for object encryption. The value of this header is a base64-encoded UTF-8 string holding JSON with the encryption context key-value pairs.<br/>SSEKMSKeyId:  Specifies the AWS KMS key ID to use for object encryption. All GET and PUT requests for an object protected by AWS KMS will fail if not made via SSL or using SigV4. Documentation on configuring any of the officially supported AWS SDKs and CLI can be found at http://docs.aws.amazon.com/AmazonS3/latest/dev/UsingAWSSDK.html#specify-signature-version<br/>SSECustomerKeyMD5:  Specifies the 128-bit MD5 digest of the encryption key according to RFC 1321. Amazon S3 uses this header for a message integrity check to ensure the encryption key was transmitted without error.<br/>SSECustomerKey:  Specifies the customer-provided encryption key for Amazon S3 to use in encrypting data. This value is used to store the object and then it is discarded; Amazon does not store the encryption key. The key must be appropriate for use with the algorithm specified in the x-amz-server-side​-encryption​-customer-algorithm header.<br/>SSECustomerAlgorithm:  Specifies the algorithm to use to when encrypting the object (e.g., AES256).<br/>WebsiteRedirectLocation:  If the bucket is configured as a website, redirects requests for this object to another object in the same bucket or to an external URL. Amazon S3 stores the value of this header in the object metadata.<br/>StorageClass:  The type of storage to use for the object. Defaults to 'STANDARD'.<br/>ServerSideEncryption:  The Server-side encryption algorithm used when storing this object in S3 (e.g., AES256, aws:kms).<br/>Metadata:  A map of metadata to store with the object in S3.<br/>Key:  Object key for which the PUT operation was initiated.<br/>GrantWriteACP:  Allows grantee to write the ACL for the applicable object.<br/>GrantReadACP:  Allows grantee to read the object ACL.<br/>GrantRead:  Allows grantee to read the object data and its metadata.<br/>GrantFullControl:  Gives the grantee READ, READ_ACP, and WRITE_ACP permissions on the object.<br/>Expires:  The date and time at which the object is no longer cacheable.<br/>ContentType:  A standard MIME type describing the format of the object data.<br/>ContentMD5:  The base64-encoded 128-bit MD5 digest of the part data. This parameter is auto-populated when using the command from the CLI. This parameted is required if object lock parameters are specified.<br/>ContentLength:  Size of the body in bytes. This parameter is useful when the size of the body cannot be determined automatically.<br/>ContentLanguage:  The language the content is in.<br/>ContentEncoding:  Specifies what content encodings have been applied to the object and thus what decoding mechanisms must be applied to obtain the media-type referenced by the Content-Type header field.<br/>ContentDisposition:  Specifies presentational information for the object.<br/>CacheControl:  Specifies caching behavior along the request/reply chain.<br/>Bucket:  Name of the bucket to which the PUT operation was initiated.<br/>Body:  Object data.<br/>ACL:  The canned ACL to apply to the object.<br/>
        :return: RequestCharged:  If present, indicates that the requester was successfully charged for the request.<br/>SSEKMSEncryptionContext:  If present, specifies the AWS KMS Encryption Context to use for object encryption. The value of this header is a base64-encoded UTF-8 string holding JSON with the encryption context key-value pairs.<br/>SSEKMSKeyId:  If present, specifies the ID of the AWS Key Management Service (KMS) master encryption key that was used for the object.<br/>SSECustomerKeyMD5:  If server-side encryption with a customer-provided encryption key was requested, the response will include this header to provide round trip message integrity verification of the customer-provided encryption key.<br/>SSECustomerAlgorithm:  If server-side encryption with a customer-provided encryption key was requested, the response will include this header confirming the encryption algorithm used.<br/>VersionId:  Version of the object.<br/>ServerSideEncryption:  The Server-side encryption algorithm used when storing this object in S3 (e.g., AES256, aws:kms).<br/>ETag:  Entity tag for the uploaded object.<br/>Expiration:  If the object expiration is configured, this will contain the expiration date (expiry-date) and rule ID (rule-id). The value of rule-id is URL encoded.<br/>        """
        pass

    def put_object_acl(self, Key: str, Bucket: str, ACL: str=None, AccessControlPolicy: AccessControlPolicy=None, ContentMD5: str=None, GrantFullControl: str=None, GrantRead: str=None, GrantReadACP: str=None, GrantWrite: str=None, GrantWriteACP: str=None, RequestPayer: str=None, VersionId: str=None) -> PutObjectAclOutput:
        """uses the acl subresource to set the access control list (ACL) permissions for an object that already exists in a bucket

        :param self:
        :param VersionId:  VersionId used to reference a specific version of the object.<br/>RequestPayer:  Confirms that the requester knows that she or he will be charged for the request. Bucket owners need not specify this parameter in their requests. Documentation on downloading objects from requester pays buckets can be found at http://docs.aws.amazon.com/AmazonS3/latest/dev/ObjectsinRequesterPaysBuckets.html<br/>Key:  <p/><br/>GrantWriteACP:  Allows grantee to write the ACL for the applicable bucket.<br/>GrantWrite:  Allows grantee to create, overwrite, and delete any object in the bucket.<br/>GrantReadACP:  Allows grantee to read the bucket ACL.<br/>GrantRead:  Allows grantee to list the objects in the bucket.<br/>GrantFullControl:  Allows grantee the read, write, read ACP, and write ACP permissions on the bucket.<br/>ContentMD5:  <p/><br/>Bucket:  <p/><br/>AccessControlPolicy:  Contains the elements that set the ACL permissions for an object per grantee.:<br/>ACL:  The canned ACL to apply to the object.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Owner</i> Container for the bucket owner's display name and ID.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Owner</b>: Container for the bucket owner's display name and ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Grants</i> A list of grants.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ID</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayName</i> <p/><br/>
        :return: RequestCharged:  If present, indicates that the requester was successfully charged for the request.<br/>        """
        pass

    def put_object_legal_hold(self, Key: str, Bucket: str, LegalHold: ObjectLockLegalHold=None, RequestPayer: str=None, VersionId: str=None, ContentMD5: str=None) -> PutObjectLegalHoldOutput:
        """Applies a Legal Hold configuration to the specified object.

        :param self:
        :param ContentMD5:  The MD5 hash for the request body.<br/>VersionId:  The version ID of the object that you want to place a Legal Hold on.<br/>RequestPayer:  Confirms that the requester knows that she or he will be charged for the request. Bucket owners need not specify this parameter in their requests. Documentation on downloading objects from requester pays buckets can be found at http://docs.aws.amazon.com/AmazonS3/latest/dev/ObjectsinRequesterPaysBuckets.html<br/>LegalHold:  Container element for the Legal Hold configuration you want to apply to the specified object.:<br/>Key:  The key name for the object that you want to place a Legal Hold on.<br/>Bucket:  The bucket containing the object that you want to place a Legal Hold on.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> Indicates whether the specified object has a Legal Hold in place.<br/>
        :return: RequestCharged:  If present, indicates that the requester was successfully charged for the request.<br/>        """
        pass

    def put_object_lock_configuration(self, Bucket: str, ObjectLockConfiguration: ObjectLockConfiguration=None, RequestPayer: str=None, Token: str=None, ContentMD5: str=None) -> PutObjectLockConfigurationOutput:
        """Places an object lock configuration on the specified bucket. The rule specified in the object lock configuration will be applied by default to every new object placed in the specified bucket.

        :param self:
        :param ContentMD5:  The MD5 hash for the request body.<br/>Token:  A token to allow Amazon S3 object lock to be enabled for an existing bucket.<br/>RequestPayer:  Confirms that the requester knows that she or he will be charged for the request. Bucket owners need not specify this parameter in their requests. Documentation on downloading objects from requester pays buckets can be found at http://docs.aws.amazon.com/AmazonS3/latest/dev/ObjectsinRequesterPaysBuckets.html<br/>ObjectLockConfiguration:  The object lock configuration that you want to apply to the specified bucket.:<br/>Bucket:  The bucket whose object lock configuration you want to create or replace.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Rule</i> The object lock rule in place for the specified object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Rule</b>: The object lock rule in place for the specified object.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ObjectLockEnabled</i> Indicates whether this bucket has an object lock configuration enabled.<br/>The default retention period that you want to apply to new objects placed in the specified bucket.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DefaultRetention</b>: The default retention period that you want to apply to new objects placed in the specified bucket.
        :return: RequestCharged:  If present, indicates that the requester was successfully charged for the request.<br/>        """
        pass

    def put_object_retention(self, Key: str, Bucket: str, Retention: ObjectLockRetention=None, RequestPayer: str=None, VersionId: str=None, BypassGovernanceRetention: BypassGovernanceRetention=None, ContentMD5: str=None) -> PutObjectRetentionOutput:
        """Places an Object Retention configuration on an object.

        :param self:
        :param ContentMD5:  The MD5 hash for the request body.<br/>BypassGovernanceRetention:  Indicates whether this operation should bypass Governance-mode restrictions.j<br/>VersionId:  The version ID for the object that you want to apply this Object Retention configuration to.<br/>RequestPayer:  Confirms that the requester knows that she or he will be charged for the request. Bucket owners need not specify this parameter in their requests. Documentation on downloading objects from requester pays buckets can be found at http://docs.aws.amazon.com/AmazonS3/latest/dev/ObjectsinRequesterPaysBuckets.html<br/>Retention:  The container element for the Object Retention configuration.:<br/>Key:  The key name for the object that you want to apply this Object Retention configuration to.<br/>Bucket:  The bucket that contains the object you want to apply this Object Retention configuration to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RetainUntilDate</i> The date on which this object lock retention expires.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> Indicates the Retention mode for the specified object.<br/>
        :return: RequestCharged:  If present, indicates that the requester was successfully charged for the request.<br/>        """
        pass

    def put_object_tagging(self, Tagging: Tagging, Key: str, Bucket: str, VersionId: str=None, ContentMD5: str=None) -> PutObjectTaggingOutput:
        """Sets the supplied tag-set to an object that already exists in a bucket

        :param self:
        :param Tagging:  <p/>:<br/>ContentMD5:  <p/><br/>VersionId:  <p/><br/>Key:  <p/><br/>Bucket:  <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TagSet</i> <p/><br/>
        :return: VersionId:  <p/><br/>        """
        pass

    def put_public_access_block(self, PublicAccessBlockConfiguration: PublicAccessBlockConfiguration, Bucket: str, ContentMD5: str=None):
        """Creates or modifies the <code>PublicAccessBlock</code> configuration for an Amazon S3 bucket.

        :param self:
        :param PublicAccessBlockConfiguration:  The <code>PublicAccessBlock</code> configuration that you want to apply to this Amazon S3 bucket. You can enable the configuration options in any combination. For more information about when Amazon S3 considers a bucket or object public, see <a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/access-control-block-public-access.html#access-control-block-public-access-policy-status">The Meaning of "Public"</a> in the <i>Amazon Simple Storage Service Developer Guide</i>.:<br/>ContentMD5:  The MD5 hash of the <code>PutPublicAccessBlock</code> request body. <br/>Bucket:  The name of the Amazon S3 bucket whose <code>PublicAccessBlock</code> configuration you want to set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestrictPublicBuckets</i> Specifies whether Amazon S3 should restrict public bucket policies for this bucket. Setting this element to <code>TRUE</code> restricts access to this bucket to only AWS services and authorized users within this account if the bucket has a public policy. Enabling this setting doesn't affect previously stored bucket policies, except that public and cross-account access within any public bucket policy, including non-public delegation to specific accounts, is blocked.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BlockPublicPolicy</i> Specifies whether Amazon S3 should block public bucket policies for this bucket. Setting this element to <code>TRUE</code> causes Amazon S3 to reject calls to PUT Bucket policy if the specified bucket policy allows public access.  Enabling this setting doesn't affect existing bucket policies.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IgnorePublicAcls</i> Specifies whether Amazon S3 should ignore public ACLs for this bucket and objects in this bucket. Setting this element to <code>TRUE</code> causes Amazon S3 to ignore all public ACLs on this bucket and objects in this bucket. Enabling this setting doesn't affect the persistence of any existing ACLs and doesn't prevent new public ACLs from being set.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BlockPublicAcls</i> Specifies whether Amazon S3 should block public access control lists (ACLs) for this bucket and objects in this bucket. Setting this element to <code>TRUE</code> causes the following behavior: <ul> <li> PUT Bucket acl and PUT Object acl calls fail if the specified ACL is public. </li> <li> PUT Object calls fail if the request includes a public ACL. </li> </ul> Enabling this setting doesn't affect existing policies or ACLs.<br/>
        :return:         """
        pass

    def restore_object(self, Key: str, Bucket: str, VersionId: str=None, RestoreRequest: RestoreRequest=None, RequestPayer: str=None) -> RestoreObjectOutput:
        """Restores an archived copy of an object back into Amazon S3

        :param self:
        :param RequestPayer:  Confirms that the requester knows that she or he will be charged for the request. Bucket owners need not specify this parameter in their requests. Documentation on downloading objects from requester pays buckets can be found at http://docs.aws.amazon.com/AmazonS3/latest/dev/ObjectsinRequesterPaysBuckets.html<br/>RestoreRequest:  <p/>:<br/>VersionId:  <p/><br/>Key:  <p/><br/>Bucket:  <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputLocation</i> Describes the location where the restore job's output is stored.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>OutputLocation</b>: Describes the location where the restore job's output is stored.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SelectParameters</i> Describes the parameters for Select job types.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SelectParameters</b>: Describes the parameters for Select job types.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The optional description for the job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tier</i> Glacier retrieval tier at which the restore will be processed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> Type of restore request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlacierJobParameters</i> Glacier related parameters pertaining to this job. Do not use with restores that specify OutputLocation.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>GlacierJobParameters</b>: Glacier related parameters pertaining to this job. Do not use with restores that specify OutputLocation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Days</i> Lifetime of the active copy in days. Do not use with restores that specify OutputLocation.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Tier</i> Glacier retrieval tier at which the restore will be processed.<br/>Describes the serialization format of the object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InputSerialization</b>: Describes the serialization format of the object.Describes an S3 location that will receive the results of the restore request.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3</b>: Describes an S3 location that will receive the results of the restore request.
        :return: RestoreOutputPath:  Indicates the path in the provided S3 output location where Select results will be restored to.<br/>RequestCharged:  If present, indicates that the requester was successfully charged for the request.<br/>        """
        pass

    def select_object_content(self, OutputSerialization: OutputSerialization, InputSerialization: InputSerialization, ExpressionType: str, Expression: str, Key: str, Bucket: str, SSECustomerAlgorithm: str=None, SSECustomerKey: str=None, SSECustomerKeyMD5: str=None, RequestProgress: RequestProgress=None) -> SelectObjectContentOutput:
        """This operation filters the contents of an Amazon S3 object based on a simple Structured Query Language (SQL) statement. In the request, along with the SQL expression, you must also specify a data serialization format (JSON or CSV) of the object. Amazon S3 uses this to parse object data into records, and returns only records that match the specified SQL expression. You must also specify the data serialization format for the response.

        :param self:
        :param OutputSerialization:  Describes the format of the data that you want Amazon S3 to return in response.:<br/>InputSerialization:  Describes the format of the data in the object that is being queried.:<br/>RequestProgress:  Specifies if periodic request progress information should be enabled.:<br/>ExpressionType:  The type of the provided expression (for example., SQL).<br/>Expression:  The expression that is used to query the object.<br/>SSECustomerKeyMD5:  The SSE Customer Key MD5. For more information, see <a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/ServerSideEncryptionCustomerKeys.html"> Server-Side Encryption (Using Customer-Provided Encryption Keys</a>. <br/>SSECustomerKey:  The SSE Customer Key. For more information, see <a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/ServerSideEncryptionCustomerKeys.html"> Server-Side Encryption (Using Customer-Provided Encryption Keys</a>. <br/>SSECustomerAlgorithm:  The SSE Algorithm used to encrypt the object. For more information, see <a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/ServerSideEncryptionCustomerKeys.html"> Server-Side Encryption (Using Customer-Provided Encryption Keys</a>. <br/>Key:  The object key.<br/>Bucket:  The S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Specifies whether periodic QueryProgress frames should be sent. Valid values: TRUE, FALSE. Default value: FALSE.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JSON</i> Specifies JSON as object's input serialization format.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>JSON</b>: Specifies JSON as object's input serialization format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompressionType</i> Specifies object's compression format. Valid values: NONE, GZIP, BZIP2. Default Value: NONE.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CSV</i> Describes the serialization of a CSV-encoded object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CSV</b>: Describes the serialization of a CSV-encoded object.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AllowQuotedRecordDelimiter</i> Specifies that CSV field values may contain quoted record delimiters and such records should be allowed. Default value is FALSE. Setting this value to TRUE may lower performance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>QuoteCharacter</i> Value used for escaping where the field delimiter is part of the value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FieldDelimiter</i> The value used to separate individual fields in a record.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordDelimiter</i> The value used to separate individual records.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>QuoteEscapeCharacter</i> The single character used for escaping the quote character inside an already escaped value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Comments</i> The single character used to indicate a row should be ignored when present at the start of a row.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FileHeaderInfo</i> Describes the first line of input. Valid values: None, Ignore, Use.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of JSON. Valid values: Document, Lines.<br/>Specifies Parquet as object's input serialization format.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Parquet</b>: Specifies Parquet as object's input serialization format.&nbsp;&nbsp;&nbsp;&nbsp; <i>JSON</i> Specifies JSON as request's output serialization format.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>JSON</b>: Specifies JSON as request's output serialization format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CSV</i> Describes the serialization of CSV-encoded Select results.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CSV</b>: Describes the serialization of CSV-encoded Select results.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>QuoteCharacter</i> The value used for escaping where the field delimiter is part of the value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FieldDelimiter</i> The value used to separate individual fields in a record.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordDelimiter</i> The value used to separate individual records.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>QuoteEscapeCharacter</i> Th single character used for escaping the quote character inside an already escaped value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>QuoteFields</i> Indicates whether or not all output fields should be quoted.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordDelimiter</i> The value used to separate individual records in the output.<br/>
        :return: Payload:  <p/>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Progress</i> The Progress Event.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Progress</b>: The Progress Event.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Stats</i> The Stats Event.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Stats</b>: The Stats Event.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Records</i> The Records Event.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Records</b>: The Records Event.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Payload</i> The byte array of partial, one or more result records.<br/>The Stats event details.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Details</b>: The Stats event details.The Progress event details.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Details</b>: The Progress event details.The Continuation Event.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Cont</b>: The Continuation Event.        """
        pass

    def upload_part(self, UploadId: str, PartNumber: int, Key: str, Bucket: str, Body: Body=None, ContentLength: ContentLength=None, ContentMD5: str=None, SSECustomerAlgorithm: str=None, SSECustomerKey: str=None, SSECustomerKeyMD5: str=None, RequestPayer: str=None) -> UploadPartOutput:
        """Uploads a part in a multipart upload.  <b>Note:</b> After you initiate multipart upload and upload one or more parts, you must either complete or abort multipart upload in order to stop getting charged for storage of the uploaded parts. Only after you either complete or abort multipart upload, Amazon S3 frees up the parts storage and stops charging you for the parts storage.

        :param self:
        :param RequestPayer:  Confirms that the requester knows that she or he will be charged for the request. Bucket owners need not specify this parameter in their requests. Documentation on downloading objects from requester pays buckets can be found at http://docs.aws.amazon.com/AmazonS3/latest/dev/ObjectsinRequesterPaysBuckets.html<br/>SSECustomerKeyMD5:  Specifies the 128-bit MD5 digest of the encryption key according to RFC 1321. Amazon S3 uses this header for a message integrity check to ensure the encryption key was transmitted without error.<br/>SSECustomerKey:  Specifies the customer-provided encryption key for Amazon S3 to use in encrypting data. This value is used to store the object and then it is discarded; Amazon does not store the encryption key. The key must be appropriate for use with the algorithm specified in the x-amz-server-side​-encryption​-customer-algorithm header. This must be the same encryption key specified in the initiate multipart upload request.<br/>SSECustomerAlgorithm:  Specifies the algorithm to use to when encrypting the object (e.g., AES256).<br/>UploadId:  Upload ID identifying the multipart upload whose part is being uploaded.<br/>PartNumber:  Part number of part being uploaded. This is a positive integer between 1 and 10,000.<br/>Key:  Object key for which the multipart upload was initiated.<br/>ContentMD5:  The base64-encoded 128-bit MD5 digest of the part data. This parameter is auto-populated when using the command from the CLI. This parameted is required if object lock parameters are specified.<br/>ContentLength:  Size of the body in bytes. This parameter is useful when the size of the body cannot be determined automatically.<br/>Bucket:  Name of the bucket to which the multipart upload was initiated.<br/>Body:  Object data.<br/>
        :return: RequestCharged:  If present, indicates that the requester was successfully charged for the request.<br/>SSEKMSKeyId:  If present, specifies the ID of the AWS Key Management Service (KMS) master encryption key that was used for the object.<br/>SSECustomerKeyMD5:  If server-side encryption with a customer-provided encryption key was requested, the response will include this header to provide round trip message integrity verification of the customer-provided encryption key.<br/>SSECustomerAlgorithm:  If server-side encryption with a customer-provided encryption key was requested, the response will include this header confirming the encryption algorithm used.<br/>ETag:  Entity tag for the uploaded object.<br/>ServerSideEncryption:  The Server-side encryption algorithm used when storing this object in S3 (e.g., AES256, aws:kms).<br/>        """
        pass

    def upload_part_copy(self, UploadId: str, PartNumber: int, Key: str, CopySource: str, Bucket: str, CopySourceIfMatch: str=None, CopySourceIfModifiedSince: CopySourceIfModifiedSince=None, CopySourceIfNoneMatch: str=None, CopySourceIfUnmodifiedSince: CopySourceIfUnmodifiedSince=None, CopySourceRange: str=None, SSECustomerAlgorithm: str=None, SSECustomerKey: str=None, SSECustomerKeyMD5: str=None, CopySourceSSECustomerAlgorithm: str=None, CopySourceSSECustomerKey: str=None, CopySourceSSECustomerKeyMD5: str=None, RequestPayer: str=None) -> UploadPartCopyOutput:
        """Uploads a part by copying data from an existing object as data source.

        :param self:
        :param RequestPayer:  Confirms that the requester knows that she or he will be charged for the request. Bucket owners need not specify this parameter in their requests. Documentation on downloading objects from requester pays buckets can be found at http://docs.aws.amazon.com/AmazonS3/latest/dev/ObjectsinRequesterPaysBuckets.html<br/>CopySourceSSECustomerKeyMD5:  Specifies the 128-bit MD5 digest of the encryption key according to RFC 1321. Amazon S3 uses this header for a message integrity check to ensure the encryption key was transmitted without error.<br/>CopySourceSSECustomerKey:  Specifies the customer-provided encryption key for Amazon S3 to use to decrypt the source object. The encryption key provided in this header must be one that was used when the source object was created.<br/>CopySourceSSECustomerAlgorithm:  Specifies the algorithm to use when decrypting the source object (e.g., AES256).<br/>SSECustomerKeyMD5:  Specifies the 128-bit MD5 digest of the encryption key according to RFC 1321. Amazon S3 uses this header for a message integrity check to ensure the encryption key was transmitted without error.<br/>SSECustomerKey:  Specifies the customer-provided encryption key for Amazon S3 to use in encrypting data. This value is used to store the object and then it is discarded; Amazon does not store the encryption key. The key must be appropriate for use with the algorithm specified in the x-amz-server-side​-encryption​-customer-algorithm header. This must be the same encryption key specified in the initiate multipart upload request.<br/>SSECustomerAlgorithm:  Specifies the algorithm to use to when encrypting the object (e.g., AES256).<br/>UploadId:  Upload ID identifying the multipart upload whose part is being copied.<br/>PartNumber:  Part number of part being copied. This is a positive integer between 1 and 10,000.<br/>Key:  <p/><br/>CopySourceRange:  The range of bytes to copy from the source object. The range value must use the form bytes=first-last, where the first and last are the zero-based byte offsets to copy. For example, bytes=0-9 indicates that you want to copy the first ten bytes of the source. You can copy a range only if the source object is greater than 5 MB.<br/>CopySourceIfUnmodifiedSince:  Copies the object if it hasn't been modified since the specified time.<br/>CopySourceIfNoneMatch:  Copies the object if its entity tag (ETag) is different than the specified ETag.<br/>CopySourceIfModifiedSince:  Copies the object if it has been modified since the specified time.<br/>CopySourceIfMatch:  Copies the object if its entity tag (ETag) matches the specified tag.<br/>CopySource:  The name of the source bucket and key name of the source object, separated by a slash (/). Must be URL-encoded.<br/>Bucket:  <p/><br/>
        :return: RequestCharged:  If present, indicates that the requester was successfully charged for the request.<br/>SSEKMSKeyId:  If present, specifies the ID of the AWS Key Management Service (KMS) master encryption key that was used for the object.<br/>SSECustomerKeyMD5:  If server-side encryption with a customer-provided encryption key was requested, the response will include this header to provide round trip message integrity verification of the customer-provided encryption key.<br/>SSECustomerAlgorithm:  If server-side encryption with a customer-provided encryption key was requested, the response will include this header confirming the encryption algorithm used.<br/>ServerSideEncryption:  The Server-side encryption algorithm used when storing this object in S3 (e.g., AES256, aws:kms).<br/>CopyPartResult:  <p/>:<br/>CopySourceVersionId:  The version of the source object that was copied, if you have enabled versioning on the source bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModified</i> Date and time at which the object was uploaded.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ETag</i> Entity tag of the object.<br/>        """
        pass

    class S3Resource:
        
        class Bucket:
            """Creates a Bucket resource.::

  bucket = s3.Bucket('name')

:type name: string
:param name: The Bucket's name identifier. This **must** be set.

:rtype: :py:class:`S3.Bucket`
:returns: A Bucket resource"""

            def __init__(self, name):
                pass
            
            class CreationDate(object):
                pass
        
            name: str=None
            creation_date: CreationDate=None
            
            def wait_until_exists(self):
                pass

            def wait_until_not_exists(self):
                pass

            def create(self, **kwargs):
                # type: (object) -> dict
                return {}

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def delete_objects(self, **kwargs):
                # type: (object) -> dict
                return {}

            def put_object(self, **kwargs):
                # type: (object) -> Object
                return Object()


        
        class BucketAcl:
            """Creates a BucketAcl resource.::

  bucket_acl = s3.BucketAcl('bucket_name')

:type bucket_name: string
:param bucket_name: The BucketAcl's bucket_name identifier. This **must** be set.

:rtype: :py:class:`S3.BucketAcl`
:returns: A BucketAcl resource"""

            def __init__(self, bucket_name):
                pass
            
            class Owner(Mapping):
                pass

            class Grant(Mapping):
                pass
        
            owner: Owner=None
            grants: List[Grant]=None
            
            def put(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class BucketCors:
            """Creates a BucketCors resource.::

  bucket_cors = s3.BucketCors('bucket_name')

:type bucket_name: string
:param bucket_name: The BucketCors's bucket_name identifier. This **must** be set.

:rtype: :py:class:`S3.BucketCors`
:returns: A BucketCors resource"""

            def __init__(self, bucket_name):
                pass
            
            class CORSRule(Mapping):
                pass
        
            cors_rules: List[CORSRule]=None
            
            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def put(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class BucketLifecycle:
            """Creates a BucketLifecycle resource.::

  bucket_lifecycle = s3.BucketLifecycle('bucket_name')

:type bucket_name: string
:param bucket_name: The BucketLifecycle's bucket_name identifier. This **must** be set.

:rtype: :py:class:`S3.BucketLifecycle`
:returns: A BucketLifecycle resource"""

            def __init__(self, bucket_name):
                pass
            
            class Rule(Mapping):
                pass
        
            rules: List[Rule]=None
            
            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def put(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class BucketLifecycleConfiguration:
            """Creates a BucketLifecycleConfiguration resource.::

  bucket_lifecycle_configuration = s3.BucketLifecycleConfiguration('bucket_name')

:type bucket_name: string
:param bucket_name: The BucketLifecycleConfiguration's bucket_name identifier. This **must** be set.

:rtype: :py:class:`S3.BucketLifecycleConfiguration`
:returns: A BucketLifecycleConfiguration resource"""

            def __init__(self, bucket_name):
                pass
            
            class LifecycleRule(Mapping):
                pass
        
            rules: List[LifecycleRule]=None
            
            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def put(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class BucketLogging:
            """Creates a BucketLogging resource.::

  bucket_logging = s3.BucketLogging('bucket_name')

:type bucket_name: string
:param bucket_name: The BucketLogging's bucket_name identifier. This **must** be set.

:rtype: :py:class:`S3.BucketLogging`
:returns: A BucketLogging resource"""

            def __init__(self, bucket_name):
                pass
            
            class LoggingEnabled(Mapping):
                pass
        
            logging_enabled: LoggingEnabled=None
            
            def put(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class BucketNotification:
            """Creates a BucketNotification resource.::

  bucket_notification = s3.BucketNotification('bucket_name')

:type bucket_name: string
:param bucket_name: The BucketNotification's bucket_name identifier. This **must** be set.

:rtype: :py:class:`S3.BucketNotification`
:returns: A BucketNotification resource"""

            def __init__(self, bucket_name):
                pass
            
            class QueueConfiguration(Mapping):
                pass

            class TopicConfiguration(Mapping):
                pass

            class LambdaFunctionConfiguration(Mapping):
                pass
        
            topic_configurations: List[TopicConfiguration]=None
            queue_configurations: List[QueueConfiguration]=None
            lambda_function_configurations: List[LambdaFunctionConfiguration]=None
            
            def put(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class BucketPolicy:
            """Creates a BucketPolicy resource.::

  bucket_policy = s3.BucketPolicy('bucket_name')

:type bucket_name: string
:param bucket_name: The BucketPolicy's bucket_name identifier. This **must** be set.

:rtype: :py:class:`S3.BucketPolicy`
:returns: A BucketPolicy resource"""

            def __init__(self, bucket_name):
                pass
            
        
            policy: str=None
            
            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def put(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class BucketRequestPayment:
            """Creates a BucketRequestPayment resource.::

  bucket_request_payment = s3.BucketRequestPayment('bucket_name')

:type bucket_name: string
:param bucket_name: The BucketRequestPayment's bucket_name identifier. This **must** be set.

:rtype: :py:class:`S3.BucketRequestPayment`
:returns: A BucketRequestPayment resource"""

            def __init__(self, bucket_name):
                pass
            
        
            payer: str=None
            
            def put(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class BucketTagging:
            """Creates a BucketTagging resource.::

  bucket_tagging = s3.BucketTagging('bucket_name')

:type bucket_name: string
:param bucket_name: The BucketTagging's bucket_name identifier. This **must** be set.

:rtype: :py:class:`S3.BucketTagging`
:returns: A BucketTagging resource"""

            def __init__(self, bucket_name):
                pass
            
            class Tag(Mapping):
                pass
        
            tag_set: List[Tag]=None
            
            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def put(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class BucketVersioning:
            """Creates a BucketVersioning resource.::

  bucket_versioning = s3.BucketVersioning('bucket_name')

:type bucket_name: string
:param bucket_name: The BucketVersioning's bucket_name identifier. This **must** be set.

:rtype: :py:class:`S3.BucketVersioning`
:returns: A BucketVersioning resource"""

            def __init__(self, bucket_name):
                pass
            
        
            status: str=None
            mfa_delete: str=None
            
            def enable(self, **kwargs):
                # type: (object) -> dict
                return {}

            def put(self, **kwargs):
                # type: (object) -> dict
                return {}

            def suspend(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class BucketWebsite:
            """Creates a BucketWebsite resource.::

  bucket_website = s3.BucketWebsite('bucket_name')

:type bucket_name: string
:param bucket_name: The BucketWebsite's bucket_name identifier. This **must** be set.

:rtype: :py:class:`S3.BucketWebsite`
:returns: A BucketWebsite resource"""

            def __init__(self, bucket_name):
                pass
            
            class RedirectAllRequestsTo(Mapping):
                pass

            class IndexDocument(Mapping):
                pass

            class RoutingRule(Mapping):
                pass

            class ErrorDocument(Mapping):
                pass
        
            redirect_all_requests_to: RedirectAllRequestsTo=None
            index_document: IndexDocument=None
            error_document: ErrorDocument=None
            routing_rules: List[RoutingRule]=None
            
            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def put(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class MultipartUpload:
            """Creates a MultipartUpload resource.::

  multipart_upload = s3.MultipartUpload('bucket_name','object_key','id')

:type bucket_name: string
:param bucket_name: The MultipartUpload's bucket_name identifier. This **must** be set.
:type object_key: string
:param object_key: The MultipartUpload's object_key identifier. This **must** be set.
:type id: string
:param id: The MultipartUpload's id identifier. This **must** be set.

:rtype: :py:class:`S3.MultipartUpload`
:returns: A MultipartUpload resource"""

            def __init__(self, bucket_name, object_key, id):
                pass
            
            class Initiator(Mapping):
                pass

            class Owner(Mapping):
                pass

            class Initiated(object):
                pass
        
            upload_id: str=None
            key: str=None
            initiated: Initiated=None
            storage_class: str=None
            owner: Owner=None
            initiator: Initiator=None
            
            def abort(self, **kwargs):
                # type: (object) -> dict
                return {}

            def complete(self, **kwargs):
                # type: (object) -> Object
                return Object()


        
        class MultipartUploadPart:
            """Creates a MultipartUploadPart resource.::

  multipart_upload_part = s3.MultipartUploadPart('bucket_name','object_key','multipart_upload_id','part_number')

:type bucket_name: string
:param bucket_name: The MultipartUploadPart's bucket_name identifier. This **must** be set.
:type object_key: string
:param object_key: The MultipartUploadPart's object_key identifier. This **must** be set.
:type multipart_upload_id: string
:param multipart_upload_id: The MultipartUploadPart's multipart_upload_id identifier. This **must** be set.
:type part_number: string
:param part_number: The MultipartUploadPart's part_number identifier. This **must** be set.

:rtype: :py:class:`S3.MultipartUploadPart`
:returns: A MultipartUploadPart resource"""

            def __init__(self, bucket_name, object_key, multipart_upload_id, part_number):
                pass
            
            class LastModified(object):
                pass
        
            part_number: int=None
            last_modified: LastModified=None
            e_tag: str=None
            size: int=None
            
            def copy_from(self, **kwargs):
                # type: (object) -> dict
                return {}

            def upload(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class Object:
            """Creates a Object resource.::

  object = s3.Object('bucket_name','key')

:type bucket_name: string
:param bucket_name: The Object's bucket_name identifier. This **must** be set.
:type key: string
:param key: The Object's key identifier. This **must** be set.

:rtype: :py:class:`S3.Object`
:returns: A Object resource"""

            def __init__(self, bucket_name, key):
                pass
            
            class DeleteMarker(object):
                pass

            class Expires(object):
                pass

            class Metadata(object):
                pass

            class ContentLength(object):
                pass

            class ObjectLockRetainUntilDate(object):
                pass

            class LastModified(object):
                pass
        
            delete_marker: DeleteMarker=None
            accept_ranges: str=None
            expiration: str=None
            restore: str=None
            last_modified: LastModified=None
            content_length: ContentLength=None
            e_tag: str=None
            missing_meta: int=None
            version_id: str=None
            cache_control: str=None
            content_disposition: str=None
            content_encoding: str=None
            content_language: str=None
            content_type: str=None
            expires: Expires=None
            website_redirect_location: str=None
            server_side_encryption: str=None
            metadata: Metadata=None
            sse_customer_algorithm: str=None
            sse_customer_key_md5: str=None
            ssekms_key_id: str=None
            storage_class: str=None
            request_charged: str=None
            replication_status: str=None
            parts_count: int=None
            object_lock_mode: str=None
            object_lock_retain_until_date: ObjectLockRetainUntilDate=None
            object_lock_legal_hold_status: str=None
            
            def wait_until_exists(self):
                pass

            def wait_until_not_exists(self):
                pass

            def copy_from(self, **kwargs):
                # type: (object) -> dict
                return {}

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def get(self, **kwargs):
                # type: (object) -> dict
                return {}

            def initiate_multipart_upload(self, **kwargs):
                # type: (object) -> MultipartUpload
                return MultipartUpload()

            def put(self, **kwargs):
                # type: (object) -> dict
                return {}

            def restore_object(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class ObjectAcl:
            """Creates a ObjectAcl resource.::

  object_acl = s3.ObjectAcl('bucket_name','object_key')

:type bucket_name: string
:param bucket_name: The ObjectAcl's bucket_name identifier. This **must** be set.
:type object_key: string
:param object_key: The ObjectAcl's object_key identifier. This **must** be set.

:rtype: :py:class:`S3.ObjectAcl`
:returns: A ObjectAcl resource"""

            def __init__(self, bucket_name, object_key):
                pass
            
            class Owner(Mapping):
                pass

            class Grant(Mapping):
                pass
        
            owner: Owner=None
            grants: List[Grant]=None
            request_charged: str=None
            
            def put(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class ObjectSummary:
            """Creates a ObjectSummary resource.::

  object_summary = s3.ObjectSummary('bucket_name','key')

:type bucket_name: string
:param bucket_name: The ObjectSummary's bucket_name identifier. This **must** be set.
:type key: string
:param key: The ObjectSummary's key identifier. This **must** be set.

:rtype: :py:class:`S3.ObjectSummary`
:returns: A ObjectSummary resource"""

            def __init__(self, bucket_name, key):
                pass
            
            class Owner(Mapping):
                pass

            class LastModified(object):
                pass
        
            key: str=None
            last_modified: LastModified=None
            e_tag: str=None
            size: int=None
            storage_class: str=None
            owner: Owner=None
            
            def wait_until_exists(self):
                pass

            def wait_until_not_exists(self):
                pass

            def copy_from(self, **kwargs):
                # type: (object) -> dict
                return {}

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def get(self, **kwargs):
                # type: (object) -> dict
                return {}

            def initiate_multipart_upload(self, **kwargs):
                # type: (object) -> MultipartUpload
                return MultipartUpload()

            def put(self, **kwargs):
                # type: (object) -> dict
                return {}

            def restore_object(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class ObjectVersion:
            """Creates a ObjectVersion resource.::

  object_version = s3.ObjectVersion('bucket_name','object_key','id')

:type bucket_name: string
:param bucket_name: The ObjectVersion's bucket_name identifier. This **must** be set.
:type object_key: string
:param object_key: The ObjectVersion's object_key identifier. This **must** be set.
:type id: string
:param id: The ObjectVersion's id identifier. This **must** be set.

:rtype: :py:class:`S3.ObjectVersion`
:returns: A ObjectVersion resource"""

            def __init__(self, bucket_name, object_key, id):
                pass
            
            class LastModified(object):
                pass

            class Owner(Mapping):
                pass

            class IsLatest(object):
                pass
        
            e_tag: str=None
            size: int=None
            storage_class: str=None
            key: str=None
            version_id: str=None
            is_latest: IsLatest=None
            last_modified: LastModified=None
            owner: Owner=None
            
            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def get(self, **kwargs):
                # type: (object) -> dict
                return {}

            def head(self, **kwargs):
                # type: (object) -> dict
                return {}


        def create_bucket(self, **kwargs):
            # type: (object) -> Bucket
            pass

        def get_available_subresources(self) -> List[str]:
            pass

        class BucketResourceCollection(List[Bucket], ResourceCollection):
            pass

        buckets: BucketResourceCollection = None




class S3Control(BaseClient):
    """ AWS S3 Control provides access to Amazon S3 control plane operations. """




    class CreateJobRequest(Mapping):
        pass
    
    class CreateJobResult(Mapping):
        pass
    
    class ConfirmationRequired(object):
        pass
    
    class JobOperation(Mapping):
        pass
    
    class JobReport(Mapping):
        pass
    
    class JobManifest(Mapping):
        pass
    
    class DeletePublicAccessBlockRequest(Mapping):
        pass
    
    class DescribeJobRequest(Mapping):
        pass
    
    class DescribeJobResult(Mapping):
        pass
    
    class GetPublicAccessBlockRequest(Mapping):
        pass
    
    class GetPublicAccessBlockOutput(Mapping):
        pass
    
    class ListJobsRequest(Mapping):
        pass
    
    class ListJobsResult(Mapping):
        pass
    
    class JobStatus(object):
        pass
    
    class PutPublicAccessBlockRequest(Mapping):
        pass
    
    class PublicAccessBlockConfiguration(Mapping):
        pass
    
    class UpdateJobPriorityRequest(Mapping):
        pass
    
    class UpdateJobPriorityResult(Mapping):
        pass
    
    class UpdateJobStatusRequest(Mapping):
        pass
    
    class UpdateJobStatusResult(Mapping):
        pass
    
    def create_job(self, RoleArn: str, Priority: int, Manifest: JobManifest, ClientRequestToken: str, Report: JobReport, Operation: JobOperation, AccountId: str, ConfirmationRequired: ConfirmationRequired=None, Description: str=None) -> CreateJobResult:
        """Creates an Amazon S3 batch operations job.

        :param self:
        :param RoleArn:  The Amazon Resource Name (ARN) for the Identity and Access Management (IAM) Role that batch operations will use to execute this job's operation on each object in the manifest.<br/>Priority:  The numerical priority for this job. Higher numbers indicate higher priority.<br/>Description:  A description for this job. You can use any string within the permitted length. Descriptions don't need to be unique and can be used for multiple jobs.<br/>Manifest:  Configuration parameters for the manifest.:<br/>ClientRequestToken:  An idempotency token to ensure that you don't accidentally submit the same request twice. You can use any string up to the maximum length.<br/>Report:  Configuration parameters for the optional job-completion report.:<br/>Operation:  The operation that you want this job to perform on each object listed in the manifest. For more information about the available operations, see <a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/batch-ops-operations.html">Available Operations</a> in the <i>Amazon Simple Storage Service Developer Guide</i>.:<br/>ConfirmationRequired:  Indicates whether confirmation is required before Amazon S3 runs the job. Confirmation is only required for jobs created through the Amazon S3 console.<br/>AccountId:  <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3InitiateRestoreObject</i> Directs the specified job to execute an Initiate Glacier Restore call on each object in the manifest.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3InitiateRestoreObject</b>: Directs the specified job to execute an Initiate Glacier Restore call on each object in the manifest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3PutObjectTagging</i> Directs the specified job to execute a PUT Object tagging call on each object in the manifest.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3PutObjectTagging</b>: Directs the specified job to execute a PUT Object tagging call on each object in the manifest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3PutObjectAcl</i> Directs the specified job to execute a PUT Object acl call on each object in the manifest.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3PutObjectAcl</b>: Directs the specified job to execute a PUT Object acl call on each object in the manifest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3PutObjectCopy</i> Directs the specified job to execute a PUT Copy object call on each object in the manifest.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3PutObjectCopy</b>: Directs the specified job to execute a PUT Copy object call on each object in the manifest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LambdaInvoke</i> Directs the specified job to invoke an AWS Lambda function on each object in the manifest.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LambdaInvoke</b>: Directs the specified job to invoke an AWS Lambda function on each object in the manifest.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FunctionArn</i> The Amazon Resource Name (ARN) for the AWS Lambda function that the specified job will invoke for each object in the manifest.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ModifiedSinceConstraint</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MetadataDirective</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AccessControlGrants</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CannedAccessControlList</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetResource</i> <p/><br/><p/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NewObjectMetadata</b>: <p/><p/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AccessControlPolicy</b>: <p/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TagSet</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>GlacierJobTier</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpirationInDays</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReportScope</i> Indicates whether the job-completion report will include details of all tasks or only failed tasks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> An optional prefix to describe where in the specified bucket the job-completion report will be stored. Amazon S3 will store the job-completion report at &lt;prefix&gt;/job-&lt;job-id&gt;/report.json.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Indicates whether the specified job will generate a job-completion report.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> The format of the specified job-completion report.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> The bucket where specified job-completion report will be stored.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Location</i> Contains the information required to locate the specified job's manifest.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Location</b>: Contains the information required to locate the specified job's manifest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Spec</i> Describes the format of the specified job's manifest. If the manifest is in CSV format, also describes the columns contained within the manifest.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Spec</b>: Describes the format of the specified job's manifest. If the manifest is in CSV format, also describes the columns contained within the manifest.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Fields</i> If the specified manifest object is in the <code>S3BatchOperations_CSV_20180820</code> format, this element describes which columns contain the required data.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> Indicates which of the available formats the specified manifest uses.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ETag</i> The ETag for the specified manifest object.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ObjectVersionId</i> The optional version ID to identify a specific version of the manifest object.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ObjectArn</i> The Amazon Resource Name (ARN) for a manifest object.<br/>
        :return: JobId:  The ID for this job. Amazon S3 generates this ID automatically and returns it after a successful <code>Create Job</code> request.<br/>        """
        pass

    def delete_public_access_block(self, AccountId: str):
        """Deletes the block public access configuration for the specified account.

        :param self:
        :param AccountId:  The account ID for the AWS account whose block public access configuration you want to delete.<br/>
        :return:         """
        pass

    def describe_job(self, JobId: str, AccountId: str) -> DescribeJobResult:
        """Retrieves the configuration parameters and status for a batch operations job.

        :param self:
        :param JobId:  The ID for the job whose information you want to retrieve.<br/>AccountId:  <p/><br/>
        :return: Job:  Contains the configuration parameters and status for the job specified in the <code>Describe Job</code> request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SuspendedCause</i> The reason why the specified job was suspended. A job is only suspended if you create it through the Amazon S3 console. When you create the job, it enters the <code>Suspended</code> state to await confirmation before running. After you confirm the job, it automatically exits the <code>Suspended</code> state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SuspendedDate</i> The timestamp when this job was suspended, if it has been suspended.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The Amazon Resource Name (ARN) for the Identity and Access Management (IAM) Role assigned to execute the tasks for this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TerminationDate</i> A timestamp indicating when this job terminated. A job's termination date is the date and time when it succeeded, failed, or was canceled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> A timestamp indicating when this job was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Report</i> Contains the configuration information for the job-completion report if you requested one in the <code>Create Job</code> request.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Report</b>: Contains the configuration information for the job-completion report if you requested one in the <code>Create Job</code> request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureReasons</i> If the specified job failed, this field contains information describing the failure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusUpdateReason</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProgressSummary</i> Describes the total number of tasks that the specified job has executed, the number of tasks that succeeded, and the number of tasks that failed.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ProgressSummary</b>: Describes the total number of tasks that the specified job has executed, the number of tasks that succeeded, and the number of tasks that failed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Priority</i> The priority of the specified job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Operation</i> The operation that the specified job is configured to execute on the objects listed in the manifest.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Operation</b>: The operation that the specified job is configured to execute on the objects listed in the manifest.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Manifest</i> The configuration information for the specified job's manifest object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Manifest</b>: The configuration information for the specified job's manifest object.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current status of the specified job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobArn</i> The Amazon Resource Name (ARN) for this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description for this job, if one was provided in this job's <code>Create Job</code> request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConfirmationRequired</i> Indicates whether confirmation is required before Amazon S3 begins running the specified job. Confirmation is required only for jobs created through the Amazon S3 console.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobId</i> The ID for the specified job.<br/>Describes the format of the specified job's manifest. If the manifest is in CSV format, also describes the columns contained within the manifest.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Spec</b>: Describes the format of the specified job's manifest. If the manifest is in CSV format, also describes the columns contained within the manifest.Directs the specified job to invoke an AWS Lambda function on each object in the manifest.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LambdaInvoke</b>: Directs the specified job to invoke an AWS Lambda function on each object in the manifest.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfTasksFailed</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfTasksSucceeded</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalNumberOfTasks</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReportScope</i> Indicates whether the job-completion report will include details of all tasks or only failed tasks.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> An optional prefix to describe where in the specified bucket the job-completion report will be stored. Amazon S3 will store the job-completion report at &lt;prefix&gt;/job-&lt;job-id&gt;/report.json.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Indicates whether the specified job will generate a job-completion report.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> The format of the specified job-completion report.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> The bucket where specified job-completion report will be stored.<br/>        """
        pass

    def get_public_access_block(self, AccountId: str) -> GetPublicAccessBlockOutput:
        """<p/>

        :param self:
        :param AccountId:  <p/><br/>
        :return: PublicAccessBlockConfiguration:  <p/>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestrictPublicBuckets</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BlockPublicPolicy</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IgnorePublicAcls</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BlockPublicAcls</i> <p/><br/>        """
        pass

    def list_jobs(self, AccountId: str, JobStatuses: List[JobStatus]=None, NextToken: str=None, MaxResults: int=None) -> ListJobsResult:
        """Lists current jobs and jobs that have ended within the last 30 days for the AWS account making the request.

        :param self:
        :param MaxResults:  The maximum number of jobs that Amazon S3 will include in the <code>List Jobs</code> response. If there are more jobs than this number, the response will include a pagination token in the <code>NextToken</code> field to enable you to retrieve the next page of results.<br/>NextToken:  A pagination token to request the next page of results. Use the token that Amazon S3 returned in the <code>NextToken</code> element of the <code>ListJobsResult</code> from the previous <code>List Jobs</code> request.<br/>JobStatuses:  The <code>List Jobs</code> request returns jobs that match the statuses listed in this element.<br/>AccountId:  <p/><br/>
        :return: Jobs:  The list of current jobs and jobs that have ended within the last 30 days.<br/>NextToken:  If the <code>List Jobs</code> request produced more than the maximum number of results, you can pass this value into a subsequent <code>List Jobs</code> request in order to retrieve the next page of results.<br/>        """
        pass

    def put_public_access_block(self, AccountId: str, PublicAccessBlockConfiguration: PublicAccessBlockConfiguration):
        """<p/>

        :param self:
        :param AccountId:  <p/><br/>PublicAccessBlockConfiguration:  <p/>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RestrictPublicBuckets</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BlockPublicPolicy</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IgnorePublicAcls</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BlockPublicAcls</i> <p/><br/>
        :return:         """
        pass

    def update_job_priority(self, Priority: int, JobId: str, AccountId: str) -> UpdateJobPriorityResult:
        """Updates an existing job's priority.

        :param self:
        :param Priority:  The priority you want to assign to this job.<br/>JobId:  The ID for the job whose priority you want to update.<br/>AccountId:  <p/><br/>
        :return: Priority:  The new priority assigned to the specified job.<br/>JobId:  The ID for the job whose priority Amazon S3 updated.<br/>        """
        pass

    def update_job_status(self, RequestedJobStatus: str, JobId: str, AccountId: str, StatusUpdateReason: str=None) -> UpdateJobStatusResult:
        """Updates the status for the specified job. Use this operation to confirm that you want to run a job or to cancel an existing job.

        :param self:
        :param StatusUpdateReason:  A description of the reason why you want to change the specified job's status. This field can be any string up to the maximum length.<br/>RequestedJobStatus:  The status that you want to move the specified job to.<br/>JobId:  The ID of the job whose status you want to update.<br/>AccountId:  <p/><br/>
        :return: StatusUpdateReason:  The reason that the specified job's status was updated.<br/>Status:  The current status for the specified job.<br/>JobId:  The ID for the job whose status was updated.<br/>        """
        pass



class SageMaker(BaseClient):
    """Provides APIs for creating and managing Amazon SageMaker resources."""

    class EndpointDeletedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_endpoint` for valid parameters"""
            pass
    class EndpointInServiceWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_endpoint` for valid parameters"""
            pass
    class NotebookInstanceDeletedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_notebook_instance` for valid parameters"""
            pass
    class NotebookInstanceInServiceWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_notebook_instance` for valid parameters"""
            pass
    class NotebookInstanceStoppedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_notebook_instance` for valid parameters"""
            pass
    class TrainingJobCompletedOrStoppedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_training_job` for valid parameters"""
            pass
    class TransformJobCompletedOrStoppedWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_transform_job` for valid parameters"""
            pass


    class ListTrainingJobsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_training_jobs` for valid parameters"""
            pass
    class ListEndpointsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_endpoints` for valid parameters"""
            pass
    class ListEndpointConfigsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_endpoint_configs` for valid parameters"""
            pass
    class ListNotebookInstancesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_notebook_instances` for valid parameters"""
            pass
    class ListTagsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_tags` for valid parameters"""
            pass
    class ListModelsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_models` for valid parameters"""
            pass
    class ListAlgorithmsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_algorithms` for valid parameters"""
            pass
    class ListCodeRepositoriesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_code_repositories` for valid parameters"""
            pass
    class ListCompilationJobsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_compilation_jobs` for valid parameters"""
            pass
    class ListHyperParameterTuningJobsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_hyper_parameter_tuning_jobs` for valid parameters"""
            pass
    class ListLabelingJobsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_labeling_jobs` for valid parameters"""
            pass
    class ListLabelingJobsForWorkteamPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_labeling_jobs_for_workteam` for valid parameters"""
            pass
    class ListModelPackagesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_model_packages` for valid parameters"""
            pass
    class ListNotebookInstanceLifecycleConfigsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_notebook_instance_lifecycle_configs` for valid parameters"""
            pass
    class ListSubscribedWorkteamsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_subscribed_workteams` for valid parameters"""
            pass
    class ListTrainingJobsForHyperParameterTuningJobPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_training_jobs_for_hyper_parameter_tuning_job` for valid parameters"""
            pass
    class ListTransformJobsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_transform_jobs` for valid parameters"""
            pass
    class ListWorkteamsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_workteams` for valid parameters"""
            pass
    class SearchPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `search` for valid parameters"""
            pass


    class AddTagsInput(Mapping):
        pass
    
    class AddTagsOutput(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreateAlgorithmInput(Mapping):
        pass
    
    class CreateAlgorithmOutput(Mapping):
        pass
    
    class TrainingSpecification(Mapping):
        pass
    
    class InferenceSpecification(Mapping):
        pass
    
    class AlgorithmValidationSpecification(Mapping):
        pass
    
    class CertifyForMarketplace(object):
        pass
    
    class CreateCodeRepositoryInput(Mapping):
        pass
    
    class CreateCodeRepositoryOutput(Mapping):
        pass
    
    class GitConfig(Mapping):
        pass
    
    class CreateCompilationJobRequest(Mapping):
        pass
    
    class CreateCompilationJobResponse(Mapping):
        pass
    
    class InputConfig(Mapping):
        pass
    
    class OutputConfig(Mapping):
        pass
    
    class StoppingCondition(Mapping):
        pass
    
    class CreateEndpointInput(Mapping):
        pass
    
    class CreateEndpointOutput(Mapping):
        pass
    
    class CreateEndpointConfigInput(Mapping):
        pass
    
    class CreateEndpointConfigOutput(Mapping):
        pass
    
    class ProductionVariant(Mapping):
        pass
    
    class CreateHyperParameterTuningJobRequest(Mapping):
        pass
    
    class CreateHyperParameterTuningJobResponse(Mapping):
        pass
    
    class HyperParameterTuningJobConfig(Mapping):
        pass
    
    class HyperParameterTrainingJobDefinition(Mapping):
        pass
    
    class HyperParameterTuningJobWarmStartConfig(Mapping):
        pass
    
    class CreateLabelingJobRequest(Mapping):
        pass
    
    class CreateLabelingJobResponse(Mapping):
        pass
    
    class LabelingJobInputConfig(Mapping):
        pass
    
    class LabelingJobOutputConfig(Mapping):
        pass
    
    class LabelingJobStoppingConditions(Mapping):
        pass
    
    class LabelingJobAlgorithmsConfig(Mapping):
        pass
    
    class HumanTaskConfig(Mapping):
        pass
    
    class CreateModelInput(Mapping):
        pass
    
    class CreateModelOutput(Mapping):
        pass
    
    class ContainerDefinition(Mapping):
        pass
    
    class VpcConfig(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class CreateModelPackageInput(Mapping):
        pass
    
    class CreateModelPackageOutput(Mapping):
        pass
    
    class ModelPackageValidationSpecification(Mapping):
        pass
    
    class SourceAlgorithmSpecification(Mapping):
        pass
    
    class CreateNotebookInstanceInput(Mapping):
        pass
    
    class CreateNotebookInstanceOutput(Mapping):
        pass
    
    class SecurityGroupId(object):
        pass
    
    class NotebookInstanceAcceleratorType(object):
        pass
    
    class CodeRepositoryNameOrUrl(object):
        pass
    
    class CreateNotebookInstanceLifecycleConfigInput(Mapping):
        pass
    
    class CreateNotebookInstanceLifecycleConfigOutput(Mapping):
        pass
    
    class NotebookInstanceLifecycleHook(Mapping):
        pass
    
    class CreatePresignedNotebookInstanceUrlInput(Mapping):
        pass
    
    class CreatePresignedNotebookInstanceUrlOutput(Mapping):
        pass
    
    class CreateTrainingJobRequest(Mapping):
        pass
    
    class CreateTrainingJobResponse(Mapping):
        pass
    
    class HyperParameters(object):
        pass
    
    class AlgorithmSpecification(Mapping):
        pass
    
    class Channel(Mapping):
        pass
    
    class OutputDataConfig(Mapping):
        pass
    
    class ResourceConfig(Mapping):
        pass
    
    class CreateTransformJobRequest(Mapping):
        pass
    
    class CreateTransformJobResponse(Mapping):
        pass
    
    class TransformEnvironmentMap(object):
        pass
    
    class TransformInput(Mapping):
        pass
    
    class TransformOutput(Mapping):
        pass
    
    class TransformResources(Mapping):
        pass
    
    class DataProcessing(Mapping):
        pass
    
    class CreateWorkteamRequest(Mapping):
        pass
    
    class CreateWorkteamResponse(Mapping):
        pass
    
    class MemberDefinition(Mapping):
        pass
    
    class NotificationConfiguration(Mapping):
        pass
    
    class DeleteAlgorithmInput(Mapping):
        pass
    
    class DeleteCodeRepositoryInput(Mapping):
        pass
    
    class DeleteEndpointInput(Mapping):
        pass
    
    class DeleteEndpointConfigInput(Mapping):
        pass
    
    class DeleteModelInput(Mapping):
        pass
    
    class DeleteModelPackageInput(Mapping):
        pass
    
    class DeleteNotebookInstanceInput(Mapping):
        pass
    
    class DeleteNotebookInstanceLifecycleConfigInput(Mapping):
        pass
    
    class DeleteTagsInput(Mapping):
        pass
    
    class DeleteTagsOutput(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class DeleteWorkteamRequest(Mapping):
        pass
    
    class DeleteWorkteamResponse(Mapping):
        pass
    
    class DescribeAlgorithmInput(Mapping):
        pass
    
    class DescribeAlgorithmOutput(Mapping):
        pass
    
    class DescribeCodeRepositoryInput(Mapping):
        pass
    
    class DescribeCodeRepositoryOutput(Mapping):
        pass
    
    class DescribeCompilationJobRequest(Mapping):
        pass
    
    class DescribeCompilationJobResponse(Mapping):
        pass
    
    class DescribeEndpointInput(Mapping):
        pass
    
    class DescribeEndpointOutput(Mapping):
        pass
    
    class DescribeEndpointConfigInput(Mapping):
        pass
    
    class DescribeEndpointConfigOutput(Mapping):
        pass
    
    class DescribeHyperParameterTuningJobRequest(Mapping):
        pass
    
    class DescribeHyperParameterTuningJobResponse(Mapping):
        pass
    
    class DescribeLabelingJobRequest(Mapping):
        pass
    
    class DescribeLabelingJobResponse(Mapping):
        pass
    
    class DescribeModelInput(Mapping):
        pass
    
    class DescribeModelOutput(Mapping):
        pass
    
    class DescribeModelPackageInput(Mapping):
        pass
    
    class DescribeModelPackageOutput(Mapping):
        pass
    
    class DescribeNotebookInstanceInput(Mapping):
        pass
    
    class DescribeNotebookInstanceOutput(Mapping):
        pass
    
    class DescribeNotebookInstanceLifecycleConfigInput(Mapping):
        pass
    
    class DescribeNotebookInstanceLifecycleConfigOutput(Mapping):
        pass
    
    class DescribeSubscribedWorkteamRequest(Mapping):
        pass
    
    class DescribeSubscribedWorkteamResponse(Mapping):
        pass
    
    class DescribeTrainingJobRequest(Mapping):
        pass
    
    class DescribeTrainingJobResponse(Mapping):
        pass
    
    class DescribeTransformJobRequest(Mapping):
        pass
    
    class DescribeTransformJobResponse(Mapping):
        pass
    
    class DescribeWorkteamRequest(Mapping):
        pass
    
    class DescribeWorkteamResponse(Mapping):
        pass
    
    class GetSearchSuggestionsRequest(Mapping):
        pass
    
    class GetSearchSuggestionsResponse(Mapping):
        pass
    
    class SuggestionQuery(Mapping):
        pass
    
    class ListAlgorithmsInput(Mapping):
        pass
    
    class ListAlgorithmsOutput(Mapping):
        pass
    
    class CreationTime(object):
        pass
    
    class ListCodeRepositoriesInput(Mapping):
        pass
    
    class ListCodeRepositoriesOutput(Mapping):
        pass
    
    class Timestamp(object):
        pass
    
    class ListCompilationJobsRequest(Mapping):
        pass
    
    class ListCompilationJobsResponse(Mapping):
        pass
    
    class LastModifiedTime(object):
        pass
    
    class ListEndpointConfigsInput(Mapping):
        pass
    
    class ListEndpointConfigsOutput(Mapping):
        pass
    
    class ListEndpointsInput(Mapping):
        pass
    
    class ListEndpointsOutput(Mapping):
        pass
    
    class ListHyperParameterTuningJobsRequest(Mapping):
        pass
    
    class ListHyperParameterTuningJobsResponse(Mapping):
        pass
    
    class ListLabelingJobsRequest(Mapping):
        pass
    
    class ListLabelingJobsResponse(Mapping):
        pass
    
    class ListLabelingJobsForWorkteamRequest(Mapping):
        pass
    
    class ListLabelingJobsForWorkteamResponse(Mapping):
        pass
    
    class ListModelPackagesInput(Mapping):
        pass
    
    class ListModelPackagesOutput(Mapping):
        pass
    
    class ListModelsInput(Mapping):
        pass
    
    class ListModelsOutput(Mapping):
        pass
    
    class ListNotebookInstanceLifecycleConfigsInput(Mapping):
        pass
    
    class ListNotebookInstanceLifecycleConfigsOutput(Mapping):
        pass
    
    class ListNotebookInstancesInput(Mapping):
        pass
    
    class ListNotebookInstancesOutput(Mapping):
        pass
    
    class ListSubscribedWorkteamsRequest(Mapping):
        pass
    
    class ListSubscribedWorkteamsResponse(Mapping):
        pass
    
    class ListTagsInput(Mapping):
        pass
    
    class ListTagsOutput(Mapping):
        pass
    
    class ListTrainingJobsRequest(Mapping):
        pass
    
    class ListTrainingJobsResponse(Mapping):
        pass
    
    class ListTrainingJobsForHyperParameterTuningJobRequest(Mapping):
        pass
    
    class ListTrainingJobsForHyperParameterTuningJobResponse(Mapping):
        pass
    
    class ListTransformJobsRequest(Mapping):
        pass
    
    class ListTransformJobsResponse(Mapping):
        pass
    
    class ListWorkteamsRequest(Mapping):
        pass
    
    class ListWorkteamsResponse(Mapping):
        pass
    
    class RenderUiTemplateRequest(Mapping):
        pass
    
    class RenderUiTemplateResponse(Mapping):
        pass
    
    class UiTemplate(Mapping):
        pass
    
    class RenderableTask(Mapping):
        pass
    
    class SearchRequest(Mapping):
        pass
    
    class SearchResponse(Mapping):
        pass
    
    class SearchExpression(Mapping):
        pass
    
    class StartNotebookInstanceInput(Mapping):
        pass
    
    class StopCompilationJobRequest(Mapping):
        pass
    
    class StopHyperParameterTuningJobRequest(Mapping):
        pass
    
    class StopLabelingJobRequest(Mapping):
        pass
    
    class StopNotebookInstanceInput(Mapping):
        pass
    
    class StopTrainingJobRequest(Mapping):
        pass
    
    class StopTransformJobRequest(Mapping):
        pass
    
    class UpdateCodeRepositoryInput(Mapping):
        pass
    
    class UpdateCodeRepositoryOutput(Mapping):
        pass
    
    class GitConfigForUpdate(Mapping):
        pass
    
    class UpdateEndpointInput(Mapping):
        pass
    
    class UpdateEndpointOutput(Mapping):
        pass
    
    class UpdateEndpointWeightsAndCapacitiesInput(Mapping):
        pass
    
    class UpdateEndpointWeightsAndCapacitiesOutput(Mapping):
        pass
    
    class DesiredWeightAndCapacity(Mapping):
        pass
    
    class UpdateNotebookInstanceInput(Mapping):
        pass
    
    class UpdateNotebookInstanceOutput(Mapping):
        pass
    
    class DisassociateNotebookInstanceLifecycleConfig(object):
        pass
    
    class DisassociateNotebookInstanceAcceleratorTypes(object):
        pass
    
    class DisassociateDefaultCodeRepository(object):
        pass
    
    class DisassociateAdditionalCodeRepositories(object):
        pass
    
    class UpdateNotebookInstanceLifecycleConfigInput(Mapping):
        pass
    
    class UpdateNotebookInstanceLifecycleConfigOutput(Mapping):
        pass
    
    class UpdateWorkteamRequest(Mapping):
        pass
    
    class UpdateWorkteamResponse(Mapping):
        pass
    
    def add_tags(self, Tags: List[Tag], ResourceArn: str) -> AddTagsOutput:
        """Adds or overwrites one or more tags for the specified Amazon SageMaker resource. You can add tags to notebook instances, training jobs, hyperparameter tuning jobs, batch transform jobs, models, labeling jobs, work teams, endpoint configurations, and endpoints. Each tag consists of a key and an optional value. Tag keys must be unique per resource. For more information about tags, see For more information, see <a href="https://aws.amazon.com/answers/account-management/aws-tagging-strategies/">AWS Tagging Strategies</a>. <note> Tags that you add to a hyperparameter tuning job by calling this API are also added to any training jobs that the hyperparameter tuning job launches after you call this API, but not to training jobs that the hyperparameter tuning job launched before you called this API. To make sure that the tags associated with a hyperparameter tuning job are also added to all training jobs that the hyperparameter tuning job launches, add the tags when you first create the tuning job by specifying them in the <code>Tags</code> parameter of <a>CreateHyperParameterTuningJob</a>  </note>

        :param self:
        :param Tags:  An array of <code>Tag</code> objects. Each tag is a key-value pair. Only the <code>key</code> parameter is required. If you don't specify a value, Amazon SageMaker sets the value to an empty string. <br/>ResourceArn:  The Amazon Resource Name (ARN) of the resource that you want to tag.<br/>
        :return: Tags:  A list of tags associated with the Amazon SageMaker resource.<br/>        """
        pass

    def create_algorithm(self, TrainingSpecification: TrainingSpecification, AlgorithmName: str, AlgorithmDescription: str=None, InferenceSpecification: InferenceSpecification=None, ValidationSpecification: AlgorithmValidationSpecification=None, CertifyForMarketplace: CertifyForMarketplace=None) -> CreateAlgorithmOutput:
        """Create a machine learning algorithm that you can use in Amazon SageMaker and list in the AWS Marketplace.

        :param self:
        :param CertifyForMarketplace:  Whether to certify the algorithm so that it can be listed in AWS Marketplace.<br/>ValidationSpecification:  Specifies configurations for one or more training jobs and that Amazon SageMaker runs to test the algorithm's training code and, optionally, one or more batch transform jobs that Amazon SageMaker runs to test the algorithm's inference code.:<br/>InferenceSpecification:  Specifies details about inference jobs that the algorithm runs, including the following: <ul> <li> The Amazon ECR paths of containers that contain the inference code and model artifacts. </li> <li> The instance types that the algorithm supports for transform jobs and real-time endpoints used for inference. </li> <li> The input and output content formats that the algorithm supports for inference. </li> </ul>:<br/>TrainingSpecification:  Specifies details about training jobs run by this algorithm, including the following: <ul> <li> The Amazon ECR path of the container and the version digest of the algorithm. </li> <li> The hyperparameters that the algorithm supports. </li> <li> The instance types that the algorithm supports for training. </li> <li> Whether the algorithm supports distributed training. </li> <li> The metrics that the algorithm emits to Amazon CloudWatch. </li> <li> Which metrics that the algorithm emits can be used as the objective metric for hyperparameter tuning jobs. </li> <li> The input channels that the algorithm supports for training data. For example, an algorithm might support <code>train</code>, <code>validation</code>, and <code>test</code> channels. </li> </ul>:<br/>AlgorithmDescription:  A description of the algorithm.<br/>AlgorithmName:  The name of the algorithm.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedTuningJobObjectiveMetrics</i> A list of the metrics that the algorithm emits that can be used as the objective metric in a hyperparameter tuning job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingChannels</i> A list of <code>ChannelSpecification</code> objects, which specify the input sources to be used by the algorithm.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricDefinitions</i> A list of <code>MetricDefinition</code> objects, which are used for parsing metrics generated by the algorithm.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportsDistributedTraining</i> Indicates whether the algorithm supports distributed training. If set to false, buyers can’t request more than one instance during training.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedTrainingInstanceTypes</i> A list of the instance types that this algorithm can use for training.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedHyperParameters</i> A list of the <code>HyperParameterSpecification</code> objects, that define the supported hyperparameters. This is required if the algorithm supports automatic model tuning.&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingImageDigest</i> An MD5 hash of the training algorithm that identifies the Docker image used for training.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingImage</i> The Amazon ECR registry path of the Docker image that contains the training algorithm.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedResponseMIMETypes</i> The supported MIME types for the output data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedContentTypes</i> The supported MIME types for the input data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedRealtimeInferenceInstanceTypes</i> A list of the instance types that are used to generate inferences in real-time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedTransformInstanceTypes</i> A list of the instance types on which a transformation job can be run or on which an endpoint can be deployed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Containers</i> The Amazon ECR registry path of the Docker image that contains the inference code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ValidationProfiles</i> An array of <code>AlgorithmValidationProfile</code> objects, each of which specifies a training job and batch transform job that Amazon SageMaker runs to validate your algorithm.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ValidationRole</i> The IAM roles that Amazon SageMaker uses to run the training jobs.<br/>
        :return: AlgorithmArn:  The Amazon Resource Name (ARN) of the new algorithm.<br/>        """
        pass

    def create_code_repository(self, GitConfig: GitConfig, CodeRepositoryName: str) -> CreateCodeRepositoryOutput:
        """Creates a Git repository as a resource in your Amazon SageMaker account. You can associate the repository with notebook instances so that you can use Git source control for the notebooks you create. The Git repository is a resource in your Amazon SageMaker account, so it can be associated with more than one notebook instance, and it persists independently from the lifecycle of any notebook instances it is associated with. The repository can be hosted either in <a href="http://docs.aws.amazon.com/codecommit/latest/userguide/welcome.html">AWS CodeCommit</a> or in any other Git repository.

        :param self:
        :param GitConfig:  Specifies details about the repository, including the URL where the repository is located, the default branch, and credentials to use to access the repository.:<br/>CodeRepositoryName:  The name of the Git repository. The name must have 1 to 63 characters. Valid characters are a-z, A-Z, 0-9, and - (hyphen).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecretArn</i> The Amazon Resource Name (ARN) of the AWS Secrets Manager secret that contains the credentials used to access the git repository. The secret must have a staging label of <code>AWSCURRENT</code> and must be in the following format:  <code>{"username": <i>UserName</i>, "password": <i>Password</i>}</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Branch</i> The default branch for the Git repository.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RepositoryUrl</i> The URL where the Git repository is located.<br/>
        :return: CodeRepositoryArn:  The Amazon Resource Name (ARN) of the new repository.<br/>        """
        pass

    def create_compilation_job(self, StoppingCondition: StoppingCondition, OutputConfig: OutputConfig, InputConfig: InputConfig, RoleArn: str, CompilationJobName: str) -> CreateCompilationJobResponse:
        """Starts a model compilation job. After the model has been compiled, Amazon SageMaker saves the resulting model artifacts to an Amazon Simple Storage Service (Amazon S3) bucket that you specify.  If you choose to host your model using Amazon SageMaker hosting services, you can use the resulting model artifacts as part of the model. You can also use the artifacts with AWS IoT Greengrass. In that case, deploy them as an ML resource. In the request body, you provide the following: <ul> <li> A name for the compilation job </li> <li>  Information about the input model artifacts  </li> <li> The output location for the compiled model and the device (target) that the model runs on  </li> <li>  <code>The Amazon Resource Name (ARN) of the IAM role that Amazon SageMaker assumes to perform the model compilation job</code>  </li> </ul> You can also provide a <code>Tag</code> to track the model compilation job's resource use and costs. The response body contains the <code>CompilationJobArn</code> for the compiled job. To stop a model compilation job, use <a>StopCompilationJob</a>. To get information about a particular model compilation job, use <a>DescribeCompilationJob</a>. To get information about multiple model compilation jobs, use <a>ListCompilationJobs</a>.

        :param self:
        :param StoppingCondition:  Specifies a limit to how long a model compilation job can run. When the job reaches the time limit, Amazon SageMaker ends the compilation job. Use this API to cap model training costs.:<br/>OutputConfig:  Provides information about the output location for the compiled model and the target device the model runs on.:<br/>InputConfig:  Provides information about the location of input model artifacts, the name and shape of the expected data inputs, and the framework in which the model was trained.:<br/>RoleArn:  The Amazon Resource Name (ARN) of an IAM role that enables Amazon SageMaker to perform tasks on your behalf.  During model compilation, Amazon SageMaker needs your permission to: <ul> <li> Read input data from an S3 bucket </li> <li> Write model artifacts to an S3 bucket </li> <li> Write logs to Amazon CloudWatch Logs </li> <li> Publish metrics to Amazon CloudWatch </li> </ul> You grant permissions for all of these tasks to an IAM role. To pass this role to Amazon SageMaker, the caller of this API must have the <code>iam:PassRole</code> permission. For more information, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/sagemaker-roles.html">Amazon SageMaker Roles.</a> <br/>CompilationJobName:  A name for the model compilation job. The name must be unique within the AWS Region and within your AWS account. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Framework</i> Identifies the framework in which the model was trained. For example: TENSORFLOW.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataInputConfig</i> Specifies the name and shape of the expected data inputs for your trained model with a JSON dictionary form. The data inputs are <a>InputConfig$Framework</a> specific.  <ul> <li>  <code>TensorFlow</code>: You must specify the name and shape (NHWC format) of the expected data inputs using a dictionary format for your trained model. The dictionary formats required for the console and CLI are different. <ul> <li> Examples for one input: <ul> <li> If using the console, <code>{"input":[1,1024,1024,3]}</code>  </li> <li> If using the CLI, <code>{\"input\":[1,1024,1024,3]}</code>  </li> </ul> </li> <li> Examples for two inputs: <ul> <li> If using the console, <code>{"data1": [1,28,28,1], "data2":[1,28,28,1]}</code>  </li> <li> If using the CLI, <code>{\"data1\": [1,28,28,1], \"data2\":[1,28,28,1]}</code>  </li> </ul> </li> </ul> </li> <li>  <code>MXNET/ONNX</code>: You must specify the name and shape (NCHW format) of the expected data inputs in order using a dictionary format for your trained model. The dictionary formats required for the console and CLI are different. <ul> <li> Examples for one input: <ul> <li> If using the console, <code>{"data":[1,3,1024,1024]}</code>  </li> <li> If using the CLI, <code>{\"data\":[1,3,1024,1024]}</code>  </li> </ul> </li> <li> Examples for two inputs: <ul> <li> If using the console, <code>{"var1": [1,1,28,28], "var2":[1,1,28,28]} </code>  </li> <li> If using the CLI, <code>{\"var1\": [1,1,28,28], \"var2\":[1,1,28,28]}</code>  </li> </ul> </li> </ul> </li> <li>  <code>PyTorch</code>: You can either specify the name and shape (NCHW format) of expected data inputs in order using a dictionary format for your trained model or you can specify the shape only using a list format. The dictionary formats required for the console and CLI are different. The list formats for the console and CLI are the same. <ul> <li> Examples for one input in dictionary format: <ul> <li> If using the console, <code>{"input0":[1,3,224,224]}</code>  </li> <li> If using the CLI, <code>{\"input0\":[1,3,224,224]}</code>  </li> </ul> </li> <li> Example for one input in list format: <code>[[1,3,224,224]]</code>  </li> <li> Examples for two inputs in dictionary format: <ul> <li> If using the console, <code>{"input0":[1,3,224,224], "input1":[1,3,224,224]}</code>  </li> <li> If using the CLI, <code>{\"input0\":[1,3,224,224], \"input1\":[1,3,224,224]} </code>  </li> </ul> </li> <li> Example for two inputs in list format: <code>[[1,3,224,224], [1,3,224,224]]</code>  </li> </ul> </li> <li>  <code>XGBOOST</code>: input data name and shape are not needed. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> The S3 path where the model artifacts, which result from model training, are stored. This path must point to a single gzip compressed tar archive (.tar.gz suffix).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetDevice</i> Identifies the device that you want to run your model on after it has been compiled. For example: ml_c5.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3OutputLocation</i> Identifies the S3 path where you want Amazon SageMaker to store the model artifacts. For example, s3://bucket-name/key-name-prefix.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxRuntimeInSeconds</i> The maximum length of time, in seconds, that the training or compilation job can run. If job does not complete during this time, Amazon SageMaker ends the job. If value is not specified, default value is 1 day. The maximum value is 28 days.<br/>
        :return: CompilationJobArn:  If the action is successful, the service sends back an HTTP 200 response. Amazon SageMaker returns the following data in JSON format: <ul> <li>  <code>CompilationJobArn</code>: The Amazon Resource Name (ARN) of the compiled job. </li> </ul><br/>        """
        pass

    def create_endpoint(self, EndpointConfigName: str, EndpointName: str, Tags: List[Tag]=None) -> CreateEndpointOutput:
        """Creates an endpoint using the endpoint configuration specified in the request. Amazon SageMaker uses the endpoint to provision resources and deploy models. You create the endpoint configuration with the <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/API_CreateEndpointConfig.html">CreateEndpointConfig</a> API.  <note>  Use this API only for hosting models using Amazon SageMaker hosting services.   You must not delete an <code>EndpointConfig</code> in use by an endpoint that is live or while the <code>UpdateEndpoint</code> or <code>CreateEndpoint</code> operations are being performed on the endpoint. To update an endpoint, you must create a new <code>EndpointConfig</code>. </note> The endpoint name must be unique within an AWS Region in your AWS account.  When it receives the request, Amazon SageMaker creates the endpoint, launches the resources (ML compute instances), and deploys the model(s) on them.  When Amazon SageMaker receives the request, it sets the endpoint status to <code>Creating</code>. After it creates the endpoint, it sets the status to <code>InService</code>. Amazon SageMaker can then process incoming requests for inferences. To check the status of an endpoint, use the <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/API_DescribeEndpoint.html">DescribeEndpoint</a> API. For an example, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/ex1.html">Exercise 1: Using the K-Means Algorithm Provided by Amazon SageMaker</a>.  If any of the models hosted at this endpoint get model data from an Amazon S3 location, Amazon SageMaker uses AWS Security Token Service to download model artifacts from the S3 path you provided. AWS STS is activated in your IAM user account by default. If you previously deactivated AWS STS for a region, you need to reactivate AWS STS for that region. For more information, see <a href="http://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_temp_enable-regions.html">Activating and Deactivating AWS STS i an AWS Region</a> in the <i>AWS Identity and Access Management User Guide</i>.

        :param self:
        :param Tags:  An array of key-value pairs. For more information, see <a href="https://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/cost-alloc-tags.html#allocation-what">Using Cost Allocation Tags</a>in the <i>AWS Billing and Cost Management User Guide</i>. <br/>EndpointConfigName:  The name of an endpoint configuration. For more information, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/API_CreateEndpointConfig.html">CreateEndpointConfig</a>. <br/>EndpointName:  The name of the endpoint. The name must be unique within an AWS Region in your AWS account.<br/>
        :return: EndpointArn:  The Amazon Resource Name (ARN) of the endpoint.<br/>        """
        pass

    def create_endpoint_config(self, ProductionVariants: List[ProductionVariant], EndpointConfigName: str, Tags: List[Tag]=None, KmsKeyId: str=None) -> CreateEndpointConfigOutput:
        """Creates an endpoint configuration that Amazon SageMaker hosting services uses to deploy models. In the configuration, you identify one or more models, created using the <code>CreateModel</code> API, to deploy and the resources that you want Amazon SageMaker to provision. Then you call the <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/API_CreateEndpoint.html">CreateEndpoint</a> API. <note>  Use this API only if you want to use Amazon SageMaker hosting services to deploy models into production.  </note> In the request, you define one or more <code>ProductionVariant</code>s, each of which identifies a model. Each <code>ProductionVariant</code> parameter also describes the resources that you want Amazon SageMaker to provision. This includes the number and type of ML compute instances to deploy.  If you are hosting multiple models, you also assign a <code>VariantWeight</code> to specify how much traffic you want to allocate to each model. For example, suppose that you want to host two models, A and B, and you assign traffic weight 2 for model A and 1 for model B. Amazon SageMaker distributes two-thirds of the traffic to Model A, and one-third to model B. 

        :param self:
        :param KmsKeyId:  The Amazon Resource Name (ARN) of a AWS Key Management Service key that Amazon SageMaker uses to encrypt data on the storage volume attached to the ML compute instance that hosts the endpoint.<br/>Tags:  A list of key-value pairs. For more information, see <a href="https://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/cost-alloc-tags.html#allocation-what">Using Cost Allocation Tags</a> in the <i> AWS Billing and Cost Management User Guide</i>. <br/>ProductionVariants:  An list of <code>ProductionVariant</code> objects, one for each model that you want to host at this endpoint.<br/>EndpointConfigName:  The name of the endpoint configuration. You specify this name in a <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/API_CreateEndpoint.html">CreateEndpoint</a> request. <br/>
        :return: EndpointConfigArn:  The Amazon Resource Name (ARN) of the endpoint configuration. <br/>        """
        pass

    def create_hyper_parameter_tuning_job(self, HyperParameterTuningJobConfig: HyperParameterTuningJobConfig, HyperParameterTuningJobName: str, TrainingJobDefinition: HyperParameterTrainingJobDefinition=None, WarmStartConfig: HyperParameterTuningJobWarmStartConfig=None, Tags: List[Tag]=None) -> CreateHyperParameterTuningJobResponse:
        """Starts a hyperparameter tuning job. A hyperparameter tuning job finds the best version of a model by running many training jobs on your dataset using the algorithm you choose and values for hyperparameters within ranges that you specify. It then chooses the hyperparameter values that result in a model that performs the best, as measured by an objective metric that you choose.

        :param self:
        :param Tags:  An array of key-value pairs. You can use tags to categorize your AWS resources in different ways, for example, by purpose, owner, or environment. For more information, see <a href="https://aws.amazon.com/answers/account-management/aws-tagging-strategies/">AWS Tagging Strategies</a>. Tags that you specify for the tuning job are also added to all training jobs that the tuning job launches.<br/>WarmStartConfig:  Specifies the configuration for starting the hyperparameter tuning job using one or more previous tuning jobs as a starting point. The results of previous tuning jobs are used to inform which combinations of hyperparameters to search over in the new tuning job. All training jobs launched by the new hyperparameter tuning job are evaluated by using the objective metric. If you specify <code>IDENTICAL_DATA_AND_ALGORITHM</code> as the <code>WarmStartType</code> value for the warm start configuration, the training job that performs the best in the new tuning job is compared to the best training jobs from the parent tuning jobs. From these, the training job that performs the best as measured by the objective metric is returned as the overall best training job. <note> All training jobs launched by parent hyperparameter tuning jobs and the new hyperparameter tuning jobs count against the limit of training jobs for the tuning job. </note>:<br/>TrainingJobDefinition:  The <a>HyperParameterTrainingJobDefinition</a> object that describes the training jobs that this tuning job launches, including static hyperparameters, input data configuration, output data configuration, resource configuration, and stopping condition.:<br/>HyperParameterTuningJobConfig:  The <a>HyperParameterTuningJobConfig</a> object that describes the tuning job, including the search strategy, the objective metric used to evaluate training jobs, ranges of parameters to search, and resource limits for the tuning job. For more information, see <a>automatic-model-tuning</a> :<br/>HyperParameterTuningJobName:  The name of the tuning job. This name is the prefix for the names of all training jobs that this tuning job launches. The name must be unique within the same AWS account and AWS Region. The name must have { } to { } characters. Valid characters are a-z, A-Z, 0-9, and : + = @ _ % - (hyphen). The name is not case sensitive.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingJobEarlyStoppingType</i> Specifies whether to use early stopping for training jobs launched by the hyperparameter tuning job. This can be one of the following values (the default value is <code>OFF</code>): <dl> <dt>OFF</dt> <dd> Training jobs launched by the hyperparameter tuning job do not use early stopping. </dd> <dt>AUTO</dt> <dd> Amazon SageMaker stops training jobs launched by the hyperparameter tuning job when they are unlikely to perform better than previously completed training jobs. For more information, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/automatic-model-tuning-early-stopping.html">Stop Training Jobs Early</a>. </dd> </dl><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterRanges</i> The <a>ParameterRanges</a> object that specifies the ranges of hyperparameters that this tuning job searches.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ParameterRanges</b>: The <a>ParameterRanges</a> object that specifies the ranges of hyperparameters that this tuning job searches.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceLimits</i> The <a>ResourceLimits</a> object that specifies the maximum number of training jobs and parallel training jobs for this tuning job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResourceLimits</b>: The <a>ResourceLimits</a> object that specifies the maximum number of training jobs and parallel training jobs for this tuning job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HyperParameterTuningJobObjective</i> The <a>HyperParameterTuningJobObjective</a> object that specifies the objective metric for this tuning job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>HyperParameterTuningJobObjective</b>: The <a>HyperParameterTuningJobObjective</a> object that specifies the objective metric for this tuning job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Strategy</i> Specifies how hyperparameter tuning chooses the combinations of hyperparameter values to use for the training job it launches. To use the Bayesian search stategy, set this to <code>Bayesian</code>. To randomly search, set it to <code>Random</code>. For information about search strategies, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/automatic-model-tuning-how-it-works.html">How Hyperparameter Tuning Works</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricName</i> The name of the metric to use for the objective metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> Whether to minimize or maximize the objective metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxParallelTrainingJobs</i> The maximum number of concurrent training jobs that a hyperparameter tuning job can launch.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxNumberOfTrainingJobs</i> The maximum number of training jobs that a hyperparameter tuning job can launch.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CategoricalParameterRanges</i> The array of <a>CategoricalParameterRange</a> objects that specify ranges of categorical hyperparameters that a hyperparameter tuning job searches.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ContinuousParameterRanges</i> The array of <a>ContinuousParameterRange</a> objects that specify ranges of continuous hyperparameters that a hyperparameter tuning job searches.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IntegerParameterRanges</i> The array of <a>IntegerParameterRange</a> objects that specify ranges of integer hyperparameters that a hyperparameter tuning job searches.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnableInterContainerTrafficEncryption</i> To encrypt all communications between ML compute instances in distributed training, choose <code>True</code>. Encryption provides greater security for distributed training, but training might take longer. How long it takes depends on the amount of communication between compute instances, especially if you use a deep learning algorithm in distributed training.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnableNetworkIsolation</i> Isolates the training container. No inbound or outbound network calls can be made, except for calls between peers within a training cluster for distributed training. If network isolation is used for training jobs that are configured to use a VPC, Amazon SageMaker downloads and uploads customer data and model artifacts through the specified VPC, but the training container does not have network access. <note> The Semantic Segmentation built-in algorithm does not support network isolation. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StoppingCondition</i> Specifies a limit to how long a model hyperparameter training job can run. When the job reaches the time limit, Amazon SageMaker ends the training job. Use this API to cap model training costs.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StoppingCondition</b>: Specifies a limit to how long a model hyperparameter training job can run. When the job reaches the time limit, Amazon SageMaker ends the training job. Use this API to cap model training costs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceConfig</i> The resources, including the compute instances and storage volumes, to use for the training jobs that the tuning job launches. Storage volumes store model artifacts and incremental states. Training algorithms might also use storage volumes for scratch space. If you want Amazon SageMaker to use the storage volume to store the training data, choose <code>File</code> as the <code>TrainingInputMode</code> in the algorithm specification. For distributed training algorithms, specify an instance count greater than 1.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResourceConfig</b>: The resources, including the compute instances and storage volumes, to use for the training jobs that the tuning job launches. Storage volumes store model artifacts and incremental states. Training algorithms might also use storage volumes for scratch space. If you want Amazon SageMaker to use the storage volume to store the training data, choose <code>File</code> as the <code>TrainingInputMode</code> in the algorithm specification. For distributed training algorithms, specify an instance count greater than 1.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputDataConfig</i> Specifies the path to the Amazon S3 bucket where you store model artifacts from the training jobs that the tuning job launches.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>OutputDataConfig</b>: Specifies the path to the Amazon S3 bucket where you store model artifacts from the training jobs that the tuning job launches.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcConfig</i> The <a>VpcConfig</a> object that specifies the VPC that you want the training jobs that this hyperparameter tuning job launches to connect to. Control access to and from your training container by configuring the VPC. For more information, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/train-vpc.html">Protect Training Jobs by Using an Amazon Virtual Private Cloud</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VpcConfig</b>: The <a>VpcConfig</a> object that specifies the VPC that you want the training jobs that this hyperparameter tuning job launches to connect to. Control access to and from your training container by configuring the VPC. For more information, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/train-vpc.html">Protect Training Jobs by Using an Amazon Virtual Private Cloud</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputDataConfig</i> An array of <a>Channel</a> objects that specify the input for the training jobs that the tuning job launches.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The Amazon Resource Name (ARN) of the IAM role associated with the training jobs that the tuning job launches.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AlgorithmSpecification</i> The <a>HyperParameterAlgorithmSpecification</a> object that specifies the resource algorithm to use for the training jobs that the tuning job launches.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AlgorithmSpecification</b>: The <a>HyperParameterAlgorithmSpecification</a> object that specifies the resource algorithm to use for the training jobs that the tuning job launches.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StaticHyperParameters</i> Specifies the values of hyperparameters that do not change for the tuning job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricDefinitions</i> An array of <a>MetricDefinition</a> objects that specify the metrics that the algorithm emits.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AlgorithmName</i> The name of the resource algorithm to use for the hyperparameter tuning job. If you specify a value for this parameter, do not specify a value for <code>TrainingImage</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingInputMode</i> The input mode that the algorithm supports: File or Pipe. In File input mode, Amazon SageMaker downloads the training data from Amazon S3 to the storage volume that is attached to the training instance and mounts the directory to the Docker volume for the training container. In Pipe input mode, Amazon SageMaker streams data directly from Amazon S3 to the container.  If you specify File mode, make sure that you provision the storage volume that is attached to the training instance with enough capacity to accommodate the training data downloaded from Amazon S3, the model artifacts, and intermediate information. <p/> For more information about input modes, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/algos.html">Algorithms</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingImage</i>  The registry path of the Docker image that contains the training algorithm. For information about Docker registry paths for built-in algorithms, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/sagemaker-algo-docker-registry-paths.html">Algorithms Provided by Amazon SageMaker: Common Parameters</a>. Amazon SageMaker supports both <code>registry/repository[:tag]</code> and <code>registry/repository[@digest]</code> image path formats. For more information, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/your-algorithms.html">Using Your Own Algorithms with Amazon SageMaker</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> The ID of the subnets in the VPC to which you want to connect your training job or model.  <note> Amazon EC2 P3 accelerated computing instances are not available in the c/d/e availability zones of region us-east-1. If you want to create endpoints with P3 instances in VPC mode in region us-east-1, create subnets in a/b/f availability zones instead. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The VPC security group IDs, in the form sg-xxxxxxxx. Specify the security groups for the VPC that is specified in the <code>Subnets</code> field.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3OutputPath</i> Identifies the S3 path where you want Amazon SageMaker to store the model artifacts. For example, <code>s3://bucket-name/key-name-prefix</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS Key Management Service (AWS KMS) key that Amazon SageMaker uses to encrypt the model artifacts at rest using Amazon S3 server-side encryption. The <code>KmsKeyId</code> can be any of the following formats:  <ul> <li> // KMS Key ID  <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> // Amazon Resource Name (ARN) of a KMS Key  <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> // KMS Key Alias  <code>"alias/ExampleAlias"</code>  </li> <li> // Amazon Resource Name (ARN) of a KMS Key Alias  <code>"arn:aws:kms:us-west-2:111122223333:alias/ExampleAlias"</code>  </li> </ul> If you use a KMS key ID or an alias of your master key, the Amazon SageMaker execution role must include permissions to call <code>kms:Encrypt</code>. If you don't provide a KMS key ID, Amazon SageMaker uses the default KMS key for Amazon S3 for your role's account. Amazon SageMaker uses server-side encryption with KMS-managed keys for <code>OutputDataConfig</code>. If you use a bucket policy with an <code>s3:PutObject</code> permission that only allows objects with server-side encryption, set the condition key of <code>s3:x-amz-server-side-encryption</code> to <code>"aws:kms"</code>. For more information, see <a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/UsingKMSEncryption.html">KMS-Managed Encryption Keys</a> in the <i>Amazon Simple Storage Service Developer Guide.</i>  The KMS key policy must grant permission to the IAM role that you specify in your <code>CreateTrainingJob</code>, <code>CreateTransformJob</code>, or <code>CreateHyperParameterTuningJob</code> requests. For more information, see <a href="http://docs.aws.amazon.com/kms/latest/developerguide/key-policies.html">Using Key Policies in AWS KMS</a> in the <i>AWS Key Management Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeKmsKeyId</i> The AWS Key Management Service (AWS KMS) key that Amazon SageMaker uses to encrypt data on the storage volume attached to the ML compute instance(s) that run the training job. The <code>VolumeKmsKeyId</code> can be any of the following formats: <ul> <li> // KMS Key ID  <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> // Amazon Resource Name (ARN) of a KMS Key  <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeSizeInGB</i> The size of the ML storage volume that you want to provision.  ML storage volumes store model artifacts and incremental states. Training algorithms might also use the ML storage volume for scratch space. If you want to store the training data in the ML storage volume, choose <code>File</code> as the <code>TrainingInputMode</code> in the algorithm specification.  You must specify sufficient ML storage for your scenario.  <note>  Amazon SageMaker supports only the General Purpose SSD (gp2) ML storage volume type.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCount</i> The number of ML compute instances to use. For distributed training, provide a value greater than 1. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i> The ML compute instance type. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxRuntimeInSeconds</i> The maximum length of time, in seconds, that the training or compilation job can run. If job does not complete during this time, Amazon SageMaker ends the job. If value is not specified, default value is 1 day. The maximum value is 28 days.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WarmStartType</i> Specifies one of the following: <dl> <dt>IDENTICAL_DATA_AND_ALGORITHM</dt> <dd> The new hyperparameter tuning job uses the same input data and training image as the parent tuning jobs. You can change the hyperparameter ranges to search and the maximum number of training jobs that the hyperparameter tuning job launches. You cannot use a new version of the training algorithm, unless the changes in the new version do not affect the algorithm itself. For example, changes that improve logging or adding support for a different data format are allowed. You can also change hyperparameters from tunable to static, and from static to tunable, but the total number of static plus tunable hyperparameters must remain the same as it is in all parent jobs. The objective metric for the new tuning job must be the same as for all parent jobs. </dd> <dt>TRANSFER_LEARNING</dt> <dd> The new hyperparameter tuning job can include input data, hyperparameter ranges, maximum number of concurrent training jobs, and maximum number of training jobs that are different than those of its parent hyperparameter tuning jobs. The training image can also be a different version from the version used in the parent hyperparameter tuning job. You can also change hyperparameters from tunable to static, and from static to tunable, but the total number of static plus tunable hyperparameters must remain the same as it is in all parent jobs. The objective metric for the new tuning job must be the same as for all parent jobs. </dd> </dl><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParentHyperParameterTuningJobs</i> An array of hyperparameter tuning jobs that are used as the starting point for the new hyperparameter tuning job. For more information about warm starting a hyperparameter tuning job, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/automatic-model-tuning-warm-start.html">Using a Previous Hyperparameter Tuning Job as a Starting Point</a>. Hyperparameter tuning jobs created before October 1, 2018 cannot be used as parent jobs for warm start tuning jobs.<br/>
        :return: HyperParameterTuningJobArn:  The Amazon Resource Name (ARN) of the tuning job. Amazon SageMaker assigns an ARN to a hyperparameter tuning job when you create it.<br/>        """
        pass

    def create_labeling_job(self, HumanTaskConfig: HumanTaskConfig, RoleArn: str, OutputConfig: LabelingJobOutputConfig, InputConfig: LabelingJobInputConfig, LabelAttributeName: str, LabelingJobName: str, LabelCategoryConfigS3Uri: str=None, StoppingConditions: LabelingJobStoppingConditions=None, LabelingJobAlgorithmsConfig: LabelingJobAlgorithmsConfig=None, Tags: List[Tag]=None) -> CreateLabelingJobResponse:
        """Creates a job that uses workers to label the data objects in your input dataset. You can use the labeled data to train machine learning models. You can select your workforce from one of three providers: <ul> <li> A private workforce that you create. It can include employees, contractors, and outside experts. Use a private workforce when want the data to stay within your organization or when a specific set of skills is required. </li> <li> One or more vendors that you select from the AWS Marketplace. Vendors provide expertise in specific areas.  </li> <li> The Amazon Mechanical Turk workforce. This is the largest workforce, but it should only be used for public data or data that has been stripped of any personally identifiable information. </li> </ul> You can also use <i>automated data labeling</i> to reduce the number of data objects that need to be labeled by a human. Automated data labeling uses <i>active learning</i> to determine if a data object can be labeled by machine or if it needs to be sent to a human worker. For more information, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/sms-automated-labeling.html">Using Automated Data Labeling</a>. The data objects to be labeled are contained in an Amazon S3 bucket. You create a <i>manifest file</i> that describes the location of each object. For more information, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/sms-data.html">Using Input and Output Data</a>. The output can be used as the manifest file for another labeling job or as training data for your machine learning models.

        :param self:
        :param Tags:  An array of key/value pairs. For more information, see <a href="http://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/cost-alloc-tags.html#allocation-what">Using Cost Allocation Tags</a> in the <i>AWS Billing and Cost Management User Guide</i>.<br/>HumanTaskConfig:  Configures the information required for human workers to complete a labeling task.:<br/>LabelingJobAlgorithmsConfig:  Configures the information required to perform automated data labeling.:<br/>StoppingConditions:  A set of conditions for stopping the labeling job. If any of the conditions are met, the job is automatically stopped. You can use these conditions to control the cost of data labeling.:<br/>LabelCategoryConfigS3Uri:  The S3 URL of the file that defines the categories used to label the data objects. The file is a JSON structure in the following format:  <code>{</code>   <code> "document-version": "2018-11-28"</code>   <code> "labels": [</code>   <code> {</code>   <code> "label": "<i>label 1</i>"</code>   <code> },</code>   <code> {</code>   <code> "label": "<i>label 2</i>"</code>   <code> },</code>   <code> ...</code>   <code> {</code>   <code> "label": "<i>label n</i>"</code>   <code> }</code>   <code> ]</code>   <code>}</code> <br/>RoleArn:  The Amazon Resource Number (ARN) that Amazon SageMaker assumes to perform tasks on your behalf during data labeling. You must grant this role the necessary permissions so that Amazon SageMaker can successfully complete data labeling.<br/>OutputConfig:  The location of the output data and the AWS Key Management Service key ID for the key used to encrypt the output data, if any.:<br/>InputConfig:  Input data for the labeling job, such as the Amazon S3 location of the data objects and the location of the manifest file that describes the data objects.:<br/>LabelAttributeName:  The attribute name to use for the label in the output manifest file. This is the key for the key/value pair formed with the label that a worker assigns to the object. The name can't end with "-metadata". If you are running a semantic segmentation labeling job, the attribute name must end with "-ref". If you are running any other kind of labeling job, the attribute name must not end with "-ref".<br/>LabelingJobName:  The name of the labeling job. This name is used to identify the job in a list of labeling jobs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataAttributes</i> Attributes of the data specified by the customer.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DataAttributes</b>: Attributes of the data specified by the customer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataSource</i> The location of the input data.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DataSource</b>: The location of the input data.<br/>The Amazon S3 location of the input data objects.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3DataSource</b>: The Amazon S3 location of the input data objects.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ContentClassifiers</i> Declares that your content is free of personally identifiable information or adult content. Amazon SageMaker may restrict the Amazon Mechanical Turk workers that can view your task based on this information.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS Key Management Service ID of the key used to encrypt the output data, if any. If you use a KMS key ID or an alias of your master key, the Amazon SageMaker execution role must include permissions to call <code>kms:Encrypt</code>. If you don't provide a KMS key ID, Amazon SageMaker uses the default KMS key for Amazon S3 for your role's account. Amazon SageMaker uses server-side encryption with KMS-managed keys for <code>LabelingJobOutputConfig</code>. If you use a bucket policy with an <code>s3:PutObject</code> permission that only allows objects with server-side encryption, set the condition key of <code>s3:x-amz-server-side-encryption</code> to <code>"aws:kms"</code>. For more information, see <a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/UsingKMSEncryption.html">KMS-Managed Encryption Keys</a> in the <i>Amazon Simple Storage Service Developer Guide.</i>  The KMS key policy must grant permission to the IAM role that you specify in your <code>CreateLabelingJob</code> request. For more information, see <a href="http://docs.aws.amazon.com/kms/latest/developerguide/key-policies.html">Using Key Policies in AWS KMS</a> in the <i>AWS Key Management Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3OutputPath</i> The Amazon S3 location to write output data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxPercentageOfInputDatasetLabeled</i> The maximum number of input data objects that should be labeled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxHumanLabeledObjectCount</i> The maximum number of objects that can be labeled by human workers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LabelingJobResourceConfig</i> Provides configuration information for a labeling job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LabelingJobResourceConfig</b>: Provides configuration information for a labeling job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InitialActiveLearningModelArn</i> At the end of an auto-label job Amazon SageMaker Ground Truth sends the Amazon Resource Nam (ARN) of the final model used for auto-labeling. You can use this model as the starting point for subsequent similar jobs by providing the ARN of the model here. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LabelingJobAlgorithmSpecificationArn</i> Specifies the Amazon Resource Name (ARN) of the algorithm used for auto-labeling. You must select one of the following ARNs: <ul> <li>  <i>Image classification</i>   <code>arn:aws:sagemaker:<i>region</i>:027400017018:labeling-job-algorithm-specification/image-classification</code>  </li> <li>  <i>Text classification</i>   <code>arn:aws:sagemaker:<i>region</i>:027400017018:labeling-job-algorithm-specification/text-classification</code>  </li> <li>  <i>Object detection</i>   <code>arn:aws:sagemaker:<i>region</i>:027400017018:labeling-job-algorithm-specification/object-detection</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeKmsKeyId</i> The AWS Key Management Service key ID for the key used to encrypt the output data, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PublicWorkforceTaskPrice</i> The price that you pay for each task performed by a public worker.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PublicWorkforceTaskPrice</b>: The price that you pay for each task performed by a public worker.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AnnotationConsolidationConfig</i> Configures how labels are consolidated across human workers.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AnnotationConsolidationConfig</b>: Configures how labels are consolidated across human workers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxConcurrentTaskCount</i> Defines the maximum number of data objects that can be labeled by human workers at the same time. Each object may have more than one worker at one time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TaskAvailabilityLifetimeInSeconds</i> The length of time that a task remains available for labelling by human workers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TaskTimeLimitInSeconds</i> The amount of time that a worker has to complete a task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfHumanWorkersPerDataObject</i> The number of human workers that will label an object. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TaskDescription</i> A description of the task for your human workers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TaskTitle</i> A title for the task for your human workers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TaskKeywords</i> Keywords used to describe the task so that workers on Amazon Mechanical Turk can discover the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreHumanTaskLambdaArn</i> The Amazon Resource Name (ARN) of a Lambda function that is run before a data object is sent to a human worker. Use this function to provide input to a custom labeling job. For the built-in bounding box, image classification, semantic segmentation, and text classification task types, Amazon SageMaker Ground Truth provides the following Lambda functions:  <b>US East (Northern Virginia) (us-east-1):</b>  <ul> <li>  <code>arn:aws:lambda:us-east-1:432418664414:function:PRE-BoundingBox</code>  </li> <li>  <code>arn:aws:lambda:us-east-1:432418664414:function:PRE-ImageMultiClass</code>  </li> <li>  <code>arn:aws:lambda:us-east-1:432418664414:function:PRE-SemanticSegmentation</code>  </li> <li>  <code>arn:aws:lambda:us-east-1:432418664414:function:PRE-TextMultiClass</code>  </li> </ul>  <b>US East (Ohio) (us-east-2):</b>  <ul> <li>  <code>arn:aws:lambda:us-east-2:266458841044:function:PRE-BoundingBox</code>  </li> <li>  <code>arn:aws:lambda:us-east-2:266458841044:function:PRE-ImageMultiClass</code>  </li> <li>  <code>arn:aws:lambda:us-east-2:266458841044:function:PRE-SemanticSegmentation</code>  </li> <li>  <code>arn:aws:lambda:us-east-2:266458841044:function:PRE-TextMultiClass</code>  </li> </ul>  <b>US West (Oregon) (us-west-2):</b>  <ul> <li>  <code>arn:aws:lambda:us-west-2:081040173940:function:PRE-BoundingBox</code>  </li> <li>  <code>arn:aws:lambda:us-west-2:081040173940:function:PRE-ImageMultiClass</code>  </li> <li>  <code>arn:aws:lambda:us-west-2:081040173940:function:PRE-SemanticSegmentation</code>  </li> <li>  <code>arn:aws:lambda:us-west-2:081040173940:function:PRE-TextMultiClass</code>  </li> </ul>  <b>EU (Ireland) (eu-west-1):</b>  <ul> <li>  <code>arn:aws:lambda:eu-west-1:568282634449:function:PRE-BoundingBox</code>  </li> <li>  <code>arn:aws:lambda:eu-west-1:568282634449:function:PRE-ImageMultiClass</code>  </li> <li>  <code>arn:aws:lambda:eu-west-1:568282634449:function:PRE-SemanticSegmentation</code>  </li> <li>  <code>arn:aws:lambda:eu-west-1:568282634449:function:PRE-TextMultiClass</code>  </li> </ul>  <b>Asia Pacific (Tokyo) (ap-northeast-1):</b>  <ul> <li>  <code>arn:aws:lambda:ap-northeast-1:477331159723:function:PRE-BoundingBox</code>  </li> <li>  <code>arn:aws:lambda:ap-northeast-1:477331159723:function:PRE-ImageMultiClass</code>  </li> <li>  <code>arn:aws:lambda:ap-northeast-1:477331159723:function:PRE-SemanticSegmentation</code>  </li> <li>  <code>arn:aws:lambda:ap-northeast-1:477331159723:function:PRE-TextMultiClass</code>  </li> </ul>  <b>Asia Pacific (Sydney) (ap-southeast-1):</b>  <ul> <li>  <code>arn:aws:lambda:ap-southeast-2:454466003867:function:PRE-BoundingBox</code>  </li> <li>  <code>arn:aws:lambda:ap-southeast-2:454466003867:function:PRE-ImageMultiClass</code>  </li> <li>  <code>arn:aws:lambda:ap-southeast-2:454466003867:function:PRE-SemanticSegmentation</code>  </li> <li>  <code>arn:aws:lambda:ap-southeast-2:454466003867:function:PRE-TextMultiClass</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UiConfig</i> Information about the user interface that workers use to complete the labeling task.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>UiConfig</b>: Information about the user interface that workers use to complete the labeling task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WorkteamArn</i> The Amazon Resource Name (ARN) of the work team assigned to complete the tasks.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UiTemplateS3Uri</i> The Amazon S3 bucket location of the UI template. For more information about the contents of a UI template, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/sms-custom-templates-step2.html"> Creating Your Custom Labeling Task Template</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AnnotationConsolidationLambdaArn</i> The Amazon Resource Name (ARN) of a Lambda function implements the logic for annotation consolidation. For the built-in bounding box, image classification, semantic segmentation, and text classification task types, Amazon SageMaker Ground Truth provides the following Lambda functions: <ul> <li>  <i>Bounding box</i> - Finds the most similar boxes from different workers based on the Jaccard index of the boxes.  <code>arn:aws:lambda:us-east-1:432418664414:function:ACS-BoundingBox</code>   <code>arn:aws:lambda:us-east-2:266458841044:function:ACS-BoundingBox</code>   <code>arn:aws:lambda:us-west-2:081040173940:function:ACS-BoundingBox</code>   <code>arn:aws:lambda:eu-west-1:568282634449:function:ACS-BoundingBox</code>   <code>arn:aws:lambda:ap-northeast-1:477331159723:function:ACS-BoundingBox</code>   <code>arn:aws:lambda:ap-southeast-2:454466003867:function:ACS-BoundingBox</code>  </li> <li>  <i>Image classification</i> - Uses a variant of the Expectation Maximization approach to estimate the true class of an image based on annotations from individual workers.  <code>arn:aws:lambda:us-east-1:432418664414:function:ACS-ImageMultiClass</code>   <code>arn:aws:lambda:us-east-2:266458841044:function:ACS-ImageMultiClass</code>   <code>arn:aws:lambda:us-west-2:081040173940:function:ACS-ImageMultiClass</code>   <code>arn:aws:lambda:eu-west-1:568282634449:function:ACS-ImageMultiClass</code>   <code>arn:aws:lambda:ap-northeast-1:477331159723:function:ACS-ImageMultiClass</code>   <code>arn:aws:lambda:ap-southeast-2:454466003867:function:ACS-ImageMultiClass</code>  </li> <li>  <i>Semantic segmentation</i> - Treats each pixel in an image as a multi-class classification and treats pixel annotations from workers as "votes" for the correct label.  <code>arn:aws:lambda:us-east-1:432418664414:function:ACS-SemanticSegmentation</code>   <code>arn:aws:lambda:us-east-2:266458841044:function:ACS-SemanticSegmentation</code>   <code>arn:aws:lambda:us-west-2:081040173940:function:ACS-SemanticSegmentation</code>   <code>arn:aws:lambda:eu-west-1:568282634449:function:ACS-SemanticSegmentation</code>   <code>arn:aws:lambda:ap-northeast-1:477331159723:function:ACS-SemanticSegmentation</code>   <code>arn:aws:lambda:ap-southeast-2:454466003867:function:ACS-SemanticSegmentation</code>  </li> <li>  <i>Text classification</i> - Uses a variant of the Expectation Maximization approach to estimate the true class of text based on annotations from individual workers.  <code>arn:aws:lambda:us-east-1:432418664414:function:ACS-TextMultiClass</code>   <code>arn:aws:lambda:us-east-2:266458841044:function:ACS-TextMultiClass</code>   <code>arn:aws:lambda:us-west-2:081040173940:function:ACS-TextMultiClass</code>   <code>arn:aws:lambda:eu-west-1:568282634449:function:ACS-TextMultiClass</code>   <code>arn:aws:lambda:ap-northeast-1:477331159723:function:ACS-TextMultiClass</code>   <code>arn:aws:lambda:ap-southeast-2:454466003867:function:ACS-TextMultiClass</code>  </li> </ul> For more information, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/sms-annotation-consolidation.html">Annotation Consolidation</a>.<br/>Defines the amount of money paid to a worker in United States dollars.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AmountInUsd</b>: Defines the amount of money paid to a worker in United States dollars.
        :return: LabelingJobArn:  The Amazon Resource Name (ARN) of the labeling job. You use this ARN to identify the labeling job.<br/>        """
        pass

    def create_model(self, ExecutionRoleArn: str, ModelName: str, PrimaryContainer: ContainerDefinition=None, Containers: List[ContainerDefinition]=None, Tags: List[Tag]=None, VpcConfig: VpcConfig=None, EnableNetworkIsolation: Boolean=None) -> CreateModelOutput:
        """Creates a model in Amazon SageMaker. In the request, you name the model and describe a primary container. For the primary container, you specify the docker image containing inference code, artifacts (from prior training), and custom environment map that the inference code uses when you deploy the model for predictions. Use this API to create a model if you want to use Amazon SageMaker hosting services or run a batch transform job. To host your model, you create an endpoint configuration with the <code>CreateEndpointConfig</code> API, and then create an endpoint with the <code>CreateEndpoint</code> API. Amazon SageMaker then deploys all of the containers that you defined for the model in the hosting environment.  To run a batch transform using your model, you start a job with the <code>CreateTransformJob</code> API. Amazon SageMaker uses your model and your dataset to get inferences which are then saved to a specified S3 location. In the <code>CreateModel</code> request, you must define a container with the <code>PrimaryContainer</code> parameter. In the request, you also provide an IAM role that Amazon SageMaker can assume to access model artifacts and docker image for deployment on ML compute hosting instances or for batch transform jobs. In addition, you also use the IAM role to manage permissions the inference code needs. For example, if the inference code access any other AWS resources, you grant necessary permissions via this role.

        :param self:
        :param EnableNetworkIsolation:  Isolates the model container. No inbound or outbound network calls can be made to or from the model container. <note> The Semantic Segmentation built-in algorithm does not support network isolation. </note><br/>VpcConfig:  A <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/API_VpcConfig.html">VpcConfig</a> object that specifies the VPC that you want your model to connect to. Control access to and from your model container by configuring the VPC. <code>VpcConfig</code> is used in hosting services and in batch transform. For more information, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/host-vpc.html">Protect Endpoints by Using an Amazon Virtual Private Cloud</a> and <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/batch-vpc.html">Protect Data in Batch Transform Jobs by Using an Amazon Virtual Private Cloud</a>.:<br/>Tags:  An array of key-value pairs. For more information, see <a href="https://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/cost-alloc-tags.html#allocation-what">Using Cost Allocation Tags</a> in the <i>AWS Billing and Cost Management User Guide</i>. <br/>ExecutionRoleArn:  The Amazon Resource Name (ARN) of the IAM role that Amazon SageMaker can assume to access model artifacts and docker image for deployment on ML compute instances or for batch transform jobs. Deploying on ML compute instances is part of model hosting. For more information, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/sagemaker-roles.html">Amazon SageMaker Roles</a>.  <note> To be able to pass this role to Amazon SageMaker, the caller of this API must have the <code>iam:PassRole</code> permission. </note><br/>Containers:  Specifies the containers in the inference pipeline.<br/>PrimaryContainer:  The location of the primary docker image containing inference code, associated artifacts, and custom environment map that the inference code uses when the model is deployed for predictions. :<br/>ModelName:  The name of the new model.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModelPackageName</i> The name or Amazon Resource Name (ARN) of the model package to use to create the model.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Environment</i> The environment variables to set in the Docker container. Each key and value in the <code>Environment</code> string to string map can have length of up to 1024. We support up to 16 entries in the map. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModelDataUrl</i> The S3 path where the model artifacts, which result from model training, are stored. This path must point to a single gzip compressed tar archive (.tar.gz suffix). The S3 path is required for Amazon SageMaker built-in algorithms, but not if you use your own algorithms. For more information on built-in algorithms, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/sagemaker-algo-docker-registry-paths.html">Common Parameters</a>.  If you provide a value for this parameter, Amazon SageMaker uses AWS Security Token Service to download model artifacts from the S3 path you provide. AWS STS is activated in your IAM user account by default. If you previously deactivated AWS STS for a region, you need to reactivate AWS STS for that region. For more information, see <a href="http://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_temp_enable-regions.html">Activating and Deactivating AWS STS in an AWS Region</a> in the <i>AWS Identity and Access Management User Guide</i>. <important> If you use a built-in algorithm to create a model, Amazon SageMaker requires that you provide a S3 path to the model artifacts in <code>ModelDataUrl</code>. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Image</i> The Amazon EC2 Container Registry (Amazon ECR) path where inference code is stored. If you are using your own custom algorithm instead of an algorithm provided by Amazon SageMaker, the inference code must meet Amazon SageMaker requirements. Amazon SageMaker supports both <code>registry/repository[:tag]</code> and <code>registry/repository[@digest]</code> image path formats. For more information, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/your-algorithms.html">Using Your Own Algorithms with Amazon SageMaker</a> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContainerHostname</i> This parameter is ignored for models that contain only a <code>PrimaryContainer</code>. When a <code>ContainerDefinition</code> is part of an inference pipeline, the value of ths parameter uniquely identifies the container for the purposes of logging and metrics. For information, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/inference-pipeline-logs-metrics.html">Use Logs and Metrics to Monitor an Inference Pipeline</a>. If you don't specify a value for this parameter for a <code>ContainerDefinition</code> that is part of an inference pipeline, a unique name is automatically assigned based on the position of the <code>ContainerDefinition</code> in the pipeline. If you specify a value for the <code>ContainerHostName</code> for any <code>ContainerDefinition</code> that is part of an inference pipeline, you must specify a value for the <code>ContainerHostName</code> parameter of every <code>ContainerDefinition</code> in that pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> The ID of the subnets in the VPC to which you want to connect your training job or model.  <note> Amazon EC2 P3 accelerated computing instances are not available in the c/d/e availability zones of region us-east-1. If you want to create endpoints with P3 instances in VPC mode in region us-east-1, create subnets in a/b/f availability zones instead. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The VPC security group IDs, in the form sg-xxxxxxxx. Specify the security groups for the VPC that is specified in the <code>Subnets</code> field.<br/>
        :return: ModelArn:  The ARN of the model created in Amazon SageMaker.<br/>        """
        pass

    def create_model_package(self, ModelPackageName: str, ModelPackageDescription: str=None, InferenceSpecification: InferenceSpecification=None, ValidationSpecification: ModelPackageValidationSpecification=None, SourceAlgorithmSpecification: SourceAlgorithmSpecification=None, CertifyForMarketplace: CertifyForMarketplace=None) -> CreateModelPackageOutput:
        """Creates a model package that you can use to create Amazon SageMaker models or list on AWS Marketplace. Buyers can subscribe to model packages listed on AWS Marketplace to create models in Amazon SageMaker. To create a model package by specifying a Docker container that contains your inference code and the Amazon S3 location of your model artifacts, provide values for <code>InferenceSpecification</code>. To create a model from an algorithm resource that you created or subscribed to in AWS Marketplace, provide a value for <code>SourceAlgorithmSpecification</code>.

        :param self:
        :param CertifyForMarketplace:  Whether to certify the model package for listing on AWS Marketplace.<br/>SourceAlgorithmSpecification:  Details about the algorithm that was used to create the model package.:<br/>ValidationSpecification:  Specifies configurations for one or more transform jobs that Amazon SageMaker runs to test the model package.:<br/>InferenceSpecification:  Specifies details about inference jobs that can be run with models based on this model package, including the following: <ul> <li> The Amazon ECR paths of containers that contain the inference code and model artifacts. </li> <li> The instance types that the model package supports for transform jobs and real-time endpoints used for inference. </li> <li> The input and output content formats that the model package supports for inference. </li> </ul>:<br/>ModelPackageDescription:  A description of the model package.<br/>ModelPackageName:  The name of the model package. The name must have 1 to 63 characters. Valid characters are a-z, A-Z, 0-9, and - (hyphen).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedResponseMIMETypes</i> The supported MIME types for the output data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedContentTypes</i> The supported MIME types for the input data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedRealtimeInferenceInstanceTypes</i> A list of the instance types that are used to generate inferences in real-time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedTransformInstanceTypes</i> A list of the instance types on which a transformation job can be run or on which an endpoint can be deployed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Containers</i> The Amazon ECR registry path of the Docker image that contains the inference code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ValidationProfiles</i> An array of <code>ModelPackageValidationProfile</code> objects, each of which specifies a batch transform job that Amazon SageMaker runs to validate your model package.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ValidationRole</i> The IAM roles to be used for the validation of the model package.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceAlgorithms</i> A list of the algorithms that were used to create a model package.<br/>
        :return: ModelPackageArn:  The Amazon Resource Name (ARN) of the new model package.<br/>        """
        pass

    def create_notebook_instance(self, RoleArn: str, InstanceType: str, NotebookInstanceName: str, SubnetId: str=None, SecurityGroupIds: List[SecurityGroupId]=None, KmsKeyId: str=None, Tags: List[Tag]=None, LifecycleConfigName: str=None, DirectInternetAccess: str=None, VolumeSizeInGB: int=None, AcceleratorTypes: List[NotebookInstanceAcceleratorType]=None, DefaultCodeRepository: str=None, AdditionalCodeRepositories: List[CodeRepositoryNameOrUrl]=None, RootAccess: str=None) -> CreateNotebookInstanceOutput:
        """Creates an Amazon SageMaker notebook instance. A notebook instance is a machine learning (ML) compute instance running on a Jupyter notebook.  In a <code>CreateNotebookInstance</code> request, specify the type of ML compute instance that you want to run. Amazon SageMaker launches the instance, installs common libraries that you can use to explore datasets for model training, and attaches an ML storage volume to the notebook instance.  Amazon SageMaker also provides a set of example notebooks. Each notebook demonstrates how to use Amazon SageMaker with a specific algorithm or with a machine learning framework.  After receiving the request, Amazon SageMaker does the following: <ol> <li> Creates a network interface in the Amazon SageMaker VPC. </li> <li> (Option) If you specified <code>SubnetId</code>, Amazon SageMaker creates a network interface in your own VPC, which is inferred from the subnet ID that you provide in the input. When creating this network interface, Amazon SageMaker attaches the security group that you specified in the request to the network interface that it creates in your VPC. </li> <li> Launches an EC2 instance of the type specified in the request in the Amazon SageMaker VPC. If you specified <code>SubnetId</code> of your VPC, Amazon SageMaker specifies both network interfaces when launching this instance. This enables inbound traffic from your own VPC to the notebook instance, assuming that the security groups allow it. </li> </ol> After creating the notebook instance, Amazon SageMaker returns its Amazon Resource Name (ARN). After Amazon SageMaker creates the notebook instance, you can connect to the Jupyter server and work in Jupyter notebooks. For example, you can write code to explore a dataset that you can use for model training, train a model, host models by creating Amazon SageMaker endpoints, and validate hosted models.  For more information, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/how-it-works.html">How It Works</a>. 

        :param self:
        :param RootAccess:  Whether root access is enabled or disabled for users of the notebook instance. The default value is <code>Enabled</code>. <note> Lifecycle configurations need root access to be able to set up a notebook instance. Because of this, lifecycle configurations associated with a notebook instance always run with root access even if you disable root access for users. </note><br/>AdditionalCodeRepositories:  An array of up to three Git repositories to associate with the notebook instance. These can be either the names of Git repositories stored as resources in your account, or the URL of Git repositories in <a href="http://docs.aws.amazon.com/codecommit/latest/userguide/welcome.html">AWS CodeCommit</a> or in any other Git repository. These repositories are cloned at the same level as the default repository of your notebook instance. For more information, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/nbi-git-repo.html">Associating Git Repositories with Amazon SageMaker Notebook Instances</a>.<br/>DefaultCodeRepository:  A Git repository to associate with the notebook instance as its default code repository. This can be either the name of a Git repository stored as a resource in your account, or the URL of a Git repository in <a href="http://docs.aws.amazon.com/codecommit/latest/userguide/welcome.html">AWS CodeCommit</a> or in any other Git repository. When you open a notebook instance, it opens in the directory that contains this repository. For more information, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/nbi-git-repo.html">Associating Git Repositories with Amazon SageMaker Notebook Instances</a>.<br/>AcceleratorTypes:  A list of Elastic Inference (EI) instance types to associate with this notebook instance. Currently, only one instance type can be associated with a notebook instance. For more information, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/ei.html">Using Elastic Inference in Amazon SageMaker</a>.<br/>VolumeSizeInGB:  The size, in GB, of the ML storage volume to attach to the notebook instance. The default value is 5 GB.<br/>DirectInternetAccess:  Sets whether Amazon SageMaker provides internet access to the notebook instance. If you set this to <code>Disabled</code> this notebook instance will be able to access resources only in your VPC, and will not be able to connect to Amazon SageMaker training and endpoint services unless your configure a NAT Gateway in your VPC. For more information, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/appendix-additional-considerations.html#appendix-notebook-and-internet-access">Notebook Instances Are Internet-Enabled by Default</a>. You can set the value of this parameter to <code>Disabled</code> only if you set a value for the <code>SubnetId</code> parameter.<br/>LifecycleConfigName:  The name of a lifecycle configuration to associate with the notebook instance. For information about lifestyle configurations, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/notebook-lifecycle-config.html">Step 2.1: (Optional) Customize a Notebook Instance</a>.<br/>Tags:  A list of tags to associate with the notebook instance. You can add tags later by using the <code>CreateTags</code> API.<br/>KmsKeyId:  The Amazon Resource Name (ARN) of a AWS Key Management Service key that Amazon SageMaker uses to encrypt data on the storage volume attached to your notebook instance. The KMS key you provide must be enabled. For information, see <a href="http://docs.aws.amazon.com/kms/latest/developerguide/enabling-keys.html">Enabling and Disabling Keys</a> in the <i>AWS Key Management Service Developer Guide</i>.<br/>RoleArn:   When you send any requests to AWS resources from the notebook instance, Amazon SageMaker assumes this role to perform tasks on your behalf. You must grant this role necessary permissions so Amazon SageMaker can perform these tasks. The policy must allow the Amazon SageMaker service principal (sagemaker.amazonaws.com) permissions to assume this role. For more information, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/sagemaker-roles.html">Amazon SageMaker Roles</a>.  <note> To be able to pass this role to Amazon SageMaker, the caller of this API must have the <code>iam:PassRole</code> permission. </note><br/>SecurityGroupIds:  The VPC security group IDs, in the form sg-xxxxxxxx. The security groups must be for the same VPC as specified in the subnet. <br/>SubnetId:  The ID of the subnet in a VPC to which you would like to have a connectivity from your ML compute instance. <br/>InstanceType:  The type of ML compute instance to launch for the notebook instance.<br/>NotebookInstanceName:  The name of the new notebook instance.<br/>
        :return: NotebookInstanceArn:  The Amazon Resource Name (ARN) of the notebook instance. <br/>        """
        pass

    def create_notebook_instance_lifecycle_config(self, NotebookInstanceLifecycleConfigName: str, OnCreate: List[NotebookInstanceLifecycleHook]=None, OnStart: List[NotebookInstanceLifecycleHook]=None) -> CreateNotebookInstanceLifecycleConfigOutput:
        """Creates a lifecycle configuration that you can associate with a notebook instance. A <i>lifecycle configuration</i> is a collection of shell scripts that run when you create or start a notebook instance. Each lifecycle configuration script has a limit of 16384 characters. The value of the <code>$PATH</code> environment variable that is available to both scripts is <code>/sbin:bin:/usr/sbin:/usr/bin</code>. View CloudWatch Logs for notebook instance lifecycle configurations in log group <code>/aws/sagemaker/NotebookInstances</code> in log stream <code>[notebook-instance-name]/[LifecycleConfigHook]</code>. Lifecycle configuration scripts cannot run for longer than 5 minutes. If a script runs for longer than 5 minutes, it fails and the notebook instance is not created or started. For information about notebook instance lifestyle configurations, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/notebook-lifecycle-config.html">Step 2.1: (Optional) Customize a Notebook Instance</a>.

        :param self:
        :param OnStart:  A shell script that runs every time you start a notebook instance, including when you create the notebook instance. The shell script must be a base64-encoded string.<br/>OnCreate:  A shell script that runs only once, when you create a notebook instance. The shell script must be a base64-encoded string.<br/>NotebookInstanceLifecycleConfigName:  The name of the lifecycle configuration.<br/>
        :return: NotebookInstanceLifecycleConfigArn:  The Amazon Resource Name (ARN) of the lifecycle configuration.<br/>        """
        pass

    def create_presigned_notebook_instance_url(self, NotebookInstanceName: str, SessionExpirationDurationInSeconds: int=None) -> CreatePresignedNotebookInstanceUrlOutput:
        """Returns a URL that you can use to connect to the Jupyter server from a notebook instance. In the Amazon SageMaker console, when you choose <code>Open</code> next to a notebook instance, Amazon SageMaker opens a new tab showing the Jupyter server home page from the notebook instance. The console uses this API to get the URL and show the page. IAM authorization policies for this API are also enforced for every HTTP request and WebSocket frame that attempts to connect to the notebook instance.For example, you can restrict access to this API and to the URL that it returns to a list of IP addresses that you specify. Use the <code>NotIpAddress</code> condition operator and the <code>aws:SourceIP</code> condition context key to specify the list of IP addresses that you want to have access to the notebook instance. For more information, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/nbi-ip-filter.html">Limit Access to a Notebook Instance by IP Address</a>. <note> The URL that you get from a call to is valid only for 5 minutes. If you try to use the URL after the 5-minute limit expires, you are directed to the AWS console sign-in page. </note>

        :param self:
        :param SessionExpirationDurationInSeconds:  The duration of the session, in seconds. The default is 12 hours.<br/>NotebookInstanceName:  The name of the notebook instance.<br/>
        :return: AuthorizedUrl:  A JSON object that contains the URL string. <br/>        """
        pass

    def create_training_job(self, StoppingCondition: StoppingCondition, ResourceConfig: ResourceConfig, OutputDataConfig: OutputDataConfig, RoleArn: str, AlgorithmSpecification: AlgorithmSpecification, TrainingJobName: str, HyperParameters: HyperParameters=None, InputDataConfig: List[Channel]=None, VpcConfig: VpcConfig=None, Tags: List[Tag]=None, EnableNetworkIsolation: Boolean=None, EnableInterContainerTrafficEncryption: Boolean=None) -> CreateTrainingJobResponse:
        """Starts a model training job. After training completes, Amazon SageMaker saves the resulting model artifacts to an Amazon S3 location that you specify.  If you choose to host your model using Amazon SageMaker hosting services, you can use the resulting model artifacts as part of the model. You can also use the artifacts in a machine learning service other than Amazon SageMaker, provided that you know how to use them for inferences.  In the request body, you provide the following:  <ul> <li>  <code>AlgorithmSpecification</code> - Identifies the training algorithm to use.  </li> <li>  <code>HyperParameters</code> - Specify these algorithm-specific parameters to enable the estimation of model parameters during training. Hyperparameters can be tuned to optimize this learning process. For a list of hyperparameters for each training algorithm provided by Amazon SageMaker, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/algos.html">Algorithms</a>.  </li> <li>  <code>InputDataConfig</code> - Describes the training dataset and the Amazon S3 location where it is stored. </li> <li>  <code>OutputDataConfig</code> - Identifies the Amazon S3 location where you want Amazon SageMaker to save the results of model training.  <p/> </li> <li>  <code>ResourceConfig</code> - Identifies the resources, ML compute instances, and ML storage volumes to deploy for model training. In distributed training, you specify more than one instance.  </li> <li>  <code>RoleARN</code> - The Amazon Resource Number (ARN) that Amazon SageMaker assumes to perform tasks on your behalf during model training. You must grant this role the necessary permissions so that Amazon SageMaker can successfully complete model training.  </li> <li>  <code>StoppingCondition</code> - Sets a time limit for training. Use this parameter to cap model training costs.  </li> </ul>  For more information about Amazon SageMaker, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/how-it-works.html">How It Works</a>. 

        :param self:
        :param EnableInterContainerTrafficEncryption:  To encrypt all communications between ML compute instances in distributed training, choose <code>True</code>. Encryption provides greater security for distributed training, but training might take longer. How long it takes depends on the amount of communication between compute instances, especially if you use a deep learning algorithm in distributed training. For more information, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/train-encrypt.html">Protect Communications Between ML Compute Instances in a Distributed Training Job</a>.<br/>EnableNetworkIsolation:  Isolates the training container. No inbound or outbound network calls can be made, except for calls between peers within a training cluster for distributed training. If you enable network isolation for training jobs that are configured to use a VPC, Amazon SageMaker downloads and uploads customer data and model artifacts through the specified VPC, but the training container does not have network access. <note> The Semantic Segmentation built-in algorithm does not support network isolation. </note><br/>Tags:  An array of key-value pairs. For more information, see <a href="https://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/cost-alloc-tags.html#allocation-what">Using Cost Allocation Tags</a> in the <i>AWS Billing and Cost Management User Guide</i>. <br/>StoppingCondition:  Specifies a limit to how long a model training job can run. When the job reaches the time limit, Amazon SageMaker ends the training job. Use this API to cap model training costs. To stop a job, Amazon SageMaker sends the algorithm the <code>SIGTERM</code> signal, which delays job termination for 120 seconds. Algorithms can use this 120-second window to save the model artifacts, so the results of training are not lost. :<br/>VpcConfig:  A <a>VpcConfig</a> object that specifies the VPC that you want your training job to connect to. Control access to and from your training container by configuring the VPC. For more information, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/train-vpc.html">Protect Training Jobs by Using an Amazon Virtual Private Cloud</a>.:<br/>ResourceConfig:  The resources, including the ML compute instances and ML storage volumes, to use for model training.  ML storage volumes store model artifacts and incremental states. Training algorithms might also use ML storage volumes for scratch space. If you want Amazon SageMaker to use the ML storage volume to store the training data, choose <code>File</code> as the <code>TrainingInputMode</code> in the algorithm specification. For distributed training algorithms, specify an instance count greater than 1.:<br/>OutputDataConfig:  Specifies the path to the S3 bucket where you want to store model artifacts. Amazon SageMaker creates subfolders for the artifacts. :<br/>InputDataConfig:  An array of <code>Channel</code> objects. Each channel is a named input source. <code>InputDataConfig</code> describes the input data and its location.  Algorithms can accept input data from one or more channels. For example, an algorithm might have two channels of input data, <code>training_data</code> and <code>validation_data</code>. The configuration for each channel provides the S3 location where the input data is stored. It also provides information about the stored data: the MIME type, compression method, and whether the data is wrapped in RecordIO format.  Depending on the input mode that the algorithm supports, Amazon SageMaker either copies input data files from an S3 bucket to a local directory in the Docker container, or makes it available as input streams. <br/>RoleArn:  The Amazon Resource Name (ARN) of an IAM role that Amazon SageMaker can assume to perform tasks on your behalf.  During model training, Amazon SageMaker needs your permission to read input data from an S3 bucket, download a Docker image that contains training code, write model artifacts to an S3 bucket, write logs to Amazon CloudWatch Logs, and publish metrics to Amazon CloudWatch. You grant permissions for all of these tasks to an IAM role. For more information, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/sagemaker-roles.html">Amazon SageMaker Roles</a>.  <note> To be able to pass this role to Amazon SageMaker, the caller of this API must have the <code>iam:PassRole</code> permission. </note><br/>AlgorithmSpecification:  The registry path of the Docker image that contains the training algorithm and algorithm-specific metadata, including the input mode. For more information about algorithms provided by Amazon SageMaker, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/algos.html">Algorithms</a>. For information about providing your own algorithms, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/your-algorithms.html">Using Your Own Algorithms with Amazon SageMaker</a>. :<br/>HyperParameters:  Algorithm-specific parameters that influence the quality of the model. You set hyperparameters before you start the learning process. For a list of hyperparameters for each training algorithm provided by Amazon SageMaker, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/algos.html">Algorithms</a>.  You can specify a maximum of 100 hyperparameters. Each hyperparameter is a key-value pair. Each key and value is limited to 256 characters, as specified by the <code>Length Constraint</code>. <br/>TrainingJobName:  The name of the training job. The name must be unique within an AWS Region in an AWS account. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricDefinitions</i> A list of metric definition objects. Each object specifies the metric name and regular expressions used to parse algorithm logs. Amazon SageMaker publishes each metric to Amazon CloudWatch.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingInputMode</i> The input mode that the algorithm supports. For the input modes that Amazon SageMaker algorithms support, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/algos.html">Algorithms</a>. If an algorithm supports the <code>File</code> input mode, Amazon SageMaker downloads the training data from S3 to the provisioned ML storage Volume, and mounts the directory to docker volume for training container. If an algorithm supports the <code>Pipe</code> input mode, Amazon SageMaker streams data directly from S3 to the container.   In File mode, make sure you provision ML storage volume with sufficient capacity to accommodate the data download from S3. In addition to the training data, the ML storage volume also stores the output model. The algorithm container use ML storage volume to also store intermediate information, if any.   For distributed algorithms using File mode, training data is distributed uniformly, and your training duration is predictable if the input data objects size is approximately same. Amazon SageMaker does not split the files any further for model training. If the object sizes are skewed, training won't be optimal as the data distribution is also skewed where one host in a training cluster is overloaded, thus becoming bottleneck in training. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AlgorithmName</i> The name of the algorithm resource to use for the training job. This must be an algorithm resource that you created or subscribe to on AWS Marketplace. If you specify a value for this parameter, you can't specify a value for <code>TrainingImage</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingImage</i> The registry path of the Docker image that contains the training algorithm. For information about docker registry paths for built-in algorithms, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/sagemaker-algo-docker-registry-paths.html">Algorithms Provided by Amazon SageMaker: Common Parameters</a>. Amazon SageMaker supports both <code>registry/repository[:tag]</code> and <code>registry/repository[@digest]</code> image path formats. For more information, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/your-algorithms.html">Using Your Own Algorithms with Amazon SageMaker</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3OutputPath</i> Identifies the S3 path where you want Amazon SageMaker to store the model artifacts. For example, <code>s3://bucket-name/key-name-prefix</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS Key Management Service (AWS KMS) key that Amazon SageMaker uses to encrypt the model artifacts at rest using Amazon S3 server-side encryption. The <code>KmsKeyId</code> can be any of the following formats:  <ul> <li> // KMS Key ID  <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> // Amazon Resource Name (ARN) of a KMS Key  <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> // KMS Key Alias  <code>"alias/ExampleAlias"</code>  </li> <li> // Amazon Resource Name (ARN) of a KMS Key Alias  <code>"arn:aws:kms:us-west-2:111122223333:alias/ExampleAlias"</code>  </li> </ul> If you use a KMS key ID or an alias of your master key, the Amazon SageMaker execution role must include permissions to call <code>kms:Encrypt</code>. If you don't provide a KMS key ID, Amazon SageMaker uses the default KMS key for Amazon S3 for your role's account. Amazon SageMaker uses server-side encryption with KMS-managed keys for <code>OutputDataConfig</code>. If you use a bucket policy with an <code>s3:PutObject</code> permission that only allows objects with server-side encryption, set the condition key of <code>s3:x-amz-server-side-encryption</code> to <code>"aws:kms"</code>. For more information, see <a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/UsingKMSEncryption.html">KMS-Managed Encryption Keys</a> in the <i>Amazon Simple Storage Service Developer Guide.</i>  The KMS key policy must grant permission to the IAM role that you specify in your <code>CreateTrainingJob</code>, <code>CreateTransformJob</code>, or <code>CreateHyperParameterTuningJob</code> requests. For more information, see <a href="http://docs.aws.amazon.com/kms/latest/developerguide/key-policies.html">Using Key Policies in AWS KMS</a> in the <i>AWS Key Management Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeKmsKeyId</i> The AWS Key Management Service (AWS KMS) key that Amazon SageMaker uses to encrypt data on the storage volume attached to the ML compute instance(s) that run the training job. The <code>VolumeKmsKeyId</code> can be any of the following formats: <ul> <li> // KMS Key ID  <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> // Amazon Resource Name (ARN) of a KMS Key  <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeSizeInGB</i> The size of the ML storage volume that you want to provision.  ML storage volumes store model artifacts and incremental states. Training algorithms might also use the ML storage volume for scratch space. If you want to store the training data in the ML storage volume, choose <code>File</code> as the <code>TrainingInputMode</code> in the algorithm specification.  You must specify sufficient ML storage for your scenario.  <note>  Amazon SageMaker supports only the General Purpose SSD (gp2) ML storage volume type.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCount</i> The number of ML compute instances to use. For distributed training, provide a value greater than 1. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i> The ML compute instance type. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> The ID of the subnets in the VPC to which you want to connect your training job or model.  <note> Amazon EC2 P3 accelerated computing instances are not available in the c/d/e availability zones of region us-east-1. If you want to create endpoints with P3 instances in VPC mode in region us-east-1, create subnets in a/b/f availability zones instead. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The VPC security group IDs, in the form sg-xxxxxxxx. Specify the security groups for the VPC that is specified in the <code>Subnets</code> field.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxRuntimeInSeconds</i> The maximum length of time, in seconds, that the training or compilation job can run. If job does not complete during this time, Amazon SageMaker ends the job. If value is not specified, default value is 1 day. The maximum value is 28 days.<br/>
        :return: TrainingJobArn:  The Amazon Resource Name (ARN) of the training job.<br/>        """
        pass

    def create_transform_job(self, TransformResources: TransformResources, TransformOutput: TransformOutput, TransformInput: TransformInput, ModelName: str, TransformJobName: str, MaxConcurrentTransforms: int=None, MaxPayloadInMB: int=None, BatchStrategy: str=None, Environment: TransformEnvironmentMap=None, DataProcessing: DataProcessing=None, Tags: List[Tag]=None) -> CreateTransformJobResponse:
        """Starts a transform job. A transform job uses a trained model to get inferences on a dataset and saves these results to an Amazon S3 location that you specify. To perform batch transformations, you create a transform job and use the data that you have readily available. In the request body, you provide the following: <ul> <li>  <code>TransformJobName</code> - Identifies the transform job. The name must be unique within an AWS Region in an AWS account. </li> <li>  <code>ModelName</code> - Identifies the model to use. <code>ModelName</code> must be the name of an existing Amazon SageMaker model in the same AWS Region and AWS account. For information on creating a model, see <a>CreateModel</a>. </li> <li>  <code>TransformInput</code> - Describes the dataset to be transformed and the Amazon S3 location where it is stored. </li> <li>  <code>TransformOutput</code> - Identifies the Amazon S3 location where you want Amazon SageMaker to save the results from the transform job. </li> <li>  <code>TransformResources</code> - Identifies the ML compute instances for the transform job. </li> </ul>  For more information about how batch transformation works Amazon SageMaker, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/batch-transform.html">How It Works</a>. 

        :param self:
        :param Tags:  (Optional) An array of key-value pairs. For more information, see <a href="https://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/cost-alloc-tags.html#allocation-what">Using Cost Allocation Tags</a> in the <i>AWS Billing and Cost Management User Guide</i>.<br/>DataProcessing:  The data structure used for combining the input data and inference in the output file. For more information, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/batch-transform-io-join.html">Batch Transform I/O Join</a>.:<br/>TransformResources:  Describes the resources, including ML instance types and ML instance count, to use for the transform job.:<br/>TransformOutput:  Describes the results of the transform job.:<br/>TransformInput:  Describes the input source and the way the transform job consumes it.:<br/>Environment:  The environment variables to set in the Docker container. We support up to 16 key and values entries in the map.<br/>BatchStrategy:  Specifies the number of records to include in a mini-batch for an HTTP inference request. A <i>record</i> <i/> is a single unit of input data that inference can be made on. For example, a single line in a CSV file is a record.  To enable the batch strategy, you must set <code>SplitType</code> to <code>Line</code>, <code>RecordIO</code>, or <code>TFRecord</code>. To use only one record when making an HTTP invocation request to a container, set <code>BatchStrategy</code> to <code>SingleRecord</code> and <code>SplitType</code> to <code>Line</code>. To fit as many records in a mini-batch as can fit within the <code>MaxPayloadInMB</code> limit, set <code>BatchStrategy</code> to <code>MultiRecord</code> and <code>SplitType</code> to <code>Line</code>.<br/>MaxPayloadInMB:  The maximum allowed size of the payload, in MB. A <i>payload</i> is the data portion of a record (without metadata). The value in <code>MaxPayloadInMB</code> must be greater than, or equal to, the size of a single record. To estimate the size of a record in MB, divide the size of your dataset by the number of records. To ensure that the records fit within the maximum payload size, we recommend using a slightly larger value. The default value is <code>6</code> MB.  For cases where the payload might be arbitrarily large and is transmitted using HTTP chunked encoding, set the value to <code>0</code>. This feature works only in supported algorithms. Currently, Amazon SageMaker built-in algorithms do not support HTTP chunked encoding.<br/>MaxConcurrentTransforms:  The maximum number of parallel requests that can be sent to each instance in a transform job. If <code>MaxConcurrentTransforms</code> is set to <code>0</code> or left unset, Amazon SageMaker checks the optional execution-parameters to determine the optimal settings for your chosen algorithm. If the execution-parameters endpoint is not enabled, the default value is <code>1</code>. For more information on execution-parameters, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/your-algorithms-batch-code.html#your-algorithms-batch-code-how-containe-serves-requests">How Containers Serve Requests</a>. For built-in algorithms, you don't need to set a value for <code>MaxConcurrentTransforms</code>.<br/>ModelName:  The name of the model that you want to use for the transform job. <code>ModelName</code> must be the name of an existing Amazon SageMaker model within an AWS Region in an AWS account.<br/>TransformJobName:  The name of the transform job. The name must be unique within an AWS Region in an AWS account. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SplitType</i> The method to use to split the transform job's data files into smaller batches. Splitting is necessary when the total size of each object is too large to fit in a single request. You can also use data splitting to improve performance by processing multiple concurrent mini-batches. The default value for <code>SplitType</code> is <code>None</code>, which indicates that input data files are not split, and request payloads contain the entire contents of an input object. Set the value of this parameter to <code>Line</code> to split records on a newline character boundary. <code>SplitType</code> also supports a number of record-oriented binary data formats. When splitting is enabled, the size of a mini-batch depends on the values of the <code>BatchStrategy</code> and <code>MaxPayloadInMB</code> parameters. When the value of <code>BatchStrategy</code> is <code>MultiRecord</code>, Amazon SageMaker sends the maximum number of records in each request, up to the <code>MaxPayloadInMB</code> limit. If the value of <code>BatchStrategy</code> is <code>SingleRecord</code>, Amazon SageMaker sends individual records in each request. <note> Some data formats represent a record as a binary payload wrapped with extra padding bytes. When splitting is applied to a binary data format, padding is removed if the value of <code>BatchStrategy</code> is set to <code>SingleRecord</code>. Padding is not removed if the value of <code>BatchStrategy</code> is set to <code>MultiRecord</code>. For more information about the RecordIO, see <a href="http://mxnet.io/architecture/note_data_loading.html#data-format">Data Format</a> in the MXNet documentation. For more information about the TFRecord, see <a href="https://www.tensorflow.org/guide/datasets#consuming_tfrecord_data">Consuming TFRecord data</a> in the TensorFlow documentation. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompressionType</i> If your transform data is compressed, specify the compression type. Amazon SageMaker automatically decompresses the data for the transform job accordingly. The default value is <code>None</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContentType</i> The multipurpose internet mail extension (MIME) type of the data. Amazon SageMaker uses the MIME type with each http call to transfer data to the transform job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataSource</i> Describes the location of the channel data, which is, the S3 location of the input data that the model can consume.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DataSource</b>: Describes the location of the channel data, which is, the S3 location of the input data that the model can consume.<br/>The S3 location of the data source that is associated with a channel.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3DataSource</b>: The S3 location of the data source that is associated with a channel.&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS Key Management Service (AWS KMS) key that Amazon SageMaker uses to encrypt the model artifacts at rest using Amazon S3 server-side encryption. The <code>KmsKeyId</code> can be any of the following formats:  <ul> <li> // KMS Key ID  <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> // Amazon Resource Name (ARN) of a KMS Key  <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> // KMS Key Alias  <code>"alias/ExampleAlias"</code>  </li> <li> // Amazon Resource Name (ARN) of a KMS Key Alias  <code>"arn:aws:kms:us-west-2:111122223333:alias/ExampleAlias"</code>  </li> </ul> If you don't provide a KMS key ID, Amazon SageMaker uses the default KMS key for Amazon S3 for your role's account. For more information, see <a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/UsingKMSEncryption.html">KMS-Managed Encryption Keys</a> in the <i>Amazon Simple Storage Service Developer Guide.</i>  The KMS key policy must grant permission to the IAM role that you specify in your <code>CreateTramsformJob</code> request. For more information, see <a href="http://docs.aws.amazon.com/kms/latest/developerguide/key-policies.html">Using Key Policies in AWS KMS</a> in the <i>AWS Key Management Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssembleWith</i> Defines how to assemble the results of the transform job as a single S3 object. Choose a format that is most convenient to you. To concatenate the results in binary format, specify <code>None</code>. To add a newline character at the end of every transformed record, specify <code>Line</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Accept</i> The MIME type used to specify the output data. Amazon SageMaker uses the MIME type with each http call to transfer data from the transform job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3OutputPath</i> The Amazon S3 path where you want Amazon SageMaker to store the results of the transform job. For example, <code>s3://bucket-name/key-name-prefix</code>. For every S3 object used as input for the transform job, batch transform stores the transformed data with an .<code>out</code> suffix in a corresponding subfolder in the location in the output prefix. For example, for the input data stored at <code>s3://bucket-name/input-name-prefix/dataset01/data.csv</code>, batch transform stores the transformed data at <code>s3://bucket-name/output-name-prefix/input-name-prefix/data.csv.out</code>. Batch transform doesn't upload partially processed objects. For an input S3 object that contains multiple records, it creates an .<code>out</code> file only if the transform job succeeds on the entire file. When the input contains multiple S3 objects, the batch transform job processes the listed S3 objects and uploads only the output for successfully processed objects. If any object fails in the transform job batch transform marks the job as failed to prompt investigation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeKmsKeyId</i> The AWS Key Management Service (AWS KMS) key that Amazon SageMaker uses to encrypt data on the storage volume attached to the ML compute instance(s) that run the batch transform job. The <code>VolumeKmsKeyId</code> can be any of the following formats: <ul> <li> // KMS Key ID  <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> // Amazon Resource Name (ARN) of a KMS Key  <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCount</i> The number of ML compute instances to use in the transform job. For distributed transform jobs, specify a value greater than 1. The default value is <code>1</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i> The ML compute instance type for the transform job. If you are using built-in algorithms to transform moderately sized datasets, we recommend using ml.m4.xlarge or <code>ml.m5.large</code>instance types.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JoinSource</i> Specifies the source of the data to join with the transformed data. The valid values are <code>None</code> and <code>Input</code> The default value is <code>None</code> which specifies not to join the input with the transformed data. If you want the batch transform job to join the original input data with the transformed data, set <code>JoinSource</code> to <code>Input</code>. To join input and output, the batch transform job must satisfy the <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/batch-transform-io-join.html#batch-transform-io-join-requirements">Requirements for Using Batch Transform I/O Join</a>. For JSON or JSONLines objects, such as a JSON array, Amazon SageMaker adds the transformed data to the input JSON object in an attribute called <code>SageMakerOutput</code>. The joined result for JSON must be a key-value pair object. If the input is not a key-value pair object, Amazon SageMaker creates a new JSON file. In the new JSON file, and the input data is stored under the <code>SageMakerInput</code> key and the results are stored in <code>SageMakerOutput</code>. For CSV files, Amazon SageMaker combines the transformed data with the input data at the end of the input data and stores it in the output file. The joined data has the joined input data followed by the transformed data and the output is a CSV file. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputFilter</i> A JSONPath expression used to select a portion of the joined dataset to save in the output file for a batch transform job. If you want Amazon SageMaker to store the entire input dataset in the output file, leave the default value, <code>$</code>. If you specify indexes that aren't within the dimension size of the joined dataset, you get an error. Examples: <code>"$"</code>, <code>"$[0,5:]"</code>, <code>"$.['id','SageMakerOutput']"</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputFilter</i> A JSONPath expression used to select a portion of the input data to pass to the algorithm. Use the <code>InputFilter</code> parameter to exclude fields, such as an ID column, from the input. If you want Amazon SageMaker to pass the entire input dataset to the algorithm, accept the default value <code>$</code>. Examples: <code>"$"</code>, <code>"$[1:]"</code>, <code>"$.features"</code> <br/>
        :return: TransformJobArn:  The Amazon Resource Name (ARN) of the transform job.<br/>        """
        pass

    def create_workteam(self, Description: str, MemberDefinitions: List[MemberDefinition], WorkteamName: str, NotificationConfiguration: NotificationConfiguration=None, Tags: List[Tag]=None) -> CreateWorkteamResponse:
        """Creates a new work team for labeling your data. A work team is defined by one or more Amazon Cognito user pools. You must first create the user pools before you can create a work team. You cannot create more than 25 work teams in an account and region.

        :param self:
        :param Tags:  <p/><br/>NotificationConfiguration:  Configures notification of workers regarding available or expiring work items.:<br/>Description:  A description of the work team.<br/>MemberDefinitions:  A list of <code>MemberDefinition</code> objects that contains objects that identify the Amazon Cognito user pool that makes up the work team. For more information, see <a href="http://docs.aws.amazon.com/cognito/latest/developerguide/cognito-user-identity-pools.html">Amazon Cognito User Pools</a>. All of the <code>CognitoMemberDefinition</code> objects that make up the member definition must have the same <code>ClientId</code> and <code>UserPool</code> values.<br/>WorkteamName:  The name of the work team. Use this name to identify the work team.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationTopicArn</i> The ARN for the SNS topic to which notifications should be published.<br/>
        :return: WorkteamArn:  The Amazon Resource Name (ARN) of the work team. You can use this ARN to identify the work team.<br/>        """
        pass

    def delete_algorithm(self, AlgorithmName: str):
        """Removes the specified algorithm from your account.

        :param self:
        :param AlgorithmName:  The name of the algorithm to delete.<br/>
        :return:         """
        pass

    def delete_code_repository(self, CodeRepositoryName: str):
        """Deletes the specified Git repository from your account.

        :param self:
        :param CodeRepositoryName:  The name of the Git repository to delete.<br/>
        :return:         """
        pass

    def delete_endpoint(self, EndpointName: str):
        """Deletes an endpoint. Amazon SageMaker frees up all of the resources that were deployed when the endpoint was created.  Amazon SageMaker retires any custom KMS key grants associated with the endpoint, meaning you don't need to use the <a href="http://docs.aws.amazon.com/kms/latest/APIReference/API_RevokeGrant.html">RevokeGrant</a> API call.

        :param self:
        :param EndpointName:  The name of the endpoint that you want to delete.<br/>
        :return:         """
        pass

    def delete_endpoint_config(self, EndpointConfigName: str):
        """Deletes an endpoint configuration. The <code>DeleteEndpointConfig</code> API deletes only the specified configuration. It does not delete endpoints created using the configuration. 

        :param self:
        :param EndpointConfigName:  The name of the endpoint configuration that you want to delete.<br/>
        :return:         """
        pass

    def delete_model(self, ModelName: str):
        """Deletes a model. The <code>DeleteModel</code> API deletes only the model entry that was created in Amazon SageMaker when you called the <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/API_CreateModel.html">CreateModel</a> API. It does not delete model artifacts, inference code, or the IAM role that you specified when creating the model. 

        :param self:
        :param ModelName:  The name of the model to delete.<br/>
        :return:         """
        pass

    def delete_model_package(self, ModelPackageName: str):
        """Deletes a model package. A model package is used to create Amazon SageMaker models or list on AWS Marketplace. Buyers can subscribe to model packages listed on AWS Marketplace to create models in Amazon SageMaker.

        :param self:
        :param ModelPackageName:  The name of the model package. The name must have 1 to 63 characters. Valid characters are a-z, A-Z, 0-9, and - (hyphen).<br/>
        :return:         """
        pass

    def delete_notebook_instance(self, NotebookInstanceName: str):
        """ Deletes an Amazon SageMaker notebook instance. Before you can delete a notebook instance, you must call the <code>StopNotebookInstance</code> API.  <important> When you delete a notebook instance, you lose all of your data. Amazon SageMaker removes the ML compute instance, and deletes the ML storage volume and the network interface associated with the notebook instance.  </important>

        :param self:
        :param NotebookInstanceName:  The name of the Amazon SageMaker notebook instance to delete.<br/>
        :return:         """
        pass

    def delete_notebook_instance_lifecycle_config(self, NotebookInstanceLifecycleConfigName: str):
        """Deletes a notebook instance lifecycle configuration.

        :param self:
        :param NotebookInstanceLifecycleConfigName:  The name of the lifecycle configuration to delete.<br/>
        :return:         """
        pass

    def delete_tags(self, TagKeys: List[TagKey], ResourceArn: str) -> DeleteTagsOutput:
        """Deletes the specified tags from an Amazon SageMaker resource. To list a resource's tags, use the <code>ListTags</code> API.  <note> When you call this API to delete tags from a hyperparameter tuning job, the deleted tags are not removed from training jobs that the hyperparameter tuning job launched before you called this API. </note>

        :param self:
        :param TagKeys:  An array or one or more tag keys to delete.<br/>ResourceArn:  The Amazon Resource Name (ARN) of the resource whose tags you want to delete.<br/>
        :return:         """
        pass

    def delete_workteam(self, WorkteamName: str) -> DeleteWorkteamResponse:
        """Deletes an existing work team. This operation can't be undone.

        :param self:
        :param WorkteamName:  The name of the work team to delete.<br/>
        :return: Success:  Returns <code>true</code> if the work team was successfully deleted; otherwise, returns <code>false</code>.<br/>        """
        pass

    def describe_algorithm(self, AlgorithmName: str) -> DescribeAlgorithmOutput:
        """Returns a description of the specified algorithm that is in your account.

        :param self:
        :param AlgorithmName:  The name of the algorithm to describe.<br/>
        :return: CertifyForMarketplace:  Whether the algorithm is certified to be listed in AWS Marketplace.<br/>ProductId:  The product identifier of the algorithm.<br/>AlgorithmStatusDetails:  Details about the current status of the algorithm.:<br/>AlgorithmStatus:  The current status of the algorithm.<br/>ValidationSpecification:  Details about configurations for one or more training jobs that Amazon SageMaker runs to test the algorithm.:<br/>InferenceSpecification:  Details about inference jobs that the algorithm runs.:<br/>TrainingSpecification:  Details about training jobs run by this algorithm.:<br/>CreationTime:  A timestamp specifying when the algorithm was created.<br/>AlgorithmDescription:  A brief summary about the algorithm.<br/>AlgorithmArn:  The Amazon Resource Name (ARN) of the algorithm.<br/>AlgorithmName:  The name of the algorithm being described.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedTuningJobObjectiveMetrics</i> A list of the metrics that the algorithm emits that can be used as the objective metric in a hyperparameter tuning job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingChannels</i> A list of <code>ChannelSpecification</code> objects, which specify the input sources to be used by the algorithm.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricDefinitions</i> A list of <code>MetricDefinition</code> objects, which are used for parsing metrics generated by the algorithm.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportsDistributedTraining</i> Indicates whether the algorithm supports distributed training. If set to false, buyers can’t request more than one instance during training.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedTrainingInstanceTypes</i> A list of the instance types that this algorithm can use for training.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedHyperParameters</i> A list of the <code>HyperParameterSpecification</code> objects, that define the supported hyperparameters. This is required if the algorithm supports automatic model tuning.&gt;<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingImageDigest</i> An MD5 hash of the training algorithm that identifies the Docker image used for training.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingImage</i> The Amazon ECR registry path of the Docker image that contains the training algorithm.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedResponseMIMETypes</i> The supported MIME types for the output data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedContentTypes</i> The supported MIME types for the input data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedRealtimeInferenceInstanceTypes</i> A list of the instance types that are used to generate inferences in real-time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedTransformInstanceTypes</i> A list of the instance types on which a transformation job can be run or on which an endpoint can be deployed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Containers</i> The Amazon ECR registry path of the Docker image that contains the inference code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ValidationProfiles</i> An array of <code>AlgorithmValidationProfile</code> objects, each of which specifies a training job and batch transform job that Amazon SageMaker runs to validate your algorithm.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ValidationRole</i> The IAM roles that Amazon SageMaker uses to run the training jobs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImageScanStatuses</i> The status of the scan of the algorithm's Docker image container.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ValidationStatuses</i> The status of algorithm validation.<br/>        """
        pass

    def describe_code_repository(self, CodeRepositoryName: str) -> DescribeCodeRepositoryOutput:
        """Gets details about the specified Git repository.

        :param self:
        :param CodeRepositoryName:  The name of the Git repository to describe.<br/>
        :return: GitConfig:  Configuration details about the repository, including the URL where the repository is located, the default branch, and the Amazon Resource Name (ARN) of the AWS Secrets Manager secret that contains the credentials used to access the repository.:<br/>LastModifiedTime:  The date and time that the repository was last changed.<br/>CreationTime:  The date and time that the repository was created.<br/>CodeRepositoryArn:  The Amazon Resource Name (ARN) of the Git repository.<br/>CodeRepositoryName:  The name of the Git repository.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecretArn</i> The Amazon Resource Name (ARN) of the AWS Secrets Manager secret that contains the credentials used to access the git repository. The secret must have a staging label of <code>AWSCURRENT</code> and must be in the following format:  <code>{"username": <i>UserName</i>, "password": <i>Password</i>}</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Branch</i> The default branch for the Git repository.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RepositoryUrl</i> The URL where the Git repository is located.<br/>        """
        pass

    def describe_compilation_job(self, CompilationJobName: str) -> DescribeCompilationJobResponse:
        """Returns information about a model compilation job. To create a model compilation job, use <a>CreateCompilationJob</a>. To get information about multiple model compilation jobs, use <a>ListCompilationJobs</a>.

        :param self:
        :param CompilationJobName:  The name of the model compilation job that you want information about.<br/>
        :return: OutputConfig:  Information about the output location for the compiled model and the target device that the model runs on.:<br/>InputConfig:  Information about the location in Amazon S3 of the input model artifacts, the name and shape of the expected data inputs, and the framework in which the model was trained.:<br/>RoleArn:  The Amazon Resource Name (ARN) of the model compilation job.<br/>ModelArtifacts:  Information about the location in Amazon S3 that has been configured for storing the model artifacts used in the compilation job.:<br/>FailureReason:  If a model compilation job failed, the reason it failed. <br/>LastModifiedTime:  The time that the status of the model compilation job was last modified.<br/>CreationTime:  The time that the model compilation job was created.<br/>StoppingCondition:  Specifies a limit to how long a model compilation job can run. When the job reaches the time limit, Amazon SageMaker ends the compilation job. Use this API to cap model training costs.:<br/>CompilationEndTime:  The time when the model compilation job on a compilation job instance ended. For a successful or stopped job, this is when the job's model artifacts have finished uploading. For a failed job, this is when Amazon SageMaker detected that the job failed. <br/>CompilationStartTime:  The time when the model compilation job started the <code>CompilationJob</code> instances.  You are billed for the time between this timestamp and the timestamp in the <a>DescribeCompilationJobResponse$CompilationEndTime</a> field. In Amazon CloudWatch Logs, the start time might be later than this time. That's because it takes time to download the compilation job, which depends on the size of the compilation job container. <br/>CompilationJobStatus:  The status of the model compilation job.<br/>CompilationJobArn:  The Amazon Resource Name (ARN) of an IAM role that Amazon SageMaker assumes to perform the model compilation job.<br/>CompilationJobName:  The name of the model compilation job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxRuntimeInSeconds</i> The maximum length of time, in seconds, that the training or compilation job can run. If job does not complete during this time, Amazon SageMaker ends the job. If value is not specified, default value is 1 day. The maximum value is 28 days.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3ModelArtifacts</i> The path of the S3 object that contains the model artifacts. For example, <code>s3://bucket-name/keynameprefix/model.tar.gz</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Framework</i> Identifies the framework in which the model was trained. For example: TENSORFLOW.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataInputConfig</i> Specifies the name and shape of the expected data inputs for your trained model with a JSON dictionary form. The data inputs are <a>InputConfig$Framework</a> specific.  <ul> <li>  <code>TensorFlow</code>: You must specify the name and shape (NHWC format) of the expected data inputs using a dictionary format for your trained model. The dictionary formats required for the console and CLI are different. <ul> <li> Examples for one input: <ul> <li> If using the console, <code>{"input":[1,1024,1024,3]}</code>  </li> <li> If using the CLI, <code>{\"input\":[1,1024,1024,3]}</code>  </li> </ul> </li> <li> Examples for two inputs: <ul> <li> If using the console, <code>{"data1": [1,28,28,1], "data2":[1,28,28,1]}</code>  </li> <li> If using the CLI, <code>{\"data1\": [1,28,28,1], \"data2\":[1,28,28,1]}</code>  </li> </ul> </li> </ul> </li> <li>  <code>MXNET/ONNX</code>: You must specify the name and shape (NCHW format) of the expected data inputs in order using a dictionary format for your trained model. The dictionary formats required for the console and CLI are different. <ul> <li> Examples for one input: <ul> <li> If using the console, <code>{"data":[1,3,1024,1024]}</code>  </li> <li> If using the CLI, <code>{\"data\":[1,3,1024,1024]}</code>  </li> </ul> </li> <li> Examples for two inputs: <ul> <li> If using the console, <code>{"var1": [1,1,28,28], "var2":[1,1,28,28]} </code>  </li> <li> If using the CLI, <code>{\"var1\": [1,1,28,28], \"var2\":[1,1,28,28]}</code>  </li> </ul> </li> </ul> </li> <li>  <code>PyTorch</code>: You can either specify the name and shape (NCHW format) of expected data inputs in order using a dictionary format for your trained model or you can specify the shape only using a list format. The dictionary formats required for the console and CLI are different. The list formats for the console and CLI are the same. <ul> <li> Examples for one input in dictionary format: <ul> <li> If using the console, <code>{"input0":[1,3,224,224]}</code>  </li> <li> If using the CLI, <code>{\"input0\":[1,3,224,224]}</code>  </li> </ul> </li> <li> Example for one input in list format: <code>[[1,3,224,224]]</code>  </li> <li> Examples for two inputs in dictionary format: <ul> <li> If using the console, <code>{"input0":[1,3,224,224], "input1":[1,3,224,224]}</code>  </li> <li> If using the CLI, <code>{\"input0\":[1,3,224,224], \"input1\":[1,3,224,224]} </code>  </li> </ul> </li> <li> Example for two inputs in list format: <code>[[1,3,224,224], [1,3,224,224]]</code>  </li> </ul> </li> <li>  <code>XGBOOST</code>: input data name and shape are not needed. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Uri</i> The S3 path where the model artifacts, which result from model training, are stored. This path must point to a single gzip compressed tar archive (.tar.gz suffix).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetDevice</i> Identifies the device that you want to run your model on after it has been compiled. For example: ml_c5.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3OutputLocation</i> Identifies the S3 path where you want Amazon SageMaker to store the model artifacts. For example, s3://bucket-name/key-name-prefix.<br/>        """
        pass

    def describe_endpoint(self, EndpointName: str) -> DescribeEndpointOutput:
        """Returns the description of an endpoint.

        :param self:
        :param EndpointName:  The name of the endpoint.<br/>
        :return: LastModifiedTime:  A timestamp that shows when the endpoint was last modified.<br/>CreationTime:  A timestamp that shows when the endpoint was created.<br/>FailureReason:  If the status of the endpoint is <code>Failed</code>, the reason why it failed. <br/>EndpointStatus:  The status of the endpoint. <ul> <li>  <code>OutOfService</code>: Endpoint is not available to take incoming requests. </li> <li>  <code>Creating</code>: <a>CreateEndpoint</a> is executing. </li> <li>  <code>Updating</code>: <a>UpdateEndpoint</a> or <a>UpdateEndpointWeightsAndCapacities</a> is executing. </li> <li>  <code>SystemUpdating</code>: Endpoint is undergoing maintenance and cannot be updated or deleted or re-scaled until it has completed. This maintenance operation does not change any customer-specified values such as VPC config, KMS encryption, model, instance type, or instance count. </li> <li>  <code>RollingBack</code>: Endpoint fails to scale up or down or change its variant weight and is in the process of rolling back to its previous configuration. Once the rollback completes, endpoint returns to an <code>InService</code> status. This transitional status only applies to an endpoint that has autoscaling enabled and is undergoing variant weight or capacity changes as part of an <a>UpdateEndpointWeightsAndCapacities</a> call or when the <a>UpdateEndpointWeightsAndCapacities</a> operation is called explicitly. </li> <li>  <code>InService</code>: Endpoint is available to process incoming requests. </li> <li>  <code>Deleting</code>: <a>DeleteEndpoint</a> is executing. </li> <li>  <code>Failed</code>: Endpoint could not be created, updated, or re-scaled. Use <a>DescribeEndpointOutput$FailureReason</a> for information about the failure. <a>DeleteEndpoint</a> is the only operation that can be performed on a failed endpoint. </li> </ul><br/>ProductionVariants:   An array of <a>ProductionVariantSummary</a> objects, one for each model hosted behind this endpoint. <br/>EndpointConfigName:  The name of the endpoint configuration associated with this endpoint.<br/>EndpointArn:  The Amazon Resource Name (ARN) of the endpoint.<br/>EndpointName:  Name of the endpoint.<br/>        """
        pass

    def describe_endpoint_config(self, EndpointConfigName: str) -> DescribeEndpointConfigOutput:
        """Returns the description of an endpoint configuration created using the <code>CreateEndpointConfig</code> API.

        :param self:
        :param EndpointConfigName:  The name of the endpoint configuration.<br/>
        :return: CreationTime:  A timestamp that shows when the endpoint configuration was created.<br/>KmsKeyId:  AWS KMS key ID Amazon SageMaker uses to encrypt data when storing it on the ML storage volume attached to the instance.<br/>ProductionVariants:  An array of <code>ProductionVariant</code> objects, one for each model that you want to host at this endpoint.<br/>EndpointConfigArn:  The Amazon Resource Name (ARN) of the endpoint configuration.<br/>EndpointConfigName:  Name of the Amazon SageMaker endpoint configuration.<br/>        """
        pass

    def describe_hyper_parameter_tuning_job(self, HyperParameterTuningJobName: str) -> DescribeHyperParameterTuningJobResponse:
        """Gets a description of a hyperparameter tuning job.

        :param self:
        :param HyperParameterTuningJobName:  The name of the tuning job to describe.<br/>
        :return: FailureReason:  If the tuning job failed, the reason it failed.<br/>WarmStartConfig:  The configuration for starting the hyperparameter parameter tuning job using one or more previous tuning jobs as a starting point. The results of previous tuning jobs are used to inform which combinations of hyperparameters to search over in the new tuning job.:<br/>OverallBestTrainingJob:  If the hyperparameter tuning job is an warm start tuning job with a <code>WarmStartType</code> of <code>IDENTICAL_DATA_AND_ALGORITHM</code>, this is the <a>TrainingJobSummary</a> for the training job with the best objective metric value of all training jobs launched by this tuning job and all parent jobs specified for the warm start tuning job.:<br/>BestTrainingJob:  A <a>TrainingJobSummary</a> object that describes the training job that completed with the best current <a>HyperParameterTuningJobObjective</a>.:<br/>ObjectiveStatusCounters:  The <a>ObjectiveStatusCounters</a> object that specifies the number of training jobs, categorized by the status of their final objective metric, that this tuning job launched.:<br/>TrainingJobStatusCounters:  The <a>TrainingJobStatusCounters</a> object that specifies the number of training jobs, categorized by status, that this tuning job launched.:<br/>LastModifiedTime:  The date and time that the status of the tuning job was modified. <br/>HyperParameterTuningEndTime:  The date and time that the tuning job ended.<br/>CreationTime:  The date and time that the tuning job started.<br/>HyperParameterTuningJobStatus:  The status of the tuning job: InProgress, Completed, Failed, Stopping, or Stopped.<br/>TrainingJobDefinition:  The <a>HyperParameterTrainingJobDefinition</a> object that specifies the definition of the training jobs that this tuning job launches.:<br/>HyperParameterTuningJobConfig:  The <a>HyperParameterTuningJobConfig</a> object that specifies the configuration of the tuning job.:<br/>HyperParameterTuningJobArn:  The Amazon Resource Name (ARN) of the tuning job.<br/>HyperParameterTuningJobName:  The name of the tuning job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingJobEarlyStoppingType</i> Specifies whether to use early stopping for training jobs launched by the hyperparameter tuning job. This can be one of the following values (the default value is <code>OFF</code>): <dl> <dt>OFF</dt> <dd> Training jobs launched by the hyperparameter tuning job do not use early stopping. </dd> <dt>AUTO</dt> <dd> Amazon SageMaker stops training jobs launched by the hyperparameter tuning job when they are unlikely to perform better than previously completed training jobs. For more information, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/automatic-model-tuning-early-stopping.html">Stop Training Jobs Early</a>. </dd> </dl><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterRanges</i> The <a>ParameterRanges</a> object that specifies the ranges of hyperparameters that this tuning job searches.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ParameterRanges</b>: The <a>ParameterRanges</a> object that specifies the ranges of hyperparameters that this tuning job searches.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceLimits</i> The <a>ResourceLimits</a> object that specifies the maximum number of training jobs and parallel training jobs for this tuning job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResourceLimits</b>: The <a>ResourceLimits</a> object that specifies the maximum number of training jobs and parallel training jobs for this tuning job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HyperParameterTuningJobObjective</i> The <a>HyperParameterTuningJobObjective</a> object that specifies the objective metric for this tuning job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>HyperParameterTuningJobObjective</b>: The <a>HyperParameterTuningJobObjective</a> object that specifies the objective metric for this tuning job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Strategy</i> Specifies how hyperparameter tuning chooses the combinations of hyperparameter values to use for the training job it launches. To use the Bayesian search stategy, set this to <code>Bayesian</code>. To randomly search, set it to <code>Random</code>. For information about search strategies, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/automatic-model-tuning-how-it-works.html">How Hyperparameter Tuning Works</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricName</i> The name of the metric to use for the objective metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> Whether to minimize or maximize the objective metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxParallelTrainingJobs</i> The maximum number of concurrent training jobs that a hyperparameter tuning job can launch.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxNumberOfTrainingJobs</i> The maximum number of training jobs that a hyperparameter tuning job can launch.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CategoricalParameterRanges</i> The array of <a>CategoricalParameterRange</a> objects that specify ranges of categorical hyperparameters that a hyperparameter tuning job searches.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ContinuousParameterRanges</i> The array of <a>ContinuousParameterRange</a> objects that specify ranges of continuous hyperparameters that a hyperparameter tuning job searches.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IntegerParameterRanges</i> The array of <a>IntegerParameterRange</a> objects that specify ranges of integer hyperparameters that a hyperparameter tuning job searches.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnableInterContainerTrafficEncryption</i> To encrypt all communications between ML compute instances in distributed training, choose <code>True</code>. Encryption provides greater security for distributed training, but training might take longer. How long it takes depends on the amount of communication between compute instances, especially if you use a deep learning algorithm in distributed training.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EnableNetworkIsolation</i> Isolates the training container. No inbound or outbound network calls can be made, except for calls between peers within a training cluster for distributed training. If network isolation is used for training jobs that are configured to use a VPC, Amazon SageMaker downloads and uploads customer data and model artifacts through the specified VPC, but the training container does not have network access. <note> The Semantic Segmentation built-in algorithm does not support network isolation. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StoppingCondition</i> Specifies a limit to how long a model hyperparameter training job can run. When the job reaches the time limit, Amazon SageMaker ends the training job. Use this API to cap model training costs.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StoppingCondition</b>: Specifies a limit to how long a model hyperparameter training job can run. When the job reaches the time limit, Amazon SageMaker ends the training job. Use this API to cap model training costs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceConfig</i> The resources, including the compute instances and storage volumes, to use for the training jobs that the tuning job launches. Storage volumes store model artifacts and incremental states. Training algorithms might also use storage volumes for scratch space. If you want Amazon SageMaker to use the storage volume to store the training data, choose <code>File</code> as the <code>TrainingInputMode</code> in the algorithm specification. For distributed training algorithms, specify an instance count greater than 1.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResourceConfig</b>: The resources, including the compute instances and storage volumes, to use for the training jobs that the tuning job launches. Storage volumes store model artifacts and incremental states. Training algorithms might also use storage volumes for scratch space. If you want Amazon SageMaker to use the storage volume to store the training data, choose <code>File</code> as the <code>TrainingInputMode</code> in the algorithm specification. For distributed training algorithms, specify an instance count greater than 1.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputDataConfig</i> Specifies the path to the Amazon S3 bucket where you store model artifacts from the training jobs that the tuning job launches.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>OutputDataConfig</b>: Specifies the path to the Amazon S3 bucket where you store model artifacts from the training jobs that the tuning job launches.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcConfig</i> The <a>VpcConfig</a> object that specifies the VPC that you want the training jobs that this hyperparameter tuning job launches to connect to. Control access to and from your training container by configuring the VPC. For more information, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/train-vpc.html">Protect Training Jobs by Using an Amazon Virtual Private Cloud</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>VpcConfig</b>: The <a>VpcConfig</a> object that specifies the VPC that you want the training jobs that this hyperparameter tuning job launches to connect to. Control access to and from your training container by configuring the VPC. For more information, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/train-vpc.html">Protect Training Jobs by Using an Amazon Virtual Private Cloud</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputDataConfig</i> An array of <a>Channel</a> objects that specify the input for the training jobs that the tuning job launches.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The Amazon Resource Name (ARN) of the IAM role associated with the training jobs that the tuning job launches.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AlgorithmSpecification</i> The <a>HyperParameterAlgorithmSpecification</a> object that specifies the resource algorithm to use for the training jobs that the tuning job launches.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AlgorithmSpecification</b>: The <a>HyperParameterAlgorithmSpecification</a> object that specifies the resource algorithm to use for the training jobs that the tuning job launches.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StaticHyperParameters</i> Specifies the values of hyperparameters that do not change for the tuning job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricDefinitions</i> An array of <a>MetricDefinition</a> objects that specify the metrics that the algorithm emits.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AlgorithmName</i> The name of the resource algorithm to use for the hyperparameter tuning job. If you specify a value for this parameter, do not specify a value for <code>TrainingImage</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingInputMode</i> The input mode that the algorithm supports: File or Pipe. In File input mode, Amazon SageMaker downloads the training data from Amazon S3 to the storage volume that is attached to the training instance and mounts the directory to the Docker volume for the training container. In Pipe input mode, Amazon SageMaker streams data directly from Amazon S3 to the container.  If you specify File mode, make sure that you provision the storage volume that is attached to the training instance with enough capacity to accommodate the training data downloaded from Amazon S3, the model artifacts, and intermediate information. <p/> For more information about input modes, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/algos.html">Algorithms</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingImage</i>  The registry path of the Docker image that contains the training algorithm. For information about Docker registry paths for built-in algorithms, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/sagemaker-algo-docker-registry-paths.html">Algorithms Provided by Amazon SageMaker: Common Parameters</a>. Amazon SageMaker supports both <code>registry/repository[:tag]</code> and <code>registry/repository[@digest]</code> image path formats. For more information, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/your-algorithms.html">Using Your Own Algorithms with Amazon SageMaker</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> The ID of the subnets in the VPC to which you want to connect your training job or model.  <note> Amazon EC2 P3 accelerated computing instances are not available in the c/d/e availability zones of region us-east-1. If you want to create endpoints with P3 instances in VPC mode in region us-east-1, create subnets in a/b/f availability zones instead. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The VPC security group IDs, in the form sg-xxxxxxxx. Specify the security groups for the VPC that is specified in the <code>Subnets</code> field.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3OutputPath</i> Identifies the S3 path where you want Amazon SageMaker to store the model artifacts. For example, <code>s3://bucket-name/key-name-prefix</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS Key Management Service (AWS KMS) key that Amazon SageMaker uses to encrypt the model artifacts at rest using Amazon S3 server-side encryption. The <code>KmsKeyId</code> can be any of the following formats:  <ul> <li> // KMS Key ID  <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> // Amazon Resource Name (ARN) of a KMS Key  <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> // KMS Key Alias  <code>"alias/ExampleAlias"</code>  </li> <li> // Amazon Resource Name (ARN) of a KMS Key Alias  <code>"arn:aws:kms:us-west-2:111122223333:alias/ExampleAlias"</code>  </li> </ul> If you use a KMS key ID or an alias of your master key, the Amazon SageMaker execution role must include permissions to call <code>kms:Encrypt</code>. If you don't provide a KMS key ID, Amazon SageMaker uses the default KMS key for Amazon S3 for your role's account. Amazon SageMaker uses server-side encryption with KMS-managed keys for <code>OutputDataConfig</code>. If you use a bucket policy with an <code>s3:PutObject</code> permission that only allows objects with server-side encryption, set the condition key of <code>s3:x-amz-server-side-encryption</code> to <code>"aws:kms"</code>. For more information, see <a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/UsingKMSEncryption.html">KMS-Managed Encryption Keys</a> in the <i>Amazon Simple Storage Service Developer Guide.</i>  The KMS key policy must grant permission to the IAM role that you specify in your <code>CreateTrainingJob</code>, <code>CreateTransformJob</code>, or <code>CreateHyperParameterTuningJob</code> requests. For more information, see <a href="http://docs.aws.amazon.com/kms/latest/developerguide/key-policies.html">Using Key Policies in AWS KMS</a> in the <i>AWS Key Management Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeKmsKeyId</i> The AWS Key Management Service (AWS KMS) key that Amazon SageMaker uses to encrypt data on the storage volume attached to the ML compute instance(s) that run the training job. The <code>VolumeKmsKeyId</code> can be any of the following formats: <ul> <li> // KMS Key ID  <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> // Amazon Resource Name (ARN) of a KMS Key  <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeSizeInGB</i> The size of the ML storage volume that you want to provision.  ML storage volumes store model artifacts and incremental states. Training algorithms might also use the ML storage volume for scratch space. If you want to store the training data in the ML storage volume, choose <code>File</code> as the <code>TrainingInputMode</code> in the algorithm specification.  You must specify sufficient ML storage for your scenario.  <note>  Amazon SageMaker supports only the General Purpose SSD (gp2) ML storage volume type.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCount</i> The number of ML compute instances to use. For distributed training, provide a value greater than 1. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i> The ML compute instance type. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxRuntimeInSeconds</i> The maximum length of time, in seconds, that the training or compilation job can run. If job does not complete during this time, Amazon SageMaker ends the job. If value is not specified, default value is 1 day. The maximum value is 28 days.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Stopped</i> The number of training jobs launched by a hyperparameter tuning job that were manually stopped.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NonRetryableError</i> The number of training jobs that failed and can't be retried. A failed training job can't be retried if it failed because a client error occurred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RetryableError</i> The number of training jobs that failed, but can be retried. A failed training job can be retried only if it failed because an internal service error occurred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InProgress</i> The number of in-progress training jobs launched by a hyperparameter tuning job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Completed</i> The number of completed training jobs launched by the hyperparameter tuning job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Failed</i> The number of training jobs whose final objective metric was not evaluated and used in the hyperparameter tuning process. This typically occurs when the training job failed or did not emit an objective metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Pending</i> The number of training jobs that are in progress and pending evaluation of their final objective metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Succeeded</i> The number of training jobs whose final objective metric was evaluated by the hyperparameter tuning job and used in the hyperparameter tuning process.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ObjectiveStatus</i> The status of the objective metric for the training job: <ul> <li> Succeeded: The final objective metric for the training job was evaluated by the hyperparameter tuning job and used in the hyperparameter tuning process. </li> </ul> <ul> <li> Pending: The training job is in progress and evaluation of its final objective metric is pending. </li> </ul> <ul> <li> Failed: The final objective metric for the training job was not evaluated, and was not used in the hyperparameter tuning process. This typically occurs when the training job failed or did not emit an objective metric. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FinalHyperParameterTuningJobObjectiveMetric</i> The <a>FinalHyperParameterTuningJobObjectiveMetric</a> object that specifies the value of the objective metric of the tuning job that launched this training job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>FinalHyperParameterTuningJobObjectiveMetric</b>: The <a>FinalHyperParameterTuningJobObjectiveMetric</a> object that specifies the value of the objective metric of the tuning job that launched this training job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureReason</i> The reason that the training job failed. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TunedHyperParameters</i> A list of the hyperparameters for which you specified ranges to search.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingJobStatus</i> The status of the training job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingEndTime</i> Specifies the time when the training job ends on training instances. You are billed for the time interval between the value of <code>TrainingStartTime</code> and this time. For successful jobs and stopped jobs, this is the time after model artifacts are uploaded. For failed jobs, this is the time when Amazon SageMaker detects a job failure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingStartTime</i> The date and time that the training job started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The date and time that the training job was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TuningJobName</i> The HyperParameter tuning job that launched the training job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingJobArn</i> The Amazon Resource Name (ARN) of the training job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingJobName</i> The name of the training job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The value of the objective metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricName</i> The name of the objective metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> Whether to minimize or maximize the objective metric. Valid values are Minimize and Maximize.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ObjectiveStatus</i> The status of the objective metric for the training job: <ul> <li> Succeeded: The final objective metric for the training job was evaluated by the hyperparameter tuning job and used in the hyperparameter tuning process. </li> </ul> <ul> <li> Pending: The training job is in progress and evaluation of its final objective metric is pending. </li> </ul> <ul> <li> Failed: The final objective metric for the training job was not evaluated, and was not used in the hyperparameter tuning process. This typically occurs when the training job failed or did not emit an objective metric. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FinalHyperParameterTuningJobObjectiveMetric</i> The <a>FinalHyperParameterTuningJobObjectiveMetric</a> object that specifies the value of the objective metric of the tuning job that launched this training job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>FinalHyperParameterTuningJobObjectiveMetric</b>: The <a>FinalHyperParameterTuningJobObjectiveMetric</a> object that specifies the value of the objective metric of the tuning job that launched this training job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureReason</i> The reason that the training job failed. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TunedHyperParameters</i> A list of the hyperparameters for which you specified ranges to search.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingJobStatus</i> The status of the training job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingEndTime</i> Specifies the time when the training job ends on training instances. You are billed for the time interval between the value of <code>TrainingStartTime</code> and this time. For successful jobs and stopped jobs, this is the time after model artifacts are uploaded. For failed jobs, this is the time when Amazon SageMaker detects a job failure.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingStartTime</i> The date and time that the training job started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The date and time that the training job was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TuningJobName</i> The HyperParameter tuning job that launched the training job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingJobArn</i> The Amazon Resource Name (ARN) of the training job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingJobName</i> The name of the training job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The value of the objective metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricName</i> The name of the objective metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> Whether to minimize or maximize the objective metric. Valid values are Minimize and Maximize.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WarmStartType</i> Specifies one of the following: <dl> <dt>IDENTICAL_DATA_AND_ALGORITHM</dt> <dd> The new hyperparameter tuning job uses the same input data and training image as the parent tuning jobs. You can change the hyperparameter ranges to search and the maximum number of training jobs that the hyperparameter tuning job launches. You cannot use a new version of the training algorithm, unless the changes in the new version do not affect the algorithm itself. For example, changes that improve logging or adding support for a different data format are allowed. You can also change hyperparameters from tunable to static, and from static to tunable, but the total number of static plus tunable hyperparameters must remain the same as it is in all parent jobs. The objective metric for the new tuning job must be the same as for all parent jobs. </dd> <dt>TRANSFER_LEARNING</dt> <dd> The new hyperparameter tuning job can include input data, hyperparameter ranges, maximum number of concurrent training jobs, and maximum number of training jobs that are different than those of its parent hyperparameter tuning jobs. The training image can also be a different version from the version used in the parent hyperparameter tuning job. You can also change hyperparameters from tunable to static, and from static to tunable, but the total number of static plus tunable hyperparameters must remain the same as it is in all parent jobs. The objective metric for the new tuning job must be the same as for all parent jobs. </dd> </dl><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParentHyperParameterTuningJobs</i> An array of hyperparameter tuning jobs that are used as the starting point for the new hyperparameter tuning job. For more information about warm starting a hyperparameter tuning job, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/automatic-model-tuning-warm-start.html">Using a Previous Hyperparameter Tuning Job as a Starting Point</a>. Hyperparameter tuning jobs created before October 1, 2018 cannot be used as parent jobs for warm start tuning jobs.<br/>        """
        pass

    def describe_labeling_job(self, LabelingJobName: str) -> DescribeLabelingJobResponse:
        """Gets information about a labeling job.

        :param self:
        :param LabelingJobName:  The name of the labeling job to return information for.<br/>
        :return: LabelingJobOutput:  The location of the output produced by the labeling job.:<br/>Tags:  An array of key/value pairs. For more information, see <a href="http://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/cost-alloc-tags.html#allocation-what">Using Cost Allocation Tags</a> in the <i>AWS Billing and Cost Management User Guide</i>.<br/>HumanTaskConfig:  Configuration information required for human workers to complete a labeling task.:<br/>LabelingJobAlgorithmsConfig:  Configuration information for automated data labeling.:<br/>StoppingConditions:  A set of conditions for stopping a labeling job. If any of the conditions are met, the job is automatically stopped.:<br/>LabelCategoryConfigS3Uri:  The S3 location of the JSON file that defines the categories used to label data objects. The file is a JSON structure in the following format:  <code>{</code>   <code> "document-version": "2018-11-28"</code>   <code> "labels": [</code>   <code> {</code>   <code> "label": "<i>label 1</i>"</code>   <code> },</code>   <code> {</code>   <code> "label": "<i>label 2</i>"</code>   <code> },</code>   <code> ...</code>   <code> {</code>   <code> "label": "<i>label n</i>"</code>   <code> }</code>   <code> ]</code>   <code>}</code> <br/>RoleArn:  The Amazon Resource Name (ARN) that Amazon SageMaker assumes to perform tasks on your behalf during data labeling.<br/>OutputConfig:  The location of the job's output data and the AWS Key Management Service key ID for the key used to encrypt the output data, if any.:<br/>InputConfig:  Input configuration information for the labeling job, such as the Amazon S3 location of the data objects and the location of the manifest file that describes the data objects.:<br/>LabelAttributeName:  The attribute used as the label in the output manifest file.<br/>LabelingJobArn:  The Amazon Resource Name (ARN) of the labeling job.<br/>LabelingJobName:  The name assigned to the labeling job when it was created.<br/>JobReferenceCode:  A unique identifier for work done as part of a labeling job.<br/>LastModifiedTime:  The date and time that the labeling job was last updated.<br/>CreationTime:  The date and time that the labeling job was created.<br/>FailureReason:  If the job failed, the reason that it failed. <br/>LabelCounters:  Provides a breakdown of the number of data objects labeled by humans, the number of objects labeled by machine, the number of objects than couldn't be labeled, and the total number of objects labeled. :<br/>LabelingJobStatus:  The processing status of the labeling job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Unlabeled</i> The total number of objects not yet labeled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailedNonRetryableError</i> The total number of objects that could not be labeled due to an error.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MachineLabeled</i> The total number of objects labeled by automated data labeling.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HumanLabeled</i> The total number of objects labeled by a human worker.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalLabeled</i> The total number of objects labeled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataAttributes</i> Attributes of the data specified by the customer.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DataAttributes</b>: Attributes of the data specified by the customer.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataSource</i> The location of the input data.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DataSource</b>: The location of the input data.<br/>The Amazon S3 location of the input data objects.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3DataSource</b>: The Amazon S3 location of the input data objects.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ContentClassifiers</i> Declares that your content is free of personally identifiable information or adult content. Amazon SageMaker may restrict the Amazon Mechanical Turk workers that can view your task based on this information.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS Key Management Service ID of the key used to encrypt the output data, if any. If you use a KMS key ID or an alias of your master key, the Amazon SageMaker execution role must include permissions to call <code>kms:Encrypt</code>. If you don't provide a KMS key ID, Amazon SageMaker uses the default KMS key for Amazon S3 for your role's account. Amazon SageMaker uses server-side encryption with KMS-managed keys for <code>LabelingJobOutputConfig</code>. If you use a bucket policy with an <code>s3:PutObject</code> permission that only allows objects with server-side encryption, set the condition key of <code>s3:x-amz-server-side-encryption</code> to <code>"aws:kms"</code>. For more information, see <a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/UsingKMSEncryption.html">KMS-Managed Encryption Keys</a> in the <i>Amazon Simple Storage Service Developer Guide.</i>  The KMS key policy must grant permission to the IAM role that you specify in your <code>CreateLabelingJob</code> request. For more information, see <a href="http://docs.aws.amazon.com/kms/latest/developerguide/key-policies.html">Using Key Policies in AWS KMS</a> in the <i>AWS Key Management Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3OutputPath</i> The Amazon S3 location to write output data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxPercentageOfInputDatasetLabeled</i> The maximum number of input data objects that should be labeled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxHumanLabeledObjectCount</i> The maximum number of objects that can be labeled by human workers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LabelingJobResourceConfig</i> Provides configuration information for a labeling job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LabelingJobResourceConfig</b>: Provides configuration information for a labeling job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InitialActiveLearningModelArn</i> At the end of an auto-label job Amazon SageMaker Ground Truth sends the Amazon Resource Nam (ARN) of the final model used for auto-labeling. You can use this model as the starting point for subsequent similar jobs by providing the ARN of the model here. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LabelingJobAlgorithmSpecificationArn</i> Specifies the Amazon Resource Name (ARN) of the algorithm used for auto-labeling. You must select one of the following ARNs: <ul> <li>  <i>Image classification</i>   <code>arn:aws:sagemaker:<i>region</i>:027400017018:labeling-job-algorithm-specification/image-classification</code>  </li> <li>  <i>Text classification</i>   <code>arn:aws:sagemaker:<i>region</i>:027400017018:labeling-job-algorithm-specification/text-classification</code>  </li> <li>  <i>Object detection</i>   <code>arn:aws:sagemaker:<i>region</i>:027400017018:labeling-job-algorithm-specification/object-detection</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeKmsKeyId</i> The AWS Key Management Service key ID for the key used to encrypt the output data, if any.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PublicWorkforceTaskPrice</i> The price that you pay for each task performed by a public worker.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PublicWorkforceTaskPrice</b>: The price that you pay for each task performed by a public worker.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AnnotationConsolidationConfig</i> Configures how labels are consolidated across human workers.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AnnotationConsolidationConfig</b>: Configures how labels are consolidated across human workers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxConcurrentTaskCount</i> Defines the maximum number of data objects that can be labeled by human workers at the same time. Each object may have more than one worker at one time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TaskAvailabilityLifetimeInSeconds</i> The length of time that a task remains available for labelling by human workers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TaskTimeLimitInSeconds</i> The amount of time that a worker has to complete a task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NumberOfHumanWorkersPerDataObject</i> The number of human workers that will label an object. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TaskDescription</i> A description of the task for your human workers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TaskTitle</i> A title for the task for your human workers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TaskKeywords</i> Keywords used to describe the task so that workers on Amazon Mechanical Turk can discover the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PreHumanTaskLambdaArn</i> The Amazon Resource Name (ARN) of a Lambda function that is run before a data object is sent to a human worker. Use this function to provide input to a custom labeling job. For the built-in bounding box, image classification, semantic segmentation, and text classification task types, Amazon SageMaker Ground Truth provides the following Lambda functions:  <b>US East (Northern Virginia) (us-east-1):</b>  <ul> <li>  <code>arn:aws:lambda:us-east-1:432418664414:function:PRE-BoundingBox</code>  </li> <li>  <code>arn:aws:lambda:us-east-1:432418664414:function:PRE-ImageMultiClass</code>  </li> <li>  <code>arn:aws:lambda:us-east-1:432418664414:function:PRE-SemanticSegmentation</code>  </li> <li>  <code>arn:aws:lambda:us-east-1:432418664414:function:PRE-TextMultiClass</code>  </li> </ul>  <b>US East (Ohio) (us-east-2):</b>  <ul> <li>  <code>arn:aws:lambda:us-east-2:266458841044:function:PRE-BoundingBox</code>  </li> <li>  <code>arn:aws:lambda:us-east-2:266458841044:function:PRE-ImageMultiClass</code>  </li> <li>  <code>arn:aws:lambda:us-east-2:266458841044:function:PRE-SemanticSegmentation</code>  </li> <li>  <code>arn:aws:lambda:us-east-2:266458841044:function:PRE-TextMultiClass</code>  </li> </ul>  <b>US West (Oregon) (us-west-2):</b>  <ul> <li>  <code>arn:aws:lambda:us-west-2:081040173940:function:PRE-BoundingBox</code>  </li> <li>  <code>arn:aws:lambda:us-west-2:081040173940:function:PRE-ImageMultiClass</code>  </li> <li>  <code>arn:aws:lambda:us-west-2:081040173940:function:PRE-SemanticSegmentation</code>  </li> <li>  <code>arn:aws:lambda:us-west-2:081040173940:function:PRE-TextMultiClass</code>  </li> </ul>  <b>EU (Ireland) (eu-west-1):</b>  <ul> <li>  <code>arn:aws:lambda:eu-west-1:568282634449:function:PRE-BoundingBox</code>  </li> <li>  <code>arn:aws:lambda:eu-west-1:568282634449:function:PRE-ImageMultiClass</code>  </li> <li>  <code>arn:aws:lambda:eu-west-1:568282634449:function:PRE-SemanticSegmentation</code>  </li> <li>  <code>arn:aws:lambda:eu-west-1:568282634449:function:PRE-TextMultiClass</code>  </li> </ul>  <b>Asia Pacific (Tokyo) (ap-northeast-1):</b>  <ul> <li>  <code>arn:aws:lambda:ap-northeast-1:477331159723:function:PRE-BoundingBox</code>  </li> <li>  <code>arn:aws:lambda:ap-northeast-1:477331159723:function:PRE-ImageMultiClass</code>  </li> <li>  <code>arn:aws:lambda:ap-northeast-1:477331159723:function:PRE-SemanticSegmentation</code>  </li> <li>  <code>arn:aws:lambda:ap-northeast-1:477331159723:function:PRE-TextMultiClass</code>  </li> </ul>  <b>Asia Pacific (Sydney) (ap-southeast-1):</b>  <ul> <li>  <code>arn:aws:lambda:ap-southeast-2:454466003867:function:PRE-BoundingBox</code>  </li> <li>  <code>arn:aws:lambda:ap-southeast-2:454466003867:function:PRE-ImageMultiClass</code>  </li> <li>  <code>arn:aws:lambda:ap-southeast-2:454466003867:function:PRE-SemanticSegmentation</code>  </li> <li>  <code>arn:aws:lambda:ap-southeast-2:454466003867:function:PRE-TextMultiClass</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UiConfig</i> Information about the user interface that workers use to complete the labeling task.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>UiConfig</b>: Information about the user interface that workers use to complete the labeling task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WorkteamArn</i> The Amazon Resource Name (ARN) of the work team assigned to complete the tasks.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>UiTemplateS3Uri</i> The Amazon S3 bucket location of the UI template. For more information about the contents of a UI template, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/sms-custom-templates-step2.html"> Creating Your Custom Labeling Task Template</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AnnotationConsolidationLambdaArn</i> The Amazon Resource Name (ARN) of a Lambda function implements the logic for annotation consolidation. For the built-in bounding box, image classification, semantic segmentation, and text classification task types, Amazon SageMaker Ground Truth provides the following Lambda functions: <ul> <li>  <i>Bounding box</i> - Finds the most similar boxes from different workers based on the Jaccard index of the boxes.  <code>arn:aws:lambda:us-east-1:432418664414:function:ACS-BoundingBox</code>   <code>arn:aws:lambda:us-east-2:266458841044:function:ACS-BoundingBox</code>   <code>arn:aws:lambda:us-west-2:081040173940:function:ACS-BoundingBox</code>   <code>arn:aws:lambda:eu-west-1:568282634449:function:ACS-BoundingBox</code>   <code>arn:aws:lambda:ap-northeast-1:477331159723:function:ACS-BoundingBox</code>   <code>arn:aws:lambda:ap-southeast-2:454466003867:function:ACS-BoundingBox</code>  </li> <li>  <i>Image classification</i> - Uses a variant of the Expectation Maximization approach to estimate the true class of an image based on annotations from individual workers.  <code>arn:aws:lambda:us-east-1:432418664414:function:ACS-ImageMultiClass</code>   <code>arn:aws:lambda:us-east-2:266458841044:function:ACS-ImageMultiClass</code>   <code>arn:aws:lambda:us-west-2:081040173940:function:ACS-ImageMultiClass</code>   <code>arn:aws:lambda:eu-west-1:568282634449:function:ACS-ImageMultiClass</code>   <code>arn:aws:lambda:ap-northeast-1:477331159723:function:ACS-ImageMultiClass</code>   <code>arn:aws:lambda:ap-southeast-2:454466003867:function:ACS-ImageMultiClass</code>  </li> <li>  <i>Semantic segmentation</i> - Treats each pixel in an image as a multi-class classification and treats pixel annotations from workers as "votes" for the correct label.  <code>arn:aws:lambda:us-east-1:432418664414:function:ACS-SemanticSegmentation</code>   <code>arn:aws:lambda:us-east-2:266458841044:function:ACS-SemanticSegmentation</code>   <code>arn:aws:lambda:us-west-2:081040173940:function:ACS-SemanticSegmentation</code>   <code>arn:aws:lambda:eu-west-1:568282634449:function:ACS-SemanticSegmentation</code>   <code>arn:aws:lambda:ap-northeast-1:477331159723:function:ACS-SemanticSegmentation</code>   <code>arn:aws:lambda:ap-southeast-2:454466003867:function:ACS-SemanticSegmentation</code>  </li> <li>  <i>Text classification</i> - Uses a variant of the Expectation Maximization approach to estimate the true class of text based on annotations from individual workers.  <code>arn:aws:lambda:us-east-1:432418664414:function:ACS-TextMultiClass</code>   <code>arn:aws:lambda:us-east-2:266458841044:function:ACS-TextMultiClass</code>   <code>arn:aws:lambda:us-west-2:081040173940:function:ACS-TextMultiClass</code>   <code>arn:aws:lambda:eu-west-1:568282634449:function:ACS-TextMultiClass</code>   <code>arn:aws:lambda:ap-northeast-1:477331159723:function:ACS-TextMultiClass</code>   <code>arn:aws:lambda:ap-southeast-2:454466003867:function:ACS-TextMultiClass</code>  </li> </ul> For more information, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/sms-annotation-consolidation.html">Annotation Consolidation</a>.<br/>Defines the amount of money paid to a worker in United States dollars.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>AmountInUsd</b>: Defines the amount of money paid to a worker in United States dollars.&nbsp;&nbsp;&nbsp;&nbsp; <i>FinalActiveLearningModelArn</i> The Amazon Resource Name (ARN) for the most recent Amazon SageMaker model trained as part of automated data labeling. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputDatasetS3Uri</i> The Amazon S3 bucket location of the manifest file for labeled data. <br/>        """
        pass

    def describe_model(self, ModelName: str) -> DescribeModelOutput:
        """Describes a model that you created using the <code>CreateModel</code> API.

        :param self:
        :param ModelName:  The name of the model.<br/>
        :return: EnableNetworkIsolation:  If <code>True</code>, no inbound or outbound network calls can be made to or from the model container. <note> The Semantic Segmentation built-in algorithm does not support network isolation. </note><br/>ModelArn:  The Amazon Resource Name (ARN) of the model.<br/>CreationTime:  A timestamp that shows when the model was created.<br/>VpcConfig:  A <a>VpcConfig</a> object that specifies the VPC that this model has access to. For more information, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/host-vpc.html">Protect Endpoints by Using an Amazon Virtual Private Cloud</a> :<br/>ExecutionRoleArn:  The Amazon Resource Name (ARN) of the IAM role that you specified for the model.<br/>Containers:  The containers in the inference pipeline.<br/>PrimaryContainer:  The location of the primary inference code, associated artifacts, and custom environment map that the inference code uses when it is deployed in production. :<br/>ModelName:  Name of the Amazon SageMaker model.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModelPackageName</i> The name or Amazon Resource Name (ARN) of the model package to use to create the model.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Environment</i> The environment variables to set in the Docker container. Each key and value in the <code>Environment</code> string to string map can have length of up to 1024. We support up to 16 entries in the map. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModelDataUrl</i> The S3 path where the model artifacts, which result from model training, are stored. This path must point to a single gzip compressed tar archive (.tar.gz suffix). The S3 path is required for Amazon SageMaker built-in algorithms, but not if you use your own algorithms. For more information on built-in algorithms, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/sagemaker-algo-docker-registry-paths.html">Common Parameters</a>.  If you provide a value for this parameter, Amazon SageMaker uses AWS Security Token Service to download model artifacts from the S3 path you provide. AWS STS is activated in your IAM user account by default. If you previously deactivated AWS STS for a region, you need to reactivate AWS STS for that region. For more information, see <a href="http://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_temp_enable-regions.html">Activating and Deactivating AWS STS in an AWS Region</a> in the <i>AWS Identity and Access Management User Guide</i>. <important> If you use a built-in algorithm to create a model, Amazon SageMaker requires that you provide a S3 path to the model artifacts in <code>ModelDataUrl</code>. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Image</i> The Amazon EC2 Container Registry (Amazon ECR) path where inference code is stored. If you are using your own custom algorithm instead of an algorithm provided by Amazon SageMaker, the inference code must meet Amazon SageMaker requirements. Amazon SageMaker supports both <code>registry/repository[:tag]</code> and <code>registry/repository[@digest]</code> image path formats. For more information, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/your-algorithms.html">Using Your Own Algorithms with Amazon SageMaker</a> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContainerHostname</i> This parameter is ignored for models that contain only a <code>PrimaryContainer</code>. When a <code>ContainerDefinition</code> is part of an inference pipeline, the value of ths parameter uniquely identifies the container for the purposes of logging and metrics. For information, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/inference-pipeline-logs-metrics.html">Use Logs and Metrics to Monitor an Inference Pipeline</a>. If you don't specify a value for this parameter for a <code>ContainerDefinition</code> that is part of an inference pipeline, a unique name is automatically assigned based on the position of the <code>ContainerDefinition</code> in the pipeline. If you specify a value for the <code>ContainerHostName</code> for any <code>ContainerDefinition</code> that is part of an inference pipeline, you must specify a value for the <code>ContainerHostName</code> parameter of every <code>ContainerDefinition</code> in that pipeline.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> The ID of the subnets in the VPC to which you want to connect your training job or model.  <note> Amazon EC2 P3 accelerated computing instances are not available in the c/d/e availability zones of region us-east-1. If you want to create endpoints with P3 instances in VPC mode in region us-east-1, create subnets in a/b/f availability zones instead. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The VPC security group IDs, in the form sg-xxxxxxxx. Specify the security groups for the VPC that is specified in the <code>Subnets</code> field.<br/>        """
        pass

    def describe_model_package(self, ModelPackageName: str) -> DescribeModelPackageOutput:
        """Returns a description of the specified model package, which is used to create Amazon SageMaker models or list them on AWS Marketplace. To create models in Amazon SageMaker, buyers can subscribe to model packages listed on AWS Marketplace.

        :param self:
        :param ModelPackageName:  The name of the model package to describe.<br/>
        :return: CertifyForMarketplace:  Whether the model package is certified for listing on AWS Marketplace.<br/>ModelPackageStatusDetails:  Details about the current status of the model package.:<br/>ModelPackageStatus:  The current status of the model package.<br/>ValidationSpecification:  Configurations for one or more transform jobs that Amazon SageMaker runs to test the model package.:<br/>SourceAlgorithmSpecification:  Details about the algorithm that was used to create the model package.:<br/>InferenceSpecification:  Details about inference jobs that can be run with models based on this model package.:<br/>CreationTime:  A timestamp specifying when the model package was created.<br/>ModelPackageDescription:  A brief summary of the model package.<br/>ModelPackageArn:  The Amazon Resource Name (ARN) of the model package.<br/>ModelPackageName:  The name of the model package being described.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedResponseMIMETypes</i> The supported MIME types for the output data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedContentTypes</i> The supported MIME types for the input data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedRealtimeInferenceInstanceTypes</i> A list of the instance types that are used to generate inferences in real-time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportedTransformInstanceTypes</i> A list of the instance types on which a transformation job can be run or on which an endpoint can be deployed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Containers</i> The Amazon ECR registry path of the Docker image that contains the inference code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceAlgorithms</i> A list of the algorithms that were used to create a model package.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ValidationProfiles</i> An array of <code>ModelPackageValidationProfile</code> objects, each of which specifies a batch transform job that Amazon SageMaker runs to validate your model package.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ValidationRole</i> The IAM roles to be used for the validation of the model package.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ImageScanStatuses</i> The status of the scan of the Docker image container for the model package.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ValidationStatuses</i> The validation status of the model package.<br/>        """
        pass

    def describe_notebook_instance(self, NotebookInstanceName: str) -> DescribeNotebookInstanceOutput:
        """Returns information about a notebook instance.

        :param self:
        :param NotebookInstanceName:  The name of the notebook instance that you want information about.<br/>
        :return: RootAccess:  Whether root access is enabled or disabled for users of the notebook instance. <note> Lifecycle configurations need root access to be able to set up a notebook instance. Because of this, lifecycle configurations associated with a notebook instance always run with root access even if you disable root access for users. </note><br/>AdditionalCodeRepositories:  An array of up to three Git repositories associated with the notebook instance. These can be either the names of Git repositories stored as resources in your account, or the URL of Git repositories in <a href="http://docs.aws.amazon.com/codecommit/latest/userguide/welcome.html">AWS CodeCommit</a> or in any other Git repository. These repositories are cloned at the same level as the default repository of your notebook instance. For more information, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/nbi-git-repo.html">Associating Git Repositories with Amazon SageMaker Notebook Instances</a>.<br/>DefaultCodeRepository:  The Git repository associated with the notebook instance as its default code repository. This can be either the name of a Git repository stored as a resource in your account, or the URL of a Git repository in <a href="http://docs.aws.amazon.com/codecommit/latest/userguide/welcome.html">AWS CodeCommit</a> or in any other Git repository. When you open a notebook instance, it opens in the directory that contains this repository. For more information, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/nbi-git-repo.html">Associating Git Repositories with Amazon SageMaker Notebook Instances</a>.<br/>AcceleratorTypes:  A list of the Elastic Inference (EI) instance types associated with this notebook instance. Currently only one EI instance type can be associated with a notebook instance. For more information, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/ei.html">Using Elastic Inference in Amazon SageMaker</a>.<br/>VolumeSizeInGB:  The size, in GB, of the ML storage volume attached to the notebook instance.<br/>DirectInternetAccess:  Describes whether Amazon SageMaker provides internet access to the notebook instance. If this value is set to <i>Disabled</i>, the notebook instance does not have internet access, and cannot connect to Amazon SageMaker training and endpoint services. For more information, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/appendix-additional-considerations.html#appendix-notebook-and-internet-access">Notebook Instances Are Internet-Enabled by Default</a>.<br/>NotebookInstanceLifecycleConfigName:  Returns the name of a notebook instance lifecycle configuration. For information about notebook instance lifestyle configurations, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/notebook-lifecycle-config.html">Step 2.1: (Optional) Customize a Notebook Instance</a> <br/>CreationTime:  A timestamp. Use this parameter to return the time when the notebook instance was created<br/>LastModifiedTime:  A timestamp. Use this parameter to retrieve the time when the notebook instance was last modified. <br/>NetworkInterfaceId:  The network interface IDs that Amazon SageMaker created at the time of creating the instance. <br/>KmsKeyId:  The AWS KMS key ID Amazon SageMaker uses to encrypt data when storing it on the ML storage volume attached to the instance. <br/>RoleArn:  The Amazon Resource Name (ARN) of the IAM role associated with the instance. <br/>SecurityGroups:  The IDs of the VPC security groups.<br/>SubnetId:  The ID of the VPC subnet.<br/>InstanceType:  The type of ML compute instance running on the notebook instance.<br/>Url:  The URL that you use to connect to the Jupyter notebook that is running in your notebook instance. <br/>FailureReason:  If status is <code>Failed</code>, the reason it failed.<br/>NotebookInstanceStatus:  The status of the notebook instance.<br/>NotebookInstanceName:  The name of the Amazon SageMaker notebook instance. <br/>NotebookInstanceArn:  The Amazon Resource Name (ARN) of the notebook instance.<br/>        """
        pass

    def describe_notebook_instance_lifecycle_config(self, NotebookInstanceLifecycleConfigName: str) -> DescribeNotebookInstanceLifecycleConfigOutput:
        """Returns a description of a notebook instance lifecycle configuration. For information about notebook instance lifestyle configurations, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/notebook-lifecycle-config.html">Step 2.1: (Optional) Customize a Notebook Instance</a>.

        :param self:
        :param NotebookInstanceLifecycleConfigName:  The name of the lifecycle configuration to describe.<br/>
        :return: CreationTime:  A timestamp that tells when the lifecycle configuration was created.<br/>LastModifiedTime:  A timestamp that tells when the lifecycle configuration was last modified.<br/>OnStart:  The shell script that runs every time you start a notebook instance, including when you create the notebook instance.<br/>OnCreate:  The shell script that runs only once, when you create a notebook instance.<br/>NotebookInstanceLifecycleConfigName:  The name of the lifecycle configuration.<br/>NotebookInstanceLifecycleConfigArn:  The Amazon Resource Name (ARN) of the lifecycle configuration.<br/>        """
        pass

    def describe_subscribed_workteam(self, WorkteamArn: str) -> DescribeSubscribedWorkteamResponse:
        """Gets information about a work team provided by a vendor. It returns details about the subscription with a vendor in the AWS Marketplace.

        :param self:
        :param WorkteamArn:  The Amazon Resource Name (ARN) of the subscribed work team to describe.<br/>
        :return: SubscribedWorkteam:  A <code>Workteam</code> instance that contains information about the work team.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ListingId</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MarketplaceDescription</i> The description of the vendor from the Amazon Marketplace.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SellerName</i> The name of the vendor in the Amazon Marketplace.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MarketplaceTitle</i> The title of the service provided by the vendor in the Amazon Marketplace.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WorkteamArn</i> The Amazon Resource Name (ARN) of the vendor that you have subscribed.<br/>        """
        pass

    def describe_training_job(self, TrainingJobName: str) -> DescribeTrainingJobResponse:
        """Returns information about a training job.

        :param self:
        :param TrainingJobName:  The name of the training job.<br/>
        :return: EnableInterContainerTrafficEncryption:  To encrypt all communications between ML compute instances in distributed training, choose <code>True</code>. Encryption provides greater security for distributed training, but training might take longer. How long it takes depends on the amount of communication between compute instances, especially if you use a deep learning algorithms in distributed training.<br/>EnableNetworkIsolation:  If you want to allow inbound or outbound network calls, except for calls between peers within a training cluster for distributed training, choose <code>True</code>. If you enable network isolation for training jobs that are configured to use a VPC, Amazon SageMaker downloads and uploads customer data and model artifacts through the specified VPC, but the training container does not have network access. <note> The Semantic Segmentation built-in algorithm does not support network isolation. </note><br/>FinalMetricDataList:  A collection of <code>MetricData</code> objects that specify the names, values, and dates and times that the training algorithm emitted to Amazon CloudWatch.<br/>SecondaryStatusTransitions:  A history of all of the secondary statuses that the training job has transitioned through.<br/>LastModifiedTime:  A timestamp that indicates when the status of the training job was last modified.<br/>TrainingEndTime:  Indicates the time when the training job ends on training instances. You are billed for the time interval between the value of <code>TrainingStartTime</code> and this time. For successful jobs and stopped jobs, this is the time after model artifacts are uploaded. For failed jobs, this is the time when Amazon SageMaker detects a job failure.<br/>TrainingStartTime:  Indicates the time when the training job starts on training instances. You are billed for the time interval between this time and the value of <code>TrainingEndTime</code>. The start time in CloudWatch Logs might be later than this time. The difference is due to the time it takes to download the training data and to the size of the training container.<br/>CreationTime:  A timestamp that indicates when the training job was created.<br/>StoppingCondition:  Specifies a limit to how long a model training job can run. When the job reaches the time limit, Amazon SageMaker ends the training job. Use this API to cap model training costs. To stop a job, Amazon SageMaker sends the algorithm the <code>SIGTERM</code> signal, which delays job termination for 120 seconds. Algorithms can use this 120-second window to save the model artifacts, so the results of training are not lost. :<br/>VpcConfig:  A <a>VpcConfig</a> object that specifies the VPC that this training job has access to. For more information, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/train-vpc.html">Protect Training Jobs by Using an Amazon Virtual Private Cloud</a>.:<br/>ResourceConfig:  Resources, including ML compute instances and ML storage volumes, that are configured for model training. :<br/>OutputDataConfig:  The S3 path where model artifacts that you configured when creating the job are stored. Amazon SageMaker creates subfolders for model artifacts. :<br/>InputDataConfig:  An array of <code>Channel</code> objects that describes each data input channel. <br/>RoleArn:  The AWS Identity and Access Management (IAM) role configured for the training job. <br/>AlgorithmSpecification:  Information about the algorithm used for training, and algorithm metadata. :<br/>HyperParameters:  Algorithm-specific parameters. <br/>FailureReason:  If the training job failed, the reason it failed. <br/>SecondaryStatus:   Provides detailed information about the state of the training job. For detailed information on the secondary status of the training job, see <code>StatusMessage</code> under <a>SecondaryStatusTransition</a>. Amazon SageMaker provides primary statuses and secondary statuses that apply to each of them: <dl> <dt>InProgress</dt> <dd> <ul> <li>  <code>Starting</code> - Starting the training job. </li> <li>  <code>Downloading</code> - An optional stage for algorithms that support <code>File</code> training input mode. It indicates that data is being downloaded to the ML storage volumes. </li> <li>  <code>Training</code> - Training is in progress. </li> <li>  <code>Uploading</code> - Training is complete and the model artifacts are being uploaded to the S3 location. </li> </ul> </dd> <dt>Completed</dt> <dd> <ul> <li>  <code>Completed</code> - The training job has completed. </li> </ul> </dd> <dt>Failed</dt> <dd> <ul> <li>  <code>Failed</code> - The training job has failed. The reason for the failure is returned in the <code>FailureReason</code> field of <code>DescribeTrainingJobResponse</code>. </li> </ul> </dd> <dt>Stopped</dt> <dd> <ul> <li>  <code>MaxRuntimeExceeded</code> - The job stopped because it exceeded the maximum allowed runtime. </li> <li>  <code>Stopped</code> - The training job has stopped. </li> </ul> </dd> <dt>Stopping</dt> <dd> <ul> <li>  <code>Stopping</code> - Stopping the training job. </li> </ul> </dd> </dl> <important> Valid values for <code>SecondaryStatus</code> are subject to change.  </important> We no longer support the following secondary statuses: <ul> <li>  <code>LaunchingMLInstances</code>  </li> <li>  <code>PreparingTrainingStack</code>  </li> <li>  <code>DownloadingTrainingImage</code>  </li> </ul><br/>TrainingJobStatus:  The status of the training job. Amazon SageMaker provides the following training job statuses: <ul> <li>  <code>InProgress</code> - The training is in progress. </li> <li>  <code>Completed</code> - The training job has completed. </li> <li>  <code>Failed</code> - The training job has failed. To see the reason for the failure, see the <code>FailureReason</code> field in the response to a <code>DescribeTrainingJobResponse</code> call. </li> <li>  <code>Stopping</code> - The training job is stopping. </li> <li>  <code>Stopped</code> - The training job has stopped. </li> </ul> For more detailed information, see <code>SecondaryStatus</code>. <br/>ModelArtifacts:  Information about the Amazon S3 location that is configured for storing model artifacts. :<br/>LabelingJobArn:  The Amazon Resource Name (ARN) of the Amazon SageMaker Ground Truth labeling job that created the transform or training job.<br/>TuningJobArn:  The Amazon Resource Name (ARN) of the associated hyperparameter tuning job if the training job was launched by a hyperparameter tuning job.<br/>TrainingJobArn:  The Amazon Resource Name (ARN) of the training job.<br/>TrainingJobName:   Name of the model training job. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3ModelArtifacts</i> The path of the S3 object that contains the model artifacts. For example, <code>s3://bucket-name/keynameprefix/model.tar.gz</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricDefinitions</i> A list of metric definition objects. Each object specifies the metric name and regular expressions used to parse algorithm logs. Amazon SageMaker publishes each metric to Amazon CloudWatch.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingInputMode</i> The input mode that the algorithm supports. For the input modes that Amazon SageMaker algorithms support, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/algos.html">Algorithms</a>. If an algorithm supports the <code>File</code> input mode, Amazon SageMaker downloads the training data from S3 to the provisioned ML storage Volume, and mounts the directory to docker volume for training container. If an algorithm supports the <code>Pipe</code> input mode, Amazon SageMaker streams data directly from S3 to the container.   In File mode, make sure you provision ML storage volume with sufficient capacity to accommodate the data download from S3. In addition to the training data, the ML storage volume also stores the output model. The algorithm container use ML storage volume to also store intermediate information, if any.   For distributed algorithms using File mode, training data is distributed uniformly, and your training duration is predictable if the input data objects size is approximately same. Amazon SageMaker does not split the files any further for model training. If the object sizes are skewed, training won't be optimal as the data distribution is also skewed where one host in a training cluster is overloaded, thus becoming bottleneck in training. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AlgorithmName</i> The name of the algorithm resource to use for the training job. This must be an algorithm resource that you created or subscribe to on AWS Marketplace. If you specify a value for this parameter, you can't specify a value for <code>TrainingImage</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TrainingImage</i> The registry path of the Docker image that contains the training algorithm. For information about docker registry paths for built-in algorithms, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/sagemaker-algo-docker-registry-paths.html">Algorithms Provided by Amazon SageMaker: Common Parameters</a>. Amazon SageMaker supports both <code>registry/repository[:tag]</code> and <code>registry/repository[@digest]</code> image path formats. For more information, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/your-algorithms.html">Using Your Own Algorithms with Amazon SageMaker</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3OutputPath</i> Identifies the S3 path where you want Amazon SageMaker to store the model artifacts. For example, <code>s3://bucket-name/key-name-prefix</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS Key Management Service (AWS KMS) key that Amazon SageMaker uses to encrypt the model artifacts at rest using Amazon S3 server-side encryption. The <code>KmsKeyId</code> can be any of the following formats:  <ul> <li> // KMS Key ID  <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> // Amazon Resource Name (ARN) of a KMS Key  <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> // KMS Key Alias  <code>"alias/ExampleAlias"</code>  </li> <li> // Amazon Resource Name (ARN) of a KMS Key Alias  <code>"arn:aws:kms:us-west-2:111122223333:alias/ExampleAlias"</code>  </li> </ul> If you use a KMS key ID or an alias of your master key, the Amazon SageMaker execution role must include permissions to call <code>kms:Encrypt</code>. If you don't provide a KMS key ID, Amazon SageMaker uses the default KMS key for Amazon S3 for your role's account. Amazon SageMaker uses server-side encryption with KMS-managed keys for <code>OutputDataConfig</code>. If you use a bucket policy with an <code>s3:PutObject</code> permission that only allows objects with server-side encryption, set the condition key of <code>s3:x-amz-server-side-encryption</code> to <code>"aws:kms"</code>. For more information, see <a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/UsingKMSEncryption.html">KMS-Managed Encryption Keys</a> in the <i>Amazon Simple Storage Service Developer Guide.</i>  The KMS key policy must grant permission to the IAM role that you specify in your <code>CreateTrainingJob</code>, <code>CreateTransformJob</code>, or <code>CreateHyperParameterTuningJob</code> requests. For more information, see <a href="http://docs.aws.amazon.com/kms/latest/developerguide/key-policies.html">Using Key Policies in AWS KMS</a> in the <i>AWS Key Management Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeKmsKeyId</i> The AWS Key Management Service (AWS KMS) key that Amazon SageMaker uses to encrypt data on the storage volume attached to the ML compute instance(s) that run the training job. The <code>VolumeKmsKeyId</code> can be any of the following formats: <ul> <li> // KMS Key ID  <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> // Amazon Resource Name (ARN) of a KMS Key  <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeSizeInGB</i> The size of the ML storage volume that you want to provision.  ML storage volumes store model artifacts and incremental states. Training algorithms might also use the ML storage volume for scratch space. If you want to store the training data in the ML storage volume, choose <code>File</code> as the <code>TrainingInputMode</code> in the algorithm specification.  You must specify sufficient ML storage for your scenario.  <note>  Amazon SageMaker supports only the General Purpose SSD (gp2) ML storage volume type.  </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCount</i> The number of ML compute instances to use. For distributed training, provide a value greater than 1. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i> The ML compute instance type. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subnets</i> The ID of the subnets in the VPC to which you want to connect your training job or model.  <note> Amazon EC2 P3 accelerated computing instances are not available in the c/d/e availability zones of region us-east-1. If you want to create endpoints with P3 instances in VPC mode in region us-east-1, create subnets in a/b/f availability zones instead. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecurityGroupIds</i> The VPC security group IDs, in the form sg-xxxxxxxx. Specify the security groups for the VPC that is specified in the <code>Subnets</code> field.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxRuntimeInSeconds</i> The maximum length of time, in seconds, that the training or compilation job can run. If job does not complete during this time, Amazon SageMaker ends the job. If value is not specified, default value is 1 day. The maximum value is 28 days.<br/>        """
        pass

    def describe_transform_job(self, TransformJobName: str) -> DescribeTransformJobResponse:
        """Returns information about a transform job.

        :param self:
        :param TransformJobName:  The name of the transform job that you want to view details of.<br/>
        :return: DataProcessing:  The data structure used to combine the input data and transformed data from the batch transform output into a joined dataset and to store it in an output file. It also contains information on how to filter the input data and the joined dataset. For more information, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/batch-transform-io-join.html">Batch Transform I/O Join</a>.:<br/>LabelingJobArn:  The Amazon Resource Name (ARN) of the Amazon SageMaker Ground Truth labeling job that created the transform or training job.<br/>TransformEndTime:  Indicates when the transform job has been completed, or has stopped or failed. You are billed for the time interval between this time and the value of <code>TransformStartTime</code>.<br/>TransformStartTime:  Indicates when the transform job starts on ML instances. You are billed for the time interval between this time and the value of <code>TransformEndTime</code>.<br/>CreationTime:  A timestamp that shows when the transform Job was created.<br/>TransformResources:  Describes the resources, including ML instance types and ML instance count, to use for the transform job.:<br/>TransformOutput:  Identifies the Amazon S3 location where you want Amazon SageMaker to save the results from the transform job.:<br/>TransformInput:  Describes the dataset to be transformed and the Amazon S3 location where it is stored.:<br/>Environment:  The environment variables to set in the Docker container. We support up to 16 key and values entries in the map.<br/>BatchStrategy:  Specifies the number of records to include in a mini-batch for an HTTP inference request. A <i>record</i> <i/> is a single unit of input data that inference can be made on. For example, a single line in a CSV file is a record.  To enable the batch strategy, you must set <code>SplitType</code> to <code>Line</code>, <code>RecordIO</code>, or <code>TFRecord</code>.<br/>MaxPayloadInMB:  The maximum payload size, in MB, used in the transform job.<br/>MaxConcurrentTransforms:  The maximum number of parallel requests on each instance node that can be launched in a transform job. The default value is 1.<br/>ModelName:  The name of the model used in the transform job.<br/>FailureReason:  If the transform job failed, <code>FailureReason</code> describes why it failed. A transform job creates a log file, which includes error messages, and stores it as an Amazon S3 object. For more information, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/logging-cloudwatch.html">Log Amazon SageMaker Events with Amazon CloudWatch</a>.<br/>TransformJobStatus:  The status of the transform job. If the transform job failed, the reason is returned in the <code>FailureReason</code> field.<br/>TransformJobArn:  The Amazon Resource Name (ARN) of the transform job.<br/>TransformJobName:  The name of the transform job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SplitType</i> The method to use to split the transform job's data files into smaller batches. Splitting is necessary when the total size of each object is too large to fit in a single request. You can also use data splitting to improve performance by processing multiple concurrent mini-batches. The default value for <code>SplitType</code> is <code>None</code>, which indicates that input data files are not split, and request payloads contain the entire contents of an input object. Set the value of this parameter to <code>Line</code> to split records on a newline character boundary. <code>SplitType</code> also supports a number of record-oriented binary data formats. When splitting is enabled, the size of a mini-batch depends on the values of the <code>BatchStrategy</code> and <code>MaxPayloadInMB</code> parameters. When the value of <code>BatchStrategy</code> is <code>MultiRecord</code>, Amazon SageMaker sends the maximum number of records in each request, up to the <code>MaxPayloadInMB</code> limit. If the value of <code>BatchStrategy</code> is <code>SingleRecord</code>, Amazon SageMaker sends individual records in each request. <note> Some data formats represent a record as a binary payload wrapped with extra padding bytes. When splitting is applied to a binary data format, padding is removed if the value of <code>BatchStrategy</code> is set to <code>SingleRecord</code>. Padding is not removed if the value of <code>BatchStrategy</code> is set to <code>MultiRecord</code>. For more information about the RecordIO, see <a href="http://mxnet.io/architecture/note_data_loading.html#data-format">Data Format</a> in the MXNet documentation. For more information about the TFRecord, see <a href="https://www.tensorflow.org/guide/datasets#consuming_tfrecord_data">Consuming TFRecord data</a> in the TensorFlow documentation. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompressionType</i> If your transform data is compressed, specify the compression type. Amazon SageMaker automatically decompresses the data for the transform job accordingly. The default value is <code>None</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContentType</i> The multipurpose internet mail extension (MIME) type of the data. Amazon SageMaker uses the MIME type with each http call to transfer data to the transform job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataSource</i> Describes the location of the channel data, which is, the S3 location of the input data that the model can consume.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DataSource</b>: Describes the location of the channel data, which is, the S3 location of the input data that the model can consume.<br/>The S3 location of the data source that is associated with a channel.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3DataSource</b>: The S3 location of the data source that is associated with a channel.&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyId</i> The AWS Key Management Service (AWS KMS) key that Amazon SageMaker uses to encrypt the model artifacts at rest using Amazon S3 server-side encryption. The <code>KmsKeyId</code> can be any of the following formats:  <ul> <li> // KMS Key ID  <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> // Amazon Resource Name (ARN) of a KMS Key  <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> // KMS Key Alias  <code>"alias/ExampleAlias"</code>  </li> <li> // Amazon Resource Name (ARN) of a KMS Key Alias  <code>"arn:aws:kms:us-west-2:111122223333:alias/ExampleAlias"</code>  </li> </ul> If you don't provide a KMS key ID, Amazon SageMaker uses the default KMS key for Amazon S3 for your role's account. For more information, see <a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/UsingKMSEncryption.html">KMS-Managed Encryption Keys</a> in the <i>Amazon Simple Storage Service Developer Guide.</i>  The KMS key policy must grant permission to the IAM role that you specify in your <code>CreateTramsformJob</code> request. For more information, see <a href="http://docs.aws.amazon.com/kms/latest/developerguide/key-policies.html">Using Key Policies in AWS KMS</a> in the <i>AWS Key Management Service Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssembleWith</i> Defines how to assemble the results of the transform job as a single S3 object. Choose a format that is most convenient to you. To concatenate the results in binary format, specify <code>None</code>. To add a newline character at the end of every transformed record, specify <code>Line</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Accept</i> The MIME type used to specify the output data. Amazon SageMaker uses the MIME type with each http call to transfer data from the transform job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3OutputPath</i> The Amazon S3 path where you want Amazon SageMaker to store the results of the transform job. For example, <code>s3://bucket-name/key-name-prefix</code>. For every S3 object used as input for the transform job, batch transform stores the transformed data with an .<code>out</code> suffix in a corresponding subfolder in the location in the output prefix. For example, for the input data stored at <code>s3://bucket-name/input-name-prefix/dataset01/data.csv</code>, batch transform stores the transformed data at <code>s3://bucket-name/output-name-prefix/input-name-prefix/data.csv.out</code>. Batch transform doesn't upload partially processed objects. For an input S3 object that contains multiple records, it creates an .<code>out</code> file only if the transform job succeeds on the entire file. When the input contains multiple S3 objects, the batch transform job processes the listed S3 objects and uploads only the output for successfully processed objects. If any object fails in the transform job batch transform marks the job as failed to prompt investigation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VolumeKmsKeyId</i> The AWS Key Management Service (AWS KMS) key that Amazon SageMaker uses to encrypt data on the storage volume attached to the ML compute instance(s) that run the batch transform job. The <code>VolumeKmsKeyId</code> can be any of the following formats: <ul> <li> // KMS Key ID  <code>"1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> <li> // Amazon Resource Name (ARN) of a KMS Key  <code>"arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCount</i> The number of ML compute instances to use in the transform job. For distributed transform jobs, specify a value greater than 1. The default value is <code>1</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceType</i> The ML compute instance type for the transform job. If you are using built-in algorithms to transform moderately sized datasets, we recommend using ml.m4.xlarge or <code>ml.m5.large</code>instance types.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JoinSource</i> Specifies the source of the data to join with the transformed data. The valid values are <code>None</code> and <code>Input</code> The default value is <code>None</code> which specifies not to join the input with the transformed data. If you want the batch transform job to join the original input data with the transformed data, set <code>JoinSource</code> to <code>Input</code>. To join input and output, the batch transform job must satisfy the <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/batch-transform-io-join.html#batch-transform-io-join-requirements">Requirements for Using Batch Transform I/O Join</a>. For JSON or JSONLines objects, such as a JSON array, Amazon SageMaker adds the transformed data to the input JSON object in an attribute called <code>SageMakerOutput</code>. The joined result for JSON must be a key-value pair object. If the input is not a key-value pair object, Amazon SageMaker creates a new JSON file. In the new JSON file, and the input data is stored under the <code>SageMakerInput</code> key and the results are stored in <code>SageMakerOutput</code>. For CSV files, Amazon SageMaker combines the transformed data with the input data at the end of the input data and stores it in the output file. The joined data has the joined input data followed by the transformed data and the output is a CSV file. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputFilter</i> A JSONPath expression used to select a portion of the joined dataset to save in the output file for a batch transform job. If you want Amazon SageMaker to store the entire input dataset in the output file, leave the default value, <code>$</code>. If you specify indexes that aren't within the dimension size of the joined dataset, you get an error. Examples: <code>"$"</code>, <code>"$[0,5:]"</code>, <code>"$.['id','SageMakerOutput']"</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InputFilter</i> A JSONPath expression used to select a portion of the input data to pass to the algorithm. Use the <code>InputFilter</code> parameter to exclude fields, such as an ID column, from the input. If you want Amazon SageMaker to pass the entire input dataset to the algorithm, accept the default value <code>$</code>. Examples: <code>"$"</code>, <code>"$[1:]"</code>, <code>"$.features"</code> <br/>        """
        pass

    def describe_workteam(self, WorkteamName: str) -> DescribeWorkteamResponse:
        """Gets information about a specific work team. You can see information such as the create date, the last updated date, membership information, and the work team's Amazon Resource Name (ARN).

        :param self:
        :param WorkteamName:  The name of the work team to return a description of.<br/>
        :return: Workteam:  A <code>Workteam</code> instance that contains information about the work team. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationConfiguration</i> Configures SNS notifications of available or expiring work items for work teams.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NotificationConfiguration</b>: Configures SNS notifications of available or expiring work items for work teams.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdatedDate</i> The date and time that the work team was last updated (timestamp).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateDate</i> The date and time that the work team was created (timestamp).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubDomain</i> The URI of the labeling job's user interface. Workers open this URI to start labeling your data objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of the work team.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductListingIds</i> The Amazon Marketplace identifier for a vendor's work team.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WorkteamArn</i> The Amazon Resource Name (ARN) that identifies the work team.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MemberDefinitions</i> The Amazon Cognito user groups that make up the work team.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WorkteamName</i> The name of the work team.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationTopicArn</i> The ARN for the SNS topic to which notifications should be published.<br/>        """
        pass

    def get_search_suggestions(self, Resource: str, SuggestionQuery: SuggestionQuery=None) -> GetSearchSuggestionsResponse:
        """An auto-complete API for the search functionality in the Amazon SageMaker console. It returns suggestions of possible matches for the property name to use in <code>Search</code> queries. Provides suggestions for <code>HyperParameters</code>, <code>Tags</code>, and <code>Metrics</code>.

        :param self:
        :param SuggestionQuery:  Limits the property names that are included in the response.:<br/>Resource:  The name of the Amazon SageMaker resource to Search for. The only valid <code>Resource</code> value is <code>TrainingJob</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PropertyNameQuery</i> A type of <code>SuggestionQuery</code>. Defines a property name hint. Only property names that match the specified hint are included in the response.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PropertyNameQuery</b>: A type of <code>SuggestionQuery</code>. Defines a property name hint. Only property names that match the specified hint are included in the response.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PropertyNameHint</i> Text that is part of a property's name. The property names of hyperparameter, metric, and tag key names that begin with the specified text in the <code>PropertyNameHint</code>.<br/>
        :return: PropertyNameSuggestions:  A list of property names for a <code>Resource</code> that match a <code>SuggestionQuery</code>.<br/>        """
        pass

    def list_algorithms(self, CreationTimeAfter: CreationTime=None, CreationTimeBefore: CreationTime=None, MaxResults: int=None, NameContains: str=None, NextToken: str=None, SortBy: str=None, SortOrder: str=None) -> ListAlgorithmsOutput:
        """Lists the machine learning algorithms that have been created.

        :param self:
        :param SortOrder:  The sort order for the results. The default is <code>Ascending</code>.<br/>SortBy:  The parameter by which to sort the results. The default is <code>CreationTime</code>.<br/>NextToken:  If the response to a previous <code>ListAlgorithms</code> request was truncated, the response includes a <code>NextToken</code>. To retrieve the next set of algorithms, use the token in the next request.<br/>NameContains:  A string in the algorithm name. This filter returns only algorithms whose name contains the specified string.<br/>MaxResults:  The maximum number of algorithms to return in the response.<br/>CreationTimeBefore:  A filter that returns only algorithms created before the specified time (timestamp).<br/>CreationTimeAfter:  A filter that returns only algorithms created after the specified time (timestamp).<br/>
        :return: NextToken:  If the response is truncated, Amazon SageMaker returns this token. To retrieve the next set of algorithms, use it in the subsequent request.<br/>AlgorithmSummaryList:  &gt;An array of <code>AlgorithmSummary</code> objects, each of which lists an algorithm.<br/>        """
        pass

    def list_code_repositories(self, CreationTimeAfter: CreationTime=None, CreationTimeBefore: CreationTime=None, LastModifiedTimeAfter: Timestamp=None, LastModifiedTimeBefore: Timestamp=None, MaxResults: int=None, NameContains: str=None, NextToken: str=None, SortBy: str=None, SortOrder: str=None) -> ListCodeRepositoriesOutput:
        """Gets a list of the Git repositories in your account.

        :param self:
        :param SortOrder:  The sort order for results. The default is <code>Ascending</code>.<br/>SortBy:  The field to sort results by. The default is <code>Name</code>.<br/>NextToken:  If the result of a <code>ListCodeRepositoriesOutput</code> request was truncated, the response includes a <code>NextToken</code>. To get the next set of Git repositories, use the token in the next request.<br/>NameContains:  A string in the Git repositories name. This filter returns only repositories whose name contains the specified string.<br/>MaxResults:  The maximum number of Git repositories to return in the response.<br/>LastModifiedTimeBefore:  A filter that returns only Git repositories that were last modified before the specified time.<br/>LastModifiedTimeAfter:  A filter that returns only Git repositories that were last modified after the specified time.<br/>CreationTimeBefore:  A filter that returns only Git repositories that were created before the specified time.<br/>CreationTimeAfter:  A filter that returns only Git repositories that were created after the specified time.<br/>
        :return: NextToken:  If the result of a <code>ListCodeRepositoriesOutput</code> request was truncated, the response includes a <code>NextToken</code>. To get the next set of Git repositories, use the token in the next request.<br/>CodeRepositorySummaryList:  Gets a list of summaries of the Git repositories. Each summary specifies the following values for the repository:  <ul> <li> Name </li> <li> Amazon Resource Name (ARN) </li> <li> Creation time </li> <li> Last modified time </li> <li> Configuration information, including the URL location of the repository and the ARN of the AWS Secrets Manager secret that contains the credentials used to access the repository. </li> </ul><br/>        """
        pass

    def list_compilation_jobs(self, NextToken: str=None, MaxResults: int=None, CreationTimeAfter: CreationTime=None, CreationTimeBefore: CreationTime=None, LastModifiedTimeAfter: LastModifiedTime=None, LastModifiedTimeBefore: LastModifiedTime=None, NameContains: str=None, StatusEquals: str=None, SortBy: str=None, SortOrder: str=None) -> ListCompilationJobsResponse:
        """Lists model compilation jobs that satisfy various filters. To create a model compilation job, use <a>CreateCompilationJob</a>. To get information about a particular model compilation job you have created, use <a>DescribeCompilationJob</a>.

        :param self:
        :param SortOrder:  The sort order for results. The default is <code>Ascending</code>.<br/>SortBy:  The field by which to sort results. The default is <code>CreationTime</code>.<br/>StatusEquals:  A filter that retrieves model compilation jobs with a specific <a>DescribeCompilationJobResponse$CompilationJobStatus</a> status.<br/>NameContains:  A filter that returns the model compilation jobs whose name contains a specified string.<br/>LastModifiedTimeBefore:  A filter that returns the model compilation jobs that were modified before a specified time.<br/>LastModifiedTimeAfter:  A filter that returns the model compilation jobs that were modified after a specified time.<br/>CreationTimeBefore:  A filter that returns the model compilation jobs that were created before a specified time.<br/>CreationTimeAfter:  A filter that returns the model compilation jobs that were created after a specified time. <br/>MaxResults:  The maximum number of model compilation jobs to return in the response.<br/>NextToken:  If the result of the previous <code>ListCompilationJobs</code> request was truncated, the response includes a <code>NextToken</code>. To retrieve the next set of model compilation jobs, use the token in the next request.<br/>
        :return: NextToken:  If the response is truncated, Amazon SageMaker returns this <code>NextToken</code>. To retrieve the next set of model compilation jobs, use this token in the next request.<br/>CompilationJobSummaries:  An array of <a>CompilationJobSummary</a> objects, each describing a model compilation job. <br/>        """
        pass

    def list_endpoint_configs(self, SortBy: str=None, SortOrder: str=None, NextToken: str=None, MaxResults: int=None, NameContains: str=None, CreationTimeBefore: Timestamp=None, CreationTimeAfter: Timestamp=None) -> ListEndpointConfigsOutput:
        """Lists endpoint configurations.

        :param self:
        :param CreationTimeAfter:  A filter that returns only endpoint configurations with a creation time greater than or equal to the specified time (timestamp).<br/>CreationTimeBefore:  A filter that returns only endpoint configurations created before the specified time (timestamp).<br/>NameContains:  A string in the endpoint configuration name. This filter returns only endpoint configurations whose name contains the specified string. <br/>MaxResults:  The maximum number of training jobs to return in the response.<br/>NextToken:  If the result of the previous <code>ListEndpointConfig</code> request was truncated, the response includes a <code>NextToken</code>. To retrieve the next set of endpoint configurations, use the token in the next request. <br/>SortOrder:  The sort order for results. The default is <code>Descending</code>.<br/>SortBy:  The field to sort results by. The default is <code>CreationTime</code>.<br/>
        :return: NextToken:   If the response is truncated, Amazon SageMaker returns this token. To retrieve the next set of endpoint configurations, use it in the subsequent request <br/>EndpointConfigs:  An array of endpoint configurations.<br/>        """
        pass

    def list_endpoints(self, SortBy: str=None, SortOrder: str=None, NextToken: str=None, MaxResults: int=None, NameContains: str=None, CreationTimeBefore: Timestamp=None, CreationTimeAfter: Timestamp=None, LastModifiedTimeBefore: Timestamp=None, LastModifiedTimeAfter: Timestamp=None, StatusEquals: str=None) -> ListEndpointsOutput:
        """Lists endpoints.

        :param self:
        :param StatusEquals:   A filter that returns only endpoints with the specified status.<br/>LastModifiedTimeAfter:   A filter that returns only endpoints that were modified after the specified timestamp. <br/>LastModifiedTimeBefore:   A filter that returns only endpoints that were modified before the specified timestamp. <br/>CreationTimeAfter:  A filter that returns only endpoints with a creation time greater than or equal to the specified time (timestamp).<br/>CreationTimeBefore:  A filter that returns only endpoints that were created before the specified time (timestamp).<br/>NameContains:  A string in endpoint names. This filter returns only endpoints whose name contains the specified string.<br/>MaxResults:  The maximum number of endpoints to return in the response.<br/>NextToken:  If the result of a <code>ListEndpoints</code> request was truncated, the response includes a <code>NextToken</code>. To retrieve the next set of endpoints, use the token in the next request.<br/>SortOrder:  The sort order for results. The default is <code>Descending</code>.<br/>SortBy:  Sorts the list of results. The default is <code>CreationTime</code>.<br/>
        :return: NextToken:   If the response is truncated, Amazon SageMaker returns this token. To retrieve the next set of training jobs, use it in the subsequent request. <br/>Endpoints:   An array or endpoint objects. <br/>        """
        pass

    def list_hyper_parameter_tuning_jobs(self, NextToken: str=None, MaxResults: int=None, SortBy: str=None, SortOrder: str=None, NameContains: str=None, CreationTimeAfter: Timestamp=None, CreationTimeBefore: Timestamp=None, LastModifiedTimeAfter: Timestamp=None, LastModifiedTimeBefore: Timestamp=None, StatusEquals: str=None) -> ListHyperParameterTuningJobsResponse:
        """Gets a list of <a>HyperParameterTuningJobSummary</a> objects that describe the hyperparameter tuning jobs launched in your account.

        :param self:
        :param StatusEquals:  A filter that returns only tuning jobs with the specified status.<br/>LastModifiedTimeBefore:  A filter that returns only tuning jobs that were modified before the specified time.<br/>LastModifiedTimeAfter:  A filter that returns only tuning jobs that were modified after the specified time.<br/>CreationTimeBefore:  A filter that returns only tuning jobs that were created before the specified time.<br/>CreationTimeAfter:  A filter that returns only tuning jobs that were created after the specified time.<br/>NameContains:  A string in the tuning job name. This filter returns only tuning jobs whose name contains the specified string.<br/>SortOrder:  The sort order for results. The default is <code>Ascending</code>.<br/>SortBy:  The field to sort results by. The default is <code>Name</code>.<br/>MaxResults:  The maximum number of tuning jobs to return. The default value is 10.<br/>NextToken:  If the result of the previous <code>ListHyperParameterTuningJobs</code> request was truncated, the response includes a <code>NextToken</code>. To retrieve the next set of tuning jobs, use the token in the next request.<br/>
        :return: NextToken:  If the result of this <code>ListHyperParameterTuningJobs</code> request was truncated, the response includes a <code>NextToken</code>. To retrieve the next set of tuning jobs, use the token in the next request.<br/>HyperParameterTuningJobSummaries:  A list of <a>HyperParameterTuningJobSummary</a> objects that describe the tuning jobs that the <code>ListHyperParameterTuningJobs</code> request returned.<br/>        """
        pass

    def list_labeling_jobs(self, CreationTimeAfter: Timestamp=None, CreationTimeBefore: Timestamp=None, LastModifiedTimeAfter: Timestamp=None, LastModifiedTimeBefore: Timestamp=None, MaxResults: int=None, NextToken: str=None, NameContains: str=None, SortBy: str=None, SortOrder: str=None, StatusEquals: str=None) -> ListLabelingJobsResponse:
        """Gets a list of labeling jobs.

        :param self:
        :param StatusEquals:  A filter that retrieves only labeling jobs with a specific status.<br/>SortOrder:  The sort order for results. The default is <code>Ascending</code>.<br/>SortBy:  The field to sort results by. The default is <code>CreationTime</code>.<br/>NameContains:  A string in the labeling job name. This filter returns only labeling jobs whose name contains the specified string.<br/>NextToken:  If the result of the previous <code>ListLabelingJobs</code> request was truncated, the response includes a <code>NextToken</code>. To retrieve the next set of labeling jobs, use the token in the next request.<br/>MaxResults:  The maximum number of labeling jobs to return in each page of the response.<br/>LastModifiedTimeBefore:  A filter that returns only labeling jobs modified before the specified time (timestamp).<br/>LastModifiedTimeAfter:  A filter that returns only labeling jobs modified after the specified time (timestamp).<br/>CreationTimeBefore:  A filter that returns only labeling jobs created before the specified time (timestamp).<br/>CreationTimeAfter:  A filter that returns only labeling jobs created after the specified time (timestamp).<br/>
        :return: NextToken:  If the response is truncated, Amazon SageMaker returns this token. To retrieve the next set of labeling jobs, use it in the subsequent request.<br/>LabelingJobSummaryList:  An array of <code>LabelingJobSummary</code> objects, each describing a labeling job.<br/>        """
        pass

    def list_labeling_jobs_for_workteam(self, WorkteamArn: str, MaxResults: int=None, NextToken: str=None, CreationTimeAfter: Timestamp=None, CreationTimeBefore: Timestamp=None, JobReferenceCodeContains: str=None, SortBy: str=None, SortOrder: str=None) -> ListLabelingJobsForWorkteamResponse:
        """Gets a list of labeling jobs assigned to a specified work team.

        :param self:
        :param SortOrder:  The sort order for results. The default is <code>Ascending</code>.<br/>SortBy:  The field to sort results by. The default is <code>CreationTime</code>.<br/>JobReferenceCodeContains:  A filter the limits jobs to only the ones whose job reference code contains the specified string.<br/>CreationTimeBefore:  A filter that returns only labeling jobs created before the specified time (timestamp).<br/>CreationTimeAfter:  A filter that returns only labeling jobs created after the specified time (timestamp).<br/>NextToken:  If the result of the previous <code>ListLabelingJobsForWorkteam</code> request was truncated, the response includes a <code>NextToken</code>. To retrieve the next set of labeling jobs, use the token in the next request.<br/>MaxResults:  The maximum number of labeling jobs to return in each page of the response.<br/>WorkteamArn:  The Amazon Resource Name (ARN) of the work team for which you want to see labeling jobs for.<br/>
        :return: NextToken:  If the response is truncated, Amazon SageMaker returns this token. To retrieve the next set of labeling jobs, use it in the subsequent request.<br/>LabelingJobSummaryList:  An array of <code>LabelingJobSummary</code> objects, each describing a labeling job.<br/>        """
        pass

    def list_model_packages(self, CreationTimeAfter: CreationTime=None, CreationTimeBefore: CreationTime=None, MaxResults: int=None, NameContains: str=None, NextToken: str=None, SortBy: str=None, SortOrder: str=None) -> ListModelPackagesOutput:
        """Lists the model packages that have been created.

        :param self:
        :param SortOrder:  The sort order for the results. The default is <code>Ascending</code>.<br/>SortBy:  The parameter by which to sort the results. The default is <code>CreationTime</code>.<br/>NextToken:  If the response to a previous <code>ListModelPackages</code> request was truncated, the response includes a <code>NextToken</code>. To retrieve the next set of model packages, use the token in the next request.<br/>NameContains:  A string in the model package name. This filter returns only model packages whose name contains the specified string.<br/>MaxResults:  The maximum number of model packages to return in the response.<br/>CreationTimeBefore:  A filter that returns only model packages created before the specified time (timestamp).<br/>CreationTimeAfter:  A filter that returns only model packages created after the specified time (timestamp).<br/>
        :return: NextToken:  If the response is truncated, Amazon SageMaker returns this token. To retrieve the next set of model packages, use it in the subsequent request.<br/>ModelPackageSummaryList:  An array of <code>ModelPackageSummary</code> objects, each of which lists a model package.<br/>        """
        pass

    def list_models(self, SortBy: str=None, SortOrder: str=None, NextToken: str=None, MaxResults: int=None, NameContains: str=None, CreationTimeBefore: Timestamp=None, CreationTimeAfter: Timestamp=None) -> ListModelsOutput:
        """Lists models created with the <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/API_CreateModel.html">CreateModel</a> API.

        :param self:
        :param CreationTimeAfter:  A filter that returns only models with a creation time greater than or equal to the specified time (timestamp).<br/>CreationTimeBefore:  A filter that returns only models created before the specified time (timestamp).<br/>NameContains:  A string in the training job name. This filter returns only models in the training job whose name contains the specified string.<br/>MaxResults:  The maximum number of models to return in the response.<br/>NextToken:  If the response to a previous <code>ListModels</code> request was truncated, the response includes a <code>NextToken</code>. To retrieve the next set of models, use the token in the next request.<br/>SortOrder:  The sort order for results. The default is <code>Descending</code>.<br/>SortBy:  Sorts the list of results. The default is <code>CreationTime</code>.<br/>
        :return: NextToken:   If the response is truncated, Amazon SageMaker returns this token. To retrieve the next set of models, use it in the subsequent request. <br/>Models:  An array of <code>ModelSummary</code> objects, each of which lists a model.<br/>        """
        pass

    def list_notebook_instance_lifecycle_configs(self, NextToken: str=None, MaxResults: int=None, SortBy: str=None, SortOrder: str=None, NameContains: str=None, CreationTimeBefore: CreationTime=None, CreationTimeAfter: CreationTime=None, LastModifiedTimeBefore: LastModifiedTime=None, LastModifiedTimeAfter: LastModifiedTime=None) -> ListNotebookInstanceLifecycleConfigsOutput:
        """Lists notebook instance lifestyle configurations created with the <a>CreateNotebookInstanceLifecycleConfig</a> API.

        :param self:
        :param LastModifiedTimeAfter:  A filter that returns only lifecycle configurations that were modified after the specified time (timestamp).<br/>LastModifiedTimeBefore:  A filter that returns only lifecycle configurations that were modified before the specified time (timestamp).<br/>CreationTimeAfter:  A filter that returns only lifecycle configurations that were created after the specified time (timestamp).<br/>CreationTimeBefore:  A filter that returns only lifecycle configurations that were created before the specified time (timestamp).<br/>NameContains:  A string in the lifecycle configuration name. This filter returns only lifecycle configurations whose name contains the specified string.<br/>SortOrder:  The sort order for results.<br/>SortBy:  Sorts the list of results. The default is <code>CreationTime</code>.<br/>MaxResults:  The maximum number of lifecycle configurations to return in the response.<br/>NextToken:  If the result of a <code>ListNotebookInstanceLifecycleConfigs</code> request was truncated, the response includes a <code>NextToken</code>. To get the next set of lifecycle configurations, use the token in the next request.<br/>
        :return: NotebookInstanceLifecycleConfigs:  An array of <code>NotebookInstanceLifecycleConfiguration</code> objects, each listing a lifecycle configuration.<br/>NextToken:  If the response is truncated, Amazon SageMaker returns this token. To get the next set of lifecycle configurations, use it in the next request. <br/>        """
        pass

    def list_notebook_instances(self, NextToken: str=None, MaxResults: int=None, SortBy: str=None, SortOrder: str=None, NameContains: str=None, CreationTimeBefore: CreationTime=None, CreationTimeAfter: CreationTime=None, LastModifiedTimeBefore: LastModifiedTime=None, LastModifiedTimeAfter: LastModifiedTime=None, StatusEquals: str=None, NotebookInstanceLifecycleConfigNameContains: str=None, DefaultCodeRepositoryContains: str=None, AdditionalCodeRepositoryEquals: str=None) -> ListNotebookInstancesOutput:
        """Returns a list of the Amazon SageMaker notebook instances in the requester's account in an AWS Region. 

        :param self:
        :param AdditionalCodeRepositoryEquals:  A filter that returns only notebook instances with associated with the specified git repository.<br/>DefaultCodeRepositoryContains:  A string in the name or URL of a Git repository associated with this notebook instance. This filter returns only notebook instances associated with a git repository with a name that contains the specified string.<br/>NotebookInstanceLifecycleConfigNameContains:  A string in the name of a notebook instances lifecycle configuration associated with this notebook instance. This filter returns only notebook instances associated with a lifecycle configuration with a name that contains the specified string.<br/>StatusEquals:  A filter that returns only notebook instances with the specified status.<br/>LastModifiedTimeAfter:  A filter that returns only notebook instances that were modified after the specified time (timestamp).<br/>LastModifiedTimeBefore:  A filter that returns only notebook instances that were modified before the specified time (timestamp).<br/>CreationTimeAfter:  A filter that returns only notebook instances that were created after the specified time (timestamp).<br/>CreationTimeBefore:  A filter that returns only notebook instances that were created before the specified time (timestamp). <br/>NameContains:  A string in the notebook instances' name. This filter returns only notebook instances whose name contains the specified string.<br/>SortOrder:  The sort order for results. <br/>SortBy:  The field to sort results by. The default is <code>Name</code>.<br/>MaxResults:  The maximum number of notebook instances to return.<br/>NextToken:   If the previous call to the <code>ListNotebookInstances</code> is truncated, the response includes a <code>NextToken</code>. You can use this token in your subsequent <code>ListNotebookInstances</code> request to fetch the next set of notebook instances.  <note> You might specify a filter or a sort order in your request. When response is truncated, you must use the same values for the filer and sort order in the next request.  </note><br/>
        :return: NotebookInstances:  An array of <code>NotebookInstanceSummary</code> objects, one for each notebook instance.<br/>NextToken:  If the response to the previous <code>ListNotebookInstances</code> request was truncated, Amazon SageMaker returns this token. To retrieve the next set of notebook instances, use the token in the next request.<br/>        """
        pass

    def list_subscribed_workteams(self, NameContains: str=None, NextToken: str=None, MaxResults: int=None) -> ListSubscribedWorkteamsResponse:
        """Gets a list of the work teams that you are subscribed to in the AWS Marketplace. The list may be empty if no work team satisfies the filter specified in the <code>NameContains</code> parameter.

        :param self:
        :param MaxResults:  The maximum number of work teams to return in each page of the response.<br/>NextToken:  If the result of the previous <code>ListSubscribedWorkteams</code> request was truncated, the response includes a <code>NextToken</code>. To retrieve the next set of labeling jobs, use the token in the next request.<br/>NameContains:  A string in the work team name. This filter returns only work teams whose name contains the specified string.<br/>
        :return: NextToken:  If the response is truncated, Amazon SageMaker returns this token. To retrieve the next set of work teams, use it in the subsequent request.<br/>SubscribedWorkteams:  An array of <code>Workteam</code> objects, each describing a work team.<br/>        """
        pass

    def list_tags(self, ResourceArn: str, NextToken: str=None, MaxResults: int=None) -> ListTagsOutput:
        """Returns the tags for the specified Amazon SageMaker resource.

        :param self:
        :param MaxResults:  Maximum number of tags to return.<br/>NextToken:   If the response to the previous <code>ListTags</code> request is truncated, Amazon SageMaker returns this token. To retrieve the next set of tags, use it in the subsequent request. <br/>ResourceArn:  The Amazon Resource Name (ARN) of the resource whose tags you want to retrieve.<br/>
        :return: NextToken:   If response is truncated, Amazon SageMaker includes a token in the response. You can use this token in your subsequent request to fetch next set of tokens. <br/>Tags:  An array of <code>Tag</code> objects, each with a tag key and a value.<br/>        """
        pass

    def list_training_jobs(self, NextToken: str=None, MaxResults: int=None, CreationTimeAfter: Timestamp=None, CreationTimeBefore: Timestamp=None, LastModifiedTimeAfter: Timestamp=None, LastModifiedTimeBefore: Timestamp=None, NameContains: str=None, StatusEquals: str=None, SortBy: str=None, SortOrder: str=None) -> ListTrainingJobsResponse:
        """Lists training jobs.

        :param self:
        :param SortOrder:  The sort order for results. The default is <code>Ascending</code>.<br/>SortBy:  The field to sort results by. The default is <code>CreationTime</code>.<br/>StatusEquals:  A filter that retrieves only training jobs with a specific status.<br/>NameContains:  A string in the training job name. This filter returns only training jobs whose name contains the specified string.<br/>LastModifiedTimeBefore:  A filter that returns only training jobs modified before the specified time (timestamp).<br/>LastModifiedTimeAfter:  A filter that returns only training jobs modified after the specified time (timestamp).<br/>CreationTimeBefore:  A filter that returns only training jobs created before the specified time (timestamp).<br/>CreationTimeAfter:  A filter that returns only training jobs created after the specified time (timestamp).<br/>MaxResults:  The maximum number of training jobs to return in the response.<br/>NextToken:  If the result of the previous <code>ListTrainingJobs</code> request was truncated, the response includes a <code>NextToken</code>. To retrieve the next set of training jobs, use the token in the next request. <br/>
        :return: NextToken:  If the response is truncated, Amazon SageMaker returns this token. To retrieve the next set of training jobs, use it in the subsequent request.<br/>TrainingJobSummaries:  An array of <code>TrainingJobSummary</code> objects, each listing a training job.<br/>        """
        pass

    def list_training_jobs_for_hyper_parameter_tuning_job(self, HyperParameterTuningJobName: str, NextToken: str=None, MaxResults: int=None, StatusEquals: str=None, SortBy: str=None, SortOrder: str=None) -> ListTrainingJobsForHyperParameterTuningJobResponse:
        """Gets a list of <a>TrainingJobSummary</a> objects that describe the training jobs that a hyperparameter tuning job launched.

        :param self:
        :param SortOrder:  The sort order for results. The default is <code>Ascending</code>.<br/>SortBy:  The field to sort results by. The default is <code>Name</code>. If the value of this field is <code>FinalObjectiveMetricValue</code>, any training jobs that did not return an objective metric are not listed.<br/>StatusEquals:  A filter that returns only training jobs with the specified status.<br/>MaxResults:  The maximum number of training jobs to return. The default value is 10.<br/>NextToken:  If the result of the previous <code>ListTrainingJobsForHyperParameterTuningJob</code> request was truncated, the response includes a <code>NextToken</code>. To retrieve the next set of training jobs, use the token in the next request.<br/>HyperParameterTuningJobName:  The name of the tuning job whose training jobs you want to list.<br/>
        :return: NextToken:  If the result of this <code>ListTrainingJobsForHyperParameterTuningJob</code> request was truncated, the response includes a <code>NextToken</code>. To retrieve the next set of training jobs, use the token in the next request.<br/>TrainingJobSummaries:  A list of <a>TrainingJobSummary</a> objects that describe the training jobs that the <code>ListTrainingJobsForHyperParameterTuningJob</code> request returned.<br/>        """
        pass

    def list_transform_jobs(self, CreationTimeAfter: Timestamp=None, CreationTimeBefore: Timestamp=None, LastModifiedTimeAfter: Timestamp=None, LastModifiedTimeBefore: Timestamp=None, NameContains: str=None, StatusEquals: str=None, SortBy: str=None, SortOrder: str=None, NextToken: str=None, MaxResults: int=None) -> ListTransformJobsResponse:
        """Lists transform jobs.

        :param self:
        :param MaxResults:  The maximum number of transform jobs to return in the response. The default value is <code>10</code>.<br/>NextToken:  If the result of the previous <code>ListTransformJobs</code> request was truncated, the response includes a <code>NextToken</code>. To retrieve the next set of transform jobs, use the token in the next request.<br/>SortOrder:  The sort order for results. The default is <code>Descending</code>.<br/>SortBy:  The field to sort results by. The default is <code>CreationTime</code>.<br/>StatusEquals:  A filter that retrieves only transform jobs with a specific status.<br/>NameContains:  A string in the transform job name. This filter returns only transform jobs whose name contains the specified string.<br/>LastModifiedTimeBefore:  A filter that returns only transform jobs modified before the specified time.<br/>LastModifiedTimeAfter:  A filter that returns only transform jobs modified after the specified time.<br/>CreationTimeBefore:  A filter that returns only transform jobs created before the specified time.<br/>CreationTimeAfter:  A filter that returns only transform jobs created after the specified time.<br/>
        :return: NextToken:  If the response is truncated, Amazon SageMaker returns this token. To retrieve the next set of transform jobs, use it in the next request.<br/>TransformJobSummaries:  An array of <code>TransformJobSummary</code> objects.<br/>        """
        pass

    def list_workteams(self, SortBy: str=None, SortOrder: str=None, NameContains: str=None, NextToken: str=None, MaxResults: int=None) -> ListWorkteamsResponse:
        """Gets a list of work teams that you have defined in a region. The list may be empty if no work team satisfies the filter specified in the <code>NameContains</code> parameter.

        :param self:
        :param MaxResults:  The maximum number of work teams to return in each page of the response.<br/>NextToken:  If the result of the previous <code>ListWorkteams</code> request was truncated, the response includes a <code>NextToken</code>. To retrieve the next set of labeling jobs, use the token in the next request.<br/>NameContains:  A string in the work team's name. This filter returns only work teams whose name contains the specified string.<br/>SortOrder:  The sort order for results. The default is <code>Ascending</code>.<br/>SortBy:  The field to sort results by. The default is <code>CreationTime</code>.<br/>
        :return: NextToken:  If the response is truncated, Amazon SageMaker returns this token. To retrieve the next set of work teams, use it in the subsequent request.<br/>Workteams:  An array of <code>Workteam</code> objects, each describing a work team.<br/>        """
        pass

    def render_ui_template(self, RoleArn: str, Task: RenderableTask, UiTemplate: UiTemplate) -> RenderUiTemplateResponse:
        """Renders the UI template so that you can preview the worker's experience. 

        :param self:
        :param RoleArn:  The Amazon Resource Name (ARN) that has access to the S3 objects that are used by the template.<br/>Task:  A <code>RenderableTask</code> object containing a representative task to render.:<br/>UiTemplate:  A <code>Template</code> object containing the worker UI template to render.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Content</i> The content of the Liquid template for the worker user interface.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Input</i> A JSON object that contains values for the variables defined in the template. It is made available to the template under the substitution variable <code>task.input</code>. For example, if you define a variable <code>task.input.text</code> in your template, you can supply the variable in the JSON object as <code>"text": "sample text"</code>.<br/>
        :return: Errors:  A list of one or more <code>RenderingError</code> objects if any were encountered while rendering the template. If there were no errors, the list is empty.<br/>RenderedContent:  A Liquid template that renders the HTML for the worker UI.<br/>        """
        pass

    def search(self, Resource: str, SearchExpression: SearchExpression=None, SortBy: str=None, SortOrder: str=None, NextToken: str=None, MaxResults: int=None) -> SearchResponse:
        """Finds Amazon SageMaker resources that match a search query. Matching resource objects are returned as a list of <code>SearchResult</code> objects in the response. You can sort the search results by any resource property in a ascending or descending order. You can query against the following value types: numerical, text, Booleans, and timestamps.

        :param self:
        :param MaxResults:  The maximum number of results to return in a <code>SearchResponse</code>.<br/>NextToken:  If more than <code>MaxResults</code> resource objects match the specified <code>SearchExpression</code>, the <code>SearchResponse</code> includes a <code>NextToken</code>. The <code>NextToken</code> can be passed to the next <code>SearchRequest</code> to continue retrieving results for the specified <code>SearchExpression</code> and <code>Sort</code> parameters.<br/>SortOrder:  How <code>SearchResults</code> are ordered. Valid values are <code>Ascending</code> or <code>Descending</code>. The default is <code>Descending</code>.<br/>SortBy:  The name of the resource property used to sort the <code>SearchResults</code>. The default is <code>LastModifiedTime</code>.<br/>SearchExpression:  A Boolean conditional statement. Resource objects must satisfy this condition to be included in search results. You must provide at least one subexpression, filter, or nested filter. The maximum number of recursive <code>SubExpressions</code>, <code>NestedFilters</code>, and <code>Filters</code> that can be included in a <code>SearchExpression</code> object is 50.:<br/>Resource:  The name of the Amazon SageMaker resource to search for. Currently, the only valid <code>Resource</code> value is <code>TrainingJob</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Operator</i> A Boolean operator used to evaluate the search expression. If you want every conditional statement in all lists to be satisfied for the entire search expression to be true, specify <code>And</code>. If only a single conditional statement needs to be true for the entire search expression to be true, specify <code>Or</code>. The default value is <code>And</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubExpressions</i> A list of search expression objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NestedFilters</i> A list of nested filter objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Filters</i> A list of filter objects.<br/>
        :return: NextToken:  If the result of the previous <code>Search</code> request was truncated, the response includes a NextToken. To retrieve the next set of results, use the token in the next request.<br/>Results:  A list of <code>SearchResult</code> objects.<br/>        """
        pass

    def start_notebook_instance(self, NotebookInstanceName: str):
        """Launches an ML compute instance with the latest version of the libraries and attaches your ML storage volume. After configuring the notebook instance, Amazon SageMaker sets the notebook instance status to <code>InService</code>. A notebook instance's status must be <code>InService</code> before you can connect to your Jupyter notebook. 

        :param self:
        :param NotebookInstanceName:  The name of the notebook instance to start.<br/>
        :return:         """
        pass

    def stop_compilation_job(self, CompilationJobName: str):
        """Stops a model compilation job.  To stop a job, Amazon SageMaker sends the algorithm the SIGTERM signal. This gracefully shuts the job down. If the job hasn't stopped, it sends the SIGKILL signal. When it receives a <code>StopCompilationJob</code> request, Amazon SageMaker changes the <a>CompilationJobSummary$CompilationJobStatus</a> of the job to <code>Stopping</code>. After Amazon SageMaker stops the job, it sets the <a>CompilationJobSummary$CompilationJobStatus</a> to <code>Stopped</code>. 

        :param self:
        :param CompilationJobName:  The name of the model compilation job to stop.<br/>
        :return:         """
        pass

    def stop_hyper_parameter_tuning_job(self, HyperParameterTuningJobName: str):
        """Stops a running hyperparameter tuning job and all running training jobs that the tuning job launched. All model artifacts output from the training jobs are stored in Amazon Simple Storage Service (Amazon S3). All data that the training jobs write to Amazon CloudWatch Logs are still available in CloudWatch. After the tuning job moves to the <code>Stopped</code> state, it releases all reserved resources for the tuning job.

        :param self:
        :param HyperParameterTuningJobName:  The name of the tuning job to stop.<br/>
        :return:         """
        pass

    def stop_labeling_job(self, LabelingJobName: str):
        """Stops a running labeling job. A job that is stopped cannot be restarted. Any results obtained before the job is stopped are placed in the Amazon S3 output bucket.

        :param self:
        :param LabelingJobName:  The name of the labeling job to stop.<br/>
        :return:         """
        pass

    def stop_notebook_instance(self, NotebookInstanceName: str):
        """Terminates the ML compute instance. Before terminating the instance, Amazon SageMaker disconnects the ML storage volume from it. Amazon SageMaker preserves the ML storage volume. Amazon SageMaker stops charging you for the ML compute instance when you call <code>StopNotebookInstance</code>. To access data on the ML storage volume for a notebook instance that has been terminated, call the <code>StartNotebookInstance</code> API. <code>StartNotebookInstance</code> launches another ML compute instance, configures it, and attaches the preserved ML storage volume so you can continue your work. 

        :param self:
        :param NotebookInstanceName:  The name of the notebook instance to terminate.<br/>
        :return:         """
        pass

    def stop_training_job(self, TrainingJobName: str):
        """Stops a training job. To stop a job, Amazon SageMaker sends the algorithm the <code>SIGTERM</code> signal, which delays job termination for 120 seconds. Algorithms might use this 120-second window to save the model artifacts, so the results of the training is not lost.  When it receives a <code>StopTrainingJob</code> request, Amazon SageMaker changes the status of the job to <code>Stopping</code>. After Amazon SageMaker stops the job, it sets the status to <code>Stopped</code>.

        :param self:
        :param TrainingJobName:  The name of the training job to stop.<br/>
        :return:         """
        pass

    def stop_transform_job(self, TransformJobName: str):
        """Stops a transform job. When Amazon SageMaker receives a <code>StopTransformJob</code> request, the status of the job changes to <code>Stopping</code>. After Amazon SageMaker stops the job, the status is set to <code>Stopped</code>. When you stop a transform job before it is completed, Amazon SageMaker doesn't store the job's output in Amazon S3.

        :param self:
        :param TransformJobName:  The name of the transform job to stop.<br/>
        :return:         """
        pass

    def update_code_repository(self, CodeRepositoryName: str, GitConfig: GitConfigForUpdate=None) -> UpdateCodeRepositoryOutput:
        """Updates the specified Git repository with the specified values.

        :param self:
        :param GitConfig:  The configuration of the git repository, including the URL and the Amazon Resource Name (ARN) of the AWS Secrets Manager secret that contains the credentials used to access the repository. The secret must have a staging label of <code>AWSCURRENT</code> and must be in the following format:  <code>{"username": <i>UserName</i>, "password": <i>Password</i>}</code> :<br/>CodeRepositoryName:  The name of the Git repository to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecretArn</i> The Amazon Resource Name (ARN) of the AWS Secrets Manager secret that contains the credentials used to access the git repository. The secret must have a staging label of <code>AWSCURRENT</code> and must be in the following format:  <code>{"username": <i>UserName</i>, "password": <i>Password</i>}</code> <br/>
        :return: CodeRepositoryArn:  The ARN of the Git repository.<br/>        """
        pass

    def update_endpoint(self, EndpointConfigName: str, EndpointName: str) -> UpdateEndpointOutput:
        """Deploys the new <code>EndpointConfig</code> specified in the request, switches to using newly created endpoint, and then deletes resources provisioned for the endpoint using the previous <code>EndpointConfig</code> (there is no availability loss).  When Amazon SageMaker receives the request, it sets the endpoint status to <code>Updating</code>. After updating the endpoint, it sets the status to <code>InService</code>. To check the status of an endpoint, use the <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/API_DescribeEndpoint.html">DescribeEndpoint</a> API.  <note> You must not delete an <code>EndpointConfig</code> in use by an endpoint that is live or while the <code>UpdateEndpoint</code> or <code>CreateEndpoint</code> operations are being performed on the endpoint. To update an endpoint, you must create a new <code>EndpointConfig</code>. </note>

        :param self:
        :param EndpointConfigName:  The name of the new endpoint configuration.<br/>EndpointName:  The name of the endpoint whose configuration you want to update.<br/>
        :return: EndpointArn:  The Amazon Resource Name (ARN) of the endpoint.<br/>        """
        pass

    def update_endpoint_weights_and_capacities(self, DesiredWeightsAndCapacities: List[DesiredWeightAndCapacity], EndpointName: str) -> UpdateEndpointWeightsAndCapacitiesOutput:
        """Updates variant weight of one or more variants associated with an existing endpoint, or capacity of one variant associated with an existing endpoint. When it receives the request, Amazon SageMaker sets the endpoint status to <code>Updating</code>. After updating the endpoint, it sets the status to <code>InService</code>. To check the status of an endpoint, use the <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/API_DescribeEndpoint.html">DescribeEndpoint</a> API. 

        :param self:
        :param DesiredWeightsAndCapacities:  An object that provides new capacity and weight values for a variant.<br/>EndpointName:  The name of an existing Amazon SageMaker endpoint.<br/>
        :return: EndpointArn:  The Amazon Resource Name (ARN) of the updated endpoint.<br/>        """
        pass

    def update_notebook_instance(self, NotebookInstanceName: str, InstanceType: str=None, RoleArn: str=None, LifecycleConfigName: str=None, DisassociateLifecycleConfig: DisassociateNotebookInstanceLifecycleConfig=None, VolumeSizeInGB: int=None, DefaultCodeRepository: str=None, AdditionalCodeRepositories: List[CodeRepositoryNameOrUrl]=None, AcceleratorTypes: List[NotebookInstanceAcceleratorType]=None, DisassociateAcceleratorTypes: DisassociateNotebookInstanceAcceleratorTypes=None, DisassociateDefaultCodeRepository: DisassociateDefaultCodeRepository=None, DisassociateAdditionalCodeRepositories: DisassociateAdditionalCodeRepositories=None, RootAccess: str=None) -> UpdateNotebookInstanceOutput:
        """Updates a notebook instance. NotebookInstance updates include upgrading or downgrading the ML compute instance used for your notebook instance to accommodate changes in your workload requirements.

        :param self:
        :param RootAccess:  Whether root access is enabled or disabled for users of the notebook instance. The default value is <code>Enabled</code>. <note> If you set this to <code>Disabled</code>, users don't have root access on the notebook instance, but lifecycle configuration scripts still run with root permissions. </note><br/>DisassociateAdditionalCodeRepositories:  A list of names or URLs of the default Git repositories to remove from this notebook instance. This operation is idempotent. If you specify a Git repository that is not associated with the notebook instance when you call this method, it does not throw an error.<br/>DisassociateDefaultCodeRepository:  The name or URL of the default Git repository to remove from this notebook instance. This operation is idempotent. If you specify a Git repository that is not associated with the notebook instance when you call this method, it does not throw an error.<br/>DisassociateAcceleratorTypes:  A list of the Elastic Inference (EI) instance types to remove from this notebook instance. This operation is idempotent. If you specify an accelerator type that is not associated with the notebook instance when you call this method, it does not throw an error.<br/>AcceleratorTypes:  A list of the Elastic Inference (EI) instance types to associate with this notebook instance. Currently only one EI instance type can be associated with a notebook instance. For more information, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/ei.html">Using Elastic Inference in Amazon SageMaker</a>.<br/>AdditionalCodeRepositories:  An array of up to three Git repositories to associate with the notebook instance. These can be either the names of Git repositories stored as resources in your account, or the URL of Git repositories in <a href="http://docs.aws.amazon.com/codecommit/latest/userguide/welcome.html">AWS CodeCommit</a> or in any other Git repository. These repositories are cloned at the same level as the default repository of your notebook instance. For more information, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/nbi-git-repo.html">Associating Git Repositories with Amazon SageMaker Notebook Instances</a>.<br/>DefaultCodeRepository:  The Git repository to associate with the notebook instance as its default code repository. This can be either the name of a Git repository stored as a resource in your account, or the URL of a Git repository in <a href="http://docs.aws.amazon.com/codecommit/latest/userguide/welcome.html">AWS CodeCommit</a> or in any other Git repository. When you open a notebook instance, it opens in the directory that contains this repository. For more information, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/nbi-git-repo.html">Associating Git Repositories with Amazon SageMaker Notebook Instances</a>.<br/>VolumeSizeInGB:  The size, in GB, of the ML storage volume to attach to the notebook instance. The default value is 5 GB. ML storage volumes are encrypted, so Amazon SageMaker can't determine the amount of available free space on the volume. Because of this, you can increase the volume size when you update a notebook instance, but you can't decrease the volume size. If you want to decrease the size of the ML storage volume in use, create a new notebook instance with the desired size.<br/>DisassociateLifecycleConfig:  Set to <code>true</code> to remove the notebook instance lifecycle configuration currently associated with the notebook instance. This operation is idempotent. If you specify a lifecycle configuration that is not associated with the notebook instance when you call this method, it does not throw an error.<br/>LifecycleConfigName:  The name of a lifecycle configuration to associate with the notebook instance. For information about lifestyle configurations, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/notebook-lifecycle-config.html">Step 2.1: (Optional) Customize a Notebook Instance</a>.<br/>RoleArn:  The Amazon Resource Name (ARN) of the IAM role that Amazon SageMaker can assume to access the notebook instance. For more information, see <a href="https://docs.aws.amazon.com/sagemaker/latest/dg/sagemaker-roles.html">Amazon SageMaker Roles</a>.  <note> To be able to pass this role to Amazon SageMaker, the caller of this API must have the <code>iam:PassRole</code> permission. </note><br/>InstanceType:  The Amazon ML compute instance type.<br/>NotebookInstanceName:  The name of the notebook instance to update.<br/>
        :return:         """
        pass

    def update_notebook_instance_lifecycle_config(self, NotebookInstanceLifecycleConfigName: str, OnCreate: List[NotebookInstanceLifecycleHook]=None, OnStart: List[NotebookInstanceLifecycleHook]=None) -> UpdateNotebookInstanceLifecycleConfigOutput:
        """Updates a notebook instance lifecycle configuration created with the <a>CreateNotebookInstanceLifecycleConfig</a> API.

        :param self:
        :param OnStart:  The shell script that runs every time you start a notebook instance, including when you create the notebook instance.<br/>OnCreate:  The shell script that runs only once, when you create a notebook instance<br/>NotebookInstanceLifecycleConfigName:  The name of the lifecycle configuration.<br/>
        :return:         """
        pass

    def update_workteam(self, WorkteamName: str, MemberDefinitions: List[MemberDefinition]=None, Description: str=None, NotificationConfiguration: NotificationConfiguration=None) -> UpdateWorkteamResponse:
        """Updates an existing work team with new member definitions or description.

        :param self:
        :param NotificationConfiguration:  Configures SNS topic notifications for available or expiring work items:<br/>Description:  An updated description for the work team.<br/>MemberDefinitions:  A list of <code>MemberDefinition</code> objects that contain the updated work team members.<br/>WorkteamName:  The name of the work team to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationTopicArn</i> The ARN for the SNS topic to which notifications should be published.<br/>
        :return: Workteam:  A <code>Workteam</code> object that describes the updated work team.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationConfiguration</i> Configures SNS notifications of available or expiring work items for work teams.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NotificationConfiguration</b>: Configures SNS notifications of available or expiring work items for work teams.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdatedDate</i> The date and time that the work team was last updated (timestamp).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateDate</i> The date and time that the work team was created (timestamp).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubDomain</i> The URI of the labeling job's user interface. Workers open this URI to start labeling your data objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of the work team.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductListingIds</i> The Amazon Marketplace identifier for a vendor's work team.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WorkteamArn</i> The Amazon Resource Name (ARN) that identifies the work team.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MemberDefinitions</i> The Amazon Cognito user groups that make up the work team.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WorkteamName</i> The name of the work team.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationTopicArn</i> The ARN for the SNS topic to which notifications should be published.<br/>        """
        pass



class SageMakerRuntime(BaseClient):
    """ The Amazon SageMaker runtime API. """




    class InvokeEndpointInput(Mapping):
        pass
    
    class InvokeEndpointOutput(Mapping):
        pass
    
    class BodyBlob(object):
        pass
    
    def invoke_endpoint(self, Body: BodyBlob, EndpointName: str, ContentType: str=None, Accept: str=None, CustomAttributes: str=None) -> InvokeEndpointOutput:
        """After you deploy a model into production using Amazon SageMaker hosting services, your client applications use this API to get inferences from the model hosted at the specified endpoint.  For an overview of Amazon SageMaker, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/how-it-works.html">How It Works</a>.  Amazon SageMaker strips all POST headers except those supported by the API. Amazon SageMaker might add additional headers. You should not rely on the behavior of headers outside those enumerated in the request syntax.  Cals to <code>InvokeEndpoint</code> are authenticated by using AWS Signature Version 4. For information, see <a href="http://docs.aws.amazon.com/AmazonS3/latest/API/sig-v4-authenticating-requests.html">Authenticating Requests (AWS Signature Version 4)</a> in the <i>Amazon S3 API Reference</i>. <note> Endpoints are scoped to an individual account, and are not public. The URL does not contain the account ID, but Amazon SageMaker determines the account ID from the authentication token that is supplied by the caller. </note>

        :param self:
        :param CustomAttributes:  <p/><br/>Accept:  The desired MIME type of the inference in the response.<br/>ContentType:  The MIME type of the input data in the request body.<br/>Body:  Provides input data, in the format specified in the <code>ContentType</code> request header. Amazon SageMaker passes all of the data in the body to the model.  For information about the format of the request body, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/cdf-inference.html">Common Data Formats—Inference</a>.<br/>EndpointName:  The name of the endpoint that you specified when you created the endpoint using the <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/API_CreateEndpoint.html">CreateEndpoint</a> API. <br/>
        :return: CustomAttributes:  <p/><br/>InvokedProductionVariant:  Identifies the production variant that was invoked.<br/>ContentType:  The MIME type of the inference returned in the response body.<br/>Body:  Includes the inference provided by the model. For information about the format of the response body, see <a href="http://docs.aws.amazon.com/sagemaker/latest/dg/cdf-inference.html">Common Data Formats—Inference</a>.<br/>        """
        pass



class SimpleDB(BaseClient):
    """Amazon SimpleDB is a web service providing the core database functions of data indexing and querying in the cloud. By offloading the time and effort associated with building and operating a web-scale database, SimpleDB provides developers the freedom to focus on application development.  A traditional, clustered relational database requires a sizable upfront capital outlay, is complex to design, and often requires extensive and repetitive database administration. Amazon SimpleDB is dramatically simpler, requiring no schema, automatically indexing your data and providing a simple API for storage and access. This approach eliminates the administrative burden of data modeling, index maintenance, and performance tuning. Developers gain access to this functionality within Amazon's proven computing environment, are able to scale instantly, and pay only for what they use.   Visit <a href="http://aws.amazon.com/simpledb/">http://aws.amazon.com/simpledb/</a> for more information. """


    class ListDomainsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_domains` for valid parameters"""
            pass
    class SelectPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `select` for valid parameters"""
            pass


    class BatchDeleteAttributesRequest(Mapping):
        pass
    
    class DeletableItem(Mapping):
        pass
    
    class BatchPutAttributesRequest(Mapping):
        pass
    
    class ReplaceableItem(Mapping):
        pass
    
    class CreateDomainRequest(Mapping):
        pass
    
    class DeleteAttributesRequest(Mapping):
        pass
    
    class Attribute(Mapping):
        pass
    
    class UpdateCondition(Mapping):
        pass
    
    class DeleteDomainRequest(Mapping):
        pass
    
    class DomainMetadataRequest(Mapping):
        pass
    
    class DomainMetadataResult(Mapping):
        pass
    
    class GetAttributesRequest(Mapping):
        pass
    
    class GetAttributesResult(Mapping):
        pass
    
    class String(object):
        pass
    
    class Boolean(object):
        pass
    
    class ListDomainsRequest(Mapping):
        pass
    
    class ListDomainsResult(Mapping):
        pass
    
    class PutAttributesRequest(Mapping):
        pass
    
    class ReplaceableAttribute(Mapping):
        pass
    
    class SelectRequest(Mapping):
        pass
    
    class SelectResult(Mapping):
        pass
    
    def batch_delete_attributes(self, Items: List[DeletableItem], DomainName: str):
        """ Performs multiple DeleteAttributes operations in a single call, which reduces round trips and latencies. This enables Amazon SimpleDB to optimize requests, which generally yields better throughput.   The following limitations are enforced for this operation: <ul> <li>1 MB request size</li> <li>25 item limit per BatchDeleteAttributes operation</li> </ul> 

        :param self:
        :param Items:  A list of items on which to perform the operation.<br/>DomainName:  The name of the domain in which the attributes are being deleted.<br/>
        :return:         """
        pass

    def batch_put_attributes(self, Items: List[ReplaceableItem], DomainName: str):
        """ The <code>BatchPutAttributes</code> operation creates or replaces attributes within one or more items. By using this operation, the client can perform multiple <a>PutAttribute</a> operation with a single call. This helps yield savings in round trips and latencies, enabling Amazon SimpleDB to optimize requests and generally produce better throughput.   The client may specify the item name with the <code>Item.X.ItemName</code> parameter. The client may specify new attributes using a combination of the <code>Item.X.Attribute.Y.Name</code> and <code>Item.X.Attribute.Y.Value</code> parameters. The client may specify the first attribute for the first item using the parameters <code>Item.0.Attribute.0.Name</code> and <code>Item.0.Attribute.0.Value</code>, and for the second attribute for the first item by the parameters <code>Item.0.Attribute.1.Name</code> and <code>Item.0.Attribute.1.Value</code>, and so on.   Attributes are uniquely identified within an item by their name/value combination. For example, a single item can have the attributes <code>{ "first_name", "first_value" }</code> and <code>{ "first_name", "second_value" }</code>. However, it cannot have two attribute instances where both the <code>Item.X.Attribute.Y.Name</code> and <code>Item.X.Attribute.Y.Value</code> are the same.   Optionally, the requester can supply the <code>Replace</code> parameter for each individual value. Setting this value to <code>true</code> will cause the new attribute values to replace the existing attribute values. For example, if an item <code>I</code> has the attributes <code>{ 'a', '1' }, { 'b', '2'}</code> and <code>{ 'b', '3' }</code> and the requester does a BatchPutAttributes of <code>{'I', 'b', '4' }</code> with the Replace parameter set to true, the final attributes of the item will be <code>{ 'a', '1' }</code> and <code>{ 'b', '4' }</code>, replacing the previous values of the 'b' attribute with the new value.  <important> This operation is vulnerable to exceeding the maximum URL size when making a REST request using the HTTP GET method. This operation does not support conditions using <code>Expected.X.Name</code>, <code>Expected.X.Value</code>, or <code>Expected.X.Exists</code>. </important>  You can execute multiple <code>BatchPutAttributes</code> operations and other operations in parallel. However, large numbers of concurrent <code>BatchPutAttributes</code> calls can result in Service Unavailable (503) responses.   The following limitations are enforced for this operation: <ul> <li>256 attribute name-value pairs per item</li> <li>1 MB request size</li> <li>1 billion attributes per domain</li> <li>10 GB of total user data storage per domain</li> <li>25 item limit per <code>BatchPutAttributes</code> operation</li> </ul> 

        :param self:
        :param Items:  A list of items on which to perform the operation.<br/>DomainName:  The name of the domain in which the attributes are being stored.<br/>
        :return:         """
        pass

    def create_domain(self, DomainName: str):
        """ The <code>CreateDomain</code> operation creates a new domain. The domain name should be unique among the domains associated with the Access Key ID provided in the request. The <code>CreateDomain</code> operation may take 10 or more seconds to complete.   The client can create up to 100 domains per account.   If the client requires additional domains, go to <a href="http://aws.amazon.com/contact-us/simpledb-limit-request/"> http://aws.amazon.com/contact-us/simpledb-limit-request/</a>. 

        :param self:
        :param DomainName:  The name of the domain to create. The name can range between 3 and 255 characters and can contain the following characters: a-z, A-Z, 0-9, '_', '-', and '.'.<br/>
        :return:         """
        pass

    def delete_attributes(self, ItemName: str, DomainName: str, Attributes: List[Attribute]=None, Expected: UpdateCondition=None):
        """ Deletes one or more attributes associated with an item. If all attributes of the item are deleted, the item is deleted.   <code>DeleteAttributes</code> is an idempotent operation; running it multiple times on the same item or attribute does not result in an error response.   Because Amazon SimpleDB makes multiple copies of item data and uses an eventual consistency update model, performing a <a>GetAttributes</a> or <a>Select</a> operation (read) immediately after a <code>DeleteAttributes</code> or <a>PutAttributes</a> operation (write) might not return updated item data. 

        :param self:
        :param Expected:  The update condition which, if specified, determines whether the specified attributes will be deleted or not. The update condition must be satisfied in order for this request to be processed and the attributes to be deleted.:<br/>Attributes:  A list of Attributes. Similar to columns on a spreadsheet, attributes represent categories of data that can be assigned to items.<br/>ItemName:  The name of the item. Similar to rows on a spreadsheet, items represent individual objects that contain one or more value-attribute pairs.<br/>DomainName:  The name of the domain in which to perform the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Exists</i> A value specifying whether or not the specified attribute must exist with the specified value in order for the update condition to be satisfied. Specify <code>true</code> if the attribute must exist for the update condition to be satisfied. Specify <code>false</code> if the attribute should not exist in order for the update condition to be satisfied.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The value of an attribute. This value can only be specified when the <code>Exists</code> parameter is equal to <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the attribute involved in the condition.<br/>
        :return:         """
        pass

    def delete_domain(self, DomainName: str):
        """ The <code>DeleteDomain</code> operation deletes a domain. Any items (and their attributes) in the domain are deleted as well. The <code>DeleteDomain</code> operation might take 10 or more seconds to complete. 

        :param self:
        :param DomainName:  The name of the domain to delete.<br/>
        :return:         """
        pass

    def domain_metadata(self, DomainName: str) -> DomainMetadataResult:
        """ Returns information about the domain, including when the domain was created, the number of items and attributes in the domain, and the size of the attribute names and values. 

        :param self:
        :param DomainName:  The name of the domain for which to display the metadata of.<br/>
        :return: Timestamp:  The data and time when metadata was calculated, in Epoch (UNIX) seconds.<br/>AttributeValuesSizeBytes:  The total size of all attribute values in the domain, in bytes.<br/>AttributeValueCount:  The number of all attribute name/value pairs in the domain.<br/>AttributeNamesSizeBytes:  The total size of all unique attribute names in the domain, in bytes.<br/>AttributeNameCount:  The number of unique attribute names in the domain.<br/>ItemNamesSizeBytes:  The total size of all item names in the domain, in bytes.<br/>ItemCount:  The number of all items in the domain.<br/>        """
        pass

    def get_attributes(self, ItemName: str, DomainName: str, AttributeNames: List[String]=None, ConsistentRead: Boolean=None) -> GetAttributesResult:
        """ Returns all of the attributes associated with the specified item. Optionally, the attributes returned can be limited to one or more attributes by specifying an attribute name parameter.   If the item does not exist on the replica that was accessed for this operation, an empty set is returned. The system does not return an error as it cannot guarantee the item does not exist on other replicas. 

        :param self:
        :param ConsistentRead:  Determines whether or not strong consistency should be enforced when data is read from SimpleDB. If <code>true</code>, any data previously written to SimpleDB will be returned. Otherwise, results will be consistent eventually, and the client may not see data that was written immediately before your read.<br/>AttributeNames:  The names of the attributes.<br/>ItemName:  The name of the item.<br/>DomainName:  The name of the domain in which to perform the operation.<br/>
        :return: Attributes:  The list of attributes returned by the operation.<br/>        """
        pass

    def list_domains(self, MaxNumberOfDomains: int=None, NextToken: str=None) -> ListDomainsResult:
        """ The <code>ListDomains</code> operation lists all domains associated with the Access Key ID. It returns domain names up to the limit set by <a href="#MaxNumberOfDomains">MaxNumberOfDomains</a>. A <a href="#NextToken">NextToken</a> is returned if there are more than <code>MaxNumberOfDomains</code> domains. Calling <code>ListDomains</code> successive times with the <code>NextToken</code> provided by the operation returns up to <code>MaxNumberOfDomains</code> more domain names with each successive operation call. 

        :param self:
        :param NextToken:  A string informing Amazon SimpleDB where to start the next list of domain names.<br/>MaxNumberOfDomains:  The maximum number of domain names you want returned. The range is 1 to 100. The default setting is 100.<br/>
        :return: NextToken:  An opaque token indicating that there are more domains than the specified <code>MaxNumberOfDomains</code> still available.<br/>DomainNames:  A list of domain names that match the expression.<br/>        """
        pass

    def put_attributes(self, Attributes: List[ReplaceableAttribute], ItemName: str, DomainName: str, Expected: UpdateCondition=None):
        """ The PutAttributes operation creates or replaces attributes in an item. The client may specify new attributes using a combination of the <code>Attribute.X.Name</code> and <code>Attribute.X.Value</code> parameters. The client specifies the first attribute by the parameters <code>Attribute.0.Name</code> and <code>Attribute.0.Value</code>, the second attribute by the parameters <code>Attribute.1.Name</code> and <code>Attribute.1.Value</code>, and so on.   Attributes are uniquely identified in an item by their name/value combination. For example, a single item can have the attributes <code>{ "first_name", "first_value" }</code> and <code>{ "first_name", second_value" }</code>. However, it cannot have two attribute instances where both the <code>Attribute.X.Name</code> and <code>Attribute.X.Value</code> are the same.   Optionally, the requestor can supply the <code>Replace</code> parameter for each individual attribute. Setting this value to <code>true</code> causes the new attribute value to replace the existing attribute value(s). For example, if an item has the attributes <code>{ 'a', '1' }</code>, <code>{ 'b', '2'}</code> and <code>{ 'b', '3' }</code> and the requestor calls <code>PutAttributes</code> using the attributes <code>{ 'b', '4' }</code> with the <code>Replace</code> parameter set to true, the final attributes of the item are changed to <code>{ 'a', '1' }</code> and <code>{ 'b', '4' }</code>, which replaces the previous values of the 'b' attribute with the new value.   You cannot specify an empty string as an attribute name.   Because Amazon SimpleDB makes multiple copies of client data and uses an eventual consistency update model, an immediate <a>GetAttributes</a> or <a>Select</a> operation (read) immediately after a <a>PutAttributes</a> or <a>DeleteAttributes</a> operation (write) might not return the updated data.   The following limitations are enforced for this operation: <ul> <li>256 total attribute name-value pairs per item</li> <li>One billion attributes per domain</li> <li>10 GB of total user data storage per domain</li> </ul> 

        :param self:
        :param Expected:  The update condition which, if specified, determines whether the specified attributes will be updated or not. The update condition must be satisfied in order for this request to be processed and the attributes to be updated.:<br/>Attributes:  The list of attributes.<br/>ItemName:  The name of the item.<br/>DomainName:  The name of the domain in which to perform the operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Exists</i> A value specifying whether or not the specified attribute must exist with the specified value in order for the update condition to be satisfied. Specify <code>true</code> if the attribute must exist for the update condition to be satisfied. Specify <code>false</code> if the attribute should not exist in order for the update condition to be satisfied.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The value of an attribute. This value can only be specified when the <code>Exists</code> parameter is equal to <code>true</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the attribute involved in the condition.<br/>
        :return:         """
        pass

    def select(self, SelectExpression: str, NextToken: str=None, ConsistentRead: Boolean=None) -> SelectResult:
        """ The <code>Select</code> operation returns a set of attributes for <code>ItemNames</code> that match the select expression. <code>Select</code> is similar to the standard SQL SELECT statement.   The total size of the response cannot exceed 1 MB in total size. Amazon SimpleDB automatically adjusts the number of items returned per page to enforce this limit. For example, if the client asks to retrieve 2500 items, but each individual item is 10 kB in size, the system returns 100 items and an appropriate <code>NextToken</code> so the client can access the next page of results.   For information on how to construct select expressions, see Using Select to Create Amazon SimpleDB Queries in the Developer Guide. 

        :param self:
        :param ConsistentRead:  Determines whether or not strong consistency should be enforced when data is read from SimpleDB. If <code>true</code>, any data previously written to SimpleDB will be returned. Otherwise, results will be consistent eventually, and the client may not see data that was written immediately before your read.<br/>NextToken:  A string informing Amazon SimpleDB where to start the next list of <code>ItemNames</code>.<br/>SelectExpression:  The expression used to query the domain.<br/>
        :return: NextToken:  An opaque token indicating that more items than <code>MaxNumberOfItems</code> were matched, the response size exceeded 1 megabyte, or the execution time exceeded 5 seconds.<br/>Items:  A list of items that match the select expression.<br/>        """
        pass



class SecretsManager(BaseClient):
    """<fullname>AWS Secrets Manager API Reference</fullname> AWS Secrets Manager is a web service that enables you to store, manage, and retrieve, secrets. This guide provides descriptions of the Secrets Manager API. For more information about using this service, see the <a href="https://docs.aws.amazon.com/secretsmanager/latest/userguide/introduction.html">AWS Secrets Manager User Guide</a>.  <b>API Version</b>  This version of the Secrets Manager API Reference documents the Secrets Manager API version 2017-10-17. <note> As an alternative to using the API directly, you can use one of the AWS SDKs, which consist of libraries and sample code for various programming languages and platforms (such as Java, Ruby, .NET, iOS, and Android). The SDKs provide a convenient way to create programmatic access to AWS Secrets Manager. For example, the SDKs take care of cryptographically signing requests, managing errors, and retrying requests automatically. For more information about the AWS SDKs, including how to download and install them, see <a href="http://aws.amazon.com/tools/">Tools for Amazon Web Services</a>. </note> We recommend that you use the AWS SDKs to make programmatic API calls to Secrets Manager. However, you also can use the Secrets Manager HTTP Query API to make direct calls to the Secrets Manager web service. To learn more about the Secrets Manager HTTP Query API, see <a href="https://docs.aws.amazon.com/secretsmanager/latest/userguide/query-requests.html">Making Query Requests</a> in the <i>AWS Secrets Manager User Guide</i>.  Secrets Manager supports GET and POST requests for all actions. That is, the API doesn't require you to use GET for some actions and POST for others. However, GET requests are subject to the limitation size of a URL. Therefore, for operations that require larger sizes, use a POST request.  <b>Support and Feedback for AWS Secrets Manager</b>  We welcome your feedback. Send your comments to <a href="mailto:awssecretsmanager-feedback@amazon.com">awssecretsmanager-feedback@amazon.com</a>, or post your feedback and questions in the <a href="http://forums.aws.amazon.com/forum.jspa?forumID=296">AWS Secrets Manager Discussion Forum</a>. For more information about the AWS Discussion Forums, see <a href="http://forums.aws.amazon.com/help.jspa">Forums Help</a>.  <b>How examples are presented</b>  The JSON that AWS Secrets Manager expects as your request parameters and that the service returns as a response to HTTP query requests are single, long strings without line breaks or white space formatting. The JSON shown in the examples is formatted with both line breaks and white space to improve readability. When example input parameters would also result in long strings that extend beyond the screen, we insert line breaks to enhance readability. You should always submit the input as a single JSON text string.  <b>Logging API Requests</b>  AWS Secrets Manager supports AWS CloudTrail, a service that records AWS API calls for your AWS account and delivers log files to an Amazon S3 bucket. By using information that's collected by AWS CloudTrail, you can determine which requests were successfully made to Secrets Manager, who made the request, when it was made, and so on. For more about AWS Secrets Manager and its support for AWS CloudTrail, see <a href="http://docs.aws.amazon.com/secretsmanager/latest/userguide/monitoring.html#monitoring_cloudtrail">Logging AWS Secrets Manager Events with AWS CloudTrail</a> in the <i>AWS Secrets Manager User Guide</i>. To learn more about CloudTrail, including how to turn it on and find your log files, see the <a href="https://docs.aws.amazon.com/awscloudtrail/latest/userguide/what_is_cloud_trail_top_level.html">AWS CloudTrail User Guide</a>."""


    class ListSecretsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_secrets` for valid parameters"""
            pass


    class CancelRotateSecretRequest(Mapping):
        pass
    
    class CancelRotateSecretResponse(Mapping):
        pass
    
    class CreateSecretRequest(Mapping):
        pass
    
    class CreateSecretResponse(Mapping):
        pass
    
    class SecretBinaryType(object):
        pass
    
    class Tag(Mapping):
        pass
    
    class DeleteResourcePolicyRequest(Mapping):
        pass
    
    class DeleteResourcePolicyResponse(Mapping):
        pass
    
    class DeleteSecretRequest(Mapping):
        pass
    
    class DeleteSecretResponse(Mapping):
        pass
    
    class RecoveryWindowInDaysType(object):
        pass
    
    class BooleanType(object):
        pass
    
    class DescribeSecretRequest(Mapping):
        pass
    
    class DescribeSecretResponse(Mapping):
        pass
    
    class GetRandomPasswordRequest(Mapping):
        pass
    
    class GetRandomPasswordResponse(Mapping):
        pass
    
    class PasswordLengthType(object):
        pass
    
    class ExcludeNumbersType(object):
        pass
    
    class ExcludePunctuationType(object):
        pass
    
    class ExcludeUppercaseType(object):
        pass
    
    class ExcludeLowercaseType(object):
        pass
    
    class IncludeSpaceType(object):
        pass
    
    class RequireEachIncludedTypeType(object):
        pass
    
    class GetResourcePolicyRequest(Mapping):
        pass
    
    class GetResourcePolicyResponse(Mapping):
        pass
    
    class GetSecretValueRequest(Mapping):
        pass
    
    class GetSecretValueResponse(Mapping):
        pass
    
    class ListSecretVersionIdsRequest(Mapping):
        pass
    
    class ListSecretVersionIdsResponse(Mapping):
        pass
    
    class ListSecretsRequest(Mapping):
        pass
    
    class ListSecretsResponse(Mapping):
        pass
    
    class PutResourcePolicyRequest(Mapping):
        pass
    
    class PutResourcePolicyResponse(Mapping):
        pass
    
    class PutSecretValueRequest(Mapping):
        pass
    
    class PutSecretValueResponse(Mapping):
        pass
    
    class SecretVersionStageType(object):
        pass
    
    class RestoreSecretRequest(Mapping):
        pass
    
    class RestoreSecretResponse(Mapping):
        pass
    
    class RotateSecretRequest(Mapping):
        pass
    
    class RotateSecretResponse(Mapping):
        pass
    
    class RotationRulesType(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class TagKeyType(object):
        pass
    
    class UpdateSecretRequest(Mapping):
        pass
    
    class UpdateSecretResponse(Mapping):
        pass
    
    class UpdateSecretVersionStageRequest(Mapping):
        pass
    
    class UpdateSecretVersionStageResponse(Mapping):
        pass
    
    def cancel_rotate_secret(self, SecretId: str) -> CancelRotateSecretResponse:
        """Disables automatic scheduled rotation and cancels the rotation of a secret if one is currently in progress. To re-enable scheduled rotation, call <a>RotateSecret</a> with <code>AutomaticallyRotateAfterDays</code> set to a value greater than 0. This will immediately rotate your secret and then enable the automatic schedule. <note> If you cancel a rotation that is in progress, it can leave the <code>VersionStage</code> labels in an unexpected state. Depending on what step of the rotation was in progress, you might need to remove the staging label <code>AWSPENDING</code> from the partially created version, specified by the <code>VersionId</code> response value. You should also evaluate the partially rotated new version to see if it should be deleted, which you can do by removing all staging labels from the new version's <code>VersionStage</code> field. </note> To successfully start a rotation, the staging label <code>AWSPENDING</code> must be in one of the following states: <ul> <li> Not be attached to any version at all </li> <li> Attached to the same version as the staging label <code>AWSCURRENT</code>  </li> </ul> If the staging label <code>AWSPENDING</code> is attached to a different version than the version with <code>AWSCURRENT</code> then the attempt to rotate fails.  <b>Minimum permissions</b>  To run this command, you must have the following permissions: <ul> <li> secretsmanager:CancelRotateSecret </li> </ul>  <b>Related operations</b>  <ul> <li> To configure rotation for a secret or to manually trigger a rotation, use <a>RotateSecret</a>. </li> <li> To get the rotation configuration details for a secret, use <a>DescribeSecret</a>. </li> <li> To list all of the currently available secrets, use <a>ListSecrets</a>. </li> <li> To list all of the versions currently associated with a secret, use <a>ListSecretVersionIds</a>. </li> </ul>

        :param self:
        :param SecretId:  Specifies the secret for which you want to cancel a rotation request. You can specify either the Amazon Resource Name (ARN) or the friendly name of the secret. <note> If you specify an ARN, we generally recommend that you specify a complete ARN. You can specify a partial ARN too—for example, if you don’t include the final hyphen and six random characters that Secrets Manager adds at the end of the ARN when you created the secret. A partial ARN match can work as long as it uniquely matches only one secret. However, if your secret has a name that ends in a hyphen followed by six characters (before Secrets Manager adds the hyphen and six characters to the ARN) and you try to use that as a partial ARN, then those characters cause Secrets Manager to assume that you’re specifying a complete ARN. This confusion can cause unexpected results. To avoid this situation, we recommend that you don’t create secret names that end with a hyphen followed by six characters. </note><br/>
        :return: VersionId:  The unique identifier of the version of the secret that was created during the rotation. This version might not be complete, and should be evaluated for possible deletion. At the very least, you should remove the <code>VersionStage</code> value <code>AWSPENDING</code> to enable this version to be deleted. Failing to clean up a cancelled rotation can block you from successfully starting future rotations.<br/>Name:  The friendly name of the secret for which rotation was canceled.<br/>ARN:  The ARN of the secret for which rotation was canceled.<br/>        """
        pass

    def create_secret(self, Name: str, ClientRequestToken: str=None, Description: str=None, KmsKeyId: str=None, SecretBinary: SecretBinaryType=None, SecretString: str=None, Tags: List[Tag]=None) -> CreateSecretResponse:
        """Creates a new secret. A secret in Secrets Manager consists of both the protected secret data and the important information needed to manage the secret. Secrets Manager stores the encrypted secret data in one of a collection of "versions" associated with the secret. Each version contains a copy of the encrypted secret data. Each version is associated with one or more "staging labels" that identify where the version is in the rotation cycle. The <code>SecretVersionsToStages</code> field of the secret contains the mapping of staging labels to the active versions of the secret. Versions without a staging label are considered deprecated and are not included in the list. You provide the secret data to be encrypted by putting text in either the <code>SecretString</code> parameter or binary data in the <code>SecretBinary</code> parameter, but not both. If you include <code>SecretString</code> or <code>SecretBinary</code> then Secrets Manager also creates an initial secret version and automatically attaches the staging label <code>AWSCURRENT</code> to the new version. <note> <ul> <li> If you call an operation that needs to encrypt or decrypt the <code>SecretString</code> or <code>SecretBinary</code> for a secret in the same account as the calling user and that secret doesn't specify a AWS KMS encryption key, Secrets Manager uses the account's default AWS managed customer master key (CMK) with the alias <code>aws/secretsmanager</code>. If this key doesn't already exist in your account then Secrets Manager creates it for you automatically. All users and roles in the same AWS account automatically have access to use the default CMK. Note that if an Secrets Manager API call results in AWS having to create the account's AWS-managed CMK, it can result in a one-time significant delay in returning the result. </li> <li> If the secret is in a different AWS account from the credentials calling an API that requires encryption or decryption of the secret value then you must create and use a custom AWS KMS CMK because you can't access the default CMK for the account using credentials from a different AWS account. Store the ARN of the CMK in the secret when you create the secret or when you update it by including it in the <code>KMSKeyId</code>. If you call an API that must encrypt or decrypt <code>SecretString</code> or <code>SecretBinary</code> using credentials from a different account then the AWS KMS key policy must grant cross-account access to that other account's user or role for both the kms:GenerateDataKey and kms:Decrypt operations. </li> </ul> </note>    <b>Minimum permissions</b>  To run this command, you must have the following permissions: <ul> <li> secretsmanager:CreateSecret </li> <li> kms:GenerateDataKey - needed only if you use a customer-managed AWS KMS key to encrypt the secret. You do not need this permission to use the account's default AWS managed CMK for Secrets Manager. </li> <li> kms:Decrypt - needed only if you use a customer-managed AWS KMS key to encrypt the secret. You do not need this permission to use the account's default AWS managed CMK for Secrets Manager. </li> <li> secretsmanager:TagResource - needed only if you include the <code>Tags</code> parameter.  </li> </ul>  <b>Related operations</b>  <ul> <li> To delete a secret, use <a>DeleteSecret</a>. </li> <li> To modify an existing secret, use <a>UpdateSecret</a>. </li> <li> To create a new version of a secret, use <a>PutSecretValue</a>. </li> <li> To retrieve the encrypted secure string and secure binary values, use <a>GetSecretValue</a>. </li> <li> To retrieve all other details for a secret, use <a>DescribeSecret</a>. This does not include the encrypted secure string and secure binary values. </li> <li> To retrieve the list of secret versions associated with the current secret, use <a>DescribeSecret</a> and examine the <code>SecretVersionsToStages</code> response value. </li> </ul>

        :param self:
        :param Tags:  (Optional) Specifies a list of user-defined tags that are attached to the secret. Each tag is a "Key" and "Value" pair of strings. This operation only appends tags to the existing list of tags. To remove tags, you must use <a>UntagResource</a>. <important> <ul> <li> Secrets Manager tag key names are case sensitive. A tag with the key "ABC" is a different tag from one with key "abc". </li> <li> If you check tags in IAM policy <code>Condition</code> elements as part of your security strategy, then adding or removing a tag can change permissions. If the successful completion of this operation would result in you losing your permissions for this secret, then this operation is blocked and returns an <code>Access Denied</code> error. </li> </ul> </important> This parameter requires a JSON text string argument. For information on how to format a JSON parameter for the various command line tool environments, see <a href="https://docs.aws.amazon.com/cli/latest/userguide/cli-using-param.html#cli-using-param-json">Using JSON for Parameters</a> in the <i>AWS CLI User Guide</i>. For example:  <code>[{"Key":"CostCenter","Value":"12345"},{"Key":"environment","Value":"production"}]</code>  If your command-line tool or SDK requires quotation marks around the parameter, you should use single quotes to avoid confusion with the double quotes required in the JSON text.  The following basic restrictions apply to tags: <ul> <li> Maximum number of tags per secret—50 </li> <li> Maximum key length—127 Unicode characters in UTF-8 </li> <li> Maximum value length—255 Unicode characters in UTF-8 </li> <li> Tag keys and values are case sensitive. </li> <li> Do not use the <code>aws:</code> prefix in your tag names or values because it is reserved for AWS use. You can't edit or delete tag names or values with this prefix. Tags with this prefix do not count against your tags per secret limit. </li> <li> If your tagging schema will be used across multiple services and resources, remember that other services might have restrictions on allowed characters. Generally allowed characters are: letters, spaces, and numbers representable in UTF-8, plus the following special characters: + - = . _ : / @. </li> </ul><br/>SecretString:  (Optional) Specifies text data that you want to encrypt and store in this new version of the secret. Either <code>SecretString</code> or <code>SecretBinary</code> must have a value, but not both. They cannot both be empty. If you create a secret by using the Secrets Manager console then Secrets Manager puts the protected secret text in only the <code>SecretString</code> parameter. The Secrets Manager console stores the information as a JSON structure of key/value pairs that the Lambda rotation function knows how to parse. For storing multiple values, we recommend that you use a JSON text string argument and specify key/value pairs. For information on how to format a JSON parameter for the various command line tool environments, see <a href="https://docs.aws.amazon.com/cli/latest/userguide/cli-using-param.html#cli-using-param-json">Using JSON for Parameters</a> in the <i>AWS CLI User Guide</i>. For example:  <code>[{"username":"bob"},{"password":"abc123xyz456"}]</code>  If your command-line tool or SDK requires quotation marks around the parameter, you should use single quotes to avoid confusion with the double quotes required in the JSON text. <br/>SecretBinary:  (Optional) Specifies binary data that you want to encrypt and store in the new version of the secret. To use this parameter in the command-line tools, we recommend that you store your binary data in a file and then use the appropriate technique for your tool to pass the contents of the file as a parameter. Either <code>SecretString</code> or <code>SecretBinary</code> must have a value, but not both. They cannot both be empty. This parameter is not available using the Secrets Manager console. It can be accessed only by using the AWS CLI or one of the AWS SDKs.<br/>KmsKeyId:  (Optional) Specifies the ARN, Key ID, or alias of the AWS KMS customer master key (CMK) to be used to encrypt the <code>SecretString</code> or <code>SecretBinary</code> values in the versions stored in this secret. You can specify any of the supported ways to identify a AWS KMS key ID. If you need to reference a CMK in a different account, you can use only the key ARN or the alias ARN. If you don't specify this value, then Secrets Manager defaults to using the AWS account's default CMK (the one named <code>aws/secretsmanager</code>). If a AWS KMS CMK with that name doesn't yet exist, then Secrets Manager creates it for you automatically the first time it needs to encrypt a version's <code>SecretString</code> or <code>SecretBinary</code> fields. <important> You can use the account's default CMK to encrypt and decrypt only if you call this operation using credentials from the same account that owns the secret. If the secret is in a different account, then you must create a custom CMK and specify the ARN in this field.  </important><br/>Description:  (Optional) Specifies a user-provided description of the secret.<br/>ClientRequestToken:  (Optional) If you include <code>SecretString</code> or <code>SecretBinary</code>, then an initial version is created as part of the secret, and this parameter specifies a unique identifier for the new version.  <note> If you use the AWS CLI or one of the AWS SDK to call this operation, then you can leave this parameter empty. The CLI or SDK generates a random UUID for you and includes it as the value for this parameter in the request. If you don't use the SDK and instead generate a raw HTTP request to the Secrets Manager service endpoint, then you must generate a <code>ClientRequestToken</code> yourself for the new version and include that value in the request. </note> This value helps ensure idempotency. Secrets Manager uses this value to prevent the accidental creation of duplicate versions if there are failures and retries during a rotation. We recommend that you generate a <a href="https://wikipedia.org/wiki/Universally_unique_identifier">UUID-type</a> value to ensure uniqueness of your versions within the specified secret.  <ul> <li> If the <code>ClientRequestToken</code> value isn't already associated with a version of the secret then a new version of the secret is created.  </li> <li> If a version with this value already exists and that version's <code>SecretString</code> and <code>SecretBinary</code> values are the same as those in the request, then the request is ignored (the operation is idempotent). </li> <li> If a version with this value already exists and that version's <code>SecretString</code> and <code>SecretBinary</code> values are different from those in the request then the request fails because you cannot modify an existing version. Instead, use <a>PutSecretValue</a> to create a new version. </li> </ul> This value becomes the <code>VersionId</code> of the new version.<br/>Name:  Specifies the friendly name of the new secret. The secret name must be ASCII letters, digits, or the following characters : /_+=.@- <note> Don't end your secret name with a hyphen followed by six characters. If you do so, you risk confusion and unexpected results when searching for a secret by partial ARN. This is because Secrets Manager automatically adds a hyphen and six random characters at the end of the ARN. </note><br/>
        :return: VersionId:  The unique identifier that's associated with the version of the secret you just created.<br/>Name:  The friendly name of the secret that you just created.<br/>ARN:  The Amazon Resource Name (ARN) of the secret that you just created. <note> Secrets Manager automatically adds several random characters to the name at the end of the ARN when you initially create a secret. This affects only the ARN and not the actual friendly name. This ensures that if you create a new secret with the same name as an old secret that you previously deleted, then users with access to the old secret <i>don't</i> automatically get access to the new secret because the ARNs are different. </note><br/>        """
        pass

    def delete_resource_policy(self, SecretId: str) -> DeleteResourcePolicyResponse:
        """Deletes the resource-based permission policy that's attached to the secret.  <b>Minimum permissions</b>  To run this command, you must have the following permissions: <ul> <li> secretsmanager:DeleteResourcePolicy </li> </ul>  <b>Related operations</b>  <ul> <li> To attach a resource policy to a secret, use <a>PutResourcePolicy</a>. </li> <li> To retrieve the current resource-based policy that's attached to a secret, use <a>GetResourcePolicy</a>. </li> <li> To list all of the currently available secrets, use <a>ListSecrets</a>. </li> </ul>

        :param self:
        :param SecretId:  Specifies the secret that you want to delete the attached resource-based policy for. You can specify either the Amazon Resource Name (ARN) or the friendly name of the secret. <note> If you specify an ARN, we generally recommend that you specify a complete ARN. You can specify a partial ARN too—for example, if you don’t include the final hyphen and six random characters that Secrets Manager adds at the end of the ARN when you created the secret. A partial ARN match can work as long as it uniquely matches only one secret. However, if your secret has a name that ends in a hyphen followed by six characters (before Secrets Manager adds the hyphen and six characters to the ARN) and you try to use that as a partial ARN, then those characters cause Secrets Manager to assume that you’re specifying a complete ARN. This confusion can cause unexpected results. To avoid this situation, we recommend that you don’t create secret names that end with a hyphen followed by six characters. </note><br/>
        :return: Name:  The friendly name of the secret that the resource-based policy was deleted for.<br/>ARN:  The ARN of the secret that the resource-based policy was deleted for.<br/>        """
        pass

    def delete_secret(self, SecretId: str, RecoveryWindowInDays: RecoveryWindowInDaysType=None, ForceDeleteWithoutRecovery: BooleanType=None) -> DeleteSecretResponse:
        """Deletes an entire secret and all of its versions. You can optionally include a recovery window during which you can restore the secret. If you don't specify a recovery window value, the operation defaults to 30 days. Secrets Manager attaches a <code>DeletionDate</code> stamp to the secret that specifies the end of the recovery window. At the end of the recovery window, Secrets Manager deletes the secret permanently. At any time before recovery window ends, you can use <a>RestoreSecret</a> to remove the <code>DeletionDate</code> and cancel the deletion of the secret. You cannot access the encrypted secret information in any secret that is scheduled for deletion. If you need to access that information, you must cancel the deletion with <a>RestoreSecret</a> and then retrieve the information. <note> <ul> <li> There is no explicit operation to delete a version of a secret. Instead, remove all staging labels from the <code>VersionStage</code> field of a version. That marks the version as deprecated and allows Secrets Manager to delete it as needed. Versions that do not have any staging labels do not show up in <a>ListSecretVersionIds</a> unless you specify <code>IncludeDeprecated</code>. </li> <li> The permanent secret deletion at the end of the waiting period is performed as a background task with low priority. There is no guarantee of a specific time after the recovery window for the actual delete operation to occur. </li> </ul> </note>  <b>Minimum permissions</b>  To run this command, you must have the following permissions: <ul> <li> secretsmanager:DeleteSecret </li> </ul>  <b>Related operations</b>  <ul> <li> To create a secret, use <a>CreateSecret</a>. </li> <li> To cancel deletion of a version of a secret before the recovery window has expired, use <a>RestoreSecret</a>. </li> </ul>

        :param self:
        :param ForceDeleteWithoutRecovery:  (Optional) Specifies that the secret is to be deleted without any recovery window. You can't use both this parameter and the <code>RecoveryWindowInDays</code> parameter in the same API call. An asynchronous background process performs the actual deletion, so there can be a short delay before the operation completes. If you write code to delete and then immediately recreate a secret with the same name, ensure that your code includes appropriate back off and retry logic. <important> Use this parameter with caution. This parameter causes the operation to skip the normal waiting period before the permanent deletion that AWS would normally impose with the <code>RecoveryWindowInDays</code> parameter. If you delete a secret with the <code>ForceDeleteWithouRecovery</code> parameter, then you have no opportunity to recover the secret. It is permanently lost. </important><br/>RecoveryWindowInDays:  (Optional) Specifies the number of days that Secrets Manager waits before it can delete the secret. You can't use both this parameter and the <code>ForceDeleteWithoutRecovery</code> parameter in the same API call. This value can range from 7 to 30 days. The default value is 30.<br/>SecretId:  Specifies the secret that you want to delete. You can specify either the Amazon Resource Name (ARN) or the friendly name of the secret. <note> If you specify an ARN, we generally recommend that you specify a complete ARN. You can specify a partial ARN too—for example, if you don’t include the final hyphen and six random characters that Secrets Manager adds at the end of the ARN when you created the secret. A partial ARN match can work as long as it uniquely matches only one secret. However, if your secret has a name that ends in a hyphen followed by six characters (before Secrets Manager adds the hyphen and six characters to the ARN) and you try to use that as a partial ARN, then those characters cause Secrets Manager to assume that you’re specifying a complete ARN. This confusion can cause unexpected results. To avoid this situation, we recommend that you don’t create secret names that end with a hyphen followed by six characters. </note><br/>
        :return: DeletionDate:  The date and time after which this secret can be deleted by Secrets Manager and can no longer be restored. This value is the date and time of the delete request plus the number of days specified in <code>RecoveryWindowInDays</code>.<br/>Name:  The friendly name of the secret that is now scheduled for deletion.<br/>ARN:  The ARN of the secret that is now scheduled for deletion.<br/>        """
        pass

    def describe_secret(self, SecretId: str) -> DescribeSecretResponse:
        """Retrieves the details of a secret. It does not include the encrypted fields. Only those fields that are populated with a value are returned in the response.   <b>Minimum permissions</b>  To run this command, you must have the following permissions: <ul> <li> secretsmanager:DescribeSecret </li> </ul>  <b>Related operations</b>  <ul> <li> To create a secret, use <a>CreateSecret</a>. </li> <li> To modify a secret, use <a>UpdateSecret</a>. </li> <li> To retrieve the encrypted secret information in a version of the secret, use <a>GetSecretValue</a>. </li> <li> To list all of the secrets in the AWS account, use <a>ListSecrets</a>. </li> </ul>

        :param self:
        :param SecretId:  The identifier of the secret whose details you want to retrieve. You can specify either the Amazon Resource Name (ARN) or the friendly name of the secret. <note> If you specify an ARN, we generally recommend that you specify a complete ARN. You can specify a partial ARN too—for example, if you don’t include the final hyphen and six random characters that Secrets Manager adds at the end of the ARN when you created the secret. A partial ARN match can work as long as it uniquely matches only one secret. However, if your secret has a name that ends in a hyphen followed by six characters (before Secrets Manager adds the hyphen and six characters to the ARN) and you try to use that as a partial ARN, then those characters cause Secrets Manager to assume that you’re specifying a complete ARN. This confusion can cause unexpected results. To avoid this situation, we recommend that you don’t create secret names that end with a hyphen followed by six characters. </note><br/>
        :return: VersionIdsToStages:  A list of all of the currently assigned <code>VersionStage</code> staging labels and the <code>VersionId</code> that each is attached to. Staging labels are used to keep track of the different versions during the rotation process. <note> A version that does not have any staging labels attached is considered deprecated and subject to deletion. Such versions are not included in this list. </note><br/>Tags:  The list of user-defined tags that are associated with the secret. To add tags to a secret, use <a>TagResource</a>. To remove tags, use <a>UntagResource</a>.<br/>DeletedDate:  This value exists if the secret is scheduled for deletion. Some time after the specified date and time, Secrets Manager deletes the secret and all of its versions. If a secret is scheduled for deletion, then its details, including the encrypted secret information, is not accessible. To cancel a scheduled deletion and restore access, use <a>RestoreSecret</a>.<br/>LastAccessedDate:  The last date that this secret was accessed. This value is truncated to midnight of the date and therefore shows only the date, not the time.<br/>LastChangedDate:  The last date and time that this secret was modified in any way.<br/>LastRotatedDate:  The most recent date and time that the Secrets Manager rotation process was successfully completed. This value is null if the secret has never rotated.<br/>RotationRules:  A structure that contains the rotation configuration for this secret.:<br/>RotationLambdaARN:  The ARN of a Lambda function that's invoked by Secrets Manager to rotate the secret either automatically per the schedule or manually by a call to <code>RotateSecret</code>.<br/>RotationEnabled:  Specifies whether automatic rotation is enabled for this secret. To enable rotation, use <a>RotateSecret</a> with <code>AutomaticallyRotateAfterDays</code> set to a value greater than 0. To disable rotation, use <a>CancelRotateSecret</a>.<br/>KmsKeyId:  The ARN or alias of the AWS KMS customer master key (CMK) that's used to encrypt the <code>SecretString</code> or <code>SecretBinary</code> fields in each version of the secret. If you don't provide a key, then Secrets Manager defaults to encrypting the secret fields with the default AWS KMS CMK (the one named <code>awssecretsmanager</code>) for this account.<br/>Description:  The user-provided description of the secret.<br/>Name:  The user-provided friendly name of the secret.<br/>ARN:  The ARN of the secret.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomaticallyAfterDays</i> Specifies the number of days between automatic scheduled rotations of the secret. Secrets Manager schedules the next rotation when the previous one is complete. Secrets Manager schedules the date by adding the rotation interval (number of days) to the actual date of the last rotation. The service chooses the hour within that 24-hour date window randomly. The minute is also chosen somewhat randomly, but weighted towards the top of the hour and influenced by a variety of factors that help distribute load.<br/>        """
        pass

    def get_random_password(self, PasswordLength: PasswordLengthType=None, ExcludeCharacters: str=None, ExcludeNumbers: ExcludeNumbersType=None, ExcludePunctuation: ExcludePunctuationType=None, ExcludeUppercase: ExcludeUppercaseType=None, ExcludeLowercase: ExcludeLowercaseType=None, IncludeSpace: IncludeSpaceType=None, RequireEachIncludedType: RequireEachIncludedTypeType=None) -> GetRandomPasswordResponse:
        """Generates a random password of the specified complexity. This operation is intended for use in the Lambda rotation function. Per best practice, we recommend that you specify the maximum length and include every character type that the system you are generating a password for can support.  <b>Minimum permissions</b>  To run this command, you must have the following permissions: <ul> <li> secretsmanager:GetRandomPassword </li> </ul>

        :param self:
        :param RequireEachIncludedType:  A boolean value that specifies whether the generated password must include at least one of every allowed character type. The default value is <code>True</code> and the operation requires at least one of every character type.<br/>IncludeSpace:  Specifies that the generated password can include the space character. The default if you do not include this switch parameter is that the space character is not included.<br/>ExcludeLowercase:  Specifies that the generated password should not include lowercase letters. The default if you do not include this switch parameter is that lowercase letters can be included.<br/>ExcludeUppercase:  Specifies that the generated password should not include uppercase letters. The default if you do not include this switch parameter is that uppercase letters can be included.<br/>ExcludePunctuation:  Specifies that the generated password should not include punctuation characters. The default if you do not include this switch parameter is that punctuation characters can be included. The following are the punctuation characters that <i>can</i> be included in the generated password if you don't explicitly exclude them with <code>ExcludeCharacters</code> or <code>ExcludePunctuation</code>:  <code>! " # $ % &amp; ' ( ) * + , - . / : ; &lt; = &gt; ? @ [ \ ] ^ _ ` { | } ~</code> <br/>ExcludeNumbers:  Specifies that the generated password should not include digits. The default if you do not include this switch parameter is that digits can be included.<br/>ExcludeCharacters:  A string that includes characters that should not be included in the generated password. The default is that all characters from the included sets can be used.<br/>PasswordLength:  The desired length of the generated password. The default value if you do not include this parameter is 32 characters.<br/>
        :return: RandomPassword:  A string with the generated password.<br/>        """
        pass

    def get_resource_policy(self, SecretId: str) -> GetResourcePolicyResponse:
        """Retrieves the JSON text of the resource-based policy document that's attached to the specified secret. The JSON request string input and response output are shown formatted with white space and line breaks for better readability. Submit your input as a single line JSON string.  <b>Minimum permissions</b>  To run this command, you must have the following permissions: <ul> <li> secretsmanager:GetResourcePolicy </li> </ul>  <b>Related operations</b>  <ul> <li> To attach a resource policy to a secret, use <a>PutResourcePolicy</a>. </li> <li> To delete the resource-based policy that's attached to a secret, use <a>DeleteResourcePolicy</a>. </li> <li> To list all of the currently available secrets, use <a>ListSecrets</a>. </li> </ul>

        :param self:
        :param SecretId:  Specifies the secret that you want to retrieve the attached resource-based policy for. You can specify either the Amazon Resource Name (ARN) or the friendly name of the secret. <note> If you specify an ARN, we generally recommend that you specify a complete ARN. You can specify a partial ARN too—for example, if you don’t include the final hyphen and six random characters that Secrets Manager adds at the end of the ARN when you created the secret. A partial ARN match can work as long as it uniquely matches only one secret. However, if your secret has a name that ends in a hyphen followed by six characters (before Secrets Manager adds the hyphen and six characters to the ARN) and you try to use that as a partial ARN, then those characters cause Secrets Manager to assume that you’re specifying a complete ARN. This confusion can cause unexpected results. To avoid this situation, we recommend that you don’t create secret names that end with a hyphen followed by six characters. </note><br/>
        :return: ResourcePolicy:  A JSON-formatted string that describes the permissions that are associated with the attached secret. These permissions are combined with any permissions that are associated with the user or role that attempts to access this secret. The combined permissions specify who can access the secret and what actions they can perform. For more information, see <a href="http://docs.aws.amazon.com/secretsmanager/latest/userguide/auth-and-access.html">Authentication and Access Control for AWS Secrets Manager</a> in the <i>AWS Secrets Manager User Guide</i>.<br/>Name:  The friendly name of the secret that the resource-based policy was retrieved for.<br/>ARN:  The ARN of the secret that the resource-based policy was retrieved for.<br/>        """
        pass

    def get_secret_value(self, SecretId: str, VersionId: str=None, VersionStage: str=None) -> GetSecretValueResponse:
        """Retrieves the contents of the encrypted fields <code>SecretString</code> or <code>SecretBinary</code> from the specified version of a secret, whichever contains content.  <b>Minimum permissions</b>  To run this command, you must have the following permissions: <ul> <li> secretsmanager:GetSecretValue </li> <li> kms:Decrypt - required only if you use a customer-managed AWS KMS key to encrypt the secret. You do not need this permission to use the account's default AWS managed CMK for Secrets Manager. </li> </ul>  <b>Related operations</b>  <ul> <li> To create a new version of the secret with different encrypted information, use <a>PutSecretValue</a>. </li> <li> To retrieve the non-encrypted details for the secret, use <a>DescribeSecret</a>. </li> </ul>

        :param self:
        :param VersionStage:  Specifies the secret version that you want to retrieve by the staging label attached to the version. Staging labels are used to keep track of different versions during the rotation process. If you use this parameter then don't specify <code>VersionId</code>. If you don't specify either a <code>VersionStage</code> or <code>VersionId</code>, then the default is to perform the operation on the version with the <code>VersionStage</code> value of <code>AWSCURRENT</code>.<br/>VersionId:  Specifies the unique identifier of the version of the secret that you want to retrieve. If you specify this parameter then don't specify <code>VersionStage</code>. If you don't specify either a <code>VersionStage</code> or <code>VersionId</code> then the default is to perform the operation on the version with the <code>VersionStage</code> value of <code>AWSCURRENT</code>. This value is typically a <a href="https://wikipedia.org/wiki/Universally_unique_identifier">UUID-type</a> value with 32 hexadecimal digits.<br/>SecretId:  Specifies the secret containing the version that you want to retrieve. You can specify either the Amazon Resource Name (ARN) or the friendly name of the secret. <note> If you specify an ARN, we generally recommend that you specify a complete ARN. You can specify a partial ARN too—for example, if you don’t include the final hyphen and six random characters that Secrets Manager adds at the end of the ARN when you created the secret. A partial ARN match can work as long as it uniquely matches only one secret. However, if your secret has a name that ends in a hyphen followed by six characters (before Secrets Manager adds the hyphen and six characters to the ARN) and you try to use that as a partial ARN, then those characters cause Secrets Manager to assume that you’re specifying a complete ARN. This confusion can cause unexpected results. To avoid this situation, we recommend that you don’t create secret names that end with a hyphen followed by six characters. </note><br/>
        :return: CreatedDate:  The date and time that this version of the secret was created.<br/>VersionStages:  A list of all of the staging labels currently attached to this version of the secret.<br/>SecretString:  The decrypted part of the protected secret information that was originally provided as a string. If you create this secret by using the Secrets Manager console then only the <code>SecretString</code> parameter contains data. Secrets Manager stores the information as a JSON structure of key/value pairs that the Lambda rotation function knows how to parse. If you store custom information in the secret by using the <a>CreateSecret</a>, <a>UpdateSecret</a>, or <a>PutSecretValue</a> API operations instead of the Secrets Manager console, or by using the <b>Other secret type</b> in the console, then you must code your Lambda rotation function to parse and interpret those values.<br/>SecretBinary:  The decrypted part of the protected secret information that was originally provided as binary data in the form of a byte array. The response parameter represents the binary data as a <a href="https://tools.ietf.org/html/rfc4648#section-4">base64-encoded</a> string. This parameter is not used if the secret is created by the Secrets Manager console. If you store custom information in this field of the secret, then you must code your Lambda rotation function to parse and interpret whatever you store in the <code>SecretString</code> or <code>SecretBinary</code> fields.<br/>VersionId:  The unique identifier of this version of the secret.<br/>Name:  The friendly name of the secret.<br/>ARN:  The ARN of the secret.<br/>        """
        pass

    def list_secret_version_ids(self, SecretId: str, MaxResults: int=None, NextToken: str=None, IncludeDeprecated: BooleanType=None) -> ListSecretVersionIdsResponse:
        """Lists all of the versions attached to the specified secret. The output does not include the <code>SecretString</code> or <code>SecretBinary</code> fields. By default, the list includes only versions that have at least one staging label in <code>VersionStage</code> attached. <note> Always check the <code>NextToken</code> response parameter when calling any of the <code>List*</code> operations. These operations can occasionally return an empty or shorter than expected list of results even when there are more results available. When this happens, the <code>NextToken</code> response parameter contains a value to pass to the next call to the same API to request the next part of the list. </note>  <b>Minimum permissions</b>  To run this command, you must have the following permissions: <ul> <li> secretsmanager:ListSecretVersionIds </li> </ul>  <b>Related operations</b>  <ul> <li> To list the secrets in an account, use <a>ListSecrets</a>. </li> </ul>

        :param self:
        :param IncludeDeprecated:  (Optional) Specifies that you want the results to include versions that do not have any staging labels attached to them. Such versions are considered deprecated and are subject to deletion by Secrets Manager as needed.<br/>NextToken:  (Optional) Use this parameter in a request if you receive a <code>NextToken</code> response in a previous request that indicates that there's more output available. In a subsequent call, set it to the value of the previous call's <code>NextToken</code> response to indicate where the output should continue from.<br/>MaxResults:  (Optional) Limits the number of results that you want to include in the response. If you don't include this parameter, it defaults to a value that's specific to the operation. If additional items exist beyond the maximum you specify, the <code>NextToken</code> response element is present and has a value (isn't null). Include that value as the <code>NextToken</code> request parameter in the next call to the operation to get the next part of the results. Note that Secrets Manager might return fewer results than the maximum even when there are more results available. You should check <code>NextToken</code> after every operation to ensure that you receive all of the results.<br/>SecretId:  The identifier for the secret containing the versions you want to list. You can specify either the Amazon Resource Name (ARN) or the friendly name of the secret. <note> If you specify an ARN, we generally recommend that you specify a complete ARN. You can specify a partial ARN too—for example, if you don’t include the final hyphen and six random characters that Secrets Manager adds at the end of the ARN when you created the secret. A partial ARN match can work as long as it uniquely matches only one secret. However, if your secret has a name that ends in a hyphen followed by six characters (before Secrets Manager adds the hyphen and six characters to the ARN) and you try to use that as a partial ARN, then those characters cause Secrets Manager to assume that you’re specifying a complete ARN. This confusion can cause unexpected results. To avoid this situation, we recommend that you don’t create secret names that end with a hyphen followed by six characters. </note><br/>
        :return: Name:  The friendly name of the secret.<br/>ARN:  The Amazon Resource Name (ARN) for the secret. <note> Secrets Manager automatically adds several random characters to the name at the end of the ARN when you initially create a secret. This affects only the ARN and not the actual friendly name. This ensures that if you create a new secret with the same name as an old secret that you previously deleted, then users with access to the old secret <i>don't</i> automatically get access to the new secret because the ARNs are different. </note><br/>NextToken:  If present in the response, this value indicates that there's more output available than what's included in the current response. This can occur even when the response includes no values at all, such as when you ask for a filtered view of a very long list. Use this value in the <code>NextToken</code> request parameter in a subsequent call to the operation to continue processing and get the next part of the output. You should repeat this until the <code>NextToken</code> response element comes back empty (as <code>null</code>).<br/>Versions:  The list of the currently available versions of the specified secret.<br/>        """
        pass

    def list_secrets(self, MaxResults: int=None, NextToken: str=None) -> ListSecretsResponse:
        """Lists all of the secrets that are stored by Secrets Manager in the AWS account. To list the versions currently stored for a specific secret, use <a>ListSecretVersionIds</a>. The encrypted fields <code>SecretString</code> and <code>SecretBinary</code> are not included in the output. To get that information, call the <a>GetSecretValue</a> operation. <note> Always check the <code>NextToken</code> response parameter when calling any of the <code>List*</code> operations. These operations can occasionally return an empty or shorter than expected list of results even when there are more results available. When this happens, the <code>NextToken</code> response parameter contains a value to pass to the next call to the same API to request the next part of the list. </note>  <b>Minimum permissions</b>  To run this command, you must have the following permissions: <ul> <li> secretsmanager:ListSecrets </li> </ul>  <b>Related operations</b>  <ul> <li> To list the versions attached to a secret, use <a>ListSecretVersionIds</a>. </li> </ul>

        :param self:
        :param NextToken:  (Optional) Use this parameter in a request if you receive a <code>NextToken</code> response in a previous request that indicates that there's more output available. In a subsequent call, set it to the value of the previous call's <code>NextToken</code> response to indicate where the output should continue from.<br/>MaxResults:  (Optional) Limits the number of results that you want to include in the response. If you don't include this parameter, it defaults to a value that's specific to the operation. If additional items exist beyond the maximum you specify, the <code>NextToken</code> response element is present and has a value (isn't null). Include that value as the <code>NextToken</code> request parameter in the next call to the operation to get the next part of the results. Note that Secrets Manager might return fewer results than the maximum even when there are more results available. You should check <code>NextToken</code> after every operation to ensure that you receive all of the results.<br/>
        :return: NextToken:  If present in the response, this value indicates that there's more output available than what's included in the current response. This can occur even when the response includes no values at all, such as when you ask for a filtered view of a very long list. Use this value in the <code>NextToken</code> request parameter in a subsequent call to the operation to continue processing and get the next part of the output. You should repeat this until the <code>NextToken</code> response element comes back empty (as <code>null</code>).<br/>SecretList:  A list of the secrets in the account.<br/>        """
        pass

    def put_resource_policy(self, ResourcePolicy: str, SecretId: str) -> PutResourcePolicyResponse:
        """Attaches the contents of the specified resource-based permission policy to a secret. A resource-based policy is optional. Alternatively, you can use IAM identity-based policies that specify the secret's Amazon Resource Name (ARN) in the policy statement's <code>Resources</code> element. You can also use a combination of both identity-based and resource-based policies. The affected users and roles receive the permissions that are permitted by all of the relevant policies. For more information, see <a href="http://docs.aws.amazon.com/secretsmanager/latest/userguide/auth-and-access_resource-based-policies.html">Using Resource-Based Policies for AWS Secrets Manager</a>. For the complete description of the AWS policy syntax and grammar, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_policies.html">IAM JSON Policy Reference</a> in the <i>IAM User Guide</i>.  <b>Minimum permissions</b>  To run this command, you must have the following permissions: <ul> <li> secretsmanager:PutResourcePolicy </li> </ul>  <b>Related operations</b>  <ul> <li> To retrieve the resource policy that's attached to a secret, use <a>GetResourcePolicy</a>. </li> <li> To delete the resource-based policy that's attached to a secret, use <a>DeleteResourcePolicy</a>. </li> <li> To list all of the currently available secrets, use <a>ListSecrets</a>. </li> </ul>

        :param self:
        :param ResourcePolicy:  A JSON-formatted string that's constructed according to the grammar and syntax for an AWS resource-based policy. The policy in the string identifies who can access or manage this secret and its versions. For information on how to format a JSON parameter for the various command line tool environments, see <a href="http://docs.aws.amazon.com/cli/latest/userguide/cli-using-param.html#cli-using-param-json">Using JSON for Parameters</a> in the <i>AWS CLI User Guide</i>.<br/>SecretId:  Specifies the secret that you want to attach the resource-based policy to. You can specify either the ARN or the friendly name of the secret. <note> If you specify an ARN, we generally recommend that you specify a complete ARN. You can specify a partial ARN too—for example, if you don’t include the final hyphen and six random characters that Secrets Manager adds at the end of the ARN when you created the secret. A partial ARN match can work as long as it uniquely matches only one secret. However, if your secret has a name that ends in a hyphen followed by six characters (before Secrets Manager adds the hyphen and six characters to the ARN) and you try to use that as a partial ARN, then those characters cause Secrets Manager to assume that you’re specifying a complete ARN. This confusion can cause unexpected results. To avoid this situation, we recommend that you don’t create secret names that end with a hyphen followed by six characters. </note><br/>
        :return: Name:  The friendly name of the secret that the resource-based policy was retrieved for.<br/>ARN:  The ARN of the secret that the resource-based policy was retrieved for.<br/>        """
        pass

    def put_secret_value(self, SecretId: str, ClientRequestToken: str=None, SecretBinary: SecretBinaryType=None, SecretString: str=None, VersionStages: List[SecretVersionStageType]=None) -> PutSecretValueResponse:
        """Stores a new encrypted secret value in the specified secret. To do this, the operation creates a new version and attaches it to the secret. The version can contain a new <code>SecretString</code> value or a new <code>SecretBinary</code> value. You can also specify the staging labels that are initially attached to the new version. <note> The Secrets Manager console uses only the <code>SecretString</code> field. To add binary data to a secret with the <code>SecretBinary</code> field you must use the AWS CLI or one of the AWS SDKs. </note> <ul> <li> If this operation creates the first version for the secret then Secrets Manager automatically attaches the staging label <code>AWSCURRENT</code> to the new version. </li> <li> If another version of this secret already exists, then this operation does not automatically move any staging labels other than those that you explicitly specify in the <code>VersionStages</code> parameter. </li> <li> If this operation moves the staging label <code>AWSCURRENT</code> from another version to this version (because you included it in the <code>StagingLabels</code> parameter) then Secrets Manager also automatically moves the staging label <code>AWSPREVIOUS</code> to the version that <code>AWSCURRENT</code> was removed from. </li> <li> This operation is idempotent. If a version with a <code>VersionId</code> with the same value as the <code>ClientRequestToken</code> parameter already exists and you specify the same secret data, the operation succeeds but does nothing. However, if the secret data is different, then the operation fails because you cannot modify an existing version; you can only create new ones. </li> </ul> <note> <ul> <li> If you call an operation that needs to encrypt or decrypt the <code>SecretString</code> or <code>SecretBinary</code> for a secret in the same account as the calling user and that secret doesn't specify a AWS KMS encryption key, Secrets Manager uses the account's default AWS managed customer master key (CMK) with the alias <code>aws/secretsmanager</code>. If this key doesn't already exist in your account then Secrets Manager creates it for you automatically. All users and roles in the same AWS account automatically have access to use the default CMK. Note that if an Secrets Manager API call results in AWS having to create the account's AWS-managed CMK, it can result in a one-time significant delay in returning the result. </li> <li> If the secret is in a different AWS account from the credentials calling an API that requires encryption or decryption of the secret value then you must create and use a custom AWS KMS CMK because you can't access the default CMK for the account using credentials from a different AWS account. Store the ARN of the CMK in the secret when you create the secret or when you update it by including it in the <code>KMSKeyId</code>. If you call an API that must encrypt or decrypt <code>SecretString</code> or <code>SecretBinary</code> using credentials from a different account then the AWS KMS key policy must grant cross-account access to that other account's user or role for both the kms:GenerateDataKey and kms:Decrypt operations. </li> </ul> </note>  <b>Minimum permissions</b>  To run this command, you must have the following permissions: <ul> <li> secretsmanager:PutSecretValue </li> <li> kms:GenerateDataKey - needed only if you use a customer-managed AWS KMS key to encrypt the secret. You do not need this permission to use the account's default AWS managed CMK for Secrets Manager. </li> </ul>  <b>Related operations</b>  <ul> <li> To retrieve the encrypted value you store in the version of a secret, use <a>GetSecretValue</a>. </li> <li> To create a secret, use <a>CreateSecret</a>. </li> <li> To get the details for a secret, use <a>DescribeSecret</a>. </li> <li> To list the versions attached to a secret, use <a>ListSecretVersionIds</a>. </li> </ul>

        :param self:
        :param VersionStages:  (Optional) Specifies a list of staging labels that are attached to this version of the secret. These staging labels are used to track the versions through the rotation process by the Lambda rotation function. A staging label must be unique to a single version of the secret. If you specify a staging label that's already associated with a different version of the same secret then that staging label is automatically removed from the other version and attached to this version. If you do not specify a value for <code>VersionStages</code> then Secrets Manager automatically moves the staging label <code>AWSCURRENT</code> to this new version.<br/>SecretString:  (Optional) Specifies text data that you want to encrypt and store in this new version of the secret. Either <code>SecretString</code> or <code>SecretBinary</code> must have a value, but not both. They cannot both be empty. If you create this secret by using the Secrets Manager console then Secrets Manager puts the protected secret text in only the <code>SecretString</code> parameter. The Secrets Manager console stores the information as a JSON structure of key/value pairs that the default Lambda rotation function knows how to parse. For storing multiple values, we recommend that you use a JSON text string argument and specify key/value pairs. For information on how to format a JSON parameter for the various command line tool environments, see <a href="https://docs.aws.amazon.com/cli/latest/userguide/cli-using-param.html#cli-using-param-json">Using JSON for Parameters</a> in the <i>AWS CLI User Guide</i>.  For example:  <code>[{"username":"bob"},{"password":"abc123xyz456"}]</code>  If your command-line tool or SDK requires quotation marks around the parameter, you should use single quotes to avoid confusion with the double quotes required in the JSON text.<br/>SecretBinary:  (Optional) Specifies binary data that you want to encrypt and store in the new version of the secret. To use this parameter in the command-line tools, we recommend that you store your binary data in a file and then use the appropriate technique for your tool to pass the contents of the file as a parameter. Either <code>SecretBinary</code> or <code>SecretString</code> must have a value, but not both. They cannot both be empty. This parameter is not accessible if the secret using the Secrets Manager console. <p/><br/>ClientRequestToken:  (Optional) Specifies a unique identifier for the new version of the secret.  <note> If you use the AWS CLI or one of the AWS SDK to call this operation, then you can leave this parameter empty. The CLI or SDK generates a random UUID for you and includes that in the request. If you don't use the SDK and instead generate a raw HTTP request to the Secrets Manager service endpoint, then you must generate a <code>ClientRequestToken</code> yourself for new versions and include that value in the request.  </note> This value helps ensure idempotency. Secrets Manager uses this value to prevent the accidental creation of duplicate versions if there are failures and retries during the Lambda rotation function's processing. We recommend that you generate a <a href="https://wikipedia.org/wiki/Universally_unique_identifier">UUID-type</a> value to ensure uniqueness within the specified secret.  <ul> <li> If the <code>ClientRequestToken</code> value isn't already associated with a version of the secret then a new version of the secret is created.  </li> <li> If a version with this value already exists and that version's <code>SecretString</code> or <code>SecretBinary</code> values are the same as those in the request then the request is ignored (the operation is idempotent).  </li> <li> If a version with this value already exists and that version's <code>SecretString</code> and <code>SecretBinary</code> values are different from those in the request then the request fails because you cannot modify an existing secret version. You can only create new versions to store new secret values. </li> </ul> This value becomes the <code>VersionId</code> of the new version.<br/>SecretId:  Specifies the secret to which you want to add a new version. You can specify either the Amazon Resource Name (ARN) or the friendly name of the secret. The secret must already exist. <note> If you specify an ARN, we generally recommend that you specify a complete ARN. You can specify a partial ARN too—for example, if you don’t include the final hyphen and six random characters that Secrets Manager adds at the end of the ARN when you created the secret. A partial ARN match can work as long as it uniquely matches only one secret. However, if your secret has a name that ends in a hyphen followed by six characters (before Secrets Manager adds the hyphen and six characters to the ARN) and you try to use that as a partial ARN, then those characters cause Secrets Manager to assume that you’re specifying a complete ARN. This confusion can cause unexpected results. To avoid this situation, we recommend that you don’t create secret names that end with a hyphen followed by six characters. </note><br/>
        :return: VersionStages:  The list of staging labels that are currently attached to this version of the secret. Staging labels are used to track a version as it progresses through the secret rotation process.<br/>VersionId:  The unique identifier of the version of the secret you just created or updated.<br/>Name:  The friendly name of the secret for which you just created or updated a version.<br/>ARN:  The Amazon Resource Name (ARN) for the secret for which you just created a version.<br/>        """
        pass

    def restore_secret(self, SecretId: str) -> RestoreSecretResponse:
        """Cancels the scheduled deletion of a secret by removing the <code>DeletedDate</code> time stamp. This makes the secret accessible to query once again.  <b>Minimum permissions</b>  To run this command, you must have the following permissions: <ul> <li> secretsmanager:RestoreSecret </li> </ul>  <b>Related operations</b>  <ul> <li> To delete a secret, use <a>DeleteSecret</a>. </li> </ul>

        :param self:
        :param SecretId:  Specifies the secret that you want to restore from a previously scheduled deletion. You can specify either the Amazon Resource Name (ARN) or the friendly name of the secret. <note> If you specify an ARN, we generally recommend that you specify a complete ARN. You can specify a partial ARN too—for example, if you don’t include the final hyphen and six random characters that Secrets Manager adds at the end of the ARN when you created the secret. A partial ARN match can work as long as it uniquely matches only one secret. However, if your secret has a name that ends in a hyphen followed by six characters (before Secrets Manager adds the hyphen and six characters to the ARN) and you try to use that as a partial ARN, then those characters cause Secrets Manager to assume that you’re specifying a complete ARN. This confusion can cause unexpected results. To avoid this situation, we recommend that you don’t create secret names that end with a hyphen followed by six characters. </note><br/>
        :return: Name:  The friendly name of the secret that was restored.<br/>ARN:  The ARN of the secret that was restored.<br/>        """
        pass

    def rotate_secret(self, SecretId: str, ClientRequestToken: str=None, RotationLambdaARN: str=None, RotationRules: RotationRulesType=None) -> RotateSecretResponse:
        """Configures and starts the asynchronous process of rotating this secret. If you include the configuration parameters, the operation sets those values for the secret and then immediately starts a rotation. If you do not include the configuration parameters, the operation starts a rotation with the values already stored in the secret. After the rotation completes, the protected service and its clients all use the new version of the secret.  This required configuration information includes the ARN of an AWS Lambda function and the time between scheduled rotations. The Lambda rotation function creates a new version of the secret and creates or updates the credentials on the protected service to match. After testing the new credentials, the function marks the new secret with the staging label <code>AWSCURRENT</code> so that your clients all immediately begin to use the new version. For more information about rotating secrets and how to configure a Lambda function to rotate the secrets for your protected service, see <a href="https://docs.aws.amazon.com/secretsmanager/latest/userguide/rotating-secrets.html">Rotating Secrets in AWS Secrets Manager</a> in the <i>AWS Secrets Manager User Guide</i>. Secrets Manager schedules the next rotation when the previous one is complete. Secrets Manager schedules the date by adding the rotation interval (number of days) to the actual date of the last rotation. The service chooses the hour within that 24-hour date window randomly. The minute is also chosen somewhat randomly, but weighted towards the top of the hour and influenced by a variety of factors that help distribute load. The rotation function must end with the versions of the secret in one of two states: <ul> <li> The <code>AWSPENDING</code> and <code>AWSCURRENT</code> staging labels are attached to the same version of the secret, or </li> <li> The <code>AWSPENDING</code> staging label is not attached to any version of the secret. </li> </ul> If instead the <code>AWSPENDING</code> staging label is present but is not attached to the same version as <code>AWSCURRENT</code> then any later invocation of <code>RotateSecret</code> assumes that a previous rotation request is still in progress and returns an error.  <b>Minimum permissions</b>  To run this command, you must have the following permissions: <ul> <li> secretsmanager:RotateSecret </li> <li> lambda:InvokeFunction (on the function specified in the secret's metadata) </li> </ul>  <b>Related operations</b>  <ul> <li> To list the secrets in your account, use <a>ListSecrets</a>. </li> <li> To get the details for a version of a secret, use <a>DescribeSecret</a>. </li> <li> To create a new version of a secret, use <a>CreateSecret</a>. </li> <li> To attach staging labels to or remove staging labels from a version of a secret, use <a>UpdateSecretVersionStage</a>. </li> </ul>

        :param self:
        :param RotationRules:  A structure that defines the rotation configuration for this secret.:<br/>RotationLambdaARN:  (Optional) Specifies the ARN of the Lambda function that can rotate the secret.<br/>ClientRequestToken:  (Optional) Specifies a unique identifier for the new version of the secret that helps ensure idempotency.  If you use the AWS CLI or one of the AWS SDK to call this operation, then you can leave this parameter empty. The CLI or SDK generates a random UUID for you and includes that in the request for this parameter. If you don't use the SDK and instead generate a raw HTTP request to the Secrets Manager service endpoint, then you must generate a <code>ClientRequestToken</code> yourself for new versions and include that value in the request. You only need to specify your own value if you are implementing your own retry logic and want to ensure that a given secret is not created twice. We recommend that you generate a <a href="https://wikipedia.org/wiki/Universally_unique_identifier">UUID-type</a> value to ensure uniqueness within the specified secret.  Secrets Manager uses this value to prevent the accidental creation of duplicate versions if there are failures and retries during the function's processing. This value becomes the <code>VersionId</code> of the new version.<br/>SecretId:  Specifies the secret that you want to rotate. You can specify either the Amazon Resource Name (ARN) or the friendly name of the secret. <note> If you specify an ARN, we generally recommend that you specify a complete ARN. You can specify a partial ARN too—for example, if you don’t include the final hyphen and six random characters that Secrets Manager adds at the end of the ARN when you created the secret. A partial ARN match can work as long as it uniquely matches only one secret. However, if your secret has a name that ends in a hyphen followed by six characters (before Secrets Manager adds the hyphen and six characters to the ARN) and you try to use that as a partial ARN, then those characters cause Secrets Manager to assume that you’re specifying a complete ARN. This confusion can cause unexpected results. To avoid this situation, we recommend that you don’t create secret names that end with a hyphen followed by six characters. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomaticallyAfterDays</i> Specifies the number of days between automatic scheduled rotations of the secret. Secrets Manager schedules the next rotation when the previous one is complete. Secrets Manager schedules the date by adding the rotation interval (number of days) to the actual date of the last rotation. The service chooses the hour within that 24-hour date window randomly. The minute is also chosen somewhat randomly, but weighted towards the top of the hour and influenced by a variety of factors that help distribute load.<br/>
        :return: VersionId:  The ID of the new version of the secret created by the rotation started by this request.<br/>Name:  The friendly name of the secret.<br/>ARN:  The ARN of the secret.<br/>        """
        pass

    def tag_resource(self, Tags: List[Tag], SecretId: str):
        """Attaches one or more tags, each consisting of a key name and a value, to the specified secret. Tags are part of the secret's overall metadata, and are not associated with any specific version of the secret. This operation only appends tags to the existing list of tags. To remove tags, you must use <a>UntagResource</a>. The following basic restrictions apply to tags: <ul> <li> Maximum number of tags per secret—50 </li> <li> Maximum key length—127 Unicode characters in UTF-8 </li> <li> Maximum value length—255 Unicode characters in UTF-8 </li> <li> Tag keys and values are case sensitive. </li> <li> Do not use the <code>aws:</code> prefix in your tag names or values because it is reserved for AWS use. You can't edit or delete tag names or values with this prefix. Tags with this prefix do not count against your tags per secret limit. </li> <li> If your tagging schema will be used across multiple services and resources, remember that other services might have restrictions on allowed characters. Generally allowed characters are: letters, spaces, and numbers representable in UTF-8, plus the following special characters: + - = . _ : / @. </li> </ul> <important> If you use tags as part of your security strategy, then adding or removing a tag can change permissions. If successfully completing this operation would result in you losing your permissions for this secret, then the operation is blocked and returns an Access Denied error. </important>  <b>Minimum permissions</b>  To run this command, you must have the following permissions: <ul> <li> secretsmanager:TagResource </li> </ul>  <b>Related operations</b>  <ul> <li> To remove one or more tags from the collection attached to a secret, use <a>UntagResource</a>. </li> <li> To view the list of tags attached to a secret, use <a>DescribeSecret</a>. </li> </ul>

        :param self:
        :param Tags:  The tags to attach to the secret. Each element in the list consists of a <code>Key</code> and a <code>Value</code>. This parameter to the API requires a JSON text string argument. For information on how to format a JSON parameter for the various command line tool environments, see <a href="https://docs.aws.amazon.com/cli/latest/userguide/cli-using-param.html#cli-using-param-json">Using JSON for Parameters</a> in the <i>AWS CLI User Guide</i>. For the AWS CLI, you can also use the syntax: <code>--Tags Key="Key1",Value="Value1",Key="Key2",Value="Value2"[,…]</code> <br/>SecretId:  The identifier for the secret that you want to attach tags to. You can specify either the Amazon Resource Name (ARN) or the friendly name of the secret. <note> If you specify an ARN, we generally recommend that you specify a complete ARN. You can specify a partial ARN too—for example, if you don’t include the final hyphen and six random characters that Secrets Manager adds at the end of the ARN when you created the secret. A partial ARN match can work as long as it uniquely matches only one secret. However, if your secret has a name that ends in a hyphen followed by six characters (before Secrets Manager adds the hyphen and six characters to the ARN) and you try to use that as a partial ARN, then those characters cause Secrets Manager to assume that you’re specifying a complete ARN. This confusion can cause unexpected results. To avoid this situation, we recommend that you don’t create secret names that end with a hyphen followed by six characters. </note><br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[TagKeyType], SecretId: str):
        """Removes one or more tags from the specified secret. This operation is idempotent. If a requested tag is not attached to the secret, no error is returned and the secret metadata is unchanged. <important> If you use tags as part of your security strategy, then removing a tag can change permissions. If successfully completing this operation would result in you losing your permissions for this secret, then the operation is blocked and returns an Access Denied error. </important>  <b>Minimum permissions</b>  To run this command, you must have the following permissions: <ul> <li> secretsmanager:UntagResource </li> </ul>  <b>Related operations</b>  <ul> <li> To add one or more tags to the collection attached to a secret, use <a>TagResource</a>. </li> <li> To view the list of tags attached to a secret, use <a>DescribeSecret</a>. </li> </ul>

        :param self:
        :param TagKeys:  A list of tag key names to remove from the secret. You don't specify the value. Both the key and its associated value are removed. This parameter to the API requires a JSON text string argument. For information on how to format a JSON parameter for the various command line tool environments, see <a href="https://docs.aws.amazon.com/cli/latest/userguide/cli-using-param.html#cli-using-param-json">Using JSON for Parameters</a> in the <i>AWS CLI User Guide</i>.<br/>SecretId:  The identifier for the secret that you want to remove tags from. You can specify either the Amazon Resource Name (ARN) or the friendly name of the secret. <note> If you specify an ARN, we generally recommend that you specify a complete ARN. You can specify a partial ARN too—for example, if you don’t include the final hyphen and six random characters that Secrets Manager adds at the end of the ARN when you created the secret. A partial ARN match can work as long as it uniquely matches only one secret. However, if your secret has a name that ends in a hyphen followed by six characters (before Secrets Manager adds the hyphen and six characters to the ARN) and you try to use that as a partial ARN, then those characters cause Secrets Manager to assume that you’re specifying a complete ARN. This confusion can cause unexpected results. To avoid this situation, we recommend that you don’t create secret names that end with a hyphen followed by six characters. </note><br/>
        :return:         """
        pass

    def update_secret(self, SecretId: str, ClientRequestToken: str=None, Description: str=None, KmsKeyId: str=None, SecretBinary: SecretBinaryType=None, SecretString: str=None) -> UpdateSecretResponse:
        """Modifies many of the details of the specified secret. If you include a <code>ClientRequestToken</code> and <i>either</i> <code>SecretString</code> or <code>SecretBinary</code> then it also creates a new version attached to the secret. To modify the rotation configuration of a secret, use <a>RotateSecret</a> instead. <note> The Secrets Manager console uses only the <code>SecretString</code> parameter and therefore limits you to encrypting and storing only a text string. To encrypt and store binary data as part of the version of a secret, you must use either the AWS CLI or one of the AWS SDKs. </note> <ul> <li> If a version with a <code>VersionId</code> with the same value as the <code>ClientRequestToken</code> parameter already exists, the operation results in an error. You cannot modify an existing version, you can only create a new version. </li> <li> If you include <code>SecretString</code> or <code>SecretBinary</code> to create a new secret version, Secrets Manager automatically attaches the staging label <code>AWSCURRENT</code> to the new version.  </li> </ul> <note> <ul> <li> If you call an operation that needs to encrypt or decrypt the <code>SecretString</code> or <code>SecretBinary</code> for a secret in the same account as the calling user and that secret doesn't specify a AWS KMS encryption key, Secrets Manager uses the account's default AWS managed customer master key (CMK) with the alias <code>aws/secretsmanager</code>. If this key doesn't already exist in your account then Secrets Manager creates it for you automatically. All users and roles in the same AWS account automatically have access to use the default CMK. Note that if an Secrets Manager API call results in AWS having to create the account's AWS-managed CMK, it can result in a one-time significant delay in returning the result. </li> <li> If the secret is in a different AWS account from the credentials calling an API that requires encryption or decryption of the secret value then you must create and use a custom AWS KMS CMK because you can't access the default CMK for the account using credentials from a different AWS account. Store the ARN of the CMK in the secret when you create the secret or when you update it by including it in the <code>KMSKeyId</code>. If you call an API that must encrypt or decrypt <code>SecretString</code> or <code>SecretBinary</code> using credentials from a different account then the AWS KMS key policy must grant cross-account access to that other account's user or role for both the kms:GenerateDataKey and kms:Decrypt operations. </li> </ul> </note>  <b>Minimum permissions</b>  To run this command, you must have the following permissions: <ul> <li> secretsmanager:UpdateSecret </li> <li> kms:GenerateDataKey - needed only if you use a custom AWS KMS key to encrypt the secret. You do not need this permission to use the account's AWS managed CMK for Secrets Manager. </li> <li> kms:Decrypt - needed only if you use a custom AWS KMS key to encrypt the secret. You do not need this permission to use the account's AWS managed CMK for Secrets Manager. </li> </ul>  <b>Related operations</b>  <ul> <li> To create a new secret, use <a>CreateSecret</a>. </li> <li> To add only a new version to an existing secret, use <a>PutSecretValue</a>. </li> <li> To get the details for a secret, use <a>DescribeSecret</a>. </li> <li> To list the versions contained in a secret, use <a>ListSecretVersionIds</a>. </li> </ul>

        :param self:
        :param SecretString:  (Optional) Specifies updated text data that you want to encrypt and store in this new version of the secret. Either <code>SecretBinary</code> or <code>SecretString</code> must have a value, but not both. They cannot both be empty. If you create this secret by using the Secrets Manager console then Secrets Manager puts the protected secret text in only the <code>SecretString</code> parameter. The Secrets Manager console stores the information as a JSON structure of key/value pairs that the default Lambda rotation function knows how to parse. For storing multiple values, we recommend that you use a JSON text string argument and specify key/value pairs. For information on how to format a JSON parameter for the various command line tool environments, see <a href="https://docs.aws.amazon.com/cli/latest/userguide/cli-using-param.html#cli-using-param-json">Using JSON for Parameters</a> in the <i>AWS CLI User Guide</i>. For example:  <code>[{"username":"bob"},{"password":"abc123xyz456"}]</code>  If your command-line tool or SDK requires quotation marks around the parameter, you should use single quotes to avoid confusion with the double quotes required in the JSON text. You can also 'escape' the double quote character in the embedded JSON text by prefacing each with a backslash. For example, the following string is surrounded by double-quotes. All of the embedded double quotes are escaped:  <code>"[{\"username\":\"bob\"},{\"password\":\"abc123xyz456\"}]"</code> <br/>SecretBinary:  (Optional) Specifies updated binary data that you want to encrypt and store in the new version of the secret. To use this parameter in the command-line tools, we recommend that you store your binary data in a file and then use the appropriate technique for your tool to pass the contents of the file as a parameter. Either <code>SecretBinary</code> or <code>SecretString</code> must have a value, but not both. They cannot both be empty. This parameter is not accessible using the Secrets Manager console.<br/>KmsKeyId:  (Optional) Specifies an updated ARN or alias of the AWS KMS customer master key (CMK) to be used to encrypt the protected text in new versions of this secret. <important> You can only use the account's default CMK to encrypt and decrypt if you call this operation using credentials from the same account that owns the secret. If the secret is in a different account, then you must create a custom CMK and provide the ARN of that CMK in this field. The user making the call must have permissions to both the secret and the CMK in their respective accounts. </important><br/>Description:  (Optional) Specifies an updated user-provided description of the secret.<br/>ClientRequestToken:  (Optional) If you want to add a new version to the secret, this parameter specifies a unique identifier for the new version that helps ensure idempotency.  If you use the AWS CLI or one of the AWS SDK to call this operation, then you can leave this parameter empty. The CLI or SDK generates a random UUID for you and includes that in the request. If you don't use the SDK and instead generate a raw HTTP request to the Secrets Manager service endpoint, then you must generate a <code>ClientRequestToken</code> yourself for new versions and include that value in the request. You typically only need to interact with this value if you implement your own retry logic and want to ensure that a given secret is not created twice. We recommend that you generate a <a href="https://wikipedia.org/wiki/Universally_unique_identifier">UUID-type</a> value to ensure uniqueness within the specified secret.  Secrets Manager uses this value to prevent the accidental creation of duplicate versions if there are failures and retries during the Lambda rotation function's processing. <ul> <li> If the <code>ClientRequestToken</code> value isn't already associated with a version of the secret then a new version of the secret is created.  </li> <li> If a version with this value already exists and that version's <code>SecretString</code> and <code>SecretBinary</code> values are the same as those in the request then the request is ignored (the operation is idempotent).  </li> <li> If a version with this value already exists and that version's <code>SecretString</code> and <code>SecretBinary</code> values are different from the request then an error occurs because you cannot modify an existing secret value. </li> </ul> This value becomes the <code>VersionId</code> of the new version.<br/>SecretId:  Specifies the secret that you want to modify or to which you want to add a new version. You can specify either the Amazon Resource Name (ARN) or the friendly name of the secret. <note> If you specify an ARN, we generally recommend that you specify a complete ARN. You can specify a partial ARN too—for example, if you don’t include the final hyphen and six random characters that Secrets Manager adds at the end of the ARN when you created the secret. A partial ARN match can work as long as it uniquely matches only one secret. However, if your secret has a name that ends in a hyphen followed by six characters (before Secrets Manager adds the hyphen and six characters to the ARN) and you try to use that as a partial ARN, then those characters cause Secrets Manager to assume that you’re specifying a complete ARN. This confusion can cause unexpected results. To avoid this situation, we recommend that you don’t create secret names that end with a hyphen followed by six characters. </note><br/>
        :return: VersionId:  If a new version of the secret was created by this operation, then <code>VersionId</code> contains the unique identifier of the new version.<br/>Name:  The friendly name of the secret that was updated.<br/>ARN:  The ARN of the secret that was updated. <note> Secrets Manager automatically adds several random characters to the name at the end of the ARN when you initially create a secret. This affects only the ARN and not the actual friendly name. This ensures that if you create a new secret with the same name as an old secret that you previously deleted, then users with access to the old secret <i>don't</i> automatically get access to the new secret because the ARNs are different. </note><br/>        """
        pass

    def update_secret_version_stage(self, VersionStage: str, SecretId: str, RemoveFromVersionId: str=None, MoveToVersionId: str=None) -> UpdateSecretVersionStageResponse:
        """Modifies the staging labels attached to a version of a secret. Staging labels are used to track a version as it progresses through the secret rotation process. You can attach a staging label to only one version of a secret at a time. If a staging label to be added is already attached to another version, then it is moved--removed from the other version first and then attached to this one. For more information about staging labels, see <a href="https://docs.aws.amazon.com/secretsmanager/latest/userguide/terms-concepts.html#term_staging-label">Staging Labels</a> in the <i>AWS Secrets Manager User Guide</i>.  The staging labels that you specify in the <code>VersionStage</code> parameter are added to the existing list of staging labels--they don't replace it. You can move the <code>AWSCURRENT</code> staging label to this version by including it in this call. <note> Whenever you move <code>AWSCURRENT</code>, Secrets Manager automatically moves the label <code>AWSPREVIOUS</code> to the version that <code>AWSCURRENT</code> was removed from. </note> If this action results in the last label being removed from a version, then the version is considered to be 'deprecated' and can be deleted by Secrets Manager.  <b>Minimum permissions</b>  To run this command, you must have the following permissions: <ul> <li> secretsmanager:UpdateSecretVersionStage </li> </ul>  <b>Related operations</b>  <ul> <li> To get the list of staging labels that are currently associated with a version of a secret, use <code> <a>DescribeSecret</a> </code> and examine the <code>SecretVersionsToStages</code> response value.  </li> </ul>

        :param self:
        :param MoveToVersionId:  (Optional) The secret version ID that you want to add the staging label to. If you want to remove a label from a version, then do not specify this parameter. If the staging label is already attached to a different version of the secret, then you must also specify the <code>RemoveFromVersionId</code> parameter. <br/>RemoveFromVersionId:  Specifies the secret version ID of the version that the staging label is to be removed from. If the staging label you are trying to attach to one version is already attached to a different version, then you must include this parameter and specify the version that the label is to be removed from. If the label is attached and you either do not specify this parameter, or the version ID does not match, then the operation fails.<br/>VersionStage:  The staging label to add to this version.<br/>SecretId:  Specifies the secret with the version whose list of staging labels you want to modify. You can specify either the Amazon Resource Name (ARN) or the friendly name of the secret. <note> If you specify an ARN, we generally recommend that you specify a complete ARN. You can specify a partial ARN too—for example, if you don’t include the final hyphen and six random characters that Secrets Manager adds at the end of the ARN when you created the secret. A partial ARN match can work as long as it uniquely matches only one secret. However, if your secret has a name that ends in a hyphen followed by six characters (before Secrets Manager adds the hyphen and six characters to the ARN) and you try to use that as a partial ARN, then those characters cause Secrets Manager to assume that you’re specifying a complete ARN. This confusion can cause unexpected results. To avoid this situation, we recommend that you don’t create secret names that end with a hyphen followed by six characters. </note><br/>
        :return: Name:  The friendly name of the secret with the staging label that was modified.<br/>ARN:  The ARN of the secret with the staging label that was modified.<br/>        """
        pass



class SecurityHub(BaseClient):
    """Security Hub provides you with a comprehensive view of the security state of your AWS environment and resources. It also provides you with the compliance status of your environment based on CIS AWS Foundations compliance checks. Security Hub collects security data from AWS accounts, services, and integrated third-party products and helps you analyze security trends in your environment to identify the highest priority security issues. For more information about Security Hub, see the <i> <a href="https://docs.aws.amazon.com/securityhub/latest/userguide/what-is-securityhub.html">AWS Security Hub User Guide</a> </i>. When you use operations in the Security Hub API, the requests are executed only in the AWS Region that is currently active or in the specific AWS Region that you specify in your request. Any configuration or settings change that results from the operation is applied only to that Region. To make the same change in other Regions, execute the same command for each Region to apply the change to. For example, if your Region is set to <code>us-west-2</code>, when you use <code>CreateMembers</code> to add a member account to Security Hub, the association of the member account with the master account is created only in the us-west-2 Region. Security Hub must be enabled for the member account in the same Region that the invite was sent from."""


    class GetEnabledStandardsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_enabled_standards` for valid parameters"""
            pass
    class GetFindingsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_findings` for valid parameters"""
            pass
    class GetInsightsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_insights` for valid parameters"""
            pass
    class ListEnabledProductsForImportPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_enabled_products_for_import` for valid parameters"""
            pass
    class ListInvitationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_invitations` for valid parameters"""
            pass
    class ListMembersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_members` for valid parameters"""
            pass


    class AcceptInvitationRequest(Mapping):
        pass
    
    class AcceptInvitationResponse(Mapping):
        pass
    
    class BatchDisableStandardsRequest(Mapping):
        pass
    
    class BatchDisableStandardsResponse(Mapping):
        pass
    
    class NonEmptyString(object):
        pass
    
    class BatchEnableStandardsRequest(Mapping):
        pass
    
    class BatchEnableStandardsResponse(Mapping):
        pass
    
    class StandardsSubscriptionRequest(Mapping):
        pass
    
    class BatchImportFindingsRequest(Mapping):
        pass
    
    class BatchImportFindingsResponse(Mapping):
        pass
    
    class AwsSecurityFinding(Mapping):
        pass
    
    class CreateActionTargetRequest(Mapping):
        pass
    
    class CreateActionTargetResponse(Mapping):
        pass
    
    class CreateInsightRequest(Mapping):
        pass
    
    class CreateInsightResponse(Mapping):
        pass
    
    class AwsSecurityFindingFilters(Mapping):
        pass
    
    class CreateMembersRequest(Mapping):
        pass
    
    class CreateMembersResponse(Mapping):
        pass
    
    class AccountDetails(Mapping):
        pass
    
    class DeclineInvitationsRequest(Mapping):
        pass
    
    class DeclineInvitationsResponse(Mapping):
        pass
    
    class DeleteActionTargetRequest(Mapping):
        pass
    
    class DeleteActionTargetResponse(Mapping):
        pass
    
    class DeleteInsightRequest(Mapping):
        pass
    
    class DeleteInsightResponse(Mapping):
        pass
    
    class DeleteInvitationsRequest(Mapping):
        pass
    
    class DeleteInvitationsResponse(Mapping):
        pass
    
    class DeleteMembersRequest(Mapping):
        pass
    
    class DeleteMembersResponse(Mapping):
        pass
    
    class DescribeActionTargetsRequest(Mapping):
        pass
    
    class DescribeActionTargetsResponse(Mapping):
        pass
    
    class DescribeHubRequest(Mapping):
        pass
    
    class DescribeHubResponse(Mapping):
        pass
    
    class DescribeProductsRequest(Mapping):
        pass
    
    class DescribeProductsResponse(Mapping):
        pass
    
    class DisableImportFindingsForProductRequest(Mapping):
        pass
    
    class DisableImportFindingsForProductResponse(Mapping):
        pass
    
    class DisableSecurityHubRequest(Mapping):
        pass
    
    class DisableSecurityHubResponse(Mapping):
        pass
    
    class DisassociateFromMasterAccountRequest(Mapping):
        pass
    
    class DisassociateFromMasterAccountResponse(Mapping):
        pass
    
    class DisassociateMembersRequest(Mapping):
        pass
    
    class DisassociateMembersResponse(Mapping):
        pass
    
    class EnableImportFindingsForProductRequest(Mapping):
        pass
    
    class EnableImportFindingsForProductResponse(Mapping):
        pass
    
    class EnableSecurityHubRequest(Mapping):
        pass
    
    class EnableSecurityHubResponse(Mapping):
        pass
    
    class TagMap(object):
        pass
    
    class GetEnabledStandardsRequest(Mapping):
        pass
    
    class GetEnabledStandardsResponse(Mapping):
        pass
    
    class GetFindingsRequest(Mapping):
        pass
    
    class GetFindingsResponse(Mapping):
        pass
    
    class SortCriterion(Mapping):
        pass
    
    class GetInsightResultsRequest(Mapping):
        pass
    
    class GetInsightResultsResponse(Mapping):
        pass
    
    class GetInsightsRequest(Mapping):
        pass
    
    class GetInsightsResponse(Mapping):
        pass
    
    class GetInvitationsCountRequest(Mapping):
        pass
    
    class GetInvitationsCountResponse(Mapping):
        pass
    
    class GetMasterAccountRequest(Mapping):
        pass
    
    class GetMasterAccountResponse(Mapping):
        pass
    
    class GetMembersRequest(Mapping):
        pass
    
    class GetMembersResponse(Mapping):
        pass
    
    class InviteMembersRequest(Mapping):
        pass
    
    class InviteMembersResponse(Mapping):
        pass
    
    class ListEnabledProductsForImportRequest(Mapping):
        pass
    
    class ListEnabledProductsForImportResponse(Mapping):
        pass
    
    class ListInvitationsRequest(Mapping):
        pass
    
    class ListInvitationsResponse(Mapping):
        pass
    
    class ListMembersRequest(Mapping):
        pass
    
    class ListMembersResponse(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateActionTargetRequest(Mapping):
        pass
    
    class UpdateActionTargetResponse(Mapping):
        pass
    
    class UpdateFindingsRequest(Mapping):
        pass
    
    class UpdateFindingsResponse(Mapping):
        pass
    
    class NoteUpdate(Mapping):
        pass
    
    class UpdateInsightRequest(Mapping):
        pass
    
    class UpdateInsightResponse(Mapping):
        pass
    
    def accept_invitation(self, MasterId: str=None, InvitationId: str=None) -> AcceptInvitationResponse:
        """Accepts the invitation to be a member account and be monitored by the Security Hub master account that the invitation was sent from. When the member account accepts the invitation, permission is granted to the master account to view findings generated in the member account.

        :param self:
        :param InvitationId:  The ID of the invitation sent from the Security Hub master account.<br/>MasterId:  The account ID of the Security Hub master account that sent the invitation.<br/>
        :return:         """
        pass

    def batch_disable_standards(self, StandardsSubscriptionArns: List[NonEmptyString]) -> BatchDisableStandardsResponse:
        """Disables the standards specified by the provided <code>StandardsSubscriptionArns</code>. For more information, see <a href="https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards.html">Standards Supported in AWS Security Hub</a>.

        :param self:
        :param StandardsSubscriptionArns:  The ARNs of the standards subscriptions to disable.<br/>
        :return: StandardsSubscriptions:  The details of the standards subscriptions that were disabled.<br/>        """
        pass

    def batch_enable_standards(self, StandardsSubscriptionRequests: List[StandardsSubscriptionRequest]) -> BatchEnableStandardsResponse:
        """Enables the standards specified by the provided <code>standardsArn</code>. In this release, only CIS AWS Foundations standards are supported. For more information, see <a href="https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards.html">Standards Supported in AWS Security Hub</a>.

        :param self:
        :param StandardsSubscriptionRequests:  The list of standards compliance checks to enable. <important> In this release, Security Hub supports only the CIS AWS Foundations standard. The ARN for the standard is <code>arn:aws:securityhub:::ruleset/cis-aws-foundations-benchmark/v/1.2.0</code>. </important><br/>
        :return: StandardsSubscriptions:  The details of the standards subscriptions that were enabled.<br/>        """
        pass

    def batch_import_findings(self, Findings: List[AwsSecurityFinding]) -> BatchImportFindingsResponse:
        """Imports security findings generated from an integrated third-party product into Security Hub. This action is requested by the integrated product to import its findings into Security Hub. The maximum allowed size for a finding is 240 Kb. An error is returned for any finding larger than 240 Kb.

        :param self:
        :param Findings:  A list of findings to import. To successfully import a finding, it must follow the <a href="https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-findings-format.html">AWS Security Finding Format</a>.<br/>
        :return: FailedFindings:  The list of the findings that failed to import.<br/>SuccessCount:  The number of findings that were successfully imported.<br/>FailedCount:  The number of findings that failed to import.<br/>        """
        pass

    def create_action_target(self, Id: str, Description: str, Name: str) -> CreateActionTargetResponse:
        """Creates a custom action target in Security Hub. You can use custom actions on findings and insights in Security Hub to trigger target actions in Amazon CloudWatch Events.

        :param self:
        :param Id:  The ID for the custom action target.<br/>Description:  The description for the custom action target.<br/>Name:  The name of the custom action target.<br/>
        :return: ActionTargetArn:  The ARN for the custom action target.<br/>        """
        pass

    def create_insight(self, GroupByAttribute: str, Filters: AwsSecurityFindingFilters, Name: str) -> CreateInsightResponse:
        """Creates a custom insight in Security Hub. An insight is a consolidation of findings that relate to a security issue that requires attention or remediation. Use the <code>GroupByAttribute</code> to group the related findings in the insight.

        :param self:
        :param GroupByAttribute:  The attribute used as the aggregator to group related findings for the insight.<br/>Filters:  One or more attributes used to filter the findings included in the insight. Only findings that match the criteria defined in the filters are included in the insight.:<br/>Name:  The name of the custom insight to create.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Keyword</i> A keyword for a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NoteUpdatedBy</i> The principal that created a note.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NoteUpdatedAt</i> The timestamp of when the note was updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NoteText</i> The text of a note.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RelatedFindingsId</i> The solution-generated identifier for a related finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RelatedFindingsProductArn</i> The ARN of the solution that generated a related finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordState</i> The updated record state for the finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WorkflowState</i> The workflow state of a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VerificationState</i> The veracity of a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ComplianceStatus</i> Exclusive to findings that are generated as the result of a check run against a specific rule in a supported standard (for example, CIS AWS Foundations). Contains compliance-related finding details.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceDetailsOther</i> The details of a resource that doesn't have a specific subfield for the resource type defined.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceContainerLaunchedAt</i> The date/time that the container was started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceContainerImageName</i> The name of the image related to a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceContainerImageId</i> The identifier of the image related to a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceContainerName</i> The name of the container related to a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsIamAccessKeyCreatedAt</i> The creation date/time of the IAM access key related to a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsIamAccessKeyStatus</i> The status of the IAM access key related to a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsIamAccessKeyUserName</i> The user associated with the IAM access key related to a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsS3BucketOwnerName</i> The display name of the owner of the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsS3BucketOwnerId</i> The canonical user ID of the owner of the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceLaunchedAt</i> The date/time the instance was launched.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceSubnetId</i> The identifier of the subnet that the instance was launched in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceVpcId</i> The identifier of the VPC that the instance was launched in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceIamInstanceProfileArn</i> The IAM profile ARN of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceKeyName</i> The key name associated with the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceIpV6Addresses</i> The IPv6 addresses associated with the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceIpV4Addresses</i> The IPv4 addresses associated with the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceImageId</i> The Amazon Machine Image (AMI) ID of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceType</i> The instance type of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceTags</i> A list of AWS tags associated with a resource at the time the finding was processed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceRegion</i> The canonical AWS external Region name where this resource is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourcePartition</i> The canonical AWS partition name that the Region is assigned to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceId</i> The canonical identifier for the given resource type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceType</i> Specifies the type of the resource that details are provided for.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThreatIntelIndicatorSourceUrl</i> The URL for more details from the source of the threat intel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThreatIntelIndicatorSource</i> The source of the threat intel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThreatIntelIndicatorLastObservedAt</i> The date/time of the last observation of a threat intel indicator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThreatIntelIndicatorCategory</i> The category of a threat intel indicator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThreatIntelIndicatorValue</i> The value of a threat intel indicator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThreatIntelIndicatorType</i> The type of a threat intel indicator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessTerminatedAt</i> The date/time that the process was terminated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessLaunchedAt</i> The date/time that the process was launched.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessParentPid</i> The parent process ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessPid</i> The process ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessPath</i> The path to the process executable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessName</i> The name of the process.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkDestinationDomain</i> The destination domain of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkDestinationPort</i> The destination port of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkDestinationIpV6</i> The destination IPv6 address of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkDestinationIpV4</i> The destination IPv4 address of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkSourceMac</i> The source media access control (MAC) address of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkSourceDomain</i> The source domain of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkSourcePort</i> The source port of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkSourceIpV6</i> The source IPv6 address of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkSourceIpV4</i> The source IPv4 address of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkProtocol</i> The protocol of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkDirection</i> Indicates the direction of network traffic associated with a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MalwareState</i> The state of the malware that was observed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MalwarePath</i> The filesystem path of the malware that was observed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MalwareType</i> The type of the malware that was observed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MalwareName</i> The name of the malware that was observed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserDefinedFields</i> A list of name/value string pairs associated with the finding. These are custom, user-defined fields added to a finding. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompanyName</i> The name of the findings provider (company) that owns the solution (product) that generates findings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductName</i> The name of the solution (product) that generates findings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductFields</i> A data type where security-findings providers can include additional solution-specific details that aren't part of the defined <code>AwsSecurityFinding</code> format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceUrl</i> A URL that links to a page about the current finding in the security-findings provider's solution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecommendationText</i> The recommendation of what to do about the issue described in a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A finding's description.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Title</i> A finding's title.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Criticality</i> The level of importance assigned to the resources associated with the finding. A score of 0 means that the underlying resources have no criticality, and a score of 100 is reserved for the most critical resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Confidence</i> A finding's confidence. Confidence is defined as the likelihood that a finding accurately identifies the behavior or issue that it was intended to identify. Confidence is scored on a 0-100 basis using a ratio scale, where 0 means zero percent confidence and 100 means 100 percent confidence.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SeverityLabel</i> The label of a finding's severity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SeverityNormalized</i> The normalized severity of a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SeverityProduct</i> The native severity as defined by the security-findings provider's solution that generated the finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdatedAt</i> An ISO8601-formatted timestamp that indicates when the security-findings provider last updated the finding record. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i> An ISO8601-formatted timestamp that indicates when the security-findings provider captured the potential security issue that a finding captured.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastObservedAt</i> An ISO8601-formatted timestamp that indicates when the security-findings provider most recently observed the potential security issue that a finding captured.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FirstObservedAt</i> An ISO8601-formatted timestamp that indicates when the security-findings provider first observed the potential security issue that a finding captured.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> A finding type in the format of <code>namespace/category/classifier</code> that classifies a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GeneratorId</i> The identifier for the solution-specific component (a discrete unit of logic) that generated a finding. In various security-findings providers' solutions, this generator can be called a rule, a check, a detector, a plug-in, etc.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The security findings provider-specific identifier for a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AwsAccountId</i> The AWS account ID that a finding is generated in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductArn</i> The ARN generated by Security Hub that uniquely identifies a third-party company (security findings provider) after this provider's product (solution that generates findings) is registered with Security Hub.<br/>
        :return: InsightArn:  The ARN of the insight created.<br/>        """
        pass

    def create_members(self, AccountDetails: List[AccountDetails]=None) -> CreateMembersResponse:
        """Creates a member association in Security Hub between the specified accounts and the account used to make the request, which is the master account. To successfully create a member, you must use this action from an account that already has Security Hub enabled. You can use the <a>EnableSecurityHub</a> to enable Security Hub. After you use <code>CreateMembers</code> to create member account associations in Security Hub, you need to use the <a>InviteMembers</a> action, which invites the accounts to enable Security Hub and become member accounts in Security Hub. If the invitation is accepted by the account owner, the account becomes a member account in Security Hub, and a permission policy is added that permits the master account to view the findings generated in the member account. When Security Hub is enabled in the invited account, findings start being sent to both the member and master accounts. You can remove the association between the master and member accounts by using the <a>DisassociateFromMasterAccount</a> or <a>DisassociateMembers</a> operation.

        :param self:
        :param AccountDetails:  A list of account ID and email address pairs of the accounts to associate with the Security Hub master account.<br/>
        :return: UnprocessedAccounts:  A list of account ID and email address pairs of the AWS accounts that weren't processed.<br/>        """
        pass

    def decline_invitations(self, AccountIds: List[NonEmptyString]=None) -> DeclineInvitationsResponse:
        """Declines invitations to become a member account.

        :param self:
        :param AccountIds:  A list of account IDs that specify the accounts that invitations to Security Hub are declined from.<br/>
        :return: UnprocessedAccounts:  A list of account ID and email address pairs of the AWS accounts that weren't processed.<br/>        """
        pass

    def delete_action_target(self, ActionTargetArn: str) -> DeleteActionTargetResponse:
        """Deletes a custom action target from Security Hub. Deleting a custom action target doesn't affect any findings or insights that were already sent to Amazon CloudWatch Events using the custom action.

        :param self:
        :param ActionTargetArn:  The ARN of the custom action target to delete.<br/>
        :return: ActionTargetArn:  The ARN of the custom action target that was deleted.<br/>        """
        pass

    def delete_insight(self, InsightArn: str) -> DeleteInsightResponse:
        """Deletes the insight specified by the <code>InsightArn</code>.

        :param self:
        :param InsightArn:  The ARN of the insight to delete.<br/>
        :return: InsightArn:  The ARN of the insight that was deleted.<br/>        """
        pass

    def delete_invitations(self, AccountIds: List[NonEmptyString]=None) -> DeleteInvitationsResponse:
        """Deletes invitations received by the AWS account to become a member account.

        :param self:
        :param AccountIds:  A list of the account IDs that sent the invitations to delete.<br/>
        :return: UnprocessedAccounts:  A list of account ID and email address pairs of the AWS accounts that invitations weren't deleted for.<br/>        """
        pass

    def delete_members(self, AccountIds: List[NonEmptyString]=None) -> DeleteMembersResponse:
        """Deletes the specified member accounts from Security Hub.

        :param self:
        :param AccountIds:  A list of account IDs of the member accounts to delete.<br/>
        :return: UnprocessedAccounts:  A list of account ID and email address pairs of the AWS accounts that weren't deleted.<br/>        """
        pass

    def describe_action_targets(self, ActionTargetArns: List[NonEmptyString]=None, NextToken: str=None, MaxResults: int=None) -> DescribeActionTargetsResponse:
        """Returns a list of the custom action targets in Security Hub in your account.

        :param self:
        :param MaxResults:  The maximum number of results to return.<br/>NextToken:  The token that is required for pagination.<br/>ActionTargetArns:  A list of custom action target ARNs for the custom action targets to retrieve.<br/>
        :return: NextToken:  The token that is required for pagination.<br/>ActionTargets:  A list of <code>ActionTarget</code> objects. Each object includes the <code>ActionTargetArn</code>, <code>Description</code>, and <code>Name</code> of a custom action target available in Security Hub.<br/>        """
        pass

    def describe_hub(self, HubArn: str=None) -> DescribeHubResponse:
        """Returns details about the Hub resource in your account, including the <code>HubArn</code> and the time when you enabled Security Hub.

        :param self:
        :param HubArn:  The ARN of the Hub resource to retrieve.<br/>
        :return: SubscribedAt:  The date and time when Security Hub was enabled in the account.<br/>HubArn:  The ARN of the Hub resource retrieved.<br/>        """
        pass

    def describe_products(self, NextToken: str=None, MaxResults: int=None) -> DescribeProductsResponse:
        """Returns information about the products available that you can subscribe to and integrate with Security Hub to consolidate findings.

        :param self:
        :param MaxResults:  The maximum number of results to return.<br/>NextToken:  The token that is required for pagination.<br/>
        :return: NextToken:  The token that is required for pagination.<br/>Products:  A list of products, including details for each product.<br/>        """
        pass

    def disable_import_findings_for_product(self, ProductSubscriptionArn: str) -> DisableImportFindingsForProductResponse:
        """Disables the integration of the specified product with Security Hub. Findings from that product are no longer sent to Security Hub after the integration is disabled.

        :param self:
        :param ProductSubscriptionArn:  The ARN of the integrated product to disable the integration for.<br/>
        :return:         """
        pass

    def disable_security_hub(self) -> DisableSecurityHubResponse:
        """Disables Security Hub in your account only in the current Region. To disable Security Hub in all Regions, you must submit one request per Region where you have enabled Security Hub. When you disable Security Hub for a master account, it doesn't disable Security Hub for any associated member accounts. When you disable Security Hub, your existing findings and insights and any Security Hub configuration settings are deleted after 90 days and can't be recovered. Any standards that were enabled are disabled, and your master and member account associations are removed. If you want to save your existing findings, you must export them before you disable Security Hub.

        :param self:
        :param 
        :return:         """
        pass

    def disassociate_from_master_account(self) -> DisassociateFromMasterAccountResponse:
        """Disassociates the current Security Hub member account from the associated master account.

        :param self:
        :param 
        :return:         """
        pass

    def disassociate_members(self, AccountIds: List[NonEmptyString]=None) -> DisassociateMembersResponse:
        """Disassociates the specified member accounts from the associated master account.

        :param self:
        :param AccountIds:  The account IDs of the member accounts to disassociate from the master account.<br/>
        :return:         """
        pass

    def enable_import_findings_for_product(self, ProductArn: str) -> EnableImportFindingsForProductResponse:
        """Enables the integration of a partner product with Security Hub. Integrated products send findings to Security Hub. When you enable a product integration, a permission policy that grants permission for the product to send findings to Security Hub is applied.

        :param self:
        :param ProductArn:  The ARN of the product to enable the integration for.<br/>
        :return: ProductSubscriptionArn:  The ARN of your subscription to the product to enable integrations for.<br/>        """
        pass

    def enable_security_hub(self, Tags: TagMap=None) -> EnableSecurityHubResponse:
        """Enables Security Hub for your account in the current Region or the Region you specify in the request. When you enable Security Hub, you grant to Security Hub the permissions necessary to gather findings from AWS Config, Amazon GuardDuty, Amazon Inspector, and Amazon Macie. To learn more, see <a href="https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-settingup.html">Setting Up AWS Security Hub</a>.

        :param self:
        :param Tags:  The tags to add to the Hub resource when you enable Security Hub.<br/>
        :return:         """
        pass

    def get_enabled_standards(self, StandardsSubscriptionArns: List[NonEmptyString]=None, NextToken: str=None, MaxResults: int=None) -> GetEnabledStandardsResponse:
        """Returns a list of the standards that are currently enabled.

        :param self:
        :param MaxResults:  The maximum number of results to return in the response.<br/>NextToken:  Paginates results. On your first call to the <code>GetEnabledStandards</code> operation, set the value of this parameter to <code>NULL</code>. For subsequent calls to the operation, fill <code>nextToken</code> in the request with the value of <code>nextToken</code> from the previous response to continue listing data.<br/>StandardsSubscriptionArns:  A list of the standards subscription ARNs for the standards to retrieve.<br/>
        :return: NextToken:  The token that is required for pagination.<br/>StandardsSubscriptions:  A list of <code>StandardsSubscriptions</code> objects that include information about the enabled standards.<br/>        """
        pass

    def get_findings(self, Filters: AwsSecurityFindingFilters=None, SortCriteria: List[SortCriterion]=None, NextToken: str=None, MaxResults: int=None) -> GetFindingsResponse:
        """Returns a list of findings that match the specified criteria.

        :param self:
        :param MaxResults:  The maximum number of findings to return.<br/>NextToken:  Paginates results. On your first call to the <code>GetFindings</code> operation, set the value of this parameter to <code>NULL</code>. For subsequent calls to the operation, fill <code>nextToken</code> in the request with the value of <code>nextToken</code> from the previous response to continue listing data.<br/>SortCriteria:  Findings attributes used to sort the list of findings returned.<br/>Filters:  The findings attributes used to define a condition to filter the findings returned.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Keyword</i> A keyword for a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NoteUpdatedBy</i> The principal that created a note.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NoteUpdatedAt</i> The timestamp of when the note was updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NoteText</i> The text of a note.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RelatedFindingsId</i> The solution-generated identifier for a related finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RelatedFindingsProductArn</i> The ARN of the solution that generated a related finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordState</i> The updated record state for the finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WorkflowState</i> The workflow state of a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VerificationState</i> The veracity of a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ComplianceStatus</i> Exclusive to findings that are generated as the result of a check run against a specific rule in a supported standard (for example, CIS AWS Foundations). Contains compliance-related finding details.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceDetailsOther</i> The details of a resource that doesn't have a specific subfield for the resource type defined.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceContainerLaunchedAt</i> The date/time that the container was started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceContainerImageName</i> The name of the image related to a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceContainerImageId</i> The identifier of the image related to a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceContainerName</i> The name of the container related to a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsIamAccessKeyCreatedAt</i> The creation date/time of the IAM access key related to a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsIamAccessKeyStatus</i> The status of the IAM access key related to a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsIamAccessKeyUserName</i> The user associated with the IAM access key related to a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsS3BucketOwnerName</i> The display name of the owner of the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsS3BucketOwnerId</i> The canonical user ID of the owner of the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceLaunchedAt</i> The date/time the instance was launched.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceSubnetId</i> The identifier of the subnet that the instance was launched in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceVpcId</i> The identifier of the VPC that the instance was launched in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceIamInstanceProfileArn</i> The IAM profile ARN of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceKeyName</i> The key name associated with the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceIpV6Addresses</i> The IPv6 addresses associated with the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceIpV4Addresses</i> The IPv4 addresses associated with the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceImageId</i> The Amazon Machine Image (AMI) ID of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceType</i> The instance type of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceTags</i> A list of AWS tags associated with a resource at the time the finding was processed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceRegion</i> The canonical AWS external Region name where this resource is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourcePartition</i> The canonical AWS partition name that the Region is assigned to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceId</i> The canonical identifier for the given resource type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceType</i> Specifies the type of the resource that details are provided for.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThreatIntelIndicatorSourceUrl</i> The URL for more details from the source of the threat intel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThreatIntelIndicatorSource</i> The source of the threat intel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThreatIntelIndicatorLastObservedAt</i> The date/time of the last observation of a threat intel indicator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThreatIntelIndicatorCategory</i> The category of a threat intel indicator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThreatIntelIndicatorValue</i> The value of a threat intel indicator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThreatIntelIndicatorType</i> The type of a threat intel indicator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessTerminatedAt</i> The date/time that the process was terminated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessLaunchedAt</i> The date/time that the process was launched.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessParentPid</i> The parent process ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessPid</i> The process ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessPath</i> The path to the process executable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessName</i> The name of the process.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkDestinationDomain</i> The destination domain of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkDestinationPort</i> The destination port of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkDestinationIpV6</i> The destination IPv6 address of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkDestinationIpV4</i> The destination IPv4 address of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkSourceMac</i> The source media access control (MAC) address of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkSourceDomain</i> The source domain of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkSourcePort</i> The source port of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkSourceIpV6</i> The source IPv6 address of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkSourceIpV4</i> The source IPv4 address of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkProtocol</i> The protocol of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkDirection</i> Indicates the direction of network traffic associated with a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MalwareState</i> The state of the malware that was observed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MalwarePath</i> The filesystem path of the malware that was observed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MalwareType</i> The type of the malware that was observed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MalwareName</i> The name of the malware that was observed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserDefinedFields</i> A list of name/value string pairs associated with the finding. These are custom, user-defined fields added to a finding. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompanyName</i> The name of the findings provider (company) that owns the solution (product) that generates findings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductName</i> The name of the solution (product) that generates findings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductFields</i> A data type where security-findings providers can include additional solution-specific details that aren't part of the defined <code>AwsSecurityFinding</code> format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceUrl</i> A URL that links to a page about the current finding in the security-findings provider's solution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecommendationText</i> The recommendation of what to do about the issue described in a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A finding's description.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Title</i> A finding's title.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Criticality</i> The level of importance assigned to the resources associated with the finding. A score of 0 means that the underlying resources have no criticality, and a score of 100 is reserved for the most critical resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Confidence</i> A finding's confidence. Confidence is defined as the likelihood that a finding accurately identifies the behavior or issue that it was intended to identify. Confidence is scored on a 0-100 basis using a ratio scale, where 0 means zero percent confidence and 100 means 100 percent confidence.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SeverityLabel</i> The label of a finding's severity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SeverityNormalized</i> The normalized severity of a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SeverityProduct</i> The native severity as defined by the security-findings provider's solution that generated the finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdatedAt</i> An ISO8601-formatted timestamp that indicates when the security-findings provider last updated the finding record. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i> An ISO8601-formatted timestamp that indicates when the security-findings provider captured the potential security issue that a finding captured.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastObservedAt</i> An ISO8601-formatted timestamp that indicates when the security-findings provider most recently observed the potential security issue that a finding captured.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FirstObservedAt</i> An ISO8601-formatted timestamp that indicates when the security-findings provider first observed the potential security issue that a finding captured.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> A finding type in the format of <code>namespace/category/classifier</code> that classifies a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GeneratorId</i> The identifier for the solution-specific component (a discrete unit of logic) that generated a finding. In various security-findings providers' solutions, this generator can be called a rule, a check, a detector, a plug-in, etc.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The security findings provider-specific identifier for a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AwsAccountId</i> The AWS account ID that a finding is generated in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductArn</i> The ARN generated by Security Hub that uniquely identifies a third-party company (security findings provider) after this provider's product (solution that generates findings) is registered with Security Hub.<br/>
        :return: NextToken:  The token that is required for pagination.<br/>Findings:  The findings that matched the filters specified in the request.<br/>        """
        pass

    def get_insight_results(self, InsightArn: str) -> GetInsightResultsResponse:
        """Lists the results of the Security Hub insight that the insight ARN specifies.

        :param self:
        :param InsightArn:  The ARN of the insight whose results you want to see.<br/>
        :return: InsightResults:  The insight results returned by the operation.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResultValues</i> The list of insight result values returned by the <code>GetInsightResults</code> operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupByAttribute</i> The attribute that the findings are grouped by for the insight whose results are returned by the <code>GetInsightResults</code> operation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InsightArn</i> The ARN of the insight whose results are returned by the <code>GetInsightResults</code> operation.<br/>        """
        pass

    def get_insights(self, InsightArns: List[NonEmptyString]=None, NextToken: str=None, MaxResults: int=None) -> GetInsightsResponse:
        """Lists and describes insights that insight ARNs specify.

        :param self:
        :param MaxResults:  The maximum number of items that you want in the response.<br/>NextToken:  Paginates results. On your first call to the <code>GetInsights</code> operation, set the value of this parameter to <code>NULL</code>. For subsequent calls to the operation, fill <code>nextToken</code> in the request with the value of <code>nextToken</code> from the previous response to continue listing data.<br/>InsightArns:  The ARNs of the insights that you want to describe.<br/>
        :return: NextToken:  The token that is required for pagination.<br/>Insights:  The insights returned by the operation.<br/>        """
        pass

    def get_invitations_count(self) -> GetInvitationsCountResponse:
        """Returns the count of all Security Hub membership invitations that were sent to the current member account, not including the currently accepted invitation. 

        :param self:
        :param 
        :return: InvitationsCount:  The number of all membership invitations sent to this Security Hub member account, not including the currently accepted invitation. <br/>        """
        pass

    def get_master_account(self) -> GetMasterAccountResponse:
        """Provides the details for the Security Hub master account to the current member account. 

        :param self:
        :param 
        :return: Master:  A list of details about the Security Hub master account for the current member account. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MemberStatus</i> The current status of the association between member and master accounts.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InvitedAt</i> The timestamp of when the invitation was sent.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InvitationId</i> The ID of the invitation sent to the member account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccountId</i> The account ID of the Security Hub master account that the invitation was sent from.<br/>        """
        pass

    def get_members(self, AccountIds: List[NonEmptyString]) -> GetMembersResponse:
        """Returns the details on the Security Hub member accounts that the account IDs specify.

        :param self:
        :param AccountIds:  A list of account IDs for the Security Hub member accounts that you want to return the details for. <br/>
        :return: UnprocessedAccounts:  A list of account ID and email address pairs of the AWS accounts that couldn't be processed.<br/>Members:  A list of details about the Security Hub member accounts.<br/>        """
        pass

    def invite_members(self, AccountIds: List[NonEmptyString]=None) -> InviteMembersResponse:
        """Invites other AWS accounts to become member accounts for the Security Hub master account that the invitation is sent from. Before you can use this action to invite a member, you must first create the member account in Security Hub by using the <a>CreateMembers</a> action. When the account owner accepts the invitation to become a member account and enables Security Hub, the master account can view the findings generated from member account.

        :param self:
        :param AccountIds:  A list of IDs of the AWS accounts that you want to invite to Security Hub as members. <br/>
        :return: UnprocessedAccounts:  A list of account ID and email address pairs of the AWS accounts that couldn't be processed. <br/>        """
        pass

    def list_enabled_products_for_import(self, NextToken: str=None, MaxResults: int=None) -> ListEnabledProductsForImportResponse:
        """Lists all findings-generating solutions (products) whose findings you have subscribed to receive in Security Hub.

        :param self:
        :param MaxResults:  The maximum number of items that you want in the response.<br/>NextToken:  Paginates results. On your first call to the <code>ListEnabledProductsForImport</code> operation, set the value of this parameter to <code>NULL</code>. For subsequent calls to the operation, fill <code>nextToken</code> in the request with the value of <code>NextToken</code> from the previous response to continue listing data.<br/>
        :return: NextToken:  The token that is required for pagination.<br/>ProductSubscriptions:  A list of ARNs for the resources that represent your subscriptions to products. <br/>        """
        pass

    def list_invitations(self, MaxResults: int=None, NextToken: str=None) -> ListInvitationsResponse:
        """Lists all Security Hub membership invitations that were sent to the current AWS account. 

        :param self:
        :param NextToken:  Paginates results. On your first call to the <code>ListInvitations</code> operation, set the value of this parameter to <code>NULL</code>. For subsequent calls to the operation, fill <code>nextToken</code> in the request with the value of <code>NextToken</code> from the previous response to continue listing data. <br/>MaxResults:  The maximum number of items that you want in the response. <br/>
        :return: NextToken:  The token that is required for pagination.<br/>Invitations:  The details of the invitations returned by the operation.<br/>        """
        pass

    def list_members(self, OnlyAssociated: Boolean=None, MaxResults: int=None, NextToken: str=None) -> ListMembersResponse:
        """Lists details about all member accounts for the current Security Hub master account.

        :param self:
        :param NextToken:  Paginates results. Set the value of this parameter to <code>NULL</code> on your first call to the <code>ListMembers</code> operation. For subsequent calls to the operation, fill <code>nextToken</code> in the request with the value of <code>nextToken</code> from the previous response to continue listing data. <br/>MaxResults:  The maximum number of items that you want in the response. <br/>OnlyAssociated:  Specifies which member accounts the response includes based on their relationship status with the master account. The default value is <code>TRUE</code>. If <code>onlyAssociated</code> is set to <code>TRUE</code>, the response includes member accounts whose relationship status with the master is set to <code>ENABLED</code> or <code>DISABLED</code>. If <code>onlyAssociated</code> is set to <code>FALSE</code>, the response includes all existing member accounts. <br/>
        :return: NextToken:  The token that is required for pagination.<br/>Members:  Member details returned by the operation.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceArn: str) -> ListTagsForResourceResponse:
        """Returns a list of tags associated with a resource.

        :param self:
        :param ResourceArn:  The ARN of the resource to retrieve tags for.<br/>
        :return: Tags:  The tags associated with a resource.<br/>        """
        pass

    def tag_resource(self, Tags: TagMap, ResourceArn: str) -> TagResourceResponse:
        """Adds one or more tags to a resource.

        :param self:
        :param Tags:  The tags to add to the resource.<br/>ResourceArn:  The ARN of the resource to apply the tags to.<br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[TagKey], ResourceArn: str) -> UntagResourceResponse:
        """Removes one or more tags from a resource.

        :param self:
        :param TagKeys:  The tag keys associated with the tags to remove from the resource.<br/>ResourceArn:  The ARN of the resource to remove the tags from.<br/>
        :return:         """
        pass

    def update_action_target(self, ActionTargetArn: str, Name: str=None, Description: str=None) -> UpdateActionTargetResponse:
        """Updates the name and description of a custom action target in Security Hub.

        :param self:
        :param Description:  The updated description for the custom action target.<br/>Name:  The updated name of the custom action target.<br/>ActionTargetArn:  The ARN of the custom action target to update.<br/>
        :return:         """
        pass

    def update_findings(self, Filters: AwsSecurityFindingFilters, Note: NoteUpdate=None, RecordState: str=None) -> UpdateFindingsResponse:
        """Updates the <code>Note</code> and <code>RecordState</code> of the Security Hub-aggregated findings that the filter attributes specify. Any member account that can view the finding also sees the update to the finding.

        :param self:
        :param RecordState:  The updated record state for the finding.<br/>Note:  The updated note for the finding.:<br/>Filters:  A collection of attributes that specify which findings you want to update.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Keyword</i> A keyword for a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NoteUpdatedBy</i> The principal that created a note.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NoteUpdatedAt</i> The timestamp of when the note was updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NoteText</i> The text of a note.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RelatedFindingsId</i> The solution-generated identifier for a related finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RelatedFindingsProductArn</i> The ARN of the solution that generated a related finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordState</i> The updated record state for the finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WorkflowState</i> The workflow state of a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VerificationState</i> The veracity of a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ComplianceStatus</i> Exclusive to findings that are generated as the result of a check run against a specific rule in a supported standard (for example, CIS AWS Foundations). Contains compliance-related finding details.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceDetailsOther</i> The details of a resource that doesn't have a specific subfield for the resource type defined.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceContainerLaunchedAt</i> The date/time that the container was started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceContainerImageName</i> The name of the image related to a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceContainerImageId</i> The identifier of the image related to a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceContainerName</i> The name of the container related to a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsIamAccessKeyCreatedAt</i> The creation date/time of the IAM access key related to a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsIamAccessKeyStatus</i> The status of the IAM access key related to a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsIamAccessKeyUserName</i> The user associated with the IAM access key related to a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsS3BucketOwnerName</i> The display name of the owner of the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsS3BucketOwnerId</i> The canonical user ID of the owner of the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceLaunchedAt</i> The date/time the instance was launched.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceSubnetId</i> The identifier of the subnet that the instance was launched in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceVpcId</i> The identifier of the VPC that the instance was launched in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceIamInstanceProfileArn</i> The IAM profile ARN of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceKeyName</i> The key name associated with the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceIpV6Addresses</i> The IPv6 addresses associated with the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceIpV4Addresses</i> The IPv4 addresses associated with the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceImageId</i> The Amazon Machine Image (AMI) ID of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceType</i> The instance type of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceTags</i> A list of AWS tags associated with a resource at the time the finding was processed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceRegion</i> The canonical AWS external Region name where this resource is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourcePartition</i> The canonical AWS partition name that the Region is assigned to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceId</i> The canonical identifier for the given resource type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceType</i> Specifies the type of the resource that details are provided for.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThreatIntelIndicatorSourceUrl</i> The URL for more details from the source of the threat intel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThreatIntelIndicatorSource</i> The source of the threat intel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThreatIntelIndicatorLastObservedAt</i> The date/time of the last observation of a threat intel indicator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThreatIntelIndicatorCategory</i> The category of a threat intel indicator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThreatIntelIndicatorValue</i> The value of a threat intel indicator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThreatIntelIndicatorType</i> The type of a threat intel indicator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessTerminatedAt</i> The date/time that the process was terminated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessLaunchedAt</i> The date/time that the process was launched.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessParentPid</i> The parent process ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessPid</i> The process ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessPath</i> The path to the process executable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessName</i> The name of the process.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkDestinationDomain</i> The destination domain of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkDestinationPort</i> The destination port of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkDestinationIpV6</i> The destination IPv6 address of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkDestinationIpV4</i> The destination IPv4 address of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkSourceMac</i> The source media access control (MAC) address of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkSourceDomain</i> The source domain of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkSourcePort</i> The source port of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkSourceIpV6</i> The source IPv6 address of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkSourceIpV4</i> The source IPv4 address of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkProtocol</i> The protocol of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkDirection</i> Indicates the direction of network traffic associated with a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MalwareState</i> The state of the malware that was observed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MalwarePath</i> The filesystem path of the malware that was observed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MalwareType</i> The type of the malware that was observed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MalwareName</i> The name of the malware that was observed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserDefinedFields</i> A list of name/value string pairs associated with the finding. These are custom, user-defined fields added to a finding. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompanyName</i> The name of the findings provider (company) that owns the solution (product) that generates findings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductName</i> The name of the solution (product) that generates findings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductFields</i> A data type where security-findings providers can include additional solution-specific details that aren't part of the defined <code>AwsSecurityFinding</code> format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceUrl</i> A URL that links to a page about the current finding in the security-findings provider's solution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecommendationText</i> The recommendation of what to do about the issue described in a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A finding's description.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Title</i> A finding's title.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Criticality</i> The level of importance assigned to the resources associated with the finding. A score of 0 means that the underlying resources have no criticality, and a score of 100 is reserved for the most critical resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Confidence</i> A finding's confidence. Confidence is defined as the likelihood that a finding accurately identifies the behavior or issue that it was intended to identify. Confidence is scored on a 0-100 basis using a ratio scale, where 0 means zero percent confidence and 100 means 100 percent confidence.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SeverityLabel</i> The label of a finding's severity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SeverityNormalized</i> The normalized severity of a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SeverityProduct</i> The native severity as defined by the security-findings provider's solution that generated the finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdatedAt</i> An ISO8601-formatted timestamp that indicates when the security-findings provider last updated the finding record. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i> An ISO8601-formatted timestamp that indicates when the security-findings provider captured the potential security issue that a finding captured.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastObservedAt</i> An ISO8601-formatted timestamp that indicates when the security-findings provider most recently observed the potential security issue that a finding captured.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FirstObservedAt</i> An ISO8601-formatted timestamp that indicates when the security-findings provider first observed the potential security issue that a finding captured.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> A finding type in the format of <code>namespace/category/classifier</code> that classifies a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GeneratorId</i> The identifier for the solution-specific component (a discrete unit of logic) that generated a finding. In various security-findings providers' solutions, this generator can be called a rule, a check, a detector, a plug-in, etc.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The security findings provider-specific identifier for a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AwsAccountId</i> The AWS account ID that a finding is generated in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductArn</i> The ARN generated by Security Hub that uniquely identifies a third-party company (security findings provider) after this provider's product (solution that generates findings) is registered with Security Hub.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdatedBy</i> The principal that updated the note.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Text</i> The updated note text.<br/>
        :return:         """
        pass

    def update_insight(self, InsightArn: str, Name: str=None, Filters: AwsSecurityFindingFilters=None, GroupByAttribute: str=None) -> UpdateInsightResponse:
        """Updates the Security Hub insight that the insight ARN specifies.

        :param self:
        :param GroupByAttribute:  The updated <code>GroupBy</code> attribute that defines this insight.<br/>Filters:  The updated filters that define this insight.:<br/>Name:  The updated name for the insight.<br/>InsightArn:  The ARN of the insight that you want to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Keyword</i> A keyword for a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NoteUpdatedBy</i> The principal that created a note.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NoteUpdatedAt</i> The timestamp of when the note was updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NoteText</i> The text of a note.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RelatedFindingsId</i> The solution-generated identifier for a related finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RelatedFindingsProductArn</i> The ARN of the solution that generated a related finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordState</i> The updated record state for the finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WorkflowState</i> The workflow state of a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VerificationState</i> The veracity of a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ComplianceStatus</i> Exclusive to findings that are generated as the result of a check run against a specific rule in a supported standard (for example, CIS AWS Foundations). Contains compliance-related finding details.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceDetailsOther</i> The details of a resource that doesn't have a specific subfield for the resource type defined.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceContainerLaunchedAt</i> The date/time that the container was started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceContainerImageName</i> The name of the image related to a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceContainerImageId</i> The identifier of the image related to a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceContainerName</i> The name of the container related to a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsIamAccessKeyCreatedAt</i> The creation date/time of the IAM access key related to a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsIamAccessKeyStatus</i> The status of the IAM access key related to a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsIamAccessKeyUserName</i> The user associated with the IAM access key related to a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsS3BucketOwnerName</i> The display name of the owner of the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsS3BucketOwnerId</i> The canonical user ID of the owner of the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceLaunchedAt</i> The date/time the instance was launched.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceSubnetId</i> The identifier of the subnet that the instance was launched in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceVpcId</i> The identifier of the VPC that the instance was launched in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceIamInstanceProfileArn</i> The IAM profile ARN of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceKeyName</i> The key name associated with the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceIpV6Addresses</i> The IPv6 addresses associated with the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceIpV4Addresses</i> The IPv4 addresses associated with the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceImageId</i> The Amazon Machine Image (AMI) ID of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceAwsEc2InstanceType</i> The instance type of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceTags</i> A list of AWS tags associated with a resource at the time the finding was processed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceRegion</i> The canonical AWS external Region name where this resource is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourcePartition</i> The canonical AWS partition name that the Region is assigned to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceId</i> The canonical identifier for the given resource type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceType</i> Specifies the type of the resource that details are provided for.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThreatIntelIndicatorSourceUrl</i> The URL for more details from the source of the threat intel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThreatIntelIndicatorSource</i> The source of the threat intel.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThreatIntelIndicatorLastObservedAt</i> The date/time of the last observation of a threat intel indicator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThreatIntelIndicatorCategory</i> The category of a threat intel indicator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThreatIntelIndicatorValue</i> The value of a threat intel indicator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThreatIntelIndicatorType</i> The type of a threat intel indicator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessTerminatedAt</i> The date/time that the process was terminated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessLaunchedAt</i> The date/time that the process was launched.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessParentPid</i> The parent process ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessPid</i> The process ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessPath</i> The path to the process executable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProcessName</i> The name of the process.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkDestinationDomain</i> The destination domain of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkDestinationPort</i> The destination port of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkDestinationIpV6</i> The destination IPv6 address of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkDestinationIpV4</i> The destination IPv4 address of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkSourceMac</i> The source media access control (MAC) address of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkSourceDomain</i> The source domain of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkSourcePort</i> The source port of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkSourceIpV6</i> The source IPv6 address of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkSourceIpV4</i> The source IPv4 address of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkProtocol</i> The protocol of network-related information about a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NetworkDirection</i> Indicates the direction of network traffic associated with a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MalwareState</i> The state of the malware that was observed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MalwarePath</i> The filesystem path of the malware that was observed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MalwareType</i> The type of the malware that was observed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MalwareName</i> The name of the malware that was observed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserDefinedFields</i> A list of name/value string pairs associated with the finding. These are custom, user-defined fields added to a finding. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompanyName</i> The name of the findings provider (company) that owns the solution (product) that generates findings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductName</i> The name of the solution (product) that generates findings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductFields</i> A data type where security-findings providers can include additional solution-specific details that aren't part of the defined <code>AwsSecurityFinding</code> format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceUrl</i> A URL that links to a page about the current finding in the security-findings provider's solution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecommendationText</i> The recommendation of what to do about the issue described in a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A finding's description.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Title</i> A finding's title.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Criticality</i> The level of importance assigned to the resources associated with the finding. A score of 0 means that the underlying resources have no criticality, and a score of 100 is reserved for the most critical resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Confidence</i> A finding's confidence. Confidence is defined as the likelihood that a finding accurately identifies the behavior or issue that it was intended to identify. Confidence is scored on a 0-100 basis using a ratio scale, where 0 means zero percent confidence and 100 means 100 percent confidence.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SeverityLabel</i> The label of a finding's severity.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SeverityNormalized</i> The normalized severity of a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SeverityProduct</i> The native severity as defined by the security-findings provider's solution that generated the finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdatedAt</i> An ISO8601-formatted timestamp that indicates when the security-findings provider last updated the finding record. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i> An ISO8601-formatted timestamp that indicates when the security-findings provider captured the potential security issue that a finding captured.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastObservedAt</i> An ISO8601-formatted timestamp that indicates when the security-findings provider most recently observed the potential security issue that a finding captured.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FirstObservedAt</i> An ISO8601-formatted timestamp that indicates when the security-findings provider first observed the potential security issue that a finding captured.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> A finding type in the format of <code>namespace/category/classifier</code> that classifies a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GeneratorId</i> The identifier for the solution-specific component (a discrete unit of logic) that generated a finding. In various security-findings providers' solutions, this generator can be called a rule, a check, a detector, a plug-in, etc.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The security findings provider-specific identifier for a finding.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AwsAccountId</i> The AWS account ID that a finding is generated in.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductArn</i> The ARN generated by Security Hub that uniquely identifies a third-party company (security findings provider) after this provider's product (solution that generates findings) is registered with Security Hub.<br/>
        :return:         """
        pass



class ServerlessApplicationRepository(BaseClient):
    """The AWS Serverless Application Repository makes it easy for developers and enterprises to quickly find
 and deploy serverless applications in the AWS Cloud. For more information about serverless applications,
 see Serverless Computing and Applications on the AWS website.The AWS Serverless Application Repository is deeply integrated with the AWS Lambda console, so that developers of 
 all levels can get started with serverless computing without needing to learn anything new. You can use category 
 keywords to browse for applications such as web and mobile backends, data processing applications, or chatbots. 
 You can also search for applications by name, publisher, or event source. To use an application, you simply choose it, 
 configure any required fields, and deploy it with a few clicks. You can also easily publish applications, sharing them publicly with the community at large, or privately
 within your team or across your organization. To publish a serverless application (or app), you can use the
 AWS Management Console, AWS Command Line Interface (AWS CLI), or AWS SDKs to upload the code. Along with the
 code, you upload a simple manifest file, also known as the AWS Serverless Application Model (AWS SAM) template.
 For more information about AWS SAM, see AWS Serverless Application Model (AWS SAM) on the AWS Labs
 GitHub repository.The AWS Serverless Application Repository Developer Guide contains more information about the two developer
 experiences available:<ul>
 <li>
 Consuming Applications – Browse for applications and view information about them, including
 source code and readme files. Also install, configure, and deploy applications of your choosing. 
 Publishing Applications – Configure and upload applications to make them available to other
 developers, and publish new versions of applications. 
 </li>
 </ul>"""


    class ListApplicationDependenciesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_application_dependencies` for valid parameters"""
            pass
    class ListApplicationVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_application_versions` for valid parameters"""
            pass
    class ListApplicationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_applications` for valid parameters"""
            pass


    class CreateApplicationRequest(Mapping):
        pass
    
    class CreateApplicationResponse(Mapping):
        pass
    
    class __string(object):
        pass
    
    class CreateApplicationVersionRequest(Mapping):
        pass
    
    class CreateApplicationVersionResponse(Mapping):
        pass
    
    class CreateCloudFormationChangeSetRequest(Mapping):
        pass
    
    class CreateCloudFormationChangeSetResponse(Mapping):
        pass
    
    class ParameterValue(Mapping):
        pass
    
    class RollbackConfiguration(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreateCloudFormationTemplateRequest(Mapping):
        pass
    
    class CreateCloudFormationTemplateResponse(Mapping):
        pass
    
    class DeleteApplicationRequest(Mapping):
        pass
    
    class GetApplicationRequest(Mapping):
        pass
    
    class GetApplicationResponse(Mapping):
        pass
    
    class GetApplicationPolicyRequest(Mapping):
        pass
    
    class GetApplicationPolicyResponse(Mapping):
        pass
    
    class GetCloudFormationTemplateRequest(Mapping):
        pass
    
    class GetCloudFormationTemplateResponse(Mapping):
        pass
    
    class ListApplicationDependenciesRequest(Mapping):
        pass
    
    class ListApplicationDependenciesResponse(Mapping):
        pass
    
    class ListApplicationVersionsRequest(Mapping):
        pass
    
    class ListApplicationVersionsResponse(Mapping):
        pass
    
    class ListApplicationsRequest(Mapping):
        pass
    
    class ListApplicationsResponse(Mapping):
        pass
    
    class PutApplicationPolicyRequest(Mapping):
        pass
    
    class PutApplicationPolicyResponse(Mapping):
        pass
    
    class ApplicationPolicyStatement(Mapping):
        pass
    
    class UpdateApplicationRequest(Mapping):
        pass
    
    class UpdateApplicationResponse(Mapping):
        pass
    
    def create_application(self, Name: str, Description: str, Author: str, HomePageUrl: str=None, Labels: List[__string]=None, LicenseBody: str=None, LicenseUrl: str=None, ReadmeBody: str=None, ReadmeUrl: str=None, SemanticVersion: str=None, SourceCodeArchiveUrl: str=None, SourceCodeUrl: str=None, SpdxLicenseId: str=None, TemplateBody: str=None, TemplateUrl: str=None) -> CreateApplicationResponse:
        """Creates an application, optionally including an AWS SAM file to create the first application version in the same call.

        :param self:
        :param TemplateUrl:  A link to the S3 object containing the packaged AWS SAM template of your application.You can specify only one of templateBody and templateUrl; otherwise an error results.<br/>TemplateBody:  The local raw packaged AWS SAM template file of your application.
 The file has the format file://&lt;path>/&lt;filename>.You can specify only one of templateBody and templateUrl; otherwise an error results.<br/>SpdxLicenseId:  A valid identifier from <a href="https://spdx.org/licenses/">https://spdx.org/licenses/</a>.<br/>SourceCodeUrl:  A link to a public repository for the source code of your application, for example the URL of a specific GitHub commit.<br/>SourceCodeArchiveUrl:  A link to the S3 object that contains the ZIP archive of the source code for this version of your application.Maximum size 50 MB<br/>SemanticVersion:  The semantic version of the application:
 <a href="https://semver.org/">https://semver.org/</a>
 <br/>ReadmeUrl:  A link to the S3 object in Markdown language that contains a more detailed description of the application and how it works.Maximum size 5 MBYou can specify only one of readmeBody and readmeUrl; otherwise, an error results.<br/>ReadmeBody:  A local text readme file in Markdown language that contains a more detailed description of the application and how it works.
 The file has the format file://&lt;path>/&lt;filename>.Maximum size 5 MBYou can specify only one of readmeBody and readmeUrl; otherwise, an error results.<br/>Name:  The name of the application that you want to publish.Minimum length=1. Maximum length=140Pattern: "[a-zA-Z0-9\\-]+";<br/>LicenseUrl:  A link to the S3 object that contains the license of the app that matches the spdxLicenseID value of your application.Maximum size 5 MBYou can specify only one of licenseBody and licenseUrl; otherwise, an error results.<br/>LicenseBody:  A local text file that contains the license of the app that matches the spdxLicenseID value of your application.
 The file has the format file://&lt;path>/&lt;filename>.Maximum size 5 MBYou can specify only one of licenseBody and licenseUrl; otherwise, an error results.<br/>Labels:  Labels to improve discovery of apps in search results.Minimum length=1. Maximum length=127. Maximum number of labels: 10Pattern: "^[a-zA-Z0-9+\\-_:\\/@]+$";<br/>HomePageUrl:  A URL with more information about the application, for example the location of your GitHub repository for the application.<br/>Description:  The description of the application.Minimum length=1. Maximum length=256<br/>Author:  The name of the author publishing the app.Minimum length=1. Maximum length=127.Pattern "^[a-z0-9](([a-z0-9]|-(?!-))*[a-z0-9])?$";<br/>
        :return: Version:  Version information about the application.:<br/>SpdxLicenseId:  A valid identifier from https://spdx.org/licenses/.<br/>ReadmeUrl:  A link to the readme file in Markdown language that contains a more detailed description of the application and how it works.Maximum size 5 MB<br/>Name:  The name of the application.Minimum length=1. Maximum length=140Pattern: "[a-zA-Z0-9\\-]+";<br/>LicenseUrl:  A link to a license file of the app that matches the spdxLicenseID value of your application.Maximum size 5 MB<br/>Labels:  Labels to improve discovery of apps in search results.Minimum length=1. Maximum length=127. Maximum number of labels: 10Pattern: "^[a-zA-Z0-9+\\-_:\\/@]+$";<br/>HomePageUrl:  A URL with more information about the application, for example the location of your GitHub repository for the application.<br/>Description:  The description of the application.Minimum length=1. Maximum length=256<br/>CreationTime:  The date and time this resource was created.<br/>Author:  The name of the author publishing the app.Minimum length=1. Maximum length=127.Pattern "^[a-z0-9](([a-z0-9]|-(?!-))*[a-z0-9])?$";<br/>ApplicationId:  The application Amazon Resource Name (ARN).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TemplateUrl</i> A link to the packaged AWS SAM template of your application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceCodeUrl</i> A link to a public repository for the source code of your application, for example the URL of a specific GitHub commit.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceCodeArchiveUrl</i> A link to the S3 object that contains the ZIP archive of the source code for this version of your application.Maximum size 50 MB<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SemanticVersion</i> The semantic version of the application:
 <a href="https://semver.org/">https://semver.org/</a>
 <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourcesSupported</i> Whether all of the AWS resources contained in this application are supported in the region
 in which it is being retrieved.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequiredCapabilities</i> A list of values that you must specify before you can deploy certain applications.
 Some applications might include resources that can affect permissions in your AWS
 account, for example, by creating new AWS Identity and Access Management (IAM) users.
 For those applications, you must explicitly acknowledge their capabilities by
 specifying this parameter.The only valid values are CAPABILITY_IAM, CAPABILITY_NAMED_IAM,
 CAPABILITY_RESOURCE_POLICY, and CAPABILITY_AUTO_EXPAND.The following resources require you to specify CAPABILITY_IAM or
 CAPABILITY_NAMED_IAM:
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-group.html">AWS::IAM::Group</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-instanceprofile.html">AWS::IAM::InstanceProfile</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-policy.html">AWS::IAM::Policy</a>, and
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-role.html">AWS::IAM::Role</a>.
 If the application contains IAM resources, you can specify either CAPABILITY_IAM
 or CAPABILITY_NAMED_IAM. If the application contains IAM resources
 with custom names, you must specify CAPABILITY_NAMED_IAM.The following resources require you to specify CAPABILITY_RESOURCE_POLICY:
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-permission.html">AWS::Lambda::Permission</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-policy.html">AWS::IAM:Policy</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-applicationautoscaling-scalingpolicy.html">AWS::ApplicationAutoScaling::ScalingPolicy</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-policy.html">AWS::S3::BucketPolicy</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sqs-policy.html">AWS::SQS::QueuePolicy</a>, and
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sns-policy.html">AWS::SNS::TopicPolicy</a>.Applications that contain one or more nested applications require you to specify
 CAPABILITY_AUTO_EXPAND.If your application template contains any of the above resources, we recommend that you review
 all permissions associated with the application before deploying. If you don't specify
 this parameter for an application that requires capabilities, the call will fail.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterDefinitions</i> An array of parameter types supported by the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The date and time this resource was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The application Amazon Resource Name (ARN).<br/>        """
        pass

    def create_application_version(self, SemanticVersion: str, ApplicationId: str, SourceCodeArchiveUrl: str=None, SourceCodeUrl: str=None, TemplateBody: str=None, TemplateUrl: str=None) -> CreateApplicationVersionResponse:
        """Creates an application version.

        :param self:
        :param TemplateUrl:  A link to the packaged AWS SAM template of your application.<br/>TemplateBody:  The raw packaged AWS SAM template of your application.<br/>SourceCodeUrl:  A link to a public repository for the source code of your application, for example the URL of a specific GitHub commit.<br/>SourceCodeArchiveUrl:  A link to the S3 object that contains the ZIP archive of the source code for this version of your application.Maximum size 50 MB<br/>SemanticVersion:  The semantic version of the new version.<br/>ApplicationId:  The Amazon Resource Name (ARN) of the application.<br/>
        :return: TemplateUrl:  A link to the packaged AWS SAM template of your application.<br/>SourceCodeUrl:  A link to a public repository for the source code of your application, for example the URL of a specific GitHub commit.<br/>SourceCodeArchiveUrl:  A link to the S3 object that contains the ZIP archive of the source code for this version of your application.Maximum size 50 MB<br/>SemanticVersion:  The semantic version of the application:
 <a href="https://semver.org/">https://semver.org/</a>
 <br/>ResourcesSupported:  Whether all of the AWS resources contained in this application are supported in the region
 in which it is being retrieved.<br/>RequiredCapabilities:  A list of values that you must specify before you can deploy certain applications.
 Some applications might include resources that can affect permissions in your AWS
 account, for example, by creating new AWS Identity and Access Management (IAM) users.
 For those applications, you must explicitly acknowledge their capabilities by
 specifying this parameter.The only valid values are CAPABILITY_IAM, CAPABILITY_NAMED_IAM,
 CAPABILITY_RESOURCE_POLICY, and CAPABILITY_AUTO_EXPAND.The following resources require you to specify CAPABILITY_IAM or
 CAPABILITY_NAMED_IAM:
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-group.html">AWS::IAM::Group</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-instanceprofile.html">AWS::IAM::InstanceProfile</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-policy.html">AWS::IAM::Policy</a>, and
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-role.html">AWS::IAM::Role</a>.
 If the application contains IAM resources, you can specify either CAPABILITY_IAM
 or CAPABILITY_NAMED_IAM. If the application contains IAM resources
 with custom names, you must specify CAPABILITY_NAMED_IAM.The following resources require you to specify CAPABILITY_RESOURCE_POLICY:
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-permission.html">AWS::Lambda::Permission</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-policy.html">AWS::IAM:Policy</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-applicationautoscaling-scalingpolicy.html">AWS::ApplicationAutoScaling::ScalingPolicy</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-policy.html">AWS::S3::BucketPolicy</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sqs-policy.html">AWS::SQS::QueuePolicy</a>, and
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sns-policy.html">AWS::SNS::TopicPolicy</a>.Applications that contain one or more nested applications require you to specify
 CAPABILITY_AUTO_EXPAND.If your application template contains any of the above resources, we recommend that you review
 all permissions associated with the application before deploying. If you don't specify
 this parameter for an application that requires capabilities, the call will fail.<br/>ParameterDefinitions:  An array of parameter types supported by the application.<br/>CreationTime:  The date and time this resource was created.<br/>ApplicationId:  The application Amazon Resource Name (ARN).<br/>        """
        pass

    def create_cloud_formation_change_set(self, StackName: str, ApplicationId: str, Capabilities: List[__string]=None, ChangeSetName: str=None, ClientToken: str=None, Description: str=None, NotificationArns: List[__string]=None, ParameterOverrides: List[ParameterValue]=None, ResourceTypes: List[__string]=None, RollbackConfiguration: RollbackConfiguration=None, SemanticVersion: str=None, Tags: List[Tag]=None, TemplateId: str=None) -> CreateCloudFormationChangeSetResponse:
        """Creates an AWS CloudFormation change set for the given application.

        :param self:
        :param TemplateId:  The UUID returned by CreateCloudFormationTemplate.Pattern: [0-9a-fA-F]{8}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{12}<br/>Tags:  This property corresponds to the parameter of the same name for the <i>AWS CloudFormation <a href="https://docs.aws.amazon.com/goto/WebAPI/cloudformation-2010-05-15/CreateChangeSet">CreateChangeSet</a>
 </i> API.<br/>StackName:  This property corresponds to the parameter of the same name for the <i>AWS CloudFormation <a href="https://docs.aws.amazon.com/goto/WebAPI/cloudformation-2010-05-15/CreateChangeSet">CreateChangeSet</a>
 </i> API.<br/>SemanticVersion:  The semantic version of the application:
 <a href="https://semver.org/">https://semver.org/</a>
 <br/>RollbackConfiguration:  This property corresponds to the parameter of the same name for the <i>AWS CloudFormation <a href="https://docs.aws.amazon.com/goto/WebAPI/cloudformation-2010-05-15/CreateChangeSet">CreateChangeSet</a>
 </i> API.:<br/>ResourceTypes:  This property corresponds to the parameter of the same name for the <i>AWS CloudFormation <a href="https://docs.aws.amazon.com/goto/WebAPI/cloudformation-2010-05-15/CreateChangeSet">CreateChangeSet</a>
 </i> API.<br/>ParameterOverrides:  A list of parameter values for the parameters of the application.<br/>NotificationArns:  This property corresponds to the parameter of the same name for the <i>AWS CloudFormation <a href="https://docs.aws.amazon.com/goto/WebAPI/cloudformation-2010-05-15/CreateChangeSet">CreateChangeSet</a>
 </i> API.<br/>Description:  This property corresponds to the parameter of the same name for the <i>AWS CloudFormation <a href="https://docs.aws.amazon.com/goto/WebAPI/cloudformation-2010-05-15/CreateChangeSet">CreateChangeSet</a>
 </i> API.<br/>ClientToken:  This property corresponds to the parameter of the same name for the <i>AWS CloudFormation <a href="https://docs.aws.amazon.com/goto/WebAPI/cloudformation-2010-05-15/CreateChangeSet">CreateChangeSet</a>
 </i> API.<br/>ChangeSetName:  This property corresponds to the parameter of the same name for the <i>AWS CloudFormation <a href="https://docs.aws.amazon.com/goto/WebAPI/cloudformation-2010-05-15/CreateChangeSet">CreateChangeSet</a>
 </i> API.<br/>Capabilities:  A list of values that you must specify before you can deploy certain applications.
 Some applications might include resources that can affect permissions in your AWS
 account, for example, by creating new AWS Identity and Access Management (IAM) users.
 For those applications, you must explicitly acknowledge their capabilities by
 specifying this parameter.The only valid values are CAPABILITY_IAM, CAPABILITY_NAMED_IAM,
 CAPABILITY_RESOURCE_POLICY, and CAPABILITY_AUTO_EXPAND.The following resources require you to specify CAPABILITY_IAM or
 CAPABILITY_NAMED_IAM:
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-group.html">AWS::IAM::Group</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-instanceprofile.html">AWS::IAM::InstanceProfile</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-policy.html">AWS::IAM::Policy</a>, and
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-role.html">AWS::IAM::Role</a>.
 If the application contains IAM resources, you can specify either CAPABILITY_IAM
 or CAPABILITY_NAMED_IAM. If the application contains IAM resources
 with custom names, you must specify CAPABILITY_NAMED_IAM.The following resources require you to specify CAPABILITY_RESOURCE_POLICY:
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-permission.html">AWS::Lambda::Permission</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-policy.html">AWS::IAM:Policy</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-applicationautoscaling-scalingpolicy.html">AWS::ApplicationAutoScaling::ScalingPolicy</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-policy.html">AWS::S3::BucketPolicy</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sqs-policy.html">AWS::SQS::QueuePolicy</a>, and
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sns-policy.html">AWS::SNS:TopicPolicy</a>.Applications that contain one or more nested applications require you to specify
 CAPABILITY_AUTO_EXPAND.If your application template contains any of the above resources, we recommend that you review
 all permissions associated with the application before deploying. If you don't specify
 this parameter for an application that requires capabilities, the call will fail.<br/>ApplicationId:  The Amazon Resource Name (ARN) of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RollbackTriggers</i> This property corresponds to the content of the same name for the <i>AWS CloudFormation <a href="https://docs.aws.amazon.com/goto/WebAPI/cloudformation-2010-05-15/RollbackConfiguration">RollbackConfiguration</a>
 </i> Data Type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MonitoringTimeInMinutes</i> This property corresponds to the content of the same name for the <i>AWS CloudFormation <a href="https://docs.aws.amazon.com/goto/WebAPI/cloudformation-2010-05-15/RollbackConfiguration">RollbackConfiguration</a>
 </i> Data Type.<br/>
        :return: StackId:  The unique ID of the stack.<br/>SemanticVersion:  The semantic version of the application:
 <a href="https://semver.org/">https://semver.org/</a>
 <br/>ChangeSetId:  The Amazon Resource Name (ARN) of the change set.Length constraints: Minimum length of 1.Pattern: ARN:[-a-zA-Z0-9:/]*<br/>ApplicationId:  The application Amazon Resource Name (ARN).<br/>        """
        pass

    def create_cloud_formation_template(self, ApplicationId: str, SemanticVersion: str=None) -> CreateCloudFormationTemplateResponse:
        """Creates an AWS CloudFormation template.

        :param self:
        :param SemanticVersion:  The semantic version of the application:
 <a href="https://semver.org/">https://semver.org/</a>
 <br/>ApplicationId:  The Amazon Resource Name (ARN) of the application.<br/>
        :return: TemplateUrl:  A link to the template that can be used to deploy the application using
 AWS CloudFormation.<br/>TemplateId:  The UUID returned by CreateCloudFormationTemplate.Pattern: [0-9a-fA-F]{8}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{12}<br/>Status:  Status of the template creation workflow.Possible values: PREPARING | ACTIVE | EXPIRED
 <br/>SemanticVersion:  The semantic version of the application:
 <a href="https://semver.org/">https://semver.org/</a>
 <br/>ExpirationTime:  The date and time this template expires. Templates
 expire 1 hour after creation.<br/>CreationTime:  The date and time this resource was created.<br/>ApplicationId:  The application Amazon Resource Name (ARN).<br/>        """
        pass

    def delete_application(self, ApplicationId: str):
        """Deletes the specified application.

        :param self:
        :param ApplicationId:  The Amazon Resource Name (ARN) of the application.<br/>
        :return:         """
        pass

    def get_application(self, ApplicationId: str, SemanticVersion: str=None) -> GetApplicationResponse:
        """Gets the specified application.

        :param self:
        :param SemanticVersion:  The semantic version of the application to get.<br/>ApplicationId:  The Amazon Resource Name (ARN) of the application.<br/>
        :return: Version:  Version information about the application.:<br/>SpdxLicenseId:  A valid identifier from https://spdx.org/licenses/.<br/>ReadmeUrl:  A link to the readme file in Markdown language that contains a more detailed description of the application and how it works.Maximum size 5 MB<br/>Name:  The name of the application.Minimum length=1. Maximum length=140Pattern: "[a-zA-Z0-9\\-]+";<br/>LicenseUrl:  A link to a license file of the app that matches the spdxLicenseID value of your application.Maximum size 5 MB<br/>Labels:  Labels to improve discovery of apps in search results.Minimum length=1. Maximum length=127. Maximum number of labels: 10Pattern: "^[a-zA-Z0-9+\\-_:\\/@]+$";<br/>HomePageUrl:  A URL with more information about the application, for example the location of your GitHub repository for the application.<br/>Description:  The description of the application.Minimum length=1. Maximum length=256<br/>CreationTime:  The date and time this resource was created.<br/>Author:  The name of the author publishing the app.Minimum length=1. Maximum length=127.Pattern "^[a-z0-9](([a-z0-9]|-(?!-))*[a-z0-9])?$";<br/>ApplicationId:  The application Amazon Resource Name (ARN).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TemplateUrl</i> A link to the packaged AWS SAM template of your application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceCodeUrl</i> A link to a public repository for the source code of your application, for example the URL of a specific GitHub commit.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceCodeArchiveUrl</i> A link to the S3 object that contains the ZIP archive of the source code for this version of your application.Maximum size 50 MB<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SemanticVersion</i> The semantic version of the application:
 <a href="https://semver.org/">https://semver.org/</a>
 <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourcesSupported</i> Whether all of the AWS resources contained in this application are supported in the region
 in which it is being retrieved.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequiredCapabilities</i> A list of values that you must specify before you can deploy certain applications.
 Some applications might include resources that can affect permissions in your AWS
 account, for example, by creating new AWS Identity and Access Management (IAM) users.
 For those applications, you must explicitly acknowledge their capabilities by
 specifying this parameter.The only valid values are CAPABILITY_IAM, CAPABILITY_NAMED_IAM,
 CAPABILITY_RESOURCE_POLICY, and CAPABILITY_AUTO_EXPAND.The following resources require you to specify CAPABILITY_IAM or
 CAPABILITY_NAMED_IAM:
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-group.html">AWS::IAM::Group</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-instanceprofile.html">AWS::IAM::InstanceProfile</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-policy.html">AWS::IAM::Policy</a>, and
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-role.html">AWS::IAM::Role</a>.
 If the application contains IAM resources, you can specify either CAPABILITY_IAM
 or CAPABILITY_NAMED_IAM. If the application contains IAM resources
 with custom names, you must specify CAPABILITY_NAMED_IAM.The following resources require you to specify CAPABILITY_RESOURCE_POLICY:
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-permission.html">AWS::Lambda::Permission</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-policy.html">AWS::IAM:Policy</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-applicationautoscaling-scalingpolicy.html">AWS::ApplicationAutoScaling::ScalingPolicy</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-policy.html">AWS::S3::BucketPolicy</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sqs-policy.html">AWS::SQS::QueuePolicy</a>, and
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sns-policy.html">AWS::SNS::TopicPolicy</a>.Applications that contain one or more nested applications require you to specify
 CAPABILITY_AUTO_EXPAND.If your application template contains any of the above resources, we recommend that you review
 all permissions associated with the application before deploying. If you don't specify
 this parameter for an application that requires capabilities, the call will fail.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterDefinitions</i> An array of parameter types supported by the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The date and time this resource was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The application Amazon Resource Name (ARN).<br/>        """
        pass

    def get_application_policy(self, ApplicationId: str) -> GetApplicationPolicyResponse:
        """Retrieves the policy for the application.

        :param self:
        :param ApplicationId:  The Amazon Resource Name (ARN) of the application.<br/>
        :return: Statements:  An array of policy statements applied to the application.<br/>        """
        pass

    def get_cloud_formation_template(self, TemplateId: str, ApplicationId: str) -> GetCloudFormationTemplateResponse:
        """Gets the specified AWS CloudFormation template.

        :param self:
        :param TemplateId:  The UUID returned by CreateCloudFormationTemplate.Pattern: [0-9a-fA-F]{8}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{12}<br/>ApplicationId:  The Amazon Resource Name (ARN) of the application.<br/>
        :return: TemplateUrl:  A link to the template that can be used to deploy the application using
 AWS CloudFormation.<br/>TemplateId:  The UUID returned by CreateCloudFormationTemplate.Pattern: [0-9a-fA-F]{8}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{12}<br/>Status:  Status of the template creation workflow.Possible values: PREPARING | ACTIVE | EXPIRED
 <br/>SemanticVersion:  The semantic version of the application:
 <a href="https://semver.org/">https://semver.org/</a>
 <br/>ExpirationTime:  The date and time this template expires. Templates
 expire 1 hour after creation.<br/>CreationTime:  The date and time this resource was created.<br/>ApplicationId:  The application Amazon Resource Name (ARN).<br/>        """
        pass

    def list_application_dependencies(self, ApplicationId: str, MaxItems: int=None, NextToken: str=None, SemanticVersion: str=None) -> ListApplicationDependenciesResponse:
        """Retrieves the list of applications nested in the containing application.

        :param self:
        :param SemanticVersion:  The semantic version of the application to get.<br/>NextToken:  A token to specify where to start paginating.<br/>MaxItems:  The total number of items to return.<br/>ApplicationId:  The Amazon Resource Name (ARN) of the application.<br/>
        :return: NextToken:  The token to request the next page of results.<br/>Dependencies:  An array of application summaries nested in the application.<br/>        """
        pass

    def list_application_versions(self, ApplicationId: str, MaxItems: int=None, NextToken: str=None) -> ListApplicationVersionsResponse:
        """Lists versions for the specified application.

        :param self:
        :param NextToken:  A token to specify where to start paginating.<br/>MaxItems:  The total number of items to return.<br/>ApplicationId:  The Amazon Resource Name (ARN) of the application.<br/>
        :return: Versions:  An array of version summaries for the application.<br/>NextToken:  The token to request the next page of results.<br/>        """
        pass

    def list_applications(self, MaxItems: int=None, NextToken: str=None) -> ListApplicationsResponse:
        """Lists applications owned by the requester.

        :param self:
        :param NextToken:  A token to specify where to start paginating.<br/>MaxItems:  The total number of items to return.<br/>
        :return: NextToken:  The token to request the next page of results.<br/>Applications:  An array of application summaries.<br/>        """
        pass

    def put_application_policy(self, Statements: List[ApplicationPolicyStatement], ApplicationId: str) -> PutApplicationPolicyResponse:
        """Sets the permission policy for an application. For the list of actions supported for this operation, see
 <a href="https://docs.aws.amazon.com/serverlessrepo/latest/devguide/access-control-resource-based.html#application-permissions">Application 
 Permissions</a>
 .

        :param self:
        :param Statements:  An array of policy statements applied to the application.<br/>ApplicationId:  The Amazon Resource Name (ARN) of the application.<br/>
        :return: Statements:  An array of policy statements applied to the application.<br/>        """
        pass

    def update_application(self, ApplicationId: str, Author: str=None, Description: str=None, HomePageUrl: str=None, Labels: List[__string]=None, ReadmeBody: str=None, ReadmeUrl: str=None) -> UpdateApplicationResponse:
        """Updates the specified application.

        :param self:
        :param ReadmeUrl:  A link to the readme file in Markdown language that contains a more detailed description of the application and how it works.Maximum size 5 MB<br/>ReadmeBody:  A text readme file in Markdown language that contains a more detailed description of the application and how it works.Maximum size 5 MB<br/>Labels:  Labels to improve discovery of apps in search results.Minimum length=1. Maximum length=127. Maximum number of labels: 10Pattern: "^[a-zA-Z0-9+\\-_:\\/@]+$";<br/>HomePageUrl:  A URL with more information about the application, for example the location of your GitHub repository for the application.<br/>Description:  The description of the application.Minimum length=1. Maximum length=256<br/>Author:  The name of the author publishing the app.Minimum length=1. Maximum length=127.Pattern "^[a-z0-9](([a-z0-9]|-(?!-))*[a-z0-9])?$";<br/>ApplicationId:  The Amazon Resource Name (ARN) of the application.<br/>
        :return: Version:  Version information about the application.:<br/>SpdxLicenseId:  A valid identifier from https://spdx.org/licenses/.<br/>ReadmeUrl:  A link to the readme file in Markdown language that contains a more detailed description of the application and how it works.Maximum size 5 MB<br/>Name:  The name of the application.Minimum length=1. Maximum length=140Pattern: "[a-zA-Z0-9\\-]+";<br/>LicenseUrl:  A link to a license file of the app that matches the spdxLicenseID value of your application.Maximum size 5 MB<br/>Labels:  Labels to improve discovery of apps in search results.Minimum length=1. Maximum length=127. Maximum number of labels: 10Pattern: "^[a-zA-Z0-9+\\-_:\\/@]+$";<br/>HomePageUrl:  A URL with more information about the application, for example the location of your GitHub repository for the application.<br/>Description:  The description of the application.Minimum length=1. Maximum length=256<br/>CreationTime:  The date and time this resource was created.<br/>Author:  The name of the author publishing the app.Minimum length=1. Maximum length=127.Pattern "^[a-z0-9](([a-z0-9]|-(?!-))*[a-z0-9])?$";<br/>ApplicationId:  The application Amazon Resource Name (ARN).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TemplateUrl</i> A link to the packaged AWS SAM template of your application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceCodeUrl</i> A link to a public repository for the source code of your application, for example the URL of a specific GitHub commit.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceCodeArchiveUrl</i> A link to the S3 object that contains the ZIP archive of the source code for this version of your application.Maximum size 50 MB<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SemanticVersion</i> The semantic version of the application:
 <a href="https://semver.org/">https://semver.org/</a>
 <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourcesSupported</i> Whether all of the AWS resources contained in this application are supported in the region
 in which it is being retrieved.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequiredCapabilities</i> A list of values that you must specify before you can deploy certain applications.
 Some applications might include resources that can affect permissions in your AWS
 account, for example, by creating new AWS Identity and Access Management (IAM) users.
 For those applications, you must explicitly acknowledge their capabilities by
 specifying this parameter.The only valid values are CAPABILITY_IAM, CAPABILITY_NAMED_IAM,
 CAPABILITY_RESOURCE_POLICY, and CAPABILITY_AUTO_EXPAND.The following resources require you to specify CAPABILITY_IAM or
 CAPABILITY_NAMED_IAM:
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-group.html">AWS::IAM::Group</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-instanceprofile.html">AWS::IAM::InstanceProfile</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-policy.html">AWS::IAM::Policy</a>, and
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-role.html">AWS::IAM::Role</a>.
 If the application contains IAM resources, you can specify either CAPABILITY_IAM
 or CAPABILITY_NAMED_IAM. If the application contains IAM resources
 with custom names, you must specify CAPABILITY_NAMED_IAM.The following resources require you to specify CAPABILITY_RESOURCE_POLICY:
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-permission.html">AWS::Lambda::Permission</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-policy.html">AWS::IAM:Policy</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-applicationautoscaling-scalingpolicy.html">AWS::ApplicationAutoScaling::ScalingPolicy</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-policy.html">AWS::S3::BucketPolicy</a>,
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sqs-policy.html">AWS::SQS::QueuePolicy</a>, and
 <a href="https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sns-policy.html">AWS::SNS::TopicPolicy</a>.Applications that contain one or more nested applications require you to specify
 CAPABILITY_AUTO_EXPAND.If your application template contains any of the above resources, we recommend that you review
 all permissions associated with the application before deploying. If you don't specify
 this parameter for an application that requires capabilities, the call will fail.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterDefinitions</i> An array of parameter types supported by the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> The date and time this resource was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ApplicationId</i> The application Amazon Resource Name (ARN).<br/>        """
        pass



class ServiceQuotas(BaseClient):
    """ Service Quotas is a web service that you can use to manage many of your AWS service quotas. Quotas, also referred to as limits, are the maximum values for a resource, item, or operation. This guide provide descriptions of the Service Quotas actions that you can call from an API. For the Service Quotas user guide, which explains how to use Service Quotas from the console, see <a href="https://docs.aws.amazon.com/servicequotas/latest/userguide/intro.html">What is Service Quotas</a>.  <note> AWS provides SDKs that consist of libraries and sample code for programming languages and platforms (Java, Ruby, .NET, iOS, Android, etc...,). The SDKs provide a convenient way to create programmatic access to Service Quotas and AWS. For information about the AWS SDKs, including how to download and install them, see the <a href="https://docs.aws.amazon.com/aws.amazon.com/tools">Tools for Amazon Web Services</a> page. </note>"""


    class ListAWSDefaultServiceQuotasPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_aws_default_service_quotas` for valid parameters"""
            pass
    class ListRequestedServiceQuotaChangeHistoryPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_requested_service_quota_change_history` for valid parameters"""
            pass
    class ListRequestedServiceQuotaChangeHistoryByQuotaPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_requested_service_quota_change_history_by_quota` for valid parameters"""
            pass
    class ListServiceQuotaIncreaseRequestsInTemplatePaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_service_quota_increase_requests_in_template` for valid parameters"""
            pass
    class ListServiceQuotasPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_service_quotas` for valid parameters"""
            pass
    class ListServicesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_services` for valid parameters"""
            pass


    class AssociateServiceQuotaTemplateRequest(Mapping):
        pass
    
    class AssociateServiceQuotaTemplateResponse(Mapping):
        pass
    
    class DeleteServiceQuotaIncreaseRequestFromTemplateRequest(Mapping):
        pass
    
    class DeleteServiceQuotaIncreaseRequestFromTemplateResponse(Mapping):
        pass
    
    class DisassociateServiceQuotaTemplateRequest(Mapping):
        pass
    
    class DisassociateServiceQuotaTemplateResponse(Mapping):
        pass
    
    class GetAWSDefaultServiceQuotaRequest(Mapping):
        pass
    
    class GetAWSDefaultServiceQuotaResponse(Mapping):
        pass
    
    class GetAssociationForServiceQuotaTemplateRequest(Mapping):
        pass
    
    class GetAssociationForServiceQuotaTemplateResponse(Mapping):
        pass
    
    class GetRequestedServiceQuotaChangeRequest(Mapping):
        pass
    
    class GetRequestedServiceQuotaChangeResponse(Mapping):
        pass
    
    class GetServiceQuotaRequest(Mapping):
        pass
    
    class GetServiceQuotaResponse(Mapping):
        pass
    
    class GetServiceQuotaIncreaseRequestFromTemplateRequest(Mapping):
        pass
    
    class GetServiceQuotaIncreaseRequestFromTemplateResponse(Mapping):
        pass
    
    class ListAWSDefaultServiceQuotasRequest(Mapping):
        pass
    
    class ListAWSDefaultServiceQuotasResponse(Mapping):
        pass
    
    class ListRequestedServiceQuotaChangeHistoryRequest(Mapping):
        pass
    
    class ListRequestedServiceQuotaChangeHistoryResponse(Mapping):
        pass
    
    class ListRequestedServiceQuotaChangeHistoryByQuotaRequest(Mapping):
        pass
    
    class ListRequestedServiceQuotaChangeHistoryByQuotaResponse(Mapping):
        pass
    
    class ListServiceQuotaIncreaseRequestsInTemplateRequest(Mapping):
        pass
    
    class ListServiceQuotaIncreaseRequestsInTemplateResponse(Mapping):
        pass
    
    class ListServiceQuotasRequest(Mapping):
        pass
    
    class ListServiceQuotasResponse(Mapping):
        pass
    
    class ListServicesRequest(Mapping):
        pass
    
    class ListServicesResponse(Mapping):
        pass
    
    class PutServiceQuotaIncreaseRequestIntoTemplateRequest(Mapping):
        pass
    
    class PutServiceQuotaIncreaseRequestIntoTemplateResponse(Mapping):
        pass
    
    class QuotaValue(object):
        pass
    
    class RequestServiceQuotaIncreaseRequest(Mapping):
        pass
    
    class RequestServiceQuotaIncreaseResponse(Mapping):
        pass
    
    def associate_service_quota_template(self) -> AssociateServiceQuotaTemplateResponse:
        """Associates the Service Quotas template with your organization so that when new accounts are created in your organization, the template submits increase requests for the specified service quotas. Use the Service Quotas template to request an increase for any adjustable quota value. After you define the Service Quotas template, use this operation to associate, or enable, the template. 

        :param self:
        :param 
        :return:         """
        pass

    def delete_service_quota_increase_request_from_template(self, AwsRegion: str, QuotaCode: str, ServiceCode: str) -> DeleteServiceQuotaIncreaseRequestFromTemplateResponse:
        """Removes a service quota increase request from the Service Quotas template. 

        :param self:
        :param AwsRegion:  Specifies the AWS Region for the quota that you want to delete.<br/>QuotaCode:  Specifies the code for the quota that you want to delete.<br/>ServiceCode:  Specifies the code for the service that you want to delete.<br/>
        :return:         """
        pass

    def disassociate_service_quota_template(self) -> DisassociateServiceQuotaTemplateResponse:
        """Disables the Service Quotas template. Once the template is disabled, it does not request quota increases for new accounts in your organization. Disabling the quota template does not apply the quota increase requests from the template.   <b>Related operations</b>  <ul> <li> To enable the quota template, call <a>AssociateServiceQuotaTemplate</a>.  </li> <li> To delete a specific service quota from the template, use <a>DeleteServiceQuotaIncreaseRequestFromTemplate</a>. </li> </ul>

        :param self:
        :param 
        :return:         """
        pass

    def get_aws_default_service_quota(self, QuotaCode: str, ServiceCode: str) -> GetAWSDefaultServiceQuotaResponse:
        """Retrieves the default service quotas values. The Value returned for each quota is the AWS default value, even if the quotas have been increased.. 

        :param self:
        :param QuotaCode:  Identifies the service quota you want to select.<br/>ServiceCode:  Specifies the service that you want to use.<br/>
        :return: Quota:  Returns the <a>ServiceQuota</a> object which contains all values for a quota.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorReason</i> Specifies the <code>ErrorCode</code> and <code>ErrorMessage</code> when success isn't achieved.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ErrorReason</b>: Specifies the <code>ErrorCode</code> and <code>ErrorMessage</code> when success isn't achieved.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Period</i> Identifies the unit and value of how time is measured.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Period</b>: Identifies the unit and value of how time is measured.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UsageMetric</i> Specifies the details about the measurement. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>UsageMetric</b>: Specifies the details about the measurement. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalQuota</i> Specifies if the quota is global.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Adjustable</i> Specifies if the quota value can be increased.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Unit</i> The unit of measurement for the value of the service quota.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The value of service quota.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QuotaName</i> The name identifier of the service quota.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QuotaCode</i> The code identifier for the service quota specified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QuotaArn</i> The Amazon Resource Name (ARN) of the service quota.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceName</i> The name of the AWS service specified in the increase request. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceCode</i> Specifies the service that you want to use.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricStatisticRecommendation</i> Statistics are metric data aggregations over specified periods of time. This is the recommended statistic to use when comparing usage in the CloudWatch Metric against your Service Quota.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricDimensions</i> A dimension is a name/value pair that is part of the identity of a metric. Every metric has specific characteristics that describe it, and you can think of dimensions as categories for those characteristics. These dimensions are part of the CloudWatch Metric Identity that measures usage against a particular service quota.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricName</i> The name of the CloudWatch metric that measures usage of a service quota. This is a required field.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricNamespace</i> The namespace of the metric. The namespace is a container for CloudWatch metrics. You can specify a name for the namespace when you create a metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PeriodUnit</i> The time unit of a period.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PeriodValue</i> The value of a period.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorMessage</i> The error message that provides more detail.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorCode</i> Service Quotas returns the following error values.   <code>DEPENDENCY_ACCESS_DENIED_ERROR</code> is returned when the caller does not have permission to call the service or service quota. To resolve the error, you need permission to access the service or service quota.  <code>DEPENDENCY_THROTTLING_ERROR</code> is returned when the service being called is throttling Service Quotas.  <code>DEPENDENCY_SERVICE_ERROR</code> is returned when the service being called has availability issues.  <code>SERVICE_QUOTA_NOT_AVAILABLE_ERROR</code> is returned when there was an error in Service Quotas.<br/>        """
        pass

    def get_association_for_service_quota_template(self) -> GetAssociationForServiceQuotaTemplateResponse:
        """Retrieves the <code>ServiceQuotaTemplateAssociationStatus</code> value from the service. Use this action to determine if the Service Quota template is associated, or enabled. 

        :param self:
        :param 
        :return: ServiceQuotaTemplateAssociationStatus:  Specifies whether the template is <code>ASSOCIATED</code> or <code>DISASSOCIATED</code>. If the template is <code>ASSOCIATED</code>, then it requests service quota increases for all new accounts created in your organization. <br/>        """
        pass

    def get_requested_service_quota_change(self, RequestId: str) -> GetRequestedServiceQuotaChangeResponse:
        """Retrieves the details for a particular increase request. 

        :param self:
        :param RequestId:  Identifies the quota increase request.<br/>
        :return: RequestedQuota:  Returns the <code>RequestedServiceQuotaChange</code> object for the specific increase request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Unit</i> Specifies the unit used for the quota.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalQuota</i> Identifies if the quota is global.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QuotaArn</i> The Amazon Resource Name (ARN) of the service quota.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Requester</i> The IAM identity who submitted the service quota increase request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdated</i> The date and time of the most recent change in the service quota increase request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Created</i> The date and time when the service quota increase request was received and the case Id was created. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> State of the service quota increase request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DesiredValue</i> New increased value for the service quota.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QuotaName</i> Name of the service quota.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QuotaCode</i> Specifies the service quota that you want to use.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceName</i> The name of the AWS service specified in the increase request. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceCode</i> Specifies the service that you want to use.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CaseId</i> The case Id for the service quota increase request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier of a requested service quota change.<br/>        """
        pass

    def get_service_quota(self, QuotaCode: str, ServiceCode: str) -> GetServiceQuotaResponse:
        """Returns the details for the specified service quota. This operation provides a different Value than the <code>GetAWSDefaultServiceQuota</code> operation. This operation returns the applied value for each quota. <code>GetAWSDefaultServiceQuota</code> returns the default AWS value for each quota. 

        :param self:
        :param QuotaCode:  Identifies the service quota you want to select.<br/>ServiceCode:  Specifies the service that you want to use.<br/>
        :return: Quota:  Returns the <a>ServiceQuota</a> object which contains all values for a quota.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorReason</i> Specifies the <code>ErrorCode</code> and <code>ErrorMessage</code> when success isn't achieved.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ErrorReason</b>: Specifies the <code>ErrorCode</code> and <code>ErrorMessage</code> when success isn't achieved.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Period</i> Identifies the unit and value of how time is measured.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Period</b>: Identifies the unit and value of how time is measured.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UsageMetric</i> Specifies the details about the measurement. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>UsageMetric</b>: Specifies the details about the measurement. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalQuota</i> Specifies if the quota is global.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Adjustable</i> Specifies if the quota value can be increased.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Unit</i> The unit of measurement for the value of the service quota.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The value of service quota.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QuotaName</i> The name identifier of the service quota.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QuotaCode</i> The code identifier for the service quota specified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QuotaArn</i> The Amazon Resource Name (ARN) of the service quota.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceName</i> The name of the AWS service specified in the increase request. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceCode</i> Specifies the service that you want to use.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricStatisticRecommendation</i> Statistics are metric data aggregations over specified periods of time. This is the recommended statistic to use when comparing usage in the CloudWatch Metric against your Service Quota.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricDimensions</i> A dimension is a name/value pair that is part of the identity of a metric. Every metric has specific characteristics that describe it, and you can think of dimensions as categories for those characteristics. These dimensions are part of the CloudWatch Metric Identity that measures usage against a particular service quota.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricName</i> The name of the CloudWatch metric that measures usage of a service quota. This is a required field.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricNamespace</i> The namespace of the metric. The namespace is a container for CloudWatch metrics. You can specify a name for the namespace when you create a metric.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PeriodUnit</i> The time unit of a period.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>PeriodValue</i> The value of a period.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorMessage</i> The error message that provides more detail.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorCode</i> Service Quotas returns the following error values.   <code>DEPENDENCY_ACCESS_DENIED_ERROR</code> is returned when the caller does not have permission to call the service or service quota. To resolve the error, you need permission to access the service or service quota.  <code>DEPENDENCY_THROTTLING_ERROR</code> is returned when the service being called is throttling Service Quotas.  <code>DEPENDENCY_SERVICE_ERROR</code> is returned when the service being called has availability issues.  <code>SERVICE_QUOTA_NOT_AVAILABLE_ERROR</code> is returned when there was an error in Service Quotas.<br/>        """
        pass

    def get_service_quota_increase_request_from_template(self, AwsRegion: str, QuotaCode: str, ServiceCode: str) -> GetServiceQuotaIncreaseRequestFromTemplateResponse:
        """Returns the details of the service quota increase request in your template.

        :param self:
        :param AwsRegion:  Specifies the AWS Region for the quota that you want to use.<br/>QuotaCode:  Specifies the quota you want.<br/>ServiceCode:  Specifies the service that you want to use.<br/>
        :return: ServiceQuotaIncreaseRequestInTemplate:  This object contains the details about the quota increase request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalQuota</i> Specifies if the quota is a global quota.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Unit</i> The unit of measure for the increase request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AwsRegion</i> The AWS Region where the increase request occurs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DesiredValue</i> Identifies the new, increased value of the service quota in the increase request. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QuotaName</i> The name of the service quota in the increase request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QuotaCode</i> The code identifier for the service quota specified in the increase request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceName</i> The name of the AWS service specified in the increase request. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceCode</i> The code identifier for the AWS service specified in the increase request.<br/>        """
        pass

    def list_aws_default_service_quotas(self, ServiceCode: str, NextToken: str=None, MaxResults: int=None) -> ListAWSDefaultServiceQuotasResponse:
        """Lists all default service quotas for the specified AWS service or all AWS services. ListAWSDefaultServiceQuotas is similar to <a>ListServiceQuotas</a> except for the Value object. The Value object returned by <code>ListAWSDefaultServiceQuotas</code> is the default value assigned by AWS. This request returns a list of all service quotas for the specified service. The listing of each you'll see the default values are the values that AWS provides for the quotas.  <note> Always check the <code>NextToken</code> response parameter when calling any of the <code>List*</code> operations. These operations can return an unexpected list of results, even when there are more results available. When this happens, the <code>NextToken</code> response parameter contains a value to pass the next call to the same API to request the next part of the list. </note>

        :param self:
        :param MaxResults:  (Optional) Limits the number of results that you want to include in the response. If you don't include this parameter, the response defaults to a value that's specific to the operation. If additional items exist beyond the specified maximum, the <code>NextToken</code> element is present and has a value (isn't null). Include that value as the <code>NextToken</code> request parameter in the call to the operation to get the next part of the results. You should check <code>NextToken</code> after every operation to ensure that you receive all of the results.<br/>NextToken:  (Optional) Use this parameter in a request if you receive a <code>NextToken</code> response in a previous request that indicates that there's more output available. In a subsequent call, set it to the value of the previous call's <code>NextToken</code> response to indicate where the output should continue from. If additional items exist beyond the specified maximum, the <code>NextToken</code> element is present and has a value (isn't null). Include that value as the <code>NextToken</code> request parameter in the call to the operation to get the next part of the results. You should check <code>NextToken</code> after every operation to ensure that you receive all of the results.<br/>ServiceCode:  Specifies the service that you want to use.<br/>
        :return: Quotas:  A list of the quotas in the account with the AWS default values. <br/>NextToken:  (Optional) Use this parameter in a request if you receive a <code>NextToken</code> response in a previous request that indicates that there's more output available. In a subsequent call, set it to the value of the previous call's <code>NextToken</code> response to indicate where the output should continue from.<br/>        """
        pass

    def list_requested_service_quota_change_history(self, ServiceCode: str=None, Status: str=None, NextToken: str=None, MaxResults: int=None) -> ListRequestedServiceQuotaChangeHistoryResponse:
        """Requests a list of the changes to quotas for a service.

        :param self:
        :param MaxResults:  (Optional) Limits the number of results that you want to include in the response. If you don't include this parameter, the response defaults to a value that's specific to the operation. If additional items exist beyond the specified maximum, the <code>NextToken</code> element is present and has a value (isn't null). Include that value as the <code>NextToken</code> request parameter in the call to the operation to get the next part of the results. You should check <code>NextToken</code> after every operation to ensure that you receive all of the results.<br/>NextToken:  (Optional) Use this parameter in a request if you receive a <code>NextToken</code> response in a previous request that indicates that there's more output available. In a subsequent call, set it to the value of the previous call's <code>NextToken</code> response to indicate where the output should continue from.<br/>Status:  Specifies the status value of the quota increase request.<br/>ServiceCode:  Specifies the service that you want to use.<br/>
        :return: RequestedQuotas:  Returns a list of service quota requests.<br/>NextToken:  If present in the response, this value indicates there's more output available that what's included in the current response. This can occur even when the response includes no values at all, such as when you ask for a filtered view of a very long list. Use this value in the <code>NextToken</code> request parameter in a subsequent call to the operation to continue processing and get the next part of the output. You should repeat this until the <code>NextToken</code> response element comes back empty (as <code>null</code>).<br/>        """
        pass

    def list_requested_service_quota_change_history_by_quota(self, QuotaCode: str, ServiceCode: str, Status: str=None, NextToken: str=None, MaxResults: int=None) -> ListRequestedServiceQuotaChangeHistoryByQuotaResponse:
        """Requests a list of the changes to specific service quotas. This command provides additional granularity over the <code>ListRequestedServiceQuotaChangeHistory</code> command. Once a quota change request has reached <code>CASE_CLOSED, APPROVED,</code> or <code>DENIED</code>, the history has been kept for 90 days.

        :param self:
        :param MaxResults:  (Optional) Limits the number of results that you want to include in the response. If you don't include this parameter, the response defaults to a value that's specific to the operation. If additional items exist beyond the specified maximum, the <code>NextToken</code> element is present and has a value (isn't null). Include that value as the <code>NextToken</code> request parameter in the call to the operation to get the next part of the results. You should check <code>NextToken</code> after every operation to ensure that you receive all of the results.<br/>NextToken:  (Optional) Use this parameter in a request if you receive a <code>NextToken</code> response in a previous request that indicates that there's more output available. In a subsequent call, set it to the value of the previous call's <code>NextToken</code> response to indicate where the output should continue from.<br/>Status:  Specifies the status value of the quota increase request.<br/>QuotaCode:  Specifies the service quota that you want to use<br/>ServiceCode:  Specifies the service that you want to use.<br/>
        :return: RequestedQuotas:  Returns a list of service quota requests.<br/>NextToken:  If present in the response, this value indicates there's more output available that what's included in the current response. This can occur even when the response includes no values at all, such as when you ask for a filtered view of a very long list. Use this value in the <code>NextToken</code> request parameter in a subsequent call to the operation to continue processing and get the next part of the output. You should repeat this until the <code>NextToken</code> response element comes back empty (as <code>null</code>).<br/>        """
        pass

    def list_service_quota_increase_requests_in_template(self, ServiceCode: str=None, AwsRegion: str=None, NextToken: str=None, MaxResults: int=None) -> ListServiceQuotaIncreaseRequestsInTemplateResponse:
        """Returns a list of the quota increase requests in the template. 

        :param self:
        :param MaxResults:  (Optional) Limits the number of results that you want to include in the response. If you don't include this parameter, the response defaults to a value that's specific to the operation. If additional items exist beyond the specified maximum, the <code>NextToken</code> element is present and has a value (isn't null). Include that value as the <code>NextToken</code> request parameter in the call to the operation to get the next part of the results. You should check <code>NextToken</code> after every operation to ensure that you receive all of the results.<br/>NextToken:  (Optional) Use this parameter in a request if you receive a <code>NextToken</code> response in a previous request that indicates that there's more output available. In a subsequent call, set it to the value of the previous call's <code>NextToken</code> response to indicate where the output should continue from.<br/>AwsRegion:  Specifies the AWS Region for the quota that you want to use.<br/>ServiceCode:  The identifier for a service. When performing an operation, use the <code>ServiceCode</code> to specify a particular service. <br/>
        :return: NextToken:  If present in the response, this value indicates there's more output available that what's included in the current response. This can occur even when the response includes no values at all, such as when you ask for a filtered view of a very long list. Use this value in the <code>NextToken</code> request parameter in a subsequent call to the operation to continue processing and get the next part of the output. You should repeat this until the <code>NextToken</code> response element comes back empty (as <code>null</code>).<br/>ServiceQuotaIncreaseRequestInTemplateList:  Returns the list of values of the quota increase request in the template.<br/>        """
        pass

    def list_service_quotas(self, ServiceCode: str, NextToken: str=None, MaxResults: int=None) -> ListServiceQuotasResponse:
        """Lists all service quotas for the specified AWS service. This request returns a list of the service quotas for the specified service. you'll see the default values are the values that AWS provides for the quotas.  <note> Always check the <code>NextToken</code> response parameter when calling any of the <code>List*</code> operations. These operations can return an unexpected list of results, even when there are more results available. When this happens, the <code>NextToken</code> response parameter contains a value to pass the next call to the same API to request the next part of the list. </note>

        :param self:
        :param MaxResults:  (Optional) Limits the number of results that you want to include in the response. If you don't include this parameter, the response defaults to a value that's specific to the operation. If additional items exist beyond the specified maximum, the <code>NextToken</code> element is present and has a value (isn't null). Include that value as the <code>NextToken</code> request parameter in the call to the operation to get the next part of the results. You should check <code>NextToken</code> after every operation to ensure that you receive all of the results.<br/>NextToken:  (Optional) Use this parameter in a request if you receive a <code>NextToken</code> response in a previous request that indicates that there's more output available. In a subsequent call, set it to the value of the previous call's <code>NextToken</code> response to indicate where the output should continue from.<br/>ServiceCode:  The identifier for a service. When performing an operation, use the <code>ServiceCode</code> to specify a particular service. <br/>
        :return: Quotas:  The response information for a quota lists all attribute information for the quota. <br/>NextToken:  If present in the response, this value indicates there's more output available that what's included in the current response. This can occur even when the response includes no values at all, such as when you ask for a filtered view of a very long list. Use this value in the <code>NextToken</code> request parameter in a subsequent call to the operation to continue processing and get the next part of the output. You should repeat this until the <code>NextToken</code> response element comes back empty (as <code>null</code>).<br/>        """
        pass

    def list_services(self, NextToken: str=None, MaxResults: int=None) -> ListServicesResponse:
        """Lists the AWS services available in Service Quotas. Not all AWS services are available in Service Quotas. To list the see the list of the service quotas for a specific service, use <a>ListServiceQuotas</a>.

        :param self:
        :param MaxResults:  (Optional) Limits the number of results that you want to include in the response. If you don't include this parameter, the response defaults to a value that's specific to the operation. If additional items exist beyond the specified maximum, the <code>NextToken</code> element is present and has a value (isn't null). Include that value as the <code>NextToken</code> request parameter in the call to the operation to get the next part of the results. You should check <code>NextToken</code> after every operation to ensure that you receive all of the results.<br/>NextToken:  (Optional) Use this parameter in a request if you receive a <code>NextToken</code> response in a previous request that indicates that there's more output available. In a subsequent call, set it to the value of the previous call's <code>NextToken</code> response to indicate where the output should continue from.<br/>
        :return: Services:  Returns a list of services. <br/>NextToken:  If present in the response, this value indicates there's more output available that what's included in the current response. This can occur even when the response includes no values at all, such as when you ask for a filtered view of a very long list. Use this value in the <code>NextToken</code> request parameter in a subsequent call to the operation to continue processing and get the next part of the output. You should repeat this until the <code>NextToken</code> response element comes back empty (as <code>null</code>).<br/>        """
        pass

    def put_service_quota_increase_request_into_template(self, DesiredValue: QuotaValue, AwsRegion: str, ServiceCode: str, QuotaCode: str) -> PutServiceQuotaIncreaseRequestIntoTemplateResponse:
        """Defines and adds a quota to the service quota template. To add a quota to the template, you must provide the <code>ServiceCode</code>, <code>QuotaCode</code>, <code>AwsRegion</code>, and <code>DesiredValue</code>. Once you add a quota to the template, use <a>ListServiceQuotaIncreaseRequestsInTemplate</a> to see the list of quotas in the template.

        :param self:
        :param DesiredValue:  Specifies the new, increased value for the quota. <br/>AwsRegion:  Specifies the AWS Region for the quota. <br/>ServiceCode:  Specifies the service that you want to use.<br/>QuotaCode:  Specifies the service quota that you want to use.<br/>
        :return: ServiceQuotaIncreaseRequestInTemplate:  A structure that contains information about one service quota increase request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalQuota</i> Specifies if the quota is a global quota.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Unit</i> The unit of measure for the increase request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AwsRegion</i> The AWS Region where the increase request occurs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DesiredValue</i> Identifies the new, increased value of the service quota in the increase request. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QuotaName</i> The name of the service quota in the increase request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QuotaCode</i> The code identifier for the service quota specified in the increase request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceName</i> The name of the AWS service specified in the increase request. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceCode</i> The code identifier for the AWS service specified in the increase request.<br/>        """
        pass

    def request_service_quota_increase(self, DesiredValue: QuotaValue, QuotaCode: str, ServiceCode: str) -> RequestServiceQuotaIncreaseResponse:
        """Retrieves the details of a service quota increase request. The response to this command provides the details in the <a>RequestedServiceQuotaChange</a> object. 

        :param self:
        :param DesiredValue:  Specifies the value submitted in the service quota increase request. <br/>QuotaCode:  Specifies the service quota that you want to use.<br/>ServiceCode:  Specifies the service that you want to use.<br/>
        :return: RequestedQuota:  Returns a list of service quota requests.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Unit</i> Specifies the unit used for the quota.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GlobalQuota</i> Identifies if the quota is global.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QuotaArn</i> The Amazon Resource Name (ARN) of the service quota.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Requester</i> The IAM identity who submitted the service quota increase request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdated</i> The date and time of the most recent change in the service quota increase request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Created</i> The date and time when the service quota increase request was received and the case Id was created. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> State of the service quota increase request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DesiredValue</i> New increased value for the service quota.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QuotaName</i> Name of the service quota.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>QuotaCode</i> Specifies the service quota that you want to use.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceName</i> The name of the AWS service specified in the increase request. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceCode</i> Specifies the service that you want to use.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CaseId</i> The case Id for the service quota increase request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier of a requested service quota change.<br/>        """
        pass



class ServiceCatalog(BaseClient):
    """<fullname>AWS Service Catalog</fullname>  <a href="https://aws.amazon.com/servicecatalog/">AWS Service Catalog</a> enables organizations to create and manage catalogs of IT services that are approved for use on AWS. To get the most out of this documentation, you should be familiar with the terminology discussed in <a href="http://docs.aws.amazon.com/servicecatalog/latest/adminguide/what-is_concepts.html">AWS Service Catalog Concepts</a>."""


    class SearchProductsAsAdminPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `search_products_as_admin` for valid parameters"""
            pass
    class ListAcceptedPortfolioSharesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_accepted_portfolio_shares` for valid parameters"""
            pass
    class ListPortfoliosPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_portfolios` for valid parameters"""
            pass
    class ListConstraintsForPortfolioPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_constraints_for_portfolio` for valid parameters"""
            pass
    class ListLaunchPathsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_launch_paths` for valid parameters"""
            pass
    class ListTagOptionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_tag_options` for valid parameters"""
            pass
    class ListPortfoliosForProductPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_portfolios_for_product` for valid parameters"""
            pass
    class ListPrincipalsForPortfolioPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_principals_for_portfolio` for valid parameters"""
            pass
    class ListResourcesForTagOptionPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_resources_for_tag_option` for valid parameters"""
            pass
    class ListOrganizationPortfolioAccessPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_organization_portfolio_access` for valid parameters"""
            pass
    class ListProvisionedProductPlansPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_provisioned_product_plans` for valid parameters"""
            pass
    class ListProvisioningArtifactsForServiceActionPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_provisioning_artifacts_for_service_action` for valid parameters"""
            pass
    class ListRecordHistoryPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_record_history` for valid parameters"""
            pass
    class ListServiceActionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_service_actions` for valid parameters"""
            pass
    class ListServiceActionsForProvisioningArtifactPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_service_actions_for_provisioning_artifact` for valid parameters"""
            pass
    class ScanProvisionedProductsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `scan_provisioned_products` for valid parameters"""
            pass


    class AcceptPortfolioShareInput(Mapping):
        pass
    
    class AcceptPortfolioShareOutput(Mapping):
        pass
    
    class AssociateBudgetWithResourceInput(Mapping):
        pass
    
    class AssociateBudgetWithResourceOutput(Mapping):
        pass
    
    class AssociatePrincipalWithPortfolioInput(Mapping):
        pass
    
    class AssociatePrincipalWithPortfolioOutput(Mapping):
        pass
    
    class AssociateProductWithPortfolioInput(Mapping):
        pass
    
    class AssociateProductWithPortfolioOutput(Mapping):
        pass
    
    class AssociateServiceActionWithProvisioningArtifactInput(Mapping):
        pass
    
    class AssociateServiceActionWithProvisioningArtifactOutput(Mapping):
        pass
    
    class AssociateTagOptionWithResourceInput(Mapping):
        pass
    
    class AssociateTagOptionWithResourceOutput(Mapping):
        pass
    
    class BatchAssociateServiceActionWithProvisioningArtifactInput(Mapping):
        pass
    
    class BatchAssociateServiceActionWithProvisioningArtifactOutput(Mapping):
        pass
    
    class ServiceActionAssociation(Mapping):
        pass
    
    class BatchDisassociateServiceActionFromProvisioningArtifactInput(Mapping):
        pass
    
    class BatchDisassociateServiceActionFromProvisioningArtifactOutput(Mapping):
        pass
    
    class CopyProductInput(Mapping):
        pass
    
    class CopyProductOutput(Mapping):
        pass
    
    class SourceProvisioningArtifactPropertiesMap(object):
        pass
    
    class CopyOption(object):
        pass
    
    class CreateConstraintInput(Mapping):
        pass
    
    class CreateConstraintOutput(Mapping):
        pass
    
    class CreatePortfolioInput(Mapping):
        pass
    
    class CreatePortfolioOutput(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreatePortfolioShareInput(Mapping):
        pass
    
    class CreatePortfolioShareOutput(Mapping):
        pass
    
    class OrganizationNode(Mapping):
        pass
    
    class CreateProductInput(Mapping):
        pass
    
    class CreateProductOutput(Mapping):
        pass
    
    class ProvisioningArtifactProperties(Mapping):
        pass
    
    class CreateProvisionedProductPlanInput(Mapping):
        pass
    
    class CreateProvisionedProductPlanOutput(Mapping):
        pass
    
    class NotificationArn(object):
        pass
    
    class UpdateProvisioningParameter(Mapping):
        pass
    
    class CreateProvisioningArtifactInput(Mapping):
        pass
    
    class CreateProvisioningArtifactOutput(Mapping):
        pass
    
    class CreateServiceActionInput(Mapping):
        pass
    
    class CreateServiceActionOutput(Mapping):
        pass
    
    class ServiceActionDefinitionMap(object):
        pass
    
    class CreateTagOptionInput(Mapping):
        pass
    
    class CreateTagOptionOutput(Mapping):
        pass
    
    class DeleteConstraintInput(Mapping):
        pass
    
    class DeleteConstraintOutput(Mapping):
        pass
    
    class DeletePortfolioInput(Mapping):
        pass
    
    class DeletePortfolioOutput(Mapping):
        pass
    
    class DeletePortfolioShareInput(Mapping):
        pass
    
    class DeletePortfolioShareOutput(Mapping):
        pass
    
    class DeleteProductInput(Mapping):
        pass
    
    class DeleteProductOutput(Mapping):
        pass
    
    class DeleteProvisionedProductPlanInput(Mapping):
        pass
    
    class DeleteProvisionedProductPlanOutput(Mapping):
        pass
    
    class IgnoreErrors(object):
        pass
    
    class DeleteProvisioningArtifactInput(Mapping):
        pass
    
    class DeleteProvisioningArtifactOutput(Mapping):
        pass
    
    class DeleteServiceActionInput(Mapping):
        pass
    
    class DeleteServiceActionOutput(Mapping):
        pass
    
    class DeleteTagOptionInput(Mapping):
        pass
    
    class DeleteTagOptionOutput(Mapping):
        pass
    
    class DescribeConstraintInput(Mapping):
        pass
    
    class DescribeConstraintOutput(Mapping):
        pass
    
    class DescribeCopyProductStatusInput(Mapping):
        pass
    
    class DescribeCopyProductStatusOutput(Mapping):
        pass
    
    class DescribePortfolioInput(Mapping):
        pass
    
    class DescribePortfolioOutput(Mapping):
        pass
    
    class DescribePortfolioShareStatusInput(Mapping):
        pass
    
    class DescribePortfolioShareStatusOutput(Mapping):
        pass
    
    class DescribeProductInput(Mapping):
        pass
    
    class DescribeProductOutput(Mapping):
        pass
    
    class DescribeProductAsAdminInput(Mapping):
        pass
    
    class DescribeProductAsAdminOutput(Mapping):
        pass
    
    class DescribeProductViewInput(Mapping):
        pass
    
    class DescribeProductViewOutput(Mapping):
        pass
    
    class DescribeProvisionedProductInput(Mapping):
        pass
    
    class DescribeProvisionedProductOutput(Mapping):
        pass
    
    class DescribeProvisionedProductPlanInput(Mapping):
        pass
    
    class DescribeProvisionedProductPlanOutput(Mapping):
        pass
    
    class DescribeProvisioningArtifactInput(Mapping):
        pass
    
    class DescribeProvisioningArtifactOutput(Mapping):
        pass
    
    class Verbose(object):
        pass
    
    class DescribeProvisioningParametersInput(Mapping):
        pass
    
    class DescribeProvisioningParametersOutput(Mapping):
        pass
    
    class DescribeRecordInput(Mapping):
        pass
    
    class DescribeRecordOutput(Mapping):
        pass
    
    class DescribeServiceActionInput(Mapping):
        pass
    
    class DescribeServiceActionOutput(Mapping):
        pass
    
    class DescribeServiceActionExecutionParametersInput(Mapping):
        pass
    
    class DescribeServiceActionExecutionParametersOutput(Mapping):
        pass
    
    class DescribeTagOptionInput(Mapping):
        pass
    
    class DescribeTagOptionOutput(Mapping):
        pass
    
    class DisableAWSOrganizationsAccessInput(Mapping):
        pass
    
    class DisableAWSOrganizationsAccessOutput(Mapping):
        pass
    
    class DisassociateBudgetFromResourceInput(Mapping):
        pass
    
    class DisassociateBudgetFromResourceOutput(Mapping):
        pass
    
    class DisassociatePrincipalFromPortfolioInput(Mapping):
        pass
    
    class DisassociatePrincipalFromPortfolioOutput(Mapping):
        pass
    
    class DisassociateProductFromPortfolioInput(Mapping):
        pass
    
    class DisassociateProductFromPortfolioOutput(Mapping):
        pass
    
    class DisassociateServiceActionFromProvisioningArtifactInput(Mapping):
        pass
    
    class DisassociateServiceActionFromProvisioningArtifactOutput(Mapping):
        pass
    
    class DisassociateTagOptionFromResourceInput(Mapping):
        pass
    
    class DisassociateTagOptionFromResourceOutput(Mapping):
        pass
    
    class EnableAWSOrganizationsAccessInput(Mapping):
        pass
    
    class EnableAWSOrganizationsAccessOutput(Mapping):
        pass
    
    class ExecuteProvisionedProductPlanInput(Mapping):
        pass
    
    class ExecuteProvisionedProductPlanOutput(Mapping):
        pass
    
    class ExecuteProvisionedProductServiceActionInput(Mapping):
        pass
    
    class ExecuteProvisionedProductServiceActionOutput(Mapping):
        pass
    
    class ExecutionParameterMap(object):
        pass
    
    class GetAWSOrganizationsAccessStatusInput(Mapping):
        pass
    
    class GetAWSOrganizationsAccessStatusOutput(Mapping):
        pass
    
    class ListAcceptedPortfolioSharesInput(Mapping):
        pass
    
    class ListAcceptedPortfolioSharesOutput(Mapping):
        pass
    
    class ListBudgetsForResourceInput(Mapping):
        pass
    
    class ListBudgetsForResourceOutput(Mapping):
        pass
    
    class ListConstraintsForPortfolioInput(Mapping):
        pass
    
    class ListConstraintsForPortfolioOutput(Mapping):
        pass
    
    class ListLaunchPathsInput(Mapping):
        pass
    
    class ListLaunchPathsOutput(Mapping):
        pass
    
    class ListOrganizationPortfolioAccessInput(Mapping):
        pass
    
    class ListOrganizationPortfolioAccessOutput(Mapping):
        pass
    
    class ListPortfolioAccessInput(Mapping):
        pass
    
    class ListPortfolioAccessOutput(Mapping):
        pass
    
    class ListPortfoliosInput(Mapping):
        pass
    
    class ListPortfoliosOutput(Mapping):
        pass
    
    class ListPortfoliosForProductInput(Mapping):
        pass
    
    class ListPortfoliosForProductOutput(Mapping):
        pass
    
    class ListPrincipalsForPortfolioInput(Mapping):
        pass
    
    class ListPrincipalsForPortfolioOutput(Mapping):
        pass
    
    class ListProvisionedProductPlansInput(Mapping):
        pass
    
    class ListProvisionedProductPlansOutput(Mapping):
        pass
    
    class AccessLevelFilter(Mapping):
        pass
    
    class ListProvisioningArtifactsInput(Mapping):
        pass
    
    class ListProvisioningArtifactsOutput(Mapping):
        pass
    
    class ListProvisioningArtifactsForServiceActionInput(Mapping):
        pass
    
    class ListProvisioningArtifactsForServiceActionOutput(Mapping):
        pass
    
    class ListRecordHistoryInput(Mapping):
        pass
    
    class ListRecordHistoryOutput(Mapping):
        pass
    
    class ListRecordHistorySearchFilter(Mapping):
        pass
    
    class ListResourcesForTagOptionInput(Mapping):
        pass
    
    class ListResourcesForTagOptionOutput(Mapping):
        pass
    
    class ListServiceActionsInput(Mapping):
        pass
    
    class ListServiceActionsOutput(Mapping):
        pass
    
    class ListServiceActionsForProvisioningArtifactInput(Mapping):
        pass
    
    class ListServiceActionsForProvisioningArtifactOutput(Mapping):
        pass
    
    class ListStackInstancesForProvisionedProductInput(Mapping):
        pass
    
    class ListStackInstancesForProvisionedProductOutput(Mapping):
        pass
    
    class ListTagOptionsInput(Mapping):
        pass
    
    class ListTagOptionsOutput(Mapping):
        pass
    
    class ListTagOptionsFilters(Mapping):
        pass
    
    class ProvisionProductInput(Mapping):
        pass
    
    class ProvisionProductOutput(Mapping):
        pass
    
    class ProvisioningParameter(Mapping):
        pass
    
    class ProvisioningPreferences(Mapping):
        pass
    
    class RejectPortfolioShareInput(Mapping):
        pass
    
    class RejectPortfolioShareOutput(Mapping):
        pass
    
    class ScanProvisionedProductsInput(Mapping):
        pass
    
    class ScanProvisionedProductsOutput(Mapping):
        pass
    
    class SearchProductsInput(Mapping):
        pass
    
    class SearchProductsOutput(Mapping):
        pass
    
    class ProductViewFilters(object):
        pass
    
    class SearchProductsAsAdminInput(Mapping):
        pass
    
    class SearchProductsAsAdminOutput(Mapping):
        pass
    
    class SearchProvisionedProductsInput(Mapping):
        pass
    
    class SearchProvisionedProductsOutput(Mapping):
        pass
    
    class ProvisionedProductFilters(object):
        pass
    
    class TerminateProvisionedProductInput(Mapping):
        pass
    
    class TerminateProvisionedProductOutput(Mapping):
        pass
    
    class UpdateConstraintInput(Mapping):
        pass
    
    class UpdateConstraintOutput(Mapping):
        pass
    
    class UpdatePortfolioInput(Mapping):
        pass
    
    class UpdatePortfolioOutput(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateProductInput(Mapping):
        pass
    
    class UpdateProductOutput(Mapping):
        pass
    
    class UpdateProvisionedProductInput(Mapping):
        pass
    
    class UpdateProvisionedProductOutput(Mapping):
        pass
    
    class UpdateProvisioningPreferences(Mapping):
        pass
    
    class UpdateProvisionedProductPropertiesInput(Mapping):
        pass
    
    class UpdateProvisionedProductPropertiesOutput(Mapping):
        pass
    
    class ProvisionedProductProperties(object):
        pass
    
    class UpdateProvisioningArtifactInput(Mapping):
        pass
    
    class UpdateProvisioningArtifactOutput(Mapping):
        pass
    
    class ProvisioningArtifactActive(object):
        pass
    
    class UpdateServiceActionInput(Mapping):
        pass
    
    class UpdateServiceActionOutput(Mapping):
        pass
    
    class UpdateTagOptionInput(Mapping):
        pass
    
    class UpdateTagOptionOutput(Mapping):
        pass
    
    class TagOptionActive(object):
        pass
    
    def accept_portfolio_share(self, PortfolioId: str, AcceptLanguage: str=None, PortfolioShareType: str=None) -> AcceptPortfolioShareOutput:
        """Accepts an offer to share the specified portfolio.

        :param self:
        :param PortfolioShareType:  The type of shared portfolios to accept. The default is to accept imported portfolios. <ul> <li>  <code>AWS_ORGANIZATIONS</code> - Accept portfolios shared by the master account of your organization. </li> <li>  <code>IMPORTED</code> - Accept imported portfolios. </li> <li>  <code>AWS_SERVICECATALOG</code> - Not supported. (Throws ResourceNotFoundException.) </li> </ul> For example, <code>aws servicecatalog accept-portfolio-share --portfolio-id "port-2qwzkwxt3y5fk" --portfolio-share-type AWS_ORGANIZATIONS</code> <br/>PortfolioId:  The portfolio identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return:         """
        pass

    def associate_budget_with_resource(self, ResourceId: str, BudgetName: str) -> AssociateBudgetWithResourceOutput:
        """Associates the specified budget with the specified resource.

        :param self:
        :param ResourceId:   The resource identifier. Either a portfolio-id or a product-id.<br/>BudgetName:  The name of the budget you want to associate.<br/>
        :return:         """
        pass

    def associate_principal_with_portfolio(self, PrincipalType: str, PrincipalARN: str, PortfolioId: str, AcceptLanguage: str=None) -> AssociatePrincipalWithPortfolioOutput:
        """Associates the specified principal ARN with the specified portfolio.

        :param self:
        :param PrincipalType:  The principal type. The supported value is <code>IAM</code>.<br/>PrincipalARN:  The ARN of the principal (IAM user, role, or group).<br/>PortfolioId:  The portfolio identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return:         """
        pass

    def associate_product_with_portfolio(self, PortfolioId: str, ProductId: str, AcceptLanguage: str=None, SourcePortfolioId: str=None) -> AssociateProductWithPortfolioOutput:
        """Associates the specified product with the specified portfolio.

        :param self:
        :param SourcePortfolioId:  The identifier of the source portfolio.<br/>PortfolioId:  The portfolio identifier.<br/>ProductId:  The product identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return:         """
        pass

    def associate_service_action_with_provisioning_artifact(self, ServiceActionId: str, ProvisioningArtifactId: str, ProductId: str, AcceptLanguage: str=None) -> AssociateServiceActionWithProvisioningArtifactOutput:
        """Associates a self-service action with a provisioning artifact.

        :param self:
        :param AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>ServiceActionId:  The self-service action identifier. For example, <code>act-fs7abcd89wxyz</code>.<br/>ProvisioningArtifactId:  The identifier of the provisioning artifact. For example, <code>pa-4abcdjnxjj6ne</code>.<br/>ProductId:  The product identifier. For example, <code>prod-abcdzk7xy33qa</code>.<br/>
        :return:         """
        pass

    def associate_tag_option_with_resource(self, TagOptionId: str, ResourceId: str) -> AssociateTagOptionWithResourceOutput:
        """Associate the specified TagOption with the specified portfolio or product.

        :param self:
        :param TagOptionId:  The TagOption identifier.<br/>ResourceId:  The resource identifier.<br/>
        :return:         """
        pass

    def batch_associate_service_action_with_provisioning_artifact(self, ServiceActionAssociations: List[ServiceActionAssociation], AcceptLanguage: str=None) -> BatchAssociateServiceActionWithProvisioningArtifactOutput:
        """Associates multiple self-service actions with provisioning artifacts.

        :param self:
        :param AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>ServiceActionAssociations:  One or more associations, each consisting of the Action ID, the Product ID, and the Provisioning Artifact ID.<br/>
        :return: FailedServiceActionAssociations:  An object that contains a list of errors, along with information to help you identify the self-service action.<br/>        """
        pass

    def batch_disassociate_service_action_from_provisioning_artifact(self, ServiceActionAssociations: List[ServiceActionAssociation], AcceptLanguage: str=None) -> BatchDisassociateServiceActionFromProvisioningArtifactOutput:
        """Disassociates a batch of self-service actions from the specified provisioning artifact.

        :param self:
        :param AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>ServiceActionAssociations:  One or more associations, each consisting of the Action ID, the Product ID, and the Provisioning Artifact ID.<br/>
        :return: FailedServiceActionAssociations:  An object that contains a list of errors, along with information to help you identify the self-service action.<br/>        """
        pass

    def copy_product(self, IdempotencyToken: str, SourceProductArn: str, AcceptLanguage: str=None, TargetProductId: str=None, TargetProductName: str=None, SourceProvisioningArtifactIdentifiers: List[SourceProvisioningArtifactPropertiesMap]=None, CopyOptions: List[CopyOption]=None) -> CopyProductOutput:
        """Copies the specified source product to the specified target product or a new product. You can copy a product to the same account or another account. You can copy a product to the same region or another region. This operation is performed asynchronously. To track the progress of the operation, use <a>DescribeCopyProductStatus</a>.

        :param self:
        :param IdempotencyToken:   A unique identifier that you provide to ensure idempotency. If multiple requests differ only by the idempotency token, the same response is returned for each repeated request. <br/>CopyOptions:  The copy options. If the value is <code>CopyTags</code>, the tags from the source product are copied to the target product.<br/>SourceProvisioningArtifactIdentifiers:  The identifiers of the provisioning artifacts (also known as versions) of the product to copy. By default, all provisioning artifacts are copied.<br/>TargetProductName:  A name for the target product. The default is the name of the source product.<br/>TargetProductId:  The identifier of the target product. By default, a new product is created.<br/>SourceProductArn:  The Amazon Resource Name (ARN) of the source product.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: CopyProductToken:  The token to use to track the progress of the operation.<br/>        """
        pass

    def create_constraint(self, IdempotencyToken: str, Type: str, Parameters: str, ProductId: str, PortfolioId: str, AcceptLanguage: str=None, Description: str=None) -> CreateConstraintOutput:
        """Creates a constraint.

        :param self:
        :param IdempotencyToken:  A unique identifier that you provide to ensure idempotency. If multiple requests differ only by the idempotency token, the same response is returned for each repeated request.<br/>Description:  The description of the constraint.<br/>Type:  The type of constraint. <ul> <li>  <code>LAUNCH</code>  </li> <li>  <code>NOTIFICATION</code>  </li> <li>  <code>RESOURCE_UPDATE</code>  </li> <li>  <code>STACKSET</code>  </li> <li>  <code>TEMPLATE</code>  </li> </ul><br/>Parameters:  The constraint parameters, in JSON format. The syntax depends on the constraint type as follows: <dl> <dt>LAUNCH</dt> <dd> Specify the <code>RoleArn</code> property as follows:  <code>{"RoleArn" : "arn:aws:iam::123456789012:role/LaunchRole"}</code>  You cannot have both a <code>LAUNCH</code> and a <code>STACKSET</code> constraint. You also cannot have more than one <code>LAUNCH</code> constraint on a product and portfolio. </dd> <dt>NOTIFICATION</dt> <dd> Specify the <code>NotificationArns</code> property as follows:  <code>{"NotificationArns" : ["arn:aws:sns:us-east-1:123456789012:Topic"]}</code>  </dd> <dt>RESOURCE_UPDATE</dt> <dd> Specify the <code>TagUpdatesOnProvisionedProduct</code> property as follows:  <code>{"Version":"2.0","Properties":{"TagUpdateOnProvisionedProduct":"String"}}</code>  The <code>TagUpdatesOnProvisionedProduct</code> property accepts a string value of <code>ALLOWED</code> or <code>NOT_ALLOWED</code>. </dd> <dt>STACKSET</dt> <dd> Specify the <code>Parameters</code> property as follows:  <code>{"Version": "String", "Properties": {"AccountList": [ "String" ], "RegionList": [ "String" ], "AdminRole": "String", "ExecutionRole": "String"}}</code>  You cannot have both a <code>LAUNCH</code> and a <code>STACKSET</code> constraint. You also cannot have more than one <code>STACKSET</code> constraint on a product and portfolio. Products with a <code>STACKSET</code> constraint will launch an AWS CloudFormation stack set. </dd> <dt>TEMPLATE</dt> <dd> Specify the <code>Rules</code> property. For more information, see <a href="http://docs.aws.amazon.com/servicecatalog/latest/adminguide/reference-template_constraint_rules.html">Template Constraint Rules</a>. </dd> </dl><br/>ProductId:  The product identifier.<br/>PortfolioId:  The portfolio identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: Status:  The status of the current request.<br/>ConstraintParameters:  The constraint parameters.<br/>ConstraintDetail:  Information about the constraint.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Owner</i> The owner of the constraint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the constraint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of constraint. <ul> <li>  <code>LAUNCH</code>  </li> <li>  <code>NOTIFICATION</code>  </li> <li> STACKSET </li> <li>  <code>TEMPLATE</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConstraintId</i> The identifier of the constraint.<br/>        """
        pass

    def create_portfolio(self, IdempotencyToken: str, ProviderName: str, DisplayName: str, AcceptLanguage: str=None, Description: str=None, Tags: List[Tag]=None) -> CreatePortfolioOutput:
        """Creates a portfolio.

        :param self:
        :param IdempotencyToken:  A unique identifier that you provide to ensure idempotency. If multiple requests differ only by the idempotency token, the same response is returned for each repeated request.<br/>Tags:  One or more tags.<br/>ProviderName:  The name of the portfolio provider.<br/>Description:  The description of the portfolio.<br/>DisplayName:  The name to use for display purposes.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: Tags:  Information about the tags associated with the portfolio.<br/>PortfolioDetail:  Information about the portfolio.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProviderName</i> The name of the portfolio provider.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The UTC time stamp of the creation time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the portfolio.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayName</i> The name to use for display purposes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ARN</i> The ARN assigned to the portfolio.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The portfolio identifier.<br/>        """
        pass

    def create_portfolio_share(self, PortfolioId: str, AcceptLanguage: str=None, AccountId: str=None, OrganizationNode: OrganizationNode=None) -> CreatePortfolioShareOutput:
        """Shares the specified portfolio with the specified account or organization node. Shares to an organization node can only be created by the master account of an Organization. AWSOrganizationsAccess must be enabled in order to create a portfolio share to an organization node.

        :param self:
        :param OrganizationNode:  The organization node to whom you are going to share. If <code>OrganizationNode</code> is passed in, <code>PortfolioShare</code> will be created for the node and its children (when applies), and a <code>PortfolioShareToken</code> will be returned in the output in order for the administrator to monitor the status of the <code>PortfolioShare</code> creation process.:<br/>AccountId:  The AWS account ID. For example, <code>123456789012</code>.<br/>PortfolioId:  The portfolio identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The identifier of the organization node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The organization node type.<br/>
        :return: PortfolioShareToken:  The portfolio share unique identifier. This will only be returned if portfolio is shared to an organization node.<br/>        """
        pass

    def create_product(self, IdempotencyToken: str, ProvisioningArtifactParameters: ProvisioningArtifactProperties, ProductType: str, Owner: str, Name: str, AcceptLanguage: str=None, Description: str=None, Distributor: str=None, SupportDescription: str=None, SupportEmail: str=None, SupportUrl: str=None, Tags: List[Tag]=None) -> CreateProductOutput:
        """Creates a product.

        :param self:
        :param IdempotencyToken:  A unique identifier that you provide to ensure idempotency. If multiple requests differ only by the idempotency token, the same response is returned for each repeated request.<br/>ProvisioningArtifactParameters:  The configuration of the provisioning artifact.:<br/>Tags:  One or more tags.<br/>ProductType:  The type of product.<br/>SupportUrl:  The contact URL for product support.<br/>SupportEmail:  The contact email for product support.<br/>SupportDescription:  The support information about the product.<br/>Distributor:  The distributor of the product.<br/>Description:  The description of the product.<br/>Owner:  The owner of the product.<br/>Name:  The name of the product.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisableTemplateValidation</i> If set to true, AWS Service Catalog stops validating the specified provisioning artifact even if it is invalid.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of provisioning artifact. <ul> <li>  <code>CLOUD_FORMATION_TEMPLATE</code> - AWS CloudFormation template </li> <li>  <code>MARKETPLACE_AMI</code> - AWS Marketplace AMI </li> <li>  <code>MARKETPLACE_CAR</code> - AWS Marketplace Clusters and AWS Resources </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Info</i> The URL of the CloudFormation template in Amazon S3. Specify the URL in JSON format as follows:  <code>"LoadTemplateFromURL": "https://s3.amazonaws.com/cf-templates-ozkq9d3hgiq2-us-east-1/..."</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the provisioning artifact, including how it differs from the previous provisioning artifact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the provisioning artifact (for example, v1 v2beta). No spaces are allowed.<br/>
        :return: Tags:  Information about the tags associated with the product.<br/>ProvisioningArtifactDetail:  Information about the provisioning artifact.:<br/>ProductViewDetail:  Information about the product view.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The UTC time stamp of the creation time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductARN</i> The ARN of the product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the product. <ul> <li>  <code>AVAILABLE</code> - The product is ready for use. </li> <li>  <code>CREATING</code> - Product creation has started; the product is not ready for use. </li> <li>  <code>FAILED</code> - An action failed. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductViewSummary</i> Summary information about the product view.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ProductViewSummary</b>: Summary information about the product view.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportUrl</i> The URL information to obtain support for this Product.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportDescription</i> The description of the support for this Product.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportEmail</i> The email contact information to obtain support for this Product.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HasDefaultPath</i> Indicates whether the product has a default path. If the product does not have a default path, call <a>ListLaunchPaths</a> to disambiguate between paths. Otherwise, <a>ListLaunchPaths</a> is not required, and the output of <a>ProductViewSummary</a> can be used directly with <a>DescribeProvisioningParameters</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Distributor</i> The distributor of the product. Contact the product administrator for the significance of this value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The product type. Contact the product administrator for the significance of this value. If this value is <code>MARKETPLACE</code>, the product was created by AWS Marketplace.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ShortDescription</i> Short description of the product.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Owner</i> The owner of the product. Contact the product administrator for the significance of this value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the product.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductId</i> The product identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The product view identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Guidance</i> Information set by the administrator to provide guidance to end users about which provisioning artifacts to use.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Active</i> Indicates whether the product version is active.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The UTC time stamp of the creation time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of provisioning artifact. <ul> <li>  <code>CLOUD_FORMATION_TEMPLATE</code> - AWS CloudFormation template </li> <li>  <code>MARKETPLACE_AMI</code> - AWS Marketplace AMI </li> <li>  <code>MARKETPLACE_CAR</code> - AWS Marketplace Clusters and AWS Resources </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the provisioning artifact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the provisioning artifact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier of the provisioning artifact.<br/>        """
        pass

    def create_provisioned_product_plan(self, IdempotencyToken: str, ProvisioningArtifactId: str, ProvisionedProductName: str, ProductId: str, PlanType: str, PlanName: str, AcceptLanguage: str=None, NotificationArns: List[NotificationArn]=None, PathId: str=None, ProvisioningParameters: List[UpdateProvisioningParameter]=None, Tags: List[Tag]=None) -> CreateProvisionedProductPlanOutput:
        """Creates a plan. A plan includes the list of resources to be created (when provisioning a new product) or modified (when updating a provisioned product) when the plan is executed. You can create one plan per provisioned product. To create a plan for an existing provisioned product, the product status must be AVAILBLE or TAINTED. To view the resource changes in the change set, use <a>DescribeProvisionedProductPlan</a>. To create or modify the provisioned product, use <a>ExecuteProvisionedProductPlan</a>.

        :param self:
        :param Tags:  One or more tags. If the plan is for an existing provisioned product, the product must have a <code>RESOURCE_UPDATE</code> constraint with <code>TagUpdatesOnProvisionedProduct</code> set to <code>ALLOWED</code> to allow tag updates.<br/>IdempotencyToken:  A unique identifier that you provide to ensure idempotency. If multiple requests differ only by the idempotency token, the same response is returned for each repeated request.<br/>ProvisioningParameters:  Parameters specified by the administrator that are required for provisioning the product.<br/>ProvisioningArtifactId:  The identifier of the provisioning artifact.<br/>ProvisionedProductName:  A user-friendly name for the provisioned product. This value must be unique for the AWS account and cannot be updated after the product is provisioned.<br/>ProductId:  The product identifier.<br/>PathId:  The path identifier of the product. This value is optional if the product has a default path, and required if the product has more than one path. To list the paths for a product, use <a>ListLaunchPaths</a>.<br/>NotificationArns:  Passed to CloudFormation. The SNS topic ARNs to which to publish stack-related events.<br/>PlanType:  The plan type.<br/>PlanName:  The name of the plan.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: ProvisioningArtifactId:  The identifier of the provisioning artifact.<br/>ProvisionedProductName:  The user-friendly name of the provisioned product.<br/>ProvisionProductId:  The product identifier.<br/>PlanId:  The plan identifier.<br/>PlanName:  The name of the plan.<br/>        """
        pass

    def create_provisioning_artifact(self, IdempotencyToken: str, Parameters: ProvisioningArtifactProperties, ProductId: str, AcceptLanguage: str=None) -> CreateProvisioningArtifactOutput:
        """Creates a provisioning artifact (also known as a version) for the specified product. You cannot create a provisioning artifact for a product that was shared with you.

        :param self:
        :param IdempotencyToken:  A unique identifier that you provide to ensure idempotency. If multiple requests differ only by the idempotency token, the same response is returned for each repeated request.<br/>Parameters:  The configuration for the provisioning artifact.:<br/>ProductId:  The product identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisableTemplateValidation</i> If set to true, AWS Service Catalog stops validating the specified provisioning artifact even if it is invalid.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of provisioning artifact. <ul> <li>  <code>CLOUD_FORMATION_TEMPLATE</code> - AWS CloudFormation template </li> <li>  <code>MARKETPLACE_AMI</code> - AWS Marketplace AMI </li> <li>  <code>MARKETPLACE_CAR</code> - AWS Marketplace Clusters and AWS Resources </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Info</i> The URL of the CloudFormation template in Amazon S3. Specify the URL in JSON format as follows:  <code>"LoadTemplateFromURL": "https://s3.amazonaws.com/cf-templates-ozkq9d3hgiq2-us-east-1/..."</code> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the provisioning artifact, including how it differs from the previous provisioning artifact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the provisioning artifact (for example, v1 v2beta). No spaces are allowed.<br/>
        :return: Status:  The status of the current request.<br/>Info:  The URL of the CloudFormation template in Amazon S3, in JSON format.<br/>ProvisioningArtifactDetail:  Information about the provisioning artifact.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Guidance</i> Information set by the administrator to provide guidance to end users about which provisioning artifacts to use.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Active</i> Indicates whether the product version is active.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The UTC time stamp of the creation time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of provisioning artifact. <ul> <li>  <code>CLOUD_FORMATION_TEMPLATE</code> - AWS CloudFormation template </li> <li>  <code>MARKETPLACE_AMI</code> - AWS Marketplace AMI </li> <li>  <code>MARKETPLACE_CAR</code> - AWS Marketplace Clusters and AWS Resources </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the provisioning artifact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the provisioning artifact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier of the provisioning artifact.<br/>        """
        pass

    def create_service_action(self, IdempotencyToken: str, Definition: ServiceActionDefinitionMap, DefinitionType: str, Name: str, Description: str=None, AcceptLanguage: str=None) -> CreateServiceActionOutput:
        """Creates a self-service action.

        :param self:
        :param IdempotencyToken:  A unique identifier that you provide to ensure idempotency. If multiple requests differ only by the idempotency token, the same response is returned for each repeated request.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>Description:  The self-service action description.<br/>Definition:  The self-service action definition. Can be one of the following: <dl> <dt>Name</dt> <dd> The name of the AWS Systems Manager Document. For example, <code>AWS-RestartEC2Instance</code>. </dd> <dt>Version</dt> <dd> The AWS Systems Manager automation document version. For example, <code>"Version": "1"</code>  </dd> <dt>AssumeRole</dt> <dd> The Amazon Resource Name (ARN) of the role that performs the self-service actions on your behalf. For example, <code>"AssumeRole": "arn:aws:iam::12345678910:role/ActionRole"</code>. To reuse the provisioned product launch role, set to <code>"AssumeRole": "LAUNCH_ROLE"</code>. </dd> <dt>Parameters</dt> <dd> The list of parameters in JSON format. For example: <code>[{\"Name\":\"InstanceId\",\"Type\":\"TARGET\"}]</code>. </dd> </dl><br/>DefinitionType:  The service action definition type. For example, <code>SSM_AUTOMATION</code>.<br/>Name:  The self-service action name.<br/>
        :return: ServiceActionDetail:  An object containing information about the self-service action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Definition</i> A map that defines the self-service action.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceActionSummary</i> Summary information about the self-service action.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ServiceActionSummary</b>: Summary information about the self-service action.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DefinitionType</i> The self-service action definition type. For example, <code>SSM_AUTOMATION</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The self-service action description.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The self-service action name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The self-service action identifier.<br/>        """
        pass

    def create_tag_option(self, Value: str, Key: str) -> CreateTagOptionOutput:
        """Creates a TagOption.

        :param self:
        :param Value:  The TagOption value.<br/>Key:  The TagOption key.<br/>
        :return: TagOptionDetail:  Information about the TagOption.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The TagOption identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Active</i> The TagOption active state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The TagOption value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> The TagOption key.<br/>        """
        pass

    def delete_constraint(self, Id: str, AcceptLanguage: str=None) -> DeleteConstraintOutput:
        """Deletes the specified constraint.

        :param self:
        :param Id:  The identifier of the constraint.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return:         """
        pass

    def delete_portfolio(self, Id: str, AcceptLanguage: str=None) -> DeletePortfolioOutput:
        """Deletes the specified portfolio. You cannot delete a portfolio if it was shared with you or if it has associated products, users, constraints, or shared accounts.

        :param self:
        :param Id:  The portfolio identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return:         """
        pass

    def delete_portfolio_share(self, PortfolioId: str, AcceptLanguage: str=None, AccountId: str=None, OrganizationNode: OrganizationNode=None) -> DeletePortfolioShareOutput:
        """Stops sharing the specified portfolio with the specified account or organization node. Shares to an organization node can only be deleted by the master account of an Organization.

        :param self:
        :param OrganizationNode:  The organization node to whom you are going to stop sharing.:<br/>AccountId:  The AWS account ID.<br/>PortfolioId:  The portfolio identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The identifier of the organization node.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The organization node type.<br/>
        :return: PortfolioShareToken:  The portfolio share unique identifier. This will only be returned if delete is made to an organization node.<br/>        """
        pass

    def delete_product(self, Id: str, AcceptLanguage: str=None) -> DeleteProductOutput:
        """Deletes the specified product. You cannot delete a product if it was shared with you or is associated with a portfolio.

        :param self:
        :param Id:  The product identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return:         """
        pass

    def delete_provisioned_product_plan(self, PlanId: str, AcceptLanguage: str=None, IgnoreErrors: IgnoreErrors=None) -> DeleteProvisionedProductPlanOutput:
        """Deletes the specified plan.

        :param self:
        :param IgnoreErrors:  If set to true, AWS Service Catalog stops managing the specified provisioned product even if it cannot delete the underlying resources.<br/>PlanId:  The plan identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return:         """
        pass

    def delete_provisioning_artifact(self, ProvisioningArtifactId: str, ProductId: str, AcceptLanguage: str=None) -> DeleteProvisioningArtifactOutput:
        """Deletes the specified provisioning artifact (also known as a version) for the specified product. You cannot delete a provisioning artifact associated with a product that was shared with you. You cannot delete the last provisioning artifact for a product, because a product must have at least one provisioning artifact.

        :param self:
        :param ProvisioningArtifactId:  The identifier of the provisioning artifact.<br/>ProductId:  The product identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return:         """
        pass

    def delete_service_action(self, Id: str, AcceptLanguage: str=None) -> DeleteServiceActionOutput:
        """Deletes a self-service action.

        :param self:
        :param AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>Id:  The self-service action identifier. For example, <code>act-fs7abcd89wxyz</code>.<br/>
        :return:         """
        pass

    def delete_tag_option(self, Id: str) -> DeleteTagOptionOutput:
        """Deletes the specified TagOption. You cannot delete a TagOption if it is associated with a product or portfolio.

        :param self:
        :param Id:  The TagOption identifier.<br/>
        :return:         """
        pass

    def describe_constraint(self, Id: str, AcceptLanguage: str=None) -> DescribeConstraintOutput:
        """Gets information about the specified constraint.

        :param self:
        :param Id:  The identifier of the constraint.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: Status:  The status of the current request.<br/>ConstraintParameters:  The constraint parameters.<br/>ConstraintDetail:  Information about the constraint.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Owner</i> The owner of the constraint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the constraint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of constraint. <ul> <li>  <code>LAUNCH</code>  </li> <li>  <code>NOTIFICATION</code>  </li> <li> STACKSET </li> <li>  <code>TEMPLATE</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConstraintId</i> The identifier of the constraint.<br/>        """
        pass

    def describe_copy_product_status(self, CopyProductToken: str, AcceptLanguage: str=None) -> DescribeCopyProductStatusOutput:
        """Gets the status of the specified copy product operation.

        :param self:
        :param CopyProductToken:  The token for the copy product operation. This token is returned by <a>CopyProduct</a>.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: StatusDetail:  The status message.<br/>TargetProductId:  The identifier of the copied product.<br/>CopyProductStatus:  The status of the copy product operation.<br/>        """
        pass

    def describe_portfolio(self, Id: str, AcceptLanguage: str=None) -> DescribePortfolioOutput:
        """Gets information about the specified portfolio.

        :param self:
        :param Id:  The portfolio identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: Budgets:  Information about the associated budgets.<br/>TagOptions:  Information about the TagOptions associated with the portfolio.<br/>Tags:  Information about the tags associated with the portfolio.<br/>PortfolioDetail:  Information about the portfolio.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProviderName</i> The name of the portfolio provider.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The UTC time stamp of the creation time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the portfolio.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayName</i> The name to use for display purposes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ARN</i> The ARN assigned to the portfolio.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The portfolio identifier.<br/>        """
        pass

    def describe_portfolio_share_status(self, PortfolioShareToken: str) -> DescribePortfolioShareStatusOutput:
        """Gets the status of the specified portfolio share operation. This API can only be called by the master account in the organization.

        :param self:
        :param PortfolioShareToken:  The token for the portfolio share operation. This token is returned either by CreatePortfolioShare or by DeletePortfolioShare.<br/>
        :return: ShareDetails:  Information about the portfolio share operation.:<br/>Status:  Status of the portfolio share operation.<br/>OrganizationNodeValue:  Organization node identifier. It can be either account id, organizational unit id or organization id.<br/>PortfolioId:  The portfolio identifier.<br/>PortfolioShareToken:  The token for the portfolio share operation. For example, <code>share-6v24abcdefghi</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ShareErrors</i> List of errors.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SuccessfulShares</i> List of accounts for whom the operation succeeded.<br/>        """
        pass

    def describe_product(self, Id: str, AcceptLanguage: str=None) -> DescribeProductOutput:
        """Gets information about the specified product.

        :param self:
        :param Id:  The product identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: Budgets:  Information about the associated budgets.<br/>ProvisioningArtifacts:  Information about the provisioning artifacts for the specified product.<br/>ProductViewSummary:  Summary information about the product view.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportUrl</i> The URL information to obtain support for this Product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportDescription</i> The description of the support for this Product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportEmail</i> The email contact information to obtain support for this Product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasDefaultPath</i> Indicates whether the product has a default path. If the product does not have a default path, call <a>ListLaunchPaths</a> to disambiguate between paths. Otherwise, <a>ListLaunchPaths</a> is not required, and the output of <a>ProductViewSummary</a> can be used directly with <a>DescribeProvisioningParameters</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Distributor</i> The distributor of the product. Contact the product administrator for the significance of this value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The product type. Contact the product administrator for the significance of this value. If this value is <code>MARKETPLACE</code>, the product was created by AWS Marketplace.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ShortDescription</i> Short description of the product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Owner</i> The owner of the product. Contact the product administrator for the significance of this value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductId</i> The product identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The product view identifier.<br/>        """
        pass

    def describe_product_as_admin(self, Id: str, AcceptLanguage: str=None) -> DescribeProductAsAdminOutput:
        """Gets information about the specified product. This operation is run with administrator access.

        :param self:
        :param Id:  The product identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: Budgets:  Information about the associated budgets.<br/>TagOptions:  Information about the TagOptions associated with the product.<br/>Tags:  Information about the tags associated with the product.<br/>ProvisioningArtifactSummaries:  Information about the provisioning artifacts (also known as versions) for the specified product.<br/>ProductViewDetail:  Information about the product view.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The UTC time stamp of the creation time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductARN</i> The ARN of the product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the product. <ul> <li>  <code>AVAILABLE</code> - The product is ready for use. </li> <li>  <code>CREATING</code> - Product creation has started; the product is not ready for use. </li> <li>  <code>FAILED</code> - An action failed. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductViewSummary</i> Summary information about the product view.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ProductViewSummary</b>: Summary information about the product view.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportUrl</i> The URL information to obtain support for this Product.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportDescription</i> The description of the support for this Product.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportEmail</i> The email contact information to obtain support for this Product.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HasDefaultPath</i> Indicates whether the product has a default path. If the product does not have a default path, call <a>ListLaunchPaths</a> to disambiguate between paths. Otherwise, <a>ListLaunchPaths</a> is not required, and the output of <a>ProductViewSummary</a> can be used directly with <a>DescribeProvisioningParameters</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Distributor</i> The distributor of the product. Contact the product administrator for the significance of this value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The product type. Contact the product administrator for the significance of this value. If this value is <code>MARKETPLACE</code>, the product was created by AWS Marketplace.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ShortDescription</i> Short description of the product.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Owner</i> The owner of the product. Contact the product administrator for the significance of this value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the product.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductId</i> The product identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The product view identifier.<br/>        """
        pass

    def describe_product_view(self, Id: str, AcceptLanguage: str=None) -> DescribeProductViewOutput:
        """Gets information about the specified product.

        :param self:
        :param Id:  The product view identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: ProvisioningArtifacts:  Information about the provisioning artifacts for the product.<br/>ProductViewSummary:  Summary information about the product.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportUrl</i> The URL information to obtain support for this Product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportDescription</i> The description of the support for this Product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportEmail</i> The email contact information to obtain support for this Product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HasDefaultPath</i> Indicates whether the product has a default path. If the product does not have a default path, call <a>ListLaunchPaths</a> to disambiguate between paths. Otherwise, <a>ListLaunchPaths</a> is not required, and the output of <a>ProductViewSummary</a> can be used directly with <a>DescribeProvisioningParameters</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Distributor</i> The distributor of the product. Contact the product administrator for the significance of this value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The product type. Contact the product administrator for the significance of this value. If this value is <code>MARKETPLACE</code>, the product was created by AWS Marketplace.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ShortDescription</i> Short description of the product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Owner</i> The owner of the product. Contact the product administrator for the significance of this value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductId</i> The product identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The product view identifier.<br/>        """
        pass

    def describe_provisioned_product(self, Id: str, AcceptLanguage: str=None) -> DescribeProvisionedProductOutput:
        """Gets information about the specified provisioned product.

        :param self:
        :param Id:  The provisioned product identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: CloudWatchDashboards:  Any CloudWatch dashboards that were created when provisioning the product.<br/>ProvisionedProductDetail:  Information about the provisioned product.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisioningArtifactId</i> The identifier of the provisioning artifact. For example, <code>pa-4abcdjnxjj6ne</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductId</i> The product identifier. For example, <code>prod-abcdzk7xy33qa</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastRecordId</i> The record identifier of the last request performed on this provisioned product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IdempotencyToken</i> A unique identifier that you provide to ensure idempotency. If multiple requests differ only by the idempotency token, the same response is returned for each repeated request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The UTC time stamp of the creation time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> The current status message of the provisioned product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The current status of the provisioned product. <ul> <li>  <code>AVAILABLE</code> - Stable state, ready to perform any operation. The most recent operation succeeded and completed. </li> <li>  <code>UNDER_CHANGE</code> - Transitive state. Operations performed might not have valid results. Wait for an <code>AVAILABLE</code> status before performing operations. </li> <li>  <code>TAINTED</code> - Stable state, ready to perform any operation. The stack has completed the requested operation but is not exactly what was requested. For example, a request to update to a new version failed and the stack rolled back to the current version. </li> <li>  <code>ERROR</code> - An unexpected error occurred. The provisioned product exists but the stack is not running. For example, CloudFormation received a parameter value that was not valid and could not launch the stack. </li> <li>  <code>PLAN_IN_PROGRESS</code> - Transitive state. The plan operations were performed to provision a new product, but resources have not yet been created. After reviewing the list of resources to be created, execute the plan. Wait for an <code>AVAILABLE</code> status before performing operations. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier of the provisioned product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of provisioned product. The supported values are <code>CFN_STACK</code> and <code>CFN_STACKSET</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN of the provisioned product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The user-friendly name of the provisioned product.<br/>        """
        pass

    def describe_provisioned_product_plan(self, PlanId: str, AcceptLanguage: str=None, PageSize: int=None, PageToken: str=None) -> DescribeProvisionedProductPlanOutput:
        """Gets information about the resource changes for the specified plan.

        :param self:
        :param PageToken:  The page token for the next set of results. To retrieve the first set of results, use null.<br/>PageSize:  The maximum number of items to return with this call.<br/>PlanId:  The plan identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: NextPageToken:  The page token to use to retrieve the next set of results. If there are no additional results, this value is null.<br/>ResourceChanges:  Information about the resource changes that will occur when the plan is executed.<br/>ProvisionedProductPlanDetails:  Information about the plan.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusMessage</i> The status message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> One or more tags.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisioningParameters</i> Parameters specified by the administrator that are required for provisioning the product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationArns</i> Passed to CloudFormation. The SNS topic ARNs to which to publish stack-related events.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdatedTime</i> The time when the plan was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisioningArtifactId</i> The identifier of the provisioning artifact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlanType</i> The plan type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisionProductName</i> The user-friendly name of the provisioned product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisionProductId</i> The product identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlanId</i> The plan identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlanName</i> The name of the plan.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductId</i> The product identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PathId</i> The path identifier of the product. This value is optional if the product has a default path, and required if the product has more than one path. To list the paths for a product, use <a>ListLaunchPaths</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The UTC time stamp of the creation time.<br/>        """
        pass

    def describe_provisioning_artifact(self, ProductId: str, ProvisioningArtifactId: str, AcceptLanguage: str=None, Verbose: Verbose=None) -> DescribeProvisioningArtifactOutput:
        """Gets information about the specified provisioning artifact (also known as a version) for the specified product.

        :param self:
        :param Verbose:  Indicates whether a verbose level of detail is enabled.<br/>ProductId:  The product identifier.<br/>ProvisioningArtifactId:  The identifier of the provisioning artifact.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: Status:  The status of the current request.<br/>Info:  The URL of the CloudFormation template in Amazon S3.<br/>ProvisioningArtifactDetail:  Information about the provisioning artifact.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Guidance</i> Information set by the administrator to provide guidance to end users about which provisioning artifacts to use.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Active</i> Indicates whether the product version is active.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The UTC time stamp of the creation time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of provisioning artifact. <ul> <li>  <code>CLOUD_FORMATION_TEMPLATE</code> - AWS CloudFormation template </li> <li>  <code>MARKETPLACE_AMI</code> - AWS Marketplace AMI </li> <li>  <code>MARKETPLACE_CAR</code> - AWS Marketplace Clusters and AWS Resources </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the provisioning artifact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the provisioning artifact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier of the provisioning artifact.<br/>        """
        pass

    def describe_provisioning_parameters(self, ProvisioningArtifactId: str, ProductId: str, AcceptLanguage: str=None, PathId: str=None) -> DescribeProvisioningParametersOutput:
        """Gets information about the configuration required to provision the specified product using the specified provisioning artifact. If the output contains a TagOption key with an empty list of values, there is a TagOption conflict for that key. The end user cannot take action to fix the conflict, and launch is not blocked. In subsequent calls to <a>ProvisionProduct</a>, do not include conflicted TagOption keys as tags, or this causes the error "Parameter validation failed: Missing required parameter in Tags[<i>N</i>]:<i>Value</i>". Tag the provisioned product with the value <code>sc-tagoption-conflict-portfolioId-productId</code>.

        :param self:
        :param PathId:  The path identifier of the product. This value is optional if the product has a default path, and required if the product has more than one path. To list the paths for a product, use <a>ListLaunchPaths</a>.<br/>ProvisioningArtifactId:  The identifier of the provisioning artifact.<br/>ProductId:  The product identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: ProvisioningArtifactPreferences:  An object that contains information about preferences, such as regions and accounts, for the provisioning artifact.:<br/>TagOptions:  Information about the TagOptions associated with the resource.<br/>UsageInstructions:  Any additional metadata specifically related to the provisioning of the product. For example, see the <code>Version</code> field of the CloudFormation template.<br/>ConstraintSummaries:  Information about the constraints used to provision the product.<br/>ProvisioningArtifactParameters:  Information about the parameters used to provision the product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StackSetRegions</i> One or more AWS Regions where stack instances are deployed from the stack set. These regions can be scoped in <code>ProvisioningPreferences$StackSetRegions</code> and <code>UpdateProvisioningPreferences$StackSetRegions</code>. Applicable only to a <code>CFN_STACKSET</code> provisioned product type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StackSetAccounts</i> One or more AWS accounts where stack instances are deployed from the stack set. These accounts can be scoped in <code>ProvisioningPreferences$StackSetAccounts</code> and <code>UpdateProvisioningPreferences$StackSetAccounts</code>. Applicable only to a <code>CFN_STACKSET</code> provisioned product type.<br/>        """
        pass

    def describe_record(self, Id: str, AcceptLanguage: str=None, PageToken: str=None, PageSize: int=None) -> DescribeRecordOutput:
        """Gets information about the specified request operation. Use this operation after calling a request operation (for example, <a>ProvisionProduct</a>, <a>TerminateProvisionedProduct</a>, or <a>UpdateProvisionedProduct</a>).  <note> If a provisioned product was transferred to a new owner using <a>UpdateProvisionedProductProperties</a>, the new owner will be able to describe all past records for that product. The previous owner will no longer be able to describe the records, but will be able to use <a>ListRecordHistory</a> to see the product's history from when he was the owner. </note>

        :param self:
        :param PageSize:  The maximum number of items to return with this call.<br/>PageToken:  The page token for the next set of results. To retrieve the first set of results, use null.<br/>Id:  The record identifier of the provisioned product. This identifier is returned by the request operation.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: NextPageToken:  The page token to use to retrieve the next set of results. If there are no additional results, this value is null.<br/>RecordOutputs:  Information about the product created as the result of a request. For example, the output for a CloudFormation-backed product that creates an S3 bucket would include the S3 bucket URL.<br/>RecordDetail:  Information about the product.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordTags</i> One or more tags.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordErrors</i> The errors that occurred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PathId</i> The path identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisioningArtifactId</i> The identifier of the provisioning artifact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductId</i> The product identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisionedProductId</i> The identifier of the provisioned product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordType</i> The record type. <ul> <li>  <code>PROVISION_PRODUCT</code>  </li> <li>  <code>UPDATE_PROVISIONED_PRODUCT</code>  </li> <li>  <code>TERMINATE_PROVISIONED_PRODUCT</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisionedProductType</i> The type of provisioned product. The supported values are <code>CFN_STACK</code> and <code>CFN_STACKSET</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdatedTime</i> The time when the record was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The UTC time stamp of the creation time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the provisioned product. <ul> <li>  <code>CREATED</code> - The request was created but the operation has not started. </li> <li>  <code>IN_PROGRESS</code> - The requested operation is in progress. </li> <li>  <code>IN_PROGRESS_IN_ERROR</code> - The provisioned product is under change but the requested operation failed and some remediation is occurring. For example, a rollback. </li> <li>  <code>SUCCEEDED</code> - The requested operation has successfully completed. </li> <li>  <code>FAILED</code> - The requested operation has unsuccessfully completed. Investigate using the error messages returned. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisionedProductName</i> The user-friendly name of the provisioned product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordId</i> The identifier of the record.<br/>        """
        pass

    def describe_service_action(self, Id: str, AcceptLanguage: str=None) -> DescribeServiceActionOutput:
        """Describes a self-service action.

        :param self:
        :param AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>Id:  The self-service action identifier.<br/>
        :return: ServiceActionDetail:  Detailed information about the self-service action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Definition</i> A map that defines the self-service action.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceActionSummary</i> Summary information about the self-service action.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ServiceActionSummary</b>: Summary information about the self-service action.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DefinitionType</i> The self-service action definition type. For example, <code>SSM_AUTOMATION</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The self-service action description.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The self-service action name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The self-service action identifier.<br/>        """
        pass

    def describe_service_action_execution_parameters(self, ServiceActionId: str, ProvisionedProductId: str, AcceptLanguage: str=None) -> DescribeServiceActionExecutionParametersOutput:
        """

        :param self:
        :param AcceptLanguage:  <br/>ServiceActionId:  <br/>ProvisionedProductId:  <br/>
        :return: ServiceActionParameters:  <br/>        """
        pass

    def describe_tag_option(self, Id: str) -> DescribeTagOptionOutput:
        """Gets information about the specified TagOption.

        :param self:
        :param Id:  The TagOption identifier.<br/>
        :return: TagOptionDetail:  Information about the TagOption.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The TagOption identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Active</i> The TagOption active state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The TagOption value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> The TagOption key.<br/>        """
        pass

    def disable_aws_organizations_access(self) -> DisableAWSOrganizationsAccessOutput:
        """Disable portfolio sharing through AWS Organizations feature. This feature will not delete your current shares but it will prevent you from creating new shares throughout your organization. Current shares will not be in sync with your organization structure if it changes after calling this API. This API can only be called by the master account in the organization.

        :param self:
        :param 
        :return:         """
        pass

    def disassociate_budget_from_resource(self, ResourceId: str, BudgetName: str) -> DisassociateBudgetFromResourceOutput:
        """Disassociates the specified budget from the specified resource.

        :param self:
        :param ResourceId:  The resource identifier you want to disassociate from. Either a portfolio-id or a product-id.<br/>BudgetName:  The name of the budget you want to disassociate.<br/>
        :return:         """
        pass

    def disassociate_principal_from_portfolio(self, PrincipalARN: str, PortfolioId: str, AcceptLanguage: str=None) -> DisassociatePrincipalFromPortfolioOutput:
        """Disassociates a previously associated principal ARN from a specified portfolio.

        :param self:
        :param PrincipalARN:  The ARN of the principal (IAM user, role, or group).<br/>PortfolioId:  The portfolio identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return:         """
        pass

    def disassociate_product_from_portfolio(self, PortfolioId: str, ProductId: str, AcceptLanguage: str=None) -> DisassociateProductFromPortfolioOutput:
        """Disassociates the specified product from the specified portfolio. 

        :param self:
        :param PortfolioId:  The portfolio identifier.<br/>ProductId:  The product identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return:         """
        pass

    def disassociate_service_action_from_provisioning_artifact(self, ServiceActionId: str, ProvisioningArtifactId: str, ProductId: str, AcceptLanguage: str=None) -> DisassociateServiceActionFromProvisioningArtifactOutput:
        """Disassociates the specified self-service action association from the specified provisioning artifact.

        :param self:
        :param AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>ServiceActionId:  The self-service action identifier. For example, <code>act-fs7abcd89wxyz</code>.<br/>ProvisioningArtifactId:  The identifier of the provisioning artifact. For example, <code>pa-4abcdjnxjj6ne</code>.<br/>ProductId:  The product identifier. For example, <code>prod-abcdzk7xy33qa</code>.<br/>
        :return:         """
        pass

    def disassociate_tag_option_from_resource(self, TagOptionId: str, ResourceId: str) -> DisassociateTagOptionFromResourceOutput:
        """Disassociates the specified TagOption from the specified resource.

        :param self:
        :param TagOptionId:  The TagOption identifier.<br/>ResourceId:  The resource identifier.<br/>
        :return:         """
        pass

    def enable_aws_organizations_access(self) -> EnableAWSOrganizationsAccessOutput:
        """Enable portfolio sharing feature through AWS Organizations. This API will allow Service Catalog to receive updates on your organization in order to sync your shares with the current structure. This API can only be called by the master account in the organization. By calling this API Service Catalog will make a call to organizations:EnableAWSServiceAccess on your behalf so that your shares can be in sync with any changes in your AWS Organizations structure.

        :param self:
        :param 
        :return:         """
        pass

    def execute_provisioned_product_plan(self, IdempotencyToken: str, PlanId: str, AcceptLanguage: str=None) -> ExecuteProvisionedProductPlanOutput:
        """Provisions or modifies a product based on the resource changes for the specified plan.

        :param self:
        :param IdempotencyToken:  A unique identifier that you provide to ensure idempotency. If multiple requests differ only by the idempotency token, the same response is returned for each repeated request.<br/>PlanId:  The plan identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: RecordDetail:  Information about the result of provisioning the product.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordTags</i> One or more tags.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordErrors</i> The errors that occurred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PathId</i> The path identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisioningArtifactId</i> The identifier of the provisioning artifact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductId</i> The product identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisionedProductId</i> The identifier of the provisioned product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordType</i> The record type. <ul> <li>  <code>PROVISION_PRODUCT</code>  </li> <li>  <code>UPDATE_PROVISIONED_PRODUCT</code>  </li> <li>  <code>TERMINATE_PROVISIONED_PRODUCT</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisionedProductType</i> The type of provisioned product. The supported values are <code>CFN_STACK</code> and <code>CFN_STACKSET</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdatedTime</i> The time when the record was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The UTC time stamp of the creation time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the provisioned product. <ul> <li>  <code>CREATED</code> - The request was created but the operation has not started. </li> <li>  <code>IN_PROGRESS</code> - The requested operation is in progress. </li> <li>  <code>IN_PROGRESS_IN_ERROR</code> - The provisioned product is under change but the requested operation failed and some remediation is occurring. For example, a rollback. </li> <li>  <code>SUCCEEDED</code> - The requested operation has successfully completed. </li> <li>  <code>FAILED</code> - The requested operation has unsuccessfully completed. Investigate using the error messages returned. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisionedProductName</i> The user-friendly name of the provisioned product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordId</i> The identifier of the record.<br/>        """
        pass

    def execute_provisioned_product_service_action(self, ExecuteToken: str, ServiceActionId: str, ProvisionedProductId: str, AcceptLanguage: str=None, Parameters: ExecutionParameterMap=None) -> ExecuteProvisionedProductServiceActionOutput:
        """Executes a self-service action against a provisioned product.

        :param self:
        :param Parameters:  <br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>ExecuteToken:  An idempotency token that uniquely identifies the execute request.<br/>ServiceActionId:  The self-service action identifier. For example, <code>act-fs7abcd89wxyz</code>.<br/>ProvisionedProductId:  The identifier of the provisioned product.<br/>
        :return: RecordDetail:  An object containing detailed information about the result of provisioning the product.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordTags</i> One or more tags.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordErrors</i> The errors that occurred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PathId</i> The path identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisioningArtifactId</i> The identifier of the provisioning artifact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductId</i> The product identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisionedProductId</i> The identifier of the provisioned product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordType</i> The record type. <ul> <li>  <code>PROVISION_PRODUCT</code>  </li> <li>  <code>UPDATE_PROVISIONED_PRODUCT</code>  </li> <li>  <code>TERMINATE_PROVISIONED_PRODUCT</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisionedProductType</i> The type of provisioned product. The supported values are <code>CFN_STACK</code> and <code>CFN_STACKSET</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdatedTime</i> The time when the record was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The UTC time stamp of the creation time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the provisioned product. <ul> <li>  <code>CREATED</code> - The request was created but the operation has not started. </li> <li>  <code>IN_PROGRESS</code> - The requested operation is in progress. </li> <li>  <code>IN_PROGRESS_IN_ERROR</code> - The provisioned product is under change but the requested operation failed and some remediation is occurring. For example, a rollback. </li> <li>  <code>SUCCEEDED</code> - The requested operation has successfully completed. </li> <li>  <code>FAILED</code> - The requested operation has unsuccessfully completed. Investigate using the error messages returned. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisionedProductName</i> The user-friendly name of the provisioned product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordId</i> The identifier of the record.<br/>        """
        pass

    def get_aws_organizations_access_status(self) -> GetAWSOrganizationsAccessStatusOutput:
        """Get the Access Status for AWS Organization portfolio share feature. This API can only be called by the master account in the organization.

        :param self:
        :param 
        :return: AccessStatus:  The status of the portfolio share feature.<br/>        """
        pass

    def list_accepted_portfolio_shares(self, AcceptLanguage: str=None, PageToken: str=None, PageSize: int=None, PortfolioShareType: str=None) -> ListAcceptedPortfolioSharesOutput:
        """Lists all portfolios for which sharing was accepted by this account.

        :param self:
        :param PortfolioShareType:  The type of shared portfolios to list. The default is to list imported portfolios. <ul> <li>  <code>AWS_ORGANIZATIONS</code> - List portfolios shared by the master account of your organization </li> <li>  <code>AWS_SERVICECATALOG</code> - List default portfolios </li> <li>  <code>IMPORTED</code> - List imported portfolios </li> </ul><br/>PageSize:  The maximum number of items to return with this call.<br/>PageToken:  The page token for the next set of results. To retrieve the first set of results, use null.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: NextPageToken:  The page token to use to retrieve the next set of results. If there are no additional results, this value is null.<br/>PortfolioDetails:  Information about the portfolios.<br/>        """
        pass

    def list_budgets_for_resource(self, ResourceId: str, AcceptLanguage: str=None, PageSize: int=None, PageToken: str=None) -> ListBudgetsForResourceOutput:
        """Lists all the budgets associated to the specified resource.

        :param self:
        :param PageToken:  The page token for the next set of results. To retrieve the first set of results, use null.<br/>PageSize:  The maximum number of items to return with this call.<br/>ResourceId:  The resource identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: NextPageToken:  The page token to use to retrieve the next set of results. If there are no additional results, this value is null.<br/>Budgets:  Information about the associated budgets.<br/>        """
        pass

    def list_constraints_for_portfolio(self, PortfolioId: str, AcceptLanguage: str=None, ProductId: str=None, PageSize: int=None, PageToken: str=None) -> ListConstraintsForPortfolioOutput:
        """Lists the constraints for the specified portfolio and product.

        :param self:
        :param PageToken:  The page token for the next set of results. To retrieve the first set of results, use null.<br/>PageSize:  The maximum number of items to return with this call.<br/>ProductId:  The product identifier.<br/>PortfolioId:  The portfolio identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: NextPageToken:  The page token to use to retrieve the next set of results. If there are no additional results, this value is null.<br/>ConstraintDetails:  Information about the constraints.<br/>        """
        pass

    def list_launch_paths(self, ProductId: str, AcceptLanguage: str=None, PageSize: int=None, PageToken: str=None) -> ListLaunchPathsOutput:
        """Lists the paths to the specified product. A path is how the user has access to a specified product, and is necessary when provisioning a product. A path also determines the constraints put on the product.

        :param self:
        :param PageToken:  The page token for the next set of results. To retrieve the first set of results, use null.<br/>PageSize:  The maximum number of items to return with this call.<br/>ProductId:  The product identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: NextPageToken:  The page token to use to retrieve the next set of results. If there are no additional results, this value is null.<br/>LaunchPathSummaries:  Information about the launch path.<br/>        """
        pass

    def list_organization_portfolio_access(self, OrganizationNodeType: str, PortfolioId: str, AcceptLanguage: str=None, PageToken: str=None, PageSize: int=None) -> ListOrganizationPortfolioAccessOutput:
        """Lists the organization nodes that have access to the specified portfolio. This API can only be called by the master account in the organization.

        :param self:
        :param PageSize:  The maximum number of items to return with this call.<br/>PageToken:  The page token for the next set of results. To retrieve the first set of results, use null.<br/>OrganizationNodeType:  The organization node type that will be returned in the output. <ul> <li>  <code>ORGANIZATION</code> - Organization that has access to the portfolio.  </li> <li>  <code>ORGANIZATIONAL_UNIT</code> - Organizational unit that has access to the portfolio within your organization. </li> <li>  <code>ACCOUNT</code> - Account that has access to the portfolio within your organization. </li> </ul><br/>PortfolioId:  The portfolio identifier. For example, <code>port-2abcdext3y5fk</code>.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: NextPageToken:  The page token to use to retrieve the next set of results. If there are no additional results, this value is null.<br/>OrganizationNodes:  Displays information about the organization nodes.<br/>        """
        pass

    def list_portfolio_access(self, PortfolioId: str, AcceptLanguage: str=None) -> ListPortfolioAccessOutput:
        """Lists the account IDs that have access to the specified portfolio.

        :param self:
        :param PortfolioId:  The portfolio identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: NextPageToken:  The page token to use to retrieve the next set of results. If there are no additional results, this value is null.<br/>AccountIds:  Information about the AWS accounts with access to the portfolio.<br/>        """
        pass

    def list_portfolios(self, AcceptLanguage: str=None, PageToken: str=None, PageSize: int=None) -> ListPortfoliosOutput:
        """Lists all portfolios in the catalog.

        :param self:
        :param PageSize:  The maximum number of items to return with this call.<br/>PageToken:  The page token for the next set of results. To retrieve the first set of results, use null.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: NextPageToken:  The page token to use to retrieve the next set of results. If there are no additional results, this value is null.<br/>PortfolioDetails:  Information about the portfolios.<br/>        """
        pass

    def list_portfolios_for_product(self, ProductId: str, AcceptLanguage: str=None, PageToken: str=None, PageSize: int=None) -> ListPortfoliosForProductOutput:
        """Lists all portfolios that the specified product is associated with.

        :param self:
        :param PageSize:  The maximum number of items to return with this call.<br/>PageToken:  The page token for the next set of results. To retrieve the first set of results, use null.<br/>ProductId:  The product identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: NextPageToken:  The page token to use to retrieve the next set of results. If there are no additional results, this value is null.<br/>PortfolioDetails:  Information about the portfolios.<br/>        """
        pass

    def list_principals_for_portfolio(self, PortfolioId: str, AcceptLanguage: str=None, PageSize: int=None, PageToken: str=None) -> ListPrincipalsForPortfolioOutput:
        """Lists all principal ARNs associated with the specified portfolio.

        :param self:
        :param PageToken:  The page token for the next set of results. To retrieve the first set of results, use null.<br/>PageSize:  The maximum number of items to return with this call.<br/>PortfolioId:  The portfolio identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: NextPageToken:  The page token to use to retrieve the next set of results. If there are no additional results, this value is null.<br/>Principals:  The IAM principals (users or roles) associated with the portfolio.<br/>        """
        pass

    def list_provisioned_product_plans(self, AcceptLanguage: str=None, ProvisionProductId: str=None, PageSize: int=None, PageToken: str=None, AccessLevelFilter: AccessLevelFilter=None) -> ListProvisionedProductPlansOutput:
        """Lists the plans for the specified provisioned product or all plans to which the user has access.

        :param self:
        :param AccessLevelFilter:  The access level to use to obtain results. The default is <code>User</code>.:<br/>PageToken:  The page token for the next set of results. To retrieve the first set of results, use null.<br/>PageSize:  The maximum number of items to return with this call.<br/>ProvisionProductId:  The product identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The user to which the access level applies. The only supported value is <code>Self</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> The access level. <ul> <li>  <code>Account</code> - Filter results based on the account. </li> <li>  <code>Role</code> - Filter results based on the federated role of the specified user. </li> <li>  <code>User</code> - Filter results based on the specified user. </li> </ul><br/>
        :return: NextPageToken:  The page token to use to retrieve the next set of results. If there are no additional results, this value is null.<br/>ProvisionedProductPlans:  Information about the plans.<br/>        """
        pass

    def list_provisioning_artifacts(self, ProductId: str, AcceptLanguage: str=None) -> ListProvisioningArtifactsOutput:
        """Lists all provisioning artifacts (also known as versions) for the specified product.

        :param self:
        :param ProductId:  The product identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: NextPageToken:  The page token to use to retrieve the next set of results. If there are no additional results, this value is null.<br/>ProvisioningArtifactDetails:  Information about the provisioning artifacts.<br/>        """
        pass

    def list_provisioning_artifacts_for_service_action(self, ServiceActionId: str, PageSize: int=None, PageToken: str=None, AcceptLanguage: str=None) -> ListProvisioningArtifactsForServiceActionOutput:
        """Lists all provisioning artifacts (also known as versions) for the specified self-service action.

        :param self:
        :param AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>PageToken:  The page token for the next set of results. To retrieve the first set of results, use null.<br/>PageSize:  The maximum number of items to return with this call.<br/>ServiceActionId:  The self-service action identifier. For example, <code>act-fs7abcd89wxyz</code>.<br/>
        :return: NextPageToken:  The page token to use to retrieve the next set of results. If there are no additional results, this value is null.<br/>ProvisioningArtifactViews:  An array of objects with information about product views and provisioning artifacts.<br/>        """
        pass

    def list_record_history(self, AcceptLanguage: str=None, AccessLevelFilter: AccessLevelFilter=None, SearchFilter: ListRecordHistorySearchFilter=None, PageSize: int=None, PageToken: str=None) -> ListRecordHistoryOutput:
        """Lists the specified requests or all performed requests.

        :param self:
        :param PageToken:  The page token for the next set of results. To retrieve the first set of results, use null.<br/>PageSize:  The maximum number of items to return with this call.<br/>SearchFilter:  The search filter to scope the results.:<br/>AccessLevelFilter:  The access level to use to obtain results. The default is <code>User</code>.:<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The user to which the access level applies. The only supported value is <code>Self</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> The access level. <ul> <li>  <code>Account</code> - Filter results based on the account. </li> <li>  <code>Role</code> - Filter results based on the federated role of the specified user. </li> <li>  <code>User</code> - Filter results based on the specified user. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The filter value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> The filter key. <ul> <li>  <code>product</code> - Filter results based on the specified product identifier. </li> <li>  <code>provisionedproduct</code> - Filter results based on the provisioned product identifier. </li> </ul><br/>
        :return: NextPageToken:  The page token to use to retrieve the next set of results. If there are no additional results, this value is null.<br/>RecordDetails:  The records, in reverse chronological order.<br/>        """
        pass

    def list_resources_for_tag_option(self, TagOptionId: str, ResourceType: str=None, PageSize: int=None, PageToken: str=None) -> ListResourcesForTagOptionOutput:
        """Lists the resources associated with the specified TagOption.

        :param self:
        :param PageToken:  The page token for the next set of results. To retrieve the first set of results, use null.<br/>PageSize:  The maximum number of items to return with this call.<br/>ResourceType:  The resource type. <ul> <li>  <code>Portfolio</code>  </li> <li>  <code>Product</code>  </li> </ul><br/>TagOptionId:  The TagOption identifier.<br/>
        :return: PageToken:  The page token for the next set of results. To retrieve the first set of results, use null.<br/>ResourceDetails:  Information about the resources.<br/>        """
        pass

    def list_service_actions(self, AcceptLanguage: str=None, PageSize: int=None, PageToken: str=None) -> ListServiceActionsOutput:
        """Lists all self-service actions.

        :param self:
        :param PageToken:  The page token for the next set of results. To retrieve the first set of results, use null.<br/>PageSize:  The maximum number of items to return with this call.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: NextPageToken:  The page token to use to retrieve the next set of results. If there are no additional results, this value is null.<br/>ServiceActionSummaries:  An object containing information about the service actions associated with the provisioning artifact.<br/>        """
        pass

    def list_service_actions_for_provisioning_artifact(self, ProvisioningArtifactId: str, ProductId: str, PageSize: int=None, PageToken: str=None, AcceptLanguage: str=None) -> ListServiceActionsForProvisioningArtifactOutput:
        """Returns a paginated list of self-service actions associated with the specified Product ID and Provisioning Artifact ID.

        :param self:
        :param AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>PageToken:  The page token for the next set of results. To retrieve the first set of results, use null.<br/>PageSize:  The maximum number of items to return with this call.<br/>ProvisioningArtifactId:  The identifier of the provisioning artifact. For example, <code>pa-4abcdjnxjj6ne</code>.<br/>ProductId:  The product identifier. For example, <code>prod-abcdzk7xy33qa</code>.<br/>
        :return: NextPageToken:  The page token to use to retrieve the next set of results. If there are no additional results, this value is null.<br/>ServiceActionSummaries:  An object containing information about the self-service actions associated with the provisioning artifact.<br/>        """
        pass

    def list_stack_instances_for_provisioned_product(self, ProvisionedProductId: str, AcceptLanguage: str=None, PageToken: str=None, PageSize: int=None) -> ListStackInstancesForProvisionedProductOutput:
        """Returns summary information about stack instances that are associated with the specified <code>CFN_STACKSET</code> type provisioned product. You can filter for stack instances that are associated with a specific AWS account name or region. 

        :param self:
        :param PageSize:  The maximum number of items to return with this call.<br/>PageToken:  The page token for the next set of results. To retrieve the first set of results, use null.<br/>ProvisionedProductId:  The identifier of the provisioned product.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: NextPageToken:  The page token to use to retrieve the next set of results. If there are no additional results, this value is null.<br/>StackInstances:  List of stack instances.<br/>        """
        pass

    def list_tag_options(self, Filters: ListTagOptionsFilters=None, PageSize: int=None, PageToken: str=None) -> ListTagOptionsOutput:
        """Lists the specified TagOptions or all TagOptions.

        :param self:
        :param PageToken:  The page token for the next set of results. To retrieve the first set of results, use null.<br/>PageSize:  The maximum number of items to return with this call.<br/>Filters:  The search filters. If no search filters are specified, the output includes all TagOptions.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Active</i> The active state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The TagOption value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> The TagOption key.<br/>
        :return: PageToken:  The page token for the next set of results. To retrieve the first set of results, use null.<br/>TagOptionDetails:  Information about the TagOptions.<br/>        """
        pass

    def provision_product(self, ProvisionToken: str, ProvisionedProductName: str, ProvisioningArtifactId: str, ProductId: str, AcceptLanguage: str=None, PathId: str=None, ProvisioningParameters: List[ProvisioningParameter]=None, ProvisioningPreferences: ProvisioningPreferences=None, Tags: List[Tag]=None, NotificationArns: List[NotificationArn]=None) -> ProvisionProductOutput:
        """Provisions the specified product. A provisioned product is a resourced instance of a product. For example, provisioning a product based on a CloudFormation template launches a CloudFormation stack and its underlying resources. You can check the status of this request using <a>DescribeRecord</a>. If the request contains a tag key with an empty list of values, there is a tag conflict for that key. Do not include conflicted keys as tags, or this causes the error "Parameter validation failed: Missing required parameter in Tags[<i>N</i>]:<i>Value</i>".

        :param self:
        :param ProvisionToken:  An idempotency token that uniquely identifies the provisioning request.<br/>NotificationArns:  Passed to CloudFormation. The SNS topic ARNs to which to publish stack-related events.<br/>Tags:  One or more tags.<br/>ProvisioningPreferences:  An object that contains information about the provisioning preferences for a stack set.:<br/>ProvisioningParameters:  Parameters specified by the administrator that are required for provisioning the product.<br/>ProvisionedProductName:  A user-friendly name for the provisioned product. This value must be unique for the AWS account and cannot be updated after the product is provisioned.<br/>PathId:  The path identifier of the product. This value is optional if the product has a default path, and required if the product has more than one path. To list the paths for a product, use <a>ListLaunchPaths</a>.<br/>ProvisioningArtifactId:  The identifier of the provisioning artifact.<br/>ProductId:  The product identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StackSetMaxConcurrencyPercentage</i> The maximum percentage of accounts in which to perform this operation at one time. When calculating the number of accounts based on the specified percentage, AWS Service Catalog rounds down to the next whole number. This is true except in cases where rounding down would result is zero. In this case, AWS Service Catalog sets the number as <code>1</code> instead. Note that this setting lets you specify the maximum for operations. For large deployments, under certain circumstances the actual number of accounts acted upon concurrently may be lower due to service throttling. Applicable only to a <code>CFN_STACKSET</code> provisioned product type. Conditional: You must specify either <code>StackSetMaxConcurrentCount</code> or <code>StackSetMaxConcurrentPercentage</code>, but not both.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StackSetMaxConcurrencyCount</i> The maximum number of accounts in which to perform this operation at one time. This is dependent on the value of <code>StackSetFailureToleranceCount</code>. <code>StackSetMaxConcurrentCount</code> is at most one more than the <code>StackSetFailureToleranceCount</code>. Note that this setting lets you specify the maximum for operations. For large deployments, under certain circumstances the actual number of accounts acted upon concurrently may be lower due to service throttling. Applicable only to a <code>CFN_STACKSET</code> provisioned product type. Conditional: You must specify either <code>StackSetMaxConcurrentCount</code> or <code>StackSetMaxConcurrentPercentage</code>, but not both.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StackSetFailureTolerancePercentage</i> The percentage of accounts, per region, for which this stack operation can fail before AWS Service Catalog stops the operation in that region. If the operation is stopped in a region, AWS Service Catalog doesn't attempt the operation in any subsequent regions. When calculating the number of accounts based on the specified percentage, AWS Service Catalog rounds down to the next whole number. Applicable only to a <code>CFN_STACKSET</code> provisioned product type. Conditional: You must specify either <code>StackSetFailureToleranceCount</code> or <code>StackSetFailureTolerancePercentage</code>, but not both.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StackSetFailureToleranceCount</i> The number of accounts, per region, for which this operation can fail before AWS Service Catalog stops the operation in that region. If the operation is stopped in a region, AWS Service Catalog doesn't attempt the operation in any subsequent regions. Applicable only to a <code>CFN_STACKSET</code> provisioned product type. Conditional: You must specify either <code>StackSetFailureToleranceCount</code> or <code>StackSetFailureTolerancePercentage</code>, but not both. The default value is <code>0</code> if no value is specified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StackSetRegions</i> One or more AWS Regions where the provisioned product will be available. Applicable only to a <code>CFN_STACKSET</code> provisioned product type. The specified regions should be within the list of regions from the <code>STACKSET</code> constraint. To get the list of regions in the <code>STACKSET</code> constraint, use the <code>DescribeProvisioningParameters</code> operation. If no values are specified, the default value is all regions from the <code>STACKSET</code> constraint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StackSetAccounts</i> One or more AWS accounts that will have access to the provisioned product. Applicable only to a <code>CFN_STACKSET</code> provisioned product type. The AWS accounts specified should be within the list of accounts in the <code>STACKSET</code> constraint. To get the list of accounts in the <code>STACKSET</code> constraint, use the <code>DescribeProvisioningParameters</code> operation. If no values are specified, the default value is all accounts from the <code>STACKSET</code> constraint.<br/>
        :return: RecordDetail:  Information about the result of provisioning the product.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordTags</i> One or more tags.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordErrors</i> The errors that occurred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PathId</i> The path identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisioningArtifactId</i> The identifier of the provisioning artifact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductId</i> The product identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisionedProductId</i> The identifier of the provisioned product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordType</i> The record type. <ul> <li>  <code>PROVISION_PRODUCT</code>  </li> <li>  <code>UPDATE_PROVISIONED_PRODUCT</code>  </li> <li>  <code>TERMINATE_PROVISIONED_PRODUCT</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisionedProductType</i> The type of provisioned product. The supported values are <code>CFN_STACK</code> and <code>CFN_STACKSET</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdatedTime</i> The time when the record was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The UTC time stamp of the creation time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the provisioned product. <ul> <li>  <code>CREATED</code> - The request was created but the operation has not started. </li> <li>  <code>IN_PROGRESS</code> - The requested operation is in progress. </li> <li>  <code>IN_PROGRESS_IN_ERROR</code> - The provisioned product is under change but the requested operation failed and some remediation is occurring. For example, a rollback. </li> <li>  <code>SUCCEEDED</code> - The requested operation has successfully completed. </li> <li>  <code>FAILED</code> - The requested operation has unsuccessfully completed. Investigate using the error messages returned. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisionedProductName</i> The user-friendly name of the provisioned product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordId</i> The identifier of the record.<br/>        """
        pass

    def reject_portfolio_share(self, PortfolioId: str, AcceptLanguage: str=None, PortfolioShareType: str=None) -> RejectPortfolioShareOutput:
        """Rejects an offer to share the specified portfolio.

        :param self:
        :param PortfolioShareType:  The type of shared portfolios to reject. The default is to reject imported portfolios. <ul> <li>  <code>AWS_ORGANIZATIONS</code> - Reject portfolios shared by the master account of your organization. </li> <li>  <code>IMPORTED</code> - Reject imported portfolios. </li> <li>  <code>AWS_SERVICECATALOG</code> - Not supported. (Throws ResourceNotFoundException.) </li> </ul> For example, <code>aws servicecatalog reject-portfolio-share --portfolio-id "port-2qwzkwxt3y5fk" --portfolio-share-type AWS_ORGANIZATIONS</code> <br/>PortfolioId:  The portfolio identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return:         """
        pass

    def scan_provisioned_products(self, AcceptLanguage: str=None, AccessLevelFilter: AccessLevelFilter=None, PageSize: int=None, PageToken: str=None) -> ScanProvisionedProductsOutput:
        """Lists the provisioned products that are available (not terminated). To use additional filtering, see <a>SearchProvisionedProducts</a>.

        :param self:
        :param PageToken:  The page token for the next set of results. To retrieve the first set of results, use null.<br/>PageSize:  The maximum number of items to return with this call.<br/>AccessLevelFilter:  The access level to use to obtain results. The default is <code>User</code>.:<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The user to which the access level applies. The only supported value is <code>Self</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> The access level. <ul> <li>  <code>Account</code> - Filter results based on the account. </li> <li>  <code>Role</code> - Filter results based on the federated role of the specified user. </li> <li>  <code>User</code> - Filter results based on the specified user. </li> </ul><br/>
        :return: NextPageToken:  The page token to use to retrieve the next set of results. If there are no additional results, this value is null.<br/>ProvisionedProducts:  Information about the provisioned products.<br/>        """
        pass

    def search_products(self, AcceptLanguage: str=None, Filters: ProductViewFilters=None, PageSize: int=None, SortBy: str=None, SortOrder: str=None, PageToken: str=None) -> SearchProductsOutput:
        """Gets information about the products to which the caller has access.

        :param self:
        :param PageToken:  The page token for the next set of results. To retrieve the first set of results, use null.<br/>SortOrder:  The sort order. If no value is specified, the results are not sorted.<br/>SortBy:  The sort field. If no value is specified, the results are not sorted.<br/>PageSize:  The maximum number of items to return with this call.<br/>Filters:  The search filters. If no search filters are specified, the output includes all products to which the caller has access.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: NextPageToken:  The page token to use to retrieve the next set of results. If there are no additional results, this value is null.<br/>ProductViewAggregations:  The product view aggregations.<br/>ProductViewSummaries:  Information about the product views.<br/>        """
        pass

    def search_products_as_admin(self, AcceptLanguage: str=None, PortfolioId: str=None, Filters: ProductViewFilters=None, SortBy: str=None, SortOrder: str=None, PageToken: str=None, PageSize: int=None, ProductSource: str=None) -> SearchProductsAsAdminOutput:
        """Gets information about the products for the specified portfolio or all products.

        :param self:
        :param ProductSource:  Access level of the source of the product.<br/>PageSize:  The maximum number of items to return with this call.<br/>PageToken:  The page token for the next set of results. To retrieve the first set of results, use null.<br/>SortOrder:  The sort order. If no value is specified, the results are not sorted.<br/>SortBy:  The sort field. If no value is specified, the results are not sorted.<br/>Filters:  The search filters. If no search filters are specified, the output includes all products to which the administrator has access.<br/>PortfolioId:  The portfolio identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: NextPageToken:  The page token to use to retrieve the next set of results. If there are no additional results, this value is null.<br/>ProductViewDetails:  Information about the product views.<br/>        """
        pass

    def search_provisioned_products(self, AcceptLanguage: str=None, AccessLevelFilter: AccessLevelFilter=None, Filters: ProvisionedProductFilters=None, SortBy: str=None, SortOrder: str=None, PageSize: int=None, PageToken: str=None) -> SearchProvisionedProductsOutput:
        """Gets information about the provisioned products that meet the specified criteria.

        :param self:
        :param PageToken:  The page token for the next set of results. To retrieve the first set of results, use null.<br/>PageSize:  The maximum number of items to return with this call.<br/>SortOrder:  The sort order. If no value is specified, the results are not sorted.<br/>SortBy:  The sort field. If no value is specified, the results are not sorted. The valid values are <code>arn</code>, <code>id</code>, <code>name</code>, and <code>lastRecordId</code>.<br/>Filters:  The search filters. When the key is <code>SearchQuery</code>, the searchable fields are <code>arn</code>, <code>createdTime</code>, <code>id</code>, <code>lastRecordId</code>, <code>idempotencyToken</code>, <code>name</code>, <code>physicalId</code>, <code>productId</code>, <code>provisioningArtifact</code>, <code>type</code>, <code>status</code>, <code>tags</code>, <code>userArn</code>, and <code>userArnSession</code>. Example: <code>"SearchQuery":["status:AVAILABLE"]</code> <br/>AccessLevelFilter:  The access level to use to obtain results. The default is <code>User</code>.:<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The user to which the access level applies. The only supported value is <code>Self</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> The access level. <ul> <li>  <code>Account</code> - Filter results based on the account. </li> <li>  <code>Role</code> - Filter results based on the federated role of the specified user. </li> <li>  <code>User</code> - Filter results based on the specified user. </li> </ul><br/>
        :return: NextPageToken:  The page token to use to retrieve the next set of results. If there are no additional results, this value is null.<br/>TotalResultsCount:  The number of provisioned products found.<br/>ProvisionedProducts:  Information about the provisioned products.<br/>        """
        pass

    def terminate_provisioned_product(self, TerminateToken: str, ProvisionedProductName: str=None, ProvisionedProductId: str=None, IgnoreErrors: IgnoreErrors=None, AcceptLanguage: str=None) -> TerminateProvisionedProductOutput:
        """Terminates the specified provisioned product. This operation does not delete any records associated with the provisioned product. You can check the status of this request using <a>DescribeRecord</a>.

        :param self:
        :param AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>IgnoreErrors:  If set to true, AWS Service Catalog stops managing the specified provisioned product even if it cannot delete the underlying resources.<br/>TerminateToken:  An idempotency token that uniquely identifies the termination request. This token is only valid during the termination process. After the provisioned product is terminated, subsequent requests to terminate the same provisioned product always return <b>ResourceNotFound</b>.<br/>ProvisionedProductId:  The identifier of the provisioned product. You cannot specify both <code>ProvisionedProductName</code> and <code>ProvisionedProductId</code>.<br/>ProvisionedProductName:  The name of the provisioned product. You cannot specify both <code>ProvisionedProductName</code> and <code>ProvisionedProductId</code>.<br/>
        :return: RecordDetail:  Information about the result of this request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordTags</i> One or more tags.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordErrors</i> The errors that occurred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PathId</i> The path identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisioningArtifactId</i> The identifier of the provisioning artifact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductId</i> The product identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisionedProductId</i> The identifier of the provisioned product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordType</i> The record type. <ul> <li>  <code>PROVISION_PRODUCT</code>  </li> <li>  <code>UPDATE_PROVISIONED_PRODUCT</code>  </li> <li>  <code>TERMINATE_PROVISIONED_PRODUCT</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisionedProductType</i> The type of provisioned product. The supported values are <code>CFN_STACK</code> and <code>CFN_STACKSET</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdatedTime</i> The time when the record was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The UTC time stamp of the creation time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the provisioned product. <ul> <li>  <code>CREATED</code> - The request was created but the operation has not started. </li> <li>  <code>IN_PROGRESS</code> - The requested operation is in progress. </li> <li>  <code>IN_PROGRESS_IN_ERROR</code> - The provisioned product is under change but the requested operation failed and some remediation is occurring. For example, a rollback. </li> <li>  <code>SUCCEEDED</code> - The requested operation has successfully completed. </li> <li>  <code>FAILED</code> - The requested operation has unsuccessfully completed. Investigate using the error messages returned. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisionedProductName</i> The user-friendly name of the provisioned product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordId</i> The identifier of the record.<br/>        """
        pass

    def update_constraint(self, Id: str, AcceptLanguage: str=None, Description: str=None, Parameters: str=None) -> UpdateConstraintOutput:
        """Updates the specified constraint.

        :param self:
        :param Parameters:  The constraint parameters, in JSON format. The syntax depends on the constraint type as follows: <dl> <dt>LAUNCH</dt> <dd> Specify the <code>RoleArn</code> property as follows:  <code>{"RoleArn" : "arn:aws:iam::123456789012:role/LaunchRole"}</code>  You cannot have both a <code>LAUNCH</code> and a <code>STACKSET</code> constraint. You also cannot have more than one <code>LAUNCH</code> constraint on a product and portfolio. </dd> <dt>NOTIFICATION</dt> <dd> Specify the <code>NotificationArns</code> property as follows:  <code>{"NotificationArns" : ["arn:aws:sns:us-east-1:123456789012:Topic"]}</code>  </dd> <dt>RESOURCE_UPDATE</dt> <dd> Specify the <code>TagUpdatesOnProvisionedProduct</code> property as follows:  <code>{"Version":"2.0","Properties":{"TagUpdateOnProvisionedProduct":"String"}}</code>  The <code>TagUpdatesOnProvisionedProduct</code> property accepts a string value of <code>ALLOWED</code> or <code>NOT_ALLOWED</code>. </dd> <dt>STACKSET</dt> <dd> Specify the <code>Parameters</code> property as follows:  <code>{"Version": "String", "Properties": {"AccountList": [ "String" ], "RegionList": [ "String" ], "AdminRole": "String", "ExecutionRole": "String"}}</code>  You cannot have both a <code>LAUNCH</code> and a <code>STACKSET</code> constraint. You also cannot have more than one <code>STACKSET</code> constraint on a product and portfolio. Products with a <code>STACKSET</code> constraint will launch an AWS CloudFormation stack set. </dd> <dt>TEMPLATE</dt> <dd> Specify the <code>Rules</code> property. For more information, see <a href="http://docs.aws.amazon.com/servicecatalog/latest/adminguide/reference-template_constraint_rules.html">Template Constraint Rules</a>. </dd> </dl><br/>Description:  The updated description of the constraint.<br/>Id:  The identifier of the constraint.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: Status:  The status of the current request.<br/>ConstraintParameters:  The constraint parameters.<br/>ConstraintDetail:  Information about the constraint.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Owner</i> The owner of the constraint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the constraint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of constraint. <ul> <li>  <code>LAUNCH</code>  </li> <li>  <code>NOTIFICATION</code>  </li> <li> STACKSET </li> <li>  <code>TEMPLATE</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ConstraintId</i> The identifier of the constraint.<br/>        """
        pass

    def update_portfolio(self, Id: str, AcceptLanguage: str=None, DisplayName: str=None, Description: str=None, ProviderName: str=None, AddTags: List[Tag]=None, RemoveTags: List[TagKey]=None) -> UpdatePortfolioOutput:
        """Updates the specified portfolio. You cannot update a product that was shared with you.

        :param self:
        :param RemoveTags:  The tags to remove.<br/>AddTags:  The tags to add.<br/>ProviderName:  The updated name of the portfolio provider.<br/>Description:  The updated description of the portfolio.<br/>DisplayName:  The name to use for display purposes.<br/>Id:  The portfolio identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: Tags:  Information about the tags associated with the portfolio.<br/>PortfolioDetail:  Information about the portfolio.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProviderName</i> The name of the portfolio provider.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The UTC time stamp of the creation time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the portfolio.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DisplayName</i> The name to use for display purposes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ARN</i> The ARN assigned to the portfolio.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The portfolio identifier.<br/>        """
        pass

    def update_product(self, Id: str, AcceptLanguage: str=None, Name: str=None, Owner: str=None, Description: str=None, Distributor: str=None, SupportDescription: str=None, SupportEmail: str=None, SupportUrl: str=None, AddTags: List[Tag]=None, RemoveTags: List[TagKey]=None) -> UpdateProductOutput:
        """Updates the specified product.

        :param self:
        :param RemoveTags:  The tags to remove from the product.<br/>AddTags:  The tags to add to the product.<br/>SupportUrl:  The updated support URL for the product.<br/>SupportEmail:  The updated support email for the product.<br/>SupportDescription:  The updated support description for the product.<br/>Distributor:  The updated distributor of the product.<br/>Description:  The updated description of the product.<br/>Owner:  The updated owner of the product.<br/>Name:  The updated product name.<br/>Id:  The product identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: Tags:  Information about the tags associated with the product.<br/>ProductViewDetail:  Information about the product view.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The UTC time stamp of the creation time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductARN</i> The ARN of the product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the product. <ul> <li>  <code>AVAILABLE</code> - The product is ready for use. </li> <li>  <code>CREATING</code> - Product creation has started; the product is not ready for use. </li> <li>  <code>FAILED</code> - An action failed. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductViewSummary</i> Summary information about the product view.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ProductViewSummary</b>: Summary information about the product view.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportUrl</i> The URL information to obtain support for this Product.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportDescription</i> The description of the support for this Product.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SupportEmail</i> The email contact information to obtain support for this Product.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HasDefaultPath</i> Indicates whether the product has a default path. If the product does not have a default path, call <a>ListLaunchPaths</a> to disambiguate between paths. Otherwise, <a>ListLaunchPaths</a> is not required, and the output of <a>ProductViewSummary</a> can be used directly with <a>DescribeProvisioningParameters</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Distributor</i> The distributor of the product. Contact the product administrator for the significance of this value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The product type. Contact the product administrator for the significance of this value. If this value is <code>MARKETPLACE</code>, the product was created by AWS Marketplace.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ShortDescription</i> Short description of the product.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Owner</i> The owner of the product. Contact the product administrator for the significance of this value.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the product.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductId</i> The product identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The product view identifier.<br/>        """
        pass

    def update_provisioned_product(self, UpdateToken: str, AcceptLanguage: str=None, ProvisionedProductName: str=None, ProvisionedProductId: str=None, ProductId: str=None, ProvisioningArtifactId: str=None, PathId: str=None, ProvisioningParameters: List[UpdateProvisioningParameter]=None, ProvisioningPreferences: UpdateProvisioningPreferences=None, Tags: List[Tag]=None) -> UpdateProvisionedProductOutput:
        """Requests updates to the configuration of the specified provisioned product. If there are tags associated with the object, they cannot be updated or added. Depending on the specific updates requested, this operation can update with no interruption, with some interruption, or replace the provisioned product entirely. You can check the status of this request using <a>DescribeRecord</a>.

        :param self:
        :param UpdateToken:  The idempotency token that uniquely identifies the provisioning update request.<br/>Tags:  One or more tags. Requires the product to have <code>RESOURCE_UPDATE</code> constraint with <code>TagUpdatesOnProvisionedProduct</code> set to <code>ALLOWED</code> to allow tag updates.<br/>ProvisioningPreferences:  An object that contains information about the provisioning preferences for a stack set.:<br/>ProvisioningParameters:  The new parameters.<br/>PathId:  The new path identifier. This value is optional if the product has a default path, and required if the product has more than one path.<br/>ProvisioningArtifactId:  The identifier of the provisioning artifact.<br/>ProductId:  The identifier of the product.<br/>ProvisionedProductId:  The identifier of the provisioned product. You cannot specify both <code>ProvisionedProductName</code> and <code>ProvisionedProductId</code>.<br/>ProvisionedProductName:  The name of the provisioned product. You cannot specify both <code>ProvisionedProductName</code> and <code>ProvisionedProductId</code>.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StackSetOperationType</i> Determines what action AWS Service Catalog performs to a stack set or a stack instance represented by the provisioned product. The default value is <code>UPDATE</code> if nothing is specified. Applicable only to a <code>CFN_STACKSET</code> provisioned product type. <dl> <dt>CREATE</dt> <dd> Creates a new stack instance in the stack set represented by the provisioned product. In this case, only new stack instances are created based on accounts and regions; if new ProductId or ProvisioningArtifactID are passed, they will be ignored. </dd> <dt>UPDATE</dt> <dd> Updates the stack set represented by the provisioned product and also its stack instances. </dd> <dt>DELETE</dt> <dd> Deletes a stack instance in the stack set represented by the provisioned product. </dd> </dl><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StackSetMaxConcurrencyPercentage</i> The maximum percentage of accounts in which to perform this operation at one time. When calculating the number of accounts based on the specified percentage, AWS Service Catalog rounds down to the next whole number. This is true except in cases where rounding down would result is zero. In this case, AWS Service Catalog sets the number as <code>1</code> instead. Note that this setting lets you specify the maximum for operations. For large deployments, under certain circumstances the actual number of accounts acted upon concurrently may be lower due to service throttling. Applicable only to a <code>CFN_STACKSET</code> provisioned product type. Conditional: You must specify either <code>StackSetMaxConcurrentCount</code> or <code>StackSetMaxConcurrentPercentage</code>, but not both.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StackSetMaxConcurrencyCount</i> The maximum number of accounts in which to perform this operation at one time. This is dependent on the value of <code>StackSetFailureToleranceCount</code>. <code>StackSetMaxConcurrentCount</code> is at most one more than the <code>StackSetFailureToleranceCount</code>. Note that this setting lets you specify the maximum for operations. For large deployments, under certain circumstances the actual number of accounts acted upon concurrently may be lower due to service throttling. Applicable only to a <code>CFN_STACKSET</code> provisioned product type. Conditional: You must specify either <code>StackSetMaxConcurrentCount</code> or <code>StackSetMaxConcurrentPercentage</code>, but not both.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StackSetFailureTolerancePercentage</i> The percentage of accounts, per region, for which this stack operation can fail before AWS Service Catalog stops the operation in that region. If the operation is stopped in a region, AWS Service Catalog doesn't attempt the operation in any subsequent regions. When calculating the number of accounts based on the specified percentage, AWS Service Catalog rounds down to the next whole number. Applicable only to a <code>CFN_STACKSET</code> provisioned product type. Conditional: You must specify either <code>StackSetFailureToleranceCount</code> or <code>StackSetFailureTolerancePercentage</code>, but not both.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StackSetFailureToleranceCount</i> The number of accounts, per region, for which this operation can fail before AWS Service Catalog stops the operation in that region. If the operation is stopped in a region, AWS Service Catalog doesn't attempt the operation in any subsequent regions. Applicable only to a <code>CFN_STACKSET</code> provisioned product type. Conditional: You must specify either <code>StackSetFailureToleranceCount</code> or <code>StackSetFailureTolerancePercentage</code>, but not both. The default value is <code>0</code> if no value is specified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StackSetRegions</i> One or more AWS Regions where the provisioned product will be available. Applicable only to a <code>CFN_STACKSET</code> provisioned product type. The specified regions should be within the list of regions from the <code>STACKSET</code> constraint. To get the list of regions in the <code>STACKSET</code> constraint, use the <code>DescribeProvisioningParameters</code> operation. If no values are specified, the default value is all regions from the <code>STACKSET</code> constraint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StackSetAccounts</i> One or more AWS accounts that will have access to the provisioned product. Applicable only to a <code>CFN_STACKSET</code> provisioned product type. The AWS accounts specified should be within the list of accounts in the <code>STACKSET</code> constraint. To get the list of accounts in the <code>STACKSET</code> constraint, use the <code>DescribeProvisioningParameters</code> operation. If no values are specified, the default value is all accounts from the <code>STACKSET</code> constraint.<br/>
        :return: RecordDetail:  Information about the result of the request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordTags</i> One or more tags.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordErrors</i> The errors that occurred.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PathId</i> The path identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisioningArtifactId</i> The identifier of the provisioning artifact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProductId</i> The product identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisionedProductId</i> The identifier of the provisioned product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordType</i> The record type. <ul> <li>  <code>PROVISION_PRODUCT</code>  </li> <li>  <code>UPDATE_PROVISIONED_PRODUCT</code>  </li> <li>  <code>TERMINATE_PROVISIONED_PRODUCT</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisionedProductType</i> The type of provisioned product. The supported values are <code>CFN_STACK</code> and <code>CFN_STACKSET</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdatedTime</i> The time when the record was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The UTC time stamp of the creation time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the provisioned product. <ul> <li>  <code>CREATED</code> - The request was created but the operation has not started. </li> <li>  <code>IN_PROGRESS</code> - The requested operation is in progress. </li> <li>  <code>IN_PROGRESS_IN_ERROR</code> - The provisioned product is under change but the requested operation failed and some remediation is occurring. For example, a rollback. </li> <li>  <code>SUCCEEDED</code> - The requested operation has successfully completed. </li> <li>  <code>FAILED</code> - The requested operation has unsuccessfully completed. Investigate using the error messages returned. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProvisionedProductName</i> The user-friendly name of the provisioned product.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecordId</i> The identifier of the record.<br/>        """
        pass

    def update_provisioned_product_properties(self, IdempotencyToken: str, ProvisionedProductProperties: ProvisionedProductProperties, ProvisionedProductId: str, AcceptLanguage: str=None) -> UpdateProvisionedProductPropertiesOutput:
        """Requests updates to the properties of the specified provisioned product.

        :param self:
        :param IdempotencyToken:  The idempotency token that uniquely identifies the provisioning product update request.<br/>ProvisionedProductProperties:  A map that contains the provisioned product properties to be updated. The <code>OWNER</code> key only accepts user ARNs. The owner is the user that is allowed to see, update, terminate, and execute service actions in the provisioned product. The administrator can change the owner of a provisioned product to another IAM user within the same account. Both end user owners and administrators can see ownership history of the provisioned product using the <code>ListRecordHistory</code> API. The new owner can describe all past records for the provisioned product using the <code>DescribeRecord</code> API. The previous owner can no longer use <code>DescribeRecord</code>, but can still see the product's history from when he was an owner using <code>ListRecordHistory</code>. If a provisioned product ownership is assigned to an end user, they can see and perform any action through the API or Service Catalog console such as update, terminate, and execute service actions. If an end user provisions a product and the owner is updated to someone else, they will no longer be able to see or perform any actions through API or the Service Catalog console on that provisioned product.<br/>ProvisionedProductId:  The identifier of the provisioned product.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: Status:  The status of the request.<br/>RecordId:  The identifier of the record.<br/>ProvisionedProductProperties:  A map that contains the properties updated.<br/>ProvisionedProductId:  The provisioned product identifier.<br/>        """
        pass

    def update_provisioning_artifact(self, ProvisioningArtifactId: str, ProductId: str, AcceptLanguage: str=None, Name: str=None, Description: str=None, Active: ProvisioningArtifactActive=None, Guidance: str=None) -> UpdateProvisioningArtifactOutput:
        """Updates the specified provisioning artifact (also known as a version) for the specified product. You cannot update a provisioning artifact for a product that was shared with you.

        :param self:
        :param Guidance:  Information set by the administrator to provide guidance to end users about which provisioning artifacts to use. The <code>DEFAULT</code> value indicates that the product version is active. The administrator can set the guidance to <code>DEPRECATED</code> to inform users that the product version is deprecated. Users are able to make updates to a provisioned product of a deprecated version but cannot launch new provisioned products using a deprecated version.<br/>Active:  Indicates whether the product version is active.<br/>Description:  The updated description of the provisioning artifact.<br/>Name:  The updated name of the provisioning artifact.<br/>ProvisioningArtifactId:  The identifier of the provisioning artifact.<br/>ProductId:  The product identifier.<br/>AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>
        :return: Status:  The status of the current request.<br/>Info:  The URL of the CloudFormation template in Amazon S3.<br/>ProvisioningArtifactDetail:  Information about the provisioning artifact.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Guidance</i> Information set by the administrator to provide guidance to end users about which provisioning artifacts to use.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Active</i> Indicates whether the product version is active.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The UTC time stamp of the creation time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of provisioning artifact. <ul> <li>  <code>CLOUD_FORMATION_TEMPLATE</code> - AWS CloudFormation template </li> <li>  <code>MARKETPLACE_AMI</code> - AWS Marketplace AMI </li> <li>  <code>MARKETPLACE_CAR</code> - AWS Marketplace Clusters and AWS Resources </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the provisioning artifact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the provisioning artifact.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The identifier of the provisioning artifact.<br/>        """
        pass

    def update_service_action(self, Id: str, Name: str=None, Definition: ServiceActionDefinitionMap=None, Description: str=None, AcceptLanguage: str=None) -> UpdateServiceActionOutput:
        """Updates a self-service action.

        :param self:
        :param AcceptLanguage:  The language code. <ul> <li>  <code>en</code> - English (default) </li> <li>  <code>jp</code> - Japanese </li> <li>  <code>zh</code> - Chinese </li> </ul><br/>Description:  The self-service action description.<br/>Definition:  A map that defines the self-service action.<br/>Name:  The self-service action name.<br/>Id:  The self-service action identifier.<br/>
        :return: ServiceActionDetail:  Detailed information about the self-service action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Definition</i> A map that defines the self-service action.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceActionSummary</i> Summary information about the self-service action.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ServiceActionSummary</b>: Summary information about the self-service action.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DefinitionType</i> The self-service action definition type. For example, <code>SSM_AUTOMATION</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The self-service action description.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The self-service action name.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The self-service action identifier.<br/>        """
        pass

    def update_tag_option(self, Id: str, Value: str=None, Active: TagOptionActive=None) -> UpdateTagOptionOutput:
        """Updates the specified TagOption.

        :param self:
        :param Active:  The updated active state.<br/>Value:  The updated value.<br/>Id:  The TagOption identifier.<br/>
        :return: TagOptionDetail:  Information about the TagOption.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The TagOption identifier.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Active</i> The TagOption active state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The TagOption value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Key</i> The TagOption key.<br/>        """
        pass



class ServiceDiscovery(BaseClient):
    """AWS Cloud Map lets you configure public DNS, private DNS, or HTTP namespaces that your microservice applications run in. When an instance of the service becomes available, you can call the AWS Cloud Map API to register the instance with AWS Cloud Map. For public or private DNS namespaces, AWS Cloud Map automatically creates DNS records and an optional health check. Clients that submit public or private DNS queries, or HTTP requests, for the service receive an answer that contains up to eight healthy records. """


    class ListServicesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_services` for valid parameters"""
            pass
    class ListInstancesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_instances` for valid parameters"""
            pass
    class ListNamespacesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_namespaces` for valid parameters"""
            pass
    class ListOperationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_operations` for valid parameters"""
            pass


    class CreateHttpNamespaceRequest(Mapping):
        pass
    
    class CreateHttpNamespaceResponse(Mapping):
        pass
    
    class CreatePrivateDnsNamespaceRequest(Mapping):
        pass
    
    class CreatePrivateDnsNamespaceResponse(Mapping):
        pass
    
    class CreatePublicDnsNamespaceRequest(Mapping):
        pass
    
    class CreatePublicDnsNamespaceResponse(Mapping):
        pass
    
    class CreateServiceRequest(Mapping):
        pass
    
    class CreateServiceResponse(Mapping):
        pass
    
    class DnsConfig(Mapping):
        pass
    
    class HealthCheckConfig(Mapping):
        pass
    
    class HealthCheckCustomConfig(Mapping):
        pass
    
    class DeleteNamespaceRequest(Mapping):
        pass
    
    class DeleteNamespaceResponse(Mapping):
        pass
    
    class DeleteServiceRequest(Mapping):
        pass
    
    class DeleteServiceResponse(Mapping):
        pass
    
    class DeregisterInstanceRequest(Mapping):
        pass
    
    class DeregisterInstanceResponse(Mapping):
        pass
    
    class DiscoverInstancesRequest(Mapping):
        pass
    
    class DiscoverInstancesResponse(Mapping):
        pass
    
    class Attributes(object):
        pass
    
    class GetInstanceRequest(Mapping):
        pass
    
    class GetInstanceResponse(Mapping):
        pass
    
    class GetInstancesHealthStatusRequest(Mapping):
        pass
    
    class GetInstancesHealthStatusResponse(Mapping):
        pass
    
    class ResourceId(object):
        pass
    
    class GetNamespaceRequest(Mapping):
        pass
    
    class GetNamespaceResponse(Mapping):
        pass
    
    class GetOperationRequest(Mapping):
        pass
    
    class GetOperationResponse(Mapping):
        pass
    
    class GetServiceRequest(Mapping):
        pass
    
    class GetServiceResponse(Mapping):
        pass
    
    class ListInstancesRequest(Mapping):
        pass
    
    class ListInstancesResponse(Mapping):
        pass
    
    class ListNamespacesRequest(Mapping):
        pass
    
    class ListNamespacesResponse(Mapping):
        pass
    
    class NamespaceFilter(Mapping):
        pass
    
    class ListOperationsRequest(Mapping):
        pass
    
    class ListOperationsResponse(Mapping):
        pass
    
    class OperationFilter(Mapping):
        pass
    
    class ListServicesRequest(Mapping):
        pass
    
    class ListServicesResponse(Mapping):
        pass
    
    class ServiceFilter(Mapping):
        pass
    
    class RegisterInstanceRequest(Mapping):
        pass
    
    class RegisterInstanceResponse(Mapping):
        pass
    
    class UpdateInstanceCustomHealthStatusRequest(Mapping):
        pass
    
    class UpdateServiceRequest(Mapping):
        pass
    
    class UpdateServiceResponse(Mapping):
        pass
    
    class ServiceChange(Mapping):
        pass
    
    def create_http_namespace(self, Name: str, CreatorRequestId: str=None, Description: str=None) -> CreateHttpNamespaceResponse:
        """Creates an HTTP namespace. Service instances that you register using an HTTP namespace can be discovered using a <code>DiscoverInstances</code> request but can't be discovered using DNS.  For the current limit on the number of namespaces that you can create using the same AWS account, see <a href="http://docs.aws.amazon.com/cloud-map/latest/dg/cloud-map-limits.html">AWS Cloud Map Limits</a> in the <i>AWS Cloud Map Developer Guide</i>.

        :param self:
        :param Description:  A description for the namespace.<br/>CreatorRequestId:  A unique string that identifies the request and that allows failed <code>CreateHttpNamespace</code> requests to be retried without the risk of executing the operation twice. <code>CreatorRequestId</code> can be any unique string, for example, a date/time stamp.<br/>Name:  The name that you want to assign to this namespace.<br/>
        :return: OperationId:  A value that you can use to determine whether the request completed successfully. To get the status of the operation, see <a>GetOperation</a>.<br/>        """
        pass

    def create_private_dns_namespace(self, Vpc: str, Name: str, CreatorRequestId: str=None, Description: str=None) -> CreatePrivateDnsNamespaceResponse:
        """Creates a private namespace based on DNS, which will be visible only inside a specified Amazon VPC. The namespace defines your service naming scheme. For example, if you name your namespace <code>example.com</code> and name your service <code>backend</code>, the resulting DNS name for the service will be <code>backend.example.com</code>. For the current limit on the number of namespaces that you can create using the same AWS account, see <a href="http://docs.aws.amazon.com/cloud-map/latest/dg/cloud-map-limits.html">AWS Cloud Map Limits</a> in the <i>AWS Cloud Map Developer Guide</i>.

        :param self:
        :param Vpc:  The ID of the Amazon VPC that you want to associate the namespace with.<br/>Description:  A description for the namespace.<br/>CreatorRequestId:  A unique string that identifies the request and that allows failed <code>CreatePrivateDnsNamespace</code> requests to be retried without the risk of executing the operation twice. <code>CreatorRequestId</code> can be any unique string, for example, a date/time stamp.<br/>Name:  The name that you want to assign to this namespace. When you create a private DNS namespace, AWS Cloud Map automatically creates an Amazon Route 53 private hosted zone that has the same name as the namespace.<br/>
        :return: OperationId:  A value that you can use to determine whether the request completed successfully. To get the status of the operation, see <a>GetOperation</a>.<br/>        """
        pass

    def create_public_dns_namespace(self, Name: str, CreatorRequestId: str=None, Description: str=None) -> CreatePublicDnsNamespaceResponse:
        """Creates a public namespace based on DNS, which will be visible on the internet. The namespace defines your service naming scheme. For example, if you name your namespace <code>example.com</code> and name your service <code>backend</code>, the resulting DNS name for the service will be <code>backend.example.com</code>. For the current limit on the number of namespaces that you can create using the same AWS account, see <a href="http://docs.aws.amazon.com/cloud-map/latest/dg/cloud-map-limits.html">AWS Cloud Map Limits</a> in the <i>AWS Cloud Map Developer Guide</i>.

        :param self:
        :param Description:  A description for the namespace.<br/>CreatorRequestId:  A unique string that identifies the request and that allows failed <code>CreatePublicDnsNamespace</code> requests to be retried without the risk of executing the operation twice. <code>CreatorRequestId</code> can be any unique string, for example, a date/time stamp.<br/>Name:  The name that you want to assign to this namespace.<br/>
        :return: OperationId:  A value that you can use to determine whether the request completed successfully. To get the status of the operation, see <a>GetOperation</a>.<br/>        """
        pass

    def create_service(self, Name: str, NamespaceId: str=None, CreatorRequestId: str=None, Description: str=None, DnsConfig: DnsConfig=None, HealthCheckConfig: HealthCheckConfig=None, HealthCheckCustomConfig: HealthCheckCustomConfig=None) -> CreateServiceResponse:
        """Creates a service, which defines the configuration for the following entities: <ul> <li> For public and private DNS namespaces, one of the following combinations of DNS records in Amazon Route 53: <ul> <li> A </li> <li> AAAA </li> <li> A and AAAA </li> <li> SRV </li> <li> CNAME </li> </ul> </li> <li> Optionally, a health check </li> </ul> After you create the service, you can submit a <a>RegisterInstance</a> request, and AWS Cloud Map uses the values in the configuration to create the specified entities. For the current limit on the number of instances that you can register using the same namespace and using the same service, see <a href="http://docs.aws.amazon.com/cloud-map/latest/dg/cloud-map-limits.html">AWS Cloud Map Limits</a> in the <i>AWS Cloud Map Developer Guide</i>.

        :param self:
        :param HealthCheckCustomConfig:  A complex type that contains information about an optional custom health check. <important> If you specify a health check configuration, you can specify either <code>HealthCheckCustomConfig</code> or <code>HealthCheckConfig</code> but not both. </important>:<br/>HealthCheckConfig:   <i>Public DNS namespaces only.</i> A complex type that contains settings for an optional Route 53 health check. If you specify settings for a health check, AWS Cloud Map associates the health check with all the Route 53 DNS records that you specify in <code>DnsConfig</code>. <important> If you specify a health check configuration, you can specify either <code>HealthCheckCustomConfig</code> or <code>HealthCheckConfig</code> but not both. </important> For information about the charges for health checks, see <a href="http://aws.amazon.com/cloud-map/pricing/">AWS Cloud Map Pricing</a>.:<br/>DnsConfig:  A complex type that contains information about the Amazon Route 53 records that you want AWS Cloud Map to create when you register an instance. :<br/>Description:  A description for the service.<br/>CreatorRequestId:  A unique string that identifies the request and that allows failed <code>CreateService</code> requests to be retried without the risk of executing the operation twice. <code>CreatorRequestId</code> can be any unique string, for example, a date/time stamp.<br/>NamespaceId:  The ID of the namespace that you want to use to create the service.<br/>Name:  The name that you want to assign to the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DnsRecords</i> An array that contains one <code>DnsRecord</code> object for each Route 53 DNS record that you want AWS Cloud Map to create when you register an instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoutingPolicy</i> The routing policy that you want to apply to all Route 53 DNS records that AWS Cloud Map creates when you register an instance and specify this service. <note> If you want to use this service to register instances that create alias records, specify <code>WEIGHTED</code> for the routing policy. </note> You can specify the following values:  <b>MULTIVALUE</b>  If you define a health check for the service and the health check is healthy, Route 53 returns the applicable value for up to eight instances. For example, suppose the service includes configurations for one A record and a health check, and you use the service to register 10 instances. Route 53 responds to DNS queries with IP addresses for up to eight healthy instances. If fewer than eight instances are healthy, Route 53 responds to every DNS query with the IP addresses for all of the healthy instances. If you don't define a health check for the service, Route 53 assumes that all instances are healthy and returns the values for up to eight instances. For more information about the multivalue routing policy, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/routing-policy.html#routing-policy-multivalue">Multivalue Answer Routing</a> in the <i>Route 53 Developer Guide</i>.  <b>WEIGHTED</b>  Route 53 returns the applicable value from one randomly selected instance from among the instances that you registered using the same service. Currently, all records have the same weight, so you can't route more or less traffic to any instances. For example, suppose the service includes configurations for one A record and a health check, and you use the service to register 10 instances. Route 53 responds to DNS queries with the IP address for one randomly selected instance from among the healthy instances. If no instances are healthy, Route 53 responds to DNS queries as if all of the instances were healthy. If you don't define a health check for the service, Route 53 assumes that all instances are healthy and returns the applicable value for one randomly selected instance. For more information about the weighted routing policy, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/routing-policy.html#routing-policy-weighted">Weighted Routing</a> in the <i>Route 53 Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NamespaceId</i> The ID of the namespace to use for DNS configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureThreshold</i> The number of consecutive health checks that an endpoint must pass or fail for Route 53 to change the current status of the endpoint from unhealthy to healthy or vice versa. For more information, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/dns-failover-determining-health-of-endpoints.html">How Route 53 Determines Whether an Endpoint Is Healthy</a> in the <i>Route 53 Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourcePath</i> The path that you want Route 53 to request when performing health checks. The path can be any value for which your endpoint will return an HTTP status code of 2xx or 3xx when the endpoint is healthy, such as the file <code>/docs/route53-health-check.html</code>. Route 53 automatically adds the DNS name for the service. If you don't specify a value for <code>ResourcePath</code>, the default value is <code>/</code>. If you specify <code>TCP</code> for <code>Type</code>, you must <i>not</i> specify a value for <code>ResourcePath</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of health check that you want to create, which indicates how Route 53 determines whether an endpoint is healthy. <important> You can't change the value of <code>Type</code> after you create a health check. </important> You can create the following types of health checks: <ul> <li>  <b>HTTP</b>: Route 53 tries to establish a TCP connection. If successful, Route 53 submits an HTTP request and waits for an HTTP status code of 200 or greater and less than 400. </li> <li>  <b>HTTPS</b>: Route 53 tries to establish a TCP connection. If successful, Route 53 submits an HTTPS request and waits for an HTTP status code of 200 or greater and less than 400. <important> If you specify HTTPS for the value of <code>Type</code>, the endpoint must support TLS v1.0 or later. </important> </li> <li>  <b>TCP</b>: Route 53 tries to establish a TCP connection. If you specify <code>TCP</code> for <code>Type</code>, don't specify a value for <code>ResourcePath</code>. </li> </ul> For more information, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/dns-failover-determining-health-of-endpoints.html">How Route 53 Determines Whether an Endpoint Is Healthy</a> in the <i>Route 53 Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureThreshold</i> The number of 30-second intervals that you want Cloud Map to wait after receiving an <code>UpdateInstanceCustomHealthStatus</code> request before it changes the health status of a service instance. For example, suppose you specify a value of <code>2</code> for <code>FailureTheshold</code>, and then your application sends an <code>UpdateInstanceCustomHealthStatus</code> request. Cloud Map waits for approximately 60 seconds (2 x 30) before changing the status of the service instance based on that request. Sending a second or subsequent <code>UpdateInstanceCustomHealthStatus</code> request with the same value before <code>FailureThreshold x 30</code> seconds has passed doesn't accelerate the change. Cloud Map still waits <code>FailureThreshold x 30</code> seconds after the first request to make the change.<br/>
        :return: Service:  A complex type that contains information about the new service.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatorRequestId</i> A unique string that identifies the request and that allows failed requests to be retried without the risk of executing the operation twice. <code>CreatorRequestId</code> can be any unique string, for example, a date/time stamp.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateDate</i> The date and time that the service was created, in Unix format and Coordinated Universal Time (UTC). The value of <code>CreateDate</code> is accurate to milliseconds. For example, the value <code>1516925490.087</code> represents Friday, January 26, 2018 12:11:30.087 AM.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HealthCheckCustomConfig</i> A complex type that contains information about an optional custom health check. <important> If you specify a health check configuration, you can specify either <code>HealthCheckCustomConfig</code> or <code>HealthCheckConfig</code> but not both. </important>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>HealthCheckCustomConfig</b>: A complex type that contains information about an optional custom health check. <important> If you specify a health check configuration, you can specify either <code>HealthCheckCustomConfig</code> or <code>HealthCheckConfig</code> but not both. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HealthCheckConfig</i>  <i>Public DNS namespaces only.</i> A complex type that contains settings for an optional health check. If you specify settings for a health check, AWS Cloud Map associates the health check with the records that you specify in <code>DnsConfig</code>. For information about the charges for health checks, see <a href="http://aws.amazon.com/route53/pricing/">Amazon Route 53 Pricing</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>HealthCheckConfig</b>:  <i>Public DNS namespaces only.</i> A complex type that contains settings for an optional health check. If you specify settings for a health check, AWS Cloud Map associates the health check with the records that you specify in <code>DnsConfig</code>. For information about the charges for health checks, see <a href="http://aws.amazon.com/route53/pricing/">Amazon Route 53 Pricing</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DnsConfig</i> A complex type that contains information about the Route 53 DNS records that you want AWS Cloud Map to create when you register an instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DnsConfig</b>: A complex type that contains information about the Route 53 DNS records that you want AWS Cloud Map to create when you register an instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCount</i> The number of instances that are currently associated with the service. Instances that were previously associated with the service but that have been deleted are not included in the count.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NamespaceId</i> The ID of the namespace that was used to create the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) that AWS Cloud Map assigns to the service when you create it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID that AWS Cloud Map assigned to the service when you created it.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DnsRecords</i> An array that contains one <code>DnsRecord</code> object for each Route 53 DNS record that you want AWS Cloud Map to create when you register an instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoutingPolicy</i> The routing policy that you want to apply to all Route 53 DNS records that AWS Cloud Map creates when you register an instance and specify this service. <note> If you want to use this service to register instances that create alias records, specify <code>WEIGHTED</code> for the routing policy. </note> You can specify the following values:  <b>MULTIVALUE</b>  If you define a health check for the service and the health check is healthy, Route 53 returns the applicable value for up to eight instances. For example, suppose the service includes configurations for one A record and a health check, and you use the service to register 10 instances. Route 53 responds to DNS queries with IP addresses for up to eight healthy instances. If fewer than eight instances are healthy, Route 53 responds to every DNS query with the IP addresses for all of the healthy instances. If you don't define a health check for the service, Route 53 assumes that all instances are healthy and returns the values for up to eight instances. For more information about the multivalue routing policy, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/routing-policy.html#routing-policy-multivalue">Multivalue Answer Routing</a> in the <i>Route 53 Developer Guide</i>.  <b>WEIGHTED</b>  Route 53 returns the applicable value from one randomly selected instance from among the instances that you registered using the same service. Currently, all records have the same weight, so you can't route more or less traffic to any instances. For example, suppose the service includes configurations for one A record and a health check, and you use the service to register 10 instances. Route 53 responds to DNS queries with the IP address for one randomly selected instance from among the healthy instances. If no instances are healthy, Route 53 responds to DNS queries as if all of the instances were healthy. If you don't define a health check for the service, Route 53 assumes that all instances are healthy and returns the applicable value for one randomly selected instance. For more information about the weighted routing policy, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/routing-policy.html#routing-policy-weighted">Weighted Routing</a> in the <i>Route 53 Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NamespaceId</i> The ID of the namespace to use for DNS configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureThreshold</i> The number of consecutive health checks that an endpoint must pass or fail for Route 53 to change the current status of the endpoint from unhealthy to healthy or vice versa. For more information, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/dns-failover-determining-health-of-endpoints.html">How Route 53 Determines Whether an Endpoint Is Healthy</a> in the <i>Route 53 Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourcePath</i> The path that you want Route 53 to request when performing health checks. The path can be any value for which your endpoint will return an HTTP status code of 2xx or 3xx when the endpoint is healthy, such as the file <code>/docs/route53-health-check.html</code>. Route 53 automatically adds the DNS name for the service. If you don't specify a value for <code>ResourcePath</code>, the default value is <code>/</code>. If you specify <code>TCP</code> for <code>Type</code>, you must <i>not</i> specify a value for <code>ResourcePath</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of health check that you want to create, which indicates how Route 53 determines whether an endpoint is healthy. <important> You can't change the value of <code>Type</code> after you create a health check. </important> You can create the following types of health checks: <ul> <li>  <b>HTTP</b>: Route 53 tries to establish a TCP connection. If successful, Route 53 submits an HTTP request and waits for an HTTP status code of 200 or greater and less than 400. </li> <li>  <b>HTTPS</b>: Route 53 tries to establish a TCP connection. If successful, Route 53 submits an HTTPS request and waits for an HTTP status code of 200 or greater and less than 400. <important> If you specify HTTPS for the value of <code>Type</code>, the endpoint must support TLS v1.0 or later. </important> </li> <li>  <b>TCP</b>: Route 53 tries to establish a TCP connection. If you specify <code>TCP</code> for <code>Type</code>, don't specify a value for <code>ResourcePath</code>. </li> </ul> For more information, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/dns-failover-determining-health-of-endpoints.html">How Route 53 Determines Whether an Endpoint Is Healthy</a> in the <i>Route 53 Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureThreshold</i> The number of 30-second intervals that you want Cloud Map to wait after receiving an <code>UpdateInstanceCustomHealthStatus</code> request before it changes the health status of a service instance. For example, suppose you specify a value of <code>2</code> for <code>FailureTheshold</code>, and then your application sends an <code>UpdateInstanceCustomHealthStatus</code> request. Cloud Map waits for approximately 60 seconds (2 x 30) before changing the status of the service instance based on that request. Sending a second or subsequent <code>UpdateInstanceCustomHealthStatus</code> request with the same value before <code>FailureThreshold x 30</code> seconds has passed doesn't accelerate the change. Cloud Map still waits <code>FailureThreshold x 30</code> seconds after the first request to make the change.<br/>        """
        pass

    def delete_namespace(self, Id: str) -> DeleteNamespaceResponse:
        """Deletes a namespace from the current account. If the namespace still contains one or more services, the request fails.

        :param self:
        :param Id:  The ID of the namespace that you want to delete.<br/>
        :return: OperationId:  A value that you can use to determine whether the request completed successfully. To get the status of the operation, see <a>GetOperation</a>.<br/>        """
        pass

    def delete_service(self, Id: str) -> DeleteServiceResponse:
        """Deletes a specified service. If the service still contains one or more registered instances, the request fails.

        :param self:
        :param Id:  The ID of the service that you want to delete.<br/>
        :return:         """
        pass

    def deregister_instance(self, InstanceId: str, ServiceId: str) -> DeregisterInstanceResponse:
        """Deletes the Amazon Route 53 DNS records and health check, if any, that AWS Cloud Map created for the specified instance.

        :param self:
        :param InstanceId:  The value that you specified for <code>Id</code> in the <a>RegisterInstance</a> request.<br/>ServiceId:  The ID of the service that the instance is associated with.<br/>
        :return: OperationId:  A value that you can use to determine whether the request completed successfully. For more information, see <a>GetOperation</a>.<br/>        """
        pass

    def discover_instances(self, ServiceName: str, NamespaceName: str, MaxResults: int=None, QueryParameters: Attributes=None, HealthStatus: str=None) -> DiscoverInstancesResponse:
        """Discovers registered instances for a specified namespace and service.

        :param self:
        :param HealthStatus:  The health status of the instances that you want to discover.<br/>QueryParameters:  A string map that contains attributes with values that you can use to filter instances by any custom attribute that you specified when you registered the instance. Only instances that match all the specified key/value pairs will be returned.<br/>MaxResults:  The maximum number of instances that you want Cloud Map to return in the response to a <code>DiscoverInstances</code> request. If you don't specify a value for <code>MaxResults</code>, Cloud Map returns up to 100 instances.<br/>ServiceName:  The name of the service that you specified when you registered the instance.<br/>NamespaceName:  The name of the namespace that you specified when you registered the instance.<br/>
        :return: Instances:  A complex type that contains one <code>HttpInstanceSummary</code> for each registered instance.<br/>        """
        pass

    def get_instance(self, InstanceId: str, ServiceId: str) -> GetInstanceResponse:
        """Gets information about a specified instance.

        :param self:
        :param InstanceId:  The ID of the instance that you want to get information about.<br/>ServiceId:  The ID of the service that the instance is associated with.<br/>
        :return: Instance:  A complex type that contains information about a specified instance.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Attributes</i> A string map that contains the following information for the service that you specify in <code>ServiceId</code>: <ul> <li> The attributes that apply to the records that are defined in the service.  </li> <li> For each attribute, the applicable value. </li> </ul> Supported attribute keys include the following:  <b>AWS_ALIAS_DNS_NAME</b>   <b/>  If you want AWS Cloud Map to create a Route 53 alias record that routes traffic to an Elastic Load Balancing load balancer, specify the DNS name that is associated with the load balancer. For information about how to get the DNS name, see "DNSName" in the topic <a href="http://docs.aws.amazon.com/Route53/latest/APIReference/API_AliasTarget.html">AliasTarget</a>. Note the following: <ul> <li> The configuration for the service that is specified by <code>ServiceId</code> must include settings for an A record, an AAAA record, or both. </li> <li> In the service that is specified by <code>ServiceId</code>, the value of <code>RoutingPolicy</code> must be <code>WEIGHTED</code>. </li> <li> If the service that is specified by <code>ServiceId</code> includes <code>HealthCheckConfig</code> settings, AWS Cloud Map will create the health check, but it won't associate the health check with the alias record. </li> <li> Auto naming currently doesn't support creating alias records that route traffic to AWS resources other than ELB load balancers. </li> <li> If you specify a value for <code>AWS_ALIAS_DNS_NAME</code>, don't specify values for any of the <code>AWS_INSTANCE</code> attributes. </li> </ul>  <b>AWS_INSTANCE_CNAME</b>  If the service configuration includes a CNAME record, the domain name that you want Route 53 to return in response to DNS queries, for example, <code>example.com</code>. This value is required if the service specified by <code>ServiceId</code> includes settings for an CNAME record.  <b>AWS_INSTANCE_IPV4</b>  If the service configuration includes an A record, the IPv4 address that you want Route 53 to return in response to DNS queries, for example, <code>192.0.2.44</code>. This value is required if the service specified by <code>ServiceId</code> includes settings for an A record. If the service includes settings for an SRV record, you must specify a value for <code>AWS_INSTANCE_IPV4</code>, <code>AWS_INSTANCE_IPV6</code>, or both.  <b>AWS_INSTANCE_IPV6</b>  If the service configuration includes an AAAA record, the IPv6 address that you want Route 53 to return in response to DNS queries, for example, <code>2001:0db8:85a3:0000:0000:abcd:0001:2345</code>. This value is required if the service specified by <code>ServiceId</code> includes settings for an AAAA record. If the service includes settings for an SRV record, you must specify a value for <code>AWS_INSTANCE_IPV4</code>, <code>AWS_INSTANCE_IPV6</code>, or both.  <b>AWS_INSTANCE_PORT</b>  If the service includes an SRV record, the value that you want Route 53 to return for the port. If the service includes <code>HealthCheckConfig</code>, the port on the endpoint that you want Route 53 to send requests to.  This value is required if you specified settings for an SRV record when you created the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatorRequestId</i> A unique string that identifies the request and that allows failed <code>RegisterInstance</code> requests to be retried without the risk of executing the operation twice. You must use a unique <code>CreatorRequestId</code> string every time you submit a <code>RegisterInstance</code> request if you're registering additional instances for the same namespace and service. <code>CreatorRequestId</code> can be any unique string, for example, a date/time stamp.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> An identifier that you want to associate with the instance. Note the following: <ul> <li> If the service that is specified by <code>ServiceId</code> includes settings for an SRV record, the value of <code>InstanceId</code> is automatically included as part of the value for the SRV record. For more information, see <a>DnsRecord$Type</a>. </li> <li> You can use this value to update an existing instance. </li> <li> To register a new instance, you must specify a value that is unique among instances that you register by using the same service.  </li> <li> If you specify an existing <code>InstanceId</code> and <code>ServiceId</code>, AWS Cloud Map updates the existing DNS records. If there's also an existing health check, AWS Cloud Map deletes the old health check and creates a new one.  <note> The health check isn't deleted immediately, so it will still appear for a while if you submit a <code>ListHealthChecks</code> request, for example. </note> </li> </ul><br/>        """
        pass

    def get_instances_health_status(self, ServiceId: str, Instances: List[ResourceId]=None, MaxResults: int=None, NextToken: str=None) -> GetInstancesHealthStatusResponse:
        """Gets the current health status (<code>Healthy</code>, <code>Unhealthy</code>, or <code>Unknown</code>) of one or more instances that are associated with a specified service. <note> There is a brief delay between when you register an instance and when the health status for the instance is available.  </note>

        :param self:
        :param NextToken:  For the first <code>GetInstancesHealthStatus</code> request, omit this value. If more than <code>MaxResults</code> instances match the specified criteria, you can submit another <code>GetInstancesHealthStatus</code> request to get the next group of results. Specify the value of <code>NextToken</code> from the previous response in the next request.<br/>MaxResults:  The maximum number of instances that you want AWS Cloud Map to return in the response to a <code>GetInstancesHealthStatus</code> request. If you don't specify a value for <code>MaxResults</code>, AWS Cloud Map returns up to 100 instances.<br/>Instances:  An array that contains the IDs of all the instances that you want to get the health status for. If you omit <code>Instances</code>, AWS Cloud Map returns the health status for all the instances that are associated with the specified service. <note> To get the IDs for the instances that you've registered by using a specified service, submit a <a>ListInstances</a> request. </note><br/>ServiceId:  The ID of the service that the instance is associated with.<br/>
        :return: NextToken:  If more than <code>MaxResults</code> instances match the specified criteria, you can submit another <code>GetInstancesHealthStatus</code> request to get the next group of results. Specify the value of <code>NextToken</code> from the previous response in the next request.<br/>Status:  A complex type that contains the IDs and the health status of the instances that you specified in the <code>GetInstancesHealthStatus</code> request.<br/>        """
        pass

    def get_namespace(self, Id: str) -> GetNamespaceResponse:
        """Gets information about a namespace.

        :param self:
        :param Id:  The ID of the namespace that you want to get information about.<br/>
        :return: Namespace:  A complex type that contains information about the specified namespace.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatorRequestId</i> A unique string that identifies the request and that allows failed requests to be retried without the risk of executing an operation twice. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateDate</i> The date that the namespace was created, in Unix date/time format and Coordinated Universal Time (UTC). The value of <code>CreateDate</code> is accurate to milliseconds. For example, the value <code>1516925490.087</code> represents Friday, January 26, 2018 12:11:30.087 AM.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Properties</i> A complex type that contains information that's specific to the type of the namespace.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Properties</b>: A complex type that contains information that's specific to the type of the namespace.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceCount</i> The number of services that are associated with the namespace.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description that you specify for the namespace when you create it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of the namespace. Valid values are <code>DNS_PUBLIC</code> and <code>DNS_PRIVATE</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the namespace, such as <code>example.com</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) that AWS Cloud Map assigns to the namespace when you create it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of a namespace.<br/>A complex type that contains the ID for the Route 53 hosted zone that AWS Cloud Map creates when you create a namespace.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DnsProperties</b>: A complex type that contains the ID for the Route 53 hosted zone that AWS Cloud Map creates when you create a namespace.        """
        pass

    def get_operation(self, OperationId: str) -> GetOperationResponse:
        """Gets information about any operation that returns an operation ID in the response, such as a <code>CreateService</code> request. <note> To get a list of operations that match specified criteria, see <a>ListOperations</a>. </note>

        :param self:
        :param OperationId:  The ID of the operation that you want to get more information about.<br/>
        :return: Operation:  A complex type that contains information about the operation.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Targets</i> The name of the target entity that is associated with the operation: <ul> <li>  <b>NAMESPACE</b>: The namespace ID is returned in the <code>ResourceId</code> property. </li> <li>  <b>SERVICE</b>: The service ID is returned in the <code>ResourceId</code> property. </li> <li>  <b>INSTANCE</b>: The instance ID is returned in the <code>ResourceId</code> property. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UpdateDate</i> The date and time that the value of <code>Status</code> changed to the current value, in Unix date/time format and Coordinated Universal Time (UTC). The value of <code>UpdateDate</code> is accurate to milliseconds. For example, the value <code>1516925490.087</code> represents Friday, January 26, 2018 12:11:30.087 AM.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateDate</i> The date and time that the request was submitted, in Unix date/time format and Coordinated Universal Time (UTC). The value of <code>CreateDate</code> is accurate to milliseconds. For example, the value <code>1516925490.087</code> represents Friday, January 26, 2018 12:11:30.087 AM.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorCode</i> The code associated with <code>ErrorMessage</code>. Values for <code>ErrorCode</code> include the following: <ul> <li>  <code>ACCESS_DENIED</code>  </li> <li>  <code>CANNOT_CREATE_HOSTED_ZONE</code>  </li> <li>  <code>EXPIRED_TOKEN</code>  </li> <li>  <code>HOSTED_ZONE_NOT_FOUND</code>  </li> <li>  <code>INTERNAL_FAILURE</code>  </li> <li>  <code>INVALID_CHANGE_BATCH</code>  </li> <li>  <code>THROTTLED_REQUEST</code>  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorMessage</i> If the value of <code>Status</code> is <code>FAIL</code>, the reason that the operation failed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the operation. Values include the following: <ul> <li>  <b>SUBMITTED</b>: This is the initial state immediately after you submit a request. </li> <li>  <b>PENDING</b>: AWS Cloud Map is performing the operation. </li> <li>  <b>SUCCESS</b>: The operation succeeded. </li> <li>  <b>FAIL</b>: The operation failed. For the failure reason, see <code>ErrorMessage</code>. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The name of the operation that is associated with the specified ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the operation that you want to get information about.<br/>        """
        pass

    def get_service(self, Id: str) -> GetServiceResponse:
        """Gets the settings for a specified service.

        :param self:
        :param Id:  The ID of the service that you want to get settings for.<br/>
        :return: Service:  A complex type that contains information about the service.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatorRequestId</i> A unique string that identifies the request and that allows failed requests to be retried without the risk of executing the operation twice. <code>CreatorRequestId</code> can be any unique string, for example, a date/time stamp.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreateDate</i> The date and time that the service was created, in Unix format and Coordinated Universal Time (UTC). The value of <code>CreateDate</code> is accurate to milliseconds. For example, the value <code>1516925490.087</code> represents Friday, January 26, 2018 12:11:30.087 AM.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HealthCheckCustomConfig</i> A complex type that contains information about an optional custom health check. <important> If you specify a health check configuration, you can specify either <code>HealthCheckCustomConfig</code> or <code>HealthCheckConfig</code> but not both. </important>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>HealthCheckCustomConfig</b>: A complex type that contains information about an optional custom health check. <important> If you specify a health check configuration, you can specify either <code>HealthCheckCustomConfig</code> or <code>HealthCheckConfig</code> but not both. </important><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HealthCheckConfig</i>  <i>Public DNS namespaces only.</i> A complex type that contains settings for an optional health check. If you specify settings for a health check, AWS Cloud Map associates the health check with the records that you specify in <code>DnsConfig</code>. For information about the charges for health checks, see <a href="http://aws.amazon.com/route53/pricing/">Amazon Route 53 Pricing</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>HealthCheckConfig</b>:  <i>Public DNS namespaces only.</i> A complex type that contains settings for an optional health check. If you specify settings for a health check, AWS Cloud Map associates the health check with the records that you specify in <code>DnsConfig</code>. For information about the charges for health checks, see <a href="http://aws.amazon.com/route53/pricing/">Amazon Route 53 Pricing</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DnsConfig</i> A complex type that contains information about the Route 53 DNS records that you want AWS Cloud Map to create when you register an instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DnsConfig</b>: A complex type that contains information about the Route 53 DNS records that you want AWS Cloud Map to create when you register an instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceCount</i> The number of instances that are currently associated with the service. Instances that were previously associated with the service but that have been deleted are not included in the count.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NamespaceId</i> The ID of the namespace that was used to create the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The Amazon Resource Name (ARN) that AWS Cloud Map assigns to the service when you create it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID that AWS Cloud Map assigned to the service when you created it.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DnsRecords</i> An array that contains one <code>DnsRecord</code> object for each Route 53 DNS record that you want AWS Cloud Map to create when you register an instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RoutingPolicy</i> The routing policy that you want to apply to all Route 53 DNS records that AWS Cloud Map creates when you register an instance and specify this service. <note> If you want to use this service to register instances that create alias records, specify <code>WEIGHTED</code> for the routing policy. </note> You can specify the following values:  <b>MULTIVALUE</b>  If you define a health check for the service and the health check is healthy, Route 53 returns the applicable value for up to eight instances. For example, suppose the service includes configurations for one A record and a health check, and you use the service to register 10 instances. Route 53 responds to DNS queries with IP addresses for up to eight healthy instances. If fewer than eight instances are healthy, Route 53 responds to every DNS query with the IP addresses for all of the healthy instances. If you don't define a health check for the service, Route 53 assumes that all instances are healthy and returns the values for up to eight instances. For more information about the multivalue routing policy, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/routing-policy.html#routing-policy-multivalue">Multivalue Answer Routing</a> in the <i>Route 53 Developer Guide</i>.  <b>WEIGHTED</b>  Route 53 returns the applicable value from one randomly selected instance from among the instances that you registered using the same service. Currently, all records have the same weight, so you can't route more or less traffic to any instances. For example, suppose the service includes configurations for one A record and a health check, and you use the service to register 10 instances. Route 53 responds to DNS queries with the IP address for one randomly selected instance from among the healthy instances. If no instances are healthy, Route 53 responds to DNS queries as if all of the instances were healthy. If you don't define a health check for the service, Route 53 assumes that all instances are healthy and returns the applicable value for one randomly selected instance. For more information about the weighted routing policy, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/routing-policy.html#routing-policy-weighted">Weighted Routing</a> in the <i>Route 53 Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NamespaceId</i> The ID of the namespace to use for DNS configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureThreshold</i> The number of consecutive health checks that an endpoint must pass or fail for Route 53 to change the current status of the endpoint from unhealthy to healthy or vice versa. For more information, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/dns-failover-determining-health-of-endpoints.html">How Route 53 Determines Whether an Endpoint Is Healthy</a> in the <i>Route 53 Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourcePath</i> The path that you want Route 53 to request when performing health checks. The path can be any value for which your endpoint will return an HTTP status code of 2xx or 3xx when the endpoint is healthy, such as the file <code>/docs/route53-health-check.html</code>. Route 53 automatically adds the DNS name for the service. If you don't specify a value for <code>ResourcePath</code>, the default value is <code>/</code>. If you specify <code>TCP</code> for <code>Type</code>, you must <i>not</i> specify a value for <code>ResourcePath</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of health check that you want to create, which indicates how Route 53 determines whether an endpoint is healthy. <important> You can't change the value of <code>Type</code> after you create a health check. </important> You can create the following types of health checks: <ul> <li>  <b>HTTP</b>: Route 53 tries to establish a TCP connection. If successful, Route 53 submits an HTTP request and waits for an HTTP status code of 200 or greater and less than 400. </li> <li>  <b>HTTPS</b>: Route 53 tries to establish a TCP connection. If successful, Route 53 submits an HTTPS request and waits for an HTTP status code of 200 or greater and less than 400. <important> If you specify HTTPS for the value of <code>Type</code>, the endpoint must support TLS v1.0 or later. </important> </li> <li>  <b>TCP</b>: Route 53 tries to establish a TCP connection. If you specify <code>TCP</code> for <code>Type</code>, don't specify a value for <code>ResourcePath</code>. </li> </ul> For more information, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/dns-failover-determining-health-of-endpoints.html">How Route 53 Determines Whether an Endpoint Is Healthy</a> in the <i>Route 53 Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureThreshold</i> The number of 30-second intervals that you want Cloud Map to wait after receiving an <code>UpdateInstanceCustomHealthStatus</code> request before it changes the health status of a service instance. For example, suppose you specify a value of <code>2</code> for <code>FailureTheshold</code>, and then your application sends an <code>UpdateInstanceCustomHealthStatus</code> request. Cloud Map waits for approximately 60 seconds (2 x 30) before changing the status of the service instance based on that request. Sending a second or subsequent <code>UpdateInstanceCustomHealthStatus</code> request with the same value before <code>FailureThreshold x 30</code> seconds has passed doesn't accelerate the change. Cloud Map still waits <code>FailureThreshold x 30</code> seconds after the first request to make the change.<br/>        """
        pass

    def list_instances(self, ServiceId: str, NextToken: str=None, MaxResults: int=None) -> ListInstancesResponse:
        """Lists summary information about the instances that you registered by using a specified service.

        :param self:
        :param MaxResults:  The maximum number of instances that you want AWS Cloud Map to return in the response to a <code>ListInstances</code> request. If you don't specify a value for <code>MaxResults</code>, AWS Cloud Map returns up to 100 instances.<br/>NextToken:  For the first <code>ListInstances</code> request, omit this value. If more than <code>MaxResults</code> instances match the specified criteria, you can submit another <code>ListInstances</code> request to get the next group of results. Specify the value of <code>NextToken</code> from the previous response in the next request.<br/>ServiceId:  The ID of the service that you want to list instances for.<br/>
        :return: NextToken:  If more than <code>MaxResults</code> instances match the specified criteria, you can submit another <code>ListInstances</code> request to get the next group of results. Specify the value of <code>NextToken</code> from the previous response in the next request.<br/>Instances:  Summary information about the instances that are associated with the specified service.<br/>        """
        pass

    def list_namespaces(self, NextToken: str=None, MaxResults: int=None, Filters: List[NamespaceFilter]=None) -> ListNamespacesResponse:
        """Lists summary information about the namespaces that were created by the current AWS account.

        :param self:
        :param Filters:  A complex type that contains specifications for the namespaces that you want to list. If you specify more than one filter, a namespace must match all filters to be returned by <code>ListNamespaces</code>.<br/>MaxResults:  The maximum number of namespaces that you want AWS Cloud Map to return in the response to a <code>ListNamespaces</code> request. If you don't specify a value for <code>MaxResults</code>, AWS Cloud Map returns up to 100 namespaces.<br/>NextToken:  For the first <code>ListNamespaces</code> request, omit this value. If the response contains <code>NextToken</code>, submit another <code>ListNamespaces</code> request to get the next group of results. Specify the value of <code>NextToken</code> from the previous response in the next request. <note> AWS Cloud Map gets <code>MaxResults</code> namespaces and then filters them based on the specified criteria. It's possible that no namespaces in the first <code>MaxResults</code> namespaces matched the specified criteria but that subsequent groups of <code>MaxResults</code> namespaces do contain namespaces that match the criteria. </note><br/>
        :return: NextToken:  If the response contains <code>NextToken</code>, submit another <code>ListNamespaces</code> request to get the next group of results. Specify the value of <code>NextToken</code> from the previous response in the next request. <note> AWS Cloud Map gets <code>MaxResults</code> namespaces and then filters them based on the specified criteria. It's possible that no namespaces in the first <code>MaxResults</code> namespaces matched the specified criteria but that subsequent groups of <code>MaxResults</code> namespaces do contain namespaces that match the criteria. </note><br/>Namespaces:  An array that contains one <code>NamespaceSummary</code> object for each namespace that matches the specified filter criteria.<br/>        """
        pass

    def list_operations(self, NextToken: str=None, MaxResults: int=None, Filters: List[OperationFilter]=None) -> ListOperationsResponse:
        """Lists operations that match the criteria that you specify.

        :param self:
        :param Filters:  A complex type that contains specifications for the operations that you want to list, for example, operations that you started between a specified start date and end date. If you specify more than one filter, an operation must match all filters to be returned by <code>ListOperations</code>.<br/>MaxResults:  The maximum number of items that you want AWS Cloud Map to return in the response to a <code>ListOperations</code> request. If you don't specify a value for <code>MaxResults</code>, AWS Cloud Map returns up to 100 operations.<br/>NextToken:  For the first <code>ListOperations</code> request, omit this value. If the response contains <code>NextToken</code>, submit another <code>ListOperations</code> request to get the next group of results. Specify the value of <code>NextToken</code> from the previous response in the next request. <note> AWS Cloud Map gets <code>MaxResults</code> operations and then filters them based on the specified criteria. It's possible that no operations in the first <code>MaxResults</code> operations matched the specified criteria but that subsequent groups of <code>MaxResults</code> operations do contain operations that match the criteria. </note><br/>
        :return: NextToken:  If the response contains <code>NextToken</code>, submit another <code>ListOperations</code> request to get the next group of results. Specify the value of <code>NextToken</code> from the previous response in the next request. <note> AWS Cloud Map gets <code>MaxResults</code> operations and then filters them based on the specified criteria. It's possible that no operations in the first <code>MaxResults</code> operations matched the specified criteria but that subsequent groups of <code>MaxResults</code> operations do contain operations that match the criteria. </note><br/>Operations:  Summary information about the operations that match the specified criteria.<br/>        """
        pass

    def list_services(self, NextToken: str=None, MaxResults: int=None, Filters: List[ServiceFilter]=None) -> ListServicesResponse:
        """Lists summary information for all the services that are associated with one or more specified namespaces.

        :param self:
        :param Filters:  A complex type that contains specifications for the namespaces that you want to list services for.  If you specify more than one filter, an operation must match all filters to be returned by <code>ListServices</code>.<br/>MaxResults:  The maximum number of services that you want AWS Cloud Map to return in the response to a <code>ListServices</code> request. If you don't specify a value for <code>MaxResults</code>, AWS Cloud Map returns up to 100 services.<br/>NextToken:  For the first <code>ListServices</code> request, omit this value. If the response contains <code>NextToken</code>, submit another <code>ListServices</code> request to get the next group of results. Specify the value of <code>NextToken</code> from the previous response in the next request. <note> AWS Cloud Map gets <code>MaxResults</code> services and then filters them based on the specified criteria. It's possible that no services in the first <code>MaxResults</code> services matched the specified criteria but that subsequent groups of <code>MaxResults</code> services do contain services that match the criteria. </note><br/>
        :return: NextToken:  If the response contains <code>NextToken</code>, submit another <code>ListServices</code> request to get the next group of results. Specify the value of <code>NextToken</code> from the previous response in the next request. <note> AWS Cloud Map gets <code>MaxResults</code> services and then filters them based on the specified criteria. It's possible that no services in the first <code>MaxResults</code> services matched the specified criteria but that subsequent groups of <code>MaxResults</code> services do contain services that match the criteria. </note><br/>Services:  An array that contains one <code>ServiceSummary</code> object for each service that matches the specified filter criteria.<br/>        """
        pass

    def register_instance(self, Attributes: Attributes, InstanceId: str, ServiceId: str, CreatorRequestId: str=None) -> RegisterInstanceResponse:
        """Creates or updates one or more records and, optionally, creates a health check based on the settings in a specified service. When you submit a <code>RegisterInstance</code> request, the following occurs: <ul> <li> For each DNS record that you define in the service that is specified by <code>ServiceId</code>, a record is created or updated in the hosted zone that is associated with the corresponding namespace. </li> <li> If the service includes <code>HealthCheckConfig</code>, a health check is created based on the settings in the health check configuration. </li> <li> The health check, if any, is associated with each of the new or updated records. </li> </ul> <important> One <code>RegisterInstance</code> request must complete before you can submit another request and specify the same service ID and instance ID. </important> For more information, see <a>CreateService</a>. When AWS Cloud Map receives a DNS query for the specified DNS name, it returns the applicable value: <ul> <li>  <b>If the health check is healthy</b>: returns all the records </li> <li>  <b>If the health check is unhealthy</b>: returns the applicable value for the last healthy instance </li> <li>  <b>If you didn't specify a health check configuration</b>: returns all the records </li> </ul> For the current limit on the number of instances that you can register using the same namespace and using the same service, see <a href="http://docs.aws.amazon.com/cloud-map/latest/dg/cloud-map-limits.html">AWS Cloud Map Limits</a> in the <i>AWS Cloud Map Developer Guide</i>.

        :param self:
        :param Attributes:  A string map that contains the following information for the service that you specify in <code>ServiceId</code>: <ul> <li> The attributes that apply to the records that are defined in the service.  </li> <li> For each attribute, the applicable value. </li> </ul> Supported attribute keys include the following:  <b>AWS_ALIAS_DNS_NAME</b>   <b/>  If you want AWS Cloud Map to create an Amazon Route 53 alias record that routes traffic to an Elastic Load Balancing load balancer, specify the DNS name that is associated with the load balancer. For information about how to get the DNS name, see "DNSName" in the topic <a href="http://docs.aws.amazon.com/Route53/latest/APIReference/API_AliasTarget.html">AliasTarget</a> in the <i>Route 53 API Reference</i>. Note the following: <ul> <li> The configuration for the service that is specified by <code>ServiceId</code> must include settings for an A record, an AAAA record, or both. </li> <li> In the service that is specified by <code>ServiceId</code>, the value of <code>RoutingPolicy</code> must be <code>WEIGHTED</code>. </li> <li> If the service that is specified by <code>ServiceId</code> includes <code>HealthCheckConfig</code> settings, AWS Cloud Map will create the Route 53 health check, but it won't associate the health check with the alias record. </li> <li> Auto naming currently doesn't support creating alias records that route traffic to AWS resources other than ELB load balancers. </li> <li> If you specify a value for <code>AWS_ALIAS_DNS_NAME</code>, don't specify values for any of the <code>AWS_INSTANCE</code> attributes. </li> </ul>  <b>AWS_INIT_HEALTH_STATUS</b>  If the service configuration includes <code>HealthCheckCustomConfig</code>, you can optionally use <code>AWS_INIT_HEALTH_STATUS</code> to specify the initial status of the custom health check, <code>HEALTHY</code> or <code>UNHEALTHY</code>. If you don't specify a value for <code>AWS_INIT_HEALTH_STATUS</code>, the initial status is <code>HEALTHY</code>.  <b>AWS_INSTANCE_CNAME</b>  If the service configuration includes a CNAME record, the domain name that you want Route 53 to return in response to DNS queries, for example, <code>example.com</code>. This value is required if the service specified by <code>ServiceId</code> includes settings for an CNAME record.  <b>AWS_INSTANCE_IPV4</b>  If the service configuration includes an A record, the IPv4 address that you want Route 53 to return in response to DNS queries, for example, <code>192.0.2.44</code>. This value is required if the service specified by <code>ServiceId</code> includes settings for an A record. If the service includes settings for an SRV record, you must specify a value for <code>AWS_INSTANCE_IPV4</code>, <code>AWS_INSTANCE_IPV6</code>, or both.  <b>AWS_INSTANCE_IPV6</b>  If the service configuration includes an AAAA record, the IPv6 address that you want Route 53 to return in response to DNS queries, for example, <code>2001:0db8:85a3:0000:0000:abcd:0001:2345</code>. This value is required if the service specified by <code>ServiceId</code> includes settings for an AAAA record. If the service includes settings for an SRV record, you must specify a value for <code>AWS_INSTANCE_IPV4</code>, <code>AWS_INSTANCE_IPV6</code>, or both.  <b>AWS_INSTANCE_PORT</b>  If the service includes an SRV record, the value that you want Route 53 to return for the port. If the service includes <code>HealthCheckConfig</code>, the port on the endpoint that you want Route 53 to send requests to.  This value is required if you specified settings for an SRV record when you created the service.  <b>Custom attributes</b>  You can add up to 30 custom attributes. For each key-value pair, the maximum length of the attribute name is 255 characters, and the maximum length of the attribute value is 1,024 characters. <br/>CreatorRequestId:  A unique string that identifies the request and that allows failed <code>RegisterInstance</code> requests to be retried without the risk of executing the operation twice. You must use a unique <code>CreatorRequestId</code> string every time you submit a <code>RegisterInstance</code> request if you're registering additional instances for the same namespace and service. <code>CreatorRequestId</code> can be any unique string, for example, a date/time stamp.<br/>InstanceId:  An identifier that you want to associate with the instance. Note the following: <ul> <li> If the service that is specified by <code>ServiceId</code> includes settings for an SRV record, the value of <code>InstanceId</code> is automatically included as part of the value for the SRV record. For more information, see <a>DnsRecord$Type</a>. </li> <li> You can use this value to update an existing instance. </li> <li> To register a new instance, you must specify a value that is unique among instances that you register by using the same service.  </li> <li> If you specify an existing <code>InstanceId</code> and <code>ServiceId</code>, AWS Cloud Map updates the existing DNS records, if any. If there's also an existing health check, AWS Cloud Map deletes the old health check and creates a new one.  <note> The health check isn't deleted immediately, so it will still appear for a while if you submit a <code>ListHealthChecks</code> request, for example. </note> </li> </ul><br/>ServiceId:  The ID of the service that you want to use for settings for the instance.<br/>
        :return: OperationId:  A value that you can use to determine whether the request completed successfully. To get the status of the operation, see <a>GetOperation</a>.<br/>        """
        pass

    def update_instance_custom_health_status(self, Status: str, InstanceId: str, ServiceId: str):
        """Submits a request to change the health status of a custom health check to healthy or unhealthy. You can use <code>UpdateInstanceCustomHealthStatus</code> to change the status only for custom health checks, which you define using <code>HealthCheckCustomConfig</code> when you create a service. You can't use it to change the status for Route 53 health checks, which you define using <code>HealthCheckConfig</code>. For more information, see <a>HealthCheckCustomConfig</a>.

        :param self:
        :param Status:  The new status of the instance, <code>HEALTHY</code> or <code>UNHEALTHY</code>.<br/>InstanceId:  The ID of the instance that you want to change the health status for.<br/>ServiceId:  The ID of the service that includes the configuration for the custom health check that you want to change the status for.<br/>
        :return:         """
        pass

    def update_service(self, Service: ServiceChange, Id: str) -> UpdateServiceResponse:
        """Submits a request to perform the following operations: <ul> <li> Add or delete <code>DnsRecords</code> configurations </li> <li> Update the TTL setting for existing <code>DnsRecords</code> configurations </li> <li> Add, update, or delete <code>HealthCheckConfig</code> for a specified service </li> </ul> For public and private DNS namespaces, you must specify all <code>DnsRecords</code> configurations (and, optionally, <code>HealthCheckConfig</code>) that you want to appear in the updated service. Any current configurations that don't appear in an <code>UpdateService</code> request are deleted. When you update the TTL setting for a service, AWS Cloud Map also updates the corresponding settings in all the records and health checks that were created by using the specified service.

        :param self:
        :param Service:  A complex type that contains the new settings for the service.:<br/>Id:  The ID of the service that you want to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HealthCheckConfig</i>  <i>Public DNS namespaces only.</i> A complex type that contains settings for an optional health check. If you specify settings for a health check, AWS Cloud Map associates the health check with the records that you specify in <code>DnsConfig</code>. <important> If you specify a health check configuration, you can specify either <code>HealthCheckCustomConfig</code> or <code>HealthCheckConfig</code> but not both. </important> Health checks are basic Route 53 health checks that monitor an AWS endpoint. For information about pricing for health checks, see <a href="http://aws.amazon.com/route53/pricing/">Amazon Route 53 Pricing</a>. Note the following about configuring health checks.  <b>A and AAAA records</b>  If <code>DnsConfig</code> includes configurations for both A and AAAA records, AWS Cloud Map creates a health check that uses the IPv4 address to check the health of the resource. If the endpoint that is specified by the IPv4 address is unhealthy, Route 53 considers both the A and AAAA records to be unhealthy.   <b>CNAME records</b>  You can't specify settings for <code>HealthCheckConfig</code> when the <code>DNSConfig</code> includes <code>CNAME</code> for the value of <code>Type</code>. If you do, the <code>CreateService</code> request will fail with an <code>InvalidInput</code> error.  <b>Request interval</b>  A Route 53 health checker in each health-checking region sends a health check request to an endpoint every 30 seconds. On average, your endpoint receives a health check request about every two seconds. However, health checkers don't coordinate with one another, so you'll sometimes see several requests per second followed by a few seconds with no health checks at all.  <b>Health checking regions</b>  Health checkers perform checks from all Route 53 health-checking regions. For a list of the current regions, see <a href="http://docs.aws.amazon.com/Route53/latest/APIReference/API_HealthCheckConfig.html#Route53-Type-HealthCheckConfig-Regions">Regions</a>.  <b>Alias records</b>  When you register an instance, if you include the <code>AWS_ALIAS_DNS_NAME</code> attribute, AWS Cloud Map creates a Route 53 alias record. Note the following: <ul> <li> Route 53 automatically sets <code>EvaluateTargetHealth</code> to true for alias records. When <code>EvaluateTargetHealth</code> is true, the alias record inherits the health of the referenced AWS resource. such as an ELB load balancer. For more information, see <a href="http://docs.aws.amazon.com/Route53/latest/APIReference/API_AliasTarget.html#Route53-Type-AliasTarget-EvaluateTargetHealth">EvaluateTargetHealth</a>. </li> <li> If you include <code>HealthCheckConfig</code> and then use the service to register an instance that creates an alias record, Route 53 doesn't create the health check. </li> </ul>  <b>Charges for health checks</b>  Health checks are basic Route 53 health checks that monitor an AWS endpoint. For information about pricing for health checks, see <a href="http://aws.amazon.com/route53/pricing/">Amazon Route 53 Pricing</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>HealthCheckConfig</b>:  <i>Public DNS namespaces only.</i> A complex type that contains settings for an optional health check. If you specify settings for a health check, AWS Cloud Map associates the health check with the records that you specify in <code>DnsConfig</code>. <important> If you specify a health check configuration, you can specify either <code>HealthCheckCustomConfig</code> or <code>HealthCheckConfig</code> but not both. </important> Health checks are basic Route 53 health checks that monitor an AWS endpoint. For information about pricing for health checks, see <a href="http://aws.amazon.com/route53/pricing/">Amazon Route 53 Pricing</a>. Note the following about configuring health checks.  <b>A and AAAA records</b>  If <code>DnsConfig</code> includes configurations for both A and AAAA records, AWS Cloud Map creates a health check that uses the IPv4 address to check the health of the resource. If the endpoint that is specified by the IPv4 address is unhealthy, Route 53 considers both the A and AAAA records to be unhealthy.   <b>CNAME records</b>  You can't specify settings for <code>HealthCheckConfig</code> when the <code>DNSConfig</code> includes <code>CNAME</code> for the value of <code>Type</code>. If you do, the <code>CreateService</code> request will fail with an <code>InvalidInput</code> error.  <b>Request interval</b>  A Route 53 health checker in each health-checking region sends a health check request to an endpoint every 30 seconds. On average, your endpoint receives a health check request about every two seconds. However, health checkers don't coordinate with one another, so you'll sometimes see several requests per second followed by a few seconds with no health checks at all.  <b>Health checking regions</b>  Health checkers perform checks from all Route 53 health-checking regions. For a list of the current regions, see <a href="http://docs.aws.amazon.com/Route53/latest/APIReference/API_HealthCheckConfig.html#Route53-Type-HealthCheckConfig-Regions">Regions</a>.  <b>Alias records</b>  When you register an instance, if you include the <code>AWS_ALIAS_DNS_NAME</code> attribute, AWS Cloud Map creates a Route 53 alias record. Note the following: <ul> <li> Route 53 automatically sets <code>EvaluateTargetHealth</code> to true for alias records. When <code>EvaluateTargetHealth</code> is true, the alias record inherits the health of the referenced AWS resource. such as an ELB load balancer. For more information, see <a href="http://docs.aws.amazon.com/Route53/latest/APIReference/API_AliasTarget.html#Route53-Type-AliasTarget-EvaluateTargetHealth">EvaluateTargetHealth</a>. </li> <li> If you include <code>HealthCheckConfig</code> and then use the service to register an instance that creates an alias record, Route 53 doesn't create the health check. </li> </ul>  <b>Charges for health checks</b>  Health checks are basic Route 53 health checks that monitor an AWS endpoint. For information about pricing for health checks, see <a href="http://aws.amazon.com/route53/pricing/">Amazon Route 53 Pricing</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DnsConfig</i> A complex type that contains information about the Route 53 DNS records that you want AWS Cloud Map to create when you register an instance.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DnsConfig</b>: A complex type that contains information about the Route 53 DNS records that you want AWS Cloud Map to create when you register an instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description for the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DnsRecords</i> An array that contains one <code>DnsRecord</code> object for each Route 53 record that you want AWS Cloud Map to create when you register an instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureThreshold</i> The number of consecutive health checks that an endpoint must pass or fail for Route 53 to change the current status of the endpoint from unhealthy to healthy or vice versa. For more information, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/dns-failover-determining-health-of-endpoints.html">How Route 53 Determines Whether an Endpoint Is Healthy</a> in the <i>Route 53 Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourcePath</i> The path that you want Route 53 to request when performing health checks. The path can be any value for which your endpoint will return an HTTP status code of 2xx or 3xx when the endpoint is healthy, such as the file <code>/docs/route53-health-check.html</code>. Route 53 automatically adds the DNS name for the service. If you don't specify a value for <code>ResourcePath</code>, the default value is <code>/</code>. If you specify <code>TCP</code> for <code>Type</code>, you must <i>not</i> specify a value for <code>ResourcePath</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of health check that you want to create, which indicates how Route 53 determines whether an endpoint is healthy. <important> You can't change the value of <code>Type</code> after you create a health check. </important> You can create the following types of health checks: <ul> <li>  <b>HTTP</b>: Route 53 tries to establish a TCP connection. If successful, Route 53 submits an HTTP request and waits for an HTTP status code of 200 or greater and less than 400. </li> <li>  <b>HTTPS</b>: Route 53 tries to establish a TCP connection. If successful, Route 53 submits an HTTPS request and waits for an HTTP status code of 200 or greater and less than 400. <important> If you specify HTTPS for the value of <code>Type</code>, the endpoint must support TLS v1.0 or later. </important> </li> <li>  <b>TCP</b>: Route 53 tries to establish a TCP connection. If you specify <code>TCP</code> for <code>Type</code>, don't specify a value for <code>ResourcePath</code>. </li> </ul> For more information, see <a href="http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/dns-failover-determining-health-of-endpoints.html">How Route 53 Determines Whether an Endpoint Is Healthy</a> in the <i>Route 53 Developer Guide</i>.<br/>
        :return: OperationId:  A value that you can use to determine whether the request completed successfully. To get the status of the operation, see <a>GetOperation</a>.<br/>        """
        pass



class SES(BaseClient):
    """<fullname>Amazon Simple Email Service</fullname>  This document contains reference information for the <a href="https://aws.amazon.com/ses/">Amazon Simple Email Service</a> (Amazon SES) API, version 2010-12-01. This document is best used in conjunction with the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/Welcome.html">Amazon SES Developer Guide</a>.  <note>  For a list of Amazon SES endpoints to use in service requests, see <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/regions.html">Regions and Amazon SES</a> in the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/Welcome.html">Amazon SES Developer Guide</a>. </note>"""

    class IdentityExistsWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `get_identity_verification_attributes` for valid parameters"""
            pass


    class ListIdentitiesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_identities` for valid parameters"""
            pass
    class ListCustomVerificationEmailTemplatesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_custom_verification_email_templates` for valid parameters"""
            pass
    class ListConfigurationSetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_configuration_sets` for valid parameters"""
            pass
    class ListReceiptRuleSetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_receipt_rule_sets` for valid parameters"""
            pass
    class ListTemplatesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_templates` for valid parameters"""
            pass


    class CloneReceiptRuleSetRequest(Mapping):
        pass
    
    class CloneReceiptRuleSetResponse(Mapping):
        pass
    
    class CreateConfigurationSetRequest(Mapping):
        pass
    
    class CreateConfigurationSetResponse(Mapping):
        pass
    
    class ConfigurationSet(Mapping):
        pass
    
    class CreateConfigurationSetEventDestinationRequest(Mapping):
        pass
    
    class CreateConfigurationSetEventDestinationResponse(Mapping):
        pass
    
    class EventDestination(Mapping):
        pass
    
    class CreateConfigurationSetTrackingOptionsRequest(Mapping):
        pass
    
    class CreateConfigurationSetTrackingOptionsResponse(Mapping):
        pass
    
    class TrackingOptions(Mapping):
        pass
    
    class CreateCustomVerificationEmailTemplateRequest(Mapping):
        pass
    
    class CreateReceiptFilterRequest(Mapping):
        pass
    
    class CreateReceiptFilterResponse(Mapping):
        pass
    
    class ReceiptFilter(Mapping):
        pass
    
    class CreateReceiptRuleRequest(Mapping):
        pass
    
    class CreateReceiptRuleResponse(Mapping):
        pass
    
    class ReceiptRule(Mapping):
        pass
    
    class CreateReceiptRuleSetRequest(Mapping):
        pass
    
    class CreateReceiptRuleSetResponse(Mapping):
        pass
    
    class CreateTemplateRequest(Mapping):
        pass
    
    class CreateTemplateResponse(Mapping):
        pass
    
    class Template(Mapping):
        pass
    
    class DeleteConfigurationSetRequest(Mapping):
        pass
    
    class DeleteConfigurationSetResponse(Mapping):
        pass
    
    class DeleteConfigurationSetEventDestinationRequest(Mapping):
        pass
    
    class DeleteConfigurationSetEventDestinationResponse(Mapping):
        pass
    
    class DeleteConfigurationSetTrackingOptionsRequest(Mapping):
        pass
    
    class DeleteConfigurationSetTrackingOptionsResponse(Mapping):
        pass
    
    class DeleteCustomVerificationEmailTemplateRequest(Mapping):
        pass
    
    class DeleteIdentityRequest(Mapping):
        pass
    
    class DeleteIdentityResponse(Mapping):
        pass
    
    class DeleteIdentityPolicyRequest(Mapping):
        pass
    
    class DeleteIdentityPolicyResponse(Mapping):
        pass
    
    class DeleteReceiptFilterRequest(Mapping):
        pass
    
    class DeleteReceiptFilterResponse(Mapping):
        pass
    
    class DeleteReceiptRuleRequest(Mapping):
        pass
    
    class DeleteReceiptRuleResponse(Mapping):
        pass
    
    class DeleteReceiptRuleSetRequest(Mapping):
        pass
    
    class DeleteReceiptRuleSetResponse(Mapping):
        pass
    
    class DeleteTemplateRequest(Mapping):
        pass
    
    class DeleteTemplateResponse(Mapping):
        pass
    
    class DeleteVerifiedEmailAddressRequest(Mapping):
        pass
    
    class DescribeActiveReceiptRuleSetRequest(Mapping):
        pass
    
    class DescribeActiveReceiptRuleSetResponse(Mapping):
        pass
    
    class DescribeConfigurationSetRequest(Mapping):
        pass
    
    class DescribeConfigurationSetResponse(Mapping):
        pass
    
    class ConfigurationSetAttribute(object):
        pass
    
    class DescribeReceiptRuleRequest(Mapping):
        pass
    
    class DescribeReceiptRuleResponse(Mapping):
        pass
    
    class DescribeReceiptRuleSetRequest(Mapping):
        pass
    
    class DescribeReceiptRuleSetResponse(Mapping):
        pass
    
    class GetAccountSendingEnabledResponse(Mapping):
        pass
    
    class GetCustomVerificationEmailTemplateRequest(Mapping):
        pass
    
    class GetCustomVerificationEmailTemplateResponse(Mapping):
        pass
    
    class GetIdentityDkimAttributesRequest(Mapping):
        pass
    
    class GetIdentityDkimAttributesResponse(Mapping):
        pass
    
    class Identity(object):
        pass
    
    class GetIdentityMailFromDomainAttributesRequest(Mapping):
        pass
    
    class GetIdentityMailFromDomainAttributesResponse(Mapping):
        pass
    
    class GetIdentityNotificationAttributesRequest(Mapping):
        pass
    
    class GetIdentityNotificationAttributesResponse(Mapping):
        pass
    
    class GetIdentityPoliciesRequest(Mapping):
        pass
    
    class GetIdentityPoliciesResponse(Mapping):
        pass
    
    class PolicyName(object):
        pass
    
    class GetIdentityVerificationAttributesRequest(Mapping):
        pass
    
    class GetIdentityVerificationAttributesResponse(Mapping):
        pass
    
    class GetSendQuotaResponse(Mapping):
        pass
    
    class GetSendStatisticsResponse(Mapping):
        pass
    
    class GetTemplateRequest(Mapping):
        pass
    
    class GetTemplateResponse(Mapping):
        pass
    
    class ListConfigurationSetsRequest(Mapping):
        pass
    
    class ListConfigurationSetsResponse(Mapping):
        pass
    
    class ListCustomVerificationEmailTemplatesRequest(Mapping):
        pass
    
    class ListCustomVerificationEmailTemplatesResponse(Mapping):
        pass
    
    class ListIdentitiesRequest(Mapping):
        pass
    
    class ListIdentitiesResponse(Mapping):
        pass
    
    class ListIdentityPoliciesRequest(Mapping):
        pass
    
    class ListIdentityPoliciesResponse(Mapping):
        pass
    
    class ListReceiptFiltersRequest(Mapping):
        pass
    
    class ListReceiptFiltersResponse(Mapping):
        pass
    
    class ListReceiptRuleSetsRequest(Mapping):
        pass
    
    class ListReceiptRuleSetsResponse(Mapping):
        pass
    
    class ListTemplatesRequest(Mapping):
        pass
    
    class ListTemplatesResponse(Mapping):
        pass
    
    class ListVerifiedEmailAddressesResponse(Mapping):
        pass
    
    class PutConfigurationSetDeliveryOptionsRequest(Mapping):
        pass
    
    class PutConfigurationSetDeliveryOptionsResponse(Mapping):
        pass
    
    class DeliveryOptions(Mapping):
        pass
    
    class PutIdentityPolicyRequest(Mapping):
        pass
    
    class PutIdentityPolicyResponse(Mapping):
        pass
    
    class ReorderReceiptRuleSetRequest(Mapping):
        pass
    
    class ReorderReceiptRuleSetResponse(Mapping):
        pass
    
    class ReceiptRuleName(object):
        pass
    
    class SendBounceRequest(Mapping):
        pass
    
    class SendBounceResponse(Mapping):
        pass
    
    class MessageDsn(Mapping):
        pass
    
    class BouncedRecipientInfo(Mapping):
        pass
    
    class SendBulkTemplatedEmailRequest(Mapping):
        pass
    
    class SendBulkTemplatedEmailResponse(Mapping):
        pass
    
    class Address(object):
        pass
    
    class MessageTag(Mapping):
        pass
    
    class BulkEmailDestination(Mapping):
        pass
    
    class SendCustomVerificationEmailRequest(Mapping):
        pass
    
    class SendCustomVerificationEmailResponse(Mapping):
        pass
    
    class SendEmailRequest(Mapping):
        pass
    
    class SendEmailResponse(Mapping):
        pass
    
    class Destination(Mapping):
        pass
    
    class Message(Mapping):
        pass
    
    class SendRawEmailRequest(Mapping):
        pass
    
    class SendRawEmailResponse(Mapping):
        pass
    
    class RawMessage(Mapping):
        pass
    
    class SendTemplatedEmailRequest(Mapping):
        pass
    
    class SendTemplatedEmailResponse(Mapping):
        pass
    
    class SetActiveReceiptRuleSetRequest(Mapping):
        pass
    
    class SetActiveReceiptRuleSetResponse(Mapping):
        pass
    
    class SetIdentityDkimEnabledRequest(Mapping):
        pass
    
    class SetIdentityDkimEnabledResponse(Mapping):
        pass
    
    class Enabled(object):
        pass
    
    class SetIdentityFeedbackForwardingEnabledRequest(Mapping):
        pass
    
    class SetIdentityFeedbackForwardingEnabledResponse(Mapping):
        pass
    
    class SetIdentityHeadersInNotificationsEnabledRequest(Mapping):
        pass
    
    class SetIdentityHeadersInNotificationsEnabledResponse(Mapping):
        pass
    
    class SetIdentityMailFromDomainRequest(Mapping):
        pass
    
    class SetIdentityMailFromDomainResponse(Mapping):
        pass
    
    class SetIdentityNotificationTopicRequest(Mapping):
        pass
    
    class SetIdentityNotificationTopicResponse(Mapping):
        pass
    
    class SetReceiptRulePositionRequest(Mapping):
        pass
    
    class SetReceiptRulePositionResponse(Mapping):
        pass
    
    class TestRenderTemplateRequest(Mapping):
        pass
    
    class TestRenderTemplateResponse(Mapping):
        pass
    
    class UpdateAccountSendingEnabledRequest(Mapping):
        pass
    
    class UpdateConfigurationSetEventDestinationRequest(Mapping):
        pass
    
    class UpdateConfigurationSetEventDestinationResponse(Mapping):
        pass
    
    class UpdateConfigurationSetReputationMetricsEnabledRequest(Mapping):
        pass
    
    class UpdateConfigurationSetSendingEnabledRequest(Mapping):
        pass
    
    class UpdateConfigurationSetTrackingOptionsRequest(Mapping):
        pass
    
    class UpdateConfigurationSetTrackingOptionsResponse(Mapping):
        pass
    
    class UpdateCustomVerificationEmailTemplateRequest(Mapping):
        pass
    
    class UpdateReceiptRuleRequest(Mapping):
        pass
    
    class UpdateReceiptRuleResponse(Mapping):
        pass
    
    class UpdateTemplateRequest(Mapping):
        pass
    
    class UpdateTemplateResponse(Mapping):
        pass
    
    class VerifyDomainDkimRequest(Mapping):
        pass
    
    class VerifyDomainDkimResponse(Mapping):
        pass
    
    class VerifyDomainIdentityRequest(Mapping):
        pass
    
    class VerifyDomainIdentityResponse(Mapping):
        pass
    
    class VerifyEmailAddressRequest(Mapping):
        pass
    
    class VerifyEmailIdentityRequest(Mapping):
        pass
    
    class VerifyEmailIdentityResponse(Mapping):
        pass
    
    def clone_receipt_rule_set(self, OriginalRuleSetName: str, RuleSetName: str) -> CloneReceiptRuleSetResponse:
        """Creates a receipt rule set by cloning an existing one. All receipt rules and configurations are copied to the new receipt rule set and are completely independent of the source rule set. For information about setting up rule sets, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/receiving-email-receipt-rule-set.html">Amazon SES Developer Guide</a>. You can execute this operation no more than once per second.

        :param self:
        :param OriginalRuleSetName:  The name of the rule set to clone.<br/>RuleSetName:  The name of the rule set to create. The name must: <ul> <li> This value can only contain ASCII letters (a-z, A-Z), numbers (0-9), underscores (_), or dashes (-). </li> <li> Start and end with a letter or number. </li> <li> Contain less than 64 characters. </li> </ul><br/>
        :return:         """
        pass

    def create_configuration_set(self, ConfigurationSet: ConfigurationSet) -> CreateConfigurationSetResponse:
        """Creates a configuration set. Configuration sets enable you to publish email sending events. For information about using configuration sets, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/monitor-sending-activity.html">Amazon SES Developer Guide</a>. You can execute this operation no more than once per second.

        :param self:
        :param ConfigurationSet:  A data structure that contains the name of the configuration set.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the configuration set. The name must meet the following requirements: <ul> <li> Contain only letters (a-z, A-Z), numbers (0-9), underscores (_), or dashes (-). </li> <li> Contain 64 characters or fewer. </li> </ul><br/>
        :return:         """
        pass

    def create_configuration_set_event_destination(self, EventDestination: EventDestination, ConfigurationSetName: str) -> CreateConfigurationSetEventDestinationResponse:
        """Creates a configuration set event destination. <note> When you create or update an event destination, you must provide one, and only one, destination. The destination can be CloudWatch, Amazon Kinesis Firehose, or Amazon Simple Notification Service (Amazon SNS). </note> An event destination is the AWS service to which Amazon SES publishes the email sending events associated with a configuration set. For information about using configuration sets, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/monitor-sending-activity.html">Amazon SES Developer Guide</a>. You can execute this operation no more than once per second.

        :param self:
        :param EventDestination:  An object that describes the AWS service that email sending event information will be published to.:<br/>ConfigurationSetName:  The name of the configuration set that the event destination should be associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SNSDestination</i> An object that contains the topic ARN associated with an Amazon Simple Notification Service (Amazon SNS) event destination.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SNSDestination</b>: An object that contains the topic ARN associated with an Amazon Simple Notification Service (Amazon SNS) event destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchDestination</i> An object that contains the names, default values, and sources of the dimensions associated with an Amazon CloudWatch event destination.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CloudWatchDestination</b>: An object that contains the names, default values, and sources of the dimensions associated with an Amazon CloudWatch event destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KinesisFirehoseDestination</i> An object that contains the delivery stream ARN and the IAM role ARN associated with an Amazon Kinesis Firehose event destination.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KinesisFirehoseDestination</b>: An object that contains the delivery stream ARN and the IAM role ARN associated with an Amazon Kinesis Firehose event destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MatchingEventTypes</i> The type of email sending events to publish to the event destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Sets whether Amazon SES publishes events to this destination when you send an email with the associated configuration set. Set to <code>true</code> to enable publishing to this destination; set to <code>false</code> to prevent publishing to this destination. The default value is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the event destination. The name must: <ul> <li> This value can only contain ASCII letters (a-z, A-Z), numbers (0-9), underscores (_), or dashes (-). </li> <li> Contain less than 64 characters. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DeliveryStreamARN</i> The ARN of the Amazon Kinesis Firehose stream that email sending events should be published to.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMRoleARN</i> The ARN of the IAM role under which Amazon SES publishes email sending events to the Amazon Kinesis Firehose stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DimensionConfigurations</i> A list of dimensions upon which to categorize your emails when you publish email sending events to Amazon CloudWatch.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicARN</i> The ARN of the Amazon SNS topic that email sending events will be published to. An example of an Amazon SNS topic ARN is <code>arn:aws:sns:us-west-2:123456789012:MyTopic</code>. For more information about Amazon SNS topics, see the <a href="https://docs.aws.amazon.com/sns/latest/dg/CreateTopic.html">Amazon SNS Developer Guide</a>.<br/>
        :return:         """
        pass

    def create_configuration_set_tracking_options(self, TrackingOptions: TrackingOptions, ConfigurationSetName: str) -> CreateConfigurationSetTrackingOptionsResponse:
        """Creates an association between a configuration set and a custom domain for open and click event tracking.  By default, images and links used for tracking open and click events are hosted on domains operated by Amazon SES. You can configure a subdomain of your own to handle these events. For information about using custom domains, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/configure-custom-open-click-domains.html">Amazon SES Developer Guide</a>.

        :param self:
        :param TrackingOptions:  A domain that is used to redirect email recipients to an Amazon SES-operated domain. This domain captures open and click events generated by Amazon SES emails. For more information, see <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/configure-custom-open-click-domains.html">Configuring Custom Domains to Handle Open and Click Tracking</a> in the <i>Amazon SES Developer Guide</i>.:<br/>ConfigurationSetName:  The name of the configuration set that the tracking options should be associated with.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomRedirectDomain</i> The custom subdomain that will be used to redirect email recipients to the Amazon SES event tracking domain.<br/>
        :return:         """
        pass

    def create_custom_verification_email_template(self, FailureRedirectionURL: str, SuccessRedirectionURL: str, TemplateContent: str, TemplateSubject: str, FromEmailAddress: str, TemplateName: str):
        """Creates a new custom verification email template. For more information about custom verification email templates, see <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/custom-verification-emails.html">Using Custom Verification Email Templates</a> in the <i>Amazon SES Developer Guide</i>. You can execute this operation no more than once per second.

        :param self:
        :param FailureRedirectionURL:  The URL that the recipient of the verification email is sent to if his or her address is not successfully verified.<br/>SuccessRedirectionURL:  The URL that the recipient of the verification email is sent to if his or her address is successfully verified.<br/>TemplateContent:  The content of the custom verification email. The total size of the email must be less than 10 MB. The message body may contain HTML, with some limitations. For more information, see <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/custom-verification-emails.html#custom-verification-emails-faq">Custom Verification Email Frequently Asked Questions</a> in the <i>Amazon SES Developer Guide</i>.<br/>TemplateSubject:  The subject line of the custom verification email.<br/>FromEmailAddress:  The email address that the custom verification email is sent from.<br/>TemplateName:  The name of the custom verification email template.<br/>
        :return:         """
        pass

    def create_receipt_filter(self, Filter: ReceiptFilter) -> CreateReceiptFilterResponse:
        """Creates a new IP address filter. For information about setting up IP address filters, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/receiving-email-ip-filters.html">Amazon SES Developer Guide</a>. You can execute this operation no more than once per second.

        :param self:
        :param Filter:  A data structure that describes the IP address filter to create, which consists of a name, an IP address range, and whether to allow or block mail from it.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IpFilter</i> A structure that provides the IP addresses to block or allow, and whether to block or allow incoming mail from them.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>IpFilter</b>: A structure that provides the IP addresses to block or allow, and whether to block or allow incoming mail from them.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the IP address filter. The name must: <ul> <li> This value can only contain ASCII letters (a-z, A-Z), numbers (0-9), underscores (_), or dashes (-). </li> <li> Start and end with a letter or number. </li> <li> Contain less than 64 characters. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Cidr</i> A single IP address or a range of IP addresses that you want to block or allow, specified in Classless Inter-Domain Routing (CIDR) notation. An example of a single email address is 10.0.0.1. An example of a range of IP addresses is 10.0.0.1/24. For more information about CIDR notation, see <a href="https://tools.ietf.org/html/rfc2317">RFC 2317</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Policy</i> Indicates whether to block or allow incoming mail from the specified IP addresses.<br/>
        :return:         """
        pass

    def create_receipt_rule(self, Rule: ReceiptRule, RuleSetName: str, After: str=None) -> CreateReceiptRuleResponse:
        """Creates a receipt rule. For information about setting up receipt rules, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/receiving-email-receipt-rules.html">Amazon SES Developer Guide</a>. You can execute this operation no more than once per second.

        :param self:
        :param Rule:  A data structure that contains the specified rule's name, actions, recipients, domains, enabled status, scan status, and TLS policy.:<br/>After:  The name of an existing rule after which the new rule will be placed. If this parameter is null, the new rule will be inserted at the beginning of the rule list.<br/>RuleSetName:  The name of the rule set that the receipt rule will be added to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ScanEnabled</i> If <code>true</code>, then messages that this receipt rule applies to are scanned for spam and viruses. The default value is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Actions</i> An ordered list of actions to perform on messages that match at least one of the recipient email addresses or domains specified in the receipt rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Recipients</i> The recipient domains and email addresses that the receipt rule applies to. If this field is not specified, this rule will match all recipients under all verified domains.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TlsPolicy</i> Specifies whether Amazon SES should require that incoming email is delivered over a connection encrypted with Transport Layer Security (TLS). If this parameter is set to <code>Require</code>, Amazon SES will bounce emails that are not received over TLS. The default is <code>Optional</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> If <code>true</code>, the receipt rule is active. The default value is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the receipt rule. The name must: <ul> <li> This value can only contain ASCII letters (a-z, A-Z), numbers (0-9), underscores (_), or dashes (-). </li> <li> Start and end with a letter or number. </li> <li> Contain less than 64 characters. </li> </ul><br/>
        :return:         """
        pass

    def create_receipt_rule_set(self, RuleSetName: str) -> CreateReceiptRuleSetResponse:
        """Creates an empty receipt rule set. For information about setting up receipt rule sets, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/receiving-email-receipt-rule-set.html">Amazon SES Developer Guide</a>. You can execute this operation no more than once per second.

        :param self:
        :param RuleSetName:  The name of the rule set to create. The name must: <ul> <li> This value can only contain ASCII letters (a-z, A-Z), numbers (0-9), underscores (_), or dashes (-). </li> <li> Start and end with a letter or number. </li> <li> Contain less than 64 characters. </li> </ul><br/>
        :return:         """
        pass

    def create_template(self, Template: Template) -> CreateTemplateResponse:
        """Creates an email template. Email templates enable you to send personalized email to one or more destinations in a single API operation. For more information, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/send-personalized-email-api.html">Amazon SES Developer Guide</a>. You can execute this operation no more than once per second.

        :param self:
        :param Template:  The content of the email, composed of a subject line, an HTML part, and a text-only part.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HtmlPart</i> The HTML body of the email.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TextPart</i> The email body that will be visible to recipients whose email clients do not display HTML.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubjectPart</i> The subject line of the email.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TemplateName</i> The name of the template. You will refer to this name when you send email using the <code>SendTemplatedEmail</code> or <code>SendBulkTemplatedEmail</code> operations.<br/>
        :return:         """
        pass

    def delete_configuration_set(self, ConfigurationSetName: str) -> DeleteConfigurationSetResponse:
        """Deletes a configuration set. Configuration sets enable you to publish email sending events. For information about using configuration sets, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/monitor-sending-activity.html">Amazon SES Developer Guide</a>. You can execute this operation no more than once per second.

        :param self:
        :param ConfigurationSetName:  The name of the configuration set to delete.<br/>
        :return:         """
        pass

    def delete_configuration_set_event_destination(self, EventDestinationName: str, ConfigurationSetName: str) -> DeleteConfigurationSetEventDestinationResponse:
        """Deletes a configuration set event destination. Configuration set event destinations are associated with configuration sets, which enable you to publish email sending events. For information about using configuration sets, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/monitor-sending-activity.html">Amazon SES Developer Guide</a>. You can execute this operation no more than once per second.

        :param self:
        :param EventDestinationName:  The name of the event destination to delete.<br/>ConfigurationSetName:  The name of the configuration set from which to delete the event destination.<br/>
        :return:         """
        pass

    def delete_configuration_set_tracking_options(self, ConfigurationSetName: str) -> DeleteConfigurationSetTrackingOptionsResponse:
        """Deletes an association between a configuration set and a custom domain for open and click event tracking. By default, images and links used for tracking open and click events are hosted on domains operated by Amazon SES. You can configure a subdomain of your own to handle these events. For information about using custom domains, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/configure-custom-open-click-domains.html">Amazon SES Developer Guide</a>. <note> Deleting this kind of association will result in emails sent using the specified configuration set to capture open and click events using the standard, Amazon SES-operated domains. </note>

        :param self:
        :param ConfigurationSetName:  The name of the configuration set from which you want to delete the tracking options.<br/>
        :return:         """
        pass

    def delete_custom_verification_email_template(self, TemplateName: str):
        """Deletes an existing custom verification email template.  For more information about custom verification email templates, see <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/custom-verification-emails.html">Using Custom Verification Email Templates</a> in the <i>Amazon SES Developer Guide</i>. You can execute this operation no more than once per second.

        :param self:
        :param TemplateName:  The name of the custom verification email template that you want to delete.<br/>
        :return:         """
        pass

    def delete_identity(self, Identity: str) -> DeleteIdentityResponse:
        """Deletes the specified identity (an email address or a domain) from the list of verified identities. You can execute this operation no more than once per second.

        :param self:
        :param Identity:  The identity to be removed from the list of identities for the AWS Account.<br/>
        :return:         """
        pass

    def delete_identity_policy(self, PolicyName: str, Identity: str) -> DeleteIdentityPolicyResponse:
        """Deletes the specified sending authorization policy for the given identity (an email address or a domain). This API returns successfully even if a policy with the specified name does not exist. <note> This API is for the identity owner only. If you have not verified the identity, this API will return an error. </note> Sending authorization is a feature that enables an identity owner to authorize other senders to use its identities. For information about using sending authorization, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/sending-authorization.html">Amazon SES Developer Guide</a>. You can execute this operation no more than once per second.

        :param self:
        :param PolicyName:  The name of the policy to be deleted.<br/>Identity:  The identity that is associated with the policy that you want to delete. You can specify the identity by using its name or by using its Amazon Resource Name (ARN). Examples: <code>user@example.com</code>, <code>example.com</code>, <code>arn:aws:ses:us-east-1:123456789012:identity/example.com</code>. To successfully call this API, you must own the identity.<br/>
        :return:         """
        pass

    def delete_receipt_filter(self, FilterName: str) -> DeleteReceiptFilterResponse:
        """Deletes the specified IP address filter. For information about managing IP address filters, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/receiving-email-managing-ip-filters.html">Amazon SES Developer Guide</a>. You can execute this operation no more than once per second.

        :param self:
        :param FilterName:  The name of the IP address filter to delete.<br/>
        :return:         """
        pass

    def delete_receipt_rule(self, RuleName: str, RuleSetName: str) -> DeleteReceiptRuleResponse:
        """Deletes the specified receipt rule. For information about managing receipt rules, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/receiving-email-managing-receipt-rules.html">Amazon SES Developer Guide</a>. You can execute this operation no more than once per second.

        :param self:
        :param RuleName:  The name of the receipt rule to delete.<br/>RuleSetName:  The name of the receipt rule set that contains the receipt rule to delete.<br/>
        :return:         """
        pass

    def delete_receipt_rule_set(self, RuleSetName: str) -> DeleteReceiptRuleSetResponse:
        """Deletes the specified receipt rule set and all of the receipt rules it contains. <note> The currently active rule set cannot be deleted. </note> For information about managing receipt rule sets, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/receiving-email-managing-receipt-rule-sets.html">Amazon SES Developer Guide</a>. You can execute this operation no more than once per second.

        :param self:
        :param RuleSetName:  The name of the receipt rule set to delete.<br/>
        :return:         """
        pass

    def delete_template(self, TemplateName: str) -> DeleteTemplateResponse:
        """Deletes an email template. You can execute this operation no more than once per second.

        :param self:
        :param TemplateName:  The name of the template to be deleted.<br/>
        :return:         """
        pass

    def delete_verified_email_address(self, EmailAddress: str):
        """Deprecated. Use the <code>DeleteIdentity</code> operation to delete email addresses and domains.

        :param self:
        :param EmailAddress:  An email address to be removed from the list of verified addresses.<br/>
        :return:         """
        pass

    def describe_active_receipt_rule_set(self) -> DescribeActiveReceiptRuleSetResponse:
        """Returns the metadata and receipt rules for the receipt rule set that is currently active. For information about setting up receipt rule sets, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/receiving-email-receipt-rule-set.html">Amazon SES Developer Guide</a>. You can execute this operation no more than once per second.

        :param self:
        :param 
        :return: Rules:  The receipt rules that belong to the active rule set.<br/>Metadata:  The metadata for the currently active receipt rule set. The metadata consists of the rule set name and a timestamp of when the rule set was created.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The date and time the receipt rule set was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the receipt rule set. The name must: <ul> <li> This value can only contain ASCII letters (a-z, A-Z), numbers (0-9), underscores (_), or dashes (-). </li> <li> Start and end with a letter or number. </li> <li> Contain less than 64 characters. </li> </ul><br/>        """
        pass

    def describe_configuration_set(self, ConfigurationSetName: str, ConfigurationSetAttributeNames: List[ConfigurationSetAttribute]=None) -> DescribeConfigurationSetResponse:
        """Returns the details of the specified configuration set. For information about using configuration sets, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/monitor-sending-activity.html">Amazon SES Developer Guide</a>. You can execute this operation no more than once per second.

        :param self:
        :param ConfigurationSetAttributeNames:  A list of configuration set attributes to return.<br/>ConfigurationSetName:  The name of the configuration set to describe.<br/>
        :return: ReputationOptions:  An object that represents the reputation settings for the configuration set. :<br/>DeliveryOptions:  Specifies whether messages that use the configuration set are required to use Transport Layer Security (TLS).:<br/>TrackingOptions:  The name of the custom open and click tracking domain associated with the configuration set.:<br/>EventDestinations:  A list of event destinations associated with the configuration set. <br/>ConfigurationSet:  The configuration set object associated with the specified configuration set.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the configuration set. The name must meet the following requirements: <ul> <li> Contain only letters (a-z, A-Z), numbers (0-9), underscores (_), or dashes (-). </li> <li> Contain 64 characters or fewer. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomRedirectDomain</i> The custom subdomain that will be used to redirect email recipients to the Amazon SES event tracking domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TlsPolicy</i> Specifies whether messages that use the configuration set are required to use Transport Layer Security (TLS). If the value is <code>Require</code>, messages are only delivered if a TLS connection can be established. If the value is <code>Optional</code>, messages can be delivered in plain text if a TLS connection can't be established.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastFreshStart</i> The date and time at which the reputation metrics for the configuration set were last reset. Resetting these metrics is known as a <i>fresh start</i>. When you disable email sending for a configuration set using <a>UpdateConfigurationSetSendingEnabled</a> and later re-enable it, the reputation metrics for the configuration set (but not for the entire Amazon SES account) are reset. If email sending for the configuration set has never been disabled and later re-enabled, the value of this attribute is <code>null</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReputationMetricsEnabled</i> Describes whether or not Amazon SES publishes reputation metrics for the configuration set, such as bounce and complaint rates, to Amazon CloudWatch. If the value is <code>true</code>, reputation metrics are published. If the value is <code>false</code>, reputation metrics are not published. The default value is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SendingEnabled</i> Describes whether email sending is enabled or disabled for the configuration set. If the value is <code>true</code>, then Amazon SES will send emails that use the configuration set. If the value is <code>false</code>, Amazon SES will not send emails that use the configuration set. The default value is <code>true</code>. You can change this setting using <a>UpdateConfigurationSetSendingEnabled</a>.<br/>        """
        pass

    def describe_receipt_rule(self, RuleName: str, RuleSetName: str) -> DescribeReceiptRuleResponse:
        """Returns the details of the specified receipt rule. For information about setting up receipt rules, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/receiving-email-receipt-rules.html">Amazon SES Developer Guide</a>. You can execute this operation no more than once per second.

        :param self:
        :param RuleName:  The name of the receipt rule.<br/>RuleSetName:  The name of the receipt rule set that the receipt rule belongs to.<br/>
        :return: Rule:  A data structure that contains the specified receipt rule's name, actions, recipients, domains, enabled status, scan status, and Transport Layer Security (TLS) policy.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ScanEnabled</i> If <code>true</code>, then messages that this receipt rule applies to are scanned for spam and viruses. The default value is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Actions</i> An ordered list of actions to perform on messages that match at least one of the recipient email addresses or domains specified in the receipt rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Recipients</i> The recipient domains and email addresses that the receipt rule applies to. If this field is not specified, this rule will match all recipients under all verified domains.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TlsPolicy</i> Specifies whether Amazon SES should require that incoming email is delivered over a connection encrypted with Transport Layer Security (TLS). If this parameter is set to <code>Require</code>, Amazon SES will bounce emails that are not received over TLS. The default is <code>Optional</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> If <code>true</code>, the receipt rule is active. The default value is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the receipt rule. The name must: <ul> <li> This value can only contain ASCII letters (a-z, A-Z), numbers (0-9), underscores (_), or dashes (-). </li> <li> Start and end with a letter or number. </li> <li> Contain less than 64 characters. </li> </ul><br/>        """
        pass

    def describe_receipt_rule_set(self, RuleSetName: str) -> DescribeReceiptRuleSetResponse:
        """Returns the details of the specified receipt rule set. For information about managing receipt rule sets, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/receiving-email-managing-receipt-rule-sets.html">Amazon SES Developer Guide</a>. You can execute this operation no more than once per second.

        :param self:
        :param RuleSetName:  The name of the receipt rule set to describe.<br/>
        :return: Rules:  A list of the receipt rules that belong to the specified receipt rule set.<br/>Metadata:  The metadata for the receipt rule set, which consists of the rule set name and the timestamp of when the rule set was created.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The date and time the receipt rule set was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the receipt rule set. The name must: <ul> <li> This value can only contain ASCII letters (a-z, A-Z), numbers (0-9), underscores (_), or dashes (-). </li> <li> Start and end with a letter or number. </li> <li> Contain less than 64 characters. </li> </ul><br/>        """
        pass

    def get_account_sending_enabled(self) -> GetAccountSendingEnabledResponse:
        """Returns the email sending status of the Amazon SES account for the current region. You can execute this operation no more than once per second.

        :param self:
        :param 
        :return: Enabled:  Describes whether email sending is enabled or disabled for your Amazon SES account in the current AWS Region.<br/>        """
        pass

    def get_custom_verification_email_template(self, TemplateName: str) -> GetCustomVerificationEmailTemplateResponse:
        """Returns the custom email verification template for the template name you specify. For more information about custom verification email templates, see <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/custom-verification-emails.html">Using Custom Verification Email Templates</a> in the <i>Amazon SES Developer Guide</i>. You can execute this operation no more than once per second.

        :param self:
        :param TemplateName:  The name of the custom verification email template that you want to retrieve.<br/>
        :return: FailureRedirectionURL:  The URL that the recipient of the verification email is sent to if his or her address is not successfully verified.<br/>SuccessRedirectionURL:  The URL that the recipient of the verification email is sent to if his or her address is successfully verified.<br/>TemplateContent:  The content of the custom verification email.<br/>TemplateSubject:  The subject line of the custom verification email.<br/>FromEmailAddress:  The email address that the custom verification email is sent from.<br/>TemplateName:  The name of the custom verification email template.<br/>        """
        pass

    def get_identity_dkim_attributes(self, Identities: List[Identity]) -> GetIdentityDkimAttributesResponse:
        """Returns the current status of Easy DKIM signing for an entity. For domain name identities, this operation also returns the DKIM tokens that are required for Easy DKIM signing, and whether Amazon SES has successfully verified that these tokens have been published. This operation takes a list of identities as input and returns the following information for each: <ul> <li> Whether Easy DKIM signing is enabled or disabled. </li> <li> A set of DKIM tokens that represent the identity. If the identity is an email address, the tokens represent the domain of that address. </li> <li> Whether Amazon SES has successfully verified the DKIM tokens published in the domain's DNS. This information is only returned for domain name identities, not for email addresses. </li> </ul> This operation is throttled at one request per second and can only get DKIM attributes for up to 100 identities at a time. For more information about creating DNS records using DKIM tokens, go to the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/easy-dkim-dns-records.html">Amazon SES Developer Guide</a>.

        :param self:
        :param Identities:  A list of one or more verified identities - email addresses, domains, or both.<br/>
        :return: DkimAttributes:  The DKIM attributes for an email address or a domain.<br/>        """
        pass

    def get_identity_mail_from_domain_attributes(self, Identities: List[Identity]) -> GetIdentityMailFromDomainAttributesResponse:
        """Returns the custom MAIL FROM attributes for a list of identities (email addresses : domains). This operation is throttled at one request per second and can only get custom MAIL FROM attributes for up to 100 identities at a time.

        :param self:
        :param Identities:  A list of one or more identities.<br/>
        :return: MailFromDomainAttributes:  A map of identities to custom MAIL FROM attributes.<br/>        """
        pass

    def get_identity_notification_attributes(self, Identities: List[Identity]) -> GetIdentityNotificationAttributesResponse:
        """Given a list of verified identities (email addresses and/or domains), returns a structure describing identity notification attributes. This operation is throttled at one request per second and can only get notification attributes for up to 100 identities at a time. For more information about using notifications with Amazon SES, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/notifications.html">Amazon SES Developer Guide</a>.

        :param self:
        :param Identities:  A list of one or more identities. You can specify an identity by using its name or by using its Amazon Resource Name (ARN). Examples: <code>user@example.com</code>, <code>example.com</code>, <code>arn:aws:ses:us-east-1:123456789012:identity/example.com</code>.<br/>
        :return: NotificationAttributes:  A map of Identity to IdentityNotificationAttributes.<br/>        """
        pass

    def get_identity_policies(self, PolicyNames: List[PolicyName], Identity: str) -> GetIdentityPoliciesResponse:
        """Returns the requested sending authorization policies for the given identity (an email address or a domain). The policies are returned as a map of policy names to policy contents. You can retrieve a maximum of 20 policies at a time. <note> This API is for the identity owner only. If you have not verified the identity, this API will return an error. </note> Sending authorization is a feature that enables an identity owner to authorize other senders to use its identities. For information about using sending authorization, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/sending-authorization.html">Amazon SES Developer Guide</a>. You can execute this operation no more than once per second.

        :param self:
        :param PolicyNames:  A list of the names of policies to be retrieved. You can retrieve a maximum of 20 policies at a time. If you do not know the names of the policies that are attached to the identity, you can use <code>ListIdentityPolicies</code>.<br/>Identity:  The identity for which the policies will be retrieved. You can specify an identity by using its name or by using its Amazon Resource Name (ARN). Examples: <code>user@example.com</code>, <code>example.com</code>, <code>arn:aws:ses:us-east-1:123456789012:identity/example.com</code>. To successfully call this API, you must own the identity.<br/>
        :return: Policies:  A map of policy names to policies.<br/>        """
        pass

    def get_identity_verification_attributes(self, Identities: List[Identity]) -> GetIdentityVerificationAttributesResponse:
        """Given a list of identities (email addresses and/or domains), returns the verification status and (for domain identities) the verification token for each identity. The verification status of an email address is "Pending" until the email address owner clicks the link within the verification email that Amazon SES sent to that address. If the email address owner clicks the link within 24 hours, the verification status of the email address changes to "Success". If the link is not clicked within 24 hours, the verification status changes to "Failed." In that case, if you still want to verify the email address, you must restart the verification process from the beginning. For domain identities, the domain's verification status is "Pending" as Amazon SES searches for the required TXT record in the DNS settings of the domain. When Amazon SES detects the record, the domain's verification status changes to "Success". If Amazon SES is unable to detect the record within 72 hours, the domain's verification status changes to "Failed." In that case, if you still want to verify the domain, you must restart the verification process from the beginning. This operation is throttled at one request per second and can only get verification attributes for up to 100 identities at a time.

        :param self:
        :param Identities:  A list of identities.<br/>
        :return: VerificationAttributes:  A map of Identities to IdentityVerificationAttributes objects.<br/>        """
        pass

    def get_send_quota(self) -> GetSendQuotaResponse:
        """Provides the sending limits for the Amazon SES account.  You can execute this operation no more than once per second.

        :param self:
        :param 
        :return: SentLast24Hours:  The number of emails sent during the previous 24 hours.<br/>MaxSendRate:  The maximum number of emails that Amazon SES can accept from the user's account per second. <note> The rate at which Amazon SES accepts the user's messages might be less than the maximum send rate. </note><br/>Max24HourSend:  The maximum number of emails the user is allowed to send in a 24-hour interval. A value of -1 signifies an unlimited quota.<br/>        """
        pass

    def get_send_statistics(self) -> GetSendStatisticsResponse:
        """Provides sending statistics for the current AWS Region. The result is a list of data points, representing the last two weeks of sending activity. Each data point in the list contains statistics for a 15-minute period of time. You can execute this operation no more than once per second.

        :param self:
        :param 
        :return: SendDataPoints:  A list of data points, each of which represents 15 minutes of activity.<br/>        """
        pass

    def get_template(self, TemplateName: str) -> GetTemplateResponse:
        """Displays the template object (which includes the Subject line, HTML part and text part) for the template you specify. You can execute this operation no more than once per second.

        :param self:
        :param TemplateName:  The name of the template you want to retrieve.<br/>
        :return: Template:  The content of the email, composed of a subject line, an HTML part, and a text-only part.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HtmlPart</i> The HTML body of the email.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TextPart</i> The email body that will be visible to recipients whose email clients do not display HTML.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubjectPart</i> The subject line of the email.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TemplateName</i> The name of the template. You will refer to this name when you send email using the <code>SendTemplatedEmail</code> or <code>SendBulkTemplatedEmail</code> operations.<br/>        """
        pass

    def list_configuration_sets(self, NextToken: str=None, MaxItems: int=None) -> ListConfigurationSetsResponse:
        """Provides a list of the configuration sets associated with your Amazon SES account in the current AWS Region. For information about using configuration sets, see <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/monitor-sending-activity.html">Monitoring Your Amazon SES Sending Activity</a> in the <i>Amazon SES Developer Guide.</i>  You can execute this operation no more than once per second. This operation will return up to 1,000 configuration sets each time it is run. If your Amazon SES account has more than 1,000 configuration sets, this operation will also return a NextToken element. You can then execute the <code>ListConfigurationSets</code> operation again, passing the <code>NextToken</code> parameter and the value of the NextToken element to retrieve additional results.

        :param self:
        :param MaxItems:  The number of configuration sets to return.<br/>NextToken:  A token returned from a previous call to <code>ListConfigurationSets</code> to indicate the position of the configuration set in the configuration set list.<br/>
        :return: NextToken:  A token indicating that there are additional configuration sets available to be listed. Pass this token to successive calls of <code>ListConfigurationSets</code>. <br/>ConfigurationSets:  A list of configuration sets.<br/>        """
        pass

    def list_custom_verification_email_templates(self, NextToken: str=None, MaxResults: int=None) -> ListCustomVerificationEmailTemplatesResponse:
        """Lists the existing custom verification email templates for your account in the current AWS Region. For more information about custom verification email templates, see <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/custom-verification-emails.html">Using Custom Verification Email Templates</a> in the <i>Amazon SES Developer Guide</i>. You can execute this operation no more than once per second.

        :param self:
        :param MaxResults:  The maximum number of custom verification email templates to return. This value must be at least 1 and less than or equal to 50. If you do not specify a value, or if you specify a value less than 1 or greater than 50, the operation will return up to 50 results.<br/>NextToken:  An array the contains the name and creation time stamp for each template in your Amazon SES account.<br/>
        :return: NextToken:  A token indicating that there are additional custom verification email templates available to be listed. Pass this token to a subsequent call to <code>ListTemplates</code> to retrieve the next 50 custom verification email templates.<br/>CustomVerificationEmailTemplates:  A list of the custom verification email templates that exist in your account.<br/>        """
        pass

    def list_identities(self, IdentityType: str=None, NextToken: str=None, MaxItems: int=None) -> ListIdentitiesResponse:
        """Returns a list containing all of the identities (email addresses and domains) for your AWS account in the current AWS Region, regardless of verification status. You can execute this operation no more than once per second.

        :param self:
        :param MaxItems:  The maximum number of identities per page. Possible values are 1-1000 inclusive.<br/>NextToken:  The token to use for pagination.<br/>IdentityType:  The type of the identities to list. Possible values are "EmailAddress" and "Domain". If this parameter is omitted, then all identities will be listed.<br/>
        :return: NextToken:  The token used for pagination.<br/>Identities:  A list of identities.<br/>        """
        pass

    def list_identity_policies(self, Identity: str) -> ListIdentityPoliciesResponse:
        """Returns a list of sending authorization policies that are attached to the given identity (an email address or a domain). This API returns only a list. If you want the actual policy content, you can use <code>GetIdentityPolicies</code>. <note> This API is for the identity owner only. If you have not verified the identity, this API will return an error. </note> Sending authorization is a feature that enables an identity owner to authorize other senders to use its identities. For information about using sending authorization, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/sending-authorization.html">Amazon SES Developer Guide</a>. You can execute this operation no more than once per second.

        :param self:
        :param Identity:  The identity that is associated with the policy for which the policies will be listed. You can specify an identity by using its name or by using its Amazon Resource Name (ARN). Examples: <code>user@example.com</code>, <code>example.com</code>, <code>arn:aws:ses:us-east-1:123456789012:identity/example.com</code>. To successfully call this API, you must own the identity.<br/>
        :return: PolicyNames:  A list of names of policies that apply to the specified identity.<br/>        """
        pass

    def list_receipt_filters(self) -> ListReceiptFiltersResponse:
        """Lists the IP address filters associated with your AWS account in the current AWS Region. For information about managing IP address filters, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/receiving-email-managing-ip-filters.html">Amazon SES Developer Guide</a>. You can execute this operation no more than once per second.

        :param self:
        :param 
        :return: Filters:  A list of IP address filter data structures, which each consist of a name, an IP address range, and whether to allow or block mail from it.<br/>        """
        pass

    def list_receipt_rule_sets(self, NextToken: str=None) -> ListReceiptRuleSetsResponse:
        """Lists the receipt rule sets that exist under your AWS account in the current AWS Region. If there are additional receipt rule sets to be retrieved, you will receive a <code>NextToken</code> that you can provide to the next call to <code>ListReceiptRuleSets</code> to retrieve the additional entries. For information about managing receipt rule sets, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/receiving-email-managing-receipt-rule-sets.html">Amazon SES Developer Guide</a>. You can execute this operation no more than once per second.

        :param self:
        :param NextToken:  A token returned from a previous call to <code>ListReceiptRuleSets</code> to indicate the position in the receipt rule set list.<br/>
        :return: NextToken:  A token indicating that there are additional receipt rule sets available to be listed. Pass this token to successive calls of <code>ListReceiptRuleSets</code> to retrieve up to 100 receipt rule sets at a time.<br/>RuleSets:  The metadata for the currently active receipt rule set. The metadata consists of the rule set name and the timestamp of when the rule set was created.<br/>        """
        pass

    def list_templates(self, NextToken: str=None, MaxItems: int=None) -> ListTemplatesResponse:
        """Lists the email templates present in your Amazon SES account in the current AWS Region. You can execute this operation no more than once per second.

        :param self:
        :param MaxItems:  The maximum number of templates to return. This value must be at least 1 and less than or equal to 10. If you do not specify a value, or if you specify a value less than 1 or greater than 10, the operation will return up to 10 results.<br/>NextToken:  A token returned from a previous call to <code>ListTemplates</code> to indicate the position in the list of email templates.<br/>
        :return: NextToken:  A token indicating that there are additional email templates available to be listed. Pass this token to a subsequent call to <code>ListTemplates</code> to retrieve the next 50 email templates.<br/>TemplatesMetadata:  An array the contains the name and creation time stamp for each template in your Amazon SES account.<br/>        """
        pass

    def list_verified_email_addresses(self) -> ListVerifiedEmailAddressesResponse:
        """Deprecated. Use the <code>ListIdentities</code> operation to list the email addresses and domains associated with your account.

        :param self:
        :param 
        :return: VerifiedEmailAddresses:  A list of email addresses that have been verified.<br/>        """
        pass

    def put_configuration_set_delivery_options(self, ConfigurationSetName: str, DeliveryOptions: DeliveryOptions=None) -> PutConfigurationSetDeliveryOptionsResponse:
        """Adds or updates the delivery options for a configuration set.

        :param self:
        :param DeliveryOptions:  Specifies whether messages that use the configuration set are required to use Transport Layer Security (TLS).:<br/>ConfigurationSetName:  The name of the configuration set that you want to specify the delivery options for.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TlsPolicy</i> Specifies whether messages that use the configuration set are required to use Transport Layer Security (TLS). If the value is <code>Require</code>, messages are only delivered if a TLS connection can be established. If the value is <code>Optional</code>, messages can be delivered in plain text if a TLS connection can't be established.<br/>
        :return:         """
        pass

    def put_identity_policy(self, Policy: str, PolicyName: str, Identity: str) -> PutIdentityPolicyResponse:
        """Adds or updates a sending authorization policy for the specified identity (an email address or a domain). <note> This API is for the identity owner only. If you have not verified the identity, this API will return an error. </note> Sending authorization is a feature that enables an identity owner to authorize other senders to use its identities. For information about using sending authorization, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/sending-authorization.html">Amazon SES Developer Guide</a>. You can execute this operation no more than once per second.

        :param self:
        :param Policy:  The text of the policy in JSON format. The policy cannot exceed 4 KB. For information about the syntax of sending authorization policies, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/sending-authorization-policies.html">Amazon SES Developer Guide</a>. <br/>PolicyName:  The name of the policy. The policy name cannot exceed 64 characters and can only include alphanumeric characters, dashes, and underscores.<br/>Identity:  The identity that the policy will apply to. You can specify an identity by using its name or by using its Amazon Resource Name (ARN). Examples: <code>user@example.com</code>, <code>example.com</code>, <code>arn:aws:ses:us-east-1:123456789012:identity/example.com</code>. To successfully call this API, you must own the identity.<br/>
        :return:         """
        pass

    def reorder_receipt_rule_set(self, RuleNames: List[ReceiptRuleName], RuleSetName: str) -> ReorderReceiptRuleSetResponse:
        """Reorders the receipt rules within a receipt rule set. <note> All of the rules in the rule set must be represented in this request. That is, this API will return an error if the reorder request doesn't explicitly position all of the rules. </note> For information about managing receipt rule sets, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/receiving-email-managing-receipt-rule-sets.html">Amazon SES Developer Guide</a>. You can execute this operation no more than once per second.

        :param self:
        :param RuleNames:  A list of the specified receipt rule set's receipt rules in the order that you want to put them.<br/>RuleSetName:  The name of the receipt rule set to reorder.<br/>
        :return:         """
        pass

    def send_bounce(self, BouncedRecipientInfoList: List[BouncedRecipientInfo], BounceSender: str, OriginalMessageId: str, Explanation: str=None, MessageDsn: MessageDsn=None, BounceSenderArn: str=None) -> SendBounceResponse:
        """Generates and sends a bounce message to the sender of an email you received through Amazon SES. You can only use this API on an email up to 24 hours after you receive it. <note> You cannot use this API to send generic bounces for mail that was not received by Amazon SES. </note> For information about receiving email through Amazon SES, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/receiving-email.html">Amazon SES Developer Guide</a>. You can execute this operation no more than once per second.

        :param self:
        :param BounceSenderArn:  This parameter is used only for sending authorization. It is the ARN of the identity that is associated with the sending authorization policy that permits you to use the address in the "From" header of the bounce. For more information about sending authorization, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/sending-authorization.html">Amazon SES Developer Guide</a>.<br/>BouncedRecipientInfoList:  A list of recipients of the bounced message, including the information required to create the Delivery Status Notifications (DSNs) for the recipients. You must specify at least one <code>BouncedRecipientInfo</code> in the list.<br/>MessageDsn:  Message-related DSN fields. If not specified, Amazon SES will choose the values.:<br/>Explanation:  Human-readable text for the bounce message to explain the failure. If not specified, the text will be auto-generated based on the bounced recipient information.<br/>BounceSender:  The address to use in the "From" header of the bounce message. This must be an identity that you have verified with Amazon SES.<br/>OriginalMessageId:  The message ID of the message to be bounced.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExtensionFields</i> Additional X-headers to include in the DSN.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ArrivalDate</i> When the message was received by the reporting mail transfer agent (MTA), in <a href="https://www.ietf.org/rfc/rfc0822.txt">RFC 822</a> date-time format.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReportingMta</i> The reporting MTA that attempted to deliver the message, formatted as specified in <a href="https://tools.ietf.org/html/rfc3464">RFC 3464</a> (<code>mta-name-type; mta-name</code>). The default value is <code>dns; inbound-smtp.[region].amazonaws.com</code>.<br/>
        :return: MessageId:  The message ID of the bounce message.<br/>        """
        pass

    def send_bulk_templated_email(self, Destinations: List[BulkEmailDestination], Template: str, Source: str, SourceArn: str=None, ReplyToAddresses: List[Address]=None, ReturnPath: str=None, ReturnPathArn: str=None, ConfigurationSetName: str=None, DefaultTags: List[MessageTag]=None, TemplateArn: str=None, DefaultTemplateData: str=None) -> SendBulkTemplatedEmailResponse:
        """Composes an email message to multiple destinations. The message body is created using an email template. In order to send email using the <code>SendBulkTemplatedEmail</code> operation, your call to the API must meet the following requirements: <ul> <li> The call must refer to an existing email template. You can create email templates using the <a>CreateTemplate</a> operation. </li> <li> The message must be sent from a verified email address or domain. </li> <li> If your account is still in the Amazon SES sandbox, you may only send to verified addresses or domains, or to email addresses associated with the Amazon SES Mailbox Simulator. For more information, see <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/verify-addresses-and-domains.html">Verifying Email Addresses and Domains</a> in the <i>Amazon SES Developer Guide.</i>  </li> <li> The maximum message size is 10 MB. </li> <li> Each <code>Destination</code> parameter must include at least one recipient email address. The recipient address can be a To: address, a CC: address, or a BCC: address. If a recipient email address is invalid (that is, it is not in the format <i>UserName@[SubDomain.]Domain.TopLevelDomain</i>), the entire message will be rejected, even if the message contains other recipients that are valid. </li> <li> The message may not include more than 50 recipients, across the To:, CC: and BCC: fields. If you need to send an email message to a larger audience, you can divide your recipient list into groups of 50 or fewer, and then call the <code>SendBulkTemplatedEmail</code> operation several times to send the message to each group. </li> <li> The number of destinations you can contact in a single call to the API may be limited by your account's maximum sending rate. </li> </ul>

        :param self:
        :param Destinations:  One or more <code>Destination</code> objects. All of the recipients in a <code>Destination</code> will receive the same version of the email. You can specify up to 50 <code>Destination</code> objects within a <code>Destinations</code> array.<br/>DefaultTemplateData:  A list of replacement values to apply to the template when replacement data is not specified in a Destination object. These values act as a default or fallback option when no other data is available. The template data is a JSON object, typically consisting of key-value pairs in which the keys correspond to replacement tags in the email template.<br/>TemplateArn:  The ARN of the template to use when sending this email.<br/>Template:  The template to use when sending this email.<br/>DefaultTags:  A list of tags, in the form of name/value pairs, to apply to an email that you send to a destination using <code>SendBulkTemplatedEmail</code>.<br/>ConfigurationSetName:  The name of the configuration set to use when you send an email using <code>SendBulkTemplatedEmail</code>.<br/>ReturnPathArn:  This parameter is used only for sending authorization. It is the ARN of the identity that is associated with the sending authorization policy that permits you to use the email address specified in the <code>ReturnPath</code> parameter. For example, if the owner of <code>example.com</code> (which has ARN <code>arn:aws:ses:us-east-1:123456789012:identity/example.com</code>) attaches a policy to it that authorizes you to use <code>feedback@example.com</code>, then you would specify the <code>ReturnPathArn</code> to be <code>arn:aws:ses:us-east-1:123456789012:identity/example.com</code>, and the <code>ReturnPath</code> to be <code>feedback@example.com</code>. For more information about sending authorization, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/sending-authorization.html">Amazon SES Developer Guide</a>.<br/>ReturnPath:  The email address that bounces and complaints will be forwarded to when feedback forwarding is enabled. If the message cannot be delivered to the recipient, then an error message will be returned from the recipient's ISP; this message will then be forwarded to the email address specified by the <code>ReturnPath</code> parameter. The <code>ReturnPath</code> parameter is never overwritten. This email address must be either individually verified with Amazon SES, or from a domain that has been verified with Amazon SES. <br/>ReplyToAddresses:  The reply-to email address(es) for the message. If the recipient replies to the message, each reply-to address will receive the reply.<br/>SourceArn:  This parameter is used only for sending authorization. It is the ARN of the identity that is associated with the sending authorization policy that permits you to send for the email address specified in the <code>Source</code> parameter. For example, if the owner of <code>example.com</code> (which has ARN <code>arn:aws:ses:us-east-1:123456789012:identity/example.com</code>) attaches a policy to it that authorizes you to send from <code>user@example.com</code>, then you would specify the <code>SourceArn</code> to be <code>arn:aws:ses:us-east-1:123456789012:identity/example.com</code>, and the <code>Source</code> to be <code>user@example.com</code>. For more information about sending authorization, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/sending-authorization.html">Amazon SES Developer Guide</a>.<br/>Source:  The email address that is sending the email. This email address must be either individually verified with Amazon SES, or from a domain that has been verified with Amazon SES. For information about verifying identities, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/verify-addresses-and-domains.html">Amazon SES Developer Guide</a>. If you are sending on behalf of another user and have been permitted to do so by a sending authorization policy, then you must also specify the <code>SourceArn</code> parameter. For more information about sending authorization, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/sending-authorization.html">Amazon SES Developer Guide</a>. <note> Amazon SES does not support the SMTPUTF8 extension, as described in <a href="https://tools.ietf.org/html/rfc6531">RFC6531</a>. For this reason, the <i>local part</i> of a source email address (the part of the email address that precedes the @ sign) may only contain <a href="https://en.wikipedia.org/wiki/Email_address#Local-part">7-bit ASCII characters</a>. If the <i>domain part</i> of an address (the part after the @ sign) contains non-ASCII characters, they must be encoded using Punycode, as described in <a href="https://tools.ietf.org/html/rfc3492.html">RFC3492</a>. The sender name (also known as the <i>friendly name</i>) may contain non-ASCII characters. These characters must be encoded using MIME encoded-word syntax, as described in <a href="https://tools.ietf.org/html/rfc2047">RFC 2047</a>. MIME encoded-word syntax uses the following form: <code>=?charset?encoding?encoded-text?=</code>. </note><br/>
        :return: Status:  The unique message identifier returned from the <code>SendBulkTemplatedEmail</code> action.<br/>        """
        pass

    def send_custom_verification_email(self, TemplateName: str, EmailAddress: str, ConfigurationSetName: str=None) -> SendCustomVerificationEmailResponse:
        """Adds an email address to the list of identities for your Amazon SES account in the current AWS Region and attempts to verify it. As a result of executing this operation, a customized verification email is sent to the specified address. To use this operation, you must first create a custom verification email template. For more information about creating and using custom verification email templates, see <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/custom-verification-emails.html">Using Custom Verification Email Templates</a> in the <i>Amazon SES Developer Guide</i>. You can execute this operation no more than once per second.

        :param self:
        :param ConfigurationSetName:  Name of a configuration set to use when sending the verification email.<br/>TemplateName:  The name of the custom verification email template to use when sending the verification email.<br/>EmailAddress:  The email address to verify.<br/>
        :return: MessageId:  The unique message identifier returned from the <code>SendCustomVerificationEmail</code> operation.<br/>        """
        pass

    def send_email(self, Message: Message, Destination: Destination, Source: str, ReplyToAddresses: List[Address]=None, ReturnPath: str=None, SourceArn: str=None, ReturnPathArn: str=None, Tags: List[MessageTag]=None, ConfigurationSetName: str=None) -> SendEmailResponse:
        """Composes an email message and immediately queues it for sending. In order to send email using the <code>SendEmail</code> operation, your message must meet the following requirements: <ul> <li> The message must be sent from a verified email address or domain. If you attempt to send email using a non-verified address or domain, the operation will result in an "Email address not verified" error.  </li> <li> If your account is still in the Amazon SES sandbox, you may only send to verified addresses or domains, or to email addresses associated with the Amazon SES Mailbox Simulator. For more information, see <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/verify-addresses-and-domains.html">Verifying Email Addresses and Domains</a> in the <i>Amazon SES Developer Guide.</i>  </li> <li> The maximum message size is 10 MB. </li> <li> The message must include at least one recipient email address. The recipient address can be a To: address, a CC: address, or a BCC: address. If a recipient email address is invalid (that is, it is not in the format <i>UserName@[SubDomain.]Domain.TopLevelDomain</i>), the entire message will be rejected, even if the message contains other recipients that are valid. </li> <li> The message may not include more than 50 recipients, across the To:, CC: and BCC: fields. If you need to send an email message to a larger audience, you can divide your recipient list into groups of 50 or fewer, and then call the <code>SendEmail</code> operation several times to send the message to each group. </li> </ul> <important> For every message that you send, the total number of recipients (including each recipient in the To:, CC: and BCC: fields) is counted against the maximum number of emails you can send in a 24-hour period (your <i>sending quota</i>). For more information about sending quotas in Amazon SES, see <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/manage-sending-limits.html">Managing Your Amazon SES Sending Limits</a> in the <i>Amazon SES Developer Guide.</i>  </important>

        :param self:
        :param ConfigurationSetName:  The name of the configuration set to use when you send an email using <code>SendEmail</code>.<br/>Tags:  A list of tags, in the form of name/value pairs, to apply to an email that you send using <code>SendEmail</code>. Tags correspond to characteristics of the email that you define, so that you can publish email sending events.<br/>ReturnPathArn:  This parameter is used only for sending authorization. It is the ARN of the identity that is associated with the sending authorization policy that permits you to use the email address specified in the <code>ReturnPath</code> parameter. For example, if the owner of <code>example.com</code> (which has ARN <code>arn:aws:ses:us-east-1:123456789012:identity/example.com</code>) attaches a policy to it that authorizes you to use <code>feedback@example.com</code>, then you would specify the <code>ReturnPathArn</code> to be <code>arn:aws:ses:us-east-1:123456789012:identity/example.com</code>, and the <code>ReturnPath</code> to be <code>feedback@example.com</code>. For more information about sending authorization, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/sending-authorization.html">Amazon SES Developer Guide</a>.<br/>SourceArn:  This parameter is used only for sending authorization. It is the ARN of the identity that is associated with the sending authorization policy that permits you to send for the email address specified in the <code>Source</code> parameter. For example, if the owner of <code>example.com</code> (which has ARN <code>arn:aws:ses:us-east-1:123456789012:identity/example.com</code>) attaches a policy to it that authorizes you to send from <code>user@example.com</code>, then you would specify the <code>SourceArn</code> to be <code>arn:aws:ses:us-east-1:123456789012:identity/example.com</code>, and the <code>Source</code> to be <code>user@example.com</code>. For more information about sending authorization, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/sending-authorization.html">Amazon SES Developer Guide</a>.<br/>ReturnPath:  The email address that bounces and complaints will be forwarded to when feedback forwarding is enabled. If the message cannot be delivered to the recipient, then an error message will be returned from the recipient's ISP; this message will then be forwarded to the email address specified by the <code>ReturnPath</code> parameter. The <code>ReturnPath</code> parameter is never overwritten. This email address must be either individually verified with Amazon SES, or from a domain that has been verified with Amazon SES. <br/>ReplyToAddresses:  The reply-to email address(es) for the message. If the recipient replies to the message, each reply-to address will receive the reply.<br/>Message:  The message to be sent.:<br/>Destination:  The destination for this email, composed of To:, CC:, and BCC: fields.:<br/>Source:  The email address that is sending the email. This email address must be either individually verified with Amazon SES, or from a domain that has been verified with Amazon SES. For information about verifying identities, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/verify-addresses-and-domains.html">Amazon SES Developer Guide</a>. If you are sending on behalf of another user and have been permitted to do so by a sending authorization policy, then you must also specify the <code>SourceArn</code> parameter. For more information about sending authorization, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/sending-authorization.html">Amazon SES Developer Guide</a>. <note> Amazon SES does not support the SMTPUTF8 extension, as described in <a href="https://tools.ietf.org/html/rfc6531">RFC6531</a>. For this reason, the <i>local part</i> of a source email address (the part of the email address that precedes the @ sign) may only contain <a href="https://en.wikipedia.org/wiki/Email_address#Local-part">7-bit ASCII characters</a>. If the <i>domain part</i> of an address (the part after the @ sign) contains non-ASCII characters, they must be encoded using Punycode, as described in <a href="https://tools.ietf.org/html/rfc3492.html">RFC3492</a>. The sender name (also known as the <i>friendly name</i>) may contain non-ASCII characters. These characters must be encoded using MIME encoded-word syntax, as described in <a href="https://tools.ietf.org/html/rfc2047">RFC 2047</a>. MIME encoded-word syntax uses the following form: <code>=?charset?encoding?encoded-text?=</code>. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BccAddresses</i> The BCC: field(s) of the message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CcAddresses</i> The CC: field(s) of the message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ToAddresses</i> The To: field(s) of the message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Body</i> The message body.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Body</b>: The message body.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Subject</i> The subject of the message: A short summary of the content, which will appear in the recipient's inbox.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Subject</b>: The subject of the message: A short summary of the content, which will appear in the recipient's inbox.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Charset</i> The character set of the content.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Data</i> The textual data of the content.<br/>The content of the message, in text format. Use this for text-based email clients, or clients on high-latency networks (such as mobile devices).&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Text</b>: The content of the message, in text format. Use this for text-based email clients, or clients on high-latency networks (such as mobile devices).
        :return: MessageId:  The unique message identifier returned from the <code>SendEmail</code> action. <br/>        """
        pass

    def send_raw_email(self, RawMessage: RawMessage, Source: str=None, Destinations: List[Address]=None, FromArn: str=None, SourceArn: str=None, ReturnPathArn: str=None, Tags: List[MessageTag]=None, ConfigurationSetName: str=None) -> SendRawEmailResponse:
        """Composes an email message and immediately queues it for sending. This operation is more flexible than the <code>SendEmail</code> API operation. When you use the <code>SendRawEmail</code> operation, you can specify the headers of the message as well as its content. This flexibility is useful, for example, when you want to send a multipart MIME email (such a message that contains both a text and an HTML version). You can also use this operation to send messages that include attachments. The <code>SendRawEmail</code> operation has the following requirements: <ul> <li> You can only send email from <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/verify-addresses-and-domains.html">verified email addresses or domains</a>. If you try to send email from an address that isn't verified, the operation results in an "Email address not verified" error. </li> <li> If your account is still in the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/request-production-access.html">Amazon SES sandbox</a>, you can only send email to other verified addresses in your account, or to addresses that are associated with the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/mailbox-simulator.html">Amazon SES mailbox simulator</a>. </li> <li> The maximum message size, including attachments, is 10 MB. </li> <li> Each message has to include at least one recipient address. A recipient address includes any address on the To:, CC:, or BCC: lines. </li> <li> If you send a single message to more than one recipient address, and one of the recipient addresses isn't in a valid format (that is, it's not in the format <i>UserName@[SubDomain.]Domain.TopLevelDomain</i>), Amazon SES rejects the entire message, even if the other addresses are valid. </li> <li> Each message can include up to 50 recipient addresses across the To:, CC:, or BCC: lines. If you need to send a single message to more than 50 recipients, you have to split the list of recipient addresses into groups of less than 50 recipients, and send separate messages to each group. </li> <li> Amazon SES allows you to specify 8-bit Content-Transfer-Encoding for MIME message parts. However, if Amazon SES has to modify the contents of your message (for example, if you use open and click tracking), 8-bit content isn't preserved. For this reason, we highly recommend that you encode all content that isn't 7-bit ASCII. For more information, see <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/send-email-raw.html#send-email-mime-encoding">MIME Encoding</a> in the <i>Amazon SES Developer Guide</i>. </li> </ul> Additionally, keep the following considerations in mind when using the <code>SendRawEmail</code> operation: <ul> <li> Although you can customize the message headers when using the <code>SendRawEmail</code> operation, Amazon SES will automatically apply its own <code>Message-ID</code> and <code>Date</code> headers; if you passed these headers when creating the message, they will be overwritten by the values that Amazon SES provides. </li> <li> If you are using sending authorization to send on behalf of another user, <code>SendRawEmail</code> enables you to specify the cross-account identity for the email's Source, From, and Return-Path parameters in one of two ways: you can pass optional parameters <code>SourceArn</code>, <code>FromArn</code>, and/or <code>ReturnPathArn</code> to the API, or you can include the following X-headers in the header of your raw email: <ul> <li>  <code>X-SES-SOURCE-ARN</code>  </li> <li>  <code>X-SES-FROM-ARN</code>  </li> <li>  <code>X-SES-RETURN-PATH-ARN</code>  </li> </ul> <important> Do not include these X-headers in the DKIM signature; Amazon SES will remove them before sending the email. </important> For most common sending authorization scenarios, we recommend that you specify the <code>SourceIdentityArn</code> parameter and not the <code>FromIdentityArn</code> or <code>ReturnPathIdentityArn</code> parameters. If you only specify the <code>SourceIdentityArn</code> parameter, Amazon SES will set the From and Return Path addresses to the identity specified in <code>SourceIdentityArn</code>. For more information about sending authorization, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/sending-authorization.html">Using Sending Authorization with Amazon SES</a> in the <i>Amazon SES Developer Guide.</i>  </li> <li> For every message that you send, the total number of recipients (including each recipient in the To:, CC: and BCC: fields) is counted against the maximum number of emails you can send in a 24-hour period (your <i>sending quota</i>). For more information about sending quotas in Amazon SES, see <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/manage-sending-limits.html">Managing Your Amazon SES Sending Limits</a> in the <i>Amazon SES Developer Guide.</i>  </li> </ul>

        :param self:
        :param ConfigurationSetName:  The name of the configuration set to use when you send an email using <code>SendRawEmail</code>.<br/>Tags:  A list of tags, in the form of name/value pairs, to apply to an email that you send using <code>SendRawEmail</code>. Tags correspond to characteristics of the email that you define, so that you can publish email sending events.<br/>ReturnPathArn:  This parameter is used only for sending authorization. It is the ARN of the identity that is associated with the sending authorization policy that permits you to use the email address specified in the <code>ReturnPath</code> parameter. For example, if the owner of <code>example.com</code> (which has ARN <code>arn:aws:ses:us-east-1:123456789012:identity/example.com</code>) attaches a policy to it that authorizes you to use <code>feedback@example.com</code>, then you would specify the <code>ReturnPathArn</code> to be <code>arn:aws:ses:us-east-1:123456789012:identity/example.com</code>, and the <code>ReturnPath</code> to be <code>feedback@example.com</code>. Instead of using this parameter, you can use the X-header <code>X-SES-RETURN-PATH-ARN</code> in the raw message of the email. If you use both the <code>ReturnPathArn</code> parameter and the corresponding X-header, Amazon SES uses the value of the <code>ReturnPathArn</code> parameter. <note> For information about when to use this parameter, see the description of <code>SendRawEmail</code> in this guide, or see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/sending-authorization-delegate-sender-tasks-email.html">Amazon SES Developer Guide</a>. </note><br/>SourceArn:  This parameter is used only for sending authorization. It is the ARN of the identity that is associated with the sending authorization policy that permits you to send for the email address specified in the <code>Source</code> parameter. For example, if the owner of <code>example.com</code> (which has ARN <code>arn:aws:ses:us-east-1:123456789012:identity/example.com</code>) attaches a policy to it that authorizes you to send from <code>user@example.com</code>, then you would specify the <code>SourceArn</code> to be <code>arn:aws:ses:us-east-1:123456789012:identity/example.com</code>, and the <code>Source</code> to be <code>user@example.com</code>. Instead of using this parameter, you can use the X-header <code>X-SES-SOURCE-ARN</code> in the raw message of the email. If you use both the <code>SourceArn</code> parameter and the corresponding X-header, Amazon SES uses the value of the <code>SourceArn</code> parameter. <note> For information about when to use this parameter, see the description of <code>SendRawEmail</code> in this guide, or see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/sending-authorization-delegate-sender-tasks-email.html">Amazon SES Developer Guide</a>. </note><br/>FromArn:  This parameter is used only for sending authorization. It is the ARN of the identity that is associated with the sending authorization policy that permits you to specify a particular "From" address in the header of the raw email. Instead of using this parameter, you can use the X-header <code>X-SES-FROM-ARN</code> in the raw message of the email. If you use both the <code>FromArn</code> parameter and the corresponding X-header, Amazon SES uses the value of the <code>FromArn</code> parameter. <note> For information about when to use this parameter, see the description of <code>SendRawEmail</code> in this guide, or see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/sending-authorization-delegate-sender-tasks-email.html">Amazon SES Developer Guide</a>. </note><br/>RawMessage:  The raw email message itself. The message has to meet the following criteria: <ul> <li> The message has to contain a header and a body, separated by a blank line. </li> <li> All of the required header fields must be present in the message. </li> <li> Each part of a multipart MIME message must be formatted properly. </li> <li> Attachments must be of a content type that Amazon SES supports. For a list on unsupported content types, see <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/mime-types.html">Unsupported Attachment Types</a> in the <i>Amazon SES Developer Guide</i>. </li> <li> The entire message must be base64-encoded. </li> <li> If any of the MIME parts in your message contain content that is outside of the 7-bit ASCII character range, we highly recommend that you encode that content. For more information, see <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/send-email-raw.html">Sending Raw Email</a> in the <i>Amazon SES Developer Guide</i>. </li> <li> Per <a href="https://tools.ietf.org/html/rfc5321#section-4.5.3.1.6">RFC 5321</a>, the maximum length of each line of text, including the &lt;CRLF&gt;, must not exceed 1,000 characters. </li> </ul>:<br/>Destinations:  A list of destinations for the message, consisting of To:, CC:, and BCC: addresses.<br/>Source:  The identity's email address. If you do not provide a value for this parameter, you must specify a "From" address in the raw text of the message. (You can also specify both.) <note> Amazon SES does not support the SMTPUTF8 extension, as described in<a href="https://tools.ietf.org/html/rfc6531">RFC6531</a>. For this reason, the <i>local part</i> of a source email address (the part of the email address that precedes the @ sign) may only contain <a href="https://en.wikipedia.org/wiki/Email_address#Local-part">7-bit ASCII characters</a>. If the <i>domain part</i> of an address (the part after the @ sign) contains non-ASCII characters, they must be encoded using Punycode, as described in <a href="https://tools.ietf.org/html/rfc3492.html">RFC3492</a>. The sender name (also known as the <i>friendly name</i>) may contain non-ASCII characters. These characters must be encoded using MIME encoded-word syntax, as described in <a href="https://tools.ietf.org/html/rfc2047">RFC 2047</a>. MIME encoded-word syntax uses the following form: <code>=?charset?encoding?encoded-text?=</code>. </note> If you specify the <code>Source</code> parameter and have feedback forwarding enabled, then bounces and complaints will be sent to this email address. This takes precedence over any Return-Path header that you might include in the raw text of the message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Data</i> The raw data of the message. This data needs to base64-encoded if you are accessing Amazon SES directly through the HTTPS interface. If you are accessing Amazon SES using an AWS SDK, the SDK takes care of the base 64-encoding for you. In all cases, the client must ensure that the message format complies with Internet email standards regarding email header fields, MIME types, and MIME encoding. The To:, CC:, and BCC: headers in the raw message can contain a group list. If you are using <code>SendRawEmail</code> with sending authorization, you can include X-headers in the raw message to specify the "Source," "From," and "Return-Path" addresses. For more information, see the documentation for <code>SendRawEmail</code>.  <important> Do not include these X-headers in the DKIM signature, because they are removed by Amazon SES before sending the email. </important> For more information, go to the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/send-email-raw.html">Amazon SES Developer Guide</a>.<br/>
        :return: MessageId:  The unique message identifier returned from the <code>SendRawEmail</code> action. <br/>        """
        pass

    def send_templated_email(self, TemplateData: str, Template: str, Destination: Destination, Source: str, ReplyToAddresses: List[Address]=None, ReturnPath: str=None, SourceArn: str=None, ReturnPathArn: str=None, Tags: List[MessageTag]=None, ConfigurationSetName: str=None, TemplateArn: str=None) -> SendTemplatedEmailResponse:
        """Composes an email message using an email template and immediately queues it for sending. In order to send email using the <code>SendTemplatedEmail</code> operation, your call to the API must meet the following requirements: <ul> <li> The call must refer to an existing email template. You can create email templates using the <a>CreateTemplate</a> operation. </li> <li> The message must be sent from a verified email address or domain. </li> <li> If your account is still in the Amazon SES sandbox, you may only send to verified addresses or domains, or to email addresses associated with the Amazon SES Mailbox Simulator. For more information, see <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/verify-addresses-and-domains.html">Verifying Email Addresses and Domains</a> in the <i>Amazon SES Developer Guide.</i>  </li> <li> The maximum message size is 10 MB. </li> <li> Calls to the <code>SendTemplatedEmail</code> operation may only include one <code>Destination</code> parameter. A destination is a set of recipients who will receive the same version of the email. The <code>Destination</code> parameter can include up to 50 recipients, across the To:, CC: and BCC: fields. </li> <li> The <code>Destination</code> parameter must include at least one recipient email address. The recipient address can be a To: address, a CC: address, or a BCC: address. If a recipient email address is invalid (that is, it is not in the format <i>UserName@[SubDomain.]Domain.TopLevelDomain</i>), the entire message will be rejected, even if the message contains other recipients that are valid. </li> </ul> <important> If your call to the <code>SendTemplatedEmail</code> operation includes all of the required parameters, Amazon SES accepts it and returns a Message ID. However, if Amazon SES can't render the email because the template contains errors, it doesn't send the email. Additionally, because it already accepted the message, Amazon SES doesn't return a message stating that it was unable to send the email. For these reasons, we highly recommend that you set up Amazon SES to send you notifications when Rendering Failure events occur. For more information, see <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/send-personalized-email-api.html">Sending Personalized Email Using the Amazon SES API</a> in the <i>Amazon Simple Email Service Developer Guide</i>. </important>

        :param self:
        :param TemplateData:  A list of replacement values to apply to the template. This parameter is a JSON object, typically consisting of key-value pairs in which the keys correspond to replacement tags in the email template.<br/>TemplateArn:  The ARN of the template to use when sending this email.<br/>Template:  The template to use when sending this email.<br/>ConfigurationSetName:  The name of the configuration set to use when you send an email using <code>SendTemplatedEmail</code>.<br/>Tags:  A list of tags, in the form of name/value pairs, to apply to an email that you send using <code>SendTemplatedEmail</code>. Tags correspond to characteristics of the email that you define, so that you can publish email sending events.<br/>ReturnPathArn:  This parameter is used only for sending authorization. It is the ARN of the identity that is associated with the sending authorization policy that permits you to use the email address specified in the <code>ReturnPath</code> parameter. For example, if the owner of <code>example.com</code> (which has ARN <code>arn:aws:ses:us-east-1:123456789012:identity/example.com</code>) attaches a policy to it that authorizes you to use <code>feedback@example.com</code>, then you would specify the <code>ReturnPathArn</code> to be <code>arn:aws:ses:us-east-1:123456789012:identity/example.com</code>, and the <code>ReturnPath</code> to be <code>feedback@example.com</code>. For more information about sending authorization, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/sending-authorization.html">Amazon SES Developer Guide</a>.<br/>SourceArn:  This parameter is used only for sending authorization. It is the ARN of the identity that is associated with the sending authorization policy that permits you to send for the email address specified in the <code>Source</code> parameter. For example, if the owner of <code>example.com</code> (which has ARN <code>arn:aws:ses:us-east-1:123456789012:identity/example.com</code>) attaches a policy to it that authorizes you to send from <code>user@example.com</code>, then you would specify the <code>SourceArn</code> to be <code>arn:aws:ses:us-east-1:123456789012:identity/example.com</code>, and the <code>Source</code> to be <code>user@example.com</code>. For more information about sending authorization, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/sending-authorization.html">Amazon SES Developer Guide</a>.<br/>ReturnPath:  The email address that bounces and complaints will be forwarded to when feedback forwarding is enabled. If the message cannot be delivered to the recipient, then an error message will be returned from the recipient's ISP; this message will then be forwarded to the email address specified by the <code>ReturnPath</code> parameter. The <code>ReturnPath</code> parameter is never overwritten. This email address must be either individually verified with Amazon SES, or from a domain that has been verified with Amazon SES. <br/>ReplyToAddresses:  The reply-to email address(es) for the message. If the recipient replies to the message, each reply-to address will receive the reply.<br/>Destination:  The destination for this email, composed of To:, CC:, and BCC: fields. A Destination can include up to 50 recipients across these three fields.:<br/>Source:  The email address that is sending the email. This email address must be either individually verified with Amazon SES, or from a domain that has been verified with Amazon SES. For information about verifying identities, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/verify-addresses-and-domains.html">Amazon SES Developer Guide</a>. If you are sending on behalf of another user and have been permitted to do so by a sending authorization policy, then you must also specify the <code>SourceArn</code> parameter. For more information about sending authorization, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/sending-authorization.html">Amazon SES Developer Guide</a>. <note> Amazon SES does not support the SMTPUTF8 extension, as described in <a href="https://tools.ietf.org/html/rfc6531">RFC6531</a>. For this reason, the <i>local part</i> of a source email address (the part of the email address that precedes the @ sign) may only contain <a href="https://en.wikipedia.org/wiki/Email_address#Local-part">7-bit ASCII characters</a>. If the <i>domain part</i> of an address (the part after the @ sign) contains non-ASCII characters, they must be encoded using Punycode, as described in <a href="https://tools.ietf.org/html/rfc3492.html">RFC3492</a>. The sender name (also known as the <i>friendly name</i>) may contain non-ASCII characters. These characters must be encoded using MIME encoded-word syntax, as described in<a href="https://tools.ietf.org/html/rfc2047">RFC 2047</a>. MIME encoded-word syntax uses the following form: <code>=?charset?encoding?encoded-text?=</code>. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BccAddresses</i> The BCC: field(s) of the message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CcAddresses</i> The CC: field(s) of the message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ToAddresses</i> The To: field(s) of the message.<br/>
        :return: MessageId:  The unique message identifier returned from the <code>SendTemplatedEmail</code> action. <br/>        """
        pass

    def set_active_receipt_rule_set(self, RuleSetName: str=None) -> SetActiveReceiptRuleSetResponse:
        """Sets the specified receipt rule set as the active receipt rule set. <note> To disable your email-receiving through Amazon SES completely, you can call this API with RuleSetName set to null. </note> For information about managing receipt rule sets, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/receiving-email-managing-receipt-rule-sets.html">Amazon SES Developer Guide</a>. You can execute this operation no more than once per second.

        :param self:
        :param RuleSetName:  The name of the receipt rule set to make active. Setting this value to null disables all email receiving.<br/>
        :return:         """
        pass

    def set_identity_dkim_enabled(self, DkimEnabled: Enabled, Identity: str) -> SetIdentityDkimEnabledResponse:
        """Enables or disables Easy DKIM signing of email sent from an identity: <ul> <li> If Easy DKIM signing is enabled for a domain name identity (such as <code>example.com</code>), then Amazon SES will DKIM-sign all email sent by addresses under that domain name (for example, <code>user@example.com</code>). </li> <li> If Easy DKIM signing is enabled for an email address, then Amazon SES will DKIM-sign all email sent by that email address. </li> </ul> For email addresses (for example, <code>user@example.com</code>), you can only enable Easy DKIM signing if the corresponding domain (in this case, <code>example.com</code>) has been set up for Easy DKIM using the AWS Console or the <code>VerifyDomainDkim</code> operation. You can execute this operation no more than once per second. For more information about Easy DKIM signing, go to the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/easy-dkim.html">Amazon SES Developer Guide</a>.

        :param self:
        :param DkimEnabled:  Sets whether DKIM signing is enabled for an identity. Set to <code>true</code> to enable DKIM signing for this identity; <code>false</code> to disable it. <br/>Identity:  The identity for which DKIM signing should be enabled or disabled.<br/>
        :return:         """
        pass

    def set_identity_feedback_forwarding_enabled(self, ForwardingEnabled: Enabled, Identity: str) -> SetIdentityFeedbackForwardingEnabledResponse:
        """Given an identity (an email address or a domain), enables or disables whether Amazon SES forwards bounce and complaint notifications as email. Feedback forwarding can only be disabled when Amazon Simple Notification Service (Amazon SNS) topics are specified for both bounces and complaints. <note> Feedback forwarding does not apply to delivery notifications. Delivery notifications are only available through Amazon SNS. </note> You can execute this operation no more than once per second. For more information about using notifications with Amazon SES, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/notifications.html">Amazon SES Developer Guide</a>.

        :param self:
        :param ForwardingEnabled:  Sets whether Amazon SES will forward bounce and complaint notifications as email. <code>true</code> specifies that Amazon SES will forward bounce and complaint notifications as email, in addition to any Amazon SNS topic publishing otherwise specified. <code>false</code> specifies that Amazon SES will publish bounce and complaint notifications only through Amazon SNS. This value can only be set to <code>false</code> when Amazon SNS topics are set for both <code>Bounce</code> and <code>Complaint</code> notification types.<br/>Identity:  The identity for which to set bounce and complaint notification forwarding. Examples: <code>user@example.com</code>, <code>example.com</code>.<br/>
        :return:         """
        pass

    def set_identity_headers_in_notifications_enabled(self, Enabled: Enabled, NotificationType: str, Identity: str) -> SetIdentityHeadersInNotificationsEnabledResponse:
        """Given an identity (an email address or a domain), sets whether Amazon SES includes the original email headers in the Amazon Simple Notification Service (Amazon SNS) notifications of a specified type. You can execute this operation no more than once per second. For more information about using notifications with Amazon SES, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/notifications.html">Amazon SES Developer Guide</a>.

        :param self:
        :param Enabled:  Sets whether Amazon SES includes the original email headers in Amazon SNS notifications of the specified notification type. A value of <code>true</code> specifies that Amazon SES will include headers in notifications, and a value of <code>false</code> specifies that Amazon SES will not include headers in notifications. This value can only be set when <code>NotificationType</code> is already set to use a particular Amazon SNS topic.<br/>NotificationType:  The notification type for which to enable or disable headers in notifications. <br/>Identity:  The identity for which to enable or disable headers in notifications. Examples: <code>user@example.com</code>, <code>example.com</code>.<br/>
        :return:         """
        pass

    def set_identity_mail_from_domain(self, Identity: str, MailFromDomain: str=None, BehaviorOnMXFailure: str=None) -> SetIdentityMailFromDomainResponse:
        """Enables or disables the custom MAIL FROM domain setup for a verified identity (an email address or a domain). <important> To send emails using the specified MAIL FROM domain, you must add an MX record to your MAIL FROM domain's DNS settings. If you want your emails to pass Sender Policy Framework (SPF) checks, you must also add or update an SPF record. For more information, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/mail-from-set.html">Amazon SES Developer Guide</a>. </important> You can execute this operation no more than once per second.

        :param self:
        :param BehaviorOnMXFailure:  The action that you want Amazon SES to take if it cannot successfully read the required MX record when you send an email. If you choose <code>UseDefaultValue</code>, Amazon SES will use amazonses.com (or a subdomain of that) as the MAIL FROM domain. If you choose <code>RejectMessage</code>, Amazon SES will return a <code>MailFromDomainNotVerified</code> error and not send the email. The action specified in <code>BehaviorOnMXFailure</code> is taken when the custom MAIL FROM domain setup is in the <code>Pending</code>, <code>Failed</code>, and <code>TemporaryFailure</code> states.<br/>MailFromDomain:  The custom MAIL FROM domain that you want the verified identity to use. The MAIL FROM domain must 1) be a subdomain of the verified identity, 2) not be used in a "From" address if the MAIL FROM domain is the destination of email feedback forwarding (for more information, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/mail-from.html">Amazon SES Developer Guide</a>), and 3) not be used to receive emails. A value of <code>null</code> disables the custom MAIL FROM setting for the identity.<br/>Identity:  The verified identity for which you want to enable or disable the specified custom MAIL FROM domain.<br/>
        :return:         """
        pass

    def set_identity_notification_topic(self, NotificationType: str, Identity: str, SnsTopic: str=None) -> SetIdentityNotificationTopicResponse:
        """Sets an Amazon Simple Notification Service (Amazon SNS) topic to use when delivering notifications. When you use this operation, you specify a verified identity, such as an email address or domain. When you send an email that uses the chosen identity in the Source field, Amazon SES sends notifications to the topic you specified. You can send bounce, complaint, or delivery notifications (or any combination of the three) to the Amazon SNS topic that you specify. You can execute this operation no more than once per second. For more information about feedback notification, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/notifications.html">Amazon SES Developer Guide</a>.

        :param self:
        :param SnsTopic:  The Amazon Resource Name (ARN) of the Amazon SNS topic. If the parameter is omitted from the request or a null value is passed, <code>SnsTopic</code> is cleared and publishing is disabled.<br/>NotificationType:  The type of notifications that will be published to the specified Amazon SNS topic.<br/>Identity:  The identity (email address or domain) that you want to set the Amazon SNS topic for. <important> You can only specify a verified identity for this parameter. </important> You can specify an identity by using its name or by using its Amazon Resource Name (ARN). The following examples are all valid identities: <code>sender@example.com</code>, <code>example.com</code>, <code>arn:aws:ses:us-east-1:123456789012:identity/example.com</code>.<br/>
        :return:         """
        pass

    def set_receipt_rule_position(self, RuleName: str, RuleSetName: str, After: str=None) -> SetReceiptRulePositionResponse:
        """Sets the position of the specified receipt rule in the receipt rule set. For information about managing receipt rules, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/receiving-email-managing-receipt-rules.html">Amazon SES Developer Guide</a>. You can execute this operation no more than once per second.

        :param self:
        :param After:  The name of the receipt rule after which to place the specified receipt rule.<br/>RuleName:  The name of the receipt rule to reposition.<br/>RuleSetName:  The name of the receipt rule set that contains the receipt rule to reposition.<br/>
        :return:         """
        pass

    def test_render_template(self, TemplateData: str, TemplateName: str) -> TestRenderTemplateResponse:
        """Creates a preview of the MIME content of an email when provided with a template and a set of replacement data. You can execute this operation no more than once per second.

        :param self:
        :param TemplateData:  A list of replacement values to apply to the template. This parameter is a JSON object, typically consisting of key-value pairs in which the keys correspond to replacement tags in the email template.<br/>TemplateName:  The name of the template that you want to render.<br/>
        :return: RenderedTemplate:  The complete MIME message rendered by applying the data in the TemplateData parameter to the template specified in the TemplateName parameter.<br/>        """
        pass

    def update_account_sending_enabled(self, Enabled: Enabled=None):
        """Enables or disables email sending across your entire Amazon SES account in the current AWS Region. You can use this operation in conjunction with Amazon CloudWatch alarms to temporarily pause email sending across your Amazon SES account in a given AWS Region when reputation metrics (such as your bounce or complaint rates) reach certain thresholds. You can execute this operation no more than once per second.

        :param self:
        :param Enabled:  Describes whether email sending is enabled or disabled for your Amazon SES account in the current AWS Region.<br/>
        :return:         """
        pass

    def update_configuration_set_event_destination(self, EventDestination: EventDestination, ConfigurationSetName: str) -> UpdateConfigurationSetEventDestinationResponse:
        """Updates the event destination of a configuration set. Event destinations are associated with configuration sets, which enable you to publish email sending events to Amazon CloudWatch, Amazon Kinesis Firehose, or Amazon Simple Notification Service (Amazon SNS). For information about using configuration sets, see <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/monitor-sending-activity.html">Monitoring Your Amazon SES Sending Activity</a> in the <i>Amazon SES Developer Guide.</i>  <note> When you create or update an event destination, you must provide one, and only one, destination. The destination can be Amazon CloudWatch, Amazon Kinesis Firehose, or Amazon Simple Notification Service (Amazon SNS). </note> You can execute this operation no more than once per second.

        :param self:
        :param EventDestination:  The event destination object that you want to apply to the specified configuration set.:<br/>ConfigurationSetName:  The name of the configuration set that contains the event destination that you want to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SNSDestination</i> An object that contains the topic ARN associated with an Amazon Simple Notification Service (Amazon SNS) event destination.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SNSDestination</b>: An object that contains the topic ARN associated with an Amazon Simple Notification Service (Amazon SNS) event destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchDestination</i> An object that contains the names, default values, and sources of the dimensions associated with an Amazon CloudWatch event destination.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CloudWatchDestination</b>: An object that contains the names, default values, and sources of the dimensions associated with an Amazon CloudWatch event destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KinesisFirehoseDestination</i> An object that contains the delivery stream ARN and the IAM role ARN associated with an Amazon Kinesis Firehose event destination.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KinesisFirehoseDestination</b>: An object that contains the delivery stream ARN and the IAM role ARN associated with an Amazon Kinesis Firehose event destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MatchingEventTypes</i> The type of email sending events to publish to the event destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Sets whether Amazon SES publishes events to this destination when you send an email with the associated configuration set. Set to <code>true</code> to enable publishing to this destination; set to <code>false</code> to prevent publishing to this destination. The default value is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the event destination. The name must: <ul> <li> This value can only contain ASCII letters (a-z, A-Z), numbers (0-9), underscores (_), or dashes (-). </li> <li> Contain less than 64 characters. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DeliveryStreamARN</i> The ARN of the Amazon Kinesis Firehose stream that email sending events should be published to.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IAMRoleARN</i> The ARN of the IAM role under which Amazon SES publishes email sending events to the Amazon Kinesis Firehose stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DimensionConfigurations</i> A list of dimensions upon which to categorize your emails when you publish email sending events to Amazon CloudWatch.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicARN</i> The ARN of the Amazon SNS topic that email sending events will be published to. An example of an Amazon SNS topic ARN is <code>arn:aws:sns:us-west-2:123456789012:MyTopic</code>. For more information about Amazon SNS topics, see the <a href="https://docs.aws.amazon.com/sns/latest/dg/CreateTopic.html">Amazon SNS Developer Guide</a>.<br/>
        :return:         """
        pass

    def update_configuration_set_reputation_metrics_enabled(self, Enabled: Enabled, ConfigurationSetName: str):
        """Enables or disables the publishing of reputation metrics for emails sent using a specific configuration set in a given AWS Region. Reputation metrics include bounce and complaint rates. These metrics are published to Amazon CloudWatch. By using CloudWatch, you can create alarms when bounce or complaint rates exceed certain thresholds. You can execute this operation no more than once per second.

        :param self:
        :param Enabled:  Describes whether or not Amazon SES will publish reputation metrics for the configuration set, such as bounce and complaint rates, to Amazon CloudWatch.<br/>ConfigurationSetName:  The name of the configuration set that you want to update.<br/>
        :return:         """
        pass

    def update_configuration_set_sending_enabled(self, Enabled: Enabled, ConfigurationSetName: str):
        """Enables or disables email sending for messages sent using a specific configuration set in a given AWS Region. You can use this operation in conjunction with Amazon CloudWatch alarms to temporarily pause email sending for a configuration set when the reputation metrics for that configuration set (such as your bounce on complaint rate) exceed certain thresholds. You can execute this operation no more than once per second.

        :param self:
        :param Enabled:  Describes whether email sending is enabled or disabled for the configuration set. <br/>ConfigurationSetName:  The name of the configuration set that you want to update.<br/>
        :return:         """
        pass

    def update_configuration_set_tracking_options(self, TrackingOptions: TrackingOptions, ConfigurationSetName: str) -> UpdateConfigurationSetTrackingOptionsResponse:
        """Modifies an association between a configuration set and a custom domain for open and click event tracking.  By default, images and links used for tracking open and click events are hosted on domains operated by Amazon SES. You can configure a subdomain of your own to handle these events. For information about using custom domains, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/configure-custom-open-click-domains.html">Amazon SES Developer Guide</a>.

        :param self:
        :param TrackingOptions:  A domain that is used to redirect email recipients to an Amazon SES-operated domain. This domain captures open and click events generated by Amazon SES emails. For more information, see <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/configure-custom-open-click-domains.html">Configuring Custom Domains to Handle Open and Click Tracking</a> in the <i>Amazon SES Developer Guide</i>.:<br/>ConfigurationSetName:  The name of the configuration set for which you want to update the custom tracking domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CustomRedirectDomain</i> The custom subdomain that will be used to redirect email recipients to the Amazon SES event tracking domain.<br/>
        :return:         """
        pass

    def update_custom_verification_email_template(self, TemplateName: str, FromEmailAddress: str=None, TemplateSubject: str=None, TemplateContent: str=None, SuccessRedirectionURL: str=None, FailureRedirectionURL: str=None):
        """Updates an existing custom verification email template. For more information about custom verification email templates, see <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/custom-verification-emails.html">Using Custom Verification Email Templates</a> in the <i>Amazon SES Developer Guide</i>. You can execute this operation no more than once per second.

        :param self:
        :param FailureRedirectionURL:  The URL that the recipient of the verification email is sent to if his or her address is not successfully verified.<br/>SuccessRedirectionURL:  The URL that the recipient of the verification email is sent to if his or her address is successfully verified.<br/>TemplateContent:  The content of the custom verification email. The total size of the email must be less than 10 MB. The message body may contain HTML, with some limitations. For more information, see <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/custom-verification-emails.html#custom-verification-emails-faq">Custom Verification Email Frequently Asked Questions</a> in the <i>Amazon SES Developer Guide</i>.<br/>TemplateSubject:  The subject line of the custom verification email.<br/>FromEmailAddress:  The email address that the custom verification email is sent from.<br/>TemplateName:  The name of the custom verification email template that you want to update.<br/>
        :return:         """
        pass

    def update_receipt_rule(self, Rule: ReceiptRule, RuleSetName: str) -> UpdateReceiptRuleResponse:
        """Updates a receipt rule. For information about managing receipt rules, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/receiving-email-managing-receipt-rules.html">Amazon SES Developer Guide</a>. You can execute this operation no more than once per second.

        :param self:
        :param Rule:  A data structure that contains the updated receipt rule information.:<br/>RuleSetName:  The name of the receipt rule set that the receipt rule belongs to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ScanEnabled</i> If <code>true</code>, then messages that this receipt rule applies to are scanned for spam and viruses. The default value is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Actions</i> An ordered list of actions to perform on messages that match at least one of the recipient email addresses or domains specified in the receipt rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Recipients</i> The recipient domains and email addresses that the receipt rule applies to. If this field is not specified, this rule will match all recipients under all verified domains.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TlsPolicy</i> Specifies whether Amazon SES should require that incoming email is delivered over a connection encrypted with Transport Layer Security (TLS). If this parameter is set to <code>Require</code>, Amazon SES will bounce emails that are not received over TLS. The default is <code>Optional</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> If <code>true</code>, the receipt rule is active. The default value is <code>false</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the receipt rule. The name must: <ul> <li> This value can only contain ASCII letters (a-z, A-Z), numbers (0-9), underscores (_), or dashes (-). </li> <li> Start and end with a letter or number. </li> <li> Contain less than 64 characters. </li> </ul><br/>
        :return:         """
        pass

    def update_template(self, Template: Template) -> UpdateTemplateResponse:
        """Updates an email template. Email templates enable you to send personalized email to one or more destinations in a single API operation. For more information, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/send-personalized-email-api.html">Amazon SES Developer Guide</a>. You can execute this operation no more than once per second.

        :param self:
        :param Template:  The content of the email, composed of a subject line, an HTML part, and a text-only part.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HtmlPart</i> The HTML body of the email.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TextPart</i> The email body that will be visible to recipients whose email clients do not display HTML.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubjectPart</i> The subject line of the email.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TemplateName</i> The name of the template. You will refer to this name when you send email using the <code>SendTemplatedEmail</code> or <code>SendBulkTemplatedEmail</code> operations.<br/>
        :return:         """
        pass

    def verify_domain_dkim(self, Domain: str) -> VerifyDomainDkimResponse:
        """Returns a set of DKIM tokens for a domain identity. <important> When you execute the <code>VerifyDomainDkim</code> operation, the domain that you specify is added to the list of identities that are associated with your account. This is true even if you haven't already associated the domain with your account by using the <code>VerifyDomainIdentity</code> operation. However, you can't send email from the domain until you either successfully <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/verify-domains.html">verify it</a> or you successfully <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/easy-dkim.html">set up DKIM for it</a>. </important> You use the tokens that are generated by this operation to create CNAME records. When Amazon SES detects that you've added these records to the DNS configuration for a domain, you can start sending email from that domain. You can start sending email even if you haven't added the TXT record provided by the VerifyDomainIdentity operation to the DNS configuration for your domain. All email that you send from the domain is authenticated using DKIM. To create the CNAME records for DKIM authentication, use the following values: <ul> <li>  <b>Name</b>: <i>token</i>._domainkey.<i>example.com</i>  </li> <li>  <b>Type</b>: CNAME </li> <li>  <b>Value</b>: <i>token</i>.dkim.amazonses.com </li> </ul> In the preceding example, replace <i>token</i> with one of the tokens that are generated when you execute this operation. Replace <i>example.com</i> with your domain. Repeat this process for each token that's generated by this operation. You can execute this operation no more than once per second.

        :param self:
        :param Domain:  The name of the domain to be verified for Easy DKIM signing.<br/>
        :return: DkimTokens:  A set of character strings that represent the domain's identity. If the identity is an email address, the tokens represent the domain of that address. Using these tokens, you need to create DNS CNAME records that point to DKIM public keys that are hosted by Amazon SES. Amazon Web Services eventually detects that you've updated your DNS records. This detection process might take up to 72 hours. After successful detection, Amazon SES is able to DKIM-sign email originating from that domain. (This only applies to domain identities, not email address identities.) For more information about creating DNS records using DKIM tokens, see the <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/easy-dkim.html">Amazon SES Developer Guide</a>.<br/>        """
        pass

    def verify_domain_identity(self, Domain: str) -> VerifyDomainIdentityResponse:
        """Adds a domain to the list of identities for your Amazon SES account in the current AWS Region and attempts to verify it. For more information about verifying domains, see <a href="https://docs.aws.amazon.com/ses/latest/DeveloperGuide/verify-addresses-and-domains.html">Verifying Email Addresses and Domains</a> in the <i>Amazon SES Developer Guide.</i>  You can execute this operation no more than once per second.

        :param self:
        :param Domain:  The domain to be verified.<br/>
        :return: VerificationToken:  A TXT record that you must place in the DNS settings of the domain to complete domain verification with Amazon SES. As Amazon SES searches for the TXT record, the domain's verification status is "Pending". When Amazon SES detects the record, the domain's verification status changes to "Success". If Amazon SES is unable to detect the record within 72 hours, the domain's verification status changes to "Failed." In that case, if you still want to verify the domain, you must restart the verification process from the beginning.<br/>        """
        pass

    def verify_email_address(self, EmailAddress: str):
        """Deprecated. Use the <code>VerifyEmailIdentity</code> operation to verify a new email address.

        :param self:
        :param EmailAddress:  The email address to be verified.<br/>
        :return:         """
        pass

    def verify_email_identity(self, EmailAddress: str) -> VerifyEmailIdentityResponse:
        """Adds an email address to the list of identities for your Amazon SES account in the current AWS region and attempts to verify it. As a result of executing this operation, a verification email is sent to the specified address. You can execute this operation no more than once per second.

        :param self:
        :param EmailAddress:  The email address to be verified.<br/>
        :return:         """
        pass



class Shield(BaseClient):
    """<fullname>AWS Shield Advanced</fullname> This is the <i>AWS Shield Advanced API Reference</i>. This guide is for developers who need detailed information about the AWS Shield Advanced API actions, data types, and errors. For detailed information about AWS WAF and AWS Shield Advanced features and an overview of how to use the AWS WAF and AWS Shield Advanced APIs, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF and AWS Shield Developer Guide</a>."""


    class ListProtectionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_protections` for valid parameters"""
            pass
    class ListAttacksPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_attacks` for valid parameters"""
            pass


    class AssociateDRTLogBucketRequest(Mapping):
        pass
    
    class AssociateDRTLogBucketResponse(Mapping):
        pass
    
    class AssociateDRTRoleRequest(Mapping):
        pass
    
    class AssociateDRTRoleResponse(Mapping):
        pass
    
    class CreateProtectionRequest(Mapping):
        pass
    
    class CreateProtectionResponse(Mapping):
        pass
    
    class CreateSubscriptionRequest(Mapping):
        pass
    
    class CreateSubscriptionResponse(Mapping):
        pass
    
    class DeleteProtectionRequest(Mapping):
        pass
    
    class DeleteProtectionResponse(Mapping):
        pass
    
    class DeleteSubscriptionRequest(Mapping):
        pass
    
    class DeleteSubscriptionResponse(Mapping):
        pass
    
    class DescribeAttackRequest(Mapping):
        pass
    
    class DescribeAttackResponse(Mapping):
        pass
    
    class DescribeDRTAccessRequest(Mapping):
        pass
    
    class DescribeDRTAccessResponse(Mapping):
        pass
    
    class DescribeEmergencyContactSettingsRequest(Mapping):
        pass
    
    class DescribeEmergencyContactSettingsResponse(Mapping):
        pass
    
    class DescribeProtectionRequest(Mapping):
        pass
    
    class DescribeProtectionResponse(Mapping):
        pass
    
    class DescribeSubscriptionRequest(Mapping):
        pass
    
    class DescribeSubscriptionResponse(Mapping):
        pass
    
    class DisassociateDRTLogBucketRequest(Mapping):
        pass
    
    class DisassociateDRTLogBucketResponse(Mapping):
        pass
    
    class DisassociateDRTRoleRequest(Mapping):
        pass
    
    class DisassociateDRTRoleResponse(Mapping):
        pass
    
    class GetSubscriptionStateRequest(Mapping):
        pass
    
    class GetSubscriptionStateResponse(Mapping):
        pass
    
    class ListAttacksRequest(Mapping):
        pass
    
    class ListAttacksResponse(Mapping):
        pass
    
    class ResourceArn(object):
        pass
    
    class TimeRange(Mapping):
        pass
    
    class ListProtectionsRequest(Mapping):
        pass
    
    class ListProtectionsResponse(Mapping):
        pass
    
    class UpdateEmergencyContactSettingsRequest(Mapping):
        pass
    
    class UpdateEmergencyContactSettingsResponse(Mapping):
        pass
    
    class EmergencyContact(Mapping):
        pass
    
    class UpdateSubscriptionRequest(Mapping):
        pass
    
    class UpdateSubscriptionResponse(Mapping):
        pass
    
    def associate_drt_log_bucket(self, LogBucket: str) -> AssociateDRTLogBucketResponse:
        """Authorizes the DDoS Response team (DRT) to access the specified Amazon S3 bucket containing your flow logs. You can associate up to 10 Amazon S3 buckets with your subscription. To use the services of the DRT and make an <code>AssociateDRTLogBucket</code> request, you must be subscribed to the <a href="https://aws.amazon.com/premiumsupport/business-support/">Business Support plan</a> or the <a href="https://aws.amazon.com/premiumsupport/enterprise-support/">Enterprise Support plan</a>.

        :param self:
        :param LogBucket:  The Amazon S3 bucket that contains your flow logs.<br/>
        :return:         """
        pass

    def associate_drt_role(self, RoleArn: str) -> AssociateDRTRoleResponse:
        """Authorizes the DDoS Response team (DRT), using the specified role, to access your AWS account to assist with DDoS attack mitigation during potential attacks. This enables the DRT to inspect your AWS WAF configuration and create or update AWS WAF rules and web ACLs. You can associate only one <code>RoleArn</code> with your subscription. If you submit an <code>AssociateDRTRole</code> request for an account that already has an associated role, the new <code>RoleArn</code> will replace the existing <code>RoleArn</code>.  Prior to making the <code>AssociateDRTRole</code> request, you must attach the <a href="https://console.aws.amazon.com/iam/home?#/policies/arn:aws:iam::aws:policy/service-role/AWSShieldDRTAccessPolicy">AWSShieldDRTAccessPolicy</a> managed policy to the role you will specify in the request. For more information see <a href=" https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies_manage-attach-detach.html">Attaching and Detaching IAM Policies</a>. The role must also trust the service principal <code> drt.shield.amazonaws.com</code>. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_policies_elements_principal.html">IAM JSON Policy Elements: Principal</a>. The DRT will have access only to your AWS WAF and Shield resources. By submitting this request, you authorize the DRT to inspect your AWS WAF and Shield configuration and create and update AWS WAF rules and web ACLs on your behalf. The DRT takes these actions only if explicitly authorized by you. You must have the <code>iam:PassRole</code> permission to make an <code>AssociateDRTRole</code> request. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_use_passrole.html">Granting a User Permissions to Pass a Role to an AWS Service</a>.  To use the services of the DRT and make an <code>AssociateDRTRole</code> request, you must be subscribed to the <a href="https://aws.amazon.com/premiumsupport/business-support/">Business Support plan</a> or the <a href="https://aws.amazon.com/premiumsupport/enterprise-support/">Enterprise Support plan</a>.

        :param self:
        :param RoleArn:  The Amazon Resource Name (ARN) of the role the DRT will use to access your AWS account. Prior to making the <code>AssociateDRTRole</code> request, you must attach the <a href="https://console.aws.amazon.com/iam/home?#/policies/arn:aws:iam::aws:policy/service-role/AWSShieldDRTAccessPolicy">AWSShieldDRTAccessPolicy</a> managed policy to this role. For more information see <a href=" https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies_manage-attach-detach.html">Attaching and Detaching IAM Policies</a>.<br/>
        :return:         """
        pass

    def create_protection(self, ResourceArn: str, Name: str) -> CreateProtectionResponse:
        """Enables AWS Shield Advanced for a specific AWS resource. The resource can be an Amazon CloudFront distribution, Elastic Load Balancing load balancer, AWS Global Accelerator accelerator, Elastic IP Address, or an Amazon Route 53 hosted zone. You can add protection to only a single resource with each CreateProtection request. If you want to add protection to multiple resources at once, use the <a href="https://console.aws.amazon.com/waf/">AWS WAF console</a>. For more information see <a href="https://docs.aws.amazon.com/waf/latest/developerguide/getting-started-ddos.html">Getting Started with AWS Shield Advanced</a> and <a href="https://docs.aws.amazon.com/waf/latest/developerguide/configure-new-protection.html">Add AWS Shield Advanced Protection to more AWS Resources</a>.

        :param self:
        :param ResourceArn:  The ARN (Amazon Resource Name) of the resource to be protected. The ARN should be in one of the following formats: <ul> <li> For an Application Load Balancer: <code>arn:aws:elasticloadbalancing:<i>region</i>:<i>account-id</i>:loadbalancer/app/<i>load-balancer-name</i>/<i>load-balancer-id</i> </code>  </li> <li> For an Elastic Load Balancer (Classic Load Balancer): <code>arn:aws:elasticloadbalancing:<i>region</i>:<i>account-id</i>:loadbalancer/<i>load-balancer-name</i> </code>  </li> <li> For an AWS CloudFront distribution: <code>arn:aws:cloudfront::<i>account-id</i>:distribution/<i>distribution-id</i> </code>  </li> <li> For an AWS Global Accelerator accelerator: <code>arn:aws:globalaccelerator::<i>account-id</i>:accelerator/<i>accelerator-id</i> </code>  </li> <li> For Amazon Route 53: <code>arn:aws:route53:::hostedzone/<i>hosted-zone-id</i> </code>  </li> <li> For an Elastic IP address: <code>arn:aws:ec2:<i>region</i>:<i>account-id</i>:eip-allocation/<i>allocation-id</i> </code>  </li> </ul><br/>Name:  Friendly name for the <code>Protection</code> you are creating.<br/>
        :return: ProtectionId:  The unique identifier (ID) for the <a>Protection</a> object that is created.<br/>        """
        pass

    def create_subscription(self) -> CreateSubscriptionResponse:
        """Activates AWS Shield Advanced for an account. As part of this request you can specify <code>EmergencySettings</code> that automaticaly grant the DDoS response team (DRT) needed permissions to assist you during a suspected DDoS attack. For more information see <a href="https://docs.aws.amazon.com/waf/latest/developerguide/authorize-DRT.html">Authorize the DDoS Response Team to Create Rules and Web ACLs on Your Behalf</a>. When you initally create a subscription, your subscription is set to be automatically renewed at the end of the existing subscription period. You can change this by submitting an <code>UpdateSubscription</code> request. 

        :param self:
        :param 
        :return:         """
        pass

    def delete_protection(self, ProtectionId: str) -> DeleteProtectionResponse:
        """Deletes an AWS Shield Advanced <a>Protection</a>.

        :param self:
        :param ProtectionId:  The unique identifier (ID) for the <a>Protection</a> object to be deleted.<br/>
        :return:         """
        pass

    def delete_subscription(self) -> DeleteSubscriptionResponse:
        """Removes AWS Shield Advanced from an account. AWS Shield Advanced requires a 1-year subscription commitment. You cannot delete a subscription prior to the completion of that commitment. 

        :param self:
        :param 
        :return:         """
        pass

    def describe_attack(self, AttackId: str) -> DescribeAttackResponse:
        """Describes the details of a DDoS attack. 

        :param self:
        :param AttackId:  The unique identifier (ID) for the attack that to be described.<br/>
        :return: Attack:  The attack that is described.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Mitigations</i> List of mitigation actions taken for the attack.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttackProperties</i> The array of <a>AttackProperty</a> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttackCounters</i> List of counters that describe the attack for the specified time period.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> The time the attack ended, in Unix time in seconds. For more information see <a href="http://docs.aws.amazon.com/cli/latest/userguide/cli-using-param.html#parameter-types">timestamp</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTime</i> The time the attack started, in Unix time in seconds. For more information see <a href="http://docs.aws.amazon.com/cli/latest/userguide/cli-using-param.html#parameter-types">timestamp</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubResources</i> If applicable, additional detail about the resource being attacked, for example, IP address or URL.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceArn</i> The ARN (Amazon Resource Name) of the resource that was attacked.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttackId</i> The unique identifier (ID) of the attack.<br/>        """
        pass

    def describe_drt_access(self) -> DescribeDRTAccessResponse:
        """Returns the current role and list of Amazon S3 log buckets used by the DDoS Response team (DRT) to access your AWS account while assisting with attack mitigation.

        :param self:
        :param 
        :return: LogBucketList:  The list of Amazon S3 buckets accessed by the DRT.<br/>RoleArn:  The Amazon Resource Name (ARN) of the role the DRT used to access your AWS account.<br/>        """
        pass

    def describe_emergency_contact_settings(self) -> DescribeEmergencyContactSettingsResponse:
        """Lists the email addresses that the DRT can use to contact you during a suspected attack.

        :param self:
        :param 
        :return: EmergencyContactList:  A list of email addresses that the DRT can use to contact you during a suspected attack.<br/>        """
        pass

    def describe_protection(self, ProtectionId: str=None, ResourceArn: str=None) -> DescribeProtectionResponse:
        """Lists the details of a <a>Protection</a> object.

        :param self:
        :param ResourceArn:  The ARN (Amazon Resource Name) of the AWS resource for the <a>Protection</a> object that is described. When submitting the <code>DescribeProtection</code> request you must provide either the <code>ResourceArn</code> or the <code>ProtectionID</code>, but not both.<br/>ProtectionId:  The unique identifier (ID) for the <a>Protection</a> object that is described. When submitting the <code>DescribeProtection</code> request you must provide either the <code>ResourceArn</code> or the <code>ProtectionID</code>, but not both.<br/>
        :return: Protection:  The <a>Protection</a> object that is described.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceArn</i> The ARN (Amazon Resource Name) of the AWS resource that is protected.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The friendly name of the protection. For example, <code>My CloudFront distributions</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The unique identifier (ID) of the protection.<br/>        """
        pass

    def describe_subscription(self) -> DescribeSubscriptionResponse:
        """Provides details about the AWS Shield Advanced subscription for an account.

        :param self:
        :param 
        :return: Subscription:  The AWS Shield Advanced subscription details for an account.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Limits</i> Specifies how many protections of a given type you can create.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoRenew</i> If <code>ENABLED</code>, the subscription will be automatically renewed at the end of the existing subscription period. When you initally create a subscription, <code>AutoRenew</code> is set to <code>ENABLED</code>. You can change this by submitting an <code>UpdateSubscription</code> request. If the <code>UpdateSubscription</code> request does not included a value for <code>AutoRenew</code>, the existing value for <code>AutoRenew</code> remains unchanged.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeCommitmentInSeconds</i> The length, in seconds, of the AWS Shield Advanced subscription for the account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> The date and time your subscription will end.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTime</i> The start time of the subscription, in Unix time in seconds. For more information see <a href="http://docs.aws.amazon.com/cli/latest/userguide/cli-using-param.html#parameter-types">timestamp</a>.<br/>        """
        pass

    def disassociate_drt_log_bucket(self, LogBucket: str) -> DisassociateDRTLogBucketResponse:
        """Removes the DDoS Response team's (DRT) access to the specified Amazon S3 bucket containing your flow logs. To make a <code>DisassociateDRTLogBucket</code> request, you must be subscribed to the <a href="https://aws.amazon.com/premiumsupport/business-support/">Business Support plan</a> or the <a href="https://aws.amazon.com/premiumsupport/enterprise-support/">Enterprise Support plan</a>. However, if you are not subscribed to one of these support plans, but had been previously and had granted the DRT access to your account, you can submit a <code>DisassociateDRTLogBucket</code> request to remove this access.

        :param self:
        :param LogBucket:  The Amazon S3 bucket that contains your flow logs.<br/>
        :return:         """
        pass

    def disassociate_drt_role(self) -> DisassociateDRTRoleResponse:
        """Removes the DDoS Response team's (DRT) access to your AWS account. To make a <code>DisassociateDRTRole</code> request, you must be subscribed to the <a href="https://aws.amazon.com/premiumsupport/business-support/">Business Support plan</a> or the <a href="https://aws.amazon.com/premiumsupport/enterprise-support/">Enterprise Support plan</a>. However, if you are not subscribed to one of these support plans, but had been previously and had granted the DRT access to your account, you can submit a <code>DisassociateDRTRole</code> request to remove this access.

        :param self:
        :param 
        :return:         """
        pass

    def get_subscription_state(self) -> GetSubscriptionStateResponse:
        """Returns the <code>SubscriptionState</code>, either <code>Active</code> or <code>Inactive</code>.

        :param self:
        :param 
        :return: SubscriptionState:  The status of the subscription.<br/>        """
        pass

    def list_attacks(self, ResourceArns: List[ResourceArn]=None, StartTime: TimeRange=None, EndTime: TimeRange=None, NextToken: str=None, MaxResults: int=None) -> ListAttacksResponse:
        """Returns all ongoing DDoS attacks or all DDoS attacks during a specified time period.

        :param self:
        :param MaxResults:  The maximum number of <a>AttackSummary</a> objects to be returned. If this is left blank, the first 20 results will be returned. This is a maximum value; it is possible that AWS WAF will return the results in smaller batches. That is, the number of <a>AttackSummary</a> objects returned could be less than <code>MaxResults</code>, even if there are still more <a>AttackSummary</a> objects yet to return. If there are more <a>AttackSummary</a> objects to return, AWS WAF will always also return a <code>NextToken</code>.<br/>NextToken:  The <code>ListAttacksRequest.NextMarker</code> value from a previous call to <code>ListAttacksRequest</code>. Pass null if this is the first call.<br/>EndTime:  The end of the time period for the attacks. This is a <code>timestamp</code> type. The sample request above indicates a <code>number</code> type because the default used by WAF is Unix time in seconds. However any valid <a href="http://docs.aws.amazon.com/cli/latest/userguide/cli-using-param.html#parameter-types">timestamp format</a> is allowed. :<br/>StartTime:  The start of the time period for the attacks. This is a <code>timestamp</code> type. The sample request above indicates a <code>number</code> type because the default used by WAF is Unix time in seconds. However any valid <a href="http://docs.aws.amazon.com/cli/latest/userguide/cli-using-param.html#parameter-types">timestamp format</a> is allowed. :<br/>ResourceArns:  The ARN (Amazon Resource Name) of the resource that was attacked. If this is left blank, all applicable resources for this account will be included.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ToExclusive</i> The end time, in Unix time in seconds. For more information see <a href="http://docs.aws.amazon.com/cli/latest/userguide/cli-using-param.html#parameter-types">timestamp</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FromInclusive</i> The start time, in Unix time in seconds. For more information see <a href="http://docs.aws.amazon.com/cli/latest/userguide/cli-using-param.html#parameter-types">timestamp</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ToExclusive</i> The end time, in Unix time in seconds. For more information see <a href="http://docs.aws.amazon.com/cli/latest/userguide/cli-using-param.html#parameter-types">timestamp</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FromInclusive</i> The start time, in Unix time in seconds. For more information see <a href="http://docs.aws.amazon.com/cli/latest/userguide/cli-using-param.html#parameter-types">timestamp</a>.<br/>
        :return: NextToken:  The token returned by a previous call to indicate that there is more data available. If not null, more results are available. Pass this value for the <code>NextMarker</code> parameter in a subsequent call to <code>ListAttacks</code> to retrieve the next set of items. AWS WAF might return the list of <a>AttackSummary</a> objects in batches smaller than the number specified by MaxResults. If there are more <a>AttackSummary</a> objects to return, AWS WAF will always also return a <code>NextToken</code>.<br/>AttackSummaries:  The attack information for the specified time range.<br/>        """
        pass

    def list_protections(self, NextToken: str=None, MaxResults: int=None) -> ListProtectionsResponse:
        """Lists all <a>Protection</a> objects for the account.

        :param self:
        :param MaxResults:  The maximum number of <a>Protection</a> objects to be returned. If this is left blank the first 20 results will be returned. This is a maximum value; it is possible that AWS WAF will return the results in smaller batches. That is, the number of <a>Protection</a> objects returned could be less than <code>MaxResults</code>, even if there are still more <a>Protection</a> objects yet to return. If there are more <a>Protection</a> objects to return, AWS WAF will always also return a <code>NextToken</code>.<br/>NextToken:  The <code>ListProtectionsRequest.NextToken</code> value from a previous call to <code>ListProtections</code>. Pass null if this is the first call.<br/>
        :return: NextToken:  If you specify a value for <code>MaxResults</code> and you have more Protections than the value of MaxResults, AWS Shield Advanced returns a NextToken value in the response that allows you to list another group of Protections. For the second and subsequent ListProtections requests, specify the value of NextToken from the previous response to get information about another batch of Protections. AWS WAF might return the list of <a>Protection</a> objects in batches smaller than the number specified by MaxResults. If there are more <a>Protection</a> objects to return, AWS WAF will always also return a <code>NextToken</code>.<br/>Protections:  The array of enabled <a>Protection</a> objects.<br/>        """
        pass

    def update_emergency_contact_settings(self, EmergencyContactList: List[EmergencyContact]=None) -> UpdateEmergencyContactSettingsResponse:
        """Updates the details of the list of email addresses that the DRT can use to contact you during a suspected attack.

        :param self:
        :param EmergencyContactList:  A list of email addresses that the DRT can use to contact you during a suspected attack.<br/>
        :return:         """
        pass

    def update_subscription(self, AutoRenew: str=None) -> UpdateSubscriptionResponse:
        """Updates the details of an existing subscription. Only enter values for parameters you want to change. Empty parameters are not updated.

        :param self:
        :param AutoRenew:  When you initally create a subscription, <code>AutoRenew</code> is set to <code>ENABLED</code>. If <code>ENABLED</code>, the subscription will be automatically renewed at the end of the existing subscription period. You can change this by submitting an <code>UpdateSubscription</code> request. If the <code>UpdateSubscription</code> request does not included a value for <code>AutoRenew</code>, the existing value for <code>AutoRenew</code> remains unchanged.<br/>
        :return:         """
        pass



class signer(BaseClient):
    """You can use Code Signing for Amazon FreeRTOS (AWS Signer) to sign code that you created for any of the IoT devices that Amazon Web Services supports. AWS Signer is integrated with Amazon FreeRTOS, AWS Certificate Manager, and AWS CloudTrail. Amazon FreeRTOS customers can use AWS Signer to sign code images before making them available for microcontrollers. You can use ACM to import third-party certificates to be used by AWS Signer. For general information about using AWS Signer, see the <a href="http://docs.aws.amazon.com/signer/latest/developerguide/Welcome.html">Code Signing for Amazon FreeRTOS Developer Guide</a>."""

    class SuccessfulSigningJobWaiter(Waiter):
        def wait(self, **kwargs):
            """see function `describe_signing_job` for valid parameters"""
            pass


    class ListSigningJobsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_signing_jobs` for valid parameters"""
            pass
    class ListSigningPlatformsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_signing_platforms` for valid parameters"""
            pass
    class ListSigningProfilesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_signing_profiles` for valid parameters"""
            pass


    class CancelSigningProfileRequest(Mapping):
        pass
    
    class DescribeSigningJobRequest(Mapping):
        pass
    
    class DescribeSigningJobResponse(Mapping):
        pass
    
    class GetSigningPlatformRequest(Mapping):
        pass
    
    class GetSigningPlatformResponse(Mapping):
        pass
    
    class GetSigningProfileRequest(Mapping):
        pass
    
    class GetSigningProfileResponse(Mapping):
        pass
    
    class ListSigningJobsRequest(Mapping):
        pass
    
    class ListSigningJobsResponse(Mapping):
        pass
    
    class ListSigningPlatformsRequest(Mapping):
        pass
    
    class ListSigningPlatformsResponse(Mapping):
        pass
    
    class ListSigningProfilesRequest(Mapping):
        pass
    
    class ListSigningProfilesResponse(Mapping):
        pass
    
    class bool(object):
        pass
    
    class PutSigningProfileRequest(Mapping):
        pass
    
    class PutSigningProfileResponse(Mapping):
        pass
    
    class SigningMaterial(Mapping):
        pass
    
    class SigningPlatformOverrides(Mapping):
        pass
    
    class SigningParameters(object):
        pass
    
    class StartSigningJobRequest(Mapping):
        pass
    
    class StartSigningJobResponse(Mapping):
        pass
    
    class Source(Mapping):
        pass
    
    class Destination(Mapping):
        pass
    
    def cancel_signing_profile(self, profileName: str):
        """Changes the state of an <code>ACTIVE</code> signing profile to <code>CANCELED</code>. A canceled profile is still viewable with the <code>ListSigningProfiles</code> operation, but it cannot perform new signing jobs, and is deleted two years after cancelation.

        :param self:
        :param profileName:  The name of the signing profile to be canceled.<br/>
        :return:         """
        pass

    def describe_signing_job(self, jobId: str) -> DescribeSigningJobResponse:
        """Returns information about a specific code signing job. You specify the job by using the <code>jobId</code> value that is returned by the <a>StartSigningJob</a> operation. 

        :param self:
        :param jobId:  The ID of the signing job on input.<br/>
        :return: signedObject:  Name of the S3 bucket where the signed code image is saved by AWS Signer.:<br/>statusReason:  String value that contains the status reason.<br/>status:  Status of the signing job.<br/>requestedBy:  The IAM principal that requested the signing job.<br/>completedAt:  Date and time that the signing job was completed.<br/>createdAt:  Date and time that the signing job was created.<br/>signingParameters:  Map of user-assigned key-value pairs used during signing. These values contain any information that you specified for use in your signing job. <br/>overrides:  A list of any overrides that were applied to the signing operation.:<br/>profileName:  The name of the profile that initiated the signing operation.<br/>platformId:  The microcontroller platform to which your signed code image will be distributed.<br/>signingMaterial:  Amazon Resource Name (ARN) of your code signing certificate.:<br/>source:  The object that contains the name of your S3 bucket or your raw code.:<br/>jobId:  The ID of the signing job on output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>s3</i> The <code>S3Source</code> object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>s3</b>: The <code>S3Source</code> object.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> Version of your source image in your version enabled S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>key</i> Key name of the bucket object that contains your unsigned code.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>bucketName</i> Name of the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>certificateArn</i> The Amazon Resource Name (ARN) of the certificates that is used to sign your code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>signingConfiguration</i> A signing configuration that overrides the default encryption or hash algorithm of a signing job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>signingConfiguration</b>: A signing configuration that overrides the default encryption or hash algorithm of a signing job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>hashAlgorithm</i> A specified override of the default hash algorithm that is used in an AWS Signer job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>encryptionAlgorithm</i> A specified override of the default encryption algorithm that is used in an AWS Signer job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>s3</i> The <code>S3SignedObject</code>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>s3</b>: The <code>S3SignedObject</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>key</i> Key name that uniquely identifies a signed code image in your bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>bucketName</i> Name of the S3 bucket.<br/>        """
        pass

    def get_signing_platform(self, platformId: str) -> GetSigningPlatformResponse:
        """Returns information on a specific signing platform.

        :param self:
        :param platformId:  The ID of the target signing platform.<br/>
        :return: maxSizeInMB:  The maximum size (in MB) of the payload that can be signed by the target platform.<br/>signingImageFormat:  The format of the target platform's signing image.:<br/>signingConfiguration:  A list of configurations applied to the target platform at signing.:<br/>category:  The category type of the target signing platform.<br/>target:  The validation template that is used by the target signing platform.<br/>partner:  A list of partner entities that use the target signing platform.<br/>displayName:  The display name of the target signing platform.<br/>platformId:  The ID of the target signing platform.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>hashAlgorithmOptions</i> The hash algorithm options that are available for an AWS Signer job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>hashAlgorithmOptions</b>: The hash algorithm options that are available for an AWS Signer job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>encryptionAlgorithmOptions</i> The encryption algorithm options that are available for an AWS Signer job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>encryptionAlgorithmOptions</b>: The encryption algorithm options that are available for an AWS Signer job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultValue</i> The default encryption algorithm that is used by an AWS Signer job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>allowedValues</i> The set of accepted encryption algorithms that are allowed in an AWS Signer job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultValue</i> The default hash algorithm that is used in an AWS Signer job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>allowedValues</i> The set of accepted hash algorithms allowed in an AWS Signer job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultFormat</i> The default format of an AWS Signer signing image.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>supportedFormats</i> The supported formats of an AWS Signer signing image.<br/>        """
        pass

    def get_signing_profile(self, profileName: str) -> GetSigningProfileResponse:
        """Returns information on a specific signing profile.

        :param self:
        :param profileName:  The name of the target signing profile.<br/>
        :return: status:  The status of the target signing profile.<br/>signingParameters:  A map of key-value pairs for signing operations that is attached to the target signing profile.<br/>overrides:  A list of overrides applied by the target signing profile for signing operations.:<br/>platformId:  The ID of the platform that is used by the target signing profile.<br/>signingMaterial:  The ARN of the certificate that the target profile uses for signing operations.:<br/>profileName:  The name of the target signing profile.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>certificateArn</i> The Amazon Resource Name (ARN) of the certificates that is used to sign your code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>signingConfiguration</i> A signing configuration that overrides the default encryption or hash algorithm of a signing job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>signingConfiguration</b>: A signing configuration that overrides the default encryption or hash algorithm of a signing job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>hashAlgorithm</i> A specified override of the default hash algorithm that is used in an AWS Signer job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>encryptionAlgorithm</i> A specified override of the default encryption algorithm that is used in an AWS Signer job.<br/>        """
        pass

    def list_signing_jobs(self, status: str=None, platformId: str=None, requestedBy: str=None, maxResults: int=None, nextToken: str=None) -> ListSigningJobsResponse:
        """Lists all your signing jobs. You can use the <code>maxResults</code> parameter to limit the number of signing jobs that are returned in the response. If additional jobs remain to be listed, AWS Signer returns a <code>nextToken</code> value. Use this value in subsequent calls to <code>ListSigningJobs</code> to fetch the remaining values. You can continue calling <code>ListSigningJobs</code> with your <code>maxResults</code> parameter and with new values that AWS Signer returns in the <code>nextToken</code> parameter until all of your signing jobs have been returned. 

        :param self:
        :param nextToken:  String for specifying the next set of paginated results to return. After you receive a response with truncated results, use this parameter in a subsequent request. Set it to the value of <code>nextToken</code> from the response that you just received.<br/>maxResults:  Specifies the maximum number of items to return in the response. Use this parameter when paginating results. If additional items exist beyond the number you specify, the <code>nextToken</code> element is set in the response. Use the <code>nextToken</code> value in a subsequent request to retrieve additional items. <br/>requestedBy:  The IAM principal that requested the signing job.<br/>platformId:  The ID of microcontroller platform that you specified for the distribution of your code image.<br/>status:  A status value with which to filter your results.<br/>
        :return: nextToken:  String for specifying the next set of paginated results.<br/>jobs:  A list of your signing jobs.<br/>        """
        pass

    def list_signing_platforms(self, category: str=None, partner: str=None, target: str=None, maxResults: int=None, nextToken: str=None) -> ListSigningPlatformsResponse:
        """Lists all signing platforms available in AWS Signer that match the request parameters. If additional jobs remain to be listed, AWS Signer returns a <code>nextToken</code> value. Use this value in subsequent calls to <code>ListSigningJobs</code> to fetch the remaining values. You can continue calling <code>ListSigningJobs</code> with your <code>maxResults</code> parameter and with new values that AWS Signer returns in the <code>nextToken</code> parameter until all of your signing jobs have been returned.

        :param self:
        :param nextToken:  Value for specifying the next set of paginated results to return. After you receive a response with truncated results, use this parameter in a subsequent request. Set it to the value of <code>nextToken</code> from the response that you just received.<br/>maxResults:  The maximum number of results to be returned by this operation.<br/>target:  The validation template that is used by the target signing platform.<br/>partner:  Any partner entities connected to a signing platform.<br/>category:  The category type of a signing platform.<br/>
        :return: nextToken:  Value for specifying the next set of paginated results to return.<br/>platforms:  A list of all platforms that match the request parameters.<br/>        """
        pass

    def list_signing_profiles(self, includeCanceled: bool=None, maxResults: int=None, nextToken: str=None) -> ListSigningProfilesResponse:
        """Lists all available signing profiles in your AWS account. Returns only profiles with an <code>ACTIVE</code> status unless the <code>includeCanceled</code> request field is set to <code>true</code>. If additional jobs remain to be listed, AWS Signer returns a <code>nextToken</code> value. Use this value in subsequent calls to <code>ListSigningJobs</code> to fetch the remaining values. You can continue calling <code>ListSigningJobs</code> with your <code>maxResults</code> parameter and with new values that AWS Signer returns in the <code>nextToken</code> parameter until all of your signing jobs have been returned.

        :param self:
        :param nextToken:  Value for specifying the next set of paginated results to return. After you receive a response with truncated results, use this parameter in a subsequent request. Set it to the value of <code>nextToken</code> from the response that you just received.<br/>maxResults:  The maximum number of profiles to be returned.<br/>includeCanceled:  Designates whether to include profiles with the status of <code>CANCELED</code>.<br/>
        :return: nextToken:  Value for specifying the next set of paginated results to return.<br/>profiles:  A list of profiles that are available in the AWS account. This includes profiles with the status of <code>CANCELED</code> if the <code>includeCanceled</code> parameter is set to <code>true</code>.<br/>        """
        pass

    def put_signing_profile(self, platformId: str, signingMaterial: SigningMaterial, profileName: str, overrides: SigningPlatformOverrides=None, signingParameters: SigningParameters=None) -> PutSigningProfileResponse:
        """Creates a signing profile. A signing profile is an AWS Signer template that can be used to carry out a pre-defined signing job. For more information, see <a href="http://docs.aws.amazon.com/signer/latest/developerguide/gs-profile.html">http://docs.aws.amazon.com/signer/latest/developerguide/gs-profile.html</a> 

        :param self:
        :param signingParameters:  Map of key-value pairs for signing. These can include any information that you want to use during signing.<br/>overrides:  A subfield of <code>platform</code>. This specifies any different configuration options that you want to apply to the chosen platform (such as a different <code>hash-algorithm</code> or <code>signing-algorithm</code>).:<br/>platformId:  The ID of the signing profile to be created.<br/>signingMaterial:  The AWS Certificate Manager certificate that will be used to sign code with the new signing profile.:<br/>profileName:  The name of the signing profile to be created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>certificateArn</i> The Amazon Resource Name (ARN) of the certificates that is used to sign your code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>signingConfiguration</i> A signing configuration that overrides the default encryption or hash algorithm of a signing job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>signingConfiguration</b>: A signing configuration that overrides the default encryption or hash algorithm of a signing job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>hashAlgorithm</i> A specified override of the default hash algorithm that is used in an AWS Signer job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>encryptionAlgorithm</i> A specified override of the default encryption algorithm that is used in an AWS Signer job.<br/>
        :return: arn:  The Amazon Resource Name (ARN) of the signing profile created.<br/>        """
        pass

    def start_signing_job(self, clientRequestToken: str, destination: Destination, source: Source, profileName: str=None) -> StartSigningJobResponse:
        """Initiates a signing job to be performed on the code provided. Signing jobs are viewable by the <code>ListSigningJobs</code> operation for two years after they are performed. Note the following requirements:  <ul> <li>  You must create an Amazon S3 source bucket. For more information, see <a href="http://docs.aws.amazon.com/AmazonS3/latest/gsg/CreatingABucket.html">Create a Bucket</a> in the <i>Amazon S3 Getting Started Guide</i>.  </li> <li> Your S3 source bucket must be version enabled. </li> <li> You must create an S3 destination bucket. AWS Signer uses your S3 destination bucket to write your signed code. </li> <li> You specify the name of the source and destination buckets when calling the <code>StartSigningJob</code> operation. </li> <li> You must also specify a request token that identifies your request to AWS Signer.  </li> </ul> You can call the <a>DescribeSigningJob</a> and the <a>ListSigningJobs</a> actions after you call <code>StartSigningJob</code>. For a Java example that shows how to use this action, see <a href="http://docs.aws.amazon.com/acm/latest/userguide/">http://docs.aws.amazon.com/acm/latest/userguide/</a> 

        :param self:
        :param clientRequestToken:  String that identifies the signing request. All calls after the first that use this token return the same response as the first call.<br/>profileName:  The name of the signing profile.<br/>destination:  The S3 bucket in which to save your signed object. The destination contains the name of your bucket and an optional prefix.:<br/>source:  The S3 bucket that contains the object to sign or a BLOB that contains your raw code.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>s3</i> The <code>S3Source</code> object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>s3</b>: The <code>S3Source</code> object.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> Version of your source image in your version enabled S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>key</i> Key name of the bucket object that contains your unsigned code.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>bucketName</i> Name of the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>s3</i> The <code>S3Destination</code> object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>s3</b>: The <code>S3Destination</code> object.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>prefix</i> An Amazon S3 prefix that you can use to limit responses to those that begin with the specified prefix.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>bucketName</i> Name of the S3 bucket.<br/>
        :return: jobId:  The ID of your signing job.<br/>        """
        pass



class SMS(BaseClient):
    """<fullname>AAWS Sever Migration Service</fullname> This is the <i>AWS Sever Migration Service API Reference</i>. It provides descriptions, syntax, and usage examples for each of the actions and data types for the AWS Sever Migration Service (AWS SMS). The topic for each action shows the Query API request parameters and the XML response. You can also view the XML request elements in the WSDL. Alternatively, you can use one of the AWS SDKs to access an API that's tailored to the programming language or platform that you're using. For more information, see <a href="http://aws.amazon.com/tools/#SDKs">AWS SDKs</a>. To learn more about the Server Migration Service, see the following resources: <ul> <li>  <a href="https://aws.amazon.com/server-migration-service/">AWS Sever Migration Service product page</a>  </li> <li>  <a href="https://docs.aws.amazon.com/server-migration-service/latest/userguide/server-migration.html">AWS Sever Migration Service User Guide</a>  </li> </ul>"""


    class GetReplicationJobsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_replication_jobs` for valid parameters"""
            pass
    class GetReplicationRunsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_replication_runs` for valid parameters"""
            pass
    class GetConnectorsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_connectors` for valid parameters"""
            pass
    class GetServersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_servers` for valid parameters"""
            pass
    class ListAppsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_apps` for valid parameters"""
            pass


    class CreateAppRequest(Mapping):
        pass
    
    class CreateAppResponse(Mapping):
        pass
    
    class ServerGroup(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreateReplicationJobRequest(Mapping):
        pass
    
    class CreateReplicationJobResponse(Mapping):
        pass
    
    class Timestamp(object):
        pass
    
    class RunOnce(object):
        pass
    
    class Encrypted(object):
        pass
    
    class DeleteAppRequest(Mapping):
        pass
    
    class DeleteAppResponse(Mapping):
        pass
    
    class ForceStopAppReplication(object):
        pass
    
    class ForceTerminateApp(object):
        pass
    
    class DeleteAppLaunchConfigurationRequest(Mapping):
        pass
    
    class DeleteAppLaunchConfigurationResponse(Mapping):
        pass
    
    class DeleteAppReplicationConfigurationRequest(Mapping):
        pass
    
    class DeleteAppReplicationConfigurationResponse(Mapping):
        pass
    
    class DeleteReplicationJobRequest(Mapping):
        pass
    
    class DeleteReplicationJobResponse(Mapping):
        pass
    
    class DeleteServerCatalogRequest(Mapping):
        pass
    
    class DeleteServerCatalogResponse(Mapping):
        pass
    
    class DisassociateConnectorRequest(Mapping):
        pass
    
    class DisassociateConnectorResponse(Mapping):
        pass
    
    class GenerateChangeSetRequest(Mapping):
        pass
    
    class GenerateChangeSetResponse(Mapping):
        pass
    
    class GenerateTemplateRequest(Mapping):
        pass
    
    class GenerateTemplateResponse(Mapping):
        pass
    
    class GetAppRequest(Mapping):
        pass
    
    class GetAppResponse(Mapping):
        pass
    
    class GetAppLaunchConfigurationRequest(Mapping):
        pass
    
    class GetAppLaunchConfigurationResponse(Mapping):
        pass
    
    class GetAppReplicationConfigurationRequest(Mapping):
        pass
    
    class GetAppReplicationConfigurationResponse(Mapping):
        pass
    
    class GetConnectorsRequest(Mapping):
        pass
    
    class GetConnectorsResponse(Mapping):
        pass
    
    class GetReplicationJobsRequest(Mapping):
        pass
    
    class GetReplicationJobsResponse(Mapping):
        pass
    
    class GetReplicationRunsRequest(Mapping):
        pass
    
    class GetReplicationRunsResponse(Mapping):
        pass
    
    class GetServersRequest(Mapping):
        pass
    
    class GetServersResponse(Mapping):
        pass
    
    class VmServerAddress(Mapping):
        pass
    
    class ImportServerCatalogRequest(Mapping):
        pass
    
    class ImportServerCatalogResponse(Mapping):
        pass
    
    class LaunchAppRequest(Mapping):
        pass
    
    class LaunchAppResponse(Mapping):
        pass
    
    class ListAppsRequest(Mapping):
        pass
    
    class ListAppsResponse(Mapping):
        pass
    
    class AppId(object):
        pass
    
    class PutAppLaunchConfigurationRequest(Mapping):
        pass
    
    class PutAppLaunchConfigurationResponse(Mapping):
        pass
    
    class ServerGroupLaunchConfiguration(Mapping):
        pass
    
    class PutAppReplicationConfigurationRequest(Mapping):
        pass
    
    class PutAppReplicationConfigurationResponse(Mapping):
        pass
    
    class ServerGroupReplicationConfiguration(Mapping):
        pass
    
    class StartAppReplicationRequest(Mapping):
        pass
    
    class StartAppReplicationResponse(Mapping):
        pass
    
    class StartOnDemandReplicationRunRequest(Mapping):
        pass
    
    class StartOnDemandReplicationRunResponse(Mapping):
        pass
    
    class StopAppReplicationRequest(Mapping):
        pass
    
    class StopAppReplicationResponse(Mapping):
        pass
    
    class TerminateAppRequest(Mapping):
        pass
    
    class TerminateAppResponse(Mapping):
        pass
    
    class UpdateAppRequest(Mapping):
        pass
    
    class UpdateAppResponse(Mapping):
        pass
    
    class UpdateReplicationJobRequest(Mapping):
        pass
    
    class UpdateReplicationJobResponse(Mapping):
        pass
    
    def create_app(self, name: str=None, description: str=None, roleName: str=None, clientToken: str=None, serverGroups: List[ServerGroup]=None, tags: List[Tag]=None) -> CreateAppResponse:
        """Creates an application. An application consists of one or more server groups. Each server group contain one or more servers.

        :param self:
        :param tags:  List of tags to be associated with the application.<br/>serverGroups:  List of server groups to include in the application.<br/>clientToken:  A unique, case-sensitive identifier you provide to ensure idempotency of application creation.<br/>roleName:  Name of service role in customer's account to be used by AWS SMS.<br/>description:  Description of the new application<br/>name:  Name of the new application.<br/>
        :return: tags:  List of taags associated with the application.<br/>serverGroups:  List of server groups included in the application.<br/>appSummary:  Summary description of the application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>totalServers</i> Number of servers present in the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>totalServerGroups</i> Number of server groups present in the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleName</i> Name of the service role in the customer's account used by AWS SMS.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastModified</i> Timestamp of the application's creation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationTime</i> Time of creation of this application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>launchDetails</i> Details about the latest launch of the application.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>launchDetails</b>: Details about the latest launch of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>launchStatusMessage</i> A message related to the launch status of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>launchStatus</i> Launch status of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>latestReplicationTime</i> Timestamp of the application's most recent successful replication.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>replicationStatusMessage</i> A message related to the replication status of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>replicationStatus</i> Replication status of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusMessage</i> A message related to the status of the application<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> Status of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> Description of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> Name of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>appId</i> Unique ID of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>stackId</i> Identifier of the latest stack launched for this application.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>stackName</i> Name of the latest stack launched for this application.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>latestLaunchTime</i> Latest time this application was launched successfully.<br/>        """
        pass

    def create_replication_job(self, seedReplicationTime: Timestamp, serverId: str, frequency: int=None, runOnce: RunOnce=None, licenseType: str=None, roleName: str=None, description: str=None, numberOfRecentAmisToKeep: int=None, encrypted: Encrypted=None, kmsKeyId: str=None) -> CreateReplicationJobResponse:
        """Creates a replication job. The replication job schedules periodic replication runs to replicate your server to AWS. Each replication run creates an Amazon Machine Image (AMI).

        :param self:
        :param kmsKeyId:  KMS key ID for replication jobs that produce encrypted AMIs. Can be any of the following:  <ul> <li> KMS key ID </li> <li> KMS key alias </li> <li> ARN referring to KMS key ID </li> <li> ARN referring to KMS key alias </li> </ul>  If encrypted is <i>true</i> but a KMS key id is not specified, the customer's default KMS key for EBS is used. <br/>encrypted:  When <i>true</i>, the replication job produces encrypted AMIs. See also <code>KmsKeyId</code> below.<br/>numberOfRecentAmisToKeep:  The maximum number of SMS-created AMIs to retain. The oldest will be deleted once the maximum number is reached and a new AMI is created.<br/>description:  The description of the replication job.<br/>roleName:  The name of the IAM role to be used by the AWS SMS.<br/>licenseType:  The license type to be used for the AMI created by a successful replication run.<br/>runOnce:  <p/><br/>frequency:  The time between consecutive replication runs, in hours.<br/>seedReplicationTime:  The seed replication time.<br/>serverId:  The identifier of the server.<br/>
        :return: replicationJobId:  The unique identifier of the replication job.<br/>        """
        pass

    def delete_app(self, appId: str=None, forceStopAppReplication: ForceStopAppReplication=None, forceTerminateApp: ForceTerminateApp=None) -> DeleteAppResponse:
        """Deletes an existing application. Optionally deletes the launched stack associated with the application and all AWS SMS replication jobs for servers in the application.

        :param self:
        :param forceTerminateApp:  While deleting the application, terminate the stack corresponding to the application.<br/>forceStopAppReplication:  While deleting the application, stop all replication jobs corresponding to the servers in the application.<br/>appId:  ID of the application to delete.<br/>
        :return:         """
        pass

    def delete_app_launch_configuration(self, appId: str=None) -> DeleteAppLaunchConfigurationResponse:
        """Deletes existing launch configuration for an application.

        :param self:
        :param appId:  ID of the application associated with the launch configuration.<br/>
        :return:         """
        pass

    def delete_app_replication_configuration(self, appId: str=None) -> DeleteAppReplicationConfigurationResponse:
        """Deletes existing replication configuration for an application.

        :param self:
        :param appId:  ID of the application associated with the replication configuration.<br/>
        :return:         """
        pass

    def delete_replication_job(self, replicationJobId: str) -> DeleteReplicationJobResponse:
        """Deletes the specified replication job. After you delete a replication job, there are no further replication runs. AWS deletes the contents of the Amazon S3 bucket used to store AWS SMS artifacts. The AMIs created by the replication runs are not deleted.

        :param self:
        :param replicationJobId:  The identifier of the replication job.<br/>
        :return:         """
        pass

    def delete_server_catalog(self) -> DeleteServerCatalogResponse:
        """Deletes all servers from your server catalog.

        :param self:
        :param 
        :return:         """
        pass

    def disassociate_connector(self, connectorId: str) -> DisassociateConnectorResponse:
        """Disassociates the specified connector from AWS SMS. After you disassociate a connector, it is no longer available to support replication jobs.

        :param self:
        :param connectorId:  The identifier of the connector.<br/>
        :return:         """
        pass

    def generate_change_set(self, appId: str=None, changesetFormat: str=None) -> GenerateChangeSetResponse:
        """Generates a target change set for a currently launched stack and writes it to an Amazon S3 object in the customer’s Amazon S3 bucket.

        :param self:
        :param changesetFormat:  Format for the change set.<br/>appId:  ID of the application associated with the change set.<br/>
        :return: s3Location:  Location of the Amazon S3 object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>key</i> Amazon S3 bucket key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>bucket</i> Amazon S3 bucket name.<br/>        """
        pass

    def generate_template(self, appId: str=None, templateFormat: str=None) -> GenerateTemplateResponse:
        """Generates an Amazon CloudFormation template based on the current launch configuration and writes it to an Amazon S3 object in the customer’s Amazon S3 bucket.

        :param self:
        :param templateFormat:  Format for generating the Amazon CloudFormation template.<br/>appId:  ID of the application associated with the Amazon CloudFormation template.<br/>
        :return: s3Location:  Location of the Amazon S3 object.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>key</i> Amazon S3 bucket key.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>bucket</i> Amazon S3 bucket name.<br/>        """
        pass

    def get_app(self, appId: str=None) -> GetAppResponse:
        """Retrieve information about an application.

        :param self:
        :param appId:  ID of the application whose information is being retrieved.<br/>
        :return: tags:  List of tags associated with the application.<br/>serverGroups:  List of server groups belonging to the application.<br/>appSummary:  Information about the application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>totalServers</i> Number of servers present in the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>totalServerGroups</i> Number of server groups present in the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleName</i> Name of the service role in the customer's account used by AWS SMS.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastModified</i> Timestamp of the application's creation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationTime</i> Time of creation of this application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>launchDetails</i> Details about the latest launch of the application.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>launchDetails</b>: Details about the latest launch of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>launchStatusMessage</i> A message related to the launch status of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>launchStatus</i> Launch status of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>latestReplicationTime</i> Timestamp of the application's most recent successful replication.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>replicationStatusMessage</i> A message related to the replication status of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>replicationStatus</i> Replication status of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusMessage</i> A message related to the status of the application<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> Status of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> Description of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> Name of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>appId</i> Unique ID of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>stackId</i> Identifier of the latest stack launched for this application.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>stackName</i> Name of the latest stack launched for this application.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>latestLaunchTime</i> Latest time this application was launched successfully.<br/>        """
        pass

    def get_app_launch_configuration(self, appId: str=None) -> GetAppLaunchConfigurationResponse:
        """Retrieves the application launch configuration associated with an application.

        :param self:
        :param appId:  ID of the application launch configuration.<br/>
        :return: serverGroupLaunchConfigurations:  List of launch configurations for server groups in this application.<br/>roleName:  Name of the service role in the customer's account that Amazon CloudFormation uses to launch the application.<br/>appId:  ID of the application associated with the launch configuration.<br/>        """
        pass

    def get_app_replication_configuration(self, appId: str=None) -> GetAppReplicationConfigurationResponse:
        """Retrieves an application replication configuration associatd with an application.

        :param self:
        :param appId:  ID of the application associated with the replication configuration.<br/>
        :return: serverGroupReplicationConfigurations:  Replication configurations associated with server groups in this application.<br/>        """
        pass

    def get_connectors(self, nextToken: str=None, maxResults: int=None) -> GetConnectorsResponse:
        """Describes the connectors registered with the AWS SMS.

        :param self:
        :param maxResults:  The maximum number of results to return in a single call. The default value is 50. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value.<br/>nextToken:  The token for the next set of results.<br/>
        :return: nextToken:  The token required to retrieve the next set of results. This value is null when there are no more results to return.<br/>connectorList:  Information about the registered connectors.<br/>        """
        pass

    def get_replication_jobs(self, replicationJobId: str=None, nextToken: str=None, maxResults: int=None) -> GetReplicationJobsResponse:
        """Describes the specified replication job or all of your replication jobs.

        :param self:
        :param maxResults:  The maximum number of results to return in a single call. The default value is 50. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value.<br/>nextToken:  The token for the next set of results.<br/>replicationJobId:  The identifier of the replication job.<br/>
        :return: nextToken:  The token required to retrieve the next set of results. This value is null when there are no more results to return.<br/>replicationJobList:  Information about the replication jobs.<br/>        """
        pass

    def get_replication_runs(self, replicationJobId: str, nextToken: str=None, maxResults: int=None) -> GetReplicationRunsResponse:
        """Describes the replication runs for the specified replication job.

        :param self:
        :param maxResults:  The maximum number of results to return in a single call. The default value is 50. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value.<br/>nextToken:  The token for the next set of results.<br/>replicationJobId:  The identifier of the replication job.<br/>
        :return: nextToken:  The token required to retrieve the next set of results. This value is null when there are no more results to return.<br/>replicationRunList:  Information about the replication runs.<br/>replicationJob:  Information about the replication job.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>replicationRunList</i> Information about the replication runs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>kmsKeyId</i> KMS key ID for replication jobs that produce encrypted AMIs. Can be any of the following:  <ul> <li> KMS key ID </li> <li> KMS key alias </li> <li> ARN referring to KMS key ID </li> <li> ARN referring to KMS key alias </li> </ul>  If encrypted is <i>true</i> but a KMS key id is not specified, the customer's default KMS key for EBS is used. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>encrypted</i> Whether the replication job should produce encrypted AMIs or not. See also <code>KmsKeyId</code> below.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>numberOfRecentAmisToKeep</i> Number of recent AMIs to keep in the customer's account for a replication job. By default the value is set to zero, meaning that all AMIs are kept.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The description of the replication job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusMessage</i> The description of the current status of the replication job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>state</i> The state of the replication job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>latestAmiId</i> The ID of the latest Amazon Machine Image (AMI).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleName</i> The name of the IAM role to be used by the Server Migration Service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>licenseType</i> The license type to be used for the AMI created by a successful replication run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>nextReplicationRunStartTime</i> The start time of the next replication run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>runOnce</i> <p/><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>frequency</i> The time between consecutive replication runs, in hours.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>seedReplicationTime</i> The seed replication time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>vmServer</i> Information about the VM server.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>vmServer</b>: Information about the VM server.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serverType</i> The type of server.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>serverId</i> The identifier of the server.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>replicationJobId</i> The identifier of the replication job.<br/>Information about the VM server location.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>vmServerAddress</b>: Information about the VM server location.        """
        pass

    def get_servers(self, nextToken: str=None, maxResults: int=None, vmServerAddressList: List[VmServerAddress]=None) -> GetServersResponse:
        """Describes the servers in your server catalog. Before you can describe your servers, you must import them using <a>ImportServerCatalog</a>.

        :param self:
        :param vmServerAddressList:  List of <code>VmServerAddress</code> objects<br/>maxResults:  The maximum number of results to return in a single call. The default value is 50. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value.<br/>nextToken:  The token for the next set of results.<br/>
        :return: nextToken:  The token required to retrieve the next set of results. This value is null when there are no more results to return.<br/>serverList:  Information about the servers.<br/>serverCatalogStatus:  The status of the server catalog.<br/>lastModifiedOn:  The time when the server was last modified.<br/>        """
        pass

    def import_server_catalog(self) -> ImportServerCatalogResponse:
        """Gathers a complete list of on-premises servers. Connectors must be installed and monitoring all servers that you want to import. This call returns immediately, but might take additional time to retrieve all the servers.

        :param self:
        :param 
        :return:         """
        pass

    def launch_app(self, appId: str=None) -> LaunchAppResponse:
        """Launches an application stack.

        :param self:
        :param appId:  ID of the application to launch.<br/>
        :return:         """
        pass

    def list_apps(self, appIds: List[AppId]=None, nextToken: str=None, maxResults: int=None) -> ListAppsResponse:
        """Returns a list of summaries for all applications.

        :param self:
        :param maxResults:  The maximum number of results to return in a single call. The default value is 50. To retrieve the remaining results, make another call with the returned <code>NextToken</code> value. <br/>nextToken:  The token for the next set of results.<br/>appIds:  <p/><br/>
        :return: nextToken:  The token required to retrieve the next set of results. This value is null when there are no more results to return.<br/>apps:  A list of application summaries.<br/>        """
        pass

    def put_app_launch_configuration(self, appId: str=None, roleName: str=None, serverGroupLaunchConfigurations: List[ServerGroupLaunchConfiguration]=None) -> PutAppLaunchConfigurationResponse:
        """Creates a launch configuration for an application.

        :param self:
        :param serverGroupLaunchConfigurations:  Launch configurations for server groups in the application.<br/>roleName:  Name of service role in the customer's account that Amazon CloudFormation uses to launch the application.<br/>appId:  ID of the application associated with the launch configuration.<br/>
        :return:         """
        pass

    def put_app_replication_configuration(self, appId: str=None, serverGroupReplicationConfigurations: List[ServerGroupReplicationConfiguration]=None) -> PutAppReplicationConfigurationResponse:
        """Creates or updates a replication configuration for an application.

        :param self:
        :param serverGroupReplicationConfigurations:  Replication configurations for server groups in the application.<br/>appId:  ID of the application tassociated with the replication configuration.<br/>
        :return:         """
        pass

    def start_app_replication(self, appId: str=None) -> StartAppReplicationResponse:
        """Starts replicating an application.

        :param self:
        :param appId:  ID of the application to replicate.<br/>
        :return:         """
        pass

    def start_on_demand_replication_run(self, replicationJobId: str, description: str=None) -> StartOnDemandReplicationRunResponse:
        """Starts an on-demand replication run for the specified replication job. This replication run starts immediately. This replication run is in addition to the ones already scheduled. There is a limit on the number of on-demand replications runs you can request in a 24-hour period.

        :param self:
        :param description:  The description of the replication run.<br/>replicationJobId:  The identifier of the replication job.<br/>
        :return: replicationRunId:  The identifier of the replication run.<br/>        """
        pass

    def stop_app_replication(self, appId: str=None) -> StopAppReplicationResponse:
        """Stops replicating an application.

        :param self:
        :param appId:  ID of the application to stop replicating.<br/>
        :return:         """
        pass

    def terminate_app(self, appId: str=None) -> TerminateAppResponse:
        """Terminates the stack for an application.

        :param self:
        :param appId:  ID of the application to terminate.<br/>
        :return:         """
        pass

    def update_app(self, appId: str=None, name: str=None, description: str=None, roleName: str=None, serverGroups: List[ServerGroup]=None, tags: List[Tag]=None) -> UpdateAppResponse:
        """Updates an application.

        :param self:
        :param tags:  List of tags to associate with the application.<br/>serverGroups:  List of server groups in the application to update.<br/>roleName:  Name of the service role in the customer's account used by AWS SMS.<br/>description:  New description of the application.<br/>name:  New name of the application.<br/>appId:  ID of the application to update.<br/>
        :return: tags:  List of tags associated with the application.<br/>serverGroups:  List of updated server groups in the application.<br/>appSummary:  Summary description of the application.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>totalServers</i> Number of servers present in the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>totalServerGroups</i> Number of server groups present in the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>roleName</i> Name of the service role in the customer's account used by AWS SMS.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lastModified</i> Timestamp of the application's creation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationTime</i> Time of creation of this application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>launchDetails</i> Details about the latest launch of the application.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>launchDetails</b>: Details about the latest launch of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>launchStatusMessage</i> A message related to the launch status of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>launchStatus</i> Launch status of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>latestReplicationTime</i> Timestamp of the application's most recent successful replication.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>replicationStatusMessage</i> A message related to the replication status of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>replicationStatus</i> Replication status of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>statusMessage</i> A message related to the status of the application<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> Status of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> Description of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> Name of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>appId</i> Unique ID of the application.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>stackId</i> Identifier of the latest stack launched for this application.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>stackName</i> Name of the latest stack launched for this application.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>latestLaunchTime</i> Latest time this application was launched successfully.<br/>        """
        pass

    def update_replication_job(self, replicationJobId: str, frequency: int=None, nextReplicationRunStartTime: Timestamp=None, licenseType: str=None, roleName: str=None, description: str=None, numberOfRecentAmisToKeep: int=None, encrypted: Encrypted=None, kmsKeyId: str=None) -> UpdateReplicationJobResponse:
        """Updates the specified settings for the specified replication job.

        :param self:
        :param kmsKeyId:  <p/> KMS key ID for replication jobs that produce encrypted AMIs. Can be any of the following:  <ul> <li> KMS key ID </li> <li> KMS key alias </li> <li> ARN referring to KMS key ID </li> <li> ARN referring to KMS key alias </li> </ul>  If encrypted is <i>true</i> but a KMS key id is not specified, the customer's default KMS key for EBS is used. <br/>encrypted:  When true, the replication job produces encrypted AMIs . See also <code>KmsKeyId</code> below.<br/>numberOfRecentAmisToKeep:  The maximum number of SMS-created AMIs to retain. The oldest will be deleted once the maximum number is reached and a new AMI is created.<br/>description:  The description of the replication job.<br/>roleName:  The name of the IAM role to be used by AWS SMS.<br/>licenseType:  The license type to be used for the AMI created by a successful replication run.<br/>nextReplicationRunStartTime:  The start time of the next replication run.<br/>frequency:  The time between consecutive replication runs, in hours.<br/>replicationJobId:  The identifier of the replication job.<br/>
        :return:         """
        pass



class PinpointSMSVoice(BaseClient):
    """Pinpoint SMS and Voice Messaging public facing APIs"""



    class CreateConfigurationSetRequest(Mapping):
        pass
    
    class CreateConfigurationSetResponse(Mapping):
        pass
    
    class CreateConfigurationSetEventDestinationRequest(Mapping):
        pass
    
    class CreateConfigurationSetEventDestinationResponse(Mapping):
        pass
    
    class EventDestinationDefinition(Mapping):
        pass
    
    class DeleteConfigurationSetRequest(Mapping):
        pass
    
    class DeleteConfigurationSetResponse(Mapping):
        pass
    
    class DeleteConfigurationSetEventDestinationRequest(Mapping):
        pass
    
    class DeleteConfigurationSetEventDestinationResponse(Mapping):
        pass
    
    class GetConfigurationSetEventDestinationsRequest(Mapping):
        pass
    
    class GetConfigurationSetEventDestinationsResponse(Mapping):
        pass
    
    class ListConfigurationSetsRequest(Mapping):
        pass
    
    class ListConfigurationSetsResponse(Mapping):
        pass
    
    class SendVoiceMessageRequest(Mapping):
        pass
    
    class SendVoiceMessageResponse(Mapping):
        pass
    
    class VoiceMessageContent(Mapping):
        pass
    
    class UpdateConfigurationSetEventDestinationRequest(Mapping):
        pass
    
    class UpdateConfigurationSetEventDestinationResponse(Mapping):
        pass
    
    def create_configuration_set(self, ConfigurationSetName: str=None) -> CreateConfigurationSetResponse:
        """Create a new configuration set. After you create the configuration set, you can add one or more event destinations to it.

        :param self:
        :param ConfigurationSetName:  The name that you want to give the configuration set.<br/>
        :return:         """
        pass

    def create_configuration_set_event_destination(self, ConfigurationSetName: str, EventDestination: EventDestinationDefinition=None, EventDestinationName: str=None) -> CreateConfigurationSetEventDestinationResponse:
        """Create a new event destination in a configuration set.

        :param self:
        :param EventDestinationName:  A name that identifies the event destination.<br/>EventDestination:  An object that defines a single event destination.:<br/>ConfigurationSetName:  ConfigurationSetName<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsDestination</i> An object that contains information about an event destination that sends data to Amazon SNS.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SnsDestination</b>: An object that contains information about an event destination that sends data to Amazon SNS.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MatchingEventTypes</i> An array of EventDestination objects. Each EventDestination object includes ARNs and other information that define an event destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KinesisFirehoseDestination</i> An object that contains information about an event destination that sends data to Amazon Kinesis Data Firehose.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KinesisFirehoseDestination</b>: An object that contains information about an event destination that sends data to Amazon Kinesis Data Firehose.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Indicates whether or not the event destination is enabled. If the event destination is enabled, then Amazon Pinpoint sends response data to the specified event destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchLogsDestination</i> An object that contains information about an event destination that sends data to Amazon CloudWatch Logs.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CloudWatchLogsDestination</b>: An object that contains information about an event destination that sends data to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LogGroupArn</i> The name of the Amazon CloudWatch Log Group that you want to record events in.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoleArn</i> The Amazon Resource Name (ARN) of an Amazon Identity and Access Management (IAM) role that is able to write event data to an Amazon CloudWatch destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoleArn</i> The Amazon Resource Name (ARN) of the Amazon Kinesis Data Firehose destination that you want to use in the event destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DeliveryStreamArn</i> The Amazon Resource Name (ARN) of an IAM role that can write data to an Amazon Kinesis Data Firehose stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicArn</i> The Amazon Resource Name (ARN) of the Amazon SNS topic that you want to publish events to.<br/>
        :return:         """
        pass

    def delete_configuration_set(self, ConfigurationSetName: str) -> DeleteConfigurationSetResponse:
        """Deletes an existing configuration set.

        :param self:
        :param ConfigurationSetName:  ConfigurationSetName<br/>
        :return:         """
        pass

    def delete_configuration_set_event_destination(self, EventDestinationName: str, ConfigurationSetName: str) -> DeleteConfigurationSetEventDestinationResponse:
        """Deletes an event destination in a configuration set.

        :param self:
        :param EventDestinationName:  EventDestinationName<br/>ConfigurationSetName:  ConfigurationSetName<br/>
        :return:         """
        pass

    def get_configuration_set_event_destinations(self, ConfigurationSetName: str) -> GetConfigurationSetEventDestinationsResponse:
        """Obtain information about an event destination, including the types of events it reports, the Amazon Resource Name (ARN) of the destination, and the name of the event destination.

        :param self:
        :param ConfigurationSetName:  ConfigurationSetName<br/>
        :return: EventDestinations:  An array of EventDestination objects. Each EventDestination object includes ARNs and other information that define an event destination.<br/>        """
        pass

    def list_configuration_sets(self, NextToken: str=None, PageSize: str=None) -> ListConfigurationSetsResponse:
        """List all of the configuration sets associated with your Amazon Pinpoint account in the current region.

        :param self:
        :param PageSize:  Used to specify the number of items that should be returned in the response.<br/>NextToken:  A token returned from a previous call to the API that indicates the position in the list of results.<br/>
        :return: NextToken:  A token returned from a previous call to ListConfigurationSets to indicate the position in the list of configuration sets.<br/>ConfigurationSets:  An object that contains a list of configuration sets for your account in the current region.<br/>        """
        pass

    def send_voice_message(self, CallerId: str=None, ConfigurationSetName: str=None, Content: VoiceMessageContent=None, DestinationPhoneNumber: str=None, OriginationPhoneNumber: str=None) -> SendVoiceMessageResponse:
        """Create a new voice message and send it to a recipient's phone number.

        :param self:
        :param OriginationPhoneNumber:  The phone number that Amazon Pinpoint should use to send the voice message. This isn't necessarily the phone number that appears on recipients' devices when they receive the message, because you can specify a CallerId parameter in the request.<br/>DestinationPhoneNumber:  The phone number that you want to send the voice message to.<br/>Content:  An object that contains a voice message and information about the recipient that you want to send it to.:<br/>ConfigurationSetName:  The name of the configuration set that you want to use to send the message.<br/>CallerId:  The phone number that appears on recipients' devices when they receive the message.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SSMLMessage</i> An object that defines a message that contains SSML-formatted text.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SSMLMessage</b>: An object that defines a message that contains SSML-formatted text.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlainTextMessage</i> An object that defines a message that contains unformatted text.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>PlainTextMessage</b>: An object that defines a message that contains unformatted text.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CallInstructionsMessage</i> An object that defines a message that contains text formatted using Amazon Pinpoint Voice Instructions markup.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CallInstructionsMessage</b>: An object that defines a message that contains text formatted using Amazon Pinpoint Voice Instructions markup.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Text</i> The language to use when delivering the message. For a complete list of supported languages, see the Amazon Polly Developer Guide.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VoiceId</i> The name of the voice that you want to use to deliver the message. For a complete list of supported voices, see the Amazon Polly Developer Guide.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Text</i> The plain (not SSML-formatted) text to deliver to the recipient.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LanguageCode</i> The language to use when delivering the message. For a complete list of supported languages, see the Amazon Polly Developer Guide.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VoiceId</i> The name of the voice that you want to use to deliver the message. For a complete list of supported voices, see the Amazon Polly Developer Guide.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Text</i> The SSML-formatted text to deliver to the recipient.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LanguageCode</i> The language to use when delivering the message. For a complete list of supported languages, see the Amazon Polly Developer Guide.<br/>
        :return: MessageId:  A unique identifier for the voice message.<br/>        """
        pass

    def update_configuration_set_event_destination(self, EventDestinationName: str, ConfigurationSetName: str, EventDestination: EventDestinationDefinition=None) -> UpdateConfigurationSetEventDestinationResponse:
        """Update an event destination in a configuration set. An event destination is a location that you publish information about your voice calls to. For example, you can log an event to an Amazon CloudWatch destination when a call fails.

        :param self:
        :param EventDestinationName:  EventDestinationName<br/>EventDestination:  An object that defines a single event destination.:<br/>ConfigurationSetName:  ConfigurationSetName<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsDestination</i> An object that contains information about an event destination that sends data to Amazon SNS.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SnsDestination</b>: An object that contains information about an event destination that sends data to Amazon SNS.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MatchingEventTypes</i> An array of EventDestination objects. Each EventDestination object includes ARNs and other information that define an event destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KinesisFirehoseDestination</i> An object that contains information about an event destination that sends data to Amazon Kinesis Data Firehose.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>KinesisFirehoseDestination</b>: An object that contains information about an event destination that sends data to Amazon Kinesis Data Firehose.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Enabled</i> Indicates whether or not the event destination is enabled. If the event destination is enabled, then Amazon Pinpoint sends response data to the specified event destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchLogsDestination</i> An object that contains information about an event destination that sends data to Amazon CloudWatch Logs.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CloudWatchLogsDestination</b>: An object that contains information about an event destination that sends data to Amazon CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LogGroupArn</i> The name of the Amazon CloudWatch Log Group that you want to record events in.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoleArn</i> The Amazon Resource Name (ARN) of an Amazon Identity and Access Management (IAM) role that is able to write event data to an Amazon CloudWatch destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>IamRoleArn</i> The Amazon Resource Name (ARN) of the Amazon Kinesis Data Firehose destination that you want to use in the event destination.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DeliveryStreamArn</i> The Amazon Resource Name (ARN) of an IAM role that can write data to an Amazon Kinesis Data Firehose stream.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TopicArn</i> The Amazon Resource Name (ARN) of the Amazon SNS topic that you want to publish events to.<br/>
        :return:         """
        pass



class Snowball(BaseClient):
    """AWS Snowball is a petabyte-scale data transport solution that uses secure devices to transfer large amounts of data between your on-premises data centers and Amazon Simple Storage Service (Amazon S3). The commands described here provide access to the same functionality that is available in the AWS Snowball Management Console, which enables you to create and manage jobs for Snowball and Snowball Edge devices. To transfer data locally with a device, you'll need to use the Snowball client or the Amazon S3 API adapter for Snowball."""


    class ListJobsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_jobs` for valid parameters"""
            pass
    class DescribeAddressesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_addresses` for valid parameters"""
            pass
    class ListClusterJobsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_cluster_jobs` for valid parameters"""
            pass
    class ListClustersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_clusters` for valid parameters"""
            pass
    class ListCompatibleImagesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_compatible_images` for valid parameters"""
            pass


    class CancelClusterRequest(Mapping):
        pass
    
    class CancelClusterResult(Mapping):
        pass
    
    class CancelJobRequest(Mapping):
        pass
    
    class CancelJobResult(Mapping):
        pass
    
    class CreateAddressRequest(Mapping):
        pass
    
    class CreateAddressResult(Mapping):
        pass
    
    class Address(Mapping):
        pass
    
    class CreateClusterRequest(Mapping):
        pass
    
    class CreateClusterResult(Mapping):
        pass
    
    class JobResource(Mapping):
        pass
    
    class Notification(Mapping):
        pass
    
    class CreateJobRequest(Mapping):
        pass
    
    class CreateJobResult(Mapping):
        pass
    
    class DescribeAddressRequest(Mapping):
        pass
    
    class DescribeAddressResult(Mapping):
        pass
    
    class DescribeAddressesRequest(Mapping):
        pass
    
    class DescribeAddressesResult(Mapping):
        pass
    
    class DescribeClusterRequest(Mapping):
        pass
    
    class DescribeClusterResult(Mapping):
        pass
    
    class DescribeJobRequest(Mapping):
        pass
    
    class DescribeJobResult(Mapping):
        pass
    
    class GetJobManifestRequest(Mapping):
        pass
    
    class GetJobManifestResult(Mapping):
        pass
    
    class GetJobUnlockCodeRequest(Mapping):
        pass
    
    class GetJobUnlockCodeResult(Mapping):
        pass
    
    class GetSnowballUsageRequest(Mapping):
        pass
    
    class GetSnowballUsageResult(Mapping):
        pass
    
    class ListClusterJobsRequest(Mapping):
        pass
    
    class ListClusterJobsResult(Mapping):
        pass
    
    class ListClustersRequest(Mapping):
        pass
    
    class ListClustersResult(Mapping):
        pass
    
    class ListCompatibleImagesRequest(Mapping):
        pass
    
    class ListCompatibleImagesResult(Mapping):
        pass
    
    class ListJobsRequest(Mapping):
        pass
    
    class ListJobsResult(Mapping):
        pass
    
    class UpdateClusterRequest(Mapping):
        pass
    
    class UpdateClusterResult(Mapping):
        pass
    
    class UpdateJobRequest(Mapping):
        pass
    
    class UpdateJobResult(Mapping):
        pass
    
    def cancel_cluster(self, ClusterId: str) -> CancelClusterResult:
        """Cancels a cluster job. You can only cancel a cluster job while it's in the <code>AwaitingQuorum</code> status. You'll have at least an hour after creating a cluster job to cancel it.

        :param self:
        :param ClusterId:  The 39-character ID for the cluster that you want to cancel, for example <code>CID123e4567-e89b-12d3-a456-426655440000</code>.<br/>
        :return:         """
        pass

    def cancel_job(self, JobId: str) -> CancelJobResult:
        """Cancels the specified job. You can only cancel a job before its <code>JobState</code> value changes to <code>PreparingAppliance</code>. Requesting the <code>ListJobs</code> or <code>DescribeJob</code> action returns a job's <code>JobState</code> as part of the response element data returned.

        :param self:
        :param JobId:  The 39-character job ID for the job that you want to cancel, for example <code>JID123e4567-e89b-12d3-a456-426655440000</code>.<br/>
        :return:         """
        pass

    def create_address(self, Address: Address) -> CreateAddressResult:
        """Creates an address for a Snowball to be shipped to. In most regions, addresses are validated at the time of creation. The address you provide must be located within the serviceable area of your region. If the address is invalid or unsupported, then an exception is thrown.

        :param self:
        :param Address:  The address that you want the Snowball shipped to.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsRestricted</i> If the address you are creating is a primary address, then set this option to true. This field is not supported in most regions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneNumber</i> The phone number associated with an address that a Snowball is to be delivered to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PostalCode</i> The postal code in an address that a Snowball is to be delivered to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Country</i> The country in an address that a Snowball is to be delivered to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Landmark</i> This field is no longer used and the value is ignored.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrefectureOrDistrict</i> This field is no longer used and the value is ignored.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StateOrProvince</i> The state or province in an address that a Snowball is to be delivered to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>City</i> The city in an address that a Snowball is to be delivered to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Street3</i> The third line in a street address that a Snowball is to be delivered to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Street2</i> The second line in a street address that a Snowball is to be delivered to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Street1</i> The first line in a street address that a Snowball is to be delivered to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Company</i> The name of the company to receive a Snowball at an address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of a person to receive a Snowball at an address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressId</i> The unique ID for an address.<br/>
        :return: AddressId:  The automatically generated ID for a specific address. You'll use this ID when you create a job to specify which address you want the Snowball for that job shipped to.<br/>        """
        pass

    def create_cluster(self, ShippingOption: str, RoleARN: str, AddressId: str, Resources: JobResource, JobType: str, Description: str=None, KmsKeyARN: str=None, SnowballType: str=None, Notification: Notification=None, ForwardingAddressId: str=None) -> CreateClusterResult:
        """Creates an empty cluster. Each cluster supports five nodes. You use the <a>CreateJob</a> action separately to create the jobs for each of these nodes. The cluster does not ship until these five node jobs have been created.

        :param self:
        :param ForwardingAddressId:  The forwarding address ID for a cluster. This field is not supported in most regions.<br/>Notification:  The Amazon Simple Notification Service (Amazon SNS) notification settings for this cluster.:<br/>ShippingOption:  The shipping speed for each node in this cluster. This speed doesn't dictate how soon you'll get each Snowball Edge device, rather it represents how quickly each device moves to its destination while in transit. Regional shipping speeds are as follows: <ul> <li> In Australia, you have access to express shipping. Typically, devices shipped express are delivered in about a day. </li> <li> In the European Union (EU), you have access to express shipping. Typically, Snowball Edges shipped express are delivered in about a day. In addition, most countries in the EU have access to standard shipping, which typically takes less than a week, one way. </li> <li> In India, devices are delivered in one to seven days. </li> <li> In the US, you have access to one-day shipping and two-day shipping. </li> </ul><br/>SnowballType:  The type of AWS Snowball device to use for this cluster. The only supported device types for cluster jobs are <code>EDGE</code>, <code>EDGE_C</code>, and <code>EDGE_CG</code>.<br/>RoleARN:  The <code>RoleARN</code> that you want to associate with this cluster. <code>RoleArn</code> values are created by using the <a href="http://docs.aws.amazon.com/IAM/latest/APIReference/API_CreateRole.html">CreateRole</a> API action in AWS Identity and Access Management (IAM).<br/>KmsKeyARN:  The <code>KmsKeyARN</code> value that you want to associate with this cluster. <code>KmsKeyARN</code> values are created by using the <a href="http://docs.aws.amazon.com/kms/latest/APIReference/API_CreateKey.html">CreateKey</a> API action in AWS Key Management Service (AWS KMS). <br/>AddressId:  The ID for the address that you want the cluster shipped to.<br/>Description:  An optional description of this specific cluster, for example <code>Environmental Data Cluster-01</code>.<br/>Resources:  The resources associated with the cluster job. These resources include Amazon S3 buckets and optional AWS Lambda functions written in the Python language. :<br/>JobType:  The type of job for this cluster. Currently, the only job type supported for clusters is <code>LOCAL_USE</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Ec2AmiResources</i> The Amazon Machine Images (AMIs) associated with this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LambdaResources</i> The Python-language Lambda functions for this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Resources</i> An array of <code>S3Resource</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotifyAll</i> Any change in job state will trigger a notification for this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobStatesToNotify</i> The list of job states that will trigger a notification for this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsTopicARN</i> The new SNS <code>TopicArn</code> that you want to associate with this job. You can create Amazon Resource Names (ARNs) for topics by using the <a href="http://docs.aws.amazon.com/sns/latest/api/API_CreateTopic.html">CreateTopic</a> Amazon SNS API action. You can subscribe email addresses to an Amazon SNS topic through the AWS Management Console, or by using the <a href="http://docs.aws.amazon.com/sns/latest/api/API_Subscribe.html">Subscribe</a> AWS Simple Notification Service (SNS) API action.<br/>
        :return: ClusterId:  The automatically generated ID for a cluster.<br/>        """
        pass

    def create_job(self, JobType: str=None, Resources: JobResource=None, Description: str=None, AddressId: str=None, KmsKeyARN: str=None, RoleARN: str=None, SnowballCapacityPreference: str=None, ShippingOption: str=None, Notification: Notification=None, ClusterId: str=None, SnowballType: str=None, ForwardingAddressId: str=None) -> CreateJobResult:
        """Creates a job to import or export data between Amazon S3 and your on-premises data center. Your AWS account must have the right trust policies and permissions in place to create a job for Snowball. If you're creating a job for a node in a cluster, you only need to provide the <code>clusterId</code> value; the other job attributes are inherited from the cluster. 

        :param self:
        :param ForwardingAddressId:  The forwarding address ID for a job. This field is not supported in most regions.<br/>SnowballType:  The type of AWS Snowball device to use for this job. The only supported device types for cluster jobs are <code>EDGE</code>, <code>EDGE_C</code>, and <code>EDGE_CG</code>.<br/>ClusterId:  The ID of a cluster. If you're creating a job for a node in a cluster, you need to provide only this <code>clusterId</code> value. The other job attributes are inherited from the cluster.<br/>Notification:  Defines the Amazon Simple Notification Service (Amazon SNS) notification settings for this job.:<br/>ShippingOption:  The shipping speed for this job. This speed doesn't dictate how soon you'll get the Snowball, rather it represents how quickly the Snowball moves to its destination while in transit. Regional shipping speeds are as follows: <ul> <li> In Australia, you have access to express shipping. Typically, Snowballs shipped express are delivered in about a day. </li> <li> In the European Union (EU), you have access to express shipping. Typically, Snowballs shipped express are delivered in about a day. In addition, most countries in the EU have access to standard shipping, which typically takes less than a week, one way. </li> <li> In India, Snowballs are delivered in one to seven days. </li> <li> In the US, you have access to one-day shipping and two-day shipping. </li> </ul><br/>SnowballCapacityPreference:  If your job is being created in one of the US regions, you have the option of specifying what size Snowball you'd like for this job. In all other regions, Snowballs come with 80 TB in storage capacity.<br/>RoleARN:  The <code>RoleARN</code> that you want to associate with this job. <code>RoleArn</code>s are created using the <a href="http://docs.aws.amazon.com/IAM/latest/APIReference/API_CreateRole.html">CreateRole</a> AWS Identity and Access Management (IAM) API action.<br/>KmsKeyARN:  The <code>KmsKeyARN</code> that you want to associate with this job. <code>KmsKeyARN</code>s are created using the <a href="http://docs.aws.amazon.com/kms/latest/APIReference/API_CreateKey.html">CreateKey</a> AWS Key Management Service (KMS) API action.<br/>AddressId:  The ID for the address that you want the Snowball shipped to.<br/>Description:  Defines an optional description of this specific job, for example <code>Important Photos 2016-08-11</code>.<br/>Resources:  Defines the Amazon S3 buckets associated with this job. With <code>IMPORT</code> jobs, you specify the bucket or buckets that your transferred data will be imported into. With <code>EXPORT</code> jobs, you specify the bucket or buckets that your transferred data will be exported from. Optionally, you can also specify a <code>KeyRange</code> value. If you choose to export a range, you define the length of the range by providing either an inclusive <code>BeginMarker</code> value, an inclusive <code>EndMarker</code> value, or both. Ranges are UTF-8 binary sorted.:<br/>JobType:  Defines the type of job that you're creating. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Ec2AmiResources</i> The Amazon Machine Images (AMIs) associated with this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LambdaResources</i> The Python-language Lambda functions for this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Resources</i> An array of <code>S3Resource</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotifyAll</i> Any change in job state will trigger a notification for this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobStatesToNotify</i> The list of job states that will trigger a notification for this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsTopicARN</i> The new SNS <code>TopicArn</code> that you want to associate with this job. You can create Amazon Resource Names (ARNs) for topics by using the <a href="http://docs.aws.amazon.com/sns/latest/api/API_CreateTopic.html">CreateTopic</a> Amazon SNS API action. You can subscribe email addresses to an Amazon SNS topic through the AWS Management Console, or by using the <a href="http://docs.aws.amazon.com/sns/latest/api/API_Subscribe.html">Subscribe</a> AWS Simple Notification Service (SNS) API action.<br/>
        :return: JobId:  The automatically generated ID for a job, for example <code>JID123e4567-e89b-12d3-a456-426655440000</code>.<br/>        """
        pass

    def describe_address(self, AddressId: str) -> DescribeAddressResult:
        """Takes an <code>AddressId</code> and returns specific details about that address in the form of an <code>Address</code> object.

        :param self:
        :param AddressId:  The automatically generated ID for a specific address.<br/>
        :return: Address:  The address that you want the Snowball or Snowballs associated with a specific job to be shipped to.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IsRestricted</i> If the address you are creating is a primary address, then set this option to true. This field is not supported in most regions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PhoneNumber</i> The phone number associated with an address that a Snowball is to be delivered to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PostalCode</i> The postal code in an address that a Snowball is to be delivered to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Country</i> The country in an address that a Snowball is to be delivered to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Landmark</i> This field is no longer used and the value is ignored.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PrefectureOrDistrict</i> This field is no longer used and the value is ignored.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StateOrProvince</i> The state or province in an address that a Snowball is to be delivered to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>City</i> The city in an address that a Snowball is to be delivered to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Street3</i> The third line in a street address that a Snowball is to be delivered to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Street2</i> The second line in a street address that a Snowball is to be delivered to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Street1</i> The first line in a street address that a Snowball is to be delivered to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Company</i> The name of the company to receive a Snowball at an address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of a person to receive a Snowball at an address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressId</i> The unique ID for an address.<br/>        """
        pass

    def describe_addresses(self, MaxResults: int=None, NextToken: str=None) -> DescribeAddressesResult:
        """Returns a specified number of <code>ADDRESS</code> objects. Calling this API in one of the US regions will return addresses from the list of all addresses associated with this account in all US regions.

        :param self:
        :param NextToken:  HTTP requests are stateless. To identify what object comes "next" in the list of <code>ADDRESS</code> objects, you have the option of specifying a value for <code>NextToken</code> as the starting point for your list of returned addresses.<br/>MaxResults:  The number of <code>ADDRESS</code> objects to return.<br/>
        :return: NextToken:  HTTP requests are stateless. If you use the automatically generated <code>NextToken</code> value in your next <code>DescribeAddresses</code> call, your list of returned addresses will start from this point in the array.<br/>Addresses:  The Snowball shipping addresses that were created for this account.<br/>        """
        pass

    def describe_cluster(self, ClusterId: str) -> DescribeClusterResult:
        """Returns information about a specific cluster including shipping information, cluster status, and other important metadata.

        :param self:
        :param ClusterId:  The automatically generated ID for a cluster.<br/>
        :return: ClusterMetadata:  Information about a specific cluster, including shipping information, cluster status, and other important metadata.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ForwardingAddressId</i> The ID of the address that you want a cluster shipped to, after it will be shipped to its primary address. This field is not supported in most regions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Notification</i> The Amazon Simple Notification Service (Amazon SNS) notification settings for this cluster.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Notification</b>: The Amazon Simple Notification Service (Amazon SNS) notification settings for this cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ShippingOption</i> The shipping speed for each node in this cluster. This speed doesn't dictate how soon you'll get each device, rather it represents how quickly each device moves to its destination while in transit. Regional shipping speeds are as follows: <ul> <li> In Australia, you have access to express shipping. Typically, devices shipped express are delivered in about a day. </li> <li> In the European Union (EU), you have access to express shipping. Typically, devices shipped express are delivered in about a day. In addition, most countries in the EU have access to standard shipping, which typically takes less than a week, one way. </li> <li> In India, devices are delivered in one to seven days. </li> <li> In the US, you have access to one-day shipping and two-day shipping. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressId</i> The automatically generated ID for a specific address.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Resources</i> The arrays of <a>JobResource</a> objects that can include updated <a>S3Resource</a> objects or <a>LambdaResource</a> objects.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Resources</b>: The arrays of <a>JobResource</a> objects that can include updated <a>S3Resource</a> objects or <a>LambdaResource</a> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The creation date for this cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnowballType</i> The type of AWS Snowball device to use for this cluster. The only supported device types for cluster jobs are <code>EDGE</code>, <code>EDGE_C</code>, and <code>EDGE_CG</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobType</i> The type of job for this cluster. Currently, the only job type supported for clusters is <code>LOCAL_USE</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterState</i> The current status of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> The role ARN associated with this cluster. This ARN was created using the <a href="http://docs.aws.amazon.com/IAM/latest/APIReference/API_CreateRole.html">CreateRole</a> API action in AWS Identity and Access Management (IAM).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyARN</i> The <code>KmsKeyARN</code> Amazon Resource Name (ARN) associated with this cluster. This ARN was created using the <a href="http://docs.aws.amazon.com/kms/latest/APIReference/API_CreateKey.html">CreateKey</a> API action in AWS Key Management Service (AWS KMS).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The optional description of the cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterId</i> The automatically generated ID for a cluster.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Ec2AmiResources</i> The Amazon Machine Images (AMIs) associated with this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LambdaResources</i> The Python-language Lambda functions for this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Resources</i> An array of <code>S3Resource</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NotifyAll</i> Any change in job state will trigger a notification for this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>JobStatesToNotify</i> The list of job states that will trigger a notification for this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsTopicARN</i> The new SNS <code>TopicArn</code> that you want to associate with this job. You can create Amazon Resource Names (ARNs) for topics by using the <a href="http://docs.aws.amazon.com/sns/latest/api/API_CreateTopic.html">CreateTopic</a> Amazon SNS API action. You can subscribe email addresses to an Amazon SNS topic through the AWS Management Console, or by using the <a href="http://docs.aws.amazon.com/sns/latest/api/API_Subscribe.html">Subscribe</a> AWS Simple Notification Service (SNS) API action.<br/>        """
        pass

    def describe_job(self, JobId: str) -> DescribeJobResult:
        """Returns information about a specific job including shipping information, job status, and other important metadata. 

        :param self:
        :param JobId:  The automatically generated ID for a job, for example <code>JID123e4567-e89b-12d3-a456-426655440000</code>.<br/>
        :return: SubJobMetadata:  Information about a specific job part (in the case of an export job), including shipping information, job status, and other important metadata.<br/>JobMetadata:  Information about a specific job, including shipping information, job status, and other important metadata.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ForwardingAddressId</i> The ID of the address that you want a job shipped to, after it will be shipped to its primary address. This field is not supported in most regions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ClusterId</i> The 39-character ID for the cluster, for example <code>CID123e4567-e89b-12d3-a456-426655440000</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobLogInfo</i> Links to Amazon S3 presigned URLs for the job report and logs. For import jobs, the PDF job report becomes available at the end of the import process. For export jobs, your job report typically becomes available while the Snowball for your job part is being delivered to you.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>JobLogInfo</b>: Links to Amazon S3 presigned URLs for the job report and logs. For import jobs, the PDF job report becomes available at the end of the import process. For export jobs, your job report typically becomes available while the Snowball for your job part is being delivered to you.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DataTransferProgress</i> A value that defines the real-time status of a Snowball's data transfer while the device is at AWS. This data is only available while a job has a <code>JobState</code> value of <code>InProgress</code>, for both import and export jobs.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DataTransferProgress</b>: A value that defines the real-time status of a Snowball's data transfer while the device is at AWS. This data is only available while a job has a <code>JobState</code> value of <code>InProgress</code>, for both import and export jobs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Notification</i> The Amazon Simple Notification Service (Amazon SNS) notification settings associated with a specific job. The <code>Notification</code> object is returned as a part of the response syntax of the <code>DescribeJob</code> action in the <code>JobMetadata</code> data type.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Notification</b>: The Amazon Simple Notification Service (Amazon SNS) notification settings associated with a specific job. The <code>Notification</code> object is returned as a part of the response syntax of the <code>DescribeJob</code> action in the <code>JobMetadata</code> data type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnowballCapacityPreference</i> The Snowball capacity preference for this job, specified at job creation. In US regions, you can choose between 50 TB and 80 TB Snowballs. All other regions use 80 TB capacity Snowballs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ShippingDetails</i> A job's shipping information, including inbound and outbound tracking numbers and shipping speed options.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ShippingDetails</b>: A job's shipping information, including inbound and outbound tracking numbers and shipping speed options.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AddressId</i> The ID for the address that you want the Snowball shipped to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleARN</i> The role ARN associated with this job. This ARN was created using the <a href="http://docs.aws.amazon.com/IAM/latest/APIReference/API_CreateRole.html">CreateRole</a> API action in AWS Identity and Access Management (IAM).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KmsKeyARN</i> The Amazon Resource Name (ARN) for the AWS Key Management Service (AWS KMS) key associated with this job. This ARN was created using the <a href="http://docs.aws.amazon.com/kms/latest/APIReference/API_CreateKey.html">CreateKey</a> API action in AWS KMS.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the job, provided at job creation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Resources</i> An array of <code>S3Resource</code> objects. Each <code>S3Resource</code> object represents an Amazon S3 bucket that your transferred data will be exported from or imported into.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Resources</b>: An array of <code>S3Resource</code> objects. Each <code>S3Resource</code> object represents an Amazon S3 bucket that your transferred data will be exported from or imported into.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationDate</i> The creation date for this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnowballType</i> The type of device used with this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobType</i> The type of job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobState</i> The current status of the jobs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobId</i> The automatically generated ID for a job, for example <code>JID123e4567-e89b-12d3-a456-426655440000</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Ec2AmiResources</i> The Amazon Machine Images (AMIs) associated with this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>LambdaResources</i> The Python-language Lambda functions for this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Resources</i> An array of <code>S3Resource</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ShippingOption</i> The shipping speed for a particular job. This speed doesn't dictate how soon you'll get the Snowball from the job's creation date. This speed represents how quickly it moves to its destination while in transit. Regional shipping speeds are as follows: <ul> <li> In Australia, you have access to express shipping. Typically, Snowballs shipped express are delivered in about a day. </li> <li> In the European Union (EU), you have access to express shipping. Typically, Snowballs shipped express are delivered in about a day. In addition, most countries in the EU have access to standard shipping, which typically takes less than a week, one way. </li> <li> In India, Snowballs are delivered in one to seven days. </li> <li> In the United States of America (US), you have access to one-day shipping and two-day shipping. </li> </ul><br/>The <code>Status</code> and <code>TrackingNumber</code> values for a Snowball being returned to AWS for a particular job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>InboundShipment</b>: The <code>Status</code> and <code>TrackingNumber</code> values for a Snowball being returned to AWS for a particular job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NotifyAll</i> Any change in job state will trigger a notification for this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>JobStatesToNotify</i> The list of job states that will trigger a notification for this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsTopicARN</i> The new SNS <code>TopicArn</code> that you want to associate with this job. You can create Amazon Resource Names (ARNs) for topics by using the <a href="http://docs.aws.amazon.com/sns/latest/api/API_CreateTopic.html">CreateTopic</a> Amazon SNS API action. You can subscribe email addresses to an Amazon SNS topic through the AWS Management Console, or by using the <a href="http://docs.aws.amazon.com/sns/latest/api/API_Subscribe.html">Subscribe</a> AWS Simple Notification Service (SNS) API action.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalObjects</i> The total number of objects for a transfer between a Snowball and Amazon S3. This value is set to 0 (zero) until all the keys that will be transferred have been listed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalBytes</i> The total bytes of data for a transfer between a Snowball and Amazon S3. This value is set to 0 (zero) until all the keys that will be transferred have been listed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ObjectsTransferred</i> The number of objects transferred between a Snowball and Amazon S3.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>BytesTransferred</i> The number of bytes transferred between a Snowball and Amazon S3.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>JobFailureLogURI</i> A link to an Amazon S3 presigned URL where the job failure log is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>JobSuccessLogURI</i> A link to an Amazon S3 presigned URL where the job success log is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>JobCompletionReportURI</i> A link to an Amazon S3 presigned URL where the job completion report is located.<br/>        """
        pass

    def get_job_manifest(self, JobId: str) -> GetJobManifestResult:
        """Returns a link to an Amazon S3 presigned URL for the manifest file associated with the specified <code>JobId</code> value. You can access the manifest file for up to 60 minutes after this request has been made. To access the manifest file after 60 minutes have passed, you'll have to make another call to the <code>GetJobManifest</code> action. The manifest is an encrypted file that you can download after your job enters the <code>WithCustomer</code> status. The manifest is decrypted by using the <code>UnlockCode</code> code value, when you pass both values to the Snowball through the Snowball client when the client is started for the first time. As a best practice, we recommend that you don't save a copy of an <code>UnlockCode</code> value in the same location as the manifest file for that job. Saving these separately helps prevent unauthorized parties from gaining access to the Snowball associated with that job. The credentials of a given job, including its manifest file and unlock code, expire 90 days after the job is created.

        :param self:
        :param JobId:  The ID for a job that you want to get the manifest file for, for example <code>JID123e4567-e89b-12d3-a456-426655440000</code>.<br/>
        :return: ManifestURI:  The Amazon S3 presigned URL for the manifest file associated with the specified <code>JobId</code> value.<br/>        """
        pass

    def get_job_unlock_code(self, JobId: str) -> GetJobUnlockCodeResult:
        """Returns the <code>UnlockCode</code> code value for the specified job. A particular <code>UnlockCode</code> value can be accessed for up to 90 days after the associated job has been created. The <code>UnlockCode</code> value is a 29-character code with 25 alphanumeric characters and 4 hyphens. This code is used to decrypt the manifest file when it is passed along with the manifest to the Snowball through the Snowball client when the client is started for the first time. As a best practice, we recommend that you don't save a copy of the <code>UnlockCode</code> in the same location as the manifest file for that job. Saving these separately helps prevent unauthorized parties from gaining access to the Snowball associated with that job.

        :param self:
        :param JobId:  The ID for the job that you want to get the <code>UnlockCode</code> value for, for example <code>JID123e4567-e89b-12d3-a456-426655440000</code>.<br/>
        :return: UnlockCode:  The <code>UnlockCode</code> value for the specified job. The <code>UnlockCode</code> value can be accessed for up to 90 days after the job has been created.<br/>        """
        pass

    def get_snowball_usage(self) -> GetSnowballUsageResult:
        """Returns information about the Snowball service limit for your account, and also the number of Snowballs your account has in use. The default service limit for the number of Snowballs that you can have at one time is 1. If you want to increase your service limit, contact AWS Support.

        :param self:
        :param 
        :return: SnowballsInUse:  The number of Snowballs that this account is currently using.<br/>SnowballLimit:  The service limit for number of Snowballs this account can have at once. The default service limit is 1 (one).<br/>        """
        pass

    def list_cluster_jobs(self, ClusterId: str, MaxResults: int=None, NextToken: str=None) -> ListClusterJobsResult:
        """Returns an array of <code>JobListEntry</code> objects of the specified length. Each <code>JobListEntry</code> object is for a job in the specified cluster and contains a job's state, a job's ID, and other information.

        :param self:
        :param NextToken:  HTTP requests are stateless. To identify what object comes "next" in the list of <code>JobListEntry</code> objects, you have the option of specifying <code>NextToken</code> as the starting point for your returned list.<br/>MaxResults:  The number of <code>JobListEntry</code> objects to return.<br/>ClusterId:  The 39-character ID for the cluster that you want to list, for example <code>CID123e4567-e89b-12d3-a456-426655440000</code>.<br/>
        :return: NextToken:  HTTP requests are stateless. If you use the automatically generated <code>NextToken</code> value in your next <code>ListClusterJobsResult</code> call, your list of returned jobs will start from this point in the array.<br/>JobListEntries:  Each <code>JobListEntry</code> object contains a job's state, a job's ID, and a value that indicates whether the job is a job part, in the case of export jobs. <br/>        """
        pass

    def list_clusters(self, MaxResults: int=None, NextToken: str=None) -> ListClustersResult:
        """Returns an array of <code>ClusterListEntry</code> objects of the specified length. Each <code>ClusterListEntry</code> object contains a cluster's state, a cluster's ID, and other important status information.

        :param self:
        :param NextToken:  HTTP requests are stateless. To identify what object comes "next" in the list of <code>ClusterListEntry</code> objects, you have the option of specifying <code>NextToken</code> as the starting point for your returned list.<br/>MaxResults:  The number of <code>ClusterListEntry</code> objects to return.<br/>
        :return: NextToken:  HTTP requests are stateless. If you use the automatically generated <code>NextToken</code> value in your next <code>ClusterListEntry</code> call, your list of returned clusters will start from this point in the array.<br/>ClusterListEntries:  Each <code>ClusterListEntry</code> object contains a cluster's state, a cluster's ID, and other important status information.<br/>        """
        pass

    def list_compatible_images(self, MaxResults: int=None, NextToken: str=None) -> ListCompatibleImagesResult:
        """This action returns a list of the different Amazon EC2 Amazon Machine Images (AMIs) that are owned by your AWS account that would be supported for use on <code>EDGE</code>, <code>EDGE_C</code>, and <code>EDGE_CG</code> devices. For more information on compatible AMIs, see <a href="http://docs.aws.amazon.com/snowball/latest/developer-guide/using-ec2.html">Using Amazon EC2 Compute Instances</a> in the <i>AWS Snowball Developer Guide</i>.

        :param self:
        :param NextToken:  HTTP requests are stateless. To identify what object comes "next" in the list of compatible images, you can specify a value for <code>NextToken</code> as the starting point for your list of returned images.<br/>MaxResults:  The maximum number of results for the list of compatible images. Currently, each supported device can store 10 AMIs.<br/>
        :return: NextToken:  Because HTTP requests are stateless, this is the starting point for your next list of returned images.<br/>CompatibleImages:  A JSON-formatted object that describes a compatible AMI.<br/>        """
        pass

    def list_jobs(self, MaxResults: int=None, NextToken: str=None) -> ListJobsResult:
        """Returns an array of <code>JobListEntry</code> objects of the specified length. Each <code>JobListEntry</code> object contains a job's state, a job's ID, and a value that indicates whether the job is a job part, in the case of export jobs. Calling this API action in one of the US regions will return jobs from the list of all jobs associated with this account in all US regions.

        :param self:
        :param NextToken:  HTTP requests are stateless. To identify what object comes "next" in the list of <code>JobListEntry</code> objects, you have the option of specifying <code>NextToken</code> as the starting point for your returned list.<br/>MaxResults:  The number of <code>JobListEntry</code> objects to return.<br/>
        :return: NextToken:  HTTP requests are stateless. If you use this automatically generated <code>NextToken</code> value in your next <code>ListJobs</code> call, your returned <code>JobListEntry</code> objects will start from this point in the array.<br/>JobListEntries:  Each <code>JobListEntry</code> object contains a job's state, a job's ID, and a value that indicates whether the job is a job part, in the case of export jobs. <br/>        """
        pass

    def update_cluster(self, ClusterId: str, RoleARN: str=None, Description: str=None, Resources: JobResource=None, AddressId: str=None, ShippingOption: str=None, Notification: Notification=None, ForwardingAddressId: str=None) -> UpdateClusterResult:
        """While a cluster's <code>ClusterState</code> value is in the <code>AwaitingQuorum</code> state, you can update some of the information associated with a cluster. Once the cluster changes to a different job state, usually 60 minutes after the cluster being created, this action is no longer available.

        :param self:
        :param ForwardingAddressId:  The updated ID for the forwarding address for a cluster. This field is not supported in most regions.<br/>Notification:  The new or updated <a>Notification</a> object.:<br/>ShippingOption:  The updated shipping option value of this cluster's <a>ShippingDetails</a> object.<br/>AddressId:  The ID of the updated <a>Address</a> object.<br/>Resources:  The updated arrays of <a>JobResource</a> objects that can include updated <a>S3Resource</a> objects or <a>LambdaResource</a> objects.:<br/>Description:  The updated description of this cluster.<br/>RoleARN:  The new role Amazon Resource Name (ARN) that you want to associate with this cluster. To create a role ARN, use the <a href="http://docs.aws.amazon.com/IAM/latest/APIReference/API_CreateRole.html">CreateRole</a> API action in AWS Identity and Access Management (IAM).<br/>ClusterId:  The cluster ID of the cluster that you want to update, for example <code>CID123e4567-e89b-12d3-a456-426655440000</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Ec2AmiResources</i> The Amazon Machine Images (AMIs) associated with this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LambdaResources</i> The Python-language Lambda functions for this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Resources</i> An array of <code>S3Resource</code> objects.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotifyAll</i> Any change in job state will trigger a notification for this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobStatesToNotify</i> The list of job states that will trigger a notification for this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsTopicARN</i> The new SNS <code>TopicArn</code> that you want to associate with this job. You can create Amazon Resource Names (ARNs) for topics by using the <a href="http://docs.aws.amazon.com/sns/latest/api/API_CreateTopic.html">CreateTopic</a> Amazon SNS API action. You can subscribe email addresses to an Amazon SNS topic through the AWS Management Console, or by using the <a href="http://docs.aws.amazon.com/sns/latest/api/API_Subscribe.html">Subscribe</a> AWS Simple Notification Service (SNS) API action.<br/>
        :return:         """
        pass

    def update_job(self, JobId: str, RoleARN: str=None, Notification: Notification=None, Resources: JobResource=None, AddressId: str=None, ShippingOption: str=None, Description: str=None, SnowballCapacityPreference: str=None, ForwardingAddressId: str=None) -> UpdateJobResult:
        """While a job's <code>JobState</code> value is <code>New</code>, you can update some of the information associated with a job. Once the job changes to a different job state, usually within 60 minutes of the job being created, this action is no longer available.

        :param self:
        :param ForwardingAddressId:  The updated ID for the forwarding address for a job. This field is not supported in most regions.<br/>SnowballCapacityPreference:  The updated <code>SnowballCapacityPreference</code> of this job's <a>JobMetadata</a> object. The 50 TB Snowballs are only available in the US regions.<br/>Description:  The updated description of this job's <a>JobMetadata</a> object.<br/>ShippingOption:  The updated shipping option value of this job's <a>ShippingDetails</a> object.<br/>AddressId:  The ID of the updated <a>Address</a> object.<br/>Resources:  The updated <code>JobResource</code> object, or the updated <a>JobResource</a> object. :<br/>Notification:  The new or updated <a>Notification</a> object.:<br/>RoleARN:  The new role Amazon Resource Name (ARN) that you want to associate with this job. To create a role ARN, use the <a href="http://docs.aws.amazon.com/IAM/latest/APIReference/API_CreateRole.html">CreateRole</a>AWS Identity and Access Management (IAM) API action.<br/>JobId:  The job ID of the job that you want to update, for example <code>JID123e4567-e89b-12d3-a456-426655440000</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotifyAll</i> Any change in job state will trigger a notification for this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>JobStatesToNotify</i> The list of job states that will trigger a notification for this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SnsTopicARN</i> The new SNS <code>TopicArn</code> that you want to associate with this job. You can create Amazon Resource Names (ARNs) for topics by using the <a href="http://docs.aws.amazon.com/sns/latest/api/API_CreateTopic.html">CreateTopic</a> Amazon SNS API action. You can subscribe email addresses to an Amazon SNS topic through the AWS Management Console, or by using the <a href="http://docs.aws.amazon.com/sns/latest/api/API_Subscribe.html">Subscribe</a> AWS Simple Notification Service (SNS) API action.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Ec2AmiResources</i> The Amazon Machine Images (AMIs) associated with this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LambdaResources</i> The Python-language Lambda functions for this job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Resources</i> An array of <code>S3Resource</code> objects.<br/>
        :return:         """
        pass



class SNS(BaseClient):
    """<fullname>Amazon Simple Notification Service</fullname> Amazon Simple Notification Service (Amazon SNS) is a web service that enables you to build distributed web-enabled applications. Applications can use Amazon SNS to easily push real-time notification messages to interested subscribers over multiple delivery protocols. For more information about this product see <a href="http://aws.amazon.com/sns/">https://aws.amazon.com/sns</a>. For detailed information about Amazon SNS features and their associated API calls, see the <a href="https://docs.aws.amazon.com/sns/latest/dg/">Amazon SNS Developer Guide</a>.  We also provide SDKs that enable you to access Amazon SNS from your preferred programming language. The SDKs contain functionality that automatically takes care of tasks such as: cryptographically signing your service requests, retrying requests, and handling error responses. For a list of available SDKs, go to <a href="http://aws.amazon.com/tools/">Tools for Amazon Web Services</a>. """


    class ListEndpointsByPlatformApplicationPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_endpoints_by_platform_application` for valid parameters"""
            pass
    class ListPlatformApplicationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_platform_applications` for valid parameters"""
            pass
    class ListSubscriptionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_subscriptions` for valid parameters"""
            pass
    class ListSubscriptionsByTopicPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_subscriptions_by_topic` for valid parameters"""
            pass
    class ListTopicsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_topics` for valid parameters"""
            pass
    class ListPhoneNumbersOptedOutPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_phone_numbers_opted_out` for valid parameters"""
            pass


    class AddPermissionInput(Mapping):
        pass
    
    class delegate(object):
        pass
    
    class action(object):
        pass
    
    class CheckIfPhoneNumberIsOptedOutInput(Mapping):
        pass
    
    class CheckIfPhoneNumberIsOptedOutResponse(Mapping):
        pass
    
    class ConfirmSubscriptionInput(Mapping):
        pass
    
    class ConfirmSubscriptionResponse(Mapping):
        pass
    
    class CreatePlatformApplicationInput(Mapping):
        pass
    
    class CreatePlatformApplicationResponse(Mapping):
        pass
    
    class MapStringToString(object):
        pass
    
    class CreatePlatformEndpointInput(Mapping):
        pass
    
    class CreateEndpointResponse(Mapping):
        pass
    
    class CreateTopicInput(Mapping):
        pass
    
    class CreateTopicResponse(Mapping):
        pass
    
    class TopicAttributesMap(object):
        pass
    
    class Tag(Mapping):
        pass
    
    class DeleteEndpointInput(Mapping):
        pass
    
    class DeletePlatformApplicationInput(Mapping):
        pass
    
    class DeleteTopicInput(Mapping):
        pass
    
    class GetEndpointAttributesInput(Mapping):
        pass
    
    class GetEndpointAttributesResponse(Mapping):
        pass
    
    class GetPlatformApplicationAttributesInput(Mapping):
        pass
    
    class GetPlatformApplicationAttributesResponse(Mapping):
        pass
    
    class GetSMSAttributesInput(Mapping):
        pass
    
    class GetSMSAttributesResponse(Mapping):
        pass
    
    class String(object):
        pass
    
    class GetSubscriptionAttributesInput(Mapping):
        pass
    
    class GetSubscriptionAttributesResponse(Mapping):
        pass
    
    class GetTopicAttributesInput(Mapping):
        pass
    
    class GetTopicAttributesResponse(Mapping):
        pass
    
    class ListEndpointsByPlatformApplicationInput(Mapping):
        pass
    
    class ListEndpointsByPlatformApplicationResponse(Mapping):
        pass
    
    class ListPhoneNumbersOptedOutInput(Mapping):
        pass
    
    class ListPhoneNumbersOptedOutResponse(Mapping):
        pass
    
    class ListPlatformApplicationsInput(Mapping):
        pass
    
    class ListPlatformApplicationsResponse(Mapping):
        pass
    
    class ListSubscriptionsInput(Mapping):
        pass
    
    class ListSubscriptionsResponse(Mapping):
        pass
    
    class ListSubscriptionsByTopicInput(Mapping):
        pass
    
    class ListSubscriptionsByTopicResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class ListTopicsInput(Mapping):
        pass
    
    class ListTopicsResponse(Mapping):
        pass
    
    class OptInPhoneNumberInput(Mapping):
        pass
    
    class OptInPhoneNumberResponse(Mapping):
        pass
    
    class PublishInput(Mapping):
        pass
    
    class PublishResponse(Mapping):
        pass
    
    class MessageAttributeMap(object):
        pass
    
    class RemovePermissionInput(Mapping):
        pass
    
    class SetEndpointAttributesInput(Mapping):
        pass
    
    class SetPlatformApplicationAttributesInput(Mapping):
        pass
    
    class SetSMSAttributesInput(Mapping):
        pass
    
    class SetSMSAttributesResponse(Mapping):
        pass
    
    class SetSubscriptionAttributesInput(Mapping):
        pass
    
    class SetTopicAttributesInput(Mapping):
        pass
    
    class SubscribeInput(Mapping):
        pass
    
    class SubscribeResponse(Mapping):
        pass
    
    class SubscriptionAttributesMap(object):
        pass
    
    class boolean(object):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UnsubscribeInput(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    def add_permission(self, ActionName: List[action], AWSAccountId: List[delegate], Label: str, TopicArn: str):
        """Adds a statement to a topic's access control policy, granting access for the specified AWS accounts to the specified actions.

        :param self:
        :param ActionName:  The action you want to allow for the specified principal(s). Valid values: any Amazon SNS action name.<br/>AWSAccountId:  The AWS account IDs of the users (principals) who will be given access to the specified actions. The users must have AWS accounts, but do not need to be signed up for this service.<br/>Label:  A unique identifier for the new policy statement.<br/>TopicArn:  The ARN of the topic whose access control policy you wish to modify.<br/>
        :return:         """
        pass

    def check_if_phone_number_is_opted_out(self, phoneNumber: str) -> CheckIfPhoneNumberIsOptedOutResponse:
        """Accepts a phone number and indicates whether the phone holder has opted out of receiving SMS messages from your account. You cannot send SMS messages to a number that is opted out. To resume sending messages, you can opt in the number by using the <code>OptInPhoneNumber</code> action.

        :param self:
        :param phoneNumber:  The phone number for which you want to check the opt out status.<br/>
        :return: isOptedOut:  Indicates whether the phone number is opted out: <ul> <li>  <code>true</code> – The phone number is opted out, meaning you cannot publish SMS messages to it. </li> <li>  <code>false</code> – The phone number is opted in, meaning you can publish SMS messages to it. </li> </ul><br/>        """
        pass

    def confirm_subscription(self, Token: str, TopicArn: str, AuthenticateOnUnsubscribe: str=None) -> ConfirmSubscriptionResponse:
        """Verifies an endpoint owner's intent to receive messages by validating the token sent to the endpoint by an earlier <code>Subscribe</code> action. If the token is valid, the action creates a new subscription and returns its Amazon Resource Name (ARN). This call requires an AWS signature only when the <code>AuthenticateOnUnsubscribe</code> flag is set to "true".

        :param self:
        :param AuthenticateOnUnsubscribe:  Disallows unauthenticated unsubscribes of the subscription. If the value of this parameter is <code>true</code> and the request has an AWS signature, then only the topic owner and the subscription owner can unsubscribe the endpoint. The unsubscribe action requires AWS authentication. <br/>Token:  Short-lived token sent to an endpoint during the <code>Subscribe</code> action.<br/>TopicArn:  The ARN of the topic for which you wish to confirm a subscription.<br/>
        :return: SubscriptionArn:  The ARN of the created subscription.<br/>        """
        pass

    def create_platform_application(self, Attributes: MapStringToString, Platform: str, Name: str) -> CreatePlatformApplicationResponse:
        """Creates a platform application object for one of the supported push notification services, such as APNS and FCM, to which devices and mobile apps may register. You must specify PlatformPrincipal and PlatformCredential attributes when using the <code>CreatePlatformApplication</code> action. The PlatformPrincipal is received from the notification service. For APNS/APNS_SANDBOX, PlatformPrincipal is "SSL certificate". For GCM, PlatformPrincipal is not applicable. For ADM, PlatformPrincipal is "client id". The PlatformCredential is also received from the notification service. For WNS, PlatformPrincipal is "Package Security Identifier". For MPNS, PlatformPrincipal is "TLS certificate". For Baidu, PlatformPrincipal is "API key". For APNS/APNS_SANDBOX, PlatformCredential is "private key". For GCM, PlatformCredential is "API key". For ADM, PlatformCredential is "client secret". For WNS, PlatformCredential is "secret key". For MPNS, PlatformCredential is "private key". For Baidu, PlatformCredential is "secret key". The PlatformApplicationArn that is returned when using <code>CreatePlatformApplication</code> is then used as an attribute for the <code>CreatePlatformEndpoint</code> action. For more information, see <a href="https://docs.aws.amazon.com/sns/latest/dg/SNSMobilePush.html">Using Amazon SNS Mobile Push Notifications</a>. For more information about obtaining the PlatformPrincipal and PlatformCredential for each of the supported push notification services, see <a href="https://docs.aws.amazon.com/sns/latest/dg/mobile-push-apns.html">Getting Started with Apple Push Notification Service</a>, <a href="https://docs.aws.amazon.com/sns/latest/dg/mobile-push-adm.html">Getting Started with Amazon Device Messaging</a>, <a href="https://docs.aws.amazon.com/sns/latest/dg/mobile-push-baidu.html">Getting Started with Baidu Cloud Push</a>, <a href="https://docs.aws.amazon.com/sns/latest/dg/mobile-push-gcm.html">Getting Started with Google Cloud Messaging for Android</a>, <a href="https://docs.aws.amazon.com/sns/latest/dg/mobile-push-mpns.html">Getting Started with MPNS</a>, or <a href="https://docs.aws.amazon.com/sns/latest/dg/mobile-push-wns.html">Getting Started with WNS</a>. 

        :param self:
        :param Attributes:  For a list of attributes, see <a href="https://docs.aws.amazon.com/sns/latest/api/API_SetPlatformApplicationAttributes.html">SetPlatformApplicationAttributes</a> <br/>Platform:  The following platforms are supported: ADM (Amazon Device Messaging), APNS (Apple Push Notification Service), APNS_SANDBOX, and GCM (Google Cloud Messaging).<br/>Name:  Application names must be made up of only uppercase and lowercase ASCII letters, numbers, underscores, hyphens, and periods, and must be between 1 and 256 characters long.<br/>
        :return: PlatformApplicationArn:  PlatformApplicationArn is returned.<br/>        """
        pass

    def create_platform_endpoint(self, Token: str, PlatformApplicationArn: str, CustomUserData: str=None, Attributes: MapStringToString=None) -> CreateEndpointResponse:
        """Creates an endpoint for a device and mobile app on one of the supported push notification services, such as GCM and APNS. <code>CreatePlatformEndpoint</code> requires the PlatformApplicationArn that is returned from <code>CreatePlatformApplication</code>. The EndpointArn that is returned when using <code>CreatePlatformEndpoint</code> can then be used by the <code>Publish</code> action to send a message to a mobile app or by the <code>Subscribe</code> action for subscription to a topic. The <code>CreatePlatformEndpoint</code> action is idempotent, so if the requester already owns an endpoint with the same device token and attributes, that endpoint's ARN is returned without creating a new endpoint. For more information, see <a href="https://docs.aws.amazon.com/sns/latest/dg/SNSMobilePush.html">Using Amazon SNS Mobile Push Notifications</a>.  When using <code>CreatePlatformEndpoint</code> with Baidu, two attributes must be provided: ChannelId and UserId. The token field must also contain the ChannelId. For more information, see <a href="https://docs.aws.amazon.com/sns/latest/dg/SNSMobilePushBaiduEndpoint.html">Creating an Amazon SNS Endpoint for Baidu</a>. 

        :param self:
        :param Attributes:  For a list of attributes, see <a href="https://docs.aws.amazon.com/sns/latest/api/API_SetEndpointAttributes.html">SetEndpointAttributes</a>.<br/>CustomUserData:  Arbitrary user data to associate with the endpoint. Amazon SNS does not use this data. The data must be in UTF-8 format and less than 2KB.<br/>Token:  Unique identifier created by the notification service for an app on a device. The specific name for Token will vary, depending on which notification service is being used. For example, when using APNS as the notification service, you need the device token. Alternatively, when using GCM or ADM, the device token equivalent is called the registration ID.<br/>PlatformApplicationArn:  PlatformApplicationArn returned from CreatePlatformApplication is used to create a an endpoint.<br/>
        :return: EndpointArn:  EndpointArn returned from CreateEndpoint action.<br/>        """
        pass

    def create_topic(self, Name: str, Attributes: TopicAttributesMap=None, Tags: List[Tag]=None) -> CreateTopicResponse:
        """Creates a topic to which notifications can be published. Users can create at most 100,000 topics. For more information, see <a href="http://aws.amazon.com/sns/">https://aws.amazon.com/sns</a>. This action is idempotent, so if the requester already owns a topic with the specified name, that topic's ARN is returned without creating a new topic.

        :param self:
        :param Tags:  The list of tags to add to a new topic.<br/>Attributes:  A map of attributes with their corresponding values. The following lists the names, descriptions, and values of the special request parameters that the <code>CreateTopic</code> action uses: <ul> <li>  <code>DeliveryPolicy</code> – The policy that defines how Amazon SNS retries failed deliveries to HTTP/S endpoints. </li> <li>  <code>DisplayName</code> – The display name to use for a topic with SMS subscriptions. </li> <li>  <code>Policy</code> – The policy that defines who can access your topic. By default, only the topic owner can publish or subscribe to the topic. </li> </ul> The following attribute applies only to <a href="https://docs.aws.amazon.com/sns/latest/dg/sns-server-side-encryption.html">server-side-encryption</a>: <ul> <li>  <code>KmsMasterKeyId</code> - The ID of an AWS-managed customer master key (CMK) for Amazon SNS or a custom CMK. For more information, see <a href="https://docs.aws.amazon.com/sns/latest/dg/sns-server-side-encryption.html#sse-key-terms">Key Terms</a>. For more examples, see <a href="https://docs.aws.amazon.com/kms/latest/APIReference/API_DescribeKey.html#API_DescribeKey_RequestParameters">KeyId</a> in the <i>AWS Key Management Service API Reference</i>.  </li> </ul><br/>Name:  The name of the topic you want to create. Constraints: Topic names must be made up of only uppercase and lowercase ASCII letters, numbers, underscores, and hyphens, and must be between 1 and 256 characters long.<br/>
        :return: TopicArn:  The Amazon Resource Name (ARN) assigned to the created topic.<br/>        """
        pass

    def delete_endpoint(self, EndpointArn: str):
        """Deletes the endpoint for a device and mobile app from Amazon SNS. This action is idempotent. For more information, see <a href="https://docs.aws.amazon.com/sns/latest/dg/SNSMobilePush.html">Using Amazon SNS Mobile Push Notifications</a>.  When you delete an endpoint that is also subscribed to a topic, then you must also unsubscribe the endpoint from the topic.

        :param self:
        :param EndpointArn:  EndpointArn of endpoint to delete.<br/>
        :return:         """
        pass

    def delete_platform_application(self, PlatformApplicationArn: str):
        """Deletes a platform application object for one of the supported push notification services, such as APNS and GCM. For more information, see <a href="https://docs.aws.amazon.com/sns/latest/dg/SNSMobilePush.html">Using Amazon SNS Mobile Push Notifications</a>. 

        :param self:
        :param PlatformApplicationArn:  PlatformApplicationArn of platform application object to delete.<br/>
        :return:         """
        pass

    def delete_topic(self, TopicArn: str):
        """Deletes a topic and all its subscriptions. Deleting a topic might prevent some messages previously sent to the topic from being delivered to subscribers. This action is idempotent, so deleting a topic that does not exist does not result in an error.

        :param self:
        :param TopicArn:  The ARN of the topic you want to delete.<br/>
        :return:         """
        pass

    def get_endpoint_attributes(self, EndpointArn: str) -> GetEndpointAttributesResponse:
        """Retrieves the endpoint attributes for a device on one of the supported push notification services, such as GCM and APNS. For more information, see <a href="https://docs.aws.amazon.com/sns/latest/dg/SNSMobilePush.html">Using Amazon SNS Mobile Push Notifications</a>. 

        :param self:
        :param EndpointArn:  EndpointArn for GetEndpointAttributes input.<br/>
        :return: Attributes:  Attributes include the following: <ul> <li>  <code>CustomUserData</code> – arbitrary user data to associate with the endpoint. Amazon SNS does not use this data. The data must be in UTF-8 format and less than 2KB. </li> <li>  <code>Enabled</code> – flag that enables/disables delivery to the endpoint. Amazon SNS will set this to false when a notification service indicates to Amazon SNS that the endpoint is invalid. Users can set it back to true, typically after updating Token. </li> <li>  <code>Token</code> – device token, also referred to as a registration id, for an app and mobile device. This is returned from the notification service when an app and mobile device are registered with the notification service. </li> </ul><br/>        """
        pass

    def get_platform_application_attributes(self, PlatformApplicationArn: str) -> GetPlatformApplicationAttributesResponse:
        """Retrieves the attributes of the platform application object for the supported push notification services, such as APNS and GCM. For more information, see <a href="https://docs.aws.amazon.com/sns/latest/dg/SNSMobilePush.html">Using Amazon SNS Mobile Push Notifications</a>. 

        :param self:
        :param PlatformApplicationArn:  PlatformApplicationArn for GetPlatformApplicationAttributesInput.<br/>
        :return: Attributes:  Attributes include the following: <ul> <li>  <code>EventEndpointCreated</code> – Topic ARN to which EndpointCreated event notifications should be sent. </li> <li>  <code>EventEndpointDeleted</code> – Topic ARN to which EndpointDeleted event notifications should be sent. </li> <li>  <code>EventEndpointUpdated</code> – Topic ARN to which EndpointUpdate event notifications should be sent. </li> <li>  <code>EventDeliveryFailure</code> – Topic ARN to which DeliveryFailure event notifications should be sent upon Direct Publish delivery failure (permanent) to one of the application's endpoints. </li> </ul><br/>        """
        pass

    def get_sms_attributes(self, attributes: List[String]=None) -> GetSMSAttributesResponse:
        """Returns the settings for sending SMS messages from your account. These settings are set with the <code>SetSMSAttributes</code> action.

        :param self:
        :param attributes:  A list of the individual attribute names, such as <code>MonthlySpendLimit</code>, for which you want values. For all attribute names, see <a href="https://docs.aws.amazon.com/sns/latest/api/API_SetSMSAttributes.html">SetSMSAttributes</a>. If you don't use this parameter, Amazon SNS returns all SMS attributes.<br/>
        :return: attributes:  The SMS attribute names and their values.<br/>        """
        pass

    def get_subscription_attributes(self, SubscriptionArn: str) -> GetSubscriptionAttributesResponse:
        """Returns all of the properties of a subscription.

        :param self:
        :param SubscriptionArn:  The ARN of the subscription whose properties you want to get.<br/>
        :return: Attributes:  A map of the subscription's attributes. Attributes in this map include the following: <ul> <li>  <code>ConfirmationWasAuthenticated</code> – <code>true</code> if the subscription confirmation request was authenticated. </li> <li>  <code>DeliveryPolicy</code> – The JSON serialization of the subscription's delivery policy. </li> <li>  <code>EffectiveDeliveryPolicy</code> – The JSON serialization of the effective delivery policy that takes into account the topic delivery policy and account system defaults. </li> <li>  <code>FilterPolicy</code> – The filter policy JSON that is assigned to the subscription. </li> <li>  <code>Owner</code> – The AWS account ID of the subscription's owner. </li> <li>  <code>PendingConfirmation</code> – <code>true</code> if the subscription hasn't been confirmed. To confirm a pending subscription, call the <code>ConfirmSubscription</code> action with a confirmation token. </li> <li>  <code>RawMessageDelivery</code> – <code>true</code> if raw message delivery is enabled for the subscription. Raw messages are free of JSON formatting and can be sent to HTTP/S and Amazon SQS endpoints. </li> <li>  <code>SubscriptionArn</code> – The subscription's ARN. </li> <li>  <code>TopicArn</code> – The topic ARN that the subscription is associated with. </li> </ul><br/>        """
        pass

    def get_topic_attributes(self, TopicArn: str) -> GetTopicAttributesResponse:
        """Returns all of the properties of a topic. Topic properties returned might differ based on the authorization of the user.

        :param self:
        :param TopicArn:  The ARN of the topic whose properties you want to get.<br/>
        :return: Attributes:  A map of the topic's attributes. Attributes in this map include the following: <ul> <li>  <code>TopicArn</code> – the topic's ARN </li> <li>  <code>Owner</code> – the AWS account ID of the topic's owner </li> <li>  <code>Policy</code> – the JSON serialization of the topic's access control policy </li> <li>  <code>DisplayName</code> – the human-readable name used in the "From" field for notifications to email and email-json endpoints </li> <li>  <code>SubscriptionsPending</code> – the number of subscriptions pending confirmation on this topic </li> <li>  <code>SubscriptionsConfirmed</code> – the number of confirmed subscriptions on this topic </li> <li>  <code>SubscriptionsDeleted</code> – the number of deleted subscriptions on this topic </li> <li>  <code>DeliveryPolicy</code> – the JSON serialization of the topic's delivery policy </li> <li>  <code>EffectiveDeliveryPolicy</code> – the JSON serialization of the effective delivery policy that takes into account system defaults </li> </ul><br/>        """
        pass

    def list_endpoints_by_platform_application(self, PlatformApplicationArn: str, NextToken: str=None) -> ListEndpointsByPlatformApplicationResponse:
        """Lists the endpoints and endpoint attributes for devices in a supported push notification service, such as GCM and APNS. The results for <code>ListEndpointsByPlatformApplication</code> are paginated and return a limited list of endpoints, up to 100. If additional records are available after the first page results, then a NextToken string will be returned. To receive the next page, you call <code>ListEndpointsByPlatformApplication</code> again using the NextToken string received from the previous call. When there are no more records to return, NextToken will be null. For more information, see <a href="https://docs.aws.amazon.com/sns/latest/dg/SNSMobilePush.html">Using Amazon SNS Mobile Push Notifications</a>.  This action is throttled at 30 transactions per second (TPS).

        :param self:
        :param NextToken:  NextToken string is used when calling ListEndpointsByPlatformApplication action to retrieve additional records that are available after the first page results.<br/>PlatformApplicationArn:  PlatformApplicationArn for ListEndpointsByPlatformApplicationInput action.<br/>
        :return: NextToken:  NextToken string is returned when calling ListEndpointsByPlatformApplication action if additional records are available after the first page results.<br/>Endpoints:  Endpoints returned for ListEndpointsByPlatformApplication action.<br/>        """
        pass

    def list_phone_numbers_opted_out(self, nextToken: str=None) -> ListPhoneNumbersOptedOutResponse:
        """Returns a list of phone numbers that are opted out, meaning you cannot send SMS messages to them. The results for <code>ListPhoneNumbersOptedOut</code> are paginated, and each page returns up to 100 phone numbers. If additional phone numbers are available after the first page of results, then a <code>NextToken</code> string will be returned. To receive the next page, you call <code>ListPhoneNumbersOptedOut</code> again using the <code>NextToken</code> string received from the previous call. When there are no more records to return, <code>NextToken</code> will be null.

        :param self:
        :param nextToken:  A <code>NextToken</code> string is used when you call the <code>ListPhoneNumbersOptedOut</code> action to retrieve additional records that are available after the first page of results.<br/>
        :return: nextToken:  A <code>NextToken</code> string is returned when you call the <code>ListPhoneNumbersOptedOut</code> action if additional records are available after the first page of results.<br/>phoneNumbers:  A list of phone numbers that are opted out of receiving SMS messages. The list is paginated, and each page can contain up to 100 phone numbers.<br/>        """
        pass

    def list_platform_applications(self, NextToken: str=None) -> ListPlatformApplicationsResponse:
        """Lists the platform application objects for the supported push notification services, such as APNS and GCM. The results for <code>ListPlatformApplications</code> are paginated and return a limited list of applications, up to 100. If additional records are available after the first page results, then a NextToken string will be returned. To receive the next page, you call <code>ListPlatformApplications</code> using the NextToken string received from the previous call. When there are no more records to return, NextToken will be null. For more information, see <a href="https://docs.aws.amazon.com/sns/latest/dg/SNSMobilePush.html">Using Amazon SNS Mobile Push Notifications</a>.  This action is throttled at 15 transactions per second (TPS).

        :param self:
        :param NextToken:  NextToken string is used when calling ListPlatformApplications action to retrieve additional records that are available after the first page results.<br/>
        :return: NextToken:  NextToken string is returned when calling ListPlatformApplications action if additional records are available after the first page results.<br/>PlatformApplications:  Platform applications returned when calling ListPlatformApplications action.<br/>        """
        pass

    def list_subscriptions(self, NextToken: str=None) -> ListSubscriptionsResponse:
        """Returns a list of the requester's subscriptions. Each call returns a limited list of subscriptions, up to 100. If there are more subscriptions, a <code>NextToken</code> is also returned. Use the <code>NextToken</code> parameter in a new <code>ListSubscriptions</code> call to get further results. This action is throttled at 30 transactions per second (TPS).

        :param self:
        :param NextToken:  Token returned by the previous <code>ListSubscriptions</code> request.<br/>
        :return: NextToken:  Token to pass along to the next <code>ListSubscriptions</code> request. This element is returned if there are more subscriptions to retrieve.<br/>Subscriptions:  A list of subscriptions.<br/>        """
        pass

    def list_subscriptions_by_topic(self, TopicArn: str, NextToken: str=None) -> ListSubscriptionsByTopicResponse:
        """Returns a list of the subscriptions to a specific topic. Each call returns a limited list of subscriptions, up to 100. If there are more subscriptions, a <code>NextToken</code> is also returned. Use the <code>NextToken</code> parameter in a new <code>ListSubscriptionsByTopic</code> call to get further results. This action is throttled at 30 transactions per second (TPS).

        :param self:
        :param NextToken:  Token returned by the previous <code>ListSubscriptionsByTopic</code> request.<br/>TopicArn:  The ARN of the topic for which you wish to find subscriptions.<br/>
        :return: NextToken:  Token to pass along to the next <code>ListSubscriptionsByTopic</code> request. This element is returned if there are more subscriptions to retrieve.<br/>Subscriptions:  A list of subscriptions.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceArn: str) -> ListTagsForResourceResponse:
        """List all tags added to the specified Amazon SNS topic. For an overview, see <a href="https://docs.aws.amazon.com/sns/latest/dg/sns-tags.html">Amazon SNS Tags</a> in the <i>Amazon Simple Notification Service Developer Guide</i>.

        :param self:
        :param ResourceArn:  The ARN of the topic for which to list tags.<br/>
        :return: Tags:  The tags associated with the specified topic.<br/>        """
        pass

    def list_topics(self, NextToken: str=None) -> ListTopicsResponse:
        """Returns a list of the requester's topics. Each call returns a limited list of topics, up to 100. If there are more topics, a <code>NextToken</code> is also returned. Use the <code>NextToken</code> parameter in a new <code>ListTopics</code> call to get further results. This action is throttled at 30 transactions per second (TPS).

        :param self:
        :param NextToken:  Token returned by the previous <code>ListTopics</code> request.<br/>
        :return: NextToken:  Token to pass along to the next <code>ListTopics</code> request. This element is returned if there are additional topics to retrieve.<br/>Topics:  A list of topic ARNs.<br/>        """
        pass

    def opt_in_phone_number(self, phoneNumber: str) -> OptInPhoneNumberResponse:
        """Use this request to opt in a phone number that is opted out, which enables you to resume sending SMS messages to the number. You can opt in a phone number only once every 30 days.

        :param self:
        :param phoneNumber:  The phone number to opt in.<br/>
        :return:         """
        pass

    def publish(self, Message: str, TopicArn: str=None, TargetArn: str=None, PhoneNumber: str=None, Subject: str=None, MessageStructure: str=None, MessageAttributes: MessageAttributeMap=None) -> PublishResponse:
        """Sends a message to an Amazon SNS topic or sends a text message (SMS message) directly to a phone number.  If you send a message to a topic, Amazon SNS delivers the message to each endpoint that is subscribed to the topic. The format of the message depends on the notification protocol for each subscribed endpoint. When a <code>messageId</code> is returned, the message has been saved and Amazon SNS will attempt to deliver it shortly. To use the <code>Publish</code> action for sending a message to a mobile endpoint, such as an app on a Kindle device or mobile phone, you must specify the EndpointArn for the TargetArn parameter. The EndpointArn is returned when making a call with the <code>CreatePlatformEndpoint</code> action.  For more information about formatting messages, see <a href="https://docs.aws.amazon.com/sns/latest/dg/mobile-push-send-custommessage.html">Send Custom Platform-Specific Payloads in Messages to Mobile Devices</a>. 

        :param self:
        :param MessageAttributes:  Message attributes for Publish action.<br/>MessageStructure:  Set <code>MessageStructure</code> to <code>json</code> if you want to send a different message for each protocol. For example, using one publish action, you can send a short message to your SMS subscribers and a longer message to your email subscribers. If you set <code>MessageStructure</code> to <code>json</code>, the value of the <code>Message</code> parameter must:  <ul> <li> be a syntactically valid JSON object; and </li> <li> contain at least a top-level JSON key of "default" with a value that is a string. </li> </ul> You can define other top-level keys that define the message you want to send to a specific transport protocol (e.g., "http"). For information about sending different messages for each protocol using the AWS Management Console, go to <a href="https://docs.aws.amazon.com/sns/latest/gsg/Publish.html#sns-message-formatting-by-protocol">Create Different Messages for Each Protocol</a> in the <i>Amazon Simple Notification Service Getting Started Guide</i>.  Valid value: <code>json</code> <br/>Subject:  Optional parameter to be used as the "Subject" line when the message is delivered to email endpoints. This field will also be included, if present, in the standard JSON messages delivered to other endpoints. Constraints: Subjects must be ASCII text that begins with a letter, number, or punctuation mark; must not include line breaks or control characters; and must be less than 100 characters long.<br/>Message:  The message you want to send. <important> The <code>Message</code> parameter is always a string. If you set <code>MessageStructure</code> to <code>json</code>, you must string-encode the <code>Message</code> parameter. </important> If you are publishing to a topic and you want to send the same message to all transport protocols, include the text of the message as a String value. If you want to send different messages for each transport protocol, set the value of the <code>MessageStructure</code> parameter to <code>json</code> and use a JSON object for the <code>Message</code> parameter.  <p/> Constraints: <ul> <li> With the exception of SMS, messages must be UTF-8 encoded strings and at most 256 KB in size (262,144 bytes, not 262,144 characters). </li> <li> For SMS, each message can contain up to 140 characters. This character limit depends on the encoding schema. For example, an SMS message can contain 160 GSM characters, 140 ASCII characters, or 70 UCS-2 characters. If you publish a message that exceeds this size limit, Amazon SNS sends the message as multiple messages, each fitting within the size limit. Messages aren't truncated mid-word but are cut off at whole-word boundaries. The total size limit for a single SMS <code>Publish</code> action is 1,600 characters. </li> </ul> JSON-specific constraints: <ul> <li> Keys in the JSON object that correspond to supported transport protocols must have simple JSON string values. </li> <li> The values will be parsed (unescaped) before they are used in outgoing messages. </li> <li> Outbound notifications are JSON encoded (meaning that the characters will be reescaped for sending). </li> <li> Values have a minimum length of 0 (the empty string, "", is allowed). </li> <li> Values have a maximum length bounded by the overall message size (so, including multiple protocols may limit message sizes). </li> <li> Non-string values will cause the key to be ignored. </li> <li> Keys that do not correspond to supported transport protocols are ignored. </li> <li> Duplicate keys are not allowed. </li> <li> Failure to parse or validate any key or value in the message will cause the <code>Publish</code> call to return an error (no partial delivery). </li> </ul><br/>PhoneNumber:  The phone number to which you want to deliver an SMS message. Use E.164 format. If you don't specify a value for the <code>PhoneNumber</code> parameter, you must specify a value for the <code>TargetArn</code> or <code>TopicArn</code> parameters.<br/>TargetArn:  If you don't specify a value for the <code>TargetArn</code> parameter, you must specify a value for the <code>PhoneNumber</code> or <code>TopicArn</code> parameters.<br/>TopicArn:  The topic you want to publish to. If you don't specify a value for the <code>TopicArn</code> parameter, you must specify a value for the <code>PhoneNumber</code> or <code>TargetArn</code> parameters.<br/>
        :return: MessageId:  Unique identifier assigned to the published message. Length Constraint: Maximum 100 characters<br/>        """
        pass

    def remove_permission(self, Label: str, TopicArn: str):
        """Removes a statement from a topic's access control policy.

        :param self:
        :param Label:  The unique label of the statement you want to remove.<br/>TopicArn:  The ARN of the topic whose access control policy you wish to modify.<br/>
        :return:         """
        pass

    def set_endpoint_attributes(self, Attributes: MapStringToString, EndpointArn: str):
        """Sets the attributes for an endpoint for a device on one of the supported push notification services, such as GCM and APNS. For more information, see <a href="https://docs.aws.amazon.com/sns/latest/dg/SNSMobilePush.html">Using Amazon SNS Mobile Push Notifications</a>. 

        :param self:
        :param Attributes:  A map of the endpoint attributes. Attributes in this map include the following: <ul> <li>  <code>CustomUserData</code> – arbitrary user data to associate with the endpoint. Amazon SNS does not use this data. The data must be in UTF-8 format and less than 2KB. </li> <li>  <code>Enabled</code> – flag that enables/disables delivery to the endpoint. Amazon SNS will set this to false when a notification service indicates to Amazon SNS that the endpoint is invalid. Users can set it back to true, typically after updating Token. </li> <li>  <code>Token</code> – device token, also referred to as a registration id, for an app and mobile device. This is returned from the notification service when an app and mobile device are registered with the notification service. </li> </ul><br/>EndpointArn:  EndpointArn used for SetEndpointAttributes action.<br/>
        :return:         """
        pass

    def set_platform_application_attributes(self, Attributes: MapStringToString, PlatformApplicationArn: str):
        """Sets the attributes of the platform application object for the supported push notification services, such as APNS and GCM. For more information, see <a href="https://docs.aws.amazon.com/sns/latest/dg/SNSMobilePush.html">Using Amazon SNS Mobile Push Notifications</a>. For information on configuring attributes for message delivery status, see <a href="https://docs.aws.amazon.com/sns/latest/dg/sns-msg-status.html">Using Amazon SNS Application Attributes for Message Delivery Status</a>. 

        :param self:
        :param Attributes:  A map of the platform application attributes. Attributes in this map include the following: <ul> <li>  <code>PlatformCredential</code> – The credential received from the notification service. For APNS/APNS_SANDBOX, PlatformCredential is private key. For GCM, PlatformCredential is "API key". For ADM, PlatformCredential is "client secret". </li> <li>  <code>PlatformPrincipal</code> – The principal received from the notification service. For APNS/APNS_SANDBOX, PlatformPrincipal is SSL certificate. For GCM, PlatformPrincipal is not applicable. For ADM, PlatformPrincipal is "client id". </li> <li>  <code>EventEndpointCreated</code> – Topic ARN to which EndpointCreated event notifications should be sent. </li> <li>  <code>EventEndpointDeleted</code> – Topic ARN to which EndpointDeleted event notifications should be sent. </li> <li>  <code>EventEndpointUpdated</code> – Topic ARN to which EndpointUpdate event notifications should be sent. </li> <li>  <code>EventDeliveryFailure</code> – Topic ARN to which DeliveryFailure event notifications should be sent upon Direct Publish delivery failure (permanent) to one of the application's endpoints. </li> <li>  <code>SuccessFeedbackRoleArn</code> – IAM role ARN used to give Amazon SNS write access to use CloudWatch Logs on your behalf. </li> <li>  <code>FailureFeedbackRoleArn</code> – IAM role ARN used to give Amazon SNS write access to use CloudWatch Logs on your behalf. </li> <li>  <code>SuccessFeedbackSampleRate</code> – Sample rate percentage (0-100) of successfully delivered messages. </li> </ul><br/>PlatformApplicationArn:  PlatformApplicationArn for SetPlatformApplicationAttributes action.<br/>
        :return:         """
        pass

    def set_sms_attributes(self, attributes: MapStringToString) -> SetSMSAttributesResponse:
        """Use this request to set the default settings for sending SMS messages and receiving daily SMS usage reports. You can override some of these settings for a single message when you use the <code>Publish</code> action with the <code>MessageAttributes.entry.N</code> parameter. For more information, see <a href="https://docs.aws.amazon.com/sns/latest/dg/sms_publish-to-phone.html">Sending an SMS Message</a> in the <i>Amazon SNS Developer Guide</i>.

        :param self:
        :param attributes:  The default settings for sending SMS messages from your account. You can set values for the following attribute names:  <code>MonthlySpendLimit</code> – The maximum amount in USD that you are willing to spend each month to send SMS messages. When Amazon SNS determines that sending an SMS message would incur a cost that exceeds this limit, it stops sending SMS messages within minutes. <important> Amazon SNS stops sending SMS messages within minutes of the limit being crossed. During that interval, if you continue to send SMS messages, you will incur costs that exceed your limit. </important> By default, the spend limit is set to the maximum allowed by Amazon SNS. If you want to raise the limit, submit an <a href="https://console.aws.amazon.com/support/home#/case/create?issueType=service-limit-increase&amp;limitType=service-code-sns">SNS Limit Increase case</a>. For <b>New limit value</b>, enter your desired monthly spend limit. In the <b>Use Case Description</b> field, explain that you are requesting an SMS monthly spend limit increase.  <code>DeliveryStatusIAMRole</code> – The ARN of the IAM role that allows Amazon SNS to write logs about SMS deliveries in CloudWatch Logs. For each SMS message that you send, Amazon SNS writes a log that includes the message price, the success or failure status, the reason for failure (if the message failed), the message dwell time, and other information.  <code>DeliveryStatusSuccessSamplingRate</code> – The percentage of successful SMS deliveries for which Amazon SNS will write logs in CloudWatch Logs. The value can be an integer from 0 - 100. For example, to write logs only for failed deliveries, set this value to <code>0</code>. To write logs for 10% of your successful deliveries, set it to <code>10</code>.  <code>DefaultSenderID</code> – A string, such as your business brand, that is displayed as the sender on the receiving device. Support for sender IDs varies by country. The sender ID can be 1 - 11 alphanumeric characters, and it must contain at least one letter.  <code>DefaultSMSType</code> – The type of SMS message that you will send by default. You can assign the following values: <ul> <li>  <code>Promotional</code> – (Default) Noncritical messages, such as marketing messages. Amazon SNS optimizes the message delivery to incur the lowest cost. </li> <li>  <code>Transactional</code> – Critical messages that support customer transactions, such as one-time passcodes for multi-factor authentication. Amazon SNS optimizes the message delivery to achieve the highest reliability. </li> </ul>  <code>UsageReportS3Bucket</code> – The name of the Amazon S3 bucket to receive daily SMS usage reports from Amazon SNS. Each day, Amazon SNS will deliver a usage report as a CSV file to the bucket. The report includes the following information for each SMS message that was successfully delivered by your account: <ul> <li> Time that the message was published (in UTC) </li> <li> Message ID </li> <li> Destination phone number </li> <li> Message type </li> <li> Delivery status </li> <li> Message price (in USD) </li> <li> Part number (a message is split into multiple parts if it is too long for a single message) </li> <li> Total number of parts </li> </ul> To receive the report, the bucket must have a policy that allows the Amazon SNS service principle to perform the <code>s3:PutObject</code> and <code>s3:GetBucketLocation</code> actions. For an example bucket policy and usage report, see <a href="https://docs.aws.amazon.com/sns/latest/dg/sms_stats.html">Monitoring SMS Activity</a> in the <i>Amazon SNS Developer Guide</i>.<br/>
        :return:         """
        pass

    def set_subscription_attributes(self, AttributeName: str, SubscriptionArn: str, AttributeValue: str=None):
        """Allows a subscription owner to set an attribute of the subscription to a new value.

        :param self:
        :param AttributeValue:  The new value for the attribute in JSON format.<br/>AttributeName:  A map of attributes with their corresponding values. The following lists the names, descriptions, and values of the special request parameters that the <code>SetTopicAttributes</code> action uses: <ul> <li>  <code>DeliveryPolicy</code> – The policy that defines how Amazon SNS retries failed deliveries to HTTP/S endpoints. </li> <li>  <code>FilterPolicy</code> – The simple JSON object that lets your subscriber receive only a subset of messages, rather than receiving every message published to the topic. </li> <li>  <code>RawMessageDelivery</code> – When set to <code>true</code>, enables raw message delivery to Amazon SQS or HTTP/S endpoints. This eliminates the need for the endpoints to process JSON formatting, which is otherwise created for Amazon SNS metadata. </li> </ul><br/>SubscriptionArn:  The ARN of the subscription to modify.<br/>
        :return:         """
        pass

    def set_topic_attributes(self, AttributeName: str, TopicArn: str, AttributeValue: str=None):
        """Allows a topic owner to set an attribute of the topic to a new value.

        :param self:
        :param AttributeValue:  The new value for the attribute.<br/>AttributeName:  A map of attributes with their corresponding values. The following lists the names, descriptions, and values of the special request parameters that the <code>SetTopicAttributes</code> action uses: <ul> <li>  <code>DeliveryPolicy</code> – The policy that defines how Amazon SNS retries failed deliveries to HTTP/S endpoints. </li> <li>  <code>DisplayName</code> – The display name to use for a topic with SMS subscriptions. </li> <li>  <code>Policy</code> – The policy that defines who can access your topic. By default, only the topic owner can publish or subscribe to the topic. </li> </ul> The following attribute applies only to <a href="https://docs.aws.amazon.com/sns/latest/dg/sns-server-side-encryption.html">server-side-encryption</a>: <ul> <li>  <code>KmsMasterKeyId</code> - The ID of an AWS-managed customer master key (CMK) for Amazon SNS or a custom CMK. For more information, see <a href="https://docs.aws.amazon.com/sns/latest/dg/sns-server-side-encryption.html#sse-key-terms">Key Terms</a>. For more examples, see <a href="https://docs.aws.amazon.com/kms/latest/APIReference/API_DescribeKey.html#API_DescribeKey_RequestParameters">KeyId</a> in the <i>AWS Key Management Service API Reference</i>.  </li> </ul><br/>TopicArn:  The ARN of the topic to modify.<br/>
        :return:         """
        pass

    def subscribe(self, Protocol: str, TopicArn: str, Endpoint: str=None, Attributes: SubscriptionAttributesMap=None, ReturnSubscriptionArn: boolean=None) -> SubscribeResponse:
        """Prepares to subscribe an endpoint by sending the endpoint a confirmation message. To actually create a subscription, the endpoint owner must call the <code>ConfirmSubscription</code> action with the token from the confirmation message. Confirmation tokens are valid for three days. This action is throttled at 100 transactions per second (TPS).

        :param self:
        :param ReturnSubscriptionArn:  Sets whether the response from the <code>Subscribe</code> request includes the subscription ARN, even if the subscription is not yet confirmed. If you set this parameter to <code>false</code>, the response includes the ARN for confirmed subscriptions, but it includes an ARN value of "pending subscription" for subscriptions that are not yet confirmed. A subscription becomes confirmed when the subscriber calls the <code>ConfirmSubscription</code> action with a confirmation token. If you set this parameter to <code>true</code>, the response includes the ARN in all cases, even if the subscription is not yet confirmed. The default value is <code>false</code>.<br/>Attributes:  A map of attributes with their corresponding values. The following lists the names, descriptions, and values of the special request parameters that the <code>SetTopicAttributes</code> action uses: <ul> <li>  <code>DeliveryPolicy</code> – The policy that defines how Amazon SNS retries failed deliveries to HTTP/S endpoints. </li> <li>  <code>FilterPolicy</code> – The simple JSON object that lets your subscriber receive only a subset of messages, rather than receiving every message published to the topic. </li> <li>  <code>RawMessageDelivery</code> – When set to <code>true</code>, enables raw message delivery to Amazon SQS or HTTP/S endpoints. This eliminates the need for the endpoints to process JSON formatting, which is otherwise created for Amazon SNS metadata. </li> </ul><br/>Endpoint:  The endpoint that you want to receive notifications. Endpoints vary by protocol: <ul> <li> For the <code>http</code> protocol, the endpoint is an URL beginning with "https://" </li> <li> For the <code>https</code> protocol, the endpoint is a URL beginning with "https://" </li> <li> For the <code>email</code> protocol, the endpoint is an email address </li> <li> For the <code>email-json</code> protocol, the endpoint is an email address </li> <li> For the <code>sms</code> protocol, the endpoint is a phone number of an SMS-enabled device </li> <li> For the <code>sqs</code> protocol, the endpoint is the ARN of an Amazon SQS queue </li> <li> For the <code>application</code> protocol, the endpoint is the EndpointArn of a mobile app and device. </li> <li> For the <code>lambda</code> protocol, the endpoint is the ARN of an AWS Lambda function. </li> </ul><br/>Protocol:  The protocol you want to use. Supported protocols include: <ul> <li>  <code>http</code> – delivery of JSON-encoded message via HTTP POST </li> <li>  <code>https</code> – delivery of JSON-encoded message via HTTPS POST </li> <li>  <code>email</code> – delivery of message via SMTP </li> <li>  <code>email-json</code> – delivery of JSON-encoded message via SMTP </li> <li>  <code>sms</code> – delivery of message via SMS </li> <li>  <code>sqs</code> – delivery of JSON-encoded message to an Amazon SQS queue </li> <li>  <code>application</code> – delivery of JSON-encoded message to an EndpointArn for a mobile app and device. </li> <li>  <code>lambda</code> – delivery of JSON-encoded message to an AWS Lambda function. </li> </ul><br/>TopicArn:  The ARN of the topic you want to subscribe to.<br/>
        :return: SubscriptionArn:  The ARN of the subscription if it is confirmed, or the string "pending confirmation" if the subscription requires confirmation. However, if the API request parameter <code>ReturnSubscriptionArn</code> is true, then the value is always the subscription ARN, even if the subscription requires confirmation.<br/>        """
        pass

    def tag_resource(self, Tags: List[Tag], ResourceArn: str) -> TagResourceResponse:
        """Add tags to the specified Amazon SNS topic. For an overview, see <a href="https://docs.aws.amazon.com/sns/latest/dg/sns-tags.html">Amazon SNS Tags</a> in the <i>Amazon SNS Developer Guide</i>. When you use topic tags, keep the following guidelines in mind: <ul> <li> Adding more than 50 tags to a topic isn't recommended. </li> <li> Tags don't have any semantic meaning. Amazon SNS interprets tags as character strings. </li> <li> Tags are case-sensitive. </li> <li> A new tag with a key identical to that of an existing tag overwrites the existing tag. </li> <li> Tagging actions are limited to 10 TPS per AWS account. If your application requires a higher throughput, file a <a href="https://console.aws.amazon.com/support/home#/case/create?issueType=technical">technical support request</a>. </li> </ul> For a full list of tag restrictions, see <a href="https://docs.aws.amazon.com/sns/latest/dg/sns-limits.html#limits-topics">Limits Related to Topics</a> in the <i>Amazon SNS Developer Guide</i>.

        :param self:
        :param Tags:  The tags to be added to the specified topic. A tag consists of a required key and an optional value.<br/>ResourceArn:  The ARN of the topic to which to add tags.<br/>
        :return:         """
        pass

    def unsubscribe(self, SubscriptionArn: str):
        """Deletes a subscription. If the subscription requires authentication for deletion, only the owner of the subscription or the topic's owner can unsubscribe, and an AWS signature is required. If the <code>Unsubscribe</code> call does not require authentication and the requester is not the subscription owner, a final cancellation message is delivered to the endpoint, so that the endpoint owner can easily resubscribe to the topic if the <code>Unsubscribe</code> request was unintended. This action is throttled at 100 transactions per second (TPS).

        :param self:
        :param SubscriptionArn:  The ARN of the subscription to be deleted.<br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[TagKey], ResourceArn: str) -> UntagResourceResponse:
        """Remove tags from the specified Amazon SNS topic. For an overview, see <a href="https://docs.aws.amazon.com/sns/latest/dg/sns-tags.html">Amazon SNS Tags</a> in the <i>Amazon SNS Developer Guide</i>.

        :param self:
        :param TagKeys:  The list of tag keys to remove from the specified topic.<br/>ResourceArn:  The ARN of the topic from which to remove tags.<br/>
        :return:         """
        pass

    class SnsResource:
        
        class PlatformApplication:
            """Creates a PlatformApplication resource.::

  platform_application = sns.PlatformApplication('arn')

:type arn: string
:param arn: The PlatformApplication's arn identifier. This **must** be set.

:rtype: :py:class:`SNS.PlatformApplication`
:returns: A PlatformApplication resource"""

            def __init__(self, arn):
                pass
            
            class MapStringToString(object):
                pass
        
            attributes: MapStringToString=None
            
            def create_platform_endpoint(self, **kwargs):
                # type: (object) -> PlatformEndpoint
                return PlatformEndpoint()

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def set_attributes(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class PlatformEndpoint:
            """Creates a PlatformEndpoint resource.::

  platform_endpoint = sns.PlatformEndpoint('arn')

:type arn: string
:param arn: The PlatformEndpoint's arn identifier. This **must** be set.

:rtype: :py:class:`SNS.PlatformEndpoint`
:returns: A PlatformEndpoint resource"""

            def __init__(self, arn):
                pass
            
            class MapStringToString(object):
                pass
        
            attributes: MapStringToString=None
            
            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def publish(self, **kwargs):
                # type: (object) -> dict
                return {}

            def set_attributes(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class Subscription:
            """Creates a Subscription resource.::

  subscription = sns.Subscription('arn')

:type arn: string
:param arn: The Subscription's arn identifier. This **must** be set.

:rtype: :py:class:`SNS.Subscription`
:returns: A Subscription resource"""

            def __init__(self, arn):
                pass
            
            class SubscriptionAttributesMap(object):
                pass
        
            attributes: SubscriptionAttributesMap=None
            
            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def set_attributes(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class Topic:
            """Creates a Topic resource.::

  topic = sns.Topic('arn')

:type arn: string
:param arn: The Topic's arn identifier. This **must** be set.

:rtype: :py:class:`SNS.Topic`
:returns: A Topic resource"""

            def __init__(self, arn):
                pass
            
            class TopicAttributesMap(object):
                pass
        
            attributes: TopicAttributesMap=None
            
            def add_permission(self, **kwargs):
                # type: (object) -> dict
                return {}

            def confirm_subscription(self, **kwargs):
                # type: (object) -> Subscription
                return Subscription()

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def publish(self, **kwargs):
                # type: (object) -> dict
                return {}

            def remove_permission(self, **kwargs):
                # type: (object) -> dict
                return {}

            def set_attributes(self, **kwargs):
                # type: (object) -> dict
                return {}

            def subscribe(self, **kwargs):
                # type: (object) -> Subscription
                return Subscription()


        def create_platform_application(self, **kwargs):
            # type: (object) -> PlatformApplication
            pass

        def create_topic(self, **kwargs):
            # type: (object) -> Topic
            pass

        def get_available_subresources(self) -> List[str]:
            pass

        class PlatformApplicationResourceCollection(List[PlatformApplication], ResourceCollection):
            pass

        platform_applications: PlatformApplicationResourceCollection = None
        class SubscriptionResourceCollection(List[Subscription], ResourceCollection):
            pass

        subscriptions: SubscriptionResourceCollection = None
        class TopicResourceCollection(List[Topic], ResourceCollection):
            pass

        topics: TopicResourceCollection = None




class SQS(BaseClient):
    """Welcome to the <i>Amazon Simple Queue Service API Reference</i>. Amazon Simple Queue Service (Amazon SQS) is a reliable, highly-scalable hosted queue for storing messages as they travel between applications or microservices. Amazon SQS moves data between distributed application components and helps you decouple these components. <note>  <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/standard-queues.html">Standard queues</a> are available in all regions. <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/FIFO-queues.html">FIFO queues</a> are available in the US East (N. Virginia), US East (Ohio), US West (Oregon), and EU (Ireland) regions. </note> You can use <a href="http://aws.amazon.com/tools/#sdk">AWS SDKs</a> to access Amazon SQS using your favorite programming language. The SDKs perform tasks such as the following automatically: <ul> <li> Cryptographically sign your service requests </li> <li> Retry requests </li> <li> Handle error responses </li> </ul>  <b>Additional Information</b>  <ul> <li>  <a href="http://aws.amazon.com/sqs/">Amazon SQS Product Page</a>  </li> <li>  <i>Amazon Simple Queue Service Developer Guide</i>  <ul> <li>  <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-making-api-requests.html">Making API Requests</a>  </li> <li>  <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-message-attributes.html">Amazon SQS Message Attributes</a>  </li> <li>  <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-dead-letter-queues.html">Amazon SQS Dead-Letter Queues</a>  </li> </ul> </li> <li>  <a href="http://docs.aws.amazon.com/cli/latest/reference/sqs/index.html">Amazon SQS in the <i>AWS CLI Command Reference</i> </a>  </li> <li>  <i>Amazon Web Services General Reference</i>  <ul> <li>  <a href="http://docs.aws.amazon.com/general/latest/gr/rande.html#sqs_region">Regions and Endpoints</a>  </li> </ul> </li> </ul>"""




    class AddPermissionRequest(Mapping):
        pass
    
    class String(object):
        pass
    
    class ChangeMessageVisibilityRequest(Mapping):
        pass
    
    class ChangeMessageVisibilityBatchRequest(Mapping):
        pass
    
    class ChangeMessageVisibilityBatchResult(Mapping):
        pass
    
    class ChangeMessageVisibilityBatchRequestEntry(Mapping):
        pass
    
    class CreateQueueRequest(Mapping):
        pass
    
    class CreateQueueResult(Mapping):
        pass
    
    class QueueAttributeMap(object):
        pass
    
    class DeleteMessageRequest(Mapping):
        pass
    
    class DeleteMessageBatchRequest(Mapping):
        pass
    
    class DeleteMessageBatchResult(Mapping):
        pass
    
    class DeleteMessageBatchRequestEntry(Mapping):
        pass
    
    class DeleteQueueRequest(Mapping):
        pass
    
    class GetQueueAttributesRequest(Mapping):
        pass
    
    class GetQueueAttributesResult(Mapping):
        pass
    
    class QueueAttributeName(object):
        pass
    
    class GetQueueUrlRequest(Mapping):
        pass
    
    class GetQueueUrlResult(Mapping):
        pass
    
    class ListDeadLetterSourceQueuesRequest(Mapping):
        pass
    
    class ListDeadLetterSourceQueuesResult(Mapping):
        pass
    
    class ListQueueTagsRequest(Mapping):
        pass
    
    class ListQueueTagsResult(Mapping):
        pass
    
    class ListQueuesRequest(Mapping):
        pass
    
    class ListQueuesResult(Mapping):
        pass
    
    class PurgeQueueRequest(Mapping):
        pass
    
    class ReceiveMessageRequest(Mapping):
        pass
    
    class ReceiveMessageResult(Mapping):
        pass
    
    class MessageAttributeName(object):
        pass
    
    class RemovePermissionRequest(Mapping):
        pass
    
    class SendMessageRequest(Mapping):
        pass
    
    class SendMessageResult(Mapping):
        pass
    
    class MessageBodyAttributeMap(object):
        pass
    
    class SendMessageBatchRequest(Mapping):
        pass
    
    class SendMessageBatchResult(Mapping):
        pass
    
    class SendMessageBatchRequestEntry(Mapping):
        pass
    
    class SetQueueAttributesRequest(Mapping):
        pass
    
    class TagQueueRequest(Mapping):
        pass
    
    class TagMap(object):
        pass
    
    class UntagQueueRequest(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    def add_permission(self, Actions: List[String], AWSAccountIds: List[String], Label: str, QueueUrl: str):
        """Adds a permission to a queue for a specific <a href="http://docs.aws.amazon.com/general/latest/gr/glos-chap.html#P">principal</a>. This allows sharing access to the queue. When you create a queue, you have full control access rights for the queue. Only you, the owner of the queue, can grant or deny permissions to the queue. For more information about these permissions, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-writing-an-sqs-policy.html#write-messages-to-shared-queue">Allow Developers to Write Messages to a Shared Queue</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. <note>  <code>AddPermission</code> writes an Amazon-SQS-generated policy. If you want to write your own policy, use <code> <a>SetQueueAttributes</a> </code> to upload your policy. For more information about writing your own policy, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-creating-custom-policies.html">Using Custom Policies with the Amazon SQS Access Policy Language</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. An Amazon SQS policy can have a maximum of 7 actions. </note> Some actions take lists of parameters. These lists are specified using the <code>param.n</code> notation. Values of <code>n</code> are integers starting from 1. For example, a parameter list with two elements looks like this:  <code>&amp;Attribute.1=first</code>   <code>&amp;Attribute.2=second</code>  <note> Cross-account permissions don't apply to this action. For more information, see see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-customer-managed-policy-examples.html#grant-cross-account-permissions-to-role-and-user-name">Grant Cross-Account Permissions to a Role and a User Name</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. </note>

        :param self:
        :param Actions:  The action the client wants to allow for the specified principal. Valid values: the name of any action or <code>*</code>. For more information about these actions, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-overview-of-managing-access.html">Overview of Managing Access Permissions to Your Amazon Simple Queue Service Resource</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. Specifying <code>SendMessage</code>, <code>DeleteMessage</code>, or <code>ChangeMessageVisibility</code> for <code>ActionName.n</code> also grants permissions for the corresponding batch versions of those actions: <code>SendMessageBatch</code>, <code>DeleteMessageBatch</code>, and <code>ChangeMessageVisibilityBatch</code>.<br/>AWSAccountIds:  The AWS account number of the <a href="http://docs.aws.amazon.com/general/latest/gr/glos-chap.html#P">principal</a> who is given permission. The principal must have an AWS account, but does not need to be signed up for Amazon SQS. For information about locating the AWS account identification, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-making-api-requests.html#sqs-api-request-authentication">Your AWS Identifiers</a> in the <i>Amazon Simple Queue Service Developer Guide</i>.<br/>Label:  The unique identification of the permission you're setting (for example, <code>AliceSendMessage</code>). Maximum 80 characters. Allowed characters include alphanumeric characters, hyphens (<code>-</code>), and underscores (<code>_</code>).<br/>QueueUrl:  The URL of the Amazon SQS queue to which permissions are added. Queue URLs and names are case-sensitive.<br/>
        :return:         """
        pass

    def change_message_visibility(self, VisibilityTimeout: int, ReceiptHandle: str, QueueUrl: str):
        """Changes the visibility timeout of a specified message in a queue to a new value. The maximum allowed timeout value is 12 hours. For more information, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-visibility-timeout.html">Visibility Timeout</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. For example, you have a message with a visibility timeout of 5 minutes. After 3 minutes, you call <code>ChangeMessageVisibility</code> with a timeout of 10 minutes. You can continue to call <code>ChangeMessageVisibility</code> to extend the visibility timeout to a maximum of 12 hours. If you try to extend the visibility timeout beyond 12 hours, your request is rejected. A message is considered to be <i>in flight</i> after it's received from a queue by a consumer, but not yet deleted from the queue. For standard queues, there can be a maximum of 120,000 inflight messages per queue. If you reach this limit, Amazon SQS returns the <code>OverLimit</code> error message. To avoid reaching the limit, you should delete messages from the queue after they're processed. You can also increase the number of queues you use to process your messages. For FIFO queues, there can be a maximum of 20,000 inflight messages per queue. If you reach this limit, Amazon SQS returns no error messages. <important> If you attempt to set the <code>VisibilityTimeout</code> to a value greater than the maximum time left, Amazon SQS returns an error. Amazon SQS doesn't automatically recalculate and increase the timeout to the maximum remaining time. Unlike with a queue, when you change the visibility timeout for a specific message the timeout value is applied immediately but isn't saved in memory for that message. If you don't delete a message after it is received, the visibility timeout for the message reverts to the original timeout value (not to the value you set using the <code>ChangeMessageVisibility</code> action) the next time the message is received. </important>

        :param self:
        :param VisibilityTimeout:  The new value for the message's visibility timeout (in seconds). Values values: <code>0</code> to <code>43200</code>. Maximum: 12 hours.<br/>ReceiptHandle:  The receipt handle associated with the message whose visibility timeout is changed. This parameter is returned by the <code> <a>ReceiveMessage</a> </code> action.<br/>QueueUrl:  The URL of the Amazon SQS queue whose message's visibility is changed. Queue URLs and names are case-sensitive.<br/>
        :return:         """
        pass

    def change_message_visibility_batch(self, Entries: List[ChangeMessageVisibilityBatchRequestEntry], QueueUrl: str) -> ChangeMessageVisibilityBatchResult:
        """Changes the visibility timeout of multiple messages. This is a batch version of <code> <a>ChangeMessageVisibility</a>.</code> The result of the action on each message is reported individually in the response. You can send up to 10 <code> <a>ChangeMessageVisibility</a> </code> requests with each <code>ChangeMessageVisibilityBatch</code> action. <important> Because the batch request can result in a combination of successful and unsuccessful actions, you should check for batch errors even when the call returns an HTTP status code of <code>200</code>. </important> Some actions take lists of parameters. These lists are specified using the <code>param.n</code> notation. Values of <code>n</code> are integers starting from 1. For example, a parameter list with two elements looks like this:  <code>&amp;Attribute.1=first</code>   <code>&amp;Attribute.2=second</code> 

        :param self:
        :param Entries:  A list of receipt handles of the messages for which the visibility timeout must be changed.<br/>QueueUrl:  The URL of the Amazon SQS queue whose messages' visibility is changed. Queue URLs and names are case-sensitive.<br/>
        :return: Failed:  A list of <code> <a>BatchResultErrorEntry</a> </code> items.<br/>Successful:  A list of <code> <a>ChangeMessageVisibilityBatchResultEntry</a> </code> items.<br/>        """
        pass

    def create_queue(self, QueueName: str, Attributes: QueueAttributeMap=None) -> CreateQueueResult:
        """Creates a new standard or FIFO queue. You can pass one or more attributes in the request. Keep the following caveats in mind: <ul> <li> If you don't specify the <code>FifoQueue</code> attribute, Amazon SQS creates a standard queue. <note>  You can't change the queue type after you create it and you can't convert an existing standard queue into a FIFO queue. You must either create a new FIFO queue for your application or delete your existing standard queue and recreate it as a FIFO queue. For more information, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/FIFO-queues.html#FIFO-queues-moving">Moving From a Standard Queue to a FIFO Queue</a> in the <i>Amazon Simple Queue Service Developer Guide</i>.  </note> </li> <li> If you don't provide a value for an attribute, the queue is created with the default value for the attribute. </li> <li> If you delete a queue, you must wait at least 60 seconds before creating a queue with the same name. </li> </ul> To successfully create a new queue, you must provide a queue name that adheres to the <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/limits-queues.html">limits related to queues</a> and is unique within the scope of your queues. To get the queue URL, use the <code> <a>GetQueueUrl</a> </code> action. <code> <a>GetQueueUrl</a> </code> requires only the <code>QueueName</code> parameter. be aware of existing queue names: <ul> <li> If you provide the name of an existing queue along with the exact names and values of all the queue's attributes, <code>CreateQueue</code> returns the queue URL for the existing queue. </li> <li> If the queue name, attribute names, or attribute values don't match an existing queue, <code>CreateQueue</code> returns an error. </li> </ul> Some actions take lists of parameters. These lists are specified using the <code>param.n</code> notation. Values of <code>n</code> are integers starting from 1. For example, a parameter list with two elements looks like this:  <code>&amp;Attribute.1=first</code>   <code>&amp;Attribute.2=second</code>  <note> Cross-account permissions don't apply to this action. For more information, see see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-customer-managed-policy-examples.html#grant-cross-account-permissions-to-role-and-user-name">Grant Cross-Account Permissions to a Role and a User Name</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. </note>

        :param self:
        :param Attributes:  A map of attributes with their corresponding values. The following lists the names, descriptions, and values of the special request parameters that the <code>CreateQueue</code> action uses: <ul> <li>  <code>DelaySeconds</code> - The length of time, in seconds, for which the delivery of all messages in the queue is delayed. Valid values: An integer from 0 to 900 seconds (15 minutes). Default: 0.  </li> <li>  <code>MaximumMessageSize</code> - The limit of how many bytes a message can contain before Amazon SQS rejects it. Valid values: An integer from 1,024 bytes (1 KiB) to 262,144 bytes (256 KiB). Default: 262,144 (256 KiB).  </li> <li>  <code>MessageRetentionPeriod</code> - The length of time, in seconds, for which Amazon SQS retains a message. Valid values: An integer from 60 seconds (1 minute) to 1,209,600 seconds (14 days). Default: 345,600 (4 days).  </li> <li>  <code>Policy</code> - The queue's policy. A valid AWS policy. For more information about policy structure, see <a href="http://docs.aws.amazon.com/IAM/latest/UserGuide/PoliciesOverview.html">Overview of AWS IAM Policies</a> in the <i>Amazon IAM User Guide</i>.  </li> <li>  <code>ReceiveMessageWaitTimeSeconds</code> - The length of time, in seconds, for which a <code> <a>ReceiveMessage</a> </code> action waits for a message to arrive. Valid values: An integer from 0 to 20 (seconds). Default: 0.  </li> <li>  <code>RedrivePolicy</code> - The string that includes the parameters for the dead-letter queue functionality of the source queue. For more information about the redrive policy and dead-letter queues, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-dead-letter-queues.html">Using Amazon SQS Dead-Letter Queues</a> in the <i>Amazon Simple Queue Service Developer Guide</i>.  <ul> <li>  <code>deadLetterTargetArn</code> - The Amazon Resource Name (ARN) of the dead-letter queue to which Amazon SQS moves messages after the value of <code>maxReceiveCount</code> is exceeded. </li> <li>  <code>maxReceiveCount</code> - The number of times a message is delivered to the source queue before being moved to the dead-letter queue. When the <code>ReceiveCount</code> for a message exceeds the <code>maxReceiveCount</code> for a queue, Amazon SQS moves the message to the dead-letter-queue. </li> </ul> <note> The dead-letter queue of a FIFO queue must also be a FIFO queue. Similarly, the dead-letter queue of a standard queue must also be a standard queue. </note> </li> <li>  <code>VisibilityTimeout</code> - The visibility timeout for the queue, in seconds. Valid values: An integer from 0 to 43,200 (12 hours). Default: 30. For more information about the visibility timeout, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-visibility-timeout.html">Visibility Timeout</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. </li> </ul> The following attributes apply only to <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-server-side-encryption.html">server-side-encryption</a>: <ul> <li>  <code>KmsMasterKeyId</code> - The ID of an AWS-managed customer master key (CMK) for Amazon SQS or a custom CMK. For more information, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-server-side-encryption.html#sqs-sse-key-terms">Key Terms</a>. While the alias of the AWS-managed CMK for Amazon SQS is always <code>alias/aws/sqs</code>, the alias of a custom CMK can, for example, be <code>alias/<i>MyAlias</i> </code>. For more examples, see <a href="http://docs.aws.amazon.com/kms/latest/APIReference/API_DescribeKey.html#API_DescribeKey_RequestParameters">KeyId</a> in the <i>AWS Key Management Service API Reference</i>.  </li> <li>  <code>KmsDataKeyReusePeriodSeconds</code> - The length of time, in seconds, for which Amazon SQS can reuse a <a href="http://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#data-keys">data key</a> to encrypt or decrypt messages before calling AWS KMS again. An integer representing seconds, between 60 seconds (1 minute) and 86,400 seconds (24 hours). Default: 300 (5 minutes). A shorter time period provides better security but results in more calls to KMS which might incur charges after Free Tier. For more information, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-server-side-encryption.html#sqs-how-does-the-data-key-reuse-period-work">How Does the Data Key Reuse Period Work?</a>.  </li> </ul> The following attributes apply only to <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/FIFO-queues.html">FIFO (first-in-first-out) queues</a>: <ul> <li>  <code>FifoQueue</code> - Designates a queue as FIFO. Valid values: <code>true</code>, <code>false</code>. You can provide this attribute only during queue creation. You can't change it for an existing queue. When you set this attribute, you must also provide the <code>MessageGroupId</code> for your messages explicitly. For more information, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/FIFO-queues.html#FIFO-queues-understanding-logic">FIFO Queue Logic</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. </li> <li>  <code>ContentBasedDeduplication</code> - Enables content-based deduplication. Valid values: <code>true</code>, <code>false</code>. For more information, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/FIFO-queues.html#FIFO-queues-exactly-once-processing">Exactly-Once Processing</a> in the <i>Amazon Simple Queue Service Developer Guide</i>.  <ul> <li> Every message must have a unique <code>MessageDeduplicationId</code>, <ul> <li> You may provide a <code>MessageDeduplicationId</code> explicitly. </li> <li> If you aren't able to provide a <code>MessageDeduplicationId</code> and you enable <code>ContentBasedDeduplication</code> for your queue, Amazon SQS uses a SHA-256 hash to generate the <code>MessageDeduplicationId</code> using the body of the message (but not the attributes of the message).  </li> <li> If you don't provide a <code>MessageDeduplicationId</code> and the queue doesn't have <code>ContentBasedDeduplication</code> set, the action fails with an error. </li> <li> If the queue has <code>ContentBasedDeduplication</code> set, your <code>MessageDeduplicationId</code> overrides the generated one. </li> </ul> </li> <li> When <code>ContentBasedDeduplication</code> is in effect, messages with identical content sent within the deduplication interval are treated as duplicates and only one copy of the message is delivered. </li> <li> If you send one message with <code>ContentBasedDeduplication</code> enabled and then another message with a <code>MessageDeduplicationId</code> that is the same as the one generated for the first <code>MessageDeduplicationId</code>, the two messages are treated as duplicates and only one copy of the message is delivered.  </li> </ul> </li> </ul><br/>QueueName:  The name of the new queue. The following limits apply to this name: <ul> <li> A queue name can have up to 80 characters. </li> <li> Valid values: alphanumeric characters, hyphens (<code>-</code>), and underscores (<code>_</code>). </li> <li> A FIFO queue name must end with the <code>.fifo</code> suffix. </li> </ul> Queue URLs and names are case-sensitive.<br/>
        :return: QueueUrl:  The URL of the created Amazon SQS queue.<br/>        """
        pass

    def delete_message(self, ReceiptHandle: str, QueueUrl: str):
        """Deletes the specified message from the specified queue. To select the message to delete, use the <code>ReceiptHandle</code> of the message (<i>not</i> the <code>MessageId</code> which you receive when you send the message). Amazon SQS can delete a message from a queue even if a visibility timeout setting causes the message to be locked by another consumer. Amazon SQS automatically deletes messages left in a queue longer than the retention period configured for the queue.  <note> The <code>ReceiptHandle</code> is associated with a <i>specific instance</i> of receiving a message. If you receive a message more than once, the <code>ReceiptHandle</code> is different each time you receive a message. When you use the <code>DeleteMessage</code> action, you must provide the most recently received <code>ReceiptHandle</code> for the message (otherwise, the request succeeds, but the message might not be deleted). For standard queues, it is possible to receive a message even after you delete it. This might happen on rare occasions if one of the servers which stores a copy of the message is unavailable when you send the request to delete the message. The copy remains on the server and might be returned to you during a subsequent receive request. You should ensure that your application is idempotent, so that receiving a message more than once does not cause issues. </note>

        :param self:
        :param ReceiptHandle:  The receipt handle associated with the message to delete.<br/>QueueUrl:  The URL of the Amazon SQS queue from which messages are deleted. Queue URLs and names are case-sensitive.<br/>
        :return:         """
        pass

    def delete_message_batch(self, Entries: List[DeleteMessageBatchRequestEntry], QueueUrl: str) -> DeleteMessageBatchResult:
        """Deletes up to ten messages from the specified queue. This is a batch version of <code> <a>DeleteMessage</a>.</code> The result of the action on each message is reported individually in the response. <important> Because the batch request can result in a combination of successful and unsuccessful actions, you should check for batch errors even when the call returns an HTTP status code of <code>200</code>. </important> Some actions take lists of parameters. These lists are specified using the <code>param.n</code> notation. Values of <code>n</code> are integers starting from 1. For example, a parameter list with two elements looks like this:  <code>&amp;Attribute.1=first</code>   <code>&amp;Attribute.2=second</code> 

        :param self:
        :param Entries:  A list of receipt handles for the messages to be deleted.<br/>QueueUrl:  The URL of the Amazon SQS queue from which messages are deleted. Queue URLs and names are case-sensitive.<br/>
        :return: Failed:  A list of <code> <a>BatchResultErrorEntry</a> </code> items.<br/>Successful:  A list of <code> <a>DeleteMessageBatchResultEntry</a> </code> items.<br/>        """
        pass

    def delete_queue(self, QueueUrl: str):
        """Deletes the queue specified by the <code>QueueUrl</code>, regardless of the queue's contents. If the specified queue doesn't exist, Amazon SQS returns a successful response. <important> Be careful with the <code>DeleteQueue</code> action: When you delete a queue, any messages in the queue are no longer available.  </important> When you delete a queue, the deletion process takes up to 60 seconds. Requests you send involving that queue during the 60 seconds might succeed. For example, a <code> <a>SendMessage</a> </code> request might succeed, but after 60 seconds the queue and the message you sent no longer exist. When you delete a queue, you must wait at least 60 seconds before creating a queue with the same name. <note> Cross-account permissions don't apply to this action. For more information, see see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-customer-managed-policy-examples.html#grant-cross-account-permissions-to-role-and-user-name">Grant Cross-Account Permissions to a Role and a User Name</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. </note>

        :param self:
        :param QueueUrl:  The URL of the Amazon SQS queue to delete. Queue URLs and names are case-sensitive.<br/>
        :return:         """
        pass

    def get_queue_attributes(self, QueueUrl: str, AttributeNames: List[QueueAttributeName]=None) -> GetQueueAttributesResult:
        """Gets attributes for the specified queue. <note> To determine whether a queue is <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/FIFO-queues.html">FIFO</a>, you can check whether <code>QueueName</code> ends with the <code>.fifo</code> suffix. </note> Some actions take lists of parameters. These lists are specified using the <code>param.n</code> notation. Values of <code>n</code> are integers starting from 1. For example, a parameter list with two elements looks like this:  <code>&amp;Attribute.1=first</code>   <code>&amp;Attribute.2=second</code> 

        :param self:
        :param AttributeNames:  A list of attributes for which to retrieve information. <note> In the future, new attributes might be added. If you write code that calls this action, we recommend that you structure your code so that it can handle new attributes gracefully. </note> The following attributes are supported: <ul> <li>  <code>All</code> - Returns all values.  </li> <li>  <code>ApproximateNumberOfMessages</code> - Returns the approximate number of messages available for retrieval from the queue. </li> <li>  <code>ApproximateNumberOfMessagesDelayed</code> - Returns the approximate number of messages in the queue that are delayed and not available for reading immediately. This can happen when the queue is configured as a delay queue or when a message has been sent with a delay parameter. </li> <li>  <code>ApproximateNumberOfMessagesNotVisible</code> - Returns the approximate number of messages that are in flight. Messages are considered to be <i>in flight</i> if they have been sent to a client but have not yet been deleted or have not yet reached the end of their visibility window.  </li> <li>  <code>CreatedTimestamp</code> - Returns the time when the queue was created in seconds (<a href="http://en.wikipedia.org/wiki/Unix_time">epoch time</a>). </li> <li>  <code>DelaySeconds</code> - Returns the default delay on the queue in seconds. </li> <li>  <code>LastModifiedTimestamp</code> - Returns the time when the queue was last changed in seconds (<a href="http://en.wikipedia.org/wiki/Unix_time">epoch time</a>). </li> <li>  <code>MaximumMessageSize</code> - Returns the limit of how many bytes a message can contain before Amazon SQS rejects it. </li> <li>  <code>MessageRetentionPeriod</code> - Returns the length of time, in seconds, for which Amazon SQS retains a message. </li> <li>  <code>Policy</code> - Returns the policy of the queue. </li> <li>  <code>QueueArn</code> - Returns the Amazon resource name (ARN) of the queue. </li> <li>  <code>ReceiveMessageWaitTimeSeconds</code> - Returns the length of time, in seconds, for which the <code>ReceiveMessage</code> action waits for a message to arrive.  </li> <li>  <code>RedrivePolicy</code> - Returns the string that includes the parameters for dead-letter queue functionality of the source queue. For more information about the redrive policy and dead-letter queues, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-dead-letter-queues.html">Using Amazon SQS Dead-Letter Queues</a> in the <i>Amazon Simple Queue Service Developer Guide</i>.  <ul> <li>  <code>deadLetterTargetArn</code> - The Amazon Resource Name (ARN) of the dead-letter queue to which Amazon SQS moves messages after the value of <code>maxReceiveCount</code> is exceeded. </li> <li>  <code>maxReceiveCount</code> - The number of times a message is delivered to the source queue before being moved to the dead-letter queue. When the <code>ReceiveCount</code> for a message exceeds the <code>maxReceiveCount</code> for a queue, Amazon SQS moves the message to the dead-letter-queue. </li> </ul> </li> <li>  <code>VisibilityTimeout</code> - Returns the visibility timeout for the queue. For more information about the visibility timeout, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-visibility-timeout.html">Visibility Timeout</a> in the <i>Amazon Simple Queue Service Developer Guide</i>.  </li> </ul> The following attributes apply only to <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-server-side-encryption.html">server-side-encryption</a>: <ul> <li>  <code>KmsMasterKeyId</code> - Returns the ID of an AWS-managed customer master key (CMK) for Amazon SQS or a custom CMK. For more information, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-server-side-encryption.html#sqs-sse-key-terms">Key Terms</a>.  </li> <li>  <code>KmsDataKeyReusePeriodSeconds</code> - Returns the length of time, in seconds, for which Amazon SQS can reuse a data key to encrypt or decrypt messages before calling AWS KMS again. For more information, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-server-side-encryption.html#sqs-how-does-the-data-key-reuse-period-work">How Does the Data Key Reuse Period Work?</a>.  </li> </ul> The following attributes apply only to <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/FIFO-queues.html">FIFO (first-in-first-out) queues</a>: <ul> <li>  <code>FifoQueue</code> - Returns whether the queue is FIFO. For more information, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/FIFO-queues.html#FIFO-queues-understanding-logic">FIFO Queue Logic</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. <note> To determine whether a queue is <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/FIFO-queues.html">FIFO</a>, you can check whether <code>QueueName</code> ends with the <code>.fifo</code> suffix. </note> </li> <li>  <code>ContentBasedDeduplication</code> - Returns whether content-based deduplication is enabled for the queue. For more information, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/FIFO-queues.html#FIFO-queues-exactly-once-processing">Exactly-Once Processing</a> in the <i>Amazon Simple Queue Service Developer Guide</i>.  </li> </ul><br/>QueueUrl:  The URL of the Amazon SQS queue whose attribute information is retrieved. Queue URLs and names are case-sensitive.<br/>
        :return: Attributes:  A map of attributes to their respective values.<br/>        """
        pass

    def get_queue_url(self, QueueName: str, QueueOwnerAWSAccountId: str=None) -> GetQueueUrlResult:
        """Returns the URL of an existing Amazon SQS queue. To access a queue that belongs to another AWS account, use the <code>QueueOwnerAWSAccountId</code> parameter to specify the account ID of the queue's owner. The queue's owner must grant you permission to access the queue. For more information about shared queue access, see <code> <a>AddPermission</a> </code> or see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-writing-an-sqs-policy.html#write-messages-to-shared-queue">Allow Developers to Write Messages to a Shared Queue</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. 

        :param self:
        :param QueueOwnerAWSAccountId:  The AWS account ID of the account that created the queue.<br/>QueueName:  The name of the queue whose URL must be fetched. Maximum 80 characters. Valid values: alphanumeric characters, hyphens (<code>-</code>), and underscores (<code>_</code>). Queue URLs and names are case-sensitive.<br/>
        :return: QueueUrl:  The URL of the queue.<br/>        """
        pass

    def list_dead_letter_source_queues(self, QueueUrl: str) -> ListDeadLetterSourceQueuesResult:
        """Returns a list of your queues that have the <code>RedrivePolicy</code> queue attribute configured with a dead-letter queue. For more information about using dead-letter queues, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-dead-letter-queues.html">Using Amazon SQS Dead-Letter Queues</a> in the <i>Amazon Simple Queue Service Developer Guide</i>.

        :param self:
        :param QueueUrl:  The URL of a dead-letter queue. Queue URLs and names are case-sensitive.<br/>
        :return: queueUrls:  A list of source queue URLs that have the <code>RedrivePolicy</code> queue attribute configured with a dead-letter queue.<br/>        """
        pass

    def list_queue_tags(self, QueueUrl: str) -> ListQueueTagsResult:
        """List all cost allocation tags added to the specified Amazon SQS queue. For an overview, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-queue-tags.html">Tagging Your Amazon SQS Queues</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. When you use queue tags, keep the following guidelines in mind: <ul> <li> Adding more than 50 tags to a queue isn't recommended. </li> <li> Tags don't have any semantic meaning. Amazon SQS interprets tags as character strings. </li> <li> Tags are case-sensitive. </li> <li> A new tag with a key identical to that of an existing tag overwrites the existing tag. </li> <li> Tagging actions are limited to 5 TPS per AWS account. If your application requires a higher throughput, file a <a href="https://console.aws.amazon.com/support/home#/case/create?issueType=technical">technical support request</a>. </li> </ul> For a full list of tag restrictions, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-limits.html#limits-queues">Limits Related to Queues</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. <note> Cross-account permissions don't apply to this action. For more information, see see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-customer-managed-policy-examples.html#grant-cross-account-permissions-to-role-and-user-name">Grant Cross-Account Permissions to a Role and a User Name</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. </note>

        :param self:
        :param QueueUrl:  The URL of the queue.<br/>
        :return: Tags:  The list of all tags added to the specified queue.<br/>        """
        pass

    def list_queues(self, QueueNamePrefix: str=None) -> ListQueuesResult:
        """Returns a list of your queues. The maximum number of queues that can be returned is 1,000. If you specify a value for the optional <code>QueueNamePrefix</code> parameter, only queues with a name that begins with the specified value are returned. <note> Cross-account permissions don't apply to this action. For more information, see see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-customer-managed-policy-examples.html#grant-cross-account-permissions-to-role-and-user-name">Grant Cross-Account Permissions to a Role and a User Name</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. </note>

        :param self:
        :param QueueNamePrefix:  A string to use for filtering the list results. Only those queues whose name begins with the specified string are returned. Queue URLs and names are case-sensitive.<br/>
        :return: QueueUrls:  A list of queue URLs, up to 1,000 entries.<br/>        """
        pass

    def purge_queue(self, QueueUrl: str):
        """Deletes the messages in a queue specified by the <code>QueueURL</code> parameter. <important> When you use the <code>PurgeQueue</code> action, you can't retrieve any messages deleted from a queue. The message deletion process takes up to 60 seconds. We recommend waiting for 60 seconds regardless of your queue's size.  </important> Messages sent to the queue <i>before</i> you call <code>PurgeQueue</code> might be received but are deleted within the next minute. Messages sent to the queue <i>after</i> you call <code>PurgeQueue</code> might be deleted while the queue is being purged.

        :param self:
        :param QueueUrl:  The URL of the queue from which the <code>PurgeQueue</code> action deletes messages. Queue URLs and names are case-sensitive.<br/>
        :return:         """
        pass

    def receive_message(self, QueueUrl: str, AttributeNames: List[QueueAttributeName]=None, MessageAttributeNames: List[MessageAttributeName]=None, MaxNumberOfMessages: int=None, VisibilityTimeout: int=None, WaitTimeSeconds: int=None, ReceiveRequestAttemptId: str=None) -> ReceiveMessageResult:
        """Retrieves one or more messages (up to 10), from the specified queue. Using the <code>WaitTimeSeconds</code> parameter enables long-poll support. For more information, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-long-polling.html">Amazon SQS Long Polling</a> in the <i>Amazon Simple Queue Service Developer Guide</i>.  Short poll is the default behavior where a weighted random set of machines is sampled on a <code>ReceiveMessage</code> call. Thus, only the messages on the sampled machines are returned. If the number of messages in the queue is small (fewer than 1,000), you most likely get fewer messages than you requested per <code>ReceiveMessage</code> call. If the number of messages in the queue is extremely small, you might not receive any messages in a particular <code>ReceiveMessage</code> response. If this happens, repeat the request.  For each message returned, the response includes the following: <ul> <li> The message body. </li> <li> An MD5 digest of the message body. For information about MD5, see <a href="https://www.ietf.org/rfc/rfc1321.txt">RFC1321</a>. </li> <li> The <code>MessageId</code> you received when you sent the message to the queue. </li> <li> The receipt handle. </li> <li> The message attributes. </li> <li> An MD5 digest of the message attributes. </li> </ul> The receipt handle is the identifier you must provide when deleting the message. For more information, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-queue-message-identifiers.html">Queue and Message Identifiers</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. You can provide the <code>VisibilityTimeout</code> parameter in your request. The parameter is applied to the messages that Amazon SQS returns in the response. If you don't include the parameter, the overall visibility timeout for the queue is used for the returned messages. For more information, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-visibility-timeout.html">Visibility Timeout</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. A message that isn't deleted or a message whose visibility isn't extended before the visibility timeout expires counts as a failed receive. Depending on the configuration of the queue, the message might be sent to the dead-letter queue. <note> In the future, new attributes might be added. If you write code that calls this action, we recommend that you structure your code so that it can handle new attributes gracefully. </note>

        :param self:
        :param ReceiveRequestAttemptId:  This parameter applies only to FIFO (first-in-first-out) queues. The token used for deduplication of <code>ReceiveMessage</code> calls. If a networking issue occurs after a <code>ReceiveMessage</code> action, and instead of a response you receive a generic error, you can retry the same action with an identical <code>ReceiveRequestAttemptId</code> to retrieve the same set of messages, even if their visibility timeout has not yet expired. <ul> <li> You can use <code>ReceiveRequestAttemptId</code> only for 5 minutes after a <code>ReceiveMessage</code> action. </li> <li> When you set <code>FifoQueue</code>, a caller of the <code>ReceiveMessage</code> action can provide a <code>ReceiveRequestAttemptId</code> explicitly. </li> <li> If a caller of the <code>ReceiveMessage</code> action doesn't provide a <code>ReceiveRequestAttemptId</code>, Amazon SQS generates a <code>ReceiveRequestAttemptId</code>. </li> <li> You can retry the <code>ReceiveMessage</code> action with the same <code>ReceiveRequestAttemptId</code> if none of the messages have been modified (deleted or had their visibility changes). </li> <li> During a visibility timeout, subsequent calls with the same <code>ReceiveRequestAttemptId</code> return the same messages and receipt handles. If a retry occurs within the deduplication interval, it resets the visibility timeout. For more information, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-visibility-timeout.html">Visibility Timeout</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. <important> If a caller of the <code>ReceiveMessage</code> action still processes messages when the visibility timeout expires and messages become visible, another worker consuming from the same queue can receive the same messages and therefore process duplicates. Also, if a consumer whose message processing time is longer than the visibility timeout tries to delete the processed messages, the action fails with an error. To mitigate this effect, ensure that your application observes a safe threshold before the visibility timeout expires and extend the visibility timeout as necessary. </important> </li> <li> While messages with a particular <code>MessageGroupId</code> are invisible, no more messages belonging to the same <code>MessageGroupId</code> are returned until the visibility timeout expires. You can still receive messages with another <code>MessageGroupId</code> as long as it is also visible. </li> <li> If a caller of <code>ReceiveMessage</code> can't track the <code>ReceiveRequestAttemptId</code>, no retries work until the original visibility timeout expires. As a result, delays might occur but the messages in the queue remain in a strict order. </li> </ul> The length of <code>ReceiveRequestAttemptId</code> is 128 characters. <code>ReceiveRequestAttemptId</code> can contain alphanumeric characters (<code>a-z</code>, <code>A-Z</code>, <code>0-9</code>) and punctuation (<code>!"#$%&amp;'()*+,-./:;&lt;=&gt;?@[\]^_`{|}~</code>). For best practices of using <code>ReceiveRequestAttemptId</code>, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/using-receiverequestattemptid-request-parameter.html">Using the ReceiveRequestAttemptId Request Parameter</a> in the <i>Amazon Simple Queue Service Developer Guide</i>.<br/>WaitTimeSeconds:  The duration (in seconds) for which the call waits for a message to arrive in the queue before returning. If a message is available, the call returns sooner than <code>WaitTimeSeconds</code>. If no messages are available and the wait time expires, the call returns successfully with an empty list of messages.<br/>VisibilityTimeout:  The duration (in seconds) that the received messages are hidden from subsequent retrieve requests after being retrieved by a <code>ReceiveMessage</code> request.<br/>MaxNumberOfMessages:  The maximum number of messages to return. Amazon SQS never returns more messages than this value (however, fewer messages might be returned). Valid values: 1 to 10. Default: 1.<br/>MessageAttributeNames:  The name of the message attribute, where <i>N</i> is the index. <ul> <li> The name can contain alphanumeric characters and the underscore (<code>_</code>), hyphen (<code>-</code>), and period (<code>.</code>). </li> <li> The name is case-sensitive and must be unique among all attribute names for the message. </li> <li> The name must not start with AWS-reserved prefixes such as <code>AWS.</code> or <code>Amazon.</code> (or any casing variants). </li> <li> The name must not start or end with a period (<code>.</code>), and it should not have periods in succession (<code>..</code>). </li> <li> The name can be up to 256 characters long. </li> </ul> When using <code>ReceiveMessage</code>, you can send a list of attribute names to receive, or you can return all of the attributes by specifying <code>All</code> or <code>.*</code> in your request. You can also use all message attributes starting with a prefix, for example <code>bar.*</code>.<br/>AttributeNames:  A list of s that need to be returned along with each message. These attributes include: <ul> <li>  <code>All</code> - Returns all values. </li> <li>  <code>ApproximateFirstReceiveTimestamp</code> - Returns the time the message was first received from the queue (<a href="http://en.wikipedia.org/wiki/Unix_time">epoch time</a> in milliseconds). </li> <li>  <code>ApproximateReceiveCount</code> - Returns the number of times a message has been received from the queue but not deleted. </li> <li>  <code>SenderId</code>  <ul> <li> For an IAM user, returns the IAM user ID, for example <code>ABCDEFGHI1JKLMNOPQ23R</code>. </li> <li> For an IAM role, returns the IAM role ID, for example <code>ABCDE1F2GH3I4JK5LMNOP:i-a123b456</code>. </li> </ul> </li> <li>  <code>SentTimestamp</code> - Returns the time the message was sent to the queue (<a href="http://en.wikipedia.org/wiki/Unix_time">epoch time</a> in milliseconds). </li> <li>  <code>MessageDeduplicationId</code> - Returns the value provided by the producer that calls the <code> <a>SendMessage</a> </code> action. </li> <li>  <code>MessageGroupId</code> - Returns the value provided by the producer that calls the <code> <a>SendMessage</a> </code> action. Messages with the same <code>MessageGroupId</code> are returned in sequence. </li> <li>  <code>SequenceNumber</code> - Returns the value provided by Amazon SQS. </li> </ul><br/>QueueUrl:  The URL of the Amazon SQS queue from which messages are received. Queue URLs and names are case-sensitive.<br/>
        :return: Messages:  A list of messages.<br/>        """
        pass

    def remove_permission(self, Label: str, QueueUrl: str):
        """Revokes any permissions in the queue policy that matches the specified <code>Label</code> parameter. <note> Only the owner of a queue can remove permissions from it. Cross-account permissions don't apply to this action. For more information, see see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-customer-managed-policy-examples.html#grant-cross-account-permissions-to-role-and-user-name">Grant Cross-Account Permissions to a Role and a User Name</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. </note>

        :param self:
        :param Label:  The identification of the permission to remove. This is the label added using the <code> <a>AddPermission</a> </code> action.<br/>QueueUrl:  The URL of the Amazon SQS queue from which permissions are removed. Queue URLs and names are case-sensitive.<br/>
        :return:         """
        pass

    def send_message(self, MessageBody: str, QueueUrl: str, DelaySeconds: int=None, MessageAttributes: MessageBodyAttributeMap=None, MessageDeduplicationId: str=None, MessageGroupId: str=None) -> SendMessageResult:
        """Delivers a message to the specified queue. <important> A message can include only XML, JSON, and unformatted text. The following Unicode characters are allowed:  <code>#x9</code> | <code>#xA</code> | <code>#xD</code> | <code>#x20</code> to <code>#xD7FF</code> | <code>#xE000</code> to <code>#xFFFD</code> | <code>#x10000</code> to <code>#x10FFFF</code>  Any characters not included in this list will be rejected. For more information, see the <a href="http://www.w3.org/TR/REC-xml/#charsets">W3C specification for characters</a>. </important>

        :param self:
        :param MessageGroupId:  This parameter applies only to FIFO (first-in-first-out) queues. The tag that specifies that a message belongs to a specific message group. Messages that belong to the same message group are processed in a FIFO manner (however, messages in different message groups might be processed out of order). To interleave multiple ordered streams within a single queue, use <code>MessageGroupId</code> values (for example, session data for multiple users). In this scenario, multiple consumers can process the queue, but the session data of each user is processed in a FIFO fashion. <ul> <li> You must associate a non-empty <code>MessageGroupId</code> with a message. If you don't provide a <code>MessageGroupId</code>, the action fails. </li> <li>  <code>ReceiveMessage</code> might return messages with multiple <code>MessageGroupId</code> values. For each <code>MessageGroupId</code>, the messages are sorted by time sent. The caller can't specify a <code>MessageGroupId</code>. </li> </ul> The length of <code>MessageGroupId</code> is 128 characters. Valid values: alphanumeric characters and punctuation <code>(!"#$%&amp;'()*+,-./:;&lt;=&gt;?@[\]^_`{|}~)</code>. For best practices of using <code>MessageGroupId</code>, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/using-messagegroupid-property.html">Using the MessageGroupId Property</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. <important>  <code>MessageGroupId</code> is required for FIFO queues. You can't use it for Standard queues. </important><br/>MessageDeduplicationId:  This parameter applies only to FIFO (first-in-first-out) queues. The token used for deduplication of sent messages. If a message with a particular <code>MessageDeduplicationId</code> is sent successfully, any messages sent with the same <code>MessageDeduplicationId</code> are accepted successfully but aren't delivered during the 5-minute deduplication interval. For more information, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/FIFO-queues.html#FIFO-queues-exactly-once-processing"> Exactly-Once Processing</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. <ul> <li> Every message must have a unique <code>MessageDeduplicationId</code>, <ul> <li> You may provide a <code>MessageDeduplicationId</code> explicitly. </li> <li> If you aren't able to provide a <code>MessageDeduplicationId</code> and you enable <code>ContentBasedDeduplication</code> for your queue, Amazon SQS uses a SHA-256 hash to generate the <code>MessageDeduplicationId</code> using the body of the message (but not the attributes of the message).  </li> <li> If you don't provide a <code>MessageDeduplicationId</code> and the queue doesn't have <code>ContentBasedDeduplication</code> set, the action fails with an error. </li> <li> If the queue has <code>ContentBasedDeduplication</code> set, your <code>MessageDeduplicationId</code> overrides the generated one. </li> </ul> </li> <li> When <code>ContentBasedDeduplication</code> is in effect, messages with identical content sent within the deduplication interval are treated as duplicates and only one copy of the message is delivered. </li> <li> If you send one message with <code>ContentBasedDeduplication</code> enabled and then another message with a <code>MessageDeduplicationId</code> that is the same as the one generated for the first <code>MessageDeduplicationId</code>, the two messages are treated as duplicates and only one copy of the message is delivered.  </li> </ul> <note> The <code>MessageDeduplicationId</code> is available to the consumer of the message (this can be useful for troubleshooting delivery issues). If a message is sent successfully but the acknowledgement is lost and the message is resent with the same <code>MessageDeduplicationId</code> after the deduplication interval, Amazon SQS can't detect duplicate messages. Amazon SQS continues to keep track of the message deduplication ID even after the message is received and deleted. </note> The length of <code>MessageDeduplicationId</code> is 128 characters. <code>MessageDeduplicationId</code> can contain alphanumeric characters (<code>a-z</code>, <code>A-Z</code>, <code>0-9</code>) and punctuation (<code>!"#$%&amp;'()*+,-./:;&lt;=&gt;?@[\]^_`{|}~</code>). For best practices of using <code>MessageDeduplicationId</code>, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/using-messagededuplicationid-property.html">Using the MessageDeduplicationId Property</a> in the <i>Amazon Simple Queue Service Developer Guide</i>.<br/>MessageAttributes:  Each message attribute consists of a <code>Name</code>, <code>Type</code>, and <code>Value</code>. For more information, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-message-attributes.html">Amazon SQS Message Attributes</a> in the <i>Amazon Simple Queue Service Developer Guide</i>.<br/>DelaySeconds:   The length of time, in seconds, for which to delay a specific message. Valid values: 0 to 900. Maximum: 15 minutes. Messages with a positive <code>DelaySeconds</code> value become available for processing after the delay period is finished. If you don't specify a value, the default value for the queue applies.  <note> When you set <code>FifoQueue</code>, you can't set <code>DelaySeconds</code> per message. You can set this parameter only on a queue level. </note><br/>MessageBody:  The message to send. The maximum string size is 256 KB. <important> A message can include only XML, JSON, and unformatted text. The following Unicode characters are allowed:  <code>#x9</code> | <code>#xA</code> | <code>#xD</code> | <code>#x20</code> to <code>#xD7FF</code> | <code>#xE000</code> to <code>#xFFFD</code> | <code>#x10000</code> to <code>#x10FFFF</code>  Any characters not included in this list will be rejected. For more information, see the <a href="http://www.w3.org/TR/REC-xml/#charsets">W3C specification for characters</a>. </important><br/>QueueUrl:  The URL of the Amazon SQS queue to which a message is sent. Queue URLs and names are case-sensitive.<br/>
        :return: SequenceNumber:  This parameter applies only to FIFO (first-in-first-out) queues. The large, non-consecutive number that Amazon SQS assigns to each message. The length of <code>SequenceNumber</code> is 128 bits. <code>SequenceNumber</code> continues to increase for a particular <code>MessageGroupId</code>.<br/>MessageId:  An attribute containing the <code>MessageId</code> of the message sent to the queue. For more information, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-queue-message-identifiers.html">Queue and Message Identifiers</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. <br/>MD5OfMessageAttributes:  An MD5 digest of the non-URL-encoded message attribute string. You can use this attribute to verify that Amazon SQS received the message correctly. Amazon SQS URL-decodes the message before creating the MD5 digest. For information about MD5, see <a href="https://www.ietf.org/rfc/rfc1321.txt">RFC1321</a>.<br/>MD5OfMessageBody:  An MD5 digest of the non-URL-encoded message attribute string. You can use this attribute to verify that Amazon SQS received the message correctly. Amazon SQS URL-decodes the message before creating the MD5 digest. For information about MD5, see <a href="https://www.ietf.org/rfc/rfc1321.txt">RFC1321</a>.<br/>        """
        pass

    def send_message_batch(self, Entries: List[SendMessageBatchRequestEntry], QueueUrl: str) -> SendMessageBatchResult:
        """Delivers up to ten messages to the specified queue. This is a batch version of <code> <a>SendMessage</a>.</code> For a FIFO queue, multiple messages within a single batch are enqueued in the order they are sent. The result of sending each message is reported individually in the response. Because the batch request can result in a combination of successful and unsuccessful actions, you should check for batch errors even when the call returns an HTTP status code of <code>200</code>. The maximum allowed individual message size and the maximum total payload size (the sum of the individual lengths of all of the batched messages) are both 256 KB (262,144 bytes). <important> A message can include only XML, JSON, and unformatted text. The following Unicode characters are allowed:  <code>#x9</code> | <code>#xA</code> | <code>#xD</code> | <code>#x20</code> to <code>#xD7FF</code> | <code>#xE000</code> to <code>#xFFFD</code> | <code>#x10000</code> to <code>#x10FFFF</code>  Any characters not included in this list will be rejected. For more information, see the <a href="http://www.w3.org/TR/REC-xml/#charsets">W3C specification for characters</a>. </important> If you don't specify the <code>DelaySeconds</code> parameter for an entry, Amazon SQS uses the default value for the queue. Some actions take lists of parameters. These lists are specified using the <code>param.n</code> notation. Values of <code>n</code> are integers starting from 1. For example, a parameter list with two elements looks like this:  <code>&amp;Attribute.1=first</code>   <code>&amp;Attribute.2=second</code> 

        :param self:
        :param Entries:  A list of <code> <a>SendMessageBatchRequestEntry</a> </code> items.<br/>QueueUrl:  The URL of the Amazon SQS queue to which batched messages are sent. Queue URLs and names are case-sensitive.<br/>
        :return: Failed:  A list of <code> <a>BatchResultErrorEntry</a> </code> items with error details about each message that can't be enqueued.<br/>Successful:  A list of <code> <a>SendMessageBatchResultEntry</a> </code> items.<br/>        """
        pass

    def set_queue_attributes(self, Attributes: QueueAttributeMap, QueueUrl: str):
        """Sets the value of one or more queue attributes. When you change a queue's attributes, the change can take up to 60 seconds for most of the attributes to propagate throughout the Amazon SQS system. Changes made to the <code>MessageRetentionPeriod</code> attribute can take up to 15 minutes. <note> In the future, new attributes might be added. If you write code that calls this action, we recommend that you structure your code so that it can handle new attributes gracefully. Cross-account permissions don't apply to this action. For more information, see see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-customer-managed-policy-examples.html#grant-cross-account-permissions-to-role-and-user-name">Grant Cross-Account Permissions to a Role and a User Name</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. </note>

        :param self:
        :param Attributes:  A map of attributes to set. The following lists the names, descriptions, and values of the special request parameters that the <code>SetQueueAttributes</code> action uses: <ul> <li>  <code>DelaySeconds</code> - The length of time, in seconds, for which the delivery of all messages in the queue is delayed. Valid values: An integer from 0 to 900 (15 minutes). Default: 0.  </li> <li>  <code>MaximumMessageSize</code> - The limit of how many bytes a message can contain before Amazon SQS rejects it. Valid values: An integer from 1,024 bytes (1 KiB) up to 262,144 bytes (256 KiB). Default: 262,144 (256 KiB).  </li> <li>  <code>MessageRetentionPeriod</code> - The length of time, in seconds, for which Amazon SQS retains a message. Valid values: An integer representing seconds, from 60 (1 minute) to 1,209,600 (14 days). Default: 345,600 (4 days).  </li> <li>  <code>Policy</code> - The queue's policy. A valid AWS policy. For more information about policy structure, see <a href="http://docs.aws.amazon.com/IAM/latest/UserGuide/PoliciesOverview.html">Overview of AWS IAM Policies</a> in the <i>Amazon IAM User Guide</i>.  </li> <li>  <code>ReceiveMessageWaitTimeSeconds</code> - The length of time, in seconds, for which a <code> <a>ReceiveMessage</a> </code> action waits for a message to arrive. Valid values: an integer from 0 to 20 (seconds). Default: 0.  </li> <li>  <code>RedrivePolicy</code> - The string that includes the parameters for the dead-letter queue functionality of the source queue. For more information about the redrive policy and dead-letter queues, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-dead-letter-queues.html">Using Amazon SQS Dead-Letter Queues</a> in the <i>Amazon Simple Queue Service Developer Guide</i>.  <ul> <li>  <code>deadLetterTargetArn</code> - The Amazon Resource Name (ARN) of the dead-letter queue to which Amazon SQS moves messages after the value of <code>maxReceiveCount</code> is exceeded. </li> <li>  <code>maxReceiveCount</code> - The number of times a message is delivered to the source queue before being moved to the dead-letter queue. When the <code>ReceiveCount</code> for a message exceeds the <code>maxReceiveCount</code> for a queue, Amazon SQS moves the message to the dead-letter-queue. </li> </ul> <note> The dead-letter queue of a FIFO queue must also be a FIFO queue. Similarly, the dead-letter queue of a standard queue must also be a standard queue. </note> </li> <li>  <code>VisibilityTimeout</code> - The visibility timeout for the queue, in seconds. Valid values: an integer from 0 to 43,200 (12 hours). Default: 30. For more information about the visibility timeout, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-visibility-timeout.html">Visibility Timeout</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. </li> </ul> The following attributes apply only to <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-server-side-encryption.html">server-side-encryption</a>: <ul> <li>  <code>KmsMasterKeyId</code> - The ID of an AWS-managed customer master key (CMK) for Amazon SQS or a custom CMK. For more information, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-server-side-encryption.html#sqs-sse-key-terms">Key Terms</a>. While the alias of the AWS-managed CMK for Amazon SQS is always <code>alias/aws/sqs</code>, the alias of a custom CMK can, for example, be <code>alias/<i>MyAlias</i> </code>. For more examples, see <a href="http://docs.aws.amazon.com/kms/latest/APIReference/API_DescribeKey.html#API_DescribeKey_RequestParameters">KeyId</a> in the <i>AWS Key Management Service API Reference</i>.  </li> <li>  <code>KmsDataKeyReusePeriodSeconds</code> - The length of time, in seconds, for which Amazon SQS can reuse a <a href="http://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#data-keys">data key</a> to encrypt or decrypt messages before calling AWS KMS again. An integer representing seconds, between 60 seconds (1 minute) and 86,400 seconds (24 hours). Default: 300 (5 minutes). A shorter time period provides better security but results in more calls to KMS which might incur charges after Free Tier. For more information, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-server-side-encryption.html#sqs-how-does-the-data-key-reuse-period-work">How Does the Data Key Reuse Period Work?</a>.  </li> </ul> The following attribute applies only to <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/FIFO-queues.html">FIFO (first-in-first-out) queues</a>: <ul> <li>  <code>ContentBasedDeduplication</code> - Enables content-based deduplication. For more information, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/FIFO-queues.html#FIFO-queues-exactly-once-processing">Exactly-Once Processing</a> in the <i>Amazon Simple Queue Service Developer Guide</i>.  <ul> <li> Every message must have a unique <code>MessageDeduplicationId</code>, <ul> <li> You may provide a <code>MessageDeduplicationId</code> explicitly. </li> <li> If you aren't able to provide a <code>MessageDeduplicationId</code> and you enable <code>ContentBasedDeduplication</code> for your queue, Amazon SQS uses a SHA-256 hash to generate the <code>MessageDeduplicationId</code> using the body of the message (but not the attributes of the message).  </li> <li> If you don't provide a <code>MessageDeduplicationId</code> and the queue doesn't have <code>ContentBasedDeduplication</code> set, the action fails with an error. </li> <li> If the queue has <code>ContentBasedDeduplication</code> set, your <code>MessageDeduplicationId</code> overrides the generated one. </li> </ul> </li> <li> When <code>ContentBasedDeduplication</code> is in effect, messages with identical content sent within the deduplication interval are treated as duplicates and only one copy of the message is delivered. </li> <li> If you send one message with <code>ContentBasedDeduplication</code> enabled and then another message with a <code>MessageDeduplicationId</code> that is the same as the one generated for the first <code>MessageDeduplicationId</code>, the two messages are treated as duplicates and only one copy of the message is delivered.  </li> </ul> </li> </ul><br/>QueueUrl:  The URL of the Amazon SQS queue whose attributes are set. Queue URLs and names are case-sensitive.<br/>
        :return:         """
        pass

    def tag_queue(self, Tags: TagMap, QueueUrl: str):
        """Add cost allocation tags to the specified Amazon SQS queue. For an overview, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-queue-tags.html">Tagging Your Amazon SQS Queues</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. When you use queue tags, keep the following guidelines in mind: <ul> <li> Adding more than 50 tags to a queue isn't recommended. </li> <li> Tags don't have any semantic meaning. Amazon SQS interprets tags as character strings. </li> <li> Tags are case-sensitive. </li> <li> A new tag with a key identical to that of an existing tag overwrites the existing tag. </li> <li> Tagging actions are limited to 5 TPS per AWS account. If your application requires a higher throughput, file a <a href="https://console.aws.amazon.com/support/home#/case/create?issueType=technical">technical support request</a>. </li> </ul> For a full list of tag restrictions, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-limits.html#limits-queues">Limits Related to Queues</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. <note> Cross-account permissions don't apply to this action. For more information, see see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-customer-managed-policy-examples.html#grant-cross-account-permissions-to-role-and-user-name">Grant Cross-Account Permissions to a Role and a User Name</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. </note>

        :param self:
        :param Tags:  The list of tags to be added to the specified queue.<br/>QueueUrl:  The URL of the queue.<br/>
        :return:         """
        pass

    def untag_queue(self, TagKeys: List[TagKey], QueueUrl: str):
        """Remove cost allocation tags from the specified Amazon SQS queue. For an overview, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-queue-tags.html">Tagging Your Amazon SQS Queues</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. When you use queue tags, keep the following guidelines in mind: <ul> <li> Adding more than 50 tags to a queue isn't recommended. </li> <li> Tags don't have any semantic meaning. Amazon SQS interprets tags as character strings. </li> <li> Tags are case-sensitive. </li> <li> A new tag with a key identical to that of an existing tag overwrites the existing tag. </li> <li> Tagging actions are limited to 5 TPS per AWS account. If your application requires a higher throughput, file a <a href="https://console.aws.amazon.com/support/home#/case/create?issueType=technical">technical support request</a>. </li> </ul> For a full list of tag restrictions, see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-limits.html#limits-queues">Limits Related to Queues</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. <note> Cross-account permissions don't apply to this action. For more information, see see <a href="http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-customer-managed-policy-examples.html#grant-cross-account-permissions-to-role-and-user-name">Grant Cross-Account Permissions to a Role and a User Name</a> in the <i>Amazon Simple Queue Service Developer Guide</i>. </note>

        :param self:
        :param TagKeys:  The list of tags to be removed from the specified queue.<br/>QueueUrl:  The URL of the queue.<br/>
        :return:         """
        pass

    class SqsResource:
        
        class Message:
            """Creates a Message resource.::

  message = sqs.Message('queue_url','receipt_handle')

:type queue_url: string
:param queue_url: The Message's queue_url identifier. This **must** be set.
:type receipt_handle: string
:param receipt_handle: The Message's receipt_handle identifier. This **must** be set.

:rtype: :py:class:`SQS.Message`
:returns: A Message resource"""

            def __init__(self, queue_url, receipt_handle):
                pass
            
            class MessageSystemAttributeMap(object):
                pass

            class MessageBodyAttributeMap(object):
                pass
        
            message_id: str=None
            receipt_handle: str=None
            md5_of_body: str=None
            body: str=None
            attributes: MessageSystemAttributeMap=None
            md5_of_message_attributes: str=None
            message_attributes: MessageBodyAttributeMap=None
            
            def change_visibility(self, **kwargs):
                # type: (object) -> dict
                return {}

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}


        
        class Queue:
            """Creates a Queue resource.::

  queue = sqs.Queue('url')

:type url: string
:param url: The Queue's url identifier. This **must** be set.

:rtype: :py:class:`SQS.Queue`
:returns: A Queue resource"""

            def __init__(self, url):
                pass
            
            class QueueAttributeMap(object):
                pass
        
            attributes: QueueAttributeMap=None
            
            def add_permission(self, **kwargs):
                # type: (object) -> dict
                return {}

            def change_message_visibility_batch(self, **kwargs):
                # type: (object) -> dict
                return {}

            def delete(self, **kwargs):
                # type: (object) -> dict
                return {}

            def delete_messages(self, **kwargs):
                # type: (object) -> dict
                return {}

            def purge(self, **kwargs):
                # type: (object) -> dict
                return {}

            def receive_messages(self, **kwargs):
                # type: (object) -> Message
                return Message()

            def remove_permission(self, **kwargs):
                # type: (object) -> dict
                return {}

            def send_message(self, **kwargs):
                # type: (object) -> dict
                return {}

            def send_messages(self, **kwargs):
                # type: (object) -> dict
                return {}

            def set_attributes(self, **kwargs):
                # type: (object) -> dict
                return {}


        def create_queue(self, **kwargs):
            # type: (object) -> Queue
            pass

        def get_queue_by_name(self, **kwargs):
            # type: (object) -> Queue
            pass

        def get_available_subresources(self) -> List[str]:
            pass

        class QueueResourceCollection(List[Queue], ResourceCollection):
            pass

        queues: QueueResourceCollection = None




class SSM(BaseClient):
    """<fullname>AWS Systems Manager</fullname> AWS Systems Manager is a collection of capabilities that helps you automate management tasks such as collecting system inventory, applying operating system (OS) patches, automating the creation of Amazon Machine Images (AMIs), and configuring operating systems (OSs) and applications at scale. Systems Manager lets you remotely and securely manage the configuration of your managed instances. A <i>managed instance</i> is any Amazon EC2 instance or on-premises machine in your hybrid environment that has been configured for Systems Manager. This reference is intended to be used with the <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/">AWS Systems Manager User Guide</a>. To get started, verify prerequisites and configure managed instances. For more information, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/systems-manager-setting-up.html">Setting Up AWS Systems Manager</a> in the <i>AWS Systems Manager User Guide</i>. For information about other API actions you can perform on Amazon EC2 instances, see the <a href="http://docs.aws.amazon.com/AWSEC2/latest/APIReference/">Amazon EC2 API Reference</a>. For information about how to use a Query API, see <a href="http://docs.aws.amazon.com/AWSEC2/latest/APIReference/making-api-requests.html">Making API Requests</a>. """


    class ListAssociationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_associations` for valid parameters"""
            pass
    class ListCommandInvocationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_command_invocations` for valid parameters"""
            pass
    class ListCommandsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_commands` for valid parameters"""
            pass
    class ListDocumentsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_documents` for valid parameters"""
            pass
    class DescribeInstanceInformationPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_instance_information` for valid parameters"""
            pass
    class DescribeActivationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_activations` for valid parameters"""
            pass
    class DescribeParametersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_parameters` for valid parameters"""
            pass
    class DescribeAssociationExecutionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_association_executions` for valid parameters"""
            pass
    class DescribeAssociationExecutionTargetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_association_execution_targets` for valid parameters"""
            pass
    class GetInventoryPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_inventory` for valid parameters"""
            pass
    class GetParametersByPathPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_parameters_by_path` for valid parameters"""
            pass
    class GetParameterHistoryPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_parameter_history` for valid parameters"""
            pass
    class DescribeAutomationExecutionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_automation_executions` for valid parameters"""
            pass
    class DescribeAutomationStepExecutionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_automation_step_executions` for valid parameters"""
            pass
    class DescribeAvailablePatchesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_available_patches` for valid parameters"""
            pass
    class DescribeEffectiveInstanceAssociationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_effective_instance_associations` for valid parameters"""
            pass
    class DescribeEffectivePatchesForPatchBaselinePaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_effective_patches_for_patch_baseline` for valid parameters"""
            pass
    class DescribeInstanceAssociationsStatusPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_instance_associations_status` for valid parameters"""
            pass
    class DescribeInstancePatchStatesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_instance_patch_states` for valid parameters"""
            pass
    class DescribeInstancePatchStatesForPatchGroupPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_instance_patch_states_for_patch_group` for valid parameters"""
            pass
    class DescribeInstancePatchesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_instance_patches` for valid parameters"""
            pass
    class DescribeInventoryDeletionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_inventory_deletions` for valid parameters"""
            pass
    class DescribeMaintenanceWindowExecutionTaskInvocationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_maintenance_window_execution_task_invocations` for valid parameters"""
            pass
    class DescribeMaintenanceWindowExecutionTasksPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_maintenance_window_execution_tasks` for valid parameters"""
            pass
    class DescribeMaintenanceWindowExecutionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_maintenance_window_executions` for valid parameters"""
            pass
    class DescribeMaintenanceWindowSchedulePaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_maintenance_window_schedule` for valid parameters"""
            pass
    class DescribeMaintenanceWindowTargetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_maintenance_window_targets` for valid parameters"""
            pass
    class DescribeMaintenanceWindowTasksPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_maintenance_window_tasks` for valid parameters"""
            pass
    class DescribeMaintenanceWindowsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_maintenance_windows` for valid parameters"""
            pass
    class DescribeMaintenanceWindowsForTargetPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_maintenance_windows_for_target` for valid parameters"""
            pass
    class DescribePatchBaselinesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_patch_baselines` for valid parameters"""
            pass
    class DescribePatchGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_patch_groups` for valid parameters"""
            pass
    class DescribeSessionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_sessions` for valid parameters"""
            pass
    class GetInventorySchemaPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_inventory_schema` for valid parameters"""
            pass
    class ListAssociationVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_association_versions` for valid parameters"""
            pass
    class ListComplianceItemsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_compliance_items` for valid parameters"""
            pass
    class ListComplianceSummariesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_compliance_summaries` for valid parameters"""
            pass
    class ListDocumentVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_document_versions` for valid parameters"""
            pass
    class ListResourceComplianceSummariesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_resource_compliance_summaries` for valid parameters"""
            pass
    class ListResourceDataSyncPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_resource_data_sync` for valid parameters"""
            pass


    class AddTagsToResourceRequest(Mapping):
        pass
    
    class AddTagsToResourceResult(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CancelCommandRequest(Mapping):
        pass
    
    class CancelCommandResult(Mapping):
        pass
    
    class InstanceId(object):
        pass
    
    class CancelMaintenanceWindowExecutionRequest(Mapping):
        pass
    
    class CancelMaintenanceWindowExecutionResult(Mapping):
        pass
    
    class CreateActivationRequest(Mapping):
        pass
    
    class CreateActivationResult(Mapping):
        pass
    
    class ExpirationDate(object):
        pass
    
    class CreateAssociationRequest(Mapping):
        pass
    
    class CreateAssociationResult(Mapping):
        pass
    
    class Parameters(object):
        pass
    
    class Target(Mapping):
        pass
    
    class InstanceAssociationOutputLocation(Mapping):
        pass
    
    class CreateAssociationBatchRequest(Mapping):
        pass
    
    class CreateAssociationBatchResult(Mapping):
        pass
    
    class CreateAssociationBatchRequestEntry(Mapping):
        pass
    
    class CreateDocumentRequest(Mapping):
        pass
    
    class CreateDocumentResult(Mapping):
        pass
    
    class AttachmentsSource(Mapping):
        pass
    
    class CreateMaintenanceWindowRequest(Mapping):
        pass
    
    class CreateMaintenanceWindowResult(Mapping):
        pass
    
    class MaintenanceWindowAllowUnassociatedTargets(object):
        pass
    
    class CreateOpsItemRequest(Mapping):
        pass
    
    class CreateOpsItemResponse(Mapping):
        pass
    
    class OpsItemOperationalData(object):
        pass
    
    class OpsItemNotification(Mapping):
        pass
    
    class RelatedOpsItem(Mapping):
        pass
    
    class CreatePatchBaselineRequest(Mapping):
        pass
    
    class CreatePatchBaselineResult(Mapping):
        pass
    
    class PatchFilterGroup(Mapping):
        pass
    
    class PatchRuleGroup(Mapping):
        pass
    
    class PatchId(object):
        pass
    
    class Boolean(object):
        pass
    
    class PatchSource(Mapping):
        pass
    
    class CreateResourceDataSyncRequest(Mapping):
        pass
    
    class CreateResourceDataSyncResult(Mapping):
        pass
    
    class ResourceDataSyncS3Destination(Mapping):
        pass
    
    class DeleteActivationRequest(Mapping):
        pass
    
    class DeleteActivationResult(Mapping):
        pass
    
    class DeleteAssociationRequest(Mapping):
        pass
    
    class DeleteAssociationResult(Mapping):
        pass
    
    class DeleteDocumentRequest(Mapping):
        pass
    
    class DeleteDocumentResult(Mapping):
        pass
    
    class DeleteInventoryRequest(Mapping):
        pass
    
    class DeleteInventoryResult(Mapping):
        pass
    
    class DryRun(object):
        pass
    
    class DeleteMaintenanceWindowRequest(Mapping):
        pass
    
    class DeleteMaintenanceWindowResult(Mapping):
        pass
    
    class DeleteParameterRequest(Mapping):
        pass
    
    class DeleteParameterResult(Mapping):
        pass
    
    class DeleteParametersRequest(Mapping):
        pass
    
    class DeleteParametersResult(Mapping):
        pass
    
    class PSParameterName(object):
        pass
    
    class DeletePatchBaselineRequest(Mapping):
        pass
    
    class DeletePatchBaselineResult(Mapping):
        pass
    
    class DeleteResourceDataSyncRequest(Mapping):
        pass
    
    class DeleteResourceDataSyncResult(Mapping):
        pass
    
    class DeregisterManagedInstanceRequest(Mapping):
        pass
    
    class DeregisterManagedInstanceResult(Mapping):
        pass
    
    class DeregisterPatchBaselineForPatchGroupRequest(Mapping):
        pass
    
    class DeregisterPatchBaselineForPatchGroupResult(Mapping):
        pass
    
    class DeregisterTargetFromMaintenanceWindowRequest(Mapping):
        pass
    
    class DeregisterTargetFromMaintenanceWindowResult(Mapping):
        pass
    
    class DeregisterTaskFromMaintenanceWindowRequest(Mapping):
        pass
    
    class DeregisterTaskFromMaintenanceWindowResult(Mapping):
        pass
    
    class DescribeActivationsRequest(Mapping):
        pass
    
    class DescribeActivationsResult(Mapping):
        pass
    
    class DescribeActivationsFilter(Mapping):
        pass
    
    class DescribeAssociationRequest(Mapping):
        pass
    
    class DescribeAssociationResult(Mapping):
        pass
    
    class DescribeAssociationExecutionTargetsRequest(Mapping):
        pass
    
    class DescribeAssociationExecutionTargetsResult(Mapping):
        pass
    
    class AssociationExecutionTargetsFilter(Mapping):
        pass
    
    class DescribeAssociationExecutionsRequest(Mapping):
        pass
    
    class DescribeAssociationExecutionsResult(Mapping):
        pass
    
    class AssociationExecutionFilter(Mapping):
        pass
    
    class DescribeAutomationExecutionsRequest(Mapping):
        pass
    
    class DescribeAutomationExecutionsResult(Mapping):
        pass
    
    class AutomationExecutionFilter(Mapping):
        pass
    
    class DescribeAutomationStepExecutionsRequest(Mapping):
        pass
    
    class DescribeAutomationStepExecutionsResult(Mapping):
        pass
    
    class StepExecutionFilter(Mapping):
        pass
    
    class DescribeAvailablePatchesRequest(Mapping):
        pass
    
    class DescribeAvailablePatchesResult(Mapping):
        pass
    
    class PatchOrchestratorFilter(Mapping):
        pass
    
    class DescribeDocumentRequest(Mapping):
        pass
    
    class DescribeDocumentResult(Mapping):
        pass
    
    class DescribeDocumentPermissionRequest(Mapping):
        pass
    
    class DescribeDocumentPermissionResponse(Mapping):
        pass
    
    class DescribeEffectiveInstanceAssociationsRequest(Mapping):
        pass
    
    class DescribeEffectiveInstanceAssociationsResult(Mapping):
        pass
    
    class DescribeEffectivePatchesForPatchBaselineRequest(Mapping):
        pass
    
    class DescribeEffectivePatchesForPatchBaselineResult(Mapping):
        pass
    
    class DescribeInstanceAssociationsStatusRequest(Mapping):
        pass
    
    class DescribeInstanceAssociationsStatusResult(Mapping):
        pass
    
    class DescribeInstanceInformationRequest(Mapping):
        pass
    
    class DescribeInstanceInformationResult(Mapping):
        pass
    
    class InstanceInformationFilter(Mapping):
        pass
    
    class InstanceInformationStringFilter(Mapping):
        pass
    
    class DescribeInstancePatchStatesRequest(Mapping):
        pass
    
    class DescribeInstancePatchStatesResult(Mapping):
        pass
    
    class DescribeInstancePatchStatesForPatchGroupRequest(Mapping):
        pass
    
    class DescribeInstancePatchStatesForPatchGroupResult(Mapping):
        pass
    
    class InstancePatchStateFilter(Mapping):
        pass
    
    class DescribeInstancePatchesRequest(Mapping):
        pass
    
    class DescribeInstancePatchesResult(Mapping):
        pass
    
    class DescribeInventoryDeletionsRequest(Mapping):
        pass
    
    class DescribeInventoryDeletionsResult(Mapping):
        pass
    
    class DescribeMaintenanceWindowExecutionTaskInvocationsRequest(Mapping):
        pass
    
    class DescribeMaintenanceWindowExecutionTaskInvocationsResult(Mapping):
        pass
    
    class MaintenanceWindowFilter(Mapping):
        pass
    
    class DescribeMaintenanceWindowExecutionTasksRequest(Mapping):
        pass
    
    class DescribeMaintenanceWindowExecutionTasksResult(Mapping):
        pass
    
    class DescribeMaintenanceWindowExecutionsRequest(Mapping):
        pass
    
    class DescribeMaintenanceWindowExecutionsResult(Mapping):
        pass
    
    class DescribeMaintenanceWindowScheduleRequest(Mapping):
        pass
    
    class DescribeMaintenanceWindowScheduleResult(Mapping):
        pass
    
    class DescribeMaintenanceWindowTargetsRequest(Mapping):
        pass
    
    class DescribeMaintenanceWindowTargetsResult(Mapping):
        pass
    
    class DescribeMaintenanceWindowTasksRequest(Mapping):
        pass
    
    class DescribeMaintenanceWindowTasksResult(Mapping):
        pass
    
    class DescribeMaintenanceWindowsRequest(Mapping):
        pass
    
    class DescribeMaintenanceWindowsResult(Mapping):
        pass
    
    class DescribeMaintenanceWindowsForTargetRequest(Mapping):
        pass
    
    class DescribeMaintenanceWindowsForTargetResult(Mapping):
        pass
    
    class DescribeOpsItemsRequest(Mapping):
        pass
    
    class DescribeOpsItemsResponse(Mapping):
        pass
    
    class OpsItemFilter(Mapping):
        pass
    
    class DescribeParametersRequest(Mapping):
        pass
    
    class DescribeParametersResult(Mapping):
        pass
    
    class ParametersFilter(Mapping):
        pass
    
    class ParameterStringFilter(Mapping):
        pass
    
    class DescribePatchBaselinesRequest(Mapping):
        pass
    
    class DescribePatchBaselinesResult(Mapping):
        pass
    
    class DescribePatchGroupStateRequest(Mapping):
        pass
    
    class DescribePatchGroupStateResult(Mapping):
        pass
    
    class DescribePatchGroupsRequest(Mapping):
        pass
    
    class DescribePatchGroupsResult(Mapping):
        pass
    
    class DescribePatchPropertiesRequest(Mapping):
        pass
    
    class DescribePatchPropertiesResult(Mapping):
        pass
    
    class DescribeSessionsRequest(Mapping):
        pass
    
    class DescribeSessionsResponse(Mapping):
        pass
    
    class SessionFilter(Mapping):
        pass
    
    class GetAutomationExecutionRequest(Mapping):
        pass
    
    class GetAutomationExecutionResult(Mapping):
        pass
    
    class GetCommandInvocationRequest(Mapping):
        pass
    
    class GetCommandInvocationResult(Mapping):
        pass
    
    class GetConnectionStatusRequest(Mapping):
        pass
    
    class GetConnectionStatusResponse(Mapping):
        pass
    
    class GetDefaultPatchBaselineRequest(Mapping):
        pass
    
    class GetDefaultPatchBaselineResult(Mapping):
        pass
    
    class GetDeployablePatchSnapshotForInstanceRequest(Mapping):
        pass
    
    class GetDeployablePatchSnapshotForInstanceResult(Mapping):
        pass
    
    class GetDocumentRequest(Mapping):
        pass
    
    class GetDocumentResult(Mapping):
        pass
    
    class GetInventoryRequest(Mapping):
        pass
    
    class GetInventoryResult(Mapping):
        pass
    
    class InventoryFilter(Mapping):
        pass
    
    class InventoryAggregator(Mapping):
        pass
    
    class ResultAttribute(Mapping):
        pass
    
    class GetInventorySchemaRequest(Mapping):
        pass
    
    class GetInventorySchemaResult(Mapping):
        pass
    
    class AggregatorSchemaOnly(object):
        pass
    
    class IsSubTypeSchema(object):
        pass
    
    class GetMaintenanceWindowRequest(Mapping):
        pass
    
    class GetMaintenanceWindowResult(Mapping):
        pass
    
    class GetMaintenanceWindowExecutionRequest(Mapping):
        pass
    
    class GetMaintenanceWindowExecutionResult(Mapping):
        pass
    
    class GetMaintenanceWindowExecutionTaskRequest(Mapping):
        pass
    
    class GetMaintenanceWindowExecutionTaskResult(Mapping):
        pass
    
    class GetMaintenanceWindowExecutionTaskInvocationRequest(Mapping):
        pass
    
    class GetMaintenanceWindowExecutionTaskInvocationResult(Mapping):
        pass
    
    class GetMaintenanceWindowTaskRequest(Mapping):
        pass
    
    class GetMaintenanceWindowTaskResult(Mapping):
        pass
    
    class GetOpsItemRequest(Mapping):
        pass
    
    class GetOpsItemResponse(Mapping):
        pass
    
    class GetOpsSummaryRequest(Mapping):
        pass
    
    class GetOpsSummaryResult(Mapping):
        pass
    
    class OpsFilter(Mapping):
        pass
    
    class OpsAggregator(Mapping):
        pass
    
    class GetParameterRequest(Mapping):
        pass
    
    class GetParameterResult(Mapping):
        pass
    
    class GetParameterHistoryRequest(Mapping):
        pass
    
    class GetParameterHistoryResult(Mapping):
        pass
    
    class GetParametersRequest(Mapping):
        pass
    
    class GetParametersResult(Mapping):
        pass
    
    class GetParametersByPathRequest(Mapping):
        pass
    
    class GetParametersByPathResult(Mapping):
        pass
    
    class GetPatchBaselineRequest(Mapping):
        pass
    
    class GetPatchBaselineResult(Mapping):
        pass
    
    class GetPatchBaselineForPatchGroupRequest(Mapping):
        pass
    
    class GetPatchBaselineForPatchGroupResult(Mapping):
        pass
    
    class GetServiceSettingRequest(Mapping):
        pass
    
    class GetServiceSettingResult(Mapping):
        pass
    
    class LabelParameterVersionRequest(Mapping):
        pass
    
    class LabelParameterVersionResult(Mapping):
        pass
    
    class PSParameterVersion(object):
        pass
    
    class ParameterLabel(object):
        pass
    
    class ListAssociationVersionsRequest(Mapping):
        pass
    
    class ListAssociationVersionsResult(Mapping):
        pass
    
    class ListAssociationsRequest(Mapping):
        pass
    
    class ListAssociationsResult(Mapping):
        pass
    
    class AssociationFilter(Mapping):
        pass
    
    class ListCommandInvocationsRequest(Mapping):
        pass
    
    class ListCommandInvocationsResult(Mapping):
        pass
    
    class CommandFilter(Mapping):
        pass
    
    class ListCommandsRequest(Mapping):
        pass
    
    class ListCommandsResult(Mapping):
        pass
    
    class ListComplianceItemsRequest(Mapping):
        pass
    
    class ListComplianceItemsResult(Mapping):
        pass
    
    class ComplianceStringFilter(Mapping):
        pass
    
    class ComplianceResourceId(object):
        pass
    
    class ComplianceResourceType(object):
        pass
    
    class ListComplianceSummariesRequest(Mapping):
        pass
    
    class ListComplianceSummariesResult(Mapping):
        pass
    
    class ListDocumentVersionsRequest(Mapping):
        pass
    
    class ListDocumentVersionsResult(Mapping):
        pass
    
    class ListDocumentsRequest(Mapping):
        pass
    
    class ListDocumentsResult(Mapping):
        pass
    
    class DocumentFilter(Mapping):
        pass
    
    class DocumentKeyValuesFilter(Mapping):
        pass
    
    class ListInventoryEntriesRequest(Mapping):
        pass
    
    class ListInventoryEntriesResult(Mapping):
        pass
    
    class ListResourceComplianceSummariesRequest(Mapping):
        pass
    
    class ListResourceComplianceSummariesResult(Mapping):
        pass
    
    class ListResourceDataSyncRequest(Mapping):
        pass
    
    class ListResourceDataSyncResult(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResult(Mapping):
        pass
    
    class ModifyDocumentPermissionRequest(Mapping):
        pass
    
    class ModifyDocumentPermissionResponse(Mapping):
        pass
    
    class AccountId(object):
        pass
    
    class PutComplianceItemsRequest(Mapping):
        pass
    
    class PutComplianceItemsResult(Mapping):
        pass
    
    class ComplianceExecutionSummary(Mapping):
        pass
    
    class ComplianceItemEntry(Mapping):
        pass
    
    class PutInventoryRequest(Mapping):
        pass
    
    class PutInventoryResult(Mapping):
        pass
    
    class InventoryItem(Mapping):
        pass
    
    class PutParameterRequest(Mapping):
        pass
    
    class PutParameterResult(Mapping):
        pass
    
    class RegisterDefaultPatchBaselineRequest(Mapping):
        pass
    
    class RegisterDefaultPatchBaselineResult(Mapping):
        pass
    
    class RegisterPatchBaselineForPatchGroupRequest(Mapping):
        pass
    
    class RegisterPatchBaselineForPatchGroupResult(Mapping):
        pass
    
    class RegisterTargetWithMaintenanceWindowRequest(Mapping):
        pass
    
    class RegisterTargetWithMaintenanceWindowResult(Mapping):
        pass
    
    class RegisterTaskWithMaintenanceWindowRequest(Mapping):
        pass
    
    class RegisterTaskWithMaintenanceWindowResult(Mapping):
        pass
    
    class MaintenanceWindowTaskParameters(object):
        pass
    
    class MaintenanceWindowTaskInvocationParameters(Mapping):
        pass
    
    class LoggingInfo(Mapping):
        pass
    
    class RemoveTagsFromResourceRequest(Mapping):
        pass
    
    class RemoveTagsFromResourceResult(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class ResetServiceSettingRequest(Mapping):
        pass
    
    class ResetServiceSettingResult(Mapping):
        pass
    
    class ResumeSessionRequest(Mapping):
        pass
    
    class ResumeSessionResponse(Mapping):
        pass
    
    class SendAutomationSignalRequest(Mapping):
        pass
    
    class SendAutomationSignalResult(Mapping):
        pass
    
    class AutomationParameterMap(object):
        pass
    
    class SendCommandRequest(Mapping):
        pass
    
    class SendCommandResult(Mapping):
        pass
    
    class NotificationConfig(Mapping):
        pass
    
    class CloudWatchOutputConfig(Mapping):
        pass
    
    class StartAssociationsOnceRequest(Mapping):
        pass
    
    class StartAssociationsOnceResult(Mapping):
        pass
    
    class AssociationId(object):
        pass
    
    class StartAutomationExecutionRequest(Mapping):
        pass
    
    class StartAutomationExecutionResult(Mapping):
        pass
    
    class TargetMap(object):
        pass
    
    class TargetLocation(Mapping):
        pass
    
    class StartSessionRequest(Mapping):
        pass
    
    class StartSessionResponse(Mapping):
        pass
    
    class SessionManagerParameters(object):
        pass
    
    class StopAutomationExecutionRequest(Mapping):
        pass
    
    class StopAutomationExecutionResult(Mapping):
        pass
    
    class TerminateSessionRequest(Mapping):
        pass
    
    class TerminateSessionResponse(Mapping):
        pass
    
    class UpdateAssociationRequest(Mapping):
        pass
    
    class UpdateAssociationResult(Mapping):
        pass
    
    class UpdateAssociationStatusRequest(Mapping):
        pass
    
    class UpdateAssociationStatusResult(Mapping):
        pass
    
    class AssociationStatus(Mapping):
        pass
    
    class UpdateDocumentRequest(Mapping):
        pass
    
    class UpdateDocumentResult(Mapping):
        pass
    
    class UpdateDocumentDefaultVersionRequest(Mapping):
        pass
    
    class UpdateDocumentDefaultVersionResult(Mapping):
        pass
    
    class UpdateMaintenanceWindowRequest(Mapping):
        pass
    
    class UpdateMaintenanceWindowResult(Mapping):
        pass
    
    class MaintenanceWindowEnabled(object):
        pass
    
    class UpdateMaintenanceWindowTargetRequest(Mapping):
        pass
    
    class UpdateMaintenanceWindowTargetResult(Mapping):
        pass
    
    class UpdateMaintenanceWindowTaskRequest(Mapping):
        pass
    
    class UpdateMaintenanceWindowTaskResult(Mapping):
        pass
    
    class UpdateManagedInstanceRoleRequest(Mapping):
        pass
    
    class UpdateManagedInstanceRoleResult(Mapping):
        pass
    
    class UpdateOpsItemRequest(Mapping):
        pass
    
    class UpdateOpsItemResponse(Mapping):
        pass
    
    class String(object):
        pass
    
    class UpdatePatchBaselineRequest(Mapping):
        pass
    
    class UpdatePatchBaselineResult(Mapping):
        pass
    
    class UpdateServiceSettingRequest(Mapping):
        pass
    
    class UpdateServiceSettingResult(Mapping):
        pass
    
    def add_tags_to_resource(self, Tags: List[Tag], ResourceId: str, ResourceType: str) -> AddTagsToResourceResult:
        """Adds or overwrites one or more tags for the specified resource. Tags are metadata that you can assign to your documents, managed instances, maintenance windows, Parameter Store parameters, and patch baselines. Tags enable you to categorize your resources in different ways, for example, by purpose, owner, or environment. Each tag consists of a key and an optional value, both of which you define. For example, you could define a set of tags for your account's managed instances that helps you track each instance's owner and stack level. For example: Key=Owner and Value=DbAdmin, SysAdmin, or Dev. Or Key=Stack and Value=Production, Pre-Production, or Test. Each resource can have a maximum of 50 tags.  We recommend that you devise a set of tag keys that meets your needs for each resource type. Using a consistent set of tag keys makes it easier for you to manage your resources. You can search and filter the resources based on the tags you add. Tags don't have any semantic meaning to Amazon EC2 and are interpreted strictly as a string of characters.  For more information about tags, see <a href="http://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Using_Tags.html">Tagging Your Amazon EC2 Resources</a> in the <i>Amazon EC2 User Guide</i>.

        :param self:
        :param Tags:   One or more tags. The value parameter is required, but if you don't want the tag to have a value, specify the parameter with no value, and we set the value to an empty string.  <important> Do not enter personally identifiable information in this field. </important><br/>ResourceId:  The resource ID you want to tag. Use the ID of the resource. Here are some examples: ManagedInstance: mi-012345abcde MaintenanceWindow: mw-012345abcde PatchBaseline: pb-012345abcde For the Document and Parameter values, use the name of the resource. <note> The ManagedInstance type for this API action is only for on-premises managed instances. You must specify the name of the managed instance in the following format: mi-ID_number. For example, mi-1a2b3c4d5e6f. </note><br/>ResourceType:  Specifies the type of resource you are tagging. <note> The ManagedInstance type for this API action is for on-premises managed instances. You must specify the name of the managed instance in the following format: mi-ID_number. For example, mi-1a2b3c4d5e6f. </note><br/>
        :return:         """
        pass

    def cancel_command(self, CommandId: str, InstanceIds: List[InstanceId]=None) -> CancelCommandResult:
        """Attempts to cancel the command specified by the Command ID. There is no guarantee that the command will be terminated and the underlying process stopped.

        :param self:
        :param InstanceIds:  (Optional) A list of instance IDs on which you want to cancel the command. If not provided, the command is canceled on every instance on which it was requested.<br/>CommandId:  The ID of the command you want to cancel.<br/>
        :return:         """
        pass

    def cancel_maintenance_window_execution(self, WindowExecutionId: str) -> CancelMaintenanceWindowExecutionResult:
        """Stops a maintenance window execution that is already in progress and cancels any tasks in the window that have not already starting running. (Tasks already in progress will continue to completion.)

        :param self:
        :param WindowExecutionId:  The ID of the maintenance window execution to stop.<br/>
        :return: WindowExecutionId:  The ID of the maintenance window execution that has been stopped.<br/>        """
        pass

    def create_activation(self, IamRole: str, Description: str=None, DefaultInstanceName: str=None, RegistrationLimit: int=None, ExpirationDate: ExpirationDate=None, Tags: List[Tag]=None) -> CreateActivationResult:
        """Registers your on-premises server or virtual machine with Amazon EC2 so that you can manage these resources using Run Command. An on-premises server or virtual machine that has been registered with EC2 is called a managed instance. For more information about activations, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/systems-manager-managedinstances.html">Setting Up AWS Systems Manager for Hybrid Environments</a>.

        :param self:
        :param Tags:  Optional metadata that you assign to a resource. Tags enable you to categorize a resource in different ways, such as by purpose, owner, or environment. For example, you might want to tag an activation to identify which servers or virtual machines (VMs) in your on-premises environment you intend to activate. In this case, you could specify the following key name/value pairs: <ul> <li>  <code>Key=OS,Value=Windows</code>  </li> <li>  <code>Key=Environment,Value=Production</code>  </li> </ul> <important> When you install SSM Agent on your on-premises servers and VMs, you specify an activation ID and code. When you specify the activation ID and code, tags assigned to the activation are automatically applied to the on-premises servers or VMs. </important> You can't add tags to or delete tags from an existing activation. You can tag your on-premises servers and VMs after they connect to Systems Manager for the first time and are assigned a managed instance ID. This means they are listed in the AWS Systems Manager console with an ID that is prefixed with "mi-". For information about how to add tags to your managed instances, see <a>AddTagsToResource</a>. For information about how to remove tags from your managed instances, see <a>RemoveTagsFromResource</a>.<br/>ExpirationDate:  The date by which this activation request should expire. The default value is 24 hours.<br/>RegistrationLimit:  Specify the maximum number of managed instances you want to register. The default value is 1 instance.<br/>IamRole:  The Amazon Identity and Access Management (IAM) role that you want to assign to the managed instance. <br/>DefaultInstanceName:  The name of the registered, managed instance as it will appear in the Amazon EC2 console or when you use the AWS command line tools to list EC2 resources. <important> Do not enter personally identifiable information in this field. </important><br/>Description:  A user-defined description of the resource that you want to register with Amazon EC2.  <important> Do not enter personally identifiable information in this field. </important><br/>
        :return: ActivationCode:  The code the system generates when it processes the activation. The activation code functions like a password to validate the activation ID. <br/>ActivationId:  The ID number generated by the system when it processed the activation. The activation ID functions like a user name.<br/>        """
        pass

    def create_association(self, Name: str, DocumentVersion: str=None, InstanceId: str=None, Parameters: Parameters=None, Targets: List[Target]=None, ScheduleExpression: str=None, OutputLocation: InstanceAssociationOutputLocation=None, AssociationName: str=None, AutomationTargetParameterName: str=None, MaxErrors: str=None, MaxConcurrency: str=None, ComplianceSeverity: str=None) -> CreateAssociationResult:
        """Associates the specified Systems Manager document with the specified instances or targets. When you associate a document with one or more instances using instance IDs or tags, SSM Agent running on the instance processes the document and configures the instance as specified. If you associate a document with an instance that already has an associated document, the system returns the AssociationAlreadyExists exception.

        :param self:
        :param ComplianceSeverity:  The severity level to assign to the association.<br/>MaxConcurrency:  The maximum number of targets allowed to run the association at the same time. You can specify a number, for example 10, or a percentage of the target set, for example 10%. The default value is 100%, which means all targets run the association at the same time. If a new instance starts and attempts to run an association while Systems Manager is running MaxConcurrency associations, the association is allowed to run. During the next association interval, the new instance will process its association within the limit specified for MaxConcurrency.<br/>MaxErrors:  The number of errors that are allowed before the system stops sending requests to run the association on additional targets. You can specify either an absolute number of errors, for example 10, or a percentage of the target set, for example 10%. If you specify 3, for example, the system stops sending requests when the fourth error is received. If you specify 0, then the system stops sending requests after the first error is returned. If you run an association on 50 instances and set MaxError to 10%, then the system stops sending the request when the sixth error is received. Executions that are already running an association when MaxErrors is reached are allowed to complete, but some of these executions may fail as well. If you need to ensure that there won't be more than max-errors failed executions, set MaxConcurrency to 1 so that executions proceed one at a time.<br/>AutomationTargetParameterName:  Specify the target for the association. This target is required for associations that use an Automation document and target resources by using rate controls.<br/>AssociationName:  Specify a descriptive name for the association.<br/>OutputLocation:  An Amazon S3 bucket where you want to store the output details of the request.:<br/>ScheduleExpression:  A cron expression when the association will be applied to the target(s).<br/>Targets:  The targets (either instances or tags) for the association.<br/>Parameters:  The parameters for the runtime configuration of the document.<br/>InstanceId:  The instance ID.<br/>DocumentVersion:  The document version you want to associate with the target(s). Can be a specific version or the default version.<br/>Name:  The name of the SSM document that contains the configuration information for the instance. You can specify Command or Automation documents. You can specify AWS-predefined documents, documents you created, or a document that is shared with you from another account. For SSM documents that are shared with you from other AWS accounts, you must specify the complete SSM document ARN, in the following format:  <code>arn:<i>partition</i>:ssm:<i>region</i>:<i>account-id</i>:document/<i>document-name</i> </code>  For example:  <code>arn:aws:ssm:us-east-2:12345678912:document/My-Shared-Document</code>  For AWS-predefined documents and SSM documents you created in your account, you only need to specify the document name. For example, <code>AWS-ApplyPatchBaseline</code> or <code>My-Document</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Location</i> An Amazon S3 bucket where you want to store the results of this request.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Location</b>: An Amazon S3 bucket where you want to store the results of this request.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputS3KeyPrefix</i> The Amazon S3 bucket subfolder.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputS3BucketName</i> The name of the Amazon S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputS3Region</i> (Deprecated) You can no longer specify this parameter. The system ignores it. Instead, Systems Manager automatically determines the Amazon S3 bucket region.<br/>
        :return: AssociationDescription:  Information about the association.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ComplianceSeverity</i> The severity level that is assigned to the association.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxConcurrency</i> The maximum number of targets allowed to run the association at the same time. You can specify a number, for example 10, or a percentage of the target set, for example 10%. The default value is 100%, which means all targets run the association at the same time. If a new instance starts and attempts to run an association while Systems Manager is running MaxConcurrency associations, the association is allowed to run. During the next association interval, the new instance will process its association within the limit specified for MaxConcurrency.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxErrors</i> The number of errors that are allowed before the system stops sending requests to run the association on additional targets. You can specify either an absolute number of errors, for example 10, or a percentage of the target set, for example 10%. If you specify 3, for example, the system stops sending requests when the fourth error is received. If you specify 0, then the system stops sending requests after the first error is returned. If you run an association on 50 instances and set MaxError to 10%, then the system stops sending the request when the sixth error is received. Executions that are already running an association when MaxErrors is reached are allowed to complete, but some of these executions may fail as well. If you need to ensure that there won't be more than max-errors failed executions, set MaxConcurrency to 1 so that executions proceed one at a time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociationName</i> The association name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastSuccessfulExecutionDate</i> The last date on which the association was successfully run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastExecutionDate</i> The date on which the association was last run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputLocation</i> An Amazon S3 bucket where you want to store the output details of the request.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>OutputLocation</b>: An Amazon S3 bucket where you want to store the output details of the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ScheduleExpression</i> A cron expression that specifies a schedule when the association runs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Targets</i> The instances targeted by the request. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociationId</i> The association ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> A description of the parameters for a document. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomationTargetParameterName</i> Specify the target for the association. This target is required for associations that use an Automation document and target resources by using rate controls.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentVersion</i> The document version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Overview</i> Information about the association.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Overview</b>: Information about the association.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The association status.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>: The association status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdateAssociationDate</i> The date when the association was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Date</i> The date when the association was made.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociationVersion</i> The association version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceId</i> The ID of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the Systems Manager document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AdditionalInfo</i> A user-defined string.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> The reason for the status.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The status.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Date</i> The date when the status changed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociationStatusAggregatedCount</i> Returns the number of targets for the association status. For example, if you created an association with two instances, and one of them was successful, this would return the count of instances by status.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DetailedStatus</i> A detailed status of the association.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the association. Status can be: Pending, Success, or Failed.<br/>An Amazon S3 bucket where you want to store the results of this request.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Location</b>: An Amazon S3 bucket where you want to store the results of this request.        """
        pass

    def create_association_batch(self, Entries: List[CreateAssociationBatchRequestEntry]) -> CreateAssociationBatchResult:
        """Associates the specified Systems Manager document with the specified instances or targets. When you associate a document with one or more instances using instance IDs or tags, SSM Agent running on the instance processes the document and configures the instance as specified. If you associate a document with an instance that already has an associated document, the system returns the AssociationAlreadyExists exception.

        :param self:
        :param Entries:  One or more associations.<br/>
        :return: Failed:  Information about the associations that failed.<br/>Successful:  Information about the associations that succeeded.<br/>        """
        pass

    def create_document(self, Name: str, Content: str, Attachments: List[AttachmentsSource]=None, VersionName: str=None, DocumentType: str=None, DocumentFormat: str=None, TargetType: str=None, Tags: List[Tag]=None) -> CreateDocumentResult:
        """Creates a Systems Manager document. After you create a document, you can use CreateAssociation to associate it with one or more running instances.

        :param self:
        :param Tags:  Optional metadata that you assign to a resource. Tags enable you to categorize a resource in different ways, such as by purpose, owner, or environment. For example, you might want to tag an SSM document to identify the types of targets or the environment where it will run. In this case, you could specify the following key name/value pairs: <ul> <li>  <code>Key=OS,Value=Windows</code>  </li> <li>  <code>Key=Environment,Value=Production</code>  </li> </ul> <note> To add tags to an existing SSM document, use the <a>AddTagsToResource</a> action. </note><br/>TargetType:  Specify a target type to define the kinds of resources the document can run on. For example, to run a document on EC2 instances, specify the following value: /AWS::EC2::Instance. If you specify a value of '/' the document can run on all types of resources. If you don't specify a value, the document can't run on any resources. For a list of valid resource types, see <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-template-resource-type-ref.html">AWS Resource Types Reference</a> in the <i>AWS CloudFormation User Guide</i>. <br/>DocumentFormat:  Specify the document format for the request. The document format can be either JSON or YAML. JSON is the default format.<br/>DocumentType:  The type of document to create. Valid document types include: <code>Command</code>, <code>Policy</code>, <code>Automation</code>, <code>Session</code>, and <code>Package</code>.<br/>VersionName:  An optional field specifying the version of the artifact you are creating with the document. For example, "Release 12, Update 6". This value is unique across all versions of a document, and cannot be changed.<br/>Name:  A name for the Systems Manager document. <important> Do not use the following to begin the names of documents you create. They are reserved by AWS for use as document prefixes: <ul> <li>  <code>aws</code>  </li> <li>  <code>amazon</code>  </li> <li>  <code>amzn</code>  </li> </ul> </important><br/>Attachments:  A list of key and value pairs that describe attachments to a version of a document.<br/>Content:  A valid JSON or YAML string.<br/>
        :return: DocumentDescription:  Information about the Systems Manager document.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttachmentsInformation</i> Details about the document attachments, including names, locations, sizes, etc.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The tags, or metadata, that have been applied to the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetType</i> The target type which defines the kinds of resources the document can run on. For example, /AWS::EC2::Instance. For a list of valid resource types, see <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-template-resource-type-ref.html">AWS Resource Types Reference</a> in the <i>AWS CloudFormation User Guide</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentFormat</i> The document format, either JSON or YAML.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultVersion</i> The default version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestVersion</i> The latest version of the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SchemaVersion</i> The schema version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentType</i> The type of document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlatformTypes</i> The list of OS platforms compatible with this Systems Manager document. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> A description of the parameters for a document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of the document. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentVersion</i> The document version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusInformation</i> A message returned by AWS Systems Manager that explains the <code>Status</code> value. For example, a <code>Failed</code> status might be explained by the <code>StatusInformation</code> message, "The specified S3 bucket does not exist. Verify that the URL of the S3 bucket is correct."<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the Systems Manager document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedDate</i> The date when the document was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Owner</i> The AWS user account that created the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VersionName</i> The version of the artifact associated with the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the Systems Manager document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HashType</i> The hash type of the document. Valid values include <code>Sha256</code> or <code>Sha1</code>. <note> Sha1 hashes have been deprecated. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Hash</i> The Sha256 or Sha1 hash created by the system when the document was created.  <note> Sha1 hashes have been deprecated. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Sha1</i> The SHA1 hash of the document, which you can use for verification.<br/>        """
        pass

    def create_maintenance_window(self, AllowUnassociatedTargets: MaintenanceWindowAllowUnassociatedTargets, Cutoff: int, Duration: int, Schedule: str, Name: str, Description: str=None, StartDate: str=None, EndDate: str=None, ScheduleTimezone: str=None, ClientToken: str=None, Tags: List[Tag]=None) -> CreateMaintenanceWindowResult:
        """Creates a new maintenance window.

        :param self:
        :param Tags:  Optional metadata that you assign to a resource. Tags enable you to categorize a resource in different ways, such as by purpose, owner, or environment. For example, you might want to tag a maintenance window to identify the type of tasks it will run, the types of targets, and the environment it will run in. In this case, you could specify the following key name/value pairs: <ul> <li>  <code>Key=TaskType,Value=AgentUpdate</code>  </li> <li>  <code>Key=OS,Value=Windows</code>  </li> <li>  <code>Key=Environment,Value=Production</code>  </li> </ul> <note> To add tags to an existing maintenance window, use the <a>AddTagsToResource</a> action. </note><br/>ClientToken:  User-provided idempotency token.<br/>AllowUnassociatedTargets:  Enables a maintenance window task to run on managed instances, even if you have not registered those instances as targets. If enabled, then you must specify the unregistered instances (by instance ID) when you register a task with the maintenance window. If you don't enable this option, then you must specify previously-registered targets when you register a task with the maintenance window.<br/>Cutoff:  The number of hours before the end of the maintenance window that Systems Manager stops scheduling new tasks for execution.<br/>Duration:  The duration of the maintenance window in hours.<br/>ScheduleTimezone:  The time zone that the scheduled maintenance window executions are based on, in Internet Assigned Numbers Authority (IANA) format. For example: "America/Los_Angeles", "etc/UTC", or "Asia/Seoul". For more information, see the <a href="https://www.iana.org/time-zones">Time Zone Database</a> on the IANA website.<br/>Schedule:  The schedule of the maintenance window in the form of a cron or rate expression.<br/>EndDate:  The date and time, in ISO-8601 Extended format, for when you want the maintenance window to become inactive. EndDate allows you to set a date and time in the future when the maintenance window will no longer run.<br/>StartDate:  The date and time, in ISO-8601 Extended format, for when you want the maintenance window to become active. StartDate allows you to delay activation of the maintenance window until the specified future date.<br/>Description:  An optional description for the maintenance window. We recommend specifying a description to help you organize your maintenance windows. <br/>Name:  The name of the maintenance window.<br/>
        :return: WindowId:  The ID of the created maintenance window.<br/>        """
        pass

    def create_ops_item(self, Title: str, Source: str, Description: str, OperationalData: OpsItemOperationalData=None, Notifications: List[OpsItemNotification]=None, Priority: int=None, RelatedOpsItems: List[RelatedOpsItem]=None, Tags: List[Tag]=None) -> CreateOpsItemResponse:
        """Creates a new OpsItem. You must have permission in AWS Identity and Access Management (IAM) to create a new OpsItem. For more information, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/OpsCenter-getting-started.html">Getting Started with OpsCenter</a> in the <i>AWS Systems Manager User Guide</i>. Operations engineers and IT professionals use OpsCenter to view, investigate, and remediate operational issues impacting the performance and health of their AWS resources. For more information, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/OpsCenter.html">AWS Systems Manager OpsCenter</a> in the <i>AWS Systems Manager User Guide</i>. 

        :param self:
        :param Tags:  Optional metadata that you assign to a resource. You can restrict access to OpsItems by using an inline IAM policy that specifies tags. For more information, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/OpsCenter-getting-started.html#OpsCenter-getting-started-user-permissions">Getting Started with OpsCenter</a> in the <i>AWS Systems Manager User Guide</i>. Tags use a key-value pair. For example:  <code>Key=Department,Value=Finance</code>  <note> To add tags to an existing OpsItem, use the <a>AddTagsToResource</a> action. </note><br/>Title:  A short heading that describes the nature of the OpsItem and the impacted resource.<br/>Source:  The origin of the OpsItem, such as Amazon EC2 or AWS Systems Manager.<br/>RelatedOpsItems:  One or more OpsItems that share something in common with the current OpsItems. For example, related OpsItems can include OpsItems with similar error messages, impacted resources, or statuses for the impacted resource.<br/>Priority:  The importance of this OpsItem in relation to other OpsItems in the system.<br/>Notifications:  The Amazon Resource Name (ARN) of an SNS topic where notifications are sent when this OpsItem is edited or changed.<br/>OperationalData:  Operational data is custom data that provides useful reference details about the OpsItem. For example, you can specify log files, error strings, license keys, troubleshooting tips, or other relevant data. You enter operational data as key-value pairs. The key has a maximum length of 128 characters. The value has a maximum size of 20 KB. <important> Operational data keys <i>can't</i> begin with the following: amazon, aws, amzn, ssm, /amazon, /aws, /amzn, /ssm. </important> You can choose to make the data searchable by other users in the account or you can restrict search access. Searchable data means that all users with access to the OpsItem Overview page (as provided by the <a>DescribeOpsItems</a> API action) can view and search on the specified data. Operational data that is not searchable is only viewable by users who have access to the OpsItem (as provided by the <a>GetOpsItem</a> API action). Use the <code>/aws/resources</code> key in OperationalData to specify a related resource in the request. Use the <code>/aws/automations</code> key in OperationalData to associate an Automation runbook with the OpsItem. To view AWS CLI example commands that use these keys, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/OpsCenter-creating-OpsItems.html#OpsCenter-manually-create-OpsItems">Creating OpsItems Manually</a> in the <i>AWS Systems Manager User Guide</i>.<br/>Description:  Information about the OpsItem. <br/>
        :return: OpsItemId:  The ID of the OpsItem.<br/>        """
        pass

    def create_patch_baseline(self, Name: str, OperatingSystem: str=None, GlobalFilters: PatchFilterGroup=None, ApprovalRules: PatchRuleGroup=None, ApprovedPatches: List[PatchId]=None, ApprovedPatchesComplianceLevel: str=None, ApprovedPatchesEnableNonSecurity: Boolean=None, RejectedPatches: List[PatchId]=None, RejectedPatchesAction: str=None, Description: str=None, Sources: List[PatchSource]=None, ClientToken: str=None, Tags: List[Tag]=None) -> CreatePatchBaselineResult:
        """Creates a patch baseline. <note> For information about valid key and value pairs in <code>PatchFilters</code> for each supported operating system type, see <a href="http://docs.aws.amazon.com/systems-manager/latest/APIReference/API_PatchFilter.html">PatchFilter</a>. </note>

        :param self:
        :param Tags:  Optional metadata that you assign to a resource. Tags enable you to categorize a resource in different ways, such as by purpose, owner, or environment. For example, you might want to tag a patch baseline to identify the severity level of patches it specifies and the operating system family it applies to. In this case, you could specify the following key name/value pairs: <ul> <li>  <code>Key=PatchSeverity,Value=Critical</code>  </li> <li>  <code>Key=OS,Value=Windows</code>  </li> </ul> <note> To add tags to an existing patch baseline, use the <a>AddTagsToResource</a> action. </note><br/>ClientToken:  User-provided idempotency token.<br/>Sources:  Information about the patches to use to update the instances, including target operating systems and source repositories. Applies to Linux instances only.<br/>Description:  A description of the patch baseline.<br/>RejectedPatchesAction:  The action for Patch Manager to take on patches included in the RejectedPackages list. <ul> <li>  <b>ALLOW_AS_DEPENDENCY</b>: A package in the Rejected patches list is installed only if it is a dependency of another package. It is considered compliant with the patch baseline, and its status is reported as <i>InstalledOther</i>. This is the default action if no option is specified. </li> <li>  <b>BLOCK</b>: Packages in the RejectedPatches list, and packages that include them as dependencies, are not installed under any circumstances. If a package was installed before it was added to the Rejected patches list, it is considered non-compliant with the patch baseline, and its status is reported as <i>InstalledRejected</i>. </li> </ul><br/>RejectedPatches:  A list of explicitly rejected patches for the baseline. For information about accepted formats for lists of approved patches and rejected patches, see <a href="https://docs.aws.amazon.com/systems-manager/latest/userguide/patch-manager-approved-rejected-package-name-formats.html">Package Name Formats for Approved and Rejected Patch Lists</a> in the <i>AWS Systems Manager User Guide</i>.<br/>ApprovedPatchesEnableNonSecurity:  Indicates whether the list of approved patches includes non-security updates that should be applied to the instances. The default value is 'false'. Applies to Linux instances only.<br/>ApprovedPatchesComplianceLevel:  Defines the compliance level for approved patches. This means that if an approved patch is reported as missing, this is the severity of the compliance violation. The default value is UNSPECIFIED.<br/>ApprovedPatches:  A list of explicitly approved patches for the baseline. For information about accepted formats for lists of approved patches and rejected patches, see <a href="https://docs.aws.amazon.com/systems-manager/latest/userguide/patch-manager-approved-rejected-package-name-formats.html">Package Name Formats for Approved and Rejected Patch Lists</a> in the <i>AWS Systems Manager User Guide</i>.<br/>ApprovalRules:  A set of rules used to include patches in the baseline.:<br/>GlobalFilters:  A set of global filters used to include patches in the baseline.:<br/>Name:  The name of the patch baseline.<br/>OperatingSystem:  Defines the operating system the patch baseline applies to. The Default value is WINDOWS.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PatchFilters</i> The set of patch filters that make up the group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PatchRules</i> The rules that make up the rule group.<br/>
        :return: BaselineId:  The ID of the created patch baseline.<br/>        """
        pass

    def create_resource_data_sync(self, S3Destination: ResourceDataSyncS3Destination, SyncName: str) -> CreateResourceDataSyncResult:
        """Creates a resource data sync configuration to a single bucket in Amazon S3. This is an asynchronous operation that returns immediately. After a successful initial sync is completed, the system continuously syncs data to the Amazon S3 bucket. To check the status of the sync, use the <a>ListResourceDataSync</a>. By default, data is not encrypted in Amazon S3. We strongly recommend that you enable encryption in Amazon S3 to ensure secure data storage. We also recommend that you secure access to the Amazon S3 bucket by creating a restrictive bucket policy. For more information, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/sysman-inventory-datasync.html">Configuring Resource Data Sync for Inventory</a> in the <i>AWS Systems Manager User Guide</i>.

        :param self:
        :param S3Destination:  Amazon S3 configuration details for the sync.:<br/>SyncName:  A name for the configuration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AWSKMSKeyARN</i> The ARN of an encryption key for a destination in Amazon S3. Must belong to the same Region as the destination Amazon S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Region</i> The AWS Region with the Amazon S3 bucket targeted by the Resource Data Sync.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SyncFormat</i> A supported sync format. The following format is currently supported: JsonSerDe<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Prefix</i> An Amazon S3 prefix for the bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>BucketName</i> The name of the Amazon S3 bucket where the aggregated data is stored.<br/>
        :return:         """
        pass

    def delete_activation(self, ActivationId: str) -> DeleteActivationResult:
        """Deletes an activation. You are not required to delete an activation. If you delete an activation, you can no longer use it to register additional managed instances. Deleting an activation does not de-register managed instances. You must manually de-register managed instances.

        :param self:
        :param ActivationId:  The ID of the activation that you want to delete.<br/>
        :return:         """
        pass

    def delete_association(self, Name: str=None, InstanceId: str=None, AssociationId: str=None) -> DeleteAssociationResult:
        """Disassociates the specified Systems Manager document from the specified instance. When you disassociate a document from an instance, it does not change the configuration of the instance. To change the configuration state of an instance after you disassociate a document, you must create a new document with the desired configuration and associate it with the instance.

        :param self:
        :param AssociationId:  The association ID that you want to delete.<br/>InstanceId:  The ID of the instance.<br/>Name:  The name of the Systems Manager document.<br/>
        :return:         """
        pass

    def delete_document(self, Name: str, DocumentVersion: str=None, VersionName: str=None) -> DeleteDocumentResult:
        """Deletes the Systems Manager document and all instance associations to the document. Before you delete the document, we recommend that you use <a>DeleteAssociation</a> to disassociate all instances that are associated with the document.

        :param self:
        :param VersionName:  (Optional) The version name of the document that you want to delete. If not provided, all versions of the document are deleted.<br/>DocumentVersion:  (Optional) The version of the document that you want to delete. If not provided, all versions of the document are deleted.<br/>Name:  The name of the document.<br/>
        :return:         """
        pass

    def delete_inventory(self, TypeName: str, SchemaDeleteOption: str=None, DryRun: DryRun=None, ClientToken: str=None) -> DeleteInventoryResult:
        """Delete a custom inventory type, or the data associated with a custom Inventory type. Deleting a custom inventory type is also referred to as deleting a custom inventory schema.

        :param self:
        :param ClientToken:  User-provided idempotency token.<br/>DryRun:  Use this option to view a summary of the deletion request without deleting any data or the data type. This option is useful when you only want to understand what will be deleted. Once you validate that the data to be deleted is what you intend to delete, you can run the same command without specifying the <code>DryRun</code> option.<br/>SchemaDeleteOption:  Use the <code>SchemaDeleteOption</code> to delete a custom inventory type (schema). If you don't choose this option, the system only deletes existing inventory data associated with the custom inventory type. Choose one of the following options: DisableSchema: If you choose this option, the system ignores all inventory data for the specified version, and any earlier versions. To enable this schema again, you must call the <code>PutInventory</code> action for a version greater than the disabled version. DeleteSchema: This option deletes the specified custom type from the Inventory service. You can recreate the schema later, if you want.<br/>TypeName:  The name of the custom inventory type for which you want to delete either all previously collected data, or the inventory type itself. <br/>
        :return: DeletionSummary:  A summary of the delete operation. For more information about this summary, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/sysman-inventory-delete.html#sysman-inventory-delete-summary">Understanding the Delete Inventory Summary</a> in the <i>AWS Systems Manager User Guide</i>.:<br/>TypeName:  The name of the inventory data type specified in the request.<br/>DeletionId:  Every <code>DeleteInventory</code> action is assigned a unique ID. This option returns a unique ID. You can use this ID to query the status of a delete operation. This option is useful for ensuring that a delete operation has completed before you begin other actions. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SummaryItems</i> A list of counts and versions for deleted items.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RemainingCount</i> Remaining number of items to delete.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalCount</i> The total number of items to delete. This count does not change during the delete operation.<br/>        """
        pass

    def delete_maintenance_window(self, WindowId: str) -> DeleteMaintenanceWindowResult:
        """Deletes a maintenance window.

        :param self:
        :param WindowId:  The ID of the maintenance window to delete.<br/>
        :return: WindowId:  The ID of the deleted maintenance window.<br/>        """
        pass

    def delete_parameter(self, Name: str) -> DeleteParameterResult:
        """Delete a parameter from the system.

        :param self:
        :param Name:  The name of the parameter to delete.<br/>
        :return:         """
        pass

    def delete_parameters(self, Names: List[PSParameterName]) -> DeleteParametersResult:
        """Delete a list of parameters. This API is used to delete parameters by using the Amazon EC2 console.

        :param self:
        :param Names:  The names of the parameters to delete.<br/>
        :return: InvalidParameters:  The names of parameters that weren't deleted because the parameters are not valid.<br/>DeletedParameters:  The names of the deleted parameters.<br/>        """
        pass

    def delete_patch_baseline(self, BaselineId: str) -> DeletePatchBaselineResult:
        """Deletes a patch baseline.

        :param self:
        :param BaselineId:  The ID of the patch baseline to delete.<br/>
        :return: BaselineId:  The ID of the deleted patch baseline.<br/>        """
        pass

    def delete_resource_data_sync(self, SyncName: str) -> DeleteResourceDataSyncResult:
        """Deletes a Resource Data Sync configuration. After the configuration is deleted, changes to inventory data on managed instances are no longer synced with the target Amazon S3 bucket. Deleting a sync configuration does not delete data in the target Amazon S3 bucket.

        :param self:
        :param SyncName:  The name of the configuration to delete.<br/>
        :return:         """
        pass

    def deregister_managed_instance(self, InstanceId: str) -> DeregisterManagedInstanceResult:
        """Removes the server or virtual machine from the list of registered servers. You can reregister the instance again at any time. If you don't plan to use Run Command on the server, we suggest uninstalling SSM Agent first.

        :param self:
        :param InstanceId:  The ID assigned to the managed instance when you registered it using the activation process. <br/>
        :return:         """
        pass

    def deregister_patch_baseline_for_patch_group(self, PatchGroup: str, BaselineId: str) -> DeregisterPatchBaselineForPatchGroupResult:
        """Removes a patch group from a patch baseline.

        :param self:
        :param PatchGroup:  The name of the patch group that should be deregistered from the patch baseline.<br/>BaselineId:  The ID of the patch baseline to deregister the patch group from.<br/>
        :return: PatchGroup:  The name of the patch group deregistered from the patch baseline.<br/>BaselineId:  The ID of the patch baseline the patch group was deregistered from.<br/>        """
        pass

    def deregister_target_from_maintenance_window(self, WindowTargetId: str, WindowId: str, Safe: Boolean=None) -> DeregisterTargetFromMaintenanceWindowResult:
        """Removes a target from a maintenance window.

        :param self:
        :param Safe:  The system checks if the target is being referenced by a task. If the target is being referenced, the system returns an error and does not deregister the target from the maintenance window.<br/>WindowTargetId:  The ID of the target definition to remove.<br/>WindowId:  The ID of the maintenance window the target should be removed from.<br/>
        :return: WindowTargetId:  The ID of the removed target definition.<br/>WindowId:  The ID of the maintenance window the target was removed from.<br/>        """
        pass

    def deregister_task_from_maintenance_window(self, WindowTaskId: str, WindowId: str) -> DeregisterTaskFromMaintenanceWindowResult:
        """Removes a task from a maintenance window.

        :param self:
        :param WindowTaskId:  The ID of the task to remove from the maintenance window.<br/>WindowId:  The ID of the maintenance window the task should be removed from.<br/>
        :return: WindowTaskId:  The ID of the task removed from the maintenance window.<br/>WindowId:  The ID of the maintenance window the task was removed from.<br/>        """
        pass

    def describe_activations(self, Filters: List[DescribeActivationsFilter]=None, MaxResults: int=None, NextToken: str=None) -> DescribeActivationsResult:
        """Details about the activation, including: the date and time the activation was created, the expiration date, the IAM role assigned to the instances in the activation, and the number of instances activated by this registration.

        :param self:
        :param NextToken:  A token to start the list. Use this token to get the next set of results. <br/>MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>Filters:  A filter to view information about your activations.<br/>
        :return: NextToken:  The token for the next set of items to return. Use this token to get the next set of results. <br/>ActivationList:  A list of activations for your AWS account.<br/>        """
        pass

    def describe_association(self, Name: str=None, InstanceId: str=None, AssociationId: str=None, AssociationVersion: str=None) -> DescribeAssociationResult:
        """Describes the association for the specified target or instance. If you created the association by using the <code>Targets</code> parameter, then you must retrieve the association by using the association ID. If you created the association by specifying an instance ID and a Systems Manager document, then you retrieve the association by specifying the document name and the instance ID. 

        :param self:
        :param AssociationVersion:  Specify the association version to retrieve. To view the latest version, either specify <code>$LATEST</code> for this parameter, or omit this parameter. To view a list of all associations for an instance, use <a>ListAssociations</a>. To get a list of versions for a specific association, use <a>ListAssociationVersions</a>. <br/>AssociationId:  The association ID for which you want information.<br/>InstanceId:  The instance ID.<br/>Name:  The name of the Systems Manager document.<br/>
        :return: AssociationDescription:  Information about the association.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ComplianceSeverity</i> The severity level that is assigned to the association.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxConcurrency</i> The maximum number of targets allowed to run the association at the same time. You can specify a number, for example 10, or a percentage of the target set, for example 10%. The default value is 100%, which means all targets run the association at the same time. If a new instance starts and attempts to run an association while Systems Manager is running MaxConcurrency associations, the association is allowed to run. During the next association interval, the new instance will process its association within the limit specified for MaxConcurrency.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxErrors</i> The number of errors that are allowed before the system stops sending requests to run the association on additional targets. You can specify either an absolute number of errors, for example 10, or a percentage of the target set, for example 10%. If you specify 3, for example, the system stops sending requests when the fourth error is received. If you specify 0, then the system stops sending requests after the first error is returned. If you run an association on 50 instances and set MaxError to 10%, then the system stops sending the request when the sixth error is received. Executions that are already running an association when MaxErrors is reached are allowed to complete, but some of these executions may fail as well. If you need to ensure that there won't be more than max-errors failed executions, set MaxConcurrency to 1 so that executions proceed one at a time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociationName</i> The association name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastSuccessfulExecutionDate</i> The last date on which the association was successfully run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastExecutionDate</i> The date on which the association was last run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputLocation</i> An Amazon S3 bucket where you want to store the output details of the request.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>OutputLocation</b>: An Amazon S3 bucket where you want to store the output details of the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ScheduleExpression</i> A cron expression that specifies a schedule when the association runs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Targets</i> The instances targeted by the request. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociationId</i> The association ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> A description of the parameters for a document. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomationTargetParameterName</i> Specify the target for the association. This target is required for associations that use an Automation document and target resources by using rate controls.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentVersion</i> The document version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Overview</i> Information about the association.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Overview</b>: Information about the association.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The association status.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>: The association status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdateAssociationDate</i> The date when the association was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Date</i> The date when the association was made.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociationVersion</i> The association version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceId</i> The ID of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the Systems Manager document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AdditionalInfo</i> A user-defined string.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> The reason for the status.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The status.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Date</i> The date when the status changed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociationStatusAggregatedCount</i> Returns the number of targets for the association status. For example, if you created an association with two instances, and one of them was successful, this would return the count of instances by status.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DetailedStatus</i> A detailed status of the association.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the association. Status can be: Pending, Success, or Failed.<br/>An Amazon S3 bucket where you want to store the results of this request.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Location</b>: An Amazon S3 bucket where you want to store the results of this request.        """
        pass

    def describe_association_execution_targets(self, ExecutionId: str, AssociationId: str, Filters: List[AssociationExecutionTargetsFilter]=None, MaxResults: int=None, NextToken: str=None) -> DescribeAssociationExecutionTargetsResult:
        """Use this API action to view information about a specific execution of a specific association.

        :param self:
        :param NextToken:  A token to start the list. Use this token to get the next set of results. <br/>MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>Filters:  Filters for the request. You can specify the following filters and values. Status (EQUAL) ResourceId (EQUAL) ResourceType (EQUAL)<br/>ExecutionId:  The execution ID for which you want to view details.<br/>AssociationId:  The association ID that includes the execution for which you want to view details.<br/>
        :return: NextToken:  The token for the next set of items to return. Use this token to get the next set of results.<br/>AssociationExecutionTargets:  Information about the execution.<br/>        """
        pass

    def describe_association_executions(self, AssociationId: str, Filters: List[AssociationExecutionFilter]=None, MaxResults: int=None, NextToken: str=None) -> DescribeAssociationExecutionsResult:
        """Use this API action to view all executions for a specific association ID. 

        :param self:
        :param NextToken:  A token to start the list. Use this token to get the next set of results. <br/>MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>Filters:  Filters for the request. You can specify the following filters and values. ExecutionId (EQUAL) Status (EQUAL) CreatedTime (EQUAL, GREATER_THAN, LESS_THAN)<br/>AssociationId:  The association ID for which you want to view execution history details.<br/>
        :return: NextToken:  The token for the next set of items to return. Use this token to get the next set of results.<br/>AssociationExecutions:  A list of the executions for the specified association ID.<br/>        """
        pass

    def describe_automation_executions(self, Filters: List[AutomationExecutionFilter]=None, MaxResults: int=None, NextToken: str=None) -> DescribeAutomationExecutionsResult:
        """Provides details about all active and terminated Automation executions.

        :param self:
        :param NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>Filters:  Filters used to limit the scope of executions that are requested.<br/>
        :return: NextToken:  The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.<br/>AutomationExecutionMetadataList:  The list of details about each automation execution which has occurred which matches the filter specification, if any.<br/>        """
        pass

    def describe_automation_step_executions(self, AutomationExecutionId: str, Filters: List[StepExecutionFilter]=None, NextToken: str=None, MaxResults: int=None, ReverseOrder: Boolean=None) -> DescribeAutomationStepExecutionsResult:
        """Information about all active and terminated step executions in an Automation workflow.

        :param self:
        :param ReverseOrder:  A boolean that indicates whether to list step executions in reverse order by start time. The default value is false.<br/>MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>Filters:  One or more filters to limit the number of step executions returned by the request.<br/>AutomationExecutionId:  The Automation execution ID for which you want step execution descriptions.<br/>
        :return: NextToken:  The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.<br/>StepExecutions:  A list of details about the current state of all steps that make up an execution.<br/>        """
        pass

    def describe_available_patches(self, Filters: List[PatchOrchestratorFilter]=None, MaxResults: int=None, NextToken: str=None) -> DescribeAvailablePatchesResult:
        """Lists all patches that could possibly be included in a patch baseline.

        :param self:
        :param NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>MaxResults:  The maximum number of patches to return (per page).<br/>Filters:  Filters used to scope down the returned patches.<br/>
        :return: NextToken:  The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.<br/>Patches:  An array of patches. Each entry in the array is a patch structure.<br/>        """
        pass

    def describe_document(self, Name: str, DocumentVersion: str=None, VersionName: str=None) -> DescribeDocumentResult:
        """Describes the specified Systems Manager document.

        :param self:
        :param VersionName:  An optional field specifying the version of the artifact associated with the document. For example, "Release 12, Update 6". This value is unique across all versions of a document, and cannot be changed.<br/>DocumentVersion:  The document version for which you want information. Can be a specific version or the default version.<br/>Name:  The name of the Systems Manager document.<br/>
        :return: Document:  Information about the Systems Manager document.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttachmentsInformation</i> Details about the document attachments, including names, locations, sizes, etc.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The tags, or metadata, that have been applied to the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetType</i> The target type which defines the kinds of resources the document can run on. For example, /AWS::EC2::Instance. For a list of valid resource types, see <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-template-resource-type-ref.html">AWS Resource Types Reference</a> in the <i>AWS CloudFormation User Guide</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentFormat</i> The document format, either JSON or YAML.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultVersion</i> The default version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestVersion</i> The latest version of the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SchemaVersion</i> The schema version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentType</i> The type of document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlatformTypes</i> The list of OS platforms compatible with this Systems Manager document. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> A description of the parameters for a document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of the document. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentVersion</i> The document version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusInformation</i> A message returned by AWS Systems Manager that explains the <code>Status</code> value. For example, a <code>Failed</code> status might be explained by the <code>StatusInformation</code> message, "The specified S3 bucket does not exist. Verify that the URL of the S3 bucket is correct."<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the Systems Manager document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedDate</i> The date when the document was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Owner</i> The AWS user account that created the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VersionName</i> The version of the artifact associated with the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the Systems Manager document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HashType</i> The hash type of the document. Valid values include <code>Sha256</code> or <code>Sha1</code>. <note> Sha1 hashes have been deprecated. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Hash</i> The Sha256 or Sha1 hash created by the system when the document was created.  <note> Sha1 hashes have been deprecated. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Sha1</i> The SHA1 hash of the document, which you can use for verification.<br/>        """
        pass

    def describe_document_permission(self, PermissionType: str, Name: str) -> DescribeDocumentPermissionResponse:
        """Describes the permissions for a Systems Manager document. If you created the document, you are the owner. If a document is shared, it can either be shared privately (by specifying a user's AWS account ID) or publicly (<i>All</i>). 

        :param self:
        :param PermissionType:  The permission type for the document. The permission type can be <i>Share</i>.<br/>Name:  The name of the document for which you are the owner.<br/>
        :return: AccountIds:  The account IDs that have permission to use this document. The ID can be either an AWS account or <i>All</i>.<br/>        """
        pass

    def describe_effective_instance_associations(self, InstanceId: str, MaxResults: int=None, NextToken: str=None) -> DescribeEffectiveInstanceAssociationsResult:
        """All associations for the instance(s).

        :param self:
        :param NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>InstanceId:  The instance ID for which you want to view all associations.<br/>
        :return: NextToken:  The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.<br/>Associations:  The associations for the requested instance.<br/>        """
        pass

    def describe_effective_patches_for_patch_baseline(self, BaselineId: str, MaxResults: int=None, NextToken: str=None) -> DescribeEffectivePatchesForPatchBaselineResult:
        """Retrieves the current effective patches (the patch and the approval state) for the specified patch baseline. Note that this API applies only to Windows patch baselines.

        :param self:
        :param NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>MaxResults:  The maximum number of patches to return (per page).<br/>BaselineId:  The ID of the patch baseline to retrieve the effective patches for.<br/>
        :return: NextToken:  The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.<br/>EffectivePatches:  An array of patches and patch status.<br/>        """
        pass

    def describe_instance_associations_status(self, InstanceId: str, MaxResults: int=None, NextToken: str=None) -> DescribeInstanceAssociationsStatusResult:
        """The status of the associations for the instance(s).

        :param self:
        :param NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>InstanceId:  The instance IDs for which you want association status information.<br/>
        :return: NextToken:  The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.<br/>InstanceAssociationStatusInfos:  Status information about the association.<br/>        """
        pass

    def describe_instance_information(self, InstanceInformationFilterList: List[InstanceInformationFilter]=None, Filters: List[InstanceInformationStringFilter]=None, MaxResults: int=None, NextToken: str=None) -> DescribeInstanceInformationResult:
        """Describes one or more of your instances. You can use this to get information about instances like the operating system platform, the SSM Agent version (Linux), status etc. If you specify one or more instance IDs, it returns information for those instances. If you do not specify instance IDs, it returns information for all your instances. If you specify an instance ID that is not valid or an instance that you do not own, you receive an error.  <note> The IamRole field for this API action is the Amazon Identity and Access Management (IAM) role assigned to on-premises instances. This call does not return the IAM role for Amazon EC2 instances. </note>

        :param self:
        :param NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results. <br/>Filters:  One or more filters. Use a filter to return a more specific list of instances. You can filter on Amazon EC2 tag. Specify tags by using a key-value mapping.<br/>InstanceInformationFilterList:  This is a legacy method. We recommend that you don't use this method. Instead, use the <a>InstanceInformationFilter</a> action. The <code>InstanceInformationFilter</code> action enables you to return instance information by using tags that are specified as a key-value mapping.  If you do use this method, then you can't use the <code>InstanceInformationFilter</code> action. Using this method and the <code>InstanceInformationFilter</code> action causes an exception error. <br/>
        :return: NextToken:  The token to use when requesting the next set of items. If there are no additional items to return, the string is empty. <br/>InstanceInformationList:  The instance information list.<br/>        """
        pass

    def describe_instance_patch_states(self, InstanceIds: List[InstanceId], NextToken: str=None, MaxResults: int=None) -> DescribeInstancePatchStatesResult:
        """Retrieves the high-level patch state of one or more instances.

        :param self:
        :param MaxResults:  The maximum number of instances to return (per page).<br/>NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>InstanceIds:  The ID of the instance whose patch state information should be retrieved.<br/>
        :return: NextToken:  The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.<br/>InstancePatchStates:  The high-level patch state for the requested instances.<br/>        """
        pass

    def describe_instance_patch_states_for_patch_group(self, PatchGroup: str, Filters: List[InstancePatchStateFilter]=None, NextToken: str=None, MaxResults: int=None) -> DescribeInstancePatchStatesForPatchGroupResult:
        """Retrieves the high-level patch state for the instances in the specified patch group.

        :param self:
        :param MaxResults:  The maximum number of patches to return (per page).<br/>NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>Filters:  Each entry in the array is a structure containing: Key (string between 1 and 200 characters)  Values (array containing a single string)  Type (string "Equal", "NotEqual", "LessThan", "GreaterThan")<br/>PatchGroup:  The name of the patch group for which the patch state information should be retrieved.<br/>
        :return: NextToken:  The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.<br/>InstancePatchStates:  The high-level patch state for the requested instances. <br/>        """
        pass

    def describe_instance_patches(self, InstanceId: str, Filters: List[PatchOrchestratorFilter]=None, NextToken: str=None, MaxResults: int=None) -> DescribeInstancePatchesResult:
        """Retrieves information about the patches on the specified instance and their state relative to the patch baseline being used for the instance.

        :param self:
        :param MaxResults:  The maximum number of patches to return (per page).<br/>NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>Filters:  Each entry in the array is a structure containing: Key (string, between 1 and 128 characters) Values (array of strings, each string between 1 and 256 characters)<br/>InstanceId:  The ID of the instance whose patch state information should be retrieved.<br/>
        :return: NextToken:  The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.<br/>Patches:  Each entry in the array is a structure containing: Title (string) KBId (string) Classification (string) Severity (string) State (string, such as "INSTALLED" or "FAILED") InstalledTime (DateTime) InstalledBy (string)<br/>        """
        pass

    def describe_inventory_deletions(self, DeletionId: str=None, NextToken: str=None, MaxResults: int=None) -> DescribeInventoryDeletionsResult:
        """Describes a specific delete inventory operation.

        :param self:
        :param MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>NextToken:  A token to start the list. Use this token to get the next set of results. <br/>DeletionId:  Specify the delete inventory ID for which you want information. This ID was returned by the <code>DeleteInventory</code> action.<br/>
        :return: NextToken:  The token for the next set of items to return. Use this token to get the next set of results.<br/>InventoryDeletions:  A list of status items for deleted inventory.<br/>        """
        pass

    def describe_maintenance_window_execution_task_invocations(self, TaskId: str, WindowExecutionId: str, Filters: List[MaintenanceWindowFilter]=None, MaxResults: int=None, NextToken: str=None) -> DescribeMaintenanceWindowExecutionTaskInvocationsResult:
        """Retrieves the individual task executions (one per target) for a particular task run as part of a maintenance window execution.

        :param self:
        :param NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>Filters:  Optional filters used to scope down the returned task invocations. The supported filter key is STATUS with the corresponding values PENDING, IN_PROGRESS, SUCCESS, FAILED, TIMED_OUT, CANCELLING, and CANCELLED.<br/>TaskId:  The ID of the specific task in the maintenance window task that should be retrieved.<br/>WindowExecutionId:  The ID of the maintenance window execution the task is part of.<br/>
        :return: NextToken:  The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.<br/>WindowExecutionTaskInvocationIdentities:  Information about the task invocation results per invocation.<br/>        """
        pass

    def describe_maintenance_window_execution_tasks(self, WindowExecutionId: str, Filters: List[MaintenanceWindowFilter]=None, MaxResults: int=None, NextToken: str=None) -> DescribeMaintenanceWindowExecutionTasksResult:
        """For a given maintenance window execution, lists the tasks that were run.

        :param self:
        :param NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>Filters:  Optional filters used to scope down the returned tasks. The supported filter key is STATUS with the corresponding values PENDING, IN_PROGRESS, SUCCESS, FAILED, TIMED_OUT, CANCELLING, and CANCELLED. <br/>WindowExecutionId:  The ID of the maintenance window execution whose task executions should be retrieved.<br/>
        :return: NextToken:  The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.<br/>WindowExecutionTaskIdentities:  Information about the task executions.<br/>        """
        pass

    def describe_maintenance_window_executions(self, WindowId: str, Filters: List[MaintenanceWindowFilter]=None, MaxResults: int=None, NextToken: str=None) -> DescribeMaintenanceWindowExecutionsResult:
        """Lists the executions of a maintenance window. This includes information about when the maintenance window was scheduled to be active, and information about tasks registered and run with the maintenance window.

        :param self:
        :param NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>Filters:  Each entry in the array is a structure containing: Key (string, between 1 and 128 characters) Values (array of strings, each string is between 1 and 256 characters) The supported Keys are ExecutedBefore and ExecutedAfter with the value being a date/time string such as 2016-11-04T05:00:00Z.<br/>WindowId:  The ID of the maintenance window whose executions should be retrieved.<br/>
        :return: NextToken:  The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.<br/>WindowExecutions:  Information about the maintenance window executions.<br/>        """
        pass

    def describe_maintenance_window_schedule(self, WindowId: str=None, Targets: List[Target]=None, ResourceType: str=None, Filters: List[PatchOrchestratorFilter]=None, MaxResults: int=None, NextToken: str=None) -> DescribeMaintenanceWindowScheduleResult:
        """Retrieves information about upcoming executions of a maintenance window.

        :param self:
        :param NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>Filters:  Filters used to limit the range of results. For example, you can limit maintenance window executions to only those scheduled before or after a certain date and time.<br/>ResourceType:  The type of resource you want to retrieve information about. For example, "INSTANCE".<br/>Targets:  The instance ID or key/value pair to retrieve information about.<br/>WindowId:  The ID of the maintenance window to retrieve information about.<br/>
        :return: NextToken:  The token for the next set of items to return. (You use this token in the next call.)<br/>ScheduledWindowExecutions:  Information about maintenance window executions scheduled for the specified time range.<br/>        """
        pass

    def describe_maintenance_window_targets(self, WindowId: str, Filters: List[MaintenanceWindowFilter]=None, MaxResults: int=None, NextToken: str=None) -> DescribeMaintenanceWindowTargetsResult:
        """Lists the targets registered with the maintenance window.

        :param self:
        :param NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>Filters:  Optional filters that can be used to narrow down the scope of the returned window targets. The supported filter keys are Type, WindowTargetId and OwnerInformation.<br/>WindowId:  The ID of the maintenance window whose targets should be retrieved.<br/>
        :return: NextToken:  The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.<br/>Targets:  Information about the targets in the maintenance window.<br/>        """
        pass

    def describe_maintenance_window_tasks(self, WindowId: str, Filters: List[MaintenanceWindowFilter]=None, MaxResults: int=None, NextToken: str=None) -> DescribeMaintenanceWindowTasksResult:
        """Lists the tasks in a maintenance window.

        :param self:
        :param NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>Filters:  Optional filters used to narrow down the scope of the returned tasks. The supported filter keys are WindowTaskId, TaskArn, Priority, and TaskType.<br/>WindowId:  The ID of the maintenance window whose tasks should be retrieved.<br/>
        :return: NextToken:  The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.<br/>Tasks:  Information about the tasks in the maintenance window.<br/>        """
        pass

    def describe_maintenance_windows(self, Filters: List[MaintenanceWindowFilter]=None, MaxResults: int=None, NextToken: str=None) -> DescribeMaintenanceWindowsResult:
        """Retrieves the maintenance windows in an AWS account.

        :param self:
        :param NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>Filters:  Optional filters used to narrow down the scope of the returned maintenance windows. Supported filter keys are <b>Name</b> and <b>Enabled</b>.<br/>
        :return: NextToken:  The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.<br/>WindowIdentities:  Information about the maintenance windows.<br/>        """
        pass

    def describe_maintenance_windows_for_target(self, ResourceType: str, Targets: List[Target], MaxResults: int=None, NextToken: str=None) -> DescribeMaintenanceWindowsForTargetResult:
        """Retrieves information about the maintenance window targets or tasks that an instance is associated with.

        :param self:
        :param NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>ResourceType:  The type of resource you want to retrieve information about. For example, "INSTANCE".<br/>Targets:  The instance ID or key/value pair to retrieve information about.<br/>
        :return: NextToken:  The token for the next set of items to return. (You use this token in the next call.)<br/>WindowIdentities:  Information about the maintenance window targets and tasks an instance is associated with.<br/>        """
        pass

    def describe_ops_items(self, OpsItemFilters: List[OpsItemFilter]=None, MaxResults: int=None, NextToken: str=None) -> DescribeOpsItemsResponse:
        """Query a set of OpsItems. You must have permission in AWS Identity and Access Management (IAM) to query a list of OpsItems. For more information, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/OpsCenter-getting-started.html">Getting Started with OpsCenter</a> in the <i>AWS Systems Manager User Guide</i>. Operations engineers and IT professionals use OpsCenter to view, investigate, and remediate operational issues impacting the performance and health of their AWS resources. For more information, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/OpsCenter.html">AWS Systems Manager OpsCenter</a> in the <i>AWS Systems Manager User Guide</i>. 

        :param self:
        :param NextToken:  A token to start the list. Use this token to get the next set of results.<br/>MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>OpsItemFilters:  One or more filters to limit the reponse. <ul> <li> Key: CreatedTime Operations: GreaterThan, LessThan </li> <li> Key: LastModifiedBy Operations: Contains, Equals </li> <li> Key: LastModifiedTime Operations: GreaterThan, LessThan </li> <li> Key: Priority Operations: Equals </li> <li> Key: Source Operations: Contains, Equals </li> <li> Key: Status Operations: Equals </li> <li> Key: Title Operations: Contains </li> <li> Key: OperationalData* Operations: Equals </li> <li> Key: OperationalDataKey Operations: Equals </li> <li> Key: OperationalDataValue Operations: Equals, Contains </li> <li> Key: OpsItemId Operations: Equals </li> <li> Key: ResourceId Operations: Contains </li> <li> Key: AutomationId Operations: Equals </li> </ul> *If you filter the response by using the OperationalData operator, specify a key-value pair by using the following JSON format: {"key":"key_name","value":"a_value"}<br/>
        :return: OpsItemSummaries:  A list of OpsItems.<br/>NextToken:  The token for the next set of items to return. Use this token to get the next set of results.<br/>        """
        pass

    def describe_parameters(self, Filters: List[ParametersFilter]=None, ParameterFilters: List[ParameterStringFilter]=None, MaxResults: int=None, NextToken: str=None) -> DescribeParametersResult:
        """Get information about a parameter. Request results are returned on a best-effort basis. If you specify <code>MaxResults</code> in the request, the response includes information up to the limit specified. The number of items returned, however, can be between zero and the value of <code>MaxResults</code>. If the service reaches an internal limit while processing the results, it stops the operation and returns the matching values up to that point and a <code>NextToken</code>. You can specify the <code>NextToken</code> in a subsequent call to get the next set of results.

        :param self:
        :param NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>ParameterFilters:  Filters to limit the request results.<br/>Filters:  One or more filters. Use a filter to return a more specific list of results.<br/>
        :return: NextToken:  The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.<br/>Parameters:  Parameters returned by the request.<br/>        """
        pass

    def describe_patch_baselines(self, Filters: List[PatchOrchestratorFilter]=None, MaxResults: int=None, NextToken: str=None) -> DescribePatchBaselinesResult:
        """Lists the patch baselines in your AWS account.

        :param self:
        :param NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>MaxResults:  The maximum number of patch baselines to return (per page).<br/>Filters:  Each element in the array is a structure containing:  Key: (string, "NAME_PREFIX" or "OWNER") Value: (array of strings, exactly 1 entry, between 1 and 255 characters)<br/>
        :return: NextToken:  The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.<br/>BaselineIdentities:  An array of PatchBaselineIdentity elements.<br/>        """
        pass

    def describe_patch_group_state(self, PatchGroup: str) -> DescribePatchGroupStateResult:
        """Returns high-level aggregated patch compliance state for a patch group.

        :param self:
        :param PatchGroup:  The name of the patch group whose patch snapshot should be retrieved.<br/>
        :return: InstancesWithUnreportedNotApplicablePatches:  The number of instances with <code>NotApplicable</code> patches beyond the supported limit, which are not reported by name to Systems Manager Inventory.<br/>InstancesWithNotApplicablePatches:  The number of instances with patches that aren't applicable.<br/>InstancesWithFailedPatches:  The number of instances with patches from the patch baseline that failed to install.<br/>InstancesWithMissingPatches:  The number of instances with missing patches from the patch baseline.<br/>InstancesWithInstalledRejectedPatches:  The number of instances with patches installed that are specified in a RejectedPatches list. Patches with a status of <i>INSTALLED_REJECTED</i> were typically installed before they were added to a RejectedPatches list. <note> If ALLOW_AS_DEPENDENCY is the specified option for RejectedPatchesAction, the value of InstancesWithInstalledRejectedPatches will always be 0 (zero). </note><br/>InstancesWithInstalledOtherPatches:  The number of instances with patches installed that aren't defined in the patch baseline.<br/>InstancesWithInstalledPatches:  The number of instances with installed patches.<br/>Instances:  The number of instances in the patch group.<br/>        """
        pass

    def describe_patch_groups(self, MaxResults: int=None, Filters: List[PatchOrchestratorFilter]=None, NextToken: str=None) -> DescribePatchGroupsResult:
        """Lists all patch groups that have been registered with patch baselines.

        :param self:
        :param NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>Filters:  One or more filters. Use a filter to return a more specific list of results.<br/>MaxResults:  The maximum number of patch groups to return (per page).<br/>
        :return: NextToken:  The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.<br/>Mappings:  Each entry in the array contains: PatchGroup: string (between 1 and 256 characters, Regex: ^([\p{L}\p{Z}\p{N}_.:/=+\-@]*)$) PatchBaselineIdentity: A PatchBaselineIdentity element. <br/>        """
        pass

    def describe_patch_properties(self, Property: str, OperatingSystem: str, PatchSet: str=None, MaxResults: int=None, NextToken: str=None) -> DescribePatchPropertiesResult:
        """Lists the properties of available patches organized by product, product family, classification, severity, and other properties of available patches. You can use the reported properties in the filters you specify in requests for actions such as <a>CreatePatchBaseline</a>, <a>UpdatePatchBaseline</a>, <a>DescribeAvailablePatches</a>, and <a>DescribePatchBaselines</a>. The following section lists the properties that can be used in filters for each major operating system type: <dl> <dt>WINDOWS</dt> <dd> Valid properties: PRODUCT, PRODUCT_FAMILY, CLASSIFICATION, MSRC_SEVERITY </dd> <dt>AMAZON_LINUX</dt> <dd> Valid properties: PRODUCT, CLASSIFICATION, SEVERITY </dd> <dt>AMAZON_LINUX_2</dt> <dd> Valid properties: PRODUCT, CLASSIFICATION, SEVERITY </dd> <dt>UBUNTU </dt> <dd> Valid properties: PRODUCT, PRIORITY </dd> <dt>REDHAT_ENTERPRISE_LINUX</dt> <dd> Valid properties: PRODUCT, CLASSIFICATION, SEVERITY </dd> <dt>SUSE</dt> <dd> Valid properties: PRODUCT, CLASSIFICATION, SEVERITY </dd> <dt>CENTOS</dt> <dd> Valid properties: PRODUCT, CLASSIFICATION, SEVERITY </dd> </dl>

        :param self:
        :param NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>PatchSet:  Indicates whether to list patches for the Windows operating system or for Microsoft applications. Not applicable for Linux operating systems.<br/>Property:  The patch property for which you want to view patch details. <br/>OperatingSystem:  The operating system type for which to list patches.<br/>
        :return: NextToken:  The token for the next set of items to return. (You use this token in the next call.)<br/>Properties:  A list of the properties for patches matching the filter request parameters.<br/>        """
        pass

    def describe_sessions(self, State: str, MaxResults: int=None, NextToken: str=None, Filters: List[SessionFilter]=None) -> DescribeSessionsResponse:
        """Retrieves a list of all active sessions (both connected and disconnected) or terminated sessions from the past 30 days.

        :param self:
        :param Filters:  One or more filters to limit the type of sessions returned by the request.<br/>NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>State:  The session status to retrieve a list of sessions for. For example, "Active".<br/>
        :return: NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>Sessions:  A list of sessions meeting the request parameters.<br/>        """
        pass

    def get_automation_execution(self, AutomationExecutionId: str) -> GetAutomationExecutionResult:
        """Get detailed information about a particular Automation execution.

        :param self:
        :param AutomationExecutionId:  The unique identifier for an existing automation execution to examine. The execution ID is returned by StartAutomationExecution when the execution of an Automation document is initiated.<br/>
        :return: AutomationExecution:  Detailed information about the current state of an automation execution.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ProgressCounters</i> An aggregate of step execution statuses displayed in the AWS Console for a multi-Region and multi-account Automation execution.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ProgressCounters</b>: An aggregate of step execution statuses displayed in the AWS Console for a multi-Region and multi-account Automation execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetLocations</i> The combination of AWS Regions and/or AWS accounts where you want to run the Automation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Target</i> The target of the execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxErrors</i> The MaxErrors value specified by the user when the execution started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxConcurrency</i> The MaxConcurrency value specified by the user when the execution started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResolvedTargets</i> A list of resolved targets in the rate control execution.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>ResolvedTargets</b>: A list of resolved targets in the rate control execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetMaps</i> The specified key-value mapping of document parameters to target resources.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Targets</i> The specified targets.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetParameterName</i> The parameter name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentAction</i> The action of the step that is currently running.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CurrentStepName</i> The name of the step that is currently running.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExecutedBy</i> The Amazon Resource Name (ARN) of the user who ran the automation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParentAutomationExecutionId</i> The AutomationExecutionId of the parent automation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Mode</i> The automation execution mode.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureMessage</i> A message describing why an execution has failed, if the status is set to Failed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Outputs</i> The list of execution outputs as defined in the automation document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> The key-value map of execution parameters, which were supplied when calling StartAutomationExecution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StepExecutionsTruncated</i> A boolean value that indicates if the response contains the full list of the Automation step executions. If true, use the DescribeAutomationStepExecutions API action to get the full list of step executions.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StepExecutions</i> A list of details about the current state of all steps that comprise an execution. An Automation document contains a list of steps that are run in order.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomationExecutionStatus</i> The execution status of the Automation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExecutionEndTime</i> The time the execution finished.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExecutionStartTime</i> The time the execution started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentVersion</i> The version of the document to use during execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentName</i> The name of the Automation document used during the execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomationExecutionId</i> The execution ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Truncated</i> A boolean value indicating whether the resolved target list is truncated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ParameterValues</i> A list of parameter values sent to targets that resolved during the Automation execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TimedOutSteps</i> The total number of steps that timed out in all specified AWS Regions and accounts for the current Automation execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CancelledSteps</i> The total number of steps that the system cancelled in all specified AWS Regions and accounts for the current Automation execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FailedSteps</i> The total number of steps that failed to run in all specified AWS Regions and accounts for the current Automation execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>SuccessSteps</i> The total number of steps that successfully completed in all specified AWS Regions and accounts for the current Automation execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TotalSteps</i> The total number of steps run in all specified AWS Regions and accounts for the current Automation execution.<br/>        """
        pass

    def get_command_invocation(self, InstanceId: str, CommandId: str, PluginName: str=None) -> GetCommandInvocationResult:
        """Returns detailed information about command execution for an invocation or plugin. 

        :param self:
        :param PluginName:  (Optional) The name of the plugin for which you want detailed results. If the document contains only one plugin, the name can be omitted and the details will be returned.<br/>InstanceId:  (Required) The ID of the managed instance targeted by the command. A managed instance can be an Amazon EC2 instance or an instance in your hybrid environment that is configured for Systems Manager.<br/>CommandId:  (Required) The parent command ID of the invocation plugin.<br/>
        :return: CloudWatchOutputConfig:  CloudWatch Logs information where Systems Manager sent the command output.:<br/>StandardErrorUrl:  The URL for the complete text written by the plugin to stderr. If the command has not finished running, then this string is empty.<br/>StandardErrorContent:  The first 8,000 characters written by the plugin to stderr. If the command has not finished running, then this string is empty.<br/>StandardOutputUrl:  The URL for the complete text written by the plugin to stdout in Amazon S3. If an Amazon S3 bucket was not specified, then this string is empty.<br/>StandardOutputContent:  The first 24,000 characters written by the plugin to stdout. If the command has not finished running, if ExecutionStatus is neither Succeeded nor Failed, then this string is empty.<br/>StatusDetails:  A detailed status of the command execution for an invocation. StatusDetails includes more information than Status because it includes states resulting from error and concurrency control parameters. StatusDetails can show different results than Status. For more information about these statuses, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/monitor-commands.html">Understanding Command Statuses</a> in the <i>AWS Systems Manager User Guide</i>. StatusDetails can be one of the following values: <ul> <li> Pending: The command has not been sent to the instance. </li> <li> In Progress: The command has been sent to the instance but has not reached a terminal state. </li> <li> Delayed: The system attempted to send the command to the target, but the target was not available. The instance might not be available because of network issues, the instance was stopped, etc. The system will try to deliver the command again. </li> <li> Success: The command or plugin was run successfully. This is a terminal state. </li> <li> Delivery Timed Out: The command was not delivered to the instance before the delivery timeout expired. Delivery timeouts do not count against the parent command's MaxErrors limit, but they do contribute to whether the parent command status is Success or Incomplete. This is a terminal state. </li> <li> Execution Timed Out: The command started to run on the instance, but the execution was not complete before the timeout expired. Execution timeouts count against the MaxErrors limit of the parent command. This is a terminal state. </li> <li> Failed: The command wasn't run successfully on the instance. For a plugin, this indicates that the result code was not zero. For a command invocation, this indicates that the result code for one or more plugins was not zero. Invocation failures count against the MaxErrors limit of the parent command. This is a terminal state. </li> <li> Canceled: The command was terminated before it was completed. This is a terminal state. </li> <li> Undeliverable: The command can't be delivered to the instance. The instance might not exist or might not be responding. Undeliverable invocations don't count against the parent command's MaxErrors limit and don't contribute to whether the parent command status is Success or Incomplete. This is a terminal state. </li> <li> Terminated: The parent command exceeded its MaxErrors limit and subsequent command invocations were canceled by the system. This is a terminal state. </li> </ul><br/>Status:  The status of this invocation plugin. This status can be different than StatusDetails.<br/>ExecutionEndDateTime:  The date and time the plugin was finished running. Date and time are written in ISO 8601 format. For example, June 7, 2017 is represented as 2017-06-7. The following sample AWS CLI command uses the <code>InvokedAfter</code> filter.  <code>aws ssm list-commands --filters key=InvokedAfter,value=2017-06-07T00:00:00Z</code>  If the plugin has not started to run, the string is empty.<br/>ExecutionElapsedTime:  Duration since ExecutionStartDateTime.<br/>ExecutionStartDateTime:  The date and time the plugin started running. Date and time are written in ISO 8601 format. For example, June 7, 2017 is represented as 2017-06-7. The following sample AWS CLI command uses the <code>InvokedBefore</code> filter.  <code>aws ssm list-commands --filters key=InvokedBefore,value=2017-06-07T00:00:00Z</code>  If the plugin has not started to run, the string is empty.<br/>ResponseCode:  The error level response code for the plugin script. If the response code is -1, then the command has not started running on the instance, or it was not received by the instance.<br/>PluginName:  The name of the plugin for which you want detailed results. For example, aws:RunShellScript is a plugin.<br/>DocumentVersion:  The SSM document version used in the request.<br/>DocumentName:  The name of the document that was run. For example, AWS-RunShellScript.<br/>Comment:  The comment text for the command.<br/>InstanceId:  The ID of the managed instance targeted by the command. A managed instance can be an Amazon EC2 instance or an instance in your hybrid environment that is configured for Systems Manager.<br/>CommandId:  The parent command ID of the invocation plugin.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchOutputEnabled</i> Enables Systems Manager to send command output to CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchLogGroupName</i> The name of the CloudWatch log group where you want to send command output. If you don't specify a group name, Systems Manager automatically creates a log group for you. The log group uses the following naming format: aws/ssm/<i>SystemsManagerDocumentName</i>.<br/>        """
        pass

    def get_connection_status(self, Target: str) -> GetConnectionStatusResponse:
        """Retrieves the Session Manager connection status for an instance to determine whether it is connected and ready to receive Session Manager connections.

        :param self:
        :param Target:  The ID of the instance.<br/>
        :return: Status:  The status of the connection to the instance. For example, 'Connected' or 'Not Connected'.<br/>Target:  The ID of the instance to check connection status. <br/>        """
        pass

    def get_default_patch_baseline(self, OperatingSystem: str=None) -> GetDefaultPatchBaselineResult:
        """Retrieves the default patch baseline. Note that Systems Manager supports creating multiple default patch baselines. For example, you can create a default patch baseline for each operating system. If you do not specify an operating system value, the default patch baseline for Windows is returned.

        :param self:
        :param OperatingSystem:  Returns the default patch baseline for the specified operating system.<br/>
        :return: OperatingSystem:  The operating system for the returned patch baseline. <br/>BaselineId:  The ID of the default patch baseline.<br/>        """
        pass

    def get_deployable_patch_snapshot_for_instance(self, SnapshotId: str, InstanceId: str) -> GetDeployablePatchSnapshotForInstanceResult:
        """Retrieves the current snapshot for the patch baseline the instance uses. This API is primarily used by the AWS-RunPatchBaseline Systems Manager document. 

        :param self:
        :param SnapshotId:  The user-defined snapshot ID.<br/>InstanceId:  The ID of the instance for which the appropriate patch snapshot should be retrieved.<br/>
        :return: Product:  Returns the specific operating system (for example Windows Server 2012 or Amazon Linux 2015.09) on the instance for the specified patch snapshot.<br/>SnapshotDownloadUrl:  A pre-signed Amazon S3 URL that can be used to download the patch snapshot.<br/>SnapshotId:  The user-defined snapshot ID.<br/>InstanceId:  The ID of the instance.<br/>        """
        pass

    def get_document(self, Name: str, VersionName: str=None, DocumentVersion: str=None, DocumentFormat: str=None) -> GetDocumentResult:
        """Gets the contents of the specified Systems Manager document.

        :param self:
        :param DocumentFormat:  Returns the document in the specified format. The document format can be either JSON or YAML. JSON is the default format.<br/>DocumentVersion:  The document version for which you want information.<br/>VersionName:  An optional field specifying the version of the artifact associated with the document. For example, "Release 12, Update 6". This value is unique across all versions of a document, and cannot be changed.<br/>Name:  The name of the Systems Manager document.<br/>
        :return: AttachmentsContent:  A description of the document attachments, including names, locations, sizes, etc.<br/>DocumentFormat:  The document format, either JSON or YAML.<br/>DocumentType:  The document type.<br/>Content:  The contents of the Systems Manager document.<br/>StatusInformation:  A message returned by AWS Systems Manager that explains the <code>Status</code> value. For example, a <code>Failed</code> status might be explained by the <code>StatusInformation</code> message, "The specified S3 bucket does not exist. Verify that the URL of the S3 bucket is correct."<br/>Status:  The status of the Systems Manager document, such as <code>Creating</code>, <code>Active</code>, <code>Updating</code>, <code>Failed</code>, and <code>Deleting</code>.<br/>DocumentVersion:  The document version.<br/>VersionName:  The version of the artifact associated with the document. For example, "Release 12, Update 6". This value is unique across all versions of a document, and cannot be changed.<br/>Name:  The name of the Systems Manager document.<br/>        """
        pass

    def get_inventory(self, Filters: List[InventoryFilter]=None, Aggregators: List[InventoryAggregator]=None, ResultAttributes: List[ResultAttribute]=None, NextToken: str=None, MaxResults: int=None) -> GetInventoryResult:
        """Query inventory information.

        :param self:
        :param MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>ResultAttributes:  The list of inventory item types to return.<br/>Aggregators:  Returns counts of inventory types based on one or more expressions. For example, if you aggregate by using an expression that uses the <code>AWS:InstanceInformation.PlatformType</code> type, you can see a count of how many Windows and Linux instances exist in your inventoried fleet.<br/>Filters:  One or more filters. Use a filter to return a more specific list of results.<br/>
        :return: NextToken:  The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.<br/>Entities:  Collection of inventory entities such as a collection of instance inventory. <br/>        """
        pass

    def get_inventory_schema(self, TypeName: str=None, NextToken: str=None, MaxResults: int=None, Aggregator: AggregatorSchemaOnly=None, SubType: IsSubTypeSchema=None) -> GetInventorySchemaResult:
        """Return a list of inventory type names for the account, or return a list of attribute names for a specific Inventory item type. 

        :param self:
        :param SubType:  Returns the sub-type schema for a specified inventory type.<br/>Aggregator:  Returns inventory schemas that support aggregation. For example, this call returns the <code>AWS:InstanceInformation</code> type, because it supports aggregation based on the <code>PlatformName</code>, <code>PlatformType</code>, and <code>PlatformVersion</code> attributes.<br/>MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>TypeName:  The type of inventory item to return.<br/>
        :return: NextToken:  The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.<br/>Schemas:  Inventory schemas returned by the request.<br/>        """
        pass

    def get_maintenance_window(self, WindowId: str) -> GetMaintenanceWindowResult:
        """Retrieves a maintenance window.

        :param self:
        :param WindowId:  The ID of the maintenance window for which you want to retrieve information.<br/>
        :return: ModifiedDate:  The date the maintenance window was last modified.<br/>CreatedDate:  The date the maintenance window was created.<br/>Enabled:  Indicates whether the maintenance window is enabled.<br/>AllowUnassociatedTargets:  Whether targets must be registered with the maintenance window before tasks can be defined for those targets.<br/>Cutoff:  The number of hours before the end of the maintenance window that Systems Manager stops scheduling new tasks for execution.<br/>Duration:  The duration of the maintenance window in hours.<br/>NextExecutionTime:  The next time the maintenance window will actually run, taking into account any specified times for the maintenance window to become active or inactive.<br/>ScheduleTimezone:  The time zone that the scheduled maintenance window executions are based on, in Internet Assigned Numbers Authority (IANA) format. For example: "America/Los_Angeles", "etc/UTC", or "Asia/Seoul". For more information, see the <a href="https://www.iana.org/time-zones">Time Zone Database</a> on the IANA website.<br/>Schedule:  The schedule of the maintenance window in the form of a cron or rate expression.<br/>EndDate:  The date and time, in ISO-8601 Extended format, for when the maintenance window is scheduled to become inactive. The maintenance window will not run after this specified time.<br/>StartDate:  The date and time, in ISO-8601 Extended format, for when the maintenance window is scheduled to become active. The maintenance window will not run before this specified time.<br/>Description:  The description of the maintenance window.<br/>Name:  The name of the maintenance window.<br/>WindowId:  The ID of the created maintenance window.<br/>        """
        pass

    def get_maintenance_window_execution(self, WindowExecutionId: str) -> GetMaintenanceWindowExecutionResult:
        """Retrieves details about a specific task run as part of a maintenance window execution.

        :param self:
        :param WindowExecutionId:  The ID of the maintenance window execution that includes the task.<br/>
        :return: EndTime:  The time the maintenance window finished running.<br/>StartTime:  The time the maintenance window started running.<br/>StatusDetails:  The details explaining the Status. Only available for certain status values.<br/>Status:  The status of the maintenance window execution.<br/>TaskIds:  The ID of the task executions from the maintenance window execution.<br/>WindowExecutionId:  The ID of the maintenance window execution.<br/>        """
        pass

    def get_maintenance_window_execution_task(self, TaskId: str, WindowExecutionId: str) -> GetMaintenanceWindowExecutionTaskResult:
        """Retrieves the details about a specific task run as part of a maintenance window execution.

        :param self:
        :param TaskId:  The ID of the specific task execution in the maintenance window task that should be retrieved.<br/>WindowExecutionId:  The ID of the maintenance window execution that includes the task.<br/>
        :return: EndTime:  The time the task execution completed.<br/>StartTime:  The time the task execution started.<br/>StatusDetails:  The details explaining the Status. Only available for certain status values.<br/>Status:  The status of the task.<br/>MaxErrors:  The defined maximum number of task execution errors allowed before scheduling of the task execution would have been stopped.<br/>MaxConcurrency:  The defined maximum number of task executions that could be run in parallel.<br/>Priority:  The priority of the task.<br/>TaskParameters:  The parameters passed to the task when it was run. <note>  <code>TaskParameters</code> has been deprecated. To specify parameters to pass to a task when it runs, instead use the <code>Parameters</code> option in the <code>TaskInvocationParameters</code> structure. For information about how Systems Manager handles these options for the supported maintenance window task types, see <a>MaintenanceWindowTaskInvocationParameters</a>. </note> The map has the following format: Key: string, between 1 and 255 characters Value: an array of strings, each string is between 1 and 255 characters<br/>Type:  The type of task that was run.<br/>ServiceRole:  The role that was assumed when running the task.<br/>TaskArn:  The ARN of the task that ran.<br/>TaskExecutionId:  The ID of the specific task execution in the maintenance window task that was retrieved.<br/>WindowExecutionId:  The ID of the maintenance window execution that includes the task.<br/>        """
        pass

    def get_maintenance_window_execution_task_invocation(self, InvocationId: str, TaskId: str, WindowExecutionId: str) -> GetMaintenanceWindowExecutionTaskInvocationResult:
        """Retrieves a task invocation. A task invocation is a specific task running on a specific target. maintenance windows report status for all invocations. 

        :param self:
        :param InvocationId:  The invocation ID to retrieve.<br/>TaskId:  The ID of the specific task in the maintenance window task that should be retrieved. <br/>WindowExecutionId:  The ID of the maintenance window execution for which the task is a part.<br/>
        :return: WindowTargetId:  The maintenance window target ID.<br/>OwnerInformation:  User-provided value to be included in any CloudWatch events raised while running tasks for these targets in this maintenance window. <br/>EndTime:  The time that the task finished running on the target.<br/>StartTime:  The time that the task started running on the target.<br/>StatusDetails:  The details explaining the status. Details are only available for certain status values.<br/>Status:  The task status for an invocation.<br/>Parameters:  The parameters used at the time that the task ran.<br/>TaskType:  Retrieves the task type for a maintenance window. Task types include the following: LAMBDA, STEP_FUNCTION, AUTOMATION, RUN_COMMAND.<br/>ExecutionId:  The execution ID.<br/>InvocationId:  The invocation ID.<br/>TaskExecutionId:  The task execution ID.<br/>WindowExecutionId:  The maintenance window execution ID.<br/>        """
        pass

    def get_maintenance_window_task(self, WindowTaskId: str, WindowId: str) -> GetMaintenanceWindowTaskResult:
        """Lists the tasks in a maintenance window.

        :param self:
        :param WindowTaskId:  The maintenance window task ID to retrieve.<br/>WindowId:  The maintenance window ID that includes the task to retrieve.<br/>
        :return: Description:  The retrieved task description.<br/>Name:  The retrieved task name.<br/>LoggingInfo:  The location in Amazon S3 where the task results are logged. <note>  <code>LoggingInfo</code> has been deprecated. To specify an S3 bucket to contain logs, instead use the <code>OutputS3BucketName</code> and <code>OutputS3KeyPrefix</code> options in the <code>TaskInvocationParameters</code> structure. For information about how Systems Manager handles these options for the supported maintenance window task types, see <a>MaintenanceWindowTaskInvocationParameters</a>. </note>:<br/>MaxErrors:  The maximum number of errors allowed before the task stops being scheduled.<br/>MaxConcurrency:  The maximum number of targets allowed to run this task in parallel.<br/>Priority:  The priority of the task when it runs. The lower the number, the higher the priority. Tasks that have the same priority are scheduled in parallel.<br/>TaskInvocationParameters:  The parameters to pass to the task when it runs.:<br/>TaskParameters:  The parameters to pass to the task when it runs. <note>  <code>TaskParameters</code> has been deprecated. To specify parameters to pass to a task when it runs, instead use the <code>Parameters</code> option in the <code>TaskInvocationParameters</code> structure. For information about how Systems Manager handles these options for the supported maintenance window task types, see <a>MaintenanceWindowTaskInvocationParameters</a>. </note><br/>TaskType:  The type of task to run.<br/>ServiceRoleArn:  The ARN of the IAM service role to use to publish Amazon Simple Notification Service (Amazon SNS) notifications for maintenance window Run Command tasks.<br/>TaskArn:  The resource that the task used during execution. For RUN_COMMAND and AUTOMATION task types, the TaskArn is the Systems Manager Document name/ARN. For LAMBDA tasks, the value is the function name/ARN. For STEP_FUNCTION tasks, the value is the state machine ARN.<br/>Targets:  The targets where the task should run.<br/>WindowTaskId:  The retrieved maintenance window task ID.<br/>WindowId:  The retrieved maintenance window ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Lambda</i> The parameters for a LAMBDA task type.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Lambda</b>: The parameters for a LAMBDA task type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StepFunctions</i> The parameters for a STEP_FUNCTION task type.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StepFunctions</b>: The parameters for a STEP_FUNCTION task type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Automation</i> The parameters for an AUTOMATION task type.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Automation</b>: The parameters for an AUTOMATION task type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RunCommand</i> The parameters for a RUN_COMMAND task type.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RunCommand</b>: The parameters for a RUN_COMMAND task type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentHashType</i> SHA-256 or SHA-1. SHA-1 hashes have been deprecated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentHash</i> The SHA-256 or SHA-1 hash created by the system when the document was created. SHA-1 hashes have been deprecated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> Information about the commands to run.<br/>Configurations for sending notifications about command status changes on a per-instance basis.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NotificationConfig</b>: Configurations for sending notifications about command status changes on a per-instance basis.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> The parameters for the AUTOMATION task. For information about specifying and updating task parameters, see <a>RegisterTaskWithMaintenanceWindow</a> and <a>UpdateMaintenanceWindowTask</a>. <note>  <code>LoggingInfo</code> has been deprecated. To specify an S3 bucket to contain logs, instead use the <code>OutputS3BucketName</code> and <code>OutputS3KeyPrefix</code> options in the <code>TaskInvocationParameters</code> structure. For information about how Systems Manager handles these options for the supported maintenance window task types, see <a>MaintenanceWindowTaskInvocationParameters</a>.  <code>TaskParameters</code> has been deprecated. To specify parameters to pass to a task when it runs, instead use the <code>Parameters</code> option in the <code>TaskInvocationParameters</code> structure. For information about how Systems Manager handles these options for the supported maintenance window task types, see <a>MaintenanceWindowTaskInvocationParameters</a>. For AUTOMATION task types, Systems Manager ignores any values specified for these parameters. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentVersion</i> The version of an Automation document to use during task execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the STEP_FUNCTION task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Input</i> The inputs for the STEP_FUNCTION task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Payload</i> JSON to provide to your Lambda function as input.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Qualifier</i> (Optional) Specify a Lambda function version or alias name. If you specify a function version, the action uses the qualified function ARN to invoke a specific Lambda function. If you specify an alias name, the action uses the alias ARN to invoke the Lambda function version to which the alias points.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientContext</i> Pass client-specific information to the Lambda function that you are invoking. You can then process the client information in your Lambda function as you choose through the context variable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Region</i> The region where the Amazon S3 bucket is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3KeyPrefix</i> (Optional) The Amazon S3 bucket subfolder. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3BucketName</i> The name of an Amazon S3 bucket where execution logs are stored .<br/>        """
        pass

    def get_ops_item(self, OpsItemId: str) -> GetOpsItemResponse:
        """Get information about an OpsItem by using the ID. You must have permission in AWS Identity and Access Management (IAM) to view information about an OpsItem. For more information, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/OpsCenter-getting-started.html">Getting Started with OpsCenter</a> in the <i>AWS Systems Manager User Guide</i>. Operations engineers and IT professionals use OpsCenter to view, investigate, and remediate operational issues impacting the performance and health of their AWS resources. For more information, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/OpsCenter.html">AWS Systems Manager OpsCenter</a> in the <i>AWS Systems Manager User Guide</i>. 

        :param self:
        :param OpsItemId:  The ID of the OpsItem that you want to get.<br/>
        :return: OpsItem:  The OpsItem.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OperationalData</i> Operational data is custom data that provides useful reference details about the OpsItem. For example, you can specify log files, error strings, license keys, troubleshooting tips, or other relevant data. You enter operational data as key-value pairs. The key has a maximum length of 128 characters. The value has a maximum size of 20 KB. <important> Operational data keys <i>can't</i> begin with the following: amazon, aws, amzn, ssm, /amazon, /aws, /amzn, /ssm. </important> You can choose to make the data searchable by other users in the account or you can restrict search access. Searchable data means that all users with access to the OpsItem Overview page (as provided by the <a>DescribeOpsItems</a> API action) can view and search on the specified data. Operational data that is not searchable is only viewable by users who have access to the OpsItem (as provided by the <a>GetOpsItem</a> API action). Use the <code>/aws/resources</code> key in OperationalData to specify a related resource in the request. Use the <code>/aws/automations</code> key in OperationalData to associate an Automation runbook with the OpsItem. To view AWS CLI example commands that use these keys, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/OpsCenter-creating-OpsItems.html#OpsCenter-manually-create-OpsItems">Creating OpsItems Manually</a> in the <i>AWS Systems Manager User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Source</i> The origin of the OpsItem, such as Amazon EC2 or AWS Systems Manager. The impacted resource is a subset of source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Title</i> A short heading that describes the nature of the OpsItem and the impacted resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version of this OpsItem. Each time the OpsItem is edited the version number increments by one.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OpsItemId</i> The ID of the OpsItem.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The OpsItem status. Status can be <code>Open</code>, <code>In Progress</code>, or <code>Resolved</code>. For more information, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/OpsCenter-working-with-OpsItems-editing-details.html">Editing OpsItem Details</a> in the <i>AWS Systems Manager User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RelatedOpsItems</i> One or more OpsItems that share something in common with the current OpsItem. For example, related OpsItems can include OpsItems with similar error messages, impacted resources, or statuses for the impacted resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Priority</i> The importance of this OpsItem in relation to other OpsItems in the system.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Notifications</i> The Amazon Resource Name (ARN) of an SNS topic where notifications are sent when this OpsItem is edited or changed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedTime</i> The date and time the OpsItem was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedBy</i> The ARN of the AWS account that last updated the OpsItem.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The OpsItem description.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTime</i> The date and time the OpsItem was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedBy</i> The ARN of the AWS account that created the OpsItem.<br/>        """
        pass

    def get_ops_summary(self, Aggregators: List[OpsAggregator], Filters: List[OpsFilter]=None, NextToken: str=None, MaxResults: int=None) -> GetOpsSummaryResult:
        """View a summary of OpsItems based on specified filters and aggregators.

        :param self:
        :param MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>NextToken:  A token to start the list. Use this token to get the next set of results. <br/>Aggregators:  Optional aggregators that return counts of OpsItems based on one or more expressions.<br/>Filters:  Optional filters used to scope down the returned OpsItems. <br/>
        :return: NextToken:  The token for the next set of items to return. Use this token to get the next set of results.<br/>Entities:  The list of aggregated and filtered OpsItems.<br/>        """
        pass

    def get_parameter(self, Name: str, WithDecryption: Boolean=None) -> GetParameterResult:
        """Get information about a parameter by using the parameter name. Don't confuse this API action with the <a>GetParameters</a> API action.

        :param self:
        :param WithDecryption:  Return decrypted values for secure string parameters. This flag is ignored for String and StringList parameter types.<br/>Name:  The name of the parameter you want to query.<br/>
        :return: Parameter:  Information about a parameter.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ARN</i> The Amazon Resource Name (ARN) of the parameter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> Date the parameter was last changed or updated and the parameter version was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceResult</i> Applies to parameters that reference information in other AWS services. SourceResult is the raw result or response from the source.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Selector</i> Either the version number or the label used to retrieve the parameter value. Specify selectors by using one of the following formats: parameter_name:version parameter_name:label<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The parameter version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The parameter value.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of parameter. Valid values include the following: String, String list, Secure string.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the parameter.<br/>        """
        pass

    def get_parameter_history(self, Name: str, WithDecryption: Boolean=None, MaxResults: int=None, NextToken: str=None) -> GetParameterHistoryResult:
        """Query a list of all parameters used by the AWS account.

        :param self:
        :param NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>WithDecryption:  Return decrypted values for secure string parameters. This flag is ignored for String and StringList parameter types.<br/>Name:  The name of a parameter you want to query.<br/>
        :return: NextToken:  The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.<br/>Parameters:  A list of parameters returned by the request.<br/>        """
        pass

    def get_parameters(self, Names: List[PSParameterName], WithDecryption: Boolean=None) -> GetParametersResult:
        """Get details of a parameter. Don't confuse this API action with the <a>GetParameter</a> API action.

        :param self:
        :param WithDecryption:  Return decrypted secure string value. Return decrypted values for secure string parameters. This flag is ignored for String and StringList parameter types.<br/>Names:  Names of the parameters for which you want to query information.<br/>
        :return: InvalidParameters:  A list of parameters that are not formatted correctly or do not run during an execution.<br/>Parameters:  A list of details for a parameter.<br/>        """
        pass

    def get_parameters_by_path(self, Path: str, Recursive: Boolean=None, ParameterFilters: List[ParameterStringFilter]=None, WithDecryption: Boolean=None, MaxResults: int=None, NextToken: str=None) -> GetParametersByPathResult:
        """Retrieve parameters in a specific hierarchy. For more information, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/sysman-paramstore-working.html">Working with Systems Manager Parameters</a> in the <i>AWS Systems Manager User Guide</i>.  Request results are returned on a best-effort basis. If you specify <code>MaxResults</code> in the request, the response includes information up to the limit specified. The number of items returned, however, can be between zero and the value of <code>MaxResults</code>. If the service reaches an internal limit while processing the results, it stops the operation and returns the matching values up to that point and a <code>NextToken</code>. You can specify the <code>NextToken</code> in a subsequent call to get the next set of results. <note> This API action doesn't support filtering by tags.  </note>

        :param self:
        :param NextToken:  A token to start the list. Use this token to get the next set of results. <br/>MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>WithDecryption:  Retrieve all parameters in a hierarchy with their value decrypted.<br/>ParameterFilters:  Filters to limit the request results. <note> You can't filter using the parameter name. </note><br/>Recursive:  Retrieve all parameters within a hierarchy. <important> If a user has access to a path, then the user can access all levels of that path. For example, if a user has permission to access path <code>/a</code>, then the user can also access <code>/a/b</code>. Even if a user has explicitly been denied access in IAM for parameter <code>/a/b</code>, they can still call the GetParametersByPath API action recursively for <code>/a</code> and view <code>/a/b</code>. </important><br/>Path:  The hierarchy for the parameter. Hierarchies start with a forward slash (/) and end with the parameter name. A parameter name hierarchy can have a maximum of 15 levels. Here is an example of a hierarchy: <code>/Finance/Prod/IAD/WinServ2016/license33</code> <br/>
        :return: NextToken:  The token for the next set of items to return. Use this token to get the next set of results.<br/>Parameters:  A list of parameters found in the specified hierarchy.<br/>        """
        pass

    def get_patch_baseline(self, BaselineId: str) -> GetPatchBaselineResult:
        """Retrieves information about a patch baseline.

        :param self:
        :param BaselineId:  The ID of the patch baseline to retrieve.<br/>
        :return: Sources:  Information about the patches to use to update the instances, including target operating systems and source repositories. Applies to Linux instances only.<br/>Description:  A description of the patch baseline.<br/>ModifiedDate:  The date the patch baseline was last modified.<br/>CreatedDate:  The date the patch baseline was created.<br/>PatchGroups:  Patch groups included in the patch baseline.<br/>RejectedPatchesAction:  The action specified to take on patches included in the RejectedPatches list. A patch can be allowed only if it is a dependency of another package, or blocked entirely along with packages that include it as a dependency.<br/>RejectedPatches:  A list of explicitly rejected patches for the baseline.<br/>ApprovedPatchesEnableNonSecurity:  Indicates whether the list of approved patches includes non-security updates that should be applied to the instances. The default value is 'false'. Applies to Linux instances only.<br/>ApprovedPatchesComplianceLevel:  Returns the specified compliance severity level for approved patches in the patch baseline.<br/>ApprovedPatches:  A list of explicitly approved patches for the baseline.<br/>ApprovalRules:  A set of rules used to include patches in the baseline.:<br/>GlobalFilters:  A set of global filters used to exclude patches from the baseline.:<br/>OperatingSystem:  Returns the operating system specified for the patch baseline.<br/>Name:  The name of the patch baseline.<br/>BaselineId:  The ID of the retrieved patch baseline.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PatchFilters</i> The set of patch filters that make up the group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PatchRules</i> The rules that make up the rule group.<br/>        """
        pass

    def get_patch_baseline_for_patch_group(self, PatchGroup: str, OperatingSystem: str=None) -> GetPatchBaselineForPatchGroupResult:
        """Retrieves the patch baseline that should be used for the specified patch group.

        :param self:
        :param OperatingSystem:  Returns he operating system rule specified for patch groups using the patch baseline.<br/>PatchGroup:  The name of the patch group whose patch baseline should be retrieved.<br/>
        :return: OperatingSystem:  The operating system rule specified for patch groups using the patch baseline.<br/>PatchGroup:  The name of the patch group.<br/>BaselineId:  The ID of the patch baseline that should be used for the patch group.<br/>        """
        pass

    def get_service_setting(self, SettingId: str) -> GetServiceSettingResult:
        """ <code>ServiceSetting</code> is an account-level setting for an AWS service. This setting defines how a user interacts with or uses a service or a feature of a service. For example, if an AWS service charges money to the account based on feature or service usage, then the AWS service team might create a default setting of "false". This means the user can't use this feature unless they change the setting to "true" and intentionally opt in for a paid feature. Services map a <code>SettingId</code> object to a setting value. AWS services teams define the default value for a <code>SettingId</code>. You can't create a new <code>SettingId</code>, but you can overwrite the default value if you have the <code>ssm:UpdateServiceSetting</code> permission for the setting. Use the <a>UpdateServiceSetting</a> API action to change the default setting. Or use the <a>ResetServiceSetting</a> to change the value back to the original value defined by the AWS service team. Query the current service setting for the account. 

        :param self:
        :param SettingId:  The ID of the service setting to get.<br/>
        :return: ServiceSetting:  The query result of the current service setting.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the service setting. The value can be Default, Customized or PendingUpdate. <ul> <li> Default: The current setting uses a default value provisioned by the AWS service team. </li> <li> Customized: The current setting use a custom value specified by the customer. </li> <li> PendingUpdate: The current setting uses a default or custom value, but a setting change request is pending approval. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ARN</i> The ARN of the service setting.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedUser</i> The ARN of the last modified user. This field is populated only if the setting value was overwritten.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The last time the service setting was modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SettingValue</i> The value of the service setting.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SettingId</i> The ID of the service setting.<br/>        """
        pass

    def label_parameter_version(self, Labels: List[ParameterLabel], Name: str, ParameterVersion: PSParameterVersion=None) -> LabelParameterVersionResult:
        """A parameter label is a user-defined alias to help you manage different versions of a parameter. When you modify a parameter, Systems Manager automatically saves a new version and increments the version number by one. A label can help you remember the purpose of a parameter when there are multiple versions.  Parameter labels have the following requirements and restrictions. <ul> <li> A version of a parameter can have a maximum of 10 labels. </li> <li> You can't attach the same label to different versions of the same parameter. For example, if version 1 has the label Production, then you can't attach Production to version 2. </li> <li> You can move a label from one version of a parameter to another. </li> <li> You can't create a label when you create a new parameter. You must attach a label to a specific version of a parameter. </li> <li> You can't delete a parameter label. If you no longer want to use a parameter label, then you must move it to a different version of a parameter. </li> <li> A label can have a maximum of 100 characters. </li> <li> Labels can contain letters (case sensitive), numbers, periods (.), hyphens (-), or underscores (_). </li> <li> Labels can't begin with a number, "aws," or "ssm" (not case sensitive). If a label fails to meet these requirements, then the label is not associated with a parameter and the system displays it in the list of InvalidLabels. </li> </ul>

        :param self:
        :param Labels:  One or more labels to attach to the specified parameter version.<br/>ParameterVersion:  The specific version of the parameter on which you want to attach one or more labels. If no version is specified, the system attaches the label to the latest version.)<br/>Name:  The parameter name on which you want to attach one or more labels.<br/>
        :return: InvalidLabels:  The label does not meet the requirements. For information about parameter label requirements, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/sysman-paramstore-labels.html">Labeling Parameters</a> in the <i>AWS Systems Manager User Guide</i>.<br/>        """
        pass

    def list_association_versions(self, AssociationId: str, MaxResults: int=None, NextToken: str=None) -> ListAssociationVersionsResult:
        """Retrieves all versions of an association for a specific association ID.

        :param self:
        :param NextToken:  A token to start the list. Use this token to get the next set of results. <br/>MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>AssociationId:  The association ID for which you want to view all versions.<br/>
        :return: NextToken:  The token for the next set of items to return. Use this token to get the next set of results.<br/>AssociationVersions:  Information about all versions of the association for the specified association ID.<br/>        """
        pass

    def list_associations(self, AssociationFilterList: List[AssociationFilter]=None, MaxResults: int=None, NextToken: str=None) -> ListAssociationsResult:
        """Lists the associations for the specified Systems Manager document or instance.

        :param self:
        :param NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>AssociationFilterList:  One or more filters. Use a filter to return a more specific list of results.<br/>
        :return: NextToken:  The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.<br/>Associations:  The associations.<br/>        """
        pass

    def list_command_invocations(self, CommandId: str=None, InstanceId: str=None, MaxResults: int=None, NextToken: str=None, Filters: List[CommandFilter]=None, Details: Boolean=None) -> ListCommandInvocationsResult:
        """An invocation is copy of a command sent to a specific instance. A command can apply to one or more instances. A command invocation applies to one instance. For example, if a user runs SendCommand against three instances, then a command invocation is created for each requested instance ID. ListCommandInvocations provide status about command execution.

        :param self:
        :param Details:  (Optional) If set this returns the response of the command executions and any command output. By default this is set to False. <br/>Filters:  (Optional) One or more filters. Use a filter to return a more specific list of results.<br/>NextToken:  (Optional) The token for the next set of items to return. (You received this token from a previous call.)<br/>MaxResults:  (Optional) The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>InstanceId:  (Optional) The command execution details for a specific instance ID.<br/>CommandId:  (Optional) The invocations for a specific command ID.<br/>
        :return: NextToken:  (Optional) The token for the next set of items to return. (You received this token from a previous call.)<br/>CommandInvocations:  (Optional) A list of all invocations. <br/>        """
        pass

    def list_commands(self, CommandId: str=None, InstanceId: str=None, MaxResults: int=None, NextToken: str=None, Filters: List[CommandFilter]=None) -> ListCommandsResult:
        """Lists the commands requested by users of the AWS account.

        :param self:
        :param Filters:  (Optional) One or more filters. Use a filter to return a more specific list of results. <br/>NextToken:  (Optional) The token for the next set of items to return. (You received this token from a previous call.)<br/>MaxResults:  (Optional) The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>InstanceId:  (Optional) Lists commands issued against this instance ID.<br/>CommandId:  (Optional) If provided, lists only the specified command.<br/>
        :return: NextToken:  (Optional) The token for the next set of items to return. (You received this token from a previous call.)<br/>Commands:  (Optional) The list of commands requested by the user. <br/>        """
        pass

    def list_compliance_items(self, Filters: List[ComplianceStringFilter]=None, ResourceIds: List[ComplianceResourceId]=None, ResourceTypes: List[ComplianceResourceType]=None, NextToken: str=None, MaxResults: int=None) -> ListComplianceItemsResult:
        """For a specified resource ID, this API action returns a list of compliance statuses for different resource types. Currently, you can only specify one resource ID per call. List results depend on the criteria specified in the filter. 

        :param self:
        :param MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>NextToken:  A token to start the list. Use this token to get the next set of results. <br/>ResourceTypes:  The type of resource from which to get compliance information. Currently, the only supported resource type is <code>ManagedInstance</code>.<br/>ResourceIds:  The ID for the resources from which to get compliance information. Currently, you can only specify one resource ID.<br/>Filters:  One or more compliance filters. Use a filter to return a more specific list of results.<br/>
        :return: NextToken:  The token for the next set of items to return. Use this token to get the next set of results.<br/>ComplianceItems:  A list of compliance information for the specified resource ID. <br/>        """
        pass

    def list_compliance_summaries(self, Filters: List[ComplianceStringFilter]=None, NextToken: str=None, MaxResults: int=None) -> ListComplianceSummariesResult:
        """Returns a summary count of compliant and non-compliant resources for a compliance type. For example, this call can return State Manager associations, patches, or custom compliance types according to the filter criteria that you specify. 

        :param self:
        :param MaxResults:  The maximum number of items to return for this call. Currently, you can specify null or 50. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>NextToken:  A token to start the list. Use this token to get the next set of results. <br/>Filters:  One or more compliance or inventory filters. Use a filter to return a more specific list of results.<br/>
        :return: NextToken:  The token for the next set of items to return. Use this token to get the next set of results.<br/>ComplianceSummaryItems:  A list of compliant and non-compliant summary counts based on compliance types. For example, this call returns State Manager associations, patches, or custom compliance types according to the filter criteria that you specified.<br/>        """
        pass

    def list_document_versions(self, Name: str, MaxResults: int=None, NextToken: str=None) -> ListDocumentVersionsResult:
        """List all versions for a document.

        :param self:
        :param NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>Name:  The name of the document about which you want version information.<br/>
        :return: NextToken:  The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.<br/>DocumentVersions:  The document versions.<br/>        """
        pass

    def list_documents(self, DocumentFilterList: List[DocumentFilter]=None, Filters: List[DocumentKeyValuesFilter]=None, MaxResults: int=None, NextToken: str=None) -> ListDocumentsResult:
        """Describes one or more of your Systems Manager documents.

        :param self:
        :param NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>Filters:  One or more filters. Use a filter to return a more specific list of results.<br/>DocumentFilterList:  One or more filters. Use a filter to return a more specific list of results.<br/>
        :return: NextToken:  The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.<br/>DocumentIdentifiers:  The names of the Systems Manager documents.<br/>        """
        pass

    def list_inventory_entries(self, TypeName: str, InstanceId: str, Filters: List[InventoryFilter]=None, NextToken: str=None, MaxResults: int=None) -> ListInventoryEntriesResult:
        """A list of inventory items returned by the request.

        :param self:
        :param MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>NextToken:  The token for the next set of items to return. (You received this token from a previous call.)<br/>Filters:  One or more filters. Use a filter to return a more specific list of results.<br/>TypeName:  The type of inventory item for which you want information.<br/>InstanceId:  The instance ID for which you want inventory information.<br/>
        :return: NextToken:  The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.<br/>Entries:  A list of inventory items on the instance(s).<br/>CaptureTime:  The time that inventory information was collected for the instance(s).<br/>SchemaVersion:  The inventory schema version used by the instance(s).<br/>InstanceId:  The instance ID targeted by the request to query inventory information.<br/>TypeName:  The type of inventory item returned by the request.<br/>        """
        pass

    def list_resource_compliance_summaries(self, Filters: List[ComplianceStringFilter]=None, NextToken: str=None, MaxResults: int=None) -> ListResourceComplianceSummariesResult:
        """Returns a resource-level summary count. The summary includes information about compliant and non-compliant statuses and detailed compliance-item severity counts, according to the filter criteria you specify.

        :param self:
        :param MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>NextToken:  A token to start the list. Use this token to get the next set of results. <br/>Filters:  One or more filters. Use a filter to return a more specific list of results.<br/>
        :return: NextToken:  The token for the next set of items to return. Use this token to get the next set of results.<br/>ResourceComplianceSummaryItems:  A summary count for specified or targeted managed instances. Summary count includes information about compliant and non-compliant State Manager associations, patch status, or custom items according to the filter criteria that you specify. <br/>        """
        pass

    def list_resource_data_sync(self, NextToken: str=None, MaxResults: int=None) -> ListResourceDataSyncResult:
        """Lists your resource data sync configurations. Includes information about the last time a sync attempted to start, the last sync status, and the last time a sync successfully completed. The number of sync configurations might be too large to return using a single call to <code>ListResourceDataSync</code>. You can limit the number of sync configurations returned by using the <code>MaxResults</code> parameter. To determine whether there are more sync configurations to list, check the value of <code>NextToken</code> in the output. If there are more sync configurations to list, you can request them by specifying the <code>NextToken</code> returned in the call to the parameter of a subsequent call. 

        :param self:
        :param MaxResults:  The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results.<br/>NextToken:  A token to start the list. Use this token to get the next set of results. <br/>
        :return: NextToken:  The token for the next set of items to return. Use this token to get the next set of results.<br/>ResourceDataSyncItems:  A list of your current Resource Data Sync configurations and their statuses.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceId: str, ResourceType: str) -> ListTagsForResourceResult:
        """Returns a list of the tags assigned to the specified resource.

        :param self:
        :param ResourceId:  The resource ID for which you want to see a list of tags.<br/>ResourceType:  Returns a list of tags for a specific resource type.<br/>
        :return: TagList:  A list of tags.<br/>        """
        pass

    def modify_document_permission(self, PermissionType: str, Name: str, AccountIdsToAdd: List[AccountId]=None, AccountIdsToRemove: List[AccountId]=None) -> ModifyDocumentPermissionResponse:
        """Shares a Systems Manager document publicly or privately. If you share a document privately, you must specify the AWS user account IDs for those people who can use the document. If you share a document publicly, you must specify <i>All</i> as the account ID.

        :param self:
        :param AccountIdsToRemove:  The AWS user accounts that should no longer have access to the document. The AWS user account can either be a group of account IDs or <i>All</i>. This action has a higher priority than <i>AccountIdsToAdd</i>. If you specify an account ID to add and the same ID to remove, the system removes access to the document.<br/>AccountIdsToAdd:  The AWS user accounts that should have access to the document. The account IDs can either be a group of account IDs or <i>All</i>.<br/>PermissionType:  The permission type for the document. The permission type can be <i>Share</i>.<br/>Name:  The name of the document that you want to share.<br/>
        :return:         """
        pass

    def put_compliance_items(self, Items: List[ComplianceItemEntry], ExecutionSummary: ComplianceExecutionSummary, ComplianceType: str, ResourceType: str, ResourceId: str, ItemContentHash: str=None) -> PutComplianceItemsResult:
        """Registers a compliance type and other compliance details on a designated resource. This action lets you register custom compliance details with a resource. This call overwrites existing compliance information on the resource, so you must provide a full list of compliance items each time that you send the request. ComplianceType can be one of the following: <ul> <li> ExecutionId: The execution ID when the patch, association, or custom compliance item was applied. </li> <li> ExecutionType: Specify patch, association, or Custom:<code>string</code>. </li> <li> ExecutionTime. The time the patch, association, or custom compliance item was applied to the instance. </li> <li> Id: The patch, association, or custom compliance ID. </li> <li> Title: A title. </li> <li> Status: The status of the compliance item. For example, <code>approved</code> for patches, or <code>Failed</code> for associations. </li> <li> Severity: A patch severity. For example, <code>critical</code>. </li> <li> DocumentName: A SSM document name. For example, AWS-RunPatchBaseline. </li> <li> DocumentVersion: An SSM document version number. For example, 4. </li> <li> Classification: A patch classification. For example, <code>security updates</code>. </li> <li> PatchBaselineId: A patch baseline ID. </li> <li> PatchSeverity: A patch severity. For example, <code>Critical</code>. </li> <li> PatchState: A patch state. For example, <code>InstancesWithFailedPatches</code>. </li> <li> PatchGroup: The name of a patch group. </li> <li> InstalledTime: The time the association, patch, or custom compliance item was applied to the resource. Specify the time by using the following format: yyyy-MM-dd'T'HH:mm:ss'Z' </li> </ul>

        :param self:
        :param ItemContentHash:  MD5 or SHA-256 content hash. The content hash is used to determine if existing information should be overwritten or ignored. If the content hashes match, the request to put compliance information is ignored.<br/>Items:  Information about the compliance as defined by the resource type. For example, for a patch compliance type, <code>Items</code> includes information about the PatchSeverity, Classification, etc.<br/>ExecutionSummary:  A summary of the call execution that includes an execution ID, the type of execution (for example, <code>Command</code>), and the date/time of the execution using a datetime object that is saved in the following format: yyyy-MM-dd'T'HH:mm:ss'Z'.:<br/>ComplianceType:  Specify the compliance type. For example, specify Association (for a State Manager association), Patch, or Custom:<code>string</code>.<br/>ResourceType:  Specify the type of resource. <code>ManagedInstance</code> is currently the only supported resource type.<br/>ResourceId:  Specify an ID for this resource. For a managed instance, this is the instance ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExecutionType</i> The type of execution. For example, <code>Command</code> is a valid execution type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExecutionId</i> An ID created by the system when <code>PutComplianceItems</code> was called. For example, <code>CommandID</code> is a valid execution ID. You can use this ID in subsequent calls.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExecutionTime</i> The time the execution ran as a datetime object that is saved in the following format: yyyy-MM-dd'T'HH:mm:ss'Z'.<br/>
        :return:         """
        pass

    def put_inventory(self, Items: List[InventoryItem], InstanceId: str) -> PutInventoryResult:
        """Bulk update custom inventory items on one more instance. The request adds an inventory item, if it doesn't already exist, or updates an inventory item, if it does exist.

        :param self:
        :param Items:  The inventory items that you want to add or update on instances.<br/>InstanceId:  One or more instance IDs where you want to add or update inventory items.<br/>
        :return: Message:  Information about the request.<br/>        """
        pass

    def put_parameter(self, Type: str, Value: str, Name: str, Description: str=None, KeyId: str=None, Overwrite: Boolean=None, AllowedPattern: str=None, Tags: List[Tag]=None, Tier: str=None, Policies: str=None) -> PutParameterResult:
        """Add a parameter to the system.

        :param self:
        :param Policies:  One or more policies to apply to a parameter. This action takes a JSON array. Parameter Store supports the following policy types: Expiration: This policy deletes the parameter after it expires. When you create the policy, you specify the expiration date. You can update the expiration date and time by updating the policy. Updating the <i>parameter</i> does not affect the expiration date and time. When the expiration time is reached, Parameter Store deletes the parameter. ExpirationNotification: This policy triggers an event in Amazon CloudWatch Events that notifies you about the expiration. By using this policy, you can receive notification before or after the expiration time is reached, in units of days or hours. NoChangeNotification: This policy triggers a CloudWatch event if a parameter has not been modified for a specified period of time. This policy type is useful when, for example, a secret needs to be changed within a period of time, but it has not been changed. All existing policies are preserved until you send new policies or an empty policy. For more information about parameter policies, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/sysman-paramstore-su-policies.html">Working with Parameter Policies</a>. <br/>Tier:  Parameter Store offers a standard tier and an advanced tier for parameters. Standard parameters have a value limit of 4 KB and can't be configured to use parameter policies. You can create a maximum of 10,000 standard parameters per account and per Region. Standard parameters are offered at no additional cost. Advanced parameters have a value limit of 8 KB and can be configured to use parameter policies. You can create a maximum of 100,000 advanced parameters per account and per Region. Advanced parameters incur a charge. If you don't specify a parameter tier when you create a new parameter, the parameter defaults to using the standard tier. You can change a standard parameter to an advanced parameter at any time. But you can't revert an advanced parameter to a standard parameter. Reverting an advanced parameter to a standard parameter would result in data loss because the system would truncate the size of the parameter from 8 KB to 4 KB. Reverting would also remove any policies attached to the parameter. Lastly, advanced parameters use a different form of encryption than standard parameters. If you no longer need an advanced parameter, or if you no longer want to incur charges for an advanced parameter, you must delete it and recreate it as a new standard parameter. For more information, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/parameter-store-advanced-parameters.html">About Advanced Parameters</a> in the <i>AWS Systems Manager User Guide</i>.<br/>Tags:  Optional metadata that you assign to a resource. Tags enable you to categorize a resource in different ways, such as by purpose, owner, or environment. For example, you might want to tag a Systems Manager parameter to identify the type of resource to which it applies, the environment, or the type of configuration data referenced by the parameter. In this case, you could specify the following key name/value pairs: <ul> <li>  <code>Key=Resource,Value=S3bucket</code>  </li> <li>  <code>Key=OS,Value=Windows</code>  </li> <li>  <code>Key=ParameterType,Value=LicenseKey</code>  </li> </ul> <note> To add tags to an existing Systems Manager parameter, use the <a>AddTagsToResource</a> action. </note><br/>AllowedPattern:  A regular expression used to validate the parameter value. For example, for String types with values restricted to numbers, you can specify the following: AllowedPattern=^\d+$ <br/>Overwrite:  Overwrite an existing parameter. If not specified, will default to "false".<br/>KeyId:  The KMS Key ID that you want to use to encrypt a parameter. Either the default AWS Key Management Service (AWS KMS) key automatically assigned to your AWS account or a custom key. Required for parameters that use the <code>SecureString</code> data type. If you don't specify a key ID, the system uses the default key associated with your AWS account. <ul> <li> To use your default AWS KMS key, choose the <code>SecureString</code> data type, and do <i>not</i> specify the <code>Key ID</code> when you create the parameter. The system automatically populates <code>Key ID</code> with your default KMS key. </li> <li> To use a custom KMS key, choose the <code>SecureString</code> data type with the <code>Key ID</code> parameter. </li> </ul><br/>Type:  The type of parameter that you want to add to the system. Items in a <code>StringList</code> must be separated by a comma (,). You can't use other punctuation or special character to escape items in the list. If you have a parameter value that requires a comma, then use the <code>String</code> data type. <note>  <code>SecureString</code> is not currently supported for AWS CloudFormation templates or in the China Regions. </note><br/>Value:  The parameter value that you want to add to the system. Standard parameters have a value limit of 4 KB. Advanced parameters have a value limit of 8 KB.<br/>Description:  Information about the parameter that you want to add to the system. Optional but recommended. <important> Do not enter personally identifiable information in this field. </important><br/>Name:  The fully qualified name of the parameter that you want to add to the system. The fully qualified name includes the complete hierarchy of the parameter path and name. For example: <code>/Dev/DBServer/MySQL/db-string13</code>  Naming Constraints: <ul> <li> Parameter names are case sensitive. </li> <li> A parameter name must be unique within an AWS Region </li> <li> A parameter name can't be prefixed with "aws" or "ssm" (case-insensitive). </li> <li> Parameter names can include only the following symbols and letters: <code>a-zA-Z0-9_.-/</code>  </li> <li> A parameter name can't include spaces. </li> <li> Parameter hierarchies are limited to a maximum depth of fifteen levels. </li> </ul> For additional information about valid values for parameter names, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/sysman-parameter-name-constraints.html">Requirements and Constraints for Parameter Names</a> in the <i>AWS Systems Manager User Guide</i>. <note> The maximum length constraint listed below includes capacity for additional system attributes that are not part of the name. The maximum length for the fully qualified parameter name is 1011 characters.  </note><br/>
        :return: Version:  The new version number of a parameter. If you edit a parameter value, Parameter Store automatically creates a new version and assigns this new version a unique ID. You can reference a parameter version ID in API actions or in Systems Manager documents (SSM documents). By default, if you don't specify a specific version, the system returns the latest parameter value when a parameter is called.<br/>        """
        pass

    def register_default_patch_baseline(self, BaselineId: str) -> RegisterDefaultPatchBaselineResult:
        """Defines the default patch baseline.

        :param self:
        :param BaselineId:  The ID of the patch baseline that should be the default patch baseline.<br/>
        :return: BaselineId:  The ID of the default patch baseline.<br/>        """
        pass

    def register_patch_baseline_for_patch_group(self, PatchGroup: str, BaselineId: str) -> RegisterPatchBaselineForPatchGroupResult:
        """Registers a patch baseline for a patch group.

        :param self:
        :param PatchGroup:  The name of the patch group that should be registered with the patch baseline.<br/>BaselineId:  The ID of the patch baseline to register the patch group with.<br/>
        :return: PatchGroup:  The name of the patch group registered with the patch baseline.<br/>BaselineId:  The ID of the patch baseline the patch group was registered with.<br/>        """
        pass

    def register_target_with_maintenance_window(self, Targets: List[Target], ResourceType: str, WindowId: str, OwnerInformation: str=None, Name: str=None, Description: str=None, ClientToken: str=None) -> RegisterTargetWithMaintenanceWindowResult:
        """Registers a target with a maintenance window.

        :param self:
        :param ClientToken:  User-provided idempotency token.<br/>Description:  An optional description for the target.<br/>Name:  An optional name for the target.<br/>OwnerInformation:  User-provided value that will be included in any CloudWatch events raised while running tasks for these targets in this maintenance window.<br/>Targets:  The targets to register with the maintenance window. In other words, the instances to run commands on when the maintenance window runs. You can specify targets using either instance IDs or tags that have been applied to instances.  <b>Example 1</b>: Specify instance IDs  <code>Key=InstanceIds,Values=<i>instance-id-1</i>,<i>instance-id-2</i>,<i>instance-id-3</i> </code>   <b>Example 2</b>: Use tag key-pairs applied to instances  <code>Key=tag:<i>my-tag-key</i>,Values=<i>my-tag-value-1</i>,<i>my-tag-value-2</i> </code>   <b>Example 3</b>: Use tag-keys applied to instances  <code>Key=tag-key,Values=<i>my-tag-key-1</i>,<i>my-tag-key-2</i> </code>  For more information about these examples formats, including the best use case for each one, see <a href="https://docs.aws.amazon.com/systems-manager/latest/userguide/mw-cli-tutorial-targets-examples.html">Examples: Register Targets with a Maintenance Window</a> in the <i>AWS Systems Manager User Guide</i>.<br/>ResourceType:  The type of target being registered with the maintenance window.<br/>WindowId:  The ID of the maintenance window the target should be registered with.<br/>
        :return: WindowTargetId:  The ID of the target definition in this maintenance window.<br/>        """
        pass

    def register_task_with_maintenance_window(self, MaxErrors: str, MaxConcurrency: str, TaskType: str, TaskArn: str, Targets: List[Target], WindowId: str, ServiceRoleArn: str=None, TaskParameters: MaintenanceWindowTaskParameters=None, TaskInvocationParameters: MaintenanceWindowTaskInvocationParameters=None, Priority: int=None, LoggingInfo: LoggingInfo=None, Name: str=None, Description: str=None, ClientToken: str=None) -> RegisterTaskWithMaintenanceWindowResult:
        """Adds a new task to a maintenance window.

        :param self:
        :param ClientToken:  User-provided idempotency token.<br/>Description:  An optional description for the task.<br/>Name:  An optional name for the task.<br/>LoggingInfo:  A structure containing information about an Amazon S3 bucket to write instance-level logs to.  <note>  <code>LoggingInfo</code> has been deprecated. To specify an S3 bucket to contain logs, instead use the <code>OutputS3BucketName</code> and <code>OutputS3KeyPrefix</code> options in the <code>TaskInvocationParameters</code> structure. For information about how Systems Manager handles these options for the supported maintenance window task types, see <a>MaintenanceWindowTaskInvocationParameters</a>. </note>:<br/>MaxErrors:  The maximum number of errors allowed before this task stops being scheduled.<br/>MaxConcurrency:  The maximum number of targets this task can be run for in parallel.<br/>Priority:  The priority of the task in the maintenance window, the lower the number the higher the priority. Tasks in a maintenance window are scheduled in priority order with tasks that have the same priority scheduled in parallel.<br/>TaskInvocationParameters:  The parameters that the task should use during execution. Populate only the fields that match the task type. All other fields should be empty. :<br/>TaskParameters:  The parameters that should be passed to the task when it is run. <note>  <code>TaskParameters</code> has been deprecated. To specify parameters to pass to a task when it runs, instead use the <code>Parameters</code> option in the <code>TaskInvocationParameters</code> structure. For information about how Systems Manager handles these options for the supported maintenance window task types, see <a>MaintenanceWindowTaskInvocationParameters</a>. </note><br/>TaskType:  The type of task being registered.<br/>ServiceRoleArn:  The ARN of the IAM service role for Systems Manager to assume when running a maintenance window task. If you do not specify a service role ARN, Systems Manager uses your account's service-linked role. If no service-linked role for Systems Manager exists in your account, it is created when you run <code>RegisterTaskWithMaintenanceWindow</code>. For more information, see the following topics in the in the <i>AWS Systems Manager User Guide</i>: <ul> <li>  <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/using-service-linked-roles.html#slr-permissions">Service-Linked Role Permissions for Systems Manager</a>  </li> <li>  <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/sysman-maintenance-permissions.html#maintenance-window-tasks-service-role">Should I Use a Service-Linked Role or a Custom Service Role to Run Maintenance Window Tasks? </a>  </li> </ul><br/>TaskArn:  The ARN of the task to run.<br/>Targets:  The targets (either instances or maintenance window targets). Specify instances using the following format:   <code>Key=InstanceIds,Values=&lt;instance-id-1&gt;,&lt;instance-id-2&gt;</code>  Specify maintenance window targets using the following format:  <code>Key=&lt;WindowTargetIds&gt;,Values=&lt;window-target-id-1&gt;,&lt;window-target-id-2&gt;</code> <br/>WindowId:  The ID of the maintenance window the task should be added to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Lambda</i> The parameters for a LAMBDA task type.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Lambda</b>: The parameters for a LAMBDA task type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StepFunctions</i> The parameters for a STEP_FUNCTION task type.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StepFunctions</b>: The parameters for a STEP_FUNCTION task type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Automation</i> The parameters for an AUTOMATION task type.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Automation</b>: The parameters for an AUTOMATION task type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RunCommand</i> The parameters for a RUN_COMMAND task type.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RunCommand</b>: The parameters for a RUN_COMMAND task type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentHashType</i> SHA-256 or SHA-1. SHA-1 hashes have been deprecated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentHash</i> The SHA-256 or SHA-1 hash created by the system when the document was created. SHA-1 hashes have been deprecated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> Information about the commands to run.<br/>Configurations for sending notifications about command status changes on a per-instance basis.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NotificationConfig</b>: Configurations for sending notifications about command status changes on a per-instance basis.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> The parameters for the AUTOMATION task. For information about specifying and updating task parameters, see <a>RegisterTaskWithMaintenanceWindow</a> and <a>UpdateMaintenanceWindowTask</a>. <note>  <code>LoggingInfo</code> has been deprecated. To specify an S3 bucket to contain logs, instead use the <code>OutputS3BucketName</code> and <code>OutputS3KeyPrefix</code> options in the <code>TaskInvocationParameters</code> structure. For information about how Systems Manager handles these options for the supported maintenance window task types, see <a>MaintenanceWindowTaskInvocationParameters</a>.  <code>TaskParameters</code> has been deprecated. To specify parameters to pass to a task when it runs, instead use the <code>Parameters</code> option in the <code>TaskInvocationParameters</code> structure. For information about how Systems Manager handles these options for the supported maintenance window task types, see <a>MaintenanceWindowTaskInvocationParameters</a>. For AUTOMATION task types, Systems Manager ignores any values specified for these parameters. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentVersion</i> The version of an Automation document to use during task execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the STEP_FUNCTION task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Input</i> The inputs for the STEP_FUNCTION task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Payload</i> JSON to provide to your Lambda function as input.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Qualifier</i> (Optional) Specify a Lambda function version or alias name. If you specify a function version, the action uses the qualified function ARN to invoke a specific Lambda function. If you specify an alias name, the action uses the alias ARN to invoke the Lambda function version to which the alias points.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientContext</i> Pass client-specific information to the Lambda function that you are invoking. You can then process the client information in your Lambda function as you choose through the context variable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Region</i> The region where the Amazon S3 bucket is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3KeyPrefix</i> (Optional) The Amazon S3 bucket subfolder. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3BucketName</i> The name of an Amazon S3 bucket where execution logs are stored .<br/>
        :return: WindowTaskId:  The ID of the task in the maintenance window.<br/>        """
        pass

    def remove_tags_from_resource(self, TagKeys: List[TagKey], ResourceId: str, ResourceType: str) -> RemoveTagsFromResourceResult:
        """Removes all tags from the specified resource.

        :param self:
        :param TagKeys:  Tag keys that you want to remove from the specified resource.<br/>ResourceId:  The resource ID for which you want to remove tags. Use the ID of the resource. Here are some examples: ManagedInstance: mi-012345abcde MaintenanceWindow: mw-012345abcde PatchBaseline: pb-012345abcde For the Document and Parameter values, use the name of the resource. <note> The ManagedInstance type for this API action is only for on-premises managed instances. You must specify the name of the managed instance in the following format: mi-ID_number. For example, mi-1a2b3c4d5e6f. </note><br/>ResourceType:  The type of resource of which you want to remove a tag. <note> The ManagedInstance type for this API action is only for on-premises managed instances. You must specify the name of the managed instance in the following format: mi-ID_number. For example, mi-1a2b3c4d5e6f. </note><br/>
        :return:         """
        pass

    def reset_service_setting(self, SettingId: str) -> ResetServiceSettingResult:
        """ <code>ServiceSetting</code> is an account-level setting for an AWS service. This setting defines how a user interacts with or uses a service or a feature of a service. For example, if an AWS service charges money to the account based on feature or service usage, then the AWS service team might create a default setting of "false". This means the user can't use this feature unless they change the setting to "true" and intentionally opt in for a paid feature. Services map a <code>SettingId</code> object to a setting value. AWS services teams define the default value for a <code>SettingId</code>. You can't create a new <code>SettingId</code>, but you can overwrite the default value if you have the <code>ssm:UpdateServiceSetting</code> permission for the setting. Use the <a>GetServiceSetting</a> API action to view the current value. Use the <a>UpdateServiceSetting</a> API action to change the default setting.  Reset the service setting for the account to the default value as provisioned by the AWS service team. 

        :param self:
        :param SettingId:  The ID of the service setting to reset.<br/>
        :return: ServiceSetting:  The current, effective service setting after calling the ResetServiceSetting API action.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the service setting. The value can be Default, Customized or PendingUpdate. <ul> <li> Default: The current setting uses a default value provisioned by the AWS service team. </li> <li> Customized: The current setting use a custom value specified by the customer. </li> <li> PendingUpdate: The current setting uses a default or custom value, but a setting change request is pending approval. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ARN</i> The ARN of the service setting.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedUser</i> The ARN of the last modified user. This field is populated only if the setting value was overwritten.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastModifiedDate</i> The last time the service setting was modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SettingValue</i> The value of the service setting.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SettingId</i> The ID of the service setting.<br/>        """
        pass

    def resume_session(self, SessionId: str) -> ResumeSessionResponse:
        """Reconnects a session to an instance after it has been disconnected. Connections can be resumed for disconnected sessions, but not terminated sessions. <note> This command is primarily for use by client machines to automatically reconnect during intermittent network issues. It is not intended for any other use. </note>

        :param self:
        :param SessionId:  The ID of the disconnected session to resume.<br/>
        :return: StreamUrl:  A URL back to SSM Agent on the instance that the Session Manager client uses to send commands and receive output from the instance. Format: <code>wss://ssm-messages.<b>region</b>.amazonaws.com/v1/data-channel/<b>session-id</b>?stream=(input|output)</code>.  <b>region</b> represents the Region identifier for an AWS Region supported by AWS Systems Manager, such as <code>us-east-2</code> for the US East (Ohio) Region. For a list of supported <b>region</b> values, see the <b>Region</b> column in the <a href="http://docs.aws.amazon.com/general/latest/gr/rande.html#ssm_region">AWS Systems Manager table of regions and endpoints</a> in the <i>AWS General Reference</i>.  <b>session-id</b> represents the ID of a Session Manager session, such as <code>1a2b3c4dEXAMPLE</code>.<br/>TokenValue:  An encrypted token value containing session and caller information. Used to authenticate the connection to the instance.<br/>SessionId:  The ID of the session.<br/>        """
        pass

    def send_automation_signal(self, SignalType: str, AutomationExecutionId: str, Payload: AutomationParameterMap=None) -> SendAutomationSignalResult:
        """Sends a signal to an Automation execution to change the current behavior or status of the execution. 

        :param self:
        :param Payload:  The data sent with the signal. The data schema depends on the type of signal used in the request. For <code>Approve</code> and <code>Reject</code> signal types, the payload is an optional comment that you can send with the signal type. For example:  <code>Comment="Looks good"</code>  For <code>StartStep</code> and <code>Resume</code> signal types, you must send the name of the Automation step to start or resume as the payload. For example:  <code>StepName="step1"</code>  For the <code>StopStep</code> signal type, you must send the step execution ID as the payload. For example:  <code>StepExecutionId="97fff367-fc5a-4299-aed8-0123456789ab"</code> <br/>SignalType:  The type of signal to send to an Automation execution. <br/>AutomationExecutionId:  The unique identifier for an existing Automation execution that you want to send the signal to.<br/>
        :return:         """
        pass

    def send_command(self, DocumentName: str, InstanceIds: List[InstanceId]=None, Targets: List[Target]=None, DocumentVersion: str=None, DocumentHash: str=None, DocumentHashType: str=None, TimeoutSeconds: int=None, Comment: str=None, Parameters: Parameters=None, OutputS3Region: str=None, OutputS3BucketName: str=None, OutputS3KeyPrefix: str=None, MaxConcurrency: str=None, MaxErrors: str=None, ServiceRoleArn: str=None, NotificationConfig: NotificationConfig=None, CloudWatchOutputConfig: CloudWatchOutputConfig=None) -> SendCommandResult:
        """Runs commands on one or more managed instances.

        :param self:
        :param CloudWatchOutputConfig:  Enables Systems Manager to send Run Command output to Amazon CloudWatch Logs. :<br/>NotificationConfig:  Configurations for sending notifications.:<br/>ServiceRoleArn:  The ARN of the IAM service role to use to publish Amazon Simple Notification Service (Amazon SNS) notifications for Run Command commands.<br/>MaxErrors:  The maximum number of errors allowed without the command failing. When the command fails one more time beyond the value of MaxErrors, the systems stops sending the command to additional targets. You can specify a number like 10 or a percentage like 10%. The default value is 0. For more information about how to use MaxErrors, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/send-commands-multiple.html#send-commands-maxerrors">Using Error Controls</a> in the <i>AWS Systems Manager User Guide</i>.<br/>MaxConcurrency:  (Optional) The maximum number of instances that are allowed to run the command at the same time. You can specify a number such as 10 or a percentage such as 10%. The default value is 50. For more information about how to use MaxConcurrency, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/send-commands-multiple.html#send-commands-velocity">Using Concurrency Controls</a> in the <i>AWS Systems Manager User Guide</i>.<br/>OutputS3KeyPrefix:  The directory structure within the S3 bucket where the responses should be stored.<br/>OutputS3BucketName:  The name of the S3 bucket where command execution responses should be stored.<br/>OutputS3Region:  (Deprecated) You can no longer specify this parameter. The system ignores it. Instead, Systems Manager automatically determines the Amazon S3 bucket region.<br/>Parameters:  The required and optional parameters specified in the document being run.<br/>Comment:  User-specified information about the command, such as a brief description of what the command should do.<br/>TimeoutSeconds:  If this time is reached and the command has not already started running, it will not run.<br/>DocumentHashType:  Sha256 or Sha1. <note> Sha1 hashes have been deprecated. </note><br/>DocumentHash:  The Sha256 or Sha1 hash created by the system when the document was created.  <note> Sha1 hashes have been deprecated. </note><br/>DocumentVersion:  The SSM document version to use in the request. You can specify $DEFAULT, $LATEST, or a specific version number. If you run commands by using the AWS CLI, then you must escape the first two options by using a backslash. If you specify a version number, then you don't need to use the backslash. For example: --document-version "\$DEFAULT" --document-version "\$LATEST" --document-version "3"<br/>DocumentName:  Required. The name of the Systems Manager document to run. This can be a public document or a custom document.<br/>Targets:  (Optional) An array of search criteria that targets instances using a Key,Value combination that you specify. Targets is required if you don't provide one or more instance IDs in the call. For more information about how to use targets, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/send-commands-multiple.html">Sending Commands to a Fleet</a> in the <i>AWS Systems Manager User Guide</i>.<br/>InstanceIds:  The instance IDs where the command should run. You can specify a maximum of 50 IDs. If you prefer not to list individual instance IDs, you can instead send commands to a fleet of instances using the Targets parameter, which accepts EC2 tags. For more information about how to use targets, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/send-commands-multiple.html">Sending Commands to a Fleet</a> in the <i>AWS Systems Manager User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationType</i> Command: Receive notification when the status of a command changes. Invocation: For commands sent to multiple instances, receive notification on a per-instance basis when the status of a command changes. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationEvents</i> The different events for which you can receive notifications. These events include the following: All (events), InProgress, Success, TimedOut, Cancelled, Failed. To learn more about these events, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/monitoring-sns-notifications.html">Configuring Amazon SNS Notifications for AWS Systems Manager</a> in the <i>AWS Systems Manager User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationArn</i> An Amazon Resource Name (ARN) for an Amazon Simple Notification Service (Amazon SNS) topic. Run Command pushes notifications about command status changes to this topic.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchOutputEnabled</i> Enables Systems Manager to send command output to CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchLogGroupName</i> The name of the CloudWatch log group where you want to send command output. If you don't specify a group name, Systems Manager automatically creates a log group for you. The log group uses the following naming format: aws/ssm/<i>SystemsManagerDocumentName</i>.<br/>
        :return: Command:  The request as it was received by Systems Manager. Also provides the command ID which can be used future references to this request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchOutputConfig</i> CloudWatch Logs information where you want Systems Manager to send the command output.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>CloudWatchOutputConfig</b>: CloudWatch Logs information where you want Systems Manager to send the command output.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationConfig</i> Configurations for sending notifications about command status changes. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NotificationConfig</b>: Configurations for sending notifications about command status changes. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceRole</i> The IAM service role that Run Command uses to act on your behalf when sending notifications about command status changes. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DeliveryTimedOutCount</i> The number of targets for which the status is Delivery Timed Out.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ErrorCount</i> The number of targets for which the status is Failed or Execution Timed Out.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompletedCount</i> The number of targets for which the command invocation reached a terminal state. Terminal states include the following: Success, Failed, Execution Timed Out, Delivery Timed Out, Canceled, Terminated, or Undeliverable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetCount</i> The number of targets for the command.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxErrors</i> The maximum number of errors allowed before the system stops sending the command to additional targets. You can specify a number of errors, such as 10, or a percentage or errors, such as 10%. The default value is 0. For more information about how to use MaxErrors, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/run-command.html">Running Commands Using Systems Manager Run Command</a> in the <i>AWS Systems Manager User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxConcurrency</i> The maximum number of instances that are allowed to run the command at the same time. You can specify a number of instances, such as 10, or a percentage of instances, such as 10%. The default value is 50. For more information about how to use MaxConcurrency, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/run-command.html">Running Commands Using Systems Manager Run Command</a> in the <i>AWS Systems Manager User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputS3KeyPrefix</i> The S3 directory path inside the bucket where the responses to the command executions should be stored. This was requested when issuing the command.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputS3BucketName</i> The S3 bucket where the responses to the command executions should be stored. This was requested when issuing the command.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputS3Region</i> (Deprecated) You can no longer specify this parameter. The system ignores it. Instead, Systems Manager automatically determines the Amazon S3 bucket region.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusDetails</i> A detailed status of the command execution. StatusDetails includes more information than Status because it includes states resulting from error and concurrency control parameters. StatusDetails can show different results than Status. For more information about these statuses, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/monitor-commands.html">Understanding Command Statuses</a> in the <i>AWS Systems Manager User Guide</i>. StatusDetails can be one of the following values: <ul> <li> Pending: The command has not been sent to any instances. </li> <li> In Progress: The command has been sent to at least one instance but has not reached a final state on all instances. </li> <li> Success: The command successfully ran on all invocations. This is a terminal state. </li> <li> Delivery Timed Out: The value of MaxErrors or more command invocations shows a status of Delivery Timed Out. This is a terminal state. </li> <li> Execution Timed Out: The value of MaxErrors or more command invocations shows a status of Execution Timed Out. This is a terminal state. </li> <li> Failed: The value of MaxErrors or more command invocations shows a status of Failed. This is a terminal state. </li> <li> Incomplete: The command was attempted on all instances and one or more invocations does not have a value of Success but not enough invocations failed for the status to be Failed. This is a terminal state. </li> <li> Canceled: The command was terminated before it was completed. This is a terminal state. </li> <li> Rate Exceeded: The number of instances targeted by the command exceeded the account limit for pending invocations. The system has canceled the command before running it on any instance. This is a terminal state. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the command.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RequestedDateTime</i> The date and time the command was requested.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Targets</i> An array of search criteria that targets instances using a Key,Value combination that you specify. Targets is required if you don't provide one or more instance IDs in the call.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceIds</i> The instance IDs against which this command was requested.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> The parameter values to be inserted in the document when running the command.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ExpiresAfter</i> If this time is reached and the command has not already started running, it will not run. Calculated based on the ExpiresAfter user input provided as part of the SendCommand API.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> User-specified information about the command, such as a brief description of what the command should do.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentVersion</i> The SSM document version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentName</i> The name of the document requested for execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CommandId</i> A unique identifier for this command.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationType</i> Command: Receive notification when the status of a command changes. Invocation: For commands sent to multiple instances, receive notification on a per-instance basis when the status of a command changes. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationEvents</i> The different events for which you can receive notifications. These events include the following: All (events), InProgress, Success, TimedOut, Cancelled, Failed. To learn more about these events, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/monitoring-sns-notifications.html">Configuring Amazon SNS Notifications for AWS Systems Manager</a> in the <i>AWS Systems Manager User Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>NotificationArn</i> An Amazon Resource Name (ARN) for an Amazon Simple Notification Service (Amazon SNS) topic. Run Command pushes notifications about command status changes to this topic.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchOutputEnabled</i> Enables Systems Manager to send command output to CloudWatch Logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CloudWatchLogGroupName</i> The name of the CloudWatch log group where you want to send command output. If you don't specify a group name, Systems Manager automatically creates a log group for you. The log group uses the following naming format: aws/ssm/<i>SystemsManagerDocumentName</i>.<br/>        """
        pass

    def start_associations_once(self, AssociationIds: List[AssociationId]) -> StartAssociationsOnceResult:
        """Use this API action to run an association immediately and only one time. This action can be helpful when troubleshooting associations.

        :param self:
        :param AssociationIds:  The association IDs that you want to run immediately and only one time.<br/>
        :return:         """
        pass

    def start_automation_execution(self, DocumentName: str, DocumentVersion: str=None, Parameters: AutomationParameterMap=None, ClientToken: str=None, Mode: str=None, TargetParameterName: str=None, Targets: List[Target]=None, TargetMaps: List[TargetMap]=None, MaxConcurrency: str=None, MaxErrors: str=None, TargetLocations: List[TargetLocation]=None) -> StartAutomationExecutionResult:
        """Initiates execution of an Automation document.

        :param self:
        :param TargetLocations:  A location is a combination of AWS Regions and/or AWS accounts where you want to run the Automation. Use this action to start an Automation in multiple Regions and multiple accounts. For more information, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/systems-manager-automation-multiple-accounts-and-regions.html">Executing Automations in Multiple AWS Regions and Accounts</a> in the <i>AWS Systems Manager User Guide</i>. <br/>MaxErrors:  The number of errors that are allowed before the system stops running the automation on additional targets. You can specify either an absolute number of errors, for example 10, or a percentage of the target set, for example 10%. If you specify 3, for example, the system stops running the automation when the fourth error is received. If you specify 0, then the system stops running the automation on additional targets after the first error result is returned. If you run an automation on 50 resources and set max-errors to 10%, then the system stops running the automation on additional targets when the sixth error is received. Executions that are already running an automation when max-errors is reached are allowed to complete, but some of these executions may fail as well. If you need to ensure that there won't be more than max-errors failed executions, set max-concurrency to 1 so the executions proceed one at a time.<br/>MaxConcurrency:  The maximum number of targets allowed to run this task in parallel. You can specify a number, such as 10, or a percentage, such as 10%. The default value is 10.<br/>TargetMaps:  A key-value mapping of document parameters to target resources. Both Targets and TargetMaps cannot be specified together.<br/>Targets:  A key-value mapping to target resources. Required if you specify TargetParameterName.<br/>TargetParameterName:  The name of the parameter used as the target resource for the rate-controlled execution. Required if you specify targets.<br/>Mode:  The execution mode of the automation. Valid modes include the following: Auto and Interactive. The default mode is Auto.<br/>ClientToken:  User-provided idempotency token. The token must be unique, is case insensitive, enforces the UUID format, and can't be reused.<br/>Parameters:  A key-value map of execution parameters, which match the declared parameters in the Automation document.<br/>DocumentVersion:  The version of the Automation document to use for this execution.<br/>DocumentName:  The name of the Automation document to use for this execution.<br/>
        :return: AutomationExecutionId:  The unique ID of a newly scheduled automation execution.<br/>        """
        pass

    def start_session(self, Target: str, DocumentName: str=None, Parameters: SessionManagerParameters=None) -> StartSessionResponse:
        """Initiates a connection to a target (for example, an instance) for a Session Manager session. Returns a URL and token that can be used to open a WebSocket connection for sending input and receiving outputs. <note> AWS CLI usage: <code>start-session</code> is an interactive command that requires the Session Manager plugin to be installed on the client machine making the call. For information, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/session-manager-working-with-install-plugin.html"> Install the Session Manager Plugin for the AWS CLI</a> in the <i>AWS Systems Manager User Guide</i>. </note>

        :param self:
        :param Parameters:  Reserved for future use.<br/>DocumentName:  The name of the SSM document to define the parameters and plugin settings for the session. For example, <code>SSM-SessionManagerRunShell</code>. If no document name is provided, a shell to the instance is launched by default.<br/>Target:  The instance to connect to for the session.<br/>
        :return: StreamUrl:  A URL back to SSM Agent on the instance that the Session Manager client uses to send commands and receive output from the instance. Format: <code>wss://ssm-messages.<b>region</b>.amazonaws.com/v1/data-channel/<b>session-id</b>?stream=(input|output)</code>   <b>region</b> represents the Region identifier for an AWS Region supported by AWS Systems Manager, such as <code>us-east-2</code> for the US East (Ohio) Region. For a list of supported <b>region</b> values, see the <b>Region</b> column in the <a href="http://docs.aws.amazon.com/general/latest/gr/rande.html#ssm_region">AWS Systems Manager table of regions and endpoints</a> in the <i>AWS General Reference</i>.  <b>session-id</b> represents the ID of a Session Manager session, such as <code>1a2b3c4dEXAMPLE</code>.<br/>TokenValue:  An encrypted token value containing session and caller information. Used to authenticate the connection to the instance.<br/>SessionId:  The ID of the session.<br/>        """
        pass

    def stop_automation_execution(self, AutomationExecutionId: str, Type: str=None) -> StopAutomationExecutionResult:
        """Stop an Automation that is currently running.

        :param self:
        :param Type:  The stop request type. Valid types include the following: Cancel and Complete. The default type is Cancel.<br/>AutomationExecutionId:  The execution ID of the Automation to stop.<br/>
        :return:         """
        pass

    def terminate_session(self, SessionId: str) -> TerminateSessionResponse:
        """Permanently ends a session and closes the data connection between the Session Manager client and SSM Agent on the instance. A terminated session cannot be resumed.

        :param self:
        :param SessionId:  The ID of the session to terminate.<br/>
        :return: SessionId:  The ID of the session that has been terminated.<br/>        """
        pass

    def update_association(self, AssociationId: str, Parameters: Parameters=None, DocumentVersion: str=None, ScheduleExpression: str=None, OutputLocation: InstanceAssociationOutputLocation=None, Name: str=None, Targets: List[Target]=None, AssociationName: str=None, AssociationVersion: str=None, AutomationTargetParameterName: str=None, MaxErrors: str=None, MaxConcurrency: str=None, ComplianceSeverity: str=None) -> UpdateAssociationResult:
        """Updates an association. You can update the association name and version, the document version, schedule, parameters, and Amazon S3 output. <important> When you update an association, the association immediately runs against the specified targets. </important>

        :param self:
        :param ComplianceSeverity:  The severity level to assign to the association.<br/>MaxConcurrency:  The maximum number of targets allowed to run the association at the same time. You can specify a number, for example 10, or a percentage of the target set, for example 10%. The default value is 100%, which means all targets run the association at the same time. If a new instance starts and attempts to run an association while Systems Manager is running MaxConcurrency associations, the association is allowed to run. During the next association interval, the new instance will process its association within the limit specified for MaxConcurrency.<br/>MaxErrors:  The number of errors that are allowed before the system stops sending requests to run the association on additional targets. You can specify either an absolute number of errors, for example 10, or a percentage of the target set, for example 10%. If you specify 3, for example, the system stops sending requests when the fourth error is received. If you specify 0, then the system stops sending requests after the first error is returned. If you run an association on 50 instances and set MaxError to 10%, then the system stops sending the request when the sixth error is received. Executions that are already running an association when MaxErrors is reached are allowed to complete, but some of these executions may fail as well. If you need to ensure that there won't be more than max-errors failed executions, set MaxConcurrency to 1 so that executions proceed one at a time.<br/>AutomationTargetParameterName:  Specify the target for the association. This target is required for associations that use an Automation document and target resources by using rate controls.<br/>AssociationVersion:  This parameter is provided for concurrency control purposes. You must specify the latest association version in the service. If you want to ensure that this request succeeds, either specify <code>$LATEST</code>, or omit this parameter.<br/>AssociationName:  The name of the association that you want to update.<br/>Targets:  The targets of the association.<br/>Name:  The name of the SSM document that contains the configuration information for the instance. You can specify Command or Automation documents. You can specify AWS-predefined documents, documents you created, or a document that is shared with you from another account. For SSM documents that are shared with you from other AWS accounts, you must specify the complete SSM document ARN, in the following format:  <code>arn:aws:ssm:<i>region</i>:<i>account-id</i>:document/<i>document-name</i> </code>  For example:  <code>arn:aws:ssm:us-east-2:12345678912:document/My-Shared-Document</code>  For AWS-predefined documents and SSM documents you created in your account, you only need to specify the document name. For example, <code>AWS-ApplyPatchBaseline</code> or <code>My-Document</code>.<br/>OutputLocation:  An Amazon S3 bucket where you want to store the results of this request.:<br/>ScheduleExpression:  The cron expression used to schedule the association that you want to update.<br/>DocumentVersion:  The document version you want update for the association. <br/>Parameters:  The parameters you want to update for the association. If you create a parameter using Parameter Store, you can reference the parameter using {{ssm:parameter-name}}<br/>AssociationId:  The ID of the association you want to update. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Location</i> An Amazon S3 bucket where you want to store the results of this request.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Location</b>: An Amazon S3 bucket where you want to store the results of this request.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputS3KeyPrefix</i> The Amazon S3 bucket subfolder.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputS3BucketName</i> The name of the Amazon S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputS3Region</i> (Deprecated) You can no longer specify this parameter. The system ignores it. Instead, Systems Manager automatically determines the Amazon S3 bucket region.<br/>
        :return: AssociationDescription:  The description of the association that was updated.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ComplianceSeverity</i> The severity level that is assigned to the association.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxConcurrency</i> The maximum number of targets allowed to run the association at the same time. You can specify a number, for example 10, or a percentage of the target set, for example 10%. The default value is 100%, which means all targets run the association at the same time. If a new instance starts and attempts to run an association while Systems Manager is running MaxConcurrency associations, the association is allowed to run. During the next association interval, the new instance will process its association within the limit specified for MaxConcurrency.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxErrors</i> The number of errors that are allowed before the system stops sending requests to run the association on additional targets. You can specify either an absolute number of errors, for example 10, or a percentage of the target set, for example 10%. If you specify 3, for example, the system stops sending requests when the fourth error is received. If you specify 0, then the system stops sending requests after the first error is returned. If you run an association on 50 instances and set MaxError to 10%, then the system stops sending the request when the sixth error is received. Executions that are already running an association when MaxErrors is reached are allowed to complete, but some of these executions may fail as well. If you need to ensure that there won't be more than max-errors failed executions, set MaxConcurrency to 1 so that executions proceed one at a time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociationName</i> The association name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastSuccessfulExecutionDate</i> The last date on which the association was successfully run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastExecutionDate</i> The date on which the association was last run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputLocation</i> An Amazon S3 bucket where you want to store the output details of the request.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>OutputLocation</b>: An Amazon S3 bucket where you want to store the output details of the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ScheduleExpression</i> A cron expression that specifies a schedule when the association runs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Targets</i> The instances targeted by the request. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociationId</i> The association ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> A description of the parameters for a document. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomationTargetParameterName</i> Specify the target for the association. This target is required for associations that use an Automation document and target resources by using rate controls.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentVersion</i> The document version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Overview</i> Information about the association.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Overview</b>: Information about the association.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The association status.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>: The association status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdateAssociationDate</i> The date when the association was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Date</i> The date when the association was made.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociationVersion</i> The association version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceId</i> The ID of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the Systems Manager document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AdditionalInfo</i> A user-defined string.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> The reason for the status.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The status.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Date</i> The date when the status changed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociationStatusAggregatedCount</i> Returns the number of targets for the association status. For example, if you created an association with two instances, and one of them was successful, this would return the count of instances by status.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DetailedStatus</i> A detailed status of the association.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the association. Status can be: Pending, Success, or Failed.<br/>An Amazon S3 bucket where you want to store the results of this request.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Location</b>: An Amazon S3 bucket where you want to store the results of this request.        """
        pass

    def update_association_status(self, AssociationStatus: AssociationStatus, InstanceId: str, Name: str) -> UpdateAssociationStatusResult:
        """Updates the status of the Systems Manager document associated with the specified instance.

        :param self:
        :param AssociationStatus:  The association status.:<br/>InstanceId:  The ID of the instance.<br/>Name:  The name of the Systems Manager document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AdditionalInfo</i> A user-defined string.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> The reason for the status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Date</i> The date when the status changed.<br/>
        :return: AssociationDescription:  Information about the association.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ComplianceSeverity</i> The severity level that is assigned to the association.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxConcurrency</i> The maximum number of targets allowed to run the association at the same time. You can specify a number, for example 10, or a percentage of the target set, for example 10%. The default value is 100%, which means all targets run the association at the same time. If a new instance starts and attempts to run an association while Systems Manager is running MaxConcurrency associations, the association is allowed to run. During the next association interval, the new instance will process its association within the limit specified for MaxConcurrency.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxErrors</i> The number of errors that are allowed before the system stops sending requests to run the association on additional targets. You can specify either an absolute number of errors, for example 10, or a percentage of the target set, for example 10%. If you specify 3, for example, the system stops sending requests when the fourth error is received. If you specify 0, then the system stops sending requests after the first error is returned. If you run an association on 50 instances and set MaxError to 10%, then the system stops sending the request when the sixth error is received. Executions that are already running an association when MaxErrors is reached are allowed to complete, but some of these executions may fail as well. If you need to ensure that there won't be more than max-errors failed executions, set MaxConcurrency to 1 so that executions proceed one at a time.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociationName</i> The association name.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastSuccessfulExecutionDate</i> The last date on which the association was successfully run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastExecutionDate</i> The date on which the association was last run.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OutputLocation</i> An Amazon S3 bucket where you want to store the output details of the request.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>OutputLocation</b>: An Amazon S3 bucket where you want to store the output details of the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ScheduleExpression</i> A cron expression that specifies a schedule when the association runs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Targets</i> The instances targeted by the request. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociationId</i> The association ID.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> A description of the parameters for a document. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutomationTargetParameterName</i> Specify the target for the association. This target is required for associations that use an Automation document and target resources by using rate controls.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentVersion</i> The document version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Overview</i> Information about the association.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Overview</b>: Information about the association.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The association status.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Status</b>: The association status.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdateAssociationDate</i> The date when the association was last updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Date</i> The date when the association was made.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociationVersion</i> The association version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InstanceId</i> The ID of the instance.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the Systems Manager document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AdditionalInfo</i> A user-defined string.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Message</i> The reason for the status.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The status.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Date</i> The date when the status changed.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>AssociationStatusAggregatedCount</i> Returns the number of targets for the association status. For example, if you created an association with two instances, and one of them was successful, this would return the count of instances by status.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DetailedStatus</i> A detailed status of the association.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the association. Status can be: Pending, Success, or Failed.<br/>An Amazon S3 bucket where you want to store the results of this request.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Location</b>: An Amazon S3 bucket where you want to store the results of this request.        """
        pass

    def update_document(self, Name: str, Content: str, Attachments: List[AttachmentsSource]=None, VersionName: str=None, DocumentVersion: str=None, DocumentFormat: str=None, TargetType: str=None) -> UpdateDocumentResult:
        """The document you want to update.

        :param self:
        :param TargetType:  Specify a new target type for the document.<br/>DocumentFormat:  Specify the document format for the new document version. Systems Manager supports JSON and YAML documents. JSON is the default format.<br/>DocumentVersion:  The version of the document that you want to update.<br/>VersionName:  An optional field specifying the version of the artifact you are updating with the document. For example, "Release 12, Update 6". This value is unique across all versions of a document, and cannot be changed.<br/>Name:  The name of the document that you want to update.<br/>Attachments:  A list of key and value pairs that describe attachments to a version of a document.<br/>Content:  A valid JSON or YAML string.<br/>
        :return: DocumentDescription:  A description of the document that was updated.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AttachmentsInformation</i> Details about the document attachments, including names, locations, sizes, etc.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> The tags, or metadata, that have been applied to the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetType</i> The target type which defines the kinds of resources the document can run on. For example, /AWS::EC2::Instance. For a list of valid resource types, see <a href="http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-template-resource-type-ref.html">AWS Resource Types Reference</a> in the <i>AWS CloudFormation User Guide</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentFormat</i> The document format, either JSON or YAML.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultVersion</i> The default version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestVersion</i> The latest version of the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SchemaVersion</i> The schema version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentType</i> The type of document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PlatformTypes</i> The list of OS platforms compatible with this Systems Manager document. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> A description of the parameters for a document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> A description of the document. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentVersion</i> The document version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StatusInformation</i> A message returned by AWS Systems Manager that explains the <code>Status</code> value. For example, a <code>Failed</code> status might be explained by the <code>StatusInformation</code> message, "The specified S3 bucket does not exist. Verify that the URL of the S3 bucket is correct."<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the Systems Manager document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedDate</i> The date when the document was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Owner</i> The AWS user account that created the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VersionName</i> The version of the artifact associated with the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the Systems Manager document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HashType</i> The hash type of the document. Valid values include <code>Sha256</code> or <code>Sha1</code>. <note> Sha1 hashes have been deprecated. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Hash</i> The Sha256 or Sha1 hash created by the system when the document was created.  <note> Sha1 hashes have been deprecated. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Sha1</i> The SHA1 hash of the document, which you can use for verification.<br/>        """
        pass

    def update_document_default_version(self, DocumentVersion: str, Name: str) -> UpdateDocumentDefaultVersionResult:
        """Set the default version of a document. 

        :param self:
        :param DocumentVersion:  The version of a custom document that you want to set as the default version.<br/>Name:  The name of a custom document that you want to set as the default version.<br/>
        :return: Description:  The description of a custom document that you want to set as the default version.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultVersionName</i> The default version of the artifact associated with the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultVersion</i> The default version of the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the document.<br/>        """
        pass

    def update_maintenance_window(self, WindowId: str, Name: str=None, Description: str=None, StartDate: str=None, EndDate: str=None, Schedule: str=None, ScheduleTimezone: str=None, Duration: int=None, Cutoff: int=None, AllowUnassociatedTargets: MaintenanceWindowAllowUnassociatedTargets=None, Enabled: MaintenanceWindowEnabled=None, Replace: Boolean=None) -> UpdateMaintenanceWindowResult:
        """Updates an existing maintenance window. Only specified parameters are modified.

        :param self:
        :param Replace:  If True, then all fields that are required by the CreateMaintenanceWindow action are also required for this API request. Optional fields that are not specified are set to null. <br/>Enabled:  Whether the maintenance window is enabled.<br/>AllowUnassociatedTargets:  Whether targets must be registered with the maintenance window before tasks can be defined for those targets.<br/>Cutoff:  The number of hours before the end of the maintenance window that Systems Manager stops scheduling new tasks for execution.<br/>Duration:  The duration of the maintenance window in hours.<br/>ScheduleTimezone:  The time zone that the scheduled maintenance window executions are based on, in Internet Assigned Numbers Authority (IANA) format. For example: "America/Los_Angeles", "etc/UTC", or "Asia/Seoul". For more information, see the <a href="https://www.iana.org/time-zones">Time Zone Database</a> on the IANA website.<br/>Schedule:  The schedule of the maintenance window in the form of a cron or rate expression.<br/>EndDate:  The date and time, in ISO-8601 Extended format, for when you want the maintenance window to become inactive. EndDate allows you to set a date and time in the future when the maintenance window will no longer run.<br/>StartDate:  The time zone that the scheduled maintenance window executions are based on, in Internet Assigned Numbers Authority (IANA) format. For example: "America/Los_Angeles", "etc/UTC", or "Asia/Seoul". For more information, see the <a href="https://www.iana.org/time-zones">Time Zone Database</a> on the IANA website.<br/>Description:  An optional description for the update request.<br/>Name:  The name of the maintenance window.<br/>WindowId:  The ID of the maintenance window to update.<br/>
        :return: Enabled:  Whether the maintenance window is enabled.<br/>AllowUnassociatedTargets:  Whether targets must be registered with the maintenance window before tasks can be defined for those targets.<br/>Cutoff:  The number of hours before the end of the maintenance window that Systems Manager stops scheduling new tasks for execution.<br/>Duration:  The duration of the maintenance window in hours.<br/>ScheduleTimezone:  The time zone that the scheduled maintenance window executions are based on, in Internet Assigned Numbers Authority (IANA) format. For example: "America/Los_Angeles", "etc/UTC", or "Asia/Seoul". For more information, see the <a href="https://www.iana.org/time-zones">Time Zone Database</a> on the IANA website.<br/>Schedule:  The schedule of the maintenance window in the form of a cron or rate expression.<br/>EndDate:  The date and time, in ISO-8601 Extended format, for when the maintenance window is scheduled to become inactive. The maintenance window will not run after this specified time.<br/>StartDate:  The date and time, in ISO-8601 Extended format, for when the maintenance window is scheduled to become active. The maintenance window will not run before this specified time.<br/>Description:  An optional description of the update.<br/>Name:  The name of the maintenance window.<br/>WindowId:  The ID of the created maintenance window.<br/>        """
        pass

    def update_maintenance_window_target(self, WindowTargetId: str, WindowId: str, Targets: List[Target]=None, OwnerInformation: str=None, Name: str=None, Description: str=None, Replace: Boolean=None) -> UpdateMaintenanceWindowTargetResult:
        """Modifies the target of an existing maintenance window. You can't change the target type, but you can change the following: The target from being an ID target to a Tag target, or a Tag target to an ID target. IDs for an ID target. Tags for a Tag target. Owner. Name. Description. If a parameter is null, then the corresponding field is not modified.

        :param self:
        :param Replace:  If True, then all fields that are required by the RegisterTargetWithMaintenanceWindow action are also required for this API request. Optional fields that are not specified are set to null.<br/>Description:  An optional description for the update.<br/>Name:  A name for the update.<br/>OwnerInformation:  User-provided value that will be included in any CloudWatch events raised while running tasks for these targets in this maintenance window.<br/>Targets:  The targets to add or replace.<br/>WindowTargetId:  The target ID to modify.<br/>WindowId:  The maintenance window ID with which to modify the target.<br/>
        :return: Description:  The updated description.<br/>Name:  The updated name.<br/>OwnerInformation:  The updated owner.<br/>Targets:  The updated targets.<br/>WindowTargetId:  The target ID specified in the update request.<br/>WindowId:  The maintenance window ID specified in the update request.<br/>        """
        pass

    def update_maintenance_window_task(self, WindowTaskId: str, WindowId: str, Targets: List[Target]=None, TaskArn: str=None, ServiceRoleArn: str=None, TaskParameters: MaintenanceWindowTaskParameters=None, TaskInvocationParameters: MaintenanceWindowTaskInvocationParameters=None, Priority: int=None, MaxConcurrency: str=None, MaxErrors: str=None, LoggingInfo: LoggingInfo=None, Name: str=None, Description: str=None, Replace: Boolean=None) -> UpdateMaintenanceWindowTaskResult:
        """Modifies a task assigned to a maintenance window. You can't change the task type, but you can change the following values: <ul> <li> TaskARN. For example, you can change a RUN_COMMAND task from AWS-RunPowerShellScript to AWS-RunShellScript. </li> <li> ServiceRoleArn </li> <li> TaskInvocationParameters </li> <li> Priority </li> <li> MaxConcurrency </li> <li> MaxErrors </li> </ul> If a parameter is null, then the corresponding field is not modified. Also, if you set Replace to true, then all fields required by the <a>RegisterTaskWithMaintenanceWindow</a> action are required for this request. Optional fields that aren't specified are set to null.

        :param self:
        :param Replace:  If True, then all fields that are required by the RegisterTaskWithMaintenanceWndow action are also required for this API request. Optional fields that are not specified are set to null.<br/>Description:  The new task description to specify.<br/>Name:  The new task name to specify.<br/>LoggingInfo:  The new logging location in Amazon S3 to specify. <note>  <code>LoggingInfo</code> has been deprecated. To specify an S3 bucket to contain logs, instead use the <code>OutputS3BucketName</code> and <code>OutputS3KeyPrefix</code> options in the <code>TaskInvocationParameters</code> structure. For information about how Systems Manager handles these options for the supported maintenance window task types, see <a>MaintenanceWindowTaskInvocationParameters</a>. </note>:<br/>MaxErrors:  The new <code>MaxErrors</code> value to specify. <code>MaxErrors</code> is the maximum number of errors that are allowed before the task stops being scheduled.<br/>MaxConcurrency:  The new <code>MaxConcurrency</code> value you want to specify. <code>MaxConcurrency</code> is the number of targets that are allowed to run this task in parallel.<br/>Priority:  The new task priority to specify. The lower the number, the higher the priority. Tasks that have the same priority are scheduled in parallel.<br/>TaskInvocationParameters:  The parameters that the task should use during execution. Populate only the fields that match the task type. All other fields should be empty.:<br/>TaskParameters:  The parameters to modify. <note>  <code>TaskParameters</code> has been deprecated. To specify parameters to pass to a task when it runs, instead use the <code>Parameters</code> option in the <code>TaskInvocationParameters</code> structure. For information about how Systems Manager handles these options for the supported maintenance window task types, see <a>MaintenanceWindowTaskInvocationParameters</a>. </note> The map has the following format: Key: string, between 1 and 255 characters Value: an array of strings, each string is between 1 and 255 characters<br/>ServiceRoleArn:  The ARN of the IAM service role for Systems Manager to assume when running a maintenance window task. If you do not specify a service role ARN, Systems Manager uses your account's service-linked role. If no service-linked role for Systems Manager exists in your account, it is created when you run <code>RegisterTaskWithMaintenanceWindow</code>. For more information, see the following topics in the in the <i>AWS Systems Manager User Guide</i>: <ul> <li>  <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/using-service-linked-roles.html#slr-permissions">Service-Linked Role Permissions for Systems Manager</a>  </li> <li>  <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/sysman-maintenance-permissions.html#maintenance-window-tasks-service-role">Should I Use a Service-Linked Role or a Custom Service Role to Run Maintenance Window Tasks? </a>  </li> </ul><br/>TaskArn:  The task ARN to modify.<br/>Targets:  The targets (either instances or tags) to modify. Instances are specified using Key=instanceids,Values=instanceID_1,instanceID_2. Tags are specified using Key=tag_name,Values=tag_value. <br/>WindowTaskId:  The task ID to modify.<br/>WindowId:  The maintenance window ID that contains the task to modify.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Lambda</i> The parameters for a LAMBDA task type.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Lambda</b>: The parameters for a LAMBDA task type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StepFunctions</i> The parameters for a STEP_FUNCTION task type.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StepFunctions</b>: The parameters for a STEP_FUNCTION task type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Automation</i> The parameters for an AUTOMATION task type.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Automation</b>: The parameters for an AUTOMATION task type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RunCommand</i> The parameters for a RUN_COMMAND task type.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RunCommand</b>: The parameters for a RUN_COMMAND task type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentHashType</i> SHA-256 or SHA-1. SHA-1 hashes have been deprecated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentHash</i> The SHA-256 or SHA-1 hash created by the system when the document was created. SHA-1 hashes have been deprecated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> Information about the commands to run.<br/>Configurations for sending notifications about command status changes on a per-instance basis.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NotificationConfig</b>: Configurations for sending notifications about command status changes on a per-instance basis.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> The parameters for the AUTOMATION task. For information about specifying and updating task parameters, see <a>RegisterTaskWithMaintenanceWindow</a> and <a>UpdateMaintenanceWindowTask</a>. <note>  <code>LoggingInfo</code> has been deprecated. To specify an S3 bucket to contain logs, instead use the <code>OutputS3BucketName</code> and <code>OutputS3KeyPrefix</code> options in the <code>TaskInvocationParameters</code> structure. For information about how Systems Manager handles these options for the supported maintenance window task types, see <a>MaintenanceWindowTaskInvocationParameters</a>.  <code>TaskParameters</code> has been deprecated. To specify parameters to pass to a task when it runs, instead use the <code>Parameters</code> option in the <code>TaskInvocationParameters</code> structure. For information about how Systems Manager handles these options for the supported maintenance window task types, see <a>MaintenanceWindowTaskInvocationParameters</a>. For AUTOMATION task types, Systems Manager ignores any values specified for these parameters. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentVersion</i> The version of an Automation document to use during task execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the STEP_FUNCTION task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Input</i> The inputs for the STEP_FUNCTION task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Payload</i> JSON to provide to your Lambda function as input.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Qualifier</i> (Optional) Specify a Lambda function version or alias name. If you specify a function version, the action uses the qualified function ARN to invoke a specific Lambda function. If you specify an alias name, the action uses the alias ARN to invoke the Lambda function version to which the alias points.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientContext</i> Pass client-specific information to the Lambda function that you are invoking. You can then process the client information in your Lambda function as you choose through the context variable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Region</i> The region where the Amazon S3 bucket is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3KeyPrefix</i> (Optional) The Amazon S3 bucket subfolder. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3BucketName</i> The name of an Amazon S3 bucket where execution logs are stored .<br/>
        :return: Description:  The updated task description.<br/>Name:  The updated task name.<br/>LoggingInfo:  The updated logging information in Amazon S3. <note>  <code>LoggingInfo</code> has been deprecated. To specify an S3 bucket to contain logs, instead use the <code>OutputS3BucketName</code> and <code>OutputS3KeyPrefix</code> options in the <code>TaskInvocationParameters</code> structure. For information about how Systems Manager handles these options for the supported maintenance window task types, see <a>MaintenanceWindowTaskInvocationParameters</a>. </note>:<br/>MaxErrors:  The updated MaxErrors value.<br/>MaxConcurrency:  The updated MaxConcurrency value.<br/>Priority:  The updated priority value.<br/>TaskInvocationParameters:  The updated parameter values.:<br/>TaskParameters:  The updated parameter values. <note>  <code>TaskParameters</code> has been deprecated. To specify parameters to pass to a task when it runs, instead use the <code>Parameters</code> option in the <code>TaskInvocationParameters</code> structure. For information about how Systems Manager handles these options for the supported maintenance window task types, see <a>MaintenanceWindowTaskInvocationParameters</a>. </note><br/>ServiceRoleArn:  The ARN of the IAM service role to use to publish Amazon Simple Notification Service (Amazon SNS) notifications for maintenance window Run Command tasks.<br/>TaskArn:  The updated task ARN value.<br/>Targets:  The updated target values.<br/>WindowTaskId:  The task ID of the maintenance window that was updated.<br/>WindowId:  The ID of the maintenance window that was updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Lambda</i> The parameters for a LAMBDA task type.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Lambda</b>: The parameters for a LAMBDA task type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StepFunctions</i> The parameters for a STEP_FUNCTION task type.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StepFunctions</b>: The parameters for a STEP_FUNCTION task type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Automation</i> The parameters for an AUTOMATION task type.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Automation</b>: The parameters for an AUTOMATION task type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RunCommand</i> The parameters for a RUN_COMMAND task type.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>RunCommand</b>: The parameters for a RUN_COMMAND task type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentHashType</i> SHA-256 or SHA-1. SHA-1 hashes have been deprecated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentHash</i> The SHA-256 or SHA-1 hash created by the system when the document was created. SHA-1 hashes have been deprecated.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Comment</i> Information about the commands to run.<br/>Configurations for sending notifications about command status changes on a per-instance basis.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>NotificationConfig</b>: Configurations for sending notifications about command status changes on a per-instance basis.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Parameters</i> The parameters for the AUTOMATION task. For information about specifying and updating task parameters, see <a>RegisterTaskWithMaintenanceWindow</a> and <a>UpdateMaintenanceWindowTask</a>. <note>  <code>LoggingInfo</code> has been deprecated. To specify an S3 bucket to contain logs, instead use the <code>OutputS3BucketName</code> and <code>OutputS3KeyPrefix</code> options in the <code>TaskInvocationParameters</code> structure. For information about how Systems Manager handles these options for the supported maintenance window task types, see <a>MaintenanceWindowTaskInvocationParameters</a>.  <code>TaskParameters</code> has been deprecated. To specify parameters to pass to a task when it runs, instead use the <code>Parameters</code> option in the <code>TaskInvocationParameters</code> structure. For information about how Systems Manager handles these options for the supported maintenance window task types, see <a>MaintenanceWindowTaskInvocationParameters</a>. For AUTOMATION task types, Systems Manager ignores any values specified for these parameters. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>DocumentVersion</i> The version of an Automation document to use during task execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the STEP_FUNCTION task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Input</i> The inputs for the STEP_FUNCTION task.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Payload</i> JSON to provide to your Lambda function as input.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Qualifier</i> (Optional) Specify a Lambda function version or alias name. If you specify a function version, the action uses the qualified function ARN to invoke a specific Lambda function. If you specify an alias name, the action uses the alias ARN to invoke the Lambda function version to which the alias points.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ClientContext</i> Pass client-specific information to the Lambda function that you are invoking. You can then process the client information in your Lambda function as you choose through the context variable.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Region</i> The region where the Amazon S3 bucket is located.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3KeyPrefix</i> (Optional) The Amazon S3 bucket subfolder. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3BucketName</i> The name of an Amazon S3 bucket where execution logs are stored .<br/>        """
        pass

    def update_managed_instance_role(self, IamRole: str, InstanceId: str) -> UpdateManagedInstanceRoleResult:
        """Assigns or changes an Amazon Identity and Access Management (IAM) role to the managed instance.

        :param self:
        :param IamRole:  The IAM role you want to assign or change.<br/>InstanceId:  The ID of the managed instance where you want to update the role.<br/>
        :return:         """
        pass

    def update_ops_item(self, OpsItemId: str, Description: str=None, OperationalData: OpsItemOperationalData=None, OperationalDataToDelete: List[String]=None, Notifications: List[OpsItemNotification]=None, Priority: int=None, RelatedOpsItems: List[RelatedOpsItem]=None, Status: str=None, Title: str=None) -> UpdateOpsItemResponse:
        """Edit or change an OpsItem. You must have permission in AWS Identity and Access Management (IAM) to update an OpsItem. For more information, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/OpsCenter-getting-started.html">Getting Started with OpsCenter</a> in the <i>AWS Systems Manager User Guide</i>. Operations engineers and IT professionals use OpsCenter to view, investigate, and remediate operational issues impacting the performance and health of their AWS resources. For more information, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/OpsCenter.html">AWS Systems Manager OpsCenter</a> in the <i>AWS Systems Manager User Guide</i>. 

        :param self:
        :param Title:  A short heading that describes the nature of the OpsItem and the impacted resource.<br/>OpsItemId:  The ID of the OpsItem.<br/>Status:  The OpsItem status. Status can be <code>Open</code>, <code>In Progress</code>, or <code>Resolved</code>. For more information, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/OpsCenter-working-with-OpsItems-editing-details.html">Editing OpsItem Details</a> in the <i>AWS Systems Manager User Guide</i>.<br/>RelatedOpsItems:  One or more OpsItems that share something in common with the current OpsItems. For example, related OpsItems can include OpsItems with similar error messages, impacted resources, or statuses for the impacted resource.<br/>Priority:  The importance of this OpsItem in relation to other OpsItems in the system.<br/>Notifications:  The Amazon Resource Name (ARN) of an SNS topic where notifications are sent when this OpsItem is edited or changed.<br/>OperationalDataToDelete:  Keys that you want to remove from the OperationalData map.<br/>OperationalData:  Add new keys or edit existing key-value pairs of the OperationalData map in the OpsItem object. Operational data is custom data that provides useful reference details about the OpsItem. For example, you can specify log files, error strings, license keys, troubleshooting tips, or other relevant data. You enter operational data as key-value pairs. The key has a maximum length of 128 characters. The value has a maximum size of 20 KB. <important> Operational data keys <i>can't</i> begin with the following: amazon, aws, amzn, ssm, /amazon, /aws, /amzn, /ssm. </important> You can choose to make the data searchable by other users in the account or you can restrict search access. Searchable data means that all users with access to the OpsItem Overview page (as provided by the <a>DescribeOpsItems</a> API action) can view and search on the specified data. Operational data that is not searchable is only viewable by users who have access to the OpsItem (as provided by the <a>GetOpsItem</a> API action). Use the <code>/aws/resources</code> key in OperationalData to specify a related resource in the request. Use the <code>/aws/automations</code> key in OperationalData to associate an Automation runbook with the OpsItem. To view AWS CLI example commands that use these keys, see <a href="http://docs.aws.amazon.com/systems-manager/latest/userguide/OpsCenter-creating-OpsItems.html#OpsCenter-manually-create-OpsItems">Creating OpsItems Manually</a> in the <i>AWS Systems Manager User Guide</i>.<br/>Description:  Update the information about the OpsItem. Provide enough information so that users reading this OpsItem for the first time understand the issue. <br/>
        :return:         """
        pass

    def update_patch_baseline(self, BaselineId: str, Name: str=None, GlobalFilters: PatchFilterGroup=None, ApprovalRules: PatchRuleGroup=None, ApprovedPatches: List[PatchId]=None, ApprovedPatchesComplianceLevel: str=None, ApprovedPatchesEnableNonSecurity: Boolean=None, RejectedPatches: List[PatchId]=None, RejectedPatchesAction: str=None, Description: str=None, Sources: List[PatchSource]=None, Replace: Boolean=None) -> UpdatePatchBaselineResult:
        """Modifies an existing patch baseline. Fields not specified in the request are left unchanged. <note> For information about valid key and value pairs in <code>PatchFilters</code> for each supported operating system type, see <a href="http://docs.aws.amazon.com/systems-manager/latest/APIReference/API_PatchFilter.html">PatchFilter</a>. </note>

        :param self:
        :param Replace:  If True, then all fields that are required by the CreatePatchBaseline action are also required for this API request. Optional fields that are not specified are set to null.<br/>Sources:  Information about the patches to use to update the instances, including target operating systems and source repositories. Applies to Linux instances only.<br/>Description:  A description of the patch baseline.<br/>RejectedPatchesAction:  The action for Patch Manager to take on patches included in the RejectedPackages list. <ul> <li>  <b>ALLOW_AS_DEPENDENCY</b>: A package in the Rejected patches list is installed only if it is a dependency of another package. It is considered compliant with the patch baseline, and its status is reported as <i>InstalledOther</i>. This is the default action if no option is specified. </li> <li>  <b>BLOCK</b>: Packages in the RejectedPatches list, and packages that include them as dependencies, are not installed under any circumstances. If a package was installed before it was added to the Rejected patches list, it is considered non-compliant with the patch baseline, and its status is reported as <i>InstalledRejected</i>. </li> </ul><br/>RejectedPatches:  A list of explicitly rejected patches for the baseline. For information about accepted formats for lists of approved patches and rejected patches, see <a href="https://docs.aws.amazon.com/systems-manager/latest/userguide/patch-manager-approved-rejected-package-name-formats.html">Package Name Formats for Approved and Rejected Patch Lists</a> in the <i>AWS Systems Manager User Guide</i>.<br/>ApprovedPatchesEnableNonSecurity:  Indicates whether the list of approved patches includes non-security updates that should be applied to the instances. The default value is 'false'. Applies to Linux instances only.<br/>ApprovedPatchesComplianceLevel:  Assigns a new compliance severity level to an existing patch baseline.<br/>ApprovedPatches:  A list of explicitly approved patches for the baseline. For information about accepted formats for lists of approved patches and rejected patches, see <a href="https://docs.aws.amazon.com/systems-manager/latest/userguide/patch-manager-approved-rejected-package-name-formats.html">Package Name Formats for Approved and Rejected Patch Lists</a> in the <i>AWS Systems Manager User Guide</i>.<br/>ApprovalRules:  A set of rules used to include patches in the baseline.:<br/>GlobalFilters:  A set of global filters used to include patches in the baseline.:<br/>Name:  The name of the patch baseline.<br/>BaselineId:  The ID of the patch baseline to update.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PatchFilters</i> The set of patch filters that make up the group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PatchRules</i> The rules that make up the rule group.<br/>
        :return: Sources:  Information about the patches to use to update the instances, including target operating systems and source repositories. Applies to Linux instances only.<br/>Description:  A description of the Patch Baseline.<br/>ModifiedDate:  The date when the patch baseline was last modified.<br/>CreatedDate:  The date when the patch baseline was created.<br/>RejectedPatchesAction:  The action specified to take on patches included in the RejectedPatches list. A patch can be allowed only if it is a dependency of another package, or blocked entirely along with packages that include it as a dependency.<br/>RejectedPatches:  A list of explicitly rejected patches for the baseline.<br/>ApprovedPatchesEnableNonSecurity:  Indicates whether the list of approved patches includes non-security updates that should be applied to the instances. The default value is 'false'. Applies to Linux instances only.<br/>ApprovedPatchesComplianceLevel:  The compliance severity level assigned to the patch baseline after the update completed.<br/>ApprovedPatches:  A list of explicitly approved patches for the baseline.<br/>ApprovalRules:  A set of rules used to include patches in the baseline.:<br/>GlobalFilters:  A set of global filters used to exclude patches from the baseline.:<br/>OperatingSystem:  The operating system rule used by the updated patch baseline.<br/>Name:  The name of the patch baseline.<br/>BaselineId:  The ID of the deleted patch baseline.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PatchFilters</i> The set of patch filters that make up the group.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>PatchRules</i> The rules that make up the rule group.<br/>        """
        pass

    def update_service_setting(self, SettingValue: str, SettingId: str) -> UpdateServiceSettingResult:
        """ <code>ServiceSetting</code> is an account-level setting for an AWS service. This setting defines how a user interacts with or uses a service or a feature of a service. For example, if an AWS service charges money to the account based on feature or service usage, then the AWS service team might create a default setting of "false". This means the user can't use this feature unless they change the setting to "true" and intentionally opt in for a paid feature. Services map a <code>SettingId</code> object to a setting value. AWS services teams define the default value for a <code>SettingId</code>. You can't create a new <code>SettingId</code>, but you can overwrite the default value if you have the <code>ssm:UpdateServiceSetting</code> permission for the setting. Use the <a>GetServiceSetting</a> API action to view the current value. Or, use the <a>ResetServiceSetting</a> to change the value back to the original value defined by the AWS service team. Update the service setting for the account. 

        :param self:
        :param SettingValue:  The new value to specify for the service setting.<br/>SettingId:  The ID of the service setting to update.<br/>
        :return:         """
        pass



class SFN(BaseClient):
    """<fullname>AWS Step Functions</fullname> AWS Step Functions is a service that lets you coordinate the components of distributed applications and microservices using visual workflows. You can use Step Functions to build applications from individual components, each of which performs a discrete function, or <i>task</i>, allowing you to scale and change applications quickly. Step Functions provides a console that helps visualize the components of your application as a series of steps. Step Functions automatically triggers and tracks each step, and retries steps when there are errors, so your application executes predictably and in the right order every time. Step Functions logs the state of each step, so you can quickly diagnose and debug any issues. Step Functions manages operations and underlying infrastructure to ensure your application is available at any scale. You can run tasks on AWS, your own servers, or any system that has access to AWS. You can access and use Step Functions using the console, the AWS SDKs, or an HTTP API. For more information about Step Functions, see the <i> <a href="https://docs.aws.amazon.com/step-functions/latest/dg/welcome.html">AWS Step Functions Developer Guide</a> </i>."""


    class GetExecutionHistoryPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_execution_history` for valid parameters"""
            pass
    class ListActivitiesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_activities` for valid parameters"""
            pass
    class ListExecutionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_executions` for valid parameters"""
            pass
    class ListStateMachinesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_state_machines` for valid parameters"""
            pass


    class CreateActivityInput(Mapping):
        pass
    
    class CreateActivityOutput(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreateStateMachineInput(Mapping):
        pass
    
    class CreateStateMachineOutput(Mapping):
        pass
    
    class DeleteActivityInput(Mapping):
        pass
    
    class DeleteActivityOutput(Mapping):
        pass
    
    class DeleteStateMachineInput(Mapping):
        pass
    
    class DeleteStateMachineOutput(Mapping):
        pass
    
    class DescribeActivityInput(Mapping):
        pass
    
    class DescribeActivityOutput(Mapping):
        pass
    
    class DescribeExecutionInput(Mapping):
        pass
    
    class DescribeExecutionOutput(Mapping):
        pass
    
    class DescribeStateMachineInput(Mapping):
        pass
    
    class DescribeStateMachineOutput(Mapping):
        pass
    
    class DescribeStateMachineForExecutionInput(Mapping):
        pass
    
    class DescribeStateMachineForExecutionOutput(Mapping):
        pass
    
    class GetActivityTaskInput(Mapping):
        pass
    
    class GetActivityTaskOutput(Mapping):
        pass
    
    class GetExecutionHistoryInput(Mapping):
        pass
    
    class GetExecutionHistoryOutput(Mapping):
        pass
    
    class ReverseOrder(object):
        pass
    
    class ListActivitiesInput(Mapping):
        pass
    
    class ListActivitiesOutput(Mapping):
        pass
    
    class ListExecutionsInput(Mapping):
        pass
    
    class ListExecutionsOutput(Mapping):
        pass
    
    class ListStateMachinesInput(Mapping):
        pass
    
    class ListStateMachinesOutput(Mapping):
        pass
    
    class ListTagsForResourceInput(Mapping):
        pass
    
    class ListTagsForResourceOutput(Mapping):
        pass
    
    class SendTaskFailureInput(Mapping):
        pass
    
    class SendTaskFailureOutput(Mapping):
        pass
    
    class SendTaskHeartbeatInput(Mapping):
        pass
    
    class SendTaskHeartbeatOutput(Mapping):
        pass
    
    class SendTaskSuccessInput(Mapping):
        pass
    
    class SendTaskSuccessOutput(Mapping):
        pass
    
    class StartExecutionInput(Mapping):
        pass
    
    class StartExecutionOutput(Mapping):
        pass
    
    class StopExecutionInput(Mapping):
        pass
    
    class StopExecutionOutput(Mapping):
        pass
    
    class TagResourceInput(Mapping):
        pass
    
    class TagResourceOutput(Mapping):
        pass
    
    class UntagResourceInput(Mapping):
        pass
    
    class UntagResourceOutput(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateStateMachineInput(Mapping):
        pass
    
    class UpdateStateMachineOutput(Mapping):
        pass
    
    def create_activity(self, name: str, tags: List[Tag]=None) -> CreateActivityOutput:
        """Creates an activity. An activity is a task that you write in any programming language and host on any machine that has access to AWS Step Functions. Activities must poll Step Functions using the <code>GetActivityTask</code> API action and respond using <code>SendTask*</code> API actions. This function lets Step Functions know the existence of your activity and returns an identifier for use in a state machine and when polling from the activity. <note> This operation is eventually consistent. The results are best effort and may not reflect very recent updates and changes. </note>

        :param self:
        :param tags:  The list of tags to add to a resource.<br/>name:  The name of the activity to create. This name must be unique for your AWS account and region for 90 days. For more information, see <a href="https://docs.aws.amazon.com/step-functions/latest/dg/limits.html#service-limits-state-machine-executions"> Limits Related to State Machine Executions</a> in the <i>AWS Step Functions Developer Guide</i>. A name must <i>not</i> contain: <ul> <li> whitespace </li> <li> brackets <code>&lt; &gt; { } [ ]</code>  </li> <li> wildcard characters <code>? *</code>  </li> <li> special characters <code>" # % \ ^ | ~ ` $ &amp; , ; : /</code>  </li> <li> control characters (<code>U+0000-001F</code>, <code>U+007F-009F</code>) </li> </ul><br/>
        :return: creationDate:  The date the activity is created.<br/>activityArn:  The Amazon Resource Name (ARN) that identifies the created activity.<br/>        """
        pass

    def create_state_machine(self, roleArn: str, definition: str, name: str, tags: List[Tag]=None) -> CreateStateMachineOutput:
        """Creates a state machine. A state machine consists of a collection of states that can do work (<code>Task</code> states), determine to which states to transition next (<code>Choice</code> states), stop an execution with an error (<code>Fail</code> states), and so on. State machines are specified using a JSON-based, structured language. <note> This operation is eventually consistent. The results are best effort and may not reflect very recent updates and changes. </note>

        :param self:
        :param tags:  Tags to be added when creating a state machine.<br/>roleArn:  The Amazon Resource Name (ARN) of the IAM role to use for this state machine.<br/>definition:  The Amazon States Language definition of the state machine. See <a href="https://docs.aws.amazon.com/step-functions/latest/dg/concepts-amazon-states-language.html">Amazon States Language</a>.<br/>name:  The name of the state machine.  A name must <i>not</i> contain: <ul> <li> whitespace </li> <li> brackets <code>&lt; &gt; { } [ ]</code>  </li> <li> wildcard characters <code>? *</code>  </li> <li> special characters <code>" # % \ ^ | ~ ` $ &amp; , ; : /</code>  </li> <li> control characters (<code>U+0000-001F</code>, <code>U+007F-009F</code>) </li> </ul><br/>
        :return: creationDate:  The date the state machine is created.<br/>stateMachineArn:  The Amazon Resource Name (ARN) that identifies the created state machine.<br/>        """
        pass

    def delete_activity(self, activityArn: str) -> DeleteActivityOutput:
        """Deletes an activity.

        :param self:
        :param activityArn:  The Amazon Resource Name (ARN) of the activity to delete.<br/>
        :return:         """
        pass

    def delete_state_machine(self, stateMachineArn: str) -> DeleteStateMachineOutput:
        """Deletes a state machine. This is an asynchronous operation: It sets the state machine's status to <code>DELETING</code> and begins the deletion process. Each state machine execution is deleted the next time it makes a state transition. <note> The state machine itself is deleted after all executions are completed or deleted. </note>

        :param self:
        :param stateMachineArn:  The Amazon Resource Name (ARN) of the state machine to delete.<br/>
        :return:         """
        pass

    def describe_activity(self, activityArn: str) -> DescribeActivityOutput:
        """Describes an activity. <note> This operation is eventually consistent. The results are best effort and may not reflect very recent updates and changes. </note>

        :param self:
        :param activityArn:  The Amazon Resource Name (ARN) of the activity to describe.<br/>
        :return: creationDate:  The date the activity is created.<br/>name:  The name of the activity. A name must <i>not</i> contain: <ul> <li> whitespace </li> <li> brackets <code>&lt; &gt; { } [ ]</code>  </li> <li> wildcard characters <code>? *</code>  </li> <li> special characters <code>" # % \ ^ | ~ ` $ &amp; , ; : /</code>  </li> <li> control characters (<code>U+0000-001F</code>, <code>U+007F-009F</code>) </li> </ul><br/>activityArn:  The Amazon Resource Name (ARN) that identifies the activity.<br/>        """
        pass

    def describe_execution(self, executionArn: str) -> DescribeExecutionOutput:
        """Describes an execution. <note> This operation is eventually consistent. The results are best effort and may not reflect very recent updates and changes. </note>

        :param self:
        :param executionArn:  The Amazon Resource Name (ARN) of the execution to describe.<br/>
        :return: output:  The JSON output data of the execution. <note> This field is set only if the execution succeeds. If the execution fails, this field is null. </note><br/>input:  The string that contains the JSON input data of the execution.<br/>stopDate:  If the execution has already ended, the date the execution stopped.<br/>startDate:  The date the execution is started.<br/>status:  The current status of the execution.<br/>name:  The name of the execution. A name must <i>not</i> contain: <ul> <li> whitespace </li> <li> brackets <code>&lt; &gt; { } [ ]</code>  </li> <li> wildcard characters <code>? *</code>  </li> <li> special characters <code>" # % \ ^ | ~ ` $ &amp; , ; : /</code>  </li> <li> control characters (<code>U+0000-001F</code>, <code>U+007F-009F</code>) </li> </ul><br/>stateMachineArn:  The Amazon Resource Name (ARN) of the executed stated machine.<br/>executionArn:  The Amazon Resource Name (ARN) that identifies the execution.<br/>        """
        pass

    def describe_state_machine(self, stateMachineArn: str) -> DescribeStateMachineOutput:
        """Describes a state machine. <note> This operation is eventually consistent. The results are best effort and may not reflect very recent updates and changes. </note>

        :param self:
        :param stateMachineArn:  The Amazon Resource Name (ARN) of the state machine to describe.<br/>
        :return: creationDate:  The date the state machine is created.<br/>roleArn:  The Amazon Resource Name (ARN) of the IAM role used when creating this state machine. (The IAM role maintains security by granting Step Functions access to AWS resources.)<br/>definition:  The Amazon States Language definition of the state machine. See <a href="https://docs.aws.amazon.com/step-functions/latest/dg/concepts-amazon-states-language.html">Amazon States Language</a>.<br/>status:  The current status of the state machine.<br/>name:  The name of the state machine. A name must <i>not</i> contain: <ul> <li> whitespace </li> <li> brackets <code>&lt; &gt; { } [ ]</code>  </li> <li> wildcard characters <code>? *</code>  </li> <li> special characters <code>" # % \ ^ | ~ ` $ &amp; , ; : /</code>  </li> <li> control characters (<code>U+0000-001F</code>, <code>U+007F-009F</code>) </li> </ul><br/>stateMachineArn:  The Amazon Resource Name (ARN) that identifies the state machine.<br/>        """
        pass

    def describe_state_machine_for_execution(self, executionArn: str) -> DescribeStateMachineForExecutionOutput:
        """Describes the state machine associated with a specific execution. <note> This operation is eventually consistent. The results are best effort and may not reflect very recent updates and changes. </note>

        :param self:
        :param executionArn:  The Amazon Resource Name (ARN) of the execution you want state machine information for.<br/>
        :return: updateDate:  The date and time the state machine associated with an execution was updated. For a newly created state machine, this is the creation date.<br/>roleArn:  The Amazon Resource Name (ARN) of the IAM role of the State Machine for the execution. <br/>definition:  The Amazon States Language definition of the state machine. See <a href="https://docs.aws.amazon.com/step-functions/latest/dg/concepts-amazon-states-language.html">Amazon States Language</a>.<br/>name:  The name of the state machine associated with the execution.<br/>stateMachineArn:  The Amazon Resource Name (ARN) of the state machine associated with the execution.<br/>        """
        pass

    def get_activity_task(self, activityArn: str, workerName: str=None) -> GetActivityTaskOutput:
        """Used by workers to retrieve a task (with the specified activity ARN) which has been scheduled for execution by a running state machine. This initiates a long poll, where the service holds the HTTP connection open and responds as soon as a task becomes available (i.e. an execution of a task of this type is needed.) The maximum time the service holds on to the request before responding is 60 seconds. If no task is available within 60 seconds, the poll returns a <code>taskToken</code> with a null string. <important> Workers should set their client side socket timeout to at least 65 seconds (5 seconds higher than the maximum time the service may hold the poll request). Polling with <code>GetActivityTask</code> can cause latency in some implementations. See <a href="https://docs.aws.amazon.com/step-functions/latest/dg/bp-activity-pollers.html">Avoid Latency When Polling for Activity Tasks</a> in the Step Functions Developer Guide. </important>

        :param self:
        :param workerName:  You can provide an arbitrary name in order to identify the worker that the task is assigned to. This name is used when it is logged in the execution history.<br/>activityArn:  The Amazon Resource Name (ARN) of the activity to retrieve tasks from (assigned when you create the task using <a>CreateActivity</a>.)<br/>
        :return: input:  The string that contains the JSON input data for the task.<br/>taskToken:  A token that identifies the scheduled task. This token must be copied and included in subsequent calls to <a>SendTaskHeartbeat</a>, <a>SendTaskSuccess</a> or <a>SendTaskFailure</a> in order to report the progress or completion of the task.<br/>        """
        pass

    def get_execution_history(self, executionArn: str, maxResults: int=None, reverseOrder: ReverseOrder=None, nextToken: str=None) -> GetExecutionHistoryOutput:
        """Returns the history of the specified execution as a list of events. By default, the results are returned in ascending order of the <code>timeStamp</code> of the events. Use the <code>reverseOrder</code> parameter to get the latest events first. If <code>nextToken</code> is returned, there are more results available. The value of <code>nextToken</code> is a unique pagination token for each page. Make the call again using the returned token to retrieve the next page. Keep all other arguments unchanged. Each pagination token expires after 24 hours. Using an expired pagination token will return an <i>HTTP 400 InvalidToken</i> error.

        :param self:
        :param nextToken:  If <code>nextToken</code> is returned, there are more results available. The value of <code>nextToken</code> is a unique pagination token for each page. Make the call again using the returned token to retrieve the next page. Keep all other arguments unchanged. Each pagination token expires after 24 hours. Using an expired pagination token will return an <i>HTTP 400 InvalidToken</i> error.<br/>reverseOrder:  Lists events in descending order of their <code>timeStamp</code>.<br/>maxResults:  The maximum number of results that are returned per call. You can use <code>nextToken</code> to obtain further pages of results. The default is 100 and the maximum allowed page size is 1000. A value of 0 uses the default. This is only an upper limit. The actual number of results returned per call might be fewer than the specified maximum.<br/>executionArn:  The Amazon Resource Name (ARN) of the execution.<br/>
        :return: nextToken:  If <code>nextToken</code> is returned, there are more results available. The value of <code>nextToken</code> is a unique pagination token for each page. Make the call again using the returned token to retrieve the next page. Keep all other arguments unchanged. Each pagination token expires after 24 hours. Using an expired pagination token will return an <i>HTTP 400 InvalidToken</i> error.<br/>events:  The list of events that occurred in the execution.<br/>        """
        pass

    def list_activities(self, maxResults: int=None, nextToken: str=None) -> ListActivitiesOutput:
        """Lists the existing activities. If <code>nextToken</code> is returned, there are more results available. The value of <code>nextToken</code> is a unique pagination token for each page. Make the call again using the returned token to retrieve the next page. Keep all other arguments unchanged. Each pagination token expires after 24 hours. Using an expired pagination token will return an <i>HTTP 400 InvalidToken</i> error. <note> This operation is eventually consistent. The results are best effort and may not reflect very recent updates and changes. </note>

        :param self:
        :param nextToken:  If <code>nextToken</code> is returned, there are more results available. The value of <code>nextToken</code> is a unique pagination token for each page. Make the call again using the returned token to retrieve the next page. Keep all other arguments unchanged. Each pagination token expires after 24 hours. Using an expired pagination token will return an <i>HTTP 400 InvalidToken</i> error.<br/>maxResults:  The maximum number of results that are returned per call. You can use <code>nextToken</code> to obtain further pages of results. The default is 100 and the maximum allowed page size is 1000. A value of 0 uses the default. This is only an upper limit. The actual number of results returned per call might be fewer than the specified maximum.<br/>
        :return: nextToken:  If <code>nextToken</code> is returned, there are more results available. The value of <code>nextToken</code> is a unique pagination token for each page. Make the call again using the returned token to retrieve the next page. Keep all other arguments unchanged. Each pagination token expires after 24 hours. Using an expired pagination token will return an <i>HTTP 400 InvalidToken</i> error.<br/>activities:  The list of activities.<br/>        """
        pass

    def list_executions(self, stateMachineArn: str, statusFilter: str=None, maxResults: int=None, nextToken: str=None) -> ListExecutionsOutput:
        """Lists the executions of a state machine that meet the filtering criteria. Results are sorted by time, with the most recent execution first. If <code>nextToken</code> is returned, there are more results available. The value of <code>nextToken</code> is a unique pagination token for each page. Make the call again using the returned token to retrieve the next page. Keep all other arguments unchanged. Each pagination token expires after 24 hours. Using an expired pagination token will return an <i>HTTP 400 InvalidToken</i> error. <note> This operation is eventually consistent. The results are best effort and may not reflect very recent updates and changes. </note>

        :param self:
        :param nextToken:  If <code>nextToken</code> is returned, there are more results available. The value of <code>nextToken</code> is a unique pagination token for each page. Make the call again using the returned token to retrieve the next page. Keep all other arguments unchanged. Each pagination token expires after 24 hours. Using an expired pagination token will return an <i>HTTP 400 InvalidToken</i> error.<br/>maxResults:  The maximum number of results that are returned per call. You can use <code>nextToken</code> to obtain further pages of results. The default is 100 and the maximum allowed page size is 1000. A value of 0 uses the default. This is only an upper limit. The actual number of results returned per call might be fewer than the specified maximum.<br/>statusFilter:  If specified, only list the executions whose current execution status matches the given filter.<br/>stateMachineArn:  The Amazon Resource Name (ARN) of the state machine whose executions is listed.<br/>
        :return: nextToken:  If <code>nextToken</code> is returned, there are more results available. The value of <code>nextToken</code> is a unique pagination token for each page. Make the call again using the returned token to retrieve the next page. Keep all other arguments unchanged. Each pagination token expires after 24 hours. Using an expired pagination token will return an <i>HTTP 400 InvalidToken</i> error.<br/>executions:  The list of matching executions.<br/>        """
        pass

    def list_state_machines(self, maxResults: int=None, nextToken: str=None) -> ListStateMachinesOutput:
        """Lists the existing state machines. If <code>nextToken</code> is returned, there are more results available. The value of <code>nextToken</code> is a unique pagination token for each page. Make the call again using the returned token to retrieve the next page. Keep all other arguments unchanged. Each pagination token expires after 24 hours. Using an expired pagination token will return an <i>HTTP 400 InvalidToken</i> error. <note> This operation is eventually consistent. The results are best effort and may not reflect very recent updates and changes. </note>

        :param self:
        :param nextToken:  If <code>nextToken</code> is returned, there are more results available. The value of <code>nextToken</code> is a unique pagination token for each page. Make the call again using the returned token to retrieve the next page. Keep all other arguments unchanged. Each pagination token expires after 24 hours. Using an expired pagination token will return an <i>HTTP 400 InvalidToken</i> error.<br/>maxResults:  The maximum number of results that are returned per call. You can use <code>nextToken</code> to obtain further pages of results. The default is 100 and the maximum allowed page size is 1000. A value of 0 uses the default. This is only an upper limit. The actual number of results returned per call might be fewer than the specified maximum.<br/>
        :return: nextToken:  If <code>nextToken</code> is returned, there are more results available. The value of <code>nextToken</code> is a unique pagination token for each page. Make the call again using the returned token to retrieve the next page. Keep all other arguments unchanged. Each pagination token expires after 24 hours. Using an expired pagination token will return an <i>HTTP 400 InvalidToken</i> error.<br/>stateMachines:  <br/>        """
        pass

    def list_tags_for_resource(self, resourceArn: str) -> ListTagsForResourceOutput:
        """List tags for a given resource.

        :param self:
        :param resourceArn:  The Amazon Resource Name (ARN) for the Step Functions state machine or activity.<br/>
        :return: tags:  An array of tags associated with the resource.<br/>        """
        pass

    def send_task_failure(self, taskToken: str, error: str=None, cause: str=None) -> SendTaskFailureOutput:
        """Used by workers to report that the task identified by the <code>taskToken</code> failed.

        :param self:
        :param cause:  A more detailed explanation of the cause of the failure.<br/>error:  The error code of the failure.<br/>taskToken:  The token that represents this task. Task tokens are generated by the service when the tasks are assigned to a worker (see GetActivityTask::taskToken).<br/>
        :return:         """
        pass

    def send_task_heartbeat(self, taskToken: str) -> SendTaskHeartbeatOutput:
        """Used by workers to report to the service that the task represented by the specified <code>taskToken</code> is still making progress. This action resets the <code>Heartbeat</code> clock. The <code>Heartbeat</code> threshold is specified in the state machine's Amazon States Language definition. This action does not in itself create an event in the execution history. However, if the task times out, the execution history contains an <code>ActivityTimedOut</code> event. <note> The <code>Timeout</code> of a task, defined in the state machine's Amazon States Language definition, is its maximum allowed duration, regardless of the number of <a>SendTaskHeartbeat</a> requests received. </note> <note> This operation is only useful for long-lived tasks to report the liveliness of the task. </note>

        :param self:
        :param taskToken:  The token that represents this task. Task tokens are generated by the service when the tasks are assigned to a worker (see <a>GetActivityTaskOutput$taskToken</a>).<br/>
        :return:         """
        pass

    def send_task_success(self, output: str, taskToken: str) -> SendTaskSuccessOutput:
        """Used by workers to report that the task identified by the <code>taskToken</code> completed successfully.

        :param self:
        :param output:  The JSON output of the task.<br/>taskToken:  The token that represents this task. Task tokens are generated by the service when the tasks are assigned to a worker (see <a>GetActivityTaskOutput$taskToken</a>).<br/>
        :return:         """
        pass

    def start_execution(self, stateMachineArn: str, name: str=None, input: str=None) -> StartExecutionOutput:
        """Starts a state machine execution. <note>  <code>StartExecution</code> is idempotent. If <code>StartExecution</code> is called with the same name and input as a running execution, the call will succeed and return the same response as the original request. If the execution is closed or if the input is different, it will return a 400 <code>ExecutionAlreadyExists</code> error. Names can be reused after 90 days.  </note>

        :param self:
        :param input:  The string that contains the JSON input data for the execution, for example:  <code>"input": "{\"first_name\" : \"test\"}"</code>  <note> If you don't include any JSON input data, you still must include the two braces, for example: <code>"input": "{}"</code>  </note><br/>name:  The name of the execution. This name must be unique for your AWS account, region, and state machine for 90 days. For more information, see <a href="https://docs.aws.amazon.com/step-functions/latest/dg/limits.html#service-limits-state-machine-executions"> Limits Related to State Machine Executions</a> in the <i>AWS Step Functions Developer Guide</i>. A name must <i>not</i> contain: <ul> <li> whitespace </li> <li> brackets <code>&lt; &gt; { } [ ]</code>  </li> <li> wildcard characters <code>? *</code>  </li> <li> special characters <code>" # % \ ^ | ~ ` $ &amp; , ; : /</code>  </li> <li> control characters (<code>U+0000-001F</code>, <code>U+007F-009F</code>) </li> </ul><br/>stateMachineArn:  The Amazon Resource Name (ARN) of the state machine to execute.<br/>
        :return: startDate:  The date the execution is started.<br/>executionArn:  The Amazon Resource Name (ARN) that identifies the execution.<br/>        """
        pass

    def stop_execution(self, executionArn: str, error: str=None, cause: str=None) -> StopExecutionOutput:
        """Stops an execution.

        :param self:
        :param cause:  A more detailed explanation of the cause of the failure.<br/>error:  The error code of the failure.<br/>executionArn:  The Amazon Resource Name (ARN) of the execution to stop.<br/>
        :return: stopDate:  The date the execution is stopped.<br/>        """
        pass

    def tag_resource(self, tags: List[Tag], resourceArn: str) -> TagResourceOutput:
        """Add a tag to a Step Functions resource.

        :param self:
        :param tags:  The list of tags to add to a resource. Tags may only contain unicode letters, digits, whitespace, or these symbols: <code>_ . : / = + - @</code>.<br/>resourceArn:  The Amazon Resource Name (ARN) for the Step Functions state machine or activity.<br/>
        :return:         """
        pass

    def untag_resource(self, tagKeys: List[TagKey], resourceArn: str) -> UntagResourceOutput:
        """Remove a tag from a Step Functions resource

        :param self:
        :param tagKeys:  The list of tags to remove from the resource.<br/>resourceArn:  The Amazon Resource Name (ARN) for the Step Functions state machine or activity.<br/>
        :return:         """
        pass

    def update_state_machine(self, stateMachineArn: str, definition: str=None, roleArn: str=None) -> UpdateStateMachineOutput:
        """Updates an existing state machine by modifying its <code>definition</code> and/or <code>roleArn</code>. Running executions will continue to use the previous <code>definition</code> and <code>roleArn</code>. You must include at least one of <code>definition</code> or <code>roleArn</code> or you will receive a <code>MissingRequiredParameter</code> error. <note> All <code>StartExecution</code> calls within a few seconds will use the updated <code>definition</code> and <code>roleArn</code>. Executions started immediately after calling <code>UpdateStateMachine</code> may use the previous state machine <code>definition</code> and <code>roleArn</code>.  </note>

        :param self:
        :param roleArn:  The Amazon Resource Name (ARN) of the IAM role of the state machine.<br/>definition:  The Amazon States Language definition of the state machine. See <a href="https://docs.aws.amazon.com/step-functions/latest/dg/concepts-amazon-states-language.html">Amazon States Language</a>.<br/>stateMachineArn:  The Amazon Resource Name (ARN) of the state machine.<br/>
        :return: updateDate:  The date and time the state machine was updated.<br/>        """
        pass



class StorageGateway(BaseClient):
    """<fullname>AWS Storage Gateway Service</fullname> AWS Storage Gateway is the service that connects an on-premises software appliance with cloud-based storage to provide seamless and secure integration between an organization's on-premises IT environment and the AWS storage infrastructure. The service enables you to securely upload data to the AWS cloud for cost effective backup and rapid disaster recovery. Use the following links to get started using the <i>AWS Storage Gateway Service API Reference</i>: <ul> <li>  <a href="https://docs.aws.amazon.com/storagegateway/latest/userguide/AWSStorageGatewayAPI.html#AWSStorageGatewayHTTPRequestsHeaders">AWS Storage Gateway Required Request Headers</a>: Describes the required headers that you must send with every POST request to AWS Storage Gateway. </li> <li>  <a href="https://docs.aws.amazon.com/storagegateway/latest/userguide/AWSStorageGatewayAPI.html#AWSStorageGatewaySigningRequests">Signing Requests</a>: AWS Storage Gateway requires that you authenticate every request you send; this topic describes how sign such a request. </li> <li>  <a href="https://docs.aws.amazon.com/storagegateway/latest/userguide/AWSStorageGatewayAPI.html#APIErrorResponses">Error Responses</a>: Provides reference information about AWS Storage Gateway errors. </li> <li>  <a href="https://docs.aws.amazon.com/storagegateway/latest/APIReference/API_Operations.html">Operations in AWS Storage Gateway</a>: Contains detailed descriptions of all AWS Storage Gateway operations, their request parameters, response elements, possible errors, and examples of requests and responses. </li> <li>  <a href="http://docs.aws.amazon.com/general/latest/gr/rande.html#sg_region">AWS Storage Gateway Regions and Endpoints:</a> Provides a list of each AWS region and endpoints available for use with AWS Storage Gateway.  </li> </ul> <note> AWS Storage Gateway resource IDs are in uppercase. When you use these resource IDs with the Amazon EC2 API, EC2 expects resource IDs in lowercase. You must change your resource ID to lowercase to use it with the EC2 API. For example, in Storage Gateway the ID for a volume might be <code>vol-AA22BB012345DAF670</code>. When you use this ID with the EC2 API, you must change it to <code>vol-aa22bb012345daf670</code>. Otherwise, the EC2 API might not behave as expected. </note> <important> IDs for Storage Gateway volumes and Amazon EBS snapshots created from gateway volumes are changing to a longer format. Starting in December 2016, all new volumes and snapshots will be created with a 17-character string. Starting in April 2016, you will be able to use these longer IDs so you can test your systems with the new format. For more information, see <a href="https://aws.amazon.com/ec2/faqs/#longer-ids">Longer EC2 and EBS Resource IDs</a>.   For example, a volume Amazon Resource Name (ARN) with the longer volume ID format looks like the following:  <code>arn:aws:storagegateway:us-west-2:111122223333:gateway/sgw-12A3456B/volume/vol-1122AABBCCDDEEFFG</code>. A snapshot ID with the longer ID format looks like the following: <code>snap-78e226633445566ee</code>. For more information, see <a href="https://forums.aws.amazon.com/ann.jspa?annID=3557">Announcement: Heads-up – Longer AWS Storage Gateway volume and snapshot IDs coming in 2016</a>. </important>"""


    class DescribeTapeArchivesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_tape_archives` for valid parameters"""
            pass
    class DescribeTapeRecoveryPointsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_tape_recovery_points` for valid parameters"""
            pass
    class DescribeTapesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_tapes` for valid parameters"""
            pass
    class DescribeVTLDevicesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_vtl_devices` for valid parameters"""
            pass
    class ListGatewaysPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_gateways` for valid parameters"""
            pass
    class ListVolumesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_volumes` for valid parameters"""
            pass
    class ListTapesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_tapes` for valid parameters"""
            pass
    class ListFileSharesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_file_shares` for valid parameters"""
            pass
    class ListTagsForResourcePaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_tags_for_resource` for valid parameters"""
            pass


    class ActivateGatewayInput(Mapping):
        pass
    
    class ActivateGatewayOutput(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class AddCacheInput(Mapping):
        pass
    
    class AddCacheOutput(Mapping):
        pass
    
    class DiskId(object):
        pass
    
    class AddTagsToResourceInput(Mapping):
        pass
    
    class AddTagsToResourceOutput(Mapping):
        pass
    
    class AddUploadBufferInput(Mapping):
        pass
    
    class AddUploadBufferOutput(Mapping):
        pass
    
    class AddWorkingStorageInput(Mapping):
        pass
    
    class AddWorkingStorageOutput(Mapping):
        pass
    
    class AssignTapePoolInput(Mapping):
        pass
    
    class AssignTapePoolOutput(Mapping):
        pass
    
    class AttachVolumeInput(Mapping):
        pass
    
    class AttachVolumeOutput(Mapping):
        pass
    
    class CancelArchivalInput(Mapping):
        pass
    
    class CancelArchivalOutput(Mapping):
        pass
    
    class CancelRetrievalInput(Mapping):
        pass
    
    class CancelRetrievalOutput(Mapping):
        pass
    
    class CreateCachediSCSIVolumeInput(Mapping):
        pass
    
    class CreateCachediSCSIVolumeOutput(Mapping):
        pass
    
    class long(object):
        pass
    
    class Boolean(object):
        pass
    
    class CreateNFSFileShareInput(Mapping):
        pass
    
    class CreateNFSFileShareOutput(Mapping):
        pass
    
    class NFSFileShareDefaults(Mapping):
        pass
    
    class IPV4AddressCIDR(object):
        pass
    
    class CreateSMBFileShareInput(Mapping):
        pass
    
    class CreateSMBFileShareOutput(Mapping):
        pass
    
    class FileShareUser(object):
        pass
    
    class CreateSnapshotInput(Mapping):
        pass
    
    class CreateSnapshotOutput(Mapping):
        pass
    
    class CreateSnapshotFromVolumeRecoveryPointInput(Mapping):
        pass
    
    class CreateSnapshotFromVolumeRecoveryPointOutput(Mapping):
        pass
    
    class CreateStorediSCSIVolumeInput(Mapping):
        pass
    
    class CreateStorediSCSIVolumeOutput(Mapping):
        pass
    
    class boolean(object):
        pass
    
    class CreateTapeWithBarcodeInput(Mapping):
        pass
    
    class CreateTapeWithBarcodeOutput(Mapping):
        pass
    
    class TapeSize(object):
        pass
    
    class CreateTapesInput(Mapping):
        pass
    
    class CreateTapesOutput(Mapping):
        pass
    
    class DeleteBandwidthRateLimitInput(Mapping):
        pass
    
    class DeleteBandwidthRateLimitOutput(Mapping):
        pass
    
    class DeleteChapCredentialsInput(Mapping):
        pass
    
    class DeleteChapCredentialsOutput(Mapping):
        pass
    
    class DeleteFileShareInput(Mapping):
        pass
    
    class DeleteFileShareOutput(Mapping):
        pass
    
    class DeleteGatewayInput(Mapping):
        pass
    
    class DeleteGatewayOutput(Mapping):
        pass
    
    class DeleteSnapshotScheduleInput(Mapping):
        pass
    
    class DeleteSnapshotScheduleOutput(Mapping):
        pass
    
    class DeleteTapeInput(Mapping):
        pass
    
    class DeleteTapeOutput(Mapping):
        pass
    
    class DeleteTapeArchiveInput(Mapping):
        pass
    
    class DeleteTapeArchiveOutput(Mapping):
        pass
    
    class DeleteVolumeInput(Mapping):
        pass
    
    class DeleteVolumeOutput(Mapping):
        pass
    
    class DescribeBandwidthRateLimitInput(Mapping):
        pass
    
    class DescribeBandwidthRateLimitOutput(Mapping):
        pass
    
    class DescribeCacheInput(Mapping):
        pass
    
    class DescribeCacheOutput(Mapping):
        pass
    
    class DescribeCachediSCSIVolumesInput(Mapping):
        pass
    
    class DescribeCachediSCSIVolumesOutput(Mapping):
        pass
    
    class VolumeARN(object):
        pass
    
    class DescribeChapCredentialsInput(Mapping):
        pass
    
    class DescribeChapCredentialsOutput(Mapping):
        pass
    
    class DescribeGatewayInformationInput(Mapping):
        pass
    
    class DescribeGatewayInformationOutput(Mapping):
        pass
    
    class DescribeMaintenanceStartTimeInput(Mapping):
        pass
    
    class DescribeMaintenanceStartTimeOutput(Mapping):
        pass
    
    class DescribeNFSFileSharesInput(Mapping):
        pass
    
    class DescribeNFSFileSharesOutput(Mapping):
        pass
    
    class FileShareARN(object):
        pass
    
    class DescribeSMBFileSharesInput(Mapping):
        pass
    
    class DescribeSMBFileSharesOutput(Mapping):
        pass
    
    class DescribeSMBSettingsInput(Mapping):
        pass
    
    class DescribeSMBSettingsOutput(Mapping):
        pass
    
    class DescribeSnapshotScheduleInput(Mapping):
        pass
    
    class DescribeSnapshotScheduleOutput(Mapping):
        pass
    
    class DescribeStorediSCSIVolumesInput(Mapping):
        pass
    
    class DescribeStorediSCSIVolumesOutput(Mapping):
        pass
    
    class DescribeTapeArchivesInput(Mapping):
        pass
    
    class DescribeTapeArchivesOutput(Mapping):
        pass
    
    class TapeARN(object):
        pass
    
    class DescribeTapeRecoveryPointsInput(Mapping):
        pass
    
    class DescribeTapeRecoveryPointsOutput(Mapping):
        pass
    
    class DescribeTapesInput(Mapping):
        pass
    
    class DescribeTapesOutput(Mapping):
        pass
    
    class DescribeUploadBufferInput(Mapping):
        pass
    
    class DescribeUploadBufferOutput(Mapping):
        pass
    
    class DescribeVTLDevicesInput(Mapping):
        pass
    
    class DescribeVTLDevicesOutput(Mapping):
        pass
    
    class VTLDeviceARN(object):
        pass
    
    class DescribeWorkingStorageInput(Mapping):
        pass
    
    class DescribeWorkingStorageOutput(Mapping):
        pass
    
    class DetachVolumeInput(Mapping):
        pass
    
    class DetachVolumeOutput(Mapping):
        pass
    
    class DisableGatewayInput(Mapping):
        pass
    
    class DisableGatewayOutput(Mapping):
        pass
    
    class JoinDomainInput(Mapping):
        pass
    
    class JoinDomainOutput(Mapping):
        pass
    
    class Host(object):
        pass
    
    class ListFileSharesInput(Mapping):
        pass
    
    class ListFileSharesOutput(Mapping):
        pass
    
    class ListGatewaysInput(Mapping):
        pass
    
    class ListGatewaysOutput(Mapping):
        pass
    
    class ListLocalDisksInput(Mapping):
        pass
    
    class ListLocalDisksOutput(Mapping):
        pass
    
    class ListTagsForResourceInput(Mapping):
        pass
    
    class ListTagsForResourceOutput(Mapping):
        pass
    
    class ListTapesInput(Mapping):
        pass
    
    class ListTapesOutput(Mapping):
        pass
    
    class ListVolumeInitiatorsInput(Mapping):
        pass
    
    class ListVolumeInitiatorsOutput(Mapping):
        pass
    
    class ListVolumeRecoveryPointsInput(Mapping):
        pass
    
    class ListVolumeRecoveryPointsOutput(Mapping):
        pass
    
    class ListVolumesInput(Mapping):
        pass
    
    class ListVolumesOutput(Mapping):
        pass
    
    class NotifyWhenUploadedInput(Mapping):
        pass
    
    class NotifyWhenUploadedOutput(Mapping):
        pass
    
    class RefreshCacheInput(Mapping):
        pass
    
    class RefreshCacheOutput(Mapping):
        pass
    
    class Folder(object):
        pass
    
    class RemoveTagsFromResourceInput(Mapping):
        pass
    
    class RemoveTagsFromResourceOutput(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class ResetCacheInput(Mapping):
        pass
    
    class ResetCacheOutput(Mapping):
        pass
    
    class RetrieveTapeArchiveInput(Mapping):
        pass
    
    class RetrieveTapeArchiveOutput(Mapping):
        pass
    
    class RetrieveTapeRecoveryPointInput(Mapping):
        pass
    
    class RetrieveTapeRecoveryPointOutput(Mapping):
        pass
    
    class SetLocalConsolePasswordInput(Mapping):
        pass
    
    class SetLocalConsolePasswordOutput(Mapping):
        pass
    
    class SetSMBGuestPasswordInput(Mapping):
        pass
    
    class SetSMBGuestPasswordOutput(Mapping):
        pass
    
    class ShutdownGatewayInput(Mapping):
        pass
    
    class ShutdownGatewayOutput(Mapping):
        pass
    
    class StartGatewayInput(Mapping):
        pass
    
    class StartGatewayOutput(Mapping):
        pass
    
    class UpdateBandwidthRateLimitInput(Mapping):
        pass
    
    class UpdateBandwidthRateLimitOutput(Mapping):
        pass
    
    class BandwidthUploadRateLimit(object):
        pass
    
    class BandwidthDownloadRateLimit(object):
        pass
    
    class UpdateChapCredentialsInput(Mapping):
        pass
    
    class UpdateChapCredentialsOutput(Mapping):
        pass
    
    class UpdateGatewayInformationInput(Mapping):
        pass
    
    class UpdateGatewayInformationOutput(Mapping):
        pass
    
    class UpdateGatewaySoftwareNowInput(Mapping):
        pass
    
    class UpdateGatewaySoftwareNowOutput(Mapping):
        pass
    
    class UpdateMaintenanceStartTimeInput(Mapping):
        pass
    
    class UpdateMaintenanceStartTimeOutput(Mapping):
        pass
    
    class UpdateNFSFileShareInput(Mapping):
        pass
    
    class UpdateNFSFileShareOutput(Mapping):
        pass
    
    class UpdateSMBFileShareInput(Mapping):
        pass
    
    class UpdateSMBFileShareOutput(Mapping):
        pass
    
    class UpdateSMBSecurityStrategyInput(Mapping):
        pass
    
    class UpdateSMBSecurityStrategyOutput(Mapping):
        pass
    
    class UpdateSnapshotScheduleInput(Mapping):
        pass
    
    class UpdateSnapshotScheduleOutput(Mapping):
        pass
    
    class UpdateVTLDeviceTypeInput(Mapping):
        pass
    
    class UpdateVTLDeviceTypeOutput(Mapping):
        pass
    
    def activate_gateway(self, GatewayRegion: str, GatewayTimezone: str, GatewayName: str, ActivationKey: str, GatewayType: str=None, TapeDriveType: str=None, MediumChangerType: str=None, Tags: List[Tag]=None) -> ActivateGatewayOutput:
        """Activates the gateway you previously deployed on your host. In the activation process, you specify information such as the region you want to use for storing snapshots or tapes, the time zone for scheduled snapshots the gateway snapshot schedule window, an activation key, and a name for your gateway. The activation process also associates your gateway with your account; for more information, see <a>UpdateGatewayInformation</a>. <note> You must turn on the gateway VM before you can activate your gateway. </note>

        :param self:
        :param Tags:  A list of up to 50 tags that can be assigned to the gateway. Each tag is a key-value pair. <note> Valid characters for key and value are letters, spaces, and numbers representable in UTF-8 format, and the following special characters: + - = . _ : / @. The maximum length of a tag's key is 128 characters, and the maximum length for a tag's value is 256. </note><br/>MediumChangerType:  The value that indicates the type of medium changer to use for tape gateway. This field is optional.  Valid Values: "STK-L700", "AWS-Gateway-VTL"<br/>TapeDriveType:  The value that indicates the type of tape drive to use for tape gateway. This field is optional.  Valid Values: "IBM-ULT3580-TD5" <br/>GatewayType:  A value that defines the type of gateway to activate. The type specified is critical to all later functions of the gateway and cannot be changed after activation. The default value is <code>CACHED</code>.   Valid Values: "STORED", "CACHED", "VTL", "FILE_S3"<br/>GatewayRegion:  A value that indicates the region where you want to store your data. The gateway region specified must be the same region as the region in your <code>Host</code> header in the request. For more information about available regions and endpoints for AWS Storage Gateway, see <a href="https://docs.aws.amazon.com/general/latest/gr/rande.html#sg_region">Regions and Endpoints</a> in the <i>Amazon Web Services Glossary</i>.  Valid Values: See <a href="https://docs.aws.amazon.com/general/latest/gr/rande.html#sg_region">AWS Storage Gateway Regions and Endpoints</a> in the AWS General Reference. <br/>GatewayTimezone:  A value that indicates the time zone you want to set for the gateway. The time zone is of the format "GMT-hr:mm" or "GMT+hr:mm". For example, GMT-4:00 indicates the time is 4 hours behind GMT. GMT+2:00 indicates the time is 2 hours ahead of GMT. The time zone is used, for example, for scheduling snapshots and your gateway's maintenance schedule.<br/>GatewayName:  The name you configured for your gateway.<br/>ActivationKey:  Your gateway activation key. You can obtain the activation key by sending an HTTP GET request with redirects enabled to the gateway IP address (port 80). The redirect URL returned in the response provides you the activation key for your gateway in the query string parameter <code>activationKey</code>. It may also include other activation-related parameters, however, these are merely defaults -- the arguments you pass to the <code>ActivateGateway</code> API call determine the actual configuration of your gateway.  For more information, see https://docs.aws.amazon.com/storagegateway/latest/userguide/get-activation-key.html in the Storage Gateway User Guide.<br/>
        :return: GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>        """
        pass

    def add_cache(self, DiskIds: List[DiskId], GatewayARN: str) -> AddCacheOutput:
        """Configures one or more gateway local disks as cache for a gateway. This operation is only supported in the cached volume, tape and file gateway type (see <a href="https://docs.aws.amazon.com/storagegateway/latest/userguide/StorageGatewayConcepts.html">Storage Gateway Concepts</a>). In the request, you specify the gateway Amazon Resource Name (ARN) to which you want to add cache, and one or more disk IDs that you want to configure as cache.

        :param self:
        :param DiskIds:  An array of strings that identify disks that are to be configured as working storage. Each string have a minimum length of 1 and maximum length of 300. You can get the disk IDs from the <a>ListLocalDisks</a> API.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>        """
        pass

    def add_tags_to_resource(self, Tags: List[Tag], ResourceARN: str) -> AddTagsToResourceOutput:
        """Adds one or more tags to the specified resource. You use tags to add metadata to resources, which you can use to categorize these resources. For example, you can categorize resources by purpose, owner, environment, or team. Each tag consists of a key and a value, which you define. You can add tags to the following AWS Storage Gateway resources: <ul> <li> Storage gateways of all types </li> <li> Storage volumes </li> <li> Virtual tapes </li> <li> NFS and SMB file shares </li> </ul> You can create a maximum of 50 tags for each resource. Virtual tapes and storage volumes that are recovered to a new gateway maintain their tags.

        :param self:
        :param Tags:  The key-value pair that represents the tag you want to add to the resource. The value can be an empty string. <note> Valid characters for key and value are letters, spaces, and numbers representable in UTF-8 format, and the following special characters: + - = . _ : / @. The maximum length of a tag's key is 128 characters, and the maximum length for a tag's value is 256. </note><br/>ResourceARN:  The Amazon Resource Name (ARN) of the resource you want to add tags to.<br/>
        :return: ResourceARN:  The Amazon Resource Name (ARN) of the resource you want to add tags to.<br/>        """
        pass

    def add_upload_buffer(self, DiskIds: List[DiskId], GatewayARN: str) -> AddUploadBufferOutput:
        """Configures one or more gateway local disks as upload buffer for a specified gateway. This operation is supported for the stored volume, cached volume and tape gateway types. In the request, you specify the gateway Amazon Resource Name (ARN) to which you want to add upload buffer, and one or more disk IDs that you want to configure as upload buffer.

        :param self:
        :param DiskIds:  An array of strings that identify disks that are to be configured as working storage. Each string have a minimum length of 1 and maximum length of 300. You can get the disk IDs from the <a>ListLocalDisks</a> API.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>        """
        pass

    def add_working_storage(self, DiskIds: List[DiskId], GatewayARN: str) -> AddWorkingStorageOutput:
        """Configures one or more gateway local disks as working storage for a gateway. This operation is only supported in the stored volume gateway type. This operation is deprecated in cached volume API version 20120630. Use <a>AddUploadBuffer</a> instead. <note> Working storage is also referred to as upload buffer. You can also use the <a>AddUploadBuffer</a> operation to add upload buffer to a stored volume gateway. </note> In the request, you specify the gateway Amazon Resource Name (ARN) to which you want to add working storage, and one or more disk IDs that you want to configure as working storage.

        :param self:
        :param DiskIds:  An array of strings that identify disks that are to be configured as working storage. Each string have a minimum length of 1 and maximum length of 300. You can get the disk IDs from the <a>ListLocalDisks</a> API.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>        """
        pass

    def assign_tape_pool(self, PoolId: str, TapeARN: str) -> AssignTapePoolOutput:
        """Assigns a tape to a tape pool for archiving. The tape assigned to a pool is archived in the S3 storage class that is associated with the pool. When you use your backup application to eject the tape, the tape is archived directly into the S3 storage class (Glacier or Deep Archive) that corresponds to the pool. Valid values: "GLACIER", "DEEP_ARCHIVE"

        :param self:
        :param PoolId:  The ID of the pool that you want to add your tape to for archiving. The tape in this pool is archived in the S3 storage class that is associated with the pool. When you use your backup application to eject the tape, the tape is archived directly into the storage class (Glacier or Deep Archive) that corresponds to the pool. Valid values: "GLACIER", "DEEP_ARCHIVE"<br/>TapeARN:  The unique Amazon Resource Name (ARN) of the virtual tape that you want to add to the tape pool.<br/>
        :return: TapeARN:  The unique Amazon Resource Names (ARN) of the virtual tape that was added to the tape pool.<br/>        """
        pass

    def attach_volume(self, NetworkInterfaceId: str, VolumeARN: str, GatewayARN: str, TargetName: str=None, DiskId: str=None) -> AttachVolumeOutput:
        """Connects a volume to an iSCSI connection and then attaches the volume to the specified gateway. Detaching and attaching a volume enables you to recover your data from one gateway to a different gateway without creating a snapshot. It also makes it easier to move your volumes from an on-premises gateway to a gateway hosted on an Amazon EC2 instance.

        :param self:
        :param DiskId:  The unique device ID or other distinguishing data that identifies the local disk used to create the volume. This value is only required when you are attaching a stored volume.<br/>NetworkInterfaceId:  The network interface of the gateway on which to expose the iSCSI target. Only IPv4 addresses are accepted. Use <a>DescribeGatewayInformation</a> to get a list of the network interfaces available on a gateway.  Valid Values: A valid IP address.<br/>VolumeARN:  The Amazon Resource Name (ARN) of the volume to attach to the specified gateway.<br/>TargetName:  The name of the iSCSI target used by an initiator to connect to a volume and used as a suffix for the target ARN. For example, specifying <code>TargetName</code> as <i>myvolume</i> results in the target ARN of <code>arn:aws:storagegateway:us-east-2:111122223333:gateway/sgw-12A3456B/target/iqn.1997-05.com.amazon:myvolume</code>. The target name must be unique across all volumes on a gateway. If you don't specify a value, Storage Gateway uses the value that was previously used for this volume as the new target name.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway that you want to attach the volume to.<br/>
        :return: TargetARN:  The Amazon Resource Name (ARN) of the volume target, which includes the iSCSI name for the initiator that was used to connect to the target.<br/>VolumeARN:  The Amazon Resource Name (ARN) of the volume that was attached to the gateway.<br/>        """
        pass

    def cancel_archival(self, TapeARN: str, GatewayARN: str) -> CancelArchivalOutput:
        """Cancels archiving of a virtual tape to the virtual tape shelf (VTS) after the archiving process is initiated. This operation is only supported in the tape gateway type.

        :param self:
        :param TapeARN:  The Amazon Resource Name (ARN) of the virtual tape you want to cancel archiving for.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: TapeARN:  The Amazon Resource Name (ARN) of the virtual tape for which archiving was canceled.<br/>        """
        pass

    def cancel_retrieval(self, TapeARN: str, GatewayARN: str) -> CancelRetrievalOutput:
        """Cancels retrieval of a virtual tape from the virtual tape shelf (VTS) to a gateway after the retrieval process is initiated. The virtual tape is returned to the VTS. This operation is only supported in the tape gateway type.

        :param self:
        :param TapeARN:  The Amazon Resource Name (ARN) of the virtual tape you want to cancel retrieval for.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: TapeARN:  The Amazon Resource Name (ARN) of the virtual tape for which retrieval was canceled.<br/>        """
        pass

    def create_cached_iscsi_volume(self, ClientToken: str, NetworkInterfaceId: str, TargetName: str, VolumeSizeInBytes: long, GatewayARN: str, SnapshotId: str=None, SourceVolumeARN: str=None, KMSEncrypted: Boolean=None, KMSKey: str=None, Tags: List[Tag]=None) -> CreateCachediSCSIVolumeOutput:
        """Creates a cached volume on a specified cached volume gateway. This operation is only supported in the cached volume gateway type. <note> Cache storage must be allocated to the gateway before you can create a cached volume. Use the <a>AddCache</a> operation to add cache storage to a gateway.  </note> In the request, you must specify the gateway, size of the volume in bytes, the iSCSI target name, an IP address on which to expose the target, and a unique client token. In response, the gateway creates the volume and returns information about it. This information includes the volume Amazon Resource Name (ARN), its size, and the iSCSI target ARN that initiators can use to connect to the volume target. Optionally, you can provide the ARN for an existing volume as the <code>SourceVolumeARN</code> for this cached volume, which creates an exact copy of the existing volume’s latest recovery point. The <code>VolumeSizeInBytes</code> value must be equal to or larger than the size of the copied volume, in bytes.

        :param self:
        :param Tags:  A list of up to 50 tags that can be assigned to a cached volume. Each tag is a key-value pair. <note> Valid characters for key and value are letters, spaces, and numbers representable in UTF-8 format, and the following special characters: + - = . _ : / @. The maximum length of a tag's key is 128 characters, and the maximum length for a tag's value is 256. </note><br/>KMSKey:  The Amazon Resource Name (ARN) of the AWS KMS key used for Amazon S3 server side encryption. This value can only be set when KMSEncrypted is true. Optional.<br/>KMSEncrypted:  True to use Amazon S3 server side encryption with your own AWS KMS key, or false to use a key managed by Amazon S3. Optional.<br/>ClientToken:  A unique identifier that you use to retry a request. If you retry a request, use the same <code>ClientToken</code> you specified in the initial request.<br/>NetworkInterfaceId:  The network interface of the gateway on which to expose the iSCSI target. Only IPv4 addresses are accepted. Use <a>DescribeGatewayInformation</a> to get a list of the network interfaces available on a gateway.  Valid Values: A valid IP address.<br/>SourceVolumeARN:  The ARN for an existing volume. Specifying this ARN makes the new volume into an exact copy of the specified existing volume's latest recovery point. The <code>VolumeSizeInBytes</code> value for this new volume must be equal to or larger than the size of the existing volume, in bytes.<br/>TargetName:  The name of the iSCSI target used by an initiator to connect to a volume and used as a suffix for the target ARN. For example, specifying <code>TargetName</code> as <i>myvolume</i> results in the target ARN of <code>arn:aws:storagegateway:us-east-2:111122223333:gateway/sgw-12A3456B/target/iqn.1997-05.com.amazon:myvolume</code>. The target name must be unique across all volumes on a gateway. If you don't specify a value, Storage Gateway uses the value that was previously used for this volume as the new target name.<br/>SnapshotId:  The snapshot ID (e.g. "snap-1122aabb") of the snapshot to restore as the new cached volume. Specify this field if you want to create the iSCSI storage volume from a snapshot otherwise do not include this field. To list snapshots for your account use <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/ApiReference-query-DescribeSnapshots.html">DescribeSnapshots</a> in the <i>Amazon Elastic Compute Cloud API Reference</i>.<br/>VolumeSizeInBytes:  The size of the volume in bytes.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: TargetARN:  The Amazon Resource Name (ARN) of the volume target, which includes the iSCSI name that initiators can use to connect to the target.<br/>VolumeARN:  The Amazon Resource Name (ARN) of the configured volume.<br/>        """
        pass

    def create_nfs_file_share(self, LocationARN: str, Role: str, GatewayARN: str, ClientToken: str, NFSFileShareDefaults: NFSFileShareDefaults=None, KMSEncrypted: Boolean=None, KMSKey: str=None, DefaultStorageClass: str=None, ObjectACL: str=None, ClientList: List[IPV4AddressCIDR]=None, Squash: str=None, ReadOnly: Boolean=None, GuessMIMETypeEnabled: Boolean=None, RequesterPays: Boolean=None, Tags: List[Tag]=None) -> CreateNFSFileShareOutput:
        """Creates a Network File System (NFS) file share on an existing file gateway. In Storage Gateway, a file share is a file system mount point backed by Amazon S3 cloud storage. Storage Gateway exposes file shares using a NFS interface. This operation is only supported for file gateways. <important> File gateway requires AWS Security Token Service (AWS STS) to be activated to enable you create a file share. Make sure AWS STS is activated in the region you are creating your file gateway in. If AWS STS is not activated in the region, activate it. For information about how to activate AWS STS, see Activating and Deactivating AWS STS in an AWS Region in the AWS Identity and Access Management User Guide.  File gateway does not support creating hard or symbolic links on a file share. </important>

        :param self:
        :param Tags:  A list of up to 50 tags that can be assigned to the NFS file share. Each tag is a key-value pair. <note> Valid characters for key and value are letters, spaces, and numbers representable in UTF-8 format, and the following special characters: + - = . _ : / @. The maximum length of a tag's key is 128 characters, and the maximum length for a tag's value is 256. </note><br/>RequesterPays:  A value that sets who pays the cost of the request and the cost associated with data download from the S3 bucket. If this value is set to true, the requester pays the costs. Otherwise the S3 bucket owner pays. However, the S3 bucket owner always pays the cost of storing data. <note>  <code>RequesterPays</code> is a configuration for the S3 bucket that backs the file share, so make sure that the configuration on the file share is the same as the S3 bucket configuration. </note><br/>GuessMIMETypeEnabled:  A value that enables guessing of the MIME type for uploaded objects based on file extensions. Set this value to true to enable MIME type guessing, and otherwise to false. The default value is true.<br/>ReadOnly:  A value that sets the write status of a file share. This value is true if the write status is read-only, and otherwise false.<br/>Squash:  A value that maps a user to anonymous user. Valid options are the following:  <ul> <li>  <code>RootSquash</code> - Only root is mapped to anonymous user. </li> <li>  <code>NoSquash</code> - No one is mapped to anonymous user </li> <li>  <code>AllSquash</code> - Everyone is mapped to anonymous user. </li> </ul><br/>ClientList:  The list of clients that are allowed to access the file gateway. The list must contain either valid IP addresses or valid CIDR blocks. <br/>ObjectACL:  A value that sets the access control list permission for objects in the S3 bucket that a file gateway puts objects into. The default value is "private".<br/>DefaultStorageClass:  The default storage class for objects put into an Amazon S3 bucket by the file gateway. Possible values are <code>S3_STANDARD</code>, <code>S3_STANDARD_IA</code>, or <code>S3_ONEZONE_IA</code>. If this field is not populated, the default value <code>S3_STANDARD</code> is used. Optional.<br/>LocationARN:  The ARN of the backed storage used for storing file data. <br/>Role:  The ARN of the AWS Identity and Access Management (IAM) role that a file gateway assumes when it accesses the underlying storage. <br/>KMSKey:  The Amazon Resource Name (ARN) AWS KMS key used for Amazon S3 server side encryption. This value can only be set when KMSEncrypted is true. Optional.<br/>KMSEncrypted:  True to use Amazon S3 server side encryption with your own AWS KMS key, or false to use a key managed by Amazon S3. Optional.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the file gateway on which you want to create a file share.<br/>NFSFileShareDefaults:  File share default values. Optional.:<br/>ClientToken:  A unique string value that you supply that is used by file gateway to ensure idempotent file share creation.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The default owner ID for files in the file share (unless the files have another owner ID specified). The default value is nfsnobody. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupId</i> The default group ID for the file share (unless the files have another group ID specified). The default value is nfsnobody. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DirectoryMode</i> The Unix directory mode in the form "nnnn". For example, "0666" represents the default access mode for all directories inside the file share. The default value is 0777.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FileMode</i> The Unix file mode in the form "nnnn". For example, "0666" represents the default file mode inside the file share. The default value is 0666. <br/>
        :return: FileShareARN:  The Amazon Resource Name (ARN) of the newly created file share. <br/>        """
        pass

    def create_smb_file_share(self, LocationARN: str, Role: str, GatewayARN: str, ClientToken: str, KMSEncrypted: Boolean=None, KMSKey: str=None, DefaultStorageClass: str=None, ObjectACL: str=None, ReadOnly: Boolean=None, GuessMIMETypeEnabled: Boolean=None, RequesterPays: Boolean=None, SMBACLEnabled: Boolean=None, AdminUserList: List[FileShareUser]=None, ValidUserList: List[FileShareUser]=None, InvalidUserList: List[FileShareUser]=None, Authentication: str=None, Tags: List[Tag]=None) -> CreateSMBFileShareOutput:
        """Creates a Server Message Block (SMB) file share on an existing file gateway. In Storage Gateway, a file share is a file system mount point backed by Amazon S3 cloud storage. Storage Gateway expose file shares using a SMB interface. This operation is only supported for file gateways. <important> File gateways require AWS Security Token Service (AWS STS) to be activated to enable you to create a file share. Make sure that AWS STS is activated in the AWS Region you are creating your file gateway in. If AWS STS is not activated in this AWS Region, activate it. For information about how to activate AWS STS, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_temp_enable-regions.html">Activating and Deactivating AWS STS in an AWS Region</a> in the <i>AWS Identity and Access Management User Guide.</i>  File gateways don't support creating hard or symbolic links on a file share. </important>

        :param self:
        :param Tags:  A list of up to 50 tags that can be assigned to the NFS file share. Each tag is a key-value pair. <note> Valid characters for key and value are letters, spaces, and numbers representable in UTF-8 format, and the following special characters: + - = . _ : / @. The maximum length of a tag's key is 128 characters, and the maximum length for a tag's value is 256. </note><br/>Authentication:  The authentication method that users use to access the file share. Valid values are <code>ActiveDirectory</code> or <code>GuestAccess</code>. The default is <code>ActiveDirectory</code>.<br/>InvalidUserList:  A list of users or groups in the Active Directory that are not allowed to access the file share. A group must be prefixed with the @ character. For example <code>@group1</code>. Can only be set if Authentication is set to <code>ActiveDirectory</code>.<br/>ValidUserList:  A list of users or groups in the Active Directory that are allowed to access the file share. A group must be prefixed with the @ character. For example <code>@group1</code>. Can only be set if Authentication is set to <code>ActiveDirectory</code>.<br/>AdminUserList:  A list of users or groups in the Active Directory that have administrator rights to the file share. A group must be prefixed with the @ character. For example <code>@group1</code>. Can only be set if Authentication is set to <code>ActiveDirectory</code>.<br/>SMBACLEnabled:  Set this value to "true to enable ACL (access control list) on the SMB file share. Set it to "false" to map file and directory permissions to the POSIX permissions. For more information, see https://docs.aws.amazon.com/storagegateway/latest/userguide/smb-acl.html in the Storage Gateway User Guide.<br/>RequesterPays:  A value that sets who pays the cost of the request and the cost associated with data download from the S3 bucket. If this value is set to true, the requester pays the costs. Otherwise the S3 bucket owner pays. However, the S3 bucket owner always pays the cost of storing data. <note>  <code>RequesterPays</code> is a configuration for the S3 bucket that backs the file share, so make sure that the configuration on the file share is the same as the S3 bucket configuration. </note><br/>GuessMIMETypeEnabled:  A value that enables guessing of the MIME type for uploaded objects based on file extensions. Set this value to true to enable MIME type guessing, and otherwise to false. The default value is true.<br/>ReadOnly:  A value that sets the write status of a file share. This value is true if the write status is read-only, and otherwise false.<br/>ObjectACL:  A value that sets the access control list permission for objects in the S3 bucket that a file gateway puts objects into. The default value is "private".<br/>DefaultStorageClass:  The default storage class for objects put into an Amazon S3 bucket by the file gateway. Possible values are <code>S3_STANDARD</code>, <code>S3_STANDARD_IA</code>, or <code>S3_ONEZONE_IA</code>. If this field is not populated, the default value <code>S3_STANDARD</code> is used. Optional.<br/>LocationARN:  The ARN of the backed storage used for storing file data. <br/>Role:  The ARN of the AWS Identity and Access Management (IAM) role that a file gateway assumes when it accesses the underlying storage. <br/>KMSKey:  The Amazon Resource Name (ARN) of the AWS KMS key used for Amazon S3 server side encryption. This value can only be set when KMSEncrypted is true. Optional.<br/>KMSEncrypted:  True to use Amazon S3 server side encryption with your own AWS KMS key, or false to use a key managed by Amazon S3. Optional.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the file gateway on which you want to create a file share.<br/>ClientToken:  A unique string value that you supply that is used by file gateway to ensure idempotent file share creation.<br/>
        :return: FileShareARN:  The Amazon Resource Name (ARN) of the newly created file share. <br/>        """
        pass

    def create_snapshot(self, SnapshotDescription: str, VolumeARN: str, Tags: List[Tag]=None) -> CreateSnapshotOutput:
        """Initiates a snapshot of a volume. AWS Storage Gateway provides the ability to back up point-in-time snapshots of your data to Amazon Simple Storage (S3) for durable off-site recovery, as well as import the data to an Amazon Elastic Block Store (EBS) volume in Amazon Elastic Compute Cloud (EC2). You can take snapshots of your gateway volume on a scheduled or ad hoc basis. This API enables you to take ad-hoc snapshot. For more information, see <a href="https://docs.aws.amazon.com/storagegateway/latest/userguide/managing-volumes.html#SchedulingSnapshot">Editing a Snapshot Schedule</a>. In the CreateSnapshot request you identify the volume by providing its Amazon Resource Name (ARN). You must also provide description for the snapshot. When AWS Storage Gateway takes the snapshot of specified volume, the snapshot and description appears in the AWS Storage Gateway Console. In response, AWS Storage Gateway returns you a snapshot ID. You can use this snapshot ID to check the snapshot progress or later use it when you want to create a volume from a snapshot. This operation is only supported in stored and cached volume gateway type. <note> To list or delete a snapshot, you must use the Amazon EC2 API. For more information, see DescribeSnapshots or DeleteSnapshot in the <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_Operations.html">EC2 API reference</a>. </note> <important> Volume and snapshot IDs are changing to a longer length ID format. For more information, see the important note on the <a href="https://docs.aws.amazon.com/storagegateway/latest/APIReference/Welcome.html">Welcome</a> page. </important>

        :param self:
        :param Tags:  A list of up to 50 tags that can be assigned to a snapshot. Each tag is a key-value pair. <note> Valid characters for key and value are letters, spaces, and numbers representable in UTF-8 format, and the following special characters: + - = . _ : / @. The maximum length of a tag's key is 128 characters, and the maximum length for a tag's value is 256. </note><br/>SnapshotDescription:  Textual description of the snapshot that appears in the Amazon EC2 console, Elastic Block Store snapshots panel in the <b>Description</b> field, and in the AWS Storage Gateway snapshot <b>Details</b> pane, <b>Description</b> field<br/>VolumeARN:  The Amazon Resource Name (ARN) of the volume. Use the <a>ListVolumes</a> operation to return a list of gateway volumes.<br/>
        :return: SnapshotId:  The snapshot ID that is used to refer to the snapshot in future operations such as describing snapshots (Amazon Elastic Compute Cloud API <code>DescribeSnapshots</code>) or creating a volume from a snapshot (<a>CreateStorediSCSIVolume</a>).<br/>VolumeARN:  The Amazon Resource Name (ARN) of the volume of which the snapshot was taken.<br/>        """
        pass

    def create_snapshot_from_volume_recovery_point(self, SnapshotDescription: str, VolumeARN: str) -> CreateSnapshotFromVolumeRecoveryPointOutput:
        """Initiates a snapshot of a gateway from a volume recovery point. This operation is only supported in the cached volume gateway type. A volume recovery point is a point in time at which all data of the volume is consistent and from which you can create a snapshot. To get a list of volume recovery point for cached volume gateway, use <a>ListVolumeRecoveryPoints</a>. In the <code>CreateSnapshotFromVolumeRecoveryPoint</code> request, you identify the volume by providing its Amazon Resource Name (ARN). You must also provide a description for the snapshot. When the gateway takes a snapshot of the specified volume, the snapshot and its description appear in the AWS Storage Gateway console. In response, the gateway returns you a snapshot ID. You can use this snapshot ID to check the snapshot progress or later use it when you want to create a volume from a snapshot. <note> To list or delete a snapshot, you must use the Amazon EC2 API. For more information, in <i>Amazon Elastic Compute Cloud API Reference</i>. </note>

        :param self:
        :param SnapshotDescription:  Textual description of the snapshot that appears in the Amazon EC2 console, Elastic Block Store snapshots panel in the <b>Description</b> field, and in the AWS Storage Gateway snapshot <b>Details</b> pane, <b>Description</b> field<br/>VolumeARN:  The Amazon Resource Name (ARN) of the iSCSI volume target. Use the <a>DescribeStorediSCSIVolumes</a> operation to return to retrieve the TargetARN for specified VolumeARN.<br/>
        :return: VolumeRecoveryPointTime:  The time the volume was created from the recovery point.<br/>VolumeARN:  The Amazon Resource Name (ARN) of the iSCSI volume target. Use the <a>DescribeStorediSCSIVolumes</a> operation to return to retrieve the TargetARN for specified VolumeARN.<br/>SnapshotId:  The ID of the snapshot.<br/>        """
        pass

    def create_stored_iscsi_volume(self, NetworkInterfaceId: str, TargetName: str, PreserveExistingData: boolean, DiskId: str, GatewayARN: str, SnapshotId: str=None, KMSEncrypted: Boolean=None, KMSKey: str=None, Tags: List[Tag]=None) -> CreateStorediSCSIVolumeOutput:
        """Creates a volume on a specified gateway. This operation is only supported in the stored volume gateway type. The size of the volume to create is inferred from the disk size. You can choose to preserve existing data on the disk, create volume from an existing snapshot, or create an empty volume. If you choose to create an empty gateway volume, then any existing data on the disk is erased. In the request you must specify the gateway and the disk information on which you are creating the volume. In response, the gateway creates the volume and returns volume information such as the volume Amazon Resource Name (ARN), its size, and the iSCSI target ARN that initiators can use to connect to the volume target.

        :param self:
        :param Tags:  A list of up to 50 tags that can be assigned to a stored volume. Each tag is a key-value pair. <note> Valid characters for key and value are letters, spaces, and numbers representable in UTF-8 format, and the following special characters: + - = . _ : / @. The maximum length of a tag's key is 128 characters, and the maximum length for a tag's value is 256. </note><br/>KMSKey:  The Amazon Resource Name (ARN) of the KMS key used for Amazon S3 server side encryption. This value can only be set when KMSEncrypted is true. Optional.<br/>KMSEncrypted:  True to use Amazon S3 server side encryption with your own AWS KMS key, or false to use a key managed by Amazon S3. Optional.<br/>NetworkInterfaceId:  The network interface of the gateway on which to expose the iSCSI target. Only IPv4 addresses are accepted. Use <a>DescribeGatewayInformation</a> to get a list of the network interfaces available on a gateway.  Valid Values: A valid IP address.<br/>TargetName:  The name of the iSCSI target used by an initiator to connect to a volume and used as a suffix for the target ARN. For example, specifying <code>TargetName</code> as <i>myvolume</i> results in the target ARN of <code>arn:aws:storagegateway:us-east-2:111122223333:gateway/sgw-12A3456B/target/iqn.1997-05.com.amazon:myvolume</code>. The target name must be unique across all volumes on a gateway. If you don't specify a value, Storage Gateway uses the value that was previously used for this volume as the new target name.<br/>PreserveExistingData:  Specify this field as true if you want to preserve the data on the local disk. Otherwise, specifying this field as false creates an empty volume.  Valid Values: true, false<br/>SnapshotId:  The snapshot ID (e.g. "snap-1122aabb") of the snapshot to restore as the new stored volume. Specify this field if you want to create the iSCSI storage volume from a snapshot otherwise do not include this field. To list snapshots for your account use <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/ApiReference-query-DescribeSnapshots.html">DescribeSnapshots</a> in the <i>Amazon Elastic Compute Cloud API Reference</i>.<br/>DiskId:  The unique identifier for the gateway local disk that is configured as a stored volume. Use <a href="https://docs.aws.amazon.com/storagegateway/latest/userguide/API_ListLocalDisks.html">ListLocalDisks</a> to list disk IDs for a gateway.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: TargetARN:  The Amazon Resource Name (ARN) of the volume target, which includes the iSCSI name that initiators can use to connect to the target.<br/>VolumeSizeInBytes:  The size of the volume in bytes.<br/>VolumeARN:  The Amazon Resource Name (ARN) of the configured volume.<br/>        """
        pass

    def create_tape_with_barcode(self, TapeBarcode: str, TapeSizeInBytes: TapeSize, GatewayARN: str, KMSEncrypted: Boolean=None, KMSKey: str=None, PoolId: str=None, Tags: List[Tag]=None) -> CreateTapeWithBarcodeOutput:
        """Creates a virtual tape by using your own barcode. You write data to the virtual tape and then archive the tape. A barcode is unique and can not be reused if it has already been used on a tape . This applies to barcodes used on deleted tapes. This operation is only supported in the tape gateway type. <note> Cache storage must be allocated to the gateway before you can create a virtual tape. Use the <a>AddCache</a> operation to add cache storage to a gateway. </note>

        :param self:
        :param Tags:  A list of up to 50 tags that can be assigned to a virtual tape that has a barcode. Each tag is a key-value pair. <note> Valid characters for key and value are letters, spaces, and numbers representable in UTF-8 format, and the following special characters: + - = . _ : / @. The maximum length of a tag's key is 128 characters, and the maximum length for a tag's value is 256. </note><br/>PoolId:  The ID of the pool that you want to add your tape to for archiving. The tape in this pool is archived in the S3 storage class that is associated with the pool. When you use your backup application to eject the tape, the tape is archived directly into the storage class (Glacier or Deep Archive) that corresponds to the pool. Valid values: "GLACIER", "DEEP_ARCHIVE"<br/>KMSKey:  The Amazon Resource Name (ARN) of the AWS KMS Key used for Amazon S3 server side encryption. This value can only be set when KMSEncrypted is true. Optional.<br/>KMSEncrypted:  True to use Amazon S3 server side encryption with your own AWS KMS key, or false to use a key managed by Amazon S3. Optional.<br/>TapeBarcode:  The barcode that you want to assign to the tape. <note> Barcodes cannot be reused. This includes barcodes used for tapes that have been deleted. </note><br/>TapeSizeInBytes:  The size, in bytes, of the virtual tape that you want to create. <note> The size must be aligned by gigabyte (1024*1024*1024 byte). </note><br/>GatewayARN:  The unique Amazon Resource Name (ARN) that represents the gateway to associate the virtual tape with. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: TapeARN:  A unique Amazon Resource Name (ARN) that represents the virtual tape that was created.<br/>        """
        pass

    def create_tapes(self, TapeBarcodePrefix: str, NumTapesToCreate: int, ClientToken: str, TapeSizeInBytes: TapeSize, GatewayARN: str, KMSEncrypted: Boolean=None, KMSKey: str=None, PoolId: str=None, Tags: List[Tag]=None) -> CreateTapesOutput:
        """Creates one or more virtual tapes. You write data to the virtual tapes and then archive the tapes. This operation is only supported in the tape gateway type. <note> Cache storage must be allocated to the gateway before you can create virtual tapes. Use the <a>AddCache</a> operation to add cache storage to a gateway.  </note>

        :param self:
        :param Tags:  A list of up to 50 tags that can be assigned to a virtual tape. Each tag is a key-value pair. <note> Valid characters for key and value are letters, spaces, and numbers representable in UTF-8 format, and the following special characters: + - = . _ : / @. The maximum length of a tag's key is 128 characters, and the maximum length for a tag's value is 256. </note><br/>PoolId:  The ID of the pool that you want to add your tape to for archiving. The tape in this pool is archived in the S3 storage class that is associated with the pool. When you use your backup application to eject the tape, the tape is archived directly into the storage class (Glacier or Deep Archive) that corresponds to the pool. Valid values: "GLACIER", "DEEP_ARCHIVE"<br/>KMSKey:  The Amazon Resource Name (ARN) of the AWS KMS key used for Amazon S3 server side encryption. This value can only be set when KMSEncrypted is true. Optional.<br/>KMSEncrypted:  True to use Amazon S3 server side encryption with your own AWS KMS key, or false to use a key managed by Amazon S3. Optional.<br/>TapeBarcodePrefix:  A prefix that you append to the barcode of the virtual tape you are creating. This prefix makes the barcode unique. <note> The prefix must be 1 to 4 characters in length and must be one of the uppercase letters from A to Z. </note><br/>NumTapesToCreate:  The number of virtual tapes that you want to create.<br/>ClientToken:  A unique identifier that you use to retry a request. If you retry a request, use the same <code>ClientToken</code> you specified in the initial request. <note> Using the same <code>ClientToken</code> prevents creating the tape multiple times. </note><br/>TapeSizeInBytes:  The size, in bytes, of the virtual tapes that you want to create. <note> The size must be aligned by gigabyte (1024*1024*1024 byte). </note><br/>GatewayARN:  The unique Amazon Resource Name (ARN) that represents the gateway to associate the virtual tapes with. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: TapeARNs:  A list of unique Amazon Resource Names (ARNs) that represents the virtual tapes that were created.<br/>        """
        pass

    def delete_bandwidth_rate_limit(self, BandwidthType: str, GatewayARN: str) -> DeleteBandwidthRateLimitOutput:
        """Deletes the bandwidth rate limits of a gateway. You can delete either the upload and download bandwidth rate limit, or you can delete both. If you delete only one of the limits, the other limit remains unchanged. To specify which gateway to work with, use the Amazon Resource Name (ARN) of the gateway in your request.

        :param self:
        :param BandwidthType:  One of the BandwidthType values that indicates the gateway bandwidth rate limit to delete. Valid Values: <code>Upload</code>, <code>Download</code>, <code>All</code>.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>        """
        pass

    def delete_chap_credentials(self, InitiatorName: str, TargetARN: str) -> DeleteChapCredentialsOutput:
        """Deletes Challenge-Handshake Authentication Protocol (CHAP) credentials for a specified iSCSI target and initiator pair.

        :param self:
        :param InitiatorName:  The iSCSI initiator that connects to the target.<br/>TargetARN:  The Amazon Resource Name (ARN) of the iSCSI volume target. Use the <a>DescribeStorediSCSIVolumes</a> operation to return to retrieve the TargetARN for specified VolumeARN.<br/>
        :return: InitiatorName:  The iSCSI initiator that connects to the target.<br/>TargetARN:  The Amazon Resource Name (ARN) of the target.<br/>        """
        pass

    def delete_file_share(self, FileShareARN: str, ForceDelete: boolean=None) -> DeleteFileShareOutput:
        """Deletes a file share from a file gateway. This operation is only supported for file gateways.

        :param self:
        :param ForceDelete:  If this value is set to true, the operation deletes a file share immediately and aborts all data uploads to AWS. Otherwise, the file share is not deleted until all data is uploaded to AWS. This process aborts the data upload process, and the file share enters the FORCE_DELETING status.<br/>FileShareARN:  The Amazon Resource Name (ARN) of the file share to be deleted. <br/>
        :return: FileShareARN:  The Amazon Resource Name (ARN) of the deleted file share. <br/>        """
        pass

    def delete_gateway(self, GatewayARN: str) -> DeleteGatewayOutput:
        """Deletes a gateway. To specify which gateway to delete, use the Amazon Resource Name (ARN) of the gateway in your request. The operation deletes the gateway; however, it does not delete the gateway virtual machine (VM) from your host computer. After you delete a gateway, you cannot reactivate it. Completed snapshots of the gateway volumes are not deleted upon deleting the gateway, however, pending snapshots will not complete. After you delete a gateway, your next step is to remove it from your environment. <important> You no longer pay software charges after the gateway is deleted; however, your existing Amazon EBS snapshots persist and you will continue to be billed for these snapshots. You can choose to remove all remaining Amazon EBS snapshots by canceling your Amazon EC2 subscription.  If you prefer not to cancel your Amazon EC2 subscription, you can delete your snapshots using the Amazon EC2 console. For more information, see the <a href="http://aws.amazon.com/storagegateway"> AWS Storage Gateway Detail Page</a>.  </important>

        :param self:
        :param GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>        """
        pass

    def delete_snapshot_schedule(self, VolumeARN: str) -> DeleteSnapshotScheduleOutput:
        """Deletes a snapshot of a volume. You can take snapshots of your gateway volumes on a scheduled or ad hoc basis. This API action enables you to delete a snapshot schedule for a volume. For more information, see <a href="https://docs.aws.amazon.com/storagegateway/latest/userguide/WorkingWithSnapshots.html">Working with Snapshots</a>. In the <code>DeleteSnapshotSchedule</code> request, you identify the volume by providing its Amazon Resource Name (ARN). This operation is only supported in stored and cached volume gateway types. <note> To list or delete a snapshot, you must use the Amazon EC2 API. in <i>Amazon Elastic Compute Cloud API Reference</i>. </note>

        :param self:
        :param VolumeARN:  The volume which snapshot schedule to delete.<br/>
        :return: VolumeARN:  The volume which snapshot schedule was deleted.<br/>        """
        pass

    def delete_tape(self, TapeARN: str, GatewayARN: str) -> DeleteTapeOutput:
        """Deletes the specified virtual tape. This operation is only supported in the tape gateway type.

        :param self:
        :param TapeARN:  The Amazon Resource Name (ARN) of the virtual tape to delete.<br/>GatewayARN:  The unique Amazon Resource Name (ARN) of the gateway that the virtual tape to delete is associated with. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: TapeARN:  The Amazon Resource Name (ARN) of the deleted virtual tape.<br/>        """
        pass

    def delete_tape_archive(self, TapeARN: str) -> DeleteTapeArchiveOutput:
        """Deletes the specified virtual tape from the virtual tape shelf (VTS). This operation is only supported in the tape gateway type.

        :param self:
        :param TapeARN:  The Amazon Resource Name (ARN) of the virtual tape to delete from the virtual tape shelf (VTS).<br/>
        :return: TapeARN:  The Amazon Resource Name (ARN) of the virtual tape that was deleted from the virtual tape shelf (VTS).<br/>        """
        pass

    def delete_volume(self, VolumeARN: str) -> DeleteVolumeOutput:
        """Deletes the specified storage volume that you previously created using the <a>CreateCachediSCSIVolume</a> or <a>CreateStorediSCSIVolume</a> API. This operation is only supported in the cached volume and stored volume types. For stored volume gateways, the local disk that was configured as the storage volume is not deleted. You can reuse the local disk to create another storage volume.  Before you delete a volume, make sure there are no iSCSI connections to the volume you are deleting. You should also make sure there is no snapshot in progress. You can use the Amazon Elastic Compute Cloud (Amazon EC2) API to query snapshots on the volume you are deleting and check the snapshot status. For more information, go to <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/ApiReference-query-DescribeSnapshots.html">DescribeSnapshots</a> in the <i>Amazon Elastic Compute Cloud API Reference</i>. In the request, you must provide the Amazon Resource Name (ARN) of the storage volume you want to delete.

        :param self:
        :param VolumeARN:  The Amazon Resource Name (ARN) of the volume. Use the <a>ListVolumes</a> operation to return a list of gateway volumes.<br/>
        :return: VolumeARN:  The Amazon Resource Name (ARN) of the storage volume that was deleted. It is the same ARN you provided in the request.<br/>        """
        pass

    def describe_bandwidth_rate_limit(self, GatewayARN: str) -> DescribeBandwidthRateLimitOutput:
        """Returns the bandwidth rate limits of a gateway. By default, these limits are not set, which means no bandwidth rate limiting is in effect. This operation only returns a value for a bandwidth rate limit only if the limit is set. If no limits are set for the gateway, then this operation returns only the gateway ARN in the response body. To specify which gateway to describe, use the Amazon Resource Name (ARN) of the gateway in your request.

        :param self:
        :param GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: AverageDownloadRateLimitInBitsPerSec:  The average download bandwidth rate limit in bits per second. This field does not appear in the response if the download rate limit is not set.<br/>AverageUploadRateLimitInBitsPerSec:  The average upload bandwidth rate limit in bits per second. This field does not appear in the response if the upload rate limit is not set.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>        """
        pass

    def describe_cache(self, GatewayARN: str) -> DescribeCacheOutput:
        """Returns information about the cache of a gateway. This operation is only supported in the cached volume, tape and file gateway types. The response includes disk IDs that are configured as cache, and it includes the amount of cache allocated and used.

        :param self:
        :param GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: CacheMissPercentage:  Percent of application read operations from the file shares that are not served from cache. The sample is taken at the end of the reporting period.<br/>CacheHitPercentage:  Percent of application read operations from the file shares that are served from cache. The sample is taken at the end of the reporting period.<br/>CacheDirtyPercentage:  The file share's contribution to the overall percentage of the gateway's cache that has not been persisted to AWS. The sample is taken at the end of the reporting period.<br/>CacheUsedPercentage:  Percent use of the gateway's cache storage. This metric applies only to the gateway-cached volume setup. The sample is taken at the end of the reporting period.<br/>CacheAllocatedInBytes:  The amount of cache in bytes allocated to the a gateway.<br/>DiskIds:  An array of strings that identify disks that are to be configured as working storage. Each string have a minimum length of 1 and maximum length of 300. You can get the disk IDs from the <a>ListLocalDisks</a> API.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>        """
        pass

    def describe_cached_iscsi_volumes(self, VolumeARNs: List[VolumeARN]) -> DescribeCachediSCSIVolumesOutput:
        """Returns a description of the gateway volumes specified in the request. This operation is only supported in the cached volume gateway types. The list of gateway volumes in the request must be from one gateway. In the response Amazon Storage Gateway returns volume information sorted by volume Amazon Resource Name (ARN).

        :param self:
        :param VolumeARNs:  An array of strings where each string represents the Amazon Resource Name (ARN) of a cached volume. All of the specified cached volumes must from the same gateway. Use <a>ListVolumes</a> to get volume ARNs for a gateway.<br/>
        :return: CachediSCSIVolumes:  An array of objects where each object contains metadata about one cached volume.<br/>        """
        pass

    def describe_chap_credentials(self, TargetARN: str) -> DescribeChapCredentialsOutput:
        """Returns an array of Challenge-Handshake Authentication Protocol (CHAP) credentials information for a specified iSCSI target, one for each target-initiator pair.

        :param self:
        :param TargetARN:  The Amazon Resource Name (ARN) of the iSCSI volume target. Use the <a>DescribeStorediSCSIVolumes</a> operation to return to retrieve the TargetARN for specified VolumeARN.<br/>
        :return: ChapCredentials:  An array of <a>ChapInfo</a> objects that represent CHAP credentials. Each object in the array contains CHAP credential information for one target-initiator pair. If no CHAP credentials are set, an empty array is returned. CHAP credential information is provided in a JSON object with the following fields: <ul> <li>  <b>InitiatorName</b>: The iSCSI initiator that connects to the target. </li> <li>  <b>SecretToAuthenticateInitiator</b>: The secret key that the initiator (for example, the Windows client) must provide to participate in mutual CHAP with the target. </li> <li>  <b>SecretToAuthenticateTarget</b>: The secret key that the target must provide to participate in mutual CHAP with the initiator (e.g. Windows client). </li> <li>  <b>TargetARN</b>: The Amazon Resource Name (ARN) of the storage volume. </li> </ul><br/>        """
        pass

    def describe_gateway_information(self, GatewayARN: str) -> DescribeGatewayInformationOutput:
        """Returns metadata about a gateway such as its name, network interfaces, configured time zone, and the state (whether the gateway is running or not). To specify which gateway to describe, use the Amazon Resource Name (ARN) of the gateway in your request.

        :param self:
        :param GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: VPCEndpoint:  The configuration settings for the virtual private cloud (VPC) endpoint for your gateway. <br/>Tags:  A list of up to 50 tags assigned to the gateway, sorted alphabetically by key name. Each tag is a key-value pair. For a gateway with more than 10 tags assigned, you can view all tags using the <code>ListTagsForResource</code> API operation.<br/>Ec2InstanceRegion:  The AWS Region where the Amazon EC2 instance is located.<br/>Ec2InstanceId:  The ID of the Amazon EC2 instance that was used to launch the gateway.<br/>LastSoftwareUpdate:  The date on which the last software update was applied to the gateway. If the gateway has never been updated, this field does not return a value in the response.<br/>NextUpdateAvailabilityDate:  The date on which an update to the gateway is available. This date is in the time zone of the gateway. If the gateway is not available for an update this field is not returned in the response.<br/>GatewayType:  The type of the gateway.<br/>GatewayNetworkInterfaces:  A <a>NetworkInterface</a> array that contains descriptions of the gateway network interfaces.<br/>GatewayState:  A value that indicates the operating state of the gateway.<br/>GatewayTimezone:  A value that indicates the time zone configured for the gateway.<br/>GatewayName:  The name you configured for your gateway.<br/>GatewayId:  The unique identifier assigned to your gateway during activation. This ID becomes part of the gateway Amazon Resource Name (ARN), which you use as input for other operations.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>        """
        pass

    def describe_maintenance_start_time(self, GatewayARN: str) -> DescribeMaintenanceStartTimeOutput:
        """Returns your gateway's weekly maintenance start time including the day and time of the week. Note that values are in terms of the gateway's time zone.

        :param self:
        :param GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: Timezone:  A value that indicates the time zone that is set for the gateway. The start time and day of week specified should be in the time zone of the gateway.<br/>DayOfMonth:  The day of the month component of the maintenance start time represented as an ordinal number from 1 to 28, where 1 represents the first day of the month and 28 represents the last day of the month. <note> This value is only available for tape and volume gateways. </note><br/>DayOfWeek:  An ordinal number between 0 and 6 that represents the day of the week, where 0 represents Sunday and 6 represents Saturday. The day of week is in the time zone of the gateway.<br/>MinuteOfHour:  The minute component of the maintenance start time represented as <i>mm</i>, where <i>mm</i> is the minute (0 to 59). The minute of the hour is in the time zone of the gateway.<br/>HourOfDay:  The hour component of the maintenance start time represented as <i>hh</i>, where <i>hh</i> is the hour (0 to 23). The hour of the day is in the time zone of the gateway.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>        """
        pass

    def describe_nfs_file_shares(self, FileShareARNList: List[FileShareARN]) -> DescribeNFSFileSharesOutput:
        """Gets a description for one or more Network File System (NFS) file shares from a file gateway. This operation is only supported for file gateways.

        :param self:
        :param FileShareARNList:  An array containing the Amazon Resource Name (ARN) of each file share to be described. <br/>
        :return: NFSFileShareInfoList:  An array containing a description for each requested file share. <br/>        """
        pass

    def describe_smb_file_shares(self, FileShareARNList: List[FileShareARN]) -> DescribeSMBFileSharesOutput:
        """Gets a description for one or more Server Message Block (SMB) file shares from a file gateway. This operation is only supported for file gateways.

        :param self:
        :param FileShareARNList:  An array containing the Amazon Resource Name (ARN) of each file share to be described. <br/>
        :return: SMBFileShareInfoList:  An array containing a description for each requested file share. <br/>        """
        pass

    def describe_smb_settings(self, GatewayARN: str) -> DescribeSMBSettingsOutput:
        """Gets a description of a Server Message Block (SMB) file share settings from a file gateway. This operation is only supported for file gateways.

        :param self:
        :param GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: SMBSecurityStrategy:  The type of security strategy that was specified for file gateway. ClientSpecified: SMBv1 is enabled, SMB signing is offered but not required, SMB encryption is offered but not required. MandatorySigning: SMBv1 is disabled, SMB signing is required, SMB encryption is offered but not required. MandatoryEncryption: SMBv1 is disabled, SMB signing is offered but not required, SMB encryption is required.<br/>SMBGuestPasswordSet:  This value is true if a password for the guest user “smbguest” is set, and otherwise false.<br/>DomainName:  The name of the domain that the gateway is joined to.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>        """
        pass

    def describe_snapshot_schedule(self, VolumeARN: str) -> DescribeSnapshotScheduleOutput:
        """Describes the snapshot schedule for the specified gateway volume. The snapshot schedule information includes intervals at which snapshots are automatically initiated on the volume. This operation is only supported in the cached volume and stored volume types.

        :param self:
        :param VolumeARN:  The Amazon Resource Name (ARN) of the volume. Use the <a>ListVolumes</a> operation to return a list of gateway volumes.<br/>
        :return: Timezone:  A value that indicates the time zone of the gateway.<br/>Description:  The snapshot description.<br/>RecurrenceInHours:  The number of hours between snapshots.<br/>StartAt:  The hour of the day at which the snapshot schedule begins represented as <i>hh</i>, where <i>hh</i> is the hour (0 to 23). The hour of the day is in the time zone of the gateway.<br/>VolumeARN:  The Amazon Resource Name (ARN) of the volume that was specified in the request.<br/>        """
        pass

    def describe_stored_iscsi_volumes(self, VolumeARNs: List[VolumeARN]) -> DescribeStorediSCSIVolumesOutput:
        """Returns the description of the gateway volumes specified in the request. The list of gateway volumes in the request must be from one gateway. In the response Amazon Storage Gateway returns volume information sorted by volume ARNs. This operation is only supported in stored volume gateway type.

        :param self:
        :param VolumeARNs:  An array of strings where each string represents the Amazon Resource Name (ARN) of a stored volume. All of the specified stored volumes must from the same gateway. Use <a>ListVolumes</a> to get volume ARNs for a gateway.<br/>
        :return: StorediSCSIVolumes:  Describes a single unit of output from <a>DescribeStorediSCSIVolumes</a>. The following fields are returned: <ul> <li>  <b>ChapEnabled</b>: Indicates whether mutual CHAP is enabled for the iSCSI target. </li> <li>  <b>LunNumber</b>: The logical disk number. </li> <li>  <b>NetworkInterfaceId</b>: The network interface ID of the stored volume that initiator use to map the stored volume as an iSCSI target. </li> <li>  <b>NetworkInterfacePort</b>: The port used to communicate with iSCSI targets. </li> <li>  <b>PreservedExistingData</b>: Indicates if when the stored volume was created, existing data on the underlying local disk was preserved. </li> <li>  <b>SourceSnapshotId</b>: If the stored volume was created from a snapshot, this field contains the snapshot ID used, e.g. snap-1122aabb. Otherwise, this field is not included. </li> <li>  <b>StorediSCSIVolumes</b>: An array of StorediSCSIVolume objects where each object contains metadata about one stored volume. </li> <li>  <b>TargetARN</b>: The Amazon Resource Name (ARN) of the volume target. </li> <li>  <b>VolumeARN</b>: The Amazon Resource Name (ARN) of the stored volume. </li> <li>  <b>VolumeDiskId</b>: The disk ID of the local disk that was specified in the <a>CreateStorediSCSIVolume</a> operation. </li> <li>  <b>VolumeId</b>: The unique identifier of the storage volume, e.g. vol-1122AABB. </li> <li>  <b>VolumeiSCSIAttributes</b>: An <a>VolumeiSCSIAttributes</a> object that represents a collection of iSCSI attributes for one stored volume. </li> <li>  <b>VolumeProgress</b>: Represents the percentage complete if the volume is restoring or bootstrapping that represents the percent of data transferred. This field does not appear in the response if the stored volume is not restoring or bootstrapping. </li> <li>  <b>VolumeSizeInBytes</b>: The size of the volume in bytes. </li> <li>  <b>VolumeStatus</b>: One of the <code>VolumeStatus</code> values that indicates the state of the volume. </li> <li>  <b>VolumeType</b>: One of the enumeration values describing the type of the volume. Currently, on STORED volumes are supported. </li> </ul><br/>        """
        pass

    def describe_tape_archives(self, TapeARNs: List[TapeARN]=None, Marker: str=None, Limit: int=None) -> DescribeTapeArchivesOutput:
        """Returns a description of specified virtual tapes in the virtual tape shelf (VTS). This operation is only supported in the tape gateway type. If a specific <code>TapeARN</code> is not specified, AWS Storage Gateway returns a description of all virtual tapes found in the VTS associated with your account.

        :param self:
        :param Limit:  Specifies that the number of virtual tapes descried be limited to the specified number.<br/>Marker:  An opaque string that indicates the position at which to begin describing virtual tapes.<br/>TapeARNs:  Specifies one or more unique Amazon Resource Names (ARNs) that represent the virtual tapes you want to describe.<br/>
        :return: Marker:  An opaque string that indicates the position at which the virtual tapes that were fetched for description ended. Use this marker in your next request to fetch the next set of virtual tapes in the virtual tape shelf (VTS). If there are no more virtual tapes to describe, this field does not appear in the response.<br/>TapeArchives:  An array of virtual tape objects in the virtual tape shelf (VTS). The description includes of the Amazon Resource Name (ARN) of the virtual tapes. The information returned includes the Amazon Resource Names (ARNs) of the tapes, size of the tapes, status of the tapes, progress of the description and tape barcode.<br/>        """
        pass

    def describe_tape_recovery_points(self, GatewayARN: str, Marker: str=None, Limit: int=None) -> DescribeTapeRecoveryPointsOutput:
        """Returns a list of virtual tape recovery points that are available for the specified tape gateway. A recovery point is a point-in-time view of a virtual tape at which all the data on the virtual tape is consistent. If your gateway crashes, virtual tapes that have recovery points can be recovered to a new gateway. This operation is only supported in the tape gateway type.

        :param self:
        :param Limit:  Specifies that the number of virtual tape recovery points that are described be limited to the specified number.<br/>Marker:  An opaque string that indicates the position at which to begin describing the virtual tape recovery points.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: Marker:  An opaque string that indicates the position at which the virtual tape recovery points that were listed for description ended. Use this marker in your next request to list the next set of virtual tape recovery points in the list. If there are no more recovery points to describe, this field does not appear in the response.<br/>TapeRecoveryPointInfos:  An array of TapeRecoveryPointInfos that are available for the specified gateway.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>        """
        pass

    def describe_tapes(self, GatewayARN: str, TapeARNs: List[TapeARN]=None, Marker: str=None, Limit: int=None) -> DescribeTapesOutput:
        """Returns a description of the specified Amazon Resource Name (ARN) of virtual tapes. If a <code>TapeARN</code> is not specified, returns a description of all virtual tapes associated with the specified gateway. This operation is only supported in the tape gateway type.

        :param self:
        :param Limit:  Specifies that the number of virtual tapes described be limited to the specified number. <note> Amazon Web Services may impose its own limit, if this field is not set. </note><br/>Marker:  A marker value, obtained in a previous call to <code>DescribeTapes</code>. This marker indicates which page of results to retrieve.  If not specified, the first page of results is retrieved.<br/>TapeARNs:  Specifies one or more unique Amazon Resource Names (ARNs) that represent the virtual tapes you want to describe. If this parameter is not specified, Tape gateway returns a description of all virtual tapes associated with the specified gateway.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: Marker:  An opaque string which can be used as part of a subsequent DescribeTapes call to retrieve the next page of results. If a response does not contain a marker, then there are no more results to be retrieved.<br/>Tapes:  An array of virtual tape descriptions.<br/>        """
        pass

    def describe_upload_buffer(self, GatewayARN: str) -> DescribeUploadBufferOutput:
        """Returns information about the upload buffer of a gateway. This operation is supported for the stored volume, cached volume and tape gateway types. The response includes disk IDs that are configured as upload buffer space, and it includes the amount of upload buffer space allocated and used.

        :param self:
        :param GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: UploadBufferAllocatedInBytes:  The total number of bytes allocated in the gateway's as upload buffer.<br/>UploadBufferUsedInBytes:  The total number of bytes being used in the gateway's upload buffer.<br/>DiskIds:  An array of the gateway's local disk IDs that are configured as working storage. Each local disk ID is specified as a string (minimum length of 1 and maximum length of 300). If no local disks are configured as working storage, then the DiskIds array is empty.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>        """
        pass

    def describe_vtl_devices(self, GatewayARN: str, VTLDeviceARNs: List[VTLDeviceARN]=None, Marker: str=None, Limit: int=None) -> DescribeVTLDevicesOutput:
        """Returns a description of virtual tape library (VTL) devices for the specified tape gateway. In the response, AWS Storage Gateway returns VTL device information. This operation is only supported in the tape gateway type.

        :param self:
        :param Limit:  Specifies that the number of VTL devices described be limited to the specified number.<br/>Marker:  An opaque string that indicates the position at which to begin describing the VTL devices.<br/>VTLDeviceARNs:  An array of strings, where each string represents the Amazon Resource Name (ARN) of a VTL device. <note> All of the specified VTL devices must be from the same gateway. If no VTL devices are specified, the result will contain all devices on the specified gateway. </note><br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: Marker:  An opaque string that indicates the position at which the VTL devices that were fetched for description ended. Use the marker in your next request to fetch the next set of VTL devices in the list. If there are no more VTL devices to describe, this field does not appear in the response.<br/>VTLDevices:  An array of VTL device objects composed of the Amazon Resource Name(ARN) of the VTL devices.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>        """
        pass

    def describe_working_storage(self, GatewayARN: str) -> DescribeWorkingStorageOutput:
        """Returns information about the working storage of a gateway. This operation is only supported in the stored volumes gateway type. This operation is deprecated in cached volumes API version (20120630). Use DescribeUploadBuffer instead. <note> Working storage is also referred to as upload buffer. You can also use the DescribeUploadBuffer operation to add upload buffer to a stored volume gateway. </note> The response includes disk IDs that are configured as working storage, and it includes the amount of working storage allocated and used.

        :param self:
        :param GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: WorkingStorageAllocatedInBytes:  The total working storage in bytes allocated for the gateway. If no working storage is configured for the gateway, this field returns 0.<br/>WorkingStorageUsedInBytes:  The total working storage in bytes in use by the gateway. If no working storage is configured for the gateway, this field returns 0.<br/>DiskIds:  An array of the gateway's local disk IDs that are configured as working storage. Each local disk ID is specified as a string (minimum length of 1 and maximum length of 300). If no local disks are configured as working storage, then the DiskIds array is empty.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>        """
        pass

    def detach_volume(self, VolumeARN: str, ForceDetach: Boolean=None) -> DetachVolumeOutput:
        """Disconnects a volume from an iSCSI connection and then detaches the volume from the specified gateway. Detaching and attaching a volume enables you to recover your data from one gateway to a different gateway without creating a snapshot. It also makes it easier to move your volumes from an on-premises gateway to a gateway hosted on an Amazon EC2 instance.

        :param self:
        :param ForceDetach:  Set to <code>true</code> to forcibly remove the iSCSI connection of the target volume and detach the volume. The default is <code>false</code>. If this value is set to <code>false</code>, you must manually disconnect the iSCSI connection from the target volume.<br/>VolumeARN:  The Amazon Resource Name (ARN) of the volume to detach from the gateway.<br/>
        :return: VolumeARN:  The Amazon Resource Name (ARN) of the volume that was detached.<br/>        """
        pass

    def disable_gateway(self, GatewayARN: str) -> DisableGatewayOutput:
        """Disables a tape gateway when the gateway is no longer functioning. For example, if your gateway VM is damaged, you can disable the gateway so you can recover virtual tapes. Use this operation for a tape gateway that is not reachable or not functioning. This operation is only supported in the tape gateway type. <important> Once a gateway is disabled it cannot be enabled. </important>

        :param self:
        :param GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: GatewayARN:  The unique Amazon Resource Name (ARN) of the disabled gateway.<br/>        """
        pass

    def join_domain(self, Password: str, UserName: str, DomainName: str, GatewayARN: str, OrganizationalUnit: str=None, DomainControllers: List[Host]=None) -> JoinDomainOutput:
        """Adds a file gateway to an Active Directory domain. This operation is only supported for file gateways that support the SMB file protocol.

        :param self:
        :param Password:  Sets the password of the user who has permission to add the gateway to the Active Directory domain.<br/>UserName:  Sets the user name of user who has permission to add the gateway to the Active Directory domain.<br/>DomainControllers:  List of IPv4 addresses, NetBIOS names, or host names of your domain server. If you need to specify the port number include it after the colon (“:”). For example, <code>mydc.mydomain.com:389</code>.<br/>OrganizationalUnit:  The organizational unit (OU) is a container with an Active Directory that can hold users, groups, computers, and other OUs and this parameter specifies the OU that the gateway will join within the AD domain.<br/>DomainName:  The name of the domain that you want the gateway to join.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <code>ListGateways</code> operation to return a list of gateways for your account and region.<br/>
        :return: GatewayARN:  The unique Amazon Resource Name (ARN) of the gateway that joined the domain.<br/>        """
        pass

    def list_file_shares(self, GatewayARN: str=None, Limit: int=None, Marker: str=None) -> ListFileSharesOutput:
        """Gets a list of the file shares for a specific file gateway, or the list of file shares that belong to the calling user account. This operation is only supported for file gateways.

        :param self:
        :param Marker:  Opaque pagination token returned from a previous ListFileShares operation. If present, <code>Marker</code> specifies where to continue the list from after a previous call to ListFileShares. Optional.<br/>Limit:  The maximum number of file shares to return in the response. The value must be an integer with a value greater than zero. Optional.<br/>GatewayARN:  The Amazon resource Name (ARN) of the gateway whose file shares you want to list. If this field is not present, all file shares under your account are listed.<br/>
        :return: FileShareInfoList:  An array of information about the file gateway's file shares. <br/>NextMarker:  If a value is present, there are more file shares to return. In a subsequent request, use <code>NextMarker</code> as the value for <code>Marker</code> to retrieve the next set of file shares. <br/>Marker:  If the request includes <code>Marker</code>, the response returns that value in this field. <br/>        """
        pass

    def list_gateways(self, Marker: str=None, Limit: int=None) -> ListGatewaysOutput:
        """Lists gateways owned by an AWS account in a region specified in the request. The returned list is ordered by gateway Amazon Resource Name (ARN). By default, the operation returns a maximum of 100 gateways. This operation supports pagination that allows you to optionally reduce the number of gateways returned in a response. If you have more gateways than are returned in a response (that is, the response returns only a truncated list of your gateways), the response contains a marker that you can specify in your next request to fetch the next page of gateways.

        :param self:
        :param Limit:  Specifies that the list of gateways returned be limited to the specified number of items.<br/>Marker:  An opaque string that indicates the position at which to begin the returned list of gateways.<br/>
        :return: Marker:  Use the marker in your next request to fetch the next set of gateways in the list. If there are no more gateways to list, this field does not appear in the response.<br/>Gateways:  An array of <a>GatewayInfo</a> objects.<br/>        """
        pass

    def list_local_disks(self, GatewayARN: str) -> ListLocalDisksOutput:
        """Returns a list of the gateway's local disks. To specify which gateway to describe, you use the Amazon Resource Name (ARN) of the gateway in the body of the request. The request returns a list of all disks, specifying which are configured as working storage, cache storage, or stored volume or not configured at all. The response includes a <code>DiskStatus</code> field. This field can have a value of present (the disk is available to use), missing (the disk is no longer connected to the gateway), or mismatch (the disk node is occupied by a disk that has incorrect metadata or the disk content is corrupted).

        :param self:
        :param GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: Disks:  A JSON object containing the following fields: <ul> <li>  <a>ListLocalDisksOutput$Disks</a>  </li> </ul><br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceARN: str, Marker: str=None, Limit: int=None) -> ListTagsForResourceOutput:
        """Lists the tags that have been added to the specified resource. This operation is only supported in the cached volume, stored volume and tape gateway type.

        :param self:
        :param Limit:  Specifies that the list of tags returned be limited to the specified number of items.<br/>Marker:  An opaque string that indicates the position at which to begin returning the list of tags.<br/>ResourceARN:  The Amazon Resource Name (ARN) of the resource for which you want to list tags.<br/>
        :return: Tags:  An array that contains the tags for the specified resource.<br/>Marker:  An opaque string that indicates the position at which to stop returning the list of tags.<br/>ResourceARN:  he Amazon Resource Name (ARN) of the resource for which you want to list tags.<br/>        """
        pass

    def list_tapes(self, TapeARNs: List[TapeARN]=None, Marker: str=None, Limit: int=None) -> ListTapesOutput:
        """Lists virtual tapes in your virtual tape library (VTL) and your virtual tape shelf (VTS). You specify the tapes to list by specifying one or more tape Amazon Resource Names (ARNs). If you don't specify a tape ARN, the operation lists all virtual tapes in both your VTL and VTS. This operation supports pagination. By default, the operation returns a maximum of up to 100 tapes. You can optionally specify the <code>Limit</code> parameter in the body to limit the number of tapes in the response. If the number of tapes returned in the response is truncated, the response includes a <code>Marker</code> element that you can use in your subsequent request to retrieve the next set of tapes. This operation is only supported in the tape gateway type.

        :param self:
        :param Limit:  An optional number limit for the tapes in the list returned by this call.<br/>Marker:  A string that indicates the position at which to begin the returned list of tapes.<br/>TapeARNs:  The Amazon Resource Name (ARN) of each of the tapes you want to list. If you don't specify a tape ARN, the response lists all tapes in both your VTL and VTS.<br/>
        :return: Marker:  A string that indicates the position at which to begin returning the next list of tapes. Use the marker in your next request to continue pagination of tapes. If there are no more tapes to list, this element does not appear in the response body.<br/>TapeInfos:  An array of <a>TapeInfo</a> objects, where each object describes an a single tape. If there not tapes in the tape library or VTS, then the <code>TapeInfos</code> is an empty array.<br/>        """
        pass

    def list_volume_initiators(self, VolumeARN: str) -> ListVolumeInitiatorsOutput:
        """Lists iSCSI initiators that are connected to a volume. You can use this operation to determine whether a volume is being used or not. This operation is only supported in the cached volume and stored volume gateway types.

        :param self:
        :param VolumeARN:  The Amazon Resource Name (ARN) of the volume. Use the <a>ListVolumes</a> operation to return a list of gateway volumes for the gateway.<br/>
        :return: Initiators:  The host names and port numbers of all iSCSI initiators that are connected to the gateway.<br/>        """
        pass

    def list_volume_recovery_points(self, GatewayARN: str) -> ListVolumeRecoveryPointsOutput:
        """Lists the recovery points for a specified gateway. This operation is only supported in the cached volume gateway type. Each cache volume has one recovery point. A volume recovery point is a point in time at which all data of the volume is consistent and from which you can create a snapshot or clone a new cached volume from a source volume. To create a snapshot from a volume recovery point use the <a>CreateSnapshotFromVolumeRecoveryPoint</a> operation.

        :param self:
        :param GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: VolumeRecoveryPointInfos:  An array of <a>VolumeRecoveryPointInfo</a> objects.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>        """
        pass

    def list_volumes(self, GatewayARN: str=None, Marker: str=None, Limit: int=None) -> ListVolumesOutput:
        """Lists the iSCSI stored volumes of a gateway. Results are sorted by volume ARN. The response includes only the volume ARNs. If you want additional volume information, use the <a>DescribeStorediSCSIVolumes</a> or the <a>DescribeCachediSCSIVolumes</a> API. The operation supports pagination. By default, the operation returns a maximum of up to 100 volumes. You can optionally specify the <code>Limit</code> field in the body to limit the number of volumes in the response. If the number of volumes returned in the response is truncated, the response includes a Marker field. You can use this Marker value in your subsequent request to retrieve the next set of volumes. This operation is only supported in the cached volume and stored volume gateway types.

        :param self:
        :param Limit:  Specifies that the list of volumes returned be limited to the specified number of items.<br/>Marker:  A string that indicates the position at which to begin the returned list of volumes. Obtain the marker from the response of a previous List iSCSI Volumes request.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: VolumeInfos:  An array of <a>VolumeInfo</a> objects, where each object describes an iSCSI volume. If no volumes are defined for the gateway, then <code>VolumeInfos</code> is an empty array "[]".<br/>Marker:  Use the marker in your next request to continue pagination of iSCSI volumes. If there are no more volumes to list, this field does not appear in the response body.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>        """
        pass

    def notify_when_uploaded(self, FileShareARN: str) -> NotifyWhenUploadedOutput:
        """Sends you notification through CloudWatch Events when all files written to your NFS file share have been uploaded to Amazon S3. AWS Storage Gateway can send a notification through Amazon CloudWatch Events when all files written to your file share up to that point in time have been uploaded to Amazon S3. These files include files written to the NFS file share up to the time that you make a request for notification. When the upload is done, Storage Gateway sends you notification through an Amazon CloudWatch Event. You can configure CloudWatch Events to send the notification through event targets such as Amazon SNS or AWS Lambda function. This operation is only supported for file gateways. For more information, see Getting File Upload Notification in the Storage Gateway User Guide (https://docs.aws.amazon.com/storagegateway/latest/userguide/monitoring-file-gateway.html#get-upload-notification). 

        :param self:
        :param FileShareARN:  The Amazon Resource Name (ARN) of the file share. <br/>
        :return: NotificationId:  The randomly generated ID of the notification that was sent. This ID is in UUID format.<br/>FileShareARN:  The Amazon Resource Name (ARN) of the file share. <br/>        """
        pass

    def refresh_cache(self, FileShareARN: str, FolderList: List[Folder]=None, Recursive: Boolean=None) -> RefreshCacheOutput:
        """Refreshes the cache for the specified file share. This operation finds objects in the Amazon S3 bucket that were added, removed or replaced since the gateway last listed the bucket's contents and cached the results. This operation is only supported in the file gateway type. You can subscribe to be notified through an Amazon CloudWatch event when your RefreshCache operation completes. For more information, see <a href="https://docs.aws.amazon.com/storagegateway/latest/userguide/monitoring-file-gateway.html#get-notification">Getting Notified About File Operations</a>. When this API is called, it only initiates the refresh operation. When the API call completes and returns a success code, it doesn't necessarily mean that the file refresh has completed. You should use the refresh-complete notification to determine that the operation has completed before you check for new files on the gateway file share. You can subscribe to be notified through an CloudWatch event when your <code>RefreshCache</code> operation completes. 

        :param self:
        :param Recursive:  A value that specifies whether to recursively refresh folders in the cache. The refresh includes folders that were in the cache the last time the gateway listed the folder's contents. If this value set to "true", each folder that is listed in <code>FolderList</code> is recursively updated. Otherwise, subfolders listed in <code>FolderList</code> are not refreshed. Only objects that are in folders listed directly under <code>FolderList</code> are found and used for the update. The default is "true".<br/>FolderList:  A comma-separated list of the paths of folders to refresh in the cache. The default is [<code>"/"</code>]. The default refreshes objects and folders at the root of the Amazon S3 bucket. If <code>Recursive</code> is set to "true", the entire S3 bucket that the file share has access to is refreshed.<br/>FileShareARN:  The Amazon Resource Name (ARN) of the file share you want to refresh.<br/>
        :return: NotificationId:  The randomly generated ID of the notification that was sent. This ID is in UUID format.<br/>FileShareARN:  The Amazon Resource Name (ARN) of the file share. <br/>        """
        pass

    def remove_tags_from_resource(self, TagKeys: List[TagKey], ResourceARN: str) -> RemoveTagsFromResourceOutput:
        """Removes one or more tags from the specified resource. This operation is only supported in the cached volume, stored volume and tape gateway types.

        :param self:
        :param TagKeys:  The keys of the tags you want to remove from the specified resource. A tag is composed of a key/value pair.<br/>ResourceARN:  The Amazon Resource Name (ARN) of the resource you want to remove the tags from.<br/>
        :return: ResourceARN:  The Amazon Resource Name (ARN) of the resource that the tags were removed from.<br/>        """
        pass

    def reset_cache(self, GatewayARN: str) -> ResetCacheOutput:
        """Resets all cache disks that have encountered a error and makes the disks available for reconfiguration as cache storage. If your cache disk encounters a error, the gateway prevents read and write operations on virtual tapes in the gateway. For example, an error can occur when a disk is corrupted or removed from the gateway. When a cache is reset, the gateway loses its cache storage. At this point you can reconfigure the disks as cache disks. This operation is only supported in the cached volume and tape types. <important> If the cache disk you are resetting contains data that has not been uploaded to Amazon S3 yet, that data can be lost. After you reset cache disks, there will be no configured cache disks left in the gateway, so you must configure at least one new cache disk for your gateway to function properly. </important>

        :param self:
        :param GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>        """
        pass

    def retrieve_tape_archive(self, GatewayARN: str, TapeARN: str) -> RetrieveTapeArchiveOutput:
        """Retrieves an archived virtual tape from the virtual tape shelf (VTS) to a tape gateway. Virtual tapes archived in the VTS are not associated with any gateway. However after a tape is retrieved, it is associated with a gateway, even though it is also listed in the VTS, that is, archive. This operation is only supported in the tape gateway type. Once a tape is successfully retrieved to a gateway, it cannot be retrieved again to another gateway. You must archive the tape again before you can retrieve it to another gateway. This operation is only supported in the tape gateway type.

        :param self:
        :param GatewayARN:  The Amazon Resource Name (ARN) of the gateway you want to retrieve the virtual tape to. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region. You retrieve archived virtual tapes to only one gateway and the gateway must be a tape gateway.<br/>TapeARN:  The Amazon Resource Name (ARN) of the virtual tape you want to retrieve from the virtual tape shelf (VTS).<br/>
        :return: TapeARN:  The Amazon Resource Name (ARN) of the retrieved virtual tape.<br/>        """
        pass

    def retrieve_tape_recovery_point(self, GatewayARN: str, TapeARN: str) -> RetrieveTapeRecoveryPointOutput:
        """Retrieves the recovery point for the specified virtual tape. This operation is only supported in the tape gateway type. A recovery point is a point in time view of a virtual tape at which all the data on the tape is consistent. If your gateway crashes, virtual tapes that have recovery points can be recovered to a new gateway. <note> The virtual tape can be retrieved to only one gateway. The retrieved tape is read-only. The virtual tape can be retrieved to only a tape gateway. There is no charge for retrieving recovery points. </note>

        :param self:
        :param GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>TapeARN:  The Amazon Resource Name (ARN) of the virtual tape for which you want to retrieve the recovery point.<br/>
        :return: TapeARN:  The Amazon Resource Name (ARN) of the virtual tape for which the recovery point was retrieved.<br/>        """
        pass

    def set_local_console_password(self, LocalConsolePassword: str, GatewayARN: str) -> SetLocalConsolePasswordOutput:
        """Sets the password for your VM local console. When you log in to the local console for the first time, you log in to the VM with the default credentials. We recommend that you set a new password. You don't need to know the default password to set a new password.

        :param self:
        :param LocalConsolePassword:  The password you want to set for your VM local console.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>        """
        pass

    def set_smb_guest_password(self, Password: str, GatewayARN: str) -> SetSMBGuestPasswordOutput:
        """Sets the password for the guest user <code>smbguest</code>. The <code>smbguest</code> user is the user when the authentication method for the file share is set to <code>GuestAccess</code>.

        :param self:
        :param Password:  The password that you want to set for your SMB Server.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the file gateway the SMB file share is associated with.<br/>
        :return: GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>        """
        pass

    def shutdown_gateway(self, GatewayARN: str) -> ShutdownGatewayOutput:
        """Shuts down a gateway. To specify which gateway to shut down, use the Amazon Resource Name (ARN) of the gateway in the body of your request. The operation shuts down the gateway service component running in the gateway's virtual machine (VM) and not the host VM. <note> If you want to shut down the VM, it is recommended that you first shut down the gateway component in the VM to avoid unpredictable conditions. </note> After the gateway is shutdown, you cannot call any other API except <a>StartGateway</a>, <a>DescribeGatewayInformation</a>, and <a>ListGateways</a>. For more information, see <a>ActivateGateway</a>. Your applications cannot read from or write to the gateway's storage volumes, and there are no snapshots taken. <note> When you make a shutdown request, you will get a <code>200 OK</code> success response immediately. However, it might take some time for the gateway to shut down. You can call the <a>DescribeGatewayInformation</a> API to check the status. For more information, see <a>ActivateGateway</a>. </note> If do not intend to use the gateway again, you must delete the gateway (using <a>DeleteGateway</a>) to no longer pay software charges associated with the gateway.

        :param self:
        :param GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>        """
        pass

    def start_gateway(self, GatewayARN: str) -> StartGatewayOutput:
        """Starts a gateway that you previously shut down (see <a>ShutdownGateway</a>). After the gateway starts, you can then make other API calls, your applications can read from or write to the gateway's storage volumes and you will be able to take snapshot backups. <note> When you make a request, you will get a 200 OK success response immediately. However, it might take some time for the gateway to be ready. You should call <a>DescribeGatewayInformation</a> and check the status before making any additional API calls. For more information, see <a>ActivateGateway</a>. </note> To specify which gateway to start, use the Amazon Resource Name (ARN) of the gateway in your request.

        :param self:
        :param GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>        """
        pass

    def update_bandwidth_rate_limit(self, GatewayARN: str, AverageUploadRateLimitInBitsPerSec: BandwidthUploadRateLimit=None, AverageDownloadRateLimitInBitsPerSec: BandwidthDownloadRateLimit=None) -> UpdateBandwidthRateLimitOutput:
        """Updates the bandwidth rate limits of a gateway. You can update both the upload and download bandwidth rate limit or specify only one of the two. If you don't set a bandwidth rate limit, the existing rate limit remains. By default, a gateway's bandwidth rate limits are not set. If you don't set any limit, the gateway does not have any limitations on its bandwidth usage and could potentially use the maximum available bandwidth. To specify which gateway to update, use the Amazon Resource Name (ARN) of the gateway in your request.

        :param self:
        :param AverageDownloadRateLimitInBitsPerSec:  The average download bandwidth rate limit in bits per second.<br/>AverageUploadRateLimitInBitsPerSec:  The average upload bandwidth rate limit in bits per second.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>        """
        pass

    def update_chap_credentials(self, InitiatorName: str, SecretToAuthenticateInitiator: str, TargetARN: str, SecretToAuthenticateTarget: str=None) -> UpdateChapCredentialsOutput:
        """Updates the Challenge-Handshake Authentication Protocol (CHAP) credentials for a specified iSCSI target. By default, a gateway does not have CHAP enabled; however, for added security, you might use it. <important> When you update CHAP credentials, all existing connections on the target are closed and initiators must reconnect with the new credentials. </important>

        :param self:
        :param SecretToAuthenticateTarget:  The secret key that the target must provide to participate in mutual CHAP with the initiator (e.g. Windows client). Byte constraints: Minimum bytes of 12. Maximum bytes of 16. <note> The secret key must be between 12 and 16 bytes when encoded in UTF-8. </note><br/>InitiatorName:  The iSCSI initiator that connects to the target.<br/>SecretToAuthenticateInitiator:  The secret key that the initiator (for example, the Windows client) must provide to participate in mutual CHAP with the target. <note> The secret key must be between 12 and 16 bytes when encoded in UTF-8. </note><br/>TargetARN:  The Amazon Resource Name (ARN) of the iSCSI volume target. Use the <a>DescribeStorediSCSIVolumes</a> operation to return the TargetARN for specified VolumeARN.<br/>
        :return: InitiatorName:  The iSCSI initiator that connects to the target. This is the same initiator name specified in the request.<br/>TargetARN:  The Amazon Resource Name (ARN) of the target. This is the same target specified in the request.<br/>        """
        pass

    def update_gateway_information(self, GatewayARN: str, GatewayName: str=None, GatewayTimezone: str=None) -> UpdateGatewayInformationOutput:
        """Updates a gateway's metadata, which includes the gateway's name and time zone. To specify which gateway to update, use the Amazon Resource Name (ARN) of the gateway in your request. <note> For Gateways activated after September 2, 2015, the gateway's ARN contains the gateway ID rather than the gateway name. However, changing the name of the gateway has no effect on the gateway's ARN. </note>

        :param self:
        :param GatewayTimezone:  A value that indicates the time zone of the gateway.<br/>GatewayName:  The name you configured for your gateway.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: GatewayName:  The name you configured for your gateway.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>        """
        pass

    def update_gateway_software_now(self, GatewayARN: str) -> UpdateGatewaySoftwareNowOutput:
        """Updates the gateway virtual machine (VM) software. The request immediately triggers the software update. <note> When you make this request, you get a <code>200 OK</code> success response immediately. However, it might take some time for the update to complete. You can call <a>DescribeGatewayInformation</a> to verify the gateway is in the <code>STATE_RUNNING</code> state. </note> <important> A software update forces a system restart of your gateway. You can minimize the chance of any disruption to your applications by increasing your iSCSI Initiators' timeouts. For more information about increasing iSCSI Initiator timeouts for Windows and Linux, see <a href="https://docs.aws.amazon.com/storagegateway/latest/userguide/ConfiguringiSCSIClientInitiatorWindowsClient.html#CustomizeWindowsiSCSISettings">Customizing Your Windows iSCSI Settings</a> and <a href="https://docs.aws.amazon.com/storagegateway/latest/userguide/ConfiguringiSCSIClientInitiatorRedHatClient.html#CustomizeLinuxiSCSISettings">Customizing Your Linux iSCSI Settings</a>, respectively. </important>

        :param self:
        :param GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>        """
        pass

    def update_maintenance_start_time(self, MinuteOfHour: int, HourOfDay: int, GatewayARN: str, DayOfWeek: int=None, DayOfMonth: int=None) -> UpdateMaintenanceStartTimeOutput:
        """Updates a gateway's weekly maintenance start time information, including day and time of the week. The maintenance time is the time in your gateway's time zone.

        :param self:
        :param DayOfMonth:  The day of the month component of the maintenance start time represented as an ordinal number from 1 to 28, where 1 represents the first day of the month and 28 represents the last day of the month. <note> This value is only available for tape and volume gateways. </note><br/>DayOfWeek:  The day of the week component of the maintenance start time week represented as an ordinal number from 0 to 6, where 0 represents Sunday and 6 Saturday.<br/>MinuteOfHour:  The minute component of the maintenance start time represented as <i>mm</i>, where <i>mm</i> is the minute (00 to 59). The minute of the hour is in the time zone of the gateway.<br/>HourOfDay:  The hour component of the maintenance start time represented as <i>hh</i>, where <i>hh</i> is the hour (00 to 23). The hour of the day is in the time zone of the gateway.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>        """
        pass

    def update_nfs_file_share(self, FileShareARN: str, KMSEncrypted: Boolean=None, KMSKey: str=None, NFSFileShareDefaults: NFSFileShareDefaults=None, DefaultStorageClass: str=None, ObjectACL: str=None, ClientList: List[IPV4AddressCIDR]=None, Squash: str=None, ReadOnly: Boolean=None, GuessMIMETypeEnabled: Boolean=None, RequesterPays: Boolean=None) -> UpdateNFSFileShareOutput:
        """Updates a Network File System (NFS) file share. This operation is only supported in the file gateway type. <note> To leave a file share field unchanged, set the corresponding input field to null. </note> Updates the following file share setting: <ul> <li> Default storage class for your S3 bucket </li> <li> Metadata defaults for your S3 bucket </li> <li> Allowed NFS clients for your file share </li> <li> Squash settings </li> <li> Write status of your file share </li> </ul> <note> To leave a file share field unchanged, set the corresponding input field to null. This operation is only supported in file gateways. </note>

        :param self:
        :param RequesterPays:  A value that sets who pays the cost of the request and the cost associated with data download from the S3 bucket. If this value is set to true, the requester pays the costs. Otherwise the S3 bucket owner pays. However, the S3 bucket owner always pays the cost of storing data. <note>  <code>RequesterPays</code> is a configuration for the S3 bucket that backs the file share, so make sure that the configuration on the file share is the same as the S3 bucket configuration. </note><br/>GuessMIMETypeEnabled:  A value that enables guessing of the MIME type for uploaded objects based on file extensions. Set this value to true to enable MIME type guessing, and otherwise to false. The default value is true.<br/>ReadOnly:  A value that sets the write status of a file share. This value is true if the write status is read-only, and otherwise false.<br/>Squash:  The user mapped to anonymous user. Valid options are the following: <ul> <li>  <code>RootSquash</code> - Only root is mapped to anonymous user. </li> <li>  <code>NoSquash</code> - No one is mapped to anonymous user </li> <li>  <code>AllSquash</code> - Everyone is mapped to anonymous user. </li> </ul><br/>ClientList:  The list of clients that are allowed to access the file gateway. The list must contain either valid IP addresses or valid CIDR blocks.<br/>ObjectACL:  A value that sets the access control list permission for objects in the S3 bucket that a file gateway puts objects into. The default value is "private".<br/>DefaultStorageClass:  The default storage class for objects put into an Amazon S3 bucket by the file gateway. Possible values are <code>S3_STANDARD</code>, <code>S3_STANDARD_IA</code>, or <code>S3_ONEZONE_IA</code>. If this field is not populated, the default value <code>S3_STANDARD</code> is used. Optional.<br/>NFSFileShareDefaults:  The default values for the file share. Optional.:<br/>KMSKey:  The Amazon Resource Name (ARN) of the AWS KMS key used for Amazon S3 server side encryption. This value can only be set when KMSEncrypted is true. Optional. <br/>KMSEncrypted:  True to use Amazon S3 server side encryption with your own AWS KMS key, or false to use a key managed by Amazon S3. Optional. <br/>FileShareARN:  The Amazon Resource Name (ARN) of the file share to be updated. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OwnerId</i> The default owner ID for files in the file share (unless the files have another owner ID specified). The default value is nfsnobody. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupId</i> The default group ID for the file share (unless the files have another group ID specified). The default value is nfsnobody. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DirectoryMode</i> The Unix directory mode in the form "nnnn". For example, "0666" represents the default access mode for all directories inside the file share. The default value is 0777.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FileMode</i> The Unix file mode in the form "nnnn". For example, "0666" represents the default file mode inside the file share. The default value is 0666. <br/>
        :return: FileShareARN:  The Amazon Resource Name (ARN) of the updated file share. <br/>        """
        pass

    def update_smb_file_share(self, FileShareARN: str, KMSEncrypted: Boolean=None, KMSKey: str=None, DefaultStorageClass: str=None, ObjectACL: str=None, ReadOnly: Boolean=None, GuessMIMETypeEnabled: Boolean=None, RequesterPays: Boolean=None, SMBACLEnabled: Boolean=None, AdminUserList: List[FileShareUser]=None, ValidUserList: List[FileShareUser]=None, InvalidUserList: List[FileShareUser]=None) -> UpdateSMBFileShareOutput:
        """Updates a Server Message Block (SMB) file share. <note> To leave a file share field unchanged, set the corresponding input field to null. This operation is only supported for file gateways. </note> <important> File gateways require AWS Security Token Service (AWS STS) to be activated to enable you to create a file share. Make sure that AWS STS is activated in the AWS Region you are creating your file gateway in. If AWS STS is not activated in this AWS Region, activate it. For information about how to activate AWS STS, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_temp_enable-regions.html">Activating and Deactivating AWS STS in an AWS Region</a> in the <i>AWS Identity and Access Management User Guide.</i>  File gateways don't support creating hard or symbolic links on a file share. </important>

        :param self:
        :param InvalidUserList:  A list of users or groups in the Active Directory that are not allowed to access the file share. A group must be prefixed with the @ character. For example <code>@group1</code>. Can only be set if Authentication is set to <code>ActiveDirectory</code>.<br/>ValidUserList:  A list of users or groups in the Active Directory that are allowed to access the file share. A group must be prefixed with the @ character. For example <code>@group1</code>. Can only be set if Authentication is set to <code>ActiveDirectory</code>.<br/>AdminUserList:  A list of users or groups in the Active Directory that have administrator rights to the file share. A group must be prefixed with the @ character. For example <code>@group1</code>. Can only be set if Authentication is set to <code>ActiveDirectory</code>.<br/>SMBACLEnabled:  Set this value to "true to enable ACL (access control list) on the SMB file share. Set it to "false" to map file and directory permissions to the POSIX permissions. For more information, see https://docs.aws.amazon.com/storagegateway/latest/userguide/smb-acl.htmlin the Storage Gateway User Guide.<br/>RequesterPays:  A value that sets who pays the cost of the request and the cost associated with data download from the S3 bucket. If this value is set to true, the requester pays the costs. Otherwise the S3 bucket owner pays. However, the S3 bucket owner always pays the cost of storing data. <note>  <code>RequesterPays</code> is a configuration for the S3 bucket that backs the file share, so make sure that the configuration on the file share is the same as the S3 bucket configuration. </note><br/>GuessMIMETypeEnabled:  A value that enables guessing of the MIME type for uploaded objects based on file extensions. Set this value to true to enable MIME type guessing, and otherwise to false. The default value is true.<br/>ReadOnly:  A value that sets the write status of a file share. This value is true if the write status is read-only, and otherwise false.<br/>ObjectACL:  A value that sets the access control list permission for objects in the S3 bucket that a file gateway puts objects into. The default value is "private".<br/>DefaultStorageClass:  The default storage class for objects put into an Amazon S3 bucket by the file gateway. Possible values are <code>S3_STANDARD</code>, <code>S3_STANDARD_IA</code>, or <code>S3_ONEZONE_IA</code>. If this field is not populated, the default value <code>S3_STANDARD</code> is used. Optional.<br/>KMSKey:  The Amazon Resource Name (ARN) of the AWS KMS key used for Amazon S3 server side encryption. This value can only be set when KMSEncrypted is true. Optional.<br/>KMSEncrypted:  True to use Amazon S3 server side encryption with your own AWS KMS key, or false to use a key managed by Amazon S3. Optional.<br/>FileShareARN:  The Amazon Resource Name (ARN) of the SMB file share that you want to update.<br/>
        :return: FileShareARN:  The Amazon Resource Name (ARN) of the updated SMB file share. <br/>        """
        pass

    def update_smb_security_strategy(self, SMBSecurityStrategy: str, GatewayARN: str) -> UpdateSMBSecurityStrategyOutput:
        """Updates the SMB security strategy on a file gateway. This action is only supported in file gateways.

        :param self:
        :param SMBSecurityStrategy:  Specifies the type of security strategy. ClientSpecified: SMBv1 is enabled, SMB signing is offered but not required, SMB encryption is offered but not required. MandatorySigning: SMBv1 is disabled, SMB signing is required, SMB encryption is offered but not required. MandatoryEncryption: SMBv1 is disabled, SMB signing is offered but not required, SMB encryption is required.<br/>GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>
        :return: GatewayARN:  The Amazon Resource Name (ARN) of the gateway. Use the <a>ListGateways</a> operation to return a list of gateways for your account and region.<br/>        """
        pass

    def update_snapshot_schedule(self, RecurrenceInHours: int, StartAt: int, VolumeARN: str, Description: str=None, Tags: List[Tag]=None) -> UpdateSnapshotScheduleOutput:
        """Updates a snapshot schedule configured for a gateway volume. This operation is only supported in the cached volume and stored volume gateway types. The default snapshot schedule for volume is once every 24 hours, starting at the creation time of the volume. You can use this API to change the snapshot schedule configured for the volume. In the request you must identify the gateway volume whose snapshot schedule you want to update, and the schedule information, including when you want the snapshot to begin on a day and the frequency (in hours) of snapshots.

        :param self:
        :param Tags:  A list of up to 50 tags that can be assigned to a snapshot. Each tag is a key-value pair. <note> Valid characters for key and value are letters, spaces, and numbers representable in UTF-8 format, and the following special characters: + - = . _ : / @. The maximum length of a tag's key is 128 characters, and the maximum length for a tag's value is 256. </note><br/>Description:  Optional description of the snapshot that overwrites the existing description.<br/>RecurrenceInHours:  Frequency of snapshots. Specify the number of hours between snapshots.<br/>StartAt:  The hour of the day at which the snapshot schedule begins represented as <i>hh</i>, where <i>hh</i> is the hour (0 to 23). The hour of the day is in the time zone of the gateway.<br/>VolumeARN:  The Amazon Resource Name (ARN) of the volume. Use the <a>ListVolumes</a> operation to return a list of gateway volumes.<br/>
        :return: VolumeARN:  The Amazon Resource Name (ARN) of the volume. Use the <a>ListVolumes</a> operation to return a list of gateway volumes.<br/>        """
        pass

    def update_vtl_device_type(self, DeviceType: str, VTLDeviceARN: str) -> UpdateVTLDeviceTypeOutput:
        """Updates the type of medium changer in a tape gateway. When you activate a tape gateway, you select a medium changer type for the tape gateway. This operation enables you to select a different type of medium changer after a tape gateway is activated. This operation is only supported in the tape gateway type.

        :param self:
        :param DeviceType:  The type of medium changer you want to select.  Valid Values: "STK-L700", "AWS-Gateway-VTL"<br/>VTLDeviceARN:  The Amazon Resource Name (ARN) of the medium changer you want to select.<br/>
        :return: VTLDeviceARN:  The Amazon Resource Name (ARN) of the medium changer you have selected.<br/>        """
        pass



class STS(BaseClient):
    """<fullname>AWS Security Token Service</fullname> The AWS Security Token Service (STS) is a web service that enables you to request temporary, limited-privilege credentials for AWS Identity and Access Management (IAM) users or for users that you authenticate (federated users). This guide provides descriptions of the STS API. For more detailed information about using this service, go to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_temp.html">Temporary Security Credentials</a>.  For information about setting up signatures and authorization through the API, go to <a href="https://docs.aws.amazon.com/general/latest/gr/signing_aws_api_requests.html">Signing AWS API Requests</a> in the <i>AWS General Reference</i>. For general information about the Query API, go to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/IAM_UsingQueryAPI.html">Making Query Requests</a> in <i>Using IAM</i>. For information about using security tokens with other AWS products, go to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_aws-services-that-work-with-iam.html">AWS Services That Work with IAM</a> in the <i>IAM User Guide</i>.  If you're new to AWS and need additional technical information about a specific AWS product, you can find the product's technical documentation at <a href="http://aws.amazon.com/documentation/">http://aws.amazon.com/documentation/</a>.   <b>Endpoints</b>  By default, AWS Security Token Service (STS) is available as a global service, and all AWS STS requests go to a single endpoint at <code>https://sts.amazonaws.com</code>. Global requests map to the US East (N. Virginia) region. AWS recommends using Regional AWS STS endpoints instead of the global endpoint to reduce latency, build in redundancy, and increase session token validity. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_temp_enable-regions.html">Managing AWS STS in an AWS Region</a> in the <i>IAM User Guide</i>. Most AWS Regions are enabled for operations in all AWS services by default. Those Regions are automatically activated for use with AWS STS. Some Regions, such as Asia Pacific (Hong Kong), must be manually enabled. To learn more about enabling and disabling AWS Regions, see <a href="https://docs.aws.amazon.com/general/latest/gr/rande-manage.html">Managing AWS Regions</a> in the <i>AWS General Reference</i>. When you enable these AWS Regions, they are automatically activated for use with AWS STS. You cannot activate the STS endpoint for a Region that is disabled. Tokens that are valid in all AWS Regions are longer than tokens that are valid in Regions that are enabled by default. Changing this setting might affect existing systems where you temporarily store tokens. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_temp_enable-regions.html#sts-regions-manage-tokens">Managing Global Endpoint Session Tokens</a> in the <i>IAM User Guide</i>. After you activate a Region for use with AWS STS, you can direct AWS STS API calls to that Region. AWS STS recommends that you provide both the Region and endpoint when you make calls to a Regional endpoint. You can provide the Region alone for manually enabled Regions, such as Asia Pacific (Hong Kong). In this case, the calls are directed to the STS Regional endpoint. However, if you provide the Region alone for Regions enabled by default, the calls are directed to the global endpoint of <code>https://sts.amazonaws.com</code>. To view the list of AWS STS endpoints and whether they are active by default, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_temp_enable-regions.html#id_credentials_temp_enable-regions_writing_code">Writing Code to Use AWS STS Regions</a> in the <i>IAM User Guide</i>.  <b>Recording API requests</b>  STS supports AWS CloudTrail, which is a service that records AWS calls for your AWS account and delivers log files to an Amazon S3 bucket. By using information collected by CloudTrail, you can determine what requests were successfully made to STS, who made the request, when it was made, and so on. If you activate AWS STS endpoints in Regions other than the default global endpoint, then you must also turn on CloudTrail logging in those Regions. This is necessary to record any AWS STS API calls that are made in those Regions. For more information, see <a href="https://docs.aws.amazon.com/awscloudtrail/latest/userguide/aggregating_logs_regions_turn_on_ct.html">Turning On CloudTrail in Additional Regions</a> in the <i>AWS CloudTrail User Guide</i>. AWS Security Token Service (STS) is a global service with a single endpoint at <code>https://sts.amazonaws.com</code>. Calls to this endpoint are logged as calls to a global service. However, because this endpoint is physically located in the US East (N. Virginia) Region, your logs list <code>us-east-1</code> as the event Region. CloudTrail does not write these logs to the US East (Ohio) Region unless you choose to include global service logs in that Region. CloudTrail writes calls to all Regional endpoints to their respective Regions. For example, calls to sts.us-east-2.amazonaws.com are published to the US East (Ohio) Region and calls to sts.eu-central-1.amazonaws.com are published to the EU (Frankfurt) Region. To learn more about CloudTrail, including how to turn it on and find your log files, see the <a href="https://docs.aws.amazon.com/awscloudtrail/latest/userguide/what_is_cloud_trail_top_level.html">AWS CloudTrail User Guide</a>."""




    class AssumeRoleRequest(Mapping):
        pass
    
    class AssumeRoleResponse(Mapping):
        pass
    
    class PolicyDescriptorType(Mapping):
        pass
    
    class AssumeRoleWithSAMLRequest(Mapping):
        pass
    
    class AssumeRoleWithSAMLResponse(Mapping):
        pass
    
    class AssumeRoleWithWebIdentityRequest(Mapping):
        pass
    
    class AssumeRoleWithWebIdentityResponse(Mapping):
        pass
    
    class DecodeAuthorizationMessageRequest(Mapping):
        pass
    
    class DecodeAuthorizationMessageResponse(Mapping):
        pass
    
    class GetCallerIdentityRequest(Mapping):
        pass
    
    class GetCallerIdentityResponse(Mapping):
        pass
    
    class GetFederationTokenRequest(Mapping):
        pass
    
    class GetFederationTokenResponse(Mapping):
        pass
    
    class GetSessionTokenRequest(Mapping):
        pass
    
    class GetSessionTokenResponse(Mapping):
        pass
    
    def assume_role(self, RoleSessionName: str, RoleArn: str, PolicyArns: List[PolicyDescriptorType]=None, Policy: str=None, DurationSeconds: int=None, ExternalId: str=None, SerialNumber: str=None, TokenCode: str=None) -> AssumeRoleResponse:
        """Returns a set of temporary security credentials that you can use to access AWS resources that you might not normally have access to. These temporary credentials consist of an access key ID, a secret access key, and a security token. Typically, you use <code>AssumeRole</code> within your account or for cross-account access. For a comparison of <code>AssumeRole</code> with other API operations that produce temporary credentials, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_temp_request.html">Requesting Temporary Security Credentials</a> and <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_temp_request.html#stsapi_comparison">Comparing the AWS STS API operations</a> in the <i>IAM User Guide</i>. <important> You cannot use AWS account root user credentials to call <code>AssumeRole</code>. You must use credentials for an IAM user or an IAM role to call <code>AssumeRole</code>. </important> For cross-account access, imagine that you own multiple accounts and need to access resources in each account. You could create long-term credentials in each account to access those resources. However, managing all those credentials and remembering which one can access which account can be time consuming. Instead, you can create one set of long-term credentials in one account. Then use temporary security credentials to access all the other accounts by assuming roles in those accounts. For more information about roles, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles.html">IAM Roles</a> in the <i>IAM User Guide</i>.  By default, the temporary security credentials created by <code>AssumeRole</code> last for one hour. However, you can use the optional <code>DurationSeconds</code> parameter to specify the duration of your session. You can provide a value from 900 seconds (15 minutes) up to the maximum session duration setting for the role. This setting can have a value from 1 hour to 12 hours. To learn how to view the maximum value for your role, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_use.html#id_roles_use_view-role-max-session">View the Maximum Session Duration Setting for a Role</a> in the <i>IAM User Guide</i>. The maximum session duration limit applies when you use the <code>AssumeRole*</code> API operations or the <code>assume-role*</code> CLI commands. However the limit does not apply when you use those operations to create a console URL. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_use.html">Using IAM Roles</a> in the <i>IAM User Guide</i>. The temporary security credentials created by <code>AssumeRole</code> can be used to make API calls to any AWS service with the following exception: You cannot call the AWS STS <code>GetFederationToken</code> or <code>GetSessionToken</code> API operations. (Optional) You can pass inline or managed <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies.html#policies_session">session policies</a> to this operation. You can pass a single JSON policy document to use as an inline session policy. You can also specify up to 10 managed policies to use as managed session policies. The plain text that you use for both inline and managed session policies shouldn't exceed 2048 characters. Passing policies to this operation returns new temporary credentials. The resulting session's permissions are the intersection of the role's identity-based policy and the session policies. You can use the role's temporary credentials in subsequent AWS API calls to access resources in the account that owns the role. You cannot use session policies to grant more permissions than those allowed by the identity-based policy of the role that is being assumed. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies.html#policies_session">Session Policies</a> in the <i>IAM User Guide</i>. To assume a role from a different account, your AWS account must be trusted by the role. The trust relationship is defined in the role's trust policy when the role is created. That trust policy states which accounts are allowed to delegate that access to users in the account.  A user who wants to access a role in a different account must also have permissions that are delegated from the user account administrator. The administrator must attach a policy that allows the user to call <code>AssumeRole</code> for the ARN of the role in the other account. If the user is in the same account as the role, then you can do either of the following: <ul> <li> Attach a policy to the user (identical to the previous user in a different account). </li> <li> Add the user as a principal directly in the role's trust policy. </li> </ul> In this case, the trust policy acts as an IAM resource-based policy. Users in the same account as the role do not need explicit permission to assume the role. For more information about trust policies and resource-based policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies.html">IAM Policies</a> in the <i>IAM User Guide</i>.  <b>Using MFA with AssumeRole</b>  (Optional) You can include multi-factor authentication (MFA) information when you call <code>AssumeRole</code>. This is useful for cross-account scenarios to ensure that the user that assumes the role has been authenticated with an AWS MFA device. In that scenario, the trust policy of the role being assumed includes a condition that tests for MFA authentication. If the caller does not include valid MFA information, the request to assume the role is denied. The condition in a trust policy that tests for MFA authentication might look like the following example.  <code>"Condition": {"Bool": {"aws:MultiFactorAuthPresent": true}}</code>  For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/MFAProtectedAPI.html">Configuring MFA-Protected API Access</a> in the <i>IAM User Guide</i> guide. To use MFA with <code>AssumeRole</code>, you pass values for the <code>SerialNumber</code> and <code>TokenCode</code> parameters. The <code>SerialNumber</code> value identifies the user's hardware or virtual MFA device. The <code>TokenCode</code> is the time-based one-time password (TOTP) that the MFA device produces. 

        :param self:
        :param TokenCode:  The value provided by the MFA device, if the trust policy of the role being assumed requires MFA (that is, if the policy includes a condition that tests for MFA). If the role being assumed requires MFA and if the <code>TokenCode</code> value is missing or expired, the <code>AssumeRole</code> call returns an "access denied" error. The format for this parameter, as described by its regex pattern, is a sequence of six numeric digits.<br/>SerialNumber:  The identification number of the MFA device that is associated with the user who is making the <code>AssumeRole</code> call. Specify this value if the trust policy of the role being assumed includes a condition that requires MFA authentication. The value is either the serial number for a hardware device (such as <code>GAHT12345678</code>) or an Amazon Resource Name (ARN) for a virtual device (such as <code>arn:aws:iam::123456789012:mfa/user</code>). The regex used to validate this parameter is a string of characters consisting of upper- and lower-case alphanumeric characters with no spaces. You can also include underscores or any of the following characters: =,.@-<br/>ExternalId:  A unique identifier that might be required when you assume a role in another account. If the administrator of the account to which the role belongs provided you with an external ID, then provide that value in the <code>ExternalId</code> parameter. This value can be any string, such as a passphrase or account number. A cross-account role is usually set up to trust everyone in an account. Therefore, the administrator of the trusting account might send an external ID to the administrator of the trusted account. That way, only someone with the ID can assume the role, rather than everyone in the account. For more information about the external ID, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_create_for-user_externalid.html">How to Use an External ID When Granting Access to Your AWS Resources to a Third Party</a> in the <i>IAM User Guide</i>. The regex used to validate this parameter is a string of characters consisting of upper- and lower-case alphanumeric characters with no spaces. You can also include underscores or any of the following characters: =,.@:/-<br/>DurationSeconds:  The duration, in seconds, of the role session. The value can range from 900 seconds (15 minutes) up to the maximum session duration setting for the role. This setting can have a value from 1 hour to 12 hours. If you specify a value higher than this setting, the operation fails. For example, if you specify a session duration of 12 hours, but your administrator set the maximum session duration to 6 hours, your operation fails. To learn how to view the maximum value for your role, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_use.html#id_roles_use_view-role-max-session">View the Maximum Session Duration Setting for a Role</a> in the <i>IAM User Guide</i>. By default, the value is set to <code>3600</code> seconds.  <note> The <code>DurationSeconds</code> parameter is separate from the duration of a console session that you might request using the returned credentials. The request to the federation endpoint for a console sign-in token takes a <code>SessionDuration</code> parameter that specifies the maximum length of the console session. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_providers_enable-console-custom-url.html">Creating a URL that Enables Federated Users to Access the AWS Management Console</a> in the <i>IAM User Guide</i>. </note><br/>Policy:  An IAM policy in JSON format that you want to use as an inline session policy. This parameter is optional. Passing policies to this operation returns new temporary credentials. The resulting session's permissions are the intersection of the role's identity-based policy and the session policies. You can use the role's temporary credentials in subsequent AWS API calls to access resources in the account that owns the role. You cannot use session policies to grant more permissions than those allowed by the identity-based policy of the role that is being assumed. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies.html#policies_session">Session Policies</a> in the <i>IAM User Guide</i>. The plain text that you use for both inline and managed session policies shouldn't exceed 2048 characters. The JSON policy characters can be any ASCII character from the space character to the end of the valid character list (\u0020 through \u00FF). It can also include the tab (\u0009), linefeed (\u000A), and carriage return (\u000D) characters. <note> The characters in this parameter count towards the 2048 character session policy guideline. However, an AWS conversion compresses the session policies into a packed binary format that has a separate limit. This is the enforced limit. The <code>PackedPolicySize</code> response element indicates by percentage how close the policy is to the upper size limit. </note><br/>PolicyArns:  The Amazon Resource Names (ARNs) of the IAM managed policies that you want to use as managed session policies. The policies must exist in the same account as the role. This parameter is optional. You can provide up to 10 managed policy ARNs. However, the plain text that you use for both inline and managed session policies shouldn't exceed 2048 characters. For more information about ARNs, see <a href="general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the AWS General Reference. <note> The characters in this parameter count towards the 2048 character session policy guideline. However, an AWS conversion compresses the session policies into a packed binary format that has a separate limit. This is the enforced limit. The <code>PackedPolicySize</code> response element indicates by percentage how close the policy is to the upper size limit. </note> Passing policies to this operation returns new temporary credentials. The resulting session's permissions are the intersection of the role's identity-based policy and the session policies. You can use the role's temporary credentials in subsequent AWS API calls to access resources in the account that owns the role. You cannot use session policies to grant more permissions than those allowed by the identity-based policy of the role that is being assumed. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies.html#policies_session">Session Policies</a> in the <i>IAM User Guide</i>.<br/>RoleSessionName:  An identifier for the assumed role session. Use the role session name to uniquely identify a session when the same role is assumed by different principals or for different reasons. In cross-account scenarios, the role session name is visible to, and can be logged by the account that owns the role. The role session name is also used in the ARN of the assumed role principal. This means that subsequent cross-account API requests that use the temporary security credentials will expose the role session name to the external account in their AWS CloudTrail logs. The regex used to validate this parameter is a string of characters consisting of upper- and lower-case alphanumeric characters with no spaces. You can also include underscores or any of the following characters: =,.@-<br/>RoleArn:  The Amazon Resource Name (ARN) of the role to assume.<br/>
        :return: PackedPolicySize:  A percentage value that indicates the size of the policy in packed form. The service rejects any policy with a packed size greater than 100 percent, which means the policy exceeded the allowed space.<br/>AssumedRoleUser:  The Amazon Resource Name (ARN) and the assumed role ID, which are identifiers that you can use to refer to the resulting temporary security credentials. For example, you can reference these credentials as a principal in a resource-based policy by using the ARN or assumed role ID. The ARN and ID include the <code>RoleSessionName</code> that you specified when you called <code>AssumeRole</code>. :<br/>Credentials:  The temporary security credentials, which include an access key ID, a secret access key, and a security (or session) token. <note> The size of the security token that STS API operations return is not fixed. We strongly recommend that you make no assumptions about the maximum size. </note>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Expiration</i> The date on which the current credentials expire.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SessionToken</i> The token that users must pass to the service API to use the temporary credentials.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecretAccessKey</i> The secret access key that can be used to sign requests.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccessKeyId</i> The access key ID that identifies the temporary security credentials.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN of the temporary security credentials that are returned from the <a>AssumeRole</a> action. For more information about ARNs and how to use them in policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_identifiers.html">IAM Identifiers</a> in <i>Using IAM</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssumedRoleId</i> A unique identifier that contains the role ID and the role session name of the role that is being assumed. The role ID is generated by AWS when the role is created.<br/>        """
        pass

    def assume_role_with_saml(self, SAMLAssertion: str, PrincipalArn: str, RoleArn: str, PolicyArns: List[PolicyDescriptorType]=None, Policy: str=None, DurationSeconds: int=None) -> AssumeRoleWithSAMLResponse:
        """Returns a set of temporary security credentials for users who have been authenticated via a SAML authentication response. This operation provides a mechanism for tying an enterprise identity store or directory to role-based AWS access without user-specific credentials or configuration. For a comparison of <code>AssumeRoleWithSAML</code> with the other API operations that produce temporary credentials, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_temp_request.html">Requesting Temporary Security Credentials</a> and <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_temp_request.html#stsapi_comparison">Comparing the AWS STS API operations</a> in the <i>IAM User Guide</i>. The temporary security credentials returned by this operation consist of an access key ID, a secret access key, and a security token. Applications can use these temporary security credentials to sign calls to AWS services. By default, the temporary security credentials created by <code>AssumeRoleWithSAML</code> last for one hour. However, you can use the optional <code>DurationSeconds</code> parameter to specify the duration of your session. Your role session lasts for the duration that you specify, or until the time specified in the SAML authentication response's <code>SessionNotOnOrAfter</code> value, whichever is shorter. You can provide a <code>DurationSeconds</code> value from 900 seconds (15 minutes) up to the maximum session duration setting for the role. This setting can have a value from 1 hour to 12 hours. To learn how to view the maximum value for your role, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_use.html#id_roles_use_view-role-max-session">View the Maximum Session Duration Setting for a Role</a> in the <i>IAM User Guide</i>. The maximum session duration limit applies when you use the <code>AssumeRole*</code> API operations or the <code>assume-role*</code> CLI commands. However the limit does not apply when you use those operations to create a console URL. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_use.html">Using IAM Roles</a> in the <i>IAM User Guide</i>. The temporary security credentials created by <code>AssumeRoleWithSAML</code> can be used to make API calls to any AWS service with the following exception: you cannot call the STS <code>GetFederationToken</code> or <code>GetSessionToken</code> API operations. (Optional) You can pass inline or managed <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies.html#policies_session">session policies</a> to this operation. You can pass a single JSON policy document to use as an inline session policy. You can also specify up to 10 managed policies to use as managed session policies. The plain text that you use for both inline and managed session policies shouldn't exceed 2048 characters. Passing policies to this operation returns new temporary credentials. The resulting session's permissions are the intersection of the role's identity-based policy and the session policies. You can use the role's temporary credentials in subsequent AWS API calls to access resources in the account that owns the role. You cannot use session policies to grant more permissions than those allowed by the identity-based policy of the role that is being assumed. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies.html#policies_session">Session Policies</a> in the <i>IAM User Guide</i>. Before your application can call <code>AssumeRoleWithSAML</code>, you must configure your SAML identity provider (IdP) to issue the claims required by AWS. Additionally, you must use AWS Identity and Access Management (IAM) to create a SAML provider entity in your AWS account that represents your identity provider. You must also create an IAM role that specifies this SAML provider in its trust policy.  Calling <code>AssumeRoleWithSAML</code> does not require the use of AWS security credentials. The identity of the caller is validated by using keys in the metadata document that is uploaded for the SAML provider entity for your identity provider.  <important> Calling <code>AssumeRoleWithSAML</code> can result in an entry in your AWS CloudTrail logs. The entry includes the value in the <code>NameID</code> element of the SAML assertion. We recommend that you use a <code>NameIDType</code> that is not associated with any personally identifiable information (PII). For example, you could instead use the Persistent Identifier (<code>urn:oasis:names:tc:SAML:2.0:nameid-format:persistent</code>). </important> For more information, see the following resources: <ul> <li>  <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_providers_saml.html">About SAML 2.0-based Federation</a> in the <i>IAM User Guide</i>.  </li> <li>  <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_providers_create_saml.html">Creating SAML Identity Providers</a> in the <i>IAM User Guide</i>.  </li> <li>  <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_providers_create_saml_relying-party.html">Configuring a Relying Party and Claims</a> in the <i>IAM User Guide</i>.  </li> <li>  <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_create_for-idp_saml.html">Creating a Role for SAML 2.0 Federation</a> in the <i>IAM User Guide</i>.  </li> </ul>

        :param self:
        :param DurationSeconds:  The duration, in seconds, of the role session. Your role session lasts for the duration that you specify for the <code>DurationSeconds</code> parameter, or until the time specified in the SAML authentication response's <code>SessionNotOnOrAfter</code> value, whichever is shorter. You can provide a <code>DurationSeconds</code> value from 900 seconds (15 minutes) up to the maximum session duration setting for the role. This setting can have a value from 1 hour to 12 hours. If you specify a value higher than this setting, the operation fails. For example, if you specify a session duration of 12 hours, but your administrator set the maximum session duration to 6 hours, your operation fails. To learn how to view the maximum value for your role, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_use.html#id_roles_use_view-role-max-session">View the Maximum Session Duration Setting for a Role</a> in the <i>IAM User Guide</i>. By default, the value is set to <code>3600</code> seconds.  <note> The <code>DurationSeconds</code> parameter is separate from the duration of a console session that you might request using the returned credentials. The request to the federation endpoint for a console sign-in token takes a <code>SessionDuration</code> parameter that specifies the maximum length of the console session. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_providers_enable-console-custom-url.html">Creating a URL that Enables Federated Users to Access the AWS Management Console</a> in the <i>IAM User Guide</i>. </note><br/>Policy:  An IAM policy in JSON format that you want to use as an inline session policy. This parameter is optional. Passing policies to this operation returns new temporary credentials. The resulting session's permissions are the intersection of the role's identity-based policy and the session policies. You can use the role's temporary credentials in subsequent AWS API calls to access resources in the account that owns the role. You cannot use session policies to grant more permissions than those allowed by the identity-based policy of the role that is being assumed. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies.html#policies_session">Session Policies</a> in the <i>IAM User Guide</i>.  The plain text that you use for both inline and managed session policies shouldn't exceed 2048 characters. The JSON policy characters can be any ASCII character from the space character to the end of the valid character list (\u0020 through \u00FF). It can also include the tab (\u0009), linefeed (\u000A), and carriage return (\u000D) characters. <note> The characters in this parameter count towards the 2048 character session policy guideline. However, an AWS conversion compresses the session policies into a packed binary format that has a separate limit. This is the enforced limit. The <code>PackedPolicySize</code> response element indicates by percentage how close the policy is to the upper size limit. </note><br/>PolicyArns:  The Amazon Resource Names (ARNs) of the IAM managed policies that you want to use as managed session policies. The policies must exist in the same account as the role. This parameter is optional. You can provide up to 10 managed policy ARNs. However, the plain text that you use for both inline and managed session policies shouldn't exceed 2048 characters. For more information about ARNs, see <a href="general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the AWS General Reference. <note> The characters in this parameter count towards the 2048 character session policy guideline. However, an AWS conversion compresses the session policies into a packed binary format that has a separate limit. This is the enforced limit. The <code>PackedPolicySize</code> response element indicates by percentage how close the policy is to the upper size limit. </note> Passing policies to this operation returns new temporary credentials. The resulting session's permissions are the intersection of the role's identity-based policy and the session policies. You can use the role's temporary credentials in subsequent AWS API calls to access resources in the account that owns the role. You cannot use session policies to grant more permissions than those allowed by the identity-based policy of the role that is being assumed. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies.html#policies_session">Session Policies</a> in the <i>IAM User Guide</i>.<br/>SAMLAssertion:  The base-64 encoded SAML authentication response provided by the IdP. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/create-role-saml-IdP-tasks.html">Configuring a Relying Party and Adding Claims</a> in the <i>IAM User Guide</i>. <br/>PrincipalArn:  The Amazon Resource Name (ARN) of the SAML provider in IAM that describes the IdP.<br/>RoleArn:  The Amazon Resource Name (ARN) of the role that the caller is assuming.<br/>
        :return: NameQualifier:  A hash value based on the concatenation of the <code>Issuer</code> response value, the AWS account ID, and the friendly name (the last part of the ARN) of the SAML provider in IAM. The combination of <code>NameQualifier</code> and <code>Subject</code> can be used to uniquely identify a federated user.  The following pseudocode shows how the hash value is calculated:  <code>BASE64 ( SHA1 ( "https://example.com/saml" + "123456789012" + "/MySAMLIdP" ) )</code> <br/>Audience:   The value of the <code>Recipient</code> attribute of the <code>SubjectConfirmationData</code> element of the SAML assertion. <br/>Issuer:  The value of the <code>Issuer</code> element of the SAML assertion.<br/>SubjectType:   The format of the name ID, as defined by the <code>Format</code> attribute in the <code>NameID</code> element of the SAML assertion. Typical examples of the format are <code>transient</code> or <code>persistent</code>.   If the format includes the prefix <code>urn:oasis:names:tc:SAML:2.0:nameid-format</code>, that prefix is removed. For example, <code>urn:oasis:names:tc:SAML:2.0:nameid-format:transient</code> is returned as <code>transient</code>. If the format includes any other prefix, the format is returned with no modifications.<br/>Subject:  The value of the <code>NameID</code> element in the <code>Subject</code> element of the SAML assertion.<br/>PackedPolicySize:  A percentage value that indicates the size of the policy in packed form. The service rejects any policy with a packed size greater than 100 percent, which means the policy exceeded the allowed space.<br/>AssumedRoleUser:  The identifiers for the temporary security credentials that the operation returns.:<br/>Credentials:  The temporary security credentials, which include an access key ID, a secret access key, and a security (or session) token. <note> The size of the security token that STS API operations return is not fixed. We strongly recommend that you make no assumptions about the maximum size. </note>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Expiration</i> The date on which the current credentials expire.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SessionToken</i> The token that users must pass to the service API to use the temporary credentials.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecretAccessKey</i> The secret access key that can be used to sign requests.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccessKeyId</i> The access key ID that identifies the temporary security credentials.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN of the temporary security credentials that are returned from the <a>AssumeRole</a> action. For more information about ARNs and how to use them in policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_identifiers.html">IAM Identifiers</a> in <i>Using IAM</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssumedRoleId</i> A unique identifier that contains the role ID and the role session name of the role that is being assumed. The role ID is generated by AWS when the role is created.<br/>        """
        pass

    def assume_role_with_web_identity(self, WebIdentityToken: str, RoleSessionName: str, RoleArn: str, ProviderId: str=None, PolicyArns: List[PolicyDescriptorType]=None, Policy: str=None, DurationSeconds: int=None) -> AssumeRoleWithWebIdentityResponse:
        """Returns a set of temporary security credentials for users who have been authenticated in a mobile or web application with a web identity provider. Example providers include Amazon Cognito, Login with Amazon, Facebook, Google, or any OpenID Connect-compatible identity provider. <note> For mobile applications, we recommend that you use Amazon Cognito. You can use Amazon Cognito with the <a href="http://aws.amazon.com/sdkforios/">AWS SDK for iOS Developer Guide</a> and the <a href="http://aws.amazon.com/sdkforandroid/">AWS SDK for Android Developer Guide</a> to uniquely identify a user. You can also supply the user with a consistent identity throughout the lifetime of an application. To learn more about Amazon Cognito, see <a href="https://docs.aws.amazon.com/mobile/sdkforandroid/developerguide/cognito-auth.html#d0e840">Amazon Cognito Overview</a> in <i>AWS SDK for Android Developer Guide</i> and <a href="https://docs.aws.amazon.com/mobile/sdkforios/developerguide/cognito-auth.html#d0e664">Amazon Cognito Overview</a> in the <i>AWS SDK for iOS Developer Guide</i>. </note> Calling <code>AssumeRoleWithWebIdentity</code> does not require the use of AWS security credentials. Therefore, you can distribute an application (for example, on mobile devices) that requests temporary security credentials without including long-term AWS credentials in the application. You also don't need to deploy server-based proxy services that use long-term AWS credentials. Instead, the identity of the caller is validated by using a token from the web identity provider. For a comparison of <code>AssumeRoleWithWebIdentity</code> with the other API operations that produce temporary credentials, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_temp_request.html">Requesting Temporary Security Credentials</a> and <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_temp_request.html#stsapi_comparison">Comparing the AWS STS API operations</a> in the <i>IAM User Guide</i>. The temporary security credentials returned by this API consist of an access key ID, a secret access key, and a security token. Applications can use these temporary security credentials to sign calls to AWS service API operations. By default, the temporary security credentials created by <code>AssumeRoleWithWebIdentity</code> last for one hour. However, you can use the optional <code>DurationSeconds</code> parameter to specify the duration of your session. You can provide a value from 900 seconds (15 minutes) up to the maximum session duration setting for the role. This setting can have a value from 1 hour to 12 hours. To learn how to view the maximum value for your role, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_use.html#id_roles_use_view-role-max-session">View the Maximum Session Duration Setting for a Role</a> in the <i>IAM User Guide</i>. The maximum session duration limit applies when you use the <code>AssumeRole*</code> API operations or the <code>assume-role*</code> CLI commands. However the limit does not apply when you use those operations to create a console URL. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_use.html">Using IAM Roles</a> in the <i>IAM User Guide</i>.  The temporary security credentials created by <code>AssumeRoleWithWebIdentity</code> can be used to make API calls to any AWS service with the following exception: you cannot call the STS <code>GetFederationToken</code> or <code>GetSessionToken</code> API operations. (Optional) You can pass inline or managed <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies.html#policies_session">session policies</a> to this operation. You can pass a single JSON policy document to use as an inline session policy. You can also specify up to 10 managed policies to use as managed session policies. The plain text that you use for both inline and managed session policies shouldn't exceed 2048 characters. Passing policies to this operation returns new temporary credentials. The resulting session's permissions are the intersection of the role's identity-based policy and the session policies. You can use the role's temporary credentials in subsequent AWS API calls to access resources in the account that owns the role. You cannot use session policies to grant more permissions than those allowed by the identity-based policy of the role that is being assumed. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies.html#policies_session">Session Policies</a> in the <i>IAM User Guide</i>. Before your application can call <code>AssumeRoleWithWebIdentity</code>, you must have an identity token from a supported identity provider and create a role that the application can assume. The role that your application assumes must trust the identity provider that is associated with the identity token. In other words, the identity provider must be specified in the role's trust policy.  <important> Calling <code>AssumeRoleWithWebIdentity</code> can result in an entry in your AWS CloudTrail logs. The entry includes the <a href="http://openid.net/specs/openid-connect-core-1_0.html#Claims">Subject</a> of the provided Web Identity Token. We recommend that you avoid using any personally identifiable information (PII) in this field. For example, you could instead use a GUID or a pairwise identifier, as <a href="http://openid.net/specs/openid-connect-core-1_0.html#SubjectIDTypes">suggested in the OIDC specification</a>. </important> For more information about how to use web identity federation and the <code>AssumeRoleWithWebIdentity</code> API, see the following resources:  <ul> <li>  <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_providers_oidc_manual.html">Using Web Identity Federation API Operations for Mobile Apps</a> and <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_temp_request.html#api_assumerolewithwebidentity">Federation Through a Web-based Identity Provider</a>.  </li> <li>  <a href="https://web-identity-federation-playground.s3.amazonaws.com/index.html"> Web Identity Federation Playground</a>. Walk through the process of authenticating through Login with Amazon, Facebook, or Google, getting temporary security credentials, and then using those credentials to make a request to AWS.  </li> <li>  <a href="http://aws.amazon.com/sdkforios/">AWS SDK for iOS Developer Guide</a> and <a href="http://aws.amazon.com/sdkforandroid/">AWS SDK for Android Developer Guide</a>. These toolkits contain sample apps that show how to invoke the identity providers, and then how to use the information from these providers to get and use temporary security credentials.  </li> <li>  <a href="http://aws.amazon.com/articles/web-identity-federation-with-mobile-applications">Web Identity Federation with Mobile Applications</a>. This article discusses web identity federation and shows an example of how to use web identity federation to get access to content in Amazon S3.  </li> </ul>

        :param self:
        :param DurationSeconds:  The duration, in seconds, of the role session. The value can range from 900 seconds (15 minutes) up to the maximum session duration setting for the role. This setting can have a value from 1 hour to 12 hours. If you specify a value higher than this setting, the operation fails. For example, if you specify a session duration of 12 hours, but your administrator set the maximum session duration to 6 hours, your operation fails. To learn how to view the maximum value for your role, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_use.html#id_roles_use_view-role-max-session">View the Maximum Session Duration Setting for a Role</a> in the <i>IAM User Guide</i>. By default, the value is set to <code>3600</code> seconds.  <note> The <code>DurationSeconds</code> parameter is separate from the duration of a console session that you might request using the returned credentials. The request to the federation endpoint for a console sign-in token takes a <code>SessionDuration</code> parameter that specifies the maximum length of the console session. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_providers_enable-console-custom-url.html">Creating a URL that Enables Federated Users to Access the AWS Management Console</a> in the <i>IAM User Guide</i>. </note><br/>Policy:  An IAM policy in JSON format that you want to use as an inline session policy. This parameter is optional. Passing policies to this operation returns new temporary credentials. The resulting session's permissions are the intersection of the role's identity-based policy and the session policies. You can use the role's temporary credentials in subsequent AWS API calls to access resources in the account that owns the role. You cannot use session policies to grant more permissions than those allowed by the identity-based policy of the role that is being assumed. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies.html#policies_session">Session Policies</a> in the <i>IAM User Guide</i>. The plain text that you use for both inline and managed session policies shouldn't exceed 2048 characters. The JSON policy characters can be any ASCII character from the space character to the end of the valid character list (\u0020 through \u00FF). It can also include the tab (\u0009), linefeed (\u000A), and carriage return (\u000D) characters. <note> The characters in this parameter count towards the 2048 character session policy guideline. However, an AWS conversion compresses the session policies into a packed binary format that has a separate limit. This is the enforced limit. The <code>PackedPolicySize</code> response element indicates by percentage how close the policy is to the upper size limit. </note><br/>PolicyArns:  The Amazon Resource Names (ARNs) of the IAM managed policies that you want to use as managed session policies. The policies must exist in the same account as the role. This parameter is optional. You can provide up to 10 managed policy ARNs. However, the plain text that you use for both inline and managed session policies shouldn't exceed 2048 characters. For more information about ARNs, see <a href="general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the AWS General Reference. <note> The characters in this parameter count towards the 2048 character session policy guideline. However, an AWS conversion compresses the session policies into a packed binary format that has a separate limit. This is the enforced limit. The <code>PackedPolicySize</code> response element indicates by percentage how close the policy is to the upper size limit. </note> Passing policies to this operation returns new temporary credentials. The resulting session's permissions are the intersection of the role's identity-based policy and the session policies. You can use the role's temporary credentials in subsequent AWS API calls to access resources in the account that owns the role. You cannot use session policies to grant more permissions than those allowed by the identity-based policy of the role that is being assumed. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies.html#policies_session">Session Policies</a> in the <i>IAM User Guide</i>.<br/>ProviderId:  The fully qualified host component of the domain name of the identity provider. Specify this value only for OAuth 2.0 access tokens. Currently <code>www.amazon.com</code> and <code>graph.facebook.com</code> are the only supported identity providers for OAuth 2.0 access tokens. Do not include URL schemes and port numbers. Do not specify this value for OpenID Connect ID tokens.<br/>WebIdentityToken:  The OAuth 2.0 access token or OpenID Connect ID token that is provided by the identity provider. Your application must get this token by authenticating the user who is using your application with a web identity provider before the application makes an <code>AssumeRoleWithWebIdentity</code> call. <br/>RoleSessionName:  An identifier for the assumed role session. Typically, you pass the name or identifier that is associated with the user who is using your application. That way, the temporary security credentials that your application will use are associated with that user. This session name is included as part of the ARN and assumed role ID in the <code>AssumedRoleUser</code> response element. The regex used to validate this parameter is a string of characters consisting of upper- and lower-case alphanumeric characters with no spaces. You can also include underscores or any of the following characters: =,.@-<br/>RoleArn:  The Amazon Resource Name (ARN) of the role that the caller is assuming.<br/>
        :return: Audience:  The intended audience (also known as client ID) of the web identity token. This is traditionally the client identifier issued to the application that requested the web identity token.<br/>Provider:   The issuing authority of the web identity token presented. For OpenID Connect ID tokens, this contains the value of the <code>iss</code> field. For OAuth 2.0 access tokens, this contains the value of the <code>ProviderId</code> parameter that was passed in the <code>AssumeRoleWithWebIdentity</code> request.<br/>PackedPolicySize:  A percentage value that indicates the size of the policy in packed form. The service rejects any policy with a packed size greater than 100 percent, which means the policy exceeded the allowed space.<br/>AssumedRoleUser:  The Amazon Resource Name (ARN) and the assumed role ID, which are identifiers that you can use to refer to the resulting temporary security credentials. For example, you can reference these credentials as a principal in a resource-based policy by using the ARN or assumed role ID. The ARN and ID include the <code>RoleSessionName</code> that you specified when you called <code>AssumeRole</code>. :<br/>SubjectFromWebIdentityToken:  The unique user identifier that is returned by the identity provider. This identifier is associated with the <code>WebIdentityToken</code> that was submitted with the <code>AssumeRoleWithWebIdentity</code> call. The identifier is typically unique to the user and the application that acquired the <code>WebIdentityToken</code> (pairwise identifier). For OpenID Connect ID tokens, this field contains the value returned by the identity provider as the token's <code>sub</code> (Subject) claim. <br/>Credentials:  The temporary security credentials, which include an access key ID, a secret access key, and a security token. <note> The size of the security token that STS API operations return is not fixed. We strongly recommend that you make no assumptions about the maximum size. </note>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Expiration</i> The date on which the current credentials expire.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SessionToken</i> The token that users must pass to the service API to use the temporary credentials.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecretAccessKey</i> The secret access key that can be used to sign requests.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccessKeyId</i> The access key ID that identifies the temporary security credentials.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN of the temporary security credentials that are returned from the <a>AssumeRole</a> action. For more information about ARNs and how to use them in policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_identifiers.html">IAM Identifiers</a> in <i>Using IAM</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AssumedRoleId</i> A unique identifier that contains the role ID and the role session name of the role that is being assumed. The role ID is generated by AWS when the role is created.<br/>        """
        pass

    def decode_authorization_message(self, EncodedMessage: str) -> DecodeAuthorizationMessageResponse:
        """Decodes additional information about the authorization status of a request from an encoded message returned in response to an AWS request. For example, if a user is not authorized to perform an operation that he or she has requested, the request returns a <code>Client.UnauthorizedOperation</code> response (an HTTP 403 response). Some AWS operations additionally return an encoded message that can provide details about this authorization failure.  <note> Only certain AWS operations return an encoded authorization message. The documentation for an individual operation indicates whether that operation returns an encoded message in addition to returning an HTTP code. </note> The message is encoded because the details of the authorization status can constitute privileged information that the user who requested the operation should not see. To decode an authorization status message, a user must be granted permissions via an IAM policy to request the <code>DecodeAuthorizationMessage</code> (<code>sts:DecodeAuthorizationMessage</code>) action.  The decoded message includes the following type of information: <ul> <li> Whether the request was denied due to an explicit deny or due to the absence of an explicit allow. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_policies_evaluation-logic.html#policy-eval-denyallow">Determining Whether a Request is Allowed or Denied</a> in the <i>IAM User Guide</i>.  </li> <li> The principal who made the request. </li> <li> The requested action. </li> <li> The requested resource. </li> <li> The values of condition keys in the context of the user's request. </li> </ul>

        :param self:
        :param EncodedMessage:  The encoded message that was returned with the response.<br/>
        :return: DecodedMessage:  An XML document that contains the decoded message.<br/>        """
        pass

    def get_caller_identity(self) -> GetCallerIdentityResponse:
        """Returns details about the IAM identity whose credentials are used to call the API.

        :param self:
        :param 
        :return: Arn:  The AWS ARN associated with the calling entity.<br/>Account:  The AWS account ID number of the account that owns or contains the calling entity.<br/>UserId:  The unique identifier of the calling entity. The exact value depends on the type of entity that is making the call. The values returned are those listed in the <b>aws:userid</b> column in the <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_policies_variables.html#principaltable">Principal table</a> found on the <b>Policy Variables</b> reference page in the <i>IAM User Guide</i>.<br/>        """
        pass

    def get_federation_token(self, Name: str, Policy: str=None, PolicyArns: List[PolicyDescriptorType]=None, DurationSeconds: int=None) -> GetFederationTokenResponse:
        """Returns a set of temporary security credentials (consisting of an access key ID, a secret access key, and a security token) for a federated user. A typical use is in a proxy application that gets temporary security credentials on behalf of distributed applications inside a corporate network. You must call the <code>GetFederationToken</code> operation using the long-term security credentials of an IAM user. As a result, this call is appropriate in contexts where those credentials can be safely stored, usually in a server-based application. For a comparison of <code>GetFederationToken</code> with the other API operations that produce temporary credentials, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_temp_request.html">Requesting Temporary Security Credentials</a> and <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_temp_request.html#stsapi_comparison">Comparing the AWS STS API operations</a> in the <i>IAM User Guide</i>. <note> You can create a mobile-based or browser-based app that can authenticate users using a web identity provider like Login with Amazon, Facebook, Google, or an OpenID Connect-compatible identity provider. In this case, we recommend that you use <a href="http://aws.amazon.com/cognito/">Amazon Cognito</a> or <code>AssumeRoleWithWebIdentity</code>. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_temp_request.html#api_assumerolewithwebidentity">Federation Through a Web-based Identity Provider</a>. </note> You can also call <code>GetFederationToken</code> using the security credentials of an AWS account root user, but we do not recommend it. Instead, we recommend that you create an IAM user for the purpose of the proxy application. Then attach a policy to the IAM user that limits federated users to only the actions and resources that they need to access. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/best-practices.html">IAM Best Practices</a> in the <i>IAM User Guide</i>.  The temporary credentials are valid for the specified duration, from 900 seconds (15 minutes) up to a maximum of 129,600 seconds (36 hours). The default is 43,200 seconds (12 hours). Temporary credentials that are obtained by using AWS account root user credentials have a maximum duration of 3,600 seconds (1 hour). The temporary security credentials created by <code>GetFederationToken</code> can be used to make API calls to any AWS service with the following exceptions: <ul> <li> You cannot use these credentials to call any IAM API operations. </li> <li> You cannot call any STS API operations except <code>GetCallerIdentity</code>. </li> </ul>  <b>Permissions</b>  You must pass an inline or managed <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies.html#policies_session">session policy</a> to this operation. You can pass a single JSON policy document to use as an inline session policy. You can also specify up to 10 managed policies to use as managed session policies. The plain text that you use for both inline and managed session policies shouldn't exceed 2048 characters. Though the session policy parameters are optional, if you do not pass a policy, then the resulting federated user session has no permissions. The only exception is when the credentials are used to access a resource that has a resource-based policy that specifically references the federated user session in the <code>Principal</code> element of the policy. When you pass session policies, the session permissions are the intersection of the IAM user policies and the session policies that you pass. This gives you a way to further restrict the permissions for a federated user. You cannot use session policies to grant more permissions than those that are defined in the permissions policy of the IAM user. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies.html#policies_session">Session Policies</a> in the <i>IAM User Guide</i>. For information about using <code>GetFederationToken</code> to create temporary security credentials, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_temp_request.html#api_getfederationtoken">GetFederationToken—Federation Through a Custom Identity Broker</a>. 

        :param self:
        :param DurationSeconds:  The duration, in seconds, that the session should last. Acceptable durations for federation sessions range from 900 seconds (15 minutes) to 129,600 seconds (36 hours), with 43,200 seconds (12 hours) as the default. Sessions obtained using AWS account root user credentials are restricted to a maximum of 3,600 seconds (one hour). If the specified duration is longer than one hour, the session obtained by using root user credentials defaults to one hour.<br/>PolicyArns:  The Amazon Resource Names (ARNs) of the IAM managed policies that you want to use as a managed session policy. The policies must exist in the same account as the IAM user that is requesting federated access. You must pass an inline or managed <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies.html#policies_session">session policy</a> to this operation. You can pass a single JSON policy document to use as an inline session policy. You can also specify up to 10 managed policies to use as managed session policies. The plain text that you use for both inline and managed session policies shouldn't exceed 2048 characters. You can provide up to 10 managed policy ARNs. For more information about ARNs, see <a href="general/latest/gr/aws-arns-and-namespaces.html">Amazon Resource Names (ARNs) and AWS Service Namespaces</a> in the AWS General Reference. This parameter is optional. However, if you do not pass any session policies, then the resulting federated user session has no permissions. The only exception is when the credentials are used to access a resource that has a resource-based policy that specifically references the federated user session in the <code>Principal</code> element of the policy. When you pass session policies, the session permissions are the intersection of the IAM user policies and the session policies that you pass. This gives you a way to further restrict the permissions for a federated user. You cannot use session policies to grant more permissions than those that are defined in the permissions policy of the IAM user. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies.html#policies_session">Session Policies</a> in the <i>IAM User Guide</i>. <note> The characters in this parameter count towards the 2048 character session policy guideline. However, an AWS conversion compresses the session policies into a packed binary format that has a separate limit. This is the enforced limit. The <code>PackedPolicySize</code> response element indicates by percentage how close the policy is to the upper size limit. </note><br/>Policy:  An IAM policy in JSON format that you want to use as an inline session policy. You must pass an inline or managed <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies.html#policies_session">session policy</a> to this operation. You can pass a single JSON policy document to use as an inline session policy. You can also specify up to 10 managed policies to use as managed session policies. This parameter is optional. However, if you do not pass any session policies, then the resulting federated user session has no permissions. The only exception is when the credentials are used to access a resource that has a resource-based policy that specifically references the federated user session in the <code>Principal</code> element of the policy. When you pass session policies, the session permissions are the intersection of the IAM user policies and the session policies that you pass. This gives you a way to further restrict the permissions for a federated user. You cannot use session policies to grant more permissions than those that are defined in the permissions policy of the IAM user. For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies.html#policies_session">Session Policies</a> in the <i>IAM User Guide</i>. The plain text that you use for both inline and managed session policies shouldn't exceed 2048 characters. The JSON policy characters can be any ASCII character from the space character to the end of the valid character list (\u0020 through \u00FF). It can also include the tab (\u0009), linefeed (\u000A), and carriage return (\u000D) characters. <note> The characters in this parameter count towards the 2048 character session policy guideline. However, an AWS conversion compresses the session policies into a packed binary format that has a separate limit. This is the enforced limit. The <code>PackedPolicySize</code> response element indicates by percentage how close the policy is to the upper size limit. </note><br/>Name:  The name of the federated user. The name is used as an identifier for the temporary security credentials (such as <code>Bob</code>). For example, you can reference the federated user name in a resource-based policy, such as in an Amazon S3 bucket policy. The regex used to validate this parameter is a string of characters consisting of upper- and lower-case alphanumeric characters with no spaces. You can also include underscores or any of the following characters: =,.@-<br/>
        :return: PackedPolicySize:  A percentage value indicating the size of the policy in packed form. The service rejects policies for which the packed size is greater than 100 percent of the allowed value.<br/>FederatedUser:  Identifiers for the federated user associated with the credentials (such as <code>arn:aws:sts::123456789012:federated-user/Bob</code> or <code>123456789012:Bob</code>). You can use the federated user's ARN in your resource-based policies, such as an Amazon S3 bucket policy. :<br/>Credentials:  The temporary security credentials, which include an access key ID, a secret access key, and a security (or session) token. <note> The size of the security token that STS API operations return is not fixed. We strongly recommend that you make no assumptions about the maximum size. </note>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Expiration</i> The date on which the current credentials expire.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SessionToken</i> The token that users must pass to the service API to use the temporary credentials.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecretAccessKey</i> The secret access key that can be used to sign requests.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccessKeyId</i> The access key ID that identifies the temporary security credentials.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> The ARN that specifies the federated user that is associated with the credentials. For more information about ARNs and how to use them in policies, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_identifiers.html">IAM Identifiers</a> in <i>Using IAM</i>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FederatedUserId</i> The string that identifies the federated user associated with the credentials, similar to the unique ID of an IAM user.<br/>        """
        pass

    def get_session_token(self, DurationSeconds: int=None, SerialNumber: str=None, TokenCode: str=None) -> GetSessionTokenResponse:
        """Returns a set of temporary credentials for an AWS account or IAM user. The credentials consist of an access key ID, a secret access key, and a security token. Typically, you use <code>GetSessionToken</code> if you want to use MFA to protect programmatic calls to specific AWS API operations like Amazon EC2 <code>StopInstances</code>. MFA-enabled IAM users would need to call <code>GetSessionToken</code> and submit an MFA code that is associated with their MFA device. Using the temporary security credentials that are returned from the call, IAM users can then make programmatic calls to API operations that require MFA authentication. If you do not supply a correct MFA code, then the API returns an access denied error. For a comparison of <code>GetSessionToken</code> with the other API operations that produce temporary credentials, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_temp_request.html">Requesting Temporary Security Credentials</a> and <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_temp_request.html#stsapi_comparison">Comparing the AWS STS API operations</a> in the <i>IAM User Guide</i>. The <code>GetSessionToken</code> operation must be called by using the long-term AWS security credentials of the AWS account root user or an IAM user. Credentials that are created by IAM users are valid for the duration that you specify. This duration can range from 900 seconds (15 minutes) up to a maximum of 129,600 seconds (36 hours), with a default of 43,200 seconds (12 hours). Credentials based on account credentials can range from 900 seconds (15 minutes) up to 3,600 seconds (1 hour), with a default of 1 hour.  The temporary security credentials created by <code>GetSessionToken</code> can be used to make API calls to any AWS service with the following exceptions: <ul> <li> You cannot call any IAM API operations unless MFA authentication information is included in the request. </li> <li> You cannot call any STS API <i>except</i> <code>AssumeRole</code> or <code>GetCallerIdentity</code>. </li> </ul> <note> We recommend that you do not call <code>GetSessionToken</code> with AWS account root user credentials. Instead, follow our <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/best-practices.html#create-iam-users">best practices</a> by creating one or more IAM users, giving them the necessary permissions, and using IAM users for everyday interaction with AWS.  </note> The credentials that are returned by <code>GetSessionToken</code> are based on permissions associated with the user whose credentials were used to call the operation. If <code>GetSessionToken</code> is called using AWS account root user credentials, the temporary credentials have root user permissions. Similarly, if <code>GetSessionToken</code> is called using the credentials of an IAM user, the temporary credentials have the same permissions as the IAM user.  For more information about using <code>GetSessionToken</code> to create temporary credentials, go to <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_temp_request.html#api_getsessiontoken">Temporary Credentials for Users in Untrusted Environments</a> in the <i>IAM User Guide</i>. 

        :param self:
        :param TokenCode:  The value provided by the MFA device, if MFA is required. If any policy requires the IAM user to submit an MFA code, specify this value. If MFA authentication is required, the user must provide a code when requesting a set of temporary security credentials. A user who fails to provide the code receives an "access denied" response when requesting resources that require MFA authentication. The format for this parameter, as described by its regex pattern, is a sequence of six numeric digits.<br/>SerialNumber:  The identification number of the MFA device that is associated with the IAM user who is making the <code>GetSessionToken</code> call. Specify this value if the IAM user has a policy that requires MFA authentication. The value is either the serial number for a hardware device (such as <code>GAHT12345678</code>) or an Amazon Resource Name (ARN) for a virtual device (such as <code>arn:aws:iam::123456789012:mfa/user</code>). You can find the device for an IAM user by going to the AWS Management Console and viewing the user's security credentials.  The regex used to validate this parameter is a string of characters consisting of upper- and lower-case alphanumeric characters with no spaces. You can also include underscores or any of the following characters: =,.@:/-<br/>DurationSeconds:  The duration, in seconds, that the credentials should remain valid. Acceptable durations for IAM user sessions range from 900 seconds (15 minutes) to 129,600 seconds (36 hours), with 43,200 seconds (12 hours) as the default. Sessions for AWS account owners are restricted to a maximum of 3,600 seconds (one hour). If the duration is longer than one hour, the session for AWS account owners defaults to one hour.<br/>
        :return: Credentials:  The temporary security credentials, which include an access key ID, a secret access key, and a security (or session) token. <note> The size of the security token that STS API operations return is not fixed. We strongly recommend that you make no assumptions about the maximum size. </note>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Expiration</i> The date on which the current credentials expire.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SessionToken</i> The token that users must pass to the service API to use the temporary credentials.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SecretAccessKey</i> The secret access key that can be used to sign requests.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AccessKeyId</i> The access key ID that identifies the temporary security credentials.<br/>        """
        pass



class Support(BaseClient):
    """<fullname>AWS Support</fullname> The AWS Support API reference is intended for programmers who need detailed information about the AWS Support operations and data types. This service enables you to manage your AWS Support cases programmatically. It uses HTTP methods that return results in JSON format. The AWS Support service also exposes a set of <a href="http://aws.amazon.com/premiumsupport/trustedadvisor/">Trusted Advisor</a> features. You can retrieve a list of checks and their descriptions, get check results, specify checks to refresh, and get the refresh status of checks.  The following list describes the AWS Support case management operations: <ul> <li>  <b>Service names, issue categories, and available severity levels. </b>The <a>DescribeServices</a> and <a>DescribeSeverityLevels</a> operations return AWS service names, service codes, service categories, and problem severity levels. You use these values when you call the <a>CreateCase</a> operation.  </li> <li>  <b>Case creation, case details, and case resolution.</b> The <a>CreateCase</a>, <a>DescribeCases</a>, <a>DescribeAttachment</a>, and <a>ResolveCase</a> operations create AWS Support cases, retrieve information about cases, and resolve cases. </li> <li>  <b>Case communication.</b> The <a>DescribeCommunications</a>, <a>AddCommunicationToCase</a>, and <a>AddAttachmentsToSet</a> operations retrieve and add communications and attachments to AWS Support cases.  </li> </ul> The following list describes the operations available from the AWS Support service for Trusted Advisor: <ul> <li>  <a>DescribeTrustedAdvisorChecks</a> returns the list of checks that run against your AWS resources. </li> <li> Using the <code>checkId</code> for a specific check returned by <a>DescribeTrustedAdvisorChecks</a>, you can call <a>DescribeTrustedAdvisorCheckResult</a> to obtain the results for the check you specified. </li> <li>  <a>DescribeTrustedAdvisorCheckSummaries</a> returns summarized results for one or more Trusted Advisor checks. </li> <li>  <a>RefreshTrustedAdvisorCheck</a> requests that Trusted Advisor rerun a specified check.  </li> <li>  <a>DescribeTrustedAdvisorCheckRefreshStatuses</a> reports the refresh status of one or more checks.  </li> </ul> For authentication of requests, AWS Support uses <a href="http://docs.aws.amazon.com/general/latest/gr/signature-version-4.html">Signature Version 4 Signing Process</a>. See <a href="http://docs.aws.amazon.com/awssupport/latest/user/Welcome.html">About the AWS Support API</a> in the <i>AWS Support User Guide</i> for information about how to use this service to create and manage your support cases, and how to call Trusted Advisor for results of checks on your resources. """


    class DescribeCasesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_cases` for valid parameters"""
            pass
    class DescribeCommunicationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_communications` for valid parameters"""
            pass


    class AddAttachmentsToSetRequest(Mapping):
        pass
    
    class AddAttachmentsToSetResponse(Mapping):
        pass
    
    class Attachment(Mapping):
        pass
    
    class AddCommunicationToCaseRequest(Mapping):
        pass
    
    class AddCommunicationToCaseResponse(Mapping):
        pass
    
    class CcEmailAddress(object):
        pass
    
    class CreateCaseRequest(Mapping):
        pass
    
    class CreateCaseResponse(Mapping):
        pass
    
    class DescribeAttachmentRequest(Mapping):
        pass
    
    class DescribeAttachmentResponse(Mapping):
        pass
    
    class DescribeCasesRequest(Mapping):
        pass
    
    class DescribeCasesResponse(Mapping):
        pass
    
    class CaseId(object):
        pass
    
    class IncludeResolvedCases(object):
        pass
    
    class IncludeCommunications(object):
        pass
    
    class DescribeCommunicationsRequest(Mapping):
        pass
    
    class DescribeCommunicationsResponse(Mapping):
        pass
    
    class DescribeServicesRequest(Mapping):
        pass
    
    class DescribeServicesResponse(Mapping):
        pass
    
    class ServiceCode(object):
        pass
    
    class DescribeSeverityLevelsRequest(Mapping):
        pass
    
    class DescribeSeverityLevelsResponse(Mapping):
        pass
    
    class DescribeTrustedAdvisorCheckRefreshStatusesRequest(Mapping):
        pass
    
    class DescribeTrustedAdvisorCheckRefreshStatusesResponse(Mapping):
        pass
    
    class String(object):
        pass
    
    class DescribeTrustedAdvisorCheckResultRequest(Mapping):
        pass
    
    class DescribeTrustedAdvisorCheckResultResponse(Mapping):
        pass
    
    class DescribeTrustedAdvisorCheckSummariesRequest(Mapping):
        pass
    
    class DescribeTrustedAdvisorCheckSummariesResponse(Mapping):
        pass
    
    class DescribeTrustedAdvisorChecksRequest(Mapping):
        pass
    
    class DescribeTrustedAdvisorChecksResponse(Mapping):
        pass
    
    class RefreshTrustedAdvisorCheckRequest(Mapping):
        pass
    
    class RefreshTrustedAdvisorCheckResponse(Mapping):
        pass
    
    class ResolveCaseRequest(Mapping):
        pass
    
    class ResolveCaseResponse(Mapping):
        pass
    
    def add_attachments_to_set(self, attachments: List[Attachment], attachmentSetId: str=None) -> AddAttachmentsToSetResponse:
        """Adds one or more attachments to an attachment set. If an <code>attachmentSetId</code> is not specified, a new attachment set is created, and the ID of the set is returned in the response. If an <code>attachmentSetId</code> is specified, the attachments are added to the specified set, if it exists. An attachment set is a temporary container for attachments that are to be added to a case or case communication. The set is available for one hour after it is created; the <code>expiryTime</code> returned in the response indicates when the set expires. The maximum number of attachments in a set is 3, and the maximum size of any attachment in the set is 5 MB.

        :param self:
        :param attachments:  One or more attachments to add to the set. The limit is 3 attachments per set, and the size limit is 5 MB per attachment.<br/>attachmentSetId:  The ID of the attachment set. If an <code>attachmentSetId</code> is not specified, a new attachment set is created, and the ID of the set is returned in the response. If an <code>attachmentSetId</code> is specified, the attachments are added to the specified set, if it exists.<br/>
        :return: expiryTime:  The time and date when the attachment set expires.<br/>attachmentSetId:  The ID of the attachment set. If an <code>attachmentSetId</code> was not specified, a new attachment set is created, and the ID of the set is returned in the response. If an <code>attachmentSetId</code> was specified, the attachments are added to the specified set, if it exists.<br/>        """
        pass

    def add_communication_to_case(self, communicationBody: str, caseId: str=None, ccEmailAddresses: List[CcEmailAddress]=None, attachmentSetId: str=None) -> AddCommunicationToCaseResponse:
        """Adds additional customer communication to an AWS Support case. You use the <code>caseId</code> value to identify the case to add communication to. You can list a set of email addresses to copy on the communication using the <code>ccEmailAddresses</code> value. The <code>communicationBody</code> value contains the text of the communication. The response indicates the success or failure of the request. This operation implements a subset of the features of the AWS Support Center.

        :param self:
        :param attachmentSetId:  The ID of a set of one or more attachments for the communication to add to the case. Create the set by calling <a>AddAttachmentsToSet</a> <br/>ccEmailAddresses:  The email addresses in the CC line of an email to be added to the support case.<br/>communicationBody:  The body of an email communication to add to the support case.<br/>caseId:  The AWS Support case ID requested or returned in the call. The case ID is an alphanumeric string formatted as shown in this example: case-<i>12345678910-2013-c4c1d2bf33c5cf47</i> <br/>
        :return: result:  True if <a>AddCommunicationToCase</a> succeeds. Otherwise, returns an error.<br/>        """
        pass

    def create_case(self, communicationBody: str, subject: str, serviceCode: str=None, severityCode: str=None, categoryCode: str=None, ccEmailAddresses: List[CcEmailAddress]=None, language: str=None, issueType: str=None, attachmentSetId: str=None) -> CreateCaseResponse:
        """Creates a new case in the AWS Support Center. This operation is modeled on the behavior of the AWS Support Center <a href="https://console.aws.amazon.com/support/home#/case/create">Create Case</a> page. Its parameters require you to specify the following information:  <ul> <li>  <b>issueType.</b> The type of issue for the case. You can specify either "customer-service" or "technical." If you do not indicate a value, the default is "technical."  </li> <li>  <b>serviceCode.</b> The code for an AWS service. You obtain the <code>serviceCode</code> by calling <a>DescribeServices</a>.  </li> <li>  <b>categoryCode.</b> The category for the service defined for the <code>serviceCode</code> value. You also obtain the category code for a service by calling <a>DescribeServices</a>. Each AWS service defines its own set of category codes.  </li> <li>  <b>severityCode.</b> A value that indicates the urgency of the case, which in turn determines the response time according to your service level agreement with AWS Support. You obtain the SeverityCode by calling <a>DescribeSeverityLevels</a>. </li> <li>  <b>subject.</b> The <b>Subject</b> field on the AWS Support Center <a href="https://console.aws.amazon.com/support/home#/case/create">Create Case</a> page. </li> <li>  <b>communicationBody.</b> The <b>Description</b> field on the AWS Support Center <a href="https://console.aws.amazon.com/support/home#/case/create">Create Case</a> page. </li> <li>  <b>attachmentSetId.</b> The ID of a set of attachments that has been created by using <a>AddAttachmentsToSet</a>. </li> <li>  <b>language.</b> The human language in which AWS Support handles the case. English and Japanese are currently supported. </li> <li>  <b>ccEmailAddresses.</b> The AWS Support Center <b>CC</b> field on the <a href="https://console.aws.amazon.com/support/home#/case/create">Create Case</a> page. You can list email addresses to be copied on any correspondence about the case. The account that opens the case is already identified by passing the AWS Credentials in the HTTP POST method or in a method or function call from one of the programming languages supported by an <a href="http://aws.amazon.com/tools/">AWS SDK</a>.  </li> </ul> <note> To add additional communication or attachments to an existing case, use <a>AddCommunicationToCase</a>. </note> A successful <a>CreateCase</a> request returns an AWS Support case number. Case numbers are used by the <a>DescribeCases</a> operation to retrieve existing AWS Support cases. 

        :param self:
        :param attachmentSetId:  The ID of a set of one or more attachments for the case. Create the set by using <a>AddAttachmentsToSet</a>.<br/>issueType:  The type of issue for the case. You can specify either "customer-service" or "technical." If you do not indicate a value, the default is "technical."<br/>language:  The ISO 639-1 code for the language in which AWS provides support. AWS Support currently supports English ("en") and Japanese ("ja"). Language parameters must be passed explicitly for operations that take them.<br/>ccEmailAddresses:  A list of email addresses that AWS Support copies on case correspondence.<br/>communicationBody:  The communication body text when you create an AWS Support case by calling <a>CreateCase</a>.<br/>categoryCode:  The category of problem for the AWS Support case.<br/>severityCode:  The code for the severity level returned by the call to <a>DescribeSeverityLevels</a>. <note> The availability of severity levels depends on each customer's support subscription. In other words, your subscription may not necessarily require the urgent level of response time. </note><br/>serviceCode:  The code for the AWS service returned by the call to <a>DescribeServices</a>.<br/>subject:  The title of the AWS Support case.<br/>
        :return: caseId:  The AWS Support case ID requested or returned in the call. The case ID is an alphanumeric string formatted as shown in this example: case-<i>12345678910-2013-c4c1d2bf33c5cf47</i> <br/>        """
        pass

    def describe_attachment(self, attachmentId: str) -> DescribeAttachmentResponse:
        """Returns the attachment that has the specified ID. Attachment IDs are generated by the case management system when you add an attachment to a case or case communication. Attachment IDs are returned in the <a>AttachmentDetails</a> objects that are returned by the <a>DescribeCommunications</a> operation.

        :param self:
        :param attachmentId:  The ID of the attachment to return. Attachment IDs are returned by the <a>DescribeCommunications</a> operation.<br/>
        :return: attachment:  The attachment content and file name.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>data</i> The content of the attachment file.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>fileName</i> The name of the attachment file.<br/>        """
        pass

    def describe_cases(self, caseIdList: List[CaseId]=None, displayId: str=None, afterTime: str=None, beforeTime: str=None, includeResolvedCases: IncludeResolvedCases=None, nextToken: str=None, maxResults: int=None, language: str=None, includeCommunications: IncludeCommunications=None) -> DescribeCasesResponse:
        """Returns a list of cases that you specify by passing one or more case IDs. In addition, you can filter the cases by date by setting values for the <code>afterTime</code> and <code>beforeTime</code> request parameters. You can set values for the <code>includeResolvedCases</code> and <code>includeCommunications</code> request parameters to control how much information is returned.  Case data is available for 12 months after creation. If a case was created more than 12 months ago, a request for data might cause an error. The response returns the following in JSON format: <ul> <li> One or more <a>CaseDetails</a> data types.  </li> <li> One or more <code>nextToken</code> values, which specify where to paginate the returned records represented by the <code>CaseDetails</code> objects. </li> </ul>

        :param self:
        :param includeCommunications:  Specifies whether communications should be included in the <a>DescribeCases</a> results. The default is <i>true</i>.<br/>language:  The ISO 639-1 code for the language in which AWS provides support. AWS Support currently supports English ("en") and Japanese ("ja"). Language parameters must be passed explicitly for operations that take them.<br/>maxResults:  The maximum number of results to return before paginating.<br/>nextToken:  A resumption point for pagination.<br/>includeResolvedCases:  Specifies whether resolved support cases should be included in the <a>DescribeCases</a> results. The default is <i>false</i>.<br/>beforeTime:  The end date for a filtered date search on support case communications. Case communications are available for 12 months after creation.<br/>afterTime:  The start date for a filtered date search on support case communications. Case communications are available for 12 months after creation.<br/>displayId:  The ID displayed for a case in the AWS Support Center user interface.<br/>caseIdList:  A list of ID numbers of the support cases you want returned. The maximum number of cases is 100.<br/>
        :return: nextToken:  A resumption point for pagination.<br/>cases:  The details for the cases that match the request.<br/>        """
        pass

    def describe_communications(self, caseId: str, beforeTime: str=None, afterTime: str=None, nextToken: str=None, maxResults: int=None) -> DescribeCommunicationsResponse:
        """Returns communications (and attachments) for one or more support cases. You can use the <code>afterTime</code> and <code>beforeTime</code> parameters to filter by date. You can use the <code>caseId</code> parameter to restrict the results to a particular case. Case data is available for 12 months after creation. If a case was created more than 12 months ago, a request for data might cause an error. You can use the <code>maxResults</code> and <code>nextToken</code> parameters to control the pagination of the result set. Set <code>maxResults</code> to the number of cases you want displayed on each page, and use <code>nextToken</code> to specify the resumption of pagination.

        :param self:
        :param maxResults:  The maximum number of results to return before paginating.<br/>nextToken:  A resumption point for pagination.<br/>afterTime:  The start date for a filtered date search on support case communications. Case communications are available for 12 months after creation.<br/>beforeTime:  The end date for a filtered date search on support case communications. Case communications are available for 12 months after creation.<br/>caseId:  The AWS Support case ID requested or returned in the call. The case ID is an alphanumeric string formatted as shown in this example: case-<i>12345678910-2013-c4c1d2bf33c5cf47</i> <br/>
        :return: nextToken:  A resumption point for pagination.<br/>communications:  The communications for the case.<br/>        """
        pass

    def describe_services(self, serviceCodeList: List[ServiceCode]=None, language: str=None) -> DescribeServicesResponse:
        """Returns the current list of AWS services and a list of service categories that applies to each one. You then use service names and categories in your <a>CreateCase</a> requests. Each AWS service has its own set of categories. The service codes and category codes correspond to the values that are displayed in the <b>Service</b> and <b>Category</b> drop-down lists on the AWS Support Center <a href="https://console.aws.amazon.com/support/home#/case/create">Create Case</a> page. The values in those fields, however, do not necessarily match the service codes and categories returned by the <code>DescribeServices</code> request. Always use the service codes and categories obtained programmatically. This practice ensures that you always have the most recent set of service and category codes.

        :param self:
        :param language:  The ISO 639-1 code for the language in which AWS provides support. AWS Support currently supports English ("en") and Japanese ("ja"). Language parameters must be passed explicitly for operations that take them.<br/>serviceCodeList:  A JSON-formatted list of service codes available for AWS services.<br/>
        :return: services:  A JSON-formatted list of AWS services.<br/>        """
        pass

    def describe_severity_levels(self, language: str=None) -> DescribeSeverityLevelsResponse:
        """Returns the list of severity levels that you can assign to an AWS Support case. The severity level for a case is also a field in the <a>CaseDetails</a> data type included in any <a>CreateCase</a> request. 

        :param self:
        :param language:  The ISO 639-1 code for the language in which AWS provides support. AWS Support currently supports English ("en") and Japanese ("ja"). Language parameters must be passed explicitly for operations that take them.<br/>
        :return: severityLevels:  The available severity levels for the support case. Available severity levels are defined by your service level agreement with AWS.<br/>        """
        pass

    def describe_trusted_advisor_check_refresh_statuses(self, checkIds: List[String]) -> DescribeTrustedAdvisorCheckRefreshStatusesResponse:
        """Returns the refresh status of the Trusted Advisor checks that have the specified check IDs. Check IDs can be obtained by calling <a>DescribeTrustedAdvisorChecks</a>. <note> Some checks are refreshed automatically, and their refresh statuses cannot be retrieved by using this operation. Use of the <code>DescribeTrustedAdvisorCheckRefreshStatuses</code> operation for these checks causes an <code>InvalidParameterValue</code> error. </note>

        :param self:
        :param checkIds:  The IDs of the Trusted Advisor checks to get the status of. <b>Note:</b> Specifying the check ID of a check that is automatically refreshed causes an <code>InvalidParameterValue</code> error.<br/>
        :return: statuses:  The refresh status of the specified Trusted Advisor checks.<br/>        """
        pass

    def describe_trusted_advisor_check_result(self, checkId: str, language: str=None) -> DescribeTrustedAdvisorCheckResultResponse:
        """Returns the results of the Trusted Advisor check that has the specified check ID. Check IDs can be obtained by calling <a>DescribeTrustedAdvisorChecks</a>. The response contains a <a>TrustedAdvisorCheckResult</a> object, which contains these three objects: <ul> <li>  <a>TrustedAdvisorCategorySpecificSummary</a>  </li> <li>  <a>TrustedAdvisorResourceDetail</a>  </li> <li>  <a>TrustedAdvisorResourcesSummary</a>  </li> </ul> In addition, the response contains these fields: <ul> <li>  <b>status.</b> The alert status of the check: "ok" (green), "warning" (yellow), "error" (red), or "not_available". </li> <li>  <b>timestamp.</b> The time of the last refresh of the check. </li> <li>  <b>checkId.</b> The unique identifier for the check. </li> </ul>

        :param self:
        :param language:  The ISO 639-1 code for the language in which AWS provides support. AWS Support currently supports English ("en") and Japanese ("ja"). Language parameters must be passed explicitly for operations that take them.<br/>checkId:  The unique identifier for the Trusted Advisor check.<br/>
        :return: result:  The detailed results of the Trusted Advisor check.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>flaggedResources</i> The details about each resource listed in the check result.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>categorySpecificSummary</i> Summary information that relates to the category of the check. Cost Optimizing is the only category that is currently supported.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>categorySpecificSummary</b>: Summary information that relates to the category of the check. Cost Optimizing is the only category that is currently supported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>resourcesSummary</i> Details about AWS resources that were analyzed in a call to Trusted Advisor <a>DescribeTrustedAdvisorCheckSummaries</a>. &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>resourcesSummary</b>: Details about AWS resources that were analyzed in a call to Trusted Advisor <a>DescribeTrustedAdvisorCheckSummaries</a>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The alert status of the check: "ok" (green), "warning" (yellow), "error" (red), or "not_available".<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>timestamp</i> The time of the last refresh of the check.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>checkId</i> The unique identifier for the Trusted Advisor check.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>resourcesSuppressed</i> The number of AWS resources ignored by Trusted Advisor because they were marked as suppressed by the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>resourcesIgnored</i> The number of AWS resources ignored by Trusted Advisor because information was unavailable.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>resourcesFlagged</i> The number of AWS resources that were flagged (listed) by the Trusted Advisor check.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>resourcesProcessed</i> The number of AWS resources that were analyzed by the Trusted Advisor check.<br/>The summary information about cost savings for a Trusted Advisor check that is in the Cost Optimizing category.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>costOptimizing</b>: The summary information about cost savings for a Trusted Advisor check that is in the Cost Optimizing category.        """
        pass

    def describe_trusted_advisor_check_summaries(self, checkIds: List[String]) -> DescribeTrustedAdvisorCheckSummariesResponse:
        """Returns the summaries of the results of the Trusted Advisor checks that have the specified check IDs. Check IDs can be obtained by calling <a>DescribeTrustedAdvisorChecks</a>. The response contains an array of <a>TrustedAdvisorCheckSummary</a> objects.

        :param self:
        :param checkIds:  The IDs of the Trusted Advisor checks.<br/>
        :return: summaries:  The summary information for the requested Trusted Advisor checks.<br/>        """
        pass

    def describe_trusted_advisor_checks(self, language: str) -> DescribeTrustedAdvisorChecksResponse:
        """Returns information about all available Trusted Advisor checks, including name, ID, category, description, and metadata. You must specify a language code; English ("en") and Japanese ("ja") are currently supported. The response contains a <a>TrustedAdvisorCheckDescription</a> for each check.

        :param self:
        :param language:  The ISO 639-1 code for the language in which AWS provides support. AWS Support currently supports English ("en") and Japanese ("ja"). Language parameters must be passed explicitly for operations that take them.<br/>
        :return: checks:  Information about all available Trusted Advisor checks.<br/>        """
        pass

    def refresh_trusted_advisor_check(self, checkId: str) -> RefreshTrustedAdvisorCheckResponse:
        """Requests a refresh of the Trusted Advisor check that has the specified check ID. Check IDs can be obtained by calling <a>DescribeTrustedAdvisorChecks</a>. <note> Some checks are refreshed automatically, and they cannot be refreshed by using this operation. Use of the <code>RefreshTrustedAdvisorCheck</code> operation for these checks causes an <code>InvalidParameterValue</code> error. </note> The response contains a <a>TrustedAdvisorCheckRefreshStatus</a> object, which contains these fields: <ul> <li>  <b>status.</b> The refresh status of the check: "none", "enqueued", "processing", "success", or "abandoned". </li> <li>  <b>millisUntilNextRefreshable.</b> The amount of time, in milliseconds, until the check is eligible for refresh. </li> <li>  <b>checkId.</b> The unique identifier for the check. </li> </ul>

        :param self:
        :param checkId:  The unique identifier for the Trusted Advisor check to refresh. <b>Note:</b> Specifying the check ID of a check that is automatically refreshed causes an <code>InvalidParameterValue</code> error.<br/>
        :return: status:  The current refresh status for a check, including the amount of time until the check is eligible for refresh.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>millisUntilNextRefreshable</i> The amount of time, in milliseconds, until the Trusted Advisor check is eligible for refresh.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the Trusted Advisor check for which a refresh has been requested: "none", "enqueued", "processing", "success", or "abandoned".<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>checkId</i> The unique identifier for the Trusted Advisor check.<br/>        """
        pass

    def resolve_case(self, caseId: str=None) -> ResolveCaseResponse:
        """Takes a <code>caseId</code> and returns the initial state of the case along with the state of the case after the call to <a>ResolveCase</a> completed.

        :param self:
        :param caseId:  The AWS Support case ID requested or returned in the call. The case ID is an alphanumeric string formatted as shown in this example: case-<i>12345678910-2013-c4c1d2bf33c5cf47</i> <br/>
        :return: finalCaseStatus:  The status of the case after the <a>ResolveCase</a> request was processed.<br/>initialCaseStatus:  The status of the case when the <a>ResolveCase</a> request was sent.<br/>        """
        pass



class SWF(BaseClient):
    """<fullname>Amazon Simple Workflow Service</fullname> The Amazon Simple Workflow Service (Amazon SWF) makes it easy to build applications that use Amazon's cloud to coordinate work across distributed components. In Amazon SWF, a <i>task</i> represents a logical unit of work that is performed by a component of your workflow. Coordinating tasks in a workflow involves managing intertask dependencies, scheduling, and concurrency in accordance with the logical flow of the application. Amazon SWF gives you full control over implementing tasks and coordinating them without worrying about underlying complexities such as tracking their progress and maintaining their state. This documentation serves as reference only. For a broader overview of the Amazon SWF programming model, see the <i> <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/">Amazon SWF Developer Guide</a> </i>."""


    class GetWorkflowExecutionHistoryPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_workflow_execution_history` for valid parameters"""
            pass
    class ListActivityTypesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_activity_types` for valid parameters"""
            pass
    class ListClosedWorkflowExecutionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_closed_workflow_executions` for valid parameters"""
            pass
    class ListDomainsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_domains` for valid parameters"""
            pass
    class ListOpenWorkflowExecutionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_open_workflow_executions` for valid parameters"""
            pass
    class ListWorkflowTypesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_workflow_types` for valid parameters"""
            pass
    class PollForDecisionTaskPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `poll_for_decision_task` for valid parameters"""
            pass


    class CountClosedWorkflowExecutionsInput(Mapping):
        pass
    
    class WorkflowExecutionCount(Mapping):
        pass
    
    class ExecutionTimeFilter(Mapping):
        pass
    
    class WorkflowExecutionFilter(Mapping):
        pass
    
    class WorkflowTypeFilter(Mapping):
        pass
    
    class TagFilter(Mapping):
        pass
    
    class CloseStatusFilter(Mapping):
        pass
    
    class CountOpenWorkflowExecutionsInput(Mapping):
        pass
    
    class CountPendingActivityTasksInput(Mapping):
        pass
    
    class PendingTaskCount(Mapping):
        pass
    
    class TaskList(Mapping):
        pass
    
    class CountPendingDecisionTasksInput(Mapping):
        pass
    
    class DeprecateActivityTypeInput(Mapping):
        pass
    
    class ActivityType(Mapping):
        pass
    
    class DeprecateDomainInput(Mapping):
        pass
    
    class DeprecateWorkflowTypeInput(Mapping):
        pass
    
    class WorkflowType(Mapping):
        pass
    
    class DescribeActivityTypeInput(Mapping):
        pass
    
    class ActivityTypeDetail(Mapping):
        pass
    
    class DescribeDomainInput(Mapping):
        pass
    
    class DomainDetail(Mapping):
        pass
    
    class DescribeWorkflowExecutionInput(Mapping):
        pass
    
    class WorkflowExecutionDetail(Mapping):
        pass
    
    class WorkflowExecution(Mapping):
        pass
    
    class DescribeWorkflowTypeInput(Mapping):
        pass
    
    class WorkflowTypeDetail(Mapping):
        pass
    
    class GetWorkflowExecutionHistoryInput(Mapping):
        pass
    
    class History(Mapping):
        pass
    
    class ReverseOrder(object):
        pass
    
    class ListActivityTypesInput(Mapping):
        pass
    
    class ActivityTypeInfos(Mapping):
        pass
    
    class ListClosedWorkflowExecutionsInput(Mapping):
        pass
    
    class WorkflowExecutionInfos(Mapping):
        pass
    
    class ListDomainsInput(Mapping):
        pass
    
    class DomainInfos(Mapping):
        pass
    
    class ListOpenWorkflowExecutionsInput(Mapping):
        pass
    
    class ListTagsForResourceInput(Mapping):
        pass
    
    class ListTagsForResourceOutput(Mapping):
        pass
    
    class ListWorkflowTypesInput(Mapping):
        pass
    
    class WorkflowTypeInfos(Mapping):
        pass
    
    class PollForActivityTaskInput(Mapping):
        pass
    
    class ActivityTask(Mapping):
        pass
    
    class PollForDecisionTaskInput(Mapping):
        pass
    
    class DecisionTask(Mapping):
        pass
    
    class RecordActivityTaskHeartbeatInput(Mapping):
        pass
    
    class ActivityTaskStatus(Mapping):
        pass
    
    class RegisterActivityTypeInput(Mapping):
        pass
    
    class RegisterDomainInput(Mapping):
        pass
    
    class ResourceTag(Mapping):
        pass
    
    class RegisterWorkflowTypeInput(Mapping):
        pass
    
    class RequestCancelWorkflowExecutionInput(Mapping):
        pass
    
    class RespondActivityTaskCanceledInput(Mapping):
        pass
    
    class RespondActivityTaskCompletedInput(Mapping):
        pass
    
    class RespondActivityTaskFailedInput(Mapping):
        pass
    
    class RespondDecisionTaskCompletedInput(Mapping):
        pass
    
    class Decision(Mapping):
        pass
    
    class SignalWorkflowExecutionInput(Mapping):
        pass
    
    class StartWorkflowExecutionInput(Mapping):
        pass
    
    class Run(Mapping):
        pass
    
    class Tag(object):
        pass
    
    class TagResourceInput(Mapping):
        pass
    
    class TerminateWorkflowExecutionInput(Mapping):
        pass
    
    class UndeprecateActivityTypeInput(Mapping):
        pass
    
    class UndeprecateDomainInput(Mapping):
        pass
    
    class UndeprecateWorkflowTypeInput(Mapping):
        pass
    
    class UntagResourceInput(Mapping):
        pass
    
    class ResourceTagKey(object):
        pass
    
    def count_closed_workflow_executions(self, domain: str, startTimeFilter: ExecutionTimeFilter=None, closeTimeFilter: ExecutionTimeFilter=None, executionFilter: WorkflowExecutionFilter=None, typeFilter: WorkflowTypeFilter=None, tagFilter: TagFilter=None, closeStatusFilter: CloseStatusFilter=None) -> WorkflowExecutionCount:
        """Returns the number of closed workflow executions within the given domain that meet the specified filtering criteria. <note> This operation is eventually consistent. The results are best effort and may not exactly reflect recent updates and changes. </note>  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> Constrain the following parameters by using a <code>Condition</code> element with the appropriate keys. <ul> <li>  <code>tagFilter.tag</code>: String constraint. The key is <code>swf:tagFilter.tag</code>. </li> <li>  <code>typeFilter.name</code>: String constraint. The key is <code>swf:typeFilter.name</code>. </li> <li>  <code>typeFilter.version</code>: String constraint. The key is <code>swf:typeFilter.version</code>. </li> </ul> </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param closeStatusFilter:  If specified, only workflow executions that match this close status are counted. This filter has an affect only if <code>executionStatus</code> is specified as <code>CLOSED</code>. <note>  <code>closeStatusFilter</code>, <code>executionFilter</code>, <code>typeFilter</code> and <code>tagFilter</code> are mutually exclusive. You can specify at most one of these in a request. </note>:<br/>tagFilter:  If specified, only executions that have a tag that matches the filter are counted. <note>  <code>closeStatusFilter</code>, <code>executionFilter</code>, <code>typeFilter</code> and <code>tagFilter</code> are mutually exclusive. You can specify at most one of these in a request. </note>:<br/>typeFilter:  If specified, indicates the type of the workflow executions to be counted. <note>  <code>closeStatusFilter</code>, <code>executionFilter</code>, <code>typeFilter</code> and <code>tagFilter</code> are mutually exclusive. You can specify at most one of these in a request. </note>:<br/>executionFilter:  If specified, only workflow executions matching the <code>WorkflowId</code> in the filter are counted. <note>  <code>closeStatusFilter</code>, <code>executionFilter</code>, <code>typeFilter</code> and <code>tagFilter</code> are mutually exclusive. You can specify at most one of these in a request. </note>:<br/>closeTimeFilter:  If specified, only workflow executions that meet the close time criteria of the filter are counted. <note>  <code>startTimeFilter</code> and <code>closeTimeFilter</code> are mutually exclusive. You must specify one of these in a request but not both. </note>:<br/>startTimeFilter:  If specified, only workflow executions that meet the start time criteria of the filter are counted. <note>  <code>startTimeFilter</code> and <code>closeTimeFilter</code> are mutually exclusive. You must specify one of these in a request but not both. </note>:<br/>domain:  The name of the domain containing the workflow executions to count.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>latestDate</i> Specifies the latest start or close date and time to return.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>oldestDate</i> Specifies the oldest start or close date and time to return.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>latestDate</i> Specifies the latest start or close date and time to return.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>oldestDate</i> Specifies the oldest start or close date and time to return.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>workflowId</i> The workflowId to pass of match the criteria of this filter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> Version of the workflow type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i>  Name of the workflow type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tag</i>  Specifies the tag that must be associated with the execution for it to meet the filter criteria. Tags may only contain unicode letters, digits, whitespace, or these symbols: <code>_ . : / = + - @</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i>  The close status that must match the close status of an execution for it to meet the criteria of this filter.<br/>
        :return: truncated:  If set to true, indicates that the actual count was more than the maximum supported by this API and the count returned is the truncated value.<br/>count:  The number of workflow executions.<br/>        """
        pass

    def count_open_workflow_executions(self, startTimeFilter: ExecutionTimeFilter, domain: str, typeFilter: WorkflowTypeFilter=None, tagFilter: TagFilter=None, executionFilter: WorkflowExecutionFilter=None) -> WorkflowExecutionCount:
        """Returns the number of open workflow executions within the given domain that meet the specified filtering criteria. <note> This operation is eventually consistent. The results are best effort and may not exactly reflect recent updates and changes. </note>  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> Constrain the following parameters by using a <code>Condition</code> element with the appropriate keys. <ul> <li>  <code>tagFilter.tag</code>: String constraint. The key is <code>swf:tagFilter.tag</code>. </li> <li>  <code>typeFilter.name</code>: String constraint. The key is <code>swf:typeFilter.name</code>. </li> <li>  <code>typeFilter.version</code>: String constraint. The key is <code>swf:typeFilter.version</code>. </li> </ul> </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param executionFilter:  If specified, only workflow executions matching the <code>WorkflowId</code> in the filter are counted. <note>  <code>executionFilter</code>, <code>typeFilter</code> and <code>tagFilter</code> are mutually exclusive. You can specify at most one of these in a request. </note>:<br/>tagFilter:  If specified, only executions that have a tag that matches the filter are counted. <note>  <code>executionFilter</code>, <code>typeFilter</code> and <code>tagFilter</code> are mutually exclusive. You can specify at most one of these in a request. </note>:<br/>typeFilter:  Specifies the type of the workflow executions to be counted. <note>  <code>executionFilter</code>, <code>typeFilter</code> and <code>tagFilter</code> are mutually exclusive. You can specify at most one of these in a request. </note>:<br/>startTimeFilter:  Specifies the start time criteria that workflow executions must meet in order to be counted.:<br/>domain:  The name of the domain containing the workflow executions to count.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>latestDate</i> Specifies the latest start or close date and time to return.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>oldestDate</i> Specifies the oldest start or close date and time to return.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> Version of the workflow type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i>  Name of the workflow type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tag</i>  Specifies the tag that must be associated with the execution for it to meet the filter criteria. Tags may only contain unicode letters, digits, whitespace, or these symbols: <code>_ . : / = + - @</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>workflowId</i> The workflowId to pass of match the criteria of this filter.<br/>
        :return: truncated:  If set to true, indicates that the actual count was more than the maximum supported by this API and the count returned is the truncated value.<br/>count:  The number of workflow executions.<br/>        """
        pass

    def count_pending_activity_tasks(self, taskList: TaskList, domain: str) -> PendingTaskCount:
        """Returns the estimated number of activity tasks in the specified task list. The count returned is an approximation and isn't guaranteed to be exact. If you specify a task list that no activity task was ever scheduled in then <code>0</code> is returned.  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> Constrain the <code>taskList.name</code> parameter by using a <code>Condition</code> element with the <code>swf:taskList.name</code> key to allow the action to access only certain task lists. </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param taskList:  The name of the task list.:<br/>domain:  The name of the domain that contains the task list.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the task list.<br/>
        :return: truncated:  If set to true, indicates that the actual count was more than the maximum supported by this API and the count returned is the truncated value.<br/>count:  The number of tasks in the task list.<br/>        """
        pass

    def count_pending_decision_tasks(self, taskList: TaskList, domain: str) -> PendingTaskCount:
        """Returns the estimated number of decision tasks in the specified task list. The count returned is an approximation and isn't guaranteed to be exact. If you specify a task list that no decision task was ever scheduled in then <code>0</code> is returned.  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> Constrain the <code>taskList.name</code> parameter by using a <code>Condition</code> element with the <code>swf:taskList.name</code> key to allow the action to access only certain task lists. </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param taskList:  The name of the task list.:<br/>domain:  The name of the domain that contains the task list.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the task list.<br/>
        :return: truncated:  If set to true, indicates that the actual count was more than the maximum supported by this API and the count returned is the truncated value.<br/>count:  The number of tasks in the task list.<br/>        """
        pass

    def deprecate_activity_type(self, activityType: ActivityType, domain: str):
        """Deprecates the specified <i>activity type</i>. After an activity type has been deprecated, you cannot create new tasks of that activity type. Tasks of this type that were scheduled before the type was deprecated continue to run. <note> This operation is eventually consistent. The results are best effort and may not exactly reflect recent updates and changes. </note>  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> Constrain the following parameters by using a <code>Condition</code> element with the appropriate keys. <ul> <li>  <code>activityType.name</code>: String constraint. The key is <code>swf:activityType.name</code>. </li> <li>  <code>activityType.version</code>: String constraint. The key is <code>swf:activityType.version</code>. </li> </ul> </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param activityType:  The activity type to deprecate.:<br/>domain:  The name of the domain in which the activity type is registered.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of this activity. <note> The combination of activity type name and version must be unique with in a domain. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of this activity. <note> The combination of activity type name and version must be unique within a domain. </note><br/>
        :return:         """
        pass

    def deprecate_domain(self, name: str):
        """Deprecates the specified domain. After a domain has been deprecated it cannot be used to create new workflow executions or register new types. However, you can still use visibility actions on this domain. Deprecating a domain also deprecates all activity and workflow types registered in the domain. Executions that were started before the domain was deprecated continues to run. <note> This operation is eventually consistent. The results are best effort and may not exactly reflect recent updates and changes. </note>  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> You cannot use an IAM policy to constrain this action's parameters. </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param name:  The name of the domain to deprecate.<br/>
        :return:         """
        pass

    def deprecate_workflow_type(self, workflowType: WorkflowType, domain: str):
        """Deprecates the specified <i>workflow type</i>. After a workflow type has been deprecated, you cannot create new executions of that type. Executions that were started before the type was deprecated continues to run. A deprecated workflow type may still be used when calling visibility actions. <note> This operation is eventually consistent. The results are best effort and may not exactly reflect recent updates and changes. </note>  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> Constrain the following parameters by using a <code>Condition</code> element with the appropriate keys. <ul> <li>  <code>workflowType.name</code>: String constraint. The key is <code>swf:workflowType.name</code>. </li> <li>  <code>workflowType.version</code>: String constraint. The key is <code>swf:workflowType.version</code>. </li> </ul> </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param workflowType:  The workflow type to deprecate.:<br/>domain:  The name of the domain in which the workflow type is registered.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i>  The version of the workflow type. <note> The combination of workflow type name and version must be unique with in a domain. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i>  The name of the workflow type. <note> The combination of workflow type name and version must be unique with in a domain. </note><br/>
        :return:         """
        pass

    def describe_activity_type(self, activityType: ActivityType, domain: str) -> ActivityTypeDetail:
        """Returns information about the specified activity type. This includes configuration settings provided when the type was registered and other general information about the type.  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> Constrain the following parameters by using a <code>Condition</code> element with the appropriate keys. <ul> <li>  <code>activityType.name</code>: String constraint. The key is <code>swf:activityType.name</code>. </li> <li>  <code>activityType.version</code>: String constraint. The key is <code>swf:activityType.version</code>. </li> </ul> </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param activityType:  The activity type to get information about. Activity types are identified by the <code>name</code> and <code>version</code> that were supplied when the activity was registered.:<br/>domain:  The name of the domain in which the activity type is registered.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of this activity. <note> The combination of activity type name and version must be unique with in a domain. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of this activity. <note> The combination of activity type name and version must be unique within a domain. </note><br/>
        :return: configuration:  The configuration settings registered with the activity type.:<br/>typeInfo:  General information about the activity type. The status of activity type (returned in the ActivityTypeInfo structure) can be one of the following. <ul> <li>  <code>REGISTERED</code> – The type is registered and available. Workers supporting this type should be running.  </li> <li>  <code>DEPRECATED</code> – The type was deprecated using <a>DeprecateActivityType</a>, but is still in use. You should keep workers supporting this type running. You cannot create new tasks of this type.  </li> </ul>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deprecationDate</i> If DEPRECATED, the date and time <a>DeprecateActivityType</a> was called.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDate</i> The date and time this activity type was created through <a>RegisterActivityType</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The description of the activity type provided in <a>RegisterActivityType</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the activity type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>activityType</i> The <a>ActivityType</a> type structure representing the activity type.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>activityType</b>: The <a>ActivityType</a> type structure representing the activity type.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of this activity. <note> The combination of activity type name and version must be unique with in a domain. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of this activity. <note> The combination of activity type name and version must be unique within a domain. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultTaskScheduleToCloseTimeout</i>  The default maximum duration, specified when registering the activity type, for tasks of this activity type. You can override this default when scheduling a task through the <code>ScheduleActivityTask</code> <a>Decision</a>. The duration is specified in seconds, an integer greater than or equal to <code>0</code>. You can use <code>NONE</code> to specify unlimited duration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultTaskScheduleToStartTimeout</i>  The default maximum duration, specified when registering the activity type, that a task of an activity type can wait before being assigned to a worker. You can override this default when scheduling a task through the <code>ScheduleActivityTask</code> <a>Decision</a>. The duration is specified in seconds, an integer greater than or equal to <code>0</code>. You can use <code>NONE</code> to specify unlimited duration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultTaskPriority</i>  The default task priority for tasks of this activity type, specified at registration. If not set, then <code>0</code> is used as the default priority. This default can be overridden when scheduling an activity task. Valid values are integers that range from Java's <code>Integer.MIN_VALUE</code> (-2147483648) to <code>Integer.MAX_VALUE</code> (2147483647). Higher numbers indicate higher priority. For more information about setting task priority, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/programming-priority.html">Setting Task Priority</a> in the <i>Amazon SWF Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultTaskList</i>  The default task list specified for this activity type at registration. This default is used if a task list isn't provided when a task is scheduled through the <code>ScheduleActivityTask</code> <a>Decision</a>. You can override the default registered task list when scheduling a task through the <code>ScheduleActivityTask</code> <a>Decision</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>defaultTaskList</b>:  The default task list specified for this activity type at registration. This default is used if a task list isn't provided when a task is scheduled through the <code>ScheduleActivityTask</code> <a>Decision</a>. You can override the default registered task list when scheduling a task through the <code>ScheduleActivityTask</code> <a>Decision</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultTaskHeartbeatTimeout</i>  The default maximum time, in seconds, before which a worker processing a task must report progress by calling <a>RecordActivityTaskHeartbeat</a>. You can specify this value only when <i>registering</i> an activity type. The registered default value can be overridden when you schedule a task through the <code>ScheduleActivityTask</code> <a>Decision</a>. If the activity worker subsequently attempts to record a heartbeat or returns a result, the activity worker receives an <code>UnknownResource</code> fault. In this case, Amazon SWF no longer considers the activity task to be valid; the activity worker should clean up the activity task. The duration is specified in seconds, an integer greater than or equal to <code>0</code>. You can use <code>NONE</code> to specify unlimited duration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultTaskStartToCloseTimeout</i>  The default maximum duration for tasks of an activity type specified when registering the activity type. You can override this default when scheduling a task through the <code>ScheduleActivityTask</code> <a>Decision</a>. The duration is specified in seconds, an integer greater than or equal to <code>0</code>. You can use <code>NONE</code> to specify unlimited duration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the task list.<br/>        """
        pass

    def describe_domain(self, name: str) -> DomainDetail:
        """Returns information about the specified domain, including description and status.  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> You cannot use an IAM policy to constrain this action's parameters. </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param name:  The name of the domain to describe.<br/>
        :return: configuration:  The domain configuration. Currently, this includes only the domain's retention period.:<br/>domainInfo:  The basic information about a domain, such as its name, status, and description.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>arn</i> The ARN of the domain.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The description of the domain provided through <a>RegisterDomain</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The status of the domain: <ul> <li>  <code>REGISTERED</code> – The domain is properly registered and available. You can use this domain for registering types and creating new workflow executions.  </li> <li>  <code>DEPRECATED</code> – The domain was deprecated using <a>DeprecateDomain</a>, but is still in use. You should not create new workflow executions in this domain.  </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the domain. This name is unique within the account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>workflowExecutionRetentionPeriodInDays</i> The retention period for workflow executions in this domain.<br/>        """
        pass

    def describe_workflow_execution(self, execution: WorkflowExecution, domain: str) -> WorkflowExecutionDetail:
        """Returns information about the specified workflow execution including its type and some statistics. <note> This operation is eventually consistent. The results are best effort and may not exactly reflect recent updates and changes. </note>  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> You cannot use an IAM policy to constrain this action's parameters. </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param execution:  The workflow execution to describe.:<br/>domain:  The name of the domain containing the workflow execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>runId</i> A system-generated unique identifier for the workflow execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>workflowId</i> The user defined identifier associated with the workflow execution.<br/>
        :return: latestExecutionContext:  The latest executionContext provided by the decider for this workflow execution. A decider can provide an executionContext (a free-form string) when closing a decision task using <a>RespondDecisionTaskCompleted</a>.<br/>latestActivityTaskTimestamp:  The time when the last activity task was scheduled for this workflow execution. You can use this information to determine if the workflow has not made progress for an unusually long period of time and might require a corrective action.<br/>openCounts:  The number of tasks for this workflow execution. This includes open and closed tasks of all types.:<br/>executionConfiguration:  The configuration settings for this workflow execution including timeout values, tasklist etc.:<br/>executionInfo:  Information about the workflow execution.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>cancelRequested</i> Set to true if a cancellation is requested for this workflow execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tagList</i> The list of tags associated with the workflow execution. Tags can be used to identify and list workflow executions of interest through the visibility APIs. A workflow execution can have a maximum of 5 tags.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>parent</i> If this workflow execution is a child of another execution then contains the workflow execution that started this execution.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>parent</b>: If this workflow execution is a child of another execution then contains the workflow execution that started this execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>closeStatus</i> If the execution status is closed then this specifies how the execution was closed: <ul> <li>  <code>COMPLETED</code> – the execution was successfully completed. </li> <li>  <code>CANCELED</code> – the execution was canceled.Cancellation allows the implementation to gracefully clean up before the execution is closed. </li> <li>  <code>TERMINATED</code> – the execution was force terminated. </li> <li>  <code>FAILED</code> – the execution failed to complete. </li> <li>  <code>TIMED_OUT</code> – the execution did not complete in the alloted time and was automatically timed out. </li> <li>  <code>CONTINUED_AS_NEW</code> – the execution is logically continued. This means the current execution was completed and a new execution was started to carry on the workflow. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>executionStatus</i> The current status of the execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>closeTimestamp</i> The time when the workflow execution was closed. Set only if the execution status is CLOSED.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>startTimestamp</i> The time when the execution was started.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>workflowType</i> The type of the workflow execution.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>workflowType</b>: The type of the workflow execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>execution</i> The workflow execution this information is about.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>execution</b>: The workflow execution this information is about.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>runId</i> A system-generated unique identifier for the workflow execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>workflowId</i> The user defined identifier associated with the workflow execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i>  The version of the workflow type. <note> The combination of workflow type name and version must be unique with in a domain. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i>  The name of the workflow type. <note> The combination of workflow type name and version must be unique with in a domain. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>runId</i> A system-generated unique identifier for the workflow execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>workflowId</i> The user defined identifier associated with the workflow execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>lambdaRole</i> The IAM role attached to the child workflow execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>childPolicy</i> The policy to use for the child workflow executions if this workflow execution is terminated, by calling the <a>TerminateWorkflowExecution</a> action explicitly or due to an expired timeout. The supported child policies are: <ul> <li>  <code>TERMINATE</code> – The child executions are terminated. </li> <li>  <code>REQUEST_CANCEL</code> – A request to cancel is attempted for each child execution by recording a <code>WorkflowExecutionCancelRequested</code> event in its history. It is up to the decider to take appropriate actions when it receives an execution history with this event. </li> <li>  <code>ABANDON</code> – No action is taken. The child executions continue to run. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>taskPriority</i> The priority assigned to decision tasks for this workflow execution. Valid values are integers that range from Java's <code>Integer.MIN_VALUE</code> (-2147483648) to <code>Integer.MAX_VALUE</code> (2147483647). Higher numbers indicate higher priority. For more information about setting task priority, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/programming-priority.html">Setting Task Priority</a> in the <i>Amazon SWF Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>taskList</i> The task list used for the decision tasks generated for this workflow execution.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>taskList</b>: The task list used for the decision tasks generated for this workflow execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>executionStartToCloseTimeout</i> The total duration for this workflow execution. The duration is specified in seconds, an integer greater than or equal to <code>0</code>. You can use <code>NONE</code> to specify unlimited duration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>taskStartToCloseTimeout</i> The maximum duration allowed for decision tasks for this workflow execution. The duration is specified in seconds, an integer greater than or equal to <code>0</code>. You can use <code>NONE</code> to specify unlimited duration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the task list.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>openLambdaFunctions</i> The count of Lambda tasks whose status is <code>OPEN</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>openChildWorkflowExecutions</i> The count of child workflow executions whose status is <code>OPEN</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>openTimers</i> The count of timers started by this workflow execution that have not fired yet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>openDecisionTasks</i> The count of decision tasks whose status is OPEN. A workflow execution can have at most one open decision task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>openActivityTasks</i> The count of activity tasks whose status is <code>OPEN</code>.<br/>        """
        pass

    def describe_workflow_type(self, workflowType: WorkflowType, domain: str) -> WorkflowTypeDetail:
        """Returns information about the specified <i>workflow type</i>. This includes configuration settings specified when the type was registered and other information such as creation date, current status, etc.  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> Constrain the following parameters by using a <code>Condition</code> element with the appropriate keys. <ul> <li>  <code>workflowType.name</code>: String constraint. The key is <code>swf:workflowType.name</code>. </li> <li>  <code>workflowType.version</code>: String constraint. The key is <code>swf:workflowType.version</code>. </li> </ul> </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param workflowType:  The workflow type to describe.:<br/>domain:  The name of the domain in which this workflow type is registered.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i>  The version of the workflow type. <note> The combination of workflow type name and version must be unique with in a domain. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i>  The name of the workflow type. <note> The combination of workflow type name and version must be unique with in a domain. </note><br/>
        :return: configuration:  Configuration settings of the workflow type registered through <a>RegisterWorkflowType</a> :<br/>typeInfo:  General information about the workflow type. The status of the workflow type (returned in the WorkflowTypeInfo structure) can be one of the following. <ul> <li>  <code>REGISTERED</code> – The type is registered and available. Workers supporting this type should be running. </li> <li>  <code>DEPRECATED</code> – The type was deprecated using <a>DeprecateWorkflowType</a>, but is still in use. You should keep workers supporting this type running. You cannot create new workflow executions of this type. </li> </ul>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>deprecationDate</i> If the type is in deprecated state, then it is set to the date when the type was deprecated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>creationDate</i> The date when this type was registered.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>description</i> The description of the type registered through <a>RegisterWorkflowType</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i> The current status of the workflow type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>workflowType</i> The workflow type this information is about.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>workflowType</b>: The workflow type this information is about.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i>  The version of the workflow type. <note> The combination of workflow type name and version must be unique with in a domain. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i>  The name of the workflow type. <note> The combination of workflow type name and version must be unique with in a domain. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultLambdaRole</i> The default IAM role attached to this workflow type. <note> Executions of this workflow type need IAM roles to invoke Lambda functions. If you don't specify an IAM role when starting this workflow type, the default Lambda role is attached to the execution. For more information, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/lambda-task.html">https://docs.aws.amazon.com/amazonswf/latest/developerguide/lambda-task.html</a> in the <i>Amazon SWF Developer Guide</i>. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultChildPolicy</i>  The default policy to use for the child workflow executions when a workflow execution of this type is terminated, by calling the <a>TerminateWorkflowExecution</a> action explicitly or due to an expired timeout. This default can be overridden when starting a workflow execution using the <a>StartWorkflowExecution</a> action or the <code>StartChildWorkflowExecution</code> <a>Decision</a>. The supported child policies are: <ul> <li>  <code>TERMINATE</code> – The child executions are terminated. </li> <li>  <code>REQUEST_CANCEL</code> – A request to cancel is attempted for each child execution by recording a <code>WorkflowExecutionCancelRequested</code> event in its history. It is up to the decider to take appropriate actions when it receives an execution history with this event. </li> <li>  <code>ABANDON</code> – No action is taken. The child executions continue to run. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultTaskPriority</i>  The default task priority, specified when registering the workflow type, for all decision tasks of this workflow type. This default can be overridden when starting a workflow execution using the <a>StartWorkflowExecution</a> action or the <code>StartChildWorkflowExecution</code> decision. Valid values are integers that range from Java's <code>Integer.MIN_VALUE</code> (-2147483648) to <code>Integer.MAX_VALUE</code> (2147483647). Higher numbers indicate higher priority. For more information about setting task priority, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/programming-priority.html">Setting Task Priority</a> in the <i>Amazon SWF Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultTaskList</i>  The default task list, specified when registering the workflow type, for decisions tasks scheduled for workflow executions of this type. This default can be overridden when starting a workflow execution using the <a>StartWorkflowExecution</a> action or the <code>StartChildWorkflowExecution</code> <a>Decision</a>.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>defaultTaskList</b>:  The default task list, specified when registering the workflow type, for decisions tasks scheduled for workflow executions of this type. This default can be overridden when starting a workflow execution using the <a>StartWorkflowExecution</a> action or the <code>StartChildWorkflowExecution</code> <a>Decision</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultExecutionStartToCloseTimeout</i>  The default maximum duration, specified when registering the workflow type, for executions of this workflow type. This default can be overridden when starting a workflow execution using the <a>StartWorkflowExecution</a> action or the <code>StartChildWorkflowExecution</code> <a>Decision</a>. The duration is specified in seconds, an integer greater than or equal to <code>0</code>. You can use <code>NONE</code> to specify unlimited duration.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>defaultTaskStartToCloseTimeout</i>  The default maximum duration, specified when registering the workflow type, that a decision task for executions of this workflow type might take before returning completion or failure. If the task doesn'tdo close in the specified time then the task is automatically timed out and rescheduled. If the decider eventually reports a completion or failure, it is ignored. This default can be overridden when starting a workflow execution using the <a>StartWorkflowExecution</a> action or the <code>StartChildWorkflowExecution</code> <a>Decision</a>. The duration is specified in seconds, an integer greater than or equal to <code>0</code>. You can use <code>NONE</code> to specify unlimited duration.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the task list.<br/>        """
        pass

    def get_workflow_execution_history(self, execution: WorkflowExecution, domain: str, nextPageToken: str=None, maximumPageSize: int=None, reverseOrder: ReverseOrder=None) -> History:
        """Returns the history of the specified workflow execution. The results may be split into multiple pages. To retrieve subsequent pages, make the call again using the <code>nextPageToken</code> returned by the initial call. <note> This operation is eventually consistent. The results are best effort and may not exactly reflect recent updates and changes. </note>  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> You cannot use an IAM policy to constrain this action's parameters. </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param reverseOrder:  When set to <code>true</code>, returns the events in reverse order. By default the results are returned in ascending order of the <code>eventTimeStamp</code> of the events.<br/>maximumPageSize:  The maximum number of results that are returned per call. Use <code>nextPageToken</code> to obtain further pages of results. <br/>nextPageToken:  If <code>NextPageToken</code> is returned there are more results available. The value of <code>NextPageToken</code> is a unique pagination token for each page. Make the call again using the returned token to retrieve the next page. Keep all other arguments unchanged. Each pagination token expires after 60 seconds. Using an expired pagination token will return a <code>400</code> error: "<code>Specified token has exceeded its maximum lifetime</code>".  The configured <code>maximumPageSize</code> determines how many results can be returned in a single call. <br/>execution:  Specifies the workflow execution for which to return the history.:<br/>domain:  The name of the domain containing the workflow execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>runId</i> A system-generated unique identifier for the workflow execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>workflowId</i> The user defined identifier associated with the workflow execution.<br/>
        :return: nextPageToken:  If a <code>NextPageToken</code> was returned by a previous call, there are more results available. To retrieve the next page of results, make the call again using the returned token in <code>nextPageToken</code>. Keep all other arguments unchanged. The configured <code>maximumPageSize</code> determines how many results can be returned in a single call.<br/>events:  The list of history events.<br/>        """
        pass

    def list_activity_types(self, registrationStatus: str, domain: str, name: str=None, nextPageToken: str=None, maximumPageSize: int=None, reverseOrder: ReverseOrder=None) -> ActivityTypeInfos:
        """Returns information about all activities registered in the specified domain that match the specified name and registration status. The result includes information like creation date, current status of the activity, etc. The results may be split into multiple pages. To retrieve subsequent pages, make the call again using the <code>nextPageToken</code> returned by the initial call.  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> You cannot use an IAM policy to constrain this action's parameters. </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param reverseOrder:  When set to <code>true</code>, returns the results in reverse order. By default, the results are returned in ascending alphabetical order by <code>name</code> of the activity types.<br/>maximumPageSize:  The maximum number of results that are returned per call. Use <code>nextPageToken</code> to obtain further pages of results. <br/>nextPageToken:  If <code>NextPageToken</code> is returned there are more results available. The value of <code>NextPageToken</code> is a unique pagination token for each page. Make the call again using the returned token to retrieve the next page. Keep all other arguments unchanged. Each pagination token expires after 60 seconds. Using an expired pagination token will return a <code>400</code> error: "<code>Specified token has exceeded its maximum lifetime</code>".  The configured <code>maximumPageSize</code> determines how many results can be returned in a single call. <br/>registrationStatus:  Specifies the registration status of the activity types to list.<br/>name:  If specified, only lists the activity types that have this name.<br/>domain:  The name of the domain in which the activity types have been registered.<br/>
        :return: nextPageToken:  If a <code>NextPageToken</code> was returned by a previous call, there are more results available. To retrieve the next page of results, make the call again using the returned token in <code>nextPageToken</code>. Keep all other arguments unchanged. The configured <code>maximumPageSize</code> determines how many results can be returned in a single call.<br/>typeInfos:  List of activity type information.<br/>        """
        pass

    def list_closed_workflow_executions(self, domain: str, startTimeFilter: ExecutionTimeFilter=None, closeTimeFilter: ExecutionTimeFilter=None, executionFilter: WorkflowExecutionFilter=None, closeStatusFilter: CloseStatusFilter=None, typeFilter: WorkflowTypeFilter=None, tagFilter: TagFilter=None, nextPageToken: str=None, maximumPageSize: int=None, reverseOrder: ReverseOrder=None) -> WorkflowExecutionInfos:
        """Returns a list of closed workflow executions in the specified domain that meet the filtering criteria. The results may be split into multiple pages. To retrieve subsequent pages, make the call again using the nextPageToken returned by the initial call. <note> This operation is eventually consistent. The results are best effort and may not exactly reflect recent updates and changes. </note>  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> Constrain the following parameters by using a <code>Condition</code> element with the appropriate keys. <ul> <li>  <code>tagFilter.tag</code>: String constraint. The key is <code>swf:tagFilter.tag</code>. </li> <li>  <code>typeFilter.name</code>: String constraint. The key is <code>swf:typeFilter.name</code>. </li> <li>  <code>typeFilter.version</code>: String constraint. The key is <code>swf:typeFilter.version</code>. </li> </ul> </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param reverseOrder:  When set to <code>true</code>, returns the results in reverse order. By default the results are returned in descending order of the start or the close time of the executions.<br/>maximumPageSize:  The maximum number of results that are returned per call. Use <code>nextPageToken</code> to obtain further pages of results. <br/>nextPageToken:  If <code>NextPageToken</code> is returned there are more results available. The value of <code>NextPageToken</code> is a unique pagination token for each page. Make the call again using the returned token to retrieve the next page. Keep all other arguments unchanged. Each pagination token expires after 60 seconds. Using an expired pagination token will return a <code>400</code> error: "<code>Specified token has exceeded its maximum lifetime</code>".  The configured <code>maximumPageSize</code> determines how many results can be returned in a single call. <br/>tagFilter:  If specified, only executions that have the matching tag are listed. <note>  <code>closeStatusFilter</code>, <code>executionFilter</code>, <code>typeFilter</code> and <code>tagFilter</code> are mutually exclusive. You can specify at most one of these in a request. </note>:<br/>typeFilter:  If specified, only executions of the type specified in the filter are returned. <note>  <code>closeStatusFilter</code>, <code>executionFilter</code>, <code>typeFilter</code> and <code>tagFilter</code> are mutually exclusive. You can specify at most one of these in a request. </note>:<br/>closeStatusFilter:  If specified, only workflow executions that match this <i>close status</i> are listed. For example, if TERMINATED is specified, then only TERMINATED workflow executions are listed. <note>  <code>closeStatusFilter</code>, <code>executionFilter</code>, <code>typeFilter</code> and <code>tagFilter</code> are mutually exclusive. You can specify at most one of these in a request. </note>:<br/>executionFilter:  If specified, only workflow executions matching the workflow ID specified in the filter are returned. <note>  <code>closeStatusFilter</code>, <code>executionFilter</code>, <code>typeFilter</code> and <code>tagFilter</code> are mutually exclusive. You can specify at most one of these in a request. </note>:<br/>closeTimeFilter:  If specified, the workflow executions are included in the returned results based on whether their close times are within the range specified by this filter. Also, if this parameter is specified, the returned results are ordered by their close times. <note>  <code>startTimeFilter</code> and <code>closeTimeFilter</code> are mutually exclusive. You must specify one of these in a request but not both. </note>:<br/>startTimeFilter:  If specified, the workflow executions are included in the returned results based on whether their start times are within the range specified by this filter. Also, if this parameter is specified, the returned results are ordered by their start times. <note>  <code>startTimeFilter</code> and <code>closeTimeFilter</code> are mutually exclusive. You must specify one of these in a request but not both. </note>:<br/>domain:  The name of the domain that contains the workflow executions to list.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>latestDate</i> Specifies the latest start or close date and time to return.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>oldestDate</i> Specifies the oldest start or close date and time to return.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>latestDate</i> Specifies the latest start or close date and time to return.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>oldestDate</i> Specifies the oldest start or close date and time to return.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>workflowId</i> The workflowId to pass of match the criteria of this filter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>status</i>  The close status that must match the close status of an execution for it to meet the criteria of this filter.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> Version of the workflow type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i>  Name of the workflow type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tag</i>  Specifies the tag that must be associated with the execution for it to meet the filter criteria. Tags may only contain unicode letters, digits, whitespace, or these symbols: <code>_ . : / = + - @</code>.<br/>
        :return: nextPageToken:  If a <code>NextPageToken</code> was returned by a previous call, there are more results available. To retrieve the next page of results, make the call again using the returned token in <code>nextPageToken</code>. Keep all other arguments unchanged. The configured <code>maximumPageSize</code> determines how many results can be returned in a single call.<br/>executionInfos:  The list of workflow information structures.<br/>        """
        pass

    def list_domains(self, registrationStatus: str, nextPageToken: str=None, maximumPageSize: int=None, reverseOrder: ReverseOrder=None) -> DomainInfos:
        """Returns the list of domains registered in the account. The results may be split into multiple pages. To retrieve subsequent pages, make the call again using the nextPageToken returned by the initial call. <note> This operation is eventually consistent. The results are best effort and may not exactly reflect recent updates and changes. </note>  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. The element must be set to <code>arn:aws:swf::AccountID:domain/*</code>, where <i>AccountID</i> is the account ID, with no dashes. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> You cannot use an IAM policy to constrain this action's parameters. </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param reverseOrder:  When set to <code>true</code>, returns the results in reverse order. By default, the results are returned in ascending alphabetical order by <code>name</code> of the domains.<br/>maximumPageSize:  The maximum number of results that are returned per call. Use <code>nextPageToken</code> to obtain further pages of results. <br/>registrationStatus:  Specifies the registration status of the domains to list.<br/>nextPageToken:  If <code>NextPageToken</code> is returned there are more results available. The value of <code>NextPageToken</code> is a unique pagination token for each page. Make the call again using the returned token to retrieve the next page. Keep all other arguments unchanged. Each pagination token expires after 60 seconds. Using an expired pagination token will return a <code>400</code> error: "<code>Specified token has exceeded its maximum lifetime</code>".  The configured <code>maximumPageSize</code> determines how many results can be returned in a single call. <br/>
        :return: nextPageToken:  If a <code>NextPageToken</code> was returned by a previous call, there are more results available. To retrieve the next page of results, make the call again using the returned token in <code>nextPageToken</code>. Keep all other arguments unchanged. The configured <code>maximumPageSize</code> determines how many results can be returned in a single call.<br/>domainInfos:  A list of DomainInfo structures.<br/>        """
        pass

    def list_open_workflow_executions(self, startTimeFilter: ExecutionTimeFilter, domain: str, typeFilter: WorkflowTypeFilter=None, tagFilter: TagFilter=None, nextPageToken: str=None, maximumPageSize: int=None, reverseOrder: ReverseOrder=None, executionFilter: WorkflowExecutionFilter=None) -> WorkflowExecutionInfos:
        """Returns a list of open workflow executions in the specified domain that meet the filtering criteria. The results may be split into multiple pages. To retrieve subsequent pages, make the call again using the nextPageToken returned by the initial call. <note> This operation is eventually consistent. The results are best effort and may not exactly reflect recent updates and changes. </note>  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> Constrain the following parameters by using a <code>Condition</code> element with the appropriate keys. <ul> <li>  <code>tagFilter.tag</code>: String constraint. The key is <code>swf:tagFilter.tag</code>. </li> <li>  <code>typeFilter.name</code>: String constraint. The key is <code>swf:typeFilter.name</code>. </li> <li>  <code>typeFilter.version</code>: String constraint. The key is <code>swf:typeFilter.version</code>. </li> </ul> </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param executionFilter:  If specified, only workflow executions matching the workflow ID specified in the filter are returned. <note>  <code>executionFilter</code>, <code>typeFilter</code> and <code>tagFilter</code> are mutually exclusive. You can specify at most one of these in a request. </note>:<br/>reverseOrder:  When set to <code>true</code>, returns the results in reverse order. By default the results are returned in descending order of the start time of the executions.<br/>maximumPageSize:  The maximum number of results that are returned per call. Use <code>nextPageToken</code> to obtain further pages of results. <br/>nextPageToken:  If <code>NextPageToken</code> is returned there are more results available. The value of <code>NextPageToken</code> is a unique pagination token for each page. Make the call again using the returned token to retrieve the next page. Keep all other arguments unchanged. Each pagination token expires after 60 seconds. Using an expired pagination token will return a <code>400</code> error: "<code>Specified token has exceeded its maximum lifetime</code>".  The configured <code>maximumPageSize</code> determines how many results can be returned in a single call. <br/>tagFilter:  If specified, only executions that have the matching tag are listed. <note>  <code>executionFilter</code>, <code>typeFilter</code> and <code>tagFilter</code> are mutually exclusive. You can specify at most one of these in a request. </note>:<br/>typeFilter:  If specified, only executions of the type specified in the filter are returned. <note>  <code>executionFilter</code>, <code>typeFilter</code> and <code>tagFilter</code> are mutually exclusive. You can specify at most one of these in a request. </note>:<br/>startTimeFilter:  Workflow executions are included in the returned results based on whether their start times are within the range specified by this filter.:<br/>domain:  The name of the domain that contains the workflow executions to list.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>latestDate</i> Specifies the latest start or close date and time to return.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>oldestDate</i> Specifies the oldest start or close date and time to return.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> Version of the workflow type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i>  Name of the workflow type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>tag</i>  Specifies the tag that must be associated with the execution for it to meet the filter criteria. Tags may only contain unicode letters, digits, whitespace, or these symbols: <code>_ . : / = + - @</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>workflowId</i> The workflowId to pass of match the criteria of this filter.<br/>
        :return: nextPageToken:  If a <code>NextPageToken</code> was returned by a previous call, there are more results available. To retrieve the next page of results, make the call again using the returned token in <code>nextPageToken</code>. Keep all other arguments unchanged. The configured <code>maximumPageSize</code> determines how many results can be returned in a single call.<br/>executionInfos:  The list of workflow information structures.<br/>        """
        pass

    def list_tags_for_resource(self, resourceArn: str) -> ListTagsForResourceOutput:
        """List tags for a given domain.

        :param self:
        :param resourceArn:  The Amazon Resource Name (ARN) for the Amazon SWF domain.<br/>
        :return: tags:  An array of tags associated with the domain.<br/>        """
        pass

    def list_workflow_types(self, registrationStatus: str, domain: str, name: str=None, nextPageToken: str=None, maximumPageSize: int=None, reverseOrder: ReverseOrder=None) -> WorkflowTypeInfos:
        """Returns information about workflow types in the specified domain. The results may be split into multiple pages that can be retrieved by making the call repeatedly.  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> You cannot use an IAM policy to constrain this action's parameters. </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param reverseOrder:  When set to <code>true</code>, returns the results in reverse order. By default the results are returned in ascending alphabetical order of the <code>name</code> of the workflow types.<br/>maximumPageSize:  The maximum number of results that are returned per call. Use <code>nextPageToken</code> to obtain further pages of results. <br/>nextPageToken:  If <code>NextPageToken</code> is returned there are more results available. The value of <code>NextPageToken</code> is a unique pagination token for each page. Make the call again using the returned token to retrieve the next page. Keep all other arguments unchanged. Each pagination token expires after 60 seconds. Using an expired pagination token will return a <code>400</code> error: "<code>Specified token has exceeded its maximum lifetime</code>".  The configured <code>maximumPageSize</code> determines how many results can be returned in a single call. <br/>registrationStatus:  Specifies the registration status of the workflow types to list.<br/>name:  If specified, lists the workflow type with this name.<br/>domain:  The name of the domain in which the workflow types have been registered.<br/>
        :return: nextPageToken:  If a <code>NextPageToken</code> was returned by a previous call, there are more results available. To retrieve the next page of results, make the call again using the returned token in <code>nextPageToken</code>. Keep all other arguments unchanged. The configured <code>maximumPageSize</code> determines how many results can be returned in a single call.<br/>typeInfos:  The list of workflow type information.<br/>        """
        pass

    def poll_for_activity_task(self, taskList: TaskList, domain: str, identity: str=None) -> ActivityTask:
        """Used by workers to get an <a>ActivityTask</a> from the specified activity <code>taskList</code>. This initiates a long poll, where the service holds the HTTP connection open and responds as soon as a task becomes available. The maximum time the service holds on to the request before responding is 60 seconds. If no task is available within 60 seconds, the poll returns an empty result. An empty result, in this context, means that an ActivityTask is returned, but that the value of taskToken is an empty string. If a task is returned, the worker should use its type to identify and process it correctly. <important> Workers should set their client side socket timeout to at least 70 seconds (10 seconds higher than the maximum time service may hold the poll request). </important>  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> Constrain the <code>taskList.name</code> parameter by using a <code>Condition</code> element with the <code>swf:taskList.name</code> key to allow the action to access only certain task lists. </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param identity:  Identity of the worker making the request, recorded in the <code>ActivityTaskStarted</code> event in the workflow history. This enables diagnostic tracing when problems arise. The form of this identity is user defined.<br/>taskList:  Specifies the task list to poll for activity tasks. The specified string must not start or end with whitespace. It must not contain a <code>:</code> (colon), <code>/</code> (slash), <code>|</code> (vertical bar), or any control characters (<code>\u0000-\u001f</code> | <code>\u007f-\u009f</code>). Also, it must not <i>be</i> the literal string <code>arn</code>.:<br/>domain:  The name of the domain that contains the task lists being polled.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the task list.<br/>
        :return: input:  The inputs provided when the activity task was scheduled. The form of the input is user defined and should be meaningful to the activity implementation.<br/>activityType:  The type of this activity task.:<br/>workflowExecution:  The workflow execution that started this activity task.:<br/>startedEventId:  The ID of the <code>ActivityTaskStarted</code> event recorded in the history.<br/>activityId:  The unique ID of the task.<br/>taskToken:  The opaque string used as a handle on the task. This token is used by workers to communicate progress and response information back to the system about the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>runId</i> A system-generated unique identifier for the workflow execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>workflowId</i> The user defined identifier associated with the workflow execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of this activity. <note> The combination of activity type name and version must be unique with in a domain. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of this activity. <note> The combination of activity type name and version must be unique within a domain. </note><br/>        """
        pass

    def poll_for_decision_task(self, taskList: TaskList, domain: str, identity: str=None, nextPageToken: str=None, maximumPageSize: int=None, reverseOrder: ReverseOrder=None) -> DecisionTask:
        """Used by deciders to get a <a>DecisionTask</a> from the specified decision <code>taskList</code>. A decision task may be returned for any open workflow execution that is using the specified task list. The task includes a paginated view of the history of the workflow execution. The decider should use the workflow type and the history to determine how to properly handle the task. This action initiates a long poll, where the service holds the HTTP connection open and responds as soon a task becomes available. If no decision task is available in the specified task list before the timeout of 60 seconds expires, an empty result is returned. An empty result, in this context, means that a DecisionTask is returned, but that the value of taskToken is an empty string. <important> Deciders should set their client side socket timeout to at least 70 seconds (10 seconds higher than the timeout). </important> <important> Because the number of workflow history events for a single workflow execution might be very large, the result returned might be split up across a number of pages. To retrieve subsequent pages, make additional calls to <code>PollForDecisionTask</code> using the <code>nextPageToken</code> returned by the initial call. Note that you do <i>not</i> call <code>GetWorkflowExecutionHistory</code> with this <code>nextPageToken</code>. Instead, call <code>PollForDecisionTask</code> again. </important>  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> Constrain the <code>taskList.name</code> parameter by using a <code>Condition</code> element with the <code>swf:taskList.name</code> key to allow the action to access only certain task lists. </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param reverseOrder:  When set to <code>true</code>, returns the events in reverse order. By default the results are returned in ascending order of the <code>eventTimestamp</code> of the events.<br/>maximumPageSize:  The maximum number of results that are returned per call. Use <code>nextPageToken</code> to obtain further pages of results.  This is an upper limit only; the actual number of results returned per call may be fewer than the specified maximum.<br/>nextPageToken:  If <code>NextPageToken</code> is returned there are more results available. The value of <code>NextPageToken</code> is a unique pagination token for each page. Make the call again using the returned token to retrieve the next page. Keep all other arguments unchanged. Each pagination token expires after 60 seconds. Using an expired pagination token will return a <code>400</code> error: "<code>Specified token has exceeded its maximum lifetime</code>".  The configured <code>maximumPageSize</code> determines how many results can be returned in a single call.  <note> The <code>nextPageToken</code> returned by this action cannot be used with <a>GetWorkflowExecutionHistory</a> to get the next page. You must call <a>PollForDecisionTask</a> again (with the <code>nextPageToken</code>) to retrieve the next page of history records. Calling <a>PollForDecisionTask</a> with a <code>nextPageToken</code> doesn't return a new decision task. </note><br/>identity:  Identity of the decider making the request, which is recorded in the DecisionTaskStarted event in the workflow history. This enables diagnostic tracing when problems arise. The form of this identity is user defined.<br/>taskList:  Specifies the task list to poll for decision tasks. The specified string must not start or end with whitespace. It must not contain a <code>:</code> (colon), <code>/</code> (slash), <code>|</code> (vertical bar), or any control characters (<code>\u0000-\u001f</code> | <code>\u007f-\u009f</code>). Also, it must not <i>be</i> the literal string <code>arn</code>.:<br/>domain:  The name of the domain containing the task lists to poll.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the task list.<br/>
        :return: previousStartedEventId:  The ID of the DecisionTaskStarted event of the previous decision task of this workflow execution that was processed by the decider. This can be used to determine the events in the history new since the last decision task received by the decider.<br/>nextPageToken:  If a <code>NextPageToken</code> was returned by a previous call, there are more results available. To retrieve the next page of results, make the call again using the returned token in <code>nextPageToken</code>. Keep all other arguments unchanged. The configured <code>maximumPageSize</code> determines how many results can be returned in a single call.<br/>events:  A paginated list of history events of the workflow execution. The decider uses this during the processing of the decision task.<br/>workflowType:  The type of the workflow execution for which this decision task was created.:<br/>workflowExecution:  The workflow execution for which this decision task was created.:<br/>startedEventId:  The ID of the <code>DecisionTaskStarted</code> event recorded in the history.<br/>taskToken:  The opaque string used as a handle on the task. This token is used by workers to communicate progress and response information back to the system about the task.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>runId</i> A system-generated unique identifier for the workflow execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>workflowId</i> The user defined identifier associated with the workflow execution.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i>  The version of the workflow type. <note> The combination of workflow type name and version must be unique with in a domain. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i>  The name of the workflow type. <note> The combination of workflow type name and version must be unique with in a domain. </note><br/>        """
        pass

    def record_activity_task_heartbeat(self, taskToken: str, details: str=None) -> ActivityTaskStatus:
        """Used by activity workers to report to the service that the <a>ActivityTask</a> represented by the specified <code>taskToken</code> is still making progress. The worker can also specify details of the progress, for example percent complete, using the <code>details</code> parameter. This action can also be used by the worker as a mechanism to check if cancellation is being requested for the activity task. If a cancellation is being attempted for the specified task, then the boolean <code>cancelRequested</code> flag returned by the service is set to <code>true</code>. This action resets the <code>taskHeartbeatTimeout</code> clock. The <code>taskHeartbeatTimeout</code> is specified in <a>RegisterActivityType</a>. This action doesn't in itself create an event in the workflow execution history. However, if the task times out, the workflow execution history contains a <code>ActivityTaskTimedOut</code> event that contains the information from the last heartbeat generated by the activity worker. <note> The <code>taskStartToCloseTimeout</code> of an activity type is the maximum duration of an activity task, regardless of the number of <a>RecordActivityTaskHeartbeat</a> requests received. The <code>taskStartToCloseTimeout</code> is also specified in <a>RegisterActivityType</a>. </note> <note> This operation is only useful for long-lived activities to report liveliness of the task and to determine if a cancellation is being attempted. </note> <important> If the <code>cancelRequested</code> flag returns <code>true</code>, a cancellation is being attempted. If the worker can cancel the activity, it should respond with <a>RespondActivityTaskCanceled</a>. Otherwise, it should ignore the cancellation request. </important>  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> You cannot use an IAM policy to constrain this action's parameters. </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param details:  If specified, contains details about the progress of the task.<br/>taskToken:  The <code>taskToken</code> of the <a>ActivityTask</a>. <important>  <code>taskToken</code> is generated by the service and should be treated as an opaque value. If the task is passed to another process, its <code>taskToken</code> must also be passed. This enables it to provide its progress and respond with results.  </important><br/>
        :return: cancelRequested:  Set to <code>true</code> if cancellation of the task is requested.<br/>        """
        pass

    def register_activity_type(self, version: str, name: str, domain: str, description: str=None, defaultTaskStartToCloseTimeout: str=None, defaultTaskHeartbeatTimeout: str=None, defaultTaskList: TaskList=None, defaultTaskPriority: str=None, defaultTaskScheduleToStartTimeout: str=None, defaultTaskScheduleToCloseTimeout: str=None):
        """Registers a new <i>activity type</i> along with its configuration settings in the specified domain. <important> A <code>TypeAlreadyExists</code> fault is returned if the type already exists in the domain. You cannot change any configuration settings of the type after its registration, and it must be registered as a new version. </important>  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> Constrain the following parameters by using a <code>Condition</code> element with the appropriate keys. <ul> <li>  <code>defaultTaskList.name</code>: String constraint. The key is <code>swf:defaultTaskList.name</code>. </li> <li>  <code>name</code>: String constraint. The key is <code>swf:name</code>. </li> <li>  <code>version</code>: String constraint. The key is <code>swf:version</code>. </li> </ul> </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param defaultTaskScheduleToCloseTimeout:  If set, specifies the default maximum duration for a task of this activity type. This default can be overridden when scheduling an activity task using the <code>ScheduleActivityTask</code> <a>Decision</a>. The duration is specified in seconds, an integer greater than or equal to <code>0</code>. You can use <code>NONE</code> to specify unlimited duration.<br/>defaultTaskScheduleToStartTimeout:  If set, specifies the default maximum duration that a task of this activity type can wait before being assigned to a worker. This default can be overridden when scheduling an activity task using the <code>ScheduleActivityTask</code> <a>Decision</a>. The duration is specified in seconds, an integer greater than or equal to <code>0</code>. You can use <code>NONE</code> to specify unlimited duration.<br/>defaultTaskPriority:  The default task priority to assign to the activity type. If not assigned, then <code>0</code> is used. Valid values are integers that range from Java's <code>Integer.MIN_VALUE</code> (-2147483648) to <code>Integer.MAX_VALUE</code> (2147483647). Higher numbers indicate higher priority. For more information about setting task priority, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/programming-priority.html">Setting Task Priority</a> in the <i>in the <i>Amazon SWF Developer Guide</i>.</i>.<br/>defaultTaskList:  If set, specifies the default task list to use for scheduling tasks of this activity type. This default task list is used if a task list isn't provided when a task is scheduled through the <code>ScheduleActivityTask</code> <a>Decision</a>.:<br/>defaultTaskHeartbeatTimeout:  If set, specifies the default maximum time before which a worker processing a task of this type must report progress by calling <a>RecordActivityTaskHeartbeat</a>. If the timeout is exceeded, the activity task is automatically timed out. This default can be overridden when scheduling an activity task using the <code>ScheduleActivityTask</code> <a>Decision</a>. If the activity worker subsequently attempts to record a heartbeat or returns a result, the activity worker receives an <code>UnknownResource</code> fault. In this case, Amazon SWF no longer considers the activity task to be valid; the activity worker should clean up the activity task. The duration is specified in seconds, an integer greater than or equal to <code>0</code>. You can use <code>NONE</code> to specify unlimited duration.<br/>defaultTaskStartToCloseTimeout:  If set, specifies the default maximum duration that a worker can take to process tasks of this activity type. This default can be overridden when scheduling an activity task using the <code>ScheduleActivityTask</code> <a>Decision</a>. The duration is specified in seconds, an integer greater than or equal to <code>0</code>. You can use <code>NONE</code> to specify unlimited duration.<br/>description:  A textual description of the activity type.<br/>version:  The version of the activity type. <note> The activity type consists of the name and version, the combination of which must be unique within the domain. </note> The specified string must not start or end with whitespace. It must not contain a <code>:</code> (colon), <code>/</code> (slash), <code>|</code> (vertical bar), or any control characters (<code>\u0000-\u001f</code> | <code>\u007f-\u009f</code>). Also, it must not <i>be</i> the literal string <code>arn</code>.<br/>name:  The name of the activity type within the domain. The specified string must not start or end with whitespace. It must not contain a <code>:</code> (colon), <code>/</code> (slash), <code>|</code> (vertical bar), or any control characters (<code>\u0000-\u001f</code> | <code>\u007f-\u009f</code>). Also, it must not <i>be</i> the literal string <code>arn</code>.<br/>domain:  The name of the domain in which this activity is to be registered.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the task list.<br/>
        :return:         """
        pass

    def register_domain(self, workflowExecutionRetentionPeriodInDays: str, name: str, description: str=None, tags: List[ResourceTag]=None):
        """Registers a new domain.  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> You cannot use an IAM policy to control domain access for this action. The name of the domain being registered is available as the resource of this action. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> You cannot use an IAM policy to constrain this action's parameters. </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param tags:  Tags to be added when registering a domain. Tags may only contain unicode letters, digits, whitespace, or these symbols: <code>_ . : / = + - @</code>.<br/>workflowExecutionRetentionPeriodInDays:  The duration (in days) that records and histories of workflow executions on the domain should be kept by the service. After the retention period, the workflow execution isn't available in the results of visibility calls. If you pass the value <code>NONE</code> or <code>0</code> (zero), then the workflow execution history isn't retained. As soon as the workflow execution completes, the execution record and its history are deleted. The maximum workflow execution retention period is 90 days. For more information about Amazon SWF service limits, see: <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dg-limits.html">Amazon SWF Service Limits</a> in the <i>Amazon SWF Developer Guide</i>.<br/>description:  A text description of the domain.<br/>name:  Name of the domain to register. The name must be unique in the region that the domain is registered in. The specified string must not start or end with whitespace. It must not contain a <code>:</code> (colon), <code>/</code> (slash), <code>|</code> (vertical bar), or any control characters (<code>\u0000-\u001f</code> | <code>\u007f-\u009f</code>). Also, it must not <i>be</i> the literal string <code>arn</code>.<br/>
        :return:         """
        pass

    def register_workflow_type(self, version: str, name: str, domain: str, description: str=None, defaultTaskStartToCloseTimeout: str=None, defaultExecutionStartToCloseTimeout: str=None, defaultTaskList: TaskList=None, defaultTaskPriority: str=None, defaultChildPolicy: str=None, defaultLambdaRole: str=None):
        """Registers a new <i>workflow type</i> and its configuration settings in the specified domain. The retention period for the workflow history is set by the <a>RegisterDomain</a> action. <important> If the type already exists, then a <code>TypeAlreadyExists</code> fault is returned. You cannot change the configuration settings of a workflow type once it is registered and it must be registered as a new version. </important>  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> Constrain the following parameters by using a <code>Condition</code> element with the appropriate keys. <ul> <li>  <code>defaultTaskList.name</code>: String constraint. The key is <code>swf:defaultTaskList.name</code>. </li> <li>  <code>name</code>: String constraint. The key is <code>swf:name</code>. </li> <li>  <code>version</code>: String constraint. The key is <code>swf:version</code>. </li> </ul> </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param defaultLambdaRole:  The default IAM role attached to this workflow type. <note> Executions of this workflow type need IAM roles to invoke Lambda functions. If you don't specify an IAM role when you start this workflow type, the default Lambda role is attached to the execution. For more information, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/lambda-task.html">https://docs.aws.amazon.com/amazonswf/latest/developerguide/lambda-task.html</a> in the <i>Amazon SWF Developer Guide</i>. </note><br/>defaultChildPolicy:  If set, specifies the default policy to use for the child workflow executions when a workflow execution of this type is terminated, by calling the <a>TerminateWorkflowExecution</a> action explicitly or due to an expired timeout. This default can be overridden when starting a workflow execution using the <a>StartWorkflowExecution</a> action or the <code>StartChildWorkflowExecution</code> <a>Decision</a>. The supported child policies are: <ul> <li>  <code>TERMINATE</code> – The child executions are terminated. </li> <li>  <code>REQUEST_CANCEL</code> – A request to cancel is attempted for each child execution by recording a <code>WorkflowExecutionCancelRequested</code> event in its history. It is up to the decider to take appropriate actions when it receives an execution history with this event. </li> <li>  <code>ABANDON</code> – No action is taken. The child executions continue to run. </li> </ul><br/>defaultTaskPriority:  The default task priority to assign to the workflow type. If not assigned, then <code>0</code> is used. Valid values are integers that range from Java's <code>Integer.MIN_VALUE</code> (-2147483648) to <code>Integer.MAX_VALUE</code> (2147483647). Higher numbers indicate higher priority. For more information about setting task priority, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/programming-priority.html">Setting Task Priority</a> in the <i>Amazon SWF Developer Guide</i>.<br/>defaultTaskList:  If set, specifies the default task list to use for scheduling decision tasks for executions of this workflow type. This default is used only if a task list isn't provided when starting the execution through the <a>StartWorkflowExecution</a> Action or <code>StartChildWorkflowExecution</code> <a>Decision</a>.:<br/>defaultExecutionStartToCloseTimeout:  If set, specifies the default maximum duration for executions of this workflow type. You can override this default when starting an execution through the <a>StartWorkflowExecution</a> Action or <code>StartChildWorkflowExecution</code> <a>Decision</a>. The duration is specified in seconds; an integer greater than or equal to 0. Unlike some of the other timeout parameters in Amazon SWF, you cannot specify a value of "NONE" for <code>defaultExecutionStartToCloseTimeout</code>; there is a one-year max limit on the time that a workflow execution can run. Exceeding this limit always causes the workflow execution to time out.<br/>defaultTaskStartToCloseTimeout:  If set, specifies the default maximum duration of decision tasks for this workflow type. This default can be overridden when starting a workflow execution using the <a>StartWorkflowExecution</a> action or the <code>StartChildWorkflowExecution</code> <a>Decision</a>. The duration is specified in seconds, an integer greater than or equal to <code>0</code>. You can use <code>NONE</code> to specify unlimited duration.<br/>description:  Textual description of the workflow type.<br/>version:  The version of the workflow type. <note> The workflow type consists of the name and version, the combination of which must be unique within the domain. To get a list of all currently registered workflow types, use the <a>ListWorkflowTypes</a> action. </note> The specified string must not start or end with whitespace. It must not contain a <code>:</code> (colon), <code>/</code> (slash), <code>|</code> (vertical bar), or any control characters (<code>\u0000-\u001f</code> | <code>\u007f-\u009f</code>). Also, it must not <i>be</i> the literal string <code>arn</code>.<br/>name:  The name of the workflow type. The specified string must not start or end with whitespace. It must not contain a <code>:</code> (colon), <code>/</code> (slash), <code>|</code> (vertical bar), or any control characters (<code>\u0000-\u001f</code> | <code>\u007f-\u009f</code>). Also, it must not <i>be</i> the literal string <code>arn</code>.<br/>domain:  The name of the domain in which to register the workflow type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the task list.<br/>
        :return:         """
        pass

    def request_cancel_workflow_execution(self, workflowId: str, domain: str, runId: str=None):
        """Records a <code>WorkflowExecutionCancelRequested</code> event in the currently running workflow execution identified by the given domain, workflowId, and runId. This logically requests the cancellation of the workflow execution as a whole. It is up to the decider to take appropriate actions when it receives an execution history with this event. <note> If the runId isn't specified, the <code>WorkflowExecutionCancelRequested</code> event is recorded in the history of the current open workflow execution with the specified workflowId in the domain. </note> <note> Because this action allows the workflow to properly clean up and gracefully close, it should be used instead of <a>TerminateWorkflowExecution</a> when possible. </note>  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> You cannot use an IAM policy to constrain this action's parameters. </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param runId:  The runId of the workflow execution to cancel.<br/>workflowId:  The workflowId of the workflow execution to cancel.<br/>domain:  The name of the domain containing the workflow execution to cancel.<br/>
        :return:         """
        pass

    def respond_activity_task_canceled(self, taskToken: str, details: str=None):
        """Used by workers to tell the service that the <a>ActivityTask</a> identified by the <code>taskToken</code> was successfully canceled. Additional <code>details</code> can be provided using the <code>details</code> argument. These <code>details</code> (if provided) appear in the <code>ActivityTaskCanceled</code> event added to the workflow history. <important> Only use this operation if the <code>canceled</code> flag of a <a>RecordActivityTaskHeartbeat</a> request returns <code>true</code> and if the activity can be safely undone or abandoned. </important> A task is considered open from the time that it is scheduled until it is closed. Therefore a task is reported as open while a worker is processing it. A task is closed after it has been specified in a call to <a>RespondActivityTaskCompleted</a>, RespondActivityTaskCanceled, <a>RespondActivityTaskFailed</a>, or the task has <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dg-basic.html#swf-dev-timeout-types">timed out</a>.  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> You cannot use an IAM policy to constrain this action's parameters. </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param details:   Information about the cancellation.<br/>taskToken:  The <code>taskToken</code> of the <a>ActivityTask</a>. <important>  <code>taskToken</code> is generated by the service and should be treated as an opaque value. If the task is passed to another process, its <code>taskToken</code> must also be passed. This enables it to provide its progress and respond with results. </important><br/>
        :return:         """
        pass

    def respond_activity_task_completed(self, taskToken: str, result: str=None):
        """Used by workers to tell the service that the <a>ActivityTask</a> identified by the <code>taskToken</code> completed successfully with a <code>result</code> (if provided). The <code>result</code> appears in the <code>ActivityTaskCompleted</code> event in the workflow history. <important> If the requested task doesn't complete successfully, use <a>RespondActivityTaskFailed</a> instead. If the worker finds that the task is canceled through the <code>canceled</code> flag returned by <a>RecordActivityTaskHeartbeat</a>, it should cancel the task, clean up and then call <a>RespondActivityTaskCanceled</a>. </important> A task is considered open from the time that it is scheduled until it is closed. Therefore a task is reported as open while a worker is processing it. A task is closed after it has been specified in a call to RespondActivityTaskCompleted, <a>RespondActivityTaskCanceled</a>, <a>RespondActivityTaskFailed</a>, or the task has <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dg-basic.html#swf-dev-timeout-types">timed out</a>.  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> You cannot use an IAM policy to constrain this action's parameters. </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param result:  The result of the activity task. It is a free form string that is implementation specific.<br/>taskToken:  The <code>taskToken</code> of the <a>ActivityTask</a>. <important>  <code>taskToken</code> is generated by the service and should be treated as an opaque value. If the task is passed to another process, its <code>taskToken</code> must also be passed. This enables it to provide its progress and respond with results. </important><br/>
        :return:         """
        pass

    def respond_activity_task_failed(self, taskToken: str, reason: str=None, details: str=None):
        """Used by workers to tell the service that the <a>ActivityTask</a> identified by the <code>taskToken</code> has failed with <code>reason</code> (if specified). The <code>reason</code> and <code>details</code> appear in the <code>ActivityTaskFailed</code> event added to the workflow history. A task is considered open from the time that it is scheduled until it is closed. Therefore a task is reported as open while a worker is processing it. A task is closed after it has been specified in a call to <a>RespondActivityTaskCompleted</a>, <a>RespondActivityTaskCanceled</a>, RespondActivityTaskFailed, or the task has <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dg-basic.html#swf-dev-timeout-types">timed out</a>.  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> You cannot use an IAM policy to constrain this action's parameters. </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param details:   Detailed information about the failure.<br/>reason:  Description of the error that may assist in diagnostics.<br/>taskToken:  The <code>taskToken</code> of the <a>ActivityTask</a>. <important>  <code>taskToken</code> is generated by the service and should be treated as an opaque value. If the task is passed to another process, its <code>taskToken</code> must also be passed. This enables it to provide its progress and respond with results. </important><br/>
        :return:         """
        pass

    def respond_decision_task_completed(self, taskToken: str, decisions: List[Decision]=None, executionContext: str=None):
        """Used by deciders to tell the service that the <a>DecisionTask</a> identified by the <code>taskToken</code> has successfully completed. The <code>decisions</code> argument specifies the list of decisions made while processing the task. A <code>DecisionTaskCompleted</code> event is added to the workflow history. The <code>executionContext</code> specified is attached to the event in the workflow execution history.  <b>Access Control</b>  If an IAM policy grants permission to use <code>RespondDecisionTaskCompleted</code>, it can express permissions for the list of decisions in the <code>decisions</code> parameter. Each of the decisions has one or more parameters, much like a regular API call. To allow for policies to be as readable as possible, you can express permissions on decisions as if they were actual API calls, including applying conditions to some parameters. For more information, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param executionContext:  User defined context to add to workflow execution.<br/>decisions:  The list of decisions (possibly empty) made by the decider while processing this decision task. See the docs for the <a>Decision</a> structure for details.<br/>taskToken:  The <code>taskToken</code> from the <a>DecisionTask</a>. <important>  <code>taskToken</code> is generated by the service and should be treated as an opaque value. If the task is passed to another process, its <code>taskToken</code> must also be passed. This enables it to provide its progress and respond with results. </important><br/>
        :return:         """
        pass

    def signal_workflow_execution(self, signalName: str, workflowId: str, domain: str, runId: str=None, input: str=None):
        """Records a <code>WorkflowExecutionSignaled</code> event in the workflow execution history and creates a decision task for the workflow execution identified by the given domain, workflowId and runId. The event is recorded with the specified user defined signalName and input (if provided). <note> If a runId isn't specified, then the <code>WorkflowExecutionSignaled</code> event is recorded in the history of the current open workflow with the matching workflowId in the domain. </note> <note> If the specified workflow execution isn't open, this method fails with <code>UnknownResource</code>. </note>  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> You cannot use an IAM policy to constrain this action's parameters. </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param input:  Data to attach to the <code>WorkflowExecutionSignaled</code> event in the target workflow execution's history.<br/>signalName:  The name of the signal. This name must be meaningful to the target workflow.<br/>runId:  The runId of the workflow execution to signal.<br/>workflowId:  The workflowId of the workflow execution to signal.<br/>domain:  The name of the domain containing the workflow execution to signal.<br/>
        :return:         """
        pass

    def start_workflow_execution(self, workflowType: WorkflowType, workflowId: str, domain: str, taskList: TaskList=None, taskPriority: str=None, input: str=None, executionStartToCloseTimeout: str=None, tagList: List[Tag]=None, taskStartToCloseTimeout: str=None, childPolicy: str=None, lambdaRole: str=None) -> Run:
        """Starts an execution of the workflow type in the specified domain using the provided <code>workflowId</code> and input data. This action returns the newly started workflow execution.  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> Constrain the following parameters by using a <code>Condition</code> element with the appropriate keys. <ul> <li>  <code>tagList.member.0</code>: The key is <code>swf:tagList.member.0</code>. </li> <li>  <code>tagList.member.1</code>: The key is <code>swf:tagList.member.1</code>. </li> <li>  <code>tagList.member.2</code>: The key is <code>swf:tagList.member.2</code>. </li> <li>  <code>tagList.member.3</code>: The key is <code>swf:tagList.member.3</code>. </li> <li>  <code>tagList.member.4</code>: The key is <code>swf:tagList.member.4</code>. </li> <li>  <code>taskList</code>: String constraint. The key is <code>swf:taskList.name</code>. </li> <li>  <code>workflowType.name</code>: String constraint. The key is <code>swf:workflowType.name</code>. </li> <li>  <code>workflowType.version</code>: String constraint. The key is <code>swf:workflowType.version</code>. </li> </ul> </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param lambdaRole:  The IAM role to attach to this workflow execution. <note> Executions of this workflow type need IAM roles to invoke Lambda functions. If you don't attach an IAM role, any attempt to schedule a Lambda task fails. This results in a <code>ScheduleLambdaFunctionFailed</code> history event. For more information, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/lambda-task.html">https://docs.aws.amazon.com/amazonswf/latest/developerguide/lambda-task.html</a> in the <i>Amazon SWF Developer Guide</i>. </note><br/>childPolicy:  If set, specifies the policy to use for the child workflow executions of this workflow execution if it is terminated, by calling the <a>TerminateWorkflowExecution</a> action explicitly or due to an expired timeout. This policy overrides the default child policy specified when registering the workflow type using <a>RegisterWorkflowType</a>. The supported child policies are: <ul> <li>  <code>TERMINATE</code> – The child executions are terminated. </li> <li>  <code>REQUEST_CANCEL</code> – A request to cancel is attempted for each child execution by recording a <code>WorkflowExecutionCancelRequested</code> event in its history. It is up to the decider to take appropriate actions when it receives an execution history with this event. </li> <li>  <code>ABANDON</code> – No action is taken. The child executions continue to run. </li> </ul> <note> A child policy for this workflow execution must be specified either as a default for the workflow type or through this parameter. If neither this parameter is set nor a default child policy was specified at registration time then a fault is returned. </note><br/>taskStartToCloseTimeout:  Specifies the maximum duration of decision tasks for this workflow execution. This parameter overrides the <code>defaultTaskStartToCloseTimout</code> specified when registering the workflow type using <a>RegisterWorkflowType</a>. The duration is specified in seconds, an integer greater than or equal to <code>0</code>. You can use <code>NONE</code> to specify unlimited duration. <note> A task start-to-close timeout for this workflow execution must be specified either as a default for the workflow type or through this parameter. If neither this parameter is set nor a default task start-to-close timeout was specified at registration time then a fault is returned. </note><br/>tagList:  The list of tags to associate with the workflow execution. You can specify a maximum of 5 tags. You can list workflow executions with a specific tag by calling <a>ListOpenWorkflowExecutions</a> or <a>ListClosedWorkflowExecutions</a> and specifying a <a>TagFilter</a>.<br/>executionStartToCloseTimeout:  The total duration for this workflow execution. This overrides the defaultExecutionStartToCloseTimeout specified when registering the workflow type. The duration is specified in seconds; an integer greater than or equal to <code>0</code>. Exceeding this limit causes the workflow execution to time out. Unlike some of the other timeout parameters in Amazon SWF, you cannot specify a value of "NONE" for this timeout; there is a one-year max limit on the time that a workflow execution can run. <note> An execution start-to-close timeout must be specified either through this parameter or as a default when the workflow type is registered. If neither this parameter nor a default execution start-to-close timeout is specified, a fault is returned. </note><br/>input:  The input for the workflow execution. This is a free form string which should be meaningful to the workflow you are starting. This <code>input</code> is made available to the new workflow execution in the <code>WorkflowExecutionStarted</code> history event.<br/>taskPriority:  The task priority to use for this workflow execution. This overrides any default priority that was assigned when the workflow type was registered. If not set, then the default task priority for the workflow type is used. Valid values are integers that range from Java's <code>Integer.MIN_VALUE</code> (-2147483648) to <code>Integer.MAX_VALUE</code> (2147483647). Higher numbers indicate higher priority. For more information about setting task priority, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/programming-priority.html">Setting Task Priority</a> in the <i>Amazon SWF Developer Guide</i>.<br/>taskList:  The task list to use for the decision tasks generated for this workflow execution. This overrides the <code>defaultTaskList</code> specified when registering the workflow type. <note> A task list for this workflow execution must be specified either as a default for the workflow type or through this parameter. If neither this parameter is set nor a default task list was specified at registration time then a fault is returned. </note> The specified string must not start or end with whitespace. It must not contain a <code>:</code> (colon), <code>/</code> (slash), <code>|</code> (vertical bar), or any control characters (<code>\u0000-\u001f</code> | <code>\u007f-\u009f</code>). Also, it must not <i>be</i> the literal string <code>arn</code>.:<br/>workflowType:  The type of the workflow to start.:<br/>workflowId:  The user defined identifier associated with the workflow execution. You can use this to associate a custom identifier with the workflow execution. You may specify the same identifier if a workflow execution is logically a <i>restart</i> of a previous execution. You cannot have two open workflow executions with the same <code>workflowId</code> at the same time within the same domain. The specified string must not start or end with whitespace. It must not contain a <code>:</code> (colon), <code>/</code> (slash), <code>|</code> (vertical bar), or any control characters (<code>\u0000-\u001f</code> | <code>\u007f-\u009f</code>). Also, it must not <i>be</i> the literal string <code>arn</code>.<br/>domain:  The name of the domain in which the workflow execution is created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i>  The version of the workflow type. <note> The combination of workflow type name and version must be unique with in a domain. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i>  The name of the workflow type. <note> The combination of workflow type name and version must be unique with in a domain. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of the task list.<br/>
        :return: runId:  The <code>runId</code> of a workflow execution. This ID is generated by the service and can be used to uniquely identify the workflow execution within a domain.<br/>        """
        pass

    def tag_resource(self, tags: List[ResourceTag], resourceArn: str):
        """Add a tag to a Amazon SWF domain. <note> Amazon SWF supports a maximum of 50 tags per resource. </note>

        :param self:
        :param tags:  The list of tags to add to a domain.  Tags may only contain unicode letters, digits, whitespace, or these symbols: <code>_ . : / = + - @</code>.<br/>resourceArn:  The Amazon Resource Name (ARN) for the Amazon SWF domain.<br/>
        :return:         """
        pass

    def terminate_workflow_execution(self, workflowId: str, domain: str, runId: str=None, reason: str=None, details: str=None, childPolicy: str=None):
        """Records a <code>WorkflowExecutionTerminated</code> event and forces closure of the workflow execution identified by the given domain, runId, and workflowId. The child policy, registered with the workflow type or specified when starting this execution, is applied to any open child workflow executions of this workflow execution. <important> If the identified workflow execution was in progress, it is terminated immediately. </important> <note> If a runId isn't specified, then the <code>WorkflowExecutionTerminated</code> event is recorded in the history of the current open workflow with the matching workflowId in the domain. </note> <note> You should consider using <a>RequestCancelWorkflowExecution</a> action instead because it allows the workflow to gracefully close while <a>TerminateWorkflowExecution</a> doesn't. </note>  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> You cannot use an IAM policy to constrain this action's parameters. </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param childPolicy:  If set, specifies the policy to use for the child workflow executions of the workflow execution being terminated. This policy overrides the child policy specified for the workflow execution at registration time or when starting the execution. The supported child policies are: <ul> <li>  <code>TERMINATE</code> – The child executions are terminated. </li> <li>  <code>REQUEST_CANCEL</code> – A request to cancel is attempted for each child execution by recording a <code>WorkflowExecutionCancelRequested</code> event in its history. It is up to the decider to take appropriate actions when it receives an execution history with this event. </li> <li>  <code>ABANDON</code> – No action is taken. The child executions continue to run. </li> </ul> <note> A child policy for this workflow execution must be specified either as a default for the workflow type or through this parameter. If neither this parameter is set nor a default child policy was specified at registration time then a fault is returned. </note><br/>details:   Details for terminating the workflow execution.<br/>reason:   A descriptive reason for terminating the workflow execution.<br/>runId:  The runId of the workflow execution to terminate.<br/>workflowId:  The workflowId of the workflow execution to terminate.<br/>domain:  The domain of the workflow execution to terminate.<br/>
        :return:         """
        pass

    def undeprecate_activity_type(self, activityType: ActivityType, domain: str):
        """Undeprecates a previously deprecated <i>activity type</i>. After an activity type has been undeprecated, you can create new tasks of that activity type. <note> This operation is eventually consistent. The results are best effort and may not exactly reflect recent updates and changes. </note>  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> Constrain the following parameters by using a <code>Condition</code> element with the appropriate keys. <ul> <li>  <code>activityType.name</code>: String constraint. The key is <code>swf:activityType.name</code>. </li> <li>  <code>activityType.version</code>: String constraint. The key is <code>swf:activityType.version</code>. </li> </ul> </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param activityType:  The activity type to undeprecate.:<br/>domain:  The name of the domain of the deprecated activity type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i> The version of this activity. <note> The combination of activity type name and version must be unique with in a domain. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i> The name of this activity. <note> The combination of activity type name and version must be unique within a domain. </note><br/>
        :return:         """
        pass

    def undeprecate_domain(self, name: str):
        """Undeprecates a previously deprecated domain. After a domain has been undeprecated it can be used to create new workflow executions or register new types. <note> This operation is eventually consistent. The results are best effort and may not exactly reflect recent updates and changes. </note>  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> You cannot use an IAM policy to constrain this action's parameters. </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param name:  The name of the domain of the deprecated workflow type.<br/>
        :return:         """
        pass

    def undeprecate_workflow_type(self, workflowType: WorkflowType, domain: str):
        """Undeprecates a previously deprecated <i>workflow type</i>. After a workflow type has been undeprecated, you can create new executions of that type.  <note> This operation is eventually consistent. The results are best effort and may not exactly reflect recent updates and changes. </note>  <b>Access Control</b>  You can use IAM policies to control this action's access to Amazon SWF resources as follows: <ul> <li> Use a <code>Resource</code> element with the domain name to limit the action to only specified domains. </li> <li> Use an <code>Action</code> element to allow or deny permission to call this action. </li> <li> Constrain the following parameters by using a <code>Condition</code> element with the appropriate keys. <ul> <li>  <code>workflowType.name</code>: String constraint. The key is <code>swf:workflowType.name</code>. </li> <li>  <code>workflowType.version</code>: String constraint. The key is <code>swf:workflowType.version</code>. </li> </ul> </li> </ul> If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's <code>cause</code> parameter is set to <code>OPERATION_NOT_PERMITTED</code>. For details and example IAM policies, see <a href="https://docs.aws.amazon.com/amazonswf/latest/developerguide/swf-dev-iam.html">Using IAM to Manage Access to Amazon SWF Workflows</a> in the <i>Amazon SWF Developer Guide</i>.

        :param self:
        :param workflowType:  The name of the domain of the deprecated workflow type.:<br/>domain:  The name of the domain of the deprecated workflow type.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>version</i>  The version of the workflow type. <note> The combination of workflow type name and version must be unique with in a domain. </note><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>name</i>  The name of the workflow type. <note> The combination of workflow type name and version must be unique with in a domain. </note><br/>
        :return:         """
        pass

    def untag_resource(self, tagKeys: List[ResourceTagKey], resourceArn: str):
        """Remove a tag from a Amazon SWF domain.

        :param self:
        :param tagKeys:  The list of tags to remove from the Amazon SWF domain.<br/>resourceArn:  The Amazon Resource Name (ARN) for the Amazon SWF domain.<br/>
        :return:         """
        pass



class Textract(BaseClient):
    """Amazon Textract detects and analyzes text in documents and converts it into machine-readable text. This is the API reference documentation for Amazon Textract."""




    class AnalyzeDocumentRequest(Mapping):
        pass
    
    class AnalyzeDocumentResponse(Mapping):
        pass
    
    class Document(Mapping):
        pass
    
    class FeatureType(object):
        pass
    
    class DetectDocumentTextRequest(Mapping):
        pass
    
    class DetectDocumentTextResponse(Mapping):
        pass
    
    class GetDocumentAnalysisRequest(Mapping):
        pass
    
    class GetDocumentAnalysisResponse(Mapping):
        pass
    
    class GetDocumentTextDetectionRequest(Mapping):
        pass
    
    class GetDocumentTextDetectionResponse(Mapping):
        pass
    
    class StartDocumentAnalysisRequest(Mapping):
        pass
    
    class StartDocumentAnalysisResponse(Mapping):
        pass
    
    class DocumentLocation(Mapping):
        pass
    
    class NotificationChannel(Mapping):
        pass
    
    class StartDocumentTextDetectionRequest(Mapping):
        pass
    
    class StartDocumentTextDetectionResponse(Mapping):
        pass
    
    def analyze_document(self, FeatureTypes: List[FeatureType], Document: Document) -> AnalyzeDocumentResponse:
        """Analyzes an input document for relationships between detected items.  The types of information returned are as follows:  <ul> <li> Words and lines that are related to nearby lines and words. The related information is returned in two <a>Block</a> objects each of type <code>KEY_VALUE_SET</code>: a KEY Block object and a VALUE Block object. For example, <i>Name: Ana Silva Carolina</i> contains a key and value. <i>Name:</i> is the key. <i>Ana Silva Carolina</i> is the value. </li> <li> Table and table cell data. A TABLE Block object contains information about a detected table. A CELL Block object is returned for each cell in a table. </li> <li> Selectable elements such as checkboxes and radio buttons. A SELECTION_ELEMENT Block object contains information about a selectable element. </li> <li> Lines and words of text. A LINE Block object contains one or more WORD Block objects. </li> </ul> You can choose which type of analysis to perform by specifying the <code>FeatureTypes</code> list.  The output is returned in a list of <code>BLOCK</code> objects.  <code>AnalyzeDocument</code> is a synchronous operation. To analyze documents asynchronously, use <a>StartDocumentAnalysis</a>. For more information, see <a href="https://docs.aws.amazon.com/textract/latest/dg/how-it-works-analyzing.html">Document Text Analysis</a>.

        :param self:
        :param FeatureTypes:  A list of the types of analysis to perform. Add TABLES to the list to return information about the tables detected in the input document. Add FORMS to return detected fields and the associated text. To perform both types of analysis, add TABLES and FORMS to <code>FeatureTypes</code>.<br/>Document:  The input document as base64-encoded bytes or an Amazon S3 object. If you use the AWS CLI to call Amazon Textract operations, you can't pass image bytes. The document must be an image in JPG or PNG format. If you are using an AWS SDK to call Amazon Textract, you might not need to base64-encode image bytes passed using the <code>Bytes</code> field. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Object</i> Identifies an S3 object as the document source. The maximum size of a document stored in an S3 bucket is 5 MB.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Object</b>: Identifies an S3 object as the document source. The maximum size of a document stored in an S3 bucket is 5 MB.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Bytes</i> A blob of base-64 encoded documents bytes. The maximum size of a document that's provided in a blob of bytes is 5 MB. The document bytes must be in PNG or JPG format. If you are using an AWS SDK to call Amazon Textract, you might not need to base64-encode image bytes passed using the <code>Bytes</code> field. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> If the bucket has versioning enabled, you can specify the object version. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The file name of the input document. It must be an image file (.JPG or .PNG format). Asynchronous operations also support PDF files.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> The name of the S3 bucket.<br/>
        :return: Blocks:  The text that's detected and analyzed by <code>AnalyzeDocument</code>.<br/>DocumentMetadata:  Metadata about the analyzed document. An example is the number of pages.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Pages</i> The number of pages detected in the document.<br/>        """
        pass

    def detect_document_text(self, Document: Document) -> DetectDocumentTextResponse:
        """Detects text in the input document. Amazon Textract can detect lines of text and the words that make up a line of text. The input document must be an image in JPG or PNG format. <code>DetectDocumentText</code> returns the detected text in an array of <a>Block</a> objects.  Each document page has as an associated <code>Block</code> of type PAGE. Each PAGE <code>Block</code> object is the parent of LINE <code>Block</code> objects that represent the lines of detected text on a page. A LINE <code>Block</code> object is a parent for each word that makes up the line. Words are represented by <code>Block</code> objects of type WORD.  <code>DetectDocumentText</code> is a synchronous operation. To analyze documents asynchronously, use <a>StartDocumentTextDetection</a>. For more information, see <a href="https://docs.aws.amazon.com/textract/latest/dg/how-it-works-detecting.html">Document Text Detection</a>.

        :param self:
        :param Document:  The input document as base64-encoded bytes or an Amazon S3 object. If you use the AWS CLI to call Amazon Textract operations, you can't pass image bytes. The document must be an image in JPG or PNG format. If you are using an AWS SDK to call Amazon Textract, you might not need to base64-encode image bytes passed using the <code>Bytes</code> field. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Object</i> Identifies an S3 object as the document source. The maximum size of a document stored in an S3 bucket is 5 MB.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Object</b>: Identifies an S3 object as the document source. The maximum size of a document stored in an S3 bucket is 5 MB.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Bytes</i> A blob of base-64 encoded documents bytes. The maximum size of a document that's provided in a blob of bytes is 5 MB. The document bytes must be in PNG or JPG format. If you are using an AWS SDK to call Amazon Textract, you might not need to base64-encode image bytes passed using the <code>Bytes</code> field. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> If the bucket has versioning enabled, you can specify the object version. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The file name of the input document. It must be an image file (.JPG or .PNG format). Asynchronous operations also support PDF files.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> The name of the S3 bucket.<br/>
        :return: Blocks:  An array of Block objects containing the text detected in the document.<br/>DocumentMetadata:  Metadata about the document. Contains the number of pages that are detected in the document.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Pages</i> The number of pages detected in the document.<br/>        """
        pass

    def get_document_analysis(self, JobId: str, MaxResults: int=None, NextToken: str=None) -> GetDocumentAnalysisResponse:
        """Gets the results for an Amazon Textract asynchronous operation that analyzes text in a document. You start asynchronous text analysis by calling <a>StartDocumentAnalysis</a>, which returns a job identifier (<code>JobId</code>). When the text analysis operation finishes, Amazon Textract publishes a completion status to the Amazon Simple Notification Service (Amazon SNS) topic that's registered in the initial call to <code>StartDocumentAnalysis</code>. To get the results of the text-detection operation, first check that the status value published to the Amazon SNS topic is <code>SUCCEEDED</code>. If so, call <code>GetDocumentAnalysis</code>, and pass the job identifier (<code>JobId</code>) from the initial call to <code>StartDocumentAnalysis</code>.  <code>GetDocumentAnalysis</code> returns an array of <a>Block</a> objects. The following types of information are returned:  <ul> <li> Words and lines that are related to nearby lines and words. The related information is returned in two <a>Block</a> objects each of type <code>KEY_VALUE_SET</code>: a KEY Block object and a VALUE Block object. For example, <i>Name: Ana Silva Carolina</i> contains a key and value. <i>Name:</i> is the key. <i>Ana Silva Carolina</i> is the value. </li> <li> Table and table cell data. A TABLE Block object contains information about a detected table. A CELL Block object is returned for each cell in a table. </li> <li> Selectable elements such as checkboxes and radio buttons. A SELECTION_ELEMENT Block object contains information about a selectable element. </li> <li> Lines and words of text. A LINE Block object contains one or more WORD Block objects. </li> </ul> Use the <code>MaxResults</code> parameter to limit the number of blocks returned. If there are more results than specified in <code>MaxResults</code>, the value of <code>NextToken</code> in the operation response contains a pagination token for getting the next set of results. To get the next page of results, call <code>GetDocumentAnalysis</code>, and populate the <code>NextToken</code> request parameter with the token value that's returned from the previous call to <code>GetDocumentAnalysis</code>. For more information, see <a href="https://docs.aws.amazon.com/textract/latest/dg/how-it-works-analyzing.html">Document Text Analysis</a>.

        :param self:
        :param NextToken:  If the previous response was incomplete (because there are more blocks to retrieve), Amazon Textract returns a pagination token in the response. You can use this pagination token to retrieve the next set of blocks.<br/>MaxResults:  The maximum number of results to return per paginated call. The largest value that you can specify is 1,000. If you specify a value greater than 1,000, a maximum of 1,000 results is returned. The default value is 1,000.<br/>JobId:  A unique identifier for the text-detection job. The <code>JobId</code> is returned from <code>StartDocumentAnalysis</code>.<br/>
        :return: StatusMessage:  The current status of an asynchronous document analysis operation.<br/>Warnings:  A list of warnings that occurred during the document analysis operation.<br/>Blocks:  The results of the text analysis operation.<br/>NextToken:  If the response is truncated, Amazon Textract returns this token. You can use this token in the subsequent request to retrieve the next set of text detection results.<br/>JobStatus:  The current status of the text detection job.<br/>DocumentMetadata:  Information about a document that Amazon Textract processed. <code>DocumentMetadata</code> is returned in every page of paginated responses from an Amazon Textract video operation.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Pages</i> The number of pages detected in the document.<br/>        """
        pass

    def get_document_text_detection(self, JobId: str, MaxResults: int=None, NextToken: str=None) -> GetDocumentTextDetectionResponse:
        """Gets the results for an Amazon Textract asynchronous operation that detects text in a document. Amazon Textract can detect lines of text and the words that make up a line of text. You start asynchronous text detection by calling <a>StartDocumentTextDetection</a>, which returns a job identifier (<code>JobId</code>). When the text detection operation finishes, Amazon Textract publishes a completion status to the Amazon Simple Notification Service (Amazon SNS) topic that's registered in the initial call to <code>StartDocumentTextDetection</code>. To get the results of the text-detection operation, first check that the status value published to the Amazon SNS topic is <code>SUCCEEDED</code>. If so, call <code>GetDocumentTextDetection</code>, and pass the job identifier (<code>JobId</code>) from the initial call to <code>StartDocumentTextDetection</code>.  <code>GetDocumentTextDetection</code> returns an array of <a>Block</a> objects.  Each document page has as an associated <code>Block</code> of type PAGE. Each PAGE <code>Block</code> object is the parent of LINE <code>Block</code> objects that represent the lines of detected text on a page. A LINE <code>Block</code> object is a parent for each word that makes up the line. Words are represented by <code>Block</code> objects of type WORD. Use the MaxResults parameter to limit the number of blocks that are returned. If there are more results than specified in <code>MaxResults</code>, the value of <code>NextToken</code> in the operation response contains a pagination token for getting the next set of results. To get the next page of results, call <code>GetDocumentTextDetection</code>, and populate the <code>NextToken</code> request parameter with the token value that's returned from the previous call to <code>GetDocumentTextDetection</code>. For more information, see <a href="https://docs.aws.amazon.com/textract/latest/dg/how-it-works-detecting.html">Document Text Detection</a>.

        :param self:
        :param NextToken:  If the previous response was incomplete (because there are more blocks to retrieve), Amazon Textract returns a pagination token in the response. You can use this pagination token to retrieve the next set of blocks.<br/>MaxResults:  The maximum number of results to return per paginated call. The largest value you can specify is 1,000. If you specify a value greater than 1,000, a maximum of 1,000 results is returned. The default value is 1,000.<br/>JobId:  A unique identifier for the text detection job. The <code>JobId</code> is returned from <code>StartDocumentTextDetection</code>.<br/>
        :return: StatusMessage:  The current status of an asynchronous document text-detection operation. <br/>Warnings:  A list of warnings that occurred during the document text-detection operation.<br/>Blocks:  The results of the text-detection operation.<br/>NextToken:  If the response is truncated, Amazon Textract returns this token. You can use this token in the subsequent request to retrieve the next set of text-detection results.<br/>JobStatus:  The current status of the text detection job.<br/>DocumentMetadata:  Information about a document that Amazon Textract processed. <code>DocumentMetadata</code> is returned in every page of paginated responses from an Amazon Textract video operation.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Pages</i> The number of pages detected in the document.<br/>        """
        pass

    def start_document_analysis(self, FeatureTypes: List[FeatureType], DocumentLocation: DocumentLocation, ClientRequestToken: str=None, JobTag: str=None, NotificationChannel: NotificationChannel=None) -> StartDocumentAnalysisResponse:
        """Starts asynchronous analysis of an input document for relationships between detected items such as key and value pairs, tables, and selection elements.  <code>StartDocumentAnalysis</code> can analyze text in documents that are in JPG, PNG, and PDF format. The documents are stored in an Amazon S3 bucket. Use <a>DocumentLocation</a> to specify the bucket name and file name of the document.   <code>StartDocumentAnalysis</code> returns a job identifier (<code>JobId</code>) that you use to get the results of the operation. When text analysis is finished, Amazon Textract publishes a completion status to the Amazon Simple Notification Service (Amazon SNS) topic that you specify in <code>NotificationChannel</code>. To get the results of the text analysis operation, first check that the status value published to the Amazon SNS topic is <code>SUCCEEDED</code>. If so, call <a>GetDocumentAnalysis</a>, and pass the job identifier (<code>JobId</code>) from the initial call to <code>StartDocumentAnalysis</code>. For more information, see <a href="https://docs.aws.amazon.com/textract/latest/dg/how-it-works-analyzing.html">Document Text Analysis</a>.

        :param self:
        :param NotificationChannel:  The Amazon SNS topic ARN that you want Amazon Textract to publish the completion status of the operation to. :<br/>JobTag:  An identifier you specify that's included in the completion notification that's published to the Amazon SNS topic. For example, you can use <code>JobTag</code> to identify the type of document, such as a tax form or a receipt, that the completion notification corresponds to.<br/>ClientRequestToken:  The idempotent token that you use to identify the start request. If you use the same token with multiple <code>StartDocumentAnalysis</code> requests, the same <code>JobId</code> is returned. Use <code>ClientRequestToken</code> to prevent the same job from being accidentally started more than once. <br/>FeatureTypes:  A list of the types of analysis to perform. Add TABLES to the list to return information about the tables that are detected in the input document. Add FORMS to return detected fields and the associated text. To perform both types of analysis, add TABLES and FORMS to <code>FeatureTypes</code>. All selectable elements (<code>SELECTION_ELEMENT</code>) that are detected are returned, whatever the value of <code>FeatureTypes</code>. <br/>DocumentLocation:  The location of the document to be processed.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Object</i> The Amazon S3 bucket that contains the input document.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Object</b>: The Amazon S3 bucket that contains the input document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> If the bucket has versioning enabled, you can specify the object version. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The file name of the input document. It must be an image file (.JPG or .PNG format). Asynchronous operations also support PDF files.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> The name of the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The Amazon Resource Name (ARN) of an IAM role that gives Amazon Textract publishing permissions to the Amazon SNS topic. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SNSTopicArn</i> The Amazon SNS topic that Amazon Textract posts the completion status to.<br/>
        :return: JobId:  The identifier for the document text detection job. Use <code>JobId</code> to identify the job in a subsequent call to <code>GetDocumentAnalysis</code>.<br/>        """
        pass

    def start_document_text_detection(self, DocumentLocation: DocumentLocation, ClientRequestToken: str=None, JobTag: str=None, NotificationChannel: NotificationChannel=None) -> StartDocumentTextDetectionResponse:
        """Starts the asynchronous detection of text in a document. Amazon Textract can detect lines of text and the words that make up a line of text.  <code>StartDocumentTextDetection</code> can analyze text in documents that are in JPG, PNG, and PDF format. The documents are stored in an Amazon S3 bucket. Use <a>DocumentLocation</a> to specify the bucket name and file name of the document.   <code>StartTextDetection</code> returns a job identifier (<code>JobId</code>) that you use to get the results of the operation. When text detection is finished, Amazon Textract publishes a completion status to the Amazon Simple Notification Service (Amazon SNS) topic that you specify in <code>NotificationChannel</code>. To get the results of the text detection operation, first check that the status value published to the Amazon SNS topic is <code>SUCCEEDED</code>. If so, call <a>GetDocumentTextDetection</a>, and pass the job identifier (<code>JobId</code>) from the initial call to <code>StartDocumentTextDetection</code>. For more information, see <a href="https://docs.aws.amazon.com/textract/latest/dg/how-it-works-detecting.html">Document Text Detection</a>.

        :param self:
        :param NotificationChannel:  The Amazon SNS topic ARN that you want Amazon Textract to publish the completion status of the operation to. :<br/>JobTag:  An identifier you specify that's included in the completion notification that's published to the Amazon SNS topic. For example, you can use <code>JobTag</code> to identify the type of document, such as a tax form or a receipt, that the completion notification corresponds to.<br/>ClientRequestToken:  The idempotent token that's used to identify the start request. If you use the same token with multiple <code>StartDocumentTextDetection</code> requests, the same <code>JobId</code> is returned. Use <code>ClientRequestToken</code> to prevent the same job from being accidentally started more than once. <br/>DocumentLocation:  The location of the document to be processed.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>S3Object</i> The Amazon S3 bucket that contains the input document.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>S3Object</b>: The Amazon S3 bucket that contains the input document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> If the bucket has versioning enabled, you can specify the object version. <br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The file name of the input document. It must be an image file (.JPG or .PNG format). Asynchronous operations also support PDF files.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Bucket</i> The name of the S3 bucket.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RoleArn</i> The Amazon Resource Name (ARN) of an IAM role that gives Amazon Textract publishing permissions to the Amazon SNS topic. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SNSTopicArn</i> The Amazon SNS topic that Amazon Textract posts the completion status to.<br/>
        :return: JobId:  The identifier for the document text-detection job. Use <code>JobId</code> to identify the job in a subsequent call to <code>GetDocumentTextDetection</code>.<br/>        """
        pass



class TranscribeService(BaseClient):
    """Operations and objects for transcribing speech to text."""




    class CreateVocabularyRequest(Mapping):
        pass
    
    class CreateVocabularyResponse(Mapping):
        pass
    
    class Phrase(object):
        pass
    
    class DeleteTranscriptionJobRequest(Mapping):
        pass
    
    class DeleteVocabularyRequest(Mapping):
        pass
    
    class GetTranscriptionJobRequest(Mapping):
        pass
    
    class GetTranscriptionJobResponse(Mapping):
        pass
    
    class GetVocabularyRequest(Mapping):
        pass
    
    class GetVocabularyResponse(Mapping):
        pass
    
    class ListTranscriptionJobsRequest(Mapping):
        pass
    
    class ListTranscriptionJobsResponse(Mapping):
        pass
    
    class ListVocabulariesRequest(Mapping):
        pass
    
    class ListVocabulariesResponse(Mapping):
        pass
    
    class StartTranscriptionJobRequest(Mapping):
        pass
    
    class StartTranscriptionJobResponse(Mapping):
        pass
    
    class Media(Mapping):
        pass
    
    class Settings(Mapping):
        pass
    
    class UpdateVocabularyRequest(Mapping):
        pass
    
    class UpdateVocabularyResponse(Mapping):
        pass
    
    def create_vocabulary(self, LanguageCode: str, VocabularyName: str, Phrases: List[Phrase]=None, VocabularyFileUri: str=None) -> CreateVocabularyResponse:
        """Creates a new custom vocabulary that you can use to change the way Amazon Transcribe handles transcription of an audio file. 

        :param self:
        :param VocabularyFileUri:  The S3 location of the text file that contains the definition of the custom vocabulary. The URI must be in the same region as the API endpoint that you are calling. The general form is   <code> https://s3-&lt;aws-region&gt;.amazonaws.com/&lt;bucket-name&gt;/&lt;keyprefix&gt;/&lt;objectkey&gt; </code>  For example:  <code>https://s3-us-east-1.amazonaws.com/examplebucket/vocab.txt</code>  For more information about S3 object names, see <a href="http://docs.aws.amazon.com/AmazonS3/latest/dev/UsingMetadata.html#object-keys">Object Keys</a> in the <i>Amazon S3 Developer Guide</i>. For more information about custom vocabularies, see <a href="http://docs.aws.amazon.com/transcribe/latest/dg/how-it-works.html#how-vocabulary">Custom Vocabularies</a>.<br/>Phrases:  An array of strings that contains the vocabulary entries. <br/>LanguageCode:  The language code of the vocabulary entries.<br/>VocabularyName:  The name of the vocabulary. The name must be unique within an AWS account. The name is case-sensitive.<br/>
        :return: FailureReason:  If the <code>VocabularyState</code> field is <code>FAILED</code>, this field contains information about why the job failed.<br/>LastModifiedTime:  The date and time that the vocabulary was created.<br/>VocabularyState:  The processing state of the vocabulary. When the <code>VocabularyState</code> field contains <code>READY</code> the vocabulary is ready to be used in a <code>StartTranscriptionJob</code> request.<br/>LanguageCode:  The language code of the vocabulary entries.<br/>VocabularyName:  The name of the vocabulary.<br/>        """
        pass

    def delete_transcription_job(self, TranscriptionJobName: str):
        """Deletes a previously submitted transcription job along with any other generated results such as the transcription, models, and so on.

        :param self:
        :param TranscriptionJobName:  The name of the transcription job to be deleted.<br/>
        :return:         """
        pass

    def delete_vocabulary(self, VocabularyName: str):
        """Deletes a vocabulary from Amazon Transcribe. 

        :param self:
        :param VocabularyName:  The name of the vocabulary to delete. <br/>
        :return:         """
        pass

    def get_transcription_job(self, TranscriptionJobName: str) -> GetTranscriptionJobResponse:
        """Returns information about a transcription job. To see the status of the job, check the <code>TranscriptionJobStatus</code> field. If the status is <code>COMPLETED</code>, the job is finished and you can find the results at the location specified in the <code>TranscriptionFileUri</code> field.

        :param self:
        :param TranscriptionJobName:  The name of the job.<br/>
        :return: TranscriptionJob:  An object that contains the results of the transcription job.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Settings</i> Optional settings for the transcription job. Use these settings to turn on speaker recognition, to set the maximum number of speakers that should be identified and to specify a custom vocabulary to use when processing the transcription job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Settings</b>: Optional settings for the transcription job. Use these settings to turn on speaker recognition, to set the maximum number of speakers that should be identified and to specify a custom vocabulary to use when processing the transcription job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureReason</i> If the <code>TranscriptionJobStatus</code> field is <code>FAILED</code>, this field contains information about why the job failed. The <code>FailureReason</code> field can contain one of the following values: <ul> <li>  <code>Unsupported media format</code> - The media format specified in the <code>MediaFormat</code> field of the request isn't valid. See the description of the <code>MediaFormat</code> field for a list of valid values. </li> <li>  <code>The media format provided does not match the detected media format</code> - The media format of the audio file doesn't match the format specified in the <code>MediaFormat</code> field in the request. Check the media format of your media file and make sure that the two values match. </li> <li>  <code>Invalid sample rate for audio file</code> - The sample rate specified in the <code>MediaSampleRateHertz</code> of the request isn't valid. The sample rate must be between 8000 and 48000 Hertz. </li> <li>  <code>The sample rate provided does not match the detected sample rate</code> - The sample rate in the audio file doesn't match the sample rate specified in the <code>MediaSampleRateHertz</code> field in the request. Check the sample rate of your media file and make sure that the two values match. </li> <li>  <code>Invalid file size: file size too large</code> - The size of your audio file is larger than Amazon Transcribe can process. For more information, see <a href="https://docs.aws.amazon.com/transcribe/latest/dg/limits-guidelines.html#limits">Limits</a> in the <i>Amazon Transcribe Developer Guide</i>. </li> <li>  <code>Invalid number of channels: number of channels too large</code> - Your audio contains more channels than Amazon Transcribe is configured to process. To request additional channels, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws_service_limits.html#limits-amazon-transcribe">Amazon Transcribe Limits</a> in the <i>Amazon Web Services General Reference</i>. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompletionTime</i> A timestamp that shows when the job was completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> A timestamp that shows when the job was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Transcript</i> An object that describes the output of the transcription job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Transcript</b>: An object that describes the output of the transcription job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Media</i> An object that describes the input media for the transcription job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Media</b>: An object that describes the input media for the transcription job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MediaFormat</i> The format of the input media file.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MediaSampleRateHertz</i> The sample rate, in Hertz, of the audio track in the input media file. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LanguageCode</i> The language code for the input speech.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TranscriptionJobStatus</i> The status of the transcription job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TranscriptionJobName</i> The name of the transcription job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MediaFileUri</i> The S3 location of the input media file. The URI must be in the same region as the API endpoint that you are calling. The general form is:  <code> https://s3-&lt;aws-region&gt;.amazonaws.com/&lt;bucket-name&gt;/&lt;keyprefix&gt;/&lt;objectkey&gt; </code>  For example:  <code>https://s3-us-east-1.amazonaws.com/examplebucket/example.mp4</code>   <code>https://s3-us-east-1.amazonaws.com/examplebucket/mediadocs/example.mp4</code>  For more information about S3 object names, see <a href="http://docs.aws.amazon.com/AmazonS3/latest/dev/UsingMetadata.html#object-keys">Object Keys</a> in the <i>Amazon S3 Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TranscriptFileUri</i> The location where the transcription is stored. Use this URI to access the transcription. If you specified an S3 bucket in the <code>OutputBucketName</code> field when you created the job, this is the URI of that bucket. If you chose to store the transcription in Amazon Transcribe, this is a shareable URL that provides secure access to that location.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ChannelIdentification</i> Instructs Amazon Transcribe to process each audio channel separately and then merge the transcription output of each channel into a single transcription.  Amazon Transcribe also produces a transcription of each item detected on an audio channel, including the start time and end time of the item and alternative transcriptions of the item including the confidence that Amazon Transcribe has in the transcription. You can't set both <code>ShowSpeakerLabels</code> and <code>ChannelIdentification</code> in the same request. If you set both, your request returns a <code>BadRequestException</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxSpeakerLabels</i> The maximum number of speakers to identify in the input audio. If there are more speakers in the audio than this number, multiple speakers will be identified as a single speaker. If you specify the <code>MaxSpeakerLabels</code> field, you must set the <code>ShowSpeakerLabels</code> field to true.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ShowSpeakerLabels</i> Determines whether the transcription job uses speaker recognition to identify different speakers in the input audio. Speaker recognition labels individual speakers in the audio file. If you set the <code>ShowSpeakerLabels</code> field to true, you must also set the maximum number of speaker labels <code>MaxSpeakerLabels</code> field. You can't set both <code>ShowSpeakerLabels</code> and <code>ChannelIdentification</code> in the same request. If you set both, your request returns a <code>BadRequestException</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VocabularyName</i> The name of a vocabulary to use when processing the transcription job.<br/>        """
        pass

    def get_vocabulary(self, VocabularyName: str) -> GetVocabularyResponse:
        """Gets information about a vocabulary. 

        :param self:
        :param VocabularyName:  The name of the vocabulary to return information about. The name is case-sensitive.<br/>
        :return: DownloadUri:  The S3 location where the vocabulary is stored. Use this URI to get the contents of the vocabulary. The URI is available for a limited time.<br/>FailureReason:  If the <code>VocabularyState</code> field is <code>FAILED</code>, this field contains information about why the job failed.<br/>LastModifiedTime:  The date and time that the vocabulary was last modified.<br/>VocabularyState:  The processing state of the vocabulary.<br/>LanguageCode:  The language code of the vocabulary entries.<br/>VocabularyName:  The name of the vocabulary to return.<br/>        """
        pass

    def list_transcription_jobs(self, Status: str=None, JobNameContains: str=None, NextToken: str=None, MaxResults: int=None) -> ListTranscriptionJobsResponse:
        """Lists transcription jobs with the specified status.

        :param self:
        :param MaxResults:  The maximum number of jobs to return in the response. If there are fewer results in the list, this response contains only the actual results.<br/>NextToken:  If the result of the previous request to <code>ListTranscriptionJobs</code> was truncated, include the <code>NextToken</code> to fetch the next set of jobs.<br/>JobNameContains:  When specified, the jobs returned in the list are limited to jobs whose name contains the specified string.<br/>Status:  When specified, returns only transcription jobs with the specified status. Jobs are ordered by creation date, with the newest jobs returned first. If you don’t specify a status, Amazon Transcribe returns all transcription jobs ordered by creation date. <br/>
        :return: TranscriptionJobSummaries:  A list of objects containing summary information for a transcription job.<br/>NextToken:  The <code>ListTranscriptionJobs</code> operation returns a page of jobs at a time. The maximum size of the page is set by the <code>MaxResults</code> parameter. If there are more jobs in the list than the page size, Amazon Transcribe returns the <code>NextPage</code> token. Include the token in the next request to the <code>ListTranscriptionJobs</code> operation to return in the next page of jobs.<br/>Status:  The requested status of the jobs returned.<br/>        """
        pass

    def list_vocabularies(self, NextToken: str=None, MaxResults: int=None, StateEquals: str=None, NameContains: str=None) -> ListVocabulariesResponse:
        """Returns a list of vocabularies that match the specified criteria. If no criteria are specified, returns the entire list of vocabularies.

        :param self:
        :param NameContains:  When specified, the vocabularies returned in the list are limited to vocabularies whose name contains the specified string. The search is case-insensitive, <code>ListVocabularies</code> will return both "vocabularyname" and "VocabularyName" in the response list.<br/>StateEquals:  When specified, only returns vocabularies with the <code>VocabularyState</code> field equal to the specified state.<br/>MaxResults:  The maximum number of vocabularies to return in the response. If there are fewer results in the list, this response contains only the actual results.<br/>NextToken:  If the result of the previous request to <code>ListVocabularies</code> was truncated, include the <code>NextToken</code> to fetch the next set of jobs.<br/>
        :return: Vocabularies:  A list of objects that describe the vocabularies that match the search criteria in the request.<br/>NextToken:  The <code>ListVocabularies</code> operation returns a page of vocabularies at a time. The maximum size of the page is set by the <code>MaxResults</code> parameter. If there are more jobs in the list than the page size, Amazon Transcribe returns the <code>NextPage</code> token. Include the token in the next request to the <code>ListVocabularies</code> operation to return in the next page of jobs.<br/>Status:  The requested vocabulary state.<br/>        """
        pass

    def start_transcription_job(self, Media: Media, MediaFormat: str, LanguageCode: str, TranscriptionJobName: str, MediaSampleRateHertz: int=None, OutputBucketName: str=None, Settings: Settings=None) -> StartTranscriptionJobResponse:
        """Starts an asynchronous job to transcribe speech to text. 

        :param self:
        :param Settings:  A <code>Settings</code> object that provides optional settings for a transcription job.:<br/>OutputBucketName:  The location where the transcription is stored. If you set the <code>OutputBucketName</code>, Amazon Transcribe puts the transcription in the specified S3 bucket. When you call the <a>GetTranscriptionJob</a> operation, the operation returns this location in the <code>TranscriptFileUri</code> field. The S3 bucket must have permissions that allow Amazon Transcribe to put files in the bucket. For more information, see <a href="https://docs.aws.amazon.com/transcribe/latest/dg/access-control-managing-permissions.html#auth-role-iam-user">Permissions Required for IAM User Roles</a>. Amazon Transcribe uses the default Amazon S3 key for server-side encryption of transcripts that are placed in your S3 bucket. You can't specify your own encryption key. If you don't set the <code>OutputBucketName</code>, Amazon Transcribe generates a pre-signed URL, a shareable URL that provides secure access to your transcription, and returns it in the <code>TranscriptFileUri</code> field. Use this URL to download the transcription.<br/>Media:  An object that describes the input media for a transcription job.:<br/>MediaFormat:  The format of the input media file.<br/>MediaSampleRateHertz:  The sample rate, in Hertz, of the audio track in the input media file. <br/>LanguageCode:  The language code for the language used in the input media file.<br/>TranscriptionJobName:  The name of the job. Note that you can't use the strings "." or ".." by themselves as the job name. The name must also be unique within an AWS account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MediaFileUri</i> The S3 location of the input media file. The URI must be in the same region as the API endpoint that you are calling. The general form is:  <code> https://s3-&lt;aws-region&gt;.amazonaws.com/&lt;bucket-name&gt;/&lt;keyprefix&gt;/&lt;objectkey&gt; </code>  For example:  <code>https://s3-us-east-1.amazonaws.com/examplebucket/example.mp4</code>   <code>https://s3-us-east-1.amazonaws.com/examplebucket/mediadocs/example.mp4</code>  For more information about S3 object names, see <a href="http://docs.aws.amazon.com/AmazonS3/latest/dev/UsingMetadata.html#object-keys">Object Keys</a> in the <i>Amazon S3 Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ChannelIdentification</i> Instructs Amazon Transcribe to process each audio channel separately and then merge the transcription output of each channel into a single transcription.  Amazon Transcribe also produces a transcription of each item detected on an audio channel, including the start time and end time of the item and alternative transcriptions of the item including the confidence that Amazon Transcribe has in the transcription. You can't set both <code>ShowSpeakerLabels</code> and <code>ChannelIdentification</code> in the same request. If you set both, your request returns a <code>BadRequestException</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxSpeakerLabels</i> The maximum number of speakers to identify in the input audio. If there are more speakers in the audio than this number, multiple speakers will be identified as a single speaker. If you specify the <code>MaxSpeakerLabels</code> field, you must set the <code>ShowSpeakerLabels</code> field to true.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ShowSpeakerLabels</i> Determines whether the transcription job uses speaker recognition to identify different speakers in the input audio. Speaker recognition labels individual speakers in the audio file. If you set the <code>ShowSpeakerLabels</code> field to true, you must also set the maximum number of speaker labels <code>MaxSpeakerLabels</code> field. You can't set both <code>ShowSpeakerLabels</code> and <code>ChannelIdentification</code> in the same request. If you set both, your request returns a <code>BadRequestException</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VocabularyName</i> The name of a vocabulary to use when processing the transcription job.<br/>
        :return: TranscriptionJob:  An object containing details of the asynchronous transcription job.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Settings</i> Optional settings for the transcription job. Use these settings to turn on speaker recognition, to set the maximum number of speakers that should be identified and to specify a custom vocabulary to use when processing the transcription job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Settings</b>: Optional settings for the transcription job. Use these settings to turn on speaker recognition, to set the maximum number of speakers that should be identified and to specify a custom vocabulary to use when processing the transcription job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FailureReason</i> If the <code>TranscriptionJobStatus</code> field is <code>FAILED</code>, this field contains information about why the job failed. The <code>FailureReason</code> field can contain one of the following values: <ul> <li>  <code>Unsupported media format</code> - The media format specified in the <code>MediaFormat</code> field of the request isn't valid. See the description of the <code>MediaFormat</code> field for a list of valid values. </li> <li>  <code>The media format provided does not match the detected media format</code> - The media format of the audio file doesn't match the format specified in the <code>MediaFormat</code> field in the request. Check the media format of your media file and make sure that the two values match. </li> <li>  <code>Invalid sample rate for audio file</code> - The sample rate specified in the <code>MediaSampleRateHertz</code> of the request isn't valid. The sample rate must be between 8000 and 48000 Hertz. </li> <li>  <code>The sample rate provided does not match the detected sample rate</code> - The sample rate in the audio file doesn't match the sample rate specified in the <code>MediaSampleRateHertz</code> field in the request. Check the sample rate of your media file and make sure that the two values match. </li> <li>  <code>Invalid file size: file size too large</code> - The size of your audio file is larger than Amazon Transcribe can process. For more information, see <a href="https://docs.aws.amazon.com/transcribe/latest/dg/limits-guidelines.html#limits">Limits</a> in the <i>Amazon Transcribe Developer Guide</i>. </li> <li>  <code>Invalid number of channels: number of channels too large</code> - Your audio contains more channels than Amazon Transcribe is configured to process. To request additional channels, see <a href="https://docs.aws.amazon.com/general/latest/gr/aws_service_limits.html#limits-amazon-transcribe">Amazon Transcribe Limits</a> in the <i>Amazon Web Services General Reference</i>. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CompletionTime</i> A timestamp that shows when the job was completed.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreationTime</i> A timestamp that shows when the job was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Transcript</i> An object that describes the output of the transcription job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Transcript</b>: An object that describes the output of the transcription job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Media</i> An object that describes the input media for the transcription job.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Media</b>: An object that describes the input media for the transcription job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MediaFormat</i> The format of the input media file.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MediaSampleRateHertz</i> The sample rate, in Hertz, of the audio track in the input media file. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LanguageCode</i> The language code for the input speech.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TranscriptionJobStatus</i> The status of the transcription job.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TranscriptionJobName</i> The name of the transcription job.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MediaFileUri</i> The S3 location of the input media file. The URI must be in the same region as the API endpoint that you are calling. The general form is:  <code> https://s3-&lt;aws-region&gt;.amazonaws.com/&lt;bucket-name&gt;/&lt;keyprefix&gt;/&lt;objectkey&gt; </code>  For example:  <code>https://s3-us-east-1.amazonaws.com/examplebucket/example.mp4</code>   <code>https://s3-us-east-1.amazonaws.com/examplebucket/mediadocs/example.mp4</code>  For more information about S3 object names, see <a href="http://docs.aws.amazon.com/AmazonS3/latest/dev/UsingMetadata.html#object-keys">Object Keys</a> in the <i>Amazon S3 Developer Guide</i>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TranscriptFileUri</i> The location where the transcription is stored. Use this URI to access the transcription. If you specified an S3 bucket in the <code>OutputBucketName</code> field when you created the job, this is the URI of that bucket. If you chose to store the transcription in Amazon Transcribe, this is a shareable URL that provides secure access to that location.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ChannelIdentification</i> Instructs Amazon Transcribe to process each audio channel separately and then merge the transcription output of each channel into a single transcription.  Amazon Transcribe also produces a transcription of each item detected on an audio channel, including the start time and end time of the item and alternative transcriptions of the item including the confidence that Amazon Transcribe has in the transcription. You can't set both <code>ShowSpeakerLabels</code> and <code>ChannelIdentification</code> in the same request. If you set both, your request returns a <code>BadRequestException</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>MaxSpeakerLabels</i> The maximum number of speakers to identify in the input audio. If there are more speakers in the audio than this number, multiple speakers will be identified as a single speaker. If you specify the <code>MaxSpeakerLabels</code> field, you must set the <code>ShowSpeakerLabels</code> field to true.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ShowSpeakerLabels</i> Determines whether the transcription job uses speaker recognition to identify different speakers in the input audio. Speaker recognition labels individual speakers in the audio file. If you set the <code>ShowSpeakerLabels</code> field to true, you must also set the maximum number of speaker labels <code>MaxSpeakerLabels</code> field. You can't set both <code>ShowSpeakerLabels</code> and <code>ChannelIdentification</code> in the same request. If you set both, your request returns a <code>BadRequestException</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VocabularyName</i> The name of a vocabulary to use when processing the transcription job.<br/>        """
        pass

    def update_vocabulary(self, LanguageCode: str, VocabularyName: str, Phrases: List[Phrase]=None, VocabularyFileUri: str=None) -> UpdateVocabularyResponse:
        """Updates an existing vocabulary with new values. The <code>UpdateVocabulary</code> operation overwrites all of the existing information with the values that you provide in the request. 

        :param self:
        :param VocabularyFileUri:  The S3 location of the text file that contains the definition of the custom vocabulary. The URI must be in the same region as the API endpoint that you are calling. The general form is   <code> https://s3-&lt;aws-region&gt;.amazonaws.com/&lt;bucket-name&gt;/&lt;keyprefix&gt;/&lt;objectkey&gt; </code>  For example:  <code>https://s3-us-east-1.amazonaws.com/examplebucket/vocab.txt</code>  For more information about S3 object names, see <a href="http://docs.aws.amazon.com/AmazonS3/latest/dev/UsingMetadata.html#object-keys">Object Keys</a> in the <i>Amazon S3 Developer Guide</i>. For more information about custom vocabularies, see <a href="http://docs.aws.amazon.com/transcribe/latest/dg/how-it-works.html#how-vocabulary">Custom Vocabularies</a>.<br/>Phrases:  An array of strings containing the vocabulary entries.<br/>LanguageCode:  The language code of the vocabulary entries.<br/>VocabularyName:  The name of the vocabulary to update. The name is case-sensitive.<br/>
        :return: VocabularyState:  The processing state of the vocabulary. When the <code>VocabularyState</code> field contains <code>READY</code> the vocabulary is ready to be used in a <code>StartTranscriptionJob</code> request.<br/>LastModifiedTime:  The date and time that the vocabulary was updated.<br/>LanguageCode:  The language code of the vocabulary entries.<br/>VocabularyName:  The name of the vocabulary that was updated.<br/>        """
        pass



class Transfer(BaseClient):
    """AWS Transfer for SFTP is a fully managed service that enables the transfer of files directly into and out of Amazon S3 using the Secure File Transfer Protocol (SFTP)—also known as Secure Shell (SSH) File Transfer Protocol. AWS helps you seamlessly migrate your file transfer workflows to AWS Transfer for SFTP—by integrating with existing authentication systems, and providing DNS routing with Amazon Route 53—so nothing changes for your customers and partners, or their applications. With your data in S3, you can use it with AWS services for processing, analytics, machine learning, and archiving. Getting started with AWS Transfer for SFTP (AWS SFTP) is easy; there is no infrastructure to buy and setup. """


    class ListServersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_servers` for valid parameters"""
            pass


    class CreateServerRequest(Mapping):
        pass
    
    class CreateServerResponse(Mapping):
        pass
    
    class EndpointDetails(Mapping):
        pass
    
    class IdentityProviderDetails(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreateUserRequest(Mapping):
        pass
    
    class CreateUserResponse(Mapping):
        pass
    
    class DeleteServerRequest(Mapping):
        pass
    
    class DeleteSshPublicKeyRequest(Mapping):
        pass
    
    class DeleteUserRequest(Mapping):
        pass
    
    class DescribeServerRequest(Mapping):
        pass
    
    class DescribeServerResponse(Mapping):
        pass
    
    class DescribeUserRequest(Mapping):
        pass
    
    class DescribeUserResponse(Mapping):
        pass
    
    class ImportSshPublicKeyRequest(Mapping):
        pass
    
    class ImportSshPublicKeyResponse(Mapping):
        pass
    
    class ListServersRequest(Mapping):
        pass
    
    class ListServersResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class ListUsersRequest(Mapping):
        pass
    
    class ListUsersResponse(Mapping):
        pass
    
    class StartServerRequest(Mapping):
        pass
    
    class StopServerRequest(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TestIdentityProviderRequest(Mapping):
        pass
    
    class TestIdentityProviderResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateServerRequest(Mapping):
        pass
    
    class UpdateServerResponse(Mapping):
        pass
    
    class UpdateUserRequest(Mapping):
        pass
    
    class UpdateUserResponse(Mapping):
        pass
    
    def create_server(self, EndpointDetails: EndpointDetails=None, EndpointType: str=None, HostKey: str=None, IdentityProviderDetails: IdentityProviderDetails=None, IdentityProviderType: str=None, LoggingRole: str=None, Tags: List[Tag]=None) -> CreateServerResponse:
        """Instantiates an autoscaling virtual server based on Secure File Transfer Protocol (SFTP) in AWS. The call returns the <code>ServerId</code> property assigned by the service to the newly created server. Reference this <code>ServerId</code> property when you make updates to your server, or work with users. The response returns the <code>ServerId</code> value for the newly created server.

        :param self:
        :param Tags:  Key-value pairs that can be used to group and search for servers.<br/>LoggingRole:  A value that allows the service to write your SFTP users' activity to your Amazon CloudWatch logs for monitoring and auditing purposes.<br/>IdentityProviderType:  The mode of authentication enabled for this service. The default value is <code>SERVICE_MANAGED</code>, which allows you to store and access SFTP user credentials within the service. An <code>IdentityProviderType</code> value of <code>API_GATEWAY</code> indicates that user authentication requires a call to an API Gateway endpoint URL provided by you to integrate an identity provider of your choice.<br/>IdentityProviderDetails:  An array containing all of the information required to call a customer-supplied authentication API. This parameter is not required when the <code>IdentityProviderType</code> value of server that is created uses the <code>SERVICE_MANAGED</code> authentication method.:<br/>HostKey:  The RSA private key as generated by <code>ssh-keygen -N "" -f my-new-server-key</code> command. <important> If you aren't planning to migrate existing users from an existing SFTP server to a new AWS SFTP server, don't update the host key. Accidentally changing a server's host key can be disruptive. For more information, see <a>change-host-key</a> in the <i>AWS SFTP User Guide.</i>  </important><br/>EndpointType:  The type of VPC endpoint that you want your SFTP server connect to. If you connect to a VPC endpoint, your SFTP server isn't accessible over the public internet.<br/>EndpointDetails:  The virtual private cloud (VPC) endpoint settings that you want to configure for your SFTP server.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcEndpointId</i> The ID of the VPC endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InvocationRole</i> The <code>Role</code> parameter provides the type of <code>InvocationRole</code> used to authenticate the user account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Url</i> The <code>IdentityProviderDetail</code> parameter contains the location of the service endpoint used to authenticate users.<br/>
        :return: ServerId:  The service-assigned ID of the SFTP server that is created.<br/>        """
        pass

    def create_user(self, UserName: str, ServerId: str, Role: str, HomeDirectory: str=None, Policy: str=None, SshPublicKeyBody: str=None, Tags: List[Tag]=None) -> CreateUserResponse:
        """Adds a user and associate them with an existing Secure File Transfer Protocol (SFTP) server. Using parameters for <code>CreateUser</code>, you can specify the user name, set the home directory, store the user's public key, and assign the user's AWS Identity and Access Management (IAM) role. You can also optionally add a scope-down policy, and assign metadata with tags that can be used to group and search for users. The response returns the <code>UserName</code> and <code>ServerId</code> values of the new user for that server.

        :param self:
        :param UserName:  A unique string that identifies a user and is associated with a server as specified by the <code>ServerId</code>. This user name must be a minimum of 3 and a maximum of 32 characters long. The following are valid characters: a-z, A-Z, 0-9, underscore, and hyphen. The user name can't start with a hyphen.<br/>Tags:  Key-value pairs that can be used to group and search for users. Tags are metadata attached to users for any purpose.<br/>SshPublicKeyBody:  The public portion of the Secure Shall (SSH) key used to authenticate the user to the SFTP server.<br/>ServerId:  A system-assigned unique identifier for an SFTP server instance. This is the specific SFTP server that you added your user to.<br/>Role:  The IAM role that controls your user's access to your Amazon S3 bucket. The policies attached to this role will determine the level of access you want to provide your users when transferring files into and out of your Amazon S3 bucket or buckets. The IAM role should also contain a trust relationship that allows the SFTP server to access your resources when servicing your SFTP user's transfer requests.<br/>Policy:  A scope-down policy for your user so you can use the same IAM role across multiple users. This policy scopes down user access to portions of their Amazon S3 bucket. Variables you can use inside this policy include <code>${Transfer:UserName}</code>, <code>${Transfer:HomeDirectory}</code>, and <code>${Transfer:HomeBucket}</code>.<br/>HomeDirectory:  The landing directory (folder) for a user when they log in to the server using their SFTP client. An example is <code>/home/<i>username</i> </code>.<br/>
        :return: UserName:  A unique string that identifies a user account associated with an SFTP server.<br/>ServerId:  The ID of the SFTP server that the user is attached to.<br/>        """
        pass

    def delete_server(self, ServerId: str):
        """Deletes the Secure File Transfer Protocol (SFTP) server that you specify. If you used <code>SERVICE_MANAGED</code> as your <code>IdentityProviderType</code>, you need to delete all users associated with this server before deleting the server itself No response returns from this call.

        :param self:
        :param ServerId:  A unique system-assigned identifier for an SFTP server instance.<br/>
        :return:         """
        pass

    def delete_ssh_public_key(self, UserName: str, SshPublicKeyId: str, ServerId: str):
        """Deletes a user's Secure Shell (SSH) public key. No response is returned from this call.

        :param self:
        :param UserName:  A unique string that identifies a user whose public key is being deleted.<br/>SshPublicKeyId:  A unique identifier used to reference your user's specific SSH key.<br/>ServerId:  A system-assigned unique identifier for a Secure File Transfer Protocol (SFTP) server instance that has the user assigned to it.<br/>
        :return:         """
        pass

    def delete_user(self, UserName: str, ServerId: str):
        """Deletes the user belonging to the server you specify. No response returns from this call. <note> When you delete a user from a server, the user's information is lost. </note>

        :param self:
        :param UserName:  A unique string that identifies a user that is being deleted from the server.<br/>ServerId:  A system-assigned unique identifier for an SFTP server instance that has the user assigned to it.<br/>
        :return:         """
        pass

    def describe_server(self, ServerId: str) -> DescribeServerResponse:
        """Describes the server that you specify by passing the <code>ServerId</code> parameter. The response contains a description of the server's properties.

        :param self:
        :param ServerId:  A system-assigned unique identifier for an SFTP server.<br/>
        :return: Server:  An array containing the properties of the server with the <code>ServerID</code> you specified.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserCount</i> The number of users that are assigned to the SFTP server you specified with the <code>ServerId</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> This property contains the key-value pairs that you can use to search for and group servers that were assigned to the server that was described.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>State</i> The condition of the SFTP server for the server that was described. A value of <code>ONLINE</code> indicates that the server can accept jobs and transfer files. A <code>State</code> value of <code>OFFLINE</code> means that the server cannot perform file transfer operations. The states of <code>STARTING</code> and <code>STOPPING</code> indicated that the server is in an intermediate state, either not fully able to respond, or not fully offline. The values of <code>START_FAILED</code> or <code>STOP_FAILED</code> can indicate an error condition.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServerId</i> This property is a unique system assigned identifier for the SFTP server that you instantiate.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LoggingRole</i> This property is an AWS Identity and Access Management (IAM) entity that allows the server to turn on Amazon CloudWatch logging for Amazon S3 events. When set, user activity can be view in your CloudWatch logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IdentityProviderType</i> This property defines the mode of authentication method enabled for this service. A value of <code>SERVICE_MANAGED</code>, means that you are using this Server to store and access SFTP user credentials within the service. A value of <code>API_GATEWAY</code> indicates that you have integrated an API Gateway endpoint that will be invoked for authenticating your user into the service.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IdentityProviderDetails</i> Specifies information to call a customer-supplied authentication API. This field is not populated when the <code>IdentityProviderType</code> of the server is <code>SERVICE_MANAGED</code>&gt;.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>IdentityProviderDetails</b>: Specifies information to call a customer-supplied authentication API. This field is not populated when the <code>IdentityProviderType</code> of the server is <code>SERVICE_MANAGED</code>&gt;.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HostKeyFingerprint</i> This value contains the Message-Digest Algorithm (MD5) hash of the server's host key. This value is equivalent to the output of <code>ssh-keygen -l -E md5 -f my-new-server-key</code> command.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointType</i> The type of endpoint that your SFTP server is connected to. If your SFTP server is connected to a VPC endpoint, your server isn't accessible over the public internet.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndpointDetails</i> The virtual private cloud (VPC) endpoint settings that you configured for your SFTP server.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EndpointDetails</b>: The virtual private cloud (VPC) endpoint settings that you configured for your SFTP server.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> Specifies the unique Amazon Resource Name (ARN) for the server to be described.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcEndpointId</i> The ID of the VPC endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>InvocationRole</i> The <code>Role</code> parameter provides the type of <code>InvocationRole</code> used to authenticate the user account.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Url</i> The <code>IdentityProviderDetail</code> parameter contains the location of the service endpoint used to authenticate users.<br/>        """
        pass

    def describe_user(self, UserName: str, ServerId: str) -> DescribeUserResponse:
        """Describes the user assigned to a specific server, as identified by its <code>ServerId</code> property. The response from this call returns the properties of the user associated with the <code>ServerId</code> value that was specified.

        :param self:
        :param UserName:  The name of the user assigned to one or more servers. User names are part of the sign-in credentials to use the AWS Transfer service and perform file transfer tasks.<br/>ServerId:  A system-assigned unique identifier for an SFTP server that has this user assigned.<br/>
        :return: User:  An array containing the properties of the user account for the <code>ServerID</code> value that you specified.:<br/>ServerId:  A system-assigned unique identifier for an SFTP server that has this user assigned.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserName</i> This property is the name of the user that was requested to be described. User names are used for authentication purposes. This is the string that will be used by your user when they log in to your SFTP server.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Tags</i> This property contains the key-value pairs for the user requested. Tag can be used to search for and group users for a variety of purposes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SshPublicKeys</i> This property contains the public key portion of the Secure Shell (SSH) keys stored for the described user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Role</i> This property specifies the IAM role that controls your user's access to your Amazon S3 bucket. The policies attached to this role will determine the level of access you want to provide your users when transferring files into and out of your Amazon S3 bucket or buckets. The IAM role should also contain a trust relationship that allows the SFTP server to access your resources when servicing your SFTP user's transfer requests.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Policy</i> Specifies the name of the policy in use for the described user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HomeDirectory</i> This property specifies the landing directory (or folder) which is the location that files are written to or read from in an Amazon S3 bucket for the described user. An example would be: <code>/<i>bucket_name</i>/home/<i>username</i> </code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i> This property contains the unique Amazon Resource Name (ARN) for the user that was requested to be described.<br/>        """
        pass

    def import_ssh_public_key(self, UserName: str, SshPublicKeyBody: str, ServerId: str) -> ImportSshPublicKeyResponse:
        """Adds a Secure Shell (SSH) public key to a user account identified by a <code>UserName</code> value assigned to a specific server, identified by <code>ServerId</code>. The response returns the <code>UserName</code> value, the <code>ServerId</code> value, and the name of the <code>SshPublicKeyId</code>.

        :param self:
        :param UserName:  The name of the user account that is assigned to one or more servers.<br/>SshPublicKeyBody:  The public key portion of an SSH key pair.<br/>ServerId:  A system-assigned unique identifier for an SFTP server.<br/>
        :return: UserName:  A user name assigned to the <code>ServerID</code> value that you specified.<br/>SshPublicKeyId:  This identifier is the name given to a public key by the system that was imported.<br/>ServerId:  A system-assigned unique identifier for an SFTP server.<br/>        """
        pass

    def list_servers(self, MaxResults: int=None, NextToken: str=None) -> ListServersResponse:
        """Lists the Secure File Transfer Protocol (SFTP) servers that are associated with your AWS account.

        :param self:
        :param NextToken:  When additional results are obtained from the ListServers command, a <code>NextToken</code> parameter is returned in the output. You can then pass the <code>NextToken</code> parameter in a subsequent command to continue listing additional servers.<br/>MaxResults:  Specifies the number of servers to return as a response to the <code>ListServers</code> query.<br/>
        :return: Servers:  An array of servers that were listed.<br/>NextToken:  When you can get additional results from the <code>ListServers</code> operation, a <code>NextToken</code> parameter is returned in the output. In a following command, you can pass in the <code>NextToken</code> parameter to continue listing additional servers.<br/>        """
        pass

    def list_tags_for_resource(self, Arn: str, MaxResults: int=None, NextToken: str=None) -> ListTagsForResourceResponse:
        """Lists all of the tags associated with the Amazon Resource Number (ARN) you specify. The resource can be a user, server, or role.

        :param self:
        :param NextToken:  When you request additional results from the <code>ListTagsForResource</code> call, a <code>NextToken</code> parameter is returned in the input. You can then pass in a subsequent command the <code>NextToken</code> parameter to continue listing additional tags.<br/>MaxResults:  Specifies the number of tags to return as a response to the <code>ListTagsForResource</code> request.<br/>Arn:  Requests the tags associated with a particular Amazon Resource Name (ARN). An ARN is an identifier for a specific AWS resource, such as a server, user, or role.<br/>
        :return: Tags:  Key-value pairs that are assigned to a resource, usually for the purpose of grouping and searching for items. Tags are metadata that you define that you can use for any purpose.<br/>NextToken:  When you can get additional results from the <code>ListTagsForResource</code> call, a <code>NextToken</code> parameter is returned in the output. You can then pass in a subsequent command the <code>NextToken</code> parameter to continue listing additional tags.<br/>Arn:  This value is the ARN you specified to list the tags of.<br/>        """
        pass

    def list_users(self, ServerId: str, MaxResults: int=None, NextToken: str=None) -> ListUsersResponse:
        """Lists the users for the server that you specify by passing the <code>ServerId</code> parameter.

        :param self:
        :param ServerId:  A system-assigned unique identifier for a Secure File Transfer Protocol (SFTP) server that has users are assigned to it.<br/>NextToken:  When you can get additional results from the <code>ListUsers</code> call, a <code>NextToken</code> parameter is returned in the output. You can then pass in a subsequent command the <code>NextToken</code> parameter to continue listing additional users.<br/>MaxResults:  Specifies the number of users to return as a response to the <code>ListUsers</code> request.<br/>
        :return: Users:  Returns the user accounts and their properties for the <code>ServerId</code> value that you specify.<br/>ServerId:  A system-assigned unique identifier for an SFTP server that the users are assigned to.<br/>NextToken:  When you can get additional results from the <code>ListUsers</code> call, a <code>NextToken</code> parameter is returned in the output. You can then pass in a subsequent command the <code>NextToken</code> parameter to continue listing additional users.<br/>        """
        pass

    def start_server(self, ServerId: str):
        """Changes the state of a Secure File Transfer Protocol (SFTP) server from <code>OFFLINE</code> to <code>ONLINE</code>. It has no impact on an SFTP server that is already <code>ONLINE</code>. An <code>ONLINE</code> server can accept and process file transfer jobs. The state of <code>STARTING</code> indicates that the server is in an intermediate state, either not fully able to respond, or not fully online. The values of <code>START_FAILED</code> can indicate an error condition.  No response is returned from this call.

        :param self:
        :param ServerId:  A system-assigned unique identifier for an SFTP server that you start.<br/>
        :return:         """
        pass

    def stop_server(self, ServerId: str):
        """Changes the state of an SFTP server from <code>ONLINE</code> to <code>OFFLINE</code>. An <code>OFFLINE</code> server cannot accept and process file transfer jobs. Information tied to your server such as server and user properties are not affected by stopping your server. Stopping a server will not reduce or impact your Secure File Transfer Protocol (SFTP) endpoint billing. The states of <code>STOPPING</code> indicates that the server is in an intermediate state, either not fully able to respond, or not fully offline. The values of <code>STOP_FAILED</code> can indicate an error condition. No response is returned from this call.

        :param self:
        :param ServerId:  A system-assigned unique identifier for an SFTP server that you stopped.<br/>
        :return:         """
        pass

    def tag_resource(self, Tags: List[Tag], Arn: str):
        """Attaches a key-value pair to a resource, as identified by its Amazon Resource Name (ARN). Resources are users, servers, roles, and other entities. There is no response returned from this call.

        :param self:
        :param Tags:  Key-value pairs assigned to ARNs that you can use to group and search for resources by type. You can attach this metadata to user accounts for any purpose.<br/>Arn:  An Amazon Resource Name (ARN) for a specific AWS resource, such as a server, user, or role.<br/>
        :return:         """
        pass

    def test_identity_provider(self, UserName: str, ServerId: str, UserPassword: str=None) -> TestIdentityProviderResponse:
        """If the <code>IdentityProviderType</code> of the server is <code>API_Gateway</code>, tests whether your API Gateway is set up successfully. We highly recommend that you call this method to test your authentication method as soon as you create your server. By doing so, you can troubleshoot issues with the API Gateway integration to ensure that your users can successfully use the service.

        :param self:
        :param UserPassword:  The password of the user account to be tested.<br/>UserName:  This request parameter is name of the user account to be tested.<br/>ServerId:  A system assigned identifier for a specific server. That server's user authentication method is tested with a user name and password.<br/>
        :return: Url:  The endpoint of the service used to authenticate a user.<br/>StatusCode:  The HTTP status code that is the response from your API Gateway.<br/>Message:  The result of the authorization test as a message. <br/>        """
        pass

    def untag_resource(self, TagKeys: List[TagKey], Arn: str):
        """Detaches a key-value pair from a resource, as identified by its Amazon Resource Name (ARN). Resources are users, servers, roles, and other entities. No response is returned from this call.

        :param self:
        :param TagKeys:  TagKeys are key-value pairs assigned to ARNs that can be used to group and search for resources by type. This metadata can be attached to resources for any purpose.<br/>Arn:  This is the value of the resource that will have the tag removed. An Amazon Resource Name (ARN) is an identifier for a specific AWS resource, such as a server, user, or role.<br/>
        :return:         """
        pass

    def update_server(self, ServerId: str, EndpointDetails: EndpointDetails=None, EndpointType: str=None, HostKey: str=None, IdentityProviderDetails: IdentityProviderDetails=None, LoggingRole: str=None) -> UpdateServerResponse:
        """Updates the server properties after that server has been created. The <code>UpdateServer</code> call returns the <code>ServerId</code> of the Secure File Transfer Protocol (SFTP) server you updated.

        :param self:
        :param ServerId:  A system-assigned unique identifier for an SFTP server instance that the user account is assigned to.<br/>LoggingRole:  A value that changes the AWS Identity and Access Management (IAM) role that allows Amazon S3 events to be logged in Amazon CloudWatch, turning logging on or off.<br/>IdentityProviderDetails:  This response parameter is an array containing all of the information required to call a customer's authentication API method.:<br/>HostKey:  The RSA private key as generated by <code>ssh-keygen -N "" -f my-new-server-key</code>. <important> If you aren't planning to migrate existing users from an existing SFTP server to a new AWS SFTP server, don't update the host key. Accidentally changing a server's host key can be disruptive. For more information, see <a>change-host-key</a> in the <i>AWS SFTP User Guide.</i>  </important><br/>EndpointType:  The type of endpoint that you want your SFTP server to connect to. You can choose to connect to the public internet or a virtual private cloud (VPC) endpoint. With a VPC endpoint, your SFTP server isn't accessible over the public internet. <br/>EndpointDetails:  The virtual private cloud (VPC) endpoint settings that are configured for your SFTP server. With a VPC endpoint, your SFTP server isn't accessible over the public internet.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>VpcEndpointId</i> The ID of the VPC endpoint.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>InvocationRole</i> The <code>Role</code> parameter provides the type of <code>InvocationRole</code> used to authenticate the user account.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Url</i> The <code>IdentityProviderDetail</code> parameter contains the location of the service endpoint used to authenticate users.<br/>
        :return: ServerId:  A system-assigned unique identifier for an SFTP server that the user account is assigned to.<br/>        """
        pass

    def update_user(self, UserName: str, ServerId: str, HomeDirectory: str=None, Policy: str=None, Role: str=None) -> UpdateUserResponse:
        """Assigns new properties to a user. Parameters you pass modify any or all of the following: the home directory, role, and policy for the <code>UserName</code> and <code>ServerId</code> you specify. The response returns the <code>ServerId</code> and the <code>UserName</code> for the updated user.

        :param self:
        :param UserName:  A unique string that identifies a user and is associated with a server as specified by the ServerId. This is the string that will be used by your user when they log in to your SFTP server. This user name is a minimum of 3 and a maximum of 32 characters long. The following are valid characters: a-z, A-Z, 0-9, underscore, and hyphen. The user name can't start with a hyphen.<br/>ServerId:  A system-assigned unique identifier for an SFTP server instance that the user account is assigned to.<br/>Role:  The IAM role that controls your user's access to your Amazon S3 bucket. The policies attached to this role will determine the level of access you want to provide your users when transferring files into and out of your Amazon S3 bucket or buckets. The IAM role should also contain a trust relationship that allows the Secure File Transfer Protocol (SFTP) server to access your resources when servicing your SFTP user's transfer requests.<br/>Policy:  Allows you to supply a scope-down policy for your user so you can use the same AWS Identity and Access Management (IAM) role across multiple users. The policy scopes down users access to portions of your Amazon S3 bucket. Variables you can use inside this policy include <code>${Transfer:UserName}</code>, <code>${Transfer:HomeDirectory}</code>, and <code>${Transfer:HomeBucket}</code>.<br/>HomeDirectory:  The HomeDirectory parameter specifies the landing directory (folder) for a user when they log in to the server using their client. An example would be: <code>/home/<i>username</i> </code>.<br/>
        :return: UserName:  The unique identifier for a user that is assigned to the SFTP server instance that was specified in the request.<br/>ServerId:  A system-assigned unique identifier for an SFTP server instance that the user account is assigned to.<br/>        """
        pass



class Translate(BaseClient):
    """Provides translation between one source language and another of the same set of languages."""


    class ListTerminologiesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_terminologies` for valid parameters"""
            pass


    class DeleteTerminologyRequest(Mapping):
        pass
    
    class GetTerminologyRequest(Mapping):
        pass
    
    class GetTerminologyResponse(Mapping):
        pass
    
    class ImportTerminologyRequest(Mapping):
        pass
    
    class ImportTerminologyResponse(Mapping):
        pass
    
    class TerminologyData(Mapping):
        pass
    
    class EncryptionKey(Mapping):
        pass
    
    class ListTerminologiesRequest(Mapping):
        pass
    
    class ListTerminologiesResponse(Mapping):
        pass
    
    class TranslateTextRequest(Mapping):
        pass
    
    class TranslateTextResponse(Mapping):
        pass
    
    class ResourceName(object):
        pass
    
    def delete_terminology(self, Name: str):
        """A synchronous action that deletes a custom terminology.

        :param self:
        :param Name:  The name of the custom terminology being deleted. <br/>
        :return:         """
        pass

    def get_terminology(self, TerminologyDataFormat: str, Name: str) -> GetTerminologyResponse:
        """Retrieves a custom terminology.

        :param self:
        :param TerminologyDataFormat:  The data format of the custom terminology being retrieved, either CSV or TMX.<br/>Name:  The name of the custom terminology being retrieved.<br/>
        :return: TerminologyDataLocation:  The data location of the custom terminology being retrieved. The custom terminology file is returned in a presigned url that has a 30 minute expiration.:<br/>TerminologyProperties:  The properties of the custom terminology being retrieved.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdatedAt</i> The time at which the custom terminology was last update, based on the timestamp.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i> The time at which the custom terminology was created, based on the timestamp.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TermCount</i> The number of terms included in the custom terminology.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SizeBytes</i> The size of the file used when importing a custom terminology.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionKey</i> The encryption key for the custom terminology.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncryptionKey</b>: The encryption key for the custom terminology.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetLanguageCodes</i> The language codes for the target languages available with the custom terminology file. All possible target languages are returned in array.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceLanguageCode</i> The language code for the source text of the translation request for which the custom terminology is being used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i>  The Amazon Resource Name (ARN) of the custom terminology. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the custom terminology properties.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the custom terminology.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The Amazon Resource Name (ARN) of the encryption key being used to encrypt the custom terminology.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of encryption key used by Amazon Translate to encrypt custom terminologies.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Location</i> The location of the custom terminology data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RepositoryType</i> The repository type for the custom terminology data.<br/>        """
        pass

    def import_terminology(self, TerminologyData: TerminologyData, MergeStrategy: str, Name: str, Description: str=None, EncryptionKey: EncryptionKey=None) -> ImportTerminologyResponse:
        """Creates or updates a custom terminology, depending on whether or not one already exists for the given terminology name. Importing a terminology with the same name as an existing one will merge the terminologies based on the chosen merge strategy. Currently, the only supported merge strategy is OVERWRITE, and so the imported terminology will overwrite an existing terminology of the same name. If you import a terminology that overwrites an existing one, the new terminology take up to 10 minutes to fully propagate and be available for use in a translation due to cache policies with the DataPlane service that performs the translations.

        :param self:
        :param EncryptionKey:  The encryption key for the custom terminology being imported.:<br/>TerminologyData:  The terminology data for the custom terminology being imported.:<br/>Description:  The description of the custom terminology being imported.<br/>MergeStrategy:  The merge strategy of the custom terminology being imported. Currently, only the OVERWRITE merge strategy is supported. In this case, the imported terminology will overwrite an existing terminology of the same name.<br/>Name:  The name of the custom terminology being imported.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Format</i> The data format of the custom terminology. Either CSV or TMX.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>File</i> The file containing the custom terminology data.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The Amazon Resource Name (ARN) of the encryption key being used to encrypt the custom terminology.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of encryption key used by Amazon Translate to encrypt custom terminologies.<br/>
        :return: TerminologyProperties:  The properties of the custom terminology being imported.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LastUpdatedAt</i> The time at which the custom terminology was last update, based on the timestamp.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i> The time at which the custom terminology was created, based on the timestamp.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TermCount</i> The number of terms included in the custom terminology.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SizeBytes</i> The size of the file used when importing a custom terminology.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EncryptionKey</i> The encryption key for the custom terminology.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>EncryptionKey</b>: The encryption key for the custom terminology.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TargetLanguageCodes</i> The language codes for the target languages available with the custom terminology file. All possible target languages are returned in array.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SourceLanguageCode</i> The language code for the source text of the translation request for which the custom terminology is being used.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Arn</i>  The Amazon Resource Name (ARN) of the custom terminology. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Description</i> The description of the custom terminology properties.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the custom terminology.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The Amazon Resource Name (ARN) of the encryption key being used to encrypt the custom terminology.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of encryption key used by Amazon Translate to encrypt custom terminologies.<br/>        """
        pass

    def list_terminologies(self, NextToken: str=None, MaxResults: int=None) -> ListTerminologiesResponse:
        """Provides a list of custom terminologies associated with your account.

        :param self:
        :param MaxResults:  The maximum number of custom terminologies returned per list request.<br/>NextToken:  If the result of the request to ListTerminologies was truncated, include the NextToken to fetch the next group of custom terminologies. <br/>
        :return: NextToken:   If the response to the ListTerminologies was truncated, the NextToken fetches the next group of custom terminologies. <br/>TerminologyPropertiesList:  The properties list of the custom terminologies returned on the list request.<br/>        """
        pass

    def translate_text(self, TargetLanguageCode: str, SourceLanguageCode: str, Text: str, TerminologyNames: List[ResourceName]=None) -> TranslateTextResponse:
        """Translates input text from the source language to the target language. It is not necessary to use English (en) as either the source or the target language but not all language combinations are supported by Amazon Translate. For more information, see <a href="http://docs.aws.amazon.com/translate/latest/dg/pairs.html">Supported Language Pairs</a>. <ul> <li> Arabic (ar) </li> <li> Chinese (Simplified) (zh) </li> <li> Chinese (Traditional) (zh-TW) </li> <li> Czech (cs) </li> <li> Danish (da) </li> <li> Dutch (nl) </li> <li> English (en) </li> <li> Finnish (fi) </li> <li> French (fr) </li> <li> German (de) </li> <li> Hebrew (he) </li> <li> Indonesian (id) </li> <li> Italian (it) </li> <li> Japanese (ja) </li> <li> Korean (ko) </li> <li> Polish (pl) </li> <li> Portuguese (pt) </li> <li> Russian (ru) </li> <li> Spanish (es) </li> <li> Swedish (sv) </li> <li> Turkish (tr) </li> </ul> To have Amazon Translate determine the source language of your text, you can specify <code>auto</code> in the <code>SourceLanguageCode</code> field. If you specify <code>auto</code>, Amazon Translate will call Amazon Comprehend to determine the source language.

        :param self:
        :param TargetLanguageCode:  The language code requested for the language of the target text. The language must be a language supported by Amazon Translate.<br/>SourceLanguageCode:  The language code for the language of the source text. The language must be a language supported by Amazon Translate.  To have Amazon Translate determine the source language of your text, you can specify <code>auto</code> in the <code>SourceLanguageCode</code> field. If you specify <code>auto</code>, Amazon Translate will call Amazon Comprehend to determine the source language.<br/>TerminologyNames:  The TerminologyNames list that is taken as input to the TranslateText request. This has a minimum length of 0 and a maximum length of 1.<br/>Text:  The text to translate. The text string can be a maximum of 5,000 bytes long. Depending on your character set, this may be fewer than 5,000 characters.<br/>
        :return: AppliedTerminologies:  The names of the custom terminologies applied to the input text by Amazon Translate for the translated text response.<br/>TargetLanguageCode:  The language code for the language of the target text. <br/>SourceLanguageCode:  The language code for the language of the source text. <br/>TranslatedText:  The the translated text. The maximum length of this text is 5kb.<br/>        """
        pass



class WAF(BaseClient):
    """This is the <i>AWS WAF API Reference</i> for using AWS WAF with Amazon CloudFront. The AWS WAF actions and data types listed in the reference are available for protecting Amazon CloudFront distributions. You can use these actions and data types via the endpoint <i>waf.amazonaws.com</i>. This guide is for developers who need detailed information about the AWS WAF API actions, data types, and errors. For detailed information about AWS WAF features and an overview of how to use the AWS WAF API, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>."""


    class ListByteMatchSetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_byte_match_sets` for valid parameters"""
            pass
    class ListIPSetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_ip_sets` for valid parameters"""
            pass
    class ListRulesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_rules` for valid parameters"""
            pass
    class ListSizeConstraintSetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_size_constraint_sets` for valid parameters"""
            pass
    class ListSqlInjectionMatchSetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_sql_injection_match_sets` for valid parameters"""
            pass
    class ListWebACLsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_web_acls` for valid parameters"""
            pass
    class ListXssMatchSetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_xss_match_sets` for valid parameters"""
            pass
    class GetRateBasedRuleManagedKeysPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_rate_based_rule_managed_keys` for valid parameters"""
            pass
    class ListActivatedRulesInRuleGroupPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_activated_rules_in_rule_group` for valid parameters"""
            pass
    class ListGeoMatchSetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_geo_match_sets` for valid parameters"""
            pass
    class ListLoggingConfigurationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_logging_configurations` for valid parameters"""
            pass
    class ListRateBasedRulesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_rate_based_rules` for valid parameters"""
            pass
    class ListRegexMatchSetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_regex_match_sets` for valid parameters"""
            pass
    class ListRegexPatternSetsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_regex_pattern_sets` for valid parameters"""
            pass
    class ListRuleGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_rule_groups` for valid parameters"""
            pass
    class ListSubscribedRuleGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_subscribed_rule_groups` for valid parameters"""
            pass


    class CreateByteMatchSetRequest(Mapping):
        pass
    
    class CreateByteMatchSetResponse(Mapping):
        pass
    
    class CreateGeoMatchSetRequest(Mapping):
        pass
    
    class CreateGeoMatchSetResponse(Mapping):
        pass
    
    class CreateIPSetRequest(Mapping):
        pass
    
    class CreateIPSetResponse(Mapping):
        pass
    
    class CreateRateBasedRuleRequest(Mapping):
        pass
    
    class CreateRateBasedRuleResponse(Mapping):
        pass
    
    class RateLimit(object):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreateRegexMatchSetRequest(Mapping):
        pass
    
    class CreateRegexMatchSetResponse(Mapping):
        pass
    
    class CreateRegexPatternSetRequest(Mapping):
        pass
    
    class CreateRegexPatternSetResponse(Mapping):
        pass
    
    class CreateRuleRequest(Mapping):
        pass
    
    class CreateRuleResponse(Mapping):
        pass
    
    class CreateRuleGroupRequest(Mapping):
        pass
    
    class CreateRuleGroupResponse(Mapping):
        pass
    
    class CreateSizeConstraintSetRequest(Mapping):
        pass
    
    class CreateSizeConstraintSetResponse(Mapping):
        pass
    
    class CreateSqlInjectionMatchSetRequest(Mapping):
        pass
    
    class CreateSqlInjectionMatchSetResponse(Mapping):
        pass
    
    class CreateWebACLRequest(Mapping):
        pass
    
    class CreateWebACLResponse(Mapping):
        pass
    
    class WafAction(Mapping):
        pass
    
    class CreateXssMatchSetRequest(Mapping):
        pass
    
    class CreateXssMatchSetResponse(Mapping):
        pass
    
    class DeleteByteMatchSetRequest(Mapping):
        pass
    
    class DeleteByteMatchSetResponse(Mapping):
        pass
    
    class DeleteGeoMatchSetRequest(Mapping):
        pass
    
    class DeleteGeoMatchSetResponse(Mapping):
        pass
    
    class DeleteIPSetRequest(Mapping):
        pass
    
    class DeleteIPSetResponse(Mapping):
        pass
    
    class DeleteLoggingConfigurationRequest(Mapping):
        pass
    
    class DeleteLoggingConfigurationResponse(Mapping):
        pass
    
    class DeletePermissionPolicyRequest(Mapping):
        pass
    
    class DeletePermissionPolicyResponse(Mapping):
        pass
    
    class DeleteRateBasedRuleRequest(Mapping):
        pass
    
    class DeleteRateBasedRuleResponse(Mapping):
        pass
    
    class DeleteRegexMatchSetRequest(Mapping):
        pass
    
    class DeleteRegexMatchSetResponse(Mapping):
        pass
    
    class DeleteRegexPatternSetRequest(Mapping):
        pass
    
    class DeleteRegexPatternSetResponse(Mapping):
        pass
    
    class DeleteRuleRequest(Mapping):
        pass
    
    class DeleteRuleResponse(Mapping):
        pass
    
    class DeleteRuleGroupRequest(Mapping):
        pass
    
    class DeleteRuleGroupResponse(Mapping):
        pass
    
    class DeleteSizeConstraintSetRequest(Mapping):
        pass
    
    class DeleteSizeConstraintSetResponse(Mapping):
        pass
    
    class DeleteSqlInjectionMatchSetRequest(Mapping):
        pass
    
    class DeleteSqlInjectionMatchSetResponse(Mapping):
        pass
    
    class DeleteWebACLRequest(Mapping):
        pass
    
    class DeleteWebACLResponse(Mapping):
        pass
    
    class DeleteXssMatchSetRequest(Mapping):
        pass
    
    class DeleteXssMatchSetResponse(Mapping):
        pass
    
    class GetByteMatchSetRequest(Mapping):
        pass
    
    class GetByteMatchSetResponse(Mapping):
        pass
    
    class GetChangeTokenRequest(Mapping):
        pass
    
    class GetChangeTokenResponse(Mapping):
        pass
    
    class GetChangeTokenStatusRequest(Mapping):
        pass
    
    class GetChangeTokenStatusResponse(Mapping):
        pass
    
    class GetGeoMatchSetRequest(Mapping):
        pass
    
    class GetGeoMatchSetResponse(Mapping):
        pass
    
    class GetIPSetRequest(Mapping):
        pass
    
    class GetIPSetResponse(Mapping):
        pass
    
    class GetLoggingConfigurationRequest(Mapping):
        pass
    
    class GetLoggingConfigurationResponse(Mapping):
        pass
    
    class GetPermissionPolicyRequest(Mapping):
        pass
    
    class GetPermissionPolicyResponse(Mapping):
        pass
    
    class GetRateBasedRuleRequest(Mapping):
        pass
    
    class GetRateBasedRuleResponse(Mapping):
        pass
    
    class GetRateBasedRuleManagedKeysRequest(Mapping):
        pass
    
    class GetRateBasedRuleManagedKeysResponse(Mapping):
        pass
    
    class GetRegexMatchSetRequest(Mapping):
        pass
    
    class GetRegexMatchSetResponse(Mapping):
        pass
    
    class GetRegexPatternSetRequest(Mapping):
        pass
    
    class GetRegexPatternSetResponse(Mapping):
        pass
    
    class GetRuleRequest(Mapping):
        pass
    
    class GetRuleResponse(Mapping):
        pass
    
    class GetRuleGroupRequest(Mapping):
        pass
    
    class GetRuleGroupResponse(Mapping):
        pass
    
    class GetSampledRequestsRequest(Mapping):
        pass
    
    class GetSampledRequestsResponse(Mapping):
        pass
    
    class TimeWindow(Mapping):
        pass
    
    class GetSampledRequestsMaxItems(object):
        pass
    
    class GetSizeConstraintSetRequest(Mapping):
        pass
    
    class GetSizeConstraintSetResponse(Mapping):
        pass
    
    class GetSqlInjectionMatchSetRequest(Mapping):
        pass
    
    class GetSqlInjectionMatchSetResponse(Mapping):
        pass
    
    class GetWebACLRequest(Mapping):
        pass
    
    class GetWebACLResponse(Mapping):
        pass
    
    class GetXssMatchSetRequest(Mapping):
        pass
    
    class GetXssMatchSetResponse(Mapping):
        pass
    
    class ListActivatedRulesInRuleGroupRequest(Mapping):
        pass
    
    class ListActivatedRulesInRuleGroupResponse(Mapping):
        pass
    
    class ListByteMatchSetsRequest(Mapping):
        pass
    
    class ListByteMatchSetsResponse(Mapping):
        pass
    
    class ListGeoMatchSetsRequest(Mapping):
        pass
    
    class ListGeoMatchSetsResponse(Mapping):
        pass
    
    class ListIPSetsRequest(Mapping):
        pass
    
    class ListIPSetsResponse(Mapping):
        pass
    
    class ListLoggingConfigurationsRequest(Mapping):
        pass
    
    class ListLoggingConfigurationsResponse(Mapping):
        pass
    
    class ListRateBasedRulesRequest(Mapping):
        pass
    
    class ListRateBasedRulesResponse(Mapping):
        pass
    
    class ListRegexMatchSetsRequest(Mapping):
        pass
    
    class ListRegexMatchSetsResponse(Mapping):
        pass
    
    class ListRegexPatternSetsRequest(Mapping):
        pass
    
    class ListRegexPatternSetsResponse(Mapping):
        pass
    
    class ListRuleGroupsRequest(Mapping):
        pass
    
    class ListRuleGroupsResponse(Mapping):
        pass
    
    class ListRulesRequest(Mapping):
        pass
    
    class ListRulesResponse(Mapping):
        pass
    
    class ListSizeConstraintSetsRequest(Mapping):
        pass
    
    class ListSizeConstraintSetsResponse(Mapping):
        pass
    
    class ListSqlInjectionMatchSetsRequest(Mapping):
        pass
    
    class ListSqlInjectionMatchSetsResponse(Mapping):
        pass
    
    class ListSubscribedRuleGroupsRequest(Mapping):
        pass
    
    class ListSubscribedRuleGroupsResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class ListWebACLsRequest(Mapping):
        pass
    
    class ListWebACLsResponse(Mapping):
        pass
    
    class ListXssMatchSetsRequest(Mapping):
        pass
    
    class ListXssMatchSetsResponse(Mapping):
        pass
    
    class PutLoggingConfigurationRequest(Mapping):
        pass
    
    class PutLoggingConfigurationResponse(Mapping):
        pass
    
    class LoggingConfiguration(Mapping):
        pass
    
    class PutPermissionPolicyRequest(Mapping):
        pass
    
    class PutPermissionPolicyResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateByteMatchSetRequest(Mapping):
        pass
    
    class UpdateByteMatchSetResponse(Mapping):
        pass
    
    class ByteMatchSetUpdate(Mapping):
        pass
    
    class UpdateGeoMatchSetRequest(Mapping):
        pass
    
    class UpdateGeoMatchSetResponse(Mapping):
        pass
    
    class GeoMatchSetUpdate(Mapping):
        pass
    
    class UpdateIPSetRequest(Mapping):
        pass
    
    class UpdateIPSetResponse(Mapping):
        pass
    
    class IPSetUpdate(Mapping):
        pass
    
    class UpdateRateBasedRuleRequest(Mapping):
        pass
    
    class UpdateRateBasedRuleResponse(Mapping):
        pass
    
    class RuleUpdate(Mapping):
        pass
    
    class UpdateRegexMatchSetRequest(Mapping):
        pass
    
    class UpdateRegexMatchSetResponse(Mapping):
        pass
    
    class RegexMatchSetUpdate(Mapping):
        pass
    
    class UpdateRegexPatternSetRequest(Mapping):
        pass
    
    class UpdateRegexPatternSetResponse(Mapping):
        pass
    
    class RegexPatternSetUpdate(Mapping):
        pass
    
    class UpdateRuleRequest(Mapping):
        pass
    
    class UpdateRuleResponse(Mapping):
        pass
    
    class UpdateRuleGroupRequest(Mapping):
        pass
    
    class UpdateRuleGroupResponse(Mapping):
        pass
    
    class RuleGroupUpdate(Mapping):
        pass
    
    class UpdateSizeConstraintSetRequest(Mapping):
        pass
    
    class UpdateSizeConstraintSetResponse(Mapping):
        pass
    
    class SizeConstraintSetUpdate(Mapping):
        pass
    
    class UpdateSqlInjectionMatchSetRequest(Mapping):
        pass
    
    class UpdateSqlInjectionMatchSetResponse(Mapping):
        pass
    
    class SqlInjectionMatchSetUpdate(Mapping):
        pass
    
    class UpdateWebACLRequest(Mapping):
        pass
    
    class UpdateWebACLResponse(Mapping):
        pass
    
    class WebACLUpdate(Mapping):
        pass
    
    class UpdateXssMatchSetRequest(Mapping):
        pass
    
    class UpdateXssMatchSetResponse(Mapping):
        pass
    
    class XssMatchSetUpdate(Mapping):
        pass
    
    def create_byte_match_set(self, ChangeToken: str, Name: str) -> CreateByteMatchSetResponse:
        """Creates a <code>ByteMatchSet</code>. You then use <a>UpdateByteMatchSet</a> to identify the part of a web request that you want AWS WAF to inspect, such as the values of the <code>User-Agent</code> header or the query string. For example, you can create a <code>ByteMatchSet</code> that matches any requests with <code>User-Agent</code> headers that contain the string <code>BadBot</code>. You can then configure AWS WAF to reject those requests. To create and configure a <code>ByteMatchSet</code>, perform the following steps: <ol> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>CreateByteMatchSet</code> request. </li> <li> Submit a <code>CreateByteMatchSet</code> request. </li> <li> Use <code>GetChangeToken</code> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <code>UpdateByteMatchSet</code> request. </li> <li> Submit an <a>UpdateByteMatchSet</a> request to specify the part of the request that you want AWS WAF to inspect (for example, the header or the URI) and the value that you want AWS WAF to watch for. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>Name:  A friendly name or description of the <a>ByteMatchSet</a>. You can't change <code>Name</code> after you create a <code>ByteMatchSet</code>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>CreateByteMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>ByteMatchSet:  A <a>ByteMatchSet</a> that contains no <code>ByteMatchTuple</code> objects.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ByteMatchTuples</i> Specifies the bytes (typically a string that corresponds with ASCII characters) that you want AWS WAF to search for in web requests, the location in requests that you want AWS WAF to search, and other settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A friendly name or description of the <a>ByteMatchSet</a>. You can't change <code>Name</code> after you create a <code>ByteMatchSet</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ByteMatchSetId</i> The <code>ByteMatchSetId</code> for a <code>ByteMatchSet</code>. You use <code>ByteMatchSetId</code> to get information about a <code>ByteMatchSet</code> (see <a>GetByteMatchSet</a>), update a <code>ByteMatchSet</code> (see <a>UpdateByteMatchSet</a>), insert a <code>ByteMatchSet</code> into a <code>Rule</code> or delete one from a <code>Rule</code> (see <a>UpdateRule</a>), and delete a <code>ByteMatchSet</code> from AWS WAF (see <a>DeleteByteMatchSet</a>).  <code>ByteMatchSetId</code> is returned by <a>CreateByteMatchSet</a> and by <a>ListByteMatchSets</a>.<br/>        """
        pass

    def create_geo_match_set(self, ChangeToken: str, Name: str) -> CreateGeoMatchSetResponse:
        """Creates an <a>GeoMatchSet</a>, which you use to specify which web requests you want to allow or block based on the country that the requests originate from. For example, if you're receiving a lot of requests from one or more countries and you want to block the requests, you can create an <code>GeoMatchSet</code> that contains those countries and then configure AWS WAF to block the requests.  To create and configure a <code>GeoMatchSet</code>, perform the following steps: <ol> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>CreateGeoMatchSet</code> request. </li> <li> Submit a <code>CreateGeoMatchSet</code> request. </li> <li> Use <code>GetChangeToken</code> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <a>UpdateGeoMatchSet</a> request. </li> <li> Submit an <code>UpdateGeoMatchSetSet</code> request to specify the countries that you want AWS WAF to watch for. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>Name:  A friendly name or description of the <a>GeoMatchSet</a>. You can't change <code>Name</code> after you create the <code>GeoMatchSet</code>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>CreateGeoMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>GeoMatchSet:  The <a>GeoMatchSet</a> returned in the <code>CreateGeoMatchSet</code> response. The <code>GeoMatchSet</code> contains no <code>GeoMatchConstraints</code>.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GeoMatchConstraints</i> An array of <a>GeoMatchConstraint</a> objects, which contain the country that you want AWS WAF to search for.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A friendly name or description of the <a>GeoMatchSet</a>. You can't change the name of an <code>GeoMatchSet</code> after you create it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GeoMatchSetId</i> The <code>GeoMatchSetId</code> for an <code>GeoMatchSet</code>. You use <code>GeoMatchSetId</code> to get information about a <code>GeoMatchSet</code> (see <a>GeoMatchSet</a>), update a <code>GeoMatchSet</code> (see <a>UpdateGeoMatchSet</a>), insert a <code>GeoMatchSet</code> into a <code>Rule</code> or delete one from a <code>Rule</code> (see <a>UpdateRule</a>), and delete a <code>GeoMatchSet</code> from AWS WAF (see <a>DeleteGeoMatchSet</a>).  <code>GeoMatchSetId</code> is returned by <a>CreateGeoMatchSet</a> and by <a>ListGeoMatchSets</a>.<br/>        """
        pass

    def create_ip_set(self, ChangeToken: str, Name: str) -> CreateIPSetResponse:
        """Creates an <a>IPSet</a>, which you use to specify which web requests that you want to allow or block based on the IP addresses that the requests originate from. For example, if you're receiving a lot of requests from one or more individual IP addresses or one or more ranges of IP addresses and you want to block the requests, you can create an <code>IPSet</code> that contains those IP addresses and then configure AWS WAF to block the requests.  To create and configure an <code>IPSet</code>, perform the following steps: <ol> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>CreateIPSet</code> request. </li> <li> Submit a <code>CreateIPSet</code> request. </li> <li> Use <code>GetChangeToken</code> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <a>UpdateIPSet</a> request. </li> <li> Submit an <code>UpdateIPSet</code> request to specify the IP addresses that you want AWS WAF to watch for. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>Name:  A friendly name or description of the <a>IPSet</a>. You can't change <code>Name</code> after you create the <code>IPSet</code>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>CreateIPSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>IPSet:  The <a>IPSet</a> returned in the <code>CreateIPSet</code> response.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IPSetDescriptors</i> The IP address type (<code>IPV4</code> or <code>IPV6</code>) and the IP address range (in CIDR notation) that web requests originate from. If the <code>WebACL</code> is associated with a CloudFront distribution and the viewer did not use an HTTP proxy or a load balancer to send the request, this is the value of the c-ip field in the CloudFront access logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A friendly name or description of the <a>IPSet</a>. You can't change the name of an <code>IPSet</code> after you create it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IPSetId</i> The <code>IPSetId</code> for an <code>IPSet</code>. You use <code>IPSetId</code> to get information about an <code>IPSet</code> (see <a>GetIPSet</a>), update an <code>IPSet</code> (see <a>UpdateIPSet</a>), insert an <code>IPSet</code> into a <code>Rule</code> or delete one from a <code>Rule</code> (see <a>UpdateRule</a>), and delete an <code>IPSet</code> from AWS WAF (see <a>DeleteIPSet</a>).  <code>IPSetId</code> is returned by <a>CreateIPSet</a> and by <a>ListIPSets</a>.<br/>        """
        pass

    def create_rate_based_rule(self, ChangeToken: str, RateLimit: RateLimit, RateKey: str, MetricName: str, Name: str, Tags: List[Tag]=None) -> CreateRateBasedRuleResponse:
        """Creates a <a>RateBasedRule</a>. The <code>RateBasedRule</code> contains a <code>RateLimit</code>, which specifies the maximum number of requests that AWS WAF allows from a specified IP address in a five-minute period. The <code>RateBasedRule</code> also contains the <code>IPSet</code> objects, <code>ByteMatchSet</code> objects, and other predicates that identify the requests that you want to count or block if these requests exceed the <code>RateLimit</code>. If you add more than one predicate to a <code>RateBasedRule</code>, a request not only must exceed the <code>RateLimit</code>, but it also must match all the specifications to be counted or blocked. For example, suppose you add the following to a <code>RateBasedRule</code>: <ul> <li> An <code>IPSet</code> that matches the IP address <code>192.0.2.44/32</code>  </li> <li> A <code>ByteMatchSet</code> that matches <code>BadBot</code> in the <code>User-Agent</code> header </li> </ul> Further, you specify a <code>RateLimit</code> of 15,000. You then add the <code>RateBasedRule</code> to a <code>WebACL</code> and specify that you want to block requests that meet the conditions in the rule. For a request to be blocked, it must come from the IP address 192.0.2.44 <i>and</i> the <code>User-Agent</code> header in the request must contain the value <code>BadBot</code>. Further, requests that match these two conditions must be received at a rate of more than 15,000 requests every five minutes. If both conditions are met and the rate is exceeded, AWS WAF blocks the requests. If the rate drops below 15,000 for a five-minute period, AWS WAF no longer blocks the requests. As a second example, suppose you want to limit requests to a particular page on your site. To do this, you could add the following to a <code>RateBasedRule</code>: <ul> <li> A <code>ByteMatchSet</code> with <code>FieldToMatch</code> of <code>URI</code>  </li> <li> A <code>PositionalConstraint</code> of <code>STARTS_WITH</code>  </li> <li> A <code>TargetString</code> of <code>login</code>  </li> </ul> Further, you specify a <code>RateLimit</code> of 15,000. By adding this <code>RateBasedRule</code> to a <code>WebACL</code>, you could limit requests to your login page without affecting the rest of your site. To create and configure a <code>RateBasedRule</code>, perform the following steps: <ol> <li> Create and update the predicates that you want to include in the rule. For more information, see <a>CreateByteMatchSet</a>, <a>CreateIPSet</a>, and <a>CreateSqlInjectionMatchSet</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>CreateRule</code> request. </li> <li> Submit a <code>CreateRateBasedRule</code> request. </li> <li> Use <code>GetChangeToken</code> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <a>UpdateRule</a> request. </li> <li> Submit an <code>UpdateRateBasedRule</code> request to specify the predicates that you want to include in the rule. </li> <li> Create and update a <code>WebACL</code> that contains the <code>RateBasedRule</code>. For more information, see <a>CreateWebACL</a>. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param Tags:  <br/>ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>CreateRateBasedRule</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>RateLimit:  The maximum number of requests, which have an identical value in the field that is specified by <code>RateKey</code>, allowed in a five-minute period. If the number of requests exceeds the <code>RateLimit</code> and the other predicates specified in the rule are also met, AWS WAF triggers the action that is specified for this rule.<br/>RateKey:  The field that AWS WAF uses to determine if requests are likely arriving from a single source and thus subject to rate monitoring. The only valid value for <code>RateKey</code> is <code>IP</code>. <code>IP</code> indicates that requests that arrive from the same IP address are subject to the <code>RateLimit</code> that is specified in the <code>RateBasedRule</code>.<br/>MetricName:  A friendly name or description for the metrics for this <code>RateBasedRule</code>. The name can contain only alphanumeric characters (A-Z, a-z, 0-9), with maximum length 128 and minimum length one. It can't contain whitespace or metric names reserved for AWS WAF, including "All" and "Default_Action." You can't change the name of the metric after you create the <code>RateBasedRule</code>.<br/>Name:  A friendly name or description of the <a>RateBasedRule</a>. You can't change the name of a <code>RateBasedRule</code> after you create it.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>CreateRateBasedRule</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>Rule:  The <a>RateBasedRule</a> that is returned in the <code>CreateRateBasedRule</code> response.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RateLimit</i> The maximum number of requests, which have an identical value in the field specified by the <code>RateKey</code>, allowed in a five-minute period. If the number of requests exceeds the <code>RateLimit</code> and the other predicates specified in the rule are also met, AWS WAF triggers the action that is specified for this rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RateKey</i> The field that AWS WAF uses to determine if requests are likely arriving from single source and thus subject to rate monitoring. The only valid value for <code>RateKey</code> is <code>IP</code>. <code>IP</code> indicates that requests arriving from the same IP address are subject to the <code>RateLimit</code> that is specified in the <code>RateBasedRule</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MatchPredicates</i> The <code>Predicates</code> object contains one <code>Predicate</code> element for each <a>ByteMatchSet</a>, <a>IPSet</a>, or <a>SqlInjectionMatchSet</a> object that you want to include in a <code>RateBasedRule</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricName</i> A friendly name or description for the metrics for a <code>RateBasedRule</code>. The name can contain only alphanumeric characters (A-Z, a-z, 0-9), with maximum length 128 and minimum length one. It can't contain whitespace or metric names reserved for AWS WAF, including "All" and "Default_Action." You can't change the name of the metric after you create the <code>RateBasedRule</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A friendly name or description for a <code>RateBasedRule</code>. You can't change the name of a <code>RateBasedRule</code> after you create it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleId</i> A unique identifier for a <code>RateBasedRule</code>. You use <code>RuleId</code> to get more information about a <code>RateBasedRule</code> (see <a>GetRateBasedRule</a>), update a <code>RateBasedRule</code> (see <a>UpdateRateBasedRule</a>), insert a <code>RateBasedRule</code> into a <code>WebACL</code> or delete one from a <code>WebACL</code> (see <a>UpdateWebACL</a>), or delete a <code>RateBasedRule</code> from AWS WAF (see <a>DeleteRateBasedRule</a>).<br/>        """
        pass

    def create_regex_match_set(self, ChangeToken: str, Name: str) -> CreateRegexMatchSetResponse:
        """Creates a <a>RegexMatchSet</a>. You then use <a>UpdateRegexMatchSet</a> to identify the part of a web request that you want AWS WAF to inspect, such as the values of the <code>User-Agent</code> header or the query string. For example, you can create a <code>RegexMatchSet</code> that contains a <code>RegexMatchTuple</code> that looks for any requests with <code>User-Agent</code> headers that match a <code>RegexPatternSet</code> with pattern <code>B[a@]dB[o0]t</code>. You can then configure AWS WAF to reject those requests. To create and configure a <code>RegexMatchSet</code>, perform the following steps: <ol> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>CreateRegexMatchSet</code> request. </li> <li> Submit a <code>CreateRegexMatchSet</code> request. </li> <li> Use <code>GetChangeToken</code> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <code>UpdateRegexMatchSet</code> request. </li> <li> Submit an <a>UpdateRegexMatchSet</a> request to specify the part of the request that you want AWS WAF to inspect (for example, the header or the URI) and the value, using a <code>RegexPatternSet</code>, that you want AWS WAF to watch for. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>Name:  A friendly name or description of the <a>RegexMatchSet</a>. You can't change <code>Name</code> after you create a <code>RegexMatchSet</code>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>CreateRegexMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>RegexMatchSet:  A <a>RegexMatchSet</a> that contains no <code>RegexMatchTuple</code> objects.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RegexMatchTuples</i> Contains an array of <a>RegexMatchTuple</a> objects. Each <code>RegexMatchTuple</code> object contains:  <ul> <li> The part of a web request that you want AWS WAF to inspect, such as a query string or the value of the <code>User-Agent</code> header.  </li> <li> The identifier of the pattern (a regular expression) that you want AWS WAF to look for. For more information, see <a>RegexPatternSet</a>. </li> <li> Whether to perform any conversions on the request, such as converting it to lowercase, before inspecting it for the specified string. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A friendly name or description of the <a>RegexMatchSet</a>. You can't change <code>Name</code> after you create a <code>RegexMatchSet</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RegexMatchSetId</i> The <code>RegexMatchSetId</code> for a <code>RegexMatchSet</code>. You use <code>RegexMatchSetId</code> to get information about a <code>RegexMatchSet</code> (see <a>GetRegexMatchSet</a>), update a <code>RegexMatchSet</code> (see <a>UpdateRegexMatchSet</a>), insert a <code>RegexMatchSet</code> into a <code>Rule</code> or delete one from a <code>Rule</code> (see <a>UpdateRule</a>), and delete a <code>RegexMatchSet</code> from AWS WAF (see <a>DeleteRegexMatchSet</a>).  <code>RegexMatchSetId</code> is returned by <a>CreateRegexMatchSet</a> and by <a>ListRegexMatchSets</a>.<br/>        """
        pass

    def create_regex_pattern_set(self, ChangeToken: str, Name: str) -> CreateRegexPatternSetResponse:
        """Creates a <code>RegexPatternSet</code>. You then use <a>UpdateRegexPatternSet</a> to specify the regular expression (regex) pattern that you want AWS WAF to search for, such as <code>B[a@]dB[o0]t</code>. You can then configure AWS WAF to reject those requests. To create and configure a <code>RegexPatternSet</code>, perform the following steps: <ol> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>CreateRegexPatternSet</code> request. </li> <li> Submit a <code>CreateRegexPatternSet</code> request. </li> <li> Use <code>GetChangeToken</code> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <code>UpdateRegexPatternSet</code> request. </li> <li> Submit an <a>UpdateRegexPatternSet</a> request to specify the string that you want AWS WAF to watch for. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>Name:  A friendly name or description of the <a>RegexPatternSet</a>. You can't change <code>Name</code> after you create a <code>RegexPatternSet</code>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>CreateRegexPatternSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>RegexPatternSet:  A <a>RegexPatternSet</a> that contains no objects.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RegexPatternStrings</i> Specifies the regular expression (regex) patterns that you want AWS WAF to search for, such as <code>B[a@]dB[o0]t</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A friendly name or description of the <a>RegexPatternSet</a>. You can't change <code>Name</code> after you create a <code>RegexPatternSet</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RegexPatternSetId</i> The identifier for the <code>RegexPatternSet</code>. You use <code>RegexPatternSetId</code> to get information about a <code>RegexPatternSet</code>, update a <code>RegexPatternSet</code>, remove a <code>RegexPatternSet</code> from a <code>RegexMatchSet</code>, and delete a <code>RegexPatternSet</code> from AWS WAF.  <code>RegexMatchSetId</code> is returned by <a>CreateRegexPatternSet</a> and by <a>ListRegexPatternSets</a>.<br/>        """
        pass

    def create_rule(self, ChangeToken: str, MetricName: str, Name: str, Tags: List[Tag]=None) -> CreateRuleResponse:
        """Creates a <code>Rule</code>, which contains the <code>IPSet</code> objects, <code>ByteMatchSet</code> objects, and other predicates that identify the requests that you want to block. If you add more than one predicate to a <code>Rule</code>, a request must match all of the specifications to be allowed or blocked. For example, suppose that you add the following to a <code>Rule</code>: <ul> <li> An <code>IPSet</code> that matches the IP address <code>192.0.2.44/32</code>  </li> <li> A <code>ByteMatchSet</code> that matches <code>BadBot</code> in the <code>User-Agent</code> header </li> </ul> You then add the <code>Rule</code> to a <code>WebACL</code> and specify that you want to blocks requests that satisfy the <code>Rule</code>. For a request to be blocked, it must come from the IP address 192.0.2.44 <i>and</i> the <code>User-Agent</code> header in the request must contain the value <code>BadBot</code>. To create and configure a <code>Rule</code>, perform the following steps: <ol> <li> Create and update the predicates that you want to include in the <code>Rule</code>. For more information, see <a>CreateByteMatchSet</a>, <a>CreateIPSet</a>, and <a>CreateSqlInjectionMatchSet</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>CreateRule</code> request. </li> <li> Submit a <code>CreateRule</code> request. </li> <li> Use <code>GetChangeToken</code> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <a>UpdateRule</a> request. </li> <li> Submit an <code>UpdateRule</code> request to specify the predicates that you want to include in the <code>Rule</code>. </li> <li> Create and update a <code>WebACL</code> that contains the <code>Rule</code>. For more information, see <a>CreateWebACL</a>. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param Tags:  <br/>ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>MetricName:  A friendly name or description for the metrics for this <code>Rule</code>. The name can contain only alphanumeric characters (A-Z, a-z, 0-9), with maximum length 128 and minimum length one. It can't contain whitespace or metric names reserved for AWS WAF, including "All" and "Default_Action." You can't change the name of the metric after you create the <code>Rule</code>.<br/>Name:  A friendly name or description of the <a>Rule</a>. You can't change the name of a <code>Rule</code> after you create it.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>CreateRule</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>Rule:  The <a>Rule</a> returned in the <code>CreateRule</code> response.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Predicates</i> The <code>Predicates</code> object contains one <code>Predicate</code> element for each <a>ByteMatchSet</a>, <a>IPSet</a>, or <a>SqlInjectionMatchSet</a> object that you want to include in a <code>Rule</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricName</i> A friendly name or description for the metrics for this <code>Rule</code>. The name can contain only alphanumeric characters (A-Z, a-z, 0-9), with maximum length 128 and minimum length one. It can't contain whitespace or metric names reserved for AWS WAF, including "All" and "Default_Action." You can't change <code>MetricName</code> after you create the <code>Rule</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The friendly name or description for the <code>Rule</code>. You can't change the name of a <code>Rule</code> after you create it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleId</i> A unique identifier for a <code>Rule</code>. You use <code>RuleId</code> to get more information about a <code>Rule</code> (see <a>GetRule</a>), update a <code>Rule</code> (see <a>UpdateRule</a>), insert a <code>Rule</code> into a <code>WebACL</code> or delete a one from a <code>WebACL</code> (see <a>UpdateWebACL</a>), or delete a <code>Rule</code> from AWS WAF (see <a>DeleteRule</a>).  <code>RuleId</code> is returned by <a>CreateRule</a> and by <a>ListRules</a>.<br/>        """
        pass

    def create_rule_group(self, ChangeToken: str, MetricName: str, Name: str, Tags: List[Tag]=None) -> CreateRuleGroupResponse:
        """Creates a <code>RuleGroup</code>. A rule group is a collection of predefined rules that you add to a web ACL. You use <a>UpdateRuleGroup</a> to add rules to the rule group. Rule groups are subject to the following limits: <ul> <li> Three rule groups per account. You can request an increase to this limit by contacting customer support. </li> <li> One rule group per web ACL. </li> <li> Ten rules per rule group. </li> </ul> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param Tags:  <br/>ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>MetricName:  A friendly name or description for the metrics for this <code>RuleGroup</code>. The name can contain only alphanumeric characters (A-Z, a-z, 0-9), with maximum length 128 and minimum length one. It can't contain whitespace or metric names reserved for AWS WAF, including "All" and "Default_Action." You can't change the name of the metric after you create the <code>RuleGroup</code>.<br/>Name:  A friendly name or description of the <a>RuleGroup</a>. You can't change <code>Name</code> after you create a <code>RuleGroup</code>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>CreateRuleGroup</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>RuleGroup:  An empty <a>RuleGroup</a>.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricName</i> A friendly name or description for the metrics for this <code>RuleGroup</code>. The name can contain only alphanumeric characters (A-Z, a-z, 0-9), with maximum length 128 and minimum length one. It can't contain whitespace or metric names reserved for AWS WAF, including "All" and "Default_Action." You can't change the name of the metric after you create the <code>RuleGroup</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The friendly name or description for the <code>RuleGroup</code>. You can't change the name of a <code>RuleGroup</code> after you create it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleGroupId</i> A unique identifier for a <code>RuleGroup</code>. You use <code>RuleGroupId</code> to get more information about a <code>RuleGroup</code> (see <a>GetRuleGroup</a>), update a <code>RuleGroup</code> (see <a>UpdateRuleGroup</a>), insert a <code>RuleGroup</code> into a <code>WebACL</code> or delete a one from a <code>WebACL</code> (see <a>UpdateWebACL</a>), or delete a <code>RuleGroup</code> from AWS WAF (see <a>DeleteRuleGroup</a>).  <code>RuleGroupId</code> is returned by <a>CreateRuleGroup</a> and by <a>ListRuleGroups</a>.<br/>        """
        pass

    def create_size_constraint_set(self, ChangeToken: str, Name: str) -> CreateSizeConstraintSetResponse:
        """Creates a <code>SizeConstraintSet</code>. You then use <a>UpdateSizeConstraintSet</a> to identify the part of a web request that you want AWS WAF to check for length, such as the length of the <code>User-Agent</code> header or the length of the query string. For example, you can create a <code>SizeConstraintSet</code> that matches any requests that have a query string that is longer than 100 bytes. You can then configure AWS WAF to reject those requests. To create and configure a <code>SizeConstraintSet</code>, perform the following steps: <ol> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>CreateSizeConstraintSet</code> request. </li> <li> Submit a <code>CreateSizeConstraintSet</code> request. </li> <li> Use <code>GetChangeToken</code> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <code>UpdateSizeConstraintSet</code> request. </li> <li> Submit an <a>UpdateSizeConstraintSet</a> request to specify the part of the request that you want AWS WAF to inspect (for example, the header or the URI) and the value that you want AWS WAF to watch for. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>Name:  A friendly name or description of the <a>SizeConstraintSet</a>. You can't change <code>Name</code> after you create a <code>SizeConstraintSet</code>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>CreateSizeConstraintSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>SizeConstraintSet:  A <a>SizeConstraintSet</a> that contains no <code>SizeConstraint</code> objects.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SizeConstraints</i> Specifies the parts of web requests that you want to inspect the size of.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name, if any, of the <code>SizeConstraintSet</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SizeConstraintSetId</i> A unique identifier for a <code>SizeConstraintSet</code>. You use <code>SizeConstraintSetId</code> to get information about a <code>SizeConstraintSet</code> (see <a>GetSizeConstraintSet</a>), update a <code>SizeConstraintSet</code> (see <a>UpdateSizeConstraintSet</a>), insert a <code>SizeConstraintSet</code> into a <code>Rule</code> or delete one from a <code>Rule</code> (see <a>UpdateRule</a>), and delete a <code>SizeConstraintSet</code> from AWS WAF (see <a>DeleteSizeConstraintSet</a>).  <code>SizeConstraintSetId</code> is returned by <a>CreateSizeConstraintSet</a> and by <a>ListSizeConstraintSets</a>.<br/>        """
        pass

    def create_sql_injection_match_set(self, ChangeToken: str, Name: str) -> CreateSqlInjectionMatchSetResponse:
        """Creates a <a>SqlInjectionMatchSet</a>, which you use to allow, block, or count requests that contain snippets of SQL code in a specified part of web requests. AWS WAF searches for character sequences that are likely to be malicious strings. To create and configure a <code>SqlInjectionMatchSet</code>, perform the following steps: <ol> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>CreateSqlInjectionMatchSet</code> request. </li> <li> Submit a <code>CreateSqlInjectionMatchSet</code> request. </li> <li> Use <code>GetChangeToken</code> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <a>UpdateSqlInjectionMatchSet</a> request. </li> <li> Submit an <a>UpdateSqlInjectionMatchSet</a> request to specify the parts of web requests in which you want to allow, block, or count malicious SQL code. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>Name:  A friendly name or description for the <a>SqlInjectionMatchSet</a> that you're creating. You can't change <code>Name</code> after you create the <code>SqlInjectionMatchSet</code>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>CreateSqlInjectionMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>SqlInjectionMatchSet:  A <a>SqlInjectionMatchSet</a>.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SqlInjectionMatchTuples</i> Specifies the parts of web requests that you want to inspect for snippets of malicious SQL code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name, if any, of the <code>SqlInjectionMatchSet</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SqlInjectionMatchSetId</i> A unique identifier for a <code>SqlInjectionMatchSet</code>. You use <code>SqlInjectionMatchSetId</code> to get information about a <code>SqlInjectionMatchSet</code> (see <a>GetSqlInjectionMatchSet</a>), update a <code>SqlInjectionMatchSet</code> (see <a>UpdateSqlInjectionMatchSet</a>), insert a <code>SqlInjectionMatchSet</code> into a <code>Rule</code> or delete one from a <code>Rule</code> (see <a>UpdateRule</a>), and delete a <code>SqlInjectionMatchSet</code> from AWS WAF (see <a>DeleteSqlInjectionMatchSet</a>).  <code>SqlInjectionMatchSetId</code> is returned by <a>CreateSqlInjectionMatchSet</a> and by <a>ListSqlInjectionMatchSets</a>.<br/>        """
        pass

    def create_web_acl(self, ChangeToken: str, DefaultAction: WafAction, MetricName: str, Name: str, Tags: List[Tag]=None) -> CreateWebACLResponse:
        """Creates a <code>WebACL</code>, which contains the <code>Rules</code> that identify the CloudFront web requests that you want to allow, block, or count. AWS WAF evaluates <code>Rules</code> in order based on the value of <code>Priority</code> for each <code>Rule</code>. You also specify a default action, either <code>ALLOW</code> or <code>BLOCK</code>. If a web request doesn't match any of the <code>Rules</code> in a <code>WebACL</code>, AWS WAF responds to the request with the default action.  To create and configure a <code>WebACL</code>, perform the following steps: <ol> <li> Create and update the <code>ByteMatchSet</code> objects and other predicates that you want to include in <code>Rules</code>. For more information, see <a>CreateByteMatchSet</a>, <a>UpdateByteMatchSet</a>, <a>CreateIPSet</a>, <a>UpdateIPSet</a>, <a>CreateSqlInjectionMatchSet</a>, and <a>UpdateSqlInjectionMatchSet</a>. </li> <li> Create and update the <code>Rules</code> that you want to include in the <code>WebACL</code>. For more information, see <a>CreateRule</a> and <a>UpdateRule</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>CreateWebACL</code> request. </li> <li> Submit a <code>CreateWebACL</code> request. </li> <li> Use <code>GetChangeToken</code> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <a>UpdateWebACL</a> request. </li> <li> Submit an <a>UpdateWebACL</a> request to specify the <code>Rules</code> that you want to include in the <code>WebACL</code>, to specify the default action, and to associate the <code>WebACL</code> with a CloudFront distribution. </li> </ol> For more information about how to use the AWS WAF API, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param Tags:  <br/>ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>DefaultAction:  The action that you want AWS WAF to take when a request doesn't match the criteria specified in any of the <code>Rule</code> objects that are associated with the <code>WebACL</code>.:<br/>MetricName:  A friendly name or description for the metrics for this <code>WebACL</code>.The name can contain only alphanumeric characters (A-Z, a-z, 0-9), with maximum length 128 and minimum length one. It can't contain whitespace or metric names reserved for AWS WAF, including "All" and "Default_Action." You can't change <code>MetricName</code> after you create the <code>WebACL</code>.<br/>Name:  A friendly name or description of the <a>WebACL</a>. You can't change <code>Name</code> after you create the <code>WebACL</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> Specifies how you want AWS WAF to respond to requests that match the settings in a <code>Rule</code>. Valid settings include the following: <ul> <li>  <code>ALLOW</code>: AWS WAF allows requests </li> <li>  <code>BLOCK</code>: AWS WAF blocks requests </li> <li>  <code>COUNT</code>: AWS WAF increments a counter of the requests that match all of the conditions in the rule. AWS WAF then continues to inspect the web request based on the remaining rules in the web ACL. You can't specify <code>COUNT</code> for the default action for a <code>WebACL</code>. </li> </ul><br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>CreateWebACL</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>WebACL:  The <a>WebACL</a> returned in the <code>CreateWebACL</code> response.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WebACLArn</i> Tha Amazon Resource Name (ARN) of the web ACL.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Rules</i> An array that contains the action for each <code>Rule</code> in a <code>WebACL</code>, the priority of the <code>Rule</code>, and the ID of the <code>Rule</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultAction</i> The action to perform if none of the <code>Rules</code> contained in the <code>WebACL</code> match. The action is specified by the <a>WafAction</a> object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DefaultAction</b>: The action to perform if none of the <code>Rules</code> contained in the <code>WebACL</code> match. The action is specified by the <a>WafAction</a> object.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricName</i> A friendly name or description for the metrics for this <code>WebACL</code>. The name can contain only alphanumeric characters (A-Z, a-z, 0-9), with maximum length 128 and minimum length one. It can't contain whitespace or metric names reserved for AWS WAF, including "All" and "Default_Action." You can't change <code>MetricName</code> after you create the <code>WebACL</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A friendly name or description of the <code>WebACL</code>. You can't change the name of a <code>WebACL</code> after you create it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WebACLId</i> A unique identifier for a <code>WebACL</code>. You use <code>WebACLId</code> to get information about a <code>WebACL</code> (see <a>GetWebACL</a>), update a <code>WebACL</code> (see <a>UpdateWebACL</a>), and delete a <code>WebACL</code> from AWS WAF (see <a>DeleteWebACL</a>).  <code>WebACLId</code> is returned by <a>CreateWebACL</a> and by <a>ListWebACLs</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> Specifies how you want AWS WAF to respond to requests that match the settings in a <code>Rule</code>. Valid settings include the following: <ul> <li>  <code>ALLOW</code>: AWS WAF allows requests </li> <li>  <code>BLOCK</code>: AWS WAF blocks requests </li> <li>  <code>COUNT</code>: AWS WAF increments a counter of the requests that match all of the conditions in the rule. AWS WAF then continues to inspect the web request based on the remaining rules in the web ACL. You can't specify <code>COUNT</code> for the default action for a <code>WebACL</code>. </li> </ul><br/>        """
        pass

    def create_xss_match_set(self, ChangeToken: str, Name: str) -> CreateXssMatchSetResponse:
        """Creates an <a>XssMatchSet</a>, which you use to allow, block, or count requests that contain cross-site scripting attacks in the specified part of web requests. AWS WAF searches for character sequences that are likely to be malicious strings. To create and configure an <code>XssMatchSet</code>, perform the following steps: <ol> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>CreateXssMatchSet</code> request. </li> <li> Submit a <code>CreateXssMatchSet</code> request. </li> <li> Use <code>GetChangeToken</code> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <a>UpdateXssMatchSet</a> request. </li> <li> Submit an <a>UpdateXssMatchSet</a> request to specify the parts of web requests in which you want to allow, block, or count cross-site scripting attacks. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>Name:  A friendly name or description for the <a>XssMatchSet</a> that you're creating. You can't change <code>Name</code> after you create the <code>XssMatchSet</code>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>CreateXssMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>XssMatchSet:  An <a>XssMatchSet</a>.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>XssMatchTuples</i> Specifies the parts of web requests that you want to inspect for cross-site scripting attacks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name, if any, of the <code>XssMatchSet</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>XssMatchSetId</i> A unique identifier for an <code>XssMatchSet</code>. You use <code>XssMatchSetId</code> to get information about an <code>XssMatchSet</code> (see <a>GetXssMatchSet</a>), update an <code>XssMatchSet</code> (see <a>UpdateXssMatchSet</a>), insert an <code>XssMatchSet</code> into a <code>Rule</code> or delete one from a <code>Rule</code> (see <a>UpdateRule</a>), and delete an <code>XssMatchSet</code> from AWS WAF (see <a>DeleteXssMatchSet</a>).  <code>XssMatchSetId</code> is returned by <a>CreateXssMatchSet</a> and by <a>ListXssMatchSets</a>.<br/>        """
        pass

    def delete_byte_match_set(self, ChangeToken: str, ByteMatchSetId: str) -> DeleteByteMatchSetResponse:
        """Permanently deletes a <a>ByteMatchSet</a>. You can't delete a <code>ByteMatchSet</code> if it's still used in any <code>Rules</code> or if it still includes any <a>ByteMatchTuple</a> objects (any filters). If you just want to remove a <code>ByteMatchSet</code> from a <code>Rule</code>, use <a>UpdateRule</a>. To permanently delete a <code>ByteMatchSet</code>, perform the following steps: <ol> <li> Update the <code>ByteMatchSet</code> to remove filters, if any. For more information, see <a>UpdateByteMatchSet</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>DeleteByteMatchSet</code> request. </li> <li> Submit a <code>DeleteByteMatchSet</code> request. </li> </ol>

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>ByteMatchSetId:  The <code>ByteMatchSetId</code> of the <a>ByteMatchSet</a> that you want to delete. <code>ByteMatchSetId</code> is returned by <a>CreateByteMatchSet</a> and by <a>ListByteMatchSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>DeleteByteMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def delete_geo_match_set(self, ChangeToken: str, GeoMatchSetId: str) -> DeleteGeoMatchSetResponse:
        """Permanently deletes a <a>GeoMatchSet</a>. You can't delete a <code>GeoMatchSet</code> if it's still used in any <code>Rules</code> or if it still includes any countries. If you just want to remove a <code>GeoMatchSet</code> from a <code>Rule</code>, use <a>UpdateRule</a>. To permanently delete a <code>GeoMatchSet</code> from AWS WAF, perform the following steps: <ol> <li> Update the <code>GeoMatchSet</code> to remove any countries. For more information, see <a>UpdateGeoMatchSet</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>DeleteGeoMatchSet</code> request. </li> <li> Submit a <code>DeleteGeoMatchSet</code> request. </li> </ol>

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>GeoMatchSetId:  The <code>GeoMatchSetID</code> of the <a>GeoMatchSet</a> that you want to delete. <code>GeoMatchSetId</code> is returned by <a>CreateGeoMatchSet</a> and by <a>ListGeoMatchSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>DeleteGeoMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def delete_ip_set(self, ChangeToken: str, IPSetId: str) -> DeleteIPSetResponse:
        """Permanently deletes an <a>IPSet</a>. You can't delete an <code>IPSet</code> if it's still used in any <code>Rules</code> or if it still includes any IP addresses. If you just want to remove an <code>IPSet</code> from a <code>Rule</code>, use <a>UpdateRule</a>. To permanently delete an <code>IPSet</code> from AWS WAF, perform the following steps: <ol> <li> Update the <code>IPSet</code> to remove IP address ranges, if any. For more information, see <a>UpdateIPSet</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>DeleteIPSet</code> request. </li> <li> Submit a <code>DeleteIPSet</code> request. </li> </ol>

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>IPSetId:  The <code>IPSetId</code> of the <a>IPSet</a> that you want to delete. <code>IPSetId</code> is returned by <a>CreateIPSet</a> and by <a>ListIPSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>DeleteIPSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def delete_logging_configuration(self, ResourceArn: str) -> DeleteLoggingConfigurationResponse:
        """Permanently deletes the <a>LoggingConfiguration</a> from the specified web ACL.

        :param self:
        :param ResourceArn:  The Amazon Resource Name (ARN) of the web ACL from which you want to delete the <a>LoggingConfiguration</a>.<br/>
        :return:         """
        pass

    def delete_permission_policy(self, ResourceArn: str) -> DeletePermissionPolicyResponse:
        """Permanently deletes an IAM policy from the specified RuleGroup. The user making the request must be the owner of the RuleGroup.

        :param self:
        :param ResourceArn:  The Amazon Resource Name (ARN) of the RuleGroup from which you want to delete the policy. The user making the request must be the owner of the RuleGroup.<br/>
        :return:         """
        pass

    def delete_rate_based_rule(self, ChangeToken: str, RuleId: str) -> DeleteRateBasedRuleResponse:
        """Permanently deletes a <a>RateBasedRule</a>. You can't delete a rule if it's still used in any <code>WebACL</code> objects or if it still includes any predicates, such as <code>ByteMatchSet</code> objects. If you just want to remove a rule from a <code>WebACL</code>, use <a>UpdateWebACL</a>. To permanently delete a <code>RateBasedRule</code> from AWS WAF, perform the following steps: <ol> <li> Update the <code>RateBasedRule</code> to remove predicates, if any. For more information, see <a>UpdateRateBasedRule</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>DeleteRateBasedRule</code> request. </li> <li> Submit a <code>DeleteRateBasedRule</code> request. </li> </ol>

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>RuleId:  The <code>RuleId</code> of the <a>RateBasedRule</a> that you want to delete. <code>RuleId</code> is returned by <a>CreateRateBasedRule</a> and by <a>ListRateBasedRules</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>DeleteRateBasedRule</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def delete_regex_match_set(self, ChangeToken: str, RegexMatchSetId: str) -> DeleteRegexMatchSetResponse:
        """Permanently deletes a <a>RegexMatchSet</a>. You can't delete a <code>RegexMatchSet</code> if it's still used in any <code>Rules</code> or if it still includes any <code>RegexMatchTuples</code> objects (any filters). If you just want to remove a <code>RegexMatchSet</code> from a <code>Rule</code>, use <a>UpdateRule</a>. To permanently delete a <code>RegexMatchSet</code>, perform the following steps: <ol> <li> Update the <code>RegexMatchSet</code> to remove filters, if any. For more information, see <a>UpdateRegexMatchSet</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>DeleteRegexMatchSet</code> request. </li> <li> Submit a <code>DeleteRegexMatchSet</code> request. </li> </ol>

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>RegexMatchSetId:  The <code>RegexMatchSetId</code> of the <a>RegexMatchSet</a> that you want to delete. <code>RegexMatchSetId</code> is returned by <a>CreateRegexMatchSet</a> and by <a>ListRegexMatchSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>DeleteRegexMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def delete_regex_pattern_set(self, ChangeToken: str, RegexPatternSetId: str) -> DeleteRegexPatternSetResponse:
        """Permanently deletes a <a>RegexPatternSet</a>. You can't delete a <code>RegexPatternSet</code> if it's still used in any <code>RegexMatchSet</code> or if the <code>RegexPatternSet</code> is not empty. 

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>RegexPatternSetId:  The <code>RegexPatternSetId</code> of the <a>RegexPatternSet</a> that you want to delete. <code>RegexPatternSetId</code> is returned by <a>CreateRegexPatternSet</a> and by <a>ListRegexPatternSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>DeleteRegexPatternSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def delete_rule(self, ChangeToken: str, RuleId: str) -> DeleteRuleResponse:
        """Permanently deletes a <a>Rule</a>. You can't delete a <code>Rule</code> if it's still used in any <code>WebACL</code> objects or if it still includes any predicates, such as <code>ByteMatchSet</code> objects. If you just want to remove a <code>Rule</code> from a <code>WebACL</code>, use <a>UpdateWebACL</a>. To permanently delete a <code>Rule</code> from AWS WAF, perform the following steps: <ol> <li> Update the <code>Rule</code> to remove predicates, if any. For more information, see <a>UpdateRule</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>DeleteRule</code> request. </li> <li> Submit a <code>DeleteRule</code> request. </li> </ol>

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>RuleId:  The <code>RuleId</code> of the <a>Rule</a> that you want to delete. <code>RuleId</code> is returned by <a>CreateRule</a> and by <a>ListRules</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>DeleteRule</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def delete_rule_group(self, ChangeToken: str, RuleGroupId: str) -> DeleteRuleGroupResponse:
        """Permanently deletes a <a>RuleGroup</a>. You can't delete a <code>RuleGroup</code> if it's still used in any <code>WebACL</code> objects or if it still includes any rules. If you just want to remove a <code>RuleGroup</code> from a <code>WebACL</code>, use <a>UpdateWebACL</a>. To permanently delete a <code>RuleGroup</code> from AWS WAF, perform the following steps: <ol> <li> Update the <code>RuleGroup</code> to remove rules, if any. For more information, see <a>UpdateRuleGroup</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>DeleteRuleGroup</code> request. </li> <li> Submit a <code>DeleteRuleGroup</code> request. </li> </ol>

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>RuleGroupId:  The <code>RuleGroupId</code> of the <a>RuleGroup</a> that you want to delete. <code>RuleGroupId</code> is returned by <a>CreateRuleGroup</a> and by <a>ListRuleGroups</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>DeleteRuleGroup</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def delete_size_constraint_set(self, ChangeToken: str, SizeConstraintSetId: str) -> DeleteSizeConstraintSetResponse:
        """Permanently deletes a <a>SizeConstraintSet</a>. You can't delete a <code>SizeConstraintSet</code> if it's still used in any <code>Rules</code> or if it still includes any <a>SizeConstraint</a> objects (any filters). If you just want to remove a <code>SizeConstraintSet</code> from a <code>Rule</code>, use <a>UpdateRule</a>. To permanently delete a <code>SizeConstraintSet</code>, perform the following steps: <ol> <li> Update the <code>SizeConstraintSet</code> to remove filters, if any. For more information, see <a>UpdateSizeConstraintSet</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>DeleteSizeConstraintSet</code> request. </li> <li> Submit a <code>DeleteSizeConstraintSet</code> request. </li> </ol>

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>SizeConstraintSetId:  The <code>SizeConstraintSetId</code> of the <a>SizeConstraintSet</a> that you want to delete. <code>SizeConstraintSetId</code> is returned by <a>CreateSizeConstraintSet</a> and by <a>ListSizeConstraintSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>DeleteSizeConstraintSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def delete_sql_injection_match_set(self, ChangeToken: str, SqlInjectionMatchSetId: str) -> DeleteSqlInjectionMatchSetResponse:
        """Permanently deletes a <a>SqlInjectionMatchSet</a>. You can't delete a <code>SqlInjectionMatchSet</code> if it's still used in any <code>Rules</code> or if it still contains any <a>SqlInjectionMatchTuple</a> objects. If you just want to remove a <code>SqlInjectionMatchSet</code> from a <code>Rule</code>, use <a>UpdateRule</a>. To permanently delete a <code>SqlInjectionMatchSet</code> from AWS WAF, perform the following steps: <ol> <li> Update the <code>SqlInjectionMatchSet</code> to remove filters, if any. For more information, see <a>UpdateSqlInjectionMatchSet</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>DeleteSqlInjectionMatchSet</code> request. </li> <li> Submit a <code>DeleteSqlInjectionMatchSet</code> request. </li> </ol>

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>SqlInjectionMatchSetId:  The <code>SqlInjectionMatchSetId</code> of the <a>SqlInjectionMatchSet</a> that you want to delete. <code>SqlInjectionMatchSetId</code> is returned by <a>CreateSqlInjectionMatchSet</a> and by <a>ListSqlInjectionMatchSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>DeleteSqlInjectionMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def delete_web_acl(self, ChangeToken: str, WebACLId: str) -> DeleteWebACLResponse:
        """Permanently deletes a <a>WebACL</a>. You can't delete a <code>WebACL</code> if it still contains any <code>Rules</code>. To delete a <code>WebACL</code>, perform the following steps: <ol> <li> Update the <code>WebACL</code> to remove <code>Rules</code>, if any. For more information, see <a>UpdateWebACL</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>DeleteWebACL</code> request. </li> <li> Submit a <code>DeleteWebACL</code> request. </li> </ol>

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>WebACLId:  The <code>WebACLId</code> of the <a>WebACL</a> that you want to delete. <code>WebACLId</code> is returned by <a>CreateWebACL</a> and by <a>ListWebACLs</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>DeleteWebACL</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def delete_xss_match_set(self, ChangeToken: str, XssMatchSetId: str) -> DeleteXssMatchSetResponse:
        """Permanently deletes an <a>XssMatchSet</a>. You can't delete an <code>XssMatchSet</code> if it's still used in any <code>Rules</code> or if it still contains any <a>XssMatchTuple</a> objects. If you just want to remove an <code>XssMatchSet</code> from a <code>Rule</code>, use <a>UpdateRule</a>. To permanently delete an <code>XssMatchSet</code> from AWS WAF, perform the following steps: <ol> <li> Update the <code>XssMatchSet</code> to remove filters, if any. For more information, see <a>UpdateXssMatchSet</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>DeleteXssMatchSet</code> request. </li> <li> Submit a <code>DeleteXssMatchSet</code> request. </li> </ol>

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>XssMatchSetId:  The <code>XssMatchSetId</code> of the <a>XssMatchSet</a> that you want to delete. <code>XssMatchSetId</code> is returned by <a>CreateXssMatchSet</a> and by <a>ListXssMatchSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>DeleteXssMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def get_byte_match_set(self, ByteMatchSetId: str) -> GetByteMatchSetResponse:
        """Returns the <a>ByteMatchSet</a> specified by <code>ByteMatchSetId</code>.

        :param self:
        :param ByteMatchSetId:  The <code>ByteMatchSetId</code> of the <a>ByteMatchSet</a> that you want to get. <code>ByteMatchSetId</code> is returned by <a>CreateByteMatchSet</a> and by <a>ListByteMatchSets</a>.<br/>
        :return: ByteMatchSet:  Information about the <a>ByteMatchSet</a> that you specified in the <code>GetByteMatchSet</code> request. For more information, see the following topics: <ul> <li>  <a>ByteMatchSet</a>: Contains <code>ByteMatchSetId</code>, <code>ByteMatchTuples</code>, and <code>Name</code>  </li> <li>  <code>ByteMatchTuples</code>: Contains an array of <a>ByteMatchTuple</a> objects. Each <code>ByteMatchTuple</code> object contains <a>FieldToMatch</a>, <code>PositionalConstraint</code>, <code>TargetString</code>, and <code>TextTransformation</code>  </li> <li>  <a>FieldToMatch</a>: Contains <code>Data</code> and <code>Type</code>  </li> </ul>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ByteMatchTuples</i> Specifies the bytes (typically a string that corresponds with ASCII characters) that you want AWS WAF to search for in web requests, the location in requests that you want AWS WAF to search, and other settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A friendly name or description of the <a>ByteMatchSet</a>. You can't change <code>Name</code> after you create a <code>ByteMatchSet</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ByteMatchSetId</i> The <code>ByteMatchSetId</code> for a <code>ByteMatchSet</code>. You use <code>ByteMatchSetId</code> to get information about a <code>ByteMatchSet</code> (see <a>GetByteMatchSet</a>), update a <code>ByteMatchSet</code> (see <a>UpdateByteMatchSet</a>), insert a <code>ByteMatchSet</code> into a <code>Rule</code> or delete one from a <code>Rule</code> (see <a>UpdateRule</a>), and delete a <code>ByteMatchSet</code> from AWS WAF (see <a>DeleteByteMatchSet</a>).  <code>ByteMatchSetId</code> is returned by <a>CreateByteMatchSet</a> and by <a>ListByteMatchSets</a>.<br/>        """
        pass

    def get_change_token(self) -> GetChangeTokenResponse:
        """When you want to create, update, or delete AWS WAF objects, get a change token and include the change token in the create, update, or delete request. Change tokens ensure that your application doesn't submit conflicting requests to AWS WAF. Each create, update, or delete request must use a unique change token. If your application submits a <code>GetChangeToken</code> request and then submits a second <code>GetChangeToken</code> request before submitting a create, update, or delete request, the second <code>GetChangeToken</code> request returns the same value as the first <code>GetChangeToken</code> request. When you use a change token in a create, update, or delete request, the status of the change token changes to <code>PENDING</code>, which indicates that AWS WAF is propagating the change to all AWS WAF servers. Use <code>GetChangeTokenStatus</code> to determine the status of your change token.

        :param self:
        :param 
        :return: ChangeToken:  The <code>ChangeToken</code> that you used in the request. Use this value in a <code>GetChangeTokenStatus</code> request to get the current status of the request. <br/>        """
        pass

    def get_change_token_status(self, ChangeToken: str) -> GetChangeTokenStatusResponse:
        """Returns the status of a <code>ChangeToken</code> that you got by calling <a>GetChangeToken</a>. <code>ChangeTokenStatus</code> is one of the following values: <ul> <li>  <code>PROVISIONED</code>: You requested the change token by calling <code>GetChangeToken</code>, but you haven't used it yet in a call to create, update, or delete an AWS WAF object. </li> <li>  <code>PENDING</code>: AWS WAF is propagating the create, update, or delete request to all AWS WAF servers. </li> <li>  <code>INSYNC</code>: Propagation is complete. </li> </ul>

        :param self:
        :param ChangeToken:  The change token for which you want to get the status. This change token was previously returned in the <code>GetChangeToken</code> response.<br/>
        :return: ChangeTokenStatus:  The status of the change token.<br/>        """
        pass

    def get_geo_match_set(self, GeoMatchSetId: str) -> GetGeoMatchSetResponse:
        """Returns the <a>GeoMatchSet</a> that is specified by <code>GeoMatchSetId</code>.

        :param self:
        :param GeoMatchSetId:  The <code>GeoMatchSetId</code> of the <a>GeoMatchSet</a> that you want to get. <code>GeoMatchSetId</code> is returned by <a>CreateGeoMatchSet</a> and by <a>ListGeoMatchSets</a>.<br/>
        :return: GeoMatchSet:  Information about the <a>GeoMatchSet</a> that you specified in the <code>GetGeoMatchSet</code> request. This includes the <code>Type</code>, which for a <code>GeoMatchContraint</code> is always <code>Country</code>, as well as the <code>Value</code>, which is the identifier for a specific country.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GeoMatchConstraints</i> An array of <a>GeoMatchConstraint</a> objects, which contain the country that you want AWS WAF to search for.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A friendly name or description of the <a>GeoMatchSet</a>. You can't change the name of an <code>GeoMatchSet</code> after you create it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GeoMatchSetId</i> The <code>GeoMatchSetId</code> for an <code>GeoMatchSet</code>. You use <code>GeoMatchSetId</code> to get information about a <code>GeoMatchSet</code> (see <a>GeoMatchSet</a>), update a <code>GeoMatchSet</code> (see <a>UpdateGeoMatchSet</a>), insert a <code>GeoMatchSet</code> into a <code>Rule</code> or delete one from a <code>Rule</code> (see <a>UpdateRule</a>), and delete a <code>GeoMatchSet</code> from AWS WAF (see <a>DeleteGeoMatchSet</a>).  <code>GeoMatchSetId</code> is returned by <a>CreateGeoMatchSet</a> and by <a>ListGeoMatchSets</a>.<br/>        """
        pass

    def get_ip_set(self, IPSetId: str) -> GetIPSetResponse:
        """Returns the <a>IPSet</a> that is specified by <code>IPSetId</code>.

        :param self:
        :param IPSetId:  The <code>IPSetId</code> of the <a>IPSet</a> that you want to get. <code>IPSetId</code> is returned by <a>CreateIPSet</a> and by <a>ListIPSets</a>.<br/>
        :return: IPSet:  Information about the <a>IPSet</a> that you specified in the <code>GetIPSet</code> request. For more information, see the following topics: <ul> <li>  <a>IPSet</a>: Contains <code>IPSetDescriptors</code>, <code>IPSetId</code>, and <code>Name</code>  </li> <li>  <code>IPSetDescriptors</code>: Contains an array of <a>IPSetDescriptor</a> objects. Each <code>IPSetDescriptor</code> object contains <code>Type</code> and <code>Value</code>  </li> </ul>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IPSetDescriptors</i> The IP address type (<code>IPV4</code> or <code>IPV6</code>) and the IP address range (in CIDR notation) that web requests originate from. If the <code>WebACL</code> is associated with a CloudFront distribution and the viewer did not use an HTTP proxy or a load balancer to send the request, this is the value of the c-ip field in the CloudFront access logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A friendly name or description of the <a>IPSet</a>. You can't change the name of an <code>IPSet</code> after you create it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IPSetId</i> The <code>IPSetId</code> for an <code>IPSet</code>. You use <code>IPSetId</code> to get information about an <code>IPSet</code> (see <a>GetIPSet</a>), update an <code>IPSet</code> (see <a>UpdateIPSet</a>), insert an <code>IPSet</code> into a <code>Rule</code> or delete one from a <code>Rule</code> (see <a>UpdateRule</a>), and delete an <code>IPSet</code> from AWS WAF (see <a>DeleteIPSet</a>).  <code>IPSetId</code> is returned by <a>CreateIPSet</a> and by <a>ListIPSets</a>.<br/>        """
        pass

    def get_logging_configuration(self, ResourceArn: str) -> GetLoggingConfigurationResponse:
        """Returns the <a>LoggingConfiguration</a> for the specified web ACL.

        :param self:
        :param ResourceArn:  The Amazon Resource Name (ARN) of the web ACL for which you want to get the <a>LoggingConfiguration</a>.<br/>
        :return: LoggingConfiguration:  The <a>LoggingConfiguration</a> for the specified web ACL.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RedactedFields</i> The parts of the request that you want redacted from the logs. For example, if you redact the cookie field, the cookie field in the firehose will be <code>xxx</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogDestinationConfigs</i> An array of Amazon Kinesis Data Firehose ARNs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceArn</i> The Amazon Resource Name (ARN) of the web ACL that you want to associate with <code>LogDestinationConfigs</code>.<br/>        """
        pass

    def get_permission_policy(self, ResourceArn: str) -> GetPermissionPolicyResponse:
        """Returns the IAM policy attached to the RuleGroup.

        :param self:
        :param ResourceArn:  The Amazon Resource Name (ARN) of the RuleGroup for which you want to get the policy.<br/>
        :return: Policy:  The IAM policy attached to the specified RuleGroup.<br/>        """
        pass

    def get_rate_based_rule(self, RuleId: str) -> GetRateBasedRuleResponse:
        """Returns the <a>RateBasedRule</a> that is specified by the <code>RuleId</code> that you included in the <code>GetRateBasedRule</code> request.

        :param self:
        :param RuleId:  The <code>RuleId</code> of the <a>RateBasedRule</a> that you want to get. <code>RuleId</code> is returned by <a>CreateRateBasedRule</a> and by <a>ListRateBasedRules</a>.<br/>
        :return: Rule:  Information about the <a>RateBasedRule</a> that you specified in the <code>GetRateBasedRule</code> request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RateLimit</i> The maximum number of requests, which have an identical value in the field specified by the <code>RateKey</code>, allowed in a five-minute period. If the number of requests exceeds the <code>RateLimit</code> and the other predicates specified in the rule are also met, AWS WAF triggers the action that is specified for this rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RateKey</i> The field that AWS WAF uses to determine if requests are likely arriving from single source and thus subject to rate monitoring. The only valid value for <code>RateKey</code> is <code>IP</code>. <code>IP</code> indicates that requests arriving from the same IP address are subject to the <code>RateLimit</code> that is specified in the <code>RateBasedRule</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MatchPredicates</i> The <code>Predicates</code> object contains one <code>Predicate</code> element for each <a>ByteMatchSet</a>, <a>IPSet</a>, or <a>SqlInjectionMatchSet</a> object that you want to include in a <code>RateBasedRule</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricName</i> A friendly name or description for the metrics for a <code>RateBasedRule</code>. The name can contain only alphanumeric characters (A-Z, a-z, 0-9), with maximum length 128 and minimum length one. It can't contain whitespace or metric names reserved for AWS WAF, including "All" and "Default_Action." You can't change the name of the metric after you create the <code>RateBasedRule</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A friendly name or description for a <code>RateBasedRule</code>. You can't change the name of a <code>RateBasedRule</code> after you create it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleId</i> A unique identifier for a <code>RateBasedRule</code>. You use <code>RuleId</code> to get more information about a <code>RateBasedRule</code> (see <a>GetRateBasedRule</a>), update a <code>RateBasedRule</code> (see <a>UpdateRateBasedRule</a>), insert a <code>RateBasedRule</code> into a <code>WebACL</code> or delete one from a <code>WebACL</code> (see <a>UpdateWebACL</a>), or delete a <code>RateBasedRule</code> from AWS WAF (see <a>DeleteRateBasedRule</a>).<br/>        """
        pass

    def get_rate_based_rule_managed_keys(self, RuleId: str, NextMarker: str=None) -> GetRateBasedRuleManagedKeysResponse:
        """Returns an array of IP addresses currently being blocked by the <a>RateBasedRule</a> that is specified by the <code>RuleId</code>. The maximum number of managed keys that will be blocked is 10,000. If more than 10,000 addresses exceed the rate limit, the 10,000 addresses with the highest rates will be blocked.

        :param self:
        :param NextMarker:  A null value and not currently used. Do not include this in your request.<br/>RuleId:  The <code>RuleId</code> of the <a>RateBasedRule</a> for which you want to get a list of <code>ManagedKeys</code>. <code>RuleId</code> is returned by <a>CreateRateBasedRule</a> and by <a>ListRateBasedRules</a>.<br/>
        :return: NextMarker:  A null value and not currently used.<br/>ManagedKeys:  An array of IP addresses that currently are blocked by the specified <a>RateBasedRule</a>. <br/>        """
        pass

    def get_regex_match_set(self, RegexMatchSetId: str) -> GetRegexMatchSetResponse:
        """Returns the <a>RegexMatchSet</a> specified by <code>RegexMatchSetId</code>.

        :param self:
        :param RegexMatchSetId:  The <code>RegexMatchSetId</code> of the <a>RegexMatchSet</a> that you want to get. <code>RegexMatchSetId</code> is returned by <a>CreateRegexMatchSet</a> and by <a>ListRegexMatchSets</a>.<br/>
        :return: RegexMatchSet:  Information about the <a>RegexMatchSet</a> that you specified in the <code>GetRegexMatchSet</code> request. For more information, see <a>RegexMatchTuple</a>.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RegexMatchTuples</i> Contains an array of <a>RegexMatchTuple</a> objects. Each <code>RegexMatchTuple</code> object contains:  <ul> <li> The part of a web request that you want AWS WAF to inspect, such as a query string or the value of the <code>User-Agent</code> header.  </li> <li> The identifier of the pattern (a regular expression) that you want AWS WAF to look for. For more information, see <a>RegexPatternSet</a>. </li> <li> Whether to perform any conversions on the request, such as converting it to lowercase, before inspecting it for the specified string. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A friendly name or description of the <a>RegexMatchSet</a>. You can't change <code>Name</code> after you create a <code>RegexMatchSet</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RegexMatchSetId</i> The <code>RegexMatchSetId</code> for a <code>RegexMatchSet</code>. You use <code>RegexMatchSetId</code> to get information about a <code>RegexMatchSet</code> (see <a>GetRegexMatchSet</a>), update a <code>RegexMatchSet</code> (see <a>UpdateRegexMatchSet</a>), insert a <code>RegexMatchSet</code> into a <code>Rule</code> or delete one from a <code>Rule</code> (see <a>UpdateRule</a>), and delete a <code>RegexMatchSet</code> from AWS WAF (see <a>DeleteRegexMatchSet</a>).  <code>RegexMatchSetId</code> is returned by <a>CreateRegexMatchSet</a> and by <a>ListRegexMatchSets</a>.<br/>        """
        pass

    def get_regex_pattern_set(self, RegexPatternSetId: str) -> GetRegexPatternSetResponse:
        """Returns the <a>RegexPatternSet</a> specified by <code>RegexPatternSetId</code>.

        :param self:
        :param RegexPatternSetId:  The <code>RegexPatternSetId</code> of the <a>RegexPatternSet</a> that you want to get. <code>RegexPatternSetId</code> is returned by <a>CreateRegexPatternSet</a> and by <a>ListRegexPatternSets</a>.<br/>
        :return: RegexPatternSet:  Information about the <a>RegexPatternSet</a> that you specified in the <code>GetRegexPatternSet</code> request, including the identifier of the pattern set and the regular expression patterns you want AWS WAF to search for. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RegexPatternStrings</i> Specifies the regular expression (regex) patterns that you want AWS WAF to search for, such as <code>B[a@]dB[o0]t</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A friendly name or description of the <a>RegexPatternSet</a>. You can't change <code>Name</code> after you create a <code>RegexPatternSet</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RegexPatternSetId</i> The identifier for the <code>RegexPatternSet</code>. You use <code>RegexPatternSetId</code> to get information about a <code>RegexPatternSet</code>, update a <code>RegexPatternSet</code>, remove a <code>RegexPatternSet</code> from a <code>RegexMatchSet</code>, and delete a <code>RegexPatternSet</code> from AWS WAF.  <code>RegexMatchSetId</code> is returned by <a>CreateRegexPatternSet</a> and by <a>ListRegexPatternSets</a>.<br/>        """
        pass

    def get_rule(self, RuleId: str) -> GetRuleResponse:
        """Returns the <a>Rule</a> that is specified by the <code>RuleId</code> that you included in the <code>GetRule</code> request.

        :param self:
        :param RuleId:  The <code>RuleId</code> of the <a>Rule</a> that you want to get. <code>RuleId</code> is returned by <a>CreateRule</a> and by <a>ListRules</a>.<br/>
        :return: Rule:  Information about the <a>Rule</a> that you specified in the <code>GetRule</code> request. For more information, see the following topics: <ul> <li>  <a>Rule</a>: Contains <code>MetricName</code>, <code>Name</code>, an array of <code>Predicate</code> objects, and <code>RuleId</code>  </li> <li>  <a>Predicate</a>: Each <code>Predicate</code> object contains <code>DataId</code>, <code>Negated</code>, and <code>Type</code>  </li> </ul>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Predicates</i> The <code>Predicates</code> object contains one <code>Predicate</code> element for each <a>ByteMatchSet</a>, <a>IPSet</a>, or <a>SqlInjectionMatchSet</a> object that you want to include in a <code>Rule</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricName</i> A friendly name or description for the metrics for this <code>Rule</code>. The name can contain only alphanumeric characters (A-Z, a-z, 0-9), with maximum length 128 and minimum length one. It can't contain whitespace or metric names reserved for AWS WAF, including "All" and "Default_Action." You can't change <code>MetricName</code> after you create the <code>Rule</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The friendly name or description for the <code>Rule</code>. You can't change the name of a <code>Rule</code> after you create it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleId</i> A unique identifier for a <code>Rule</code>. You use <code>RuleId</code> to get more information about a <code>Rule</code> (see <a>GetRule</a>), update a <code>Rule</code> (see <a>UpdateRule</a>), insert a <code>Rule</code> into a <code>WebACL</code> or delete a one from a <code>WebACL</code> (see <a>UpdateWebACL</a>), or delete a <code>Rule</code> from AWS WAF (see <a>DeleteRule</a>).  <code>RuleId</code> is returned by <a>CreateRule</a> and by <a>ListRules</a>.<br/>        """
        pass

    def get_rule_group(self, RuleGroupId: str) -> GetRuleGroupResponse:
        """Returns the <a>RuleGroup</a> that is specified by the <code>RuleGroupId</code> that you included in the <code>GetRuleGroup</code> request. To view the rules in a rule group, use <a>ListActivatedRulesInRuleGroup</a>.

        :param self:
        :param RuleGroupId:  The <code>RuleGroupId</code> of the <a>RuleGroup</a> that you want to get. <code>RuleGroupId</code> is returned by <a>CreateRuleGroup</a> and by <a>ListRuleGroups</a>.<br/>
        :return: RuleGroup:  Information about the <a>RuleGroup</a> that you specified in the <code>GetRuleGroup</code> request. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricName</i> A friendly name or description for the metrics for this <code>RuleGroup</code>. The name can contain only alphanumeric characters (A-Z, a-z, 0-9), with maximum length 128 and minimum length one. It can't contain whitespace or metric names reserved for AWS WAF, including "All" and "Default_Action." You can't change the name of the metric after you create the <code>RuleGroup</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The friendly name or description for the <code>RuleGroup</code>. You can't change the name of a <code>RuleGroup</code> after you create it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleGroupId</i> A unique identifier for a <code>RuleGroup</code>. You use <code>RuleGroupId</code> to get more information about a <code>RuleGroup</code> (see <a>GetRuleGroup</a>), update a <code>RuleGroup</code> (see <a>UpdateRuleGroup</a>), insert a <code>RuleGroup</code> into a <code>WebACL</code> or delete a one from a <code>WebACL</code> (see <a>UpdateWebACL</a>), or delete a <code>RuleGroup</code> from AWS WAF (see <a>DeleteRuleGroup</a>).  <code>RuleGroupId</code> is returned by <a>CreateRuleGroup</a> and by <a>ListRuleGroups</a>.<br/>        """
        pass

    def get_sampled_requests(self, MaxItems: GetSampledRequestsMaxItems, TimeWindow: TimeWindow, RuleId: str, WebAclId: str) -> GetSampledRequestsResponse:
        """Gets detailed information about a specified number of requests--a sample--that AWS WAF randomly selects from among the first 5,000 requests that your AWS resource received during a time range that you choose. You can specify a sample size of up to 500 requests, and you can specify any time range in the previous three hours.  <code>GetSampledRequests</code> returns a time range, which is usually the time range that you specified. However, if your resource (such as a CloudFront distribution) received 5,000 requests before the specified time range elapsed, <code>GetSampledRequests</code> returns an updated time range. This new time range indicates the actual period during which AWS WAF selected the requests in the sample.

        :param self:
        :param MaxItems:  The number of requests that you want AWS WAF to return from among the first 5,000 requests that your AWS resource received during the time range. If your resource received fewer requests than the value of <code>MaxItems</code>, <code>GetSampledRequests</code> returns information about all of them. <br/>TimeWindow:  The start date and time and the end date and time of the range for which you want <code>GetSampledRequests</code> to return a sample of requests. Specify the date and time in the following format: <code>"2016-09-27T14:50Z"</code>. You can specify any time range in the previous three hours.:<br/>RuleId:   <code>RuleId</code> is one of three values: <ul> <li> The <code>RuleId</code> of the <code>Rule</code> or the <code>RuleGroupId</code> of the <code>RuleGroup</code> for which you want <code>GetSampledRequests</code> to return a sample of requests. </li> <li>  <code>Default_Action</code>, which causes <code>GetSampledRequests</code> to return a sample of the requests that didn't match any of the rules in the specified <code>WebACL</code>. </li> </ul><br/>WebAclId:  The <code>WebACLId</code> of the <code>WebACL</code> for which you want <code>GetSampledRequests</code> to return a sample of requests.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> The end of the time range from which you want <code>GetSampledRequests</code> to return a sample of the requests that your AWS resource received. Specify the date and time in the following format: <code>"2016-09-27T14:50Z"</code>. You can specify any time range in the previous three hours.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTime</i> The beginning of the time range from which you want <code>GetSampledRequests</code> to return a sample of the requests that your AWS resource received. Specify the date and time in the following format: <code>"2016-09-27T14:50Z"</code>. You can specify any time range in the previous three hours.<br/>
        :return: TimeWindow:  Usually, <code>TimeWindow</code> is the time range that you specified in the <code>GetSampledRequests</code> request. However, if your AWS resource received more than 5,000 requests during the time range that you specified in the request, <code>GetSampledRequests</code> returns the time range for the first 5,000 requests.:<br/>PopulationSize:  The total number of requests from which <code>GetSampledRequests</code> got a sample of <code>MaxItems</code> requests. If <code>PopulationSize</code> is less than <code>MaxItems</code>, the sample includes every request that your AWS resource received during the specified time range.<br/>SampledRequests:  A complex type that contains detailed information about each of the requests in the sample.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> The end of the time range from which you want <code>GetSampledRequests</code> to return a sample of the requests that your AWS resource received. Specify the date and time in the following format: <code>"2016-09-27T14:50Z"</code>. You can specify any time range in the previous three hours.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTime</i> The beginning of the time range from which you want <code>GetSampledRequests</code> to return a sample of the requests that your AWS resource received. Specify the date and time in the following format: <code>"2016-09-27T14:50Z"</code>. You can specify any time range in the previous three hours.<br/>        """
        pass

    def get_size_constraint_set(self, SizeConstraintSetId: str) -> GetSizeConstraintSetResponse:
        """Returns the <a>SizeConstraintSet</a> specified by <code>SizeConstraintSetId</code>.

        :param self:
        :param SizeConstraintSetId:  The <code>SizeConstraintSetId</code> of the <a>SizeConstraintSet</a> that you want to get. <code>SizeConstraintSetId</code> is returned by <a>CreateSizeConstraintSet</a> and by <a>ListSizeConstraintSets</a>.<br/>
        :return: SizeConstraintSet:  Information about the <a>SizeConstraintSet</a> that you specified in the <code>GetSizeConstraintSet</code> request. For more information, see the following topics: <ul> <li>  <a>SizeConstraintSet</a>: Contains <code>SizeConstraintSetId</code>, <code>SizeConstraints</code>, and <code>Name</code>  </li> <li>  <code>SizeConstraints</code>: Contains an array of <a>SizeConstraint</a> objects. Each <code>SizeConstraint</code> object contains <a>FieldToMatch</a>, <code>TextTransformation</code>, <code>ComparisonOperator</code>, and <code>Size</code>  </li> <li>  <a>FieldToMatch</a>: Contains <code>Data</code> and <code>Type</code>  </li> </ul>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SizeConstraints</i> Specifies the parts of web requests that you want to inspect the size of.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name, if any, of the <code>SizeConstraintSet</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SizeConstraintSetId</i> A unique identifier for a <code>SizeConstraintSet</code>. You use <code>SizeConstraintSetId</code> to get information about a <code>SizeConstraintSet</code> (see <a>GetSizeConstraintSet</a>), update a <code>SizeConstraintSet</code> (see <a>UpdateSizeConstraintSet</a>), insert a <code>SizeConstraintSet</code> into a <code>Rule</code> or delete one from a <code>Rule</code> (see <a>UpdateRule</a>), and delete a <code>SizeConstraintSet</code> from AWS WAF (see <a>DeleteSizeConstraintSet</a>).  <code>SizeConstraintSetId</code> is returned by <a>CreateSizeConstraintSet</a> and by <a>ListSizeConstraintSets</a>.<br/>        """
        pass

    def get_sql_injection_match_set(self, SqlInjectionMatchSetId: str) -> GetSqlInjectionMatchSetResponse:
        """Returns the <a>SqlInjectionMatchSet</a> that is specified by <code>SqlInjectionMatchSetId</code>.

        :param self:
        :param SqlInjectionMatchSetId:  The <code>SqlInjectionMatchSetId</code> of the <a>SqlInjectionMatchSet</a> that you want to get. <code>SqlInjectionMatchSetId</code> is returned by <a>CreateSqlInjectionMatchSet</a> and by <a>ListSqlInjectionMatchSets</a>.<br/>
        :return: SqlInjectionMatchSet:  Information about the <a>SqlInjectionMatchSet</a> that you specified in the <code>GetSqlInjectionMatchSet</code> request. For more information, see the following topics: <ul> <li>  <a>SqlInjectionMatchSet</a>: Contains <code>Name</code>, <code>SqlInjectionMatchSetId</code>, and an array of <code>SqlInjectionMatchTuple</code> objects </li> <li>  <a>SqlInjectionMatchTuple</a>: Each <code>SqlInjectionMatchTuple</code> object contains <code>FieldToMatch</code> and <code>TextTransformation</code>  </li> <li>  <a>FieldToMatch</a>: Contains <code>Data</code> and <code>Type</code>  </li> </ul>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SqlInjectionMatchTuples</i> Specifies the parts of web requests that you want to inspect for snippets of malicious SQL code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name, if any, of the <code>SqlInjectionMatchSet</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SqlInjectionMatchSetId</i> A unique identifier for a <code>SqlInjectionMatchSet</code>. You use <code>SqlInjectionMatchSetId</code> to get information about a <code>SqlInjectionMatchSet</code> (see <a>GetSqlInjectionMatchSet</a>), update a <code>SqlInjectionMatchSet</code> (see <a>UpdateSqlInjectionMatchSet</a>), insert a <code>SqlInjectionMatchSet</code> into a <code>Rule</code> or delete one from a <code>Rule</code> (see <a>UpdateRule</a>), and delete a <code>SqlInjectionMatchSet</code> from AWS WAF (see <a>DeleteSqlInjectionMatchSet</a>).  <code>SqlInjectionMatchSetId</code> is returned by <a>CreateSqlInjectionMatchSet</a> and by <a>ListSqlInjectionMatchSets</a>.<br/>        """
        pass

    def get_web_acl(self, WebACLId: str) -> GetWebACLResponse:
        """Returns the <a>WebACL</a> that is specified by <code>WebACLId</code>.

        :param self:
        :param WebACLId:  The <code>WebACLId</code> of the <a>WebACL</a> that you want to get. <code>WebACLId</code> is returned by <a>CreateWebACL</a> and by <a>ListWebACLs</a>.<br/>
        :return: WebACL:  Information about the <a>WebACL</a> that you specified in the <code>GetWebACL</code> request. For more information, see the following topics: <ul> <li>  <a>WebACL</a>: Contains <code>DefaultAction</code>, <code>MetricName</code>, <code>Name</code>, an array of <code>Rule</code> objects, and <code>WebACLId</code>  </li> <li>  <code>DefaultAction</code> (Data type is <a>WafAction</a>): Contains <code>Type</code>  </li> <li>  <code>Rules</code>: Contains an array of <code>ActivatedRule</code> objects, which contain <code>Action</code>, <code>Priority</code>, and <code>RuleId</code>  </li> <li>  <code>Action</code>: Contains <code>Type</code>  </li> </ul>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WebACLArn</i> Tha Amazon Resource Name (ARN) of the web ACL.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Rules</i> An array that contains the action for each <code>Rule</code> in a <code>WebACL</code>, the priority of the <code>Rule</code>, and the ID of the <code>Rule</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultAction</i> The action to perform if none of the <code>Rules</code> contained in the <code>WebACL</code> match. The action is specified by the <a>WafAction</a> object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DefaultAction</b>: The action to perform if none of the <code>Rules</code> contained in the <code>WebACL</code> match. The action is specified by the <a>WafAction</a> object.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricName</i> A friendly name or description for the metrics for this <code>WebACL</code>. The name can contain only alphanumeric characters (A-Z, a-z, 0-9), with maximum length 128 and minimum length one. It can't contain whitespace or metric names reserved for AWS WAF, including "All" and "Default_Action." You can't change <code>MetricName</code> after you create the <code>WebACL</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A friendly name or description of the <code>WebACL</code>. You can't change the name of a <code>WebACL</code> after you create it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WebACLId</i> A unique identifier for a <code>WebACL</code>. You use <code>WebACLId</code> to get information about a <code>WebACL</code> (see <a>GetWebACL</a>), update a <code>WebACL</code> (see <a>UpdateWebACL</a>), and delete a <code>WebACL</code> from AWS WAF (see <a>DeleteWebACL</a>).  <code>WebACLId</code> is returned by <a>CreateWebACL</a> and by <a>ListWebACLs</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> Specifies how you want AWS WAF to respond to requests that match the settings in a <code>Rule</code>. Valid settings include the following: <ul> <li>  <code>ALLOW</code>: AWS WAF allows requests </li> <li>  <code>BLOCK</code>: AWS WAF blocks requests </li> <li>  <code>COUNT</code>: AWS WAF increments a counter of the requests that match all of the conditions in the rule. AWS WAF then continues to inspect the web request based on the remaining rules in the web ACL. You can't specify <code>COUNT</code> for the default action for a <code>WebACL</code>. </li> </ul><br/>        """
        pass

    def get_xss_match_set(self, XssMatchSetId: str) -> GetXssMatchSetResponse:
        """Returns the <a>XssMatchSet</a> that is specified by <code>XssMatchSetId</code>.

        :param self:
        :param XssMatchSetId:  The <code>XssMatchSetId</code> of the <a>XssMatchSet</a> that you want to get. <code>XssMatchSetId</code> is returned by <a>CreateXssMatchSet</a> and by <a>ListXssMatchSets</a>.<br/>
        :return: XssMatchSet:  Information about the <a>XssMatchSet</a> that you specified in the <code>GetXssMatchSet</code> request. For more information, see the following topics: <ul> <li>  <a>XssMatchSet</a>: Contains <code>Name</code>, <code>XssMatchSetId</code>, and an array of <code>XssMatchTuple</code> objects </li> <li>  <a>XssMatchTuple</a>: Each <code>XssMatchTuple</code> object contains <code>FieldToMatch</code> and <code>TextTransformation</code>  </li> <li>  <a>FieldToMatch</a>: Contains <code>Data</code> and <code>Type</code>  </li> </ul>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>XssMatchTuples</i> Specifies the parts of web requests that you want to inspect for cross-site scripting attacks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name, if any, of the <code>XssMatchSet</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>XssMatchSetId</i> A unique identifier for an <code>XssMatchSet</code>. You use <code>XssMatchSetId</code> to get information about an <code>XssMatchSet</code> (see <a>GetXssMatchSet</a>), update an <code>XssMatchSet</code> (see <a>UpdateXssMatchSet</a>), insert an <code>XssMatchSet</code> into a <code>Rule</code> or delete one from a <code>Rule</code> (see <a>UpdateRule</a>), and delete an <code>XssMatchSet</code> from AWS WAF (see <a>DeleteXssMatchSet</a>).  <code>XssMatchSetId</code> is returned by <a>CreateXssMatchSet</a> and by <a>ListXssMatchSets</a>.<br/>        """
        pass

    def list_activated_rules_in_rule_group(self, RuleGroupId: str=None, NextMarker: str=None, Limit: int=None) -> ListActivatedRulesInRuleGroupResponse:
        """Returns an array of <a>ActivatedRule</a> objects.

        :param self:
        :param Limit:  Specifies the number of <code>ActivatedRules</code> that you want AWS WAF to return for this request. If you have more <code>ActivatedRules</code> than the number that you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of <code>ActivatedRules</code>.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <code>ActivatedRules</code> than the value of <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of <code>ActivatedRules</code>. For the second and subsequent <code>ListActivatedRulesInRuleGroup</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of <code>ActivatedRules</code>.<br/>RuleGroupId:  The <code>RuleGroupId</code> of the <a>RuleGroup</a> for which you want to get a list of <a>ActivatedRule</a> objects.<br/>
        :return: ActivatedRules:  An array of <code>ActivatedRules</code> objects.<br/>NextMarker:  If you have more <code>ActivatedRules</code> than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more <code>ActivatedRules</code>, submit another <code>ListActivatedRulesInRuleGroup</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>        """
        pass

    def list_byte_match_sets(self, NextMarker: str=None, Limit: int=None) -> ListByteMatchSetsResponse:
        """Returns an array of <a>ByteMatchSetSummary</a> objects.

        :param self:
        :param Limit:  Specifies the number of <code>ByteMatchSet</code> objects that you want AWS WAF to return for this request. If you have more <code>ByteMatchSets</code> objects than the number you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of <code>ByteMatchSet</code> objects.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <code>ByteMatchSets</code> than the value of <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of <code>ByteMatchSets</code>. For the second and subsequent <code>ListByteMatchSets</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of <code>ByteMatchSets</code>.<br/>
        :return: ByteMatchSets:  An array of <a>ByteMatchSetSummary</a> objects.<br/>NextMarker:  If you have more <code>ByteMatchSet</code> objects than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more <code>ByteMatchSet</code> objects, submit another <code>ListByteMatchSets</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>        """
        pass

    def list_geo_match_sets(self, NextMarker: str=None, Limit: int=None) -> ListGeoMatchSetsResponse:
        """Returns an array of <a>GeoMatchSetSummary</a> objects in the response.

        :param self:
        :param Limit:  Specifies the number of <code>GeoMatchSet</code> objects that you want AWS WAF to return for this request. If you have more <code>GeoMatchSet</code> objects than the number you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of <code>GeoMatchSet</code> objects.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <code>GeoMatchSet</code>s than the value of <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of <code>GeoMatchSet</code> objects. For the second and subsequent <code>ListGeoMatchSets</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of <code>GeoMatchSet</code> objects.<br/>
        :return: GeoMatchSets:  An array of <a>GeoMatchSetSummary</a> objects.<br/>NextMarker:  If you have more <code>GeoMatchSet</code> objects than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more <code>GeoMatchSet</code> objects, submit another <code>ListGeoMatchSets</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>        """
        pass

    def list_ip_sets(self, NextMarker: str=None, Limit: int=None) -> ListIPSetsResponse:
        """Returns an array of <a>IPSetSummary</a> objects in the response.

        :param self:
        :param Limit:  Specifies the number of <code>IPSet</code> objects that you want AWS WAF to return for this request. If you have more <code>IPSet</code> objects than the number you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of <code>IPSet</code> objects.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <code>IPSets</code> than the value of <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of <code>IPSets</code>. For the second and subsequent <code>ListIPSets</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of <code>IPSets</code>.<br/>
        :return: IPSets:  An array of <a>IPSetSummary</a> objects.<br/>NextMarker:  If you have more <code>IPSet</code> objects than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more <code>IPSet</code> objects, submit another <code>ListIPSets</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>        """
        pass

    def list_logging_configurations(self, NextMarker: str=None, Limit: int=None) -> ListLoggingConfigurationsResponse:
        """Returns an array of <a>LoggingConfiguration</a> objects.

        :param self:
        :param Limit:  Specifies the number of <code>LoggingConfigurations</code> that you want AWS WAF to return for this request. If you have more <code>LoggingConfigurations</code> than the number that you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of <code>LoggingConfigurations</code>.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <code>LoggingConfigurations</code> than the value of <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of <code>LoggingConfigurations</code>. For the second and subsequent <code>ListLoggingConfigurations</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of <code>ListLoggingConfigurations</code>.<br/>
        :return: NextMarker:  If you have more <code>LoggingConfigurations</code> than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more <code>LoggingConfigurations</code>, submit another <code>ListLoggingConfigurations</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>LoggingConfigurations:  An array of <a>LoggingConfiguration</a> objects.<br/>        """
        pass

    def list_rate_based_rules(self, NextMarker: str=None, Limit: int=None) -> ListRateBasedRulesResponse:
        """Returns an array of <a>RuleSummary</a> objects.

        :param self:
        :param Limit:  Specifies the number of <code>Rules</code> that you want AWS WAF to return for this request. If you have more <code>Rules</code> than the number that you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of <code>Rules</code>.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <code>Rules</code> than the value of <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of <code>Rules</code>. For the second and subsequent <code>ListRateBasedRules</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of <code>Rules</code>.<br/>
        :return: Rules:  An array of <a>RuleSummary</a> objects.<br/>NextMarker:  If you have more <code>Rules</code> than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more <code>Rules</code>, submit another <code>ListRateBasedRules</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>        """
        pass

    def list_regex_match_sets(self, NextMarker: str=None, Limit: int=None) -> ListRegexMatchSetsResponse:
        """Returns an array of <a>RegexMatchSetSummary</a> objects.

        :param self:
        :param Limit:  Specifies the number of <code>RegexMatchSet</code> objects that you want AWS WAF to return for this request. If you have more <code>RegexMatchSet</code> objects than the number you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of <code>RegexMatchSet</code> objects.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <code>RegexMatchSet</code> objects than the value of <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of <code>ByteMatchSets</code>. For the second and subsequent <code>ListRegexMatchSets</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of <code>RegexMatchSet</code> objects.<br/>
        :return: RegexMatchSets:  An array of <a>RegexMatchSetSummary</a> objects.<br/>NextMarker:  If you have more <code>RegexMatchSet</code> objects than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more <code>RegexMatchSet</code> objects, submit another <code>ListRegexMatchSets</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>        """
        pass

    def list_regex_pattern_sets(self, NextMarker: str=None, Limit: int=None) -> ListRegexPatternSetsResponse:
        """Returns an array of <a>RegexPatternSetSummary</a> objects.

        :param self:
        :param Limit:  Specifies the number of <code>RegexPatternSet</code> objects that you want AWS WAF to return for this request. If you have more <code>RegexPatternSet</code> objects than the number you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of <code>RegexPatternSet</code> objects.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <code>RegexPatternSet</code> objects than the value of <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of <code>RegexPatternSet</code> objects. For the second and subsequent <code>ListRegexPatternSets</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of <code>RegexPatternSet</code> objects.<br/>
        :return: RegexPatternSets:  An array of <a>RegexPatternSetSummary</a> objects.<br/>NextMarker:  If you have more <code>RegexPatternSet</code> objects than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more <code>RegexPatternSet</code> objects, submit another <code>ListRegexPatternSets</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>        """
        pass

    def list_rule_groups(self, NextMarker: str=None, Limit: int=None) -> ListRuleGroupsResponse:
        """Returns an array of <a>RuleGroup</a> objects.

        :param self:
        :param Limit:  Specifies the number of <code>RuleGroups</code> that you want AWS WAF to return for this request. If you have more <code>RuleGroups</code> than the number that you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of <code>RuleGroups</code>.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <code>RuleGroups</code> than the value of <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of <code>RuleGroups</code>. For the second and subsequent <code>ListRuleGroups</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of <code>RuleGroups</code>.<br/>
        :return: RuleGroups:  An array of <a>RuleGroup</a> objects.<br/>NextMarker:  If you have more <code>RuleGroups</code> than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more <code>RuleGroups</code>, submit another <code>ListRuleGroups</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>        """
        pass

    def list_rules(self, NextMarker: str=None, Limit: int=None) -> ListRulesResponse:
        """Returns an array of <a>RuleSummary</a> objects.

        :param self:
        :param Limit:  Specifies the number of <code>Rules</code> that you want AWS WAF to return for this request. If you have more <code>Rules</code> than the number that you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of <code>Rules</code>.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <code>Rules</code> than the value of <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of <code>Rules</code>. For the second and subsequent <code>ListRules</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of <code>Rules</code>.<br/>
        :return: Rules:  An array of <a>RuleSummary</a> objects.<br/>NextMarker:  If you have more <code>Rules</code> than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more <code>Rules</code>, submit another <code>ListRules</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>        """
        pass

    def list_size_constraint_sets(self, NextMarker: str=None, Limit: int=None) -> ListSizeConstraintSetsResponse:
        """Returns an array of <a>SizeConstraintSetSummary</a> objects.

        :param self:
        :param Limit:  Specifies the number of <code>SizeConstraintSet</code> objects that you want AWS WAF to return for this request. If you have more <code>SizeConstraintSets</code> objects than the number you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of <code>SizeConstraintSet</code> objects.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <code>SizeConstraintSets</code> than the value of <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of <code>SizeConstraintSets</code>. For the second and subsequent <code>ListSizeConstraintSets</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of <code>SizeConstraintSets</code>.<br/>
        :return: SizeConstraintSets:  An array of <a>SizeConstraintSetSummary</a> objects.<br/>NextMarker:  If you have more <code>SizeConstraintSet</code> objects than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more <code>SizeConstraintSet</code> objects, submit another <code>ListSizeConstraintSets</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>        """
        pass

    def list_sql_injection_match_sets(self, NextMarker: str=None, Limit: int=None) -> ListSqlInjectionMatchSetsResponse:
        """Returns an array of <a>SqlInjectionMatchSet</a> objects.

        :param self:
        :param Limit:  Specifies the number of <a>SqlInjectionMatchSet</a> objects that you want AWS WAF to return for this request. If you have more <code>SqlInjectionMatchSet</code> objects than the number you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of <code>Rules</code>.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <a>SqlInjectionMatchSet</a> objects than the value of <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of <code>SqlInjectionMatchSets</code>. For the second and subsequent <code>ListSqlInjectionMatchSets</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of <code>SqlInjectionMatchSets</code>.<br/>
        :return: SqlInjectionMatchSets:  An array of <a>SqlInjectionMatchSetSummary</a> objects.<br/>NextMarker:  If you have more <a>SqlInjectionMatchSet</a> objects than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more <code>SqlInjectionMatchSet</code> objects, submit another <code>ListSqlInjectionMatchSets</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>        """
        pass

    def list_subscribed_rule_groups(self, NextMarker: str=None, Limit: int=None) -> ListSubscribedRuleGroupsResponse:
        """Returns an array of <a>RuleGroup</a> objects that you are subscribed to.

        :param self:
        :param Limit:  Specifies the number of subscribed rule groups that you want AWS WAF to return for this request. If you have more objects than the number you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of objects.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <code>ByteMatchSets</code>subscribed rule groups than the value of <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of subscribed rule groups. For the second and subsequent <code>ListSubscribedRuleGroupsRequest</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of subscribed rule groups.<br/>
        :return: RuleGroups:  An array of <a>RuleGroup</a> objects.<br/>NextMarker:  If you have more objects than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more objects, submit another <code>ListSubscribedRuleGroups</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceARN: str, NextMarker: str=None, Limit: int=None) -> ListTagsForResourceResponse:
        """

        :param self:
        :param ResourceARN:  <br/>Limit:  <br/>NextMarker:  <br/>
        :return: TagInfoForResource:  :<br/>NextMarker:  <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TagList</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceARN</i> <br/>        """
        pass

    def list_web_acls(self, NextMarker: str=None, Limit: int=None) -> ListWebACLsResponse:
        """Returns an array of <a>WebACLSummary</a> objects in the response.

        :param self:
        :param Limit:  Specifies the number of <code>WebACL</code> objects that you want AWS WAF to return for this request. If you have more <code>WebACL</code> objects than the number that you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of <code>WebACL</code> objects.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <code>WebACL</code> objects than the number that you specify for <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of <code>WebACL</code> objects. For the second and subsequent <code>ListWebACLs</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of <code>WebACL</code> objects.<br/>
        :return: WebACLs:  An array of <a>WebACLSummary</a> objects.<br/>NextMarker:  If you have more <code>WebACL</code> objects than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more <code>WebACL</code> objects, submit another <code>ListWebACLs</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>        """
        pass

    def list_xss_match_sets(self, NextMarker: str=None, Limit: int=None) -> ListXssMatchSetsResponse:
        """Returns an array of <a>XssMatchSet</a> objects.

        :param self:
        :param Limit:  Specifies the number of <a>XssMatchSet</a> objects that you want AWS WAF to return for this request. If you have more <code>XssMatchSet</code> objects than the number you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of <code>Rules</code>.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <a>XssMatchSet</a> objects than the value of <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of <code>XssMatchSets</code>. For the second and subsequent <code>ListXssMatchSets</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of <code>XssMatchSets</code>.<br/>
        :return: XssMatchSets:  An array of <a>XssMatchSetSummary</a> objects.<br/>NextMarker:  If you have more <a>XssMatchSet</a> objects than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more <code>XssMatchSet</code> objects, submit another <code>ListXssMatchSets</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>        """
        pass

    def put_logging_configuration(self, LoggingConfiguration: LoggingConfiguration) -> PutLoggingConfigurationResponse:
        """Associates a <a>LoggingConfiguration</a> with a specified web ACL. You can access information about all traffic that AWS WAF inspects using the following steps: <ol> <li> Create an Amazon Kinesis Data Firehose .  Create the data firehose with a PUT source and in the region that you are operating. However, if you are capturing logs for Amazon CloudFront, always create the firehose in US East (N. Virginia).  <note> Do not create the data firehose using a <code>Kinesis stream</code> as your source. </note> </li> <li> Associate that firehose to your web ACL using a <code>PutLoggingConfiguration</code> request. </li> </ol> When you successfully enable logging using a <code>PutLoggingConfiguration</code> request, AWS WAF will create a service linked role with the necessary permissions to write logs to the Amazon Kinesis Data Firehose. For more information, see <a href="https://docs.aws.amazon.com/waf/latest/developerguide/logging.html">Logging Web ACL Traffic Information</a> in the <i>AWS WAF Developer Guide</i>.

        :param self:
        :param LoggingConfiguration:  The Amazon Kinesis Data Firehose that contains the inspected traffic information, the redacted fields details, and the Amazon Resource Name (ARN) of the web ACL to monitor.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RedactedFields</i> The parts of the request that you want redacted from the logs. For example, if you redact the cookie field, the cookie field in the firehose will be <code>xxx</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogDestinationConfigs</i> An array of Amazon Kinesis Data Firehose ARNs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceArn</i> The Amazon Resource Name (ARN) of the web ACL that you want to associate with <code>LogDestinationConfigs</code>.<br/>
        :return: LoggingConfiguration:  The <a>LoggingConfiguration</a> that you submitted in the request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RedactedFields</i> The parts of the request that you want redacted from the logs. For example, if you redact the cookie field, the cookie field in the firehose will be <code>xxx</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogDestinationConfigs</i> An array of Amazon Kinesis Data Firehose ARNs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceArn</i> The Amazon Resource Name (ARN) of the web ACL that you want to associate with <code>LogDestinationConfigs</code>.<br/>        """
        pass

    def put_permission_policy(self, Policy: str, ResourceArn: str) -> PutPermissionPolicyResponse:
        """Attaches a IAM policy to the specified resource. The only supported use for this action is to share a RuleGroup across accounts. The <code>PutPermissionPolicy</code> is subject to the following restrictions: <ul> <li> You can attach only one policy with each <code>PutPermissionPolicy</code> request. </li> <li> The policy must include an <code>Effect</code>, <code>Action</code> and <code>Principal</code>.  </li> <li>  <code>Effect</code> must specify <code>Allow</code>. </li> <li> The <code>Action</code> in the policy must be <code>waf:UpdateWebACL</code>, <code>waf-regional:UpdateWebACL</code>, <code>waf:GetRuleGroup</code> and <code>waf-regional:GetRuleGroup</code> . Any extra or wildcard actions in the policy will be rejected. </li> <li> The policy cannot include a <code>Resource</code> parameter. </li> <li> The ARN in the request must be a valid WAF RuleGroup ARN and the RuleGroup must exist in the same region. </li> <li> The user making the request must be the owner of the RuleGroup. </li> <li> Your policy must be composed using IAM Policy version 2012-10-17. </li> </ul> For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies.html">IAM Policies</a>.  An example of a valid policy parameter is shown in the Examples section below.

        :param self:
        :param Policy:  The policy to attach to the specified RuleGroup.<br/>ResourceArn:  The Amazon Resource Name (ARN) of the RuleGroup to which you want to attach the policy.<br/>
        :return:         """
        pass

    def tag_resource(self, Tags: List[Tag], ResourceARN: str) -> TagResourceResponse:
        """

        :param self:
        :param Tags:  <br/>ResourceARN:  <br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[TagKey], ResourceARN: str) -> UntagResourceResponse:
        """

        :param self:
        :param TagKeys:  <br/>ResourceARN:  <br/>
        :return:         """
        pass

    def update_byte_match_set(self, Updates: List[ByteMatchSetUpdate], ChangeToken: str, ByteMatchSetId: str) -> UpdateByteMatchSetResponse:
        """Inserts or deletes <a>ByteMatchTuple</a> objects (filters) in a <a>ByteMatchSet</a>. For each <code>ByteMatchTuple</code> object, you specify the following values:  <ul> <li> Whether to insert or delete the object from the array. If you want to change a <code>ByteMatchSetUpdate</code> object, you delete the existing object and add a new one. </li> <li> The part of a web request that you want AWS WAF to inspect, such as a query string or the value of the <code>User-Agent</code> header.  </li> <li> The bytes (typically a string that corresponds with ASCII characters) that you want AWS WAF to look for. For more information, including how you specify the values for the AWS WAF API and the AWS CLI or SDKs, see <code>TargetString</code> in the <a>ByteMatchTuple</a> data type.  </li> <li> Where to look, such as at the beginning or the end of a query string. </li> <li> Whether to perform any conversions on the request, such as converting it to lowercase, before inspecting it for the specified string. </li> </ul> For example, you can add a <code>ByteMatchSetUpdate</code> object that matches web requests in which <code>User-Agent</code> headers contain the string <code>BadBot</code>. You can then configure AWS WAF to block those requests. To create and configure a <code>ByteMatchSet</code>, perform the following steps: <ol> <li> Create a <code>ByteMatchSet.</code> For more information, see <a>CreateByteMatchSet</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <code>UpdateByteMatchSet</code> request. </li> <li> Submit an <code>UpdateByteMatchSet</code> request to specify the part of the request that you want AWS WAF to inspect (for example, the header or the URI) and the value that you want AWS WAF to watch for. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param Updates:  An array of <code>ByteMatchSetUpdate</code> objects that you want to insert into or delete from a <a>ByteMatchSet</a>. For more information, see the applicable data types: <ul> <li>  <a>ByteMatchSetUpdate</a>: Contains <code>Action</code> and <code>ByteMatchTuple</code>  </li> <li>  <a>ByteMatchTuple</a>: Contains <code>FieldToMatch</code>, <code>PositionalConstraint</code>, <code>TargetString</code>, and <code>TextTransformation</code>  </li> <li>  <a>FieldToMatch</a>: Contains <code>Data</code> and <code>Type</code>  </li> </ul><br/>ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>ByteMatchSetId:  The <code>ByteMatchSetId</code> of the <a>ByteMatchSet</a> that you want to update. <code>ByteMatchSetId</code> is returned by <a>CreateByteMatchSet</a> and by <a>ListByteMatchSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>UpdateByteMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def update_geo_match_set(self, Updates: List[GeoMatchSetUpdate], ChangeToken: str, GeoMatchSetId: str) -> UpdateGeoMatchSetResponse:
        """Inserts or deletes <a>GeoMatchConstraint</a> objects in an <code>GeoMatchSet</code>. For each <code>GeoMatchConstraint</code> object, you specify the following values:  <ul> <li> Whether to insert or delete the object from the array. If you want to change an <code>GeoMatchConstraint</code> object, you delete the existing object and add a new one. </li> <li> The <code>Type</code>. The only valid value for <code>Type</code> is <code>Country</code>. </li> <li> The <code>Value</code>, which is a two character code for the country to add to the <code>GeoMatchConstraint</code> object. Valid codes are listed in <a>GeoMatchConstraint$Value</a>. </li> </ul> To create and configure an <code>GeoMatchSet</code>, perform the following steps: <ol> <li> Submit a <a>CreateGeoMatchSet</a> request. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <a>UpdateGeoMatchSet</a> request. </li> <li> Submit an <code>UpdateGeoMatchSet</code> request to specify the country that you want AWS WAF to watch for. </li> </ol> When you update an <code>GeoMatchSet</code>, you specify the country that you want to add and/or the country that you want to delete. If you want to change a country, you delete the existing country and add the new one. For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param Updates:  An array of <code>GeoMatchSetUpdate</code> objects that you want to insert into or delete from an <a>GeoMatchSet</a>. For more information, see the applicable data types: <ul> <li>  <a>GeoMatchSetUpdate</a>: Contains <code>Action</code> and <code>GeoMatchConstraint</code>  </li> <li>  <a>GeoMatchConstraint</a>: Contains <code>Type</code> and <code>Value</code>  You can have only one <code>Type</code> and <code>Value</code> per <code>GeoMatchConstraint</code>. To add multiple countries, include multiple <code>GeoMatchSetUpdate</code> objects in your request. </li> </ul><br/>ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>GeoMatchSetId:  The <code>GeoMatchSetId</code> of the <a>GeoMatchSet</a> that you want to update. <code>GeoMatchSetId</code> is returned by <a>CreateGeoMatchSet</a> and by <a>ListGeoMatchSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>UpdateGeoMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def update_ip_set(self, Updates: List[IPSetUpdate], ChangeToken: str, IPSetId: str) -> UpdateIPSetResponse:
        """Inserts or deletes <a>IPSetDescriptor</a> objects in an <code>IPSet</code>. For each <code>IPSetDescriptor</code> object, you specify the following values:  <ul> <li> Whether to insert or delete the object from the array. If you want to change an <code>IPSetDescriptor</code> object, you delete the existing object and add a new one. </li> <li> The IP address version, <code>IPv4</code> or <code>IPv6</code>.  </li> <li> The IP address in CIDR notation, for example, <code>192.0.2.0/24</code> (for the range of IP addresses from <code>192.0.2.0</code> to <code>192.0.2.255</code>) or <code>192.0.2.44/32</code> (for the individual IP address <code>192.0.2.44</code>).  </li> </ul> AWS WAF supports IPv4 address ranges: /8 and any range between /16 through /32. AWS WAF supports IPv6 address ranges: /24, /32, /48, /56, /64, and /128. For more information about CIDR notation, see the Wikipedia entry <a href="https://en.wikipedia.org/wiki/Classless_Inter-Domain_Routing">Classless Inter-Domain Routing</a>. IPv6 addresses can be represented using any of the following formats: <ul> <li> 1111:0000:0000:0000:0000:0000:0000:0111/128 </li> <li> 1111:0:0:0:0:0:0:0111/128 </li> <li> 1111::0111/128 </li> <li> 1111::111/128 </li> </ul> You use an <code>IPSet</code> to specify which web requests you want to allow or block based on the IP addresses that the requests originated from. For example, if you're receiving a lot of requests from one or a small number of IP addresses and you want to block the requests, you can create an <code>IPSet</code> that specifies those IP addresses, and then configure AWS WAF to block the requests.  To create and configure an <code>IPSet</code>, perform the following steps: <ol> <li> Submit a <a>CreateIPSet</a> request. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <a>UpdateIPSet</a> request. </li> <li> Submit an <code>UpdateIPSet</code> request to specify the IP addresses that you want AWS WAF to watch for. </li> </ol> When you update an <code>IPSet</code>, you specify the IP addresses that you want to add and/or the IP addresses that you want to delete. If you want to change an IP address, you delete the existing IP address and add the new one. You can insert a maximum of 1000 addresses in a single request. For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param Updates:  An array of <code>IPSetUpdate</code> objects that you want to insert into or delete from an <a>IPSet</a>. For more information, see the applicable data types: <ul> <li>  <a>IPSetUpdate</a>: Contains <code>Action</code> and <code>IPSetDescriptor</code>  </li> <li>  <a>IPSetDescriptor</a>: Contains <code>Type</code> and <code>Value</code>  </li> </ul> You can insert a maximum of 1000 addresses in a single request.<br/>ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>IPSetId:  The <code>IPSetId</code> of the <a>IPSet</a> that you want to update. <code>IPSetId</code> is returned by <a>CreateIPSet</a> and by <a>ListIPSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>UpdateIPSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def update_rate_based_rule(self, RateLimit: RateLimit, Updates: List[RuleUpdate], ChangeToken: str, RuleId: str) -> UpdateRateBasedRuleResponse:
        """Inserts or deletes <a>Predicate</a> objects in a rule and updates the <code>RateLimit</code> in the rule.  Each <code>Predicate</code> object identifies a predicate, such as a <a>ByteMatchSet</a> or an <a>IPSet</a>, that specifies the web requests that you want to block or count. The <code>RateLimit</code> specifies the number of requests every five minutes that triggers the rule. If you add more than one predicate to a <code>RateBasedRule</code>, a request must match all the predicates and exceed the <code>RateLimit</code> to be counted or blocked. For example, suppose you add the following to a <code>RateBasedRule</code>: <ul> <li> An <code>IPSet</code> that matches the IP address <code>192.0.2.44/32</code>  </li> <li> A <code>ByteMatchSet</code> that matches <code>BadBot</code> in the <code>User-Agent</code> header </li> </ul> Further, you specify a <code>RateLimit</code> of 15,000. You then add the <code>RateBasedRule</code> to a <code>WebACL</code> and specify that you want to block requests that satisfy the rule. For a request to be blocked, it must come from the IP address 192.0.2.44 <i>and</i> the <code>User-Agent</code> header in the request must contain the value <code>BadBot</code>. Further, requests that match these two conditions much be received at a rate of more than 15,000 every five minutes. If the rate drops below this limit, AWS WAF no longer blocks the requests. As a second example, suppose you want to limit requests to a particular page on your site. To do this, you could add the following to a <code>RateBasedRule</code>: <ul> <li> A <code>ByteMatchSet</code> with <code>FieldToMatch</code> of <code>URI</code>  </li> <li> A <code>PositionalConstraint</code> of <code>STARTS_WITH</code>  </li> <li> A <code>TargetString</code> of <code>login</code>  </li> </ul> Further, you specify a <code>RateLimit</code> of 15,000. By adding this <code>RateBasedRule</code> to a <code>WebACL</code>, you could limit requests to your login page without affecting the rest of your site.

        :param self:
        :param RateLimit:  The maximum number of requests, which have an identical value in the field specified by the <code>RateKey</code>, allowed in a five-minute period. If the number of requests exceeds the <code>RateLimit</code> and the other predicates specified in the rule are also met, AWS WAF triggers the action that is specified for this rule.<br/>Updates:  An array of <code>RuleUpdate</code> objects that you want to insert into or delete from a <a>RateBasedRule</a>. <br/>ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>RuleId:  The <code>RuleId</code> of the <code>RateBasedRule</code> that you want to update. <code>RuleId</code> is returned by <code>CreateRateBasedRule</code> and by <a>ListRateBasedRules</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>UpdateRateBasedRule</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def update_regex_match_set(self, ChangeToken: str, Updates: List[RegexMatchSetUpdate], RegexMatchSetId: str) -> UpdateRegexMatchSetResponse:
        """Inserts or deletes <a>RegexMatchTuple</a> objects (filters) in a <a>RegexMatchSet</a>. For each <code>RegexMatchSetUpdate</code> object, you specify the following values:  <ul> <li> Whether to insert or delete the object from the array. If you want to change a <code>RegexMatchSetUpdate</code> object, you delete the existing object and add a new one. </li> <li> The part of a web request that you want AWS WAF to inspectupdate, such as a query string or the value of the <code>User-Agent</code> header.  </li> <li> The identifier of the pattern (a regular expression) that you want AWS WAF to look for. For more information, see <a>RegexPatternSet</a>.  </li> <li> Whether to perform any conversions on the request, such as converting it to lowercase, before inspecting it for the specified string. </li> </ul>  For example, you can create a <code>RegexPatternSet</code> that matches any requests with <code>User-Agent</code> headers that contain the string <code>B[a@]dB[o0]t</code>. You can then configure AWS WAF to reject those requests. To create and configure a <code>RegexMatchSet</code>, perform the following steps: <ol> <li> Create a <code>RegexMatchSet.</code> For more information, see <a>CreateRegexMatchSet</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <code>UpdateRegexMatchSet</code> request. </li> <li> Submit an <code>UpdateRegexMatchSet</code> request to specify the part of the request that you want AWS WAF to inspect (for example, the header or the URI) and the identifier of the <code>RegexPatternSet</code> that contain the regular expression patters you want AWS WAF to watch for. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>Updates:  An array of <code>RegexMatchSetUpdate</code> objects that you want to insert into or delete from a <a>RegexMatchSet</a>. For more information, see <a>RegexMatchTuple</a>.<br/>RegexMatchSetId:  The <code>RegexMatchSetId</code> of the <a>RegexMatchSet</a> that you want to update. <code>RegexMatchSetId</code> is returned by <a>CreateRegexMatchSet</a> and by <a>ListRegexMatchSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>UpdateRegexMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def update_regex_pattern_set(self, ChangeToken: str, Updates: List[RegexPatternSetUpdate], RegexPatternSetId: str) -> UpdateRegexPatternSetResponse:
        """Inserts or deletes <code>RegexPatternString</code> objects in a <a>RegexPatternSet</a>. For each <code>RegexPatternString</code> object, you specify the following values:  <ul> <li> Whether to insert or delete the <code>RegexPatternString</code>. </li> <li> The regular expression pattern that you want to insert or delete. For more information, see <a>RegexPatternSet</a>.  </li> </ul>  For example, you can create a <code>RegexPatternString</code> such as <code>B[a@]dB[o0]t</code>. AWS WAF will match this <code>RegexPatternString</code> to: <ul> <li> BadBot </li> <li> BadB0t </li> <li> B@dBot </li> <li> B@dB0t </li> </ul> To create and configure a <code>RegexPatternSet</code>, perform the following steps: <ol> <li> Create a <code>RegexPatternSet.</code> For more information, see <a>CreateRegexPatternSet</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <code>UpdateRegexPatternSet</code> request. </li> <li> Submit an <code>UpdateRegexPatternSet</code> request to specify the regular expression pattern that you want AWS WAF to watch for. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>Updates:  An array of <code>RegexPatternSetUpdate</code> objects that you want to insert into or delete from a <a>RegexPatternSet</a>.<br/>RegexPatternSetId:  The <code>RegexPatternSetId</code> of the <a>RegexPatternSet</a> that you want to update. <code>RegexPatternSetId</code> is returned by <a>CreateRegexPatternSet</a> and by <a>ListRegexPatternSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>UpdateRegexPatternSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def update_rule(self, Updates: List[RuleUpdate], ChangeToken: str, RuleId: str) -> UpdateRuleResponse:
        """Inserts or deletes <a>Predicate</a> objects in a <code>Rule</code>. Each <code>Predicate</code> object identifies a predicate, such as a <a>ByteMatchSet</a> or an <a>IPSet</a>, that specifies the web requests that you want to allow, block, or count. If you add more than one predicate to a <code>Rule</code>, a request must match all of the specifications to be allowed, blocked, or counted. For example, suppose that you add the following to a <code>Rule</code>:  <ul> <li> A <code>ByteMatchSet</code> that matches the value <code>BadBot</code> in the <code>User-Agent</code> header </li> <li> An <code>IPSet</code> that matches the IP address <code>192.0.2.44</code>  </li> </ul> You then add the <code>Rule</code> to a <code>WebACL</code> and specify that you want to block requests that satisfy the <code>Rule</code>. For a request to be blocked, the <code>User-Agent</code> header in the request must contain the value <code>BadBot</code> <i>and</i> the request must originate from the IP address 192.0.2.44. To create and configure a <code>Rule</code>, perform the following steps: <ol> <li> Create and update the predicates that you want to include in the <code>Rule</code>. </li> <li> Create the <code>Rule</code>. See <a>CreateRule</a>. </li> <li> Use <code>GetChangeToken</code> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <a>UpdateRule</a> request. </li> <li> Submit an <code>UpdateRule</code> request to add predicates to the <code>Rule</code>. </li> <li> Create and update a <code>WebACL</code> that contains the <code>Rule</code>. See <a>CreateWebACL</a>. </li> </ol> If you want to replace one <code>ByteMatchSet</code> or <code>IPSet</code> with another, you delete the existing one and add the new one. For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param Updates:  An array of <code>RuleUpdate</code> objects that you want to insert into or delete from a <a>Rule</a>. For more information, see the applicable data types: <ul> <li>  <a>RuleUpdate</a>: Contains <code>Action</code> and <code>Predicate</code>  </li> <li>  <a>Predicate</a>: Contains <code>DataId</code>, <code>Negated</code>, and <code>Type</code>  </li> <li>  <a>FieldToMatch</a>: Contains <code>Data</code> and <code>Type</code>  </li> </ul><br/>ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>RuleId:  The <code>RuleId</code> of the <code>Rule</code> that you want to update. <code>RuleId</code> is returned by <code>CreateRule</code> and by <a>ListRules</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>UpdateRule</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def update_rule_group(self, ChangeToken: str, Updates: List[RuleGroupUpdate], RuleGroupId: str) -> UpdateRuleGroupResponse:
        """Inserts or deletes <a>ActivatedRule</a> objects in a <code>RuleGroup</code>. You can only insert <code>REGULAR</code> rules into a rule group. You can have a maximum of ten rules per rule group. To create and configure a <code>RuleGroup</code>, perform the following steps: <ol> <li> Create and update the <code>Rules</code> that you want to include in the <code>RuleGroup</code>. See <a>CreateRule</a>. </li> <li> Use <code>GetChangeToken</code> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <a>UpdateRuleGroup</a> request. </li> <li> Submit an <code>UpdateRuleGroup</code> request to add <code>Rules</code> to the <code>RuleGroup</code>. </li> <li> Create and update a <code>WebACL</code> that contains the <code>RuleGroup</code>. See <a>CreateWebACL</a>. </li> </ol> If you want to replace one <code>Rule</code> with another, you delete the existing one and add the new one. For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>Updates:  An array of <code>RuleGroupUpdate</code> objects that you want to insert into or delete from a <a>RuleGroup</a>. You can only insert <code>REGULAR</code> rules into a rule group.  <code>ActivatedRule|OverrideAction</code> applies only when updating or adding a <code>RuleGroup</code> to a <code>WebACL</code>. In this case you do not use <code>ActivatedRule|Action</code>. For all other update requests, <code>ActivatedRule|Action</code> is used instead of <code>ActivatedRule|OverrideAction</code>.<br/>RuleGroupId:  The <code>RuleGroupId</code> of the <a>RuleGroup</a> that you want to update. <code>RuleGroupId</code> is returned by <a>CreateRuleGroup</a> and by <a>ListRuleGroups</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>UpdateRuleGroup</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def update_size_constraint_set(self, Updates: List[SizeConstraintSetUpdate], ChangeToken: str, SizeConstraintSetId: str) -> UpdateSizeConstraintSetResponse:
        """Inserts or deletes <a>SizeConstraint</a> objects (filters) in a <a>SizeConstraintSet</a>. For each <code>SizeConstraint</code> object, you specify the following values:  <ul> <li> Whether to insert or delete the object from the array. If you want to change a <code>SizeConstraintSetUpdate</code> object, you delete the existing object and add a new one. </li> <li> The part of a web request that you want AWS WAF to evaluate, such as the length of a query string or the length of the <code>User-Agent</code> header. </li> <li> Whether to perform any transformations on the request, such as converting it to lowercase, before checking its length. Note that transformations of the request body are not supported because the AWS resource forwards only the first <code>8192</code> bytes of your request to AWS WAF. You can only specify a single type of TextTransformation. </li> <li> A <code>ComparisonOperator</code> used for evaluating the selected part of the request against the specified <code>Size</code>, such as equals, greater than, less than, and so on. </li> <li> The length, in bytes, that you want AWS WAF to watch for in selected part of the request. The length is computed after applying the transformation. </li> </ul> For example, you can add a <code>SizeConstraintSetUpdate</code> object that matches web requests in which the length of the <code>User-Agent</code> header is greater than 100 bytes. You can then configure AWS WAF to block those requests. To create and configure a <code>SizeConstraintSet</code>, perform the following steps: <ol> <li> Create a <code>SizeConstraintSet.</code> For more information, see <a>CreateSizeConstraintSet</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <code>UpdateSizeConstraintSet</code> request. </li> <li> Submit an <code>UpdateSizeConstraintSet</code> request to specify the part of the request that you want AWS WAF to inspect (for example, the header or the URI) and the value that you want AWS WAF to watch for. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param Updates:  An array of <code>SizeConstraintSetUpdate</code> objects that you want to insert into or delete from a <a>SizeConstraintSet</a>. For more information, see the applicable data types: <ul> <li>  <a>SizeConstraintSetUpdate</a>: Contains <code>Action</code> and <code>SizeConstraint</code>  </li> <li>  <a>SizeConstraint</a>: Contains <code>FieldToMatch</code>, <code>TextTransformation</code>, <code>ComparisonOperator</code>, and <code>Size</code>  </li> <li>  <a>FieldToMatch</a>: Contains <code>Data</code> and <code>Type</code>  </li> </ul><br/>ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>SizeConstraintSetId:  The <code>SizeConstraintSetId</code> of the <a>SizeConstraintSet</a> that you want to update. <code>SizeConstraintSetId</code> is returned by <a>CreateSizeConstraintSet</a> and by <a>ListSizeConstraintSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>UpdateSizeConstraintSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def update_sql_injection_match_set(self, Updates: List[SqlInjectionMatchSetUpdate], ChangeToken: str, SqlInjectionMatchSetId: str) -> UpdateSqlInjectionMatchSetResponse:
        """Inserts or deletes <a>SqlInjectionMatchTuple</a> objects (filters) in a <a>SqlInjectionMatchSet</a>. For each <code>SqlInjectionMatchTuple</code> object, you specify the following values: <ul> <li>  <code>Action</code>: Whether to insert the object into or delete the object from the array. To change a <code>SqlInjectionMatchTuple</code>, you delete the existing object and add a new one. </li> <li>  <code>FieldToMatch</code>: The part of web requests that you want AWS WAF to inspect and, if you want AWS WAF to inspect a header or custom query parameter, the name of the header or parameter. </li> <li>  <code>TextTransformation</code>: Which text transformation, if any, to perform on the web request before inspecting the request for snippets of malicious SQL code. You can only specify a single type of TextTransformation. </li> </ul> You use <code>SqlInjectionMatchSet</code> objects to specify which CloudFront requests that you want to allow, block, or count. For example, if you're receiving requests that contain snippets of SQL code in the query string and you want to block the requests, you can create a <code>SqlInjectionMatchSet</code> with the applicable settings, and then configure AWS WAF to block the requests.  To create and configure a <code>SqlInjectionMatchSet</code>, perform the following steps: <ol> <li> Submit a <a>CreateSqlInjectionMatchSet</a> request. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <a>UpdateIPSet</a> request. </li> <li> Submit an <code>UpdateSqlInjectionMatchSet</code> request to specify the parts of web requests that you want AWS WAF to inspect for snippets of SQL code. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param Updates:  An array of <code>SqlInjectionMatchSetUpdate</code> objects that you want to insert into or delete from a <a>SqlInjectionMatchSet</a>. For more information, see the applicable data types: <ul> <li>  <a>SqlInjectionMatchSetUpdate</a>: Contains <code>Action</code> and <code>SqlInjectionMatchTuple</code>  </li> <li>  <a>SqlInjectionMatchTuple</a>: Contains <code>FieldToMatch</code> and <code>TextTransformation</code>  </li> <li>  <a>FieldToMatch</a>: Contains <code>Data</code> and <code>Type</code>  </li> </ul><br/>ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>SqlInjectionMatchSetId:  The <code>SqlInjectionMatchSetId</code> of the <code>SqlInjectionMatchSet</code> that you want to update. <code>SqlInjectionMatchSetId</code> is returned by <a>CreateSqlInjectionMatchSet</a> and by <a>ListSqlInjectionMatchSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>UpdateSqlInjectionMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def update_web_acl(self, ChangeToken: str, WebACLId: str, Updates: List[WebACLUpdate]=None, DefaultAction: WafAction=None) -> UpdateWebACLResponse:
        """Inserts or deletes <a>ActivatedRule</a> objects in a <code>WebACL</code>. Each <code>Rule</code> identifies web requests that you want to allow, block, or count. When you update a <code>WebACL</code>, you specify the following values: <ul> <li> A default action for the <code>WebACL</code>, either <code>ALLOW</code> or <code>BLOCK</code>. AWS WAF performs the default action if a request doesn't match the criteria in any of the <code>Rules</code> in a <code>WebACL</code>. </li> <li> The <code>Rules</code> that you want to add or delete. If you want to replace one <code>Rule</code> with another, you delete the existing <code>Rule</code> and add the new one. </li> <li> For each <code>Rule</code>, whether you want AWS WAF to allow requests, block requests, or count requests that match the conditions in the <code>Rule</code>. </li> <li> The order in which you want AWS WAF to evaluate the <code>Rules</code> in a <code>WebACL</code>. If you add more than one <code>Rule</code> to a <code>WebACL</code>, AWS WAF evaluates each request against the <code>Rules</code> in order based on the value of <code>Priority</code>. (The <code>Rule</code> that has the lowest value for <code>Priority</code> is evaluated first.) When a web request matches all the predicates (such as <code>ByteMatchSets</code> and <code>IPSets</code>) in a <code>Rule</code>, AWS WAF immediately takes the corresponding action, allow or block, and doesn't evaluate the request against the remaining <code>Rules</code> in the <code>WebACL</code>, if any.  </li> </ul> To create and configure a <code>WebACL</code>, perform the following steps: <ol> <li> Create and update the predicates that you want to include in <code>Rules</code>. For more information, see <a>CreateByteMatchSet</a>, <a>UpdateByteMatchSet</a>, <a>CreateIPSet</a>, <a>UpdateIPSet</a>, <a>CreateSqlInjectionMatchSet</a>, and <a>UpdateSqlInjectionMatchSet</a>. </li> <li> Create and update the <code>Rules</code> that you want to include in the <code>WebACL</code>. For more information, see <a>CreateRule</a> and <a>UpdateRule</a>. </li> <li> Create a <code>WebACL</code>. See <a>CreateWebACL</a>. </li> <li> Use <code>GetChangeToken</code> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <a>UpdateWebACL</a> request. </li> <li> Submit an <code>UpdateWebACL</code> request to specify the <code>Rules</code> that you want to include in the <code>WebACL</code>, to specify the default action, and to associate the <code>WebACL</code> with a CloudFront distribution.  The <code>ActivatedRule</code> can be a rule group. If you specify a rule group as your <code>ActivatedRule</code>, you can exclude specific rules from that rule group. If you already have a rule group associated with a web ACL and want to submit an <code>UpdateWebACL</code> request to exclude certain rules from that rule group, you must first remove the rule group from the web ACL, the re-insert it again, specifying the excluded rules. For details, see <a>ActivatedRule$ExcludedRules</a>.  </li> </ol> Be aware that if you try to add a RATE_BASED rule to a web ACL without setting the rule type when first creating the rule, the <a>UpdateWebACL</a> request will fail because the request tries to add a REGULAR rule (the default rule type) with the specified ID, which does not exist.  For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param DefaultAction:  A default action for the web ACL, either ALLOW or BLOCK. AWS WAF performs the default action if a request doesn't match the criteria in any of the rules in a web ACL.:<br/>Updates:  An array of updates to make to the <a>WebACL</a>. An array of <code>WebACLUpdate</code> objects that you want to insert into or delete from a <a>WebACL</a>. For more information, see the applicable data types: <ul> <li>  <a>WebACLUpdate</a>: Contains <code>Action</code> and <code>ActivatedRule</code>  </li> <li>  <a>ActivatedRule</a>: Contains <code>Action</code>, <code>OverrideAction</code>, <code>Priority</code>, <code>RuleId</code>, and <code>Type</code>. <code>ActivatedRule|OverrideAction</code> applies only when updating or adding a <code>RuleGroup</code> to a <code>WebACL</code>. In this case, you do not use <code>ActivatedRule|Action</code>. For all other update requests, <code>ActivatedRule|Action</code> is used instead of <code>ActivatedRule|OverrideAction</code>.  </li> <li>  <a>WafAction</a>: Contains <code>Type</code>  </li> </ul><br/>ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>WebACLId:  The <code>WebACLId</code> of the <a>WebACL</a> that you want to update. <code>WebACLId</code> is returned by <a>CreateWebACL</a> and by <a>ListWebACLs</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> Specifies how you want AWS WAF to respond to requests that match the settings in a <code>Rule</code>. Valid settings include the following: <ul> <li>  <code>ALLOW</code>: AWS WAF allows requests </li> <li>  <code>BLOCK</code>: AWS WAF blocks requests </li> <li>  <code>COUNT</code>: AWS WAF increments a counter of the requests that match all of the conditions in the rule. AWS WAF then continues to inspect the web request based on the remaining rules in the web ACL. You can't specify <code>COUNT</code> for the default action for a <code>WebACL</code>. </li> </ul><br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>UpdateWebACL</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def update_xss_match_set(self, Updates: List[XssMatchSetUpdate], ChangeToken: str, XssMatchSetId: str) -> UpdateXssMatchSetResponse:
        """Inserts or deletes <a>XssMatchTuple</a> objects (filters) in an <a>XssMatchSet</a>. For each <code>XssMatchTuple</code> object, you specify the following values: <ul> <li>  <code>Action</code>: Whether to insert the object into or delete the object from the array. To change an <code>XssMatchTuple</code>, you delete the existing object and add a new one. </li> <li>  <code>FieldToMatch</code>: The part of web requests that you want AWS WAF to inspect and, if you want AWS WAF to inspect a header or custom query parameter, the name of the header or parameter. </li> <li>  <code>TextTransformation</code>: Which text transformation, if any, to perform on the web request before inspecting the request for cross-site scripting attacks. You can only specify a single type of TextTransformation. </li> </ul> You use <code>XssMatchSet</code> objects to specify which CloudFront requests that you want to allow, block, or count. For example, if you're receiving requests that contain cross-site scripting attacks in the request body and you want to block the requests, you can create an <code>XssMatchSet</code> with the applicable settings, and then configure AWS WAF to block the requests.  To create and configure an <code>XssMatchSet</code>, perform the following steps: <ol> <li> Submit a <a>CreateXssMatchSet</a> request. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <a>UpdateIPSet</a> request. </li> <li> Submit an <code>UpdateXssMatchSet</code> request to specify the parts of web requests that you want AWS WAF to inspect for cross-site scripting attacks. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param Updates:  An array of <code>XssMatchSetUpdate</code> objects that you want to insert into or delete from an <a>XssMatchSet</a>. For more information, see the applicable data types: <ul> <li>  <a>XssMatchSetUpdate</a>: Contains <code>Action</code> and <code>XssMatchTuple</code>  </li> <li>  <a>XssMatchTuple</a>: Contains <code>FieldToMatch</code> and <code>TextTransformation</code>  </li> <li>  <a>FieldToMatch</a>: Contains <code>Data</code> and <code>Type</code>  </li> </ul><br/>ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>XssMatchSetId:  The <code>XssMatchSetId</code> of the <code>XssMatchSet</code> that you want to update. <code>XssMatchSetId</code> is returned by <a>CreateXssMatchSet</a> and by <a>ListXssMatchSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>UpdateXssMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass



class WAFRegional(BaseClient):
    """This is the <i>AWS WAF Regional API Reference</i> for using AWS WAF with Elastic Load Balancing (ELB) Application Load Balancers. The AWS WAF actions and data types listed in the reference are available for protecting Application Load Balancers. You can use these actions and data types by means of the endpoints listed in <a href="https://docs.aws.amazon.com/general/latest/gr/rande.html#waf_region">AWS Regions and Endpoints</a>. This guide is for developers who need detailed information about the AWS WAF API actions, data types, and errors. For detailed information about AWS WAF features and an overview of how to use the AWS WAF API, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>."""




    class AssociateWebACLRequest(Mapping):
        pass
    
    class AssociateWebACLResponse(Mapping):
        pass
    
    class CreateByteMatchSetRequest(Mapping):
        pass
    
    class CreateByteMatchSetResponse(Mapping):
        pass
    
    class CreateGeoMatchSetRequest(Mapping):
        pass
    
    class CreateGeoMatchSetResponse(Mapping):
        pass
    
    class CreateIPSetRequest(Mapping):
        pass
    
    class CreateIPSetResponse(Mapping):
        pass
    
    class CreateRateBasedRuleRequest(Mapping):
        pass
    
    class CreateRateBasedRuleResponse(Mapping):
        pass
    
    class RateLimit(object):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreateRegexMatchSetRequest(Mapping):
        pass
    
    class CreateRegexMatchSetResponse(Mapping):
        pass
    
    class CreateRegexPatternSetRequest(Mapping):
        pass
    
    class CreateRegexPatternSetResponse(Mapping):
        pass
    
    class CreateRuleRequest(Mapping):
        pass
    
    class CreateRuleResponse(Mapping):
        pass
    
    class CreateRuleGroupRequest(Mapping):
        pass
    
    class CreateRuleGroupResponse(Mapping):
        pass
    
    class CreateSizeConstraintSetRequest(Mapping):
        pass
    
    class CreateSizeConstraintSetResponse(Mapping):
        pass
    
    class CreateSqlInjectionMatchSetRequest(Mapping):
        pass
    
    class CreateSqlInjectionMatchSetResponse(Mapping):
        pass
    
    class CreateWebACLRequest(Mapping):
        pass
    
    class CreateWebACLResponse(Mapping):
        pass
    
    class WafAction(Mapping):
        pass
    
    class CreateXssMatchSetRequest(Mapping):
        pass
    
    class CreateXssMatchSetResponse(Mapping):
        pass
    
    class DeleteByteMatchSetRequest(Mapping):
        pass
    
    class DeleteByteMatchSetResponse(Mapping):
        pass
    
    class DeleteGeoMatchSetRequest(Mapping):
        pass
    
    class DeleteGeoMatchSetResponse(Mapping):
        pass
    
    class DeleteIPSetRequest(Mapping):
        pass
    
    class DeleteIPSetResponse(Mapping):
        pass
    
    class DeleteLoggingConfigurationRequest(Mapping):
        pass
    
    class DeleteLoggingConfigurationResponse(Mapping):
        pass
    
    class DeletePermissionPolicyRequest(Mapping):
        pass
    
    class DeletePermissionPolicyResponse(Mapping):
        pass
    
    class DeleteRateBasedRuleRequest(Mapping):
        pass
    
    class DeleteRateBasedRuleResponse(Mapping):
        pass
    
    class DeleteRegexMatchSetRequest(Mapping):
        pass
    
    class DeleteRegexMatchSetResponse(Mapping):
        pass
    
    class DeleteRegexPatternSetRequest(Mapping):
        pass
    
    class DeleteRegexPatternSetResponse(Mapping):
        pass
    
    class DeleteRuleRequest(Mapping):
        pass
    
    class DeleteRuleResponse(Mapping):
        pass
    
    class DeleteRuleGroupRequest(Mapping):
        pass
    
    class DeleteRuleGroupResponse(Mapping):
        pass
    
    class DeleteSizeConstraintSetRequest(Mapping):
        pass
    
    class DeleteSizeConstraintSetResponse(Mapping):
        pass
    
    class DeleteSqlInjectionMatchSetRequest(Mapping):
        pass
    
    class DeleteSqlInjectionMatchSetResponse(Mapping):
        pass
    
    class DeleteWebACLRequest(Mapping):
        pass
    
    class DeleteWebACLResponse(Mapping):
        pass
    
    class DeleteXssMatchSetRequest(Mapping):
        pass
    
    class DeleteXssMatchSetResponse(Mapping):
        pass
    
    class DisassociateWebACLRequest(Mapping):
        pass
    
    class DisassociateWebACLResponse(Mapping):
        pass
    
    class GetByteMatchSetRequest(Mapping):
        pass
    
    class GetByteMatchSetResponse(Mapping):
        pass
    
    class GetChangeTokenRequest(Mapping):
        pass
    
    class GetChangeTokenResponse(Mapping):
        pass
    
    class GetChangeTokenStatusRequest(Mapping):
        pass
    
    class GetChangeTokenStatusResponse(Mapping):
        pass
    
    class GetGeoMatchSetRequest(Mapping):
        pass
    
    class GetGeoMatchSetResponse(Mapping):
        pass
    
    class GetIPSetRequest(Mapping):
        pass
    
    class GetIPSetResponse(Mapping):
        pass
    
    class GetLoggingConfigurationRequest(Mapping):
        pass
    
    class GetLoggingConfigurationResponse(Mapping):
        pass
    
    class GetPermissionPolicyRequest(Mapping):
        pass
    
    class GetPermissionPolicyResponse(Mapping):
        pass
    
    class GetRateBasedRuleRequest(Mapping):
        pass
    
    class GetRateBasedRuleResponse(Mapping):
        pass
    
    class GetRateBasedRuleManagedKeysRequest(Mapping):
        pass
    
    class GetRateBasedRuleManagedKeysResponse(Mapping):
        pass
    
    class GetRegexMatchSetRequest(Mapping):
        pass
    
    class GetRegexMatchSetResponse(Mapping):
        pass
    
    class GetRegexPatternSetRequest(Mapping):
        pass
    
    class GetRegexPatternSetResponse(Mapping):
        pass
    
    class GetRuleRequest(Mapping):
        pass
    
    class GetRuleResponse(Mapping):
        pass
    
    class GetRuleGroupRequest(Mapping):
        pass
    
    class GetRuleGroupResponse(Mapping):
        pass
    
    class GetSampledRequestsRequest(Mapping):
        pass
    
    class GetSampledRequestsResponse(Mapping):
        pass
    
    class TimeWindow(Mapping):
        pass
    
    class GetSampledRequestsMaxItems(object):
        pass
    
    class GetSizeConstraintSetRequest(Mapping):
        pass
    
    class GetSizeConstraintSetResponse(Mapping):
        pass
    
    class GetSqlInjectionMatchSetRequest(Mapping):
        pass
    
    class GetSqlInjectionMatchSetResponse(Mapping):
        pass
    
    class GetWebACLRequest(Mapping):
        pass
    
    class GetWebACLResponse(Mapping):
        pass
    
    class GetWebACLForResourceRequest(Mapping):
        pass
    
    class GetWebACLForResourceResponse(Mapping):
        pass
    
    class GetXssMatchSetRequest(Mapping):
        pass
    
    class GetXssMatchSetResponse(Mapping):
        pass
    
    class ListActivatedRulesInRuleGroupRequest(Mapping):
        pass
    
    class ListActivatedRulesInRuleGroupResponse(Mapping):
        pass
    
    class ListByteMatchSetsRequest(Mapping):
        pass
    
    class ListByteMatchSetsResponse(Mapping):
        pass
    
    class ListGeoMatchSetsRequest(Mapping):
        pass
    
    class ListGeoMatchSetsResponse(Mapping):
        pass
    
    class ListIPSetsRequest(Mapping):
        pass
    
    class ListIPSetsResponse(Mapping):
        pass
    
    class ListLoggingConfigurationsRequest(Mapping):
        pass
    
    class ListLoggingConfigurationsResponse(Mapping):
        pass
    
    class ListRateBasedRulesRequest(Mapping):
        pass
    
    class ListRateBasedRulesResponse(Mapping):
        pass
    
    class ListRegexMatchSetsRequest(Mapping):
        pass
    
    class ListRegexMatchSetsResponse(Mapping):
        pass
    
    class ListRegexPatternSetsRequest(Mapping):
        pass
    
    class ListRegexPatternSetsResponse(Mapping):
        pass
    
    class ListResourcesForWebACLRequest(Mapping):
        pass
    
    class ListResourcesForWebACLResponse(Mapping):
        pass
    
    class ListRuleGroupsRequest(Mapping):
        pass
    
    class ListRuleGroupsResponse(Mapping):
        pass
    
    class ListRulesRequest(Mapping):
        pass
    
    class ListRulesResponse(Mapping):
        pass
    
    class ListSizeConstraintSetsRequest(Mapping):
        pass
    
    class ListSizeConstraintSetsResponse(Mapping):
        pass
    
    class ListSqlInjectionMatchSetsRequest(Mapping):
        pass
    
    class ListSqlInjectionMatchSetsResponse(Mapping):
        pass
    
    class ListSubscribedRuleGroupsRequest(Mapping):
        pass
    
    class ListSubscribedRuleGroupsResponse(Mapping):
        pass
    
    class ListTagsForResourceRequest(Mapping):
        pass
    
    class ListTagsForResourceResponse(Mapping):
        pass
    
    class ListWebACLsRequest(Mapping):
        pass
    
    class ListWebACLsResponse(Mapping):
        pass
    
    class ListXssMatchSetsRequest(Mapping):
        pass
    
    class ListXssMatchSetsResponse(Mapping):
        pass
    
    class PutLoggingConfigurationRequest(Mapping):
        pass
    
    class PutLoggingConfigurationResponse(Mapping):
        pass
    
    class LoggingConfiguration(Mapping):
        pass
    
    class PutPermissionPolicyRequest(Mapping):
        pass
    
    class PutPermissionPolicyResponse(Mapping):
        pass
    
    class TagResourceRequest(Mapping):
        pass
    
    class TagResourceResponse(Mapping):
        pass
    
    class UntagResourceRequest(Mapping):
        pass
    
    class UntagResourceResponse(Mapping):
        pass
    
    class TagKey(object):
        pass
    
    class UpdateByteMatchSetRequest(Mapping):
        pass
    
    class UpdateByteMatchSetResponse(Mapping):
        pass
    
    class ByteMatchSetUpdate(Mapping):
        pass
    
    class UpdateGeoMatchSetRequest(Mapping):
        pass
    
    class UpdateGeoMatchSetResponse(Mapping):
        pass
    
    class GeoMatchSetUpdate(Mapping):
        pass
    
    class UpdateIPSetRequest(Mapping):
        pass
    
    class UpdateIPSetResponse(Mapping):
        pass
    
    class IPSetUpdate(Mapping):
        pass
    
    class UpdateRateBasedRuleRequest(Mapping):
        pass
    
    class UpdateRateBasedRuleResponse(Mapping):
        pass
    
    class RuleUpdate(Mapping):
        pass
    
    class UpdateRegexMatchSetRequest(Mapping):
        pass
    
    class UpdateRegexMatchSetResponse(Mapping):
        pass
    
    class RegexMatchSetUpdate(Mapping):
        pass
    
    class UpdateRegexPatternSetRequest(Mapping):
        pass
    
    class UpdateRegexPatternSetResponse(Mapping):
        pass
    
    class RegexPatternSetUpdate(Mapping):
        pass
    
    class UpdateRuleRequest(Mapping):
        pass
    
    class UpdateRuleResponse(Mapping):
        pass
    
    class UpdateRuleGroupRequest(Mapping):
        pass
    
    class UpdateRuleGroupResponse(Mapping):
        pass
    
    class RuleGroupUpdate(Mapping):
        pass
    
    class UpdateSizeConstraintSetRequest(Mapping):
        pass
    
    class UpdateSizeConstraintSetResponse(Mapping):
        pass
    
    class SizeConstraintSetUpdate(Mapping):
        pass
    
    class UpdateSqlInjectionMatchSetRequest(Mapping):
        pass
    
    class UpdateSqlInjectionMatchSetResponse(Mapping):
        pass
    
    class SqlInjectionMatchSetUpdate(Mapping):
        pass
    
    class UpdateWebACLRequest(Mapping):
        pass
    
    class UpdateWebACLResponse(Mapping):
        pass
    
    class WebACLUpdate(Mapping):
        pass
    
    class UpdateXssMatchSetRequest(Mapping):
        pass
    
    class UpdateXssMatchSetResponse(Mapping):
        pass
    
    class XssMatchSetUpdate(Mapping):
        pass
    
    def associate_web_acl(self, ResourceArn: str, WebACLId: str) -> AssociateWebACLResponse:
        """Associates a web ACL with a resource, either an application load balancer or Amazon API Gateway stage.

        :param self:
        :param ResourceArn:  The ARN (Amazon Resource Name) of the resource to be protected, either an application load balancer or Amazon API Gateway stage.  The ARN should be in one of the following formats: <ul> <li> For an Application Load Balancer: <code>arn:aws:elasticloadbalancing:<i>region</i>:<i>account-id</i>:loadbalancer/app/<i>load-balancer-name</i>/<i>load-balancer-id</i> </code>  </li> <li> For an Amazon API Gateway stage: <code>arn:aws:apigateway:<i>region</i>::/restapis/<i>api-id</i>/stages/<i>stage-name</i> </code>  </li> </ul><br/>WebACLId:  A unique identifier (ID) for the web ACL. <br/>
        :return:         """
        pass

    def create_byte_match_set(self, ChangeToken: str, Name: str) -> CreateByteMatchSetResponse:
        """Creates a <code>ByteMatchSet</code>. You then use <a>UpdateByteMatchSet</a> to identify the part of a web request that you want AWS WAF to inspect, such as the values of the <code>User-Agent</code> header or the query string. For example, you can create a <code>ByteMatchSet</code> that matches any requests with <code>User-Agent</code> headers that contain the string <code>BadBot</code>. You can then configure AWS WAF to reject those requests. To create and configure a <code>ByteMatchSet</code>, perform the following steps: <ol> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>CreateByteMatchSet</code> request. </li> <li> Submit a <code>CreateByteMatchSet</code> request. </li> <li> Use <code>GetChangeToken</code> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <code>UpdateByteMatchSet</code> request. </li> <li> Submit an <a>UpdateByteMatchSet</a> request to specify the part of the request that you want AWS WAF to inspect (for example, the header or the URI) and the value that you want AWS WAF to watch for. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>Name:  A friendly name or description of the <a>ByteMatchSet</a>. You can't change <code>Name</code> after you create a <code>ByteMatchSet</code>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>CreateByteMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>ByteMatchSet:  A <a>ByteMatchSet</a> that contains no <code>ByteMatchTuple</code> objects.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ByteMatchTuples</i> Specifies the bytes (typically a string that corresponds with ASCII characters) that you want AWS WAF to search for in web requests, the location in requests that you want AWS WAF to search, and other settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A friendly name or description of the <a>ByteMatchSet</a>. You can't change <code>Name</code> after you create a <code>ByteMatchSet</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ByteMatchSetId</i> The <code>ByteMatchSetId</code> for a <code>ByteMatchSet</code>. You use <code>ByteMatchSetId</code> to get information about a <code>ByteMatchSet</code> (see <a>GetByteMatchSet</a>), update a <code>ByteMatchSet</code> (see <a>UpdateByteMatchSet</a>), insert a <code>ByteMatchSet</code> into a <code>Rule</code> or delete one from a <code>Rule</code> (see <a>UpdateRule</a>), and delete a <code>ByteMatchSet</code> from AWS WAF (see <a>DeleteByteMatchSet</a>).  <code>ByteMatchSetId</code> is returned by <a>CreateByteMatchSet</a> and by <a>ListByteMatchSets</a>.<br/>        """
        pass

    def create_geo_match_set(self, ChangeToken: str, Name: str) -> CreateGeoMatchSetResponse:
        """Creates an <a>GeoMatchSet</a>, which you use to specify which web requests you want to allow or block based on the country that the requests originate from. For example, if you're receiving a lot of requests from one or more countries and you want to block the requests, you can create an <code>GeoMatchSet</code> that contains those countries and then configure AWS WAF to block the requests.  To create and configure a <code>GeoMatchSet</code>, perform the following steps: <ol> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>CreateGeoMatchSet</code> request. </li> <li> Submit a <code>CreateGeoMatchSet</code> request. </li> <li> Use <code>GetChangeToken</code> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <a>UpdateGeoMatchSet</a> request. </li> <li> Submit an <code>UpdateGeoMatchSetSet</code> request to specify the countries that you want AWS WAF to watch for. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>Name:  A friendly name or description of the <a>GeoMatchSet</a>. You can't change <code>Name</code> after you create the <code>GeoMatchSet</code>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>CreateGeoMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>GeoMatchSet:  The <a>GeoMatchSet</a> returned in the <code>CreateGeoMatchSet</code> response. The <code>GeoMatchSet</code> contains no <code>GeoMatchConstraints</code>.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GeoMatchConstraints</i> An array of <a>GeoMatchConstraint</a> objects, which contain the country that you want AWS WAF to search for.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A friendly name or description of the <a>GeoMatchSet</a>. You can't change the name of an <code>GeoMatchSet</code> after you create it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GeoMatchSetId</i> The <code>GeoMatchSetId</code> for an <code>GeoMatchSet</code>. You use <code>GeoMatchSetId</code> to get information about a <code>GeoMatchSet</code> (see <a>GeoMatchSet</a>), update a <code>GeoMatchSet</code> (see <a>UpdateGeoMatchSet</a>), insert a <code>GeoMatchSet</code> into a <code>Rule</code> or delete one from a <code>Rule</code> (see <a>UpdateRule</a>), and delete a <code>GeoMatchSet</code> from AWS WAF (see <a>DeleteGeoMatchSet</a>).  <code>GeoMatchSetId</code> is returned by <a>CreateGeoMatchSet</a> and by <a>ListGeoMatchSets</a>.<br/>        """
        pass

    def create_ip_set(self, ChangeToken: str, Name: str) -> CreateIPSetResponse:
        """Creates an <a>IPSet</a>, which you use to specify which web requests that you want to allow or block based on the IP addresses that the requests originate from. For example, if you're receiving a lot of requests from one or more individual IP addresses or one or more ranges of IP addresses and you want to block the requests, you can create an <code>IPSet</code> that contains those IP addresses and then configure AWS WAF to block the requests.  To create and configure an <code>IPSet</code>, perform the following steps: <ol> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>CreateIPSet</code> request. </li> <li> Submit a <code>CreateIPSet</code> request. </li> <li> Use <code>GetChangeToken</code> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <a>UpdateIPSet</a> request. </li> <li> Submit an <code>UpdateIPSet</code> request to specify the IP addresses that you want AWS WAF to watch for. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>Name:  A friendly name or description of the <a>IPSet</a>. You can't change <code>Name</code> after you create the <code>IPSet</code>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>CreateIPSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>IPSet:  The <a>IPSet</a> returned in the <code>CreateIPSet</code> response.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IPSetDescriptors</i> The IP address type (<code>IPV4</code> or <code>IPV6</code>) and the IP address range (in CIDR notation) that web requests originate from. If the <code>WebACL</code> is associated with a CloudFront distribution and the viewer did not use an HTTP proxy or a load balancer to send the request, this is the value of the c-ip field in the CloudFront access logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A friendly name or description of the <a>IPSet</a>. You can't change the name of an <code>IPSet</code> after you create it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IPSetId</i> The <code>IPSetId</code> for an <code>IPSet</code>. You use <code>IPSetId</code> to get information about an <code>IPSet</code> (see <a>GetIPSet</a>), update an <code>IPSet</code> (see <a>UpdateIPSet</a>), insert an <code>IPSet</code> into a <code>Rule</code> or delete one from a <code>Rule</code> (see <a>UpdateRule</a>), and delete an <code>IPSet</code> from AWS WAF (see <a>DeleteIPSet</a>).  <code>IPSetId</code> is returned by <a>CreateIPSet</a> and by <a>ListIPSets</a>.<br/>        """
        pass

    def create_rate_based_rule(self, ChangeToken: str, RateLimit: RateLimit, RateKey: str, MetricName: str, Name: str, Tags: List[Tag]=None) -> CreateRateBasedRuleResponse:
        """Creates a <a>RateBasedRule</a>. The <code>RateBasedRule</code> contains a <code>RateLimit</code>, which specifies the maximum number of requests that AWS WAF allows from a specified IP address in a five-minute period. The <code>RateBasedRule</code> also contains the <code>IPSet</code> objects, <code>ByteMatchSet</code> objects, and other predicates that identify the requests that you want to count or block if these requests exceed the <code>RateLimit</code>. If you add more than one predicate to a <code>RateBasedRule</code>, a request not only must exceed the <code>RateLimit</code>, but it also must match all the specifications to be counted or blocked. For example, suppose you add the following to a <code>RateBasedRule</code>: <ul> <li> An <code>IPSet</code> that matches the IP address <code>192.0.2.44/32</code>  </li> <li> A <code>ByteMatchSet</code> that matches <code>BadBot</code> in the <code>User-Agent</code> header </li> </ul> Further, you specify a <code>RateLimit</code> of 15,000. You then add the <code>RateBasedRule</code> to a <code>WebACL</code> and specify that you want to block requests that meet the conditions in the rule. For a request to be blocked, it must come from the IP address 192.0.2.44 <i>and</i> the <code>User-Agent</code> header in the request must contain the value <code>BadBot</code>. Further, requests that match these two conditions must be received at a rate of more than 15,000 requests every five minutes. If both conditions are met and the rate is exceeded, AWS WAF blocks the requests. If the rate drops below 15,000 for a five-minute period, AWS WAF no longer blocks the requests. As a second example, suppose you want to limit requests to a particular page on your site. To do this, you could add the following to a <code>RateBasedRule</code>: <ul> <li> A <code>ByteMatchSet</code> with <code>FieldToMatch</code> of <code>URI</code>  </li> <li> A <code>PositionalConstraint</code> of <code>STARTS_WITH</code>  </li> <li> A <code>TargetString</code> of <code>login</code>  </li> </ul> Further, you specify a <code>RateLimit</code> of 15,000. By adding this <code>RateBasedRule</code> to a <code>WebACL</code>, you could limit requests to your login page without affecting the rest of your site. To create and configure a <code>RateBasedRule</code>, perform the following steps: <ol> <li> Create and update the predicates that you want to include in the rule. For more information, see <a>CreateByteMatchSet</a>, <a>CreateIPSet</a>, and <a>CreateSqlInjectionMatchSet</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>CreateRule</code> request. </li> <li> Submit a <code>CreateRateBasedRule</code> request. </li> <li> Use <code>GetChangeToken</code> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <a>UpdateRule</a> request. </li> <li> Submit an <code>UpdateRateBasedRule</code> request to specify the predicates that you want to include in the rule. </li> <li> Create and update a <code>WebACL</code> that contains the <code>RateBasedRule</code>. For more information, see <a>CreateWebACL</a>. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param Tags:  <br/>ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>CreateRateBasedRule</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>RateLimit:  The maximum number of requests, which have an identical value in the field that is specified by <code>RateKey</code>, allowed in a five-minute period. If the number of requests exceeds the <code>RateLimit</code> and the other predicates specified in the rule are also met, AWS WAF triggers the action that is specified for this rule.<br/>RateKey:  The field that AWS WAF uses to determine if requests are likely arriving from a single source and thus subject to rate monitoring. The only valid value for <code>RateKey</code> is <code>IP</code>. <code>IP</code> indicates that requests that arrive from the same IP address are subject to the <code>RateLimit</code> that is specified in the <code>RateBasedRule</code>.<br/>MetricName:  A friendly name or description for the metrics for this <code>RateBasedRule</code>. The name can contain only alphanumeric characters (A-Z, a-z, 0-9), with maximum length 128 and minimum length one. It can't contain whitespace or metric names reserved for AWS WAF, including "All" and "Default_Action." You can't change the name of the metric after you create the <code>RateBasedRule</code>.<br/>Name:  A friendly name or description of the <a>RateBasedRule</a>. You can't change the name of a <code>RateBasedRule</code> after you create it.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>CreateRateBasedRule</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>Rule:  The <a>RateBasedRule</a> that is returned in the <code>CreateRateBasedRule</code> response.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RateLimit</i> The maximum number of requests, which have an identical value in the field specified by the <code>RateKey</code>, allowed in a five-minute period. If the number of requests exceeds the <code>RateLimit</code> and the other predicates specified in the rule are also met, AWS WAF triggers the action that is specified for this rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RateKey</i> The field that AWS WAF uses to determine if requests are likely arriving from single source and thus subject to rate monitoring. The only valid value for <code>RateKey</code> is <code>IP</code>. <code>IP</code> indicates that requests arriving from the same IP address are subject to the <code>RateLimit</code> that is specified in the <code>RateBasedRule</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MatchPredicates</i> The <code>Predicates</code> object contains one <code>Predicate</code> element for each <a>ByteMatchSet</a>, <a>IPSet</a>, or <a>SqlInjectionMatchSet</a> object that you want to include in a <code>RateBasedRule</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricName</i> A friendly name or description for the metrics for a <code>RateBasedRule</code>. The name can contain only alphanumeric characters (A-Z, a-z, 0-9), with maximum length 128 and minimum length one. It can't contain whitespace or metric names reserved for AWS WAF, including "All" and "Default_Action." You can't change the name of the metric after you create the <code>RateBasedRule</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A friendly name or description for a <code>RateBasedRule</code>. You can't change the name of a <code>RateBasedRule</code> after you create it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleId</i> A unique identifier for a <code>RateBasedRule</code>. You use <code>RuleId</code> to get more information about a <code>RateBasedRule</code> (see <a>GetRateBasedRule</a>), update a <code>RateBasedRule</code> (see <a>UpdateRateBasedRule</a>), insert a <code>RateBasedRule</code> into a <code>WebACL</code> or delete one from a <code>WebACL</code> (see <a>UpdateWebACL</a>), or delete a <code>RateBasedRule</code> from AWS WAF (see <a>DeleteRateBasedRule</a>).<br/>        """
        pass

    def create_regex_match_set(self, ChangeToken: str, Name: str) -> CreateRegexMatchSetResponse:
        """Creates a <a>RegexMatchSet</a>. You then use <a>UpdateRegexMatchSet</a> to identify the part of a web request that you want AWS WAF to inspect, such as the values of the <code>User-Agent</code> header or the query string. For example, you can create a <code>RegexMatchSet</code> that contains a <code>RegexMatchTuple</code> that looks for any requests with <code>User-Agent</code> headers that match a <code>RegexPatternSet</code> with pattern <code>B[a@]dB[o0]t</code>. You can then configure AWS WAF to reject those requests. To create and configure a <code>RegexMatchSet</code>, perform the following steps: <ol> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>CreateRegexMatchSet</code> request. </li> <li> Submit a <code>CreateRegexMatchSet</code> request. </li> <li> Use <code>GetChangeToken</code> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <code>UpdateRegexMatchSet</code> request. </li> <li> Submit an <a>UpdateRegexMatchSet</a> request to specify the part of the request that you want AWS WAF to inspect (for example, the header or the URI) and the value, using a <code>RegexPatternSet</code>, that you want AWS WAF to watch for. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>Name:  A friendly name or description of the <a>RegexMatchSet</a>. You can't change <code>Name</code> after you create a <code>RegexMatchSet</code>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>CreateRegexMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>RegexMatchSet:  A <a>RegexMatchSet</a> that contains no <code>RegexMatchTuple</code> objects.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RegexMatchTuples</i> Contains an array of <a>RegexMatchTuple</a> objects. Each <code>RegexMatchTuple</code> object contains:  <ul> <li> The part of a web request that you want AWS WAF to inspect, such as a query string or the value of the <code>User-Agent</code> header.  </li> <li> The identifier of the pattern (a regular expression) that you want AWS WAF to look for. For more information, see <a>RegexPatternSet</a>. </li> <li> Whether to perform any conversions on the request, such as converting it to lowercase, before inspecting it for the specified string. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A friendly name or description of the <a>RegexMatchSet</a>. You can't change <code>Name</code> after you create a <code>RegexMatchSet</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RegexMatchSetId</i> The <code>RegexMatchSetId</code> for a <code>RegexMatchSet</code>. You use <code>RegexMatchSetId</code> to get information about a <code>RegexMatchSet</code> (see <a>GetRegexMatchSet</a>), update a <code>RegexMatchSet</code> (see <a>UpdateRegexMatchSet</a>), insert a <code>RegexMatchSet</code> into a <code>Rule</code> or delete one from a <code>Rule</code> (see <a>UpdateRule</a>), and delete a <code>RegexMatchSet</code> from AWS WAF (see <a>DeleteRegexMatchSet</a>).  <code>RegexMatchSetId</code> is returned by <a>CreateRegexMatchSet</a> and by <a>ListRegexMatchSets</a>.<br/>        """
        pass

    def create_regex_pattern_set(self, ChangeToken: str, Name: str) -> CreateRegexPatternSetResponse:
        """Creates a <code>RegexPatternSet</code>. You then use <a>UpdateRegexPatternSet</a> to specify the regular expression (regex) pattern that you want AWS WAF to search for, such as <code>B[a@]dB[o0]t</code>. You can then configure AWS WAF to reject those requests. To create and configure a <code>RegexPatternSet</code>, perform the following steps: <ol> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>CreateRegexPatternSet</code> request. </li> <li> Submit a <code>CreateRegexPatternSet</code> request. </li> <li> Use <code>GetChangeToken</code> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <code>UpdateRegexPatternSet</code> request. </li> <li> Submit an <a>UpdateRegexPatternSet</a> request to specify the string that you want AWS WAF to watch for. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>Name:  A friendly name or description of the <a>RegexPatternSet</a>. You can't change <code>Name</code> after you create a <code>RegexPatternSet</code>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>CreateRegexPatternSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>RegexPatternSet:  A <a>RegexPatternSet</a> that contains no objects.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RegexPatternStrings</i> Specifies the regular expression (regex) patterns that you want AWS WAF to search for, such as <code>B[a@]dB[o0]t</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A friendly name or description of the <a>RegexPatternSet</a>. You can't change <code>Name</code> after you create a <code>RegexPatternSet</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RegexPatternSetId</i> The identifier for the <code>RegexPatternSet</code>. You use <code>RegexPatternSetId</code> to get information about a <code>RegexPatternSet</code>, update a <code>RegexPatternSet</code>, remove a <code>RegexPatternSet</code> from a <code>RegexMatchSet</code>, and delete a <code>RegexPatternSet</code> from AWS WAF.  <code>RegexMatchSetId</code> is returned by <a>CreateRegexPatternSet</a> and by <a>ListRegexPatternSets</a>.<br/>        """
        pass

    def create_rule(self, ChangeToken: str, MetricName: str, Name: str, Tags: List[Tag]=None) -> CreateRuleResponse:
        """Creates a <code>Rule</code>, which contains the <code>IPSet</code> objects, <code>ByteMatchSet</code> objects, and other predicates that identify the requests that you want to block. If you add more than one predicate to a <code>Rule</code>, a request must match all of the specifications to be allowed or blocked. For example, suppose that you add the following to a <code>Rule</code>: <ul> <li> An <code>IPSet</code> that matches the IP address <code>192.0.2.44/32</code>  </li> <li> A <code>ByteMatchSet</code> that matches <code>BadBot</code> in the <code>User-Agent</code> header </li> </ul> You then add the <code>Rule</code> to a <code>WebACL</code> and specify that you want to blocks requests that satisfy the <code>Rule</code>. For a request to be blocked, it must come from the IP address 192.0.2.44 <i>and</i> the <code>User-Agent</code> header in the request must contain the value <code>BadBot</code>. To create and configure a <code>Rule</code>, perform the following steps: <ol> <li> Create and update the predicates that you want to include in the <code>Rule</code>. For more information, see <a>CreateByteMatchSet</a>, <a>CreateIPSet</a>, and <a>CreateSqlInjectionMatchSet</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>CreateRule</code> request. </li> <li> Submit a <code>CreateRule</code> request. </li> <li> Use <code>GetChangeToken</code> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <a>UpdateRule</a> request. </li> <li> Submit an <code>UpdateRule</code> request to specify the predicates that you want to include in the <code>Rule</code>. </li> <li> Create and update a <code>WebACL</code> that contains the <code>Rule</code>. For more information, see <a>CreateWebACL</a>. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param Tags:  <br/>ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>MetricName:  A friendly name or description for the metrics for this <code>Rule</code>. The name can contain only alphanumeric characters (A-Z, a-z, 0-9), with maximum length 128 and minimum length one. It can't contain whitespace or metric names reserved for AWS WAF, including "All" and "Default_Action." You can't change the name of the metric after you create the <code>Rule</code>.<br/>Name:  A friendly name or description of the <a>Rule</a>. You can't change the name of a <code>Rule</code> after you create it.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>CreateRule</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>Rule:  The <a>Rule</a> returned in the <code>CreateRule</code> response.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Predicates</i> The <code>Predicates</code> object contains one <code>Predicate</code> element for each <a>ByteMatchSet</a>, <a>IPSet</a>, or <a>SqlInjectionMatchSet</a> object that you want to include in a <code>Rule</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricName</i> A friendly name or description for the metrics for this <code>Rule</code>. The name can contain only alphanumeric characters (A-Z, a-z, 0-9), with maximum length 128 and minimum length one. It can't contain whitespace or metric names reserved for AWS WAF, including "All" and "Default_Action." You can't change <code>MetricName</code> after you create the <code>Rule</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The friendly name or description for the <code>Rule</code>. You can't change the name of a <code>Rule</code> after you create it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleId</i> A unique identifier for a <code>Rule</code>. You use <code>RuleId</code> to get more information about a <code>Rule</code> (see <a>GetRule</a>), update a <code>Rule</code> (see <a>UpdateRule</a>), insert a <code>Rule</code> into a <code>WebACL</code> or delete a one from a <code>WebACL</code> (see <a>UpdateWebACL</a>), or delete a <code>Rule</code> from AWS WAF (see <a>DeleteRule</a>).  <code>RuleId</code> is returned by <a>CreateRule</a> and by <a>ListRules</a>.<br/>        """
        pass

    def create_rule_group(self, ChangeToken: str, MetricName: str, Name: str, Tags: List[Tag]=None) -> CreateRuleGroupResponse:
        """Creates a <code>RuleGroup</code>. A rule group is a collection of predefined rules that you add to a web ACL. You use <a>UpdateRuleGroup</a> to add rules to the rule group. Rule groups are subject to the following limits: <ul> <li> Three rule groups per account. You can request an increase to this limit by contacting customer support. </li> <li> One rule group per web ACL. </li> <li> Ten rules per rule group. </li> </ul> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param Tags:  <br/>ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>MetricName:  A friendly name or description for the metrics for this <code>RuleGroup</code>. The name can contain only alphanumeric characters (A-Z, a-z, 0-9), with maximum length 128 and minimum length one. It can't contain whitespace or metric names reserved for AWS WAF, including "All" and "Default_Action." You can't change the name of the metric after you create the <code>RuleGroup</code>.<br/>Name:  A friendly name or description of the <a>RuleGroup</a>. You can't change <code>Name</code> after you create a <code>RuleGroup</code>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>CreateRuleGroup</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>RuleGroup:  An empty <a>RuleGroup</a>.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricName</i> A friendly name or description for the metrics for this <code>RuleGroup</code>. The name can contain only alphanumeric characters (A-Z, a-z, 0-9), with maximum length 128 and minimum length one. It can't contain whitespace or metric names reserved for AWS WAF, including "All" and "Default_Action." You can't change the name of the metric after you create the <code>RuleGroup</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The friendly name or description for the <code>RuleGroup</code>. You can't change the name of a <code>RuleGroup</code> after you create it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleGroupId</i> A unique identifier for a <code>RuleGroup</code>. You use <code>RuleGroupId</code> to get more information about a <code>RuleGroup</code> (see <a>GetRuleGroup</a>), update a <code>RuleGroup</code> (see <a>UpdateRuleGroup</a>), insert a <code>RuleGroup</code> into a <code>WebACL</code> or delete a one from a <code>WebACL</code> (see <a>UpdateWebACL</a>), or delete a <code>RuleGroup</code> from AWS WAF (see <a>DeleteRuleGroup</a>).  <code>RuleGroupId</code> is returned by <a>CreateRuleGroup</a> and by <a>ListRuleGroups</a>.<br/>        """
        pass

    def create_size_constraint_set(self, ChangeToken: str, Name: str) -> CreateSizeConstraintSetResponse:
        """Creates a <code>SizeConstraintSet</code>. You then use <a>UpdateSizeConstraintSet</a> to identify the part of a web request that you want AWS WAF to check for length, such as the length of the <code>User-Agent</code> header or the length of the query string. For example, you can create a <code>SizeConstraintSet</code> that matches any requests that have a query string that is longer than 100 bytes. You can then configure AWS WAF to reject those requests. To create and configure a <code>SizeConstraintSet</code>, perform the following steps: <ol> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>CreateSizeConstraintSet</code> request. </li> <li> Submit a <code>CreateSizeConstraintSet</code> request. </li> <li> Use <code>GetChangeToken</code> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <code>UpdateSizeConstraintSet</code> request. </li> <li> Submit an <a>UpdateSizeConstraintSet</a> request to specify the part of the request that you want AWS WAF to inspect (for example, the header or the URI) and the value that you want AWS WAF to watch for. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>Name:  A friendly name or description of the <a>SizeConstraintSet</a>. You can't change <code>Name</code> after you create a <code>SizeConstraintSet</code>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>CreateSizeConstraintSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>SizeConstraintSet:  A <a>SizeConstraintSet</a> that contains no <code>SizeConstraint</code> objects.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SizeConstraints</i> Specifies the parts of web requests that you want to inspect the size of.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name, if any, of the <code>SizeConstraintSet</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SizeConstraintSetId</i> A unique identifier for a <code>SizeConstraintSet</code>. You use <code>SizeConstraintSetId</code> to get information about a <code>SizeConstraintSet</code> (see <a>GetSizeConstraintSet</a>), update a <code>SizeConstraintSet</code> (see <a>UpdateSizeConstraintSet</a>), insert a <code>SizeConstraintSet</code> into a <code>Rule</code> or delete one from a <code>Rule</code> (see <a>UpdateRule</a>), and delete a <code>SizeConstraintSet</code> from AWS WAF (see <a>DeleteSizeConstraintSet</a>).  <code>SizeConstraintSetId</code> is returned by <a>CreateSizeConstraintSet</a> and by <a>ListSizeConstraintSets</a>.<br/>        """
        pass

    def create_sql_injection_match_set(self, ChangeToken: str, Name: str) -> CreateSqlInjectionMatchSetResponse:
        """Creates a <a>SqlInjectionMatchSet</a>, which you use to allow, block, or count requests that contain snippets of SQL code in a specified part of web requests. AWS WAF searches for character sequences that are likely to be malicious strings. To create and configure a <code>SqlInjectionMatchSet</code>, perform the following steps: <ol> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>CreateSqlInjectionMatchSet</code> request. </li> <li> Submit a <code>CreateSqlInjectionMatchSet</code> request. </li> <li> Use <code>GetChangeToken</code> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <a>UpdateSqlInjectionMatchSet</a> request. </li> <li> Submit an <a>UpdateSqlInjectionMatchSet</a> request to specify the parts of web requests in which you want to allow, block, or count malicious SQL code. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>Name:  A friendly name or description for the <a>SqlInjectionMatchSet</a> that you're creating. You can't change <code>Name</code> after you create the <code>SqlInjectionMatchSet</code>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>CreateSqlInjectionMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>SqlInjectionMatchSet:  A <a>SqlInjectionMatchSet</a>.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SqlInjectionMatchTuples</i> Specifies the parts of web requests that you want to inspect for snippets of malicious SQL code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name, if any, of the <code>SqlInjectionMatchSet</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SqlInjectionMatchSetId</i> A unique identifier for a <code>SqlInjectionMatchSet</code>. You use <code>SqlInjectionMatchSetId</code> to get information about a <code>SqlInjectionMatchSet</code> (see <a>GetSqlInjectionMatchSet</a>), update a <code>SqlInjectionMatchSet</code> (see <a>UpdateSqlInjectionMatchSet</a>), insert a <code>SqlInjectionMatchSet</code> into a <code>Rule</code> or delete one from a <code>Rule</code> (see <a>UpdateRule</a>), and delete a <code>SqlInjectionMatchSet</code> from AWS WAF (see <a>DeleteSqlInjectionMatchSet</a>).  <code>SqlInjectionMatchSetId</code> is returned by <a>CreateSqlInjectionMatchSet</a> and by <a>ListSqlInjectionMatchSets</a>.<br/>        """
        pass

    def create_web_acl(self, ChangeToken: str, DefaultAction: WafAction, MetricName: str, Name: str, Tags: List[Tag]=None) -> CreateWebACLResponse:
        """Creates a <code>WebACL</code>, which contains the <code>Rules</code> that identify the CloudFront web requests that you want to allow, block, or count. AWS WAF evaluates <code>Rules</code> in order based on the value of <code>Priority</code> for each <code>Rule</code>. You also specify a default action, either <code>ALLOW</code> or <code>BLOCK</code>. If a web request doesn't match any of the <code>Rules</code> in a <code>WebACL</code>, AWS WAF responds to the request with the default action.  To create and configure a <code>WebACL</code>, perform the following steps: <ol> <li> Create and update the <code>ByteMatchSet</code> objects and other predicates that you want to include in <code>Rules</code>. For more information, see <a>CreateByteMatchSet</a>, <a>UpdateByteMatchSet</a>, <a>CreateIPSet</a>, <a>UpdateIPSet</a>, <a>CreateSqlInjectionMatchSet</a>, and <a>UpdateSqlInjectionMatchSet</a>. </li> <li> Create and update the <code>Rules</code> that you want to include in the <code>WebACL</code>. For more information, see <a>CreateRule</a> and <a>UpdateRule</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>CreateWebACL</code> request. </li> <li> Submit a <code>CreateWebACL</code> request. </li> <li> Use <code>GetChangeToken</code> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <a>UpdateWebACL</a> request. </li> <li> Submit an <a>UpdateWebACL</a> request to specify the <code>Rules</code> that you want to include in the <code>WebACL</code>, to specify the default action, and to associate the <code>WebACL</code> with a CloudFront distribution. </li> </ol> For more information about how to use the AWS WAF API, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param Tags:  <br/>ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>DefaultAction:  The action that you want AWS WAF to take when a request doesn't match the criteria specified in any of the <code>Rule</code> objects that are associated with the <code>WebACL</code>.:<br/>MetricName:  A friendly name or description for the metrics for this <code>WebACL</code>.The name can contain only alphanumeric characters (A-Z, a-z, 0-9), with maximum length 128 and minimum length one. It can't contain whitespace or metric names reserved for AWS WAF, including "All" and "Default_Action." You can't change <code>MetricName</code> after you create the <code>WebACL</code>.<br/>Name:  A friendly name or description of the <a>WebACL</a>. You can't change <code>Name</code> after you create the <code>WebACL</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> Specifies how you want AWS WAF to respond to requests that match the settings in a <code>Rule</code>. Valid settings include the following: <ul> <li>  <code>ALLOW</code>: AWS WAF allows requests </li> <li>  <code>BLOCK</code>: AWS WAF blocks requests </li> <li>  <code>COUNT</code>: AWS WAF increments a counter of the requests that match all of the conditions in the rule. AWS WAF then continues to inspect the web request based on the remaining rules in the web ACL. You can't specify <code>COUNT</code> for the default action for a <code>WebACL</code>. </li> </ul><br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>CreateWebACL</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>WebACL:  The <a>WebACL</a> returned in the <code>CreateWebACL</code> response.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WebACLArn</i> Tha Amazon Resource Name (ARN) of the web ACL.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Rules</i> An array that contains the action for each <code>Rule</code> in a <code>WebACL</code>, the priority of the <code>Rule</code>, and the ID of the <code>Rule</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultAction</i> The action to perform if none of the <code>Rules</code> contained in the <code>WebACL</code> match. The action is specified by the <a>WafAction</a> object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DefaultAction</b>: The action to perform if none of the <code>Rules</code> contained in the <code>WebACL</code> match. The action is specified by the <a>WafAction</a> object.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricName</i> A friendly name or description for the metrics for this <code>WebACL</code>. The name can contain only alphanumeric characters (A-Z, a-z, 0-9), with maximum length 128 and minimum length one. It can't contain whitespace or metric names reserved for AWS WAF, including "All" and "Default_Action." You can't change <code>MetricName</code> after you create the <code>WebACL</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A friendly name or description of the <code>WebACL</code>. You can't change the name of a <code>WebACL</code> after you create it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WebACLId</i> A unique identifier for a <code>WebACL</code>. You use <code>WebACLId</code> to get information about a <code>WebACL</code> (see <a>GetWebACL</a>), update a <code>WebACL</code> (see <a>UpdateWebACL</a>), and delete a <code>WebACL</code> from AWS WAF (see <a>DeleteWebACL</a>).  <code>WebACLId</code> is returned by <a>CreateWebACL</a> and by <a>ListWebACLs</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> Specifies how you want AWS WAF to respond to requests that match the settings in a <code>Rule</code>. Valid settings include the following: <ul> <li>  <code>ALLOW</code>: AWS WAF allows requests </li> <li>  <code>BLOCK</code>: AWS WAF blocks requests </li> <li>  <code>COUNT</code>: AWS WAF increments a counter of the requests that match all of the conditions in the rule. AWS WAF then continues to inspect the web request based on the remaining rules in the web ACL. You can't specify <code>COUNT</code> for the default action for a <code>WebACL</code>. </li> </ul><br/>        """
        pass

    def create_xss_match_set(self, ChangeToken: str, Name: str) -> CreateXssMatchSetResponse:
        """Creates an <a>XssMatchSet</a>, which you use to allow, block, or count requests that contain cross-site scripting attacks in the specified part of web requests. AWS WAF searches for character sequences that are likely to be malicious strings. To create and configure an <code>XssMatchSet</code>, perform the following steps: <ol> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>CreateXssMatchSet</code> request. </li> <li> Submit a <code>CreateXssMatchSet</code> request. </li> <li> Use <code>GetChangeToken</code> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <a>UpdateXssMatchSet</a> request. </li> <li> Submit an <a>UpdateXssMatchSet</a> request to specify the parts of web requests in which you want to allow, block, or count cross-site scripting attacks. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>Name:  A friendly name or description for the <a>XssMatchSet</a> that you're creating. You can't change <code>Name</code> after you create the <code>XssMatchSet</code>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>CreateXssMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>XssMatchSet:  An <a>XssMatchSet</a>.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>XssMatchTuples</i> Specifies the parts of web requests that you want to inspect for cross-site scripting attacks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name, if any, of the <code>XssMatchSet</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>XssMatchSetId</i> A unique identifier for an <code>XssMatchSet</code>. You use <code>XssMatchSetId</code> to get information about an <code>XssMatchSet</code> (see <a>GetXssMatchSet</a>), update an <code>XssMatchSet</code> (see <a>UpdateXssMatchSet</a>), insert an <code>XssMatchSet</code> into a <code>Rule</code> or delete one from a <code>Rule</code> (see <a>UpdateRule</a>), and delete an <code>XssMatchSet</code> from AWS WAF (see <a>DeleteXssMatchSet</a>).  <code>XssMatchSetId</code> is returned by <a>CreateXssMatchSet</a> and by <a>ListXssMatchSets</a>.<br/>        """
        pass

    def delete_byte_match_set(self, ChangeToken: str, ByteMatchSetId: str) -> DeleteByteMatchSetResponse:
        """Permanently deletes a <a>ByteMatchSet</a>. You can't delete a <code>ByteMatchSet</code> if it's still used in any <code>Rules</code> or if it still includes any <a>ByteMatchTuple</a> objects (any filters). If you just want to remove a <code>ByteMatchSet</code> from a <code>Rule</code>, use <a>UpdateRule</a>. To permanently delete a <code>ByteMatchSet</code>, perform the following steps: <ol> <li> Update the <code>ByteMatchSet</code> to remove filters, if any. For more information, see <a>UpdateByteMatchSet</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>DeleteByteMatchSet</code> request. </li> <li> Submit a <code>DeleteByteMatchSet</code> request. </li> </ol>

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>ByteMatchSetId:  The <code>ByteMatchSetId</code> of the <a>ByteMatchSet</a> that you want to delete. <code>ByteMatchSetId</code> is returned by <a>CreateByteMatchSet</a> and by <a>ListByteMatchSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>DeleteByteMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def delete_geo_match_set(self, ChangeToken: str, GeoMatchSetId: str) -> DeleteGeoMatchSetResponse:
        """Permanently deletes a <a>GeoMatchSet</a>. You can't delete a <code>GeoMatchSet</code> if it's still used in any <code>Rules</code> or if it still includes any countries. If you just want to remove a <code>GeoMatchSet</code> from a <code>Rule</code>, use <a>UpdateRule</a>. To permanently delete a <code>GeoMatchSet</code> from AWS WAF, perform the following steps: <ol> <li> Update the <code>GeoMatchSet</code> to remove any countries. For more information, see <a>UpdateGeoMatchSet</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>DeleteGeoMatchSet</code> request. </li> <li> Submit a <code>DeleteGeoMatchSet</code> request. </li> </ol>

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>GeoMatchSetId:  The <code>GeoMatchSetID</code> of the <a>GeoMatchSet</a> that you want to delete. <code>GeoMatchSetId</code> is returned by <a>CreateGeoMatchSet</a> and by <a>ListGeoMatchSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>DeleteGeoMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def delete_ip_set(self, ChangeToken: str, IPSetId: str) -> DeleteIPSetResponse:
        """Permanently deletes an <a>IPSet</a>. You can't delete an <code>IPSet</code> if it's still used in any <code>Rules</code> or if it still includes any IP addresses. If you just want to remove an <code>IPSet</code> from a <code>Rule</code>, use <a>UpdateRule</a>. To permanently delete an <code>IPSet</code> from AWS WAF, perform the following steps: <ol> <li> Update the <code>IPSet</code> to remove IP address ranges, if any. For more information, see <a>UpdateIPSet</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>DeleteIPSet</code> request. </li> <li> Submit a <code>DeleteIPSet</code> request. </li> </ol>

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>IPSetId:  The <code>IPSetId</code> of the <a>IPSet</a> that you want to delete. <code>IPSetId</code> is returned by <a>CreateIPSet</a> and by <a>ListIPSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>DeleteIPSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def delete_logging_configuration(self, ResourceArn: str) -> DeleteLoggingConfigurationResponse:
        """Permanently deletes the <a>LoggingConfiguration</a> from the specified web ACL.

        :param self:
        :param ResourceArn:  The Amazon Resource Name (ARN) of the web ACL from which you want to delete the <a>LoggingConfiguration</a>.<br/>
        :return:         """
        pass

    def delete_permission_policy(self, ResourceArn: str) -> DeletePermissionPolicyResponse:
        """Permanently deletes an IAM policy from the specified RuleGroup. The user making the request must be the owner of the RuleGroup.

        :param self:
        :param ResourceArn:  The Amazon Resource Name (ARN) of the RuleGroup from which you want to delete the policy. The user making the request must be the owner of the RuleGroup.<br/>
        :return:         """
        pass

    def delete_rate_based_rule(self, ChangeToken: str, RuleId: str) -> DeleteRateBasedRuleResponse:
        """Permanently deletes a <a>RateBasedRule</a>. You can't delete a rule if it's still used in any <code>WebACL</code> objects or if it still includes any predicates, such as <code>ByteMatchSet</code> objects. If you just want to remove a rule from a <code>WebACL</code>, use <a>UpdateWebACL</a>. To permanently delete a <code>RateBasedRule</code> from AWS WAF, perform the following steps: <ol> <li> Update the <code>RateBasedRule</code> to remove predicates, if any. For more information, see <a>UpdateRateBasedRule</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>DeleteRateBasedRule</code> request. </li> <li> Submit a <code>DeleteRateBasedRule</code> request. </li> </ol>

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>RuleId:  The <code>RuleId</code> of the <a>RateBasedRule</a> that you want to delete. <code>RuleId</code> is returned by <a>CreateRateBasedRule</a> and by <a>ListRateBasedRules</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>DeleteRateBasedRule</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def delete_regex_match_set(self, ChangeToken: str, RegexMatchSetId: str) -> DeleteRegexMatchSetResponse:
        """Permanently deletes a <a>RegexMatchSet</a>. You can't delete a <code>RegexMatchSet</code> if it's still used in any <code>Rules</code> or if it still includes any <code>RegexMatchTuples</code> objects (any filters). If you just want to remove a <code>RegexMatchSet</code> from a <code>Rule</code>, use <a>UpdateRule</a>. To permanently delete a <code>RegexMatchSet</code>, perform the following steps: <ol> <li> Update the <code>RegexMatchSet</code> to remove filters, if any. For more information, see <a>UpdateRegexMatchSet</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>DeleteRegexMatchSet</code> request. </li> <li> Submit a <code>DeleteRegexMatchSet</code> request. </li> </ol>

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>RegexMatchSetId:  The <code>RegexMatchSetId</code> of the <a>RegexMatchSet</a> that you want to delete. <code>RegexMatchSetId</code> is returned by <a>CreateRegexMatchSet</a> and by <a>ListRegexMatchSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>DeleteRegexMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def delete_regex_pattern_set(self, ChangeToken: str, RegexPatternSetId: str) -> DeleteRegexPatternSetResponse:
        """Permanently deletes a <a>RegexPatternSet</a>. You can't delete a <code>RegexPatternSet</code> if it's still used in any <code>RegexMatchSet</code> or if the <code>RegexPatternSet</code> is not empty. 

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>RegexPatternSetId:  The <code>RegexPatternSetId</code> of the <a>RegexPatternSet</a> that you want to delete. <code>RegexPatternSetId</code> is returned by <a>CreateRegexPatternSet</a> and by <a>ListRegexPatternSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>DeleteRegexPatternSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def delete_rule(self, ChangeToken: str, RuleId: str) -> DeleteRuleResponse:
        """Permanently deletes a <a>Rule</a>. You can't delete a <code>Rule</code> if it's still used in any <code>WebACL</code> objects or if it still includes any predicates, such as <code>ByteMatchSet</code> objects. If you just want to remove a <code>Rule</code> from a <code>WebACL</code>, use <a>UpdateWebACL</a>. To permanently delete a <code>Rule</code> from AWS WAF, perform the following steps: <ol> <li> Update the <code>Rule</code> to remove predicates, if any. For more information, see <a>UpdateRule</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>DeleteRule</code> request. </li> <li> Submit a <code>DeleteRule</code> request. </li> </ol>

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>RuleId:  The <code>RuleId</code> of the <a>Rule</a> that you want to delete. <code>RuleId</code> is returned by <a>CreateRule</a> and by <a>ListRules</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>DeleteRule</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def delete_rule_group(self, ChangeToken: str, RuleGroupId: str) -> DeleteRuleGroupResponse:
        """Permanently deletes a <a>RuleGroup</a>. You can't delete a <code>RuleGroup</code> if it's still used in any <code>WebACL</code> objects or if it still includes any rules. If you just want to remove a <code>RuleGroup</code> from a <code>WebACL</code>, use <a>UpdateWebACL</a>. To permanently delete a <code>RuleGroup</code> from AWS WAF, perform the following steps: <ol> <li> Update the <code>RuleGroup</code> to remove rules, if any. For more information, see <a>UpdateRuleGroup</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>DeleteRuleGroup</code> request. </li> <li> Submit a <code>DeleteRuleGroup</code> request. </li> </ol>

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>RuleGroupId:  The <code>RuleGroupId</code> of the <a>RuleGroup</a> that you want to delete. <code>RuleGroupId</code> is returned by <a>CreateRuleGroup</a> and by <a>ListRuleGroups</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>DeleteRuleGroup</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def delete_size_constraint_set(self, ChangeToken: str, SizeConstraintSetId: str) -> DeleteSizeConstraintSetResponse:
        """Permanently deletes a <a>SizeConstraintSet</a>. You can't delete a <code>SizeConstraintSet</code> if it's still used in any <code>Rules</code> or if it still includes any <a>SizeConstraint</a> objects (any filters). If you just want to remove a <code>SizeConstraintSet</code> from a <code>Rule</code>, use <a>UpdateRule</a>. To permanently delete a <code>SizeConstraintSet</code>, perform the following steps: <ol> <li> Update the <code>SizeConstraintSet</code> to remove filters, if any. For more information, see <a>UpdateSizeConstraintSet</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>DeleteSizeConstraintSet</code> request. </li> <li> Submit a <code>DeleteSizeConstraintSet</code> request. </li> </ol>

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>SizeConstraintSetId:  The <code>SizeConstraintSetId</code> of the <a>SizeConstraintSet</a> that you want to delete. <code>SizeConstraintSetId</code> is returned by <a>CreateSizeConstraintSet</a> and by <a>ListSizeConstraintSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>DeleteSizeConstraintSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def delete_sql_injection_match_set(self, ChangeToken: str, SqlInjectionMatchSetId: str) -> DeleteSqlInjectionMatchSetResponse:
        """Permanently deletes a <a>SqlInjectionMatchSet</a>. You can't delete a <code>SqlInjectionMatchSet</code> if it's still used in any <code>Rules</code> or if it still contains any <a>SqlInjectionMatchTuple</a> objects. If you just want to remove a <code>SqlInjectionMatchSet</code> from a <code>Rule</code>, use <a>UpdateRule</a>. To permanently delete a <code>SqlInjectionMatchSet</code> from AWS WAF, perform the following steps: <ol> <li> Update the <code>SqlInjectionMatchSet</code> to remove filters, if any. For more information, see <a>UpdateSqlInjectionMatchSet</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>DeleteSqlInjectionMatchSet</code> request. </li> <li> Submit a <code>DeleteSqlInjectionMatchSet</code> request. </li> </ol>

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>SqlInjectionMatchSetId:  The <code>SqlInjectionMatchSetId</code> of the <a>SqlInjectionMatchSet</a> that you want to delete. <code>SqlInjectionMatchSetId</code> is returned by <a>CreateSqlInjectionMatchSet</a> and by <a>ListSqlInjectionMatchSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>DeleteSqlInjectionMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def delete_web_acl(self, ChangeToken: str, WebACLId: str) -> DeleteWebACLResponse:
        """Permanently deletes a <a>WebACL</a>. You can't delete a <code>WebACL</code> if it still contains any <code>Rules</code>. To delete a <code>WebACL</code>, perform the following steps: <ol> <li> Update the <code>WebACL</code> to remove <code>Rules</code>, if any. For more information, see <a>UpdateWebACL</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>DeleteWebACL</code> request. </li> <li> Submit a <code>DeleteWebACL</code> request. </li> </ol>

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>WebACLId:  The <code>WebACLId</code> of the <a>WebACL</a> that you want to delete. <code>WebACLId</code> is returned by <a>CreateWebACL</a> and by <a>ListWebACLs</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>DeleteWebACL</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def delete_xss_match_set(self, ChangeToken: str, XssMatchSetId: str) -> DeleteXssMatchSetResponse:
        """Permanently deletes an <a>XssMatchSet</a>. You can't delete an <code>XssMatchSet</code> if it's still used in any <code>Rules</code> or if it still contains any <a>XssMatchTuple</a> objects. If you just want to remove an <code>XssMatchSet</code> from a <code>Rule</code>, use <a>UpdateRule</a>. To permanently delete an <code>XssMatchSet</code> from AWS WAF, perform the following steps: <ol> <li> Update the <code>XssMatchSet</code> to remove filters, if any. For more information, see <a>UpdateXssMatchSet</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of a <code>DeleteXssMatchSet</code> request. </li> <li> Submit a <code>DeleteXssMatchSet</code> request. </li> </ol>

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>XssMatchSetId:  The <code>XssMatchSetId</code> of the <a>XssMatchSet</a> that you want to delete. <code>XssMatchSetId</code> is returned by <a>CreateXssMatchSet</a> and by <a>ListXssMatchSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>DeleteXssMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def disassociate_web_acl(self, ResourceArn: str) -> DisassociateWebACLResponse:
        """Removes a web ACL from the specified resource, either an application load balancer or Amazon API Gateway stage.

        :param self:
        :param ResourceArn:  The ARN (Amazon Resource Name) of the resource from which the web ACL is being removed, either an application load balancer or Amazon API Gateway stage. The ARN should be in one of the following formats: <ul> <li> For an Application Load Balancer: <code>arn:aws:elasticloadbalancing:<i>region</i>:<i>account-id</i>:loadbalancer/app/<i>load-balancer-name</i>/<i>load-balancer-id</i> </code>  </li> <li> For an Amazon API Gateway stage: <code>arn:aws:apigateway:<i>region</i>::/restapis/<i>api-id</i>/stages/<i>stage-name</i> </code>  </li> </ul><br/>
        :return:         """
        pass

    def get_byte_match_set(self, ByteMatchSetId: str) -> GetByteMatchSetResponse:
        """Returns the <a>ByteMatchSet</a> specified by <code>ByteMatchSetId</code>.

        :param self:
        :param ByteMatchSetId:  The <code>ByteMatchSetId</code> of the <a>ByteMatchSet</a> that you want to get. <code>ByteMatchSetId</code> is returned by <a>CreateByteMatchSet</a> and by <a>ListByteMatchSets</a>.<br/>
        :return: ByteMatchSet:  Information about the <a>ByteMatchSet</a> that you specified in the <code>GetByteMatchSet</code> request. For more information, see the following topics: <ul> <li>  <a>ByteMatchSet</a>: Contains <code>ByteMatchSetId</code>, <code>ByteMatchTuples</code>, and <code>Name</code>  </li> <li>  <code>ByteMatchTuples</code>: Contains an array of <a>ByteMatchTuple</a> objects. Each <code>ByteMatchTuple</code> object contains <a>FieldToMatch</a>, <code>PositionalConstraint</code>, <code>TargetString</code>, and <code>TextTransformation</code>  </li> <li>  <a>FieldToMatch</a>: Contains <code>Data</code> and <code>Type</code>  </li> </ul>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ByteMatchTuples</i> Specifies the bytes (typically a string that corresponds with ASCII characters) that you want AWS WAF to search for in web requests, the location in requests that you want AWS WAF to search, and other settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A friendly name or description of the <a>ByteMatchSet</a>. You can't change <code>Name</code> after you create a <code>ByteMatchSet</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ByteMatchSetId</i> The <code>ByteMatchSetId</code> for a <code>ByteMatchSet</code>. You use <code>ByteMatchSetId</code> to get information about a <code>ByteMatchSet</code> (see <a>GetByteMatchSet</a>), update a <code>ByteMatchSet</code> (see <a>UpdateByteMatchSet</a>), insert a <code>ByteMatchSet</code> into a <code>Rule</code> or delete one from a <code>Rule</code> (see <a>UpdateRule</a>), and delete a <code>ByteMatchSet</code> from AWS WAF (see <a>DeleteByteMatchSet</a>).  <code>ByteMatchSetId</code> is returned by <a>CreateByteMatchSet</a> and by <a>ListByteMatchSets</a>.<br/>        """
        pass

    def get_change_token(self) -> GetChangeTokenResponse:
        """When you want to create, update, or delete AWS WAF objects, get a change token and include the change token in the create, update, or delete request. Change tokens ensure that your application doesn't submit conflicting requests to AWS WAF. Each create, update, or delete request must use a unique change token. If your application submits a <code>GetChangeToken</code> request and then submits a second <code>GetChangeToken</code> request before submitting a create, update, or delete request, the second <code>GetChangeToken</code> request returns the same value as the first <code>GetChangeToken</code> request. When you use a change token in a create, update, or delete request, the status of the change token changes to <code>PENDING</code>, which indicates that AWS WAF is propagating the change to all AWS WAF servers. Use <code>GetChangeTokenStatus</code> to determine the status of your change token.

        :param self:
        :param 
        :return: ChangeToken:  The <code>ChangeToken</code> that you used in the request. Use this value in a <code>GetChangeTokenStatus</code> request to get the current status of the request. <br/>        """
        pass

    def get_change_token_status(self, ChangeToken: str) -> GetChangeTokenStatusResponse:
        """Returns the status of a <code>ChangeToken</code> that you got by calling <a>GetChangeToken</a>. <code>ChangeTokenStatus</code> is one of the following values: <ul> <li>  <code>PROVISIONED</code>: You requested the change token by calling <code>GetChangeToken</code>, but you haven't used it yet in a call to create, update, or delete an AWS WAF object. </li> <li>  <code>PENDING</code>: AWS WAF is propagating the create, update, or delete request to all AWS WAF servers. </li> <li>  <code>INSYNC</code>: Propagation is complete. </li> </ul>

        :param self:
        :param ChangeToken:  The change token for which you want to get the status. This change token was previously returned in the <code>GetChangeToken</code> response.<br/>
        :return: ChangeTokenStatus:  The status of the change token.<br/>        """
        pass

    def get_geo_match_set(self, GeoMatchSetId: str) -> GetGeoMatchSetResponse:
        """Returns the <a>GeoMatchSet</a> that is specified by <code>GeoMatchSetId</code>.

        :param self:
        :param GeoMatchSetId:  The <code>GeoMatchSetId</code> of the <a>GeoMatchSet</a> that you want to get. <code>GeoMatchSetId</code> is returned by <a>CreateGeoMatchSet</a> and by <a>ListGeoMatchSets</a>.<br/>
        :return: GeoMatchSet:  Information about the <a>GeoMatchSet</a> that you specified in the <code>GetGeoMatchSet</code> request. This includes the <code>Type</code>, which for a <code>GeoMatchContraint</code> is always <code>Country</code>, as well as the <code>Value</code>, which is the identifier for a specific country.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GeoMatchConstraints</i> An array of <a>GeoMatchConstraint</a> objects, which contain the country that you want AWS WAF to search for.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A friendly name or description of the <a>GeoMatchSet</a>. You can't change the name of an <code>GeoMatchSet</code> after you create it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GeoMatchSetId</i> The <code>GeoMatchSetId</code> for an <code>GeoMatchSet</code>. You use <code>GeoMatchSetId</code> to get information about a <code>GeoMatchSet</code> (see <a>GeoMatchSet</a>), update a <code>GeoMatchSet</code> (see <a>UpdateGeoMatchSet</a>), insert a <code>GeoMatchSet</code> into a <code>Rule</code> or delete one from a <code>Rule</code> (see <a>UpdateRule</a>), and delete a <code>GeoMatchSet</code> from AWS WAF (see <a>DeleteGeoMatchSet</a>).  <code>GeoMatchSetId</code> is returned by <a>CreateGeoMatchSet</a> and by <a>ListGeoMatchSets</a>.<br/>        """
        pass

    def get_ip_set(self, IPSetId: str) -> GetIPSetResponse:
        """Returns the <a>IPSet</a> that is specified by <code>IPSetId</code>.

        :param self:
        :param IPSetId:  The <code>IPSetId</code> of the <a>IPSet</a> that you want to get. <code>IPSetId</code> is returned by <a>CreateIPSet</a> and by <a>ListIPSets</a>.<br/>
        :return: IPSet:  Information about the <a>IPSet</a> that you specified in the <code>GetIPSet</code> request. For more information, see the following topics: <ul> <li>  <a>IPSet</a>: Contains <code>IPSetDescriptors</code>, <code>IPSetId</code>, and <code>Name</code>  </li> <li>  <code>IPSetDescriptors</code>: Contains an array of <a>IPSetDescriptor</a> objects. Each <code>IPSetDescriptor</code> object contains <code>Type</code> and <code>Value</code>  </li> </ul>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IPSetDescriptors</i> The IP address type (<code>IPV4</code> or <code>IPV6</code>) and the IP address range (in CIDR notation) that web requests originate from. If the <code>WebACL</code> is associated with a CloudFront distribution and the viewer did not use an HTTP proxy or a load balancer to send the request, this is the value of the c-ip field in the CloudFront access logs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A friendly name or description of the <a>IPSet</a>. You can't change the name of an <code>IPSet</code> after you create it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>IPSetId</i> The <code>IPSetId</code> for an <code>IPSet</code>. You use <code>IPSetId</code> to get information about an <code>IPSet</code> (see <a>GetIPSet</a>), update an <code>IPSet</code> (see <a>UpdateIPSet</a>), insert an <code>IPSet</code> into a <code>Rule</code> or delete one from a <code>Rule</code> (see <a>UpdateRule</a>), and delete an <code>IPSet</code> from AWS WAF (see <a>DeleteIPSet</a>).  <code>IPSetId</code> is returned by <a>CreateIPSet</a> and by <a>ListIPSets</a>.<br/>        """
        pass

    def get_logging_configuration(self, ResourceArn: str) -> GetLoggingConfigurationResponse:
        """Returns the <a>LoggingConfiguration</a> for the specified web ACL.

        :param self:
        :param ResourceArn:  The Amazon Resource Name (ARN) of the web ACL for which you want to get the <a>LoggingConfiguration</a>.<br/>
        :return: LoggingConfiguration:  The <a>LoggingConfiguration</a> for the specified web ACL.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RedactedFields</i> The parts of the request that you want redacted from the logs. For example, if you redact the cookie field, the cookie field in the firehose will be <code>xxx</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogDestinationConfigs</i> An array of Amazon Kinesis Data Firehose ARNs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceArn</i> The Amazon Resource Name (ARN) of the web ACL that you want to associate with <code>LogDestinationConfigs</code>.<br/>        """
        pass

    def get_permission_policy(self, ResourceArn: str) -> GetPermissionPolicyResponse:
        """Returns the IAM policy attached to the RuleGroup.

        :param self:
        :param ResourceArn:  The Amazon Resource Name (ARN) of the RuleGroup for which you want to get the policy.<br/>
        :return: Policy:  The IAM policy attached to the specified RuleGroup.<br/>        """
        pass

    def get_rate_based_rule(self, RuleId: str) -> GetRateBasedRuleResponse:
        """Returns the <a>RateBasedRule</a> that is specified by the <code>RuleId</code> that you included in the <code>GetRateBasedRule</code> request.

        :param self:
        :param RuleId:  The <code>RuleId</code> of the <a>RateBasedRule</a> that you want to get. <code>RuleId</code> is returned by <a>CreateRateBasedRule</a> and by <a>ListRateBasedRules</a>.<br/>
        :return: Rule:  Information about the <a>RateBasedRule</a> that you specified in the <code>GetRateBasedRule</code> request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RateLimit</i> The maximum number of requests, which have an identical value in the field specified by the <code>RateKey</code>, allowed in a five-minute period. If the number of requests exceeds the <code>RateLimit</code> and the other predicates specified in the rule are also met, AWS WAF triggers the action that is specified for this rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RateKey</i> The field that AWS WAF uses to determine if requests are likely arriving from single source and thus subject to rate monitoring. The only valid value for <code>RateKey</code> is <code>IP</code>. <code>IP</code> indicates that requests arriving from the same IP address are subject to the <code>RateLimit</code> that is specified in the <code>RateBasedRule</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MatchPredicates</i> The <code>Predicates</code> object contains one <code>Predicate</code> element for each <a>ByteMatchSet</a>, <a>IPSet</a>, or <a>SqlInjectionMatchSet</a> object that you want to include in a <code>RateBasedRule</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricName</i> A friendly name or description for the metrics for a <code>RateBasedRule</code>. The name can contain only alphanumeric characters (A-Z, a-z, 0-9), with maximum length 128 and minimum length one. It can't contain whitespace or metric names reserved for AWS WAF, including "All" and "Default_Action." You can't change the name of the metric after you create the <code>RateBasedRule</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A friendly name or description for a <code>RateBasedRule</code>. You can't change the name of a <code>RateBasedRule</code> after you create it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleId</i> A unique identifier for a <code>RateBasedRule</code>. You use <code>RuleId</code> to get more information about a <code>RateBasedRule</code> (see <a>GetRateBasedRule</a>), update a <code>RateBasedRule</code> (see <a>UpdateRateBasedRule</a>), insert a <code>RateBasedRule</code> into a <code>WebACL</code> or delete one from a <code>WebACL</code> (see <a>UpdateWebACL</a>), or delete a <code>RateBasedRule</code> from AWS WAF (see <a>DeleteRateBasedRule</a>).<br/>        """
        pass

    def get_rate_based_rule_managed_keys(self, RuleId: str, NextMarker: str=None) -> GetRateBasedRuleManagedKeysResponse:
        """Returns an array of IP addresses currently being blocked by the <a>RateBasedRule</a> that is specified by the <code>RuleId</code>. The maximum number of managed keys that will be blocked is 10,000. If more than 10,000 addresses exceed the rate limit, the 10,000 addresses with the highest rates will be blocked.

        :param self:
        :param NextMarker:  A null value and not currently used. Do not include this in your request.<br/>RuleId:  The <code>RuleId</code> of the <a>RateBasedRule</a> for which you want to get a list of <code>ManagedKeys</code>. <code>RuleId</code> is returned by <a>CreateRateBasedRule</a> and by <a>ListRateBasedRules</a>.<br/>
        :return: NextMarker:  A null value and not currently used.<br/>ManagedKeys:  An array of IP addresses that currently are blocked by the specified <a>RateBasedRule</a>. <br/>        """
        pass

    def get_regex_match_set(self, RegexMatchSetId: str) -> GetRegexMatchSetResponse:
        """Returns the <a>RegexMatchSet</a> specified by <code>RegexMatchSetId</code>.

        :param self:
        :param RegexMatchSetId:  The <code>RegexMatchSetId</code> of the <a>RegexMatchSet</a> that you want to get. <code>RegexMatchSetId</code> is returned by <a>CreateRegexMatchSet</a> and by <a>ListRegexMatchSets</a>.<br/>
        :return: RegexMatchSet:  Information about the <a>RegexMatchSet</a> that you specified in the <code>GetRegexMatchSet</code> request. For more information, see <a>RegexMatchTuple</a>.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RegexMatchTuples</i> Contains an array of <a>RegexMatchTuple</a> objects. Each <code>RegexMatchTuple</code> object contains:  <ul> <li> The part of a web request that you want AWS WAF to inspect, such as a query string or the value of the <code>User-Agent</code> header.  </li> <li> The identifier of the pattern (a regular expression) that you want AWS WAF to look for. For more information, see <a>RegexPatternSet</a>. </li> <li> Whether to perform any conversions on the request, such as converting it to lowercase, before inspecting it for the specified string. </li> </ul><br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A friendly name or description of the <a>RegexMatchSet</a>. You can't change <code>Name</code> after you create a <code>RegexMatchSet</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RegexMatchSetId</i> The <code>RegexMatchSetId</code> for a <code>RegexMatchSet</code>. You use <code>RegexMatchSetId</code> to get information about a <code>RegexMatchSet</code> (see <a>GetRegexMatchSet</a>), update a <code>RegexMatchSet</code> (see <a>UpdateRegexMatchSet</a>), insert a <code>RegexMatchSet</code> into a <code>Rule</code> or delete one from a <code>Rule</code> (see <a>UpdateRule</a>), and delete a <code>RegexMatchSet</code> from AWS WAF (see <a>DeleteRegexMatchSet</a>).  <code>RegexMatchSetId</code> is returned by <a>CreateRegexMatchSet</a> and by <a>ListRegexMatchSets</a>.<br/>        """
        pass

    def get_regex_pattern_set(self, RegexPatternSetId: str) -> GetRegexPatternSetResponse:
        """Returns the <a>RegexPatternSet</a> specified by <code>RegexPatternSetId</code>.

        :param self:
        :param RegexPatternSetId:  The <code>RegexPatternSetId</code> of the <a>RegexPatternSet</a> that you want to get. <code>RegexPatternSetId</code> is returned by <a>CreateRegexPatternSet</a> and by <a>ListRegexPatternSets</a>.<br/>
        :return: RegexPatternSet:  Information about the <a>RegexPatternSet</a> that you specified in the <code>GetRegexPatternSet</code> request, including the identifier of the pattern set and the regular expression patterns you want AWS WAF to search for. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RegexPatternStrings</i> Specifies the regular expression (regex) patterns that you want AWS WAF to search for, such as <code>B[a@]dB[o0]t</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A friendly name or description of the <a>RegexPatternSet</a>. You can't change <code>Name</code> after you create a <code>RegexPatternSet</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RegexPatternSetId</i> The identifier for the <code>RegexPatternSet</code>. You use <code>RegexPatternSetId</code> to get information about a <code>RegexPatternSet</code>, update a <code>RegexPatternSet</code>, remove a <code>RegexPatternSet</code> from a <code>RegexMatchSet</code>, and delete a <code>RegexPatternSet</code> from AWS WAF.  <code>RegexMatchSetId</code> is returned by <a>CreateRegexPatternSet</a> and by <a>ListRegexPatternSets</a>.<br/>        """
        pass

    def get_rule(self, RuleId: str) -> GetRuleResponse:
        """Returns the <a>Rule</a> that is specified by the <code>RuleId</code> that you included in the <code>GetRule</code> request.

        :param self:
        :param RuleId:  The <code>RuleId</code> of the <a>Rule</a> that you want to get. <code>RuleId</code> is returned by <a>CreateRule</a> and by <a>ListRules</a>.<br/>
        :return: Rule:  Information about the <a>Rule</a> that you specified in the <code>GetRule</code> request. For more information, see the following topics: <ul> <li>  <a>Rule</a>: Contains <code>MetricName</code>, <code>Name</code>, an array of <code>Predicate</code> objects, and <code>RuleId</code>  </li> <li>  <a>Predicate</a>: Each <code>Predicate</code> object contains <code>DataId</code>, <code>Negated</code>, and <code>Type</code>  </li> </ul>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Predicates</i> The <code>Predicates</code> object contains one <code>Predicate</code> element for each <a>ByteMatchSet</a>, <a>IPSet</a>, or <a>SqlInjectionMatchSet</a> object that you want to include in a <code>Rule</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricName</i> A friendly name or description for the metrics for this <code>Rule</code>. The name can contain only alphanumeric characters (A-Z, a-z, 0-9), with maximum length 128 and minimum length one. It can't contain whitespace or metric names reserved for AWS WAF, including "All" and "Default_Action." You can't change <code>MetricName</code> after you create the <code>Rule</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The friendly name or description for the <code>Rule</code>. You can't change the name of a <code>Rule</code> after you create it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleId</i> A unique identifier for a <code>Rule</code>. You use <code>RuleId</code> to get more information about a <code>Rule</code> (see <a>GetRule</a>), update a <code>Rule</code> (see <a>UpdateRule</a>), insert a <code>Rule</code> into a <code>WebACL</code> or delete a one from a <code>WebACL</code> (see <a>UpdateWebACL</a>), or delete a <code>Rule</code> from AWS WAF (see <a>DeleteRule</a>).  <code>RuleId</code> is returned by <a>CreateRule</a> and by <a>ListRules</a>.<br/>        """
        pass

    def get_rule_group(self, RuleGroupId: str) -> GetRuleGroupResponse:
        """Returns the <a>RuleGroup</a> that is specified by the <code>RuleGroupId</code> that you included in the <code>GetRuleGroup</code> request. To view the rules in a rule group, use <a>ListActivatedRulesInRuleGroup</a>.

        :param self:
        :param RuleGroupId:  The <code>RuleGroupId</code> of the <a>RuleGroup</a> that you want to get. <code>RuleGroupId</code> is returned by <a>CreateRuleGroup</a> and by <a>ListRuleGroups</a>.<br/>
        :return: RuleGroup:  Information about the <a>RuleGroup</a> that you specified in the <code>GetRuleGroup</code> request. :<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricName</i> A friendly name or description for the metrics for this <code>RuleGroup</code>. The name can contain only alphanumeric characters (A-Z, a-z, 0-9), with maximum length 128 and minimum length one. It can't contain whitespace or metric names reserved for AWS WAF, including "All" and "Default_Action." You can't change the name of the metric after you create the <code>RuleGroup</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The friendly name or description for the <code>RuleGroup</code>. You can't change the name of a <code>RuleGroup</code> after you create it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleGroupId</i> A unique identifier for a <code>RuleGroup</code>. You use <code>RuleGroupId</code> to get more information about a <code>RuleGroup</code> (see <a>GetRuleGroup</a>), update a <code>RuleGroup</code> (see <a>UpdateRuleGroup</a>), insert a <code>RuleGroup</code> into a <code>WebACL</code> or delete a one from a <code>WebACL</code> (see <a>UpdateWebACL</a>), or delete a <code>RuleGroup</code> from AWS WAF (see <a>DeleteRuleGroup</a>).  <code>RuleGroupId</code> is returned by <a>CreateRuleGroup</a> and by <a>ListRuleGroups</a>.<br/>        """
        pass

    def get_sampled_requests(self, MaxItems: GetSampledRequestsMaxItems, TimeWindow: TimeWindow, RuleId: str, WebAclId: str) -> GetSampledRequestsResponse:
        """Gets detailed information about a specified number of requests--a sample--that AWS WAF randomly selects from among the first 5,000 requests that your AWS resource received during a time range that you choose. You can specify a sample size of up to 500 requests, and you can specify any time range in the previous three hours.  <code>GetSampledRequests</code> returns a time range, which is usually the time range that you specified. However, if your resource (such as a CloudFront distribution) received 5,000 requests before the specified time range elapsed, <code>GetSampledRequests</code> returns an updated time range. This new time range indicates the actual period during which AWS WAF selected the requests in the sample.

        :param self:
        :param MaxItems:  The number of requests that you want AWS WAF to return from among the first 5,000 requests that your AWS resource received during the time range. If your resource received fewer requests than the value of <code>MaxItems</code>, <code>GetSampledRequests</code> returns information about all of them. <br/>TimeWindow:  The start date and time and the end date and time of the range for which you want <code>GetSampledRequests</code> to return a sample of requests. Specify the date and time in the following format: <code>"2016-09-27T14:50Z"</code>. You can specify any time range in the previous three hours.:<br/>RuleId:   <code>RuleId</code> is one of three values: <ul> <li> The <code>RuleId</code> of the <code>Rule</code> or the <code>RuleGroupId</code> of the <code>RuleGroup</code> for which you want <code>GetSampledRequests</code> to return a sample of requests. </li> <li>  <code>Default_Action</code>, which causes <code>GetSampledRequests</code> to return a sample of the requests that didn't match any of the rules in the specified <code>WebACL</code>. </li> </ul><br/>WebAclId:  The <code>WebACLId</code> of the <code>WebACL</code> for which you want <code>GetSampledRequests</code> to return a sample of requests.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> The end of the time range from which you want <code>GetSampledRequests</code> to return a sample of the requests that your AWS resource received. Specify the date and time in the following format: <code>"2016-09-27T14:50Z"</code>. You can specify any time range in the previous three hours.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTime</i> The beginning of the time range from which you want <code>GetSampledRequests</code> to return a sample of the requests that your AWS resource received. Specify the date and time in the following format: <code>"2016-09-27T14:50Z"</code>. You can specify any time range in the previous three hours.<br/>
        :return: TimeWindow:  Usually, <code>TimeWindow</code> is the time range that you specified in the <code>GetSampledRequests</code> request. However, if your AWS resource received more than 5,000 requests during the time range that you specified in the request, <code>GetSampledRequests</code> returns the time range for the first 5,000 requests.:<br/>PopulationSize:  The total number of requests from which <code>GetSampledRequests</code> got a sample of <code>MaxItems</code> requests. If <code>PopulationSize</code> is less than <code>MaxItems</code>, the sample includes every request that your AWS resource received during the specified time range.<br/>SampledRequests:  A complex type that contains detailed information about each of the requests in the sample.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndTime</i> The end of the time range from which you want <code>GetSampledRequests</code> to return a sample of the requests that your AWS resource received. Specify the date and time in the following format: <code>"2016-09-27T14:50Z"</code>. You can specify any time range in the previous three hours.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StartTime</i> The beginning of the time range from which you want <code>GetSampledRequests</code> to return a sample of the requests that your AWS resource received. Specify the date and time in the following format: <code>"2016-09-27T14:50Z"</code>. You can specify any time range in the previous three hours.<br/>        """
        pass

    def get_size_constraint_set(self, SizeConstraintSetId: str) -> GetSizeConstraintSetResponse:
        """Returns the <a>SizeConstraintSet</a> specified by <code>SizeConstraintSetId</code>.

        :param self:
        :param SizeConstraintSetId:  The <code>SizeConstraintSetId</code> of the <a>SizeConstraintSet</a> that you want to get. <code>SizeConstraintSetId</code> is returned by <a>CreateSizeConstraintSet</a> and by <a>ListSizeConstraintSets</a>.<br/>
        :return: SizeConstraintSet:  Information about the <a>SizeConstraintSet</a> that you specified in the <code>GetSizeConstraintSet</code> request. For more information, see the following topics: <ul> <li>  <a>SizeConstraintSet</a>: Contains <code>SizeConstraintSetId</code>, <code>SizeConstraints</code>, and <code>Name</code>  </li> <li>  <code>SizeConstraints</code>: Contains an array of <a>SizeConstraint</a> objects. Each <code>SizeConstraint</code> object contains <a>FieldToMatch</a>, <code>TextTransformation</code>, <code>ComparisonOperator</code>, and <code>Size</code>  </li> <li>  <a>FieldToMatch</a>: Contains <code>Data</code> and <code>Type</code>  </li> </ul>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SizeConstraints</i> Specifies the parts of web requests that you want to inspect the size of.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name, if any, of the <code>SizeConstraintSet</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SizeConstraintSetId</i> A unique identifier for a <code>SizeConstraintSet</code>. You use <code>SizeConstraintSetId</code> to get information about a <code>SizeConstraintSet</code> (see <a>GetSizeConstraintSet</a>), update a <code>SizeConstraintSet</code> (see <a>UpdateSizeConstraintSet</a>), insert a <code>SizeConstraintSet</code> into a <code>Rule</code> or delete one from a <code>Rule</code> (see <a>UpdateRule</a>), and delete a <code>SizeConstraintSet</code> from AWS WAF (see <a>DeleteSizeConstraintSet</a>).  <code>SizeConstraintSetId</code> is returned by <a>CreateSizeConstraintSet</a> and by <a>ListSizeConstraintSets</a>.<br/>        """
        pass

    def get_sql_injection_match_set(self, SqlInjectionMatchSetId: str) -> GetSqlInjectionMatchSetResponse:
        """Returns the <a>SqlInjectionMatchSet</a> that is specified by <code>SqlInjectionMatchSetId</code>.

        :param self:
        :param SqlInjectionMatchSetId:  The <code>SqlInjectionMatchSetId</code> of the <a>SqlInjectionMatchSet</a> that you want to get. <code>SqlInjectionMatchSetId</code> is returned by <a>CreateSqlInjectionMatchSet</a> and by <a>ListSqlInjectionMatchSets</a>.<br/>
        :return: SqlInjectionMatchSet:  Information about the <a>SqlInjectionMatchSet</a> that you specified in the <code>GetSqlInjectionMatchSet</code> request. For more information, see the following topics: <ul> <li>  <a>SqlInjectionMatchSet</a>: Contains <code>Name</code>, <code>SqlInjectionMatchSetId</code>, and an array of <code>SqlInjectionMatchTuple</code> objects </li> <li>  <a>SqlInjectionMatchTuple</a>: Each <code>SqlInjectionMatchTuple</code> object contains <code>FieldToMatch</code> and <code>TextTransformation</code>  </li> <li>  <a>FieldToMatch</a>: Contains <code>Data</code> and <code>Type</code>  </li> </ul>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SqlInjectionMatchTuples</i> Specifies the parts of web requests that you want to inspect for snippets of malicious SQL code.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name, if any, of the <code>SqlInjectionMatchSet</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SqlInjectionMatchSetId</i> A unique identifier for a <code>SqlInjectionMatchSet</code>. You use <code>SqlInjectionMatchSetId</code> to get information about a <code>SqlInjectionMatchSet</code> (see <a>GetSqlInjectionMatchSet</a>), update a <code>SqlInjectionMatchSet</code> (see <a>UpdateSqlInjectionMatchSet</a>), insert a <code>SqlInjectionMatchSet</code> into a <code>Rule</code> or delete one from a <code>Rule</code> (see <a>UpdateRule</a>), and delete a <code>SqlInjectionMatchSet</code> from AWS WAF (see <a>DeleteSqlInjectionMatchSet</a>).  <code>SqlInjectionMatchSetId</code> is returned by <a>CreateSqlInjectionMatchSet</a> and by <a>ListSqlInjectionMatchSets</a>.<br/>        """
        pass

    def get_web_acl(self, WebACLId: str) -> GetWebACLResponse:
        """Returns the <a>WebACL</a> that is specified by <code>WebACLId</code>.

        :param self:
        :param WebACLId:  The <code>WebACLId</code> of the <a>WebACL</a> that you want to get. <code>WebACLId</code> is returned by <a>CreateWebACL</a> and by <a>ListWebACLs</a>.<br/>
        :return: WebACL:  Information about the <a>WebACL</a> that you specified in the <code>GetWebACL</code> request. For more information, see the following topics: <ul> <li>  <a>WebACL</a>: Contains <code>DefaultAction</code>, <code>MetricName</code>, <code>Name</code>, an array of <code>Rule</code> objects, and <code>WebACLId</code>  </li> <li>  <code>DefaultAction</code> (Data type is <a>WafAction</a>): Contains <code>Type</code>  </li> <li>  <code>Rules</code>: Contains an array of <code>ActivatedRule</code> objects, which contain <code>Action</code>, <code>Priority</code>, and <code>RuleId</code>  </li> <li>  <code>Action</code>: Contains <code>Type</code>  </li> </ul>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WebACLArn</i> Tha Amazon Resource Name (ARN) of the web ACL.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Rules</i> An array that contains the action for each <code>Rule</code> in a <code>WebACL</code>, the priority of the <code>Rule</code>, and the ID of the <code>Rule</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>DefaultAction</i> The action to perform if none of the <code>Rules</code> contained in the <code>WebACL</code> match. The action is specified by the <a>WafAction</a> object.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>DefaultAction</b>: The action to perform if none of the <code>Rules</code> contained in the <code>WebACL</code> match. The action is specified by the <a>WafAction</a> object.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>MetricName</i> A friendly name or description for the metrics for this <code>WebACL</code>. The name can contain only alphanumeric characters (A-Z, a-z, 0-9), with maximum length 128 and minimum length one. It can't contain whitespace or metric names reserved for AWS WAF, including "All" and "Default_Action." You can't change <code>MetricName</code> after you create the <code>WebACL</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A friendly name or description of the <code>WebACL</code>. You can't change the name of a <code>WebACL</code> after you create it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WebACLId</i> A unique identifier for a <code>WebACL</code>. You use <code>WebACLId</code> to get information about a <code>WebACL</code> (see <a>GetWebACL</a>), update a <code>WebACL</code> (see <a>UpdateWebACL</a>), and delete a <code>WebACL</code> from AWS WAF (see <a>DeleteWebACL</a>).  <code>WebACLId</code> is returned by <a>CreateWebACL</a> and by <a>ListWebACLs</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> Specifies how you want AWS WAF to respond to requests that match the settings in a <code>Rule</code>. Valid settings include the following: <ul> <li>  <code>ALLOW</code>: AWS WAF allows requests </li> <li>  <code>BLOCK</code>: AWS WAF blocks requests </li> <li>  <code>COUNT</code>: AWS WAF increments a counter of the requests that match all of the conditions in the rule. AWS WAF then continues to inspect the web request based on the remaining rules in the web ACL. You can't specify <code>COUNT</code> for the default action for a <code>WebACL</code>. </li> </ul><br/>        """
        pass

    def get_web_acl_for_resource(self, ResourceArn: str) -> GetWebACLForResourceResponse:
        """Returns the web ACL for the specified resource, either an application load balancer or Amazon API Gateway stage.

        :param self:
        :param ResourceArn:  The ARN (Amazon Resource Name) of the resource for which to get the web ACL, either an application load balancer or Amazon API Gateway stage. The ARN should be in one of the following formats: <ul> <li> For an Application Load Balancer: <code>arn:aws:elasticloadbalancing:<i>region</i>:<i>account-id</i>:loadbalancer/app/<i>load-balancer-name</i>/<i>load-balancer-id</i> </code>  </li> <li> For an Amazon API Gateway stage: <code>arn:aws:apigateway:<i>region</i>::/restapis/<i>api-id</i>/stages/<i>stage-name</i> </code>  </li> </ul><br/>
        :return: WebACLSummary:  Information about the web ACL that you specified in the <code>GetWebACLForResource</code> request. If there is no associated resource, a null WebACLSummary is returned.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> A friendly name or description of the <a>WebACL</a>. You can't change the name of a <code>WebACL</code> after you create it.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>WebACLId</i> A unique identifier for a <code>WebACL</code>. You use <code>WebACLId</code> to get information about a <code>WebACL</code> (see <a>GetWebACL</a>), update a <code>WebACL</code> (see <a>UpdateWebACL</a>), and delete a <code>WebACL</code> from AWS WAF (see <a>DeleteWebACL</a>).  <code>WebACLId</code> is returned by <a>CreateWebACL</a> and by <a>ListWebACLs</a>.<br/>        """
        pass

    def get_xss_match_set(self, XssMatchSetId: str) -> GetXssMatchSetResponse:
        """Returns the <a>XssMatchSet</a> that is specified by <code>XssMatchSetId</code>.

        :param self:
        :param XssMatchSetId:  The <code>XssMatchSetId</code> of the <a>XssMatchSet</a> that you want to get. <code>XssMatchSetId</code> is returned by <a>CreateXssMatchSet</a> and by <a>ListXssMatchSets</a>.<br/>
        :return: XssMatchSet:  Information about the <a>XssMatchSet</a> that you specified in the <code>GetXssMatchSet</code> request. For more information, see the following topics: <ul> <li>  <a>XssMatchSet</a>: Contains <code>Name</code>, <code>XssMatchSetId</code>, and an array of <code>XssMatchTuple</code> objects </li> <li>  <a>XssMatchTuple</a>: Each <code>XssMatchTuple</code> object contains <code>FieldToMatch</code> and <code>TextTransformation</code>  </li> <li>  <a>FieldToMatch</a>: Contains <code>Data</code> and <code>Type</code>  </li> </ul>:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>XssMatchTuples</i> Specifies the parts of web requests that you want to inspect for cross-site scripting attacks.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name, if any, of the <code>XssMatchSet</code>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>XssMatchSetId</i> A unique identifier for an <code>XssMatchSet</code>. You use <code>XssMatchSetId</code> to get information about an <code>XssMatchSet</code> (see <a>GetXssMatchSet</a>), update an <code>XssMatchSet</code> (see <a>UpdateXssMatchSet</a>), insert an <code>XssMatchSet</code> into a <code>Rule</code> or delete one from a <code>Rule</code> (see <a>UpdateRule</a>), and delete an <code>XssMatchSet</code> from AWS WAF (see <a>DeleteXssMatchSet</a>).  <code>XssMatchSetId</code> is returned by <a>CreateXssMatchSet</a> and by <a>ListXssMatchSets</a>.<br/>        """
        pass

    def list_activated_rules_in_rule_group(self, RuleGroupId: str=None, NextMarker: str=None, Limit: int=None) -> ListActivatedRulesInRuleGroupResponse:
        """Returns an array of <a>ActivatedRule</a> objects.

        :param self:
        :param Limit:  Specifies the number of <code>ActivatedRules</code> that you want AWS WAF to return for this request. If you have more <code>ActivatedRules</code> than the number that you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of <code>ActivatedRules</code>.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <code>ActivatedRules</code> than the value of <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of <code>ActivatedRules</code>. For the second and subsequent <code>ListActivatedRulesInRuleGroup</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of <code>ActivatedRules</code>.<br/>RuleGroupId:  The <code>RuleGroupId</code> of the <a>RuleGroup</a> for which you want to get a list of <a>ActivatedRule</a> objects.<br/>
        :return: ActivatedRules:  An array of <code>ActivatedRules</code> objects.<br/>NextMarker:  If you have more <code>ActivatedRules</code> than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more <code>ActivatedRules</code>, submit another <code>ListActivatedRulesInRuleGroup</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>        """
        pass

    def list_byte_match_sets(self, NextMarker: str=None, Limit: int=None) -> ListByteMatchSetsResponse:
        """Returns an array of <a>ByteMatchSetSummary</a> objects.

        :param self:
        :param Limit:  Specifies the number of <code>ByteMatchSet</code> objects that you want AWS WAF to return for this request. If you have more <code>ByteMatchSets</code> objects than the number you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of <code>ByteMatchSet</code> objects.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <code>ByteMatchSets</code> than the value of <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of <code>ByteMatchSets</code>. For the second and subsequent <code>ListByteMatchSets</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of <code>ByteMatchSets</code>.<br/>
        :return: ByteMatchSets:  An array of <a>ByteMatchSetSummary</a> objects.<br/>NextMarker:  If you have more <code>ByteMatchSet</code> objects than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more <code>ByteMatchSet</code> objects, submit another <code>ListByteMatchSets</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>        """
        pass

    def list_geo_match_sets(self, NextMarker: str=None, Limit: int=None) -> ListGeoMatchSetsResponse:
        """Returns an array of <a>GeoMatchSetSummary</a> objects in the response.

        :param self:
        :param Limit:  Specifies the number of <code>GeoMatchSet</code> objects that you want AWS WAF to return for this request. If you have more <code>GeoMatchSet</code> objects than the number you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of <code>GeoMatchSet</code> objects.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <code>GeoMatchSet</code>s than the value of <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of <code>GeoMatchSet</code> objects. For the second and subsequent <code>ListGeoMatchSets</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of <code>GeoMatchSet</code> objects.<br/>
        :return: GeoMatchSets:  An array of <a>GeoMatchSetSummary</a> objects.<br/>NextMarker:  If you have more <code>GeoMatchSet</code> objects than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more <code>GeoMatchSet</code> objects, submit another <code>ListGeoMatchSets</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>        """
        pass

    def list_ip_sets(self, NextMarker: str=None, Limit: int=None) -> ListIPSetsResponse:
        """Returns an array of <a>IPSetSummary</a> objects in the response.

        :param self:
        :param Limit:  Specifies the number of <code>IPSet</code> objects that you want AWS WAF to return for this request. If you have more <code>IPSet</code> objects than the number you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of <code>IPSet</code> objects.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <code>IPSets</code> than the value of <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of <code>IPSets</code>. For the second and subsequent <code>ListIPSets</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of <code>IPSets</code>.<br/>
        :return: IPSets:  An array of <a>IPSetSummary</a> objects.<br/>NextMarker:  If you have more <code>IPSet</code> objects than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more <code>IPSet</code> objects, submit another <code>ListIPSets</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>        """
        pass

    def list_logging_configurations(self, NextMarker: str=None, Limit: int=None) -> ListLoggingConfigurationsResponse:
        """Returns an array of <a>LoggingConfiguration</a> objects.

        :param self:
        :param Limit:  Specifies the number of <code>LoggingConfigurations</code> that you want AWS WAF to return for this request. If you have more <code>LoggingConfigurations</code> than the number that you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of <code>LoggingConfigurations</code>.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <code>LoggingConfigurations</code> than the value of <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of <code>LoggingConfigurations</code>. For the second and subsequent <code>ListLoggingConfigurations</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of <code>ListLoggingConfigurations</code>.<br/>
        :return: NextMarker:  If you have more <code>LoggingConfigurations</code> than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more <code>LoggingConfigurations</code>, submit another <code>ListLoggingConfigurations</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>LoggingConfigurations:  An array of <a>LoggingConfiguration</a> objects.<br/>        """
        pass

    def list_rate_based_rules(self, NextMarker: str=None, Limit: int=None) -> ListRateBasedRulesResponse:
        """Returns an array of <a>RuleSummary</a> objects.

        :param self:
        :param Limit:  Specifies the number of <code>Rules</code> that you want AWS WAF to return for this request. If you have more <code>Rules</code> than the number that you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of <code>Rules</code>.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <code>Rules</code> than the value of <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of <code>Rules</code>. For the second and subsequent <code>ListRateBasedRules</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of <code>Rules</code>.<br/>
        :return: Rules:  An array of <a>RuleSummary</a> objects.<br/>NextMarker:  If you have more <code>Rules</code> than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more <code>Rules</code>, submit another <code>ListRateBasedRules</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>        """
        pass

    def list_regex_match_sets(self, NextMarker: str=None, Limit: int=None) -> ListRegexMatchSetsResponse:
        """Returns an array of <a>RegexMatchSetSummary</a> objects.

        :param self:
        :param Limit:  Specifies the number of <code>RegexMatchSet</code> objects that you want AWS WAF to return for this request. If you have more <code>RegexMatchSet</code> objects than the number you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of <code>RegexMatchSet</code> objects.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <code>RegexMatchSet</code> objects than the value of <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of <code>ByteMatchSets</code>. For the second and subsequent <code>ListRegexMatchSets</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of <code>RegexMatchSet</code> objects.<br/>
        :return: RegexMatchSets:  An array of <a>RegexMatchSetSummary</a> objects.<br/>NextMarker:  If you have more <code>RegexMatchSet</code> objects than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more <code>RegexMatchSet</code> objects, submit another <code>ListRegexMatchSets</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>        """
        pass

    def list_regex_pattern_sets(self, NextMarker: str=None, Limit: int=None) -> ListRegexPatternSetsResponse:
        """Returns an array of <a>RegexPatternSetSummary</a> objects.

        :param self:
        :param Limit:  Specifies the number of <code>RegexPatternSet</code> objects that you want AWS WAF to return for this request. If you have more <code>RegexPatternSet</code> objects than the number you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of <code>RegexPatternSet</code> objects.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <code>RegexPatternSet</code> objects than the value of <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of <code>RegexPatternSet</code> objects. For the second and subsequent <code>ListRegexPatternSets</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of <code>RegexPatternSet</code> objects.<br/>
        :return: RegexPatternSets:  An array of <a>RegexPatternSetSummary</a> objects.<br/>NextMarker:  If you have more <code>RegexPatternSet</code> objects than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more <code>RegexPatternSet</code> objects, submit another <code>ListRegexPatternSets</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>        """
        pass

    def list_resources_for_web_acl(self, WebACLId: str, ResourceType: str=None) -> ListResourcesForWebACLResponse:
        """Returns an array of resources associated with the specified web ACL.

        :param self:
        :param ResourceType:  The type of resource to list, either an application load balancer or Amazon API Gateway.<br/>WebACLId:  The unique identifier (ID) of the web ACL for which to list the associated resources.<br/>
        :return: ResourceArns:  An array of ARNs (Amazon Resource Names) of the resources associated with the specified web ACL. An array with zero elements is returned if there are no resources associated with the web ACL.<br/>        """
        pass

    def list_rule_groups(self, NextMarker: str=None, Limit: int=None) -> ListRuleGroupsResponse:
        """Returns an array of <a>RuleGroup</a> objects.

        :param self:
        :param Limit:  Specifies the number of <code>RuleGroups</code> that you want AWS WAF to return for this request. If you have more <code>RuleGroups</code> than the number that you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of <code>RuleGroups</code>.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <code>RuleGroups</code> than the value of <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of <code>RuleGroups</code>. For the second and subsequent <code>ListRuleGroups</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of <code>RuleGroups</code>.<br/>
        :return: RuleGroups:  An array of <a>RuleGroup</a> objects.<br/>NextMarker:  If you have more <code>RuleGroups</code> than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more <code>RuleGroups</code>, submit another <code>ListRuleGroups</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>        """
        pass

    def list_rules(self, NextMarker: str=None, Limit: int=None) -> ListRulesResponse:
        """Returns an array of <a>RuleSummary</a> objects.

        :param self:
        :param Limit:  Specifies the number of <code>Rules</code> that you want AWS WAF to return for this request. If you have more <code>Rules</code> than the number that you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of <code>Rules</code>.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <code>Rules</code> than the value of <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of <code>Rules</code>. For the second and subsequent <code>ListRules</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of <code>Rules</code>.<br/>
        :return: Rules:  An array of <a>RuleSummary</a> objects.<br/>NextMarker:  If you have more <code>Rules</code> than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more <code>Rules</code>, submit another <code>ListRules</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>        """
        pass

    def list_size_constraint_sets(self, NextMarker: str=None, Limit: int=None) -> ListSizeConstraintSetsResponse:
        """Returns an array of <a>SizeConstraintSetSummary</a> objects.

        :param self:
        :param Limit:  Specifies the number of <code>SizeConstraintSet</code> objects that you want AWS WAF to return for this request. If you have more <code>SizeConstraintSets</code> objects than the number you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of <code>SizeConstraintSet</code> objects.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <code>SizeConstraintSets</code> than the value of <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of <code>SizeConstraintSets</code>. For the second and subsequent <code>ListSizeConstraintSets</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of <code>SizeConstraintSets</code>.<br/>
        :return: SizeConstraintSets:  An array of <a>SizeConstraintSetSummary</a> objects.<br/>NextMarker:  If you have more <code>SizeConstraintSet</code> objects than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more <code>SizeConstraintSet</code> objects, submit another <code>ListSizeConstraintSets</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>        """
        pass

    def list_sql_injection_match_sets(self, NextMarker: str=None, Limit: int=None) -> ListSqlInjectionMatchSetsResponse:
        """Returns an array of <a>SqlInjectionMatchSet</a> objects.

        :param self:
        :param Limit:  Specifies the number of <a>SqlInjectionMatchSet</a> objects that you want AWS WAF to return for this request. If you have more <code>SqlInjectionMatchSet</code> objects than the number you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of <code>Rules</code>.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <a>SqlInjectionMatchSet</a> objects than the value of <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of <code>SqlInjectionMatchSets</code>. For the second and subsequent <code>ListSqlInjectionMatchSets</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of <code>SqlInjectionMatchSets</code>.<br/>
        :return: SqlInjectionMatchSets:  An array of <a>SqlInjectionMatchSetSummary</a> objects.<br/>NextMarker:  If you have more <a>SqlInjectionMatchSet</a> objects than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more <code>SqlInjectionMatchSet</code> objects, submit another <code>ListSqlInjectionMatchSets</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>        """
        pass

    def list_subscribed_rule_groups(self, NextMarker: str=None, Limit: int=None) -> ListSubscribedRuleGroupsResponse:
        """Returns an array of <a>RuleGroup</a> objects that you are subscribed to.

        :param self:
        :param Limit:  Specifies the number of subscribed rule groups that you want AWS WAF to return for this request. If you have more objects than the number you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of objects.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <code>ByteMatchSets</code>subscribed rule groups than the value of <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of subscribed rule groups. For the second and subsequent <code>ListSubscribedRuleGroupsRequest</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of subscribed rule groups.<br/>
        :return: RuleGroups:  An array of <a>RuleGroup</a> objects.<br/>NextMarker:  If you have more objects than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more objects, submit another <code>ListSubscribedRuleGroups</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>        """
        pass

    def list_tags_for_resource(self, ResourceARN: str, NextMarker: str=None, Limit: int=None) -> ListTagsForResourceResponse:
        """

        :param self:
        :param ResourceARN:  <br/>Limit:  <br/>NextMarker:  <br/>
        :return: TagInfoForResource:  :<br/>NextMarker:  <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TagList</i> <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceARN</i> <br/>        """
        pass

    def list_web_acls(self, NextMarker: str=None, Limit: int=None) -> ListWebACLsResponse:
        """Returns an array of <a>WebACLSummary</a> objects in the response.

        :param self:
        :param Limit:  Specifies the number of <code>WebACL</code> objects that you want AWS WAF to return for this request. If you have more <code>WebACL</code> objects than the number that you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of <code>WebACL</code> objects.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <code>WebACL</code> objects than the number that you specify for <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of <code>WebACL</code> objects. For the second and subsequent <code>ListWebACLs</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of <code>WebACL</code> objects.<br/>
        :return: WebACLs:  An array of <a>WebACLSummary</a> objects.<br/>NextMarker:  If you have more <code>WebACL</code> objects than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more <code>WebACL</code> objects, submit another <code>ListWebACLs</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>        """
        pass

    def list_xss_match_sets(self, NextMarker: str=None, Limit: int=None) -> ListXssMatchSetsResponse:
        """Returns an array of <a>XssMatchSet</a> objects.

        :param self:
        :param Limit:  Specifies the number of <a>XssMatchSet</a> objects that you want AWS WAF to return for this request. If you have more <code>XssMatchSet</code> objects than the number you specify for <code>Limit</code>, the response includes a <code>NextMarker</code> value that you can use to get another batch of <code>Rules</code>.<br/>NextMarker:  If you specify a value for <code>Limit</code> and you have more <a>XssMatchSet</a> objects than the value of <code>Limit</code>, AWS WAF returns a <code>NextMarker</code> value in the response that allows you to list another group of <code>XssMatchSets</code>. For the second and subsequent <code>ListXssMatchSets</code> requests, specify the value of <code>NextMarker</code> from the previous response to get information about another batch of <code>XssMatchSets</code>.<br/>
        :return: XssMatchSets:  An array of <a>XssMatchSetSummary</a> objects.<br/>NextMarker:  If you have more <a>XssMatchSet</a> objects than the number that you specified for <code>Limit</code> in the request, the response includes a <code>NextMarker</code> value. To list more <code>XssMatchSet</code> objects, submit another <code>ListXssMatchSets</code> request, and specify the <code>NextMarker</code> value from the response in the <code>NextMarker</code> value in the next request.<br/>        """
        pass

    def put_logging_configuration(self, LoggingConfiguration: LoggingConfiguration) -> PutLoggingConfigurationResponse:
        """Associates a <a>LoggingConfiguration</a> with a specified web ACL. You can access information about all traffic that AWS WAF inspects using the following steps: <ol> <li> Create an Amazon Kinesis Data Firehose .  Create the data firehose with a PUT source and in the region that you are operating. However, if you are capturing logs for Amazon CloudFront, always create the firehose in US East (N. Virginia).  <note> Do not create the data firehose using a <code>Kinesis stream</code> as your source. </note> </li> <li> Associate that firehose to your web ACL using a <code>PutLoggingConfiguration</code> request. </li> </ol> When you successfully enable logging using a <code>PutLoggingConfiguration</code> request, AWS WAF will create a service linked role with the necessary permissions to write logs to the Amazon Kinesis Data Firehose. For more information, see <a href="https://docs.aws.amazon.com/waf/latest/developerguide/logging.html">Logging Web ACL Traffic Information</a> in the <i>AWS WAF Developer Guide</i>.

        :param self:
        :param LoggingConfiguration:  The Amazon Kinesis Data Firehose that contains the inspected traffic information, the redacted fields details, and the Amazon Resource Name (ARN) of the web ACL to monitor.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RedactedFields</i> The parts of the request that you want redacted from the logs. For example, if you redact the cookie field, the cookie field in the firehose will be <code>xxx</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogDestinationConfigs</i> An array of Amazon Kinesis Data Firehose ARNs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceArn</i> The Amazon Resource Name (ARN) of the web ACL that you want to associate with <code>LogDestinationConfigs</code>.<br/>
        :return: LoggingConfiguration:  The <a>LoggingConfiguration</a> that you submitted in the request.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RedactedFields</i> The parts of the request that you want redacted from the logs. For example, if you redact the cookie field, the cookie field in the firehose will be <code>xxx</code>. <br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LogDestinationConfigs</i> An array of Amazon Kinesis Data Firehose ARNs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceArn</i> The Amazon Resource Name (ARN) of the web ACL that you want to associate with <code>LogDestinationConfigs</code>.<br/>        """
        pass

    def put_permission_policy(self, Policy: str, ResourceArn: str) -> PutPermissionPolicyResponse:
        """Attaches a IAM policy to the specified resource. The only supported use for this action is to share a RuleGroup across accounts. The <code>PutPermissionPolicy</code> is subject to the following restrictions: <ul> <li> You can attach only one policy with each <code>PutPermissionPolicy</code> request. </li> <li> The policy must include an <code>Effect</code>, <code>Action</code> and <code>Principal</code>.  </li> <li>  <code>Effect</code> must specify <code>Allow</code>. </li> <li> The <code>Action</code> in the policy must be <code>waf:UpdateWebACL</code>, <code>waf-regional:UpdateWebACL</code>, <code>waf:GetRuleGroup</code> and <code>waf-regional:GetRuleGroup</code> . Any extra or wildcard actions in the policy will be rejected. </li> <li> The policy cannot include a <code>Resource</code> parameter. </li> <li> The ARN in the request must be a valid WAF RuleGroup ARN and the RuleGroup must exist in the same region. </li> <li> The user making the request must be the owner of the RuleGroup. </li> <li> Your policy must be composed using IAM Policy version 2012-10-17. </li> </ul> For more information, see <a href="https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies.html">IAM Policies</a>.  An example of a valid policy parameter is shown in the Examples section below.

        :param self:
        :param Policy:  The policy to attach to the specified RuleGroup.<br/>ResourceArn:  The Amazon Resource Name (ARN) of the RuleGroup to which you want to attach the policy.<br/>
        :return:         """
        pass

    def tag_resource(self, Tags: List[Tag], ResourceARN: str) -> TagResourceResponse:
        """

        :param self:
        :param Tags:  <br/>ResourceARN:  <br/>
        :return:         """
        pass

    def untag_resource(self, TagKeys: List[TagKey], ResourceARN: str) -> UntagResourceResponse:
        """

        :param self:
        :param TagKeys:  <br/>ResourceARN:  <br/>
        :return:         """
        pass

    def update_byte_match_set(self, Updates: List[ByteMatchSetUpdate], ChangeToken: str, ByteMatchSetId: str) -> UpdateByteMatchSetResponse:
        """Inserts or deletes <a>ByteMatchTuple</a> objects (filters) in a <a>ByteMatchSet</a>. For each <code>ByteMatchTuple</code> object, you specify the following values:  <ul> <li> Whether to insert or delete the object from the array. If you want to change a <code>ByteMatchSetUpdate</code> object, you delete the existing object and add a new one. </li> <li> The part of a web request that you want AWS WAF to inspect, such as a query string or the value of the <code>User-Agent</code> header.  </li> <li> The bytes (typically a string that corresponds with ASCII characters) that you want AWS WAF to look for. For more information, including how you specify the values for the AWS WAF API and the AWS CLI or SDKs, see <code>TargetString</code> in the <a>ByteMatchTuple</a> data type.  </li> <li> Where to look, such as at the beginning or the end of a query string. </li> <li> Whether to perform any conversions on the request, such as converting it to lowercase, before inspecting it for the specified string. </li> </ul> For example, you can add a <code>ByteMatchSetUpdate</code> object that matches web requests in which <code>User-Agent</code> headers contain the string <code>BadBot</code>. You can then configure AWS WAF to block those requests. To create and configure a <code>ByteMatchSet</code>, perform the following steps: <ol> <li> Create a <code>ByteMatchSet.</code> For more information, see <a>CreateByteMatchSet</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <code>UpdateByteMatchSet</code> request. </li> <li> Submit an <code>UpdateByteMatchSet</code> request to specify the part of the request that you want AWS WAF to inspect (for example, the header or the URI) and the value that you want AWS WAF to watch for. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param Updates:  An array of <code>ByteMatchSetUpdate</code> objects that you want to insert into or delete from a <a>ByteMatchSet</a>. For more information, see the applicable data types: <ul> <li>  <a>ByteMatchSetUpdate</a>: Contains <code>Action</code> and <code>ByteMatchTuple</code>  </li> <li>  <a>ByteMatchTuple</a>: Contains <code>FieldToMatch</code>, <code>PositionalConstraint</code>, <code>TargetString</code>, and <code>TextTransformation</code>  </li> <li>  <a>FieldToMatch</a>: Contains <code>Data</code> and <code>Type</code>  </li> </ul><br/>ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>ByteMatchSetId:  The <code>ByteMatchSetId</code> of the <a>ByteMatchSet</a> that you want to update. <code>ByteMatchSetId</code> is returned by <a>CreateByteMatchSet</a> and by <a>ListByteMatchSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>UpdateByteMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def update_geo_match_set(self, Updates: List[GeoMatchSetUpdate], ChangeToken: str, GeoMatchSetId: str) -> UpdateGeoMatchSetResponse:
        """Inserts or deletes <a>GeoMatchConstraint</a> objects in an <code>GeoMatchSet</code>. For each <code>GeoMatchConstraint</code> object, you specify the following values:  <ul> <li> Whether to insert or delete the object from the array. If you want to change an <code>GeoMatchConstraint</code> object, you delete the existing object and add a new one. </li> <li> The <code>Type</code>. The only valid value for <code>Type</code> is <code>Country</code>. </li> <li> The <code>Value</code>, which is a two character code for the country to add to the <code>GeoMatchConstraint</code> object. Valid codes are listed in <a>GeoMatchConstraint$Value</a>. </li> </ul> To create and configure an <code>GeoMatchSet</code>, perform the following steps: <ol> <li> Submit a <a>CreateGeoMatchSet</a> request. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <a>UpdateGeoMatchSet</a> request. </li> <li> Submit an <code>UpdateGeoMatchSet</code> request to specify the country that you want AWS WAF to watch for. </li> </ol> When you update an <code>GeoMatchSet</code>, you specify the country that you want to add and/or the country that you want to delete. If you want to change a country, you delete the existing country and add the new one. For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param Updates:  An array of <code>GeoMatchSetUpdate</code> objects that you want to insert into or delete from an <a>GeoMatchSet</a>. For more information, see the applicable data types: <ul> <li>  <a>GeoMatchSetUpdate</a>: Contains <code>Action</code> and <code>GeoMatchConstraint</code>  </li> <li>  <a>GeoMatchConstraint</a>: Contains <code>Type</code> and <code>Value</code>  You can have only one <code>Type</code> and <code>Value</code> per <code>GeoMatchConstraint</code>. To add multiple countries, include multiple <code>GeoMatchSetUpdate</code> objects in your request. </li> </ul><br/>ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>GeoMatchSetId:  The <code>GeoMatchSetId</code> of the <a>GeoMatchSet</a> that you want to update. <code>GeoMatchSetId</code> is returned by <a>CreateGeoMatchSet</a> and by <a>ListGeoMatchSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>UpdateGeoMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def update_ip_set(self, Updates: List[IPSetUpdate], ChangeToken: str, IPSetId: str) -> UpdateIPSetResponse:
        """Inserts or deletes <a>IPSetDescriptor</a> objects in an <code>IPSet</code>. For each <code>IPSetDescriptor</code> object, you specify the following values:  <ul> <li> Whether to insert or delete the object from the array. If you want to change an <code>IPSetDescriptor</code> object, you delete the existing object and add a new one. </li> <li> The IP address version, <code>IPv4</code> or <code>IPv6</code>.  </li> <li> The IP address in CIDR notation, for example, <code>192.0.2.0/24</code> (for the range of IP addresses from <code>192.0.2.0</code> to <code>192.0.2.255</code>) or <code>192.0.2.44/32</code> (for the individual IP address <code>192.0.2.44</code>).  </li> </ul> AWS WAF supports IPv4 address ranges: /8 and any range between /16 through /32. AWS WAF supports IPv6 address ranges: /24, /32, /48, /56, /64, and /128. For more information about CIDR notation, see the Wikipedia entry <a href="https://en.wikipedia.org/wiki/Classless_Inter-Domain_Routing">Classless Inter-Domain Routing</a>. IPv6 addresses can be represented using any of the following formats: <ul> <li> 1111:0000:0000:0000:0000:0000:0000:0111/128 </li> <li> 1111:0:0:0:0:0:0:0111/128 </li> <li> 1111::0111/128 </li> <li> 1111::111/128 </li> </ul> You use an <code>IPSet</code> to specify which web requests you want to allow or block based on the IP addresses that the requests originated from. For example, if you're receiving a lot of requests from one or a small number of IP addresses and you want to block the requests, you can create an <code>IPSet</code> that specifies those IP addresses, and then configure AWS WAF to block the requests.  To create and configure an <code>IPSet</code>, perform the following steps: <ol> <li> Submit a <a>CreateIPSet</a> request. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <a>UpdateIPSet</a> request. </li> <li> Submit an <code>UpdateIPSet</code> request to specify the IP addresses that you want AWS WAF to watch for. </li> </ol> When you update an <code>IPSet</code>, you specify the IP addresses that you want to add and/or the IP addresses that you want to delete. If you want to change an IP address, you delete the existing IP address and add the new one. You can insert a maximum of 1000 addresses in a single request. For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param Updates:  An array of <code>IPSetUpdate</code> objects that you want to insert into or delete from an <a>IPSet</a>. For more information, see the applicable data types: <ul> <li>  <a>IPSetUpdate</a>: Contains <code>Action</code> and <code>IPSetDescriptor</code>  </li> <li>  <a>IPSetDescriptor</a>: Contains <code>Type</code> and <code>Value</code>  </li> </ul> You can insert a maximum of 1000 addresses in a single request.<br/>ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>IPSetId:  The <code>IPSetId</code> of the <a>IPSet</a> that you want to update. <code>IPSetId</code> is returned by <a>CreateIPSet</a> and by <a>ListIPSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>UpdateIPSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def update_rate_based_rule(self, RateLimit: RateLimit, Updates: List[RuleUpdate], ChangeToken: str, RuleId: str) -> UpdateRateBasedRuleResponse:
        """Inserts or deletes <a>Predicate</a> objects in a rule and updates the <code>RateLimit</code> in the rule.  Each <code>Predicate</code> object identifies a predicate, such as a <a>ByteMatchSet</a> or an <a>IPSet</a>, that specifies the web requests that you want to block or count. The <code>RateLimit</code> specifies the number of requests every five minutes that triggers the rule. If you add more than one predicate to a <code>RateBasedRule</code>, a request must match all the predicates and exceed the <code>RateLimit</code> to be counted or blocked. For example, suppose you add the following to a <code>RateBasedRule</code>: <ul> <li> An <code>IPSet</code> that matches the IP address <code>192.0.2.44/32</code>  </li> <li> A <code>ByteMatchSet</code> that matches <code>BadBot</code> in the <code>User-Agent</code> header </li> </ul> Further, you specify a <code>RateLimit</code> of 15,000. You then add the <code>RateBasedRule</code> to a <code>WebACL</code> and specify that you want to block requests that satisfy the rule. For a request to be blocked, it must come from the IP address 192.0.2.44 <i>and</i> the <code>User-Agent</code> header in the request must contain the value <code>BadBot</code>. Further, requests that match these two conditions much be received at a rate of more than 15,000 every five minutes. If the rate drops below this limit, AWS WAF no longer blocks the requests. As a second example, suppose you want to limit requests to a particular page on your site. To do this, you could add the following to a <code>RateBasedRule</code>: <ul> <li> A <code>ByteMatchSet</code> with <code>FieldToMatch</code> of <code>URI</code>  </li> <li> A <code>PositionalConstraint</code> of <code>STARTS_WITH</code>  </li> <li> A <code>TargetString</code> of <code>login</code>  </li> </ul> Further, you specify a <code>RateLimit</code> of 15,000. By adding this <code>RateBasedRule</code> to a <code>WebACL</code>, you could limit requests to your login page without affecting the rest of your site.

        :param self:
        :param RateLimit:  The maximum number of requests, which have an identical value in the field specified by the <code>RateKey</code>, allowed in a five-minute period. If the number of requests exceeds the <code>RateLimit</code> and the other predicates specified in the rule are also met, AWS WAF triggers the action that is specified for this rule.<br/>Updates:  An array of <code>RuleUpdate</code> objects that you want to insert into or delete from a <a>RateBasedRule</a>. <br/>ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>RuleId:  The <code>RuleId</code> of the <code>RateBasedRule</code> that you want to update. <code>RuleId</code> is returned by <code>CreateRateBasedRule</code> and by <a>ListRateBasedRules</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>UpdateRateBasedRule</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def update_regex_match_set(self, ChangeToken: str, Updates: List[RegexMatchSetUpdate], RegexMatchSetId: str) -> UpdateRegexMatchSetResponse:
        """Inserts or deletes <a>RegexMatchTuple</a> objects (filters) in a <a>RegexMatchSet</a>. For each <code>RegexMatchSetUpdate</code> object, you specify the following values:  <ul> <li> Whether to insert or delete the object from the array. If you want to change a <code>RegexMatchSetUpdate</code> object, you delete the existing object and add a new one. </li> <li> The part of a web request that you want AWS WAF to inspectupdate, such as a query string or the value of the <code>User-Agent</code> header.  </li> <li> The identifier of the pattern (a regular expression) that you want AWS WAF to look for. For more information, see <a>RegexPatternSet</a>.  </li> <li> Whether to perform any conversions on the request, such as converting it to lowercase, before inspecting it for the specified string. </li> </ul>  For example, you can create a <code>RegexPatternSet</code> that matches any requests with <code>User-Agent</code> headers that contain the string <code>B[a@]dB[o0]t</code>. You can then configure AWS WAF to reject those requests. To create and configure a <code>RegexMatchSet</code>, perform the following steps: <ol> <li> Create a <code>RegexMatchSet.</code> For more information, see <a>CreateRegexMatchSet</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <code>UpdateRegexMatchSet</code> request. </li> <li> Submit an <code>UpdateRegexMatchSet</code> request to specify the part of the request that you want AWS WAF to inspect (for example, the header or the URI) and the identifier of the <code>RegexPatternSet</code> that contain the regular expression patters you want AWS WAF to watch for. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>Updates:  An array of <code>RegexMatchSetUpdate</code> objects that you want to insert into or delete from a <a>RegexMatchSet</a>. For more information, see <a>RegexMatchTuple</a>.<br/>RegexMatchSetId:  The <code>RegexMatchSetId</code> of the <a>RegexMatchSet</a> that you want to update. <code>RegexMatchSetId</code> is returned by <a>CreateRegexMatchSet</a> and by <a>ListRegexMatchSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>UpdateRegexMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def update_regex_pattern_set(self, ChangeToken: str, Updates: List[RegexPatternSetUpdate], RegexPatternSetId: str) -> UpdateRegexPatternSetResponse:
        """Inserts or deletes <code>RegexPatternString</code> objects in a <a>RegexPatternSet</a>. For each <code>RegexPatternString</code> object, you specify the following values:  <ul> <li> Whether to insert or delete the <code>RegexPatternString</code>. </li> <li> The regular expression pattern that you want to insert or delete. For more information, see <a>RegexPatternSet</a>.  </li> </ul>  For example, you can create a <code>RegexPatternString</code> such as <code>B[a@]dB[o0]t</code>. AWS WAF will match this <code>RegexPatternString</code> to: <ul> <li> BadBot </li> <li> BadB0t </li> <li> B@dBot </li> <li> B@dB0t </li> </ul> To create and configure a <code>RegexPatternSet</code>, perform the following steps: <ol> <li> Create a <code>RegexPatternSet.</code> For more information, see <a>CreateRegexPatternSet</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <code>UpdateRegexPatternSet</code> request. </li> <li> Submit an <code>UpdateRegexPatternSet</code> request to specify the regular expression pattern that you want AWS WAF to watch for. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>Updates:  An array of <code>RegexPatternSetUpdate</code> objects that you want to insert into or delete from a <a>RegexPatternSet</a>.<br/>RegexPatternSetId:  The <code>RegexPatternSetId</code> of the <a>RegexPatternSet</a> that you want to update. <code>RegexPatternSetId</code> is returned by <a>CreateRegexPatternSet</a> and by <a>ListRegexPatternSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>UpdateRegexPatternSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def update_rule(self, Updates: List[RuleUpdate], ChangeToken: str, RuleId: str) -> UpdateRuleResponse:
        """Inserts or deletes <a>Predicate</a> objects in a <code>Rule</code>. Each <code>Predicate</code> object identifies a predicate, such as a <a>ByteMatchSet</a> or an <a>IPSet</a>, that specifies the web requests that you want to allow, block, or count. If you add more than one predicate to a <code>Rule</code>, a request must match all of the specifications to be allowed, blocked, or counted. For example, suppose that you add the following to a <code>Rule</code>:  <ul> <li> A <code>ByteMatchSet</code> that matches the value <code>BadBot</code> in the <code>User-Agent</code> header </li> <li> An <code>IPSet</code> that matches the IP address <code>192.0.2.44</code>  </li> </ul> You then add the <code>Rule</code> to a <code>WebACL</code> and specify that you want to block requests that satisfy the <code>Rule</code>. For a request to be blocked, the <code>User-Agent</code> header in the request must contain the value <code>BadBot</code> <i>and</i> the request must originate from the IP address 192.0.2.44. To create and configure a <code>Rule</code>, perform the following steps: <ol> <li> Create and update the predicates that you want to include in the <code>Rule</code>. </li> <li> Create the <code>Rule</code>. See <a>CreateRule</a>. </li> <li> Use <code>GetChangeToken</code> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <a>UpdateRule</a> request. </li> <li> Submit an <code>UpdateRule</code> request to add predicates to the <code>Rule</code>. </li> <li> Create and update a <code>WebACL</code> that contains the <code>Rule</code>. See <a>CreateWebACL</a>. </li> </ol> If you want to replace one <code>ByteMatchSet</code> or <code>IPSet</code> with another, you delete the existing one and add the new one. For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param Updates:  An array of <code>RuleUpdate</code> objects that you want to insert into or delete from a <a>Rule</a>. For more information, see the applicable data types: <ul> <li>  <a>RuleUpdate</a>: Contains <code>Action</code> and <code>Predicate</code>  </li> <li>  <a>Predicate</a>: Contains <code>DataId</code>, <code>Negated</code>, and <code>Type</code>  </li> <li>  <a>FieldToMatch</a>: Contains <code>Data</code> and <code>Type</code>  </li> </ul><br/>ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>RuleId:  The <code>RuleId</code> of the <code>Rule</code> that you want to update. <code>RuleId</code> is returned by <code>CreateRule</code> and by <a>ListRules</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>UpdateRule</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def update_rule_group(self, ChangeToken: str, Updates: List[RuleGroupUpdate], RuleGroupId: str) -> UpdateRuleGroupResponse:
        """Inserts or deletes <a>ActivatedRule</a> objects in a <code>RuleGroup</code>. You can only insert <code>REGULAR</code> rules into a rule group. You can have a maximum of ten rules per rule group. To create and configure a <code>RuleGroup</code>, perform the following steps: <ol> <li> Create and update the <code>Rules</code> that you want to include in the <code>RuleGroup</code>. See <a>CreateRule</a>. </li> <li> Use <code>GetChangeToken</code> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <a>UpdateRuleGroup</a> request. </li> <li> Submit an <code>UpdateRuleGroup</code> request to add <code>Rules</code> to the <code>RuleGroup</code>. </li> <li> Create and update a <code>WebACL</code> that contains the <code>RuleGroup</code>. See <a>CreateWebACL</a>. </li> </ol> If you want to replace one <code>Rule</code> with another, you delete the existing one and add the new one. For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>Updates:  An array of <code>RuleGroupUpdate</code> objects that you want to insert into or delete from a <a>RuleGroup</a>. You can only insert <code>REGULAR</code> rules into a rule group.  <code>ActivatedRule|OverrideAction</code> applies only when updating or adding a <code>RuleGroup</code> to a <code>WebACL</code>. In this case you do not use <code>ActivatedRule|Action</code>. For all other update requests, <code>ActivatedRule|Action</code> is used instead of <code>ActivatedRule|OverrideAction</code>.<br/>RuleGroupId:  The <code>RuleGroupId</code> of the <a>RuleGroup</a> that you want to update. <code>RuleGroupId</code> is returned by <a>CreateRuleGroup</a> and by <a>ListRuleGroups</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>UpdateRuleGroup</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def update_size_constraint_set(self, Updates: List[SizeConstraintSetUpdate], ChangeToken: str, SizeConstraintSetId: str) -> UpdateSizeConstraintSetResponse:
        """Inserts or deletes <a>SizeConstraint</a> objects (filters) in a <a>SizeConstraintSet</a>. For each <code>SizeConstraint</code> object, you specify the following values:  <ul> <li> Whether to insert or delete the object from the array. If you want to change a <code>SizeConstraintSetUpdate</code> object, you delete the existing object and add a new one. </li> <li> The part of a web request that you want AWS WAF to evaluate, such as the length of a query string or the length of the <code>User-Agent</code> header. </li> <li> Whether to perform any transformations on the request, such as converting it to lowercase, before checking its length. Note that transformations of the request body are not supported because the AWS resource forwards only the first <code>8192</code> bytes of your request to AWS WAF. You can only specify a single type of TextTransformation. </li> <li> A <code>ComparisonOperator</code> used for evaluating the selected part of the request against the specified <code>Size</code>, such as equals, greater than, less than, and so on. </li> <li> The length, in bytes, that you want AWS WAF to watch for in selected part of the request. The length is computed after applying the transformation. </li> </ul> For example, you can add a <code>SizeConstraintSetUpdate</code> object that matches web requests in which the length of the <code>User-Agent</code> header is greater than 100 bytes. You can then configure AWS WAF to block those requests. To create and configure a <code>SizeConstraintSet</code>, perform the following steps: <ol> <li> Create a <code>SizeConstraintSet.</code> For more information, see <a>CreateSizeConstraintSet</a>. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <code>UpdateSizeConstraintSet</code> request. </li> <li> Submit an <code>UpdateSizeConstraintSet</code> request to specify the part of the request that you want AWS WAF to inspect (for example, the header or the URI) and the value that you want AWS WAF to watch for. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param Updates:  An array of <code>SizeConstraintSetUpdate</code> objects that you want to insert into or delete from a <a>SizeConstraintSet</a>. For more information, see the applicable data types: <ul> <li>  <a>SizeConstraintSetUpdate</a>: Contains <code>Action</code> and <code>SizeConstraint</code>  </li> <li>  <a>SizeConstraint</a>: Contains <code>FieldToMatch</code>, <code>TextTransformation</code>, <code>ComparisonOperator</code>, and <code>Size</code>  </li> <li>  <a>FieldToMatch</a>: Contains <code>Data</code> and <code>Type</code>  </li> </ul><br/>ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>SizeConstraintSetId:  The <code>SizeConstraintSetId</code> of the <a>SizeConstraintSet</a> that you want to update. <code>SizeConstraintSetId</code> is returned by <a>CreateSizeConstraintSet</a> and by <a>ListSizeConstraintSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>UpdateSizeConstraintSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def update_sql_injection_match_set(self, Updates: List[SqlInjectionMatchSetUpdate], ChangeToken: str, SqlInjectionMatchSetId: str) -> UpdateSqlInjectionMatchSetResponse:
        """Inserts or deletes <a>SqlInjectionMatchTuple</a> objects (filters) in a <a>SqlInjectionMatchSet</a>. For each <code>SqlInjectionMatchTuple</code> object, you specify the following values: <ul> <li>  <code>Action</code>: Whether to insert the object into or delete the object from the array. To change a <code>SqlInjectionMatchTuple</code>, you delete the existing object and add a new one. </li> <li>  <code>FieldToMatch</code>: The part of web requests that you want AWS WAF to inspect and, if you want AWS WAF to inspect a header or custom query parameter, the name of the header or parameter. </li> <li>  <code>TextTransformation</code>: Which text transformation, if any, to perform on the web request before inspecting the request for snippets of malicious SQL code. You can only specify a single type of TextTransformation. </li> </ul> You use <code>SqlInjectionMatchSet</code> objects to specify which CloudFront requests that you want to allow, block, or count. For example, if you're receiving requests that contain snippets of SQL code in the query string and you want to block the requests, you can create a <code>SqlInjectionMatchSet</code> with the applicable settings, and then configure AWS WAF to block the requests.  To create and configure a <code>SqlInjectionMatchSet</code>, perform the following steps: <ol> <li> Submit a <a>CreateSqlInjectionMatchSet</a> request. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <a>UpdateIPSet</a> request. </li> <li> Submit an <code>UpdateSqlInjectionMatchSet</code> request to specify the parts of web requests that you want AWS WAF to inspect for snippets of SQL code. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param Updates:  An array of <code>SqlInjectionMatchSetUpdate</code> objects that you want to insert into or delete from a <a>SqlInjectionMatchSet</a>. For more information, see the applicable data types: <ul> <li>  <a>SqlInjectionMatchSetUpdate</a>: Contains <code>Action</code> and <code>SqlInjectionMatchTuple</code>  </li> <li>  <a>SqlInjectionMatchTuple</a>: Contains <code>FieldToMatch</code> and <code>TextTransformation</code>  </li> <li>  <a>FieldToMatch</a>: Contains <code>Data</code> and <code>Type</code>  </li> </ul><br/>ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>SqlInjectionMatchSetId:  The <code>SqlInjectionMatchSetId</code> of the <code>SqlInjectionMatchSet</code> that you want to update. <code>SqlInjectionMatchSetId</code> is returned by <a>CreateSqlInjectionMatchSet</a> and by <a>ListSqlInjectionMatchSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>UpdateSqlInjectionMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def update_web_acl(self, ChangeToken: str, WebACLId: str, Updates: List[WebACLUpdate]=None, DefaultAction: WafAction=None) -> UpdateWebACLResponse:
        """Inserts or deletes <a>ActivatedRule</a> objects in a <code>WebACL</code>. Each <code>Rule</code> identifies web requests that you want to allow, block, or count. When you update a <code>WebACL</code>, you specify the following values: <ul> <li> A default action for the <code>WebACL</code>, either <code>ALLOW</code> or <code>BLOCK</code>. AWS WAF performs the default action if a request doesn't match the criteria in any of the <code>Rules</code> in a <code>WebACL</code>. </li> <li> The <code>Rules</code> that you want to add or delete. If you want to replace one <code>Rule</code> with another, you delete the existing <code>Rule</code> and add the new one. </li> <li> For each <code>Rule</code>, whether you want AWS WAF to allow requests, block requests, or count requests that match the conditions in the <code>Rule</code>. </li> <li> The order in which you want AWS WAF to evaluate the <code>Rules</code> in a <code>WebACL</code>. If you add more than one <code>Rule</code> to a <code>WebACL</code>, AWS WAF evaluates each request against the <code>Rules</code> in order based on the value of <code>Priority</code>. (The <code>Rule</code> that has the lowest value for <code>Priority</code> is evaluated first.) When a web request matches all the predicates (such as <code>ByteMatchSets</code> and <code>IPSets</code>) in a <code>Rule</code>, AWS WAF immediately takes the corresponding action, allow or block, and doesn't evaluate the request against the remaining <code>Rules</code> in the <code>WebACL</code>, if any.  </li> </ul> To create and configure a <code>WebACL</code>, perform the following steps: <ol> <li> Create and update the predicates that you want to include in <code>Rules</code>. For more information, see <a>CreateByteMatchSet</a>, <a>UpdateByteMatchSet</a>, <a>CreateIPSet</a>, <a>UpdateIPSet</a>, <a>CreateSqlInjectionMatchSet</a>, and <a>UpdateSqlInjectionMatchSet</a>. </li> <li> Create and update the <code>Rules</code> that you want to include in the <code>WebACL</code>. For more information, see <a>CreateRule</a> and <a>UpdateRule</a>. </li> <li> Create a <code>WebACL</code>. See <a>CreateWebACL</a>. </li> <li> Use <code>GetChangeToken</code> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <a>UpdateWebACL</a> request. </li> <li> Submit an <code>UpdateWebACL</code> request to specify the <code>Rules</code> that you want to include in the <code>WebACL</code>, to specify the default action, and to associate the <code>WebACL</code> with a CloudFront distribution.  The <code>ActivatedRule</code> can be a rule group. If you specify a rule group as your <code>ActivatedRule</code>, you can exclude specific rules from that rule group. If you already have a rule group associated with a web ACL and want to submit an <code>UpdateWebACL</code> request to exclude certain rules from that rule group, you must first remove the rule group from the web ACL, the re-insert it again, specifying the excluded rules. For details, see <a>ActivatedRule$ExcludedRules</a>.  </li> </ol> Be aware that if you try to add a RATE_BASED rule to a web ACL without setting the rule type when first creating the rule, the <a>UpdateWebACL</a> request will fail because the request tries to add a REGULAR rule (the default rule type) with the specified ID, which does not exist.  For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param DefaultAction:  A default action for the web ACL, either ALLOW or BLOCK. AWS WAF performs the default action if a request doesn't match the criteria in any of the rules in a web ACL.:<br/>Updates:  An array of updates to make to the <a>WebACL</a>. An array of <code>WebACLUpdate</code> objects that you want to insert into or delete from a <a>WebACL</a>. For more information, see the applicable data types: <ul> <li>  <a>WebACLUpdate</a>: Contains <code>Action</code> and <code>ActivatedRule</code>  </li> <li>  <a>ActivatedRule</a>: Contains <code>Action</code>, <code>OverrideAction</code>, <code>Priority</code>, <code>RuleId</code>, and <code>Type</code>. <code>ActivatedRule|OverrideAction</code> applies only when updating or adding a <code>RuleGroup</code> to a <code>WebACL</code>. In this case, you do not use <code>ActivatedRule|Action</code>. For all other update requests, <code>ActivatedRule|Action</code> is used instead of <code>ActivatedRule|OverrideAction</code>.  </li> <li>  <a>WafAction</a>: Contains <code>Type</code>  </li> </ul><br/>ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>WebACLId:  The <code>WebACLId</code> of the <a>WebACL</a> that you want to update. <code>WebACLId</code> is returned by <a>CreateWebACL</a> and by <a>ListWebACLs</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> Specifies how you want AWS WAF to respond to requests that match the settings in a <code>Rule</code>. Valid settings include the following: <ul> <li>  <code>ALLOW</code>: AWS WAF allows requests </li> <li>  <code>BLOCK</code>: AWS WAF blocks requests </li> <li>  <code>COUNT</code>: AWS WAF increments a counter of the requests that match all of the conditions in the rule. AWS WAF then continues to inspect the web request based on the remaining rules in the web ACL. You can't specify <code>COUNT</code> for the default action for a <code>WebACL</code>. </li> </ul><br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>UpdateWebACL</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass

    def update_xss_match_set(self, Updates: List[XssMatchSetUpdate], ChangeToken: str, XssMatchSetId: str) -> UpdateXssMatchSetResponse:
        """Inserts or deletes <a>XssMatchTuple</a> objects (filters) in an <a>XssMatchSet</a>. For each <code>XssMatchTuple</code> object, you specify the following values: <ul> <li>  <code>Action</code>: Whether to insert the object into or delete the object from the array. To change an <code>XssMatchTuple</code>, you delete the existing object and add a new one. </li> <li>  <code>FieldToMatch</code>: The part of web requests that you want AWS WAF to inspect and, if you want AWS WAF to inspect a header or custom query parameter, the name of the header or parameter. </li> <li>  <code>TextTransformation</code>: Which text transformation, if any, to perform on the web request before inspecting the request for cross-site scripting attacks. You can only specify a single type of TextTransformation. </li> </ul> You use <code>XssMatchSet</code> objects to specify which CloudFront requests that you want to allow, block, or count. For example, if you're receiving requests that contain cross-site scripting attacks in the request body and you want to block the requests, you can create an <code>XssMatchSet</code> with the applicable settings, and then configure AWS WAF to block the requests.  To create and configure an <code>XssMatchSet</code>, perform the following steps: <ol> <li> Submit a <a>CreateXssMatchSet</a> request. </li> <li> Use <a>GetChangeToken</a> to get the change token that you provide in the <code>ChangeToken</code> parameter of an <a>UpdateIPSet</a> request. </li> <li> Submit an <code>UpdateXssMatchSet</code> request to specify the parts of web requests that you want AWS WAF to inspect for cross-site scripting attacks. </li> </ol> For more information about how to use the AWS WAF API to allow or block HTTP requests, see the <a href="https://docs.aws.amazon.com/waf/latest/developerguide/">AWS WAF Developer Guide</a>.

        :param self:
        :param Updates:  An array of <code>XssMatchSetUpdate</code> objects that you want to insert into or delete from an <a>XssMatchSet</a>. For more information, see the applicable data types: <ul> <li>  <a>XssMatchSetUpdate</a>: Contains <code>Action</code> and <code>XssMatchTuple</code>  </li> <li>  <a>XssMatchTuple</a>: Contains <code>FieldToMatch</code> and <code>TextTransformation</code>  </li> <li>  <a>FieldToMatch</a>: Contains <code>Data</code> and <code>Type</code>  </li> </ul><br/>ChangeToken:  The value returned by the most recent call to <a>GetChangeToken</a>.<br/>XssMatchSetId:  The <code>XssMatchSetId</code> of the <code>XssMatchSet</code> that you want to update. <code>XssMatchSetId</code> is returned by <a>CreateXssMatchSet</a> and by <a>ListXssMatchSets</a>.<br/>
        :return: ChangeToken:  The <code>ChangeToken</code> that you used to submit the <code>UpdateXssMatchSet</code> request. You can also use this value to query the status of the request. For more information, see <a>GetChangeTokenStatus</a>.<br/>        """
        pass



class WorkDocs(BaseClient):
    """The WorkDocs API is designed for the following use cases: <ul> <li> File Migration: File migration applications are supported for users who want to migrate their files from an on-premises or off-premises file system or service. Users can insert files into a user directory structure, as well as allow for basic metadata changes, such as modifications to the permissions of files. </li> <li> Security: Support security applications are supported for users who have additional security needs, such as antivirus or data loss prevention. The API actions, along with AWS CloudTrail, allow these applications to detect when changes occur in Amazon WorkDocs. Then, the application can take the necessary actions and replace the target file. If the target file violates the policy, the application can also choose to email the user. </li> <li> eDiscovery/Analytics: General administrative applications are supported, such as eDiscovery and analytics. These applications can choose to mimic or record the actions in an Amazon WorkDocs site, along with AWS CloudTrail, to replicate data for eDiscovery, backup, or analytical applications. </li> </ul> All Amazon WorkDocs API actions are Amazon authenticated and certificate-signed. They not only require the use of the AWS SDK, but also allow for the exclusive use of IAM users and roles to help facilitate access, trust, and permission policies. By creating a role and allowing an IAM user to access the Amazon WorkDocs site, the IAM user gains full administrative visibility into the entire Amazon WorkDocs site (or as set in the IAM policy). This includes, but is not limited to, the ability to modify file permissions and upload any file to any user. This allows developers to perform the three use cases above, as well as give users the ability to grant access on a selective basis using the IAM model."""


    class DescribeDocumentVersionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_document_versions` for valid parameters"""
            pass
    class DescribeFolderContentsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_folder_contents` for valid parameters"""
            pass
    class DescribeUsersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_users` for valid parameters"""
            pass
    class DescribeActivitiesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_activities` for valid parameters"""
            pass
    class DescribeCommentsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_comments` for valid parameters"""
            pass
    class DescribeGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_groups` for valid parameters"""
            pass
    class DescribeNotificationSubscriptionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_notification_subscriptions` for valid parameters"""
            pass
    class DescribeResourcePermissionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_resource_permissions` for valid parameters"""
            pass
    class DescribeRootFoldersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_root_folders` for valid parameters"""
            pass


    class AbortDocumentVersionUploadRequest(Mapping):
        pass
    
    class ActivateUserRequest(Mapping):
        pass
    
    class ActivateUserResponse(Mapping):
        pass
    
    class AddResourcePermissionsRequest(Mapping):
        pass
    
    class AddResourcePermissionsResponse(Mapping):
        pass
    
    class SharePrincipal(Mapping):
        pass
    
    class NotificationOptions(Mapping):
        pass
    
    class CreateCommentRequest(Mapping):
        pass
    
    class CreateCommentResponse(Mapping):
        pass
    
    class BooleanType(object):
        pass
    
    class CreateCustomMetadataRequest(Mapping):
        pass
    
    class CreateCustomMetadataResponse(Mapping):
        pass
    
    class CustomMetadataMap(object):
        pass
    
    class CreateFolderRequest(Mapping):
        pass
    
    class CreateFolderResponse(Mapping):
        pass
    
    class CreateLabelsRequest(Mapping):
        pass
    
    class CreateLabelsResponse(Mapping):
        pass
    
    class SharedLabel(object):
        pass
    
    class CreateNotificationSubscriptionRequest(Mapping):
        pass
    
    class CreateNotificationSubscriptionResponse(Mapping):
        pass
    
    class CreateUserRequest(Mapping):
        pass
    
    class CreateUserResponse(Mapping):
        pass
    
    class StorageRuleType(Mapping):
        pass
    
    class DeactivateUserRequest(Mapping):
        pass
    
    class DeleteCommentRequest(Mapping):
        pass
    
    class DeleteCustomMetadataRequest(Mapping):
        pass
    
    class DeleteCustomMetadataResponse(Mapping):
        pass
    
    class CustomMetadataKeyType(object):
        pass
    
    class DeleteDocumentRequest(Mapping):
        pass
    
    class DeleteFolderRequest(Mapping):
        pass
    
    class DeleteFolderContentsRequest(Mapping):
        pass
    
    class DeleteLabelsRequest(Mapping):
        pass
    
    class DeleteLabelsResponse(Mapping):
        pass
    
    class DeleteNotificationSubscriptionRequest(Mapping):
        pass
    
    class DeleteUserRequest(Mapping):
        pass
    
    class DescribeActivitiesRequest(Mapping):
        pass
    
    class DescribeActivitiesResponse(Mapping):
        pass
    
    class TimestampType(object):
        pass
    
    class DescribeCommentsRequest(Mapping):
        pass
    
    class DescribeCommentsResponse(Mapping):
        pass
    
    class DescribeDocumentVersionsRequest(Mapping):
        pass
    
    class DescribeDocumentVersionsResponse(Mapping):
        pass
    
    class DescribeFolderContentsRequest(Mapping):
        pass
    
    class DescribeFolderContentsResponse(Mapping):
        pass
    
    class DescribeGroupsRequest(Mapping):
        pass
    
    class DescribeGroupsResponse(Mapping):
        pass
    
    class DescribeNotificationSubscriptionsRequest(Mapping):
        pass
    
    class DescribeNotificationSubscriptionsResponse(Mapping):
        pass
    
    class DescribeResourcePermissionsRequest(Mapping):
        pass
    
    class DescribeResourcePermissionsResponse(Mapping):
        pass
    
    class DescribeRootFoldersRequest(Mapping):
        pass
    
    class DescribeRootFoldersResponse(Mapping):
        pass
    
    class DescribeUsersRequest(Mapping):
        pass
    
    class DescribeUsersResponse(Mapping):
        pass
    
    class GetCurrentUserRequest(Mapping):
        pass
    
    class GetCurrentUserResponse(Mapping):
        pass
    
    class GetDocumentRequest(Mapping):
        pass
    
    class GetDocumentResponse(Mapping):
        pass
    
    class GetDocumentPathRequest(Mapping):
        pass
    
    class GetDocumentPathResponse(Mapping):
        pass
    
    class GetDocumentVersionRequest(Mapping):
        pass
    
    class GetDocumentVersionResponse(Mapping):
        pass
    
    class GetFolderRequest(Mapping):
        pass
    
    class GetFolderResponse(Mapping):
        pass
    
    class GetFolderPathRequest(Mapping):
        pass
    
    class GetFolderPathResponse(Mapping):
        pass
    
    class GetResourcesRequest(Mapping):
        pass
    
    class GetResourcesResponse(Mapping):
        pass
    
    class InitiateDocumentVersionUploadRequest(Mapping):
        pass
    
    class InitiateDocumentVersionUploadResponse(Mapping):
        pass
    
    class SizeType(object):
        pass
    
    class RemoveAllResourcePermissionsRequest(Mapping):
        pass
    
    class RemoveResourcePermissionRequest(Mapping):
        pass
    
    class UpdateDocumentRequest(Mapping):
        pass
    
    class UpdateDocumentVersionRequest(Mapping):
        pass
    
    class UpdateFolderRequest(Mapping):
        pass
    
    class UpdateUserRequest(Mapping):
        pass
    
    class UpdateUserResponse(Mapping):
        pass
    
    def abort_document_version_upload(self, VersionId: str, DocumentId: str, AuthenticationToken: str=None):
        """Aborts the upload of the specified document version that was previously initiated by <a>InitiateDocumentVersionUpload</a>. The client should make this call only when it no longer intends to upload the document version, or fails to do so.

        :param self:
        :param VersionId:  The ID of the version.<br/>DocumentId:  The ID of the document.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return:         """
        pass

    def activate_user(self, UserId: str, AuthenticationToken: str=None) -> ActivateUserResponse:
        """Activates the specified user. Only active users can access Amazon WorkDocs.

        :param self:
        :param AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>UserId:  The ID of the user.<br/>
        :return: User:  The user information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Storage</i> The storage for the user.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Storage</b>: The storage for the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Locale</i> The locale of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeZoneId</i> The time zone ID of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModifiedTimestamp</i> The time when the user was modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The time when the user was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecycleBinFolderId</i> The ID of the recycle bin folder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RootFolderId</i> The ID of the root folder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationId</i> The ID of the organization.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Surname</i> The surname of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GivenName</i> The given name of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailAddress</i> The email address of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> The login name of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageUtilizedInBytes</i> The amount of storage used, in bytes.<br/>The storage for a user.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StorageRule</b>: The storage for a user.        """
        pass

    def add_resource_permissions(self, Principals: List[SharePrincipal], ResourceId: str, AuthenticationToken: str=None, NotificationOptions: NotificationOptions=None) -> AddResourcePermissionsResponse:
        """Creates a set of permissions for the specified folder or document. The resource permissions are overwritten if the principals already have different permissions.

        :param self:
        :param NotificationOptions:  The notification options.:<br/>Principals:  The users, groups, or organization being granted permission.<br/>ResourceId:  The ID of the resource.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailMessage</i> Text value to be included in the email body.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SendEmail</i> Boolean value to indicate an email notification should be sent to the receipients.<br/>
        :return: ShareResults:  The share results.<br/>        """
        pass

    def create_comment(self, Text: str, VersionId: str, DocumentId: str, AuthenticationToken: str=None, ParentId: str=None, ThreadId: str=None, Visibility: str=None, NotifyCollaborators: BooleanType=None) -> CreateCommentResponse:
        """Adds a new comment to the specified document version.

        :param self:
        :param NotifyCollaborators:  Set this parameter to TRUE to send an email out to the document collaborators after the comment is created.<br/>Visibility:  The visibility of the comment. Options are either PRIVATE, where the comment is visible only to the comment author and document owner and co-owners, or PUBLIC, where the comment is visible to document owners, co-owners, and contributors.<br/>Text:  The text of the comment.<br/>ThreadId:  The ID of the root comment in the thread.<br/>ParentId:  The ID of the parent comment.<br/>VersionId:  The ID of the document version.<br/>DocumentId:  The ID of the document.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return: Comment:  The comment that has been created.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecipientId</i> If the comment is a reply to another user's comment, this field contains the user ID of the user being replied to.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Visibility</i> The visibility of the comment. Options are either PRIVATE, where the comment is visible only to the comment author and document owner and co-owners, or PUBLIC, where the comment is visible to document owners, co-owners, and contributors.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the comment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The time that the comment was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Contributor</i> The details of the user who made the comment.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Contributor</b>: The details of the user who made the comment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Text</i> The text of the comment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ThreadId</i> The ID of the root comment in the thread.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParentId</i> The ID of the parent comment.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CommentId</i> The ID of the comment.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Locale</i> The locale of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeZoneId</i> The time zone ID of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ModifiedTimestamp</i> The time when the user was modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The time when the user was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RecycleBinFolderId</i> The ID of the recycle bin folder.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RootFolderId</i> The ID of the root folder.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationId</i> The ID of the organization.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Surname</i> The surname of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>GivenName</i> The given name of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailAddress</i> The email address of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> The login name of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the user.<br/>The storage for the user.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Storage</b>: The storage for the user.        """
        pass

    def create_custom_metadata(self, CustomMetadata: CustomMetadataMap, ResourceId: str, AuthenticationToken: str=None, VersionId: str=None) -> CreateCustomMetadataResponse:
        """Adds one or more custom properties to the specified resource (a folder, document, or version).

        :param self:
        :param CustomMetadata:  Custom metadata in the form of name-value pairs.<br/>VersionId:  The ID of the version, if the custom metadata is being added to a document version.<br/>ResourceId:  The ID of the resource.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return:         """
        pass

    def create_folder(self, ParentFolderId: str, AuthenticationToken: str=None, Name: str=None) -> CreateFolderResponse:
        """Creates a folder with the specified name and parent folder.

        :param self:
        :param ParentFolderId:  The ID of the parent folder.<br/>Name:  The name of the new folder.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return: Metadata:  The metadata of the folder.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestVersionSize</i> The size of the latest version of the folder metadata.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Size</i> The size of the folder metadata.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Labels</i> List of labels on the folder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Signature</i> The unique identifier created from the subfolders and documents of the folder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceState</i> The resource state of the folder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModifiedTimestamp</i> The time when the folder was updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The time when the folder was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParentFolderId</i> The ID of the parent folder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatorId</i> The ID of the creator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the folder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the folder.<br/>        """
        pass

    def create_labels(self, Labels: List[SharedLabel], ResourceId: str, AuthenticationToken: str=None) -> CreateLabelsResponse:
        """Adds the specified list of labels to the given resource (a document or folder)

        :param self:
        :param AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>Labels:  List of labels to add to the resource.<br/>ResourceId:  The ID of the resource.<br/>
        :return:         """
        pass

    def create_notification_subscription(self, SubscriptionType: str, Protocol: str, Endpoint: str, OrganizationId: str) -> CreateNotificationSubscriptionResponse:
        """Configure Amazon WorkDocs to use Amazon SNS notifications. The endpoint receives a confirmation message, and must confirm the subscription. For more information, see <a href="https://docs.aws.amazon.com/workdocs/latest/developerguide/subscribe-notifications.html">Subscribe to Notifications</a> in the <i>Amazon WorkDocs Developer Guide</i>.

        :param self:
        :param SubscriptionType:  The notification type.<br/>Protocol:  The protocol to use. The supported value is https, which delivers JSON-encoded messages using HTTPS POST.<br/>Endpoint:  The endpoint to receive the notifications. If the protocol is HTTPS, the endpoint is a URL that begins with <code>https</code>.<br/>OrganizationId:  The ID of the organization.<br/>
        :return: Subscription:  The subscription.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Protocol</i> The protocol of the subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EndPoint</i> The endpoint of the subscription.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SubscriptionId</i> The ID of the subscription.<br/>        """
        pass

    def create_user(self, Password: str, Surname: str, GivenName: str, Username: str, OrganizationId: str=None, EmailAddress: str=None, TimeZoneId: str=None, StorageRule: StorageRuleType=None, AuthenticationToken: str=None) -> CreateUserResponse:
        """Creates a user in a Simple AD or Microsoft AD directory. The status of a newly created user is "ACTIVE". New users can access Amazon WorkDocs.

        :param self:
        :param AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>StorageRule:  The amount of storage for the user.:<br/>TimeZoneId:  The time zone ID of the user.<br/>Password:  The password of the user.<br/>Surname:  The surname of the user.<br/>GivenName:  The given name of the user.<br/>EmailAddress:  The email address of the user.<br/>Username:  The login name of the user.<br/>OrganizationId:  The ID of the organization.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> The type of storage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageAllocatedInBytes</i> The amount of storage allocated, in bytes.<br/>
        :return: User:  The user information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Storage</i> The storage for the user.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Storage</b>: The storage for the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Locale</i> The locale of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeZoneId</i> The time zone ID of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModifiedTimestamp</i> The time when the user was modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The time when the user was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecycleBinFolderId</i> The ID of the recycle bin folder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RootFolderId</i> The ID of the root folder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationId</i> The ID of the organization.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Surname</i> The surname of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GivenName</i> The given name of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailAddress</i> The email address of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> The login name of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageUtilizedInBytes</i> The amount of storage used, in bytes.<br/>The storage for a user.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StorageRule</b>: The storage for a user.        """
        pass

    def deactivate_user(self, UserId: str, AuthenticationToken: str=None):
        """Deactivates the specified user, which revokes the user's access to Amazon WorkDocs.

        :param self:
        :param AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>UserId:  The ID of the user.<br/>
        :return:         """
        pass

    def delete_comment(self, CommentId: str, VersionId: str, DocumentId: str, AuthenticationToken: str=None):
        """Deletes the specified comment from the document version.

        :param self:
        :param CommentId:  The ID of the comment.<br/>VersionId:  The ID of the document version.<br/>DocumentId:  The ID of the document.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return:         """
        pass

    def delete_custom_metadata(self, ResourceId: str, AuthenticationToken: str=None, VersionId: str=None, Keys: List[CustomMetadataKeyType]=None, DeleteAll: BooleanType=None) -> DeleteCustomMetadataResponse:
        """Deletes custom metadata from the specified resource.

        :param self:
        :param DeleteAll:  Flag to indicate removal of all custom metadata properties from the specified resource.<br/>Keys:  List of properties to remove.<br/>VersionId:  The ID of the version, if the custom metadata is being deleted from a document version.<br/>ResourceId:  The ID of the resource, either a document or folder.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return:         """
        pass

    def delete_document(self, DocumentId: str, AuthenticationToken: str=None):
        """Permanently deletes the specified document and its associated metadata.

        :param self:
        :param DocumentId:  The ID of the document.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return:         """
        pass

    def delete_folder(self, FolderId: str, AuthenticationToken: str=None):
        """Permanently deletes the specified folder and its contents.

        :param self:
        :param FolderId:  The ID of the folder.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return:         """
        pass

    def delete_folder_contents(self, FolderId: str, AuthenticationToken: str=None):
        """Deletes the contents of the specified folder.

        :param self:
        :param FolderId:  The ID of the folder.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return:         """
        pass

    def delete_labels(self, ResourceId: str, AuthenticationToken: str=None, Labels: List[SharedLabel]=None, DeleteAll: BooleanType=None) -> DeleteLabelsResponse:
        """Deletes the specified list of labels from a resource.

        :param self:
        :param DeleteAll:  Flag to request removal of all labels from the specified resource.<br/>Labels:  List of labels to delete from the resource.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>ResourceId:  The ID of the resource.<br/>
        :return:         """
        pass

    def delete_notification_subscription(self, OrganizationId: str, SubscriptionId: str):
        """Deletes the specified subscription from the specified organization.

        :param self:
        :param OrganizationId:  The ID of the organization.<br/>SubscriptionId:  The ID of the subscription.<br/>
        :return:         """
        pass

    def delete_user(self, UserId: str, AuthenticationToken: str=None):
        """Deletes the specified user from a Simple AD or Microsoft AD directory.

        :param self:
        :param UserId:  The ID of the user.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return:         """
        pass

    def describe_activities(self, AuthenticationToken: str=None, StartTime: TimestampType=None, EndTime: TimestampType=None, OrganizationId: str=None, ActivityTypes: str=None, ResourceId: str=None, UserId: str=None, IncludeIndirectActivities: BooleanType=None, Limit: int=None, Marker: str=None) -> DescribeActivitiesResponse:
        """Describes the user activities in a specified time period.

        :param self:
        :param Marker:  The marker for the next set of results.<br/>Limit:  The maximum number of items to return.<br/>IncludeIndirectActivities:  Includes indirect activities. An indirect activity results from a direct activity performed on a parent resource. For example, sharing a parent folder (the direct activity) shares all of the subfolders and documents within the parent folder (the indirect activity).<br/>UserId:  The ID of the user who performed the action. The response includes activities pertaining to this user. This is an optional parameter and is only applicable for administrative API (SigV4) requests.<br/>ResourceId:  The document or folder ID for which to describe activity types.<br/>ActivityTypes:  Specifies which activity types to include in the response. If this field is left empty, all activity types are returned.<br/>OrganizationId:  The ID of the organization. This is a mandatory parameter when using administrative API (SigV4) requests.<br/>EndTime:  The timestamp that determines the end time of the activities. The response includes the activities performed before the specified timestamp.<br/>StartTime:  The timestamp that determines the starting time of the activities. The response includes the activities performed after the specified timestamp.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return: Marker:  The marker for the next set of results.<br/>UserActivities:  The list of activities for the specified user and time period.<br/>        """
        pass

    def describe_comments(self, VersionId: str, DocumentId: str, AuthenticationToken: str=None, Limit: int=None, Marker: str=None) -> DescribeCommentsResponse:
        """List all the comments for the specified document version.

        :param self:
        :param Marker:  The marker for the next set of results. This marker was received from a previous call.<br/>Limit:  The maximum number of items to return.<br/>VersionId:  The ID of the document version.<br/>DocumentId:  The ID of the document.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return: Marker:  The marker for the next set of results. This marker was received from a previous call.<br/>Comments:  The list of comments for the specified document version.<br/>        """
        pass

    def describe_document_versions(self, DocumentId: str, AuthenticationToken: str=None, Marker: str=None, Limit: int=None, Include: str=None, Fields: str=None) -> DescribeDocumentVersionsResponse:
        """Retrieves the document versions for the specified document. By default, only active versions are returned.

        :param self:
        :param Fields:  Specify "SOURCE" to include initialized versions and a URL for the source document.<br/>Include:  A comma-separated list of values. Specify "INITIALIZED" to include incomplete versions.<br/>Limit:  The maximum number of versions to return with this call.<br/>Marker:  The marker for the next set of results. (You received this marker from a previous call.)<br/>DocumentId:  The ID of the document.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return: Marker:  The marker to use when requesting the next set of results. If there are no additional results, the string is empty.<br/>DocumentVersions:  The document versions.<br/>        """
        pass

    def describe_folder_contents(self, FolderId: str, AuthenticationToken: str=None, Sort: str=None, Order: str=None, Limit: int=None, Marker: str=None, Type: str=None, Include: str=None) -> DescribeFolderContentsResponse:
        """Describes the contents of the specified folder, including its documents and subfolders. By default, Amazon WorkDocs returns the first 100 active document and folder metadata items. If there are more results, the response includes a marker that you can use to request the next set of results. You can also request initialized documents.

        :param self:
        :param Include:  The contents to include. Specify "INITIALIZED" to include initialized documents.<br/>Type:  The type of items.<br/>Marker:  The marker for the next set of results. This marker was received from a previous call.<br/>Limit:  The maximum number of items to return with this call.<br/>Order:  The order for the contents of the folder.<br/>Sort:  The sorting criteria.<br/>FolderId:  The ID of the folder.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return: Marker:  The marker to use when requesting the next set of results. If there are no additional results, the string is empty.<br/>Documents:  The documents in the specified folder.<br/>Folders:  The subfolders in the specified folder.<br/>        """
        pass

    def describe_groups(self, SearchQuery: str, AuthenticationToken: str=None, OrganizationId: str=None, Marker: str=None, Limit: int=None) -> DescribeGroupsResponse:
        """Describes the groups specified by the query. Groups are defined by the underlying Active Directory.

        :param self:
        :param Limit:  The maximum number of items to return with this call.<br/>Marker:  The marker for the next set of results. (You received this marker from a previous call.)<br/>OrganizationId:  The ID of the organization.<br/>SearchQuery:  A query to describe groups by group name.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return: Marker:  The marker to use when requesting the next set of results. If there are no additional results, the string is empty.<br/>Groups:  The list of groups.<br/>        """
        pass

    def describe_notification_subscriptions(self, OrganizationId: str, Marker: str=None, Limit: int=None) -> DescribeNotificationSubscriptionsResponse:
        """Lists the specified notification subscriptions.

        :param self:
        :param Limit:  The maximum number of items to return with this call.<br/>Marker:  The marker for the next set of results. (You received this marker from a previous call.)<br/>OrganizationId:  The ID of the organization.<br/>
        :return: Marker:  The marker to use when requesting the next set of results. If there are no additional results, the string is empty.<br/>Subscriptions:  The subscriptions.<br/>        """
        pass

    def describe_resource_permissions(self, ResourceId: str, AuthenticationToken: str=None, PrincipalId: str=None, Limit: int=None, Marker: str=None) -> DescribeResourcePermissionsResponse:
        """Describes the permissions of a specified resource.

        :param self:
        :param Marker:  The marker for the next set of results. (You received this marker from a previous call)<br/>Limit:  The maximum number of items to return with this call.<br/>PrincipalId:  The ID of the principal to filter permissions by.<br/>ResourceId:  The ID of the resource.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return: Marker:  The marker to use when requesting the next set of results. If there are no additional results, the string is empty.<br/>Principals:  The principals.<br/>        """
        pass

    def describe_root_folders(self, AuthenticationToken: str, Limit: int=None, Marker: str=None) -> DescribeRootFoldersResponse:
        """Describes the current user's special folders; the <code>RootFolder</code> and the <code>RecycleBin</code>. <code>RootFolder</code> is the root of user's files and folders and <code>RecycleBin</code> is the root of recycled items. This is not a valid action for SigV4 (administrative API) clients. This action requires an authentication token. To get an authentication token, register an application with Amazon WorkDocs. For more information, see <a href="https://docs.aws.amazon.com/workdocs/latest/developerguide/wd-auth-user.html">Authentication and Access Control for User Applications</a> in the <i>Amazon WorkDocs Developer Guide</i>.

        :param self:
        :param Marker:  The marker for the next set of results. (You received this marker from a previous call.)<br/>Limit:  The maximum number of items to return.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return: Marker:  The marker for the next set of results.<br/>Folders:  The user's special folders.<br/>        """
        pass

    def describe_users(self, AuthenticationToken: str=None, OrganizationId: str=None, UserIds: str=None, Query: str=None, Include: str=None, Order: str=None, Sort: str=None, Marker: str=None, Limit: int=None, Fields: str=None) -> DescribeUsersResponse:
        """Describes the specified users. You can describe all users or filter the results (for example, by status or organization). By default, Amazon WorkDocs returns the first 24 active or pending users. If there are more results, the response includes a marker that you can use to request the next set of results.

        :param self:
        :param Fields:  A comma-separated list of values. Specify "STORAGE_METADATA" to include the user storage quota and utilization information.<br/>Limit:  The maximum number of items to return.<br/>Marker:  The marker for the next set of results. (You received this marker from a previous call.)<br/>Sort:  The sorting criteria.<br/>Order:  The order for the results.<br/>Include:  The state of the users. Specify "ALL" to include inactive users.<br/>Query:  A query to filter users by user name.<br/>UserIds:  The IDs of the users.<br/>OrganizationId:  The ID of the organization.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return: Marker:  The marker to use when requesting the next set of results. If there are no additional results, the string is empty.<br/>TotalNumberOfUsers:  The total number of users included in the results.<br/>Users:  The users.<br/>        """
        pass

    def get_current_user(self, AuthenticationToken: str) -> GetCurrentUserResponse:
        """Retrieves details of the current user for whom the authentication token was generated. This is not a valid action for SigV4 (administrative API) clients.

        :param self:
        :param AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return: User:  Metadata of the user.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Storage</i> The storage for the user.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Storage</b>: The storage for the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Locale</i> The locale of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeZoneId</i> The time zone ID of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModifiedTimestamp</i> The time when the user was modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The time when the user was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecycleBinFolderId</i> The ID of the recycle bin folder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RootFolderId</i> The ID of the root folder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationId</i> The ID of the organization.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Surname</i> The surname of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GivenName</i> The given name of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailAddress</i> The email address of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> The login name of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageUtilizedInBytes</i> The amount of storage used, in bytes.<br/>The storage for a user.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StorageRule</b>: The storage for a user.        """
        pass

    def get_document(self, DocumentId: str, AuthenticationToken: str=None, IncludeCustomMetadata: BooleanType=None) -> GetDocumentResponse:
        """Retrieves details of a document.

        :param self:
        :param IncludeCustomMetadata:  Set this to <code>TRUE</code> to include custom metadata in the response.<br/>DocumentId:  The ID of the document.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return: CustomMetadata:  The custom metadata on the document.<br/>Metadata:  The metadata details of the document.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Labels</i> List of labels on the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceState</i> The resource state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestVersionMetadata</i> The latest version of the document.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LatestVersionMetadata</b>: The latest version of the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModifiedTimestamp</i> The time when the document was updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The time when the document was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParentFolderId</i> The ID of the parent folder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatorId</i> The ID of the creator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Source</i> The source of the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Thumbnail</i> The thumbnail of the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatorId</i> The ID of the creator.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ContentModifiedTimestamp</i> The timestamp when the content of the document was modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ContentCreatedTimestamp</i> The timestamp when the content of the document was originally created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ModifiedTimestamp</i> The timestamp when the document was last uploaded.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The timestamp when the document was first uploaded.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Signature</i> The signature of the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Size</i> The size of the document, in bytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ContentType</i> The content type of the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the version.<br/>        """
        pass

    def get_document_path(self, DocumentId: str, AuthenticationToken: str=None, Limit: int=None, Fields: str=None, Marker: str=None) -> GetDocumentPathResponse:
        """Retrieves the path information (the hierarchy from the root folder) for the requested document. By default, Amazon WorkDocs returns a maximum of 100 levels upwards from the requested document and only includes the IDs of the parent folders in the path. You can limit the maximum number of levels. You can also request the names of the parent folders.

        :param self:
        :param Marker:  This value is not supported.<br/>Fields:  A comma-separated list of values. Specify <code>NAME</code> to include the names of the parent folders.<br/>Limit:  The maximum number of levels in the hierarchy to return.<br/>DocumentId:  The ID of the document.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return: Path:  The path information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Components</i> The components of the resource path.<br/>        """
        pass

    def get_document_version(self, VersionId: str, DocumentId: str, AuthenticationToken: str=None, Fields: str=None, IncludeCustomMetadata: BooleanType=None) -> GetDocumentVersionResponse:
        """Retrieves version metadata for the specified document.

        :param self:
        :param IncludeCustomMetadata:  Set this to TRUE to include custom metadata in the response.<br/>Fields:  A comma-separated list of values. Specify "SOURCE" to include a URL for the source document.<br/>VersionId:  The version ID of the document.<br/>DocumentId:  The ID of the document.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return: CustomMetadata:  The custom metadata on the document version.<br/>Metadata:  The version metadata.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Source</i> The source of the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Thumbnail</i> The thumbnail of the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatorId</i> The ID of the creator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContentModifiedTimestamp</i> The timestamp when the content of the document was modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContentCreatedTimestamp</i> The timestamp when the content of the document was originally created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModifiedTimestamp</i> The timestamp when the document was last uploaded.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The timestamp when the document was first uploaded.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Signature</i> The signature of the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Size</i> The size of the document, in bytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ContentType</i> The content type of the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the version.<br/>        """
        pass

    def get_folder(self, FolderId: str, AuthenticationToken: str=None, IncludeCustomMetadata: BooleanType=None) -> GetFolderResponse:
        """Retrieves the metadata of the specified folder.

        :param self:
        :param IncludeCustomMetadata:  Set to TRUE to include custom metadata in the response.<br/>FolderId:  The ID of the folder.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return: CustomMetadata:  The custom metadata on the folder.<br/>Metadata:  The metadata of the folder.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestVersionSize</i> The size of the latest version of the folder metadata.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Size</i> The size of the folder metadata.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Labels</i> List of labels on the folder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Signature</i> The unique identifier created from the subfolders and documents of the folder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceState</i> The resource state of the folder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModifiedTimestamp</i> The time when the folder was updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The time when the folder was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParentFolderId</i> The ID of the parent folder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatorId</i> The ID of the creator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the folder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the folder.<br/>        """
        pass

    def get_folder_path(self, FolderId: str, AuthenticationToken: str=None, Limit: int=None, Fields: str=None, Marker: str=None) -> GetFolderPathResponse:
        """Retrieves the path information (the hierarchy from the root folder) for the specified folder. By default, Amazon WorkDocs returns a maximum of 100 levels upwards from the requested folder and only includes the IDs of the parent folders in the path. You can limit the maximum number of levels. You can also request the parent folder names.

        :param self:
        :param Marker:  This value is not supported.<br/>Fields:  A comma-separated list of values. Specify "NAME" to include the names of the parent folders.<br/>Limit:  The maximum number of levels in the hierarchy to return.<br/>FolderId:  The ID of the folder.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return: Path:  The path information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Components</i> The components of the resource path.<br/>        """
        pass

    def get_resources(self, AuthenticationToken: str=None, UserId: str=None, CollectionType: str=None, Limit: int=None, Marker: str=None) -> GetResourcesResponse:
        """Retrieves a collection of resources, including folders and documents. The only <code>CollectionType</code> supported is <code>SHARED_WITH_ME</code>.

        :param self:
        :param Marker:  The marker for the next set of results. This marker was received from a previous call.<br/>Limit:  The maximum number of resources to return.<br/>CollectionType:  The collection type.<br/>UserId:  The user ID for the resource collection. This is a required field for accessing the API operation using IAM credentials.<br/>AuthenticationToken:  The Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API operation using AWS credentials.<br/>
        :return: Marker:  The marker to use when requesting the next set of results. If there are no additional results, the string is empty.<br/>Documents:  The documents in the specified collection.<br/>Folders:  The folders in the specified folder.<br/>        """
        pass

    def initiate_document_version_upload(self, ParentFolderId: str, AuthenticationToken: str=None, Id: str=None, Name: str=None, ContentCreatedTimestamp: TimestampType=None, ContentModifiedTimestamp: TimestampType=None, ContentType: str=None, DocumentSizeInBytes: SizeType=None) -> InitiateDocumentVersionUploadResponse:
        """Creates a new document object and version object. The client specifies the parent folder ID and name of the document to upload. The ID is optionally specified when creating a new version of an existing document. This is the first step to upload a document. Next, upload the document to the URL returned from the call, and then call <a>UpdateDocumentVersion</a>. To cancel the document upload, call <a>AbortDocumentVersionUpload</a>.

        :param self:
        :param ParentFolderId:  The ID of the parent folder.<br/>DocumentSizeInBytes:  The size of the document, in bytes.<br/>ContentType:  The content type of the document.<br/>ContentModifiedTimestamp:  The timestamp when the content of the document was modified.<br/>ContentCreatedTimestamp:  The timestamp when the content of the document was originally created.<br/>Name:  The name of the document.<br/>Id:  The ID of the document.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return: UploadMetadata:  The upload metadata.:<br/>Metadata:  The document metadata.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Labels</i> List of labels on the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceState</i> The resource state.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>LatestVersionMetadata</i> The latest version of the document.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>LatestVersionMetadata</b>: The latest version of the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModifiedTimestamp</i> The time when the document was updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The time when the document was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ParentFolderId</i> The ID of the parent folder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatorId</i> The ID of the creator.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Source</i> The source of the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Thumbnail</i> The thumbnail of the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatorId</i> The ID of the creator.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ContentModifiedTimestamp</i> The timestamp when the content of the document was modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ContentCreatedTimestamp</i> The timestamp when the content of the document was originally created.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ModifiedTimestamp</i> The timestamp when the document was last uploaded.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The timestamp when the document was first uploaded.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Signature</i> The signature of the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Size</i> The size of the document, in bytes.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ContentType</i> The content type of the document.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of the version.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the version.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SignedHeaders</i> The signed headers.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UploadUrl</i> The URL of the upload.<br/>        """
        pass

    def remove_all_resource_permissions(self, ResourceId: str, AuthenticationToken: str=None):
        """Removes all the permissions from the specified resource.

        :param self:
        :param ResourceId:  The ID of the resource.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return:         """
        pass

    def remove_resource_permission(self, PrincipalId: str, ResourceId: str, AuthenticationToken: str=None, PrincipalType: str=None):
        """Removes the permission for the specified principal from the specified resource.

        :param self:
        :param PrincipalType:  The principal type of the resource.<br/>PrincipalId:  The principal ID of the resource.<br/>ResourceId:  The ID of the resource.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return:         """
        pass

    def update_document(self, DocumentId: str, AuthenticationToken: str=None, Name: str=None, ParentFolderId: str=None, ResourceState: str=None):
        """Updates the specified attributes of a document. The user must have access to both the document and its parent folder, if applicable.

        :param self:
        :param ResourceState:  The resource state of the document. Only ACTIVE and RECYCLED are supported.<br/>ParentFolderId:  The ID of the parent folder.<br/>Name:  The name of the document.<br/>DocumentId:  The ID of the document.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return:         """
        pass

    def update_document_version(self, VersionId: str, DocumentId: str, AuthenticationToken: str=None, VersionStatus: str=None):
        """Changes the status of the document version to ACTIVE.  Amazon WorkDocs also sets its document container to ACTIVE. This is the last step in a document upload, after the client uploads the document to an S3-presigned URL returned by <a>InitiateDocumentVersionUpload</a>. 

        :param self:
        :param VersionStatus:  The status of the version.<br/>VersionId:  The version ID of the document.<br/>DocumentId:  The ID of the document.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return:         """
        pass

    def update_folder(self, FolderId: str, AuthenticationToken: str=None, Name: str=None, ParentFolderId: str=None, ResourceState: str=None):
        """Updates the specified attributes of the specified folder. The user must have access to both the folder and its parent folder, if applicable.

        :param self:
        :param ResourceState:  The resource state of the folder. Only ACTIVE and RECYCLED are accepted values from the API.<br/>ParentFolderId:  The ID of the parent folder.<br/>Name:  The name of the folder.<br/>FolderId:  The ID of the folder.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>
        :return:         """
        pass

    def update_user(self, UserId: str, AuthenticationToken: str=None, GivenName: str=None, Surname: str=None, Type: str=None, StorageRule: StorageRuleType=None, TimeZoneId: str=None, Locale: str=None, GrantPoweruserPrivileges: str=None) -> UpdateUserResponse:
        """Updates the specified attributes of the specified user, and grants or revokes administrative privileges to the Amazon WorkDocs site.

        :param self:
        :param GrantPoweruserPrivileges:  Boolean value to determine whether the user is granted Poweruser privileges.<br/>Locale:  The locale of the user.<br/>TimeZoneId:  The time zone ID of the user.<br/>StorageRule:  The amount of storage for the user.:<br/>Type:  The type of the user.<br/>Surname:  The surname of the user.<br/>GivenName:  The given name of the user.<br/>UserId:  The ID of the user.<br/>AuthenticationToken:  Amazon WorkDocs authentication token. Do not set this field when using administrative API actions, as in accessing the API using AWS credentials.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageType</i> The type of storage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageAllocatedInBytes</i> The amount of storage allocated, in bytes.<br/>
        :return: User:  The user information.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Storage</i> The storage for the user.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Storage</b>: The storage for the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Locale</i> The locale of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>TimeZoneId</i> The time zone ID of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModifiedTimestamp</i> The time when the user was modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedTimestamp</i> The time when the user was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The status of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RecycleBinFolderId</i> The ID of the recycle bin folder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RootFolderId</i> The ID of the root folder.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>OrganizationId</i> The ID of the organization.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Surname</i> The surname of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GivenName</i> The given name of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>EmailAddress</i> The email address of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Username</i> The login name of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Id</i> The ID of the user.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>StorageUtilizedInBytes</i> The amount of storage used, in bytes.<br/>The storage for a user.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>StorageRule</b>: The storage for a user.        """
        pass



class WorkLink(BaseClient):
    """Amazon WorkLink is a cloud-based service that provides secure access to internal websites and web apps from iOS phones. In a single step, your users, such as employees, can access internal websites as efficiently as they access any other public website. They enter a URL in their web browser, or choose a link to an internal website in an email. Amazon WorkLink authenticates the user's access and securely renders authorized internal web content in a secure rendering service in the AWS cloud. Amazon WorkLink doesn't download or store any internal web content on mobile devices."""




    class AssociateDomainRequest(Mapping):
        pass
    
    class AssociateDomainResponse(Mapping):
        pass
    
    class AssociateWebsiteAuthorizationProviderRequest(Mapping):
        pass
    
    class AssociateWebsiteAuthorizationProviderResponse(Mapping):
        pass
    
    class AssociateWebsiteCertificateAuthorityRequest(Mapping):
        pass
    
    class AssociateWebsiteCertificateAuthorityResponse(Mapping):
        pass
    
    class CreateFleetRequest(Mapping):
        pass
    
    class CreateFleetResponse(Mapping):
        pass
    
    class Boolean(object):
        pass
    
    class DeleteFleetRequest(Mapping):
        pass
    
    class DeleteFleetResponse(Mapping):
        pass
    
    class DescribeAuditStreamConfigurationRequest(Mapping):
        pass
    
    class DescribeAuditStreamConfigurationResponse(Mapping):
        pass
    
    class DescribeCompanyNetworkConfigurationRequest(Mapping):
        pass
    
    class DescribeCompanyNetworkConfigurationResponse(Mapping):
        pass
    
    class DescribeDeviceRequest(Mapping):
        pass
    
    class DescribeDeviceResponse(Mapping):
        pass
    
    class DescribeDevicePolicyConfigurationRequest(Mapping):
        pass
    
    class DescribeDevicePolicyConfigurationResponse(Mapping):
        pass
    
    class DescribeDomainRequest(Mapping):
        pass
    
    class DescribeDomainResponse(Mapping):
        pass
    
    class DescribeFleetMetadataRequest(Mapping):
        pass
    
    class DescribeFleetMetadataResponse(Mapping):
        pass
    
    class DescribeIdentityProviderConfigurationRequest(Mapping):
        pass
    
    class DescribeIdentityProviderConfigurationResponse(Mapping):
        pass
    
    class DescribeWebsiteCertificateAuthorityRequest(Mapping):
        pass
    
    class DescribeWebsiteCertificateAuthorityResponse(Mapping):
        pass
    
    class DisassociateDomainRequest(Mapping):
        pass
    
    class DisassociateDomainResponse(Mapping):
        pass
    
    class DisassociateWebsiteAuthorizationProviderRequest(Mapping):
        pass
    
    class DisassociateWebsiteAuthorizationProviderResponse(Mapping):
        pass
    
    class DisassociateWebsiteCertificateAuthorityRequest(Mapping):
        pass
    
    class DisassociateWebsiteCertificateAuthorityResponse(Mapping):
        pass
    
    class ListDevicesRequest(Mapping):
        pass
    
    class ListDevicesResponse(Mapping):
        pass
    
    class ListDomainsRequest(Mapping):
        pass
    
    class ListDomainsResponse(Mapping):
        pass
    
    class ListFleetsRequest(Mapping):
        pass
    
    class ListFleetsResponse(Mapping):
        pass
    
    class ListWebsiteAuthorizationProvidersRequest(Mapping):
        pass
    
    class ListWebsiteAuthorizationProvidersResponse(Mapping):
        pass
    
    class ListWebsiteCertificateAuthoritiesRequest(Mapping):
        pass
    
    class ListWebsiteCertificateAuthoritiesResponse(Mapping):
        pass
    
    class RestoreDomainAccessRequest(Mapping):
        pass
    
    class RestoreDomainAccessResponse(Mapping):
        pass
    
    class RevokeDomainAccessRequest(Mapping):
        pass
    
    class RevokeDomainAccessResponse(Mapping):
        pass
    
    class SignOutUserRequest(Mapping):
        pass
    
    class SignOutUserResponse(Mapping):
        pass
    
    class UpdateAuditStreamConfigurationRequest(Mapping):
        pass
    
    class UpdateAuditStreamConfigurationResponse(Mapping):
        pass
    
    class UpdateCompanyNetworkConfigurationRequest(Mapping):
        pass
    
    class UpdateCompanyNetworkConfigurationResponse(Mapping):
        pass
    
    class SubnetId(object):
        pass
    
    class SecurityGroupId(object):
        pass
    
    class UpdateDevicePolicyConfigurationRequest(Mapping):
        pass
    
    class UpdateDevicePolicyConfigurationResponse(Mapping):
        pass
    
    class UpdateDomainMetadataRequest(Mapping):
        pass
    
    class UpdateDomainMetadataResponse(Mapping):
        pass
    
    class UpdateFleetMetadataRequest(Mapping):
        pass
    
    class UpdateFleetMetadataResponse(Mapping):
        pass
    
    class UpdateIdentityProviderConfigurationRequest(Mapping):
        pass
    
    class UpdateIdentityProviderConfigurationResponse(Mapping):
        pass
    
    def associate_domain(self, AcmCertificateArn: str, DomainName: str, FleetArn: str, DisplayName: str=None) -> AssociateDomainResponse:
        """Specifies a domain to be associated to Amazon WorkLink.

        :param self:
        :param AcmCertificateArn:  The ARN of an issued ACM certificate that is valid for the domain being associated.<br/>DisplayName:  The name to display.<br/>DomainName:  The fully qualified domain name (FQDN).<br/>FleetArn:  The Amazon Resource Name (ARN) of the fleet.<br/>
        :return:         """
        pass

    def associate_website_authorization_provider(self, AuthorizationProviderType: str, FleetArn: str, DomainName: str=None) -> AssociateWebsiteAuthorizationProviderResponse:
        """Associates a website authorization provider with a specified fleet. This is used to authorize users against associated websites in the company network.

        :param self:
        :param DomainName:  The domain name of the authorization provider. This applies only to SAML-based authorization providers.<br/>AuthorizationProviderType:  The authorization provider type.<br/>FleetArn:  The ARN of the fleet.<br/>
        :return: AuthorizationProviderId:  A unique identifier for the authorization provider.<br/>        """
        pass

    def associate_website_certificate_authority(self, Certificate: str, FleetArn: str, DisplayName: str=None) -> AssociateWebsiteCertificateAuthorityResponse:
        """Imports the root certificate of a certificate authority (CA) used to obtain TLS certificates used by associated websites within the company network.

        :param self:
        :param DisplayName:  The certificate name to display.<br/>Certificate:  The root certificate of the CA.<br/>FleetArn:  The ARN of the fleet.<br/>
        :return: WebsiteCaId:  A unique identifier for the CA.<br/>        """
        pass

    def create_fleet(self, FleetName: str, DisplayName: str=None, OptimizeForEndUserLocation: Boolean=None) -> CreateFleetResponse:
        """Creates a fleet. A fleet consists of resources and the configuration that delivers associated websites to authorized users who download and set up the Amazon WorkLink app.

        :param self:
        :param OptimizeForEndUserLocation:  The option to optimize for better performance by routing traffic through the closest AWS Region to users, which may be outside of your home Region.<br/>DisplayName:  The fleet name to display.<br/>FleetName:  A unique name for the fleet.<br/>
        :return: FleetArn:  The ARN of the fleet.<br/>        """
        pass

    def delete_fleet(self, FleetArn: str) -> DeleteFleetResponse:
        """Deletes a fleet. Prevents users from accessing previously associated websites. 

        :param self:
        :param FleetArn:  The ARN of the fleet.<br/>
        :return:         """
        pass

    def describe_audit_stream_configuration(self, FleetArn: str) -> DescribeAuditStreamConfigurationResponse:
        """Describes the configuration for delivering audit streams to the customer account.

        :param self:
        :param FleetArn:  The ARN of the fleet.<br/>
        :return: AuditStreamArn:  The ARN of the Amazon Kinesis data stream that will receive the audit events.<br/>        """
        pass

    def describe_company_network_configuration(self, FleetArn: str) -> DescribeCompanyNetworkConfigurationResponse:
        """Describes the networking configuration to access the internal websites associated with the specified fleet.

        :param self:
        :param FleetArn:  The ARN of the fleet.<br/>
        :return: SecurityGroupIds:  The security groups associated with access to the provided subnets.<br/>SubnetIds:  The subnets used for X-ENI connections from Amazon WorkLink rendering containers.<br/>VpcId:  The VPC with connectivity to associated websites.<br/>        """
        pass

    def describe_device(self, DeviceId: str, FleetArn: str) -> DescribeDeviceResponse:
        """Provides information about a user's device.

        :param self:
        :param DeviceId:  A unique identifier for a registered user's device.<br/>FleetArn:  The ARN of the fleet.<br/>
        :return: Username:  The user name associated with the device.<br/>LastAccessedTime:  The date that the device last accessed Amazon WorkLink.<br/>FirstAccessedTime:  The date that the device first signed in to Amazon WorkLink.<br/>PatchLevel:  The operating system patch level of the device.<br/>OperatingSystemVersion:  The operating system version of the device.<br/>OperatingSystem:  The operating system of the device.<br/>Manufacturer:  The manufacturer of the device.<br/>Model:  The model of the device.<br/>Status:  The current state of the device.<br/>        """
        pass

    def describe_device_policy_configuration(self, FleetArn: str) -> DescribeDevicePolicyConfigurationResponse:
        """Describes the device policy configuration for the specified fleet.

        :param self:
        :param FleetArn:  The ARN of the fleet.<br/>
        :return: DeviceCaCertificate:  The certificate chain, including intermediate certificates and the root certificate authority certificate used to issue device certificates.<br/>        """
        pass

    def describe_domain(self, DomainName: str, FleetArn: str) -> DescribeDomainResponse:
        """Provides information about the domain.

        :param self:
        :param DomainName:  The name of the domain.<br/>FleetArn:  The ARN of the fleet.<br/>
        :return: AcmCertificateArn:  The ARN of an issued ACM certificate that is valid for the domain being associated.<br/>DomainStatus:  The current state for the domain.<br/>CreatedTime:  The time that the domain was added.<br/>DisplayName:  The name to display.<br/>DomainName:  The name of the domain.<br/>        """
        pass

    def describe_fleet_metadata(self, FleetArn: str) -> DescribeFleetMetadataResponse:
        """Provides basic information for the specified fleet, excluding identity provider, networking, and device configuration details.

        :param self:
        :param FleetArn:  The ARN of the fleet.<br/>
        :return: FleetStatus:  The current state of the fleet.<br/>CompanyCode:  The identifier used by users to sign in to the Amazon WorkLink app.<br/>OptimizeForEndUserLocation:  The option to optimize for better performance by routing traffic through the closest AWS Region to users, which may be outside of your home Region.<br/>DisplayName:  The name to display.<br/>FleetName:  The name of the fleet.<br/>LastUpdatedTime:  The time that the fleet was last updated.<br/>CreatedTime:  The time that the fleet was created.<br/>        """
        pass

    def describe_identity_provider_configuration(self, FleetArn: str) -> DescribeIdentityProviderConfigurationResponse:
        """Describes the identity provider configuration of the specified fleet.

        :param self:
        :param FleetArn:  The ARN of the fleet.<br/>
        :return: IdentityProviderSamlMetadata:  The SAML metadata document provided by the user’s identity provider.<br/>ServiceProviderSamlMetadata:  The SAML metadata document uploaded to the user’s identity provider.<br/>IdentityProviderType:  The type of identity provider.<br/>        """
        pass

    def describe_website_certificate_authority(self, WebsiteCaId: str, FleetArn: str) -> DescribeWebsiteCertificateAuthorityResponse:
        """Provides information about the certificate authority.

        :param self:
        :param WebsiteCaId:  A unique identifier for the certificate authority.<br/>FleetArn:  The ARN of the fleet.<br/>
        :return: DisplayName:  The certificate name to display.<br/>CreatedTime:  The time that the certificate authority was added.<br/>Certificate:  The root certificate of the certificate authority.<br/>        """
        pass

    def disassociate_domain(self, DomainName: str, FleetArn: str) -> DisassociateDomainResponse:
        """Disassociates a domain from Amazon WorkLink. End users lose the ability to access the domain with Amazon WorkLink. 

        :param self:
        :param DomainName:  The name of the domain.<br/>FleetArn:  The ARN of the fleet.<br/>
        :return:         """
        pass

    def disassociate_website_authorization_provider(self, AuthorizationProviderId: str, FleetArn: str) -> DisassociateWebsiteAuthorizationProviderResponse:
        """Disassociates a website authorization provider from a specified fleet. After the disassociation, users can't load any associated websites that require this authorization provider.

        :param self:
        :param AuthorizationProviderId:  A unique identifier for the authorization provider.<br/>FleetArn:  The ARN of the fleet.<br/>
        :return:         """
        pass

    def disassociate_website_certificate_authority(self, WebsiteCaId: str, FleetArn: str) -> DisassociateWebsiteCertificateAuthorityResponse:
        """Removes a certificate authority (CA).

        :param self:
        :param WebsiteCaId:  A unique identifier for the CA.<br/>FleetArn:  The ARN of the fleet.<br/>
        :return:         """
        pass

    def list_devices(self, FleetArn: str, NextToken: str=None, MaxResults: int=None) -> ListDevicesResponse:
        """Retrieves a list of devices registered with the specified fleet.

        :param self:
        :param MaxResults:  The maximum number of results to be included in the next page.<br/>NextToken:  The pagination token used to retrieve the next page of results for this operation. If this value is null, it retrieves the first page.<br/>FleetArn:  The ARN of the fleet.<br/>
        :return: NextToken:  The pagination token used to retrieve the next page of results for this operation. If there are no more pages, this value is null.<br/>Devices:  Information about the devices.<br/>        """
        pass

    def list_domains(self, FleetArn: str, NextToken: str=None, MaxResults: int=None) -> ListDomainsResponse:
        """Retrieves a list of domains associated to a specified fleet.

        :param self:
        :param MaxResults:  The maximum number of results to be included in the next page.<br/>NextToken:  The pagination token used to retrieve the next page of results for this operation. If this value is null, it retrieves the first page.<br/>FleetArn:  The ARN of the fleet.<br/>
        :return: NextToken:  The pagination token used to retrieve the next page of results for this operation. If there are no more pages, this value is null.<br/>Domains:  Information about the domains.<br/>        """
        pass

    def list_fleets(self, NextToken: str=None, MaxResults: int=None) -> ListFleetsResponse:
        """Retrieves a list of fleets for the current account and Region.

        :param self:
        :param MaxResults:  The maximum number of results to be included in the next page.<br/>NextToken:  The pagination token used to retrieve the next page of results for this operation. If this value is null, it retrieves the first page.<br/>
        :return: NextToken:  The pagination token used to retrieve the next page of results for this operation. If there are no more pages, this value is null.<br/>FleetSummaryList:  The summary list of the fleets.<br/>        """
        pass

    def list_website_authorization_providers(self, FleetArn: str, NextToken: str=None, MaxResults: int=None) -> ListWebsiteAuthorizationProvidersResponse:
        """Retrieves a list of website authorization providers associated with a specified fleet.

        :param self:
        :param MaxResults:  The maximum number of results to be included in the next page.<br/>NextToken:  The pagination token to use to retrieve the next page of results for this operation. If this value is null, it retrieves the first page.<br/>FleetArn:  The ARN of the fleet.<br/>
        :return: NextToken:  The pagination token to use to retrieve the next page of results for this operation. If this value is null, it retrieves the first page.<br/>WebsiteAuthorizationProviders:  The website authorization providers.<br/>        """
        pass

    def list_website_certificate_authorities(self, FleetArn: str, MaxResults: int=None, NextToken: str=None) -> ListWebsiteCertificateAuthoritiesResponse:
        """Retrieves a list of certificate authorities added for the current account and Region.

        :param self:
        :param NextToken:  The pagination token used to retrieve the next page of results for this operation. If this value is null, it retrieves the first page.<br/>MaxResults:  The maximum number of results to be included in the next page.<br/>FleetArn:  The ARN of the fleet.<br/>
        :return: NextToken:  The pagination token used to retrieve the next page of results for this operation. If there are no more pages, this value is null.<br/>WebsiteCertificateAuthorities:  Information about the certificates.<br/>        """
        pass

    def restore_domain_access(self, DomainName: str, FleetArn: str) -> RestoreDomainAccessResponse:
        """Moves a domain to ACTIVE status if it was in the INACTIVE status.

        :param self:
        :param DomainName:  The name of the domain.<br/>FleetArn:  The ARN of the fleet.<br/>
        :return:         """
        pass

    def revoke_domain_access(self, DomainName: str, FleetArn: str) -> RevokeDomainAccessResponse:
        """Moves a domain to INACTIVE status if it was in the ACTIVE status.

        :param self:
        :param DomainName:  The name of the domain.<br/>FleetArn:  The ARN of the fleet.<br/>
        :return:         """
        pass

    def sign_out_user(self, Username: str, FleetArn: str) -> SignOutUserResponse:
        """Signs the user out from all of their devices. The user can sign in again if they have valid credentials.

        :param self:
        :param Username:  The name of the user.<br/>FleetArn:  The ARN of the fleet.<br/>
        :return:         """
        pass

    def update_audit_stream_configuration(self, FleetArn: str, AuditStreamArn: str=None) -> UpdateAuditStreamConfigurationResponse:
        """Updates the audit stream configuration for the fleet.

        :param self:
        :param AuditStreamArn:  The ARN of the Amazon Kinesis data stream that receives the audit events.<br/>FleetArn:  The ARN of the fleet.<br/>
        :return:         """
        pass

    def update_company_network_configuration(self, SecurityGroupIds: List[SecurityGroupId], SubnetIds: List[SubnetId], VpcId: str, FleetArn: str) -> UpdateCompanyNetworkConfigurationResponse:
        """Updates the company network configuration for the fleet.

        :param self:
        :param SecurityGroupIds:  The security groups associated with access to the provided subnets.<br/>SubnetIds:  The subnets used for X-ENI connections from Amazon WorkLink rendering containers.<br/>VpcId:  The VPC with connectivity to associated websites.<br/>FleetArn:  The ARN of the fleet.<br/>
        :return:         """
        pass

    def update_device_policy_configuration(self, FleetArn: str, DeviceCaCertificate: str=None) -> UpdateDevicePolicyConfigurationResponse:
        """Updates the device policy configuration for the fleet.

        :param self:
        :param DeviceCaCertificate:  The certificate chain, including intermediate certificates and the root certificate authority certificate used to issue device certificates.<br/>FleetArn:  The ARN of the fleet.<br/>
        :return:         """
        pass

    def update_domain_metadata(self, DomainName: str, FleetArn: str, DisplayName: str=None) -> UpdateDomainMetadataResponse:
        """Updates domain metadata, such as DisplayName.

        :param self:
        :param DisplayName:  The name to display.<br/>DomainName:  The name of the domain.<br/>FleetArn:  The ARN of the fleet.<br/>
        :return:         """
        pass

    def update_fleet_metadata(self, FleetArn: str, DisplayName: str=None, OptimizeForEndUserLocation: Boolean=None) -> UpdateFleetMetadataResponse:
        """Updates fleet metadata, such as DisplayName.

        :param self:
        :param OptimizeForEndUserLocation:  The option to optimize for better performance by routing traffic through the closest AWS Region to users, which may be outside of your home Region.<br/>DisplayName:  The fleet name to display. The existing DisplayName is unset if null is passed.<br/>FleetArn:  The ARN of the fleet.<br/>
        :return:         """
        pass

    def update_identity_provider_configuration(self, IdentityProviderType: str, FleetArn: str, IdentityProviderSamlMetadata: str=None) -> UpdateIdentityProviderConfigurationResponse:
        """Updates the identity provider configuration for the fleet.

        :param self:
        :param IdentityProviderSamlMetadata:  The SAML metadata document provided by the customer’s identity provider. The existing IdentityProviderSamlMetadata is unset if null is passed.<br/>IdentityProviderType:  The type of identity provider.<br/>FleetArn:  The ARN of the fleet.<br/>
        :return:         """
        pass



class WorkMail(BaseClient):
    """Amazon WorkMail is a secure, managed business email and calendaring service with support for existing desktop and mobile email clients. You can access your email, contacts, and calendars using Microsoft Outlook, your browser, or other native iOS and Android email applications. You can integrate WorkMail with your existing corporate directory and control both the keys that encrypt your data and the location in which your data is stored. The WorkMail API is designed for the following scenarios: <ul> <li> Listing and describing organizations </li> </ul> <ul> <li> Managing users </li> </ul> <ul> <li> Managing groups </li> </ul> <ul> <li> Managing resources </li> </ul> All WorkMail API operations are Amazon-authenticated and certificate-signed. They not only require the use of the AWS SDK, but also allow for the exclusive use of AWS Identity and Access Management users and roles to help facilitate access, trust, and permission policies. By creating a role and allowing an IAM user to access the WorkMail site, the IAM user gains full administrative visibility into the entire WorkMail organization (or as set in the IAM policy). This includes, but is not limited to, the ability to create, update, and delete users, groups, and resources. This allows developers to perform the scenarios listed above, as well as give users the ability to grant access on a selective basis using the IAM model."""


    class ListUsersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_users` for valid parameters"""
            pass
    class ListGroupMembersPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_group_members` for valid parameters"""
            pass
    class ListOrganizationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_organizations` for valid parameters"""
            pass
    class ListGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_groups` for valid parameters"""
            pass
    class ListResourcesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_resources` for valid parameters"""
            pass
    class ListAliasesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_aliases` for valid parameters"""
            pass
    class ListMailboxPermissionsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_mailbox_permissions` for valid parameters"""
            pass
    class ListResourceDelegatesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_resource_delegates` for valid parameters"""
            pass


    class AssociateDelegateToResourceRequest(Mapping):
        pass
    
    class AssociateDelegateToResourceResponse(Mapping):
        pass
    
    class AssociateMemberToGroupRequest(Mapping):
        pass
    
    class AssociateMemberToGroupResponse(Mapping):
        pass
    
    class CreateAliasRequest(Mapping):
        pass
    
    class CreateAliasResponse(Mapping):
        pass
    
    class CreateGroupRequest(Mapping):
        pass
    
    class CreateGroupResponse(Mapping):
        pass
    
    class CreateResourceRequest(Mapping):
        pass
    
    class CreateResourceResponse(Mapping):
        pass
    
    class CreateUserRequest(Mapping):
        pass
    
    class CreateUserResponse(Mapping):
        pass
    
    class DeleteAliasRequest(Mapping):
        pass
    
    class DeleteAliasResponse(Mapping):
        pass
    
    class DeleteGroupRequest(Mapping):
        pass
    
    class DeleteGroupResponse(Mapping):
        pass
    
    class DeleteMailboxPermissionsRequest(Mapping):
        pass
    
    class DeleteMailboxPermissionsResponse(Mapping):
        pass
    
    class DeleteResourceRequest(Mapping):
        pass
    
    class DeleteResourceResponse(Mapping):
        pass
    
    class DeleteUserRequest(Mapping):
        pass
    
    class DeleteUserResponse(Mapping):
        pass
    
    class DeregisterFromWorkMailRequest(Mapping):
        pass
    
    class DeregisterFromWorkMailResponse(Mapping):
        pass
    
    class DescribeGroupRequest(Mapping):
        pass
    
    class DescribeGroupResponse(Mapping):
        pass
    
    class DescribeOrganizationRequest(Mapping):
        pass
    
    class DescribeOrganizationResponse(Mapping):
        pass
    
    class DescribeResourceRequest(Mapping):
        pass
    
    class DescribeResourceResponse(Mapping):
        pass
    
    class DescribeUserRequest(Mapping):
        pass
    
    class DescribeUserResponse(Mapping):
        pass
    
    class DisassociateDelegateFromResourceRequest(Mapping):
        pass
    
    class DisassociateDelegateFromResourceResponse(Mapping):
        pass
    
    class DisassociateMemberFromGroupRequest(Mapping):
        pass
    
    class DisassociateMemberFromGroupResponse(Mapping):
        pass
    
    class GetMailboxDetailsRequest(Mapping):
        pass
    
    class GetMailboxDetailsResponse(Mapping):
        pass
    
    class ListAliasesRequest(Mapping):
        pass
    
    class ListAliasesResponse(Mapping):
        pass
    
    class ListGroupMembersRequest(Mapping):
        pass
    
    class ListGroupMembersResponse(Mapping):
        pass
    
    class ListGroupsRequest(Mapping):
        pass
    
    class ListGroupsResponse(Mapping):
        pass
    
    class ListMailboxPermissionsRequest(Mapping):
        pass
    
    class ListMailboxPermissionsResponse(Mapping):
        pass
    
    class ListOrganizationsRequest(Mapping):
        pass
    
    class ListOrganizationsResponse(Mapping):
        pass
    
    class ListResourceDelegatesRequest(Mapping):
        pass
    
    class ListResourceDelegatesResponse(Mapping):
        pass
    
    class ListResourcesRequest(Mapping):
        pass
    
    class ListResourcesResponse(Mapping):
        pass
    
    class ListUsersRequest(Mapping):
        pass
    
    class ListUsersResponse(Mapping):
        pass
    
    class PutMailboxPermissionsRequest(Mapping):
        pass
    
    class PutMailboxPermissionsResponse(Mapping):
        pass
    
    class PermissionType(object):
        pass
    
    class RegisterToWorkMailRequest(Mapping):
        pass
    
    class RegisterToWorkMailResponse(Mapping):
        pass
    
    class ResetPasswordRequest(Mapping):
        pass
    
    class ResetPasswordResponse(Mapping):
        pass
    
    class UpdateMailboxQuotaRequest(Mapping):
        pass
    
    class UpdateMailboxQuotaResponse(Mapping):
        pass
    
    class UpdatePrimaryEmailAddressRequest(Mapping):
        pass
    
    class UpdatePrimaryEmailAddressResponse(Mapping):
        pass
    
    class UpdateResourceRequest(Mapping):
        pass
    
    class UpdateResourceResponse(Mapping):
        pass
    
    class BookingOptions(Mapping):
        pass
    
    def associate_delegate_to_resource(self, EntityId: str, ResourceId: str, OrganizationId: str) -> AssociateDelegateToResourceResponse:
        """Adds a member (user or group) to the resource's set of delegates.

        :param self:
        :param EntityId:  The member (user or group) to associate to the resource.<br/>ResourceId:  The resource for which members (users or groups) are associated.<br/>OrganizationId:  The organization under which the resource exists.<br/>
        :return:         """
        pass

    def associate_member_to_group(self, MemberId: str, GroupId: str, OrganizationId: str) -> AssociateMemberToGroupResponse:
        """Adds a member (user or group) to the group's set.

        :param self:
        :param MemberId:  The member (user or group) to associate to the group.<br/>GroupId:  The group to which the member (user or group) is associated.<br/>OrganizationId:  The organization under which the group exists.<br/>
        :return:         """
        pass

    def create_alias(self, Alias: str, EntityId: str, OrganizationId: str) -> CreateAliasResponse:
        """Adds an alias to the set of a given member (user or group) of Amazon WorkMail.

        :param self:
        :param Alias:  The alias to add to the member set.<br/>EntityId:  The member (user or group) to which this alias is added.<br/>OrganizationId:  The organization under which the member (user or group) exists.<br/>
        :return:         """
        pass

    def create_group(self, Name: str, OrganizationId: str) -> CreateGroupResponse:
        """Creates a group that can be used in Amazon WorkMail by calling the <a>RegisterToWorkMail</a> operation.

        :param self:
        :param Name:  The name of the group.<br/>OrganizationId:  The organization under which the group is to be created.<br/>
        :return: GroupId:  The identifier of the group.<br/>        """
        pass

    def create_resource(self, Type: str, Name: str, OrganizationId: str) -> CreateResourceResponse:
        """Creates a new Amazon WorkMail resource. 

        :param self:
        :param Type:  The type of the new resource. The available types are <code>equipment</code> and <code>room</code>.<br/>Name:  The name of the new resource.<br/>OrganizationId:  The identifier associated with the organization for which the resource is created.<br/>
        :return: ResourceId:  The identifier of the new resource.<br/>        """
        pass

    def create_user(self, Password: str, DisplayName: str, Name: str, OrganizationId: str) -> CreateUserResponse:
        """Creates a user who can be used in Amazon WorkMail by calling the <a>RegisterToWorkMail</a> operation.

        :param self:
        :param Password:  The password for the new user.<br/>DisplayName:  The display name for the new user.<br/>Name:  The name for the new user. Simple AD or AD Connector user names have a maximum length of 20. All others have a maximum length of 64.<br/>OrganizationId:  The identifier of the organization for which the user is created.<br/>
        :return: UserId:  The identifier for the new user.<br/>        """
        pass

    def delete_alias(self, Alias: str, EntityId: str, OrganizationId: str) -> DeleteAliasResponse:
        """Remove one or more specified aliases from a set of aliases for a given user.

        :param self:
        :param Alias:  The aliases to be removed from the user's set of aliases. Duplicate entries in the list are collapsed into single entries (the list is transformed into a set).<br/>EntityId:  The identifier for the member (user or group) from which to have the aliases removed.<br/>OrganizationId:  The identifier for the organization under which the user exists.<br/>
        :return:         """
        pass

    def delete_group(self, GroupId: str, OrganizationId: str) -> DeleteGroupResponse:
        """Deletes a group from Amazon WorkMail.

        :param self:
        :param GroupId:  The identifier of the group to be deleted.<br/>OrganizationId:  The organization that contains the group.<br/>
        :return:         """
        pass

    def delete_mailbox_permissions(self, GranteeId: str, EntityId: str, OrganizationId: str) -> DeleteMailboxPermissionsResponse:
        """Deletes permissions granted to a member (user or group).

        :param self:
        :param GranteeId:  The identifier of the member (user or group) for which to delete granted permissions.<br/>EntityId:  The identifier of the member (user or group)that owns the mailbox.<br/>OrganizationId:  The identifier of the organization under which the member (user or group) exists.<br/>
        :return:         """
        pass

    def delete_resource(self, ResourceId: str, OrganizationId: str) -> DeleteResourceResponse:
        """Deletes the specified resource. 

        :param self:
        :param ResourceId:  The identifier of the resource to be deleted.<br/>OrganizationId:  The identifier associated with the organization from which the resource is deleted.<br/>
        :return:         """
        pass

    def delete_user(self, UserId: str, OrganizationId: str) -> DeleteUserResponse:
        """Deletes a user from Amazon WorkMail and all subsequent systems. Before you can delete a user, the user state must be <code>DISABLED</code>. Use the <a>DescribeUser</a> action to confirm the user state. Deleting a user is permanent and cannot be undone. WorkMail archives user mailboxes for 30 days before they are permanently removed.

        :param self:
        :param UserId:  The identifier of the user to be deleted.<br/>OrganizationId:  The organization that contains the user to be deleted.<br/>
        :return:         """
        pass

    def deregister_from_work_mail(self, EntityId: str, OrganizationId: str) -> DeregisterFromWorkMailResponse:
        """Mark a user, group, or resource as no longer used in Amazon WorkMail. This action disassociates the mailbox and schedules it for clean-up. WorkMail keeps mailboxes for 30 days before they are permanently removed. The functionality in the console is <i>Disable</i>.

        :param self:
        :param EntityId:  The identifier for the member (user or group) to be updated.<br/>OrganizationId:  The identifier for the organization under which the Amazon WorkMail entity exists.<br/>
        :return:         """
        pass

    def describe_group(self, GroupId: str, OrganizationId: str) -> DescribeGroupResponse:
        """Returns the data available for the group.

        :param self:
        :param GroupId:  The identifier for the group to be described.<br/>OrganizationId:  The identifier for the organization under which the group exists.<br/>
        :return: DisabledDate:  The date and time when a user was deregistered from WorkMail, in UNIX epoch time format.<br/>EnabledDate:  The date and time when a user was registered to WorkMail, in UNIX epoch time format.<br/>State:  The state of the user: enabled (registered to Amazon WorkMail) or disabled (deregistered or never registered to WorkMail).<br/>Email:  The email of the described group.<br/>Name:  The name of the described group.<br/>GroupId:  The identifier of the described group.<br/>        """
        pass

    def describe_organization(self, OrganizationId: str) -> DescribeOrganizationResponse:
        """Provides more information regarding a given organization based on its identifier.

        :param self:
        :param OrganizationId:  The identifier for the organization to be described.<br/>
        :return: ErrorMessage:  (Optional) The error message indicating if unexpected behavior was encountered with regards to the organization.<br/>CompletedDate:  The date at which the organization became usable in the WorkMail context, in UNIX epoch time format.<br/>DefaultMailDomain:  The default mail domain associated with the organization.<br/>DirectoryType:  The type of directory associated with the WorkMail organization.<br/>DirectoryId:  The identifier for the directory associated with an Amazon WorkMail organization.<br/>State:  The state of an organization.<br/>Alias:  The alias for an organization.<br/>OrganizationId:  The identifier of an organization.<br/>        """
        pass

    def describe_resource(self, ResourceId: str, OrganizationId: str) -> DescribeResourceResponse:
        """Returns the data available for the resource.

        :param self:
        :param ResourceId:  The identifier of the resource to be described.<br/>OrganizationId:  The identifier associated with the organization for which the resource is described.<br/>
        :return: DisabledDate:  The date and time when a resource was disabled from WorkMail, in UNIX epoch time format.<br/>EnabledDate:  The date and time when a resource was enabled for WorkMail, in UNIX epoch time format.<br/>State:  The state of the resource: enabled (registered to Amazon WorkMail) or disabled (deregistered or never registered to WorkMail).<br/>BookingOptions:  The booking options for the described resource.:<br/>Type:  The type of the described resource.<br/>Name:  The name of the described resource.<br/>Email:  The email of the described resource.<br/>ResourceId:  The identifier of the described resource.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoDeclineConflictingRequests</i> The resource's ability to automatically decline any conflicting requests.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoDeclineRecurringRequests</i> The resource's ability to automatically decline any recurring requests.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoAcceptRequests</i> The resource's ability to automatically reply to requests. If disabled, delegates must be associated to the resource.<br/>        """
        pass

    def describe_user(self, UserId: str, OrganizationId: str) -> DescribeUserResponse:
        """Provides information regarding the user.

        :param self:
        :param UserId:  The identifier for the user to be described.<br/>OrganizationId:  The identifier for the organization under which the user exists.<br/>
        :return: DisabledDate:  The date and time at which the user was disabled for Amazon WorkMail usage, in UNIX epoch time format.<br/>EnabledDate:  The date and time at which the user was enabled for Amazon WorkMail usage, in UNIX epoch time format.<br/>UserRole:  In certain cases, other entities are modeled as users. If interoperability is enabled, resources are imported into Amazon WorkMail as users. Because different WorkMail organizations rely on different directory types, administrators can distinguish between an unregistered user (account is disabled and has a user role) and the directory administrators. The values are USER, RESOURCE, and SYSTEM_USER.<br/>State:  The state of a user: enabled (registered to Amazon WorkMail) or disabled (deregistered or never registered to WorkMail).<br/>DisplayName:  The display name of the user.<br/>Email:  The email of the user.<br/>Name:  The name for the user.<br/>UserId:  The identifier for the described user.<br/>        """
        pass

    def disassociate_delegate_from_resource(self, EntityId: str, ResourceId: str, OrganizationId: str) -> DisassociateDelegateFromResourceResponse:
        """Removes a member from the resource's set of delegates.

        :param self:
        :param EntityId:  The identifier for the member (user, group) to be removed from the resource's delegates.<br/>ResourceId:  The identifier of the resource from which delegates' set members are removed. <br/>OrganizationId:  The identifier for the organization under which the resource exists.<br/>
        :return:         """
        pass

    def disassociate_member_from_group(self, MemberId: str, GroupId: str, OrganizationId: str) -> DisassociateMemberFromGroupResponse:
        """Removes a member from a group.

        :param self:
        :param MemberId:  The identifier for the member to be removed to the group.<br/>GroupId:  The identifier for the group from which members are removed.<br/>OrganizationId:  The identifier for the organization under which the group exists.<br/>
        :return:         """
        pass

    def get_mailbox_details(self, UserId: str, OrganizationId: str) -> GetMailboxDetailsResponse:
        """Requests a user's mailbox details for a specified organization and user.

        :param self:
        :param UserId:  The identifier for the user whose mailbox details are being requested.<br/>OrganizationId:  The identifier for the organization that contains the user whose mailbox details are being requested.<br/>
        :return: MailboxSize:  The current mailbox size, in MB, for the specified user.<br/>MailboxQuota:  The maximum allowed mailbox size, in MB, for the specified user.<br/>        """
        pass

    def list_aliases(self, EntityId: str, OrganizationId: str, NextToken: str=None, MaxResults: int=None) -> ListAliasesResponse:
        """Creates a paginated call to list the aliases associated with a given entity.

        :param self:
        :param MaxResults:  The maximum number of results to return in a single call.<br/>NextToken:  The token to use to retrieve the next page of results. The first call does not contain any tokens.<br/>EntityId:  The identifier for the entity for which to list the aliases.<br/>OrganizationId:  The identifier for the organization under which the entity exists.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. The value is "null" when there are no more results to return.<br/>Aliases:  The entity's paginated aliases.<br/>        """
        pass

    def list_group_members(self, GroupId: str, OrganizationId: str, NextToken: str=None, MaxResults: int=None) -> ListGroupMembersResponse:
        """Returns an overview of the members of a group. Users and groups can be members of a group.

        :param self:
        :param MaxResults:  The maximum number of results to return in a single call.<br/>NextToken:   The token to use to retrieve the next page of results. The first call does not contain any tokens.<br/>GroupId:  The identifier for the group to which the members (users or groups) are associated.<br/>OrganizationId:  The identifier for the organization under which the group exists.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. The first call does not contain any tokens.<br/>Members:  The members associated to the group.<br/>        """
        pass

    def list_groups(self, OrganizationId: str, NextToken: str=None, MaxResults: int=None) -> ListGroupsResponse:
        """Returns summaries of the organization's groups.

        :param self:
        :param MaxResults:  The maximum number of results to return in a single call.<br/>NextToken:  The token to use to retrieve the next page of results. The first call does not contain any tokens.<br/>OrganizationId:  The identifier for the organization under which the groups exist.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. The value is "null" when there are no more results to return.<br/>Groups:  The overview of groups for an organization.<br/>        """
        pass

    def list_mailbox_permissions(self, EntityId: str, OrganizationId: str, NextToken: str=None, MaxResults: int=None) -> ListMailboxPermissionsResponse:
        """Lists the mailbox permissions associated with a user, group, or resource mailbox.

        :param self:
        :param MaxResults:  The maximum number of results to return in a single call.<br/>NextToken:  The token to use to retrieve the next page of results. The first call does not contain any tokens.<br/>EntityId:  The identifier of the user, group, or resource for which to list mailbox permissions.<br/>OrganizationId:  The identifier of the organization under which the user, group, or resource exists.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. The value is "null" when there are no more results to return.<br/>Permissions:  One page of the user, group, or resource mailbox permissions.<br/>        """
        pass

    def list_organizations(self, NextToken: str=None, MaxResults: int=None) -> ListOrganizationsResponse:
        """Returns summaries of the customer's non-deleted organizations.

        :param self:
        :param MaxResults:  The maximum number of results to return in a single call.<br/>NextToken:  The token to use to retrieve the next page of results. The first call does not contain any tokens.<br/>
        :return: NextToken:  The token to use to retrieve the next page of results. The value is "null" when there are no more results to return.<br/>OrganizationSummaries:  The overview of owned organizations presented as a list of organization summaries.<br/>        """
        pass

    def list_resource_delegates(self, ResourceId: str, OrganizationId: str, NextToken: str=None, MaxResults: int=None) -> ListResourceDelegatesResponse:
        """Lists the delegates associated with a resource. Users and groups can be resource delegates and answer requests on behalf of the resource.

        :param self:
        :param MaxResults:  The number of maximum results in a page.<br/>NextToken:  The token used to paginate through the delegates associated with a resource.<br/>ResourceId:  The identifier for the resource whose delegates are listed.<br/>OrganizationId:  The identifier for the organization that contains the resource for which delegates are listed.<br/>
        :return: NextToken:  The token used to paginate through the delegates associated with a resource. While results are still available, it has an associated value. When the last page is reached, the token is empty. <br/>Delegates:  One page of the resource's delegates.<br/>        """
        pass

    def list_resources(self, OrganizationId: str, NextToken: str=None, MaxResults: int=None) -> ListResourcesResponse:
        """Returns summaries of the organization's resources.

        :param self:
        :param MaxResults:  The maximum number of results to return in a single call.<br/>NextToken:  The token to use to retrieve the next page of results. The first call does not contain any tokens.<br/>OrganizationId:  The identifier for the organization under which the resources exist.<br/>
        :return: NextToken:   The token used to paginate through all the organization's resources. While results are still available, it has an associated value. When the last page is reached, the token is empty.<br/>Resources:  One page of the organization's resource representation.<br/>        """
        pass

    def list_users(self, OrganizationId: str, NextToken: str=None, MaxResults: int=None) -> ListUsersResponse:
        """Returns summaries of the organization's users.

        :param self:
        :param MaxResults:  The maximum number of results to return in a single call.<br/>NextToken:  The token to use to retrieve the next page of results. The first call does not contain any tokens.<br/>OrganizationId:  The identifier for the organization under which the users exist.<br/>
        :return: NextToken:   The token to use to retrieve the next page of results. This value is `null` when there are no more results to return.<br/>Users:  The overview of users for an organization.<br/>        """
        pass

    def put_mailbox_permissions(self, PermissionValues: List[PermissionType], GranteeId: str, EntityId: str, OrganizationId: str) -> PutMailboxPermissionsResponse:
        """Sets permissions for a user, group, or resource. This replaces any pre-existing permissions.

        :param self:
        :param PermissionValues:  The permissions granted to the grantee. SEND_AS allows the grantee to send email as the owner of the mailbox (the grantee is not mentioned on these emails). SEND_ON_BEHALF allows the grantee to send email on behalf of the owner of the mailbox (the grantee is not mentioned as the physical sender of these emails). FULL_ACCESS allows the grantee full access to the mailbox, irrespective of other folder-level permissions set on the mailbox.<br/>GranteeId:  The identifier of the user, group, or resource to which to grant the permissions.<br/>EntityId:  The identifier of the user, group, or resource for which to update mailbox permissions.<br/>OrganizationId:  The identifier of the organization under which the user, group, or resource exists.<br/>
        :return:         """
        pass

    def register_to_work_mail(self, Email: str, EntityId: str, OrganizationId: str) -> RegisterToWorkMailResponse:
        """Registers an existing and disabled user, group, or resource for Amazon WorkMail use by associating a mailbox and calendaring capabilities. It performs no change if the user, group, or resource is enabled and fails if the user, group, or resource is deleted. This operation results in the accumulation of costs. For more information, see <a href="https://aws.amazon.com//workmail/pricing">Pricing</a>. The equivalent console functionality for this operation is <i>Enable</i>.  Users can either be created by calling the <a>CreateUser</a> API operation or they can be synchronized from your directory. For more information, see <a>DeregisterFromWorkMail</a>.

        :param self:
        :param Email:  The email for the user, group, or resource to be updated.<br/>EntityId:  The identifier for the user, group, or resource to be updated.<br/>OrganizationId:  The identifier for the organization under which the user, group, or resource exists.<br/>
        :return:         """
        pass

    def reset_password(self, Password: str, UserId: str, OrganizationId: str) -> ResetPasswordResponse:
        """Allows the administrator to reset the password for a user.

        :param self:
        :param Password:  The new password for the user.<br/>UserId:  The identifier of the user for whom the password is reset.<br/>OrganizationId:  The identifier of the organization that contains the user for which the password is reset.<br/>
        :return:         """
        pass

    def update_mailbox_quota(self, MailboxQuota: int, UserId: str, OrganizationId: str) -> UpdateMailboxQuotaResponse:
        """Updates a user's current mailbox quota for a specified organization and user.

        :param self:
        :param MailboxQuota:  The updated mailbox quota, in MB, for the specified user.<br/>UserId:  The identifer for the user for whom to update the mailbox quota.<br/>OrganizationId:  The identifier for the organization that contains the user for whom to update the mailbox quota.<br/>
        :return:         """
        pass

    def update_primary_email_address(self, Email: str, EntityId: str, OrganizationId: str) -> UpdatePrimaryEmailAddressResponse:
        """Updates the primary email for a user, group, or resource. The current email is moved into the list of aliases (or swapped between an existing alias and the current primary email), and the email provided in the input is promoted as the primary.

        :param self:
        :param Email:  The value of the email to be updated as primary.<br/>EntityId:  The user, group, or resource to update.<br/>OrganizationId:  The organization that contains the user, group, or resource to update.<br/>
        :return:         """
        pass

    def update_resource(self, ResourceId: str, OrganizationId: str, Name: str=None, BookingOptions: BookingOptions=None) -> UpdateResourceResponse:
        """Updates data for the resource. To have the latest information, it must be preceded by a <a>DescribeResource</a> call. The dataset in the request should be the one expected when performing another <code>DescribeResource</code> call.

        :param self:
        :param BookingOptions:  The resource's booking options to be updated.:<br/>Name:  The name of the resource to be updated.<br/>ResourceId:  The identifier of the resource to be updated.<br/>OrganizationId:  The identifier associated with the organization for which the resource is updated.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoDeclineConflictingRequests</i> The resource's ability to automatically decline any conflicting requests.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoDeclineRecurringRequests</i> The resource's ability to automatically decline any recurring requests.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>AutoAcceptRequests</i> The resource's ability to automatically reply to requests. If disabled, delegates must be associated to the resource.<br/>
        :return:         """
        pass



class WorkSpaces(BaseClient):
    """<fullname>Amazon WorkSpaces Service</fullname> Amazon WorkSpaces enables you to provision virtual, cloud-based Microsoft Windows and Amazon Linux desktops for your users."""


    class DescribeWorkspaceBundlesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_workspace_bundles` for valid parameters"""
            pass
    class DescribeWorkspaceDirectoriesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_workspace_directories` for valid parameters"""
            pass
    class DescribeWorkspacesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_workspaces` for valid parameters"""
            pass
    class DescribeAccountModificationsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_account_modifications` for valid parameters"""
            pass
    class DescribeIpGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_ip_groups` for valid parameters"""
            pass
    class DescribeWorkspaceImagesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_workspace_images` for valid parameters"""
            pass
    class DescribeWorkspacesConnectionStatusPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `describe_workspaces_connection_status` for valid parameters"""
            pass
    class ListAvailableManagementCidrRangesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `list_available_management_cidr_ranges` for valid parameters"""
            pass


    class AssociateIpGroupsRequest(Mapping):
        pass
    
    class AssociateIpGroupsResult(Mapping):
        pass
    
    class IpGroupId(object):
        pass
    
    class AuthorizeIpRulesRequest(Mapping):
        pass
    
    class AuthorizeIpRulesResult(Mapping):
        pass
    
    class IpRuleItem(Mapping):
        pass
    
    class CopyWorkspaceImageRequest(Mapping):
        pass
    
    class CopyWorkspaceImageResult(Mapping):
        pass
    
    class Tag(Mapping):
        pass
    
    class CreateIpGroupRequest(Mapping):
        pass
    
    class CreateIpGroupResult(Mapping):
        pass
    
    class CreateTagsRequest(Mapping):
        pass
    
    class CreateTagsResult(Mapping):
        pass
    
    class CreateWorkspacesRequest(Mapping):
        pass
    
    class CreateWorkspacesResult(Mapping):
        pass
    
    class WorkspaceRequest(Mapping):
        pass
    
    class DeleteIpGroupRequest(Mapping):
        pass
    
    class DeleteIpGroupResult(Mapping):
        pass
    
    class DeleteTagsRequest(Mapping):
        pass
    
    class DeleteTagsResult(Mapping):
        pass
    
    class NonEmptyString(object):
        pass
    
    class DeleteWorkspaceImageRequest(Mapping):
        pass
    
    class DeleteWorkspaceImageResult(Mapping):
        pass
    
    class DescribeAccountRequest(Mapping):
        pass
    
    class DescribeAccountResult(Mapping):
        pass
    
    class DescribeAccountModificationsRequest(Mapping):
        pass
    
    class DescribeAccountModificationsResult(Mapping):
        pass
    
    class DescribeClientPropertiesRequest(Mapping):
        pass
    
    class DescribeClientPropertiesResult(Mapping):
        pass
    
    class DescribeIpGroupsRequest(Mapping):
        pass
    
    class DescribeIpGroupsResult(Mapping):
        pass
    
    class DescribeTagsRequest(Mapping):
        pass
    
    class DescribeTagsResult(Mapping):
        pass
    
    class DescribeWorkspaceBundlesRequest(Mapping):
        pass
    
    class DescribeWorkspaceBundlesResult(Mapping):
        pass
    
    class BundleId(object):
        pass
    
    class DescribeWorkspaceDirectoriesRequest(Mapping):
        pass
    
    class DescribeWorkspaceDirectoriesResult(Mapping):
        pass
    
    class DirectoryId(object):
        pass
    
    class DescribeWorkspaceImagesRequest(Mapping):
        pass
    
    class DescribeWorkspaceImagesResult(Mapping):
        pass
    
    class WorkspaceImageId(object):
        pass
    
    class DescribeWorkspacesRequest(Mapping):
        pass
    
    class DescribeWorkspacesResult(Mapping):
        pass
    
    class WorkspaceId(object):
        pass
    
    class DescribeWorkspacesConnectionStatusRequest(Mapping):
        pass
    
    class DescribeWorkspacesConnectionStatusResult(Mapping):
        pass
    
    class DisassociateIpGroupsRequest(Mapping):
        pass
    
    class DisassociateIpGroupsResult(Mapping):
        pass
    
    class ImportWorkspaceImageRequest(Mapping):
        pass
    
    class ImportWorkspaceImageResult(Mapping):
        pass
    
    class ListAvailableManagementCidrRangesRequest(Mapping):
        pass
    
    class ListAvailableManagementCidrRangesResult(Mapping):
        pass
    
    class ModifyAccountRequest(Mapping):
        pass
    
    class ModifyAccountResult(Mapping):
        pass
    
    class ModifyClientPropertiesRequest(Mapping):
        pass
    
    class ModifyClientPropertiesResult(Mapping):
        pass
    
    class ClientProperties(Mapping):
        pass
    
    class ModifyWorkspacePropertiesRequest(Mapping):
        pass
    
    class ModifyWorkspacePropertiesResult(Mapping):
        pass
    
    class WorkspaceProperties(Mapping):
        pass
    
    class ModifyWorkspaceStateRequest(Mapping):
        pass
    
    class ModifyWorkspaceStateResult(Mapping):
        pass
    
    class RebootWorkspacesRequest(Mapping):
        pass
    
    class RebootWorkspacesResult(Mapping):
        pass
    
    class RebootRequest(Mapping):
        pass
    
    class RebuildWorkspacesRequest(Mapping):
        pass
    
    class RebuildWorkspacesResult(Mapping):
        pass
    
    class RebuildRequest(Mapping):
        pass
    
    class RevokeIpRulesRequest(Mapping):
        pass
    
    class RevokeIpRulesResult(Mapping):
        pass
    
    class IpRule(object):
        pass
    
    class StartWorkspacesRequest(Mapping):
        pass
    
    class StartWorkspacesResult(Mapping):
        pass
    
    class StartRequest(Mapping):
        pass
    
    class StopWorkspacesRequest(Mapping):
        pass
    
    class StopWorkspacesResult(Mapping):
        pass
    
    class StopRequest(Mapping):
        pass
    
    class TerminateWorkspacesRequest(Mapping):
        pass
    
    class TerminateWorkspacesResult(Mapping):
        pass
    
    class TerminateRequest(Mapping):
        pass
    
    class UpdateRulesOfIpGroupRequest(Mapping):
        pass
    
    class UpdateRulesOfIpGroupResult(Mapping):
        pass
    
    def associate_ip_groups(self, GroupIds: List[IpGroupId], DirectoryId: str) -> AssociateIpGroupsResult:
        """Associates the specified IP access control group with the specified directory.

        :param self:
        :param GroupIds:  The identifiers of one or more IP access control groups.<br/>DirectoryId:  The identifier of the directory.<br/>
        :return:         """
        pass

    def authorize_ip_rules(self, UserRules: List[IpRuleItem], GroupId: str) -> AuthorizeIpRulesResult:
        """Adds one or more rules to the specified IP access control group. This action gives users permission to access their WorkSpaces from the CIDR address ranges specified in the rules.

        :param self:
        :param UserRules:  The rules to add to the group.<br/>GroupId:  The identifier of the group.<br/>
        :return:         """
        pass

    def copy_workspace_image(self, SourceRegion: str, SourceImageId: str, Name: str, Description: str=None, Tags: List[Tag]=None) -> CopyWorkspaceImageResult:
        """Copies the specified image from the specified Region to the current Region.

        :param self:
        :param Tags:  The tags for the image.<br/>SourceRegion:  The identifier of the source Region.<br/>SourceImageId:  The identifier of the source image.<br/>Description:  A description of the image.<br/>Name:  The name of the image.<br/>
        :return: ImageId:  The identifier of the image.<br/>        """
        pass

    def create_ip_group(self, GroupName: str, GroupDesc: str=None, UserRules: List[IpRuleItem]=None, Tags: List[Tag]=None) -> CreateIpGroupResult:
        """Creates an IP access control group. An IP access control group provides you with the ability to control the IP addresses from which users are allowed to access their WorkSpaces. To specify the CIDR address ranges, add rules to your IP access control group and then associate the group with your directory. You can add rules when you create the group or at any time using <a>AuthorizeIpRules</a>. There is a default IP access control group associated with your directory. If you don't associate an IP access control group with your directory, the default group is used. The default group includes a default rule that allows users to access their WorkSpaces from anywhere. You cannot modify the default IP access control group for your directory.

        :param self:
        :param Tags:  The tags. Each WorkSpaces resource can have a maximum of 50 tags.<br/>UserRules:  The rules to add to the group.<br/>GroupDesc:  The description of the group.<br/>GroupName:  The name of the group.<br/>
        :return: GroupId:  The identifier of the group.<br/>        """
        pass

    def create_tags(self, Tags: List[Tag], ResourceId: str) -> CreateTagsResult:
        """Creates the specified tags for the specified WorkSpaces resource.

        :param self:
        :param Tags:  The tags. Each WorkSpaces resource can have a maximum of 50 tags.<br/>ResourceId:  The identifier of the WorkSpaces resource. The supported resource types are WorkSpaces, registered directories, images, custom bundles, and IP access control groups.<br/>
        :return:         """
        pass

    def create_workspaces(self, Workspaces: List[WorkspaceRequest]) -> CreateWorkspacesResult:
        """Creates one or more WorkSpaces. This operation is asynchronous and returns before the WorkSpaces are created.

        :param self:
        :param Workspaces:  The WorkSpaces to create. You can specify up to 25 WorkSpaces.<br/>
        :return: PendingRequests:  Information about the WorkSpaces that were created. Because this operation is asynchronous, the identifier returned is not immediately available for use with other operations. For example, if you call <a>DescribeWorkspaces</a> before the WorkSpace is created, the information returned can be incomplete.<br/>FailedRequests:  Information about the WorkSpaces that could not be created.<br/>        """
        pass

    def delete_ip_group(self, GroupId: str) -> DeleteIpGroupResult:
        """Deletes the specified IP access control group. You cannot delete an IP access control group that is associated with a directory.

        :param self:
        :param GroupId:  The identifier of the IP access control group.<br/>
        :return:         """
        pass

    def delete_tags(self, TagKeys: List[NonEmptyString], ResourceId: str) -> DeleteTagsResult:
        """Deletes the specified tags from the specified WorkSpaces resource.

        :param self:
        :param TagKeys:  The tag keys.<br/>ResourceId:  The identifier of the WorkSpaces resource. The supported resource types are WorkSpaces, registered directories, images, custom bundles, and IP access control groups.<br/>
        :return:         """
        pass

    def delete_workspace_image(self, ImageId: str) -> DeleteWorkspaceImageResult:
        """Deletes the specified image from your account. To delete an image, you must first delete any bundles that are associated with the image and un-share the image if it is shared with other accounts. 

        :param self:
        :param ImageId:  The identifier of the image.<br/>
        :return:         """
        pass

    def describe_account(self) -> DescribeAccountResult:
        """Retrieves a list that describes the configuration of bring your own license (BYOL) for the specified account.

        :param self:
        :param 
        :return: DedicatedTenancyManagementCidrRange:  The IP address range, specified as an IPv4 CIDR block, used for the management network interface. The management network interface is connected to a secure Amazon WorkSpaces management network. It is used for interactive streaming of the WorkSpace desktop to Amazon WorkSpaces clients, and to allow Amazon WorkSpaces to manage the WorkSpace.<br/>DedicatedTenancySupport:  The status of BYOL (whether BYOL is enabled or disabled).<br/>        """
        pass

    def describe_account_modifications(self, NextToken: str=None) -> DescribeAccountModificationsResult:
        """Retrieves a list that describes modifications to the configuration of bring your own license (BYOL) for the specified account.

        :param self:
        :param NextToken:  If you received a <code>NextToken</code> from a previous call that was paginated, provide this token to receive the next set of results.<br/>
        :return: NextToken:  The token to use to retrieve the next set of results, or null if no more results are available.<br/>AccountModifications:  The list of modifications to the configuration of BYOL.<br/>        """
        pass

    def describe_client_properties(self, ResourceIds: List[NonEmptyString]) -> DescribeClientPropertiesResult:
        """Retrieves a list that describes one or more specified Amazon WorkSpaces clients.

        :param self:
        :param ResourceIds:  The resource identifier, in the form of directory IDs.<br/>
        :return: ClientPropertiesList:  Information about the specified Amazon WorkSpaces clients.<br/>        """
        pass

    def describe_ip_groups(self, GroupIds: List[IpGroupId]=None, NextToken: str=None, MaxResults: int=None) -> DescribeIpGroupsResult:
        """Describes one or more of your IP access control groups.

        :param self:
        :param MaxResults:  The maximum number of items to return.<br/>NextToken:  If you received a <code>NextToken</code> from a previous call that was paginated, provide this token to receive the next set of results.<br/>GroupIds:  The identifiers of one or more IP access control groups.<br/>
        :return: NextToken:  The token to use to retrieve the next set of results, or null if no more results are available.<br/>Result:  Information about the IP access control groups.<br/>        """
        pass

    def describe_tags(self, ResourceId: str) -> DescribeTagsResult:
        """Describes the specified tags for the specified WorkSpaces resource.

        :param self:
        :param ResourceId:  The identifier of the WorkSpaces resource. The supported resource types are WorkSpaces, registered directories, images, custom bundles, and IP access control groups.<br/>
        :return: TagList:  The tags.<br/>        """
        pass

    def describe_workspace_bundles(self, BundleIds: List[BundleId]=None, Owner: str=None, NextToken: str=None) -> DescribeWorkspaceBundlesResult:
        """Retrieves a list that describes the available WorkSpace bundles. You can filter the results using either bundle ID or owner, but not both.

        :param self:
        :param NextToken:  The token for the next set of results. (You received this token from a previous call.)<br/>Owner:  The owner of the bundles. You cannot combine this parameter with any other filter. Specify <code>AMAZON</code> to describe the bundles provided by AWS or null to describe the bundles that belong to your account.<br/>BundleIds:  The identifiers of the bundles. You cannot combine this parameter with any other filter.<br/>
        :return: NextToken:  The token to use to retrieve the next set of results, or null if there are no more results available. This token is valid for one day and must be used within that time frame.<br/>Bundles:  Information about the bundles.<br/>        """
        pass

    def describe_workspace_directories(self, DirectoryIds: List[DirectoryId]=None, NextToken: str=None) -> DescribeWorkspaceDirectoriesResult:
        """Describes the available AWS Directory Service directories that are registered with Amazon WorkSpaces.

        :param self:
        :param NextToken:  If you received a <code>NextToken</code> from a previous call that was paginated, provide this token to receive the next set of results.<br/>DirectoryIds:  The identifiers of the directories. If the value is null, all directories are retrieved.<br/>
        :return: NextToken:  The token to use to retrieve the next set of results, or null if no more results are available.<br/>Directories:  Information about the directories.<br/>        """
        pass

    def describe_workspace_images(self, ImageIds: List[WorkspaceImageId]=None, NextToken: str=None, MaxResults: int=None) -> DescribeWorkspaceImagesResult:
        """Retrieves a list that describes one or more specified images, if the image identifiers are provided. Otherwise, all images in the account are described. 

        :param self:
        :param MaxResults:  The maximum number of items to return.<br/>NextToken:  If you received a <code>NextToken</code> from a previous call that was paginated, provide this token to receive the next set of results.<br/>ImageIds:  The identifier of the image.<br/>
        :return: NextToken:  The token to use to retrieve the next set of results, or null if no more results are available.<br/>Images:  Information about the images.<br/>        """
        pass

    def describe_workspaces(self, WorkspaceIds: List[WorkspaceId]=None, DirectoryId: str=None, UserName: str=None, BundleId: str=None, Limit: int=None, NextToken: str=None) -> DescribeWorkspacesResult:
        """Describes the specified WorkSpaces. You can filter the results by using the bundle identifier, directory identifier, or owner, but you can specify only one filter at a time.

        :param self:
        :param NextToken:  If you received a <code>NextToken</code> from a previous call that was paginated, provide this token to receive the next set of results.<br/>Limit:  The maximum number of items to return.<br/>BundleId:  The identifier of the bundle. All WorkSpaces that are created from this bundle are retrieved. You cannot combine this parameter with any other filter.<br/>UserName:  The name of the directory user. You must specify this parameter with <code>DirectoryId</code>.<br/>DirectoryId:  The identifier of the directory. In addition, you can optionally specify a specific directory user (see <code>UserName</code>). You cannot combine this parameter with any other filter.<br/>WorkspaceIds:  The identifiers of the WorkSpaces. You cannot combine this parameter with any other filter. Because the <a>CreateWorkspaces</a> operation is asynchronous, the identifier it returns is not immediately available. If you immediately call <a>DescribeWorkspaces</a> with this identifier, no information is returned.<br/>
        :return: NextToken:  The token to use to retrieve the next set of results, or null if no more results are available.<br/>Workspaces:  Information about the WorkSpaces. Because <a>CreateWorkspaces</a> is an asynchronous operation, some of the returned information could be incomplete.<br/>        """
        pass

    def describe_workspaces_connection_status(self, WorkspaceIds: List[WorkspaceId]=None, NextToken: str=None) -> DescribeWorkspacesConnectionStatusResult:
        """Describes the connection status of the specified WorkSpaces.

        :param self:
        :param NextToken:  If you received a <code>NextToken</code> from a previous call that was paginated, provide this token to receive the next set of results.<br/>WorkspaceIds:  The identifiers of the WorkSpaces. You can specify up to 25 WorkSpaces.<br/>
        :return: NextToken:  The token to use to retrieve the next set of results, or null if no more results are available.<br/>WorkspacesConnectionStatus:  Information about the connection status of the WorkSpace.<br/>        """
        pass

    def disassociate_ip_groups(self, GroupIds: List[IpGroupId], DirectoryId: str) -> DisassociateIpGroupsResult:
        """Disassociates the specified IP access control group from the specified directory.

        :param self:
        :param GroupIds:  The identifiers of one or more IP access control groups.<br/>DirectoryId:  The identifier of the directory.<br/>
        :return:         """
        pass

    def import_workspace_image(self, ImageDescription: str, ImageName: str, IngestionProcess: str, Ec2ImageId: str, Tags: List[Tag]=None) -> ImportWorkspaceImageResult:
        """Imports the specified Windows 7 or Windows 10 bring your own license (BYOL) image into Amazon WorkSpaces. The image must be an already licensed EC2 image that is in your AWS account, and you must own the image. 

        :param self:
        :param Tags:  The tags. Each WorkSpaces resource can have a maximum of 50 tags.<br/>ImageDescription:  The description of the WorkSpace image.<br/>ImageName:  The name of the WorkSpace image.<br/>IngestionProcess:  The ingestion process to be used when importing the image.<br/>Ec2ImageId:  The identifier of the EC2 image.<br/>
        :return: ImageId:  The identifier of the WorkSpace image.<br/>        """
        pass

    def list_available_management_cidr_ranges(self, ManagementCidrRangeConstraint: str, MaxResults: int=None, NextToken: str=None) -> ListAvailableManagementCidrRangesResult:
        """Retrieves a list of IP address ranges, specified as IPv4 CIDR blocks, that you can use for the network management interface when you enable bring your own license (BYOL).  The management network interface is connected to a secure Amazon WorkSpaces management network. It is used for interactive streaming of the WorkSpace desktop to Amazon WorkSpaces clients, and to allow Amazon WorkSpaces to manage the WorkSpace.

        :param self:
        :param NextToken:  If you received a <code>NextToken</code> from a previous call that was paginated, provide this token to receive the next set of results.<br/>MaxResults:  The maximum number of items to return.<br/>ManagementCidrRangeConstraint:  The IP address range to search. Specify an IP address range that is compatible with your network and in CIDR notation (that is, specify the range as an IPv4 CIDR block).<br/>
        :return: NextToken:  The token to use to retrieve the next set of results, or null if no more results are available.<br/>ManagementCidrRanges:  The list of available IP address ranges, specified as IPv4 CIDR blocks.<br/>        """
        pass

    def modify_account(self, DedicatedTenancySupport: str=None, DedicatedTenancyManagementCidrRange: str=None) -> ModifyAccountResult:
        """Modifies the configuration of bring your own license (BYOL) for the specified account.

        :param self:
        :param DedicatedTenancyManagementCidrRange:  The IP address range, specified as an IPv4 CIDR block, for the management network interface. Specify an IP address range that is compatible with your network and in CIDR notation (that is, specify the range as an IPv4 CIDR block). The CIDR block size must be /16 (for example, 203.0.113.25/16). It must also be specified as available by the <code>ListAvailableManagementCidrRanges</code> operation.<br/>DedicatedTenancySupport:  The status of BYOL.<br/>
        :return:         """
        pass

    def modify_client_properties(self, ClientProperties: ClientProperties, ResourceId: str) -> ModifyClientPropertiesResult:
        """Modifies the properties of the specified Amazon WorkSpaces clients.

        :param self:
        :param ClientProperties:  Information about the Amazon WorkSpaces client.:<br/>ResourceId:  The resource identifiers, in the form of directory IDs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReconnectEnabled</i> Specifies whether users can cache their credentials on the Amazon WorkSpaces client. When enabled, users can choose to reconnect to their WorkSpaces without re-entering their credentials. <br/>
        :return:         """
        pass

    def modify_workspace_properties(self, WorkspaceProperties: WorkspaceProperties, WorkspaceId: str) -> ModifyWorkspacePropertiesResult:
        """Modifies the specified WorkSpace properties.

        :param self:
        :param WorkspaceProperties:  The properties of the WorkSpace.:<br/>WorkspaceId:  The identifier of the WorkSpace.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ComputeTypeName</i> The compute type. For more information, see <a href="http://aws.amazon.com/workspaces/details/#Amazon_WorkSpaces_Bundles">Amazon WorkSpaces Bundles</a>.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>UserVolumeSizeGib</i> The size of the user storage.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RootVolumeSizeGib</i> The size of the root volume.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RunningModeAutoStopTimeoutInMinutes</i> The time after a user logs off when WorkSpaces are automatically stopped. Configured in 60 minute intervals.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RunningMode</i> The running mode. For more information, see <a href="https://docs.aws.amazon.com/workspaces/latest/adminguide/running-mode.html">Manage the WorkSpace Running Mode</a>.<br/>
        :return:         """
        pass

    def modify_workspace_state(self, WorkspaceState: str, WorkspaceId: str) -> ModifyWorkspaceStateResult:
        """Sets the state of the specified WorkSpace. To maintain a WorkSpace without being interrupted, set the WorkSpace state to <code>ADMIN_MAINTENANCE</code>. WorkSpaces in this state do not respond to requests to reboot, stop, start, rebuild, or restore. An AutoStop WorkSpace in this state is not stopped. Users cannot log into a WorkSpace in the <code>ADMIN_MAINTENANCE</code> state.

        :param self:
        :param WorkspaceState:  The WorkSpace state.<br/>WorkspaceId:  The identifier of the WorkSpace.<br/>
        :return:         """
        pass

    def reboot_workspaces(self, RebootWorkspaceRequests: List[RebootRequest]) -> RebootWorkspacesResult:
        """Reboots the specified WorkSpaces. You cannot reboot a WorkSpace unless its state is <code>AVAILABLE</code> or <code>UNHEALTHY</code>. This operation is asynchronous and returns before the WorkSpaces have rebooted.

        :param self:
        :param RebootWorkspaceRequests:  The WorkSpaces to reboot. You can specify up to 25 WorkSpaces.<br/>
        :return: FailedRequests:  Information about the WorkSpaces that could not be rebooted.<br/>        """
        pass

    def rebuild_workspaces(self, RebuildWorkspaceRequests: List[RebuildRequest]) -> RebuildWorkspacesResult:
        """Rebuilds the specified WorkSpace. You cannot rebuild a WorkSpace unless its state is <code>AVAILABLE</code>, <code>ERROR</code>, or <code>UNHEALTHY</code>. Rebuilding a WorkSpace is a potentially destructive action that can result in the loss of data. For more information, see <a href="https://docs.aws.amazon.com/workspaces/latest/adminguide/reset-workspace.html">Rebuild a WorkSpace</a>. This operation is asynchronous and returns before the WorkSpaces have been completely rebuilt.

        :param self:
        :param RebuildWorkspaceRequests:  The WorkSpace to rebuild. You can specify a single WorkSpace.<br/>
        :return: FailedRequests:  Information about the WorkSpace that could not be rebuilt.<br/>        """
        pass

    def revoke_ip_rules(self, UserRules: List[IpRule], GroupId: str) -> RevokeIpRulesResult:
        """Removes one or more rules from the specified IP access control group.

        :param self:
        :param UserRules:  The rules to remove from the group.<br/>GroupId:  The identifier of the group.<br/>
        :return:         """
        pass

    def start_workspaces(self, StartWorkspaceRequests: List[StartRequest]) -> StartWorkspacesResult:
        """Starts the specified WorkSpaces. You cannot start a WorkSpace unless it has a running mode of <code>AutoStop</code> and a state of <code>STOPPED</code>.

        :param self:
        :param StartWorkspaceRequests:  The WorkSpaces to start. You can specify up to 25 WorkSpaces.<br/>
        :return: FailedRequests:  Information about the WorkSpaces that could not be started.<br/>        """
        pass

    def stop_workspaces(self, StopWorkspaceRequests: List[StopRequest]) -> StopWorkspacesResult:
        """ Stops the specified WorkSpaces. You cannot stop a WorkSpace unless it has a running mode of <code>AutoStop</code> and a state of <code>AVAILABLE</code>, <code>IMPAIRED</code>, <code>UNHEALTHY</code>, or <code>ERROR</code>.

        :param self:
        :param StopWorkspaceRequests:  The WorkSpaces to stop. You can specify up to 25 WorkSpaces.<br/>
        :return: FailedRequests:  Information about the WorkSpaces that could not be stopped.<br/>        """
        pass

    def terminate_workspaces(self, TerminateWorkspaceRequests: List[TerminateRequest]) -> TerminateWorkspacesResult:
        """Terminates the specified WorkSpaces. Terminating a WorkSpace is a permanent action and cannot be undone. The user's data is destroyed. If you need to archive any user data, contact Amazon Web Services before terminating the WorkSpace. You can terminate a WorkSpace that is in any state except <code>SUSPENDED</code>. This operation is asynchronous and returns before the WorkSpaces have been completely terminated.

        :param self:
        :param TerminateWorkspaceRequests:  The WorkSpaces to terminate. You can specify up to 25 WorkSpaces.<br/>
        :return: FailedRequests:  Information about the WorkSpaces that could not be terminated.<br/>        """
        pass

    def update_rules_of_ip_group(self, UserRules: List[IpRuleItem], GroupId: str) -> UpdateRulesOfIpGroupResult:
        """Replaces the current rules of the specified IP access control group with the specified rules.

        :param self:
        :param UserRules:  One or more rules.<br/>GroupId:  The identifier of the group.<br/>
        :return:         """
        pass



class XRay(BaseClient):
    """AWS X-Ray provides APIs for managing debug traces and retrieving service maps and other data created by processing those traces."""


    class BatchGetTracesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `batch_get_traces` for valid parameters"""
            pass
    class GetServiceGraphPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_service_graph` for valid parameters"""
            pass
    class GetTraceGraphPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_trace_graph` for valid parameters"""
            pass
    class GetTraceSummariesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_trace_summaries` for valid parameters"""
            pass
    class GetGroupsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_groups` for valid parameters"""
            pass
    class GetSamplingRulesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_sampling_rules` for valid parameters"""
            pass
    class GetSamplingStatisticSummariesPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_sampling_statistic_summaries` for valid parameters"""
            pass
    class GetTimeSeriesServiceStatisticsPaginator(Paginator):
        def wait(self, **kwargs):
            """see function `get_time_series_service_statistics` for valid parameters"""
            pass


    class BatchGetTracesRequest(Mapping):
        pass
    
    class BatchGetTracesResult(Mapping):
        pass
    
    class TraceId(object):
        pass
    
    class CreateGroupRequest(Mapping):
        pass
    
    class CreateGroupResult(Mapping):
        pass
    
    class CreateSamplingRuleRequest(Mapping):
        pass
    
    class CreateSamplingRuleResult(Mapping):
        pass
    
    class SamplingRule(Mapping):
        pass
    
    class DeleteGroupRequest(Mapping):
        pass
    
    class DeleteGroupResult(Mapping):
        pass
    
    class DeleteSamplingRuleRequest(Mapping):
        pass
    
    class DeleteSamplingRuleResult(Mapping):
        pass
    
    class GetEncryptionConfigRequest(Mapping):
        pass
    
    class GetEncryptionConfigResult(Mapping):
        pass
    
    class GetGroupRequest(Mapping):
        pass
    
    class GetGroupResult(Mapping):
        pass
    
    class GetGroupsRequest(Mapping):
        pass
    
    class GetGroupsResult(Mapping):
        pass
    
    class GetSamplingRulesRequest(Mapping):
        pass
    
    class GetSamplingRulesResult(Mapping):
        pass
    
    class GetSamplingStatisticSummariesRequest(Mapping):
        pass
    
    class GetSamplingStatisticSummariesResult(Mapping):
        pass
    
    class GetSamplingTargetsRequest(Mapping):
        pass
    
    class GetSamplingTargetsResult(Mapping):
        pass
    
    class SamplingStatisticsDocument(Mapping):
        pass
    
    class GetServiceGraphRequest(Mapping):
        pass
    
    class GetServiceGraphResult(Mapping):
        pass
    
    class Timestamp(object):
        pass
    
    class GetTimeSeriesServiceStatisticsRequest(Mapping):
        pass
    
    class GetTimeSeriesServiceStatisticsResult(Mapping):
        pass
    
    class GetTraceGraphRequest(Mapping):
        pass
    
    class GetTraceGraphResult(Mapping):
        pass
    
    class GetTraceSummariesRequest(Mapping):
        pass
    
    class GetTraceSummariesResult(Mapping):
        pass
    
    class NullableBoolean(object):
        pass
    
    class SamplingStrategy(Mapping):
        pass
    
    class PutEncryptionConfigRequest(Mapping):
        pass
    
    class PutEncryptionConfigResult(Mapping):
        pass
    
    class PutTelemetryRecordsRequest(Mapping):
        pass
    
    class PutTelemetryRecordsResult(Mapping):
        pass
    
    class TelemetryRecord(Mapping):
        pass
    
    class PutTraceSegmentsRequest(Mapping):
        pass
    
    class PutTraceSegmentsResult(Mapping):
        pass
    
    class TraceSegmentDocument(object):
        pass
    
    class UpdateGroupRequest(Mapping):
        pass
    
    class UpdateGroupResult(Mapping):
        pass
    
    class UpdateSamplingRuleRequest(Mapping):
        pass
    
    class UpdateSamplingRuleResult(Mapping):
        pass
    
    class SamplingRuleUpdate(Mapping):
        pass
    
    def batch_get_traces(self, TraceIds: List[TraceId], NextToken: str=None) -> BatchGetTracesResult:
        """Retrieves a list of traces specified by ID. Each trace is a collection of segment documents that originates from a single request. Use <code>GetTraceSummaries</code> to get a list of trace IDs.

        :param self:
        :param NextToken:  Pagination token. Not used.<br/>TraceIds:  Specify the trace IDs of requests for which to retrieve segments.<br/>
        :return: NextToken:  Pagination token. Not used.<br/>UnprocessedTraceIds:  Trace IDs of requests that haven't been processed.<br/>Traces:  Full traces for the specified requests.<br/>        """
        pass

    def create_group(self, GroupName: str, FilterExpression: str=None) -> CreateGroupResult:
        """Creates a group resource with a name and a filter expression. 

        :param self:
        :param FilterExpression:  The filter expression defining criteria by which to group traces.<br/>GroupName:  The case-sensitive name of the new group. Default is a reserved name and names must be unique.<br/>
        :return: Group:  The group that was created. Contains the name of the group that was created, the ARN of the group that was generated based on the group name, and the filter expression that was assigned to the group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FilterExpression</i> The filter expression defining the parameters to include traces.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupARN</i> The ARN of the group generated based on the GroupName.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupName</i> The unique case-sensitive name of the group.<br/>        """
        pass

    def create_sampling_rule(self, SamplingRule: SamplingRule) -> CreateSamplingRuleResult:
        """Creates a rule to control sampling behavior for instrumented applications. Services retrieve rules with <a>GetSamplingRules</a>, and evaluate each rule in ascending order of <i>priority</i> for each request. If a rule matches, the service records a trace, borrowing it from the reservoir size. After 10 seconds, the service reports back to X-Ray with <a>GetSamplingTargets</a> to get updated versions of each in-use rule. The updated rule contains a trace quota that the service can use instead of borrowing from the reservoir.

        :param self:
        :param SamplingRule:  The rule definition.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Attributes</i> Matches attributes derived from the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version of the sampling rule format (<code>1</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>URLPath</i> Matches the path from a request URL.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HTTPMethod</i> Matches the HTTP method of a request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Host</i> Matches the hostname from a request URL.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceType</i> Matches the <code>origin</code> that the service uses to identify its type in segments.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceName</i> Matches the <code>name</code> that the service uses to identify itself in segments.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservoirSize</i> A fixed number of matching requests to instrument per second, prior to applying the fixed rate. The reservoir is not used directly by services, but applies to all services using the rule collectively.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FixedRate</i> The percentage of matching requests to instrument, after the reservoir is exhausted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Priority</i> The priority of the sampling rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceARN</i> Matches the ARN of the AWS resource on which the service runs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleARN</i> The ARN of the sampling rule. Specify a rule by either name or ARN, but not both.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleName</i> The name of the sampling rule. Specify a rule by either name or ARN, but not both.<br/>
        :return: SamplingRuleRecord:  The saved rule definition and metadata.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModifiedAt</i> When the rule was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i> When the rule was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SamplingRule</i> The sampling rule.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SamplingRule</b>: The sampling rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Attributes</i> Matches attributes derived from the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version of the sampling rule format (<code>1</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>URLPath</i> Matches the path from a request URL.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HTTPMethod</i> Matches the HTTP method of a request.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Host</i> Matches the hostname from a request URL.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceType</i> Matches the <code>origin</code> that the service uses to identify its type in segments.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceName</i> Matches the <code>name</code> that the service uses to identify itself in segments.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservoirSize</i> A fixed number of matching requests to instrument per second, prior to applying the fixed rate. The reservoir is not used directly by services, but applies to all services using the rule collectively.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FixedRate</i> The percentage of matching requests to instrument, after the reservoir is exhausted.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Priority</i> The priority of the sampling rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceARN</i> Matches the ARN of the AWS resource on which the service runs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleARN</i> The ARN of the sampling rule. Specify a rule by either name or ARN, but not both.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleName</i> The name of the sampling rule. Specify a rule by either name or ARN, but not both.<br/>        """
        pass

    def delete_group(self, GroupName: str=None, GroupARN: str=None) -> DeleteGroupResult:
        """Deletes a group resource.

        :param self:
        :param GroupARN:  The ARN of the group that was generated on creation.<br/>GroupName:  The case-sensitive name of the group.<br/>
        :return:         """
        pass

    def delete_sampling_rule(self, RuleName: str=None, RuleARN: str=None) -> DeleteSamplingRuleResult:
        """Deletes a sampling rule.

        :param self:
        :param RuleARN:  The ARN of the sampling rule. Specify a rule by either name or ARN, but not both.<br/>RuleName:  The name of the sampling rule. Specify a rule by either name or ARN, but not both.<br/>
        :return: SamplingRuleRecord:  The deleted rule definition and metadata.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModifiedAt</i> When the rule was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i> When the rule was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SamplingRule</i> The sampling rule.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SamplingRule</b>: The sampling rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Attributes</i> Matches attributes derived from the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version of the sampling rule format (<code>1</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>URLPath</i> Matches the path from a request URL.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HTTPMethod</i> Matches the HTTP method of a request.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Host</i> Matches the hostname from a request URL.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceType</i> Matches the <code>origin</code> that the service uses to identify its type in segments.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceName</i> Matches the <code>name</code> that the service uses to identify itself in segments.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservoirSize</i> A fixed number of matching requests to instrument per second, prior to applying the fixed rate. The reservoir is not used directly by services, but applies to all services using the rule collectively.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FixedRate</i> The percentage of matching requests to instrument, after the reservoir is exhausted.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Priority</i> The priority of the sampling rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceARN</i> Matches the ARN of the AWS resource on which the service runs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleARN</i> The ARN of the sampling rule. Specify a rule by either name or ARN, but not both.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleName</i> The name of the sampling rule. Specify a rule by either name or ARN, but not both.<br/>        """
        pass

    def get_encryption_config(self) -> GetEncryptionConfigResult:
        """Retrieves the current encryption configuration for X-Ray data.

        :param self:
        :param 
        :return: EncryptionConfig:  The encryption configuration document.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of encryption. Set to <code>KMS</code> for encryption with CMKs. Set to <code>NONE</code> for default encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The encryption status. While the status is <code>UPDATING</code>, X-Ray may encrypt data with a combination of the new and old settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyId</i> The ID of the customer master key (CMK) used for encryption, if applicable.<br/>        """
        pass

    def get_group(self, GroupName: str=None, GroupARN: str=None) -> GetGroupResult:
        """Retrieves group resource details.

        :param self:
        :param GroupARN:  The ARN of the group that was generated on creation.<br/>GroupName:  The case-sensitive name of the group.<br/>
        :return: Group:  The group that was requested. Contains the name of the group, the ARN of the group, and the filter expression that assigned to the group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FilterExpression</i> The filter expression defining the parameters to include traces.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupARN</i> The ARN of the group generated based on the GroupName.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupName</i> The unique case-sensitive name of the group.<br/>        """
        pass

    def get_groups(self, NextToken: str=None) -> GetGroupsResult:
        """Retrieves all active group details.

        :param self:
        :param NextToken:  Pagination token. Not used.<br/>
        :return: NextToken:  Pagination token. Not used.<br/>Groups:  The collection of all active groups.<br/>        """
        pass

    def get_sampling_rules(self, NextToken: str=None) -> GetSamplingRulesResult:
        """Retrieves all sampling rules.

        :param self:
        :param NextToken:  Pagination token. Not used.<br/>
        :return: NextToken:  Pagination token. Not used.<br/>SamplingRuleRecords:  Rule definitions and metadata.<br/>        """
        pass

    def get_sampling_statistic_summaries(self, NextToken: str=None) -> GetSamplingStatisticSummariesResult:
        """Retrieves information about recent sampling results for all sampling rules.

        :param self:
        :param NextToken:  Pagination token. Not used.<br/>
        :return: NextToken:  Pagination token. Not used.<br/>SamplingStatisticSummaries:  Information about the number of requests instrumented for each sampling rule.<br/>        """
        pass

    def get_sampling_targets(self, SamplingStatisticsDocuments: List[SamplingStatisticsDocument]) -> GetSamplingTargetsResult:
        """Requests a sampling quota for rules that the service is using to sample requests. 

        :param self:
        :param SamplingStatisticsDocuments:  Information about rules that the service is using to sample requests.<br/>
        :return: UnprocessedStatistics:  Information about <a>SamplingStatisticsDocument</a> that X-Ray could not process.<br/>LastRuleModification:  The last time a user changed the sampling rule configuration. If the sampling rule configuration changed since the service last retrieved it, the service should call <a>GetSamplingRules</a> to get the latest version.<br/>SamplingTargetDocuments:  Updated rules that the service should use to sample requests.<br/>        """
        pass

    def get_service_graph(self, EndTime: Timestamp, StartTime: Timestamp, GroupName: str=None, GroupARN: str=None, NextToken: str=None) -> GetServiceGraphResult:
        """Retrieves a document that describes services that process incoming requests, and downstream services that they call as a result. Root services process incoming requests and make calls to downstream services. Root services are applications that use the AWS X-Ray SDK. Downstream services can be other applications, AWS resources, HTTP web APIs, or SQL databases.

        :param self:
        :param NextToken:  Pagination token. Not used.<br/>GroupARN:  The ARN of a group to generate a graph based on.<br/>GroupName:  The name of a group to generate a graph based on.<br/>EndTime:  The end of the timeframe for which to generate a graph.<br/>StartTime:  The start of the time frame for which to generate a graph.<br/>
        :return: NextToken:  Pagination token. Not used.<br/>ContainsOldGroupVersions:  A flag indicating whether the group's filter expression has been consistent, or if the returned service graph may show traces from an older version of the group's filter expression.<br/>Services:  The services that have processed a traced request during the specified time frame.<br/>EndTime:  The end of the time frame for which the graph was generated.<br/>StartTime:  The start of the time frame for which the graph was generated.<br/>        """
        pass

    def get_time_series_service_statistics(self, EndTime: Timestamp, StartTime: Timestamp, GroupName: str=None, GroupARN: str=None, EntitySelectorExpression: str=None, Period: int=None, NextToken: str=None) -> GetTimeSeriesServiceStatisticsResult:
        """Get an aggregation of service statistics defined by a specific time range.

        :param self:
        :param NextToken:  Pagination token. Not used.<br/>Period:  Aggregation period in seconds.<br/>EntitySelectorExpression:  A filter expression defining entities that will be aggregated for statistics. Supports ID, service, and edge functions. If no selector expression is specified, edge statistics are returned. <br/>GroupARN:  The ARN of the group for which to pull statistics from.<br/>GroupName:  The case-sensitive name of the group for which to pull statistics from.<br/>EndTime:  The end of the time frame for which to aggregate statistics.<br/>StartTime:  The start of the time frame for which to aggregate statistics.<br/>
        :return: NextToken:  Pagination token. Not used.<br/>ContainsOldGroupVersions:  A flag indicating whether or not a group's filter expression has been consistent, or if a returned aggregation may show statistics from an older version of the group's filter expression.<br/>TimeSeriesServiceStatistics:  The collection of statistics.<br/>        """
        pass

    def get_trace_graph(self, TraceIds: List[TraceId], NextToken: str=None) -> GetTraceGraphResult:
        """Retrieves a service graph for one or more specific trace IDs.

        :param self:
        :param NextToken:  Pagination token. Not used.<br/>TraceIds:  Trace IDs of requests for which to generate a service graph.<br/>
        :return: NextToken:  Pagination token. Not used.<br/>Services:  The services that have processed one of the specified requests.<br/>        """
        pass

    def get_trace_summaries(self, EndTime: Timestamp, StartTime: Timestamp, TimeRangeType: str=None, Sampling: NullableBoolean=None, SamplingStrategy: SamplingStrategy=None, FilterExpression: str=None, NextToken: str=None) -> GetTraceSummariesResult:
        """Retrieves IDs and metadata for traces available for a specified time frame using an optional filter. To get the full traces, pass the trace IDs to <code>BatchGetTraces</code>. A filter expression can target traced requests that hit specific service nodes or edges, have errors, or come from a known user. For example, the following filter expression targets traces that pass through <code>api.example.com</code>:  <code>service("api.example.com")</code>  This filter expression finds traces that have an annotation named <code>account</code> with the value <code>12345</code>:  <code>annotation.account = "12345"</code>  For a full list of indexed fields and keywords that you can use in filter expressions, see <a href="https://docs.aws.amazon.com/xray/latest/devguide/xray-console-filters.html">Using Filter Expressions</a> in the <i>AWS X-Ray Developer Guide</i>.

        :param self:
        :param NextToken:  Specify the pagination token returned by a previous request to retrieve the next page of results.<br/>FilterExpression:  Specify a filter expression to retrieve trace summaries for services or requests that meet certain requirements.<br/>SamplingStrategy:  A paramater to indicate whether to enable sampling on trace summaries. Input parameters are Name and Value.:<br/>Sampling:  Set to <code>true</code> to get summaries for only a subset of available traces.<br/>TimeRangeType:  A parameter to indicate whether to query trace summaries by TraceId or Event time.<br/>EndTime:  The end of the time frame for which to retrieve traces.<br/>StartTime:  The start of the time frame for which to retrieve traces.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Value</i> The value of a sampling rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Name</i> The name of a sampling rule.<br/>
        :return: NextToken:  If the requested time frame contained more than one page of results, you can use this token to retrieve the next page. The first page contains the most most recent results, closest to the end of the time frame.<br/>TracesProcessedCount:  The total number of traces processed, including traces that did not match the specified filter expression.<br/>ApproximateTime:  The start time of this page of results.<br/>TraceSummaries:  Trace IDs and metadata for traces that were found in the specified time frame.<br/>        """
        pass

    def put_encryption_config(self, Type: str, KeyId: str=None) -> PutEncryptionConfigResult:
        """Updates the encryption configuration for X-Ray data.

        :param self:
        :param Type:  The type of encryption. Set to <code>KMS</code> to use your own key for encryption. Set to <code>NONE</code> for default encryption.<br/>KeyId:  An AWS KMS customer master key (CMK) in one of the following formats: <ul> <li>  <b>Alias</b> - The name of the key. For example, <code>alias/MyKey</code>. </li> <li>  <b>Key ID</b> - The KMS key ID of the key. For example, <code>ae4aa6d49-a4d8-9df9-a475-4ff6d7898456</code>. </li> <li>  <b>ARN</b> - The full Amazon Resource Name of the key ID or alias. For example, <code>arn:aws:kms:us-east-2:123456789012:key/ae4aa6d49-a4d8-9df9-a475-4ff6d7898456</code>. Use this format to specify a key in a different account. </li> </ul> Omit this key if you set <code>Type</code> to <code>NONE</code>.<br/>
        :return: EncryptionConfig:  The new encryption configuration.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Type</i> The type of encryption. Set to <code>KMS</code> for encryption with CMKs. Set to <code>NONE</code> for default encryption.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Status</i> The encryption status. While the status is <code>UPDATING</code>, X-Ray may encrypt data with a combination of the new and old settings.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>KeyId</i> The ID of the customer master key (CMK) used for encryption, if applicable.<br/>        """
        pass

    def put_telemetry_records(self, TelemetryRecords: List[TelemetryRecord], EC2InstanceId: str=None, Hostname: str=None, ResourceARN: str=None) -> PutTelemetryRecordsResult:
        """Used by the AWS X-Ray daemon to upload telemetry.

        :param self:
        :param ResourceARN:  <p/><br/>Hostname:  <p/><br/>EC2InstanceId:  <p/><br/>TelemetryRecords:  <p/><br/>
        :return:         """
        pass

    def put_trace_segments(self, TraceSegmentDocuments: List[TraceSegmentDocument]) -> PutTraceSegmentsResult:
        """Uploads segment documents to AWS X-Ray. The X-Ray SDK generates segment documents and sends them to the X-Ray daemon, which uploads them in batches. A segment document can be a completed segment, an in-progress segment, or an array of subsegments. Segments must include the following fields. For the full segment document schema, see <a href="https://docs.aws.amazon.com/xray/latest/devguide/xray-api-segmentdocuments.html">AWS X-Ray Segment Documents</a> in the <i>AWS X-Ray Developer Guide</i>. <p class="title"> <b>Required Segment Document Fields</b>  <ul> <li>  <code>name</code> - The name of the service that handled the request. </li> <li>  <code>id</code> - A 64-bit identifier for the segment, unique among segments in the same trace, in 16 hexadecimal digits. </li> <li>  <code>trace_id</code> - A unique identifier that connects all segments and subsegments originating from a single client request. </li> <li>  <code>start_time</code> - Time the segment or subsegment was created, in floating point seconds in epoch time, accurate to milliseconds. For example, <code>1480615200.010</code> or <code>1.480615200010E9</code>. </li> <li>  <code>end_time</code> - Time the segment or subsegment was closed. For example, <code>1480615200.090</code> or <code>1.480615200090E9</code>. Specify either an <code>end_time</code> or <code>in_progress</code>. </li> <li>  <code>in_progress</code> - Set to <code>true</code> instead of specifying an <code>end_time</code> to record that a segment has been started, but is not complete. Send an in progress segment when your application receives a request that will take a long time to serve, to trace the fact that the request was received. When the response is sent, send the complete segment to overwrite the in-progress segment. </li> </ul> A <code>trace_id</code> consists of three numbers separated by hyphens. For example, 1-58406520-a006649127e371903a2de979. This includes: <p class="title"> <b>Trace ID Format</b>  <ul> <li> The version number, i.e. <code>1</code>. </li> <li> The time of the original request, in Unix epoch time, in 8 hexadecimal digits. For example, 10:00AM December 2nd, 2016 PST in epoch time is <code>1480615200</code> seconds, or <code>58406520</code> in hexadecimal. </li> <li> A 96-bit identifier for the trace, globally unique, in 24 hexadecimal digits. </li> </ul>

        :param self:
        :param TraceSegmentDocuments:  A string containing a JSON document defining one or more segments or subsegments.<br/>
        :return: UnprocessedTraceSegments:  Segments that failed processing.<br/>        """
        pass

    def update_group(self, GroupName: str=None, GroupARN: str=None, FilterExpression: str=None) -> UpdateGroupResult:
        """Updates a group resource.

        :param self:
        :param FilterExpression:  The updated filter expression defining criteria by which to group traces.<br/>GroupARN:  The ARN that was generated upon creation.<br/>GroupName:  The case-sensitive name of the group.<br/>
        :return: Group:  The group that was updated. Contains the name of the group that was updated, the ARN of the group that was updated, and the updated filter expression assigned to the group.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FilterExpression</i> The filter expression defining the parameters to include traces.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupARN</i> The ARN of the group generated based on the GroupName.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>GroupName</i> The unique case-sensitive name of the group.<br/>        """
        pass

    def update_sampling_rule(self, SamplingRuleUpdate: SamplingRuleUpdate) -> UpdateSamplingRuleResult:
        """Modifies a sampling rule's configuration.

        :param self:
        :param SamplingRuleUpdate:  The rule and fields to change.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Attributes</i> Matches attributes derived from the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>URLPath</i> Matches the path from a request URL.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>HTTPMethod</i> Matches the HTTP method of a request.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceType</i> Matches the <code>origin</code> that the service uses to identify its type in segments.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceName</i> Matches the <code>name</code> that the service uses to identify itself in segments.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Host</i> Matches the hostname from a request URL.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservoirSize</i> A fixed number of matching requests to instrument per second, prior to applying the fixed rate. The reservoir is not used directly by services, but applies to all services using the rule collectively.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>FixedRate</i> The percentage of matching requests to instrument, after the reservoir is exhausted.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>Priority</i> The priority of the sampling rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceARN</i> Matches the ARN of the AWS resource on which the service runs.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleARN</i> The ARN of the sampling rule. Specify a rule by either name or ARN, but not both.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleName</i> The name of the sampling rule. Specify a rule by either name or ARN, but not both.<br/>
        :return: SamplingRuleRecord:  The updated rule definition and metadata.:<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>ModifiedAt</i> When the rule was last modified.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>CreatedAt</i> When the rule was created.<br/>&nbsp;&nbsp;&nbsp;&nbsp; <i>SamplingRule</i> The sampling rule.&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>SamplingRule</b>: The sampling rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Attributes</i> Matches attributes derived from the request.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Version</i> The version of the sampling rule format (<code>1</code>).<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>URLPath</i> Matches the path from a request URL.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>HTTPMethod</i> Matches the HTTP method of a request.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Host</i> Matches the hostname from a request URL.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceType</i> Matches the <code>origin</code> that the service uses to identify its type in segments.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ServiceName</i> Matches the <code>name</code> that the service uses to identify itself in segments.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ReservoirSize</i> A fixed number of matching requests to instrument per second, prior to applying the fixed rate. The reservoir is not used directly by services, but applies to all services using the rule collectively.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>FixedRate</i> The percentage of matching requests to instrument, after the reservoir is exhausted.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>Priority</i> The priority of the sampling rule.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>ResourceARN</i> Matches the ARN of the AWS resource on which the service runs.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleARN</i> The ARN of the sampling rule. Specify a rule by either name or ARN, but not both.<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i>RuleName</i> The name of the sampling rule. Specify a rule by either name or ARN, but not both.<br/>        """
        pass



